/* Example program for GNU libtextstyle.
   Copyright (C) 2018-2019 Free Software Foundation, Inc.
   Written by Bruno Haible <bruno@clisp.org>, 2018.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* List of first names and the probability that they are given to boys vs. girls
   in the USA from 1880 to 2016.
   Origin of data: <https://www.ssa.gov/oact/babynames/limits.html>.  */

struct first_name
{
  /* Given first name.  */
  const char *name;
  /* Number of uses for boys / total number of uses.  */
  float p_boy;
};

static const struct first_name names[] =
{
  { "Aaban", 1.0 },
  { "Aabha", 0.0 },
  { "Aabid", 1.0 },
  { "Aabir", 1.0 },
  { "Aabriella", 0.0 },
  { "Aada", 0.0 },
  { "Aadam", 1.0 },
  { "Aadan", 1.0 },
  { "Aadarsh", 1.0 },
  { "Aaden", 0.9988672 },
  { "Aadesh", 1.0 },
  { "Aadhav", 1.0 },
  { "Aadhavan", 1.0 },
  { "Aadhi", 1.0 },
  { "Aadhira", 0.0 },
  { "Aadhvik", 1.0 },
  { "Aadhvika", 0.0 },
  { "Aadhya", 0.0 },
  { "Aadhyan", 1.0 },
  { "Aadi", 0.97951347 },
  { "Aadian", 1.0 },
  { "Aadil", 1.0 },
  { "Aadin", 1.0 },
  { "Aadish", 1.0 },
  { "Aadison", 0.0 },
  { "Aadit", 1.0 },
  { "Aadith", 1.0 },
  { "Aadithya", 1.0 },
  { "Aaditri", 0.0 },
  { "Aaditya", 1.0 },
  { "Aadiv", 1.0 },
  { "Aadon", 1.0 },
  { "Aadrian", 1.0 },
  { "Aadrika", 0.0 },
  { "Aadrit", 1.0 },
  { "Aadriti", 0.0 },
  { "Aadvi", 0.0 },
  { "Aadvik", 1.0 },
  { "Aadvika", 0.0 },
  { "Aadya", 0.0 },
  { "Aadyn", 0.9668737 },
  { "Aafia", 0.0 },
  { "Aafiya", 0.0 },
  { "Aafiyah", 0.0 },
  { "Aafreen", 0.0 },
  { "Aagam", 1.0 },
  { "Aage", 1.0 },
  { "Aagna", 0.0 },
  { "Aagot", 0.0 },
  { "Aahaan", 1.0 },
  { "Aahan", 1.0 },
  { "Aahana", 0.0 },
  { "Aahil", 1.0 },
  { "Aahir", 1.0 },
  { "Aahliyah", 0.0 },
  { "Aahna", 0.0 },
  { "Aahron", 1.0 },
  { "Aaidan", 1.0 },
  { "Aaiden", 1.0 },
  { "Aaidyn", 1.0 },
  { "Aaila", 0.0 },
  { "Aailiyah", 0.0 },
  { "Aailyah", 0.0 },
  { "Aaima", 0.0 },
  { "Aaira", 0.0 },
  { "Aairah", 0.0 },
  { "Aaisha", 0.0 },
  { "Aaishah", 0.0 },
  { "Aaiyana", 0.0 },
  { "Aaiza", 0.0 },
  { "Aaja", 0.0 },
  { "Aajah", 0.0 },
  { "Aajaylah", 0.0 },
  { "Aajon", 1.0 },
  { "Aakanksha", 0.0 },
  { "Aakarsh", 1.0 },
  { "Aakash", 1.0 },
  { "Aakeem", 1.0 },
  { "Aakilah", 0.0 },
  { "Aakira", 0.0 },
  { "Aakiyah", 0.0 },
  { "Aakriti", 0.0 },
  { "Aala", 0.0 },
  { "Aalaiya", 0.0 },
  { "Aalaiyah", 0.0 },
  { "Aalana", 0.0 },
  { "Aalanah", 0.0 },
  { "Aalani", 0.0 },
  { "Aalap", 1.0 },
  { "Aalaya", 0.0 },
  { "Aalayah", 0.0 },
  { "Aalayiah", 0.0 },
  { "Aalayjah", 0.0 },
  { "Aalayna", 0.0 },
  { "Aalaysha", 0.0 },
  { "Aalaysia", 0.0 },
  { "Aalea", 0.0 },
  { "Aaleah", 0.0 },
  { "Aaleahya", 0.0 },
  { "Aaleayah", 0.0 },
  { "Aaleena", 0.0 },
  { "Aaleeya", 0.0 },
  { "Aaleeyah", 0.0 },
  { "Aaleiah", 0.0 },
  { "Aaleigha", 0.0 },
  { "Aaleiyah", 0.0 },
  { "Aalena", 0.0 },
  { "Aalexis", 0.0 },
  { "Aalexus", 0.0 },
  { "Aaleya", 0.0 },
  { "Aaleyah", 0.0 },
  { "Aali", 1.0 },
  { "Aalia", 0.0 },
  { "Aaliah", 0.0 },
  { "Aaliana", 0.0 },
  { "Aalias", 1.0 },
  { "Aaliayah", 0.0 },
  { "Aaliayh", 0.0 },
  { "Aalicia", 0.0 },
  { "Aaliha", 0.0 },
  { "Aalijah", 0.59032255 },
  { "Aalim", 1.0 },
  { "Aalimah", 0.0 },
  { "Aalina", 0.0 },
  { "Aalinah", 0.0 },
  { "Aalisa", 0.0 },
  { "Aalisha", 0.0 },
  { "Aalivia", 0.0 },
  { "Aaliya", 0.0 },
  { "Aaliyaa", 0.0 },
  { "Aaliyah", 0.0011201168 },
  { "Aaliyaha", 0.0 },
  { "Aaliyahmarie", 0.0 },
  { "Aaliyahrose", 0.0 },
  { "Aaliyan", 0.046296295 },
  { "Aaliyana", 0.0 },
  { "Aaliyanna", 0.0 },
  { "Aaliyha", 0.0 },
  { "Aaliyiah", 0.0 },
  { "Aalliyah", 0.0 },
  { "Aallyah", 0.0 },
  { "Aalok", 1.0 },
  { "Aalon", 1.0 },
  { "Aaloni", 0.0 },
  { "Aalya", 0.0 },
  { "Aalyah", 0.0 },
  { "Aalycia", 0.0 },
  { "Aalyha", 0.0 },
  { "Aalyia", 0.0 },
  { "Aalyiah", 0.0 },
  { "Aalyna", 0.0 },
  { "Aalysia", 0.0 },
  { "Aalyssa", 0.0 },
  { "Aalyvia", 0.0 },
  { "Aamanda", 0.0 },
  { "Aamanee", 0.0 },
  { "Aamani", 0.0 },
  { "Aamar", 1.0 },
  { "Aamari", 0.5736842 },
  { "Aamarion", 1.0 },
  { "Aamaya", 0.0 },
  { "Aamber", 0.0 },
  { "Aamena", 0.0 },
  { "Aamer", 1.0 },
  { "Aamia", 0.0 },
  { "Aamil", 1.0 },
  { "Aamilah", 0.0 },
  { "Aamina", 0.0 },
  { "Aaminah", 0.0 },
  { "Aamir", 0.99649125 },
  { "Aamira", 0.0 },
  { "Aamirah", 0.0 },
  { "Aamiya", 0.0 },
  { "Aamiyah", 0.0 },
  { "Aamna", 0.0 },
  { "Aamoni", 0.0 },
  { "Aamora", 0.0 },
  { "Aamori", 0.0 },
  { "Aamya", 0.0 },
  { "Aamyah", 0.0 },
  { "Aana", 0.0 },
  { "Aanand", 1.0 },
  { "Aanav", 1.0 },
  { "Aanay", 1.0 },
  { "Aanaya", 0.0 },
  { "Aanchal", 0.0 },
  { "Aania", 0.0 },
  { "Aaniah", 0.0 },
  { "Aanijah", 0.0 },
  { "Aanika", 0.0 },
  { "Aanisah", 0.0 },
  { "Aaniya", 0.0 },
  { "Aaniyah", 0.0 },
  { "Aaniylah", 0.0 },
  { "Aansh", 1.0 },
  { "Aanshi", 0.0 },
  { "Aanvi", 0.0 },
  { "Aanya", 0.0 },
  { "Aanyah", 0.0 },
  { "Aanyla", 0.0 },
  { "Aapri", 0.0 },
  { "Aaqib", 1.0 },
  { "Aaqil", 1.0 },
  { "Aara", 0.0 },
  { "Aarabella", 0.0 },
  { "Aarabhi", 0.0 },
  { "Aaradhana", 0.0 },
  { "Aaradhy", 1.0 },
  { "Aaradhya", 0.0 },
  { "Aaraf", 1.0 },
  { "Aaraiz", 1.0 },
  { "Aaralyn", 0.0 },
  { "Aaralynn", 0.0 },
  { "Aaran", 1.0 },
  { "Aaraon", 1.0 },
  { "Aaratrika", 0.0 },
  { "Aarav", 1.0 },
  { "Aaravi", 0.0 },
  { "Aaravreddy", 1.0 },
  { "Aaraya", 0.0 },
  { "Aaren", 0.73193914 },
  { "Aareon", 0.76744187 },
  { "Aareona", 0.0 },
  { "Aari", 0.5263158 },
  { "Aaria", 0.0 },
  { "Aariah", 0.0 },
  { "Aarian", 0.6666667 },
  { "Aariana", 0.0 },
  { "Aarianna", 0.0 },
  { "Aarib", 1.0 },
  { "Aaric", 1.0 },
  { "Aarica", 0.0 },
  { "Aarick", 1.0 },
  { "Aarie", 0.0 },
  { "Aariel", 0.0 },
  { "Aariella", 0.0 },
  { "Aarielle", 0.0 },
  { "Aarien", 1.0 },
  { "Aarik", 1.0 },
  { "Aarika", 0.0 },
  { "Aariketh", 1.0 },
  { "Aarilyn", 0.0 },
  { "Aarilynn", 0.0 },
  { "Aarin", 0.6239103 },
  { "Aarini", 0.0 },
  { "Aarion", 0.76430976 },
  { "Aariona", 0.0 },
  { "Aarionna", 0.0 },
  { "Aaris", 0.47435898 },
  { "Aarish", 1.0 },
  { "Aarit", 1.0 },
  { "Aariv", 1.0 },
  { "Aariya", 0.0 },
  { "Aariyah", 0.0 },
  { "Aariyan", 1.0 },
  { "Aariyana", 0.0 },
  { "Aariyanna", 0.0 },
  { "Aariyona", 0.0 },
  { "Aariyonna", 0.0 },
  { "Aariz", 1.0 },
  { "Aarjav", 1.0 },
  { "Aarn", 1.0 },
  { "Aarna", 0.0 },
  { "Aarnav", 1.0 },
  { "Aarnavi", 0.0 },
  { "Aarne", 1.0 },
  { "Aaro", 1.0 },
  { "Aaroh", 1.0 },
  { "Aarohan", 1.0 },
  { "Aarohi", 0.0 },
  { "Aarolyn", 0.0 },
  { "Aaron", 0.99254096 },
  { "Aarona", 0.0 },
  { "Aaronae", 0.0 },
  { "Aaronda", 0.0 },
  { "Aaronette", 0.0 },
  { "Aaronisha", 0.0 },
  { "Aaronjacob", 1.0 },
  { "Aaronjames", 1.0 },
  { "Aaronjohn", 1.0 },
  { "Aaronjoseph", 1.0 },
  { "Aaronjosh", 1.0 },
  { "Aaronjoshua", 1.0 },
  { "Aaronlee", 1.0 },
  { "Aaronmichael", 1.0 },
  { "Aaronn", 1.0 },
  { "Aaroosh", 1.0 },
  { "Aarren", 1.0 },
  { "Aarron", 1.0 },
  { "Aarsh", 1.0 },
  { "Aarshi", 0.0 },
  { "Aarshiya", 0.0 },
  { "Aarthi", 0.0 },
  { "Aarti", 0.0 },
  { "Aaruhi", 0.0 },
  { "Aarush", 1.0 },
  { "Aarushi", 0.0 },
  { "Aarvi", 0.0 },
  { "Aarvin", 1.0 },
  { "Aarya", 0.18580247 },
  { "Aaryah", 0.0 },
  { "Aaryahi", 0.0 },
  { "Aaryan", 0.99516517 },
  { "Aaryana", 0.0 },
  { "Aaryanna", 0.0 },
  { "Aaryash", 1.0 },
  { "Aaryav", 1.0 },
  { "Aaryn", 0.38123873 },
  { "Aarynn", 0.0 },
  { "Aaryon", 1.0 },
  { "Aaryona", 0.0 },
  { "Aarza", 0.0 },
  { "Aaseem", 1.0 },
  { "Aaser", 1.0 },
  { "Aasha", 0.0 },
  { "Aashana", 0.0 },
  { "Aashay", 1.0 },
  { "Aashi", 0.0 },
  { "Aashia", 0.0 },
  { "Aashika", 0.0 },
  { "Aashini", 0.0 },
  { "Aashir", 1.0 },
  { "Aashirya", 0.0 },
  { "Aashish", 1.0 },
  { "Aashita", 0.0 },
  { "Aashiyana", 0.0 },
  { "Aashka", 0.0 },
  { "Aashman", 1.0 },
  { "Aashna", 0.0 },
  { "Aashni", 0.0 },
  { "Aashray", 1.0 },
  { "Aashrita", 0.0 },
  { "Aashrith", 1.0 },
  { "Aashritha", 0.0 },
  { "Aashvi", 0.0 },
  { "Aasia", 0.0 },
  { "Aasim", 1.0 },
  { "Aasin", 1.0 },
  { "Aasir", 1.0 },
  { "Aasiya", 0.0 },
  { "Aasiyah", 0.0 },
  { "Aason", 1.0 },
  { "Aastha", 0.0 },
  { "Aathan", 1.0 },
  { "Aatif", 1.0 },
  { "Aatish", 1.0 },
  { "Aava", 0.0 },
  { "Aavah", 0.0 },
  { "Aavash", 1.0 },
  { "Aaven", 1.0 },
  { "Aavi", 1.0 },
  { "Aavin", 1.0 },
  { "Aavion", 1.0 },
  { "Aavish", 1.0 },
  { "Aavya", 0.0 },
  { "Aavyan", 1.0 },
  { "Aavyn", 0.54545456 },
  { "Aaya", 0.0 },
  { "Aayaan", 1.0 },
  { "Aayah", 0.0 },
  { "Aayam", 1.0 },
  { "Aayan", 1.0 },
  { "Aayana", 0.0 },
  { "Aayanah", 0.0 },
  { "Aayanna", 0.0 },
  { "Aayansh", 1.0 },
  { "Aayat", 0.0 },
  { "Aaydan", 1.0 },
  { "Aayden", 0.9962321 },
  { "Aayla", 0.0 },
  { "Aaylah", 0.0 },
  { "Aayliah", 0.0 },
  { "Aayra", 0.0 },
  { "Aayu", 1.0 },
  { "Aayush", 1.0 },
  { "Aayusha", 0.0 },
  { "Aayushi", 0.0 },
  { "Aazan", 1.0 },
  { "Aazeen", 0.0 },
  { "Aazil", 1.0 },
  { "Aazim", 1.0 },
  { "Aaziyah", 0.0 },
  { "Ab", 1.0 },
  { "Aba", 0.18518518 },
  { "Abaan", 1.0 },
  { "Abaas", 1.0 },
  { "Abad", 1.0 },
  { "Abagael", 0.0 },
  { "Abagail", 0.0 },
  { "Abagaile", 0.0 },
  { "Abagale", 0.0 },
  { "Abagayle", 0.0 },
  { "Abaigael", 0.0 },
  { "Abaigail", 0.0 },
  { "Abaigeal", 0.0 },
  { "Aban", 1.0 },
  { "Abanoub", 1.0 },
  { "Abas", 1.0 },
  { "Abasi", 1.0 },
  { "Abass", 1.0 },
  { "Abayomi", 1.0 },
  { "Abb", 1.0 },
  { "Abba", 0.8666667 },
  { "Abbaas", 1.0 },
  { "Abbagail", 0.0 },
  { "Abbagale", 0.0 },
  { "Abbagayle", 0.0 },
  { "Abbas", 1.0 },
  { "Abbe", 0.0 },
  { "Abbee", 0.0 },
  { "Abbegail", 0.0 },
  { "Abbegale", 0.0 },
  { "Abbegayle", 0.0 },
  { "Abbey", 0.0020493004 },
  { "Abbeygail", 0.0 },
  { "Abbeygale", 0.0 },
  { "Abbi", 0.0 },
  { "Abbie", 0.015052568 },
  { "Abbiegail", 0.0 },
  { "Abbiegale", 0.0 },
  { "Abbiegayle", 0.0 },
  { "Abbigael", 0.0 },
  { "Abbigail", 0.00044283058 },
  { "Abbigaile", 0.0 },
  { "Abbigal", 0.0 },
  { "Abbigale", 0.0 },
  { "Abbigayl", 0.0 },
  { "Abbigayle", 0.0 },
  { "Abbilyn", 0.0 },
  { "Abbot", 1.0 },
  { "Abbott", 0.975 },
  { "Abbra", 0.0 },
  { "Abbriella", 0.0 },
  { "Abbrielle", 0.0 },
  { "Abby", 0.0031726556 },
  { "Abbye", 0.0 },
  { "Abbygael", 0.0 },
  { "Abbygail", 0.0 },
  { "Abbygaile", 0.0 },
  { "Abbygale", 0.0 },
  { "Abbygayl", 0.0 },
  { "Abbygayle", 0.0 },
  { "Abbylynn", 0.0 },
  { "Abcde", 0.0 },
  { "Abd", 1.0 },
  { "Abdala", 1.0 },
  { "Abdalah", 1.0 },
  { "Abdalla", 1.0 },
  { "Abdallah", 1.0 },
  { "Abdalrahman", 1.0 },
  { "Abdalrhman", 1.0 },
  { "Abdel", 1.0 },
  { "Abdelaziz", 1.0 },
  { "Abdelhadi", 1.0 },
  { "Abdelhamid", 1.0 },
  { "Abdelkareem", 1.0 },
  { "Abdelkarim", 1.0 },
  { "Abdellah", 1.0 },
  { "Abdelrahman", 1.0 },
  { "Abdelrhman", 1.0 },
  { "Abderrahman", 1.0 },
  { "Abdi", 0.96240604 },
  { "Abdia", 0.0 },
  { "Abdias", 1.0 },
  { "Abdiasis", 1.0 },
  { "Abdiaziz", 1.0 },
  { "Abdiel", 1.0 },
  { "Abdifatah", 1.0 },
  { "Abdihafid", 1.0 },
  { "Abdihakim", 1.0 },
  { "Abdihamid", 1.0 },
  { "Abdikadir", 1.0 },
  { "Abdikarim", 1.0 },
  { "Abdikhaliq", 1.0 },
  { "Abdilahi", 1.0 },
  { "Abdimalik", 1.0 },
  { "Abdinajib", 1.0 },
  { "Abdinasir", 1.0 },
  { "Abdiqani", 1.0 },
  { "Abdirahim", 1.0 },
  { "Abdirahin", 1.0 },
  { "Abdirahman", 1.0 },
  { "Abdirashid", 1.0 },
  { "Abdirisaq", 1.0 },
  { "Abdirizak", 1.0 },
  { "Abdisalam", 1.0 },
  { "Abdisalan", 1.0 },
  { "Abdisamad", 1.0 },
  { "Abdishakur", 1.0 },
  { "Abdiwahab", 1.0 },
  { "Abdiwali", 1.0 },
  { "Abdo", 1.0 },
  { "Abdon", 1.0 },
  { "Abdou", 1.0 },
  { "Abdoul", 1.0 },
  { "Abdoulaye", 1.0 },
  { "Abdoulaziz", 1.0 },
  { "Abdoulie", 1.0 },
  { "Abdourahman", 1.0 },
  { "Abdourahmane", 1.0 },
  { "Abdrahman", 1.0 },
  { "Abdrew", 1.0 },
  { "Abdu", 1.0 },
  { "Abdual", 1.0 },
  { "Abduallah", 1.0 },
  { "Abduel", 1.0 },
  { "Abdul", 1.0 },
  { "Abdula", 1.0 },
  { "Abdulah", 1.0 },
  { "Abdulahad", 1.0 },
  { "Abdulahi", 1.0 },
  { "Abdulai", 1.0 },
  { "Abdulaye", 1.0 },
  { "Abdulazeem", 1.0 },
  { "Abdulazeez", 1.0 },
  { "Abdulaziz", 1.0 },
  { "Abdulbari", 1.0 },
  { "Abdulbasit", 1.0 },
  { "Abdule", 1.0 },
  { "Abdulelah", 1.0 },
  { "Abdulhaadi", 1.0 },
  { "Abdulhadi", 1.0 },
  { "Abdulhakeem", 1.0 },
  { "Abdulhakim", 1.0 },
  { "Abdulhalim", 1.0 },
  { "Abdulhameed", 1.0 },
  { "Abdulhamid", 1.0 },
  { "Abduljabbar", 1.0 },
  { "Abduljaleel", 1.0 },
  { "Abduljalil", 1.0 },
  { "Abdulkadir", 1.0 },
  { "Abdulkareem", 1.0 },
  { "Abdulkarim", 1.0 },
  { "Abdulkhaliq", 1.0 },
  { "Abdull", 1.0 },
  { "Abdulla", 1.0 },
  { "Abdullah", 1.0 },
  { "Abdullahi", 1.0 },
  { "Abdullatif", 1.0 },
  { "Abdulloh", 1.0 },
  { "Abdulmajeed", 1.0 },
  { "Abdulmajid", 1.0 },
  { "Abdulmalek", 1.0 },
  { "Abdulmalik", 1.0 },
  { "Abdulmohsen", 1.0 },
  { "Abdulnasir", 1.0 },
  { "Abdulqadir", 1.0 },
  { "Abdulraheem", 1.0 },
  { "Abdulrahim", 1.0 },
  { "Abdulrahman", 1.0 },
  { "Abdulrazaq", 1.0 },
  { "Abdulrehman", 1.0 },
  { "Abdulrhman", 1.0 },
  { "Abdulsalam", 1.0 },
  { "Abdulsamad", 1.0 },
  { "Abdulwadud", 1.0 },
  { "Abdulwahab", 1.0 },
  { "Abdulwahid", 1.0 },
  { "Abdulwali", 1.0 },
  { "Abdur", 1.0 },
  { "Abdurahman", 1.0 },
  { "Abdurahmon", 1.0 },
  { "Abdurraheem", 1.0 },
  { "Abdurrahim", 1.0 },
  { "Abdurrahmaan", 1.0 },
  { "Abdurrahman", 1.0 },
  { "Abdurrehman", 1.0 },
  { "Abdussamad", 1.0 },
  { "Abe", 1.0 },
  { "Abeal", 1.0 },
  { "Abed", 1.0 },
  { "Abedallah", 1.0 },
  { "Abedalrahman", 1.0 },
  { "Abednego", 1.0 },
  { "Abeeha", 0.0 },
  { "Abeer", 0.06666667 },
  { "Abeera", 0.0 },
  { "Abegail", 0.0 },
  { "Abegale", 0.0 },
  { "Abegayle", 0.0 },
  { "Abel", 0.9976223 },
  { "Abela", 0.0 },
  { "Abelardo", 1.0 },
  { "Abelina", 0.0 },
  { "Abelino", 1.0 },
  { "Abell", 0.7826087 },
  { "Abella", 0.0 },
  { "Abem", 1.0 },
  { "Aben", 1.0 },
  { "Abena", 0.0 },
  { "Abenezer", 1.0 },
  { "Abeni", 0.0 },
  { "Aber", 0.0 },
  { "Aberdeen", 0.0 },
  { "Aberham", 1.0 },
  { "Abernathy", 0.0 },
  { "Abert", 1.0 },
  { "Abery", 0.0 },
  { "Abey", 0.37037036 },
  { "Abgail", 0.0 },
  { "Abha", 0.0 },
  { "Abhay", 1.0 },
  { "Abheek", 1.0 },
  { "Abhi", 1.0 },
  { "Abhigna", 0.0 },
  { "Abhijay", 1.0 },
  { "Abhijeet", 1.0 },
  { "Abhijit", 1.0 },
  { "Abhijot", 1.0 },
  { "Abhik", 1.0 },
  { "Abhilash", 1.0 },
  { "Abhimanyu", 1.0 },
  { "Abhinav", 1.0 },
  { "Abhinay", 1.0 },
  { "Abhinaya", 0.0 },
  { "Abhiraam", 1.0 },
  { "Abhiraj", 1.0 },
  { "Abhiram", 1.0 },
  { "Abhirami", 0.0 },
  { "Abhirup", 1.0 },
  { "Abhishek", 1.0 },
  { "Abhyuday", 1.0 },
  { "Abi", 0.043137256 },
  { "Abia", 0.0 },
  { "Abiageal", 0.0 },
  { "Abiah", 0.0 },
  { "Abian", 1.0 },
  { "Abianna", 0.0 },
  { "Abibail", 0.0 },
  { "Abid", 1.0 },
  { "Abida", 0.0 },
  { "Abidah", 0.0 },
  { "Abidan", 1.0 },
  { "Abie", 0.90589887 },
  { "Abiegail", 0.0 },
  { "Abiel", 0.9787037 },
  { "Abiela", 0.0 },
  { "Abiella", 0.0 },
  { "Abiezer", 1.0 },
  { "Abigael", 0.0030826142 },
  { "Abigaelle", 0.0 },
  { "Abigahil", 0.0 },
  { "Abigai", 0.0 },
  { "Abigail", 0.0017929892 },
  { "Abigaile", 0.0 },
  { "Abigailgrace", 0.0 },
  { "Abigaille", 0.0 },
  { "Abigailmarie", 0.0 },
  { "Abigailrose", 0.0 },
  { "Abigal", 0.0 },
  { "Abigale", 0.0 },
  { "Abigayil", 0.0 },
  { "Abigayl", 0.0 },
  { "Abigayle", 0.00085984526 },
  { "Abigeal", 0.0 },
  { "Abigel", 0.0 },
  { "Abigial", 0.0 },
  { "Abiha", 0.0 },
  { "Abihail", 0.0 },
  { "Abijah", 0.8322368 },
  { "Abilene", 0.0 },
  { "Abilgail", 0.0 },
  { "Abilio", 1.0 },
  { "Abilyn", 0.0 },
  { "Abilynn", 0.0 },
  { "Abimael", 1.0 },
  { "Abimbola", 0.05882353 },
  { "Abimelec", 1.0 },
  { "Abin", 1.0 },
  { "Abinadab", 1.0 },
  { "Abinadi", 1.0 },
  { "Abinav", 1.0 },
  { "Abinaya", 0.0 },
  { "Abiodun", 1.0 },
  { "Abiola", 0.22564103 },
  { "Abiona", 0.0 },
  { "Abir", 0.7467363 },
  { "Abira", 0.0 },
  { "Abiram", 1.0 },
  { "Abirami", 0.0 },
  { "Abisag", 0.0 },
  { "Abisai", 0.9417143 },
  { "Abisaid", 1.0 },
  { "Abish", 0.0 },
  { "Abisha", 0.0 },
  { "Abishai", 0.865285 },
  { "Abishek", 1.0 },
  { "Abisola", 0.0 },
  { "Abiud", 1.0 },
  { "Abiyah", 0.29411766 },
  { "Abla", 0.0 },
  { "Able", 1.0 },
  { "Abnel", 1.0 },
  { "Abner", 1.0 },
  { "Abney", 0.33333334 },
  { "Abony", 0.0 },
  { "Abou", 1.0 },
  { "Aboubacar", 1.0 },
  { "Aboubakar", 1.0 },
  { "Abra", 0.0 },
  { "Abraam", 1.0 },
  { "Abraar", 1.0 },
  { "Abrah", 0.0 },
  { "Abraham", 0.99765337 },
  { "Abrahan", 1.0 },
  { "Abraheem", 1.0 },
  { "Abrahem", 1.0 },
  { "Abrahim", 1.0 },
  { "Abrahm", 1.0 },
  { "Abram", 1.0 },
  { "Abran", 1.0 },
  { "Abranda", 0.0 },
  { "Abrar", 0.47467437 },
  { "Abraxas", 0.54545456 },
  { "Abraya", 0.0 },
  { "Abrea", 0.0 },
  { "Abreana", 0.0 },
  { "Abreanna", 0.0 },
  { "Abree", 0.0 },
  { "Abreia", 0.0 },
  { "Abrem", 1.0 },
  { "Abren", 1.0 },
  { "Abreona", 0.0 },
  { "Abreonna", 0.0 },
  { "Abrey", 0.0 },
  { "Abreya", 0.0 },
  { "Abrham", 1.0 },
  { "Abri", 0.0 },
  { "Abria", 0.0 },
  { "Abriah", 0.0 },
  { "Abrial", 0.0 },
  { "Abriam", 1.0 },
  { "Abrian", 0.98355263 },
  { "Abriana", 0.0 },
  { "Abrianna", 0.0 },
  { "Abriannah", 0.0 },
  { "Abrianne", 0.0 },
  { "Abrie", 0.0 },
  { "Abriel", 0.37291667 },
  { "Abriela", 0.0 },
  { "Abriele", 0.0 },
  { "Abriella", 0.0 },
  { "Abrielle", 0.0 },
  { "Abrien", 1.0 },
  { "Abrienne", 0.0 },
  { "Abrigail", 0.0 },
  { "Abrihet", 0.0 },
  { "Abril", 0.00068832596 },
  { "Abrille", 0.0 },
  { "Abrina", 0.0 },
  { "Abrion", 1.0 },
  { "Abriona", 0.0 },
  { "Abrionna", 0.0 },
  { "Abrish", 0.0 },
  { "Abriya", 0.0 },
  { "Abriyah", 0.0 },
  { "Abriyana", 0.0 },
  { "Abrom", 1.0 },
  { "Abron", 1.0 },
  { "Abrum", 1.0 },
  { "Abry", 0.0 },
  { "Abryana", 0.0 },
  { "Abryanna", 0.0 },
  { "Abryella", 0.0 },
  { "Abryelle", 0.0 },
  { "Abryl", 0.0 },
  { "Absalat", 0.0 },
  { "Absalom", 1.0 },
  { "Absalon", 1.0 },
  { "Abshir", 1.0 },
  { "Absidy", 0.0 },
  { "Abtin", 1.0 },
  { "Abu", 1.0 },
  { "Abubacar", 1.0 },
  { "Abubacarr", 1.0 },
  { "Abubakar", 1.0 },
  { "Abubakarr", 1.0 },
  { "Abubakary", 1.0 },
  { "Abubaker", 1.0 },
  { "Abubakr", 1.0 },
  { "Abuk", 0.0 },
  { "Abukar", 1.0 },
  { "Abundio", 1.0 },
  { "Aby", 0.21705426 },
  { "Abyade", 0.0 },
  { "Abyan", 0.7910448 },
  { "Abygael", 0.0 },
  { "Abygail", 0.0 },
  { "Abygaile", 0.0 },
  { "Abygale", 0.0 },
  { "Abygayle", 0.0 },
  { "Abyssinia", 0.0 },
  { "Ac", 1.0 },
  { "Acacia", 0.0 },
  { "Acacius", 1.0 },
  { "Acadia", 0.0 },
  { "Acamas", 1.0 },
  { "Acari", 1.0 },
  { "Acasia", 0.0 },
  { "Accacia", 0.0 },
  { "Accalia", 0.0 },
  { "Access", 1.0 },
  { "Accie", 1.0 },
  { "Accursio", 1.0 },
  { "Ace", 0.9972386 },
  { "Acea", 0.9350649 },
  { "Acein", 1.0 },
  { "Aceion", 1.0 },
  { "Acel", 1.0 },
  { "Acelee", 0.0 },
  { "Acelin", 0.848 },
  { "Acelino", 1.0 },
  { "Acelyn", 0.029023746 },
  { "Acelynn", 0.0 },
  { "Acen", 1.0 },
  { "Acencion", 1.0 },
  { "Aceon", 1.0 },
  { "Acer", 1.0 },
  { "Aceson", 1.0 },
  { "Aceston", 1.0 },
  { "Acesyn", 1.0 },
  { "Aceton", 1.0 },
  { "Acey", 0.9489559 },
  { "Aceyn", 1.0 },
  { "Achai", 1.0 },
  { "Achaia", 0.0 },
  { "Achaius", 1.0 },
  { "Achan", 0.0 },
  { "Achante", 0.0 },
  { "Achanti", 0.0 },
  { "Achary", 1.0 },
  { "Achazia", 0.0 },
  { "Achel", 0.0 },
  { "Acheron", 1.0 },
  { "Achille", 1.0 },
  { "Achilles", 1.0 },
  { "Achilleus", 1.0 },
  { "Achillies", 1.0 },
  { "Achintya", 1.0 },
  { "Achol", 0.0 },
  { "Achraf", 1.0 },
  { "Achsa", 0.0 },
  { "Achsah", 0.0 },
  { "Achyut", 1.0 },
  { "Achyuth", 1.0 },
  { "Acia", 0.0 },
  { "Aciano", 1.0 },
  { "Acie", 0.97597307 },
  { "Aciel", 1.0 },
  { "Acil", 1.0 },
  { "Acire", 0.0 },
  { "Ackeem", 1.0 },
  { "Ackley", 1.0 },
  { "Acob", 1.0 },
  { "Acquanetta", 0.0 },
  { "Acquanette", 0.0 },
  { "Acsa", 0.0 },
  { "Acura", 0.0 },
  { "Acxel", 1.0 },
  { "Acy", 0.98207885 },
  { "Ad", 1.0 },
  { "Ada", 0.0029549503 },
  { "Adabel", 0.0 },
  { "Adabella", 0.0 },
  { "Adabelle", 0.0 },
  { "Adacia", 0.0 },
  { "Adae", 1.0 },
  { "Adael", 1.0 },
  { "Adaela", 0.0 },
  { "Adaelyn", 0.0 },
  { "Adaeze", 0.0 },
  { "Adagio", 1.0 },
  { "Adah", 0.0 },
  { "Adahir", 1.0 },
  { "Adahli", 0.0 },
  { "Adahlia", 0.0 },
  { "Adahy", 1.0 },
  { "Adai", 0.0 },
  { "Adaia", 0.0 },
  { "Adaiah", 0.14942528 },
  { "Adaija", 0.0 },
  { "Adaijah", 0.0 },
  { "Adailyn", 0.0 },
  { "Adain", 1.0 },
  { "Adair", 0.5322948 },
  { "Adaira", 0.0 },
  { "Adaire", 0.0 },
  { "Adairis", 0.0 },
  { "Adaisha", 0.0 },
  { "Adaisia", 0.0 },
  { "Adaja", 0.0 },
  { "Adajah", 0.0 },
  { "Adaku", 0.0 },
  { "Adal", 1.0 },
  { "Adala", 0.0 },
  { "Adalade", 0.0 },
  { "Adalae", 0.0 },
  { "Adalai", 0.0 },
  { "Adalaide", 0.0 },
  { "Adalay", 0.0 },
  { "Adalaya", 0.0 },
  { "Adalayde", 0.0 },
  { "Adalbert", 1.0 },
  { "Adalberto", 1.0 },
  { "Adale", 0.0 },
  { "Adalea", 0.0 },
  { "Adaleah", 0.0 },
  { "Adalee", 0.0 },
  { "Adaleen", 0.0 },
  { "Adaleena", 0.0 },
  { "Adalei", 0.0 },
  { "Adaleia", 0.0 },
  { "Adaleigh", 0.0 },
  { "Adaleine", 0.0 },
  { "Adalen", 0.0 },
  { "Adalena", 0.0 },
  { "Adalene", 0.0 },
  { "Adaley", 0.0 },
  { "Adaleya", 0.0 },
  { "Adaleyza", 0.0 },
  { "Adalhi", 0.0 },
  { "Adali", 0.055555556 },
  { "Adalia", 0.0 },
  { "Adaliah", 0.0 },
  { "Adalicia", 0.0 },
  { "Adalid", 0.9 },
  { "Adalida", 0.0 },
  { "Adalie", 0.0 },
  { "Adaliene", 0.0 },
  { "Adalin", 0.0 },
  { "Adalina", 0.0 },
  { "Adalind", 0.0 },
  { "Adalinda", 0.0 },
  { "Adaline", 0.0 },
  { "Adalinn", 0.0 },
  { "Adalinne", 0.0 },
  { "Adalis", 0.0 },
  { "Adalisa", 0.0 },
  { "Adalise", 0.0 },
  { "Adalisse", 0.0 },
  { "Adalius", 1.0 },
  { "Adaliyah", 0.0 },
  { "Adaliz", 0.0 },
  { "Adalize", 0.0 },
  { "Adallyn", 0.0 },
  { "Adaly", 0.0 },
  { "Adalya", 0.0 },
  { "Adalye", 0.0 },
  { "Adalyn", 0.0 },
  { "Adalyna", 0.0 },
  { "Adalynd", 0.0 },
  { "Adalyne", 0.0 },
  { "Adalynn", 0.0 },
  { "Adalynne", 0.0 },
  { "Adalys", 0.0 },
  { "Adalyse", 0.0 },
  { "Adam", 0.9962988 },
  { "Adama", 0.40409207 },
  { "Adamae", 0.0 },
  { "Adamari", 0.0 },
  { "Adamarie", 0.0 },
  { "Adamaris", 0.0 },
  { "Adamariz", 0.0 },
  { "Adamary", 0.0 },
  { "Adamarys", 0.0 },
  { "Adamina", 0.0 },
  { "Adamjames", 1.0 },
  { "Adamm", 1.0 },
  { "Adamma", 0.0 },
  { "Adammichael", 1.0 },
  { "Adamo", 1.0 },
  { "Adams", 1.0 },
  { "Adan", 0.9959742 },
  { "Adana", 0.0 },
  { "Adaneli", 0.0 },
  { "Adanelly", 0.0 },
  { "Adanely", 0.0 },
  { "Adanna", 0.0 },
  { "Adannaya", 0.0 },
  { "Adante", 1.0 },
  { "Adanya", 0.0 },
  { "Adaobi", 0.0 },
  { "Adaora", 0.0 },
  { "Adar", 0.6421053 },
  { "Adara", 0.0 },
  { "Adarah", 0.0 },
  { "Adari", 1.0 },
  { "Adaria", 0.0 },
  { "Adarian", 1.0 },
  { "Adarien", 1.0 },
  { "Adarion", 1.0 },
  { "Adarious", 1.0 },
  { "Adarius", 1.0 },
  { "Adarrion", 1.0 },
  { "Adarrius", 1.0 },
  { "Adarryl", 1.0 },
  { "Adarryll", 1.0 },
  { "Adarsh", 1.0 },
  { "Adaryl", 1.0 },
  { "Adaryll", 1.0 },
  { "Adasha", 0.0 },
  { "Adashia", 0.0 },
  { "Adasia", 0.0 },
  { "Adason", 0.5 },
  { "Adassa", 0.0 },
  { "Adasyn", 0.0 },
  { "Adasynn", 0.0 },
  { "Adau", 0.0 },
  { "Adaugo", 0.0 },
  { "Adaure", 0.0 },
  { "Adavia", 0.0 },
  { "Adavion", 1.0 },
  { "Adaya", 0.0 },
  { "Adayah", 0.0 },
  { "Adayla", 0.0 },
  { "Adaysha", 0.0 },
  { "Adayshia", 0.0 },
  { "Adaysia", 0.0 },
  { "Adbeel", 1.0 },
  { "Adbiel", 1.0 },
  { "Add", 1.0 },
  { "Adda", 0.0 },
  { "Addah", 0.0 },
  { "Addai", 1.0 },
  { "Addalee", 0.0 },
  { "Addaleigh", 0.0 },
  { "Addaley", 0.0 },
  { "Addalie", 0.0 },
  { "Addalin", 0.0 },
  { "Addalina", 0.0 },
  { "Addaline", 0.0 },
  { "Addallee", 0.0 },
  { "Addaly", 0.0 },
  { "Addalyn", 0.0 },
  { "Addalyne", 0.0 },
  { "Addalynn", 0.0 },
  { "Addalynne", 0.0 },
  { "Addam", 1.0 },
  { "Addan", 1.0 },
  { "Addasyn", 0.0 },
  { "Addelaide", 0.0 },
  { "Addelin", 0.0 },
  { "Addelina", 0.0 },
  { "Addeline", 0.0 },
  { "Addell", 0.0 },
  { "Addelyn", 0.0 },
  { "Addelyne", 0.0 },
  { "Addelynn", 0.0 },
  { "Addelynne", 0.0 },
  { "Adden", 1.0 },
  { "Adderly", 1.0 },
  { "Addeson", 0.0 },
  { "Addesyn", 0.0 },
  { "Addey", 0.0 },
  { "Addi", 0.0 },
  { "Addia", 0.0 },
  { "Addicus", 1.0 },
  { "Addicyn", 0.0 },
  { "Addie", 0.016165346 },
  { "Addiel", 1.0 },
  { "Addielynn", 0.0 },
  { "Addiemae", 0.0 },
  { "Addieson", 0.0 },
  { "Addilee", 0.0 },
  { "Addileigh", 0.0 },
  { "Addiley", 0.0 },
  { "Addiline", 0.0 },
  { "Addilyn", 0.0010845987 },
  { "Addilyne", 0.0 },
  { "Addilynn", 0.0 },
  { "Addilynne", 0.0 },
  { "Addine", 0.0 },
  { "Addington", 0.61538464 },
  { "Addis", 0.71482176 },
  { "Addisan", 0.0 },
  { "Addisen", 0.005550416 },
  { "Addisin", 0.0 },
  { "Addison", 0.10249436 },
  { "Addisson", 0.0 },
  { "Addiston", 0.0 },
  { "Addisun", 0.0 },
  { "Addisyn", 0.0 },
  { "Addisynn", 0.0 },
  { "Addix", 1.0 },
  { "Addle", 0.0 },
  { "Addlee", 0.0 },
  { "Addley", 0.0 },
  { "Addlie", 0.0 },
  { "Addlyn", 0.0 },
  { "Addlynn", 0.0 },
  { "Addonis", 1.0 },
  { "Addriana", 0.0 },
  { "Addrianna", 0.0 },
  { "Addrienne", 0.0 },
  { "Addsion", 0.0 },
  { "Addy", 0.019315189 },
  { "Addylan", 0.0 },
  { "Addylen", 0.0 },
  { "Addylin", 0.0 },
  { "Addyline", 0.0 },
  { "Addylyn", 0.0 },
  { "Addylynn", 0.0 },
  { "Addysan", 0.0 },
  { "Addysen", 0.0 },
  { "Addysin", 0.0 },
  { "Addysn", 0.0 },
  { "Addyson", 0.002336118 },
  { "Addysun", 0.0 },
  { "Addysyn", 0.0 },
  { "Ade", 0.9607143 },
  { "Adea", 0.0 },
  { "Adean", 0.19827586 },
  { "Adeana", 0.0 },
  { "Adeanna", 0.0 },
  { "Adebayo", 1.0 },
  { "Adebisi", 0.0 },
  { "Adebola", 0.2 },
  { "Adebowale", 1.0 },
  { "Adecyn", 0.0 },
  { "Adedamola", 1.0 },
  { "Adedayo", 1.0 },
  { "Adedeji", 1.0 },
  { "Adedotun", 1.0 },
  { "Adedoyin", 0.29411766 },
  { "Adee", 0.0 },
  { "Adeeb", 1.0 },
  { "Adeeba", 0.0 },
  { "Adeel", 1.0 },
  { "Adeem", 1.0 },
  { "Adeen", 1.0 },
  { "Adeena", 0.0 },
  { "Adefolarin", 1.0 },
  { "Adein", 1.0 },
  { "Adeja", 0.0 },
  { "Adejah", 0.0 },
  { "Adekemi", 0.0 },
  { "Adekunle", 1.0 },
  { "Adel", 0.4785399 },
  { "Adela", 0.0 },
  { "Adelade", 0.0 },
  { "Adelae", 0.0 },
  { "Adelah", 0.0 },
  { "Adelai", 0.0 },
  { "Adelaid", 0.0 },
  { "Adelaida", 0.0 },
  { "Adelaide", 0.0 },
  { "Adelaido", 1.0 },
  { "Adelaina", 0.0 },
  { "Adelaine", 0.0 },
  { "Adelais", 0.0 },
  { "Adelane", 0.0 },
  { "Adelard", 1.0 },
  { "Adelay", 0.0 },
  { "Adelaya", 0.0 },
  { "Adelayda", 0.0 },
  { "Adelayde", 0.0 },
  { "Adelayne", 0.0 },
  { "Adelbert", 1.0 },
  { "Adele", 0.00028014058 },
  { "Adelea", 0.0 },
  { "Adelean", 0.0 },
  { "Adelee", 0.0 },
  { "Adeleen", 0.0 },
  { "Adelei", 0.0 },
  { "Adeleigh", 0.0 },
  { "Adeleine", 0.0 },
  { "Adeleke", 1.0 },
  { "Adelen", 0.0 },
  { "Adelena", 0.0 },
  { "Adelene", 0.0 },
  { "Adeley", 0.0 },
  { "Adelfa", 0.0 },
  { "Adelfina", 0.0 },
  { "Adelfo", 1.0 },
  { "Adelheid", 0.0 },
  { "Adeli", 0.0 },
  { "Adelia", 0.0 },
  { "Adeliah", 0.0 },
  { "Adelice", 0.0 },
  { "Adelicia", 0.0 },
  { "Adelie", 0.0 },
  { "Adelilah", 0.0 },
  { "Adelin", 0.0 },
  { "Adelina", 0.0 },
  { "Adelinah", 0.0 },
  { "Adelind", 0.0 },
  { "Adeline", 0.00050698075 },
  { "Adelinn", 0.0 },
  { "Adelinne", 0.0 },
  { "Adelino", 1.0 },
  { "Adelis", 0.0 },
  { "Adelisa", 0.0 },
  { "Adelise", 0.0 },
  { "Adelita", 0.0 },
  { "Adeliz", 0.0 },
  { "Adell", 0.14569445 },
  { "Adella", 0.0 },
  { "Adelle", 0.0 },
  { "Adellia", 0.0 },
  { "Adelline", 0.0 },
  { "Adellyn", 0.0 },
  { "Adellynn", 0.0 },
  { "Adelma", 0.0 },
  { "Adelmira", 0.0 },
  { "Adelmo", 1.0 },
  { "Adelore", 1.0 },
  { "Adelphia", 0.0 },
  { "Adely", 0.0 },
  { "Adelya", 0.0 },
  { "Adelyn", 0.0004621428 },
  { "Adelyna", 0.0 },
  { "Adelyne", 0.0 },
  { "Adelynn", 0.0 },
  { "Adelynne", 0.0 },
  { "Adelys", 0.0 },
  { "Adem", 1.0 },
  { "Ademar", 1.0 },
  { "Ademide", 0.8888889 },
  { "Ademir", 1.0 },
  { "Ademola", 1.0 },
  { "Aden", 0.9709961 },
  { "Adena", 0.0 },
  { "Adene", 0.0 },
  { "Adenike", 0.0 },
  { "Adeniyi", 1.0 },
  { "Adenn", 1.0 },
  { "Adeola", 0.082446806 },
  { "Adeolu", 1.0 },
  { "Adeoluwa", 1.0 },
  { "Adeon", 1.0 },
  { "Ader", 0.0 },
  { "Adera", 0.0 },
  { "Aderemi", 1.0 },
  { "Aderinsola", 0.0 },
  { "Adero", 0.0 },
  { "Aderonke", 0.0 },
  { "Aderyn", 0.0 },
  { "Adesewa", 0.0 },
  { "Adesh", 1.0 },
  { "Adesha", 0.0 },
  { "Adesina", 0.0 },
  { "Adesire", 0.0 },
  { "Adesola", 0.0 },
  { "Adessa", 0.0 },
  { "Adesuwa", 0.0 },
  { "Adetayo", 1.0 },
  { "Adetokunbo", 0.94736844 },
  { "Adetola", 1.0 },
  { "Adeva", 0.0 },
  { "Adewale", 1.0 },
  { "Adey", 0.0 },
  { "Adeyemi", 1.0 },
  { "Adeyinka", 0.5 },
  { "Adger", 1.0 },
  { "Adgie", 0.41666666 },
  { "Adham", 1.0 },
  { "Adhara", 0.0 },
  { "Adharsh", 1.0 },
  { "Adharv", 1.0 },
  { "Adhav", 1.0 },
  { "Adheesh", 1.0 },
  { "Adhem", 1.0 },
  { "Adhemar", 1.0 },
  { "Adhira", 0.0 },
  { "Adhiraj", 1.0 },
  { "Adhithi", 0.0 },
  { "Adhithya", 1.0 },
  { "Adhitya", 1.0 },
  { "Adhley", 0.0 },
  { "Adhrit", 1.0 },
  { "Adhrith", 1.0 },
  { "Adhvaith", 1.0 },
  { "Adhvik", 1.0 },
  { "Adhvika", 0.0 },
  { "Adhya", 0.0 },
  { "Adhyan", 1.0 },
  { "Adi", 0.4127726 },
  { "Adia", 0.0 },
  { "Adiah", 0.0 },
  { "Adiam", 0.0 },
  { "Adian", 1.0 },
  { "Adiana", 0.0 },
  { "Adianey", 0.0 },
  { "Adianez", 0.0 },
  { "Adianna", 0.0 },
  { "Adib", 1.0 },
  { "Adiba", 0.0 },
  { "Adidas", 1.0 },
  { "Adie", 0.093023255 },
  { "Adiel", 0.9402314 },
  { "Adiela", 0.0 },
  { "Adiella", 0.0 },
  { "Adielle", 0.0 },
  { "Adien", 1.0 },
  { "Adijah", 0.0 },
  { "Adika", 1.0 },
  { "Adikus", 1.0 },
  { "Adil", 1.0 },
  { "Adila", 0.0 },
  { "Adilah", 0.0 },
  { "Adilee", 0.0 },
  { "Adileigh", 0.0 },
  { "Adilen", 0.0 },
  { "Adilena", 0.0 },
  { "Adilene", 0.0013647885 },
  { "Adileni", 0.0 },
  { "Adilenne", 0.0 },
  { "Adileny", 0.0 },
  { "Adiley", 0.0 },
  { "Adilia", 0.0 },
  { "Adiline", 0.0 },
  { "Adilson", 1.0 },
  { "Adilyn", 0.0 },
  { "Adilyne", 0.0 },
  { "Adilynn", 0.0 },
  { "Adilynne", 0.0 },
  { "Adin", 0.96885186 },
  { "Adina", 0.0 },
  { "Adinah", 0.0 },
  { "Adine", 0.0 },
  { "Adir", 1.0 },
  { "Adira", 0.0 },
  { "Adirah", 0.0 },
  { "Adis", 0.94871795 },
  { "Adisa", 0.41830066 },
  { "Adisen", 0.0 },
  { "Adish", 1.0 },
  { "Adison", 0.171875 },
  { "Adisson", 0.058139537 },
  { "Adisyn", 0.0 },
  { "Adisynn", 0.0 },
  { "Adisynne", 0.0 },
  { "Adit", 1.0 },
  { "Adith", 1.0 },
  { "Adithi", 0.0 },
  { "Adithri", 0.0 },
  { "Adithya", 1.0 },
  { "Aditi", 0.0 },
  { "Aditri", 0.0 },
  { "Aditya", 0.99889505 },
  { "Adiv", 1.0 },
  { "Adiva", 0.0 },
  { "Adiya", 0.0 },
  { "Adiyah", 0.0 },
  { "Adiyan", 1.0 },
  { "Adja", 0.0 },
  { "Adji", 0.0 },
  { "Adjoa", 0.0 },
  { "Adjua", 0.0 },
  { "Adlai", 0.8981818 },
  { "Adlan", 1.0 },
  { "Adlar", 1.0 },
  { "Adlea", 0.0 },
  { "Adlean", 0.0 },
  { "Adlee", 0.0 },
  { "Adlei", 0.0 },
  { "Adleigh", 0.0 },
  { "Adlemi", 0.0 },
  { "Adlen", 0.0 },
  { "Adlena", 0.0 },
  { "Adlene", 0.0 },
  { "Adler", 0.90428346 },
  { "Adley", 0.14347638 },
  { "Adlia", 1.0 },
  { "Adlie", 0.0 },
  { "Adlih", 0.0 },
  { "Adlin", 0.0 },
  { "Adlina", 0.0 },
  { "Adline", 0.0 },
  { "Adly", 0.0 },
  { "Adlyn", 0.0 },
  { "Adlynn", 0.0 },
  { "Adma", 1.0 },
  { "Admir", 1.0 },
  { "Admiral", 1.0 },
  { "Admire", 0.07936508 },
  { "Adn", 0.0 },
  { "Adna", 0.02948403 },
  { "Adnaan", 1.0 },
  { "Adnan", 1.0 },
  { "Adnel", 1.0 },
  { "Adner", 1.0 },
  { "Adniel", 1.0 },
  { "Adnrea", 0.0 },
  { "Adnrew", 1.0 },
  { "Ado", 1.0 },
  { "Adolf", 1.0 },
  { "Adolfo", 0.9993473 },
  { "Adoline", 0.0 },
  { "Adolph", 0.99893224 },
  { "Adolpha", 0.0 },
  { "Adolphe", 1.0 },
  { "Adolphine", 0.0 },
  { "Adolpho", 1.0 },
  { "Adolphus", 1.0 },
  { "Adom", 1.0 },
  { "Adomas", 1.0 },
  { "Adon", 1.0 },
  { "Adona", 0.0 },
  { "Adonai", 0.78740156 },
  { "Adonay", 1.0 },
  { "Adoncia", 0.0 },
  { "Adones", 1.0 },
  { "Adoni", 1.0 },
  { "Adonia", 0.0 },
  { "Adoniah", 0.6363636 },
  { "Adonias", 1.0 },
  { "Adonica", 0.0 },
  { "Adonijah", 0.9322917 },
  { "Adoniram", 1.0 },
  { "Adonis", 0.9904786 },
  { "Adoniyah", 1.0 },
  { "Adoniz", 1.0 },
  { "Adonna", 0.0 },
  { "Adonnis", 1.0 },
  { "Adontae", 1.0 },
  { "Adonte", 1.0 },
  { "Adonus", 1.0 },
  { "Adony", 1.0 },
  { "Adonya", 0.0 },
  { "Adonys", 1.0 },
  { "Ador", 0.0 },
  { "Adora", 0.0 },
  { "Adorabella", 0.0 },
  { "Adorable", 0.0 },
  { "Adorae", 0.0 },
  { "Adorah", 0.0 },
  { "Adoraim", 1.0 },
  { "Adore", 0.08154506 },
  { "Adoree", 0.0 },
  { "Adoria", 0.0 },
  { "Adorian", 1.0 },
  { "Adorion", 1.0 },
  { "Adorn", 0.0 },
  { "Adra", 0.0 },
  { "Adrain", 0.8792445 },
  { "Adraine", 0.022421524 },
  { "Adrainne", 0.0 },
  { "Adran", 1.0 },
  { "Adraya", 0.0 },
  { "Adre", 1.0 },
  { "Adrea", 0.0 },
  { "Adream", 0.0 },
  { "Adrean", 0.6390845 },
  { "Adreana", 0.0 },
  { "Adreann", 0.0 },
  { "Adreanna", 0.0 },
  { "Adreanne", 0.0 },
  { "Adreauna", 0.0 },
  { "Adree", 0.0 },
  { "Adreena", 0.0 },
  { "Adreian", 1.0 },
  { "Adrein", 1.0 },
  { "Adreina", 0.0 },
  { "Adreinne", 0.0 },
  { "Adrell", 1.0 },
  { "Adren", 1.0 },
  { "Adrena", 0.0 },
  { "Adrene", 0.0 },
  { "Adreon", 1.0 },
  { "Adreona", 0.0 },
  { "Adreonna", 0.0 },
  { "Adrew", 1.0 },
  { "Adrey", 0.0 },
  { "Adreyan", 1.0 },
  { "Adreyona", 0.0 },
  { "Adri", 0.104166664 },
  { "Adria", 0.0011780545 },
  { "Adriaan", 1.0 },
  { "Adriah", 0.0 },
  { "Adriahna", 0.0 },
  { "Adrial", 1.0 },
  { "Adrialis", 0.0 },
  { "Adrialys", 0.0 },
  { "Adriam", 1.0 },
  { "Adrian", 0.9372302 },
  { "Adriana", 0.0045941807 },
  { "Adrianah", 0.0 },
  { "Adriane", 0.043163203 },
  { "Adrianjames", 1.0 },
  { "Adriann", 0.10028929 },
  { "Adrianna", 0.00041308088 },
  { "Adriannah", 0.0 },
  { "Adrianne", 0.0023732965 },
  { "Adrianno", 1.0 },
  { "Adrianny", 0.0 },
  { "Adriano", 1.0 },
  { "Adrianpaul", 1.0 },
  { "Adriauna", 0.0 },
  { "Adriaunna", 0.0 },
  { "Adric", 1.0 },
  { "Adrick", 1.0 },
  { "Adrie", 0.0 },
  { "Adriean", 1.0 },
  { "Adrieana", 0.0 },
  { "Adrieanna", 0.0 },
  { "Adrieanne", 0.0 },
  { "Adriel", 0.9319381 },
  { "Adriela", 0.0 },
  { "Adriell", 1.0 },
  { "Adriella", 0.0 },
  { "Adrielle", 0.0 },
  { "Adrien", 0.8520091 },
  { "Adriena", 0.0 },
  { "Adriene", 0.0251962 },
  { "Adrienn", 0.20588236 },
  { "Adrienna", 0.0 },
  { "Adrienne", 0.005717878 },
  { "Adriennie", 0.0 },
  { "Adrieonna", 0.0 },
  { "Adrihana", 0.0 },
  { "Adrihanna", 0.0 },
  { "Adrija", 0.0 },
  { "Adrijana", 0.0 },
  { "Adrik", 1.0 },
  { "Adrika", 0.0 },
  { "Adrin", 0.966759 },
  { "Adrina", 0.0 },
  { "Adrine", 0.072463766 },
  { "Adrinna", 0.0 },
  { "Adrinne", 0.0 },
  { "Adrion", 1.0 },
  { "Adriona", 0.0 },
  { "Adrionna", 0.0 },
  { "Adris", 1.0 },
  { "Adrita", 0.0 },
  { "Adrith", 1.0 },
  { "Adriunna", 0.0 },
  { "Adrius", 1.0 },
  { "Adriyan", 1.0 },
  { "Adriyana", 0.0 },
  { "Adriyanna", 0.0 },
  { "Adriyel", 1.0 },
  { "Adron", 1.0 },
  { "Adry", 0.0 },
  { "Adryan", 0.8610635 },
  { "Adryana", 0.0 },
  { "Adryanah", 0.0 },
  { "Adryann", 0.0 },
  { "Adryanna", 0.0 },
  { "Adryauna", 0.0 },
  { "Adryel", 1.0 },
  { "Adryen", 1.0 },
  { "Adryian", 1.0 },
  { "Adryn", 0.8125 },
  { "Adryon", 1.0 },
  { "Adryonna", 0.0 },
  { "Adulfo", 1.0 },
  { "Adut", 0.0 },
  { "Advait", 1.0 },
  { "Advaita", 0.0 },
  { "Advaith", 1.0 },
  { "Advay", 1.0 },
  { "Advik", 1.0 },
  { "Advika", 0.0 },
  { "Advit", 1.0 },
  { "Advita", 0.0 },
  { "Advith", 1.0 },
  { "Advitha", 0.0 },
  { "Adwaith", 1.0 },
  { "Adwin", 1.0 },
  { "Adwit", 1.0 },
  { "Adwita", 0.0 },
  { "Adwitiya", 0.0 },
  { "Adwoa", 0.0 },
  { "Ady", 0.0 },
  { "Adya", 0.0 },
  { "Adyaan", 1.0 },
  { "Adyan", 1.0 },
  { "Adyanna", 0.0 },
  { "Adyant", 1.0 },
  { "Adyen", 1.0 },
  { "Adylan", 0.0 },
  { "Adylee", 0.0 },
  { "Adyleigh", 0.0 },
  { "Adylen", 0.0 },
  { "Adylene", 0.0 },
  { "Adylenne", 0.0 },
  { "Adylin", 0.0 },
  { "Adylina", 0.0 },
  { "Adyline", 0.0 },
  { "Adylinn", 0.0 },
  { "Adylyn", 0.0 },
  { "Adylynn", 0.0 },
  { "Adym", 1.0 },
  { "Adyn", 0.9129771 },
  { "Adynn", 0.7875 },
  { "Adysan", 0.0 },
  { "Adysen", 0.0 },
  { "Adysin", 0.0 },
  { "Adyson", 0.008886565 },
  { "Adysson", 0.0 },
  { "Adysun", 0.0 },
  { "Adysyn", 0.0 },
  { "Aedan", 0.98676515 },
  { "Aeddon", 1.0 },
  { "Aeden", 0.99272346 },
  { "Aedin", 1.0 },
  { "Aedon", 1.0 },
  { "Aedric", 1.0 },
  { "Aedyn", 0.8728223 },
  { "Aeiden", 1.0 },
  { "Aeiress", 0.0 },
  { "Aeisha", 0.0 },
  { "Aeja", 0.0 },
  { "Aela", 0.0 },
  { "Aelan", 0.0 },
  { "Aelani", 0.0 },
  { "Aeleen", 0.0 },
  { "Aelia", 0.0 },
  { "Aeliana", 0.0 },
  { "Aelicia", 0.0 },
  { "Aelin", 0.0 },
  { "Aelisa", 0.0 },
  { "Aelish", 0.0 },
  { "Aelita", 0.0 },
  { "Aella", 0.0 },
  { "Aelred", 1.0 },
  { "Aelyn", 0.0 },
  { "Aelynn", 0.0 },
  { "Aemelia", 0.0 },
  { "Aemilia", 0.0 },
  { "Aemon", 1.0 },
  { "Aenea", 0.0 },
  { "Aeneas", 1.0 },
  { "Aengus", 1.0 },
  { "Aeon", 0.5637584 },
  { "Aeona", 0.0 },
  { "Aeowyn", 0.0 },
  { "Aerabella", 0.0 },
  { "Aeralyn", 0.0 },
  { "Aeralynn", 0.0 },
  { "Aeri", 0.0 },
  { "Aerial", 0.0032102729 },
  { "Aerian", 1.0 },
  { "Aeriana", 0.0 },
  { "Aerianna", 0.0 },
  { "Aeric", 1.0 },
  { "Aerica", 0.0 },
  { "Aerick", 1.0 },
  { "Aericka", 0.0 },
  { "Aerie", 0.0 },
  { "Aeriel", 0.0 },
  { "Aeriell", 0.0 },
  { "Aeriella", 0.0 },
  { "Aerielle", 0.0 },
  { "Aeries", 0.0 },
  { "Aerik", 1.0 },
  { "Aerika", 0.0 },
  { "Aerilyn", 0.0 },
  { "Aerilynn", 0.0 },
  { "Aerin", 0.021563342 },
  { "Aerion", 0.82608694 },
  { "Aeriona", 0.0 },
  { "Aerionna", 0.0 },
  { "Aeris", 0.0049916804 },
  { "Aeriss", 0.0 },
  { "Aerith", 0.0 },
  { "Aerius", 1.0 },
  { "Aero", 0.82901555 },
  { "Aeron", 0.8939641 },
  { "Aerowyn", 0.0 },
  { "Aeryal", 0.0 },
  { "Aeryana", 0.0 },
  { "Aeryanna", 0.0 },
  { "Aeryk", 1.0 },
  { "Aeryn", 0.024896266 },
  { "Aeryona", 0.0 },
  { "Aesa", 0.0 },
  { "Aesha", 0.0 },
  { "Aeshia", 0.0 },
  { "Aeson", 1.0 },
  { "Aesop", 1.0 },
  { "Aethan", 1.0 },
  { "Aetna", 0.0 },
  { "Aeva", 0.0 },
  { "Aevah", 0.0 },
  { "Aevin", 1.0 },
  { "Aevry", 0.0 },
  { "Aevyn", 0.0 },
  { "Aeyden", 1.0 },
  { "Aeyla", 0.0 },
  { "Afa", 1.0 },
  { "Afaf", 0.0 },
  { "Afeef", 1.0 },
  { "Afeni", 0.0 },
  { "Aferdita", 0.0 },
  { "Affaan", 1.0 },
  { "Affan", 1.0 },
  { "Affie", 0.0 },
  { "Affinity", 0.0 },
  { "Afi", 0.0 },
  { "Afia", 0.0 },
  { "Afif", 1.0 },
  { "Afifa", 0.0 },
  { "Afifah", 0.0 },
  { "Afina", 0.0 },
  { "Afiya", 0.0 },
  { "Afiyah", 0.0 },
  { "Afnan", 0.19130434 },
  { "Afolabi", 1.0 },
  { "Afomia", 0.0 },
  { "Afonso", 1.0 },
  { "Afra", 0.0 },
  { "Afrah", 0.0 },
  { "Afraz", 1.0 },
  { "Afred", 1.0 },
  { "Afreen", 0.0 },
  { "Africa", 0.0 },
  { "Africia", 0.0 },
  { "Afrika", 0.0 },
  { "Afrim", 1.0 },
  { "Afrin", 0.0 },
  { "Afrodita", 0.0 },
  { "Afsa", 0.0 },
  { "Afsana", 0.0 },
  { "Afsaneh", 0.0 },
  { "Afshan", 0.0 },
  { "Afsheen", 0.13043478 },
  { "Afshin", 1.0 },
  { "Aftab", 1.0 },
  { "Aftan", 0.0 },
  { "Aften", 0.0 },
  { "Aftin", 0.0 },
  { "Afton", 0.1383237 },
  { "Aftyn", 0.0 },
  { "Afua", 0.0 },
  { "Afzal", 1.0 },
  { "Agam", 0.65384614 },
  { "Agamjot", 1.0 },
  { "Agamveer", 1.0 },
  { "Agape", 0.20833333 },
  { "Agapita", 0.0 },
  { "Agapito", 1.0 },
  { "Agassi", 1.0 },
  { "Agasthya", 1.0 },
  { "Agastya", 1.0 },
  { "Agata", 0.0 },
  { "Agatha", 0.0 },
  { "Agathe", 0.0 },
  { "Agda", 0.0 },
  { "Agee", 1.0 },
  { "Agel", 1.0 },
  { "Agela", 0.0 },
  { "Ageliki", 0.0 },
  { "Agena", 0.0 },
  { "Agent", 1.0 },
  { "Aggeliki", 0.0 },
  { "Aggie", 0.020311441 },
  { "Agha", 1.0 },
  { "Agigail", 0.0 },
  { "Agila", 0.0 },
  { "Agim", 1.0 },
  { "Aginah", 0.0 },
  { "Aglae", 0.0 },
  { "Aglaia", 0.0 },
  { "Agna", 0.0 },
  { "Agneda", 0.0 },
  { "Agnella", 0.0 },
  { "Agnes", 0.002942591 },
  { "Agnesa", 0.0 },
  { "Agnese", 0.0 },
  { "Agness", 0.0 },
  { "Agnessa", 0.0 },
  { "Agneta", 0.0 },
  { "Agnew", 1.0 },
  { "Agnieszka", 0.0 },
  { "Agnita", 0.0 },
  { "Agnus", 0.0 },
  { "Agostina", 0.0 },
  { "Agostino", 1.0 },
  { "Agrim", 1.0 },
  { "Agrima", 0.0 },
  { "Agripina", 0.0 },
  { "Agripino", 1.0 },
  { "Agron", 1.0 },
  { "Agueda", 0.0 },
  { "Agusta", 0.045627378 },
  { "Agustin", 0.9985329 },
  { "Agustina", 0.0 },
  { "Agustine", 1.0 },
  { "Agustus", 1.0 },
  { "Agustya", 1.0 },
  { "Ah", 0.77894735 },
  { "Ahaan", 1.0 },
  { "Ahaana", 0.0 },
  { "Ahad", 1.0 },
  { "Ahadu", 1.0 },
  { "Ahalya", 0.0 },
  { "Ahamad", 1.0 },
  { "Ahamed", 1.0 },
  { "Ahan", 1.0 },
  { "Ahana", 0.0 },
  { "Ahanna", 1.0 },
  { "Ahanu", 1.0 },
  { "Aharon", 1.0 },
  { "Aharshi", 1.0 },
  { "Ahava", 0.0 },
  { "Ahavah", 0.0 },
  { "Ahaziah", 1.0 },
  { "Ahbleza", 1.0 },
  { "Ahbree", 0.0 },
  { "Ahdia", 0.0 },
  { "Aheli", 0.0 },
  { "Ahian", 1.0 },
  { "Ahijah", 1.0 },
  { "Ahil", 1.0 },
  { "Ahilyn", 0.0 },
  { "Ahinara", 0.0 },
  { "Ahir", 1.0 },
  { "Ahisha", 0.0 },
  { "Ahitana", 0.0 },
  { "Ahja", 0.0 },
  { "Ahjah", 0.0 },
  { "Ahjahnae", 0.0 },
  { "Ahjanae", 0.0 },
  { "Ahjanay", 0.0 },
  { "Ahkai", 1.0 },
  { "Ahkari", 1.0 },
  { "Ahkeel", 1.0 },
  { "Ahkeelah", 0.0 },
  { "Ahkeem", 1.0 },
  { "Ahkil", 1.0 },
  { "Ahking", 1.0 },
  { "Ahlaam", 0.0 },
  { "Ahlai", 0.0 },
  { "Ahlam", 0.0 },
  { "Ahlana", 0.0 },
  { "Ahlani", 0.0 },
  { "Ahlanie", 0.0 },
  { "Ahlanna", 0.0 },
  { "Ahlaya", 0.0 },
  { "Ahlayah", 0.0 },
  { "Ahlayla", 0.0 },
  { "Ahlayna", 0.0 },
  { "Ahlaysia", 0.0 },
  { "Ahlea", 0.0 },
  { "Ahleah", 0.0 },
  { "Ahleena", 0.0 },
  { "Ahleeyah", 0.0 },
  { "Ahleigha", 0.0 },
  { "Ahlena", 0.0 },
  { "Ahlexis", 0.0 },
  { "Ahlexus", 0.0 },
  { "Ahley", 0.0 },
  { "Ahlia", 0.0 },
  { "Ahliana", 0.0 },
  { "Ahlias", 1.0 },
  { "Ahlijah", 1.0 },
  { "Ahlina", 0.0 },
  { "Ahlivia", 0.0 },
  { "Ahliya", 0.0 },
  { "Ahliyah", 0.0 },
  { "Ahlona", 0.0 },
  { "Ahlyana", 0.0 },
  { "Ahlyssa", 0.0 },
  { "Ahlyvia", 0.0 },
  { "Ahmaad", 1.0 },
  { "Ahmad", 0.9989052 },
  { "Ahmadou", 1.0 },
  { "Ahmaj", 1.0 },
  { "Ahman", 1.0 },
  { "Ahmand", 1.0 },
  { "Ahmani", 0.14855877 },
  { "Ahmar", 1.0 },
  { "Ahmara", 0.0 },
  { "Ahmare", 1.0 },
  { "Ahmari", 0.71352786 },
  { "Ahmaria", 0.0 },
  { "Ahmarie", 0.0 },
  { "Ahmarion", 1.0 },
  { "Ahmarria", 0.0 },
  { "Ahmaud", 1.0 },
  { "Ahmauri", 1.0 },
  { "Ahmaya", 0.0 },
  { "Ahmed", 1.0 },
  { "Ahmednur", 1.0 },
  { "Ahmeer", 1.0 },
  { "Ahmeir", 1.0 },
  { "Ahmer", 1.0 },
  { "Ahmere", 1.0 },
  { "Ahmet", 1.0 },
  { "Ahmi", 0.88372093 },
  { "Ahmia", 0.0 },
  { "Ahmiah", 0.0 },
  { "Ahmid", 1.0 },
  { "Ahmier", 1.0 },
  { "Ahmik", 1.0 },
  { "Ahmina", 0.0 },
  { "Ahmir", 0.99727666 },
  { "Ahmira", 0.0 },
  { "Ahmiracle", 0.0 },
  { "Ahmirah", 0.0 },
  { "Ahmire", 1.0 },
  { "Ahmiya", 0.0 },
  { "Ahmiyah", 0.0 },
  { "Ahmod", 1.0 },
  { "Ahmon", 1.0 },
  { "Ahmond", 1.0 },
  { "Ahmoni", 0.0 },
  { "Ahmonie", 0.0 },
  { "Ahmora", 0.0 },
  { "Ahmya", 0.0 },
  { "Ahmyah", 0.0 },
  { "Ahmyiah", 0.0 },
  { "Ahmyra", 0.0 },
  { "Ahna", 0.0 },
  { "Ahnaf", 1.0 },
  { "Ahnah", 0.0 },
  { "Ahnalee", 0.0 },
  { "Ahnaleigh", 0.0 },
  { "Ahnalia", 0.0 },
  { "Ahnalise", 0.0 },
  { "Ahnari", 0.0 },
  { "Ahnaya", 0.0 },
  { "Ahnesti", 0.0 },
  { "Ahnesty", 0.0 },
  { "Ahni", 0.0 },
  { "Ahnia", 0.0 },
  { "Ahniah", 0.0 },
  { "Ahnika", 0.0 },
  { "Ahnisty", 0.0 },
  { "Ahniya", 0.0 },
  { "Ahniyah", 0.0 },
  { "Ahniyla", 0.0 },
  { "Ahnjayla", 0.0 },
  { "Ahnna", 0.0 },
  { "Ahnya", 0.0 },
  { "Ahnyah", 0.0 },
  { "Ahnyia", 0.0 },
  { "Ahnyla", 0.0 },
  { "Ahnylah", 0.0 },
  { "Ahona", 0.0 },
  { "Ahonesti", 0.0 },
  { "Ahonesty", 0.0 },
  { "Ahrayah", 0.0 },
  { "Ahreanna", 0.0 },
  { "Ahren", 1.0 },
  { "Ahri", 0.0 },
  { "Ahria", 0.0 },
  { "Ahriah", 0.0 },
  { "Ahrian", 1.0 },
  { "Ahriana", 0.0 },
  { "Ahrianna", 0.0 },
  { "Ahriel", 0.0 },
  { "Ahriya", 0.0 },
  { "Ahriyah", 0.0 },
  { "Ahron", 1.0 },
  { "Ahrya", 0.0 },
  { "Ahsaki", 0.0 },
  { "Ahsan", 1.0 },
  { "Ahsen", 1.0 },
  { "Ahsha", 0.0 },
  { "Ahshanti", 0.0 },
  { "Ahsia", 0.0 },
  { "Ahsir", 1.0 },
  { "Ahslee", 0.0 },
  { "Ahsley", 0.0 },
  { "Ahsoka", 0.0 },
  { "Ahti", 1.0 },
  { "Ahtyana", 0.0 },
  { "Ahtziri", 0.0 },
  { "Ahtziry", 0.0 },
  { "Ahuva", 0.0 },
  { "Ahva", 0.0 },
  { "Ahviana", 0.0 },
  { "Ahvianna", 0.0 },
  { "Ahyaan", 1.0 },
  { "Ahyan", 1.0 },
  { "Ahyana", 0.0 },
  { "Ahyanna", 0.0 },
  { "Ahylin", 0.0 },
  { "Ahyonna", 0.0 },
  { "Ahzab", 1.0 },
  { "Ahzaria", 0.0 },
  { "Ahzariah", 0.0 },
  { "Ahzhane", 0.0 },
  { "Ahzir", 1.0 },
  { "Ahziyah", 0.0 },
  { "Ai", 0.029891305 },
  { "Aia", 0.0 },
  { "Aiah", 0.0 },
  { "Aian", 1.0 },
  { "Aiana", 0.0 },
  { "Aianna", 0.0 },
  { "Aibel", 1.0 },
  { "Aibhilin", 0.0 },
  { "Aibhlinn", 0.0 },
  { "Aibileen", 0.0 },
  { "Aicen", 1.0 },
  { "Aicha", 0.0 },
  { "Aichatou", 0.0 },
  { "Aicia", 0.0 },
  { "Aida", 0.00041138718 },
  { "Aidah", 0.0 },
  { "Aidaliz", 0.0 },
  { "Aidaly", 0.0 },
  { "Aidalyn", 0.0 },
  { "Aidan", 0.9761473 },
  { "Aidana", 0.0 },
  { "Aidanjames", 1.0 },
  { "Aidanjohn", 1.0 },
  { "Aidann", 1.0 },
  { "Aidar", 1.0 },
  { "Aiddan", 1.0 },
  { "Aidden", 1.0 },
  { "Aide", 0.0 },
  { "Aidean", 1.0 },
  { "Aided", 0.0 },
  { "Aidee", 0.0 },
  { "Aideen", 0.26530612 },
  { "Aidel", 0.0 },
  { "Aidelis", 0.0 },
  { "Aideliz", 0.0 },
  { "Aidelyn", 0.0 },
  { "Aiden", 0.9902006 },
  { "Aidenjames", 1.0 },
  { "Aidenjoseph", 1.0 },
  { "Aidenmatthew", 1.0 },
  { "Aidenmichael", 1.0 },
  { "Aidenn", 1.0 },
  { "Aidet", 0.0 },
  { "Aideth", 0.0 },
  { "Aideyn", 1.0 },
  { "Aidia", 0.0 },
  { "Aidian", 1.0 },
  { "Aidien", 1.0 },
  { "Aidin", 0.98098254 },
  { "Aidon", 1.0 },
  { "Aidrian", 1.0 },
  { "Aidrianna", 0.0 },
  { "Aidric", 1.0 },
  { "Aidrick", 1.0 },
  { "Aidrik", 1.0 },
  { "Aidsa", 0.0 },
  { "Aidy", 0.0 },
  { "Aidyen", 1.0 },
  { "Aidyn", 0.8247652 },
  { "Aidynn", 0.5362319 },
  { "Aiedan", 1.0 },
  { "Aieden", 1.0 },
  { "Aiesha", 0.0 },
  { "Aieshia", 0.0 },
  { "Aigne", 0.0 },
  { "Aigner", 0.0 },
  { "Aiham", 1.0 },
  { "Aiiden", 1.0 },
  { "Aija", 0.0 },
  { "Aijah", 0.0 },
  { "Aijahlon", 1.0 },
  { "Aijalon", 0.5177665 },
  { "Aijha", 0.0 },
  { "Aika", 0.0 },
  { "Aikeem", 1.0 },
  { "Aiken", 1.0 },
  { "Aiker", 1.0 },
  { "Aiko", 0.0 },
  { "Aila", 0.0 },
  { "Ailah", 0.0 },
  { "Ailaina", 0.0 },
  { "Ailan", 1.0 },
  { "Ailana", 0.0 },
  { "Ailani", 0.0 },
  { "Ailanie", 0.0 },
  { "Ailanny", 0.0 },
  { "Ailany", 0.0 },
  { "Aile", 0.0 },
  { "Ailea", 0.0 },
  { "Aileah", 0.0 },
  { "Ailean", 0.0 },
  { "Aileana", 0.0 },
  { "Ailed", 0.0 },
  { "Ailee", 0.0 },
  { "Aileen", 0.00014723638 },
  { "Aileena", 0.0 },
  { "Aileene", 0.0 },
  { "Aileigh", 0.0 },
  { "Ailen", 0.0 },
  { "Ailena", 0.0 },
  { "Ailene", 0.0 },
  { "Aileny", 0.0 },
  { "Ailet", 0.0 },
  { "Aileth", 0.0 },
  { "Ailey", 0.0 },
  { "Aili", 0.0 },
  { "Ailia", 0.0 },
  { "Ailiana", 0.0 },
  { "Ailie", 0.0 },
  { "Ailin", 0.00550055 },
  { "Ailina", 0.0 },
  { "Ailine", 0.0 },
  { "Ailis", 0.0 },
  { "Ailisa", 0.0 },
  { "Ailise", 0.0 },
  { "Ailish", 0.0 },
  { "Ailiyah", 0.0 },
  { "Ailleen", 0.0 },
  { "Aillene", 0.0 },
  { "Ailsa", 0.0 },
  { "Ailton", 1.0 },
  { "Aily", 0.0 },
  { "Ailyah", 0.0 },
  { "Ailyn", 0.0 },
  { "Ailyne", 0.0 },
  { "Ailynn", 0.0 },
  { "Aima", 0.0 },
  { "Aimal", 1.0 },
  { "Aiman", 0.75265956 },
  { "Aimani", 0.0 },
  { "Aimar", 0.5307692 },
  { "Aimara", 0.0 },
  { "Aime", 0.21974966 },
  { "Aimee", 0.0014651464 },
  { "Aimen", 0.3880597 },
  { "Aimi", 0.0 },
  { "Aimie", 0.0 },
  { "Aimme", 0.0 },
  { "Aimo", 1.0 },
  { "Aimon", 1.0 },
  { "Aimsley", 0.0 },
  { "Aimy", 0.0 },
  { "Ain", 0.0 },
  { "Aina", 0.0 },
  { "Ainara", 0.0 },
  { "Aindrea", 0.0 },
  { "Aine", 0.0 },
  { "Ainesh", 1.0 },
  { "Ainhara", 0.0 },
  { "Ainhoa", 0.0 },
  { "Aini", 0.0 },
  { "Ainka", 0.0 },
  { "Aino", 0.0 },
  { "Ainslee", 0.0 },
  { "Ainslei", 0.0 },
  { "Ainsleigh", 0.0 },
  { "Ainsley", 0.024748752 },
  { "Ainslie", 0.0 },
  { "Ainsly", 0.0 },
  { "Ainsworth", 1.0 },
  { "Ainzlee", 0.0 },
  { "Ainzley", 0.0 },
  { "Aiona", 0.0 },
  { "Aionna", 0.0 },
  { "Aira", 0.0 },
  { "Airabella", 0.0 },
  { "Airabelle", 0.0 },
  { "Airah", 0.0 },
  { "Airalyn", 0.0 },
  { "Airalynn", 0.0 },
  { "Airam", 0.1388889 },
  { "Airan", 1.0 },
  { "Airanna", 0.0 },
  { "Aire", 0.0 },
  { "Aireal", 0.0 },
  { "Aireana", 0.0 },
  { "Aireanna", 0.0 },
  { "Aireka", 0.0 },
  { "Airel", 0.0 },
  { "Airelle", 0.0 },
  { "Airen", 0.45783132 },
  { "Aireon", 0.9230769 },
  { "Aireona", 0.0 },
  { "Aireonna", 0.0 },
  { "Aires", 0.78125 },
  { "Airess", 0.0 },
  { "Aireyana", 0.0 },
  { "Airi", 0.0 },
  { "Airial", 0.0 },
  { "Airian", 0.0 },
  { "Airiana", 0.0 },
  { "Airianna", 0.0 },
  { "Airic", 1.0 },
  { "Airica", 0.0 },
  { "Airick", 1.0 },
  { "Airicka", 0.0 },
  { "Airiel", 0.0 },
  { "Airielle", 0.0 },
  { "Airik", 1.0 },
  { "Airika", 0.0 },
  { "Airin", 0.1388889 },
  { "Airion", 1.0 },
  { "Airiona", 0.0 },
  { "Airionna", 0.0 },
  { "Airis", 0.094827585 },
  { "Airius", 1.0 },
  { "Airlene", 0.0 },
  { "Airlie", 0.0 },
  { "Airon", 0.9734042 },
  { "Airrion", 1.0 },
  { "Airron", 1.0 },
  { "Airyana", 0.0 },
  { "Airyanna", 0.0 },
  { "Airyn", 0.048076924 },
  { "Airyona", 0.0 },
  { "Airyonna", 0.0 },
  { "Airys", 0.0 },
  { "Aisa", 0.010162601 },
  { "Aisah", 0.0 },
  { "Aisaiah", 1.0 },
  { "Aisatou", 0.0 },
  { "Aisea", 1.0 },
  { "Aisen", 1.0 },
  { "Aisha", 0.0013024602 },
  { "Aishah", 0.0 },
  { "Aishani", 0.0 },
  { "Aishat", 0.0 },
  { "Aishatu", 0.0 },
  { "Aishe", 0.0 },
  { "Aishi", 0.0 },
  { "Aishia", 0.0 },
  { "Aishini", 0.0 },
  { "Aishleen", 0.0 },
  { "Aishlin", 0.0 },
  { "Aishling", 0.0 },
  { "Aishlyn", 0.0 },
  { "Aishvi", 0.0 },
  { "Aishwarya", 0.0 },
  { "Aisia", 0.0 },
  { "Aisja", 0.0 },
  { "Aisla", 0.0 },
  { "Aislee", 0.0 },
  { "Aisleen", 0.0 },
  { "Aisleigh", 0.0 },
  { "Aisley", 0.0 },
  { "Aislin", 0.0 },
  { "Aisling", 0.0 },
  { "Aislinn", 0.0 },
  { "Aislyn", 0.0 },
  { "Aislynn", 0.0 },
  { "Aislynne", 0.0 },
  { "Aison", 1.0 },
  { "Aisosa", 0.0 },
  { "Aissa", 0.0 },
  { "Aissata", 0.0 },
  { "Aissatou", 0.0 },
  { "Aisya", 0.0 },
  { "Aitan", 1.0 },
  { "Aitana", 0.0 },
  { "Aitanna", 0.0 },
  { "Aithan", 1.0 },
  { "Aithana", 0.0 },
  { "Aithen", 1.0 },
  { "Aitiana", 0.0 },
  { "Aitlin", 0.0 },
  { "Aitor", 1.0 },
  { "Aitza", 0.0 },
  { "Aiva", 0.0 },
  { "Aivah", 0.0 },
  { "Aivan", 0.93421054 },
  { "Aiven", 1.0 },
  { "Aivery", 0.0 },
  { "Aivley", 0.0 },
  { "Aivree", 0.0 },
  { "Aivry", 0.0 },
  { "Aivy", 0.0 },
  { "Aixa", 0.0 },
  { "Aiya", 0.0 },
  { "Aiyah", 0.0 },
  { "Aiyahna", 0.0 },
  { "Aiyan", 1.0 },
  { "Aiyana", 0.0 },
  { "Aiyanah", 0.0 },
  { "Aiyani", 0.0 },
  { "Aiyanna", 0.0 },
  { "Aiyannah", 0.0 },
  { "Aiydan", 1.0 },
  { "Aiyden", 1.0 },
  { "Aiyla", 0.0 },
  { "Aiylah", 0.0 },
  { "Aiyona", 0.0 },
  { "Aiyonna", 0.0 },
  { "Aiysha", 0.0 },
  { "Aiza", 0.0 },
  { "Aizah", 0.0 },
  { "Aizayah", 1.0 },
  { "Aizeah", 1.0 },
  { "Aizel", 1.0 },
  { "Aizen", 1.0 },
  { "Aizha", 0.0 },
  { "Aizik", 1.0 },
  { "Aizlee", 0.0 },
  { "Aizleigh", 0.0 },
  { "Aizley", 0.0 },
  { "Aizlyn", 0.0 },
  { "Aizlynn", 0.0 },
  { "Aizza", 0.0 },
  { "Aj", 0.9829107 },
  { "Aja", 0.010819165 },
  { "Ajacia", 0.0 },
  { "Ajada", 0.0 },
  { "Ajaden", 1.0 },
  { "Ajae", 0.2682927 },
  { "Ajah", 0.0 },
  { "Ajahn", 1.0 },
  { "Ajahnae", 0.0 },
  { "Ajahnay", 0.0 },
  { "Ajahni", 1.0 },
  { "Ajai", 0.5185185 },
  { "Ajak", 1.0 },
  { "Ajala", 0.0 },
  { "Ajalae", 0.0 },
  { "Ajalon", 0.36842105 },
  { "Ajamu", 1.0 },
  { "Ajan", 1.0 },
  { "Ajana", 0.0 },
  { "Ajanae", 0.0 },
  { "Ajanai", 0.0 },
  { "Ajanay", 0.0 },
  { "Ajane", 0.0 },
  { "Ajanea", 0.0 },
  { "Ajanee", 0.0 },
  { "Ajang", 1.0 },
  { "Ajani", 0.90193903 },
  { "Ajanique", 0.0 },
  { "Ajasia", 0.0 },
  { "Ajavian", 1.0 },
  { "Ajavion", 1.0 },
  { "Ajax", 1.0 },
  { "Ajay", 0.9986996 },
  { "Ajaya", 0.0 },
  { "Ajayah", 0.0 },
  { "Ajayceon", 1.0 },
  { "Ajaycia", 0.0 },
  { "Ajayden", 1.0 },
  { "Ajaye", 1.0 },
  { "Ajayla", 0.0 },
  { "Ajaylah", 0.0 },
  { "Ajaysia", 0.0 },
  { "Ajdin", 1.0 },
  { "Aje", 0.0 },
  { "Ajea", 0.0 },
  { "Ajee", 0.104166664 },
  { "Ajeenah", 0.0 },
  { "Ajeet", 1.0 },
  { "Ajena", 0.0 },
  { "Ajenae", 0.0 },
  { "Ajene", 0.6013514 },
  { "Ajeya", 0.0 },
  { "Ajha", 0.0 },
  { "Ajhane", 0.0 },
  { "Ajhani", 1.0 },
  { "Aji", 0.0 },
  { "Ajia", 0.0 },
  { "Ajiah", 0.0 },
  { "Ajiana", 0.0 },
  { "Ajianna", 0.0 },
  { "Ajit", 1.0 },
  { "Ajita", 0.0 },
  { "Ajitesh", 1.0 },
  { "Ajith", 1.0 },
  { "Ajiya", 0.0 },
  { "Ajiyah", 0.0 },
  { "Ajla", 0.0 },
  { "Ajmal", 1.0 },
  { "Ajna", 0.0 },
  { "Ajon", 1.0 },
  { "Ajoni", 1.0 },
  { "Ajooni", 0.0 },
  { "Ajournee", 0.0 },
  { "Ajsa", 0.0 },
  { "Ajsha", 0.0 },
  { "Ajua", 0.0 },
  { "Ajuni", 0.0 },
  { "Ajwa", 0.0 },
  { "Ajwan", 0.0 },
  { "Akaash", 1.0 },
  { "Akachi", 1.0 },
  { "Akachukwu", 1.0 },
  { "Akacia", 0.0 },
  { "Akaden", 1.0 },
  { "Akadia", 0.0 },
  { "Akadian", 1.0 },
  { "Akaela", 0.0 },
  { "Akai", 0.9037037 },
  { "Akaia", 0.0 },
  { "Akaila", 0.0 },
  { "Akailah", 0.0 },
  { "Akain", 1.0 },
  { "Akaisha", 0.0 },
  { "Akaiya", 0.0 },
  { "Akaiyah", 0.0 },
  { "Akaja", 0.0 },
  { "Akala", 0.0 },
  { "Akali", 0.0 },
  { "Akam", 1.0 },
  { "Akane", 0.0 },
  { "Akanksha", 0.0 },
  { "Akanni", 1.0 },
  { "Akansha", 0.0 },
  { "Akara", 0.0 },
  { "Akari", 0.30174926 },
  { "Akaria", 0.0 },
  { "Akarsh", 1.0 },
  { "Akash", 1.0 },
  { "Akasha", 0.0 },
  { "Akashdeep", 1.0 },
  { "Akashia", 0.0 },
  { "Akasia", 0.0 },
  { "Akaya", 0.0 },
  { "Akaycia", 0.0 },
  { "Akayden", 1.0 },
  { "Akayla", 0.0 },
  { "Akaylah", 0.0 },
  { "Akaylia", 0.0 },
  { "Akaysha", 0.0 },
  { "Akaysia", 0.0 },
  { "Akbar", 1.0 },
  { "Akea", 0.0 },
  { "Akeam", 1.0 },
  { "Akeara", 0.0 },
  { "Akeba", 0.0 },
  { "Akebia", 0.0 },
  { "Akeda", 0.0 },
  { "Akeel", 1.0 },
  { "Akeela", 0.0 },
  { "Akeelah", 0.0 },
  { "Akeem", 1.0 },
  { "Akeema", 0.0 },
  { "Akeen", 1.0 },
  { "Akeena", 0.0 },
  { "Akeera", 0.0 },
  { "Akeesha", 0.0 },
  { "Akeeyla", 0.0 },
  { "Akeia", 0.0 },
  { "Akeiba", 0.0 },
  { "Akeil", 1.0 },
  { "Akeila", 0.0 },
  { "Akeilah", 0.0 },
  { "Akeim", 1.0 },
  { "Akeira", 0.0 },
  { "Akeisha", 0.0 },
  { "Akeiya", 0.0 },
  { "Akela", 0.008992806 },
  { "Akelah", 0.0 },
  { "Akelia", 0.0 },
  { "Akem", 1.0 },
  { "Akemi", 0.007032349 },
  { "Akemy", 0.0 },
  { "Aken", 1.0 },
  { "Akena", 0.0 },
  { "Akendra", 0.0 },
  { "Akenya", 0.0 },
  { "Akenzie", 0.0 },
  { "Aker", 1.0 },
  { "Akera", 0.019455252 },
  { "Akerah", 0.0 },
  { "Akeria", 0.0 },
  { "Akeriah", 0.0 },
  { "Akerra", 0.0 },
  { "Akerria", 0.0 },
  { "Akesha", 0.0 },
  { "Akeshia", 0.0 },
  { "Aketzali", 0.0 },
  { "Aketzalli", 0.0 },
  { "Aketzaly", 0.0 },
  { "Akevia", 0.0 },
  { "Akeya", 0.0 },
  { "Akeyah", 0.0 },
  { "Akeyia", 0.0 },
  { "Akeyla", 0.0 },
  { "Akeylah", 0.0 },
  { "Akeyra", 0.0 },
  { "Akhari", 1.0 },
  { "Akheem", 1.0 },
  { "Akhenaton", 1.0 },
  { "Akhi", 1.0 },
  { "Akhia", 0.0 },
  { "Akhil", 1.0 },
  { "Akhila", 0.0 },
  { "Akhilesh", 1.0 },
  { "Akhilles", 1.0 },
  { "Akhilleus", 1.0 },
  { "Akhir", 1.0 },
  { "Akhira", 0.0 },
  { "Akhirah", 0.0 },
  { "Akhiya", 0.0 },
  { "Aki", 0.55833334 },
  { "Akia", 0.058204334 },
  { "Akiah", 0.0 },
  { "Akiana", 0.0 },
  { "Akiba", 0.054347824 },
  { "Akida", 0.7368421 },
  { "Akie", 0.3125 },
  { "Akieba", 0.0 },
  { "Akiel", 1.0 },
  { "Akiela", 0.0 },
  { "Akiem", 1.0 },
  { "Akiera", 0.0 },
  { "Akierra", 0.0 },
  { "Akiesha", 0.0 },
  { "Akif", 1.0 },
  { "Akihiro", 1.0 },
  { "Akiko", 0.0 },
  { "Akil", 1.0 },
  { "Akila", 0.0 },
  { "Akilah", 0.0 },
  { "Akilan", 1.0 },
  { "Akiles", 1.0 },
  { "Akilesh", 1.0 },
  { "Akili", 0.63033175 },
  { "Akilia", 0.0 },
  { "Akilles", 1.0 },
  { "Akillian", 1.0 },
  { "Akim", 1.0 },
  { "Akima", 0.0 },
  { "Akimi", 0.0 },
  { "Akin", 1.0 },
  { "Akina", 0.0 },
  { "Aking", 1.0 },
  { "Akinola", 1.0 },
  { "Akintunde", 1.0 },
  { "Akio", 1.0 },
  { "Akir", 1.0 },
  { "Akira", 0.2204767 },
  { "Akirah", 0.0 },
  { "Akire", 0.0 },
  { "Akiria", 0.0 },
  { "Akiro", 1.0 },
  { "Akirra", 0.0 },
  { "Akisha", 0.0 },
  { "Akishia", 0.0 },
  { "Akita", 0.0 },
  { "Akito", 1.0 },
  { "Akiva", 0.9376637 },
  { "Akiya", 0.0 },
  { "Akiyah", 0.0 },
  { "Akiylah", 0.0 },
  { "Akiyra", 0.0 },
  { "Akmal", 1.0 },
  { "Ako", 0.8148148 },
  { "Akol", 1.0 },
  { "Akon", 0.7922078 },
  { "Akoni", 1.0 },
  { "Akorede", 1.0 },
  { "Akos", 1.0 },
  { "Akosha", 0.0 },
  { "Akosita", 0.0 },
  { "Akosua", 0.0 },
  { "Akoya", 0.0 },
  { "Akram", 1.0 },
  { "Akria", 0.0 },
  { "Akrish", 1.0 },
  { "Akriti", 0.0 },
  { "Aksa", 0.0 },
  { "Aksana", 0.0 },
  { "Aksel", 1.0 },
  { "Aksh", 1.0 },
  { "Aksha", 0.0 },
  { "Akshada", 0.0 },
  { "Akshadha", 0.0 },
  { "Akshaj", 1.0 },
  { "Akshan", 1.0 },
  { "Akshar", 1.0 },
  { "Akshara", 0.0 },
  { "Akshat", 1.0 },
  { "Akshata", 0.0 },
  { "Akshath", 1.0 },
  { "Akshay", 1.0 },
  { "Akshaya", 0.0 },
  { "Akshit", 1.0 },
  { "Akshita", 0.0 },
  { "Akshith", 1.0 },
  { "Akshitha", 0.0 },
  { "Aksil", 1.0 },
  { "Akua", 0.0 },
  { "Akul", 1.0 },
  { "Akur", 0.0 },
  { "Akura", 0.0 },
  { "Akwasi", 1.0 },
  { "Akwete", 0.0 },
  { "Akxel", 1.0 },
  { "Akya", 0.0 },
  { "Akyah", 0.0 },
  { "Akyia", 0.0 },
  { "Akyiah", 0.0 },
  { "Akyla", 0.0 },
  { "Akylah", 0.0 },
  { "Akyra", 0.0 },
  { "Akyrah", 0.0 },
  { "Akyria", 0.0 },
  { "Akyrie", 1.0 },
  { "Akzel", 1.0 },
  { "Al", 0.98517525 },
  { "Ala", 0.18043202 },
  { "Alaa", 0.27448276 },
  { "Alaan", 1.0 },
  { "Alaana", 0.0 },
  { "Alabama", 0.0 },
  { "Alac", 1.0 },
  { "Alacia", 0.0 },
  { "Alacyia", 0.0 },
  { "Aladdin", 1.0 },
  { "Aladin", 1.0 },
  { "Aladino", 1.0 },
  { "Aladrian", 0.0 },
  { "Alae", 0.0 },
  { "Alaejah", 0.0 },
  { "Alaena", 0.0 },
  { "Alaetra", 0.0 },
  { "Alaeya", 0.0 },
  { "Alaeyah", 0.0 },
  { "Alafia", 0.0 },
  { "Alagie", 1.0 },
  { "Alah", 0.0 },
  { "Alahia", 0.0 },
  { "Alahna", 0.0 },
  { "Alahni", 0.0 },
  { "Alahnna", 0.0 },
  { "Alai", 0.33333334 },
  { "Alaia", 0.0 },
  { "Alaiah", 0.0 },
  { "Alaida", 0.0 },
  { "Alaiha", 0.0 },
  { "Alaiia", 0.0 },
  { "Alaiiah", 0.0 },
  { "Alaija", 0.0 },
  { "Alaijah", 0.008474576 },
  { "Alaijha", 0.0 },
  { "Alaila", 0.0 },
  { "Alailah", 0.0 },
  { "Alain", 0.9983046 },
  { "Alaina", 0.00016137709 },
  { "Alainah", 0.0 },
  { "Alaine", 0.0 },
  { "Alainey", 0.0 },
  { "Alaini", 0.0 },
  { "Alainie", 0.0 },
  { "Alainna", 0.0 },
  { "Alainnah", 0.0 },
  { "Alair", 0.0 },
  { "Alaira", 0.0 },
  { "Alaire", 0.0 },
  { "Alaisa", 0.0 },
  { "Alaisha", 0.0 },
  { "Alaisia", 0.0 },
  { "Alaiya", 0.0 },
  { "Alaiyah", 0.0 },
  { "Alaiyna", 0.0 },
  { "Alaiza", 0.0 },
  { "Alaizah", 0.0 },
  { "Alaizha", 0.0 },
  { "Alaja", 0.0 },
  { "Alajah", 0.0 },
  { "Alajandro", 1.0 },
  { "Alajha", 0.0 },
  { "Alajhia", 0.0 },
  { "Alajia", 0.0 },
  { "Alajiah", 0.0 },
  { "Alajuwon", 1.0 },
  { "Alakai", 1.0 },
  { "Alakay", 1.0 },
  { "Alaki", 1.0 },
  { "Alala", 0.0 },
  { "Alalah", 0.0 },
  { "Alam", 1.0 },
  { "Alama", 0.0 },
  { "Alamae", 0.0 },
  { "Alamar", 0.0 },
  { "Alamea", 0.0 },
  { "Alameda", 0.0 },
  { "Alameen", 1.0 },
  { "Alamin", 1.0 },
  { "Alan", 0.99690986 },
  { "Alana", 0.00057415914 },
  { "Alanah", 0.0 },
  { "Alancia", 0.0 },
  { "Aland", 1.0 },
  { "Alanda", 0.0 },
  { "Alander", 1.0 },
  { "Alandis", 1.0 },
  { "Alando", 1.0 },
  { "Alandra", 0.0 },
  { "Alandre", 1.0 },
  { "Alandrea", 0.0 },
  { "Alandria", 0.0 },
  { "Alandus", 1.0 },
  { "Alane", 0.0 },
  { "Alanea", 0.0 },
  { "Alanee", 0.0 },
  { "Alaney", 0.0 },
  { "Alani", 0.007913897 },
  { "Alania", 0.0 },
  { "Alanie", 0.0 },
  { "Alanii", 0.0 },
  { "Alanis", 0.0 },
  { "Alaniz", 0.0 },
  { "Alanmichael", 1.0 },
  { "Alann", 1.0 },
  { "Alanna", 0.0003363121 },
  { "Alannah", 0.0 },
  { "Alanni", 0.0 },
  { "Alannie", 0.0 },
  { "Alannis", 0.0 },
  { "Alanny", 0.0 },
  { "Alannys", 0.0 },
  { "Alano", 1.0 },
  { "Alanood", 0.0 },
  { "Alanoud", 0.0 },
  { "Alanson", 1.0 },
  { "Alanta", 0.0 },
  { "Alantae", 1.0 },
  { "Alante", 0.8206039 },
  { "Alantis", 0.0 },
  { "Alantra", 0.0 },
  { "Alany", 0.0 },
  { "Alanya", 0.0 },
  { "Alanys", 0.0 },
  { "Alanzo", 1.0 },
  { "Alara", 0.0 },
  { "Alarah", 0.0 },
  { "Alaria", 0.0 },
  { "Alaric", 1.0 },
  { "Alarice", 0.0 },
  { "Alarick", 1.0 },
  { "Alarie", 0.0 },
  { "Alarik", 1.0 },
  { "Alaris", 0.0 },
  { "Alasdair", 1.0 },
  { "Alasha", 0.0 },
  { "Alashia", 0.0 },
  { "Alasia", 0.0 },
  { "Alasiah", 0.0 },
  { "Alaska", 0.0 },
  { "Alassandra", 0.0 },
  { "Alassane", 1.0 },
  { "Alastair", 1.0 },
  { "Alastar", 1.0 },
  { "Alaster", 1.0 },
  { "Alastor", 1.0 },
  { "Alathea", 0.0 },
  { "Alauna", 0.0 },
  { "Alaunah", 0.0 },
  { "Alaundra", 0.0 },
  { "Alauni", 0.0 },
  { "Alaunna", 0.0 },
  { "Alaura", 0.0 },
  { "Alaurah", 0.0 },
  { "Alawna", 0.0 },
  { "Alax", 1.0 },
  { "Alaxander", 1.0 },
  { "Alaxandra", 0.0 },
  { "Alaxandria", 0.0 },
  { "Alaxis", 0.0 },
  { "Alay", 1.0 },
  { "Alaya", 0.0 },
  { "Alayah", 0.0 },
  { "Alayana", 0.0 },
  { "Alayasia", 0.0 },
  { "Alaycia", 0.0 },
  { "Alayda", 0.0 },
  { "Alaye", 0.6875 },
  { "Alayha", 0.0 },
  { "Alayia", 0.0 },
  { "Alayiah", 0.0 },
  { "Alayja", 0.0 },
  { "Alayjah", 0.0 },
  { "Alayjha", 0.0 },
  { "Alayjia", 0.0 },
  { "Alayjiah", 0.0 },
  { "Alayla", 0.0 },
  { "Alaylah", 0.0 },
  { "Alayn", 1.0 },
  { "Alayna", 0.00020036868 },
  { "Alaynah", 0.0 },
  { "Alayne", 0.0 },
  { "Alaynia", 0.0 },
  { "Alaynie", 0.0 },
  { "Alaynna", 0.0 },
  { "Alaynnah", 0.0 },
  { "Alaysa", 0.0 },
  { "Alaysha", 0.0 },
  { "Alayshia", 0.0 },
  { "Alaysia", 0.0 },
  { "Alaysiah", 0.0 },
  { "Alayssa", 0.0 },
  { "Alaythia", 0.0 },
  { "Alayza", 0.0 },
  { "Alayzah", 0.0 },
  { "Alayzha", 0.0 },
  { "Alayzhia", 0.0 },
  { "Alayzia", 0.0 },
  { "Alayziah", 0.0 },
  { "Alaz", 1.0 },
  { "Alaza", 0.0 },
  { "Alazae", 0.0 },
  { "Alazah", 0.0 },
  { "Alazar", 1.0 },
  { "Alazay", 0.0 },
  { "Alazaya", 0.0 },
  { "Alaze", 0.0 },
  { "Alazea", 0.0 },
  { "Alazhia", 0.0 },
  { "Alazia", 0.0 },
  { "Alaziah", 0.0 },
  { "Alazne", 0.0 },
  { "Alba", 0.01805718 },
  { "Alban", 1.0 },
  { "Albana", 0.0 },
  { "Albani", 0.0 },
  { "Albanie", 0.0 },
  { "Albano", 1.0 },
  { "Albany", 0.008183306 },
  { "Albara", 1.0 },
  { "Albaraa", 1.0 },
  { "Albaro", 1.0 },
  { "Albee", 0.0 },
  { "Albeiro", 1.0 },
  { "Alben", 1.0 },
  { "Albena", 0.0 },
  { "Alber", 1.0 },
  { "Alberdia", 0.0 },
  { "Alberico", 1.0 },
  { "Albert", 0.99476916 },
  { "Alberta", 0.005645241 },
  { "Alberteen", 0.0 },
  { "Alberth", 1.0 },
  { "Albertha", 0.0 },
  { "Albertia", 0.0 },
  { "Albertico", 1.0 },
  { "Albertie", 0.0 },
  { "Albertina", 0.0 },
  { "Albertine", 0.0 },
  { "Albertis", 1.0 },
  { "Albertjames", 1.0 },
  { "Alberto", 0.99252635 },
  { "Albertus", 1.0 },
  { "Albesa", 0.0 },
  { "Albi", 1.0 },
  { "Albia", 0.0 },
  { "Albie", 0.48706895 },
  { "Albieri", 1.0 },
  { "Albieris", 1.0 },
  { "Albiery", 1.0 },
  { "Albin", 1.0 },
  { "Albina", 0.0 },
  { "Albino", 1.0 },
  { "Albion", 1.0 },
  { "Albiona", 0.0 },
  { "Albirda", 0.0 },
  { "Albirdia", 0.0 },
  { "Albirta", 0.0 },
  { "Albirtha", 0.0 },
  { "Albon", 1.0 },
  { "Alborz", 1.0 },
  { "Albree", 0.0 },
  { "Albrey", 0.0 },
  { "Albria", 0.0 },
  { "Alburn", 1.0 },
  { "Albus", 1.0 },
  { "Alby", 0.7169811 },
  { "Alcaria", 0.0 },
  { "Alcario", 1.0 },
  { "Alcee", 1.0 },
  { "Alchemy", 0.0 },
  { "Alcia", 0.0 },
  { "Alcibiades", 1.0 },
  { "Alcide", 1.0 },
  { "Alcides", 1.0 },
  { "Alcie", 0.02793296 },
  { "Alcina", 0.0 },
  { "Alcindor", 1.0 },
  { "Alcine", 0.0 },
  { "Alco", 1.0 },
  { "Alcuin", 1.0 },
  { "Alcus", 1.0 },
  { "Alcy", 0.0 },
  { "Alda", 0.0020157227 },
  { "Aldah", 0.0 },
  { "Aldahir", 1.0 },
  { "Aldair", 1.0 },
  { "Aldan", 1.0 },
  { "Aldana", 0.0 },
  { "Aldayr", 1.0 },
  { "Aldayshein", 1.0 },
  { "Aldea", 0.0 },
  { "Aldean", 0.31623933 },
  { "Aldeen", 0.0 },
  { "Aldemar", 1.0 },
  { "Alden", 0.95000994 },
  { "Aldena", 0.0 },
  { "Aldene", 0.014204546 },
  { "Alder", 0.93246186 },
  { "Alderic", 1.0 },
  { "Aldhair", 1.0 },
  { "Aldia", 0.0 },
  { "Aldie", 0.65625 },
  { "Aldijana", 0.0 },
  { "Aldin", 1.0 },
  { "Aldina", 0.0 },
  { "Aldine", 0.16424116 },
  { "Aldis", 1.0 },
  { "Aldo", 0.99902403 },
  { "Aldolfo", 1.0 },
  { "Aldon", 1.0 },
  { "Aldona", 0.0 },
  { "Aldonia", 0.0 },
  { "Aldonna", 0.0 },
  { "Aldor", 1.0 },
  { "Aldora", 0.0 },
  { "Aldoris", 0.0 },
  { "Aldous", 1.0 },
  { "Aldred", 1.0 },
  { "Aldren", 1.0 },
  { "Aldrena", 0.0 },
  { "Aldric", 1.0 },
  { "Aldrich", 1.0 },
  { "Aldrick", 1.0 },
  { "Aldridge", 1.0 },
  { "Aldrik", 1.0 },
  { "Aldrin", 1.0 },
  { "Aldrina", 0.0 },
  { "Alduin", 1.0 },
  { "Aldus", 1.0 },
  { "Aldwin", 1.0 },
  { "Aldyn", 0.68367344 },
  { "Aldyth", 0.0 },
  { "Ale", 0.61538464 },
  { "Alea", 0.0 },
  { "Aleacia", 0.0 },
  { "Aleah", 0.0 },
  { "Aleaha", 0.0 },
  { "Aleahya", 0.0 },
  { "Alean", 0.0 },
  { "Aleana", 0.0 },
  { "Aleanah", 0.0 },
  { "Aleander", 1.0 },
  { "Aleandra", 0.0 },
  { "Aleane", 0.0 },
  { "Aleanna", 0.0 },
  { "Aleara", 0.0 },
  { "Aleasa", 0.0 },
  { "Alease", 0.0 },
  { "Aleasha", 0.0 },
  { "Aleashia", 0.0 },
  { "Aleasia", 0.0 },
  { "Aleata", 0.0 },
  { "Aleatha", 0.0 },
  { "Aleathea", 0.0 },
  { "Aleathia", 0.0 },
  { "Aleatra", 0.0 },
  { "Aleaya", 0.0 },
  { "Aleayah", 0.0 },
  { "Alec", 0.9935015 },
  { "Aleca", 0.0 },
  { "Alecander", 1.0 },
  { "Aleccia", 0.0 },
  { "Alece", 0.0 },
  { "Alecea", 0.0 },
  { "Aleceia", 0.0 },
  { "Alecha", 0.0 },
  { "Alechia", 0.0 },
  { "Alecia", 0.0 },
  { "Aleciram", 0.0 },
  { "Aleck", 1.0 },
  { "Alecsa", 0.0 },
  { "Alecsander", 1.0 },
  { "Alecsandra", 0.0 },
  { "Alecsis", 0.0 },
  { "Alectra", 0.0 },
  { "Alecxa", 0.0 },
  { "Alecxander", 1.0 },
  { "Alecxia", 0.0 },
  { "Alecxis", 1.0 },
  { "Aleczander", 1.0 },
  { "Aleczandria", 0.0 },
  { "Aleda", 0.0 },
  { "Aledra", 0.0 },
  { "Alee", 0.12138728 },
  { "Aleea", 0.0 },
  { "Aleeah", 0.0 },
  { "Aleeana", 0.0 },
  { "Aleece", 0.0 },
  { "Aleecia", 0.0 },
  { "Aleecya", 0.0 },
  { "Aleeha", 0.0 },
  { "Aleeka", 0.0 },
  { "Aleela", 0.0 },
  { "Aleem", 1.0 },
  { "Aleema", 0.0 },
  { "Aleemah", 0.0 },
  { "Aleen", 0.0 },
  { "Aleena", 0.0 },
  { "Aleenah", 0.0 },
  { "Aleene", 0.0 },
  { "Aleera", 0.0 },
  { "Aleesa", 0.0 },
  { "Aleese", 0.0 },
  { "Aleesha", 0.0 },
  { "Aleeshia", 0.0 },
  { "Aleesia", 0.0 },
  { "Aleeta", 0.0 },
  { "Aleeya", 0.0 },
  { "Aleeyah", 0.0 },
  { "Aleeza", 0.0 },
  { "Aleezah", 0.0 },
  { "Aleezay", 0.0 },
  { "Alegacy", 0.0 },
  { "Alegandra", 0.0 },
  { "Alegandro", 1.0 },
  { "Alegend", 1.0 },
  { "Alegna", 0.0 },
  { "Alegra", 0.0 },
  { "Alegria", 0.0 },
  { "Aleha", 0.0 },
  { "Alei", 0.0 },
  { "Aleia", 0.0 },
  { "Aleiah", 0.0 },
  { "Aleicia", 0.0 },
  { "Aleida", 0.0 },
  { "Aleidy", 0.0 },
  { "Aleigh", 0.0 },
  { "Aleigha", 0.0 },
  { "Aleighana", 0.0 },
  { "Aleighanna", 0.0 },
  { "Aleighia", 0.0 },
  { "Aleighna", 0.0 },
  { "Aleighsha", 0.0 },
  { "Aleighya", 0.0 },
  { "Aleiha", 0.0 },
  { "Aleijah", 0.0 },
  { "Aleila", 0.0 },
  { "Aleily", 0.0 },
  { "Aleina", 0.0 },
  { "Aleinah", 0.0 },
  { "Aleine", 0.0 },
  { "Aleira", 0.0 },
  { "Aleisa", 0.0 },
  { "Aleise", 0.0 },
  { "Aleisha", 0.0 },
  { "Aleishia", 0.0 },
  { "Aleister", 1.0 },
  { "Aleita", 0.0 },
  { "Aleitha", 0.0 },
  { "Aleix", 1.0 },
  { "Aleiya", 0.0 },
  { "Aleiyah", 0.0 },
  { "Aleiza", 0.0 },
  { "Aleja", 0.0 },
  { "Alejadro", 1.0 },
  { "Alejah", 0.0 },
  { "Alejando", 1.0 },
  { "Alejandr", 1.0 },
  { "Alejandra", 0.010597283 },
  { "Alejandria", 0.0 },
  { "Alejandrina", 0.0 },
  { "Alejandro", 0.9935018 },
  { "Alejo", 1.0 },
  { "Alejondro", 1.0 },
  { "Alejos", 1.0 },
  { "Alek", 0.9819149 },
  { "Aleka", 0.0 },
  { "Alekai", 1.0 },
  { "Alekay", 1.0 },
  { "Alekhya", 0.0 },
  { "Aleki", 1.0 },
  { "Alekos", 1.0 },
  { "Aleks", 1.0 },
  { "Aleksa", 0.098615915 },
  { "Aleksah", 0.0 },
  { "Aleksandar", 1.0 },
  { "Aleksander", 1.0 },
  { "Aleksandr", 1.0 },
  { "Aleksandra", 0.0 },
  { "Aleksandre", 1.0 },
  { "Aleksandria", 0.0 },
  { "Aleksandrs", 1.0 },
  { "Aleksei", 0.9660057 },
  { "Aleksey", 0.9769053 },
  { "Aleksi", 0.70526314 },
  { "Aleksia", 0.0 },
  { "Aleksis", 0.06849315 },
  { "Aleksy", 1.0 },
  { "Alektra", 0.0 },
  { "Alekxa", 0.0 },
  { "Alekxander", 1.0 },
  { "Alekya", 0.0 },
  { "Alekza", 0.0 },
  { "Alekzander", 1.0 },
  { "Alekzandr", 1.0 },
  { "Alekzandra", 0.0 },
  { "Alela", 0.0 },
  { "Aleli", 0.0 },
  { "Alelia", 0.0 },
  { "Alely", 0.0 },
  { "Alem", 1.0 },
  { "Alema", 0.8076923 },
  { "Alen", 1.0 },
  { "Alena", 0.0 },
  { "Alenah", 0.0 },
  { "Alencia", 0.0 },
  { "Alenda", 0.0 },
  { "Alene", 0.00057650177 },
  { "Aleni", 0.0 },
  { "Alenia", 0.0 },
  { "Alenis", 0.0 },
  { "Alenka", 0.0 },
  { "Alenna", 0.0 },
  { "Alennah", 0.0 },
  { "Aleny", 0.0 },
  { "Aleona", 0.0 },
  { "Aleph", 0.91071427 },
  { "Aleq", 1.0 },
  { "Alera", 0.0 },
  { "Alerah", 0.0 },
  { "Aleria", 0.0 },
  { "Aleric", 1.0 },
  { "Alero", 0.0 },
  { "Aleron", 1.0 },
  { "Ales", 1.0 },
  { "Alesa", 0.0 },
  { "Alesana", 0.0 },
  { "Alesander", 1.0 },
  { "Alesandra", 0.0 },
  { "Alesandro", 1.0 },
  { "Alescia", 0.0 },
  { "Alese", 0.0 },
  { "Alesea", 0.0 },
  { "Alesha", 0.0008147746 },
  { "Aleshanee", 0.0 },
  { "Aleshea", 0.0 },
  { "Aleshia", 0.0 },
  { "Aleshka", 0.0 },
  { "Alesi", 0.0 },
  { "Alesia", 0.0 },
  { "Alesiram", 0.0 },
  { "Aleska", 0.0 },
  { "Alessa", 0.0 },
  { "Alessand", 1.0 },
  { "Alessander", 1.0 },
  { "Alessandra", 0.0 },
  { "Alessandria", 0.0 },
  { "Alessandro", 1.0 },
  { "Alessi", 0.0 },
  { "Alessia", 0.0 },
  { "Alessio", 1.0 },
  { "Alesso", 1.0 },
  { "Alessondra", 0.0 },
  { "Alessya", 0.0 },
  { "Alesya", 0.0 },
  { "Aleta", 0.0 },
  { "Aletha", 0.0 },
  { "Alethea", 0.0 },
  { "Aletheia", 0.0 },
  { "Alethia", 0.0 },
  { "Aletia", 0.0 },
  { "Aletra", 0.0 },
  { "Aletris", 0.0 },
  { "Aletse", 0.0 },
  { "Aletta", 0.0 },
  { "Alette", 0.0 },
  { "Aletza", 0.0 },
  { "Alevia", 0.0 },
  { "Alex", 0.968901 },
  { "Alexa", 0.0021406445 },
  { "Alexader", 1.0 },
  { "Alexadra", 0.0 },
  { "Alexah", 0.0 },
  { "Alexamarie", 0.0 },
  { "Alexan", 1.0 },
  { "Alexana", 0.0 },
  { "Alexand", 1.0 },
  { "Alexanda", 0.0 },
  { "Alexandar", 1.0 },
  { "Alexande", 1.0 },
  { "Alexander", 0.9932576 },
  { "Alexandera", 0.0 },
  { "Alexanderia", 0.0 },
  { "Alexanderjames", 1.0 },
  { "Alexandia", 0.0 },
  { "Alexandira", 0.0 },
  { "Alexandor", 1.0 },
  { "Alexandr", 0.5796089 },
  { "Alexandra", 0.003740996 },
  { "Alexandrah", 0.0 },
  { "Alexandre", 0.9694801 },
  { "Alexandrea", 0.0 },
  { "Alexandreia", 0.0 },
  { "Alexandrer", 1.0 },
  { "Alexandrew", 1.0 },
  { "Alexandria", 0.0023854715 },
  { "Alexandriah", 0.0 },
  { "Alexandrina", 0.0 },
  { "Alexandrine", 0.0 },
  { "Alexandro", 1.0 },
  { "Alexandros", 1.0 },
  { "Alexandru", 1.0 },
  { "Alexandrya", 0.0 },
  { "Alexaner", 1.0 },
  { "Alexann", 0.0 },
  { "Alexanna", 0.0 },
  { "Alexanndra", 0.0 },
  { "Alexanne", 0.0 },
  { "Alexanra", 0.0 },
  { "Alexarae", 0.0 },
  { "Alexaray", 0.0 },
  { "Alexas", 0.0 },
  { "Alexaundra", 0.0 },
  { "Alexaundrea", 0.0 },
  { "Alexaundria", 0.0 },
  { "Alexavia", 0.0 },
  { "Alexavier", 1.0 },
  { "Alexavior", 1.0 },
  { "Alexcia", 0.0 },
  { "Alexcis", 0.048672568 },
  { "Alexcya", 0.0 },
  { "Alexe", 0.0 },
  { "Alexea", 0.0 },
  { "Alexee", 0.0 },
  { "Alexei", 0.90346086 },
  { "Alexeia", 0.0 },
  { "Alexender", 1.0 },
  { "Alexendra", 0.0 },
  { "Alexendria", 0.0 },
  { "Alexes", 0.16567342 },
  { "Alexeus", 0.0 },
  { "Alexey", 0.95165396 },
  { "Alexeya", 0.0 },
  { "Alexi", 0.28046745 },
  { "Alexia", 0.0010367089 },
  { "Alexiah", 0.0 },
  { "Alexian", 1.0 },
  { "Alexiana", 0.0 },
  { "Alexianna", 0.0 },
  { "Alexias", 0.0 },
  { "Alexica", 0.0 },
  { "Alexicia", 0.0 },
  { "Alexie", 0.15139443 },
  { "Alexiea", 0.0 },
  { "Alexiel", 0.6666667 },
  { "Alexies", 0.0 },
  { "Alexina", 0.0 },
  { "Alexine", 0.0 },
  { "Alexio", 1.0 },
  { "Alexiona", 0.0 },
  { "Alexionna", 0.0 },
  { "Alexios", 1.0 },
  { "Alexious", 0.0 },
  { "Alexis", 0.15660518 },
  { "Alexisa", 0.0 },
  { "Alexisia", 0.0 },
  { "Alexisjade", 0.0 },
  { "Alexismarie", 0.0 },
  { "Alexisnicole", 0.0 },
  { "Alexisrose", 0.0 },
  { "Alexiss", 0.03649635 },
  { "Alexius", 0.07559926 },
  { "Alexix", 0.11904762 },
  { "Alexiya", 0.0 },
  { "Alexiyah", 0.0 },
  { "Alexiyana", 0.0 },
  { "Alexiz", 0.4844358 },
  { "Alexjandro", 1.0 },
  { "Alexnader", 1.0 },
  { "Alexnadra", 0.0 },
  { "Alexondra", 0.0 },
  { "Alexondria", 0.0 },
  { "Alexous", 0.0 },
  { "Alexs", 1.0 },
  { "Alexsa", 0.0 },
  { "Alexsander", 1.0 },
  { "Alexsandra", 0.0 },
  { "Alexsandria", 0.0 },
  { "Alexsandro", 1.0 },
  { "Alexsi", 1.0 },
  { "Alexsia", 0.0 },
  { "Alexsis", 0.13121547 },
  { "Alexson", 1.0 },
  { "Alexsondra", 0.0 },
  { "Alexssa", 0.0 },
  { "Alexsus", 0.0 },
  { "Alexsys", 0.0 },
  { "Alexuis", 0.0 },
  { "Alexus", 0.013048381 },
  { "Alexusia", 0.0 },
  { "Alexuss", 0.0 },
  { "Alexx", 0.63893807 },
  { "Alexxa", 0.0 },
  { "Alexxander", 1.0 },
  { "Alexxandra", 0.0 },
  { "Alexxandria", 0.0 },
  { "Alexxes", 0.0 },
  { "Alexxia", 0.0 },
  { "Alexxis", 0.005167959 },
  { "Alexxus", 0.0 },
  { "Alexxys", 0.0 },
  { "Alexy", 0.4465409 },
  { "Alexya", 0.0 },
  { "Alexyanna", 0.0 },
  { "Alexyia", 0.0 },
  { "Alexys", 0.08205244 },
  { "Alexyss", 0.0 },
  { "Alexyz", 0.0 },
  { "Alexz", 0.0 },
  { "Alexza", 0.0 },
  { "Alexzandar", 1.0 },
  { "Alexzander", 1.0 },
  { "Alexzandra", 0.0 },
  { "Alexzandrea", 0.0 },
  { "Alexzandria", 0.0 },
  { "Alexzandrya", 0.0 },
  { "Alexzandyr", 1.0 },
  { "Alexzavier", 1.0 },
  { "Alexzia", 0.0 },
  { "Alexzis", 0.0 },
  { "Aley", 0.15384616 },
  { "Aleya", 0.0 },
  { "Aleyah", 0.0 },
  { "Aleyana", 0.0 },
  { "Aleycia", 0.0 },
  { "Aleyda", 0.0 },
  { "Aleydis", 0.0 },
  { "Aleyha", 0.0 },
  { "Aleyia", 0.0 },
  { "Aleyiah", 0.0 },
  { "Aleyla", 0.0 },
  { "Aleyna", 0.0 },
  { "Aleynah", 0.0 },
  { "Aleynna", 0.0 },
  { "Aleysa", 0.0 },
  { "Aleyse", 0.0 },
  { "Aleysha", 0.0 },
  { "Aleyshka", 0.0 },
  { "Aleysia", 0.0 },
  { "Aleyssa", 0.0 },
  { "Aleyza", 0.0 },
  { "Alez", 1.0 },
  { "Aleza", 0.0 },
  { "Alezae", 0.0 },
  { "Alezander", 1.0 },
  { "Alezandra", 0.0 },
  { "Alezandria", 0.0 },
  { "Alezandro", 1.0 },
  { "Alezay", 0.0 },
  { "Alezia", 0.0 },
  { "Alf", 1.0 },
  { "Alfa", 0.020920502 },
  { "Alfair", 0.0 },
  { "Alfanette", 0.0 },
  { "Alfanso", 1.0 },
  { "Alfard", 1.0 },
  { "Alfaretta", 0.0 },
  { "Alferd", 1.0 },
  { "Alferdo", 1.0 },
  { "Alferetta", 0.0 },
  { "Alfhild", 0.0 },
  { "Alfia", 0.0 },
  { "Alfie", 0.82528406 },
  { "Alfio", 1.0 },
  { "Alfiya", 0.0 },
  { "Alfonce", 1.0 },
  { "Alfons", 1.0 },
  { "Alfonsa", 0.92903227 },
  { "Alfonse", 1.0 },
  { "Alfonsina", 0.0 },
  { "Alfonso", 0.99723274 },
  { "Alfonson", 1.0 },
  { "Alfonza", 1.0 },
  { "Alfonzia", 1.0 },
  { "Alfonzie", 1.0 },
  { "Alfonzo", 1.0 },
  { "Alford", 0.99895483 },
  { "Alfornia", 1.0 },
  { "Alfread", 1.0 },
  { "Alfreada", 0.0 },
  { "Alfred", 0.9946759 },
  { "Alfreda", 0.0030674846 },
  { "Alfreddie", 1.0 },
  { "Alfredia", 0.0 },
  { "Alfredo", 0.9947179 },
  { "Alfredrick", 1.0 },
  { "Alfreeda", 0.0 },
  { "Alfreida", 0.0 },
  { "Alfretta", 0.0 },
  { "Alfrida", 0.0 },
  { "Alfrieda", 0.0 },
  { "Alfy", 1.0 },
  { "Alga", 0.03649635 },
  { "Algean", 0.0 },
  { "Algene", 0.3218391 },
  { "Algenis", 1.0 },
  { "Alger", 1.0 },
  { "Algerd", 1.0 },
  { "Algeria", 0.0 },
  { "Algerine", 0.0 },
  { "Algernon", 1.0 },
  { "Algert", 1.0 },
  { "Algertha", 0.0 },
  { "Algia", 0.5576923 },
  { "Algie", 0.8089947 },
  { "Algin", 1.0 },
  { "Algird", 1.0 },
  { "Algirdas", 1.0 },
  { "Algis", 1.0 },
  { "Algot", 1.0 },
  { "Algy", 1.0 },
  { "Alhaji", 1.0 },
  { "Alhakim", 1.0 },
  { "Alham", 0.0 },
  { "Alhan", 1.0 },
  { "Alhana", 0.0 },
  { "Alhanouf", 0.0 },
  { "Alhasan", 1.0 },
  { "Alhassan", 1.0 },
  { "Alhassane", 1.0 },
  { "Alheli", 0.0 },
  { "Alhena", 0.0 },
  { "Alhia", 0.0 },
  { "Ali", 0.71903014 },
  { "Alia", 0.0 },
  { "Aliaa", 0.0 },
  { "Aliah", 0.0 },
  { "Aliaha", 0.0 },
  { "Aliahna", 0.0 },
  { "Alian", 1.0 },
  { "Aliana", 0.0 },
  { "Alianah", 0.0 },
  { "Aliandra", 0.0 },
  { "Aliani", 0.0 },
  { "Alianie", 0.0 },
  { "Alianis", 0.0 },
  { "Alianna", 0.0 },
  { "Aliannah", 0.0 },
  { "Alianne", 0.0 },
  { "Aliannie", 0.0 },
  { "Alianny", 0.0 },
  { "Aliannys", 0.0 },
  { "Aliany", 0.0 },
  { "Alianys", 0.0 },
  { "Alias", 1.0 },
  { "Aliauna", 0.0 },
  { "Aliaya", 0.0 },
  { "Aliayah", 0.0 },
  { "Aliaz", 1.0 },
  { "Alic", 1.0 },
  { "Alica", 0.0 },
  { "Alicai", 0.0 },
  { "Aliccia", 0.0 },
  { "Alice", 0.0034684525 },
  { "Alicea", 0.0 },
  { "Aliceann", 0.0 },
  { "Aliceia", 0.0 },
  { "Alicemae", 0.0 },
  { "Alicemarie", 0.0 },
  { "Alicen", 0.0 },
  { "Aliceson", 0.0 },
  { "Aliceteen", 0.0 },
  { "Aliceyn", 0.0 },
  { "Alicha", 0.0 },
  { "Alichia", 0.0 },
  { "Alicia", 0.003147272 },
  { "Aliciah", 0.0 },
  { "Aliciamarie", 0.0 },
  { "Alician", 0.0 },
  { "Aliciana", 0.0 },
  { "Alicianna", 0.0 },
  { "Aliciya", 0.0 },
  { "Alicja", 0.0 },
  { "Alick", 1.0 },
  { "Alicya", 0.0 },
  { "Alicyn", 0.0 },
  { "Alida", 0.0 },
  { "Alidia", 0.0 },
  { "Alidiya", 0.0 },
  { "Alie", 0.13138686 },
  { "Aliea", 0.0 },
  { "Alieah", 0.0 },
  { "Aliece", 0.0 },
  { "Aliecia", 0.0 },
  { "Alieda", 0.0 },
  { "Aliee", 0.0 },
  { "Alieen", 0.0 },
  { "Aliegha", 0.0 },
  { "Alien", 0.0 },
  { "Aliena", 0.0 },
  { "Aliene", 0.0 },
  { "Alienna", 0.0 },
  { "Aliera", 0.0 },
  { "Aliesa", 0.0 },
  { "Aliese", 0.0 },
  { "Aliesha", 0.0 },
  { "Alieshia", 0.0 },
  { "Aliette", 0.0 },
  { "Alieu", 1.0 },
  { "Alieya", 0.0 },
  { "Alieyah", 0.0 },
  { "Alieza", 0.0 },
  { "Aliezah", 0.0 },
  { "Alif", 1.0 },
  { "Alifa", 0.0 },
  { "Aliha", 0.0 },
  { "Alii", 0.85714287 },
  { "Aliina", 0.0 },
  { "Aliisa", 0.0 },
  { "Aliiyah", 0.0 },
  { "Alija", 0.6036036 },
  { "Alijah", 0.88125604 },
  { "Alijandra", 0.0 },
  { "Alijandro", 1.0 },
  { "Alijha", 0.7962963 },
  { "Alijiah", 1.0 },
  { "Alik", 0.9831081 },
  { "Alika", 0.5404377 },
  { "Alikah", 0.0 },
  { "Alikai", 1.0 },
  { "Alike", 0.0 },
  { "Alikhan", 1.0 },
  { "Aliki", 0.0 },
  { "Alila", 0.0 },
  { "Alilah", 0.0 },
  { "Alilia", 0.0 },
  { "Aliliana", 0.0 },
  { "Alilyana", 0.0 },
  { "Alim", 1.0 },
  { "Alima", 0.0 },
  { "Alimah", 0.0 },
  { "Alimatou", 0.0 },
  { "Alin", 0.24423076 },
  { "Alina", 0.0 },
  { "Alinah", 0.0 },
  { "Alinda", 0.0 },
  { "Aline", 0.00034884532 },
  { "Alinea", 0.0 },
  { "Alinna", 0.0 },
  { "Alinne", 0.0 },
  { "Alinson", 0.0 },
  { "Aliona", 0.0 },
  { "Alionna", 0.0 },
  { "Aliou", 1.0 },
  { "Alioune", 1.0 },
  { "Alique", 0.5 },
  { "Alira", 0.0 },
  { "Alirah", 0.0 },
  { "Alireza", 1.0 },
  { "Alis", 0.015625 },
  { "Alisa", 0.00066644454 },
  { "Alisabeth", 0.0 },
  { "Alisah", 0.0 },
  { "Alisan", 0.0 },
  { "Alisana", 0.0 },
  { "Alisande", 0.0 },
  { "Alisandra", 0.0 },
  { "Alischa", 0.0 },
  { "Aliscia", 0.0 },
  { "Alise", 0.0 },
  { "Alisea", 0.0 },
  { "Alisen", 0.0 },
  { "Alish", 0.0 },
  { "Alisha", 0.0023789203 },
  { "Alishaba", 0.0 },
  { "Alishah", 0.083333336 },
  { "Alishan", 1.0 },
  { "Alishba", 0.0 },
  { "Alishea", 0.0 },
  { "Alisher", 1.0 },
  { "Alishia", 0.0 },
  { "Alishya", 0.0 },
  { "Alisi", 0.0 },
  { "Alisia", 0.0 },
  { "Alisiah", 0.0 },
  { "Alisiana", 0.0 },
  { "Aliska", 0.0 },
  { "Alison", 0.005302962 },
  { "Aliss", 0.0 },
  { "Alissa", 0.0008326842 },
  { "Alissabeth", 0.0 },
  { "Alissah", 0.0 },
  { "Alissandra", 0.0 },
  { "Alisse", 0.0 },
  { "Alissha", 0.0 },
  { "Alissia", 0.0 },
  { "Alisson", 0.0 },
  { "Alissondra", 0.0 },
  { "Alissya", 0.0 },
  { "Alista", 0.0 },
  { "Alistair", 1.0 },
  { "Alistaire", 1.0 },
  { "Alistar", 1.0 },
  { "Alister", 1.0 },
  { "Alisun", 0.0 },
  { "Alisya", 0.0 },
  { "Alisyn", 0.0 },
  { "Alisynn", 0.0 },
  { "Alita", 0.0 },
  { "Alitha", 0.0 },
  { "Alithea", 0.0 },
  { "Alithia", 0.0 },
  { "Alitia", 0.0 },
  { "Alitza", 0.0 },
  { "Alitzah", 0.0 },
  { "Alitzel", 0.0 },
  { "Alius", 1.0 },
  { "Aliva", 0.0 },
  { "Alivea", 0.0 },
  { "Aliveah", 0.0 },
  { "Aliveya", 0.0 },
  { "Alivia", 0.0003974563 },
  { "Aliviah", 0.0 },
  { "Aliviana", 0.0 },
  { "Alivianna", 0.0 },
  { "Aliviya", 0.0 },
  { "Aliviyah", 0.0 },
  { "Alivn", 1.0 },
  { "Alivya", 0.0 },
  { "Alivyah", 0.0 },
  { "Alix", 0.20876777 },
  { "Alixa", 0.0 },
  { "Alixander", 1.0 },
  { "Alixandra", 0.0 },
  { "Alixandrea", 0.0 },
  { "Alixandria", 0.0 },
  { "Alixe", 0.0 },
  { "Alixia", 0.0 },
  { "Alixis", 0.0 },
  { "Alixon", 0.0 },
  { "Alixx", 0.0 },
  { "Alixzander", 1.0 },
  { "Alixzandria", 0.0 },
  { "Aliya", 0.0 },
  { "Aliyaah", 0.0 },
  { "Aliyaan", 1.0 },
  { "Aliyah", 0.0009307836 },
  { "Aliyaha", 0.0 },
  { "Aliyahmarie", 0.0 },
  { "Aliyahna", 0.0 },
  { "Aliyan", 0.7659575 },
  { "Aliyana", 0.0 },
  { "Aliyanah", 0.0 },
  { "Aliyani", 0.0 },
  { "Aliyanna", 0.0 },
  { "Aliyannah", 0.0 },
  { "Aliyas", 1.0 },
  { "Aliye", 0.0 },
  { "Aliyha", 0.0 },
  { "Aliyia", 0.0 },
  { "Aliyiah", 0.0 },
  { "Aliyna", 0.0 },
  { "Aliyonna", 0.0 },
  { "Aliysha", 0.0 },
  { "Aliyssa", 0.0 },
  { "Aliyya", 0.0 },
  { "Aliyyah", 0.0 },
  { "Aliz", 0.0 },
  { "Aliza", 0.00061387353 },
  { "Alizabella", 0.0 },
  { "Alizabeth", 0.0 },
  { "Alizae", 0.005740528 },
  { "Alizah", 0.006443299 },
  { "Alizai", 0.0 },
  { "Alizander", 1.0 },
  { "Alizandra", 0.0 },
  { "Alizay", 0.0068775793 },
  { "Alizaya", 0.0 },
  { "Alizayah", 0.0 },
  { "Alizaye", 0.0 },
  { "Alize", 0.0629152 },
  { "Alizea", 0.0 },
  { "Alizeah", 0.0 },
  { "Alizee", 0.0 },
  { "Alizeh", 0.0 },
  { "Alizet", 0.0 },
  { "Alizette", 0.0 },
  { "Alizey", 0.0 },
  { "Alizeya", 0.0 },
  { "Alizia", 0.0 },
  { "Aliziah", 0.0 },
  { "Alizon", 0.0 },
  { "Alizza", 0.0 },
  { "Alizzon", 0.0 },
  { "Aljandro", 1.0 },
  { "Aljaquan", 1.0 },
  { "Aljawhara", 0.0 },
  { "Aljay", 1.0 },
  { "Aljean", 0.0 },
  { "Aljohara", 0.0 },
  { "Aljoharah", 0.0 },
  { "Alka", 0.0 },
  { "Alkeem", 1.0 },
  { "Alkeria", 0.0 },
  { "Alla", 0.008718396 },
  { "Allah", 0.90697676 },
  { "Allahna", 0.0 },
  { "Allaina", 0.0 },
  { "Allaire", 0.0 },
  { "Allam", 1.0 },
  { "Allan", 0.99731266 },
  { "Allana", 0.0 },
  { "Allanah", 0.0 },
  { "Allandra", 0.0 },
  { "Allani", 0.0 },
  { "Allanie", 0.0 },
  { "Allanmichael", 1.0 },
  { "Allanna", 0.0 },
  { "Allannah", 0.0 },
  { "Allante", 0.77834177 },
  { "Allard", 1.0 },
  { "Allasia", 0.0 },
  { "Allateef", 1.0 },
  { "Allaura", 0.0 },
  { "Allaya", 0.0 },
  { "Allayah", 0.0 },
  { "Allayna", 0.0 },
  { "Allaynah", 0.0 },
  { "Allayne", 0.0 },
  { "Alle", 0.0 },
  { "Allea", 0.0 },
  { "Alleah", 0.0 },
  { "Allean", 0.0 },
  { "Alleana", 0.0 },
  { "Alleane", 0.0 },
  { "Allec", 1.0 },
  { "Allecia", 0.0 },
  { "Allee", 0.0 },
  { "Alleen", 0.0 },
  { "Alleena", 0.0 },
  { "Alleene", 0.0 },
  { "Allegra", 0.0 },
  { "Alleigh", 0.0 },
  { "Alleigha", 0.0 },
  { "Alleine", 0.0 },
  { "Alleisha", 0.0 },
  { "Allejah", 0.0 },
  { "Allen", 0.99310875 },
  { "Allena", 0.0 },
  { "Allenah", 0.0 },
  { "Allenby", 1.0 },
  { "Allene", 0.00057110225 },
  { "Allenmichael", 1.0 },
  { "Allenna", 0.0 },
  { "Allera", 0.0 },
  { "Alleria", 0.0 },
  { "Allesandra", 0.0 },
  { "Allese", 0.0 },
  { "Allesha", 0.0 },
  { "Alleshia", 0.0 },
  { "Allesia", 0.0 },
  { "Alleson", 0.0 },
  { "Allessa", 0.0 },
  { "Allessandra", 0.0 },
  { "Alleta", 0.0 },
  { "Alletha", 0.0 },
  { "Alletta", 0.0 },
  { "Allex", 0.7078189 },
  { "Allexa", 0.0 },
  { "Allexander", 1.0 },
  { "Allexandra", 0.0 },
  { "Allexandria", 0.0 },
  { "Allexia", 0.0 },
  { "Allexis", 0.0 },
  { "Allexus", 0.0 },
  { "Allexys", 0.0 },
  { "Alley", 0.042 },
  { "Alleya", 0.0 },
  { "Alleyah", 0.0 },
  { "Alleyna", 0.0 },
  { "Alleyne", 0.0 },
  { "Alli", 0.0014257199 },
  { "Allia", 0.0 },
  { "Alliah", 0.0 },
  { "Alliana", 0.0 },
  { "Allianah", 0.0 },
  { "Allianna", 0.0 },
  { "Alliannah", 0.0 },
  { "Allias", 1.0 },
  { "Allice", 0.0 },
  { "Allicen", 0.0 },
  { "Allicia", 0.0 },
  { "Allicyn", 0.0 },
  { "Allida", 0.0 },
  { "Allie", 0.0628169 },
  { "Allieana", 0.0 },
  { "Allieanna", 0.0 },
  { "Alliee", 0.0 },
  { "Alliemae", 0.0 },
  { "Allien", 0.0 },
  { "Alliene", 0.0 },
  { "Allijah", 1.0 },
  { "Allimae", 0.0 },
  { "Allin", 1.0 },
  { "Allina", 0.0 },
  { "Allinah", 0.0 },
  { "Alline", 0.0 },
  { "Allinson", 0.0 },
  { "Alliona", 0.0 },
  { "Allionna", 0.0 },
  { "Allis", 0.0 },
  { "Allisa", 0.0 },
  { "Allisan", 0.0 },
  { "Allisandra", 0.0 },
  { "Allise", 0.0 },
  { "Allisen", 0.0 },
  { "Allisha", 0.0 },
  { "Allishia", 0.0 },
  { "Allisia", 0.0 },
  { "Allisin", 0.0 },
  { "Allison", 0.012823079 },
  { "Allissa", 0.0 },
  { "Allissia", 0.0 },
  { "Allisson", 0.0 },
  { "Allistair", 1.0 },
  { "Allister", 0.99182564 },
  { "Allisun", 0.0 },
  { "Allisyn", 0.0 },
  { "Allita", 0.0 },
  { "Allivia", 0.0 },
  { "Allix", 0.0 },
  { "Alliya", 0.0 },
  { "Alliyah", 0.0 },
  { "Alliyana", 0.0 },
  { "Alliyiah", 0.0 },
  { "Allizae", 0.0 },
  { "Allize", 0.0 },
  { "Allizon", 0.0 },
  { "Allizzon", 0.0 },
  { "Allon", 1.0 },
  { "Allona", 0.0 },
  { "Allonah", 0.0 },
  { "Allondra", 0.0 },
  { "Allonte", 1.0 },
  { "Allonzo", 1.0 },
  { "Allora", 0.0 },
  { "Allorah", 0.0 },
  { "Allory", 0.0 },
  { "Allsion", 0.0 },
  { "Allson", 0.0 },
  { "Allston", 1.0 },
  { "Allura", 0.0 },
  { "Allure", 0.0 },
  { "Allwin", 1.0 },
  { "Ally", 0.0011243535 },
  { "Allya", 0.0 },
  { "Allyah", 0.0 },
  { "Allyana", 0.0 },
  { "Allyanah", 0.0 },
  { "Allyanna", 0.0 },
  { "Allyannah", 0.0 },
  { "Allyce", 0.0 },
  { "Allycia", 0.0 },
  { "Allye", 0.0 },
  { "Allyiah", 0.0 },
  { "Allyn", 0.63410264 },
  { "Allyna", 0.0 },
  { "Allyne", 0.0 },
  { "Allynn", 0.0 },
  { "Allyonna", 0.0 },
  { "Allyra", 0.0 },
  { "Allyria", 0.0 },
  { "Allysa", 0.0 },
  { "Allysah", 0.0 },
  { "Allysan", 0.0 },
  { "Allysandra", 0.0 },
  { "Allyse", 0.0 },
  { "Allysen", 0.0 },
  { "Allysha", 0.0 },
  { "Allyshia", 0.0 },
  { "Allysia", 0.0 },
  { "Allysin", 0.0 },
  { "Allyson", 0.0004188774 },
  { "Allyssa", 0.0 },
  { "Allyssah", 0.0 },
  { "Allysse", 0.0 },
  { "Allyssia", 0.0 },
  { "Allysson", 0.0 },
  { "Allysun", 0.0 },
  { "Allysyn", 0.0 },
  { "Allyx", 0.0 },
  { "Allyza", 0.0 },
  { "Allyzon", 0.0 },
  { "Alma", 0.013498957 },
  { "Almadelia", 0.0 },
  { "Almae", 0.0 },
  { "Almalik", 1.0 },
  { "Alman", 1.0 },
  { "Almanda", 0.0 },
  { "Almando", 1.0 },
  { "Almanzo", 1.0 },
  { "Almar", 0.0 },
  { "Almaree", 0.0 },
  { "Almarene", 0.0 },
  { "Almarie", 0.0 },
  { "Almarine", 0.0 },
  { "Almarion", 1.0 },
  { "Almarosa", 0.0 },
  { "Almas", 0.0 },
  { "Almatine", 0.0 },
  { "Almaz", 0.0 },
  { "Almeada", 0.0 },
  { "Almeater", 0.0 },
  { "Almeda", 0.0 },
  { "Almedia", 0.0 },
  { "Almedin", 1.0 },
  { "Almedina", 0.0 },
  { "Almee", 0.0 },
  { "Almeer", 1.0 },
  { "Almena", 0.0 },
  { "Almendra", 0.0 },
  { "Almenia", 0.0 },
  { "Almer", 0.6345062 },
  { "Almera", 0.0 },
  { "Almeria", 0.0 },
  { "Almerinda", 0.0 },
  { "Almeta", 0.0 },
  { "Almeter", 0.0 },
  { "Almetia", 0.0 },
  { "Almetra", 0.0 },
  { "Almetta", 0.0 },
  { "Almetter", 0.0 },
  { "Almida", 0.0 },
  { "Almin", 1.0 },
  { "Almina", 0.0 },
  { "Almir", 1.0 },
  { "Almira", 0.0 },
  { "Almitra", 0.0 },
  { "Almo", 0.79452056 },
  { "Almon", 0.9889367 },
  { "Almond", 1.0 },
  { "Almondo", 1.0 },
  { "Almonte", 1.0 },
  { "Almonza", 1.0 },
  { "Almonzo", 1.0 },
  { "Almore", 1.0 },
  { "Almos", 1.0 },
  { "Almous", 1.0 },
  { "Almudena", 0.0 },
  { "Almus", 1.0 },
  { "Almyra", 0.0 },
  { "Alna", 0.0 },
  { "Alneisha", 0.0 },
  { "Alnesha", 0.0 },
  { "Alnetia", 0.0 },
  { "Alnisa", 0.0 },
  { "Alnisha", 0.0 },
  { "Alnita", 0.0 },
  { "Alnora", 0.0 },
  { "Alo", 1.0 },
  { "Aloa", 0.0 },
  { "Aloah", 0.0 },
  { "Alodie", 0.0 },
  { "Aloe", 0.0 },
  { "Aloha", 0.0 },
  { "Alohi", 0.0 },
  { "Alohilani", 0.0 },
  { "Alohna", 0.0 },
  { "Alois", 0.92978483 },
  { "Aloise", 0.21875 },
  { "Aloisius", 1.0 },
  { "Alok", 1.0 },
  { "Aloma", 0.0 },
  { "Alon", 0.97565925 },
  { "Alona", 0.0 },
  { "Alonah", 0.0 },
  { "Alonda", 0.0 },
  { "Alondo", 1.0 },
  { "Alondra", 0.0033023849 },
  { "Alondre", 1.0 },
  { "Alondrea", 0.0 },
  { "Alondria", 0.0 },
  { "Aloni", 0.0 },
  { "Alonia", 0.0 },
  { "Alonie", 0.0 },
  { "Alonii", 0.0 },
  { "Alonna", 0.0 },
  { "Alonnah", 0.0 },
  { "Alonni", 0.0 },
  { "Alonnie", 0.0 },
  { "Alonso", 1.0 },
  { "Alonta", 1.0 },
  { "Alontae", 1.0 },
  { "Alontay", 1.0 },
  { "Alonte", 0.9856115 },
  { "Alonya", 0.0 },
  { "Alonza", 0.99819005 },
  { "Alonzia", 0.22727273 },
  { "Alonzo", 0.99621713 },
  { "Alora", 0.0 },
  { "Alorah", 0.0 },
  { "Alori", 0.0 },
  { "Aloria", 0.0 },
  { "Alouise", 0.0 },
  { "Aloura", 0.0 },
  { "Aloys", 1.0 },
  { "Aloyse", 0.0 },
  { "Aloysia", 0.0 },
  { "Aloysious", 1.0 },
  { "Aloysius", 1.0 },
  { "Aloysuis", 1.0 },
  { "Alp", 1.0 },
  { "Alpa", 0.0 },
  { "Alper", 1.0 },
  { "Alperen", 1.0 },
  { "Alpesh", 1.0 },
  { "Alph", 1.0 },
  { "Alpha", 0.24501696 },
  { "Alpharetta", 0.0 },
  { "Alphe", 1.0 },
  { "Alphea", 0.0 },
  { "Alphee", 1.0 },
  { "Alpheus", 1.0 },
  { "Alphia", 0.0 },
  { "Alphie", 1.0 },
  { "Alphild", 0.0 },
  { "Alphine", 0.0 },
  { "Alphon", 1.0 },
  { "Alphonce", 1.0 },
  { "Alphones", 1.0 },
  { "Alphons", 1.0 },
  { "Alphonsa", 1.0 },
  { "Alphonse", 1.0 },
  { "Alphonsine", 0.0 },
  { "Alphonso", 0.99820095 },
  { "Alphonsus", 1.0 },
  { "Alphonza", 1.0 },
  { "Alphonzo", 1.0 },
  { "Alphus", 1.0 },
  { "Alphy", 1.0 },
  { "Alpine", 0.0 },
  { "Alprentice", 1.0 },
  { "Alquan", 1.0 },
  { "Alquin", 1.0 },
  { "Alrahman", 1.0 },
  { "Alray", 1.0 },
  { "Alreem", 0.0 },
  { "Alric", 1.0 },
  { "Alrick", 1.0 },
  { "Alrik", 1.0 },
  { "Alrita", 0.0 },
  { "Alroy", 1.0 },
  { "Alsa", 0.0 },
  { "Alsace", 0.0 },
  { "Alsatia", 0.0 },
  { "Alsexander", 1.0 },
  { "Alsha", 0.0 },
  { "Alshawn", 1.0 },
  { "Alshon", 1.0 },
  { "Alsie", 0.0 },
  { "Alsiha", 0.0 },
  { "Alsion", 0.0 },
  { "Alson", 1.0 },
  { "Alston", 0.99285716 },
  { "Alta", 0.004065323 },
  { "Altaf", 1.0 },
  { "Altagracia", 0.0 },
  { "Altair", 0.46919432 },
  { "Altaira", 0.0 },
  { "Altamae", 0.0 },
  { "Altamease", 0.0 },
  { "Altamese", 0.0 },
  { "Altan", 1.0 },
  { "Altarik", 1.0 },
  { "Altariq", 1.0 },
  { "Altavious", 1.0 },
  { "Altay", 1.0 },
  { "Altee", 0.0 },
  { "Alten", 1.0 },
  { "Alter", 0.94690263 },
  { "Alterick", 1.0 },
  { "Alterik", 1.0 },
  { "Altermease", 0.0 },
  { "Altha", 0.0062038405 },
  { "Althea", 0.00030255356 },
  { "Altheda", 0.0 },
  { "Altheia", 0.0 },
  { "Althelia", 0.0 },
  { "Althena", 0.0 },
  { "Althera", 0.0 },
  { "Altheria", 0.0 },
  { "Althia", 0.0 },
  { "Altia", 0.0 },
  { "Altie", 0.015151516 },
  { "Altin", 1.0 },
  { "Altina", 0.0 },
  { "Alto", 0.9476351 },
  { "Alton", 0.9919629 },
  { "Altonette", 0.0 },
  { "Altonia", 0.5833333 },
  { "Altonio", 1.0 },
  { "Altonya", 0.0 },
  { "Altoria", 0.0 },
  { "Altouise", 0.0 },
  { "Altovese", 0.0 },
  { "Altovise", 0.0 },
  { "Altrice", 0.0 },
  { "Altrina", 0.0 },
  { "Altus", 1.0 },
  { "Alucard", 1.0 },
  { "Aluel", 0.0 },
  { "Alula", 1.0 },
  { "Aluna", 0.0 },
  { "Alundra", 0.0 },
  { "Alura", 0.0 },
  { "Alusine", 1.0 },
  { "Alva", 0.5644868 },
  { "Alvada", 0.0 },
  { "Alvah", 0.9614561 },
  { "Alvan", 1.0 },
  { "Alvana", 0.0 },
  { "Alvania", 0.0 },
  { "Alvar", 1.0 },
  { "Alvaretta", 0.0 },
  { "Alvarez", 1.0 },
  { "Alvaro", 0.99965894 },
  { "Alve", 1.0 },
  { "Alveda", 0.0 },
  { "Alveena", 0.0 },
  { "Alveiro", 1.0 },
  { "Alven", 1.0 },
  { "Alvena", 0.0 },
  { "Alvene", 0.0 },
  { "Alvenia", 0.0 },
  { "Alver", 0.9596774 },
  { "Alvera", 0.0 },
  { "Alverda", 0.0 },
  { "Alveretta", 0.0 },
  { "Alveria", 0.0 },
  { "Alvern", 0.7236842 },
  { "Alverna", 0.0 },
  { "Alverne", 0.0 },
  { "Alvernia", 0.0 },
  { "Alvernon", 1.0 },
  { "Alvert", 1.0 },
  { "Alverta", 0.0 },
  { "Alvertia", 0.0 },
  { "Alvertis", 1.0 },
  { "Alverto", 1.0 },
  { "Alvery", 1.0 },
  { "Alves", 1.0 },
  { "Alvesta", 0.0 },
  { "Alvester", 1.0 },
  { "Alveta", 0.0 },
  { "Alvetta", 0.0 },
  { "Alvey", 1.0 },
  { "Alvi", 1.0 },
  { "Alvia", 0.5325301 },
  { "Alvida", 0.0 },
  { "Alvie", 0.8709398 },
  { "Alvieri", 1.0 },
  { "Alviery", 1.0 },
  { "Alvilda", 0.0 },
  { "Alvilde", 0.0 },
  { "Alvin", 0.99310064 },
  { "Alvina", 0.00048444918 },
  { "Alvine", 0.040983606 },
  { "Alvinia", 0.0 },
  { "Alvino", 1.0 },
  { "Alvion", 1.0 },
  { "Alvira", 0.0 },
  { "Alvis", 0.93979686 },
  { "Alvita", 0.0 },
  { "Alvoid", 1.0 },
  { "Alvon", 1.0 },
  { "Alvonia", 0.0 },
  { "Alvontae", 1.0 },
  { "Alvonte", 1.0 },
  { "Alvord", 1.0 },
  { "Alvy", 1.0 },
  { "Alvyn", 1.0 },
  { "Alwaleed", 1.0 },
  { "Alward", 1.0 },
  { "Alwayne", 0.0 },
  { "Alwilda", 0.0 },
  { "Alwillie", 0.0 },
  { "Alwin", 1.0 },
  { "Alwina", 0.0 },
  { "Alwine", 0.0 },
  { "Alwood", 1.0 },
  { "Alwyn", 0.9789644 },
  { "Alxander", 1.0 },
  { "Alxavier", 1.0 },
  { "Alxis", 0.0 },
  { "Aly", 0.35123184 },
  { "Alya", 0.0 },
  { "Alyaa", 0.0 },
  { "Alyaan", 1.0 },
  { "Alyah", 0.0 },
  { "Alyan", 1.0 },
  { "Alyana", 0.0 },
  { "Alyanah", 0.0 },
  { "Alyanna", 0.0 },
  { "Alyannah", 0.0 },
  { "Alyas", 1.0 },
  { "Alyasia", 0.0 },
  { "Alyca", 0.0 },
  { "Alyce", 0.0 },
  { "Alycea", 0.0 },
  { "Alycee", 0.0 },
  { "Alyceia", 0.0 },
  { "Alycen", 0.0 },
  { "Alycia", 0.0 },
  { "Alyciah", 0.0 },
  { "Alyciana", 0.0 },
  { "Alyda", 0.0 },
  { "Alydia", 0.0 },
  { "Alye", 0.0 },
  { "Alyea", 0.0 },
  { "Alyena", 0.0 },
  { "Alyene", 0.0 },
  { "Alyese", 0.0 },
  { "Alyeska", 0.0 },
  { "Alyessa", 0.0 },
  { "Alyha", 0.0 },
  { "Alyia", 0.0 },
  { "Alyiah", 0.0 },
  { "Alyiana", 0.0 },
  { "Alyias", 1.0 },
  { "Alyicia", 0.0 },
  { "Alyis", 1.0 },
  { "Alyisa", 0.0 },
  { "Alyissa", 0.0 },
  { "Alyjah", 0.8409894 },
  { "Alyk", 1.0 },
  { "Alyka", 0.0 },
  { "Alyla", 0.0 },
  { "Alylah", 0.0 },
  { "Alympia", 0.0 },
  { "Alymuhammad", 1.0 },
  { "Alyn", 0.4279661 },
  { "Alyna", 0.0 },
  { "Alynah", 0.0 },
  { "Alyncia", 0.0 },
  { "Alynda", 0.0 },
  { "Alyne", 0.0 },
  { "Alynia", 0.0 },
  { "Alynn", 0.0 },
  { "Alynna", 0.0 },
  { "Alynnah", 0.0 },
  { "Alynne", 0.0 },
  { "Alyona", 0.0 },
  { "Alyonna", 0.0 },
  { "Alyosha", 1.0 },
  { "Alyra", 0.0 },
  { "Alyrah", 0.0 },
  { "Alyre", 1.0 },
  { "Alyria", 0.0 },
  { "Alyric", 0.048387095 },
  { "Alyrica", 0.0 },
  { "Alys", 0.0 },
  { "Alysa", 0.0 },
  { "Alysabeth", 0.0 },
  { "Alysah", 0.0 },
  { "Alysan", 0.0 },
  { "Alysandra", 0.0 },
  { "Alyscia", 0.0 },
  { "Alyse", 0.0 },
  { "Alysea", 0.0 },
  { "Alysen", 0.0 },
  { "Alysha", 0.0 },
  { "Alyshea", 0.0 },
  { "Alyshia", 0.0 },
  { "Alysia", 0.0 },
  { "Alysiah", 0.0 },
  { "Alysiana", 0.0 },
  { "Alysianna", 0.0 },
  { "Alysin", 0.0 },
  { "Alyson", 0.00036241076 },
  { "Alyss", 0.0 },
  { "Alyssa", 0.0014270047 },
  { "Alyssaann", 0.0 },
  { "Alyssabeth", 0.0 },
  { "Alyssah", 0.0 },
  { "Alyssamae", 0.0 },
  { "Alyssamarie", 0.0 },
  { "Alyssandra", 0.0 },
  { "Alyssanicole", 0.0 },
  { "Alyssarae", 0.0 },
  { "Alyssarose", 0.0 },
  { "Alysse", 0.0 },
  { "Alyssea", 0.0 },
  { "Alyssha", 0.0 },
  { "Alyssia", 0.0 },
  { "Alysson", 0.0 },
  { "Alyssum", 0.0 },
  { "Alyssya", 0.0 },
  { "Alysta", 0.0 },
  { "Alyster", 1.0 },
  { "Alysun", 0.0 },
  { "Alythia", 0.0 },
  { "Alyus", 1.0 },
  { "Alyvea", 0.0 },
  { "Alyvia", 0.0 },
  { "Alyviah", 0.0 },
  { "Alywin", 1.0 },
  { "Alyx", 0.10144927 },
  { "Alyxa", 0.0 },
  { "Alyxander", 1.0 },
  { "Alyxandra", 0.0 },
  { "Alyxandrea", 0.0 },
  { "Alyxandria", 0.0 },
  { "Alyxia", 0.0 },
  { "Alyxis", 0.0 },
  { "Alyxus", 0.0 },
  { "Alyxzander", 1.0 },
  { "Alyza", 0.0 },
  { "Alyzabeth", 0.0 },
  { "Alyzae", 0.0 },
  { "Alyzah", 0.0 },
  { "Alyze", 0.0 },
  { "Alyzea", 0.0 },
  { "Alyzia", 0.0 },
  { "Alyzon", 0.0 },
  { "Alyzza", 0.0 },
  { "Alza", 0.0 },
  { "Alzada", 0.0 },
  { "Alzahra", 0.0 },
  { "Alzain", 0.0 },
  { "Alzata", 0.0 },
  { "Alzena", 0.0 },
  { "Alzenia", 0.0 },
  { "Alzie", 0.0 },
  { "Alzina", 0.0 },
  { "Alzira", 0.0 },
  { "Alzonia", 0.0 },
  { "Alzora", 0.0 },
  { "Ama", 0.0 },
  { "Amaad", 1.0 },
  { "Amaal", 0.0 },
  { "Amaan", 1.0 },
  { "Amaani", 0.0 },
  { "Amaar", 1.0 },
  { "Amaari", 1.0 },
  { "Amabel", 0.0 },
  { "Amabella", 0.0 },
  { "Amabelle", 0.0 },
  { "Amad", 1.0 },
  { "Amada", 0.0 },
  { "Amadea", 0.0 },
  { "Amadeo", 1.0 },
  { "Amadeus", 1.0 },
  { "Amadi", 0.4448276 },
  { "Amadis", 0.0 },
  { "Amadita", 0.0 },
  { "Amado", 1.0 },
  { "Amador", 1.0 },
  { "Amadou", 1.0 },
  { "Amadu", 1.0 },
  { "Amagine", 0.0 },
  { "Amahd", 1.0 },
  { "Amahia", 0.0 },
  { "Amahni", 0.0 },
  { "Amahri", 0.65957445 },
  { "Amahya", 0.0 },
  { "Amai", 0.0 },
  { "Amaia", 0.0 },
  { "Amaiah", 0.0 },
  { "Amaijah", 0.0 },
  { "Amair", 1.0 },
  { "Amaira", 0.0 },
  { "Amairah", 0.0 },
  { "Amairani", 0.0 },
  { "Amairany", 0.0 },
  { "Amaire", 1.0 },
  { "Amais", 1.0 },
  { "Amaiya", 0.0 },
  { "Amaiyah", 0.0 },
  { "Amaj", 1.0 },
  { "Amaja", 0.0 },
  { "Amajae", 1.0 },
  { "Amajah", 0.0 },
  { "Amaje", 1.0 },
  { "Amaka", 0.0 },
  { "Amal", 0.105860114 },
  { "Amala", 0.0 },
  { "Amalea", 0.0 },
  { "Amaleah", 0.0 },
  { "Amalee", 0.0 },
  { "Amali", 0.0 },
  { "Amalia", 0.0 },
  { "Amaliah", 0.0 },
  { "Amalie", 0.0 },
  { "Amalin", 0.0 },
  { "Amalina", 0.0 },
  { "Amalio", 1.0 },
  { "Amaliya", 0.0 },
  { "Amalthea", 0.0 },
  { "Amaly", 0.0 },
  { "Amalya", 0.0 },
  { "Amalyah", 0.0 },
  { "Amalyn", 0.0 },
  { "Amamda", 0.0 },
  { "Aman", 0.9211148 },
  { "Amana", 0.0 },
  { "Amanada", 0.0 },
  { "Amanah", 0.0 },
  { "Amanat", 0.0 },
  { "Amancio", 1.0 },
  { "Amand", 0.0 },
  { "Amanda", 0.0027199944 },
  { "Amandah", 0.0 },
  { "Amandajean", 0.0 },
  { "Amandajo", 0.0 },
  { "Amandalee", 0.0 },
  { "Amandaleigh", 0.0 },
  { "Amandalyn", 0.0 },
  { "Amandalynn", 0.0 },
  { "Amandamarie", 0.0 },
  { "Amandarae", 0.0 },
  { "Amandarose", 0.0 },
  { "Amande", 0.0 },
  { "Amandeep", 0.5642857 },
  { "Amandia", 0.0 },
  { "Amandine", 0.0 },
  { "Amandip", 1.0 },
  { "Amando", 0.8980213 },
  { "Amandra", 0.0 },
  { "Amandus", 1.0 },
  { "Amandy", 0.0 },
  { "Amane", 0.0 },
  { "Amanee", 0.0 },
  { "Amani", 0.17655185 },
  { "Amania", 0.0 },
  { "Amanie", 0.029411765 },
  { "Amanii", 0.0 },
  { "Amanita", 0.0 },
  { "Amanjit", 1.0 },
  { "Amanjot", 1.0 },
  { "Amannda", 0.0 },
  { "Amanni", 0.0 },
  { "Amanoa", 0.0 },
  { "Amanpreet", 0.21383648 },
  { "Amante", 1.0 },
  { "Amantha", 0.0 },
  { "Amanti", 0.6419753 },
  { "Amanuel", 1.0 },
  { "Amany", 0.0 },
  { "Amar", 0.9488666 },
  { "Amara", 0.017323121 },
  { "Amarachi", 0.0 },
  { "Amarachukwu", 0.0 },
  { "Amarae", 0.0 },
  { "Amarah", 0.0 },
  { "Amaranta", 0.0 },
  { "Amarante", 1.0 },
  { "Amaranth", 0.0 },
  { "Amarantha", 0.0 },
  { "Amardeep", 1.0 },
  { "Amare", 0.943762 },
  { "Amarea", 0.048387095 },
  { "Amareah", 0.0 },
  { "Amaree", 0.5652174 },
  { "Amareion", 1.0 },
  { "Amareli", 0.0 },
  { "Amarelis", 0.0 },
  { "Amarely", 0.0 },
  { "Amarelys", 0.0 },
  { "Amareon", 1.0 },
  { "Amareona", 0.0 },
  { "Amarey", 0.8 },
  { "Amareya", 0.0 },
  { "Amareyon", 1.0 },
  { "Amarhi", 0.0 },
  { "Amari", 0.54339325 },
  { "Amaria", 0.0026136958 },
  { "Amariae", 1.0 },
  { "Amariah", 0.0 },
  { "Amarian", 1.0 },
  { "Amariana", 0.0 },
  { "Amariani", 0.0 },
  { "Amarianna", 0.0 },
  { "Amariauna", 0.0 },
  { "Amarie", 0.19341373 },
  { "Amariea", 0.0 },
  { "Amariee", 0.0 },
  { "Amarielle", 0.0 },
  { "Amarien", 1.0 },
  { "Amarieon", 1.0 },
  { "Amarii", 0.55396825 },
  { "Amarilis", 0.0 },
  { "Amarillis", 0.0 },
  { "Amarillys", 0.0 },
  { "Amarily", 0.0 },
  { "Amarilys", 0.0 },
  { "Amarin", 0.5 },
  { "Amarina", 0.0 },
  { "Amario", 1.0 },
  { "Amarion", 0.9866949 },
  { "Amariona", 0.0 },
  { "Amarionna", 0.0 },
  { "Amarious", 1.0 },
  { "Amaris", 0.032007046 },
  { "Amarisa", 0.0 },
  { "Amarise", 0.0 },
  { "Amariss", 0.0 },
  { "Amarissa", 0.0 },
  { "Amarius", 1.0 },
  { "Amariya", 0.0 },
  { "Amariyae", 0.0 },
  { "Amariyah", 0.0 },
  { "Amariyana", 0.0 },
  { "Amariyanna", 0.0 },
  { "Amariyon", 1.0 },
  { "Amariz", 0.0 },
  { "Amarley", 0.0 },
  { "Amaro", 1.0 },
  { "Amaron", 1.0 },
  { "Amarpreet", 0.0 },
  { "Amarr", 1.0 },
  { "Amarra", 0.0 },
  { "Amarrah", 0.0 },
  { "Amarre", 1.0 },
  { "Amarri", 0.53419596 },
  { "Amarria", 0.0 },
  { "Amarrie", 0.0 },
  { "Amarrion", 1.0 },
  { "Amartya", 1.0 },
  { "Amaru", 1.0 },
  { "Amary", 0.11111111 },
  { "Amarya", 0.0 },
  { "Amaryah", 0.0 },
  { "Amaryana", 0.0 },
  { "Amarye", 0.0 },
  { "Amaryia", 0.0 },
  { "Amarylis", 0.0 },
  { "Amaryllis", 0.0 },
  { "Amaryn", 0.0 },
  { "Amaryon", 1.0 },
  { "Amarys", 0.0 },
  { "Amaryss", 0.0 },
  { "Amasa", 1.0 },
  { "Amata", 0.0 },
  { "Amathyst", 0.0 },
  { "Amato", 1.0 },
  { "Amatullah", 0.0 },
  { "Amaud", 1.0 },
  { "Amaura", 0.0 },
  { "Amaure", 1.0 },
  { "Amauri", 0.7181673 },
  { "Amauria", 0.0 },
  { "Amauriah", 0.0 },
  { "Amaurie", 0.5555556 },
  { "Amaurion", 1.0 },
  { "Amauris", 1.0 },
  { "Amaury", 0.9957948 },
  { "Amaurys", 1.0 },
  { "Amay", 1.0 },
  { "Amaya", 0.0006371098 },
  { "Amayah", 0.0 },
  { "Amayha", 0.0 },
  { "Amayia", 0.0 },
  { "Amayiah", 0.0 },
  { "Amayla", 0.0 },
  { "Amaylee", 0.0 },
  { "Amayra", 0.0 },
  { "Amayrani", 0.0 },
  { "Amayrany", 0.0 },
  { "Amazen", 0.0 },
  { "Amaziah", 0.7606838 },
  { "Amazin", 0.0 },
  { "Amazing", 0.046153847 },
  { "Amazyn", 0.0 },
  { "Amba", 0.0 },
  { "Ambar", 0.0021626297 },
  { "Ambee", 0.0 },
  { "Ambellina", 0.0 },
  { "Amber", 0.0026507496 },
  { "Ambera", 0.0 },
  { "Amberann", 0.0 },
  { "Amberdawn", 0.0 },
  { "Ambere", 0.0 },
  { "Amberia", 0.0 },
  { "Amberjo", 0.0 },
  { "Amberle", 0.0 },
  { "Amberlea", 0.0 },
  { "Amberlee", 0.0 },
  { "Amberleigh", 0.0 },
  { "Amberley", 0.0 },
  { "Amberli", 0.0 },
  { "Amberlie", 0.0 },
  { "Amberlin", 0.0 },
  { "Amberly", 0.0 },
  { "Amberlyn", 0.0 },
  { "Amberlynn", 0.0 },
  { "Amberlynne", 0.0 },
  { "Ambermarie", 0.0 },
  { "Ambernicole", 0.0 },
  { "Ambernique", 0.0 },
  { "Amberrae", 0.0 },
  { "Amberrose", 0.0 },
  { "Ambers", 1.0 },
  { "Ambert", 0.0 },
  { "Ambi", 0.0 },
  { "Ambika", 0.0 },
  { "Ambir", 0.0 },
  { "Ambor", 0.0 },
  { "Ambr", 0.0 },
  { "Ambra", 0.0 },
  { "Ambrasia", 0.0 },
  { "Ambre", 0.0 },
  { "Ambrea", 0.0 },
  { "Ambreal", 0.0 },
  { "Ambreanna", 0.0 },
  { "Ambree", 0.0 },
  { "Ambreen", 0.0 },
  { "Ambreia", 0.0 },
  { "Ambresha", 0.0 },
  { "Ambrey", 0.0 },
  { "Ambri", 0.0 },
  { "Ambria", 0.0 },
  { "Ambriah", 0.0 },
  { "Ambriana", 0.0 },
  { "Ambrianna", 0.0 },
  { "Ambrie", 0.0 },
  { "Ambriehl", 0.0 },
  { "Ambriel", 0.0 },
  { "Ambriella", 0.0 },
  { "Ambrielle", 0.0 },
  { "Ambrocio", 1.0 },
  { "Ambros", 1.0 },
  { "Ambrose", 1.0 },
  { "Ambrosha", 0.0 },
  { "Ambrosia", 0.0 },
  { "Ambrosio", 1.0 },
  { "Ambrous", 1.0 },
  { "Ambry", 0.0 },
  { "Ambur", 0.0 },
  { "Ambus", 1.0 },
  { "Amby", 0.0 },
  { "Ambyr", 0.0 },
  { "Amdanda", 0.0 },
  { "Amdrea", 0.0 },
  { "Amdrew", 1.0 },
  { "Ame", 0.0 },
  { "Amea", 0.0 },
  { "Ameah", 0.0 },
  { "Ameal", 1.0 },
  { "Amear", 1.0 },
  { "Ameara", 0.0 },
  { "Amechi", 1.0 },
  { "Amecia", 0.0 },
  { "Amed", 1.0 },
  { "Amedee", 1.0 },
  { "Amedeo", 1.0 },
  { "Amedio", 1.0 },
  { "Amee", 0.0 },
  { "Ameelah", 0.0 },
  { "Ameelia", 0.0 },
  { "Ameen", 1.0 },
  { "Ameena", 0.0 },
  { "Ameenah", 0.0 },
  { "Ameer", 1.0 },
  { "Ameera", 0.0 },
  { "Ameerah", 0.0 },
  { "Ameere", 1.0 },
  { "Ameeria", 0.0 },
  { "Ameesha", 0.0 },
  { "Ameet", 1.0 },
  { "Ameeya", 0.0 },
  { "Ameeyah", 0.0 },
  { "Amei", 0.0 },
  { "Ameia", 0.0 },
  { "Ameiah", 0.0 },
  { "Ameika", 0.0 },
  { "Ameil", 1.0 },
  { "Ameila", 0.0 },
  { "Ameilia", 0.0 },
  { "Ameilya", 0.0 },
  { "Ameina", 0.0 },
  { "Ameir", 1.0 },
  { "Ameira", 0.0 },
  { "Ameirah", 0.0 },
  { "Ameire", 1.0 },
  { "Ameisha", 0.0 },
  { "Ameiya", 0.0 },
  { "Ameiyah", 0.0 },
  { "Ameka", 0.0 },
  { "Amekia", 0.0 },
  { "Amel", 0.6034091 },
  { "Amela", 0.0 },
  { "Amelah", 0.0 },
  { "Amelda", 0.0 },
  { "Amelea", 0.0 },
  { "Ameleah", 0.0 },
  { "Amelee", 0.0 },
  { "Ameli", 0.0 },
  { "Amelia", 0.0011865296 },
  { "Ameliagrace", 0.0 },
  { "Ameliah", 0.0 },
  { "Ameliajane", 0.0 },
  { "Ameliamae", 0.0 },
  { "Ameliana", 0.0 },
  { "Amelianna", 0.0 },
  { "Ameliarose", 0.0 },
  { "Amelie", 0.0 },
  { "Amelina", 0.0 },
  { "Amelinda", 0.0 },
  { "Ameline", 0.0 },
  { "Amelio", 1.0 },
  { "Amelita", 0.0 },
  { "Ameliya", 0.0 },
  { "Ameliyah", 0.0 },
  { "Amella", 0.0 },
  { "Amelle", 0.0 },
  { "Amellia", 0.0 },
  { "Amely", 0.0 },
  { "Amelya", 0.0 },
  { "Amelyah", 0.0 },
  { "Amen", 0.63871634 },
  { "Amena", 0.0 },
  { "Amenah", 0.0 },
  { "Amenata", 0.0 },
  { "Amenda", 0.0 },
  { "Amenia", 0.0 },
  { "Amer", 0.99018 },
  { "Amera", 0.0 },
  { "Amerah", 0.0 },
  { "Amere", 0.83666277 },
  { "Ameri", 0.046511628 },
  { "Ameria", 0.0 },
  { "Ameriah", 0.0 },
  { "Ameriana", 0.0 },
  { "America", 0.0012933473 },
  { "Americas", 0.0 },
  { "Americo", 1.0 },
  { "Americus", 0.2508834 },
  { "Amerie", 0.0 },
  { "Amerigo", 1.0 },
  { "Amerika", 0.0 },
  { "Amerion", 1.0 },
  { "Ameris", 0.0 },
  { "Amerissa", 0.0 },
  { "Amerius", 0.0 },
  { "Ameriya", 0.0 },
  { "Ameriyah", 0.0 },
  { "Amerra", 0.0 },
  { "Amerson", 0.5 },
  { "Amery", 0.17082533 },
  { "Ameryst", 0.0 },
  { "Ames", 0.9955986 },
  { "Amesha", 0.0 },
  { "Ameshia", 0.0 },
  { "Amethyst", 0.0 },
  { "Ametra", 0.0 },
  { "Ametria", 0.0 },
  { "Amey", 0.030751709 },
  { "Ameya", 0.24629879 },
  { "Ameyaa", 0.0 },
  { "Ameyah", 0.0 },
  { "Ameyali", 0.0 },
  { "Ameyalli", 0.0 },
  { "Ameyia", 0.0 },
  { "Amgela", 0.0 },
  { "Amhad", 1.0 },
  { "Ami", 0.03925482 },
  { "Amia", 0.0 },
  { "Amiah", 0.0 },
  { "Amiaha", 0.0 },
  { "Amias", 1.0 },
  { "Amiaya", 0.0 },
  { "Amiayah", 0.0 },
  { "Amica", 0.0 },
  { "Amichai", 1.0 },
  { "Amicia", 0.0 },
  { "Amid", 1.0 },
  { "Amida", 0.0 },
  { "Amidala", 0.0 },
  { "Amie", 0.0018792577 },
  { "Amiee", 0.0 },
  { "Amiel", 0.9448209 },
  { "Amiela", 0.0 },
  { "Amielia", 0.0 },
  { "Amielle", 0.0 },
  { "Amiena", 0.0 },
  { "Amier", 1.0 },
  { "Amiera", 0.0 },
  { "Amierah", 0.0 },
  { "Amiere", 0.9270833 },
  { "Amii", 0.0 },
  { "Amiia", 0.0 },
  { "Amiin", 1.0 },
  { "Amiina", 0.0 },
  { "Amiir", 1.0 },
  { "Amiira", 0.0 },
  { "Amija", 0.0 },
  { "Amijah", 0.0 },
  { "Amika", 0.0 },
  { "Amil", 0.75582576 },
  { "Amila", 0.0 },
  { "Amilah", 0.0 },
  { "Amilcar", 1.0 },
  { "Amilcare", 1.0 },
  { "Amilda", 0.0 },
  { "Amile", 1.0 },
  { "Amilea", 0.0 },
  { "Amileah", 0.0 },
  { "Amilee", 0.0 },
  { "Amileo", 1.0 },
  { "Amiley", 0.0 },
  { "Amilia", 0.0 },
  { "Amiliah", 0.0 },
  { "Amilian", 0.0 },
  { "Amiliana", 0.0 },
  { "Amiliano", 1.0 },
  { "Amilie", 0.0 },
  { "Amilio", 1.0 },
  { "Amiliya", 0.0 },
  { "Amiliyah", 0.0 },
  { "Amill", 1.0 },
  { "Amilla", 0.0 },
  { "Amilleon", 0.0 },
  { "Amilli", 0.0 },
  { "Amillia", 0.0 },
  { "Amilliah", 0.0 },
  { "Amillian", 0.0 },
  { "Amilliana", 0.0 },
  { "Amillianna", 0.0 },
  { "Amillie", 0.0 },
  { "Amillio", 1.0 },
  { "Amillion", 0.3529412 },
  { "Amilliona", 0.0 },
  { "Amilliyon", 0.0 },
  { "Amillya", 0.0 },
  { "Amily", 0.0 },
  { "Amilya", 0.0 },
  { "Amilyah", 0.0 },
  { "Amilyn", 0.0 },
  { "Amilynn", 0.0 },
  { "Amin", 1.0 },
  { "Amina", 0.0 },
  { "Aminah", 0.0 },
  { "Aminat", 0.0 },
  { "Aminata", 0.0 },
  { "Aminatou", 0.0 },
  { "Aminda", 0.0 },
  { "Amine", 1.0 },
  { "Aminna", 0.0 },
  { "Amino", 0.0 },
  { "Aminta", 0.0 },
  { "Amir", 0.98594445 },
  { "Amira", 0.0 },
  { "Amiracle", 0.0 },
  { "Amirah", 0.0 },
  { "Amirali", 1.0 },
  { "Amirbek", 1.0 },
  { "Amire", 0.86212623 },
  { "Amirea", 0.0 },
  { "Amiree", 0.0 },
  { "Amirh", 0.0 },
  { "Amiri", 0.8112927 },
  { "Amiria", 0.0 },
  { "Amiriah", 0.0 },
  { "Amirie", 0.0 },
  { "Amiriya", 0.0 },
  { "Amirjon", 1.0 },
  { "Amirr", 1.0 },
  { "Amirra", 0.0 },
  { "Amirrah", 0.0 },
  { "Amirya", 0.0 },
  { "Amiryah", 0.0 },
  { "Amirykal", 0.0 },
  { "Amis", 1.0 },
  { "Amisa", 0.0 },
  { "Amisadai", 0.03968254 },
  { "Amisaday", 0.0 },
  { "Amish", 1.0 },
  { "Amisha", 0.0 },
  { "Amishi", 0.0 },
  { "Amissa", 0.0 },
  { "Amista", 0.0 },
  { "Amistad", 1.0 },
  { "Amit", 0.93591905 },
  { "Amita", 0.0 },
  { "Amitabh", 1.0 },
  { "Amitai", 1.0 },
  { "Amith", 1.0 },
  { "Amitha", 0.0 },
  { "Amitiel", 0.0 },
  { "Amitis", 0.0 },
  { "Amitoj", 1.0 },
  { "Amittai", 1.0 },
  { "Amity", 0.0 },
  { "Amius", 1.0 },
  { "Amiya", 0.0 },
  { "Amiyaah", 0.0 },
  { "Amiyah", 0.0 },
  { "Amiyha", 0.0 },
  { "Amiyiah", 0.0 },
  { "Amiyla", 0.0 },
  { "Amiylah", 0.0 },
  { "Amiyr", 1.0 },
  { "Amiyra", 0.0 },
  { "Amiyrah", 0.0 },
  { "Amjad", 1.0 },
  { "Amjed", 1.0 },
  { "Amma", 0.0 },
  { "Ammaar", 1.0 },
  { "Ammaarah", 0.0 },
  { "Ammad", 1.0 },
  { "Amman", 1.0 },
  { "Ammanda", 0.0 },
  { "Ammanuel", 1.0 },
  { "Ammar", 1.0 },
  { "Ammara", 0.0 },
  { "Ammarah", 0.0 },
  { "Ammarie", 0.0 },
  { "Ammber", 0.0 },
  { "Amme", 0.0 },
  { "Ammer", 0.5 },
  { "Ammerie", 0.0 },
  { "Ammi", 0.0 },
  { "Ammie", 0.012359984 },
  { "Ammiel", 1.0 },
  { "Ammon", 1.0 },
  { "Ammy", 0.0 },
  { "Amna", 0.0 },
  { "Amnah", 0.0 },
  { "Amneet", 0.0 },
  { "Amneh", 0.0 },
  { "Amner", 1.0 },
  { "Amneris", 0.0 },
  { "Amnesty", 0.0 },
  { "Amo", 0.33333334 },
  { "Amod", 1.0 },
  { "Amogh", 1.0 },
  { "Amoi", 0.0 },
  { "Amol", 1.0 },
  { "Amolika", 0.0 },
  { "Amon", 1.0 },
  { "Amond", 1.0 },
  { "Amonda", 0.0 },
  { "Amondo", 1.0 },
  { "Amondre", 1.0 },
  { "Amone", 0.0 },
  { "Amonee", 0.0 },
  { "Amoni", 0.13309982 },
  { "Amonie", 0.0 },
  { "Amonii", 0.0 },
  { "Amonni", 0.0 },
  { "Amonnie", 0.0 },
  { "Amont", 1.0 },
  { "Amonta", 1.0 },
  { "Amontae", 1.0 },
  { "Amontay", 1.0 },
  { "Amonte", 1.0 },
  { "Amonti", 0.9166667 },
  { "Amor", 0.20233747 },
  { "Amora", 0.0 },
  { "Amorae", 0.0 },
  { "Amorah", 0.0 },
  { "Amore", 0.18313253 },
  { "Amoree", 0.0 },
  { "Amoreena", 0.0 },
  { "Amoret", 0.0 },
  { "Amorette", 0.0 },
  { "Amori", 0.40980393 },
  { "Amoria", 0.0 },
  { "Amoriah", 0.0 },
  { "Amorian", 1.0 },
  { "Amoriana", 0.0 },
  { "Amorie", 0.16433567 },
  { "Amorina", 0.0 },
  { "Amorion", 1.0 },
  { "Amorita", 0.0 },
  { "Amorra", 0.0 },
  { "Amorrah", 0.0 },
  { "Amory", 0.5034325 },
  { "Amos", 0.9980517 },
  { "Amose", 1.0 },
  { "Amour", 0.13861386 },
  { "Amoura", 0.0 },
  { "Amoure", 0.0 },
  { "Amouri", 0.56 },
  { "Amous", 1.0 },
  { "Amoy", 0.0 },
  { "Amoya", 0.0 },
  { "Amoz", 1.0 },
  { "Amparo", 0.005178191 },
  { "Ample", 0.0 },
  { "Amr", 1.0 },
  { "Amra", 0.0 },
  { "Amrah", 0.0 },
  { "Amram", 1.0 },
  { "Amran", 0.17982456 },
  { "Amree", 0.0 },
  { "Amreen", 0.0 },
  { "Amreet", 0.0 },
  { "Amren", 1.0 },
  { "Amri", 0.026143791 },
  { "Amrie", 0.0 },
  { "Amriel", 0.0 },
  { "Amrik", 1.0 },
  { "Amrin", 0.0 },
  { "Amrit", 0.5790698 },
  { "Amrita", 0.0 },
  { "Amrith", 1.0 },
  { "Amritha", 0.0 },
  { "Amritpal", 1.0 },
  { "Amro", 1.0 },
  { "Amrom", 1.0 },
  { "Amron", 1.0 },
  { "Amruta", 0.0 },
  { "Amruth", 1.0 },
  { "Amrutha", 0.0 },
  { "Amry", 0.0 },
  { "Amryn", 0.11320755 },
  { "Amsey", 0.0 },
  { "Amsi", 0.0 },
  { "Amuel", 1.0 },
  { "Amukta", 0.0 },
  { "Amulek", 1.0 },
  { "Amulya", 0.0 },
  { "Amun", 1.0 },
  { "Amunet", 0.0 },
  { "Amunique", 0.0 },
  { "Amuri", 1.0 },
  { "Amvi", 0.0 },
  { "Amy", 0.0026740136 },
  { "Amya", 0.0 },
  { "Amyah", 0.0 },
  { "Amyann", 0.0 },
  { "Amyas", 1.0 },
  { "Amybeth", 0.0 },
  { "Amye", 0.0 },
  { "Amyha", 0.0 },
  { "Amyia", 0.0 },
  { "Amyiah", 0.0 },
  { "Amyjo", 0.0 },
  { "Amyla", 0.0 },
  { "Amylah", 0.0 },
  { "Amylea", 0.0 },
  { "Amylee", 0.0 },
  { "Amylia", 0.0 },
  { "Amyliah", 0.0 },
  { "Amylyn", 0.0 },
  { "Amylynn", 0.0 },
  { "Amymarie", 0.0 },
  { "Amyna", 0.0 },
  { "Amyr", 1.0 },
  { "Amyra", 0.0 },
  { "Amyracle", 0.0 },
  { "Amyrah", 0.0 },
  { "Amyre", 0.10909091 },
  { "Amyree", 0.0 },
  { "Amyri", 0.0 },
  { "Amyria", 0.0 },
  { "Amyriah", 0.0 },
  { "Amyrical", 0.0 },
  { "Amyrie", 0.0 },
  { "Amyrikal", 0.0 },
  { "Amyrion", 1.0 },
  { "Amysue", 0.0 },
  { "Amythest", 0.0 },
  { "Amythyst", 0.0 },
  { "Amyya", 0.0 },
  { "Amzee", 0.0 },
  { "Amzi", 0.375 },
  { "Amzie", 0.34591195 },
  { "An", 0.45422536 },
  { "Ana", 0.005942864 },
  { "Anaahat", 0.0 },
  { "Anaalicia", 0.0 },
  { "Anaaya", 0.0 },
  { "Anab", 0.0 },
  { "Anabeatriz", 0.0 },
  { "Anabel", 0.003782557 },
  { "Anabela", 0.0 },
  { "Anabele", 0.0 },
  { "Anabelen", 0.0 },
  { "Anabeli", 0.0 },
  { "Anabelia", 0.0 },
  { "Anabell", 0.0 },
  { "Anabella", 0.0 },
  { "Anabelle", 0.0 },
  { "Anabelli", 0.0 },
  { "Anaberta", 0.0 },
  { "Anabeth", 0.0 },
  { "Anabia", 0.0 },
  { "Anabiya", 0.0 },
  { "Anabrenda", 0.0 },
  { "Anacamila", 0.0 },
  { "Anacani", 0.0 },
  { "Anacaren", 0.0 },
  { "Anacarina", 0.0 },
  { "Anacarolina", 0.0 },
  { "Anacecilia", 0.0 },
  { "Anaceli", 0.0 },
  { "Anacelia", 0.0 },
  { "Anacelis", 0.0 },
  { "Anacely", 0.0 },
  { "Anachristina", 0.0 },
  { "Anacia", 0.0 },
  { "Anaclara", 0.0 },
  { "Anacleto", 1.0 },
  { "Anacristina", 0.0 },
  { "Anadalay", 0.0 },
  { "Anadela", 0.0 },
  { "Anadelia", 0.0 },
  { "Anadia", 0.0 },
  { "Anae", 0.0 },
  { "Anaee", 0.0 },
  { "Anaeja", 0.0 },
  { "Anaejah", 0.0 },
  { "Anael", 0.19469027 },
  { "Anaeli", 0.0 },
  { "Anaelisa", 0.0 },
  { "Anaelise", 0.0 },
  { "Anaelle", 0.0 },
  { "Anaely", 0.0 },
  { "Anagabriel", 0.0 },
  { "Anagabriela", 0.0 },
  { "Anagh", 1.0 },
  { "Anagha", 0.0 },
  { "Anagrace", 0.0 },
  { "Anah", 0.0 },
  { "Anahat", 0.0 },
  { "Anahi", 0.0021260544 },
  { "Anahia", 0.0 },
  { "Anahid", 0.0 },
  { "Anahie", 0.0 },
  { "Anahii", 0.0 },
  { "Anahis", 0.0 },
  { "Anahit", 0.0 },
  { "Anahita", 0.0 },
  { "Anahla", 0.0 },
  { "Anahli", 0.0 },
  { "Anahlia", 0.0 },
  { "Anahly", 0.0 },
  { "Anahy", 0.0 },
  { "Anahya", 0.0 },
  { "Anai", 0.0 },
  { "Anaia", 0.0 },
  { "Anaiah", 0.0063931807 },
  { "Anaias", 1.0 },
  { "Anaica", 0.0 },
  { "Anaid", 0.0 },
  { "Anaida", 0.0 },
  { "Anaie", 0.0 },
  { "Anaih", 0.0 },
  { "Anaiis", 0.0 },
  { "Anaija", 0.0 },
  { "Anaijah", 0.0 },
  { "Anaika", 0.0 },
  { "Anaila", 0.0 },
  { "Anailah", 0.0 },
  { "Anaili", 0.0 },
  { "Anaily", 0.0 },
  { "Anaira", 0.0 },
  { "Anairis", 0.0 },
  { "Anais", 0.007082371 },
  { "Anaisa", 0.0 },
  { "Anaisabel", 0.0 },
  { "Anaisabella", 0.0 },
  { "Anaise", 0.0 },
  { "Anaisha", 0.0 },
  { "Anaisia", 0.0 },
  { "Anaissa", 0.0 },
  { "Anaiya", 0.0 },
  { "Anaiyah", 0.0 },
  { "Anaiz", 0.0 },
  { "Anaiza", 0.0 },
  { "Anaja", 0.0 },
  { "Anajae", 0.0 },
  { "Anajah", 0.0 },
  { "Anajee", 1.0 },
  { "Anajia", 0.0 },
  { "Anajulia", 0.0 },
  { "Anaka", 0.0 },
  { "Anakah", 0.0 },
  { "Anakaren", 0.0 },
  { "Anakarina", 0.0 },
  { "Anakin", 0.99698013 },
  { "Anakyn", 1.0 },
  { "Anala", 0.0 },
  { "Analah", 0.0 },
  { "Analaura", 0.0 },
  { "Analaya", 0.0 },
  { "Analayah", 0.0 },
  { "Analea", 0.0 },
  { "Analeah", 0.0 },
  { "Analecia", 0.0 },
  { "Analee", 0.0 },
  { "Analeece", 0.0 },
  { "Analeesa", 0.0 },
  { "Analeese", 0.0 },
  { "Analei", 0.0 },
  { "Analeia", 0.0 },
  { "Analeiah", 0.0 },
  { "Analeigh", 0.0 },
  { "Analeigha", 0.0 },
  { "Analeise", 0.0 },
  { "Analena", 0.0 },
  { "Analese", 0.0 },
  { "Analeshia", 0.0 },
  { "Analexis", 0.0 },
  { "Analey", 0.0 },
  { "Analeya", 0.0 },
  { "Analeyah", 0.0 },
  { "Anali", 0.0 },
  { "Analia", 0.0 },
  { "Analiah", 0.0 },
  { "Analice", 0.0 },
  { "Analicia", 0.0 },
  { "Analidia", 0.0 },
  { "Analie", 0.0 },
  { "Analiegh", 0.0 },
  { "Analiesa", 0.0 },
  { "Analiese", 0.0 },
  { "Analiha", 0.0 },
  { "Analilia", 0.0 },
  { "Analily", 0.0 },
  { "Analin", 0.0 },
  { "Analina", 0.0 },
  { "Analis", 0.0 },
  { "Analisa", 0.0 },
  { "Analise", 0.0 },
  { "Analisha", 0.0 },
  { "Analisia", 0.0 },
  { "Analissa", 0.0 },
  { "Analisse", 0.0 },
  { "Analiya", 0.0 },
  { "Analiyah", 0.0 },
  { "Analiz", 0.0 },
  { "Analiza", 0.0 },
  { "Analize", 0.0 },
  { "Analleli", 0.0 },
  { "Anallely", 0.0 },
  { "Anally", 0.0 },
  { "Analu", 1.0 },
  { "Analucia", 0.0 },
  { "Analuisa", 0.0 },
  { "Analuiza", 0.0 },
  { "Analuz", 0.0 },
  { "Analy", 0.0 },
  { "Analya", 0.0 },
  { "Analyah", 0.0 },
  { "Analyce", 0.0 },
  { "Analycia", 0.0 },
  { "Analydia", 0.0 },
  { "Analyn", 0.0 },
  { "Analynn", 0.0 },
  { "Analys", 0.0 },
  { "Analysa", 0.0 },
  { "Analyse", 0.0 },
  { "Analysia", 0.0 },
  { "Analyss", 0.0 },
  { "Analyssa", 0.0 },
  { "Analysse", 0.0 },
  { "Analyssia", 0.0 },
  { "Analyz", 0.0 },
  { "Anam", 0.0 },
  { "Anamae", 0.0 },
  { "Anamar", 0.0 },
  { "Anamari", 0.0 },
  { "Anamaria", 0.0 },
  { "Anamarie", 0.0 },
  { "Anamaris", 0.0 },
  { "Anamary", 0.0 },
  { "Anamda", 0.0 },
  { "Anamia", 0.0 },
  { "Anamika", 0.0 },
  { "Anamile", 0.0 },
  { "Anamta", 0.0 },
  { "Anan", 0.92537314 },
  { "Anana", 0.0 },
  { "Anand", 1.0 },
  { "Ananda", 0.05215282 },
  { "Anandi", 0.0 },
  { "Anandita", 0.0 },
  { "Anani", 0.0 },
  { "Anania", 1.0 },
  { "Ananiah", 0.0 },
  { "Ananias", 1.0 },
  { "Anannya", 0.0 },
  { "Anansa", 0.0 },
  { "Ananshi", 0.0 },
  { "Anant", 1.0 },
  { "Ananth", 1.0 },
  { "Ananya", 0.0 },
  { "Ananyaa", 0.0 },
  { "Anapaola", 0.0 },
  { "Anapatricia", 0.0 },
  { "Anapaula", 0.0 },
  { "Anar", 1.0 },
  { "Anara", 0.0 },
  { "Anarae", 0.0 },
  { "Anarah", 0.0 },
  { "Anareli", 0.0 },
  { "Anarely", 0.0 },
  { "Anari", 0.043557167 },
  { "Anaria", 0.0 },
  { "Anariah", 0.0 },
  { "Anarie", 0.0 },
  { "Anariya", 0.0 },
  { "Anariyah", 0.0 },
  { "Anarosa", 0.0 },
  { "Anarose", 0.0 },
  { "Anaruth", 0.0 },
  { "Anas", 1.0 },
  { "Anasa", 0.0 },
  { "Anasha", 0.0 },
  { "Anashe", 0.0 },
  { "Anashia", 0.0 },
  { "Anasia", 0.0 },
  { "Anasofia", 0.0 },
  { "Anasol", 0.0 },
  { "Anasophia", 0.0 },
  { "Anass", 1.0 },
  { "Anassa", 0.0 },
  { "Anastacia", 0.0 },
  { "Anastacio", 1.0 },
  { "Anastaisa", 0.0 },
  { "Anastaisha", 0.0 },
  { "Anastaja", 0.0 },
  { "Anastajia", 0.0 },
  { "Anastasha", 0.0 },
  { "Anastashia", 0.0 },
  { "Anastasi", 0.0 },
  { "Anastasia", 0.0005630906 },
  { "Anastasija", 0.0 },
  { "Anastasio", 1.0 },
  { "Anastasios", 1.0 },
  { "Anastasiya", 0.0 },
  { "Anastassia", 0.0 },
  { "Anastasya", 0.0 },
  { "Anastatia", 0.0 },
  { "Anastaysia", 0.0 },
  { "Anastazia", 0.0 },
  { "Anastazja", 0.0 },
  { "Anastin", 0.0 },
  { "Anaston", 0.0 },
  { "Anastyn", 0.0 },
  { "Anat", 0.0 },
  { "Anatalia", 0.0 },
  { "Anatasia", 0.0 },
  { "Anathea", 0.0 },
  { "Anatol", 1.0 },
  { "Anatole", 1.0 },
  { "Anatoli", 1.0 },
  { "Anatolia", 0.0 },
  { "Anatoliy", 1.0 },
  { "Anatoly", 1.0 },
  { "Anaum", 0.0 },
  { "Anauri", 0.0 },
  { "Anautica", 0.0 },
  { "Anav", 1.0 },
  { "Anavae", 0.0 },
  { "Anavaeh", 0.0 },
  { "Anavay", 0.0 },
  { "Anaveah", 0.0 },
  { "Anavel", 0.0 },
  { "Anavela", 0.0 },
  { "Anavey", 0.0 },
  { "Anavi", 0.0 },
  { "Anaviah", 0.0 },
  { "Anavictoria", 0.0 },
  { "Anay", 0.5080697 },
  { "Anaya", 0.0 },
  { "Anayaa", 0.0 },
  { "Anayah", 0.0 },
  { "Anayalee", 0.0 },
  { "Anayanci", 0.0 },
  { "Anayancy", 0.0 },
  { "Anayansi", 0.0 },
  { "Anayeli", 0.0 },
  { "Anayelis", 0.0 },
  { "Anayelli", 0.0 },
  { "Anayelly", 0.0 },
  { "Anayely", 0.0 },
  { "Anayha", 0.0 },
  { "Anayi", 0.0 },
  { "Anayia", 0.0 },
  { "Anayiah", 0.0 },
  { "Anayis", 0.0 },
  { "Anayjah", 0.0 },
  { "Anayla", 0.0 },
  { "Anaylah", 0.0 },
  { "Anayra", 0.0 },
  { "Anays", 0.0 },
  { "Anaysa", 0.0 },
  { "Anaysha", 0.0 },
  { "Anayshia", 0.0 },
  { "Anaysia", 0.0 },
  { "Anayssa", 0.0 },
  { "Anazia", 0.0 },
  { "Anber", 0.0 },
  { "Anberlin", 0.0 },
  { "Anberlyn", 0.0 },
  { "Anberlynn", 0.0 },
  { "Anbria", 0.0 },
  { "Ance", 1.0 },
  { "Ancel", 1.0 },
  { "Ancelmo", 1.0 },
  { "Anchal", 0.0 },
  { "Anchita", 0.0 },
  { "Anchor", 1.0 },
  { "Ancika", 0.0 },
  { "Ancil", 1.0 },
  { "Ancle", 1.0 },
  { "Ancy", 0.0 },
  { "Anda", 0.0 },
  { "Andaiye", 0.0 },
  { "Andalasia", 0.0 },
  { "Andalucia", 0.0 },
  { "Andalyn", 0.0 },
  { "Andalynn", 0.0 },
  { "Andamo", 1.0 },
  { "Andan", 1.0 },
  { "Andarius", 1.0 },
  { "Anddy", 1.0 },
  { "Ande", 0.1764706 },
  { "Andea", 0.0 },
  { "Andee", 0.005144033 },
  { "Andel", 1.0 },
  { "Andela", 0.0 },
  { "Andelin", 0.0 },
  { "Andella", 0.0 },
  { "Andelyn", 0.0 },
  { "Andelynn", 0.0 },
  { "Anden", 1.0 },
  { "Ander", 1.0 },
  { "Andera", 0.023904383 },
  { "Anderea", 0.0 },
  { "Anderew", 1.0 },
  { "Anderia", 0.0 },
  { "Anders", 1.0 },
  { "Andersen", 0.7838785 },
  { "Anderson", 0.9714872 },
  { "Andersson", 1.0 },
  { "Andersyn", 0.0 },
  { "Anderw", 1.0 },
  { "Anderzon", 1.0 },
  { "Andes", 1.0 },
  { "Andew", 1.0 },
  { "Andhy", 1.0 },
  { "Andi", 0.15129073 },
  { "Andia", 0.0 },
  { "Andice", 0.0 },
  { "Andie", 0.058857143 },
  { "Andiel", 1.0 },
  { "Andilyn", 0.0 },
  { "Andilynn", 0.0 },
  { "Andin", 0.9137931 },
  { "Andino", 1.0 },
  { "Andis", 1.0 },
  { "Andjela", 0.0 },
  { "Andjoua", 0.0 },
  { "Andols", 1.0 },
  { "Andon", 1.0 },
  { "Andoni", 1.0 },
  { "Andonia", 0.0 },
  { "Andony", 1.0 },
  { "Andora", 0.0 },
  { "Andra", 0.32053837 },
  { "Andrade", 1.0 },
  { "Andrae", 0.97998047 },
  { "Andraea", 0.0 },
  { "Andraia", 0.0 },
  { "Andralyn", 0.0 },
  { "Andranae", 0.0 },
  { "Andranette", 0.0 },
  { "Andranik", 1.0 },
  { "Andranique", 0.0 },
  { "Andras", 1.0 },
  { "Andray", 1.0 },
  { "Andraya", 0.0 },
  { "Andre", 0.9820701 },
  { "Andrea", 0.013426401 },
  { "Andreah", 0.0 },
  { "Andreal", 0.0 },
  { "Andreamarie", 0.0 },
  { "Andrean", 0.2 },
  { "Andreana", 0.0 },
  { "Andreanna", 0.0 },
  { "Andreas", 0.99317265 },
  { "Andreau", 1.0 },
  { "Andreaus", 1.0 },
  { "Andreaz", 1.0 },
  { "Andreca", 0.0 },
  { "Andreco", 1.0 },
  { "Andree", 0.31968948 },
  { "Andreea", 0.0 },
  { "Andreena", 0.0 },
  { "Andrei", 1.0 },
  { "Andreia", 0.0 },
  { "Andreika", 0.0 },
  { "Andreina", 0.0049792533 },
  { "Andreis", 1.0 },
  { "Andreita", 0.0 },
  { "Andrej", 1.0 },
  { "Andrejs", 1.0 },
  { "Andrek", 1.0 },
  { "Andreka", 0.0 },
  { "Andrekia", 0.0 },
  { "Andrel", 0.8648649 },
  { "Andrell", 0.6988764 },
  { "Andrella", 0.0 },
  { "Andrelle", 0.0 },
  { "Andren", 1.0 },
  { "Andrena", 0.0 },
  { "Andrenae", 0.0 },
  { "Andrene", 0.0 },
  { "Andrenette", 0.0 },
  { "Andrenika", 0.0 },
  { "Andreniki", 0.0 },
  { "Andrenique", 0.0 },
  { "Andreo", 1.0 },
  { "Andreona", 0.0 },
  { "Andreonna", 0.0 },
  { "Andreous", 1.0 },
  { "Andreq", 1.0 },
  { "Andres", 0.99450386 },
  { "Andresha", 0.0 },
  { "Andress", 1.0 },
  { "Andressa", 0.0 },
  { "Andretta", 0.0 },
  { "Andretti", 1.0 },
  { "Andreu", 1.0 },
  { "Andreus", 1.0 },
  { "Andrew", 0.9961935 },
  { "Andrewjacob", 1.0 },
  { "Andrewjames", 1.0 },
  { "Andrewjohn", 1.0 },
  { "Andrewjoseph", 1.0 },
  { "Andrewmichael", 1.0 },
  { "Andrews", 1.0 },
  { "Andrey", 0.9562555 },
  { "Andreya", 0.0 },
  { "Andreyah", 0.0 },
  { "Andreyna", 0.0 },
  { "Andrez", 1.0 },
  { "Andri", 0.95689654 },
  { "Andria", 0.0036359504 },
  { "Andriah", 0.0 },
  { "Andrian", 0.8509317 },
  { "Andriana", 0.0 },
  { "Andrianna", 0.0 },
  { "Andrianne", 0.0 },
  { "Andric", 1.0 },
  { "Andrica", 0.0 },
  { "Andrick", 1.0 },
  { "Andricka", 0.0 },
  { "Andrico", 1.0 },
  { "Andrie", 1.0 },
  { "Andriea", 0.0 },
  { "Andrieka", 0.0 },
  { "Andriel", 0.69902915 },
  { "Andrielle", 0.0 },
  { "Andriena", 0.0 },
  { "Andrienna", 0.0 },
  { "Andrienne", 0.0 },
  { "Andrietta", 0.0 },
  { "Andriette", 0.0 },
  { "Andrijana", 0.0 },
  { "Andrik", 1.0 },
  { "Andrika", 0.0 },
  { "Andrina", 0.0 },
  { "Andrine", 0.0 },
  { "Andris", 1.0 },
  { "Andrius", 1.0 },
  { "Andriy", 1.0 },
  { "Andro", 1.0 },
  { "Andromeda", 0.0 },
  { "Andron", 1.0 },
  { "Androniki", 0.0 },
  { "Andropolis", 1.0 },
  { "Andros", 1.0 },
  { "Androw", 1.0 },
  { "Andru", 1.0 },
  { "Andrue", 1.0 },
  { "Andrus", 1.0 },
  { "Andruw", 1.0 },
  { "Andrw", 1.0 },
  { "Andry", 0.9770642 },
  { "Andrya", 0.0 },
  { "Andrzej", 1.0 },
  { "Anduin", 1.0 },
  { "Andwele", 1.0 },
  { "Andy", 0.9886199 },
  { "Andyn", 1.0 },
  { "Ane", 0.0 },
  { "Anea", 0.0 },
  { "Aneah", 0.0 },
  { "Aneas", 1.0 },
  { "Anease", 0.0 },
  { "Aneatra", 0.0 },
  { "Anecia", 0.0 },
  { "Aneda", 0.0 },
  { "Anedra", 0.0 },
  { "Anee", 0.0 },
  { "Aneea", 0.0 },
  { "Aneecia", 0.0 },
  { "Aneek", 1.0 },
  { "Aneeka", 0.0 },
  { "Aneeksha", 0.0 },
  { "Aneel", 1.0 },
  { "Aneela", 0.0 },
  { "Aneeq", 1.0 },
  { "Aneeqa", 0.0 },
  { "Anees", 1.0 },
  { "Aneesa", 0.0 },
  { "Aneesah", 0.0 },
  { "Aneesh", 1.0 },
  { "Aneesha", 0.0 },
  { "Aneesia", 0.0 },
  { "Aneeya", 0.0 },
  { "Aneeza", 0.0 },
  { "Anegla", 0.0 },
  { "Aneia", 0.0 },
  { "Aneicia", 0.0 },
  { "Aneida", 0.0 },
  { "Aneika", 0.0 },
  { "Aneil", 1.0 },
  { "Aneila", 0.0 },
  { "Aneira", 0.0 },
  { "Aneisa", 0.0 },
  { "Aneisha", 0.0 },
  { "Aneissa", 0.0 },
  { "Aneita", 0.0 },
  { "Aneitra", 0.0 },
  { "Aneiya", 0.0 },
  { "Aneiyah", 0.0 },
  { "Aneka", 0.0 },
  { "Anel", 0.12580486 },
  { "Anela", 0.0 },
  { "Anelah", 0.0 },
  { "Anele", 0.0 },
  { "Aneley", 0.0 },
  { "Aneli", 0.0 },
  { "Anelia", 0.0 },
  { "Anelie", 0.0 },
  { "Aneliese", 0.0 },
  { "Anelis", 0.0 },
  { "Anelisa", 0.0 },
  { "Anelise", 0.0 },
  { "Anelisse", 0.0 },
  { "Aneliz", 0.0 },
  { "Anell", 0.0 },
  { "Anella", 0.0 },
  { "Anelle", 0.0 },
  { "Anello", 1.0 },
  { "Anelly", 0.0 },
  { "Anely", 0.0 },
  { "Anelys", 0.0 },
  { "Anelyse", 0.0 },
  { "Anemone", 0.0 },
  { "Aner", 0.5882353 },
  { "Aneres", 0.0 },
  { "Aneri", 0.0 },
  { "Aneria", 0.0 },
  { "Aneris", 0.0 },
  { "Anes", 1.0 },
  { "Anesa", 0.0 },
  { "Anesha", 0.0 },
  { "Aneshia", 0.0 },
  { "Anesia", 0.0 },
  { "Anessa", 0.0 },
  { "Anessah", 0.0 },
  { "Anessia", 0.0 },
  { "Anesti", 0.5 },
  { "Anesty", 0.0 },
  { "Anet", 0.0 },
  { "Aneta", 0.0 },
  { "Aneth", 0.0 },
  { "Anetha", 0.0 },
  { "Anetia", 0.0 },
  { "Anetra", 0.0 },
  { "Anetria", 0.0 },
  { "Anett", 0.0 },
  { "Anetta", 0.0 },
  { "Anette", 0.0 },
  { "Aneudi", 1.0 },
  { "Aneudy", 1.0 },
  { "Aneva", 0.0 },
  { "Anevaeh", 0.0 },
  { "Anevay", 0.0 },
  { "Aneya", 0.0 },
  { "Aneyah", 0.0 },
  { "Aneyda", 0.0 },
  { "Aneysa", 0.0 },
  { "Aneysha", 0.0 },
  { "Anezka", 0.0 },
  { "Anfal", 0.0 },
  { "Anferne", 1.0 },
  { "Anfernee", 1.0 },
  { "Anferney", 1.0 },
  { "Anferny", 1.0 },
  { "Anfisa", 0.0 },
  { "Anfrenee", 1.0 },
  { "Ang", 1.0 },
  { "Anga", 0.0 },
  { "Angad", 1.0 },
  { "Angala", 0.0 },
  { "Angalee", 0.0 },
  { "Angalena", 0.0 },
  { "Angalina", 0.0 },
  { "Anganette", 0.0 },
  { "Ange", 0.02849003 },
  { "Angeal", 1.0 },
  { "Angee", 0.0 },
  { "Angel", 0.7022274 },
  { "Angela", 0.0032146599 },
  { "Angelah", 0.0 },
  { "Angelamae", 0.0 },
  { "Angelamaria", 0.0 },
  { "Angelamarie", 0.0 },
  { "Angelann", 0.0 },
  { "Angelar", 0.0 },
  { "Angeldaniel", 1.0 },
  { "Angeldavid", 1.0 },
  { "Angeldejesus", 1.0 },
  { "Angele", 0.0 },
  { "Angelea", 0.0 },
  { "Angeleah", 0.0 },
  { "Angelean", 0.0 },
  { "Angeleana", 0.0 },
  { "Angeleca", 0.0 },
  { "Angelee", 0.0 },
  { "Angeleen", 0.0 },
  { "Angeleena", 0.0 },
  { "Angeleia", 0.0 },
  { "Angeleigh", 0.0 },
  { "Angelena", 0.0 },
  { "Angelene", 0.0 },
  { "Angelenia", 0.0 },
  { "Angeleque", 0.0 },
  { "Angeles", 0.020907044 },
  { "Angelese", 0.0 },
  { "Angeleta", 0.0 },
  { "Angelete", 0.0 },
  { "Angeletta", 0.0 },
  { "Angelette", 0.0 },
  { "Angelgabriel", 1.0 },
  { "Angeli", 0.0 },
  { "Angelia", 0.0009934971 },
  { "Angeliah", 0.0 },
  { "Angelian", 0.0 },
  { "Angeliana", 0.0 },
  { "Angelic", 0.0 },
  { "Angelica", 0.0036063364 },
  { "Angelicamae", 0.0 },
  { "Angelicamaria", 0.0 },
  { "Angelicamarie", 0.0 },
  { "Angelice", 0.0 },
  { "Angelicia", 0.0 },
  { "Angelick", 0.0 },
  { "Angelicque", 0.0 },
  { "Angelie", 0.0 },
  { "Angeliese", 0.0 },
  { "Angeligue", 0.0 },
  { "Angelik", 0.0 },
  { "Angelika", 0.0 },
  { "Angelike", 0.0 },
  { "Angeliki", 0.0 },
  { "Angelin", 0.0 },
  { "Angelina", 0.00095607236 },
  { "Angelinah", 0.0 },
  { "Angelinamarie", 0.0 },
  { "Angeline", 0.0002836477 },
  { "Angelinna", 0.0 },
  { "Angelinne", 0.0 },
  { "Angelino", 1.0 },
  { "Angeliq", 0.0 },
  { "Angeliqu", 0.0 },
  { "Angeliqua", 0.0 },
  { "Angelique", 0.00021581957 },
  { "Angelis", 0.07322176 },
  { "Angelisa", 0.0 },
  { "Angelise", 0.0 },
  { "Angelisha", 0.0 },
  { "Angelissa", 0.0 },
  { "Angelisse", 0.0 },
  { "Angelita", 0.0032101336 },
  { "Angelite", 0.0 },
  { "Angelith", 0.0 },
  { "Angelito", 1.0 },
  { "Angeliya", 0.0 },
  { "Angeliyah", 0.0 },
  { "Angeliz", 0.0 },
  { "Angeliza", 0.0 },
  { "Angelize", 0.0 },
  { "Angeljesus", 1.0 },
  { "Angell", 0.22628951 },
  { "Angella", 0.0 },
  { "Angelle", 0.0 },
  { "Angellea", 0.0 },
  { "Angelleah", 0.0 },
  { "Angellee", 0.0 },
  { "Angelleigh", 0.0 },
  { "Angellena", 0.0 },
  { "Angelli", 0.0 },
  { "Angellia", 0.0 },
  { "Angellica", 0.0 },
  { "Angellie", 0.0 },
  { "Angellina", 0.0 },
  { "Angelline", 0.0 },
  { "Angellique", 0.0 },
  { "Angello", 1.0 },
  { "Angelluis", 1.0 },
  { "Angelly", 0.0 },
  { "Angellyn", 0.0 },
  { "Angellyna", 0.0 },
  { "Angellynn", 0.0 },
  { "Angelmanuel", 1.0 },
  { "Angelmarie", 0.0 },
  { "Angelmiguel", 1.0 },
  { "Angelo", 0.9889882 },
  { "Angelos", 1.0 },
  { "Angelou", 1.0 },
  { "Angelous", 1.0 },
  { "Angelque", 0.0 },
  { "Angelray", 1.0 },
  { "Angelrose", 0.0 },
  { "Angeluis", 1.0 },
  { "Angelus", 0.9251497 },
  { "Angely", 0.0 },
  { "Angelyca", 0.0 },
  { "Angelyce", 0.0 },
  { "Angelyka", 0.0 },
  { "Angelyn", 0.0 },
  { "Angelyna", 0.0 },
  { "Angelyne", 0.0 },
  { "Angelynn", 0.0 },
  { "Angelynna", 0.0 },
  { "Angelynne", 0.0 },
  { "Angelys", 0.0 },
  { "Angelyse", 0.0 },
  { "Angelyssa", 0.0 },
  { "Angelysse", 0.0 },
  { "Angenetta", 0.0 },
  { "Angenette", 0.0 },
  { "Angeni", 0.0 },
  { "Angenie", 0.0 },
  { "Angenique", 0.0 },
  { "Angenita", 0.0 },
  { "Anges", 0.0 },
  { "Anggie", 0.0 },
  { "Angharad", 0.0 },
  { "Anghel", 1.0 },
  { "Angi", 0.0 },
  { "Angia", 0.0 },
  { "Angie", 0.0021134145 },
  { "Angiela", 0.0 },
  { "Angielina", 0.0 },
  { "Angil", 0.0 },
  { "Angila", 0.0 },
  { "Angilee", 0.0 },
  { "Angilina", 0.0 },
  { "Angilique", 0.0 },
  { "Anginette", 0.0 },
  { "Angiolina", 0.0 },
  { "Angla", 0.0 },
  { "Angle", 0.09811617 },
  { "Anglea", 0.0 },
  { "Anglee", 0.0 },
  { "Angleica", 0.0 },
  { "Anglene", 0.0 },
  { "Angleo", 1.0 },
  { "Angler", 1.0 },
  { "Angles", 0.0 },
  { "Anglia", 0.0 },
  { "Anglica", 0.0 },
  { "Anglie", 0.0 },
  { "Anglina", 0.0 },
  { "Angline", 0.0 },
  { "Anglique", 0.0 },
  { "Anglo", 1.0 },
  { "Anguel", 1.0 },
  { "Angula", 0.0 },
  { "Angus", 1.0 },
  { "Angy", 0.0 },
  { "Angye", 0.0 },
  { "Angyl", 0.0 },
  { "Anh", 0.30879447 },
  { "Anhad", 1.0 },
  { "Anhar", 0.0 },
  { "Anhelica", 0.0 },
  { "Anhthu", 0.0 },
  { "Anhtony", 1.0 },
  { "Anhtuan", 1.0 },
  { "Anhuar", 1.0 },
  { "Ani", 0.0019762847 },
  { "Ania", 0.0 },
  { "Aniah", 0.0 },
  { "Aniaha", 0.0 },
  { "Anias", 0.88 },
  { "Aniaya", 0.0 },
  { "Aniayah", 0.0 },
  { "Anibal", 0.9939176 },
  { "Anibella", 0.0 },
  { "Anibelle", 0.0 },
  { "Anica", 0.0 },
  { "Anice", 0.0 },
  { "Aniceta", 0.0 },
  { "Aniceto", 1.0 },
  { "Anicia", 0.0 },
  { "Anicka", 0.0 },
  { "Anida", 0.0 },
  { "Anie", 0.0 },
  { "Aniece", 0.0 },
  { "Aniecia", 0.0 },
  { "Aniel", 1.0 },
  { "Aniela", 0.0 },
  { "Anielka", 0.0 },
  { "Aniella", 0.0 },
  { "Anielle", 0.0 },
  { "Aniello", 1.0 },
  { "Aniesa", 0.0 },
  { "Aniesha", 0.0 },
  { "Aniessa", 0.0 },
  { "Anieta", 0.0 },
  { "Anietra", 0.0 },
  { "Anieya", 0.0 },
  { "Anieyah", 0.0 },
  { "Aniha", 0.0 },
  { "Anihya", 0.0 },
  { "Aniiya", 0.0 },
  { "Aniiyah", 0.0 },
  { "Anij", 0.0 },
  { "Anija", 0.0 },
  { "Anijae", 0.0 },
  { "Anijah", 0.0 },
  { "Anijha", 0.0 },
  { "Anijia", 0.0 },
  { "Anik", 0.9346154 },
  { "Anika", 0.0 },
  { "Anikah", 0.0 },
  { "Anikait", 1.0 },
  { "Anikan", 1.0 },
  { "Anike", 0.0 },
  { "Aniken", 1.0 },
  { "Aniket", 1.0 },
  { "Aniketh", 1.0 },
  { "Aniki", 1.0 },
  { "Anikia", 0.0 },
  { "Anikin", 1.0 },
  { "Anikka", 0.0 },
  { "Aniko", 0.0 },
  { "Anikyn", 1.0 },
  { "Anil", 1.0 },
  { "Anila", 0.0 },
  { "Anilah", 0.0 },
  { "Anilu", 0.0 },
  { "Anilya", 0.0 },
  { "Anilyah", 0.0 },
  { "Anilyn", 0.0 },
  { "Anime", 0.0 },
  { "Animesh", 1.0 },
  { "Anina", 0.0 },
  { "Anindita", 0.0 },
  { "Aniq", 1.0 },
  { "Aniqa", 0.0 },
  { "Aniqua", 0.0 },
  { "Anique", 0.0 },
  { "Anir", 1.0 },
  { "Anira", 0.0 },
  { "Anirah", 0.0 },
  { "Anirud", 1.0 },
  { "Aniruddh", 1.0 },
  { "Aniruddha", 1.0 },
  { "Anirudh", 1.0 },
  { "Anirvin", 1.0 },
  { "Anis", 0.79041916 },
  { "Anisa", 0.0 },
  { "Anisah", 0.0 },
  { "Anise", 0.0 },
  { "Aniseto", 1.0 },
  { "Anisette", 0.0 },
  { "Anish", 1.0 },
  { "Anisha", 0.0 },
  { "Anishia", 0.0 },
  { "Anishka", 0.0 },
  { "Anisia", 0.0 },
  { "Anisjon", 1.0 },
  { "Anisley", 0.0 },
  { "Anissa", 0.00042480882 },
  { "Anissah", 0.0 },
  { "Anisse", 0.0 },
  { "Anissia", 0.0 },
  { "Anistasia", 0.0 },
  { "Anisten", 0.0 },
  { "Anistin", 0.0 },
  { "Aniston", 0.010439561 },
  { "Anisty", 0.0 },
  { "Anistyn", 0.0 },
  { "Anistynn", 0.0 },
  { "Anita", 0.0031913107 },
  { "Anite", 0.0 },
  { "Anitha", 0.0 },
  { "Anitia", 0.0 },
  { "Anitra", 0.0 },
  { "Anitrea", 0.0 },
  { "Anitria", 0.0 },
  { "Anitrice", 0.0 },
  { "Anitta", 0.0 },
  { "Anitza", 0.0 },
  { "Aniv", 1.0 },
  { "Aniva", 0.0 },
  { "Anival", 1.0 },
  { "Anivea", 0.0 },
  { "Anivia", 0.0 },
  { "Aniya", 0.0 },
  { "Aniyah", 0.0002193656 },
  { "Aniyaha", 0.0 },
  { "Aniyahlee", 0.0 },
  { "Aniyan", 0.0 },
  { "Aniye", 0.0 },
  { "Aniyha", 0.0 },
  { "Aniyia", 0.0 },
  { "Aniyiah", 0.0 },
  { "Aniyja", 0.0 },
  { "Aniyjah", 0.0 },
  { "Aniyla", 0.0 },
  { "Aniylah", 0.0 },
  { "Aniysa", 0.0 },
  { "Aniyyah", 0.0 },
  { "Aniza", 0.0 },
  { "Anja", 0.0 },
  { "Anjae", 0.0 },
  { "Anjail", 0.0 },
  { "Anjalee", 0.0 },
  { "Anjalena", 0.0 },
  { "Anjali", 0.0 },
  { "Anjalie", 0.0 },
  { "Anjalika", 0.0 },
  { "Anjalina", 0.0 },
  { "Anjaliyah", 0.0 },
  { "Anjaly", 0.0 },
  { "Anjan", 1.0 },
  { "Anjana", 0.0 },
  { "Anjanae", 0.0 },
  { "Anjanet", 0.0 },
  { "Anjanett", 0.0 },
  { "Anjanetta", 0.0 },
  { "Anjanette", 0.0 },
  { "Anjani", 0.0 },
  { "Anjanie", 0.0 },
  { "Anjanique", 0.0 },
  { "Anjannette", 0.0 },
  { "Anjay", 1.0 },
  { "Anjeanette", 0.0 },
  { "Anjel", 0.73150104 },
  { "Anjela", 0.0 },
  { "Anjelah", 0.0 },
  { "Anjeli", 0.0 },
  { "Anjelia", 0.0 },
  { "Anjelica", 0.0 },
  { "Anjelicia", 0.0 },
  { "Anjeliett", 0.0 },
  { "Anjelika", 0.0 },
  { "Anjelina", 0.0 },
  { "Anjelique", 0.0 },
  { "Anjelita", 0.0 },
  { "Anjellica", 0.0 },
  { "Anjelo", 1.0 },
  { "Anjely", 0.0 },
  { "Anjelyka", 0.0 },
  { "Anjelyn", 0.0 },
  { "Anjelyna", 0.0 },
  { "Anjenette", 0.0 },
  { "Anjennette", 0.0 },
  { "Anji", 0.0 },
  { "Anjie", 0.0 },
  { "Anjli", 0.0 },
  { "Anjola", 0.0 },
  { "Anjolaoluwa", 0.057894737 },
  { "Anjolee", 0.0 },
  { "Anjolena", 0.0 },
  { "Anjoli", 0.0 },
  { "Anjolie", 0.0 },
  { "Anjolina", 0.0 },
  { "Anjolique", 0.0 },
  { "Anjonette", 0.0 },
  { "Anjrue", 1.0 },
  { "Anju", 0.0 },
  { "Anjuan", 1.0 },
  { "Anjuli", 0.0 },
  { "Anjulie", 0.0 },
  { "Anjum", 0.0 },
  { "Anka", 0.0 },
  { "Anke", 0.0 },
  { "Anker", 1.0 },
  { "Ankit", 1.0 },
  { "Ankita", 0.0 },
  { "Ankith", 1.0 },
  { "Ankitha", 0.0 },
  { "Ankoma", 1.0 },
  { "Ankoor", 1.0 },
  { "Ankur", 1.0 },
  { "Ankush", 1.0 },
  { "Anleigh", 0.0 },
  { "Anley", 0.0 },
  { "Anlin", 0.0 },
  { "Anllely", 0.0 },
  { "Anlly", 0.0 },
  { "Anly", 0.0 },
  { "Anmar", 1.0 },
  { "Anmarie", 0.0 },
  { "Anmay", 1.0 },
  { "Anmol", 0.5727003 },
  { "Ann", 0.0029076021 },
  { "Anna", 0.003081386 },
  { "Annaalicia", 0.0 },
  { "Annabel", 0.0 },
  { "Annabela", 0.0 },
  { "Annabele", 0.0 },
  { "Annabell", 0.0 },
  { "Annabella", 0.0 },
  { "Annabellah", 0.0 },
  { "Annabelle", 0.0001819472 },
  { "Annabellee", 0.0 },
  { "Annabellelee", 0.0 },
  { "Annabeth", 0.0 },
  { "Annacarolina", 0.0 },
  { "Annacatherine", 0.0 },
  { "Annachristina", 0.0 },
  { "Annaclaire", 0.0 },
  { "Annaclara", 0.0 },
  { "Annadell", 0.0 },
  { "Annaelise", 0.0 },
  { "Annaelizabeth", 0.0 },
  { "Annaelle", 0.0 },
  { "Annagail", 0.0 },
  { "Annagene", 0.0 },
  { "Annagrace", 0.0 },
  { "Annah", 0.0 },
  { "Annahbella", 0.0 },
  { "Annahi", 0.0 },
  { "Annahy", 0.0 },
  { "Annai", 0.0 },
  { "Annaie", 0.0 },
  { "Annaira", 0.0 },
  { "Annais", 0.0 },
  { "Annaise", 0.0 },
  { "Annajane", 0.0 },
  { "Annajean", 0.0 },
  { "Annajo", 0.0 },
  { "Annajoy", 0.0 },
  { "Annajulia", 0.0 },
  { "Annaka", 0.0 },
  { "Annakaren", 0.0 },
  { "Annakate", 0.0 },
  { "Annakatherine", 0.0 },
  { "Annakathryn", 0.0 },
  { "Annakay", 0.0 },
  { "Annakiya", 0.0 },
  { "Annalaura", 0.0 },
  { "Annalaya", 0.0 },
  { "Annalayah", 0.0 },
  { "Annalea", 0.0 },
  { "Annaleah", 0.0 },
  { "Annalease", 0.0 },
  { "Annalecia", 0.0 },
  { "Annalee", 0.0 },
  { "Annaleece", 0.0 },
  { "Annaleese", 0.0 },
  { "Annalei", 0.0 },
  { "Annaleia", 0.0 },
  { "Annaleigh", 0.0 },
  { "Annaleigha", 0.0 },
  { "Annaleisa", 0.0 },
  { "Annaleise", 0.0 },
  { "Annalena", 0.0 },
  { "Annalene", 0.0 },
  { "Annalese", 0.0 },
  { "Annalesia", 0.0 },
  { "Annaley", 0.0 },
  { "Annaleya", 0.0 },
  { "Annali", 0.0 },
  { "Annalia", 0.0 },
  { "Annaliah", 0.0 },
  { "Annalice", 0.0 },
  { "Annalicia", 0.0 },
  { "Annalie", 0.0 },
  { "Annaliece", 0.0 },
  { "Annaliesa", 0.0 },
  { "Annaliese", 0.0 },
  { "Annaliesse", 0.0 },
  { "Annaliisa", 0.0 },
  { "Annalin", 0.0 },
  { "Annalina", 0.0 },
  { "Annaline", 0.0 },
  { "Annalis", 0.0 },
  { "Annalisa", 0.0 },
  { "Annalise", 0.0 },
  { "Annalisha", 0.0 },
  { "Annalisia", 0.0 },
  { "Annalissa", 0.0 },
  { "Annalisse", 0.0 },
  { "Annaliya", 0.0 },
  { "Annaliyah", 0.0 },
  { "Annaliz", 0.0 },
  { "Annaliza", 0.0 },
  { "Annalize", 0.0 },
  { "Annalou", 0.0 },
  { "Annalouise", 0.0 },
  { "Annalucia", 0.0 },
  { "Annaluisa", 0.0 },
  { "Annaluiza", 0.0 },
  { "Annaly", 0.0 },
  { "Annalya", 0.0 },
  { "Annalyce", 0.0 },
  { "Annalycia", 0.0 },
  { "Annalyn", 0.0 },
  { "Annalyne", 0.0 },
  { "Annalynn", 0.0 },
  { "Annalynne", 0.0 },
  { "Annalys", 0.0 },
  { "Annalysa", 0.0 },
  { "Annalyse", 0.0 },
  { "Annalysia", 0.0 },
  { "Annalyss", 0.0 },
  { "Annalyssa", 0.0 },
  { "Annalysse", 0.0 },
  { "Annalyssia", 0.0 },
  { "Annam", 0.0 },
  { "Annamae", 0.0 },
  { "Annamari", 0.0 },
  { "Annamaria", 0.0 },
  { "Annamarie", 0.0 },
  { "Annamary", 0.0 },
  { "Annamay", 0.0 },
  { "Annamolly", 0.0 },
  { "Annan", 1.0 },
  { "Annanicole", 0.0 },
  { "Annanya", 0.0 },
  { "Annapaula", 0.0 },
  { "Annapurna", 0.0 },
  { "Annarae", 0.0 },
  { "Annarosa", 0.0 },
  { "Annarose", 0.0 },
  { "Annaruth", 0.0 },
  { "Annas", 0.60714287 },
  { "Annasha", 0.0 },
  { "Annasia", 0.0 },
  { "Annasofia", 0.0 },
  { "Annasophia", 0.0 },
  { "Annastacia", 0.0 },
  { "Annastashia", 0.0 },
  { "Annastasia", 0.0 },
  { "Annastassia", 0.0 },
  { "Annastazia", 0.0 },
  { "Annastin", 0.0 },
  { "Annaston", 0.0 },
  { "Annastyn", 0.0 },
  { "Annavictoria", 0.0 },
  { "Annay", 0.0 },
  { "Annaya", 0.0 },
  { "Annayah", 0.0 },
  { "Annayeli", 0.0 },
  { "Annazette", 0.0 },
  { "Anndee", 0.0 },
  { "Anndi", 0.0 },
  { "Anndrea", 0.0 },
  { "Anndria", 0.0 },
  { "Anndy", 1.0 },
  { "Anne", 0.002579212 },
  { "Annea", 0.0 },
  { "Annease", 0.0 },
  { "Annebelle", 0.0 },
  { "Annecia", 0.0 },
  { "Annecy", 0.0 },
  { "Annee", 0.0 },
  { "Anneelise", 0.0 },
  { "Anneelizabeth", 0.0 },
  { "Anneice", 0.0 },
  { "Anneisha", 0.0 },
  { "Anneka", 0.0 },
  { "Anneke", 0.0 },
  { "Annel", 0.0 },
  { "Annelee", 0.0 },
  { "Anneleise", 0.0 },
  { "Anneli", 0.0 },
  { "Annelie", 0.0 },
  { "Annelies", 0.0 },
  { "Anneliesa", 0.0 },
  { "Anneliese", 0.0 },
  { "Anneliesse", 0.0 },
  { "Annelis", 0.0 },
  { "Annelisa", 0.0 },
  { "Annelise", 0.0 },
  { "Annelisse", 0.0 },
  { "Anneliz", 0.0 },
  { "Annelizabeth", 0.0 },
  { "Annell", 0.0 },
  { "Annella", 0.0 },
  { "Annelle", 0.0 },
  { "Annelouise", 0.0 },
  { "Annely", 0.0 },
  { "Annelyn", 0.0 },
  { "Annelys", 0.0 },
  { "Annelyse", 0.0 },
  { "Annemargaret", 0.0 },
  { "Annemari", 0.0 },
  { "Annemarie", 0.0 },
  { "Annemary", 0.0 },
  { "Anner", 0.011483254 },
  { "Annesa", 0.0 },
  { "Annese", 0.0 },
  { "Annesha", 0.0 },
  { "Anneshia", 0.0 },
  { "Annesia", 0.0 },
  { "Annesley", 0.0 },
  { "Annesophie", 0.0 },
  { "Annessa", 0.0 },
  { "Annessia", 0.0 },
  { "Annet", 0.0 },
  { "Anneta", 0.0 },
  { "Annete", 0.0 },
  { "Anneth", 0.0 },
  { "Annetha", 0.0 },
  { "Annetra", 0.0 },
  { "Annett", 0.0 },
  { "Annetta", 0.0 },
  { "Annette", 0.0029674827 },
  { "Annettee", 0.0 },
  { "Annettia", 0.0 },
  { "Annettie", 0.0 },
  { "Annexie", 0.0 },
  { "Anney", 0.0 },
  { "Anngela", 0.0 },
  { "Anngelica", 0.0 },
  { "Anngelina", 0.0 },
  { "Anngie", 0.0 },
  { "Anni", 0.0 },
  { "Annia", 0.0 },
  { "Anniah", 0.0 },
  { "Annias", 1.0 },
  { "Annibale", 1.0 },
  { "Annibella", 0.0 },
  { "Annibelle", 0.0 },
  { "Annica", 0.0 },
  { "Annice", 0.0 },
  { "Annicia", 0.0 },
  { "Annick", 0.0 },
  { "Annicka", 0.0 },
  { "Annie", 0.0054887 },
  { "Anniebell", 0.0 },
  { "Anniebelle", 0.0 },
  { "Anniece", 0.0 },
  { "Annielaurie", 0.0 },
  { "Annielee", 0.0 },
  { "Annielou", 0.0 },
  { "Anniemae", 0.0 },
  { "Anniemarie", 0.0 },
  { "Annieruth", 0.0 },
  { "Annies", 0.0 },
  { "Anniesha", 0.0 },
  { "Annika", 0.0 },
  { "Annikah", 0.0 },
  { "Annikka", 0.0 },
  { "Annil", 0.0 },
  { "Annilee", 0.0 },
  { "Annily", 0.0 },
  { "Annilyn", 0.0 },
  { "Annina", 0.0 },
  { "Anniqua", 0.0 },
  { "Annique", 0.0 },
  { "Annis", 0.0023964397 },
  { "Annisa", 0.0 },
  { "Annise", 0.0 },
  { "Annisha", 0.0 },
  { "Annison", 0.0 },
  { "Annissa", 0.0 },
  { "Annissia", 0.0 },
  { "Annisten", 0.0 },
  { "Annistin", 0.0 },
  { "Anniston", 0.0 },
  { "Annistyn", 0.0 },
  { "Annita", 0.0 },
  { "Annitra", 0.0 },
  { "Annitta", 0.0 },
  { "Annitte", 0.0 },
  { "Anniya", 0.0 },
  { "Anniyah", 0.0 },
  { "Annjane", 0.0 },
  { "Annjanette", 0.0 },
  { "Annjeanette", 0.0 },
  { "Annjelica", 0.0 },
  { "Annlee", 0.0 },
  { "Annleigh", 0.0 },
  { "Annley", 0.0 },
  { "Annlouise", 0.0 },
  { "Annlynn", 0.0 },
  { "Annmargaret", 0.0 },
  { "Annmari", 0.0 },
  { "Annmaria", 0.0 },
  { "Annmarie", 0.0 },
  { "Annmary", 0.0 },
  { "Annon", 1.0 },
  { "Annora", 0.0 },
  { "Annorah", 0.0 },
  { "Annslee", 0.0 },
  { "Annsleigh", 0.0 },
  { "Annsley", 0.0 },
  { "Annslie", 0.0 },
  { "Anntionette", 0.0 },
  { "Anntoinette", 0.0 },
  { "Anntonette", 0.0 },
  { "Annum", 0.0 },
  { "Annunciata", 0.0 },
  { "Annunziata", 0.0 },
  { "Annunzio", 1.0 },
  { "Anny", 0.0 },
  { "Annya", 0.0 },
  { "Annyah", 0.0 },
  { "Annye", 0.0 },
  { "Annyka", 0.0 },
  { "Annysa", 0.0 },
  { "Annyssa", 0.0 },
  { "Annyston", 0.0 },
  { "Annzlee", 0.0 },
  { "Annzley", 0.0 },
  { "Anokhi", 0.0 },
  { "Anoki", 1.0 },
  { "Anola", 0.0 },
  { "Anon", 1.0 },
  { "Anona", 0.0 },
  { "Anonda", 0.0 },
  { "Anooj", 1.0 },
  { "Anoop", 0.8695652 },
  { "Anoosha", 0.0 },
  { "Anora", 0.0 },
  { "Anorah", 0.0 },
  { "Anorea", 0.0 },
  { "Anori", 0.0 },
  { "Anosha", 0.0 },
  { "Anothny", 1.0 },
  { "Anothony", 1.0 },
  { "Anothy", 1.0 },
  { "Anouk", 0.0 },
  { "Anoush", 0.0 },
  { "Anousha", 0.0 },
  { "Anoushka", 0.0 },
  { "Anousone", 1.0 },
  { "Anova", 0.0 },
  { "Anqi", 0.0 },
  { "Anquan", 1.0 },
  { "Anquanette", 0.0 },
  { "Anquavious", 1.0 },
  { "Anquenette", 0.0 },
  { "Anquetta", 0.0 },
  { "Anquette", 0.0 },
  { "Anquinetta", 0.0 },
  { "Anquinette", 0.0 },
  { "Anqunette", 0.0 },
  { "Anre", 1.0 },
  { "Anrea", 0.0 },
  { "Anrew", 1.0 },
  { "Anri", 1.0 },
  { "Ansa", 0.0 },
  { "Ansal", 0.0 },
  { "Ansar", 1.0 },
  { "Ansara", 1.0 },
  { "Ansel", 0.9970432 },
  { "Ansell", 1.0 },
  { "Anselm", 1.0 },
  { "Anselma", 0.0 },
  { "Anselmi", 1.0 },
  { "Anselmo", 1.0 },
  { "Ansen", 1.0 },
  { "Ansh", 1.0 },
  { "Ansha", 0.0 },
  { "Anshel", 1.0 },
  { "Anshi", 0.0 },
  { "Anshika", 0.0 },
  { "Anshita", 0.0 },
  { "Anshu", 0.08196721 },
  { "Anshul", 1.0 },
  { "Anshuman", 1.0 },
  { "Anshveer", 1.0 },
  { "Ansil", 1.0 },
  { "Anslea", 0.0 },
  { "Anslee", 0.0 },
  { "Anslei", 0.0 },
  { "Ansleigh", 0.0 },
  { "Anslem", 1.0 },
  { "Ansley", 0.029396858 },
  { "Ansli", 0.0 },
  { "Anslie", 0.0 },
  { "Anson", 0.99920624 },
  { "Ansonia", 0.0 },
  { "Ansony", 1.0 },
  { "Ansu", 1.0 },
  { "Ansumana", 1.0 },
  { "Anta", 0.0 },
  { "Antaeus", 1.0 },
  { "Antaja", 0.0 },
  { "Antajuan", 1.0 },
  { "Antal", 1.0 },
  { "Antalya", 0.0 },
  { "Antanae", 0.0 },
  { "Antanasia", 0.0 },
  { "Antanay", 0.0 },
  { "Antanea", 0.0 },
  { "Antaneisha", 0.0 },
  { "Antanesha", 0.0 },
  { "Antanette", 0.0 },
  { "Antania", 0.0 },
  { "Antanika", 0.0 },
  { "Antanique", 0.0 },
  { "Antanisha", 0.0 },
  { "Antaniya", 0.0 },
  { "Antaniyah", 0.0 },
  { "Antar", 1.0 },
  { "Antara", 0.0 },
  { "Antares", 1.0 },
  { "Antario", 1.0 },
  { "Antarious", 1.0 },
  { "Antarius", 1.0 },
  { "Antashia", 0.0 },
  { "Antasia", 0.0 },
  { "Antaun", 1.0 },
  { "Antavia", 0.0 },
  { "Antavion", 1.0 },
  { "Antavious", 1.0 },
  { "Antavis", 1.0 },
  { "Antavius", 1.0 },
  { "Antawan", 1.0 },
  { "Antawn", 1.0 },
  { "Antawon", 1.0 },
  { "Antaysia", 0.0 },
  { "Ante", 1.0 },
  { "Antea", 0.0 },
  { "Anteaus", 1.0 },
  { "Antelmo", 1.0 },
  { "Antenette", 0.0 },
  { "Anteo", 1.0 },
  { "Anteria", 0.0 },
  { "Anterio", 1.0 },
  { "Anterion", 1.0 },
  { "Anterious", 1.0 },
  { "Anterius", 1.0 },
  { "Antero", 1.0 },
  { "Anterria", 0.0 },
  { "Anterrio", 1.0 },
  { "Anterrion", 1.0 },
  { "Anterrius", 1.0 },
  { "Antesha", 0.0 },
  { "Antha", 0.0 },
  { "Anthani", 1.0 },
  { "Anthany", 1.0 },
  { "Anthea", 0.0 },
  { "Antheia", 0.0 },
  { "Anthem", 0.88461536 },
  { "Antheny", 1.0 },
  { "Anthia", 0.0 },
  { "Anthnoy", 1.0 },
  { "Anthny", 1.0 },
  { "Anthoine", 1.0 },
  { "Anthon", 1.0 },
  { "Anthone", 1.0 },
  { "Anthonee", 1.0 },
  { "Anthonella", 0.0 },
  { "Anthonette", 0.0 },
  { "Anthoney", 1.0 },
  { "Anthoni", 1.0 },
  { "Anthonia", 0.0 },
  { "Anthonie", 1.0 },
  { "Anthonio", 1.0 },
  { "Anthonique", 0.0 },
  { "Anthonny", 1.0 },
  { "Anthony", 0.99506944 },
  { "Anthonyjames", 1.0 },
  { "Anthonyjohn", 1.0 },
  { "Anthonyjoseph", 1.0 },
  { "Anthonyjr", 1.0 },
  { "Anthonymichael", 1.0 },
  { "Anthory", 1.0 },
  { "Anthoula", 0.0 },
  { "Anthoy", 1.0 },
  { "Anthuan", 1.0 },
  { "Antia", 0.0 },
  { "Antiana", 0.0 },
  { "Antianna", 0.0 },
  { "Antigone", 0.0 },
  { "Antimo", 1.0 },
  { "Antina", 0.0 },
  { "Antinette", 0.0 },
  { "Antinio", 1.0 },
  { "Antinique", 0.0 },
  { "Antinisha", 0.0 },
  { "Antino", 1.0 },
  { "Antion", 1.0 },
  { "Antiona", 0.7826087 },
  { "Antione", 0.9896274 },
  { "Antionett", 0.0 },
  { "Antionetta", 0.0 },
  { "Antionette", 0.0 },
  { "Antionio", 1.0 },
  { "Antionna", 0.0 },
  { "Antionne", 1.0 },
  { "Antiono", 1.0 },
  { "Antiqua", 0.0 },
  { "Antisha", 0.0 },
  { "Antiwan", 1.0 },
  { "Antje", 0.0 },
  { "Antjuan", 1.0 },
  { "Antoan", 1.0 },
  { "Antohny", 1.0 },
  { "Antoin", 1.0 },
  { "Antoinae", 0.0 },
  { "Antoine", 0.9862236 },
  { "Antoinese", 0.0 },
  { "Antoinesha", 0.0 },
  { "Antoinet", 0.0 },
  { "Antoinett", 0.0 },
  { "Antoinetta", 0.0 },
  { "Antoinette", 0.0008692762 },
  { "Antoiniece", 0.0 },
  { "Antoinio", 1.0 },
  { "Antoinique", 0.0 },
  { "Antoinne", 1.0 },
  { "Antoinnette", 0.0 },
  { "Antoino", 1.0 },
  { "Antojuan", 1.0 },
  { "Antolin", 1.0 },
  { "Antolina", 0.0 },
  { "Anton", 0.9990682 },
  { "Antona", 0.0 },
  { "Antonae", 0.0 },
  { "Antonasia", 0.0 },
  { "Antonay", 0.0 },
  { "Antone", 0.9982938 },
  { "Antonea", 0.0 },
  { "Antoneisha", 0.0 },
  { "Antonela", 0.0 },
  { "Antonella", 0.0 },
  { "Antonello", 1.0 },
  { "Antoneo", 1.0 },
  { "Antonese", 0.0 },
  { "Antonesha", 0.0 },
  { "Antoneshia", 0.0 },
  { "Antonett", 0.0 },
  { "Antonetta", 0.0 },
  { "Antonette", 0.0 },
  { "Antoney", 1.0 },
  { "Antoneyo", 1.0 },
  { "Antoni", 0.9973319 },
  { "Antonia", 0.07499642 },
  { "Antoniah", 0.0 },
  { "Antonial", 1.0 },
  { "Antonica", 0.0 },
  { "Antonie", 0.97269183 },
  { "Antonieo", 1.0 },
  { "Antonieta", 0.0 },
  { "Antonietta", 0.0 },
  { "Antoniette", 0.0 },
  { "Antonin", 1.0 },
  { "Antonina", 0.0 },
  { "Antonine", 1.0 },
  { "Antoninette", 0.0 },
  { "Antonino", 1.0 },
  { "Antonio", 0.99197865 },
  { "Antoniodejesus", 1.0 },
  { "Antonion", 1.0 },
  { "Antonios", 1.0 },
  { "Antonious", 1.0 },
  { "Antonique", 0.0 },
  { "Antonis", 1.0 },
  { "Antonise", 0.0 },
  { "Antonisha", 0.0 },
  { "Antonita", 0.0 },
  { "Antonitte", 0.0 },
  { "Antonius", 1.0 },
  { "Antoniya", 0.0 },
  { "Antoniyah", 0.0 },
  { "Antoniyo", 1.0 },
  { "Antonne", 1.0 },
  { "Antonnette", 0.0 },
  { "Antonnia", 0.0 },
  { "Antonnio", 1.0 },
  { "Antonny", 1.0 },
  { "Antony", 1.0 },
  { "Antonya", 0.0 },
  { "Antonyio", 1.0 },
  { "Antonyo", 1.0 },
  { "Antoria", 0.0 },
  { "Antorio", 1.0 },
  { "Antorion", 1.0 },
  { "Antorria", 0.0 },
  { "Antowain", 1.0 },
  { "Antowan", 1.0 },
  { "Antowine", 1.0 },
  { "Antown", 1.0 },
  { "Antowne", 1.0 },
  { "Antoya", 0.0 },
  { "Antranette", 0.0 },
  { "Antravion", 1.0 },
  { "Antravious", 1.0 },
  { "Antre", 1.0 },
  { "Antrea", 0.0 },
  { "Antrel", 1.0 },
  { "Antrell", 1.0 },
  { "Antrese", 0.0 },
  { "Antrez", 1.0 },
  { "Antria", 0.0 },
  { "Antrice", 0.0 },
  { "Antrina", 0.0 },
  { "Antrione", 1.0 },
  { "Antroine", 1.0 },
  { "Antron", 1.0 },
  { "Antrone", 1.0 },
  { "Antronette", 0.0 },
  { "Antroy", 1.0 },
  { "Anttwan", 1.0 },
  { "Antuan", 1.0 },
  { "Antuane", 1.0 },
  { "Antuwan", 1.0 },
  { "Antvan", 1.0 },
  { "Antwaan", 1.0 },
  { "Antwain", 1.0 },
  { "Antwaine", 1.0 },
  { "Antwan", 0.99054396 },
  { "Antwana", 0.5833333 },
  { "Antwanae", 0.0 },
  { "Antwand", 1.0 },
  { "Antwane", 1.0 },
  { "Antwanesha", 0.0 },
  { "Antwanett", 0.0 },
  { "Antwanette", 0.0 },
  { "Antwanique", 0.0 },
  { "Antwanisha", 0.0 },
  { "Antwann", 1.0 },
  { "Antwanne", 1.0 },
  { "Antwaun", 0.9964489 },
  { "Antwaune", 1.0 },
  { "Antwayne", 1.0 },
  { "Antwian", 1.0 },
  { "Antwin", 1.0 },
  { "Antwine", 1.0 },
  { "Antwinette", 0.0 },
  { "Antwion", 1.0 },
  { "Antwione", 1.0 },
  { "Antwoin", 1.0 },
  { "Antwoine", 1.0 },
  { "Antwoinette", 0.0 },
  { "Antwon", 0.9991734 },
  { "Antwone", 1.0 },
  { "Antwonette", 0.0 },
  { "Antwonn", 1.0 },
  { "Antwonne", 1.0 },
  { "Antwuan", 1.0 },
  { "Antwun", 1.0 },
  { "Antwyne", 1.0 },
  { "Antyon", 1.0 },
  { "Antyone", 1.0 },
  { "Anu", 0.04385965 },
  { "Anuar", 1.0 },
  { "Anubhav", 1.0 },
  { "Anubis", 1.0 },
  { "Anudeep", 1.0 },
  { "Anuel", 1.0 },
  { "Anuhea", 0.0 },
  { "Anuhya", 0.0 },
  { "Anuj", 1.0 },
  { "Anuja", 0.0 },
  { "Anujin", 0.0 },
  { "Anum", 0.0 },
  { "Anuoluwa", 0.22727273 },
  { "Anuoluwapo", 0.0 },
  { "Anup", 1.0 },
  { "Anupam", 1.0 },
  { "Anupama", 0.0 },
  { "Anupriya", 0.0 },
  { "Anuraag", 1.0 },
  { "Anuradha", 0.0 },
  { "Anurag", 1.0 },
  { "Anureet", 0.0 },
  { "Anush", 1.0 },
  { "Anusha", 0.0 },
  { "Anushka", 0.0 },
  { "Anushree", 0.0 },
  { "Anushri", 0.0 },
  { "Anuska", 0.0 },
  { "Anusri", 0.0 },
  { "Anuva", 0.0 },
  { "Anvay", 1.0 },
  { "Anvesha", 0.0 },
  { "Anvi", 0.0 },
  { "Anvik", 1.0 },
  { "Anvika", 0.0 },
  { "Anvit", 1.0 },
  { "Anvita", 0.0 },
  { "Anvith", 1.0 },
  { "Anvitha", 0.0 },
  { "Anwaar", 0.0 },
  { "Anwan", 1.0 },
  { "Anwar", 0.9618745 },
  { "Anwen", 0.0 },
  { "Anwesha", 0.0 },
  { "Anwita", 0.0 },
  { "Anwitha", 0.0 },
  { "Anwon", 1.0 },
  { "Anwyn", 0.0 },
  { "Any", 0.0 },
  { "Anya", 0.0 },
  { "Anyae", 0.0 },
  { "Anyah", 0.0 },
  { "Anyanka", 0.0 },
  { "Anyce", 0.0 },
  { "Anycia", 0.0 },
  { "Anye", 0.0 },
  { "Anyea", 0.0 },
  { "Anyela", 0.0 },
  { "Anyeli", 0.0 },
  { "Anyelin", 0.0 },
  { "Anyelina", 0.0 },
  { "Anyeline", 0.0 },
  { "Anyelis", 0.0 },
  { "Anyelo", 1.0 },
  { "Anyely", 0.0 },
  { "Anyha", 0.0 },
  { "Anyhia", 0.0 },
  { "Anyi", 0.0 },
  { "Anyia", 0.0 },
  { "Anyiah", 0.0 },
  { "Anyieth", 0.0 },
  { "Anyila", 0.0 },
  { "Anyja", 0.0 },
  { "Anyjah", 0.0 },
  { "Anyjha", 0.0 },
  { "Anyka", 0.0 },
  { "Anyla", 0.0 },
  { "Anylah", 0.0 },
  { "Anylia", 0.0 },
  { "Anyliah", 0.0 },
  { "Anyra", 0.0 },
  { "Anyree", 0.0 },
  { "Anyri", 0.0 },
  { "Anyria", 0.0 },
  { "Anyriah", 0.0 },
  { "Anyrie", 0.0 },
  { "Anysa", 0.0 },
  { "Anysha", 0.0 },
  { "Anyshia", 0.0 },
  { "Anysia", 0.0 },
  { "Anyssa", 0.0 },
  { "Anyssia", 0.0 },
  { "Anyston", 0.0 },
  { "Anyya", 0.0 },
  { "Anza", 0.0 },
  { "Anzal", 0.0 },
  { "Anzar", 1.0 },
  { "Anze", 1.0 },
  { "Anzel", 0.9166667 },
  { "Anzie", 0.0 },
  { "Anzio", 1.0 },
  { "Anzlee", 0.0 },
  { "Anzleigh", 0.0 },
  { "Anzley", 0.0 },
  { "Anzlie", 0.0 },
  { "Aodhan", 1.0 },
  { "Aoi", 0.119791664 },
  { "Aoibheann", 0.0 },
  { "Aoife", 0.0 },
  { "Aoki", 0.0 },
  { "Aolani", 0.0 },
  { "Aolanis", 0.0 },
  { "Aolany", 0.0 },
  { "Aonesti", 0.0 },
  { "Aonesty", 0.0 },
  { "Aonna", 0.0 },
  { "Aous", 1.0 },
  { "Aowyn", 0.0 },
  { "Apache", 0.61538464 },
  { "Aparajita", 0.0 },
  { "Aparna", 0.0 },
  { "Apasra", 0.0 },
  { "Aphrodite", 0.0 },
  { "Aphton", 0.0 },
  { "Apiffany", 0.0 },
  { "Apiphany", 0.0 },
  { "Apirl", 0.0 },
  { "Apolinar", 0.9960723 },
  { "Apolline", 0.0 },
  { "Apollo", 1.0 },
  { "Apollonia", 0.0 },
  { "Apollos", 1.0 },
  { "Apolo", 1.0 },
  { "Apolonia", 0.0 },
  { "Apolonio", 1.0 },
  { "Aponi", 0.0 },
  { "Apoorva", 0.0 },
  { "Apostolis", 1.0 },
  { "Apostolos", 1.0 },
  { "Apphia", 0.0 },
  { "Appie", 0.0 },
  { "Apple", 0.0 },
  { "Apples", 0.0 },
  { "Applonia", 0.0 },
  { "Appollonia", 0.0 },
  { "Appolonia", 0.0 },
  { "Aprecious", 0.0 },
  { "Aprel", 0.0 },
  { "Aprell", 0.0 },
  { "Apria", 0.0 },
  { "Apriel", 0.0 },
  { "April", 0.0028021913 },
  { "Aprilann", 0.0 },
  { "Aprile", 0.0 },
  { "Apriljoy", 0.0 },
  { "Aprill", 0.0 },
  { "Aprille", 0.0 },
  { "Aprillynn", 0.0 },
  { "Aprilmarie", 0.0 },
  { "Apryl", 0.0 },
  { "Apryle", 0.0 },
  { "Apryll", 0.0 },
  { "Apsara", 0.0 },
  { "Apurva", 0.21705426 },
  { "Aqeel", 1.0 },
  { "Aqib", 1.0 },
  { "Aqil", 1.0 },
  { "Aqila", 0.0 },
  { "Aqilah", 0.0 },
  { "Aqsa", 0.0 },
  { "Aqua", 0.0 },
  { "Aquaisha", 0.0 },
  { "Aquala", 0.0 },
  { "Aquan", 1.0 },
  { "Aquanetta", 0.0 },
  { "Aquanette", 0.0 },
  { "Aquanis", 1.0 },
  { "Aquaria", 0.0 },
  { "Aquarius", 0.38802084 },
  { "Aquasha", 0.0 },
  { "Aquasia", 0.0 },
  { "Aquavia", 0.0 },
  { "Aquavion", 1.0 },
  { "Aquavious", 1.0 },
  { "Aqueela", 0.0 },
  { "Aqueelah", 0.0 },
  { "Aquera", 0.0 },
  { "Aquetzalli", 0.0 },
  { "Aquia", 0.0 },
  { "Aquiel", 0.0 },
  { "Aquil", 1.0 },
  { "Aquila", 0.08897743 },
  { "Aquilah", 0.0 },
  { "Aquileo", 1.0 },
  { "Aquiles", 1.0 },
  { "Aquilina", 0.0 },
  { "Aquilino", 1.0 },
  { "Aquill", 1.0 },
  { "Aquilla", 0.10583942 },
  { "Aquille", 1.0 },
  { "Aquina", 0.0 },
  { "Aquinas", 1.0 },
  { "Aquinnah", 0.0 },
  { "Aquino", 1.0 },
  { "Aquira", 0.0 },
  { "Aquisha", 0.0 },
  { "Aquita", 0.0 },
  { "Ar", 1.0 },
  { "Ara", 0.27825466 },
  { "Araba", 0.0 },
  { "Arabel", 0.0 },
  { "Arabela", 0.0 },
  { "Arabell", 0.0 },
  { "Arabella", 0.0 },
  { "Arabellah", 0.0 },
  { "Arabellarose", 0.0 },
  { "Arabelle", 0.0 },
  { "Arabia", 0.0 },
  { "Arabian", 0.0 },
  { "Aracel", 0.0 },
  { "Aracele", 0.0 },
  { "Araceli", 0.007758039 },
  { "Aracelia", 0.0 },
  { "Aracelie", 0.0 },
  { "Aracelio", 1.0 },
  { "Aracelis", 0.0 },
  { "Araceliz", 0.0 },
  { "Aracelli", 0.0 },
  { "Aracellie", 0.0 },
  { "Aracelly", 0.0 },
  { "Aracely", 0.0012671351 },
  { "Aracelys", 0.0 },
  { "Arad", 1.0 },
  { "Aradhana", 0.0 },
  { "Aradhya", 0.031446543 },
  { "Aradia", 0.0 },
  { "Arael", 1.0 },
  { "Araelynn", 0.0 },
  { "Araeya", 0.0 },
  { "Araf", 1.0 },
  { "Arafat", 1.0 },
  { "Aragon", 1.0 },
  { "Aragorn", 1.0 },
  { "Arah", 0.0 },
  { "Arahbella", 0.0 },
  { "Arahi", 0.0 },
  { "Arai", 0.0 },
  { "Araia", 0.0 },
  { "Araiah", 0.0 },
  { "Araina", 0.0 },
  { "Arainna", 0.0 },
  { "Araiya", 0.0 },
  { "Araiyah", 0.0 },
  { "Araiza", 0.0 },
  { "Aralee", 0.0 },
  { "Arali", 0.0 },
  { "Aralia", 0.0 },
  { "Aralin", 0.0 },
  { "Aralis", 0.0 },
  { "Araly", 0.0 },
  { "Aralyn", 0.0 },
  { "Aralynn", 0.0 },
  { "Aralynne", 0.0 },
  { "Aram", 0.9852427 },
  { "Arame", 0.0 },
  { "Arami", 0.0 },
  { "Aramie", 0.0 },
  { "Araminta", 0.0 },
  { "Aramis", 0.91217566 },
  { "Aramus", 1.0 },
  { "Aramys", 1.0 },
  { "Aran", 0.962963 },
  { "Aranda", 0.0 },
  { "Aransa", 0.0 },
  { "Arantxa", 0.0 },
  { "Arantza", 0.0 },
  { "Aranxa", 0.0 },
  { "Aranya", 0.0 },
  { "Aranza", 0.0 },
  { "Aranzazu", 0.0 },
  { "Araoluwa", 0.0 },
  { "Aras", 1.0 },
  { "Arasele", 0.0 },
  { "Araseli", 0.0 },
  { "Arasely", 0.0 },
  { "Arash", 1.0 },
  { "Arashdeep", 1.0 },
  { "Arashel", 0.0 },
  { "Arata", 1.0 },
  { "Arath", 1.0 },
  { "Arathi", 0.0 },
  { "Arati", 0.0 },
  { "Arav", 1.0 },
  { "Aravella", 0.0 },
  { "Aravind", 1.0 },
  { "Aravis", 0.0 },
  { "Arawyn", 0.0 },
  { "Arax", 0.0 },
  { "Araxi", 0.0 },
  { "Araya", 0.0022935779 },
  { "Arayah", 0.0 },
  { "Arayana", 0.0 },
  { "Arayeli", 0.0 },
  { "Arayia", 0.0 },
  { "Arayiah", 0.0 },
  { "Arayla", 0.0 },
  { "Araylia", 0.0 },
  { "Arayna", 0.0 },
  { "Araz", 1.0 },
  { "Arazeli", 0.0 },
  { "Arba", 0.74603176 },
  { "Arbaaz", 1.0 },
  { "Arbadella", 0.0 },
  { "Arbay", 0.0 },
  { "Arbaz", 1.0 },
  { "Arbedella", 0.0 },
  { "Arbelia", 0.0 },
  { "Arbell", 0.0 },
  { "Arbella", 0.0 },
  { "Arbelle", 0.0 },
  { "Arben", 1.0 },
  { "Arber", 1.0 },
  { "Arbi", 1.0 },
  { "Arbie", 0.78923076 },
  { "Arbin", 1.0 },
  { "Arbon", 1.0 },
  { "Arbor", 0.21428572 },
  { "Arbra", 0.0 },
  { "Arbri", 0.0 },
  { "Arbrianna", 0.0 },
  { "Arbrielle", 0.0 },
  { "Arbutis", 0.0 },
  { "Arbutus", 0.0 },
  { "Arby", 1.0 },
  { "Arcadia", 0.0 },
  { "Arcadian", 1.0 },
  { "Arcadio", 1.0 },
  { "Arcadius", 1.0 },
  { "Arcangel", 1.0 },
  { "Arcangela", 0.0 },
  { "Arcangelo", 1.0 },
  { "Arcel", 1.0 },
  { "Arceli", 0.0 },
  { "Arcelia", 0.0 },
  { "Arcella", 0.0 },
  { "Arcely", 0.0 },
  { "Arcenia", 0.0 },
  { "Arcenio", 1.0 },
  { "Arch", 1.0 },
  { "Archa", 1.0 },
  { "Archan", 1.0 },
  { "Archana", 0.0 },
  { "Archelle", 0.0 },
  { "Archer", 0.98401064 },
  { "Archi", 0.0 },
  { "Archibald", 1.0 },
  { "Archie", 0.97547865 },
  { "Archiebald", 1.0 },
  { "Archille", 1.0 },
  { "Archimedes", 1.0 },
  { "Archisha", 0.0 },
  { "Archit", 1.0 },
  { "Archita", 0.0 },
  { "Archith", 1.0 },
  { "Architha", 0.0 },
  { "Archivaldo", 1.0 },
  { "Arcie", 0.24285714 },
  { "Arcilia", 0.0 },
  { "Arco", 1.0 },
  { "Arcola", 0.0 },
  { "Arcus", 1.0 },
  { "Arda", 0.5414365 },
  { "Ardala", 0.0 },
  { "Ardalia", 0.0 },
  { "Ardan", 1.0 },
  { "Ardarius", 1.0 },
  { "Ardath", 0.016320474 },
  { "Ardean", 0.37 },
  { "Ardeen", 0.0 },
  { "Ardel", 0.2421875 },
  { "Ardele", 0.0 },
  { "Ardelia", 0.0 },
  { "Ardell", 0.4778042 },
  { "Ardella", 0.0 },
  { "Ardelle", 0.0 },
  { "Arden", 0.57024455 },
  { "Ardena", 0.0 },
  { "Ardene", 0.0 },
  { "Ardenia", 0.0 },
  { "Ardes", 0.0 },
  { "Ardeth", 0.0 },
  { "Ardi", 1.0 },
  { "Ardia", 0.0 },
  { "Ardian", 1.0 },
  { "Ardiana", 0.0 },
  { "Ardice", 0.0 },
  { "Ardie", 0.4764977 },
  { "Ardilia", 0.0 },
  { "Ardin", 1.0 },
  { "Ardina", 0.0 },
  { "Ardine", 0.03846154 },
  { "Ardis", 0.13853224 },
  { "Ardit", 1.0 },
  { "Ardita", 0.0 },
  { "Ardith", 0.022934174 },
  { "Ardon", 1.0 },
  { "Ardonia", 0.0 },
  { "Ardoth", 0.0 },
  { "Ardra", 0.0 },
  { "Ardriana", 0.0 },
  { "Ardus", 0.0 },
  { "Ardwin", 1.0 },
  { "Ardy", 0.0 },
  { "Ardyce", 0.0 },
  { "Ardyn", 0.0 },
  { "Ardynn", 0.0 },
  { "Ardys", 0.0052137645 },
  { "Ardyth", 0.0 },
  { "Ardythe", 0.0 },
  { "Areah", 0.0 },
  { "Areal", 0.0 },
  { "Arean", 1.0 },
  { "Areana", 0.0 },
  { "Areanna", 0.0 },
  { "Areatha", 0.0 },
  { "Areather", 0.0 },
  { "Arec", 1.0 },
  { "Ared", 1.0 },
  { "Aree", 0.12195122 },
  { "Areeb", 1.0 },
  { "Areeba", 0.0 },
  { "Areebah", 0.0 },
  { "Areecia", 0.0 },
  { "Areeg", 0.0 },
  { "Areej", 0.0 },
  { "Areen", 0.036231883 },
  { "Areena", 0.0 },
  { "Areesha", 0.0 },
  { "Areeya", 0.0 },
  { "Aref", 1.0 },
  { "Areg", 1.0 },
  { "Areia", 0.0 },
  { "Areial", 0.0 },
  { "Areiana", 0.0 },
  { "Areianna", 0.0 },
  { "Areigha", 0.0 },
  { "Areil", 0.0 },
  { "Areille", 0.0 },
  { "Areion", 0.1388889 },
  { "Areiona", 0.0 },
  { "Areionna", 0.0 },
  { "Areisy", 0.0 },
  { "Arek", 1.0 },
  { "Arel", 0.9708738 },
  { "Arela", 0.0 },
  { "Arelene", 0.0 },
  { "Areli", 0.01306004 },
  { "Arelia", 0.0 },
  { "Arelie", 0.0 },
  { "Arelin", 0.0 },
  { "Arelis", 0.0 },
  { "Areliz", 0.0 },
  { "Arella", 0.0 },
  { "Arelle", 0.0 },
  { "Arelli", 0.0 },
  { "Arelly", 0.0 },
  { "Arellys", 0.0 },
  { "Arely", 0.0 },
  { "Arelyn", 0.0 },
  { "Arelys", 0.0 },
  { "Aremi", 0.0 },
  { "Aremy", 0.0 },
  { "Aren", 0.8324051 },
  { "Arena", 0.0 },
  { "Arend", 1.0 },
  { "Arenda", 0.0 },
  { "Arene", 0.0 },
  { "Areni", 0.0 },
  { "Areon", 0.6456693 },
  { "Areona", 0.0 },
  { "Areonia", 0.0 },
  { "Areonna", 0.0 },
  { "Ares", 0.9732669 },
  { "Aresenio", 1.0 },
  { "Aresha", 0.0 },
  { "Areta", 0.0 },
  { "Aretha", 0.0 },
  { "Arethea", 0.0 },
  { "Arether", 0.0 },
  { "Arethia", 0.0 },
  { "Areti", 0.0 },
  { "Aretina", 0.0 },
  { "Aretta", 0.0 },
  { "Aretzi", 0.0 },
  { "Aretzy", 0.0 },
  { "Arey", 0.0 },
  { "Areya", 0.0 },
  { "Areyah", 0.0 },
  { "Areyana", 0.0 },
  { "Areyanna", 0.0 },
  { "Areyon", 0.3125 },
  { "Areyona", 0.0 },
  { "Areyonna", 0.0 },
  { "Arezo", 0.0 },
  { "Arfa", 0.0 },
  { "Arfaan", 1.0 },
  { "Arfan", 1.0 },
  { "Argatha", 0.0 },
  { "Argel", 1.0 },
  { "Argelia", 0.0 },
  { "Argelio", 1.0 },
  { "Argelis", 0.8780488 },
  { "Argene", 1.0 },
  { "Argenis", 1.0 },
  { "Argeniz", 1.0 },
  { "Argent", 0.0 },
  { "Argentina", 0.0 },
  { "Argentine", 0.0 },
  { "Argenys", 1.0 },
  { "Argie", 0.12977777 },
  { "Argil", 1.0 },
  { "Argiro", 0.0 },
  { "Argle", 1.0 },
  { "Argo", 1.0 },
  { "Argonne", 1.0 },
  { "Argus", 1.0 },
  { "Argusta", 0.2736842 },
  { "Arguster", 0.52380955 },
  { "Argyl", 1.0 },
  { "Argyle", 0.8309859 },
  { "Argyro", 0.0 },
  { "Arha", 1.0 },
  { "Arhaam", 1.0 },
  { "Arhaan", 1.0 },
  { "Arham", 1.0 },
  { "Arhan", 1.0 },
  { "Arhant", 1.0 },
  { "Arhea", 0.0 },
  { "Arhiana", 0.0 },
  { "Arhianna", 0.0 },
  { "Arhonda", 0.0 },
  { "Arhtur", 1.0 },
  { "Arhum", 1.0 },
  { "Ari", 0.8253306 },
  { "Aria", 0.013680625 },
  { "Ariaa", 0.0 },
  { "Ariaan", 1.0 },
  { "Ariabella", 0.0 },
  { "Ariadna", 0.0 },
  { "Ariadne", 0.0 },
  { "Ariadnna", 0.0 },
  { "Ariadnne", 0.0 },
  { "Ariadny", 0.0 },
  { "Ariagna", 0.0 },
  { "Ariah", 0.00702439 },
  { "Ariaha", 0.0 },
  { "Ariahana", 0.0 },
  { "Ariahna", 0.0 },
  { "Ariahnna", 0.0 },
  { "Ariajade", 0.0 },
  { "Arial", 0.0 },
  { "Ariale", 0.0 },
  { "Arialle", 0.0 },
  { "Arialynn", 0.0 },
  { "Ariam", 0.06563707 },
  { "Ariamarie", 0.0 },
  { "Arian", 0.75865066 },
  { "Ariana", 0.0019448106 },
  { "Arianah", 0.0 },
  { "Arianamarie", 0.0 },
  { "Arianda", 0.0 },
  { "Ariane", 0.0014216662 },
  { "Ariani", 0.0 },
  { "Arianie", 0.0 },
  { "Arianis", 0.0 },
  { "Ariann", 0.014117647 },
  { "Arianna", 0.00084766716 },
  { "Ariannah", 0.0 },
  { "Arianne", 0.0 },
  { "Arianni", 0.0 },
  { "Ariannie", 0.0 },
  { "Arianny", 0.0 },
  { "Ariano", 1.0 },
  { "Ariany", 0.0 },
  { "Arianys", 0.0 },
  { "Ariarose", 0.0 },
  { "Arias", 0.8209607 },
  { "Ariat", 1.0 },
  { "Ariatna", 0.0 },
  { "Ariauna", 0.0 },
  { "Ariaunna", 0.0 },
  { "Ariaya", 0.0 },
  { "Ariayah", 0.0 },
  { "Arib", 1.0 },
  { "Ariba", 0.0 },
  { "Aribah", 0.0 },
  { "Aribel", 0.0 },
  { "Aribella", 0.0 },
  { "Aribelle", 0.0 },
  { "Aribeth", 0.0 },
  { "Aric", 0.9993146 },
  { "Arica", 0.0 },
  { "Aricca", 0.0 },
  { "Aricel", 0.0 },
  { "Aricela", 0.0 },
  { "Ariceli", 0.0 },
  { "Aricella", 0.0 },
  { "Aricely", 0.0 },
  { "Aricia", 0.0 },
  { "Aricin", 1.0 },
  { "Arick", 1.0 },
  { "Aricka", 0.0 },
  { "Aridai", 0.0 },
  { "Ariday", 0.0 },
  { "Arie", 0.39030766 },
  { "Ariea", 0.0 },
  { "Arieah", 0.0 },
  { "Arieal", 0.0 },
  { "Arieana", 0.0 },
  { "Arieanna", 0.0 },
  { "Ariebella", 0.0 },
  { "Ariee", 0.0 },
  { "Arieh", 1.0 },
  { "Ariel", 0.20655094 },
  { "Ariela", 0.0 },
  { "Arielah", 0.0 },
  { "Ariele", 0.0 },
  { "Arieli", 0.0 },
  { "Arielis", 0.0 },
  { "Arieliz", 0.0 },
  { "Ariell", 0.00622665 },
  { "Ariella", 0.0 },
  { "Ariellah", 0.0 },
  { "Arielle", 0.001712622 },
  { "Arielly", 0.0 },
  { "Arielmarie", 0.0 },
  { "Ariely", 0.0 },
  { "Arielyn", 0.0 },
  { "Arielys", 0.0 },
  { "Arielyz", 0.0 },
  { "Arien", 0.55444443 },
  { "Ariena", 0.0 },
  { "Ariene", 0.0 },
  { "Arienette", 0.0 },
  { "Arienna", 0.0 },
  { "Arienne", 0.0 },
  { "Arieon", 0.6047904 },
  { "Arieona", 0.0 },
  { "Arieonna", 0.0 },
  { "Aries", 0.5971884 },
  { "Arieta", 0.0 },
  { "Arietta", 0.0 },
  { "Arieus", 1.0 },
  { "Arieya", 0.0 },
  { "Arieyana", 0.0 },
  { "Arieyanna", 0.0 },
  { "Arieyonna", 0.0 },
  { "Ariez", 0.94186044 },
  { "Arif", 1.0 },
  { "Arifa", 0.0 },
  { "Ariha", 0.0 },
  { "Arihaan", 1.0 },
  { "Arihan", 1.0 },
  { "Arihana", 0.0 },
  { "Arihanna", 0.0 },
  { "Arihant", 1.0 },
  { "Arii", 0.0 },
  { "Ariiana", 0.0 },
  { "Arij", 0.0 },
  { "Arijah", 0.0 },
  { "Arijana", 0.0 },
  { "Arijit", 1.0 },
  { "Arik", 1.0 },
  { "Arika", 0.0 },
  { "Arikah", 0.0 },
  { "Arikka", 0.0 },
  { "Aril", 0.0 },
  { "Arilene", 0.0 },
  { "Arilla", 0.0 },
  { "Arillia", 0.0 },
  { "Arilyn", 0.0 },
  { "Arilynn", 0.0 },
  { "Arim", 0.0 },
  { "Arimentha", 0.0 },
  { "Arin", 0.4061866 },
  { "Arina", 0.0 },
  { "Arinda", 0.0 },
  { "Arine", 0.0 },
  { "Arinjay", 1.0 },
  { "Arinn", 0.0 },
  { "Arinna", 0.0 },
  { "Arinola", 0.0 },
  { "Arinze", 1.0 },
  { "Arinzechukwu", 1.0 },
  { "Ario", 1.0 },
  { "Arion", 0.73646533 },
  { "Ariona", 0.0 },
  { "Arionah", 0.0 },
  { "Arione", 0.0 },
  { "Arionna", 0.0 },
  { "Arionne", 0.0 },
  { "Ariq", 1.0 },
  { "Aris", 0.6 },
  { "Arisa", 0.0 },
  { "Arisai", 0.0 },
  { "Arisbel", 0.0 },
  { "Arisbet", 0.0 },
  { "Arisbeth", 0.0 },
  { "Arisdelsy", 0.0 },
  { "Arisela", 0.0 },
  { "Arish", 1.0 },
  { "Arisha", 0.0 },
  { "Arishka", 0.0 },
  { "Arisleidy", 0.0 },
  { "Arismel", 0.0 },
  { "Arismendy", 1.0 },
  { "Arison", 1.0 },
  { "Ariss", 0.0 },
  { "Arissa", 0.0 },
  { "Arista", 0.0 },
  { "Aristea", 0.0 },
  { "Aristede", 1.0 },
  { "Aristedes", 1.0 },
  { "Aristeo", 1.0 },
  { "Aristide", 1.0 },
  { "Aristides", 1.0 },
  { "Aristidis", 1.0 },
  { "Ariston", 1.0 },
  { "Aristotelis", 1.0 },
  { "Aristotle", 1.0 },
  { "Arit", 0.0 },
  { "Arita", 0.0 },
  { "Aritha", 0.0 },
  { "Aritra", 1.0 },
  { "Aritza", 0.0 },
  { "Aritzel", 0.0 },
  { "Aritzi", 0.0 },
  { "Aritzy", 0.0 },
  { "Ariunna", 0.0 },
  { "Arius", 0.9357798 },
  { "Ariv", 1.0 },
  { "Ariya", 0.0115074795 },
  { "Ariyah", 0.0 },
  { "Ariyahna", 0.0 },
  { "Ariyal", 0.0 },
  { "Ariyan", 0.44339624 },
  { "Ariyana", 0.0 },
  { "Ariyanah", 0.0 },
  { "Ariyani", 0.0 },
  { "Ariyanna", 0.0 },
  { "Ariyannah", 0.0 },
  { "Ariyauna", 0.0 },
  { "Ariyeh", 1.0 },
  { "Ariyel", 0.0 },
  { "Ariyella", 0.0 },
  { "Ariyelle", 0.0 },
  { "Ariyha", 0.0 },
  { "Ariyiah", 0.0 },
  { "Ariyon", 0.34765625 },
  { "Ariyona", 0.0 },
  { "Ariyonna", 0.0 },
  { "Ariyunna", 0.0 },
  { "Ariz", 1.0 },
  { "Ariza", 0.0 },
  { "Arizbeth", 0.0 },
  { "Arizel", 0.0 },
  { "Arizona", 0.04384944 },
  { "Arizonia", 0.0 },
  { "Arjae", 1.0 },
  { "Arjan", 1.0 },
  { "Arjanae", 0.0 },
  { "Arjav", 1.0 },
  { "Arjay", 1.0 },
  { "Arjean", 0.0 },
  { "Arjen", 1.0 },
  { "Arjenis", 1.0 },
  { "Arjeta", 0.0 },
  { "Arjit", 1.0 },
  { "Arjun", 1.0 },
  { "Arjuna", 1.0 },
  { "Ark", 1.0 },
  { "Arka", 1.0 },
  { "Arkadia", 0.0 },
  { "Arkadian", 1.0 },
  { "Arkadiusz", 1.0 },
  { "Arkady", 1.0 },
  { "Arkan", 1.0 },
  { "Arkangel", 1.0 },
  { "Arkayla", 0.0 },
  { "Arkeba", 0.0 },
  { "Arkee", 1.0 },
  { "Arkeem", 1.0 },
  { "Arkeen", 1.0 },
  { "Arkeisha", 0.0 },
  { "Arkeith", 1.0 },
  { "Arkel", 1.0 },
  { "Arkell", 1.0 },
  { "Arkeria", 0.0 },
  { "Arkesha", 0.0 },
  { "Arkeshia", 0.0 },
  { "Arkham", 1.0 },
  { "Arkia", 0.0 },
  { "Arkie", 0.16818182 },
  { "Arkim", 1.0 },
  { "Arkin", 1.0 },
  { "Arkisha", 0.0 },
  { "Arkita", 0.0 },
  { "Arkley", 1.0 },
  { "Arko", 1.0 },
  { "Arkyn", 1.0 },
  { "Arl", 1.0 },
  { "Arla", 0.0065109152 },
  { "Arlaina", 0.0 },
  { "Arlaine", 0.0 },
  { "Arlan", 1.0 },
  { "Arlana", 0.0 },
  { "Arland", 0.99714124 },
  { "Arlanda", 0.27173913 },
  { "Arlander", 1.0 },
  { "Arlandis", 1.0 },
  { "Arlando", 1.0 },
  { "Arlandria", 0.0 },
  { "Arlandus", 1.0 },
  { "Arlane", 0.0 },
  { "Arlayne", 0.0 },
  { "Arlea", 0.0 },
  { "Arlean", 0.0 },
  { "Arleane", 0.0 },
  { "Arleaner", 0.0 },
  { "Arlease", 0.0 },
  { "Arleatha", 0.0 },
  { "Arlecia", 0.0 },
  { "Arleda", 0.0 },
  { "Arlee", 0.30718476 },
  { "Arleen", 0.00048899755 },
  { "Arleena", 0.0 },
  { "Arleene", 0.0 },
  { "Arleeta", 0.0 },
  { "Arleht", 0.0 },
  { "Arleigh", 0.5367965 },
  { "Arlein", 0.0 },
  { "Arleine", 0.0 },
  { "Arlen", 0.91309756 },
  { "Arlena", 0.0 },
  { "Arlene", 0.0025449228 },
  { "Arlener", 0.0 },
  { "Arleni", 0.0 },
  { "Arlenis", 0.0 },
  { "Arlenne", 0.0 },
  { "Arleny", 0.0 },
  { "Arlenys", 0.0 },
  { "Arleon", 0.0 },
  { "Arles", 0.9701493 },
  { "Arlesa", 0.0 },
  { "Arlesha", 0.0 },
  { "Arlesia", 0.0 },
  { "Arless", 0.9285714 },
  { "Arlester", 1.0 },
  { "Arlet", 0.005165289 },
  { "Arleta", 0.0 },
  { "Arlete", 0.0 },
  { "Arleth", 0.0 },
  { "Arletha", 0.0 },
  { "Arlethe", 0.0 },
  { "Arlethea", 0.0 },
  { "Arlether", 0.0 },
  { "Arlethia", 0.0 },
  { "Arlett", 0.0 },
  { "Arletta", 0.0 },
  { "Arlette", 0.0 },
  { "Arlevia", 0.0 },
  { "Arlex", 1.0 },
  { "Arley", 0.834814 },
  { "Arli", 0.0 },
  { "Arliana", 0.0 },
  { "Arlice", 0.63414633 },
  { "Arlicia", 0.0 },
  { "Arlie", 0.75399286 },
  { "Arlien", 0.0 },
  { "Arliene", 0.0 },
  { "Arlillian", 0.0 },
  { "Arlin", 0.8433762 },
  { "Arlina", 0.0 },
  { "Arlind", 1.0 },
  { "Arlinda", 0.0 },
  { "Arline", 0.0020470829 },
  { "Arling", 1.0 },
  { "Arlington", 0.97343326 },
  { "Arlis", 0.69649655 },
  { "Arlisa", 0.0 },
  { "Arlisha", 0.0 },
  { "Arliss", 0.4157549 },
  { "Arlissa", 0.0 },
  { "Arlita", 0.0 },
  { "Arliz", 0.0 },
  { "Arlo", 0.9772787 },
  { "Arloa", 0.0 },
  { "Arloene", 0.0 },
  { "Arloine", 0.0 },
  { "Arlon", 1.0 },
  { "Arlonda", 0.0 },
  { "Arlondo", 1.0 },
  { "Arlone", 0.0 },
  { "Arlos", 1.0 },
  { "Arlow", 0.9084507 },
  { "Arlowe", 0.0 },
  { "Arloween", 0.0 },
  { "Arlowene", 0.0 },
  { "Arlton", 1.0 },
  { "Arlus", 1.0 },
  { "Arly", 0.06521739 },
  { "Arlyce", 0.0 },
  { "Arlyle", 0.0 },
  { "Arlyn", 0.51101756 },
  { "Arlynda", 0.0 },
  { "Arlyne", 0.0 },
  { "Arlynes", 0.0 },
  { "Arlynn", 0.12248996 },
  { "Arlynne", 0.0 },
  { "Arlys", 0.0 },
  { "Arlyss", 0.0 },
  { "Arlyssa", 0.0 },
  { "Arma", 0.0 },
  { "Armaad", 1.0 },
  { "Armaan", 1.0 },
  { "Armad", 1.0 },
  { "Armahn", 1.0 },
  { "Armahni", 0.25 },
  { "Arman", 1.0 },
  { "Armana", 0.0 },
  { "Armand", 0.99960047 },
  { "Armanda", 0.0 },
  { "Armande", 0.0 },
  { "Armandina", 0.0 },
  { "Armando", 0.994894 },
  { "Armanee", 0.0 },
  { "Armani", 0.5600205 },
  { "Armanie", 0.297561 },
  { "Armanii", 0.3283582 },
  { "Armann", 1.0 },
  { "Armanni", 0.75 },
  { "Armante", 1.0 },
  { "Armanti", 1.0 },
  { "Armany", 1.0 },
  { "Armar", 1.0 },
  { "Armard", 1.0 },
  { "Armari", 0.6953125 },
  { "Armarion", 1.0 },
  { "Armas", 1.0 },
  { "Armatha", 0.0 },
  { "Armaun", 1.0 },
  { "Armeda", 0.0 },
  { "Armeen", 0.90384614 },
  { "Armel", 1.0 },
  { "Armelda", 0.0 },
  { "Armelia", 0.0 },
  { "Armelinda", 0.0 },
  { "Armella", 0.0 },
  { "Armelle", 0.0 },
  { "Armen", 0.99730605 },
  { "Armena", 0.0 },
  { "Armend", 1.0 },
  { "Armenda", 0.0 },
  { "Armenia", 0.0 },
  { "Armenta", 0.0 },
  { "Armentha", 0.0 },
  { "Armer", 0.27272728 },
  { "Armesha", 0.0 },
  { "Armeta", 0.0 },
  { "Armetha", 0.0 },
  { "Armetrice", 0.0 },
  { "Armetta", 0.0 },
  { "Armiah", 0.0 },
  { "Armida", 0.0 },
  { "Armie", 0.0 },
  { "Armilda", 0.0 },
  { "Armilla", 0.0 },
  { "Armin", 1.0 },
  { "Armina", 0.0 },
  { "Arminda", 0.0 },
  { "Armine", 0.0 },
  { "Arminta", 0.0 },
  { "Armintha", 0.0 },
  { "Armir", 1.0 },
  { "Armisha", 0.0 },
  { "Armistead", 1.0 },
  { "Armistice", 0.52380955 },
  { "Armita", 0.0 },
  { "Armittie", 0.0 },
  { "Armiya", 0.0 },
  { "Armiyah", 0.0 },
  { "Armod", 1.0 },
  { "Armon", 1.0 },
  { "Armond", 1.0 },
  { "Armondo", 1.0 },
  { "Armone", 0.84126985 },
  { "Armonee", 0.075 },
  { "Armoney", 0.0 },
  { "Armoni", 0.37953487 },
  { "Armonie", 0.2557143 },
  { "Armonii", 0.0 },
  { "Armonni", 0.0 },
  { "Armonnie", 0.0 },
  { "Armonta", 1.0 },
  { "Armontae", 1.0 },
  { "Armonte", 1.0 },
  { "Armony", 0.0 },
  { "Armor", 1.0 },
  { "Armour", 1.0 },
  { "Armstead", 1.0 },
  { "Armster", 1.0 },
  { "Armstrong", 1.0 },
  { "Army", 0.0 },
  { "Armya", 0.0 },
  { "Arn", 1.0 },
  { "Arna", 0.0 },
  { "Arnab", 1.0 },
  { "Arnae", 0.0 },
  { "Arnald", 1.0 },
  { "Arnaldo", 1.0 },
  { "Arnardo", 1.0 },
  { "Arnasia", 0.0 },
  { "Arnaud", 1.0 },
  { "Arnav", 1.0 },
  { "Arnavi", 0.0 },
  { "Arnay", 1.0 },
  { "Arnaz", 1.0 },
  { "Arne", 1.0 },
  { "Arnecia", 0.0 },
  { "Arneda", 0.0 },
  { "Arnedra", 0.0 },
  { "Arnee", 0.0 },
  { "Arneice", 0.0 },
  { "Arneisha", 0.0 },
  { "Arneita", 0.0 },
  { "Arnel", 0.9730061 },
  { "Arnela", 0.0 },
  { "Arnelda", 0.0 },
  { "Arnelia", 0.0 },
  { "Arnell", 0.6093137 },
  { "Arnella", 0.0 },
  { "Arnelle", 0.0 },
  { "Arner", 1.0 },
  { "Arnes", 1.0 },
  { "Arnesh", 1.0 },
  { "Arnesha", 0.0 },
  { "Arneshia", 0.0 },
  { "Arnesia", 0.0 },
  { "Arness", 1.0 },
  { "Arnessa", 0.0 },
  { "Arnesto", 1.0 },
  { "Arnet", 1.0 },
  { "Arneta", 0.0 },
  { "Arnetha", 0.0 },
  { "Arnetia", 0.0 },
  { "Arnetra", 0.0 },
  { "Arnetria", 0.0 },
  { "Arnett", 0.9453751 },
  { "Arnetta", 0.0 },
  { "Arnette", 0.030330881 },
  { "Arnettie", 0.0 },
  { "Arney", 1.0 },
  { "Arnez", 0.9896907 },
  { "Arni", 0.0 },
  { "Arnica", 0.0 },
  { "Arnice", 0.0 },
  { "Arnie", 0.89546263 },
  { "Arniece", 0.0 },
  { "Arniesha", 0.0 },
  { "Arnika", 0.0 },
  { "Arnim", 1.0 },
  { "Arnis", 1.0 },
  { "Arnise", 0.0 },
  { "Arnisha", 0.0 },
  { "Arnissa", 0.0 },
  { "Arnit", 1.0 },
  { "Arnita", 0.0 },
  { "Arnitha", 0.0 },
  { "Arnitra", 0.0 },
  { "Arniya", 0.0 },
  { "Arniyah", 0.0 },
  { "Arno", 1.0 },
  { "Arnob", 1.0 },
  { "Arnol", 1.0 },
  { "Arnola", 0.0 },
  { "Arnold", 0.9946808 },
  { "Arnoldo", 1.0 },
  { "Arnolfo", 1.0 },
  { "Arnon", 1.0 },
  { "Arnt", 1.0 },
  { "Arnulfo", 1.0 },
  { "Arny", 1.0 },
  { "Aro", 1.0 },
  { "Aroara", 0.0 },
  { "Aroha", 0.0 },
  { "Arohan", 1.0 },
  { "Arohi", 0.0 },
  { "Arol", 1.0 },
  { "Arold", 1.0 },
  { "Aroldo", 1.0 },
  { "Arolyn", 0.0 },
  { "Aron", 0.9808429 },
  { "Arona", 0.2857143 },
  { "Aronda", 0.0 },
  { "Aronde", 1.0 },
  { "Arooj", 0.0 },
  { "Aroon", 1.0 },
  { "Aroosa", 0.0 },
  { "Aroosh", 0.14705883 },
  { "Arora", 0.0 },
  { "Aroura", 0.0 },
  { "Aroush", 0.0 },
  { "Arowyn", 0.0 },
  { "Aroya", 0.0 },
  { "Aroyal", 1.0 },
  { "Arpa", 0.0 },
  { "Arpad", 1.0 },
  { "Arpan", 0.9766355 },
  { "Arpi", 0.0 },
  { "Arpie", 0.0 },
  { "Arpil", 0.0 },
  { "Arpine", 0.0 },
  { "Arpit", 1.0 },
  { "Arpita", 0.0 },
  { "Arquan", 1.0 },
  { "Arquilla", 0.0 },
  { "Arquimedes", 1.0 },
  { "Arquimides", 1.0 },
  { "Arquita", 0.0 },
  { "Arra", 0.0 },
  { "Arrabella", 0.0 },
  { "Arrabelle", 0.0 },
  { "Arrah", 0.0 },
  { "Arran", 1.0 },
  { "Arraya", 0.0 },
  { "Arrayah", 0.0 },
  { "Arreana", 0.0 },
  { "Arreanna", 0.0 },
  { "Arren", 0.9315961 },
  { "Arrena", 0.0 },
  { "Arreon", 0.4 },
  { "Arreona", 0.0 },
  { "Arreonna", 0.0 },
  { "Arretta", 0.0 },
  { "Arri", 0.4651163 },
  { "Arria", 0.0 },
  { "Arriah", 0.0 },
  { "Arrian", 0.7941176 },
  { "Arriana", 0.0 },
  { "Arriane", 0.0 },
  { "Arrianna", 0.0 },
  { "Arriannah", 0.0 },
  { "Arrianne", 0.0 },
  { "Arric", 1.0 },
  { "Arrica", 0.0 },
  { "Arrick", 1.0 },
  { "Arrie", 0.15234375 },
  { "Arrieanna", 0.0 },
  { "Arriel", 0.0 },
  { "Arriell", 0.0 },
  { "Arriella", 0.0 },
  { "Arrielle", 0.0 },
  { "Arrietta", 0.0 },
  { "Arrietty", 0.0 },
  { "Arrihanna", 0.0 },
  { "Arrik", 1.0 },
  { "Arrika", 0.0 },
  { "Arrilla", 0.0 },
  { "Arrin", 0.5828221 },
  { "Arrington", 0.7048666 },
  { "Arrion", 0.76488096 },
  { "Arriona", 0.0 },
  { "Arrionna", 0.0 },
  { "Arris", 1.0 },
  { "Arrissa", 0.0 },
  { "Arrius", 1.0 },
  { "Arriya", 0.0 },
  { "Arriyah", 0.0 },
  { "Arriyana", 0.0 },
  { "Arriyanah", 0.0 },
  { "Arriyanna", 0.0 },
  { "Arro", 1.0 },
  { "Arron", 0.97847456 },
  { "Arrow", 0.71105194 },
  { "Arry", 1.0 },
  { "Arrya", 0.0 },
  { "Arryana", 0.0 },
  { "Arryanna", 0.0 },
  { "Arryn", 0.1375 },
  { "Arryon", 0.0 },
  { "Arryonna", 0.0 },
  { "Arsal", 1.0 },
  { "Arsalaan", 1.0 },
  { "Arsalan", 1.0 },
  { "Arsam", 1.0 },
  { "Arsema", 0.0 },
  { "Arsen", 1.0 },
  { "Arsene", 1.0 },
  { "Arsenia", 0.0 },
  { "Arsenial", 1.0 },
  { "Arsenio", 1.0 },
  { "Arseniy", 1.0 },
  { "Arseno", 1.0 },
  { "Arseny", 1.0 },
  { "Arsh", 1.0 },
  { "Arsha", 0.23255815 },
  { "Arshaan", 1.0 },
  { "Arshad", 1.0 },
  { "Arshag", 1.0 },
  { "Arshan", 1.0 },
  { "Arshanti", 0.0 },
  { "Arshaun", 1.0 },
  { "Arshawn", 1.0 },
  { "Arshdeep", 0.8636364 },
  { "Arshi", 0.0 },
  { "Arshia", 0.25986078 },
  { "Arshika", 0.0 },
  { "Arshiya", 0.0 },
  { "Arshon", 1.0 },
  { "Arshveer", 1.0 },
  { "Arsie", 0.0 },
  { "Arslan", 1.0 },
  { "Arson", 1.0 },
  { "Arsula", 0.0 },
  { "Arsyn", 1.0 },
  { "Art", 0.9991518 },
  { "Arta", 0.011705685 },
  { "Artadius", 1.0 },
  { "Artan", 1.0 },
  { "Artasia", 0.0 },
  { "Artavia", 0.0 },
  { "Artavian", 1.0 },
  { "Artavion", 1.0 },
  { "Artavious", 1.0 },
  { "Artavis", 1.0 },
  { "Artavius", 1.0 },
  { "Artee", 0.7222222 },
  { "Arteen", 1.0 },
  { "Artelia", 0.0 },
  { "Artell", 0.5833333 },
  { "Artella", 0.0 },
  { "Artem", 1.0 },
  { "Artemas", 1.0 },
  { "Artemia", 0.0 },
  { "Artemio", 1.0 },
  { "Artemis", 0.13175675 },
  { "Artemisa", 0.0 },
  { "Artemisia", 0.0 },
  { "Artemiy", 1.0 },
  { "Artemus", 1.0 },
  { "Artemy", 1.0 },
  { "Artena", 0.0 },
  { "Artensia", 0.0 },
  { "Arter", 1.0 },
  { "Arteria", 0.0 },
  { "Artesha", 0.0 },
  { "Artesia", 0.0 },
  { "Artez", 1.0 },
  { "Arth", 1.0 },
  { "Artha", 0.17754173 },
  { "Arthalia", 0.0 },
  { "Arthar", 1.0 },
  { "Arthas", 1.0 },
  { "Arthea", 0.0 },
  { "Arthel", 1.0 },
  { "Arthelia", 0.0 },
  { "Arthell", 1.0 },
  { "Arthella", 0.0 },
  { "Arthemise", 0.0 },
  { "Arthena", 0.0 },
  { "Arthenia", 0.0 },
  { "Arther", 0.9906412 },
  { "Artherine", 0.0 },
  { "Arthetta", 0.0 },
  { "Arthi", 0.0 },
  { "Arthor", 1.0 },
  { "Arthur", 0.9939636 },
  { "Arthuree", 0.0 },
  { "Arthurene", 0.0 },
  { "Arthurine", 0.0 },
  { "Arthuro", 1.0 },
  { "Arthus", 1.0 },
  { "Arti", 0.03088803 },
  { "Artia", 0.0 },
  { "Artiana", 0.0 },
  { "Artice", 0.7037037 },
  { "Articia", 0.0 },
  { "Artie", 0.40362006 },
  { "Artimese", 0.0 },
  { "Artimus", 1.0 },
  { "Artin", 1.0 },
  { "Artina", 0.0 },
  { "Artis", 0.84607875 },
  { "Artisha", 0.0 },
  { "Artishia", 0.0 },
  { "Artist", 0.9744898 },
  { "Arto", 1.0 },
  { "Arton", 1.0 },
  { "Artorius", 1.0 },
  { "Artra", 0.0 },
  { "Artrell", 1.0 },
  { "Artresha", 0.0 },
  { "Artrice", 0.0 },
  { "Artricia", 0.0 },
  { "Artrina", 0.0 },
  { "Artur", 1.0 },
  { "Artura", 0.0 },
  { "Arturo", 0.99606246 },
  { "Arty", 1.0 },
  { "Artyom", 1.0 },
  { "Aruba", 0.0 },
  { "Arul", 1.0 },
  { "Arun", 1.0 },
  { "Aruna", 0.0 },
  { "Arunas", 1.0 },
  { "Arundhati", 0.0 },
  { "Arunima", 0.0 },
  { "Arush", 1.0 },
  { "Arusha", 0.0 },
  { "Arushi", 0.0 },
  { "Aruther", 1.0 },
  { "Aruthur", 1.0 },
  { "Arva", 0.0 },
  { "Arvada", 0.0 },
  { "Arval", 1.0 },
  { "Arvard", 1.0 },
  { "Arve", 1.0 },
  { "Arveda", 0.0 },
  { "Arveen", 0.0 },
  { "Arvel", 1.0 },
  { "Arvell", 1.0 },
  { "Arvella", 0.0 },
  { "Arvelle", 1.0 },
  { "Arven", 1.0 },
  { "Arvena", 0.0 },
  { "Arvene", 1.0 },
  { "Arvester", 1.0 },
  { "Arveta", 0.0 },
  { "Arvetta", 0.0 },
  { "Arvey", 1.0 },
  { "Arvi", 0.6666667 },
  { "Arvia", 0.0 },
  { "Arvid", 1.0 },
  { "Arvie", 0.63951933 },
  { "Arvil", 1.0 },
  { "Arvile", 1.0 },
  { "Arvill", 1.0 },
  { "Arvilla", 0.0 },
  { "Arville", 0.9856957 },
  { "Arvillia", 0.0 },
  { "Arvin", 1.0 },
  { "Arvina", 0.0 },
  { "Arvind", 1.0 },
  { "Arvine", 1.0 },
  { "Arvion", 1.0 },
  { "Arvis", 0.77283657 },
  { "Arvle", 1.0 },
  { "Arvo", 1.0 },
  { "Arvol", 1.0 },
  { "Arvon", 1.0 },
  { "Arvy", 1.0 },
  { "Arvydas", 1.0 },
  { "Arwa", 0.0 },
  { "Arwen", 0.0 },
  { "Arwilda", 0.0 },
  { "Arwilla", 0.0 },
  { "Arwin", 1.0 },
  { "Arwood", 1.0 },
  { "Arwyn", 0.0 },
  { "Arwynn", 0.0 },
  { "Ary", 0.46570972 },
  { "Arya", 0.17255083 },
  { "Aryaa", 0.0 },
  { "Aryaan", 1.0 },
  { "Aryadne", 0.0 },
  { "Aryah", 0.0 },
  { "Aryahi", 0.0 },
  { "Aryahna", 0.0 },
  { "Aryal", 0.0 },
  { "Aryam", 0.05703422 },
  { "Aryaman", 1.0 },
  { "Aryan", 0.9003734 },
  { "Aryana", 0.0 },
  { "Aryanah", 0.0 },
  { "Aryane", 0.0 },
  { "Aryani", 0.0 },
  { "Aryann", 0.0 },
  { "Aryanna", 0.0 },
  { "Aryannah", 0.0 },
  { "Aryanne", 0.0 },
  { "Aryanni", 0.0 },
  { "Aryash", 1.0 },
  { "Aryauna", 0.0 },
  { "Aryav", 1.0 },
  { "Aryaveer", 1.0 },
  { "Arybella", 0.0 },
  { "Arye", 1.0 },
  { "Aryeh", 0.9978485 },
  { "Aryel", 0.05 },
  { "Aryela", 0.0 },
  { "Aryell", 0.0 },
  { "Aryella", 0.0 },
  { "Aryelle", 0.0 },
  { "Aryen", 1.0 },
  { "Aryes", 1.0 },
  { "Aryhanna", 0.0 },
  { "Aryia", 0.0 },
  { "Aryiah", 0.0 },
  { "Aryian", 0.3939394 },
  { "Aryiana", 0.0 },
  { "Aryianna", 0.0 },
  { "Aryiel", 0.0 },
  { "Aryion", 1.0 },
  { "Aryk", 1.0 },
  { "Aryka", 0.0 },
  { "Arykah", 0.0 },
  { "Aryn", 0.12579282 },
  { "Arynn", 0.0 },
  { "Arynne", 0.0 },
  { "Aryo", 1.0 },
  { "Aryon", 0.7823129 },
  { "Aryona", 0.0 },
  { "Aryonna", 0.0 },
  { "Arys", 0.7222222 },
  { "Arysa", 0.0 },
  { "Aryss", 0.0 },
  { "Aryssa", 0.0 },
  { "Arysta", 0.0 },
  { "Aryton", 1.0 },
  { "Aryus", 1.0 },
  { "Arza", 1.0 },
  { "Arzel", 1.0 },
  { "Arzelia", 0.0 },
  { "Arzell", 0.9101124 },
  { "Arzella", 0.0 },
  { "Arzetta", 0.0 },
  { "Arzie", 1.0 },
  { "Arzilla", 0.0 },
  { "Arzjon", 1.0 },
  { "Arzo", 1.0 },
  { "Arzola", 0.0 },
  { "Arzona", 0.0 },
  { "Arzoo", 0.0 },
  { "Arzu", 0.0 },
  { "Asa", 0.9305827 },
  { "Asaad", 1.0 },
  { "Asaan", 1.0 },
  { "Asacia", 0.0 },
  { "Asad", 1.0 },
  { "Asada", 0.0 },
  { "Asadbek", 1.0 },
  { "Asadullah", 1.0 },
  { "Asae", 0.0 },
  { "Asael", 1.0 },
  { "Asaf", 1.0 },
  { "Asah", 1.0 },
  { "Asahel", 1.0 },
  { "Asahi", 1.0 },
  { "Asai", 1.0 },
  { "Asaiah", 0.86415094 },
  { "Asaiya", 0.0 },
  { "Asako", 0.0 },
  { "Asal", 0.0 },
  { "Asala", 0.0 },
  { "Asalah", 0.0 },
  { "Asalee", 0.0 },
  { "Asalia", 0.0 },
  { "Asami", 0.0 },
  { "Asan", 1.0 },
  { "Asana", 0.0 },
  { "Asani", 0.24545455 },
  { "Asantae", 1.0 },
  { "Asante", 0.8045574 },
  { "Asanti", 0.18390805 },
  { "Asao", 1.0 },
  { "Asaph", 1.0 },
  { "Asar", 1.0 },
  { "Asaria", 0.0 },
  { "Asasha", 0.0 },
  { "Asata", 0.0 },
  { "Asaun", 1.0 },
  { "Asauni", 0.0 },
  { "Asaya", 1.0 },
  { "Asayo", 0.0 },
  { "Asbel", 1.0 },
  { "Asberry", 1.0 },
  { "Asbery", 1.0 },
  { "Asbiel", 1.0 },
  { "Asbury", 1.0 },
  { "Ascencion", 0.84639496 },
  { "Ascension", 0.6388889 },
  { "Ascher", 1.0 },
  { "Aschley", 0.0 },
  { "Aschton", 1.0 },
  { "Asdrubal", 1.0 },
  { "Ase", 0.42857143 },
  { "Asean", 1.0 },
  { "Aseante", 1.0 },
  { "Aseda", 0.75 },
  { "Aseel", 0.113259666 },
  { "Aseelah", 0.0 },
  { "Aseem", 1.0 },
  { "Asees", 0.0 },
  { "Asela", 0.0 },
  { "Aselin", 0.0 },
  { "Asem", 1.0 },
  { "Asena", 0.0 },
  { "Asenat", 0.0 },
  { "Asenath", 0.0 },
  { "Asencion", 0.7755102 },
  { "Asenet", 0.0 },
  { "Aseneth", 0.0 },
  { "Asenith", 0.0 },
  { "Asension", 1.0 },
  { "Aser", 1.0 },
  { "Aseret", 0.0 },
  { "Aset", 0.0 },
  { "Asfia", 0.0 },
  { "Asfiya", 0.0 },
  { "Ash", 0.96544915 },
  { "Asha", 0.0042570173 },
  { "Ashaad", 1.0 },
  { "Ashaan", 1.0 },
  { "Ashad", 1.0 },
  { "Ashadieeyah", 0.0 },
  { "Ashae", 0.0 },
  { "Ashai", 0.5833333 },
  { "Ashaiya", 0.0 },
  { "Ashaki", 0.0 },
  { "Ashala", 0.0 },
  { "Ashalee", 0.0 },
  { "Ashaley", 0.0 },
  { "Ashalina", 0.0 },
  { "Ashalyn", 0.0 },
  { "Ashan", 1.0 },
  { "Ashana", 0.0 },
  { "Ashanae", 0.0 },
  { "Ashanda", 0.0 },
  { "Ashane", 0.0 },
  { "Ashanee", 0.0 },
  { "Ashani", 0.0 },
  { "Ashanique", 0.0 },
  { "Ashanna", 0.0 },
  { "Ashanni", 0.0 },
  { "Ashanta", 0.0099502485 },
  { "Ashantae", 0.0 },
  { "Ashantai", 0.0 },
  { "Ashantay", 0.0 },
  { "Ashante", 0.10604192 },
  { "Ashantee", 0.0 },
  { "Ashantey", 0.0 },
  { "Ashanti", 0.06229025 },
  { "Ashantia", 0.0 },
  { "Ashantiana", 0.0 },
  { "Ashantianna", 0.0 },
  { "Ashantie", 0.0 },
  { "Ashantii", 0.0 },
  { "Ashantis", 0.0 },
  { "Ashanty", 0.0 },
  { "Ashar", 1.0 },
  { "Ashara", 0.0 },
  { "Asharee", 0.0 },
  { "Ashari", 0.0 },
  { "Asharia", 0.0 },
  { "Ashariah", 0.0 },
  { "Asharie", 0.0 },
  { "Asharra", 0.0 },
  { "Asharri", 0.0 },
  { "Ashaud", 1.0 },
  { "Ashaun", 1.0 },
  { "Ashauna", 0.0 },
  { "Ashaunie", 0.0 },
  { "Ashaunta", 0.0 },
  { "Ashauntae", 0.0 },
  { "Ashaunte", 0.0 },
  { "Ashauntee", 0.0 },
  { "Ashaunti", 0.0 },
  { "Ashaureah", 0.0 },
  { "Ashauria", 0.0 },
  { "Ashawn", 1.0 },
  { "Ashawna", 0.0 },
  { "Ashawni", 0.0 },
  { "Ashawnti", 0.0 },
  { "Ashay", 0.46 },
  { "Ashaya", 0.0 },
  { "Ashayla", 0.0 },
  { "Ashaz", 1.0 },
  { "Ashburn", 1.0 },
  { "Ashby", 0.6995808 },
  { "Ashden", 0.94074076 },
  { "Ashdon", 1.0 },
  { "Ashdyn", 0.8378378 },
  { "Ashe", 0.72649574 },
  { "Ashea", 0.0 },
  { "Ashee", 0.0 },
  { "Asheena", 0.0 },
  { "Ashelee", 0.0 },
  { "Asheley", 0.0 },
  { "Asheli", 0.0 },
  { "Ashelle", 0.0 },
  { "Ashelly", 0.0 },
  { "Ashely", 0.010759299 },
  { "Ashelyn", 0.0 },
  { "Ashelynn", 0.0 },
  { "Asher", 0.9840991 },
  { "Ashera", 0.0 },
  { "Asherah", 0.0 },
  { "Asheton", 0.55932206 },
  { "Ashey", 0.0 },
  { "Ashford", 1.0 },
  { "Ashgan", 0.0 },
  { "Ashi", 0.0 },
  { "Ashia", 0.0 },
  { "Ashiah", 0.0 },
  { "Ashiana", 0.0 },
  { "Ashianna", 0.0 },
  { "Ashika", 0.0 },
  { "Ashilee", 0.0 },
  { "Ashiley", 0.0 },
  { "Ashim", 1.0 },
  { "Ashima", 0.0 },
  { "Ashini", 0.0 },
  { "Ashir", 1.0 },
  { "Ashira", 0.0 },
  { "Ashirah", 0.0 },
  { "Ashish", 1.0 },
  { "Ashita", 0.0 },
  { "Ashiya", 0.0 },
  { "Ashiyah", 0.0 },
  { "Ashja", 0.0 },
  { "Ashka", 0.0 },
  { "Ashkan", 1.0 },
  { "Ashkii", 1.0 },
  { "Ashkon", 1.0 },
  { "Ashla", 0.0 },
  { "Ashlae", 0.0 },
  { "Ashlan", 0.038647342 },
  { "Ashland", 0.16912752 },
  { "Ashlay", 0.0 },
  { "Ashle", 0.0 },
  { "Ashlea", 0.0 },
  { "Ashleah", 0.0 },
  { "Ashlee", 0.005950322 },
  { "Ashleeann", 0.0 },
  { "Ashleen", 0.0 },
  { "Ashleey", 0.0 },
  { "Ashleh", 0.0 },
  { "Ashlei", 0.0 },
  { "Ashleigh", 0.0026262498 },
  { "Ashlely", 0.0 },
  { "Ashlen", 0.012782694 },
  { "Ashley", 0.018207176 },
  { "Ashleyann", 0.0 },
  { "Ashleyanne", 0.0 },
  { "Ashleyelizabeth", 0.0 },
  { "Ashleykate", 0.0 },
  { "Ashleylynn", 0.0 },
  { "Ashleymae", 0.0 },
  { "Ashleymarie", 0.0 },
  { "Ashleyn", 0.0 },
  { "Ashleynicole", 0.0 },
  { "Ashleyrae", 0.0 },
  { "Ashleyrose", 0.0 },
  { "Ashli", 0.0 },
  { "Ashlie", 0.00050095184 },
  { "Ashliegh", 0.0 },
  { "Ashlii", 0.0 },
  { "Ashlin", 0.06087981 },
  { "Ashling", 0.0 },
  { "Ashlinn", 0.0 },
  { "Ashliy", 0.0 },
  { "Ashlley", 0.0 },
  { "Ashlly", 0.0 },
  { "Ashlon", 0.06849315 },
  { "Ashly", 0.010992971 },
  { "Ashlye", 0.0 },
  { "Ashlyn", 0.0012220789 },
  { "Ashlynd", 0.0 },
  { "Ashlyne", 0.0 },
  { "Ashlynn", 0.0005404675 },
  { "Ashlynne", 0.0 },
  { "Ashmeet", 0.0 },
  { "Ashmi", 0.0 },
  { "Ashmit", 1.0 },
  { "Ashmita", 0.0 },
  { "Ashna", 0.0 },
  { "Ashneel", 1.0 },
  { "Ashney", 0.0 },
  { "Ashni", 0.0 },
  { "Ashok", 1.0 },
  { "Ashoka", 1.0 },
  { "Ashon", 1.0 },
  { "Ashonda", 0.0 },
  { "Ashonna", 0.0 },
  { "Ashonta", 0.0 },
  { "Ashonte", 0.0 },
  { "Ashonti", 0.0 },
  { "Ashor", 1.0 },
  { "Ashot", 1.0 },
  { "Ashraf", 1.0 },
  { "Ashrit", 1.0 },
  { "Ashrita", 0.0 },
  { "Ashrith", 1.0 },
  { "Ashritha", 0.0 },
  { "Ashriya", 0.0 },
  { "Ashston", 1.0 },
  { "Ashtan", 0.4892704 },
  { "Ashten", 0.4935662 },
  { "Ashtian", 1.0 },
  { "Ashtin", 0.5221596 },
  { "Ashtion", 1.0 },
  { "Ashton", 0.77949256 },
  { "Ashtun", 1.0 },
  { "Ashtyn", 0.19058536 },
  { "Ashtynn", 0.031496063 },
  { "Ashuna", 0.0 },
  { "Ashunta", 0.0 },
  { "Ashunte", 0.0 },
  { "Ashunti", 0.0 },
  { "Ashur", 1.0 },
  { "Ashura", 0.0 },
  { "Ashutosh", 1.0 },
  { "Ashvath", 1.0 },
  { "Ashvi", 0.0 },
  { "Ashvik", 1.0 },
  { "Ashvika", 0.0 },
  { "Ashvin", 1.0 },
  { "Ashwaq", 0.0 },
  { "Ashwath", 1.0 },
  { "Ashwika", 0.0 },
  { "Ashwin", 1.0 },
  { "Ashwini", 0.0 },
  { "Ashya", 0.0 },
  { "Ashyah", 0.0 },
  { "Ashyia", 0.0 },
  { "Ashyiah", 0.0 },
  { "Ashyla", 0.0 },
  { "Ashyr", 1.0 },
  { "Ashyra", 0.0 },
  { "Ashyria", 0.0 },
  { "Asia", 0.006265584 },
  { "Asiah", 0.10880829 },
  { "Asialyn", 0.0 },
  { "Asialynn", 0.0 },
  { "Asiamarie", 0.0 },
  { "Asian", 0.4074074 },
  { "Asiana", 0.0 },
  { "Asianae", 0.0 },
  { "Asianah", 0.0 },
  { "Asianay", 0.0 },
  { "Asiane", 0.0 },
  { "Asianna", 0.0 },
  { "Asiaonna", 0.0 },
  { "Asie", 0.5263158 },
  { "Asiel", 0.98540145 },
  { "Asier", 1.0 },
  { "Asif", 1.0 },
  { "Asiha", 0.0 },
  { "Asija", 0.0 },
  { "Asijah", 0.0 },
  { "Asil", 0.47619048 },
  { "Asilee", 0.0 },
  { "Asim", 1.0 },
  { "Asimina", 0.0 },
  { "Asir", 1.0 },
  { "Asira", 0.0 },
  { "Asire", 1.0 },
  { "Asis", 1.0 },
  { "Asisha", 0.0 },
  { "Asiya", 0.0 },
  { "Asiyah", 0.03109328 },
  { "Asja", 0.0 },
  { "Asjah", 0.0 },
  { "Asjha", 0.0 },
  { "Asjia", 0.0 },
  { "Askari", 1.0 },
  { "Askia", 1.0 },
  { "Aslam", 1.0 },
  { "Aslan", 0.8982229 },
  { "Aslean", 0.0 },
  { "Aslee", 0.0 },
  { "Asleigh", 0.0 },
  { "Asley", 0.0 },
  { "Aslhey", 0.0 },
  { "Asli", 0.0 },
  { "Aslin", 0.0 },
  { "Asly", 0.0 },
  { "Aslyn", 0.0 },
  { "Aslynn", 0.0 },
  { "Asma", 0.0 },
  { "Asmaa", 0.0 },
  { "Asmahan", 0.0 },
  { "Asmar", 1.0 },
  { "Asmara", 0.0 },
  { "Asmi", 0.0 },
  { "Asmir", 1.0 },
  { "Asmita", 0.0 },
  { "Asmitha", 0.0 },
  { "Asna", 0.0 },
  { "Ason", 0.875 },
  { "Asoni", 0.0 },
  { "Asonte", 0.625 },
  { "Aspacia", 0.0 },
  { "Aspasia", 0.0 },
  { "Aspen", 0.08812448 },
  { "Aspenn", 0.0 },
  { "Aspin", 0.0 },
  { "Aspyn", 0.006793478 },
  { "Aspynn", 0.0 },
  { "Asra", 0.0 },
  { "Asrar", 0.0 },
  { "Asriel", 0.8095238 },
  { "Assa", 0.0 },
  { "Assad", 1.0 },
  { "Assael", 1.0 },
  { "Assaf", 1.0 },
  { "Assan", 1.0 },
  { "Assandra", 0.0 },
  { "Assane", 1.0 },
  { "Assante", 1.0 },
  { "Assata", 0.0 },
  { "Asser", 1.0 },
  { "Assetou", 0.0 },
  { "Assia", 0.0 },
  { "Assiatou", 0.0 },
  { "Assie", 0.0 },
  { "Assitan", 0.0 },
  { "Assma", 0.0 },
  { "Assunta", 0.0 },
  { "Assyria", 0.0 },
  { "Asta", 0.0 },
  { "Astacia", 0.0 },
  { "Astara", 0.0 },
  { "Astaria", 0.0 },
  { "Astasia", 0.0 },
  { "Astella", 0.0 },
  { "Asten", 0.7894737 },
  { "Aster", 0.3078759 },
  { "Asteria", 0.0 },
  { "Astha", 0.0 },
  { "Asthon", 1.0 },
  { "Asti", 0.0 },
  { "Astin", 0.7048872 },
  { "Astley", 1.0 },
  { "Aston", 0.9325897 },
  { "Astor", 1.0 },
  { "Astoria", 0.0 },
  { "Astou", 0.0 },
  { "Astra", 0.0 },
  { "Astraea", 0.0 },
  { "Astraeus", 1.0 },
  { "Astraya", 0.0 },
  { "Astrea", 0.0 },
  { "Astreia", 0.0 },
  { "Astri", 0.0 },
  { "Astria", 0.0 },
  { "Astrid", 0.0 },
  { "Astride", 0.0 },
  { "Astrin", 0.0 },
  { "Astro", 1.0 },
  { "Astry", 0.0 },
  { "Astryd", 0.0 },
  { "Astyn", 0.43636364 },
  { "Asucena", 0.0 },
  { "Asuka", 0.0 },
  { "Asul", 0.0 },
  { "Asuna", 0.0 },
  { "Asuncion", 0.3838384 },
  { "Asunta", 0.0 },
  { "Asusena", 0.0 },
  { "Asuzena", 0.0 },
  { "Aswad", 1.0 },
  { "Aswath", 1.0 },
  { "Aswell", 1.0 },
  { "Aswin", 1.0 },
  { "Asya", 0.0 },
  { "Asyah", 0.0 },
  { "Asyana", 0.0 },
  { "Asyia", 0.0 },
  { "Asyiah", 0.0 },
  { "Asyra", 0.0 },
  { "Asyria", 0.0 },
  { "Aszia", 0.0 },
  { "Aszure", 0.0 },
  { "Ata", 1.0 },
  { "Atahan", 1.0 },
  { "Atai", 1.0 },
  { "Ataja", 0.0 },
  { "Atakan", 1.0 },
  { "Atal", 1.0 },
  { "Atalanta", 0.0 },
  { "Atalaya", 0.0 },
  { "Atalee", 0.0 },
  { "Atalia", 0.0 },
  { "Ataliah", 0.0 },
  { "Atalie", 0.0 },
  { "Ataly", 0.0 },
  { "Atalya", 0.0 },
  { "Atanacio", 1.0 },
  { "Atanya", 0.0 },
  { "Atara", 0.0 },
  { "Atarah", 0.0 },
  { "Atari", 0.62204725 },
  { "Atasha", 0.0 },
  { "Atasia", 0.0 },
  { "Atavia", 0.0 },
  { "Atavion", 1.0 },
  { "Ataya", 0.0 },
  { "Ateeb", 1.0 },
  { "Atef", 1.0 },
  { "Atem", 1.0 },
  { "Aten", 1.0 },
  { "Atena", 0.0 },
  { "Atenas", 0.0 },
  { "Atenea", 0.0 },
  { "Ater", 1.0 },
  { "Atesha", 0.0 },
  { "Atha", 0.06428572 },
  { "Athala", 0.0 },
  { "Athalee", 0.0 },
  { "Athaleen", 0.0 },
  { "Athalene", 0.0 },
  { "Athalia", 0.0 },
  { "Athaliah", 0.0 },
  { "Athalie", 0.0 },
  { "Athan", 1.0 },
  { "Athanasia", 0.0 },
  { "Athanasios", 1.0 },
  { "Athanasius", 1.0 },
  { "Athaniel", 1.0 },
  { "Atharv", 1.0 },
  { "Atharva", 1.0 },
  { "Athea", 0.0 },
  { "Atheana", 0.0 },
  { "Atheena", 0.0 },
  { "Atheer", 0.0 },
  { "Athel", 0.89221555 },
  { "Athelene", 0.0 },
  { "Athelia", 0.0 },
  { "Atheline", 0.0 },
  { "Athen", 1.0 },
  { "Athena", 0.0 },
  { "Athenamarie", 0.0 },
  { "Athenarose", 0.0 },
  { "Athene", 0.0 },
  { "Athenea", 0.0 },
  { "Athenia", 0.0 },
  { "Athenna", 0.0 },
  { "Athens", 0.8375 },
  { "Ather", 0.5641026 },
  { "Atherine", 0.0 },
  { "Atherton", 1.0 },
  { "Athie", 0.0 },
  { "Athina", 0.0 },
  { "Athira", 0.0 },
  { "Athlee", 0.0 },
  { "Athleen", 0.0 },
  { "Athlene", 0.0 },
  { "Athlyn", 0.0 },
  { "Athol", 1.0 },
  { "Athon", 1.0 },
  { "Athony", 1.0 },
  { "Athos", 1.0 },
  { "Athryn", 0.0 },
  { "Athulya", 0.0 },
  { "Athylene", 0.0 },
  { "Athyna", 0.0 },
  { "Athziri", 0.0 },
  { "Athziry", 0.0 },
  { "Atia", 0.0 },
  { "Atiana", 0.0 },
  { "Atianna", 0.0 },
  { "Atiba", 0.98655915 },
  { "Atie", 0.0 },
  { "Atif", 1.0 },
  { "Atifa", 0.0 },
  { "Atiim", 1.0 },
  { "Atika", 0.0 },
  { "Atiksh", 1.0 },
  { "Atilano", 1.0 },
  { "Atilio", 1.0 },
  { "Atilla", 1.0 },
  { "Atina", 0.0 },
  { "Atinuke", 0.0 },
  { "Atira", 0.0 },
  { "Atisha", 0.0 },
  { "Atit", 1.0 },
  { "Ativa", 0.0 },
  { "Atiya", 0.0 },
  { "Atiyah", 0.0 },
  { "Atiyana", 0.0 },
  { "Atiyanna", 0.0 },
  { "Atiyya", 0.0 },
  { "Atiyyah", 0.0 },
  { "Atlai", 1.0 },
  { "Atlanta", 0.0 },
  { "Atlantis", 0.038812786 },
  { "Atlas", 0.9480045 },
  { "Atlean", 0.0 },
  { "Atlee", 0.8176964 },
  { "Atleigh", 0.0 },
  { "Atley", 0.5063613 },
  { "Atline", 0.0 },
  { "Atlis", 1.0 },
  { "Atnhony", 1.0 },
  { "Ato", 1.0 },
  { "Atoli", 0.0 },
  { "Atom", 1.0 },
  { "Atong", 0.0 },
  { "Atonia", 0.0 },
  { "Atonio", 1.0 },
  { "Atonya", 0.0 },
  { "Atoria", 0.0 },
  { "Atosha", 0.0 },
  { "Atoya", 0.0 },
  { "Atravion", 1.0 },
  { "Atrayu", 1.0 },
  { "Atreal", 1.0 },
  { "Atreau", 1.0 },
  { "Atreju", 1.0 },
  { "Atrell", 1.0 },
  { "Atreo", 1.0 },
  { "Atreu", 1.0 },
  { "Atreus", 1.0 },
  { "Atreya", 0.8181818 },
  { "Atreyu", 1.0 },
  { "Atrice", 0.0 },
  { "Atricia", 0.0 },
  { "Atrick", 1.0 },
  { "Atrina", 0.0 },
  { "Atsuko", 0.0 },
  { "Atsushi", 1.0 },
  { "Atta", 0.54545456 },
  { "Attache", 0.0 },
  { "Attalia", 0.0 },
  { "Attalie", 0.0 },
  { "Attallah", 0.0 },
  { "Atthew", 1.0 },
  { "Attia", 0.0 },
  { "Attiana", 0.0 },
  { "Attica", 0.0 },
  { "Atticus", 0.99508566 },
  { "Attie", 0.0 },
  { "Attikus", 1.0 },
  { "Attila", 1.0 },
  { "Attilio", 1.0 },
  { "Attis", 1.0 },
  { "Attison", 0.0 },
  { "Attiya", 0.0 },
  { "Attiyya", 0.0 },
  { "Attley", 1.0 },
  { "Attoria", 0.0 },
  { "Atul", 1.0 },
  { "Atward", 1.0 },
  { "Atwell", 1.0 },
  { "Atwood", 1.0 },
  { "Atyana", 0.0 },
  { "Atyanna", 0.0 },
  { "Atyia", 0.0 },
  { "Atzel", 1.0 },
  { "Atzhiri", 0.0 },
  { "Atzhiry", 0.0 },
  { "Atzi", 0.0 },
  { "Atzimba", 0.0 },
  { "Atzin", 0.8267717 },
  { "Atziri", 0.0 },
  { "Atziry", 0.0 },
  { "Auberon", 1.0 },
  { "Aubert", 1.0 },
  { "Aubery", 0.85982907 },
  { "Aubie", 0.82758623 },
  { "Aubin", 0.8 },
  { "Aubra", 0.89337176 },
  { "Aubray", 0.0 },
  { "Aubre", 0.020576132 },
  { "Aubrea", 0.0 },
  { "Aubreah", 0.0 },
  { "Aubreana", 0.0 },
  { "Aubreanna", 0.0 },
  { "Aubreauna", 0.0 },
  { "Aubree", 0.0012062726 },
  { "Aubreeana", 0.0 },
  { "Aubreeann", 0.0 },
  { "Aubreeanna", 0.0 },
  { "Aubreella", 0.0 },
  { "Aubreelynn", 0.0 },
  { "Aubreerose", 0.0 },
  { "Aubrei", 0.0 },
  { "Aubreigh", 0.0 },
  { "Aubrelle", 0.0 },
  { "Aubren", 0.30555555 },
  { "Aubreona", 0.0 },
  { "Aubreonna", 0.0 },
  { "Aubrey", 0.22959164 },
  { "Aubreyana", 0.0 },
  { "Aubreyann", 0.0 },
  { "Aubreyanna", 0.0 },
  { "Aubreyella", 0.0 },
  { "Aubreyelle", 0.0 },
  { "Aubreyjo", 0.0 },
  { "Aubreylynn", 0.0 },
  { "Aubreyona", 0.0 },
  { "Aubreyonna", 0.0 },
  { "Aubreyrose", 0.0 },
  { "Aubri", 0.0 },
  { "Aubria", 0.0 },
  { "Aubriahna", 0.0 },
  { "Aubriana", 0.0 },
  { "Aubrianah", 0.0 },
  { "Aubriann", 0.0 },
  { "Aubrianna", 0.0 },
  { "Aubriannah", 0.0 },
  { "Aubrianne", 0.0 },
  { "Aubriauna", 0.0 },
  { "Aubriaunna", 0.0 },
  { "Aubrie", 0.0 },
  { "Aubrieana", 0.0 },
  { "Aubrieanna", 0.0 },
  { "Aubriee", 0.0 },
  { "Aubriegh", 0.0 },
  { "Aubriel", 0.0 },
  { "Aubriela", 0.0 },
  { "Aubriella", 0.0 },
  { "Aubrielle", 0.0 },
  { "Aubrii", 0.0 },
  { "Aubrilyn", 0.0 },
  { "Aubrin", 0.0 },
  { "Aubrina", 0.0 },
  { "Aubriona", 0.0 },
  { "Aubrionna", 0.0 },
  { "Aubriyana", 0.0 },
  { "Aubry", 0.3635321 },
  { "Aubryana", 0.0 },
  { "Aubryanna", 0.0 },
  { "Aubrye", 0.0 },
  { "Aubryella", 0.0 },
  { "Aubryelle", 0.0 },
  { "Aubryn", 0.0 },
  { "Aubrynn", 0.0 },
  { "Auburn", 0.399653 },
  { "Aubyn", 0.0 },
  { "Aud", 1.0 },
  { "Auda", 0.0 },
  { "Auddie", 1.0 },
  { "Aude", 0.0 },
  { "Audean", 0.0 },
  { "Audee", 0.0 },
  { "Audel", 1.0 },
  { "Audelia", 0.0 },
  { "Audelio", 1.0 },
  { "Audeliz", 1.0 },
  { "Audell", 0.0 },
  { "Audella", 0.0 },
  { "Auden", 0.7044649 },
  { "Audene", 0.0 },
  { "Audery", 0.0 },
  { "Audette", 0.0 },
  { "Audey", 1.0 },
  { "Audi", 0.38812786 },
  { "Audia", 0.0 },
  { "Audiana", 0.0 },
  { "Audianna", 0.0 },
  { "Audie", 0.642743 },
  { "Audiel", 1.0 },
  { "Audin", 1.0 },
  { "Audine", 0.0 },
  { "Audio", 1.0 },
  { "Audis", 1.0 },
  { "Audley", 0.9927954 },
  { "Audna", 0.0 },
  { "Audon", 1.0 },
  { "Audra", 0.007840429 },
  { "Audrae", 0.0 },
  { "Audranna", 0.0 },
  { "Audray", 0.0 },
  { "Audraya", 0.0 },
  { "Audre", 0.046511628 },
  { "Audrea", 0.0 },
  { "Audreana", 0.0 },
  { "Audreanna", 0.0 },
  { "Audreauna", 0.0 },
  { "Audree", 0.0 },
  { "Audreena", 0.0 },
  { "Audrei", 0.0 },
  { "Audreigh", 0.0 },
  { "Audrena", 0.0 },
  { "Audrene", 0.0 },
  { "Audreona", 0.0 },
  { "Audresha", 0.0 },
  { "Audrey", 0.010494992 },
  { "Audreya", 0.0 },
  { "Audreyana", 0.0 },
  { "Audreyann", 0.0 },
  { "Audreyanna", 0.0 },
  { "Audreyna", 0.0 },
  { "Audreyonna", 0.0 },
  { "Audreyrose", 0.0 },
  { "Audri", 0.0 },
  { "Audria", 0.0 },
  { "Audriana", 0.0 },
  { "Audrianna", 0.0 },
  { "Audrianne", 0.0 },
  { "Audriauna", 0.0 },
  { "Audriaunna", 0.0 },
  { "Audric", 1.0 },
  { "Audrick", 1.0 },
  { "Audrie", 0.0013758943 },
  { "Audrieana", 0.0 },
  { "Audrieanna", 0.0 },
  { "Audriel", 0.0 },
  { "Audriella", 0.0 },
  { "Audrielle", 0.0 },
  { "Audriena", 0.0 },
  { "Audrienna", 0.0 },
  { "Audrienne", 0.0 },
  { "Audrii", 0.0 },
  { "Audrina", 0.0 },
  { "Audrinah", 0.0 },
  { "Audrinna", 0.0 },
  { "Audriona", 0.0 },
  { "Audrionna", 0.0 },
  { "Audris", 0.0 },
  { "Audrone", 0.0 },
  { "Audry", 0.112090394 },
  { "Audryana", 0.0 },
  { "Audryanna", 0.0 },
  { "Audryna", 0.0 },
  { "Audrynna", 0.0 },
  { "Audvik", 1.0 },
  { "Audwin", 1.0 },
  { "Audy", 0.9589041 },
  { "Audyn", 0.078125 },
  { "Auggie", 1.0 },
  { "Augie", 0.99134946 },
  { "Augus", 1.0 },
  { "August", 0.9202088 },
  { "Augusta", 0.09388671 },
  { "Augustas", 1.0 },
  { "Auguste", 0.9111111 },
  { "Augusten", 1.0 },
  { "Auguster", 1.0 },
  { "Augustin", 1.0 },
  { "Augustina", 0.0 },
  { "Augustine", 0.7746022 },
  { "Augustino", 1.0 },
  { "Augustis", 1.0 },
  { "Augusto", 1.0 },
  { "Augustus", 0.99935794 },
  { "Augustyn", 1.0 },
  { "Auja", 0.0 },
  { "Aujah", 0.0 },
  { "Aujanae", 0.0 },
  { "Aujanai", 0.0 },
  { "Aujane", 0.0 },
  { "Aukai", 1.0 },
  { "Aukeem", 1.0 },
  { "Aukievah", 0.0 },
  { "Aul", 1.0 },
  { "Aula", 0.0 },
  { "Aulani", 0.0 },
  { "Aulbrey", 0.0 },
  { "Aulda", 0.0 },
  { "Aulden", 1.0 },
  { "Aulelei", 0.0 },
  { "Aulene", 0.0 },
  { "Aulii", 0.0 },
  { "Auline", 0.0 },
  { "Aulona", 0.0 },
  { "Aulora", 0.0 },
  { "Aulton", 1.0 },
  { "Aum", 1.0 },
  { "Aumi", 1.0 },
  { "Aumya", 0.0 },
  { "Aun", 1.0 },
  { "Auna", 0.0 },
  { "Aunah", 0.0 },
  { "Aunalee", 0.0 },
  { "Aunaleigh", 0.0 },
  { "Aundra", 0.6603625 },
  { "Aundrae", 1.0 },
  { "Aundray", 1.0 },
  { "Aundraya", 0.0 },
  { "Aundre", 1.0 },
  { "Aundrea", 0.037614297 },
  { "Aundreah", 0.0 },
  { "Aundreia", 0.0 },
  { "Aundrey", 0.9844237 },
  { "Aundreya", 0.0 },
  { "Aundria", 0.0 },
  { "Aune", 0.0 },
  { "Auner", 1.0 },
  { "Aunesti", 0.0 },
  { "Aunesty", 0.0 },
  { "Aung", 1.0 },
  { "Aungelique", 0.0 },
  { "Auni", 0.0 },
  { "Aunica", 0.0 },
  { "Aunika", 0.0 },
  { "Aunisti", 0.0 },
  { "Aunisty", 0.0 },
  { "Auniya", 0.0 },
  { "Aunjel", 0.0 },
  { "Aunna", 0.0 },
  { "Aunnika", 0.0 },
  { "Aunya", 0.0 },
  { "Aunyae", 0.0 },
  { "Aunye", 0.0 },
  { "Aura", 0.0 },
  { "Aurabella", 0.0 },
  { "Auralee", 0.0 },
  { "Auralia", 0.0 },
  { "Aurash", 1.0 },
  { "Auraya", 0.0 },
  { "Aurbree", 0.0 },
  { "Aurea", 0.0 },
  { "Aureana", 0.0 },
  { "Aureanna", 0.0 },
  { "Auree", 0.0 },
  { "Aurel", 0.8428571 },
  { "Aurele", 1.0 },
  { "Aureli", 0.0 },
  { "Aurelia", 0.0 },
  { "Aureliana", 0.0 },
  { "Aureliano", 1.0 },
  { "Aurelie", 0.0 },
  { "Aurelio", 0.99910086 },
  { "Aurelius", 1.0 },
  { "Aurella", 0.0 },
  { "Aurellia", 0.0 },
  { "Aurelya", 0.0 },
  { "Auren", 0.61165047 },
  { "Aureo", 1.0 },
  { "Aureon", 0.0 },
  { "Auretta", 0.0 },
  { "Aureus", 1.0 },
  { "Aurey", 0.0 },
  { "Auri", 0.057803467 },
  { "Auria", 0.0 },
  { "Auriah", 0.0 },
  { "Aurian", 0.3529412 },
  { "Auriana", 0.0 },
  { "Aurianna", 0.0 },
  { "Auric", 1.0 },
  { "Aurick", 1.0 },
  { "Aurie", 0.0 },
  { "Auriel", 0.054758802 },
  { "Auriella", 0.0 },
  { "Aurielle", 0.0 },
  { "Aurieona", 0.0 },
  { "Aurik", 1.0 },
  { "Auril", 0.0 },
  { "Aurilla", 0.0 },
  { "Aurin", 1.0 },
  { "Aurion", 0.44444445 },
  { "Auriona", 0.0 },
  { "Aurionna", 0.0 },
  { "Auriya", 0.0 },
  { "Auriyah", 0.0 },
  { "Auriyana", 0.0 },
  { "Auroara", 0.0 },
  { "Aurohom", 1.0 },
  { "Auron", 1.0 },
  { "Aurora", 0.001006226 },
  { "Aurorah", 0.0 },
  { "Auroralee", 0.0 },
  { "Auroralynn", 0.0 },
  { "Aurore", 0.0 },
  { "Auroura", 0.0 },
  { "Aurra", 0.0 },
  { "Aurther", 1.0 },
  { "Aurthor", 1.0 },
  { "Aurthur", 1.0 },
  { "Aury", 0.046728972 },
  { "Auryana", 0.0 },
  { "Auryanna", 0.0 },
  { "Auryn", 0.8 },
  { "Ausar", 1.0 },
  { "Ausbon", 1.0 },
  { "Ausby", 1.0 },
  { "Ausencio", 1.0 },
  { "Auset", 0.0 },
  { "Ausha", 0.0 },
  { "Ausia", 0.0 },
  { "Ausie", 1.0 },
  { "Ausitn", 1.0 },
  { "Austa", 0.0 },
  { "Austan", 1.0 },
  { "Austeja", 0.0 },
  { "Austen", 0.90684897 },
  { "Austi", 0.0 },
  { "Austie", 0.0 },
  { "Austin", 0.9891568 },
  { "Austina", 0.0 },
  { "Austine", 0.47663552 },
  { "Austinjames", 1.0 },
  { "Austinjohn", 1.0 },
  { "Austinlee", 1.0 },
  { "Austinmichael", 1.0 },
  { "Austinn", 1.0 },
  { "Austintyler", 1.0 },
  { "Auston", 1.0 },
  { "Australia", 0.03206997 },
  { "Austreberto", 1.0 },
  { "Austria", 0.0 },
  { "Austun", 1.0 },
  { "Austyn", 0.61420214 },
  { "Austynn", 0.0 },
  { "Autavia", 0.0 },
  { "Authar", 1.0 },
  { "Auther", 0.9928149 },
  { "Autherine", 0.0 },
  { "Author", 1.0 },
  { "Authur", 1.0 },
  { "Autiana", 0.0 },
  { "Autianna", 0.0 },
  { "Autie", 0.15068494 },
  { "Autin", 1.0 },
  { "Autis", 1.0 },
  { "Autmn", 0.0 },
  { "Autom", 0.0 },
  { "Automn", 0.0 },
  { "Autra", 0.0 },
  { "Autrey", 0.9102564 },
  { "Autry", 0.8516566 },
  { "Auttum", 0.0 },
  { "Auttumn", 0.0 },
  { "Autum", 0.0 },
  { "Autume", 0.0 },
  { "Autumm", 0.0 },
  { "Autumn", 0.0023095675 },
  { "Autumne", 0.0 },
  { "Autumnn", 0.0 },
  { "Autumnrose", 0.0 },
  { "Autunm", 0.0 },
  { "Auturo", 1.0 },
  { "Autym", 0.0 },
  { "Autymn", 0.0 },
  { "Autzen", 1.0 },
  { "Auviana", 0.0 },
  { "Auvianna", 0.0 },
  { "Auzhane", 0.0 },
  { "Auzie", 1.0 },
  { "Av", 1.0 },
  { "Ava", 0.0011870926 },
  { "Avaa", 0.0 },
  { "Avaah", 0.0 },
  { "Avaan", 1.0 },
  { "Avaani", 0.0 },
  { "Avabella", 0.0 },
  { "Avabelle", 0.0 },
  { "Avaclaire", 0.0 },
  { "Avacyn", 0.0 },
  { "Avadna", 0.0 },
  { "Avae", 0.0 },
  { "Avaeah", 0.0 },
  { "Avaeh", 0.0 },
  { "Avaelizabeth", 0.0 },
  { "Avagail", 0.0 },
  { "Avagrace", 0.0 },
  { "Avah", 0.0 },
  { "Avahlyn", 0.0 },
  { "Avahlynn", 0.0 },
  { "Avaia", 0.0 },
  { "Avaiah", 0.0 },
  { "Avaiya", 0.0 },
  { "Avaiyah", 0.0 },
  { "Avajames", 0.0 },
  { "Avajane", 0.0 },
  { "Avajo", 0.0 },
  { "Avajoy", 0.0 },
  { "Avakate", 0.0 },
  { "Avala", 0.0 },
  { "Avalanna", 0.0 },
  { "Avalea", 0.0 },
  { "Avalee", 0.0 },
  { "Avaleen", 0.0 },
  { "Avaleena", 0.0 },
  { "Avalei", 0.0 },
  { "Avaleigh", 0.0 },
  { "Avalena", 0.0 },
  { "Avalene", 0.0 },
  { "Avaley", 0.0 },
  { "Avali", 0.0 },
  { "Avalia", 0.0 },
  { "Avalie", 0.0 },
  { "Avaliese", 0.0 },
  { "Avalin", 0.0 },
  { "Avalina", 0.0 },
  { "Avaline", 0.0 },
  { "Avalise", 0.0 },
  { "Avalon", 0.009737099 },
  { "Avalyn", 0.0 },
  { "Avalyna", 0.0 },
  { "Avalyne", 0.0 },
  { "Avalynn", 0.0 },
  { "Avalynne", 0.0 },
  { "Avalyse", 0.0 },
  { "Avamae", 0.0 },
  { "Avamaria", 0.0 },
  { "Avamarie", 0.0 },
  { "Avamay", 0.0 },
  { "Avamonroe", 0.0 },
  { "Avan", 0.93093526 },
  { "Avana", 0.0 },
  { "Avanah", 0.0 },
  { "Avander", 1.0 },
  { "Avaneesh", 1.0 },
  { "Avanel", 0.0 },
  { "Avanell", 0.0 },
  { "Avanelle", 0.0 },
  { "Avangaline", 0.0 },
  { "Avangelene", 0.0 },
  { "Avangelia", 0.0 },
  { "Avangelina", 0.0 },
  { "Avangeline", 0.0 },
  { "Avani", 0.007518797 },
  { "Avanicole", 0.0 },
  { "Avanish", 1.0 },
  { "Avanna", 0.0 },
  { "Avannah", 0.0 },
  { "Avanni", 0.0 },
  { "Avant", 1.0 },
  { "Avanta", 1.0 },
  { "Avantae", 1.0 },
  { "Avantay", 1.0 },
  { "Avante", 0.9494008 },
  { "Avanthi", 0.0 },
  { "Avanthika", 0.0 },
  { "Avanti", 0.14942528 },
  { "Avantika", 0.0 },
  { "Avanya", 0.0 },
  { "Avarae", 0.0 },
  { "Avard", 1.0 },
  { "Avaree", 0.0 },
  { "Avareigh", 0.0 },
  { "Avarey", 0.0 },
  { "Avari", 0.11646587 },
  { "Avarie", 0.0 },
  { "Avariella", 0.0 },
  { "Avarielle", 0.0 },
  { "Avarose", 0.0 },
  { "Avary", 0.0649259 },
  { "Avash", 1.0 },
  { "Avasophia", 0.0 },
  { "Avaughn", 1.0 },
  { "Avaya", 0.0 },
  { "Avayah", 0.0 },
  { "Avayiah", 0.0 },
  { "Avayla", 0.0 },
  { "Ave", 0.0 },
  { "Avea", 0.0 },
  { "Aveah", 0.0 },
  { "Aveaha", 0.0 },
  { "Aveana", 0.0 },
  { "Aveanna", 0.0 },
  { "Aveda", 0.0 },
  { "Avedis", 1.0 },
  { "Avee", 0.0 },
  { "Aveen", 0.0 },
  { "Aveena", 0.0 },
  { "Aveer", 1.0 },
  { "Aveigha", 0.0 },
  { "Aveion", 1.0 },
  { "Avel", 1.0 },
  { "Avelardo", 1.0 },
  { "Aveleen", 0.0 },
  { "Avelene", 0.0 },
  { "Avelia", 0.0 },
  { "Avelie", 0.0 },
  { "Avelin", 0.0 },
  { "Avelina", 0.0 },
  { "Aveline", 0.0 },
  { "Avelinn", 0.0 },
  { "Avelino", 1.0 },
  { "Avella", 0.0 },
  { "Avellana", 0.0 },
  { "Avelle", 0.0 },
  { "Avellina", 0.0 },
  { "Avelyn", 0.0 },
  { "Avelynn", 0.0 },
  { "Avemaria", 0.0 },
  { "Aven", 0.59679145 },
  { "Avena", 0.0 },
  { "Avenell", 0.0 },
  { "Avenelle", 0.0 },
  { "Avenir", 1.0 },
  { "Avenlea", 0.0 },
  { "Avenlee", 0.0 },
  { "Avenleigh", 0.0 },
  { "Avenley", 0.0 },
  { "Avenly", 0.0 },
  { "Avenn", 1.0 },
  { "Aveon", 0.9483204 },
  { "Aveona", 0.0 },
  { "Aveonna", 0.0 },
  { "Aver", 0.0 },
  { "Avera", 0.0 },
  { "Averee", 0.03783102 },
  { "Avereigh", 0.0 },
  { "Averell", 1.0 },
  { "Averett", 1.0 },
  { "Averey", 0.31343284 },
  { "Averi", 0.04150597 },
  { "Averiana", 0.0 },
  { "Averianna", 0.0 },
  { "Averick", 1.0 },
  { "Averie", 0.020134227 },
  { "Averiee", 0.0 },
  { "Averiella", 0.0 },
  { "Averielle", 0.0 },
  { "Averii", 0.0 },
  { "Averil", 0.098434 },
  { "Averill", 0.6745562 },
  { "Averion", 1.0 },
  { "Averionna", 0.0 },
  { "Averlee", 0.0 },
  { "Averleigh", 0.0 },
  { "Averley", 0.0 },
  { "Averlie", 0.0 },
  { "Averly", 0.0 },
  { "Avery", 0.31904274 },
  { "Averyana", 0.0 },
  { "Averyanna", 0.0 },
  { "Averyl", 0.0 },
  { "Averylynn", 0.0 },
  { "Averymarie", 0.0 },
  { "Averyn", 1.0 },
  { "Averyon", 1.0 },
  { "Averyona", 0.0 },
  { "Averyonna", 0.0 },
  { "Averyrose", 0.0 },
  { "Aves", 0.0 },
  { "Avet", 1.0 },
  { "Avetis", 1.0 },
  { "Avett", 0.9597315 },
  { "Avey", 0.0 },
  { "Aveya", 0.0 },
  { "Aveyah", 0.0 },
  { "Aveyon", 1.0 },
  { "Avi", 0.88469136 },
  { "Avia", 0.0 },
  { "Aviad", 1.0 },
  { "Aviah", 0.0 },
  { "Aviahna", 0.0 },
  { "Avian", 0.8012912 },
  { "Aviana", 0.0 },
  { "Avianah", 0.0 },
  { "Avianca", 0.0 },
  { "Aviance", 0.0 },
  { "Avianna", 0.0 },
  { "Aviannah", 0.0 },
  { "Avianne", 0.0 },
  { "Aviara", 0.0 },
  { "Aviary", 0.0 },
  { "Avice", 0.0 },
  { "Avichai", 1.0 },
  { "Avid", 1.0 },
  { "Avidan", 1.0 },
  { "Avie", 0.008688783 },
  { "Aviel", 0.9715994 },
  { "Aviela", 0.0 },
  { "Aviella", 0.0 },
  { "Avielle", 0.025 },
  { "Avien", 1.0 },
  { "Avienda", 0.0 },
  { "Aviendha", 0.0 },
  { "Avienne", 0.0 },
  { "Avieon", 1.0 },
  { "Avier", 1.0 },
  { "Avigail", 0.0 },
  { "Avigayil", 0.0 },
  { "Avigdor", 1.0 },
  { "Avighna", 1.0 },
  { "Avik", 1.0 },
  { "Avika", 0.0 },
  { "Avila", 0.0 },
  { "Avilee", 0.0 },
  { "Avilene", 0.0 },
  { "Avilyn", 0.0 },
  { "Avilynn", 0.0 },
  { "Avimael", 1.0 },
  { "Avin", 0.92676765 },
  { "Avina", 0.0 },
  { "Avinash", 1.0 },
  { "Avinell", 0.0 },
  { "Avinoam", 1.0 },
  { "Avion", 0.8544907 },
  { "Aviona", 0.0 },
  { "Avionce", 0.0 },
  { "Avione", 0.0 },
  { "Avionna", 0.0 },
  { "Avionne", 0.0 },
  { "Avir", 1.0 },
  { "Avira", 0.0 },
  { "Aviraaj", 1.0 },
  { "Aviraj", 1.0 },
  { "Aviral", 1.0 },
  { "Aviram", 1.0 },
  { "Aviree", 0.0 },
  { "Avis", 0.044005137 },
  { "Avish", 1.0 },
  { "Avisha", 0.0 },
  { "Avishai", 1.0 },
  { "Avishek", 1.0 },
  { "Avishi", 0.0 },
  { "Avishka", 0.0 },
  { "Avita", 0.0 },
  { "Avitaj", 1.0 },
  { "Avital", 0.0 },
  { "Avitaz", 1.0 },
  { "Aviv", 0.80172414 },
  { "Aviva", 0.0 },
  { "Avivah", 0.0 },
  { "Aviya", 0.0 },
  { "Aviyah", 0.0 },
  { "Aviyana", 0.0 },
  { "Aviyanna", 0.0 },
  { "Aviyon", 0.93939394 },
  { "Aviyonna", 0.0 },
  { "Avleen", 0.0 },
  { "Avlyn", 0.0 },
  { "Avlynn", 0.0 },
  { "Avneesh", 1.0 },
  { "Avneet", 0.013020833 },
  { "Avner", 1.0 },
  { "Avni", 0.0 },
  { "Avnish", 1.0 },
  { "Avnoor", 0.0 },
  { "Avo", 0.02793296 },
  { "Avola", 0.0 },
  { "Avon", 0.6802193 },
  { "Avona", 0.0 },
  { "Avonda", 0.0 },
  { "Avondre", 1.0 },
  { "Avonel", 0.0 },
  { "Avonell", 0.0 },
  { "Avonelle", 0.0 },
  { "Avonlea", 0.0 },
  { "Avonlee", 0.0 },
  { "Avonna", 0.0 },
  { "Avonne", 0.0 },
  { "Avonni", 0.0 },
  { "Avont", 1.0 },
  { "Avonta", 1.0 },
  { "Avontae", 1.0 },
  { "Avontay", 1.0 },
  { "Avonte", 1.0 },
  { "Avor", 0.0 },
  { "Avoree", 0.0 },
  { "Avori", 0.0 },
  { "Avorie", 0.0 },
  { "Avory", 0.42771083 },
  { "Avra", 0.0 },
  { "Avraham", 1.0 },
  { "Avrahom", 1.0 },
  { "Avraj", 1.0 },
  { "Avram", 1.0 },
  { "Avree", 0.024774775 },
  { "Avreet", 0.0 },
  { "Avreigh", 0.0 },
  { "Avren", 1.0 },
  { "Avrey", 0.4420168 },
  { "Avri", 0.057356607 },
  { "Avrian", 1.0 },
  { "Avriana", 0.0 },
  { "Avrianna", 0.0 },
  { "Avrie", 0.01146789 },
  { "Avriel", 0.11678832 },
  { "Avriella", 0.0 },
  { "Avrielle", 0.0 },
  { "Avrik", 1.0 },
  { "Avril", 0.0020169422 },
  { "Avrill", 0.0 },
  { "Avrilynn", 0.0 },
  { "Avrion", 1.0 },
  { "Avroham", 1.0 },
  { "Avrohom", 1.0 },
  { "Avrom", 1.0 },
  { "Avron", 1.0 },
  { "Avrora", 0.0 },
  { "Avrum", 1.0 },
  { "Avrumi", 1.0 },
  { "Avrumy", 1.0 },
  { "Avry", 0.47635525 },
  { "Avryl", 0.0 },
  { "Avryn", 0.0 },
  { "Avshalom", 1.0 },
  { "Avva", 0.0 },
  { "Avy", 0.052980132 },
  { "Avya", 0.0 },
  { "Avyaan", 1.0 },
  { "Avyan", 1.0 },
  { "Avyana", 0.0 },
  { "Avyanah", 0.0 },
  { "Avyanna", 0.0 },
  { "Avyansh", 1.0 },
  { "Avyay", 1.0 },
  { "Avyion", 1.0 },
  { "Avyn", 0.44054055 },
  { "Avynlee", 0.0 },
  { "Avynn", 0.0 },
  { "Avyon", 1.0 },
  { "Avyona", 0.0 },
  { "Avyonna", 0.0 },
  { "Avyukt", 1.0 },
  { "Avyukth", 1.0 },
  { "Awa", 0.0 },
  { "Awab", 1.0 },
  { "Awad", 1.0 },
  { "Awais", 1.0 },
  { "Awan", 1.0 },
  { "Awanda", 0.0 },
  { "Awbree", 0.0 },
  { "Awbrey", 0.0 },
  { "Aweis", 1.0 },
  { "Awesome", 1.0 },
  { "Awet", 1.0 },
  { "Aweys", 1.0 },
  { "Awi", 0.0 },
  { "Awilda", 0.0 },
  { "Awn", 0.0 },
  { "Awna", 0.0 },
  { "Awo", 0.0 },
  { "Aws", 1.0 },
  { "Awwab", 1.0 },
  { "Awwal", 1.0 },
  { "Axa", 0.0 },
  { "Axal", 1.0 },
  { "Axavier", 1.0 },
  { "Axcel", 1.0 },
  { "Axcell", 1.0 },
  { "Axe", 1.0 },
  { "Axeel", 1.0 },
  { "Axel", 0.996702 },
  { "Axell", 1.0 },
  { "Axelle", 0.0 },
  { "Axeton", 1.0 },
  { "Axia", 0.0 },
  { "Axie", 0.0 },
  { "Axiel", 1.0 },
  { "Axil", 1.0 },
  { "Axiom", 1.0 },
  { "Axiry", 0.0 },
  { "Axis", 1.0 },
  { "Axl", 1.0 },
  { "Axle", 1.0 },
  { "Axon", 1.0 },
  { "Axsel", 1.0 },
  { "Axten", 1.0 },
  { "Axtin", 1.0 },
  { "Axton", 1.0 },
  { "Axtyn", 1.0 },
  { "Axwell", 1.0 },
  { "Axxel", 1.0 },
  { "Axyl", 1.0 },
  { "Axzel", 1.0 },
  { "Aya", 0.0 },
  { "Ayaa", 0.0 },
  { "Ayaan", 0.9643595 },
  { "Ayaana", 0.0 },
  { "Ayaansh", 1.0 },
  { "Ayaat", 0.0 },
  { "Ayad", 1.0 },
  { "Ayah", 0.0 },
  { "Ayahna", 0.0 },
  { "Ayaina", 0.0 },
  { "Ayak", 0.0 },
  { "Ayaka", 0.0 },
  { "Ayako", 0.0 },
  { "Ayal", 1.0 },
  { "Ayala", 0.0 },
  { "Ayalah", 0.0 },
  { "Ayame", 0.0 },
  { "Ayami", 0.0 },
  { "Ayan", 0.74625 },
  { "Ayana", 0.0004947066 },
  { "Ayanah", 0.0 },
  { "Ayanami", 0.0 },
  { "Ayanda", 0.0 },
  { "Ayane", 0.0 },
  { "Ayani", 0.0 },
  { "Ayania", 0.0 },
  { "Ayaniah", 0.0 },
  { "Ayanle", 1.0 },
  { "Ayanna", 0.0 },
  { "Ayannah", 0.0 },
  { "Ayanne", 0.0 },
  { "Ayanni", 0.0 },
  { "Ayannia", 0.0 },
  { "Ayano", 0.0 },
  { "Ayansh", 1.0 },
  { "Ayante", 0.0 },
  { "Ayantu", 0.0 },
  { "Ayari", 0.0 },
  { "Ayasha", 0.0 },
  { "Ayat", 0.0 },
  { "Ayati", 0.0 },
  { "Ayatollah", 1.0 },
  { "Ayatt", 0.0 },
  { "Ayauna", 0.0 },
  { "Ayaz", 1.0 },
  { "Aybel", 1.0 },
  { "Aybree", 0.0 },
  { "Aybri", 0.0 },
  { "Aybrie", 0.0 },
  { "Ayce", 1.0 },
  { "Aycen", 1.0 },
  { "Ayda", 0.0 },
  { "Aydah", 0.0 },
  { "Aydain", 1.0 },
  { "Aydali", 0.0 },
  { "Aydan", 0.9185199 },
  { "Aydann", 1.0 },
  { "Aydden", 1.0 },
  { "Ayde", 0.0 },
  { "Aydee", 0.0 },
  { "Aydeen", 1.0 },
  { "Aydelis", 0.0 },
  { "Aydeliz", 0.0 },
  { "Ayden", 0.96854347 },
  { "Aydenn", 0.97882736 },
  { "Aydenne", 0.0 },
  { "Aydia", 0.0 },
  { "Aydian", 1.0 },
  { "Aydien", 1.0 },
  { "Aydin", 0.9593271 },
  { "Aydn", 1.0 },
  { "Aydon", 1.0 },
  { "Aydree", 0.0 },
  { "Aydren", 1.0 },
  { "Aydrian", 0.83093524 },
  { "Aydriana", 0.0 },
  { "Aydrianna", 0.0 },
  { "Aydric", 1.0 },
  { "Aydrie", 0.0 },
  { "Aydrien", 1.0 },
  { "Aydrik", 1.0 },
  { "Aydun", 1.0 },
  { "Aydyn", 0.9657321 },
  { "Aye", 0.0 },
  { "Ayea", 0.0 },
  { "Ayeasha", 0.0 },
  { "Ayedan", 1.0 },
  { "Ayeden", 1.0 },
  { "Ayedin", 1.0 },
  { "Ayeesha", 0.0 },
  { "Ayeh", 0.0 },
  { "Ayeisha", 0.0 },
  { "Ayela", 0.0 },
  { "Ayele", 0.5 },
  { "Ayeleen", 0.0 },
  { "Ayelen", 0.0 },
  { "Ayelene", 0.0 },
  { "Ayelet", 0.0 },
  { "Ayelin", 0.0 },
  { "Ayen", 0.0 },
  { "Ayerim", 0.0 },
  { "Ayesha", 0.0 },
  { "Ayeshah", 0.0 },
  { "Ayeshia", 0.0 },
  { "Ayeza", 0.0 },
  { "Ayham", 1.0 },
  { "Ayhan", 1.0 },
  { "Ayhem", 1.0 },
  { "Ayia", 0.0 },
  { "Ayiana", 0.0 },
  { "Ayianna", 0.0 },
  { "Ayiden", 1.0 },
  { "Ayin", 0.0 },
  { "Ayinde", 1.0 },
  { "Ayira", 0.0 },
  { "Ayisha", 0.0 },
  { "Ayishah", 0.0 },
  { "Ayja", 0.0 },
  { "Ayjah", 0.0 },
  { "Ayla", 0.0 },
  { "Aylah", 0.0 },
  { "Aylamae", 0.0 },
  { "Aylan", 1.0 },
  { "Aylana", 0.0 },
  { "Aylani", 0.0 },
  { "Aylanie", 0.0 },
  { "Aylanna", 0.0 },
  { "Aylanni", 0.0 },
  { "Ayleah", 0.0 },
  { "Ayled", 0.0 },
  { "Aylee", 0.0 },
  { "Ayleen", 0.0 },
  { "Ayleena", 0.0 },
  { "Ayleene", 0.0 },
  { "Ayleigh", 0.0 },
  { "Aylen", 0.08490566 },
  { "Aylena", 0.0 },
  { "Aylene", 0.0 },
  { "Aylenne", 0.0 },
  { "Aylet", 0.0 },
  { "Ayleth", 0.0 },
  { "Ayli", 0.0 },
  { "Aylia", 0.0 },
  { "Ayliah", 0.0 },
  { "Ayliana", 0.0 },
  { "Aylianna", 0.0 },
  { "Aylie", 0.0 },
  { "Aylin", 0.0017774267 },
  { "Aylina", 0.0 },
  { "Ayline", 0.0 },
  { "Aylinn", 0.0 },
  { "Aylinne", 0.0 },
  { "Aylisha", 0.0 },
  { "Aylissa", 0.0 },
  { "Aylla", 0.0 },
  { "Aylon", 1.0 },
  { "Aylssa", 0.0 },
  { "Aylyn", 0.0 },
  { "Aylynn", 0.0 },
  { "Aymaan", 1.0 },
  { "Ayman", 0.98310196 },
  { "Aymar", 0.0 },
  { "Aymara", 0.0 },
  { "Ayme", 0.0 },
  { "Aymee", 0.0 },
  { "Aymen", 0.9502924 },
  { "Aymeric", 1.0 },
  { "Aymie", 0.0 },
  { "Ayn", 0.0 },
  { "Ayna", 0.0 },
  { "Aynara", 0.0 },
  { "Ayneka", 0.0 },
  { "Ayni", 0.0 },
  { "Aynia", 0.0 },
  { "Ayniah", 0.0 },
  { "Aynslee", 0.0 },
  { "Aynsleigh", 0.0 },
  { "Aynsley", 0.0 },
  { "Aynslie", 0.0 },
  { "Aynur", 0.0 },
  { "Ayo", 0.5928144 },
  { "Ayobami", 1.0 },
  { "Ayodeji", 1.0 },
  { "Ayodele", 0.70658684 },
  { "Ayofemi", 0.0 },
  { "Ayoka", 0.0 },
  { "Ayoki", 0.0 },
  { "Ayomi", 0.0 },
  { "Ayomide", 0.58945906 },
  { "Ayomikun", 0.7291667 },
  { "Ayomiposi", 0.52 },
  { "Ayon", 1.0 },
  { "Ayona", 0.0 },
  { "Ayonah", 0.0 },
  { "Ayondela", 0.0 },
  { "Ayonna", 0.0 },
  { "Ayoob", 1.0 },
  { "Ayoola", 0.5 },
  { "Ayooluwa", 0.33333334 },
  { "Ayotomi", 1.0 },
  { "Ayotunde", 1.0 },
  { "Ayoub", 1.0 },
  { "Ayra", 0.0 },
  { "Ayrabella", 0.0 },
  { "Ayrah", 0.0 },
  { "Ayram", 0.0 },
  { "Ayreana", 0.0 },
  { "Ayreanna", 0.0 },
  { "Ayren", 0.921875 },
  { "Ayreona", 0.0 },
  { "Ayreonna", 0.0 },
  { "Ayres", 1.0 },
  { "Ayria", 0.0 },
  { "Ayriah", 0.0 },
  { "Ayrial", 0.0 },
  { "Ayrian", 0.3 },
  { "Ayriana", 0.0 },
  { "Ayrianna", 0.0 },
  { "Ayriauna", 0.0 },
  { "Ayric", 1.0 },
  { "Ayrica", 0.0 },
  { "Ayrie", 0.0 },
  { "Ayriel", 0.0 },
  { "Ayrihanna", 0.0 },
  { "Ayrika", 0.0 },
  { "Ayrin", 0.0 },
  { "Ayrion", 0.5 },
  { "Ayrionna", 0.0 },
  { "Ayris", 0.0 },
  { "Ayron", 0.98207885 },
  { "Ayrton", 1.0 },
  { "Aysa", 0.040983606 },
  { "Ayse", 0.0 },
  { "Aysel", 0.0 },
  { "Aysen", 1.0 },
  { "Aysenur", 0.0 },
  { "Aysha", 0.0 },
  { "Ayshah", 0.0 },
  { "Ayshia", 0.0 },
  { "Aysia", 0.0 },
  { "Aysiah", 0.0 },
  { "Ayslee", 0.0 },
  { "Ayslin", 0.0 },
  { "Ayslinn", 0.0 },
  { "Ayslyn", 0.0 },
  { "Ayson", 1.0 },
  { "Ayssa", 0.0 },
  { "Aysu", 0.0 },
  { "Aytana", 0.0 },
  { "Ayten", 0.0 },
  { "Aythan", 1.0 },
  { "Aythen", 1.0 },
  { "Ayub", 1.0 },
  { "Ayumi", 0.0 },
  { "Ayumu", 1.0 },
  { "Ayuna", 0.0 },
  { "Ayunna", 0.0 },
  { "Ayush", 1.0 },
  { "Ayusha", 0.0 },
  { "Ayushi", 0.0 },
  { "Ayushmaan", 1.0 },
  { "Ayushman", 1.0 },
  { "Ayuub", 1.0 },
  { "Ayva", 0.0 },
  { "Ayvah", 0.0 },
  { "Ayvan", 1.0 },
  { "Ayvani", 0.0 },
  { "Ayvee", 0.0 },
  { "Ayven", 0.8938356 },
  { "Ayveon", 1.0 },
  { "Ayveri", 0.0 },
  { "Ayverie", 0.0 },
  { "Ayvia", 0.0 },
  { "Ayviana", 0.0 },
  { "Ayvin", 1.0 },
  { "Ayvion", 1.0 },
  { "Ayvree", 0.0 },
  { "Ayvri", 0.0 },
  { "Ayvrie", 0.0 },
  { "Ayyan", 1.0 },
  { "Ayyoub", 1.0 },
  { "Ayyub", 1.0 },
  { "Ayza", 0.0 },
  { "Ayzah", 0.0 },
  { "Ayzaria", 0.0 },
  { "Ayzel", 0.0 },
  { "Ayzen", 1.0 },
  { "Ayzha", 0.0 },
  { "Ayzia", 0.0 },
  { "Ayzlee", 0.0 },
  { "Ayzlin", 0.0 },
  { "Ayzlyn", 0.0 },
  { "Aza", 0.1092437 },
  { "Azaad", 1.0 },
  { "Azaan", 1.0 },
  { "Azad", 1.0 },
  { "Azadeh", 0.0 },
  { "Azael", 1.0 },
  { "Azaela", 0.0 },
  { "Azaelia", 0.0 },
  { "Azah", 0.0 },
  { "Azahel", 1.0 },
  { "Azahria", 0.0 },
  { "Azai", 1.0 },
  { "Azaia", 0.0 },
  { "Azaiah", 0.8855535 },
  { "Azaila", 0.0 },
  { "Azailya", 0.0 },
  { "Azaira", 0.0 },
  { "Azairah", 0.0 },
  { "Azaireyah", 0.0 },
  { "Azaiyah", 0.0 },
  { "Azalaya", 0.0 },
  { "Azalea", 0.0 },
  { "Azaleah", 0.0 },
  { "Azalee", 0.0 },
  { "Azaleia", 0.0 },
  { "Azaleigh", 0.0 },
  { "Azaleigha", 0.0 },
  { "Azalene", 0.0 },
  { "Azaley", 0.0 },
  { "Azaleya", 0.0 },
  { "Azaleyah", 0.0 },
  { "Azalia", 0.0 },
  { "Azaliah", 0.0 },
  { "Azalie", 0.0 },
  { "Azaline", 0.0 },
  { "Azaliya", 0.0 },
  { "Azaliyah", 0.0 },
  { "Azalya", 0.0 },
  { "Azalyn", 0.0 },
  { "Azalynn", 0.0 },
  { "Azam", 1.0 },
  { "Azan", 1.0 },
  { "Azana", 0.0 },
  { "Azanae", 0.0 },
  { "Azaneth", 0.0 },
  { "Azani", 0.0 },
  { "Azania", 0.0 },
  { "Azaniah", 0.33333334 },
  { "Azante", 1.0 },
  { "Azar", 0.91803277 },
  { "Azara", 0.0 },
  { "Azarae", 0.0 },
  { "Azarael", 1.0 },
  { "Azarah", 0.0 },
  { "Azaraya", 0.0 },
  { "Azarea", 0.0 },
  { "Azareah", 0.0 },
  { "Azaree", 0.0 },
  { "Azareel", 1.0 },
  { "Azareeyah", 0.0 },
  { "Azarel", 1.0 },
  { "Azareth", 0.0 },
  { "Azareya", 0.0 },
  { "Azareyah", 0.0 },
  { "Azari", 0.30688623 },
  { "Azaria", 0.023402523 },
  { "Azariah", 0.40652072 },
  { "Azarian", 1.0 },
  { "Azariana", 0.0 },
  { "Azarias", 1.0 },
  { "Azarie", 0.04901961 },
  { "Azariea", 0.0 },
  { "Azariel", 1.0 },
  { "Azarion", 1.0 },
  { "Azarious", 1.0 },
  { "Azarius", 1.0 },
  { "Azariya", 0.0 },
  { "Azariyah", 0.03214286 },
  { "Azarria", 0.0 },
  { "Azarriah", 0.0 },
  { "Azarya", 0.19736843 },
  { "Azaryah", 0.30054644 },
  { "Azaryia", 0.0 },
  { "Azavian", 1.0 },
  { "Azavier", 1.0 },
  { "Azavion", 1.0 },
  { "Azaya", 0.0 },
  { "Azayah", 0.36923078 },
  { "Azayla", 0.0 },
  { "Azaylah", 0.0 },
  { "Azaylea", 0.0 },
  { "Azayleah", 0.0 },
  { "Azaylee", 0.0 },
  { "Azaylia", 0.0 },
  { "Azayliah", 0.0 },
  { "Azayvion", 1.0 },
  { "Azazel", 1.0 },
  { "Azden", 1.0 },
  { "Azealia", 0.0 },
  { "Azeem", 1.0 },
  { "Azeemah", 0.0 },
  { "Azeen", 0.0 },
  { "Azeez", 1.0 },
  { "Azeeza", 0.0 },
  { "Azekiel", 1.0 },
  { "Azel", 0.82206404 },
  { "Azelea", 0.0 },
  { "Azelene", 0.0 },
  { "Azelia", 0.0 },
  { "Azelie", 0.0 },
  { "Azelin", 0.0 },
  { "Azell", 0.8064516 },
  { "Azella", 0.0 },
  { "Azelle", 0.0 },
  { "Azelya", 0.0 },
  { "Azelyn", 0.0 },
  { "Azelynn", 0.0 },
  { "Azema", 0.0 },
  { "Azena", 0.0 },
  { "Azende", 1.0 },
  { "Azenet", 0.0 },
  { "Azeneth", 0.0 },
  { "Azer", 1.0 },
  { "Azera", 0.0 },
  { "Azeria", 0.0 },
  { "Azeriah", 0.16949153 },
  { "Azerion", 1.0 },
  { "Azfar", 1.0 },
  { "Azha", 0.0 },
  { "Azhaan", 1.0 },
  { "Azhan", 1.0 },
  { "Azhanae", 0.0 },
  { "Azhane", 0.0 },
  { "Azhani", 0.0 },
  { "Azhar", 0.68014705 },
  { "Azhari", 0.0 },
  { "Azharia", 0.0 },
  { "Azhia", 0.0 },
  { "Azi", 1.0 },
  { "Azia", 0.0 },
  { "Aziah", 0.38014784 },
  { "Aziana", 0.0 },
  { "Azianna", 0.0 },
  { "Azie", 0.0 },
  { "Aziel", 0.98542273 },
  { "Azier", 1.0 },
  { "Azikiwe", 1.0 },
  { "Azilda", 0.0 },
  { "Azile", 0.0 },
  { "Azilee", 0.0 },
  { "Azilyn", 0.0 },
  { "Azilynn", 0.0 },
  { "Azim", 1.0 },
  { "Azimjon", 1.0 },
  { "Azion", 0.88461536 },
  { "Azir", 1.0 },
  { "Azira", 0.0 },
  { "Azirah", 0.0 },
  { "Aziria", 0.0 },
  { "Aziriah", 0.0 },
  { "Azita", 0.0 },
  { "Aziya", 0.0 },
  { "Aziyah", 0.03255341 },
  { "Aziz", 1.0 },
  { "Aziza", 0.0 },
  { "Azizah", 0.0 },
  { "Azizi", 0.1744186 },
  { "Azizjon", 1.0 },
  { "Azja", 0.0 },
  { "Azjah", 0.0 },
  { "Azjon", 1.0 },
  { "Azka", 0.0 },
  { "Azlaan", 1.0 },
  { "Azlan", 0.9752322 },
  { "Azlee", 0.0 },
  { "Azleigh", 0.0 },
  { "Azlin", 0.0 },
  { "Azlyn", 0.0 },
  { "Azlynn", 0.0 },
  { "Azlynne", 0.0 },
  { "Azmariah", 0.0 },
  { "Azmeer", 1.0 },
  { "Azmi", 1.0 },
  { "Azmina", 0.0 },
  { "Azmir", 1.0 },
  { "Azon", 1.0 },
  { "Azora", 0.0 },
  { "Azori", 0.0 },
  { "Azoria", 0.0 },
  { "Azoriah", 0.0 },
  { "Azra", 0.022164276 },
  { "Azrael", 0.82260185 },
  { "Azrah", 0.0 },
  { "Azreal", 1.0 },
  { "Azria", 0.0 },
  { "Azriah", 0.39285713 },
  { "Azriel", 0.75740945 },
  { "Azriela", 0.0 },
  { "Azriella", 0.0 },
  { "Azrielle", 0.0 },
  { "Azrien", 1.0 },
  { "Azryel", 1.0 },
  { "Azsa", 0.0 },
  { "Azsha", 0.0 },
  { "Azteca", 0.0 },
  { "Aztlan", 1.0 },
  { "Azucena", 0.0 },
  { "Azul", 0.02032767 },
  { "Azula", 0.0 },
  { "Azumi", 0.0 },
  { "Azur", 0.0 },
  { "Azura", 0.0 },
  { "Azurae", 0.0 },
  { "Azurah", 0.0 },
  { "Azurdee", 0.0 },
  { "Azure", 0.12834224 },
  { "Azuredee", 0.0 },
  { "Azuree", 0.0 },
  { "Azuri", 0.0 },
  { "Azuriah", 1.0 },
  { "Azurie", 0.0 },
  { "Azury", 0.0 },
  { "Azusa", 0.0 },
  { "Azusena", 0.0 },
  { "Azuzena", 0.0 },
  { "Azya", 0.0 },
  { "Azyah", 0.0 },
  { "Azyan", 1.0 },
  { "Azyia", 0.0 },
  { "Azyiah", 0.0 },
  { "Azyla", 0.0 },
  { "Azylah", 0.0 },
  { "Azyon", 1.0 },
  { "Azyra", 0.0 },
  { "Azyrah", 0.0 },
  { "Azyria", 0.0 },
  { "Azyriah", 0.0 },
  { "Azza", 0.0 },
  { "Azzahra", 0.0 },
  { "Azzam", 1.0 },
  { "Azzan", 1.0 },
  { "Azzareya", 0.0 },
  { "Azzaria", 0.0 },
  { "Azziah", 0.0 },
  { "Azzie", 0.022703819 },
  { "Azzure", 0.0625 },
  { "Baani", 0.0 },
  { "Baasil", 1.0 },
  { "Baba", 1.0 },
  { "Babacar", 1.0 },
  { "Babajide", 1.0 },
  { "Babak", 1.0 },
  { "Babara", 0.0 },
  { "Babatunde", 1.0 },
  { "Babbett", 0.0 },
  { "Babbette", 0.0 },
  { "Babbi", 0.0 },
  { "Babbie", 0.0 },
  { "Babby", 0.0 },
  { "Babe", 0.4739011 },
  { "Baber", 1.0 },
  { "Babetta", 0.0 },
  { "Babette", 0.0015285845 },
  { "Babie", 0.0 },
  { "Babita", 0.0 },
  { "Baboucarr", 1.0 },
  { "Babs", 0.0 },
  { "Baby", 0.50920707 },
  { "Babyboy", 1.0 },
  { "Babygirl", 0.0033975085 },
  { "Bacari", 1.0 },
  { "Bacarri", 1.0 },
  { "Bach", 1.0 },
  { "Bachir", 1.0 },
  { "Bacil", 1.0 },
  { "Bacilia", 0.0 },
  { "Bacilio", 1.0 },
  { "Badal", 1.0 },
  { "Baden", 1.0 },
  { "Bader", 1.0 },
  { "Badhir", 1.0 },
  { "Badi", 1.0 },
  { "Badia", 0.0 },
  { "Badr", 1.0 },
  { "Bae", 0.17857143 },
  { "Baeden", 1.0 },
  { "Baela", 0.0 },
  { "Baelee", 0.0 },
  { "Baeleigh", 0.0 },
  { "Baeley", 0.0 },
  { "Baelfire", 1.0 },
  { "Baeli", 0.0 },
  { "Baelie", 0.0 },
  { "Baelin", 1.0 },
  { "Baelor", 1.0 },
  { "Baelyn", 0.12087912 },
  { "Baelynn", 0.0 },
  { "Baer", 1.0 },
  { "Baggio", 1.0 },
  { "Bah", 0.0 },
  { "Baha", 1.0 },
  { "Bahaa", 1.0 },
  { "Bahar", 0.0 },
  { "Bahareh", 0.0 },
  { "Bahati", 0.0 },
  { "Bahe", 1.0 },
  { "Baheejah", 0.0 },
  { "Baheerah", 0.0 },
  { "Bahia", 0.0 },
  { "Bahiya", 0.0 },
  { "Bahiyah", 0.0 },
  { "Bahiyyah", 0.0 },
  { "Bahja", 0.0 },
  { "Bai", 0.5 },
  { "Baiba", 0.0 },
  { "Baiden", 1.0 },
  { "Baila", 0.0 },
  { "Bailasan", 0.0 },
  { "Baile", 0.0 },
  { "Bailea", 0.0 },
  { "Bailee", 0.016644074 },
  { "Bailei", 0.0 },
  { "Baileigh", 0.0 },
  { "Bailen", 1.0 },
  { "Bailey", 0.17892721 },
  { "Baileyann", 0.0 },
  { "Baileyrae", 0.0 },
  { "Baili", 0.0 },
  { "Bailie", 0.0 },
  { "Bailiegh", 0.0 },
  { "Baille", 0.0 },
  { "Baillee", 0.0 },
  { "Bailley", 0.0 },
  { "Baillie", 0.0 },
  { "Bailly", 0.0 },
  { "Bailor", 0.64285713 },
  { "Baily", 0.24244186 },
  { "Bailye", 0.0 },
  { "Bailyn", 0.06944445 },
  { "Bailynn", 0.0 },
  { "Bain", 1.0 },
  { "Baine", 1.0 },
  { "Bair", 1.0 },
  { "Baird", 1.0 },
  { "Bairo", 1.0 },
  { "Bairon", 1.0 },
  { "Baisha", 0.0 },
  { "Baisley", 0.0 },
  { "Baiya", 0.0 },
  { "Baize", 1.0 },
  { "Baja", 0.0 },
  { "Bakar", 1.0 },
  { "Bakari", 1.0 },
  { "Bakary", 1.0 },
  { "Baker", 0.9590643 },
  { "Bakhita", 0.0 },
  { "Bakir", 1.0 },
  { "Bakr", 1.0 },
  { "Balaji", 1.0 },
  { "Balal", 1.0 },
  { "Balam", 1.0 },
  { "Balbina", 0.0 },
  { "Baldassare", 1.0 },
  { "Baldemar", 1.0 },
  { "Baldo", 1.0 },
  { "Baldomero", 1.0 },
  { "Baldwin", 1.0 },
  { "Bale", 1.0 },
  { "Balee", 0.0 },
  { "Baleigh", 0.0 },
  { "Balen", 1.0 },
  { "Balentin", 1.0 },
  { "Baler", 1.0 },
  { "Baleria", 0.0 },
  { "Balery", 0.0 },
  { "Baley", 0.23953488 },
  { "Balfour", 1.0 },
  { "Bali", 0.0 },
  { "Balian", 1.0 },
  { "Balie", 0.0 },
  { "Baliegh", 0.0 },
  { "Balil", 1.0 },
  { "Balin", 1.0 },
  { "Balinda", 0.0 },
  { "Balint", 1.0 },
  { "Balke", 1.0 },
  { "Ballard", 1.0 },
  { "Ballarie", 0.0 },
  { "Balmore", 1.0 },
  { "Balon", 0.0 },
  { "Balqis", 0.0 },
  { "Balraj", 1.0 },
  { "Baltasar", 1.0 },
  { "Baltazar", 1.0 },
  { "Balthazar", 1.0 },
  { "Balvina", 0.0 },
  { "Balynda", 0.0 },
  { "Bam", 1.0 },
  { "Bama", 0.0 },
  { "Bambi", 0.0 },
  { "Bambie", 0.0 },
  { "Bambina", 0.0 },
  { "Bamby", 0.0 },
  { "Bamidele", 1.0 },
  { "Bamma", 0.0 },
  { "Bana", 0.0 },
  { "Banan", 1.0 },
  { "Bandar", 1.0 },
  { "Bandi", 0.0 },
  { "Bandon", 1.0 },
  { "Bandy", 0.67391306 },
  { "Bane", 1.0 },
  { "Baneen", 0.0 },
  { "Banelly", 0.0 },
  { "Banesa", 0.0 },
  { "Banessa", 0.0 },
  { "Baneza", 0.0 },
  { "Bang", 1.0 },
  { "Bangaly", 1.0 },
  { "Bani", 0.0 },
  { "Bania", 0.0 },
  { "Banisha", 0.0 },
  { "Banita", 0.0 },
  { "Banjamin", 1.0 },
  { "Banks", 1.0 },
  { "Bankston", 1.0 },
  { "Banna", 0.0 },
  { "Banner", 0.9456869 },
  { "Bannie", 0.0 },
  { "Banning", 1.0 },
  { "Bannon", 1.0 },
  { "Bansari", 0.0 },
  { "Bansi", 0.0 },
  { "Bansri", 0.0 },
  { "Banx", 1.0 },
  { "Banyan", 1.0 },
  { "Banyon", 1.0 },
  { "Bao", 0.56823266 },
  { "Baotran", 0.0 },
  { "Baptiste", 1.0 },
  { "Baqir", 1.0 },
  { "Bar", 0.425 },
  { "Bara", 1.0 },
  { "Baraa", 0.82857144 },
  { "Baraah", 0.0 },
  { "Barabara", 0.0 },
  { "Barabra", 0.0 },
  { "Barack", 1.0 },
  { "Barak", 1.0 },
  { "Baraka", 0.9268293 },
  { "Barakah", 0.0 },
  { "Barakat", 0.0 },
  { "Baran", 0.7888889 },
  { "Baranda", 0.0 },
  { "Barara", 0.0 },
  { "Barb", 0.0 },
  { "Barba", 0.0 },
  { "Barbaa", 0.0 },
  { "Barbar", 0.0 },
  { "Barbara", 0.002877878 },
  { "Barbaraa", 0.0 },
  { "Barbaraann", 0.0 },
  { "Barbaraanne", 0.0 },
  { "Barbarajean", 0.0 },
  { "Barbarajo", 0.0 },
  { "Barbaralee", 0.0 },
  { "Barbarann", 0.0 },
  { "Barbaranne", 0.0 },
  { "Barbarba", 0.0 },
  { "Barbare", 0.0 },
  { "Barbarella", 0.0 },
  { "Barbarette", 0.0 },
  { "Barbarita", 0.0 },
  { "Barbaro", 1.0 },
  { "Barbe", 0.0 },
  { "Barbee", 0.0 },
  { "Barber", 0.7826087 },
  { "Barbera", 0.0 },
  { "Barbett", 0.0 },
  { "Barbetta", 0.0 },
  { "Barbette", 0.0 },
  { "Barbi", 0.0 },
  { "Barbie", 0.0 },
  { "Barbora", 0.0 },
  { "Barbra", 0.0 },
  { "Barbro", 0.0 },
  { "Barby", 0.0 },
  { "Barclay", 0.975181 },
  { "Bard", 1.0 },
  { "Bardara", 0.0 },
  { "Bardia", 1.0 },
  { "Bardo", 1.0 },
  { "Bareera", 0.0 },
  { "Bareerah", 0.0 },
  { "Barek", 1.0 },
  { "Baren", 1.0 },
  { "Barett", 1.0 },
  { "Baretta", 0.7222222 },
  { "Bari", 0.07295466 },
  { "Bariah", 0.0 },
  { "Barika", 0.0 },
  { "Barin", 1.0 },
  { "Baris", 1.0 },
  { "Barisha", 0.0 },
  { "Barker", 1.0 },
  { "Barkim", 1.0 },
  { "Barkley", 0.98392284 },
  { "Barkon", 1.0 },
  { "Barkot", 0.47619048 },
  { "Barksdale", 1.0 },
  { "Barlow", 1.0 },
  { "Barnabas", 1.0 },
  { "Barnaby", 1.0 },
  { "Barnard", 1.0 },
  { "Barndon", 1.0 },
  { "Barnell", 0.94174755 },
  { "Barnes", 1.0 },
  { "Barnet", 1.0 },
  { "Barnett", 0.9953125 },
  { "Barnetta", 0.0 },
  { "Barnette", 0.0 },
  { "Barney", 0.9943657 },
  { "Barnie", 0.99226004 },
  { "Barnwell", 1.0 },
  { "Barny", 1.0 },
  { "Barok", 1.0 },
  { "Baron", 1.0 },
  { "Barr", 1.0 },
  { "Barre", 1.0 },
  { "Barren", 1.0 },
  { "Barret", 0.9956102 },
  { "Barrett", 0.95858335 },
  { "Barrette", 1.0 },
  { "Barri", 0.085299455 },
  { "Barrick", 1.0 },
  { "Barrie", 0.4381579 },
  { "Barrington", 1.0 },
  { "Barrion", 1.0 },
  { "Barris", 1.0 },
  { "Barron", 1.0 },
  { "Barry", 0.99576426 },
  { "Barsha", 0.0 },
  { "Bart", 0.9996941 },
  { "Barta", 0.0 },
  { "Bartek", 1.0 },
  { "Bartel", 1.0 },
  { "Barth", 1.0 },
  { "Bartha", 0.0 },
  { "Bartholome", 1.0 },
  { "Bartholomew", 1.0 },
  { "Bartie", 0.0 },
  { "Bartlett", 1.0 },
  { "Bartley", 1.0 },
  { "Bartlomiej", 1.0 },
  { "Bartly", 1.0 },
  { "Barto", 1.0 },
  { "Bartola", 0.0 },
  { "Bartolo", 1.0 },
  { "Bartolome", 1.0 },
  { "Bartolomeo", 1.0 },
  { "Barton", 1.0 },
  { "Bartosz", 1.0 },
  { "Bartow", 1.0 },
  { "Bartt", 1.0 },
  { "Bartu", 1.0 },
  { "Barty", 1.0 },
  { "Baruc", 1.0 },
  { "Baruch", 1.0 },
  { "Baruti", 1.0 },
  { "Bary", 1.0 },
  { "Baryn", 1.0 },
  { "Bas", 0.0 },
  { "Bascom", 1.0 },
  { "Bascomb", 1.0 },
  { "Bascum", 1.0 },
  { "Baseemah", 0.0 },
  { "Basel", 1.0 },
  { "Basem", 1.0 },
  { "Bash", 1.0 },
  { "Basha", 0.0 },
  { "Bashan", 1.0 },
  { "Bashar", 1.0 },
  { "Bashawn", 1.0 },
  { "Basheba", 0.0 },
  { "Basheer", 1.0 },
  { "Bashir", 1.0 },
  { "Bashira", 0.0 },
  { "Bashirah", 0.0 },
  { "Bashiri", 1.0 },
  { "Bashy", 0.0 },
  { "Basia", 0.0 },
  { "Basil", 0.9885547 },
  { "Basile", 1.0 },
  { "Basilia", 0.0 },
  { "Basiliki", 0.0 },
  { "Basilio", 1.0 },
  { "Basilios", 1.0 },
  { "Basilisa", 0.0 },
  { "Basim", 1.0 },
  { "Basimah", 0.0 },
  { "Basir", 1.0 },
  { "Basit", 1.0 },
  { "Basma", 0.0 },
  { "Basmah", 0.0 },
  { "Bass", 1.0 },
  { "Bassam", 1.0 },
  { "Bassel", 1.0 },
  { "Bassem", 1.0 },
  { "Bassey", 1.0 },
  { "Bassheva", 0.0 },
  { "Bassil", 1.0 },
  { "Bassirou", 1.0 },
  { "Bassy", 0.0 },
  { "Bastian", 1.0 },
  { "Bastien", 1.0 },
  { "Bastion", 1.0 },
  { "Basya", 0.0 },
  { "Batel", 0.0 },
  { "Bates", 1.0 },
  { "Bathsheba", 0.0 },
  { "Batina", 0.0 },
  { "Batool", 0.0 },
  { "Batoul", 0.0 },
  { "Batrice", 0.0 },
  { "Batrina", 0.0 },
  { "Batsheva", 0.0 },
  { "Battal", 1.0 },
  { "Battista", 1.0 },
  { "Battle", 1.0 },
  { "Batu", 1.0 },
  { "Batuhan", 1.0 },
  { "Batul", 0.0 },
  { "Batula", 0.0 },
  { "Batya", 0.0 },
  { "Baudel", 1.0 },
  { "Baudelia", 0.0 },
  { "Baudelio", 1.0 },
  { "Baudilio", 1.0 },
  { "Bauer", 1.0 },
  { "Bautista", 1.0 },
  { "Bavan", 0.0 },
  { "Bavly", 1.0 },
  { "Bawi", 0.83125 },
  { "Baxlee", 0.0 },
  { "Baxley", 0.42553192 },
  { "Baxter", 1.0 },
  { "Baxton", 1.0 },
  { "Bay", 0.30208334 },
  { "Baya", 0.0 },
  { "Bayah", 0.0 },
  { "Bayan", 0.13068181 },
  { "Bayani", 1.0 },
  { "Bayard", 1.0 },
  { "Bayardo", 1.0 },
  { "Bayden", 0.979661 },
  { "Baye", 0.0 },
  { "Bayete", 1.0 },
  { "Bayla", 0.0 },
  { "Baylaa", 0.0 },
  { "Baylan", 1.0 },
  { "Bayle", 0.1124498 },
  { "Baylea", 0.0 },
  { "Baylee", 0.032636248 },
  { "Bayleen", 0.0 },
  { "Baylei", 0.0 },
  { "Bayleigh", 0.0 },
  { "Baylen", 0.87819254 },
  { "Bayler", 0.7318008 },
  { "Bayless", 1.0 },
  { "Bayley", 0.18954249 },
  { "Bayli", 0.0 },
  { "Baylian", 1.0 },
  { "Baylie", 0.008049536 },
  { "Bayliegh", 0.0 },
  { "Baylin", 0.5795148 },
  { "Baylon", 1.0 },
  { "Baylor", 0.7312132 },
  { "Bayly", 0.17741935 },
  { "Baylyn", 0.0 },
  { "Baylynn", 0.0 },
  { "Baynard", 1.0 },
  { "Bayne", 1.0 },
  { "Bayoleth", 0.0 },
  { "Bayro", 1.0 },
  { "Bayron", 1.0 },
  { "Bayshawn", 1.0 },
  { "Bayyinah", 0.0 },
  { "Bayze", 1.0 },
  { "Baze", 1.0 },
  { "Bazel", 1.0 },
  { "Bazil", 1.0 },
  { "Bb", 1.0 },
  { "Be", 1.0 },
  { "Bea", 0.0 },
  { "Beach", 1.0 },
  { "Beacher", 1.0 },
  { "Beadie", 0.0 },
  { "Beah", 0.0 },
  { "Beal", 1.0 },
  { "Beale", 1.0 },
  { "Beaman", 1.0 },
  { "Beamon", 1.0 },
  { "Bean", 1.0 },
  { "Beanca", 0.0 },
  { "Beandon", 1.0 },
  { "Beaney", 0.0 },
  { "Beanna", 0.0 },
  { "Beaonca", 0.0 },
  { "Bear", 1.0 },
  { "Bearett", 1.0 },
  { "Bearl", 1.0 },
  { "Bearnice", 0.0 },
  { "Beasia", 0.0 },
  { "Beasley", 1.0 },
  { "Beata", 0.0 },
  { "Beate", 0.0 },
  { "Beatha", 0.0 },
  { "Beather", 0.0 },
  { "Beathrice", 0.0 },
  { "Beatric", 0.0 },
  { "Beatrica", 0.0 },
  { "Beatrice", 0.0038430283 },
  { "Beatris", 0.0 },
  { "Beatrix", 0.0 },
  { "Beatriz", 0.0030558119 },
  { "Beatrize", 0.0 },
  { "Beatryce", 0.0 },
  { "Beatta", 0.0 },
  { "Beattie", 1.0 },
  { "Beatty", 1.0 },
  { "Beau", 0.98412746 },
  { "Beauden", 1.0 },
  { "Beauen", 1.0 },
  { "Beauford", 0.9902057 },
  { "Beaufort", 1.0 },
  { "Beaula", 0.0 },
  { "Beaulah", 0.0 },
  { "Beaumont", 1.0 },
  { "Beauregard", 1.0 },
  { "Beautiful", 0.0 },
  { "Beautifull", 0.0 },
  { "Beauton", 0.0 },
  { "Beautrice", 0.0 },
  { "Beauty", 0.0 },
  { "Beautyful", 0.0 },
  { "Beaux", 0.90672016 },
  { "Beaver", 1.0 },
  { "Bebe", 0.0 },
  { "Beca", 0.0 },
  { "Becca", 0.0 },
  { "Becci", 0.0 },
  { "Beck", 1.0 },
  { "Becka", 0.0 },
  { "Beckah", 0.0 },
  { "Beckam", 1.0 },
  { "Beckem", 1.0 },
  { "Becker", 1.0 },
  { "Becket", 1.0 },
  { "Beckett", 0.96387523 },
  { "Beckette", 0.5 },
  { "Beckey", 0.0 },
  { "Beckham", 0.99087864 },
  { "Beckhem", 1.0 },
  { "Becki", 0.0 },
  { "Beckie", 0.0 },
  { "Beckley", 0.2173913 },
  { "Becklyn", 0.0 },
  { "Beckman", 1.0 },
  { "Beckum", 1.0 },
  { "Becky", 0.0016612828 },
  { "Beda", 0.0 },
  { "Bedar", 0.0 },
  { "Beddie", 0.0 },
  { "Bedelia", 0.0 },
  { "Bedell", 0.0 },
  { "Bedford", 1.0 },
  { "Bedie", 0.0 },
  { "Bee", 0.6245421 },
  { "Beebe", 0.1388889 },
  { "Beecher", 1.0 },
  { "Beedie", 0.0 },
  { "Beena", 0.0 },
  { "Beenish", 0.0 },
  { "Beesan", 0.0 },
  { "Beeta", 0.0 },
  { "Begum", 0.0 },
  { "Behati", 0.0 },
  { "Behnam", 1.0 },
  { "Behr", 1.0 },
  { "Behrang", 1.0 },
  { "Behrett", 1.0 },
  { "Behruz", 1.0 },
  { "Behtzy", 0.0 },
  { "Beige", 0.0 },
  { "Beighley", 0.0 },
  { "Beija", 0.0 },
  { "Beila", 0.0 },
  { "Beily", 0.0 },
  { "Beira", 0.0 },
  { "Bejamin", 1.0 },
  { "Bejan", 1.0 },
  { "Bek", 1.0 },
  { "Beka", 1.0 },
  { "Bekah", 0.0 },
  { "Bekam", 1.0 },
  { "Bekett", 1.0 },
  { "Bekham", 1.0 },
  { "Beki", 0.0 },
  { "Bekim", 1.0 },
  { "Bekka", 0.0 },
  { "Bekki", 0.0 },
  { "Bela", 0.4467881 },
  { "Belah", 0.0 },
  { "Belal", 1.0 },
  { "Belana", 0.0 },
  { "Belanna", 0.0 },
  { "Belany", 0.0 },
  { "Belarmino", 1.0 },
  { "Belden", 1.0 },
  { "Beldon", 1.0 },
  { "Belem", 0.0 },
  { "Belen", 0.0068552773 },
  { "Belenda", 0.0 },
  { "Beleta", 0.0 },
  { "Belford", 1.0 },
  { "Belgica", 0.0 },
  { "Belia", 0.0 },
  { "Belicia", 0.0 },
  { "Believe", 0.0 },
  { "Belina", 0.0 },
  { "Belinda", 0.0020874504 },
  { "Belisa", 0.0 },
  { "Belisario", 1.0 },
  { "Belissa", 0.0 },
  { "Belita", 0.0 },
  { "Beliz", 0.0 },
  { "Belize", 0.0 },
  { "Belkis", 0.0 },
  { "Belky", 0.0 },
  { "Belkys", 0.0 },
  { "Bell", 0.050801095 },
  { "Bella", 0.0006442444 },
  { "Bellaann", 0.0 },
  { "Belladonna", 0.0 },
  { "Bellagrace", 0.0 },
  { "Bellah", 0.0 },
  { "Bellaluna", 0.0 },
  { "Bellalynn", 0.0 },
  { "Bellamae", 0.0 },
  { "Bellamarie", 0.0 },
  { "Bellamay", 0.0 },
  { "Bellamee", 0.0 },
  { "Bellami", 0.0 },
  { "Bellamia", 0.0 },
  { "Bellamie", 0.0 },
  { "Bellamy", 0.09081197 },
  { "Bellani", 0.0 },
  { "Bellanie", 0.0 },
  { "Bellanira", 0.0 },
  { "Bellany", 0.0 },
  { "Bellarae", 0.0 },
  { "Bellarose", 0.0 },
  { "Bellasophia", 0.0 },
  { "Bellatrix", 0.0 },
  { "Belle", 0.0 },
  { "Bellemy", 0.0 },
  { "Bellicia", 0.0 },
  { "Bellina", 0.0 },
  { "Bellinda", 0.0 },
  { "Bellissa", 0.0 },
  { "Belma", 0.0 },
  { "Belmaris", 0.0 },
  { "Belmeda", 0.0 },
  { "Belmin", 1.0 },
  { "Belmira", 0.0 },
  { "Belmont", 1.0 },
  { "Belon", 0.0 },
  { "Beloved", 0.0 },
  { "Belsy", 0.0 },
  { "Belton", 1.0 },
  { "Belua", 0.0 },
  { "Belva", 0.0 },
  { "Belverly", 0.0 },
  { "Belvia", 0.0 },
  { "Belvie", 0.0 },
  { "Belvin", 0.9834123 },
  { "Belynda", 0.0 },
  { "Belzora", 0.0 },
  { "Bemjamin", 1.0 },
  { "Bemnet", 0.3918919 },
  { "Ben", 0.9944232 },
  { "Bena", 0.0 },
  { "Benae", 0.0 },
  { "Benaiah", 1.0 },
  { "Benajamin", 1.0 },
  { "Benajmin", 1.0 },
  { "Benancia", 0.0 },
  { "Benancio", 1.0 },
  { "Benaniah", 1.0 },
  { "Benard", 1.0 },
  { "Benardo", 1.0 },
  { "Benas", 1.0 },
  { "Benay", 0.0 },
  { "Bence", 1.0 },
  { "Benda", 0.0 },
  { "Bendetta", 0.0 },
  { "Bendrick", 1.0 },
  { "Bene", 0.0 },
  { "Benecia", 0.0 },
  { "Benecio", 1.0 },
  { "Benedek", 1.0 },
  { "Benedetta", 0.0 },
  { "Benedetto", 1.0 },
  { "Benedict", 1.0 },
  { "Benedicta", 0.0 },
  { "Benedicte", 0.0 },
  { "Benedicto", 1.0 },
  { "Benedikt", 1.0 },
  { "Benee", 0.0 },
  { "Benelli", 0.0 },
  { "Benen", 1.0 },
  { "Benesha", 0.0 },
  { "Benessa", 0.0 },
  { "Benet", 0.64912283 },
  { "Beneta", 0.0 },
  { "Benett", 1.0 },
  { "Benetta", 0.0 },
  { "Benette", 0.0 },
  { "Beneva", 0.0 },
  { "Benford", 1.0 },
  { "Bengamin", 1.0 },
  { "Bengi", 1.0 },
  { "Bengie", 1.0 },
  { "Bengiman", 1.0 },
  { "Bengt", 1.0 },
  { "Bengy", 1.0 },
  { "Benham", 1.0 },
  { "Benhamin", 1.0 },
  { "Benhard", 1.0 },
  { "Benhart", 1.0 },
  { "Benhur", 1.0 },
  { "Beni", 0.67346936 },
  { "Beniah", 1.0 },
  { "Beniamin", 1.0 },
  { "Benica", 0.0 },
  { "Benicia", 0.0 },
  { "Benicio", 1.0 },
  { "Benie", 1.0 },
  { "Benigna", 0.0 },
  { "Benigno", 1.0 },
  { "Benika", 0.0 },
  { "Benilda", 0.0 },
  { "Benilde", 0.0 },
  { "Benina", 0.0 },
  { "Benino", 1.0 },
  { "Benisha", 0.0 },
  { "Benita", 0.0 },
  { "Benitez", 1.0 },
  { "Benito", 0.99769294 },
  { "Benjahmin", 1.0 },
  { "Benjain", 1.0 },
  { "Benjaman", 1.0 },
  { "Benjamen", 1.0 },
  { "Benjamim", 1.0 },
  { "Benjamin", 0.99670476 },
  { "Benjamine", 1.0 },
  { "Benjaminjoseph", 1.0 },
  { "Benjammin", 1.0 },
  { "Benjamon", 1.0 },
  { "Benjamyn", 1.0 },
  { "Benje", 1.0 },
  { "Benjeman", 1.0 },
  { "Benjemin", 1.0 },
  { "Benjerman", 1.0 },
  { "Benjermen", 1.0 },
  { "Benjermin", 1.0 },
  { "Benji", 0.9674064 },
  { "Benjiam", 1.0 },
  { "Benjie", 0.96691567 },
  { "Benjiman", 1.0 },
  { "Benjimen", 1.0 },
  { "Benjimin", 1.0 },
  { "Benjmain", 1.0 },
  { "Benjman", 1.0 },
  { "Benjy", 1.0 },
  { "Benley", 1.0 },
  { "Benn", 1.0 },
  { "Benna", 0.0 },
  { "Bennard", 1.0 },
  { "Benne", 0.0 },
  { "Benner", 1.0 },
  { "Bennet", 0.97590363 },
  { "Bennett", 0.97284526 },
  { "Bennetta", 0.0 },
  { "Bennette", 0.32967034 },
  { "Benney", 1.0 },
  { "Benni", 0.0 },
  { "Bennie", 0.8432199 },
  { "Benning", 1.0 },
  { "Bennington", 1.0 },
  { "Bennit", 1.0 },
  { "Bennita", 0.0 },
  { "Bennjamin", 1.0 },
  { "Benno", 1.0 },
  { "Benny", 0.98795766 },
  { "Bennye", 0.0 },
  { "Beno", 1.0 },
  { "Benoit", 1.0 },
  { "Bensen", 1.0 },
  { "Bensley", 1.0 },
  { "Benson", 0.9994237 },
  { "Bensyn", 1.0 },
  { "Bente", 0.0 },
  { "Bentely", 1.0 },
  { "Benten", 1.0 },
  { "Bentlea", 0.0 },
  { "Bentlee", 0.8002624 },
  { "Bentlei", 0.5416667 },
  { "Bentleigh", 0.12958436 },
  { "Bentley", 0.9418058 },
  { "Bentli", 0.3 },
  { "Bentlie", 0.3626943 },
  { "Bently", 0.9671973 },
  { "Bento", 1.0 },
  { "Benton", 1.0 },
  { "Bentura", 0.8958333 },
  { "Bentyn", 1.0 },
  { "Bentz", 1.0 },
  { "Bentzion", 1.0 },
  { "Benuel", 1.0 },
  { "Benvinda", 0.0 },
  { "Beny", 1.0 },
  { "Benyam", 1.0 },
  { "Benyamin", 1.0 },
  { "Benz", 1.0 },
  { "Benzel", 1.0 },
  { "Benzino", 1.0 },
  { "Benzion", 1.0 },
  { "Beola", 0.0 },
  { "Beonca", 0.0 },
  { "Beonce", 0.0 },
  { "Beonica", 0.0 },
  { "Beonka", 0.0 },
  { "Beorn", 1.0 },
  { "Beowulf", 1.0 },
  { "Ber", 1.0 },
  { "Bera", 0.062176164 },
  { "Beranda", 0.0 },
  { "Berania", 0.0 },
  { "Berat", 1.0 },
  { "Berchie", 0.0 },
  { "Berchman", 1.0 },
  { "Berda", 0.0 },
  { "Berdean", 0.0 },
  { "Berdeen", 0.0 },
  { "Berdell", 0.38174275 },
  { "Berdella", 0.0 },
  { "Berdena", 0.0 },
  { "Berdene", 0.0 },
  { "Berdia", 0.0 },
  { "Berdie", 0.0 },
  { "Berdina", 0.0 },
  { "Berdine", 0.0 },
  { "Berdyne", 0.0 },
  { "Berea", 0.0 },
  { "Berek", 1.0 },
  { "Bereket", 1.0 },
  { "Berel", 1.0 },
  { "Beren", 0.81005585 },
  { "Berend", 1.0 },
  { "Berenda", 0.0 },
  { "Berenger", 1.0 },
  { "Berenice", 0.0012200533 },
  { "Berenis", 0.0 },
  { "Berenise", 0.0 },
  { "Berenisse", 0.0 },
  { "Bereniz", 0.0 },
  { "Berenize", 0.0 },
  { "Beresford", 1.0 },
  { "Beret", 0.0 },
  { "Beretta", 0.0 },
  { "Berg", 1.0 },
  { "Bergan", 0.0 },
  { "Berge", 1.0 },
  { "Bergen", 0.43376625 },
  { "Berger", 1.0 },
  { "Bergetta", 0.0 },
  { "Bergin", 0.0 },
  { "Beri", 0.0 },
  { "Beric", 1.0 },
  { "Beril", 1.0 },
  { "Berina", 0.0 },
  { "Berinda", 0.0 },
  { "Berish", 1.0 },
  { "Berit", 0.0 },
  { "Berita", 0.0 },
  { "Berj", 1.0 },
  { "Berk", 1.0 },
  { "Berkay", 1.0 },
  { "Berke", 1.0 },
  { "Berkeley", 0.38708088 },
  { "Berklee", 0.019503547 },
  { "Berkleigh", 0.0 },
  { "Berkley", 0.39888123 },
  { "Berkli", 0.0 },
  { "Berklie", 0.0 },
  { "Berkly", 0.0 },
  { "Berl", 0.99464667 },
  { "Berla", 0.0 },
  { "Berle", 0.96783626 },
  { "Berlena", 0.0 },
  { "Berlene", 0.0 },
  { "Berley", 1.0 },
  { "Berlie", 0.64726025 },
  { "Berlin", 0.5243991 },
  { "Berlina", 0.0 },
  { "Berlinda", 0.0 },
  { "Berline", 0.0 },
  { "Berlon", 1.0 },
  { "Berlyn", 0.100367196 },
  { "Berlynn", 0.0 },
  { "Berma", 0.0 },
  { "Berman", 1.0 },
  { "Bern", 1.0 },
  { "Berna", 0.0 },
  { "Bernabe", 1.0 },
  { "Bernace", 0.0 },
  { "Bernadean", 0.0 },
  { "Bernadeen", 0.0 },
  { "Bernadene", 0.0 },
  { "Bernadet", 0.0 },
  { "Bernadett", 0.0 },
  { "Bernadetta", 0.0 },
  { "Bernadette", 0.0011243114 },
  { "Bernadina", 0.0 },
  { "Bernadine", 0.0 },
  { "Bernadino", 1.0 },
  { "Bernado", 1.0 },
  { "Bernadyne", 0.0 },
  { "Bernal", 1.0 },
  { "Bernaldo", 1.0 },
  { "Bernalee", 0.0 },
  { "Bernando", 1.0 },
  { "Bernard", 0.9950736 },
  { "Bernarda", 0.0 },
  { "Bernardette", 0.0 },
  { "Bernardina", 0.0 },
  { "Bernardine", 0.0 },
  { "Bernardino", 1.0 },
  { "Bernardita", 0.0 },
  { "Bernardo", 1.0 },
  { "Bernarr", 1.0 },
  { "Bernasia", 0.0 },
  { "Bernave", 1.0 },
  { "Bernay", 0.0 },
  { "Bernd", 1.0 },
  { "Bernda", 0.0 },
  { "Berne", 1.0 },
  { "Bernece", 0.0 },
  { "Berneda", 0.0 },
  { "Bernedette", 0.0 },
  { "Berneice", 0.0 },
  { "Berneil", 0.0 },
  { "Berneita", 0.0 },
  { "Bernel", 1.0 },
  { "Bernelda", 0.0 },
  { "Bernell", 0.67419636 },
  { "Bernella", 0.0 },
  { "Bernelle", 0.0 },
  { "Bernerd", 1.0 },
  { "Bernese", 0.0 },
  { "Bernesha", 0.0 },
  { "Bernessa", 0.0 },
  { "Bernessia", 0.0 },
  { "Bernest", 1.0 },
  { "Bernestine", 0.0 },
  { "Bernet", 1.0 },
  { "Berneta", 0.0 },
  { "Bernetha", 0.0 },
  { "Bernett", 0.33766234 },
  { "Bernetta", 0.0 },
  { "Bernette", 0.0 },
  { "Berney", 1.0 },
  { "Bernhard", 1.0 },
  { "Bernhardt", 1.0 },
  { "Bernhart", 1.0 },
  { "Berni", 0.0 },
  { "Bernia", 0.0 },
  { "Bernice", 0.018381042 },
  { "Bernicia", 0.0 },
  { "Bernida", 0.0 },
  { "Bernie", 0.85596246 },
  { "Berniece", 0.0 },
  { "Bernina", 0.0 },
  { "Bernis", 0.8 },
  { "Bernise", 0.0 },
  { "Bernisha", 0.0 },
  { "Bernistine", 0.0 },
  { "Bernita", 0.0 },
  { "Bernitha", 0.0 },
  { "Bernitta", 0.0 },
  { "Berniya", 0.0 },
  { "Bernnadette", 0.0 },
  { "Bernon", 1.0 },
  { "Bernt", 1.0 },
  { "Berny", 1.0 },
  { "Bernyce", 0.0 },
  { "Beronica", 0.0 },
  { "Berra", 0.0 },
  { "Berrett", 1.0 },
  { "Berri", 0.0 },
  { "Berrie", 0.0 },
  { "Berrin", 1.0 },
  { "Berry", 0.9422917 },
  { "Berryman", 1.0 },
  { "Bersain", 1.0 },
  { "Bert", 0.98075795 },
  { "Berta", 0.0 },
  { "Berte", 0.0 },
  { "Berteal", 0.0 },
  { "Bertel", 1.0 },
  { "Bertell", 1.0 },
  { "Bertella", 0.0 },
  { "Bertelle", 0.0 },
  { "Berten", 1.0 },
  { "Bertena", 0.0 },
  { "Berth", 0.0 },
  { "Bertha", 0.0044888426 },
  { "Berthal", 0.73913044 },
  { "Berthalee", 0.0 },
  { "Berthamae", 0.0 },
  { "Berthe", 0.0 },
  { "Berthel", 0.67741936 },
  { "Berthella", 0.0 },
  { "Berthena", 0.0 },
  { "Berthenia", 0.0 },
  { "Berther", 0.0 },
  { "Berthia", 0.0 },
  { "Berthina", 0.0 },
  { "Berthine", 0.0 },
  { "Berthol", 1.0 },
  { "Berthold", 1.0 },
  { "Bertia", 0.0 },
  { "Bertice", 0.20091324 },
  { "Bertie", 0.0590467 },
  { "Bertil", 1.0 },
  { "Bertile", 0.0 },
  { "Bertilla", 0.0 },
  { "Bertille", 0.0 },
  { "Bertin", 1.0 },
  { "Bertina", 0.0 },
  { "Bertine", 0.0 },
  { "Bertis", 0.95991564 },
  { "Berto", 1.0 },
  { "Berton", 1.0 },
  { "Bertram", 1.0 },
  { "Bertran", 1.0 },
  { "Bertrand", 1.0 },
  { "Bertrice", 0.0 },
  { "Bertrum", 1.0 },
  { "Bertus", 1.0 },
  { "Bervin", 1.0 },
  { "Berwick", 1.0 },
  { "Berwin", 1.0 },
  { "Berwyn", 1.0 },
  { "Beryl", 0.1887508 },
  { "Beryle", 0.14596273 },
  { "Besa", 0.0 },
  { "Besan", 0.0 },
  { "Besart", 1.0 },
  { "Beshoy", 1.0 },
  { "Besiana", 0.0 },
  { "Besim", 1.0 },
  { "Besma", 0.0 },
  { "Besnik", 1.0 },
  { "Bess", 0.0006997901 },
  { "Besse", 0.0 },
  { "Bessie", 0.0043162266 },
  { "Bessiemae", 0.0 },
  { "Bessy", 0.0 },
  { "Bessye", 0.0 },
  { "Besty", 0.0 },
  { "Beta", 0.0 },
  { "Betania", 0.0 },
  { "Betanya", 0.0 },
  { "Betel", 0.0 },
  { "Beth", 0.002039203 },
  { "Betha", 0.0 },
  { "Bethal", 0.0 },
  { "Bethan", 0.0 },
  { "Bethanee", 0.0 },
  { "Bethaney", 0.0 },
  { "Bethani", 0.0 },
  { "Bethania", 0.0 },
  { "Bethanie", 0.0 },
  { "Bethann", 0.0 },
  { "Bethanne", 0.0 },
  { "Bethanni", 0.0 },
  { "Bethannie", 0.0 },
  { "Bethanny", 0.0 },
  { "Bethany", 0.0019029889 },
  { "Bethanya", 0.0 },
  { "Bethanye", 0.0 },
  { "Bethe", 0.0 },
  { "Bethea", 0.0 },
  { "Bethel", 0.17443688 },
  { "Bethellen", 0.0 },
  { "Bethena", 0.0 },
  { "Bethene", 0.0 },
  { "Bethenia", 0.0 },
  { "Bethenny", 0.0 },
  { "Betheny", 0.0 },
  { "Bethia", 0.0 },
  { "Bethie", 0.0 },
  { "Bethina", 0.0 },
  { "Bethine", 0.0 },
  { "Bethlehem", 0.0 },
  { "Bethlyn", 0.0 },
  { "Bethney", 0.0 },
  { "Bethony", 0.0 },
  { "Bethsaida", 0.0 },
  { "Bethsheba", 0.0 },
  { "Bethsy", 0.0 },
  { "Bethy", 0.0 },
  { "Bethyl", 0.0 },
  { "Bethzabel", 0.0 },
  { "Bethzabeth", 0.0 },
  { "Bethzaida", 0.0 },
  { "Bethzaira", 0.0 },
  { "Bethzi", 0.0 },
  { "Bethzy", 0.0 },
  { "Betim", 1.0 },
  { "Betina", 0.0 },
  { "Beto", 1.0 },
  { "Betony", 0.0 },
  { "Betrice", 0.0 },
  { "Betrina", 0.0 },
  { "Betsabe", 0.0 },
  { "Betsabet", 0.0 },
  { "Betsabeth", 0.0 },
  { "Betsaida", 0.0 },
  { "Betsayda", 0.0 },
  { "Betselot", 0.0 },
  { "Betsey", 0.0 },
  { "Betsi", 0.0 },
  { "Betsie", 0.0 },
  { "Betsua", 0.0 },
  { "Betsy", 0.00020742585 },
  { "Betsye", 0.0 },
  { "Bett", 0.0 },
  { "Betta", 0.0 },
  { "Bette", 0.0006313494 },
  { "Betteann", 0.0 },
  { "Bettee", 0.0 },
  { "Bettejane", 0.0 },
  { "Bettejean", 0.0 },
  { "Bettejo", 0.0 },
  { "Bettelou", 0.0 },
  { "Bettey", 0.0 },
  { "Betti", 0.0 },
  { "Bettiann", 0.0 },
  { "Bettianne", 0.0 },
  { "Bettie", 0.0021618523 },
  { "Bettijane", 0.0 },
  { "Bettilou", 0.0 },
  { "Bettina", 0.0 },
  { "Bettine", 0.0 },
  { "Bettsy", 0.0 },
  { "Betty", 0.003608557 },
  { "Bettyann", 0.0 },
  { "Bettyanne", 0.0 },
  { "Bettye", 0.0020463252 },
  { "Bettyjane", 0.0 },
  { "Bettyjean", 0.0 },
  { "Bettyjo", 0.0 },
  { "Bettylee", 0.0 },
  { "Bettylou", 0.0 },
  { "Bettylu", 0.0 },
  { "Bettymae", 0.0 },
  { "Bettymarie", 0.0 },
  { "Bettzy", 0.0 },
  { "Betul", 0.0 },
  { "Bety", 0.0 },
  { "Betzabe", 0.0 },
  { "Betzabel", 0.0 },
  { "Betzabet", 0.0 },
  { "Betzabeth", 0.0 },
  { "Betzaida", 0.0 },
  { "Betzaira", 0.0 },
  { "Betzalel", 1.0 },
  { "Betzayda", 0.0 },
  { "Betzayra", 0.0 },
  { "Betzhy", 0.0 },
  { "Betzi", 0.0 },
  { "Betzy", 0.0 },
  { "Beuford", 1.0 },
  { "Beula", 0.0 },
  { "Beulah", 0.002906025 },
  { "Beuna", 0.0 },
  { "Beunca", 0.0 },
  { "Beunka", 0.0 },
  { "Bev", 0.0049850447 },
  { "Beva", 0.0 },
  { "Bevan", 0.89144737 },
  { "Bevely", 0.090277776 },
  { "Bevelyn", 0.0 },
  { "Bevereley", 0.0 },
  { "Beverely", 0.0 },
  { "Beverle", 0.0 },
  { "Beverlee", 0.0 },
  { "Beverley", 0.029688902 },
  { "Beverli", 0.0 },
  { "Beverlie", 0.0 },
  { "Beverlin", 0.0 },
  { "Beverly", 0.0121478615 },
  { "Beverlyann", 0.0 },
  { "Beverlye", 0.0 },
  { "Beverlyn", 0.0 },
  { "Bevery", 0.0 },
  { "Bevie", 0.0 },
  { "Bevin", 0.11305872 },
  { "Bevis", 1.0 },
  { "Bevon", 1.0 },
  { "Bevyn", 0.0 },
  { "Bex", 1.0 },
  { "Bexar", 1.0 },
  { "Bexlee", 0.0 },
  { "Bexleigh", 0.0 },
  { "Bexley", 0.06861314 },
  { "Bexli", 0.0 },
  { "Bexlie", 0.0 },
  { "Bexton", 1.0 },
  { "Beya", 0.0 },
  { "Beyah", 0.0 },
  { "Beyan", 1.0 },
  { "Beyanka", 0.0 },
  { "Beyda", 0.0 },
  { "Beyla", 0.0 },
  { "Beyonca", 0.0 },
  { "Beyonce", 0.0 },
  { "Beyoncee", 0.0 },
  { "Beyonka", 0.0 },
  { "Beyounce", 0.0 },
  { "Beyunka", 0.0 },
  { "Beyza", 0.0 },
  { "Beza", 0.0 },
  { "Bezaleel", 1.0 },
  { "Bezalel", 1.0 },
  { "Bezawit", 0.0 },
  { "Bg", 0.0 },
  { "Bhakti", 0.0 },
  { "Bharat", 1.0 },
  { "Bharath", 1.0 },
  { "Bhargav", 1.0 },
  { "Bhargava", 1.0 },
  { "Bhargavi", 0.0 },
  { "Bhavana", 0.0 },
  { "Bhavani", 0.0 },
  { "Bhavesh", 1.0 },
  { "Bhavi", 0.0 },
  { "Bhavik", 1.0 },
  { "Bhavika", 0.0 },
  { "Bhavin", 1.0 },
  { "Bhavini", 0.0 },
  { "Bhavna", 0.0 },
  { "Bhavya", 0.19944598 },
  { "Bhoomi", 0.0 },
  { "Bhoomika", 0.0 },
  { "Bhrianna", 0.0 },
  { "Bhumi", 0.0 },
  { "Bhumika", 0.0 },
  { "Bhuvan", 1.0 },
  { "Bhuvi", 0.0 },
  { "Bia", 0.0 },
  { "Biaca", 0.0 },
  { "Biafra", 0.0 },
  { "Biaggio", 1.0 },
  { "Biagio", 1.0 },
  { "Biak", 0.17142858 },
  { "Biana", 0.0 },
  { "Bianaca", 0.0 },
  { "Bianca", 0.0027338876 },
  { "Biancamarie", 0.0 },
  { "Biancca", 0.0 },
  { "Biance", 0.0 },
  { "Biancha", 0.0 },
  { "Biancia", 0.0 },
  { "Bianco", 0.1125 },
  { "Bianet", 0.0 },
  { "Bianeth", 0.0 },
  { "Bianey", 0.0 },
  { "Bianica", 0.0 },
  { "Bianka", 0.0 },
  { "Biankah", 0.0 },
  { "Bianna", 0.0 },
  { "Biannca", 0.0 },
  { "Bianney", 0.0 },
  { "Bianny", 0.0 },
  { "Biatris", 0.0 },
  { "Biatriz", 0.0 },
  { "Biaunca", 0.0 },
  { "Bibb", 1.0 },
  { "Bibek", 1.0 },
  { "Bibi", 0.0 },
  { "Bibian", 0.0 },
  { "Bibiana", 0.0 },
  { "Bibianna", 0.0 },
  { "Bibiano", 1.0 },
  { "Bicente", 1.0 },
  { "Bich", 0.0 },
  { "Bictor", 1.0 },
  { "Bidal", 1.0 },
  { "Biddie", 0.0 },
  { "Biddy", 0.0 },
  { "Bidwell", 1.0 },
  { "Biel", 1.0 },
  { "Bielka", 0.0 },
  { "Bienvenida", 0.0 },
  { "Bienvenido", 1.0 },
  { "Biff", 1.0 },
  { "Biftu", 0.0 },
  { "Bige", 1.0 },
  { "Bihaan", 1.0 },
  { "Bijal", 0.0 },
  { "Bijan", 0.97091275 },
  { "Bijon", 0.8 },
  { "Bijou", 0.0 },
  { "Bijoux", 0.0 },
  { "Biju", 1.0 },
  { "Biko", 1.0 },
  { "Bikram", 1.0 },
  { "Bilaal", 1.0 },
  { "Bilal", 0.9988548 },
  { "Bilan", 0.0 },
  { "Bilbo", 1.0 },
  { "Bilen", 0.0 },
  { "Bilguun", 1.0 },
  { "Bilinda", 0.0 },
  { "Biljana", 0.0 },
  { "Bill", 0.9955514 },
  { "Billa", 0.0 },
  { "Billal", 1.0 },
  { "Bille", 0.4509804 },
  { "Billee", 0.008250825 },
  { "Billey", 1.0 },
  { "Billi", 0.0028968714 },
  { "Billie", 0.23764628 },
  { "Billiejean", 0.0 },
  { "Billiejo", 0.0 },
  { "Billiejoe", 0.0 },
  { "Billijo", 0.0 },
  { "Billion", 1.0 },
  { "Billison", 1.0 },
  { "Billy", 0.98618674 },
  { "Billye", 0.032258064 },
  { "Billyjack", 1.0 },
  { "Billyjo", 0.43589744 },
  { "Billyjoe", 1.0 },
  { "Billyray", 1.0 },
  { "Bilma", 0.0 },
  { "Bilol", 1.0 },
  { "Bilon", 0.0 },
  { "Bilqis", 0.0 },
  { "Bily", 1.0 },
  { "Bimal", 1.0 },
  { "Bimbo", 1.0 },
  { "Bin", 1.0 },
  { "Bina", 0.0 },
  { "Binaca", 0.0 },
  { "Binah", 0.0 },
  { "Binca", 0.0 },
  { "Bindi", 0.0 },
  { "Bindu", 0.0 },
  { "Bindy", 0.0 },
  { "Bineta", 0.0 },
  { "Binford", 1.0 },
  { "Bing", 1.0 },
  { "Bingham", 1.0 },
  { "Binh", 0.92621356 },
  { "Binisha", 0.0 },
  { "Binita", 0.0 },
  { "Binky", 0.0 },
  { "Binnie", 0.21367522 },
  { "Binta", 0.0 },
  { "Binti", 0.0 },
  { "Bintou", 0.0 },
  { "Bintu", 0.0 },
  { "Binu", 1.0 },
  { "Binyamin", 1.0 },
  { "Binyomin", 1.0 },
  { "Bioleta", 0.0 },
  { "Bion", 1.0 },
  { "Bionca", 0.0 },
  { "Bionce", 0.0 },
  { "Bionda", 0.0 },
  { "Bionka", 0.0 },
  { "Biran", 1.0 },
  { "Biranna", 0.0 },
  { "Birch", 1.0 },
  { "Birchard", 1.0 },
  { "Birche", 1.0 },
  { "Birchel", 1.0 },
  { "Birchie", 0.0 },
  { "Bird", 0.47391304 },
  { "Birda", 0.0 },
  { "Birdell", 0.054347824 },
  { "Birdella", 0.0 },
  { "Birdena", 0.0 },
  { "Birdene", 0.0 },
  { "Birder", 0.0 },
  { "Birdia", 0.0 },
  { "Birdie", 0.008691374 },
  { "Birdine", 0.0 },
  { "Birdye", 0.0 },
  { "Biren", 1.0 },
  { "Birger", 1.0 },
  { "Birgit", 0.0 },
  { "Birgitta", 0.0 },
  { "Birgitte", 0.0 },
  { "Biridiana", 0.0 },
  { "Birk", 1.0 },
  { "Birklee", 0.0 },
  { "Birkley", 0.0 },
  { "Birl", 1.0 },
  { "Birney", 1.0 },
  { "Biron", 1.0 },
  { "Birt", 0.98134327 },
  { "Birta", 0.0 },
  { "Birtha", 0.0 },
  { "Birthel", 0.0 },
  { "Birtie", 0.0 },
  { "Birttany", 0.0 },
  { "Birty", 0.0 },
  { "Biruk", 1.0 },
  { "Biruta", 0.0 },
  { "Birute", 0.0 },
  { "Birydiana", 0.0 },
  { "Bisa", 0.0 },
  { "Bisan", 0.0 },
  { "Bisceglia", 0.0 },
  { "Bisente", 1.0 },
  { "Bishara", 1.0 },
  { "Bisharo", 0.0 },
  { "Bisher", 1.0 },
  { "Bishop", 0.99903715 },
  { "Bishoy", 1.0 },
  { "Bisleidy", 0.0 },
  { "Bisma", 0.0 },
  { "Bismah", 0.0 },
  { "Bismarck", 1.0 },
  { "Bismark", 1.0 },
  { "Bisola", 0.0 },
  { "Bison", 1.0 },
  { "Bissan", 0.0 },
  { "Bissie", 0.0 },
  { "Bita", 0.0 },
  { "Bitania", 0.0 },
  { "Bitanya", 0.0 },
  { "Bitha", 0.0 },
  { "Bithiah", 0.0 },
  { "Bitia", 0.0 },
  { "Bitsy", 0.0 },
  { "Bittany", 0.0 },
  { "Bivian", 0.41666666 },
  { "Biviana", 0.0 },
  { "Bix", 1.0 },
  { "Bixby", 0.5555556 },
  { "Biyanca", 0.0 },
  { "Biyanka", 0.0 },
  { "Bj", 0.992785 },
  { "Bjana", 0.0 },
  { "Bjarne", 1.0 },
  { "Bjay", 1.0 },
  { "Bjorn", 1.0 },
  { "Bla", 0.0 },
  { "Blace", 1.0 },
  { "Blackie", 1.0 },
  { "Blade", 1.0 },
  { "Bladen", 1.0 },
  { "Bladimir", 1.0 },
  { "Bladyn", 1.0 },
  { "Blaid", 1.0 },
  { "Blaide", 1.0 },
  { "Blaiden", 1.0 },
  { "Blaidyn", 1.0 },
  { "Blaik", 1.0 },
  { "Blaike", 0.5872483 },
  { "Blaiklee", 0.0 },
  { "Blaikley", 0.0 },
  { "Blain", 0.99236643 },
  { "Blaine", 0.96781564 },
  { "Blair", 0.5145104 },
  { "Blaire", 0.015468347 },
  { "Blaise", 0.9286883 },
  { "Blaiz", 1.0 },
  { "Blaize", 0.89542896 },
  { "Blake", 0.95416826 },
  { "Blakelee", 0.03358209 },
  { "Blakeleigh", 0.0 },
  { "Blakeley", 0.123786405 },
  { "Blakeli", 0.0 },
  { "Blakelie", 0.0 },
  { "Blakely", 0.15613067 },
  { "Blakelyn", 0.0 },
  { "Blakelynn", 0.0 },
  { "Blaken", 1.0 },
  { "Blakeney", 0.0 },
  { "Blakesley", 0.0 },
  { "Blaklee", 0.0 },
  { "Blakleigh", 0.0 },
  { "Blakley", 0.1408046 },
  { "Blaklie", 0.0 },
  { "Blakney", 0.0 },
  { "Blan", 1.0 },
  { "Blanca", 0.005101008 },
  { "Blancaestela", 0.0 },
  { "Blanch", 0.004651163 },
  { "Blancha", 0.0 },
  { "Blanchard", 1.0 },
  { "Blanche", 0.002546389 },
  { "Blanchie", 0.0 },
  { "Bland", 1.0 },
  { "Blandina", 0.0 },
  { "Blandine", 0.0 },
  { "Blandon", 1.0 },
  { "Blandy", 0.0 },
  { "Blane", 0.9973778 },
  { "Blaney", 0.41666666 },
  { "Blanka", 0.0 },
  { "Blannie", 0.0 },
  { "Blanton", 1.0 },
  { "Blas", 1.0 },
  { "Blasa", 0.0 },
  { "Blase", 1.0 },
  { "Blass", 1.0 },
  { "Blayd", 1.0 },
  { "Blayde", 1.0 },
  { "Blayden", 1.0 },
  { "Blaydin", 1.0 },
  { "Blaydon", 1.0 },
  { "Blayk", 1.0 },
  { "Blayke", 0.53259003 },
  { "Blayklee", 0.0 },
  { "Blaykleigh", 0.0 },
  { "Blayn", 1.0 },
  { "Blayne", 0.89957213 },
  { "Blayr", 0.0 },
  { "Blayre", 0.0 },
  { "Blayse", 1.0 },
  { "Blayson", 1.0 },
  { "Blayten", 1.0 },
  { "Blayton", 1.0 },
  { "Blayz", 1.0 },
  { "Blayze", 0.9555302 },
  { "Blaz", 1.0 },
  { "Blaza", 0.0 },
  { "Blaze", 0.9344554 },
  { "Blazen", 1.0 },
  { "Blazer", 1.0 },
  { "Blease", 1.0 },
  { "Bleeker", 0.0 },
  { "Bleidy", 0.0 },
  { "Blen", 0.0 },
  { "Blenda", 0.0 },
  { "Blendia", 0.0 },
  { "Bleona", 0.0 },
  { "Blerim", 1.0 },
  { "Blerina", 0.0 },
  { "Blerta", 0.0 },
  { "Bless", 0.5246753 },
  { "Blessed", 0.30991736 },
  { "Blessen", 0.0 },
  { "Blessence", 0.0 },
  { "Blessin", 0.0 },
  { "Blessing", 0.078086674 },
  { "Blessings", 0.0 },
  { "Blessn", 0.0 },
  { "Blesson", 1.0 },
  { "Blessy", 0.0 },
  { "Blessyn", 0.0 },
  { "Bleu", 0.4814815 },
  { "Blia", 0.0 },
  { "Blima", 0.0 },
  { "Blimi", 0.0 },
  { "Blimie", 0.0 },
  { "Blimy", 0.0 },
  { "Blin", 1.0 },
  { "Blinda", 0.0 },
  { "Bliss", 0.12750717 },
  { "Blissany", 0.0 },
  { "Blithe", 0.0 },
  { "Blodwen", 0.0 },
  { "Blodwyn", 0.0 },
  { "Blon", 0.0 },
  { "Blondean", 0.0 },
  { "Blondell", 0.010073875 },
  { "Blondena", 0.0 },
  { "Blondie", 0.0 },
  { "Blondina", 0.0 },
  { "Blondine", 0.0 },
  { "Bloneva", 0.0 },
  { "Blong", 1.0 },
  { "Blonnie", 0.0 },
  { "Bloomie", 0.0 },
  { "Blossie", 0.0 },
  { "Blossom", 0.0 },
  { "Blouncie", 1.0 },
  { "Blu", 0.65986395 },
  { "Blue", 0.7081281 },
  { "Bluford", 1.0 },
  { "Bluma", 0.0 },
  { "Bly", 0.90909094 },
  { "Blynda", 0.0 },
  { "Blynn", 1.0 },
  { "Blyss", 0.0 },
  { "Blyth", 0.0 },
  { "Blythe", 0.048230786 },
  { "Bnai", 0.0 },
  { "Bo", 0.9402885 },
  { "Boady", 1.0 },
  { "Boaz", 1.0 },
  { "Bob", 0.9965455 },
  { "Boback", 1.0 },
  { "Bobak", 1.0 },
  { "Boban", 1.0 },
  { "Bobb", 1.0 },
  { "Bobbe", 0.0 },
  { "Bobbee", 0.0 },
  { "Bobbetta", 0.0 },
  { "Bobbette", 0.0 },
  { "Bobbi", 0.0027093387 },
  { "Bobbie", 0.17258449 },
  { "Bobbiejean", 0.0 },
  { "Bobbiejo", 0.0 },
  { "Bobbielee", 0.0 },
  { "Bobbiesue", 0.0 },
  { "Bobbijo", 0.0 },
  { "Bobbilee", 0.0 },
  { "Bobbilynn", 0.0 },
  { "Bobbisue", 0.0 },
  { "Bobby", 0.9701819 },
  { "Bobbye", 0.029681763 },
  { "Bobbyetta", 0.0 },
  { "Bobbyjo", 0.0 },
  { "Bobbyjoe", 1.0 },
  { "Bobbylee", 1.0 },
  { "Bobetta", 0.0 },
  { "Bobette", 0.0 },
  { "Bobi", 0.0 },
  { "Bobie", 0.47302383 },
  { "Bobijo", 0.0 },
  { "Bobo", 1.0 },
  { "Bobra", 0.0 },
  { "Boby", 1.0 },
  { "Bocar", 1.0 },
  { "Bocephus", 1.0 },
  { "Bodan", 1.0 },
  { "Bode", 1.0 },
  { "Bodean", 1.0 },
  { "Bodee", 0.98762375 },
  { "Boden", 1.0 },
  { "Bodey", 1.0 },
  { "Bodhan", 1.0 },
  { "Bodhi", 0.97088605 },
  { "Bodhin", 1.0 },
  { "Bodi", 1.0 },
  { "Bodie", 0.9982906 },
  { "Bodin", 1.0 },
  { "Bodyn", 1.0 },
  { "Boe", 0.99100256 },
  { "Boede", 1.0 },
  { "Boedy", 1.0 },
  { "Boen", 1.0 },
  { "Boey", 0.0 },
  { "Bogar", 1.0 },
  { "Bogart", 1.0 },
  { "Bogdan", 1.0 },
  { "Bogdana", 0.0 },
  { "Bohan", 1.0 },
  { "Bohannon", 1.0 },
  { "Bohdan", 1.0 },
  { "Bohde", 1.0 },
  { "Bohden", 1.0 },
  { "Bohdi", 1.0 },
  { "Bohen", 1.0 },
  { "Bohumil", 1.0 },
  { "Boice", 1.0 },
  { "Boise", 1.0 },
  { "Boisey", 1.0 },
  { "Boisy", 1.0 },
  { "Bojan", 1.0 },
  { "Bol", 1.0 },
  { "Bolaji", 1.0 },
  { "Bolanle", 0.0 },
  { "Bolden", 1.0 },
  { "Boleslaus", 1.0 },
  { "Boleslaw", 1.0 },
  { "Bolin", 1.0 },
  { "Bolish", 1.0 },
  { "Bolivar", 1.0 },
  { "Bolivia", 0.0 },
  { "Bolling", 1.0 },
  { "Bolton", 1.0 },
  { "Boluwatife", 0.5794392 },
  { "Bomani", 1.0 },
  { "Bomer", 1.0 },
  { "Bon", 1.0 },
  { "Bona", 0.05376344 },
  { "Bonalyn", 0.0 },
  { "Bonanza", 1.0 },
  { "Bonard", 1.0 },
  { "Boncile", 0.0 },
  { "Bond", 0.97029704 },
  { "Bonda", 0.0 },
  { "Boneita", 0.0 },
  { "Bonell", 0.5 },
  { "Boneta", 0.0 },
  { "Bonetta", 0.0 },
  { "Boneva", 0.0 },
  { "Boney", 1.0 },
  { "Bong", 1.0 },
  { "Bonham", 1.0 },
  { "Boni", 0.0 },
  { "Bonieta", 0.0 },
  { "Boniface", 1.0 },
  { "Bonifacia", 0.0 },
  { "Bonifacio", 1.0 },
  { "Bonique", 0.0 },
  { "Bonita", 0.00031804087 },
  { "Bonitta", 0.0 },
  { "Bonna", 0.0 },
  { "Bonne", 0.0 },
  { "Bonnee", 0.0 },
  { "Bonnell", 0.0 },
  { "Bonnelle", 0.0 },
  { "Bonner", 1.0 },
  { "Bonnetta", 0.0 },
  { "Bonnette", 0.0 },
  { "Bonney", 0.011061947 },
  { "Bonni", 0.0 },
  { "Bonnibel", 0.0 },
  { "Bonnie", 0.012307204 },
  { "Bonniejean", 0.0 },
  { "Bonniejo", 0.0 },
  { "Bonnielee", 0.0 },
  { "Bonnielou", 0.0 },
  { "Bonnita", 0.0 },
  { "Bonnitta", 0.0 },
  { "Bonny", 0.04057205 },
  { "Bonnye", 0.0 },
  { "Bonzie", 0.0 },
  { "Booker", 0.9995468 },
  { "Bookert", 1.0 },
  { "Boomer", 1.0 },
  { "Boon", 1.0 },
  { "Boone", 1.0 },
  { "Booth", 1.0 },
  { "Boots", 0.18300654 },
  { "Bopha", 0.0 },
  { "Bora", 0.76513314 },
  { "Borach", 1.0 },
  { "Boran", 1.0 },
  { "Borden", 1.0 },
  { "Borghild", 0.0 },
  { "Borgny", 0.0 },
  { "Boris", 1.0 },
  { "Borja", 1.0 },
  { "Born", 1.0 },
  { "Borna", 1.0 },
  { "Boruch", 1.0 },
  { "Borys", 1.0 },
  { "Bosco", 1.0 },
  { "Bose", 1.0 },
  { "Bosede", 0.0 },
  { "Bosie", 1.0 },
  { "Boss", 1.0 },
  { "Bossie", 1.0 },
  { "Bosten", 1.0 },
  { "Bostin", 1.0 },
  { "Boston", 0.9167549 },
  { "Bostyn", 0.30057803 },
  { "Bostynn", 0.0 },
  { "Boswell", 1.0 },
  { "Boubacar", 1.0 },
  { "Boulder", 1.0 },
  { "Boun", 1.0 },
  { "Bourne", 1.0 },
  { "Bouvier", 1.0 },
  { "Bow", 0.9655172 },
  { "Bowan", 1.0 },
  { "Bowden", 1.0 },
  { "Bowdie", 1.0 },
  { "Bowdrie", 1.0 },
  { "Bowdy", 1.0 },
  { "Bowe", 1.0 },
  { "Bowen", 0.9831263 },
  { "Bower", 1.0 },
  { "Bowie", 0.6741706 },
  { "Bowin", 1.0 },
  { "Bowman", 1.0 },
  { "Bowyn", 0.875 },
  { "Boy", 1.0 },
  { "Boyan", 1.0 },
  { "Boyce", 0.9893928 },
  { "Boyd", 0.997582 },
  { "Boyde", 1.0 },
  { "Boyden", 1.0 },
  { "Boyed", 1.0 },
  { "Boyer", 1.0 },
  { "Boykin", 1.0 },
  { "Boynton", 1.0 },
  { "Boysie", 1.0 },
  { "Boz", 1.0 },
  { "Bozena", 0.0 },
  { "Brace", 1.0 },
  { "Bracelyn", 0.0 },
  { "Bracelynn", 0.0 },
  { "Bracen", 1.0 },
  { "Bracey", 0.59183675 },
  { "Brach", 1.0 },
  { "Bracha", 0.0 },
  { "Bracie", 0.0 },
  { "Brack", 1.0 },
  { "Bracken", 1.0 },
  { "Brackston", 1.0 },
  { "Bracy", 0.9425287 },
  { "Bracyn", 1.0 },
  { "Brad", 0.99719274 },
  { "Bradan", 1.0 },
  { "Bradbury", 1.0 },
  { "Bradd", 1.0 },
  { "Bradden", 1.0 },
  { "Braddock", 1.0 },
  { "Braddox", 1.0 },
  { "Bradee", 0.29464287 },
  { "Bradely", 1.0 },
  { "Braden", 0.99376553 },
  { "Braderick", 1.0 },
  { "Bradey", 0.9872881 },
  { "Bradfield", 1.0 },
  { "Bradford", 0.99876857 },
  { "Bradi", 0.0 },
  { "Bradie", 0.29689807 },
  { "Bradin", 1.0 },
  { "Bradlee", 0.9317305 },
  { "Bradleigh", 0.03343465 },
  { "Bradley", 0.995082 },
  { "Bradli", 0.0 },
  { "Bradlie", 0.18965517 },
  { "Bradly", 0.99880123 },
  { "Bradlyn", 1.0 },
  { "Bradney", 1.0 },
  { "Bradnon", 1.0 },
  { "Bradon", 1.0 },
  { "Bradrick", 1.0 },
  { "Bradshaw", 1.0 },
  { "Bradson", 1.0 },
  { "Brady", 0.98064405 },
  { "Bradyen", 1.0 },
  { "Bradyn", 0.9513562 },
  { "Bradynn", 0.75 },
  { "Brae", 0.38848922 },
  { "Braecyn", 1.0 },
  { "Braedan", 1.0 },
  { "Braeden", 0.9824307 },
  { "Braedin", 1.0 },
  { "Braedon", 1.0 },
  { "Braedy", 1.0 },
  { "Braedyn", 0.87907183 },
  { "Braegan", 1.0 },
  { "Braela", 0.0 },
  { "Braelan", 1.0 },
  { "Braelee", 0.0 },
  { "Braelei", 0.0 },
  { "Braeleigh", 0.0 },
  { "Braelen", 1.0 },
  { "Braeley", 0.0 },
  { "Braeli", 0.0 },
  { "Braelie", 0.0 },
  { "Braelin", 0.5754923 },
  { "Braelinn", 0.0 },
  { "Braelon", 1.0 },
  { "Braely", 0.0 },
  { "Braelyn", 0.10013175 },
  { "Braelyne", 0.0 },
  { "Braelynn", 0.021926053 },
  { "Braelynne", 0.0 },
  { "Braesen", 1.0 },
  { "Braeson", 1.0 },
  { "Braesyn", 1.0 },
  { "Braeton", 1.0 },
  { "Braeya", 0.0 },
  { "Brage", 1.0 },
  { "Braham", 1.0 },
  { "Braheem", 1.0 },
  { "Brahian", 1.0 },
  { "Brahim", 1.0 },
  { "Brahin", 1.0 },
  { "Brahm", 1.0 },
  { "Brahms", 1.0 },
  { "Braian", 1.0 },
  { "Braidan", 1.0 },
  { "Braiden", 0.97771263 },
  { "Braidon", 1.0 },
  { "Braidy", 0.26086956 },
  { "Braidyn", 0.85565215 },
  { "Braijon", 1.0 },
  { "Brailee", 0.0 },
  { "Braileigh", 0.0 },
  { "Brailen", 1.0 },
  { "Brailey", 0.0055555557 },
  { "Brailin", 1.0 },
  { "Brailon", 1.0 },
  { "Braily", 0.0 },
  { "Brailyn", 0.25422138 },
  { "Brailynn", 0.031825796 },
  { "Brain", 0.9978242 },
  { "Braina", 0.0 },
  { "Brainard", 1.0 },
  { "Brainna", 0.0 },
  { "Brais", 1.0 },
  { "Braisen", 1.0 },
  { "Braisley", 0.0 },
  { "Braison", 1.0 },
  { "Braiya", 0.0 },
  { "Brajon", 1.0 },
  { "Bralan", 1.0 },
  { "Bralee", 0.0 },
  { "Bralei", 0.0 },
  { "Braleigh", 0.0 },
  { "Bralen", 0.9836601 },
  { "Braley", 0.042801555 },
  { "Bralie", 0.0 },
  { "Bralin", 1.0 },
  { "Brallan", 1.0 },
  { "Bralon", 1.0 },
  { "Bralyn", 0.6061885 },
  { "Bralynn", 0.18787879 },
  { "Bralynne", 0.0 },
  { "Bram", 1.0 },
  { "Bran", 1.0 },
  { "Brana", 0.0 },
  { "Branae", 0.0 },
  { "Branan", 1.0 },
  { "Branasia", 0.0 },
  { "Brance", 1.0 },
  { "Branch", 1.0 },
  { "Branco", 1.0 },
  { "Brancon", 1.0 },
  { "Brancy", 0.0 },
  { "Brand", 0.93406594 },
  { "Branda", 0.0 },
  { "Brandace", 0.0 },
  { "Brandais", 0.0 },
  { "Brandal", 1.0 },
  { "Brandale", 1.0 },
  { "Brandall", 1.0 },
  { "Brandalyn", 0.0 },
  { "Brandalynn", 0.0 },
  { "Brandan", 0.98300886 },
  { "Brandarius", 1.0 },
  { "Brandasia", 0.0 },
  { "Branddon", 1.0 },
  { "Brande", 0.0 },
  { "Brandea", 0.0 },
  { "Brandee", 0.0 },
  { "Brandeis", 0.0 },
  { "Brandel", 0.8 },
  { "Brandell", 0.9381443 },
  { "Brandelyn", 0.0 },
  { "Brandelynn", 0.0 },
  { "Branden", 0.9878774 },
  { "Brandenn", 1.0 },
  { "Brander", 1.0 },
  { "Brandessa", 0.0 },
  { "Brandey", 0.0 },
  { "Brandi", 0.0038212182 },
  { "Brandia", 0.0 },
  { "Brandiann", 0.0 },
  { "Brandice", 0.0 },
  { "Brandie", 0.0018362566 },
  { "Brandii", 0.0 },
  { "Brandilee", 0.0 },
  { "Brandilyn", 0.0 },
  { "Brandilynn", 0.0 },
  { "Brandin", 0.9512108 },
  { "Brandina", 0.0 },
  { "Brandis", 0.1031941 },
  { "Brandise", 0.0 },
  { "Brandisha", 0.0 },
  { "Brandiss", 0.0 },
  { "Brandley", 1.0 },
  { "Brandlyn", 0.0 },
  { "Brandn", 1.0 },
  { "Brando", 1.0 },
  { "Brandol", 1.0 },
  { "Brandolyn", 0.0 },
  { "Brandom", 1.0 },
  { "Brandon", 0.994531 },
  { "Brandonjames", 1.0 },
  { "Brandonkyle", 1.0 },
  { "Brandonlee", 1.0 },
  { "Brandonmichael", 1.0 },
  { "Brandonn", 1.0 },
  { "Brandonray", 1.0 },
  { "Brandson", 1.0 },
  { "Brandt", 0.99317896 },
  { "Brandtley", 1.0 },
  { "Brandtly", 1.0 },
  { "Brandton", 1.0 },
  { "Brandun", 1.0 },
  { "Brandy", 0.020573813 },
  { "Brandyann", 0.0 },
  { "Brandyce", 0.0 },
  { "Brandye", 0.0 },
  { "Brandylee", 0.0 },
  { "Brandylynn", 0.0 },
  { "Brandyn", 0.9153831 },
  { "Braneisha", 0.0 },
  { "Branen", 1.0 },
  { "Branesha", 0.0 },
  { "Branford", 1.0 },
  { "Branham", 1.0 },
  { "Brania", 0.0 },
  { "Braniah", 0.0 },
  { "Branigan", 0.44285715 },
  { "Branisha", 0.0 },
  { "Braniya", 0.0 },
  { "Braniyah", 0.0 },
  { "Branko", 1.0 },
  { "Branlee", 1.0 },
  { "Branly", 1.0 },
  { "Brann", 1.0 },
  { "Branna", 0.0 },
  { "Brannan", 0.9715762 },
  { "Branndon", 1.0 },
  { "Brannen", 1.0 },
  { "Brannick", 1.0 },
  { "Brannigan", 0.6956522 },
  { "Brannock", 1.0 },
  { "Brannon", 0.975 },
  { "Branon", 1.0 },
  { "Branoon", 1.0 },
  { "Bransen", 1.0 },
  { "Bransford", 1.0 },
  { "Branson", 0.9991697 },
  { "Branston", 1.0 },
  { "Bransyn", 1.0 },
  { "Brant", 0.9989072 },
  { "Branten", 1.0 },
  { "Brantlee", 0.8873608 },
  { "Brantleigh", 0.06329114 },
  { "Brantley", 0.96752435 },
  { "Brantlie", 0.0 },
  { "Brantly", 0.9931507 },
  { "Brantlyn", 1.0 },
  { "Branton", 1.0 },
  { "Brantson", 1.0 },
  { "Branwen", 0.0 },
  { "Branwyn", 0.0 },
  { "Brany", 0.0 },
  { "Branya", 0.0 },
  { "Branyah", 0.0 },
  { "Brasen", 1.0 },
  { "Brashad", 1.0 },
  { "Brashaun", 1.0 },
  { "Brashawn", 1.0 },
  { "Brasher", 1.0 },
  { "Brasi", 1.0 },
  { "Brasia", 0.0 },
  { "Brason", 1.0 },
  { "Braston", 1.0 },
  { "Braulia", 0.0 },
  { "Braulio", 1.0 },
  { "Braun", 1.0 },
  { "Brave", 0.8835617 },
  { "Braven", 1.0 },
  { "Bravery", 0.7307692 },
  { "Bravin", 1.0 },
  { "Bravlio", 1.0 },
  { "Bravo", 1.0 },
  { "Bravon", 1.0 },
  { "Bravyn", 1.0 },
  { "Brawley", 1.0 },
  { "Brax", 1.0 },
  { "Braxden", 1.0 },
  { "Braxdon", 1.0 },
  { "Braxdyn", 1.0 },
  { "Braxen", 1.0 },
  { "Braxlee", 0.6315789 },
  { "Braxleigh", 0.0 },
  { "Braxley", 0.6956522 },
  { "Braxon", 1.0 },
  { "Braxson", 1.0 },
  { "Braxsten", 1.0 },
  { "Braxston", 1.0 },
  { "Braxstyn", 1.0 },
  { "Braxtan", 1.0 },
  { "Braxten", 0.9958054 },
  { "Braxtin", 0.9852941 },
  { "Braxton", 0.9838219 },
  { "Braxtyn", 0.8030769 },
  { "Braxtynn", 0.52380955 },
  { "Braxx", 1.0 },
  { "Braxxton", 1.0 },
  { "Bray", 0.9411765 },
  { "Braya", 0.0 },
  { "Brayah", 0.0 },
  { "Brayam", 1.0 },
  { "Brayan", 0.9986982 },
  { "Brayana", 0.0 },
  { "Brayanna", 0.0 },
  { "Brayant", 1.0 },
  { "Brayce", 1.0 },
  { "Braycen", 1.0 },
  { "Braydan", 0.99040306 },
  { "Brayde", 1.0 },
  { "Braydee", 0.073529415 },
  { "Brayden", 0.9912042 },
  { "Braydenn", 1.0 },
  { "Braydi", 0.0 },
  { "Braydin", 0.98854166 },
  { "Braydn", 1.0 },
  { "Braydon", 0.99818224 },
  { "Braydyn", 1.0 },
  { "Brayelle", 0.0 },
  { "Brayen", 1.0 },
  { "Brayhan", 1.0 },
  { "Brayla", 0.0 },
  { "Braylah", 0.0 },
  { "Braylan", 0.9536178 },
  { "Brayland", 1.0 },
  { "Brayle", 0.0 },
  { "Braylea", 0.0 },
  { "Braylee", 0.0147125395 },
  { "Braylei", 0.0 },
  { "Brayleigh", 0.0 },
  { "Braylen", 0.94409513 },
  { "Braylend", 1.0 },
  { "Braylenn", 0.6875 },
  { "Brayley", 0.0 },
  { "Brayli", 0.0 },
  { "Braylie", 0.0 },
  { "Braylin", 0.6454979 },
  { "Braylinn", 0.0 },
  { "Brayln", 1.0 },
  { "Braylon", 0.9919112 },
  { "Braylyn", 0.42281106 },
  { "Braylynn", 0.1041456 },
  { "Braylynne", 0.0 },
  { "Brayn", 1.0 },
  { "Braynt", 1.0 },
  { "Brayon", 1.0 },
  { "Brayonna", 0.0 },
  { "Braysen", 1.0 },
  { "Brayshaun", 1.0 },
  { "Brayshawn", 1.0 },
  { "Brayson", 1.0 },
  { "Brayston", 1.0 },
  { "Brayten", 1.0 },
  { "Brayton", 0.98969555 },
  { "Brayven", 1.0 },
  { "Brayzen", 1.0 },
  { "Brazen", 1.0 },
  { "Brazil", 0.10757946 },
  { "Brazos", 1.0 },
  { "Bre", 0.0 },
  { "Brea", 0.0 },
  { "Breaanna", 0.0 },
  { "Breaden", 1.0 },
  { "Breah", 0.0 },
  { "Breahanna", 0.0 },
  { "Breahna", 0.0 },
  { "Breahnna", 0.0 },
  { "Breaira", 0.0 },
  { "Breaja", 0.0 },
  { "Breajah", 0.0 },
  { "Breaker", 1.0 },
  { "Breale", 0.0 },
  { "Brealeigh", 0.0 },
  { "Brealle", 0.0 },
  { "Brealyn", 0.0 },
  { "Brealynn", 0.0 },
  { "Brean", 0.064935066 },
  { "Breana", 0.0 },
  { "Breanah", 0.0 },
  { "Breanca", 0.0 },
  { "Breanda", 0.0 },
  { "Breandan", 1.0 },
  { "Breandra", 0.0 },
  { "Breandrea", 0.0 },
  { "Breane", 0.0 },
  { "Breania", 0.0 },
  { "Breann", 0.0 },
  { "Breanna", 0.00080373924 },
  { "Breannah", 0.0 },
  { "Breanne", 0.0 },
  { "Breannia", 0.0 },
  { "Breannon", 0.0 },
  { "Breara", 0.0 },
  { "Breasha", 0.0 },
  { "Breashia", 0.0 },
  { "Breasia", 0.0 },
  { "Breasya", 0.0 },
  { "Breaun", 0.23809524 },
  { "Breauna", 0.0 },
  { "Breaunna", 0.0 },
  { "Breawna", 0.0 },
  { "Breayla", 0.0 },
  { "Breayna", 0.0 },
  { "Breazae", 0.0 },
  { "Breazia", 0.0 },
  { "Brec", 1.0 },
  { "Brecca", 0.0 },
  { "Breccan", 1.0 },
  { "Brecia", 0.0 },
  { "Breck", 0.8378713 },
  { "Breckan", 1.0 },
  { "Brecken", 0.805 },
  { "Brecker", 1.0 },
  { "Breckin", 0.8799662 },
  { "Brecklin", 0.0 },
  { "Brecklyn", 0.0 },
  { "Brecklynn", 0.0 },
  { "Breckon", 1.0 },
  { "Breckyn", 0.22605364 },
  { "Breckynn", 0.0 },
  { "Brecon", 1.0 },
  { "Breda", 0.0 },
  { "Bredan", 1.0 },
  { "Bredyn", 1.0 },
  { "Bree", 0.0020408162 },
  { "Breea", 0.0 },
  { "Breeah", 0.0 },
  { "Breeahna", 0.0 },
  { "Breean", 0.0 },
  { "Breeana", 0.0 },
  { "Breeann", 0.0 },
  { "Breeanna", 0.0 },
  { "Breeannah", 0.0 },
  { "Breeanne", 0.0 },
  { "Breeauna", 0.0 },
  { "Breeaunna", 0.0 },
  { "Breece", 0.21568628 },
  { "Breegan", 0.0 },
  { "Breelan", 0.0 },
  { "Breeland", 0.0 },
  { "Breelee", 0.0 },
  { "Breeleigh", 0.0 },
  { "Breeley", 0.0 },
  { "Breelin", 0.0 },
  { "Breella", 0.0 },
  { "Breelle", 0.0 },
  { "Breely", 0.0 },
  { "Breelyn", 0.0 },
  { "Breelynn", 0.0 },
  { "Breen", 0.0 },
  { "Breena", 0.0 },
  { "Breeona", 0.0 },
  { "Breeonna", 0.0 },
  { "Brees", 0.7755102 },
  { "Breesa", 0.0 },
  { "Breese", 0.0 },
  { "Breesha", 0.0 },
  { "Breeya", 0.0 },
  { "Breeyah", 0.0 },
  { "Breeyana", 0.0 },
  { "Breeyanna", 0.0 },
  { "Breez", 0.47619048 },
  { "Breeza", 0.0 },
  { "Breeze", 0.06518724 },
  { "Breezi", 0.0 },
  { "Breezie", 0.0 },
  { "Breezy", 0.0 },
  { "Breh", 1.0 },
  { "Brehan", 0.0 },
  { "Brehana", 0.0 },
  { "Brehanna", 0.0 },
  { "Brehea", 0.0 },
  { "Brehon", 1.0 },
  { "Brei", 0.0 },
  { "Breia", 0.0 },
  { "Breiah", 0.0 },
  { "Breiana", 0.0 },
  { "Breiann", 0.0 },
  { "Breianna", 0.0 },
  { "Breianne", 0.0 },
  { "Breiden", 1.0 },
  { "Breidy", 0.0 },
  { "Breidyn", 1.0 },
  { "Breigh", 0.0 },
  { "Breigha", 0.0 },
  { "Breighana", 0.0 },
  { "Breighanna", 0.0 },
  { "Breighden", 1.0 },
  { "Breighlyn", 0.0 },
  { "Breighlynn", 0.0 },
  { "Breighton", 1.0 },
  { "Breilyn", 0.0 },
  { "Breilynn", 0.0 },
  { "Brein", 0.0 },
  { "Breina", 0.0 },
  { "Breindel", 0.0 },
  { "Breindy", 0.0 },
  { "Breinne", 0.0 },
  { "Breion", 0.75274724 },
  { "Breiona", 0.0 },
  { "Breionna", 0.0 },
  { "Breisha", 0.0 },
  { "Breitlin", 1.0 },
  { "Brejae", 0.0 },
  { "Brejon", 1.0 },
  { "Brek", 1.0 },
  { "Breken", 1.0 },
  { "Brekia", 0.0 },
  { "Brekin", 1.0 },
  { "Brekka", 0.0 },
  { "Brekke", 0.0 },
  { "Brekken", 0.93491125 },
  { "Brekkin", 1.0 },
  { "Brekkyn", 0.0 },
  { "Breklyn", 0.0 },
  { "Breklynn", 0.0 },
  { "Brekon", 1.0 },
  { "Brekyn", 0.33070865 },
  { "Brelan", 1.0 },
  { "Breland", 0.8613333 },
  { "Brelee", 0.0 },
  { "Breleigh", 0.0 },
  { "Brelen", 1.0 },
  { "Brelin", 1.0 },
  { "Brelle", 0.0 },
  { "Brelon", 1.0 },
  { "Brely", 0.0 },
  { "Brelyn", 0.07364341 },
  { "Brelynn", 0.007215007 },
  { "Bremen", 1.0 },
  { "Bren", 0.7692308 },
  { "Brena", 0.0 },
  { "Brenae", 0.0 },
  { "Brenai", 0.0 },
  { "Brenan", 1.0 },
  { "Brenard", 1.0 },
  { "Brenasia", 0.0 },
  { "Brenay", 0.0 },
  { "Brenaya", 0.0 },
  { "Brence", 1.0 },
  { "Brend", 0.0 },
  { "Brenda", 0.00348637 },
  { "Brendaa", 0.0 },
  { "Brendalee", 0.0 },
  { "Brendalis", 0.0 },
  { "Brendaliz", 0.0 },
  { "Brendaly", 0.0 },
  { "Brendalyn", 0.0 },
  { "Brendalynn", 0.0 },
  { "Brendan", 0.99634904 },
  { "Brendasia", 0.0 },
  { "Brendell", 0.20833333 },
  { "Brendella", 0.0 },
  { "Brenden", 0.9983046 },
  { "Brender", 0.0 },
  { "Brendetta", 0.0 },
  { "Brendi", 0.0 },
  { "Brendia", 0.0 },
  { "Brendin", 1.0 },
  { "Brendolyn", 0.0 },
  { "Brendon", 0.99886394 },
  { "Brendt", 1.0 },
  { "Brendy", 0.0 },
  { "Brendyn", 1.0 },
  { "Brene", 0.0 },
  { "Brenea", 0.0 },
  { "Brenee", 0.0 },
  { "Breneisha", 0.0 },
  { "Brenen", 1.0 },
  { "Brener", 1.0 },
  { "Brenesha", 0.0 },
  { "Breneshia", 0.0 },
  { "Brenetta", 0.0 },
  { "Brenette", 0.0 },
  { "Brenham", 1.0 },
  { "Brenia", 0.0 },
  { "Breniah", 0.0 },
  { "Brenice", 0.0 },
  { "Brenicia", 0.0 },
  { "Brenika", 0.0 },
  { "Brenin", 0.9691358 },
  { "Brenisha", 0.0 },
  { "Brenita", 0.0 },
  { "Breniya", 0.0 },
  { "Breniyah", 0.0 },
  { "Brenlee", 0.0 },
  { "Brenleigh", 0.0 },
  { "Brenley", 0.034267914 },
  { "Brenli", 0.0 },
  { "Brenlie", 0.0 },
  { "Brenly", 0.0 },
  { "Brenlyn", 0.0 },
  { "Brenlynn", 0.0 },
  { "Brenn", 0.496063 },
  { "Brenna", 0.0 },
  { "Brennae", 0.0 },
  { "Brennah", 0.0 },
  { "Brennan", 0.93885756 },
  { "Brennda", 0.0 },
  { "Brenndan", 1.0 },
  { "Brennden", 1.0 },
  { "Brenndon", 1.0 },
  { "Brennen", 0.9796548 },
  { "Brenner", 0.9902629 },
  { "Brennick", 1.0 },
  { "Brennin", 1.0 },
  { "Brennley", 0.0 },
  { "Brennon", 1.0 },
  { "Brennus", 1.0 },
  { "Brennyn", 0.46124032 },
  { "Breno", 1.0 },
  { "Brenon", 1.0 },
  { "Brensen", 1.0 },
  { "Brenson", 1.0 },
  { "Brenston", 1.0 },
  { "Brent", 0.9959225 },
  { "Brentan", 1.0 },
  { "Brenten", 1.0 },
  { "Brentin", 1.0 },
  { "Brentlee", 0.84415585 },
  { "Brentleigh", 0.0 },
  { "Brentley", 0.9351536 },
  { "Brently", 0.984141 },
  { "Brentney", 0.0 },
  { "Brentnie", 0.0 },
  { "Brenton", 0.9987265 },
  { "Brentson", 1.0 },
  { "Brentt", 1.0 },
  { "Brentten", 1.0 },
  { "Brentton", 1.0 },
  { "Brentwood", 1.0 },
  { "Brentyn", 1.0 },
  { "Brenya", 0.0 },
  { "Brenyn", 0.75333333 },
  { "Breon", 0.88114756 },
  { "Breona", 0.0 },
  { "Breonah", 0.0 },
  { "Breonca", 0.0 },
  { "Breonda", 0.0 },
  { "Breondra", 0.0 },
  { "Breonia", 0.0 },
  { "Breonica", 0.0 },
  { "Breonka", 0.0 },
  { "Breonna", 0.0 },
  { "Breonnah", 0.0 },
  { "Breonne", 0.0 },
  { "Breontae", 1.0 },
  { "Breonte", 0.54545456 },
  { "Breosha", 0.0 },
  { "Brescia", 0.0 },
  { "Breseis", 0.0 },
  { "Bresha", 0.0 },
  { "Breshae", 0.0 },
  { "Breshai", 0.0 },
  { "Breshaun", 1.0 },
  { "Breshauna", 0.0 },
  { "Breshawn", 0.556962 },
  { "Breshawna", 0.0 },
  { "Breshay", 0.0 },
  { "Breshayla", 0.0 },
  { "Breshey", 0.0 },
  { "Breshia", 0.0 },
  { "Breshonna", 0.0 },
  { "Breshuna", 0.0 },
  { "Breslin", 0.57425743 },
  { "Breslyn", 0.0 },
  { "Breslynn", 0.0 },
  { "Bret", 0.9922268 },
  { "Breta", 0.0 },
  { "Brete", 1.0 },
  { "Bretha", 0.0 },
  { "Bretley", 1.0 },
  { "Bretney", 0.0 },
  { "Breton", 1.0 },
  { "Brett", 0.97959936 },
  { "Bretta", 0.0 },
  { "Brettany", 0.0 },
  { "Brette", 0.018835617 },
  { "Bretten", 1.0 },
  { "Brettlee", 1.0 },
  { "Brettley", 1.0 },
  { "Brettly", 1.0 },
  { "Brettney", 0.0 },
  { "Brettnie", 0.0 },
  { "Bretton", 1.0 },
  { "Breuna", 0.0 },
  { "Breunna", 0.0 },
  { "Brevan", 1.0 },
  { "Brevard", 1.0 },
  { "Breven", 1.0 },
  { "Brevin", 1.0 },
  { "Brevon", 1.0 },
  { "Brevyn", 0.94634145 },
  { "Brewer", 1.0 },
  { "Brewster", 1.0 },
  { "Brex", 1.0 },
  { "Brexlee", 0.0 },
  { "Brexley", 0.0 },
  { "Brexlie", 0.0 },
  { "Brexlyn", 0.0 },
  { "Brexten", 1.0 },
  { "Brextin", 1.0 },
  { "Brexton", 1.0 },
  { "Brextyn", 1.0 },
  { "Brey", 1.0 },
  { "Breya", 0.0 },
  { "Breyah", 0.0 },
  { "Breyan", 0.6666667 },
  { "Breyana", 0.0 },
  { "Breyanah", 0.0 },
  { "Breyanna", 0.0 },
  { "Breyannah", 0.0 },
  { "Breyanne", 0.0 },
  { "Breyauna", 0.0 },
  { "Breyawna", 0.0 },
  { "Breydan", 1.0 },
  { "Breyden", 1.0 },
  { "Breydi", 0.0 },
  { "Breydin", 1.0 },
  { "Breydon", 1.0 },
  { "Breyell", 0.0 },
  { "Breyelle", 0.0 },
  { "Breyen", 1.0 },
  { "Breyer", 0.8907563 },
  { "Breylan", 1.0 },
  { "Breylee", 0.0 },
  { "Breylen", 1.0 },
  { "Breylin", 0.8012048 },
  { "Breylon", 1.0 },
  { "Breylyn", 0.0 },
  { "Breylynn", 0.0 },
  { "Breyn", 0.0 },
  { "Breyon", 0.8663854 },
  { "Breyona", 0.0 },
  { "Breyonce", 0.0 },
  { "Breyonia", 0.0 },
  { "Breyonna", 0.0 },
  { "Breyson", 1.0 },
  { "Breyton", 1.0 },
  { "Breyunna", 0.0 },
  { "Brezae", 0.0 },
  { "Breze", 0.0 },
  { "Brezhane", 0.0 },
  { "Brezlin", 0.54545456 },
  { "Brezlyn", 0.0 },
  { "Brezlynn", 0.0 },
  { "Brezzy", 0.0 },
  { "Brhea", 0.0 },
  { "Brhett", 1.0 },
  { "Brhianna", 0.0 },
  { "Bri", 0.0 },
  { "Bria", 0.0039596832 },
  { "Briah", 0.0 },
  { "Briahna", 0.0 },
  { "Briahnna", 0.0 },
  { "Briaja", 0.0 },
  { "Briajah", 0.0 },
  { "Brialle", 0.0 },
  { "Briam", 1.0 },
  { "Briamarie", 0.0 },
  { "Brian", 0.9960866 },
  { "Briana", 0.0021772804 },
  { "Brianah", 0.0 },
  { "Brianalee", 0.0 },
  { "Brianamarie", 0.0 },
  { "Briananthony", 1.0 },
  { "Brianca", 0.0 },
  { "Briance", 0.0 },
  { "Brianda", 0.0027472528 },
  { "Briandra", 0.0 },
  { "Briane", 0.15669516 },
  { "Briani", 0.0 },
  { "Briania", 0.0 },
  { "Brianica", 0.0 },
  { "Brianie", 0.0 },
  { "Brianka", 0.0 },
  { "Briann", 0.012861736 },
  { "Brianna", 0.0016840579 },
  { "Briannah", 0.0 },
  { "Briannalee", 0.0 },
  { "Briannan", 0.0 },
  { "Brianne", 0.0015899831 },
  { "Brianney", 0.0 },
  { "Brianni", 0.0 },
  { "Briannia", 0.0 },
  { "Briannica", 0.0 },
  { "Briannie", 0.0 },
  { "Briannon", 0.0 },
  { "Brianny", 0.0 },
  { "Briant", 1.0 },
  { "Briante", 0.05319149 },
  { "Briany", 0.0 },
  { "Briar", 0.6184408 },
  { "Briara", 0.0 },
  { "Briarrose", 0.0 },
  { "Briasha", 0.0 },
  { "Briasia", 0.0 },
  { "Briauna", 0.0 },
  { "Briaunna", 0.0 },
  { "Briawna", 0.0 },
  { "Briayan", 1.0 },
  { "Briazia", 0.0 },
  { "Bric", 1.0 },
  { "Brice", 0.9768173 },
  { "Briceida", 0.0 },
  { "Briceidy", 0.0 },
  { "Bricelyn", 0.0 },
  { "Bricelynn", 0.0 },
  { "Bricen", 1.0 },
  { "Briceson", 1.0 },
  { "Briceston", 1.0 },
  { "Briceton", 1.0 },
  { "Briceyda", 0.0 },
  { "Brichelle", 0.0 },
  { "Bricia", 0.0 },
  { "Brick", 1.0 },
  { "Bricker", 1.0 },
  { "Bricola", 0.0 },
  { "Bricyn", 1.0 },
  { "Bridan", 1.0 },
  { "Briden", 1.0 },
  { "Bridey", 0.0 },
  { "Bridge", 0.9056604 },
  { "Bridger", 0.998805 },
  { "Bridges", 1.0 },
  { "Bridget", 0.0016852039 },
  { "Bridgete", 0.0 },
  { "Bridgett", 0.0 },
  { "Bridgetta", 0.0 },
  { "Bridgette", 0.00069198164 },
  { "Bridgid", 0.0 },
  { "Bridgit", 0.0 },
  { "Bridgitt", 0.0 },
  { "Bridgitte", 0.0 },
  { "Bridie", 0.0 },
  { "Bridjette", 0.0 },
  { "Bridney", 0.0 },
  { "Bridon", 1.0 },
  { "Bridy", 0.0 },
  { "Bridyn", 1.0 },
  { "Brie", 0.0 },
  { "Briea", 0.0 },
  { "Brieah", 0.0 },
  { "Brieana", 0.0 },
  { "Brieann", 0.0 },
  { "Brieanna", 0.0 },
  { "Brieanne", 0.0 },
  { "Briel", 0.0 },
  { "Briela", 0.0 },
  { "Briele", 0.0 },
  { "Brielee", 0.0 },
  { "Briell", 0.0 },
  { "Briella", 0.0 },
  { "Briellah", 0.0 },
  { "Brielle", 0.0 },
  { "Brielyn", 0.0 },
  { "Brielynn", 0.0 },
  { "Brien", 0.99858916 },
  { "Briena", 0.0 },
  { "Brienna", 0.0 },
  { "Brienne", 0.0037936268 },
  { "Brieon", 0.0 },
  { "Brieona", 0.0 },
  { "Brieonna", 0.0 },
  { "Brier", 0.64788735 },
  { "Brierra", 0.0 },
  { "Brierre", 0.0 },
  { "Briesha", 0.0 },
  { "Brietta", 0.0 },
  { "Briette", 0.0 },
  { "Brieya", 0.0 },
  { "Brig", 1.0 },
  { "Brigden", 1.0 },
  { "Brigdon", 1.0 },
  { "Briget", 0.0 },
  { "Brigett", 0.0 },
  { "Brigetta", 0.0 },
  { "Brigette", 0.0 },
  { "Brigg", 1.0 },
  { "Briggett", 0.0 },
  { "Briggette", 0.0 },
  { "Briggitte", 0.0 },
  { "Briggs", 0.9940187 },
  { "Briggston", 1.0 },
  { "Brigham", 1.0 },
  { "Brighid", 0.0 },
  { "Brighley", 0.0 },
  { "Bright", 0.97596157 },
  { "Brighten", 0.6163522 },
  { "Brightly", 0.0 },
  { "Brighton", 0.6442073 },
  { "Brightyn", 0.0 },
  { "Brigid", 0.0 },
  { "Brigida", 0.0 },
  { "Brigido", 1.0 },
  { "Brigina", 0.0 },
  { "Brigit", 0.0 },
  { "Brigita", 0.0 },
  { "Brigitt", 0.0 },
  { "Brigitta", 0.0 },
  { "Brigitte", 0.0 },
  { "Brigston", 1.0 },
  { "Brihana", 0.0 },
  { "Brihanna", 0.0 },
  { "Brihany", 0.0 },
  { "Brij", 1.0 },
  { "Brijae", 0.0 },
  { "Brijesh", 1.0 },
  { "Brijet", 0.0 },
  { "Brijette", 0.0 },
  { "Brijida", 0.0 },
  { "Brijido", 1.0 },
  { "Brijin", 0.0 },
  { "Brik", 1.0 },
  { "Brilan", 1.0 },
  { "Brilea", 0.0 },
  { "Brilee", 0.021535581 },
  { "Brilei", 0.0 },
  { "Brileigh", 0.0 },
  { "Brilen", 1.0 },
  { "Briley", 0.1870328 },
  { "Brilie", 0.0 },
  { "Brilliance", 0.0 },
  { "Brilliant", 1.0 },
  { "Brilyn", 0.13377926 },
  { "Brilynn", 0.00755287 },
  { "Brilynne", 0.0 },
  { "Brin", 0.09002433 },
  { "Brina", 0.003912363 },
  { "Brinae", 0.0 },
  { "Brinda", 0.0 },
  { "Brinden", 1.0 },
  { "Brinder", 0.0 },
  { "Brindi", 0.0 },
  { "Brindin", 1.0 },
  { "Brindle", 0.0 },
  { "Brindley", 0.0 },
  { "Brindon", 1.0 },
  { "Brindy", 0.0 },
  { "Brinesha", 0.0 },
  { "Briney", 0.0 },
  { "Brinisha", 0.0 },
  { "Briniya", 0.0 },
  { "Briniyah", 0.0 },
  { "Brink", 1.0 },
  { "Brinklee", 0.0 },
  { "Brinkley", 0.036259543 },
  { "Brinlea", 0.0 },
  { "Brinlee", 0.0 },
  { "Brinlei", 0.0 },
  { "Brinleigh", 0.0 },
  { "Brinley", 0.015093787 },
  { "Brinli", 0.0 },
  { "Brinlie", 0.0 },
  { "Brinly", 0.0 },
  { "Brinlyn", 0.0 },
  { "Brinlynn", 0.0 },
  { "Brinn", 0.029069768 },
  { "Brinna", 0.0 },
  { "Brinnlee", 0.0 },
  { "Brinnley", 0.0 },
  { "Brinnly", 0.0 },
  { "Brinslee", 0.0 },
  { "Brinsley", 0.0 },
  { "Brinson", 0.9807074 },
  { "Brint", 1.0 },
  { "Brintney", 0.0 },
  { "Brinton", 1.0 },
  { "Brinya", 0.0 },
  { "Brion", 0.9696422 },
  { "Briona", 0.0 },
  { "Brionca", 0.0 },
  { "Brione", 0.16923077 },
  { "Brioni", 0.0 },
  { "Brionica", 0.0 },
  { "Brionna", 0.0 },
  { "Brionnah", 0.0 },
  { "Brionne", 0.03311258 },
  { "Brionni", 0.0 },
  { "Briony", 0.0 },
  { "Brisa", 0.0 },
  { "Brisaida", 0.0 },
  { "Brisais", 0.0 },
  { "Brisamar", 0.0 },
  { "Brisayda", 0.0 },
  { "Brisco", 1.0 },
  { "Briscoe", 1.0 },
  { "Briseida", 0.0 },
  { "Briseidy", 0.0 },
  { "Briseis", 0.0 },
  { "Briselda", 0.0 },
  { "Brisen", 1.0 },
  { "Briseyda", 0.0 },
  { "Brisha", 0.0 },
  { "Brishae", 0.0 },
  { "Brishana", 0.0 },
  { "Brishaun", 1.0 },
  { "Brishauna", 0.0 },
  { "Brishawn", 1.0 },
  { "Brishen", 1.0 },
  { "Brishon", 0.0 },
  { "Brisia", 0.0 },
  { "Brisna", 0.0 },
  { "Brisny", 0.0 },
  { "Brison", 1.0 },
  { "Brissa", 0.0 },
  { "Brisseida", 0.0 },
  { "Brisseyda", 0.0 },
  { "Brissia", 0.0 },
  { "Bristal", 0.0 },
  { "Bristen", 0.4054054 },
  { "Brister", 1.0 },
  { "Bristin", 0.33333334 },
  { "Bristol", 0.07434092 },
  { "Briston", 1.0 },
  { "Bristy", 0.0 },
  { "Bristyl", 0.0 },
  { "Bristyn", 0.0 },
  { "Brisyn", 1.0 },
  { "Brit", 0.81014305 },
  { "Brita", 0.0 },
  { "Britain", 0.54330707 },
  { "Britainy", 0.0 },
  { "Britan", 0.7027027 },
  { "Britane", 0.0 },
  { "Britanee", 0.0 },
  { "Britaney", 0.0 },
  { "Britani", 0.0 },
  { "Britania", 0.0 },
  { "Britanie", 0.0 },
  { "Britanna", 0.0 },
  { "Britanni", 0.0 },
  { "Britannia", 0.0 },
  { "Britannica", 0.0 },
  { "Britanny", 0.0 },
  { "Britany", 0.0 },
  { "Britanya", 0.0 },
  { "Briteny", 0.0 },
  { "Brithanny", 0.0 },
  { "Brithany", 0.0 },
  { "Brithney", 0.0 },
  { "Brithny", 0.0 },
  { "Britian", 0.88505745 },
  { "Britiany", 0.0 },
  { "Britinee", 0.0 },
  { "Britiney", 0.0 },
  { "Britini", 0.0 },
  { "Britiny", 0.0 },
  { "British", 0.089361705 },
  { "Britlany", 0.0 },
  { "Britlee", 0.0 },
  { "Britleigh", 0.0 },
  { "Britley", 0.0 },
  { "Britlyn", 0.0 },
  { "Britlynn", 0.0 },
  { "Britnae", 0.0 },
  { "Britnai", 0.0 },
  { "Britnay", 0.0 },
  { "Britne", 0.0 },
  { "Britnee", 0.0 },
  { "Britnei", 0.0 },
  { "Britney", 0.0024106184 },
  { "Britni", 0.0 },
  { "Britnie", 0.0 },
  { "Britny", 0.0 },
  { "Britnye", 0.0 },
  { "Briton", 0.9904943 },
  { "Britt", 0.61472034 },
  { "Britta", 0.0 },
  { "Brittain", 0.5585997 },
  { "Brittainey", 0.0 },
  { "Brittaini", 0.0 },
  { "Brittainy", 0.0 },
  { "Brittan", 0.35359117 },
  { "Brittana", 0.0 },
  { "Brittanae", 0.0 },
  { "Brittane", 0.0 },
  { "Brittanee", 0.0 },
  { "Brittaney", 0.0 },
  { "Brittani", 0.001874095 },
  { "Brittania", 0.0 },
  { "Brittanica", 0.0 },
  { "Brittanie", 0.0 },
  { "Brittanii", 0.0 },
  { "Brittanni", 0.0 },
  { "Brittannie", 0.0 },
  { "Brittanny", 0.0 },
  { "Brittant", 0.0 },
  { "Brittany", 0.0030041663 },
  { "Brittanya", 0.0 },
  { "Brittanyann", 0.0 },
  { "Brittanye", 0.0 },
  { "Brittanylee", 0.0 },
  { "Brittanymarie", 0.0 },
  { "Brittanynicole", 0.0 },
  { "Brittary", 0.0 },
  { "Brittay", 0.0 },
  { "Britte", 0.0 },
  { "Britten", 0.59348613 },
  { "Britteney", 0.0 },
  { "Britteni", 0.0 },
  { "Britteny", 0.0 },
  { "Britthany", 0.0 },
  { "Brittian", 0.76363635 },
  { "Brittiana", 0.0 },
  { "Brittiani", 0.0 },
  { "Brittiany", 0.0 },
  { "Brittie", 0.0 },
  { "Brittin", 0.33731344 },
  { "Brittina", 0.0 },
  { "Brittinay", 0.0 },
  { "Brittine", 0.0 },
  { "Brittinee", 0.0 },
  { "Brittiney", 0.0 },
  { "Brittini", 0.0 },
  { "Brittinie", 0.0 },
  { "Brittiny", 0.0 },
  { "Brittish", 0.0 },
  { "Brittlee", 0.0 },
  { "Brittley", 0.0 },
  { "Brittlyn", 0.0 },
  { "Brittlynn", 0.0 },
  { "Brittna", 0.0 },
  { "Brittnae", 0.0 },
  { "Brittnany", 0.0 },
  { "Brittnay", 0.0 },
  { "Brittne", 0.0 },
  { "Brittnea", 0.0 },
  { "Brittnee", 0.0 },
  { "Brittnei", 0.0 },
  { "Brittney", 0.0032309194 },
  { "Brittneyann", 0.0 },
  { "Brittni", 0.0010015023 },
  { "Brittnie", 0.0 },
  { "Brittnii", 0.0 },
  { "Brittny", 0.0 },
  { "Brittnye", 0.0 },
  { "Britton", 0.77795964 },
  { "Brittoni", 0.0 },
  { "Brittony", 0.0 },
  { "Britttany", 0.0 },
  { "Brittyn", 0.0 },
  { "Brityn", 0.0 },
  { "Britynn", 0.0 },
  { "Britza", 0.0 },
  { "Britzel", 0.0 },
  { "Britzy", 0.0 },
  { "Briuna", 0.0 },
  { "Briunna", 0.0 },
  { "Brix", 1.0 },
  { "Brixen", 1.0 },
  { "Brixley", 0.0 },
  { "Brixon", 1.0 },
  { "Brixten", 0.6666667 },
  { "Brixtin", 1.0 },
  { "Brixton", 0.90252393 },
  { "Brixtyn", 0.0 },
  { "Brixx", 1.0 },
  { "Briya", 0.0 },
  { "Briyah", 0.0 },
  { "Briyan", 1.0 },
  { "Briyana", 0.0 },
  { "Briyanah", 0.0 },
  { "Briyanna", 0.0 },
  { "Briyelle", 0.0 },
  { "Briyit", 0.0 },
  { "Briyith", 0.0 },
  { "Briyon", 1.0 },
  { "Briyona", 0.0 },
  { "Briyonna", 0.0 },
  { "Briza", 0.0 },
  { "Brizeida", 0.0 },
  { "Brizeth", 0.0 },
  { "Brizeyda", 0.0 },
  { "Brizia", 0.0 },
  { "Brizleth", 0.0 },
  { "Brizza", 0.0 },
  { "Brizzia", 0.0 },
  { "Brnadon", 1.0 },
  { "Brndon", 1.0 },
  { "Broadus", 1.0 },
  { "Broady", 1.0 },
  { "Broc", 1.0 },
  { "Broch", 1.0 },
  { "Brocha", 0.0 },
  { "Brock", 0.9981268 },
  { "Brockman", 1.0 },
  { "Brockston", 1.0 },
  { "Brockton", 1.0 },
  { "Brocton", 1.0 },
  { "Brodan", 1.0 },
  { "Brodderick", 1.0 },
  { "Broddrick", 1.0 },
  { "Brode", 1.0 },
  { "Brodee", 0.9744898 },
  { "Broden", 1.0 },
  { "Broder", 1.0 },
  { "Broderic", 1.0 },
  { "Broderick", 1.0 },
  { "Broderik", 1.0 },
  { "Brodey", 1.0 },
  { "Brodhi", 1.0 },
  { "Brodi", 0.8240866 },
  { "Brodie", 0.96099216 },
  { "Brodin", 1.0 },
  { "Brodix", 1.0 },
  { "Brodney", 1.0 },
  { "Brodric", 1.0 },
  { "Brodrick", 1.0 },
  { "Brodus", 1.0 },
  { "Brody", 0.99641716 },
  { "Brodyn", 1.0 },
  { "Broedy", 1.0 },
  { "Brogan", 0.8252607 },
  { "Brogen", 1.0 },
  { "Broghan", 0.825 },
  { "Brogyn", 1.0 },
  { "Brohdy", 1.0 },
  { "Brok", 1.0 },
  { "Brolin", 1.0 },
  { "Broly", 1.0 },
  { "Brom", 1.0 },
  { "Bromley", 1.0 },
  { "Bron", 1.0 },
  { "Brona", 0.0 },
  { "Bronagh", 0.0 },
  { "Bronc", 1.0 },
  { "Bronce", 1.0 },
  { "Bronco", 1.0 },
  { "Bronda", 0.0 },
  { "Brondon", 1.0 },
  { "Bronica", 0.0 },
  { "Bronis", 1.0 },
  { "Bronislaus", 1.0 },
  { "Bronislava", 0.0 },
  { "Bronislaw", 1.0 },
  { "Bronislawa", 0.0 },
  { "Bronko", 1.0 },
  { "Bronn", 1.0 },
  { "Bronna", 0.0 },
  { "Bronner", 1.0 },
  { "Bronnie", 0.10958904 },
  { "Bronsen", 1.0 },
  { "Bronson", 1.0 },
  { "Bronston", 1.0 },
  { "Bronsyn", 1.0 },
  { "Bronte", 0.021301774 },
  { "Bronti", 0.0 },
  { "Bronwen", 0.0 },
  { "Bronwyn", 0.0 },
  { "Bronwynn", 0.0 },
  { "Bronx", 0.98697066 },
  { "Bronxton", 1.0 },
  { "Bronze", 1.0 },
  { "Brook", 0.24068874 },
  { "Brookann", 0.0 },
  { "Brooke", 0.0106022 },
  { "Brookeann", 0.0 },
  { "Brookelin", 0.0 },
  { "Brookelle", 0.0 },
  { "Brookelyn", 0.0 },
  { "Brookelyne", 0.0 },
  { "Brookelynn", 0.0 },
  { "Brookelynne", 0.0 },
  { "Brooker", 1.0 },
  { "Brookes", 0.65625 },
  { "Brookie", 0.012915129 },
  { "Brooklan", 0.0 },
  { "Brooklee", 0.0 },
  { "Brookleigh", 0.0 },
  { "Brooklen", 0.0 },
  { "Brooklenn", 0.0 },
  { "Brookley", 0.0 },
  { "Brooklin", 0.0026910657 },
  { "Brooklinn", 0.0 },
  { "Brooklon", 0.0 },
  { "Brookly", 0.0 },
  { "Brooklyn", 0.0138148405 },
  { "Brooklyne", 0.0 },
  { "Brooklynmarie", 0.0 },
  { "Brooklynn", 0.0006951927 },
  { "Brooklynne", 0.0 },
  { "Brooklynrose", 0.0 },
  { "Brooks", 0.95264405 },
  { "Brooksey", 0.0 },
  { "Brooksie", 0.0 },
  { "Brooksley", 0.0 },
  { "Brookson", 1.0 },
  { "Brookston", 1.0 },
  { "Brookyln", 0.0 },
  { "Broox", 1.0 },
  { "Brooxie", 0.0 },
  { "Bror", 1.0 },
  { "Brosnan", 1.0 },
  { "Brother", 1.0 },
  { "Broughton", 1.0 },
  { "Broward", 1.0 },
  { "Browder", 1.0 },
  { "Browdy", 1.0 },
  { "Brown", 0.9965541 },
  { "Brownell", 1.0 },
  { "Brownie", 0.40942323 },
  { "Browning", 1.0 },
  { "Brownlow", 1.0 },
  { "Brox", 1.0 },
  { "Broxton", 1.0 },
  { "Brttany", 0.0 },
  { "Bruce", 0.9963468 },
  { "Brucelee", 1.0 },
  { "Brucha", 0.0 },
  { "Bruchy", 0.0 },
  { "Brucie", 0.0 },
  { "Bruin", 1.0 },
  { "Bruk", 1.0 },
  { "Bruke", 1.0 },
  { "Bruna", 0.0 },
  { "Brunell", 0.15625 },
  { "Brunella", 0.0 },
  { "Brunelle", 0.0 },
  { "Bruner", 1.0 },
  { "Brunetta", 0.0 },
  { "Brunette", 0.0 },
  { "Brunhilda", 0.0 },
  { "Brunilda", 0.0 },
  { "Brunna", 0.0 },
  { "Bruno", 1.0 },
  { "Brunson", 1.0 },
  { "Bruster", 1.0 },
  { "Brutus", 1.0 },
  { "Bry", 1.0 },
  { "Brya", 0.019607844 },
  { "Bryah", 0.0 },
  { "Bryahna", 0.0 },
  { "Bryam", 1.0 },
  { "Bryan", 0.995488 },
  { "Bryana", 0.0 },
  { "Bryanah", 0.0 },
  { "Bryand", 1.0 },
  { "Bryanda", 0.0 },
  { "Bryani", 0.0 },
  { "Bryann", 0.23126338 },
  { "Bryanna", 0.0 },
  { "Bryannah", 0.0 },
  { "Bryanne", 0.0 },
  { "Bryant", 0.99529815 },
  { "Bryar", 0.7908003 },
  { "Bryasha", 0.0 },
  { "Bryasia", 0.0 },
  { "Bryauna", 0.0 },
  { "Bryaunna", 0.0 },
  { "Bryce", 0.9739922 },
  { "Brycelyn", 0.0 },
  { "Brycelynn", 0.0 },
  { "Brycen", 0.9974298 },
  { "Brycenn", 1.0 },
  { "Bryceon", 1.0 },
  { "Bryceson", 1.0 },
  { "Bryceton", 1.0 },
  { "Brycie", 0.0 },
  { "Brycin", 1.0 },
  { "Brycn", 1.0 },
  { "Brycon", 1.0 },
  { "Brycson", 1.0 },
  { "Brycyn", 1.0 },
  { "Brydan", 1.0 },
  { "Bryden", 0.98511326 },
  { "Brydie", 0.0 },
  { "Brydin", 1.0 },
  { "Brydon", 1.0 },
  { "Brye", 0.7368421 },
  { "Bryell", 0.0 },
  { "Bryella", 0.0 },
  { "Bryelle", 0.0 },
  { "Bryen", 1.0 },
  { "Bryenna", 0.0 },
  { "Bryent", 1.0 },
  { "Bryer", 0.91508055 },
  { "Bryhana", 0.0 },
  { "Bryhanna", 0.0 },
  { "Bryheem", 1.0 },
  { "Bryia", 0.0 },
  { "Bryian", 1.0 },
  { "Bryiana", 0.0 },
  { "Bryianna", 0.0 },
  { "Bryice", 1.0 },
  { "Bryin", 1.0 },
  { "Bryken", 1.0 },
  { "Bryker", 1.0 },
  { "Bryla", 0.0 },
  { "Brylan", 0.9235352 },
  { "Bryland", 1.0 },
  { "Brylea", 0.0 },
  { "Bryleah", 0.0 },
  { "Brylee", 0.049656138 },
  { "Brylei", 0.0 },
  { "Bryleigh", 0.0 },
  { "Brylen", 0.89224136 },
  { "Bryler", 1.0 },
  { "Bryley", 0.19745223 },
  { "Bryli", 0.0 },
  { "Brylie", 0.0028901733 },
  { "Bryliee", 0.0 },
  { "Brylin", 0.53846157 },
  { "Brylinn", 0.0 },
  { "Brylon", 1.0 },
  { "Brylyn", 0.2037037 },
  { "Brylynn", 0.0 },
  { "Bryn", 0.15695772 },
  { "Bryna", 0.0 },
  { "Brynae", 0.0 },
  { "Brynda", 0.0 },
  { "Bryndal", 0.0 },
  { "Bryndan", 1.0 },
  { "Bryndee", 0.0 },
  { "Brynden", 1.0 },
  { "Bryndis", 0.0 },
  { "Bryndle", 0.0 },
  { "Bryndon", 1.0 },
  { "Bryne", 0.9478261 },
  { "Brynesha", 0.0 },
  { "Brynia", 0.0 },
  { "Bryniah", 0.0 },
  { "Brynisha", 0.0 },
  { "Brynja", 0.0 },
  { "Brynlea", 0.0 },
  { "Brynlee", 0.0010614585 },
  { "Brynlei", 0.0 },
  { "Brynleigh", 0.0 },
  { "Brynley", 0.0 },
  { "Brynli", 0.0 },
  { "Brynlie", 0.0 },
  { "Brynlin", 0.0 },
  { "Brynly", 0.0 },
  { "Brynlyn", 0.0 },
  { "Brynlynn", 0.0 },
  { "Brynn", 0.015061201 },
  { "Brynna", 0.0 },
  { "Brynnan", 0.0 },
  { "Brynne", 0.0 },
  { "Brynnen", 0.3181818 },
  { "Brynner", 1.0 },
  { "Brynnlea", 0.0 },
  { "Brynnlee", 0.0 },
  { "Brynnleigh", 0.0 },
  { "Brynnley", 0.0 },
  { "Brynnli", 0.0 },
  { "Brynnlie", 0.0 },
  { "Brynnly", 0.0 },
  { "Brynsley", 0.0 },
  { "Brynson", 1.0 },
  { "Brynt", 1.0 },
  { "Brynton", 1.0 },
  { "Brynzlee", 0.0 },
  { "Bryon", 0.9988305 },
  { "Bryona", 0.0 },
  { "Bryonna", 0.0 },
  { "Bryony", 0.0 },
  { "Bryor", 1.0 },
  { "Brysa", 0.0 },
  { "Brysan", 1.0 },
  { "Bryse", 0.96153843 },
  { "Bryseida", 0.0 },
  { "Brysen", 0.99746835 },
  { "Bryshaun", 1.0 },
  { "Bryshawn", 1.0 },
  { "Bryshere", 1.0 },
  { "Bryshon", 1.0 },
  { "Brysin", 1.0 },
  { "Bryson", 0.99577343 },
  { "Brysonn", 1.0 },
  { "Bryssa", 0.0 },
  { "Bryssia", 0.0 },
  { "Brystal", 0.0 },
  { "Brysten", 1.0 },
  { "Brystin", 1.0 },
  { "Brystle", 0.0 },
  { "Brystol", 0.018348623 },
  { "Bryston", 1.0 },
  { "Brysun", 1.0 },
  { "Brysyn", 1.0 },
  { "Brytan", 1.0 },
  { "Brytani", 0.0 },
  { "Brytanie", 0.0 },
  { "Brytanni", 0.0 },
  { "Brytany", 0.0 },
  { "Bryten", 0.9556962 },
  { "Brytin", 1.0 },
  { "Brytne", 0.0 },
  { "Brytnee", 0.0 },
  { "Brytney", 0.0 },
  { "Brytni", 0.0 },
  { "Brytnie", 0.0 },
  { "Bryton", 0.9396521 },
  { "Bryttani", 0.0 },
  { "Bryttanie", 0.0 },
  { "Bryttany", 0.0 },
  { "Bryttnee", 0.0 },
  { "Bryttney", 0.0 },
  { "Bryttni", 0.0 },
  { "Bryttnie", 0.0 },
  { "Bryttnii", 0.0 },
  { "Bryxton", 1.0 },
  { "Btittany", 0.0 },
  { "Bub", 1.0 },
  { "Bubba", 1.0 },
  { "Bubber", 1.0 },
  { "Bubbles", 0.0 },
  { "Bubby", 1.0 },
  { "Buchanan", 1.0 },
  { "Buck", 1.0 },
  { "Buckey", 1.0 },
  { "Buckie", 1.0 },
  { "Buckley", 1.0 },
  { "Buckner", 1.0 },
  { "Bucky", 1.0 },
  { "Bud", 1.0 },
  { "Budd", 1.0 },
  { "Buddie", 0.9966689 },
  { "Buddy", 0.99798524 },
  { "Bueford", 1.0 },
  { "Buel", 1.0 },
  { "Buela", 0.0 },
  { "Buelah", 0.0 },
  { "Buell", 1.0 },
  { "Buena", 0.0 },
  { "Buenaventura", 1.0 },
  { "Bufard", 1.0 },
  { "Buff", 0.3974359 },
  { "Buffey", 0.0 },
  { "Buffi", 0.0 },
  { "Buffie", 0.0 },
  { "Bufford", 1.0 },
  { "Buffy", 0.0018354236 },
  { "Buford", 0.98995525 },
  { "Bugra", 1.0 },
  { "Bukhari", 1.0 },
  { "Bula", 0.0 },
  { "Bulah", 0.0 },
  { "Bular", 0.0 },
  { "Buleah", 0.0 },
  { "Bulmaro", 1.0 },
  { "Bulut", 1.0 },
  { "Bun", 1.0 },
  { "Buna", 0.0 },
  { "Bundy", 1.0 },
  { "Bunia", 0.0 },
  { "Bunice", 0.0 },
  { "Bunie", 0.0 },
  { "Bunion", 1.0 },
  { "Bunita", 0.0 },
  { "Bunk", 1.0 },
  { "Bunni", 0.0 },
  { "Bunnie", 0.0724299 },
  { "Bunny", 0.07586558 },
  { "Bunyan", 1.0 },
  { "Bunyon", 1.0 },
  { "Burach", 1.0 },
  { "Burak", 1.0 },
  { "Bural", 1.0 },
  { "Burbon", 1.0 },
  { "Burce", 1.0 },
  { "Burch", 1.0 },
  { "Burchard", 1.0 },
  { "Burchel", 1.0 },
  { "Burchell", 1.0 },
  { "Burdell", 0.8514589 },
  { "Burdella", 0.0 },
  { "Burdelle", 0.0 },
  { "Burdena", 0.0 },
  { "Burdene", 0.0 },
  { "Burdett", 1.0 },
  { "Burdetta", 0.0 },
  { "Burdette", 0.93097824 },
  { "Burdie", 0.0 },
  { "Burdine", 0.0 },
  { "Burech", 1.0 },
  { "Burel", 1.0 },
  { "Burell", 1.0 },
  { "Buren", 1.0 },
  { "Burford", 1.0 },
  { "Burgandi", 0.0 },
  { "Burgandie", 0.0 },
  { "Burgandy", 0.0 },
  { "Burgess", 1.0 },
  { "Burgin", 1.0 },
  { "Burgundy", 0.0 },
  { "Burhan", 1.0 },
  { "Burhanuddin", 1.0 },
  { "Burie", 1.0 },
  { "Burim", 1.0 },
  { "Burk", 1.0 },
  { "Burke", 0.98987687 },
  { "Burklee", 0.0 },
  { "Burkleigh", 0.0 },
  { "Burkley", 0.5217391 },
  { "Burl", 0.9973515 },
  { "Burla", 0.0 },
  { "Burle", 1.0 },
  { "Burleen", 0.0 },
  { "Burleigh", 1.0 },
  { "Burlen", 1.0 },
  { "Burlene", 0.0 },
  { "Burley", 0.9829206 },
  { "Burlie", 0.7765363 },
  { "Burlin", 1.0 },
  { "Burline", 0.0 },
  { "Burlon", 1.0 },
  { "Burlyn", 1.0 },
  { "Burma", 0.0 },
  { "Burmah", 0.0 },
  { "Burman", 1.0 },
  { "Burna", 0.0 },
  { "Burnace", 1.0 },
  { "Burnadette", 0.0 },
  { "Burnadine", 0.0 },
  { "Burnam", 1.0 },
  { "Burnard", 1.0 },
  { "Burneice", 0.0 },
  { "Burnell", 0.8239311 },
  { "Burnelle", 0.0 },
  { "Burnes", 0.91935486 },
  { "Burnese", 0.0 },
  { "Burness", 1.0 },
  { "Burnest", 1.0 },
  { "Burnestine", 0.0 },
  { "Burnet", 1.0 },
  { "Burneta", 0.0 },
  { "Burnett", 0.8465293 },
  { "Burnetta", 0.0 },
  { "Burnette", 0.23370111 },
  { "Burney", 0.9943694 },
  { "Burnham", 1.0 },
  { "Burnice", 0.4553846 },
  { "Burnie", 0.9339019 },
  { "Burniece", 0.0 },
  { "Burnis", 0.9034335 },
  { "Burnita", 0.0 },
  { "Burnley", 1.0 },
  { "Burns", 1.0 },
  { "Buron", 1.0 },
  { "Burr", 1.0 },
  { "Burrel", 1.0 },
  { "Burrell", 1.0 },
  { "Burrill", 1.0 },
  { "Burris", 1.0 },
  { "Burrus", 1.0 },
  { "Burt", 1.0 },
  { "Burtha", 0.0 },
  { "Burtie", 0.0 },
  { "Burtis", 1.0 },
  { "Burton", 0.9978481 },
  { "Buruk", 1.0 },
  { "Burvin", 1.0 },
  { "Burwell", 1.0 },
  { "Buryl", 1.0 },
  { "Buse", 0.0 },
  { "Bush", 1.0 },
  { "Bushra", 0.0 },
  { "Busra", 0.0 },
  { "Buss", 1.0 },
  { "Buster", 0.9993802 },
  { "Butch", 1.0 },
  { "Butler", 1.0 },
  { "Butterfly", 0.0 },
  { "Buxton", 1.0 },
  { "Buz", 1.0 },
  { "Buzz", 1.0 },
  { "Buzzy", 1.0 },
  { "Bwana", 1.0 },
  { "Byan", 1.0 },
  { "Byanca", 0.0 },
  { "Byanka", 0.0 },
  { "Byard", 1.0 },
  { "Byata", 0.0 },
  { "Byford", 1.0 },
  { "Byntlee", 0.84375 },
  { "Bynum", 1.0 },
  { "Byonce", 0.0 },
  { "Byra", 0.0 },
  { "Byran", 1.0 },
  { "Byrant", 1.0 },
  { "Byrce", 1.0 },
  { "Byrd", 0.73700625 },
  { "Byrde", 0.0 },
  { "Byrdie", 0.0 },
  { "Byren", 1.0 },
  { "Byrl", 0.90924656 },
  { "Byrle", 0.53333336 },
  { "Byrne", 1.0 },
  { "Byrnece", 0.0 },
  { "Byron", 0.9962643 },
  { "Byrum", 1.0 },
  { "Byson", 1.0 },
  { "Caaden", 1.0 },
  { "Caaliyah", 0.0 },
  { "Caanan", 1.0 },
  { "Caasi", 0.0 },
  { "Cabe", 1.0 },
  { "Cabel", 1.0 },
  { "Cabela", 0.0 },
  { "Cabell", 1.0 },
  { "Cabella", 0.0 },
  { "Cable", 1.0 },
  { "Cabot", 1.0 },
  { "Cabral", 1.0 },
  { "Cabren", 1.0 },
  { "Cabria", 0.0 },
  { "Cabrielle", 0.0 },
  { "Cabrina", 0.0 },
  { "Cabrini", 0.0 },
  { "Cace", 1.0 },
  { "Cacee", 0.0 },
  { "Cacey", 0.07798165 },
  { "Cache", 0.6078767 },
  { "Cachet", 0.0 },
  { "Caci", 0.0 },
  { "Cacia", 0.0 },
  { "Cacie", 0.0 },
  { "Cacy", 0.0 },
  { "Cad", 1.0 },
  { "Cada", 0.0 },
  { "Cadan", 1.0 },
  { "Cadance", 0.005688282 },
  { "Cadarius", 1.0 },
  { "Cadarrius", 1.0 },
  { "Cadden", 1.0 },
  { "Caddie", 0.0 },
  { "Cade", 0.995956 },
  { "Cadedra", 0.0 },
  { "Cadee", 0.0 },
  { "Cadeidra", 0.0 },
  { "Cadeisha", 0.0 },
  { "Cadeja", 0.0 },
  { "Cadejah", 0.0 },
  { "Cadel", 1.0 },
  { "Cadell", 1.0 },
  { "Cadem", 1.0 },
  { "Cademon", 1.0 },
  { "Caden", 0.9856904 },
  { "Cadence", 0.065730534 },
  { "Cadense", 0.0 },
  { "Cadesha", 0.0 },
  { "Cadeshia", 0.0 },
  { "Cadesia", 0.0 },
  { "Cadet", 1.0 },
  { "Cadey", 0.0 },
  { "Cadge", 1.0 },
  { "Cadhla", 0.0 },
  { "Cadi", 0.0 },
  { "Cadia", 0.0 },
  { "Cadian", 1.0 },
  { "Cadie", 0.0 },
  { "Cadien", 1.0 },
  { "Cadience", 0.0 },
  { "Cadijah", 0.0 },
  { "Cadillac", 1.0 },
  { "Cadin", 0.9752322 },
  { "Cadince", 0.0 },
  { "Cadisha", 0.0 },
  { "Cadon", 1.0 },
  { "Cadrian", 0.6296296 },
  { "Cady", 0.008837826 },
  { "Cadyn", 0.71251935 },
  { "Cadynce", 0.0 },
  { "Caed", 1.0 },
  { "Caedan", 1.0 },
  { "Caedance", 0.0 },
  { "Caeden", 0.98432434 },
  { "Caedence", 0.0 },
  { "Caedin", 1.0 },
  { "Caedmon", 1.0 },
  { "Caedon", 1.0 },
  { "Caedyn", 0.79019076 },
  { "Cael", 0.99597526 },
  { "Caela", 0.0 },
  { "Caelainn", 0.0 },
  { "Caelan", 0.76484287 },
  { "Caeleb", 1.0 },
  { "Caelee", 0.0 },
  { "Caeleigh", 0.0 },
  { "Caelen", 0.8721374 },
  { "Caeley", 0.0 },
  { "Caeli", 0.0 },
  { "Caelia", 0.0 },
  { "Caelie", 0.0 },
  { "Caelin", 0.43137255 },
  { "Caelix", 1.0 },
  { "Caellum", 1.0 },
  { "Caelob", 1.0 },
  { "Caelon", 1.0 },
  { "Caelum", 1.0 },
  { "Caely", 0.0 },
  { "Caelyn", 0.002408478 },
  { "Caelynn", 0.0 },
  { "Caera", 0.0 },
  { "Caesar", 1.0 },
  { "Caesare", 0.8076923 },
  { "Caeser", 1.0 },
  { "Caeson", 1.0 },
  { "Caesyn", 1.0 },
  { "Caetano", 1.0 },
  { "Caetlin", 0.0 },
  { "Caetlyn", 0.0 },
  { "Caezar", 1.0 },
  { "Caffey", 0.0 },
  { "Caffie", 0.0 },
  { "Caffrey", 1.0 },
  { "Cagan", 1.0 },
  { "Cage", 1.0 },
  { "Cager", 1.0 },
  { "Cagney", 0.36082473 },
  { "Cahari", 1.0 },
  { "Cahill", 1.0 },
  { "Cahlia", 0.0 },
  { "Cahlil", 1.0 },
  { "Cahlin", 0.0 },
  { "Cahmari", 1.0 },
  { "Cahner", 1.0 },
  { "Cahterine", 0.0 },
  { "Cahya", 0.0 },
  { "Cai", 0.72930866 },
  { "Caia", 0.0 },
  { "Caiah", 0.0 },
  { "Caid", 1.0 },
  { "Caida", 0.0 },
  { "Caidan", 1.0 },
  { "Caidance", 0.0 },
  { "Caide", 1.0 },
  { "Caidee", 0.0 },
  { "Caiden", 0.973911 },
  { "Caidence", 0.09117647 },
  { "Caidenn", 1.0 },
  { "Caidin", 1.0 },
  { "Caidon", 1.0 },
  { "Caidy", 0.0 },
  { "Caidyn", 0.60105264 },
  { "Caidynce", 0.0 },
  { "Caige", 1.0 },
  { "Cail", 1.0 },
  { "Caila", 0.0 },
  { "Cailah", 0.0 },
  { "Cailan", 0.6519608 },
  { "Caile", 0.0 },
  { "Cailean", 0.8 },
  { "Caileb", 1.0 },
  { "Cailee", 0.0 },
  { "Caileen", 0.0 },
  { "Caileigh", 0.0 },
  { "Cailen", 0.51715684 },
  { "Cailey", 0.0 },
  { "Caili", 0.0 },
  { "Cailie", 0.0 },
  { "Cailin", 0.008493771 },
  { "Cailley", 0.0 },
  { "Caillou", 1.0 },
  { "Cailon", 0.0 },
  { "Cailtin", 0.0 },
  { "Cailum", 1.0 },
  { "Caily", 0.0 },
  { "Cailyn", 0.0 },
  { "Cailynn", 0.0 },
  { "Cailynne", 0.0 },
  { "Caiman", 1.0 },
  { "Caimen", 1.0 },
  { "Caimin", 1.0 },
  { "Caimon", 1.0 },
  { "Cain", 1.0 },
  { "Cainan", 1.0 },
  { "Caine", 1.0 },
  { "Cainen", 1.0 },
  { "Cainin", 1.0 },
  { "Cainon", 1.0 },
  { "Caio", 1.0 },
  { "Caira", 0.0 },
  { "Cairah", 0.0 },
  { "Cairee", 1.0 },
  { "Cairi", 0.0 },
  { "Cairl", 1.0 },
  { "Cairn", 0.0 },
  { "Cairo", 0.89958656 },
  { "Cairon", 1.0 },
  { "Cairra", 0.0 },
  { "Caisen", 1.0 },
  { "Caisey", 0.0 },
  { "Caisha", 0.0 },
  { "Caisley", 0.0 },
  { "Caislynn", 0.0 },
  { "Caison", 1.0 },
  { "Caisyn", 0.525 },
  { "Cait", 0.0 },
  { "Caite", 0.0 },
  { "Caitelyn", 0.0 },
  { "Caithlin", 0.0 },
  { "Caitie", 0.0 },
  { "Caitilin", 0.0 },
  { "Caitilyn", 0.0 },
  { "Caitin", 0.0 },
  { "Caitlain", 0.0 },
  { "Caitlan", 0.0 },
  { "Caitland", 0.0 },
  { "Caitlen", 0.0 },
  { "Caitlin", 0.0025403493 },
  { "Caitline", 0.0 },
  { "Caitlinn", 0.0 },
  { "Caitlyn", 0.0006621227 },
  { "Caitlynd", 0.0 },
  { "Caitlyne", 0.0 },
  { "Caitlynn", 0.0 },
  { "Caitlynne", 0.0 },
  { "Caitria", 0.0 },
  { "Caitrin", 0.0 },
  { "Caitriona", 0.0 },
  { "Caitryn", 0.0 },
  { "Caity", 0.0 },
  { "Caityln", 0.0 },
  { "Caitylyn", 0.0 },
  { "Caius", 1.0 },
  { "Caiya", 0.0 },
  { "Caiyah", 0.0 },
  { "Caiyden", 1.0 },
  { "Cajuan", 1.0 },
  { "Cajun", 1.0 },
  { "Cal", 0.99808025 },
  { "Cala", 0.0 },
  { "Calab", 1.0 },
  { "Calaeb", 1.0 },
  { "Calah", 0.0 },
  { "Calahan", 1.0 },
  { "Calais", 0.0 },
  { "Calamity", 0.0 },
  { "Calan", 0.9690881 },
  { "Calanda", 0.0 },
  { "Calandra", 0.0 },
  { "Calandria", 0.0 },
  { "Calani", 0.0 },
  { "Calantha", 0.0 },
  { "Calanthe", 0.0 },
  { "Calasia", 0.0 },
  { "Calaya", 0.0 },
  { "Calayah", 0.0 },
  { "Calaysia", 0.0 },
  { "Calbe", 1.0 },
  { "Calbert", 1.0 },
  { "Calcifer", 1.0 },
  { "Calden", 1.0 },
  { "Calder", 0.98093843 },
  { "Caldonia", 0.0 },
  { "Caldwell", 1.0 },
  { "Cale", 0.9893872 },
  { "Calea", 0.0 },
  { "Caleab", 1.0 },
  { "Caleah", 0.0 },
  { "Caleal", 1.0 },
  { "Caleb", 0.9978526 },
  { "Calebe", 1.0 },
  { "Calebjames", 1.0 },
  { "Caled", 1.0 },
  { "Caledonia", 0.0 },
  { "Calee", 0.0 },
  { "Caleel", 1.0 },
  { "Caleen", 0.0 },
  { "Caleena", 0.0 },
  { "Caleesha", 0.0 },
  { "Caleesi", 0.0 },
  { "Calei", 0.0 },
  { "Caleia", 0.0 },
  { "Caleiah", 0.0 },
  { "Caleigh", 0.0 },
  { "Caleigha", 0.0 },
  { "Calek", 1.0 },
  { "Calel", 1.0 },
  { "Calem", 1.0 },
  { "Calen", 0.9035777 },
  { "Calena", 0.0 },
  { "Calene", 0.0 },
  { "Caleob", 1.0 },
  { "Calep", 1.0 },
  { "Calese", 0.0 },
  { "Calesha", 0.0 },
  { "Caleshia", 0.0 },
  { "Caletha", 0.0 },
  { "Calev", 1.0 },
  { "Calex", 1.0 },
  { "Caley", 0.059285432 },
  { "Caleya", 0.0 },
  { "Caleyah", 0.0 },
  { "Calhan", 1.0 },
  { "Calhoun", 1.0 },
  { "Cali", 0.009807148 },
  { "Calia", 0.0 },
  { "Caliah", 0.0 },
  { "Calian", 1.0 },
  { "Caliana", 0.0 },
  { "Caliann", 0.0 },
  { "Calianna", 0.0 },
  { "Calianne", 0.0 },
  { "Calib", 1.0 },
  { "Caliber", 0.95454544 },
  { "Calice", 0.0 },
  { "Calicia", 0.0 },
  { "Calida", 0.0 },
  { "Calie", 0.0 },
  { "Calieb", 1.0 },
  { "Caliee", 0.0 },
  { "Caliegh", 0.0 },
  { "Caliel", 1.0 },
  { "Caliese", 0.0 },
  { "California", 0.0 },
  { "Calii", 0.0 },
  { "Calijah", 1.0 },
  { "Calik", 1.0 },
  { "Calil", 1.0 },
  { "Calilynn", 0.0 },
  { "Calimarie", 0.0 },
  { "Calin", 0.81069607 },
  { "Calina", 0.0 },
  { "Calinda", 0.0 },
  { "Caline", 0.0 },
  { "Calionna", 0.0 },
  { "Caliope", 0.0 },
  { "Calip", 1.0 },
  { "Caliph", 1.0 },
  { "Calirae", 0.0 },
  { "Calirose", 0.0 },
  { "Calis", 0.0 },
  { "Calisa", 0.0 },
  { "Calise", 0.0 },
  { "Calisha", 0.0 },
  { "Calisi", 0.0 },
  { "Calissa", 0.0 },
  { "Calisse", 0.0 },
  { "Calista", 0.0 },
  { "Calisto", 1.0 },
  { "Calistro", 1.0 },
  { "Calita", 0.0 },
  { "Calix", 1.0 },
  { "Calixta", 0.0 },
  { "Calixto", 1.0 },
  { "Calixtro", 1.0 },
  { "Caliya", 0.0 },
  { "Caliyah", 0.0 },
  { "Calizza", 0.0 },
  { "Calla", 0.0 },
  { "Callaghan", 0.87741935 },
  { "Callah", 0.0 },
  { "Callahan", 0.6927298 },
  { "Callalily", 0.0 },
  { "Callan", 0.73530877 },
  { "Callandra", 0.0 },
  { "Callasandra", 0.0 },
  { "Callaway", 0.51435405 },
  { "Calle", 0.0 },
  { "Callee", 0.0 },
  { "Calleen", 0.0 },
  { "Calleigh", 0.0 },
  { "Callen", 0.91518146 },
  { "Calletana", 0.0 },
  { "Calletano", 1.0 },
  { "Calley", 0.0 },
  { "Calli", 0.0 },
  { "Callia", 0.0 },
  { "Calliah", 0.0 },
  { "Calliana", 0.0 },
  { "Callianna", 0.0 },
  { "Callianne", 0.0 },
  { "Callie", 0.011236156 },
  { "Callieann", 0.0 },
  { "Calliegh", 0.0 },
  { "Calliemae", 0.0 },
  { "Callin", 0.9430894 },
  { "Calliope", 0.0 },
  { "Callis", 1.0 },
  { "Callisa", 0.0 },
  { "Callison", 0.0 },
  { "Callissa", 0.0 },
  { "Callista", 0.0 },
  { "Callister", 1.0 },
  { "Callisto", 0.0 },
  { "Callon", 1.0 },
  { "Calloway", 0.9138756 },
  { "Callum", 1.0 },
  { "Cally", 0.0 },
  { "Callye", 0.0 },
  { "Callyn", 0.013550135 },
  { "Callysta", 0.0 },
  { "Calman", 1.0 },
  { "Calmer", 1.0 },
  { "Calob", 1.0 },
  { "Calogera", 0.0 },
  { "Calogero", 1.0 },
  { "Calon", 1.0 },
  { "Calondra", 0.0 },
  { "Calonia", 0.0 },
  { "Calsey", 0.0 },
  { "Caltlin", 0.0 },
  { "Calton", 1.0 },
  { "Calub", 1.0 },
  { "Caludia", 0.0 },
  { "Calum", 1.0 },
  { "Calvan", 1.0 },
  { "Calvary", 0.4021739 },
  { "Calven", 1.0 },
  { "Calvert", 1.0 },
  { "Calvester", 1.0 },
  { "Calvin", 0.9939994 },
  { "Calvina", 0.0 },
  { "Calvinesha", 0.0 },
  { "Calvion", 1.0 },
  { "Calvon", 1.0 },
  { "Calvyn", 1.0 },
  { "Caly", 0.0 },
  { "Calyb", 1.0 },
  { "Calyce", 0.0 },
  { "Calyn", 0.099734046 },
  { "Calynn", 0.0 },
  { "Calynne", 0.0 },
  { "Calypso", 0.0 },
  { "Calyse", 0.0 },
  { "Calyssa", 0.0 },
  { "Calysta", 0.0 },
  { "Calyx", 0.54022986 },
  { "Cam", 0.83255816 },
  { "Cama", 0.0 },
  { "Camacho", 1.0 },
  { "Camaira", 0.0 },
  { "Camaiya", 0.0 },
  { "Camala", 0.0 },
  { "Camalla", 0.0 },
  { "Camani", 0.234375 },
  { "Camar", 1.0 },
  { "Camara", 0.1042471 },
  { "Camare", 1.0 },
  { "Camaree", 0.54545456 },
  { "Camaren", 0.91525424 },
  { "Camareon", 1.0 },
  { "Camari", 0.5948492 },
  { "Camaria", 0.0 },
  { "Camariah", 0.0 },
  { "Camarie", 0.09767442 },
  { "Camarii", 1.0 },
  { "Camario", 1.0 },
  { "Camarion", 1.0 },
  { "Camariyah", 0.0 },
  { "Camaro", 0.40625 },
  { "Camaron", 0.92896175 },
  { "Camarri", 0.5 },
  { "Camary", 0.0 },
  { "Camaryn", 0.0 },
  { "Camas", 0.0 },
  { "Camauri", 0.61413044 },
  { "Camaury", 1.0 },
  { "Camay", 0.0 },
  { "Camaya", 0.0 },
  { "Cambel", 0.0 },
  { "Cambell", 0.2744361 },
  { "Cambelle", 0.0 },
  { "Camber", 0.056016598 },
  { "Camberly", 0.0 },
  { "Cambi", 0.0 },
  { "Cambra", 0.0 },
  { "Cambre", 0.0 },
  { "Cambrea", 0.0 },
  { "Cambree", 0.0 },
  { "Cambreigh", 0.0 },
  { "Cambren", 1.0 },
  { "Cambrey", 0.0 },
  { "Cambri", 0.0 },
  { "Cambria", 0.0 },
  { "Cambriah", 0.0 },
  { "Cambridge", 0.53535354 },
  { "Cambrie", 0.0 },
  { "Cambriella", 0.0 },
  { "Cambrielle", 0.0 },
  { "Cambron", 1.0 },
  { "Cambry", 0.0 },
  { "Cambrya", 0.0 },
  { "Cambryn", 0.19135803 },
  { "Cambrynn", 0.0 },
  { "Camdan", 1.0 },
  { "Camden", 0.9347102 },
  { "Camdin", 1.0 },
  { "Camdon", 1.0 },
  { "Camdyn", 0.5481941 },
  { "Camdynn", 0.27631578 },
  { "Camea", 0.0 },
  { "Cameca", 0.0 },
  { "Camecia", 0.0 },
  { "Camee", 0.0 },
  { "Cameika", 0.0 },
  { "Cameil", 0.0 },
  { "Cameisha", 0.0 },
  { "Cameka", 0.0 },
  { "Camela", 0.0 },
  { "Camelia", 0.0 },
  { "Camella", 0.0 },
  { "Camelle", 0.0 },
  { "Camellia", 0.0 },
  { "Camello", 1.0 },
  { "Camelo", 1.0 },
  { "Camely", 0.0 },
  { "Camen", 0.84615386 },
  { "Cameo", 0.12566371 },
  { "Cameon", 1.0 },
  { "Camera", 0.0 },
  { "Cameran", 0.5216895 },
  { "Cameren", 0.8564626 },
  { "Cameria", 0.0 },
  { "Camerin", 0.7008872 },
  { "Camerina", 0.0 },
  { "Camerino", 1.0 },
  { "Camerion", 1.0 },
  { "Cameron", 0.9089988 },
  { "Camerone", 1.0 },
  { "Camerron", 1.0 },
  { "Camerson", 1.0 },
  { "Camery", 0.0 },
  { "Cameryn", 0.22112 },
  { "Camerynn", 0.0 },
  { "Camesha", 0.0 },
  { "Cameshia", 0.0 },
  { "Camey", 0.0 },
  { "Cami", 0.0 },
  { "Camia", 0.0 },
  { "Camiah", 0.0 },
  { "Camica", 0.0 },
  { "Camie", 0.0 },
  { "Camiel", 0.7 },
  { "Camielle", 0.0 },
  { "Camiesha", 0.0 },
  { "Camika", 0.0 },
  { "Camil", 0.39007092 },
  { "Camila", 0.00087950745 },
  { "Camilah", 0.0 },
  { "Camile", 0.036065575 },
  { "Camili", 0.0 },
  { "Camilia", 0.0 },
  { "Camill", 0.0 },
  { "Camilla", 0.00028684802 },
  { "Camillah", 0.0 },
  { "Camille", 0.020291308 },
  { "Camillemarie", 0.0 },
  { "Camillia", 0.0 },
  { "Camillie", 0.0 },
  { "Camillo", 1.0 },
  { "Camillus", 1.0 },
  { "Camilly", 0.0 },
  { "Camillya", 0.0 },
  { "Camilo", 1.0 },
  { "Camily", 0.0 },
  { "Camilya", 0.0 },
  { "Camira", 0.0 },
  { "Camirah", 0.0 },
  { "Camiren", 1.0 },
  { "Camiron", 1.0 },
  { "Camiryn", 0.0 },
  { "Camisa", 0.0 },
  { "Camisha", 0.0 },
  { "Camishia", 0.0 },
  { "Camiya", 0.0 },
  { "Camiyah", 0.0 },
  { "Camlin", 0.5 },
  { "Camlyn", 0.0 },
  { "Camm", 1.0 },
  { "Cammeron", 1.0 },
  { "Cammi", 0.0 },
  { "Cammie", 0.0071905497 },
  { "Cammille", 0.0 },
  { "Cammron", 1.0 },
  { "Cammy", 0.0 },
  { "Camon", 1.0 },
  { "Camoni", 0.0 },
  { "Camonte", 1.0 },
  { "Camora", 0.0 },
  { "Camorah", 0.0 },
  { "Camoren", 1.0 },
  { "Camori", 0.0 },
  { "Camorra", 0.0 },
  { "Camoya", 0.0 },
  { "Camp", 1.0 },
  { "Campbell", 0.45921385 },
  { "Campbelle", 0.0 },
  { "Camper", 1.0 },
  { "Campion", 1.0 },
  { "Campton", 1.0 },
  { "Camra", 0.0 },
  { "Camran", 0.9411765 },
  { "Camree", 0.0 },
  { "Camren", 0.91391563 },
  { "Camreon", 1.0 },
  { "Camrey", 0.0 },
  { "Camri", 0.0 },
  { "Camrie", 0.0 },
  { "Camrin", 0.8145973 },
  { "Camrion", 1.0 },
  { "Camron", 0.97170013 },
  { "Camrun", 1.0 },
  { "Camry", 0.089955024 },
  { "Camryn", 0.13131493 },
  { "Camryne", 0.0 },
  { "Camrynn", 0.0669145 },
  { "Camrynne", 0.0 },
  { "Camy", 0.0 },
  { "Camya", 0.0 },
  { "Camyah", 0.0 },
  { "Camyia", 0.0 },
  { "Camyiah", 0.0 },
  { "Camyla", 0.0 },
  { "Camylle", 0.0 },
  { "Camyra", 0.0 },
  { "Camyrah", 0.0 },
  { "Camyri", 0.0 },
  { "Camyria", 0.0 },
  { "Camyrn", 0.0 },
  { "Camyron", 0.6875 },
  { "Can", 1.0 },
  { "Cana", 0.0 },
  { "Canaan", 0.9329779 },
  { "Canace", 0.0 },
  { "Canada", 0.0 },
  { "Canan", 0.8484849 },
  { "Canary", 0.0 },
  { "Canasia", 0.0 },
  { "Canda", 0.0 },
  { "Candace", 0.0018688936 },
  { "Candan", 1.0 },
  { "Candance", 0.0 },
  { "Candas", 0.0 },
  { "Candase", 0.0 },
  { "Cande", 0.0 },
  { "Candece", 0.0 },
  { "Candee", 0.0 },
  { "Candela", 0.0 },
  { "Candelaria", 0.0 },
  { "Candelario", 1.0 },
  { "Canden", 1.0 },
  { "Candence", 0.0 },
  { "Candes", 0.0 },
  { "Candess", 0.0 },
  { "Candi", 0.0 },
  { "Candia", 0.0 },
  { "Candiace", 0.0 },
  { "Candiance", 0.0 },
  { "Candias", 0.0 },
  { "Candic", 0.0 },
  { "Candice", 0.0022894111 },
  { "Candid", 0.0 },
  { "Candida", 0.0 },
  { "Candido", 1.0 },
  { "Candie", 0.0 },
  { "Candiece", 0.0 },
  { "Candies", 0.0 },
  { "Candina", 0.0 },
  { "Candis", 0.0 },
  { "Candise", 0.0 },
  { "Candiss", 0.0 },
  { "Candita", 0.0 },
  { "Candius", 0.0 },
  { "Candle", 0.0 },
  { "Candler", 0.92537314 },
  { "Candon", 1.0 },
  { "Candra", 0.0 },
  { "Candrea", 0.0 },
  { "Candria", 0.0 },
  { "Candrice", 0.0 },
  { "Candus", 0.0 },
  { "Candy", 0.0035371936 },
  { "Candyce", 0.0 },
  { "Candye", 0.0 },
  { "Candys", 0.0 },
  { "Candyse", 0.0 },
  { "Cane", 1.0 },
  { "Canei", 0.0 },
  { "Caneisha", 0.0 },
  { "Canek", 1.0 },
  { "Canela", 0.0 },
  { "Canelo", 1.0 },
  { "Canen", 1.0 },
  { "Canesha", 0.0 },
  { "Caneshia", 0.0 },
  { "Canessa", 0.0 },
  { "Canh", 1.0 },
  { "Cania", 0.0 },
  { "Caniah", 0.0 },
  { "Canice", 0.0 },
  { "Caniesha", 0.0 },
  { "Canin", 1.0 },
  { "Canio", 1.0 },
  { "Canisha", 0.0 },
  { "Canishia", 0.0 },
  { "Canita", 0.0 },
  { "Caniya", 0.0 },
  { "Caniyah", 0.0 },
  { "Canna", 0.0 },
  { "Cannan", 1.0 },
  { "Cannen", 1.0 },
  { "Cannie", 0.022624435 },
  { "Cannin", 1.0 },
  { "Cannon", 0.97886705 },
  { "Canon", 0.9871037 },
  { "Cansas", 0.0 },
  { "Cansu", 0.0 },
  { "Cante", 0.0 },
  { "Cantina", 0.0 },
  { "Canton", 1.0 },
  { "Cantrece", 0.0 },
  { "Cantrell", 0.8611111 },
  { "Cantrice", 0.0 },
  { "Canuto", 1.0 },
  { "Canya", 0.0 },
  { "Canye", 1.0 },
  { "Canyen", 1.0 },
  { "Canyla", 0.0 },
  { "Canyn", 1.0 },
  { "Canyon", 0.9457639 },
  { "Caoilainn", 0.0 },
  { "Caoilinn", 0.0 },
  { "Caoimhe", 0.0 },
  { "Caolan", 1.0 },
  { "Cap", 1.0 },
  { "Cape", 1.0 },
  { "Capella", 0.0 },
  { "Capers", 1.0 },
  { "Capitola", 0.0 },
  { "Capone", 1.0 },
  { "Capp", 1.0 },
  { "Cappie", 0.0 },
  { "Cappy", 0.5 },
  { "Caprece", 0.0 },
  { "Caprecia", 0.0 },
  { "Capree", 0.15151516 },
  { "Capresha", 0.0 },
  { "Capri", 0.018874172 },
  { "Capria", 0.0 },
  { "Caprial", 0.0 },
  { "Capriana", 0.0 },
  { "Caprica", 0.0 },
  { "Caprice", 0.04933256 },
  { "Capricia", 0.0 },
  { "Caprina", 0.0 },
  { "Caprise", 0.0 },
  { "Caprisha", 0.0 },
  { "Captain", 1.0 },
  { "Captola", 0.0 },
  { "Captolia", 0.0 },
  { "Capucine", 0.0 },
  { "Capus", 1.0 },
  { "Car", 1.0 },
  { "Cara", 0.0012108902 },
  { "Carabella", 0.0 },
  { "Carabelle", 0.0 },
  { "Caragan", 0.0 },
  { "Caragh", 0.0 },
  { "Carah", 0.0 },
  { "Caral", 0.0 },
  { "Caralee", 0.0 },
  { "Caraleigh", 0.0 },
  { "Caralena", 0.0 },
  { "Caralina", 0.0 },
  { "Caraline", 0.0 },
  { "Caralyn", 0.0 },
  { "Caralyne", 0.0 },
  { "Caralynn", 0.0 },
  { "Caramia", 0.0 },
  { "Caramie", 0.0 },
  { "Caran", 0.0 },
  { "Carbon", 1.0 },
  { "Carcyn", 0.0 },
  { "Cardae", 1.0 },
  { "Cardale", 1.0 },
  { "Cardarius", 1.0 },
  { "Cardea", 1.0 },
  { "Cardel", 1.0 },
  { "Cardelia", 0.0 },
  { "Cardell", 0.9900433 },
  { "Cardella", 0.0 },
  { "Carden", 0.89403975 },
  { "Carder", 1.0 },
  { "Cardero", 1.0 },
  { "Cardia", 0.0 },
  { "Cardie", 0.0 },
  { "Cardier", 1.0 },
  { "Cardin", 0.9228395 },
  { "Cardine", 0.0 },
  { "Cardis", 1.0 },
  { "Cardiss", 0.0 },
  { "Cardon", 1.0 },
  { "Cardyn", 0.0 },
  { "Care", 1.0 },
  { "Caree", 0.0 },
  { "Careem", 1.0 },
  { "Careen", 0.0 },
  { "Careena", 0.0 },
  { "Careese", 0.0 },
  { "Carel", 0.70731705 },
  { "Careli", 0.0 },
  { "Carell", 1.0 },
  { "Carelle", 0.0 },
  { "Carely", 0.0 },
  { "Carelyn", 0.0 },
  { "Caren", 0.0 },
  { "Carena", 0.0 },
  { "Carenda", 0.0 },
  { "Carene", 0.0 },
  { "Careron", 1.0 },
  { "Caresa", 0.0 },
  { "Caresha", 0.0 },
  { "Caress", 0.0 },
  { "Caressa", 0.0 },
  { "Caresse", 0.0 },
  { "Caretha", 0.0 },
  { "Caretta", 0.0 },
  { "Carey", 0.56891865 },
  { "Careyann", 0.0 },
  { "Cari", 0.0043744533 },
  { "Caria", 0.0 },
  { "Cariah", 0.0 },
  { "Cariana", 0.0 },
  { "Cariann", 0.0 },
  { "Carianna", 0.0 },
  { "Carianne", 0.0 },
  { "Caribbean", 0.0 },
  { "Caribe", 1.0 },
  { "Carice", 0.0 },
  { "Caricia", 0.0 },
  { "Caridad", 0.0 },
  { "Caridee", 0.0 },
  { "Carie", 0.0 },
  { "Cariel", 0.0 },
  { "Carielle", 0.0 },
  { "Carig", 1.0 },
  { "Carigan", 0.0 },
  { "Caril", 0.0 },
  { "Carilee", 0.0 },
  { "Carilla", 0.0 },
  { "Carilyn", 0.0 },
  { "Carilynn", 0.0 },
  { "Carime", 0.0 },
  { "Carin", 0.0 },
  { "Carina", 0.002352783 },
  { "Carinda", 0.0 },
  { "Carine", 0.0 },
  { "Carington", 0.0 },
  { "Carinna", 0.0 },
  { "Carinne", 0.0 },
  { "Cario", 1.0 },
  { "Carion", 1.0 },
  { "Caris", 0.0 },
  { "Carisa", 0.0 },
  { "Carise", 0.0 },
  { "Carisha", 0.0 },
  { "Carisia", 0.0 },
  { "Carisma", 0.0 },
  { "Cariss", 0.0 },
  { "Carissa", 0.0004320121 },
  { "Carisse", 0.0 },
  { "Carissia", 0.0 },
  { "Carista", 0.0 },
  { "Carita", 0.0 },
  { "Caritina", 0.0 },
  { "Cariya", 0.0 },
  { "Cariyah", 0.0 },
  { "Carizma", 0.0 },
  { "Carl", 0.9936852 },
  { "Carla", 0.0028687448 },
  { "Carlan", 0.61538464 },
  { "Carlana", 0.0 },
  { "Carland", 1.0 },
  { "Carlann", 0.0 },
  { "Carlanthony", 1.0 },
  { "Carlas", 0.7467811 },
  { "Carlasia", 0.0 },
  { "Carlaya", 0.0 },
  { "Carlaysia", 0.0 },
  { "Carle", 0.76237625 },
  { "Carlea", 0.0 },
  { "Carleah", 0.0 },
  { "Carlean", 0.0 },
  { "Carlecia", 0.0 },
  { "Carlee", 0.026183797 },
  { "Carleen", 0.0 },
  { "Carleena", 0.0 },
  { "Carleene", 0.0 },
  { "Carlei", 0.0 },
  { "Carleigh", 0.0 },
  { "Carleigha", 0.0 },
  { "Carleisha", 0.0 },
  { "Carlen", 0.1576087 },
  { "Carlena", 0.0 },
  { "Carlene", 0.0 },
  { "Carlens", 1.0 },
  { "Carleon", 1.0 },
  { "Carles", 0.9925262 },
  { "Carlese", 0.0 },
  { "Carlesha", 0.0 },
  { "Carleshia", 0.0 },
  { "Carlesia", 0.0 },
  { "Carless", 1.0 },
  { "Carlester", 1.0 },
  { "Carlet", 0.0 },
  { "Carleta", 0.0 },
  { "Carletha", 0.0 },
  { "Carlethia", 0.0 },
  { "Carleton", 1.0 },
  { "Carlett", 0.0 },
  { "Carletta", 0.0 },
  { "Carlette", 0.0 },
  { "Carley", 0.025019832 },
  { "Carlhenry", 1.0 },
  { "Carli", 0.00060760725 },
  { "Carlia", 0.0 },
  { "Carliana", 0.0 },
  { "Carlianna", 0.0 },
  { "Carlianne", 0.0 },
  { "Carlicia", 0.0 },
  { "Carlie", 0.08507836 },
  { "Carliee", 0.0 },
  { "Carliegh", 0.0 },
  { "Carlile", 1.0 },
  { "Carlin", 0.5832933 },
  { "Carlina", 0.0 },
  { "Carlinda", 0.0 },
  { "Carline", 0.0030138637 },
  { "Carling", 0.0 },
  { "Carlis", 0.87276626 },
  { "Carlisa", 0.0 },
  { "Carlise", 0.0 },
  { "Carlisha", 0.0 },
  { "Carlishia", 0.0 },
  { "Carlisia", 0.0 },
  { "Carlisle", 0.80840874 },
  { "Carliss", 0.218509 },
  { "Carlissa", 0.0 },
  { "Carlissia", 0.0 },
  { "Carlita", 0.0 },
  { "Carlitha", 0.0 },
  { "Carlito", 1.0 },
  { "Carlitos", 1.0 },
  { "Carlitta", 0.0 },
  { "Carliyah", 0.0 },
  { "Carlla", 0.0 },
  { "Carlleen", 0.0 },
  { "Carllene", 0.0 },
  { "Carlo", 0.9944193 },
  { "Carlon", 0.8210526 },
  { "Carlos", 0.99290586 },
  { "Carlosadrian", 1.0 },
  { "Carlosalberto", 1.0 },
  { "Carlosantonio", 1.0 },
  { "Carlosdaniel", 1.0 },
  { "Carlose", 1.0 },
  { "Carloseduardo", 1.0 },
  { "Carlosjavier", 1.0 },
  { "Carlosjr", 1.0 },
  { "Carlosmanuel", 1.0 },
  { "Carloss", 1.0 },
  { "Carlota", 0.0 },
  { "Carlotta", 0.0 },
  { "Carlotte", 0.0 },
  { "Carlous", 1.0 },
  { "Carloyn", 0.0 },
  { "Carlson", 1.0 },
  { "Carlston", 1.0 },
  { "Carlton", 0.9943076 },
  { "Carlus", 0.9813084 },
  { "Carlvin", 1.0 },
  { "Carly", 0.0025593038 },
  { "Carlyann", 0.0 },
  { "Carlye", 0.0 },
  { "Carlyle", 0.9544389 },
  { "Carlyn", 0.015854236 },
  { "Carlyne", 0.0 },
  { "Carlynn", 0.0 },
  { "Carlyon", 0.0 },
  { "Carlyrose", 0.0 },
  { "Carlyse", 0.0 },
  { "Carlysha", 0.0 },
  { "Carlysia", 0.0 },
  { "Carlysle", 1.0 },
  { "Carlyssa", 0.0 },
  { "Carlyssia", 0.0 },
  { "Carma", 0.0 },
  { "Carmah", 0.0 },
  { "Carmala", 0.0 },
  { "Carmalena", 0.0 },
  { "Carmaleta", 0.0 },
  { "Carmaletta", 0.0 },
  { "Carmaline", 0.0 },
  { "Carmalita", 0.0 },
  { "Carman", 0.31138584 },
  { "Carmani", 0.0 },
  { "Carmecia", 0.0 },
  { "Carmeisha", 0.0 },
  { "Carmel", 0.15934262 },
  { "Carmela", 0.00021597338 },
  { "Carmeleta", 0.0 },
  { "Carmeletta", 0.0 },
  { "Carmelia", 0.0 },
  { "Carmelina", 0.0 },
  { "Carmeline", 0.0 },
  { "Carmelita", 0.0 },
  { "Carmelite", 0.0 },
  { "Carmelito", 1.0 },
  { "Carmelitta", 0.0 },
  { "Carmell", 0.07317073 },
  { "Carmella", 0.0 },
  { "Carmelle", 0.0 },
  { "Carmellia", 0.0 },
  { "Carmello", 0.9878893 },
  { "Carmelo", 0.99949676 },
  { "Carmelyn", 0.0 },
  { "Carmen", 0.108595684 },
  { "Carmena", 0.0 },
  { "Carmencita", 0.0 },
  { "Carmene", 0.0 },
  { "Carmenita", 0.0 },
  { "Carmenlita", 0.0 },
  { "Carmeron", 1.0 },
  { "Carmesa", 0.0 },
  { "Carmesha", 0.0 },
  { "Carmeshia", 0.0 },
  { "Carmesia", 0.0 },
  { "Carmetta", 0.0 },
  { "Carmi", 0.39285713 },
  { "Carmia", 0.0 },
  { "Carmichael", 1.0 },
  { "Carmie", 0.14018692 },
  { "Carmila", 0.0 },
  { "Carmilita", 0.0 },
  { "Carmilla", 0.0 },
  { "Carmille", 0.0 },
  { "Carmin", 0.1408297 },
  { "Carmina", 0.0 },
  { "Carmindy", 0.0 },
  { "Carmine", 0.96357363 },
  { "Carminia", 0.0 },
  { "Carmino", 1.0 },
  { "Carmisha", 0.0 },
  { "Carmita", 0.0 },
  { "Carmon", 0.4423213 },
  { "Carmya", 0.0 },
  { "Carmyn", 0.0 },
  { "Carmyne", 1.0 },
  { "Carmynn", 0.0 },
  { "Carna", 0.0 },
  { "Carnation", 0.0 },
  { "Carneal", 1.0 },
  { "Carneisha", 0.0 },
  { "Carnel", 1.0 },
  { "Carnelia", 0.0 },
  { "Carnelius", 1.0 },
  { "Carnell", 0.89177406 },
  { "Carnella", 0.0 },
  { "Carnes", 1.0 },
  { "Carnesha", 0.0 },
  { "Carneshia", 0.0 },
  { "Carnetta", 0.0 },
  { "Carney", 0.98792756 },
  { "Carnie", 0.8035714 },
  { "Carnisha", 0.0 },
  { "Carnita", 0.0 },
  { "Caro", 0.018248174 },
  { "Carol", 0.010494535 },
  { "Carola", 0.0 },
  { "Carolan", 0.0 },
  { "Carolann", 0.0 },
  { "Carolanne", 0.0 },
  { "Carold", 1.0 },
  { "Caroldean", 0.0 },
  { "Caroldine", 0.0 },
  { "Carole", 0.0023807592 },
  { "Carolea", 0.0 },
  { "Caroleann", 0.0 },
  { "Carolee", 0.0 },
  { "Caroleen", 0.0 },
  { "Caroleena", 0.0 },
  { "Carolen", 0.0 },
  { "Carolena", 0.0 },
  { "Carolene", 0.0 },
  { "Caroletta", 0.0 },
  { "Carolette", 0.0 },
  { "Carolgene", 0.0 },
  { "Carolie", 0.0 },
  { "Carolin", 0.0 },
  { "Carolina", 0.003288684 },
  { "Carolinda", 0.0 },
  { "Caroline", 0.0013974528 },
  { "Carolita", 0.0 },
  { "Caroljean", 0.0 },
  { "Caroljo", 0.0 },
  { "Caroll", 0.373436 },
  { "Carolle", 0.0 },
  { "Carollee", 0.0 },
  { "Carollou", 0.0 },
  { "Carollyn", 0.0 },
  { "Carollynn", 0.0 },
  { "Carolos", 1.0 },
  { "Carols", 1.0 },
  { "Carolsue", 0.0 },
  { "Carolus", 1.0 },
  { "Caroly", 0.0 },
  { "Carolyn", 0.0029112003 },
  { "Carolyna", 0.0 },
  { "Carolyne", 0.0 },
  { "Carolynn", 0.0 },
  { "Carolynne", 0.0 },
  { "Carolyon", 0.0 },
  { "Caron", 0.22406639 },
  { "Caronda", 0.0 },
  { "Caros", 1.0 },
  { "Caroyl", 0.0 },
  { "Caroyn", 0.0 },
  { "Carpenter", 1.0 },
  { "Carper", 1.0 },
  { "Carr", 1.0 },
  { "Carra", 0.0 },
  { "Carragan", 0.0 },
  { "Carrah", 0.0 },
  { "Carranza", 1.0 },
  { "Carre", 0.0 },
  { "Carree", 0.0 },
  { "Carreen", 0.0 },
  { "Carrel", 0.9729064 },
  { "Carrell", 0.86965376 },
  { "Carren", 0.0 },
  { "Carrena", 0.0 },
  { "Carrera", 0.0 },
  { "Carressa", 0.0 },
  { "Carrey", 0.11320755 },
  { "Carri", 0.0 },
  { "Carriana", 0.0 },
  { "Carriann", 0.0 },
  { "Carrianna", 0.0 },
  { "Carrianne", 0.0 },
  { "Carrick", 1.0 },
  { "Carrie", 0.004460571 },
  { "Carrieann", 0.0 },
  { "Carrieanne", 0.0 },
  { "Carriebell", 0.0 },
  { "Carrielynn", 0.0 },
  { "Carrigan", 0.009871668 },
  { "Carrin", 0.0 },
  { "Carrina", 0.0 },
  { "Carrine", 0.0 },
  { "Carrington", 0.41520613 },
  { "Carrion", 1.0 },
  { "Carris", 0.5151515 },
  { "Carrisa", 0.0 },
  { "Carrissa", 0.0 },
  { "Carrol", 0.5878647 },
  { "Carrold", 1.0 },
  { "Carrole", 0.0 },
  { "Carroll", 0.8570079 },
  { "Carrolle", 0.0 },
  { "Carrolyn", 0.0 },
  { "Carron", 0.17594256 },
  { "Carrson", 1.0 },
  { "Carry", 0.1809409 },
  { "Carrye", 0.0 },
  { "Carryl", 0.0 },
  { "Carryn", 0.0 },
  { "Carsan", 1.0 },
  { "Carsandra", 0.0 },
  { "Carsen", 0.7806245 },
  { "Carshena", 0.0 },
  { "Carsie", 1.0 },
  { "Carsin", 1.0 },
  { "Carson", 0.93816054 },
  { "Carsten", 0.99749124 },
  { "Carston", 1.0 },
  { "Carstyn", 1.0 },
  { "Carsyn", 0.30397505 },
  { "Carsynn", 0.0 },
  { "Cartavious", 1.0 },
  { "Cartel", 1.0 },
  { "Cartell", 1.0 },
  { "Carter", 0.96854347 },
  { "Carterjames", 1.0 },
  { "Carterlee", 1.0 },
  { "Cartez", 1.0 },
  { "Cartha", 0.0 },
  { "Carthel", 1.0 },
  { "Carthell", 1.0 },
  { "Cartier", 0.870317 },
  { "Cartina", 0.0 },
  { "Cartisha", 0.0 },
  { "Cartlin", 0.0 },
  { "Cartney", 0.0 },
  { "Cartrell", 1.0 },
  { "Cartrina", 0.0 },
  { "Caruso", 1.0 },
  { "Carvel", 1.0 },
  { "Carvell", 1.0 },
  { "Carven", 1.0 },
  { "Carver", 0.99163383 },
  { "Carveth", 1.0 },
  { "Carville", 1.0 },
  { "Carvin", 1.0 },
  { "Carvis", 1.0 },
  { "Carvon", 1.0 },
  { "Carwin", 1.0 },
  { "Carwyn", 1.0 },
  { "Cary", 0.82035744 },
  { "Carya", 0.0 },
  { "Caryann", 0.0 },
  { "Carye", 0.0 },
  { "Caryl", 0.051412344 },
  { "Caryle", 0.0 },
  { "Caryll", 0.0 },
  { "Carylon", 0.0 },
  { "Caryn", 0.0 },
  { "Caryna", 0.0 },
  { "Carynn", 0.0 },
  { "Caryol", 0.0 },
  { "Carys", 0.0 },
  { "Caryss", 0.0 },
  { "Caryssa", 0.0 },
  { "Carzell", 1.0 },
  { "Cas", 1.0 },
  { "Casadee", 0.0 },
  { "Casadi", 0.0 },
  { "Casady", 0.0 },
  { "Casandr", 0.0 },
  { "Casandra", 0.00044759896 },
  { "Casanova", 1.0 },
  { "Casara", 0.0 },
  { "Casarah", 0.0 },
  { "Casaundra", 0.0 },
  { "Casaya", 0.0 },
  { "Casch", 1.0 },
  { "Cascia", 0.0 },
  { "Casden", 1.0 },
  { "Case", 1.0 },
  { "Casea", 0.0 },
  { "Casee", 0.0 },
  { "Caselyn", 0.0 },
  { "Caselynn", 0.0 },
  { "Casen", 0.99528956 },
  { "Caseton", 1.0 },
  { "Casey", 0.5914304 },
  { "Caseyann", 0.0 },
  { "Caseylynn", 0.0 },
  { "Cash", 0.9968437 },
  { "Casha", 0.0 },
  { "Cashae", 0.0 },
  { "Cashala", 0.0 },
  { "Cashana", 0.0 },
  { "Cashanda", 0.0 },
  { "Cashanti", 0.0 },
  { "Cashara", 0.0 },
  { "Cashaun", 1.0 },
  { "Cashawn", 1.0 },
  { "Cashay", 0.0 },
  { "Cashden", 1.0 },
  { "Cashe", 0.38181818 },
  { "Cashea", 0.0 },
  { "Casheena", 0.0 },
  { "Cashel", 1.0 },
  { "Cashen", 1.0 },
  { "Cashion", 1.0 },
  { "Cashious", 1.0 },
  { "Cashis", 1.0 },
  { "Cashius", 1.0 },
  { "Cashlin", 0.46666667 },
  { "Cashlyn", 0.0 },
  { "Cashlynn", 0.0 },
  { "Cashmere", 0.36373875 },
  { "Cashmiere", 1.0 },
  { "Cashmir", 0.8378378 },
  { "Cashon", 1.0 },
  { "Cashous", 1.0 },
  { "Cashten", 1.0 },
  { "Cashtin", 1.0 },
  { "Cashton", 1.0 },
  { "Cashtyn", 1.0 },
  { "Cashus", 1.0 },
  { "Casi", 0.0 },
  { "Casia", 0.0 },
  { "Casiah", 0.0 },
  { "Casiana", 0.0 },
  { "Casiano", 1.0 },
  { "Casidee", 0.0 },
  { "Casidhe", 0.0 },
  { "Casidy", 0.0 },
  { "Casie", 0.0037894738 },
  { "Casilda", 0.0 },
  { "Casimer", 1.0 },
  { "Casimera", 0.0 },
  { "Casimere", 1.0 },
  { "Casimier", 1.0 },
  { "Casimir", 1.0 },
  { "Casimira", 0.0 },
  { "Casimiro", 1.0 },
  { "Casin", 1.0 },
  { "Casina", 0.0 },
  { "Casino", 1.0 },
  { "Casius", 1.0 },
  { "Casiyah", 0.0 },
  { "Caslyn", 0.0 },
  { "Casmer", 1.0 },
  { "Casmera", 0.0 },
  { "Casmere", 1.0 },
  { "Casmier", 1.0 },
  { "Casmin", 0.0 },
  { "Casmir", 0.95254236 },
  { "Casmira", 0.0 },
  { "Cason", 0.99232835 },
  { "Casondra", 0.0 },
  { "Casonja", 0.0 },
  { "Casonya", 0.0 },
  { "Caspar", 1.0 },
  { "Casper", 1.0 },
  { "Caspian", 1.0 },
  { "Cass", 0.92813027 },
  { "Cassadee", 0.0 },
  { "Cassadi", 0.0 },
  { "Cassadie", 0.0 },
  { "Cassadra", 0.0 },
  { "Cassady", 0.076182134 },
  { "Cassand", 0.0 },
  { "Cassanda", 0.0 },
  { "Cassander", 0.0 },
  { "Cassandera", 0.0 },
  { "Cassandr", 0.0 },
  { "Cassandra", 0.00225581 },
  { "Cassandre", 0.0 },
  { "Cassandrea", 0.0 },
  { "Cassandria", 0.0 },
  { "Cassanora", 0.0 },
  { "Cassanova", 1.0 },
  { "Cassara", 0.0 },
  { "Cassarah", 0.0 },
  { "Cassaundra", 0.0 },
  { "Cassedy", 0.0 },
  { "Cassee", 0.0 },
  { "Cassel", 1.0 },
  { "Cassell", 1.0 },
  { "Cassen", 1.0 },
  { "Cassendra", 0.0 },
  { "Cassey", 0.08511807 },
  { "Cassi", 0.0 },
  { "Cassia", 0.0 },
  { "Cassiah", 0.0 },
  { "Cassian", 1.0 },
  { "Cassiana", 0.0 },
  { "Cassiday", 0.0 },
  { "Cassidee", 0.0 },
  { "Cassidey", 0.0 },
  { "Cassidi", 0.0 },
  { "Cassidie", 0.0 },
  { "Cassidy", 0.056251634 },
  { "Cassie", 0.008512703 },
  { "Cassiel", 0.8079096 },
  { "Cassietta", 0.0 },
  { "Cassin", 1.0 },
  { "Cassina", 0.0 },
  { "Cassiopeia", 0.0 },
  { "Cassiopia", 0.0 },
  { "Cassity", 0.0 },
  { "Cassius", 1.0 },
  { "Casslyn", 0.0 },
  { "Casson", 1.0 },
  { "Cassondr", 0.0 },
  { "Cassondra", 0.0 },
  { "Cassonia", 0.0 },
  { "Cassundra", 0.0 },
  { "Cassy", 0.0 },
  { "Cassye", 0.0 },
  { "Casta", 0.0 },
  { "Castella", 0.0 },
  { "Casten", 1.0 },
  { "Caster", 1.0 },
  { "Castiel", 0.98359436 },
  { "Castin", 1.0 },
  { "Castina", 0.0 },
  { "Castle", 1.0 },
  { "Casto", 1.0 },
  { "Caston", 1.0 },
  { "Castor", 1.0 },
  { "Castoria", 0.0 },
  { "Castro", 1.0 },
  { "Castulo", 1.0 },
  { "Castyn", 1.0 },
  { "Casundra", 0.0 },
  { "Caswell", 1.0 },
  { "Casy", 0.58653843 },
  { "Casyn", 0.9100346 },
  { "Catalaya", 0.0 },
  { "Catalayah", 0.0 },
  { "Cataldo", 1.0 },
  { "Catalea", 0.0 },
  { "Cataleah", 0.0 },
  { "Cataleena", 0.0 },
  { "Cataleia", 0.0 },
  { "Catalena", 0.0 },
  { "Cataleya", 0.0 },
  { "Cataleyah", 0.0 },
  { "Catalia", 0.0 },
  { "Catalin", 0.0 },
  { "Catalina", 0.00041395624 },
  { "Catalino", 1.0 },
  { "Cataliya", 0.0 },
  { "Cataliyah", 0.0 },
  { "Catalya", 0.0 },
  { "Catalyna", 0.0 },
  { "Catana", 0.0 },
  { "Catara", 0.0 },
  { "Catarena", 0.0 },
  { "Catarina", 0.0 },
  { "Catarino", 1.0 },
  { "Catasha", 0.0 },
  { "Catch", 1.0 },
  { "Catcher", 1.0 },
  { "Cate", 0.0 },
  { "Cateena", 0.0 },
  { "Catelaya", 0.0 },
  { "Cateleya", 0.0 },
  { "Catelin", 0.0 },
  { "Catello", 1.0 },
  { "Catelyn", 0.0 },
  { "Catelynn", 0.0 },
  { "Catelynne", 0.0 },
  { "Catena", 0.0 },
  { "Catenia", 0.0 },
  { "Catera", 0.0 },
  { "Cateria", 0.0 },
  { "Caterin", 0.0 },
  { "Caterina", 0.0 },
  { "Caterine", 0.0 },
  { "Caternia", 0.0 },
  { "Caterra", 0.0 },
  { "Catessa", 0.0 },
  { "Catey", 0.0 },
  { "Cath", 0.0 },
  { "Catha", 0.0 },
  { "Cathal", 1.0 },
  { "Cathaleen", 0.0 },
  { "Cathalene", 0.0 },
  { "Cathaleya", 0.0 },
  { "Cathalia", 0.0 },
  { "Cathalina", 0.0 },
  { "Cathaline", 0.0 },
  { "Cathan", 1.0 },
  { "Catharina", 0.0 },
  { "Catharine", 0.0 },
  { "Cathay", 0.0 },
  { "Cathe", 0.0 },
  { "Cathee", 0.0 },
  { "Catheleen", 0.0 },
  { "Cathelene", 0.0 },
  { "Catheline", 0.0 },
  { "Cather", 0.0 },
  { "Catheren", 0.0 },
  { "Catherene", 0.0 },
  { "Catherin", 0.0 },
  { "Catherina", 0.0 },
  { "Catherine", 0.0027659223 },
  { "Catherline", 0.0 },
  { "Cathern", 0.0 },
  { "Catherne", 0.0 },
  { "Catheryn", 0.0 },
  { "Catheryne", 0.0 },
  { "Cathey", 0.0 },
  { "Cathi", 0.0 },
  { "Cathia", 0.0 },
  { "Cathie", 0.0 },
  { "Cathirine", 0.0 },
  { "Cathlean", 0.0 },
  { "Cathleen", 0.00033525546 },
  { "Cathlena", 0.0 },
  { "Cathlene", 0.0 },
  { "Cathlin", 0.0 },
  { "Cathlina", 0.0 },
  { "Cathline", 0.0 },
  { "Cathlyn", 0.0 },
  { "Cathreen", 0.0 },
  { "Cathren", 0.0 },
  { "Cathrin", 0.0 },
  { "Cathrina", 0.0 },
  { "Cathrine", 0.0 },
  { "Cathryn", 0.0 },
  { "Cathryne", 0.0 },
  { "Cathrynn", 0.0 },
  { "Cathy", 0.0020128044 },
  { "Cathyann", 0.0 },
  { "Cathye", 0.0 },
  { "Cathyjo", 0.0 },
  { "Cathylee", 0.0 },
  { "Cathyleen", 0.0 },
  { "Cathyrn", 0.0 },
  { "Cati", 0.0 },
  { "Catia", 0.0 },
  { "Catiana", 0.0 },
  { "Catie", 0.0 },
  { "Catilaya", 0.0 },
  { "Catileya", 0.0 },
  { "Catilin", 0.0 },
  { "Catilyn", 0.0 },
  { "Catina", 0.0019111324 },
  { "Catine", 0.0 },
  { "Catinia", 0.0 },
  { "Catinna", 0.0 },
  { "Catiria", 0.0 },
  { "Catisha", 0.0 },
  { "Catlin", 0.29713523 },
  { "Catlyn", 0.0 },
  { "Catlynn", 0.0 },
  { "Cato", 1.0 },
  { "Caton", 0.7583333 },
  { "Catonya", 0.0 },
  { "Catori", 0.0 },
  { "Catoria", 0.0 },
  { "Catosha", 0.0 },
  { "Catoya", 0.0 },
  { "Catrease", 0.0 },
  { "Catrece", 0.0 },
  { "Catreena", 0.0 },
  { "Catrell", 1.0 },
  { "Catrena", 0.0 },
  { "Catrenia", 0.0 },
  { "Catrese", 0.0 },
  { "Catressa", 0.0 },
  { "Catria", 0.0 },
  { "Catriana", 0.0 },
  { "Catrice", 0.0 },
  { "Catricia", 0.0 },
  { "Catrin", 0.0 },
  { "Catrina", 0.0 },
  { "Catrinia", 0.0 },
  { "Catrinna", 0.0 },
  { "Catriona", 0.0 },
  { "Catrisha", 0.0 },
  { "Cattalaya", 0.0 },
  { "Cattaleya", 0.0 },
  { "Cattibrie", 0.0 },
  { "Cattie", 0.0 },
  { "Cattina", 0.0 },
  { "Cattleya", 0.0 },
  { "Caty", 0.0 },
  { "Catya", 0.0 },
  { "Caua", 1.0 },
  { "Cauan", 1.0 },
  { "Caudie", 0.0 },
  { "Caulder", 1.0 },
  { "Caulen", 1.0 },
  { "Caulin", 1.0 },
  { "Caullin", 1.0 },
  { "Cauy", 1.0 },
  { "Cavalli", 1.0 },
  { "Cavan", 1.0 },
  { "Cavanaugh", 1.0 },
  { "Cavani", 1.0 },
  { "Cavari", 1.0 },
  { "Cavell", 1.0 },
  { "Caven", 1.0 },
  { "Cavett", 1.0 },
  { "Cavin", 1.0 },
  { "Cavion", 1.0 },
  { "Cavon", 1.0 },
  { "Cavonte", 1.0 },
  { "Cawanda", 0.0 },
  { "Cawanna", 0.0 },
  { "Cay", 0.043650795 },
  { "Caya", 0.0 },
  { "Cayce", 0.31156716 },
  { "Caycee", 0.0 },
  { "Caycen", 1.0 },
  { "Cayci", 0.0 },
  { "Caycie", 0.0 },
  { "Cayd", 1.0 },
  { "Caydan", 0.97863245 },
  { "Caydance", 0.0 },
  { "Cayde", 1.0 },
  { "Caydee", 0.0 },
  { "Cayden", 0.9490283 },
  { "Caydence", 0.057069574 },
  { "Caydenn", 1.0 },
  { "Caydin", 0.9128329 },
  { "Caydince", 0.0 },
  { "Caydn", 1.0 },
  { "Caydon", 1.0 },
  { "Caydyn", 1.0 },
  { "Caye", 0.0 },
  { "Cayenne", 0.06185567 },
  { "Cayetana", 0.0 },
  { "Cayetano", 1.0 },
  { "Cayl", 1.0 },
  { "Cayla", 0.0 },
  { "Caylah", 0.0 },
  { "Caylan", 0.38987342 },
  { "Caylani", 0.0 },
  { "Cayle", 0.6527415 },
  { "Caylea", 0.0 },
  { "Cayleb", 1.0 },
  { "Caylee", 0.0 },
  { "Cayleeann", 0.0 },
  { "Cayleen", 0.0 },
  { "Caylei", 0.0 },
  { "Cayleigh", 0.0 },
  { "Caylem", 1.0 },
  { "Caylen", 0.3345112 },
  { "Caylene", 0.0 },
  { "Cayler", 1.0 },
  { "Cayley", 0.0 },
  { "Cayli", 0.0 },
  { "Cayliana", 0.0 },
  { "Caylib", 1.0 },
  { "Caylie", 0.0 },
  { "Caylin", 0.03667791 },
  { "Caylinn", 0.0 },
  { "Caylob", 1.0 },
  { "Caylon", 0.9506173 },
  { "Caylor", 0.53409094 },
  { "Caylub", 1.0 },
  { "Caylum", 1.0 },
  { "Caylyn", 0.0 },
  { "Caylynn", 0.0 },
  { "Cayman", 0.7641509 },
  { "Caymen", 0.93454546 },
  { "Caymon", 1.0 },
  { "Caymus", 1.0 },
  { "Cayn", 1.0 },
  { "Caynan", 1.0 },
  { "Cayne", 1.0 },
  { "Caynen", 1.0 },
  { "Cayo", 1.0 },
  { "Cayra", 0.0 },
  { "Cayse", 0.6875 },
  { "Caysee", 0.0 },
  { "Caysen", 0.9678335 },
  { "Caysey", 1.0 },
  { "Caysi", 0.0 },
  { "Caysie", 0.0 },
  { "Caysin", 1.0 },
  { "Cayslee", 0.0 },
  { "Cayson", 0.9895022 },
  { "Caytie", 0.0 },
  { "Caytlen", 0.0 },
  { "Caytlin", 0.0 },
  { "Caytlyn", 0.0 },
  { "Caytlynn", 0.0 },
  { "Cayton", 0.8043478 },
  { "Cayvion", 1.0 },
  { "Caz", 1.0 },
  { "Cazandra", 0.0 },
  { "Cazden", 1.0 },
  { "Caziah", 0.68421054 },
  { "Caziyah", 0.0 },
  { "Cazzie", 1.0 },
  { "Cchristopher", 1.0 },
  { "Ceaira", 0.0 },
  { "Ceairra", 0.0 },
  { "Cealie", 0.0 },
  { "Cean", 1.0 },
  { "Ceana", 0.0 },
  { "Ceandra", 0.0 },
  { "Ceandre", 1.0 },
  { "Ceann", 0.0 },
  { "Ceanna", 0.0 },
  { "Ceara", 0.0 },
  { "Cearah", 0.0 },
  { "Cearia", 0.0 },
  { "Cearra", 0.0 },
  { "Ceasar", 1.0 },
  { "Ceasare", 1.0 },
  { "Ceaser", 1.0 },
  { "Ceasia", 0.0 },
  { "Ceason", 0.0 },
  { "Ceazar", 1.0 },
  { "Ceazia", 0.0 },
  { "Cebastian", 1.0 },
  { "Cebell", 0.0 },
  { "Cebert", 1.0 },
  { "Cebria", 0.0 },
  { "Cebrina", 0.0 },
  { "Cebron", 1.0 },
  { "Cece", 0.0 },
  { "Ceceila", 0.0 },
  { "Ceceilia", 0.0 },
  { "Cecel", 0.0 },
  { "Cecele", 0.0 },
  { "Cecelia", 0.00019825179 },
  { "Cecelie", 0.0 },
  { "Cecelio", 1.0 },
  { "Cecellia", 0.0 },
  { "Cecely", 0.0 },
  { "Ceci", 0.0 },
  { "Cecia", 0.0 },
  { "Cecial", 1.0 },
  { "Cecil", 0.9324071 },
  { "Cecila", 0.0 },
  { "Cecile", 0.009358157 },
  { "Cecilee", 0.0 },
  { "Ceciley", 0.0 },
  { "Cecili", 0.0 },
  { "Cecilia", 0.0018956838 },
  { "Cecilie", 0.0 },
  { "Cecilio", 1.0 },
  { "Cecilla", 0.0 },
  { "Cecille", 0.0 },
  { "Cecillia", 0.0 },
  { "Cecilly", 0.0 },
  { "Cecily", 0.0 },
  { "Cecilya", 0.0 },
  { "Ceclia", 0.0 },
  { "Cecy", 0.0 },
  { "Cecyle", 0.0 },
  { "Cecylia", 0.0 },
  { "Cedar", 0.5845552 },
  { "Cedarius", 1.0 },
  { "Ceddrick", 1.0 },
  { "Cedell", 0.9230769 },
  { "Cedella", 0.0 },
  { "Cedeno", 1.0 },
  { "Cederia", 0.0 },
  { "Cederic", 1.0 },
  { "Cederick", 1.0 },
  { "Cedra", 0.0 },
  { "Cedrea", 0.0 },
  { "Cedria", 0.0 },
  { "Cedriana", 0.0 },
  { "Cedrianna", 0.0 },
  { "Cedric", 0.9931178 },
  { "Cedrica", 0.0 },
  { "Cedrice", 1.0 },
  { "Cedrick", 0.99832636 },
  { "Cedricka", 0.0 },
  { "Cedrie", 1.0 },
  { "Cedrik", 1.0 },
  { "Cedrika", 0.0 },
  { "Cedrina", 0.0 },
  { "Cee", 0.0 },
  { "Ceeanna", 0.0 },
  { "Ceejay", 1.0 },
  { "Ceferino", 1.0 },
  { "Ceianna", 0.0 },
  { "Ceiara", 0.0 },
  { "Ceil", 0.0 },
  { "Ceila", 0.0 },
  { "Ceili", 0.0 },
  { "Ceilia", 0.0 },
  { "Ceilidh", 0.0 },
  { "Ceinna", 0.0 },
  { "Ceion", 1.0 },
  { "Ceionna", 0.0 },
  { "Ceira", 0.0 },
  { "Ceirra", 0.0 },
  { "Cejay", 1.0 },
  { "Ceji", 0.0 },
  { "Cela", 0.0 },
  { "Celaya", 0.0 },
  { "Cele", 0.0 },
  { "Celebrity", 0.0 },
  { "Celecia", 0.0 },
  { "Celedonia", 0.0 },
  { "Celedonio", 1.0 },
  { "Celeena", 0.0 },
  { "Celena", 0.0 },
  { "Celene", 0.0 },
  { "Celenia", 0.0 },
  { "Celenne", 0.0 },
  { "Celeny", 0.0 },
  { "Celerina", 0.0 },
  { "Celerino", 1.0 },
  { "Celes", 0.0 },
  { "Celese", 0.0 },
  { "Celess", 0.0 },
  { "Celesse", 0.0 },
  { "Celest", 0.0 },
  { "Celesta", 0.0 },
  { "Celeste", 0.0010161696 },
  { "Celester", 0.4488778 },
  { "Celestia", 0.0 },
  { "Celestial", 0.0 },
  { "Celestie", 0.0 },
  { "Celestin", 1.0 },
  { "Celestina", 0.0 },
  { "Celestine", 0.02372916 },
  { "Celestino", 1.0 },
  { "Celestte", 0.0 },
  { "Celeta", 0.0 },
  { "Celetha", 0.0 },
  { "Celia", 0.0013783597 },
  { "Celiana", 0.0 },
  { "Celica", 0.0 },
  { "Celicia", 0.0 },
  { "Celida", 0.0 },
  { "Celie", 0.0 },
  { "Celimar", 0.0 },
  { "Celin", 0.0 },
  { "Celina", 0.0004225352 },
  { "Celinda", 0.0 },
  { "Celine", 0.0 },
  { "Celines", 0.0 },
  { "Celinez", 0.0 },
  { "Celinna", 0.0 },
  { "Celio", 1.0 },
  { "Celis", 0.0 },
  { "Celisa", 0.0 },
  { "Celise", 0.0 },
  { "Celisha", 0.0 },
  { "Celissa", 0.0 },
  { "Celisse", 0.0 },
  { "Celita", 0.0 },
  { "Celiyah", 0.0 },
  { "Cella", 0.0 },
  { "Celleste", 0.0 },
  { "Cellestine", 0.0 },
  { "Cellia", 0.0 },
  { "Cellie", 0.0 },
  { "Cellina", 0.0 },
  { "Cellus", 1.0 },
  { "Celsa", 0.0 },
  { "Celsea", 0.0 },
  { "Celsey", 0.0 },
  { "Celsi", 0.0 },
  { "Celsie", 0.0 },
  { "Celso", 1.0 },
  { "Celton", 1.0 },
  { "Celvin", 1.0 },
  { "Cely", 0.0 },
  { "Celyn", 0.0 },
  { "Celyna", 0.0 },
  { "Celyne", 0.0 },
  { "Celynn", 0.0 },
  { "Cem", 1.0 },
  { "Cemal", 1.0 },
  { "Cemiyah", 0.0 },
  { "Cemone", 0.0 },
  { "Cemre", 0.0 },
  { "Cemya", 0.0 },
  { "Cena", 0.013986014 },
  { "Cenae", 0.0 },
  { "Cenai", 0.0 },
  { "Cenaida", 0.0 },
  { "Cendi", 0.0 },
  { "Cendra", 0.0 },
  { "Cendy", 0.0 },
  { "Cenedra", 0.0 },
  { "Cengiz", 1.0 },
  { "Cenia", 0.0 },
  { "Cenie", 0.0 },
  { "Cenith", 0.0 },
  { "Ceniya", 0.0 },
  { "Ceniyah", 0.0 },
  { "Cennedy", 0.0 },
  { "Cennie", 0.0 },
  { "Centeria", 0.0 },
  { "Centhia", 0.0 },
  { "Central", 1.0 },
  { "Centrell", 1.0 },
  { "Centwane", 1.0 },
  { "Cenya", 0.0 },
  { "Ceola", 0.009610984 },
  { "Ceolia", 0.0 },
  { "Ceon", 1.0 },
  { "Ceona", 0.0 },
  { "Ceonna", 0.0 },
  { "Cepeda", 1.0 },
  { "Cephas", 1.0 },
  { "Cephus", 1.0 },
  { "Cequan", 1.0 },
  { "Cera", 0.0 },
  { "Cerah", 0.0 },
  { "Cerdic", 1.0 },
  { "Cerelia", 0.0 },
  { "Ceren", 0.0 },
  { "Cerena", 0.0 },
  { "Cereniti", 0.0 },
  { "Cerenity", 0.0 },
  { "Ceres", 0.0 },
  { "Ceresa", 0.0 },
  { "Cerese", 0.0 },
  { "Ceretha", 0.0 },
  { "Cergio", 1.0 },
  { "Ceri", 0.0 },
  { "Ceria", 0.0 },
  { "Ceriah", 0.0 },
  { "Ceridwen", 0.0 },
  { "Cerina", 0.0 },
  { "Cerinity", 0.0 },
  { "Cerisa", 0.0 },
  { "Cerise", 0.0 },
  { "Cerissa", 0.0 },
  { "Cerita", 0.0 },
  { "Ceriyah", 0.0 },
  { "Cerjio", 1.0 },
  { "Ceron", 1.0 },
  { "Cerone", 1.0 },
  { "Cerra", 0.0 },
  { "Cerri", 0.0 },
  { "Cerria", 0.0 },
  { "Cerridwen", 0.0 },
  { "Cerrie", 0.0 },
  { "Cerrisa", 0.0 },
  { "Cerrissa", 0.0 },
  { "Cerrone", 1.0 },
  { "Cervando", 1.0 },
  { "Cervante", 1.0 },
  { "Cervantes", 1.0 },
  { "Cerys", 0.0 },
  { "Cesalie", 0.0 },
  { "Cesar", 0.995546 },
  { "Cesare", 1.0 },
  { "Cesareo", 1.0 },
  { "Cesaria", 0.0 },
  { "Cesarina", 0.0 },
  { "Cesario", 1.0 },
  { "Cesc", 1.0 },
  { "Cescily", 0.0 },
  { "Cesear", 1.0 },
  { "Cesia", 0.0 },
  { "Cesiah", 0.0 },
  { "Cesidio", 1.0 },
  { "Cesilee", 0.0 },
  { "Cesiley", 0.0 },
  { "Cesilia", 0.0 },
  { "Cesilie", 0.0 },
  { "Cesily", 0.0 },
  { "Cesira", 0.0 },
  { "Cesley", 0.0 },
  { "Ceslie", 0.0 },
  { "Cessily", 0.0 },
  { "Cessna", 0.0 },
  { "Cetera", 0.0 },
  { "Ceth", 1.0 },
  { "Cetric", 1.0 },
  { "Cevanna", 0.0 },
  { "Cevera", 0.0 },
  { "Cevilla", 0.0 },
  { "Cevin", 1.0 },
  { "Cevion", 1.0 },
  { "Cevon", 1.0 },
  { "Ceyana", 0.0 },
  { "Ceyara", 0.0 },
  { "Ceyda", 0.0 },
  { "Ceylan", 0.0 },
  { "Ceylin", 0.0 },
  { "Ceylon", 1.0 },
  { "Ceyonce", 0.0 },
  { "Ceyonna", 0.0 },
  { "Cezanne", 0.0 },
  { "Cezar", 1.0 },
  { "Cha", 0.8724138 },
  { "Chaarvi", 0.0 },
  { "Chaaya", 0.0 },
  { "Chabeli", 0.0 },
  { "Chabelli", 0.0 },
  { "Chabely", 0.0 },
  { "Chablis", 0.0 },
  { "Chabria", 0.0 },
  { "Chace", 0.9731082 },
  { "Chachi", 1.0 },
  { "Chad", 0.9956792 },
  { "Chada", 0.0 },
  { "Chadae", 0.0 },
  { "Chadarius", 1.0 },
  { "Chadd", 1.0 },
  { "Chaddrick", 1.0 },
  { "Chade", 0.11612903 },
  { "Chaden", 1.0 },
  { "Chaderick", 1.0 },
  { "Chadi", 1.0 },
  { "Chadijah", 0.0 },
  { "Chadlee", 1.0 },
  { "Chadley", 1.0 },
  { "Chadly", 1.0 },
  { "Chadman", 1.0 },
  { "Chadney", 0.8958333 },
  { "Chadric", 1.0 },
  { "Chadrick", 1.0 },
  { "Chadron", 1.0 },
  { "Chadsity", 0.0 },
  { "Chadwell", 1.0 },
  { "Chadwic", 1.0 },
  { "Chadwich", 1.0 },
  { "Chadwick", 0.99830836 },
  { "Chadwin", 1.0 },
  { "Chadwyck", 1.0 },
  { "Chadyeane", 0.0 },
  { "Chae", 0.45820433 },
  { "Chaeden", 1.0 },
  { "Chael", 1.0 },
  { "Chaela", 0.0 },
  { "Chaeli", 0.0 },
  { "Chaely", 0.0 },
  { "Chaelyn", 0.0 },
  { "Chaelynn", 0.0 },
  { "Chaena", 0.0 },
  { "Chaffee", 0.5 },
  { "Chai", 0.6334012 },
  { "Chaia", 0.0 },
  { "Chaiden", 1.0 },
  { "Chaila", 0.0 },
  { "Chailee", 0.0 },
  { "Chailey", 0.0 },
  { "Chaille", 0.0 },
  { "Chailyn", 0.0 },
  { "Chailynn", 0.0 },
  { "Chaim", 0.99929774 },
  { "Chain", 1.0 },
  { "Chairity", 0.0 },
  { "Chairty", 0.0 },
  { "Chais", 0.9632353 },
  { "Chaise", 0.7471439 },
  { "Chaislyn", 0.0 },
  { "Chaison", 1.0 },
  { "Chaisson", 1.0 },
  { "Chaisty", 0.0 },
  { "Chaitan", 1.0 },
  { "Chaitanya", 1.0 },
  { "Chaithra", 0.0 },
  { "Chaitra", 0.0 },
  { "Chaiya", 0.0 },
  { "Chaka", 0.237534 },
  { "Chakakhan", 0.0 },
  { "Chakara", 0.0 },
  { "Chakayla", 0.0 },
  { "Chakela", 0.0 },
  { "Chakelia", 0.0 },
  { "Chakera", 0.0 },
  { "Chakeria", 0.0 },
  { "Chaketa", 0.0 },
  { "Chakevia", 0.0 },
  { "Chakeya", 0.0 },
  { "Chakia", 0.0 },
  { "Chakina", 0.0 },
  { "Chakira", 0.0 },
  { "Chakita", 0.0 },
  { "Chakiya", 0.0 },
  { "Chakka", 0.0 },
  { "Chakong", 1.0 },
  { "Chakotay", 1.0 },
  { "Chakya", 0.0 },
  { "Chakyra", 0.0 },
  { "Chala", 0.0 },
  { "Chalan", 0.0 },
  { "Chalanda", 0.0 },
  { "Chalandra", 0.0 },
  { "Chale", 1.0 },
  { "Chalea", 0.0 },
  { "Chalee", 0.0 },
  { "Chalei", 0.0 },
  { "Chalen", 1.0 },
  { "Chalena", 0.0 },
  { "Chalene", 0.0 },
  { "Chales", 1.0 },
  { "Chalese", 0.0 },
  { "Chalet", 0.0 },
  { "Chalette", 0.0 },
  { "Chaley", 0.0 },
  { "Chali", 0.0 },
  { "Chalia", 0.0 },
  { "Chalice", 0.0 },
  { "Chalie", 0.0 },
  { "Chalina", 0.0 },
  { "Chalino", 1.0 },
  { "Chalis", 0.0 },
  { "Chalisa", 0.0 },
  { "Chalise", 0.0 },
  { "Chalissa", 0.0 },
  { "Chalisse", 0.0 },
  { "Chalita", 0.0 },
  { "Chaliyah", 0.0 },
  { "Challen", 0.93421054 },
  { "Challie", 0.5 },
  { "Challis", 0.2421875 },
  { "Challise", 0.0 },
  { "Chalmas", 1.0 },
  { "Chalmer", 1.0 },
  { "Chalmers", 1.0 },
  { "Chalmus", 1.0 },
  { "Chalon", 0.14193548 },
  { "Chalonda", 0.0 },
  { "Chalres", 1.0 },
  { "Chalsea", 0.0 },
  { "Chalsey", 0.0 },
  { "Chalsie", 0.0 },
  { "Chaly", 1.0 },
  { "Chalyce", 0.0 },
  { "Chalyn", 0.0 },
  { "Chalynn", 0.0 },
  { "Chalyse", 0.0 },
  { "Chama", 0.0 },
  { "Chamaine", 0.0 },
  { "Chamar", 1.0 },
  { "Chamara", 0.0 },
  { "Chamari", 0.0 },
  { "Chamarion", 1.0 },
  { "Chamarra", 0.0 },
  { "Chamaya", 0.0 },
  { "Chamber", 1.0 },
  { "Chamberlain", 0.9433962 },
  { "Chamberlin", 1.0 },
  { "Chamberlyn", 0.0 },
  { "Chambers", 1.0 },
  { "Chambre", 0.0 },
  { "Chambria", 0.0 },
  { "Chameka", 0.0 },
  { "Chamel", 0.5 },
  { "Chamelle", 0.0 },
  { "Chamere", 0.0 },
  { "Chamia", 0.0 },
  { "Chamika", 0.0 },
  { "Chamille", 0.0 },
  { "Chamique", 0.0 },
  { "Chamira", 0.0 },
  { "Chamisa", 0.0 },
  { "Chamise", 0.0 },
  { "Chamiya", 0.0 },
  { "Chamiyah", 0.0 },
  { "Champ", 1.0 },
  { "Champagne", 0.0 },
  { "Champaign", 0.0 },
  { "Champaigne", 0.0 },
  { "Champaine", 0.0 },
  { "Champale", 0.0 },
  { "Champane", 0.0 },
  { "Champayne", 0.0 },
  { "Champion", 1.0 },
  { "Chamroeun", 1.0 },
  { "Chamya", 0.0 },
  { "Chan", 0.8219983 },
  { "Chana", 0.0 },
  { "Chanae", 0.0 },
  { "Chanah", 0.0 },
  { "Chanai", 0.0 },
  { "Chanan", 1.0 },
  { "Chananya", 1.0 },
  { "Chanavia", 0.0 },
  { "Chanay", 0.0 },
  { "Chanaya", 0.0 },
  { "Chanc", 1.0 },
  { "Chance", 0.97541356 },
  { "Chancee", 0.0 },
  { "Chanceler", 1.0 },
  { "Chanceller", 1.0 },
  { "Chancellor", 1.0 },
  { "Chancelor", 1.0 },
  { "Chances", 0.0 },
  { "Chanceton", 1.0 },
  { "Chancey", 0.71343285 },
  { "Chanci", 0.0 },
  { "Chancie", 0.123853214 },
  { "Chancler", 1.0 },
  { "Chancy", 0.7546948 },
  { "Chanda", 0.0026251026 },
  { "Chandace", 0.0 },
  { "Chandal", 0.0 },
  { "Chandale", 0.0 },
  { "Chandan", 1.0 },
  { "Chandani", 0.0 },
  { "Chandar", 0.52 },
  { "Chandara", 0.87234044 },
  { "Chande", 1.0 },
  { "Chandee", 0.0 },
  { "Chandel", 0.0 },
  { "Chandell", 0.0 },
  { "Chandelle", 0.0 },
  { "Chanden", 1.0 },
  { "Chander", 1.0 },
  { "Chandi", 0.0 },
  { "Chandice", 0.0 },
  { "Chandini", 0.0 },
  { "Chandlar", 0.36363637 },
  { "Chandler", 0.8049035 },
  { "Chandley", 0.0 },
  { "Chandlor", 1.0 },
  { "Chandlyr", 0.0 },
  { "Chandni", 0.0 },
  { "Chandon", 1.0 },
  { "Chandra", 0.003814147 },
  { "Chandre", 0.0 },
  { "Chandrea", 0.0 },
  { "Chandrell", 0.0 },
  { "Chandria", 0.0 },
  { "Chandrika", 0.0 },
  { "Chandris", 0.0 },
  { "Chandy", 0.0 },
  { "Chane", 0.7117117 },
  { "Chanea", 0.0 },
  { "Chanee", 0.0 },
  { "Chaneice", 0.0 },
  { "Chaneka", 0.0 },
  { "Chanel", 0.005156828 },
  { "Chanele", 0.0 },
  { "Chanell", 0.0 },
  { "Chanella", 0.0 },
  { "Chanelle", 0.0 },
  { "Chanelly", 0.0 },
  { "Chanequa", 0.0 },
  { "Chanese", 0.0 },
  { "Chanet", 0.0 },
  { "Chaneta", 0.0 },
  { "Chanetta", 0.0 },
  { "Chanette", 0.0 },
  { "Chaney", 0.24680851 },
  { "Chaneyra", 0.0 },
  { "Chang", 0.98397434 },
  { "Changa", 1.0 },
  { "Chanh", 1.0 },
  { "Chani", 0.0 },
  { "Chania", 0.0 },
  { "Chaniah", 0.0 },
  { "Chanice", 0.0 },
  { "Chanie", 0.0 },
  { "Chaniece", 0.0 },
  { "Chaniel", 0.0 },
  { "Chanielle", 0.0 },
  { "Chanika", 0.0 },
  { "Chanille", 0.0 },
  { "Chanin", 0.035256412 },
  { "Chanina", 1.0 },
  { "Chaning", 0.4814815 },
  { "Chaniqua", 0.0 },
  { "Chanique", 0.0 },
  { "Chanise", 0.0 },
  { "Chanita", 0.0 },
  { "Chanity", 0.0 },
  { "Chaniya", 0.0 },
  { "Chaniyah", 0.0 },
  { "Chanlee", 0.0 },
  { "Chanler", 0.7204861 },
  { "Chanley", 0.0 },
  { "Channa", 0.0057670125 },
  { "Channah", 0.0 },
  { "Channan", 0.0 },
  { "Channce", 1.0 },
  { "Channel", 0.0 },
  { "Channell", 0.0 },
  { "Channelle", 0.0 },
  { "Channen", 1.0 },
  { "Channer", 1.0 },
  { "Channie", 0.0 },
  { "Channin", 0.5714286 },
  { "Channing", 0.6515233 },
  { "Channon", 0.3384755 },
  { "Channy", 0.3125 },
  { "Chano", 1.0 },
  { "Chanoa", 0.0 },
  { "Chanoch", 1.0 },
  { "Chanon", 0.29545453 },
  { "Chans", 1.0 },
  { "Chanse", 0.9848197 },
  { "Chansler", 1.0 },
  { "Chanson", 1.0 },
  { "Chanston", 1.0 },
  { "Chant", 1.0 },
  { "Chanta", 0.0 },
  { "Chantae", 0.0 },
  { "Chantai", 0.0 },
  { "Chantail", 0.0 },
  { "Chantal", 0.00084490044 },
  { "Chantale", 0.0 },
  { "Chantall", 0.0 },
  { "Chantalle", 0.0 },
  { "Chantasia", 0.0 },
  { "Chantavia", 0.0 },
  { "Chantay", 0.0 },
  { "Chantaye", 0.0 },
  { "Chante", 0.008297712 },
  { "Chantea", 0.0 },
  { "Chanteal", 0.0 },
  { "Chantee", 0.0 },
  { "Chantel", 0.0007440979 },
  { "Chantele", 0.0 },
  { "Chantell", 0.001435132 },
  { "Chantella", 0.0 },
  { "Chantelle", 0.0012277471 },
  { "Chanteria", 0.0 },
  { "Chantey", 0.0 },
  { "Chantez", 0.0 },
  { "Chantha", 0.46875 },
  { "Chanthy", 1.0 },
  { "Chanti", 0.0 },
  { "Chantia", 0.0 },
  { "Chantice", 0.0 },
  { "Chantiel", 0.0 },
  { "Chantil", 0.0 },
  { "Chantile", 0.0 },
  { "Chantill", 0.0 },
  { "Chantille", 0.0 },
  { "Chantilly", 0.0 },
  { "Chantily", 0.0 },
  { "Chantina", 0.0 },
  { "Chantis", 0.0 },
  { "Chantise", 0.0 },
  { "Chantle", 0.0 },
  { "Chantler", 1.0 },
  { "Chantol", 0.0 },
  { "Chanton", 1.0 },
  { "Chantoria", 0.0 },
  { "Chantra", 0.35185185 },
  { "Chantrea", 0.0 },
  { "Chantrel", 0.0 },
  { "Chantrell", 0.0 },
  { "Chantrelle", 0.0 },
  { "Chantrice", 0.0 },
  { "Chantry", 1.0 },
  { "Chanty", 0.0 },
  { "Chantz", 1.0 },
  { "Chany", 0.0 },
  { "Chanya", 0.0 },
  { "Chanyah", 0.0 },
  { "Chanyce", 0.0 },
  { "Chanyia", 0.0 },
  { "Chanz", 1.0 },
  { "Chanze", 1.0 },
  { "Chao", 0.8892308 },
  { "Chaos", 1.0 },
  { "Chap", 1.0 },
  { "Chapel", 0.26086956 },
  { "Chapelle", 0.0 },
  { "Chapin", 0.8381295 },
  { "Chaplin", 1.0 },
  { "Chapman", 1.0 },
  { "Chappell", 0.89361703 },
  { "Chaquan", 0.83870965 },
  { "Chaquana", 0.0 },
  { "Chaquanna", 0.0 },
  { "Chaqueta", 0.0 },
  { "Chaquetta", 0.0 },
  { "Chaquilla", 0.0 },
  { "Chaquille", 0.8148148 },
  { "Chaquita", 0.0 },
  { "Chaquitta", 0.0 },
  { "Char", 0.0 },
  { "Chara", 0.0 },
  { "Charade", 0.0 },
  { "Charae", 0.0 },
  { "Charaine", 0.0 },
  { "Charan", 1.0 },
  { "Charanda", 0.0 },
  { "Charay", 0.0 },
  { "Charbel", 1.0 },
  { "Chard", 1.0 },
  { "Charda", 0.0 },
  { "Chardae", 0.0 },
  { "Chardai", 0.0 },
  { "Chardanae", 0.0 },
  { "Chardanay", 0.0 },
  { "Chardarius", 1.0 },
  { "Chardasia", 0.0 },
  { "Charday", 0.0 },
  { "Charde", 0.0 },
  { "Chardee", 0.0 },
  { "Chardell", 0.0 },
  { "Chardey", 0.0 },
  { "Chardon", 0.0 },
  { "Chardonae", 0.0 },
  { "Chardonay", 0.0 },
  { "Chardonnae", 0.0 },
  { "Chardonnay", 0.0 },
  { "Chare", 0.0 },
  { "Charea", 0.0 },
  { "Charece", 0.0 },
  { "Charee", 0.0 },
  { "Chareen", 0.0 },
  { "Chareese", 0.0 },
  { "Charel", 0.0 },
  { "Charell", 0.0 },
  { "Charelle", 0.0 },
  { "Charels", 1.0 },
  { "Charemon", 0.0 },
  { "Charen", 0.0 },
  { "Charene", 0.0 },
  { "Chares", 1.0 },
  { "Charese", 0.0 },
  { "Charessa", 0.0 },
  { "Charesse", 0.0 },
  { "Charetta", 0.0 },
  { "Charette", 0.0 },
  { "Charger", 1.0 },
  { "Charhonda", 0.0 },
  { "Chari", 0.0 },
  { "Charia", 0.0 },
  { "Chariah", 0.0 },
  { "Charice", 0.0 },
  { "Charidy", 0.0 },
  { "Charie", 0.0 },
  { "Chariel", 0.0 },
  { "Charielle", 0.0 },
  { "Charies", 1.0 },
  { "Chariese", 0.0 },
  { "Charika", 0.0 },
  { "Charilyn", 0.0 },
  { "Charina", 0.0 },
  { "Charis", 0.012023001 },
  { "Charisa", 0.0 },
  { "Charise", 0.0 },
  { "Charish", 0.0 },
  { "Charisha", 0.0 },
  { "Charisma", 0.0 },
  { "Chariss", 0.0 },
  { "Charissa", 0.0 },
  { "Charisse", 0.0 },
  { "Charissma", 0.0 },
  { "Charistopher", 1.0 },
  { "Charita", 0.0 },
  { "Charitee", 0.0 },
  { "Charith", 1.0 },
  { "Charitha", 0.0 },
  { "Chariti", 0.0 },
  { "Charitie", 0.0 },
  { "Charito", 0.0 },
  { "Charity", 0.0011387859 },
  { "Chariya", 0.0 },
  { "Chariyah", 0.0 },
  { "Charizma", 0.0 },
  { "Charkita", 0.0 },
  { "Charl", 0.3529412 },
  { "Charla", 0.0 },
  { "Charlae", 0.0 },
  { "Charlain", 0.0 },
  { "Charlaine", 0.0 },
  { "Charlan", 0.0 },
  { "Charlana", 0.0 },
  { "Charlanda", 0.0 },
  { "Charlane", 0.0 },
  { "Charlann", 0.0 },
  { "Charlanne", 0.0 },
  { "Charlas", 1.0 },
  { "Charlay", 0.0 },
  { "Charlayne", 0.0 },
  { "Charlcie", 0.0 },
  { "Charlcy", 0.0 },
  { "Charle", 0.8764151 },
  { "Charlea", 0.0 },
  { "Charleah", 0.0 },
  { "Charlean", 0.0 },
  { "Charleane", 0.0 },
  { "Charlecia", 0.0 },
  { "Charlee", 0.021695111 },
  { "Charleeann", 0.0 },
  { "Charleen", 0.0 },
  { "Charleene", 0.0 },
  { "Charlei", 0.0 },
  { "Charleigh", 0.0 },
  { "Charlemagne", 0.72727275 },
  { "Charlen", 0.0 },
  { "Charlena", 0.0 },
  { "Charlene", 0.0028784873 },
  { "Charles", 0.99481726 },
  { "Charlesa", 0.0 },
  { "Charlesanthony", 1.0 },
  { "Charlese", 0.0 },
  { "Charlesedward", 1.0 },
  { "Charlesetta", 0.0 },
  { "Charlesha", 0.0 },
  { "Charlesia", 0.0 },
  { "Charlesjoseph", 1.0 },
  { "Charleson", 1.0 },
  { "Charlesten", 1.0 },
  { "Charlestine", 0.0 },
  { "Charleston", 0.74990845 },
  { "Charlestyn", 0.0 },
  { "Charleszetta", 0.0 },
  { "Charlet", 0.0 },
  { "Charleta", 0.0 },
  { "Charletha", 0.0 },
  { "Charleton", 1.0 },
  { "Charlett", 0.0 },
  { "Charletta", 0.0 },
  { "Charlette", 0.0 },
  { "Charley", 0.7672508 },
  { "Charli", 0.020436201 },
  { "Charlia", 0.0 },
  { "Charliann", 0.0 },
  { "Charlianne", 0.0 },
  { "Charlice", 0.0 },
  { "Charlicia", 0.0 },
  { "Charlie", 0.88713044 },
  { "Charlieann", 0.0 },
  { "Charliee", 0.0 },
  { "Charliegh", 0.0 },
  { "Charliemae", 0.0 },
  { "Charliene", 0.0 },
  { "Charlierae", 0.0 },
  { "Charlierose", 0.0 },
  { "Charlies", 1.0 },
  { "Charliese", 0.0 },
  { "Charlii", 0.0 },
  { "Charlin", 0.0 },
  { "Charlina", 0.0 },
  { "Charlinda", 0.0 },
  { "Charline", 0.0 },
  { "Charlirae", 0.0 },
  { "Charlis", 0.82258064 },
  { "Charlisa", 0.0 },
  { "Charlise", 0.0 },
  { "Charlisha", 0.0 },
  { "Charlisia", 0.0 },
  { "Charliss", 0.0 },
  { "Charlissa", 0.0 },
  { "Charlisse", 0.0 },
  { "Charlita", 0.0 },
  { "Charliyah", 0.0 },
  { "Charlize", 0.0 },
  { "Charlla", 0.0 },
  { "Charlles", 1.0 },
  { "Charloette", 0.0 },
  { "Charlon", 0.0 },
  { "Charlona", 0.0 },
  { "Charlonda", 0.0 },
  { "Charlot", 0.0 },
  { "Charlote", 0.0 },
  { "Charlott", 0.0 },
  { "Charlotta", 0.0 },
  { "Charlotte", 0.0020085392 },
  { "Charlotteann", 0.0 },
  { "Charlottee", 0.0 },
  { "Charlottemarie", 0.0 },
  { "Charlotterose", 0.0 },
  { "Charlottie", 0.0 },
  { "Charlottle", 0.0 },
  { "Charls", 1.0 },
  { "Charlsa", 0.0 },
  { "Charlse", 1.0 },
  { "Charlsea", 0.0 },
  { "Charlsey", 0.0 },
  { "Charlsie", 0.0 },
  { "Charlston", 1.0 },
  { "Charlton", 0.9985337 },
  { "Charly", 0.36056533 },
  { "Charlyann", 0.0 },
  { "Charlye", 0.0 },
  { "Charlyee", 0.0 },
  { "Charlyn", 0.0 },
  { "Charlynda", 0.0 },
  { "Charlyne", 0.0 },
  { "Charlynn", 0.0 },
  { "Charlynne", 0.0 },
  { "Charlyse", 0.0 },
  { "Charlytte", 0.0 },
  { "Charlyze", 0.0 },
  { "Charlz", 1.0 },
  { "Charm", 0.0 },
  { "Charma", 0.0 },
  { "Charmae", 0.0 },
  { "Charmagne", 0.0 },
  { "Charmaigne", 0.0 },
  { "Charmain", 0.0 },
  { "Charmaine", 0.0013080444 },
  { "Charman", 0.0 },
  { "Charmane", 0.0 },
  { "Charmarie", 0.0 },
  { "Charmayne", 0.0 },
  { "Charme", 0.0 },
  { "Charmecia", 0.0 },
  { "Charmeka", 0.0 },
  { "Charmel", 0.0 },
  { "Charmell", 0.0 },
  { "Charmella", 0.0 },
  { "Charmelle", 0.0 },
  { "Charmen", 0.0 },
  { "Charmi", 0.0 },
  { "Charmia", 0.0 },
  { "Charmian", 0.0 },
  { "Charmica", 0.0 },
  { "Charmie", 0.0 },
  { "Charmika", 0.0 },
  { "Charmin", 0.0 },
  { "Charmine", 0.0 },
  { "Charming", 1.0 },
  { "Charmion", 0.0 },
  { "Charmisa", 0.0 },
  { "Charmon", 0.0 },
  { "Charmonique", 0.0 },
  { "Charna", 0.0 },
  { "Charnae", 0.0 },
  { "Charnai", 0.0 },
  { "Charnay", 0.0 },
  { "Charnaye", 0.0 },
  { "Charne", 0.0 },
  { "Charnea", 0.0 },
  { "Charnece", 0.0 },
  { "Charnee", 0.0 },
  { "Charneice", 0.0 },
  { "Charneka", 0.0 },
  { "Charnel", 0.0 },
  { "Charnele", 0.0 },
  { "Charnell", 0.0 },
  { "Charnelle", 0.0 },
  { "Charnese", 0.0 },
  { "Charnesha", 0.0 },
  { "Charneshia", 0.0 },
  { "Charnesia", 0.0 },
  { "Charnessa", 0.0 },
  { "Charnetta", 0.0 },
  { "Charnette", 0.0 },
  { "Charney", 0.0 },
  { "Charnice", 0.0 },
  { "Charniece", 0.0 },
  { "Charnika", 0.0 },
  { "Charniqua", 0.0 },
  { "Charnique", 0.0 },
  { "Charnise", 0.0 },
  { "Charnisha", 0.0 },
  { "Charnissa", 0.0 },
  { "Charnita", 0.0 },
  { "Charny", 0.0 },
  { "Charo", 0.024590164 },
  { "Charod", 1.0 },
  { "Charol", 0.0 },
  { "Charolene", 0.0 },
  { "Charolet", 0.0 },
  { "Charolett", 0.0 },
  { "Charolette", 0.0 },
  { "Charolotte", 0.0 },
  { "Charolyn", 0.0 },
  { "Charon", 0.1768092 },
  { "Charona", 0.0 },
  { "Charonda", 0.0 },
  { "Charone", 0.23809524 },
  { "Charotte", 0.0 },
  { "Charquita", 0.0 },
  { "Charra", 0.0 },
  { "Charrell", 0.0 },
  { "Charrelle", 0.0 },
  { "Charri", 0.0 },
  { "Charrie", 0.0 },
  { "Charrise", 0.0 },
  { "Charrissa", 0.0 },
  { "Charrisse", 0.0 },
  { "Charro", 0.0 },
  { "Charron", 0.12315271 },
  { "Charry", 0.0 },
  { "Charryse", 0.0 },
  { "Charsie", 0.0 },
  { "Charta", 0.0 },
  { "Chartese", 0.0 },
  { "Chartez", 1.0 },
  { "Charu", 0.0 },
  { "Charvae", 0.0 },
  { "Charvette", 0.0 },
  { "Charvey", 0.0 },
  { "Charvez", 1.0 },
  { "Charvi", 0.0 },
  { "Charvik", 1.0 },
  { "Charvis", 1.0 },
  { "Charvon", 0.0 },
  { "Chary", 0.0 },
  { "Charyl", 0.0 },
  { "Charylene", 0.0 },
  { "Charyn", 0.0 },
  { "Charysse", 0.0 },
  { "Chas", 0.99688476 },
  { "Chasadee", 0.0 },
  { "Chasadie", 0.0 },
  { "Chasady", 0.0 },
  { "Chasaty", 0.0 },
  { "Chase", 0.97855985 },
  { "Chaselyn", 0.0 },
  { "Chaselynn", 0.0 },
  { "Chasen", 1.0 },
  { "Chasetin", 1.0 },
  { "Chaseton", 1.0 },
  { "Chasey", 0.0 },
  { "Chasia", 0.0 },
  { "Chasidee", 0.0 },
  { "Chasidi", 0.0 },
  { "Chasidy", 0.0 },
  { "Chasie", 0.0 },
  { "Chasin", 1.0 },
  { "Chasitee", 0.0 },
  { "Chasiti", 0.0 },
  { "Chasitie", 0.0 },
  { "Chasitity", 0.0 },
  { "Chasitiy", 0.0 },
  { "Chasitty", 0.0 },
  { "Chasity", 0.000965859 },
  { "Chaska", 0.8701299 },
  { "Chaske", 1.0 },
  { "Chaskel", 1.0 },
  { "Chaslyn", 0.0 },
  { "Chaslynn", 0.0 },
  { "Chasmin", 0.0 },
  { "Chasmine", 0.0 },
  { "Chasney", 0.0 },
  { "Chason", 1.0 },
  { "Chass", 1.0 },
  { "Chassady", 0.0 },
  { "Chasse", 1.0 },
  { "Chassi", 0.0 },
  { "Chassica", 0.0 },
  { "Chassidi", 0.0 },
  { "Chassidy", 0.0 },
  { "Chassie", 0.0 },
  { "Chassiti", 0.0 },
  { "Chassitty", 0.0 },
  { "Chassity", 0.0 },
  { "Chassy", 0.0 },
  { "Chasta", 0.0 },
  { "Chastain", 0.5 },
  { "Chastan", 1.0 },
  { "Chasteen", 1.0 },
  { "Chastelin", 0.0 },
  { "Chasteline", 0.0 },
  { "Chastelyn", 0.0 },
  { "Chastelyne", 0.0 },
  { "Chasten", 0.98188406 },
  { "Chastidy", 0.0 },
  { "Chastie", 0.0 },
  { "Chastin", 0.76666665 },
  { "Chastina", 0.0 },
  { "Chastine", 0.0 },
  { "Chastity", 0.0 },
  { "Chaston", 1.0 },
  { "Chasty", 0.0 },
  { "Chastyn", 1.0 },
  { "Chaswick", 1.0 },
  { "Chasya", 0.0 },
  { "Chasyn", 1.0 },
  { "Chat", 1.0 },
  { "Chata", 0.0 },
  { "Chatal", 0.0 },
  { "Chataqua", 0.0 },
  { "Chatara", 0.0 },
  { "Chatavia", 0.0 },
  { "Chatham", 0.75886524 },
  { "Chatherine", 0.0 },
  { "Chatia", 0.0 },
  { "Chatina", 0.0 },
  { "Chatman", 1.0 },
  { "Chatney", 0.0 },
  { "Chato", 1.0 },
  { "Chatoya", 0.0 },
  { "Chatrice", 0.0 },
  { "Chattie", 0.0 },
  { "Chau", 0.22384937 },
  { "Chaun", 0.97494304 },
  { "Chauna", 0.0 },
  { "Chaunce", 1.0 },
  { "Chauncey", 0.9484107 },
  { "Chaunci", 0.0 },
  { "Chauncie", 0.0 },
  { "Chauncy", 0.9738303 },
  { "Chaunda", 0.0 },
  { "Chaundra", 0.0 },
  { "Chaunice", 0.0 },
  { "Chaunta", 0.0 },
  { "Chauntae", 0.0 },
  { "Chauntay", 0.0 },
  { "Chaunte", 0.0 },
  { "Chauntee", 0.0 },
  { "Chauntel", 0.0 },
  { "Chauntell", 0.0 },
  { "Chauntelle", 0.0 },
  { "Chauntia", 0.0 },
  { "Chauntrice", 0.0 },
  { "Chaurice", 0.0 },
  { "Chava", 0.0 },
  { "Chavah", 0.0 },
  { "Chavas", 1.0 },
  { "Chavaughn", 0.0 },
  { "Chave", 0.0 },
  { "Chavela", 0.0 },
  { "Chaveli", 0.0 },
  { "Chavella", 0.0 },
  { "Chavelle", 0.0 },
  { "Chavely", 0.0 },
  { "Chavez", 0.99017197 },
  { "Chavi", 0.0 },
  { "Chavie", 0.0 },
  { "Chavis", 1.0 },
  { "Chaviva", 0.0 },
  { "Chavon", 0.07575758 },
  { "Chavonda", 0.0 },
  { "Chavone", 0.0 },
  { "Chavonna", 0.0 },
  { "Chavonne", 0.0 },
  { "Chavonte", 0.0 },
  { "Chavy", 0.0 },
  { "Chawana", 0.0 },
  { "Chawanda", 0.0 },
  { "Chawanna", 0.0 },
  { "Chawn", 0.8237885 },
  { "Chawna", 0.0 },
  { "Chay", 0.87383175 },
  { "Chaya", 0.00087435514 },
  { "Chayah", 0.0 },
  { "Chayan", 0.84375 },
  { "Chayane", 1.0 },
  { "Chayanna", 0.0 },
  { "Chayanne", 0.63857377 },
  { "Chayce", 0.8490771 },
  { "Chayden", 0.9783394 },
  { "Chaydon", 1.0 },
  { "Chaye", 0.0 },
  { "Chayenne", 0.0 },
  { "Chayil", 0.0 },
  { "Chayim", 1.0 },
  { "Chayla", 0.0 },
  { "Chaylee", 0.0 },
  { "Chaylen", 0.0 },
  { "Chaylene", 0.0 },
  { "Chaylin", 0.0 },
  { "Chaylyn", 0.0 },
  { "Chaylynn", 0.0 },
  { "Chayna", 0.0 },
  { "Chaynce", 1.0 },
  { "Chayne", 0.9056604 },
  { "Chaynee", 0.0 },
  { "Chayni", 0.0 },
  { "Chayse", 0.706104 },
  { "Chaysen", 1.0 },
  { "Chayson", 1.0 },
  { "Chaysten", 1.0 },
  { "Chayston", 1.0 },
  { "Chayten", 1.0 },
  { "Chayton", 0.99615383 },
  { "Chayzen", 1.0 },
  { "Chaz", 0.98315966 },
  { "Chazaray", 1.0 },
  { "Chazden", 1.0 },
  { "Chaze", 1.0 },
  { "Chazity", 0.0 },
  { "Chazlyn", 0.0 },
  { "Chazlynn", 0.0 },
  { "Chazman", 1.0 },
  { "Chazmin", 0.0 },
  { "Chazmine", 0.0 },
  { "Chazmon", 1.0 },
  { "Chazten", 1.0 },
  { "Chazton", 1.0 },
  { "Chazz", 0.9774495 },
  { "Che", 0.8439777 },
  { "Chea", 0.15789473 },
  { "Chealse", 0.0 },
  { "Chealsea", 0.0 },
  { "Chealsey", 0.0 },
  { "Chealsie", 0.0 },
  { "Chealsy", 0.0 },
  { "Ched", 1.0 },
  { "Chedrick", 1.0 },
  { "Chedva", 0.0 },
  { "Chee", 0.6086142 },
  { "Cheemeng", 1.0 },
  { "Cheena", 0.0 },
  { "Cheenou", 1.0 },
  { "Cheenue", 1.0 },
  { "Cheerful", 0.0 },
  { "Cheetara", 0.0 },
  { "Cheick", 1.0 },
  { "Cheikh", 1.0 },
  { "Cheila", 0.0 },
  { "Cheilon", 0.0 },
  { "Cheire", 0.0 },
  { "Cheisa", 0.0 },
  { "Cheisea", 0.0 },
  { "Chejuana", 0.0 },
  { "Chekesha", 0.0 },
  { "Cheketa", 0.0 },
  { "Chekita", 0.0 },
  { "Chela", 0.0 },
  { "Chelan", 0.0 },
  { "Chelbe", 0.0 },
  { "Chelbi", 0.0 },
  { "Chelbie", 0.0 },
  { "Chelby", 0.0 },
  { "Chelce", 0.0 },
  { "Chelcea", 0.0 },
  { "Chelcee", 0.0 },
  { "Chelcey", 0.0 },
  { "Chelci", 0.0 },
  { "Chelcia", 0.0 },
  { "Chelcie", 0.0 },
  { "Chelcy", 0.0 },
  { "Chele", 0.0 },
  { "Chelea", 0.0 },
  { "Chelena", 0.0 },
  { "Chelene", 0.0 },
  { "Chelesa", 0.0 },
  { "Chelese", 0.0 },
  { "Chelesea", 0.0 },
  { "Chelesy", 0.0 },
  { "Chelette", 0.0 },
  { "Cheli", 0.0 },
  { "Chelia", 0.0 },
  { "Chelisa", 0.0 },
  { "Chelise", 0.0 },
  { "Chelita", 0.0 },
  { "Chella", 0.0 },
  { "Chelle", 0.0 },
  { "Chelley", 0.0 },
  { "Chelli", 0.0 },
  { "Chellie", 0.0 },
  { "Chellsea", 0.0 },
  { "Chellsee", 0.0 },
  { "Chellsey", 0.0 },
  { "Chellsie", 0.0 },
  { "Chelly", 0.0 },
  { "Chelon", 0.0 },
  { "Chelonda", 0.0 },
  { "Chelsa", 0.0 },
  { "Chelsae", 0.0 },
  { "Chelsay", 0.0 },
  { "Chelse", 0.0 },
  { "Chelsea", 0.0028141525 },
  { "Chelseaann", 0.0 },
  { "Chelseaanne", 0.0 },
  { "Chelsealynn", 0.0 },
  { "Chelseamarie", 0.0 },
  { "Chelseay", 0.0 },
  { "Chelsee", 0.0 },
  { "Chelsei", 0.0 },
  { "Chelsen", 0.0 },
  { "Chelsey", 0.0020878168 },
  { "Chelsi", 0.0 },
  { "Chelsia", 0.0 },
  { "Chelsie", 0.0 },
  { "Chelssie", 0.0 },
  { "Chelssy", 0.0 },
  { "Chelsy", 0.0 },
  { "Chelsye", 0.0 },
  { "Chelton", 1.0 },
  { "Cheltzie", 0.0 },
  { "Chely", 0.0 },
  { "Chelyn", 0.0 },
  { "Chelynne", 0.0 },
  { "Chelzie", 0.0 },
  { "Chemaine", 0.0 },
  { "Chemane", 0.0 },
  { "Chemar", 1.0 },
  { "Chemeka", 0.0 },
  { "Chemene", 0.0 },
  { "Chemere", 0.0 },
  { "Chemika", 0.0 },
  { "Chemise", 0.0 },
  { "Chen", 0.75 },
  { "Chena", 0.0 },
  { "Chenae", 0.0 },
  { "Chenavia", 0.0 },
  { "Chenay", 0.0 },
  { "Chenda", 0.0 },
  { "Chendra", 0.0 },
  { "Chene", 0.14666666 },
  { "Chenea", 0.0 },
  { "Chenee", 0.0 },
  { "Cheneka", 0.0 },
  { "Chenel", 0.0 },
  { "Chenele", 0.0 },
  { "Chenell", 0.0 },
  { "Chenelle", 0.0 },
  { "Chenequa", 0.0 },
  { "Chenese", 0.0 },
  { "Cheney", 0.36391437 },
  { "Cheng", 0.9414484 },
  { "Chengyu", 1.0 },
  { "Chenia", 0.0 },
  { "Chenice", 0.0 },
  { "Cheniece", 0.0 },
  { "Chenier", 1.0 },
  { "Chenika", 0.0 },
  { "Chenille", 0.0 },
  { "Chenin", 0.0 },
  { "Cheniqua", 0.0 },
  { "Chenique", 0.0 },
  { "Chenise", 0.0 },
  { "Chenita", 0.0 },
  { "Cheniya", 0.0 },
  { "Chennel", 0.0 },
  { "Chennell", 0.0 },
  { "Chennelle", 0.0 },
  { "Chenoa", 0.0 },
  { "Chenoah", 0.0 },
  { "Chentel", 0.0 },
  { "Chenxi", 0.0 },
  { "Chenyu", 1.0 },
  { "Cheo", 1.0 },
  { "Chequetta", 0.0 },
  { "Chequita", 0.0 },
  { "Cher", 0.06097561 },
  { "Chera", 0.0 },
  { "Cherae", 0.0 },
  { "Cherakee", 0.0 },
  { "Cheral", 0.0 },
  { "Cheralee", 0.0 },
  { "Cheralyn", 0.0 },
  { "Cheramie", 0.0 },
  { "Cheray", 0.0 },
  { "Chere", 0.0 },
  { "Cherea", 0.0 },
  { "Cherece", 0.0 },
  { "Cheree", 0.0 },
  { "Chereen", 0.0 },
  { "Chereese", 0.0 },
  { "Chereka", 0.0 },
  { "Cherel", 0.0 },
  { "Cherell", 0.0 },
  { "Cherelle", 0.0 },
  { "Cherelyn", 0.0 },
  { "Cherena", 0.0 },
  { "Cherene", 0.0 },
  { "Cheresa", 0.0 },
  { "Cherese", 0.0 },
  { "Cheresse", 0.0 },
  { "Cheretta", 0.0 },
  { "Cherette", 0.0 },
  { "Cherhonda", 0.0 },
  { "Cheri", 0.00033307428 },
  { "Cheria", 0.0 },
  { "Cheriah", 0.0 },
  { "Cheriann", 0.0 },
  { "Cherianne", 0.0 },
  { "Cherica", 0.0 },
  { "Cherice", 0.0 },
  { "Chericka", 0.0 },
  { "Cherida", 0.0 },
  { "Cheridan", 0.0 },
  { "Cheridyn", 0.0 },
  { "Cherie", 0.000870322 },
  { "Cheriece", 0.0 },
  { "Cheriese", 0.0 },
  { "Cherif", 1.0 },
  { "Cherika", 0.0 },
  { "Cheril", 0.0 },
  { "Cherilee", 0.0 },
  { "Cherill", 0.0 },
  { "Cherilyn", 0.0 },
  { "Cherilynn", 0.0 },
  { "Cherin", 0.0 },
  { "Cherina", 0.0 },
  { "Cherine", 0.0 },
  { "Cheris", 0.0 },
  { "Cherisa", 0.0 },
  { "Cherise", 0.0 },
  { "Cherish", 0.0 },
  { "Cherisha", 0.0 },
  { "Cherissa", 0.0 },
  { "Cherisse", 0.0 },
  { "Cherita", 0.0 },
  { "Cherith", 0.0 },
  { "Cherity", 0.0 },
  { "Cherl", 0.0 },
  { "Cherle", 0.0 },
  { "Cherlene", 0.0 },
  { "Cherles", 1.0 },
  { "Cherlin", 0.0 },
  { "Cherline", 0.0 },
  { "Cherlonda", 0.0 },
  { "Cherly", 0.0 },
  { "Cherlyl", 0.0 },
  { "Cherlyn", 0.0 },
  { "Cherlynn", 0.0 },
  { "Chermaine", 0.0 },
  { "Chermeka", 0.0 },
  { "Cherod", 1.0 },
  { "Cherokee", 0.122238584 },
  { "Cherol", 0.0 },
  { "Cherolyn", 0.0 },
  { "Cheron", 0.13487738 },
  { "Cheronda", 0.0 },
  { "Cherone", 0.0 },
  { "Cherra", 0.0 },
  { "Cherre", 0.0 },
  { "Cherree", 0.0 },
  { "Cherrel", 0.0 },
  { "Cherrell", 0.0 },
  { "Cherrelle", 0.0 },
  { "Cherri", 0.0 },
  { "Cherrica", 0.0 },
  { "Cherrice", 0.0 },
  { "Cherrie", 0.0 },
  { "Cherril", 0.0 },
  { "Cherrill", 0.0 },
  { "Cherrilyn", 0.0 },
  { "Cherrilynn", 0.0 },
  { "Cherrise", 0.0 },
  { "Cherrish", 0.0 },
  { "Cherrisse", 0.0 },
  { "Cherrita", 0.0 },
  { "Cherron", 0.015527951 },
  { "Cherronda", 0.0 },
  { "Cherry", 0.011783189 },
  { "Cherryann", 0.0 },
  { "Cherrye", 0.0 },
  { "Cherryl", 0.0 },
  { "Cherryle", 0.0 },
  { "Cherryll", 0.0 },
  { "Cherub", 0.0 },
  { "Chervon", 0.0 },
  { "Chervonne", 0.0 },
  { "Chery", 0.0 },
  { "Cheryal", 0.0 },
  { "Cheryce", 0.0 },
  { "Cherye", 0.0 },
  { "Cheryel", 0.0 },
  { "Cheryl", 0.0022266908 },
  { "Cheryla", 0.0 },
  { "Cherylan", 0.0 },
  { "Cherylann", 0.0 },
  { "Cherylanne", 0.0 },
  { "Cheryle", 0.0 },
  { "Cherylee", 0.0 },
  { "Cherylene", 0.0 },
  { "Cherylin", 0.0 },
  { "Cheryll", 0.0 },
  { "Cherylle", 0.0 },
  { "Cheryllee", 0.0 },
  { "Cheryllyn", 0.0 },
  { "Cheryllynn", 0.0 },
  { "Cheryllynne", 0.0 },
  { "Cheryln", 0.0 },
  { "Cherylyn", 0.0 },
  { "Cherylynn", 0.0 },
  { "Cheryn", 0.0 },
  { "Cheryse", 0.0 },
  { "Cherysh", 0.0 },
  { "Ches", 1.0 },
  { "Chesa", 0.0 },
  { "Chesica", 0.0 },
  { "Cheska", 0.0 },
  { "Cheskel", 1.0 },
  { "Chesky", 1.0 },
  { "Cheslea", 0.0 },
  { "Chesleigh", 0.47619048 },
  { "Chesley", 0.79774487 },
  { "Cheslie", 0.0 },
  { "Chesna", 0.0 },
  { "Chesnee", 0.0 },
  { "Chesney", 0.055657774 },
  { "Chesni", 0.0 },
  { "Chesnie", 0.0 },
  { "Chess", 1.0 },
  { "Chessa", 0.0 },
  { "Chessica", 0.0 },
  { "Chessie", 0.0 },
  { "Chesten", 1.0 },
  { "Chester", 0.99318796 },
  { "Chesterfield", 1.0 },
  { "Chesterine", 0.0 },
  { "Chestin", 1.0 },
  { "Chestina", 0.0 },
  { "Chestine", 0.016025642 },
  { "Chestley", 1.0 },
  { "Cheston", 1.0 },
  { "Chet", 1.0 },
  { "Chetan", 1.0 },
  { "Chetana", 0.0 },
  { "Chetanna", 1.0 },
  { "Chetara", 0.0 },
  { "Chethan", 1.0 },
  { "Chett", 1.0 },
  { "Cheval", 1.0 },
  { "Chevala", 0.0 },
  { "Chevalier", 1.0 },
  { "Chevas", 1.0 },
  { "Chevaun", 0.0 },
  { "Chevell", 0.0 },
  { "Chevella", 0.0 },
  { "Chevelle", 0.07009175 },
  { "Chevette", 0.0 },
  { "Cheveyo", 1.0 },
  { "Chevez", 1.0 },
  { "Chevi", 0.0 },
  { "Chevie", 0.20987654 },
  { "Chevis", 0.96 },
  { "Chevon", 0.10194175 },
  { "Chevonda", 0.0 },
  { "Chevone", 0.0 },
  { "Chevonne", 0.0 },
  { "Chevy", 0.8273447 },
  { "Chew", 1.0 },
  { "Chey", 0.5135135 },
  { "Cheyan", 0.0 },
  { "Cheyana", 0.0 },
  { "Cheyane", 0.0 },
  { "Cheyann", 0.0 },
  { "Cheyanna", 0.0 },
  { "Cheyanne", 0.0 },
  { "Cheyeanne", 0.0 },
  { "Cheyeene", 0.0 },
  { "Cheyene", 0.029649595 },
  { "Cheyenn", 0.0 },
  { "Cheyenna", 0.0 },
  { "Cheyenne", 0.038407404 },
  { "Cheyla", 0.0 },
  { "Cheylee", 0.0 },
  { "Cheylin", 0.0 },
  { "Cheylyn", 0.0 },
  { "Cheylynn", 0.0 },
  { "Cheyna", 0.0 },
  { "Cheyne", 0.8870334 },
  { "Cheyney", 0.15151516 },
  { "Cheynne", 0.0 },
  { "Cheyrl", 0.0 },
  { "Cheyton", 1.0 },
  { "Chez", 0.8 },
  { "Cheza", 0.0 },
  { "Chezarae", 0.6 },
  { "Chezaray", 1.0 },
  { "Chezare", 0.75 },
  { "Chezkel", 1.0 },
  { "Chezky", 1.0 },
  { "Chezney", 0.0 },
  { "Cheznie", 0.0 },
  { "Chi", 0.43601894 },
  { "Chia", 0.24381626 },
  { "Chiagoziem", 1.0 },
  { "Chiaki", 0.0 },
  { "Chiam", 1.0 },
  { "Chiamaka", 0.0 },
  { "Chiamanda", 0.0 },
  { "Chian", 0.0 },
  { "Chiana", 0.0 },
  { "Chiane", 0.0 },
  { "Chiani", 0.0 },
  { "Chiann", 0.0 },
  { "Chianna", 0.0 },
  { "Chianne", 0.0 },
  { "Chiante", 0.06329114 },
  { "Chianti", 0.04901961 },
  { "Chiara", 0.0 },
  { "Chiarra", 0.0 },
  { "Chibueze", 1.0 },
  { "Chibuike", 1.0 },
  { "Chibuikem", 1.0 },
  { "Chibuzo", 1.0 },
  { "Chibuzor", 1.0 },
  { "Chic", 1.0 },
  { "Chicago", 1.0 },
  { "Chick", 1.0 },
  { "Chico", 1.0 },
  { "Chicora", 0.0 },
  { "Chicquita", 0.0 },
  { "Chidalu", 1.0 },
  { "Chidera", 0.32780847 },
  { "Chidi", 1.0 },
  { "Chidiebere", 1.0 },
  { "Chidiebube", 1.0 },
  { "Chidima", 0.0 },
  { "Chidimma", 0.0 },
  { "Chidinma", 0.0 },
  { "Chidozie", 1.0 },
  { "Chidubem", 1.0 },
  { "Chidumebi", 0.0 },
  { "Chie", 0.0 },
  { "Chiebuka", 1.0 },
  { "Chiedozie", 1.0 },
  { "Chiedu", 1.0 },
  { "Chief", 1.0 },
  { "Chieko", 0.0 },
  { "Chiem", 1.0 },
  { "Chiemeka", 1.0 },
  { "Chiemela", 0.0 },
  { "Chiemerie", 1.0 },
  { "Chiemi", 0.0 },
  { "Chien", 1.0 },
  { "Chierika", 0.0 },
  { "Chiffon", 0.0 },
  { "Chigozie", 0.95522386 },
  { "Chigozirim", 0.76 },
  { "Chih", 1.0 },
  { "Chihiro", 0.0 },
  { "Chijioke", 1.0 },
  { "Chika", 0.076712325 },
  { "Chikaima", 0.0 },
  { "Chikamara", 0.0 },
  { "Chikamso", 0.5535714 },
  { "Chikara", 0.0 },
  { "Chike", 1.0 },
  { "Chiketa", 0.0 },
  { "Chikezie", 1.0 },
  { "Chikita", 0.0 },
  { "Child", 0.5954198 },
  { "Childnotnamed", 0.0 },
  { "Chilton", 1.0 },
  { "Chilyn", 0.0 },
  { "Chima", 1.0 },
  { "Chimamaka", 0.0 },
  { "Chimamanda", 0.0 },
  { "Chimaobi", 1.0 },
  { "Chimaobim", 1.0 },
  { "Chimdi", 1.0 },
  { "Chimeka", 0.0 },
  { "Chimene", 0.0 },
  { "Chimeng", 1.0 },
  { "Chimera", 0.0 },
  { "Chimere", 0.0060313633 },
  { "Chimereze", 1.0 },
  { "Chimira", 0.0 },
  { "Chin", 1.0 },
  { "China", 0.0022168034 },
  { "Chinara", 0.0 },
  { "Chinasa", 0.0 },
  { "Chinaza", 0.0 },
  { "Chinda", 0.0 },
  { "Chinedu", 1.0 },
  { "Chinedum", 1.0 },
  { "Chineka", 0.0 },
  { "Chinelle", 0.0 },
  { "Chinelo", 0.036764707 },
  { "Chineme", 0.0 },
  { "Chinemerem", 0.88372093 },
  { "Chinenye", 0.0 },
  { "Chinere", 0.0 },
  { "Chinesta", 0.0 },
  { "Chinester", 0.0 },
  { "Chineta", 0.0 },
  { "Chinetta", 0.0 },
  { "Chineye", 0.0 },
  { "Ching", 0.7368421 },
  { "Chinh", 1.0 },
  { "Chinika", 0.0 },
  { "Chiniqua", 0.0 },
  { "Chinita", 0.0 },
  { "Chinitta", 0.0 },
  { "Chinmay", 1.0 },
  { "Chinmayi", 0.0 },
  { "Chinna", 0.0 },
  { "Chino", 1.0 },
  { "Chinomso", 1.0 },
  { "Chinonso", 1.0 },
  { "Chinonye", 0.0 },
  { "Chinou", 1.0 },
  { "Chintan", 1.0 },
  { "Chinua", 1.0 },
  { "Chinue", 0.0 },
  { "Chinwe", 0.0 },
  { "Chinwendu", 0.0 },
  { "Chinyere", 0.0 },
  { "Chioke", 1.0 },
  { "Chioma", 0.0 },
  { "Chip", 1.0 },
  { "Chipper", 1.0 },
  { "Chiquana", 0.0 },
  { "Chiqueta", 0.0 },
  { "Chiquetta", 0.0 },
  { "Chiquia", 0.0 },
  { "Chiquila", 0.0 },
  { "Chiquita", 0.00096432015 },
  { "Chiquitha", 0.0 },
  { "Chiquitia", 0.0 },
  { "Chiquitta", 0.0 },
  { "Chiquta", 0.0 },
  { "Chira", 0.0 },
  { "Chiraag", 1.0 },
  { "Chirag", 1.0 },
  { "Chirelle", 0.0 },
  { "Chiriga", 0.0 },
  { "Chirles", 1.0 },
  { "Chirley", 0.0 },
  { "Chiron", 0.91071427 },
  { "Chirs", 1.0 },
  { "Chirstian", 1.0 },
  { "Chirstina", 0.0 },
  { "Chirstine", 0.0 },
  { "Chirstopher", 1.0 },
  { "Chirsty", 0.0 },
  { "Chisa", 0.0 },
  { "Chisara", 0.0 },
  { "Chisato", 0.0 },
  { "Chisholm", 1.0 },
  { "Chisimdi", 0.0 },
  { "Chisom", 0.2925532 },
  { "Chistian", 1.0 },
  { "Chistie", 0.0 },
  { "Chistina", 0.0 },
  { "Chistine", 0.0 },
  { "Chistopher", 1.0 },
  { "Chisty", 0.0 },
  { "Chisum", 1.0 },
  { "Chitara", 0.0 },
  { "Chitina", 0.0 },
  { "Chitose", 0.0 },
  { "Chitra", 0.0 },
  { "Chivas", 1.0 },
  { "Chivon", 0.0 },
  { "Chivonne", 0.0 },
  { "Chiya", 0.0 },
  { "Chiye", 0.0 },
  { "Chiyeko", 0.0 },
  { "Chiyo", 0.0 },
  { "Chiyoka", 0.0 },
  { "Chiyoko", 0.0 },
  { "Chiyono", 0.0 },
  { "Chizara", 0.0 },
  { "Chizaram", 0.05978261 },
  { "Chizitelu", 0.0 },
  { "Chizitere", 0.0 },
  { "Chiziterem", 0.0 },
  { "Chizoba", 0.0 },
  { "Chizu", 0.0 },
  { "Chizue", 0.0 },
  { "Chizuko", 0.0 },
  { "Chlo", 0.0 },
  { "Chloa", 0.0 },
  { "Chloe", 0.0015658045 },
  { "Chloeann", 0.0 },
  { "Chloeanne", 0.0 },
  { "Chloee", 0.0 },
  { "Chloegrace", 0.0 },
  { "Chloeigh", 0.0 },
  { "Chloejane", 0.0 },
  { "Chloelynn", 0.0 },
  { "Chloemae", 0.0 },
  { "Chloemarie", 0.0 },
  { "Chloey", 0.0 },
  { "Chloi", 0.0 },
  { "Chloie", 0.0 },
  { "Chlora", 0.0 },
  { "Chlorene", 0.0 },
  { "Chlorine", 0.0 },
  { "Chloris", 0.0 },
  { "Chloye", 0.0 },
  { "Choice", 0.97619045 },
  { "Chol", 1.0 },
  { "Chole", 0.0 },
  { "Cholena", 0.0 },
  { "Chon", 1.0 },
  { "Chona", 0.0 },
  { "Chonda", 0.0 },
  { "Chondra", 0.0 },
  { "Chong", 0.59574467 },
  { "Chonita", 0.0 },
  { "Chonna", 0.0 },
  { "Chontae", 0.0 },
  { "Chonte", 0.0 },
  { "Chontel", 0.0 },
  { "Chontell", 0.0 },
  { "Chontelle", 0.0 },
  { "Chord", 1.0 },
  { "Chorlette", 0.0 },
  { "Chosen", 0.7368421 },
  { "Chosyn", 1.0 },
  { "Chou", 1.0 },
  { "Choua", 0.18067227 },
  { "Choya", 0.8202247 },
  { "Choyce", 0.56 },
  { "Chozen", 1.0 },
  { "Chquita", 0.0 },
  { "Chree", 0.0 },
  { "Chrie", 0.0 },
  { "Chris", 0.8625032 },
  { "Chrisa", 0.0 },
  { "Chrisalyn", 0.0 },
  { "Chrisalynn", 0.0 },
  { "Chrisan", 0.0 },
  { "Chrisander", 1.0 },
  { "Chrisandra", 0.0 },
  { "Chrisangel", 1.0 },
  { "Chrisann", 0.0 },
  { "Chrisanna", 0.0 },
  { "Chrisanne", 0.0 },
  { "Chrisanthe", 0.0 },
  { "Chrisanthi", 0.0 },
  { "Chrisanthony", 1.0 },
  { "Chrisaun", 1.0 },
  { "Chrisean", 1.0 },
  { "Chrisel", 0.0 },
  { "Chriselda", 0.0 },
  { "Chriselle", 0.0 },
  { "Chrisette", 0.0 },
  { "Chrisha", 0.0 },
  { "Chrishad", 1.0 },
  { "Chrishae", 0.0 },
  { "Chrishana", 0.0 },
  { "Chrishanda", 0.0 },
  { "Chrishanna", 0.0 },
  { "Chrishaud", 1.0 },
  { "Chrishaun", 0.98054475 },
  { "Chrishauna", 0.0 },
  { "Chrishaunda", 0.0 },
  { "Chrishawn", 0.91240877 },
  { "Chrishawna", 0.0 },
  { "Chrishay", 0.0 },
  { "Chrishayla", 0.0 },
  { "Chrishea", 0.0 },
  { "Chrishell", 0.0 },
  { "Chrishelle", 0.0 },
  { "Chrishiya", 0.0 },
  { "Chrishna", 0.0 },
  { "Chrishon", 1.0 },
  { "Chrishona", 0.0 },
  { "Chrishonda", 0.0 },
  { "Chrishonna", 0.0 },
  { "Chrishun", 1.0 },
  { "Chrishya", 0.0 },
  { "Chrisi", 0.0 },
  { "Chrisie", 0.0 },
  { "Chrisina", 0.0 },
  { "Chrisinda", 0.0 },
  { "Chrisine", 0.0 },
  { "Chrisitan", 1.0 },
  { "Chrisitne", 0.0 },
  { "Chrisiyah", 0.0 },
  { "Chrisley", 0.0 },
  { "Chrisly", 0.0 },
  { "Chrislyn", 0.0 },
  { "Chrislynn", 0.0 },
  { "Chrisma", 0.0 },
  { "Chrisna", 0.375 },
  { "Chrisopher", 1.0 },
  { "Chrisotpher", 1.0 },
  { "Chrisoula", 0.0 },
  { "Chrispher", 1.0 },
  { "Chrispin", 1.0 },
  { "Chriss", 0.7536946 },
  { "Chrissa", 0.0 },
  { "Chrissandra", 0.0 },
  { "Chrissean", 1.0 },
  { "Chrissette", 0.0 },
  { "Chrissey", 0.0 },
  { "Chrisshawn", 1.0 },
  { "Chrissi", 0.0 },
  { "Chrissiana", 0.0 },
  { "Chrissie", 0.0 },
  { "Chrissy", 0.0 },
  { "Christ", 0.9861922 },
  { "Christa", 0.00082855124 },
  { "Christabel", 0.0 },
  { "Christabell", 0.0 },
  { "Christabella", 0.0 },
  { "Christabelle", 0.0 },
  { "Christafer", 1.0 },
  { "Christain", 0.86976945 },
  { "Christal", 0.0 },
  { "Christale", 0.0 },
  { "Christalle", 0.0 },
  { "Christalyn", 0.0 },
  { "Christalynn", 0.0 },
  { "Christan", 0.5164602 },
  { "Christana", 0.0 },
  { "Christann", 0.0 },
  { "Christanna", 0.0 },
  { "Christany", 0.0 },
  { "Christapher", 1.0 },
  { "Christaphor", 1.0 },
  { "Christasha", 0.0 },
  { "Christasia", 0.0 },
  { "Christe", 0.0 },
  { "Christean", 0.0 },
  { "Christeanna", 0.0 },
  { "Christee", 0.0 },
  { "Christeen", 0.0 },
  { "Christeena", 0.0 },
  { "Christel", 0.0 },
  { "Christela", 0.0 },
  { "Christell", 0.0 },
  { "Christella", 0.0 },
  { "Christelle", 0.0 },
  { "Christen", 0.15215774 },
  { "Christena", 0.0 },
  { "Christene", 0.0 },
  { "Christenia", 0.0 },
  { "Christepher", 1.0 },
  { "Christer", 1.0 },
  { "Christerfer", 1.0 },
  { "Christerpher", 1.0 },
  { "Christerphor", 1.0 },
  { "Christey", 0.0 },
  { "Christhian", 1.0 },
  { "Christhoper", 1.0 },
  { "Christhopher", 1.0 },
  { "Christi", 0.0010535894 },
  { "Christia", 0.10161091 },
  { "Christiaan", 1.0 },
  { "Christiam", 1.0 },
  { "Christian", 0.9546493 },
  { "Christiana", 0.0 },
  { "Christianah", 0.0 },
  { "Christianalexan", 1.0 },
  { "Christiananthon", 1.0 },
  { "Christiandaniel", 1.0 },
  { "Christiane", 0.002676182 },
  { "Christianjacob", 1.0 },
  { "Christianjames", 1.0 },
  { "Christianjay", 1.0 },
  { "Christianjohn", 1.0 },
  { "Christianjoseph", 1.0 },
  { "Christianjoshua", 1.0 },
  { "Christianmichae", 1.0 },
  { "Christiann", 0.0 },
  { "Christianna", 0.0 },
  { "Christianne", 0.0 },
  { "Christiano", 1.0 },
  { "Christianpaul", 1.0 },
  { "Christianson", 1.0 },
  { "Christie", 0.018928189 },
  { "Christieann", 0.0 },
  { "Christielee", 0.0 },
  { "Christien", 0.868472 },
  { "Christiene", 0.0 },
  { "Christienne", 0.0 },
  { "Christifer", 1.0 },
  { "Christin", 0.08137611 },
  { "Christina", 0.004166099 },
  { "Christinamarie", 0.0 },
  { "Christine", 0.0034627456 },
  { "Christinea", 0.0 },
  { "Christineanne", 0.0 },
  { "Christinejoy", 0.0 },
  { "Christinemarie", 0.0 },
  { "Christing", 0.0 },
  { "Christinia", 0.0 },
  { "Christinna", 0.0 },
  { "Christino", 1.0 },
  { "Christion", 0.9854077 },
  { "Christiona", 0.0 },
  { "Christionna", 0.0 },
  { "Christiopher", 1.0 },
  { "Christipher", 1.0 },
  { "Christiyana", 0.0 },
  { "Christl", 0.0 },
  { "Christle", 0.0 },
  { "Christlyn", 0.0 },
  { "Christlynn", 0.0 },
  { "Christmas", 0.0 },
  { "Christna", 0.0 },
  { "Christne", 0.0 },
  { "Christo", 1.0 },
  { "Christobal", 1.0 },
  { "Christobel", 0.0 },
  { "Christof", 1.0 },
  { "Christofer", 1.0 },
  { "Christoff", 1.0 },
  { "Christoffer", 1.0 },
  { "Christofher", 1.0 },
  { "Christoher", 1.0 },
  { "Christohper", 1.0 },
  { "Christohpher", 1.0 },
  { "Christol", 0.0 },
  { "Christola", 0.0 },
  { "Christon", 0.8274832 },
  { "Christoopher", 1.0 },
  { "Christop", 1.0 },
  { "Christopehr", 1.0 },
  { "Christoper", 1.0 },
  { "Christoph", 1.0 },
  { "Christophe", 1.0 },
  { "Christophel", 1.0 },
  { "Christophen", 1.0 },
  { "Christopher", 0.995323 },
  { "Christopheranth", 1.0 },
  { "Christopherdavi", 1.0 },
  { "Christophere", 1.0 },
  { "Christopherjame", 1.0 },
  { "Christopherjohn", 1.0 },
  { "Christopherjose", 1.0 },
  { "Christopherjr", 1.0 },
  { "Christopherlee", 1.0 },
  { "Christophermich", 1.0 },
  { "Christopherpaul", 1.0 },
  { "Christopherryan", 1.0 },
  { "Christophor", 1.0 },
  { "Christophr", 1.0 },
  { "Christopoher", 1.0 },
  { "Christorpher", 1.0 },
  { "Christos", 1.0 },
  { "Christoval", 1.0 },
  { "Christpher", 1.0 },
  { "Christphor", 1.0 },
  { "Christpoher", 1.0 },
  { "Christropher", 1.0 },
  { "Christy", 0.01933536 },
  { "Christyan", 0.9820144 },
  { "Christyana", 0.0 },
  { "Christyann", 0.0 },
  { "Christyanna", 0.0 },
  { "Christye", 0.0 },
  { "Christyl", 0.0 },
  { "Christyle", 0.0 },
  { "Christylee", 0.0 },
  { "Christyn", 0.0058708414 },
  { "Christyna", 0.0 },
  { "Christyne", 0.0 },
  { "Christyonna", 0.0 },
  { "Chrisy", 0.0 },
  { "Chritian", 1.0 },
  { "Chritina", 0.0 },
  { "Chritine", 0.0 },
  { "Chritopher", 1.0 },
  { "Chrles", 1.0 },
  { "Chrostopher", 1.0 },
  { "Chrsitina", 0.0 },
  { "Chrsitine", 0.0 },
  { "Chrsitopher", 1.0 },
  { "Chrstina", 0.0 },
  { "Chrstine", 0.0 },
  { "Chrstopher", 1.0 },
  { "Chryel", 0.0 },
  { "Chryl", 0.0 },
  { "Chrys", 0.09767442 },
  { "Chrysa", 0.0 },
  { "Chrysander", 1.0 },
  { "Chrysanne", 0.0 },
  { "Chrysanthe", 0.0 },
  { "Chrysanthemum", 0.0 },
  { "Chrysanthia", 0.0 },
  { "Chrysoula", 0.0 },
  { "Chryssa", 0.0 },
  { "Chrysta", 0.0 },
  { "Chrystal", 0.0003190607 },
  { "Chrystalyn", 0.0 },
  { "Chrystan", 0.0 },
  { "Chrysteen", 0.0 },
  { "Chrystel", 0.0 },
  { "Chrystell", 0.0 },
  { "Chrystelle", 0.0 },
  { "Chrysten", 0.0 },
  { "Chrystene", 0.0 },
  { "Chrysti", 0.0 },
  { "Chrystian", 0.9109415 },
  { "Chrystiana", 0.0 },
  { "Chrystie", 0.0 },
  { "Chrystin", 0.0 },
  { "Chrystina", 0.0 },
  { "Chrystine", 0.0 },
  { "Chrystle", 0.0 },
  { "Chrystopher", 1.0 },
  { "Chrystyan", 1.0 },
  { "Chrystyna", 0.0 },
  { "Chu", 1.0 },
  { "Chua", 0.0 },
  { "Chubasco", 1.0 },
  { "Chubbie", 1.0 },
  { "Chubby", 1.0 },
  { "Chuck", 1.0 },
  { "Chuckie", 1.0 },
  { "Chucky", 1.0 },
  { "Chudney", 0.061728396 },
  { "Chue", 0.98502994 },
  { "Chuefeng", 1.0 },
  { "Chueyee", 1.0 },
  { "Chuka", 1.0 },
  { "Chukwubuikem", 1.0 },
  { "Chukwudi", 1.0 },
  { "Chukwudubem", 1.0 },
  { "Chukwuebuka", 1.0 },
  { "Chukwuemeka", 1.0 },
  { "Chukwuka", 1.0 },
  { "Chukwuma", 1.0 },
  { "Chukwunonso", 1.0 },
  { "Chuma", 1.0 },
  { "Chumani", 0.0 },
  { "Chumy", 0.0 },
  { "Chun", 0.81666666 },
  { "Chuna", 1.0 },
  { "Chundra", 0.0 },
  { "Chung", 0.94736844 },
  { "Chunta", 0.0 },
  { "Chuong", 1.0 },
  { "Church", 1.0 },
  { "Churchel", 1.0 },
  { "Churchill", 1.0 },
  { "Chutney", 0.0 },
  { "Chuy", 1.0 },
  { "Chy", 0.0 },
  { "Chyan", 0.0 },
  { "Chyana", 0.0 },
  { "Chyane", 0.0 },
  { "Chyann", 0.0 },
  { "Chyanna", 0.0 },
  { "Chyanne", 0.0017755682 },
  { "Chyeanne", 0.0 },
  { "Chyene", 0.0 },
  { "Chyenna", 0.0 },
  { "Chyenne", 0.0036737693 },
  { "Chyla", 0.0 },
  { "Chylah", 0.0 },
  { "Chylar", 0.0 },
  { "Chylee", 0.0 },
  { "Chyleen", 0.0 },
  { "Chyler", 0.17747441 },
  { "Chyles", 0.0 },
  { "Chylie", 0.0 },
  { "Chylynn", 0.0 },
  { "Chyna", 0.0 },
  { "Chynah", 0.0 },
  { "Chynia", 0.0 },
  { "Chynna", 0.0 },
  { "Chynthia", 0.0 },
  { "Chyra", 0.0 },
  { "Chyral", 0.0 },
  { "Chyrel", 0.0 },
  { "Chyrell", 0.0 },
  { "Chyrisse", 0.0 },
  { "Chyrl", 0.0 },
  { "Chyrle", 0.0 },
  { "Chyrstal", 0.0 },
  { "Chystal", 0.0 },
  { "Chyvonne", 0.0 },
  { "Cia", 0.0 },
  { "Ciah", 0.0 },
  { "Ciaira", 0.0 },
  { "Cian", 0.9912525 },
  { "Ciana", 0.0 },
  { "Cianah", 0.0 },
  { "Cianan", 1.0 },
  { "Ciandra", 0.0 },
  { "Ciane", 0.0 },
  { "Ciani", 0.0 },
  { "Cianie", 0.0 },
  { "Cianna", 0.0 },
  { "Ciannah", 0.0 },
  { "Cianne", 0.0 },
  { "Cianni", 0.0 },
  { "Cianny", 0.0 },
  { "Ciara", 0.0002118067 },
  { "Ciarah", 0.0 },
  { "Ciaran", 0.98050684 },
  { "Ciaria", 0.0 },
  { "Ciarra", 0.0 },
  { "Ciarrah", 0.0 },
  { "Ciasia", 0.0 },
  { "Ciauna", 0.0 },
  { "Ciboney", 0.0 },
  { "Cicel", 1.0 },
  { "Ciceley", 0.0 },
  { "Cicely", 0.0 },
  { "Cicero", 1.0 },
  { "Cici", 0.0 },
  { "Cicilia", 0.0 },
  { "Cicily", 0.0 },
  { "Ciclali", 0.0 },
  { "Ciclaly", 0.0 },
  { "Cicley", 0.0 },
  { "Cid", 1.0 },
  { "Cidalia", 0.0 },
  { "Cidnee", 0.0 },
  { "Cidney", 0.019165726 },
  { "Cidni", 0.0 },
  { "Cidnie", 0.0 },
  { "Cie", 0.0 },
  { "Cieana", 0.0 },
  { "Cieanna", 0.0 },
  { "Cieara", 0.0 },
  { "Ciearra", 0.0 },
  { "Ciel", 0.31986532 },
  { "Ciela", 0.0 },
  { "Cielita", 0.0 },
  { "Ciella", 0.0 },
  { "Cielle", 0.0 },
  { "Cielo", 0.014109986 },
  { "Ciena", 0.0 },
  { "Cienna", 0.0 },
  { "Ciera", 0.000445474 },
  { "Cierah", 0.0 },
  { "Cieran", 1.0 },
  { "Cieria", 0.0 },
  { "Cierra", 0.0013249908 },
  { "Cierrah", 0.0 },
  { "Cierre", 0.0 },
  { "Cierria", 0.0 },
  { "Ciersten", 0.0 },
  { "Cigi", 0.0 },
  { "Ciin", 0.0 },
  { "Cija", 0.0 },
  { "Ciji", 0.0 },
  { "Cilas", 1.0 },
  { "Ciley", 0.0 },
  { "Cilia", 0.0 },
  { "Cilicia", 0.0 },
  { "Cilla", 0.0 },
  { "Cillian", 1.0 },
  { "Cilton", 1.0 },
  { "Cimara", 0.0 },
  { "Cimarron", 0.875 },
  { "Cimberly", 0.0 },
  { "Cimone", 0.0 },
  { "Cimora", 0.0 },
  { "Cin", 1.0 },
  { "Cina", 0.0 },
  { "Cinamon", 0.0 },
  { "Cincere", 0.8013582 },
  { "Cinch", 1.0 },
  { "Cind", 0.0 },
  { "Cinda", 0.0 },
  { "Cindal", 0.0 },
  { "Cinde", 0.0 },
  { "Cindee", 0.0 },
  { "Cindel", 0.0 },
  { "Cindell", 0.0 },
  { "Cinder", 0.0 },
  { "Cinderella", 0.0 },
  { "Cindey", 0.0 },
  { "Cindi", 0.0 },
  { "Cindia", 0.0 },
  { "Cindie", 0.0 },
  { "Cindra", 0.0 },
  { "Cindy", 0.0030026522 },
  { "Cindya", 0.0 },
  { "Cindyann", 0.0 },
  { "Cindye", 0.0 },
  { "Cindylee", 0.0 },
  { "Cindylou", 0.0 },
  { "Cing", 0.0 },
  { "Ciniya", 0.0 },
  { "Ciniyah", 0.0 },
  { "Cinnamin", 0.0 },
  { "Cinnamon", 0.0 },
  { "Cinque", 1.0 },
  { "Cinsere", 0.92537314 },
  { "Cinthia", 0.0010109519 },
  { "Cinthya", 0.0 },
  { "Cintia", 0.0 },
  { "Cintra", 0.0 },
  { "Cintya", 0.0 },
  { "Cinya", 0.0 },
  { "Cinzia", 0.0 },
  { "Ciomara", 0.0 },
  { "Cion", 1.0 },
  { "Ciona", 0.0 },
  { "Cionna", 0.0 },
  { "Cionne", 0.0 },
  { "Cionni", 0.0 },
  { "Ciprian", 1.0 },
  { "Cipriana", 0.0 },
  { "Ciprianna", 0.0 },
  { "Cipriano", 1.0 },
  { "Cira", 0.0 },
  { "Circe", 0.0 },
  { "Cire", 0.5382353 },
  { "Ciree", 1.0 },
  { "Cirena", 0.0 },
  { "Cirenia", 0.0 },
  { "Ciri", 0.0 },
  { "Ciria", 0.0 },
  { "Ciriaco", 1.0 },
  { "Cirila", 0.0 },
  { "Cirildo", 1.0 },
  { "Cirilla", 0.0 },
  { "Cirilo", 1.0 },
  { "Cirino", 1.0 },
  { "Ciro", 1.0 },
  { "Cirra", 0.0 },
  { "Cirrus", 1.0 },
  { "Cirsten", 0.0 },
  { "Cisco", 1.0 },
  { "Cisely", 0.0 },
  { "Cissie", 0.0 },
  { "Cissy", 0.0 },
  { "Cita", 0.0 },
  { "Citali", 0.0 },
  { "Citalli", 0.0 },
  { "Citally", 0.0 },
  { "Cithlali", 0.0 },
  { "Cithlaly", 0.0 },
  { "Citialli", 0.0 },
  { "Citlali", 0.0 },
  { "Citlalic", 0.0 },
  { "Citlalli", 0.0 },
  { "Citlally", 0.0 },
  { "Citlaly", 0.0 },
  { "Citori", 0.0 },
  { "Cittlali", 0.0 },
  { "Cittlaly", 0.0 },
  { "Civona", 0.0 },
  { "Ciya", 0.0 },
  { "Ciyah", 0.0 },
  { "Ciyana", 0.0 },
  { "Ciyanna", 0.0 },
  { "Cj", 0.89469516 },
  { "Cjay", 1.0 },
  { "Clabe", 1.0 },
  { "Clabern", 1.0 },
  { "Clabon", 1.0 },
  { "Claborn", 1.0 },
  { "Cladie", 0.0 },
  { "Clady", 0.0 },
  { "Clae", 1.0 },
  { "Claiborn", 1.0 },
  { "Claiborne", 0.97797716 },
  { "Clair", 0.74856967 },
  { "Claira", 0.0 },
  { "Clairabelle", 0.0 },
  { "Clairah", 0.0 },
  { "Clairann", 0.0 },
  { "Claire", 0.012193291 },
  { "Clairece", 0.0 },
  { "Clairemarie", 0.0 },
  { "Clairene", 0.0 },
  { "Clairese", 0.0 },
  { "Clairessa", 0.0 },
  { "Clairice", 0.0 },
  { "Clairissa", 0.0 },
  { "Clairity", 0.0 },
  { "Clairmont", 1.0 },
  { "Clance", 1.0 },
  { "Clancey", 0.7553192 },
  { "Clancie", 0.0 },
  { "Clancy", 0.7115269 },
  { "Clanton", 1.0 },
  { "Clara", 0.0036740124 },
  { "Clarabel", 0.0 },
  { "Clarabell", 0.0 },
  { "Clarabella", 0.0 },
  { "Clarabelle", 0.0 },
  { "Clarabeth", 0.0 },
  { "Clarah", 0.0 },
  { "Clarajane", 0.0 },
  { "Claraliz", 0.0 },
  { "Claramae", 0.0 },
  { "Clarance", 1.0 },
  { "Claranell", 0.0 },
  { "Clarann", 0.0 },
  { "Claranne", 0.0 },
  { "Clararose", 0.0 },
  { "Clarcie", 0.0 },
  { "Clardie", 1.0 },
  { "Clardy", 1.0 },
  { "Clare", 0.096135065 },
  { "Clareatha", 0.0 },
  { "Clarece", 0.0 },
  { "Claree", 0.0 },
  { "Clareen", 0.0 },
  { "Clareese", 0.0 },
  { "Claren", 0.4047619 },
  { "Clarena", 0.0 },
  { "Clarenc", 1.0 },
  { "Clarence", 0.9938876 },
  { "Clarencia", 0.0 },
  { "Clarenda", 0.0 },
  { "Clarendon", 1.0 },
  { "Clarene", 0.0 },
  { "Claresa", 0.0 },
  { "Clarese", 0.0 },
  { "Claressa", 0.0 },
  { "Claretha", 0.0 },
  { "Claretta", 0.0 },
  { "Clarette", 0.0 },
  { "Claria", 0.0 },
  { "Claribel", 0.0 },
  { "Claribell", 0.0 },
  { "Claribelle", 0.0 },
  { "Clarica", 0.0 },
  { "Clarice", 0.0038123983 },
  { "Clarie", 0.0 },
  { "Clariece", 0.0 },
  { "Clarina", 0.0 },
  { "Clarince", 1.0 },
  { "Clarinda", 0.0 },
  { "Clarine", 0.0 },
  { "Clarion", 0.9186992 },
  { "Claris", 0.07333333 },
  { "Clarisa", 0.0 },
  { "Clarise", 0.0 },
  { "Clarisha", 0.0 },
  { "Clarissa", 0.00055452867 },
  { "Clarisse", 0.0 },
  { "Clarissia", 0.0 },
  { "Claristine", 0.0 },
  { "Clarita", 0.0 },
  { "Claritha", 0.0 },
  { "Claritsa", 0.0 },
  { "Clarity", 0.0 },
  { "Claritza", 0.0 },
  { "Clarivel", 0.0 },
  { "Clarixa", 0.0 },
  { "Clariza", 0.0 },
  { "Clark", 0.99610436 },
  { "Clarke", 0.8614341 },
  { "Clarkie", 0.0 },
  { "Clarkson", 1.0 },
  { "Clarnce", 1.0 },
  { "Clarnece", 1.0 },
  { "Claro", 1.0 },
  { "Claron", 1.0 },
  { "Clarra", 0.0 },
  { "Clarrisa", 0.0 },
  { "Clarrissa", 0.0 },
  { "Clarsie", 0.0 },
  { "Clary", 0.45268542 },
  { "Claryce", 0.0 },
  { "Clarys", 0.0 },
  { "Claryssa", 0.0 },
  { "Clash", 1.0 },
  { "Classie", 0.0 },
  { "Classy", 0.0 },
  { "Clata", 0.0 },
  { "Clate", 1.0 },
  { "Clatie", 0.0 },
  { "Claton", 1.0 },
  { "Claud", 0.9957038 },
  { "Clauda", 0.0 },
  { "Claude", 0.9896317 },
  { "Claudea", 0.0 },
  { "Claudean", 0.0 },
  { "Claudeen", 0.0 },
  { "Claudel", 1.0 },
  { "Claudell", 0.65603644 },
  { "Claudelle", 0.0 },
  { "Claudene", 0.0 },
  { "Claudett", 0.0 },
  { "Claudetta", 0.0 },
  { "Claudette", 0.00058798376 },
  { "Claudia", 0.0059892526 },
  { "Claudiajoy", 0.0 },
  { "Claudie", 0.4432698 },
  { "Claudina", 0.0 },
  { "Claudine", 0.0013249713 },
  { "Claudinette", 0.0 },
  { "Claudio", 0.9947121 },
  { "Claudis", 0.96961325 },
  { "Claudius", 1.0 },
  { "Claudy", 0.9574468 },
  { "Claudya", 0.0 },
  { "Claus", 1.0 },
  { "Clausell", 1.0 },
  { "Clauzell", 1.0 },
  { "Claven", 1.0 },
  { "Clavin", 1.0 },
  { "Clavon", 1.0 },
  { "Clawson", 1.0 },
  { "Claxton", 1.0 },
  { "Clay", 0.9928046 },
  { "Claybon", 1.0 },
  { "Clayborn", 1.0 },
  { "Clayborne", 1.0 },
  { "Claybourne", 1.0 },
  { "Clayburn", 1.0 },
  { "Clayden", 1.0 },
  { "Claydon", 1.0 },
  { "Claymon", 1.0 },
  { "Clayne", 1.0 },
  { "Clayre", 0.0 },
  { "Clayson", 1.0 },
  { "Clayt", 1.0 },
  { "Clayten", 1.0 },
  { "Claytie", 0.0 },
  { "Claytin", 1.0 },
  { "Clayton", 0.995502 },
  { "Clayvon", 1.0 },
  { "Clea", 0.0152207 },
  { "Cleaburn", 1.0 },
  { "Cleada", 0.0 },
  { "Cleah", 0.0 },
  { "Cleal", 1.0 },
  { "Cleamon", 1.0 },
  { "Cleao", 0.33333334 },
  { "Cleara", 0.0 },
  { "Clearance", 1.0 },
  { "Clearence", 1.0 },
  { "Clearnce", 1.0 },
  { "Cleary", 0.5555556 },
  { "Cleason", 1.0 },
  { "Cleaster", 0.1970803 },
  { "Cleaston", 1.0 },
  { "Cleat", 1.0 },
  { "Cleata", 0.0 },
  { "Cleatis", 0.90789473 },
  { "Cleatrice", 0.0 },
  { "Cleatus", 0.92634857 },
  { "Cleave", 1.0 },
  { "Cleaveland", 1.0 },
  { "Cleaven", 1.0 },
  { "Cleavie", 0.0 },
  { "Cleavland", 1.0 },
  { "Cleavon", 1.0 },
  { "Clebert", 1.0 },
  { "Cleburn", 1.0 },
  { "Cleburne", 1.0 },
  { "Cleda", 0.0 },
  { "Cledia", 0.0 },
  { "Cledis", 1.0 },
  { "Cledith", 0.4579646 },
  { "Cledus", 1.0 },
  { "Clee", 1.0 },
  { "Cleetus", 0.0 },
  { "Clegg", 1.0 },
  { "Cleidy", 0.0 },
  { "Cleighton", 1.0 },
  { "Clela", 0.0 },
  { "Cleland", 1.0 },
  { "Clelia", 0.0 },
  { "Clell", 1.0 },
  { "Clella", 0.0 },
  { "Clellan", 1.0 },
  { "Clellie", 0.41666666 },
  { "Clellon", 1.0 },
  { "Clem", 0.9874971 },
  { "Clema", 0.0 },
  { "Cleman", 1.0 },
  { "Clematine", 0.0 },
  { "Clemen", 1.0 },
  { "Clemence", 0.4675615 },
  { "Clemencia", 0.0 },
  { "Clemens", 1.0 },
  { "Clement", 0.9995612 },
  { "Clemente", 1.0 },
  { "Clementeen", 0.0 },
  { "Clementene", 0.0 },
  { "Clementina", 0.0 },
  { "Clementine", 0.0 },
  { "Clements", 1.0 },
  { "Clemet", 1.0 },
  { "Clemetine", 0.0 },
  { "Clemie", 0.023148147 },
  { "Clemintine", 0.0 },
  { "Clemma", 0.0 },
  { "Clemmie", 0.26977518 },
  { "Clemmon", 1.0 },
  { "Clemon", 1.0 },
  { "Clemons", 1.0 },
  { "Clemont", 1.0 },
  { "Clemontine", 0.0 },
  { "Clemson", 1.0 },
  { "Clen", 1.0 },
  { "Clenard", 1.0 },
  { "Clendon", 1.0 },
  { "Clennie", 0.5510204 },
  { "Clennon", 1.0 },
  { "Clenon", 1.0 },
  { "Clent", 1.0 },
  { "Clenton", 1.0 },
  { "Cleo", 0.2725047 },
  { "Cleoda", 0.0 },
  { "Cleodis", 1.0 },
  { "Cleofas", 0.7619048 },
  { "Cleofus", 1.0 },
  { "Cleola", 0.0 },
  { "Cleoma", 0.0 },
  { "Cleon", 0.93971294 },
  { "Cleona", 0.0 },
  { "Cleone", 0.003773585 },
  { "Cleonia", 0.0 },
  { "Cleonte", 1.0 },
  { "Cleopatra", 0.0 },
  { "Cleopatria", 0.0 },
  { "Cleopha", 0.0 },
  { "Cleophas", 0.9765784 },
  { "Cleophes", 1.0 },
  { "Cleophis", 1.0 },
  { "Cleophus", 0.9906954 },
  { "Cleora", 0.0 },
  { "Cleoria", 0.0 },
  { "Cleota", 0.0 },
  { "Cleotha", 0.7195005 },
  { "Cleother", 1.0 },
  { "Cleothis", 1.0 },
  { "Cleotilde", 0.0 },
  { "Cleotis", 1.0 },
  { "Clera", 0.0 },
  { "Clerance", 1.0 },
  { "Clercie", 0.0 },
  { "Clerence", 1.0 },
  { "Clerissa", 0.0 },
  { "Clermont", 1.0 },
  { "Clero", 0.0 },
  { "Clesha", 0.0 },
  { "Clessie", 0.0 },
  { "Clesson", 1.0 },
  { "Clesta", 0.0 },
  { "Cleste", 0.0 },
  { "Clester", 0.83597887 },
  { "Clestine", 0.0 },
  { "Cleston", 1.0 },
  { "Cleta", 0.0 },
  { "Clete", 1.0 },
  { "Cletes", 1.0 },
  { "Cletha", 0.0 },
  { "Cletis", 0.9096591 },
  { "Cleto", 1.0 },
  { "Cletta", 0.0 },
  { "Cletus", 0.9643606 },
  { "Cletys", 0.0 },
  { "Cleva", 0.0 },
  { "Cleve", 0.99491936 },
  { "Cleveland", 0.9960221 },
  { "Cleven", 1.0 },
  { "Clever", 1.0 },
  { "Clevester", 1.0 },
  { "Clevette", 0.0 },
  { "Clevia", 0.0 },
  { "Clevie", 0.19594595 },
  { "Clevland", 1.0 },
  { "Clevon", 1.0 },
  { "Clhoe", 0.0 },
  { "Clida", 0.0 },
  { "Clide", 0.95131576 },
  { "Clidie", 0.0 },
  { "Clidy", 0.0 },
  { "Clif", 1.0 },
  { "Cliff", 1.0 },
  { "Cliffard", 1.0 },
  { "Cliffie", 0.0 },
  { "Clifford", 0.9893151 },
  { "Cliffton", 1.0 },
  { "Cliford", 1.0 },
  { "Clift", 1.0 },
  { "Cliftina", 0.0 },
  { "Clifton", 0.9937533 },
  { "Clim", 1.0 },
  { "Climmie", 0.36061028 },
  { "Climon", 1.0 },
  { "Clinard", 1.0 },
  { "Cline", 1.0 },
  { "Clinesha", 0.0 },
  { "Clinetta", 0.0 },
  { "Clinique", 0.0 },
  { "Clinnie", 0.45454547 },
  { "Clint", 0.998307 },
  { "Clinten", 1.0 },
  { "Clintin", 1.0 },
  { "Clinton", 0.99553096 },
  { "Clintona", 0.0 },
  { "Clintonia", 0.0 },
  { "Clio", 0.03875969 },
  { "Cliona", 0.0 },
  { "Clione", 0.0 },
  { "Clista", 0.0 },
  { "Clister", 0.0 },
  { "Cliston", 1.0 },
  { "Clive", 1.0 },
  { "Clo", 0.0 },
  { "Cloa", 0.0 },
  { "Cloda", 0.0 },
  { "Clodagh", 0.0 },
  { "Clodean", 0.0 },
  { "Clodie", 0.0 },
  { "Cloe", 0.0014392631 },
  { "Cloee", 0.0 },
  { "Cloetta", 0.0 },
  { "Cloey", 0.0 },
  { "Cloi", 0.0 },
  { "Cloice", 1.0 },
  { "Cloid", 1.0 },
  { "Cloie", 0.0 },
  { "Clois", 0.72791165 },
  { "Cloise", 1.0 },
  { "Cloma", 0.0 },
  { "Clomer", 1.0 },
  { "Clora", 0.0 },
  { "Cloral", 1.0 },
  { "Cloree", 0.0 },
  { "Clorence", 1.0 },
  { "Clorene", 0.0 },
  { "Cloretta", 0.0 },
  { "Cloria", 0.0 },
  { "Clorice", 0.0 },
  { "Clorie", 0.0 },
  { "Clorinda", 0.0 },
  { "Clorine", 0.0 },
  { "Cloris", 0.0 },
  { "Clorisa", 0.0 },
  { "Clorise", 0.0 },
  { "Clorissa", 0.0 },
  { "Clota", 0.0 },
  { "Clote", 0.0 },
  { "Cloteal", 0.0 },
  { "Clotee", 0.0 },
  { "Cloteen", 0.0 },
  { "Clotell", 0.0 },
  { "Clotene", 0.0 },
  { "Clothilda", 0.0 },
  { "Clothilde", 0.0 },
  { "Clotie", 0.0 },
  { "Clotiel", 0.0 },
  { "Clotilda", 0.0 },
  { "Clotilde", 0.0 },
  { "Clotile", 0.0 },
  { "Clotine", 0.0 },
  { "Clotis", 1.0 },
  { "Cloud", 1.0 },
  { "Cloude", 1.0 },
  { "Cloudia", 0.0 },
  { "Cloudie", 0.625 },
  { "Clouis", 1.0 },
  { "Clova", 0.0 },
  { "Clover", 0.012825349 },
  { "Clovia", 0.0 },
  { "Clovie", 0.0 },
  { "Clovis", 0.8258333 },
  { "Clowey", 0.0 },
  { "Clowie", 0.0 },
  { "Cloy", 0.96183205 },
  { "Cloyce", 0.87875754 },
  { "Cloyd", 1.0 },
  { "Cloyde", 1.0 },
  { "Cloye", 0.0 },
  { "Cloyed", 1.0 },
  { "Cloys", 1.0 },
  { "Cloyse", 1.0 },
  { "Clu", 1.0 },
  { "Cludia", 0.0 },
  { "Clura", 0.0 },
  { "Cluster", 1.0 },
  { "Clutch", 1.0 },
  { "Clyda", 0.0 },
  { "Clyde", 0.97301936 },
  { "Clydean", 0.0 },
  { "Clydeen", 0.0 },
  { "Clydel", 0.0 },
  { "Clydell", 0.6227273 },
  { "Clydene", 0.0 },
  { "Clydetta", 0.0 },
  { "Clydette", 0.0 },
  { "Clydia", 0.0 },
  { "Clydie", 0.025142858 },
  { "Clydine", 0.0 },
  { "Clyne", 1.0 },
  { "Clynt", 1.0 },
  { "Clynton", 1.0 },
  { "Clysta", 0.0 },
  { "Clyta", 0.0 },
  { "Clytee", 0.0 },
  { "Clytia", 0.0 },
  { "Clytie", 0.0 },
  { "Cmone", 0.0 },
  { "Cmya", 0.0 },
  { "Cne", 0.0 },
  { "Cniya", 0.0 },
  { "Cniyah", 0.0 },
  { "Cnya", 0.0 },
  { "Cnythia", 0.0 },
  { "Coady", 1.0 },
  { "Coal", 1.0 },
  { "Coalson", 1.0 },
  { "Coalton", 1.0 },
  { "Coast", 1.0 },
  { "Cobain", 1.0 },
  { "Cobalt", 1.0 },
  { "Cobb", 1.0 },
  { "Cobbie", 1.0 },
  { "Cobe", 1.0 },
  { "Cobee", 1.0 },
  { "Coben", 1.0 },
  { "Cobert", 1.0 },
  { "Cobey", 1.0 },
  { "Cobi", 0.7518685 },
  { "Cobie", 0.7261663 },
  { "Cobin", 1.0 },
  { "Cobina", 0.0 },
  { "Cobra", 1.0 },
  { "Coburn", 1.0 },
  { "Coby", 0.9585893 },
  { "Cobyn", 1.0 },
  { "Cochise", 1.0 },
  { "Coco", 0.0 },
  { "Cocoa", 0.0 },
  { "Coda", 0.87304074 },
  { "Coddy", 1.0 },
  { "Codee", 0.5804989 },
  { "Codell", 1.0 },
  { "Coden", 1.0 },
  { "Codera", 1.0 },
  { "Codero", 1.0 },
  { "Coderro", 1.0 },
  { "Codey", 0.99121433 },
  { "Codi", 0.3680872 },
  { "Codie", 0.6057853 },
  { "Cody", 0.9827713 },
  { "Codyallen", 1.0 },
  { "Codye", 1.0 },
  { "Codyjames", 1.0 },
  { "Codylee", 1.0 },
  { "Coe", 1.0 },
  { "Coedy", 1.0 },
  { "Coehn", 1.0 },
  { "Coel", 1.0 },
  { "Coella", 0.0 },
  { "Coen", 1.0 },
  { "Coeta", 0.0 },
  { "Coetta", 0.0 },
  { "Coey", 0.5 },
  { "Coffy", 0.0 },
  { "Cogan", 1.0 },
  { "Cohan", 1.0 },
  { "Cohen", 0.9899666 },
  { "Cohl", 1.0 },
  { "Cohlton", 1.0 },
  { "Cohner", 1.0 },
  { "Cohutta", 1.0 },
  { "Cohyn", 1.0 },
  { "Coi", 0.5945946 },
  { "Coila", 0.0 },
  { "Coilin", 1.0 },
  { "Coire", 1.0 },
  { "Coit", 1.0 },
  { "Coke", 1.0 },
  { "Coker", 1.0 },
  { "Cola", 0.2382353 },
  { "Colan", 1.0 },
  { "Colanda", 0.0 },
  { "Colandra", 0.0 },
  { "Colbe", 0.94957983 },
  { "Colbee", 0.17857143 },
  { "Colben", 1.0 },
  { "Colbert", 1.0 },
  { "Colbey", 0.94899815 },
  { "Colbi", 0.122370936 },
  { "Colbie", 0.14899452 },
  { "Colbin", 1.0 },
  { "Colburn", 1.0 },
  { "Colby", 0.93045056 },
  { "Colbyn", 1.0 },
  { "Colden", 1.0 },
  { "Coldin", 1.0 },
  { "Cole", 0.99648464 },
  { "Colean", 0.0 },
  { "Coleby", 1.0 },
  { "Coleden", 1.0 },
  { "Coledon", 1.0 },
  { "Colee", 0.0 },
  { "Coleen", 0.0 },
  { "Coleena", 0.0 },
  { "Coleene", 0.0 },
  { "Coleeta", 0.0 },
  { "Coleigh", 0.0 },
  { "Coleman", 0.9981824 },
  { "Colemon", 1.0 },
  { "Colen", 1.0 },
  { "Colena", 0.0 },
  { "Colene", 0.0 },
  { "Coleon", 1.0 },
  { "Coleone", 1.0 },
  { "Coleridge", 1.0 },
  { "Colesha", 0.0 },
  { "Coleson", 1.0 },
  { "Coleston", 1.0 },
  { "Colesyn", 1.0 },
  { "Coleta", 0.0 },
  { "Coleten", 1.0 },
  { "Coletha", 0.0 },
  { "Coletin", 1.0 },
  { "Coleton", 1.0 },
  { "Coletta", 0.0 },
  { "Colette", 0.0 },
  { "Coletyn", 1.0 },
  { "Coley", 0.88526076 },
  { "Colgan", 1.0 },
  { "Colicia", 0.0 },
  { "Colie", 0.84766215 },
  { "Colin", 0.9964621 },
  { "Colina", 0.0 },
  { "Colinda", 0.0 },
  { "Coline", 0.0 },
  { "Colisa", 0.0 },
  { "Colisha", 0.0 },
  { "Colista", 0.0 },
  { "Colita", 0.0 },
  { "Collan", 1.0 },
  { "Colleen", 0.0020321072 },
  { "Colleena", 0.0 },
  { "Colleene", 0.0 },
  { "Collen", 0.5419643 },
  { "Collena", 0.0 },
  { "Collene", 0.0 },
  { "Collenn", 0.0 },
  { "Collete", 0.0 },
  { "Collett", 0.0 },
  { "Colletta", 0.0 },
  { "Collette", 0.0 },
  { "Colley", 1.0 },
  { "Collie", 0.7705457 },
  { "Collier", 0.9378173 },
  { "Collin", 0.99419075 },
  { "Colline", 0.0 },
  { "Collins", 0.61246663 },
  { "Collis", 0.9923606 },
  { "Collyn", 0.79704016 },
  { "Collynn", 0.5 },
  { "Collyns", 0.0 },
  { "Colm", 1.0 },
  { "Colman", 1.0 },
  { "Colmon", 1.0 },
  { "Colon", 0.9917231 },
  { "Colonel", 1.0 },
  { "Colorado", 1.0 },
  { "Colquitt", 1.0 },
  { "Colsen", 1.0 },
  { "Colson", 1.0 },
  { "Colsten", 1.0 },
  { "Colston", 1.0 },
  { "Colsyn", 1.0 },
  { "Colt", 0.99923885 },
  { "Coltan", 1.0 },
  { "Colten", 1.0 },
  { "Colter", 1.0 },
  { "Coltin", 1.0 },
  { "Coltn", 1.0 },
  { "Colton", 0.99845564 },
  { "Coltrane", 1.0 },
  { "Coltyn", 1.0 },
  { "Colum", 1.0 },
  { "Columba", 0.0 },
  { "Columbia", 0.007751938 },
  { "Columbine", 0.0 },
  { "Columbo", 1.0 },
  { "Columbus", 1.0 },
  { "Colvin", 1.0 },
  { "Colwyn", 1.0 },
  { "Colyer", 1.0 },
  { "Colyn", 0.98542273 },
  { "Colynn", 0.0 },
  { "Coma", 0.22727273 },
  { "Comaneci", 0.0 },
  { "Comari", 1.0 },
  { "Comeka", 0.0 },
  { "Comekia", 0.0 },
  { "Comelia", 0.0 },
  { "Comer", 1.0 },
  { "Comesha", 0.0 },
  { "Comfort", 0.0 },
  { "Comilla", 0.0 },
  { "Comisha", 0.0 },
  { "Commer", 1.0 },
  { "Commie", 0.6964286 },
  { "Commodore", 1.0 },
  { "Common", 1.0 },
  { "Compton", 1.0 },
  { "Con", 1.0 },
  { "Cona", 0.0 },
  { "Conagher", 1.0 },
  { "Conal", 1.0 },
  { "Conall", 1.0 },
  { "Conan", 1.0 },
  { "Conant", 1.0 },
  { "Conar", 1.0 },
  { "Conard", 1.0 },
  { "Concepcion", 0.1577061 },
  { "Concepsion", 0.0 },
  { "Conception", 0.0 },
  { "Concetta", 0.00035468538 },
  { "Concettina", 0.0 },
  { "Concetto", 1.0 },
  { "Concha", 0.0 },
  { "Conchata", 0.0 },
  { "Concheta", 0.0 },
  { "Conchetta", 0.0 },
  { "Conchita", 0.0 },
  { "Conda", 0.06818182 },
  { "Condace", 0.0 },
  { "Conde", 1.0 },
  { "Condie", 1.0 },
  { "Condol", 0.0 },
  { "Condola", 0.0 },
  { "Condoleezza", 0.0 },
  { "Condon", 1.0 },
  { "Condra", 0.0 },
  { "Condredge", 1.0 },
  { "Condy", 1.0 },
  { "Conelia", 0.0 },
  { "Conell", 1.0 },
  { "Coner", 1.0 },
  { "Conesha", 0.0 },
  { "Coney", 1.0 },
  { "Conferina", 0.0 },
  { "Confesor", 1.0 },
  { "Cong", 1.0 },
  { "Congetta", 0.0 },
  { "Congress", 1.0 },
  { "Coni", 0.0 },
  { "Conie", 0.0 },
  { "Conisha", 0.0 },
  { "Conita", 0.0 },
  { "Conjetta", 0.0 },
  { "Conlan", 1.0 },
  { "Conlee", 0.6813187 },
  { "Conleigh", 0.0 },
  { "Conley", 0.93057024 },
  { "Conlin", 1.0 },
  { "Conlon", 1.0 },
  { "Conn", 1.0 },
  { "Conna", 0.0 },
  { "Connal", 1.0 },
  { "Connall", 1.0 },
  { "Connan", 1.0 },
  { "Connar", 0.9501608 },
  { "Connard", 1.0 },
  { "Conne", 0.0 },
  { "Connee", 0.0 },
  { "Connel", 1.0 },
  { "Connell", 0.99665326 },
  { "Connelly", 0.41588786 },
  { "Conner", 0.98477405 },
  { "Connery", 1.0 },
  { "Conney", 1.0 },
  { "Conni", 0.0 },
  { "Connice", 0.0 },
  { "Connie", 0.02545549 },
  { "Connis", 1.0 },
  { "Connley", 1.0 },
  { "Connolly", 0.7826087 },
  { "Connor", 0.99231774 },
  { "Connstance", 0.0 },
  { "Conny", 0.27670753 },
  { "Connye", 0.0 },
  { "Cono", 1.0 },
  { "Conola", 0.0 },
  { "Conor", 0.9967946 },
  { "Conquista", 0.0 },
  { "Conrad", 0.9995323 },
  { "Conrada", 0.0 },
  { "Conrado", 1.0 },
  { "Conred", 1.0 },
  { "Conroy", 1.0 },
  { "Consandra", 0.0 },
  { "Consepcion", 0.02189781 },
  { "Consetta", 0.0 },
  { "Consiglia", 0.0 },
  { "Constanc", 0.0 },
  { "Constance", 0.0030089552 },
  { "Constancia", 0.0 },
  { "Constancio", 1.0 },
  { "Constande", 0.0 },
  { "Constandina", 0.0 },
  { "Constandinos", 1.0 },
  { "Constant", 0.93157893 },
  { "Constantin", 1.0 },
  { "Constantina", 0.0 },
  { "Constantine", 0.9946918 },
  { "Constantino", 1.0 },
  { "Constantinos", 1.0 },
  { "Constanza", 0.0 },
  { "Constence", 0.0 },
  { "Constina", 0.0 },
  { "Consuela", 0.0 },
  { "Consuella", 0.0 },
  { "Consuello", 0.0 },
  { "Consuelo", 0.0053317538 },
  { "Conswala", 0.0 },
  { "Conswella", 0.0 },
  { "Conswello", 0.0 },
  { "Contance", 0.0 },
  { "Contenia", 0.0 },
  { "Content", 0.0 },
  { "Contesa", 0.0 },
  { "Contessa", 0.0 },
  { "Contessia", 0.0 },
  { "Contina", 0.0 },
  { "Continia", 0.0 },
  { "Contrell", 1.0 },
  { "Contrella", 0.0 },
  { "Contrena", 0.0 },
  { "Contrina", 0.0 },
  { "Converse", 1.0 },
  { "Conway", 1.0 },
  { "Conwell", 1.0 },
  { "Conya", 0.0 },
  { "Conye", 1.0 },
  { "Conzuelo", 0.0 },
  { "Conzy", 0.0 },
  { "Cookie", 0.0 },
  { "Cooledge", 1.0 },
  { "Cooley", 1.0 },
  { "Coolidge", 1.0 },
  { "Coopar", 1.0 },
  { "Cooper", 0.9796478 },
  { "Cope", 1.0 },
  { "Copelan", 1.0 },
  { "Copeland", 0.84502447 },
  { "Copelin", 1.0 },
  { "Copelyn", 0.0 },
  { "Copelynn", 0.0 },
  { "Copen", 1.0 },
  { "Copper", 0.7368421 },
  { "Cor", 1.0 },
  { "Cora", 0.0030041218 },
  { "Corabel", 0.0 },
  { "Corabell", 0.0 },
  { "Corabella", 0.0 },
  { "Corabelle", 0.0 },
  { "Corah", 0.0 },
  { "Coraima", 0.0 },
  { "Coraine", 0.0 },
  { "Coral", 0.01915555 },
  { "Coralai", 0.0 },
  { "Coralea", 0.0 },
  { "Coralee", 0.0 },
  { "Coraleen", 0.0 },
  { "Coralei", 0.0 },
  { "Coraleigh", 0.0 },
  { "Coralene", 0.0 },
  { "Coralia", 0.0 },
  { "Coralie", 0.0 },
  { "Coralin", 0.0 },
  { "Coralina", 0.0 },
  { "Coraline", 0.0 },
  { "Coralis", 0.0 },
  { "Coralito", 0.0 },
  { "Coraliz", 0.0 },
  { "Coralline", 0.0 },
  { "Coraly", 0.0 },
  { "Coralyn", 0.0 },
  { "Coralyne", 0.0 },
  { "Coralynn", 0.0 },
  { "Coralynne", 0.0 },
  { "Coralys", 0.0 },
  { "Coramae", 0.0 },
  { "Coran", 1.0 },
  { "Corando", 1.0 },
  { "Coray", 1.0 },
  { "Corayma", 0.0 },
  { "Corazon", 0.0 },
  { "Corban", 0.99802566 },
  { "Corben", 1.0 },
  { "Corbet", 1.0 },
  { "Corbett", 1.0 },
  { "Corbi", 0.0 },
  { "Corbie", 0.29411766 },
  { "Corbin", 0.9840507 },
  { "Corbit", 1.0 },
  { "Corbitt", 1.0 },
  { "Corbon", 1.0 },
  { "Corby", 0.81546897 },
  { "Corbyn", 0.8738375 },
  { "Corbynn", 0.0 },
  { "Cord", 1.0 },
  { "Corda", 0.020725388 },
  { "Cordae", 0.9898167 },
  { "Cordai", 1.0 },
  { "Cordairo", 1.0 },
  { "Cordale", 1.0 },
  { "Cordalro", 1.0 },
  { "Cordara", 1.0 },
  { "Cordarell", 1.0 },
  { "Cordarian", 1.0 },
  { "Cordario", 1.0 },
  { "Cordarious", 1.0 },
  { "Cordaris", 1.0 },
  { "Cordarius", 1.0 },
  { "Cordaro", 1.0 },
  { "Cordarrel", 1.0 },
  { "Cordarrell", 1.0 },
  { "Cordarrius", 1.0 },
  { "Cordarro", 1.0 },
  { "Cordarrow", 1.0 },
  { "Cordarryl", 1.0 },
  { "Cordaryl", 1.0 },
  { "Cordasia", 0.0 },
  { "Corday", 1.0 },
  { "Corddaryl", 1.0 },
  { "Corde", 0.9913644 },
  { "Cordea", 1.0 },
  { "Cordeja", 0.0 },
  { "Cordel", 1.0 },
  { "Cordelia", 0.0 },
  { "Cordell", 0.9854888 },
  { "Cordella", 0.0 },
  { "Cordelle", 0.9468085 },
  { "Cordellia", 0.0 },
  { "Cordelra", 1.0 },
  { "Cordelro", 1.0 },
  { "Corden", 1.0 },
  { "Cordera", 1.0 },
  { "Corderal", 1.0 },
  { "Corderall", 1.0 },
  { "Corderia", 1.0 },
  { "Corderial", 1.0 },
  { "Corderick", 1.0 },
  { "Corderio", 1.0 },
  { "Corderious", 1.0 },
  { "Corderius", 1.0 },
  { "Cordero", 0.99469966 },
  { "Corderra", 1.0 },
  { "Corderrius", 1.0 },
  { "Corderro", 1.0 },
  { "Cordes", 1.0 },
  { "Cordeshia", 0.0 },
  { "Cordey", 1.0 },
  { "Cordez", 1.0 },
  { "Cordia", 0.0 },
  { "Cordie", 0.07485432 },
  { "Cordilia", 0.0 },
  { "Cordin", 1.0 },
  { "Cordis", 1.0 },
  { "Cordney", 0.9405941 },
  { "Cordon", 1.0 },
  { "Cordula", 0.0 },
  { "Cordy", 0.43835616 },
  { "Corea", 0.0 },
  { "Corean", 0.0 },
  { "Coreatha", 0.0 },
  { "Coree", 0.29749104 },
  { "Coreen", 0.0 },
  { "Coreena", 0.0 },
  { "Coreene", 0.0 },
  { "Corell", 1.0 },
  { "Corella", 0.0 },
  { "Corelle", 0.0 },
  { "Coren", 0.5674603 },
  { "Corena", 0.0 },
  { "Corene", 0.0 },
  { "Corenia", 0.0 },
  { "Corenna", 0.0 },
  { "Corenne", 0.0 },
  { "Corenthia", 0.0 },
  { "Corentin", 1.0 },
  { "Coreon", 1.0 },
  { "Coreta", 0.0 },
  { "Coretha", 0.0 },
  { "Coretta", 0.0 },
  { "Corette", 0.0 },
  { "Corey", 0.9532697 },
  { "Coreyana", 0.0 },
  { "Coreyanna", 0.0 },
  { "Coreyion", 1.0 },
  { "Coreyon", 1.0 },
  { "Coreyona", 0.0 },
  { "Coreyonna", 0.0 },
  { "Corgan", 1.0 },
  { "Cori", 0.09356963 },
  { "Coriah", 0.0 },
  { "Corian", 0.9306931 },
  { "Coriana", 0.0 },
  { "Coriann", 0.0 },
  { "Corianna", 0.0 },
  { "Corianne", 0.0 },
  { "Corie", 0.22126816 },
  { "Corielle", 0.0 },
  { "Coriene", 0.0 },
  { "Corienne", 0.0 },
  { "Corieon", 1.0 },
  { "Corii", 0.0 },
  { "Corin", 0.33985624 },
  { "Corina", 0.0 },
  { "Corinda", 0.0 },
  { "Corine", 0.0011702348 },
  { "Corinn", 0.0 },
  { "Corinna", 0.0 },
  { "Corinne", 0.00031621553 },
  { "Corinth", 0.0 },
  { "Corintha", 0.0 },
  { "Corinthia", 0.0 },
  { "Corinthian", 0.7784027 },
  { "Corinthians", 1.0 },
  { "Corion", 1.0 },
  { "Corionna", 0.0 },
  { "Corisa", 0.0 },
  { "Corisha", 0.0 },
  { "Corissa", 0.0 },
  { "Corita", 0.0 },
  { "Coriyah", 0.0 },
  { "Corkey", 1.0 },
  { "Corky", 0.9229607 },
  { "Corla", 0.0 },
  { "Corleen", 0.0 },
  { "Corlene", 0.0 },
  { "Corleone", 1.0 },
  { "Corless", 0.0 },
  { "Corletta", 0.0 },
  { "Corlette", 0.0 },
  { "Corley", 0.46125463 },
  { "Corlin", 1.0 },
  { "Corlis", 0.062135924 },
  { "Corliss", 0.10748982 },
  { "Corlyn", 0.0 },
  { "Corma", 0.0 },
  { "Cormac", 1.0 },
  { "Cormack", 1.0 },
  { "Corman", 1.0 },
  { "Cormari", 1.0 },
  { "Cormarion", 1.0 },
  { "Cormick", 1.0 },
  { "Corneal", 0.8148148 },
  { "Cornealius", 1.0 },
  { "Corneall", 1.0 },
  { "Cornecia", 0.0 },
  { "Corneil", 1.0 },
  { "Corneilius", 1.0 },
  { "Corneilous", 1.0 },
  { "Corneilus", 1.0 },
  { "Corneisha", 0.0 },
  { "Cornel", 1.0 },
  { "Cornelia", 0.0011967228 },
  { "Cornelieus", 1.0 },
  { "Cornelio", 1.0 },
  { "Cornelious", 0.95621854 },
  { "Cornelis", 1.0 },
  { "Cornelius", 0.9822676 },
  { "Cornell", 0.96948975 },
  { "Cornella", 0.0 },
  { "Cornelle", 1.0 },
  { "Cornellia", 0.0 },
  { "Cornellius", 1.0 },
  { "Corneluis", 1.0 },
  { "Cornesha", 0.0 },
  { "Corneshia", 0.0 },
  { "Cornesia", 0.0 },
  { "Cornetta", 0.0 },
  { "Corney", 1.0 },
  { "Cornia", 0.0 },
  { "Cornie", 0.49411765 },
  { "Cornisha", 0.0 },
  { "Corniya", 0.0 },
  { "Corniyah", 0.0 },
  { "Corny", 1.0 },
  { "Corolyn", 0.0 },
  { "Coron", 1.0 },
  { "Corona", 0.0 },
  { "Coronda", 0.0 },
  { "Corra", 0.0 },
  { "Corrado", 1.0 },
  { "Corrah", 0.0 },
  { "Corraine", 0.0 },
  { "Corran", 1.0 },
  { "Correan", 0.0 },
  { "Corree", 0.0 },
  { "Correen", 0.0 },
  { "Correena", 0.0 },
  { "Correll", 1.0 },
  { "Corren", 0.16666667 },
  { "Correna", 0.0 },
  { "Correne", 0.0 },
  { "Correon", 1.0 },
  { "Corretta", 0.0 },
  { "Correy", 0.959256 },
  { "Corri", 0.038135592 },
  { "Corriana", 0.0 },
  { "Corrianna", 0.0 },
  { "Corrice", 1.0 },
  { "Corrida", 0.0 },
  { "Corrie", 0.13279848 },
  { "Corrigan", 0.83035713 },
  { "Corrin", 0.057599224 },
  { "Corrina", 0.0 },
  { "Corrinda", 0.0 },
  { "Corrine", 0.0 },
  { "Corrinn", 0.0 },
  { "Corrinna", 0.0 },
  { "Corrinne", 0.0 },
  { "Corrion", 1.0 },
  { "Corris", 0.62857145 },
  { "Corrisa", 0.0 },
  { "Corrissa", 0.0 },
  { "Corron", 1.0 },
  { "Corry", 0.87115306 },
  { "Corryn", 0.0 },
  { "Corryne", 0.0 },
  { "Corsandra", 0.0 },
  { "Corsha", 0.0 },
  { "Corsica", 0.0 },
  { "Corson", 1.0 },
  { "Cort", 1.0 },
  { "Cortana", 0.0 },
  { "Cortasia", 0.0 },
  { "Cortavia", 0.0 },
  { "Cortavion", 1.0 },
  { "Cortavious", 1.0 },
  { "Cortavius", 1.0 },
  { "Cortaz", 1.0 },
  { "Cortazia", 0.0 },
  { "Cortell", 1.0 },
  { "Corteney", 0.0 },
  { "Cortes", 1.0 },
  { "Cortez", 0.99098736 },
  { "Corteze", 1.0 },
  { "Cortina", 0.0 },
  { "Cortisha", 0.0 },
  { "Cortlan", 1.0 },
  { "Cortland", 0.983454 },
  { "Cortlandt", 1.0 },
  { "Cortlen", 1.0 },
  { "Cortlin", 1.0 },
  { "Cortlyn", 0.04373178 },
  { "Cortlynn", 0.0 },
  { "Cortne", 0.0 },
  { "Cortnee", 0.0 },
  { "Cortnei", 0.0 },
  { "Cortney", 0.15496695 },
  { "Cortni", 0.0 },
  { "Cortnie", 0.0 },
  { "Cortny", 0.0 },
  { "Cortrell", 1.0 },
  { "Corutney", 0.0 },
  { "Corvell", 1.0 },
  { "Corvetta", 0.0 },
  { "Corvette", 0.0 },
  { "Corvin", 1.0 },
  { "Corvina", 0.0 },
  { "Corvon", 1.0 },
  { "Corvus", 1.0 },
  { "Corvyn", 1.0 },
  { "Corwin", 1.0 },
  { "Corwyn", 1.0 },
  { "Corwynn", 1.0 },
  { "Cory", 0.95081794 },
  { "Coryana", 0.0 },
  { "Coryann", 0.0 },
  { "Corydon", 1.0 },
  { "Corye", 1.0 },
  { "Coryell", 1.0 },
  { "Coryion", 1.0 },
  { "Coryn", 0.0 },
  { "Coryna", 0.0 },
  { "Coryne", 0.0 },
  { "Corynn", 0.0 },
  { "Corynne", 0.0 },
  { "Coryon", 1.0 },
  { "Cosandra", 0.0 },
  { "Cosby", 0.9021277 },
  { "Cosetta", 0.0 },
  { "Cosette", 0.0 },
  { "Cosha", 0.0 },
  { "Cosie", 0.0 },
  { "Cosima", 0.0 },
  { "Cosimo", 1.0 },
  { "Cosma", 0.0 },
  { "Cosmas", 1.0 },
  { "Cosme", 1.0 },
  { "Cosmo", 1.0 },
  { "Cosmos", 1.0 },
  { "Cossandra", 0.0 },
  { "Cossette", 0.0 },
  { "Cossie", 1.0 },
  { "Costa", 1.0 },
  { "Costantino", 1.0 },
  { "Costanzo", 1.0 },
  { "Costas", 1.0 },
  { "Costella", 0.0 },
  { "Costello", 0.91803277 },
  { "Coston", 1.0 },
  { "Cote", 1.0 },
  { "Cotenia", 0.0 },
  { "Cotey", 1.0 },
  { "Cotha", 0.0 },
  { "Coti", 0.18333334 },
  { "Cotie", 0.4857143 },
  { "Cotina", 0.0 },
  { "Cotis", 1.0 },
  { "Cotrina", 0.0 },
  { "Cott", 1.0 },
  { "Cotter", 1.0 },
  { "Cottie", 0.0 },
  { "Cotton", 1.0 },
  { "Cottrell", 1.0 },
  { "Coty", 0.875969 },
  { "Coua", 0.0 },
  { "Cougar", 1.0 },
  { "Coula", 0.0 },
  { "Coulette", 0.0 },
  { "Coulson", 1.0 },
  { "Coulten", 1.0 },
  { "Coulter", 1.0 },
  { "Coulton", 1.0 },
  { "Coumba", 0.0 },
  { "Council", 1.0 },
  { "Count", 1.0 },
  { "Countess", 0.0 },
  { "Countney", 0.0 },
  { "Couper", 1.0 },
  { "Courage", 0.8926554 },
  { "Courey", 1.0 },
  { "Couri", 0.0 },
  { "Courney", 0.0 },
  { "Courntey", 0.0 },
  { "Court", 1.0 },
  { "Courtany", 0.0 },
  { "Courtenay", 0.17385258 },
  { "Courteney", 0.0044286977 },
  { "Courteny", 0.0 },
  { "Courtez", 1.0 },
  { "Courtlan", 1.0 },
  { "Courtland", 0.944474 },
  { "Courtlandt", 1.0 },
  { "Courtlin", 0.796875 },
  { "Courtlyn", 0.013717421 },
  { "Courtlynn", 0.0 },
  { "Courtnay", 0.02096436 },
  { "Courtne", 0.0 },
  { "Courtnee", 0.0049474337 },
  { "Courtnei", 0.0 },
  { "Courtney", 0.08061614 },
  { "Courtneyann", 0.0 },
  { "Courtni", 0.0 },
  { "Courtnie", 0.0 },
  { "Courtny", 0.00990099 },
  { "Courtnye", 0.0 },
  { "Courvoisier", 1.0 },
  { "Coury", 1.0 },
  { "Coutney", 0.0 },
  { "Couture", 0.0 },
  { "Couy", 1.0 },
  { "Cova", 0.046296295 },
  { "Cove", 0.93639576 },
  { "Coven", 1.0 },
  { "Covert", 1.0 },
  { "Covey", 0.9632353 },
  { "Covie", 0.0 },
  { "Covin", 1.0 },
  { "Covington", 1.0 },
  { "Covy", 1.0 },
  { "Cowan", 1.0 },
  { "Cowana", 0.0 },
  { "Cowanda", 0.0 },
  { "Cowen", 1.0 },
  { "Coy", 0.9493952 },
  { "Coya", 0.0 },
  { "Coyal", 1.0 },
  { "Coye", 0.90625 },
  { "Coyer", 1.0 },
  { "Coyla", 0.0 },
  { "Coyle", 1.0 },
  { "Coyote", 1.0 },
  { "Coyt", 1.0 },
  { "Coyte", 1.0 },
  { "Cozell", 1.0 },
  { "Cozella", 0.0 },
  { "Cozetta", 0.0 },
  { "Cozette", 0.0 },
  { "Cozie", 0.0 },
  { "Cozmo", 1.0 },
  { "Cozy", 0.17213115 },
  { "Crae", 1.0 },
  { "Crafton", 1.0 },
  { "Crag", 1.0 },
  { "Cragi", 1.0 },
  { "Craig", 0.9966197 },
  { "Craige", 1.0 },
  { "Craigory", 1.0 },
  { "Cramer", 1.0 },
  { "Cramon", 1.0 },
  { "Crandall", 1.0 },
  { "Crandon", 1.0 },
  { "Crane", 1.0 },
  { "Cranford", 1.0 },
  { "Cranston", 1.0 },
  { "Crash", 1.0 },
  { "Crate", 1.0 },
  { "Craven", 1.0 },
  { "Crawford", 0.99546105 },
  { "Crawley", 1.0 },
  { "Cray", 1.0 },
  { "Crayson", 1.0 },
  { "Crayton", 1.0 },
  { "Crea", 0.0 },
  { "Cread", 1.0 },
  { "Creasie", 0.0 },
  { "Crecencio", 1.0 },
  { "Creda", 0.0 },
  { "Credell", 1.0 },
  { "Credence", 0.6666667 },
  { "Cree", 0.35128418 },
  { "Creed", 1.0 },
  { "Creedan", 1.0 },
  { "Creedance", 0.41666666 },
  { "Creede", 1.0 },
  { "Creeden", 1.0 },
  { "Creedence", 0.7 },
  { "Creedon", 1.0 },
  { "Creedyn", 1.0 },
  { "Creek", 1.0 },
  { "Creel", 1.0 },
  { "Creg", 1.0 },
  { "Cregg", 1.0 },
  { "Creig", 1.0 },
  { "Creigh", 1.0 },
  { "Creighton", 0.9968088 },
  { "Creo", 1.0 },
  { "Creola", 0.0 },
  { "Creolia", 0.0 },
  { "Creon", 1.0 },
  { "Creosha", 0.0 },
  { "Crescencia", 0.0 },
  { "Crescencio", 1.0 },
  { "Crescent", 0.0 },
  { "Crescentia", 0.0 },
  { "Crescenzo", 1.0 },
  { "Cresencia", 0.0 },
  { "Cresenciano", 1.0 },
  { "Cresencio", 1.0 },
  { "Cresha", 0.0 },
  { "Creshawn", 1.0 },
  { "Cresie", 0.0 },
  { "Crespin", 1.0 },
  { "Cress", 1.0 },
  { "Cressa", 0.0 },
  { "Cressida", 0.0 },
  { "Cressie", 0.0 },
  { "Cresta", 0.0 },
  { "Crestina", 0.0 },
  { "Creston", 1.0 },
  { "Creta", 0.0 },
  { "Crete", 0.0 },
  { "Cretia", 0.0 },
  { "Crettie", 0.0 },
  { "Crew", 0.9981825 },
  { "Crewe", 1.0 },
  { "Crews", 1.0 },
  { "Creyton", 1.0 },
  { "Crhistopher", 1.0 },
  { "Criag", 1.0 },
  { "Crichton", 1.0 },
  { "Cricket", 0.0 },
  { "Crickett", 0.0 },
  { "Crimson", 0.24873525 },
  { "Crimsyn", 0.0 },
  { "Cris", 0.725599 },
  { "Crisangel", 1.0 },
  { "Crisann", 0.0 },
  { "Crisanna", 0.0 },
  { "Crisanta", 0.0 },
  { "Crisanto", 1.0 },
  { "Crisbel", 0.0 },
  { "Criscia", 0.0 },
  { "Crisel", 0.0 },
  { "Criselda", 0.0 },
  { "Criselle", 0.0 },
  { "Crisette", 0.0 },
  { "Crisha", 0.0 },
  { "Crishaun", 1.0 },
  { "Crishawn", 1.0 },
  { "Crishon", 1.0 },
  { "Crisinda", 0.0 },
  { "Crislyn", 0.0 },
  { "Crislynn", 0.0 },
  { "Crisma", 0.0 },
  { "Crisoforo", 1.0 },
  { "Crisol", 0.0 },
  { "Crispin", 0.9970291 },
  { "Crispina", 0.0 },
  { "Crispus", 1.0 },
  { "Criss", 0.82985073 },
  { "Crissa", 0.0 },
  { "Crissandra", 0.0 },
  { "Crissangel", 1.0 },
  { "Crissey", 0.0 },
  { "Crissi", 0.0 },
  { "Crissie", 0.0 },
  { "Crisslyn", 0.0 },
  { "Crisstopher", 1.0 },
  { "Crissy", 0.0 },
  { "Crist", 1.0 },
  { "Crista", 0.0 },
  { "Cristabel", 0.0 },
  { "Cristabella", 0.0 },
  { "Cristain", 1.0 },
  { "Cristal", 0.0035723564 },
  { "Cristalina", 0.0 },
  { "Cristalle", 0.0 },
  { "Cristaly", 0.0 },
  { "Cristalyn", 0.0 },
  { "Cristan", 0.44077134 },
  { "Cristee", 0.0 },
  { "Cristeen", 0.0 },
  { "Cristel", 0.0 },
  { "Cristela", 0.0 },
  { "Cristell", 0.0 },
  { "Cristella", 0.0 },
  { "Cristelle", 0.0 },
  { "Cristen", 0.045487363 },
  { "Cristena", 0.0 },
  { "Cristene", 0.0 },
  { "Cristhian", 1.0 },
  { "Cristhofer", 1.0 },
  { "Cristi", 0.0 },
  { "Cristia", 0.0 },
  { "Cristian", 0.98189116 },
  { "Cristiana", 0.0 },
  { "Cristianna", 0.0 },
  { "Cristiano", 1.0 },
  { "Cristie", 0.0 },
  { "Cristien", 1.0 },
  { "Cristin", 0.027422303 },
  { "Cristina", 0.005280141 },
  { "Cristine", 0.0 },
  { "Cristinia", 0.0 },
  { "Cristino", 1.0 },
  { "Cristion", 1.0 },
  { "Cristle", 0.0 },
  { "Cristo", 1.0 },
  { "Cristobal", 0.99840903 },
  { "Cristofer", 1.0 },
  { "Cristoffer", 1.0 },
  { "Cristofher", 1.0 },
  { "Cristol", 0.0 },
  { "Criston", 0.8362069 },
  { "Cristopher", 1.0 },
  { "Cristos", 1.0 },
  { "Cristoval", 1.0 },
  { "Cristy", 0.006908463 },
  { "Cristyan", 1.0 },
  { "Cristyl", 0.0 },
  { "Cristyle", 0.0 },
  { "Cristyn", 0.0 },
  { "Cristyna", 0.0 },
  { "Crit", 1.0 },
  { "Crixus", 1.0 },
  { "Criztian", 1.0 },
  { "Crmen", 0.0 },
  { "Crockett", 1.0 },
  { "Croix", 1.0 },
  { "Crol", 0.0 },
  { "Cromer", 1.0 },
  { "Cromwell", 1.0 },
  { "Crosbie", 0.0 },
  { "Crosby", 0.9168183 },
  { "Crosley", 0.6851852 },
  { "Cross", 1.0 },
  { "Crossley", 1.0 },
  { "Crosslyn", 0.0 },
  { "Crowell", 1.0 },
  { "Crowley", 1.0 },
  { "Crown", 1.0 },
  { "Croy", 1.0 },
  { "Crstal", 0.0 },
  { "Cru", 1.0 },
  { "Crucita", 0.0 },
  { "Crue", 1.0 },
  { "Cruise", 1.0 },
  { "Cruiz", 1.0 },
  { "Cruize", 1.0 },
  { "Crus", 1.0 },
  { "Cruse", 1.0 },
  { "Crusita", 0.0 },
  { "Crusoe", 1.0 },
  { "Crustal", 0.0 },
  { "Cruz", 0.840729 },
  { "Cruze", 1.0 },
  { "Cruzita", 0.0 },
  { "Cruzito", 1.0 },
  { "Crys", 0.0 },
  { "Crysal", 0.0 },
  { "Crysania", 0.0 },
  { "Crysany", 0.0 },
  { "Crysta", 0.0 },
  { "Crystabel", 0.0 },
  { "Crystal", 0.0035843595 },
  { "Crystalann", 0.0 },
  { "Crystale", 0.0 },
  { "Crystalee", 0.0 },
  { "Crystalgayle", 0.0 },
  { "Crystalin", 0.0 },
  { "Crystalina", 0.0 },
  { "Crystaline", 0.0 },
  { "Crystall", 0.0 },
  { "Crystalle", 0.0 },
  { "Crystallee", 0.0 },
  { "Crystallynn", 0.0 },
  { "Crystalmarie", 0.0 },
  { "Crystalrose", 0.0 },
  { "Crystalyn", 0.0 },
  { "Crystalynn", 0.0 },
  { "Crystan", 0.0 },
  { "Crystel", 0.0 },
  { "Crystell", 0.0 },
  { "Crystella", 0.0 },
  { "Crystelle", 0.0 },
  { "Crysten", 0.0 },
  { "Crysti", 0.0 },
  { "Crystian", 1.0 },
  { "Crystiana", 0.0 },
  { "Crystianna", 0.0 },
  { "Crystie", 0.0 },
  { "Crystil", 0.0 },
  { "Crystin", 0.0 },
  { "Crystina", 0.0 },
  { "Crystine", 0.0 },
  { "Crystl", 0.0 },
  { "Crystle", 0.0 },
  { "Crystofer", 1.0 },
  { "Crystol", 0.0 },
  { "Crystopher", 1.0 },
  { "Crystral", 0.0 },
  { "Crystyl", 0.0 },
  { "Crystyle", 0.0 },
  { "Crytal", 0.0 },
  { "Csilla", 0.0 },
  { "Cuahtemoc", 1.0 },
  { "Cuahutemoc", 1.0 },
  { "Cuauhtemoc", 1.0 },
  { "Cuauhtli", 1.0 },
  { "Cub", 1.0 },
  { "Cuba", 0.07679739 },
  { "Cubby", 1.0 },
  { "Cubert", 1.0 },
  { "Cubia", 0.0 },
  { "Cubie", 0.2037037 },
  { "Cuca", 0.0 },
  { "Cuinn", 1.0 },
  { "Cuitlahuac", 1.0 },
  { "Cula", 0.0 },
  { "Cullan", 1.0 },
  { "Culleen", 0.0 },
  { "Cullen", 0.99225545 },
  { "Culley", 1.0 },
  { "Cullie", 1.0 },
  { "Cullin", 1.0 },
  { "Cullus", 1.0 },
  { "Cully", 1.0 },
  { "Cullyn", 1.0 },
  { "Culver", 1.0 },
  { "Cuma", 0.0 },
  { "Cumi", 0.0 },
  { "Cung", 1.0 },
  { "Cuong", 1.0 },
  { "Cupertino", 1.0 },
  { "Cupid", 1.0 },
  { "Cura", 0.0 },
  { "Curby", 1.0 },
  { "Curie", 0.0 },
  { "Curissa", 0.0 },
  { "Curits", 1.0 },
  { "Curl", 1.0 },
  { "Curlee", 0.8971292 },
  { "Curlene", 0.0 },
  { "Curley", 0.8290653 },
  { "Curlie", 0.32457787 },
  { "Curly", 0.9745763 },
  { "Curran", 0.97275203 },
  { "Curren", 1.0 },
  { "Currie", 0.78313255 },
  { "Currin", 1.0 },
  { "Curron", 1.0 },
  { "Curry", 0.9464398 },
  { "Curstin", 0.0 },
  { "Curstyn", 0.0 },
  { "Curt", 1.0 },
  { "Curtavious", 1.0 },
  { "Curteria", 0.0 },
  { "Curtesha", 0.0 },
  { "Curtez", 1.0 },
  { "Curtia", 0.0 },
  { "Curtice", 1.0 },
  { "Curties", 1.0 },
  { "Curtina", 0.0 },
  { "Curtis", 0.99106354 },
  { "Curtisa", 0.0 },
  { "Curtisha", 0.0 },
  { "Curtiss", 1.0 },
  { "Curtissa", 0.0 },
  { "Curtistine", 0.0 },
  { "Curtney", 0.0 },
  { "Curtrina", 0.0 },
  { "Curtus", 1.0 },
  { "Curvin", 1.0 },
  { "Cushena", 0.0 },
  { "Custer", 1.0 },
  { "Custodio", 1.0 },
  { "Cutberto", 1.0 },
  { "Cuthbert", 1.0 },
  { "Cutina", 0.0 },
  { "Cutler", 1.0 },
  { "Cutter", 1.0 },
  { "Cuyler", 0.9842767 },
  { "Cy", 0.99525505 },
  { "Cya", 0.0 },
  { "Cyaira", 0.0 },
  { "Cyaire", 1.0 },
  { "Cyan", 0.16999133 },
  { "Cyana", 0.0 },
  { "Cyane", 0.0 },
  { "Cyani", 0.0 },
  { "Cyann", 0.0 },
  { "Cyanna", 0.0 },
  { "Cyanne", 0.0 },
  { "Cyanni", 0.0 },
  { "Cyanthia", 0.0 },
  { "Cyara", 0.0 },
  { "Cyarah", 0.0 },
  { "Cyarra", 0.0 },
  { "Cyasia", 0.0 },
  { "Cybele", 0.0 },
  { "Cybelle", 0.0 },
  { "Cybil", 0.0 },
  { "Cybill", 0.0 },
  { "Cyd", 0.0 },
  { "Cydne", 0.0 },
  { "Cydnee", 0.0 },
  { "Cydnei", 0.0 },
  { "Cydney", 0.0050301813 },
  { "Cydni", 0.0 },
  { "Cydnie", 0.0 },
  { "Cydny", 0.0 },
  { "Cye", 1.0 },
  { "Cyenna", 0.0 },
  { "Cyenthia", 0.0 },
  { "Cyera", 0.0 },
  { "Cyerah", 0.0 },
  { "Cyerra", 0.0 },
  { "Cyinthia", 0.0 },
  { "Cyion", 1.0 },
  { "Cyla", 0.0 },
  { "Cylah", 0.0 },
  { "Cylan", 1.0 },
  { "Cylar", 1.0 },
  { "Cylas", 1.0 },
  { "Cyle", 0.9923352 },
  { "Cylee", 0.0 },
  { "Cyleigh", 0.0 },
  { "Cylen", 1.0 },
  { "Cylena", 0.0 },
  { "Cyler", 0.9907579 },
  { "Cylie", 0.0 },
  { "Cylin", 1.0 },
  { "Cylinda", 0.0 },
  { "Cylis", 1.0 },
  { "Cylus", 1.0 },
  { "Cylvia", 0.0 },
  { "Cymande", 1.0 },
  { "Cymantha", 0.0 },
  { "Cymbre", 0.0 },
  { "Cymon", 1.0 },
  { "Cymone", 0.0 },
  { "Cymphonique", 0.0 },
  { "Cymphony", 0.0 },
  { "Cynae", 0.0 },
  { "Cynai", 0.0 },
  { "Cynamon", 0.0 },
  { "Cynara", 0.0 },
  { "Cynarra", 0.0 },
  { "Cyncere", 0.65609753 },
  { "Cynda", 0.0 },
  { "Cyndal", 0.0 },
  { "Cynde", 0.0 },
  { "Cyndee", 0.0 },
  { "Cyndel", 0.0 },
  { "Cyndi", 0.0 },
  { "Cyndia", 0.0 },
  { "Cyndie", 0.0 },
  { "Cyndil", 0.0 },
  { "Cyndle", 0.0 },
  { "Cyndra", 0.0 },
  { "Cyndy", 0.0 },
  { "Cynequa", 0.0 },
  { "Cynetha", 0.0 },
  { "Cynethia", 0.0 },
  { "Cynia", 0.0 },
  { "Cyniah", 0.0 },
  { "Cynii", 0.0 },
  { "Cynita", 0.0 },
  { "Cynitha", 0.0 },
  { "Cynithia", 0.0 },
  { "Cyniya", 0.0 },
  { "Cyniyah", 0.0 },
  { "Cynnamon", 0.0 },
  { "Cynne", 0.0 },
  { "Cynnthia", 0.0 },
  { "Cynteria", 0.0 },
  { "Cyntha", 0.0 },
  { "Cynthea", 0.0 },
  { "Cyntheia", 0.0 },
  { "Cynthi", 0.0 },
  { "Cynthia", 0.0027609956 },
  { "Cynthiaa", 0.0 },
  { "Cynthiaann", 0.0 },
  { "Cynthie", 0.0 },
  { "Cynthina", 0.0 },
  { "Cynthis", 0.0 },
  { "Cynthnia", 0.0 },
  { "Cynthya", 0.0 },
  { "Cyntia", 0.0 },
  { "Cyntoria", 0.0 },
  { "Cyntrell", 0.0 },
  { "Cyon", 1.0 },
  { "Cyonna", 0.0 },
  { "Cypher", 1.0 },
  { "Cypress", 0.4497768 },
  { "Cyprian", 1.0 },
  { "Cyprus", 0.91525424 },
  { "Cyra", 0.0 },
  { "Cyrah", 0.0 },
  { "Cyrano", 1.0 },
  { "Cyre", 0.0 },
  { "Cyree", 1.0 },
  { "Cyrell", 1.0 },
  { "Cyrelle", 0.0 },
  { "Cyrena", 0.0 },
  { "Cyrene", 0.0 },
  { "Cyrenity", 0.0 },
  { "Cyrenna", 0.0 },
  { "Cyress", 1.0 },
  { "Cyria", 0.0 },
  { "Cyriah", 0.0 },
  { "Cyriana", 0.0 },
  { "Cyric", 1.0 },
  { "Cyrie", 1.0 },
  { "Cyriel", 1.0 },
  { "Cyrielle", 0.0 },
  { "Cyril", 0.99173975 },
  { "Cyrill", 1.0 },
  { "Cyrilla", 0.0 },
  { "Cyrille", 0.8064516 },
  { "Cyrina", 0.0 },
  { "Cyrine", 0.0 },
  { "Cyris", 1.0 },
  { "Cyrita", 0.0 },
  { "Cyron", 1.0 },
  { "Cyrstal", 0.0 },
  { "Cyrus", 0.9997531 },
  { "Cyruss", 1.0 },
  { "Cyson", 1.0 },
  { "Cystal", 0.0 },
  { "Cythia", 0.0 },
  { "Cythina", 0.0 },
  { "Cythnia", 0.0 },
  { "Cytlali", 0.0 },
  { "Cytlalli", 0.0 },
  { "Cytlaly", 0.0 },
  { "Cytnhia", 0.0 },
  { "Czar", 1.0 },
  { "Czaria", 0.0 },
  { "Czarina", 0.0 },
  { "Czeslaw", 1.0 },
  { "Czeslawa", 0.0 },
  { "Da", 0.615894 },
  { "Daaiel", 1.0 },
  { "Daaimah", 0.0 },
  { "Daaiyah", 0.0 },
  { "Daan", 1.0 },
  { "Daana", 0.0 },
  { "Daandre", 1.0 },
  { "Daania", 0.0 },
  { "Daanish", 1.0 },
  { "Daanya", 0.0 },
  { "Daanyal", 1.0 },
  { "Daarina", 0.0 },
  { "Daaron", 1.0 },
  { "Daasia", 0.0 },
  { "Dabid", 1.0 },
  { "Dabin", 1.0 },
  { "Dabne", 0.0 },
  { "Dabney", 0.38410595 },
  { "Dabria", 0.0 },
  { "Dabriel", 1.0 },
  { "Dabrielle", 0.0 },
  { "Dabriyon", 1.0 },
  { "Dacari", 1.0 },
  { "Dacarri", 1.0 },
  { "Dace", 0.9564165 },
  { "Dacee", 0.0 },
  { "Dacen", 1.0 },
  { "Dacey", 0.0 },
  { "Dache", 0.0 },
  { "Dachelle", 0.0 },
  { "Dachia", 0.0 },
  { "Daci", 0.0 },
  { "Dacia", 0.0 },
  { "Dacian", 1.0 },
  { "Daciana", 0.0 },
  { "Dacie", 0.035460994 },
  { "Dacien", 1.0 },
  { "Dacio", 1.0 },
  { "Dacion", 1.0 },
  { "Dack", 1.0 },
  { "Dacoda", 0.81931466 },
  { "Dacorey", 1.0 },
  { "Dacorian", 1.0 },
  { "Dacorion", 1.0 },
  { "Dacota", 0.7267267 },
  { "Dacotah", 0.7037037 },
  { "Dacy", 0.0 },
  { "Daddy", 1.0 },
  { "Dade", 1.0 },
  { "Daden", 1.0 },
  { "Dadne", 0.0 },
  { "Dadra", 0.0 },
  { "Dadrian", 1.0 },
  { "Dadriana", 0.0 },
  { "Dadrianna", 0.0 },
  { "Dadrien", 1.0 },
  { "Dae", 0.65 },
  { "Daebreon", 0.2777778 },
  { "Daebreona", 0.0 },
  { "Daedalus", 1.0 },
  { "Daedra", 0.0 },
  { "Daedrian", 1.0 },
  { "Daedric", 1.0 },
  { "Daegan", 0.9408867 },
  { "Daegen", 1.0 },
  { "Daegon", 1.0 },
  { "Daehan", 1.0 },
  { "Daeisha", 0.0 },
  { "Daeja", 0.0 },
  { "Daejah", 0.0 },
  { "Daejanae", 0.0 },
  { "Daejanay", 0.0 },
  { "Daejaun", 1.0 },
  { "Daejha", 0.0 },
  { "Daejhun", 1.0 },
  { "Daejia", 0.0 },
  { "Daejion", 1.0 },
  { "Daejohn", 1.0 },
  { "Daejon", 1.0 },
  { "Daejuan", 1.0 },
  { "Daekwan", 1.0 },
  { "Daekwon", 1.0 },
  { "Dael", 1.0 },
  { "Daela", 0.0 },
  { "Daelan", 1.0 },
  { "Daelani", 0.0 },
  { "Daelen", 1.0 },
  { "Daelin", 0.8705584 },
  { "Daelon", 1.0 },
  { "Daely", 0.0 },
  { "Daelyn", 0.317662 },
  { "Daelynn", 0.030567685 },
  { "Daemen", 1.0 },
  { "Daemeon", 1.0 },
  { "Daemian", 1.0 },
  { "Daemien", 1.0 },
  { "Daemion", 1.0 },
  { "Daemon", 1.0 },
  { "Daemond", 1.0 },
  { "Daemyn", 1.0 },
  { "Daena", 0.0 },
  { "Daenerys", 0.0 },
  { "Daenna", 0.0 },
  { "Daeon", 1.0 },
  { "Daeonna", 0.0 },
  { "Daequan", 1.0 },
  { "Daequon", 1.0 },
  { "Daeqwon", 1.0 },
  { "Daerion", 1.0 },
  { "Daeron", 1.0 },
  { "Daesean", 1.0 },
  { "Daesha", 0.0 },
  { "Daeshaun", 1.0 },
  { "Daeshawn", 0.99338627 },
  { "Daeshawna", 0.0 },
  { "Daeshia", 0.0 },
  { "Daeshon", 1.0 },
  { "Daeshun", 1.0 },
  { "Daesia", 0.0 },
  { "Daeson", 1.0 },
  { "Daesy", 0.0 },
  { "Daeton", 1.0 },
  { "Daeun", 0.0 },
  { "Daevaughn", 1.0 },
  { "Daeveon", 1.0 },
  { "Daevian", 1.0 },
  { "Daevin", 1.0 },
  { "Daevion", 1.0 },
  { "Daevon", 1.0 },
  { "Daevyn", 1.0 },
  { "Daewon", 1.0 },
  { "Daeyon", 1.0 },
  { "Daeyonna", 0.0 },
  { "Daezha", 0.0 },
  { "Daezon", 1.0 },
  { "Daffany", 0.0 },
  { "Daffie", 0.0 },
  { "Daffne", 0.0 },
  { "Daffney", 0.0 },
  { "Daffy", 0.0 },
  { "Dafina", 0.0 },
  { "Dafna", 0.0 },
  { "Dafne", 0.0 },
  { "Dafnee", 0.0 },
  { "Dafney", 0.0 },
  { "Dafni", 0.0 },
  { "Dafnne", 0.0 },
  { "Dafny", 0.0 },
  { "Dag", 1.0 },
  { "Dagan", 0.9902344 },
  { "Dagem", 1.0 },
  { "Dagen", 0.91768295 },
  { "Dagger", 1.0 },
  { "Dagim", 1.0 },
  { "Dagmar", 0.0 },
  { "Dagmara", 0.0 },
  { "Dagmawi", 1.0 },
  { "Dagne", 0.0 },
  { "Dagney", 0.0 },
  { "Dagny", 0.0 },
  { "Dago", 1.0 },
  { "Dagoberto", 1.0 },
  { "Dagon", 1.0 },
  { "Daguan", 1.0 },
  { "Dah", 0.5405405 },
  { "Dahabo", 0.0 },
  { "Dahana", 0.0 },
  { "Dahani", 1.0 },
  { "Dahiana", 0.0 },
  { "Dahianna", 0.0 },
  { "Dahir", 1.0 },
  { "Dahja", 0.0 },
  { "Dahl", 1.0 },
  { "Dahlia", 0.0 },
  { "Dahliah", 0.0 },
  { "Dahliana", 0.0 },
  { "Dahlila", 0.0 },
  { "Dahlilah", 0.0 },
  { "Dahlin", 1.0 },
  { "Dahlon", 1.0 },
  { "Dahlton", 1.0 },
  { "Dahlya", 0.0 },
  { "Dahlyla", 0.0 },
  { "Dahmari", 1.0 },
  { "Dahmir", 1.0 },
  { "Dahn", 0.23809524 },
  { "Dahna", 0.0 },
  { "Dahndray", 1.0 },
  { "Dahntay", 1.0 },
  { "Dahnya", 0.0 },
  { "Dahquan", 1.0 },
  { "Dai", 0.65591395 },
  { "Daia", 0.0 },
  { "Daiah", 0.0 },
  { "Daian", 1.0 },
  { "Daiana", 0.0 },
  { "Daianera", 0.0 },
  { "Daianna", 0.0 },
  { "Daiceon", 1.0 },
  { "Daichi", 1.0 },
  { "Daicy", 0.0 },
  { "Daid", 1.0 },
  { "Daiden", 1.0 },
  { "Daidra", 0.0 },
  { "Daidre", 0.0 },
  { "Daidrian", 1.0 },
  { "Daiel", 1.0 },
  { "Daielle", 0.0 },
  { "Daigan", 1.0 },
  { "Daigen", 1.0 },
  { "Daigo", 1.0 },
  { "Daigon", 1.0 },
  { "Daiguan", 1.0 },
  { "Daija", 0.0 },
  { "Daijah", 0.0 },
  { "Daijana", 0.0 },
  { "Daijanae", 0.0 },
  { "Daijanay", 0.0 },
  { "Daijaun", 1.0 },
  { "Daijha", 0.0 },
  { "Daijia", 0.0 },
  { "Daijiah", 0.0 },
  { "Daijohn", 1.0 },
  { "Daijon", 0.99240124 },
  { "Daijour", 1.0 },
  { "Daijsha", 0.0 },
  { "Daijuan", 1.0 },
  { "Daiki", 1.0 },
  { "Daikwon", 1.0 },
  { "Dail", 0.7595357 },
  { "Daila", 0.0 },
  { "Dailah", 0.0 },
  { "Dailan", 1.0 },
  { "Dailani", 0.0 },
  { "Dailany", 0.0 },
  { "Dailee", 0.0 },
  { "Daileen", 0.0 },
  { "Daileigh", 0.0 },
  { "Dailen", 0.98452014 },
  { "Dailene", 0.0 },
  { "Dailey", 0.36042404 },
  { "Dailin", 0.21666667 },
  { "Dailon", 1.0 },
  { "Daily", 0.33031675 },
  { "Dailyn", 0.22653429 },
  { "Dailynn", 0.035842296 },
  { "Dailynne", 0.0 },
  { "Daiman", 1.0 },
  { "Daimar", 1.0 },
  { "Daimarely", 0.0 },
  { "Daimarion", 1.0 },
  { "Daimen", 1.0 },
  { "Daimeon", 1.0 },
  { "Daimian", 1.0 },
  { "Daimien", 1.0 },
  { "Daimion", 1.0 },
  { "Daimon", 1.0 },
  { "Daimond", 0.0 },
  { "Daimyan", 1.0 },
  { "Dain", 1.0 },
  { "Daina", 0.0020508613 },
  { "Dainan", 1.0 },
  { "Dainara", 0.0 },
  { "Daine", 0.5 },
  { "Dainel", 1.0 },
  { "Dainelle", 0.0 },
  { "Dainen", 1.0 },
  { "Dainera", 0.0 },
  { "Dainette", 0.0 },
  { "Dainian", 1.0 },
  { "Dainna", 0.0 },
  { "Dainon", 1.0 },
  { "Daintry", 0.0 },
  { "Dainty", 0.0 },
  { "Daion", 1.0 },
  { "Daiona", 0.0 },
  { "Daionna", 0.0 },
  { "Daiquan", 1.0 },
  { "Daiquiri", 0.0 },
  { "Daiquon", 1.0 },
  { "Dair", 1.0 },
  { "Daira", 0.0 },
  { "Daire", 0.92771083 },
  { "Dairel", 1.0 },
  { "Dairen", 1.0 },
  { "Dairian", 0.0 },
  { "Dairin", 0.0 },
  { "Dairion", 1.0 },
  { "Dairius", 1.0 },
  { "Dairl", 1.0 },
  { "Dairo", 1.0 },
  { "Dairon", 1.0 },
  { "Dairra", 0.0 },
  { "Dairus", 1.0 },
  { "Dairy", 0.0 },
  { "Dairyn", 0.0 },
  { "Daisa", 0.0 },
  { "Daise", 0.0 },
  { "Daisean", 1.0 },
  { "Daisee", 0.0 },
  { "Daisey", 0.0 },
  { "Daisha", 0.0 },
  { "Daishae", 0.0 },
  { "Daishana", 0.0 },
  { "Daishanae", 0.0 },
  { "Daishanay", 0.0 },
  { "Daishaun", 1.0 },
  { "Daishawn", 1.0 },
  { "Daishia", 0.0 },
  { "Daishon", 1.0 },
  { "Daishun", 1.0 },
  { "Daisi", 0.0 },
  { "Daisia", 0.0 },
  { "Daisie", 0.0 },
  { "Daisja", 0.0 },
  { "Daisjah", 0.0 },
  { "Daison", 1.0 },
  { "Daissy", 0.0 },
  { "Daisuke", 1.0 },
  { "Daisy", 0.004504764 },
  { "Daisya", 0.0 },
  { "Daisye", 0.0 },
  { "Daisylynn", 0.0 },
  { "Daisymae", 0.0 },
  { "Daisymarie", 0.0 },
  { "Daiten", 1.0 },
  { "Daithi", 1.0 },
  { "Daiton", 1.0 },
  { "Daityn", 0.0 },
  { "Daiva", 0.0 },
  { "Daivd", 1.0 },
  { "Daiven", 1.0 },
  { "Daiveon", 1.0 },
  { "Daivian", 1.0 },
  { "Daivid", 1.0 },
  { "Daivik", 1.0 },
  { "Daivion", 1.0 },
  { "Daivon", 1.0 },
  { "Daivyon", 1.0 },
  { "Daiwik", 1.0 },
  { "Daiya", 0.0 },
  { "Daiyaan", 1.0 },
  { "Daiyah", 0.0 },
  { "Daiyan", 1.0 },
  { "Daiyana", 0.0 },
  { "Daiyon", 1.0 },
  { "Daiza", 0.0 },
  { "Daizah", 0.0 },
  { "Daizee", 0.0 },
  { "Daizha", 0.0 },
  { "Daizhane", 0.0 },
  { "Daizhaun", 1.0 },
  { "Daizhon", 1.0 },
  { "Daizia", 0.0 },
  { "Daizie", 0.0 },
  { "Daizja", 0.0 },
  { "Daizjah", 0.0 },
  { "Daizon", 1.0 },
  { "Daizsa", 0.0 },
  { "Daizy", 0.0 },
  { "Daja", 0.0 },
  { "Dajae", 0.0 },
  { "Dajah", 0.003567788 },
  { "Dajaha", 0.0 },
  { "Dajahn", 1.0 },
  { "Dajahnae", 0.0 },
  { "Dajahnay", 0.0 },
  { "Dajai", 0.0 },
  { "Dajan", 1.0 },
  { "Dajana", 0.0 },
  { "Dajanae", 0.0 },
  { "Dajanai", 0.0 },
  { "Dajanay", 0.0 },
  { "Dajane", 0.0 },
  { "Dajanea", 0.0 },
  { "Dajanee", 0.0 },
  { "Dajanique", 0.0 },
  { "Dajaun", 1.0 },
  { "Dajauna", 0.0 },
  { "Dajavette", 0.0 },
  { "Daje", 0.0 },
  { "Dajea", 0.0 },
  { "Dajean", 1.0 },
  { "Dajee", 0.0 },
  { "Dajha", 0.0 },
  { "Dajhia", 0.0 },
  { "Dajhon", 1.0 },
  { "Dajia", 0.0 },
  { "Dajiah", 0.0 },
  { "Dajiana", 0.0 },
  { "Dajion", 1.0 },
  { "Dajohn", 1.0 },
  { "Dajon", 0.9576784 },
  { "Dajona", 0.0 },
  { "Dajonae", 0.0 },
  { "Dajonique", 0.0 },
  { "Dajonna", 0.0 },
  { "Dajoun", 1.0 },
  { "Dajour", 0.96068376 },
  { "Dajoure", 1.0 },
  { "Dajsha", 0.0 },
  { "Dajshia", 0.0 },
  { "Dajua", 0.0 },
  { "Dajuan", 1.0 },
  { "Dajuana", 0.0 },
  { "Dajun", 1.0 },
  { "Dajuon", 1.0 },
  { "Dajza", 0.0 },
  { "Dak", 1.0 },
  { "Dakai", 1.0 },
  { "Dakaiden", 1.0 },
  { "Dakar", 1.0 },
  { "Dakara", 0.0 },
  { "Dakarai", 0.98444253 },
  { "Dakari", 0.9407202 },
  { "Dakaria", 0.0 },
  { "Dakarie", 1.0 },
  { "Dakarii", 1.0 },
  { "Dakarion", 1.0 },
  { "Dakarri", 1.0 },
  { "Dakayden", 1.0 },
  { "Dakayla", 0.0 },
  { "Dake", 1.0 },
  { "Dakeem", 1.0 },
  { "Dakeisha", 0.0 },
  { "Daken", 1.0 },
  { "Dakera", 0.0 },
  { "Dakeria", 0.0 },
  { "Dakesha", 0.0 },
  { "Dakhari", 1.0 },
  { "Dakhi", 1.0 },
  { "Dakhia", 0.0 },
  { "Dakia", 0.0 },
  { "Dakim", 1.0 },
  { "Dakin", 1.0 },
  { "Daking", 1.0 },
  { "Dakira", 0.0 },
  { "Dakisha", 0.0 },
  { "Dakita", 0.0 },
  { "Dakiya", 0.0 },
  { "Dakiyah", 0.0 },
  { "Dakkota", 0.87234044 },
  { "Dakoata", 1.0 },
  { "Dakobe", 1.0 },
  { "Dakoda", 0.79988265 },
  { "Dakodah", 0.5875 },
  { "Dakori", 1.0 },
  { "Dakota", 0.73385084 },
  { "Dakotah", 0.6112161 },
  { "Dakotajames", 1.0 },
  { "Dakotta", 0.6292135 },
  { "Daks", 1.0 },
  { "Daksh", 1.0 },
  { "Daksha", 0.0 },
  { "Daktari", 1.0 },
  { "Dakuan", 1.0 },
  { "Dakwan", 1.0 },
  { "Dakwon", 1.0 },
  { "Dakya", 0.0 },
  { "Dakyah", 0.0 },
  { "Dakyla", 0.0 },
  { "Dakyra", 0.0 },
  { "Dakyrah", 0.0 },
  { "Dal", 1.0 },
  { "Dala", 0.0 },
  { "Dalai", 0.0 },
  { "Dalaila", 0.0 },
  { "Dalaina", 0.0 },
  { "Dalaine", 0.0 },
  { "Dalainee", 0.0 },
  { "Dalainey", 0.0 },
  { "Dalajah", 0.0 },
  { "Dalal", 0.0 },
  { "Dalan", 1.0 },
  { "Dalana", 0.0 },
  { "Dalane", 0.0 },
  { "Dalanee", 0.0 },
  { "Dalaney", 0.0 },
  { "Dalani", 0.0 },
  { "Dalanie", 0.0 },
  { "Dalanna", 0.0 },
  { "Dalano", 1.0 },
  { "Dalante", 1.0 },
  { "Dalany", 0.0 },
  { "Dalari", 0.0 },
  { "Dalarie", 0.0 },
  { "Dalary", 0.0 },
  { "Dalas", 0.7058824 },
  { "Dalasha", 0.0 },
  { "Dalasia", 0.0 },
  { "Dalay", 0.0 },
  { "Dalaya", 0.0 },
  { "Dalayah", 0.0 },
  { "Dalayjah", 0.0 },
  { "Dalayla", 0.0 },
  { "Dalaylah", 0.0 },
  { "Dalayna", 0.0 },
  { "Dalaynee", 0.0 },
  { "Dalayni", 0.0 },
  { "Dalaynie", 0.0 },
  { "Dalaysa", 0.0 },
  { "Dalaysha", 0.0 },
  { "Dalayshia", 0.0 },
  { "Dalaysia", 0.0 },
  { "Dalayssa", 0.0 },
  { "Dalayza", 0.0 },
  { "Dalbert", 1.0 },
  { "Dale", 0.9268393 },
  { "Dalea", 0.0 },
  { "Daleah", 0.0 },
  { "Dalecia", 0.0 },
  { "Dalee", 0.0 },
  { "Daleen", 0.0 },
  { "Daleena", 0.0 },
  { "Daleesa", 0.0 },
  { "Daleigh", 0.0 },
  { "Daleila", 0.0 },
  { "Daleisa", 0.0 },
  { "Daleisha", 0.0 },
  { "Daleiza", 0.0 },
  { "Dalen", 0.99102134 },
  { "Dalena", 0.0 },
  { "Dalene", 0.0 },
  { "Dalenna", 0.0 },
  { "Daleon", 1.0 },
  { "Daler", 1.0 },
  { "Dalerie", 0.0 },
  { "Dalery", 0.0 },
  { "Dalesa", 0.0 },
  { "Dalesha", 0.0 },
  { "Daleshia", 0.0 },
  { "Dalesia", 0.0 },
  { "Dalessandro", 1.0 },
  { "Dalet", 0.0 },
  { "Daleth", 0.0 },
  { "Dalevon", 1.0 },
  { "Dalexa", 0.0 },
  { "Dalexia", 0.0 },
  { "Daley", 0.22593321 },
  { "Daleya", 0.0 },
  { "Daleyah", 0.0 },
  { "Daleysa", 0.0 },
  { "Daleysha", 0.0 },
  { "Daleysi", 0.0 },
  { "Daleyssa", 0.0 },
  { "Daleyza", 0.0 },
  { "Daleyzah", 0.0 },
  { "Daleyzza", 0.0 },
  { "Dalhart", 1.0 },
  { "Dalhia", 0.0 },
  { "Dali", 0.25 },
  { "Dalia", 0.00051146536 },
  { "Daliah", 0.0 },
  { "Dalian", 1.0 },
  { "Daliana", 0.0 },
  { "Dalianna", 0.0 },
  { "Dalibor", 1.0 },
  { "Dalice", 0.0 },
  { "Dalicia", 0.0 },
  { "Dalida", 0.0 },
  { "Dalijah", 0.0 },
  { "Dalil", 1.0 },
  { "Dalila", 0.0 },
  { "Dalilah", 0.0 },
  { "Dalin", 0.9402655 },
  { "Dalina", 0.0 },
  { "Dalinda", 0.0 },
  { "Dalis", 0.040441178 },
  { "Dalisa", 0.0 },
  { "Dalisha", 0.0 },
  { "Dalisia", 0.0 },
  { "Dalissa", 0.0 },
  { "Daliya", 0.0 },
  { "Daliyah", 0.0 },
  { "Daliyla", 0.0 },
  { "Daliylah", 0.0 },
  { "Daliza", 0.0 },
  { "Dall", 1.0 },
  { "Dallace", 0.0 },
  { "Dallan", 1.0 },
  { "Dallana", 0.0 },
  { "Dallanara", 0.0 },
  { "Dallary", 0.0 },
  { "Dallas", 0.8353299 },
  { "Dallee", 0.0 },
  { "Dallen", 1.0 },
  { "Dallene", 0.0 },
  { "Dalles", 1.0 },
  { "Dalli", 0.0 },
  { "Dallie", 0.01984127 },
  { "Dallin", 1.0 },
  { "Dallion", 1.0 },
  { "Dallis", 0.5620389 },
  { "Dallon", 1.0 },
  { "Dallton", 1.0 },
  { "Dally", 0.0 },
  { "Dallyce", 0.0 },
  { "Dallyn", 0.88584477 },
  { "Dallys", 0.0 },
  { "Dalma", 0.0 },
  { "Dalmar", 1.0 },
  { "Dalmer", 1.0 },
  { "Dalon", 1.0 },
  { "Dalonda", 0.0 },
  { "Dalonna", 0.0 },
  { "Dalonta", 1.0 },
  { "Dalontae", 1.0 },
  { "Dalonte", 1.0 },
  { "Dalonzo", 1.0 },
  { "Dalores", 0.0 },
  { "Daloris", 0.0 },
  { "Dalpha", 0.0 },
  { "Dalphine", 0.0 },
  { "Dalphus", 1.0 },
  { "Dalshawn", 1.0 },
  { "Dalson", 1.0 },
  { "Dalston", 1.0 },
  { "Daltan", 1.0 },
  { "Dalten", 1.0 },
  { "Daltin", 1.0 },
  { "Dalton", 0.9950426 },
  { "Daltry", 1.0 },
  { "Daltyn", 1.0 },
  { "Dalva", 0.0 },
  { "Dalven", 1.0 },
  { "Dalvin", 1.0 },
  { "Dalvon", 1.0 },
  { "Dalvyn", 1.0 },
  { "Daly", 0.12969284 },
  { "Dalya", 0.0 },
  { "Dalyah", 0.0 },
  { "Dalyce", 0.0 },
  { "Dalyla", 0.0 },
  { "Dalylah", 0.0 },
  { "Dalyn", 0.6736658 },
  { "Dalynn", 0.103286386 },
  { "Dalys", 0.29166666 },
  { "Dalyss", 0.0 },
  { "Dalyssa", 0.0 },
  { "Dalziel", 1.0 },
  { "Dam", 1.0 },
  { "Dama", 0.0 },
  { "Damacio", 1.0 },
  { "Damaiah", 0.0 },
  { "Damain", 1.0 },
  { "Damaine", 1.0 },
  { "Damaire", 1.0 },
  { "Damaiya", 0.0 },
  { "Damaj", 1.0 },
  { "Damajae", 1.0 },
  { "Damajah", 1.0 },
  { "Damaje", 1.0 },
  { "Damal", 1.0 },
  { "Damali", 0.0 },
  { "Daman", 1.0 },
  { "Damani", 0.80508137 },
  { "Damante", 1.0 },
  { "Damany", 1.0 },
  { "Damar", 0.99493927 },
  { "Damara", 0.0 },
  { "Damarae", 1.0 },
  { "Damarco", 1.0 },
  { "Damarcus", 1.0 },
  { "Damare", 1.0 },
  { "Damarea", 0.9375 },
  { "Damaree", 1.0 },
  { "Damareion", 1.0 },
  { "Damareon", 1.0 },
  { "Damari", 0.8342512 },
  { "Damaria", 0.15825687 },
  { "Damariae", 1.0 },
  { "Damariah", 0.0 },
  { "Damarian", 1.0 },
  { "Damariana", 0.0 },
  { "Damariay", 1.0 },
  { "Damarie", 0.6923077 },
  { "Damarien", 1.0 },
  { "Damarieon", 1.0 },
  { "Damarii", 1.0 },
  { "Damario", 1.0 },
  { "Damarion", 1.0 },
  { "Damariona", 0.0 },
  { "Damarious", 1.0 },
  { "Damaris", 0.0427736 },
  { "Damarius", 1.0 },
  { "Damariya", 0.0 },
  { "Damariye", 1.0 },
  { "Damariyon", 1.0 },
  { "Damariz", 0.0 },
  { "Damarkis", 1.0 },
  { "Damarko", 1.0 },
  { "Damarkus", 1.0 },
  { "Damarlo", 1.0 },
  { "Damarques", 1.0 },
  { "Damarquez", 1.0 },
  { "Damarquis", 1.0 },
  { "Damarr", 1.0 },
  { "Damarrea", 1.0 },
  { "Damarri", 1.0 },
  { "Damarria", 0.0 },
  { "Damarrion", 1.0 },
  { "Damarrius", 1.0 },
  { "Damarte", 1.0 },
  { "Damarus", 1.0 },
  { "Damary", 0.0 },
  { "Damaryon", 1.0 },
  { "Damarys", 0.0 },
  { "Damascus", 1.0 },
  { "Damase", 1.0 },
  { "Damasio", 1.0 },
  { "Damaso", 1.0 },
  { "Damato", 1.0 },
  { "Damauni", 1.0 },
  { "Damauri", 1.0 },
  { "Damaurion", 1.0 },
  { "Damaury", 1.0 },
  { "Damaya", 0.0 },
  { "Damayah", 0.0 },
  { "Damayanti", 0.0 },
  { "Damayia", 0.0 },
  { "Damber", 0.0 },
  { "Damean", 1.0 },
  { "Damecia", 0.0 },
  { "Dameer", 1.0 },
  { "Dameian", 1.0 },
  { "Dameika", 0.0 },
  { "Damein", 1.0 },
  { "Dameion", 1.0 },
  { "Dameir", 1.0 },
  { "Dameisha", 0.0 },
  { "Damek", 1.0 },
  { "Dameka", 0.0 },
  { "Damekia", 0.0 },
  { "Dameko", 1.0 },
  { "Damel", 1.0 },
  { "Damell", 1.0 },
  { "Damen", 1.0 },
  { "Dameon", 1.0 },
  { "Damera", 0.0 },
  { "Damere", 1.0 },
  { "Dameria", 0.0 },
  { "Damerion", 1.0 },
  { "Damerius", 1.0 },
  { "Dameron", 0.7222222 },
  { "Damesha", 0.0 },
  { "Dameshia", 0.0 },
  { "Dameta", 0.0 },
  { "Dametra", 0.0 },
  { "Dametre", 1.0 },
  { "Dametri", 1.0 },
  { "Dametria", 0.0 },
  { "Dametrice", 0.0 },
  { "Dametrious", 1.0 },
  { "Dametris", 1.0 },
  { "Dametrius", 1.0 },
  { "Dameyon", 1.0 },
  { "Dameyune", 1.0 },
  { "Dami", 0.0 },
  { "Damia", 0.0 },
  { "Damiah", 0.0 },
  { "Damiam", 1.0 },
  { "Damian", 0.9959847 },
  { "Damiana", 0.0 },
  { "Damiane", 1.0 },
  { "Damiani", 1.0 },
  { "Damiann", 1.0 },
  { "Damianna", 0.0 },
  { "Damiano", 1.0 },
  { "Damica", 0.0 },
  { "Damichael", 1.0 },
  { "Damico", 1.0 },
  { "Damie", 0.13333334 },
  { "Damiean", 1.0 },
  { "Damiel", 1.0 },
  { "Damielle", 0.0 },
  { "Damien", 0.99522734 },
  { "Damiene", 1.0 },
  { "Damieon", 1.0 },
  { "Damier", 1.0 },
  { "Damiere", 1.0 },
  { "Damiesha", 0.0 },
  { "Damika", 0.0 },
  { "Damilare", 1.0 },
  { "Damilola", 0.2160804 },
  { "Damin", 1.0 },
  { "Damina", 0.0 },
  { "Damine", 1.0 },
  { "Damini", 0.0 },
  { "Damion", 0.9975524 },
  { "Damione", 1.0 },
  { "Damionna", 0.0 },
  { "Damir", 0.9964106 },
  { "Damira", 0.0 },
  { "Damiracle", 0.0 },
  { "Damirah", 0.0 },
  { "Damire", 1.0 },
  { "Damisha", 0.0 },
  { "Damishia", 0.0 },
  { "Damisi", 1.0 },
  { "Damiso", 1.0 },
  { "Damita", 0.0 },
  { "Damitra", 0.0 },
  { "Damitri", 1.0 },
  { "Damitrius", 1.0 },
  { "Damiun", 1.0 },
  { "Damius", 1.0 },
  { "Damiya", 0.0 },
  { "Damiyah", 0.0 },
  { "Damiyan", 1.0 },
  { "Damiyon", 1.0 },
  { "Damla", 0.0 },
  { "Dammian", 1.0 },
  { "Dammion", 1.0 },
  { "Dammon", 1.0 },
  { "Damne", 0.0 },
  { "Damon", 0.99497306 },
  { "Damona", 0.0 },
  { "Damond", 0.99755263 },
  { "Damondre", 1.0 },
  { "Damone", 1.0 },
  { "Damonee", 1.0 },
  { "Damonei", 1.0 },
  { "Damoney", 1.0 },
  { "Damoni", 0.73967683 },
  { "Damonica", 0.0 },
  { "Damonie", 0.6897959 },
  { "Damonii", 1.0 },
  { "Damonique", 0.0 },
  { "Damonn", 1.0 },
  { "Damonni", 1.0 },
  { "Damonnie", 1.0 },
  { "Damont", 1.0 },
  { "Damonta", 1.0 },
  { "Damontae", 1.0 },
  { "Damontay", 1.0 },
  { "Damonte", 1.0 },
  { "Damontez", 1.0 },
  { "Damontra", 1.0 },
  { "Damontrae", 1.0 },
  { "Damontray", 1.0 },
  { "Damontre", 1.0 },
  { "Damori", 1.0 },
  { "Damorian", 1.0 },
  { "Damorion", 1.0 },
  { "Damorris", 1.0 },
  { "Damu", 1.0 },
  { "Damus", 1.0 },
  { "Damya", 0.0 },
  { "Damyah", 0.0 },
  { "Damyan", 1.0 },
  { "Damyen", 1.0 },
  { "Damyia", 0.0 },
  { "Damyiah", 0.0 },
  { "Damyla", 0.0 },
  { "Damyn", 1.0 },
  { "Damyon", 1.0 },
  { "Damyra", 0.0 },
  { "Damyria", 0.0 },
  { "Damyrion", 1.0 },
  { "Damyron", 1.0 },
  { "Dan", 0.99487114 },
  { "Dana", 0.21713783 },
  { "Danaca", 0.0 },
  { "Danae", 0.0009277872 },
  { "Danaeja", 0.0 },
  { "Danaejah", 0.0 },
  { "Danah", 0.0 },
  { "Danahe", 0.0 },
  { "Danahi", 0.0 },
  { "Danai", 0.017421603 },
  { "Danaia", 0.0 },
  { "Danaija", 0.0 },
  { "Danaijah", 0.0 },
  { "Danail", 1.0 },
  { "Danaira", 0.0 },
  { "Danaisa", 0.0 },
  { "Danaisha", 0.0 },
  { "Danait", 0.0 },
  { "Danaiya", 0.0 },
  { "Danaiyah", 0.0 },
  { "Danaja", 0.0 },
  { "Danajah", 0.0 },
  { "Danajha", 0.0 },
  { "Danajia", 0.0 },
  { "Danaka", 0.0 },
  { "Danal", 1.0 },
  { "Danald", 1.0 },
  { "Danalee", 0.0 },
  { "Danali", 0.0 },
  { "Danaly", 0.0 },
  { "Danalyn", 0.0 },
  { "Danalynn", 0.0 },
  { "Danamarie", 0.0 },
  { "Danan", 1.0 },
  { "Danapaola", 0.0 },
  { "Danara", 0.0 },
  { "Danard", 1.0 },
  { "Danari", 0.6976744 },
  { "Danaria", 0.0 },
  { "Danariah", 0.0 },
  { "Danarius", 1.0 },
  { "Danasha", 0.0 },
  { "Danashia", 0.0 },
  { "Danasia", 0.0 },
  { "Danautica", 0.0 },
  { "Danavia", 0.0 },
  { "Danay", 0.0 },
  { "Danaya", 0.0 },
  { "Danayah", 0.0 },
  { "Danayara", 0.0 },
  { "Danaye", 0.0 },
  { "Danayia", 0.0 },
  { "Danayja", 0.0 },
  { "Danayjah", 0.0 },
  { "Danayla", 0.0 },
  { "Danaysha", 0.0 },
  { "Danayshia", 0.0 },
  { "Danaysia", 0.0 },
  { "Danaysy", 0.0 },
  { "Danazha", 0.0 },
  { "Danazia", 0.0 },
  { "Dancia", 0.0 },
  { "Dancy", 0.0 },
  { "Dandi", 0.0 },
  { "Dandra", 0.09022556 },
  { "Dandrae", 1.0 },
  { "Dandre", 0.9948165 },
  { "Dandrea", 0.1605166 },
  { "Dandrell", 1.0 },
  { "Dandrew", 1.0 },
  { "Dandria", 0.0 },
  { "Dandrick", 1.0 },
  { "Dandridge", 1.0 },
  { "Dandy", 0.95238096 },
  { "Dane", 0.9880289 },
  { "Danea", 0.0 },
  { "Daneah", 0.0 },
  { "Daneal", 0.7777778 },
  { "Danean", 0.0 },
  { "Daneca", 0.0 },
  { "Danecia", 0.0 },
  { "Danee", 0.0 },
  { "Daneeka", 0.0 },
  { "Daneel", 1.0 },
  { "Daneen", 0.0 },
  { "Daneesha", 0.0 },
  { "Daneida", 0.0 },
  { "Daneika", 0.0 },
  { "Daneil", 0.97425586 },
  { "Daneila", 0.0 },
  { "Daneille", 0.0 },
  { "Daneisha", 0.0 },
  { "Daneisy", 0.0 },
  { "Daneja", 0.0 },
  { "Danek", 1.0 },
  { "Daneka", 0.0 },
  { "Danel", 0.683871 },
  { "Danela", 0.0 },
  { "Danele", 0.0 },
  { "Daneli", 0.0 },
  { "Danelia", 0.0 },
  { "Danelis", 0.0 },
  { "Danell", 0.23365617 },
  { "Danella", 0.0 },
  { "Danelle", 0.00649259 },
  { "Danelly", 0.0 },
  { "Danely", 0.0 },
  { "Danelys", 0.0 },
  { "Danen", 1.0 },
  { "Danene", 0.0 },
  { "Danequa", 0.0 },
  { "Danera", 0.0 },
  { "Danery", 0.0 },
  { "Danesa", 0.0 },
  { "Danese", 0.0 },
  { "Danesh", 1.0 },
  { "Danesha", 0.0 },
  { "Daneshia", 0.0 },
  { "Danesia", 0.0 },
  { "Danessa", 0.0 },
  { "Danet", 0.0 },
  { "Danetra", 0.0 },
  { "Danetria", 0.0 },
  { "Danett", 0.0 },
  { "Danetta", 0.0 },
  { "Danette", 0.0 },
  { "Daney", 0.15068494 },
  { "Daneya", 0.0 },
  { "Daneyah", 0.0 },
  { "Daneysha", 0.0 },
  { "Danford", 1.0 },
  { "Danforth", 1.0 },
  { "Dang", 0.9802632 },
  { "Dangel", 0.6052632 },
  { "Dangela", 0.0 },
  { "Dangello", 1.0 },
  { "Dangelo", 1.0 },
  { "Danger", 1.0 },
  { "Dangkhoa", 1.0 },
  { "Danh", 1.0 },
  { "Dani", 0.10388313 },
  { "Dania", 0.0007247427 },
  { "Daniah", 0.0 },
  { "Danial", 0.991396 },
  { "Daniale", 0.0 },
  { "Danialle", 0.0 },
  { "Danian", 1.0 },
  { "Daniana", 0.0 },
  { "Daniaya", 0.0 },
  { "Danica", 0.0 },
  { "Danicah", 0.0 },
  { "Danice", 0.0 },
  { "Danicia", 0.0 },
  { "Danicka", 0.0 },
  { "Danie", 0.67657995 },
  { "Danieal", 0.38297874 },
  { "Daniece", 0.0 },
  { "Danieka", 0.0 },
  { "Daniel", 0.9957214 },
  { "Daniela", 0.0028596993 },
  { "Danield", 1.0 },
  { "Daniele", 0.17438883 },
  { "Danielee", 0.0 },
  { "Danielia", 0.0 },
  { "Danielis", 0.0 },
  { "Danieliz", 0.0 },
  { "Danieljames", 1.0 },
  { "Danieljohn", 1.0 },
  { "Danieljoseph", 1.0 },
  { "Danieljr", 1.0 },
  { "Daniell", 0.28039354 },
  { "Daniella", 0.00022100261 },
  { "Daniellah", 0.0 },
  { "Danielle", 0.0051596216 },
  { "Daniellee", 1.0 },
  { "Daniellemarie", 0.0 },
  { "Daniellerose", 0.0 },
  { "Daniellie", 0.0 },
  { "Danielly", 0.0 },
  { "Daniels", 1.0 },
  { "Danielson", 1.0 },
  { "Daniely", 0.0 },
  { "Danielynn", 0.0 },
  { "Danielys", 0.0 },
  { "Danien", 1.0 },
  { "Daniesha", 0.0 },
  { "Daniette", 0.0 },
  { "Daniil", 1.0 },
  { "Danija", 0.0 },
  { "Danijah", 0.0 },
  { "Danijel", 1.0 },
  { "Danijela", 0.0 },
  { "Danik", 1.0 },
  { "Danika", 0.0 },
  { "Danikah", 0.0 },
  { "Danikka", 0.0 },
  { "Danil", 1.0 },
  { "Danila", 0.16447368 },
  { "Danilah", 0.0 },
  { "Danile", 0.974359 },
  { "Danilee", 0.0 },
  { "Danilla", 0.0 },
  { "Danille", 0.0 },
  { "Danillie", 0.0 },
  { "Danilo", 1.0 },
  { "Danilyn", 0.0 },
  { "Danilynn", 0.0 },
  { "Danina", 0.0 },
  { "Danine", 0.0 },
  { "Danion", 1.0 },
  { "Daniqua", 0.0 },
  { "Danique", 0.0 },
  { "Danira", 0.0 },
  { "Danis", 0.76865673 },
  { "Danisa", 0.0 },
  { "Danise", 0.0 },
  { "Danish", 1.0 },
  { "Danisha", 0.0 },
  { "Danishia", 0.0 },
  { "Danissa", 0.0 },
  { "Danit", 0.0 },
  { "Danita", 0.0 },
  { "Danitra", 0.0 },
  { "Danity", 0.0 },
  { "Danitza", 0.0 },
  { "Danixa", 0.0 },
  { "Daniya", 0.0 },
  { "Daniyah", 0.0 },
  { "Daniyal", 1.0 },
  { "Daniyel", 1.0 },
  { "Daniyla", 0.0 },
  { "Daniylah", 0.0 },
  { "Daniza", 0.0 },
  { "Danja", 0.0 },
  { "Danka", 0.0 },
  { "Danl", 1.0 },
  { "Danley", 1.0 },
  { "Dann", 0.48769575 },
  { "Danna", 0.0074110674 },
  { "Dannae", 0.0 },
  { "Dannah", 0.0 },
  { "Dannaka", 0.0 },
  { "Dannalee", 0.0 },
  { "Dannaly", 0.0 },
  { "Dannalyn", 0.0 },
  { "Dannan", 0.0 },
  { "Dannapaola", 0.0 },
  { "Dannay", 0.0 },
  { "Danne", 0.2675841 },
  { "Dannea", 0.0 },
  { "Dannee", 0.0 },
  { "Dannel", 0.8507463 },
  { "Dannell", 0.3776596 },
  { "Dannelle", 0.0 },
  { "Dannely", 0.0 },
  { "Dannen", 1.0 },
  { "Danner", 0.98452014 },
  { "Dannesha", 0.0 },
  { "Dannett", 0.0 },
  { "Dannetta", 0.0 },
  { "Dannette", 0.0 },
  { "Danney", 0.99032885 },
  { "Danni", 0.033930253 },
  { "Dannia", 0.0 },
  { "Dannial", 1.0 },
  { "Dannica", 0.0 },
  { "Dannicka", 0.0 },
  { "Dannie", 0.8068254 },
  { "Danniel", 0.92577815 },
  { "Danniela", 0.0 },
  { "Danniele", 0.0 },
  { "Danniell", 0.0 },
  { "Danniella", 0.0 },
  { "Dannielle", 0.0 },
  { "Dannielyn", 0.0 },
  { "Dannielynn", 0.0 },
  { "Dannika", 0.0 },
  { "Dannille", 0.0 },
  { "Dannilyn", 0.0 },
  { "Dannilynn", 0.0 },
  { "Dannion", 1.0 },
  { "Dannis", 1.0 },
  { "Dannisha", 0.0 },
  { "Dannita", 0.0 },
  { "Dannon", 0.84110534 },
  { "Danny", 0.9944613 },
  { "Dannya", 0.0 },
  { "Dannye", 0.78125 },
  { "Dannyel", 1.0 },
  { "Dannyell", 0.0 },
  { "Dannyelle", 0.0 },
  { "Dannylynn", 0.0 },
  { "Dannyray", 1.0 },
  { "Dano", 1.0 },
  { "Danon", 0.8969072 },
  { "Danovan", 1.0 },
  { "Danquan", 1.0 },
  { "Dansby", 1.0 },
  { "Danson", 1.0 },
  { "Danta", 0.9754902 },
  { "Dantae", 1.0 },
  { "Dantasia", 0.0 },
  { "Dantavious", 1.0 },
  { "Dantavis", 1.0 },
  { "Dantavius", 1.0 },
  { "Dantay", 1.0 },
  { "Dante", 0.9891096 },
  { "Dantee", 1.0 },
  { "Dantel", 1.0 },
  { "Danterius", 1.0 },
  { "Danterrius", 1.0 },
  { "Dantes", 1.0 },
  { "Dantez", 1.0 },
  { "Danthony", 1.0 },
  { "Danti", 1.0 },
  { "Dantley", 1.0 },
  { "Dantoinette", 0.0 },
  { "Danton", 1.0 },
  { "Dantoni", 1.0 },
  { "Dantonio", 1.0 },
  { "Dantre", 1.0 },
  { "Dantrel", 1.0 },
  { "Dantrell", 1.0 },
  { "Dantwan", 1.0 },
  { "Danual", 1.0 },
  { "Danuel", 0.98245615 },
  { "Danuta", 0.0 },
  { "Danute", 0.0 },
  { "Dany", 0.91033924 },
  { "Danya", 0.008745477 },
  { "Danyaal", 1.0 },
  { "Danyae", 0.0 },
  { "Danyael", 1.0 },
  { "Danyah", 0.0 },
  { "Danyail", 0.0 },
  { "Danyal", 0.71929824 },
  { "Danyale", 0.051177073 },
  { "Danyall", 0.0 },
  { "Danyalle", 0.0 },
  { "Danyca", 0.0 },
  { "Danye", 0.7735849 },
  { "Danyea", 0.0 },
  { "Danyeal", 0.0 },
  { "Danyeil", 0.0 },
  { "Danyel", 0.2073441 },
  { "Danyela", 0.0 },
  { "Danyele", 0.0 },
  { "Danyell", 0.114245415 },
  { "Danyella", 0.0 },
  { "Danyelle", 0.006480995 },
  { "Danyetta", 0.0 },
  { "Danyette", 0.0 },
  { "Danyia", 0.0 },
  { "Danyiah", 0.0 },
  { "Danyiel", 0.0 },
  { "Danyielle", 0.0 },
  { "Danyka", 0.0 },
  { "Danykah", 0.0 },
  { "Danyl", 0.5597015 },
  { "Danyla", 0.0 },
  { "Danylah", 0.0 },
  { "Danyle", 0.0 },
  { "Danyll", 0.0 },
  { "Danylle", 0.0 },
  { "Danylo", 1.0 },
  { "Danyne", 0.0 },
  { "Danyon", 1.0 },
  { "Danyra", 0.0 },
  { "Danysha", 0.0 },
  { "Danytza", 0.0 },
  { "Danzel", 1.0 },
  { "Danzell", 1.0 },
  { "Danzelle", 1.0 },
  { "Danziel", 1.0 },
  { "Danzig", 1.0 },
  { "Dao", 0.68231046 },
  { "Daon", 1.0 },
  { "Daonna", 0.0 },
  { "Daonte", 1.0 },
  { "Daoud", 1.0 },
  { "Daouda", 1.0 },
  { "Daphane", 0.0 },
  { "Daphanie", 0.0 },
  { "Daphen", 0.0 },
  { "Daphene", 0.0 },
  { "Daphine", 0.0 },
  { "Daphna", 0.0 },
  { "Daphne", 0.0002044094 },
  { "Daphnee", 0.0 },
  { "Daphney", 0.0 },
  { "Daphni", 0.0 },
  { "Daphnie", 0.0 },
  { "Daphny", 0.0 },
  { "Daphyne", 0.0 },
  { "Daquain", 1.0 },
  { "Daquan", 0.9992012 },
  { "Daquana", 0.0 },
  { "Daquanda", 0.0 },
  { "Daquandre", 1.0 },
  { "Daquane", 1.0 },
  { "Daquann", 1.0 },
  { "Daquanna", 0.0 },
  { "Daquante", 1.0 },
  { "Daquarious", 1.0 },
  { "Daquarius", 1.0 },
  { "Daquasha", 0.0 },
  { "Daquasia", 0.0 },
  { "Daquavion", 1.0 },
  { "Daquavious", 1.0 },
  { "Daquavius", 1.0 },
  { "Daquawn", 1.0 },
  { "Daquel", 1.0 },
  { "Daquesha", 0.0 },
  { "Daquez", 1.0 },
  { "Daquian", 1.0 },
  { "Daquin", 1.0 },
  { "Daquinn", 1.0 },
  { "Daquion", 1.0 },
  { "Daquisha", 0.0 },
  { "Daquita", 0.0 },
  { "Daquon", 1.0 },
  { "Daquwan", 1.0 },
  { "Daqwan", 1.0 },
  { "Daqwane", 1.0 },
  { "Daqwon", 1.0 },
  { "Dar", 1.0 },
  { "Dara", 0.046863962 },
  { "Darae", 0.33333334 },
  { "Darah", 0.0 },
  { "Darail", 1.0 },
  { "Darain", 1.0 },
  { "Daral", 1.0 },
  { "Darald", 1.0 },
  { "Daralee", 0.0 },
  { "Daralene", 0.0 },
  { "Daraly", 0.0 },
  { "Daralyn", 0.0 },
  { "Daralynn", 0.0 },
  { "Daran", 0.99140894 },
  { "Darann", 0.0 },
  { "Darasimi", 0.0 },
  { "Darathy", 0.0 },
  { "Daray", 1.0 },
  { "Darayl", 1.0 },
  { "Darbee", 0.0 },
  { "Darbey", 0.0 },
  { "Darbi", 0.0 },
  { "Darbie", 0.0 },
  { "Darby", 0.2670268 },
  { "Darcas", 0.0 },
  { "Darce", 0.0 },
  { "Darcee", 0.0 },
  { "Darcel", 0.09555069 },
  { "Darcell", 0.24477611 },
  { "Darcella", 0.0 },
  { "Darcelle", 0.0 },
  { "Darcey", 0.04596577 },
  { "Darchell", 0.0 },
  { "Darchelle", 0.0 },
  { "Darci", 0.0 },
  { "Darcia", 0.0 },
  { "Darcie", 0.0 },
  { "Darco", 1.0 },
  { "Darcus", 0.029810298 },
  { "Darcy", 0.11001629 },
  { "Dardan", 1.0 },
  { "Dardanella", 0.0 },
  { "Darden", 1.0 },
  { "Dardenella", 0.0 },
  { "Dardi", 0.0 },
  { "Dare", 0.4921875 },
  { "Darea", 0.0 },
  { "Darean", 0.68292683 },
  { "Dareck", 1.0 },
  { "Dareece", 1.0 },
  { "Dareen", 0.029239766 },
  { "Dareese", 0.0 },
  { "Dareian", 1.0 },
  { "Darein", 0.7619048 },
  { "Dareion", 1.0 },
  { "Darek", 1.0 },
  { "Dareka", 0.0 },
  { "Darel", 0.99752843 },
  { "Dareld", 1.0 },
  { "Darelene", 0.0 },
  { "Dareli", 0.0 },
  { "Darell", 0.9979184 },
  { "Darelle", 0.962069 },
  { "Darely", 0.0 },
  { "Darelyn", 0.0 },
  { "Darelys", 0.0 },
  { "Daren", 0.98192656 },
  { "Darena", 0.0 },
  { "Darence", 1.0 },
  { "Darenda", 0.0 },
  { "Darene", 0.0 },
  { "Dareon", 0.9830508 },
  { "Dareona", 0.0 },
  { "Darese", 1.0 },
  { "Daresha", 0.0 },
  { "Dareth", 0.0 },
  { "Daretha", 0.0 },
  { "Daretta", 0.0 },
  { "Dareus", 1.0 },
  { "Darey", 1.0 },
  { "Dargan", 1.0 },
  { "Darhl", 1.0 },
  { "Darhonda", 0.0 },
  { "Darhyl", 1.0 },
  { "Dari", 0.045614034 },
  { "Daria", 0.0006164468 },
  { "Dariah", 0.0 },
  { "Darian", 0.7074534 },
  { "Dariana", 0.0 },
  { "Dariane", 0.0 },
  { "Dariann", 0.0 },
  { "Darianna", 0.0 },
  { "Darianne", 0.0 },
  { "Darianny", 0.0 },
  { "Dariany", 0.0 },
  { "Darianys", 0.0 },
  { "Darias", 1.0 },
  { "Daric", 1.0 },
  { "Darica", 0.0 },
  { "Darice", 0.009961686 },
  { "Darick", 1.0 },
  { "Darico", 1.0 },
  { "Darie", 0.0 },
  { "Dariean", 0.0 },
  { "Dariel", 0.8759584 },
  { "Dariela", 0.0 },
  { "Darielis", 0.0 },
  { "Dariell", 0.74358976 },
  { "Dariella", 0.0 },
  { "Darielle", 0.0 },
  { "Dariely", 0.0 },
  { "Darielys", 0.0 },
  { "Darien", 0.8421144 },
  { "Dariene", 0.0 },
  { "Darienne", 0.0 },
  { "Darieon", 1.0 },
  { "Daries", 1.0 },
  { "Darieus", 1.0 },
  { "Darihanna", 0.0 },
  { "Darik", 1.0 },
  { "Darika", 0.0 },
  { "Darikson", 1.0 },
  { "Daril", 1.0 },
  { "Darilyn", 0.0 },
  { "Darilynn", 0.0 },
  { "Darin", 0.9888148 },
  { "Darina", 0.0 },
  { "Darinda", 0.0 },
  { "Darine", 0.0 },
  { "Darinel", 1.0 },
  { "Darinesha", 0.0 },
  { "Darinka", 0.0 },
  { "Dario", 1.0 },
  { "Darion", 0.9225993 },
  { "Dariona", 0.0 },
  { "Darionna", 0.0 },
  { "Darionne", 1.0 },
  { "Darionte", 1.0 },
  { "Darious", 1.0 },
  { "Darique", 1.0 },
  { "Daris", 0.64880604 },
  { "Darise", 0.0 },
  { "Darisha", 0.0 },
  { "Darisley", 0.0 },
  { "Darissa", 0.0 },
  { "Darita", 0.0 },
  { "Darith", 1.0 },
  { "Daritza", 0.0 },
  { "Dariun", 1.0 },
  { "Darius", 0.99458945 },
  { "Dariush", 1.0 },
  { "Dariusz", 1.0 },
  { "Darivs", 1.0 },
  { "Dariya", 0.0 },
  { "Dariyah", 0.0 },
  { "Dariyan", 0.45238096 },
  { "Dariyana", 0.0 },
  { "Dariyelle", 0.0 },
  { "Dariyon", 0.94623655 },
  { "Darkiel", 1.0 },
  { "Darko", 1.0 },
  { "Darl", 0.9665072 },
  { "Darla", 0.0006442684 },
  { "Darlah", 0.0 },
  { "Darlaine", 0.0 },
  { "Darlana", 0.0 },
  { "Darlane", 0.0 },
  { "Darle", 0.8507463 },
  { "Darlean", 0.0 },
  { "Darleane", 0.0 },
  { "Darlee", 0.0 },
  { "Darleen", 0.0 },
  { "Darleena", 0.0 },
  { "Darleene", 0.0 },
  { "Darlen", 0.0 },
  { "Darlena", 0.0 },
  { "Darlene", 0.0029588114 },
  { "Darleni", 0.0 },
  { "Darlenis", 0.0 },
  { "Darlenne", 0.0 },
  { "Darleny", 0.0 },
  { "Darlenys", 0.0 },
  { "Darles", 0.0 },
  { "Darlesha", 0.0 },
  { "Darleta", 0.0 },
  { "Darleth", 0.0 },
  { "Darletha", 0.0 },
  { "Darletta", 0.0 },
  { "Darlette", 0.0 },
  { "Darley", 0.5 },
  { "Darlia", 0.0 },
  { "Darlicia", 0.0 },
  { "Darlie", 0.0 },
  { "Darlien", 0.0 },
  { "Darliene", 0.0 },
  { "Darlin", 0.06451613 },
  { "Darlina", 0.0 },
  { "Darlinda", 0.0 },
  { "Darline", 0.0 },
  { "Darling", 0.006527415 },
  { "Darlis", 0.0 },
  { "Darlisa", 0.0 },
  { "Darlise", 0.0 },
  { "Darlisha", 0.0 },
  { "Darliss", 0.0 },
  { "Darlita", 0.0 },
  { "Darlo", 1.0 },
  { "Darlow", 1.0 },
  { "Darly", 0.59090906 },
  { "Darlyene", 0.0 },
  { "Darlyn", 0.012418687 },
  { "Darlynda", 0.0 },
  { "Darlyne", 0.0 },
  { "Darlynn", 0.0 },
  { "Darlynne", 0.0 },
  { "Darlys", 0.0 },
  { "Darma", 0.0 },
  { "Darmani", 1.0 },
  { "Darmarcus", 1.0 },
  { "Darmesha", 0.0 },
  { "Darmon", 1.0 },
  { "Darnae", 0.0 },
  { "Darnaja", 0.0 },
  { "Darnasia", 0.0 },
  { "Darnay", 1.0 },
  { "Darnecia", 0.0 },
  { "Darnee", 1.0 },
  { "Darneil", 1.0 },
  { "Darneisha", 0.0 },
  { "Darnel", 0.95155036 },
  { "Darnell", 0.92086905 },
  { "Darnella", 0.0 },
  { "Darnelle", 0.5891089 },
  { "Darnese", 0.0 },
  { "Darnesha", 0.0 },
  { "Darneshia", 0.0 },
  { "Darnetha", 0.0 },
  { "Darnetta", 0.0 },
  { "Darnette", 0.0 },
  { "Darney", 1.0 },
  { "Darnez", 1.0 },
  { "Darnice", 0.0 },
  { "Darniece", 0.0 },
  { "Darniesha", 0.0 },
  { "Darnika", 0.0 },
  { "Darnise", 0.0 },
  { "Darnisha", 0.0 },
  { "Darnishia", 0.0 },
  { "Darnita", 0.0 },
  { "Darnley", 1.0 },
  { "Darol", 1.0 },
  { "Darold", 1.0 },
  { "Darolyn", 0.0 },
  { "Daron", 0.97862107 },
  { "Daronda", 0.0 },
  { "Darone", 1.0 },
  { "Daronn", 1.0 },
  { "Daronta", 1.0 },
  { "Daronte", 1.0 },
  { "Darothy", 0.0 },
  { "Darquan", 1.0 },
  { "Darquez", 1.0 },
  { "Darquis", 1.0 },
  { "Darquise", 1.0 },
  { "Darquisha", 0.0 },
  { "Darr", 1.0 },
  { "Darra", 0.0 },
  { "Darragh", 0.82843137 },
  { "Darrah", 0.0 },
  { "Darrain", 1.0 },
  { "Darral", 1.0 },
  { "Darrall", 1.0 },
  { "Darran", 1.0 },
  { "Darrayl", 1.0 },
  { "Darrean", 0.47058824 },
  { "Darreck", 1.0 },
  { "Darrein", 0.8 },
  { "Darreion", 1.0 },
  { "Darrek", 1.0 },
  { "Darrel", 0.99588835 },
  { "Darreld", 1.0 },
  { "Darrell", 0.9925623 },
  { "Darrelle", 0.79153097 },
  { "Darrellyn", 0.0 },
  { "Darrelyn", 0.0 },
  { "Darren", 0.99447334 },
  { "Darrence", 1.0 },
  { "Darrent", 1.0 },
  { "Darreon", 1.0 },
  { "Darresha", 0.0 },
  { "Darrett", 1.0 },
  { "Darreus", 1.0 },
  { "Darreyl", 1.0 },
  { "Darri", 0.0 },
  { "Darria", 0.0 },
  { "Darriah", 0.0 },
  { "Darrian", 0.7694134 },
  { "Darriana", 0.0 },
  { "Darriane", 0.0 },
  { "Darrianna", 0.0 },
  { "Darrias", 1.0 },
  { "Darric", 1.0 },
  { "Darrica", 0.0 },
  { "Darrick", 0.99862224 },
  { "Darricka", 0.0 },
  { "Darrie", 0.832 },
  { "Darriel", 0.7913279 },
  { "Darriell", 0.16923077 },
  { "Darrielle", 0.0 },
  { "Darrien", 0.89351034 },
  { "Darries", 1.0 },
  { "Darrik", 1.0 },
  { "Darrika", 0.0 },
  { "Darril", 1.0 },
  { "Darrill", 1.0 },
  { "Darrilyn", 0.0 },
  { "Darrin", 0.99546987 },
  { "Darrington", 1.0 },
  { "Darrio", 1.0 },
  { "Darrion", 0.96453226 },
  { "Darriona", 0.0 },
  { "Darrione", 1.0 },
  { "Darrionna", 0.0 },
  { "Darrious", 1.0 },
  { "Darris", 0.9518427 },
  { "Darrisha", 0.0 },
  { "Darrius", 1.0 },
  { "Darriyah", 0.0 },
  { "Darrly", 1.0 },
  { "Darrlyn", 1.0 },
  { "Darrnell", 1.0 },
  { "Darro", 1.0 },
  { "Darrol", 1.0 },
  { "Darrold", 1.0 },
  { "Darroll", 1.0 },
  { "Darron", 0.9991715 },
  { "Darrow", 1.0 },
  { "Darrus", 1.0 },
  { "Darry", 1.0 },
  { "Darryal", 1.0 },
  { "Darryan", 0.44117647 },
  { "Darryel", 1.0 },
  { "Darryell", 1.0 },
  { "Darryl", 0.9877311 },
  { "Darryle", 1.0 },
  { "Darrylin", 0.0 },
  { "Darryll", 1.0 },
  { "Darryn", 0.9765343 },
  { "Darryon", 1.0 },
  { "Darryus", 1.0 },
  { "Darsey", 0.0 },
  { "Darsh", 1.0 },
  { "Darsha", 0.0 },
  { "Darshae", 0.0 },
  { "Darshan", 0.98333335 },
  { "Darshana", 0.0 },
  { "Darshanna", 0.0 },
  { "Darshaun", 1.0 },
  { "Darshawn", 1.0 },
  { "Darshay", 0.0 },
  { "Darshea", 0.0 },
  { "Darshell", 0.0 },
  { "Darshelle", 0.0 },
  { "Darshi", 0.0 },
  { "Darshik", 1.0 },
  { "Darshil", 1.0 },
  { "Darshini", 0.0 },
  { "Darshon", 1.0 },
  { "Darsi", 0.0 },
  { "Darsie", 0.0 },
  { "Darson", 1.0 },
  { "Darsy", 0.0 },
  { "Dart", 1.0 },
  { "Dartagnan", 1.0 },
  { "Dartagnon", 1.0 },
  { "Dartanian", 1.0 },
  { "Dartanion", 1.0 },
  { "Dartanya", 0.0 },
  { "Dartanyan", 1.0 },
  { "Dartanyon", 1.0 },
  { "Dartavious", 1.0 },
  { "Dartavius", 1.0 },
  { "Dartez", 1.0 },
  { "Darth", 1.0 },
  { "Dartha", 0.0 },
  { "Darthea", 0.0 },
  { "Darthie", 0.0 },
  { "Darthula", 0.0 },
  { "Darthy", 0.0 },
  { "Dartisha", 0.0 },
  { "Dartrell", 1.0 },
  { "Daruis", 1.0 },
  { "Darus", 1.0 },
  { "Darvel", 1.0 },
  { "Darvell", 1.0 },
  { "Darvens", 1.0 },
  { "Darvi", 0.0 },
  { "Darvin", 1.0 },
  { "Darvis", 1.0 },
  { "Darvon", 1.0 },
  { "Darweshi", 1.0 },
  { "Darwin", 0.9968241 },
  { "Darwing", 1.0 },
  { "Darwood", 1.0 },
  { "Darwuin", 1.0 },
  { "Darwyn", 1.0 },
  { "Dary", 0.85276073 },
  { "Darya", 0.0 },
  { "Daryah", 0.0 },
  { "Daryal", 1.0 },
  { "Daryan", 0.63172805 },
  { "Daryana", 0.0 },
  { "Daryann", 0.0 },
  { "Daryanna", 0.0 },
  { "Daryel", 1.0 },
  { "Daryell", 1.0 },
  { "Daryen", 0.7307692 },
  { "Daryiah", 0.0 },
  { "Daryian", 0.3529412 },
  { "Daryk", 1.0 },
  { "Daryl", 0.9338117 },
  { "Daryla", 0.0 },
  { "Darylann", 0.0 },
  { "Daryle", 0.93003535 },
  { "Darylene", 0.0 },
  { "Daryll", 0.989905 },
  { "Darylle", 1.0 },
  { "Daryn", 0.7160187 },
  { "Daryna", 0.0 },
  { "Darynne", 0.0 },
  { "Daryon", 1.0 },
  { "Daryus", 1.0 },
  { "Darzell", 1.0 },
  { "Dasa", 0.0 },
  { "Dasaan", 1.0 },
  { "Dasael", 1.0 },
  { "Dasai", 1.0 },
  { "Dasan", 1.0 },
  { "Dasani", 0.26450115 },
  { "Dasanii", 0.0 },
  { "Dasany", 0.0 },
  { "Dasaun", 1.0 },
  { "Dascha", 0.0 },
  { "Daschel", 1.0 },
  { "Daschle", 1.0 },
  { "Dasean", 1.0 },
  { "Dasen", 1.0 },
  { "Dash", 0.9787353 },
  { "Dasha", 0.0 },
  { "Dashaan", 1.0 },
  { "Dashad", 1.0 },
  { "Dashae", 0.02016129 },
  { "Dashai", 0.0 },
  { "Dashal", 1.0 },
  { "Dashall", 1.0 },
  { "Dashan", 0.95348835 },
  { "Dashana", 0.0 },
  { "Dashanae", 0.0 },
  { "Dashanay", 0.0 },
  { "Dashanda", 0.0 },
  { "Dashane", 0.6666667 },
  { "Dashani", 0.0 },
  { "Dashanique", 0.0 },
  { "Dashanna", 0.0 },
  { "Dashanta", 0.0 },
  { "Dashanti", 0.0 },
  { "Dashara", 0.0 },
  { "Dashari", 0.0 },
  { "Dasharia", 0.0 },
  { "Dashaun", 0.98811734 },
  { "Dashauna", 0.0 },
  { "Dashawn", 0.9382279 },
  { "Dashawna", 0.0 },
  { "Dashawnda", 0.0 },
  { "Dashawnna", 0.0 },
  { "Dashay", 0.0754717 },
  { "Dashaya", 0.0 },
  { "Dashayla", 0.0 },
  { "Dashea", 0.0 },
  { "Dasheem", 1.0 },
  { "Dasheena", 0.0 },
  { "Dasheika", 0.0 },
  { "Dashel", 0.98678416 },
  { "Dashell", 0.38961038 },
  { "Dashelle", 0.0 },
  { "Dashia", 0.0 },
  { "Dashiah", 0.0 },
  { "Dashiel", 1.0 },
  { "Dashiell", 0.99459726 },
  { "Dashima", 0.0 },
  { "Dashira", 0.0 },
  { "Dashiya", 0.0 },
  { "Dashiyah", 0.0 },
  { "Dashley", 0.0 },
  { "Dashly", 0.0 },
  { "Dashna", 0.0 },
  { "Dashon", 0.9782609 },
  { "Dashona", 0.0 },
  { "Dashonda", 0.0 },
  { "Dashone", 1.0 },
  { "Dashonna", 0.0 },
  { "Dashonte", 0.0 },
  { "Dashown", 1.0 },
  { "Dashton", 1.0 },
  { "Dashuan", 1.0 },
  { "Dashun", 1.0 },
  { "Dashya", 0.0 },
  { "Dashyia", 0.0 },
  { "Dashyra", 0.0 },
  { "Dasia", 0.0 },
  { "Dasiah", 0.0 },
  { "Dasian", 1.0 },
  { "Dasianae", 0.0 },
  { "Dasie", 0.0 },
  { "Dasir", 1.0 },
  { "Dasiya", 0.0 },
  { "Dasiyah", 0.0 },
  { "Dasja", 0.0 },
  { "Dasjah", 0.0 },
  { "Dasjia", 0.0 },
  { "Dasmine", 0.41379312 },
  { "Dasmon", 1.0 },
  { "Dasmond", 1.0 },
  { "Dason", 1.0 },
  { "Dassie", 0.0 },
  { "Dassy", 0.0 },
  { "Dastan", 1.0 },
  { "Dastyn", 1.0 },
  { "Dasya", 0.0 },
  { "Dat", 1.0 },
  { "Datari", 1.0 },
  { "Datavia", 0.0 },
  { "Datavian", 1.0 },
  { "Datavion", 1.0 },
  { "Datavious", 1.0 },
  { "Datavius", 1.0 },
  { "Datha", 0.0 },
  { "Dathan", 1.0 },
  { "Dathel", 0.0 },
  { "Dathen", 1.0 },
  { "Dathol", 0.0 },
  { "Dathon", 1.0 },
  { "Datia", 0.0 },
  { "Datid", 1.0 },
  { "Daton", 1.0 },
  { "Datra", 0.0 },
  { "Datrell", 1.0 },
  { "Datreon", 1.0 },
  { "Datrice", 0.0 },
  { "Datrick", 1.0 },
  { "Datril", 0.54545456 },
  { "Datrion", 1.0 },
  { "Datron", 1.0 },
  { "Datwan", 1.0 },
  { "Datwon", 1.0 },
  { "Daud", 1.0 },
  { "Dauda", 1.0 },
  { "Daudi", 1.0 },
  { "Daughn", 0.0 },
  { "Daughtry", 1.0 },
  { "Daulton", 1.0 },
  { "Daun", 0.03354037 },
  { "Dauna", 0.0 },
  { "Daundra", 0.0 },
  { "Daundre", 1.0 },
  { "Daune", 0.22695035 },
  { "Daunta", 1.0 },
  { "Dauntae", 1.0 },
  { "Dauntay", 1.0 },
  { "Daunte", 1.0 },
  { "Dauphine", 0.0 },
  { "Daureen", 0.0 },
  { "Dauri", 0.0 },
  { "Daurice", 0.055555556 },
  { "Daury", 1.0 },
  { "Dausen", 1.0 },
  { "Dauson", 1.0 },
  { "Dav", 1.0 },
  { "Dava", 0.0 },
  { "Davada", 0.0 },
  { "Davae", 0.09615385 },
  { "Davaeh", 0.0 },
  { "Davahn", 1.0 },
  { "Davalyn", 0.0 },
  { "Davan", 0.76 },
  { "Davana", 0.0 },
  { "Davanee", 0.0 },
  { "Davaney", 0.0 },
  { "Davani", 0.0 },
  { "Davanie", 0.0 },
  { "Davanna", 0.0 },
  { "Davant", 1.0 },
  { "Davanta", 1.0 },
  { "Davantae", 1.0 },
  { "Davantay", 1.0 },
  { "Davante", 1.0 },
  { "Davany", 0.0 },
  { "Davar", 1.0 },
  { "Davares", 1.0 },
  { "Davari", 1.0 },
  { "Davaria", 0.0 },
  { "Davarian", 1.0 },
  { "Davario", 1.0 },
  { "Davarion", 1.0 },
  { "Davarious", 1.0 },
  { "Davaris", 1.0 },
  { "Davarius", 1.0 },
  { "Davarous", 1.0 },
  { "Davarrio", 1.0 },
  { "Davarus", 1.0 },
  { "Davasha", 0.0 },
  { "Davasia", 0.0 },
  { "Davaughn", 1.0 },
  { "Davaun", 1.0 },
  { "Davaunte", 1.0 },
  { "Davaya", 0.0 },
  { "Davayah", 0.0 },
  { "Davd", 1.0 },
  { "Dave", 0.99821544 },
  { "Daveah", 0.0 },
  { "Davean", 1.0 },
  { "Daved", 1.0 },
  { "Daveda", 0.0 },
  { "Davee", 0.52380955 },
  { "Daveed", 1.0 },
  { "Daveen", 0.0 },
  { "Daveena", 0.0 },
  { "Daveigh", 0.0 },
  { "Davein", 1.0 },
  { "Daveion", 1.0 },
  { "Daveisha", 0.0 },
  { "Davel", 1.0 },
  { "Davell", 1.0 },
  { "Davelle", 0.95 },
  { "Davelyn", 0.0 },
  { "Daven", 0.9607843 },
  { "Davena", 0.0 },
  { "Davene", 0.020920502 },
  { "Daveney", 0.0 },
  { "Davenia", 0.0 },
  { "Davens", 1.0 },
  { "Daveon", 0.993361 },
  { "Daveona", 0.0 },
  { "Daveonna", 0.0 },
  { "Daveontae", 1.0 },
  { "Daveonte", 1.0 },
  { "Davesha", 0.0 },
  { "Daveta", 0.0 },
  { "Davetta", 0.0 },
  { "Davette", 0.0 },
  { "Davey", 0.97882426 },
  { "Daveya", 0.0 },
  { "Daveyon", 1.0 },
  { "Davi", 0.6931524 },
  { "Davia", 0.0 },
  { "Daviah", 0.0 },
  { "Davian", 0.98020977 },
  { "Daviana", 0.0 },
  { "Davianna", 0.0 },
  { "Daviante", 1.0 },
  { "Daviaun", 1.0 },
  { "David", 0.99643487 },
  { "Davida", 0.0012575453 },
  { "Davidalexander", 1.0 },
  { "Davidallen", 1.0 },
  { "Davidanthony", 1.0 },
  { "Davidchristophe", 1.0 },
  { "Davide", 1.0 },
  { "Davidjames", 1.0 },
  { "Davidjeremiah", 1.0 },
  { "Davidjohn", 1.0 },
  { "Davidjoseph", 1.0 },
  { "Davidjr", 1.0 },
  { "Davidlee", 1.0 },
  { "Davidmichael", 1.0 },
  { "Davidpaul", 1.0 },
  { "Davidson", 1.0 },
  { "Davie", 0.80986273 },
  { "Davied", 1.0 },
  { "Daviel", 1.0 },
  { "Davielle", 0.0 },
  { "Davien", 1.0 },
  { "Davieon", 1.0 },
  { "Davier", 1.0 },
  { "Daviere", 1.0 },
  { "Davies", 1.0 },
  { "Daviette", 0.0 },
  { "Davika", 0.0 },
  { "Davilyn", 0.0 },
  { "Davin", 0.966185 },
  { "Davina", 0.0 },
  { "Davinah", 0.0 },
  { "Davinci", 1.0 },
  { "Davinder", 1.0 },
  { "Davindra", 1.0 },
  { "Davine", 0.22966507 },
  { "Davinee", 0.0 },
  { "Daviney", 0.0 },
  { "Davinia", 0.0 },
  { "Davinity", 0.0 },
  { "Davinna", 0.0 },
  { "Davio", 1.0 },
  { "Davion", 0.98110074 },
  { "Daviona", 0.0 },
  { "Davione", 0.9578544 },
  { "Davionn", 1.0 },
  { "Davionna", 0.0 },
  { "Davionne", 0.71308017 },
  { "Davionta", 1.0 },
  { "Daviontae", 1.0 },
  { "Davionte", 1.0 },
  { "Davious", 1.0 },
  { "Davis", 0.97196424 },
  { "Davisha", 0.0 },
  { "Davison", 1.0 },
  { "Davit", 1.0 },
  { "Davita", 0.0 },
  { "Davius", 1.0 },
  { "Daviya", 0.0 },
  { "Daviyah", 0.0 },
  { "Daviyana", 0.0 },
  { "Daviyon", 1.0 },
  { "Davlyn", 0.0 },
  { "Davlynn", 0.0 },
  { "Davod", 1.0 },
  { "Davohn", 1.0 },
  { "Davon", 0.97101104 },
  { "Davona", 0.0 },
  { "Davonda", 0.0 },
  { "Davondre", 1.0 },
  { "Davone", 0.9734982 },
  { "Davonia", 0.0 },
  { "Davonn", 1.0 },
  { "Davonna", 0.0 },
  { "Davonne", 0.66211605 },
  { "Davonta", 1.0 },
  { "Davontae", 1.0 },
  { "Davontai", 1.0 },
  { "Davontay", 1.0 },
  { "Davontaye", 1.0 },
  { "Davonte", 0.9988238 },
  { "Davontea", 1.0 },
  { "Davontre", 1.0 },
  { "Davonya", 0.0 },
  { "Davor", 1.0 },
  { "Davron", 1.0 },
  { "Davuan", 1.0 },
  { "Davud", 1.0 },
  { "Davvid", 1.0 },
  { "Davy", 0.94408274 },
  { "Davyan", 1.0 },
  { "Davyd", 1.0 },
  { "Davyen", 1.0 },
  { "Davyeon", 1.0 },
  { "Davyion", 1.0 },
  { "Davyn", 0.8827586 },
  { "Davyne", 0.0 },
  { "Davynn", 0.0 },
  { "Davyon", 1.0 },
  { "Dawain", 1.0 },
  { "Dawan", 0.9576271 },
  { "Dawana", 0.0 },
  { "Dawanda", 0.0 },
  { "Dawane", 1.0 },
  { "Dawann", 1.0 },
  { "Dawanna", 0.0 },
  { "Dawaun", 1.0 },
  { "Dawayne", 1.0 },
  { "Dawens", 1.0 },
  { "Dawes", 1.0 },
  { "Dawid", 1.0 },
  { "Dawin", 1.0 },
  { "Dawit", 1.0 },
  { "Dawlton", 1.0 },
  { "Dawm", 0.0 },
  { "Dawn", 0.0035677564 },
  { "Dawna", 0.0 },
  { "Dawnae", 0.0 },
  { "Dawnda", 0.0 },
  { "Dawndee", 0.0 },
  { "Dawndi", 0.0 },
  { "Dawndra", 0.0 },
  { "Dawndria", 0.0 },
  { "Dawne", 0.0 },
  { "Dawnee", 0.0 },
  { "Dawneen", 0.0 },
  { "Dawneisha", 0.0 },
  { "Dawnel", 0.0 },
  { "Dawnell", 0.0 },
  { "Dawnella", 0.0 },
  { "Dawnelle", 0.0 },
  { "Dawnene", 0.0 },
  { "Dawnesha", 0.0 },
  { "Dawnett", 0.0 },
  { "Dawnetta", 0.0 },
  { "Dawnette", 0.0 },
  { "Dawni", 0.0 },
  { "Dawnica", 0.0 },
  { "Dawnie", 0.0 },
  { "Dawniel", 0.0 },
  { "Dawnielle", 0.0 },
  { "Dawnisha", 0.0 },
  { "Dawnita", 0.0 },
  { "Dawnmarie", 0.0 },
  { "Dawnn", 0.0 },
  { "Dawnna", 0.0 },
  { "Dawnne", 0.0 },
  { "Dawnta", 0.0 },
  { "Dawnte", 0.45454547 },
  { "Dawny", 0.0 },
  { "Dawnya", 0.0 },
  { "Dawnyel", 0.0 },
  { "Dawnyell", 0.0 },
  { "Dawnyelle", 0.0 },
  { "Dawon", 1.0 },
  { "Dawone", 1.0 },
  { "Dawood", 1.0 },
  { "Dawoud", 1.0 },
  { "Dawsen", 0.9863388 },
  { "Dawson", 0.9818685 },
  { "Dawsyn", 0.49119717 },
  { "Dawsynn", 0.0 },
  { "Dawt", 0.0 },
  { "Dawton", 1.0 },
  { "Dawuan", 1.0 },
  { "Dawud", 1.0 },
  { "Dawyn", 0.0 },
  { "Dawyne", 1.0 },
  { "Dax", 0.9983095 },
  { "Daxel", 1.0 },
  { "Daxen", 1.0 },
  { "Daxia", 0.0 },
  { "Daxin", 1.0 },
  { "Daxon", 1.0 },
  { "Daxson", 1.0 },
  { "Daxston", 1.0 },
  { "Daxten", 1.0 },
  { "Daxter", 1.0 },
  { "Daxtin", 1.0 },
  { "Daxton", 1.0 },
  { "Daxtyn", 1.0 },
  { "Daxx", 1.0 },
  { "Daxxon", 1.0 },
  { "Daxxton", 1.0 },
  { "Daxyn", 1.0 },
  { "Day", 0.4797048 },
  { "Daya", 0.0 },
  { "Dayah", 0.0 },
  { "Dayahna", 0.0 },
  { "Dayami", 0.0 },
  { "Dayamin", 0.0 },
  { "Dayamir", 0.0 },
  { "Dayamit", 0.0 },
  { "Dayamy", 0.0 },
  { "Dayan", 0.6238185 },
  { "Dayana", 0.0 },
  { "Dayanah", 0.0 },
  { "Dayanaira", 0.0 },
  { "Dayanara", 0.0 },
  { "Dayanari", 0.0 },
  { "Dayane", 0.0 },
  { "Dayanera", 0.0 },
  { "Dayani", 0.0 },
  { "Dayanie", 0.0 },
  { "Dayanira", 0.0 },
  { "Dayanis", 0.0 },
  { "Dayann", 0.0 },
  { "Dayanna", 0.0 },
  { "Dayannara", 0.0 },
  { "Dayanne", 0.0 },
  { "Dayanni", 0.0 },
  { "Dayany", 0.0 },
  { "Dayari", 0.0 },
  { "Dayasia", 0.0 },
  { "Dayatra", 0.0 },
  { "Daycee", 0.0 },
  { "Daycen", 1.0 },
  { "Dayceon", 1.0 },
  { "Daycia", 0.0 },
  { "Dayde", 1.0 },
  { "Dayden", 1.0 },
  { "Daydra", 0.0 },
  { "Daydrian", 1.0 },
  { "Daye", 0.0 },
  { "Dayeli", 0.0 },
  { "Dayelin", 0.0 },
  { "Dayesha", 0.0 },
  { "Daygan", 1.0 },
  { "Daygen", 1.0 },
  { "Daygoro", 1.0 },
  { "Dayiana", 0.0 },
  { "Dayion", 1.0 },
  { "Dayja", 0.0 },
  { "Dayjah", 0.0 },
  { "Dayjon", 1.0 },
  { "Daykota", 1.0 },
  { "Dayl", 0.25 },
  { "Dayla", 0.0 },
  { "Daylah", 0.0 },
  { "Daylan", 0.96368355 },
  { "Dayland", 1.0 },
  { "Daylani", 0.0 },
  { "Daylanie", 0.0 },
  { "Daylany", 0.0 },
  { "Dayle", 0.3451515 },
  { "Daylee", 0.016574586 },
  { "Dayleen", 0.0 },
  { "Dayleigh", 0.0 },
  { "Daylen", 0.91413116 },
  { "Daylene", 0.0 },
  { "Dayleon", 1.0 },
  { "Dayli", 0.0 },
  { "Daylia", 0.0 },
  { "Daylie", 0.0 },
  { "Daylin", 0.5710121 },
  { "Dayline", 0.0 },
  { "Daylinn", 0.0 },
  { "Dayln", 0.9325843 },
  { "Daylon", 0.99632895 },
  { "Daylyn", 0.46882793 },
  { "Daylynn", 0.09034268 },
  { "Dayman", 1.0 },
  { "Daymar", 1.0 },
  { "Daymein", 1.0 },
  { "Daymen", 1.0 },
  { "Daymeon", 1.0 },
  { "Daymi", 0.0 },
  { "Daymian", 1.0 },
  { "Daymien", 1.0 },
  { "Daymin", 1.0 },
  { "Daymion", 1.0 },
  { "Daymon", 1.0 },
  { "Daymond", 1.0 },
  { "Dayn", 1.0 },
  { "Dayna", 0.006104835 },
  { "Daynah", 0.0 },
  { "Dayne", 0.949232 },
  { "Daynesha", 0.0 },
  { "Daynisha", 0.0 },
  { "Daynna", 0.0 },
  { "Dayo", 0.0 },
  { "Dayon", 0.9833837 },
  { "Dayona", 0.0 },
  { "Dayonna", 0.0 },
  { "Dayonte", 1.0 },
  { "Dayquan", 1.0 },
  { "Dayquon", 1.0 },
  { "Dayqwan", 1.0 },
  { "Dayra", 0.0 },
  { "Dayren", 1.0 },
  { "Dayri", 0.0 },
  { "Dayrin", 0.0 },
  { "Dayris", 0.0 },
  { "Dayro", 1.0 },
  { "Dayron", 1.0 },
  { "Daysan", 1.0 },
  { "Dayse", 0.0 },
  { "Daysean", 1.0 },
  { "Daysen", 1.0 },
  { "Daysha", 0.0 },
  { "Dayshana", 0.0 },
  { "Dayshanay", 0.0 },
  { "Dayshaun", 1.0 },
  { "Dayshawn", 0.99597096 },
  { "Dayshawna", 0.0 },
  { "Dayshia", 0.0 },
  { "Dayshon", 1.0 },
  { "Dayshun", 1.0 },
  { "Daysi", 0.0 },
  { "Daysia", 0.0 },
  { "Daysie", 0.0 },
  { "Dayson", 1.0 },
  { "Daysun", 1.0 },
  { "Daysy", 0.0 },
  { "Dayten", 1.0 },
  { "Daythan", 1.0 },
  { "Daytin", 1.0 },
  { "Dayton", 0.96559924 },
  { "Daytona", 0.25707546 },
  { "Daytron", 1.0 },
  { "Daytwan", 1.0 },
  { "Daytwon", 1.0 },
  { "Dayva", 0.0 },
  { "Dayvanee", 0.0 },
  { "Dayveion", 1.0 },
  { "Dayven", 1.0 },
  { "Dayveon", 1.0 },
  { "Dayvian", 1.0 },
  { "Dayvid", 1.0 },
  { "Dayvien", 1.0 },
  { "Dayvin", 1.0 },
  { "Dayvion", 1.0 },
  { "Dayvon", 1.0 },
  { "Dayvonna", 0.0 },
  { "Dayvonne", 1.0 },
  { "Dayyan", 1.0 },
  { "Dayza", 0.0 },
  { "Dayzah", 0.0 },
  { "Dayzee", 0.0 },
  { "Dayzha", 0.0 },
  { "Dayzhane", 0.0 },
  { "Dayzi", 0.0 },
  { "Dayzia", 0.0 },
  { "Dayzie", 0.0 },
  { "Daz", 1.0 },
  { "Dazah", 0.0 },
  { "Dazani", 0.0 },
  { "Dazaria", 0.0 },
  { "Dazariah", 0.0 },
  { "Dazarion", 1.0 },
  { "Dazavion", 1.0 },
  { "Dazay", 0.0 },
  { "Dazaya", 0.0 },
  { "Dazha", 0.0 },
  { "Dazhan", 1.0 },
  { "Dazhanae", 0.0 },
  { "Dazhane", 0.0 },
  { "Dazhaun", 1.0 },
  { "Dazhia", 0.0 },
  { "Dazhon", 1.0 },
  { "Dazia", 0.0 },
  { "Daziah", 0.0 },
  { "Dazie", 0.0 },
  { "Daziel", 1.0 },
  { "Dazion", 1.0 },
  { "Dazire", 0.0 },
  { "Daziya", 0.0 },
  { "Daziyah", 0.0 },
  { "Dazja", 0.0 },
  { "Dazjah", 0.0 },
  { "Dazlyn", 0.0 },
  { "Dazlynn", 0.0 },
  { "Dazmine", 0.0 },
  { "Dazmon", 1.0 },
  { "Dazmond", 1.0 },
  { "Dazon", 1.0 },
  { "Dazsha", 0.0 },
  { "Dazya", 0.0 },
  { "Dazza", 0.0 },
  { "Dazzlyn", 0.0 },
  { "Dcarlo", 1.0 },
  { "Dcarlos", 1.0 },
  { "Dcorian", 1.0 },
  { "Ddnald", 1.0 },
  { "Ddnna", 0.0 },
  { "De", 0.5151286 },
  { "Dea", 0.010059171 },
  { "Deaaron", 1.0 },
  { "Deacan", 1.0 },
  { "Deacon", 1.0 },
  { "Deadra", 0.0 },
  { "Deadre", 0.0 },
  { "Deadria", 0.0 },
  { "Deadrian", 1.0 },
  { "Deadrick", 1.0 },
  { "Deagan", 1.0 },
  { "Deagen", 1.0 },
  { "Deaglan", 1.0 },
  { "Deago", 1.0 },
  { "Deagon", 1.0 },
  { "Deah", 0.0 },
  { "Deahna", 0.0 },
  { "Deaija", 0.0 },
  { "Deaijah", 0.0 },
  { "Deair", 1.0 },
  { "Deaira", 0.0 },
  { "Deairah", 0.0 },
  { "Deaire", 1.0 },
  { "Deaires", 1.0 },
  { "Deairra", 0.0 },
  { "Deaisa", 0.0 },
  { "Deaisha", 0.0 },
  { "Deaja", 0.0 },
  { "Deajah", 0.0 },
  { "Deajanae", 0.0 },
  { "Deajane", 0.0 },
  { "Deajha", 0.0 },
  { "Deajia", 0.0 },
  { "Deajon", 1.0 },
  { "Deaken", 1.0 },
  { "Deakin", 1.0 },
  { "Deakon", 1.0 },
  { "Deakyn", 1.0 },
  { "Deal", 1.0 },
  { "Dealia", 0.0 },
  { "Dealie", 0.0 },
  { "Deallen", 1.0 },
  { "Dealva", 0.0 },
  { "Deamber", 0.0 },
  { "Deambra", 0.0 },
  { "Deamontae", 1.0 },
  { "Deamonte", 1.0 },
  { "Dean", 0.9835261 },
  { "Deana", 0.0006001886 },
  { "Deanah", 0.0 },
  { "Deanca", 0.0 },
  { "Deanda", 0.0 },
  { "Deandra", 0.14834298 },
  { "Deandrae", 1.0 },
  { "Deandray", 1.0 },
  { "Deandre", 0.9835229 },
  { "Deandrea", 0.24340075 },
  { "Deandres", 1.0 },
  { "Deandrew", 1.0 },
  { "Deandrey", 1.0 },
  { "Deandria", 0.0 },
  { "Deandrick", 1.0 },
  { "Deane", 0.6757832 },
  { "Deaneen", 0.0 },
  { "Deanesha", 0.0 },
  { "Deanette", 0.0 },
  { "Deangalo", 1.0 },
  { "Deangel", 0.0 },
  { "Deangela", 0.0 },
  { "Deangelia", 0.0 },
  { "Deangelio", 1.0 },
  { "Deangelis", 0.0 },
  { "Deangello", 1.0 },
  { "Deangelo", 0.9980841 },
  { "Deangilo", 1.0 },
  { "Deangleo", 1.0 },
  { "Deanglo", 1.0 },
  { "Deani", 0.0 },
  { "Deania", 0.0 },
  { "Deanie", 0.02670227 },
  { "Deanine", 0.0 },
  { "Deanise", 0.0 },
  { "Deanjelo", 1.0 },
  { "Deann", 0.0 },
  { "Deanna", 0.0019893353 },
  { "Deannah", 0.0 },
  { "Deanndra", 0.0 },
  { "Deanne", 0.00033708624 },
  { "Deannea", 0.0 },
  { "Deannette", 0.0 },
  { "Deannia", 0.0 },
  { "Deannie", 0.0 },
  { "Deano", 1.0 },
  { "Deanta", 1.0 },
  { "Deantae", 1.0 },
  { "Deantay", 1.0 },
  { "Deante", 0.9980292 },
  { "Deanthony", 1.0 },
  { "Deantoine", 1.0 },
  { "Deantonio", 1.0 },
  { "Deantony", 1.0 },
  { "Deantrae", 1.0 },
  { "Deantre", 1.0 },
  { "Deantwan", 1.0 },
  { "Deantwon", 1.0 },
  { "Deanza", 0.0 },
  { "Deaquan", 1.0 },
  { "Deara", 0.0 },
  { "Deardra", 0.0 },
  { "Deareon", 1.0 },
  { "Deari", 0.083333336 },
  { "Dearia", 0.0 },
  { "Deariah", 0.0 },
  { "Dearie", 0.0 },
  { "Dearies", 1.0 },
  { "Dearion", 0.925 },
  { "Dearious", 1.0 },
  { "Dearis", 1.0 },
  { "Dearius", 1.0 },
  { "Dearl", 1.0 },
  { "Dearld", 1.0 },
  { "Dearmond", 1.0 },
  { "Dearon", 1.0 },
  { "Dearra", 0.0 },
  { "Dearria", 0.0 },
  { "Dearrius", 1.0 },
  { "Dearron", 1.0 },
  { "Deasha", 0.0 },
  { "Deashawn", 1.0 },
  { "Deashia", 0.0 },
  { "Deashley", 0.0 },
  { "Deasia", 0.0 },
  { "Deasiah", 0.0 },
  { "Deasja", 0.0 },
  { "Deason", 1.0 },
  { "Deasya", 0.0 },
  { "Deaton", 1.0 },
  { "Deatra", 0.0 },
  { "Deatria", 0.0 },
  { "Deatrice", 0.0 },
  { "Deatrick", 1.0 },
  { "Deatta", 0.0 },
  { "Deaudra", 0.0 },
  { "Deaudre", 1.0 },
  { "Deaun", 0.0 },
  { "Deauna", 0.0 },
  { "Deaundra", 0.2739726 },
  { "Deaundray", 1.0 },
  { "Deaundre", 1.0 },
  { "Deaundrea", 0.15384616 },
  { "Deaundrey", 1.0 },
  { "Deaundria", 0.0 },
  { "Deaunna", 0.0 },
  { "Deaunta", 1.0 },
  { "Deauntae", 1.0 },
  { "Deaunte", 1.0 },
  { "Deauntre", 1.0 },
  { "Deautre", 1.0 },
  { "Deavan", 1.0 },
  { "Deavante", 1.0 },
  { "Deaven", 0.75697213 },
  { "Deaveon", 0.6969697 },
  { "Deavian", 0.0 },
  { "Deavin", 0.9621212 },
  { "Deavion", 0.63721806 },
  { "Deavionne", 0.0 },
  { "Deavon", 0.93363845 },
  { "Deavonte", 1.0 },
  { "Deavyn", 0.0 },
  { "Deawn", 0.0 },
  { "Deayra", 0.0 },
  { "Deaysha", 0.0 },
  { "Deaysia", 0.0 },
  { "Deazia", 0.0 },
  { "Deb", 0.0 },
  { "Deba", 0.0 },
  { "Debanhi", 0.0 },
  { "Debanhy", 0.0 },
  { "Debany", 0.0 },
  { "Debar", 0.0 },
  { "Debara", 0.0 },
  { "Debarah", 0.0 },
  { "Debarh", 0.0 },
  { "Debaroh", 0.0 },
  { "Debb", 0.0 },
  { "Debbe", 0.0 },
  { "Debbee", 0.0 },
  { "Debbera", 0.0 },
  { "Debbey", 0.0 },
  { "Debbi", 0.0 },
  { "Debbie", 0.002307774 },
  { "Debbora", 0.0 },
  { "Debborah", 0.0 },
  { "Debborrah", 0.0 },
  { "Debbra", 0.0 },
  { "Debbrah", 0.0 },
  { "Debby", 0.0 },
  { "Debbye", 0.0 },
  { "Debe", 0.0 },
  { "Debera", 0.0 },
  { "Deberah", 0.0 },
  { "Debhora", 0.0 },
  { "Debhra", 0.0 },
  { "Debi", 0.0 },
  { "Debie", 0.0 },
  { "Deboah", 0.0 },
  { "Deboarh", 0.0 },
  { "Debony", 0.0 },
  { "Debooah", 0.0 },
  { "Debor", 0.0 },
  { "Debora", 0.00095322396 },
  { "Deboraa", 0.0 },
  { "Deboraah", 0.0 },
  { "Deborah", 0.0022355046 },
  { "Deboraha", 0.0 },
  { "Deborahann", 0.0 },
  { "Deborahh", 0.0 },
  { "Deboral", 0.0 },
  { "Deboran", 0.0 },
  { "Deborh", 0.0 },
  { "Deborha", 0.0 },
  { "Deboria", 0.0 },
  { "Deboris", 1.0 },
  { "Deboroh", 0.0 },
  { "Deborra", 0.0 },
  { "Deborrah", 0.0 },
  { "Debr", 0.0 },
  { "Debra", 0.0020782102 },
  { "Debraa", 0.0 },
  { "Debraann", 0.0 },
  { "Debraca", 0.0 },
  { "Debrah", 0.0 },
  { "Debraha", 0.0 },
  { "Debrajo", 0.0 },
  { "Debralee", 0.0 },
  { "Debralyn", 0.0 },
  { "Debrann", 0.0 },
  { "Debraoh", 0.0 },
  { "Debrea", 0.0 },
  { "Debrha", 0.0 },
  { "Debria", 0.0 },
  { "Debriana", 0.0 },
  { "Debrianna", 0.0 },
  { "Debrina", 0.0 },
  { "Debrisha", 0.0 },
  { "Debro", 0.0 },
  { "Debroa", 0.0 },
  { "Debroah", 0.0 },
  { "Debroh", 0.0 },
  { "Debroha", 0.0 },
  { "Debron", 1.0 },
  { "Debrorah", 0.0 },
  { "Debrra", 0.0 },
  { "Debs", 1.0 },
  { "Deby", 0.0 },
  { "Decameron", 1.0 },
  { "Decan", 1.0 },
  { "Decara", 0.0 },
  { "Decari", 1.0 },
  { "Decario", 1.0 },
  { "Decarion", 1.0 },
  { "Decarius", 1.0 },
  { "Decarla", 0.0 },
  { "Decarlo", 1.0 },
  { "Decarlos", 1.0 },
  { "Decarri", 1.0 },
  { "Decarter", 1.0 },
  { "Decatur", 1.0 },
  { "Decca", 0.0 },
  { "Decedric", 1.0 },
  { "December", 0.0045011253 },
  { "Dechane", 1.0 },
  { "Dechanel", 0.0 },
  { "Dechaun", 1.0 },
  { "Dechelle", 0.0 },
  { "Dechen", 0.0 },
  { "Dechlan", 1.0 },
  { "Decia", 0.0 },
  { "Deciderio", 1.0 },
  { "Decie", 0.0 },
  { "Decima", 0.0 },
  { "Decimus", 1.0 },
  { "Deckard", 1.0 },
  { "Decker", 0.98519164 },
  { "Decklan", 1.0 },
  { "Decklen", 1.0 },
  { "Decklin", 1.0 },
  { "Decklyn", 0.7091837 },
  { "Declan", 0.99528384 },
  { "Declen", 1.0 },
  { "Declin", 1.0 },
  { "Declyn", 0.6354515 },
  { "Declynn", 0.0 },
  { "Decoda", 1.0 },
  { "Decody", 1.0 },
  { "Decola", 0.0 },
  { "Decole", 0.0 },
  { "Decora", 0.0 },
  { "Decorey", 1.0 },
  { "Decorian", 1.0 },
  { "Decorion", 1.0 },
  { "Decorius", 1.0 },
  { "Decory", 1.0 },
  { "Decota", 0.8780488 },
  { "Dectrick", 1.0 },
  { "Deda", 0.0 },
  { "Dedan", 1.0 },
  { "Deddie", 0.0 },
  { "Deddrick", 1.0 },
  { "Dede", 0.0 },
  { "Dedee", 0.0 },
  { "Dedera", 0.0 },
  { "Dederick", 1.0 },
  { "Dedi", 0.0 },
  { "Dedie", 0.0 },
  { "Dedorah", 0.0 },
  { "Dedra", 0.0 },
  { "Dedrah", 0.0 },
  { "Dedre", 0.0 },
  { "Dedrea", 0.0 },
  { "Dedrek", 1.0 },
  { "Dedria", 0.0 },
  { "Dedrian", 1.0 },
  { "Dedriana", 0.0 },
  { "Dedrianna", 0.0 },
  { "Dedric", 0.98896855 },
  { "Dedrick", 0.996245 },
  { "Dedrie", 0.0 },
  { "Dee", 0.3137 },
  { "Deea", 0.0 },
  { "Deeana", 0.0 },
  { "Deeandra", 0.0 },
  { "Deeandre", 1.0 },
  { "Deeann", 0.0 },
  { "Deeanna", 0.0 },
  { "Deeanne", 0.0 },
  { "Deeasia", 0.0 },
  { "Deeba", 0.0 },
  { "Deeda", 0.0 },
  { "Deeddra", 0.0 },
  { "Deede", 0.0 },
  { "Deedee", 0.0 },
  { "Deedgra", 0.0 },
  { "Deedie", 0.0 },
  { "Deedra", 0.0 },
  { "Deedre", 0.0 },
  { "Deedrick", 1.0 },
  { "Deedy", 0.0 },
  { "Deegan", 0.99517876 },
  { "Deejay", 0.96 },
  { "Deeken", 1.0 },
  { "Deeksha", 0.0 },
  { "Deekshitha", 0.0 },
  { "Deelan", 1.0 },
  { "Deelda", 0.0 },
  { "Deelilah", 0.0 },
  { "Deella", 0.0 },
  { "Deelynn", 0.0 },
  { "Deem", 0.0 },
  { "Deema", 0.0 },
  { "Deemah", 0.0 },
  { "Deems", 1.0 },
  { "Deen", 0.9891068 },
  { "Deena", 0.00037372002 },
  { "Deenah", 0.0 },
  { "Deene", 0.0 },
  { "Deeneen", 0.0 },
  { "Deenie", 0.0 },
  { "Deenna", 0.0 },
  { "Deep", 1.0 },
  { "Deepa", 0.0 },
  { "Deepak", 1.0 },
  { "Deepesh", 1.0 },
  { "Deepika", 0.0 },
  { "Deepshika", 0.0 },
  { "Deepthi", 0.0 },
  { "Deepti", 0.0 },
  { "Deeqa", 0.0 },
  { "Deera", 0.0 },
  { "Deeric", 1.0 },
  { "Deerica", 0.0 },
  { "Deerick", 1.0 },
  { "Deericka", 0.0 },
  { "Deerika", 0.0 },
  { "Deesha", 0.0 },
  { "Deeshawn", 1.0 },
  { "Deethya", 0.0 },
  { "Deetra", 0.0 },
  { "Deett", 0.0 },
  { "Deetta", 0.0 },
  { "Deette", 0.0 },
  { "Deetya", 0.0 },
  { "Deeva", 0.0 },
  { "Deevon", 1.0 },
  { "Deeya", 0.0 },
  { "Defne", 0.0 },
  { "Defonta", 1.0 },
  { "Deforest", 1.0 },
  { "Deforrest", 1.0 },
  { "Degan", 1.0 },
  { "Degen", 1.0 },
  { "Deguan", 1.0 },
  { "Dehaven", 1.0 },
  { "Dehja", 0.0 },
  { "Dehlia", 0.0 },
  { "Dehlila", 0.0 },
  { "Deia", 0.0 },
  { "Deiadra", 0.0 },
  { "Deiah", 0.0 },
  { "Deiana", 0.0 },
  { "Deianeira", 0.0 },
  { "Deianira", 0.0 },
  { "Deianna", 0.0 },
  { "Deiara", 0.0 },
  { "Deiby", 1.0 },
  { "Deici", 0.0 },
  { "Deicy", 0.0 },
  { "Deida", 0.0 },
  { "Deidra", 0.0 },
  { "Deidre", 0.0007502899 },
  { "Deidrea", 0.0 },
  { "Deidree", 0.0 },
  { "Deidri", 0.0 },
  { "Deidria", 0.0 },
  { "Deidrick", 1.0 },
  { "Deighton", 1.0 },
  { "Deigo", 1.0 },
  { "Deija", 0.0 },
  { "Deijah", 0.0 },
  { "Deijon", 1.0 },
  { "Deila", 0.0 },
  { "Deilani", 0.0 },
  { "Deilany", 0.0 },
  { "Deilyn", 0.0 },
  { "Deimos", 1.0 },
  { "Deimy", 0.0 },
  { "Dein", 1.0 },
  { "Deina", 0.0 },
  { "Deion", 0.97646385 },
  { "Deiona", 0.0 },
  { "Deiondra", 0.0 },
  { "Deiondre", 1.0 },
  { "Deione", 1.0 },
  { "Deionna", 0.0 },
  { "Deionta", 1.0 },
  { "Deiontae", 1.0 },
  { "Deiontay", 1.0 },
  { "Deionte", 1.0 },
  { "Deiontre", 1.0 },
  { "Deira", 0.0 },
  { "Deirde", 0.0 },
  { "Deirdra", 0.0 },
  { "Deirdre", 0.0 },
  { "Deirore", 0.0 },
  { "Deirra", 0.0 },
  { "Deise", 0.0 },
  { "Deisel", 1.0 },
  { "Deisha", 0.0 },
  { "Deisi", 0.0 },
  { "Deisree", 0.0 },
  { "Deissy", 0.0 },
  { "Deisy", 0.0 },
  { "Deith", 1.0 },
  { "Deitra", 0.0 },
  { "Deitre", 0.0 },
  { "Deitric", 1.0 },
  { "Deitrich", 1.0 },
  { "Deitrick", 1.0 },
  { "Deiveon", 1.0 },
  { "Deivi", 1.0 },
  { "Deivid", 1.0 },
  { "Deivin", 1.0 },
  { "Deivion", 0.5882353 },
  { "Deivy", 1.0 },
  { "Deizel", 1.0 },
  { "Deja", 0.0054987213 },
  { "Dejaa", 0.0 },
  { "Dejae", 0.0 },
  { "Dejah", 0.0016528926 },
  { "Dejahn", 1.0 },
  { "Dejahna", 0.0 },
  { "Dejahnae", 0.0 },
  { "Dejahnay", 0.0 },
  { "Dejahne", 0.0 },
  { "Dejahnique", 0.0 },
  { "Dejai", 0.0 },
  { "Dejamarie", 0.0 },
  { "Dejan", 0.9597523 },
  { "Dejana", 0.0 },
  { "Dejanae", 0.0 },
  { "Dejanai", 0.0 },
  { "Dejanay", 0.0 },
  { "Dejanaye", 0.0 },
  { "Dejane", 0.0 },
  { "Dejanea", 0.0 },
  { "Dejanee", 0.0 },
  { "Dejaneira", 0.0 },
  { "Dejanelle", 0.0 },
  { "Dejania", 0.0 },
  { "Dejanique", 0.0 },
  { "Dejanira", 0.0 },
  { "Dejanna", 0.0 },
  { "Dejarae", 0.0 },
  { "Dejarvis", 1.0 },
  { "Dejaun", 1.0 },
  { "Dejavion", 1.0 },
  { "Dejay", 1.0 },
  { "Dejean", 1.0 },
  { "Dejenae", 0.0 },
  { "Dejesus", 1.0 },
  { "Dejha", 0.0 },
  { "Dejhana", 0.0 },
  { "Dejhanae", 0.0 },
  { "Dejhon", 1.0 },
  { "Dejia", 0.0 },
  { "Dejiah", 0.0 },
  { "Dejion", 1.0 },
  { "Dejohn", 1.0 },
  { "Dejoire", 0.0 },
  { "Dejon", 0.96898913 },
  { "Dejona", 0.0 },
  { "Dejonae", 0.0 },
  { "Dejone", 0.0 },
  { "Dejonna", 0.0 },
  { "Dejonnae", 0.0 },
  { "Dejontae", 1.0 },
  { "Dejonte", 1.0 },
  { "Dejoun", 1.0 },
  { "Dejour", 0.9414226 },
  { "Dejsha", 0.0 },
  { "Dejuan", 0.99228543 },
  { "Dejuana", 0.0 },
  { "Dejuane", 1.0 },
  { "Dejuante", 1.0 },
  { "Dejun", 1.0 },
  { "Dejuna", 0.0 },
  { "Dekai", 1.0 },
  { "Dekan", 1.0 },
  { "Dekanye", 1.0 },
  { "Dekara", 0.0 },
  { "Dekari", 1.0 },
  { "Dekayden", 1.0 },
  { "Dekayla", 0.0 },
  { "Deke", 1.0 },
  { "Dekeisha", 0.0 },
  { "Dekel", 1.0 },
  { "Deken", 1.0 },
  { "Dekendra", 0.0 },
  { "Dekendrick", 1.0 },
  { "Deker", 1.0 },
  { "Dekesha", 0.0 },
  { "Dekeshia", 0.0 },
  { "Dekevion", 1.0 },
  { "Dekevious", 1.0 },
  { "Dekhari", 1.0 },
  { "Dekia", 0.0 },
  { "Dekin", 1.0 },
  { "Deking", 1.0 },
  { "Dekira", 0.0 },
  { "Dekisha", 0.0 },
  { "Dekita", 0.0 },
  { "Dekiya", 0.0 },
  { "Dekiyah", 0.0 },
  { "Dekker", 1.0 },
  { "Deklan", 1.0 },
  { "Deklen", 1.0 },
  { "Deklin", 1.0 },
  { "Deklyn", 0.81489843 },
  { "Deklynn", 0.0 },
  { "Dekobe", 1.0 },
  { "Dekoda", 0.81666666 },
  { "Dekon", 1.0 },
  { "Dekorion", 1.0 },
  { "Dekota", 0.73 },
  { "Dekotah", 0.0 },
  { "Dekoven", 1.0 },
  { "Dekwan", 1.0 },
  { "Dekwon", 1.0 },
  { "Dekyla", 0.0 },
  { "Dekyra", 0.0 },
  { "Del", 0.86209726 },
  { "Dela", 0.01091703 },
  { "Delacey", 0.0 },
  { "Delacy", 0.2 },
  { "Delaenie", 0.0 },
  { "Delaia", 0.0 },
  { "Delaiah", 0.0 },
  { "Delaila", 0.0 },
  { "Delailah", 0.0 },
  { "Delaili", 0.0 },
  { "Delain", 0.14285715 },
  { "Delaina", 0.0 },
  { "Delainah", 0.0 },
  { "Delaine", 0.093472384 },
  { "Delainee", 0.0 },
  { "Delainey", 0.0 },
  { "Delaini", 0.0 },
  { "Delainie", 0.0 },
  { "Delainy", 0.0 },
  { "Delajah", 0.0 },
  { "Delali", 0.0 },
  { "Delan", 1.0 },
  { "Delana", 0.0 },
  { "Delance", 1.0 },
  { "Delancey", 0.26666668 },
  { "Delancy", 0.23809524 },
  { "Deland", 1.0 },
  { "Delanda", 0.0 },
  { "Delando", 1.0 },
  { "Delandra", 0.0 },
  { "Delandre", 1.0 },
  { "Delandria", 0.0 },
  { "Delane", 0.49490616 },
  { "Delanea", 0.0 },
  { "Delanee", 0.0 },
  { "Delanei", 0.0 },
  { "Delaney", 0.031278305 },
  { "Delani", 0.013623978 },
  { "Delania", 0.0 },
  { "Delanie", 0.0 },
  { "Delann", 0.0 },
  { "Delanna", 0.0 },
  { "Delanne", 0.0 },
  { "Delannie", 0.0 },
  { "Delanny", 0.0 },
  { "Delano", 0.98653775 },
  { "Delanor", 1.0 },
  { "Delanta", 1.0 },
  { "Delantae", 1.0 },
  { "Delante", 1.0 },
  { "Delany", 0.0 },
  { "Delanya", 0.0 },
  { "Delaphine", 0.0 },
  { "Delara", 0.0 },
  { "Delaris", 0.0 },
  { "Delasha", 0.0 },
  { "Delasia", 0.0 },
  { "Delaun", 1.0 },
  { "Delaura", 0.0 },
  { "Delauren", 0.0 },
  { "Delavan", 1.0 },
  { "Delavina", 0.0 },
  { "Delawrence", 1.0 },
  { "Delaya", 0.0 },
  { "Delayah", 0.0 },
  { "Delayla", 0.0 },
  { "Delaylah", 0.0 },
  { "Delayna", 0.0 },
  { "Delayne", 0.08523592 },
  { "Delaynee", 0.0 },
  { "Delayney", 0.0 },
  { "Delayni", 0.0 },
  { "Delaynie", 0.0 },
  { "Delaysha", 0.0 },
  { "Delaysia", 0.0 },
  { "Delayza", 0.0 },
  { "Delba", 0.0 },
  { "Delbert", 0.9973975 },
  { "Delberta", 0.0 },
  { "Delborah", 0.0 },
  { "Delbra", 0.0 },
  { "Delbreco", 1.0 },
  { "Delcan", 1.0 },
  { "Delcenia", 0.0 },
  { "Delci", 0.0 },
  { "Delcia", 0.0 },
  { "Delcie", 0.0 },
  { "Delcina", 0.0 },
  { "Delcine", 0.0 },
  { "Delcy", 0.0 },
  { "Deldon", 1.0 },
  { "Deldra", 0.0 },
  { "Deldrick", 1.0 },
  { "Delea", 0.0 },
  { "Deleah", 0.0 },
  { "Delean", 0.0 },
  { "Deleana", 0.0 },
  { "Deleatrice", 0.0 },
  { "Delecia", 0.0 },
  { "Delee", 0.0 },
  { "Deleen", 0.0 },
  { "Deleena", 0.0 },
  { "Deleesa", 0.0 },
  { "Delema", 0.0 },
  { "Delena", 0.0 },
  { "Delene", 0.0 },
  { "Delenia", 0.0 },
  { "Delenn", 0.0 },
  { "Delenna", 0.0 },
  { "Deleno", 1.0 },
  { "Deleon", 0.9840365 },
  { "Delephine", 0.0 },
  { "Delesa", 0.0 },
  { "Delesha", 0.0 },
  { "Deleshia", 0.0 },
  { "Delesia", 0.0 },
  { "Deleta", 0.0 },
  { "Deletha", 0.0 },
  { "Delethia", 0.0 },
  { "Deletta", 0.0 },
  { "Delexis", 0.0 },
  { "Delexus", 0.0 },
  { "Deleyza", 0.0 },
  { "Delfa", 0.0 },
  { "Delfin", 1.0 },
  { "Delfina", 0.0 },
  { "Delfinia", 0.0 },
  { "Delfino", 1.0 },
  { "Delford", 1.0 },
  { "Delfred", 1.0 },
  { "Delfreda", 0.0 },
  { "Delia", 0.0017220309 },
  { "Deliah", 0.0 },
  { "Deliana", 0.0 },
  { "Delianna", 0.0 },
  { "Deliany", 0.0 },
  { "Delianys", 0.0 },
  { "Delica", 0.0 },
  { "Delice", 0.0 },
  { "Delicia", 0.0 },
  { "Delie", 0.0 },
  { "Delight", 0.0 },
  { "Delijah", 0.4084507 },
  { "Delila", 0.0 },
  { "Delilah", 0.0 },
  { "Delilahrose", 0.0 },
  { "Delilha", 0.0 },
  { "Delilia", 0.0 },
  { "Deliliah", 0.0 },
  { "Delima", 0.0 },
  { "Delimar", 0.0 },
  { "Delina", 0.0 },
  { "Delinah", 0.0 },
  { "Delinda", 0.0 },
  { "Delinia", 0.0 },
  { "Delino", 1.0 },
  { "Delio", 1.0 },
  { "Delisa", 0.0 },
  { "Delise", 0.0 },
  { "Delisha", 0.0 },
  { "Delishia", 0.0 },
  { "Delisia", 0.0 },
  { "Delissa", 0.0 },
  { "Delissia", 0.0 },
  { "Delita", 0.0 },
  { "Delite", 0.0 },
  { "Delitha", 0.0 },
  { "Deliyah", 0.0 },
  { "Deliylah", 0.0 },
  { "Deliza", 0.0 },
  { "Deljuan", 1.0 },
  { "Dell", 0.60915655 },
  { "Della", 0.002062651 },
  { "Dellah", 0.0 },
  { "Dellamae", 0.0 },
  { "Dellan", 1.0 },
  { "Dellana", 0.0 },
  { "Dellaney", 0.0 },
  { "Dellanie", 0.0 },
  { "Dellanira", 0.0 },
  { "Dellar", 0.0 },
  { "Dellareese", 0.0 },
  { "Dellas", 1.0 },
  { "Delle", 0.0 },
  { "Dellen", 1.0 },
  { "Dellena", 0.0 },
  { "Dellene", 0.0 },
  { "Deller", 0.0 },
  { "Dellia", 0.0 },
  { "Dellie", 0.017054264 },
  { "Dellis", 1.0 },
  { "Dellon", 1.0 },
  { "Dellora", 0.0 },
  { "Delloyd", 1.0 },
  { "Delma", 0.1463242 },
  { "Delman", 1.0 },
  { "Delmar", 0.99055606 },
  { "Delmarco", 1.0 },
  { "Delmarie", 0.0 },
  { "Delmario", 1.0 },
  { "Delmas", 1.0 },
  { "Delmer", 0.9979036 },
  { "Delmi", 0.0 },
  { "Delmira", 0.0 },
  { "Delmis", 0.5882353 },
  { "Delmo", 1.0 },
  { "Delmon", 1.0 },
  { "Delmond", 1.0 },
  { "Delmonica", 0.0 },
  { "Delmont", 1.0 },
  { "Delmonta", 1.0 },
  { "Delmonte", 1.0 },
  { "Delmor", 1.0 },
  { "Delmore", 1.0 },
  { "Delmos", 1.0 },
  { "Delmus", 1.0 },
  { "Delmy", 0.0 },
  { "Delna", 0.0 },
  { "Delno", 1.0 },
  { "Deloice", 0.0 },
  { "Delois", 0.014149849 },
  { "Deloise", 0.0 },
  { "Deloma", 0.0 },
  { "Delon", 0.9709035 },
  { "Delona", 0.018382354 },
  { "Delonda", 0.0 },
  { "Delondo", 1.0 },
  { "Delone", 0.52873564 },
  { "Deloni", 1.0 },
  { "Delonia", 0.0 },
  { "Delonna", 0.0 },
  { "Delonta", 1.0 },
  { "Delontae", 1.0 },
  { "Delontay", 1.0 },
  { "Delonte", 1.0 },
  { "Delonzo", 1.0 },
  { "Delor", 1.0 },
  { "Delora", 0.0 },
  { "Deloras", 0.0 },
  { "Delorce", 0.0 },
  { "Delore", 0.75609756 },
  { "Delorean", 0.8466667 },
  { "Deloren", 1.0 },
  { "Delorenzo", 1.0 },
  { "Delores", 0.003641815 },
  { "Delorese", 0.0 },
  { "Deloria", 0.0 },
  { "Delorian", 0.7826087 },
  { "Delorice", 0.0 },
  { "Delories", 0.0 },
  { "Deloris", 0.0026751813 },
  { "Delorise", 0.0 },
  { "Deloros", 0.0 },
  { "Delors", 0.0 },
  { "Delorse", 0.0 },
  { "Delorus", 0.0 },
  { "Delos", 1.0 },
  { "Deloss", 1.0 },
  { "Delouise", 0.0 },
  { "Delouris", 0.0 },
  { "Deloy", 1.0 },
  { "Deloyce", 0.0 },
  { "Deloyd", 1.0 },
  { "Delpha", 0.0 },
  { "Delphene", 0.0 },
  { "Delphenia", 0.0 },
  { "Delphia", 0.0 },
  { "Delphie", 0.0 },
  { "Delphin", 1.0 },
  { "Delphina", 0.0 },
  { "Delphine", 0.0032728354 },
  { "Delphinia", 0.0 },
  { "Delphis", 1.0 },
  { "Delphus", 1.0 },
  { "Delquan", 1.0 },
  { "Delrae", 0.0 },
  { "Delray", 1.0 },
  { "Delrick", 1.0 },
  { "Delrico", 1.0 },
  { "Delrita", 0.0 },
  { "Delron", 1.0 },
  { "Delrose", 0.0 },
  { "Delroy", 1.0 },
  { "Delsa", 0.0 },
  { "Delsean", 1.0 },
  { "Delsen", 1.0 },
  { "Delsey", 0.0 },
  { "Delsha", 0.0 },
  { "Delshaun", 1.0 },
  { "Delshawn", 1.0 },
  { "Delshon", 1.0 },
  { "Delsia", 0.0 },
  { "Delsie", 0.0 },
  { "Delsin", 1.0 },
  { "Delson", 1.0 },
  { "Delsy", 0.0 },
  { "Delta", 0.023385301 },
  { "Deltha", 0.0 },
  { "Deltina", 0.0 },
  { "Delton", 1.0 },
  { "Deltrick", 1.0 },
  { "Deluca", 1.0 },
  { "Deluvina", 0.0 },
  { "Delva", 0.0 },
  { "Delvan", 1.0 },
  { "Delvante", 1.0 },
  { "Delvaughn", 1.0 },
  { "Delvecchio", 1.0 },
  { "Delver", 1.0 },
  { "Delvia", 0.0 },
  { "Delvin", 0.99565816 },
  { "Delvina", 0.0 },
  { "Delvion", 1.0 },
  { "Delvis", 1.0 },
  { "Delvon", 1.0 },
  { "Delvonta", 1.0 },
  { "Delvontae", 1.0 },
  { "Delvonte", 1.0 },
  { "Delwayne", 1.0 },
  { "Delwin", 1.0 },
  { "Delwood", 1.0 },
  { "Delwyn", 1.0 },
  { "Dely", 0.0 },
  { "Delya", 0.0 },
  { "Delycia", 0.0 },
  { "Delyla", 0.0 },
  { "Delylah", 0.0 },
  { "Delyle", 1.0 },
  { "Delyn", 0.16176471 },
  { "Delynda", 0.0 },
  { "Delynn", 0.121026896 },
  { "Delynne", 0.0 },
  { "Delysa", 0.0 },
  { "Delyssa", 0.0 },
  { "Delyza", 0.0 },
  { "Delzora", 0.0 },
  { "Dema", 0.0 },
  { "Demacio", 1.0 },
  { "Demaine", 1.0 },
  { "Demaj", 1.0 },
  { "Demajae", 1.0 },
  { "Demaje", 1.0 },
  { "Demandre", 1.0 },
  { "Demani", 0.7742616 },
  { "Demante", 1.0 },
  { "Demanuel", 1.0 },
  { "Demar", 1.0 },
  { "Demara", 0.0 },
  { "Demarcio", 1.0 },
  { "Demarco", 1.0 },
  { "Demarcos", 1.0 },
  { "Demarcus", 0.99770516 },
  { "Demare", 1.0 },
  { "Demarea", 1.0 },
  { "Demarean", 1.0 },
  { "Demaree", 0.8358209 },
  { "Demareo", 1.0 },
  { "Demareon", 1.0 },
  { "Demari", 0.88901097 },
  { "Demaria", 0.15733333 },
  { "Demariae", 1.0 },
  { "Demariah", 0.0 },
  { "Demarian", 1.0 },
  { "Demariana", 0.0 },
  { "Demarie", 0.3809524 },
  { "Demariea", 1.0 },
  { "Demarien", 1.0 },
  { "Demario", 0.9971279 },
  { "Demarion", 1.0 },
  { "Demariona", 0.0 },
  { "Demarionna", 0.0 },
  { "Demarious", 1.0 },
  { "Demaris", 0.21600622 },
  { "Demarius", 1.0 },
  { "Demariya", 0.0 },
  { "Demariyah", 0.0 },
  { "Demariyon", 1.0 },
  { "Demark", 1.0 },
  { "Demarkas", 1.0 },
  { "Demarkio", 1.0 },
  { "Demarkis", 1.0 },
  { "Demarko", 1.0 },
  { "Demarkus", 1.0 },
  { "Demarlo", 1.0 },
  { "Demaro", 1.0 },
  { "Demarque", 1.0 },
  { "Demarques", 1.0 },
  { "Demarquez", 1.0 },
  { "Demarquis", 1.0 },
  { "Demarquise", 1.0 },
  { "Demarqus", 1.0 },
  { "Demarr", 1.0 },
  { "Demarre", 1.0 },
  { "Demarrea", 1.0 },
  { "Demarreon", 1.0 },
  { "Demarri", 1.0 },
  { "Demarrie", 1.0 },
  { "Demarrio", 1.0 },
  { "Demarrion", 1.0 },
  { "Demarrius", 1.0 },
  { "Demartez", 1.0 },
  { "Demarus", 1.0 },
  { "Demaryius", 1.0 },
  { "Demas", 1.0 },
  { "Demaurea", 1.0 },
  { "Demauri", 1.0 },
  { "Demauria", 0.45 },
  { "Demaurie", 1.0 },
  { "Demaurio", 1.0 },
  { "Demaurion", 1.0 },
  { "Demaury", 1.0 },
  { "Demaya", 0.0 },
  { "Demba", 1.0 },
  { "Demea", 0.7058824 },
  { "Demeatra", 0.0 },
  { "Demeatrice", 0.30508474 },
  { "Demeatrius", 1.0 },
  { "Demeca", 0.0 },
  { "Demecia", 0.0 },
  { "Demecio", 1.0 },
  { "Demeco", 1.0 },
  { "Demedrick", 1.0 },
  { "Demeir", 1.0 },
  { "Demeisha", 0.0 },
  { "Demeka", 0.0 },
  { "Demekia", 0.0 },
  { "Demeko", 1.0 },
  { "Demel", 1.0 },
  { "Demelza", 0.0 },
  { "Dementrius", 1.0 },
  { "Demere", 1.0 },
  { "Demeree", 0.0 },
  { "Demeri", 0.0 },
  { "Demeria", 0.0 },
  { "Demerick", 1.0 },
  { "Demerion", 1.0 },
  { "Demeris", 1.0 },
  { "Demerius", 1.0 },
  { "Demerrick", 1.0 },
  { "Demery", 0.2 },
  { "Demesha", 0.0 },
  { "Demeshia", 0.0 },
  { "Demeta", 0.0 },
  { "Demeter", 1.0 },
  { "Demeteria", 0.0 },
  { "Demeterius", 1.0 },
  { "Demethra", 0.0 },
  { "Demetia", 0.0 },
  { "Demetirus", 1.0 },
  { "Demetra", 0.0029119146 },
  { "Demetras", 0.0 },
  { "Demetre", 0.98107255 },
  { "Demetree", 1.0 },
  { "Demetres", 0.94174755 },
  { "Demetress", 0.08955224 },
  { "Demetreus", 1.0 },
  { "Demetrey", 1.0 },
  { "Demetri", 0.9567123 },
  { "Demetria", 0.013022998 },
  { "Demetrial", 0.0 },
  { "Demetrian", 1.0 },
  { "Demetriana", 0.0 },
  { "Demetrianna", 0.0 },
  { "Demetrias", 0.72159094 },
  { "Demetric", 0.8317642 },
  { "Demetrica", 0.0 },
  { "Demetrice", 0.47875187 },
  { "Demetrices", 1.0 },
  { "Demetricia", 0.0 },
  { "Demetrick", 1.0 },
  { "Demetrics", 1.0 },
  { "Demetricus", 1.0 },
  { "Demetrie", 0.94585985 },
  { "Demetries", 1.0 },
  { "Demetrik", 1.0 },
  { "Demetrio", 1.0 },
  { "Demetrion", 1.0 },
  { "Demetriona", 0.0 },
  { "Demetrios", 1.0 },
  { "Demetrious", 0.96604216 },
  { "Demetris", 0.7374154 },
  { "Demetrise", 0.19354838 },
  { "Demetrish", 0.0 },
  { "Demetriss", 0.0 },
  { "Demetriu", 1.0 },
  { "Demetrius", 0.95118093 },
  { "Demetrous", 1.0 },
  { "Demetruis", 0.99384993 },
  { "Demetrus", 0.9664122 },
  { "Demetry", 1.0 },
  { "Demeturis", 1.0 },
  { "Demi", 0.0 },
  { "Demia", 0.0 },
  { "Demiah", 0.0 },
  { "Demian", 0.99153566 },
  { "Demiana", 0.0 },
  { "Demica", 0.0 },
  { "Demicah", 1.0 },
  { "Demichael", 1.0 },
  { "Demicheal", 1.0 },
  { "Demico", 1.0 },
  { "Demid", 1.0 },
  { "Demie", 0.0 },
  { "Demien", 1.0 },
  { "Demier", 1.0 },
  { "Demii", 0.0 },
  { "Demika", 0.0 },
  { "Demiko", 1.0 },
  { "Demilade", 0.65384614 },
  { "Demille", 1.0 },
  { "Deming", 1.0 },
  { "Demingo", 1.0 },
  { "Demion", 1.0 },
  { "Demir", 1.0 },
  { "Demira", 0.0 },
  { "Demiracle", 0.0 },
  { "Demire", 1.0 },
  { "Demis", 1.0 },
  { "Demisha", 0.0 },
  { "Demita", 0.0 },
  { "Demitra", 0.0 },
  { "Demitre", 1.0 },
  { "Demitri", 0.9873834 },
  { "Demitria", 0.0 },
  { "Demitric", 1.0 },
  { "Demitrice", 0.45454547 },
  { "Demitrie", 1.0 },
  { "Demitrio", 1.0 },
  { "Demitrios", 1.0 },
  { "Demitrious", 1.0 },
  { "Demitris", 1.0 },
  { "Demitrius", 0.9940379 },
  { "Demitrus", 1.0 },
  { "Demitry", 1.0 },
  { "Demiya", 0.0 },
  { "Demiyah", 0.0 },
  { "Demmi", 0.0 },
  { "Demmie", 0.0 },
  { "Demon", 1.0 },
  { "Demona", 0.0 },
  { "Demond", 0.9955697 },
  { "Demonde", 1.0 },
  { "Demondre", 1.0 },
  { "Demone", 1.0 },
  { "Demonei", 1.0 },
  { "Demoni", 0.7361111 },
  { "Demonica", 0.0 },
  { "Demonie", 0.9104478 },
  { "Demont", 1.0 },
  { "Demonta", 1.0 },
  { "Demontae", 1.0 },
  { "Demontavious", 1.0 },
  { "Demontay", 1.0 },
  { "Demonte", 1.0 },
  { "Demontez", 1.0 },
  { "Demontra", 1.0 },
  { "Demontrae", 1.0 },
  { "Demontray", 1.0 },
  { "Demontre", 1.0 },
  { "Demontreal", 1.0 },
  { "Demontrell", 1.0 },
  { "Demontrey", 1.0 },
  { "Demontrez", 1.0 },
  { "Demorea", 1.0 },
  { "Demorian", 1.0 },
  { "Demorio", 1.0 },
  { "Demorion", 1.0 },
  { "Demoris", 1.0 },
  { "Demorrio", 1.0 },
  { "Demorris", 1.0 },
  { "Demos", 1.0 },
  { "Demosthenes", 1.0 },
  { "Demple", 0.0 },
  { "Dempsey", 0.92977667 },
  { "Dempsie", 1.0 },
  { "Dempsy", 1.0 },
  { "Demri", 0.0 },
  { "Demtrius", 1.0 },
  { "Demus", 1.0 },
  { "Demy", 0.0 },
  { "Demya", 0.0 },
  { "Demyah", 0.0 },
  { "Demyan", 1.0 },
  { "Demyia", 0.0 },
  { "Demyiah", 0.0 },
  { "Demyla", 0.0 },
  { "Demyra", 0.0 },
  { "Den", 1.0 },
  { "Dena", 0.0017378351 },
  { "Denae", 0.0 },
  { "Denaeja", 0.0 },
  { "Denah", 0.0 },
  { "Denahi", 1.0 },
  { "Denai", 0.0 },
  { "Denaija", 0.0 },
  { "Denaijah", 0.0 },
  { "Denaisha", 0.0 },
  { "Denaiya", 0.0 },
  { "Denaja", 0.0 },
  { "Denajah", 0.0 },
  { "Denajia", 0.0 },
  { "Denali", 0.28029293 },
  { "Denalia", 0.0 },
  { "Denaly", 0.0 },
  { "Denamarie", 0.0 },
  { "Denard", 1.0 },
  { "Denardo", 1.0 },
  { "Denari", 0.88095236 },
  { "Denaria", 0.0 },
  { "Denario", 1.0 },
  { "Denarious", 1.0 },
  { "Denarius", 1.0 },
  { "Denaro", 1.0 },
  { "Denarrius", 1.0 },
  { "Denasha", 0.0 },
  { "Denashia", 0.0 },
  { "Denasia", 0.0 },
  { "Denautica", 0.0 },
  { "Denay", 0.0 },
  { "Denaya", 0.0 },
  { "Denayah", 0.0 },
  { "Denaye", 0.0 },
  { "Denaysha", 0.0 },
  { "Denaysia", 0.0 },
  { "Denazia", 0.0 },
  { "Denby", 1.0 },
  { "Dencie", 0.0 },
  { "Dencil", 1.0 },
  { "Dendrick", 1.0 },
  { "Dene", 0.28050315 },
  { "Denea", 0.0 },
  { "Denean", 0.0 },
  { "Deneane", 0.0 },
  { "Denease", 0.0 },
  { "Denece", 0.0 },
  { "Denecia", 0.0 },
  { "Deneco", 1.0 },
  { "Denedra", 0.0 },
  { "Denee", 0.0 },
  { "Deneem", 0.0 },
  { "Deneen", 0.0019395954 },
  { "Deneena", 0.0 },
  { "Deneene", 0.0 },
  { "Deneesha", 0.0 },
  { "Deneice", 0.0 },
  { "Deneil", 0.0 },
  { "Deneise", 0.0 },
  { "Deneisha", 0.0 },
  { "Deneishia", 0.0 },
  { "Deneka", 0.0 },
  { "Denekia", 0.0 },
  { "Denel", 0.0 },
  { "Denelda", 0.0 },
  { "Denell", 0.12159329 },
  { "Denelle", 0.0093896715 },
  { "Denena", 0.0 },
  { "Denene", 0.0 },
  { "Denequa", 0.0 },
  { "Denerick", 1.0 },
  { "Denerio", 1.0 },
  { "Denero", 1.0 },
  { "Denes", 1.0 },
  { "Denesa", 0.0 },
  { "Denese", 0.0 },
  { "Denesha", 0.0 },
  { "Deneshia", 0.0 },
  { "Denesia", 0.0 },
  { "Denessa", 0.0 },
  { "Deneta", 0.0 },
  { "Denetia", 0.0 },
  { "Denetra", 0.0 },
  { "Denetria", 0.0 },
  { "Denetrice", 0.0 },
  { "Denetris", 0.0 },
  { "Denetrius", 0.0 },
  { "Denett", 0.0 },
  { "Denetta", 0.0 },
  { "Denette", 0.0 },
  { "Deneva", 0.0 },
  { "Deney", 1.0 },
  { "Denford", 1.0 },
  { "Deng", 1.0 },
  { "Denham", 1.0 },
  { "Deni", 0.19015957 },
  { "Denia", 0.0 },
  { "Deniah", 0.0 },
  { "Denica", 0.0 },
  { "Denice", 0.0 },
  { "Denicia", 0.0 },
  { "Denicka", 0.0 },
  { "Denico", 1.0 },
  { "Denie", 0.0 },
  { "Deniece", 0.0 },
  { "Deniecia", 0.0 },
  { "Deniel", 1.0 },
  { "Deniella", 0.0 },
  { "Denielle", 0.0 },
  { "Deniene", 0.0 },
  { "Deniese", 0.0 },
  { "Deniesha", 0.0 },
  { "Denija", 0.0 },
  { "Denijah", 0.0 },
  { "Denika", 0.0 },
  { "Deniko", 1.0 },
  { "Denilah", 0.0 },
  { "Denille", 0.0 },
  { "Denilson", 1.0 },
  { "Denilzon", 1.0 },
  { "Denim", 0.73913044 },
  { "Denin", 1.0 },
  { "Denina", 0.0 },
  { "Denine", 0.0 },
  { "Deniqua", 0.0 },
  { "Denique", 0.0 },
  { "Denira", 0.0 },
  { "Deniro", 1.0 },
  { "Denis", 0.9605914 },
  { "Denisa", 0.0 },
  { "Denise", 0.0036531175 },
  { "Denisha", 0.0 },
  { "Denishea", 0.0 },
  { "Denishia", 0.0 },
  { "Denisia", 0.0 },
  { "Denison", 1.0 },
  { "Deniss", 0.057471264 },
  { "Denissa", 0.0 },
  { "Denisse", 0.0 },
  { "Denita", 0.0 },
  { "Denitra", 0.0 },
  { "Denitta", 0.0 },
  { "Deniya", 0.0 },
  { "Deniyah", 0.0 },
  { "Deniz", 0.5519836 },
  { "Denize", 0.0 },
  { "Denley", 1.0 },
  { "Denman", 1.0 },
  { "Denmark", 1.0 },
  { "Denna", 0.0 },
  { "Dennae", 0.0 },
  { "Dennard", 1.0 },
  { "Denne", 0.0 },
  { "Dennell", 0.2631579 },
  { "Dennen", 0.0 },
  { "Dennes", 1.0 },
  { "Dennese", 0.0 },
  { "Dennesly", 0.0 },
  { "Dennett", 1.0 },
  { "Dennetta", 0.0 },
  { "Dennette", 0.0 },
  { "Denney", 1.0 },
  { "Denni", 0.0862069 },
  { "Dennia", 0.0 },
  { "Dennice", 0.0 },
  { "Dennie", 0.8220507 },
  { "Dennielle", 0.0 },
  { "Dennies", 1.0 },
  { "Dennine", 0.0 },
  { "Dennis", 0.99570495 },
  { "Dennise", 0.024057971 },
  { "Dennisha", 0.0 },
  { "Dennishia", 0.0 },
  { "Dennison", 1.0 },
  { "Dennisse", 0.0 },
  { "Dennita", 0.0 },
  { "Dennon", 1.0 },
  { "Denny", 0.9746062 },
  { "Dennys", 0.83693516 },
  { "Deno", 1.0 },
  { "Denon", 1.0 },
  { "Denora", 0.0 },
  { "Denoris", 1.0 },
  { "Denorris", 1.0 },
  { "Denotra", 0.0 },
  { "Denroy", 1.0 },
  { "Densel", 1.0 },
  { "Densie", 0.0 },
  { "Densil", 1.0 },
  { "Denson", 1.0 },
  { "Dent", 1.0 },
  { "Denton", 1.0 },
  { "Dentrell", 1.0 },
  { "Denver", 0.91012716 },
  { "Denvil", 1.0 },
  { "Deny", 1.0 },
  { "Denya", 0.0 },
  { "Denyah", 0.0 },
  { "Denyce", 0.0 },
  { "Denyel", 0.0 },
  { "Denyell", 0.0 },
  { "Denyelle", 0.0 },
  { "Denyia", 0.0 },
  { "Denyla", 0.0 },
  { "Denylah", 0.0 },
  { "Denym", 0.48547718 },
  { "Denys", 0.2755102 },
  { "Denyse", 0.0 },
  { "Denysha", 0.0 },
  { "Denysse", 0.0 },
  { "Denzal", 1.0 },
  { "Denzale", 1.0 },
  { "Denzel", 0.9959602 },
  { "Denzell", 1.0 },
  { "Denzelle", 1.0 },
  { "Denziel", 1.0 },
  { "Denzil", 0.9822129 },
  { "Denzle", 1.0 },
  { "Denzyl", 1.0 },
  { "Deo", 1.0 },
  { "Deobrah", 0.0 },
  { "Deola", 0.0 },
  { "Deolinda", 0.0 },
  { "Deon", 0.89342 },
  { "Deona", 0.0 },
  { "Deonate", 1.0 },
  { "Deonca", 0.0 },
  { "Deonda", 0.0 },
  { "Deondra", 0.23459582 },
  { "Deondrae", 1.0 },
  { "Deondray", 1.0 },
  { "Deondre", 0.99597853 },
  { "Deondrea", 0.43589744 },
  { "Deondrey", 1.0 },
  { "Deondria", 0.0 },
  { "Deondric", 1.0 },
  { "Deondrick", 1.0 },
  { "Deone", 0.37671232 },
  { "Deoni", 0.0 },
  { "Deonia", 0.0 },
  { "Deonica", 0.0 },
  { "Deonie", 0.0 },
  { "Deonka", 0.0 },
  { "Deonn", 0.0 },
  { "Deonna", 0.001396258 },
  { "Deonne", 0.09710744 },
  { "Deonni", 0.0 },
  { "Deonnie", 0.0 },
  { "Deonsha", 0.0 },
  { "Deonshay", 0.0 },
  { "Deonta", 0.9856287 },
  { "Deontae", 1.0 },
  { "Deontai", 1.0 },
  { "Deontavious", 1.0 },
  { "Deontay", 1.0 },
  { "Deontaye", 1.0 },
  { "Deonte", 0.994618 },
  { "Deontea", 1.0 },
  { "Deontee", 1.0 },
  { "Deontez", 1.0 },
  { "Deontia", 1.0 },
  { "Deontra", 0.88235295 },
  { "Deontrae", 1.0 },
  { "Deontray", 1.0 },
  { "Deontre", 1.0 },
  { "Deontrey", 1.0 },
  { "Deontrez", 1.0 },
  { "Deonza", 0.0 },
  { "Deora", 0.0 },
  { "Deosha", 0.0 },
  { "Deotha", 0.0 },
  { "Deotis", 1.0 },
  { "Deoveon", 0.21428572 },
  { "Deovian", 0.0 },
  { "Deovion", 0.622807 },
  { "Depaul", 1.0 },
  { "Depree", 1.0 },
  { "Depriest", 1.0 },
  { "Deprise", 0.0 },
  { "Deqa", 0.0 },
  { "Dequan", 0.9986184 },
  { "Dequana", 0.0 },
  { "Dequanda", 0.0 },
  { "Dequandra", 0.0 },
  { "Dequandre", 1.0 },
  { "Dequane", 1.0 },
  { "Dequann", 1.0 },
  { "Dequanna", 0.0 },
  { "Dequanta", 1.0 },
  { "Dequantae", 1.0 },
  { "Dequante", 1.0 },
  { "Dequantez", 1.0 },
  { "Dequarious", 1.0 },
  { "Dequarius", 1.0 },
  { "Dequasha", 0.0 },
  { "Dequaveon", 1.0 },
  { "Dequavion", 1.0 },
  { "Dequavious", 1.0 },
  { "Dequavius", 1.0 },
  { "Dequawn", 1.0 },
  { "Dequay", 1.0 },
  { "Dequentin", 1.0 },
  { "Dequesha", 0.0 },
  { "Dequez", 1.0 },
  { "Dequilla", 0.0 },
  { "Dequincy", 1.0 },
  { "Dequindre", 1.0 },
  { "Dequinn", 1.0 },
  { "Dequinton", 1.0 },
  { "Dequisha", 0.0 },
  { "Dequita", 0.0 },
  { "Dequon", 1.0 },
  { "Dequone", 1.0 },
  { "Deqwan", 1.0 },
  { "Der", 0.0 },
  { "Dera", 0.0 },
  { "Derak", 1.0 },
  { "Deral", 1.0 },
  { "Derald", 1.0 },
  { "Deralyn", 0.0 },
  { "Deran", 1.0 },
  { "Deray", 1.0 },
  { "Derba", 0.0 },
  { "Derby", 0.85294116 },
  { "Derck", 1.0 },
  { "Derean", 1.0 },
  { "Derec", 1.0 },
  { "Dereck", 1.0 },
  { "Dereginald", 1.0 },
  { "Dereion", 1.0 },
  { "Derek", 0.9960223 },
  { "Dereka", 0.0 },
  { "Dereke", 1.0 },
  { "Derekia", 0.0 },
  { "Derel", 1.0 },
  { "Derell", 1.0 },
  { "Derelle", 1.0 },
  { "Derelys", 0.0 },
  { "Deremy", 1.0 },
  { "Deren", 1.0 },
  { "Derena", 0.0 },
  { "Derenda", 0.0 },
  { "Derene", 0.0 },
  { "Dereon", 0.7611444 },
  { "Dereona", 0.0 },
  { "Dereonna", 0.0 },
  { "Deresa", 0.0 },
  { "Deresha", 0.0 },
  { "Dereth", 0.0 },
  { "Deretha", 0.0 },
  { "Derex", 1.0 },
  { "Dereyon", 1.0 },
  { "Derez", 1.0 },
  { "Derf", 1.0 },
  { "Derhonda", 0.0 },
  { "Deri", 0.0 },
  { "Deria", 0.0 },
  { "Deriah", 0.0 },
  { "Derian", 0.8962337 },
  { "Deriana", 0.0 },
  { "Derianna", 0.0 },
  { "Deric", 1.0 },
  { "Derica", 0.0 },
  { "Derice", 1.0 },
  { "Derick", 0.9978977 },
  { "Dericka", 0.0 },
  { "Derico", 1.0 },
  { "Deridre", 0.0 },
  { "Deriek", 1.0 },
  { "Deriel", 1.0 },
  { "Derielle", 0.0 },
  { "Derien", 1.0 },
  { "Derik", 1.0 },
  { "Derika", 0.0 },
  { "Derike", 1.0 },
  { "Derin", 0.845339 },
  { "Derina", 0.0 },
  { "Derinda", 0.0 },
  { "Derion", 0.94901145 },
  { "Deriona", 0.0 },
  { "Derionna", 0.0 },
  { "Derious", 1.0 },
  { "Derique", 1.0 },
  { "Deris", 1.0 },
  { "Derise", 0.0 },
  { "Derisha", 0.0 },
  { "Derita", 0.0 },
  { "Derith", 0.0 },
  { "Derius", 1.0 },
  { "Deriya", 0.0 },
  { "Deriyah", 0.0 },
  { "Deriyon", 1.0 },
  { "Derk", 1.0 },
  { "Derl", 1.0 },
  { "Derle", 1.0 },
  { "Derlene", 0.0 },
  { "Derlin", 1.0 },
  { "Derlis", 1.0 },
  { "Derly", 1.0 },
  { "Dermaine", 1.0 },
  { "Dermarr", 1.0 },
  { "Dermont", 1.0 },
  { "Dermot", 1.0 },
  { "Dermott", 1.0 },
  { "Derna", 0.0 },
  { "Dernell", 1.0 },
  { "Dero", 1.0 },
  { "Deroderick", 1.0 },
  { "Deroe", 1.0 },
  { "Derold", 1.0 },
  { "Derome", 1.0 },
  { "Deron", 0.9936674 },
  { "Deronda", 0.0 },
  { "Derone", 1.0 },
  { "Deronn", 1.0 },
  { "Deronna", 0.0 },
  { "Deronne", 1.0 },
  { "Deronta", 1.0 },
  { "Derontae", 1.0 },
  { "Deronte", 1.0 },
  { "Derotha", 0.0 },
  { "Deroy", 1.0 },
  { "Derquan", 1.0 },
  { "Derra", 0.0 },
  { "Derral", 1.0 },
  { "Derrald", 1.0 },
  { "Derrall", 1.0 },
  { "Derran", 1.0 },
  { "Derrance", 1.0 },
  { "Derreck", 1.0 },
  { "Derrek", 1.0 },
  { "Derrel", 1.0 },
  { "Derrell", 0.9972164 },
  { "Derrelle", 1.0 },
  { "Derren", 1.0 },
  { "Derreon", 1.0 },
  { "Derri", 0.0 },
  { "Derria", 0.0 },
  { "Derriah", 0.0 },
  { "Derrian", 0.8787879 },
  { "Derriana", 0.0 },
  { "Derrianna", 0.0 },
  { "Derric", 1.0 },
  { "Derrica", 0.0 },
  { "Derrice", 1.0 },
  { "Derrich", 1.0 },
  { "Derrick", 0.99298596 },
  { "Derricka", 0.0 },
  { "Derrico", 1.0 },
  { "Derrie", 0.69863015 },
  { "Derrien", 1.0 },
  { "Derrik", 1.0 },
  { "Derrika", 0.0 },
  { "Derrike", 1.0 },
  { "Derril", 1.0 },
  { "Derrill", 1.0 },
  { "Derrin", 1.0 },
  { "Derringer", 1.0 },
  { "Derrion", 0.966716 },
  { "Derriona", 0.0 },
  { "Derrione", 1.0 },
  { "Derrionna", 0.0 },
  { "Derrious", 1.0 },
  { "Derris", 1.0 },
  { "Derrisha", 0.0 },
  { "Derrius", 1.0 },
  { "Derriyah", 0.0 },
  { "Derrol", 1.0 },
  { "Derrold", 1.0 },
  { "Derron", 1.0 },
  { "Derrus", 1.0 },
  { "Derry", 0.91835517 },
  { "Derryck", 1.0 },
  { "Derryk", 1.0 },
  { "Derryl", 0.98362446 },
  { "Derryon", 1.0 },
  { "Dervin", 1.0 },
  { "Dervon", 1.0 },
  { "Derward", 1.0 },
  { "Derwin", 1.0 },
  { "Derwood", 1.0 },
  { "Derwyn", 1.0 },
  { "Derya", 0.0 },
  { "Deryan", 1.0 },
  { "Deryck", 1.0 },
  { "Deryk", 1.0 },
  { "Derykah", 0.0 },
  { "Deryl", 0.9057072 },
  { "Deryle", 1.0 },
  { "Deryn", 0.15714286 },
  { "Deryon", 1.0 },
  { "Des", 0.54545456 },
  { "Desa", 0.0 },
  { "Desai", 1.0 },
  { "Desales", 0.0 },
  { "Desandra", 0.0 },
  { "Desani", 0.2 },
  { "Desara", 0.0 },
  { "Desarae", 0.0 },
  { "Desarai", 0.0 },
  { "Desaray", 0.0 },
  { "Desaraye", 0.0 },
  { "Desare", 0.0 },
  { "Desarea", 0.0 },
  { "Desaree", 0.0 },
  { "Desarey", 0.0 },
  { "Desari", 0.0 },
  { "Desarie", 0.0 },
  { "Desaun", 1.0 },
  { "Desda", 0.0 },
  { "Desdemona", 0.0 },
  { "Desean", 0.9970222 },
  { "Desera", 0.0 },
  { "Deserae", 0.0 },
  { "Deserai", 0.0 },
  { "Deseray", 0.0 },
  { "Desere", 0.0 },
  { "Deserea", 0.0 },
  { "Deseree", 0.0 },
  { "Deseret", 0.0 },
  { "Deserey", 0.0 },
  { "Deseri", 0.0 },
  { "Deseria", 0.0 },
  { "Deserie", 0.0 },
  { "Desery", 0.0 },
  { "Desha", 0.006119951 },
  { "Deshad", 1.0 },
  { "Deshae", 0.18141593 },
  { "Deshai", 0.0 },
  { "Deshaila", 0.0 },
  { "Deshala", 0.0 },
  { "Deshan", 0.9527559 },
  { "Deshana", 0.0 },
  { "Deshanae", 0.0 },
  { "Deshanay", 0.0 },
  { "Deshanda", 0.0 },
  { "Deshandra", 0.0 },
  { "Deshane", 0.93333334 },
  { "Deshanique", 0.0 },
  { "Deshanna", 0.0 },
  { "Deshannon", 0.3713355 },
  { "Deshanon", 1.0 },
  { "Deshanta", 0.0 },
  { "Deshante", 0.21839081 },
  { "Deshanti", 0.0 },
  { "Deshara", 0.0 },
  { "Deshard", 1.0 },
  { "Deshaud", 1.0 },
  { "Deshaun", 0.9571826 },
  { "Deshauna", 0.0 },
  { "Deshaunda", 0.0 },
  { "Deshaundra", 0.0 },
  { "Deshaune", 1.0 },
  { "Deshaunta", 0.0 },
  { "Deshaunte", 1.0 },
  { "Deshawn", 0.9253345 },
  { "Deshawna", 0.0 },
  { "Deshawnda", 0.0 },
  { "Deshawndre", 1.0 },
  { "Deshawnna", 0.0 },
  { "Deshawnte", 1.0 },
  { "Deshay", 0.38689217 },
  { "Deshaya", 0.0 },
  { "Deshayla", 0.0 },
  { "Deshea", 0.09248555 },
  { "Desheena", 0.0 },
  { "Desheila", 0.0 },
  { "Deshelia", 0.0 },
  { "Deshell", 0.0 },
  { "Deshera", 0.0 },
  { "Deshia", 0.0 },
  { "Deshion", 1.0 },
  { "Deshira", 0.0 },
  { "Deshiya", 0.0 },
  { "Deshna", 0.0 },
  { "Deshon", 0.93333334 },
  { "Deshona", 0.0 },
  { "Deshonda", 0.0 },
  { "Deshondra", 0.0 },
  { "Deshone", 0.8449848 },
  { "Deshonia", 0.0 },
  { "Deshonna", 0.0 },
  { "Deshonta", 0.0 },
  { "Deshonte", 0.45454547 },
  { "Deshown", 1.0 },
  { "Deshuan", 1.0 },
  { "Deshun", 0.9671323 },
  { "Deshuna", 0.0 },
  { "Deshunda", 0.0 },
  { "Deshundra", 0.0 },
  { "Deshya", 0.0 },
  { "Desi", 0.7381119 },
  { "Desia", 0.0 },
  { "Desiah", 0.0 },
  { "Desiderio", 1.0 },
  { "Desieree", 0.0 },
  { "Desinae", 0.0 },
  { "Desira", 0.0 },
  { "Desirae", 0.0 },
  { "Desiraee", 0.0 },
  { "Desirai", 0.0 },
  { "Desiray", 0.0 },
  { "Desiraye", 0.0 },
  { "Desire", 0.008785416 },
  { "Desirea", 0.0 },
  { "Desireah", 0.0 },
  { "Desiree", 0.002423204 },
  { "Desirey", 0.0 },
  { "Desiri", 0.0 },
  { "Desirie", 0.0 },
  { "Desirrae", 0.0 },
  { "Desirre", 0.0 },
  { "Desirree", 0.0 },
  { "Desitny", 0.0 },
  { "Desiya", 0.0 },
  { "Desiyah", 0.0 },
  { "Desja", 0.0 },
  { "Desjah", 0.0 },
  { "Desjambra", 0.0 },
  { "Desjon", 1.0 },
  { "Desjuan", 1.0 },
  { "Deslie", 0.0 },
  { "Deslyn", 0.0 },
  { "Deslynn", 0.0 },
  { "Desma", 0.0 },
  { "Desman", 1.0 },
  { "Desmand", 1.0 },
  { "Desmariah", 0.0 },
  { "Desmarie", 0.0 },
  { "Desmen", 1.0 },
  { "Desmend", 1.0 },
  { "Desmin", 0.9705882 },
  { "Desmine", 1.0 },
  { "Desmon", 1.0 },
  { "Desmona", 0.0 },
  { "Desmond", 0.9927376 },
  { "Desmone", 0.87234044 },
  { "Desmonique", 0.0 },
  { "Desmund", 1.0 },
  { "Desnee", 0.0 },
  { "Despena", 0.0 },
  { "Despina", 0.0 },
  { "Desra", 0.0 },
  { "Desrae", 0.0 },
  { "Desray", 0.0 },
  { "Desree", 0.0 },
  { "Desriee", 0.0 },
  { "Dessa", 0.0 },
  { "Dessarae", 0.0 },
  { "Dessel", 0.0 },
  { "Dessence", 0.0 },
  { "Desseray", 0.0 },
  { "Dessia", 0.0 },
  { "Dessiah", 0.26086956 },
  { "Dessica", 0.0 },
  { "Dessie", 0.027168324 },
  { "Dessirae", 0.0 },
  { "Dessire", 0.0 },
  { "Dessiree", 0.0 },
  { "Desta", 0.0 },
  { "Destainy", 0.0 },
  { "Destan", 1.0 },
  { "Destane", 0.0 },
  { "Destanee", 0.0 },
  { "Destaney", 0.0 },
  { "Destani", 0.0 },
  { "Destanie", 0.0 },
  { "Destannie", 0.0 },
  { "Destany", 0.0 },
  { "Desten", 1.0 },
  { "Destene", 0.0 },
  { "Destenee", 0.0 },
  { "Desteney", 0.0 },
  { "Desteni", 0.0 },
  { "Destenie", 0.0 },
  { "Desteny", 0.0 },
  { "Dester", 0.42857143 },
  { "Destery", 1.0 },
  { "Destiany", 0.0 },
  { "Destin", 0.88590014 },
  { "Destina", 0.0 },
  { "Destinae", 0.0 },
  { "Destinay", 0.0 },
  { "Destine", 0.13476703 },
  { "Destinea", 0.0 },
  { "Destined", 1.0 },
  { "Destinee", 0.0 },
  { "Destinei", 0.0 },
  { "Destiney", 0.0 },
  { "Desting", 0.0 },
  { "Destini", 0.0 },
  { "Destinie", 0.0 },
  { "Destinii", 0.0 },
  { "Destinique", 0.0 },
  { "Destinni", 0.0 },
  { "Destinnie", 0.0 },
  { "Destinny", 0.0 },
  { "Destiny", 0.0033449922 },
  { "Destinyann", 0.0 },
  { "Destinye", 0.0 },
  { "Destinymarie", 0.0 },
  { "Destinyrose", 0.0 },
  { "Destney", 0.0 },
  { "Deston", 1.0 },
  { "Destoni", 0.0 },
  { "Destony", 0.0 },
  { "Destri", 0.16129032 },
  { "Destry", 0.9077021 },
  { "Destyn", 0.7505519 },
  { "Destyne", 0.0 },
  { "Destynee", 0.0 },
  { "Destyni", 0.0 },
  { "Destynie", 0.0 },
  { "Destyny", 0.0 },
  { "Desuan", 1.0 },
  { "Desyrae", 0.0 },
  { "Desyre", 0.0 },
  { "Desyree", 0.0 },
  { "Deta", 0.0 },
  { "Detarius", 1.0 },
  { "Detavion", 1.0 },
  { "Detavious", 1.0 },
  { "Detavius", 1.0 },
  { "Deterrio", 1.0 },
  { "Deterrion", 1.0 },
  { "Deterrius", 1.0 },
  { "Detisha", 0.0 },
  { "Detoya", 0.0 },
  { "Detra", 0.0 },
  { "Detrail", 1.0 },
  { "Detravion", 1.0 },
  { "Detravious", 1.0 },
  { "Detrell", 1.0 },
  { "Detrevion", 1.0 },
  { "Detrez", 1.0 },
  { "Detri", 0.0 },
  { "Detria", 0.0 },
  { "Detric", 0.9882353 },
  { "Detrica", 0.0 },
  { "Detrice", 0.0 },
  { "Detrich", 0.9140625 },
  { "Detricia", 0.0 },
  { "Detrick", 0.9832552 },
  { "Detrina", 0.0 },
  { "Detroit", 1.0 },
  { "Detron", 1.0 },
  { "Detroy", 1.0 },
  { "Detta", 0.0 },
  { "Dette", 0.0 },
  { "Dettrick", 1.0 },
  { "Detwan", 1.0 },
  { "Detzani", 0.0 },
  { "Deucalion", 1.0 },
  { "Deuce", 1.0 },
  { "Deuel", 1.0 },
  { "Deundra", 0.61509436 },
  { "Deundrae", 1.0 },
  { "Deundray", 1.0 },
  { "Deundre", 1.0 },
  { "Deundrea", 1.0 },
  { "Deundria", 0.0 },
  { "Deunta", 1.0 },
  { "Deuntae", 1.0 },
  { "Deuntay", 1.0 },
  { "Deunte", 1.0 },
  { "Deuntray", 1.0 },
  { "Deus", 1.0 },
  { "Dev", 1.0 },
  { "Deva", 0.017372422 },
  { "Devaansh", 1.0 },
  { "Devaeh", 0.0 },
  { "Devahn", 1.0 },
  { "Devaj", 1.0 },
  { "Devaki", 0.0 },
  { "Deval", 1.0 },
  { "Devale", 1.0 },
  { "Devam", 1.0 },
  { "Devan", 0.72004426 },
  { "Devana", 0.0 },
  { "Devanae", 0.0 },
  { "Devanand", 1.0 },
  { "Devance", 1.0 },
  { "Devane", 1.0 },
  { "Devanee", 0.0 },
  { "Devaney", 0.0 },
  { "Devang", 1.0 },
  { "Devanhi", 0.0 },
  { "Devani", 0.0 },
  { "Devanie", 0.0 },
  { "Devann", 0.051282052 },
  { "Devanna", 0.0 },
  { "Devanne", 0.0 },
  { "Devannie", 0.0 },
  { "Devanny", 0.0 },
  { "Devansh", 1.0 },
  { "Devanshi", 0.0 },
  { "Devanta", 1.0 },
  { "Devantae", 1.0 },
  { "Devante", 0.9926669 },
  { "Devanti", 1.0 },
  { "Devantre", 1.0 },
  { "Devany", 0.0 },
  { "Devar", 1.0 },
  { "Devara", 0.0 },
  { "Devari", 1.0 },
  { "Devarian", 1.0 },
  { "Devario", 1.0 },
  { "Devarion", 1.0 },
  { "Devarious", 1.0 },
  { "Devaris", 1.0 },
  { "Devarius", 1.0 },
  { "Devaron", 1.0 },
  { "Devarsh", 1.0 },
  { "Devarus", 1.0 },
  { "Devaugh", 1.0 },
  { "Devaughn", 1.0 },
  { "Devaughnte", 1.0 },
  { "Devaun", 1.0 },
  { "Devaunte", 1.0 },
  { "Devaya", 0.0 },
  { "Devayah", 0.0 },
  { "Deveah", 0.0 },
  { "Devean", 1.0 },
  { "Deveda", 0.0 },
  { "Deveion", 1.0 },
  { "Devel", 1.0 },
  { "Devell", 1.0 },
  { "Develle", 1.0 },
  { "Develyn", 0.0 },
  { "Deven", 0.88632995 },
  { "Devena", 0.0 },
  { "Devendra", 1.0 },
  { "Deveney", 0.0 },
  { "Deveni", 0.0 },
  { "Devente", 1.0 },
  { "Deveny", 0.0 },
  { "Deveon", 1.0 },
  { "Dever", 1.0 },
  { "Devera", 0.0 },
  { "Deveraux", 1.0 },
  { "Devere", 1.0 },
  { "Devereaux", 0.9548872 },
  { "Devereux", 1.0 },
  { "Deverick", 1.0 },
  { "Deverie", 0.0 },
  { "Deverl", 1.0 },
  { "Deverly", 0.0 },
  { "Devern", 1.0 },
  { "Deverne", 1.0 },
  { "Deveron", 1.0 },
  { "Devery", 0.76368874 },
  { "Devesh", 1.0 },
  { "Deveshi", 0.0 },
  { "Deveta", 0.0 },
  { "Devetta", 0.0 },
  { "Devette", 0.0 },
  { "Deveyon", 1.0 },
  { "Devi", 0.0 },
  { "Devia", 0.0 },
  { "Devian", 0.9211469 },
  { "Deviana", 0.0 },
  { "Devida", 0.0 },
  { "Devie", 0.0 },
  { "Devien", 1.0 },
  { "Devika", 0.0 },
  { "Deville", 1.0 },
  { "Devin", 0.89888144 },
  { "Devina", 0.0 },
  { "Devindra", 1.0 },
  { "Devine", 0.5197947 },
  { "Devinee", 0.0 },
  { "Devinity", 0.0 },
  { "Devinlee", 1.0 },
  { "Devinn", 0.7304582 },
  { "Devinne", 0.0 },
  { "Deviny", 0.0 },
  { "Devion", 0.984945 },
  { "Devione", 1.0 },
  { "Devionna", 0.0 },
  { "Devionne", 1.0 },
  { "Deviontae", 1.0 },
  { "Devionte", 1.0 },
  { "Devita", 0.0 },
  { "Devito", 1.0 },
  { "Deviyon", 1.0 },
  { "Devki", 0.0 },
  { "Devlan", 1.0 },
  { "Devlen", 1.0 },
  { "Devlin", 0.9484601 },
  { "Devlon", 1.0 },
  { "Devlyn", 0.6070727 },
  { "Devlynn", 0.0 },
  { "Devn", 1.0 },
  { "Devoe", 1.0 },
  { "Devohn", 1.0 },
  { "Devoiry", 0.0 },
  { "Devola", 0.0 },
  { "Devon", 0.778636 },
  { "Devona", 0.0 },
  { "Devonae", 0.0 },
  { "Devonaire", 1.0 },
  { "Devonda", 0.0 },
  { "Devondra", 0.0 },
  { "Devondre", 1.0 },
  { "Devondrick", 1.0 },
  { "Devone", 0.8542413 },
  { "Devonee", 0.0 },
  { "Devoney", 0.0 },
  { "Devoni", 0.0 },
  { "Devonia", 0.0 },
  { "Devonie", 0.0 },
  { "Devonn", 0.9474886 },
  { "Devonna", 0.0 },
  { "Devonne", 0.40973505 },
  { "Devonnte", 1.0 },
  { "Devonta", 0.9878049 },
  { "Devontae", 1.0 },
  { "Devontai", 1.0 },
  { "Devontavius", 1.0 },
  { "Devontay", 1.0 },
  { "Devontaye", 1.0 },
  { "Devonte", 0.99702907 },
  { "Devontea", 1.0 },
  { "Devontee", 1.0 },
  { "Devontez", 1.0 },
  { "Devonti", 1.0 },
  { "Devontia", 1.0 },
  { "Devontra", 1.0 },
  { "Devontrae", 1.0 },
  { "Devontray", 1.0 },
  { "Devontre", 1.0 },
  { "Devontrey", 1.0 },
  { "Devony", 0.0 },
  { "Devonya", 0.0 },
  { "Devora", 0.0 },
  { "Devorah", 0.0 },
  { "Devoria", 0.0 },
  { "Devoris", 1.0 },
  { "Devory", 0.0 },
  { "Devota", 0.0 },
  { "Devra", 0.0 },
  { "Devraj", 1.0 },
  { "Devree", 0.0 },
  { "Devren", 1.0 },
  { "Devri", 0.0 },
  { "Devrick", 1.0 },
  { "Devrie", 0.0 },
  { "Devrim", 1.0 },
  { "Devrin", 1.0 },
  { "Devron", 1.0 },
  { "Devry", 0.7105263 },
  { "Devun", 1.0 },
  { "Devunta", 1.0 },
  { "Devunte", 1.0 },
  { "Devven", 1.0 },
  { "Devvin", 1.0 },
  { "Devvon", 0.921875 },
  { "Devy", 0.0 },
  { "Devyani", 0.0 },
  { "Devyn", 0.45214352 },
  { "Devyne", 0.32163742 },
  { "Devynn", 0.17086332 },
  { "Devynne", 0.0 },
  { "Devyon", 1.0 },
  { "Dewain", 1.0 },
  { "Dewaine", 1.0 },
  { "Dewan", 0.97480315 },
  { "Dewana", 0.0 },
  { "Dewand", 1.0 },
  { "Dewanda", 0.0 },
  { "Dewane", 1.0 },
  { "Dewann", 1.0 },
  { "Dewanna", 0.0 },
  { "Deward", 1.0 },
  { "Dewarren", 1.0 },
  { "Dewaun", 1.0 },
  { "Dewayna", 0.0 },
  { "Dewayne", 0.99573845 },
  { "Dewel", 1.0 },
  { "Dewell", 1.0 },
  { "Dewetta", 0.0 },
  { "Dewey", 0.98422086 },
  { "Dewi", 0.0 },
  { "Dewie", 0.93700784 },
  { "Dewight", 1.0 },
  { "Dewilla", 0.0 },
  { "Dewin", 1.0 },
  { "Dewit", 1.0 },
  { "Dewitt", 1.0 },
  { "Dewitte", 1.0 },
  { "Dewon", 1.0 },
  { "Dewone", 1.0 },
  { "Dewuan", 1.0 },
  { "Dewy", 1.0 },
  { "Dewyane", 1.0 },
  { "Dewyne", 1.0 },
  { "Dex", 1.0 },
  { "Dexiree", 0.0 },
  { "Dexten", 1.0 },
  { "Dexter", 0.9932759 },
  { "Dextin", 1.0 },
  { "Dexton", 1.0 },
  { "Dextyn", 1.0 },
  { "Dexx", 1.0 },
  { "Deya", 0.0 },
  { "Deyadira", 0.0 },
  { "Deyala", 0.0 },
  { "Deyan", 1.0 },
  { "Deyana", 0.0 },
  { "Deyanara", 0.0 },
  { "Deyaneira", 0.0 },
  { "Deyani", 0.0 },
  { "Deyanira", 0.0 },
  { "Deyanna", 0.0 },
  { "Deyannah", 0.0 },
  { "Deyanne", 0.0 },
  { "Deyante", 1.0 },
  { "Deybi", 1.0 },
  { "Deyci", 0.0 },
  { "Deycy", 0.0 },
  { "Deyja", 0.0 },
  { "Deyjah", 0.0 },
  { "Deyla", 0.0 },
  { "Deylan", 1.0 },
  { "Deylani", 0.0 },
  { "Deylen", 1.0 },
  { "Deyli", 0.0 },
  { "Deylin", 0.33333334 },
  { "Deylon", 1.0 },
  { "Deymar", 1.0 },
  { "Deymi", 0.0 },
  { "Deymian", 1.0 },
  { "Deyna", 0.0 },
  { "Deyon", 0.97795594 },
  { "Deyona", 0.0 },
  { "Deyonce", 0.10769231 },
  { "Deyonna", 0.0 },
  { "Deyonne", 0.0 },
  { "Deyonta", 1.0 },
  { "Deyontae", 1.0 },
  { "Deyonte", 1.0 },
  { "Deyra", 0.0 },
  { "Deyren", 1.0 },
  { "Deysha", 0.0 },
  { "Deyshaun", 1.0 },
  { "Deyshawn", 1.0 },
  { "Deysi", 0.0 },
  { "Deysia", 0.0 },
  { "Deyssi", 0.0 },
  { "Deysy", 0.0 },
  { "Deyton", 1.0 },
  { "Deyvi", 1.0 },
  { "Deyvid", 1.0 },
  { "Deyvin", 1.0 },
  { "Deyvion", 1.0 },
  { "Deyvon", 1.0 },
  { "Dez", 1.0 },
  { "Deza", 0.0 },
  { "Dezani", 0.0 },
  { "Dezarae", 0.0 },
  { "Dezarai", 0.0 },
  { "Dezaray", 0.0 },
  { "Dezaraye", 0.0 },
  { "Dezare", 0.0 },
  { "Dezarea", 0.0 },
  { "Dezaree", 0.0 },
  { "Dezarey", 0.0 },
  { "Dezaria", 0.0 },
  { "Dezariah", 0.0 },
  { "Dezariyah", 0.0 },
  { "Dezavion", 1.0 },
  { "Dezden", 1.0 },
  { "Dezera", 0.0 },
  { "Dezerae", 0.0 },
  { "Dezerai", 0.0 },
  { "Dezeray", 0.0 },
  { "Dezere", 0.0 },
  { "Dezerea", 0.0 },
  { "Dezeree", 0.0 },
  { "Dezerey", 0.0 },
  { "Dezha", 0.0 },
  { "Dezhan", 1.0 },
  { "Dezhanae", 0.0 },
  { "Dezhane", 0.0 },
  { "Dezhaun", 1.0 },
  { "Dezhon", 1.0 },
  { "Dezi", 0.5905512 },
  { "Dezia", 0.0 },
  { "Deziah", 0.025906736 },
  { "Dezion", 1.0 },
  { "Dezira", 0.0 },
  { "Dezirae", 0.0 },
  { "Deziray", 0.0 },
  { "Deziraye", 0.0 },
  { "Dezire", 0.0 },
  { "Dezirea", 0.0 },
  { "Deziree", 0.0 },
  { "Dezirey", 0.0 },
  { "Deziya", 0.0 },
  { "Deziyah", 0.0 },
  { "Dezja", 0.0 },
  { "Dezjuan", 1.0 },
  { "Dezlyn", 0.0 },
  { "Dezlynn", 0.0 },
  { "Dezman", 1.0 },
  { "Dezmand", 1.0 },
  { "Dezmariah", 0.0 },
  { "Dezmarie", 0.0 },
  { "Dezmen", 1.0 },
  { "Dezmend", 1.0 },
  { "Dezmin", 1.0 },
  { "Dezmon", 1.0 },
  { "Dezmond", 1.0 },
  { "Dezmund", 1.0 },
  { "Dezmyn", 0.7619048 },
  { "Dezon", 1.0 },
  { "Dezra", 0.0 },
  { "Dezrae", 0.0 },
  { "Dezree", 0.0 },
  { "Dezstany", 0.0 },
  { "Deztany", 0.0 },
  { "Deztinee", 0.0 },
  { "Deztiney", 0.0 },
  { "Deztini", 0.0 },
  { "Deztiny", 0.0 },
  { "Deztyni", 0.0 },
  { "Dezya", 0.0 },
  { "Dezyah", 0.0 },
  { "Dezyrae", 0.0 },
  { "Dezyre", 0.0 },
  { "Dezzarae", 0.0 },
  { "Dezzie", 0.020242915 },
  { "Dhahran", 1.0 },
  { "Dhairya", 1.0 },
  { "Dhakirah", 0.0 },
  { "Dhalia", 0.0 },
  { "Dhamani", 1.0 },
  { "Dhamar", 0.0 },
  { "Dhameer", 1.0 },
  { "Dhana", 0.0 },
  { "Dhananjay", 1.0 },
  { "Dhane", 1.0 },
  { "Dhanesh", 1.0 },
  { "Dhani", 0.7256944 },
  { "Dhanna", 0.0 },
  { "Dhanush", 1.0 },
  { "Dhanvi", 0.0 },
  { "Dhanvin", 1.0 },
  { "Dhanya", 0.0 },
  { "Dhara", 0.0 },
  { "Dharius", 1.0 },
  { "Dharma", 0.0121951215 },
  { "Dharmesh", 1.0 },
  { "Dharmik", 1.0 },
  { "Dharshini", 0.0 },
  { "Dharti", 0.0 },
  { "Dhati", 1.0 },
  { "Dhatri", 0.0 },
  { "Dhaval", 1.0 },
  { "Dheeksha", 0.0 },
  { "Dheer", 1.0 },
  { "Dheera", 1.0 },
  { "Dheeraj", 1.0 },
  { "Dheeran", 1.0 },
  { "Dhiago", 1.0 },
  { "Dhilan", 1.0 },
  { "Dhillon", 1.0 },
  { "Dhir", 1.0 },
  { "Dhiraj", 1.0 },
  { "Dhiren", 1.0 },
  { "Dhivya", 0.0 },
  { "Dhiya", 0.0 },
  { "Dhrish", 1.0 },
  { "Dhrithi", 0.0 },
  { "Dhriti", 0.0 },
  { "Dhru", 1.0 },
  { "Dhruthi", 0.0 },
  { "Dhruti", 0.0 },
  { "Dhruv", 1.0 },
  { "Dhruva", 1.0 },
  { "Dhruvan", 1.0 },
  { "Dhruvi", 0.0 },
  { "Dhruvin", 1.0 },
  { "Dhvani", 0.0 },
  { "Dhwani", 0.0 },
  { "Dhyaan", 1.0 },
  { "Dhyan", 1.0 },
  { "Dhyana", 0.0 },
  { "Dhyani", 0.0 },
  { "Dhyey", 1.0 },
  { "Dhylan", 1.0 },
  { "Di", 0.18571429 },
  { "Dia", 0.054342307 },
  { "Diab", 1.0 },
  { "Diablo", 1.0 },
  { "Diabolique", 0.33333334 },
  { "Diadra", 0.0 },
  { "Diago", 1.0 },
  { "Diahann", 0.0 },
  { "Diahanna", 0.0 },
  { "Diahn", 0.0 },
  { "Diaja", 0.0 },
  { "Diajah", 0.0 },
  { "Dial", 1.0 },
  { "Diala", 0.0 },
  { "Diallo", 0.99332446 },
  { "Dialo", 1.0 },
  { "Diamani", 0.0 },
  { "Diamante", 0.69417477 },
  { "Diamantina", 0.0 },
  { "Diamon", 0.04699248 },
  { "Diamond", 0.06400511 },
  { "Diamonde", 0.0 },
  { "Diamondique", 0.0 },
  { "Diamondnique", 0.0 },
  { "Diamonds", 0.0 },
  { "Diamone", 0.0 },
  { "Diamoni", 0.0 },
  { "Diamonique", 0.0 },
  { "Diamonta", 1.0 },
  { "Diamontae", 1.0 },
  { "Diamonte", 0.93415636 },
  { "Diamyn", 0.0 },
  { "Dian", 0.017925736 },
  { "Diana", 0.0035943654 },
  { "Dianah", 0.0 },
  { "Dianalaura", 0.0 },
  { "Dianalee", 0.0 },
  { "Dianaliz", 0.0 },
  { "Dianalynn", 0.0 },
  { "Dianalys", 0.0 },
  { "Dianamarie", 0.0 },
  { "Dianara", 0.0 },
  { "Dianca", 0.0 },
  { "Diandra", 0.0 },
  { "Diandre", 0.9734982 },
  { "Diandrea", 0.0 },
  { "Diandria", 0.0 },
  { "Diane", 0.0026689826 },
  { "Dianela", 0.0 },
  { "Dianeli", 0.0 },
  { "Dianelis", 0.0 },
  { "Dianeliz", 0.0 },
  { "Dianelly", 0.0 },
  { "Dianely", 0.0 },
  { "Dianelys", 0.0 },
  { "Dianet", 0.0 },
  { "Dianetta", 0.0 },
  { "Dianette", 0.0 },
  { "Dianey", 0.0 },
  { "Diangela", 0.0 },
  { "Diangelo", 1.0 },
  { "Diani", 0.0 },
  { "Diania", 0.0 },
  { "Dianie", 0.0 },
  { "Dianira", 0.0 },
  { "Dianita", 0.0 },
  { "Diann", 0.0 },
  { "Dianna", 0.0011513691 },
  { "Diannah", 0.0 },
  { "Dianne", 0.0019519567 },
  { "Dianney", 0.0 },
  { "Dianni", 0.0 },
  { "Diannia", 0.0 },
  { "Diannie", 0.0 },
  { "Dianny", 0.0 },
  { "Diante", 0.984814 },
  { "Diantha", 0.0 },
  { "Dianthe", 0.0 },
  { "Dianthia", 0.0 },
  { "Diany", 0.0 },
  { "Diaquan", 1.0 },
  { "Diar", 1.0 },
  { "Diara", 0.0 },
  { "Diarmuid", 1.0 },
  { "Diarra", 0.21081081 },
  { "Diary", 0.0 },
  { "Diasha", 0.0 },
  { "Diasia", 0.0 },
  { "Diasy", 0.0 },
  { "Diaundra", 0.0 },
  { "Diavian", 0.027777778 },
  { "Diavion", 0.13157895 },
  { "Diavione", 0.0 },
  { "Diavionne", 0.0 },
  { "Diavonte", 1.0 },
  { "Diaz", 1.0 },
  { "Diba", 0.0 },
  { "Dibanhi", 0.0 },
  { "Dibbie", 0.0 },
  { "Dicey", 0.0 },
  { "Dichelle", 0.0 },
  { "Dicie", 0.0 },
  { "Dick", 0.99836284 },
  { "Dickey", 1.0 },
  { "Dicki", 0.0 },
  { "Dickie", 0.94490355 },
  { "Dicksie", 0.0 },
  { "Dickson", 1.0 },
  { "Dicky", 1.0 },
  { "Dicy", 0.0 },
  { "Didi", 0.0 },
  { "Didier", 1.0 },
  { "Didra", 0.0 },
  { "Dieatra", 0.0 },
  { "Diedra", 0.0 },
  { "Diedre", 0.0 },
  { "Diedri", 0.0 },
  { "Diedrich", 1.0 },
  { "Diedrick", 1.0 },
  { "Diego", 0.9978593 },
  { "Diella", 0.0 },
  { "Diellza", 0.0 },
  { "Diem", 0.08963093 },
  { "Diena", 0.0 },
  { "Dienna", 0.0 },
  { "Dienne", 0.0 },
  { "Dieon", 1.0 },
  { "Diep", 0.22222222 },
  { "Diera", 0.0 },
  { "Dierdra", 0.0 },
  { "Dierdre", 0.0 },
  { "Dierick", 1.0 },
  { "Dierk", 1.0 },
  { "Dierks", 1.0 },
  { "Dierra", 0.0 },
  { "Dierre", 1.0 },
  { "Diesel", 1.0 },
  { "Diesha", 0.0 },
  { "Dieter", 1.0 },
  { "Dietmar", 1.0 },
  { "Dietra", 0.0 },
  { "Dietrich", 0.9217437 },
  { "Dietrick", 1.0 },
  { "Dietta", 0.0 },
  { "Dieu", 0.0 },
  { "Dieudonne", 0.0 },
  { "Dieynaba", 0.0 },
  { "Diezel", 1.0 },
  { "Digby", 1.0 },
  { "Digna", 0.0 },
  { "Dihanna", 0.0 },
  { "Dija", 0.0 },
  { "Dijana", 0.0 },
  { "Dijion", 1.0 },
  { "Dijohn", 1.0 },
  { "Dijon", 0.9509943 },
  { "Dijonae", 0.0 },
  { "Dijonay", 0.0 },
  { "Dijone", 0.0 },
  { "Dijonna", 0.0 },
  { "Dijonnae", 0.0 },
  { "Dijonnaise", 0.0 },
  { "Dijoun", 1.0 },
  { "Dijuan", 1.0 },
  { "Dikembe", 1.0 },
  { "Dikran", 1.0 },
  { "Diksha", 0.0 },
  { "Dila", 0.0 },
  { "Dilan", 0.97606456 },
  { "Dilana", 0.0 },
  { "Diland", 1.0 },
  { "Dilanjan", 1.0 },
  { "Dilann", 1.0 },
  { "Dilany", 0.0 },
  { "Dilara", 0.0 },
  { "Dilayla", 0.0 },
  { "Dilbert", 1.0 },
  { "Dilcia", 0.0 },
  { "Dilek", 0.0 },
  { "Dilen", 1.0 },
  { "Dilenny", 0.0 },
  { "Dileydi", 0.0 },
  { "Dilia", 0.0 },
  { "Diliana", 0.0 },
  { "Dililah", 0.0 },
  { "Dilin", 1.0 },
  { "Dill", 1.0 },
  { "Dilla", 0.0 },
  { "Dillan", 0.9578003 },
  { "Dillard", 1.0 },
  { "Dillen", 0.9233983 },
  { "Dillian", 1.0 },
  { "Dillie", 0.0 },
  { "Dillin", 1.0 },
  { "Dillinger", 1.0 },
  { "Dillion", 1.0 },
  { "Dillon", 0.9904377 },
  { "Dillonger", 1.0 },
  { "Dillyn", 0.7055655 },
  { "Dillynn", 0.0 },
  { "Dilma", 0.0 },
  { "Dilmon", 1.0 },
  { "Dilon", 1.0 },
  { "Dilpreet", 0.50666666 },
  { "Dilraj", 1.0 },
  { "Dilreet", 0.0 },
  { "Dilshaan", 1.0 },
  { "Dilshan", 1.0 },
  { "Dilson", 1.0 },
  { "Dilver", 1.0 },
  { "Dilworth", 1.0 },
  { "Dilyla", 0.0 },
  { "Dilylah", 0.0 },
  { "Dilyn", 0.6293996 },
  { "Dilynn", 0.10230179 },
  { "Dilys", 0.0 },
  { "Dim", 0.0 },
  { "Dima", 0.18404908 },
  { "Dimaggio", 1.0 },
  { "Dimani", 1.0 },
  { "Dimante", 1.0 },
  { "Dimar", 1.0 },
  { "Dimarco", 1.0 },
  { "Dimarcus", 1.0 },
  { "Dimari", 1.0 },
  { "Dimario", 1.0 },
  { "Dimarion", 1.0 },
  { "Dimas", 1.0 },
  { "Dimesha", 0.0 },
  { "Dimetra", 0.0 },
  { "Dimetri", 1.0 },
  { "Dimetrius", 1.0 },
  { "Dimia", 0.0 },
  { "Diminic", 1.0 },
  { "Dimitar", 1.0 },
  { "Dimitra", 0.0 },
  { "Dimitre", 1.0 },
  { "Dimitri", 0.9872881 },
  { "Dimitria", 0.0 },
  { "Dimitric", 1.0 },
  { "Dimitrie", 1.0 },
  { "Dimitrios", 1.0 },
  { "Dimitrious", 1.0 },
  { "Dimitris", 1.0 },
  { "Dimitrius", 1.0 },
  { "Dimitriy", 1.0 },
  { "Dimitry", 1.0 },
  { "Dimon", 0.16949153 },
  { "Dimond", 0.0 },
  { "Dimonique", 0.0 },
  { "Dimonte", 1.0 },
  { "Dimple", 0.0 },
  { "Dimples", 0.0 },
  { "Dimya", 0.0 },
  { "Din", 1.0 },
  { "Dina", 0.002494011 },
  { "Dinae", 0.0 },
  { "Dinah", 0.0 },
  { "Dinahlee", 0.0 },
  { "Dinalee", 0.0 },
  { "Dinamarie", 0.0 },
  { "Dinara", 0.0 },
  { "Dinari", 1.0 },
  { "Dinasia", 0.0 },
  { "Dinasti", 0.0 },
  { "Dinasty", 0.0 },
  { "Dindi", 0.0 },
  { "Dinean", 0.0 },
  { "Dineen", 0.0 },
  { "Dineisha", 0.0 },
  { "Dinelle", 0.0 },
  { "Dinene", 0.0 },
  { "Dinero", 1.0 },
  { "Dinese", 0.0 },
  { "Dinesh", 1.0 },
  { "Dinesha", 0.0 },
  { "Dineshia", 0.0 },
  { "Dinetta", 0.0 },
  { "Dinh", 1.0 },
  { "Dinia", 0.0 },
  { "Dinise", 0.0 },
  { "Dinisha", 0.0 },
  { "Dinita", 0.0 },
  { "Dink", 1.0 },
  { "Dinna", 0.0 },
  { "Dinnah", 0.0 },
  { "Dinneen", 0.0 },
  { "Dinnia", 0.0 },
  { "Dinnie", 0.84375 },
  { "Dinnis", 1.0 },
  { "Dino", 1.0 },
  { "Dinora", 0.0 },
  { "Dinorah", 0.0 },
  { "Dinos", 1.0 },
  { "Diny", 0.0 },
  { "Dio", 1.0 },
  { "Diocelina", 0.0 },
  { "Diogenes", 1.0 },
  { "Diogo", 1.0 },
  { "Diolinda", 0.0 },
  { "Diomar", 1.0 },
  { "Diomedes", 1.0 },
  { "Dion", 0.9369462 },
  { "Diona", 0.0 },
  { "Dionah", 0.0 },
  { "Dionca", 0.0 },
  { "Diondra", 0.007215007 },
  { "Diondray", 1.0 },
  { "Diondre", 1.0 },
  { "Diondria", 0.0 },
  { "Dione", 0.19021173 },
  { "Dionel", 1.0 },
  { "Dioni", 0.44444445 },
  { "Dionicia", 0.0 },
  { "Dionicio", 1.0 },
  { "Dionis", 1.0 },
  { "Dionisia", 0.0 },
  { "Dionisio", 1.0 },
  { "Dionisios", 1.0 },
  { "Dionn", 1.0 },
  { "Dionna", 0.0013709898 },
  { "Dionnah", 0.0 },
  { "Dionne", 0.041399073 },
  { "Dionni", 0.0 },
  { "Dionnie", 0.0 },
  { "Dionta", 1.0 },
  { "Diontae", 1.0 },
  { "Diontay", 1.0 },
  { "Diontaye", 1.0 },
  { "Dionte", 0.99122804 },
  { "Diontray", 1.0 },
  { "Diontre", 1.0 },
  { "Diony", 1.0 },
  { "Dionysia", 0.0 },
  { "Dionysios", 1.0 },
  { "Dionysius", 1.0 },
  { "Dionysus", 1.0 },
  { "Dionza", 0.0 },
  { "Diop", 1.0 },
  { "Dior", 0.27626136 },
  { "Diora", 0.0 },
  { "Diore", 0.0 },
  { "Dioselin", 0.0 },
  { "Dioselina", 0.0 },
  { "Dioseline", 0.0 },
  { "Dioselyn", 0.0 },
  { "Diovanni", 1.0 },
  { "Dipa", 0.0 },
  { "Dipak", 1.0 },
  { "Dipali", 0.0 },
  { "Dipesh", 1.0 },
  { "Dipson", 1.0 },
  { "Dipti", 0.0 },
  { "Diquan", 1.0 },
  { "Diquon", 1.0 },
  { "Diran", 1.0 },
  { "Dirck", 1.0 },
  { "Diretha", 0.0 },
  { "Dirinda", 0.0 },
  { "Dirk", 1.0 },
  { "Dirl", 1.0 },
  { "Diron", 1.0 },
  { "Dirrick", 1.0 },
  { "Disa", 0.0 },
  { "Disaya", 0.0 },
  { "Diseree", 0.0 },
  { "Disha", 0.0 },
  { "Dishan", 1.0 },
  { "Dishaun", 1.0 },
  { "Dishawn", 1.0 },
  { "Disheka", 0.0 },
  { "Dishita", 0.0 },
  { "Dishon", 1.0 },
  { "Disiree", 0.0 },
  { "Disney", 0.037037037 },
  { "Dison", 1.0 },
  { "Dissie", 0.0 },
  { "Distin", 1.0 },
  { "Distiny", 0.0 },
  { "Dita", 0.0 },
  { "Diti", 0.0 },
  { "Ditya", 0.0 },
  { "Diva", 0.0 },
  { "Divam", 1.0 },
  { "Divan", 1.0 },
  { "Divante", 1.0 },
  { "Divany", 0.0 },
  { "Diven", 1.0 },
  { "Divesh", 1.0 },
  { "Divia", 0.0 },
  { "Diviana", 0.0 },
  { "Divid", 1.0 },
  { "Divij", 1.0 },
  { "Divija", 0.0 },
  { "Divin", 1.0 },
  { "Divina", 0.0 },
  { "Divine", 0.42593277 },
  { "Divinity", 0.0 },
  { "Divion", 1.0 },
  { "Divisha", 0.0 },
  { "Divit", 1.0 },
  { "Divith", 1.0 },
  { "Divleen", 0.0 },
  { "Divon", 1.0 },
  { "Divontae", 1.0 },
  { "Divonte", 1.0 },
  { "Divya", 0.006802721 },
  { "Divyam", 1.0 },
  { "Divyansh", 1.0 },
  { "Divyanshi", 0.0 },
  { "Divyesh", 1.0 },
  { "Divyne", 0.0 },
  { "Dix", 1.0 },
  { "Dixee", 0.0 },
  { "Dixi", 0.0 },
  { "Dixiana", 0.0 },
  { "Dixianna", 0.0 },
  { "Dixie", 0.017295308 },
  { "Dixielee", 0.0 },
  { "Dixielynn", 0.0 },
  { "Dixon", 0.99810964 },
  { "Dixy", 0.0 },
  { "Diya", 0.0032249396 },
  { "Diyaa", 0.0 },
  { "Diyala", 0.0 },
  { "Diyan", 1.0 },
  { "Diyana", 0.0 },
  { "Diyanna", 0.0 },
  { "Diyar", 1.0 },
  { "Diyari", 1.0 },
  { "Diyon", 1.0 },
  { "Diyonna", 0.0 },
  { "Diyor", 1.0 },
  { "Diyora", 0.0 },
  { "Dizzy", 1.0 },
  { "Dj", 0.9813218 },
  { "Djamila", 0.0 },
  { "Djana", 0.0 },
  { "Djanae", 0.0 },
  { "Django", 1.0 },
  { "Djaun", 1.0 },
  { "Djavan", 1.0 },
  { "Djavon", 1.0 },
  { "Djay", 1.0 },
  { "Djellza", 0.0 },
  { "Djenaba", 0.0 },
  { "Djenabou", 0.0 },
  { "Djene", 0.0 },
  { "Djeneba", 0.0 },
  { "Djenne", 0.0 },
  { "Djibril", 1.0 },
  { "Djimon", 0.95473254 },
  { "Djon", 1.0 },
  { "Djordje", 1.0 },
  { "Djuan", 0.99517244 },
  { "Djuana", 0.0 },
  { "Djuanna", 0.0 },
  { "Djuna", 0.0 },
  { "Dkaiden", 1.0 },
  { "Dkari", 1.0 },
  { "Dkayla", 0.0 },
  { "Dkhari", 1.0 },
  { "Dkota", 1.0 },
  { "Dkwon", 1.0 },
  { "Dlana", 0.0 },
  { "Dlaney", 0.0 },
  { "Dlayna", 0.0 },
  { "Dlila", 0.0 },
  { "Dlilah", 0.0 },
  { "Dlinda", 0.0 },
  { "Dlisa", 0.0 },
  { "Dliyah", 0.0 },
  { "Dlorah", 0.0 },
  { "Dlyan", 1.0 },
  { "Dlyla", 0.0 },
  { "Dlylah", 0.0 },
  { "Dlynn", 0.0 },
  { "Dmani", 0.7282609 },
  { "Dmar", 1.0 },
  { "Dmarco", 1.0 },
  { "Dmarcus", 1.0 },
  { "Dmareon", 1.0 },
  { "Dmari", 1.0 },
  { "Dmaria", 0.0 },
  { "Dmarian", 1.0 },
  { "Dmario", 1.0 },
  { "Dmarion", 1.0 },
  { "Dmarius", 1.0 },
  { "Dmarko", 1.0 },
  { "Dmarkus", 1.0 },
  { "Dmarquis", 1.0 },
  { "Dmarrion", 1.0 },
  { "Dmauri", 1.0 },
  { "Dmaurion", 1.0 },
  { "Dmaya", 0.0 },
  { "Dmetri", 1.0 },
  { "Dmetrius", 1.0 },
  { "Dmia", 0.0 },
  { "Dmichael", 1.0 },
  { "Dmir", 1.0 },
  { "Dmiracle", 0.0 },
  { "Dmitri", 1.0 },
  { "Dmitris", 1.0 },
  { "Dmitrius", 1.0 },
  { "Dmitriy", 1.0 },
  { "Dmitry", 1.0 },
  { "Dmiya", 0.0 },
  { "Dmiyah", 0.0 },
  { "Dmon", 1.0 },
  { "Dmoni", 1.0 },
  { "Dmontae", 1.0 },
  { "Dmonte", 1.0 },
  { "Dmorea", 1.0 },
  { "Dmya", 0.0 },
  { "Dmyah", 0.0 },
  { "Dnae", 0.0 },
  { "Dnaiel", 1.0 },
  { "Dnaielle", 0.0 },
  { "Dnaja", 0.0 },
  { "Dnajah", 0.0 },
  { "Dnasia", 0.0 },
  { "Dnaya", 0.0 },
  { "Dneisha", 0.0 },
  { "Dnesha", 0.0 },
  { "Dnia", 0.0 },
  { "Dniah", 0.0 },
  { "Dniel", 1.0 },
  { "Dnielle", 0.0 },
  { "Dnijah", 0.0 },
  { "Dnisha", 0.0 },
  { "Dniya", 0.0 },
  { "Dniyah", 0.0 },
  { "Dnya", 0.0 },
  { "Dnyah", 0.0 },
  { "Dnyia", 0.0 },
  { "Dnyla", 0.0 },
  { "Dnylah", 0.0 },
  { "Do", 1.0 },
  { "Doaa", 0.0 },
  { "Doak", 1.0 },
  { "Doan", 0.6851852 },
  { "Doane", 1.0 },
  { "Doanld", 1.0 },
  { "Dobbie", 0.6875 },
  { "Dobby", 1.0 },
  { "Dobie", 1.0 },
  { "Doborah", 0.0 },
  { "Doc", 1.0 },
  { "Docia", 0.0 },
  { "Docie", 0.0 },
  { "Dock", 1.0 },
  { "Dockie", 1.0 },
  { "Doctor", 1.0 },
  { "Dodd", 1.0 },
  { "Doddie", 0.0 },
  { "Dodge", 1.0 },
  { "Dodger", 1.0 },
  { "Dodi", 0.010799136 },
  { "Dodie", 0.0 },
  { "Dodson", 1.0 },
  { "Dody", 0.0 },
  { "Doe", 0.0 },
  { "Doel", 1.0 },
  { "Doha", 0.0 },
  { "Dohn", 1.0 },
  { "Dohnovan", 1.0 },
  { "Doil", 1.0 },
  { "Dois", 0.61935484 },
  { "Dokken", 1.0 },
  { "Dokota", 1.0 },
  { "Dola", 0.0 },
  { "Dolan", 1.0 },
  { "Dolce", 0.0 },
  { "Dolen", 1.0 },
  { "Dolena", 0.0 },
  { "Dolene", 0.0 },
  { "Doletha", 0.0 },
  { "Doll", 0.062015504 },
  { "Dolleen", 0.0 },
  { "Dollena", 0.0 },
  { "Dollene", 0.0 },
  { "Dolley", 0.0 },
  { "Dolli", 0.0 },
  { "Dollicia", 0.0 },
  { "Dollie", 0.003176587 },
  { "Dollinda", 0.0 },
  { "Dolline", 0.0 },
  { "Dolly", 0.0022171831 },
  { "Dollye", 0.0 },
  { "Dolora", 0.0 },
  { "Doloras", 0.0 },
  { "Dolores", 0.0062941676 },
  { "Dolorese", 0.0 },
  { "Dolorez", 0.0 },
  { "Doloria", 0.0 },
  { "Dolories", 0.0 },
  { "Doloris", 0.0 },
  { "Dolorita", 0.0 },
  { "Doloros", 0.0 },
  { "Dolorosa", 0.0 },
  { "Dolorous", 0.0 },
  { "Dolorse", 0.0 },
  { "Dolph", 1.0 },
  { "Dolphine", 0.0 },
  { "Dolphis", 1.0 },
  { "Dolphus", 1.0 },
  { "Dolres", 0.0 },
  { "Dolton", 1.0 },
  { "Dom", 1.0 },
  { "Domani", 0.9846626 },
  { "Domanic", 0.9903509 },
  { "Domanick", 1.0 },
  { "Domanik", 1.0 },
  { "Domanique", 0.2802303 },
  { "Domari", 1.0 },
  { "Domarion", 1.0 },
  { "Domeeka", 0.0 },
  { "Domenic", 1.0 },
  { "Domenica", 0.0 },
  { "Domenick", 1.0 },
  { "Domenico", 1.0 },
  { "Domenik", 0.97727275 },
  { "Domenique", 0.25757575 },
  { "Domer", 1.0 },
  { "Dometrius", 1.0 },
  { "Domian", 1.0 },
  { "Domiano", 1.0 },
  { "Domica", 0.0 },
  { "Domicella", 0.0 },
  { "Domico", 1.0 },
  { "Domie", 0.0 },
  { "Domina", 0.0 },
  { "Dominador", 1.0 },
  { "Dominae", 0.0 },
  { "Dominant", 1.0 },
  { "Dominck", 1.0 },
  { "Domineck", 1.0 },
  { "Dominee", 0.0 },
  { "Domineek", 1.0 },
  { "Dominek", 0.77272725 },
  { "Domineke", 1.0 },
  { "Domineque", 0.36585367 },
  { "Dominesha", 0.0 },
  { "Dominga", 0.0026123303 },
  { "Domingo", 0.9980983 },
  { "Domingos", 1.0 },
  { "Domingue", 0.21839081 },
  { "Domini", 0.0 },
  { "Dominic", 0.9925706 },
  { "Dominica", 0.0 },
  { "Dominicc", 1.0 },
  { "Dominice", 0.0 },
  { "Dominicia", 0.0 },
  { "Dominick", 0.995667 },
  { "Dominicka", 0.0 },
  { "Dominico", 1.0 },
  { "Dominie", 0.95804197 },
  { "Dominigue", 0.21800947 },
  { "Dominik", 0.9883278 },
  { "Dominika", 0.0 },
  { "Dominike", 1.0 },
  { "Dominio", 1.0 },
  { "Dominion", 1.0 },
  { "Dominiq", 0.9822695 },
  { "Dominiqu", 0.0 },
  { "Dominiqua", 0.0 },
  { "Dominique", 0.30688015 },
  { "Dominisha", 0.0 },
  { "Domino", 0.052083332 },
  { "Dominoe", 0.0 },
  { "Dominque", 0.37974682 },
  { "Dominquie", 0.5 },
  { "Dominus", 1.0 },
  { "Dominyc", 1.0 },
  { "Dominyck", 1.0 },
  { "Dominyk", 1.0 },
  { "Dominykas", 1.0 },
  { "Domique", 0.32850242 },
  { "Domita", 0.0 },
  { "Domitila", 0.0 },
  { "Domitilo", 1.0 },
  { "Domminic", 1.0 },
  { "Domminick", 1.0 },
  { "Dommonique", 0.0 },
  { "Domnic", 1.0 },
  { "Domnick", 1.0 },
  { "Domnik", 1.0 },
  { "Domnique", 0.33971292 },
  { "Domoni", 1.0 },
  { "Domonic", 0.9762496 },
  { "Domonica", 0.0 },
  { "Domonick", 1.0 },
  { "Domonigue", 0.0 },
  { "Domonik", 1.0 },
  { "Domonique", 0.25622466 },
  { "Domonque", 0.0 },
  { "Domunique", 1.0 },
  { "Domynic", 1.0 },
  { "Domynick", 1.0 },
  { "Domynik", 1.0 },
  { "Domynique", 0.0 },
  { "Don", 0.99281263 },
  { "Dona", 0.009206166 },
  { "Donabelle", 0.0 },
  { "Donaciana", 0.0 },
  { "Donaciano", 1.0 },
  { "Donae", 0.02955665 },
  { "Donagene", 0.0 },
  { "Donah", 0.0 },
  { "Donahue", 1.0 },
  { "Donail", 1.0 },
  { "Donajean", 0.0 },
  { "Donaji", 0.0 },
  { "Donal", 0.99864167 },
  { "Donald", 0.9960697 },
  { "Donalda", 0.0 },
  { "Donaldeen", 0.0 },
  { "Donaldine", 0.0 },
  { "Donaldo", 1.0 },
  { "Donaldson", 1.0 },
  { "Donalee", 0.0 },
  { "Donaleen", 0.0 },
  { "Donalene", 0.0 },
  { "Donall", 1.0 },
  { "Donalyn", 0.0 },
  { "Donalynn", 0.0 },
  { "Donamarie", 0.0 },
  { "Donard", 1.0 },
  { "Donashia", 0.0 },
  { "Donasia", 0.0 },
  { "Donat", 1.0 },
  { "Donata", 0.0 },
  { "Donate", 1.0 },
  { "Donatella", 0.0 },
  { "Donatello", 1.0 },
  { "Donathan", 1.0 },
  { "Donathon", 1.0 },
  { "Donato", 1.0 },
  { "Donavan", 0.99935883 },
  { "Donaven", 1.0 },
  { "Donavin", 1.0 },
  { "Donavon", 1.0 },
  { "Donavyn", 1.0 },
  { "Doncella", 0.0 },
  { "Donchevell", 1.0 },
  { "Donda", 0.0 },
  { "Dondee", 0.0 },
  { "Dondi", 0.67076385 },
  { "Dondie", 0.0 },
  { "Dondra", 0.03929273 },
  { "Dondrae", 1.0 },
  { "Dondray", 1.0 },
  { "Dondre", 1.0 },
  { "Dondrea", 0.1827957 },
  { "Dondrell", 1.0 },
  { "Dondrey", 1.0 },
  { "Dondria", 0.0 },
  { "Dondrick", 1.0 },
  { "Doneal", 1.0 },
  { "Donecia", 0.0 },
  { "Doneen", 0.0 },
  { "Doneisha", 0.0 },
  { "Doneka", 0.0 },
  { "Donel", 0.95839113 },
  { "Doneld", 1.0 },
  { "Donelda", 0.0 },
  { "Donelia", 0.0 },
  { "Donell", 0.88857466 },
  { "Donella", 0.0 },
  { "Donelle", 0.21898822 },
  { "Donene", 0.0 },
  { "Donese", 0.0 },
  { "Donesha", 0.0 },
  { "Doneshia", 0.0 },
  { "Donesia", 0.0 },
  { "Doneta", 0.0 },
  { "Donetta", 0.0 },
  { "Donette", 0.0 },
  { "Doneva", 0.0 },
  { "Donevan", 1.0 },
  { "Donevin", 1.0 },
  { "Doney", 0.5 },
  { "Dong", 1.0 },
  { "Doni", 0.16502462 },
  { "Donia", 0.0 },
  { "Donica", 0.0 },
  { "Donice", 0.0 },
  { "Donicia", 0.0 },
  { "Donicio", 1.0 },
  { "Donie", 0.12051518 },
  { "Doniece", 0.0 },
  { "Doniel", 0.98172325 },
  { "Doniell", 0.0 },
  { "Donielle", 0.044585988 },
  { "Doniesha", 0.0 },
  { "Donika", 0.0 },
  { "Doninic", 1.0 },
  { "Doninique", 0.20652173 },
  { "Doniqua", 0.0 },
  { "Donique", 0.2278481 },
  { "Donis", 0.16172107 },
  { "Donise", 0.0 },
  { "Donisha", 0.0 },
  { "Donishia", 0.0 },
  { "Donita", 0.0 },
  { "Donitra", 0.0 },
  { "Donitta", 0.0 },
  { "Donivan", 1.0 },
  { "Donivee", 0.0 },
  { "Doniven", 1.0 },
  { "Donivin", 1.0 },
  { "Donivon", 1.0 },
  { "Doniya", 0.0 },
  { "Doniyah", 0.0 },
  { "Donja", 0.0 },
  { "Donjae", 1.0 },
  { "Donjanae", 0.0 },
  { "Donjay", 1.0 },
  { "Donje", 1.0 },
  { "Donjuan", 1.0 },
  { "Donlad", 1.0 },
  { "Donld", 1.0 },
  { "Donley", 1.0 },
  { "Donminique", 1.0 },
  { "Donmonique", 0.0 },
  { "Donn", 0.99060035 },
  { "Donna", 0.0026801089 },
  { "Donnabel", 0.0 },
  { "Donnabell", 0.0 },
  { "Donnabelle", 0.0 },
  { "Donnae", 0.0 },
  { "Donnah", 0.0 },
  { "Donnajean", 0.0 },
  { "Donnajo", 0.0 },
  { "Donnal", 1.0 },
  { "Donnald", 1.0 },
  { "Donnalee", 0.0 },
  { "Donnalou", 0.0 },
  { "Donnalyn", 0.0 },
  { "Donnalynn", 0.0 },
  { "Donnamae", 0.0 },
  { "Donnamaria", 0.0 },
  { "Donnamarie", 0.0 },
  { "Donnarae", 0.0 },
  { "Donnas", 0.0 },
  { "Donnasia", 0.0 },
  { "Donnavan", 1.0 },
  { "Donnaven", 1.0 },
  { "Donnavin", 1.0 },
  { "Donnavon", 1.0 },
  { "Donnay", 0.0 },
  { "Donne", 0.40609136 },
  { "Donneisha", 0.0 },
  { "Donnel", 0.94247246 },
  { "Donnell", 0.93150866 },
  { "Donnella", 0.0 },
  { "Donnelle", 0.4259928 },
  { "Donnelly", 1.0 },
  { "Donner", 1.0 },
  { "Donnesha", 0.0 },
  { "Donneshia", 0.0 },
  { "Donneta", 0.0 },
  { "Donnett", 0.0 },
  { "Donnetta", 0.0 },
  { "Donnette", 0.0 },
  { "Donney", 1.0 },
  { "Donni", 0.032432433 },
  { "Donnia", 0.0 },
  { "Donnica", 0.0 },
  { "Donnice", 0.0 },
  { "Donnie", 0.8940832 },
  { "Donniece", 0.0 },
  { "Donnielle", 0.0 },
  { "Donniesha", 0.0 },
  { "Donnika", 0.0 },
  { "Donnis", 0.30359522 },
  { "Donnise", 0.0 },
  { "Donnisha", 0.0 },
  { "Donnita", 0.0 },
  { "Donnitta", 0.0 },
  { "Donnivan", 1.0 },
  { "Donnivin", 1.0 },
  { "Donnovan", 1.0 },
  { "Donnovin", 1.0 },
  { "Donny", 0.9968407 },
  { "Donnye", 1.0 },
  { "Donoban", 1.0 },
  { "Donold", 1.0 },
  { "Donovan", 0.99616 },
  { "Donovann", 1.0 },
  { "Donoven", 1.0 },
  { "Donovin", 1.0 },
  { "Donovon", 1.0 },
  { "Donovyn", 1.0 },
  { "Donquarius", 1.0 },
  { "Donquavious", 1.0 },
  { "Donquell", 1.0 },
  { "Donquez", 1.0 },
  { "Donrico", 1.0 },
  { "Donsha", 0.49019608 },
  { "Donshae", 0.325 },
  { "Donshay", 0.6956522 },
  { "Donta", 0.9760841 },
  { "Dontae", 0.99686635 },
  { "Dontai", 1.0 },
  { "Dontajah", 0.0 },
  { "Dontario", 1.0 },
  { "Dontarious", 1.0 },
  { "Dontarius", 1.0 },
  { "Dontarrius", 1.0 },
  { "Dontasia", 0.0 },
  { "Dontate", 1.0 },
  { "Dontavia", 0.09243698 },
  { "Dontavian", 1.0 },
  { "Dontavias", 1.0 },
  { "Dontavion", 1.0 },
  { "Dontavious", 1.0 },
  { "Dontavis", 1.0 },
  { "Dontavius", 1.0 },
  { "Dontay", 1.0 },
  { "Dontaye", 1.0 },
  { "Dontaysha", 0.0 },
  { "Dontaz", 1.0 },
  { "Donte", 0.9908189 },
  { "Dontea", 1.0 },
  { "Dontee", 1.0 },
  { "Dontel", 1.0 },
  { "Dontell", 1.0 },
  { "Donterio", 1.0 },
  { "Donterious", 1.0 },
  { "Donterius", 1.0 },
  { "Donterrio", 1.0 },
  { "Donterrious", 1.0 },
  { "Donterrius", 1.0 },
  { "Dontevion", 1.0 },
  { "Dontevious", 1.0 },
  { "Dontevius", 1.0 },
  { "Dontey", 1.0 },
  { "Dontez", 1.0 },
  { "Donti", 1.0 },
  { "Dontia", 0.26530612 },
  { "Dontonio", 1.0 },
  { "Dontra", 1.0 },
  { "Dontrae", 1.0 },
  { "Dontrail", 1.0 },
  { "Dontravious", 1.0 },
  { "Dontravius", 1.0 },
  { "Dontray", 1.0 },
  { "Dontre", 1.0 },
  { "Dontrea", 1.0 },
  { "Dontreal", 1.0 },
  { "Dontrel", 1.0 },
  { "Dontrell", 0.99381286 },
  { "Dontrelle", 1.0 },
  { "Dontrey", 1.0 },
  { "Dontrez", 1.0 },
  { "Dontrice", 0.0 },
  { "Donvan", 1.0 },
  { "Dony", 1.0 },
  { "Donya", 0.10663984 },
  { "Donyae", 0.58674467 },
  { "Donyai", 1.0 },
  { "Donyale", 0.18244803 },
  { "Donye", 0.74927115 },
  { "Donyea", 0.6501901 },
  { "Donyel", 0.45142856 },
  { "Donyell", 0.5587583 },
  { "Donyelle", 0.038659792 },
  { "Donyetta", 0.0 },
  { "Donza", 0.0 },
  { "Donzaleigh", 0.0 },
  { "Donzel", 1.0 },
  { "Donzell", 0.9776699 },
  { "Donzella", 0.0 },
  { "Donzetta", 0.0 },
  { "Dooley", 1.0 },
  { "Dor", 0.6875 },
  { "Dora", 0.005202766 },
  { "Dorabelle", 0.0 },
  { "Dorace", 0.0 },
  { "Dorae", 0.0 },
  { "Doraelia", 0.0 },
  { "Dorah", 0.0 },
  { "Dorain", 0.85106385 },
  { "Doraine", 0.0 },
  { "Doral", 0.7152318 },
  { "Doralee", 0.0 },
  { "Doralene", 0.0 },
  { "Doraline", 0.0 },
  { "Doraliz", 0.0 },
  { "Doraly", 0.0 },
  { "Doralyn", 0.0 },
  { "Doralynn", 0.0 },
  { "Doramae", 0.0 },
  { "Doran", 0.9791667 },
  { "Dorance", 1.0 },
  { "Doranda", 0.0 },
  { "Dorane", 0.0 },
  { "Dorann", 0.0 },
  { "Doranna", 0.0 },
  { "Doranne", 0.0 },
  { "Doras", 0.0 },
  { "Doratha", 0.0 },
  { "Dorathea", 0.0 },
  { "Dorathy", 0.0 },
  { "Doray", 0.0 },
  { "Dorca", 0.0 },
  { "Dorcas", 0.0 },
  { "Dorce", 1.0 },
  { "Dorcia", 0.0 },
  { "Dorcie", 0.1923077 },
  { "Dorcus", 0.017421603 },
  { "Dorcy", 1.0 },
  { "Dore", 0.09389672 },
  { "Dorea", 0.0 },
  { "Dorean", 0.2639296 },
  { "Doreatha", 0.0 },
  { "Doreathea", 0.0 },
  { "Doreather", 0.0 },
  { "Doree", 0.0 },
  { "Doreen", 0.0012527809 },
  { "Doreena", 0.0 },
  { "Doreene", 0.0 },
  { "Dorel", 1.0 },
  { "Dorelia", 0.0 },
  { "Dorell", 1.0 },
  { "Dorella", 0.0 },
  { "Dorelle", 0.0 },
  { "Doremus", 1.0 },
  { "Doren", 0.89915967 },
  { "Dorena", 0.0 },
  { "Dorenda", 0.0 },
  { "Dorene", 0.0 },
  { "Doreon", 1.0 },
  { "Dores", 0.2173913 },
  { "Doresa", 0.0 },
  { "Dorese", 0.0 },
  { "Doresha", 0.0 },
  { "Doretha", 0.0 },
  { "Dorethea", 0.0 },
  { "Dorether", 0.0 },
  { "Dorethia", 0.0 },
  { "Dorethy", 0.0 },
  { "Doretta", 0.0 },
  { "Dorette", 0.0 },
  { "Dorey", 0.12727273 },
  { "Dori", 0.0 },
  { "Doria", 0.0 },
  { "Dorian", 0.8142731 },
  { "Doriana", 0.0 },
  { "Doriane", 0.0 },
  { "Doriann", 0.0 },
  { "Dorianna", 0.0 },
  { "Dorianne", 0.0 },
  { "Dorica", 0.0 },
  { "Dorice", 0.0 },
  { "Dorie", 0.020466173 },
  { "Doriel", 1.0 },
  { "Dorielle", 0.0 },
  { "Dorien", 0.9404145 },
  { "Doriene", 0.0 },
  { "Dorienne", 0.0 },
  { "Dories", 0.0 },
  { "Dorilla", 0.0 },
  { "Dorin", 0.8727273 },
  { "Dorina", 0.0 },
  { "Dorinda", 0.0 },
  { "Dorine", 0.0 },
  { "Dorinna", 0.0 },
  { "Dorinne", 0.0 },
  { "Dorion", 1.0 },
  { "Doris", 0.006970432 },
  { "Dorisa", 0.0 },
  { "Dorisann", 0.0 },
  { "Dorise", 0.0 },
  { "Dorisha", 0.0 },
  { "Dorislee", 0.0 },
  { "Dorismae", 0.0 },
  { "Dorismar", 0.0 },
  { "Dorissa", 0.0 },
  { "Doristeen", 0.0 },
  { "Doristene", 0.0 },
  { "Doristine", 0.0 },
  { "Dorit", 0.0 },
  { "Dorita", 0.0 },
  { "Doritha", 0.0 },
  { "Dorithy", 0.0 },
  { "Dorjan", 1.0 },
  { "Dorla", 0.0 },
  { "Dorlan", 1.0 },
  { "Dorland", 1.0 },
  { "Dorlas", 0.0 },
  { "Dorlean", 0.0 },
  { "Dorlee", 0.0 },
  { "Dorleen", 0.0 },
  { "Dorlene", 0.0 },
  { "Dorles", 0.0 },
  { "Dorline", 0.0 },
  { "Dorlis", 0.0 },
  { "Dorlisa", 0.0 },
  { "Dorma", 0.0 },
  { "Dormalee", 0.0 },
  { "Dorman", 1.0 },
  { "Dormilee", 0.0 },
  { "Dormon", 1.0 },
  { "Dorn", 0.9493671 },
  { "Dorna", 0.0 },
  { "Dorne", 1.0 },
  { "Dornell", 1.0 },
  { "Dornisha", 0.0 },
  { "Doro", 1.0 },
  { "Dorohty", 0.0 },
  { "Doron", 1.0 },
  { "Dorota", 0.0 },
  { "Dorotea", 0.0 },
  { "Doroteo", 1.0 },
  { "Dorotha", 0.0 },
  { "Dorothe", 0.0 },
  { "Dorothea", 0.00017749904 },
  { "Dorothee", 0.0 },
  { "Dorothene", 0.0 },
  { "Dorothey", 0.0 },
  { "Dorothia", 0.0 },
  { "Dorothie", 0.0 },
  { "Dorothy", 0.0033118608 },
  { "Dorothyann", 0.0 },
  { "Dorothye", 0.0 },
  { "Dorothymae", 0.0 },
  { "Doroty", 0.0 },
  { "Dorpha", 0.0 },
  { "Dorr", 1.0 },
  { "Dorra", 0.0 },
  { "Dorrace", 0.0 },
  { "Dorraine", 0.0 },
  { "Dorrance", 1.0 },
  { "Dorreen", 0.0 },
  { "Dorrel", 1.0 },
  { "Dorrell", 1.0 },
  { "Dorren", 0.33333334 },
  { "Dorrence", 1.0 },
  { "Dorrene", 0.0 },
  { "Dorretta", 0.0 },
  { "Dorri", 0.0 },
  { "Dorrian", 0.9329004 },
  { "Dorrie", 0.0 },
  { "Dorrien", 1.0 },
  { "Dorrine", 0.0 },
  { "Dorrion", 1.0 },
  { "Dorris", 0.18539014 },
  { "Dorrit", 0.0 },
  { "Dorron", 1.0 },
  { "Dorry", 0.0 },
  { "Dorsa", 0.0 },
  { "Dorse", 1.0 },
  { "Dorsel", 1.0 },
  { "Dorsett", 1.0 },
  { "Dorsey", 0.8685861 },
  { "Dorsha", 0.0 },
  { "Dorsi", 0.0 },
  { "Dorsie", 0.39379844 },
  { "Dorth", 0.0 },
  { "Dortha", 0.0 },
  { "Dorthay", 0.0 },
  { "Dorthe", 0.0 },
  { "Dorthea", 0.0 },
  { "Dorthella", 0.0 },
  { "Dorthey", 0.0 },
  { "Dorthia", 0.0 },
  { "Dorthie", 0.0 },
  { "Dorthula", 0.0 },
  { "Dorthy", 0.0038835907 },
  { "Doruk", 1.0 },
  { "Dorus", 1.0 },
  { "Dorvin", 1.0 },
  { "Dorwin", 1.0 },
  { "Dory", 0.11888783 },
  { "Doryan", 1.0 },
  { "Doryce", 0.0 },
  { "Dorylee", 0.0 },
  { "Dorys", 0.0 },
  { "Dosha", 0.0 },
  { "Doshia", 0.0 },
  { "Doshie", 0.0 },
  { "Dosia", 0.0 },
  { "Dosie", 0.3 },
  { "Doss", 1.0 },
  { "Dossie", 0.45067698 },
  { "Dot", 0.016528925 },
  { "Dotha", 0.0 },
  { "Dotsie", 0.0 },
  { "Dotson", 1.0 },
  { "Dotsy", 0.0 },
  { "Dott", 0.0 },
  { "Dotti", 0.0 },
  { "Dottie", 0.0 },
  { "Dotty", 0.0 },
  { "Dottye", 0.0 },
  { "Doty", 0.0 },
  { "Doua", 0.58208954 },
  { "Douaa", 0.0 },
  { "Douachee", 0.0 },
  { "Doug", 0.99977744 },
  { "Dougal", 1.0 },
  { "Dougald", 1.0 },
  { "Dougals", 1.0 },
  { "Douglas", 0.9962845 },
  { "Douglass", 0.99549395 },
  { "Dougles", 1.0 },
  { "Douglus", 1.0 },
  { "Doulgas", 1.0 },
  { "Dounia", 0.0 },
  { "Dov", 1.0 },
  { "Dova", 0.0 },
  { "Dovan", 1.0 },
  { "Dovber", 1.0 },
  { "Dove", 0.02739726 },
  { "Dover", 1.0 },
  { "Dovetta", 0.0 },
  { "Dovey", 0.0 },
  { "Dovi", 1.0 },
  { "Dovid", 1.0 },
  { "Dovie", 0.0011214534 },
  { "Dovion", 0.0 },
  { "Dovon", 1.0 },
  { "Dovonte", 1.0 },
  { "Dow", 1.0 },
  { "Doward", 1.0 },
  { "Dowd", 1.0 },
  { "Dowe", 1.0 },
  { "Dowell", 1.0 },
  { "Dowl", 1.0 },
  { "Down", 0.0 },
  { "Downey", 1.0 },
  { "Downing", 1.0 },
  { "Doxie", 0.0 },
  { "Doy", 0.95555556 },
  { "Doyal", 1.0 },
  { "Doyce", 0.84615386 },
  { "Doye", 0.84126985 },
  { "Doyel", 1.0 },
  { "Doyl", 1.0 },
  { "Doyla", 0.0 },
  { "Doyle", 0.9917488 },
  { "Doylene", 0.0 },
  { "Doyne", 0.8769531 },
  { "Doyt", 1.0 },
  { "Dozie", 1.0 },
  { "Dozier", 1.0 },
  { "Dquan", 1.0 },
  { "Dquarious", 1.0 },
  { "Dquarius", 1.0 },
  { "Dquavion", 1.0 },
  { "Drace", 1.0 },
  { "Dracen", 1.0 },
  { "Draco", 1.0 },
  { "Draconis", 1.0 },
  { "Dracy", 1.0 },
  { "Draden", 1.0 },
  { "Dradyn", 1.0 },
  { "Drae", 1.0 },
  { "Draeden", 1.0 },
  { "Draedon", 1.0 },
  { "Draedyn", 1.0 },
  { "Draegan", 1.0 },
  { "Draelyn", 1.0 },
  { "Draeson", 1.0 },
  { "Draevyn", 1.0 },
  { "Dragan", 1.0 },
  { "Dragen", 1.0 },
  { "Drago", 1.0 },
  { "Dragon", 1.0 },
  { "Draiden", 1.0 },
  { "Draidyn", 1.0 },
  { "Draike", 1.0 },
  { "Draiman", 1.0 },
  { "Draiven", 1.0 },
  { "Draizy", 0.0 },
  { "Drakar", 1.0 },
  { "Drake", 0.99759454 },
  { "Draken", 1.0 },
  { "Drakkar", 1.0 },
  { "Drako", 1.0 },
  { "Draper", 1.0 },
  { "Drapper", 1.0 },
  { "Draquan", 1.0 },
  { "Drashawn", 1.0 },
  { "Drashti", 0.0 },
  { "Drason", 1.0 },
  { "Draven", 0.9646432 },
  { "Dravid", 1.0 },
  { "Dravin", 1.0 },
  { "Dravion", 1.0 },
  { "Dravon", 1.0 },
  { "Dravyn", 1.0 },
  { "Drax", 1.0 },
  { "Draxie", 0.0 },
  { "Draxton", 1.0 },
  { "Dray", 1.0 },
  { "Draya", 0.0 },
  { "Drayah", 0.0 },
  { "Drayce", 1.0 },
  { "Draycen", 1.0 },
  { "Drayden", 1.0 },
  { "Draydin", 1.0 },
  { "Draydon", 1.0 },
  { "Drayk", 1.0 },
  { "Drayke", 1.0 },
  { "Draylan", 1.0 },
  { "Draylee", 0.0 },
  { "Draylen", 1.0 },
  { "Draylin", 1.0 },
  { "Draylon", 1.0 },
  { "Draymond", 1.0 },
  { "Drayon", 1.0 },
  { "Draysen", 1.0 },
  { "Drayson", 1.0 },
  { "Drayten", 1.0 },
  { "Drayton", 1.0 },
  { "Drayven", 1.0 },
  { "Drayvin", 1.0 },
  { "Drayvon", 1.0 },
  { "Drazen", 1.0 },
  { "Dre", 1.0 },
  { "Drea", 0.0 },
  { "Dreah", 0.0 },
  { "Dream", 0.07232472 },
  { "Dreama", 0.0 },
  { "Dreamer", 0.058139537 },
  { "Drean", 1.0 },
  { "Dreana", 0.0 },
  { "Dreanna", 0.0 },
  { "Dreauna", 0.0 },
  { "Dreda", 0.0 },
  { "Dredan", 1.0 },
  { "Dreddy", 1.0 },
  { "Dreden", 1.0 },
  { "Dredon", 1.0 },
  { "Dredyn", 1.0 },
  { "Dree", 0.0 },
  { "Dreema", 0.0 },
  { "Dreena", 0.0 },
  { "Dregan", 1.0 },
  { "Dreighton", 1.0 },
  { "Dreion", 1.0 },
  { "Drekwon", 1.0 },
  { "Drelan", 1.0 },
  { "Drelen", 1.0 },
  { "Drelin", 1.0 },
  { "Drelon", 1.0 },
  { "Drelyn", 1.0 },
  { "Drelynn", 0.61290324 },
  { "Drema", 0.0 },
  { "Dremon", 1.0 },
  { "Dremond", 1.0 },
  { "Dren", 1.0 },
  { "Drena", 0.0 },
  { "Drenda", 0.0 },
  { "Drene", 0.0 },
  { "Drenisha", 0.0 },
  { "Dreniyah", 0.0 },
  { "Drenna", 0.0 },
  { "Drennan", 1.0 },
  { "Drennen", 1.0 },
  { "Drennon", 1.0 },
  { "Dreon", 1.0 },
  { "Dreona", 0.0 },
  { "Dreonna", 0.0 },
  { "Drequan", 1.0 },
  { "Dresden", 0.8034682 },
  { "Dresdyn", 1.0 },
  { "Dresean", 1.0 },
  { "Dresha", 0.0 },
  { "Dreshan", 1.0 },
  { "Dreshaun", 1.0 },
  { "Dreshawn", 1.0 },
  { "Dreshon", 1.0 },
  { "Dreshun", 1.0 },
  { "Dreson", 1.0 },
  { "Dresyn", 1.0 },
  { "Dretha", 0.0 },
  { "Dreu", 0.8 },
  { "Dreux", 1.0 },
  { "Drevan", 1.0 },
  { "Drevaughn", 1.0 },
  { "Dreven", 1.0 },
  { "Dreveon", 1.0 },
  { "Drevian", 1.0 },
  { "Drevin", 1.0 },
  { "Drevion", 1.0 },
  { "Drevon", 1.0 },
  { "Drevonn", 1.0 },
  { "Drevonte", 1.0 },
  { "Drevyn", 1.0 },
  { "Drew", 0.91509974 },
  { "Drewann", 0.0 },
  { "Drewcilla", 0.0 },
  { "Drewey", 1.0 },
  { "Drewry", 1.0 },
  { "Drex", 1.0 },
  { "Drexal", 1.0 },
  { "Drexel", 0.97003746 },
  { "Drexell", 1.0 },
  { "Drexler", 1.0 },
  { "Drexton", 1.0 },
  { "Drey", 1.0 },
  { "Dreya", 0.0 },
  { "Dreyah", 0.0 },
  { "Dreyanna", 0.0 },
  { "Dreydan", 1.0 },
  { "Dreyden", 1.0 },
  { "Dreydin", 1.0 },
  { "Dreydon", 1.0 },
  { "Dreyken", 1.0 },
  { "Dreylan", 1.0 },
  { "Dreylin", 1.0 },
  { "Dreylon", 1.0 },
  { "Dreyon", 1.0 },
  { "Dreysen", 1.0 },
  { "Dreyson", 1.0 },
  { "Dreyten", 1.0 },
  { "Dreyton", 1.0 },
  { "Dreyven", 1.0 },
  { "Dreyvin", 1.0 },
  { "Drezden", 1.0 },
  { "Drian", 1.0 },
  { "Driana", 0.0 },
  { "Drianna", 0.0 },
  { "Driggs", 1.0 },
  { "Drilon", 1.0 },
  { "Drin", 1.0 },
  { "Drina", 0.0 },
  { "Drinda", 0.0 },
  { "Drisana", 0.0 },
  { "Driscilla", 0.0 },
  { "Driscoll", 1.0 },
  { "Drish", 1.0 },
  { "Drisha", 0.0 },
  { "Drishti", 0.0 },
  { "Drishya", 0.0 },
  { "Driss", 1.0 },
  { "Dristan", 1.0 },
  { "Dristen", 0.8979592 },
  { "Dristi", 0.0 },
  { "Dristin", 0.73333335 },
  { "Driston", 1.0 },
  { "Dristy", 0.0 },
  { "Drita", 0.0 },
  { "Dritan", 1.0 },
  { "Driton", 1.0 },
  { "Driver", 1.0 },
  { "Driyah", 0.0 },
  { "Drizzt", 1.0 },
  { "Dron", 1.0 },
  { "Dru", 0.59771335 },
  { "Druann", 0.0 },
  { "Druanne", 0.0 },
  { "Druce", 1.0 },
  { "Drucella", 0.0 },
  { "Drucie", 0.0 },
  { "Drucilla", 0.0 },
  { "Drue", 0.5657542 },
  { "Druecilla", 0.0 },
  { "Druey", 1.0 },
  { "Drummond", 1.0 },
  { "Drury", 1.0 },
  { "Druscilla", 0.0 },
  { "Drusella", 0.0 },
  { "Drusie", 0.0 },
  { "Drusilla", 0.0 },
  { "Druzella", 0.0 },
  { "Dryden", 1.0 },
  { "Drystal", 0.0 },
  { "Drystan", 1.0 },
  { "Dryver", 1.0 },
  { "Dsean", 1.0 },
  { "Dshae", 0.0 },
  { "Dshanti", 0.0 },
  { "Dshaun", 1.0 },
  { "Dshawn", 1.0 },
  { "Dshayla", 0.0 },
  { "Dshon", 1.0 },
  { "Dshun", 1.0 },
  { "Du", 1.0 },
  { "Dua", 0.0 },
  { "Duaa", 0.0 },
  { "Duain", 1.0 },
  { "Duaine", 1.0 },
  { "Dual", 1.0 },
  { "Duan", 1.0 },
  { "Duana", 0.0 },
  { "Duane", 0.9912321 },
  { "Duann", 0.0 },
  { "Duanna", 0.0 },
  { "Duanne", 0.66871166 },
  { "Duante", 1.0 },
  { "Duard", 1.0 },
  { "Duarte", 1.0 },
  { "Duayne", 1.0 },
  { "Dub", 1.0 },
  { "Duban", 1.0 },
  { "Dublin", 0.7675841 },
  { "Dublyn", 0.0 },
  { "Dubois", 1.0 },
  { "Dubraska", 0.0 },
  { "Duc", 1.0 },
  { "Ducati", 1.0 },
  { "Duce", 1.0 },
  { "Duchein", 1.0 },
  { "Duchess", 0.0 },
  { "Dud", 1.0 },
  { "Dude", 1.0 },
  { "Dudley", 0.98992443 },
  { "Duel", 1.0 },
  { "Duell", 1.0 },
  { "Duella", 0.0 },
  { "Duewa", 0.0 },
  { "Duey", 1.0 },
  { "Duff", 1.0 },
  { "Duffie", 0.85 },
  { "Duffy", 0.97511667 },
  { "Dugald", 1.0 },
  { "Dugan", 1.0 },
  { "Duglas", 1.0 },
  { "Duha", 0.0 },
  { "Duice", 0.0 },
  { "Duie", 1.0 },
  { "Duilio", 1.0 },
  { "Dujon", 1.0 },
  { "Dujuan", 1.0 },
  { "Dujuana", 0.0 },
  { "Duke", 1.0 },
  { "Dukens", 1.0 },
  { "Dula", 0.0 },
  { "Dulan", 1.0 },
  { "Dulaney", 0.0 },
  { "Dulani", 1.0 },
  { "Dulce", 0.0028536485 },
  { "Dulcemaria", 0.0 },
  { "Dulcey", 0.0 },
  { "Dulcia", 0.0 },
  { "Dulcie", 0.0 },
  { "Dulcinea", 0.0 },
  { "Dulcy", 0.0 },
  { "Dulio", 1.0 },
  { "Dulse", 0.0 },
  { "Dulton", 1.0 },
  { "Dumaka", 1.0 },
  { "Dumar", 1.0 },
  { "Dumas", 1.0 },
  { "Dumont", 1.0 },
  { "Duna", 0.0 },
  { "Dunbar", 1.0 },
  { "Duncan", 0.9994279 },
  { "Dundre", 1.0 },
  { "Dune", 1.0 },
  { "Dung", 0.6203704 },
  { "Dunia", 0.0 },
  { "Duniya", 0.0 },
  { "Dunk", 1.0 },
  { "Dunn", 1.0 },
  { "Dunstan", 1.0 },
  { "Dunta", 1.0 },
  { "Duntae", 1.0 },
  { "Dunte", 1.0 },
  { "Dunya", 0.0 },
  { "Duong", 1.0 },
  { "Dupre", 1.0 },
  { "Dupree", 1.0 },
  { "Dupri", 1.0 },
  { "Duquan", 1.0 },
  { "Dura", 0.0 },
  { "Durah", 0.0 },
  { "Dural", 1.0 },
  { "Duran", 1.0 },
  { "Durand", 1.0 },
  { "Durane", 1.0 },
  { "Durango", 1.0 },
  { "Durant", 1.0 },
  { "Durante", 1.0 },
  { "Durbin", 1.0 },
  { "Dureen", 0.0 },
  { "Durel", 1.0 },
  { "Durell", 1.0 },
  { "Durelle", 0.8067227 },
  { "Durenda", 0.0 },
  { "Durene", 0.0 },
  { "Duretta", 0.0 },
  { "Durga", 0.0 },
  { "Durham", 1.0 },
  { "Durie", 1.0 },
  { "Duriel", 1.0 },
  { "Durinda", 0.0 },
  { "Duriyah", 0.0 },
  { "Durk", 1.0 },
  { "Durke", 1.0 },
  { "Durl", 1.0 },
  { "Durland", 1.0 },
  { "Durlene", 0.0 },
  { "Duron", 1.0 },
  { "Durrani", 0.0 },
  { "Durrel", 1.0 },
  { "Durrell", 1.0 },
  { "Durriyah", 0.0 },
  { "Durron", 1.0 },
  { "Duru", 0.0 },
  { "Durva", 0.0 },
  { "Durville", 1.0 },
  { "Durward", 1.0 },
  { "Durwin", 1.0 },
  { "Durwood", 1.0 },
  { "Duryea", 1.0 },
  { "Dusan", 1.0 },
  { "Dusean", 1.0 },
  { "Dushan", 1.0 },
  { "Dushane", 1.0 },
  { "Dusharme", 0.0 },
  { "Dushaun", 1.0 },
  { "Dushawn", 0.9753363 },
  { "Dushon", 0.85 },
  { "Dusin", 1.0 },
  { "Dusitn", 1.0 },
  { "Duska", 0.0 },
  { "Duskin", 1.0 },
  { "Dustan", 1.0 },
  { "Dustee", 0.0 },
  { "Dusten", 1.0 },
  { "Dusti", 0.005113636 },
  { "Dustie", 0.012114538 },
  { "Dustin", 0.9932858 },
  { "Dustina", 0.0 },
  { "Dustine", 0.3966942 },
  { "Dustinjames", 1.0 },
  { "Dustion", 1.0 },
  { "Dustn", 1.0 },
  { "Duston", 1.0 },
  { "Dustun", 1.0 },
  { "Dusty", 0.70460117 },
  { "Dustyn", 0.976834 },
  { "Dut", 1.0 },
  { "Dutch", 1.0 },
  { "Dutchess", 0.0 },
  { "Duuana", 0.0 },
  { "Duuna", 0.0 },
  { "Duval", 1.0 },
  { "Duvall", 1.0 },
  { "Duvan", 1.0 },
  { "Duveen", 0.0 },
  { "Duvid", 1.0 },
  { "Duvier", 1.0 },
  { "Duvon", 1.0 },
  { "Duwaine", 1.0 },
  { "Duwan", 1.0 },
  { "Duwana", 0.0 },
  { "Duwane", 1.0 },
  { "Duwanna", 0.0 },
  { "Duward", 1.0 },
  { "Duwayne", 1.0 },
  { "Duy", 1.0 },
  { "Duyen", 0.0 },
  { "Dvante", 1.0 },
  { "Dvaughn", 1.0 },
  { "Dvid", 1.0 },
  { "Dvir", 1.0 },
  { "Dvon", 1.0 },
  { "Dvonta", 1.0 },
  { "Dvontae", 1.0 },
  { "Dvonte", 1.0 },
  { "Dvora", 0.0 },
  { "Dvyne", 0.0 },
  { "Dwain", 1.0 },
  { "Dwaina", 0.0 },
  { "Dwaine", 1.0 },
  { "Dwala", 0.0 },
  { "Dwan", 0.5709571 },
  { "Dwana", 0.0 },
  { "Dwanda", 0.0 },
  { "Dwane", 1.0 },
  { "Dwania", 0.0 },
  { "Dwann", 0.8 },
  { "Dwanna", 0.0 },
  { "Dwanye", 1.0 },
  { "Dward", 1.0 },
  { "Dwaun", 1.0 },
  { "Dwaylon", 1.0 },
  { "Dwayn", 1.0 },
  { "Dwayna", 0.0 },
  { "Dwayne", 0.99394506 },
  { "Dwen", 1.0 },
  { "Dwendolyn", 0.0 },
  { "Dwight", 0.99584776 },
  { "Dwij", 1.0 },
  { "Dwija", 0.0 },
  { "Dwon", 1.0 },
  { "Dwone", 1.0 },
  { "Dwright", 1.0 },
  { "Dwuan", 1.0 },
  { "Dwyane", 1.0 },
  { "Dwyer", 1.0 },
  { "Dwyla", 0.0 },
  { "Dwyn", 0.0 },
  { "Dwyne", 1.0 },
  { "Dyala", 0.0 },
  { "Dyami", 0.9833887 },
  { "Dyamon", 0.0 },
  { "Dyamond", 0.008539709 },
  { "Dyan", 0.037866667 },
  { "Dyana", 0.0 },
  { "Dyanara", 0.0 },
  { "Dyandra", 0.0 },
  { "Dyane", 0.0 },
  { "Dyani", 0.009124087 },
  { "Dyanira", 0.0 },
  { "Dyann", 0.0 },
  { "Dyanna", 0.0 },
  { "Dyanne", 0.0 },
  { "Dyanni", 0.0 },
  { "Dyante", 1.0 },
  { "Dyasia", 0.0 },
  { "Dyce", 1.0 },
  { "Dycen", 1.0 },
  { "Dyego", 1.0 },
  { "Dyeisha", 0.0 },
  { "Dyemond", 0.0 },
  { "Dyer", 1.0 },
  { "Dyesha", 0.0 },
  { "Dyian", 1.0 },
  { "Dyisha", 0.0 },
  { "Dyke", 1.0 },
  { "Dykeria", 0.0 },
  { "Dylahn", 1.0 },
  { "Dylam", 1.0 },
  { "Dylan", 0.9697274 },
  { "Dylana", 0.0 },
  { "Dyland", 1.0 },
  { "Dylane", 0.0 },
  { "Dylaney", 0.0 },
  { "Dylanger", 1.0 },
  { "Dylanie", 0.0 },
  { "Dylanjacob", 1.0 },
  { "Dylanjames", 1.0 },
  { "Dylanmichael", 1.0 },
  { "Dylann", 0.29225737 },
  { "Dylanne", 0.0 },
  { "Dylanthomas", 1.0 },
  { "Dyle", 1.0 },
  { "Dylen", 0.97689074 },
  { "Dylenn", 1.0 },
  { "Dylhan", 1.0 },
  { "Dylian", 1.0 },
  { "Dylilah", 0.0 },
  { "Dylin", 0.82913166 },
  { "Dyllan", 0.88007957 },
  { "Dyllen", 1.0 },
  { "Dyllian", 1.0 },
  { "Dyllin", 1.0 },
  { "Dyllon", 1.0 },
  { "Dyllyn", 1.0 },
  { "Dylon", 1.0 },
  { "Dylyn", 0.7826087 },
  { "Dylynn", 0.0 },
  { "Dyman", 0.0 },
  { "Dymere", 1.0 },
  { "Dymin", 0.0 },
  { "Dymir", 1.0 },
  { "Dymon", 0.056265984 },
  { "Dymond", 0.04038257 },
  { "Dymone", 0.0 },
  { "Dymonique", 0.0 },
  { "Dymphna", 0.0 },
  { "Dymple", 0.0 },
  { "Dyna", 0.0 },
  { "Dynah", 0.0 },
  { "Dynalee", 0.0 },
  { "Dynasha", 0.0 },
  { "Dynasia", 0.0 },
  { "Dynastee", 0.0 },
  { "Dynasti", 0.0 },
  { "Dynastie", 0.0 },
  { "Dynasty", 0.0 },
  { "Dyneisha", 0.0 },
  { "Dynell", 0.90625 },
  { "Dynelle", 0.0 },
  { "Dynesha", 0.0 },
  { "Dyneshia", 0.0 },
  { "Dynesti", 0.0 },
  { "Dynesty", 0.0 },
  { "Dynetta", 0.0 },
  { "Dynette", 0.0 },
  { "Dynisha", 0.0 },
  { "Dynisty", 0.0 },
  { "Dynita", 0.0 },
  { "Dynver", 0.0 },
  { "Dyon", 0.97206706 },
  { "Dyona", 0.0 },
  { "Dyondre", 1.0 },
  { "Dyoni", 0.0 },
  { "Dyonna", 0.0 },
  { "Dyonne", 0.0 },
  { "Dyontae", 1.0 },
  { "Dyonte", 1.0 },
  { "Dyquan", 1.0 },
  { "Dyral", 1.0 },
  { "Dyran", 1.0 },
  { "Dyrell", 1.0 },
  { "Dyrk", 1.0 },
  { "Dyron", 1.0 },
  { "Dysan", 1.0 },
  { "Dysean", 1.0 },
  { "Dysen", 1.0 },
  { "Dysha", 0.0 },
  { "Dyshaun", 1.0 },
  { "Dyshawn", 1.0 },
  { "Dysheka", 0.0 },
  { "Dyshon", 1.0 },
  { "Dyson", 0.99613005 },
  { "Dystanie", 0.0 },
  { "Dystany", 0.0 },
  { "Dystinee", 0.0 },
  { "Dystini", 0.0 },
  { "Dystiny", 0.0 },
  { "Dyuthi", 0.0 },
  { "Dyuti", 0.0 },
  { "Dyvion", 1.0 },
  { "Dyvon", 1.0 },
  { "Dyvonne", 0.0 },
  { "Dywan", 1.0 },
  { "Dywana", 0.0 },
  { "Dywane", 1.0 },
  { "Dywanna", 0.0 },
  { "Dywayne", 1.0 },
  { "Dzaria", 0.0 },
  { "Dzejla", 0.0 },
  { "Dzenan", 1.0 },
  { "Dzeneta", 0.0 },
  { "Dzenita", 0.0 },
  { "Dziah", 0.0 },
  { "Dzion", 1.0 },
  { "Dzire", 0.0 },
  { "Dziyah", 0.0 },
  { "Dzung", 1.0 },
  { "Dzuy", 1.0 },
  { "Dzya", 0.0 },
  { "Dzyre", 0.0 },
  { "Ea", 0.0 },
  { "Eabha", 0.0 },
  { "Eaden", 0.64827585 },
  { "Eadie", 0.0 },
  { "Eadon", 1.0 },
  { "Eadric", 1.0 },
  { "Eadyn", 0.0 },
  { "Eaen", 1.0 },
  { "Eagan", 1.0 },
  { "Eagle", 1.0 },
  { "Eain", 1.0 },
  { "Ealon", 1.0 },
  { "Ealyn", 0.0 },
  { "Eames", 1.0 },
  { "Eammon", 1.0 },
  { "Eamon", 1.0 },
  { "Eamonn", 1.0 },
  { "Ean", 1.0 },
  { "Eann", 1.0 },
  { "Eanna", 0.0 },
  { "Eara", 0.0 },
  { "Eardie", 0.0 },
  { "Earic", 1.0 },
  { "Earica", 0.0 },
  { "Earie", 0.0 },
  { "Earin", 1.0 },
  { "Earl", 0.99226516 },
  { "Earla", 0.0 },
  { "Earland", 1.0 },
  { "Earldean", 0.0 },
  { "Earldene", 0.0 },
  { "Earldine", 0.0 },
  { "Earle", 0.9758249 },
  { "Earlean", 0.0 },
  { "Earleane", 0.0 },
  { "Earlee", 0.24719101 },
  { "Earleen", 0.0 },
  { "Earleene", 0.0 },
  { "Earlen", 0.0 },
  { "Earlena", 0.0 },
  { "Earlene", 0.00035285816 },
  { "Earlesha", 0.0 },
  { "Earletta", 0.0 },
  { "Earlette", 0.0 },
  { "Earley", 1.0 },
  { "Earlie", 0.6299081 },
  { "Earlin", 1.0 },
  { "Earlina", 0.0 },
  { "Earline", 0.003558075 },
  { "Earling", 1.0 },
  { "Earlis", 1.0 },
  { "Earlisa", 0.0 },
  { "Earlisha", 0.0 },
  { "Earlon", 1.0 },
  { "Early", 0.9138369 },
  { "Earlyn", 0.0 },
  { "Earlyne", 0.0 },
  { "Earlynn", 0.0 },
  { "Earma", 0.0 },
  { "Earman", 1.0 },
  { "Earmel", 1.0 },
  { "Earmer", 0.0 },
  { "Earmie", 0.0 },
  { "Earmon", 1.0 },
  { "Earn", 1.0 },
  { "Earna", 0.0 },
  { "Earnease", 0.0 },
  { "Earnell", 0.77272725 },
  { "Earnest", 0.9876673 },
  { "Earnestean", 0.0 },
  { "Earnesteen", 0.0 },
  { "Earnestene", 0.0 },
  { "Earnestine", 0.0037815387 },
  { "Earney", 1.0 },
  { "Earnice", 0.0 },
  { "Earnie", 0.78338873 },
  { "Earnistine", 0.0 },
  { "Earnst", 1.0 },
  { "Earnstine", 0.0 },
  { "Earon", 0.9709302 },
  { "Earron", 1.0 },
  { "Earsel", 0.9177215 },
  { "Earsell", 0.0 },
  { "Earsie", 0.029761905 },
  { "Earskin", 1.0 },
  { "Earsley", 0.0 },
  { "Earstine", 0.0 },
  { "Earth", 0.0 },
  { "Eartha", 0.010862186 },
  { "Earthalee", 0.0 },
  { "Earthel", 0.0 },
  { "Earther", 0.0 },
  { "Earthy", 1.0 },
  { "Earven", 1.0 },
  { "Earvie", 1.0 },
  { "Earvin", 1.0 },
  { "Eary", 0.0 },
  { "Easha", 0.0 },
  { "Eashaan", 1.0 },
  { "Eashan", 1.0 },
  { "Easley", 1.0 },
  { "Eason", 1.0 },
  { "Eastan", 1.0 },
  { "Easten", 1.0 },
  { "Easter", 0.011840689 },
  { "Easther", 0.0 },
  { "Eastin", 0.9726776 },
  { "Eastlyn", 0.0 },
  { "Eastlynn", 0.0 },
  { "Eastman", 1.0 },
  { "Easton", 0.98443484 },
  { "Eastyn", 0.5934959 },
  { "Eastynn", 0.0 },
  { "Eathan", 1.0 },
  { "Eathel", 0.026819924 },
  { "Eathen", 1.0 },
  { "Eather", 0.010928961 },
  { "Eathin", 1.0 },
  { "Eathon", 1.0 },
  { "Eathyn", 1.0 },
  { "Eaton", 1.0 },
  { "Eava", 0.0 },
  { "Eavan", 0.03726708 },
  { "Eaven", 1.0 },
  { "Eavie", 0.0 },
  { "Eb", 1.0 },
  { "Ebaad", 1.0 },
  { "Eban", 1.0 },
  { "Ebanie", 0.0 },
  { "Ebany", 0.0 },
  { "Ebaristo", 1.0 },
  { "Ebay", 1.0 },
  { "Ebb", 1.0 },
  { "Ebba", 0.006289308 },
  { "Ebbe", 1.0 },
  { "Ebben", 1.0 },
  { "Ebbie", 0.48545176 },
  { "Ebbin", 1.0 },
  { "Ebbony", 0.0 },
  { "Ebby", 0.18518518 },
  { "Ebe", 0.0 },
  { "Ebecca", 0.0 },
  { "Ebelia", 0.0 },
  { "Ebelin", 0.0 },
  { "Ebelina", 0.0 },
  { "Ebelyn", 0.0 },
  { "Eben", 0.99813604 },
  { "Ebenezer", 0.97307223 },
  { "Eber", 1.0 },
  { "Eberardo", 1.0 },
  { "Ebere", 0.0 },
  { "Eberhard", 1.0 },
  { "Ebert", 1.0 },
  { "Ebin", 1.0 },
  { "Ebna", 0.0 },
  { "Ebon", 1.0 },
  { "Ebone", 0.0 },
  { "Ebonee", 0.0 },
  { "Eboney", 0.0 },
  { "Eboni", 0.00077208155 },
  { "Ebonie", 0.0 },
  { "Ebonii", 0.0 },
  { "Ebonique", 0.0 },
  { "Ebonne", 0.0 },
  { "Ebonnie", 0.0 },
  { "Ebony", 0.011618406 },
  { "Ebonye", 0.0 },
  { "Ebraheem", 1.0 },
  { "Ebrahim", 1.0 },
  { "Ebrahima", 1.0 },
  { "Ebrima", 1.0 },
  { "Ebru", 0.0 },
  { "Ebubechukwu", 1.0 },
  { "Ebunoluwa", 0.0 },
  { "Ebward", 1.0 },
  { "Ecaterina", 0.0 },
  { "Ece", 0.0 },
  { "Ecequiel", 1.0 },
  { "Echelle", 0.0 },
  { "Echo", 0.026490066 },
  { "Echoe", 0.0 },
  { "Echol", 1.0 },
  { "Ecker", 1.0 },
  { "Ecko", 0.104761906 },
  { "Ecrin", 0.0 },
  { "Ecstasy", 0.0 },
  { "Ector", 1.0 },
  { "Ed", 0.99373174 },
  { "Eda", 0.0 },
  { "Edahi", 1.0 },
  { "Edalia", 0.0 },
  { "Edaline", 0.0 },
  { "Edalyn", 0.0 },
  { "Edan", 0.8758568 },
  { "Edana", 0.0 },
  { "Edawrd", 1.0 },
  { "Edberg", 1.0 },
  { "Edbert", 1.0 },
  { "Edd", 0.9976147 },
  { "Edda", 0.0 },
  { "Edden", 0.8636364 },
  { "Edder", 1.0 },
  { "Eddi", 0.952 },
  { "Eddica", 0.0 },
  { "Eddie", 0.9301784 },
  { "Eddieberto", 1.0 },
  { "Eddiel", 1.0 },
  { "Eddin", 1.0 },
  { "Eddis", 0.120300755 },
  { "Eddison", 0.98417723 },
  { "Eddith", 0.0 },
  { "Eddrick", 1.0 },
  { "Eddward", 1.0 },
  { "Eddy", 0.9756152 },
  { "Eddye", 0.02 },
  { "Eddyson", 1.0 },
  { "Eddythe", 0.0 },
  { "Ede", 0.0 },
  { "Edee", 0.0 },
  { "Edel", 0.8924485 },
  { "Edelia", 0.0 },
  { "Edelin", 0.0 },
  { "Edelina", 0.0 },
  { "Edeline", 0.0 },
  { "Edell", 0.15306123 },
  { "Edelle", 0.0 },
  { "Edelmira", 0.0 },
  { "Edelmiro", 1.0 },
  { "Edelweiss", 0.0 },
  { "Edelyn", 0.0 },
  { "Edelynn", 0.0 },
  { "Edem", 1.0 },
  { "Eden", 0.14770995 },
  { "Edena", 0.0 },
  { "Edengrace", 0.0 },
  { "Edenilson", 1.0 },
  { "Edenrose", 0.0 },
  { "Eder", 1.0 },
  { "Ederick", 1.0 },
  { "Ederson", 1.0 },
  { "Edessa", 0.0 },
  { "Edford", 1.0 },
  { "Edgar", 0.995327 },
  { "Edgard", 1.0 },
  { "Edgardo", 1.0 },
  { "Edgareduardo", 1.0 },
  { "Edgbert", 1.0 },
  { "Edge", 1.0 },
  { "Edgel", 1.0 },
  { "Edger", 1.0 },
  { "Edgerin", 1.0 },
  { "Edgerrin", 1.0 },
  { "Edguin", 1.0 },
  { "Edher", 1.0 },
  { "Edi", 0.9310345 },
  { "Edia", 0.0 },
  { "Ediberto", 1.0 },
  { "Edica", 0.0 },
  { "Edice", 0.0 },
  { "Edid", 0.0 },
  { "Edie", 0.017123288 },
  { "Ediel", 1.0 },
  { "Edier", 1.0 },
  { "Edieth", 0.0 },
  { "Edika", 0.0 },
  { "Edil", 0.83870965 },
  { "Edilberto", 1.0 },
  { "Edilia", 0.0 },
  { "Edilson", 1.0 },
  { "Edin", 0.94158417 },
  { "Edina", 0.0 },
  { "Edinson", 1.0 },
  { "Edir", 1.0 },
  { "Edis", 0.74050635 },
  { "Edisa", 0.0 },
  { "Edisha", 0.0 },
  { "Edison", 0.9904349 },
  { "Edisson", 1.0 },
  { "Edit", 0.0 },
  { "Edita", 0.0 },
  { "Edith", 0.003100548 },
  { "Editha", 0.0 },
  { "Edithe", 0.0 },
  { "Edithmae", 0.0 },
  { "Ediz", 1.0 },
  { "Edjuan", 1.0 },
  { "Edker", 1.0 },
  { "Edla", 0.0 },
  { "Edley", 1.0 },
  { "Edlin", 0.046979867 },
  { "Edlow", 1.0 },
  { "Edlyn", 0.0 },
  { "Edlynn", 0.0 },
  { "Edma", 0.0 },
  { "Edmae", 0.0 },
  { "Edman", 1.0 },
  { "Edmanuel", 1.0 },
  { "Edmar", 1.0 },
  { "Edmay", 0.0 },
  { "Edmee", 0.0 },
  { "Edmere", 0.0 },
  { "Edmilson", 1.0 },
  { "Edmon", 1.0 },
  { "Edmona", 0.0 },
  { "Edmond", 0.9997117 },
  { "Edmondo", 1.0 },
  { "Edmonia", 0.0 },
  { "Edmont", 1.0 },
  { "Edmound", 1.0 },
  { "Edmund", 0.9988147 },
  { "Edmundo", 1.0 },
  { "Edna", 0.0035300734 },
  { "Ednah", 0.0 },
  { "Ednamae", 0.0 },
  { "Ednamay", 0.0 },
  { "Ednar", 0.0 },
  { "Edner", 0.175 },
  { "Ednesha", 0.0 },
  { "Edney", 1.0 },
  { "Ednita", 0.0 },
  { "Edo", 1.0 },
  { "Edoardo", 1.0 },
  { "Edolia", 0.0 },
  { "Edom", 0.0 },
  { "Edon", 1.0 },
  { "Edona", 0.0 },
  { "Edonis", 1.0 },
  { "Edony", 0.0 },
  { "Edora", 0.0 },
  { "Edouard", 1.0 },
  { "Edra", 0.0 },
  { "Edras", 1.0 },
  { "Edrees", 1.0 },
  { "Edrei", 1.0 },
  { "Edrena", 0.0 },
  { "Edrey", 1.0 },
  { "Edria", 0.0 },
  { "Edrian", 1.0 },
  { "Edriana", 0.0 },
  { "Edrianna", 0.0 },
  { "Edric", 1.0 },
  { "Edrica", 0.0 },
  { "Edrick", 1.0 },
  { "Edricka", 0.0 },
  { "Edrie", 0.0 },
  { "Edriel", 1.0 },
  { "Edrik", 1.0 },
  { "Edrin", 1.0 },
  { "Edrina", 0.0 },
  { "Edris", 0.2462845 },
  { "Edro", 1.0 },
  { "Edroy", 1.0 },
  { "Edsel", 1.0 },
  { "Edsell", 1.0 },
  { "Edsil", 1.0 },
  { "Edsol", 1.0 },
  { "Edson", 1.0 },
  { "Edsson", 1.0 },
  { "Edu", 1.0 },
  { "Eduan", 1.0 },
  { "Eduar", 1.0 },
  { "Eduard", 1.0 },
  { "Eduarda", 0.0 },
  { "Eduardo", 0.99574393 },
  { "Eduardoluis", 1.0 },
  { "Eduin", 1.0 },
  { "Edurdo", 1.0 },
  { "Eduvigen", 0.0 },
  { "Eduviges", 0.0 },
  { "Eduvijes", 0.0 },
  { "Eduvina", 0.0 },
  { "Edvard", 1.0 },
  { "Edvardo", 1.0 },
  { "Edvin", 1.0 },
  { "Edw", 1.0 },
  { "Edwa", 0.0 },
  { "Edwad", 1.0 },
  { "Edwana", 0.0 },
  { "Edwar", 1.0 },
  { "Edward", 0.99584836 },
  { "Edwarda", 0.0 },
  { "Edwardd", 1.0 },
  { "Edwardine", 0.0 },
  { "Edwardjames", 1.0 },
  { "Edwardo", 1.0 },
  { "Edwards", 1.0 },
  { "Edweina", 0.0 },
  { "Edwell", 1.0 },
  { "Edwen", 1.0 },
  { "Edwena", 0.0 },
  { "Edwidge", 0.45454547 },
  { "Edwill", 1.0 },
  { "Edwin", 0.99504167 },
  { "Edwina", 0.0 },
  { "Edwind", 1.0 },
  { "Edwing", 1.0 },
  { "Edwinn", 1.0 },
  { "Edwinna", 0.0 },
  { "Edwon", 1.0 },
  { "Edwood", 1.0 },
  { "Edword", 1.0 },
  { "Edwrd", 1.0 },
  { "Edwuin", 1.0 },
  { "Edwyn", 1.0 },
  { "Edwyna", 0.0 },
  { "Edwynn", 1.0 },
  { "Edy", 0.74285716 },
  { "Edyce", 0.0 },
  { "Edye", 0.0 },
  { "Edyn", 0.09541063 },
  { "Edynn", 0.0 },
  { "Edyson", 1.0 },
  { "Edyta", 0.0 },
  { "Edyth", 0.0 },
  { "Edytha", 0.0 },
  { "Edythe", 0.0 },
  { "Edzel", 1.0 },
  { "Edzon", 1.0 },
  { "Eean", 1.0 },
  { "Eelijah", 1.0 },
  { "Eelis", 1.0 },
  { "Eeliyah", 0.0 },
  { "Eelyn", 0.0 },
  { "Eeman", 0.0 },
  { "Eera", 0.0 },
  { "Eero", 1.0 },
  { "Eesa", 1.0 },
  { "Eesha", 0.0 },
  { "Eeshan", 1.0 },
  { "Eeva", 0.0 },
  { "Eevee", 0.0 },
  { "Eevie", 0.0 },
  { "Eeyan", 1.0 },
  { "Efe", 1.0 },
  { "Effa", 0.0 },
  { "Effie", 0.005501153 },
  { "Efford", 1.0 },
  { "Effrem", 1.0 },
  { "Effren", 1.0 },
  { "Effrey", 1.0 },
  { "Effy", 0.0 },
  { "Efia", 0.0 },
  { "Efigenia", 0.0 },
  { "Efosa", 1.0 },
  { "Efraim", 1.0 },
  { "Efrain", 0.99831903 },
  { "Efram", 1.0 },
  { "Efran", 1.0 },
  { "Efrat", 0.0 },
  { "Efrata", 0.0 },
  { "Efrayim", 1.0 },
  { "Efrem", 1.0 },
  { "Efren", 1.0 },
  { "Efrian", 1.0 },
  { "Efrin", 1.0 },
  { "Efrosini", 0.0 },
  { "Efstathia", 0.0 },
  { "Efstathios", 1.0 },
  { "Efstratios", 1.0 },
  { "Efthimia", 0.0 },
  { "Efthimios", 1.0 },
  { "Efthymia", 0.0 },
  { "Eftihia", 0.0 },
  { "Efton", 1.0 },
  { "Efua", 0.0 },
  { "Egan", 0.9501526 },
  { "Egbert", 1.0 },
  { "Ege", 1.0 },
  { "Eghosa", 1.0 },
  { "Egidio", 1.0 },
  { "Egina", 0.0 },
  { "Egla", 0.0 },
  { "Egon", 1.0 },
  { "Egor", 1.0 },
  { "Egypt", 0.09988942 },
  { "Egzon", 1.0 },
  { "Eh", 0.5796767 },
  { "Ehaan", 1.0 },
  { "Ehab", 1.0 },
  { "Ehan", 1.0 },
  { "Ehitan", 1.0 },
  { "Ehlana", 0.0 },
  { "Ehlena", 0.0 },
  { "Ehlijah", 1.0 },
  { "Ehren", 0.9850877 },
  { "Ehrin", 0.4 },
  { "Ehron", 1.0 },
  { "Ehsan", 1.0 },
  { "Ehtan", 1.0 },
  { "Ehud", 1.0 },
  { "Ehvan", 1.0 },
  { "Eiad", 1.0 },
  { "Eian", 1.0 },
  { "Eibhleann", 0.0 },
  { "Eibhlin", 0.0 },
  { "Eichael", 1.0 },
  { "Eichi", 1.0 },
  { "Eid", 1.0 },
  { "Eidan", 1.0 },
  { "Eidel", 0.0 },
  { "Eiden", 1.0 },
  { "Eider", 0.0 },
  { "Eidy", 0.0 },
  { "Eiichi", 1.0 },
  { "Eijah", 1.0 },
  { "Eiji", 1.0 },
  { "Eiker", 1.0 },
  { "Eiko", 0.0 },
  { "Eila", 0.0 },
  { "Eilah", 0.0 },
  { "Eilam", 1.0 },
  { "Eilan", 1.0 },
  { "Eilani", 0.0 },
  { "Eilany", 0.0 },
  { "Eilean", 0.0 },
  { "Eilee", 0.0 },
  { "Eileen", 0.0021879603 },
  { "Eileena", 0.0 },
  { "Eileene", 0.0 },
  { "Eileigh", 0.0 },
  { "Eilen", 0.0 },
  { "Eilene", 0.0 },
  { "Eiler", 1.0 },
  { "Eilert", 1.0 },
  { "Eiley", 0.0 },
  { "Eiliana", 0.0 },
  { "Eilidh", 0.0 },
  { "Eilijah", 1.0 },
  { "Eilin", 0.0 },
  { "Eilis", 0.0 },
  { "Eilise", 0.0 },
  { "Eilish", 0.0 },
  { "Eiliyah", 0.0 },
  { "Eiljah", 1.0 },
  { "Eilleen", 0.0 },
  { "Eillen", 0.0 },
  { "Eilley", 0.0 },
  { "Eillie", 0.0 },
  { "Eilon", 1.0 },
  { "Eilonwy", 0.0 },
  { "Eily", 0.0 },
  { "Eilyn", 0.0 },
  { "Eilynn", 0.0 },
  { "Eilzabeth", 0.0 },
  { "Eimaan", 0.0 },
  { "Eimaj", 1.0 },
  { "Eiman", 0.0 },
  { "Eimear", 0.0 },
  { "Eimi", 0.0 },
  { "Eimile", 0.0 },
  { "Eimly", 0.0 },
  { "Eimmy", 0.0 },
  { "Eimy", 0.0 },
  { "Ein", 1.0 },
  { "Eina", 0.0 },
  { "Einar", 1.0 },
  { "Einard", 1.0 },
  { "Einav", 0.0 },
  { "Einer", 1.0 },
  { "Einin", 0.0 },
  { "Eino", 1.0 },
  { "Einstein", 1.0 },
  { "Eion", 1.0 },
  { "Eira", 0.0 },
  { "Eirc", 1.0 },
  { "Eire", 0.0 },
  { "Eireann", 0.0 },
  { "Eireen", 0.0 },
  { "Eiren", 1.0 },
  { "Eirene", 0.0 },
  { "Eirian", 0.0 },
  { "Eirik", 1.0 },
  { "Eirini", 0.0 },
  { "Eirinn", 0.0 },
  { "Eirnin", 1.0 },
  { "Eisa", 0.7370518 },
  { "Eisele", 0.0 },
  { "Eisen", 1.0 },
  { "Eisenhower", 1.0 },
  { "Eisha", 0.0 },
  { "Eislee", 0.0 },
  { "Eisleigh", 0.0 },
  { "Eisley", 0.012479201 },
  { "Eissa", 1.0 },
  { "Eita", 1.0 },
  { "Eitan", 1.0 },
  { "Eitel", 1.0 },
  { "Eith", 1.0 },
  { "Eithan", 1.0 },
  { "Eithel", 0.0 },
  { "Eithen", 1.0 },
  { "Eithne", 0.0 },
  { "Eithyn", 1.0 },
  { "Eito", 1.0 },
  { "Eiva", 0.0 },
  { "Eivan", 1.0 },
  { "Eiven", 1.0 },
  { "Eivin", 1.0 },
  { "Eiyanna", 0.0 },
  { "Eiza", 0.0 },
  { "Eizabella", 0.0 },
  { "Eizabeth", 0.0 },
  { "Eizan", 1.0 },
  { "Eizley", 0.0 },
  { "Ej", 1.0 },
  { "Eja", 0.0 },
  { "Ejay", 1.0 },
  { "Ejaz", 1.0 },
  { "Ekaansh", 1.0 },
  { "Ekam", 0.6371951 },
  { "Ekambir", 1.0 },
  { "Ekamjeet", 1.0 },
  { "Ekamjit", 1.0 },
  { "Ekamjot", 0.7755102 },
  { "Ekamveer", 1.0 },
  { "Ekansh", 1.0 },
  { "Ekaterina", 0.0 },
  { "Ekaterini", 0.0 },
  { "Ekco", 0.0 },
  { "Ekene", 0.54285717 },
  { "Eker", 1.0 },
  { "Ekin", 0.95862067 },
  { "Ekko", 0.0 },
  { "Eknoor", 0.10784314 },
  { "Eko", 1.0 },
  { "Ekow", 1.0 },
  { "Ekram", 0.0 },
  { "Ekrem", 1.0 },
  { "Ekta", 0.0 },
  { "El", 0.8637993 },
  { "Ela", 0.0 },
  { "Elaan", 0.5 },
  { "Elad", 1.0 },
  { "Eladia", 0.0 },
  { "Eladio", 1.0 },
  { "Elaena", 0.0 },
  { "Elaf", 0.0 },
  { "Elah", 0.0 },
  { "Elaheh", 0.0 },
  { "Elahna", 0.0 },
  { "Elahni", 0.0 },
  { "Elai", 1.0 },
  { "Elaia", 0.0 },
  { "Elaida", 0.0 },
  { "Elaijah", 0.035460994 },
  { "Elain", 0.0 },
  { "Elaina", 0.0 },
  { "Elainah", 0.0 },
  { "Elaine", 0.002540753 },
  { "Elainea", 0.0 },
  { "Elainey", 0.0 },
  { "Elaini", 0.0 },
  { "Elainie", 0.0 },
  { "Elainna", 0.0 },
  { "Elainy", 0.0 },
  { "Elaisa", 0.0 },
  { "Elaise", 0.0 },
  { "Elaisha", 0.0 },
  { "Elaiya", 0.0 },
  { "Elaiza", 0.0 },
  { "Elajah", 0.0 },
  { "Elajuwon", 1.0 },
  { "Elam", 1.0 },
  { "Elan", 0.79387105 },
  { "Elana", 0.0 },
  { "Elanah", 0.0 },
  { "Eland", 1.0 },
  { "Elanda", 0.0 },
  { "Elander", 0.0 },
  { "Elandra", 0.0 },
  { "Elane", 0.0 },
  { "Elaney", 0.0 },
  { "Elani", 0.0 },
  { "Elania", 0.0 },
  { "Elanie", 0.0 },
  { "Elanna", 0.0 },
  { "Elannah", 0.0 },
  { "Elanni", 0.0 },
  { "Elannie", 0.0 },
  { "Elanny", 0.0 },
  { "Elanor", 0.0 },
  { "Elanora", 0.0 },
  { "Elanore", 0.0 },
  { "Elante", 1.0 },
  { "Elantra", 0.0 },
  { "Elanur", 0.0 },
  { "Elany", 0.0 },
  { "Elara", 0.0 },
  { "Elaria", 0.0 },
  { "Elario", 1.0 },
  { "Elasha", 0.0 },
  { "Elasia", 0.0 },
  { "Elauna", 0.0 },
  { "Elaura", 0.0 },
  { "Elay", 1.0 },
  { "Elaya", 0.0 },
  { "Elayah", 0.0 },
  { "Elayiah", 0.0 },
  { "Elayjah", 0.0 },
  { "Elayla", 0.0 },
  { "Elayna", 0.0 },
  { "Elaynah", 0.0 },
  { "Elayne", 0.0 },
  { "Elaysha", 0.0 },
  { "Elayshia", 0.0 },
  { "Elaysia", 0.0 },
  { "Elazar", 1.0 },
  { "Elazia", 0.0 },
  { "Elba", 0.08632101 },
  { "Elber", 1.0 },
  { "Elbern", 1.0 },
  { "Elbert", 0.9971062 },
  { "Elberta", 0.0 },
  { "Elbertha", 0.0 },
  { "Elbia", 0.0 },
  { "Elbie", 0.94186044 },
  { "Elbin", 1.0 },
  { "Elbira", 0.0 },
  { "Elbis", 1.0 },
  { "Elbonie", 0.0 },
  { "Elbony", 0.0 },
  { "Elbridge", 1.0 },
  { "Elburn", 1.0 },
  { "Elby", 1.0 },
  { "Elchanan", 1.0 },
  { "Elchonon", 1.0 },
  { "Elcid", 1.0 },
  { "Elcie", 0.0 },
  { "Elco", 1.0 },
  { "Elcy", 0.0 },
  { "Elda", 0.0047323946 },
  { "Eldad", 1.0 },
  { "Eldamae", 0.0 },
  { "Eldan", 1.0 },
  { "Eldana", 0.0 },
  { "Eldar", 1.0 },
  { "Eldean", 0.39071038 },
  { "Eldee", 0.54545456 },
  { "Eldeen", 0.0 },
  { "Elden", 0.99802583 },
  { "Eldena", 0.0 },
  { "Eldene", 0.0 },
  { "Elder", 0.973029 },
  { "Elderine", 0.0 },
  { "Eldie", 0.6458333 },
  { "Eldin", 1.0 },
  { "Eldina", 0.0 },
  { "Eldine", 0.0 },
  { "Eldion", 1.0 },
  { "Eldo", 1.0 },
  { "Eldon", 0.9978495 },
  { "Eldona", 0.0 },
  { "Eldonna", 0.0 },
  { "Eldor", 1.0 },
  { "Eldora", 0.0 },
  { "Eldoris", 0.0 },
  { "Eldra", 0.6438356 },
  { "Eldred", 0.92720973 },
  { "Eldredge", 1.0 },
  { "Eldren", 1.0 },
  { "Eldric", 1.0 },
  { "Eldrich", 1.0 },
  { "Eldrick", 1.0 },
  { "Eldrid", 0.8214286 },
  { "Eldridge", 1.0 },
  { "Eldrige", 1.0 },
  { "Eldrin", 1.0 },
  { "Eldwin", 1.0 },
  { "Ele", 0.0 },
  { "Elea", 0.0 },
  { "Eleah", 0.0 },
  { "Elean", 0.3125 },
  { "Eleana", 0.0 },
  { "Eleanar", 0.0 },
  { "Eleane", 0.0 },
  { "Eleaner", 0.0 },
  { "Eleanna", 0.0 },
  { "Eleanor", 0.002164019 },
  { "Eleanora", 0.0 },
  { "Eleanore", 0.0 },
  { "Eleany", 0.0 },
  { "Eleasa", 0.0 },
  { "Eleasah", 0.0 },
  { "Eleasar", 1.0 },
  { "Elease", 0.0 },
  { "Eleasha", 0.0 },
  { "Eleazar", 0.9984845 },
  { "Eleazer", 1.0 },
  { "Elec", 1.0 },
  { "Elecia", 0.0 },
  { "Eleck", 1.0 },
  { "Elecktra", 0.0 },
  { "Electa", 0.0 },
  { "Electra", 0.0 },
  { "Eleda", 0.0 },
  { "Elee", 0.0 },
  { "Eleecia", 0.0 },
  { "Eleen", 0.0 },
  { "Eleena", 0.0 },
  { "Eleene", 0.0 },
  { "Eleesa", 0.0 },
  { "Eleese", 0.0 },
  { "Eleesha", 0.0 },
  { "Eleesia", 0.0 },
  { "Eleeza", 0.0 },
  { "Elefteria", 0.0 },
  { "Elefterios", 1.0 },
  { "Eleftheria", 0.0 },
  { "Eleftherios", 1.0 },
  { "Eleigh", 0.0 },
  { "Eleigha", 0.0 },
  { "Eleina", 0.0 },
  { "Eleisa", 0.0 },
  { "Eleise", 0.0 },
  { "Eleisha", 0.0 },
  { "Elek", 1.0 },
  { "Elektra", 0.0 },
  { "Elemer", 1.0 },
  { "Elen", 0.0 },
  { "Elena", 0.0013248618 },
  { "Elenah", 0.0 },
  { "Elenamarie", 0.0 },
  { "Elend", 1.0 },
  { "Elender", 0.0 },
  { "Elene", 0.0 },
  { "Elener", 0.0 },
  { "Eleni", 0.0 },
  { "Elenia", 0.0 },
  { "Elenie", 0.0 },
  { "Elenis", 0.0 },
  { "Elenita", 0.0 },
  { "Elenna", 0.0 },
  { "Elenny", 0.0 },
  { "Eleno", 1.0 },
  { "Elenoa", 0.0 },
  { "Elenor", 0.0 },
  { "Elenora", 0.0 },
  { "Elenore", 0.0 },
  { "Eleny", 0.0 },
  { "Eleodoro", 1.0 },
  { "Eleonor", 0.0 },
  { "Eleonora", 0.0 },
  { "Eleonore", 0.0 },
  { "Eleora", 0.0 },
  { "Eleri", 0.0 },
  { "Elery", 1.0 },
  { "Eles", 1.0 },
  { "Elesa", 0.0 },
  { "Elese", 0.0 },
  { "Elesha", 0.0 },
  { "Eleshia", 0.0 },
  { "Elesia", 0.0 },
  { "Elessa", 0.0 },
  { "Elester", 0.9627329 },
  { "Elestine", 0.0 },
  { "Elet", 1.0 },
  { "Eleta", 0.0 },
  { "Eletha", 0.0 },
  { "Elethia", 0.0 },
  { "Eletise", 1.0 },
  { "Eleuteria", 0.0 },
  { "Eleuterio", 1.0 },
  { "Elex", 1.0 },
  { "Elexa", 0.0 },
  { "Elexander", 1.0 },
  { "Elexas", 0.0 },
  { "Elexcia", 0.0 },
  { "Elexes", 0.0 },
  { "Elexi", 0.0 },
  { "Elexia", 0.0 },
  { "Elexis", 0.0 },
  { "Elexius", 0.0 },
  { "Elexsis", 0.0 },
  { "Elexus", 0.0 },
  { "Elexys", 0.0 },
  { "Eley", 1.0 },
  { "Eleya", 0.0 },
  { "Eleyah", 0.0 },
  { "Eleyana", 0.0 },
  { "Eleyna", 0.0 },
  { "Elezabeth", 0.0 },
  { "Elfa", 0.0 },
  { "Elfago", 1.0 },
  { "Elfego", 1.0 },
  { "Elfida", 0.0 },
  { "Elfie", 0.0 },
  { "Elford", 1.0 },
  { "Elfred", 1.0 },
  { "Elfreda", 0.0 },
  { "Elfredia", 0.0 },
  { "Elfreida", 0.0 },
  { "Elfrida", 0.0 },
  { "Elfrieda", 0.0 },
  { "Elfriede", 0.0 },
  { "Elga", 0.08 },
  { "Elgan", 1.0 },
  { "Elgar", 1.0 },
  { "Elgene", 0.40677965 },
  { "Elger", 1.0 },
  { "Elgia", 0.32352942 },
  { "Elgie", 0.60610574 },
  { "Elgin", 0.98891467 },
  { "Elgina", 0.0 },
  { "Elhadj", 1.0 },
  { "Elhadji", 1.0 },
  { "Elham", 0.07224335 },
  { "Elhana", 0.0 },
  { "Eli", 0.9935094 },
  { "Elia", 0.109556176 },
  { "Eliab", 1.0 },
  { "Eliabeth", 0.0 },
  { "Eliah", 0.7090576 },
  { "Eliahna", 0.0 },
  { "Eliahs", 1.0 },
  { "Eliahu", 1.0 },
  { "Eliajah", 1.0 },
  { "Eliakim", 1.0 },
  { "Eliam", 1.0 },
  { "Elian", 0.9909353 },
  { "Eliana", 0.00084075483 },
  { "Elianagrace", 0.0 },
  { "Elianah", 0.0 },
  { "Eliandra", 0.0 },
  { "Eliane", 0.0 },
  { "Eliani", 0.0 },
  { "Elianie", 0.0 },
  { "Elianis", 0.0 },
  { "Elianna", 0.0 },
  { "Eliannah", 0.0 },
  { "Elianne", 0.0 },
  { "Elianni", 0.0 },
  { "Eliannie", 0.0 },
  { "Elianny", 0.0 },
  { "Eliano", 1.0 },
  { "Eliany", 0.0 },
  { "Elianys", 0.0 },
  { "Eliara", 0.0 },
  { "Elias", 0.99759614 },
  { "Eliasar", 1.0 },
  { "Eliase", 1.0 },
  { "Eliasib", 1.0 },
  { "Eliason", 1.0 },
  { "Eliasz", 1.0 },
  { "Eliav", 1.0 },
  { "Eliaz", 1.0 },
  { "Eliazar", 1.0 },
  { "Eliazbeth", 0.0 },
  { "Eliazer", 1.0 },
  { "Eliberto", 1.0 },
  { "Elibeth", 0.0 },
  { "Eliborio", 1.0 },
  { "Elic", 1.0 },
  { "Elica", 0.0 },
  { "Elice", 0.028688524 },
  { "Eliceo", 1.0 },
  { "Elicia", 0.0 },
  { "Eliciana", 0.0 },
  { "Elicio", 1.0 },
  { "Elick", 1.0 },
  { "Elida", 0.0 },
  { "Elide", 0.0 },
  { "Elided", 0.0 },
  { "Elidet", 0.0 },
  { "Elideth", 0.0 },
  { "Elidia", 0.0 },
  { "Elidio", 1.0 },
  { "Elie", 0.9068677 },
  { "Elieana", 0.0 },
  { "Eliecer", 1.0 },
  { "Elieen", 0.0 },
  { "Eliejah", 1.0 },
  { "Eliel", 1.0 },
  { "Elien", 0.0 },
  { "Elienai", 0.026086956 },
  { "Eliene", 0.0 },
  { "Elier", 1.0 },
  { "Eliese", 0.0 },
  { "Elieser", 1.0 },
  { "Eliesha", 0.0 },
  { "Eliette", 0.0 },
  { "Eliezar", 1.0 },
  { "Eliezel", 1.0 },
  { "Eliezer", 1.0 },
  { "Elif", 0.0 },
  { "Eliga", 1.0 },
  { "Eligah", 1.0 },
  { "Elige", 1.0 },
  { "Eliger", 1.0 },
  { "Eligh", 1.0 },
  { "Eligha", 1.0 },
  { "Elight", 1.0 },
  { "Eligio", 1.0 },
  { "Elih", 1.0 },
  { "Elihu", 1.0 },
  { "Elihue", 1.0 },
  { "Elii", 1.0 },
  { "Eliijah", 1.0 },
  { "Elija", 1.0 },
  { "Elijaah", 1.0 },
  { "Elijah", 0.9960252 },
  { "Elijaha", 1.0 },
  { "Elijahh", 1.0 },
  { "Elijahjames", 1.0 },
  { "Elijahjohn", 1.0 },
  { "Elijahjuan", 1.0 },
  { "Elijahpaul", 1.0 },
  { "Elijahray", 1.0 },
  { "Elijahwan", 1.0 },
  { "Elijahwon", 1.0 },
  { "Elijames", 1.0 },
  { "Elijan", 1.0 },
  { "Elijawon", 1.0 },
  { "Elijay", 1.0 },
  { "Elijha", 1.0 },
  { "Elijhah", 1.0 },
  { "Elijia", 1.0 },
  { "Elijiah", 1.0 },
  { "Elijio", 1.0 },
  { "Elijsha", 1.0 },
  { "Elijuah", 1.0 },
  { "Elik", 1.0 },
  { "Elika", 0.0 },
  { "Elikai", 1.0 },
  { "Elikem", 1.0 },
  { "Elilah", 0.0 },
  { "Elilta", 0.0 },
  { "Elim", 0.59205776 },
  { "Elimar", 0.0 },
  { "Elimelec", 1.0 },
  { "Elimelech", 1.0 },
  { "Elin", 0.026058631 },
  { "Elina", 0.0 },
  { "Elinah", 0.0 },
  { "Elinam", 0.0 },
  { "Elinar", 0.0 },
  { "Elinda", 0.0 },
  { "Eline", 0.0 },
  { "Eliner", 0.0 },
  { "Elinn", 0.0 },
  { "Elinor", 0.0 },
  { "Elinora", 0.0 },
  { "Elinore", 0.0 },
  { "Elio", 1.0 },
  { "Eliodoro", 1.0 },
  { "Elioenai", 1.0 },
  { "Elion", 1.0 },
  { "Eliona", 0.0 },
  { "Elionaid", 1.0 },
  { "Elionna", 0.0 },
  { "Elior", 1.0 },
  { "Eliora", 0.0 },
  { "Eliorah", 0.0 },
  { "Eliose", 0.0 },
  { "Eliot", 0.94268346 },
  { "Elioth", 1.0 },
  { "Eliott", 1.0 },
  { "Eliotte", 0.0 },
  { "Elira", 0.0 },
  { "Eliran", 1.0 },
  { "Elis", 0.4848485 },
  { "Elisa", 0.003300012 },
  { "Elisabel", 0.0 },
  { "Elisabella", 0.0 },
  { "Elisabet", 0.0 },
  { "Elisabeta", 0.0 },
  { "Elisabeth", 0.00059515936 },
  { "Elisabethe", 0.0 },
  { "Elisabetta", 0.0 },
  { "Elisah", 0.7338129 },
  { "Elisama", 0.0 },
  { "Elisandra", 0.0 },
  { "Elisandro", 1.0 },
  { "Elisaul", 1.0 },
  { "Elisavet", 0.0 },
  { "Elisbeth", 0.0 },
  { "Eliscia", 0.0 },
  { "Elise", 0.00097201864 },
  { "Elisea", 0.0 },
  { "Elisebeth", 0.0 },
  { "Elisee", 1.0 },
  { "Elisei", 1.0 },
  { "Eliseo", 1.0 },
  { "Eliset", 0.0 },
  { "Elisey", 1.0 },
  { "Elish", 1.0 },
  { "Elisha", 0.49215153 },
  { "Elishah", 0.8879668 },
  { "Elishea", 0.0 },
  { "Elisheba", 0.0 },
  { "Elisheva", 0.0 },
  { "Elishia", 0.0 },
  { "Elishka", 0.0 },
  { "Elishua", 1.0 },
  { "Elisia", 0.0 },
  { "Elisiah", 1.0 },
  { "Elisiana", 0.0 },
  { "Elisianna", 0.0 },
  { "Elisio", 1.0 },
  { "Elisjah", 1.0 },
  { "Eliska", 0.0 },
  { "Elison", 0.8196721 },
  { "Elissa", 0.0 },
  { "Elissabeth", 0.0 },
  { "Elissar", 0.0 },
  { "Elisse", 0.0 },
  { "Elissha", 0.0 },
  { "Elissia", 0.0 },
  { "Elisya", 0.0 },
  { "Elita", 0.0 },
  { "Elitania", 0.0 },
  { "Elite", 1.0 },
  { "Elitha", 0.0 },
  { "Elithia", 0.0 },
  { "Eliu", 1.0 },
  { "Eliud", 1.0 },
  { "Elius", 1.0 },
  { "Eliut", 1.0 },
  { "Eliuth", 1.0 },
  { "Eliverto", 1.0 },
  { "Elivia", 0.0 },
  { "Elix", 1.0 },
  { "Elixabeth", 0.0 },
  { "Elixander", 1.0 },
  { "Elixis", 0.0 },
  { "Eliya", 0.13915858 },
  { "Eliyah", 0.3534884 },
  { "Eliyahna", 0.0 },
  { "Eliyahu", 1.0 },
  { "Eliyana", 0.0 },
  { "Eliyanah", 0.0 },
  { "Eliyanna", 0.0 },
  { "Eliyas", 1.0 },
  { "Eliyjah", 1.0 },
  { "Eliyohu", 1.0 },
  { "Eliz", 0.0 },
  { "Eliza", 0.0036040752 },
  { "Elizabath", 0.0 },
  { "Elizabe", 0.0 },
  { "Elizabeath", 0.0 },
  { "Elizabeht", 0.0 },
  { "Elizabel", 0.0 },
  { "Elizabella", 0.0 },
  { "Elizabelle", 0.0 },
  { "Elizaberh", 0.0 },
  { "Elizabet", 0.0 },
  { "Elizabeta", 0.0 },
  { "Elizabete", 0.0 },
  { "Elizabeth", 0.0031812363 },
  { "Elizabetha", 0.0 },
  { "Elizabethann", 0.0 },
  { "Elizabethanne", 0.0 },
  { "Elizabethe", 0.0 },
  { "Elizabethgrace", 0.0 },
  { "Elizabethmarie", 0.0 },
  { "Elizabethrose", 0.0 },
  { "Elizabith", 0.0 },
  { "Elizabth", 0.0 },
  { "Elizadeth", 0.0 },
  { "Elizaeth", 0.0 },
  { "Elizah", 0.54031414 },
  { "Elizaida", 0.0 },
  { "Elizajane", 0.0 },
  { "Elizama", 0.0 },
  { "Elizamarie", 0.0 },
  { "Elizander", 1.0 },
  { "Elizandra", 0.0 },
  { "Elizandro", 1.0 },
  { "Elizar", 1.0 },
  { "Elizardo", 1.0 },
  { "Elizaveta", 0.0 },
  { "Elizbeath", 0.0 },
  { "Elizbeth", 0.0 },
  { "Elize", 0.055944055 },
  { "Elizeabeth", 0.0 },
  { "Elizebath", 0.0 },
  { "Elizebeth", 0.0 },
  { "Elizebth", 0.0 },
  { "Elizeo", 1.0 },
  { "Elizer", 1.0 },
  { "Elizet", 0.0 },
  { "Elizeth", 0.0 },
  { "Elizette", 0.0 },
  { "Elizha", 1.0 },
  { "Elizia", 0.0 },
  { "Eliziah", 1.0 },
  { "Elizibeth", 0.0 },
  { "Elizjah", 1.0 },
  { "Elizza", 0.0 },
  { "Elizzabeth", 0.0 },
  { "Elizzie", 0.0 },
  { "Eljay", 1.0 },
  { "Eljean", 0.0 },
  { "Eljin", 1.0 },
  { "Elka", 0.0 },
  { "Elkan", 1.0 },
  { "Elkanah", 1.0 },
  { "Elke", 0.0 },
  { "Elkin", 1.0 },
  { "Elkins", 1.0 },
  { "Elky", 0.0 },
  { "Ell", 0.9350649 },
  { "Ella", 0.0030046105 },
  { "Ellabelle", 0.0 },
  { "Ellaclaire", 0.0 },
  { "Elladean", 0.0 },
  { "Ellagrace", 0.0 },
  { "Ellah", 0.0 },
  { "Ellaina", 0.0 },
  { "Ellaine", 0.0 },
  { "Ellajane", 0.0 },
  { "Ellajean", 0.0 },
  { "Ellajo", 0.0 },
  { "Ellakate", 0.0 },
  { "Ellalee", 0.0 },
  { "Ellalynn", 0.0 },
  { "Ellamae", 0.0 },
  { "Ellamaria", 0.0 },
  { "Ellamarie", 0.0 },
  { "Ellamay", 0.0 },
  { "Ellan", 0.0 },
  { "Ellana", 0.0 },
  { "Ellanah", 0.0 },
  { "Ellani", 0.0 },
  { "Ellanie", 0.0 },
  { "Ellanor", 0.0 },
  { "Ellanora", 0.0 },
  { "Ellanore", 0.0 },
  { "Ellany", 0.0 },
  { "Ellar", 0.0 },
  { "Ellarae", 0.0 },
  { "Ellard", 1.0 },
  { "Ellaree", 0.0 },
  { "Ellarene", 0.0 },
  { "Ellaria", 0.0 },
  { "Ellarie", 0.0 },
  { "Ellarine", 0.0 },
  { "Ellarose", 0.0 },
  { "Ellary", 0.0 },
  { "Ellasandra", 0.0 },
  { "Ellasia", 0.0 },
  { "Ellason", 0.0 },
  { "Ellasophia", 0.0 },
  { "Ellasyn", 0.0 },
  { "Ellawease", 0.0 },
  { "Ellawee", 0.0 },
  { "Ellawese", 0.0 },
  { "Ellawyn", 0.0 },
  { "Ellaya", 0.0 },
  { "Ellayah", 0.0 },
  { "Ellayna", 0.0 },
  { "Elle", 0.0 },
  { "Ellea", 0.0 },
  { "Elleah", 0.0 },
  { "Ellean", 0.0 },
  { "Elleana", 0.0 },
  { "Elleanna", 0.0 },
  { "Elleanor", 0.0 },
  { "Elleanora", 0.0 },
  { "Ellease", 0.0 },
  { "Ellece", 0.0 },
  { "Ellee", 0.0 },
  { "Elleen", 0.0 },
  { "Elleigh", 0.0 },
  { "Ellen", 0.0027223132 },
  { "Ellena", 0.0 },
  { "Ellenah", 0.0 },
  { "Ellender", 0.0 },
  { "Ellene", 0.0 },
  { "Elleni", 0.0 },
  { "Ellenie", 0.0 },
  { "Ellenmarie", 0.0 },
  { "Ellenor", 0.0 },
  { "Ellenora", 0.0 },
  { "Ellenore", 0.0 },
  { "Elleny", 0.0 },
  { "Elleona", 0.0 },
  { "Elleora", 0.0 },
  { "Eller", 0.0 },
  { "Elleri", 0.0 },
  { "Ellerie", 0.0 },
  { "Ellery", 0.42141053 },
  { "Elles", 1.0 },
  { "Ellese", 0.0 },
  { "Ellesha", 0.0 },
  { "Elleson", 0.0 },
  { "Ellesse", 0.0 },
  { "Ellesyn", 0.0 },
  { "Ellet", 1.0 },
  { "Ellette", 0.0 },
  { "Ellexa", 0.0 },
  { "Ellexis", 0.0 },
  { "Ellexus", 0.0 },
  { "Elley", 0.0 },
  { "Elli", 0.013272312 },
  { "Ellia", 0.0 },
  { "Elliah", 0.0 },
  { "Elliahna", 0.0 },
  { "Ellian", 1.0 },
  { "Elliana", 0.0 },
  { "Ellianah", 0.0 },
  { "Elliani", 0.0 },
  { "Elliann", 0.0 },
  { "Ellianna", 0.0 },
  { "Elliannah", 0.0 },
  { "Ellianne", 0.0 },
  { "Ellias", 1.0 },
  { "Elliauna", 0.0 },
  { "Ellice", 0.0 },
  { "Ellicia", 0.0 },
  { "Ellie", 0.020339536 },
  { "Ellieana", 0.0 },
  { "Ellieann", 0.0 },
  { "Ellieanna", 0.0 },
  { "Ellieanne", 0.0 },
  { "Elliegrace", 0.0 },
  { "Elliekate", 0.0 },
  { "Elliemae", 0.0 },
  { "Elliemarie", 0.0 },
  { "Elliemay", 0.0 },
  { "Elliena", 0.0 },
  { "Ellierose", 0.0 },
  { "Elliet", 0.0 },
  { "Elliett", 0.0 },
  { "Elliette", 0.0 },
  { "Ellijah", 1.0 },
  { "Ellika", 0.0 },
  { "Ellin", 0.0 },
  { "Ellina", 0.0 },
  { "Elline", 0.0 },
  { "Ellington", 0.48926014 },
  { "Ellinor", 0.0 },
  { "Ellinore", 0.0 },
  { "Elliona", 0.0 },
  { "Ellionna", 0.0 },
  { "Elliora", 0.0 },
  { "Elliot", 0.9150583 },
  { "Elliott", 0.9280352 },
  { "Elliotte", 0.10352423 },
  { "Ellis", 0.9176578 },
  { "Ellisa", 0.0 },
  { "Ellise", 0.0 },
  { "Ellisen", 0.0 },
  { "Ellisha", 0.0 },
  { "Ellisia", 0.0 },
  { "Ellison", 0.39742646 },
  { "Ellissa", 0.0 },
  { "Ellissia", 0.0 },
  { "Elliston", 0.53846157 },
  { "Ellisyn", 0.0 },
  { "Ellivia", 0.0 },
  { "Elliya", 0.0 },
  { "Elliyah", 0.0 },
  { "Elliyana", 0.0 },
  { "Elliyanna", 0.0 },
  { "Elliza", 0.0 },
  { "Ellizabeth", 0.0 },
  { "Ellnora", 0.0 },
  { "Ellodie", 0.0 },
  { "Ellody", 0.0 },
  { "Elloise", 0.0 },
  { "Ellon", 0.0 },
  { "Ellora", 0.0 },
  { "Elloree", 0.0 },
  { "Ellorie", 0.0 },
  { "Ellory", 0.0 },
  { "Ellouise", 0.0 },
  { "Elloween", 0.0 },
  { "Ellowyn", 0.0 },
  { "Ellsa", 0.0 },
  { "Ellsie", 0.0 },
  { "Ellsworth", 1.0 },
  { "Ellwood", 1.0 },
  { "Ellwyn", 1.0 },
  { "Elly", 0.0029291154 },
  { "Ellya", 0.0 },
  { "Ellyana", 0.0 },
  { "Ellyanah", 0.0 },
  { "Ellyanna", 0.0 },
  { "Ellyce", 0.0 },
  { "Ellycia", 0.0 },
  { "Ellye", 0.0 },
  { "Ellyette", 0.0 },
  { "Ellymae", 0.0 },
  { "Ellyn", 0.0 },
  { "Ellyne", 0.0 },
  { "Ellynn", 0.0 },
  { "Ellynor", 0.0 },
  { "Ellyona", 0.0 },
  { "Ellyonna", 0.0 },
  { "Ellyot", 0.0 },
  { "Ellyott", 0.0 },
  { "Ellys", 0.0 },
  { "Ellysa", 0.0 },
  { "Ellyse", 0.0 },
  { "Ellysen", 0.0 },
  { "Ellysia", 0.0 },
  { "Ellyson", 0.0 },
  { "Ellyssa", 0.0 },
  { "Ellyza", 0.0 },
  { "Elma", 0.022034671 },
  { "Elmae", 0.0 },
  { "Elman", 1.0 },
  { "Elmar", 0.93485343 },
  { "Elmarie", 0.0 },
  { "Elmeda", 0.0 },
  { "Elmedin", 1.0 },
  { "Elmedina", 0.0 },
  { "Elmen", 1.0 },
  { "Elmer", 0.9864971 },
  { "Elmerine", 0.0 },
  { "Elmeta", 0.0 },
  { "Elmi", 0.6666667 },
  { "Elmin", 1.0 },
  { "Elmina", 0.0 },
  { "Elmir", 1.0 },
  { "Elmira", 0.0 },
  { "Elmire", 0.0 },
  { "Elmo", 0.9760798 },
  { "Elmon", 1.0 },
  { "Elmond", 1.0 },
  { "Elmont", 1.0 },
  { "Elmor", 1.0 },
  { "Elmore", 0.99747604 },
  { "Elmus", 1.0 },
  { "Elmyra", 0.0 },
  { "Elna", 0.0 },
  { "Elnara", 0.0 },
  { "Elnatan", 1.0 },
  { "Elnathan", 1.0 },
  { "Elnaz", 0.0 },
  { "Elneda", 0.0 },
  { "Elner", 0.012562814 },
  { "Elneta", 0.0 },
  { "Elnita", 0.0 },
  { "Elno", 1.0 },
  { "Elnor", 0.0 },
  { "Elnora", 0.0012740657 },
  { "Elnorah", 0.0 },
  { "Elnore", 0.0 },
  { "Elnoria", 0.0 },
  { "Elo", 1.0 },
  { "Eloda", 0.0 },
  { "Elodee", 0.0 },
  { "Elodia", 0.0 },
  { "Elodie", 0.0 },
  { "Elody", 0.0 },
  { "Elof", 1.0 },
  { "Elohim", 1.0 },
  { "Eloi", 1.0 },
  { "Eloida", 0.0 },
  { "Eloina", 0.0 },
  { "Eloine", 0.0 },
  { "Elois", 0.0037721614 },
  { "Eloisa", 0.0 },
  { "Eloise", 0.0022440562 },
  { "Eloiza", 0.0 },
  { "Elom", 1.0 },
  { "Elon", 0.75986195 },
  { "Elona", 0.0 },
  { "Elonda", 0.0 },
  { "Elondra", 0.0 },
  { "Eloni", 0.0 },
  { "Elonie", 0.0 },
  { "Elonna", 0.0 },
  { "Elonte", 1.0 },
  { "Elonzo", 1.0 },
  { "Elora", 0.0 },
  { "Elorah", 0.0 },
  { "Eloria", 0.0 },
  { "Eloris", 0.0 },
  { "Elosia", 0.0 },
  { "Elosie", 0.0 },
  { "Elouise", 0.0005304477 },
  { "Elovie", 0.0 },
  { "Elowen", 0.0 },
  { "Elowese", 0.0 },
  { "Elowise", 0.0 },
  { "Elowyn", 0.0 },
  { "Elowynn", 0.0 },
  { "Eloy", 1.0 },
  { "Eloyce", 0.0 },
  { "Eloyd", 1.0 },
  { "Eloyse", 0.0 },
  { "Elpha", 0.0 },
  { "Elphege", 1.0 },
  { "Elphie", 0.0 },
  { "Elpida", 0.0 },
  { "Elpidia", 0.0 },
  { "Elpidio", 1.0 },
  { "Elra", 1.0 },
  { "Elray", 1.0 },
  { "Elree", 0.7083333 },
  { "Elrena", 0.0 },
  { "Elrene", 0.0 },
  { "Elreta", 0.0 },
  { "Elrey", 1.0 },
  { "Elric", 1.0 },
  { "Elrick", 1.0 },
  { "Elrico", 1.0 },
  { "Elridge", 1.0 },
  { "Elrita", 0.0 },
  { "Elrod", 1.0 },
  { "Elroi", 1.0 },
  { "Elron", 1.0 },
  { "Elrose", 0.0 },
  { "Elroy", 0.99917513 },
  { "Elry", 1.0 },
  { "Elsa", 0.00074807636 },
  { "Elsabeth", 0.0 },
  { "Elsah", 0.0 },
  { "Elsbeth", 0.0 },
  { "Else", 0.0 },
  { "Elsee", 0.0 },
  { "Elsey", 0.0 },
  { "Elsha", 0.0 },
  { "Elshaddai", 0.42857143 },
  { "Elsi", 0.0 },
  { "Elsia", 0.0 },
  { "Elsie", 0.005004118 },
  { "Elsiemae", 0.0 },
  { "Elsmer", 1.0 },
  { "Elso", 1.0 },
  { "Elson", 1.0 },
  { "Elspeth", 0.0 },
  { "Elssa", 0.0 },
  { "Elster", 1.0 },
  { "Elston", 0.9952875 },
  { "Elsworth", 1.0 },
  { "Elsy", 0.0 },
  { "Elsye", 0.0 },
  { "Elta", 0.0 },
  { "Eltha", 0.0 },
  { "Elthea", 0.0 },
  { "Eltis", 0.0 },
  { "Elton", 0.9909849 },
  { "Eltra", 0.0 },
  { "Elua", 0.0 },
  { "Eluid", 1.0 },
  { "Eluster", 1.0 },
  { "Eluteria", 0.0 },
  { "Eluterio", 1.0 },
  { "Eluzer", 1.0 },
  { "Elva", 0.025408687 },
  { "Elvada", 0.0 },
  { "Elvan", 1.0 },
  { "Elve", 0.36363637 },
  { "Elveda", 0.0 },
  { "Elvedin", 1.0 },
  { "Elven", 1.0 },
  { "Elvena", 0.0 },
  { "Elvenia", 0.0 },
  { "Elver", 0.8501529 },
  { "Elvera", 0.0 },
  { "Elverda", 0.0 },
  { "Elveria", 0.0 },
  { "Elvern", 1.0 },
  { "Elverna", 0.0 },
  { "Elvert", 1.0 },
  { "Elverta", 0.0 },
  { "Elves", 1.0 },
  { "Elvester", 1.0 },
  { "Elveta", 0.0 },
  { "Elvi", 0.13445379 },
  { "Elvia", 0.0041897553 },
  { "Elvida", 0.0 },
  { "Elvie", 0.22473651 },
  { "Elvin", 0.98840266 },
  { "Elvina", 0.0 },
  { "Elvine", 0.0 },
  { "Elvio", 1.0 },
  { "Elvir", 1.0 },
  { "Elvira", 0.001653141 },
  { "Elvire", 0.0 },
  { "Elviria", 0.0 },
  { "Elvis", 0.99092865 },
  { "Elvita", 0.0 },
  { "Elviz", 1.0 },
  { "Elvon", 1.0 },
  { "Elvy", 0.3986014 },
  { "Elvyn", 0.88297874 },
  { "Elwanda", 0.0 },
  { "Elward", 1.0 },
  { "Elway", 1.0 },
  { "Elwell", 1.0 },
  { "Elwillie", 0.0 },
  { "Elwin", 0.9990105 },
  { "Elwood", 0.9974006 },
  { "Elworth", 1.0 },
  { "Elwyn", 0.97623515 },
  { "Ely", 0.9102359 },
  { "Elya", 0.56763923 },
  { "Elyah", 0.5 },
  { "Elyahna", 0.0 },
  { "Elyan", 1.0 },
  { "Elyana", 0.0 },
  { "Elyanah", 0.0 },
  { "Elyane", 0.0 },
  { "Elyanna", 0.0 },
  { "Elyannah", 0.0 },
  { "Elyas", 1.0 },
  { "Elyce", 0.0 },
  { "Elycia", 0.0 },
  { "Elyda", 0.0 },
  { "Elydia", 0.0 },
  { "Elye", 1.0 },
  { "Elyes", 1.0 },
  { "Elyh", 1.0 },
  { "Elyijah", 1.0 },
  { "Elyis", 1.0 },
  { "Elyja", 1.0 },
  { "Elyjah", 1.0 },
  { "Elyjiah", 1.0 },
  { "Elyn", 0.024017468 },
  { "Elyna", 0.0 },
  { "Elyne", 0.0 },
  { "Elynn", 0.0 },
  { "Elynna", 0.0 },
  { "Elynne", 0.0 },
  { "Elynor", 0.0 },
  { "Elynore", 0.0 },
  { "Elyon", 1.0 },
  { "Elyott", 1.0 },
  { "Elysa", 0.0 },
  { "Elysabeth", 0.0 },
  { "Elyse", 0.0010578475 },
  { "Elysha", 0.037410073 },
  { "Elyshia", 0.0 },
  { "Elysia", 0.0019471488 },
  { "Elysian", 0.0 },
  { "Elysiana", 0.0 },
  { "Elysium", 0.09615385 },
  { "Elyssa", 0.0 },
  { "Elysse", 0.0 },
  { "Elyssia", 0.0 },
  { "Elyus", 1.0 },
  { "Elyvia", 0.0 },
  { "Elyza", 0.0 },
  { "Elyzabeth", 0.0 },
  { "Elyzah", 0.0 },
  { "Elyze", 0.0 },
  { "Elyzza", 0.0 },
  { "Elza", 0.74699223 },
  { "Elzabeth", 0.0 },
  { "Elzada", 0.0 },
  { "Elzadie", 0.0 },
  { "Elzbieta", 0.0 },
  { "Elze", 1.0 },
  { "Elzear", 1.0 },
  { "Elzena", 0.0 },
  { "Elzenia", 0.0 },
  { "Elzia", 0.88304096 },
  { "Elziabeth", 0.0 },
  { "Elzie", 0.8658097 },
  { "Elzina", 0.0 },
  { "Elzira", 0.0 },
  { "Elzo", 1.0 },
  { "Elzora", 0.0 },
  { "Elzy", 1.0 },
  { "Ema", 0.0 },
  { "Emaad", 1.0 },
  { "Emaan", 0.033415843 },
  { "Emad", 1.0 },
  { "Emagene", 0.0 },
  { "Emagin", 0.0 },
  { "Emagine", 0.0 },
  { "Emah", 0.0 },
  { "Emahni", 0.0 },
  { "Emaje", 1.0 },
  { "Emajean", 0.0 },
  { "Emalea", 0.0 },
  { "Emaleah", 0.0 },
  { "Emalee", 0.0 },
  { "Emalei", 0.0 },
  { "Emaleigh", 0.0 },
  { "Emalena", 0.0 },
  { "Emalene", 0.0 },
  { "Emaley", 0.0 },
  { "Emali", 0.0 },
  { "Emalia", 0.0 },
  { "Emaliah", 0.0 },
  { "Emalie", 0.0 },
  { "Emalin", 0.0 },
  { "Emalina", 0.0 },
  { "Emaline", 0.0 },
  { "Emalise", 0.0 },
  { "Emaliya", 0.0 },
  { "Emaly", 0.0 },
  { "Emalyn", 0.0 },
  { "Emalyne", 0.0 },
  { "Emalynn", 0.0 },
  { "Emalynne", 0.0 },
  { "Eman", 0.09081365 },
  { "Emanda", 0.0 },
  { "Emane", 0.0 },
  { "Emanee", 0.0 },
  { "Emani", 0.06333404 },
  { "Emanie", 0.0 },
  { "Emanii", 0.0 },
  { "Emanni", 0.0 },
  { "Emannuel", 1.0 },
  { "Emanual", 1.0 },
  { "Emanuel", 0.99809396 },
  { "Emanuela", 0.0 },
  { "Emanuele", 1.0 },
  { "Emanuell", 1.0 },
  { "Emanuella", 0.0 },
  { "Emanuelle", 0.5552408 },
  { "Emanuelly", 0.0 },
  { "Emar", 1.0 },
  { "Emara", 0.0 },
  { "Emare", 1.0 },
  { "Emaree", 0.0 },
  { "Emareon", 1.0 },
  { "Emari", 0.45892736 },
  { "Emaria", 0.0 },
  { "Emariah", 0.0 },
  { "Emarie", 0.0 },
  { "Emarii", 1.0 },
  { "Emarion", 1.0 },
  { "Emariya", 0.0 },
  { "Emarosa", 0.0 },
  { "Emarri", 1.0 },
  { "Emary", 0.11016949 },
  { "Emauni", 0.0 },
  { "Emauri", 0.8067227 },
  { "Emaya", 0.0 },
  { "Emayah", 0.0 },
  { "Ember", 0.01370224 },
  { "Emberlee", 0.0 },
  { "Emberleigh", 0.0 },
  { "Emberley", 0.0 },
  { "Emberli", 0.0 },
  { "Emberlie", 0.0 },
  { "Emberlin", 0.0 },
  { "Emberly", 0.0 },
  { "Emberlyn", 0.0 },
  { "Emberlynn", 0.0 },
  { "Emberlynne", 0.0 },
  { "Embert", 1.0 },
  { "Embra", 0.0 },
  { "Embree", 0.042918455 },
  { "Embrey", 0.0 },
  { "Embri", 0.0 },
  { "Embrie", 0.0 },
  { "Embry", 0.47683108 },
  { "Embrya", 0.0 },
  { "Embyr", 0.0 },
  { "Eme", 0.0 },
  { "Emeal", 1.0 },
  { "Emedio", 1.0 },
  { "Emee", 0.0 },
  { "Emeilia", 0.0 },
  { "Emeka", 0.96028036 },
  { "Emel", 0.3529412 },
  { "Emelda", 0.0 },
  { "Emele", 0.0 },
  { "Emelee", 0.0 },
  { "Emelene", 0.0 },
  { "Emeli", 0.0 },
  { "Emelia", 0.0 },
  { "Emeliah", 0.0 },
  { "Emeliana", 0.0 },
  { "Emelie", 0.0 },
  { "Emelin", 0.0 },
  { "Emelina", 0.0 },
  { "Emelinda", 0.0 },
  { "Emeline", 0.0 },
  { "Emelio", 1.0 },
  { "Emelita", 0.0 },
  { "Emellia", 0.0 },
  { "Emellie", 0.0 },
  { "Emelly", 0.0 },
  { "Emely", 0.00089721306 },
  { "Emelya", 0.0 },
  { "Emelye", 0.0 },
  { "Emelyn", 0.0 },
  { "Emelyne", 0.0 },
  { "Emelynn", 0.0 },
  { "Emem", 0.0 },
  { "Emer", 0.24509804 },
  { "Emera", 0.0 },
  { "Emeral", 0.15873016 },
  { "Emerald", 0.052300658 },
  { "Emeralda", 0.0 },
  { "Emeraude", 0.0 },
  { "Emercyn", 0.0 },
  { "Emere", 1.0 },
  { "Emeree", 0.0 },
  { "Emerey", 0.0 },
  { "Emeri", 0.0442035 },
  { "Emeria", 0.0 },
  { "Emeric", 1.0 },
  { "Emerich", 1.0 },
  { "Emerick", 1.0 },
  { "Emerie", 0.0041067763 },
  { "Emerii", 0.0 },
  { "Emerik", 1.0 },
  { "Emeril", 0.79746836 },
  { "Emerine", 0.0 },
  { "Emeris", 1.0 },
  { "Emerita", 0.0 },
  { "Emerito", 1.0 },
  { "Emerlee", 0.0 },
  { "Emerleigh", 0.0 },
  { "Emerline", 0.0 },
  { "Emerly", 0.0 },
  { "Emerlyn", 0.0 },
  { "Emerlynn", 0.0 },
  { "Emersen", 0.07038513 },
  { "Emerson", 0.5633216 },
  { "Emersyn", 0.014120667 },
  { "Emersynn", 0.0 },
  { "Emert", 1.0 },
  { "Emery", 0.5116865 },
  { "Emeryk", 1.0 },
  { "Emeryn", 0.0 },
  { "Emerzon", 1.0 },
  { "Emese", 0.0 },
  { "Emet", 1.0 },
  { "Emeterio", 1.0 },
  { "Emeth", 1.0 },
  { "Emett", 1.0 },
  { "Emi", 0.0 },
  { "Emia", 0.0 },
  { "Emiah", 0.0 },
  { "Emidio", 1.0 },
  { "Emie", 0.0 },
  { "Emiel", 1.0 },
  { "Emier", 1.0 },
  { "Emigdio", 1.0 },
  { "Emijah", 0.0 },
  { "Emika", 0.0 },
  { "Emiko", 0.0036845985 },
  { "Emil", 0.9957862 },
  { "Emila", 0.0 },
  { "Emilce", 0.0 },
  { "Emilda", 0.0 },
  { "Emile", 0.9258531 },
  { "Emilea", 0.0 },
  { "Emileah", 0.0 },
  { "Emilee", 0.00036199094 },
  { "Emileigh", 0.0 },
  { "Emilene", 0.0 },
  { "Emileo", 1.0 },
  { "Emiley", 0.0 },
  { "Emili", 0.0 },
  { "Emilia", 0.0 },
  { "Emiliah", 0.0 },
  { "Emilian", 1.0 },
  { "Emiliana", 0.0 },
  { "Emiliann", 0.0 },
  { "Emilianna", 0.0 },
  { "Emilianne", 0.0 },
  { "Emiliano", 0.99975055 },
  { "Emilie", 0.00028218978 },
  { "Emiliee", 0.0 },
  { "Emilien", 1.0 },
  { "Emilienne", 0.0 },
  { "Emilija", 0.0 },
  { "Emilin", 0.0 },
  { "Emilina", 0.0 },
  { "Emiline", 0.0 },
  { "Emilio", 0.99847746 },
  { "Emiliya", 0.0 },
  { "Emille", 0.0 },
  { "Emillee", 0.0 },
  { "Emillia", 0.0 },
  { "Emilliana", 0.0 },
  { "Emillianna", 0.0 },
  { "Emilliano", 1.0 },
  { "Emillie", 0.0 },
  { "Emillio", 1.0 },
  { "Emilly", 0.0 },
  { "Emilo", 1.0 },
  { "Emilse", 0.0 },
  { "Emilson", 1.0 },
  { "Emily", 0.002086638 },
  { "Emilya", 0.0 },
  { "Emilyah", 0.0 },
  { "Emilyann", 0.0 },
  { "Emilyanne", 0.0 },
  { "Emilye", 0.0 },
  { "Emilyelizabeth", 0.0 },
  { "Emilygrace", 0.0 },
  { "Emilyjane", 0.0 },
  { "Emilyjo", 0.0 },
  { "Emilymae", 0.0 },
  { "Emilymarie", 0.0 },
  { "Emilyn", 0.0 },
  { "Emilyna", 0.0 },
  { "Emilyne", 0.0 },
  { "Emilynn", 0.0 },
  { "Emilyrose", 0.0 },
  { "Emin", 1.0 },
  { "Emina", 0.0 },
  { "Emine", 0.0 },
  { "Emir", 1.0 },
  { "Emira", 0.0 },
  { "Emirah", 0.0 },
  { "Emiree", 0.0 },
  { "Emireth", 0.0 },
  { "Emirhan", 1.0 },
  { "Emiri", 0.0 },
  { "Emirra", 0.0 },
  { "Emiry", 0.0 },
  { "Emisha", 0.0 },
  { "Emit", 1.0 },
  { "Emiterio", 1.0 },
  { "Emitt", 1.0 },
  { "Emitte", 1.0 },
  { "Emiya", 0.0 },
  { "Emiyah", 0.0 },
  { "Emjay", 0.43076923 },
  { "Emlee", 0.0 },
  { "Emley", 0.0 },
  { "Emly", 0.0 },
  { "Emlyn", 0.016778523 },
  { "Emlynn", 0.0 },
  { "Emma", 0.0025216853 },
  { "Emmabella", 0.0 },
  { "Emmabelle", 0.0 },
  { "Emmabeth", 0.0 },
  { "Emmaclaire", 0.0 },
  { "Emmadell", 0.0 },
  { "Emmagene", 0.0 },
  { "Emmagrace", 0.0 },
  { "Emmah", 0.0 },
  { "Emmajane", 0.0 },
  { "Emmajean", 0.0 },
  { "Emmajo", 0.0 },
  { "Emmakate", 0.0 },
  { "Emmakay", 0.0 },
  { "Emmalani", 0.0 },
  { "Emmalea", 0.0 },
  { "Emmaleah", 0.0 },
  { "Emmalee", 0.0 },
  { "Emmaleen", 0.0 },
  { "Emmalei", 0.0 },
  { "Emmaleigh", 0.0 },
  { "Emmalena", 0.0 },
  { "Emmalene", 0.0 },
  { "Emmaley", 0.0 },
  { "Emmali", 0.0 },
  { "Emmalia", 0.0 },
  { "Emmalie", 0.0 },
  { "Emmalin", 0.0 },
  { "Emmalina", 0.0 },
  { "Emmaline", 0.0 },
  { "Emmalinn", 0.0 },
  { "Emmalisa", 0.0 },
  { "Emmalise", 0.0 },
  { "Emmalou", 0.0 },
  { "Emmalouise", 0.0 },
  { "Emmaly", 0.0 },
  { "Emmalyn", 0.0 },
  { "Emmalyne", 0.0 },
  { "Emmalynn", 0.0 },
  { "Emmalynne", 0.0 },
  { "Emmalyse", 0.0 },
  { "Emmamae", 0.0 },
  { "Emmamarie", 0.0 },
  { "Emman", 0.0 },
  { "Emmanel", 1.0 },
  { "Emmaneul", 1.0 },
  { "Emmani", 0.0 },
  { "Emmanouel", 1.0 },
  { "Emmanual", 1.0 },
  { "Emmanuel", 0.99507713 },
  { "Emmanuela", 0.0 },
  { "Emmanuell", 1.0 },
  { "Emmanuella", 0.0 },
  { "Emmanuelle", 0.20060106 },
  { "Emmanul", 1.0 },
  { "Emmarae", 0.0 },
  { "Emmaree", 0.0 },
  { "Emmari", 0.0 },
  { "Emmariah", 0.0 },
  { "Emmarie", 0.0 },
  { "Emmarose", 0.0 },
  { "Emmary", 0.0 },
  { "Emmasofia", 0.0 },
  { "Emmasophia", 0.0 },
  { "Emmasue", 0.0 },
  { "Emmauel", 1.0 },
  { "Emmaus", 0.939759 },
  { "Emme", 0.0 },
  { "Emmee", 0.0 },
  { "Emmelee", 0.0 },
  { "Emmelene", 0.0 },
  { "Emmelia", 0.0 },
  { "Emmelie", 0.0 },
  { "Emmelin", 0.0 },
  { "Emmelina", 0.0 },
  { "Emmeline", 0.0 },
  { "Emmely", 0.0 },
  { "Emmelyn", 0.0 },
  { "Emmelyne", 0.0 },
  { "Emmelynn", 0.0 },
  { "Emmer", 0.0050784857 },
  { "Emmeri", 0.0 },
  { "Emmeric", 1.0 },
  { "Emmerich", 1.0 },
  { "Emmerick", 1.0 },
  { "Emmerie", 0.0 },
  { "Emmerly", 0.0 },
  { "Emmersen", 0.0 },
  { "Emmerson", 0.18568665 },
  { "Emmersyn", 0.0 },
  { "Emmert", 1.0 },
  { "Emmery", 0.03732504 },
  { "Emmeryn", 0.0 },
  { "Emmet", 1.0 },
  { "Emmett", 0.99737084 },
  { "Emmette", 1.0 },
  { "Emmey", 0.0 },
  { "Emmi", 0.0 },
  { "Emmie", 0.0 },
  { "Emmielou", 0.0 },
  { "Emmilee", 0.0 },
  { "Emmilia", 0.0 },
  { "Emmiliano", 1.0 },
  { "Emmilie", 0.0 },
  { "Emmiline", 0.0 },
  { "Emmilou", 0.0 },
  { "Emmily", 0.0 },
  { "Emmilyn", 0.0 },
  { "Emmilynn", 0.0 },
  { "Emmit", 1.0 },
  { "Emmitt", 1.0 },
  { "Emmitte", 1.0 },
  { "Emmo", 0.0 },
  { "Emmogene", 0.0 },
  { "Emmon", 1.0 },
  { "Emmoni", 0.0 },
  { "Emmons", 1.0 },
  { "Emmori", 0.0 },
  { "Emmorie", 0.0 },
  { "Emmory", 0.048543688 },
  { "Emmrie", 0.0 },
  { "Emmry", 0.0 },
  { "Emmy", 0.0 },
  { "Emmye", 0.0 },
  { "Emmylou", 0.0 },
  { "Emmytt", 1.0 },
  { "Emna", 0.0 },
  { "Emnet", 0.0 },
  { "Emo", 0.5555556 },
  { "Emogean", 0.0 },
  { "Emogene", 0.0 },
  { "Emogine", 0.0 },
  { "Emojean", 0.0 },
  { "Emolene", 0.0 },
  { "Emon", 0.8185484 },
  { "Emond", 1.0 },
  { "Emone", 0.083333336 },
  { "Emonee", 0.0 },
  { "Emoney", 0.0 },
  { "Emoni", 0.05100182 },
  { "Emonie", 0.015317286 },
  { "Emonii", 0.0 },
  { "Emonni", 0.0 },
  { "Emonnie", 0.0 },
  { "Emontae", 1.0 },
  { "Emonte", 1.0 },
  { "Emony", 0.0 },
  { "Emora", 0.0 },
  { "Emoree", 0.0 },
  { "Emori", 0.12606838 },
  { "Emorie", 0.0 },
  { "Emory", 0.79237086 },
  { "Emperatriz", 0.0 },
  { "Emperess", 0.0 },
  { "Emperor", 1.0 },
  { "Empress", 0.0 },
  { "Empris", 0.0 },
  { "Empriss", 0.0 },
  { "Empryss", 0.0 },
  { "Emra", 0.27272728 },
  { "Emraan", 1.0 },
  { "Emrah", 1.0 },
  { "Emran", 1.0 },
  { "Emre", 1.0 },
  { "Emree", 0.015652174 },
  { "Emreigh", 0.0 },
  { "Emrey", 0.1878453 },
  { "Emri", 0.065789476 },
  { "Emric", 1.0 },
  { "Emrick", 1.0 },
  { "Emrie", 0.00660502 },
  { "Emrik", 1.0 },
  { "Emron", 1.0 },
  { "Emroy", 1.0 },
  { "Emry", 0.3241791 },
  { "Emryck", 1.0 },
  { "Emryk", 1.0 },
  { "Emryn", 0.0 },
  { "Emrys", 0.79551125 },
  { "Emslee", 0.0 },
  { "Emsley", 0.023255814 },
  { "Emslie", 0.0 },
  { "Emuel", 1.0 },
  { "Emuna", 0.0 },
  { "Emunah", 0.0 },
  { "Emy", 0.0 },
  { "Emya", 0.0 },
  { "Emyah", 0.0 },
  { "Emyia", 0.0 },
  { "Emyiah", 0.0 },
  { "Emyla", 0.0 },
  { "Emylah", 0.0 },
  { "Emyle", 0.0 },
  { "Emylee", 0.0 },
  { "Emyleigh", 0.0 },
  { "Emyli", 0.0 },
  { "Emylia", 0.0 },
  { "Emylie", 0.0 },
  { "Emyly", 0.0 },
  { "Emyr", 1.0 },
  { "Emyree", 0.0 },
  { "Emyri", 0.0 },
  { "Emyrie", 0.0 },
  { "Emyrson", 0.0 },
  { "Emytt", 1.0 },
  { "Emzie", 1.0 },
  { "Emzley", 0.0 },
  { "Emzy", 1.0 },
  { "En", 1.0 },
  { "Ena", 0.0 },
  { "Enaiya", 0.0 },
  { "Enaja", 0.0 },
  { "Enajah", 0.0 },
  { "Enajiah", 0.0 },
  { "Enalina", 0.0 },
  { "Enam", 1.0 },
  { "Enan", 1.0 },
  { "Enara", 0.0 },
  { "Enari", 0.0 },
  { "Enas", 0.0 },
  { "Enasia", 0.0 },
  { "Enaya", 0.0 },
  { "Enayah", 0.0 },
  { "Enaysia", 0.0 },
  { "Encarnacion", 0.86489254 },
  { "Enchante", 0.0 },
  { "Enchantee", 0.0 },
  { "Enchantra", 0.0 },
  { "Encil", 1.0 },
  { "Enda", 0.0 },
  { "Endea", 0.0 },
  { "Ender", 1.0 },
  { "Enderson", 1.0 },
  { "Endesha", 0.0 },
  { "Endi", 1.0 },
  { "Endia", 0.0 },
  { "Endiah", 0.0 },
  { "Endie", 0.0 },
  { "Endiya", 0.0 },
  { "Endiyah", 0.0 },
  { "Endora", 0.0 },
  { "Endra", 0.0 },
  { "Endre", 1.0 },
  { "Endrew", 1.0 },
  { "Endri", 1.0 },
  { "Endrick", 1.0 },
  { "Endrit", 1.0 },
  { "Endsley", 0.0 },
  { "Endy", 0.9684874 },
  { "Endya", 0.0 },
  { "Endyah", 0.0 },
  { "Endyia", 0.0 },
  { "Enea", 1.0 },
  { "Eneas", 1.0 },
  { "Enedelia", 0.0 },
  { "Enedina", 0.0 },
  { "Enedino", 1.0 },
  { "Eneida", 0.0 },
  { "Enely", 0.0 },
  { "Enemencio", 1.0 },
  { "Ener", 1.0 },
  { "Eneri", 0.0 },
  { "Enes", 0.34608984 },
  { "Enesa", 0.0 },
  { "Enesha", 0.0 },
  { "Enessa", 0.0 },
  { "Enette", 0.0 },
  { "Eneyda", 0.0 },
  { "Enez", 0.0 },
  { "Enfinity", 0.0 },
  { "Enga", 0.0 },
  { "Engel", 1.0 },
  { "Engelbert", 1.0 },
  { "Engelberto", 1.0 },
  { "Engels", 1.0 },
  { "Enger", 0.0 },
  { "Engie", 0.0 },
  { "England", 0.12087912 },
  { "English", 0.41241685 },
  { "Engracia", 0.0 },
  { "Engrid", 0.0 },
  { "Eni", 0.0 },
  { "Enia", 0.0 },
  { "Eniah", 0.0 },
  { "Enice", 0.0 },
  { "Enid", 0.0006718925 },
  { "Enija", 0.0 },
  { "Enijah", 0.0 },
  { "Enika", 0.0 },
  { "Enilda", 0.0 },
  { "Enio", 1.0 },
  { "Eniola", 0.11210762 },
  { "Enis", 0.47396767 },
  { "Enisa", 0.0 },
  { "Enise", 0.0 },
  { "Enisha", 0.0 },
  { "Enissa", 0.0 },
  { "Enita", 0.0 },
  { "Enith", 0.0 },
  { "Eniya", 0.0 },
  { "Eniyah", 0.0 },
  { "Eniylah", 0.0 },
  { "Enjamin", 1.0 },
  { "Enjoli", 0.0 },
  { "Enjolie", 0.0 },
  { "Enki", 1.0 },
  { "Enloe", 1.0 },
  { "Enma", 0.0 },
  { "Enmanuel", 1.0 },
  { "Enna", 0.0 },
  { "Enner", 1.0 },
  { "Enneth", 1.0 },
  { "Ennette", 0.0 },
  { "Ennice", 0.0 },
  { "Ennie", 0.0 },
  { "Ennifer", 0.0 },
  { "Ennio", 1.0 },
  { "Ennis", 0.90643275 },
  { "Enno", 1.0 },
  { "Enny", 0.0 },
  { "Eno", 1.0 },
  { "Enoc", 1.0 },
  { "Enoch", 1.0 },
  { "Enock", 1.0 },
  { "Enoe", 1.0 },
  { "Enok", 1.0 },
  { "Enola", 0.0 },
  { "Enolia", 0.0 },
  { "Enora", 0.0 },
  { "Enos", 1.0 },
  { "Enosh", 1.0 },
  { "Enri", 1.0 },
  { "Enric", 1.0 },
  { "Enrica", 0.0 },
  { "Enrick", 1.0 },
  { "Enrico", 1.0 },
  { "Enrigue", 1.0 },
  { "Enrika", 0.0 },
  { "Enrike", 1.0 },
  { "Enrique", 0.9953873 },
  { "Enriqueta", 0.0 },
  { "Enriquetta", 0.0 },
  { "Enriquez", 1.0 },
  { "Enrrique", 1.0 },
  { "Enrriqueta", 0.0 },
  { "Enry", 1.0 },
  { "Ensar", 1.0 },
  { "Ensign", 1.0 },
  { "Enslee", 0.0 },
  { "Ensleigh", 0.0 },
  { "Ensley", 0.028985508 },
  { "Enslie", 0.0 },
  { "Enso", 1.0 },
  { "Enson", 1.0 },
  { "Envee", 0.0 },
  { "Enver", 1.0 },
  { "Envi", 0.0 },
  { "Envie", 0.0 },
  { "Envy", 0.010893246 },
  { "Envyi", 0.0 },
  { "Enya", 0.0 },
  { "Enyah", 0.0 },
  { "Enyce", 0.0 },
  { "Enyia", 0.0 },
  { "Enyla", 0.0 },
  { "Enylah", 0.0 },
  { "Enyssa", 0.0 },
  { "Enza", 0.0 },
  { "Enzi", 1.0 },
  { "Enzie", 0.0 },
  { "Enzio", 1.0 },
  { "Enzlee", 0.0 },
  { "Enzley", 0.0 },
  { "Enzlie", 0.0 },
  { "Enzo", 1.0 },
  { "Enzogabriel", 1.0 },
  { "Enzzo", 1.0 },
  { "Eoghan", 1.0 },
  { "Eoin", 1.0 },
  { "Eola", 0.0 },
  { "Eolia", 0.0 },
  { "Eoline", 0.0 },
  { "Eon", 1.0 },
  { "Eona", 0.0 },
  { "Eowyn", 0.0 },
  { "Eph", 1.0 },
  { "Ephraim", 1.0 },
  { "Ephrain", 1.0 },
  { "Ephram", 1.0 },
  { "Ephrata", 0.0 },
  { "Ephratah", 0.0 },
  { "Ephrem", 1.0 },
  { "Ephriam", 1.0 },
  { "Ephrim", 1.0 },
  { "Ephron", 1.0 },
  { "Epic", 0.79569894 },
  { "Epifania", 0.0 },
  { "Epifanio", 1.0 },
  { "Epigmenio", 1.0 },
  { "Epimenia", 0.0 },
  { "Epimenio", 1.0 },
  { "Epiphany", 0.0 },
  { "Epitacio", 1.0 },
  { "Epolito", 1.0 },
  { "Eponine", 0.0 },
  { "Eppie", 0.15224913 },
  { "Epsie", 0.0 },
  { "Equan", 1.0 },
  { "Equasha", 0.0 },
  { "Equasia", 0.0 },
  { "Equilla", 0.0 },
  { "Era", 0.0060975607 },
  { "Erabella", 0.0 },
  { "Eraclio", 1.0 },
  { "Eragon", 1.0 },
  { "Erah", 0.0 },
  { "Eraina", 0.0 },
  { "Eraine", 0.0 },
  { "Eraldo", 1.0 },
  { "Eralia", 0.0 },
  { "Eralyn", 0.0 },
  { "Eralynn", 0.0 },
  { "Eran", 0.8665644 },
  { "Erandi", 0.0 },
  { "Erandy", 0.0 },
  { "Erasmo", 1.0 },
  { "Erasmus", 1.0 },
  { "Erasto", 1.0 },
  { "Erastus", 1.0 },
  { "Eray", 1.0 },
  { "Erbey", 1.0 },
  { "Erbie", 0.94949496 },
  { "Erbin", 1.0 },
  { "Erblin", 1.0 },
  { "Erby", 1.0 },
  { "Ercel", 0.4548105 },
  { "Ercell", 0.32722512 },
  { "Ercelle", 0.0 },
  { "Ercia", 0.0 },
  { "Ercie", 0.0 },
  { "Ercil", 0.9268293 },
  { "Ercilia", 0.0 },
  { "Ercole", 1.0 },
  { "Erda", 0.0 },
  { "Erdell", 0.0 },
  { "Erdem", 1.0 },
  { "Erdene", 0.0 },
  { "Erdi", 1.0 },
  { "Erdine", 0.0 },
  { "Erdman", 1.0 },
  { "Erec", 1.0 },
  { "Ereca", 0.0 },
  { "Erek", 1.0 },
  { "Ereka", 0.0 },
  { "Erelene", 0.0 },
  { "Ereline", 0.0 },
  { "Eremy", 1.0 },
  { "Eren", 0.8324084 },
  { "Erena", 0.0 },
  { "Erendida", 0.0 },
  { "Erendira", 0.0 },
  { "Erene", 0.0 },
  { "Erenest", 1.0 },
  { "Erenst", 1.0 },
  { "Ereny", 0.0 },
  { "Eres", 0.0 },
  { "Eresmia", 0.0 },
  { "Eretria", 0.0 },
  { "Eretta", 0.0 },
  { "Erez", 1.0 },
  { "Erfan", 1.0 },
  { "Erford", 1.0 },
  { "Erhan", 1.0 },
  { "Erhard", 1.0 },
  { "Erhardt", 1.0 },
  { "Erhart", 1.0 },
  { "Eri", 0.5373609 },
  { "Eria", 0.0 },
  { "Eriah", 0.0 },
  { "Erial", 0.0 },
  { "Erian", 0.45679012 },
  { "Eriana", 0.0 },
  { "Erianah", 0.0 },
  { "Eriane", 0.0 },
  { "Eriani", 0.0 },
  { "Eriann", 0.0 },
  { "Erianna", 0.0 },
  { "Eriannah", 0.0 },
  { "Erianne", 0.0 },
  { "Erianny", 0.0 },
  { "Eribella", 0.0 },
  { "Eriberto", 1.0 },
  { "Eric", 0.9951008 },
  { "Erica", 0.0044442904 },
  { "Ericah", 0.0 },
  { "Erical", 0.0 },
  { "Ericalynn", 0.0 },
  { "Ericamarie", 0.0 },
  { "Ericanicole", 0.0 },
  { "Ericanthony", 1.0 },
  { "Ericberto", 1.0 },
  { "Ericc", 1.0 },
  { "Ericca", 0.0 },
  { "Ericdaniel", 1.0 },
  { "Erice", 0.8863636 },
  { "Erich", 0.9972275 },
  { "Ericha", 0.0 },
  { "Ericia", 0.0 },
  { "Ericjames", 1.0 },
  { "Erick", 0.9940836 },
  { "Ericka", 0.0020002609 },
  { "Erickah", 0.0 },
  { "Erickia", 0.0 },
  { "Erickson", 1.0 },
  { "Ericlee", 1.0 },
  { "Ericmichael", 1.0 },
  { "Erico", 1.0 },
  { "Ericson", 1.0 },
  { "Ericsson", 1.0 },
  { "Eridani", 0.0 },
  { "Erie", 0.58281296 },
  { "Erieanna", 0.0 },
  { "Eriel", 0.113144755 },
  { "Eriella", 0.0 },
  { "Erielle", 0.0 },
  { "Erien", 0.0 },
  { "Erienne", 0.0 },
  { "Erieonna", 0.0 },
  { "Eries", 1.0 },
  { "Erihanna", 0.0 },
  { "Erik", 0.99560386 },
  { "Erika", 0.0044238144 },
  { "Erikah", 0.0 },
  { "Erikc", 1.0 },
  { "Erike", 0.39285713 },
  { "Erikia", 0.0 },
  { "Erikk", 1.0 },
  { "Erikka", 0.0 },
  { "Eriko", 0.0 },
  { "Eriks", 1.0 },
  { "Eriksen", 1.0 },
  { "Erikson", 1.0 },
  { "Eriksson", 1.0 },
  { "Eril", 1.0 },
  { "Erilyn", 0.0 },
  { "Erilynn", 0.0 },
  { "Erim", 0.0 },
  { "Erin", 0.028576236 },
  { "Erina", 0.0 },
  { "Erine", 0.12 },
  { "Erineo", 1.0 },
  { "Erini", 0.0 },
  { "Erinique", 0.0 },
  { "Erinisha", 0.0 },
  { "Erinmarie", 0.0 },
  { "Erinn", 0.040729135 },
  { "Erinne", 0.0 },
  { "Eriny", 0.0 },
  { "Erioluwa", 0.47916666 },
  { "Erion", 0.7989324 },
  { "Eriona", 0.0 },
  { "Erionna", 0.0 },
  { "Erionne", 0.0 },
  { "Eriq", 1.0 },
  { "Eriqa", 0.0 },
  { "Eriqua", 0.0 },
  { "Erique", 1.0 },
  { "Eris", 0.17663552 },
  { "Erisa", 0.0 },
  { "Erisha", 0.0 },
  { "Erison", 1.0 },
  { "Eriss", 0.0 },
  { "Erissa", 0.0 },
  { "Eritrea", 0.0 },
  { "Erivan", 1.0 },
  { "Eriverto", 1.0 },
  { "Erix", 1.0 },
  { "Eriyah", 0.0 },
  { "Eriyan", 0.0 },
  { "Eriyana", 0.0 },
  { "Eriyanna", 0.0 },
  { "Eriyon", 0.52380955 },
  { "Eriyona", 0.0 },
  { "Eriyonna", 0.0 },
  { "Erjon", 1.0 },
  { "Erka", 0.0 },
  { "Erkan", 1.0 },
  { "Erla", 0.0 },
  { "Erlan", 1.0 },
  { "Erland", 1.0 },
  { "Erle", 1.0 },
  { "Erlean", 0.0 },
  { "Erleen", 0.0 },
  { "Erlene", 0.0 },
  { "Erlin", 0.94930875 },
  { "Erlinda", 0.0 },
  { "Erline", 0.0 },
  { "Erling", 1.0 },
  { "Erlon", 1.0 },
  { "Erlyne", 0.0 },
  { "Erlys", 0.0 },
  { "Erma", 0.0043286895 },
  { "Ermadean", 0.0 },
  { "Ermadine", 0.0 },
  { "Ermagene", 0.0 },
  { "Ermajean", 0.0 },
  { "Ermal", 0.43283582 },
  { "Ermalea", 0.0 },
  { "Ermalee", 0.0 },
  { "Ermalene", 0.0 },
  { "Ermalinda", 0.0 },
  { "Ermaline", 0.0 },
  { "Erman", 1.0 },
  { "Ermel", 0.4649123 },
  { "Ermelinda", 0.0 },
  { "Ermenia", 0.0 },
  { "Ermer", 0.0 },
  { "Ermia", 0.0 },
  { "Ermias", 1.0 },
  { "Ermie", 0.035460994 },
  { "Ermil", 1.0 },
  { "Ermila", 0.0 },
  { "Ermilo", 1.0 },
  { "Ermin", 1.0 },
  { "Ermina", 0.0 },
  { "Erminda", 0.0 },
  { "Ermine", 0.11375212 },
  { "Erminia", 0.0 },
  { "Erminie", 0.0 },
  { "Erminio", 1.0 },
  { "Ermis", 1.0 },
  { "Ermma", 0.0 },
  { "Ermon", 0.92462313 },
  { "Ermond", 1.0 },
  { "Erna", 0.0012142621 },
  { "Ernad", 1.0 },
  { "Ernal", 1.0 },
  { "Ernan", 1.0 },
  { "Ernel", 1.0 },
  { "Ernell", 0.0 },
  { "Ernesha", 0.0 },
  { "Erneshia", 0.0 },
  { "Ernest", 0.99386686 },
  { "Ernesta", 0.0 },
  { "Ernesteen", 0.0 },
  { "Ernestene", 0.0 },
  { "Ernestina", 0.0 },
  { "Ernestine", 0.0036005434 },
  { "Ernesto", 0.9971306 },
  { "Ernestor", 1.0 },
  { "Ernestyne", 0.0 },
  { "Ernetta", 0.0 },
  { "Ernette", 0.0 },
  { "Erney", 1.0 },
  { "Ernice", 0.0 },
  { "Ernie", 0.9581941 },
  { "Erniest", 1.0 },
  { "Ernisha", 0.0 },
  { "Ernist", 1.0 },
  { "Ernistine", 0.0 },
  { "Erno", 1.0 },
  { "Ernst", 1.0 },
  { "Ernstine", 0.0 },
  { "Erny", 1.0 },
  { "Ero", 1.0 },
  { "Erol", 1.0 },
  { "Eron", 0.8674033 },
  { "Eronda", 0.0 },
  { "Eros", 1.0 },
  { "Eroz", 1.0 },
  { "Erran", 1.0 },
  { "Errett", 1.0 },
  { "Erric", 1.0 },
  { "Errica", 0.0 },
  { "Errick", 1.0 },
  { "Erricka", 0.0 },
  { "Errik", 1.0 },
  { "Errika", 0.0 },
  { "Errin", 0.24348608 },
  { "Errion", 1.0 },
  { "Erris", 1.0 },
  { "Errol", 1.0 },
  { "Errold", 1.0 },
  { "Erroll", 1.0 },
  { "Erron", 0.98141265 },
  { "Erryn", 0.0 },
  { "Ersa", 0.0 },
  { "Ersel", 0.75 },
  { "Ersell", 0.0 },
  { "Erselle", 0.0 },
  { "Ersheen", 0.0 },
  { "Ershel", 1.0 },
  { "Ersie", 0.011904762 },
  { "Ersilia", 0.0 },
  { "Erskin", 1.0 },
  { "Erskine", 1.0 },
  { "Ersula", 0.0 },
  { "Ertha", 0.0 },
  { "Ertie", 0.0 },
  { "Ertis", 0.33333334 },
  { "Erubey", 1.0 },
  { "Erubiel", 1.0 },
  { "Erum", 0.0 },
  { "Erva", 0.0 },
  { "Ervan", 1.0 },
  { "Erven", 1.0 },
  { "Ervena", 0.0 },
  { "Ervene", 0.0 },
  { "Ervey", 1.0 },
  { "Ervie", 0.8247863 },
  { "Ervil", 1.0 },
  { "Erville", 1.0 },
  { "Ervin", 0.9963823 },
  { "Ervina", 0.0 },
  { "Ervine", 1.0 },
  { "Erving", 1.0 },
  { "Ervy", 1.0 },
  { "Erwan", 1.0 },
  { "Erwin", 0.99764174 },
  { "Ery", 1.0 },
  { "Eryana", 0.0 },
  { "Eryanna", 0.0 },
  { "Eryc", 1.0 },
  { "Eryca", 0.0 },
  { "Eryck", 1.0 },
  { "Erycka", 0.0 },
  { "Eryk", 1.0 },
  { "Eryka", 0.0 },
  { "Erykah", 0.0 },
  { "Eryn", 0.02133879 },
  { "Eryne", 0.0 },
  { "Erynn", 0.0 },
  { "Erynne", 0.0 },
  { "Eryonna", 0.0 },
  { "Eryss", 0.0 },
  { "Eryx", 1.0 },
  { "Erza", 0.27027026 },
  { "Erzsebet", 0.0 },
  { "Esa", 0.8449848 },
  { "Esabel", 0.0 },
  { "Esabella", 0.0 },
  { "Esabelle", 0.0 },
  { "Esad", 1.0 },
  { "Esai", 1.0 },
  { "Esaiah", 1.0 },
  { "Esaias", 1.0 },
  { "Esaie", 1.0 },
  { "Esain", 1.0 },
  { "Esam", 1.0 },
  { "Esat", 1.0 },
  { "Esau", 1.0 },
  { "Esaul", 1.0 },
  { "Esaw", 1.0 },
  { "Esbeidi", 0.0 },
  { "Esbeidy", 0.0 },
  { "Esben", 1.0 },
  { "Esbeydi", 0.0 },
  { "Esbeydy", 0.0 },
  { "Escar", 1.0 },
  { "Escarlet", 0.0 },
  { "Escarleth", 0.0 },
  { "Escarlett", 0.0 },
  { "Eschelle", 0.0 },
  { "Escher", 1.0 },
  { "Eschol", 1.0 },
  { "Esco", 1.0 },
  { "Escolastica", 0.0 },
  { "Esdras", 1.0 },
  { "Esean", 1.0 },
  { "Esekiel", 1.0 },
  { "Esenia", 0.0 },
  { "Eseosa", 0.0 },
  { "Esequiel", 1.0 },
  { "Eseta", 0.0 },
  { "Esey", 1.0 },
  { "Esgar", 1.0 },
  { "Esgardo", 1.0 },
  { "Esha", 0.0 },
  { "Eshaal", 0.0 },
  { "Eshaan", 1.0 },
  { "Eshal", 0.0 },
  { "Eshan", 1.0 },
  { "Eshana", 0.0 },
  { "Eshani", 0.0 },
  { "Eshanti", 0.0 },
  { "Eshaun", 1.0 },
  { "Eshawn", 1.0 },
  { "Eshawna", 0.0 },
  { "Eshe", 0.0 },
  { "Eshell", 0.0 },
  { "Esher", 1.0 },
  { "Eshika", 0.0 },
  { "Eshin", 1.0 },
  { "Eshita", 0.0 },
  { "Eshon", 1.0 },
  { "Eshter", 0.0 },
  { "Eshton", 1.0 },
  { "Eshwar", 1.0 },
  { "Esi", 0.0 },
  { "Esiah", 1.0 },
  { "Esias", 1.0 },
  { "Esiquiel", 1.0 },
  { "Esiquio", 1.0 },
  { "Eskel", 1.0 },
  { "Esker", 1.0 },
  { "Eskil", 1.0 },
  { "Esko", 1.0 },
  { "Eslem", 0.0 },
  { "Esley", 0.9605735 },
  { "Esli", 0.22510822 },
  { "Eslie", 0.625 },
  { "Esly", 0.019933555 },
  { "Eslyn", 0.0 },
  { "Esma", 0.0 },
  { "Esmae", 0.0 },
  { "Esmael", 1.0 },
  { "Esmail", 1.0 },
  { "Esmaralda", 0.0 },
  { "Esmarie", 0.0 },
  { "Esmay", 0.0 },
  { "Esme", 0.0 },
  { "Esmee", 0.0 },
  { "Esmer", 0.0 },
  { "Esmerai", 0.0 },
  { "Esmeraida", 0.0 },
  { "Esmerald", 0.0 },
  { "Esmeralda", 0.004064857 },
  { "Esmeraldo", 0.0 },
  { "Esmerelda", 0.0 },
  { "Esmerie", 0.0 },
  { "Esmerlda", 0.0 },
  { "Esmi", 0.0 },
  { "Esmia", 0.0 },
  { "Esmie", 0.0 },
  { "Esmirna", 0.0 },
  { "Esmond", 1.0 },
  { "Esnaider", 1.0 },
  { "Esneider", 1.0 },
  { "Esneyder", 1.0 },
  { "Esohe", 0.0 },
  { "Esosa", 0.24390244 },
  { "Espanola", 0.0 },
  { "Esparanza", 0.0 },
  { "Espen", 0.9375 },
  { "Esper", 0.8684211 },
  { "Esperance", 0.0 },
  { "Esperansa", 0.0 },
  { "Esperanza", 0.0018817204 },
  { "Espie", 0.7083333 },
  { "Espiridion", 1.0 },
  { "Espn", 0.9 },
  { "Esprit", 0.0 },
  { "Espy", 1.0 },
  { "Espyn", 0.15555556 },
  { "Esque", 1.0 },
  { "Esquiel", 1.0 },
  { "Esquire", 1.0 },
  { "Esra", 0.0 },
  { "Esraa", 0.0 },
  { "Esrael", 1.0 },
  { "Esrom", 1.0 },
  { "Essa", 0.75259876 },
  { "Essam", 1.0 },
  { "Essance", 0.0 },
  { "Essence", 0.0 },
  { "Essense", 0.0 },
  { "Essex", 1.0 },
  { "Essey", 1.0 },
  { "Essfa", 0.0 },
  { "Essiah", 1.0 },
  { "Essica", 0.0 },
  { "Essie", 0.02995181 },
  { "Essiemae", 0.0 },
  { "Essien", 1.0 },
  { "Essined", 0.0 },
  { "Essix", 1.0 },
  { "Essye", 0.0 },
  { "Essynce", 0.0 },
  { "Esta", 0.0061169006 },
  { "Estaban", 1.0 },
  { "Estacia", 0.0 },
  { "Estafani", 0.0 },
  { "Estafania", 0.0 },
  { "Estal", 1.0 },
  { "Estalee", 0.0 },
  { "Estaleen", 0.0 },
  { "Estalene", 0.0 },
  { "Estaline", 0.0 },
  { "Estanislada", 0.0 },
  { "Estanislado", 1.0 },
  { "Estanislao", 1.0 },
  { "Estarlin", 1.0 },
  { "Estavan", 1.0 },
  { "Estavon", 1.0 },
  { "Esteban", 0.9989448 },
  { "Estee", 0.057563588 },
  { "Esteen", 0.0 },
  { "Estefan", 1.0 },
  { "Estefana", 0.0 },
  { "Estefani", 0.0038976858 },
  { "Estefania", 0.0008229098 },
  { "Estefanie", 0.0 },
  { "Estefanita", 0.0 },
  { "Estefanny", 0.0 },
  { "Estefano", 1.0 },
  { "Estefany", 0.0010834236 },
  { "Estefanya", 0.0 },
  { "Esteffany", 0.0 },
  { "Estefhany", 0.0 },
  { "Estel", 0.839052 },
  { "Estela", 0.0 },
  { "Estele", 0.0 },
  { "Estelee", 0.0 },
  { "Estelene", 0.0 },
  { "Esteline", 0.0 },
  { "Estelita", 0.0 },
  { "Estell", 0.11445909 },
  { "Estella", 0.0014195014 },
  { "Estellar", 0.0 },
  { "Estelle", 0.002998303 },
  { "Estellene", 0.0 },
  { "Esteller", 0.0 },
  { "Estelline", 0.0 },
  { "Estellise", 0.0 },
  { "Esten", 1.0 },
  { "Estephan", 1.0 },
  { "Estephani", 0.0 },
  { "Estephania", 0.0 },
  { "Estephanie", 0.0 },
  { "Estephany", 0.0 },
  { "Estephen", 1.0 },
  { "Ester", 0.05594274 },
  { "Estera", 0.0 },
  { "Esterine", 0.0 },
  { "Esterlene", 0.0 },
  { "Esterline", 0.0 },
  { "Estes", 0.969697 },
  { "Estevan", 0.9987956 },
  { "Esteve", 1.0 },
  { "Esteven", 1.0 },
  { "Estevon", 1.0 },
  { "Esteysi", 0.0 },
  { "Estha", 0.0 },
  { "Esthefani", 0.0 },
  { "Esthefania", 0.0 },
  { "Esthefany", 0.0 },
  { "Esthel", 0.0 },
  { "Esthela", 0.0 },
  { "Esther", 0.0037848833 },
  { "Estherlene", 0.0 },
  { "Esthermae", 0.0 },
  { "Esti", 0.0 },
  { "Estibaliz", 0.0 },
  { "Estiben", 1.0 },
  { "Estie", 0.0 },
  { "Estil", 1.0 },
  { "Estill", 0.99441344 },
  { "Estilla", 0.0 },
  { "Estin", 1.0 },
  { "Estine", 0.0 },
  { "Estis", 1.0 },
  { "Estivalis", 0.0 },
  { "Estiven", 1.0 },
  { "Estle", 0.8941799 },
  { "Estol", 1.0 },
  { "Eston", 1.0 },
  { "Estoria", 0.0 },
  { "Estrada", 1.0 },
  { "Estralita", 0.0 },
  { "Estrelita", 0.0 },
  { "Estrella", 0.0 },
  { "Estrellita", 0.0 },
  { "Estreya", 0.0 },
  { "Estuardo", 1.0 },
  { "Estus", 1.0 },
  { "Esty", 0.0 },
  { "Esveidy", 0.0 },
  { "Esven", 1.0 },
  { "Esvin", 1.0 },
  { "Eswin", 1.0 },
  { "Eszter", 0.0 },
  { "Eta", 0.0 },
  { "Etai", 1.0 },
  { "Etalo", 1.0 },
  { "Etan", 1.0 },
  { "Etana", 0.0 },
  { "Etash", 1.0 },
  { "Etasha", 0.0 },
  { "Etelka", 0.0 },
  { "Etelvina", 0.0 },
  { "Eternal", 1.0 },
  { "Eterniti", 0.0 },
  { "Eternity", 0.0 },
  { "Etha", 0.0 },
  { "Ethaen", 1.0 },
  { "Ethal", 0.0 },
  { "Etham", 1.0 },
  { "Ethan", 0.9983381 },
  { "Ethanael", 1.0 },
  { "Ethanalexander", 1.0 },
  { "Ethanandrew", 1.0 },
  { "Ethane", 1.0 },
  { "Ethaneal", 1.0 },
  { "Ethangabriel", 1.0 },
  { "Ethanial", 1.0 },
  { "Ethaniel", 1.0 },
  { "Ethanjacob", 1.0 },
  { "Ethanjames", 1.0 },
  { "Ethanjay", 1.0 },
  { "Ethanjohn", 1.0 },
  { "Ethanjoseph", 1.0 },
  { "Ethanmatthew", 1.0 },
  { "Ethanmichael", 1.0 },
  { "Ethann", 1.0 },
  { "Ethanpaul", 1.0 },
  { "Ethany", 0.0 },
  { "Ethean", 1.0 },
  { "Ethel", 0.0053184084 },
  { "Ethelann", 0.0 },
  { "Ethelbert", 0.9726776 },
  { "Ethelda", 0.0 },
  { "Etheldra", 0.0 },
  { "Etheldreda", 0.0 },
  { "Ethele", 0.0 },
  { "Ethelean", 0.0 },
  { "Ethelee", 0.0 },
  { "Etheleen", 0.0 },
  { "Ethelen", 0.0 },
  { "Ethelene", 0.0 },
  { "Ethelind", 0.0 },
  { "Etheline", 0.0 },
  { "Ethell", 0.0 },
  { "Ethelle", 0.0 },
  { "Ethellee", 0.0 },
  { "Ethelmae", 0.0 },
  { "Ethelmay", 0.0 },
  { "Ethelreda", 0.0 },
  { "Ethelrene", 0.0 },
  { "Ethelrine", 0.0 },
  { "Ethelwyn", 0.0 },
  { "Ethelyn", 0.0 },
  { "Ethelyne", 0.0 },
  { "Ethelynn", 0.0 },
  { "Ethen", 1.0 },
  { "Ether", 0.0515625 },
  { "Etheridge", 1.0 },
  { "Etherine", 0.0 },
  { "Ethian", 1.0 },
  { "Ethie", 0.0 },
  { "Ethil", 0.0 },
  { "Ethin", 1.0 },
  { "Ethleen", 0.0 },
  { "Ethlyn", 0.0 },
  { "Ethna", 0.0 },
  { "Ethne", 0.0 },
  { "Ethny", 0.0 },
  { "Ethon", 1.0 },
  { "Ethredge", 1.0 },
  { "Ethridge", 1.0 },
  { "Ethyl", 0.0 },
  { "Ethyle", 0.0 },
  { "Ethyleen", 0.0 },
  { "Ethylene", 0.0 },
  { "Ethyn", 1.0 },
  { "Ethynn", 1.0 },
  { "Etienne", 0.94425327 },
  { "Etinosa", 1.0 },
  { "Etna", 0.0 },
  { "Etnie", 0.0 },
  { "Etoile", 0.0 },
  { "Etola", 0.0 },
  { "Etolia", 0.0 },
  { "Eton", 1.0 },
  { "Etonya", 0.0 },
  { "Etosha", 0.0 },
  { "Etoshia", 0.0 },
  { "Etoy", 0.0 },
  { "Etoya", 0.0 },
  { "Etoyi", 1.0 },
  { "Etoyle", 0.0 },
  { "Etrulia", 0.0 },
  { "Etsel", 1.0 },
  { "Etsub", 0.0 },
  { "Etsuko", 0.0 },
  { "Etsuo", 1.0 },
  { "Etta", 0.000563132 },
  { "Ettalyn", 0.0 },
  { "Ettamae", 0.0 },
  { "Ettel", 0.0 },
  { "Etter", 0.0 },
  { "Etti", 0.0 },
  { "Ettie", 0.0 },
  { "Ettore", 1.0 },
  { "Etty", 0.0 },
  { "Etzio", 1.0 },
  { "Eua", 0.0 },
  { "Eual", 1.0 },
  { "Euan", 1.0 },
  { "Euba", 0.0 },
  { "Eubert", 1.0 },
  { "Euclid", 1.0 },
  { "Euclide", 1.0 },
  { "Euclides", 1.0 },
  { "Euda", 0.0 },
  { "Eudean", 0.0 },
  { "Eudelia", 0.0 },
  { "Eudell", 0.3538874 },
  { "Eudella", 0.0 },
  { "Eudena", 0.0 },
  { "Eudene", 0.0 },
  { "Eudice", 0.0 },
  { "Eudine", 0.0 },
  { "Eudon", 1.0 },
  { "Eudora", 0.0 },
  { "Eudy", 1.0 },
  { "Euel", 1.0 },
  { "Euell", 1.0 },
  { "Euella", 0.0 },
  { "Eufaula", 0.0 },
  { "Eufelia", 0.0 },
  { "Eufemia", 0.0 },
  { "Eufemio", 1.0 },
  { "Eufracia", 0.0 },
  { "Eufracio", 1.0 },
  { "Eufrasia", 0.0 },
  { "Eugean", 1.0 },
  { "Eugen", 1.0 },
  { "Eugena", 0.0 },
  { "Eugene", 0.99310553 },
  { "Eugenia", 0.0005088368 },
  { "Eugenie", 0.0 },
  { "Eugenio", 1.0 },
  { "Eugie", 0.0 },
  { "Eugina", 0.0 },
  { "Eugine", 1.0 },
  { "Euginia", 0.0 },
  { "Eugne", 1.0 },
  { "Eugonda", 0.0 },
  { "Euguene", 1.0 },
  { "Eugune", 1.0 },
  { "Euin", 1.0 },
  { "Eujin", 0.0 },
  { "Eula", 0.0070056976 },
  { "Eulah", 0.0 },
  { "Eulala", 0.0 },
  { "Eulalah", 0.0 },
  { "Eulalee", 0.0 },
  { "Eulalia", 0.0 },
  { "Eulalie", 0.0 },
  { "Eulalio", 1.0 },
  { "Eulamae", 0.0 },
  { "Eulan", 1.0 },
  { "Eulanda", 0.0 },
  { "Eulane", 0.0 },
  { "Eular", 0.0 },
  { "Eulas", 1.0 },
  { "Eulean", 0.0 },
  { "Eulee", 0.0 },
  { "Eulema", 0.0 },
  { "Eulene", 0.0 },
  { "Eules", 1.0 },
  { "Euless", 1.0 },
  { "Euleta", 0.0 },
  { "Eulia", 0.0 },
  { "Eulice", 0.9790795 },
  { "Eulices", 1.0 },
  { "Eulie", 1.0 },
  { "Eulina", 0.0 },
  { "Eulinda", 0.0 },
  { "Euline", 0.0 },
  { "Eulis", 1.0 },
  { "Eulises", 1.0 },
  { "Eulla", 0.0 },
  { "Eulogia", 0.0 },
  { "Eulogio", 1.0 },
  { "Eulojia", 0.0 },
  { "Eulojio", 1.0 },
  { "Eulon", 1.0 },
  { "Eulonda", 0.0 },
  { "Eulos", 1.0 },
  { "Eulus", 1.0 },
  { "Eulys", 1.0 },
  { "Eumeka", 0.0 },
  { "Eun", 0.01724138 },
  { "Euna", 0.0 },
  { "Eunetta", 0.0 },
  { "Eunia", 0.0 },
  { "Eunice", 0.014626455 },
  { "Eunie", 0.0 },
  { "Euniece", 0.0 },
  { "Eunika", 0.0 },
  { "Euniqua", 0.0 },
  { "Eunique", 0.016949153 },
  { "Eunise", 0.0 },
  { "Eunita", 0.0 },
  { "Eunji", 0.0 },
  { "Eupha", 0.0 },
  { "Euphemia", 0.0 },
  { "Euphemie", 0.0 },
  { "Euphoria", 0.0 },
  { "Euple", 0.0 },
  { "Eura", 0.05465192 },
  { "Euradell", 0.0 },
  { "Eural", 1.0 },
  { "Eurania", 0.0 },
  { "Eurasia", 0.0 },
  { "Euree", 0.0 },
  { "Eureeka", 0.0 },
  { "Eureka", 0.005952381 },
  { "Eurel", 1.0 },
  { "Euretha", 0.0 },
  { "Euretta", 0.0 },
  { "Euri", 0.53846157 },
  { "Euriah", 0.9230769 },
  { "Eurica", 0.0 },
  { "Euricka", 0.0 },
  { "Eurie", 0.6666667 },
  { "Euriel", 1.0 },
  { "Eurijah", 1.0 },
  { "Eurika", 0.0 },
  { "Euris", 1.0 },
  { "Eurith", 0.0 },
  { "Eurma", 0.0 },
  { "Eury", 1.0 },
  { "Eurydice", 0.0 },
  { "Eusebia", 0.0 },
  { "Eusebio", 1.0 },
  { "Eusevia", 0.0 },
  { "Eusevio", 1.0 },
  { "Eustace", 1.0 },
  { "Eustacia", 0.0 },
  { "Eustacio", 1.0 },
  { "Eustaquio", 1.0 },
  { "Eustice", 1.0 },
  { "Eustis", 1.0 },
  { "Eustolia", 0.0 },
  { "Eustolio", 1.0 },
  { "Eutha", 0.0 },
  { "Eutimia", 0.0 },
  { "Eutimio", 1.0 },
  { "Eutiquio", 1.0 },
  { "Euva", 0.0 },
  { "Euzelia", 0.0 },
  { "Eva", 0.0038258636 },
  { "Evaan", 1.0 },
  { "Evabelle", 0.0 },
  { "Evada", 0.0 },
  { "Evadean", 0.0 },
  { "Evadell", 0.0 },
  { "Evadene", 0.0 },
  { "Evadine", 0.0 },
  { "Evadna", 0.0 },
  { "Evadne", 0.0 },
  { "Evaeh", 0.0 },
  { "Evagelia", 0.0 },
  { "Evagelos", 1.0 },
  { "Evagene", 0.0 },
  { "Evaggelia", 0.0 },
  { "Evagrace", 0.0 },
  { "Evah", 0.0 },
  { "Evaine", 0.0 },
  { "Evajane", 0.0 },
  { "Evajean", 0.0 },
  { "Evajo", 0.0 },
  { "Evalani", 0.0 },
  { "Evald", 1.0 },
  { "Evalean", 0.0 },
  { "Evalee", 0.0 },
  { "Evaleen", 0.0 },
  { "Evaleena", 0.0 },
  { "Evalei", 0.0 },
  { "Evaleigh", 0.0 },
  { "Evalen", 0.0 },
  { "Evalena", 0.0 },
  { "Evalene", 0.0 },
  { "Evalet", 0.0 },
  { "Evalett", 0.0 },
  { "Evalette", 0.0 },
  { "Evalia", 0.0 },
  { "Evalie", 0.0 },
  { "Evalin", 0.0 },
  { "Evalina", 0.0 },
  { "Evalinda", 0.0 },
  { "Evaline", 0.0 },
  { "Evalisa", 0.0 },
  { "Evalise", 0.0 },
  { "Evalisse", 0.0 },
  { "Evaliz", 0.0 },
  { "Evalou", 0.0 },
  { "Evaluna", 0.0 },
  { "Evaly", 0.0 },
  { "Evalyn", 0.0 },
  { "Evalyna", 0.0 },
  { "Evalyne", 0.0 },
  { "Evalynn", 0.0 },
  { "Evalynne", 0.0 },
  { "Evalyse", 0.0 },
  { "Evamae", 0.0 },
  { "Evamaria", 0.0 },
  { "Evamarie", 0.0 },
  { "Evan", 0.9845699 },
  { "Evana", 0.0 },
  { "Evander", 1.0 },
  { "Evanee", 0.0 },
  { "Evanell", 0.0 },
  { "Evanelle", 0.0 },
  { "Evaney", 0.0 },
  { "Evangalene", 0.0 },
  { "Evangaline", 0.0 },
  { "Evangel", 1.0 },
  { "Evangela", 0.0 },
  { "Evangeleen", 0.0 },
  { "Evangelena", 0.0 },
  { "Evangelene", 0.0 },
  { "Evangelia", 0.0 },
  { "Evangelin", 0.0 },
  { "Evangelina", 0.0 },
  { "Evangeline", 0.0 },
  { "Evangelita", 0.0 },
  { "Evangelo", 1.0 },
  { "Evangelos", 1.0 },
  { "Evangely", 0.0 },
  { "Evangelyn", 0.0 },
  { "Evangelyne", 0.0 },
  { "Evangelynn", 0.0 },
  { "Evangline", 0.0 },
  { "Evani", 0.0 },
  { "Evania", 0.0 },
  { "Evanie", 0.0 },
  { "Evanjames", 1.0 },
  { "Evanjelina", 0.0 },
  { "Evann", 0.5536232 },
  { "Evanna", 0.0 },
  { "Evannah", 0.0 },
  { "Evanne", 0.0 },
  { "Evanni", 0.0 },
  { "Evannie", 0.0 },
  { "Evanny", 0.0 },
  { "Evanora", 0.0 },
  { "Evans", 0.9912191 },
  { "Evanston", 1.0 },
  { "Evante", 1.0 },
  { "Evanthia", 0.0 },
  { "Evany", 0.0 },
  { "Evanyelin", 0.0 },
  { "Evar", 1.0 },
  { "Evard", 1.0 },
  { "Evarista", 0.0 },
  { "Evaristo", 1.0 },
  { "Evarose", 0.0 },
  { "Evart", 1.0 },
  { "Evaughn", 0.0 },
  { "Evay", 1.0 },
  { "Evaya", 0.0 },
  { "Evdokia", 0.0 },
  { "Evdoxia", 0.0 },
  { "Eve", 0.0 },
  { "Evea", 0.0 },
  { "Eveah", 0.0 },
  { "Eveana", 0.0 },
  { "Eveanna", 0.0 },
  { "Evee", 0.0 },
  { "Evel", 1.0 },
  { "Evelean", 0.0 },
  { "Evelee", 0.0 },
  { "Eveleen", 0.0 },
  { "Eveleigh", 0.0 },
  { "Evelen", 0.0 },
  { "Evelena", 0.0 },
  { "Evelene", 0.0 },
  { "Evelet", 0.0 },
  { "Evelett", 0.0 },
  { "Evelette", 0.0 },
  { "Eveli", 0.0 },
  { "Evelia", 0.0 },
  { "Evelie", 0.0 },
  { "Evelien", 0.0 },
  { "Evelin", 0.0007524454 },
  { "Evelina", 0.0 },
  { "Evelinda", 0.0 },
  { "Eveline", 0.0 },
  { "Evelinn", 0.0 },
  { "Evelio", 1.0 },
  { "Evelis", 0.0 },
  { "Evelise", 0.0 },
  { "Evelisse", 0.0 },
  { "Eveliz", 0.0 },
  { "Evell", 1.0 },
  { "Evella", 0.0 },
  { "Evellyn", 0.0 },
  { "Evely", 0.0 },
  { "Evelyn", 0.0033030047 },
  { "Evelyna", 0.0 },
  { "Evelyne", 0.0 },
  { "Evelynn", 0.0 },
  { "Evelynne", 0.0 },
  { "Evelynrose", 0.0 },
  { "Evelyon", 0.0 },
  { "Evelyse", 0.0 },
  { "Evemarie", 0.0 },
  { "Even", 1.0 },
  { "Evenie", 0.0 },
  { "Evening", 0.0 },
  { "Evennie", 0.0 },
  { "Evenny", 0.0 },
  { "Evens", 1.0 },
  { "Eveny", 0.0 },
  { "Ever", 0.60946137 },
  { "Evera", 0.0 },
  { "Everado", 1.0 },
  { "Everal", 0.5 },
  { "Everard", 1.0 },
  { "Everardo", 1.0 },
  { "Everdeen", 0.0 },
  { "Everee", 0.0 },
  { "Everest", 0.8819444 },
  { "Everet", 1.0 },
  { "Everett", 0.9942536 },
  { "Everetta", 0.0 },
  { "Everette", 0.9850256 },
  { "Evergreen", 0.0 },
  { "Everhett", 1.0 },
  { "Everick", 1.0 },
  { "Everitt", 1.0 },
  { "Everlea", 0.0 },
  { "Everlean", 0.0 },
  { "Everleaner", 0.0 },
  { "Everlee", 0.0 },
  { "Everleen", 0.0 },
  { "Everlei", 0.0 },
  { "Everleigh", 0.0 },
  { "Everlena", 0.0 },
  { "Everlene", 0.0 },
  { "Everlener", 0.0 },
  { "Everley", 0.0 },
  { "Everli", 0.0 },
  { "Everlie", 0.0 },
  { "Everliegh", 0.0 },
  { "Everlin", 0.0 },
  { "Everlina", 0.0 },
  { "Everline", 0.0 },
  { "Everly", 0.0012406948 },
  { "Everlyn", 0.0 },
  { "Everlynn", 0.0 },
  { "Everlyse", 0.0 },
  { "Evern", 0.6792453 },
  { "Everrett", 1.0 },
  { "Everrette", 1.0 },
  { "Everson", 1.0 },
  { "Eversyn", 0.0 },
  { "Evert", 1.0 },
  { "Everth", 1.0 },
  { "Everton", 1.0 },
  { "Evertt", 1.0 },
  { "Evertte", 1.0 },
  { "Every", 0.5 },
  { "Evester", 1.0 },
  { "Evet", 0.0 },
  { "Evett", 0.0 },
  { "Evetta", 0.0 },
  { "Evette", 0.0 },
  { "Evey", 0.0 },
  { "Eveyln", 0.0 },
  { "Evgenia", 0.0 },
  { "Evi", 0.0 },
  { "Evia", 0.0 },
  { "Evian", 0.48358864 },
  { "Eviana", 0.0 },
  { "Evianna", 0.0 },
  { "Evie", 0.007474813 },
  { "Evienne", 0.0 },
  { "Evila", 0.0 },
  { "Evilin", 0.0 },
  { "Evilo", 0.0 },
  { "Evilyn", 0.0 },
  { "Evilynn", 0.0 },
  { "Evin", 0.85662866 },
  { "Evina", 0.0 },
  { "Evins", 1.0 },
  { "Evion", 1.0 },
  { "Evionna", 0.0 },
  { "Evita", 0.0 },
  { "Evleen", 0.0 },
  { "Evleyn", 0.0 },
  { "Evlin", 0.0 },
  { "Evline", 0.0 },
  { "Evlyn", 0.0 },
  { "Evlynn", 0.0 },
  { "Evnika", 0.0 },
  { "Evo", 1.0 },
  { "Evodio", 1.0 },
  { "Evola", 0.0 },
  { "Evoleht", 0.0 },
  { "Evolet", 0.0 },
  { "Evoleth", 0.0 },
  { "Evolett", 0.0 },
  { "Evolette", 0.0 },
  { "Evon", 0.16205803 },
  { "Evona", 0.0 },
  { "Evonda", 0.0 },
  { "Evone", 0.012048192 },
  { "Evoni", 0.0 },
  { "Evonia", 0.0 },
  { "Evonie", 0.0 },
  { "Evonn", 0.0 },
  { "Evonna", 0.0 },
  { "Evonne", 0.0 },
  { "Evonnie", 0.0 },
  { "Evonny", 0.0 },
  { "Evontae", 1.0 },
  { "Evonte", 1.0 },
  { "Evony", 0.0 },
  { "Evora", 0.0 },
  { "Evorn", 0.0 },
  { "Evra", 0.0 },
  { "Evran", 1.0 },
  { "Evren", 0.95527154 },
  { "Evrett", 1.0 },
  { "Evrhett", 1.0 },
  { "Evrin", 1.0 },
  { "Evva", 0.0 },
  { "Evvie", 0.0 },
  { "Evvy", 0.0 },
  { "Evy", 0.0 },
  { "Evyan", 0.0 },
  { "Evyana", 0.0 },
  { "Evyanna", 0.0 },
  { "Evyenia", 0.0 },
  { "Evylen", 0.0 },
  { "Evylene", 0.0 },
  { "Evylin", 0.0 },
  { "Evyn", 0.44585988 },
  { "Evynn", 0.021645023 },
  { "Evynne", 0.0 },
  { "Evyonne", 0.0 },
  { "Ewa", 0.0 },
  { "Ewald", 1.0 },
  { "Ewalt", 1.0 },
  { "Ewan", 1.0 },
  { "Ewanda", 0.0 },
  { "Ewaoluwa", 0.0 },
  { "Eward", 1.0 },
  { "Ewart", 1.0 },
  { "Ewel", 1.0 },
  { "Ewelina", 0.0 },
  { "Ewell", 0.99659866 },
  { "Ewen", 1.0 },
  { "Ewin", 1.0 },
  { "Ewing", 1.0 },
  { "Exa", 0.0 },
  { "Exael", 1.0 },
  { "Exander", 1.0 },
  { "Exar", 1.0 },
  { "Exavier", 1.0 },
  { "Exavion", 1.0 },
  { "Exavior", 1.0 },
  { "Excel", 0.36666667 },
  { "Excell", 0.9880096 },
  { "Exel", 1.0 },
  { "Exell", 1.0 },
  { "Exequiel", 1.0 },
  { "Exer", 0.0 },
  { "Exia", 0.0 },
  { "Exie", 0.0029112082 },
  { "Exilda", 0.0 },
  { "Exiquio", 1.0 },
  { "Exodus", 0.8547297 },
  { "Exton", 1.0 },
  { "Exum", 1.0 },
  { "Exzander", 1.0 },
  { "Exzavier", 1.0 },
  { "Exzavion", 1.0 },
  { "Exzavior", 1.0 },
  { "Eya", 0.0 },
  { "Eyad", 1.0 },
  { "Eyal", 1.0 },
  { "Eyan", 1.0 },
  { "Eyana", 0.0 },
  { "Eyani", 0.0 },
  { "Eyanna", 0.0 },
  { "Eyas", 1.0 },
  { "Eyasu", 1.0 },
  { "Eydan", 1.0 },
  { "Eyden", 0.9764151 },
  { "Eydi", 0.0 },
  { "Eydie", 0.0 },
  { "Eyen", 1.0 },
  { "Eygpt", 0.0 },
  { "Eyian", 1.0 },
  { "Eyla", 0.0 },
  { "Eylah", 0.0 },
  { "Eylan", 1.0 },
  { "Eyleen", 0.0 },
  { "Eylin", 0.0 },
  { "Eylul", 0.0 },
  { "Eymen", 1.0 },
  { "Eymi", 0.0 },
  { "Eymy", 0.0 },
  { "Eyoab", 1.0 },
  { "Eyob", 1.0 },
  { "Eyoel", 1.0 },
  { "Eyon", 1.0 },
  { "Eyona", 0.0 },
  { "Eyonna", 0.0 },
  { "Eyosias", 1.0 },
  { "Eyra", 0.0 },
  { "Eyram", 0.3125 },
  { "Eytan", 1.0 },
  { "Eythan", 1.0 },
  { "Eythel", 0.0 },
  { "Eythen", 1.0 },
  { "Eyuel", 1.0 },
  { "Eyva", 0.0 },
  { "Eyvan", 1.0 },
  { "Eyvette", 0.0 },
  { "Eyvin", 1.0 },
  { "Eyvone", 0.0 },
  { "Eyvonne", 0.0 },
  { "Ezaan", 1.0 },
  { "Ezabel", 0.0 },
  { "Ezabella", 0.0 },
  { "Ezabelle", 0.0 },
  { "Ezael", 1.0 },
  { "Ezai", 1.0 },
  { "Ezaiah", 1.0 },
  { "Ezana", 1.0 },
  { "Ezara", 0.0 },
  { "Ezaria", 0.0 },
  { "Ezariah", 0.6969697 },
  { "Ezariyah", 0.0 },
  { "Ezavier", 1.0 },
  { "Ezayah", 1.0 },
  { "Eze", 1.0 },
  { "Ezechiel", 1.0 },
  { "Ezekeal", 1.0 },
  { "Ezekeial", 1.0 },
  { "Ezekeil", 1.0 },
  { "Ezekiah", 1.0 },
  { "Ezekial", 1.0 },
  { "Ezekias", 1.0 },
  { "Ezekiel", 0.99964136 },
  { "Ezekio", 1.0 },
  { "Ezekyel", 1.0 },
  { "Ezel", 1.0 },
  { "Ezell", 0.88630706 },
  { "Ezella", 0.0 },
  { "Ezelle", 0.3508772 },
  { "Ezequel", 1.0 },
  { "Ezequias", 1.0 },
  { "Ezequiel", 1.0 },
  { "Ezequil", 1.0 },
  { "Ezera", 1.0 },
  { "Ezeriah", 1.0 },
  { "Ezgi", 0.0 },
  { "Eziah", 0.97619045 },
  { "Ezias", 1.0 },
  { "Ezichi", 0.0 },
  { "Eziel", 1.0 },
  { "Ezikiel", 1.0 },
  { "Ezinne", 0.0 },
  { "Ezio", 1.0 },
  { "Ezion", 1.0 },
  { "Eziquel", 1.0 },
  { "Eziquiel", 1.0 },
  { "Eziquio", 1.0 },
  { "Eziya", 0.0 },
  { "Eziyah", 0.0 },
  { "Ezkiel", 1.0 },
  { "Ezlynn", 0.0 },
  { "Ezma", 0.0 },
  { "Ezmae", 0.0 },
  { "Ezmariah", 0.0 },
  { "Ezmay", 0.0 },
  { "Ezme", 0.0 },
  { "Ezmeralda", 0.0 },
  { "Ezmerelda", 0.0 },
  { "Ezmie", 0.0 },
  { "Ezola", 0.0 },
  { "Ezora", 0.0 },
  { "Ezoza", 0.0 },
  { "Ezperanza", 0.0 },
  { "Ezquiel", 1.0 },
  { "Ezra", 0.9569183 },
  { "Ezrael", 1.0 },
  { "Ezrah", 0.7786017 },
  { "Ezreal", 1.0 },
  { "Ezren", 1.0 },
  { "Ezri", 0.15 },
  { "Ezria", 0.0 },
  { "Ezriah", 1.0 },
  { "Ezrie", 0.0 },
  { "Ezriel", 1.0 },
  { "Eztli", 0.0 },
  { "Ezykiel", 1.0 },
  { "Ezzah", 0.0 },
  { "Ezzard", 1.0 },
  { "Ezzie", 0.13194445 },
  { "Faaiz", 1.0 },
  { "Faaizah", 0.0 },
  { "Faaris", 1.0 },
  { "Faatima", 0.0 },
  { "Faatimah", 0.0 },
  { "Faaz", 1.0 },
  { "Fabain", 1.0 },
  { "Faber", 1.0 },
  { "Fabia", 0.0 },
  { "Fabian", 0.99139816 },
  { "Fabiana", 0.0 },
  { "Fabianna", 0.0 },
  { "Fabiano", 1.0 },
  { "Fabien", 1.0 },
  { "Fabiene", 0.0 },
  { "Fabienne", 0.0 },
  { "Fabiha", 0.0 },
  { "Fabin", 1.0 },
  { "Fabio", 1.0 },
  { "Fabiola", 0.006010433 },
  { "Fabion", 1.0 },
  { "Fabius", 1.0 },
  { "Fable", 0.04950495 },
  { "Fabriana", 0.0 },
  { "Fabrice", 1.0 },
  { "Fabricio", 1.0 },
  { "Fabrienne", 0.0 },
  { "Fabrisio", 1.0 },
  { "Fabrizio", 1.0 },
  { "Fabrizzio", 1.0 },
  { "Faby", 0.0 },
  { "Fabyan", 1.0 },
  { "Fachon", 0.0 },
  { "Facundo", 1.0 },
  { "Fada", 0.0 },
  { "Fadak", 0.0 },
  { "Fadeelah", 0.0 },
  { "Fadel", 1.0 },
  { "Fadi", 1.0 },
  { "Fadia", 0.0 },
  { "Fadil", 1.0 },
  { "Fadila", 0.0 },
  { "Fadima", 0.0 },
  { "Fadra", 0.0 },
  { "Faduma", 0.0 },
  { "Fadumo", 0.0 },
  { "Fadwa", 0.0 },
  { "Fady", 1.0 },
  { "Fae", 0.0 },
  { "Faedra", 0.0 },
  { "Faelan", 0.8333333 },
  { "Faelyn", 0.0 },
  { "Faelynn", 0.0 },
  { "Faeryn", 0.0 },
  { "Fahad", 1.0 },
  { "Fahd", 1.0 },
  { "Fahed", 1.0 },
  { "Faheem", 1.0 },
  { "Faheemah", 0.0 },
  { "Fahim", 1.0 },
  { "Fahima", 0.0 },
  { "Fahm", 0.0 },
  { "Fahmi", 1.0 },
  { "Fahmida", 0.0 },
  { "Fahren", 0.0 },
  { "Fahtima", 0.0 },
  { "Faida", 0.0 },
  { "Faiga", 0.0 },
  { "Faige", 0.0 },
  { "Faigy", 0.0 },
  { "Faiha", 0.0 },
  { "Fain", 0.9220779 },
  { "Faina", 0.0 },
  { "Faiq", 1.0 },
  { "Fair", 0.10869565 },
  { "Faira", 0.0 },
  { "Fairah", 0.0 },
  { "Fairbanks", 1.0 },
  { "Fairen", 0.0 },
  { "Fairie", 0.0 },
  { "Fairley", 1.0 },
  { "Fairy", 0.0 },
  { "Fairyn", 0.0 },
  { "Faisa", 0.0 },
  { "Faisal", 1.0 },
  { "Faison", 1.0 },
  { "Faith", 0.001743885 },
  { "Faithann", 0.0 },
  { "Faithanne", 0.0 },
  { "Faithe", 0.0 },
  { "Faithful", 0.0 },
  { "Faithlyn", 0.0 },
  { "Faithlynn", 0.0 },
  { "Faithmarie", 0.0 },
  { "Faiyaz", 1.0 },
  { "Faiz", 1.0 },
  { "Faiza", 0.0 },
  { "Faizaan", 1.0 },
  { "Faizah", 0.0 },
  { "Faizal", 1.0 },
  { "Faizan", 1.0 },
  { "Faizon", 1.0 },
  { "Fajar", 0.0 },
  { "Fajr", 0.0 },
  { "Falak", 0.0 },
  { "Falan", 0.0 },
  { "Falana", 0.0 },
  { "Falanda", 0.0 },
  { "Falando", 1.0 },
  { "Falba", 0.0 },
  { "Falco", 1.0 },
  { "Falcon", 0.93352604 },
  { "Falecia", 0.0 },
  { "Faleisha", 0.0 },
  { "Falen", 0.0 },
  { "Falena", 0.0 },
  { "Falencia", 0.0 },
  { "Falesha", 0.0 },
  { "Faleshia", 0.0 },
  { "Falicia", 0.0 },
  { "Falicity", 0.0 },
  { "Falin", 0.0 },
  { "Falina", 0.0 },
  { "Faline", 0.0 },
  { "Falisa", 0.0 },
  { "Falisha", 0.0 },
  { "Falishia", 0.0 },
  { "Fallan", 0.0 },
  { "Fallen", 0.0 },
  { "Fallie", 0.0 },
  { "Fallin", 0.0 },
  { "Fallon", 0.043010753 },
  { "Fallou", 1.0 },
  { "Fallyn", 0.0 },
  { "Fallynn", 0.0 },
  { "Falon", 0.042783506 },
  { "Falona", 0.0 },
  { "Falonda", 0.0 },
  { "Falone", 0.0 },
  { "Falyn", 0.0 },
  { "Falynn", 0.0 },
  { "Falynne", 0.0 },
  { "Falysha", 0.0 },
  { "Fama", 0.0 },
  { "Fame", 0.0 },
  { "Famie", 0.0 },
  { "Famous", 1.0 },
  { "Fan", 0.0 },
  { "Fana", 0.0 },
  { "Fanchon", 0.0 },
  { "Fanci", 0.0 },
  { "Fancie", 0.0 },
  { "Fancy", 0.0 },
  { "Faneisha", 0.0 },
  { "Fanessa", 0.0 },
  { "Fani", 0.0 },
  { "Fania", 0.0 },
  { "Fanie", 0.0 },
  { "Fanisha", 0.0 },
  { "Fanita", 0.0 },
  { "Fanna", 0.0 },
  { "Fannette", 0.0 },
  { "Fannie", 0.0043294616 },
  { "Fanniemae", 0.0 },
  { "Fanny", 0.0 },
  { "Fannye", 0.0 },
  { "Fanon", 1.0 },
  { "Fanta", 0.0 },
  { "Fantaisa", 0.0 },
  { "Fantasha", 0.0 },
  { "Fantashia", 0.0 },
  { "Fantasia", 0.0 },
  { "Fantasy", 0.0 },
  { "Fantaysia", 0.0 },
  { "Fantazia", 0.0 },
  { "Fanuel", 1.0 },
  { "Fany", 0.0 },
  { "Fanya", 0.0 },
  { "Faolan", 1.0 },
  { "Faora", 0.0 },
  { "Fara", 0.0 },
  { "Faraaz", 1.0 },
  { "Farad", 1.0 },
  { "Faraday", 0.0 },
  { "Farah", 0.018200202 },
  { "Farai", 1.0 },
  { "Faraj", 1.0 },
  { "Faraji", 1.0 },
  { "Faran", 0.2 },
  { "Faraz", 1.0 },
  { "Farbod", 1.0 },
  { "Fard", 1.0 },
  { "Fardeen", 1.0 },
  { "Fardi", 1.0 },
  { "Fardin", 1.0 },
  { "Fardosa", 0.0 },
  { "Fardowsa", 0.0 },
  { "Fardy", 1.0 },
  { "Fareed", 1.0 },
  { "Fareeda", 0.0 },
  { "Fareedah", 0.0 },
  { "Fareeha", 0.0 },
  { "Faren", 0.0 },
  { "Fares", 1.0 },
  { "Farha", 0.0 },
  { "Farhaan", 1.0 },
  { "Farhad", 1.0 },
  { "Farhan", 1.0 },
  { "Farhana", 0.0 },
  { "Farheen", 0.0 },
  { "Farhiya", 0.0 },
  { "Farhiyo", 0.0 },
  { "Fari", 0.0 },
  { "Faria", 0.0 },
  { "Fariah", 0.0 },
  { "Fariba", 0.0 },
  { "Farica", 0.0 },
  { "Farice", 0.0 },
  { "Farid", 1.0 },
  { "Farida", 0.0 },
  { "Faridah", 0.0 },
  { "Fariha", 0.0 },
  { "Farihah", 0.0 },
  { "Farin", 0.1491228 },
  { "Farina", 0.0 },
  { "Faris", 0.9908592 },
  { "Farisa", 0.0 },
  { "Fariz", 1.0 },
  { "Fariza", 0.0 },
  { "Farjana", 0.0 },
  { "Farley", 0.9906367 },
  { "Farm", 0.0 },
  { "Farmer", 1.0 },
  { "Farnaz", 0.0 },
  { "Farnum", 1.0 },
  { "Farol", 0.0 },
  { "Faron", 0.97806835 },
  { "Farooq", 1.0 },
  { "Farouk", 1.0 },
  { "Farouq", 1.0 },
  { "Farra", 0.0 },
  { "Farrad", 1.0 },
  { "Farrah", 0.000680206 },
  { "Farrakhan", 1.0 },
  { "Farran", 0.0 },
  { "Farrar", 0.0 },
  { "Farrel", 0.9562716 },
  { "Farrell", 0.90104824 },
  { "Farren", 0.3024055 },
  { "Farrie", 0.0 },
  { "Farrin", 0.115384616 },
  { "Farris", 0.8501917 },
  { "Farron", 0.83406115 },
  { "Farrow", 1.0 },
  { "Farrukh", 1.0 },
  { "Farryn", 0.0 },
  { "Farshad", 1.0 },
  { "Farshid", 1.0 },
  { "Fartun", 0.0 },
  { "Faruk", 1.0 },
  { "Faruq", 1.0 },
  { "Farwa", 0.0 },
  { "Faryal", 0.0 },
  { "Faryl", 0.0 },
  { "Faryn", 0.0 },
  { "Farynn", 0.0 },
  { "Farzad", 1.0 },
  { "Farzan", 1.0 },
  { "Farzana", 0.0 },
  { "Farzeen", 0.0 },
  { "Farzin", 1.0 },
  { "Farzona", 0.0 },
  { "Fasha", 0.0 },
  { "Fashionette", 0.0 },
  { "Fatai", 1.0 },
  { "Fate", 0.9064465 },
  { "Fateema", 0.0 },
  { "Fateemah", 0.0 },
  { "Fateen", 1.0 },
  { "Fateh", 1.0 },
  { "Fateha", 0.0 },
  { "Fatema", 0.0 },
  { "Fatemah", 0.0 },
  { "Fatemeh", 0.0 },
  { "Faten", 0.0 },
  { "Fathi", 0.8108108 },
  { "Fathia", 0.0 },
  { "Fathima", 0.0 },
  { "Fathma", 0.0 },
  { "Fatia", 0.0 },
  { "Fatih", 1.0 },
  { "Fatiha", 0.0 },
  { "Fatim", 0.0 },
  { "Fatima", 0.0015038577 },
  { "Fatimah", 0.0 },
  { "Fatimat", 0.0 },
  { "Fatimata", 0.0 },
  { "Fatimatou", 0.0 },
  { "Fatimazahra", 0.0 },
  { "Fatime", 0.0 },
  { "Fatimetou", 0.0 },
  { "Fatin", 0.14492753 },
  { "Fatina", 0.0 },
  { "Fatisha", 0.0 },
  { "Fatma", 0.0 },
  { "Fatmah", 0.0 },
  { "Fatmata", 0.0 },
  { "Fatme", 0.0 },
  { "Fatmeh", 0.0 },
  { "Fatmir", 1.0 },
  { "Fatmire", 0.0 },
  { "Fatou", 0.0 },
  { "Fatouma", 0.0 },
  { "Fatoumata", 0.0 },
  { "Fatoumatta", 0.0 },
  { "Fattima", 0.0 },
  { "Fatu", 0.0 },
  { "Fatuma", 0.0 },
  { "Fatumata", 0.0 },
  { "Fatumo", 0.0 },
  { "Fatyma", 0.0 },
  { "Faun", 0.0 },
  { "Fauna", 0.0 },
  { "Faune", 0.0 },
  { "Fauniel", 0.0 },
  { "Faust", 1.0 },
  { "Fausta", 0.0 },
  { "Faustin", 1.0 },
  { "Faustina", 0.0 },
  { "Faustine", 0.0 },
  { "Faustino", 1.0 },
  { "Fausto", 1.0 },
  { "Fauzia", 0.0 },
  { "Faven", 0.0 },
  { "Favian", 1.0 },
  { "Faviana", 0.0 },
  { "Favio", 1.0 },
  { "Faviola", 0.0 },
  { "Favion", 1.0 },
  { "Favor", 0.17613636 },
  { "Favour", 0.10451978 },
  { "Fawad", 1.0 },
  { "Fawaz", 1.0 },
  { "Fawkes", 1.0 },
  { "Fawn", 0.0 },
  { "Fawna", 0.0 },
  { "Fawnda", 0.0 },
  { "Fawne", 0.0 },
  { "Fawnia", 0.0 },
  { "Fawnna", 0.0 },
  { "Fawwaz", 1.0 },
  { "Fawzan", 1.0 },
  { "Fawzi", 1.0 },
  { "Fawzia", 0.0 },
  { "Fay", 0.11488447 },
  { "Faya", 0.0 },
  { "Fayann", 0.0 },
  { "Fayaz", 1.0 },
  { "Faybian", 1.0 },
  { "Faydean", 0.0 },
  { "Faydell", 0.0 },
  { "Fayden", 1.0 },
  { "Faydene", 0.0 },
  { "Faydra", 0.0 },
  { "Faye", 0.0075943056 },
  { "Fayelyn", 0.0 },
  { "Fayelynn", 0.0 },
  { "Fayerene", 0.0 },
  { "Fayeth", 0.0 },
  { "Fayetta", 0.0 },
  { "Fayette", 0.43067846 },
  { "Fayez", 1.0 },
  { "Fayga", 0.0 },
  { "Fayha", 0.0 },
  { "Fayla", 0.0 },
  { "Faylee", 0.0 },
  { "Fayleen", 0.0 },
  { "Faylene", 0.0 },
  { "Faylin", 0.0 },
  { "Faylinn", 0.0 },
  { "Faylyn", 0.0 },
  { "Faylynn", 0.0 },
  { "Fayma", 0.0 },
  { "Fayne", 0.3529412 },
  { "Faynell", 0.0 },
  { "Fayola", 0.0 },
  { "Fayrene", 0.0 },
  { "Fayrouz", 0.0 },
  { "Faysal", 1.0 },
  { "Fayt", 1.0 },
  { "Fayte", 0.0 },
  { "Fayth", 0.0 },
  { "Faythe", 0.0 },
  { "Fayza", 0.0 },
  { "Fazal", 1.0 },
  { "Fazon", 1.0 },
  { "Fe", 0.0 },
  { "Feather", 0.0 },
  { "Febbie", 0.0 },
  { "Febe", 0.0 },
  { "Feben", 0.0 },
  { "Febie", 0.0 },
  { "Febra", 0.0 },
  { "Fed", 1.0 },
  { "Feda", 0.0 },
  { "Fedaa", 0.0 },
  { "Feddie", 0.0 },
  { "Fedel", 1.0 },
  { "Fedele", 1.0 },
  { "Fedelia", 0.0 },
  { "Fedelina", 0.0 },
  { "Federica", 0.0 },
  { "Federick", 1.0 },
  { "Federico", 0.999401 },
  { "Fedor", 1.0 },
  { "Fedora", 0.0 },
  { "Fedra", 0.0 },
  { "Fedrick", 1.0 },
  { "Feffrey", 1.0 },
  { "Feiga", 0.0 },
  { "Feige", 0.0 },
  { "Feigy", 0.0 },
  { "Feish", 1.0 },
  { "Fela", 0.12820514 },
  { "Felando", 1.0 },
  { "Felcia", 0.0 },
  { "Felder", 1.0 },
  { "Felechia", 0.0 },
  { "Felecia", 0.0 },
  { "Felecie", 0.0 },
  { "Felecity", 0.0 },
  { "Feleica", 0.0 },
  { "Feleicia", 0.0 },
  { "Feleisha", 0.0 },
  { "Felena", 0.0 },
  { "Felesha", 0.0 },
  { "Feleshia", 0.0 },
  { "Felesia", 0.0 },
  { "Feliberto", 1.0 },
  { "Felica", 0.0 },
  { "Felice", 0.14271027 },
  { "Felicea", 0.0 },
  { "Felicha", 0.0 },
  { "Felichia", 0.0 },
  { "Felicia", 0.0030643851 },
  { "Feliciana", 0.0 },
  { "Feliciano", 1.0 },
  { "Felicidad", 0.0 },
  { "Felicie", 0.0 },
  { "Felicita", 0.0 },
  { "Felicitas", 0.0 },
  { "Felicite", 0.0 },
  { "Feliciti", 0.0 },
  { "Felicity", 0.00051475636 },
  { "Feliks", 1.0 },
  { "Felimon", 1.0 },
  { "Felina", 0.0 },
  { "Felinda", 0.0 },
  { "Felipa", 0.0 },
  { "Felipe", 0.99549043 },
  { "Felipedejesus", 1.0 },
  { "Felipita", 0.0 },
  { "Felis", 0.59615386 },
  { "Felisa", 0.0 },
  { "Feliscia", 0.0 },
  { "Felise", 0.0 },
  { "Felisha", 0.0 },
  { "Felishia", 0.0 },
  { "Felisia", 0.0 },
  { "Felisiano", 1.0 },
  { "Felisita", 0.0 },
  { "Felisitas", 0.0 },
  { "Felisity", 0.0 },
  { "Felissa", 0.0 },
  { "Felissia", 0.0 },
  { "Felita", 0.0 },
  { "Felix", 0.9915717 },
  { "Feliz", 0.4334828 },
  { "Feliza", 0.0 },
  { "Felizardo", 1.0 },
  { "Felizia", 0.0 },
  { "Fellicia", 0.0 },
  { "Fellipe", 1.0 },
  { "Felma", 0.0 },
  { "Feloniz", 0.0 },
  { "Felton", 0.9992336 },
  { "Feltus", 1.0 },
  { "Felty", 1.0 },
  { "Felycia", 0.0 },
  { "Felyx", 1.0 },
  { "Female", 0.00385505 },
  { "Femi", 0.5294118 },
  { "Femke", 0.0 },
  { "Fender", 1.0 },
  { "Fendi", 0.0 },
  { "Fenet", 0.0 },
  { "Feng", 1.0 },
  { "Fenisha", 0.0 },
  { "Fenix", 0.77022654 },
  { "Fenley", 0.0 },
  { "Fenn", 1.0 },
  { "Fenna", 0.0 },
  { "Fennec", 1.0 },
  { "Fennell", 1.0 },
  { "Fenner", 1.0 },
  { "Fennie", 0.0 },
  { "Fenrir", 1.0 },
  { "Fenris", 1.0 },
  { "Fenton", 1.0 },
  { "Fentress", 1.0 },
  { "Fenway", 1.0 },
  { "Fenwick", 1.0 },
  { "Feodor", 1.0 },
  { "Feodora", 0.0 },
  { "Feona", 0.0 },
  { "Ferando", 1.0 },
  { "Feras", 1.0 },
  { "Ferd", 1.0 },
  { "Ferdie", 1.0 },
  { "Ferdinan", 1.0 },
  { "Ferdinand", 1.0 },
  { "Ferdinando", 1.0 },
  { "Ferdy", 1.0 },
  { "Ferdynand", 1.0 },
  { "Ferenc", 1.0 },
  { "Fergie", 0.0 },
  { "Fergus", 1.0 },
  { "Ferguson", 1.0 },
  { "Ferid", 1.0 },
  { "Feride", 0.0 },
  { "Ferin", 0.0 },
  { "Ferlando", 1.0 },
  { "Ferlin", 1.0 },
  { "Ferma", 0.0 },
  { "Ferman", 1.0 },
  { "Fermen", 1.0 },
  { "Fermin", 1.0 },
  { "Fermina", 0.0 },
  { "Fermon", 1.0 },
  { "Fern", 0.013768803 },
  { "Ferna", 0.0 },
  { "Fernado", 1.0 },
  { "Fernan", 1.0 },
  { "Fernand", 1.0 },
  { "Fernanda", 0.0017800904 },
  { "Fernande", 0.0 },
  { "Fernandeo", 1.0 },
  { "Fernandez", 1.0 },
  { "Fernando", 0.9938296 },
  { "Ferne", 0.0 },
  { "Fernell", 0.54545456 },
  { "Fernetta", 0.0 },
  { "Ferney", 1.0 },
  { "Fernie", 0.9690141 },
  { "Fernley", 1.0 },
  { "Ferol", 0.015923567 },
  { "Feroz", 1.0 },
  { "Ferra", 0.0 },
  { "Ferrah", 0.0 },
  { "Ferran", 0.5 },
  { "Ferrante", 1.0 },
  { "Ferrari", 0.13461539 },
  { "Ferras", 1.0 },
  { "Ferrel", 0.9448621 },
  { "Ferrell", 0.9218524 },
  { "Ferren", 0.0 },
  { "Ferrie", 0.0 },
  { "Ferril", 1.0 },
  { "Ferrill", 1.0 },
  { "Ferrin", 0.23076923 },
  { "Ferris", 0.9073189 },
  { "Ferrol", 0.4054054 },
  { "Ferron", 0.8938053 },
  { "Ferry", 0.85714287 },
  { "Ferryn", 0.0 },
  { "Feryal", 0.0 },
  { "Feryl", 0.0 },
  { "Fess", 1.0 },
  { "Fester", 1.0 },
  { "Festus", 1.0 },
  { "Fetina", 0.0 },
  { "Feven", 0.0 },
  { "Fey", 0.0 },
  { "Feyisayo", 0.0 },
  { "Feynman", 1.0 },
  { "Feysal", 1.0 },
  { "Feyza", 0.0 },
  { "Fher", 1.0 },
  { "Fhernando", 1.0 },
  { "Fia", 0.0 },
  { "Fiala", 0.0 },
  { "Fiama", 0.0 },
  { "Fiamma", 0.0 },
  { "Fiana", 0.0 },
  { "Fianna", 0.0 },
  { "Fibbie", 0.0 },
  { "Fidel", 1.0 },
  { "Fidela", 0.0 },
  { "Fidelia", 0.0 },
  { "Fidelina", 0.0 },
  { "Fidelio", 1.0 },
  { "Fidelis", 0.3006135 },
  { "Fidella", 0.0 },
  { "Fidelmar", 1.0 },
  { "Fidencia", 0.0 },
  { "Fidencio", 1.0 },
  { "Fidensio", 1.0 },
  { "Field", 1.0 },
  { "Fielden", 1.0 },
  { "Fielder", 1.0 },
  { "Fielding", 1.0 },
  { "Fieldon", 1.0 },
  { "Fields", 1.0 },
  { "Fiera", 0.0 },
  { "Fierra", 0.0 },
  { "Fiesta", 0.0 },
  { "Fiffany", 0.0 },
  { "Fifi", 0.0 },
  { "Fiifi", 1.0 },
  { "Fikir", 0.0 },
  { "Fikisha", 0.0 },
  { "Filadelfio", 1.0 },
  { "Filamena", 0.0 },
  { "Filbert", 1.0 },
  { "Filberto", 1.0 },
  { "Filemon", 1.0 },
  { "Filex", 1.0 },
  { "Fili", 1.0 },
  { "Filiberto", 1.0 },
  { "Filicia", 0.0 },
  { "Filicity", 0.0 },
  { "Filimon", 1.0 },
  { "Filip", 1.0 },
  { "Filipa", 0.0 },
  { "Filipe", 1.0 },
  { "Filippa", 0.0 },
  { "Filippo", 1.0 },
  { "Filippos", 1.0 },
  { "Filisha", 0.0 },
  { "Filiz", 0.0 },
  { "Fillip", 1.0 },
  { "Fillmore", 1.0 },
  { "Filmon", 1.0 },
  { "Filmore", 1.0 },
  { "Filomena", 0.0 },
  { "Filomeno", 1.0 },
  { "Filomina", 0.0 },
  { "Filsan", 0.0 },
  { "Filza", 0.0 },
  { "Fin", 1.0 },
  { "Fina", 0.0 },
  { "Finas", 1.0 },
  { "Finau", 0.54545456 },
  { "Finbar", 1.0 },
  { "Finbarr", 1.0 },
  { "Finch", 1.0 },
  { "Findlay", 0.863388 },
  { "Findley", 0.7167382 },
  { "Fineas", 1.0 },
  { "Finely", 0.0 },
  { "Fines", 1.0 },
  { "Finessa", 0.0 },
  { "Finesse", 0.08018868 },
  { "Finest", 1.0 },
  { "Finian", 1.0 },
  { "Finis", 0.9977273 },
  { "Finland", 1.0 },
  { "Finlay", 0.83074933 },
  { "Finlea", 0.0 },
  { "Finlee", 0.104477614 },
  { "Finleigh", 0.0 },
  { "Finley", 0.44458798 },
  { "Finly", 1.0 },
  { "Finn", 0.9849169 },
  { "Finna", 0.0 },
  { "Finnan", 1.0 },
  { "Finnbar", 1.0 },
  { "Finnean", 1.0 },
  { "Finneas", 1.0 },
  { "Finnegan", 0.9932976 },
  { "Finneus", 1.0 },
  { "Finnian", 1.0 },
  { "Finnick", 1.0 },
  { "Finnie", 0.8333333 },
  { "Finnigan", 1.0 },
  { "Finnis", 1.0 },
  { "Finnlay", 1.0 },
  { "Finnlee", 0.5591398 },
  { "Finnleigh", 0.0 },
  { "Finnley", 0.6434136 },
  { "Finnly", 0.67391306 },
  { "Finola", 0.0 },
  { "Fintan", 1.0 },
  { "Finton", 1.0 },
  { "Finus", 1.0 },
  { "Fion", 0.0 },
  { "Fiona", 0.0002055414 },
  { "Fionn", 1.0 },
  { "Fionna", 0.0 },
  { "Fionnlagh", 1.0 },
  { "Fionnuala", 0.0 },
  { "Fionnula", 0.0 },
  { "Fiora", 0.0 },
  { "Fiore", 0.97689074 },
  { "Fiorela", 0.0 },
  { "Fiorella", 0.0 },
  { "Fiorenza", 0.0 },
  { "Fiori", 0.3529412 },
  { "Fira", 0.0 },
  { "Firas", 1.0 },
  { "Firdaus", 0.0 },
  { "Firdavs", 1.0 },
  { "Firdaws", 0.0 },
  { "Firdous", 0.0 },
  { "Firman", 1.0 },
  { "Firmin", 1.0 },
  { "Firyal", 0.0 },
  { "Fischer", 0.99448276 },
  { "Fishel", 1.0 },
  { "Fisher", 0.9946065 },
  { "Fisnik", 1.0 },
  { "Fitima", 0.0 },
  { "Fitz", 1.0 },
  { "Fitzgerald", 1.0 },
  { "Fitzhugh", 1.0 },
  { "Fitzpatrick", 1.0 },
  { "Fitzroy", 1.0 },
  { "Fitzwilliam", 1.0 },
  { "Fiyinfoluwa", 0.31578946 },
  { "Fiza", 0.0 },
  { "Fizza", 0.0 },
  { "Fjolla", 0.0 },
  { "Flabio", 1.0 },
  { "Flake", 1.0 },
  { "Flame", 0.0 },
  { "Flamur", 1.0 },
  { "Flannery", 0.0 },
  { "Flara", 0.0 },
  { "Flarence", 0.0 },
  { "Flarrie", 0.0 },
  { "Flash", 1.0 },
  { "Flaura", 0.0 },
  { "Flava", 0.0 },
  { "Flavel", 1.0 },
  { "Flavia", 0.0 },
  { "Flavian", 1.0 },
  { "Flavil", 1.0 },
  { "Flavio", 1.0 },
  { "Flavious", 1.0 },
  { "Flavis", 1.0 },
  { "Flavius", 1.0 },
  { "Flay", 0.9411765 },
  { "Flechia", 0.0 },
  { "Flecia", 0.0 },
  { "Fleda", 0.0 },
  { "Fleet", 1.0 },
  { "Fleeta", 0.0 },
  { "Fleetwood", 1.0 },
  { "Flem", 1.0 },
  { "Fleming", 1.0 },
  { "Flemmie", 0.0 },
  { "Flemming", 1.0 },
  { "Flemon", 1.0 },
  { "Flerida", 0.0 },
  { "Flesha", 0.0 },
  { "Fleshia", 0.0 },
  { "Fleta", 0.0 },
  { "Fletcher", 0.9905297 },
  { "Fleur", 0.0 },
  { "Fleurette", 0.0 },
  { "Flex", 1.0 },
  { "Flicia", 0.0 },
  { "Flint", 1.0 },
  { "Flip", 1.0 },
  { "Flo", 0.0 },
  { "Floda", 0.0 },
  { "Flodie", 0.0 },
  { "Floe", 0.0 },
  { "Floella", 0.0 },
  { "Floetta", 0.0 },
  { "Floice", 0.0 },
  { "Floid", 1.0 },
  { "Floie", 0.0 },
  { "Flois", 0.0 },
  { "Flona", 0.0 },
  { "Flonnie", 0.0 },
  { "Flor", 0.013372093 },
  { "Flora", 0.0025343313 },
  { "Florabel", 0.0 },
  { "Florabell", 0.0 },
  { "Florabelle", 0.0 },
  { "Floraida", 0.0 },
  { "Floraine", 0.0 },
  { "Floral", 0.0 },
  { "Floralee", 0.0 },
  { "Floramae", 0.0 },
  { "Floran", 1.0 },
  { "Florance", 0.0 },
  { "Florann", 0.0 },
  { "Floranne", 0.0 },
  { "Florastine", 0.0 },
  { "Flord", 1.0 },
  { "Flordemaria", 0.0 },
  { "Flordia", 0.0 },
  { "Flore", 0.0 },
  { "Florean", 0.0 },
  { "Florece", 0.0 },
  { "Florecita", 0.0 },
  { "Floreda", 0.0 },
  { "Floree", 0.0 },
  { "Floreen", 0.0 },
  { "Floreine", 0.0 },
  { "Florella", 0.0 },
  { "Floren", 1.0 },
  { "Florena", 0.0 },
  { "Florence", 0.0042722463 },
  { "Florencia", 0.0 },
  { "Florencio", 1.0 },
  { "Florenda", 0.0 },
  { "Florene", 0.0 },
  { "Florent", 1.0 },
  { "Florentina", 0.0 },
  { "Florentine", 0.017699115 },
  { "Florentino", 1.0 },
  { "Florenz", 0.71875 },
  { "Florenzo", 1.0 },
  { "Flores", 0.6333333 },
  { "Floresita", 0.0 },
  { "Florestela", 0.0 },
  { "Florestine", 0.0 },
  { "Floreta", 0.0 },
  { "Floretha", 0.0 },
  { "Floretta", 0.0 },
  { "Florette", 0.0 },
  { "Flori", 0.0 },
  { "Floria", 0.0 },
  { "Florian", 0.9842362 },
  { "Floriana", 0.0 },
  { "Floribel", 0.0 },
  { "Floriberto", 1.0 },
  { "Florice", 0.0 },
  { "Florida", 0.0014156286 },
  { "Floridalma", 0.0 },
  { "Floride", 0.0 },
  { "Florie", 0.0 },
  { "Florien", 0.0 },
  { "Floriene", 0.0 },
  { "Florin", 1.0 },
  { "Florina", 0.0 },
  { "Florince", 0.0 },
  { "Florinda", 0.0 },
  { "Florindo", 1.0 },
  { "Florine", 0.00044412861 },
  { "Florio", 1.0 },
  { "Floris", 0.031837918 },
  { "Florisa", 0.0 },
  { "Florisel", 0.0 },
  { "Florita", 0.0 },
  { "Flormaria", 0.0 },
  { "Flornce", 0.0 },
  { "Florrie", 0.0 },
  { "Florrine", 0.0 },
  { "Flory", 0.6458333 },
  { "Floryan", 1.0 },
  { "Flosie", 0.0 },
  { "Floss", 0.0 },
  { "Flossie", 0.0016949152 },
  { "Flossy", 0.0 },
  { "Flourish", 0.0 },
  { "Flournoy", 1.0 },
  { "Flower", 0.0 },
  { "Floy", 0.046582058 },
  { "Floya", 0.0 },
  { "Floyce", 0.02268431 },
  { "Floyd", 0.9911194 },
  { "Floyda", 0.0 },
  { "Floyde", 1.0 },
  { "Floydell", 0.0 },
  { "Floydene", 0.0 },
  { "Floydia", 0.0 },
  { "Floye", 0.0 },
  { "Floyed", 1.0 },
  { "Flozell", 1.0 },
  { "Flynn", 0.91861963 },
  { "Flynt", 1.0 },
  { "Foch", 1.0 },
  { "Foday", 1.0 },
  { "Fode", 1.0 },
  { "Foister", 1.0 },
  { "Fola", 0.0 },
  { "Folami", 0.0 },
  { "Folarin", 1.0 },
  { "Folasade", 0.0 },
  { "Folashade", 0.0 },
  { "Foley", 1.0 },
  { "Folia", 0.0 },
  { "Folke", 1.0 },
  { "Foluke", 0.0 },
  { "Fon", 1.0 },
  { "Fonda", 0.0 },
  { "Fong", 0.9836601 },
  { "Fonisha", 0.0 },
  { "Fonnie", 0.20967741 },
  { "Fonta", 0.47826087 },
  { "Fontaine", 0.58484846 },
  { "Fontana", 0.0 },
  { "Fontane", 1.0 },
  { "Fontella", 0.0 },
  { "Fontelle", 0.0 },
  { "Fontilla", 0.0 },
  { "Fonza", 1.0 },
  { "Fonzie", 1.0 },
  { "Fonzo", 1.0 },
  { "Forace", 1.0 },
  { "Foram", 0.0 },
  { "Forbes", 1.0 },
  { "Forbus", 1.0 },
  { "Ford", 0.99917066 },
  { "Forde", 1.0 },
  { "Fordham", 1.0 },
  { "Fordie", 0.0 },
  { "Fordyce", 1.0 },
  { "Forest", 0.97037756 },
  { "Forester", 1.0 },
  { "Forestine", 0.0 },
  { "Forestt", 1.0 },
  { "Forever", 0.037037037 },
  { "Forman", 1.0 },
  { "Formeka", 0.0 },
  { "Forney", 1.0 },
  { "Forrest", 0.98032016 },
  { "Forrester", 1.0 },
  { "Forrestine", 0.0 },
  { "Forrestt", 1.0 },
  { "Forris", 1.0 },
  { "Forster", 1.0 },
  { "Fort", 1.0 },
  { "Fortino", 1.0 },
  { "Fortuna", 0.0 },
  { "Fortunata", 0.0 },
  { "Fortunate", 0.0 },
  { "Fortunato", 1.0 },
  { "Fortune", 0.374269 },
  { "Forum", 0.0 },
  { "Fosteen", 0.0 },
  { "Foster", 0.9885405 },
  { "Fostina", 0.0 },
  { "Fostine", 0.0 },
  { "Fotima", 0.0 },
  { "Fotini", 0.0 },
  { "Fotios", 1.0 },
  { "Fotis", 1.0 },
  { "Foua", 0.0 },
  { "Fouad", 1.0 },
  { "Fount", 1.0 },
  { "Fountain", 1.0 },
  { "Fouzia", 0.0 },
  { "Fowler", 1.0 },
  { "Fox", 0.98655915 },
  { "Foxx", 1.0 },
  { "Foxy", 0.0 },
  { "Foy", 0.83379394 },
  { "Foye", 0.31903484 },
  { "Foyster", 1.0 },
  { "Fozie", 0.0 },
  { "Fraak", 1.0 },
  { "Fraces", 0.0 },
  { "Fracine", 0.0 },
  { "Fracisco", 1.0 },
  { "Fradel", 0.0 },
  { "Frady", 0.0 },
  { "Frager", 1.0 },
  { "Fraida", 0.0 },
  { "Fraidel", 0.0 },
  { "Fraidy", 0.0 },
  { "Frak", 1.0 },
  { "Frampton", 1.0 },
  { "Fran", 0.06772983 },
  { "Franc", 0.61538464 },
  { "Franca", 0.0 },
  { "Franccesca", 0.0 },
  { "France", 0.24428478 },
  { "Francee", 0.0 },
  { "Franceen", 0.0 },
  { "Francel", 0.0 },
  { "Francelia", 0.0 },
  { "Francelina", 0.0 },
  { "Francell", 0.33333334 },
  { "Francella", 0.0 },
  { "Francelle", 0.0 },
  { "Francellia", 0.0 },
  { "Francely", 0.0 },
  { "Francena", 0.0 },
  { "Francene", 0.0 },
  { "Francenia", 0.0 },
  { "Frances", 0.008383074 },
  { "Francesa", 0.0 },
  { "Francesc", 0.3448276 },
  { "Francesca", 0.0011698394 },
  { "Francesco", 1.0 },
  { "Francese", 0.0 },
  { "Franceska", 0.0 },
  { "Francess", 0.0 },
  { "Francessca", 0.0 },
  { "Francetta", 0.0 },
  { "Francey", 0.0 },
  { "Franchell", 0.0 },
  { "Franchelle", 0.0 },
  { "Franchesca", 0.0 },
  { "Franchesco", 1.0 },
  { "Francheska", 0.0 },
  { "Franchessca", 0.0 },
  { "Franchesska", 0.0 },
  { "Franchester", 0.0 },
  { "Franchetta", 0.0 },
  { "Franchette", 0.0 },
  { "Franchezca", 0.0 },
  { "Franchon", 0.0 },
  { "Franchot", 1.0 },
  { "Franci", 0.0 },
  { "Francia", 0.0 },
  { "Francico", 1.0 },
  { "Francie", 0.0032661948 },
  { "Franciene", 0.0 },
  { "Francies", 0.013471503 },
  { "Francile", 0.0 },
  { "Francille", 0.0 },
  { "Francina", 0.0 },
  { "Francine", 0.00030762347 },
  { "Francis", 0.9073856 },
  { "Francisc", 1.0 },
  { "Francisca", 0.0050123255 },
  { "Francisco", 0.9927938 },
  { "Franciscojavier", 1.0 },
  { "Franciscojr", 1.0 },
  { "Franciscus", 1.0 },
  { "Francise", 0.0 },
  { "Franciska", 0.0 },
  { "Franciso", 1.0 },
  { "Francisquita", 0.0 },
  { "Franciszek", 1.0 },
  { "Franciszka", 0.0 },
  { "Francita", 0.0 },
  { "Franck", 1.0 },
  { "Franclin", 1.0 },
  { "Franco", 1.0 },
  { "Francois", 0.99702734 },
  { "Francoise", 0.0 },
  { "Francsico", 1.0 },
  { "Francy", 0.0 },
  { "Francyne", 0.0 },
  { "Francys", 0.0 },
  { "Franda", 0.0 },
  { "Frandy", 1.0 },
  { "Franes", 0.0 },
  { "Franey", 0.0 },
  { "Frani", 0.0 },
  { "Frania", 0.0 },
  { "Franics", 1.0 },
  { "Franis", 1.0 },
  { "Franisco", 1.0 },
  { "Frank", 0.99495643 },
  { "Franka", 0.0 },
  { "Franke", 1.0 },
  { "Frankee", 0.0 },
  { "Frankel", 1.0 },
  { "Frankey", 0.9652778 },
  { "Franki", 0.073737375 },
  { "Frankie", 0.5505101 },
  { "Frankin", 1.0 },
  { "Franklen", 1.0 },
  { "Franklin", 0.9947716 },
  { "Frankline", 1.0 },
  { "Frankly", 1.0 },
  { "Franklyn", 0.9966119 },
  { "Franklynn", 0.0 },
  { "Franko", 1.0 },
  { "Franky", 0.9839858 },
  { "Frankye", 0.022058824 },
  { "Frann", 0.0 },
  { "Franne", 0.0 },
  { "Frannie", 0.0 },
  { "Franny", 0.0 },
  { "Frans", 1.0 },
  { "Franshesca", 0.0 },
  { "Fransheska", 0.0 },
  { "Fransica", 0.0 },
  { "Fransico", 1.0 },
  { "Fransisca", 0.0 },
  { "Fransisco", 1.0 },
  { "Frantasia", 0.0 },
  { "Frantisek", 1.0 },
  { "Frantz", 1.0 },
  { "Frantzy", 1.0 },
  { "Franyelis", 0.0 },
  { "Franz", 1.0 },
  { "Franzetta", 0.0 },
  { "Franziska", 0.0 },
  { "Fraser", 1.0 },
  { "Frasier", 1.0 },
  { "Fray", 1.0 },
  { "Fraya", 0.0 },
  { "Frayah", 0.0 },
  { "Frayda", 0.0 },
  { "Frazer", 1.0 },
  { "Frazier", 1.0 },
  { "Fread", 1.0 },
  { "Freada", 0.0 },
  { "Fred", 0.9941154 },
  { "Freda", 0.00061291846 },
  { "Fredarius", 1.0 },
  { "Fredasia", 0.0 },
  { "Fredda", 0.0 },
  { "Fredderick", 1.0 },
  { "Freddi", 0.5263158 },
  { "Freddia", 0.0 },
  { "Freddick", 1.0 },
  { "Freddie", 0.8761916 },
  { "Freddrick", 1.0 },
  { "Freddy", 0.9932891 },
  { "Freddye", 0.0 },
  { "Fredeline", 0.0 },
  { "Frederi", 1.0 },
  { "Frederic", 1.0 },
  { "Frederica", 0.0 },
  { "Frederich", 1.0 },
  { "Frederick", 0.99577266 },
  { "Fredericka", 0.0 },
  { "Frederico", 1.0 },
  { "Frederik", 1.0 },
  { "Frederika", 0.0 },
  { "Frederique", 0.0 },
  { "Fredi", 0.90611356 },
  { "Fredia", 0.0 },
  { "Fredick", 1.0 },
  { "Fredie", 0.96745324 },
  { "Fredina", 0.0 },
  { "Fredine", 0.0 },
  { "Fredirick", 1.0 },
  { "Fredis", 1.0 },
  { "Fredith", 0.0 },
  { "Fredlyn", 0.0 },
  { "Fredna", 0.0 },
  { "Fredo", 1.0 },
  { "Fredonia", 0.0 },
  { "Fredreck", 1.0 },
  { "Fredreick", 1.0 },
  { "Fredreka", 0.0 },
  { "Fredric", 1.0 },
  { "Fredrica", 0.0 },
  { "Fredrich", 1.0 },
  { "Fredrick", 0.99593335 },
  { "Fredricka", 0.0 },
  { "Fredricks", 1.0 },
  { "Fredrico", 1.0 },
  { "Fredrik", 1.0 },
  { "Fredrika", 0.0 },
  { "Fredy", 1.0 },
  { "Fredys", 1.0 },
  { "Free", 0.47916666 },
  { "Freeda", 0.0 },
  { "Freedia", 0.0 },
  { "Freedom", 0.34763408 },
  { "Freeland", 1.0 },
  { "Freelin", 1.0 },
  { "Freelon", 1.0 },
  { "Freeman", 0.9995322 },
  { "Freemon", 1.0 },
  { "Freemont", 1.0 },
  { "Freesia", 0.0 },
  { "Freida", 0.0 },
  { "Freidy", 0.0 },
  { "Freja", 0.0 },
  { "Freland", 1.0 },
  { "Freman", 1.0 },
  { "Fremon", 1.0 },
  { "Fremont", 1.0 },
  { "French", 0.99114263 },
  { "Frenchie", 0.18888889 },
  { "Frenda", 0.0 },
  { "Frenesi", 0.0 },
  { "Freta", 0.0 },
  { "Frey", 1.0 },
  { "Freya", 0.0 },
  { "Freyah", 0.0 },
  { "Freyda", 0.0 },
  { "Freyja", 0.0 },
  { "Freylin", 1.0 },
  { "Frezell", 1.0 },
  { "Frica", 0.0 },
  { "Frida", 0.0 },
  { "Friday", 0.0 },
  { "Fridda", 0.0 },
  { "Friddie", 1.0 },
  { "Frieda", 0.0 },
  { "Friedrich", 1.0 },
  { "Friel", 1.0 },
  { "Friend", 1.0 },
  { "Frika", 0.0 },
  { "Frimet", 0.0 },
  { "Frimmy", 0.0 },
  { "Frimy", 0.0 },
  { "Frink", 1.0 },
  { "Frisco", 1.0 },
  { "Frita", 0.0 },
  { "Fritha", 0.0 },
  { "Fritz", 1.0 },
  { "Fritzgerald", 1.0 },
  { "Fritzi", 0.0 },
  { "Fritzie", 0.0 },
  { "Fritzy", 0.0 },
  { "Frizell", 1.0 },
  { "Froilan", 1.0 },
  { "Frona", 0.0 },
  { "Froney", 0.0 },
  { "Fronia", 0.0 },
  { "Fronie", 0.0 },
  { "Fronnie", 0.0 },
  { "Frontis", 1.0 },
  { "Frost", 1.0 },
  { "Frosty", 1.0 },
  { "Froy", 1.0 },
  { "Froylan", 1.0 },
  { "Frozine", 0.0 },
  { "Fruma", 0.0 },
  { "Frumencio", 1.0 },
  { "Frumie", 0.0 },
  { "Frutoso", 1.0 },
  { "Fryda", 0.0 },
  { "Fu", 1.0 },
  { "Fuad", 1.0 },
  { "Fue", 1.0 },
  { "Fujie", 0.0 },
  { "Fujiko", 0.0 },
  { "Fujio", 1.0 },
  { "Fujiye", 0.0 },
  { "Fuka", 0.0 },
  { "Fukuichi", 1.0 },
  { "Fulgencio", 1.0 },
  { "Fuller", 1.0 },
  { "Fulton", 1.0 },
  { "Fulvio", 1.0 },
  { "Fumi", 0.0 },
  { "Fumie", 0.0 },
  { "Fumiko", 0.0 },
  { "Fumio", 1.0 },
  { "Fumiye", 0.0 },
  { "Fumiyo", 0.0 },
  { "Funda", 0.0 },
  { "Funmilayo", 0.0 },
  { "Funston", 1.0 },
  { "Fuquan", 1.0 },
  { "Furaha", 0.0 },
  { "Furiosa", 0.0 },
  { "Furious", 1.0 },
  { "Furkan", 1.0 },
  { "Furlon", 1.0 },
  { "Furman", 1.0 },
  { "Furnell", 1.0 },
  { "Furney", 1.0 },
  { "Furqan", 1.0 },
  { "Fury", 0.0 },
  { "Fusae", 0.0 },
  { "Fusako", 0.0 },
  { "Fusaye", 0.0 },
  { "Fushia", 0.0 },
  { "Future", 0.77037036 },
  { "Fuzail", 1.0 },
  { "Fynlee", 0.0 },
  { "Fynleigh", 0.0 },
  { "Fynley", 0.0 },
  { "Fynlie", 0.0 },
  { "Fynn", 0.9303226 },
  { "Fynnegan", 1.0 },
  { "Fynnigan", 1.0 },
  { "Fynnlee", 0.0 },
  { "Fynnleigh", 0.0 },
  { "Fynnley", 0.1875 },
  { "Fyodor", 1.0 },
  { "Fyrn", 0.0 },
  { "Fysher", 1.0 },
  { "Gaal", 0.0 },
  { "Gabbanelli", 0.0 },
  { "Gabbi", 0.0 },
  { "Gabbie", 0.0 },
  { "Gabbriel", 0.0 },
  { "Gabbriella", 0.0 },
  { "Gabbrielle", 0.0 },
  { "Gabby", 0.020915033 },
  { "Gabe", 1.0 },
  { "Gabel", 1.0 },
  { "Gaber", 1.0 },
  { "Gaberial", 0.96460176 },
  { "Gaberiel", 1.0 },
  { "Gaberielle", 0.0 },
  { "Gaberille", 0.0 },
  { "Gabi", 0.0 },
  { "Gabie", 0.0 },
  { "Gabiel", 1.0 },
  { "Gabiela", 0.0 },
  { "Gabija", 0.0 },
  { "Gabin", 1.0 },
  { "Gabina", 0.0 },
  { "Gabino", 1.0 },
  { "Gabirel", 1.0 },
  { "Gabirelle", 0.0 },
  { "Gable", 0.9912537 },
  { "Gabor", 1.0 },
  { "Gabrael", 1.0 },
  { "Gabrail", 1.0 },
  { "Gabreal", 0.77717394 },
  { "Gabreella", 0.0 },
  { "Gabreial", 0.0 },
  { "Gabreil", 0.9126984 },
  { "Gabreilla", 0.0 },
  { "Gabreille", 0.0 },
  { "Gabrel", 1.0 },
  { "Gabrella", 0.0 },
  { "Gabrelle", 0.0 },
  { "Gabria", 0.0 },
  { "Gabrial", 0.8557377 },
  { "Gabriala", 0.0 },
  { "Gabrialla", 0.0 },
  { "Gabrialle", 0.0 },
  { "Gabrian", 1.0 },
  { "Gabriana", 0.0 },
  { "Gabrianna", 0.0 },
  { "Gabrieal", 0.53846157 },
  { "Gabriel", 0.9801806 },
  { "Gabriela", 0.0047777076 },
  { "Gabrielah", 0.0 },
  { "Gabrielalexande", 1.0 },
  { "Gabrielangelo", 1.0 },
  { "Gabriele", 0.27101448 },
  { "Gabrieljohn", 1.0 },
  { "Gabriell", 0.21985383 },
  { "Gabriella", 0.00062225474 },
  { "Gabriellah", 0.0 },
  { "Gabriellamarie", 0.0 },
  { "Gabrielle", 0.005533161 },
  { "Gabriellia", 0.0 },
  { "Gabrielly", 0.0 },
  { "Gabrielmichael", 1.0 },
  { "Gabriely", 0.0 },
  { "Gabrien", 1.0 },
  { "Gabril", 1.0 },
  { "Gabrille", 0.0 },
  { "Gabrionna", 0.0 },
  { "Gabriyel", 0.45833334 },
  { "Gabryal", 1.0 },
  { "Gabryel", 0.664825 },
  { "Gabryela", 0.0 },
  { "Gabryele", 0.0 },
  { "Gabryell", 0.0 },
  { "Gabryella", 0.0 },
  { "Gabryelle", 0.0 },
  { "Gabryl", 1.0 },
  { "Gabryle", 1.0 },
  { "Gabrylle", 0.0 },
  { "Gaby", 0.022589052 },
  { "Gad", 1.0 },
  { "Gaddiel", 1.0 },
  { "Gaddis", 1.0 },
  { "Gadeer", 0.0 },
  { "Gadge", 1.0 },
  { "Gadi", 1.0 },
  { "Gadiel", 1.0 },
  { "Gadriel", 1.0 },
  { "Gadsden", 1.0 },
  { "Gae", 0.0 },
  { "Gaea", 0.0 },
  { "Gaebriel", 1.0 },
  { "Gaege", 1.0 },
  { "Gael", 0.9700557 },
  { "Gaela", 0.0 },
  { "Gaelan", 1.0 },
  { "Gaelen", 0.8965517 },
  { "Gaell", 1.0 },
  { "Gaelle", 0.0 },
  { "Gaely", 0.0 },
  { "Gaelyn", 0.0 },
  { "Gaetan", 1.0 },
  { "Gaetana", 0.0 },
  { "Gaetano", 1.0 },
  { "Gaeton", 1.0 },
  { "Gagan", 0.875 },
  { "Gagandeep", 0.7206704 },
  { "Gage", 0.99595267 },
  { "Gagik", 1.0 },
  { "Gahan", 1.0 },
  { "Gahel", 1.0 },
  { "Gai", 0.0 },
  { "Gaia", 0.0 },
  { "Gaibriel", 1.0 },
  { "Gaiden", 1.0 },
  { "Gaige", 0.9728453 },
  { "Gaije", 1.0 },
  { "Gail", 0.04251469 },
  { "Gaila", 0.0 },
  { "Gailand", 1.0 },
  { "Gailann", 0.0 },
  { "Gailanne", 0.0 },
  { "Gailard", 1.0 },
  { "Gaile", 0.0055370987 },
  { "Gaileen", 0.0 },
  { "Gailen", 0.9765625 },
  { "Gailene", 0.0 },
  { "Gaillard", 1.0 },
  { "Gailon", 1.0 },
  { "Gailord", 1.0 },
  { "Gailya", 0.0 },
  { "Gailyn", 0.0 },
  { "Gailynn", 0.0 },
  { "Gaines", 1.0 },
  { "Gains", 1.0 },
  { "Gaither", 0.9907407 },
  { "Gaitlin", 0.0 },
  { "Gaius", 1.0 },
  { "Gaje", 1.0 },
  { "Gajuan", 1.0 },
  { "Gal", 0.48135594 },
  { "Gala", 0.0 },
  { "Galadriel", 0.0 },
  { "Galan", 1.0 },
  { "Galatea", 0.0 },
  { "Galathia", 0.0 },
  { "Galatia", 0.0 },
  { "Galaxie", 0.0 },
  { "Galaxy", 0.06451613 },
  { "Galdino", 1.0 },
  { "Gale", 0.38311931 },
  { "Galen", 0.9663502 },
  { "Galena", 0.0 },
  { "Galene", 0.0 },
  { "Gali", 0.0 },
  { "Galia", 0.0 },
  { "Galiana", 0.0 },
  { "Galicia", 0.0 },
  { "Galilea", 0.0 },
  { "Galileah", 0.0 },
  { "Galilee", 0.0 },
  { "Galileo", 1.0 },
  { "Galin", 1.0 },
  { "Galina", 0.0 },
  { "Galinda", 0.0 },
  { "Galisa", 0.0 },
  { "Galit", 0.0 },
  { "Galiyah", 0.0 },
  { "Gallagher", 1.0 },
  { "Gallie", 0.0 },
  { "Galloway", 1.0 },
  { "Galo", 1.0 },
  { "Galon", 1.0 },
  { "Galvester", 1.0 },
  { "Galveston", 1.0 },
  { "Galvin", 1.0 },
  { "Galya", 0.0 },
  { "Galylea", 0.0 },
  { "Galyn", 0.41573033 },
  { "Galynn", 0.0 },
  { "Gamal", 1.0 },
  { "Gamaliel", 1.0 },
  { "Gamalier", 1.0 },
  { "Gamari", 1.0 },
  { "Gambit", 1.0 },
  { "Gamble", 0.94382024 },
  { "Gamel", 1.0 },
  { "Games", 1.0 },
  { "Gamila", 0.0 },
  { "Gamir", 1.0 },
  { "Gamya", 0.0 },
  { "Gana", 0.0 },
  { "Ganae", 0.0 },
  { "Ganajah", 0.0 },
  { "Gandalf", 1.0 },
  { "Gandolfo", 1.0 },
  { "Ganell", 0.0 },
  { "Ganelle", 0.0 },
  { "Ganesa", 0.0 },
  { "Ganesh", 1.0 },
  { "Ganessa", 0.0 },
  { "Gani", 1.0 },
  { "Gania", 0.0 },
  { "Ganiya", 0.0 },
  { "Ganiyah", 0.0 },
  { "Ganna", 0.0 },
  { "Gannen", 1.0 },
  { "Gannicus", 1.0 },
  { "Gannon", 1.0 },
  { "Gannyn", 1.0 },
  { "Gano", 1.0 },
  { "Ganon", 1.0 },
  { "Gant", 1.0 },
  { "Gantt", 1.0 },
  { "Ganyn", 1.0 },
  { "Ganza", 1.0 },
  { "Gao", 0.0 },
  { "Gaolee", 0.0 },
  { "Gaonou", 0.0 },
  { "Gaosheng", 0.0 },
  { "Gaoussou", 1.0 },
  { "Gaozong", 0.0 },
  { "Gaquan", 1.0 },
  { "Gar", 1.0 },
  { "Gara", 0.0 },
  { "Garak", 1.0 },
  { "Garald", 1.0 },
  { "Garan", 1.0 },
  { "Garang", 1.0 },
  { "Garbiel", 1.0 },
  { "Garbielle", 0.0 },
  { "Garbriel", 1.0 },
  { "Garbrielle", 0.0 },
  { "Garcelle", 0.0 },
  { "Garcia", 0.92156863 },
  { "Gard", 1.0 },
  { "Garda", 0.0 },
  { "Gardell", 1.0 },
  { "Garden", 1.0 },
  { "Gardenia", 0.0 },
  { "Gardina", 0.0 },
  { "Gardiner", 1.0 },
  { "Gardner", 0.9982651 },
  { "Gardy", 1.0 },
  { "Gared", 1.0 },
  { "Garek", 1.0 },
  { "Gareld", 1.0 },
  { "Garen", 0.98851675 },
  { "Garet", 1.0 },
  { "Gareth", 0.99871397 },
  { "Garett", 1.0 },
  { "Garette", 1.0 },
  { "Garey", 1.0 },
  { "Garfield", 1.0 },
  { "Gargi", 0.0 },
  { "Garhett", 1.0 },
  { "Gari", 0.23981191 },
  { "Garian", 1.0 },
  { "Garic", 1.0 },
  { "Garick", 1.0 },
  { "Garie", 0.22727273 },
  { "Gariel", 0.75 },
  { "Gariella", 0.0 },
  { "Garielle", 0.0 },
  { "Garik", 1.0 },
  { "Garima", 0.0 },
  { "Garin", 1.0 },
  { "Garine", 0.0 },
  { "Garion", 1.0 },
  { "Garison", 1.0 },
  { "Garit", 1.0 },
  { "Garitt", 1.0 },
  { "Garius", 1.0 },
  { "Garl", 1.0 },
  { "Garla", 0.0 },
  { "Garlan", 1.0 },
  { "Garland", 0.9661259 },
  { "Garlen", 1.0 },
  { "Garlene", 0.0 },
  { "Garlin", 1.0 },
  { "Garlon", 1.0 },
  { "Garlond", 1.0 },
  { "Garlund", 1.0 },
  { "Garlyn", 0.6875 },
  { "Garman", 1.0 },
  { "Garmon", 1.0 },
  { "Garn", 1.0 },
  { "Garna", 0.0 },
  { "Garnel", 1.0 },
  { "Garnell", 0.8039604 },
  { "Garnelle", 0.0 },
  { "Garner", 0.9891714 },
  { "Garnet", 0.22301994 },
  { "Garneta", 0.0 },
  { "Garnett", 0.5782457 },
  { "Garnetta", 0.0 },
  { "Garnette", 0.0 },
  { "Garney", 1.0 },
  { "Garnie", 0.84615386 },
  { "Garnieta", 0.0 },
  { "Garnita", 0.0 },
  { "Garo", 1.0 },
  { "Garold", 1.0 },
  { "Garon", 1.0 },
  { "Garr", 1.0 },
  { "Garrad", 1.0 },
  { "Garran", 1.0 },
  { "Garrard", 1.0 },
  { "Garratt", 1.0 },
  { "Garred", 1.0 },
  { "Garrek", 1.0 },
  { "Garrel", 1.0 },
  { "Garrell", 1.0 },
  { "Garren", 0.9903134 },
  { "Garret", 0.9996153 },
  { "Garreth", 1.0 },
  { "Garrett", 0.9965001 },
  { "Garrette", 1.0 },
  { "Garrey", 1.0 },
  { "Garrhett", 1.0 },
  { "Garri", 0.0 },
  { "Garric", 1.0 },
  { "Garrick", 1.0 },
  { "Garrie", 0.82168674 },
  { "Garrik", 1.0 },
  { "Garrin", 1.0 },
  { "Garrion", 1.0 },
  { "Garris", 1.0 },
  { "Garrison", 0.99931574 },
  { "Garrit", 1.0 },
  { "Garritt", 1.0 },
  { "Garrod", 1.0 },
  { "Garron", 1.0 },
  { "Garrus", 1.0 },
  { "Garry", 0.9962429 },
  { "Garryn", 0.0 },
  { "Garson", 1.0 },
  { "Garth", 1.0 },
  { "Gartha", 0.0 },
  { "Gartrell", 1.0 },
  { "Garv", 1.0 },
  { "Garvens", 1.0 },
  { "Garvey", 1.0 },
  { "Garvice", 1.0 },
  { "Garvie", 1.0 },
  { "Garvin", 1.0 },
  { "Garvis", 1.0 },
  { "Garwin", 1.0 },
  { "Garwood", 1.0 },
  { "Gary", 0.99635804 },
  { "Garyl", 1.0 },
  { "Garylee", 1.0 },
  { "Garyn", 0.75420874 },
  { "Garyon", 1.0 },
  { "Garyson", 1.0 },
  { "Garytt", 1.0 },
  { "Gasia", 0.0 },
  { "Gaspar", 1.0 },
  { "Gaspard", 1.0 },
  { "Gaspare", 1.0 },
  { "Gasper", 1.0 },
  { "Gaston", 1.0 },
  { "Gaten", 1.0 },
  { "Gates", 0.86904764 },
  { "Gatha", 0.0 },
  { "Gathel", 0.0 },
  { "Gather", 1.0 },
  { "Gatlen", 1.0 },
  { "Gatlin", 0.9974516 },
  { "Gatlyn", 0.9596774 },
  { "Gatlynn", 0.42105263 },
  { "Gator", 1.0 },
  { "Gatsby", 0.9137931 },
  { "Gattis", 1.0 },
  { "Gattlin", 1.0 },
  { "Gaudalupe", 0.0 },
  { "Gaudencio", 1.0 },
  { "Gaudy", 0.0 },
  { "Gauge", 1.0 },
  { "Gaurav", 1.0 },
  { "Gauri", 0.0 },
  { "Gaurika", 0.0 },
  { "Gautam", 1.0 },
  { "Gautham", 1.0 },
  { "Gavan", 1.0 },
  { "Gaven", 1.0 },
  { "Gavi", 0.0 },
  { "Gavian", 1.0 },
  { "Gavien", 1.0 },
  { "Gavin", 0.99780655 },
  { "Gavina", 0.0 },
  { "Gavinn", 1.0 },
  { "Gavino", 1.0 },
  { "Gaviota", 0.0 },
  { "Gavon", 1.0 },
  { "Gavriel", 0.9961059 },
  { "Gavriela", 0.0 },
  { "Gavriella", 0.0 },
  { "Gavrielle", 0.0 },
  { "Gavril", 1.0 },
  { "Gavvin", 1.0 },
  { "Gavy", 0.0 },
  { "Gavyn", 0.97622985 },
  { "Gavyne", 1.0 },
  { "Gavynn", 1.0 },
  { "Gawain", 1.0 },
  { "Gawaine", 1.0 },
  { "Gay", 0.07136218 },
  { "Gaya", 0.0 },
  { "Gayane", 0.0 },
  { "Gayann", 0.0 },
  { "Gayanne", 0.0 },
  { "Gayathri", 0.0 },
  { "Gayatri", 0.0 },
  { "Gaye", 0.0 },
  { "Gayel", 0.0 },
  { "Gayge", 1.0 },
  { "Gayl", 0.024115756 },
  { "Gayla", 0.00055330136 },
  { "Gaylan", 0.9125475 },
  { "Gayland", 1.0 },
  { "Gaylard", 1.0 },
  { "Gayle", 0.07471585 },
  { "Gaylee", 0.0 },
  { "Gayleen", 0.0 },
  { "Gaylen", 0.85328835 },
  { "Gaylene", 0.0 },
  { "Gaylia", 0.0 },
  { "Gaylin", 0.57591623 },
  { "Gaylon", 0.9409214 },
  { "Gaylor", 1.0 },
  { "Gaylord", 0.9991791 },
  { "Gaylyn", 0.0 },
  { "Gaylynn", 0.0 },
  { "Gaylynne", 0.0 },
  { "Gayna", 0.0 },
  { "Gaynard", 1.0 },
  { "Gaynel", 0.0 },
  { "Gaynell", 0.004216444 },
  { "Gaynelle", 0.0 },
  { "Gaynol", 0.0 },
  { "Gaynor", 0.475 },
  { "Gayola", 0.0 },
  { "Gayron", 1.0 },
  { "Gaytha", 0.0 },
  { "Gaythel", 0.0 },
  { "Gayton", 1.0 },
  { "Gaza", 1.0 },
  { "Gazal", 0.0 },
  { "Gazella", 0.0 },
  { "Gazelle", 0.0 },
  { "Ge", 0.91071427 },
  { "Gea", 0.0 },
  { "Gean", 0.45261437 },
  { "Geana", 0.0 },
  { "Geancarlo", 1.0 },
  { "Geane", 0.0 },
  { "Geanette", 0.0 },
  { "Geanie", 0.0 },
  { "Geanine", 0.0 },
  { "Geanna", 0.0 },
  { "Geanne", 0.0 },
  { "Geannie", 0.0 },
  { "Geannine", 0.0 },
  { "Gearald", 1.0 },
  { "Geargia", 0.0 },
  { "Gearl", 0.7866667 },
  { "Gearld", 0.9988334 },
  { "Gearldean", 0.0 },
  { "Gearldene", 0.0 },
  { "Gearldine", 0.0010660981 },
  { "Gearlean", 0.0 },
  { "Gearleen", 0.0 },
  { "Gearlene", 0.0 },
  { "Gearline", 0.0 },
  { "Gearold", 1.0 },
  { "Geary", 1.0 },
  { "Geatano", 1.0 },
  { "Gecenia", 0.0 },
  { "Gedalia", 1.0 },
  { "Gedaliah", 1.0 },
  { "Gedalya", 1.0 },
  { "Geddes", 1.0 },
  { "Geddy", 1.0 },
  { "Gedeon", 1.0 },
  { "Gee", 0.79012346 },
  { "Geena", 0.0 },
  { "Geet", 0.0 },
  { "Geeta", 0.0 },
  { "Geetanjali", 0.0 },
  { "Geetha", 0.0 },
  { "Geethika", 0.0 },
  { "Geetika", 0.0 },
  { "Gefen", 0.0 },
  { "Geff", 1.0 },
  { "Geffen", 1.0 },
  { "Geffery", 1.0 },
  { "Geffory", 1.0 },
  { "Geffrey", 1.0 },
  { "Gegory", 1.0 },
  { "Gehad", 0.0 },
  { "Gehard", 1.0 },
  { "Gehrig", 1.0 },
  { "Geidi", 0.0 },
  { "Geidy", 0.0 },
  { "Geisha", 0.0 },
  { "Geissie", 0.0 },
  { "Gejuan", 1.0 },
  { "Gela", 0.0 },
  { "Gelacio", 1.0 },
  { "Geleah", 0.0 },
  { "Gelen", 0.0 },
  { "Gelena", 0.0 },
  { "Gelene", 0.0 },
  { "Gelia", 0.0 },
  { "Gelila", 0.0 },
  { "Gelina", 0.0 },
  { "Gelinda", 0.0 },
  { "Geline", 0.0 },
  { "Gelisa", 0.0 },
  { "Gelisha", 0.0 },
  { "Gelissa", 0.0 },
  { "Gella", 0.0 },
  { "Gelsey", 0.0 },
  { "Gelsomina", 0.0 },
  { "Gelson", 1.0 },
  { "Gem", 0.0 },
  { "Gema", 0.0 },
  { "Gemar", 1.0 },
  { "Gemari", 1.0 },
  { "Gemarion", 1.0 },
  { "Gemayel", 1.0 },
  { "Gemelli", 0.0 },
  { "Gemia", 0.0 },
  { "Gemini", 0.3425926 },
  { "Gemise", 0.0 },
  { "Gemma", 0.0 },
  { "Gemmalynn", 0.0 },
  { "Gen", 0.8378378 },
  { "Gena", 0.0 },
  { "Genae", 0.0 },
  { "Genai", 0.0 },
  { "Genara", 0.0 },
  { "Genard", 1.0 },
  { "Genaro", 1.0 },
  { "Genasis", 0.0 },
  { "Genavecia", 0.0 },
  { "Genavee", 0.0 },
  { "Genaveve", 0.0 },
  { "Genavie", 0.0 },
  { "Genavieve", 0.0 },
  { "Genavive", 0.0 },
  { "Genay", 0.0 },
  { "Genaya", 0.0 },
  { "Gencie", 0.0 },
  { "Gene", 0.9475799 },
  { "Genea", 0.0 },
  { "Geneal", 0.0 },
  { "Genean", 0.0 },
  { "Geneane", 0.0 },
  { "Genece", 0.0 },
  { "Genecis", 0.0 },
  { "Genee", 0.0 },
  { "Geneen", 0.0 },
  { "Geneice", 0.0 },
  { "Geneieve", 0.0 },
  { "Geneil", 0.0 },
  { "Geneine", 0.0 },
  { "Geneiva", 0.0 },
  { "Geneive", 0.0 },
  { "Geneiveve", 0.0 },
  { "Geneivieve", 0.0 },
  { "Genel", 0.0 },
  { "Genelda", 0.0 },
  { "Genele", 0.0 },
  { "Genelia", 0.0 },
  { "Genell", 0.0 },
  { "Genella", 0.0 },
  { "Genelle", 0.0 },
  { "Genena", 0.0 },
  { "Genendy", 0.0 },
  { "Genene", 0.0 },
  { "Genera", 0.0 },
  { "General", 1.0 },
  { "Genero", 1.0 },
  { "Generosa", 0.0 },
  { "Generose", 0.0 },
  { "Generoso", 1.0 },
  { "Generra", 0.0 },
  { "Genesa", 0.0 },
  { "Genese", 0.0 },
  { "Genesee", 0.0 },
  { "Geneses", 0.0 },
  { "Genesha", 0.0 },
  { "Genesi", 0.0 },
  { "Genesia", 0.0 },
  { "Genesis", 0.03002964 },
  { "Genesiss", 0.0 },
  { "Genessa", 0.0 },
  { "Genesse", 0.0 },
  { "Genessee", 0.0 },
  { "Genessi", 0.0 },
  { "Genessis", 0.0 },
  { "Genessy", 0.0 },
  { "Genessys", 0.0 },
  { "Genesy", 0.0 },
  { "Genesys", 0.0 },
  { "Genet", 0.0 },
  { "Geneta", 0.0 },
  { "Genetha", 0.0 },
  { "Genett", 0.0 },
  { "Genetta", 0.0 },
  { "Genette", 0.0 },
  { "Geneva", 0.0035218892 },
  { "Geneve", 0.0 },
  { "Genevea", 0.0 },
  { "Genevee", 0.0 },
  { "Geneveive", 0.0 },
  { "Genever", 0.0 },
  { "Geneveve", 0.0 },
  { "Genevia", 0.0 },
  { "Genevie", 0.0 },
  { "Geneviene", 0.0 },
  { "Geneviev", 0.0 },
  { "Genevieve", 0.0013822143 },
  { "Genevive", 0.0 },
  { "Genevra", 0.0 },
  { "Genevy", 0.0 },
  { "Genghis", 1.0 },
  { "Geni", 0.0 },
  { "Genia", 0.0 },
  { "Geniah", 0.0 },
  { "Genica", 0.0 },
  { "Genice", 0.0 },
  { "Genie", 0.06032078 },
  { "Geniece", 0.0 },
  { "Geniel", 0.0 },
  { "Geniene", 0.0 },
  { "Genieva", 0.0 },
  { "Genieve", 0.0 },
  { "Genievieve", 0.0 },
  { "Genifer", 0.0 },
  { "Geniffer", 0.0 },
  { "Genika", 0.0 },
  { "Genina", 0.0 },
  { "Genine", 0.0 },
  { "Genise", 0.0 },
  { "Genises", 0.0 },
  { "Genisha", 0.0 },
  { "Genisis", 0.0 },
  { "Genissa", 0.0 },
  { "Genisse", 0.0 },
  { "Genisus", 0.0 },
  { "Genisys", 0.0 },
  { "Genita", 0.0 },
  { "Geniva", 0.0 },
  { "Genive", 0.0 },
  { "Genivieve", 0.0 },
  { "Genivive", 0.0 },
  { "Geniya", 0.0 },
  { "Geniyah", 0.0 },
  { "Genki", 1.0 },
  { "Genna", 0.0 },
  { "Gennady", 1.0 },
  { "Gennaro", 1.0 },
  { "Gennavieve", 0.0 },
  { "Gennavive", 0.0 },
  { "Gennell", 0.0 },
  { "Gennelle", 0.0 },
  { "Genner", 1.0 },
  { "Gennesis", 0.0 },
  { "Gennessy", 0.0 },
  { "Gennesy", 0.0 },
  { "Gennesys", 0.0 },
  { "Genneth", 0.0 },
  { "Gennett", 0.0 },
  { "Gennetta", 0.0 },
  { "Gennette", 0.0 },
  { "Gennevieve", 0.0 },
  { "Genni", 0.0 },
  { "Gennia", 0.0 },
  { "Gennie", 0.07262774 },
  { "Gennieve", 0.0 },
  { "Gennifer", 0.0 },
  { "Gennine", 0.0 },
  { "Genny", 0.0 },
  { "Geno", 1.0 },
  { "Genoa", 0.0 },
  { "Genola", 0.0 },
  { "Genora", 0.0 },
  { "Genoria", 0.0 },
  { "Genova", 0.0 },
  { "Genoveva", 0.0 },
  { "Genovevo", 1.0 },
  { "Genovia", 0.0 },
  { "Genovieve", 0.0 },
  { "Genowefa", 0.0 },
  { "Genrose", 0.0 },
  { "Gensen", 1.0 },
  { "Gensie", 0.0 },
  { "Gensis", 0.0 },
  { "Gent", 1.0 },
  { "Gentiana", 0.0 },
  { "Gentle", 1.0 },
  { "Gentri", 0.0 },
  { "Gentrie", 0.0 },
  { "Gentry", 0.64186144 },
  { "Genuine", 1.0 },
  { "Genva", 0.0 },
  { "Genvieve", 0.0 },
  { "Genya", 0.0 },
  { "Genysis", 0.0 },
  { "Geo", 0.98378927 },
  { "Geobani", 1.0 },
  { "Geof", 1.0 },
  { "Geoff", 1.0 },
  { "Geoffery", 1.0 },
  { "Geoffory", 1.0 },
  { "Geoffrey", 0.9974113 },
  { "Geoffry", 1.0 },
  { "Geofrey", 1.0 },
  { "Geoge", 1.0 },
  { "Geoggrey", 1.0 },
  { "Geogia", 0.0 },
  { "Geogory", 1.0 },
  { "Geomar", 1.0 },
  { "Geon", 1.0 },
  { "Geona", 0.0 },
  { "Geonna", 0.0 },
  { "Geonni", 1.0 },
  { "Geontae", 1.0 },
  { "Geonte", 1.0 },
  { "Geordan", 0.90802675 },
  { "Georden", 1.0 },
  { "Geordi", 1.0 },
  { "Geordie", 1.0 },
  { "Geordon", 1.0 },
  { "Geordy", 1.0 },
  { "Geordyn", 0.0 },
  { "Geore", 1.0 },
  { "Georffrey", 1.0 },
  { "Georg", 0.9791667 },
  { "Georga", 0.0 },
  { "Georgan", 0.0 },
  { "Georgana", 0.0 },
  { "Georgann", 0.0 },
  { "Georganna", 0.0 },
  { "Georganne", 0.0 },
  { "George", 0.99324715 },
  { "Georgea", 0.0 },
  { "Georgean", 0.0 },
  { "Georgeana", 0.0 },
  { "Georgeann", 0.0 },
  { "Georgeanna", 0.0 },
  { "Georgeanne", 0.0 },
  { "Georgeanthony", 1.0 },
  { "Georgeen", 0.0 },
  { "Georgeina", 0.0 },
  { "Georgella", 0.0 },
  { "Georgemichael", 1.0 },
  { "Georgena", 0.0 },
  { "Georgene", 0.0 },
  { "Georgenia", 0.0 },
  { "Georgenna", 0.0 },
  { "Georgenne", 0.0 },
  { "Georges", 1.0 },
  { "Georgett", 0.0 },
  { "Georgetta", 0.0 },
  { "Georgette", 0.0 },
  { "Georgi", 0.1286031 },
  { "Georgia", 0.0048423572 },
  { "Georgiaann", 0.0 },
  { "Georgialee", 0.0 },
  { "Georgiamae", 0.0 },
  { "Georgian", 0.0 },
  { "Georgiana", 0.0 },
  { "Georgiane", 0.0 },
  { "Georgiann", 0.0 },
  { "Georgianna", 0.0 },
  { "Georgianne", 0.0 },
  { "Georgie", 0.11359238 },
  { "Georgieann", 0.0 },
  { "Georgieanna", 0.0 },
  { "Georgiena", 0.0 },
  { "Georgietta", 0.0 },
  { "Georgina", 0.0 },
  { "Georgine", 0.0 },
  { "Georginia", 0.0 },
  { "Georgio", 0.9917355 },
  { "Georgios", 1.0 },
  { "Georgiy", 1.0 },
  { "Georgy", 0.5833333 },
  { "Georjean", 0.0 },
  { "Geovan", 1.0 },
  { "Geovana", 0.0 },
  { "Geovani", 1.0 },
  { "Geovanie", 1.0 },
  { "Geovanna", 0.0 },
  { "Geovanne", 1.0 },
  { "Geovanni", 1.0 },
  { "Geovannie", 1.0 },
  { "Geovanny", 1.0 },
  { "Geovany", 1.0 },
  { "Geovoni", 1.0 },
  { "Geovonie", 1.0 },
  { "Geovonni", 1.0 },
  { "Geovonnie", 1.0 },
  { "Gequan", 1.0 },
  { "Ger", 0.81395346 },
  { "Gera", 0.0 },
  { "Gerad", 1.0 },
  { "Geradette", 0.0 },
  { "Geradine", 0.0 },
  { "Gerado", 1.0 },
  { "Geraine", 0.0 },
  { "Geral", 0.60215056 },
  { "Gerald", 0.9946921 },
  { "Geralda", 0.0 },
  { "Geraldean", 0.0 },
  { "Geraldeen", 0.0 },
  { "Geraldene", 0.0 },
  { "Geraldi", 0.0 },
  { "Geraldin", 0.0 },
  { "Geraldina", 0.0 },
  { "Geraldine", 0.0035448389 },
  { "Geraldinne", 0.0 },
  { "Geraldo", 1.0 },
  { "Geraldy", 0.0 },
  { "Geraldyn", 0.0 },
  { "Geraldyne", 0.0 },
  { "Geraleen", 0.0 },
  { "Geralene", 0.0 },
  { "Geraline", 0.0 },
  { "Gerall", 1.0 },
  { "Geralyn", 0.0 },
  { "Geralynn", 0.0 },
  { "Gerame", 1.0 },
  { "Geramiah", 1.0 },
  { "Geramie", 1.0 },
  { "Geramy", 1.0 },
  { "Geran", 1.0 },
  { "Gerane", 0.0 },
  { "Gerard", 0.99806154 },
  { "Gerarda", 0.0 },
  { "Gerardette", 0.0 },
  { "Gerardina", 0.0 },
  { "Gerardine", 0.0 },
  { "Gerardo", 0.99529034 },
  { "Gerardojr", 1.0 },
  { "Gerasimos", 1.0 },
  { "Geraud", 1.0 },
  { "Gerben", 1.0 },
  { "Gerber", 1.0 },
  { "Gerd", 0.4848485 },
  { "Gerda", 0.0 },
  { "Gerdie", 0.0 },
  { "Gere", 0.33333334 },
  { "Gered", 1.0 },
  { "Gerek", 1.0 },
  { "Gereld", 1.0 },
  { "Gereldine", 0.0 },
  { "Gerelene", 0.0 },
  { "Gereline", 0.0 },
  { "Gerell", 1.0 },
  { "Geremia", 1.0 },
  { "Geremiah", 1.0 },
  { "Geremias", 1.0 },
  { "Geremie", 1.0 },
  { "Geremy", 1.0 },
  { "Geren", 1.0 },
  { "Gerene", 0.0 },
  { "Geretha", 0.0 },
  { "Gerette", 0.0 },
  { "Gergory", 1.0 },
  { "Gerhard", 1.0 },
  { "Gerhardt", 1.0 },
  { "Gerhart", 1.0 },
  { "Geri", 0.0006235969 },
  { "Geriah", 0.0 },
  { "Geriann", 0.0 },
  { "Gerianne", 0.0 },
  { "Geric", 1.0 },
  { "Gerica", 0.0 },
  { "Gericho", 1.0 },
  { "Gerick", 1.0 },
  { "Gerid", 1.0 },
  { "Gerie", 0.0 },
  { "Geriel", 1.0 },
  { "Gerik", 1.0 },
  { "Gerika", 0.0 },
  { "Gerilyn", 0.0 },
  { "Gerilynn", 0.0 },
  { "Gerimiah", 1.0 },
  { "Gerin", 1.0 },
  { "Gerise", 0.0 },
  { "Gerita", 0.0 },
  { "Geriyah", 0.0 },
  { "Gerlad", 1.0 },
  { "Gerladine", 0.0 },
  { "Gerld", 1.0 },
  { "Gerldean", 0.0 },
  { "Gerldine", 0.0 },
  { "Gerlean", 0.0 },
  { "Gerlene", 0.0 },
  { "Gerline", 0.0 },
  { "Gerlisa", 0.0 },
  { "Gerlyn", 0.0 },
  { "Germain", 0.95770395 },
  { "Germaine", 0.22497298 },
  { "German", 0.99945295 },
  { "Germane", 1.0 },
  { "Germani", 0.0 },
  { "Germanii", 0.0 },
  { "Germany", 0.39597315 },
  { "Germar", 1.0 },
  { "Germari", 1.0 },
  { "Germarion", 1.0 },
  { "Germayne", 1.0 },
  { "Germell", 1.0 },
  { "Germey", 1.0 },
  { "Germon", 1.0 },
  { "Germya", 0.0 },
  { "Gernard", 1.0 },
  { "Gernie", 1.0 },
  { "Gerniya", 0.0 },
  { "Gero", 1.0 },
  { "Gerod", 1.0 },
  { "Geroge", 1.0 },
  { "Gerogia", 0.0 },
  { "Gerold", 1.0 },
  { "Geroldine", 0.0 },
  { "Geroline", 0.0 },
  { "Gerolyn", 0.0 },
  { "Gerome", 1.0 },
  { "Geromy", 1.0 },
  { "Geron", 1.0 },
  { "Gerone", 1.0 },
  { "Geronima", 0.0 },
  { "Geronimo", 1.0 },
  { "Gerrad", 1.0 },
  { "Gerrald", 1.0 },
  { "Gerran", 1.0 },
  { "Gerrard", 1.0 },
  { "Gerre", 0.0 },
  { "Gerred", 1.0 },
  { "Gerrel", 1.0 },
  { "Gerrell", 1.0 },
  { "Gerren", 0.93559325 },
  { "Gerret", 1.0 },
  { "Gerrett", 1.0 },
  { "Gerri", 0.0031118433 },
  { "Gerriann", 0.0 },
  { "Gerrianne", 0.0 },
  { "Gerrica", 0.0 },
  { "Gerrick", 1.0 },
  { "Gerrid", 1.0 },
  { "Gerrie", 0.007982584 },
  { "Gerrika", 0.0 },
  { "Gerrilyn", 0.0 },
  { "Gerrilynn", 0.0 },
  { "Gerrin", 1.0 },
  { "Gerrit", 1.0 },
  { "Gerritt", 1.0 },
  { "Gerrod", 1.0 },
  { "Gerrold", 1.0 },
  { "Gerron", 1.0 },
  { "Gerry", 0.68346965 },
  { "Gerrye", 0.0 },
  { "Gerryn", 0.0 },
  { "Gersain", 1.0 },
  { "Gershom", 1.0 },
  { "Gershon", 1.0 },
  { "Gerson", 1.0 },
  { "Gerta", 0.0 },
  { "Gertha", 0.0 },
  { "Gerthie", 0.0 },
  { "Gertie", 0.0 },
  { "Gertis", 0.17857143 },
  { "Gertrud", 0.0 },
  { "Gertrude", 0.00320184 },
  { "Gertrudes", 0.0 },
  { "Gertrudis", 0.0 },
  { "Gertrue", 0.0 },
  { "Gertude", 0.0 },
  { "Gerturde", 0.0 },
  { "Gervais", 1.0 },
  { "Gervaise", 0.0 },
  { "Gervase", 0.9752475 },
  { "Gerver", 1.0 },
  { "Gervin", 1.0 },
  { "Gervis", 1.0 },
  { "Gervon", 1.0 },
  { "Gery", 1.0 },
  { "Geryl", 0.0952381 },
  { "Gerzon", 1.0 },
  { "Gesel", 0.0 },
  { "Gesele", 0.0 },
  { "Gesell", 0.0 },
  { "Geselle", 0.0 },
  { "Gesenia", 0.0 },
  { "Geshia", 0.0 },
  { "Gesica", 0.0 },
  { "Gesina", 0.0 },
  { "Gessel", 0.0 },
  { "Gessell", 0.0 },
  { "Gesselle", 0.0 },
  { "Gessica", 0.0 },
  { "Gesualdo", 1.0 },
  { "Geter", 1.0 },
  { "Getha", 0.0 },
  { "Gethsemane", 0.0 },
  { "Gethsemani", 0.0 },
  { "Getrude", 0.0 },
  { "Getrudes", 0.0 },
  { "Getsemani", 0.024590164 },
  { "Getsemany", 0.0 },
  { "Gettie", 0.0 },
  { "Gettis", 1.0 },
  { "Getty", 1.0 },
  { "Gettys", 1.0 },
  { "Getzel", 1.0 },
  { "Getzemani", 0.0 },
  { "Getzemany", 0.0 },
  { "Gevalia", 0.0 },
  { "Gevan", 1.0 },
  { "Gevena", 0.0 },
  { "Gevin", 1.0 },
  { "Gevon", 1.0 },
  { "Gevonte", 1.0 },
  { "Gevorg", 1.0 },
  { "Gevork", 1.0 },
  { "Gewel", 0.0 },
  { "Geysha", 0.0 },
  { "Geza", 1.0 },
  { "Gezelle", 0.0 },
  { "Ghabriel", 1.0 },
  { "Ghada", 0.0 },
  { "Ghadah", 0.0 },
  { "Ghadeer", 0.0 },
  { "Ghael", 1.0 },
  { "Ghaida", 0.0 },
  { "Ghais", 1.0 },
  { "Ghaith", 1.0 },
  { "Ghala", 0.0 },
  { "Ghaleb", 1.0 },
  { "Ghali", 1.0 },
  { "Ghalia", 0.0 },
  { "Ghana", 0.0 },
  { "Ghanem", 1.0 },
  { "Ghassan", 1.0 },
  { "Ghavin", 1.0 },
  { "Ghazal", 0.0 },
  { "Ghazi", 1.0 },
  { "Ghena", 0.0 },
  { "Ghenet", 0.0 },
  { "Ghia", 0.0 },
  { "Ghian", 1.0 },
  { "Ghianna", 0.0 },
  { "Ghina", 0.0 },
  { "Ghislaine", 0.0 },
  { "Ghislane", 0.0 },
  { "Ghita", 0.0 },
  { "Ghofran", 0.0 },
  { "Ghulam", 1.0 },
  { "Gi", 0.0 },
  { "Gia", 0.00036646143 },
  { "Giabella", 0.0 },
  { "Giacinto", 1.0 },
  { "Giacobbe", 1.0 },
  { "Giacomina", 0.0 },
  { "Giacomo", 1.0 },
  { "Giada", 0.0 },
  { "Giah", 0.0 },
  { "Giahan", 0.0 },
  { "Giahna", 0.0 },
  { "Giahnna", 0.0 },
  { "Giamarie", 0.0 },
  { "Gian", 1.0 },
  { "Giana", 0.0 },
  { "Gianah", 0.0 },
  { "Giancarlo", 1.0 },
  { "Giancarlos", 1.0 },
  { "Gianella", 0.0 },
  { "Gianelle", 0.0 },
  { "Gianelly", 0.0 },
  { "Gianessa", 0.0 },
  { "Gianetta", 0.0 },
  { "Gianfranco", 1.0 },
  { "Giang", 0.84 },
  { "Giani", 0.578199 },
  { "Gianina", 0.0 },
  { "Gianinna", 0.0 },
  { "Giankarlo", 1.0 },
  { "Gianluca", 1.0 },
  { "Gianlucas", 1.0 },
  { "Gianlucca", 1.0 },
  { "Gianluigi", 1.0 },
  { "Gianluka", 1.0 },
  { "Gianmarco", 1.0 },
  { "Gianmarcos", 1.0 },
  { "Gianmichael", 1.0 },
  { "Giann", 1.0 },
  { "Gianna", 0.00046298592 },
  { "Giannah", 0.0 },
  { "Giannamarie", 0.0 },
  { "Giannarose", 0.0 },
  { "Gianncarlo", 1.0 },
  { "Gianne", 0.0 },
  { "Giannella", 0.0 },
  { "Gianni", 0.8345816 },
  { "Giannie", 0.0 },
  { "Giannina", 0.0 },
  { "Giannis", 1.0 },
  { "Gianno", 1.0 },
  { "Gianny", 0.52272725 },
  { "Giano", 1.0 },
  { "Gianpaolo", 1.0 },
  { "Gianpaul", 1.0 },
  { "Gianpiero", 1.0 },
  { "Giany", 0.0 },
  { "Giara", 0.0 },
  { "Giavana", 0.0 },
  { "Giavanna", 0.0 },
  { "Giavannah", 0.0 },
  { "Giavanni", 0.307947 },
  { "Giavona", 0.0 },
  { "Giavonna", 0.0 },
  { "Giavonni", 0.38721806 },
  { "Giavonnie", 0.0 },
  { "Gib", 1.0 },
  { "Gibb", 1.0 },
  { "Gibbs", 1.0 },
  { "Gibelly", 0.0 },
  { "Gibert", 1.0 },
  { "Gibram", 1.0 },
  { "Gibran", 1.0 },
  { "Gibril", 1.0 },
  { "Gibson", 0.97140133 },
  { "Gicel", 0.0 },
  { "Gicela", 0.0 },
  { "Gicell", 0.0 },
  { "Gicelle", 0.0 },
  { "Gid", 1.0 },
  { "Giddeon", 1.0 },
  { "Gideon", 0.9991309 },
  { "Gidget", 0.0 },
  { "Gidgett", 0.0 },
  { "Gidgette", 0.0 },
  { "Gieselle", 0.0 },
  { "Giezi", 1.0 },
  { "Gifford", 1.0 },
  { "Gift", 0.0 },
  { "Gifty", 0.0 },
  { "Gig", 1.0 },
  { "Giget", 0.0 },
  { "Gigette", 0.0 },
  { "Gigi", 0.0 },
  { "Gihanna", 0.0 },
  { "Gil", 1.0 },
  { "Gila", 0.0 },
  { "Gilad", 1.0 },
  { "Gilana", 0.0 },
  { "Gilbert", 0.99478775 },
  { "Gilberta", 0.0 },
  { "Gilberte", 0.0 },
  { "Gilberto", 0.9978889 },
  { "Gilchrist", 1.0 },
  { "Gilda", 0.0 },
  { "Gildardo", 1.0 },
  { "Gildo", 1.0 },
  { "Gile", 1.0 },
  { "Gilead", 1.0 },
  { "Giles", 1.0 },
  { "Gilford", 1.0 },
  { "Gilfred", 1.0 },
  { "Gili", 0.0 },
  { "Gilia", 0.0 },
  { "Gilian", 0.0 },
  { "Giliana", 0.0 },
  { "Gill", 0.95672697 },
  { "Gillan", 0.0 },
  { "Gillard", 1.0 },
  { "Gillen", 1.0 },
  { "Gillermina", 0.0 },
  { "Gillermo", 1.0 },
  { "Gilles", 1.0 },
  { "Gillespie", 1.0 },
  { "Gillette", 1.0 },
  { "Gilliam", 0.93630576 },
  { "Gillian", 0.0042979023 },
  { "Gilliana", 0.0 },
  { "Gillianna", 0.0 },
  { "Gillianne", 0.0 },
  { "Gillie", 0.3723197 },
  { "Gillis", 1.0 },
  { "Gillyan", 0.0 },
  { "Gilma", 0.0 },
  { "Gilman", 1.0 },
  { "Gilmar", 1.0 },
  { "Gilmer", 1.0 },
  { "Gilmore", 1.0 },
  { "Gilroy", 1.0 },
  { "Gilson", 1.0 },
  { "Gilverto", 1.0 },
  { "Gimena", 0.0 },
  { "Gin", 0.76666665 },
  { "Gina", 0.002617595 },
  { "Ginae", 0.0 },
  { "Ginamarie", 0.0 },
  { "Ginaya", 0.0 },
  { "Gindy", 0.0 },
  { "Gineen", 0.0 },
  { "Ginelie", 0.0 },
  { "Ginell", 0.0 },
  { "Ginelle", 0.0 },
  { "Giner", 0.0 },
  { "Ginessa", 0.0 },
  { "Ginetta", 0.0 },
  { "Ginette", 0.0 },
  { "Gineva", 0.0 },
  { "Ginevra", 0.0 },
  { "Ging", 0.0 },
  { "Ginger", 0.0008951965 },
  { "Gini", 0.0 },
  { "Ginia", 0.0 },
  { "Ginine", 0.0 },
  { "Ginna", 0.0 },
  { "Ginneh", 0.0 },
  { "Ginnette", 0.0 },
  { "Ginney", 0.0 },
  { "Ginni", 0.0 },
  { "Ginnie", 0.0 },
  { "Ginnifer", 0.0 },
  { "Ginno", 1.0 },
  { "Ginny", 0.0 },
  { "Gino", 0.9989775 },
  { "Ginobili", 1.0 },
  { "Gio", 0.98730963 },
  { "Gioacchino", 1.0 },
  { "Gioanna", 0.0 },
  { "Gioanni", 1.0 },
  { "Giobani", 1.0 },
  { "Giobany", 1.0 },
  { "Gioconda", 0.0 },
  { "Gioele", 1.0 },
  { "Gioia", 0.0 },
  { "Giomar", 1.0 },
  { "Gion", 1.0 },
  { "Giona", 0.0 },
  { "Gioni", 1.0 },
  { "Gionna", 0.0 },
  { "Gionni", 0.9522059 },
  { "Gionnie", 1.0 },
  { "Gionny", 1.0 },
  { "Giordan", 0.890625 },
  { "Giordana", 0.0 },
  { "Giordano", 1.0 },
  { "Giorgi", 1.0 },
  { "Giorgia", 0.0 },
  { "Giorgina", 0.0 },
  { "Giorgio", 1.0 },
  { "Giorgios", 1.0 },
  { "Giorgos", 1.0 },
  { "Giovan", 1.0 },
  { "Giovana", 0.0 },
  { "Giovani", 0.9991882 },
  { "Giovanie", 1.0 },
  { "Giovanii", 1.0 },
  { "Giovann", 1.0 },
  { "Giovanna", 0.0 },
  { "Giovannah", 0.0 },
  { "Giovanne", 1.0 },
  { "Giovanni", 0.9864755 },
  { "Giovannie", 0.99029124 },
  { "Giovannii", 1.0 },
  { "Giovannina", 0.0 },
  { "Giovanny", 0.99492383 },
  { "Giovante", 1.0 },
  { "Giovany", 1.0 },
  { "Giovina", 0.0 },
  { "Giovoni", 1.0 },
  { "Giovonie", 1.0 },
  { "Giovonna", 0.0 },
  { "Giovonni", 1.0 },
  { "Giovonnie", 1.0 },
  { "Gipson", 1.0 },
  { "Gipsy", 0.0 },
  { "Girard", 1.0 },
  { "Girdie", 0.0 },
  { "Girish", 1.0 },
  { "Girl", 0.0 },
  { "Girlean", 0.0 },
  { "Girlee", 0.0 },
  { "Girlene", 0.0 },
  { "Girlie", 0.0 },
  { "Giro", 1.0 },
  { "Girolama", 0.0 },
  { "Girolamo", 1.0 },
  { "Girtha", 0.0 },
  { "Girther", 0.0 },
  { "Girthie", 0.0 },
  { "Girtie", 0.0 },
  { "Girtrude", 0.0 },
  { "Girtrue", 0.0 },
  { "Girty", 0.0 },
  { "Giscard", 1.0 },
  { "Gisel", 0.0 },
  { "Gisela", 0.0 },
  { "Giselda", 0.0 },
  { "Gisele", 0.0 },
  { "Gisell", 0.0 },
  { "Gisella", 0.0 },
  { "Giselle", 0.0010555863 },
  { "Giselly", 0.0 },
  { "Gisely", 0.0 },
  { "Gisette", 0.0 },
  { "Gislaine", 0.0 },
  { "Gissel", 0.0 },
  { "Gissela", 0.0 },
  { "Gissele", 0.0 },
  { "Gissell", 0.0 },
  { "Gissella", 0.0 },
  { "Gisselle", 0.0 },
  { "Gissette", 0.0 },
  { "Gita", 0.0 },
  { "Gitana", 0.0 },
  { "Gitanjali", 0.0 },
  { "Gitel", 0.0 },
  { "Gitta", 0.0 },
  { "Gittel", 0.0 },
  { "Gittle", 0.0 },
  { "Gitty", 0.0 },
  { "Gitzel", 0.0 },
  { "Giuditta", 0.0 },
  { "Giulia", 0.0 },
  { "Giulian", 1.0 },
  { "Giuliana", 0.0 },
  { "Giuliani", 1.0 },
  { "Giulianna", 0.0 },
  { "Giuliano", 1.0 },
  { "Giulietta", 0.0 },
  { "Giuliette", 0.0 },
  { "Giulio", 1.0 },
  { "Giulliana", 0.0 },
  { "Giuseppa", 0.0 },
  { "Giuseppe", 1.0 },
  { "Giuseppi", 1.0 },
  { "Giuseppina", 0.0 },
  { "Giustina", 0.0 },
  { "Givanna", 0.0 },
  { "Givanni", 1.0 },
  { "Given", 1.0 },
  { "Givon", 1.0 },
  { "Givonna", 0.0 },
  { "Givonni", 1.0 },
  { "Giya", 0.0 },
  { "Giyana", 0.0 },
  { "Giyanna", 0.0 },
  { "Gizel", 0.0 },
  { "Gizela", 0.0 },
  { "Gizele", 0.0 },
  { "Gizell", 0.0 },
  { "Gizella", 0.0 },
  { "Gizelle", 0.0 },
  { "Gizem", 0.0 },
  { "Gizzel", 0.0 },
  { "Gizzelle", 0.0 },
  { "Gjon", 1.0 },
  { "Glada", 0.0 },
  { "Gladden", 1.0 },
  { "Glade", 1.0 },
  { "Gladene", 0.0 },
  { "Glades", 0.0 },
  { "Gladie", 0.0 },
  { "Gladies", 0.0 },
  { "Gladimir", 1.0 },
  { "Gladine", 0.0 },
  { "Gladiola", 0.0 },
  { "Gladis", 0.001682935 },
  { "Gladiz", 0.0 },
  { "Gladolia", 0.0 },
  { "Gladston", 1.0 },
  { "Gladstone", 1.0 },
  { "Gladus", 0.0 },
  { "Gladwin", 1.0 },
  { "Gladwyn", 1.0 },
  { "Glady", 0.009208103 },
  { "Gladyce", 0.0 },
  { "Gladyes", 0.0 },
  { "Gladyne", 0.0 },
  { "Gladys", 0.0038288364 },
  { "Gladyse", 0.0 },
  { "Gladystine", 0.0 },
  { "Glafira", 0.0 },
  { "Glanda", 0.0 },
  { "Glasco", 1.0 },
  { "Glasper", 1.0 },
  { "Glatha", 0.0 },
  { "Glayds", 0.0 },
  { "Glean", 1.0 },
  { "Gleason", 1.0 },
  { "Gleb", 1.0 },
  { "Gleda", 0.0 },
  { "Glee", 0.02496879 },
  { "Gleen", 1.0 },
  { "Gleeta", 0.0 },
  { "Gleice", 0.0 },
  { "Glema", 0.0 },
  { "Glen", 0.989334 },
  { "Glena", 0.0 },
  { "Glenadine", 0.0 },
  { "Glenalee", 0.0 },
  { "Glenard", 1.0 },
  { "Glenda", 0.0026989689 },
  { "Glendal", 0.97354496 },
  { "Glendale", 0.8704 },
  { "Glendalee", 0.0 },
  { "Glendalis", 0.0 },
  { "Glendaliz", 0.0 },
  { "Glendall", 1.0 },
  { "Glendaly", 0.0 },
  { "Glendalys", 0.0 },
  { "Glendean", 0.0 },
  { "Glendel", 0.96875 },
  { "Glendell", 0.8421955 },
  { "Glenden", 1.0 },
  { "Glendene", 0.0 },
  { "Glender", 0.0 },
  { "Glendi", 0.0 },
  { "Glendia", 0.0 },
  { "Glendine", 0.0 },
  { "Glendle", 1.0 },
  { "Glendola", 0.0 },
  { "Glendolyn", 0.0 },
  { "Glendon", 0.98619795 },
  { "Glendora", 0.0 },
  { "Glendoria", 0.0 },
  { "Glendoris", 0.0 },
  { "Glendy", 0.0 },
  { "Glenell", 0.0 },
  { "Glenese", 0.0 },
  { "Glenesha", 0.0 },
  { "Glenetta", 0.0 },
  { "Glenette", 0.0 },
  { "Gleneva", 0.0 },
  { "Glenford", 1.0 },
  { "Glenice", 0.0 },
  { "Glenis", 0.06323185 },
  { "Glenise", 0.0 },
  { "Glenisha", 0.0 },
  { "Glenita", 0.0 },
  { "Glenmore", 1.0 },
  { "Glenn", 0.99144095 },
  { "Glenna", 0.00019564111 },
  { "Glennard", 1.0 },
  { "Glennda", 0.0 },
  { "Glenndora", 0.0 },
  { "Glenne", 0.0 },
  { "Glennell", 0.0 },
  { "Glennetta", 0.0 },
  { "Glennette", 0.0 },
  { "Glennia", 0.0 },
  { "Glennice", 0.0 },
  { "Glennie", 0.13174461 },
  { "Glennis", 0.2598198 },
  { "Glennisha", 0.0 },
  { "Glennon", 1.0 },
  { "Glennora", 0.0 },
  { "Glennwood", 1.0 },
  { "Glenny", 0.3902439 },
  { "Glennys", 0.0 },
  { "Glenola", 0.0 },
  { "Glenora", 0.0 },
  { "Glenrose", 0.0 },
  { "Glenroy", 1.0 },
  { "Glenston", 1.0 },
  { "Glenton", 1.0 },
  { "Glenva", 0.0 },
  { "Glenville", 1.0 },
  { "Glenward", 1.0 },
  { "Glenwood", 1.0 },
  { "Gleny", 0.0 },
  { "Glenyce", 0.0 },
  { "Glenys", 0.0 },
  { "Gleora", 0.0 },
  { "Glessie", 0.0 },
  { "Glida", 0.0 },
  { "Glin", 1.0 },
  { "Glinda", 0.0 },
  { "Glinnie", 0.0 },
  { "Glo", 0.0 },
  { "Glodine", 0.0 },
  { "Glora", 0.0 },
  { "Glorene", 0.0 },
  { "Gloretha", 0.0 },
  { "Gloretta", 0.0 },
  { "Glori", 0.0 },
  { "Gloria", 0.0042111846 },
  { "Gloriajean", 0.0 },
  { "Glorian", 0.0 },
  { "Gloriana", 0.0 },
  { "Gloriane", 0.0 },
  { "Gloriann", 0.0 },
  { "Glorianna", 0.0 },
  { "Glorianne", 0.0 },
  { "Gloribel", 0.0 },
  { "Glorice", 0.0 },
  { "Gloricely", 0.0 },
  { "Glorida", 0.0 },
  { "Glorie", 0.0 },
  { "Glories", 0.0 },
  { "Glorietta", 0.0 },
  { "Gloriette", 0.0 },
  { "Glorimar", 0.0 },
  { "Glorine", 0.0 },
  { "Glorious", 0.0 },
  { "Gloris", 0.0 },
  { "Gloristeen", 0.0 },
  { "Gloristine", 0.0 },
  { "Glorius", 0.0 },
  { "Glory", 0.00673989 },
  { "Glorya", 0.0 },
  { "Gloryann", 0.0 },
  { "Gloryvee", 0.0 },
  { "Glossie", 0.0 },
  { "Gloster", 1.0 },
  { "Glover", 0.99551165 },
  { "Gloyd", 1.0 },
  { "Glyda", 0.0 },
  { "Glyde", 1.0 },
  { "Glyn", 0.93762714 },
  { "Glyna", 0.0 },
  { "Glynda", 0.0 },
  { "Glyndia", 0.0 },
  { "Glyndon", 0.9354839 },
  { "Glyndora", 0.0 },
  { "Glynes", 0.0 },
  { "Glynice", 0.0 },
  { "Glynis", 0.0049407114 },
  { "Glyniss", 0.0 },
  { "Glynn", 0.92215794 },
  { "Glynna", 0.0 },
  { "Glynne", 0.0 },
  { "Glynnis", 0.0 },
  { "Gniya", 0.0 },
  { "Gobel", 1.0 },
  { "Gobind", 1.0 },
  { "Goble", 1.0 },
  { "God", 1.0 },
  { "Goddess", 0.0 },
  { "Godfred", 1.0 },
  { "Godfrey", 1.0 },
  { "Godiva", 0.0 },
  { "Godofredo", 1.0 },
  { "Godric", 1.0 },
  { "Godrick", 1.0 },
  { "Godson", 1.0 },
  { "Godswill", 1.0 },
  { "Godwill", 1.0 },
  { "Godwin", 1.0 },
  { "Goebel", 1.0 },
  { "Goeffrey", 1.0 },
  { "Goerge", 1.0 },
  { "Goff", 1.0 },
  { "Gogi", 0.0 },
  { "Gohan", 1.0 },
  { "Gohar", 0.0 },
  { "Goku", 1.0 },
  { "Gokul", 1.0 },
  { "Gola", 0.01607717 },
  { "Golan", 1.0 },
  { "Gold", 0.5 },
  { "Golda", 0.0 },
  { "Golde", 0.0 },
  { "Goldean", 0.0 },
  { "Golden", 0.5678042 },
  { "Goldena", 0.0 },
  { "Goldene", 0.0 },
  { "Goldi", 0.0 },
  { "Goldia", 0.0 },
  { "Goldie", 0.011982481 },
  { "Goldina", 0.0 },
  { "Goldine", 0.0 },
  { "Goldman", 1.0 },
  { "Goldmon", 1.0 },
  { "Goldy", 0.0 },
  { "Goldye", 0.0 },
  { "Goldyn", 0.0 },
  { "Golena", 0.0 },
  { "Goliath", 1.0 },
  { "Golie", 0.0 },
  { "Golnaz", 0.0 },
  { "Gomer", 1.0 },
  { "Gomez", 1.0 },
  { "Gonsalo", 1.0 },
  { "Gonzala", 0.0 },
  { "Gonzales", 1.0 },
  { "Gonzalo", 1.0 },
  { "Gonzella", 0.0 },
  { "Goodluck", 1.0 },
  { "Goodman", 1.0 },
  { "Goodness", 0.0 },
  { "Goodwin", 1.0 },
  { "Gopal", 1.0 },
  { "Gopi", 0.0 },
  { "Gopika", 0.0 },
  { "Gor", 1.0 },
  { "Goran", 1.0 },
  { "Gordan", 1.0 },
  { "Gordana", 0.0 },
  { "Gordean", 0.0 },
  { "Gorden", 1.0 },
  { "Gordie", 0.71985817 },
  { "Gordon", 0.99563205 },
  { "Gordy", 1.0 },
  { "Goree", 1.0 },
  { "Gorete", 0.0 },
  { "Goretti", 0.0 },
  { "Goretty", 0.0 },
  { "Gorge", 1.0 },
  { "Gorgeous", 0.0 },
  { "Gorgonio", 1.0 },
  { "Gorham", 1.0 },
  { "Gorizia", 0.0 },
  { "Gorje", 1.0 },
  { "Gorkem", 1.0 },
  { "Gorman", 1.0 },
  { "Goro", 1.0 },
  { "Goshen", 1.0 },
  { "Gospel", 1.0 },
  { "Goten", 1.0 },
  { "Gotham", 1.0 },
  { "Gottfred", 1.0 },
  { "Gottfried", 1.0 },
  { "Gotti", 1.0 },
  { "Gottlieb", 1.0 },
  { "Gottlob", 1.0 },
  { "Gould", 1.0 },
  { "Gouri", 0.0 },
  { "Govan", 1.0 },
  { "Govani", 1.0 },
  { "Govanna", 0.0 },
  { "Govanni", 1.0 },
  { "Govanny", 1.0 },
  { "Govany", 1.0 },
  { "Governor", 1.0 },
  { "Govind", 1.0 },
  { "Govinda", 1.0 },
  { "Gowri", 0.0 },
  { "Gowtham", 1.0 },
  { "Grabiel", 1.0 },
  { "Grabiela", 0.0 },
  { "Grabriel", 1.0 },
  { "Grabriela", 0.0 },
  { "Grace", 0.0027209031 },
  { "Gracean", 0.0 },
  { "Graceann", 0.0 },
  { "Graceanna", 0.0 },
  { "Graceanne", 0.0 },
  { "Gracee", 0.0 },
  { "Graceelizabeth", 0.0 },
  { "Gracelan", 0.0 },
  { "Graceland", 0.0 },
  { "Gracelee", 0.0 },
  { "Graceleigh", 0.0 },
  { "Gracelin", 0.0 },
  { "Gracelinn", 0.0 },
  { "Gracella", 0.0 },
  { "Gracely", 0.0 },
  { "Gracelyn", 0.0 },
  { "Gracelynd", 0.0 },
  { "Gracelynn", 0.0 },
  { "Gracelynne", 0.0 },
  { "Gracemarie", 0.0 },
  { "Gracemary", 0.0 },
  { "Gracen", 0.2926926 },
  { "Graceson", 0.9037657 },
  { "Gracesyn", 0.0 },
  { "Gracey", 0.0 },
  { "Graceyn", 0.0 },
  { "Graci", 0.0 },
  { "Gracia", 0.0 },
  { "Gracian", 1.0 },
  { "Graciana", 0.0 },
  { "Gracianna", 0.0 },
  { "Gracianne", 0.0 },
  { "Graciano", 1.0 },
  { "Gracie", 0.000628018 },
  { "Gracieann", 0.0 },
  { "Graciee", 0.0 },
  { "Graciela", 0.0025384014 },
  { "Graciella", 0.0 },
  { "Gracielle", 0.0 },
  { "Gracielynn", 0.0 },
  { "Graciemae", 0.0 },
  { "Gracilyn", 0.0 },
  { "Gracilynn", 0.0 },
  { "Gracin", 0.67180276 },
  { "Gracious", 0.0 },
  { "Graclyn", 0.0 },
  { "Graclynn", 0.0 },
  { "Gracy", 0.0 },
  { "Gracye", 0.0 },
  { "Gracyn", 0.033745103 },
  { "Gracynn", 0.0 },
  { "Graddy", 1.0 },
  { "Graden", 1.0 },
  { "Gradey", 1.0 },
  { "Gradie", 0.68488747 },
  { "Gradin", 1.0 },
  { "Gradon", 1.0 },
  { "Grady", 0.98560005 },
  { "Gradyn", 1.0 },
  { "Gradys", 0.0 },
  { "Grae", 0.43609023 },
  { "Graecen", 1.0 },
  { "Graecie", 0.0 },
  { "Graecyn", 0.09090909 },
  { "Graeden", 1.0 },
  { "Graedy", 1.0 },
  { "Graeham", 1.0 },
  { "Graelyn", 0.0 },
  { "Graelynn", 0.0 },
  { "Graem", 1.0 },
  { "Graeme", 1.0 },
  { "Graer", 1.0 },
  { "Graesen", 1.0 },
  { "Graeson", 0.9718574 },
  { "Graesyn", 0.7777778 },
  { "Grafton", 1.0 },
  { "Graham", 0.9964698 },
  { "Grahame", 1.0 },
  { "Grahm", 1.0 },
  { "Graice", 0.0 },
  { "Graicen", 1.0 },
  { "Graicyn", 0.0 },
  { "Graiden", 1.0 },
  { "Graidy", 1.0 },
  { "Graidys", 0.0 },
  { "Graig", 1.0 },
  { "Grainger", 1.0 },
  { "Grainne", 0.0 },
  { "Graisen", 1.0 },
  { "Graison", 0.8842444 },
  { "Graisyn", 0.40625 },
  { "Gralin", 1.0 },
  { "Gralyn", 1.0 },
  { "Gram", 1.0 },
  { "Gramm", 1.0 },
  { "Granada", 0.0 },
  { "Granderson", 1.0 },
  { "Grandin", 1.0 },
  { "Grandison", 1.0 },
  { "Grandon", 1.0 },
  { "Grandville", 1.0 },
  { "Granger", 1.0 },
  { "Granite", 1.0 },
  { "Grant", 0.99832225 },
  { "Grantham", 1.0 },
  { "Grantland", 1.0 },
  { "Grantlee", 1.0 },
  { "Grantley", 1.0 },
  { "Granvel", 1.0 },
  { "Granvil", 1.0 },
  { "Granvill", 1.0 },
  { "Granville", 1.0 },
  { "Graple", 0.0 },
  { "Grasen", 1.0 },
  { "Grasha", 0.0 },
  { "Grasiela", 0.0 },
  { "Grason", 1.0 },
  { "Grasyn", 0.6911765 },
  { "Gratia", 0.0 },
  { "Graven", 1.0 },
  { "Graves", 1.0 },
  { "Graviel", 1.0 },
  { "Graviela", 0.0 },
  { "Gray", 0.912027 },
  { "Grayce", 0.0 },
  { "Graycee", 0.0 },
  { "Graycelyn", 0.0 },
  { "Graycelynn", 0.0 },
  { "Graycen", 0.31157637 },
  { "Grayci", 0.0 },
  { "Graycie", 0.0 },
  { "Graycin", 0.44594595 },
  { "Grayden", 1.0 },
  { "Graydin", 1.0 },
  { "Graydon", 1.0 },
  { "Graye", 0.33333334 },
  { "Grayer", 1.0 },
  { "Graylan", 1.0 },
  { "Grayland", 1.0 },
  { "Graylee", 0.0 },
  { "Graylen", 1.0 },
  { "Graylin", 0.9318637 },
  { "Grayling", 1.0 },
  { "Graylon", 1.0 },
  { "Graylyn", 0.1 },
  { "Graylynn", 0.0 },
  { "Graysen", 0.7989418 },
  { "Graysie", 0.0 },
  { "Graysin", 1.0 },
  { "Grayson", 0.94803876 },
  { "Graysyn", 0.9019608 },
  { "Grayton", 1.0 },
  { "Grazia", 0.0 },
  { "Graziana", 0.0 },
  { "Graziano", 1.0 },
  { "Graziella", 0.0 },
  { "Greagory", 1.0 },
  { "Great", 1.0 },
  { "Greathel", 0.0 },
  { "Greatness", 1.0 },
  { "Grechen", 0.0 },
  { "Grecia", 0.0 },
  { "Greco", 1.0 },
  { "Greda", 0.0 },
  { "Gredmarie", 0.0 },
  { "Greeley", 0.6229508 },
  { "Greely", 1.0 },
  { "Green", 1.0 },
  { "Greenberry", 1.0 },
  { "Greene", 1.0 },
  { "Greenlea", 0.0 },
  { "Greenlee", 0.0 },
  { "Greenleigh", 0.0 },
  { "Greenley", 0.0 },
  { "Greenly", 0.0 },
  { "Greenville", 1.0 },
  { "Greenwood", 1.0 },
  { "Greer", 0.18550412 },
  { "Greeta", 0.0 },
  { "Greg", 0.9981589 },
  { "Gregario", 1.0 },
  { "Gregary", 1.0 },
  { "Gregeory", 1.0 },
  { "Gregery", 1.0 },
  { "Gregg", 0.9969088 },
  { "Greggery", 1.0 },
  { "Greggory", 1.0 },
  { "Gregoire", 1.0 },
  { "Gregor", 1.0 },
  { "Gregorey", 1.0 },
  { "Gregorg", 1.0 },
  { "Gregori", 1.0 },
  { "Gregoria", 0.0027014858 },
  { "Gregorie", 1.0 },
  { "Gregorio", 0.9995153 },
  { "Gregorios", 1.0 },
  { "Gregorita", 0.0 },
  { "Gregory", 0.99596334 },
  { "Gregoy", 1.0 },
  { "Gregrey", 1.0 },
  { "Gregrory", 1.0 },
  { "Gregroy", 1.0 },
  { "Gregry", 1.0 },
  { "Gregson", 1.0 },
  { "Greicy", 0.0 },
  { "Greidi", 0.0 },
  { "Greidis", 0.0 },
  { "Greidy", 0.0 },
  { "Greidys", 0.0 },
  { "Greig", 1.0 },
  { "Greighson", 1.0 },
  { "Greison", 1.0 },
  { "Greisy", 0.0 },
  { "Greko", 1.0 },
  { "Grenda", 0.0 },
  { "Grenisha", 0.0 },
  { "Grenville", 1.0 },
  { "Greogory", 1.0 },
  { "Greogry", 1.0 },
  { "Greory", 1.0 },
  { "Gresa", 0.0 },
  { "Gresham", 1.0 },
  { "Gresia", 0.0 },
  { "Greta", 0.00020475859 },
  { "Gretal", 0.0 },
  { "Gretchan", 0.0 },
  { "Gretchen", 0.00071386975 },
  { "Gretchin", 0.0 },
  { "Gretchyn", 0.0 },
  { "Grete", 0.0 },
  { "Gretel", 0.0 },
  { "Gretell", 0.0 },
  { "Gretha", 0.0 },
  { "Grethe", 0.0 },
  { "Grethel", 0.0 },
  { "Gretna", 0.0 },
  { "Gretta", 0.0 },
  { "Grettel", 0.0 },
  { "Grettell", 0.0 },
  { "Grey", 0.86740625 },
  { "Greyce", 0.0 },
  { "Greycen", 0.89830506 },
  { "Greycin", 1.0 },
  { "Greyden", 1.0 },
  { "Greydi", 0.0 },
  { "Greydis", 0.0 },
  { "Greydon", 1.0 },
  { "Greydy", 0.0 },
  { "Greydys", 0.0 },
  { "Greyer", 1.0 },
  { "Greylan", 1.0 },
  { "Greylin", 0.7619048 },
  { "Greyling", 1.0 },
  { "Greylon", 1.0 },
  { "Greylyn", 0.0 },
  { "Greylynn", 0.0 },
  { "Greys", 0.0 },
  { "Greysen", 0.8919949 },
  { "Greysi", 0.0 },
  { "Greysin", 1.0 },
  { "Greyson", 0.97980267 },
  { "Greyston", 1.0 },
  { "Greysun", 1.0 },
  { "Greysyn", 0.8913044 },
  { "Greyton", 1.0 },
  { "Gricel", 0.0 },
  { "Gricelda", 0.0 },
  { "Gricell", 0.0 },
  { "Grieg", 1.0 },
  { "Grier", 0.5 },
  { "Griezmann", 1.0 },
  { "Griff", 1.0 },
  { "Griffen", 1.0 },
  { "Griffey", 1.0 },
  { "Griffin", 0.9850389 },
  { "Griffinn", 1.0 },
  { "Griffith", 1.0 },
  { "Griffon", 1.0 },
  { "Griffyn", 0.8770764 },
  { "Grifin", 1.0 },
  { "Grigor", 1.0 },
  { "Grigoriy", 1.0 },
  { "Grigory", 1.0 },
  { "Grimm", 1.0 },
  { "Grinda", 0.0 },
  { "Grindl", 0.0 },
  { "Grindle", 0.0 },
  { "Griscelda", 0.0 },
  { "Grisel", 0.0043130005 },
  { "Grisela", 0.0 },
  { "Griselda", 0.0069420733 },
  { "Grisell", 0.0 },
  { "Griselle", 0.0 },
  { "Grisham", 1.0 },
  { "Grishma", 0.0 },
  { "Grissel", 0.0 },
  { "Grissom", 1.0 },
  { "Gristian", 1.0 },
  { "Grizel", 0.0 },
  { "Grizelda", 0.0 },
  { "Grizzly", 1.0 },
  { "Gross", 1.0 },
  { "Grove", 1.0 },
  { "Grovene", 0.0 },
  { "Grover", 0.9965893 },
  { "Groves", 1.0 },
  { "Grundy", 1.0 },
  { "Gryffen", 1.0 },
  { "Gryffin", 1.0 },
  { "Gryffon", 1.0 },
  { "Gryphon", 1.0 },
  { "Grzegorz", 1.0 },
  { "Guadalupe", 0.2771791 },
  { "Guadelupe", 0.0 },
  { "Guadlupe", 0.4489796 },
  { "Guadulupe", 0.0 },
  { "Guage", 1.0 },
  { "Gualberto", 1.0 },
  { "Guan", 1.0 },
  { "Guardian", 1.0 },
  { "Guarionex", 1.0 },
  { "Gudalupe", 0.15625 },
  { "Gudelia", 0.0 },
  { "Gudrun", 0.0 },
  { "Guelda", 0.0 },
  { "Guendalina", 0.0 },
  { "Guendi", 0.0 },
  { "Guendolyn", 0.0 },
  { "Guenevere", 0.0 },
  { "Guenter", 1.0 },
  { "Guenther", 1.0 },
  { "Guerdon", 1.0 },
  { "Guerin", 1.0 },
  { "Guerina", 0.0 },
  { "Guerino", 1.0 },
  { "Guerline", 0.0 },
  { "Guerrero", 1.0 },
  { "Guerrino", 1.0 },
  { "Guerry", 1.0 },
  { "Guess", 1.0 },
  { "Guhan", 1.0 },
  { "Guida", 0.0 },
  { "Guido", 1.0 },
  { "Guila", 0.0 },
  { "Guilbert", 1.0 },
  { "Guilda", 0.0 },
  { "Guilford", 1.0 },
  { "Guilherme", 1.0 },
  { "Guilian", 1.0 },
  { "Guiliana", 0.0 },
  { "Guilianna", 0.0 },
  { "Guiliano", 1.0 },
  { "Guilio", 1.0 },
  { "Guillaume", 1.0 },
  { "Guillerma", 0.0 },
  { "Guillermina", 0.0 },
  { "Guillermo", 0.9974896 },
  { "Guilliana", 0.0 },
  { "Guilmar", 1.0 },
  { "Guin", 1.0 },
  { "Guinda", 0.0 },
  { "Guinette", 0.0 },
  { "Guinevere", 0.0 },
  { "Guinivere", 0.0 },
  { "Guinn", 1.0 },
  { "Guinness", 1.0 },
  { "Guinnevere", 0.0 },
  { "Guisel", 0.0 },
  { "Guisela", 0.0 },
  { "Guisell", 0.0 },
  { "Guiselle", 0.0 },
  { "Guiseppe", 1.0 },
  { "Guiseppi", 1.0 },
  { "Guisselle", 0.0 },
  { "Guistino", 1.0 },
  { "Gul", 0.0 },
  { "Gula", 0.0 },
  { "Guled", 1.0 },
  { "Gulian", 1.0 },
  { "Guliana", 0.0 },
  { "Gulianna", 0.0 },
  { "Gullermo", 1.0 },
  { "Gumaro", 1.0 },
  { "Gumecindo", 1.0 },
  { "Gumercindo", 1.0 },
  { "Gumesindo", 1.0 },
  { "Gunar", 1.0 },
  { "Gunars", 1.0 },
  { "Gunda", 0.0 },
  { "Gunder", 1.0 },
  { "Guneet", 0.0 },
  { "Guner", 1.0 },
  { "Gunhild", 0.0 },
  { "Gunnar", 0.9987183 },
  { "Gunnard", 1.0 },
  { "Gunnarr", 1.0 },
  { "Gunner", 0.9992574 },
  { "Gunnison", 1.0 },
  { "Gunter", 1.0 },
  { "Gunther", 1.0 },
  { "Gunvor", 0.0 },
  { "Guransh", 1.0 },
  { "Gurbani", 0.0 },
  { "Gurbir", 1.0 },
  { "Gurdeep", 1.0 },
  { "Gurdit", 1.0 },
  { "Gurdon", 1.0 },
  { "Gurfateh", 1.0 },
  { "Gurinder", 1.0 },
  { "Gurjeet", 1.0 },
  { "Gurjit", 1.0 },
  { "Gurjot", 0.9653179 },
  { "Gurkamal", 1.0 },
  { "Gurkirat", 0.84210527 },
  { "Gurleen", 0.0 },
  { "Gurley", 1.0 },
  { "Gurlie", 0.0 },
  { "Gurman", 0.9015151 },
  { "Gurnaaz", 0.0 },
  { "Gurneet", 0.0 },
  { "Gurney", 1.0 },
  { "Gurnie", 1.0 },
  { "Gurnoor", 0.3559322 },
  { "Gurpal", 1.0 },
  { "Gurpreet", 0.64389235 },
  { "Gurseerat", 0.0 },
  { "Gurshaan", 1.0 },
  { "Gurshan", 1.0 },
  { "Gurshawn", 1.0 },
  { "Gursimar", 0.0 },
  { "Gursimran", 0.23076923 },
  { "Gursirat", 0.0 },
  { "Gurtaaj", 1.0 },
  { "Gurtaj", 1.0 },
  { "Gurtej", 1.0 },
  { "Gurtha", 0.0 },
  { "Gurtie", 0.0 },
  { "Gurtrude", 0.0 },
  { "Gurtrue", 0.0 },
  { "Guru", 1.0 },
  { "Gurveen", 0.0 },
  { "Gurveer", 1.0 },
  { "Gurvinder", 1.0 },
  { "Gurvir", 1.0 },
  { "Gurvis", 1.0 },
  { "Gus", 0.99973 },
  { "Guss", 1.0 },
  { "Gussie", 0.07742754 },
  { "Gust", 1.0 },
  { "Gusta", 0.0 },
  { "Gustabo", 1.0 },
  { "Gustaf", 1.0 },
  { "Gustas", 1.0 },
  { "Gustauo", 1.0 },
  { "Gustav", 1.0 },
  { "Gustava", 0.0 },
  { "Gustave", 1.0 },
  { "Gustavia", 0.0 },
  { "Gustavo", 0.9971461 },
  { "Gustavus", 1.0 },
  { "Gustena", 0.0 },
  { "Guster", 1.0 },
  { "Gustie", 0.045045044 },
  { "Gustin", 1.0 },
  { "Gustina", 0.0 },
  { "Gustine", 0.0 },
  { "Guston", 1.0 },
  { "Gusty", 1.0 },
  { "Guthrie", 0.9844961 },
  { "Guy", 0.99477285 },
  { "Guyann", 0.0 },
  { "Guyanne", 0.0 },
  { "Guydra", 0.0 },
  { "Guye", 1.0 },
  { "Guyla", 0.0 },
  { "Guylene", 0.0 },
  { "Guymon", 1.0 },
  { "Guynell", 0.0 },
  { "Guynelle", 0.0 },
  { "Guyneth", 0.0 },
  { "Guyon", 1.0 },
  { "Guyton", 1.0 },
  { "Guztavo", 1.0 },
  { "Gwain", 1.0 },
  { "Gwan", 0.0 },
  { "Gwana", 0.0 },
  { "Gwanda", 0.0 },
  { "Gwelda", 0.0 },
  { "Gwen", 0.009847596 },
  { "Gwena", 0.0 },
  { "Gwenaelle", 0.0 },
  { "Gwenavere", 0.0 },
  { "Gwenda", 0.0 },
  { "Gwendalin", 0.0 },
  { "Gwendalina", 0.0 },
  { "Gwendalyn", 0.0 },
  { "Gwendalynn", 0.0 },
  { "Gwendel", 0.0 },
  { "Gwendell", 0.09090909 },
  { "Gwendelyn", 0.0 },
  { "Gwendelynn", 0.0 },
  { "Gwender", 0.0 },
  { "Gwendia", 0.0 },
  { "Gwendloyn", 0.0 },
  { "Gwendlyn", 0.0 },
  { "Gwendlyon", 0.0 },
  { "Gwendol", 0.0 },
  { "Gwendola", 0.0 },
  { "Gwendolen", 0.0 },
  { "Gwendolene", 0.0 },
  { "Gwendolin", 0.0 },
  { "Gwendoline", 0.0 },
  { "Gwendoly", 0.0 },
  { "Gwendolyn", 0.00283902 },
  { "Gwendolyne", 0.0 },
  { "Gwendolynn", 0.0 },
  { "Gwendolynne", 0.0 },
  { "Gwendoyln", 0.0 },
  { "Gwendy", 0.0 },
  { "Gwendylan", 0.0 },
  { "Gwendylon", 0.0 },
  { "Gwenell", 0.0 },
  { "Gweneth", 0.0 },
  { "Gwenetta", 0.0 },
  { "Gwenette", 0.0 },
  { "Gweneva", 0.0 },
  { "Gwenevere", 0.0 },
  { "Gweneviere", 0.0 },
  { "Gwenevieve", 0.0 },
  { "Gwenith", 0.0 },
  { "Gwenivere", 0.0 },
  { "Gwenlyn", 0.0 },
  { "Gwenn", 0.0 },
  { "Gwenna", 0.0 },
  { "Gwennan", 0.0 },
  { "Gwenne", 0.0 },
  { "Gwenneth", 0.0 },
  { "Gwennetta", 0.0 },
  { "Gwennette", 0.0 },
  { "Gwennyth", 0.0 },
  { "Gwenyth", 0.0 },
  { "Gwenyvere", 0.0 },
  { "Gwili", 0.0 },
  { "Gwilym", 1.0 },
  { "Gwin", 0.7189189 },
  { "Gwinda", 0.0 },
  { "Gwindolyn", 0.0 },
  { "Gwinn", 0.7826087 },
  { "Gwladys", 0.0 },
  { "Gwydion", 1.0 },
  { "Gwyn", 0.1282824 },
  { "Gwynavere", 0.0 },
  { "Gwynda", 0.0 },
  { "Gwyndolin", 0.0 },
  { "Gwyndolyn", 0.0 },
  { "Gwyne", 0.0 },
  { "Gwyned", 0.0 },
  { "Gwyneth", 0.0 },
  { "Gwynetta", 0.0 },
  { "Gwynette", 0.0 },
  { "Gwynevere", 0.0 },
  { "Gwynith", 0.0 },
  { "Gwynn", 0.12337662 },
  { "Gwynne", 0.021382751 },
  { "Gwynneth", 0.0 },
  { "Gwytha", 0.0 },
  { "Gy", 1.0 },
  { "Gyan", 1.0 },
  { "Gyana", 0.0 },
  { "Gyanna", 0.0 },
  { "Gyanni", 0.0 },
  { "Gyasi", 1.0 },
  { "Gyda", 0.0 },
  { "Gyla", 0.0 },
  { "Gyle", 1.0 },
  { "Gyles", 1.0 },
  { "Gyllian", 0.0 },
  { "Gyna", 0.0 },
  { "Gynesis", 0.0 },
  { "Gyneth", 0.0 },
  { "Gynith", 0.0 },
  { "Gypsie", 0.0 },
  { "Gypsy", 0.0 },
  { "Gyra", 0.0 },
  { "Gysel", 0.0 },
  { "Gyselle", 0.0 },
  { "Gyzelle", 0.0 },
  { "Gzim", 1.0 },
  { "Ha", 0.14747475 },
  { "Haadi", 1.0 },
  { "Haadiya", 0.0 },
  { "Haadiyah", 0.0 },
  { "Haaheo", 1.0 },
  { "Haajar", 0.0 },
  { "Haaken", 1.0 },
  { "Haakim", 1.0 },
  { "Haakon", 1.0 },
  { "Haala", 0.0 },
  { "Haamid", 1.0 },
  { "Haani", 0.0 },
  { "Haania", 0.0 },
  { "Haaniya", 0.0 },
  { "Haarika", 0.0 },
  { "Haaris", 1.0 },
  { "Haasan", 1.0 },
  { "Haashim", 1.0 },
  { "Haashir", 1.0 },
  { "Haasini", 0.0 },
  { "Haaziq", 1.0 },
  { "Habacuc", 1.0 },
  { "Habeba", 0.0 },
  { "Habeeb", 1.0 },
  { "Habeeba", 0.0 },
  { "Haben", 0.8039216 },
  { "Habert", 1.0 },
  { "Habib", 1.0 },
  { "Habiba", 0.0 },
  { "Habibah", 0.0 },
  { "Habibatou", 0.0 },
  { "Habil", 1.0 },
  { "Habon", 0.0 },
  { "Habraham", 1.0 },
  { "Habram", 1.0 },
  { "Habsa", 0.0 },
  { "Haby", 0.0 },
  { "Hachiro", 1.0 },
  { "Had", 1.0 },
  { "Hada", 0.0 },
  { "Hadalyn", 0.0 },
  { "Hadan", 1.0 },
  { "Hadar", 0.0 },
  { "Hadara", 0.0 },
  { "Hadarah", 0.0 },
  { "Hadari", 1.0 },
  { "Hadas", 0.0 },
  { "Hadasa", 0.0 },
  { "Hadasah", 0.0 },
  { "Hadasha", 0.0 },
  { "Hadassa", 0.0 },
  { "Hadassah", 0.0 },
  { "Haddasah", 0.0 },
  { "Hadden", 0.94335514 },
  { "Haddi", 0.0 },
  { "Haddie", 0.0 },
  { "Haddison", 0.0 },
  { "Haddley", 0.0 },
  { "Haddon", 0.9829684 },
  { "Haddox", 1.0 },
  { "Haddy", 0.03816794 },
  { "Hade", 0.0 },
  { "Hadeed", 1.0 },
  { "Hadeel", 0.0 },
  { "Hadeer", 0.0 },
  { "Hadel", 0.0 },
  { "Hadelyn", 0.0 },
  { "Haden", 0.95822245 },
  { "Hades", 1.0 },
  { "Hadessa", 0.0 },
  { "Hadessah", 0.0 },
  { "Hadi", 1.0 },
  { "Hadia", 0.0 },
  { "Hadiah", 0.0 },
  { "Hadiatou", 0.0 },
  { "Hadie", 0.0 },
  { "Hadija", 0.0 },
  { "Hadil", 0.0 },
  { "Hadilyn", 0.0 },
  { "Hadin", 1.0 },
  { "Hadiya", 0.0 },
  { "Hadiyah", 0.0 },
  { "Hadiyyah", 0.0 },
  { "Hadiza", 0.0 },
  { "Hadja", 0.0 },
  { "Hadji", 1.0 },
  { "Hadlea", 0.0 },
  { "Hadlee", 0.0062552127 },
  { "Hadlei", 0.0 },
  { "Hadleigh", 0.0 },
  { "Hadley", 0.07144515 },
  { "Hadleyann", 0.0 },
  { "Hadli", 0.0 },
  { "Hadlie", 0.0 },
  { "Hadly", 0.0 },
  { "Hadlyn", 0.0 },
  { "Hadlynn", 0.0 },
  { "Hadrian", 1.0 },
  { "Hadriel", 1.0 },
  { "Hadrien", 1.0 },
  { "Hady", 0.78571427 },
  { "Hadya", 0.0 },
  { "Hadyn", 0.5402414 },
  { "Hae", 0.0 },
  { "Haeden", 0.8768116 },
  { "Haedon", 1.0 },
  { "Haedyn", 0.3632653 },
  { "Haegan", 1.0 },
  { "Haegen", 1.0 },
  { "Haein", 0.0 },
  { "Hael", 0.5 },
  { "Haelee", 0.0 },
  { "Haeleigh", 0.0 },
  { "Haeley", 0.0 },
  { "Haeli", 0.0 },
  { "Haelie", 0.0 },
  { "Haely", 0.0 },
  { "Haelyn", 0.0 },
  { "Haelynn", 0.0 },
  { "Haether", 0.0 },
  { "Haeun", 0.0 },
  { "Haeven", 0.11764706 },
  { "Haevyn", 0.0 },
  { "Haevynn", 0.0 },
  { "Hafeez", 1.0 },
  { "Hafeeza", 0.0 },
  { "Hafeezah", 0.0 },
  { "Hafford", 1.0 },
  { "Hafid", 1.0 },
  { "Hafiz", 1.0 },
  { "Hafsa", 0.0 },
  { "Hafsah", 0.0 },
  { "Hafso", 0.0 },
  { "Hagan", 0.90692306 },
  { "Hagar", 0.0 },
  { "Hagen", 0.9336775 },
  { "Hageo", 1.0 },
  { "Hager", 0.68292683 },
  { "Haggai", 1.0 },
  { "Haggard", 1.0 },
  { "Hagood", 1.0 },
  { "Hagop", 1.0 },
  { "Hahna", 0.0 },
  { "Hai", 0.96428573 },
  { "Haidan", 0.9104478 },
  { "Haidar", 1.0 },
  { "Haide", 0.0 },
  { "Haidee", 0.0 },
  { "Haiden", 0.69850683 },
  { "Haider", 1.0 },
  { "Haidi", 0.0 },
  { "Haidin", 0.8076923 },
  { "Haidon", 1.0 },
  { "Haidy", 0.0 },
  { "Haidyn", 0.25667152 },
  { "Haidynn", 0.0 },
  { "Haiely", 0.0 },
  { "Haifa", 0.0 },
  { "Haig", 1.0 },
  { "Haigen", 0.8064516 },
  { "Haik", 1.0 },
  { "Haila", 0.0 },
  { "Haile", 0.19740778 },
  { "Hailea", 0.0 },
  { "Haileah", 0.0 },
  { "Hailee", 0.00041666668 },
  { "Haileen", 0.0 },
  { "Hailei", 0.0 },
  { "Haileigh", 0.0 },
  { "Hailen", 0.0 },
  { "Hailey", 0.0016877363 },
  { "Haileyann", 0.0 },
  { "Haileyjade", 0.0 },
  { "Haileyjo", 0.0 },
  { "Haileymarie", 0.0 },
  { "Haileyrose", 0.0 },
  { "Haili", 0.0 },
  { "Hailie", 0.0 },
  { "Hailiey", 0.0 },
  { "Haille", 0.0 },
  { "Haillee", 0.0 },
  { "Hailley", 0.0 },
  { "Haillie", 0.0 },
  { "Hailly", 0.0 },
  { "Hailo", 0.0 },
  { "Haily", 0.0 },
  { "Hailye", 0.0 },
  { "Hailyn", 0.0 },
  { "Hailynn", 0.0 },
  { "Haim", 1.0 },
  { "Haines", 1.0 },
  { "Hairl", 1.0 },
  { "Hairo", 1.0 },
  { "Haislee", 0.0 },
  { "Haisley", 0.0 },
  { "Haisten", 1.0 },
  { "Haitham", 1.0 },
  { "Haithem", 1.0 },
  { "Haiti", 0.0 },
  { "Haiven", 0.125 },
  { "Haivyn", 0.0 },
  { "Haiyden", 1.0 },
  { "Haiylee", 0.0 },
  { "Haize", 0.7368421 },
  { "Haizel", 0.0 },
  { "Haizen", 1.0 },
  { "Haizlee", 0.0 },
  { "Haizley", 0.0 },
  { "Haja", 0.0 },
  { "Hajar", 0.0 },
  { "Hajara", 0.0 },
  { "Hajari", 1.0 },
  { "Hajer", 0.0 },
  { "Haji", 1.0 },
  { "Hajime", 1.0 },
  { "Hajin", 1.0 },
  { "Hajira", 0.0 },
  { "Hajirah", 0.0 },
  { "Hajj", 1.0 },
  { "Hajoon", 1.0 },
  { "Hajra", 0.0 },
  { "Hajrah", 0.0 },
  { "Hajun", 1.0 },
  { "Hakan", 1.0 },
  { "Hakeem", 1.0 },
  { "Hakeema", 0.0 },
  { "Hakeen", 1.0 },
  { "Hakeim", 1.0 },
  { "Haki", 1.0 },
  { "Hakiem", 1.0 },
  { "Hakim", 1.0 },
  { "Hakima", 0.0 },
  { "Hakimah", 0.0 },
  { "Hakob", 1.0 },
  { "Hakop", 1.0 },
  { "Haku", 1.0 },
  { "Hal", 0.99927443 },
  { "Hala", 0.005090909 },
  { "Halah", 0.0 },
  { "Halaina", 0.0 },
  { "Halana", 0.0 },
  { "Halane", 0.0 },
  { "Halani", 0.0 },
  { "Halas", 1.0 },
  { "Halaya", 0.0 },
  { "Halayah", 0.0 },
  { "Halayna", 0.0 },
  { "Halbert", 1.0 },
  { "Halcy", 0.0 },
  { "Halcyon", 0.0 },
  { "Halden", 1.0 },
  { "Haldon", 1.0 },
  { "Haldor", 1.0 },
  { "Haldrin", 1.0 },
  { "Hale", 0.9879931 },
  { "Halea", 0.0 },
  { "Haleah", 0.0 },
  { "Halee", 0.0 },
  { "Haleem", 1.0 },
  { "Haleema", 0.0 },
  { "Haleemah", 0.0 },
  { "Haleena", 0.0 },
  { "Haleh", 0.0 },
  { "Halei", 0.0 },
  { "Haleia", 0.0 },
  { "Haleigh", 0.00066269055 },
  { "Haleigha", 0.0 },
  { "Halema", 0.0 },
  { "Halen", 0.63316214 },
  { "Halena", 0.0 },
  { "Halene", 0.0 },
  { "Haley", 0.005326627 },
  { "Haleyann", 0.0 },
  { "Haleyjo", 0.0 },
  { "Halford", 1.0 },
  { "Halfred", 1.0 },
  { "Hali", 0.0022686026 },
  { "Halia", 0.0 },
  { "Haliana", 0.0 },
  { "Halid", 1.0 },
  { "Halie", 0.0 },
  { "Haliee", 0.0 },
  { "Haliegh", 0.0 },
  { "Haliey", 0.0 },
  { "Halil", 1.0 },
  { "Halim", 1.0 },
  { "Halima", 0.0 },
  { "Halimah", 0.0 },
  { "Halimatou", 0.0 },
  { "Halimo", 0.0 },
  { "Halina", 0.0 },
  { "Haline", 0.0 },
  { "Halit", 1.0 },
  { "Haliyah", 0.0 },
  { "Hall", 1.0 },
  { "Halla", 0.0 },
  { "Hallah", 0.0 },
  { "Hallard", 1.0 },
  { "Halle", 0.0004553734 },
  { "Hallee", 0.0 },
  { "Halleh", 0.0 },
  { "Hallei", 0.0 },
  { "Halleigh", 0.0 },
  { "Hallel", 0.0 },
  { "Halleli", 0.0 },
  { "Hallelujah", 0.0 },
  { "Hallema", 0.0 },
  { "Hallene", 0.0 },
  { "Hallet", 1.0 },
  { "Hallett", 1.0 },
  { "Halley", 0.0328125 },
  { "Halli", 0.0 },
  { "Hallie", 0.04352362 },
  { "Hallis", 1.0 },
  { "Hally", 0.0078125 },
  { "Hallye", 0.0 },
  { "Halo", 0.124370955 },
  { "Haloa", 1.0 },
  { "Halona", 0.0 },
  { "Halsey", 0.6559486 },
  { "Halstead", 1.0 },
  { "Halsten", 1.0 },
  { "Halston", 0.7904 },
  { "Halton", 1.0 },
  { "Halvor", 1.0 },
  { "Haly", 0.0 },
  { "Halye", 0.0 },
  { "Halyn", 0.0 },
  { "Halyna", 0.0 },
  { "Halynn", 0.0 },
  { "Ham", 1.0 },
  { "Hamaad", 1.0 },
  { "Hamad", 1.0 },
  { "Hamadi", 1.0 },
  { "Hamda", 0.0 },
  { "Hamdaan", 1.0 },
  { "Hamdan", 1.0 },
  { "Hamdi", 0.27602905 },
  { "Hamed", 1.0 },
  { "Hameed", 1.0 },
  { "Hameedah", 0.0 },
  { "Hameen", 1.0 },
  { "Hamer", 1.0 },
  { "Hamid", 1.0 },
  { "Hamida", 0.0 },
  { "Hamidah", 0.0 },
  { "Hamidou", 1.0 },
  { "Hamilton", 1.0 },
  { "Hamim", 1.0 },
  { "Hamin", 1.0 },
  { "Hamir", 1.0 },
  { "Hamish", 1.0 },
  { "Hamiz", 1.0 },
  { "Hamlet", 1.0 },
  { "Hamlin", 1.0 },
  { "Hammad", 1.0 },
  { "Hammed", 1.0 },
  { "Hammie", 1.0 },
  { "Hammond", 1.0 },
  { "Hamna", 0.0 },
  { "Hamnah", 0.0 },
  { "Hamp", 1.0 },
  { "Hampton", 0.9746103 },
  { "Hamsa", 1.0 },
  { "Hamse", 1.0 },
  { "Hamsika", 0.0 },
  { "Hamsini", 0.0 },
  { "Hamza", 1.0 },
  { "Hamzah", 1.0 },
  { "Hamze", 1.0 },
  { "Hamzeh", 1.0 },
  { "Han", 0.50419384 },
  { "Hana", 0.0006252605 },
  { "Hanaa", 0.0 },
  { "Hanaan", 0.0 },
  { "Hanad", 1.0 },
  { "Hanadi", 0.0 },
  { "Hanady", 0.0 },
  { "Hanae", 0.0 },
  { "Hanah", 0.0 },
  { "Hanai", 0.0 },
  { "Hanako", 0.0 },
  { "Hanalee", 0.0 },
  { "Hanalei", 0.0 },
  { "Hanan", 0.057635177 },
  { "Hanane", 0.0 },
  { "Hananiah", 0.4032258 },
  { "Hanayo", 0.0 },
  { "Hance", 1.0 },
  { "Hancel", 1.0 },
  { "Handerson", 1.0 },
  { "Handley", 1.0 },
  { "Handsome", 1.0 },
  { "Handy", 0.9907919 },
  { "Haneef", 1.0 },
  { "Haneefa", 0.0 },
  { "Haneefah", 0.0 },
  { "Haneen", 0.0 },
  { "Haneesh", 1.0 },
  { "Hanen", 0.0 },
  { "Haney", 1.0 },
  { "Hanford", 1.0 },
  { "Hang", 0.0 },
  { "Hanh", 0.044210527 },
  { "Hani", 0.88605696 },
  { "Hania", 0.0 },
  { "Haniah", 0.0 },
  { "Haniel", 0.9779736 },
  { "Hanif", 1.0 },
  { "Hanifa", 0.0 },
  { "Hanifah", 0.0 },
  { "Haniff", 1.0 },
  { "Hanika", 0.0 },
  { "Hanin", 0.0 },
  { "Hanish", 1.0 },
  { "Haniya", 0.0 },
  { "Haniyah", 0.0 },
  { "Haniyyah", 0.0 },
  { "Hank", 1.0 },
  { "Hanlee", 0.0 },
  { "Hanley", 0.7614424 },
  { "Hanna", 0.00343717 },
  { "Hannabella", 0.0 },
  { "Hannabelle", 0.0 },
  { "Hannah", 0.0016189464 },
  { "Hannaha", 0.0 },
  { "Hannahelizabeth", 0.0 },
  { "Hannahgrace", 0.0 },
  { "Hannahjo", 0.0 },
  { "Hannahlee", 0.0 },
  { "Hannahlynn", 0.0 },
  { "Hannahmae", 0.0 },
  { "Hannahmarie", 0.0 },
  { "Hannahrae", 0.0 },
  { "Hannahrose", 0.0 },
  { "Hannalee", 0.0 },
  { "Hannaleigh", 0.0 },
  { "Hannaley", 0.0 },
  { "Hannalise", 0.0 },
  { "Hannan", 0.17575757 },
  { "Hanne", 0.0 },
  { "Hannelore", 0.0 },
  { "Hanner", 0.0 },
  { "Hannes", 1.0 },
  { "Hannha", 0.0 },
  { "Hanni", 0.0 },
  { "Hannia", 0.0 },
  { "Hanniah", 0.0 },
  { "Hannibal", 1.0 },
  { "Hannie", 0.0 },
  { "Hanniel", 1.0 },
  { "Hannon", 0.8245614 },
  { "Hanns", 1.0 },
  { "Hanny", 0.0 },
  { "Hannya", 0.0 },
  { "Hanoch", 1.0 },
  { "Hanora", 0.0 },
  { "Hans", 0.9996547 },
  { "Hansel", 0.9982021 },
  { "Hansell", 1.0 },
  { "Hansen", 1.0 },
  { "Hansford", 1.0 },
  { "Hansika", 0.0 },
  { "Hansini", 0.0 },
  { "Hansley", 1.0 },
  { "Hanson", 1.0 },
  { "Hanssel", 1.0 },
  { "Hantz", 1.0 },
  { "Hanvika", 0.0 },
  { "Hanvitha", 0.0 },
  { "Hany", 0.88095236 },
  { "Hanya", 0.0 },
  { "Hanz", 1.0 },
  { "Hanzalah", 1.0 },
  { "Hanzel", 1.0 },
  { "Hanzo", 1.0 },
  { "Hao", 0.9882075 },
  { "Haochen", 1.0 },
  { "Haoran", 1.0 },
  { "Haowen", 1.0 },
  { "Haoxuan", 1.0 },
  { "Haoyang", 1.0 },
  { "Haoyu", 1.0 },
  { "Hap", 1.0 },
  { "Happiness", 0.0 },
  { "Happy", 0.5707763 },
  { "Hara", 0.0 },
  { "Harace", 1.0 },
  { "Haralabos", 1.0 },
  { "Haralambos", 1.0 },
  { "Harald", 1.0 },
  { "Haram", 0.29411766 },
  { "Haran", 1.0 },
  { "Harbert", 1.0 },
  { "Harbin", 1.0 },
  { "Harbor", 0.35362998 },
  { "Harbour", 0.33333334 },
  { "Harce", 1.0 },
  { "Harcourt", 1.0 },
  { "Hardee", 1.0 },
  { "Hardeep", 0.742268 },
  { "Harden", 1.0 },
  { "Hardie", 1.0 },
  { "Hardik", 1.0 },
  { "Hardin", 1.0 },
  { "Harding", 1.0 },
  { "Hardison", 1.0 },
  { "Hardwick", 1.0 },
  { "Hardy", 0.9981529 },
  { "Hareem", 0.0 },
  { "Harel", 1.0 },
  { "Hareld", 1.0 },
  { "Haresh", 1.0 },
  { "Harfateh", 1.0 },
  { "Harford", 1.0 },
  { "Hargie", 1.0 },
  { "Hargis", 1.0 },
  { "Hargun", 0.21186441 },
  { "Hargus", 1.0 },
  { "Hari", 1.0 },
  { "Harice", 1.0 },
  { "Harihar", 1.0 },
  { "Hariharan", 1.0 },
  { "Harika", 0.0 },
  { "Hariklia", 0.0 },
  { "Harilaos", 1.0 },
  { "Harim", 1.0 },
  { "Harin", 1.0 },
  { "Harinder", 1.0 },
  { "Harini", 0.0 },
  { "Haripriya", 0.0 },
  { "Haris", 1.0 },
  { "Harish", 1.0 },
  { "Harison", 1.0 },
  { "Harita", 0.0 },
  { "Harith", 1.0 },
  { "Harjaap", 1.0 },
  { "Harjap", 0.0 },
  { "Harjas", 0.7017544 },
  { "Harjot", 0.75 },
  { "Harkaran", 1.0 },
  { "Harkeem", 1.0 },
  { "Harker", 1.0 },
  { "Harkiran", 0.0 },
  { "Harkirat", 0.7209302 },
  { "Harl", 1.0 },
  { "Harla", 0.0 },
  { "Harlan", 0.99240667 },
  { "Harland", 0.9985133 },
  { "Harld", 1.0 },
  { "Harle", 1.0 },
  { "Harlea", 0.0 },
  { "Harlean", 0.0 },
  { "Harlee", 0.075656004 },
  { "Harleen", 0.0 },
  { "Harleequinn", 0.0 },
  { "Harlei", 0.0 },
  { "Harleigh", 0.0 },
  { "Harlem", 0.7308721 },
  { "Harlen", 0.9924007 },
  { "Harlene", 0.0 },
  { "Harlequin", 0.0 },
  { "Harlequinn", 0.0 },
  { "Harles", 1.0 },
  { "Harless", 1.0 },
  { "Harleth", 1.0 },
  { "Harley", 0.6413403 },
  { "Harleyann", 0.0 },
  { "Harleyquinn", 0.0 },
  { "Harleyrose", 0.0 },
  { "Harli", 0.0 },
  { "Harlie", 0.1831019 },
  { "Harliee", 0.0 },
  { "Harliegh", 0.0 },
  { "Harlin", 0.97898424 },
  { "Harline", 0.0 },
  { "Harliquinn", 0.0 },
  { "Harlis", 1.0 },
  { "Harliv", 0.0 },
  { "Harlo", 0.5063694 },
  { "Harlod", 1.0 },
  { "Harloe", 0.0 },
  { "Harlon", 1.0 },
  { "Harlow", 0.27479208 },
  { "Harlowe", 0.08306709 },
  { "Harly", 0.14973262 },
  { "Harlym", 0.1322314 },
  { "Harlyn", 0.3494282 },
  { "Harlynn", 0.0 },
  { "Harm", 1.0 },
  { "Harman", 0.9381139 },
  { "Harmandeep", 1.0 },
  { "Harmani", 0.0 },
  { "Harmanie", 0.0 },
  { "Harmanpreet", 0.46153846 },
  { "Harmany", 0.0 },
  { "Harmeet", 0.3529412 },
  { "Harmie", 1.0 },
  { "Harminnie", 0.0 },
  { "Harmon", 0.99674517 },
  { "Harmond", 1.0 },
  { "Harmone", 0.0 },
  { "Harmonee", 0.0 },
  { "Harmonei", 0.0 },
  { "Harmoney", 0.0 },
  { "Harmoni", 0.0 },
  { "Harmonie", 0.0 },
  { "Harmoniee", 0.0 },
  { "Harmonii", 0.0 },
  { "Harmony", 0.0008318011 },
  { "Harneet", 0.0 },
  { "Harney", 1.0 },
  { "Harnoor", 0.2688822 },
  { "Harol", 1.0 },
  { "Harold", 0.99520046 },
  { "Haroldean", 0.6315789 },
  { "Haroldene", 0.0 },
  { "Haroldine", 0.0 },
  { "Haroldo", 1.0 },
  { "Harolene", 0.0 },
  { "Harolyn", 0.0 },
  { "Haron", 1.0 },
  { "Haroon", 1.0 },
  { "Haroun", 1.0 },
  { "Harout", 1.0 },
  { "Haroutun", 1.0 },
  { "Haroutyun", 1.0 },
  { "Harpal", 1.0 },
  { "Harpar", 0.0 },
  { "Harper", 0.08197049 },
  { "Harpergrace", 0.0 },
  { "Harpreet", 0.4950298 },
  { "Harpyr", 0.0 },
  { "Harrel", 1.0 },
  { "Harrell", 1.0 },
  { "Harrey", 1.0 },
  { "Harrie", 0.8888889 },
  { "Harriel", 1.0 },
  { "Harriet", 0.001369522 },
  { "Harriett", 0.00023228803 },
  { "Harrietta", 0.0 },
  { "Harriette", 0.0 },
  { "Harril", 1.0 },
  { "Harrill", 1.0 },
  { "Harrilyn", 0.0 },
  { "Harrington", 1.0 },
  { "Harriot", 0.0 },
  { "Harris", 0.9922168 },
  { "Harrisen", 1.0 },
  { "Harrison", 0.99671084 },
  { "Harrisson", 1.0 },
  { "Harrol", 1.0 },
  { "Harrold", 1.0 },
  { "Harroll", 1.0 },
  { "Harron", 1.0 },
  { "Harry", 0.99522626 },
  { "Harryette", 0.0 },
  { "Harryson", 1.0 },
  { "Harseerat", 0.0 },
  { "Harsh", 1.0 },
  { "Harsha", 0.9151786 },
  { "Harshaan", 1.0 },
  { "Harshal", 1.0 },
  { "Harshan", 1.0 },
  { "Harshdeep", 1.0 },
  { "Harshi", 0.0 },
  { "Harshika", 0.0 },
  { "Harshil", 1.0 },
  { "Harshini", 0.0 },
  { "Harshit", 1.0 },
  { "Harshita", 0.0 },
  { "Harshith", 1.0 },
  { "Harshitha", 0.0 },
  { "Harshiv", 1.0 },
  { "Harshveer", 1.0 },
  { "Harsimar", 1.0 },
  { "Harsimran", 0.25396827 },
  { "Harsimrat", 0.0 },
  { "Hart", 0.9767981 },
  { "Hartaj", 1.0 },
  { "Hartej", 1.0 },
  { "Hartense", 0.0 },
  { "Hartford", 1.0 },
  { "Hartie", 0.0 },
  { "Hartlee", 0.0 },
  { "Hartleigh", 0.0 },
  { "Hartley", 0.5688744 },
  { "Hartli", 0.0 },
  { "Hartlie", 0.0 },
  { "Hartly", 0.0 },
  { "Hartlyn", 0.0 },
  { "Hartlynn", 0.0 },
  { "Hartman", 1.0 },
  { "Hartsel", 1.0 },
  { "Hartsell", 1.0 },
  { "Hartwell", 1.0 },
  { "Hartzel", 1.0 },
  { "Hartzell", 1.0 },
  { "Haru", 0.6 },
  { "Harue", 0.0 },
  { "Haruka", 0.0 },
  { "Haruki", 1.0 },
  { "Haruko", 0.0 },
  { "Haruma", 1.0 },
  { "Harumi", 0.078947365 },
  { "Harun", 1.0 },
  { "Haruna", 0.0 },
  { "Haruno", 0.0 },
  { "Haruo", 1.0 },
  { "Harut", 1.0 },
  { "Haruto", 1.0 },
  { "Harutun", 1.0 },
  { "Harutyun", 1.0 },
  { "Haruye", 0.0 },
  { "Haruyo", 0.0 },
  { "Haruyoshi", 1.0 },
  { "Harvard", 1.0 },
  { "Harve", 1.0 },
  { "Harveen", 0.0 },
  { "Harveer", 1.0 },
  { "Harvel", 1.0 },
  { "Harvell", 1.0 },
  { "Harver", 1.0 },
  { "Harvest", 0.47990543 },
  { "Harvester", 1.0 },
  { "Harvetta", 0.0 },
  { "Harvey", 0.99419624 },
  { "Harvi", 0.0 },
  { "Harvie", 0.9860267 },
  { "Harvin", 0.92105263 },
  { "Harvir", 1.0 },
  { "Harvis", 1.0 },
  { "Harvy", 1.0 },
  { "Harwell", 1.0 },
  { "Harwood", 1.0 },
  { "Hary", 1.0 },
  { "Hasaan", 1.0 },
  { "Hasan", 1.0 },
  { "Hasana", 0.0 },
  { "Hasanah", 0.0 },
  { "Hasanat", 0.0 },
  { "Hasani", 0.9883475 },
  { "Hasaun", 1.0 },
  { "Hascal", 1.0 },
  { "Haseeb", 1.0 },
  { "Haseebullah", 1.0 },
  { "Haseem", 1.0 },
  { "Haseena", 0.0 },
  { "Hasel", 0.014778325 },
  { "Hasen", 1.0 },
  { "Haset", 0.0 },
  { "Hasheem", 1.0 },
  { "Hashem", 1.0 },
  { "Hashim", 1.0 },
  { "Hashir", 1.0 },
  { "Hashley", 0.0 },
  { "Hasib", 1.0 },
  { "Hasiel", 1.0 },
  { "Hasika", 0.0 },
  { "Hasim", 1.0 },
  { "Hasin", 1.0 },
  { "Hasina", 0.0 },
  { "Hasinah", 0.0 },
  { "Hasini", 0.0 },
  { "Hasir", 1.0 },
  { "Hasita", 0.0 },
  { "Hasitha", 0.0 },
  { "Haskel", 1.0 },
  { "Haskell", 0.9972588 },
  { "Hasker", 1.0 },
  { "Haskie", 1.0 },
  { "Haskle", 1.0 },
  { "Hasley", 0.0 },
  { "Hasly", 0.0 },
  { "Haslyn", 0.0 },
  { "Hasmik", 0.0 },
  { "Hasna", 0.0 },
  { "Hasnaa", 0.0 },
  { "Hasnain", 1.0 },
  { "Hason", 1.0 },
  { "Hassaan", 1.0 },
  { "Hassain", 1.0 },
  { "Hassam", 1.0 },
  { "Hassan", 0.9992346 },
  { "Hassana", 0.0 },
  { "Hassanah", 0.0 },
  { "Hassanatou", 0.0 },
  { "Hassane", 1.0 },
  { "Hassani", 1.0 },
  { "Hassel", 1.0 },
  { "Hassell", 1.0 },
  { "Hasseltine", 0.0 },
  { "Hassen", 1.0 },
  { "Hasset", 0.0 },
  { "Hassie", 0.00643915 },
  { "Hassiel", 1.0 },
  { "Hassiem", 1.0 },
  { "Hasson", 1.0 },
  { "Hasten", 1.0 },
  { "Hasti", 0.0 },
  { "Hastiin", 1.0 },
  { "Hastin", 1.0 },
  { "Hasting", 1.0 },
  { "Hastings", 0.8333333 },
  { "Haston", 1.0 },
  { "Hasty", 0.0 },
  { "Hasya", 0.0 },
  { "Hatch", 1.0 },
  { "Hatcher", 1.0 },
  { "Hatem", 1.0 },
  { "Hateya", 0.0 },
  { "Hathaway", 0.0 },
  { "Hather", 0.0 },
  { "Hatice", 0.0 },
  { "Hatim", 1.0 },
  { "Hatina", 0.0 },
  { "Hatley", 0.0 },
  { "Hatsue", 0.0 },
  { "Hatsuko", 0.0 },
  { "Hatsumi", 0.0 },
  { "Hatsuye", 0.0 },
  { "Hatsuyo", 0.0 },
  { "Hattan", 1.0 },
  { "Hatti", 0.0 },
  { "Hattie", 0.004990655 },
  { "Hatton", 1.0 },
  { "Hatty", 0.0 },
  { "Hattye", 0.0 },
  { "Hatziry", 0.0 },
  { "Hau", 0.85365856 },
  { "Haunani", 0.0 },
  { "Hava", 0.0 },
  { "Havah", 0.0 },
  { "Haval", 1.0 },
  { "Havala", 0.0 },
  { "Havan", 0.0 },
  { "Havana", 0.0 },
  { "Havanah", 0.0 },
  { "Havanna", 0.0 },
  { "Havannah", 0.0 },
  { "Havard", 1.0 },
  { "Haven", 0.2409184 },
  { "Havery", 1.0 },
  { "Havick", 1.0 },
  { "Havier", 1.0 },
  { "Havik", 1.0 },
  { "Havilah", 0.0 },
  { "Haviland", 0.0 },
  { "Havilland", 0.0 },
  { "Havin", 0.0 },
  { "Havis", 1.0 },
  { "Havish", 1.0 },
  { "Havisha", 0.0 },
  { "Havoc", 1.0 },
  { "Havok", 1.0 },
  { "Havya", 0.0 },
  { "Havyn", 0.06732118 },
  { "Havynn", 0.0 },
  { "Hawa", 0.0 },
  { "Hawah", 0.0 },
  { "Hawaii", 0.0 },
  { "Hawanatu", 0.0 },
  { "Hawanya", 0.0 },
  { "Haward", 1.0 },
  { "Hawi", 0.0 },
  { "Hawk", 1.0 },
  { "Hawke", 1.0 },
  { "Hawken", 1.0 },
  { "Hawkens", 1.0 },
  { "Hawkeye", 1.0 },
  { "Hawkin", 1.0 },
  { "Hawkins", 1.0 },
  { "Hawley", 0.7109005 },
  { "Hawo", 0.0 },
  { "Hawra", 0.0 },
  { "Hawraa", 0.0 },
  { "Hawthorn", 1.0 },
  { "Hawthorne", 0.9710145 },
  { "Hawwa", 0.0 },
  { "Hawwaa", 0.0 },
  { "Haxton", 1.0 },
  { "Hay", 1.0 },
  { "Haya", 0.0 },
  { "Hayaa", 0.0 },
  { "Hayaan", 1.0 },
  { "Hayah", 0.0 },
  { "Hayam", 0.0 },
  { "Hayat", 0.0 },
  { "Hayato", 1.0 },
  { "Haydan", 0.7479452 },
  { "Haydar", 1.0 },
  { "Hayde", 0.0 },
  { "Haydee", 0.0 },
  { "Hayden", 0.79512 },
  { "Haydenn", 0.23076923 },
  { "Hayder", 1.0 },
  { "Haydi", 0.0 },
  { "Haydin", 0.60633487 },
  { "Haydn", 0.8248472 },
  { "Haydon", 0.94222224 },
  { "Haydyn", 0.5201613 },
  { "Hayes", 0.96544033 },
  { "Haygan", 1.0 },
  { "Haygen", 0.91943127 },
  { "Hayk", 1.0 },
  { "Hayla", 0.0 },
  { "Haylah", 0.0 },
  { "Haylan", 0.0 },
  { "Hayle", 0.0 },
  { "Haylea", 0.0 },
  { "Hayleah", 0.0 },
  { "Haylee", 0.0007293148 },
  { "Hayleen", 0.0 },
  { "Haylei", 0.0 },
  { "Hayleigh", 0.0 },
  { "Haylen", 0.16594517 },
  { "Hayley", 0.00095445587 },
  { "Hayli", 0.0 },
  { "Haylie", 0.0 },
  { "Hayliee", 0.0 },
  { "Hayliegh", 0.0 },
  { "Haylin", 0.0 },
  { "Hayllie", 0.0 },
  { "Haylo", 0.0 },
  { "Hayly", 0.0 },
  { "Haylyn", 0.0 },
  { "Haylynn", 0.0 },
  { "Haymon", 1.0 },
  { "Haymond", 1.0 },
  { "Hayne", 1.0 },
  { "Haynes", 0.989339 },
  { "Haynie", 1.0 },
  { "Hayoon", 0.0 },
  { "Hayoung", 0.0 },
  { "Hays", 1.0 },
  { "Hayse", 1.0 },
  { "Hayslee", 0.0 },
  { "Hayson", 1.0 },
  { "Hayston", 1.0 },
  { "Haytham", 1.0 },
  { "Haythem", 1.0 },
  { "Hayven", 0.1729785 },
  { "Hayvin", 0.0 },
  { "Hayvn", 0.0 },
  { "Hayward", 1.0 },
  { "Haywood", 1.0 },
  { "Hayword", 1.0 },
  { "Hayyan", 1.0 },
  { "Hayz", 1.0 },
  { "Hayze", 1.0 },
  { "Hayzel", 0.0 },
  { "Hayzen", 1.0 },
  { "Hayzlee", 0.0 },
  { "Hayzlie", 0.0 },
  { "Hazael", 1.0 },
  { "Hazaiah", 1.0 },
  { "Hazal", 0.0 },
  { "Haze", 0.9587629 },
  { "Hazel", 0.010594325 },
  { "Hazelann", 0.0 },
  { "Hazele", 0.0 },
  { "Hazelee", 0.0 },
  { "Hazeleen", 0.0 },
  { "Hazeleigh", 0.0 },
  { "Hazelene", 0.0 },
  { "Hazelgrace", 0.0 },
  { "Hazeline", 0.0 },
  { "Hazell", 0.022222223 },
  { "Hazelle", 0.0 },
  { "Hazelmae", 0.0 },
  { "Hazelmarie", 0.0 },
  { "Hazelrose", 0.0 },
  { "Hazeltine", 0.0 },
  { "Hazely", 0.0 },
  { "Hazelyn", 0.0 },
  { "Hazelynn", 0.0 },
  { "Hazem", 1.0 },
  { "Hazen", 0.9751157 },
  { "Haziel", 0.99241275 },
  { "Hazim", 1.0 },
  { "Hazin", 1.0 },
  { "Haziq", 1.0 },
  { "Hazir", 1.0 },
  { "Hazle", 0.017423015 },
  { "Hazlee", 0.0 },
  { "Hazleigh", 0.0 },
  { "Hazley", 0.0 },
  { "Hazlie", 0.0 },
  { "Hazlyn", 0.0 },
  { "Hazlynn", 0.0 },
  { "Hazy", 0.0 },
  { "Hazyl", 0.0 },
  { "Hazyn", 1.0 },
  { "Hazzel", 0.0 },
  { "Hazzie", 1.0 },
  { "Heahter", 0.0 },
  { "Healani", 0.0 },
  { "Healey", 0.0 },
  { "Healy", 0.0 },
  { "Heang", 0.0 },
  { "Heard", 1.0 },
  { "Hearl", 1.0 },
  { "Hearld", 1.0 },
  { "Hearman", 1.0 },
  { "Hearold", 1.0 },
  { "Heart", 0.0 },
  { "Hearther", 0.0 },
  { "Heartlee", 0.0 },
  { "Heartley", 0.0 },
  { "Heartly", 0.0 },
  { "Heartlyn", 0.0 },
  { "Heartlynn", 0.0 },
  { "Heater", 0.0 },
  { "Heath", 0.99342704 },
  { "Heathcliff", 1.0 },
  { "Heathe", 0.32 },
  { "Heather", 0.0028186373 },
  { "Heatherann", 0.0 },
  { "Heatherlee", 0.0 },
  { "Heatherly", 0.0 },
  { "Heatherlyn", 0.0 },
  { "Heatherlynn", 0.0 },
  { "Heathermarie", 0.0 },
  { "Heathr", 0.0 },
  { "Heathre", 0.0 },
  { "Heathyr", 0.0 },
  { "Heaton", 1.0 },
  { "Heavan", 0.0 },
  { "Heaven", 0.01197407 },
  { "Heavenlee", 0.0 },
  { "Heavenlei", 0.0 },
  { "Heavenleigh", 0.0 },
  { "Heavenley", 0.0 },
  { "Heavenli", 0.0 },
  { "Heavenly", 0.0 },
  { "Heavenlyjoy", 0.0 },
  { "Heavenlyn", 0.0 },
  { "Heavenor", 0.0 },
  { "Heavin", 0.0 },
  { "Heavon", 0.0 },
  { "Heavyn", 0.0 },
  { "Heavynn", 0.0 },
  { "Heba", 0.0 },
  { "Hebah", 0.0 },
  { "Hebe", 0.0 },
  { "Heber", 1.0 },
  { "Hebert", 1.0 },
  { "Heberth", 1.0 },
  { "Heberto", 1.0 },
  { "Hebron", 0.9137931 },
  { "Hec", 1.0 },
  { "Hector", 0.995234 },
  { "Hedaya", 0.0 },
  { "Hedda", 0.0 },
  { "Heddie", 0.0 },
  { "Heddy", 0.0 },
  { "Heder", 1.0 },
  { "Hedgar", 1.0 },
  { "Hedi", 0.0 },
  { "Hedie", 0.0 },
  { "Hedieh", 0.0 },
  { "Hedit", 0.0 },
  { "Hedley", 1.0 },
  { "Hedrick", 1.0 },
  { "Hedvig", 0.0 },
  { "Hedwig", 0.0 },
  { "Hedwige", 0.0 },
  { "Hedy", 0.0 },
  { "Hee", 0.0 },
  { "Heeba", 0.0 },
  { "Heela", 0.0 },
  { "Heena", 0.0 },
  { "Heer", 0.0 },
  { "Heera", 0.0 },
  { "Heet", 1.0 },
  { "Heeya", 0.0 },
  { "Heflin", 1.0 },
  { "Hefziba", 0.0 },
  { "Heida", 0.0 },
  { "Heide", 0.0 },
  { "Heidee", 0.0 },
  { "Heidemarie", 0.0 },
  { "Heiden", 1.0 },
  { "Heidi", 0.0025437225 },
  { "Heidiann", 0.0 },
  { "Heidie", 0.0 },
  { "Heidimarie", 0.0 },
  { "Heidy", 0.0 },
  { "Heike", 0.0 },
  { "Heiko", 1.0 },
  { "Heiley", 0.0 },
  { "Heili", 0.0 },
  { "Heilly", 0.0 },
  { "Heily", 0.0 },
  { "Heilyn", 0.0 },
  { "Hein", 1.0 },
  { "Heiner", 1.0 },
  { "Heinrich", 1.0 },
  { "Heinz", 1.0 },
  { "Heiress", 0.0 },
  { "Heiry", 0.0 },
  { "Heisy", 0.0 },
  { "Heith", 1.0 },
  { "Heitor", 1.0 },
  { "Hektor", 1.0 },
  { "Hela", 0.0 },
  { "Heladio", 1.0 },
  { "Helaena", 0.0 },
  { "Helaina", 0.0 },
  { "Helaine", 0.0 },
  { "Helal", 1.0 },
  { "Helam", 1.0 },
  { "Helaman", 1.0 },
  { "Helan", 0.0 },
  { "Helana", 0.0 },
  { "Helane", 0.0 },
  { "Helayna", 0.0 },
  { "Helayne", 0.0 },
  { "Heldana", 0.0 },
  { "Helder", 1.0 },
  { "Heleana", 0.0 },
  { "Heleen", 0.0 },
  { "Heleena", 0.0 },
  { "Heleina", 0.0 },
  { "Helem", 0.0 },
  { "Helen", 0.0030412797 },
  { "Helena", 0.0 },
  { "Helenann", 0.0 },
  { "Helene", 0.0 },
  { "Helenmae", 0.0 },
  { "Helenmarie", 0.0 },
  { "Helenna", 0.0 },
  { "Heleyna", 0.0 },
  { "Helga", 0.0 },
  { "Helge", 1.0 },
  { "Heli", 0.11627907 },
  { "Helia", 0.0 },
  { "Heliana", 0.0 },
  { "Helin", 0.0 },
  { "Helina", 0.0 },
  { "Helio", 1.0 },
  { "Heliodoro", 1.0 },
  { "Helios", 1.0 },
  { "Helissa", 0.0 },
  { "Helius", 1.0 },
  { "Helix", 1.0 },
  { "Hellan", 0.0 },
  { "Hellen", 0.0 },
  { "Hellena", 0.0 },
  { "Hellene", 0.0 },
  { "Heller", 0.0 },
  { "Hellon", 0.0 },
  { "Helly", 0.0 },
  { "Helma", 0.0 },
  { "Helmar", 1.0 },
  { "Helmer", 1.0 },
  { "Helmi", 0.0 },
  { "Helmie", 0.0 },
  { "Helmut", 1.0 },
  { "Helmuth", 1.0 },
  { "Helo", 1.0 },
  { "Heloisa", 0.0 },
  { "Heloise", 0.0 },
  { "Helon", 0.007451565 },
  { "Helsea", 0.0 },
  { "Helton", 1.0 },
  { "Helvi", 0.0 },
  { "Helvie", 0.0 },
  { "Helyn", 0.0 },
  { "Helyne", 0.0 },
  { "Hema", 0.0 },
  { "Hemal", 1.0 },
  { "Hemali", 0.0 },
  { "Heman", 1.0 },
  { "Hemani", 0.0 },
  { "Hemant", 1.0 },
  { "Hemanth", 1.0 },
  { "Hemen", 0.0 },
  { "Hemerson", 1.0 },
  { "Hemi", 0.65957445 },
  { "Hemingway", 1.0 },
  { "Hemma", 0.0 },
  { "Hemza", 1.0 },
  { "Hena", 0.0 },
  { "Henach", 1.0 },
  { "Hence", 1.0 },
  { "Henchy", 0.0 },
  { "Hend", 0.0 },
  { "Hendel", 0.0 },
  { "Henderson", 1.0 },
  { "Hendley", 0.0 },
  { "Hendrex", 1.0 },
  { "Hendric", 1.0 },
  { "Hendrick", 1.0 },
  { "Hendricks", 1.0 },
  { "Hendrik", 1.0 },
  { "Hendrix", 0.94006896 },
  { "Hendrixx", 1.0 },
  { "Hendry", 1.0 },
  { "Hendryx", 1.0 },
  { "Hendy", 0.0 },
  { "Heneretta", 0.0 },
  { "Henerietta", 0.0 },
  { "Henery", 1.0 },
  { "Henesis", 0.0 },
  { "Henessey", 0.0 },
  { "Henessy", 0.0 },
  { "Henesy", 0.0 },
  { "Heng", 1.0 },
  { "Henil", 1.0 },
  { "Henlee", 0.0 },
  { "Henleigh", 0.0 },
  { "Henley", 0.23186034 },
  { "Henli", 0.0 },
  { "Henlie", 0.0 },
  { "Henly", 0.4347826 },
  { "Henna", 0.0 },
  { "Hennah", 0.0 },
  { "Hennesey", 0.0 },
  { "Hennesie", 0.0 },
  { "Hennessey", 0.0 },
  { "Hennessy", 0.045045044 },
  { "Hennesy", 0.0 },
  { "Hennie", 0.0 },
  { "Henning", 1.0 },
  { "Hennley", 0.0 },
  { "Hennry", 1.0 },
  { "Henny", 0.086181276 },
  { "Henoc", 1.0 },
  { "Henoch", 1.0 },
  { "Henock", 1.0 },
  { "Henok", 1.0 },
  { "Henon", 1.0 },
  { "Henos", 1.0 },
  { "Henreitta", 0.0 },
  { "Henrene", 0.0 },
  { "Henretta", 0.0 },
  { "Henretter", 0.0 },
  { "Henrettia", 0.0 },
  { "Henrey", 1.0 },
  { "Henri", 0.8714256 },
  { "Henrick", 1.0 },
  { "Henrie", 0.28169015 },
  { "Henriella", 0.0 },
  { "Henrietta", 0.0010252972 },
  { "Henriette", 0.0 },
  { "Henrik", 1.0 },
  { "Henrika", 0.0 },
  { "Henrine", 0.0 },
  { "Henrique", 1.0 },
  { "Henritta", 0.0 },
  { "Henrri", 1.0 },
  { "Henrry", 1.0 },
  { "Henry", 0.9933173 },
  { "Henryetta", 0.0 },
  { "Henryk", 1.0 },
  { "Henryka", 0.0 },
  { "Hensel", 1.0 },
  { "Henslee", 0.0 },
  { "Hensleigh", 0.0 },
  { "Hensley", 0.14136125 },
  { "Henson", 1.0 },
  { "Henton", 1.0 },
  { "Henya", 0.0 },
  { "Henzlee", 0.0 },
  { "Henzley", 0.0 },
  { "Hephzibah", 0.0 },
  { "Her", 1.0 },
  { "Hera", 0.0 },
  { "Heraclio", 1.0 },
  { "Herald", 1.0 },
  { "Heraldo", 1.0 },
  { "Heran", 0.0 },
  { "Herb", 1.0 },
  { "Herber", 1.0 },
  { "Herbert", 0.995118 },
  { "Herberta", 0.0 },
  { "Herberth", 1.0 },
  { "Herberto", 1.0 },
  { "Herbet", 1.0 },
  { "Herbey", 1.0 },
  { "Herbie", 1.0 },
  { "Herby", 1.0 },
  { "Herchel", 1.0 },
  { "Herchell", 1.0 },
  { "Hercilia", 0.0 },
  { "Herculano", 1.0 },
  { "Hercules", 1.0 },
  { "Herdis", 1.0 },
  { "Heri", 1.0 },
  { "Heriberta", 0.0 },
  { "Heriberto", 0.9983519 },
  { "Herica", 0.0 },
  { "Herick", 1.0 },
  { "Herik", 1.0 },
  { "Herika", 0.0 },
  { "Herlaine", 0.0 },
  { "Herlane", 0.0 },
  { "Herlene", 0.0 },
  { "Herley", 1.0 },
  { "Herlin", 1.0 },
  { "Herlinda", 0.0 },
  { "Herline", 0.0 },
  { "Herma", 0.0 },
  { "Hermalinda", 0.0 },
  { "Herman", 0.99416375 },
  { "Hermance", 0.0 },
  { "Hermanda", 0.0 },
  { "Hermania", 0.0 },
  { "Hermann", 1.0 },
  { "Hermas", 1.0 },
  { "Hermela", 0.0 },
  { "Hermelinda", 0.0 },
  { "Hermelindo", 1.0 },
  { "Hermen", 1.0 },
  { "Hermena", 0.0 },
  { "Hermene", 0.0 },
  { "Hermenegildo", 1.0 },
  { "Hermenia", 0.0 },
  { "Hermes", 1.0 },
  { "Hermia", 0.0 },
  { "Hermie", 0.03785489 },
  { "Hermila", 0.0 },
  { "Hermilinda", 0.0 },
  { "Hermilo", 1.0 },
  { "Hermina", 0.0 },
  { "Hermine", 0.0 },
  { "Herminia", 0.0 },
  { "Herminio", 1.0 },
  { "Hermino", 1.0 },
  { "Hermione", 0.0 },
  { "Hermit", 1.0 },
  { "Hermoine", 0.0 },
  { "Hermon", 0.99097747 },
  { "Hermond", 1.0 },
  { "Hermonie", 0.0 },
  { "Hernaldo", 1.0 },
  { "Hernan", 1.0 },
  { "Hernandez", 1.0 },
  { "Hernando", 1.0 },
  { "Herndon", 1.0 },
  { "Hernesto", 1.0 },
  { "Herney", 1.0 },
  { "Herny", 1.0 },
  { "Hero", 0.6412777 },
  { "Herod", 1.0 },
  { "Herold", 1.0 },
  { "Heron", 0.9905123 },
  { "Herrel", 1.0 },
  { "Herrell", 1.0 },
  { "Herrick", 1.0 },
  { "Herron", 1.0 },
  { "Herry", 1.0 },
  { "Hersch", 1.0 },
  { "Herschal", 1.0 },
  { "Herschel", 1.0 },
  { "Herschell", 1.0 },
  { "Hersel", 1.0 },
  { "Hersey", 1.0 },
  { "Hersh", 1.0 },
  { "Hershal", 1.0 },
  { "Hershall", 1.0 },
  { "Hershel", 0.99949807 },
  { "Hershell", 1.0 },
  { "Hershey", 1.0 },
  { "Hershy", 1.0 },
  { "Hersie", 1.0 },
  { "Herson", 1.0 },
  { "Herta", 0.0 },
  { "Hertha", 0.0 },
  { "Hertis", 1.0 },
  { "Heru", 1.0 },
  { "Herve", 1.0 },
  { "Hervey", 1.0 },
  { "Hervie", 1.0 },
  { "Hervin", 1.0 },
  { "Hervy", 1.0 },
  { "Hery", 1.0 },
  { "Hesham", 1.0 },
  { "Hesher", 1.0 },
  { "Heshimu", 1.0 },
  { "Heshy", 1.0 },
  { "Hesper", 0.0 },
  { "Hess", 1.0 },
  { "Hessa", 0.0 },
  { "Hessel", 1.0 },
  { "Hessie", 0.014109347 },
  { "Hesston", 1.0 },
  { "Hesta", 0.0 },
  { "Hestel", 0.0 },
  { "Hester", 0.036416654 },
  { "Hestia", 0.0 },
  { "Heston", 1.0 },
  { "Het", 1.0 },
  { "Hetal", 0.0 },
  { "Hether", 0.0 },
  { "Hettie", 0.0 },
  { "Hetty", 0.0 },
  { "Hettye", 0.0 },
  { "Hetvi", 0.0 },
  { "Hevan", 0.0 },
  { "Heven", 0.0 },
  { "Hever", 1.0 },
  { "Hevin", 0.0 },
  { "Hevyn", 0.0 },
  { "Hewan", 0.0 },
  { "Hewell", 1.0 },
  { "Hewey", 1.0 },
  { "Hewitt", 1.0 },
  { "Hewlett", 1.0 },
  { "Heyab", 0.0 },
  { "Heyam", 0.0 },
  { "Heydan", 1.0 },
  { "Heyden", 1.0 },
  { "Heydi", 0.0 },
  { "Heydy", 0.0 },
  { "Heylee", 0.0 },
  { "Heylen", 0.0 },
  { "Heyley", 0.0 },
  { "Heyli", 0.0 },
  { "Heylie", 0.0 },
  { "Heylin", 0.0 },
  { "Heymi", 0.0 },
  { "Heysel", 0.0 },
  { "Heyward", 1.0 },
  { "Heywood", 1.0 },
  { "Hezakiah", 1.0 },
  { "Hezeki", 1.0 },
  { "Hezekiah", 0.99924606 },
  { "Hezekiel", 1.0 },
  { "Hezekyah", 1.0 },
  { "Hezikiah", 1.0 },
  { "Hezkiah", 1.0 },
  { "Hezron", 1.0 },
  { "Hezzie", 1.0 },
  { "Hiatt", 1.0 },
  { "Hiawatha", 0.7538712 },
  { "Hiba", 0.0 },
  { "Hibah", 0.0 },
  { "Hibaq", 0.0 },
  { "Hibba", 0.0 },
  { "Hibbard", 1.0 },
  { "Hibo", 0.0 },
  { "Hicham", 1.0 },
  { "Hickman", 1.0 },
  { "Hickory", 1.0 },
  { "Hicks", 1.0 },
  { "Hickson", 1.0 },
  { "Hidaya", 0.0 },
  { "Hidayah", 0.0 },
  { "Hideki", 1.0 },
  { "Hideko", 0.0 },
  { "Hidemi", 1.0 },
  { "Hideo", 1.0 },
  { "Hidi", 0.0 },
  { "Hidie", 0.0 },
  { "Hiede", 0.0 },
  { "Hiedi", 0.0 },
  { "Hien", 0.48259494 },
  { "Hiep", 1.0 },
  { "Hieu", 0.9655172 },
  { "Hifza", 0.0 },
  { "Higinia", 0.0 },
  { "Higinio", 1.0 },
  { "Hiiaka", 0.0 },
  { "Hiilani", 0.0 },
  { "Hiilei", 0.0 },
  { "Hijinio", 1.0 },
  { "Hikari", 0.0 },
  { "Hikaru", 0.8888889 },
  { "Hikeem", 1.0 },
  { "Hikma", 0.0 },
  { "Hikmah", 0.0 },
  { "Hila", 0.0 },
  { "Hilah", 0.0 },
  { "Hilaire", 0.6041667 },
  { "Hilal", 0.33898306 },
  { "Hilaree", 0.0 },
  { "Hilari", 0.0 },
  { "Hilaria", 0.0 },
  { "Hilarie", 0.0 },
  { "Hilario", 1.0 },
  { "Hilarion", 1.0 },
  { "Hilary", 0.084003985 },
  { "Hilbert", 1.0 },
  { "Hilberto", 1.0 },
  { "Hilburn", 1.0 },
  { "Hilda", 0.00226631 },
  { "Hildagard", 0.0 },
  { "Hildagarde", 0.0 },
  { "Hildah", 0.0 },
  { "Hildana", 0.0 },
  { "Hilde", 0.0 },
  { "Hildebrando", 1.0 },
  { "Hildegard", 0.0 },
  { "Hildegarde", 0.0 },
  { "Hilder", 0.16129032 },
  { "Hildia", 0.0 },
  { "Hildie", 0.0 },
  { "Hilding", 1.0 },
  { "Hildon", 1.0 },
  { "Hildra", 0.0 },
  { "Hildred", 0.14872521 },
  { "Hildreth", 0.10948081 },
  { "Hildur", 0.0 },
  { "Hildy", 0.0 },
  { "Hilery", 0.66972476 },
  { "Hiley", 0.0 },
  { "Hilia", 0.0 },
  { "Hiliana", 0.0 },
  { "Hiliary", 0.0 },
  { "Hilinai", 0.0 },
  { "Hilja", 0.0 },
  { "Hill", 1.0 },
  { "Hilla", 0.0 },
  { "Hillard", 1.0 },
  { "Hillaree", 0.0 },
  { "Hillarey", 0.0 },
  { "Hillari", 0.0 },
  { "Hillarie", 0.0 },
  { "Hillary", 0.042437285 },
  { "Hillel", 1.0 },
  { "Hillery", 0.50301206 },
  { "Hilliard", 1.0 },
  { "Hilliary", 0.016233766 },
  { "Hillie", 1.0 },
  { "Hillip", 1.0 },
  { "Hillis", 0.9690476 },
  { "Hillman", 1.0 },
  { "Hillory", 0.18518518 },
  { "Hilma", 0.0 },
  { "Hilman", 1.0 },
  { "Hilmar", 1.0 },
  { "Hilmer", 1.0 },
  { "Hilmon", 1.0 },
  { "Hilo", 1.0 },
  { "Hilrey", 1.0 },
  { "Hilry", 1.0 },
  { "Hilton", 0.99615157 },
  { "Hilyard", 1.0 },
  { "Hima", 0.0 },
  { "Himani", 0.0 },
  { "Himansh", 1.0 },
  { "Himanshu", 1.0 },
  { "Himari", 0.0 },
  { "Himmat", 1.0 },
  { "Hina", 0.0 },
  { "Hinaea", 0.0 },
  { "Hinako", 0.0 },
  { "Hinal", 0.0 },
  { "Hinami", 0.0 },
  { "Hinano", 0.0 },
  { "Hinata", 0.040268455 },
  { "Hinatea", 0.0 },
  { "Hinckley", 1.0 },
  { "Hind", 0.0 },
  { "Hinda", 0.0 },
  { "Hindel", 0.0 },
  { "Hindy", 0.0 },
  { "Hines", 1.0 },
  { "Hing", 1.0 },
  { "Hinlee", 0.0 },
  { "Hinley", 0.0 },
  { "Hinsley", 0.0 },
  { "Hinson", 1.0 },
  { "Hinton", 1.0 },
  { "Hipolita", 0.0 },
  { "Hipolito", 1.0 },
  { "Hira", 0.0 },
  { "Hirah", 0.0 },
  { "Hiral", 0.0 },
  { "Hiram", 1.0 },
  { "Hiran", 1.0 },
  { "Hiren", 1.0 },
  { "Hirma", 0.0 },
  { "Hiro", 0.98759305 },
  { "Hiroaki", 1.0 },
  { "Hiroki", 1.0 },
  { "Hiroko", 0.0 },
  { "Hiromi", 0.066390045 },
  { "Hiromu", 1.0 },
  { "Hiromy", 0.0 },
  { "Hiroshi", 1.0 },
  { "Hiroto", 1.0 },
  { "Hiroyuki", 1.0 },
  { "Hirsch", 1.0 },
  { "Hirsh", 1.0 },
  { "Hirva", 0.0 },
  { "Hirvin", 1.0 },
  { "Hirving", 1.0 },
  { "Hisa", 0.0 },
  { "Hisae", 0.0 },
  { "Hisako", 0.0 },
  { "Hisao", 1.0 },
  { "Hisashi", 1.0 },
  { "Hisaye", 0.0 },
  { "Hisayo", 0.0 },
  { "Hisela", 0.0 },
  { "Hishaam", 1.0 },
  { "Hisham", 1.0 },
  { "Hiten", 1.0 },
  { "Hitesh", 1.0 },
  { "Hitha", 0.0 },
  { "Hitney", 0.0 },
  { "Hitomi", 0.0 },
  { "Hitoshi", 1.0 },
  { "Hitzel", 0.0 },
  { "Hiwot", 0.0 },
  { "Hixie", 0.0 },
  { "Hixon", 1.0 },
  { "Hiya", 0.0 },
  { "Hiyab", 0.16666667 },
  { "Hiyam", 0.0 },
  { "Hiyan", 1.0 },
  { "Hjalmar", 1.0 },
  { "Hjalmer", 1.0 },
  { "Hjordis", 0.0 },
  { "Hla", 0.0 },
  { "Hlee", 0.0 },
  { "Hli", 0.0 },
  { "Ho", 1.0 },
  { "Hoa", 0.4924812 },
  { "Hoai", 0.85365856 },
  { "Hoan", 1.0 },
  { "Hoang", 0.9351145 },
  { "Hoarce", 1.0 },
  { "Hobart", 1.0 },
  { "Hobbes", 1.0 },
  { "Hobbie", 1.0 },
  { "Hobbs", 1.0 },
  { "Hobby", 1.0 },
  { "Hobert", 1.0 },
  { "Hobie", 1.0 },
  { "Hobson", 1.0 },
  { "Hoby", 1.0 },
  { "Hoda", 0.0 },
  { "Hodalis", 0.0 },
  { "Hodan", 0.0 },
  { "Hodari", 1.0 },
  { "Hodaya", 0.0 },
  { "Hodge", 1.0 },
  { "Hodges", 1.0 },
  { "Hoffman", 1.0 },
  { "Hogan", 1.0 },
  { "Hoke", 1.0 },
  { "Hoku", 1.0 },
  { "Hokulani", 0.0 },
  { "Holbert", 1.0 },
  { "Holbrook", 1.0 },
  { "Holdan", 1.0 },
  { "Holden", 0.9925496 },
  { "Holder", 1.0 },
  { "Holdin", 1.0 },
  { "Holdon", 1.0 },
  { "Holdyn", 0.94033414 },
  { "Holger", 1.0 },
  { "Holiday", 0.0 },
  { "Hollace", 0.06017192 },
  { "Hollan", 0.24770643 },
  { "Holland", 0.37309605 },
  { "Holle", 0.0 },
  { "Hollee", 0.0 },
  { "Holleigh", 0.0 },
  { "Hollen", 0.0 },
  { "Holley", 0.047419805 },
  { "Holli", 0.0005805852 },
  { "Holliann", 0.0 },
  { "Hollianne", 0.0 },
  { "Hollice", 0.71428573 },
  { "Holliday", 0.0 },
  { "Hollie", 0.07536307 },
  { "Hollin", 0.0 },
  { "Hollis", 0.7761243 },
  { "Hollister", 0.7706422 },
  { "Holloway", 1.0 },
  { "Holly", 0.0076759467 },
  { "Hollyann", 0.0 },
  { "Hollyanne", 0.0 },
  { "Hollye", 0.0 },
  { "Hollylynn", 0.0 },
  { "Hollymarie", 0.0 },
  { "Hollyn", 0.0 },
  { "Hollynd", 0.0 },
  { "Hollynn", 0.0 },
  { "Holman", 1.0 },
  { "Holmer", 1.0 },
  { "Holmes", 1.0 },
  { "Holsey", 1.0 },
  { "Holsten", 1.0 },
  { "Holston", 1.0 },
  { "Holt", 1.0 },
  { "Holten", 1.0 },
  { "Holter", 1.0 },
  { "Holton", 1.0 },
  { "Holy", 0.057471264 },
  { "Holyn", 0.0 },
  { "Homar", 1.0 },
  { "Homas", 1.0 },
  { "Hombre", 1.0 },
  { "Homer", 0.9939843 },
  { "Homero", 1.0 },
  { "Hommer", 1.0 },
  { "Hommy", 1.0 },
  { "Hondo", 1.0 },
  { "Honest", 0.3951613 },
  { "Honestee", 0.0 },
  { "Honesti", 0.0 },
  { "Honestie", 0.0 },
  { "Honestii", 0.0 },
  { "Honesty", 0.0 },
  { "Honey", 0.0 },
  { "Hong", 0.30448222 },
  { "Hongan", 0.0 },
  { "Hongyu", 1.0 },
  { "Honi", 0.0 },
  { "Honie", 0.0 },
  { "Honoka", 0.0 },
  { "Honor", 0.18949687 },
  { "Honora", 0.0 },
  { "Honorae", 0.0 },
  { "Honorato", 1.0 },
  { "Honore", 0.10714286 },
  { "Honoria", 0.0 },
  { "Honorine", 0.0 },
  { "Honorio", 1.0 },
  { "Honour", 0.1978022 },
  { "Hood", 1.0 },
  { "Hooker", 1.0 },
  { "Hooman", 1.0 },
  { "Hooper", 1.0 },
  { "Hoor", 0.0 },
  { "Hoorain", 0.0 },
  { "Hooria", 0.0 },
  { "Hooriya", 0.0 },
  { "Hoover", 0.9937556 },
  { "Hope", 0.005770892 },
  { "Hopelyn", 0.0 },
  { "Hopelynn", 0.0 },
  { "Hopemarie", 0.0 },
  { "Hopeton", 1.0 },
  { "Hopi", 0.0 },
  { "Hopie", 0.0 },
  { "Hopper", 1.0 },
  { "Hopson", 1.0 },
  { "Horace", 0.9962711 },
  { "Horacio", 1.0 },
  { "Horald", 1.0 },
  { "Horatio", 1.0 },
  { "Horeb", 1.0 },
  { "Horice", 1.0 },
  { "Horizon", 0.0 },
  { "Horlando", 1.0 },
  { "Horold", 1.0 },
  { "Horrace", 1.0 },
  { "Horris", 1.0 },
  { "Horst", 1.0 },
  { "Hortence", 0.0 },
  { "Hortencia", 0.0 },
  { "Hortense", 0.0 },
  { "Hortensia", 0.0 },
  { "Horton", 1.0 },
  { "Horus", 1.0 },
  { "Hosam", 1.0 },
  { "Hosanna", 0.0 },
  { "Hosannah", 0.0 },
  { "Hoscar", 1.0 },
  { "Hose", 1.0 },
  { "Hosea", 0.9959799 },
  { "Hosey", 1.0 },
  { "Hoshi", 0.0 },
  { "Hosia", 1.0 },
  { "Hosie", 0.99114335 },
  { "Hoskie", 1.0 },
  { "Hoss", 1.0 },
  { "Hossain", 1.0 },
  { "Hossam", 1.0 },
  { "Hossein", 1.0 },
  { "Hosteen", 1.0 },
  { "Hoston", 1.0 },
  { "Houa", 0.30384615 },
  { "Houda", 0.0 },
  { "Houghton", 1.0 },
  { "Housten", 1.0 },
  { "Houstin", 1.0 },
  { "Houston", 0.9811058 },
  { "Hovanes", 1.0 },
  { "Hovannes", 1.0 },
  { "Hover", 1.0 },
  { "Hovey", 1.0 },
  { "Hovhannes", 1.0 },
  { "Hovie", 1.0 },
  { "Hovig", 1.0 },
  { "Hovik", 1.0 },
  { "Hovsep", 1.0 },
  { "Howard", 0.9951583 },
  { "Howardine", 0.0 },
  { "Howe", 1.0 },
  { "Howell", 1.0 },
  { "Howie", 1.0 },
  { "Howland", 1.0 },
  { "Hoy", 1.0 },
  { "Hoyal", 1.0 },
  { "Hoyd", 1.0 },
  { "Hoye", 1.0 },
  { "Hoyet", 1.0 },
  { "Hoyett", 1.0 },
  { "Hoyle", 1.0 },
  { "Hoyt", 0.9987318 },
  { "Hoyte", 1.0 },
  { "Hoytt", 1.0 },
  { "Hozel", 0.0 },
  { "Hozie", 1.0 },
  { "Hrach", 1.0 },
  { "Hrag", 1.0 },
  { "Hrehaan", 1.0 },
  { "Hridaan", 1.0 },
  { "Hridan", 1.0 },
  { "Hriday", 1.0 },
  { "Hridhaan", 1.0 },
  { "Hridya", 0.0 },
  { "Hrihaan", 1.0 },
  { "Hripsime", 0.0 },
  { "Hrisha", 0.0 },
  { "Hrishi", 1.0 },
  { "Hrishik", 1.0 },
  { "Hrishikesh", 1.0 },
  { "Hristina", 0.0 },
  { "Hristine", 0.0 },
  { "Hristo", 1.0 },
  { "Hristopher", 1.0 },
  { "Hristos", 1.0 },
  { "Hrithik", 1.0 },
  { "Hrithika", 0.0 },
  { "Hritik", 1.0 },
  { "Hritika", 0.0 },
  { "Hriyaan", 1.0 },
  { "Hsa", 1.0 },
  { "Hser", 0.31746033 },
  { "Htoo", 1.0 },
  { "Hu", 1.0 },
  { "Huai", 0.0 },
  { "Huan", 1.0 },
  { "Huascar", 1.0 },
  { "Hub", 1.0 },
  { "Hubart", 1.0 },
  { "Hubbard", 1.0 },
  { "Hubbert", 1.0 },
  { "Hubbie", 0.0 },
  { "Huber", 1.0 },
  { "Hubert", 0.9967114 },
  { "Huberta", 0.0 },
  { "Hubertine", 0.0 },
  { "Huberto", 1.0 },
  { "Hubery", 1.0 },
  { "Hubie", 1.0 },
  { "Huburt", 1.0 },
  { "Huck", 1.0 },
  { "Huckleberry", 1.0 },
  { "Hucksley", 1.0 },
  { "Huckson", 1.0 },
  { "Huckston", 1.0 },
  { "Hud", 1.0 },
  { "Huda", 0.0 },
  { "Hudaifa", 1.0 },
  { "Hudayfa", 1.0 },
  { "Hudayfi", 1.0 },
  { "Hudes", 0.0 },
  { "Hudeyfa", 1.0 },
  { "Hudeyfi", 1.0 },
  { "Hudhayfah", 1.0 },
  { "Hudie", 1.0 },
  { "Hudis", 0.0 },
  { "Hudsen", 0.96507937 },
  { "Hudson", 0.9804712 },
  { "Hudsyn", 0.40681818 },
  { "Hudsynn", 0.0 },
  { "Hudy", 0.0 },
  { "Hue", 0.83905965 },
  { "Huebert", 1.0 },
  { "Huel", 1.0 },
  { "Huell", 1.0 },
  { "Hueston", 1.0 },
  { "Huey", 0.9992643 },
  { "Hufsa", 0.0 },
  { "Hugh", 0.9956476 },
  { "Hughe", 1.0 },
  { "Hughes", 1.0 },
  { "Hughey", 1.0 },
  { "Hughie", 1.0 },
  { "Hughlene", 0.0 },
  { "Hughlon", 1.0 },
  { "Hughston", 1.0 },
  { "Hughy", 1.0 },
  { "Hugo", 0.99902356 },
  { "Hugues", 1.0 },
  { "Huguette", 0.0 },
  { "Hui", 0.0 },
  { "Huie", 1.0 },
  { "Hula", 0.0 },
  { "Hulali", 0.0 },
  { "Hulan", 1.0 },
  { "Hulbert", 1.0 },
  { "Hulda", 0.0 },
  { "Huldah", 0.0 },
  { "Huldia", 0.0 },
  { "Hulen", 1.0 },
  { "Hulene", 0.0 },
  { "Hulet", 1.0 },
  { "Hulett", 1.0 },
  { "Huley", 1.0 },
  { "Hulices", 1.0 },
  { "Hulin", 1.0 },
  { "Hulises", 1.0 },
  { "Hulon", 1.0 },
  { "Huma", 0.0 },
  { "Humaira", 0.0 },
  { "Humayra", 0.0 },
  { "Humbert", 1.0 },
  { "Humberto", 0.99937624 },
  { "Hume", 1.0 },
  { "Humeyra", 0.0 },
  { "Humna", 0.0 },
  { "Humphery", 1.0 },
  { "Humphrey", 1.0 },
  { "Humza", 1.0 },
  { "Humzah", 1.0 },
  { "Hunberto", 1.0 },
  { "Huner", 1.0 },
  { "Hung", 1.0 },
  { "Hunner", 1.0 },
  { "Hunnie", 0.0 },
  { "Hunny", 0.0 },
  { "Hunt", 1.0 },
  { "Hunter", 0.95146394 },
  { "Huntington", 1.0 },
  { "Huntlee", 0.48076922 },
  { "Huntleigh", 0.0 },
  { "Huntley", 0.7375643 },
  { "Huntter", 1.0 },
  { "Huntur", 1.0 },
  { "Huntyr", 0.27450982 },
  { "Huong", 0.0 },
  { "Hurain", 0.0 },
  { "Hurbert", 1.0 },
  { "Hurchel", 1.0 },
  { "Hurdis", 1.0 },
  { "Hurel", 1.0 },
  { "Huriel", 1.0 },
  { "Hurl", 1.0 },
  { "Hurley", 0.97141737 },
  { "Hurlie", 1.0 },
  { "Hurman", 1.0 },
  { "Hurmon", 1.0 },
  { "Huron", 1.0 },
  { "Hurrem", 0.0 },
  { "Hurschel", 1.0 },
  { "Hursel", 1.0 },
  { "Hurshel", 1.0 },
  { "Hurshell", 1.0 },
  { "Hurst", 1.0 },
  { "Hurston", 1.0 },
  { "Hurtha", 0.0 },
  { "Hurtis", 1.0 },
  { "Husai", 1.0 },
  { "Husain", 1.0 },
  { "Husaina", 0.0 },
  { "Husam", 1.0 },
  { "Husani", 1.0 },
  { "Husayn", 1.0 },
  { "Husein", 1.0 },
  { "Husna", 0.0 },
  { "Husnain", 1.0 },
  { "Hussain", 1.0 },
  { "Hussam", 1.0 },
  { "Hussan", 1.0 },
  { "Hussein", 1.0 },
  { "Hussen", 1.0 },
  { "Hussien", 1.0 },
  { "Hussin", 1.0 },
  { "Huston", 1.0 },
  { "Hutch", 1.0 },
  { "Hutchins", 1.0 },
  { "Hutchinson", 1.0 },
  { "Hutchison", 1.0 },
  { "Hutson", 1.0 },
  { "Hutton", 0.9581994 },
  { "Hux", 1.0 },
  { "Huxlee", 0.6 },
  { "Huxleigh", 0.0 },
  { "Huxley", 0.9155722 },
  { "Huxlie", 0.0 },
  { "Huxon", 1.0 },
  { "Huxton", 1.0 },
  { "Huy", 1.0 },
  { "Huyen", 0.0 },
  { "Huynh", 1.0 },
  { "Huzaifa", 1.0 },
  { "Huzaifah", 1.0 },
  { "Hy", 1.0 },
  { "Hyacinth", 0.0 },
  { "Hyatt", 0.9131737 },
  { "Hyde", 1.0 },
  { "Hydea", 0.0 },
  { "Hydee", 0.0 },
  { "Hydeia", 0.0 },
  { "Hyden", 1.0 },
  { "Hyder", 1.0 },
  { "Hydi", 0.0 },
  { "Hydia", 0.0 },
  { "Hydiah", 0.0 },
  { "Hydie", 0.0 },
  { "Hydiea", 0.0 },
  { "Hydiyah", 0.0 },
  { "Hye", 0.0 },
  { "Hykeem", 1.0 },
  { "Hykeim", 1.0 },
  { "Hyla", 0.0 },
  { "Hylan", 1.0 },
  { "Hyland", 0.9354839 },
  { "Hylda", 0.0 },
  { "Hyle", 1.0 },
  { "Hylee", 0.0 },
  { "Hyleigh", 0.0 },
  { "Hylie", 0.0 },
  { "Hylton", 1.0 },
  { "Hyman", 1.0 },
  { "Hymen", 1.0 },
  { "Hymie", 1.0 },
  { "Hynlee", 0.0 },
  { "Hynleigh", 0.0 },
  { "Hyon", 1.0 },
  { "Hypatia", 0.0 },
  { "Hyram", 1.0 },
  { "Hyrum", 1.0 },
  { "Hyson", 1.0 },
  { "Hytham", 1.0 },
  { "Hyun", 0.6102088 },
  { "Hyung", 1.0 },
  { "Ia", 0.0 },
  { "Iaan", 1.0 },
  { "Iago", 1.0 },
  { "Iah", 0.0 },
  { "Iahn", 1.0 },
  { "Iain", 1.0 },
  { "Iaisha", 0.0 },
  { "Iakona", 1.0 },
  { "Ialene", 0.0 },
  { "Ialiyah", 0.0 },
  { "Iam", 1.0 },
  { "Ian", 0.9966133 },
  { "Iana", 0.0 },
  { "Ianah", 0.0 },
  { "Iancarlo", 1.0 },
  { "Ianmichael", 1.0 },
  { "Iann", 1.0 },
  { "Ianna", 0.0 },
  { "Iannah", 0.0 },
  { "Iantha", 0.0 },
  { "Ianthe", 0.0 },
  { "Ianthia", 0.0 },
  { "Ianto", 1.0 },
  { "Iara", 0.0 },
  { "Iasha", 0.0 },
  { "Iashia", 0.0 },
  { "Iasia", 0.0 },
  { "Iasiah", 0.9066667 },
  { "Iason", 1.0 },
  { "Iassac", 1.0 },
  { "Iayana", 0.0 },
  { "Iba", 0.0 },
  { "Ibaad", 1.0 },
  { "Ibad", 1.0 },
  { "Iban", 1.0 },
  { "Ibbie", 0.0 },
  { "Ibe", 1.0 },
  { "Ibet", 0.0 },
  { "Ibeth", 0.0 },
  { "Ibette", 0.0 },
  { "Ibhan", 1.0 },
  { "Ibin", 1.0 },
  { "Ibis", 0.17391305 },
  { "Ibiza", 0.0 },
  { "Ibn", 1.0 },
  { "Ibon", 0.0 },
  { "Ibraaheem", 1.0 },
  { "Ibraham", 1.0 },
  { "Ibraheem", 1.0 },
  { "Ibraheim", 1.0 },
  { "Ibrahem", 1.0 },
  { "Ibrahim", 1.0 },
  { "Ibrahima", 1.0 },
  { "Ibrohim", 1.0 },
  { "Ibtihaj", 0.0 },
  { "Ibtihal", 0.0 },
  { "Ibtisam", 0.0 },
  { "Ibukunoluwa", 0.5686275 },
  { "Ica", 0.0 },
  { "Icarus", 1.0 },
  { "Icee", 0.0 },
  { "Iceis", 0.0 },
  { "Icel", 0.0 },
  { "Icela", 0.0 },
  { "Iceland", 0.0 },
  { "Icelean", 0.0 },
  { "Icelene", 0.0 },
  { "Icely", 0.0 },
  { "Icelyn", 0.0 },
  { "Icelynn", 0.0 },
  { "Iceola", 0.0 },
  { "Ices", 0.0 },
  { "Icesis", 0.0 },
  { "Icess", 0.0 },
  { "Icesys", 0.0 },
  { "Icey", 0.0 },
  { "Ichael", 1.0 },
  { "Ichard", 1.0 },
  { "Ichelle", 0.0 },
  { "Ichigo", 1.0 },
  { "Ichiro", 1.0 },
  { "Icholas", 1.0 },
  { "Icia", 0.0 },
  { "Icie", 0.0 },
  { "Icis", 0.0 },
  { "Iciss", 0.0 },
  { "Icker", 1.0 },
  { "Icle", 0.0 },
  { "Icole", 0.0 },
  { "Icsel", 0.0 },
  { "Icsis", 0.0 },
  { "Icy", 0.0 },
  { "Icysis", 0.0 },
  { "Icyss", 0.0 },
  { "Ida", 0.0041015646 },
  { "Idabel", 0.0 },
  { "Idabell", 0.0 },
  { "Idabelle", 0.0 },
  { "Idah", 0.0 },
  { "Idahlia", 0.0 },
  { "Idalee", 0.0 },
  { "Idalene", 0.0 },
  { "Idali", 0.0 },
  { "Idalia", 0.0 },
  { "Idalie", 0.0 },
  { "Idalina", 0.0 },
  { "Idalis", 0.0 },
  { "Idaliz", 0.0 },
  { "Idalmis", 0.0 },
  { "Idalou", 0.0 },
  { "Idaly", 0.0 },
  { "Idalynn", 0.0 },
  { "Idalys", 0.0 },
  { "Idalyz", 0.0 },
  { "Idamae", 0.0 },
  { "Idamay", 0.0 },
  { "Idan", 1.0 },
  { "Idanell", 0.0 },
  { "Idania", 0.0 },
  { "Idara", 0.0 },
  { "Idas", 1.0 },
  { "Idasia", 0.0 },
  { "Idaya", 0.0 },
  { "Iddo", 1.0 },
  { "Ideal", 0.0 },
  { "Idean", 1.0 },
  { "Idel", 0.2173913 },
  { "Idele", 0.0 },
  { "Idelfonso", 1.0 },
  { "Idelia", 0.0 },
  { "Idelisa", 0.0 },
  { "Idell", 0.010973144 },
  { "Idella", 0.0 },
  { "Idellar", 0.0 },
  { "Idelle", 0.0 },
  { "Idellia", 0.0 },
  { "Iden", 1.0 },
  { "Idena", 0.0 },
  { "Idesha", 0.0 },
  { "Idessa", 0.0 },
  { "Idette", 0.0 },
  { "Idhant", 1.0 },
  { "Idi", 1.0 },
  { "Idia", 0.0 },
  { "Idil", 0.0 },
  { "Idiris", 1.0 },
  { "Idman", 0.0 },
  { "Ido", 1.0 },
  { "Idola", 0.0 },
  { "Idolina", 0.0 },
  { "Idoma", 0.0 },
  { "Idona", 0.0 },
  { "Idonia", 0.0 },
  { "Idonna", 0.0 },
  { "Idora", 0.0 },
  { "Idrees", 1.0 },
  { "Idrena", 0.0 },
  { "Idris", 0.9877351 },
  { "Idriss", 1.0 },
  { "Idrissa", 0.94545454 },
  { "Iduma", 0.0 },
  { "Idus", 1.0 },
  { "Idy", 0.0 },
  { "Iean", 1.0 },
  { "Ieasha", 0.0 },
  { "Ieashia", 0.0 },
  { "Ieda", 0.0 },
  { "Ieesha", 0.0 },
  { "Ieisha", 0.0 },
  { "Ieishia", 0.0 },
  { "Ien", 1.0 },
  { "Iesa", 1.0 },
  { "Iesha", 0.002018571 },
  { "Ieshea", 0.0 },
  { "Ieshia", 0.0 },
  { "Iestyn", 1.0 },
  { "Ieuan", 1.0 },
  { "Ieva", 0.0 },
  { "Ieysha", 0.0 },
  { "Ife", 0.0 },
  { "Ifeanyi", 1.0 },
  { "Ifeanyichukwu", 1.0 },
  { "Ifechukwu", 0.0 },
  { "Ifenna", 1.0 },
  { "Ifeoluwa", 0.30246913 },
  { "Ifeoma", 0.0 },
  { "Ifetayo", 0.0 },
  { "Ifeyinwa", 0.0 },
  { "Iffany", 0.0 },
  { "Iffat", 0.0 },
  { "Ifra", 0.0 },
  { "Ifrah", 0.0 },
  { "Ifrain", 1.0 },
  { "Iftu", 0.0 },
  { "Ifunanya", 0.0 },
  { "Iggy", 1.0 },
  { "Ignac", 1.0 },
  { "Ignace", 1.0 },
  { "Ignacia", 0.0 },
  { "Ignacio", 0.9987626 },
  { "Ignacita", 0.0 },
  { "Ignasio", 1.0 },
  { "Ignatia", 0.0 },
  { "Ignatius", 1.0 },
  { "Ignatuis", 1.0 },
  { "Ignatz", 1.0 },
  { "Ignazio", 1.0 },
  { "Igor", 1.0 },
  { "Ihab", 1.0 },
  { "Ihan", 1.0 },
  { "Ihla", 0.0 },
  { "Ihor", 1.0 },
  { "Ihsaan", 0.88095236 },
  { "Ihsan", 0.9852941 },
  { "Ihuoma", 0.0 },
  { "Ii", 1.0 },
  { "Iian", 1.0 },
  { "Iiana", 0.0 },
  { "Iiesha", 0.0 },
  { "Iisha", 0.0 },
  { "Iishia", 0.0 },
  { "Iiyana", 0.0 },
  { "Ija", 0.0 },
  { "Ijah", 0.16176471 },
  { "Ijahnae", 0.0 },
  { "Ijana", 0.0 },
  { "Ijanae", 0.0 },
  { "Ijanay", 0.0 },
  { "Ijaz", 1.0 },
  { "Ijeoma", 0.0 },
  { "Ijnanya", 0.0 },
  { "Ikai", 1.0 },
  { "Ikaia", 1.0 },
  { "Ikaika", 1.0 },
  { "Ike", 1.0 },
  { "Ikea", 0.015280136 },
  { "Ikeam", 1.0 },
  { "Ikechi", 1.0 },
  { "Ikechukwu", 1.0 },
  { "Ikeda", 0.0 },
  { "Ikee", 1.0 },
  { "Ikeem", 1.0 },
  { "Ikeer", 1.0 },
  { "Ikeia", 0.0 },
  { "Ikeisha", 0.0 },
  { "Ikem", 1.0 },
  { "Ikenna", 1.0 },
  { "Iker", 1.0 },
  { "Ikera", 0.0 },
  { "Ikeria", 0.0 },
  { "Ikesha", 0.0 },
  { "Ikeshia", 0.0 },
  { "Ikey", 1.0 },
  { "Ikeya", 0.0 },
  { "Ikeyia", 0.0 },
  { "Ikher", 1.0 },
  { "Ikhlaas", 0.0 },
  { "Ikhlas", 0.0 },
  { "Ikia", 0.0 },
  { "Ikie", 1.0 },
  { "Ikira", 0.0 },
  { "Ikisha", 0.0 },
  { "Ikponmwosa", 1.0 },
  { "Ikra", 0.0 },
  { "Ikraan", 0.0 },
  { "Ikram", 0.06439394 },
  { "Ikran", 0.0 },
  { "Iksha", 0.0 },
  { "Iktan", 1.0 },
  { "Ikuko", 0.0 },
  { "Ila", 0.0027214587 },
  { "Ilah", 0.0 },
  { "Ilai", 1.0 },
  { "Ilaina", 0.0 },
  { "Ilaisaane", 0.0 },
  { "Ilam", 1.0 },
  { "Ilamae", 0.0 },
  { "Ilan", 0.98087686 },
  { "Ilana", 0.0 },
  { "Ilanah", 0.0 },
  { "Ilani", 0.0 },
  { "Ilanie", 0.0 },
  { "Ilanna", 0.0 },
  { "Ilanny", 0.0 },
  { "Ilany", 0.0 },
  { "Ilar", 0.0 },
  { "Ilaria", 0.0 },
  { "Ilario", 1.0 },
  { "Ilay", 1.0 },
  { "Ilaya", 0.0 },
  { "Ilayah", 0.0 },
  { "Ilayda", 0.0 },
  { "Ilayna", 0.0 },
  { "Ilce", 0.0 },
  { "Ilcia", 0.0 },
  { "Ilda", 0.0 },
  { "Ildefonso", 1.0 },
  { "Ildiko", 0.0 },
  { "Ilea", 0.0 },
  { "Ileah", 0.0 },
  { "Ilean", 0.0 },
  { "Ileana", 0.0 },
  { "Ileane", 0.0 },
  { "Ileanna", 0.0 },
  { "Ilee", 0.0 },
  { "Ileen", 0.0 },
  { "Ileena", 0.0 },
  { "Ileene", 0.0 },
  { "Ileia", 0.0 },
  { "Ileigh", 0.0 },
  { "Ilena", 0.0 },
  { "Ilene", 0.0 },
  { "Ilenia", 0.0 },
  { "Ilenna", 0.0 },
  { "Ilenne", 0.0 },
  { "Iler", 0.0 },
  { "Ilesha", 0.0 },
  { "Ilet", 0.0 },
  { "Ileta", 0.0 },
  { "Iletta", 0.0 },
  { "Ilette", 0.0 },
  { "Iley", 0.4864865 },
  { "Ileyah", 0.0 },
  { "Ilga", 0.0 },
  { "Ilhaan", 0.0 },
  { "Ilham", 0.029940119 },
  { "Ilhan", 0.2560241 },
  { "Ilhana", 0.0 },
  { "Ilia", 0.13953489 },
  { "Iliah", 0.0 },
  { "Ilian", 0.74820143 },
  { "Iliana", 0.0 },
  { "Ilianah", 0.0 },
  { "Iliani", 0.0 },
  { "Ilianie", 0.0 },
  { "Ilianis", 0.0 },
  { "Ilianna", 0.0 },
  { "Ilianne", 0.0 },
  { "Ilianny", 0.0 },
  { "Iliany", 0.0 },
  { "Ilias", 1.0 },
  { "Ilicia", 0.0 },
  { "Ilihia", 0.0 },
  { "Ilija", 1.0 },
  { "Ilijah", 1.0 },
  { "Ilima", 0.0 },
  { "Ilina", 0.0 },
  { "Ilinca", 0.0 },
  { "Iline", 0.0 },
  { "Ilir", 1.0 },
  { "Ilirian", 1.0 },
  { "Ilirida", 0.0 },
  { "Ilisa", 0.0 },
  { "Ilise", 0.0 },
  { "Ilish", 0.0 },
  { "Ilisha", 0.0 },
  { "Ilissa", 0.0 },
  { "Ilithia", 0.0 },
  { "Ilithya", 0.0 },
  { "Ilithyia", 0.0 },
  { "Ilitia", 0.0 },
  { "Iliya", 0.8552632 },
  { "Iliyaas", 1.0 },
  { "Iliyah", 0.0 },
  { "Iliyan", 1.0 },
  { "Iliyana", 0.0 },
  { "Iliza", 0.0 },
  { "Ilka", 0.0 },
  { "Illa", 0.0 },
  { "Illah", 0.0 },
  { "Illana", 0.0 },
  { "Illeana", 0.0 },
  { "Illeanna", 0.0 },
  { "Illene", 0.0 },
  { "Illia", 0.0 },
  { "Illiam", 1.0 },
  { "Illiana", 0.0 },
  { "Illianna", 0.0 },
  { "Illias", 1.0 },
  { "Illidan", 1.0 },
  { "Illijah", 1.0 },
  { "Illinois", 0.3125 },
  { "Illissa", 0.0 },
  { "Illiyana", 0.0 },
  { "Illona", 0.0 },
  { "Illya", 0.9194805 },
  { "Illyana", 0.0 },
  { "Illyanna", 0.0 },
  { "Illyas", 1.0 },
  { "Illyria", 0.0 },
  { "Illyssa", 0.0 },
  { "Illythia", 0.0 },
  { "Ilma", 0.0 },
  { "Ilmi", 0.0 },
  { "Ilo", 0.033195022 },
  { "Ilomae", 0.0 },
  { "Ilomay", 0.0 },
  { "Ilona", 0.0 },
  { "Ilora", 0.0 },
  { "Ilori", 0.0 },
  { "Ilsa", 0.0 },
  { "Ilse", 0.0 },
  { "Ilsi", 0.0 },
  { "Ilsy", 0.0 },
  { "Ilva", 0.0 },
  { "Ilwad", 0.0 },
  { "Ily", 0.0 },
  { "Ilya", 0.8669355 },
  { "Ilyaas", 1.0 },
  { "Ilyan", 1.0 },
  { "Ilyana", 0.0 },
  { "Ilyanna", 0.0 },
  { "Ilyas", 1.0 },
  { "Ilyass", 1.0 },
  { "Ilyjah", 1.0 },
  { "Ilyn", 0.0 },
  { "Ilyne", 0.0 },
  { "Ilynn", 0.0 },
  { "Ilysa", 0.0 },
  { "Ilyse", 0.0 },
  { "Ilyssa", 0.0 },
  { "Ilythia", 0.0 },
  { "Ilyza", 0.0 },
  { "Ilze", 0.0 },
  { "Ima", 0.0 },
  { "Imaad", 1.0 },
  { "Imaan", 0.083478265 },
  { "Imaani", 0.0 },
  { "Imad", 1.0 },
  { "Imagean", 0.0 },
  { "Imagen", 0.0 },
  { "Imagene", 0.0 },
  { "Imagin", 0.0 },
  { "Imagine", 0.0 },
  { "Imahni", 0.0 },
  { "Imajae", 0.0 },
  { "Imajean", 0.0 },
  { "Imajen", 0.0 },
  { "Imal", 0.0 },
  { "Imala", 0.0 },
  { "Imalai", 0.0 },
  { "Imalay", 0.0 },
  { "Imam", 1.0 },
  { "Imamu", 1.0 },
  { "Iman", 0.16777474 },
  { "Imana", 0.0 },
  { "Imane", 0.0 },
  { "Imanee", 0.0 },
  { "Imani", 0.043385703 },
  { "Imanie", 0.0 },
  { "Imanii", 0.0 },
  { "Imanni", 0.0 },
  { "Imanol", 1.0 },
  { "Imanuel", 1.0 },
  { "Imany", 0.0 },
  { "Imar", 0.4074074 },
  { "Imara", 0.0 },
  { "Imari", 0.26190478 },
  { "Imaria", 0.0 },
  { "Imarion", 1.0 },
  { "Imauri", 1.0 },
  { "Imaya", 0.0 },
  { "Imberly", 0.0 },
  { "Imee", 0.0 },
  { "Imelda", 0.0 },
  { "Imena", 0.0 },
  { "Imer", 1.0 },
  { "Imere", 1.0 },
  { "Imesha", 0.0 },
  { "Imhotep", 1.0 },
  { "Imia", 0.0 },
  { "Imir", 1.0 },
  { "Imira", 0.0 },
  { "Imisioluwa", 0.0 },
  { "Imiya", 0.0 },
  { "Imiyah", 0.0 },
  { "Immacolata", 0.0 },
  { "Immaculata", 0.0 },
  { "Immaculate", 0.0 },
  { "Immani", 0.0 },
  { "Immanol", 1.0 },
  { "Immanuel", 1.0 },
  { "Immer", 1.0 },
  { "Immogene", 0.0 },
  { "Imo", 0.0055066077 },
  { "Imogean", 0.0 },
  { "Imogen", 0.0 },
  { "Imogene", 0.0020072993 },
  { "Imogin", 0.0 },
  { "Imogine", 0.0 },
  { "Imojean", 0.0 },
  { "Imojene", 0.0 },
  { "Imon", 0.6862745 },
  { "Imoni", 0.0 },
  { "Imonie", 0.0 },
  { "Imothy", 1.0 },
  { "Imperia", 0.0 },
  { "Impi", 0.0 },
  { "Imraan", 1.0 },
  { "Imran", 1.0 },
  { "Imrane", 1.0 },
  { "Imre", 1.0 },
  { "Imri", 0.7704918 },
  { "Imron", 1.0 },
  { "Imtiaz", 1.0 },
  { "Imunique", 0.0 },
  { "Imya", 0.0 },
  { "Imyah", 0.0 },
  { "In", 0.23809524 },
  { "Ina", 0.0019305593 },
  { "Inaara", 0.0 },
  { "Inaaya", 0.0 },
  { "Inaayah", 0.0 },
  { "Inabelle", 0.0 },
  { "Inacio", 1.0 },
  { "Inah", 0.0 },
  { "Inaki", 1.0 },
  { "Inaky", 1.0 },
  { "Inamae", 0.0 },
  { "Inanna", 0.0 },
  { "Inara", 0.0 },
  { "Inarah", 0.0 },
  { "Inari", 0.0 },
  { "Inas", 0.0 },
  { "Inasia", 0.0 },
  { "Inass", 0.0 },
  { "Inaya", 0.0 },
  { "Inayah", 0.0 },
  { "Inayat", 0.0 },
  { "Inbal", 0.0 },
  { "Inbar", 0.0 },
  { "Inchara", 0.0 },
  { "Inci", 0.0 },
  { "Inda", 0.0 },
  { "Indalecio", 1.0 },
  { "Indasha", 0.0 },
  { "Indasia", 0.0 },
  { "Indea", 0.0 },
  { "Indee", 0.0 },
  { "Independence", 0.0 },
  { "Inderjit", 1.0 },
  { "Inderpreet", 1.0 },
  { "Indeya", 0.0 },
  { "Indhira", 0.0 },
  { "Indi", 0.024390243 },
  { "India", 0.0012356979 },
  { "Indiah", 0.0 },
  { "Indian", 0.0 },
  { "Indiana", 0.3979849 },
  { "Indianna", 0.0 },
  { "Indica", 0.0 },
  { "Indie", 0.09309967 },
  { "Indigo", 0.19919607 },
  { "Indika", 0.0 },
  { "Indio", 1.0 },
  { "Indira", 0.0 },
  { "Indiya", 0.0 },
  { "Indiyah", 0.0 },
  { "Indonesia", 0.0 },
  { "Indra", 0.043165468 },
  { "Indrani", 0.0 },
  { "Indria", 0.0 },
  { "Indsay", 0.0 },
  { "Indu", 0.0 },
  { "Indy", 0.40902874 },
  { "Indya", 0.0 },
  { "Indyah", 0.0 },
  { "Indyia", 0.0 },
  { "Ineda", 0.0 },
  { "Ineisha", 0.0 },
  { "Ineka", 0.0 },
  { "Ineke", 0.0 },
  { "Inelda", 0.0 },
  { "Inell", 0.0 },
  { "Inella", 0.0 },
  { "Inelle", 0.0 },
  { "Ines", 0.06781101 },
  { "Inesa", 0.0 },
  { "Inese", 0.0 },
  { "Inesh", 1.0 },
  { "Inesha", 0.0 },
  { "Ineshia", 0.0 },
  { "Iness", 0.0 },
  { "Inessa", 0.0 },
  { "Ineta", 0.0 },
  { "Inetha", 0.0 },
  { "Inett", 0.0 },
  { "Inetta", 0.0 },
  { "Inette", 0.0 },
  { "Ineva", 0.0 },
  { "Iney", 0.0 },
  { "Inez", 0.013823805 },
  { "Ineza", 0.0 },
  { "Ineze", 0.0 },
  { "Infant", 0.52071685 },
  { "Infantboy", 1.0 },
  { "Infantfemale", 0.0 },
  { "Infantgirl", 0.0 },
  { "Infantmale", 1.0 },
  { "Infantof", 0.45967743 },
  { "Infboy", 1.0 },
  { "Infinite", 1.0 },
  { "Infiniti", 0.0 },
  { "Infinity", 0.0073099416 },
  { "Inga", 0.0 },
  { "Ingar", 0.0 },
  { "Inge", 0.0 },
  { "Ingeborg", 0.0 },
  { "Ingemar", 1.0 },
  { "Inger", 0.0 },
  { "Ingmar", 1.0 },
  { "Ingo", 1.0 },
  { "Ingolf", 1.0 },
  { "Ingra", 0.0 },
  { "Ingram", 0.9655172 },
  { "Ingred", 0.0 },
  { "Ingri", 0.0 },
  { "Ingrid", 0.00066570076 },
  { "Ingrida", 0.0 },
  { "Ingris", 0.0 },
  { "Ingrit", 0.0 },
  { "Ingry", 0.0 },
  { "Ingvald", 1.0 },
  { "Inia", 0.0 },
  { "Inice", 0.0 },
  { "Inigo", 1.0 },
  { "Inika", 0.0 },
  { "Iniki", 0.13461539 },
  { "Inikki", 0.0 },
  { "Iniko", 1.0 },
  { "Inioluwa", 0.37869823 },
  { "Inis", 0.0 },
  { "Inisha", 0.0 },
  { "Inita", 0.0 },
  { "Iniya", 0.0 },
  { "Iniyah", 0.0 },
  { "Iniyan", 1.0 },
  { "Inman", 1.0 },
  { "Inmar", 1.0 },
  { "Inmer", 1.0 },
  { "Inna", 0.0 },
  { "Innaya", 0.0 },
  { "Innessa", 0.0 },
  { "Innie", 0.0 },
  { "Innis", 0.88372093 },
  { "Innocence", 0.0 },
  { "Innocent", 1.0 },
  { "Inocencia", 0.0 },
  { "Inocencio", 1.0 },
  { "Inocente", 1.0 },
  { "Inoke", 1.0 },
  { "Inola", 0.0 },
  { "Inona", 0.0 },
  { "Inori", 0.0 },
  { "Inri", 1.0 },
  { "Insha", 0.0 },
  { "Inshirah", 0.0 },
  { "Insiya", 0.0 },
  { "Insiyah", 0.0 },
  { "Insley", 0.0 },
  { "Inta", 0.0 },
  { "Inti", 1.0 },
  { "Intisar", 0.0 },
  { "Inus", 0.0 },
  { "Inya", 0.0 },
  { "Inza", 0.0 },
  { "Io", 0.1092437 },
  { "Ioan", 0.9489796 },
  { "Ioana", 0.0 },
  { "Ioane", 1.0 },
  { "Ioanna", 0.0 },
  { "Ioannis", 1.0 },
  { "Iokepa", 1.0 },
  { "Iola", 0.0 },
  { "Iolana", 0.0 },
  { "Iolanda", 0.0 },
  { "Iolani", 0.0 },
  { "Iole", 0.0 },
  { "Iolene", 0.0 },
  { "Iolia", 0.0 },
  { "Ioma", 0.0 },
  { "Ion", 1.0 },
  { "Iona", 0.00042793565 },
  { "Ione", 0.0 },
  { "Ioni", 0.0 },
  { "Ionia", 0.0 },
  { "Ionna", 0.0 },
  { "Iori", 1.0 },
  { "Iory", 1.0 },
  { "Iosefa", 1.0 },
  { "Ioseph", 1.0 },
  { "Iosif", 1.0 },
  { "Iosua", 1.0 },
  { "Iota", 0.0 },
  { "Iovanna", 0.0 },
  { "Iowa", 0.0 },
  { "Ipek", 0.0 },
  { "Ipolito", 1.0 },
  { "Ipsa", 0.0 },
  { "Iqbal", 1.0 },
  { "Iqlas", 0.0 },
  { "Iqra", 0.0 },
  { "Iquan", 1.0 },
  { "Ira", 0.9039373 },
  { "Iracema", 0.0 },
  { "Irael", 1.0 },
  { "Irah", 0.5833333 },
  { "Iraida", 0.0 },
  { "Iraima", 0.0 },
  { "Irais", 0.0 },
  { "Iraiz", 0.0 },
  { "Iraj", 0.0 },
  { "Iralene", 0.0 },
  { "Iralyn", 0.0 },
  { "Iralynn", 0.0 },
  { "Iram", 0.870229 },
  { "Iran", 0.7531157 },
  { "Irandy", 0.0 },
  { "Irani", 0.0 },
  { "Irania", 0.0 },
  { "Irany", 0.0 },
  { "Iras", 0.0 },
  { "Irasema", 0.0 },
  { "Iratze", 0.0 },
  { "Irayda", 0.0 },
  { "Irazema", 0.0 },
  { "Irbin", 1.0 },
  { "Irby", 0.9769912 },
  { "Ire", 1.0 },
  { "Ireal", 0.0 },
  { "Irean", 0.0 },
  { "Ireana", 0.0 },
  { "Ireanna", 0.0 },
  { "Ireatha", 0.0 },
  { "Iredell", 1.0 },
  { "Iree", 0.0 },
  { "Ireen", 0.0 },
  { "Ireene", 0.0 },
  { "Ireion", 0.0 },
  { "Ireka", 0.0 },
  { "Irelan", 0.0 },
  { "Ireland", 0.036511943 },
  { "Irelia", 0.0 },
  { "Irely", 0.0 },
  { "Irelyn", 0.0 },
  { "Irelynd", 0.0 },
  { "Irelynn", 0.0 },
  { "Irelynne", 0.0 },
  { "Irem", 0.0 },
  { "Iremide", 0.64705884 },
  { "Iren", 0.92753625 },
  { "Irena", 0.0 },
  { "Irene", 0.0038123443 },
  { "Ireneo", 1.0 },
  { "Ireoluwa", 0.43220338 },
  { "Ireon", 0.25 },
  { "Ireona", 0.0 },
  { "Ireonna", 0.0 },
  { "Ireri", 0.0 },
  { "Iresha", 0.0 },
  { "Ireta", 0.0 },
  { "Iretha", 0.0 },
  { "Iretta", 0.0 },
  { "Irey", 1.0 },
  { "Ireyanna", 0.0 },
  { "Irfaan", 1.0 },
  { "Irfan", 1.0 },
  { "Irha", 0.0 },
  { "Irhaa", 0.0 },
  { "Iri", 0.0 },
  { "Iria", 0.16949153 },
  { "Irian", 0.0 },
  { "Iriana", 0.0 },
  { "Irianna", 0.0 },
  { "Iric", 1.0 },
  { "Iridessa", 0.0 },
  { "Iridian", 0.011904762 },
  { "Iridiana", 0.0 },
  { "Irie", 0.21086262 },
  { "Irieana", 0.0 },
  { "Iriee", 0.0 },
  { "Iriel", 0.0 },
  { "Irielle", 0.0 },
  { "Irihanna", 0.0 },
  { "Irin", 0.5 },
  { "Irina", 0.0 },
  { "Irine", 0.0 },
  { "Irineo", 1.0 },
  { "Irini", 0.0 },
  { "Iris", 0.008350571 },
  { "Irisa", 0.0 },
  { "Irish", 0.07931262 },
  { "Irisha", 0.0 },
  { "Irissa", 0.0 },
  { "Iristine", 0.0 },
  { "Irit", 0.0 },
  { "Irita", 0.0 },
  { "Iriyana", 0.0 },
  { "Iriz", 0.0 },
  { "Irja", 0.0 },
  { "Irl", 1.0 },
  { "Irland", 1.0 },
  { "Irlanda", 0.0 },
  { "Irlene", 0.0 },
  { "Irline", 0.0 },
  { "Irma", 0.0034305318 },
  { "Irmagene", 0.0 },
  { "Irmak", 0.0 },
  { "Irmalee", 0.0 },
  { "Irmalinda", 0.0 },
  { "Irmani", 0.0 },
  { "Irmgard", 0.0 },
  { "Irmina", 0.0 },
  { "Irmuun", 1.0 },
  { "Irna", 0.0 },
  { "Iroh", 1.0 },
  { "Iroha", 0.0 },
  { "Iron", 1.0 },
  { "Irona", 0.0 },
  { "Ironesha", 0.0 },
  { "Irsa", 0.0 },
  { "Irshad", 1.0 },
  { "Irulan", 0.0 },
  { "Irv", 1.0 },
  { "Irva", 0.0 },
  { "Irvan", 1.0 },
  { "Irven", 1.0 },
  { "Irvin", 0.99810976 },
  { "Irvina", 0.0 },
  { "Irvine", 0.9944629 },
  { "Irving", 0.9980452 },
  { "Irvyn", 1.0 },
  { "Irwin", 0.99924153 },
  { "Irwing", 1.0 },
  { "Iry", 1.0 },
  { "Iryana", 0.0 },
  { "Iryanna", 0.0 },
  { "Iryna", 0.0 },
  { "Irys", 0.0 },
  { "Iryss", 0.0 },
  { "Isa", 0.39906433 },
  { "Isaabella", 0.0 },
  { "Isaac", 0.9973136 },
  { "Isaack", 1.0 },
  { "Isaah", 1.0 },
  { "Isaak", 1.0 },
  { "Isaam", 1.0 },
  { "Isaaq", 1.0 },
  { "Isabeau", 0.0 },
  { "Isabel", 0.015553045 },
  { "Isabela", 0.0 },
  { "Isabelah", 0.0 },
  { "Isabele", 0.0 },
  { "Isabeli", 0.0 },
  { "Isabelita", 0.0 },
  { "Isabell", 0.0003508772 },
  { "Isabella", 0.0011640073 },
  { "Isabellagrace", 0.0 },
  { "Isabellah", 0.0 },
  { "Isabellamarie", 0.0 },
  { "Isabellarose", 0.0 },
  { "Isabellasophia", 0.0 },
  { "Isabelle", 0.000229392 },
  { "Isabellemarie", 0.0 },
  { "Isabellia", 0.0 },
  { "Isabelly", 0.0 },
  { "Isabelo", 1.0 },
  { "Isabeth", 0.0 },
  { "Isabeya", 0.0 },
  { "Isable", 0.0 },
  { "Isac", 1.0 },
  { "Isacc", 1.0 },
  { "Isack", 1.0 },
  { "Isador", 1.0 },
  { "Isadora", 0.0 },
  { "Isadore", 0.9718457 },
  { "Isael", 1.0 },
  { "Isah", 0.8780488 },
  { "Isaha", 1.0 },
  { "Isahbella", 0.0 },
  { "Isahi", 1.0 },
  { "Isahia", 1.0 },
  { "Isahiah", 1.0 },
  { "Isahias", 1.0 },
  { "Isai", 0.995254 },
  { "Isaia", 1.0 },
  { "Isaiah", 0.9970088 },
  { "Isaiaha", 1.0 },
  { "Isaiahmichael", 1.0 },
  { "Isaiahs", 1.0 },
  { "Isaian", 1.0 },
  { "Isaias", 1.0 },
  { "Isaic", 1.0 },
  { "Isaid", 1.0 },
  { "Isaih", 1.0 },
  { "Isaiha", 1.0 },
  { "Isaihas", 1.0 },
  { "Isain", 1.0 },
  { "Isair", 1.0 },
  { "Isaira", 0.0 },
  { "Isairis", 0.0 },
  { "Isais", 1.0 },
  { "Isaish", 1.0 },
  { "Isaiyah", 1.0 },
  { "Isajah", 1.0 },
  { "Isak", 1.0 },
  { "Isal", 0.0 },
  { "Isalei", 0.0 },
  { "Isalena", 0.0 },
  { "Isalia", 0.0 },
  { "Isaly", 0.0 },
  { "Isam", 1.0 },
  { "Isamar", 0.041120652 },
  { "Isamara", 0.0 },
  { "Isamari", 0.0 },
  { "Isamarie", 0.0 },
  { "Isamary", 0.0 },
  { "Isamel", 1.0 },
  { "Isami", 1.0 },
  { "Isamu", 1.0 },
  { "Isan", 1.0 },
  { "Isana", 0.0 },
  { "Isander", 1.0 },
  { "Isandro", 1.0 },
  { "Isao", 1.0 },
  { "Isaque", 1.0 },
  { "Isara", 0.0 },
  { "Isarael", 1.0 },
  { "Isarel", 1.0 },
  { "Isarely", 0.0 },
  { "Isata", 0.0 },
  { "Isatou", 0.0 },
  { "Isatu", 0.0 },
  { "Isau", 1.0 },
  { "Isaul", 1.0 },
  { "Isaura", 0.0 },
  { "Isauro", 1.0 },
  { "Isavel", 0.0 },
  { "Isavela", 0.0 },
  { "Isavella", 0.0 },
  { "Isay", 1.0 },
  { "Isaya", 1.0 },
  { "Isayah", 1.0 },
  { "Isayana", 0.0 },
  { "Isayiah", 1.0 },
  { "Isbel", 0.0 },
  { "Isbell", 0.0 },
  { "Isbella", 0.0 },
  { "Isco", 1.0 },
  { "Iseah", 1.0 },
  { "Isebell", 0.0 },
  { "Isebella", 0.0 },
  { "Isel", 0.0 },
  { "Isela", 0.0 },
  { "Iselda", 0.0 },
  { "Iselin", 0.0 },
  { "Iselis", 0.0 },
  { "Isella", 0.0 },
  { "Iselle", 0.0 },
  { "Iselys", 0.0 },
  { "Isen", 1.0 },
  { "Isenia", 0.0 },
  { "Iseral", 1.0 },
  { "Ises", 0.0 },
  { "Iseult", 0.0 },
  { "Ish", 1.0 },
  { "Isha", 0.0 },
  { "Ishaal", 0.0 },
  { "Ishaan", 1.0 },
  { "Ishaani", 0.0 },
  { "Ishaanvi", 0.0 },
  { "Ishaaq", 1.0 },
  { "Ishah", 0.0 },
  { "Ishak", 1.0 },
  { "Isham", 1.0 },
  { "Ishamel", 1.0 },
  { "Ishan", 1.0 },
  { "Ishana", 0.0 },
  { "Ishanae", 0.0 },
  { "Ishani", 0.0 },
  { "Ishant", 1.0 },
  { "Ishanth", 1.0 },
  { "Ishanti", 0.0 },
  { "Ishanvi", 0.0 },
  { "Ishaq", 1.0 },
  { "Ishara", 0.0 },
  { "Ishaun", 1.0 },
  { "Ishawn", 1.0 },
  { "Ishayu", 1.0 },
  { "Ishbel", 0.0 },
  { "Ishea", 0.0 },
  { "Ishi", 0.088607594 },
  { "Ishia", 0.0 },
  { "Ishiah", 1.0 },
  { "Ishika", 0.0 },
  { "Ishita", 0.0 },
  { "Ishitha", 0.0 },
  { "Ishmael", 1.0 },
  { "Ishmail", 1.0 },
  { "Ishmam", 1.0 },
  { "Ishman", 1.0 },
  { "Ishmeal", 1.0 },
  { "Ishmeet", 0.33333334 },
  { "Ishmel", 1.0 },
  { "Ishmil", 1.0 },
  { "Ishna", 0.0 },
  { "Ishraq", 1.0 },
  { "Ishrat", 0.0 },
  { "Ishta", 0.0 },
  { "Ishwaq", 0.0 },
  { "Ishwari", 0.0 },
  { "Ishya", 0.0 },
  { "Isia", 0.11363637 },
  { "Isiaah", 1.0 },
  { "Isiac", 1.0 },
  { "Isiah", 0.9995299 },
  { "Isiaha", 1.0 },
  { "Isiaih", 1.0 },
  { "Isias", 1.0 },
  { "Isiash", 1.0 },
  { "Isibeal", 0.0 },
  { "Isibella", 0.0 },
  { "Isidor", 1.0 },
  { "Isidora", 0.0 },
  { "Isidore", 1.0 },
  { "Isidoro", 1.0 },
  { "Isidra", 0.0 },
  { "Isidro", 1.0 },
  { "Isileli", 1.0 },
  { "Isioma", 0.0 },
  { "Isis", 0.0011007155 },
  { "Isiss", 0.0 },
  { "Iskandar", 1.0 },
  { "Iskander", 1.0 },
  { "Isla", 0.0 },
  { "Islah", 0.07575758 },
  { "Islam", 0.80216384 },
  { "Island", 0.024590164 },
  { "Islarose", 0.0 },
  { "Islay", 0.0 },
  { "Islee", 0.0 },
  { "Isley", 0.22653316 },
  { "Islynn", 0.0 },
  { "Isma", 0.0 },
  { "Ismaaeel", 1.0 },
  { "Ismaeel", 1.0 },
  { "Ismael", 0.9978832 },
  { "Ismaela", 0.0 },
  { "Ismah", 0.0 },
  { "Ismahan", 0.0 },
  { "Ismail", 1.0 },
  { "Ismaila", 1.0 },
  { "Ismar", 0.8942308 },
  { "Ismari", 0.0 },
  { "Ismary", 0.0 },
  { "Ismat", 0.0 },
  { "Ismay", 0.0 },
  { "Ismeal", 1.0 },
  { "Ismel", 1.0 },
  { "Ismelda", 0.0 },
  { "Ismene", 0.0 },
  { "Ismenia", 0.0 },
  { "Ismerai", 0.0 },
  { "Ismeralda", 0.0 },
  { "Ismet", 1.0 },
  { "Isobel", 0.0 },
  { "Isobell", 0.0 },
  { "Isobella", 0.0 },
  { "Isobelle", 0.0 },
  { "Isoke", 0.0 },
  { "Isoken", 0.0 },
  { "Isola", 0.0 },
  { "Isolde", 0.0 },
  { "Isolene", 0.0 },
  { "Isolina", 0.0 },
  { "Isom", 1.0 },
  { "Ison", 1.0 },
  { "Isra", 0.0056561087 },
  { "Israa", 0.0 },
  { "Israe", 0.0 },
  { "Israel", 0.98084545 },
  { "Israela", 0.0 },
  { "Israella", 0.0 },
  { "Israelle", 0.0 },
  { "Israh", 0.0 },
  { "Israt", 0.0 },
  { "Israyel", 1.0 },
  { "Isreal", 0.98651475 },
  { "Isriel", 1.0 },
  { "Isrrael", 1.0 },
  { "Issa", 0.7916219 },
  { "Issaac", 1.0 },
  { "Issabel", 0.0 },
  { "Issabela", 0.0 },
  { "Issabell", 0.0 },
  { "Issabella", 0.0 },
  { "Issabelle", 0.0 },
  { "Issac", 0.9991931 },
  { "Issacc", 1.0 },
  { "Issachar", 1.0 },
  { "Issack", 1.0 },
  { "Issael", 1.0 },
  { "Issah", 1.0 },
  { "Issai", 1.0 },
  { "Issaiah", 1.0 },
  { "Issaias", 1.0 },
  { "Issaic", 1.0 },
  { "Issaih", 1.0 },
  { "Issak", 1.0 },
  { "Issam", 1.0 },
  { "Issamar", 0.0 },
  { "Issay", 1.0 },
  { "Isse", 1.0 },
  { "Issei", 1.0 },
  { "Issel", 0.0 },
  { "Issela", 0.0 },
  { "Issey", 1.0 },
  { "Issiac", 1.0 },
  { "Issiah", 1.0 },
  { "Issiaka", 1.0 },
  { "Issic", 1.0 },
  { "Issie", 0.45454547 },
  { "Issis", 0.0 },
  { "Issys", 0.0 },
  { "Istvan", 1.0 },
  { "Isys", 0.0 },
  { "Isyss", 0.0 },
  { "Iszabella", 0.0 },
  { "Iszac", 1.0 },
  { "Iszak", 1.0 },
  { "Ita", 0.0 },
  { "Itai", 1.0 },
  { "Italee", 0.0 },
  { "Itali", 0.0 },
  { "Italia", 0.0 },
  { "Italie", 0.0 },
  { "Italo", 1.0 },
  { "Italy", 0.0044682752 },
  { "Italya", 0.0 },
  { "Itamar", 0.93700784 },
  { "Itan", 1.0 },
  { "Itasca", 0.0 },
  { "Itasha", 0.0 },
  { "Itaska", 0.0 },
  { "Itati", 0.0 },
  { "Itaty", 0.0 },
  { "Itay", 1.0 },
  { "Itayetzi", 0.0 },
  { "Itcel", 0.0 },
  { "Itha", 0.0 },
  { "Ithalia", 0.0 },
  { "Ithan", 1.0 },
  { "Ithel", 1.0 },
  { "Ithiel", 1.0 },
  { "Ithzel", 0.0 },
  { "Itia", 0.0 },
  { "Itiana", 0.0 },
  { "Itianna", 0.0 },
  { "Itiel", 1.0 },
  { "Itisha", 0.0 },
  { "Itohan", 0.0 },
  { "Itsamari", 0.0 },
  { "Itsel", 0.0 },
  { "Itsue", 0.0 },
  { "Itsuki", 1.0 },
  { "Itsuko", 0.0 },
  { "Itsuo", 1.0 },
  { "Itta", 0.0 },
  { "Ittai", 1.0 },
  { "Itxel", 0.0 },
  { "Itza", 0.0 },
  { "Itzabella", 0.0 },
  { "Itzabelle", 0.0 },
  { "Itzae", 0.8218085 },
  { "Itzael", 1.0 },
  { "Itzamar", 0.0 },
  { "Itzamara", 0.0 },
  { "Itzamaray", 0.0 },
  { "Itzanami", 0.0 },
  { "Itzany", 0.0 },
  { "Itzary", 0.0 },
  { "Itzayana", 0.0 },
  { "Itzayani", 0.0 },
  { "Itzayanna", 0.0 },
  { "Itzcoatl", 1.0 },
  { "Itzel", 0.006542722 },
  { "Itzela", 0.0 },
  { "Itzell", 0.0 },
  { "Itzelle", 0.0 },
  { "Itzelt", 0.0 },
  { "Itzely", 0.0 },
  { "Itzhak", 1.0 },
  { "Itzhel", 0.0 },
  { "Itzia", 0.0 },
  { "Itziar", 1.0 },
  { "Itzuri", 0.0 },
  { "Itzury", 0.0 },
  { "Itzy", 0.0 },
  { "Iula", 0.0 },
  { "Iuma", 0.0 },
  { "Iva", 0.014861269 },
  { "Ivaan", 1.0 },
  { "Ivadean", 0.0 },
  { "Ivadel", 0.0 },
  { "Ivadell", 0.0 },
  { "Ivadelle", 0.0 },
  { "Ivagene", 0.0 },
  { "Ivah", 0.0 },
  { "Ival", 0.6968974 },
  { "Ivalee", 0.0 },
  { "Ivalene", 0.0 },
  { "Ivaline", 0.0 },
  { "Ivalou", 0.0 },
  { "Ivan", 0.99500465 },
  { "Ivana", 0.0 },
  { "Ivanah", 0.0 },
  { "Ivandejesus", 1.0 },
  { "Ivane", 0.0 },
  { "Ivanell", 0.0 },
  { "Ivanelle", 0.0 },
  { "Ivanhoe", 1.0 },
  { "Ivani", 0.0 },
  { "Ivania", 0.0 },
  { "Ivaniel", 1.0 },
  { "Ivanka", 0.0 },
  { "Ivann", 1.0 },
  { "Ivanna", 0.0 },
  { "Ivannah", 0.0 },
  { "Ivannia", 0.0 },
  { "Ivano", 1.0 },
  { "Ivany", 0.0 },
  { "Ivanya", 0.0 },
  { "Ivar", 1.0 },
  { "Ivars", 1.0 },
  { "Ivary", 0.0 },
  { "Ivaya", 0.0 },
  { "Ive", 0.21428572 },
  { "Ivee", 0.0 },
  { "Ivel", 0.48387095 },
  { "Ivelin", 0.0 },
  { "Ivelis", 0.0 },
  { "Ivelise", 0.0 },
  { "Ivelisse", 0.0 },
  { "Iveliz", 0.0 },
  { "Ivell", 0.0 },
  { "Ivelyn", 0.0 },
  { "Iven", 1.0 },
  { "Ivena", 0.0 },
  { "Iver", 0.9245902 },
  { "Ivera", 0.0 },
  { "Iverna", 0.0 },
  { "Iverson", 0.989318 },
  { "Ivery", 0.67918086 },
  { "Ives", 1.0 },
  { "Ivet", 0.0 },
  { "Iveth", 0.0 },
  { "Ivett", 0.0 },
  { "Ivetta", 0.0 },
  { "Ivette", 0.0009573959 },
  { "Ivey", 0.3627831 },
  { "Ivi", 0.0 },
  { "Ivia", 0.0 },
  { "Ivian", 0.0 },
  { "Iviana", 0.0 },
  { "Ivianna", 0.0 },
  { "Iviannah", 0.0 },
  { "Ivica", 1.0 },
  { "Ivie", 0.047262248 },
  { "Ivin", 1.0 },
  { "Ivion", 1.0 },
  { "Iviona", 0.0 },
  { "Ivionna", 0.0 },
  { "Ivis", 0.22137405 },
  { "Ivison", 1.0 },
  { "Ivo", 0.9930556 },
  { "Ivol", 1.0 },
  { "Ivola", 0.0 },
  { "Ivon", 0.3195938 },
  { "Ivona", 0.0 },
  { "Ivone", 0.0 },
  { "Ivonn", 0.0 },
  { "Ivonna", 0.0 },
  { "Ivonne", 0.0023894862 },
  { "Ivor", 0.94383395 },
  { "Ivoree", 0.0 },
  { "Ivori", 0.0 },
  { "Ivorie", 0.0 },
  { "Ivory", 0.47107533 },
  { "Ivoryana", 0.0 },
  { "Ivoryanna", 0.0 },
  { "Ivoryrose", 0.0 },
  { "Ivra", 0.0 },
  { "Ivree", 0.0 },
  { "Ivri", 1.0 },
  { "Ivry", 0.07462686 },
  { "Ivy", 0.06937266 },
  { "Ivyana", 0.0 },
  { "Ivyann", 0.0 },
  { "Ivyanna", 0.0 },
  { "Ivyanne", 0.0 },
  { "Ivye", 0.0 },
  { "Ivyl", 1.0 },
  { "Ivylynn", 0.0 },
  { "Ivymarie", 0.0 },
  { "Ivyn", 1.0 },
  { "Ivyon", 1.0 },
  { "Ivyona", 0.0 },
  { "Ivyonna", 0.0 },
  { "Ivyrose", 0.0 },
  { "Ivyunna", 0.0 },
  { "Iwalani", 0.0 },
  { "Iwan", 1.0 },
  { "Iwana", 0.0 },
  { "Iwao", 1.0 },
  { "Iwona", 0.0 },
  { "Ixareli", 0.0 },
  { "Ixayana", 0.0 },
  { "Ixcel", 0.0 },
  { "Ixchel", 0.0 },
  { "Ixchell", 0.0 },
  { "Ixel", 0.0 },
  { "Ixsel", 0.0 },
  { "Ixtzel", 0.0 },
  { "Ixzel", 0.0 },
  { "Iya", 0.0 },
  { "Iyaad", 1.0 },
  { "Iyad", 1.0 },
  { "Iyah", 0.0 },
  { "Iyahna", 0.0 },
  { "Iyali", 0.0 },
  { "Iyan", 1.0 },
  { "Iyana", 0.0 },
  { "Iyanah", 0.0 },
  { "Iyani", 0.0 },
  { "Iyania", 0.0 },
  { "Iyanla", 0.0 },
  { "Iyanna", 0.0 },
  { "Iyannah", 0.0 },
  { "Iyanni", 0.0 },
  { "Iyanuoluwa", 0.13636364 },
  { "Iyari", 0.042857144 },
  { "Iyasia", 0.0 },
  { "Iyauna", 0.0 },
  { "Iyaunna", 0.0 },
  { "Iycess", 0.0 },
  { "Iyeisha", 0.0 },
  { "Iyesha", 0.0 },
  { "Iyeshia", 0.0 },
  { "Iyhana", 0.0 },
  { "Iyiana", 0.0 },
  { "Iyinoluwa", 0.0 },
  { "Iyisha", 0.0 },
  { "Iyla", 0.0 },
  { "Iylah", 0.0 },
  { "Iylan", 0.0 },
  { "Iylee", 0.0 },
  { "Iyleen", 0.0 },
  { "Iymona", 0.0 },
  { "Iyon", 1.0 },
  { "Iyona", 0.0 },
  { "Iyonah", 0.0 },
  { "Iyone", 0.0 },
  { "Iyoni", 0.0 },
  { "Iyonia", 0.0 },
  { "Iyonna", 0.0 },
  { "Iyonnah", 0.0 },
  { "Iyonnie", 0.0 },
  { "Iysha", 0.0 },
  { "Iyshia", 0.0 },
  { "Iysis", 0.0 },
  { "Iysiss", 0.0 },
  { "Iyssis", 0.0 },
  { "Iyuna", 0.0 },
  { "Iyunna", 0.0 },
  { "Iza", 0.0 },
  { "Izaac", 1.0 },
  { "Izaack", 1.0 },
  { "Izaah", 1.0 },
  { "Izaak", 1.0 },
  { "Izaan", 1.0 },
  { "Izabel", 0.0 },
  { "Izabela", 0.0 },
  { "Izabelah", 0.0 },
  { "Izabele", 0.0 },
  { "Izabell", 0.0 },
  { "Izabella", 0.0 },
  { "Izabellah", 0.0 },
  { "Izabelle", 0.0 },
  { "Izac", 1.0 },
  { "Izacc", 1.0 },
  { "Izach", 1.0 },
  { "Izack", 1.0 },
  { "Izadora", 0.0 },
  { "Izaeah", 1.0 },
  { "Izaeh", 1.0 },
  { "Izael", 1.0 },
  { "Izah", 1.0 },
  { "Izaha", 1.0 },
  { "Izahbella", 0.0 },
  { "Izahia", 1.0 },
  { "Izai", 1.0 },
  { "Izaia", 1.0 },
  { "Izaiah", 0.99951285 },
  { "Izaias", 1.0 },
  { "Izaic", 1.0 },
  { "Izaih", 1.0 },
  { "Izaiha", 1.0 },
  { "Izair", 1.0 },
  { "Izais", 1.0 },
  { "Izaiya", 1.0 },
  { "Izaiyah", 1.0 },
  { "Izak", 1.0 },
  { "Izalea", 0.0 },
  { "Izamar", 0.013035382 },
  { "Izamara", 0.0 },
  { "Izamary", 0.0 },
  { "Izan", 1.0 },
  { "Izara", 0.0 },
  { "Izaria", 0.0 },
  { "Izariah", 0.0 },
  { "Izarra", 0.0 },
  { "Izavella", 0.0 },
  { "Izavier", 1.0 },
  { "Izavion", 1.0 },
  { "Izaya", 1.0 },
  { "Izayah", 1.0 },
  { "Izayha", 1.0 },
  { "Izayiah", 1.0 },
  { "Izayuh", 1.0 },
  { "Izea", 1.0 },
  { "Izeah", 1.0 },
  { "Izeal", 1.0 },
  { "Izear", 1.0 },
  { "Izebella", 0.0 },
  { "Izeck", 1.0 },
  { "Izeiah", 1.0 },
  { "Izek", 1.0 },
  { "Izekiel", 1.0 },
  { "Izel", 0.20510775 },
  { "Izela", 0.0 },
  { "Izelia", 0.0 },
  { "Izell", 0.97507334 },
  { "Izella", 0.0 },
  { "Izellah", 0.0 },
  { "Izelle", 0.0 },
  { "Izen", 1.0 },
  { "Izena", 0.0 },
  { "Izetta", 0.0 },
  { "Izeyah", 1.0 },
  { "Izhane", 0.0 },
  { "Izhar", 1.0 },
  { "Iziah", 1.0 },
  { "Iziaha", 1.0 },
  { "Izibella", 0.0 },
  { "Izic", 1.0 },
  { "Izick", 1.0 },
  { "Izidora", 0.0 },
  { "Izik", 1.0 },
  { "Izma", 0.0 },
  { "Izmael", 1.0 },
  { "Izna", 0.0 },
  { "Izobel", 0.0 },
  { "Izola", 0.0 },
  { "Izona", 0.0 },
  { "Izora", 0.0 },
  { "Izra", 1.0 },
  { "Izrael", 1.0 },
  { "Izreal", 1.0 },
  { "Izriel", 1.0 },
  { "Izsabella", 0.0 },
  { "Izsak", 1.0 },
  { "Izuchukwu", 1.0 },
  { "Izumi", 0.0 },
  { "Izyaan", 1.0 },
  { "Izyah", 1.0 },
  { "Izyan", 1.0 },
  { "Izybella", 0.0 },
  { "Izyck", 1.0 },
  { "Izyk", 1.0 },
  { "Izza", 0.0 },
  { "Izzabel", 0.0 },
  { "Izzabela", 0.0 },
  { "Izzabell", 0.0 },
  { "Izzabella", 0.0 },
  { "Izzabellah", 0.0 },
  { "Izzabelle", 0.0 },
  { "Izzac", 1.0 },
  { "Izzack", 1.0 },
  { "Izzah", 0.0 },
  { "Izzaiah", 1.0 },
  { "Izzak", 1.0 },
  { "Izzat", 1.0 },
  { "Izzi", 0.0 },
  { "Izziah", 1.0 },
  { "Izzibella", 0.0 },
  { "Izzie", 0.0 },
  { "Izzik", 1.0 },
  { "Izzy", 0.3091873 },
  { "Izzybella", 0.0 },
  { "Ja", 0.66 },
  { "Jaabir", 1.0 },
  { "Jaad", 1.0 },
  { "Jaada", 0.0 },
  { "Jaade", 0.0 },
  { "Jaaden", 1.0 },
  { "Jaafar", 1.0 },
  { "Jaala", 0.0 },
  { "Jaalah", 0.0 },
  { "Jaalen", 1.0 },
  { "Jaali", 1.0 },
  { "Jaaliyah", 0.0 },
  { "Jaalyn", 0.54545456 },
  { "Jaamal", 1.0 },
  { "Jaan", 1.0 },
  { "Jaana", 0.0 },
  { "Jaanai", 0.0 },
  { "Jaanna", 0.0 },
  { "Jaanvi", 0.0 },
  { "Jaaron", 1.0 },
  { "Jaasia", 0.0 },
  { "Jaasiel", 1.0 },
  { "Jaasir", 1.0 },
  { "Jaason", 1.0 },
  { "Jaasritha", 0.0 },
  { "Jaavon", 1.0 },
  { "Jaazaniah", 0.25 },
  { "Jaaziah", 0.96407187 },
  { "Jaaziel", 0.98507464 },
  { "Jab", 1.0 },
  { "Jabaar", 1.0 },
  { "Jabahri", 1.0 },
  { "Jabali", 1.0 },
  { "Jabar", 1.0 },
  { "Jabare", 1.0 },
  { "Jabaree", 1.0 },
  { "Jabari", 1.0 },
  { "Jabarie", 1.0 },
  { "Jabarion", 1.0 },
  { "Jabaris", 1.0 },
  { "Jabarius", 1.0 },
  { "Jabarr", 1.0 },
  { "Jabarri", 1.0 },
  { "Jabarrie", 1.0 },
  { "Jabary", 1.0 },
  { "Jabbaar", 1.0 },
  { "Jabbar", 1.0 },
  { "Jabdiel", 1.0 },
  { "Jabe", 1.0 },
  { "Jabel", 1.0 },
  { "Jaben", 1.0 },
  { "Jaber", 1.0 },
  { "Jabes", 1.0 },
  { "Jabez", 1.0 },
  { "Jabian", 1.0 },
  { "Jabien", 1.0 },
  { "Jabier", 1.0 },
  { "Jabin", 1.0 },
  { "Jabina", 0.0 },
  { "Jabir", 1.0 },
  { "Jabo", 1.0 },
  { "Jaboa", 0.0 },
  { "Jabob", 1.0 },
  { "Jabon", 1.0 },
  { "Jabora", 0.0 },
  { "Jaboree", 1.0 },
  { "Jabori", 1.0 },
  { "Jaborie", 1.0 },
  { "Jaboris", 1.0 },
  { "Jabrae", 1.0 },
  { "Jabrail", 1.0 },
  { "Jabrandon", 1.0 },
  { "Jabraun", 1.0 },
  { "Jabrayah", 0.0 },
  { "Jabrayden", 1.0 },
  { "Jabraylen", 1.0 },
  { "Jabraylin", 1.0 },
  { "Jabraylon", 1.0 },
  { "Jabre", 0.9264706 },
  { "Jabrea", 0.0 },
  { "Jabree", 0.81333333 },
  { "Jabreel", 1.0 },
  { "Jabreena", 0.0 },
  { "Jabreia", 0.0 },
  { "Jabrel", 1.0 },
  { "Jabrell", 1.0 },
  { "Jabreon", 1.0 },
  { "Jabreya", 0.0 },
  { "Jabri", 0.9227053 },
  { "Jabria", 0.0 },
  { "Jabriah", 0.0 },
  { "Jabrian", 1.0 },
  { "Jabriel", 1.0 },
  { "Jabrielle", 0.0 },
  { "Jabrien", 1.0 },
  { "Jabril", 1.0 },
  { "Jabrill", 1.0 },
  { "Jabrina", 0.0 },
  { "Jabrion", 1.0 },
  { "Jabriya", 0.0 },
  { "Jabriyah", 0.0 },
  { "Jabron", 1.0 },
  { "Jabryan", 1.0 },
  { "Jabulani", 1.0 },
  { "Jabus", 1.0 },
  { "Jac", 1.0 },
  { "Jacab", 1.0 },
  { "Jacaden", 1.0 },
  { "Jacai", 1.0 },
  { "Jacaiden", 1.0 },
  { "Jacaleb", 1.0 },
  { "Jacalyn", 0.0 },
  { "Jacalynn", 0.0 },
  { "Jacameron", 1.0 },
  { "Jacan", 1.0 },
  { "Jacara", 0.0 },
  { "Jacaranda", 0.0 },
  { "Jacare", 1.0 },
  { "Jacaree", 1.0 },
  { "Jacarey", 1.0 },
  { "Jacari", 0.9581448 },
  { "Jacaria", 0.0 },
  { "Jacarie", 1.0 },
  { "Jacarii", 1.0 },
  { "Jacarion", 1.0 },
  { "Jacarious", 1.0 },
  { "Jacarius", 1.0 },
  { "Jacarla", 0.0 },
  { "Jacarr", 1.0 },
  { "Jacarra", 0.0 },
  { "Jacarri", 1.0 },
  { "Jacarria", 0.0 },
  { "Jacarter", 1.0 },
  { "Jacary", 1.0 },
  { "Jacaryous", 1.0 },
  { "Jacauri", 1.0 },
  { "Jacaury", 1.0 },
  { "Jacavion", 1.0 },
  { "Jacayden", 1.0 },
  { "Jacayla", 0.0 },
  { "Jacbo", 1.0 },
  { "Jaccari", 1.0 },
  { "Jacci", 0.0 },
  { "Jaccob", 1.0 },
  { "Jace", 0.9887581 },
  { "Jacee", 0.02364865 },
  { "Jaceion", 1.0 },
  { "Jacek", 1.0 },
  { "Jacelin", 0.0 },
  { "Jaceline", 0.0 },
  { "Jacella", 0.0 },
  { "Jacelle", 0.0 },
  { "Jacely", 0.0 },
  { "Jacelyn", 0.0 },
  { "Jacelynn", 0.0 },
  { "Jacen", 1.0 },
  { "Jacenda", 0.0 },
  { "Jacenia", 0.0 },
  { "Jacent", 1.0 },
  { "Jacenta", 0.0 },
  { "Jaceon", 1.0 },
  { "Jacere", 1.0 },
  { "Jaceson", 1.0 },
  { "Jacetin", 1.0 },
  { "Jaceton", 1.0 },
  { "Jacey", 0.056512024 },
  { "Jaceyon", 1.0 },
  { "Jacgueline", 0.0 },
  { "Jachai", 0.9653979 },
  { "Jache", 0.0 },
  { "Jachelle", 0.0 },
  { "Jachin", 1.0 },
  { "Jachob", 1.0 },
  { "Jaci", 0.0 },
  { "Jacia", 0.0 },
  { "Jaciah", 0.0 },
  { "Jacian", 1.0 },
  { "Jaciana", 0.0 },
  { "Jacianna", 0.0 },
  { "Jacie", 0.0013455328 },
  { "Jaciel", 0.9917898 },
  { "Jacieon", 1.0 },
  { "Jacier", 1.0 },
  { "Jacilyn", 0.0 },
  { "Jacilynn", 0.0 },
  { "Jacin", 1.0 },
  { "Jacina", 0.0 },
  { "Jacinda", 0.0 },
  { "Jacine", 0.0 },
  { "Jacinta", 0.0 },
  { "Jacinth", 0.47826087 },
  { "Jacintha", 0.0 },
  { "Jacinto", 1.0 },
  { "Jacion", 1.0 },
  { "Jacione", 1.0 },
  { "Jacionna", 0.0 },
  { "Jacir", 1.0 },
  { "Jaciya", 0.0 },
  { "Jaciyah", 0.0 },
  { "Jaciyon", 1.0 },
  { "Jack", 0.9961177 },
  { "Jackalin", 0.0 },
  { "Jackaline", 0.0 },
  { "Jackalyn", 0.0 },
  { "Jackalynn", 0.0 },
  { "Jacke", 1.0 },
  { "Jackee", 0.0 },
  { "Jackelin", 0.0 },
  { "Jackeline", 0.0 },
  { "Jackelinne", 0.0 },
  { "Jackelyn", 0.0 },
  { "Jackelyne", 0.0 },
  { "Jackelynn", 0.0 },
  { "Jackey", 0.9472954 },
  { "Jackhenry", 1.0 },
  { "Jacki", 0.0014208582 },
  { "Jackia", 0.0 },
  { "Jackie", 0.4639088 },
  { "Jackielyn", 0.0 },
  { "Jackilyn", 0.0 },
  { "Jackilynn", 0.0 },
  { "Jackjohn", 1.0 },
  { "Jackleen", 0.0 },
  { "Jacklen", 0.0 },
  { "Jacklene", 0.0 },
  { "Jacklin", 0.0 },
  { "Jackline", 0.0 },
  { "Jacklon", 0.0 },
  { "Jacklyn", 0.0003071253 },
  { "Jacklyne", 0.0 },
  { "Jacklynn", 0.0 },
  { "Jacklynne", 0.0 },
  { "Jackman", 1.0 },
  { "Jackob", 1.0 },
  { "Jackolyn", 0.0 },
  { "Jackquelin", 0.0 },
  { "Jackqueline", 0.0 },
  { "Jackquelyn", 0.0 },
  { "Jackqulin", 0.0 },
  { "Jackquline", 0.0 },
  { "Jackqulyn", 0.0 },
  { "Jacks", 1.0 },
  { "Jacksen", 1.0 },
  { "Jacksin", 1.0 },
  { "Jackson", 0.99734765 },
  { "Jackston", 1.0 },
  { "Jacksyn", 0.79464287 },
  { "Jackthomas", 1.0 },
  { "Jackueline", 0.0 },
  { "Jacky", 0.8500728 },
  { "Jackye", 0.0 },
  { "Jacleen", 0.0 },
  { "Jaclene", 0.0 },
  { "Jaclin", 0.0 },
  { "Jaclyn", 0.0021114345 },
  { "Jaclyne", 0.0 },
  { "Jaclynn", 0.0 },
  { "Jaclynne", 0.0 },
  { "Jaco", 1.0 },
  { "Jacob", 0.99754834 },
  { "Jacoba", 0.0 },
  { "Jacobalexander", 1.0 },
  { "Jacobanthony", 1.0 },
  { "Jacobb", 1.0 },
  { "Jacobdaniel", 1.0 },
  { "Jacobe", 1.0 },
  { "Jacobee", 1.0 },
  { "Jacobey", 1.0 },
  { "Jacobi", 0.9181071 },
  { "Jacobia", 0.33333334 },
  { "Jacobian", 1.0 },
  { "Jacobie", 0.97317076 },
  { "Jacobjames", 1.0 },
  { "Jacoblee", 1.0 },
  { "Jacobmatthew", 1.0 },
  { "Jacobo", 1.0 },
  { "Jacobrobert", 1.0 },
  { "Jacobryan", 1.0 },
  { "Jacobs", 1.0 },
  { "Jacobson", 1.0 },
  { "Jacobthomas", 1.0 },
  { "Jacobus", 1.0 },
  { "Jacoby", 0.98778003 },
  { "Jacodi", 1.0 },
  { "Jacody", 1.0 },
  { "Jacolbi", 1.0 },
  { "Jacolby", 0.9893314 },
  { "Jacole", 0.03787879 },
  { "Jacolyn", 0.0 },
  { "Jacon", 1.0 },
  { "Jacop", 1.0 },
  { "Jacopo", 1.0 },
  { "Jacora", 0.0 },
  { "Jacore", 1.0 },
  { "Jacoree", 1.0 },
  { "Jacorey", 1.0 },
  { "Jacori", 0.98607886 },
  { "Jacoria", 0.0 },
  { "Jacorian", 1.0 },
  { "Jacorie", 1.0 },
  { "Jacorien", 1.0 },
  { "Jacorion", 1.0 },
  { "Jacorious", 1.0 },
  { "Jacorius", 1.0 },
  { "Jacorri", 1.0 },
  { "Jacorrian", 1.0 },
  { "Jacorrion", 1.0 },
  { "Jacory", 1.0 },
  { "Jacoub", 1.0 },
  { "Jacoury", 1.0 },
  { "Jacovian", 1.0 },
  { "Jacoy", 1.0 },
  { "Jacoya", 0.0 },
  { "Jacqeline", 0.0 },
  { "Jacqelyn", 0.0 },
  { "Jacqlene", 0.0 },
  { "Jacqline", 0.0 },
  { "Jacqlyn", 0.0 },
  { "Jacqlynn", 0.0 },
  { "Jacqua", 0.0 },
  { "Jacqual", 1.0 },
  { "Jacqualin", 0.0 },
  { "Jacqualine", 0.0 },
  { "Jacqualyn", 0.0 },
  { "Jacqualyne", 0.0 },
  { "Jacqualynn", 0.0 },
  { "Jacquan", 1.0 },
  { "Jacquana", 0.0 },
  { "Jacquane", 1.0 },
  { "Jacquanna", 0.0 },
  { "Jacquari", 1.0 },
  { "Jacquarius", 1.0 },
  { "Jacquay", 0.56666666 },
  { "Jacque", 0.32313442 },
  { "Jacquece", 0.0 },
  { "Jacquee", 0.0 },
  { "Jacquees", 1.0 },
  { "Jacquel", 0.30988592 },
  { "Jacqueleen", 0.0 },
  { "Jacquelen", 0.0 },
  { "Jacquelene", 0.0 },
  { "Jacqueli", 0.0 },
  { "Jacquelin", 0.0 },
  { "Jacquelina", 0.0 },
  { "Jacqueline", 0.0032913173 },
  { "Jacqueling", 0.0 },
  { "Jacquelinne", 0.0 },
  { "Jacquell", 0.516129 },
  { "Jacquella", 0.0 },
  { "Jacquelle", 0.0 },
  { "Jacquelline", 0.0 },
  { "Jacquely", 0.0 },
  { "Jacquelyn", 0.0013053591 },
  { "Jacquelyne", 0.0 },
  { "Jacquelynn", 0.0 },
  { "Jacquelynne", 0.0 },
  { "Jacquenette", 0.0 },
  { "Jacques", 0.9741977 },
  { "Jacquese", 0.39430895 },
  { "Jacquesha", 0.0 },
  { "Jacquetta", 0.0 },
  { "Jacquette", 0.0 },
  { "Jacquez", 1.0 },
  { "Jacqueze", 1.0 },
  { "Jacqui", 0.014705882 },
  { "Jacquia", 0.0 },
  { "Jacquie", 0.0 },
  { "Jacquil", 1.0 },
  { "Jacquiline", 0.0 },
  { "Jacquilla", 0.0 },
  { "Jacquille", 1.0 },
  { "Jacquilyn", 0.0 },
  { "Jacquilynn", 0.0 },
  { "Jacquin", 0.67391306 },
  { "Jacquis", 0.88505745 },
  { "Jacquise", 0.60833335 },
  { "Jacquisha", 0.0 },
  { "Jacquita", 0.0 },
  { "Jacquitta", 0.0 },
  { "Jacquleen", 0.0 },
  { "Jacqulene", 0.0 },
  { "Jacqulin", 0.0 },
  { "Jacqulina", 0.0 },
  { "Jacquline", 0.0 },
  { "Jacqulyn", 0.0 },
  { "Jacqulyne", 0.0 },
  { "Jacqulynn", 0.0 },
  { "Jacquoline", 0.0 },
  { "Jacquolyn", 0.0 },
  { "Jacquon", 1.0 },
  { "Jacsen", 1.0 },
  { "Jacson", 1.0 },
  { "Jacub", 1.0 },
  { "Jacueline", 0.0 },
  { "Jaculin", 0.0 },
  { "Jaculine", 0.0 },
  { "Jacy", 0.17770392 },
  { "Jacyeon", 1.0 },
  { "Jacyion", 1.0 },
  { "Jacyln", 0.0 },
  { "Jacylyn", 0.0 },
  { "Jacyn", 0.5 },
  { "Jacynda", 0.0 },
  { "Jacynth", 0.0 },
  { "Jacynthia", 0.0 },
  { "Jacyon", 1.0 },
  { "Jacyra", 0.0 },
  { "Jad", 1.0 },
  { "Jada", 0.0038166132 },
  { "Jadaan", 1.0 },
  { "Jadae", 0.0 },
  { "Jadah", 0.0 },
  { "Jadai", 0.0 },
  { "Jadaiah", 0.0 },
  { "Jadaija", 0.0 },
  { "Jadaisha", 0.0 },
  { "Jadakis", 1.0 },
  { "Jadakiss", 0.8712871 },
  { "Jadalee", 0.0 },
  { "Jadali", 0.0 },
  { "Jadalin", 0.0 },
  { "Jadalise", 0.0 },
  { "Jadaliz", 0.0 },
  { "Jadalyn", 0.0 },
  { "Jadalynn", 0.0 },
  { "Jadalys", 0.0 },
  { "Jadalyse", 0.0 },
  { "Jadamarie", 0.0 },
  { "Jadan", 0.76571035 },
  { "Jadaria", 0.0 },
  { "Jadarian", 1.0 },
  { "Jadarien", 1.0 },
  { "Jadarion", 1.0 },
  { "Jadarious", 1.0 },
  { "Jadarius", 1.0 },
  { "Jadarose", 0.0 },
  { "Jadarrian", 1.0 },
  { "Jadarrien", 1.0 },
  { "Jadarrion", 1.0 },
  { "Jadarrious", 1.0 },
  { "Jadarrius", 1.0 },
  { "Jadasha", 0.0 },
  { "Jadasia", 0.0 },
  { "Jadaveon", 1.0 },
  { "Jadavian", 1.0 },
  { "Jadavion", 1.0 },
  { "Jadaya", 0.0 },
  { "Jadayah", 0.0 },
  { "Jadd", 1.0 },
  { "Jadda", 0.0 },
  { "Jaddan", 1.0 },
  { "Jadden", 0.9161677 },
  { "Jaddiel", 1.0 },
  { "Jade", 0.06419434 },
  { "Jadea", 0.0 },
  { "Jadeah", 0.0 },
  { "Jadean", 0.25 },
  { "Jadeann", 0.0 },
  { "Jaded", 0.0 },
  { "Jadee", 0.0 },
  { "Jadeen", 0.25 },
  { "Jadein", 1.0 },
  { "Jadel", 1.0 },
  { "Jadelin", 0.0 },
  { "Jadeline", 0.0 },
  { "Jadell", 1.0 },
  { "Jadelyn", 0.0 },
  { "Jadelynn", 0.0 },
  { "Jadelynne", 0.0 },
  { "Jademarie", 0.0 },
  { "Jaden", 0.84259 },
  { "Jadence", 0.15706806 },
  { "Jadene", 0.0 },
  { "Jadenn", 0.875 },
  { "Jadenne", 0.0 },
  { "Jadeon", 1.0 },
  { "Jader", 1.0 },
  { "Jadereon", 1.0 },
  { "Jaderian", 1.0 },
  { "Jaderion", 1.0 },
  { "Jaderious", 1.0 },
  { "Jaderius", 1.0 },
  { "Jaderrick", 1.0 },
  { "Jadesha", 0.0 },
  { "Jadesola", 0.0 },
  { "Jadeth", 0.0 },
  { "Jadeveon", 1.0 },
  { "Jadey", 0.0 },
  { "Jadeyn", 0.061516453 },
  { "Jadha", 0.0 },
  { "Jadhiel", 1.0 },
  { "Jadi", 0.0 },
  { "Jadia", 0.0 },
  { "Jadiah", 0.59375 },
  { "Jadiamond", 0.0 },
  { "Jadian", 0.8873239 },
  { "Jadiana", 0.0 },
  { "Jadie", 0.07953064 },
  { "Jadiel", 1.0 },
  { "Jadielys", 0.0 },
  { "Jadien", 0.87083334 },
  { "Jadier", 1.0 },
  { "Jadin", 0.6959811 },
  { "Jadine", 0.0 },
  { "Jadir", 1.0 },
  { "Jadira", 0.0 },
  { "Jadis", 0.52014655 },
  { "Jadison", 1.0 },
  { "Jadiss", 1.0 },
  { "Jadius", 1.0 },
  { "Jadiyah", 0.0 },
  { "Jadlyn", 0.0 },
  { "Jadlynn", 0.0 },
  { "Jadn", 1.0 },
  { "Jadon", 0.98236406 },
  { "Jadonna", 0.0 },
  { "Jadonte", 1.0 },
  { "Jadora", 0.0 },
  { "Jadore", 0.09508716 },
  { "Jadrian", 0.9084249 },
  { "Jadriel", 1.0 },
  { "Jadrien", 0.98554915 },
  { "Jadus", 1.0 },
  { "Jadwiga", 0.0 },
  { "Jady", 0.024630541 },
  { "Jadyel", 1.0 },
  { "Jadyen", 1.0 },
  { "Jadyn", 0.22464655 },
  { "Jadynce", 0.0 },
  { "Jadyne", 0.45833334 },
  { "Jadynn", 0.059753954 },
  { "Jadynne", 0.0 },
  { "Jadzia", 0.0 },
  { "Jadziah", 0.0 },
  { "Jae", 0.5621688 },
  { "Jaea", 0.0 },
  { "Jaeana", 0.0 },
  { "Jaeanna", 0.0 },
  { "Jaece", 1.0 },
  { "Jaecee", 0.0 },
  { "Jaeceion", 1.0 },
  { "Jaeceon", 1.0 },
  { "Jaeci", 0.0 },
  { "Jaecia", 0.0 },
  { "Jaecion", 1.0 },
  { "Jaecob", 1.0 },
  { "Jaecyon", 1.0 },
  { "Jaeda", 0.0 },
  { "Jaedah", 0.0 },
  { "Jaedalyn", 0.0 },
  { "Jaedan", 0.8485401 },
  { "Jaede", 0.0 },
  { "Jaeden", 0.8434298 },
  { "Jaedin", 0.7732342 },
  { "Jaedon", 0.98421055 },
  { "Jaedyn", 0.43070114 },
  { "Jaedynn", 0.1 },
  { "Jaegar", 1.0 },
  { "Jaeger", 1.0 },
  { "Jaekob", 1.0 },
  { "Jaekwon", 1.0 },
  { "Jael", 0.39619827 },
  { "Jaela", 0.0 },
  { "Jaelah", 0.0 },
  { "Jaelan", 0.8592058 },
  { "Jaelani", 0.0 },
  { "Jaelanie", 0.0 },
  { "Jaelea", 0.0 },
  { "Jaeleah", 0.0 },
  { "Jaelee", 0.0 },
  { "Jaeleen", 0.0 },
  { "Jaelei", 0.0 },
  { "Jaeleigh", 0.0 },
  { "Jaelen", 0.8507565 },
  { "Jaelene", 0.0 },
  { "Jaeley", 0.0 },
  { "Jaeli", 0.0 },
  { "Jaelia", 0.0 },
  { "Jaeliah", 0.0 },
  { "Jaeliana", 0.0 },
  { "Jaeliani", 0.0 },
  { "Jaelianna", 0.0 },
  { "Jaelie", 0.0 },
  { "Jaelin", 0.5730503 },
  { "Jaelina", 0.0 },
  { "Jaeline", 0.0 },
  { "Jaelinn", 0.0 },
  { "Jaelithe", 0.0 },
  { "Jaeliyah", 0.0 },
  { "Jaell", 1.0 },
  { "Jaella", 0.0 },
  { "Jaelle", 0.0 },
  { "Jaelon", 1.0 },
  { "Jaely", 0.0 },
  { "Jaelyn", 0.08352932 },
  { "Jaelyne", 0.0 },
  { "Jaelynn", 0.01398218 },
  { "Jaelynne", 0.0 },
  { "Jaelys", 0.0 },
  { "Jaemarie", 0.0 },
  { "Jaemeson", 1.0 },
  { "Jaemi", 0.0 },
  { "Jaemie", 0.0 },
  { "Jaemin", 1.0 },
  { "Jaemir", 1.0 },
  { "Jaemon", 1.0 },
  { "Jaems", 1.0 },
  { "Jaen", 1.0 },
  { "Jaena", 0.0 },
  { "Jaenelle", 0.0 },
  { "Jaeon", 1.0 },
  { "Jaeona", 0.0 },
  { "Jaeonna", 0.0 },
  { "Jaequan", 1.0 },
  { "Jaeron", 1.0 },
  { "Jaes", 1.0 },
  { "Jaesa", 0.0 },
  { "Jaesean", 1.0 },
  { "Jaesha", 0.0 },
  { "Jaeshaun", 1.0 },
  { "Jaeshawn", 1.0 },
  { "Jaeshon", 1.0 },
  { "Jaeson", 1.0 },
  { "Jaesun", 1.0 },
  { "Jaesyn", 1.0 },
  { "Jaethan", 1.0 },
  { "Jaetyn", 1.0 },
  { "Jaevaughn", 1.0 },
  { "Jaeven", 1.0 },
  { "Jaeveon", 1.0 },
  { "Jaevian", 1.0 },
  { "Jaevien", 1.0 },
  { "Jaevier", 1.0 },
  { "Jaevin", 1.0 },
  { "Jaevion", 1.0 },
  { "Jaevon", 1.0 },
  { "Jaevyn", 1.0 },
  { "Jaewon", 1.0 },
  { "Jaewoo", 1.0 },
  { "Jaeya", 0.0 },
  { "Jaeyla", 0.0 },
  { "Jaeyon", 1.0 },
  { "Jafar", 1.0 },
  { "Jafari", 1.0 },
  { "Jafer", 1.0 },
  { "Jafet", 1.0 },
  { "Jafeth", 1.0 },
  { "Jaffar", 1.0 },
  { "Jaffer", 1.0 },
  { "Jaffet", 1.0 },
  { "Jafus", 1.0 },
  { "Jag", 1.0 },
  { "Jagan", 1.0 },
  { "Jagar", 1.0 },
  { "Jagdeep", 1.0 },
  { "Jagen", 1.0 },
  { "Jager", 1.0 },
  { "Jaggar", 1.0 },
  { "Jagger", 0.9845294 },
  { "Jaggyr", 1.0 },
  { "Jagjit", 1.0 },
  { "Jagjot", 1.0 },
  { "Jagmeet", 1.0 },
  { "Jago", 1.0 },
  { "Jagr", 1.0 },
  { "Jagroop", 1.0 },
  { "Jaguar", 1.0 },
  { "Jagur", 1.0 },
  { "Jah", 1.0 },
  { "Jahaad", 1.0 },
  { "Jahaan", 0.95 },
  { "Jahad", 1.0 },
  { "Jahai", 1.0 },
  { "Jahaida", 0.0 },
  { "Jahair", 1.0 },
  { "Jahaira", 0.0 },
  { "Jahaire", 1.0 },
  { "Jahairy", 0.0 },
  { "Jahala", 0.0 },
  { "Jahan", 0.94483984 },
  { "Jahana", 0.0 },
  { "Jahanara", 0.0 },
  { "Jahanna", 0.0 },
  { "Jahara", 0.027322404 },
  { "Jahare", 1.0 },
  { "Jaharee", 1.0 },
  { "Jahari", 0.9010989 },
  { "Jaharie", 1.0 },
  { "Jaharri", 1.0 },
  { "Jahasia", 0.0 },
  { "Jahaud", 1.0 },
  { "Jahaun", 1.0 },
  { "Jahaven", 1.0 },
  { "Jahayra", 0.0 },
  { "Jahaziah", 1.0 },
  { "Jahaziel", 1.0 },
  { "Jahbari", 1.0 },
  { "Jahcari", 1.0 },
  { "Jahcere", 1.0 },
  { "Jahcure", 1.0 },
  { "Jahda", 0.0 },
  { "Jahdae", 0.0 },
  { "Jahdai", 0.33333334 },
  { "Jahdari", 1.0 },
  { "Jahde", 0.0 },
  { "Jahden", 1.0 },
  { "Jahdiel", 1.0 },
  { "Jahdir", 1.0 },
  { "Jahdon", 1.0 },
  { "Jaheam", 1.0 },
  { "Jaheed", 1.0 },
  { "Jaheem", 1.0 },
  { "Jaheen", 1.0 },
  { "Jaheer", 1.0 },
  { "Jaheim", 0.99858356 },
  { "Jaheir", 1.0 },
  { "Jahel", 0.96644294 },
  { "Jahem", 1.0 },
  { "Jahfari", 1.0 },
  { "Jahi", 1.0 },
  { "Jahia", 0.0 },
  { "Jahid", 1.0 },
  { "Jahida", 0.0 },
  { "Jahidi", 1.0 },
  { "Jahiel", 1.0 },
  { "Jahiem", 1.0 },
  { "Jahier", 1.0 },
  { "Jahim", 1.0 },
  { "Jahime", 1.0 },
  { "Jahin", 1.0 },
  { "Jahir", 1.0 },
  { "Jahira", 0.0 },
  { "Jahirah", 0.0 },
  { "Jahiro", 1.0 },
  { "Jahiya", 0.0 },
  { "Jahiyah", 0.0 },
  { "Jahjuan", 1.0 },
  { "Jahkai", 1.0 },
  { "Jahkari", 1.0 },
  { "Jahkayla", 0.0 },
  { "Jahkeem", 1.0 },
  { "Jahkeim", 1.0 },
  { "Jahkel", 1.0 },
  { "Jahkhi", 1.0 },
  { "Jahki", 1.0 },
  { "Jahkia", 0.0 },
  { "Jahkim", 1.0 },
  { "Jahking", 1.0 },
  { "Jahkir", 1.0 },
  { "Jahkira", 0.0 },
  { "Jahkobe", 1.0 },
  { "Jahkobi", 1.0 },
  { "Jahkye", 1.0 },
  { "Jahla", 0.0 },
  { "Jahlani", 0.9447236 },
  { "Jahlaya", 0.0 },
  { "Jahlayah", 0.0 },
  { "Jahlea", 0.0 },
  { "Jahleah", 0.0 },
  { "Jahleal", 1.0 },
  { "Jahlee", 1.0 },
  { "Jahleel", 1.0 },
  { "Jahleil", 1.0 },
  { "Jahlen", 1.0 },
  { "Jahleya", 0.0 },
  { "Jahlia", 0.0 },
  { "Jahliah", 0.0 },
  { "Jahliel", 1.0 },
  { "Jahlik", 1.0 },
  { "Jahlil", 1.0 },
  { "Jahlin", 1.0 },
  { "Jahlique", 1.0 },
  { "Jahlisa", 0.0 },
  { "Jahlissa", 0.0 },
  { "Jahliya", 0.0 },
  { "Jahliyah", 0.0 },
  { "Jahlon", 1.0 },
  { "Jahlyn", 1.0 },
  { "Jahlynn", 0.0 },
  { "Jahmaad", 1.0 },
  { "Jahmaal", 1.0 },
  { "Jahmad", 1.0 },
  { "Jahmai", 1.0 },
  { "Jahmaine", 1.0 },
  { "Jahmair", 1.0 },
  { "Jahmal", 1.0 },
  { "Jahmali", 1.0 },
  { "Jahmall", 1.0 },
  { "Jahmani", 1.0 },
  { "Jahmar", 1.0 },
  { "Jahmarcus", 1.0 },
  { "Jahmaree", 1.0 },
  { "Jahmari", 1.0 },
  { "Jahmarion", 1.0 },
  { "Jahmarley", 1.0 },
  { "Jahmauri", 1.0 },
  { "Jahmaya", 0.0 },
  { "Jahmeek", 1.0 },
  { "Jahmeel", 1.0 },
  { "Jahmeer", 1.0 },
  { "Jahmeir", 1.0 },
  { "Jahmel", 1.0 },
  { "Jahmela", 0.0 },
  { "Jahmelia", 0.0 },
  { "Jahmell", 1.0 },
  { "Jahmere", 1.0 },
  { "Jahmez", 1.0 },
  { "Jahmi", 1.0 },
  { "Jahmia", 0.0 },
  { "Jahmiah", 0.23913044 },
  { "Jahmiel", 1.0 },
  { "Jahmier", 1.0 },
  { "Jahmiere", 1.0 },
  { "Jahmil", 1.0 },
  { "Jahmila", 0.0 },
  { "Jahmir", 1.0 },
  { "Jahmira", 0.0 },
  { "Jahmire", 1.0 },
  { "Jahmiya", 0.0 },
  { "Jahmiyah", 0.0 },
  { "Jahmon", 1.0 },
  { "Jahmya", 0.0 },
  { "Jahmyah", 0.0 },
  { "Jahmyia", 0.0 },
  { "Jahn", 1.0 },
  { "Jahna", 0.0 },
  { "Jahnae", 0.0 },
  { "Jahnai", 0.096491225 },
  { "Jahnari", 0.20833333 },
  { "Jahnasia", 0.0 },
  { "Jahnathan", 1.0 },
  { "Jahnavi", 0.0 },
  { "Jahnay", 0.0 },
  { "Jahnaya", 0.0 },
  { "Jahne", 0.0 },
  { "Jahnea", 0.0 },
  { "Jahnee", 0.0 },
  { "Jahneil", 1.0 },
  { "Jahnel", 0.0 },
  { "Jahnell", 0.0 },
  { "Jahnelle", 0.0 },
  { "Jahnessa", 0.0 },
  { "Jahni", 0.41262135 },
  { "Jahnia", 0.0 },
  { "Jahniah", 0.0 },
  { "Jahnice", 0.0 },
  { "Jahniece", 0.0 },
  { "Jahniel", 1.0 },
  { "Jahnique", 0.0 },
  { "Jahnise", 0.0 },
  { "Jahniya", 0.0 },
  { "Jahniyah", 0.0 },
  { "Jahnna", 0.0 },
  { "Jahnvi", 0.0 },
  { "Jahnya", 0.0 },
  { "Jahnyah", 0.0 },
  { "Jahnyia", 0.0 },
  { "Jahnyla", 0.0 },
  { "Jahod", 1.0 },
  { "Jahon", 1.0 },
  { "Jahongir", 1.0 },
  { "Jahonna", 0.0 },
  { "Jahquan", 1.0 },
  { "Jahque", 1.0 },
  { "Jahquel", 1.0 },
  { "Jahquell", 1.0 },
  { "Jahquez", 1.0 },
  { "Jahray", 1.0 },
  { "Jahred", 1.0 },
  { "Jahree", 1.0 },
  { "Jahrel", 1.0 },
  { "Jahrell", 1.0 },
  { "Jahri", 1.0 },
  { "Jahria", 0.0 },
  { "Jahriah", 0.0 },
  { "Jahriel", 1.0 },
  { "Jahrod", 1.0 },
  { "Jahron", 1.0 },
  { "Jahsai", 1.0 },
  { "Jahsan", 1.0 },
  { "Jahsean", 1.0 },
  { "Jahseem", 1.0 },
  { "Jahseer", 1.0 },
  { "Jahseim", 1.0 },
  { "Jahseir", 1.0 },
  { "Jahshaun", 1.0 },
  { "Jahshawn", 1.0 },
  { "Jahshua", 1.0 },
  { "Jahsi", 1.0 },
  { "Jahsiah", 1.0 },
  { "Jahsier", 1.0 },
  { "Jahsim", 1.0 },
  { "Jahsir", 1.0 },
  { "Jahsire", 1.0 },
  { "Jahsiyah", 1.0 },
  { "Jahson", 1.0 },
  { "Jahsun", 1.0 },
  { "Jahtavious", 1.0 },
  { "Jahvari", 1.0 },
  { "Jahvaughn", 1.0 },
  { "Jahvel", 1.0 },
  { "Jahvier", 1.0 },
  { "Jahvion", 1.0 },
  { "Jahvon", 1.0 },
  { "Jahvonte", 1.0 },
  { "Jahwan", 1.0 },
  { "Jahya", 0.0 },
  { "Jahyda", 0.0 },
  { "Jahyden", 1.0 },
  { "Jahyr", 1.0 },
  { "Jahyra", 0.0 },
  { "Jahzair", 1.0 },
  { "Jahzara", 0.0 },
  { "Jahzarah", 0.0 },
  { "Jahzari", 0.0 },
  { "Jahzaria", 0.0 },
  { "Jahzarra", 0.0 },
  { "Jahzeel", 0.75132275 },
  { "Jahzel", 0.07936508 },
  { "Jahzell", 0.0 },
  { "Jahzelle", 0.0 },
  { "Jahziah", 0.9017341 },
  { "Jahziel", 0.9558824 },
  { "Jahzier", 1.0 },
  { "Jahzion", 1.0 },
  { "Jahzir", 1.0 },
  { "Jahziya", 0.0 },
  { "Jahziyah", 0.0 },
  { "Jahzlynn", 0.0 },
  { "Jahzon", 1.0 },
  { "Jai", 0.783105 },
  { "Jaia", 0.0 },
  { "Jaiah", 0.0 },
  { "Jaiana", 0.0 },
  { "Jaianna", 0.0 },
  { "Jaiannah", 0.0 },
  { "Jaice", 0.8229167 },
  { "Jaicee", 0.0 },
  { "Jaiceion", 1.0 },
  { "Jaicen", 1.0 },
  { "Jaiceon", 1.0 },
  { "Jaicey", 0.0 },
  { "Jaiceyon", 1.0 },
  { "Jaici", 0.0 },
  { "Jaicie", 0.0 },
  { "Jaicion", 1.0 },
  { "Jaico", 1.0 },
  { "Jaicob", 1.0 },
  { "Jaicyon", 1.0 },
  { "Jaid", 0.028901733 },
  { "Jaida", 0.0 },
  { "Jaidah", 0.0 },
  { "Jaidalyn", 0.0 },
  { "Jaidalynn", 0.0 },
  { "Jaidan", 0.6127285 },
  { "Jaidden", 1.0 },
  { "Jaide", 0.0 },
  { "Jaidee", 0.0 },
  { "Jaideep", 1.0 },
  { "Jaidel", 1.0 },
  { "Jaidelyn", 0.0 },
  { "Jaiden", 0.76834327 },
  { "Jaidence", 0.13636364 },
  { "Jaidenn", 1.0 },
  { "Jaidev", 1.0 },
  { "Jaidi", 0.0 },
  { "Jaidin", 0.60625 },
  { "Jaidlyn", 0.0 },
  { "Jaidn", 1.0 },
  { "Jaidon", 0.9513473 },
  { "Jaidy", 0.0 },
  { "Jaidyn", 0.34131885 },
  { "Jaidynn", 0.121212125 },
  { "Jaie", 0.14705883 },
  { "Jaiel", 0.5555556 },
  { "Jaielle", 0.0 },
  { "Jaier", 1.0 },
  { "Jaiheem", 1.0 },
  { "Jaiheim", 1.0 },
  { "Jaiiden", 1.0 },
  { "Jaijuan", 1.0 },
  { "Jaik", 1.0 },
  { "Jaikob", 1.0 },
  { "Jaikub", 1.0 },
  { "Jaila", 0.0 },
  { "Jailah", 0.0 },
  { "Jailan", 0.82222223 },
  { "Jailani", 0.0 },
  { "Jaileah", 0.0 },
  { "Jailee", 0.0 },
  { "Jaileen", 0.0 },
  { "Jaileene", 0.0 },
  { "Jailei", 0.0 },
  { "Jaileigh", 0.0 },
  { "Jailen", 0.8068182 },
  { "Jailene", 0.0 },
  { "Jailenne", 0.0 },
  { "Jailey", 0.0 },
  { "Jaili", 0.0 },
  { "Jailia", 0.0 },
  { "Jailin", 0.4090909 },
  { "Jailine", 0.0 },
  { "Jailinne", 0.0 },
  { "Jailiyah", 0.0 },
  { "Jailon", 1.0 },
  { "Jaily", 0.0 },
  { "Jailyn", 0.1366382 },
  { "Jailyne", 0.0 },
  { "Jailynn", 0.023000523 },
  { "Jailynne", 0.0 },
  { "Jaima", 0.0 },
  { "Jaimal", 1.0 },
  { "Jaiman", 1.0 },
  { "Jaimar", 1.0 },
  { "Jaimari", 1.0 },
  { "Jaimarie", 0.0 },
  { "Jaimarion", 1.0 },
  { "Jaimaya", 0.0 },
  { "Jaime", 0.57447463 },
  { "Jaimee", 0.009621839 },
  { "Jaimelee", 0.0 },
  { "Jaimelyn", 0.0 },
  { "Jaimelynn", 0.0 },
  { "Jaimen", 1.0 },
  { "Jaimere", 1.0 },
  { "Jaimes", 1.0 },
  { "Jaimeson", 0.9377432 },
  { "Jaimey", 0.1724138 },
  { "Jaimi", 0.0 },
  { "Jaimie", 0.086893484 },
  { "Jaimielee", 0.0 },
  { "Jaimin", 1.0 },
  { "Jaimir", 1.0 },
  { "Jaimison", 0.75 },
  { "Jaimmie", 0.0 },
  { "Jaimon", 1.0 },
  { "Jaimy", 0.07073171 },
  { "Jaimya", 0.0 },
  { "Jaimyn", 1.0 },
  { "Jaimz", 1.0 },
  { "Jain", 0.0 },
  { "Jaina", 0.0 },
  { "Jainaba", 0.0 },
  { "Jainah", 0.0 },
  { "Jaine", 0.0 },
  { "Jainee", 0.0 },
  { "Jaini", 0.0 },
  { "Jainie", 0.0 },
  { "Jainil", 1.0 },
  { "Jainiya", 0.0 },
  { "Jainiyah", 0.0 },
  { "Jainy", 0.0 },
  { "Jaion", 1.0 },
  { "Jaiona", 0.0 },
  { "Jaionna", 0.0 },
  { "Jaiquan", 1.0 },
  { "Jaiquez", 1.0 },
  { "Jair", 0.9979783 },
  { "Jaira", 0.0 },
  { "Jairah", 0.0 },
  { "Jairdan", 1.0 },
  { "Jairden", 1.0 },
  { "Jaire", 0.9100257 },
  { "Jaired", 1.0 },
  { "Jairee", 1.0 },
  { "Jairen", 1.0 },
  { "Jairius", 1.0 },
  { "Jairo", 1.0 },
  { "Jairon", 1.0 },
  { "Jairus", 1.0 },
  { "Jairy", 0.0 },
  { "Jais", 1.0 },
  { "Jaisa", 0.0 },
  { "Jaisal", 1.0 },
  { "Jaise", 0.92156863 },
  { "Jaisean", 1.0 },
  { "Jaisen", 1.0 },
  { "Jaiseon", 1.0 },
  { "Jaisha", 0.0 },
  { "Jaishaun", 1.0 },
  { "Jaishawn", 1.0 },
  { "Jaishon", 1.0 },
  { "Jaisiah", 1.0 },
  { "Jaisley", 0.0 },
  { "Jaislyn", 0.0 },
  { "Jaison", 1.0 },
  { "Jaisy", 0.0 },
  { "Jaisyn", 0.5 },
  { "Jaithan", 1.0 },
  { "Jaithen", 1.0 },
  { "Jaiton", 1.0 },
  { "Jaival", 1.0 },
  { "Jaivan", 1.0 },
  { "Jaiveer", 1.0 },
  { "Jaiven", 1.0 },
  { "Jaiveon", 1.0 },
  { "Jaiver", 1.0 },
  { "Jaivian", 1.0 },
  { "Jaivien", 1.0 },
  { "Jaivin", 1.0 },
  { "Jaivion", 1.0 },
  { "Jaivon", 1.0 },
  { "Jaivyn", 1.0 },
  { "Jaivyon", 1.0 },
  { "Jaiwan", 1.0 },
  { "Jaiya", 0.0 },
  { "Jaiyah", 0.0 },
  { "Jaiyana", 0.0 },
  { "Jaiyanna", 0.0 },
  { "Jaiyce", 1.0 },
  { "Jaiyda", 0.0 },
  { "Jaiyden", 0.9677419 },
  { "Jaiyla", 0.0 },
  { "Jaiylah", 0.0 },
  { "Jaiyon", 1.0 },
  { "Jaiza", 0.0 },
  { "Jaizen", 1.0 },
  { "Jaizon", 1.0 },
  { "Jaja", 1.0 },
  { "Jajaira", 0.0 },
  { "Jajaun", 1.0 },
  { "Jajuan", 0.99523395 },
  { "Jajuana", 0.0 },
  { "Jajuane", 1.0 },
  { "Jak", 1.0 },
  { "Jakab", 1.0 },
  { "Jakada", 0.0 },
  { "Jakaden", 1.0 },
  { "Jakaela", 0.0 },
  { "Jakahri", 1.0 },
  { "Jakai", 0.9080861 },
  { "Jakaia", 0.0 },
  { "Jakaiden", 1.0 },
  { "Jakaila", 0.0 },
  { "Jakailah", 0.0 },
  { "Jakailyn", 0.0 },
  { "Jakaira", 0.0 },
  { "Jakaiya", 0.0 },
  { "Jakaiyah", 0.0 },
  { "Jakala", 0.0 },
  { "Jakalah", 0.0 },
  { "Jakale", 1.0 },
  { "Jakaleb", 1.0 },
  { "Jakalen", 1.0 },
  { "Jakalia", 0.0 },
  { "Jakalya", 0.0 },
  { "Jakalyn", 0.0 },
  { "Jakalynn", 0.0 },
  { "Jakameron", 1.0 },
  { "Jakanye", 1.0 },
  { "Jakar", 1.0 },
  { "Jakara", 0.0 },
  { "Jakarah", 0.0 },
  { "Jakare", 1.0 },
  { "Jakaree", 1.0 },
  { "Jakari", 0.9277467 },
  { "Jakaria", 0.0 },
  { "Jakarie", 1.0 },
  { "Jakarii", 1.0 },
  { "Jakarion", 1.0 },
  { "Jakarious", 1.0 },
  { "Jakaris", 1.0 },
  { "Jakarius", 1.0 },
  { "Jakarra", 0.0 },
  { "Jakarri", 1.0 },
  { "Jakashia", 0.0 },
  { "Jakasia", 0.0 },
  { "Jakauri", 1.0 },
  { "Jakavian", 1.0 },
  { "Jakavion", 1.0 },
  { "Jakavious", 1.0 },
  { "Jakaya", 0.0 },
  { "Jakayah", 0.0 },
  { "Jakayden", 1.0 },
  { "Jakayia", 0.0 },
  { "Jakayla", 0.0 },
  { "Jakaylah", 0.0 },
  { "Jakayle", 0.0 },
  { "Jakaylee", 0.0 },
  { "Jakaylen", 1.0 },
  { "Jakaylin", 0.2 },
  { "Jakaylon", 1.0 },
  { "Jake", 0.9982389 },
  { "Jakea", 0.0 },
  { "Jakeal", 1.0 },
  { "Jakeara", 0.0 },
  { "Jakeb", 1.0 },
  { "Jakeel", 1.0 },
  { "Jakeela", 0.0 },
  { "Jakeelah", 0.0 },
  { "Jakeem", 1.0 },
  { "Jakeena", 0.0 },
  { "Jakeia", 0.0 },
  { "Jakeil", 1.0 },
  { "Jakeim", 1.0 },
  { "Jakeira", 0.0 },
  { "Jakeisha", 0.0 },
  { "Jakeith", 1.0 },
  { "Jakeitha", 0.0 },
  { "Jakel", 0.96153843 },
  { "Jakela", 0.0 },
  { "Jakelia", 0.0 },
  { "Jakelin", 0.0 },
  { "Jakeline", 0.0 },
  { "Jakell", 0.962963 },
  { "Jakelle", 0.0 },
  { "Jakelyn", 0.0 },
  { "Jakelyne", 0.0 },
  { "Jakelynn", 0.0 },
  { "Jakema", 0.0 },
  { "Jaken", 1.0 },
  { "Jakendra", 0.0 },
  { "Jakendrick", 1.0 },
  { "Jakenya", 0.0 },
  { "Jakenzie", 0.0 },
  { "Jakeob", 1.0 },
  { "Jakera", 0.0 },
  { "Jakerah", 0.0 },
  { "Jakeria", 0.0 },
  { "Jakerion", 1.0 },
  { "Jakeriya", 0.0 },
  { "Jakerra", 0.0 },
  { "Jakerria", 0.0 },
  { "Jakeryan", 1.0 },
  { "Jakesha", 0.0 },
  { "Jakeshia", 0.0 },
  { "Jaketa", 0.0 },
  { "Jakeveon", 1.0 },
  { "Jakevia", 0.0 },
  { "Jakevion", 1.0 },
  { "Jakevious", 1.0 },
  { "Jakevis", 1.0 },
  { "Jakey", 1.0 },
  { "Jakeya", 0.0 },
  { "Jakeyah", 0.0 },
  { "Jakeyla", 0.0 },
  { "Jakhai", 0.98 },
  { "Jakhari", 1.0 },
  { "Jakhi", 1.0 },
  { "Jakhia", 0.0 },
  { "Jakhira", 0.0 },
  { "Jakhiya", 0.0 },
  { "Jakhy", 1.0 },
  { "Jakhya", 0.0 },
  { "Jakhye", 1.0 },
  { "Jakhyia", 0.0 },
  { "Jakhyla", 0.0 },
  { "Jaki", 0.6632997 },
  { "Jakia", 0.0 },
  { "Jakiah", 0.13004485 },
  { "Jakiara", 0.0 },
  { "Jakiaya", 0.0 },
  { "Jakie", 0.9780928 },
  { "Jakiel", 1.0 },
  { "Jakiem", 1.0 },
  { "Jakier", 1.0 },
  { "Jakiera", 0.0 },
  { "Jakierra", 0.0 },
  { "Jakii", 1.0 },
  { "Jakil", 1.0 },
  { "Jakila", 0.0 },
  { "Jakilah", 0.0 },
  { "Jakim", 1.0 },
  { "Jakima", 0.0 },
  { "Jakin", 1.0 },
  { "Jaking", 1.0 },
  { "Jakir", 1.0 },
  { "Jakira", 0.0 },
  { "Jakirah", 0.0 },
  { "Jakiria", 0.0 },
  { "Jakirra", 0.0 },
  { "Jakirrah", 0.0 },
  { "Jakirria", 0.0 },
  { "Jakisha", 0.0 },
  { "Jakita", 0.0 },
  { "Jakius", 1.0 },
  { "Jakiya", 0.0 },
  { "Jakiyah", 0.0 },
  { "Jakiyla", 0.0 },
  { "Jakiylah", 0.0 },
  { "Jakiyra", 0.0 },
  { "Jakk", 1.0 },
  { "Jakkar", 1.0 },
  { "Jakki", 0.0 },
  { "Jakkia", 0.33333334 },
  { "Jakkob", 1.0 },
  { "Jakkson", 1.0 },
  { "Jaklyn", 0.0 },
  { "Jaklynn", 0.0 },
  { "Jakob", 1.0 },
  { "Jakobe", 1.0 },
  { "Jakobee", 1.0 },
  { "Jakobey", 1.0 },
  { "Jakobi", 0.980445 },
  { "Jakobie", 1.0 },
  { "Jakoby", 1.0 },
  { "Jakoda", 1.0 },
  { "Jakodi", 1.0 },
  { "Jakolbe", 1.0 },
  { "Jakolbi", 1.0 },
  { "Jakolby", 1.0 },
  { "Jakora", 0.0 },
  { "Jakorey", 1.0 },
  { "Jakori", 0.9883721 },
  { "Jakoria", 0.0 },
  { "Jakorian", 1.0 },
  { "Jakorie", 1.0 },
  { "Jakory", 1.0 },
  { "Jakota", 1.0 },
  { "Jakov", 1.0 },
  { "Jaks", 1.0 },
  { "Jaksen", 1.0 },
  { "Jakson", 1.0 },
  { "Jaksyn", 1.0 },
  { "Jakub", 1.0 },
  { "Jakwan", 1.0 },
  { "Jakwaun", 1.0 },
  { "Jakwon", 1.0 },
  { "Jakya", 0.0 },
  { "Jakyah", 0.0 },
  { "Jakyan", 1.0 },
  { "Jakye", 1.0 },
  { "Jakyi", 1.0 },
  { "Jakyia", 0.0 },
  { "Jakyiah", 0.0 },
  { "Jakyla", 0.0 },
  { "Jakylah", 0.0 },
  { "Jakylan", 1.0 },
  { "Jakyle", 1.0 },
  { "Jakylee", 0.0 },
  { "Jakylen", 1.0 },
  { "Jakylia", 0.0 },
  { "Jakylin", 1.0 },
  { "Jakylon", 1.0 },
  { "Jakyra", 0.0 },
  { "Jakyrah", 0.0 },
  { "Jakyran", 1.0 },
  { "Jakyre", 1.0 },
  { "Jakyree", 1.0 },
  { "Jakyren", 1.0 },
  { "Jakyri", 1.0 },
  { "Jakyria", 0.0 },
  { "Jakyriah", 0.0 },
  { "Jakyrian", 1.0 },
  { "Jakyrie", 1.0 },
  { "Jakyrin", 1.0 },
  { "Jakyrion", 1.0 },
  { "Jakyron", 1.0 },
  { "Jakzenny", 0.0 },
  { "Jal", 1.0 },
  { "Jala", 0.0 },
  { "Jalaa", 0.0 },
  { "Jalaal", 1.0 },
  { "Jalacia", 0.0 },
  { "Jalae", 0.0 },
  { "Jalah", 0.009009009 },
  { "Jalaia", 0.0 },
  { "Jalaiah", 0.0 },
  { "Jalaila", 0.0 },
  { "Jalaina", 0.0 },
  { "Jalaine", 0.0 },
  { "Jalaiya", 0.0 },
  { "Jalaiyah", 0.0 },
  { "Jalal", 1.0 },
  { "Jalan", 0.92569 },
  { "Jalana", 0.0 },
  { "Jaland", 1.0 },
  { "Jalanda", 0.0 },
  { "Jalane", 0.0 },
  { "Jalani", 0.57210964 },
  { "Jalanie", 0.0 },
  { "Jalanna", 0.0 },
  { "Jalanni", 0.0 },
  { "Jalante", 1.0 },
  { "Jalasha", 0.0 },
  { "Jalashia", 0.0 },
  { "Jalasia", 0.0 },
  { "Jalaun", 1.0 },
  { "Jalay", 0.0 },
  { "Jalaya", 0.0 },
  { "Jalayah", 0.0 },
  { "Jalaycia", 0.0 },
  { "Jalayha", 0.0 },
  { "Jalayia", 0.0 },
  { "Jalayiah", 0.0 },
  { "Jalayla", 0.0 },
  { "Jalaylah", 0.0 },
  { "Jalayna", 0.0 },
  { "Jalayne", 0.0 },
  { "Jalaysha", 0.0 },
  { "Jalayshia", 0.0 },
  { "Jalaysia", 0.0 },
  { "Jalea", 0.0 },
  { "Jaleah", 0.0 },
  { "Jaleaha", 0.0 },
  { "Jaleal", 1.0 },
  { "Jalean", 1.0 },
  { "Jaleana", 0.0 },
  { "Jaleasa", 0.0 },
  { "Jaleb", 1.0 },
  { "Jalecia", 0.0 },
  { "Jalee", 0.0 },
  { "Jaleea", 0.0 },
  { "Jaleeah", 0.0 },
  { "Jaleek", 1.0 },
  { "Jaleel", 0.9941691 },
  { "Jaleela", 0.0 },
  { "Jaleen", 0.52155775 },
  { "Jaleena", 0.0 },
  { "Jaleesa", 0.0019163206 },
  { "Jaleese", 0.0 },
  { "Jaleesha", 0.0 },
  { "Jaleesia", 0.0 },
  { "Jaleeya", 0.0 },
  { "Jaleeyah", 0.0 },
  { "Jaleeza", 0.0 },
  { "Jaleh", 0.0 },
  { "Jalei", 0.0 },
  { "Jaleia", 0.0 },
  { "Jaleiah", 0.0 },
  { "Jaleigh", 0.0 },
  { "Jaleigha", 0.0 },
  { "Jaleil", 1.0 },
  { "Jalein", 1.0 },
  { "Jaleisa", 0.0 },
  { "Jaleisha", 0.0 },
  { "Jaleiya", 0.0 },
  { "Jaleiyah", 0.0 },
  { "Jalem", 1.0 },
  { "Jalen", 0.9559515 },
  { "Jalena", 0.0 },
  { "Jalencia", 0.0 },
  { "Jalend", 1.0 },
  { "Jalene", 0.05560166 },
  { "Jalenn", 1.0 },
  { "Jaleny", 0.0 },
  { "Jaleo", 1.0 },
  { "Jaleon", 1.0 },
  { "Jaleria", 0.0 },
  { "Jalesa", 0.0 },
  { "Jalese", 0.0 },
  { "Jalesha", 0.0 },
  { "Jaleshia", 0.0 },
  { "Jalesia", 0.0 },
  { "Jalessa", 0.0 },
  { "Jalesse", 0.0 },
  { "Jalessia", 0.0 },
  { "Jalexa", 0.0 },
  { "Jalexi", 0.0 },
  { "Jalexia", 0.0 },
  { "Jalexie", 0.0 },
  { "Jalexis", 0.0 },
  { "Jalexus", 0.0 },
  { "Jalexy", 0.0 },
  { "Jaley", 0.0 },
  { "Jaleya", 0.0 },
  { "Jaleyah", 0.0 },
  { "Jaleyiah", 0.0 },
  { "Jaleyza", 0.0 },
  { "Jali", 0.0 },
  { "Jalia", 0.0 },
  { "Jaliah", 0.0068649887 },
  { "Jalian", 1.0 },
  { "Jaliana", 0.0 },
  { "Jaliayah", 0.0 },
  { "Jalica", 0.0 },
  { "Jalice", 0.0 },
  { "Jalicia", 0.0 },
  { "Jalie", 0.0 },
  { "Jaliea", 0.0 },
  { "Jalieah", 0.0 },
  { "Jaliek", 1.0 },
  { "Jaliel", 1.0 },
  { "Jalien", 0.9382716 },
  { "Jaliene", 0.0 },
  { "Jaliesa", 0.0 },
  { "Jalijah", 1.0 },
  { "Jalik", 1.0 },
  { "Jalil", 0.9980143 },
  { "Jalila", 0.0 },
  { "Jalilah", 0.0 },
  { "Jalill", 1.0 },
  { "Jalin", 0.7818013 },
  { "Jalina", 0.0 },
  { "Jalinda", 0.0 },
  { "Jaline", 0.0 },
  { "Jalique", 1.0 },
  { "Jalis", 0.0 },
  { "Jalisa", 0.0010131713 },
  { "Jalise", 0.0 },
  { "Jalisha", 0.0 },
  { "Jalishia", 0.0 },
  { "Jalisia", 0.0 },
  { "Jalissa", 0.0 },
  { "Jalisse", 0.0 },
  { "Jalissia", 0.0 },
  { "Jalita", 0.0 },
  { "Jalitza", 0.0 },
  { "Jalivia", 0.0 },
  { "Jaliya", 0.0 },
  { "Jaliyah", 0.0 },
  { "Jaliyha", 0.0 },
  { "Jaliyiah", 0.0 },
  { "Jaliyl", 1.0 },
  { "Jaliza", 0.0 },
  { "Jallen", 1.0 },
  { "Jallisa", 0.0 },
  { "Jalliyah", 0.0 },
  { "Jalmer", 1.0 },
  { "Jalna", 0.0 },
  { "Jalob", 1.0 },
  { "Jalon", 0.9685775 },
  { "Jalonda", 0.0 },
  { "Jaloni", 0.66233766 },
  { "Jalonni", 1.0 },
  { "Jalonnie", 1.0 },
  { "Jalontae", 1.0 },
  { "Jalonte", 1.0 },
  { "Jalpa", 0.0 },
  { "Jalun", 1.0 },
  { "Jalya", 0.0 },
  { "Jalyah", 0.0 },
  { "Jalyce", 0.0 },
  { "Jalycia", 0.0 },
  { "Jalyia", 0.0 },
  { "Jalyiah", 0.0 },
  { "Jalyla", 0.0 },
  { "Jalylah", 0.0 },
  { "Jalyn", 0.2586152 },
  { "Jalyna", 0.0 },
  { "Jalyne", 0.06369427 },
  { "Jalynn", 0.07380152 },
  { "Jalynne", 0.0 },
  { "Jalyric", 0.1 },
  { "Jalysa", 0.0 },
  { "Jalyse", 0.0 },
  { "Jalysia", 0.0 },
  { "Jalyssa", 0.0 },
  { "Jalyssia", 0.0 },
  { "Jalyza", 0.0 },
  { "Jam", 1.0 },
  { "Jama", 0.046683047 },
  { "Jamaa", 1.0 },
  { "Jamaad", 1.0 },
  { "Jamaah", 1.0 },
  { "Jamaal", 0.99451756 },
  { "Jamaar", 1.0 },
  { "Jamaari", 1.0 },
  { "Jamaca", 0.0 },
  { "Jamacia", 0.0 },
  { "Jamad", 1.0 },
  { "Jamae", 0.06329114 },
  { "Jamael", 1.0 },
  { "Jamah", 1.0 },
  { "Jamahd", 1.0 },
  { "Jamahl", 1.0 },
  { "Jamahri", 1.0 },
  { "Jamai", 0.88209605 },
  { "Jamaia", 0.0 },
  { "Jamaiah", 0.0 },
  { "Jamaica", 0.021433355 },
  { "Jamaicia", 0.0 },
  { "Jamaika", 0.0 },
  { "Jamail", 1.0 },
  { "Jamaile", 1.0 },
  { "Jamain", 1.0 },
  { "Jamaine", 0.9877676 },
  { "Jamair", 1.0 },
  { "Jamaira", 0.0 },
  { "Jamaire", 1.0 },
  { "Jamaiya", 0.0 },
  { "Jamaiyah", 0.0 },
  { "Jamaka", 0.0 },
  { "Jamal", 0.9915119 },
  { "Jamala", 0.0 },
  { "Jamalachi", 1.0 },
  { "Jamale", 1.0 },
  { "Jamalia", 0.0 },
  { "Jamall", 1.0 },
  { "Jamalle", 1.0 },
  { "Jamalyn", 0.0 },
  { "Jaman", 1.0 },
  { "Jamanda", 0.0 },
  { "Jamane", 1.0 },
  { "Jamani", 0.5300752 },
  { "Jamantha", 0.0 },
  { "Jamar", 0.99512 },
  { "Jamara", 0.04928806 },
  { "Jamarae", 1.0 },
  { "Jamarah", 0.0 },
  { "Jamarco", 1.0 },
  { "Jamarcus", 1.0 },
  { "Jamard", 1.0 },
  { "Jamare", 1.0 },
  { "Jamarea", 0.70689654 },
  { "Jamaree", 0.99292785 },
  { "Jamareo", 1.0 },
  { "Jamareon", 1.0 },
  { "Jamarey", 1.0 },
  { "Jamari", 0.93739545 },
  { "Jamaria", 0.032314412 },
  { "Jamariae", 1.0 },
  { "Jamariah", 0.0 },
  { "Jamarian", 0.9928656 },
  { "Jamariana", 0.0 },
  { "Jamarie", 0.8614173 },
  { "Jamariea", 1.0 },
  { "Jamarien", 1.0 },
  { "Jamarieon", 1.0 },
  { "Jamarii", 1.0 },
  { "Jamarin", 1.0 },
  { "Jamario", 1.0 },
  { "Jamarion", 1.0 },
  { "Jamariona", 0.0 },
  { "Jamarious", 1.0 },
  { "Jamaris", 0.82579184 },
  { "Jamariun", 1.0 },
  { "Jamarius", 1.0 },
  { "Jamariya", 0.0 },
  { "Jamariyah", 0.0 },
  { "Jamariyon", 1.0 },
  { "Jamarkis", 1.0 },
  { "Jamarkus", 1.0 },
  { "Jamarl", 1.0 },
  { "Jamarley", 1.0 },
  { "Jamarlon", 1.0 },
  { "Jamaro", 1.0 },
  { "Jamarques", 1.0 },
  { "Jamarquez", 1.0 },
  { "Jamarquis", 1.0 },
  { "Jamarqus", 1.0 },
  { "Jamarr", 1.0 },
  { "Jamarra", 0.0 },
  { "Jamarre", 1.0 },
  { "Jamarreon", 1.0 },
  { "Jamarri", 1.0 },
  { "Jamarria", 0.0 },
  { "Jamarrian", 1.0 },
  { "Jamarrie", 1.0 },
  { "Jamarrio", 1.0 },
  { "Jamarrion", 1.0 },
  { "Jamarris", 1.0 },
  { "Jamarrius", 1.0 },
  { "Jamarus", 1.0 },
  { "Jamarvion", 1.0 },
  { "Jamary", 0.8076923 },
  { "Jamarya", 0.0 },
  { "Jamaryon", 1.0 },
  { "Jamas", 1.0 },
  { "Jamasia", 0.0 },
  { "Jamason", 1.0 },
  { "Jamaul", 1.0 },
  { "Jamaun", 1.0 },
  { "Jamaur", 1.0 },
  { "Jamauri", 0.98682636 },
  { "Jamauria", 0.0 },
  { "Jamaurie", 1.0 },
  { "Jamaurion", 1.0 },
  { "Jamaury", 1.0 },
  { "Jamay", 0.0 },
  { "Jamaya", 0.0 },
  { "Jamayah", 0.0 },
  { "Jamayca", 0.0 },
  { "Jamayel", 1.0 },
  { "Jamayia", 0.0 },
  { "Jamayiah", 0.0 },
  { "Jamayka", 0.0 },
  { "Jamayla", 0.0 },
  { "Jamayra", 0.0 },
  { "Jame", 0.894475 },
  { "Jamea", 0.0 },
  { "Jameah", 0.0 },
  { "Jameal", 1.0 },
  { "Jamear", 1.0 },
  { "Jameca", 0.0 },
  { "Jamecca", 0.0 },
  { "Jamece", 0.0 },
  { "Jamecia", 0.0 },
  { "Jamee", 0.074555404 },
  { "Jameece", 0.0 },
  { "Jameek", 1.0 },
  { "Jameeka", 0.0 },
  { "Jameel", 0.99353683 },
  { "Jameela", 0.0 },
  { "Jameelah", 0.0 },
  { "Jameele", 0.0 },
  { "Jameen", 1.0 },
  { "Jameer", 1.0 },
  { "Jameera", 0.0 },
  { "Jameerah", 0.0 },
  { "Jameese", 0.0 },
  { "Jameesha", 0.0 },
  { "Jamei", 0.0 },
  { "Jameia", 0.0 },
  { "Jameiah", 0.0 },
  { "Jameica", 0.0 },
  { "Jameice", 0.0 },
  { "Jameika", 0.0 },
  { "Jameil", 1.0 },
  { "Jameila", 0.0 },
  { "Jameion", 1.0 },
  { "Jameir", 1.0 },
  { "Jameira", 0.0 },
  { "Jameire", 1.0 },
  { "Jameis", 1.0 },
  { "Jameisha", 0.0 },
  { "Jameison", 1.0 },
  { "Jameka", 0.0 },
  { "Jamekia", 0.0 },
  { "Jamel", 0.96644723 },
  { "Jamela", 0.0 },
  { "Jamelah", 0.0 },
  { "Jamele", 1.0 },
  { "Jamelia", 0.0 },
  { "Jameliah", 0.0 },
  { "Jamell", 0.96385545 },
  { "Jamella", 0.0 },
  { "Jamellah", 0.0 },
  { "Jamelle", 0.6701209 },
  { "Jamelvin", 1.0 },
  { "Jamely", 0.0 },
  { "Jamelyn", 0.0 },
  { "Jamen", 1.0 },
  { "Jamena", 0.0 },
  { "Jamene", 0.0 },
  { "Jameon", 1.0 },
  { "Jamequa", 0.0 },
  { "Jamer", 1.0 },
  { "Jamera", 0.0 },
  { "Jamerah", 0.0 },
  { "Jamere", 0.98463684 },
  { "Jameria", 0.0 },
  { "Jameriah", 0.0 },
  { "Jamerial", 0.0 },
  { "Jamerian", 1.0 },
  { "Jamerica", 0.0 },
  { "Jamerio", 1.0 },
  { "Jamerion", 1.0 },
  { "Jamerious", 1.0 },
  { "Jameris", 1.0 },
  { "Jamerius", 1.0 },
  { "Jameriya", 0.0 },
  { "Jamerra", 0.0 },
  { "Jamerria", 0.0 },
  { "Jamerson", 1.0 },
  { "James", 0.99548775 },
  { "Jamesa", 0.0 },
  { "Jamesandrew", 1.0 },
  { "Jamesanthony", 1.0 },
  { "Jamescia", 0.0 },
  { "Jamesdaniel", 1.0 },
  { "Jamesdavid", 1.0 },
  { "Jamesdean", 1.0 },
  { "Jamese", 0.0 },
  { "Jamesedward", 1.0 },
  { "Jamesen", 0.9576271 },
  { "Jamesena", 0.0 },
  { "Jamesetta", 0.0 },
  { "Jamesha", 0.0 },
  { "Jameshia", 0.0 },
  { "Jamesia", 0.0 },
  { "Jamesina", 0.0 },
  { "Jamesjoseph", 1.0 },
  { "Jameslee", 1.0 },
  { "Jamesley", 1.0 },
  { "Jamesly", 1.0 },
  { "Jamesmatthew", 1.0 },
  { "Jamesmichael", 1.0 },
  { "Jameson", 0.9706907 },
  { "Jamespatrick", 1.0 },
  { "Jamespaul", 1.0 },
  { "Jamesrobert", 1.0 },
  { "Jamesryan", 1.0 },
  { "Jamess", 1.0 },
  { "Jamessa", 0.0 },
  { "Jamesson", 1.0 },
  { "Jamesthomas", 1.0 },
  { "Jameswilliam", 1.0 },
  { "Jamesyn", 0.43846154 },
  { "Jameta", 0.0 },
  { "Jametra", 0.0 },
  { "Jametria", 0.0 },
  { "Jametrice", 0.0 },
  { "Jametrius", 1.0 },
  { "Jametta", 0.0 },
  { "Jamette", 0.0 },
  { "Jamey", 0.68750656 },
  { "Jameya", 0.0 },
  { "Jameyah", 0.0 },
  { "Jamez", 1.0 },
  { "Jamhal", 1.0 },
  { "Jami", 0.040621214 },
  { "Jamia", 0.001507159 },
  { "Jamiah", 0.13386825 },
  { "Jamial", 1.0 },
  { "Jamian", 1.0 },
  { "Jamiann", 0.0 },
  { "Jamianne", 0.0 },
  { "Jamias", 1.0 },
  { "Jamiaya", 0.0 },
  { "Jamica", 0.0 },
  { "Jamicah", 1.0 },
  { "Jamice", 0.0 },
  { "Jamichael", 1.0 },
  { "Jamicheal", 1.0 },
  { "Jamicia", 0.0 },
  { "Jamicka", 0.0 },
  { "Jamie", 0.24138665 },
  { "Jamieann", 0.0 },
  { "Jamiece", 0.0 },
  { "Jamiee", 0.0 },
  { "Jamieka", 0.0 },
  { "Jamiel", 0.9758577 },
  { "Jamiela", 0.0 },
  { "Jamielee", 0.0 },
  { "Jamielle", 0.0 },
  { "Jamielyn", 0.0 },
  { "Jamielynn", 0.0 },
  { "Jamiemarie", 0.0 },
  { "Jamien", 0.9892009 },
  { "Jamieon", 1.0 },
  { "Jamier", 1.0 },
  { "Jamiera", 0.0 },
  { "Jamiere", 1.0 },
  { "Jamiesha", 0.0 },
  { "Jamieson", 0.94598335 },
  { "Jamih", 1.0 },
  { "Jamii", 0.0 },
  { "Jamiir", 1.0 },
  { "Jamik", 1.0 },
  { "Jamika", 0.0 },
  { "Jamikal", 1.0 },
  { "Jamikel", 1.0 },
  { "Jamikka", 0.0 },
  { "Jamil", 0.964104 },
  { "Jamila", 0.0 },
  { "Jamilah", 0.00729927 },
  { "Jamile", 0.5233645 },
  { "Jamilee", 0.0 },
  { "Jamileh", 0.0 },
  { "Jamiles", 0.0 },
  { "Jamilet", 0.0 },
  { "Jamileth", 0.0 },
  { "Jamilett", 0.0 },
  { "Jamilette", 0.0 },
  { "Jamilex", 0.0 },
  { "Jamiley", 0.0 },
  { "Jamilia", 0.0 },
  { "Jamilka", 0.0 },
  { "Jamill", 1.0 },
  { "Jamilla", 0.0 },
  { "Jamillah", 0.0 },
  { "Jamille", 0.36879432 },
  { "Jamillette", 0.0 },
  { "Jamillia", 0.0 },
  { "Jamilliah", 0.0 },
  { "Jamily", 0.0 },
  { "Jamilya", 0.0 },
  { "Jamilyah", 0.0 },
  { "Jamilyn", 0.0 },
  { "Jamilynn", 0.0 },
  { "Jamima", 0.0 },
  { "Jamin", 0.95778525 },
  { "Jamina", 0.0 },
  { "Jamine", 0.29718006 },
  { "Jamion", 1.0 },
  { "Jamiqua", 0.0 },
  { "Jamique", 1.0 },
  { "Jamir", 0.996083 },
  { "Jamira", 0.0 },
  { "Jamiracle", 0.0 },
  { "Jamirah", 0.0 },
  { "Jamire", 1.0 },
  { "Jamiria", 0.0 },
  { "Jamirion", 1.0 },
  { "Jamiron", 1.0 },
  { "Jamiroquai", 1.0 },
  { "Jamiroquan", 1.0 },
  { "Jamirr", 1.0 },
  { "Jamirra", 0.0 },
  { "Jamirrah", 0.0 },
  { "Jamirya", 0.0 },
  { "Jamis", 1.0 },
  { "Jamisa", 0.0 },
  { "Jamise", 0.0 },
  { "Jamisen", 0.6185567 },
  { "Jamisha", 0.0 },
  { "Jamison", 0.9030557 },
  { "Jamisyn", 0.06451613 },
  { "Jamita", 0.0 },
  { "Jamius", 1.0 },
  { "Jamiya", 0.0 },
  { "Jamiyah", 0.0 },
  { "Jamiyha", 0.0 },
  { "Jamiyl", 1.0 },
  { "Jamiyla", 0.0 },
  { "Jamiylah", 0.0 },
  { "Jammal", 1.0 },
  { "Jammar", 1.0 },
  { "Jammel", 1.0 },
  { "Jammes", 1.0 },
  { "Jammey", 1.0 },
  { "Jammi", 0.0 },
  { "Jammie", 0.3751931 },
  { "Jamms", 1.0 },
  { "Jammy", 0.7186147 },
  { "Jamod", 1.0 },
  { "Jamol", 1.0 },
  { "Jamon", 0.9976726 },
  { "Jamond", 1.0 },
  { "Jamone", 1.0 },
  { "Jamoni", 0.5770751 },
  { "Jamonica", 0.0 },
  { "Jamonie", 0.87234044 },
  { "Jamont", 1.0 },
  { "Jamonta", 1.0 },
  { "Jamontae", 1.0 },
  { "Jamontay", 1.0 },
  { "Jamonte", 1.0 },
  { "Jamontez", 1.0 },
  { "Jamor", 1.0 },
  { "Jamora", 0.0 },
  { "Jamore", 1.0 },
  { "Jamori", 1.0 },
  { "Jamoria", 0.0 },
  { "Jamorian", 1.0 },
  { "Jamorie", 1.0 },
  { "Jamorion", 1.0 },
  { "Jamorris", 1.0 },
  { "Jams", 1.0 },
  { "Jamse", 1.0 },
  { "Jamson", 1.0 },
  { "Jamual", 1.0 },
  { "Jamuel", 1.0 },
  { "Jamus", 1.0 },
  { "Jamy", 0.3377425 },
  { "Jamya", 0.0 },
  { "Jamyah", 0.0 },
  { "Jamye", 0.014167651 },
  { "Jamyia", 0.0 },
  { "Jamyiah", 0.0 },
  { "Jamyis", 1.0 },
  { "Jamykal", 1.0 },
  { "Jamyl", 1.0 },
  { "Jamyla", 0.0 },
  { "Jamylah", 0.0 },
  { "Jamyle", 1.0 },
  { "Jamyleth", 0.0 },
  { "Jamylex", 0.0 },
  { "Jamyn", 1.0 },
  { "Jamyr", 1.0 },
  { "Jamyra", 0.0 },
  { "Jamyrah", 0.0 },
  { "Jamyre", 1.0 },
  { "Jamyri", 0.26086956 },
  { "Jamyria", 0.0 },
  { "Jamyriah", 0.0 },
  { "Jamyrie", 0.0 },
  { "Jamyrion", 1.0 },
  { "Jamyron", 1.0 },
  { "Jamyson", 1.0 },
  { "Jan", 0.28555512 },
  { "Jana", 0.0008825926 },
  { "Janaa", 0.0 },
  { "Janaan", 0.0 },
  { "Janace", 0.0 },
  { "Janacia", 0.0 },
  { "Janada", 0.0 },
  { "Janadia", 0.0 },
  { "Janae", 0.00036460866 },
  { "Janaea", 0.0 },
  { "Janaee", 0.0 },
  { "Janaeh", 0.0 },
  { "Janael", 0.0 },
  { "Janaesha", 0.0 },
  { "Janaeya", 0.0 },
  { "Janaeyah", 0.0 },
  { "Janah", 0.0 },
  { "Janahi", 0.0 },
  { "Janahla", 0.0 },
  { "Janai", 0.030706622 },
  { "Janaia", 0.0 },
  { "Janaiah", 0.0 },
  { "Janaih", 0.0 },
  { "Janaii", 0.0 },
  { "Janaija", 0.0 },
  { "Janaijah", 0.0 },
  { "Janaila", 0.0 },
  { "Janaina", 0.0 },
  { "Janaira", 0.0 },
  { "Janais", 0.0 },
  { "Janaisa", 0.0 },
  { "Janaisha", 0.0 },
  { "Janaisia", 0.0 },
  { "Janaiya", 0.0 },
  { "Janaiyah", 0.0 },
  { "Janaja", 0.0 },
  { "Janajah", 0.0 },
  { "Janak", 1.0 },
  { "Janaka", 1.0 },
  { "Janaki", 0.0 },
  { "Janal", 0.067567565 },
  { "Janala", 0.0 },
  { "Janalee", 0.0 },
  { "Janalene", 0.0 },
  { "Janalis", 0.0 },
  { "Janalise", 0.0 },
  { "Janaliz", 0.0 },
  { "Janalle", 0.0 },
  { "Janalyn", 0.0 },
  { "Janalynn", 0.0 },
  { "Janalyse", 0.0 },
  { "Janan", 0.015384615 },
  { "Janani", 0.0 },
  { "Janann", 0.0 },
  { "Jananne", 0.0 },
  { "Janara", 0.0 },
  { "Janard", 1.0 },
  { "Janari", 0.6631579 },
  { "Janaria", 0.0 },
  { "Janariah", 0.0 },
  { "Janaris", 1.0 },
  { "Janarius", 1.0 },
  { "Janas", 0.10869565 },
  { "Janascia", 0.0 },
  { "Janasha", 0.0 },
  { "Janashia", 0.0 },
  { "Janasia", 0.0 },
  { "Janat", 0.0 },
  { "Janath", 0.0 },
  { "Janathan", 1.0 },
  { "Janathean", 1.0 },
  { "Janautica", 0.0 },
  { "Janavi", 0.0 },
  { "Janavia", 0.0 },
  { "Janay", 0.0 },
  { "Janaya", 0.0 },
  { "Janayah", 0.0 },
  { "Janaye", 0.0 },
  { "Janayia", 0.0 },
  { "Janayiah", 0.0 },
  { "Janayja", 0.0 },
  { "Janayla", 0.0 },
  { "Janayra", 0.0 },
  { "Janaysha", 0.0 },
  { "Janayshia", 0.0 },
  { "Janaysia", 0.0 },
  { "Janazia", 0.0 },
  { "Jancarlo", 1.0 },
  { "Jancarlos", 1.0 },
  { "Jance", 0.9350649 },
  { "Jancey", 0.0 },
  { "Janci", 0.0 },
  { "Jancie", 0.0 },
  { "Jancy", 0.015723271 },
  { "Janda", 0.0 },
  { "Jande", 0.0 },
  { "Jandee", 0.0 },
  { "Jandel", 0.9800797 },
  { "Jandell", 0.78571427 },
  { "Jandery", 0.0 },
  { "Jandi", 0.0 },
  { "Jandiel", 1.0 },
  { "Jandra", 0.0 },
  { "Jandre", 1.0 },
  { "Jandriel", 1.0 },
  { "Jandy", 0.083333336 },
  { "Jane", 0.0030617286 },
  { "Janea", 0.0 },
  { "Janeah", 0.0 },
  { "Janeal", 0.0 },
  { "Janean", 0.0 },
  { "Janeane", 0.0 },
  { "Janeann", 0.0 },
  { "Janeanne", 0.0 },
  { "Janear", 0.0 },
  { "Janece", 0.0 },
  { "Janecia", 0.0 },
  { "Janee", 0.0018650917 },
  { "Janeece", 0.0 },
  { "Janeeka", 0.0 },
  { "Janeel", 0.0 },
  { "Janeen", 0.0 },
  { "Janeene", 0.0 },
  { "Janeesa", 0.0 },
  { "Janeese", 0.0 },
  { "Janeesha", 0.0 },
  { "Janeeva", 0.0 },
  { "Janei", 0.0 },
  { "Janeia", 0.0 },
  { "Janeiah", 0.0 },
  { "Janeice", 0.0 },
  { "Janeicia", 0.0 },
  { "Janeida", 0.0 },
  { "Janeidy", 0.0 },
  { "Janeigh", 0.0 },
  { "Janeika", 0.0 },
  { "Janeil", 0.09583333 },
  { "Janeily", 0.0 },
  { "Janeira", 0.0 },
  { "Janeiro", 1.0 },
  { "Janeiry", 0.0 },
  { "Janeisa", 0.0 },
  { "Janeise", 0.0 },
  { "Janeisha", 0.0 },
  { "Janeisy", 0.0 },
  { "Janeiya", 0.0 },
  { "Janek", 1.0 },
  { "Janeka", 0.0 },
  { "Janekia", 0.0 },
  { "Janel", 0.0026967807 },
  { "Janela", 0.0 },
  { "Janelda", 0.0 },
  { "Janele", 0.0 },
  { "Janelee", 0.0 },
  { "Janeli", 0.0 },
  { "Janelie", 0.0 },
  { "Janelis", 0.0 },
  { "Janelise", 0.0 },
  { "Janelisse", 0.0 },
  { "Janeliz", 0.0 },
  { "Janell", 0.0014854043 },
  { "Janella", 0.0 },
  { "Janelle", 0.0016286029 },
  { "Janellen", 0.0 },
  { "Janelli", 0.0 },
  { "Janellie", 0.0 },
  { "Janelly", 0.0 },
  { "Janellys", 0.0 },
  { "Janely", 0.0 },
  { "Janelyn", 0.0 },
  { "Janelys", 0.0 },
  { "Janelyz", 0.0 },
  { "Janemarie", 0.0 },
  { "Janene", 0.0 },
  { "Janenne", 0.0 },
  { "Janequa", 0.0 },
  { "Janera", 0.0 },
  { "Janeria", 0.0 },
  { "Janerica", 0.0 },
  { "Janes", 1.0 },
  { "Janesa", 0.0 },
  { "Janese", 0.0 },
  { "Janesha", 0.0 },
  { "Janeshia", 0.0 },
  { "Janesia", 0.0 },
  { "Janess", 0.0 },
  { "Janessa", 0.0 },
  { "Janessah", 0.0 },
  { "Janesse", 0.0 },
  { "Janessia", 0.0 },
  { "Janet", 0.0027239546 },
  { "Janeta", 0.0 },
  { "Janete", 0.0 },
  { "Janeth", 0.0 },
  { "Janetha", 0.0 },
  { "Janetlee", 0.0 },
  { "Janetra", 0.0 },
  { "Janett", 0.0 },
  { "Janetta", 0.0 },
  { "Janette", 0.0013169447 },
  { "Janetzi", 0.0 },
  { "Janetzy", 0.0 },
  { "Janeva", 0.0 },
  { "Janey", 0.0 },
  { "Janeya", 0.0 },
  { "Janeyah", 0.0 },
  { "Jangelo", 1.0 },
  { "Janhavi", 0.0 },
  { "Jani", 0.04814138 },
  { "Jania", 0.0 },
  { "Janiah", 0.0 },
  { "Janiaha", 0.0 },
  { "Janiaya", 0.0 },
  { "Janiayah", 0.0 },
  { "Janic", 0.0 },
  { "Janica", 0.0 },
  { "Janice", 0.0028479937 },
  { "Janicia", 0.0 },
  { "Janick", 1.0 },
  { "Janicka", 0.0 },
  { "Janie", 0.0036978978 },
  { "Janiece", 0.0 },
  { "Janiecia", 0.0 },
  { "Janieka", 0.0 },
  { "Janiel", 0.7496977 },
  { "Janielis", 0.0 },
  { "Janieliz", 0.0 },
  { "Janielle", 0.0 },
  { "Janiely", 0.0 },
  { "Janielys", 0.0 },
  { "Janien", 0.0 },
  { "Janiene", 0.0 },
  { "Janier", 1.0 },
  { "Janiera", 0.0 },
  { "Janiese", 0.0 },
  { "Janiesha", 0.0 },
  { "Janieya", 0.0 },
  { "Janifer", 0.0 },
  { "Janiha", 0.0 },
  { "Janihya", 0.0 },
  { "Janii", 0.0 },
  { "Janiia", 0.0 },
  { "Janijah", 0.0 },
  { "Janika", 0.0 },
  { "Janikqua", 0.0 },
  { "Janil", 0.0 },
  { "Janila", 0.0 },
  { "Janilah", 0.0 },
  { "Janilee", 0.0 },
  { "Janill", 0.0 },
  { "Janille", 0.0 },
  { "Janilya", 0.0 },
  { "Janilyn", 0.0 },
  { "Janin", 0.0 },
  { "Janina", 0.0 },
  { "Janine", 0.0003306779 },
  { "Janinne", 0.0 },
  { "Janiqua", 0.0 },
  { "Janique", 0.0 },
  { "Janira", 0.0 },
  { "Janirah", 0.0 },
  { "Janiris", 0.0 },
  { "Janis", 0.003334902 },
  { "Janisa", 0.0 },
  { "Janise", 0.0 },
  { "Janisha", 0.0 },
  { "Janishia", 0.0 },
  { "Janisia", 0.0 },
  { "Janissa", 0.0 },
  { "Janisse", 0.0 },
  { "Janit", 0.0 },
  { "Janita", 0.0 },
  { "Janith", 0.0 },
  { "Janitha", 0.0 },
  { "Janitra", 0.0 },
  { "Janitta", 0.0 },
  { "Janitza", 0.0 },
  { "Janitzy", 0.0 },
  { "Janivea", 0.0 },
  { "Janiya", 0.00061050063 },
  { "Janiyaa", 0.0 },
  { "Janiyah", 0.0 },
  { "Janiyan", 0.0 },
  { "Janiyha", 0.0 },
  { "Janiyia", 0.0 },
  { "Janiyla", 0.0 },
  { "Janiylah", 0.0 },
  { "Janiyyah", 0.0 },
  { "Janiza", 0.0 },
  { "Jankarlo", 1.0 },
  { "Janki", 0.0 },
  { "Janko", 1.0 },
  { "Janluca", 1.0 },
  { "Janlucas", 1.0 },
  { "Janluis", 1.0 },
  { "Janlyn", 0.0 },
  { "Janmarco", 1.0 },
  { "Janmarcos", 1.0 },
  { "Janmarie", 0.0 },
  { "Janmichael", 1.0 },
  { "Jann", 0.05856451 },
  { "Janna", 0.0 },
  { "Jannae", 0.0 },
  { "Jannah", 0.0 },
  { "Jannai", 0.0 },
  { "Jannali", 0.0 },
  { "Jannalyn", 0.0 },
  { "Jannan", 0.0 },
  { "Jannat", 0.0 },
  { "Jannatul", 0.0 },
  { "Jannay", 0.0 },
  { "Janne", 0.0 },
  { "Jannea", 0.0 },
  { "Jannean", 0.0 },
  { "Jannel", 0.0 },
  { "Janneli", 0.0 },
  { "Jannell", 0.0 },
  { "Jannelle", 0.0 },
  { "Jannelly", 0.0 },
  { "Jannely", 0.0 },
  { "Jannene", 0.0 },
  { "Jannesa", 0.0 },
  { "Jannessa", 0.0 },
  { "Jannet", 0.0 },
  { "Jannete", 0.0 },
  { "Janneth", 0.0 },
  { "Jannett", 0.0 },
  { "Jannetta", 0.0 },
  { "Jannette", 0.0 },
  { "Janney", 0.0 },
  { "Janni", 0.0 },
  { "Jannia", 0.0 },
  { "Janniah", 0.0 },
  { "Jannica", 0.0 },
  { "Jannice", 0.0 },
  { "Jannie", 0.0018920198 },
  { "Janniel", 1.0 },
  { "Jannifer", 0.0 },
  { "Jannik", 1.0 },
  { "Jannika", 0.0 },
  { "Jannike", 0.0 },
  { "Jannina", 0.0 },
  { "Jannine", 0.0 },
  { "Jannis", 0.0 },
  { "Jannita", 0.0 },
  { "Janniyah", 0.0 },
  { "Jannuel", 1.0 },
  { "Janny", 0.0 },
  { "Janoah", 1.0 },
  { "Janon", 0.0 },
  { "Janora", 0.0 },
  { "Janorris", 1.0 },
  { "Janos", 1.0 },
  { "Janovia", 0.0 },
  { "Jansel", 1.0 },
  { "Jansen", 0.8903127 },
  { "Jansiel", 1.0 },
  { "Janson", 1.0 },
  { "Janssen", 1.0 },
  { "Jansyn", 0.16666667 },
  { "Jante", 0.74545455 },
  { "Jantel", 0.0 },
  { "Janthony", 1.0 },
  { "Jantsen", 1.0 },
  { "Jantz", 1.0 },
  { "Jantzen", 0.97862595 },
  { "Januari", 0.0 },
  { "Januarie", 0.0 },
  { "January", 0.0 },
  { "Januel", 1.0 },
  { "Januell", 1.0 },
  { "Januelle", 1.0 },
  { "Januita", 0.0 },
  { "Janus", 0.19620253 },
  { "Janusz", 1.0 },
  { "Janvi", 0.0 },
  { "Janvier", 0.53125 },
  { "Janxiel", 1.0 },
  { "Jany", 0.0 },
  { "Janya", 0.0 },
  { "Janyah", 0.0 },
  { "Janyce", 0.0 },
  { "Janye", 0.0 },
  { "Janyha", 0.0 },
  { "Janyhia", 0.0 },
  { "Janyi", 0.0 },
  { "Janyia", 0.0 },
  { "Janyiah", 0.0 },
  { "Janyl", 0.0 },
  { "Janyla", 0.0 },
  { "Janylah", 0.0 },
  { "Janyll", 0.0 },
  { "Janyne", 0.0 },
  { "Janyra", 0.0 },
  { "Janyria", 0.0 },
  { "Janyriah", 0.0 },
  { "Janys", 0.0 },
  { "Janyse", 0.0 },
  { "Janyssa", 0.0 },
  { "Janyth", 0.0 },
  { "Janyus", 1.0 },
  { "Janzel", 1.0 },
  { "Janzen", 1.0 },
  { "Janziel", 1.0 },
  { "Jaona", 0.0 },
  { "Jaoni", 0.0 },
  { "Jaonna", 0.0 },
  { "Jaonte", 1.0 },
  { "Jaosn", 1.0 },
  { "Jap", 1.0 },
  { "Japaul", 1.0 },
  { "Japera", 0.0 },
  { "Japeth", 1.0 },
  { "Japhet", 1.0 },
  { "Japheth", 1.0 },
  { "Japhy", 1.0 },
  { "Japji", 0.0 },
  { "Japleen", 0.0 },
  { "Japneet", 0.0 },
  { "Japnoor", 0.0 },
  { "Japonica", 0.0 },
  { "Japree", 1.0 },
  { "Japria", 0.0 },
  { "Japrice", 0.0 },
  { "Jaqai", 1.0 },
  { "Jaqari", 1.0 },
  { "Jaqaun", 1.0 },
  { "Jaqavious", 1.0 },
  { "Jaqlyn", 0.0 },
  { "Jaqua", 0.6923077 },
  { "Jaquae", 0.85 },
  { "Jaquai", 1.0 },
  { "Jaquail", 1.0 },
  { "Jaquain", 1.0 },
  { "Jaquaisha", 0.0 },
  { "Jaqual", 1.0 },
  { "Jaquala", 0.0 },
  { "Jaquale", 1.0 },
  { "Jaqualen", 1.0 },
  { "Jaqualin", 1.0 },
  { "Jaqualine", 0.0 },
  { "Jaqualla", 0.0 },
  { "Jaqualon", 1.0 },
  { "Jaqualyn", 0.89719623 },
  { "Jaquan", 0.9941503 },
  { "Jaquana", 0.0 },
  { "Jaquanda", 0.0 },
  { "Jaquandra", 0.0 },
  { "Jaquandre", 1.0 },
  { "Jaquane", 1.0 },
  { "Jaquann", 1.0 },
  { "Jaquanna", 0.0 },
  { "Jaquantae", 1.0 },
  { "Jaquante", 1.0 },
  { "Jaquar", 1.0 },
  { "Jaquari", 1.0 },
  { "Jaquarion", 1.0 },
  { "Jaquarious", 1.0 },
  { "Jaquaris", 1.0 },
  { "Jaquarius", 1.0 },
  { "Jaquarrius", 1.0 },
  { "Jaquasha", 0.0 },
  { "Jaquashia", 0.0 },
  { "Jaquasia", 0.0 },
  { "Jaquata", 0.0 },
  { "Jaquaun", 1.0 },
  { "Jaquaveon", 1.0 },
  { "Jaquavia", 0.0 },
  { "Jaquavian", 1.0 },
  { "Jaquavien", 1.0 },
  { "Jaquavion", 1.0 },
  { "Jaquavious", 1.0 },
  { "Jaquavis", 1.0 },
  { "Jaquavius", 1.0 },
  { "Jaquawn", 1.0 },
  { "Jaquay", 0.7067669 },
  { "Jaquaya", 0.0 },
  { "Jaquayla", 0.0 },
  { "Jaquaylin", 1.0 },
  { "Jaquaylon", 1.0 },
  { "Jaquayvion", 1.0 },
  { "Jaquaze", 1.0 },
  { "Jaque", 0.8544304 },
  { "Jaqueal", 1.0 },
  { "Jaqueisha", 0.0 },
  { "Jaquel", 0.75449103 },
  { "Jaquela", 0.0 },
  { "Jaquelene", 0.0 },
  { "Jaquelin", 0.0 },
  { "Jaquelina", 0.0 },
  { "Jaqueline", 0.0015517474 },
  { "Jaquelinee", 0.0 },
  { "Jaquelinne", 0.0 },
  { "Jaquell", 0.978355 },
  { "Jaquella", 0.0 },
  { "Jaquelle", 1.0 },
  { "Jaquelline", 0.0 },
  { "Jaquelyn", 0.0 },
  { "Jaquelyne", 0.0 },
  { "Jaquelynn", 0.0 },
  { "Jaqueria", 0.0 },
  { "Jaques", 0.98392284 },
  { "Jaquese", 0.7181818 },
  { "Jaquesha", 0.0 },
  { "Jaquest", 1.0 },
  { "Jaqueta", 0.0 },
  { "Jaquetta", 0.0 },
  { "Jaquette", 0.0 },
  { "Jaquez", 1.0 },
  { "Jaqueze", 1.0 },
  { "Jaqui", 0.575 },
  { "Jaquia", 0.0 },
  { "Jaquice", 0.0 },
  { "Jaquil", 1.0 },
  { "Jaquila", 0.0 },
  { "Jaquill", 1.0 },
  { "Jaquilla", 0.0 },
  { "Jaquille", 1.0 },
  { "Jaquilyn", 0.0 },
  { "Jaquin", 0.973822 },
  { "Jaquincy", 1.0 },
  { "Jaquinn", 1.0 },
  { "Jaquinta", 0.0 },
  { "Jaquinton", 1.0 },
  { "Jaquira", 0.0 },
  { "Jaquis", 0.9590164 },
  { "Jaquise", 0.8312236 },
  { "Jaquisha", 0.0 },
  { "Jaquita", 0.0 },
  { "Jaquitta", 0.0 },
  { "Jaquline", 0.0 },
  { "Jaqulyn", 0.0 },
  { "Jaqulynn", 0.0 },
  { "Jaquoia", 0.0 },
  { "Jaquon", 1.0 },
  { "Jaquori", 1.0 },
  { "Jaquory", 1.0 },
  { "Jaquoya", 0.0 },
  { "Jaquwan", 1.0 },
  { "Jaqwan", 1.0 },
  { "Jaqwon", 1.0 },
  { "Jara", 0.0 },
  { "Jaraad", 1.0 },
  { "Jarad", 1.0 },
  { "Jarade", 1.0 },
  { "Jarae", 0.23396227 },
  { "Jarael", 1.0 },
  { "Jarah", 0.02606635 },
  { "Jarai", 1.0 },
  { "Jarail", 1.0 },
  { "Jarald", 1.0 },
  { "Jaramiah", 1.0 },
  { "Jaramie", 1.0 },
  { "Jaramy", 1.0 },
  { "Jaran", 1.0 },
  { "Jarard", 1.0 },
  { "Jaray", 1.0 },
  { "Jaraya", 0.0 },
  { "Jarayah", 0.0 },
  { "Jard", 1.0 },
  { "Jardan", 1.0 },
  { "Jarden", 1.0 },
  { "Jardin", 0.41666666 },
  { "Jardon", 1.0 },
  { "Jardyn", 0.0 },
  { "Jare", 1.0 },
  { "Jareal", 1.0 },
  { "Jareb", 1.0 },
  { "Jared", 0.9961466 },
  { "Jaredd", 1.0 },
  { "Jaree", 0.46464646 },
  { "Jareem", 1.0 },
  { "Jareese", 1.0 },
  { "Jarek", 1.0 },
  { "Jareka", 0.0 },
  { "Jarel", 0.9943128 },
  { "Jareli", 0.0 },
  { "Jarelin", 0.0 },
  { "Jarelis", 0.0 },
  { "Jareliz", 0.0 },
  { "Jarell", 1.0 },
  { "Jarelle", 1.0 },
  { "Jarelly", 0.0 },
  { "Jarely", 0.0 },
  { "Jarelyn", 0.0 },
  { "Jarelys", 0.0 },
  { "Jarem", 1.0 },
  { "Jaremiah", 1.0 },
  { "Jaremy", 1.0 },
  { "Jaren", 0.9496918 },
  { "Jarena", 0.0 },
  { "Jareny", 0.0 },
  { "Jareo", 1.0 },
  { "Jaresha", 0.0 },
  { "Jaret", 0.992305 },
  { "Jareth", 0.9789196 },
  { "Jarethzy", 0.0 },
  { "Jaretsi", 0.0 },
  { "Jaretssi", 0.0 },
  { "Jaretsy", 0.0 },
  { "Jarett", 1.0 },
  { "Jaretta", 0.0 },
  { "Jarette", 1.0 },
  { "Jaretzi", 0.0 },
  { "Jaretzy", 0.0 },
  { "Jarexi", 0.0 },
  { "Jarexy", 0.0 },
  { "Jarez", 1.0 },
  { "Jarhett", 1.0 },
  { "Jarhia", 0.0 },
  { "Jarhonda", 0.0 },
  { "Jari", 0.36935166 },
  { "Jaria", 0.0 },
  { "Jariah", 0.15996504 },
  { "Jarian", 1.0 },
  { "Jariana", 0.0 },
  { "Jarianna", 0.0 },
  { "Jarib", 1.0 },
  { "Jaric", 1.0 },
  { "Jarica", 0.0 },
  { "Jarick", 1.0 },
  { "Jaricka", 0.0 },
  { "Jarico", 1.0 },
  { "Jarid", 1.0 },
  { "Jarie", 0.3529412 },
  { "Jariel", 0.97522235 },
  { "Jarielis", 0.0 },
  { "Jarieliz", 0.0 },
  { "Jariell", 1.0 },
  { "Jarielle", 0.0 },
  { "Jariely", 0.0 },
  { "Jarielys", 0.0 },
  { "Jarien", 1.0 },
  { "Jarif", 1.0 },
  { "Jarik", 1.0 },
  { "Jarika", 0.0 },
  { "Jarilyn", 0.0 },
  { "Jarimiah", 1.0 },
  { "Jarin", 0.9674235 },
  { "Jarina", 0.0 },
  { "Jario", 1.0 },
  { "Jarion", 1.0 },
  { "Jarious", 1.0 },
  { "Jarique", 1.0 },
  { "Jaris", 0.9268293 },
  { "Jarisa", 0.0 },
  { "Jarison", 1.0 },
  { "Jarissa", 0.0 },
  { "Jarit", 1.0 },
  { "Jarita", 0.0 },
  { "Jaritsa", 0.0 },
  { "Jaritza", 0.0 },
  { "Jaritzi", 0.0 },
  { "Jaritzy", 0.0 },
  { "Jarius", 1.0 },
  { "Jariya", 0.0 },
  { "Jariyah", 0.014025246 },
  { "Jarl", 1.0 },
  { "Jarlene", 0.0 },
  { "Jarlin", 0.0 },
  { "Jarline", 0.0 },
  { "Jarlyn", 0.0 },
  { "Jarma", 0.0 },
  { "Jarmain", 1.0 },
  { "Jarmaine", 1.0 },
  { "Jarmal", 1.0 },
  { "Jarmall", 1.0 },
  { "Jarman", 1.0 },
  { "Jarmani", 1.0 },
  { "Jarmar", 1.0 },
  { "Jarmarcus", 1.0 },
  { "Jarmarion", 1.0 },
  { "Jarmaris", 1.0 },
  { "Jarmel", 1.0 },
  { "Jarmell", 1.0 },
  { "Jarmen", 1.0 },
  { "Jarmila", 0.0 },
  { "Jarmon", 1.0 },
  { "Jarnae", 0.0 },
  { "Jarnell", 1.0 },
  { "Jaro", 1.0 },
  { "Jarod", 0.9995709 },
  { "Jarodd", 1.0 },
  { "Jarol", 1.0 },
  { "Jarold", 1.0 },
  { "Jarom", 1.0 },
  { "Jarome", 1.0 },
  { "Jaromir", 1.0 },
  { "Jaron", 0.99549425 },
  { "Jaronda", 0.0 },
  { "Jarone", 1.0 },
  { "Jaronn", 1.0 },
  { "Jaronte", 1.0 },
  { "Jaroslav", 1.0 },
  { "Jaroslaw", 1.0 },
  { "Jarquavious", 1.0 },
  { "Jarquavius", 1.0 },
  { "Jarques", 1.0 },
  { "Jarquez", 1.0 },
  { "Jarquis", 1.0 },
  { "Jarquise", 1.0 },
  { "Jarrad", 1.0 },
  { "Jarrah", 0.078125 },
  { "Jarran", 1.0 },
  { "Jarrard", 1.0 },
  { "Jarratt", 1.0 },
  { "Jarreau", 1.0 },
  { "Jarred", 0.9993882 },
  { "Jarrek", 1.0 },
  { "Jarrel", 1.0 },
  { "Jarrell", 0.9945585 },
  { "Jarrelle", 1.0 },
  { "Jarren", 0.9950544 },
  { "Jarret", 1.0 },
  { "Jarreth", 1.0 },
  { "Jarrett", 0.9964738 },
  { "Jarrette", 1.0 },
  { "Jarriah", 0.0 },
  { "Jarrian", 1.0 },
  { "Jarrica", 0.0 },
  { "Jarrick", 1.0 },
  { "Jarrid", 1.0 },
  { "Jarriel", 1.0 },
  { "Jarriet", 1.0 },
  { "Jarrin", 1.0 },
  { "Jarrion", 1.0 },
  { "Jarris", 1.0 },
  { "Jarrison", 1.0 },
  { "Jarrit", 1.0 },
  { "Jarritt", 1.0 },
  { "Jarrius", 1.0 },
  { "Jarrod", 0.9974059 },
  { "Jarrold", 1.0 },
  { "Jarron", 1.0 },
  { "Jarrot", 1.0 },
  { "Jarrott", 1.0 },
  { "Jarry", 1.0 },
  { "Jarryd", 1.0 },
  { "Jarryl", 1.0 },
  { "Jarryn", 1.0 },
  { "Jartavious", 1.0 },
  { "Jartavis", 1.0 },
  { "Jartavius", 1.0 },
  { "Jaruis", 1.0 },
  { "Jarule", 1.0 },
  { "Jarus", 1.0 },
  { "Jarvais", 1.0 },
  { "Jarvaris", 1.0 },
  { "Jarvas", 1.0 },
  { "Jarvell", 1.0 },
  { "Jarvez", 1.0 },
  { "Jarvie", 1.0 },
  { "Jarvin", 1.0 },
  { "Jarvion", 1.0 },
  { "Jarvis", 0.9927651 },
  { "Jarvius", 1.0 },
  { "Jarvon", 1.0 },
  { "Jarvous", 1.0 },
  { "Jarvus", 1.0 },
  { "Jary", 1.0 },
  { "Jarya", 0.0 },
  { "Jaryah", 0.0 },
  { "Jaryan", 1.0 },
  { "Jaryd", 1.0 },
  { "Jaryel", 1.0 },
  { "Jaryia", 0.0 },
  { "Jaryiah", 0.0 },
  { "Jaryl", 1.0 },
  { "Jaryn", 0.61316395 },
  { "Jas", 0.8898305 },
  { "Jasa", 0.0 },
  { "Jasaan", 1.0 },
  { "Jasabelle", 0.0 },
  { "Jasahn", 1.0 },
  { "Jasai", 0.98214287 },
  { "Jasaiah", 0.9029126 },
  { "Jasalin", 0.0 },
  { "Jasalyn", 0.0 },
  { "Jasalynn", 0.0 },
  { "Jasamine", 0.0 },
  { "Jasan", 1.0 },
  { "Jasana", 0.0 },
  { "Jasandra", 0.0 },
  { "Jasani", 0.2793296 },
  { "Jasara", 0.0 },
  { "Jasarah", 0.0 },
  { "Jasaun", 1.0 },
  { "Jasavion", 1.0 },
  { "Jasaya", 0.0 },
  { "Jascha", 1.0 },
  { "Jasdeep", 0.8171642 },
  { "Jase", 0.9940483 },
  { "Jasean", 1.0 },
  { "Jasear", 1.0 },
  { "Jasee", 0.0 },
  { "Jaseem", 1.0 },
  { "Jaseena", 0.0 },
  { "Jaseer", 1.0 },
  { "Jaseir", 1.0 },
  { "Jasek", 1.0 },
  { "Jasel", 0.0 },
  { "Jaselin", 0.0 },
  { "Jasella", 0.0 },
  { "Jaselle", 0.0 },
  { "Jaselyn", 0.0 },
  { "Jaselynn", 0.0 },
  { "Jasem", 1.0 },
  { "Jasemine", 0.0 },
  { "Jasen", 1.0 },
  { "Jasenia", 0.0 },
  { "Jasenya", 0.0 },
  { "Jaseon", 1.0 },
  { "Jaseph", 1.0 },
  { "Jaser", 1.0 },
  { "Jasera", 0.0 },
  { "Jasere", 1.0 },
  { "Jasey", 0.009505703 },
  { "Jash", 1.0 },
  { "Jasha", 0.0 },
  { "Jashad", 1.0 },
  { "Jashae", 0.0 },
  { "Jashala", 0.0 },
  { "Jashan", 0.9766355 },
  { "Jashana", 0.0 },
  { "Jashandeep", 1.0 },
  { "Jashanna", 0.0 },
  { "Jashanpreet", 1.0 },
  { "Jashante", 0.0 },
  { "Jashanti", 0.0 },
  { "Jashara", 0.0 },
  { "Jashaun", 1.0 },
  { "Jashauna", 0.0 },
  { "Jashawn", 0.9834791 },
  { "Jashawna", 0.0 },
  { "Jashay", 0.0 },
  { "Jashayla", 0.0 },
  { "Jashea", 0.0 },
  { "Jasheem", 1.0 },
  { "Jashelle", 0.0 },
  { "Jasher", 1.0 },
  { "Jashia", 0.0 },
  { "Jashira", 0.0 },
  { "Jashiya", 0.0 },
  { "Jashiyah", 0.0 },
  { "Jashley", 0.0 },
  { "Jashod", 1.0 },
  { "Jashon", 0.99298245 },
  { "Jashona", 0.0 },
  { "Jashonda", 0.0 },
  { "Jashone", 1.0 },
  { "Jashonna", 0.0 },
  { "Jashua", 1.0 },
  { "Jashun", 1.0 },
  { "Jashya", 0.0 },
  { "Jashyia", 0.0 },
  { "Jashyra", 0.0 },
  { "Jasi", 0.36 },
  { "Jasia", 0.012605042 },
  { "Jasiah", 0.8963037 },
  { "Jasian", 1.0 },
  { "Jasiana", 0.0 },
  { "Jasianna", 0.0 },
  { "Jasias", 1.0 },
  { "Jasibe", 0.0 },
  { "Jasicianna", 0.0 },
  { "Jasie", 0.0 },
  { "Jasiel", 0.92391306 },
  { "Jasielle", 0.0 },
  { "Jasier", 1.0 },
  { "Jasik", 1.0 },
  { "Jasilyn", 0.0 },
  { "Jasilynn", 0.0 },
  { "Jasim", 1.0 },
  { "Jasiman", 0.0 },
  { "Jasime", 0.0 },
  { "Jasimen", 0.0 },
  { "Jasimine", 0.0 },
  { "Jasin", 1.0 },
  { "Jasine", 0.0 },
  { "Jasinto", 1.0 },
  { "Jasion", 1.0 },
  { "Jasir", 1.0 },
  { "Jasira", 0.0 },
  { "Jasirah", 0.0 },
  { "Jasire", 1.0 },
  { "Jasiri", 0.75757575 },
  { "Jasiya", 0.039215688 },
  { "Jasiyah", 0.38086957 },
  { "Jasjeet", 1.0 },
  { "Jaskaran", 1.0 },
  { "Jaskarn", 1.0 },
  { "Jaskiran", 0.0 },
  { "Jaskirat", 0.7016129 },
  { "Jaslean", 0.0 },
  { "Jasleen", 0.0 },
  { "Jasleene", 0.0 },
  { "Jaslen", 0.0 },
  { "Jaslena", 0.0 },
  { "Jaslene", 0.0 },
  { "Jaslenne", 0.0 },
  { "Jasley", 0.0 },
  { "Jaslin", 0.0 },
  { "Jasline", 0.0 },
  { "Jaslinn", 0.0 },
  { "Jasly", 0.0 },
  { "Jaslyn", 0.0 },
  { "Jaslyne", 0.0 },
  { "Jaslynn", 0.0 },
  { "Jaslynne", 0.0 },
  { "Jasma", 0.0 },
  { "Jasmain", 0.0 },
  { "Jasmaine", 0.0072815535 },
  { "Jasman", 0.15081967 },
  { "Jasmane", 0.0 },
  { "Jasmany", 1.0 },
  { "Jasmari", 0.0 },
  { "Jasmarie", 0.0 },
  { "Jasmary", 0.0 },
  { "Jasmeen", 0.0 },
  { "Jasmeet", 0.2549801 },
  { "Jasmeh", 0.0 },
  { "Jasmely", 0.0 },
  { "Jasmen", 0.017828202 },
  { "Jasmene", 0.0 },
  { "Jasmere", 0.0 },
  { "Jasmia", 0.0 },
  { "Jasmie", 0.0 },
  { "Jasmien", 0.0 },
  { "Jasmijn", 0.0 },
  { "Jasmil", 0.0 },
  { "Jasmin", 0.013279065 },
  { "Jasmina", 0.0 },
  { "Jasminda", 0.0 },
  { "Jasmine", 0.005181816 },
  { "Jasminemarie", 0.0 },
  { "Jasminerose", 0.0 },
  { "Jasminn", 0.0 },
  { "Jasminne", 0.0 },
  { "Jasmir", 0.7692308 },
  { "Jasmire", 0.0 },
  { "Jasmon", 0.36633664 },
  { "Jasmond", 0.6532258 },
  { "Jasmone", 0.0 },
  { "Jasmonique", 0.0 },
  { "Jasmyn", 0.0 },
  { "Jasmyne", 0.0 },
  { "Jasmynn", 0.0 },
  { "Jasmynne", 0.0 },
  { "Jasn", 1.0 },
  { "Jasna", 0.0 },
  { "Jasneet", 0.0 },
  { "Jasniel", 1.0 },
  { "Jasnoor", 0.14141414 },
  { "Jaso", 1.0 },
  { "Jasom", 1.0 },
  { "Jason", 0.9951328 },
  { "Jasona", 0.0 },
  { "Jasonanthony", 1.0 },
  { "Jasonjr", 1.0 },
  { "Jasonlee", 1.0 },
  { "Jasonn", 1.0 },
  { "Jasonna", 0.0 },
  { "Jasonpaul", 1.0 },
  { "Jasontyler", 1.0 },
  { "Jaspal", 1.0 },
  { "Jaspar", 1.0 },
  { "Jaspen", 1.0 },
  { "Jasper", 0.98162603 },
  { "Jaspreet", 0.3956044 },
  { "Jaspyr", 1.0 },
  { "Jasraj", 1.0 },
  { "Jasreen", 0.0 },
  { "Jasreet", 0.0 },
  { "Jassan", 1.0 },
  { "Jassar", 1.0 },
  { "Jassel", 0.0 },
  { "Jasselle", 0.0 },
  { "Jassem", 1.0 },
  { "Jassen", 1.0 },
  { "Jasser", 1.0 },
  { "Jassiah", 0.9201878 },
  { "Jassica", 0.0 },
  { "Jassidy", 0.0 },
  { "Jassie", 0.0 },
  { "Jassiel", 0.98673743 },
  { "Jassiem", 1.0 },
  { "Jassim", 1.0 },
  { "Jassir", 1.0 },
  { "Jasslyn", 0.0 },
  { "Jasslynn", 0.0 },
  { "Jassmen", 0.0 },
  { "Jassmin", 0.0 },
  { "Jassmine", 0.0 },
  { "Jassmyn", 0.0 },
  { "Jasson", 1.0 },
  { "Jasten", 1.0 },
  { "Jastin", 0.93607306 },
  { "Jaston", 1.0 },
  { "Jastyn", 0.0 },
  { "Jasuan", 1.0 },
  { "Jasun", 1.0 },
  { "Jasur", 1.0 },
  { "Jaswinder", 1.0 },
  { "Jaswiry", 0.0 },
  { "Jasy", 0.0 },
  { "Jasya", 0.0 },
  { "Jasyah", 0.7567568 },
  { "Jasye", 1.0 },
  { "Jasyia", 0.0 },
  { "Jasyiah", 0.37037036 },
  { "Jasyn", 1.0 },
  { "Jasyra", 0.0 },
  { "Jaszlyn", 0.0 },
  { "Jaszmin", 0.0 },
  { "Jaszmine", 0.0 },
  { "Jata", 0.0 },
  { "Jataiya", 0.0 },
  { "Jatana", 0.0 },
  { "Jatanna", 0.0 },
  { "Jatara", 0.0 },
  { "Jatari", 1.0 },
  { "Jatarius", 1.0 },
  { "Jatasha", 0.0 },
  { "Jatasia", 0.0 },
  { "Jataun", 0.0 },
  { "Jataurus", 1.0 },
  { "Jatava", 0.0 },
  { "Jataveon", 1.0 },
  { "Jatavia", 0.0 },
  { "Jatavian", 1.0 },
  { "Jatavien", 1.0 },
  { "Jatavion", 1.0 },
  { "Jatavious", 1.0 },
  { "Jatavis", 1.0 },
  { "Jatavius", 1.0 },
  { "Jataya", 0.0 },
  { "Jatayvion", 1.0 },
  { "Jaten", 1.0 },
  { "Jateria", 0.0 },
  { "Jaterica", 0.0 },
  { "Jaterion", 1.0 },
  { "Jaterious", 1.0 },
  { "Jaterius", 1.0 },
  { "Jaterra", 0.0 },
  { "Jaterria", 0.0 },
  { "Jaterrian", 1.0 },
  { "Jaterrica", 0.0 },
  { "Jaterrion", 1.0 },
  { "Jaterrious", 1.0 },
  { "Jaterrius", 1.0 },
  { "Jathan", 1.0 },
  { "Jathaniel", 1.0 },
  { "Jathen", 1.0 },
  { "Jathniel", 1.0 },
  { "Jathon", 1.0 },
  { "Jathziry", 0.0 },
  { "Jatia", 0.0 },
  { "Jatiana", 0.0 },
  { "Jatin", 1.0 },
  { "Jatina", 0.0 },
  { "Jatinder", 1.0 },
  { "Jatisha", 0.0 },
  { "Jatiya", 0.0 },
  { "Jatniel", 1.0 },
  { "Jaton", 0.47008547 },
  { "Jatonya", 0.0 },
  { "Jatori", 0.20833333 },
  { "Jatoria", 0.0 },
  { "Jatorian", 1.0 },
  { "Jatorria", 0.0 },
  { "Jatory", 1.0 },
  { "Jatoya", 0.0 },
  { "Jatwan", 1.0 },
  { "Jatyler", 1.0 },
  { "Jatyra", 0.0 },
  { "Jatziri", 0.0 },
  { "Jatziry", 0.0 },
  { "Jauan", 1.0 },
  { "Jaudiel", 1.0 },
  { "Jauier", 1.0 },
  { "Jaun", 1.0 },
  { "Jauna", 0.0 },
  { "Jaunell", 0.0 },
  { "Jauneta", 0.0 },
  { "Jaunice", 0.0 },
  { "Jaunita", 0.0 },
  { "Jaunna", 0.0 },
  { "Jaunte", 1.0 },
  { "Jauquan", 1.0 },
  { "Jaurice", 1.0 },
  { "Jauron", 1.0 },
  { "Jaushua", 1.0 },
  { "Jaustin", 1.0 },
  { "Jauwana", 0.0 },
  { "Java", 0.5405405 },
  { "Javad", 1.0 },
  { "Javae", 0.24404761 },
  { "Javaeh", 0.0 },
  { "Javahn", 1.0 },
  { "Javair", 1.0 },
  { "Javale", 1.0 },
  { "Javan", 0.9871642 },
  { "Javana", 0.0 },
  { "Javani", 1.0 },
  { "Javanna", 0.0 },
  { "Javanni", 1.0 },
  { "Javant", 1.0 },
  { "Javanta", 1.0 },
  { "Javantay", 1.0 },
  { "Javante", 1.0 },
  { "Javar", 1.0 },
  { "Javare", 1.0 },
  { "Javaree", 1.0 },
  { "Javareon", 1.0 },
  { "Javares", 1.0 },
  { "Javari", 1.0 },
  { "Javaria", 0.0 },
  { "Javarian", 1.0 },
  { "Javarie", 1.0 },
  { "Javario", 1.0 },
  { "Javarion", 1.0 },
  { "Javarious", 1.0 },
  { "Javaris", 1.0 },
  { "Javarius", 1.0 },
  { "Javarous", 1.0 },
  { "Javarri", 1.0 },
  { "Javarrie", 1.0 },
  { "Javarrion", 1.0 },
  { "Javarris", 1.0 },
  { "Javarrius", 1.0 },
  { "Javarron", 1.0 },
  { "Javarus", 1.0 },
  { "Javary", 1.0 },
  { "Javas", 1.0 },
  { "Javaughn", 1.0 },
  { "Javaun", 1.0 },
  { "Javaunte", 1.0 },
  { "Javawn", 1.0 },
  { "Javaya", 0.0 },
  { "Javayah", 0.0 },
  { "Jave", 1.0 },
  { "Javea", 0.0 },
  { "Javeah", 0.0 },
  { "Javean", 1.0 },
  { "Javed", 1.0 },
  { "Javeed", 1.0 },
  { "Javein", 1.0 },
  { "Javeion", 1.0 },
  { "Javel", 1.0 },
  { "Javelin", 0.0 },
  { "Javell", 1.0 },
  { "Javelle", 1.0 },
  { "Javelyn", 0.0 },
  { "Javen", 0.96747965 },
  { "Javene", 0.0 },
  { "Javeon", 1.0 },
  { "Javere", 1.0 },
  { "Javeria", 0.0 },
  { "Javery", 1.0 },
  { "Javetta", 0.0 },
  { "Javette", 0.0 },
  { "Javeya", 0.0 },
  { "Javeyah", 0.0 },
  { "Javeyon", 1.0 },
  { "Javez", 1.0 },
  { "Javi", 1.0 },
  { "Javia", 0.0 },
  { "Javiah", 0.0 },
  { "Javiair", 1.0 },
  { "Javian", 0.99739355 },
  { "Javiana", 0.0 },
  { "Javianna", 0.0 },
  { "Javiar", 1.0 },
  { "Javid", 1.0 },
  { "Javiel", 1.0 },
  { "Javien", 1.0 },
  { "Javieon", 1.0 },
  { "Javier", 0.9948692 },
  { "Javiera", 0.0 },
  { "Javiere", 1.0 },
  { "Javierjr", 1.0 },
  { "Javin", 0.9976744 },
  { "Javina", 0.0 },
  { "Javion", 0.99795955 },
  { "Javiona", 0.0 },
  { "Javione", 1.0 },
  { "Javionna", 0.0 },
  { "Javionne", 1.0 },
  { "Javionte", 1.0 },
  { "Javious", 1.0 },
  { "Javir", 1.0 },
  { "Javis", 0.99401915 },
  { "Javita", 0.0 },
  { "Javius", 1.0 },
  { "Javiyon", 1.0 },
  { "Javlyn", 0.0 },
  { "Javohn", 1.0 },
  { "Javon", 0.95490384 },
  { "Javona", 0.0 },
  { "Javonda", 0.0 },
  { "Javondre", 1.0 },
  { "Javone", 0.9744186 },
  { "Javoni", 0.953917 },
  { "Javonia", 0.0 },
  { "Javonie", 1.0 },
  { "Javonn", 1.0 },
  { "Javonna", 0.0 },
  { "Javonne", 0.45970148 },
  { "Javonni", 0.88643533 },
  { "Javonnie", 0.8979592 },
  { "Javonta", 1.0 },
  { "Javontae", 1.0 },
  { "Javontai", 1.0 },
  { "Javontay", 1.0 },
  { "Javontaye", 1.0 },
  { "Javonte", 0.9889706 },
  { "Javontez", 1.0 },
  { "Javonti", 1.0 },
  { "Javor", 1.0 },
  { "Javori", 1.0 },
  { "Javoris", 1.0 },
  { "Javorius", 1.0 },
  { "Javy", 1.0 },
  { "Javyion", 1.0 },
  { "Javyn", 1.0 },
  { "Javyon", 1.0 },
  { "Jawaad", 1.0 },
  { "Jawaan", 1.0 },
  { "Jawad", 1.0 },
  { "Jawaher", 0.0 },
  { "Jawan", 0.9913793 },
  { "Jawana", 0.0 },
  { "Jawanda", 0.0 },
  { "Jawane", 1.0 },
  { "Jawann", 1.0 },
  { "Jawanna", 0.0 },
  { "Jawanza", 0.97572815 },
  { "Jawara", 0.98207885 },
  { "Jawarren", 1.0 },
  { "Jawaski", 1.0 },
  { "Jawaun", 1.0 },
  { "Jawayne", 1.0 },
  { "Jawdat", 1.0 },
  { "Jawhar", 1.0 },
  { "Jawon", 0.9933599 },
  { "Jawone", 1.0 },
  { "Jawonna", 0.0 },
  { "Jaworski", 1.0 },
  { "Jawuan", 1.0 },
  { "Jawun", 1.0 },
  { "Jawvan", 1.0 },
  { "Jawwaad", 1.0 },
  { "Jawwad", 1.0 },
  { "Jax", 0.99094063 },
  { "Jaxan", 1.0 },
  { "Jaxcyn", 1.0 },
  { "Jaxden", 1.0 },
  { "Jaxel", 1.0 },
  { "Jaxen", 0.98866564 },
  { "Jaxi", 0.0 },
  { "Jaxie", 0.0 },
  { "Jaxiel", 1.0 },
  { "Jaxin", 1.0 },
  { "Jaxion", 1.0 },
  { "Jaxlyn", 0.0 },
  { "Jaxlynn", 0.0 },
  { "Jaxn", 1.0 },
  { "Jaxom", 1.0 },
  { "Jaxon", 0.99577814 },
  { "Jaxonjames", 1.0 },
  { "Jaxper", 1.0 },
  { "Jaxs", 1.0 },
  { "Jaxsen", 0.99535245 },
  { "Jaxsin", 1.0 },
  { "Jaxson", 0.99809045 },
  { "Jaxsten", 1.0 },
  { "Jaxstin", 1.0 },
  { "Jaxston", 1.0 },
  { "Jaxstyn", 1.0 },
  { "Jaxsun", 1.0 },
  { "Jaxsyn", 0.8558719 },
  { "Jaxten", 1.0 },
  { "Jaxtin", 1.0 },
  { "Jaxton", 1.0 },
  { "Jaxtyn", 1.0 },
  { "Jaxun", 1.0 },
  { "Jaxx", 0.9874477 },
  { "Jaxxen", 1.0 },
  { "Jaxxin", 1.0 },
  { "Jaxxon", 1.0 },
  { "Jaxxson", 1.0 },
  { "Jaxxton", 1.0 },
  { "Jaxxyn", 1.0 },
  { "Jaxyn", 0.7669441 },
  { "Jaxzen", 1.0 },
  { "Jaxzon", 1.0 },
  { "Jaxzyn", 1.0 },
  { "Jay", 0.9869592 },
  { "Jaya", 0.0074481866 },
  { "Jayace", 1.0 },
  { "Jayah", 0.0 },
  { "Jayahna", 0.0 },
  { "Jayan", 1.0 },
  { "Jayana", 0.0 },
  { "Jayanah", 0.0 },
  { "Jayandre", 1.0 },
  { "Jayani", 0.0 },
  { "Jayann", 0.0 },
  { "Jayanna", 0.0 },
  { "Jayannah", 0.0 },
  { "Jayanne", 0.0 },
  { "Jayanni", 0.0 },
  { "Jayansh", 1.0 },
  { "Jayant", 1.0 },
  { "Jayanth", 1.0 },
  { "Jayanthony", 1.0 },
  { "Jayanti", 0.0 },
  { "Jayar", 1.0 },
  { "Jayaun", 1.0 },
  { "Jayauna", 0.0 },
  { "Jayben", 1.0 },
  { "Jaybin", 1.0 },
  { "Jaybree", 0.0 },
  { "Jaybriel", 1.0 },
  { "Jayc", 1.0 },
  { "Jayce", 0.94467014 },
  { "Jaycean", 1.0 },
  { "Jaycee", 0.116983496 },
  { "Jayceeon", 1.0 },
  { "Jayceion", 1.0 },
  { "Jaycek", 1.0 },
  { "Jaycelyn", 0.0 },
  { "Jaycelynn", 0.0 },
  { "Jaycen", 0.994382 },
  { "Jayceon", 0.9951771 },
  { "Jayceona", 0.0 },
  { "Jayceonna", 0.0 },
  { "Jaycey", 0.0 },
  { "Jayceyon", 1.0 },
  { "Jayci", 0.0 },
  { "Jaycia", 0.0 },
  { "Jaycian", 1.0 },
  { "Jayciana", 0.0 },
  { "Jaycie", 0.0015033073 },
  { "Jaycieon", 1.0 },
  { "Jaycii", 0.0 },
  { "Jaycin", 1.0 },
  { "Jaycion", 1.0 },
  { "Jayco", 1.0 },
  { "Jaycob", 1.0 },
  { "Jaycon", 1.0 },
  { "Jaycub", 1.0 },
  { "Jayd", 0.24367817 },
  { "Jayda", 0.0013518081 },
  { "Jaydah", 0.0 },
  { "Jaydalee", 0.0 },
  { "Jaydalin", 0.0 },
  { "Jaydalis", 0.0 },
  { "Jaydalise", 0.0 },
  { "Jaydaliz", 0.0 },
  { "Jaydalyn", 0.0 },
  { "Jaydalynn", 0.0 },
  { "Jaydamarie", 0.0 },
  { "Jaydan", 0.82305545 },
  { "Jaydann", 0.0 },
  { "Jaydden", 1.0 },
  { "Jayde", 0.05174741 },
  { "Jaydean", 1.0 },
  { "Jaydee", 0.44771242 },
  { "Jaydeen", 0.5947368 },
  { "Jaydeep", 1.0 },
  { "Jaydehn", 1.0 },
  { "Jaydein", 1.0 },
  { "Jaydel", 1.0 },
  { "Jaydelin", 0.0 },
  { "Jaydeliz", 0.0 },
  { "Jaydelyn", 0.0 },
  { "Jaydelynn", 0.0 },
  { "Jaydem", 1.0 },
  { "Jayden", 0.90273684 },
  { "Jaydenalexander", 1.0 },
  { "Jaydence", 0.42857143 },
  { "Jaydenjames", 1.0 },
  { "Jaydenlee", 1.0 },
  { "Jaydenmatthew", 1.0 },
  { "Jaydenmichael", 1.0 },
  { "Jaydenn", 0.8600583 },
  { "Jaydhen", 1.0 },
  { "Jaydi", 0.0 },
  { "Jaydia", 0.0 },
  { "Jaydian", 1.0 },
  { "Jaydie", 0.0 },
  { "Jaydiel", 1.0 },
  { "Jaydien", 1.0 },
  { "Jaydin", 0.7218007 },
  { "Jaydis", 0.90225565 },
  { "Jaydn", 0.61957616 },
  { "Jaydon", 0.97664165 },
  { "Jaydrian", 1.0 },
  { "Jaydrien", 1.0 },
  { "Jaydun", 1.0 },
  { "Jaydus", 1.0 },
  { "Jaydy", 0.0 },
  { "Jaydyn", 0.5650752 },
  { "Jaydynn", 0.275 },
  { "Jaye", 0.30339733 },
  { "Jayeda", 0.0 },
  { "Jayeden", 1.0 },
  { "Jayel", 0.47972974 },
  { "Jayela", 0.0 },
  { "Jayelin", 1.0 },
  { "Jayelle", 0.0 },
  { "Jayelyn", 0.0 },
  { "Jayelynn", 0.0 },
  { "Jayen", 1.0 },
  { "Jayesh", 1.0 },
  { "Jayetta", 0.0 },
  { "Jayger", 1.0 },
  { "Jayhden", 1.0 },
  { "Jayia", 0.0 },
  { "Jayiah", 0.0 },
  { "Jayiden", 1.0 },
  { "Jayin", 1.0 },
  { "Jayion", 1.0 },
  { "Jayjay", 1.0 },
  { "Jayjuan", 1.0 },
  { "Jayk", 1.0 },
  { "Jayke", 1.0 },
  { "Jayken", 1.0 },
  { "Jaykin", 1.0 },
  { "Jayko", 1.0 },
  { "Jaykob", 1.0 },
  { "Jaykon", 1.0 },
  { "Jaykub", 1.0 },
  { "Jaykwan", 1.0 },
  { "Jaykwon", 1.0 },
  { "Jayla", 0.0013728868 },
  { "Jaylaa", 0.0 },
  { "Jaylaah", 0.0 },
  { "Jaylaan", 0.0 },
  { "Jaylah", 0.0 },
  { "Jaylamarie", 0.0 },
  { "Jaylan", 0.84388864 },
  { "Jaylana", 0.0 },
  { "Jayland", 1.0 },
  { "Jaylani", 0.031767957 },
  { "Jaylanie", 0.0 },
  { "Jaylanis", 0.0 },
  { "Jaylann", 0.0 },
  { "Jaylanni", 0.0 },
  { "Jaylannie", 0.0 },
  { "Jaylany", 0.0 },
  { "Jaylarose", 0.0 },
  { "Jaylaun", 1.0 },
  { "Jayle", 0.0 },
  { "Jaylea", 0.0 },
  { "Jayleah", 0.0 },
  { "Jaylean", 0.0 },
  { "Jayleana", 0.0 },
  { "Jayleanna", 0.0 },
  { "Jaylece", 0.0 },
  { "Jaylee", 0.015714286 },
  { "Jayleen", 0.02211357 },
  { "Jayleena", 0.0 },
  { "Jayleene", 0.0 },
  { "Jaylei", 0.0 },
  { "Jayleigh", 0.0 },
  { "Jaylein", 0.23809524 },
  { "Jaylem", 1.0 },
  { "Jaylen", 0.88257533 },
  { "Jaylena", 0.0 },
  { "Jaylend", 1.0 },
  { "Jaylene", 0.009338812 },
  { "Jaylenn", 0.36679536 },
  { "Jaylenne", 0.0 },
  { "Jayleon", 1.0 },
  { "Jaylese", 0.0 },
  { "Jaylex", 1.0 },
  { "Jayley", 0.0 },
  { "Jayli", 0.0 },
  { "Jaylia", 0.0 },
  { "Jayliah", 0.0 },
  { "Jaylian", 1.0 },
  { "Jayliana", 0.0 },
  { "Jayliani", 0.0 },
  { "Jaylianie", 0.0 },
  { "Jaylianis", 0.0 },
  { "Jaylianiz", 0.0 },
  { "Jaylianna", 0.0 },
  { "Jayliannie", 0.0 },
  { "Jayliany", 0.0 },
  { "Jaylicia", 0.0 },
  { "Jaylie", 0.0 },
  { "Jaylien", 0.6041667 },
  { "Jaylin", 0.557292 },
  { "Jaylina", 0.0 },
  { "Jaylind", 1.0 },
  { "Jayline", 0.006127451 },
  { "Jaylinn", 0.04307692 },
  { "Jaylinne", 0.0 },
  { "Jaylis", 0.0 },
  { "Jaylisa", 0.0 },
  { "Jaylise", 0.0 },
  { "Jaylissa", 0.0 },
  { "Jaylisse", 0.0 },
  { "Jayliyah", 0.0 },
  { "Jayliz", 0.0 },
  { "Jayllen", 1.0 },
  { "Jayln", 0.646875 },
  { "Jaylnn", 0.0 },
  { "Jaylon", 0.9729643 },
  { "Jaylond", 1.0 },
  { "Jayloni", 1.0 },
  { "Jaylonn", 1.0 },
  { "Jaylun", 1.0 },
  { "Jayly", 0.0 },
  { "Jaylyn", 0.23667377 },
  { "Jaylyne", 0.048076924 },
  { "Jaylynn", 0.056526948 },
  { "Jaylynne", 0.0 },
  { "Jayma", 0.0 },
  { "Jayman", 1.0 },
  { "Jaymani", 1.0 },
  { "Jaymar", 1.0 },
  { "Jaymari", 1.0 },
  { "Jaymarie", 0.0 },
  { "Jaymarion", 1.0 },
  { "Jaymason", 1.0 },
  { "Jayme", 0.2124 },
  { "Jaymee", 0.011241447 },
  { "Jaymeir", 1.0 },
  { "Jaymel", 1.0 },
  { "Jaymen", 1.0 },
  { "Jaymere", 1.0 },
  { "Jaymes", 0.9733807 },
  { "Jaymesha", 0.0 },
  { "Jaymeson", 0.96103895 },
  { "Jaymez", 1.0 },
  { "Jaymi", 0.0037907506 },
  { "Jaymiah", 0.0 },
  { "Jaymian", 1.0 },
  { "Jaymichael", 1.0 },
  { "Jaymie", 0.10823408 },
  { "Jaymien", 1.0 },
  { "Jaymier", 1.0 },
  { "Jaymin", 1.0 },
  { "Jaymion", 1.0 },
  { "Jaymir", 1.0 },
  { "Jaymis", 1.0 },
  { "Jaymisha", 0.0 },
  { "Jaymison", 1.0 },
  { "Jaymiya", 0.0 },
  { "Jaymon", 1.0 },
  { "Jaymond", 1.0 },
  { "Jayms", 1.0 },
  { "Jaymya", 0.0 },
  { "Jaymz", 1.0 },
  { "Jayn", 0.0 },
  { "Jayna", 0.0 },
  { "Jaynae", 0.0 },
  { "Jaynah", 0.0 },
  { "Jaynalis", 0.0 },
  { "Jayne", 0.0011219409 },
  { "Jaynee", 0.0 },
  { "Jayneen", 0.0 },
  { "Jaynell", 0.0 },
  { "Jaynelle", 0.0 },
  { "Jaynesha", 0.0 },
  { "Jaynette", 0.0 },
  { "Jayni", 0.15625 },
  { "Jaynia", 0.0 },
  { "Jayniah", 0.0 },
  { "Jaynice", 0.0 },
  { "Jaynie", 0.0 },
  { "Jaynisha", 0.0 },
  { "Jayniyah", 0.0 },
  { "Jaynne", 0.0 },
  { "Jayon", 1.0 },
  { "Jayona", 0.0 },
  { "Jayonah", 0.0 },
  { "Jayonn", 1.0 },
  { "Jayonna", 0.0 },
  { "Jayonni", 0.52380955 },
  { "Jayonte", 1.0 },
  { "Jayquan", 1.0 },
  { "Jayquin", 1.0 },
  { "Jayquon", 1.0 },
  { "Jayqwan", 1.0 },
  { "Jayr", 1.0 },
  { "Jayra", 0.0 },
  { "Jayre", 0.0 },
  { "Jayree", 1.0 },
  { "Jayrell", 1.0 },
  { "Jayren", 1.0 },
  { "Jayriel", 1.0 },
  { "Jayrin", 1.0 },
  { "Jayro", 1.0 },
  { "Jayron", 1.0 },
  { "Jaysa", 0.0 },
  { "Jaysan", 1.0 },
  { "Jaysaun", 1.0 },
  { "Jayse", 0.9823279 },
  { "Jaysea", 0.0 },
  { "Jaysean", 1.0 },
  { "Jaysen", 1.0 },
  { "Jayseon", 1.0 },
  { "Jaysha", 0.0 },
  { "Jayshan", 1.0 },
  { "Jayshaun", 1.0 },
  { "Jayshawn", 1.0 },
  { "Jayshon", 1.0 },
  { "Jayshun", 1.0 },
  { "Jaysia", 0.0 },
  { "Jaysiah", 1.0 },
  { "Jaysian", 1.0 },
  { "Jaysie", 0.0 },
  { "Jaysin", 1.0 },
  { "Jaysion", 1.0 },
  { "Jayslin", 0.0 },
  { "Jayson", 0.99897444 },
  { "Jaysson", 1.0 },
  { "Jaysten", 1.0 },
  { "Jaystin", 1.0 },
  { "Jayston", 1.0 },
  { "Jaysun", 1.0 },
  { "Jaytee", 1.0 },
  { "Jayten", 1.0 },
  { "Jaythan", 1.0 },
  { "Jaythen", 1.0 },
  { "Jaythian", 1.0 },
  { "Jaython", 1.0 },
  { "Jaytin", 1.0 },
  { "Jayton", 1.0 },
  { "Jayva", 0.0 },
  { "Jayvan", 1.0 },
  { "Jayvaughn", 1.0 },
  { "Jayvaun", 1.0 },
  { "Jayvean", 1.0 },
  { "Jayvee", 1.0 },
  { "Jayveer", 1.0 },
  { "Jayvein", 1.0 },
  { "Jayveion", 1.0 },
  { "Jayven", 1.0 },
  { "Jayveon", 1.0 },
  { "Jayvia", 0.0 },
  { "Jayvian", 1.0 },
  { "Jayvien", 1.0 },
  { "Jayvier", 1.0 },
  { "Jayvin", 1.0 },
  { "Jayvion", 1.0 },
  { "Jayviona", 0.0 },
  { "Jayvionna", 0.0 },
  { "Jayvis", 1.0 },
  { "Jayvius", 1.0 },
  { "Jayvn", 1.0 },
  { "Jayvon", 1.0 },
  { "Jayvone", 1.0 },
  { "Jayvonn", 1.0 },
  { "Jayvonna", 0.0 },
  { "Jayvonne", 1.0 },
  { "Jayvonni", 1.0 },
  { "Jayvonte", 1.0 },
  { "Jayvyn", 1.0 },
  { "Jaywana", 0.0 },
  { "Jaywanna", 0.0 },
  { "Jaywon", 1.0 },
  { "Jayz", 1.0 },
  { "Jayza", 0.0 },
  { "Jayze", 1.0 },
  { "Jayzen", 1.0 },
  { "Jayzeon", 1.0 },
  { "Jayziah", 1.0 },
  { "Jayziel", 1.0 },
  { "Jayzier", 1.0 },
  { "Jayzin", 1.0 },
  { "Jayzion", 1.0 },
  { "Jayzon", 1.0 },
  { "Jaz", 0.671875 },
  { "Jaza", 0.0 },
  { "Jazabel", 0.0 },
  { "Jazabell", 0.0 },
  { "Jazabella", 0.0 },
  { "Jazabelle", 0.0 },
  { "Jazae", 0.0 },
  { "Jazael", 1.0 },
  { "Jazahria", 0.0 },
  { "Jazai", 1.0 },
  { "Jazaiah", 0.64285713 },
  { "Jazaih", 1.0 },
  { "Jazaiya", 0.0 },
  { "Jazale", 0.0 },
  { "Jazalin", 0.0 },
  { "Jazalle", 0.0 },
  { "Jazalyn", 0.0 },
  { "Jazalynn", 0.0 },
  { "Jazamine", 0.0 },
  { "Jazanae", 0.0 },
  { "Jazaniah", 0.0 },
  { "Jazara", 0.0 },
  { "Jazarah", 0.0 },
  { "Jazari", 0.30985916 },
  { "Jazaria", 0.0 },
  { "Jazariah", 0.0 },
  { "Jazarian", 1.0 },
  { "Jazarion", 1.0 },
  { "Jazariya", 0.0 },
  { "Jazariyah", 0.0 },
  { "Jazavian", 1.0 },
  { "Jazavier", 1.0 },
  { "Jazavion", 1.0 },
  { "Jazay", 0.0 },
  { "Jazaya", 0.0 },
  { "Jazayah", 0.0 },
  { "Jazayvion", 1.0 },
  { "Jazbel", 0.0 },
  { "Jaze", 1.0 },
  { "Jazeel", 1.0 },
  { "Jazeer", 1.0 },
  { "Jazeera", 0.0 },
  { "Jazel", 0.018867925 },
  { "Jazell", 0.0 },
  { "Jazella", 0.0 },
  { "Jazelle", 0.0 },
  { "Jazelyn", 0.0 },
  { "Jazelynn", 0.0 },
  { "Jazemine", 0.0 },
  { "Jazen", 1.0 },
  { "Jazer", 1.0 },
  { "Jazette", 0.0 },
  { "Jazharia", 0.0 },
  { "Jazhiel", 1.0 },
  { "Jazia", 0.0 },
  { "Jaziah", 0.5396073 },
  { "Jazib", 1.0 },
  { "Jaziel", 0.9773313 },
  { "Jazier", 1.0 },
  { "Jazilyn", 0.0 },
  { "Jazilynn", 0.0 },
  { "Jazimine", 0.0 },
  { "Jazin", 1.0 },
  { "Jazion", 1.0 },
  { "Jazir", 1.0 },
  { "Jazira", 0.0 },
  { "Jazirah", 0.0 },
  { "Jazire", 0.0 },
  { "Jazive", 0.0 },
  { "Jaziya", 0.0 },
  { "Jaziyah", 0.033379693 },
  { "Jazlean", 0.0 },
  { "Jazlee", 0.0 },
  { "Jazleen", 0.0 },
  { "Jazleene", 0.0 },
  { "Jazlen", 0.0 },
  { "Jazlena", 0.0 },
  { "Jazlene", 0.0 },
  { "Jazlin", 0.0 },
  { "Jazlina", 0.0 },
  { "Jazline", 0.0 },
  { "Jazly", 0.0 },
  { "Jazlyn", 0.0 },
  { "Jazlyne", 0.0 },
  { "Jazlynn", 0.0 },
  { "Jazlynne", 0.0 },
  { "Jazma", 0.0 },
  { "Jazmaine", 0.0 },
  { "Jazman", 0.032352943 },
  { "Jazmane", 0.0 },
  { "Jazmany", 1.0 },
  { "Jazmari", 0.0 },
  { "Jazmariah", 0.0 },
  { "Jazmarie", 0.0 },
  { "Jazmeen", 0.0 },
  { "Jazmein", 0.0 },
  { "Jazmen", 0.0046554934 },
  { "Jazmene", 0.0 },
  { "Jazmenn", 0.0 },
  { "Jazmere", 0.0 },
  { "Jazmia", 0.0 },
  { "Jazmin", 0.0056146425 },
  { "Jazmina", 0.0 },
  { "Jazmine", 0.002814659 },
  { "Jazminn", 0.0 },
  { "Jazminne", 0.0 },
  { "Jazmon", 0.13793103 },
  { "Jazmond", 0.6296296 },
  { "Jazmone", 0.0 },
  { "Jazmun", 0.0 },
  { "Jazmyn", 0.0 },
  { "Jazmyne", 0.0 },
  { "Jazmynn", 0.0 },
  { "Jazmynne", 0.0 },
  { "Jazon", 1.0 },
  { "Jazper", 1.0 },
  { "Jazsmin", 0.0 },
  { "Jazsmine", 0.0 },
  { "Jazuri", 0.0 },
  { "Jazya", 0.0 },
  { "Jazyah", 0.0 },
  { "Jazyia", 0.0 },
  { "Jazyiah", 0.0 },
  { "Jazymne", 0.0 },
  { "Jazyon", 1.0 },
  { "Jazyra", 0.0 },
  { "Jazyria", 0.0 },
  { "Jazyriah", 0.0 },
  { "Jazz", 0.5856905 },
  { "Jazzabella", 0.0 },
  { "Jazzabelle", 0.0 },
  { "Jazzae", 0.0 },
  { "Jazzalyn", 0.0 },
  { "Jazzalynn", 0.0 },
  { "Jazzanae", 0.0 },
  { "Jazzarae", 0.0 },
  { "Jazzariah", 0.0 },
  { "Jazzel", 0.0 },
  { "Jazzell", 0.0 },
  { "Jazzelle", 0.0 },
  { "Jazzelyn", 0.0 },
  { "Jazzi", 0.0 },
  { "Jazzia", 0.0 },
  { "Jazziah", 0.0 },
  { "Jazzie", 0.0 },
  { "Jazziel", 1.0 },
  { "Jazzilyn", 0.0 },
  { "Jazzilynn", 0.0 },
  { "Jazzleen", 0.0 },
  { "Jazzlene", 0.0 },
  { "Jazzlin", 0.0 },
  { "Jazzlyn", 0.0 },
  { "Jazzlyne", 0.0 },
  { "Jazzlynn", 0.0 },
  { "Jazzlynne", 0.0 },
  { "Jazzma", 0.0 },
  { "Jazzmaine", 0.0 },
  { "Jazzman", 0.09090909 },
  { "Jazzmarie", 0.0 },
  { "Jazzmen", 0.0 },
  { "Jazzmene", 0.0 },
  { "Jazzmin", 0.0 },
  { "Jazzmine", 0.0 },
  { "Jazzminn", 0.0 },
  { "Jazzmon", 0.0 },
  { "Jazzmond", 0.0 },
  { "Jazzmun", 0.0 },
  { "Jazzmyn", 0.0 },
  { "Jazzmyne", 0.0 },
  { "Jazzmynn", 0.0 },
  { "Jazzy", 0.0 },
  { "Jb", 1.0 },
  { "Jc", 0.9780415 },
  { "Jceion", 1.0 },
  { "Jceon", 1.0 },
  { "Jceyon", 1.0 },
  { "Jcion", 1.0 },
  { "Jcorey", 1.0 },
  { "Jcyon", 1.0 },
  { "Jd", 0.97692305 },
  { "Jda", 0.0 },
  { "Jdah", 0.0 },
  { "Jdan", 1.0 },
  { "Jden", 1.0 },
  { "Jdin", 1.0 },
  { "Jdon", 1.0 },
  { "Jdyn", 1.0 },
  { "Jdynn", 1.0 },
  { "Je", 1.0 },
  { "Jea", 0.0 },
  { "Jeaden", 1.0 },
  { "Jeadon", 1.0 },
  { "Jealisa", 0.0 },
  { "Jealousy", 0.0 },
  { "Jealyn", 0.0 },
  { "Jeamileth", 0.0 },
  { "Jean", 0.049225822 },
  { "Jeana", 0.0 },
  { "Jeanae", 0.0 },
  { "Jeanann", 0.0 },
  { "Jeananne", 0.0 },
  { "Jeanasia", 0.0 },
  { "Jeanaya", 0.0 },
  { "Jeanbaptiste", 1.0 },
  { "Jeancarlo", 1.0 },
  { "Jeancarlos", 1.0 },
  { "Jeanchristophe", 1.0 },
  { "Jeanclaude", 1.0 },
  { "Jeane", 0.0035603025 },
  { "Jeanea", 0.0 },
  { "Jeanean", 0.0 },
  { "Jeaneane", 0.0 },
  { "Jeanee", 0.0 },
  { "Jeaneen", 0.0 },
  { "Jeanel", 0.0 },
  { "Jeanell", 0.0 },
  { "Jeanella", 0.0 },
  { "Jeanelle", 0.0 },
  { "Jeanelly", 0.0 },
  { "Jeanene", 0.0 },
  { "Jeanenne", 0.0 },
  { "Jeanessa", 0.0 },
  { "Jeanet", 0.0 },
  { "Jeanete", 0.0 },
  { "Jeaneth", 0.0 },
  { "Jeanett", 0.0 },
  { "Jeanetta", 0.0 },
  { "Jeanette", 0.0032939448 },
  { "Jeanettie", 0.0 },
  { "Jeani", 0.0 },
  { "Jeania", 0.0 },
  { "Jeanice", 0.0 },
  { "Jeanie", 0.0 },
  { "Jeaniece", 0.0 },
  { "Jeaniene", 0.0 },
  { "Jeanifer", 0.0 },
  { "Jeanina", 0.0 },
  { "Jeanine", 0.00046437015 },
  { "Jeaninne", 0.0 },
  { "Jeanique", 0.0 },
  { "Jeanise", 0.0 },
  { "Jeanita", 0.0 },
  { "Jeanitta", 0.0 },
  { "Jeaniyah", 0.0 },
  { "Jeanjacques", 1.0 },
  { "Jeanluc", 1.0 },
  { "Jeanluca", 1.0 },
  { "Jeanlucas", 1.0 },
  { "Jeanluke", 1.0 },
  { "Jeanmarc", 1.0 },
  { "Jeanmarco", 1.0 },
  { "Jeanmarie", 0.0 },
  { "Jeanmichael", 1.0 },
  { "Jeanmichel", 1.0 },
  { "Jeann", 0.0 },
  { "Jeanna", 0.0 },
  { "Jeanne", 0.002211224 },
  { "Jeannea", 0.0 },
  { "Jeannean", 0.0 },
  { "Jeannedarc", 0.0 },
  { "Jeannee", 0.0 },
  { "Jeannell", 0.0 },
  { "Jeannelle", 0.0 },
  { "Jeannemarie", 0.0 },
  { "Jeannene", 0.0 },
  { "Jeannet", 0.0 },
  { "Jeannete", 0.0 },
  { "Jeannett", 0.0 },
  { "Jeannetta", 0.0 },
  { "Jeannette", 0.00070930197 },
  { "Jeanney", 0.0 },
  { "Jeanni", 0.0 },
  { "Jeannia", 0.0 },
  { "Jeannice", 0.0 },
  { "Jeannicole", 0.0 },
  { "Jeannie", 0.001529137 },
  { "Jeannifer", 0.0 },
  { "Jeannine", 0.0002499063 },
  { "Jeanny", 0.0 },
  { "Jeanpaul", 1.0 },
  { "Jeanphilippe", 1.0 },
  { "Jeanpierre", 1.0 },
  { "Jeantte", 0.0 },
  { "Jeany", 0.0 },
  { "Jearl", 0.9087838 },
  { "Jearld", 1.0 },
  { "Jearldean", 0.0 },
  { "Jearldine", 0.0 },
  { "Jearlean", 0.0 },
  { "Jearlene", 0.0 },
  { "Jearline", 0.0 },
  { "Jearold", 1.0 },
  { "Jeason", 1.0 },
  { "Jeaven", 1.0 },
  { "Jeb", 1.0 },
  { "Jebadiah", 1.0 },
  { "Jebb", 1.0 },
  { "Jebediah", 1.0 },
  { "Jebidiah", 1.0 },
  { "Jebreel", 1.0 },
  { "Jebria", 0.0 },
  { "Jebron", 1.0 },
  { "Jecaryous", 1.0 },
  { "Jeccica", 0.0 },
  { "Jecenia", 0.0 },
  { "Jeckson", 1.0 },
  { "Jeconiah", 1.0 },
  { "Jecorey", 1.0 },
  { "Jecori", 1.0 },
  { "Jecory", 1.0 },
  { "Jed", 1.0 },
  { "Jeda", 0.0 },
  { "Jedadia", 1.0 },
  { "Jedadiah", 1.0 },
  { "Jedah", 0.0 },
  { "Jedaiah", 1.0 },
  { "Jedarius", 1.0 },
  { "Jedd", 1.0 },
  { "Jeddie", 1.0 },
  { "Jedediah", 1.0 },
  { "Jeden", 1.0 },
  { "Jedi", 1.0 },
  { "Jediael", 1.0 },
  { "Jediah", 1.0 },
  { "Jedidah", 0.0 },
  { "Jedidiah", 0.995667 },
  { "Jediel", 1.0 },
  { "Jedikiah", 1.0 },
  { "Jedrek", 1.0 },
  { "Jedric", 1.0 },
  { "Jedrick", 1.0 },
  { "Jedriel", 1.0 },
  { "Jedrik", 1.0 },
  { "Jee", 0.11627907 },
  { "Jeel", 0.5 },
  { "Jeena", 0.0 },
  { "Jeenifer", 0.0 },
  { "Jeese", 1.0 },
  { "Jeesica", 0.0 },
  { "Jeet", 1.0 },
  { "Jeeva", 1.0 },
  { "Jeevan", 1.0 },
  { "Jeevika", 0.0 },
  { "Jeeya", 0.0 },
  { "Jeeyoung", 0.0 },
  { "Jef", 1.0 },
  { "Jeferson", 1.0 },
  { "Jefery", 1.0 },
  { "Jeff", 0.9978515 },
  { "Jeffary", 1.0 },
  { "Jeffer", 1.0 },
  { "Jefferey", 1.0 },
  { "Jefferi", 1.0 },
  { "Jefferie", 1.0 },
  { "Jefferson", 1.0 },
  { "Jeffery", 0.99448204 },
  { "Jeffey", 1.0 },
  { "Jeffie", 0.4008289 },
  { "Jeffifer", 0.0 },
  { "Jeffon", 1.0 },
  { "Jefford", 1.0 },
  { "Jefforey", 1.0 },
  { "Jeffory", 1.0 },
  { "Jeffray", 1.0 },
  { "Jeffre", 1.0 },
  { "Jeffree", 1.0 },
  { "Jeffren", 1.0 },
  { "Jeffrery", 1.0 },
  { "Jeffrey", 0.9964156 },
  { "Jeffreyjames", 1.0 },
  { "Jeffri", 1.0 },
  { "Jeffrie", 0.9152709 },
  { "Jeffries", 1.0 },
  { "Jeffro", 1.0 },
  { "Jeffry", 0.99934286 },
  { "Jeffy", 1.0 },
  { "Jefre", 1.0 },
  { "Jefren", 1.0 },
  { "Jefrey", 1.0 },
  { "Jefri", 1.0 },
  { "Jefrin", 1.0 },
  { "Jefry", 1.0 },
  { "Jefte", 1.0 },
  { "Jeg", 1.0 },
  { "Jehad", 1.0 },
  { "Jehan", 0.24311927 },
  { "Jehiel", 1.0 },
  { "Jehieli", 0.0 },
  { "Jehilyn", 0.0 },
  { "Jehlani", 0.6 },
  { "Jehna", 0.0 },
  { "Jehnna", 0.0 },
  { "Jehonadab", 1.0 },
  { "Jehoshua", 1.0 },
  { "Jehovah", 1.0 },
  { "Jehovany", 1.0 },
  { "Jehremy", 1.0 },
  { "Jehron", 1.0 },
  { "Jehu", 1.0 },
  { "Jeice", 1.0 },
  { "Jeicob", 1.0 },
  { "Jeida", 0.0 },
  { "Jeidan", 1.0 },
  { "Jeiden", 1.0 },
  { "Jeidi", 0.0 },
  { "Jeidy", 0.0 },
  { "Jeidyn", 1.0 },
  { "Jeiel", 1.0 },
  { "Jeila", 0.0 },
  { "Jeilani", 0.0 },
  { "Jeilany", 0.0 },
  { "Jeileen", 0.0 },
  { "Jeilene", 0.0 },
  { "Jeilianys", 0.0 },
  { "Jeilin", 0.0 },
  { "Jeily", 0.0 },
  { "Jeilyn", 0.0 },
  { "Jeilynn", 0.0 },
  { "Jeimi", 0.0 },
  { "Jeimmy", 0.0 },
  { "Jeimy", 0.0 },
  { "Jeiner", 1.0 },
  { "Jeinny", 0.0 },
  { "Jeiny", 0.0 },
  { "Jeira", 0.0 },
  { "Jeiren", 1.0 },
  { "Jeiri", 0.0 },
  { "Jeiry", 0.0 },
  { "Jeison", 1.0 },
  { "Jeisy", 0.0 },
  { "Jeiza", 0.0 },
  { "Jejuan", 1.0 },
  { "Jekai", 1.0 },
  { "Jekalyn", 0.0 },
  { "Jekari", 1.0 },
  { "Jekayla", 0.0 },
  { "Jekeria", 0.0 },
  { "Jekhari", 1.0 },
  { "Jekhi", 1.0 },
  { "Jekia", 0.0 },
  { "Jekiah", 0.0 },
  { "Jekobe", 1.0 },
  { "Jekori", 1.0 },
  { "Jela", 0.0 },
  { "Jelah", 0.0 },
  { "Jelaina", 0.0 },
  { "Jelaine", 0.0 },
  { "Jelan", 1.0 },
  { "Jelana", 0.0 },
  { "Jelani", 0.9015544 },
  { "Jelanie", 0.0 },
  { "Jelany", 0.0 },
  { "Jelaya", 0.0 },
  { "Jelayne", 0.0 },
  { "Jeleah", 0.0 },
  { "Jelecia", 0.0 },
  { "Jeleesa", 0.0 },
  { "Jelen", 1.0 },
  { "Jelena", 0.0 },
  { "Jelene", 0.0 },
  { "Jelesa", 0.0 },
  { "Jelesia", 0.0 },
  { "Jelessa", 0.0 },
  { "Jelia", 0.0 },
  { "Jeliah", 0.0 },
  { "Jeliana", 0.0 },
  { "Jelica", 0.0 },
  { "Jelicia", 0.0 },
  { "Jeliel", 1.0 },
  { "Jelina", 0.0 },
  { "Jelisa", 0.0 },
  { "Jelise", 0.0 },
  { "Jelisha", 0.0 },
  { "Jelisia", 0.0 },
  { "Jelissa", 0.0 },
  { "Jelitza", 0.0 },
  { "Jeliyah", 0.0 },
  { "Jellisa", 0.0 },
  { "Jelon", 1.0 },
  { "Jeloni", 1.0 },
  { "Jelsy", 0.0 },
  { "Jelyn", 0.0 },
  { "Jelynn", 0.0 },
  { "Jelysa", 0.0 },
  { "Jelyssa", 0.0 },
  { "Jem", 0.34285715 },
  { "Jema", 0.0 },
  { "Jemaine", 1.0 },
  { "Jemal", 1.0 },
  { "Jemale", 1.0 },
  { "Jemar", 1.0 },
  { "Jemarcus", 1.0 },
  { "Jemari", 1.0 },
  { "Jemario", 1.0 },
  { "Jemarion", 1.0 },
  { "Jemarius", 1.0 },
  { "Jemarr", 1.0 },
  { "Jemauri", 1.0 },
  { "Jemaya", 0.0 },
  { "Jemeka", 0.0 },
  { "Jemekia", 0.0 },
  { "Jemel", 1.0 },
  { "Jemelia", 0.0 },
  { "Jemell", 1.0 },
  { "Jemelle", 1.0 },
  { "Jemely", 0.0 },
  { "Jemere", 1.0 },
  { "Jemeria", 0.0 },
  { "Jemerrio", 1.0 },
  { "Jemery", 1.0 },
  { "Jemes", 1.0 },
  { "Jemi", 0.0 },
  { "Jemia", 0.0 },
  { "Jemiah", 0.33673468 },
  { "Jemica", 0.0 },
  { "Jemichael", 1.0 },
  { "Jemier", 1.0 },
  { "Jemika", 0.0 },
  { "Jemil", 1.0 },
  { "Jemila", 0.0 },
  { "Jemilla", 0.0 },
  { "Jemily", 0.0 },
  { "Jemima", 0.0 },
  { "Jemimah", 0.0 },
  { "Jemina", 0.0 },
  { "Jemini", 0.24637681 },
  { "Jemir", 1.0 },
  { "Jemira", 0.0 },
  { "Jemiya", 0.0 },
  { "Jemiyah", 0.0 },
  { "Jemma", 0.0 },
  { "Jemmah", 0.0 },
  { "Jemmerio", 1.0 },
  { "Jemmie", 0.5472637 },
  { "Jemmy", 0.0 },
  { "Jemon", 1.0 },
  { "Jemond", 1.0 },
  { "Jemuel", 1.0 },
  { "Jemya", 0.0 },
  { "Jemyah", 0.0 },
  { "Jen", 0.21234567 },
  { "Jena", 0.0005395822 },
  { "Jenabelle", 0.0 },
  { "Jenacia", 0.0 },
  { "Jenae", 0.0 },
  { "Jenafer", 0.0 },
  { "Jenah", 0.0 },
  { "Jenai", 0.0 },
  { "Jenaia", 0.0 },
  { "Jenaiah", 0.0 },
  { "Jenaiya", 0.0 },
  { "Jenaka", 0.0 },
  { "Jenal", 0.0 },
  { "Jenalea", 0.0 },
  { "Jenalee", 0.0 },
  { "Jenali", 0.0 },
  { "Jenalis", 0.0 },
  { "Jenalise", 0.0 },
  { "Jenaliz", 0.0 },
  { "Jenalyn", 0.0 },
  { "Jenalynn", 0.0 },
  { "Jenalys", 0.0 },
  { "Jenalyse", 0.0 },
  { "Jenan", 0.0 },
  { "Jenanne", 0.0 },
  { "Jenara", 0.0 },
  { "Jenard", 1.0 },
  { "Jenaro", 1.0 },
  { "Jenascia", 0.0 },
  { "Jenasia", 0.0 },
  { "Jenasis", 0.0 },
  { "Jenasys", 0.0 },
  { "Jenave", 0.0 },
  { "Jenavecia", 0.0 },
  { "Jenavee", 0.0 },
  { "Jenaveve", 0.0 },
  { "Jenavi", 0.0 },
  { "Jenavicia", 0.0 },
  { "Jenavie", 0.0 },
  { "Jenavieve", 0.0 },
  { "Jenavive", 0.0 },
  { "Jenavy", 0.0 },
  { "Jenay", 0.00524659 },
  { "Jenaya", 0.0 },
  { "Jenayah", 0.0 },
  { "Jenaye", 0.0 },
  { "Jencarlo", 1.0 },
  { "Jencarlos", 1.0 },
  { "Jenci", 0.0 },
  { "Jencie", 0.0 },
  { "Jency", 0.046296295 },
  { "Jencyn", 0.0 },
  { "Jendaya", 0.0 },
  { "Jendayi", 0.0 },
  { "Jendriel", 1.0 },
  { "Jendry", 1.0 },
  { "Jene", 0.2912534 },
  { "Jenea", 0.0 },
  { "Jeneal", 0.0 },
  { "Jenean", 0.0 },
  { "Jeneane", 0.0 },
  { "Jeneanne", 0.0 },
  { "Jeneba", 0.0 },
  { "Jeneca", 0.0 },
  { "Jenece", 0.0 },
  { "Jenecia", 0.0 },
  { "Jenedy", 0.0 },
  { "Jenee", 0.0 },
  { "Jeneen", 0.0 },
  { "Jeneene", 0.0 },
  { "Jenefer", 0.0 },
  { "Jeneffer", 0.0 },
  { "Jeneice", 0.0 },
  { "Jeneil", 0.0 },
  { "Jeneine", 0.0 },
  { "Jeneka", 0.0 },
  { "Jenel", 0.0 },
  { "Jenele", 0.0 },
  { "Jenelee", 0.0 },
  { "Jenell", 0.0 },
  { "Jenella", 0.0 },
  { "Jenelle", 0.0 },
  { "Jenelly", 0.0 },
  { "Jenely", 0.0 },
  { "Jenene", 0.0 },
  { "Jenera", 0.0 },
  { "Jenesa", 0.0 },
  { "Jenese", 0.0 },
  { "Jenesha", 0.0 },
  { "Jenesi", 0.0 },
  { "Jenesia", 0.0 },
  { "Jenesis", 0.011825573 },
  { "Jeness", 0.0 },
  { "Jenessa", 0.0 },
  { "Jenesse", 0.0 },
  { "Jenessia", 0.0 },
  { "Jenessis", 0.0 },
  { "Jenessy", 0.0 },
  { "Jenesy", 0.0 },
  { "Jenesys", 0.0 },
  { "Jenet", 0.0 },
  { "Jenett", 0.0 },
  { "Jenetta", 0.0 },
  { "Jenette", 0.0 },
  { "Jenettie", 0.0 },
  { "Jeneva", 0.0 },
  { "Jenevi", 0.0 },
  { "Jenevie", 0.0 },
  { "Jenevieve", 0.0 },
  { "Jenevive", 0.0 },
  { "Jenevy", 0.0 },
  { "Jeneya", 0.0 },
  { "Jeni", 0.0 },
  { "Jenia", 0.0 },
  { "Jeniah", 0.0 },
  { "Jenibelle", 0.0 },
  { "Jenica", 0.0 },
  { "Jenice", 0.0 },
  { "Jenicia", 0.0 },
  { "Jenicka", 0.0 },
  { "Jenie", 0.0 },
  { "Jeniece", 0.0 },
  { "Jeniel", 1.0 },
  { "Jenielle", 0.0 },
  { "Jeniene", 0.0 },
  { "Jenifer", 0.0008658009 },
  { "Jeniffer", 0.0 },
  { "Jenika", 0.0 },
  { "Jenil", 1.0 },
  { "Jenilca", 0.0 },
  { "Jenilee", 0.0 },
  { "Jenille", 0.0 },
  { "Jenilyn", 0.0 },
  { "Jenin", 0.0 },
  { "Jenina", 0.0 },
  { "Jenine", 0.0 },
  { "Jeninne", 0.0 },
  { "Jenipher", 0.0 },
  { "Jeniqua", 0.0 },
  { "Jenique", 0.0 },
  { "Jenis", 0.0 },
  { "Jenisa", 0.0 },
  { "Jenise", 0.0 },
  { "Jenish", 1.0 },
  { "Jenisha", 0.0 },
  { "Jenisis", 0.0 },
  { "Jenissa", 0.0 },
  { "Jenisse", 0.0 },
  { "Jenisys", 0.0 },
  { "Jenita", 0.0 },
  { "Jenith", 0.0 },
  { "Jenitha", 0.0 },
  { "Jenitza", 0.0 },
  { "Jeniva", 0.0 },
  { "Jenive", 0.0 },
  { "Jeniveve", 0.0 },
  { "Jenivieve", 0.0 },
  { "Jeniya", 0.0 },
  { "Jeniyah", 0.0 },
  { "Jenkarlo", 1.0 },
  { "Jenkins", 1.0 },
  { "Jenn", 0.0 },
  { "Jenna", 0.0017886225 },
  { "Jennabel", 0.0 },
  { "Jennabella", 0.0 },
  { "Jennabelle", 0.0 },
  { "Jennaca", 0.0 },
  { "Jennae", 0.0 },
  { "Jennafer", 0.0 },
  { "Jennah", 0.0 },
  { "Jennaka", 0.0 },
  { "Jennalea", 0.0 },
  { "Jennalee", 0.0 },
  { "Jennaleigh", 0.0 },
  { "Jennalise", 0.0 },
  { "Jennaliz", 0.0 },
  { "Jennalyn", 0.0 },
  { "Jennalynn", 0.0 },
  { "Jennalyse", 0.0 },
  { "Jennamarie", 0.0 },
  { "Jennarae", 0.0 },
  { "Jennarose", 0.0 },
  { "Jennasis", 0.0 },
  { "Jennavecia", 0.0 },
  { "Jennavee", 0.0 },
  { "Jennaveve", 0.0 },
  { "Jennavi", 0.0 },
  { "Jennavicia", 0.0 },
  { "Jennavie", 0.0 },
  { "Jennavieve", 0.0 },
  { "Jennavive", 0.0 },
  { "Jennay", 0.0 },
  { "Jennaya", 0.0 },
  { "Jenne", 0.0 },
  { "Jennea", 0.0 },
  { "Jennean", 0.0 },
  { "Jennedy", 0.0 },
  { "Jennee", 0.0 },
  { "Jennefer", 0.0 },
  { "Jenneffer", 0.0 },
  { "Jenneh", 0.0 },
  { "Jenneifer", 0.0 },
  { "Jennel", 0.0 },
  { "Jennell", 0.0 },
  { "Jennelle", 0.0 },
  { "Jennely", 0.0 },
  { "Jennelyn", 0.0 },
  { "Jennene", 0.0 },
  { "Jenner", 0.9698795 },
  { "Jennesa", 0.0 },
  { "Jennesis", 0.0 },
  { "Jenness", 0.0 },
  { "Jennessa", 0.0 },
  { "Jennessy", 0.0 },
  { "Jennesy", 0.0 },
  { "Jennet", 0.0 },
  { "Jenneth", 0.0 },
  { "Jennett", 0.0 },
  { "Jennetta", 0.0 },
  { "Jennette", 0.0 },
  { "Jennevie", 0.0 },
  { "Jennevieve", 0.0 },
  { "Jennevy", 0.0 },
  { "Jenney", 0.0 },
  { "Jennfer", 0.0 },
  { "Jennfier", 0.0 },
  { "Jenni", 0.0 },
  { "Jennia", 0.0 },
  { "Jenniah", 0.0 },
  { "Jennica", 0.0 },
  { "Jennice", 0.0 },
  { "Jennicka", 0.0 },
  { "Jennie", 0.0037708222 },
  { "Jenniefer", 0.0 },
  { "Jennieke", 0.0 },
  { "Jennielee", 0.0 },
  { "Jennier", 0.0 },
  { "Jennieve", 0.0 },
  { "Jennife", 0.0 },
  { "Jennifer", 0.0032909752 },
  { "Jenniferann", 0.0 },
  { "Jenniferanne", 0.0 },
  { "Jennifere", 0.0 },
  { "Jenniferlee", 0.0 },
  { "Jenniferlynn", 0.0 },
  { "Jennifermarie", 0.0 },
  { "Jenniferr", 0.0 },
  { "Jenniferrose", 0.0 },
  { "Jenniffer", 0.0 },
  { "Jennifier", 0.0 },
  { "Jennifr", 0.0 },
  { "Jenniger", 0.0 },
  { "Jennika", 0.0 },
  { "Jennilee", 0.0 },
  { "Jennille", 0.0 },
  { "Jennilyn", 0.0 },
  { "Jennilynn", 0.0 },
  { "Jennine", 0.0 },
  { "Jenning", 1.0 },
  { "Jennings", 0.97045827 },
  { "Jenniper", 0.0 },
  { "Jennipher", 0.0 },
  { "Jennis", 0.59183675 },
  { "Jennisa", 0.0 },
  { "Jennise", 0.0 },
  { "Jenniver", 0.0 },
  { "Jenniya", 0.0 },
  { "Jenniyah", 0.0 },
  { "Jennnifer", 0.0 },
  { "Jenny", 0.0066433144 },
  { "Jennya", 0.0 },
  { "Jennye", 0.0 },
  { "Jennyfer", 0.0 },
  { "Jennylee", 0.0 },
  { "Jennylyn", 0.0 },
  { "Jennylynn", 0.0 },
  { "Jeno", 1.0 },
  { "Jenoah", 1.0 },
  { "Jenohn", 0.0 },
  { "Jenola", 0.0 },
  { "Jenora", 0.0 },
  { "Jenova", 0.0 },
  { "Jenri", 1.0 },
  { "Jenrri", 1.0 },
  { "Jenrry", 1.0 },
  { "Jenry", 1.0 },
  { "Jens", 1.0 },
  { "Jensen", 0.75850165 },
  { "Jensie", 0.0 },
  { "Jensiel", 1.0 },
  { "Jensin", 1.0 },
  { "Jensine", 0.0 },
  { "Jenson", 0.9815195 },
  { "Jenssy", 1.0 },
  { "Jensy", 0.6020408 },
  { "Jensyn", 0.16820702 },
  { "Jentezen", 1.0 },
  { "Jentri", 0.0 },
  { "Jentrie", 0.0 },
  { "Jentry", 0.2808425 },
  { "Jentzen", 1.0 },
  { "Jenuel", 1.0 },
  { "Jeny", 0.0 },
  { "Jenya", 0.0 },
  { "Jenyah", 0.0 },
  { "Jenyce", 0.0 },
  { "Jenyfer", 0.0 },
  { "Jenyiah", 0.0 },
  { "Jenyka", 0.0 },
  { "Jenyla", 0.0 },
  { "Jenysis", 0.0 },
  { "Jenyssa", 0.0 },
  { "Jenzel", 1.0 },
  { "Jenzen", 1.0 },
  { "Jenziel", 1.0 },
  { "Jeoffrey", 1.0 },
  { "Jeon", 1.0 },
  { "Jeong", 1.0 },
  { "Jeopardy", 0.0 },
  { "Jeorge", 1.0 },
  { "Jeorgia", 0.0 },
  { "Jeorgina", 0.0 },
  { "Jeovan", 1.0 },
  { "Jeovana", 0.0 },
  { "Jeovani", 1.0 },
  { "Jeovanna", 0.0 },
  { "Jeovanni", 1.0 },
  { "Jeovanny", 1.0 },
  { "Jeovany", 1.0 },
  { "Jep", 1.0 },
  { "Jephte", 1.0 },
  { "Jephthah", 1.0 },
  { "Jephthe", 1.0 },
  { "Jeppie", 1.0 },
  { "Jepson", 1.0 },
  { "Jeptha", 1.0 },
  { "Jequan", 1.0 },
  { "Jequarius", 1.0 },
  { "Jequetta", 0.0 },
  { "Jequita", 0.0 },
  { "Jer", 0.45454547 },
  { "Jera", 0.0 },
  { "Jerad", 1.0 },
  { "Jerade", 1.0 },
  { "Jerae", 0.08955224 },
  { "Jerah", 0.0 },
  { "Jerahmy", 1.0 },
  { "Jerail", 1.0 },
  { "Jeral", 0.9767981 },
  { "Jerald", 0.9960789 },
  { "Jeraldean", 0.0 },
  { "Jeraldene", 0.0 },
  { "Jeraldi", 0.0 },
  { "Jeraldin", 0.0 },
  { "Jeraldine", 0.0 },
  { "Jeraldo", 1.0 },
  { "Jeraldy", 0.0 },
  { "Jeraldyn", 0.0 },
  { "Jeralee", 0.0 },
  { "Jeralene", 0.0 },
  { "Jeraline", 0.0 },
  { "Jeralyn", 0.0 },
  { "Jeralynn", 0.0 },
  { "Jeramaine", 1.0 },
  { "Jerame", 1.0 },
  { "Jeramee", 1.0 },
  { "Jeramey", 1.0 },
  { "Jerami", 0.984252 },
  { "Jeramia", 1.0 },
  { "Jeramiah", 1.0 },
  { "Jeramie", 0.9879163 },
  { "Jeramih", 1.0 },
  { "Jeramiha", 1.0 },
  { "Jeramine", 1.0 },
  { "Jeramiyah", 1.0 },
  { "Jeramy", 0.99887717 },
  { "Jeramya", 1.0 },
  { "Jeramyah", 1.0 },
  { "Jeran", 0.9838384 },
  { "Jerane", 0.0 },
  { "Jerard", 1.0 },
  { "Jerardo", 1.0 },
  { "Jerauld", 1.0 },
  { "Jeray", 1.0 },
  { "Jerdean", 0.0 },
  { "Jerdine", 0.0 },
  { "Jere", 0.78643817 },
  { "Jereal", 1.0 },
  { "Jereck", 1.0 },
  { "Jered", 1.0 },
  { "Jerediah", 1.0 },
  { "Jeree", 0.024154589 },
  { "Jereimah", 1.0 },
  { "Jerek", 1.0 },
  { "Jerel", 0.99858034 },
  { "Jereld", 1.0 },
  { "Jereldine", 0.0 },
  { "Jerelean", 0.0 },
  { "Jerelene", 0.0 },
  { "Jerelin", 0.0 },
  { "Jereline", 0.0 },
  { "Jerell", 0.99689054 },
  { "Jerelle", 0.9396985 },
  { "Jerelly", 0.0 },
  { "Jerely", 0.0 },
  { "Jerelyn", 0.0 },
  { "Jerem", 1.0 },
  { "Jeremah", 1.0 },
  { "Jeremai", 1.0 },
  { "Jeremaiah", 1.0 },
  { "Jeremaih", 1.0 },
  { "Jeremaine", 1.0 },
  { "Jereme", 1.0 },
  { "Jeremee", 1.0 },
  { "Jeremery", 1.0 },
  { "Jeremey", 1.0 },
  { "Jeremi", 0.9719567 },
  { "Jeremia", 0.98435056 },
  { "Jeremiah", 0.9965228 },
  { "Jeremiaha", 1.0 },
  { "Jeremiahs", 1.0 },
  { "Jeremial", 1.0 },
  { "Jeremian", 1.0 },
  { "Jeremias", 1.0 },
  { "Jeremie", 0.98333335 },
  { "Jeremih", 1.0 },
  { "Jeremiha", 1.0 },
  { "Jeremiya", 1.0 },
  { "Jeremiyah", 1.0 },
  { "Jeremmy", 1.0 },
  { "Jeremy", 0.99470204 },
  { "Jeremya", 0.7307692 },
  { "Jeremyah", 1.0 },
  { "Jeremyh", 1.0 },
  { "Jeremyiah", 1.0 },
  { "Jeren", 1.0 },
  { "Jerena", 0.0 },
  { "Jerene", 0.0 },
  { "Jereny", 1.0 },
  { "Jereomy", 1.0 },
  { "Jeresa", 0.0 },
  { "Jeresiah", 1.0 },
  { "Jeret", 1.0 },
  { "Jereth", 1.0 },
  { "Jerett", 1.0 },
  { "Jeretta", 0.0 },
  { "Jerey", 1.0 },
  { "Jerez", 1.0 },
  { "Jerhonda", 0.0 },
  { "Jeri", 0.016459288 },
  { "Jeria", 0.0 },
  { "Jeriah", 0.7974203 },
  { "Jerian", 1.0 },
  { "Jeriana", 0.0 },
  { "Jeriann", 0.0 },
  { "Jerianna", 0.0 },
  { "Jerianne", 0.0 },
  { "Jeric", 1.0 },
  { "Jerica", 0.0 },
  { "Jericah", 0.0 },
  { "Jericca", 0.0 },
  { "Jericha", 0.0 },
  { "Jericho", 0.938564 },
  { "Jericia", 0.0 },
  { "Jerick", 1.0 },
  { "Jericka", 0.0 },
  { "Jericko", 1.0 },
  { "Jerico", 0.9898089 },
  { "Jerid", 1.0 },
  { "Jerie", 0.0 },
  { "Jeriel", 1.0 },
  { "Jeries", 1.0 },
  { "Jeriesha", 0.0 },
  { "Jerik", 1.0 },
  { "Jerika", 0.0 },
  { "Jerikah", 0.0 },
  { "Jerikka", 0.0 },
  { "Jeriko", 1.0 },
  { "Jerilee", 0.0 },
  { "Jerilene", 0.0 },
  { "Jerilou", 0.0 },
  { "Jerilyn", 0.0 },
  { "Jerilynn", 0.0 },
  { "Jerilynne", 0.0 },
  { "Jerimah", 1.0 },
  { "Jerimaine", 1.0 },
  { "Jerime", 1.0 },
  { "Jerimee", 1.0 },
  { "Jerimey", 1.0 },
  { "Jerimi", 1.0 },
  { "Jerimia", 1.0 },
  { "Jerimiah", 1.0 },
  { "Jerimie", 1.0 },
  { "Jerimih", 1.0 },
  { "Jerimiha", 1.0 },
  { "Jerimiyah", 1.0 },
  { "Jerimy", 1.0 },
  { "Jerimya", 1.0 },
  { "Jerimyah", 1.0 },
  { "Jerin", 0.9083192 },
  { "Jerina", 0.0 },
  { "Jerine", 0.0 },
  { "Jeriod", 1.0 },
  { "Jerion", 1.0 },
  { "Jerious", 1.0 },
  { "Jeris", 0.59861594 },
  { "Jerisha", 0.0 },
  { "Jerison", 1.0 },
  { "Jerissa", 0.0 },
  { "Jerit", 1.0 },
  { "Jerita", 0.0 },
  { "Jerith", 1.0 },
  { "Jerius", 1.0 },
  { "Jeriyah", 0.0 },
  { "Jerjuan", 1.0 },
  { "Jerkayla", 0.0 },
  { "Jerkiya", 0.0 },
  { "Jerl", 1.0 },
  { "Jerldean", 0.0 },
  { "Jerldine", 0.0 },
  { "Jerlean", 0.0 },
  { "Jerleen", 0.0 },
  { "Jerlene", 0.0 },
  { "Jerlin", 0.0 },
  { "Jerline", 0.0 },
  { "Jerlisa", 0.0 },
  { "Jerlisha", 0.0 },
  { "Jerlyn", 0.0 },
  { "Jermail", 1.0 },
  { "Jermain", 1.0 },
  { "Jermaine", 0.97660524 },
  { "Jermal", 1.0 },
  { "Jermale", 1.0 },
  { "Jermall", 1.0 },
  { "Jerman", 1.0 },
  { "Jermane", 1.0 },
  { "Jermanee", 0.0 },
  { "Jermani", 0.18526785 },
  { "Jermanie", 0.8828125 },
  { "Jermanii", 0.0 },
  { "Jermanine", 1.0 },
  { "Jermany", 0.46606335 },
  { "Jermar", 1.0 },
  { "Jermarcus", 1.0 },
  { "Jermari", 1.0 },
  { "Jermaria", 0.0 },
  { "Jermario", 1.0 },
  { "Jermarion", 1.0 },
  { "Jermarius", 1.0 },
  { "Jermarkus", 1.0 },
  { "Jermaul", 1.0 },
  { "Jermaya", 0.0 },
  { "Jermayne", 1.0 },
  { "Jerme", 1.0 },
  { "Jermecia", 0.0 },
  { "Jermeisha", 0.0 },
  { "Jermeka", 0.0 },
  { "Jermel", 0.996136 },
  { "Jermell", 1.0 },
  { "Jermelle", 1.0 },
  { "Jermere", 1.0 },
  { "Jermeria", 0.0 },
  { "Jermery", 1.0 },
  { "Jermesha", 0.0 },
  { "Jermeshia", 0.0 },
  { "Jermey", 1.0 },
  { "Jermia", 0.0 },
  { "Jermiah", 0.9596514 },
  { "Jermiane", 1.0 },
  { "Jermica", 0.0 },
  { "Jermichael", 1.0 },
  { "Jermicheal", 1.0 },
  { "Jermie", 1.0 },
  { "Jermih", 1.0 },
  { "Jermika", 0.0 },
  { "Jermil", 1.0 },
  { "Jermine", 1.0 },
  { "Jermir", 1.0 },
  { "Jermira", 0.0 },
  { "Jermisha", 0.0 },
  { "Jermiya", 0.0 },
  { "Jermiyah", 0.09550562 },
  { "Jermol", 1.0 },
  { "Jermon", 1.0 },
  { "Jermond", 1.0 },
  { "Jermone", 1.0 },
  { "Jermonica", 0.0 },
  { "Jermont", 1.0 },
  { "Jermonte", 1.0 },
  { "Jermy", 1.0 },
  { "Jermya", 0.0 },
  { "Jermyah", 0.5135135 },
  { "Jermyia", 0.0 },
  { "Jermyn", 1.0 },
  { "Jermyra", 0.0 },
  { "Jernae", 0.0 },
  { "Jernard", 1.0 },
  { "Jernee", 0.0 },
  { "Jernei", 0.0 },
  { "Jernell", 0.18518518 },
  { "Jerneshia", 0.0 },
  { "Jerney", 0.071428575 },
  { "Jerni", 0.0 },
  { "Jerniah", 0.0 },
  { "Jernice", 0.0 },
  { "Jernie", 0.0 },
  { "Jernika", 0.0 },
  { "Jernisha", 0.0 },
  { "Jerniya", 0.0 },
  { "Jerniyah", 0.0 },
  { "Jernya", 0.0 },
  { "Jerod", 1.0 },
  { "Jerode", 1.0 },
  { "Jeroen", 1.0 },
  { "Jerol", 0.9305556 },
  { "Jerold", 1.0 },
  { "Jeroldine", 0.0 },
  { "Jerolene", 0.0 },
  { "Jeroline", 0.0 },
  { "Jerolyn", 0.0 },
  { "Jerom", 1.0 },
  { "Jeroma", 0.0 },
  { "Jerome", 0.9933878 },
  { "Jeromee", 1.0 },
  { "Jeromey", 1.0 },
  { "Jeromi", 1.0 },
  { "Jeromiah", 1.0 },
  { "Jeromie", 1.0 },
  { "Jeromy", 1.0 },
  { "Jeron", 1.0 },
  { "Jerona", 0.0 },
  { "Jeronda", 0.0 },
  { "Jerone", 1.0 },
  { "Jeronica", 0.0 },
  { "Jeronimo", 1.0 },
  { "Jerquan", 1.0 },
  { "Jerra", 0.0 },
  { "Jerrad", 1.0 },
  { "Jerrae", 0.0 },
  { "Jerrah", 0.0 },
  { "Jerrail", 1.0 },
  { "Jerral", 1.0 },
  { "Jerrald", 1.0 },
  { "Jerralee", 0.0 },
  { "Jerrall", 1.0 },
  { "Jerralyn", 0.0 },
  { "Jerrame", 1.0 },
  { "Jerramie", 1.0 },
  { "Jerramy", 1.0 },
  { "Jerran", 1.0 },
  { "Jerrard", 1.0 },
  { "Jerray", 1.0 },
  { "Jerre", 0.56303114 },
  { "Jerred", 1.0 },
  { "Jerrel", 0.98934865 },
  { "Jerrell", 0.9870592 },
  { "Jerrelle", 1.0 },
  { "Jerremiah", 1.0 },
  { "Jerremy", 1.0 },
  { "Jerren", 1.0 },
  { "Jerrene", 0.0 },
  { "Jerret", 1.0 },
  { "Jerrett", 1.0 },
  { "Jerri", 0.017774098 },
  { "Jerria", 0.0 },
  { "Jerriah", 0.0 },
  { "Jerrian", 1.0 },
  { "Jerriana", 0.0 },
  { "Jerriann", 0.0 },
  { "Jerrianna", 0.0 },
  { "Jerrianne", 0.0 },
  { "Jerric", 1.0 },
  { "Jerrica", 0.0 },
  { "Jerricca", 0.0 },
  { "Jerricha", 0.0 },
  { "Jerricho", 1.0 },
  { "Jerrick", 1.0 },
  { "Jerricka", 0.0 },
  { "Jerrico", 1.0 },
  { "Jerrid", 1.0 },
  { "Jerrie", 0.061651178 },
  { "Jerriel", 1.0 },
  { "Jerrik", 1.0 },
  { "Jerrika", 0.0 },
  { "Jerril", 1.0 },
  { "Jerrilee", 0.0 },
  { "Jerrill", 1.0 },
  { "Jerrilyn", 0.0 },
  { "Jerrilyne", 0.0 },
  { "Jerrilynn", 0.0 },
  { "Jerrime", 1.0 },
  { "Jerrimiah", 1.0 },
  { "Jerrimy", 1.0 },
  { "Jerrin", 0.96767676 },
  { "Jerrina", 0.0 },
  { "Jerrine", 0.0 },
  { "Jerrion", 1.0 },
  { "Jerriona", 0.0 },
  { "Jerrious", 1.0 },
  { "Jerris", 0.58 },
  { "Jerrisa", 0.0 },
  { "Jerrisha", 0.0 },
  { "Jerrison", 1.0 },
  { "Jerrit", 1.0 },
  { "Jerrita", 0.0 },
  { "Jerritt", 1.0 },
  { "Jerrius", 1.0 },
  { "Jerriyah", 0.0 },
  { "Jerrlyn", 0.0 },
  { "Jerrod", 0.9989139 },
  { "Jerrol", 1.0 },
  { "Jerrold", 1.0 },
  { "Jerroll", 1.0 },
  { "Jerrolyn", 0.0 },
  { "Jerrome", 1.0 },
  { "Jerron", 1.0 },
  { "Jerrud", 1.0 },
  { "Jerry", 0.97248685 },
  { "Jerryca", 0.0 },
  { "Jerryd", 1.0 },
  { "Jerrye", 0.0 },
  { "Jerryl", 0.9607843 },
  { "Jerrylee", 1.0 },
  { "Jerrylene", 0.0 },
  { "Jerryn", 1.0 },
  { "Jersain", 1.0 },
  { "Jersee", 0.0 },
  { "Jersei", 0.0 },
  { "Jersen", 1.0 },
  { "Jersey", 0.07765668 },
  { "Jersi", 0.0 },
  { "Jersiah", 1.0 },
  { "Jersie", 0.0 },
  { "Jerson", 1.0 },
  { "Jersy", 0.0 },
  { "Jertavious", 1.0 },
  { "Jeru", 1.0 },
  { "Jerud", 1.0 },
  { "Jerusalem", 0.2166065 },
  { "Jerusalen", 0.0 },
  { "Jerusha", 0.0 },
  { "Jervaughn", 1.0 },
  { "Jervin", 1.0 },
  { "Jervis", 1.0 },
  { "Jervon", 1.0 },
  { "Jervontae", 1.0 },
  { "Jervonte", 1.0 },
  { "Jerwan", 1.0 },
  { "Jerwin", 1.0 },
  { "Jery", 1.0 },
  { "Jeryd", 1.0 },
  { "Jeryiah", 0.0 },
  { "Jeryka", 0.0 },
  { "Jerykah", 0.0 },
  { "Jeryl", 0.43054786 },
  { "Jeryme", 1.0 },
  { "Jerymiah", 1.0 },
  { "Jeryn", 0.6822917 },
  { "Jerzee", 0.0 },
  { "Jerzei", 0.0 },
  { "Jerzey", 0.029940119 },
  { "Jerzi", 0.0 },
  { "Jerzie", 0.0 },
  { "Jerzy", 0.48387095 },
  { "Jes", 1.0 },
  { "Jesa", 0.0 },
  { "Jesabel", 0.0 },
  { "Jesai", 1.0 },
  { "Jesaiah", 1.0 },
  { "Jesaias", 1.0 },
  { "Jesalee", 0.0 },
  { "Jesalyn", 0.0 },
  { "Jesalynn", 0.0 },
  { "Jesara", 0.0 },
  { "Jescenia", 0.0 },
  { "Jescie", 0.0 },
  { "Jesco", 1.0 },
  { "Jese", 1.0 },
  { "Jesean", 1.0 },
  { "Jeseca", 0.0 },
  { "Jesee", 0.8863636 },
  { "Jeseka", 0.0 },
  { "Jesel", 0.0 },
  { "Jeselle", 0.0 },
  { "Jeselyn", 0.0 },
  { "Jesenia", 0.0013057671 },
  { "Jesenya", 0.0 },
  { "Jeser", 1.0 },
  { "Jesha", 0.0 },
  { "Jeshaiah", 1.0 },
  { "Jeshaun", 1.0 },
  { "Jeshawn", 1.0 },
  { "Jeshia", 0.0 },
  { "Jeshon", 1.0 },
  { "Jeshua", 0.99774873 },
  { "Jeshuah", 1.0 },
  { "Jeshurun", 1.0 },
  { "Jeshwa", 1.0 },
  { "Jesi", 0.06495727 },
  { "Jesiah", 0.91190666 },
  { "Jesica", 0.0 },
  { "Jesicah", 0.0 },
  { "Jesicca", 0.0 },
  { "Jesie", 0.40625 },
  { "Jesiel", 1.0 },
  { "Jesika", 0.0 },
  { "Jesikah", 0.0 },
  { "Jesilyn", 0.0 },
  { "Jesiree", 0.0 },
  { "Jesiyah", 1.0 },
  { "Jeslee", 0.0 },
  { "Jesleen", 0.0 },
  { "Jeslene", 0.0 },
  { "Jesley", 0.0 },
  { "Jesli", 0.0 },
  { "Jeslie", 0.0 },
  { "Jeslin", 0.0 },
  { "Jesly", 0.0 },
  { "Jeslyn", 0.0 },
  { "Jeslynn", 0.0 },
  { "Jesmarie", 0.0 },
  { "Jesmin", 0.0 },
  { "Jesmine", 0.0 },
  { "Jeson", 1.0 },
  { "Jesper", 1.0 },
  { "Jess", 0.9830314 },
  { "Jessa", 0.0010072523 },
  { "Jessabel", 0.0 },
  { "Jessabell", 0.0 },
  { "Jessabella", 0.0 },
  { "Jessabelle", 0.0 },
  { "Jessaca", 0.0 },
  { "Jessah", 0.0 },
  { "Jessalee", 0.0 },
  { "Jessalin", 0.0 },
  { "Jessalyn", 0.0 },
  { "Jessalynn", 0.0 },
  { "Jessalynne", 0.0 },
  { "Jessamae", 0.0 },
  { "Jessamarie", 0.0 },
  { "Jessamine", 0.0 },
  { "Jessamy", 0.0 },
  { "Jessamyn", 0.007002801 },
  { "Jessca", 0.0 },
  { "Jesscia", 0.0 },
  { "Jesse", 0.9721773 },
  { "Jesseca", 0.0 },
  { "Jessee", 0.8418848 },
  { "Jesseica", 0.0 },
  { "Jessejames", 1.0 },
  { "Jesseka", 0.0 },
  { "Jessel", 0.05586592 },
  { "Jesselee", 1.0 },
  { "Jesselin", 0.0 },
  { "Jessell", 0.0 },
  { "Jesselle", 0.0 },
  { "Jessely", 0.0 },
  { "Jesselyn", 0.0 },
  { "Jesselynn", 0.0 },
  { "Jessen", 1.0 },
  { "Jessena", 0.0 },
  { "Jessenia", 0.0 },
  { "Jessenya", 0.0 },
  { "Jesseray", 1.0 },
  { "Jesses", 1.0 },
  { "Jessey", 0.76349616 },
  { "Jessi", 0.17531404 },
  { "Jessia", 0.0 },
  { "Jessiah", 0.8348559 },
  { "Jessiana", 0.0 },
  { "Jessianna", 0.0 },
  { "Jessianne", 0.0 },
  { "Jessic", 0.0 },
  { "Jessica", 0.0034148172 },
  { "Jessicaann", 0.0 },
  { "Jessicaanne", 0.0 },
  { "Jessicah", 0.0 },
  { "Jessical", 0.0 },
  { "Jessicalee", 0.0 },
  { "Jessicalynn", 0.0 },
  { "Jessicamae", 0.0 },
  { "Jessicamarie", 0.0 },
  { "Jessican", 0.0 },
  { "Jessicanicole", 0.0 },
  { "Jessicarose", 0.0 },
  { "Jessicca", 0.0 },
  { "Jessice", 0.0 },
  { "Jessicia", 0.0 },
  { "Jessicka", 0.0 },
  { "Jessico", 0.0 },
  { "Jessie", 0.39734405 },
  { "Jessieca", 0.0 },
  { "Jessiejames", 1.0 },
  { "Jessiel", 1.0 },
  { "Jessielynn", 0.0 },
  { "Jessiemae", 0.0 },
  { "Jessika", 0.0 },
  { "Jessikah", 0.0 },
  { "Jessilyn", 0.0 },
  { "Jessilynn", 0.0 },
  { "Jessimae", 0.0 },
  { "Jessimar", 0.0 },
  { "Jessina", 0.0 },
  { "Jessinia", 0.0 },
  { "Jessiqua", 0.0 },
  { "Jessiyah", 0.0 },
  { "Jesska", 0.0 },
  { "Jesslene", 0.0 },
  { "Jesslin", 0.0 },
  { "Jessly", 0.0 },
  { "Jesslyn", 0.0 },
  { "Jesslynn", 0.0 },
  { "Jessmarie", 0.0 },
  { "Jesson", 1.0 },
  { "Jesssica", 0.0 },
  { "Jessten", 1.0 },
  { "Jesstin", 1.0 },
  { "Jesstina", 0.0 },
  { "Jesstine", 0.0 },
  { "Jesston", 1.0 },
  { "Jessuly", 0.0 },
  { "Jessup", 1.0 },
  { "Jessus", 1.0 },
  { "Jessy", 0.71472 },
  { "Jessyca", 0.0 },
  { "Jessye", 0.0 },
  { "Jessyka", 0.0 },
  { "Jesteen", 0.0 },
  { "Jesten", 1.0 },
  { "Jestene", 0.0 },
  { "Jester", 1.0 },
  { "Jestin", 1.0 },
  { "Jestina", 0.0 },
  { "Jestine", 0.0 },
  { "Jestiny", 0.0 },
  { "Jeston", 1.0 },
  { "Jestyn", 1.0 },
  { "Jesu", 1.0 },
  { "Jesua", 1.0 },
  { "Jesualdo", 1.0 },
  { "Jesuan", 1.0 },
  { "Jesucita", 0.0 },
  { "Jesue", 1.0 },
  { "Jesus", 0.98826706 },
  { "Jesusa", 0.0 },
  { "Jesusalberto", 1.0 },
  { "Jesusangel", 1.0 },
  { "Jesusantonio", 1.0 },
  { "Jesusdaniel", 1.0 },
  { "Jesusenrique", 1.0 },
  { "Jesusita", 0.0 },
  { "Jesusjr", 1.0 },
  { "Jesusmanuel", 1.0 },
  { "Jesy", 0.6666667 },
  { "Jesyca", 0.0 },
  { "Jesyka", 0.0 },
  { "Jet", 0.9700338 },
  { "Jetaime", 0.0 },
  { "Jetaun", 0.0 },
  { "Jete", 0.0 },
  { "Jeter", 1.0 },
  { "Jeth", 1.0 },
  { "Jethero", 1.0 },
  { "Jethro", 1.0 },
  { "Jethroe", 1.0 },
  { "Jetli", 1.0 },
  { "Jetlyn", 0.0 },
  { "Jetmir", 1.0 },
  { "Jetoi", 0.0 },
  { "Jeton", 1.0 },
  { "Jetsen", 1.0 },
  { "Jetson", 1.0 },
  { "Jett", 0.9717242 },
  { "Jetta", 0.0 },
  { "Jette", 0.0625 },
  { "Jettie", 0.050526317 },
  { "Jetton", 1.0 },
  { "Jettrin", 1.0 },
  { "Jettsen", 1.0 },
  { "Jettson", 1.0 },
  { "Jetty", 0.023622047 },
  { "Jettye", 0.0 },
  { "Jetzabel", 0.0 },
  { "Jetzael", 1.0 },
  { "Jetzibe", 0.0 },
  { "Jetziel", 1.0 },
  { "Jeudy", 1.0 },
  { "Jeune", 0.0 },
  { "Jeury", 1.0 },
  { "Jevaeh", 0.0 },
  { "Jevan", 1.0 },
  { "Jevante", 1.0 },
  { "Jevaughn", 1.0 },
  { "Jevaun", 1.0 },
  { "Jeveah", 0.0 },
  { "Jeven", 1.0 },
  { "Jeveon", 1.0 },
  { "Jevette", 0.0 },
  { "Jevin", 1.0 },
  { "Jevion", 1.0 },
  { "Jevita", 0.0 },
  { "Jevon", 0.98083806 },
  { "Jevonda", 0.0 },
  { "Jevone", 1.0 },
  { "Jevonn", 1.0 },
  { "Jevonna", 0.0 },
  { "Jevonne", 0.0 },
  { "Jevonta", 1.0 },
  { "Jevontae", 1.0 },
  { "Jevontay", 1.0 },
  { "Jevonte", 1.0 },
  { "Jevyn", 1.0 },
  { "Jewan", 1.0 },
  { "Jewel", 0.11559188 },
  { "Jeweldean", 0.0 },
  { "Jeweldene", 0.0 },
  { "Jeweldine", 0.0 },
  { "Jeweleah", 0.0 },
  { "Jewelean", 0.0 },
  { "Jewelee", 0.0 },
  { "Jewelene", 0.0 },
  { "Jewelia", 0.0 },
  { "Jewelian", 0.41666666 },
  { "Jeweliana", 0.0 },
  { "Jewelianna", 0.0 },
  { "Jewelie", 0.0 },
  { "Jeweliet", 0.0 },
  { "Jeweline", 0.0 },
  { "Jewelisa", 0.0 },
  { "Jewelissa", 0.0 },
  { "Jewell", 0.13669108 },
  { "Jewelle", 0.0 },
  { "Jewellee", 0.0 },
  { "Jewellene", 0.0 },
  { "Jewellia", 0.0 },
  { "Jewels", 0.008859358 },
  { "Jewely", 0.0 },
  { "Jewelya", 0.0 },
  { "Jewelyn", 0.0 },
  { "Jewelyssa", 0.0 },
  { "Jewelz", 0.18076923 },
  { "Jewett", 1.0 },
  { "Jewl", 0.0 },
  { "Jewlia", 0.0 },
  { "Jewlian", 1.0 },
  { "Jewliana", 0.0 },
  { "Jewlius", 1.0 },
  { "Jewlz", 0.0 },
  { "Jex", 1.0 },
  { "Jexiel", 1.0 },
  { "Jexton", 1.0 },
  { "Jey", 1.0 },
  { "Jeyceon", 1.0 },
  { "Jeycob", 1.0 },
  { "Jeyda", 0.0 },
  { "Jeydan", 1.0 },
  { "Jeyden", 1.0 },
  { "Jeydi", 0.0 },
  { "Jeydon", 1.0 },
  { "Jeydy", 0.0 },
  { "Jeyko", 1.0 },
  { "Jeyla", 0.0 },
  { "Jeylah", 0.0 },
  { "Jeylan", 0.78571427 },
  { "Jeylani", 0.54545456 },
  { "Jeylanie", 0.0 },
  { "Jeyleen", 0.0 },
  { "Jeylen", 0.29411766 },
  { "Jeylene", 0.0 },
  { "Jeyli", 0.0 },
  { "Jeylianis", 0.0 },
  { "Jeylin", 0.0 },
  { "Jeyline", 0.0 },
  { "Jeylon", 1.0 },
  { "Jeyly", 0.0 },
  { "Jeymi", 0.0 },
  { "Jeyren", 1.0 },
  { "Jeysen", 1.0 },
  { "Jeyshangelise", 0.0 },
  { "Jeyson", 1.0 },
  { "Jezabel", 0.0 },
  { "Jezabell", 0.0 },
  { "Jezabella", 0.0 },
  { "Jezabelle", 0.0 },
  { "Jezebel", 0.0 },
  { "Jezebell", 0.0 },
  { "Jezebelle", 0.0 },
  { "Jezel", 0.0 },
  { "Jezell", 0.0 },
  { "Jezelle", 0.0 },
  { "Jezenia", 0.0 },
  { "Jezeniah", 0.0 },
  { "Jezer", 1.0 },
  { "Jeziah", 0.93351066 },
  { "Jeziel", 1.0 },
  { "Jezika", 0.0 },
  { "Jeziyah", 0.0 },
  { "Jezlin", 0.0 },
  { "Jezlyn", 0.0 },
  { "Jezlynn", 0.0 },
  { "Jezrael", 1.0 },
  { "Jezreel", 0.7628866 },
  { "Jezzel", 0.0 },
  { "Jezzelle", 0.0 },
  { "Jguadalupe", 1.0 },
  { "Jhace", 1.0 },
  { "Jhada", 0.0 },
  { "Jhade", 0.0 },
  { "Jhaden", 1.0 },
  { "Jhadiel", 1.0 },
  { "Jhaeda", 0.0 },
  { "Jhael", 1.0 },
  { "Jhai", 0.76744187 },
  { "Jhaiden", 1.0 },
  { "Jhair", 1.0 },
  { "Jhaki", 1.0 },
  { "Jhala", 0.0 },
  { "Jhalen", 1.0 },
  { "Jhalia", 0.0 },
  { "Jhalil", 1.0 },
  { "Jhalin", 1.0 },
  { "Jhalyn", 1.0 },
  { "Jhamal", 1.0 },
  { "Jhamar", 1.0 },
  { "Jhamari", 1.0 },
  { "Jhamarion", 1.0 },
  { "Jhamel", 1.0 },
  { "Jhamere", 1.0 },
  { "Jhamilet", 0.0 },
  { "Jhamir", 1.0 },
  { "Jhamya", 0.0 },
  { "Jhan", 0.9700599 },
  { "Jhana", 0.0 },
  { "Jhanae", 0.0 },
  { "Jhanai", 0.0 },
  { "Jhancarlos", 1.0 },
  { "Jhane", 0.0 },
  { "Jhanee", 0.0 },
  { "Jhanel", 0.0 },
  { "Jhanelle", 0.0 },
  { "Jhanet", 0.0 },
  { "Jhani", 0.0 },
  { "Jhania", 0.0 },
  { "Jhaniah", 0.0 },
  { "Jhaniya", 0.0 },
  { "Jhaniyah", 0.0 },
  { "Jhanna", 0.0 },
  { "Jhanvi", 0.0 },
  { "Jhanya", 0.0 },
  { "Jhared", 1.0 },
  { "Jhari", 1.0 },
  { "Jhariel", 1.0 },
  { "Jharline", 0.0 },
  { "Jhase", 1.0 },
  { "Jhasir", 1.0 },
  { "Jhasmin", 0.0 },
  { "Jhasmine", 0.0 },
  { "Jhavier", 1.0 },
  { "Jhavon", 1.0 },
  { "Jhayce", 1.0 },
  { "Jhayda", 0.0 },
  { "Jhayden", 0.9649123 },
  { "Jhayla", 0.0 },
  { "Jhaylen", 1.0 },
  { "Jhaziel", 1.0 },
  { "Jhena", 0.0 },
  { "Jhenae", 0.0 },
  { "Jhene", 0.0 },
  { "Jhenesis", 0.0 },
  { "Jhenifer", 0.0 },
  { "Jhenna", 0.0 },
  { "Jheremy", 1.0 },
  { "Jheri", 0.0 },
  { "Jherica", 0.0 },
  { "Jherika", 0.0 },
  { "Jherrica", 0.0 },
  { "Jheryl", 1.0 },
  { "Jhet", 1.0 },
  { "Jhett", 0.9930939 },
  { "Jhia", 0.0 },
  { "Jhiya", 0.0 },
  { "Jhoan", 1.0 },
  { "Jhoana", 0.0 },
  { "Jhoanna", 0.0 },
  { "Jhobany", 1.0 },
  { "Jhoel", 1.0 },
  { "Jhomar", 1.0 },
  { "Jhon", 1.0 },
  { "Jhonae", 0.0 },
  { "Jhonael", 1.0 },
  { "Jhonas", 1.0 },
  { "Jhonatan", 1.0 },
  { "Jhonathan", 1.0 },
  { "Jhonathon", 1.0 },
  { "Jhonattan", 1.0 },
  { "Jhonen", 1.0 },
  { "Jhonie", 1.0 },
  { "Jhoniel", 1.0 },
  { "Jhonna", 0.0 },
  { "Jhonnatan", 1.0 },
  { "Jhonni", 0.0 },
  { "Jhonnie", 0.91061455 },
  { "Jhonny", 1.0 },
  { "Jhony", 1.0 },
  { "Jhordan", 0.688 },
  { "Jhordy", 1.0 },
  { "Jhordyn", 0.12931034 },
  { "Jhori", 0.0 },
  { "Jhoselin", 0.0 },
  { "Jhoseline", 0.0 },
  { "Jhoselyn", 0.0 },
  { "Jhoselyne", 0.0 },
  { "Jhosep", 1.0 },
  { "Jhoseph", 1.0 },
  { "Jhoshua", 1.0 },
  { "Jhosmar", 1.0 },
  { "Jhosselyn", 0.0 },
  { "Jhostin", 1.0 },
  { "Jhosue", 1.0 },
  { "Jhournee", 0.0 },
  { "Jhourni", 0.0 },
  { "Jhovan", 1.0 },
  { "Jhovana", 0.0 },
  { "Jhovani", 1.0 },
  { "Jhovanna", 0.0 },
  { "Jhovanni", 1.0 },
  { "Jhovanny", 1.0 },
  { "Jhovany", 1.0 },
  { "Jhue", 1.0 },
  { "Ji", 0.3872549 },
  { "Jia", 0.15296641 },
  { "Jiaan", 1.0 },
  { "Jiacheng", 1.0 },
  { "Jiada", 0.0 },
  { "Jiah", 0.0 },
  { "Jiahao", 1.0 },
  { "Jiahna", 0.0 },
  { "Jiair", 1.0 },
  { "Jiaire", 1.0 },
  { "Jiali", 0.0 },
  { "Jialin", 0.0 },
  { "Jiamei", 0.0 },
  { "Jiaming", 1.0 },
  { "Jian", 0.9440559 },
  { "Jiana", 0.0 },
  { "Jianah", 0.0 },
  { "Jiancarlo", 1.0 },
  { "Jiancarlos", 1.0 },
  { "Jiani", 0.0 },
  { "Jianna", 0.0 },
  { "Jiannah", 0.0 },
  { "Jianne", 0.0 },
  { "Jianni", 0.5230126 },
  { "Jianny", 1.0 },
  { "Jiapsi", 0.0 },
  { "Jiaqi", 0.0 },
  { "Jiar", 1.0 },
  { "Jiara", 0.0 },
  { "Jiarui", 1.0 },
  { "Jiavanna", 0.0 },
  { "Jiaxin", 0.0 },
  { "Jiaya", 0.0 },
  { "Jiayi", 0.22105263 },
  { "Jiayue", 0.0 },
  { "Jibrael", 1.0 },
  { "Jibrail", 1.0 },
  { "Jibran", 1.0 },
  { "Jibreal", 1.0 },
  { "Jibreel", 1.0 },
  { "Jibri", 1.0 },
  { "Jibria", 0.0 },
  { "Jibriel", 1.0 },
  { "Jibril", 1.0 },
  { "Jicel", 0.0 },
  { "Jicela", 0.0 },
  { "Jidenna", 1.0 },
  { "Jie", 0.8269231 },
  { "Jiesha", 0.0 },
  { "Jieun", 0.0 },
  { "Jigar", 1.0 },
  { "Jiggs", 1.0 },
  { "Jigme", 1.0 },
  { "Jigna", 0.0 },
  { "Jihaad", 1.0 },
  { "Jihad", 0.9972988 },
  { "Jihae", 0.0 },
  { "Jihan", 0.09210526 },
  { "Jihanna", 0.0 },
  { "Jiho", 1.0 },
  { "Jihoo", 1.0 },
  { "Jihoon", 1.0 },
  { "Jihyun", 0.0 },
  { "Jikai", 1.0 },
  { "Jikayla", 0.0 },
  { "Jil", 0.0 },
  { "Jilberto", 1.0 },
  { "Jilda", 0.0 },
  { "Jildardo", 1.0 },
  { "Jileen", 0.0 },
  { "Jilene", 0.0 },
  { "Jiles", 1.0 },
  { "Jilian", 0.0 },
  { "Jiliana", 0.0 },
  { "Jilisa", 0.0 },
  { "Jilissa", 0.0 },
  { "Jill", 0.0024092107 },
  { "Jilla", 0.0 },
  { "Jillaine", 0.0 },
  { "Jillan", 0.0 },
  { "Jillana", 0.0 },
  { "Jillane", 0.0 },
  { "Jillann", 0.0 },
  { "Jillanna", 0.0 },
  { "Jillanne", 0.0 },
  { "Jillayne", 0.0 },
  { "Jillean", 0.0 },
  { "Jilleen", 0.0 },
  { "Jillena", 0.0 },
  { "Jillene", 0.0 },
  { "Jillesa", 0.0 },
  { "Jillia", 0.0 },
  { "Jilliam", 0.0 },
  { "Jillian", 0.0028093364 },
  { "Jilliana", 0.0 },
  { "Jilliane", 0.0 },
  { "Jilliann", 0.0 },
  { "Jillianna", 0.0 },
  { "Jillianne", 0.0 },
  { "Jillien", 0.0 },
  { "Jilliene", 0.0 },
  { "Jillienne", 0.0 },
  { "Jillinda", 0.0 },
  { "Jillion", 0.0 },
  { "Jillisa", 0.0 },
  { "Jillissa", 0.0 },
  { "Jilliyn", 0.0 },
  { "Jillmarie", 0.0 },
  { "Jillyan", 0.0 },
  { "Jillyn", 0.0 },
  { "Jillynn", 0.0 },
  { "Jilma", 0.0 },
  { "Jilyan", 0.0 },
  { "Jim", 0.99468803 },
  { "Jima", 0.0 },
  { "Jimal", 1.0 },
  { "Jimalee", 0.0 },
  { "Jimar", 1.0 },
  { "Jimari", 1.0 },
  { "Jimarion", 1.0 },
  { "Jimaya", 0.0 },
  { "Jimbo", 1.0 },
  { "Jimbob", 1.0 },
  { "Jimeka", 0.0 },
  { "Jimel", 1.0 },
  { "Jimell", 1.0 },
  { "Jimella", 0.0 },
  { "Jimena", 0.000709157 },
  { "Jimenez", 1.0 },
  { "Jimenna", 0.0 },
  { "Jimere", 1.0 },
  { "Jimesha", 0.0 },
  { "Jimetta", 0.0 },
  { "Jimette", 0.0 },
  { "Jimey", 1.0 },
  { "Jimi", 0.64357543 },
  { "Jimia", 0.0 },
  { "Jimiah", 0.0 },
  { "Jimica", 0.0 },
  { "Jimie", 0.9756098 },
  { "Jimika", 0.0 },
  { "Jimin", 0.42465752 },
  { "Jimique", 1.0 },
  { "Jimisha", 0.0 },
  { "Jimiya", 0.0 },
  { "Jimiyah", 0.0 },
  { "Jimm", 1.0 },
  { "Jimma", 0.0 },
  { "Jimme", 1.0 },
  { "Jimmeka", 0.0 },
  { "Jimmel", 1.0 },
  { "Jimmer", 1.0 },
  { "Jimmesha", 0.0 },
  { "Jimmey", 1.0 },
  { "Jimmi", 0.14848486 },
  { "Jimmia", 0.0 },
  { "Jimmiah", 0.0 },
  { "Jimmica", 0.0 },
  { "Jimmie", 0.8296426 },
  { "Jimmy", 0.99229205 },
  { "Jimmya", 0.0 },
  { "Jimmye", 0.042471044 },
  { "Jimmylee", 1.0 },
  { "Jimon", 1.0 },
  { "Jimson", 1.0 },
  { "Jimy", 1.0 },
  { "Jimya", 0.0 },
  { "Jimyah", 0.0 },
  { "Jin", 0.7457507 },
  { "Jina", 0.0 },
  { "Jinah", 0.0 },
  { "Jinal", 0.0 },
  { "Jinan", 0.029411765 },
  { "Jinane", 0.0 },
  { "Jinay", 1.0 },
  { "Jinaya", 0.0 },
  { "Jinayah", 0.0 },
  { "Jincy", 0.0 },
  { "Jineane", 0.0 },
  { "Jineen", 0.0 },
  { "Jinelle", 0.0 },
  { "Jinessa", 0.0 },
  { "Jinette", 0.0 },
  { "Jing", 0.074074075 },
  { "Jinger", 0.0 },
  { "Jingyi", 0.0 },
  { "Jinhee", 0.0 },
  { "Jini", 0.0 },
  { "Jiniyah", 0.0 },
  { "Jinjer", 0.0 },
  { "Jinks", 1.0 },
  { "Jinna", 0.0 },
  { "Jinnah", 0.0 },
  { "Jinni", 0.0 },
  { "Jinnie", 0.0 },
  { "Jinnifer", 0.0 },
  { "Jinny", 0.0 },
  { "Jino", 1.0 },
  { "Jinora", 0.0 },
  { "Jinu", 1.0 },
  { "Jinwoo", 1.0 },
  { "Jinx", 0.0 },
  { "Jio", 1.0 },
  { "Jion", 1.0 },
  { "Jioni", 1.0 },
  { "Jionna", 0.0 },
  { "Jionni", 0.9812362 },
  { "Jionnie", 1.0 },
  { "Jionny", 1.0 },
  { "Jiovana", 0.0 },
  { "Jiovani", 1.0 },
  { "Jiovanna", 0.0 },
  { "Jiovanni", 1.0 },
  { "Jiovannie", 1.0 },
  { "Jiovanny", 1.0 },
  { "Jiovany", 1.0 },
  { "Jiovonni", 1.0 },
  { "Jiquan", 1.0 },
  { "Jiquez", 1.0 },
  { "Jira", 0.0 },
  { "Jirah", 0.11 },
  { "Jiraiya", 0.9847561 },
  { "Jiram", 1.0 },
  { "Jireh", 0.47836348 },
  { "Jireth", 0.0 },
  { "Jiri", 1.0 },
  { "Jiro", 1.0 },
  { "Jiromi", 0.0 },
  { "Jiron", 1.0 },
  { "Jisaiah", 1.0 },
  { "Jisel", 0.0 },
  { "Jisela", 0.0 },
  { "Jisele", 0.0 },
  { "Jisell", 0.0 },
  { "Jisella", 0.0 },
  { "Jiselle", 0.0 },
  { "Jisha", 0.0 },
  { "Jishnu", 1.0 },
  { "Jisiah", 1.0 },
  { "Jisoo", 0.0 },
  { "Jissel", 0.0 },
  { "Jissela", 0.0 },
  { "Jissele", 0.0 },
  { "Jissell", 0.0 },
  { "Jisselle", 0.0 },
  { "Jissie", 0.8181818 },
  { "Jiten", 1.0 },
  { "Jitesh", 1.0 },
  { "Jitsuo", 1.0 },
  { "Jitu", 1.0 },
  { "Jitzel", 0.0 },
  { "Jiuliana", 0.0 },
  { "Jivan", 1.0 },
  { "Jivon", 1.0 },
  { "Jiwon", 0.0 },
  { "Jiwoo", 0.24615385 },
  { "Jiya", 0.0 },
  { "Jiyaan", 1.0 },
  { "Jiyah", 0.0 },
  { "Jiyan", 1.0 },
  { "Jiyana", 0.0 },
  { "Jiyanna", 0.0 },
  { "Jiyoung", 0.0 },
  { "Jiyu", 0.0 },
  { "Jizel", 0.0 },
  { "Jizele", 0.0 },
  { "Jizell", 0.0 },
  { "Jizelle", 0.0 },
  { "Jizzel", 0.0 },
  { "Jizzelle", 0.0 },
  { "Jj", 1.0 },
  { "Jjames", 1.0 },
  { "Jjesus", 1.0 },
  { "Jjuan", 1.0 },
  { "Jkai", 1.0 },
  { "Jkari", 1.0 },
  { "Jkayla", 0.0 },
  { "Jkhari", 1.0 },
  { "Jkia", 0.0 },
  { "Jkira", 0.0 },
  { "Jkiyah", 0.0 },
  { "Jkobe", 1.0 },
  { "Jkobi", 1.0 },
  { "Jkwon", 1.0 },
  { "Jkyla", 0.0 },
  { "Jl", 1.0 },
  { "Jla", 0.0 },
  { "Jlah", 0.0 },
  { "Jlan", 1.0 },
  { "Jlaya", 0.0 },
  { "Jlayah", 0.0 },
  { "Jlea", 0.0 },
  { "Jleah", 0.0 },
  { "Jlee", 0.0 },
  { "Jleigh", 0.0 },
  { "Jlen", 1.0 },
  { "Jliana", 0.0 },
  { "Jlin", 1.0 },
  { "Jlisa", 0.0 },
  { "Jliyah", 0.0 },
  { "Jlon", 1.0 },
  { "Jlyn", 0.40145984 },
  { "Jlynn", 0.086021505 },
  { "Jlyssa", 0.0 },
  { "Jm", 1.0 },
  { "Jmar", 1.0 },
  { "Jmarcus", 1.0 },
  { "Jmari", 1.0 },
  { "Jmarie", 0.0 },
  { "Jmarion", 1.0 },
  { "Jmauri", 1.0 },
  { "Jmaya", 0.0 },
  { "Jme", 0.0 },
  { "Jmere", 1.0 },
  { "Jmia", 0.0 },
  { "Jmichael", 1.0 },
  { "Jmir", 1.0 },
  { "Jmiyah", 0.0 },
  { "Jmya", 0.0 },
  { "Jmyah", 0.0 },
  { "Jmyiah", 0.0 },
  { "Jmyra", 0.0 },
  { "Jnae", 0.0 },
  { "Jnai", 0.0 },
  { "Jnaiya", 0.0 },
  { "Jnasia", 0.0 },
  { "Jnay", 0.0 },
  { "Jnaya", 0.0 },
  { "Jnia", 0.0 },
  { "Jniah", 0.0 },
  { "Jniaya", 0.0 },
  { "Jniya", 0.0 },
  { "Jniyah", 0.0 },
  { "Jno", 1.0 },
  { "Jnya", 0.0 },
  { "Jnyah", 0.0 },
  { "Jnyia", 0.0 },
  { "Jnyla", 0.0 },
  { "Jo", 0.009986983 },
  { "Joa", 0.25 },
  { "Joab", 1.0 },
  { "Joachim", 1.0 },
  { "Joacim", 1.0 },
  { "Joah", 1.0 },
  { "Joahan", 1.0 },
  { "Joahn", 1.0 },
  { "Joahna", 0.0 },
  { "Joahnna", 0.0 },
  { "Joahua", 1.0 },
  { "Joakim", 1.0 },
  { "Joakima", 0.0 },
  { "Joakin", 1.0 },
  { "Joal", 0.795082 },
  { "Joaly", 0.0 },
  { "Joan", 0.01229617 },
  { "Joana", 0.0 },
  { "Joanah", 0.0 },
  { "Joanann", 0.0 },
  { "Joanathan", 1.0 },
  { "Joandaly", 0.0 },
  { "Joandra", 0.0 },
  { "Joandre", 1.0 },
  { "Joandri", 1.0 },
  { "Joandry", 1.0 },
  { "Joandy", 1.0 },
  { "Joane", 0.0 },
  { "Joanel", 1.0 },
  { "Joaneliz", 0.0 },
  { "Joanell", 0.0 },
  { "Joanelle", 0.0 },
  { "Joanetta", 0.0 },
  { "Joanette", 0.0 },
  { "Joangel", 1.0 },
  { "Joangela", 0.0 },
  { "Joani", 0.0 },
  { "Joanie", 0.0 },
  { "Joanita", 0.0 },
  { "Joanmarie", 0.0 },
  { "Joann", 0.0034941402 },
  { "Joanna", 0.0030071347 },
  { "Joannah", 0.0 },
  { "Joannamarie", 0.0 },
  { "Joanne", 0.0023808512 },
  { "Joannette", 0.0 },
  { "Joannie", 0.0 },
  { "Joanny", 0.0295858 },
  { "Joanthan", 1.0 },
  { "Joanthon", 1.0 },
  { "Joanthony", 1.0 },
  { "Joany", 0.078125 },
  { "Joao", 1.0 },
  { "Joaogabriel", 1.0 },
  { "Joaolucas", 1.0 },
  { "Joaopaulo", 1.0 },
  { "Joaopedro", 1.0 },
  { "Joaovictor", 1.0 },
  { "Joaovitor", 1.0 },
  { "Joaquim", 1.0 },
  { "Joaquin", 0.9980093 },
  { "Joaquina", 0.0 },
  { "Joas", 1.0 },
  { "Joash", 1.0 },
  { "Joason", 1.0 },
  { "Joathan", 1.0 },
  { "Job", 1.0 },
  { "Joban", 1.0 },
  { "Jobana", 0.0 },
  { "Jobani", 1.0 },
  { "Jobanny", 1.0 },
  { "Jobany", 1.0 },
  { "Jobari", 1.0 },
  { "Jobe", 1.0 },
  { "Joben", 1.0 },
  { "Jobeth", 0.0 },
  { "Jobey", 1.0 },
  { "Jobi", 0.125 },
  { "Jobie", 0.8875502 },
  { "Jobin", 1.0 },
  { "Jobina", 0.0 },
  { "Jobita", 0.0 },
  { "Joby", 0.9425676 },
  { "Jobyna", 0.0 },
  { "Joc", 1.0 },
  { "Jocabed", 0.0 },
  { "Jocabeth", 0.0 },
  { "Jocalyn", 0.0 },
  { "Jocalynn", 0.0 },
  { "Jocarol", 0.0 },
  { "Jocasta", 0.0 },
  { "Joce", 1.0 },
  { "Jocee", 0.0 },
  { "Joceleen", 0.0 },
  { "Jocelene", 0.0 },
  { "Jocelin", 0.0 },
  { "Joceline", 0.0 },
  { "Jocelinne", 0.0 },
  { "Jocell", 0.0 },
  { "Jocelle", 0.0 },
  { "Jocellyn", 0.0 },
  { "Jocely", 0.0 },
  { "Jocelyn", 0.0034191532 },
  { "Jocelyne", 0.0 },
  { "Jocelynmarie", 0.0 },
  { "Jocelynn", 0.0 },
  { "Jocelynne", 0.0 },
  { "Jocene", 0.0 },
  { "Jocephus", 1.0 },
  { "Jocey", 0.0 },
  { "Joceyln", 0.0 },
  { "Jochebed", 0.0 },
  { "Jochelle", 0.0 },
  { "Joci", 0.0 },
  { "Jociah", 1.0 },
  { "Jocie", 0.0 },
  { "Jociel", 1.0 },
  { "Jocile", 0.0 },
  { "Jocilyn", 0.0 },
  { "Jocilynn", 0.0 },
  { "Jocinda", 0.0 },
  { "Jock", 1.0 },
  { "Jocko", 1.0 },
  { "Joclyn", 0.0 },
  { "Joclynn", 0.0 },
  { "Jocob", 1.0 },
  { "Jocoby", 1.0 },
  { "Jocqua", 1.0 },
  { "Jocqueline", 0.0 },
  { "Jocques", 1.0 },
  { "Jocquez", 1.0 },
  { "Jocqui", 1.0 },
  { "Jocsan", 1.0 },
  { "Jocylin", 0.0 },
  { "Jocylyn", 0.0 },
  { "Joda", 0.15178572 },
  { "Jodan", 1.0 },
  { "Jodana", 0.0 },
  { "Jodanna", 0.0 },
  { "Jodarius", 1.0 },
  { "Joddie", 0.26086956 },
  { "Jode", 0.14619882 },
  { "Jodean", 0.0 },
  { "Jodeane", 0.0 },
  { "Jodeci", 0.48923078 },
  { "Jodee", 0.010583707 },
  { "Jodeen", 0.0 },
  { "Jodel", 0.0 },
  { "Jodell", 0.0 },
  { "Jodelle", 0.0 },
  { "Joden", 1.0 },
  { "Jodena", 0.0 },
  { "Jodene", 0.0 },
  { "Jodette", 0.0 },
  { "Jodey", 0.6503497 },
  { "Jodhveer", 1.0 },
  { "Jodi", 0.011394946 },
  { "Jodiann", 0.0 },
  { "Jodianne", 0.0 },
  { "Jodice", 1.0 },
  { "Jodie", 0.11700671 },
  { "Jodiel", 1.0 },
  { "Jodilyn", 0.0 },
  { "Jodilynn", 0.0 },
  { "Jodina", 0.0 },
  { "Jodine", 0.0 },
  { "Jodonna", 0.0 },
  { "Jody", 0.35882673 },
  { "Jodyann", 0.0 },
  { "Jodye", 0.0 },
  { "Jodyne", 0.0 },
  { "Joe", 0.9754826 },
  { "Joean", 0.0 },
  { "Joeana", 0.0 },
  { "Joeangel", 1.0 },
  { "Joeann", 0.0 },
  { "Joeanna", 0.0 },
  { "Joeanne", 0.0 },
  { "Joeanthony", 1.0 },
  { "Joeb", 1.0 },
  { "Joed", 1.0 },
  { "Joedon", 1.0 },
  { "Joedy", 0.9230769 },
  { "Joee", 0.056179777 },
  { "Joeen", 0.0 },
  { "Joei", 0.0 },
  { "Joeisha", 0.0 },
  { "Joel", 0.99040896 },
  { "Joela", 0.0 },
  { "Joelaine", 0.0 },
  { "Joelan", 1.0 },
  { "Joele", 0.0 },
  { "Joelee", 0.0 },
  { "Joeleen", 0.0 },
  { "Joelene", 0.0 },
  { "Joeli", 0.0 },
  { "Joelia", 0.0 },
  { "Joelie", 0.0 },
  { "Joeliel", 1.0 },
  { "Joelina", 0.0 },
  { "Joeline", 0.0 },
  { "Joelis", 0.0 },
  { "Joelisa", 0.0 },
  { "Joelissa", 0.0 },
  { "Joeliz", 0.0 },
  { "Joell", 0.484985 },
  { "Joella", 0.0 },
  { "Joelle", 0.027560242 },
  { "Joellen", 0.0 },
  { "Joelliane", 0.0 },
  { "Joellie", 0.0 },
  { "Joelly", 0.0 },
  { "Joellyn", 0.0 },
  { "Joelouis", 1.0 },
  { "Joely", 0.0 },
  { "Joelyn", 0.0 },
  { "Joelyne", 0.0 },
  { "Joelynn", 0.0 },
  { "Joelys", 0.0 },
  { "Joemar", 1.0 },
  { "Joemichael", 1.0 },
  { "Joemy", 0.0 },
  { "Joen", 0.5903614 },
  { "Joena", 0.0 },
  { "Joenathan", 1.0 },
  { "Joene", 0.0 },
  { "Joenell", 0.0 },
  { "Joeph", 1.0 },
  { "Joephine", 0.0 },
  { "Joequan", 1.0 },
  { "Joeray", 1.0 },
  { "Joerell", 1.0 },
  { "Joes", 1.0 },
  { "Joesef", 1.0 },
  { "Joeseph", 1.0 },
  { "Joesette", 0.0 },
  { "Joesha", 0.0 },
  { "Joesiah", 1.0 },
  { "Joesiyah", 1.0 },
  { "Joesph", 0.99922603 },
  { "Joesphine", 0.0 },
  { "Joetta", 0.0 },
  { "Joette", 0.0 },
  { "Joevan", 1.0 },
  { "Joevani", 1.0 },
  { "Joevanni", 1.0 },
  { "Joevanny", 1.0 },
  { "Joevany", 1.0 },
  { "Joevon", 1.0 },
  { "Joey", 0.89147717 },
  { "Joeylynn", 0.0 },
  { "Joezette", 0.0 },
  { "Joeziah", 1.0 },
  { "Joffre", 1.0 },
  { "Joffrey", 1.0 },
  { "Jofiel", 1.0 },
  { "Jogina", 0.0 },
  { "Joh", 1.0 },
  { "Johaan", 1.0 },
  { "Johah", 1.0 },
  { "Johan", 0.98682415 },
  { "Johana", 0.0018272426 },
  { "Johanah", 0.0 },
  { "Johanan", 1.0 },
  { "Johandry", 1.0 },
  { "Johandy", 1.0 },
  { "Johann", 0.86105174 },
  { "Johanna", 0.002556531 },
  { "Johannah", 0.0 },
  { "Johanne", 0.0 },
  { "Johannes", 1.0 },
  { "Johannie", 0.0 },
  { "Johanny", 0.13125 },
  { "Johansel", 1.0 },
  { "Johansen", 1.0 },
  { "Johanson", 1.0 },
  { "Johanthan", 1.0 },
  { "Johany", 0.43103448 },
  { "Johao", 1.0 },
  { "Johar", 1.0 },
  { "Johara", 0.0 },
  { "Johari", 0.3781321 },
  { "Joharis", 0.0 },
  { "Johary", 1.0 },
  { "Johathan", 1.0 },
  { "Johathon", 1.0 },
  { "Johaun", 1.0 },
  { "Johel", 1.0 },
  { "Johhny", 1.0 },
  { "Johileny", 0.0 },
  { "Johm", 1.0 },
  { "John", 0.9957743 },
  { "Johna", 0.0024420025 },
  { "Johnaaron", 1.0 },
  { "Johnadam", 1.0 },
  { "Johnae", 0.0 },
  { "Johnael", 1.0 },
  { "Johnah", 0.0 },
  { "Johnai", 0.0 },
  { "Johnalbert", 1.0 },
  { "Johnalexander", 1.0 },
  { "Johnallen", 1.0 },
  { "Johnalyn", 0.0 },
  { "Johnan", 1.0 },
  { "Johnandrew", 1.0 },
  { "Johnanna", 0.0 },
  { "Johnanthan", 1.0 },
  { "Johnanthony", 1.0 },
  { "Johnas", 1.0 },
  { "Johnasia", 0.0 },
  { "Johnatan", 1.0 },
  { "Johnatha", 1.0 },
  { "Johnathan", 0.9962094 },
  { "Johnathen", 1.0 },
  { "Johnathin", 1.0 },
  { "Johnathn", 1.0 },
  { "Johnatho", 1.0 },
  { "Johnathon", 0.99789757 },
  { "Johnathyn", 1.0 },
  { "Johnaton", 1.0 },
  { "Johnattan", 1.0 },
  { "Johnavan", 1.0 },
  { "Johnaven", 1.0 },
  { "Johnavin", 1.0 },
  { "Johnavon", 1.0 },
  { "Johnay", 0.0 },
  { "Johnaya", 0.0 },
  { "Johnbenedict", 1.0 },
  { "Johncarl", 1.0 },
  { "Johncarlo", 1.0 },
  { "Johncarlos", 1.0 },
  { "Johncarter", 1.0 },
  { "Johncharles", 1.0 },
  { "Johnchristian", 1.0 },
  { "Johnchristopher", 1.0 },
  { "Johnda", 0.0 },
  { "Johndaniel", 1.0 },
  { "Johndavid", 1.0 },
  { "Johndrea", 0.0 },
  { "Johne", 0.9264706 },
  { "Johnea", 0.0 },
  { "Johnecia", 0.0 },
  { "Johnedward", 1.0 },
  { "Johnee", 0.0 },
  { "Johneen", 0.0 },
  { "Johneice", 0.0 },
  { "Johneisha", 0.0 },
  { "Johneka", 0.0 },
  { "Johnel", 0.98015875 },
  { "Johnell", 0.7715573 },
  { "Johnella", 0.0 },
  { "Johnelle", 0.030023094 },
  { "Johnene", 0.0 },
  { "Johneric", 1.0 },
  { "Johnese", 0.0 },
  { "Johnesha", 0.0 },
  { "Johneshia", 0.0 },
  { "Johnesia", 0.0 },
  { "Johnessa", 0.0 },
  { "Johnethan", 1.0 },
  { "Johnetta", 0.0 },
  { "Johnette", 0.0 },
  { "Johney", 0.99606556 },
  { "Johnfrancis", 1.0 },
  { "Johngabriel", 1.0 },
  { "Johnhenry", 1.0 },
  { "Johni", 0.0 },
  { "Johnia", 0.0 },
  { "Johniah", 0.0 },
  { "Johnica", 0.0 },
  { "Johnice", 0.0 },
  { "Johnicia", 0.0 },
  { "Johnie", 0.8025855 },
  { "Johniece", 0.0 },
  { "Johniel", 1.0 },
  { "Johniesha", 0.0 },
  { "Johnika", 0.0 },
  { "Johnine", 0.0 },
  { "Johniqua", 0.0 },
  { "Johnique", 0.0 },
  { "Johnise", 0.0 },
  { "Johnisha", 0.0 },
  { "Johnita", 0.0 },
  { "Johnitta", 0.0 },
  { "Johniya", 0.0 },
  { "Johniyah", 0.0 },
  { "Johnjack", 1.0 },
  { "Johnjames", 1.0 },
  { "Johnjoseph", 1.0 },
  { "Johnlee", 1.0 },
  { "Johnlloyd", 1.0 },
  { "Johnlucas", 1.0 },
  { "Johnluis", 1.0 },
  { "Johnluke", 1.0 },
  { "Johnmarco", 1.0 },
  { "Johnmark", 1.0 },
  { "Johnmartin", 1.0 },
  { "Johnmatthew", 1.0 },
  { "Johnmichael", 1.0 },
  { "Johnmorgan", 1.0 },
  { "Johnn", 1.0 },
  { "Johnna", 0.0 },
  { "Johnnae", 0.0 },
  { "Johnnathan", 1.0 },
  { "Johnnathon", 1.0 },
  { "Johnnay", 0.0 },
  { "Johnneisha", 0.0 },
  { "Johnnel", 1.0 },
  { "Johnnell", 1.0 },
  { "Johnnesha", 0.0 },
  { "Johnnessa", 0.0 },
  { "Johnnetta", 0.0 },
  { "Johnnette", 0.0 },
  { "Johnney", 1.0 },
  { "Johnni", 0.052380953 },
  { "Johnnia", 0.0 },
  { "Johnnice", 0.0 },
  { "Johnnie", 0.6747319 },
  { "Johnniemae", 0.0 },
  { "Johnniqua", 0.0 },
  { "Johnnisha", 0.0 },
  { "Johnnita", 0.0 },
  { "Johnny", 0.9881854 },
  { "Johnnyangel", 1.0 },
  { "Johnnye", 0.040955633 },
  { "Johnnyjoe", 1.0 },
  { "Johnnylee", 1.0 },
  { "Johnnyray", 1.0 },
  { "Johnoliver", 1.0 },
  { "Johnothan", 1.0 },
  { "Johnovan", 1.0 },
  { "Johnparker", 1.0 },
  { "Johnpatrick", 1.0 },
  { "Johnpaul", 1.0 },
  { "Johnpeter", 1.0 },
  { "Johnphilip", 1.0 },
  { "Johnphillip", 1.0 },
  { "Johnray", 1.0 },
  { "Johnrichard", 1.0 },
  { "Johnrobert", 1.0 },
  { "Johnross", 1.0 },
  { "Johnrussell", 1.0 },
  { "Johnryan", 1.0 },
  { "Johnscott", 1.0 },
  { "Johnse", 1.0 },
  { "Johnsie", 0.0 },
  { "Johnson", 0.99666464 },
  { "Johnston", 1.0 },
  { "Johnsy", 0.0 },
  { "Johnta", 0.9781659 },
  { "Johntae", 0.977707 },
  { "Johntavia", 0.0 },
  { "Johntavious", 1.0 },
  { "Johntavis", 1.0 },
  { "Johntavius", 1.0 },
  { "Johntay", 1.0 },
  { "Johnte", 1.0 },
  { "Johntel", 1.0 },
  { "Johntell", 1.0 },
  { "Johntez", 1.0 },
  { "Johnthan", 1.0 },
  { "Johnthomas", 1.0 },
  { "Johnthon", 1.0 },
  { "Johntrell", 1.0 },
  { "Johntyler", 1.0 },
  { "Johnvictor", 1.0 },
  { "Johnvincent", 1.0 },
  { "Johnwesley", 1.0 },
  { "Johnwilliam", 1.0 },
  { "Johny", 0.9924942 },
  { "Johnya", 0.0 },
  { "Johnye", 0.0 },
  { "Johnyla", 0.0 },
  { "Johon", 1.0 },
  { "Johonna", 0.0 },
  { "Johrdan", 1.0 },
  { "Johsua", 1.0 },
  { "Johua", 1.0 },
  { "Johusa", 1.0 },
  { "Johvanna", 0.0 },
  { "Johvanny", 1.0 },
  { "Joi", 0.0 },
  { "Joia", 0.0 },
  { "Joice", 0.0 },
  { "Joicy", 0.0 },
  { "Joie", 0.09173616 },
  { "Joielle", 0.0 },
  { "Joilyn", 0.0 },
  { "Joise", 0.0 },
  { "Joiya", 0.0 },
  { "Jojean", 0.0 },
  { "Joji", 1.0 },
  { "Jojo", 0.64220184 },
  { "Jojuan", 1.0 },
  { "Jokari", 1.0 },
  { "Joksan", 1.0 },
  { "Jola", 0.0 },
  { "Jolaine", 0.0 },
  { "Jolan", 0.6 },
  { "Jolana", 0.0 },
  { "Jolanda", 0.0 },
  { "Jolane", 0.0 },
  { "Jolani", 0.0 },
  { "Jolanta", 0.0 },
  { "Jolaoluwa", 0.0 },
  { "Jolayne", 0.0 },
  { "Jole", 0.0862069 },
  { "Jolea", 0.0 },
  { "Joleah", 0.0 },
  { "Jolean", 0.0 },
  { "Jolecia", 0.0 },
  { "Jolee", 0.0 },
  { "Joleen", 0.0 },
  { "Joleena", 0.0 },
  { "Joleene", 0.0 },
  { "Joleesa", 0.0 },
  { "Jolei", 0.0 },
  { "Joleigh", 0.0 },
  { "Joleigha", 0.0 },
  { "Jolen", 0.78723407 },
  { "Jolena", 0.0 },
  { "Jolene", 0.00039100685 },
  { "Jolenna", 0.0 },
  { "Jolesa", 0.0 },
  { "Jolesha", 0.0 },
  { "Jolet", 0.0 },
  { "Joleta", 0.0 },
  { "Joleth", 0.0 },
  { "Joletha", 0.0 },
  { "Jolett", 0.0 },
  { "Joletta", 0.0 },
  { "Jolette", 0.0 },
  { "Joley", 0.0 },
  { "Joli", 0.0 },
  { "Jolia", 0.0 },
  { "Jolian", 1.0 },
  { "Joliana", 0.0 },
  { "Jolianna", 0.0 },
  { "Jolicia", 0.0 },
  { "Jolie", 0.0 },
  { "Jolieann", 0.0 },
  { "Joliee", 0.0 },
  { "Jolien", 0.0 },
  { "Joliene", 0.0 },
  { "Joliet", 0.0 },
  { "Joliette", 0.0 },
  { "Jolin", 0.10268949 },
  { "Jolina", 0.0 },
  { "Jolinda", 0.0 },
  { "Joline", 0.0 },
  { "Jolisa", 0.0 },
  { "Jolisha", 0.0 },
  { "Jolissa", 0.0 },
  { "Jolita", 0.0 },
  { "Joliyah", 0.0 },
  { "Jolleen", 0.0 },
  { "Jollene", 0.0 },
  { "Jollette", 0.0 },
  { "Jollie", 1.0 },
  { "Jolly", 0.7532467 },
  { "Jolon", 1.0 },
  { "Jolonda", 0.0 },
  { "Jolyn", 0.0 },
  { "Jolynda", 0.0 },
  { "Jolyne", 0.0 },
  { "Jolynn", 0.0 },
  { "Jolynne", 0.0 },
  { "Jolyon", 1.0 },
  { "Jolysa", 0.0 },
  { "Jolyssa", 0.0 },
  { "Jomaira", 0.0 },
  { "Jomal", 1.0 },
  { "Jomana", 0.0 },
  { "Jomanda", 0.0 },
  { "Jomar", 1.0 },
  { "Jomara", 0.0 },
  { "Jomarcus", 1.0 },
  { "Jomari", 0.8238806 },
  { "Jomarie", 0.010660981 },
  { "Jomarion", 1.0 },
  { "Jomaris", 0.0 },
  { "Jomary", 0.0 },
  { "Jomayra", 0.0 },
  { "Jomei", 1.0 },
  { "Jomeka", 0.0 },
  { "Jomel", 1.0 },
  { "Jomes", 1.0 },
  { "Jometh", 1.0 },
  { "Jomiah", 0.0 },
  { "Jomira", 0.0 },
  { "Jomo", 1.0 },
  { "Jon", 0.99250704 },
  { "Jona", 0.1750503 },
  { "Jonadab", 1.0 },
  { "Jonae", 0.0 },
  { "Jonael", 1.0 },
  { "Jonah", 0.99104416 },
  { "Jonahan", 1.0 },
  { "Jonahel", 1.0 },
  { "Jonahs", 1.0 },
  { "Jonahtan", 1.0 },
  { "Jonai", 0.032894738 },
  { "Jonalee", 0.0 },
  { "Jonalyn", 0.0 },
  { "Jonam", 1.0 },
  { "Jonan", 1.0 },
  { "Jonanthan", 1.0 },
  { "Jonanthony", 1.0 },
  { "Jonas", 0.99878746 },
  { "Jonasia", 0.0 },
  { "Jonatan", 1.0 },
  { "Jonath", 1.0 },
  { "Jonatha", 0.9147059 },
  { "Jonatham", 1.0 },
  { "Jonathan", 0.99576205 },
  { "Jonathandavid", 1.0 },
  { "Jonathanjames", 1.0 },
  { "Jonathanjoseph", 1.0 },
  { "Jonathanmichael", 1.0 },
  { "Jonathaon", 1.0 },
  { "Jonathen", 1.0 },
  { "Jonathin", 1.0 },
  { "Jonathn", 1.0 },
  { "Jonathna", 1.0 },
  { "Jonathon", 0.9965277 },
  { "Jonathyn", 1.0 },
  { "Jonaton", 1.0 },
  { "Jonattan", 1.0 },
  { "Jonatthan", 1.0 },
  { "Jonavan", 1.0 },
  { "Jonaven", 1.0 },
  { "Jonavin", 1.0 },
  { "Jonavon", 1.0 },
  { "Jonay", 0.0 },
  { "Jonaya", 0.0 },
  { "Jonaye", 0.0 },
  { "Jonbenet", 0.0 },
  { "Joncarlo", 1.0 },
  { "Joncarlos", 1.0 },
  { "Jonce", 1.0 },
  { "Jonchristopher", 1.0 },
  { "Jonda", 0.0 },
  { "Jondarius", 1.0 },
  { "Jondavid", 1.0 },
  { "Jondra", 0.0 },
  { "Jondrea", 0.0 },
  { "Jone", 0.033755273 },
  { "Jonea", 0.0 },
  { "Jonecia", 0.0 },
  { "Jonee", 0.0 },
  { "Joneen", 0.0 },
  { "Joneer", 0.0 },
  { "Joneisha", 0.0 },
  { "Joneka", 0.0 },
  { "Jonel", 0.49615976 },
  { "Jonell", 0.077848814 },
  { "Jonella", 0.0 },
  { "Jonelle", 0.0066266963 },
  { "Jonene", 0.0 },
  { "Joneric", 1.0 },
  { "Jonerik", 1.0 },
  { "Jones", 0.99640936 },
  { "Jonesha", 0.0 },
  { "Joneshia", 0.0 },
  { "Jonessa", 0.0 },
  { "Jonesy", 1.0 },
  { "Jonet", 0.0 },
  { "Jonethan", 1.0 },
  { "Jonethen", 1.0 },
  { "Jonetta", 0.0 },
  { "Jonette", 0.0 },
  { "Joney", 1.0 },
  { "Jong", 1.0 },
  { "Jonh", 1.0 },
  { "Jonhatan", 1.0 },
  { "Jonhathan", 1.0 },
  { "Jonhenry", 1.0 },
  { "Joni", 0.018902864 },
  { "Jonia", 0.0 },
  { "Joniah", 0.0 },
  { "Jonibek", 1.0 },
  { "Jonica", 0.0 },
  { "Jonice", 0.0 },
  { "Jonicia", 0.0 },
  { "Jonie", 0.034455754 },
  { "Joniece", 0.0 },
  { "Joniel", 1.0 },
  { "Joniell", 1.0 },
  { "Jonier", 0.0 },
  { "Joniesha", 0.0 },
  { "Jonika", 0.0 },
  { "Jonikka", 0.0 },
  { "Jonilee", 0.0 },
  { "Jonina", 0.0 },
  { "Jonine", 0.0 },
  { "Joniqua", 0.0 },
  { "Jonique", 0.0 },
  { "Jonis", 1.0 },
  { "Jonise", 0.0 },
  { "Jonisha", 0.0 },
  { "Jonita", 0.0 },
  { "Jonithan", 1.0 },
  { "Joniya", 0.0 },
  { "Joniyah", 0.0 },
  { "Jonjon", 1.0 },
  { "Jonluc", 1.0 },
  { "Jonluca", 1.0 },
  { "Jonlucas", 1.0 },
  { "Jonluke", 1.0 },
  { "Jonlyn", 0.0 },
  { "Jonmarc", 1.0 },
  { "Jonmark", 1.0 },
  { "Jonmatthew", 1.0 },
  { "Jonmichael", 1.0 },
  { "Jonn", 1.0 },
  { "Jonna", 0.0 },
  { "Jonnae", 0.0 },
  { "Jonnah", 0.25641027 },
  { "Jonnatan", 1.0 },
  { "Jonnathan", 1.0 },
  { "Jonnathon", 1.0 },
  { "Jonnay", 0.0 },
  { "Jonne", 0.0 },
  { "Jonnell", 0.1594203 },
  { "Jonnelle", 0.0 },
  { "Jonnesha", 0.0 },
  { "Jonnetta", 0.0 },
  { "Jonnette", 0.0 },
  { "Jonney", 1.0 },
  { "Jonni", 0.0041017225 },
  { "Jonnica", 0.0 },
  { "Jonnie", 0.35478085 },
  { "Jonniel", 1.0 },
  { "Jonnika", 0.0 },
  { "Jonnique", 0.0 },
  { "Jonnisha", 0.0 },
  { "Jonnita", 0.0 },
  { "Jonny", 0.9590753 },
  { "Jonnye", 0.0 },
  { "Jonoah", 1.0 },
  { "Jonothan", 1.0 },
  { "Jonothon", 1.0 },
  { "Jonovan", 1.0 },
  { "Jonpatrick", 1.0 },
  { "Jonpaul", 1.0 },
  { "Jonpierre", 1.0 },
  { "Jonquavious", 1.0 },
  { "Jonquel", 1.0 },
  { "Jonquez", 1.0 },
  { "Jonquil", 0.20446096 },
  { "Jonquin", 1.0 },
  { "Jonrobert", 1.0 },
  { "Jonross", 1.0 },
  { "Jonryan", 1.0 },
  { "Jonson", 1.0 },
  { "Jonta", 0.8674699 },
  { "Jontae", 0.81641024 },
  { "Jontarius", 1.0 },
  { "Jontasia", 0.0 },
  { "Jontavia", 0.0 },
  { "Jontavion", 1.0 },
  { "Jontavious", 1.0 },
  { "Jontavis", 1.0 },
  { "Jontavius", 1.0 },
  { "Jontay", 0.9074074 },
  { "Jontaya", 0.0 },
  { "Jontaye", 1.0 },
  { "Jonte", 0.8385714 },
  { "Jontel", 0.60784316 },
  { "Jontelle", 0.0 },
  { "Jonteria", 0.0 },
  { "Jontez", 1.0 },
  { "Jonthan", 1.0 },
  { "Jonthomas", 1.0 },
  { "Jontia", 0.0 },
  { "Jontre", 1.0 },
  { "Jontrell", 1.0 },
  { "Jontue", 0.6936937 },
  { "Jontyler", 1.0 },
  { "Jonuel", 1.0 },
  { "Jonus", 1.0 },
  { "Jonwesley", 1.0 },
  { "Jony", 1.0 },
  { "Jonya", 0.0 },
  { "Jonylah", 0.0 },
  { "Joo", 0.33333334 },
  { "Jood", 0.17791411 },
  { "Joon", 0.90726817 },
  { "Joory", 0.0 },
  { "Jooyoung", 1.0 },
  { "Jophiel", 1.0 },
  { "Jophy", 1.0 },
  { "Joplin", 0.024509804 },
  { "Joplyn", 0.0 },
  { "Joplynn", 0.0 },
  { "Joquan", 1.0 },
  { "Joquavious", 1.0 },
  { "Joquetta", 0.0 },
  { "Joquin", 1.0 },
  { "Joquita", 0.0 },
  { "Jora", 0.0 },
  { "Jorah", 0.46268657 },
  { "Joram", 1.0 },
  { "Joran", 1.0 },
  { "Jorawar", 1.0 },
  { "Joray", 1.0 },
  { "Jorda", 1.0 },
  { "Jordaan", 1.0 },
  { "Jordache", 1.0 },
  { "Jordae", 0.0 },
  { "Jordahn", 0.0 },
  { "Jordai", 0.0 },
  { "Jordain", 0.21532847 },
  { "Jordan", 0.73822314 },
  { "Jordana", 0.0 },
  { "Jordanalexander", 1.0 },
  { "Jordanchristoph", 1.0 },
  { "Jordane", 0.04191617 },
  { "Jordani", 1.0 },
  { "Jordania", 0.0 },
  { "Jordanlee", 1.0 },
  { "Jordanmarie", 0.0 },
  { "Jordanmichael", 1.0 },
  { "Jordann", 0.0505145 },
  { "Jordanna", 0.0 },
  { "Jordanne", 0.0 },
  { "Jordanny", 1.0 },
  { "Jordano", 1.0 },
  { "Jordany", 1.0 },
  { "Jordayn", 0.0 },
  { "Jorddy", 1.0 },
  { "Jorde", 1.0 },
  { "Jordee", 0.0 },
  { "Jordell", 1.0 },
  { "Jorden", 0.73989433 },
  { "Jordeyn", 0.0 },
  { "Jordi", 0.9372477 },
  { "Jordian", 0.0 },
  { "Jordie", 0.75 },
  { "Jordin", 0.34877127 },
  { "Jordinn", 0.0 },
  { "Jordis", 0.021978023 },
  { "Jordon", 0.8790301 },
  { "Jordun", 1.0 },
  { "Jordy", 0.9873495 },
  { "Jordyan", 0.14285715 },
  { "Jordyn", 0.08892522 },
  { "Jordyne", 0.0 },
  { "Jordynn", 0.023431595 },
  { "Jordynne", 0.0 },
  { "Joree", 0.0 },
  { "Joreen", 0.0 },
  { "Jorel", 1.0 },
  { "Jorell", 1.0 },
  { "Jorelys", 0.0 },
  { "Joren", 1.0 },
  { "Jorene", 0.0 },
  { "Joretha", 0.0 },
  { "Joretta", 0.0 },
  { "Jorey", 0.8979592 },
  { "Jorga", 0.0 },
  { "Jorgan", 1.0 },
  { "Jorge", 0.99464315 },
  { "Jorgealberto", 1.0 },
  { "Jorgeantonio", 1.0 },
  { "Jorgedavid", 1.0 },
  { "Jorgejr", 1.0 },
  { "Jorgeluis", 1.0 },
  { "Jorgen", 1.0 },
  { "Jorgi", 0.0 },
  { "Jorgia", 0.0 },
  { "Jorgie", 0.0 },
  { "Jorgina", 0.0 },
  { "Jori", 0.069350146 },
  { "Joria", 0.0 },
  { "Joriah", 0.54545456 },
  { "Jorian", 0.9093484 },
  { "Joriann", 0.0 },
  { "Jorianna", 0.0 },
  { "Jorie", 0.012269938 },
  { "Joriel", 1.0 },
  { "Jorien", 1.0 },
  { "Jorim", 1.0 },
  { "Jorin", 1.0 },
  { "Joris", 1.0 },
  { "Jorita", 0.0 },
  { "Jorja", 0.0 },
  { "Jorje", 1.0 },
  { "Jorjia", 0.0 },
  { "Jorleny", 0.0 },
  { "Jorley", 0.0 },
  { "Jorma", 1.0 },
  { "Jorman", 1.0 },
  { "Jorn", 1.0 },
  { "Jorome", 1.0 },
  { "Joron", 1.0 },
  { "Jorrel", 1.0 },
  { "Jorrell", 1.0 },
  { "Jorren", 1.0 },
  { "Jorri", 0.0 },
  { "Jorrie", 0.0 },
  { "Jorrin", 1.0 },
  { "Jorryn", 0.49044585 },
  { "Jory", 0.85365856 },
  { "Joryel", 1.0 },
  { "Joryn", 0.0 },
  { "Jos", 1.0 },
  { "Josa", 1.0 },
  { "Josabet", 0.0 },
  { "Josabeth", 0.0 },
  { "Josael", 1.0 },
  { "Josafat", 1.0 },
  { "Josai", 1.0 },
  { "Josaia", 1.0 },
  { "Josaiah", 1.0 },
  { "Josaih", 1.0 },
  { "Josalin", 0.0 },
  { "Josalina", 0.0 },
  { "Josaline", 0.0 },
  { "Josalyn", 0.0 },
  { "Josalyne", 0.0 },
  { "Josalynn", 0.0 },
  { "Josan", 1.0 },
  { "Josann", 0.0 },
  { "Josanna", 0.0 },
  { "Josanne", 0.0 },
  { "Josaphat", 1.0 },
  { "Josaphina", 0.0 },
  { "Josaphine", 0.0 },
  { "Josbel", 1.0 },
  { "Joscar", 1.0 },
  { "Joscelin", 0.0 },
  { "Josceline", 0.0 },
  { "Joscelyn", 0.0 },
  { "Joscelyne", 0.0 },
  { "Joscelynn", 0.0 },
  { "Josclyn", 0.0 },
  { "Jose", 0.9925729 },
  { "Josea", 1.0 },
  { "Joseadrian", 1.0 },
  { "Josealberto", 1.0 },
  { "Josealejandro", 1.0 },
  { "Josealfredo", 1.0 },
  { "Josean", 1.0 },
  { "Joseandres", 1.0 },
  { "Joseangel", 1.0 },
  { "Joseantonio", 1.0 },
  { "Josearmando", 1.0 },
  { "Josecarlos", 1.0 },
  { "Josecruz", 1.0 },
  { "Josedaniel", 1.0 },
  { "Josedavid", 1.0 },
  { "Josede", 1.0 },
  { "Josedejesus", 1.0 },
  { "Joseduardo", 1.0 },
  { "Josee", 0.0 },
  { "Joseeduardo", 1.0 },
  { "Joseenrique", 1.0 },
  { "Josef", 1.0 },
  { "Josefa", 0.0 },
  { "Josefernando", 1.0 },
  { "Joseff", 1.0 },
  { "Josefina", 0.002652922 },
  { "Josefine", 0.0 },
  { "Josefita", 0.0 },
  { "Josefrancisco", 1.0 },
  { "Josegabriel", 1.0 },
  { "Joseguadalupe", 1.0 },
  { "Joseh", 1.0 },
  { "Josehine", 0.0 },
  { "Josehp", 1.0 },
  { "Josehua", 1.0 },
  { "Josei", 0.0 },
  { "Joseiah", 1.0 },
  { "Joseignacio", 1.0 },
  { "Josejavier", 1.0 },
  { "Josejesus", 1.0 },
  { "Josejr", 1.0 },
  { "Josejuan", 1.0 },
  { "Josejulian", 1.0 },
  { "Josel", 1.0 },
  { "Joselen", 0.0 },
  { "Joselene", 0.0 },
  { "Joselin", 0.0 },
  { "Joselina", 0.0 },
  { "Joseline", 0.0 },
  { "Joselinne", 0.0 },
  { "Joselito", 1.0 },
  { "Joselle", 0.0 },
  { "Josellyn", 0.0 },
  { "Joseluis", 1.0 },
  { "Joseluiz", 1.0 },
  { "Josely", 0.0 },
  { "Joselyn", 0.0003082994 },
  { "Joselyne", 0.0 },
  { "Joselynn", 0.0 },
  { "Joselynne", 0.0 },
  { "Josemanuel", 1.0 },
  { "Josemaria", 1.0 },
  { "Josemiguel", 1.0 },
  { "Josen", 1.0 },
  { "Josenid", 0.0 },
  { "Josep", 1.0 },
  { "Josepablo", 1.0 },
  { "Joseph", 0.9959212 },
  { "Josepha", 0.0 },
  { "Josephallen", 1.0 },
  { "Josephandrew", 1.0 },
  { "Josephanthony", 1.0 },
  { "Josephdaniel", 1.0 },
  { "Josephe", 1.0 },
  { "Josephene", 0.0 },
  { "Josephin", 0.0 },
  { "Josephina", 0.0 },
  { "Josephine", 0.0032608623 },
  { "Josephjames", 1.0 },
  { "Josephjohn", 1.0 },
  { "Josephlee", 1.0 },
  { "Josephmichael", 1.0 },
  { "Josephpatrick", 1.0 },
  { "Josephpaul", 1.0 },
  { "Josephthomas", 1.0 },
  { "Josephus", 1.0 },
  { "Josephy", 1.0 },
  { "Josephyne", 0.0 },
  { "Josepine", 0.0 },
  { "Josera", 0.0 },
  { "Joseramon", 1.0 },
  { "Josericardo", 1.0 },
  { "Joseroberto", 1.0 },
  { "Joses", 1.0 },
  { "Joset", 0.6875 },
  { "Joseth", 1.0 },
  { "Josett", 0.0 },
  { "Josetta", 0.0 },
  { "Josette", 0.0 },
  { "Josey", 0.24871717 },
  { "Josgar", 1.0 },
  { "Josgart", 1.0 },
  { "Josh", 1.0 },
  { "Josha", 0.938326 },
  { "Joshalyn", 0.0 },
  { "Joshalynn", 0.0 },
  { "Joshanae", 0.0 },
  { "Joshau", 1.0 },
  { "Joshaua", 1.0 },
  { "Joshaun", 1.0 },
  { "Joshawa", 1.0 },
  { "Joshawn", 1.0 },
  { "Joshelin", 0.0 },
  { "Joshelyn", 0.0 },
  { "Joshep", 1.0 },
  { "Josheph", 1.0 },
  { "Josheua", 1.0 },
  { "Joshewa", 1.0 },
  { "Joshia", 1.0 },
  { "Joshiah", 1.0 },
  { "Joshie", 0.0 },
  { "Joshika", 0.0 },
  { "Joshitha", 0.0 },
  { "Joshlin", 0.23333333 },
  { "Joshlyn", 0.004191115 },
  { "Joshlynn", 0.0 },
  { "Joshoa", 1.0 },
  { "Joshoua", 1.0 },
  { "Joshton", 1.0 },
  { "Joshu", 1.0 },
  { "Joshua", 0.9959106 },
  { "Joshuaa", 1.0 },
  { "Joshuaaaron", 1.0 },
  { "Joshuaalexander", 1.0 },
  { "Joshuaanthony", 1.0 },
  { "Joshuacaleb", 1.0 },
  { "Joshuadaniel", 1.0 },
  { "Joshuadavid", 1.0 },
  { "Joshuah", 1.0 },
  { "Joshuajames", 1.0 },
  { "Joshuajay", 1.0 },
  { "Joshuajohn", 1.0 },
  { "Joshuajoseph", 1.0 },
  { "Joshual", 1.0 },
  { "Joshualee", 1.0 },
  { "Joshuamichael", 1.0 },
  { "Joshuan", 1.0 },
  { "Joshuapaul", 1.0 },
  { "Joshuar", 1.0 },
  { "Joshuaray", 1.0 },
  { "Joshuarobert", 1.0 },
  { "Joshuaryan", 1.0 },
  { "Joshue", 1.0 },
  { "Joshuea", 1.0 },
  { "Joshuel", 1.0 },
  { "Joshuia", 1.0 },
  { "Joshula", 1.0 },
  { "Joshus", 1.0 },
  { "Joshusa", 1.0 },
  { "Joshuwa", 1.0 },
  { "Joshva", 1.0 },
  { "Joshwa", 1.0 },
  { "Joshya", 1.0 },
  { "Josi", 0.0 },
  { "Josia", 0.9579832 },
  { "Josiah", 0.9963208 },
  { "Josiaha", 1.0 },
  { "Josiahs", 1.0 },
  { "Josian", 1.0 },
  { "Josiana", 0.0 },
  { "Josiane", 0.0 },
  { "Josianna", 0.0 },
  { "Josianne", 0.0 },
  { "Josias", 1.0 },
  { "Josie", 0.008507209 },
  { "Josiel", 1.0 },
  { "Josielyn", 0.0 },
  { "Josielynn", 0.0 },
  { "Josiephene", 0.0 },
  { "Josiephine", 0.0 },
  { "Josif", 1.0 },
  { "Josiha", 1.0 },
  { "Josiiah", 1.0 },
  { "Josilin", 0.0 },
  { "Josilyn", 0.0 },
  { "Josilyne", 0.0 },
  { "Josilynn", 0.0 },
  { "Josimar", 1.0 },
  { "Josina", 0.0 },
  { "Josine", 0.0 },
  { "Josip", 1.0 },
  { "Josiphine", 0.0 },
  { "Josiya", 1.0 },
  { "Josiyah", 1.0 },
  { "Josje", 1.0 },
  { "Josjeph", 1.0 },
  { "Joskar", 1.0 },
  { "Joslen", 0.0 },
  { "Joslene", 0.0 },
  { "Joslin", 0.007751938 },
  { "Josline", 0.0 },
  { "Joslyn", 0.0028615228 },
  { "Joslyne", 0.0 },
  { "Joslynn", 0.0 },
  { "Joslynne", 0.0 },
  { "Josman", 1.0 },
  { "Josmar", 1.0 },
  { "Josmarie", 0.0 },
  { "Josmary", 0.0 },
  { "Josmel", 1.0 },
  { "Josmine", 0.0 },
  { "Josniel", 1.0 },
  { "Joson", 1.0 },
  { "Jospeh", 1.0 },
  { "Josph", 1.0 },
  { "Josphe", 1.0 },
  { "Josphine", 0.0 },
  { "Josphua", 1.0 },
  { "Joss", 0.62580645 },
  { "Jossalin", 0.0 },
  { "Jossalyn", 0.0 },
  { "Jossalynn", 0.0 },
  { "Josse", 1.0 },
  { "Josselin", 0.0 },
  { "Josseline", 0.0 },
  { "Josselyn", 0.0 },
  { "Josselyne", 0.0 },
  { "Josselynn", 0.0 },
  { "Jossette", 0.0 },
  { "Jossiah", 1.0 },
  { "Jossica", 0.0 },
  { "Jossie", 0.008555612 },
  { "Jossiel", 1.0 },
  { "Jossiephine", 0.0 },
  { "Jossilyn", 0.0 },
  { "Jossilynn", 0.0 },
  { "Jossimar", 1.0 },
  { "Josslin", 0.0 },
  { "Josslyn", 0.0 },
  { "Josslynn", 0.0 },
  { "Josslynne", 0.0 },
  { "Jossue", 1.0 },
  { "Jossy", 0.0 },
  { "Jostein", 1.0 },
  { "Josten", 1.0 },
  { "Jostin", 1.0 },
  { "Joston", 1.0 },
  { "Jostyn", 0.8 },
  { "Josua", 1.0 },
  { "Josuah", 1.0 },
  { "Josue", 0.9960184 },
  { "Josuel", 1.0 },
  { "Josuha", 1.0 },
  { "Josuhe", 1.0 },
  { "Josy", 0.0 },
  { "Josyah", 1.0 },
  { "Josyiah", 1.0 },
  { "Josylin", 0.0 },
  { "Josziah", 1.0 },
  { "Jotaro", 1.0 },
  { "Jotavious", 1.0 },
  { "Jotham", 1.0 },
  { "Jothan", 1.0 },
  { "Joua", 0.0 },
  { "Jouan", 1.0 },
  { "Jouanna", 0.0 },
  { "Joud", 0.35748792 },
  { "Jouette", 0.0 },
  { "Joule", 0.0 },
  { "Joules", 0.0 },
  { "Joumana", 0.0 },
  { "Jourdain", 0.82439023 },
  { "Jourdan", 0.43417084 },
  { "Jourden", 0.68773234 },
  { "Jourdin", 0.35682818 },
  { "Jourdon", 0.9 },
  { "Jourdyn", 0.1737931 },
  { "Jourdynn", 0.0 },
  { "Jouri", 0.0 },
  { "Journae", 0.0 },
  { "Journe", 0.0 },
  { "Journee", 0.00971942 },
  { "Journei", 0.0 },
  { "Journeigh", 0.0 },
  { "Journeii", 0.0 },
  { "Journey", 0.08640438 },
  { "Journi", 0.0 },
  { "Journie", 0.004765687 },
  { "Journiee", 0.0 },
  { "Journii", 0.0 },
  { "Journy", 0.0 },
  { "Journye", 0.0 },
  { "Joury", 0.0 },
  { "Jouse", 1.0 },
  { "Jousha", 1.0 },
  { "Joushua", 1.0 },
  { "Jovaan", 1.0 },
  { "Jovahn", 1.0 },
  { "Jovahni", 1.0 },
  { "Jovahny", 1.0 },
  { "Jovan", 0.89282876 },
  { "Jovana", 0.0 },
  { "Jovanah", 0.0 },
  { "Jovanda", 0.0 },
  { "Jovane", 1.0 },
  { "Jovani", 0.98157537 },
  { "Jovania", 0.0 },
  { "Jovanie", 0.97089946 },
  { "Jovaniel", 1.0 },
  { "Jovanii", 1.0 },
  { "Jovanka", 0.0 },
  { "Jovann", 0.9478261 },
  { "Jovanna", 0.0 },
  { "Jovannah", 0.0 },
  { "Jovanne", 0.68817204 },
  { "Jovanni", 0.9686558 },
  { "Jovannie", 0.96636087 },
  { "Jovanny", 0.9962782 },
  { "Jovanta", 1.0 },
  { "Jovantae", 1.0 },
  { "Jovante", 1.0 },
  { "Jovany", 0.99741787 },
  { "Jovar", 1.0 },
  { "Jovari", 1.0 },
  { "Jovaughn", 1.0 },
  { "Jovaun", 1.0 },
  { "Jovawn", 1.0 },
  { "Jove", 1.0 },
  { "Jovee", 0.0 },
  { "Jovel", 1.0 },
  { "Jovell", 1.0 },
  { "Joven", 1.0 },
  { "Jovetta", 0.0 },
  { "Jovette", 0.0 },
  { "Jovey", 0.0 },
  { "Jovi", 0.2794971 },
  { "Jovia", 0.0 },
  { "Jovial", 0.0 },
  { "Jovian", 0.98493975 },
  { "Joviana", 0.0 },
  { "Jovianna", 0.0 },
  { "Jovianne", 0.0 },
  { "Jovie", 0.0 },
  { "Jovienne", 0.0 },
  { "Jovin", 1.0 },
  { "Jovina", 0.0 },
  { "Jovita", 0.0015394088 },
  { "Jovohn", 1.0 },
  { "Jovon", 0.8973722 },
  { "Jovona", 0.0 },
  { "Jovonda", 0.0 },
  { "Jovone", 1.0 },
  { "Jovoni", 0.939759 },
  { "Jovonie", 1.0 },
  { "Jovonn", 0.8148148 },
  { "Jovonna", 0.0 },
  { "Jovonne", 0.13658537 },
  { "Jovonni", 1.0 },
  { "Jovonnie", 0.8235294 },
  { "Jovonny", 1.0 },
  { "Jovonta", 1.0 },
  { "Jovontae", 1.0 },
  { "Jovonte", 1.0 },
  { "Jowan", 1.0 },
  { "Jowana", 0.0 },
  { "Jowanda", 0.0 },
  { "Jowanna", 0.0 },
  { "Jowel", 1.0 },
  { "Jowell", 0.99226004 },
  { "Jowen", 1.0 },
  { "Jowharah", 0.0 },
  { "Joxiel", 1.0 },
  { "Joy", 0.018200545 },
  { "Joya", 0.0 },
  { "Joyace", 0.0 },
  { "Joyah", 0.0 },
  { "Joyal", 0.0 },
  { "Joyana", 0.0 },
  { "Joyann", 0.0 },
  { "Joyanna", 0.0 },
  { "Joyanne", 0.0 },
  { "Joyce", 0.005606791 },
  { "Joyceann", 0.0 },
  { "Joycee", 0.0 },
  { "Joycelene", 0.0 },
  { "Joycelin", 0.0 },
  { "Joyceline", 0.0 },
  { "Joycelyn", 0.0 },
  { "Joycelynn", 0.0 },
  { "Joycene", 0.0 },
  { "Joycie", 0.0 },
  { "Joyclyn", 0.0 },
  { "Joydan", 0.0 },
  { "Joyden", 0.5 },
  { "Joye", 0.0023249055 },
  { "Joyee", 0.0 },
  { "Joyel", 0.0 },
  { "Joyell", 0.0 },
  { "Joyelle", 0.0 },
  { "Joyetta", 0.0 },
  { "Joyette", 0.0 },
  { "Joyful", 0.0 },
  { "Joyia", 0.0 },
  { "Joyice", 0.0 },
  { "Joyl", 0.0 },
  { "Joyleen", 0.0 },
  { "Joylene", 0.0 },
  { "Joylin", 0.0 },
  { "Joylyn", 0.0 },
  { "Joylynn", 0.0 },
  { "Joylynne", 0.0 },
  { "Joymarie", 0.0 },
  { "Joynae", 0.0 },
  { "Joynell", 0.0 },
  { "Joyner", 1.0 },
  { "Joyous", 0.0 },
  { "Joyse", 0.0 },
  { "Joyzelle", 0.0 },
  { "Jozaiah", 1.0 },
  { "Jozalyn", 0.0 },
  { "Jozalynn", 0.0 },
  { "Jozee", 0.0 },
  { "Jozef", 1.0 },
  { "Jozefa", 0.0 },
  { "Jozelle", 0.0 },
  { "Jozelyn", 0.0 },
  { "Jozelynn", 0.0 },
  { "Jozeph", 1.0 },
  { "Jozephine", 0.0 },
  { "Jozett", 0.0 },
  { "Jozetta", 0.0 },
  { "Jozette", 0.0 },
  { "Jozey", 0.0 },
  { "Jozi", 0.0 },
  { "Joziah", 0.9981189 },
  { "Jozie", 0.0 },
  { "Joziel", 1.0 },
  { "Jozilyn", 0.0 },
  { "Joziyah", 1.0 },
  { "Jozlin", 0.0 },
  { "Jozlyn", 0.0 },
  { "Jozlyne", 0.0 },
  { "Jozlynn", 0.0 },
  { "Jozlynne", 0.0 },
  { "Jozsef", 1.0 },
  { "Jozy", 0.13333334 },
  { "Jozyah", 1.0 },
  { "Jozzlynn", 0.0 },
  { "Jp", 1.0 },
  { "Jquan", 1.0 },
  { "Jqwon", 1.0 },
  { "Jr", 1.0 },
  { "Jrake", 1.0 },
  { "Jraya", 0.0 },
  { "Jrayden", 1.0 },
  { "Jream", 0.18421052 },
  { "Jrew", 1.0 },
  { "Jron", 1.0 },
  { "Jru", 0.8269231 },
  { "Jrue", 0.85714287 },
  { "Jsai", 1.0 },
  { "Jsamine", 0.0 },
  { "Jsaon", 1.0 },
  { "Jsean", 1.0 },
  { "Jshaun", 1.0 },
  { "Jshawn", 1.0 },
  { "Jshin", 1.0 },
  { "Jshon", 1.0 },
  { "Jshun", 1.0 },
  { "Jsiah", 1.0 },
  { "Jsoeph", 1.0 },
  { "Json", 1.0 },
  { "Jssica", 0.0 },
  { "Jsutin", 1.0 },
  { "Jt", 1.0 },
  { "Ju", 0.33333334 },
  { "Jua", 1.0 },
  { "Juaita", 0.0 },
  { "Jual", 1.0 },
  { "Juan", 0.9920682 },
  { "Juana", 0.005053986 },
  { "Juanalberto", 1.0 },
  { "Juanandres", 1.0 },
  { "Juanangel", 1.0 },
  { "Juanantonio", 1.0 },
  { "Juancarlo", 1.0 },
  { "Juancarlos", 1.0 },
  { "Juanda", 0.0 },
  { "Juandalynn", 0.0 },
  { "Juandaniel", 1.0 },
  { "Juandavid", 1.0 },
  { "Juandedios", 1.0 },
  { "Juandiego", 1.0 },
  { "Juane", 1.0 },
  { "Juanell", 0.0 },
  { "Juanelle", 0.0 },
  { "Juanenrique", 1.0 },
  { "Juanesha", 0.0 },
  { "Juanesteban", 1.0 },
  { "Juanetta", 0.0 },
  { "Juanette", 0.0 },
  { "Juanfelipe", 1.0 },
  { "Juanfernando", 1.0 },
  { "Juanfrancisco", 1.0 },
  { "Juanice", 0.0 },
  { "Juanika", 0.0 },
  { "Juaniqua", 0.0 },
  { "Juanisha", 0.0 },
  { "Juanita", 0.0047642626 },
  { "Juanito", 0.99526966 },
  { "Juanitta", 0.0 },
  { "Juanjesus", 1.0 },
  { "Juanjose", 1.0 },
  { "Juanjr", 1.0 },
  { "Juanluis", 1.0 },
  { "Juanmanuel", 1.0 },
  { "Juanmarcos", 1.0 },
  { "Juanmartin", 1.0 },
  { "Juanmiguel", 1.0 },
  { "Juanna", 0.0 },
  { "Juanpablo", 1.0 },
  { "Juanpedro", 1.0 },
  { "Juanramon", 1.0 },
  { "Juansebastian", 1.0 },
  { "Juante", 1.0 },
  { "Juantia", 0.0 },
  { "Juany", 0.0 },
  { "Juanya", 1.0 },
  { "Juanyae", 1.0 },
  { "Juanye", 1.0 },
  { "Juaquin", 1.0 },
  { "Juaquina", 0.0 },
  { "Juarez", 1.0 },
  { "Juaria", 0.0 },
  { "Juawan", 1.0 },
  { "Juawana", 0.0 },
  { "Juba", 1.0 },
  { "Jubal", 1.0 },
  { "Jubei", 1.0 },
  { "Jubentino", 1.0 },
  { "Jubilee", 0.002948403 },
  { "Jubril", 1.0 },
  { "Jud", 1.0 },
  { "Juda", 0.6138107 },
  { "Judaea", 0.0 },
  { "Judah", 0.9732488 },
  { "Judaline", 0.0 },
  { "Judalon", 0.0 },
  { "Judas", 1.0 },
  { "Judd", 1.0 },
  { "Juddie", 0.4074074 },
  { "Juddson", 1.0 },
  { "Jude", 0.94298375 },
  { "Judea", 0.23414634 },
  { "Judeah", 0.0 },
  { "Judean", 0.0 },
  { "Judee", 0.0 },
  { "Judeen", 0.0 },
  { "Judell", 0.0 },
  { "Juden", 1.0 },
  { "Judene", 0.0 },
  { "Judeth", 0.0 },
  { "Judette", 0.0 },
  { "Judey", 0.0 },
  { "Judge", 1.0 },
  { "Judi", 0.0 },
  { "Judia", 0.0 },
  { "Judiah", 1.0 },
  { "Judiann", 0.0 },
  { "Judianne", 0.0 },
  { "Judie", 0.0 },
  { "Judiel", 1.0 },
  { "Judieth", 0.0 },
  { "Judilyn", 0.0 },
  { "Judilynn", 0.0 },
  { "Judine", 0.0 },
  { "Judit", 0.0 },
  { "Judite", 0.0 },
  { "Judith", 0.0027390972 },
  { "Judithann", 0.0 },
  { "Judithanne", 0.0 },
  { "Judithe", 0.0 },
  { "Judon", 1.0 },
  { "Judsen", 1.0 },
  { "Judson", 1.0 },
  { "Judtih", 0.0 },
  { "Judy", 0.0028572474 },
  { "Judyann", 0.0 },
  { "Judye", 0.0 },
  { "Judylynn", 0.0 },
  { "Judyne", 0.0 },
  { "Judyth", 0.0 },
  { "Judythe", 0.0 },
  { "Juel", 0.55813956 },
  { "Juelene", 0.0 },
  { "Juelez", 1.0 },
  { "Juelian", 1.0 },
  { "Juell", 0.8333333 },
  { "Juella", 0.0 },
  { "Juelle", 0.23188406 },
  { "Juellz", 1.0 },
  { "Juels", 1.0 },
  { "Juelz", 0.9560895 },
  { "Juelze", 1.0 },
  { "Juergen", 1.0 },
  { "Jugraj", 1.0 },
  { "Juhee", 0.0 },
  { "Juhi", 0.0 },
  { "Juila", 0.0 },
  { "Juile", 0.0 },
  { "Juiliana", 0.0 },
  { "Juilianna", 0.0 },
  { "Juilo", 1.0 },
  { "Jujhar", 1.0 },
  { "Jujuan", 0.98891354 },
  { "Jula", 0.0 },
  { "Julain", 1.0 },
  { "Julaine", 0.0 },
  { "Julana", 0.0 },
  { "Julane", 0.0 },
  { "Julani", 0.7368421 },
  { "Julann", 0.0 },
  { "Julanne", 0.0 },
  { "Julayne", 0.0 },
  { "Jule", 0.4128495 },
  { "Julea", 0.0 },
  { "Juleah", 0.0 },
  { "Julean", 1.0 },
  { "Juleana", 0.0 },
  { "Juleanna", 0.0 },
  { "Julee", 0.0 },
  { "Juleeanna", 0.0 },
  { "Juleen", 0.0 },
  { "Juleena", 0.0 },
  { "Juleesa", 0.0 },
  { "Juleidy", 0.0 },
  { "Juleigh", 0.0 },
  { "Juleimy", 0.0 },
  { "Julein", 1.0 },
  { "Juleisy", 0.0 },
  { "Julen", 1.0 },
  { "Julena", 0.0 },
  { "Julene", 0.0 },
  { "Julenny", 0.0 },
  { "Juleny", 0.0 },
  { "Juleon", 1.0 },
  { "Julep", 0.0 },
  { "Jules", 0.8710344 },
  { "Julessa", 0.0 },
  { "Juletta", 0.0 },
  { "Julette", 0.0 },
  { "Juleus", 1.0 },
  { "Juley", 0.0 },
  { "Julez", 0.90756303 },
  { "Juli", 0.0 },
  { "Julia", 0.0038121426 },
  { "Juliaann", 0.0 },
  { "Juliaanne", 0.0 },
  { "Juliaette", 0.0 },
  { "Juliagrace", 0.0 },
  { "Juliah", 0.0 },
  { "Juliahna", 0.0 },
  { "Juliahnna", 0.0 },
  { "Juliam", 1.0 },
  { "Juliamae", 0.0 },
  { "Juliamarie", 0.0 },
  { "Julian", 0.98453176 },
  { "Juliana", 0.00090399233 },
  { "Julianah", 0.0 },
  { "Juliane", 0.0 },
  { "Juliani", 0.2761194 },
  { "Julianie", 0.0 },
  { "Julianis", 0.0 },
  { "Julianita", 0.0 },
  { "Juliann", 0.004217687 },
  { "Julianna", 0.0006153118 },
  { "Juliannah", 0.0 },
  { "Juliannamarie", 0.0 },
  { "Julianne", 0.0 },
  { "Julianni", 0.1388889 },
  { "Juliannie", 0.0 },
  { "Julianno", 1.0 },
  { "Julianny", 0.0 },
  { "Juliano", 1.0 },
  { "Juliany", 0.0 },
  { "Julianys", 0.0 },
  { "Juliarose", 0.0 },
  { "Julias", 1.0 },
  { "Juliauna", 0.0 },
  { "Juliaunna", 0.0 },
  { "Julibeth", 0.0 },
  { "Julica", 0.0 },
  { "Julice", 1.0 },
  { "Julicia", 0.0 },
  { "Julie", 0.0029955243 },
  { "Juliea", 0.0 },
  { "Juliean", 1.0 },
  { "Julieana", 0.0 },
  { "Julieann", 0.0 },
  { "Julieanna", 0.0 },
  { "Julieanne", 0.0 },
  { "Juliella", 0.0 },
  { "Juliemarie", 0.0 },
  { "Julien", 0.981948 },
  { "Juliena", 0.0 },
  { "Juliene", 0.0 },
  { "Julienn", 1.0 },
  { "Julienna", 0.0 },
  { "Julienne", 0.012403101 },
  { "Julieonna", 0.0 },
  { "Julies", 1.0 },
  { "Juliessa", 0.0 },
  { "Juliet", 0.0 },
  { "Julieta", 0.0 },
  { "Juliete", 0.0 },
  { "Julieth", 0.0 },
  { "Juliett", 0.0 },
  { "Julietta", 0.0 },
  { "Juliette", 0.0 },
  { "Julieza", 0.0 },
  { "Julin", 0.0 },
  { "Julina", 0.0 },
  { "Julinda", 0.0 },
  { "Juline", 0.0 },
  { "Julio", 0.9929132 },
  { "Julioalberto", 1.0 },
  { "Juliocesar", 1.0 },
  { "Julion", 1.0 },
  { "Juliona", 0.0 },
  { "Julionna", 0.0 },
  { "Julious", 1.0 },
  { "Julis", 0.9778761 },
  { "Julisa", 0.0 },
  { "Julisha", 0.0 },
  { "Julisia", 0.0 },
  { "Julissa", 0.00047911494 },
  { "Julissia", 0.0 },
  { "Julita", 0.0 },
  { "Julitsa", 0.0 },
  { "Julitta", 0.0 },
  { "Julitza", 0.0 },
  { "Juliun", 1.0 },
  { "Julius", 0.99465287 },
  { "Juliuz", 1.0 },
  { "Julivette", 0.0 },
  { "Julixa", 0.0 },
  { "Juliya", 0.0 },
  { "Juliyah", 0.0 },
  { "Juliyan", 1.0 },
  { "Juliyana", 0.0 },
  { "Juliyanna", 0.0 },
  { "Juliza", 0.0 },
  { "Julizza", 0.0 },
  { "Jull", 0.0 },
  { "Jullia", 0.0 },
  { "Jullian", 0.7221571 },
  { "Julliana", 0.0 },
  { "Jullianna", 0.0 },
  { "Jullianne", 0.0 },
  { "Jullie", 0.0 },
  { "Jullien", 1.0 },
  { "Julliet", 0.0 },
  { "Julliette", 0.0 },
  { "Jullisa", 0.0 },
  { "Jullissa", 0.0 },
  { "Jullius", 1.0 },
  { "Jully", 0.0 },
  { "Julonda", 0.0 },
  { "Juluis", 1.0 },
  { "Julus", 1.0 },
  { "July", 0.18802395 },
  { "Julya", 0.0 },
  { "Julyan", 1.0 },
  { "Julyana", 0.0 },
  { "Julyanna", 0.0 },
  { "Julye", 0.0 },
  { "Julyen", 1.0 },
  { "Julyn", 0.0 },
  { "Julynn", 0.0 },
  { "Julysa", 0.0 },
  { "Julyssa", 0.0 },
  { "Juma", 1.0 },
  { "Jumaane", 1.0 },
  { "Jumah", 1.0 },
  { "Jumal", 1.0 },
  { "Jumalay", 0.0 },
  { "Juman", 0.0 },
  { "Jumana", 0.0 },
  { "Jumanah", 0.0 },
  { "Jumanji", 1.0 },
  { "Jumanne", 1.0 },
  { "Jumar", 1.0 },
  { "Jumarcus", 1.0 },
  { "Jumari", 1.0 },
  { "Jumarion", 1.0 },
  { "Jumel", 1.0 },
  { "Jumoke", 1.0 },
  { "Jun", 0.9239216 },
  { "Juna", 0.007575758 },
  { "Junae", 0.0 },
  { "Junah", 0.9189189 },
  { "Junaid", 1.0 },
  { "Junathaen", 1.0 },
  { "Junathean", 1.0 },
  { "Junayd", 1.0 },
  { "June", 0.016569914 },
  { "Junea", 0.0 },
  { "Juneann", 0.0 },
  { "Juneau", 0.125 },
  { "Junell", 0.0 },
  { "Junella", 0.0 },
  { "Junelle", 0.0 },
  { "Junellen", 0.0 },
  { "Junerose", 0.0 },
  { "Junetta", 0.0 },
  { "Junette", 0.0 },
  { "Juneve", 0.0 },
  { "Jung", 0.503268 },
  { "Jungwoo", 1.0 },
  { "Junhao", 1.0 },
  { "Junhee", 1.0 },
  { "Junho", 1.0 },
  { "Juni", 0.2 },
  { "Junia", 0.0 },
  { "Juniah", 0.0 },
  { "Juniata", 0.0 },
  { "Junice", 0.0 },
  { "Junichi", 1.0 },
  { "Junie", 0.10884831 },
  { "Juniel", 1.0 },
  { "Junies", 1.0 },
  { "Junietta", 0.0 },
  { "Junilla", 0.0 },
  { "Junior", 0.99515396 },
  { "Junious", 1.0 },
  { "Juniper", 0.009232155 },
  { "Junis", 0.627451 },
  { "Junita", 0.0 },
  { "Junito", 1.0 },
  { "Junius", 1.0 },
  { "Juniya", 0.0 },
  { "Juniyah", 0.0 },
  { "Junjie", 1.0 },
  { "Junko", 0.0 },
  { "Junming", 1.0 },
  { "Junne", 0.0 },
  { "Junnie", 0.13402061 },
  { "Juno", 0.21031208 },
  { "Junor", 1.0 },
  { "Junot", 1.0 },
  { "Junpei", 1.0 },
  { "Junuis", 1.0 },
  { "Junus", 1.0 },
  { "Junxi", 1.0 },
  { "Juny", 1.0 },
  { "Junya", 0.0 },
  { "Junyi", 1.0 },
  { "Junyu", 1.0 },
  { "Junzhe", 1.0 },
  { "Jupiter", 0.5448276 },
  { "Juquan", 1.0 },
  { "Juquana", 0.0 },
  { "Jura", 0.0 },
  { "Jurany", 0.0 },
  { "Jurate", 0.0 },
  { "Jurea", 0.0 },
  { "Jurel", 1.0 },
  { "Jurell", 0.9652778 },
  { "Jurem", 1.0 },
  { "Jurene", 0.0 },
  { "Juretta", 0.0 },
  { "Jurgen", 1.0 },
  { "Juri", 0.03787879 },
  { "Juriah", 1.0 },
  { "Jurian", 1.0 },
  { "Juriana", 0.0 },
  { "Juridia", 0.0 },
  { "Jurie", 0.0 },
  { "Juriel", 1.0 },
  { "Juris", 1.0 },
  { "Jurita", 0.0 },
  { "Juritza", 0.0 },
  { "Jurlean", 0.0 },
  { "Jurlene", 0.0 },
  { "Jurline", 0.0 },
  { "Jurnee", 0.0025454545 },
  { "Jurnei", 0.0 },
  { "Jurney", 0.027777778 },
  { "Jurni", 0.0 },
  { "Jurnie", 0.0 },
  { "Jurnii", 0.0 },
  { "Juron", 1.0 },
  { "Jurrell", 1.0 },
  { "Jurrien", 1.0 },
  { "Jury", 0.05357143 },
  { "Jushawn", 1.0 },
  { "Jushua", 1.0 },
  { "Jusiah", 1.0 },
  { "Jusin", 1.0 },
  { "Jusitn", 1.0 },
  { "Jussiah", 1.0 },
  { "Jussica", 0.0 },
  { "Justa", 0.0 },
  { "Justain", 1.0 },
  { "Justan", 1.0 },
  { "Justas", 1.0 },
  { "Justeen", 0.0 },
  { "Justeena", 0.0 },
  { "Justen", 0.9987345 },
  { "Justene", 0.0 },
  { "Justess", 0.0 },
  { "Justhin", 1.0 },
  { "Justi", 0.22727273 },
  { "Justian", 1.0 },
  { "Justic", 1.0 },
  { "Justice", 0.52253103 },
  { "Justien", 0.0 },
  { "Justilia", 0.0 },
  { "Justin", 0.99510014 },
  { "Justina", 0.000507872 },
  { "Justinanthony", 1.0 },
  { "Justine", 0.05990519 },
  { "Justinia", 0.0 },
  { "Justinian", 1.0 },
  { "Justiniano", 1.0 },
  { "Justinjames", 1.0 },
  { "Justinkyle", 1.0 },
  { "Justinlee", 1.0 },
  { "Justinmichael", 1.0 },
  { "Justinn", 1.0 },
  { "Justinne", 0.0 },
  { "Justino", 1.0 },
  { "Justinpaul", 1.0 },
  { "Justinray", 1.0 },
  { "Justinrobert", 1.0 },
  { "Justinryan", 1.0 },
  { "Justinthomas", 1.0 },
  { "Justion", 1.0 },
  { "Justis", 0.77654254 },
  { "Justise", 0.2112676 },
  { "Justiss", 0.11320755 },
  { "Justiz", 1.0 },
  { "Justn", 1.0 },
  { "Justo", 1.0 },
  { "Juston", 1.0 },
  { "Justun", 1.0 },
  { "Justus", 0.9191185 },
  { "Justy", 0.0 },
  { "Justyce", 0.20612583 },
  { "Justyn", 0.9647985 },
  { "Justyna", 0.0 },
  { "Justyne", 0.0 },
  { "Justys", 1.0 },
  { "Justyse", 0.0 },
  { "Jusuf", 1.0 },
  { "Jusus", 1.0 },
  { "Jutin", 1.0 },
  { "Jutta", 0.0 },
  { "Juvenal", 1.0 },
  { "Juvencio", 1.0 },
  { "Juvens", 1.0 },
  { "Juventina", 0.0 },
  { "Juventino", 1.0 },
  { "Juvia", 0.0 },
  { "Juvon", 1.0 },
  { "Juvonte", 1.0 },
  { "Juwaan", 1.0 },
  { "Juwahn", 1.0 },
  { "Juwairiyah", 0.0 },
  { "Juwan", 0.99721605 },
  { "Juwana", 0.0 },
  { "Juwanda", 0.0 },
  { "Juwann", 1.0 },
  { "Juwanna", 0.0 },
  { "Juwaun", 1.0 },
  { "Juwayria", 0.0 },
  { "Juwayriya", 0.0 },
  { "Juwel", 1.0 },
  { "Juwell", 1.0 },
  { "Juwelz", 1.0 },
  { "Juwon", 1.0 },
  { "Juwuan", 1.0 },
  { "Juyne", 0.0 },
  { "Juztin", 1.0 },
  { "Jvante", 1.0 },
  { "Jvaughn", 1.0 },
  { "Jven", 1.0 },
  { "Jveon", 1.0 },
  { "Jvian", 1.0 },
  { "Jvier", 1.0 },
  { "Jvion", 1.0 },
  { "Jvon", 1.0 },
  { "Jvonn", 1.0 },
  { "Jvonne", 1.0 },
  { "Jvontae", 1.0 },
  { "Jvonte", 1.0 },
  { "Jw", 1.0 },
  { "Jwan", 0.8143712 },
  { "Jwana", 0.0 },
  { "Jwaun", 1.0 },
  { "Jwon", 1.0 },
  { "Jwuan", 1.0 },
  { "Jwyanza", 1.0 },
  { "Jya", 0.0 },
  { "Jyah", 0.0 },
  { "Jyair", 1.0 },
  { "Jyaire", 1.0 },
  { "Jyana", 0.0 },
  { "Jyanna", 0.0 },
  { "Jyasia", 0.0 },
  { "Jyden", 1.0 },
  { "Jye", 1.0 },
  { "Jyelle", 0.0 },
  { "Jyena", 0.0 },
  { "Jyheem", 1.0 },
  { "Jyheim", 1.0 },
  { "Jyhiem", 1.0 },
  { "Jyia", 0.0 },
  { "Jyiah", 0.0 },
  { "Jyion", 1.0 },
  { "Jykeria", 0.0 },
  { "Jyl", 0.0 },
  { "Jyla", 0.0 },
  { "Jylan", 1.0 },
  { "Jylen", 1.0 },
  { "Jylene", 0.0 },
  { "Jyler", 1.0 },
  { "Jyles", 1.0 },
  { "Jylian", 0.0 },
  { "Jylin", 1.0 },
  { "Jylisa", 0.0 },
  { "Jylissa", 0.0 },
  { "Jyll", 0.0 },
  { "Jyllian", 0.0 },
  { "Jylon", 1.0 },
  { "Jymari", 1.0 },
  { "Jyme", 0.0 },
  { "Jymere", 1.0 },
  { "Jymir", 1.0 },
  { "Jymme", 0.0 },
  { "Jymon", 1.0 },
  { "Jynae", 0.0 },
  { "Jynell", 0.0 },
  { "Jynelle", 0.0 },
  { "Jynesis", 0.0 },
  { "Jynessa", 0.0 },
  { "Jynia", 0.0 },
  { "Jyniah", 0.0 },
  { "Jynna", 0.0 },
  { "Jyon", 1.0 },
  { "Jyonna", 0.0 },
  { "Jyoti", 0.0 },
  { "Jyotsna", 0.0 },
  { "Jyquan", 1.0 },
  { "Jyquavious", 1.0 },
  { "Jyquez", 1.0 },
  { "Jyra", 0.0 },
  { "Jyrah", 0.0 },
  { "Jyran", 1.0 },
  { "Jyree", 1.0 },
  { "Jyrell", 1.0 },
  { "Jyrelle", 1.0 },
  { "Jyren", 1.0 },
  { "Jyrese", 1.0 },
  { "Jyrin", 1.0 },
  { "Jyrine", 1.0 },
  { "Jyron", 1.0 },
  { "Jysaiah", 1.0 },
  { "Jyselle", 0.0 },
  { "Jyshaun", 1.0 },
  { "Jyshawn", 1.0 },
  { "Jyshon", 1.0 },
  { "Jysiah", 1.0 },
  { "Jysir", 1.0 },
  { "Jyson", 1.0 },
  { "Jyssica", 0.0 },
  { "Jyvon", 1.0 },
  { "Jywon", 1.0 },
  { "Jyzaiah", 1.0 },
  { "Jyzelle", 0.0 },
  { "Jzabehl", 0.0 },
  { "Jziah", 1.0 },
  { "Jzon", 1.0 },
  { "Ka", 0.14364035 },
  { "Kaaden", 1.0 },
  { "Kaajal", 0.0 },
  { "Kaala", 0.0 },
  { "Kaaliyah", 0.0 },
  { "Kaalyn", 0.0 },
  { "Kaamil", 1.0 },
  { "Kaamilah", 0.0 },
  { "Kaamilya", 0.0 },
  { "Kaan", 1.0 },
  { "Kaara", 0.0 },
  { "Kaaran", 0.0 },
  { "Kaare", 1.0 },
  { "Kaaren", 0.0 },
  { "Kaari", 0.0 },
  { "Kaarin", 0.0 },
  { "Kaarina", 0.0 },
  { "Kaarlo", 1.0 },
  { "Kaaron", 0.0 },
  { "Kaashif", 1.0 },
  { "Kaashvi", 0.0 },
  { "Kaavya", 0.0 },
  { "Kaaya", 0.0 },
  { "Kaayla", 0.0 },
  { "Kabao", 0.0 },
  { "Kabe", 1.0 },
  { "Kabeer", 1.0 },
  { "Kabel", 1.0 },
  { "Kabella", 0.0 },
  { "Kaben", 1.0 },
  { "Kabir", 1.0 },
  { "Kable", 1.0 },
  { "Kabrea", 0.0 },
  { "Kabree", 0.0 },
  { "Kabren", 1.0 },
  { "Kabresha", 0.0 },
  { "Kabreyia", 0.0 },
  { "Kabri", 0.0 },
  { "Kabria", 0.0 },
  { "Kabriel", 1.0 },
  { "Kabrina", 0.0 },
  { "Kabron", 1.0 },
  { "Kabryn", 1.0 },
  { "Kace", 0.98029554 },
  { "Kacee", 0.08885942 },
  { "Kacei", 0.0 },
  { "Kacelyn", 0.0 },
  { "Kacelynn", 0.0 },
  { "Kacen", 1.0 },
  { "Kaceon", 1.0 },
  { "Kaceson", 1.0 },
  { "Kaceton", 1.0 },
  { "Kacey", 0.16083688 },
  { "Kaceyon", 1.0 },
  { "Kache", 0.56153846 },
  { "Kachelle", 0.0 },
  { "Kachet", 0.0 },
  { "Kachina", 0.0 },
  { "Kachiri", 0.0 },
  { "Kaci", 0.009093934 },
  { "Kacia", 0.0 },
  { "Kacie", 0.0026956447 },
  { "Kacin", 1.0 },
  { "Kacper", 1.0 },
  { "Kacy", 0.15968858 },
  { "Kacyn", 0.6910569 },
  { "Kada", 0.0 },
  { "Kadaija", 0.0 },
  { "Kadaisha", 0.0 },
  { "Kadaja", 0.0 },
  { "Kadajah", 0.0 },
  { "Kadan", 0.98419863 },
  { "Kadance", 0.021437578 },
  { "Kadar", 1.0 },
  { "Kadari", 1.0 },
  { "Kadarian", 1.0 },
  { "Kadarion", 1.0 },
  { "Kadarious", 1.0 },
  { "Kadarius", 1.0 },
  { "Kadarrius", 1.0 },
  { "Kadasha", 0.0 },
  { "Kadashia", 0.0 },
  { "Kadasia", 0.0 },
  { "Kadavion", 1.0 },
  { "Kadaysha", 0.0 },
  { "Kadaysia", 0.0 },
  { "Kadden", 1.0 },
  { "Kaddie", 0.0 },
  { "Kaddy", 0.0 },
  { "Kade", 0.9881826 },
  { "Kadean", 1.0 },
  { "Kadedra", 0.0 },
  { "Kadedria", 0.0 },
  { "Kadee", 0.0 },
  { "Kadeedra", 0.0 },
  { "Kadeeja", 0.0 },
  { "Kadeejah", 0.0 },
  { "Kadeem", 1.0 },
  { "Kadeen", 1.0 },
  { "Kadeesha", 0.0 },
  { "Kadeidra", 0.0 },
  { "Kadeidre", 0.0 },
  { "Kadeija", 0.0 },
  { "Kadeijah", 0.0 },
  { "Kadein", 1.0 },
  { "Kadeisha", 0.0 },
  { "Kadeja", 0.0 },
  { "Kadejah", 0.0 },
  { "Kadejha", 0.0 },
  { "Kadejia", 0.0 },
  { "Kadejiah", 0.0 },
  { "Kadel", 1.0 },
  { "Kadell", 1.0 },
  { "Kadelyn", 0.0 },
  { "Kaden", 0.982328 },
  { "Kadence", 0.08114056 },
  { "Kadenn", 1.0 },
  { "Kadenze", 0.0 },
  { "Kader", 1.0 },
  { "Kadera", 0.0 },
  { "Kaderius", 1.0 },
  { "Kadesh", 1.0 },
  { "Kadesha", 0.0 },
  { "Kadeshia", 0.0 },
  { "Kadesia", 0.0 },
  { "Kadetra", 0.0 },
  { "Kadetria", 0.0 },
  { "Kadey", 0.0 },
  { "Kadeyn", 1.0 },
  { "Kadezha", 0.0 },
  { "Kadezia", 0.0 },
  { "Kadi", 0.0 },
  { "Kadia", 0.0 },
  { "Kadian", 0.33587787 },
  { "Kadiatou", 0.0 },
  { "Kadiatu", 0.0 },
  { "Kadidia", 0.0 },
  { "Kadidiatou", 0.0 },
  { "Kadidja", 0.0 },
  { "Kadidra", 0.0 },
  { "Kadie", 0.0 },
  { "Kadiedra", 0.0 },
  { "Kadien", 1.0 },
  { "Kadience", 0.0 },
  { "Kadiesha", 0.0 },
  { "Kadija", 0.0 },
  { "Kadijah", 0.0 },
  { "Kadijatou", 0.0 },
  { "Kadijatu", 0.0 },
  { "Kadijha", 0.0 },
  { "Kadin", 0.9639895 },
  { "Kadince", 0.0 },
  { "Kadir", 1.0 },
  { "Kadira", 0.0 },
  { "Kadis", 1.0 },
  { "Kadisha", 0.0 },
  { "Kadison", 0.42857143 },
  { "Kadjah", 0.0 },
  { "Kadmiel", 1.0 },
  { "Kado", 1.0 },
  { "Kadon", 1.0 },
  { "Kadra", 0.0 },
  { "Kadraya", 0.0 },
  { "Kadren", 1.0 },
  { "Kadri", 0.61538464 },
  { "Kadria", 0.0 },
  { "Kadrian", 0.9632107 },
  { "Kadriana", 0.0 },
  { "Kadrianna", 0.0 },
  { "Kadrien", 1.0 },
  { "Kady", 0.0 },
  { "Kadyn", 0.6759329 },
  { "Kadynce", 0.007151371 },
  { "Kadynn", 0.18309858 },
  { "Kae", 0.0 },
  { "Kaebri", 0.0 },
  { "Kaecyn", 1.0 },
  { "Kaed", 1.0 },
  { "Kaeda", 0.0 },
  { "Kaedan", 0.9848024 },
  { "Kaedance", 0.0 },
  { "Kaede", 0.15486726 },
  { "Kaeden", 0.9626124 },
  { "Kaedence", 0.026970955 },
  { "Kaedin", 0.98571426 },
  { "Kaedn", 1.0 },
  { "Kaedon", 1.0 },
  { "Kaedyn", 0.6451807 },
  { "Kaedynce", 0.0 },
  { "Kaedynn", 0.23809524 },
  { "Kaegan", 0.64028776 },
  { "Kaegen", 1.0 },
  { "Kaeisha", 0.0 },
  { "Kael", 0.99476266 },
  { "Kaela", 0.0 },
  { "Kaelah", 0.0 },
  { "Kaelan", 0.76807404 },
  { "Kaelana", 0.0 },
  { "Kaelani", 0.0 },
  { "Kaele", 0.0 },
  { "Kaelea", 0.0 },
  { "Kaeleah", 0.0 },
  { "Kaeleb", 1.0 },
  { "Kaelee", 0.0 },
  { "Kaeleen", 0.0 },
  { "Kaelei", 0.0 },
  { "Kaeleigh", 0.0 },
  { "Kaelem", 1.0 },
  { "Kaelen", 0.6468717 },
  { "Kaelene", 0.0 },
  { "Kaeley", 0.0 },
  { "Kaeli", 0.0 },
  { "Kaelia", 0.0 },
  { "Kaeliana", 0.0 },
  { "Kaelib", 1.0 },
  { "Kaelie", 0.0 },
  { "Kaeliegh", 0.0 },
  { "Kaelin", 0.24363425 },
  { "Kaella", 0.0 },
  { "Kaelly", 0.0 },
  { "Kaelob", 1.0 },
  { "Kaelon", 1.0 },
  { "Kaeloni", 0.0 },
  { "Kaelub", 1.0 },
  { "Kaelum", 1.0 },
  { "Kaely", 0.0 },
  { "Kaelyb", 1.0 },
  { "Kaelyn", 0.005568412 },
  { "Kaelyne", 0.0 },
  { "Kaelynn", 0.0 },
  { "Kaelynne", 0.0 },
  { "Kaeman", 1.0 },
  { "Kaemen", 1.0 },
  { "Kaemon", 1.0 },
  { "Kaen", 1.0 },
  { "Kaena", 0.20224719 },
  { "Kaenan", 1.0 },
  { "Kaenon", 1.0 },
  { "Kaeo", 1.0 },
  { "Kaeon", 1.0 },
  { "Kaeron", 1.0 },
  { "Kaery", 0.0 },
  { "Kaesen", 1.0 },
  { "Kaesha", 0.0 },
  { "Kaeshawn", 1.0 },
  { "Kaeshon", 1.0 },
  { "Kaesin", 1.0 },
  { "Kaeson", 1.0 },
  { "Kaeston", 1.0 },
  { "Kaesyn", 0.8224852 },
  { "Kaethe", 0.0 },
  { "Kaetlin", 0.0 },
  { "Kaetlyn", 0.0 },
  { "Kaetlynn", 0.0 },
  { "Kaeto", 1.0 },
  { "Kaeveon", 1.0 },
  { "Kaevion", 1.0 },
  { "Kaevon", 1.0 },
  { "Kaeya", 0.0 },
  { "Kaeyla", 0.0 },
  { "Kaffie", 0.0 },
  { "Kafi", 0.0 },
  { "Kagan", 0.8862928 },
  { "Kage", 1.0 },
  { "Kagen", 0.9664694 },
  { "Kager", 1.0 },
  { "Kagome", 0.0 },
  { "Kahaan", 1.0 },
  { "Kahala", 0.0 },
  { "Kahan", 1.0 },
  { "Kahana", 0.0 },
  { "Kahani", 0.0 },
  { "Kahanuola", 1.0 },
  { "Kahari", 0.9821883 },
  { "Kahdeem", 1.0 },
  { "Kahdejah", 0.0 },
  { "Kahden", 1.0 },
  { "Kahdijah", 0.0 },
  { "Kahea", 0.0 },
  { "Kahealani", 0.0 },
  { "Kaheem", 1.0 },
  { "Kaheim", 1.0 },
  { "Kahekili", 1.0 },
  { "Kahia", 0.0 },
  { "Kahiau", 1.0 },
  { "Kahiem", 1.0 },
  { "Kahil", 1.0 },
  { "Kahila", 0.0 },
  { "Kahir", 1.0 },
  { "Kahl", 1.0 },
  { "Kahla", 0.0 },
  { "Kahlan", 0.011876484 },
  { "Kahlani", 0.0 },
  { "Kahlaya", 0.0 },
  { "Kahle", 1.0 },
  { "Kahlea", 0.0 },
  { "Kahleah", 0.0 },
  { "Kahleb", 1.0 },
  { "Kahlee", 0.25688073 },
  { "Kahleel", 1.0 },
  { "Kahleesi", 0.0 },
  { "Kahlei", 0.0 },
  { "Kahleia", 0.0 },
  { "Kahleil", 1.0 },
  { "Kahlel", 1.0 },
  { "Kahlen", 0.2005988 },
  { "Kahler", 0.8695652 },
  { "Kahley", 0.0 },
  { "Kahleya", 0.0 },
  { "Kahli", 0.21493213 },
  { "Kahlia", 0.0 },
  { "Kahliah", 0.0 },
  { "Kahlid", 1.0 },
  { "Kahlie", 0.0 },
  { "Kahliel", 1.0 },
  { "Kahlif", 1.0 },
  { "Kahlil", 1.0 },
  { "Kahlila", 0.0 },
  { "Kahlilah", 0.0 },
  { "Kahlill", 1.0 },
  { "Kahlin", 0.8428571 },
  { "Kahlina", 0.0 },
  { "Kahlis", 0.0 },
  { "Kahliya", 0.0 },
  { "Kahliyah", 0.0 },
  { "Kahlo", 1.0 },
  { "Kahloni", 0.0 },
  { "Kahlyn", 0.0 },
  { "Kahlynn", 0.0 },
  { "Kahmal", 1.0 },
  { "Kahmani", 0.78 },
  { "Kahmar", 1.0 },
  { "Kahmari", 0.7408638 },
  { "Kahmel", 1.0 },
  { "Kahmia", 0.0 },
  { "Kahmiah", 0.0 },
  { "Kahmila", 0.0 },
  { "Kahmir", 1.0 },
  { "Kahmiya", 0.0 },
  { "Kahmiyah", 0.0 },
  { "Kahmora", 0.0 },
  { "Kahmya", 0.0 },
  { "Kahmyah", 0.0 },
  { "Kahmyra", 0.0 },
  { "Kahn", 1.0 },
  { "Kahne", 0.973822 },
  { "Kahner", 1.0 },
  { "Kahniya", 0.0 },
  { "Kahniyah", 0.0 },
  { "Kahnye", 1.0 },
  { "Kaho", 0.0 },
  { "Kahree", 1.0 },
  { "Kahreem", 1.0 },
  { "Kahri", 0.5915493 },
  { "Kahrin", 0.0 },
  { "Kahron", 1.0 },
  { "Kahryn", 0.0 },
  { "Kahseem", 1.0 },
  { "Kahtai", 1.0 },
  { "Kahyla", 0.0 },
  { "Kai", 0.8527915 },
  { "Kaia", 0.0037965071 },
  { "Kaiah", 0.0 },
  { "Kaian", 1.0 },
  { "Kaiana", 0.0 },
  { "Kaiann", 0.0 },
  { "Kaianna", 0.0 },
  { "Kaianne", 0.0 },
  { "Kaiba", 1.0 },
  { "Kaibree", 0.0 },
  { "Kaibren", 1.0 },
  { "Kaicee", 0.0 },
  { "Kaicen", 1.0 },
  { "Kaicey", 0.0 },
  { "Kaicy", 0.0 },
  { "Kaicyn", 0.7058824 },
  { "Kaid", 1.0 },
  { "Kaida", 0.0 },
  { "Kaidan", 0.9311828 },
  { "Kaidance", 0.0 },
  { "Kaide", 1.0 },
  { "Kaidee", 0.0 },
  { "Kaiden", 0.9592352 },
  { "Kaidence", 0.08733778 },
  { "Kaidenn", 1.0 },
  { "Kaidin", 0.97089946 },
  { "Kaidince", 0.0 },
  { "Kaido", 1.0 },
  { "Kaidon", 1.0 },
  { "Kaidra", 0.0 },
  { "Kaidy", 0.0 },
  { "Kaidyn", 0.61370826 },
  { "Kaidynce", 0.017910447 },
  { "Kaidynn", 0.35135135 },
  { "Kaie", 1.0 },
  { "Kaiea", 0.9285714 },
  { "Kaiel", 1.0 },
  { "Kaielle", 0.0 },
  { "Kaien", 1.0 },
  { "Kaiesha", 0.0 },
  { "Kaif", 1.0 },
  { "Kaigan", 0.33333334 },
  { "Kaige", 0.9859155 },
  { "Kaigen", 0.8684211 },
  { "Kaihan", 1.0 },
  { "Kaii", 1.0 },
  { "Kaiicen", 1.0 },
  { "Kaiiden", 1.0 },
  { "Kaiis", 1.0 },
  { "Kaija", 0.0 },
  { "Kaijah", 0.0 },
  { "Kaikane", 1.0 },
  { "Kaikea", 1.0 },
  { "Kaikoa", 1.0 },
  { "Kaiky", 1.0 },
  { "Kail", 0.93835616 },
  { "Kaila", 0.0 },
  { "Kailah", 0.0 },
  { "Kailahni", 0.0 },
  { "Kailan", 0.41884816 },
  { "Kailana", 0.0 },
  { "Kailand", 0.8194444 },
  { "Kailani", 0.005154639 },
  { "Kailanie", 0.0 },
  { "Kailanni", 0.0 },
  { "Kailanny", 0.0 },
  { "Kailany", 0.0 },
  { "Kailar", 1.0 },
  { "Kailas", 1.0 },
  { "Kailash", 1.0 },
  { "Kailaya", 0.0 },
  { "Kailayah", 0.0 },
  { "Kaile", 0.030726258 },
  { "Kailea", 0.0 },
  { "Kaileah", 0.0 },
  { "Kaileb", 1.0 },
  { "Kailee", 0.002015599 },
  { "Kaileeann", 0.0 },
  { "Kaileemarie", 0.0 },
  { "Kaileen", 0.0 },
  { "Kaileena", 0.0 },
  { "Kailei", 0.0 },
  { "Kaileia", 0.0 },
  { "Kaileigh", 0.0 },
  { "Kaileigha", 0.0 },
  { "Kailen", 0.49658832 },
  { "Kailena", 0.0 },
  { "Kailene", 0.0 },
  { "Kaileo", 1.0 },
  { "Kailer", 0.9513889 },
  { "Kailey", 0.0004064545 },
  { "Kaili", 0.015401954 },
  { "Kailia", 0.0 },
  { "Kailiah", 0.0 },
  { "Kailiana", 0.0 },
  { "Kailianna", 0.0 },
  { "Kailib", 1.0 },
  { "Kailie", 0.0 },
  { "Kailin", 0.11372867 },
  { "Kailina", 0.0 },
  { "Kailiyah", 0.0 },
  { "Kailla", 0.0 },
  { "Kaille", 0.0 },
  { "Kaillou", 1.0 },
  { "Kailly", 0.0 },
  { "Kailo", 1.0 },
  { "Kailob", 1.0 },
  { "Kailon", 0.96517414 },
  { "Kailoni", 0.0 },
  { "Kailor", 0.5833333 },
  { "Kailub", 1.0 },
  { "Kailum", 1.0 },
  { "Kaily", 0.0 },
  { "Kailye", 0.0 },
  { "Kailyn", 0.0061437804 },
  { "Kailyne", 0.0 },
  { "Kailynn", 0.0 },
  { "Kailynne", 0.0 },
  { "Kaimalu", 1.0 },
  { "Kaiman", 1.0 },
  { "Kaimana", 0.9145183 },
  { "Kaimani", 0.9019608 },
  { "Kaimarie", 0.0 },
  { "Kaimen", 1.0 },
  { "Kaimere", 1.0 },
  { "Kaimi", 0.59375 },
  { "Kaimilani", 0.0 },
  { "Kaiming", 1.0 },
  { "Kaimipono", 1.0 },
  { "Kaimya", 0.0 },
  { "Kain", 1.0 },
  { "Kaina", 0.0 },
  { "Kainaan", 1.0 },
  { "Kainaat", 0.0 },
  { "Kainalu", 1.0 },
  { "Kainan", 1.0 },
  { "Kainani", 0.0 },
  { "Kainat", 0.0 },
  { "Kaine", 1.0 },
  { "Kainen", 1.0 },
  { "Kainin", 1.0 },
  { "Kainoa", 0.9856 },
  { "Kainoah", 1.0 },
  { "Kainon", 1.0 },
  { "Kainyn", 1.0 },
  { "Kaio", 1.0 },
  { "Kaion", 1.0 },
  { "Kaiona", 0.0 },
  { "Kaipo", 1.0 },
  { "Kaique", 1.0 },
  { "Kaira", 0.0 },
  { "Kairaba", 1.0 },
  { "Kairah", 0.0 },
  { "Kairan", 1.0 },
  { "Kairav", 1.0 },
  { "Kaire", 0.9047619 },
  { "Kairee", 0.53649634 },
  { "Kairen", 1.0 },
  { "Kairi", 0.07431694 },
  { "Kairie", 0.0 },
  { "Kairo", 0.98537236 },
  { "Kairon", 1.0 },
  { "Kairos", 1.0 },
  { "Kairy", 0.047244094 },
  { "Kairyn", 0.8333333 },
  { "Kais", 1.0 },
  { "Kaisa", 0.0 },
  { "Kaisan", 1.0 },
  { "Kaise", 1.0 },
  { "Kaisean", 1.0 },
  { "Kaisee", 0.0 },
  { "Kaisei", 1.0 },
  { "Kaisen", 0.9911894 },
  { "Kaiser", 0.9960968 },
  { "Kaisey", 0.0 },
  { "Kaisha", 0.0 },
  { "Kaishaun", 1.0 },
  { "Kaishawn", 1.0 },
  { "Kaishon", 1.0 },
  { "Kaishonta", 0.0 },
  { "Kaisin", 1.0 },
  { "Kaislee", 0.0 },
  { "Kaisleigh", 0.0 },
  { "Kaisley", 0.0 },
  { "Kaislyn", 0.0 },
  { "Kaislynn", 0.0 },
  { "Kaison", 0.99751985 },
  { "Kaiston", 1.0 },
  { "Kaisy", 0.0 },
  { "Kaisyn", 0.65789473 },
  { "Kait", 0.0 },
  { "Kaite", 0.0 },
  { "Kaitee", 0.0 },
  { "Kaitelyn", 0.0 },
  { "Kaitelynn", 0.0 },
  { "Kaitey", 0.0 },
  { "Kaithleen", 0.0 },
  { "Kaithlin", 0.0 },
  { "Kaithlyn", 0.0 },
  { "Kaithlynn", 0.0 },
  { "Kaiti", 0.0 },
  { "Kaitie", 0.0 },
  { "Kaitin", 0.0 },
  { "Kaitlain", 0.0 },
  { "Kaitlan", 0.0 },
  { "Kaitland", 0.0 },
  { "Kaitlee", 0.0 },
  { "Kaitleen", 0.0 },
  { "Kaitlen", 0.0 },
  { "Kaitley", 0.0 },
  { "Kaitlin", 0.0013154203 },
  { "Kaitlind", 0.0 },
  { "Kaitlinn", 0.0 },
  { "Kaitlon", 0.0 },
  { "Kaitly", 0.0 },
  { "Kaitlyn", 0.0014362746 },
  { "Kaitlynd", 0.0 },
  { "Kaitlyne", 0.0 },
  { "Kaitlynmarie", 0.0 },
  { "Kaitlynn", 0.0003185728 },
  { "Kaitlynne", 0.0 },
  { "Kaito", 1.0 },
  { "Kaitrin", 0.0 },
  { "Kaitryn", 0.0 },
  { "Kaity", 0.0 },
  { "Kaityln", 0.0 },
  { "Kaitylyn", 0.0 },
  { "Kaityn", 0.0 },
  { "Kaiulani", 0.0 },
  { "Kaius", 1.0 },
  { "Kaivan", 1.0 },
  { "Kaiven", 1.0 },
  { "Kaiveon", 1.0 },
  { "Kaivion", 1.0 },
  { "Kaivon", 1.0 },
  { "Kaiwen", 1.0 },
  { "Kaiya", 0.0054835496 },
  { "Kaiyah", 0.0 },
  { "Kaiyan", 1.0 },
  { "Kaiyana", 0.0 },
  { "Kaiyanna", 0.0 },
  { "Kaiyden", 0.95238096 },
  { "Kaiyel", 1.0 },
  { "Kaiyen", 1.0 },
  { "Kaiyla", 0.0 },
  { "Kaiylee", 0.0 },
  { "Kaiyon", 1.0 },
  { "Kaiyonna", 0.0 },
  { "Kaiyu", 1.0 },
  { "Kaiza", 0.0 },
  { "Kaizen", 1.0 },
  { "Kaizer", 1.0 },
  { "Kaizlee", 0.0 },
  { "Kaizleigh", 0.0 },
  { "Kaizley", 0.0 },
  { "Kaizlyn", 0.0 },
  { "Kaj", 1.0 },
  { "Kaja", 0.0 },
  { "Kajah", 0.0 },
  { "Kajal", 0.0 },
  { "Kajana", 1.0 },
  { "Kajaun", 1.0 },
  { "Kaje", 1.0 },
  { "Kajetan", 1.0 },
  { "Kajia", 0.0 },
  { "Kajira", 0.0 },
  { "Kajol", 0.0 },
  { "Kajon", 1.0 },
  { "Kajsa", 0.0 },
  { "Kajuan", 1.0 },
  { "Kajuana", 0.0 },
  { "Kajun", 0.0 },
  { "Kajus", 1.0 },
  { "Kakisha", 0.0 },
  { "Kal", 1.0 },
  { "Kala", 0.012070322 },
  { "Kalab", 1.0 },
  { "Kaladin", 1.0 },
  { "Kalae", 0.29411766 },
  { "Kalaeb", 1.0 },
  { "Kalah", 0.0 },
  { "Kalahikiola", 1.0 },
  { "Kalahni", 0.0 },
  { "Kalai", 0.82051283 },
  { "Kalaia", 0.0 },
  { "Kalaiah", 0.0 },
  { "Kalaijah", 0.0 },
  { "Kalaila", 0.0 },
  { "Kalaina", 0.0 },
  { "Kalaiya", 0.0 },
  { "Kalaiyah", 0.0 },
  { "Kalajah", 0.0 },
  { "Kalan", 0.82273746 },
  { "Kalana", 0.0 },
  { "Kalandra", 0.0 },
  { "Kalandria", 0.0 },
  { "Kalani", 0.3568173 },
  { "Kalanie", 0.0 },
  { "Kalanii", 0.0 },
  { "Kalanni", 0.0 },
  { "Kalany", 0.0 },
  { "Kalashia", 0.0 },
  { "Kalasia", 0.0 },
  { "Kalauni", 0.0 },
  { "Kalaya", 0.0 },
  { "Kalayah", 0.0 },
  { "Kalayia", 0.0 },
  { "Kalayiah", 0.0 },
  { "Kalayla", 0.0 },
  { "Kalayna", 0.0 },
  { "Kalaysha", 0.0 },
  { "Kalayshia", 0.0 },
  { "Kalaysia", 0.0 },
  { "Kalden", 1.0 },
  { "Kalder", 1.0 },
  { "Kale", 0.9692475 },
  { "Kalea", 0.0 },
  { "Kaleab", 1.0 },
  { "Kaleah", 0.0 },
  { "Kaleaha", 0.0 },
  { "Kaleal", 1.0 },
  { "Kalean", 0.0 },
  { "Kaleana", 0.0 },
  { "Kaleb", 0.99736524 },
  { "Kalecia", 0.0 },
  { "Kaled", 1.0 },
  { "Kalee", 0.017923037 },
  { "Kaleea", 0.0 },
  { "Kaleeah", 0.0 },
  { "Kaleeb", 1.0 },
  { "Kaleece", 0.0 },
  { "Kaleef", 1.0 },
  { "Kaleek", 1.0 },
  { "Kaleel", 1.0 },
  { "Kaleela", 0.0 },
  { "Kaleem", 1.0 },
  { "Kaleema", 0.0 },
  { "Kaleemah", 0.0 },
  { "Kaleen", 0.0 },
  { "Kaleena", 0.0 },
  { "Kaleese", 0.0 },
  { "Kaleesha", 0.0 },
  { "Kaleesi", 0.0 },
  { "Kaleeya", 0.0 },
  { "Kaleeyah", 0.0 },
  { "Kalei", 0.09528908 },
  { "Kaleia", 0.0 },
  { "Kaleiah", 0.0 },
  { "Kaleigh", 0.0003938817 },
  { "Kaleigha", 0.0 },
  { "Kaleil", 1.0 },
  { "Kaleila", 0.0 },
  { "Kaleina", 0.0 },
  { "Kaleinani", 0.0 },
  { "Kaleis", 0.0 },
  { "Kaleisha", 0.0 },
  { "Kaleiya", 0.0 },
  { "Kaleiyah", 0.0 },
  { "Kalek", 1.0 },
  { "Kalel", 1.0 },
  { "Kalell", 1.0 },
  { "Kalem", 1.0 },
  { "Kalen", 0.7358002 },
  { "Kalena", 0.0 },
  { "Kalene", 0.0 },
  { "Kalenna", 0.0 },
  { "Kaleo", 1.0 },
  { "Kaleob", 1.0 },
  { "Kalep", 1.0 },
  { "Kaler", 1.0 },
  { "Kalesa", 0.0 },
  { "Kalese", 0.0 },
  { "Kalesha", 0.0 },
  { "Kaleshia", 0.0 },
  { "Kalesia", 0.0 },
  { "Kalet", 1.0 },
  { "Kaleta", 0.0 },
  { "Kaleth", 1.0 },
  { "Kaletha", 0.0 },
  { "Kalev", 1.0 },
  { "Kalex", 1.0 },
  { "Kalexy", 0.0 },
  { "Kaley", 0.001944466 },
  { "Kaleya", 0.0 },
  { "Kaleyah", 0.0 },
  { "Kali", 0.026455026 },
  { "Kalia", 0.0 },
  { "Kaliah", 0.0 },
  { "Kaliana", 0.0 },
  { "Kaliann", 0.0 },
  { "Kalianna", 0.0 },
  { "Kaliannah", 0.0 },
  { "Kalianne", 0.0 },
  { "Kalib", 1.0 },
  { "Kaliber", 1.0 },
  { "Kalice", 0.0 },
  { "Kalicia", 0.0 },
  { "Kalid", 1.0 },
  { "Kalie", 0.0 },
  { "Kaliea", 0.0 },
  { "Kalieah", 0.0 },
  { "Kalieb", 1.0 },
  { "Kaliece", 0.0 },
  { "Kaliee", 0.0 },
  { "Kalief", 1.0 },
  { "Kaliegh", 0.0 },
  { "Kaliel", 1.0 },
  { "Kaliese", 0.0 },
  { "Kaliesha", 0.0 },
  { "Kaliey", 0.0 },
  { "Kalif", 1.0 },
  { "Kalifa", 0.0 },
  { "Kalifah", 0.0 },
  { "Kaliff", 1.0 },
  { "Kalifornia", 0.0 },
  { "Kaliha", 0.0 },
  { "Kalii", 0.0 },
  { "Kalijah", 0.72083336 },
  { "Kalik", 1.0 },
  { "Kalika", 0.0 },
  { "Kalil", 1.0 },
  { "Kalila", 0.0 },
  { "Kalilah", 0.0 },
  { "Kalill", 1.0 },
  { "Kalim", 1.0 },
  { "Kalima", 0.0 },
  { "Kalimah", 0.0 },
  { "Kalimba", 0.0 },
  { "Kalin", 0.5340803 },
  { "Kalina", 0.0 },
  { "Kalinah", 0.0 },
  { "Kalinda", 0.0 },
  { "Kaline", 0.0 },
  { "Kalino", 1.0 },
  { "Kaliona", 0.0 },
  { "Kalionna", 0.0 },
  { "Kaliope", 0.0 },
  { "Kaliopi", 0.0 },
  { "Kalip", 1.0 },
  { "Kaliph", 1.0 },
  { "Kaliq", 1.0 },
  { "Kalique", 1.0 },
  { "Kalirose", 0.0 },
  { "Kalis", 0.022624435 },
  { "Kalisa", 0.0 },
  { "Kalise", 0.0 },
  { "Kalisee", 0.0 },
  { "Kalisha", 0.0 },
  { "Kalishia", 0.0 },
  { "Kalisi", 0.0 },
  { "Kaliska", 0.0 },
  { "Kalissa", 0.0 },
  { "Kalisse", 0.0 },
  { "Kalista", 0.0 },
  { "Kalita", 0.0 },
  { "Kalix", 1.0 },
  { "Kaliya", 0.0 },
  { "Kaliyah", 0.0006554798 },
  { "Kaliyan", 0.0 },
  { "Kaliyana", 0.0 },
  { "Kaliyanei", 0.0 },
  { "Kaliyanna", 0.0 },
  { "Kaliyha", 0.0 },
  { "Kaliym", 1.0 },
  { "Kalki", 0.0 },
  { "Kalkidan", 0.0 },
  { "Kalla", 0.0 },
  { "Kallan", 0.5448819 },
  { "Kalle", 0.09217877 },
  { "Kallea", 0.0 },
  { "Kalleb", 1.0 },
  { "Kallee", 0.0 },
  { "Kalleen", 0.0 },
  { "Kalleigh", 0.0 },
  { "Kallel", 1.0 },
  { "Kallen", 0.8416115 },
  { "Kalley", 0.0 },
  { "Kalli", 0.0 },
  { "Kallia", 0.0 },
  { "Kalliann", 0.0 },
  { "Kallie", 0.0 },
  { "Kallieann", 0.0 },
  { "Kallin", 0.84713376 },
  { "Kallina", 0.0 },
  { "Kalliope", 0.0 },
  { "Kalliopi", 0.0 },
  { "Kallissa", 0.0 },
  { "Kallista", 0.0 },
  { "Kalliyah", 0.0 },
  { "Kalliyan", 0.0 },
  { "Kallon", 1.0 },
  { "Kallum", 1.0 },
  { "Kally", 0.003875969 },
  { "Kallye", 0.0 },
  { "Kallyn", 0.0 },
  { "Kallysta", 0.0 },
  { "Kalman", 1.0 },
  { "Kalmen", 1.0 },
  { "Kalmer", 1.0 },
  { "Kalo", 1.0 },
  { "Kalob", 1.0 },
  { "Kalobe", 1.0 },
  { "Kalolaine", 0.0 },
  { "Kalon", 0.98061574 },
  { "Kalona", 0.0 },
  { "Kalonda", 0.0 },
  { "Kaloni", 0.05483871 },
  { "Kalonie", 0.0 },
  { "Kalonji", 1.0 },
  { "Kalonni", 0.0 },
  { "Kalop", 1.0 },
  { "Kaloyan", 1.0 },
  { "Kalp", 1.0 },
  { "Kalpana", 0.0 },
  { "Kalpesh", 1.0 },
  { "Kalsey", 0.0 },
  { "Kaltrina", 0.0 },
  { "Kalu", 1.0 },
  { "Kalub", 1.0 },
  { "Kalum", 1.0 },
  { "Kalun", 1.0 },
  { "Kalup", 1.0 },
  { "Kalven", 1.0 },
  { "Kalvin", 1.0 },
  { "Kalvyn", 1.0 },
  { "Kaly", 0.0 },
  { "Kalya", 0.0 },
  { "Kalyah", 0.0 },
  { "Kalyan", 1.0 },
  { "Kalyana", 0.0 },
  { "Kalyani", 0.0 },
  { "Kalyann", 0.0 },
  { "Kalyanna", 0.0 },
  { "Kalyb", 1.0 },
  { "Kalyce", 0.0 },
  { "Kalyia", 0.0 },
  { "Kalyiah", 0.0 },
  { "Kalyl", 1.0 },
  { "Kalyn", 0.049145583 },
  { "Kalyna", 0.0 },
  { "Kalynda", 0.0 },
  { "Kalyne", 0.0 },
  { "Kalynn", 0.009587449 },
  { "Kalynne", 0.0 },
  { "Kalypso", 0.0 },
  { "Kalysa", 0.0 },
  { "Kalyse", 0.0 },
  { "Kalysia", 0.0 },
  { "Kalyssa", 0.0 },
  { "Kalysta", 0.0 },
  { "Kalyx", 1.0 },
  { "Kam", 0.5855759 },
  { "Kama", 0.0 },
  { "Kamaal", 1.0 },
  { "Kamaar", 1.0 },
  { "Kamaehu", 1.0 },
  { "Kamahao", 1.0 },
  { "Kamahl", 1.0 },
  { "Kamahri", 1.0 },
  { "Kamai", 1.0 },
  { "Kamaia", 0.0 },
  { "Kamaiah", 0.0 },
  { "Kamaile", 0.0 },
  { "Kamaira", 0.0 },
  { "Kamaire", 1.0 },
  { "Kamaiya", 0.0 },
  { "Kamaiyah", 0.0 },
  { "Kamaj", 1.0 },
  { "Kamakana", 1.0 },
  { "Kamakani", 1.0 },
  { "Kamal", 0.9904398 },
  { "Kamala", 0.0 },
  { "Kamalani", 0.024390243 },
  { "Kamalei", 0.21008404 },
  { "Kamali", 0.55617976 },
  { "Kamaliyah", 0.0 },
  { "Kamalu", 1.0 },
  { "Kaman", 1.0 },
  { "Kamani", 0.44415358 },
  { "Kamar", 0.98149574 },
  { "Kamara", 0.0074626864 },
  { "Kamarah", 0.0 },
  { "Kamarcus", 1.0 },
  { "Kamare", 0.973262 },
  { "Kamarea", 0.0 },
  { "Kamaree", 0.6489104 },
  { "Kamaren", 1.0 },
  { "Kamareon", 1.0 },
  { "Kamari", 0.63957256 },
  { "Kamaria", 0.0 },
  { "Kamariah", 0.0 },
  { "Kamarian", 1.0 },
  { "Kamariana", 0.0 },
  { "Kamarianna", 0.0 },
  { "Kamarie", 0.30598292 },
  { "Kamarien", 1.0 },
  { "Kamarii", 0.46031746 },
  { "Kamarin", 0.0 },
  { "Kamario", 1.0 },
  { "Kamarion", 0.99635035 },
  { "Kamariona", 0.0 },
  { "Kamarionna", 0.0 },
  { "Kamarious", 1.0 },
  { "Kamaris", 1.0 },
  { "Kamarius", 1.0 },
  { "Kamariya", 0.0 },
  { "Kamariyah", 0.0 },
  { "Kamariyon", 1.0 },
  { "Kamaron", 0.95 },
  { "Kamarr", 1.0 },
  { "Kamarra", 0.0 },
  { "Kamarrah", 0.0 },
  { "Kamarre", 1.0 },
  { "Kamarri", 0.708 },
  { "Kamarria", 0.0 },
  { "Kamarrion", 1.0 },
  { "Kamaryn", 0.0 },
  { "Kamau", 1.0 },
  { "Kamaura", 0.0 },
  { "Kamauri", 0.68818516 },
  { "Kamauria", 0.0 },
  { "Kamaurie", 0.6666667 },
  { "Kamaurion", 1.0 },
  { "Kamaury", 0.95614034 },
  { "Kamaya", 0.0 },
  { "Kamayah", 0.0 },
  { "Kamayia", 0.0 },
  { "Kamayla", 0.0 },
  { "Kambel", 0.0 },
  { "Kambell", 0.0 },
  { "Kambelle", 0.0 },
  { "Kamber", 0.0 },
  { "Kamberly", 0.0 },
  { "Kamberlyn", 0.0 },
  { "Kambra", 0.0 },
  { "Kambre", 0.0 },
  { "Kambrea", 0.0 },
  { "Kambree", 0.0 },
  { "Kambreigh", 0.0 },
  { "Kambrey", 0.0 },
  { "Kambri", 0.0 },
  { "Kambria", 0.0 },
  { "Kambrie", 0.0 },
  { "Kambry", 0.0 },
  { "Kambrya", 0.0 },
  { "Kambryn", 0.096 },
  { "Kamdan", 1.0 },
  { "Kamden", 0.93853277 },
  { "Kamdin", 0.9800797 },
  { "Kamdon", 1.0 },
  { "Kamdyn", 0.58391607 },
  { "Kamdynn", 0.0 },
  { "Kamea", 0.0 },
  { "Kameah", 0.0 },
  { "Kameal", 0.0 },
  { "Kamecia", 0.0 },
  { "Kamee", 0.0 },
  { "Kameel", 1.0 },
  { "Kameela", 0.0 },
  { "Kameelah", 0.0 },
  { "Kameera", 0.0 },
  { "Kameesha", 0.0 },
  { "Kameia", 0.0 },
  { "Kameika", 0.0 },
  { "Kameil", 0.0 },
  { "Kameira", 0.0 },
  { "Kameisha", 0.0 },
  { "Kameka", 0.0 },
  { "Kamekia", 0.0 },
  { "Kameko", 0.02347418 },
  { "Kamel", 1.0 },
  { "Kamela", 0.0 },
  { "Kamelah", 0.0 },
  { "Kamelia", 0.0 },
  { "Kamell", 1.0 },
  { "Kamella", 0.0 },
  { "Kamellia", 0.0 },
  { "Kamelo", 1.0 },
  { "Kamelya", 0.0 },
  { "Kamen", 1.0 },
  { "Kameo", 0.041322313 },
  { "Kameon", 1.0 },
  { "Kamera", 0.0 },
  { "Kameran", 0.69508195 },
  { "Kameren", 0.8703125 },
  { "Kameria", 0.0 },
  { "Kameriah", 0.0 },
  { "Kamerin", 0.6562986 },
  { "Kamerion", 1.0 },
  { "Kameron", 0.88329387 },
  { "Kamerra", 0.0 },
  { "Kamery", 0.0 },
  { "Kameryn", 0.26927686 },
  { "Kamesha", 0.0 },
  { "Kameshia", 0.0 },
  { "Kametra", 0.0 },
  { "Kametria", 0.0 },
  { "Kamey", 0.0 },
  { "Kameya", 0.0 },
  { "Kameyah", 0.0 },
  { "Kami", 0.0023590468 },
  { "Kamia", 0.0 },
  { "Kamiah", 0.0 },
  { "Kamiaya", 0.0 },
  { "Kamica", 0.0 },
  { "Kamie", 0.0 },
  { "Kamielle", 0.0 },
  { "Kamien", 1.0 },
  { "Kamiera", 0.0 },
  { "Kamiesha", 0.0 },
  { "Kamijah", 0.0 },
  { "Kamika", 0.0 },
  { "Kamiko", 0.0 },
  { "Kamil", 0.79302216 },
  { "Kamila", 0.0 },
  { "Kamilah", 0.0 },
  { "Kamile", 0.0 },
  { "Kamili", 0.0 },
  { "Kamilia", 0.0 },
  { "Kamill", 0.0 },
  { "Kamilla", 0.0 },
  { "Kamillah", 0.0 },
  { "Kamille", 0.0 },
  { "Kamillia", 0.0 },
  { "Kamilly", 0.0 },
  { "Kamilo", 1.0 },
  { "Kamily", 0.0 },
  { "Kamilya", 0.0 },
  { "Kamilyah", 0.0 },
  { "Kamin", 1.0 },
  { "Kamina", 0.0 },
  { "Kamini", 0.0 },
  { "Kamion", 1.0 },
  { "Kamir", 1.0 },
  { "Kamira", 0.0 },
  { "Kamirah", 0.0 },
  { "Kamiria", 0.0 },
  { "Kamiron", 1.0 },
  { "Kamirra", 0.0 },
  { "Kamiryn", 0.0 },
  { "Kamisha", 0.0 },
  { "Kamishia", 0.0 },
  { "Kamiya", 0.0 },
  { "Kamiyah", 0.0 },
  { "Kamiyha", 0.0 },
  { "Kamiyla", 0.0 },
  { "Kamiylah", 0.0 },
  { "Kamiyra", 0.0 },
  { "Kamla", 0.0 },
  { "Kamlyn", 0.0 },
  { "Kamm", 1.0 },
  { "Kammeron", 1.0 },
  { "Kammi", 0.0 },
  { "Kammie", 0.0 },
  { "Kammron", 1.0 },
  { "Kammy", 0.0 },
  { "Kamon", 1.0 },
  { "Kamoni", 0.36414567 },
  { "Kamonie", 0.0 },
  { "Kamontae", 1.0 },
  { "Kamonte", 1.0 },
  { "Kamor", 0.0 },
  { "Kamora", 0.0 },
  { "Kamorah", 0.0 },
  { "Kamore", 0.0 },
  { "Kamoree", 0.0 },
  { "Kamori", 0.31914893 },
  { "Kamoria", 0.0 },
  { "Kamorian", 1.0 },
  { "Kamorie", 0.1388889 },
  { "Kamorion", 1.0 },
  { "Kamorra", 0.0 },
  { "Kamouri", 0.0 },
  { "Kampbell", 0.0 },
  { "Kamra", 0.0 },
  { "Kamraan", 1.0 },
  { "Kamran", 0.99462366 },
  { "Kamree", 0.0 },
  { "Kamren", 0.88314945 },
  { "Kamrey", 0.0 },
  { "Kamri", 0.0 },
  { "Kamrie", 0.0 },
  { "Kamrin", 0.7057221 },
  { "Kamron", 0.9546521 },
  { "Kamronbek", 1.0 },
  { "Kamrun", 1.0 },
  { "Kamry", 0.020176545 },
  { "Kamryn", 0.14910598 },
  { "Kamryne", 0.0 },
  { "Kamrynn", 0.029366305 },
  { "Kamrynne", 0.0 },
  { "Kamsi", 0.5 },
  { "Kamsiyochi", 0.0 },
  { "Kamsiyochukwu", 0.6754967 },
  { "Kamuela", 1.0 },
  { "Kamuri", 1.0 },
  { "Kamy", 0.0 },
  { "Kamya", 0.0 },
  { "Kamyah", 0.0 },
  { "Kamyar", 1.0 },
  { "Kamyia", 0.0 },
  { "Kamyiah", 0.0 },
  { "Kamyjah", 0.0 },
  { "Kamyl", 0.0 },
  { "Kamyla", 0.0 },
  { "Kamylah", 0.0 },
  { "Kamyle", 0.0 },
  { "Kamylla", 0.0 },
  { "Kamylle", 0.0 },
  { "Kamyra", 0.0 },
  { "Kamyrah", 0.0 },
  { "Kamyree", 0.0 },
  { "Kamyri", 0.0 },
  { "Kamyria", 0.0 },
  { "Kamyrie", 0.0 },
  { "Kamyrn", 0.14583333 },
  { "Kamyron", 0.9019608 },
  { "Kan", 1.0 },
  { "Kana", 0.024291499 },
  { "Kanaan", 1.0 },
  { "Kanae", 0.0 },
  { "Kanah", 0.0 },
  { "Kanai", 0.6992754 },
  { "Kanaiya", 0.0 },
  { "Kanaiyah", 0.0 },
  { "Kanaja", 0.0 },
  { "Kanajah", 0.0 },
  { "Kanako", 0.0 },
  { "Kanaloa", 1.0 },
  { "Kanalu", 1.0 },
  { "Kaname", 0.6875 },
  { "Kanan", 0.96961325 },
  { "Kanani", 0.006518905 },
  { "Kanard", 1.0 },
  { "Kanari", 0.42592594 },
  { "Kanaria", 0.0 },
  { "Kanasha", 0.0 },
  { "Kanasia", 0.0 },
  { "Kanav", 1.0 },
  { "Kanavis", 1.0 },
  { "Kanaya", 0.0 },
  { "Kanchan", 0.0 },
  { "Kanda", 0.0 },
  { "Kandace", 0.0 },
  { "Kandance", 0.0 },
  { "Kandas", 0.0 },
  { "Kande", 0.0 },
  { "Kandee", 0.0 },
  { "Kanden", 1.0 },
  { "Kandence", 0.0 },
  { "Kandi", 0.0 },
  { "Kandia", 0.25 },
  { "Kandice", 0.0 },
  { "Kandida", 0.0 },
  { "Kandie", 0.0 },
  { "Kandis", 0.0 },
  { "Kandise", 0.0 },
  { "Kandiss", 0.0 },
  { "Kandle", 0.0 },
  { "Kandon", 1.0 },
  { "Kandra", 0.0 },
  { "Kandrea", 0.0 },
  { "Kandria", 0.0 },
  { "Kandus", 0.0 },
  { "Kandy", 0.00094625284 },
  { "Kandyce", 0.0 },
  { "Kandyn", 1.0 },
  { "Kandys", 0.0 },
  { "Kandyse", 0.0 },
  { "Kane", 0.99790025 },
  { "Kanea", 0.0 },
  { "Kanecia", 0.0 },
  { "Kanedra", 0.0 },
  { "Kaneesha", 0.0 },
  { "Kaneeshia", 0.0 },
  { "Kaneidra", 0.0 },
  { "Kaneil", 1.0 },
  { "Kaneisha", 0.0 },
  { "Kaneka", 0.0 },
  { "Kanella", 0.0 },
  { "Kanen", 1.0 },
  { "Kanesha", 0.0 },
  { "Kaneshia", 0.0 },
  { "Kanesia", 0.0 },
  { "Kanessa", 0.0 },
  { "Kanessha", 0.0 },
  { "Kanetha", 0.0 },
  { "Kanethia", 0.0 },
  { "Kanetra", 0.0 },
  { "Kanette", 0.0 },
  { "Kang", 0.28947368 },
  { "Kani", 0.74222225 },
  { "Kania", 0.0 },
  { "Kaniah", 0.0 },
  { "Kaniala", 1.0 },
  { "Kanica", 0.0 },
  { "Kanichi", 1.0 },
  { "Kaniece", 0.0 },
  { "Kaniel", 1.0 },
  { "Kaniela", 1.0 },
  { "Kaniesha", 0.0 },
  { "Kanija", 0.0 },
  { "Kanijah", 0.0 },
  { "Kanijha", 0.0 },
  { "Kanika", 0.0 },
  { "Kanila", 0.0 },
  { "Kanilah", 0.0 },
  { "Kanin", 1.0 },
  { "Kaniqua", 0.0 },
  { "Kanira", 0.0 },
  { "Kanisa", 0.0 },
  { "Kanise", 0.0 },
  { "Kanish", 1.0 },
  { "Kanisha", 0.0 },
  { "Kanishia", 0.0 },
  { "Kanishk", 1.0 },
  { "Kanishka", 0.0 },
  { "Kanissa", 0.0 },
  { "Kanita", 0.0 },
  { "Kanitha", 0.0 },
  { "Kanitra", 0.0 },
  { "Kaniya", 0.0 },
  { "Kaniyah", 0.0 },
  { "Kaniyha", 0.0 },
  { "Kaniylah", 0.0 },
  { "Kanji", 1.0 },
  { "Kanna", 0.0 },
  { "Kannan", 1.0 },
  { "Kannen", 1.0 },
  { "Kannin", 1.0 },
  { "Kannon", 0.97831136 },
  { "Kannyn", 0.3529412 },
  { "Kano", 1.0 },
  { "Kanoa", 1.0 },
  { "Kanoe", 0.0 },
  { "Kanoelani", 0.0 },
  { "Kanoi", 0.0 },
  { "Kanon", 0.8698384 },
  { "Kanora", 0.0 },
  { "Kanosha", 0.0 },
  { "Kansas", 0.053201083 },
  { "Kanta", 1.0 },
  { "Kantina", 0.0 },
  { "Kanton", 1.0 },
  { "Kantrell", 1.0 },
  { "Kantrice", 0.0 },
  { "Kanwal", 0.0 },
  { "Kany", 0.0 },
  { "Kanya", 0.043706294 },
  { "Kanyae", 1.0 },
  { "Kanyah", 0.0 },
  { "Kanyan", 1.0 },
  { "Kanyatta", 0.0 },
  { "Kanye", 0.9819277 },
  { "Kanyen", 0.84375 },
  { "Kanyia", 0.0 },
  { "Kanyiah", 0.0 },
  { "Kanyjah", 0.0 },
  { "Kanyla", 0.0 },
  { "Kanylah", 0.0 },
  { "Kanyon", 0.92527676 },
  { "Kanyra", 0.0 },
  { "Kanysha", 0.0 },
  { "Kao", 0.6379822 },
  { "Kaoir", 0.0 },
  { "Kaola", 0.0 },
  { "Kaolin", 0.0 },
  { "Kaomi", 0.0 },
  { "Kaon", 1.0 },
  { "Kaonou", 0.0 },
  { "Kaori", 0.061173532 },
  { "Kaoru", 0.6693227 },
  { "Kaory", 0.0 },
  { "Kaos", 1.0 },
  { "Kap", 1.0 },
  { "Kapena", 1.0 },
  { "Kapil", 1.0 },
  { "Kapiolani", 0.0 },
  { "Kapish", 1.0 },
  { "Kaplan", 1.0 },
  { "Kapone", 1.0 },
  { "Kapono", 1.0 },
  { "Kapree", 0.0 },
  { "Kapresha", 0.0 },
  { "Kapri", 0.013888889 },
  { "Kapria", 0.0 },
  { "Kaprice", 0.0 },
  { "Kapricia", 0.0 },
  { "Kaprisha", 0.0 },
  { "Kaptain", 1.0 },
  { "Kar", 0.0 },
  { "Kara", 0.0028676314 },
  { "Karaann", 0.0 },
  { "Karabella", 0.0 },
  { "Karac", 1.0 },
  { "Karadyn", 0.0 },
  { "Karagan", 0.0 },
  { "Karagen", 0.0 },
  { "Karah", 0.0 },
  { "Karai", 0.0 },
  { "Karalea", 0.0 },
  { "Karalee", 0.0 },
  { "Karaleigh", 0.0 },
  { "Karalena", 0.0 },
  { "Karalin", 0.0 },
  { "Karalina", 0.0 },
  { "Karaline", 0.0 },
  { "Karalyn", 0.0 },
  { "Karalyne", 0.0 },
  { "Karalynn", 0.0 },
  { "Karalynne", 0.0 },
  { "Karam", 1.0 },
  { "Karamia", 0.0 },
  { "Karamo", 1.0 },
  { "Karamvir", 1.0 },
  { "Karan", 0.2857143 },
  { "Karana", 0.0 },
  { "Karanbir", 1.0 },
  { "Karanda", 0.0 },
  { "Karandeep", 1.0 },
  { "Karanja", 1.0 },
  { "Karanveer", 1.0 },
  { "Karanvir", 1.0 },
  { "Karapet", 1.0 },
  { "Karar", 1.0 },
  { "Karas", 1.0 },
  { "Karaya", 0.0 },
  { "Karbon", 1.0 },
  { "Karcen", 1.0 },
  { "Karch", 1.0 },
  { "Karcher", 1.0 },
  { "Karcyn", 0.14150943 },
  { "Karcynn", 0.0 },
  { "Kardae", 1.0 },
  { "Kardale", 1.0 },
  { "Kardarius", 1.0 },
  { "Karde", 1.0 },
  { "Kardell", 1.0 },
  { "Karden", 1.0 },
  { "Karder", 0.9688889 },
  { "Kardier", 1.0 },
  { "Kardin", 1.0 },
  { "Kardyn", 0.0 },
  { "Kare", 0.0 },
  { "Kaream", 1.0 },
  { "Kareana", 0.0 },
  { "Karee", 0.097323604 },
  { "Kareem", 0.9866379 },
  { "Kareema", 0.0 },
  { "Kareemah", 0.0 },
  { "Kareen", 0.3634568 },
  { "Kareena", 0.0 },
  { "Kareese", 0.0 },
  { "Kareesha", 0.0 },
  { "Karega", 1.0 },
  { "Karel", 0.29971015 },
  { "Kareli", 0.0 },
  { "Karelia", 0.0 },
  { "Karelie", 0.0 },
  { "Karelin", 0.0 },
  { "Karelis", 0.0 },
  { "Karell", 1.0 },
  { "Karelly", 0.0 },
  { "Karely", 0.0 },
  { "Karelyn", 0.0 },
  { "Karelys", 0.0 },
  { "Karem", 0.35457063 },
  { "Karema", 0.0 },
  { "Kareme", 1.0 },
  { "Karen", 0.002798974 },
  { "Karena", 0.0 },
  { "Karenann", 0.0 },
  { "Karenda", 0.0 },
  { "Karene", 0.0 },
  { "Karenina", 0.0 },
  { "Karenlee", 0.0 },
  { "Karenn", 0.0 },
  { "Karenna", 0.0 },
  { "Karensa", 0.0 },
  { "Karent", 0.0 },
  { "Kareny", 0.0 },
  { "Karenza", 0.0 },
  { "Karesa", 0.0 },
  { "Karese", 0.0 },
  { "Karesha", 0.0 },
  { "Karess", 0.0 },
  { "Karessa", 0.0 },
  { "Karesse", 0.0 },
  { "Karetha", 0.0 },
  { "Karey", 0.21075425 },
  { "Karhonda", 0.0 },
  { "Kari", 0.010813734 },
  { "Karia", 0.0 },
  { "Kariah", 0.0 },
  { "Karian", 0.0 },
  { "Kariana", 0.0 },
  { "Kariann", 0.0 },
  { "Karianna", 0.0 },
  { "Karianne", 0.0 },
  { "Karibe", 1.0 },
  { "Karicia", 0.0 },
  { "Karidee", 0.0 },
  { "Karie", 0.0 },
  { "Kariel", 0.24285714 },
  { "Karieliz", 0.0 },
  { "Karielle", 0.0 },
  { "Karielys", 0.0 },
  { "Kariem", 1.0 },
  { "Karif", 1.0 },
  { "Karigan", 0.0 },
  { "Karil", 0.0 },
  { "Karilee", 0.0 },
  { "Karilyn", 0.0 },
  { "Karilynn", 0.0 },
  { "Karim", 0.98780745 },
  { "Karima", 0.0 },
  { "Karimah", 0.0 },
  { "Karime", 0.0 },
  { "Karimen", 0.0 },
  { "Karin", 0.004910902 },
  { "Karina", 0.004388704 },
  { "Karinah", 0.0 },
  { "Karinda", 0.0 },
  { "Karine", 0.0 },
  { "Karington", 0.0 },
  { "Karinn", 0.0 },
  { "Karinna", 0.0 },
  { "Karinne", 0.0 },
  { "Karion", 0.93421054 },
  { "Kariona", 0.0 },
  { "Karionna", 0.0 },
  { "Karis", 0.013264945 },
  { "Karisa", 0.0 },
  { "Karise", 0.0 },
  { "Karisha", 0.0 },
  { "Karishma", 0.0 },
  { "Karisia", 0.0 },
  { "Karisma", 0.0 },
  { "Karismah", 0.0 },
  { "Kariss", 0.0 },
  { "Karissa", 0.0 },
  { "Karissia", 0.0 },
  { "Karissma", 0.0 },
  { "Karista", 0.0 },
  { "Karita", 0.0 },
  { "Karitza", 0.0 },
  { "Kariya", 0.0 },
  { "Kariyah", 0.0 },
  { "Kariyana", 0.0 },
  { "Karizma", 0.0 },
  { "Karizmah", 0.0 },
  { "Karl", 0.9909312 },
  { "Karla", 0.0035103664 },
  { "Karlah", 0.0 },
  { "Karlan", 1.0 },
  { "Karle", 0.37333333 },
  { "Karlea", 0.0 },
  { "Karleah", 0.0 },
  { "Karlee", 0.0 },
  { "Karleen", 0.0 },
  { "Karleena", 0.0 },
  { "Karlei", 0.0 },
  { "Karleigh", 0.0 },
  { "Karlen", 0.09815951 },
  { "Karlena", 0.0 },
  { "Karlene", 0.0 },
  { "Karlesha", 0.0 },
  { "Karletta", 0.0 },
  { "Karlette", 0.0 },
  { "Karley", 0.0 },
  { "Karli", 0.0 },
  { "Karlia", 0.0 },
  { "Karliah", 0.0 },
  { "Karlianys", 0.0 },
  { "Karlie", 0.0 },
  { "Karliee", 0.0 },
  { "Karlin", 0.43606138 },
  { "Karlina", 0.0 },
  { "Karline", 0.0 },
  { "Karlis", 1.0 },
  { "Karlisa", 0.0 },
  { "Karlise", 0.0 },
  { "Karlisha", 0.0 },
  { "Karlissa", 0.0 },
  { "Karlita", 0.0 },
  { "Karlito", 1.0 },
  { "Karliyah", 0.0 },
  { "Karlo", 1.0 },
  { "Karlon", 1.0 },
  { "Karlos", 1.0 },
  { "Karlotta", 0.0 },
  { "Karloz", 1.0 },
  { "Karlson", 1.0 },
  { "Karlton", 1.0 },
  { "Karly", 0.0004973145 },
  { "Karlye", 0.0 },
  { "Karlyee", 0.0 },
  { "Karlyle", 1.0 },
  { "Karlyn", 0.0031124945 },
  { "Karlyne", 0.0 },
  { "Karlynn", 0.0 },
  { "Karlysia", 0.0 },
  { "Karm", 1.0 },
  { "Karma", 0.02020036 },
  { "Karmah", 0.0 },
  { "Karman", 0.06759099 },
  { "Karmani", 0.33333334 },
  { "Karmann", 0.0 },
  { "Karmari", 1.0 },
  { "Karmel", 0.14184397 },
  { "Karmela", 0.0 },
  { "Karmell", 1.0 },
  { "Karmella", 0.0 },
  { "Karmello", 1.0 },
  { "Karmelo", 1.0 },
  { "Karmen", 0.0016616816 },
  { "Karmesha", 0.0 },
  { "Karmia", 0.0 },
  { "Karmin", 0.0 },
  { "Karmina", 0.0 },
  { "Karmindy", 0.0 },
  { "Karmine", 1.0 },
  { "Karmisha", 0.0 },
  { "Karmon", 0.02631579 },
  { "Karmoni", 0.0 },
  { "Karmya", 0.0 },
  { "Karmyn", 0.0 },
  { "Karmyne", 1.0 },
  { "Karmynn", 0.0 },
  { "Karn", 0.0295858 },
  { "Karna", 0.011342155 },
  { "Karne", 0.0 },
  { "Karnecia", 0.0 },
  { "Karneisha", 0.0 },
  { "Karnel", 1.0 },
  { "Karnell", 1.0 },
  { "Karnesha", 0.0 },
  { "Karneshia", 0.0 },
  { "Karnisha", 0.0 },
  { "Karo", 1.0 },
  { "Karol", 0.13034873 },
  { "Karola", 0.0 },
  { "Karolann", 0.0 },
  { "Karole", 0.0 },
  { "Karolee", 0.0 },
  { "Karoleena", 0.0 },
  { "Karoleigh", 0.0 },
  { "Karolena", 0.0 },
  { "Karolin", 0.0 },
  { "Karolina", 0.0 },
  { "Karoline", 0.0 },
  { "Karolis", 1.0 },
  { "Karoll", 0.0 },
  { "Karolyn", 0.0 },
  { "Karolyna", 0.0 },
  { "Karolyne", 0.0 },
  { "Karolynn", 0.0 },
  { "Karon", 0.31365395 },
  { "Karona", 0.0 },
  { "Karonda", 0.0 },
  { "Karra", 0.0 },
  { "Karrah", 0.0 },
  { "Karran", 0.0 },
  { "Karrar", 1.0 },
  { "Karras", 1.0 },
  { "Karre", 0.0 },
  { "Karreem", 1.0 },
  { "Karrell", 1.0 },
  { "Karrem", 1.0 },
  { "Karren", 0.0 },
  { "Karrena", 0.0 },
  { "Karri", 0.0 },
  { "Karriann", 0.0 },
  { "Karrianne", 0.0 },
  { "Karrick", 1.0 },
  { "Karrie", 0.0 },
  { "Karriem", 0.989899 },
  { "Karrien", 1.0 },
  { "Karrigan", 0.0 },
  { "Karrin", 0.0 },
  { "Karrina", 0.0 },
  { "Karrine", 0.0 },
  { "Karrington", 0.17730978 },
  { "Karris", 0.0 },
  { "Karrisa", 0.0 },
  { "Karrissa", 0.0 },
  { "Karrol", 0.118644066 },
  { "Karroll", 0.45454547 },
  { "Karron", 0.3037037 },
  { "Karrson", 1.0 },
  { "Karry", 0.3485342 },
  { "Karryn", 0.0 },
  { "Karsan", 1.0 },
  { "Karsen", 0.6134221 },
  { "Karsha", 0.0 },
  { "Karsin", 0.83157897 },
  { "Karslyn", 0.0 },
  { "Karson", 0.87743944 },
  { "Karsten", 0.94825315 },
  { "Karstin", 0.6666667 },
  { "Karston", 1.0 },
  { "Karstyn", 0.0 },
  { "Karsyn", 0.15877712 },
  { "Karsynn", 0.0 },
  { "Kartar", 1.0 },
  { "Kartel", 1.0 },
  { "Kartell", 1.0 },
  { "Karter", 0.87108535 },
  { "Kartez", 1.0 },
  { "Karthik", 1.0 },
  { "Karthika", 0.0 },
  { "Karthikeya", 1.0 },
  { "Karthikeyan", 1.0 },
  { "Kartier", 0.8701299 },
  { "Kartik", 1.0 },
  { "Kartika", 0.0 },
  { "Kartikeya", 1.0 },
  { "Kartina", 0.0 },
  { "Kartrina", 0.0 },
  { "Kartyr", 1.0 },
  { "Karun", 1.0 },
  { "Karuna", 0.0 },
  { "Karver", 1.0 },
  { "Karvin", 1.0 },
  { "Kary", 0.4239521 },
  { "Karyah", 0.0 },
  { "Karyana", 0.0 },
  { "Karyl", 0.0 },
  { "Karyle", 0.0 },
  { "Karylle", 0.0 },
  { "Karym", 1.0 },
  { "Karyme", 0.0 },
  { "Karyn", 0.00035529028 },
  { "Karyna", 0.0 },
  { "Karyne", 0.0 },
  { "Karynn", 0.0 },
  { "Karynna", 0.0 },
  { "Karynne", 0.0 },
  { "Karys", 0.0 },
  { "Karysa", 0.0 },
  { "Karysma", 0.0 },
  { "Karyss", 0.0 },
  { "Karyssa", 0.0 },
  { "Karyzma", 0.0 },
  { "Kas", 1.0 },
  { "Kasa", 0.0 },
  { "Kasaan", 1.0 },
  { "Kasadee", 0.0 },
  { "Kasadie", 0.0 },
  { "Kasady", 0.0 },
  { "Kasahn", 1.0 },
  { "Kasai", 1.0 },
  { "Kasan", 1.0 },
  { "Kasandra", 0.0 },
  { "Kasani", 0.25 },
  { "Kasara", 0.0 },
  { "Kasarah", 0.0 },
  { "Kasaun", 1.0 },
  { "Kasaundra", 0.0 },
  { "Kasch", 1.0 },
  { "Kascha", 0.0 },
  { "Kase", 1.0 },
  { "Kasea", 0.0 },
  { "Kasean", 1.0 },
  { "Kasee", 0.0 },
  { "Kaseem", 1.0 },
  { "Kaseen", 1.0 },
  { "Kaselyn", 0.0 },
  { "Kaselynn", 0.0 },
  { "Kasem", 1.0 },
  { "Kasen", 0.9822695 },
  { "Kaser", 1.0 },
  { "Kaseton", 1.0 },
  { "Kasey", 0.26413658 },
  { "Kash", 0.9880535 },
  { "Kasha", 0.0 },
  { "Kashad", 1.0 },
  { "Kashae", 0.0 },
  { "Kashaf", 0.0 },
  { "Kashala", 0.0 },
  { "Kashan", 1.0 },
  { "Kashana", 0.0 },
  { "Kashanda", 0.0 },
  { "Kashandra", 0.0 },
  { "Kashanna", 0.0 },
  { "Kashanta", 0.0 },
  { "Kashanti", 0.0 },
  { "Kashara", 0.0 },
  { "Kashari", 0.0 },
  { "Kashaun", 1.0 },
  { "Kashauna", 0.0 },
  { "Kashaunda", 0.0 },
  { "Kashaundra", 0.0 },
  { "Kashawn", 0.9269663 },
  { "Kashawna", 0.0 },
  { "Kashay", 0.0 },
  { "Kashayla", 0.0 },
  { "Kashden", 1.0 },
  { "Kashdyn", 1.0 },
  { "Kashe", 0.4871795 },
  { "Kashea", 0.0 },
  { "Kasheem", 1.0 },
  { "Kasheen", 1.0 },
  { "Kasheena", 0.0 },
  { "Kashema", 0.0 },
  { "Kashen", 1.0 },
  { "Kashena", 0.0 },
  { "Kasher", 1.0 },
  { "Kashes", 1.0 },
  { "Kashi", 0.33333334 },
  { "Kashia", 0.0 },
  { "Kashief", 1.0 },
  { "Kashif", 1.0 },
  { "Kashiff", 1.0 },
  { "Kashika", 0.0 },
  { "Kashima", 0.0 },
  { "Kashina", 0.0 },
  { "Kashira", 0.0 },
  { "Kashis", 1.0 },
  { "Kashish", 0.0 },
  { "Kashius", 1.0 },
  { "Kashiya", 0.0 },
  { "Kashka", 0.42857143 },
  { "Kashlee", 0.0 },
  { "Kashley", 0.0 },
  { "Kashlyn", 0.0 },
  { "Kashlynn", 0.0 },
  { "Kashmeir", 1.0 },
  { "Kashmere", 0.34516767 },
  { "Kashmiere", 0.0 },
  { "Kashmir", 0.3452188 },
  { "Kashmira", 0.0 },
  { "Kashon", 1.0 },
  { "Kashona", 0.0 },
  { "Kashonda", 0.0 },
  { "Kashondra", 0.0 },
  { "Kashonna", 0.0 },
  { "Kashten", 1.0 },
  { "Kashtian", 1.0 },
  { "Kashtin", 1.0 },
  { "Kashton", 0.9957483 },
  { "Kashtyn", 0.9019608 },
  { "Kashun", 1.0 },
  { "Kashunda", 0.0 },
  { "Kashus", 1.0 },
  { "Kashvi", 0.0 },
  { "Kashya", 0.0 },
  { "Kashyap", 1.0 },
  { "Kashyia", 0.0 },
  { "Kashyra", 0.0 },
  { "Kasi", 0.0115350485 },
  { "Kasia", 0.0 },
  { "Kasiah", 0.057416268 },
  { "Kasib", 1.0 },
  { "Kasidee", 0.0 },
  { "Kasidi", 0.0 },
  { "Kasidy", 0.0 },
  { "Kasie", 0.0050983247 },
  { "Kasiem", 1.0 },
  { "Kasim", 1.0 },
  { "Kasima", 0.0 },
  { "Kasimer", 1.0 },
  { "Kasimir", 1.0 },
  { "Kasimu", 1.0 },
  { "Kasin", 1.0 },
  { "Kasinda", 0.0 },
  { "Kasir", 1.0 },
  { "Kasity", 0.0 },
  { "Kasiya", 0.0 },
  { "Kasiyah", 0.0 },
  { "Kaslyn", 0.0 },
  { "Kaslynn", 0.0 },
  { "Kasmer", 1.0 },
  { "Kasmin", 0.0 },
  { "Kasmine", 0.0 },
  { "Kasmir", 1.0 },
  { "Kasmira", 0.0 },
  { "Kason", 0.9952436 },
  { "Kasondra", 0.0 },
  { "Kasonya", 0.0 },
  { "Kaspar", 1.0 },
  { "Kaspen", 1.0 },
  { "Kasper", 1.0 },
  { "Kaspian", 1.0 },
  { "Kasra", 1.0 },
  { "Kass", 1.0 },
  { "Kassadee", 0.0 },
  { "Kassadi", 0.0 },
  { "Kassadie", 0.0 },
  { "Kassady", 0.0 },
  { "Kassan", 1.0 },
  { "Kassandr", 0.0 },
  { "Kassandra", 0.00050740497 },
  { "Kassandre", 0.0 },
  { "Kassandrea", 0.0 },
  { "Kassara", 0.0 },
  { "Kassaundra", 0.0 },
  { "Kassedy", 0.0 },
  { "Kassee", 0.0 },
  { "Kassel", 1.0 },
  { "Kassem", 1.0 },
  { "Kassen", 1.0 },
  { "Kassey", 0.00761035 },
  { "Kassi", 0.0 },
  { "Kassia", 0.0 },
  { "Kassiah", 0.0 },
  { "Kassiana", 0.0 },
  { "Kassiani", 0.0 },
  { "Kasside", 0.0 },
  { "Kassidee", 0.0 },
  { "Kassidey", 0.0 },
  { "Kassidi", 0.0 },
  { "Kassidie", 0.0 },
  { "Kassidy", 0.017708449 },
  { "Kassie", 0.0 },
  { "Kassim", 1.0 },
  { "Kassity", 0.0 },
  { "Kassius", 1.0 },
  { "Kasson", 1.0 },
  { "Kassondra", 0.0 },
  { "Kassy", 0.0 },
  { "Kassydi", 0.0 },
  { "Kasten", 1.0 },
  { "Kastiel", 1.0 },
  { "Kastin", 1.0 },
  { "Kastle", 0.89361703 },
  { "Kaston", 1.0 },
  { "Kastriot", 1.0 },
  { "Kastyn", 0.7619048 },
  { "Kasumi", 0.0 },
  { "Kasundra", 0.0 },
  { "Kasy", 0.0 },
  { "Kasydi", 0.0 },
  { "Kasye", 0.0 },
  { "Kasyn", 0.80632913 },
  { "Kat", 0.0 },
  { "Kata", 0.0 },
  { "Katai", 1.0 },
  { "Katalaya", 0.0 },
  { "Katalayah", 0.0 },
  { "Katalea", 0.0 },
  { "Kataleah", 0.0 },
  { "Kataleena", 0.0 },
  { "Kataleia", 0.0 },
  { "Kataleiya", 0.0 },
  { "Katalena", 0.0 },
  { "Kataleya", 0.0 },
  { "Kataleyah", 0.0 },
  { "Katalia", 0.0 },
  { "Katalin", 0.0 },
  { "Katalina", 0.0 },
  { "Kataliya", 0.0 },
  { "Kataliyah", 0.0 },
  { "Katalya", 0.0 },
  { "Katalyn", 0.0 },
  { "Katalyna", 0.0 },
  { "Katalynna", 0.0 },
  { "Katana", 0.0 },
  { "Katanna", 0.0 },
  { "Katanya", 0.0 },
  { "Katara", 0.0 },
  { "Katarena", 0.0 },
  { "Katari", 0.45454547 },
  { "Kataria", 0.0 },
  { "Katarina", 0.0 },
  { "Katarra", 0.0 },
  { "Kataryna", 0.0 },
  { "Katarzyna", 0.0 },
  { "Katasha", 0.0 },
  { "Katashia", 0.0 },
  { "Katasia", 0.0 },
  { "Katavia", 0.0 },
  { "Katavion", 1.0 },
  { "Kataya", 0.0 },
  { "Kate", 0.0011050005 },
  { "Kateara", 0.0 },
  { "Katedra", 0.0 },
  { "Katee", 0.0 },
  { "Kateena", 0.0 },
  { "Kateisha", 0.0 },
  { "Katelan", 0.0 },
  { "Kateland", 0.0 },
  { "Katelee", 0.0 },
  { "Kateleen", 0.0 },
  { "Kateleigh", 0.0 },
  { "Katelen", 0.0 },
  { "Kateleya", 0.0 },
  { "Katelin", 0.0 },
  { "Katelina", 0.0 },
  { "Katelind", 0.0 },
  { "Kateline", 0.0 },
  { "Katelinn", 0.0 },
  { "Katelon", 0.0 },
  { "Katelund", 0.0 },
  { "Kately", 0.0 },
  { "Katelyn", 0.0012360845 },
  { "Katelynd", 0.0 },
  { "Katelyne", 0.0 },
  { "Katelynn", 0.0003720112 },
  { "Katelynne", 0.0 },
  { "Katen", 0.0 },
  { "Katena", 0.0 },
  { "Katence", 0.0 },
  { "Katenia", 0.0 },
  { "Kater", 1.0 },
  { "Katera", 0.0 },
  { "Katerena", 0.0 },
  { "Kateri", 0.0 },
  { "Kateria", 0.0 },
  { "Katerin", 0.0 },
  { "Katerina", 0.0 },
  { "Katerine", 0.0 },
  { "Katerinne", 0.0 },
  { "Katerra", 0.0 },
  { "Katerria", 0.0 },
  { "Kateryn", 0.0 },
  { "Kateryna", 0.0 },
  { "Katesha", 0.0 },
  { "Kateshia", 0.0 },
  { "Katessa", 0.0 },
  { "Katey", 0.0 },
  { "Kath", 0.0 },
  { "Katha", 0.0 },
  { "Kathaleen", 0.0 },
  { "Kathalene", 0.0 },
  { "Kathaleya", 0.0 },
  { "Kathalia", 0.0 },
  { "Kathalina", 0.0 },
  { "Kathaline", 0.0 },
  { "Kathalyn", 0.0 },
  { "Kathan", 1.0 },
  { "Katharin", 0.0 },
  { "Katharina", 0.0 },
  { "Katharine", 0.00026938337 },
  { "Katharyn", 0.0 },
  { "Katharyne", 0.0 },
  { "Kathay", 0.0 },
  { "Kathe", 0.0 },
  { "Kathee", 0.0 },
  { "Katheen", 0.0 },
  { "Katheine", 0.0 },
  { "Kathelean", 0.0 },
  { "Katheleen", 0.0 },
  { "Kathelene", 0.0 },
  { "Katheline", 0.0 },
  { "Kathelyn", 0.0 },
  { "Kathen", 1.0 },
  { "Kather", 0.0 },
  { "Katheran", 0.0 },
  { "Katheren", 0.0 },
  { "Katherene", 0.0 },
  { "Katheriene", 0.0 },
  { "Katherin", 0.0 },
  { "Katherina", 0.0 },
  { "Katherine", 0.0028754624 },
  { "Katherinne", 0.0 },
  { "Katherleen", 0.0 },
  { "Katherlene", 0.0 },
  { "Katherline", 0.0 },
  { "Kathern", 0.0 },
  { "Katherne", 0.0 },
  { "Katheryn", 0.0 },
  { "Katheryne", 0.0 },
  { "Katherynn", 0.0 },
  { "Kathey", 0.0 },
  { "Kathi", 0.0 },
  { "Kathia", 0.0 },
  { "Kathiana", 0.0 },
  { "Kathie", 0.0 },
  { "Kathileen", 0.0 },
  { "Kathina", 0.0 },
  { "Kathir", 1.0 },
  { "Kathiria", 0.0 },
  { "Kathlean", 0.0 },
  { "Kathlee", 0.0 },
  { "Kathleeen", 0.0 },
  { "Kathleen", 0.0023745273 },
  { "Kathleena", 0.0 },
  { "Kathleenann", 0.0 },
  { "Kathleene", 0.0 },
  { "Kathleenmary", 0.0 },
  { "Kathlen", 0.0 },
  { "Kathlena", 0.0 },
  { "Kathlene", 0.0 },
  { "Kathlin", 0.0 },
  { "Kathlina", 0.0 },
  { "Kathline", 0.0 },
  { "Kathllen", 0.0 },
  { "Kathlyn", 0.0 },
  { "Kathlyne", 0.0 },
  { "Kathlynn", 0.0 },
  { "Katholeen", 0.0 },
  { "Kathran", 0.0 },
  { "Kathrean", 0.0 },
  { "Kathreen", 0.0 },
  { "Kathren", 0.0 },
  { "Kathrene", 0.0 },
  { "Kathrin", 0.0 },
  { "Kathrina", 0.0 },
  { "Kathrine", 0.0 },
  { "Kathry", 0.0 },
  { "Kathryn", 0.0024227232 },
  { "Kathrynann", 0.0 },
  { "Kathrynanne", 0.0 },
  { "Kathryne", 0.0 },
  { "Kathrynn", 0.0 },
  { "Kathrynne", 0.0 },
  { "Kathy", 0.0022114692 },
  { "Kathya", 0.0 },
  { "Kathyann", 0.0 },
  { "Kathyanne", 0.0 },
  { "Kathye", 0.0 },
  { "Kathyern", 0.0 },
  { "Kathyjo", 0.0 },
  { "Kathyleen", 0.0 },
  { "Kathylene", 0.0 },
  { "Kathyn", 0.0 },
  { "Kathyren", 0.0 },
  { "Kathyria", 0.0 },
  { "Kathyrine", 0.0 },
  { "Kathyrn", 0.0 },
  { "Kathyrne", 0.0 },
  { "Kathyy", 0.0 },
  { "Kati", 0.0 },
  { "Katia", 0.0 },
  { "Katiana", 0.0 },
  { "Katiann", 0.0 },
  { "Katianna", 0.0 },
  { "Katianne", 0.0 },
  { "Katie", 0.0021158233 },
  { "Katieann", 0.0 },
  { "Katieanne", 0.0 },
  { "Katiejo", 0.0 },
  { "Katielee", 0.0 },
  { "Katielyn", 0.0 },
  { "Katielynn", 0.0 },
  { "Katiemae", 0.0 },
  { "Katiemarie", 0.0 },
  { "Katiera", 0.0 },
  { "Katierra", 0.0 },
  { "Katija", 0.0 },
  { "Katika", 0.0 },
  { "Katilaya", 0.0 },
  { "Katilin", 0.0 },
  { "Katilyn", 0.0 },
  { "Katilynn", 0.0 },
  { "Katima", 0.0 },
  { "Katin", 0.7692308 },
  { "Katina", 0.003017662 },
  { "Katine", 0.0 },
  { "Katinia", 0.0 },
  { "Katinka", 0.0 },
  { "Katinna", 0.0 },
  { "Katira", 0.0 },
  { "Katiria", 0.0 },
  { "Katisha", 0.0 },
  { "Katishia", 0.0 },
  { "Katiuscia", 0.0 },
  { "Katiuska", 0.0 },
  { "Katiya", 0.0 },
  { "Katiyah", 0.0 },
  { "Katja", 0.0 },
  { "Katlain", 0.0 },
  { "Katlan", 0.0 },
  { "Katleen", 0.0 },
  { "Katlen", 0.0 },
  { "Katleyn", 0.0 },
  { "Katlin", 0.023076924 },
  { "Katline", 0.0 },
  { "Katlyn", 0.0002736128 },
  { "Katlynd", 0.0 },
  { "Katlyne", 0.0 },
  { "Katlynn", 0.0 },
  { "Katlynne", 0.0 },
  { "Katniss", 0.0 },
  { "Kato", 1.0 },
  { "Katon", 1.0 },
  { "Katona", 0.0 },
  { "Katonia", 0.0 },
  { "Katonna", 0.0 },
  { "Katonya", 0.0 },
  { "Katora", 0.0 },
  { "Katori", 0.0 },
  { "Katoria", 0.0 },
  { "Katosha", 0.0 },
  { "Katoya", 0.0 },
  { "Katravia", 0.0 },
  { "Katrea", 0.0 },
  { "Katrece", 0.0 },
  { "Katreen", 0.0 },
  { "Katreena", 0.0 },
  { "Katreina", 0.0 },
  { "Katrell", 0.9067245 },
  { "Katrena", 0.0 },
  { "Katrenia", 0.0 },
  { "Katrenna", 0.0 },
  { "Katresa", 0.0 },
  { "Katrese", 0.0 },
  { "Katri", 0.0 },
  { "Katria", 0.0 },
  { "Katriana", 0.0 },
  { "Katrianna", 0.0 },
  { "Katrica", 0.0 },
  { "Katrice", 0.0 },
  { "Katricia", 0.0 },
  { "Katriece", 0.0 },
  { "Katriel", 0.08469945 },
  { "Katrielle", 0.0 },
  { "Katrien", 0.0 },
  { "Katriena", 0.0 },
  { "Katrin", 0.0 },
  { "Katrina", 0.0023680178 },
  { "Katrinamarie", 0.0 },
  { "Katrine", 0.0 },
  { "Katrinia", 0.0 },
  { "Katrinka", 0.0 },
  { "Katrinna", 0.0 },
  { "Katriona", 0.0 },
  { "Katrisa", 0.0 },
  { "Katrisha", 0.0 },
  { "Katron", 1.0 },
  { "Katryn", 0.0 },
  { "Katryna", 0.0 },
  { "Katsue", 0.0 },
  { "Katsuji", 1.0 },
  { "Katsuko", 0.0 },
  { "Katsumi", 0.9814815 },
  { "Kattaleya", 0.0 },
  { "Katti", 0.0 },
  { "Kattia", 0.0 },
  { "Kattie", 0.0 },
  { "Kattina", 0.0 },
  { "Kattleya", 0.0 },
  { "Kattrina", 0.0 },
  { "Katty", 0.0 },
  { "Kattya", 0.0 },
  { "Katura", 0.0 },
  { "Katurah", 0.0 },
  { "Katy", 0.0002482745 },
  { "Katya", 0.0 },
  { "Katyana", 0.0 },
  { "Katyann", 0.0 },
  { "Katyanna", 0.0 },
  { "Katybeth", 0.0 },
  { "Katye", 0.0 },
  { "Katylin", 0.0 },
  { "Katyln", 0.0 },
  { "Katylyn", 0.0 },
  { "Katylynn", 0.0 },
  { "Katyna", 0.0 },
  { "Katyra", 0.0 },
  { "Katyria", 0.0 },
  { "Kaua", 1.0 },
  { "Kauai", 1.0 },
  { "Kauan", 1.0 },
  { "Kaui", 0.0 },
  { "Kaula", 0.0 },
  { "Kaulana", 0.95652175 },
  { "Kaulder", 1.0 },
  { "Kaulin", 1.0 },
  { "Kauner", 1.0 },
  { "Kaung", 1.0 },
  { "Kauri", 0.1632653 },
  { "Kausar", 0.0 },
  { "Kausha", 0.0 },
  { "Kaushal", 1.0 },
  { "Kaushik", 1.0 },
  { "Kaustav", 1.0 },
  { "Kaustubh", 1.0 },
  { "Kava", 0.0 },
  { "Kavalli", 1.0 },
  { "Kavan", 1.0 },
  { "Kavanaugh", 1.0 },
  { "Kavante", 1.0 },
  { "Kavari", 1.0 },
  { "Kavarion", 1.0 },
  { "Kavaris", 1.0 },
  { "Kavaughn", 1.0 },
  { "Kavayah", 0.0 },
  { "Kaveah", 0.0 },
  { "Kaveen", 1.0 },
  { "Kaveer", 1.0 },
  { "Kaveesh", 1.0 },
  { "Kaveh", 1.0 },
  { "Kaveion", 1.0 },
  { "Kavell", 1.0 },
  { "Kaven", 1.0 },
  { "Kaveon", 1.0 },
  { "Kaveona", 0.0 },
  { "Kaveri", 0.0 },
  { "Kavi", 0.9907063 },
  { "Kavia", 0.0 },
  { "Kavian", 1.0 },
  { "Kavien", 1.0 },
  { "Kavik", 1.0 },
  { "Kavika", 1.0 },
  { "Kavin", 1.0 },
  { "Kavina", 0.0 },
  { "Kavion", 1.0 },
  { "Kaviona", 0.0 },
  { "Kavious", 1.0 },
  { "Kavir", 1.0 },
  { "Kavis", 1.0 },
  { "Kavish", 1.0 },
  { "Kavisha", 0.0 },
  { "Kavita", 0.0 },
  { "Kavitha", 0.0 },
  { "Kaviya", 0.0 },
  { "Kaviyon", 1.0 },
  { "Kavleen", 0.0 },
  { "Kavon", 0.9834008 },
  { "Kavona", 0.0 },
  { "Kavonda", 0.0 },
  { "Kavone", 1.0 },
  { "Kavonna", 0.0 },
  { "Kavonta", 1.0 },
  { "Kavontae", 1.0 },
  { "Kavonte", 1.0 },
  { "Kavy", 1.0 },
  { "Kavya", 0.004178273 },
  { "Kavyn", 1.0 },
  { "Kaw", 1.0 },
  { "Kawai", 0.8333333 },
  { "Kawaii", 0.0 },
  { "Kawailani", 0.0 },
  { "Kawaiola", 1.0 },
  { "Kawan", 0.97159094 },
  { "Kawana", 0.0 },
  { "Kawanda", 0.0 },
  { "Kawanis", 0.0 },
  { "Kawanna", 0.0 },
  { "Kawanza", 0.0 },
  { "Kawaski", 1.0 },
  { "Kawaun", 1.0 },
  { "Kawayne", 1.0 },
  { "Kawehi", 0.0 },
  { "Kawena", 0.07936508 },
  { "Kawhi", 1.0 },
  { "Kawika", 1.0 },
  { "Kawliga", 1.0 },
  { "Kawon", 1.0 },
  { "Kawona", 0.0 },
  { "Kawonna", 0.0 },
  { "Kawsar", 0.0 },
  { "Kawtar", 0.0 },
  { "Kawthar", 0.0 },
  { "Kawther", 0.0 },
  { "Kawuan", 1.0 },
  { "Kay", 0.03760548 },
  { "Kaya", 0.061140988 },
  { "Kayaan", 1.0 },
  { "Kayah", 0.0 },
  { "Kayahna", 0.0 },
  { "Kayal", 0.0 },
  { "Kayala", 0.0 },
  { "Kayan", 0.9298246 },
  { "Kayana", 0.0 },
  { "Kayani", 0.0 },
  { "Kayann", 0.0 },
  { "Kayanna", 0.0 },
  { "Kayannah", 0.0 },
  { "Kayanne", 0.0 },
  { "Kayatana", 0.0 },
  { "Kaybre", 0.0 },
  { "Kaybree", 0.0 },
  { "Kaybri", 0.0 },
  { "Kaybrie", 0.0 },
  { "Kayce", 0.06 },
  { "Kaycee", 0.032599837 },
  { "Kaycen", 0.9142091 },
  { "Kayceon", 1.0 },
  { "Kaycey", 0.0 },
  { "Kayci", 0.0 },
  { "Kaycia", 0.0 },
  { "Kaycie", 0.0 },
  { "Kaycin", 1.0 },
  { "Kaycion", 1.0 },
  { "Kayd", 1.0 },
  { "Kayda", 0.0 },
  { "Kaydan", 0.78352183 },
  { "Kaydance", 0.0 },
  { "Kayde", 0.4458042 },
  { "Kaydee", 0.0 },
  { "Kayden", 0.8551641 },
  { "Kaydence", 0.026408087 },
  { "Kaydenn", 1.0 },
  { "Kaydens", 0.0 },
  { "Kaydense", 0.0 },
  { "Kaydenze", 0.0 },
  { "Kaydi", 0.0 },
  { "Kaydia", 0.0 },
  { "Kaydian", 1.0 },
  { "Kaydie", 0.0 },
  { "Kaydien", 1.0 },
  { "Kaydience", 0.0 },
  { "Kaydin", 0.6683871 },
  { "Kaydince", 0.0 },
  { "Kaydn", 0.77131784 },
  { "Kaydnce", 0.0 },
  { "Kaydon", 0.9281867 },
  { "Kaydra", 0.0 },
  { "Kaydree", 0.0 },
  { "Kaydren", 1.0 },
  { "Kaydrian", 1.0 },
  { "Kaydrie", 0.0 },
  { "Kaydyn", 0.5175644 },
  { "Kaydynce", 0.0 },
  { "Kaydynn", 0.54545456 },
  { "Kaye", 0.036993455 },
  { "Kayeden", 1.0 },
  { "Kayedence", 0.0 },
  { "Kayedon", 1.0 },
  { "Kayel", 1.0 },
  { "Kayela", 0.0 },
  { "Kayelani", 0.0 },
  { "Kayelee", 0.0 },
  { "Kayeleigh", 0.0 },
  { "Kayelin", 0.0 },
  { "Kayella", 0.0 },
  { "Kayelle", 0.0 },
  { "Kayeloni", 0.0 },
  { "Kayelyn", 0.0 },
  { "Kayelynn", 0.0 },
  { "Kayen", 0.91935486 },
  { "Kayenat", 0.0 },
  { "Kayenne", 0.0 },
  { "Kayesha", 0.0 },
  { "Kayetta", 0.0 },
  { "Kaygan", 0.0 },
  { "Kayge", 1.0 },
  { "Kaygen", 0.5 },
  { "Kayhan", 1.0 },
  { "Kayhla", 0.0 },
  { "Kayia", 0.0 },
  { "Kayin", 0.8733459 },
  { "Kayisha", 0.0 },
  { "Kayja", 0.0 },
  { "Kayjah", 0.0 },
  { "Kayjuan", 1.0 },
  { "Kayke", 1.0 },
  { "Kayky", 1.0 },
  { "Kayl", 0.89237666 },
  { "Kayla", 0.0016868894 },
  { "Kaylaa", 0.0 },
  { "Kaylaann", 0.0 },
  { "Kaylaanne", 0.0 },
  { "Kaylab", 1.0 },
  { "Kaylae", 0.0 },
  { "Kaylah", 0.0 },
  { "Kaylahni", 0.0 },
  { "Kaylalynn", 0.0 },
  { "Kaylamae", 0.0 },
  { "Kaylamarie", 0.0 },
  { "Kaylan", 0.13345633 },
  { "Kaylana", 0.0 },
  { "Kayland", 0.24 },
  { "Kaylanee", 0.0 },
  { "Kaylani", 0.0 },
  { "Kaylanicole", 0.0 },
  { "Kaylanie", 0.0 },
  { "Kaylanii", 0.0 },
  { "Kaylann", 0.0 },
  { "Kaylanna", 0.0 },
  { "Kaylanne", 0.0 },
  { "Kaylanni", 0.0 },
  { "Kaylannie", 0.0 },
  { "Kaylany", 0.0 },
  { "Kaylar", 0.0 },
  { "Kaylarae", 0.0 },
  { "Kaylauni", 0.0 },
  { "Kayle", 0.14039022 },
  { "Kaylea", 0.0 },
  { "Kayleah", 0.0 },
  { "Kaylean", 0.0 },
  { "Kayleana", 0.0 },
  { "Kayleann", 0.0 },
  { "Kayleanna", 0.0 },
  { "Kayleb", 1.0 },
  { "Kaylee", 0.0014451583 },
  { "Kayleeana", 0.0 },
  { "Kayleeann", 0.0 },
  { "Kayleeanna", 0.0 },
  { "Kayleeanne", 0.0 },
  { "Kayleeh", 0.0 },
  { "Kayleejo", 0.0 },
  { "Kayleemae", 0.0 },
  { "Kayleemarie", 0.0 },
  { "Kayleen", 0.0 },
  { "Kayleena", 0.0 },
  { "Kayleene", 0.0 },
  { "Kayleeonna", 0.0 },
  { "Kayleh", 0.0 },
  { "Kaylei", 0.0 },
  { "Kayleigh", 0.00024037636 },
  { "Kayleigha", 0.0 },
  { "Kayleighann", 0.0 },
  { "Kaylem", 1.0 },
  { "Kaylen", 0.1351017 },
  { "Kaylena", 0.0 },
  { "Kaylene", 0.0 },
  { "Kaylenn", 0.0 },
  { "Kayler", 0.46521738 },
  { "Kaylese", 0.0 },
  { "Kayley", 0.0 },
  { "Kayli", 0.0 },
  { "Kaylia", 0.0 },
  { "Kayliah", 0.0 },
  { "Kaylian", 0.0 },
  { "Kayliana", 0.0 },
  { "Kayliani", 0.0 },
  { "Kayliann", 0.0 },
  { "Kaylianna", 0.0 },
  { "Kaylianne", 0.0 },
  { "Kaylib", 1.0 },
  { "Kaylie", 0.0 },
  { "Kaylieann", 0.0 },
  { "Kayliee", 0.0 },
  { "Kayliegh", 0.0 },
  { "Kaylien", 0.0 },
  { "Kaylii", 0.0 },
  { "Kaylin", 0.029490467 },
  { "Kaylina", 0.0 },
  { "Kaylinda", 0.0 },
  { "Kayline", 0.0 },
  { "Kaylinn", 0.0 },
  { "Kaylis", 0.0 },
  { "Kaylisa", 0.0 },
  { "Kaylise", 0.0 },
  { "Kaylisha", 0.0 },
  { "Kaylissa", 0.0 },
  { "Kaylix", 0.0 },
  { "Kayliyah", 0.0 },
  { "Kaylla", 0.0 },
  { "Kaylle", 0.0 },
  { "Kayllie", 0.0 },
  { "Kayln", 0.0 },
  { "Kaylnn", 0.0 },
  { "Kaylo", 1.0 },
  { "Kaylob", 1.0 },
  { "Kaylon", 0.631101 },
  { "Kayloni", 0.0 },
  { "Kaylonie", 0.0 },
  { "Kaylonni", 0.0 },
  { "Kaylonnie", 0.0 },
  { "Kaylor", 0.36865342 },
  { "Kaylub", 1.0 },
  { "Kaylum", 1.0 },
  { "Kaylup", 1.0 },
  { "Kayly", 0.0 },
  { "Kaylyn", 0.0028378859 },
  { "Kaylyne", 0.0 },
  { "Kaylynn", 0.00047678078 },
  { "Kaylynne", 0.0 },
  { "Kayman", 0.8670886 },
  { "Kaymani", 0.0 },
  { "Kaymarie", 0.0 },
  { "Kayme", 0.0 },
  { "Kaymen", 0.93048126 },
  { "Kaymi", 0.0 },
  { "Kaymiah", 0.0 },
  { "Kaymie", 0.0 },
  { "Kaymin", 0.0 },
  { "Kaymon", 1.0 },
  { "Kaymoni", 0.0 },
  { "Kayn", 1.0 },
  { "Kayna", 0.0 },
  { "Kaynan", 1.0 },
  { "Kayne", 0.99129933 },
  { "Kaynen", 1.0 },
  { "Kaynin", 1.0 },
  { "Kaynon", 1.0 },
  { "Kayo", 0.9734042 },
  { "Kayode", 1.0 },
  { "Kayoko", 0.0 },
  { "Kayon", 0.97727275 },
  { "Kayona", 0.0 },
  { "Kayoni", 0.0 },
  { "Kayonia", 0.0 },
  { "Kayonna", 0.0 },
  { "Kayori", 0.0 },
  { "Kayra", 0.008976661 },
  { "Kayrah", 0.0 },
  { "Kayren", 0.0 },
  { "Kayrene", 0.0 },
  { "Kayri", 0.0 },
  { "Kayro", 1.0 },
  { "Kayron", 0.7792642 },
  { "Kaysa", 0.0 },
  { "Kaysaan", 1.0 },
  { "Kaysan", 1.0 },
  { "Kaysar", 1.0 },
  { "Kaysaun", 1.0 },
  { "Kayse", 0.051454138 },
  { "Kaysea", 0.0 },
  { "Kaysean", 1.0 },
  { "Kaysee", 0.0 },
  { "Kaysen", 0.8768898 },
  { "Kayser", 1.0 },
  { "Kaysey", 0.0 },
  { "Kaysha", 0.0 },
  { "Kayshaun", 1.0 },
  { "Kayshawn", 1.0 },
  { "Kayshia", 0.0 },
  { "Kayshla", 0.0 },
  { "Kayshon", 1.0 },
  { "Kaysi", 0.0 },
  { "Kaysia", 0.0 },
  { "Kaysie", 0.0 },
  { "Kaysin", 0.9659864 },
  { "Kayslee", 0.0 },
  { "Kaysleigh", 0.0 },
  { "Kaysley", 0.0 },
  { "Kayslie", 0.0 },
  { "Kaysn", 1.0 },
  { "Kayson", 0.96452504 },
  { "Kaysten", 1.0 },
  { "Kayston", 1.0 },
  { "Kaysun", 1.0 },
  { "Kaysyn", 1.0 },
  { "Kayta", 0.0 },
  { "Kayte", 0.0 },
  { "Kaytee", 0.0 },
  { "Kaytelyn", 0.0 },
  { "Kaytelynn", 0.0 },
  { "Kayten", 0.0 },
  { "Kayti", 0.0 },
  { "Kaytie", 0.0 },
  { "Kaytin", 0.0 },
  { "Kaytlan", 0.0 },
  { "Kaytlen", 0.0 },
  { "Kaytlin", 0.0 },
  { "Kaytlinn", 0.0 },
  { "Kaytlyn", 0.0 },
  { "Kaytlyne", 0.0 },
  { "Kaytlynn", 0.0 },
  { "Kaytlynne", 0.0 },
  { "Kayto", 1.0 },
  { "Kayton", 0.40873015 },
  { "Kayvan", 1.0 },
  { "Kayven", 1.0 },
  { "Kayveon", 1.0 },
  { "Kayvien", 1.0 },
  { "Kayvin", 1.0 },
  { "Kayvion", 1.0 },
  { "Kayvon", 1.0 },
  { "Kayvonna", 0.0 },
  { "Kaywin", 0.0 },
  { "Kayzen", 1.0 },
  { "Kayzia", 0.0 },
  { "Kayzlee", 0.0 },
  { "Kayzlie", 0.0 },
  { "Kayzlyn", 0.0 },
  { "Kaz", 1.0 },
  { "Kazandra", 0.0 },
  { "Kazaria", 0.0 },
  { "Kazariah", 0.0 },
  { "Kazden", 1.0 },
  { "Kaze", 1.0 },
  { "Kazeem", 1.0 },
  { "Kazen", 1.0 },
  { "Kazi", 0.7678019 },
  { "Kazia", 0.0 },
  { "Kaziah", 0.048295453 },
  { "Kazier", 1.0 },
  { "Kazim", 1.0 },
  { "Kazimer", 1.0 },
  { "Kazimiera", 0.0 },
  { "Kazimierz", 1.0 },
  { "Kazimir", 1.0 },
  { "Kazir", 1.0 },
  { "Kaziya", 0.0 },
  { "Kaziyah", 0.0 },
  { "Kazlyn", 0.0 },
  { "Kazlynn", 0.0 },
  { "Kazmer", 1.0 },
  { "Kazmiera", 0.0 },
  { "Kazmir", 1.0 },
  { "Kazmira", 0.0 },
  { "Kazoua", 0.0 },
  { "Kazu", 0.5 },
  { "Kazue", 0.0 },
  { "Kazuhiro", 1.0 },
  { "Kazuki", 1.0 },
  { "Kazuko", 0.0 },
  { "Kazuma", 1.0 },
  { "Kazumi", 0.83458644 },
  { "Kazuo", 1.0 },
  { "Kazuto", 1.0 },
  { "Kazuya", 1.0 },
  { "Kazuye", 0.0 },
  { "Kazuyo", 0.0 },
  { "Kazuyoshi", 1.0 },
  { "Kazuyuki", 1.0 },
  { "Kazz", 1.0 },
  { "Kazzandra", 0.0 },
  { "Kc", 0.69016844 },
  { "Kcee", 0.0 },
  { "Kceon", 1.0 },
  { "Kci", 1.0 },
  { "Kd", 0.0 },
  { "Kden", 1.0 },
  { "Kdence", 0.0 },
  { "Kdrian", 1.0 },
  { "Kdyn", 1.0 },
  { "Kdynce", 0.0 },
  { "Kdynn", 1.0 },
  { "Ke", 1.0 },
  { "Kea", 0.0 },
  { "Keacha", 0.0 },
  { "Keaden", 1.0 },
  { "Keadon", 1.0 },
  { "Keadra", 0.0 },
  { "Keadrian", 1.0 },
  { "Keadyn", 1.0 },
  { "Keagan", 0.8398633 },
  { "Keagen", 0.97727275 },
  { "Keaghan", 0.752809 },
  { "Keaghlan", 0.0 },
  { "Keagin", 1.0 },
  { "Keagon", 1.0 },
  { "Keagyn", 0.5 },
  { "Keah", 0.0 },
  { "Keahi", 0.9141631 },
  { "Keahna", 0.0 },
  { "Keair", 1.0 },
  { "Keaira", 0.0 },
  { "Keairah", 0.0 },
  { "Keaire", 0.4 },
  { "Keairra", 0.0 },
  { "Keaisa", 0.0 },
  { "Keaisha", 0.0 },
  { "Keaja", 0.0 },
  { "Keajah", 0.0 },
  { "Keajia", 0.0 },
  { "Keaka", 1.0 },
  { "Keala", 0.028833551 },
  { "Kealan", 1.0 },
  { "Kealana", 0.0 },
  { "Kealani", 0.0 },
  { "Kealee", 0.0 },
  { "Kealeigh", 0.0 },
  { "Kealen", 1.0 },
  { "Kealey", 0.0 },
  { "Keali", 0.0 },
  { "Kealia", 0.0 },
  { "Kealie", 0.0 },
  { "Kealii", 1.0 },
  { "Kealin", 1.0 },
  { "Keally", 0.0 },
  { "Kealoha", 0.47058824 },
  { "Kealohi", 0.0 },
  { "Kealohilani", 0.0 },
  { "Kealon", 1.0 },
  { "Kealy", 0.0 },
  { "Kealyn", 0.0 },
  { "Kealynn", 0.0 },
  { "Keamber", 0.0 },
  { "Keambra", 0.0 },
  { "Keambria", 0.0 },
  { "Kean", 1.0 },
  { "Keana", 0.0 },
  { "Keanah", 0.0 },
  { "Keanan", 1.0 },
  { "Keanda", 0.0 },
  { "Keandra", 0.0072780205 },
  { "Keandrae", 1.0 },
  { "Keandre", 0.99660504 },
  { "Keandrea", 0.0 },
  { "Keandria", 0.0 },
  { "Keane", 1.0 },
  { "Keanen", 1.0 },
  { "Keangelo", 1.0 },
  { "Keani", 0.06772009 },
  { "Keann", 0.0 },
  { "Keanna", 0.0 },
  { "Keannah", 0.0 },
  { "Keanne", 0.0 },
  { "Keano", 1.0 },
  { "Keanon", 1.0 },
  { "Keantae", 1.0 },
  { "Keantay", 1.0 },
  { "Keante", 0.9411765 },
  { "Keanthony", 1.0 },
  { "Keanu", 0.95894307 },
  { "Keanua", 0.0 },
  { "Keanya", 0.0 },
  { "Keaon", 1.0 },
  { "Keara", 0.0 },
  { "Kearah", 0.0 },
  { "Kearea", 0.0 },
  { "Keari", 0.0 },
  { "Kearia", 0.0 },
  { "Kearie", 0.0 },
  { "Kearius", 1.0 },
  { "Kearn", 0.0 },
  { "Kearney", 0.8409786 },
  { "Kearra", 0.0 },
  { "Kearrah", 0.0 },
  { "Kearria", 0.0 },
  { "Kearson", 0.0 },
  { "Kearstan", 0.0 },
  { "Kearsten", 0.0 },
  { "Kearstin", 0.0 },
  { "Kearston", 0.013986014 },
  { "Kearstyn", 0.0 },
  { "Keary", 0.8479532 },
  { "Keasha", 0.0 },
  { "Keashawn", 1.0 },
  { "Keashia", 0.0 },
  { "Keasia", 0.0 },
  { "Keasiah", 0.0 },
  { "Keason", 1.0 },
  { "Keaston", 1.0 },
  { "Keasya", 0.0 },
  { "Keatan", 1.0 },
  { "Keaten", 0.98559076 },
  { "Keath", 1.0 },
  { "Keathan", 1.0 },
  { "Keather", 0.0 },
  { "Keatin", 1.0 },
  { "Keating", 0.72727275 },
  { "Keaton", 0.94283116 },
  { "Keats", 1.0 },
  { "Keatyn", 0.344 },
  { "Keaun", 1.0 },
  { "Keauna", 0.0 },
  { "Keaundra", 0.0 },
  { "Keaundre", 1.0 },
  { "Keaundrea", 0.0 },
  { "Keaundria", 0.0 },
  { "Keaunna", 0.0 },
  { "Keaunte", 1.0 },
  { "Keaura", 0.0 },
  { "Keauri", 0.0 },
  { "Keausha", 0.0 },
  { "Keaven", 1.0 },
  { "Keavin", 1.0 },
  { "Keavion", 1.0 },
  { "Keavon", 1.0 },
  { "Keawe", 1.0 },
  { "Keayla", 0.0 },
  { "Keayon", 1.0 },
  { "Keayra", 0.0 },
  { "Keaysia", 0.0 },
  { "Keazia", 0.0 },
  { "Keb", 1.0 },
  { "Keba", 0.0 },
  { "Kebba", 1.0 },
  { "Kebbie", 0.0 },
  { "Keben", 1.0 },
  { "Kebin", 1.0 },
  { "Kebra", 0.0 },
  { "Kebria", 0.0 },
  { "Kebrina", 0.0 },
  { "Kebron", 0.8103448 },
  { "Kecha", 0.0 },
  { "Kechelle", 0.0 },
  { "Kechia", 0.0 },
  { "Kecia", 0.0 },
  { "Kedan", 1.0 },
  { "Kedar", 1.0 },
  { "Kedarian", 1.0 },
  { "Kedarious", 1.0 },
  { "Kedarius", 1.0 },
  { "Kedarrius", 1.0 },
  { "Keddrick", 1.0 },
  { "Kededra", 0.0 },
  { "Kedeem", 1.0 },
  { "Keden", 1.0 },
  { "Kederick", 1.0 },
  { "Kedesha", 0.0 },
  { "Kedeshia", 0.0 },
  { "Kedisha", 0.0 },
  { "Kedra", 0.0 },
  { "Kedran", 1.0 },
  { "Kedrian", 1.0 },
  { "Kedric", 1.0 },
  { "Kedrick", 1.0 },
  { "Kedrin", 1.0 },
  { "Kedron", 0.96666664 },
  { "Kedryn", 1.0 },
  { "Kedus", 1.0 },
  { "Keduse", 1.0 },
  { "Kedwin", 1.0 },
  { "Kedzie", 0.0 },
  { "Kee", 0.99293786 },
  { "Keeaira", 0.0 },
  { "Keean", 1.0 },
  { "Keeana", 0.0 },
  { "Keeanna", 0.0 },
  { "Keeara", 0.0 },
  { "Keeba", 0.0 },
  { "Keecha", 0.0 },
  { "Keedah", 0.0 },
  { "Keedan", 1.0 },
  { "Keeden", 1.0 },
  { "Keedon", 1.0 },
  { "Keedra", 0.0 },
  { "Keedyn", 1.0 },
  { "Keefe", 1.0 },
  { "Keefer", 1.0 },
  { "Keegan", 0.91186756 },
  { "Keegen", 1.0 },
  { "Keeghan", 0.80929095 },
  { "Keegin", 1.0 },
  { "Keegon", 1.0 },
  { "Keegun", 1.0 },
  { "Keeisha", 0.0 },
  { "Keekee", 0.0 },
  { "Keela", 0.0 },
  { "Keelah", 0.0 },
  { "Keelan", 0.90140396 },
  { "Keeland", 1.0 },
  { "Keelani", 0.0 },
  { "Keele", 0.0 },
  { "Keelee", 0.0 },
  { "Keeleigh", 0.0 },
  { "Keelen", 1.0 },
  { "Keeler", 1.0 },
  { "Keeley", 0.0083617745 },
  { "Keeli", 0.0 },
  { "Keelia", 0.0 },
  { "Keelie", 0.0 },
  { "Keelin", 0.22125985 },
  { "Keelon", 1.0 },
  { "Keely", 0.001744186 },
  { "Keelyn", 0.22084624 },
  { "Keelynn", 0.073529415 },
  { "Keema", 0.0 },
  { "Keemia", 0.0 },
  { "Keemo", 1.0 },
  { "Keemon", 1.0 },
  { "Keemora", 0.0 },
  { "Keemya", 0.0 },
  { "Keen", 1.0 },
  { "Keena", 0.062169313 },
  { "Keenan", 0.980297 },
  { "Keene", 1.0 },
  { "Keenen", 1.0 },
  { "Keener", 1.0 },
  { "Keeng", 1.0 },
  { "Keenin", 1.0 },
  { "Keeno", 1.0 },
  { "Keenon", 1.0 },
  { "Keenya", 0.0 },
  { "Keenyah", 0.0 },
  { "Keenyn", 1.0 },
  { "Keeon", 1.0 },
  { "Keeona", 0.0 },
  { "Keera", 0.0 },
  { "Keerah", 0.0 },
  { "Keeran", 1.0 },
  { "Keerat", 0.061728396 },
  { "Keerica", 0.0 },
  { "Keeron", 1.0 },
  { "Keerra", 0.0 },
  { "Keerstin", 0.0 },
  { "Keertana", 0.0 },
  { "Keerthan", 1.0 },
  { "Keerthana", 0.0 },
  { "Keerthi", 0.0 },
  { "Keerti", 0.0 },
  { "Kees", 1.0 },
  { "Keesa", 0.0 },
  { "Keesean", 1.0 },
  { "Keesha", 0.0 },
  { "Keeshan", 0.0 },
  { "Keeshaun", 1.0 },
  { "Keeshawn", 1.0 },
  { "Keeshia", 0.0 },
  { "Keeshon", 1.0 },
  { "Keeson", 1.0 },
  { "Keeston", 1.0 },
  { "Keeta", 0.0 },
  { "Keetan", 1.0 },
  { "Keethan", 1.0 },
  { "Keeton", 1.0 },
  { "Keetra", 0.0 },
  { "Keeva", 0.0 },
  { "Keevan", 1.0 },
  { "Keeven", 1.0 },
  { "Keevin", 1.0 },
  { "Keevis", 1.0 },
  { "Keevon", 1.0 },
  { "Keeya", 0.0 },
  { "Keeyan", 1.0 },
  { "Keeyana", 0.0 },
  { "Keeyon", 1.0 },
  { "Kefentse", 1.0 },
  { "Keffer", 1.0 },
  { "Kefira", 0.0 },
  { "Kefren", 1.0 },
  { "Kegan", 0.93976533 },
  { "Keghan", 1.0 },
  { "Kehara", 0.0 },
  { "Kehaulani", 0.0 },
  { "Kehila", 0.0 },
  { "Kehinde", 0.6309013 },
  { "Kehlani", 0.0 },
  { "Kehley", 0.0 },
  { "Kehly", 0.0 },
  { "Kei", 0.73501575 },
  { "Keia", 0.0 },
  { "Keiah", 0.0 },
  { "Keiaira", 0.0 },
  { "Keiairra", 0.0 },
  { "Keiaja", 0.0 },
  { "Keiajah", 0.0 },
  { "Keian", 1.0 },
  { "Keiana", 0.0 },
  { "Keiandra", 0.0 },
  { "Keiandre", 1.0 },
  { "Keiani", 0.0 },
  { "Keiann", 0.0 },
  { "Keianna", 0.0 },
  { "Keiara", 0.0 },
  { "Keiarah", 0.0 },
  { "Keiari", 0.0 },
  { "Keiarra", 0.0 },
  { "Keiarriyen", 1.0 },
  { "Keiasha", 0.0 },
  { "Keiasia", 0.0 },
  { "Keiauna", 0.0 },
  { "Keica", 0.0 },
  { "Keicha", 0.0 },
  { "Keichia", 0.0 },
  { "Keicy", 0.0 },
  { "Keida", 0.0 },
  { "Keidan", 1.0 },
  { "Keiden", 1.0 },
  { "Keidi", 0.0 },
  { "Keidon", 1.0 },
  { "Keidra", 0.0 },
  { "Keidren", 1.0 },
  { "Keidrick", 1.0 },
  { "Keidy", 0.0 },
  { "Keidyn", 1.0 },
  { "Keiera", 0.0 },
  { "Keierra", 0.0 },
  { "Keif", 1.0 },
  { "Keifer", 1.0 },
  { "Keiffer", 1.0 },
  { "Keigan", 0.8767123 },
  { "Keigen", 1.0 },
  { "Keighan", 0.8639456 },
  { "Keighla", 0.0 },
  { "Keighley", 0.0 },
  { "Keighly", 0.0 },
  { "Keighton", 1.0 },
  { "Keigo", 1.0 },
  { "Keijah", 0.0 },
  { "Keiji", 1.0 },
  { "Keijon", 1.0 },
  { "Keijuan", 1.0 },
  { "Keikilani", 0.0 },
  { "Keiko", 0.0 },
  { "Keil", 1.0 },
  { "Keila", 0.0 },
  { "Keilah", 0.0 },
  { "Keilan", 0.9580078 },
  { "Keilana", 0.0 },
  { "Keiland", 1.0 },
  { "Keilani", 0.0023980816 },
  { "Keilanie", 0.0 },
  { "Keilany", 0.0 },
  { "Keile", 0.0 },
  { "Keileb", 1.0 },
  { "Keilee", 0.0 },
  { "Keileen", 0.0 },
  { "Keilei", 0.0 },
  { "Keileigh", 0.0 },
  { "Keilen", 0.86283183 },
  { "Keiler", 1.0 },
  { "Keiley", 0.0 },
  { "Keili", 0.0 },
  { "Keilia", 0.0 },
  { "Keiliana", 0.0 },
  { "Keiliany", 0.0 },
  { "Keilianys", 0.0 },
  { "Keilie", 0.0 },
  { "Keilin", 0.2712766 },
  { "Keilly", 0.0 },
  { "Keilon", 1.0 },
  { "Keily", 0.0 },
  { "Keilyn", 0.1809909 },
  { "Keilynn", 0.0 },
  { "Keimani", 0.22818792 },
  { "Keimar", 1.0 },
  { "Keimara", 0.0 },
  { "Keimari", 0.6962025 },
  { "Keimarion", 1.0 },
  { "Keimaya", 0.0 },
  { "Keimiyah", 0.0 },
  { "Keimon", 1.0 },
  { "Keimoni", 0.5555556 },
  { "Keimora", 0.0 },
  { "Keimy", 0.0 },
  { "Keimya", 0.0 },
  { "Keimyah", 0.0 },
  { "Kein", 1.0 },
  { "Keina", 0.0 },
  { "Keinan", 1.0 },
  { "Keiner", 1.0 },
  { "Keing", 1.0 },
  { "Keino", 1.0 },
  { "Keinya", 0.0 },
  { "Keion", 0.9958949 },
  { "Keiona", 0.0 },
  { "Keiondra", 0.0 },
  { "Keiondre", 1.0 },
  { "Keionna", 0.0 },
  { "Keiontae", 1.0 },
  { "Keiontay", 1.0 },
  { "Keionte", 1.0 },
  { "Keiora", 0.0 },
  { "Keiosha", 0.0 },
  { "Keir", 0.9356137 },
  { "Keira", 0.000252945 },
  { "Keirah", 0.0 },
  { "Keiralee", 0.0 },
  { "Keiralyn", 0.0 },
  { "Keiran", 0.8536336 },
  { "Keiren", 0.91071427 },
  { "Keiri", 0.0 },
  { "Keirin", 0.0 },
  { "Keirnan", 1.0 },
  { "Keiron", 1.0 },
  { "Keirra", 0.0 },
  { "Keirrah", 0.0 },
  { "Keirslyn", 0.0 },
  { "Keirstan", 0.0 },
  { "Keirsten", 0.0 },
  { "Keirstin", 0.0 },
  { "Keirston", 0.0 },
  { "Keirstyn", 0.0 },
  { "Keiry", 0.0 },
  { "Keirya", 0.0 },
  { "Keiryn", 0.0 },
  { "Keisa", 0.0 },
  { "Keischa", 0.0 },
  { "Keisean", 1.0 },
  { "Keisel", 1.0 },
  { "Keiser", 1.0 },
  { "Keisey", 0.0 },
  { "Keish", 0.0 },
  { "Keisha", 0.0020112447 },
  { "Keishana", 0.0 },
  { "Keishanda", 0.0 },
  { "Keishanna", 0.0 },
  { "Keishara", 0.0 },
  { "Keishaun", 1.0 },
  { "Keishauna", 0.0 },
  { "Keishawn", 0.91828793 },
  { "Keishawna", 0.0 },
  { "Keisher", 0.0 },
  { "Keishi", 1.0 },
  { "Keishia", 0.0 },
  { "Keishla", 0.0 },
  { "Keishon", 1.0 },
  { "Keishona", 0.0 },
  { "Keishonna", 0.0 },
  { "Keishun", 1.0 },
  { "Keisi", 0.0 },
  { "Keison", 1.0 },
  { "Keissy", 0.0 },
  { "Keiston", 1.0 },
  { "Keisuke", 1.0 },
  { "Keisy", 0.0 },
  { "Keita", 0.69711536 },
  { "Keitaro", 1.0 },
  { "Keiten", 1.0 },
  { "Keith", 0.9949188 },
  { "Keitha", 0.0 },
  { "Keithan", 1.0 },
  { "Keithen", 1.0 },
  { "Keither", 1.0 },
  { "Keithin", 1.0 },
  { "Keithon", 1.0 },
  { "Keithrick", 1.0 },
  { "Keitlyn", 0.0 },
  { "Keito", 1.0 },
  { "Keiton", 1.0 },
  { "Keitra", 0.0 },
  { "Keitric", 1.0 },
  { "Keitrick", 1.0 },
  { "Keity", 0.0 },
  { "Keiuna", 0.0 },
  { "Keiundra", 0.0 },
  { "Keiva", 0.0 },
  { "Keivan", 1.0 },
  { "Keiven", 1.0 },
  { "Keiveon", 1.0 },
  { "Keivin", 1.0 },
  { "Keivn", 1.0 },
  { "Keivon", 1.0 },
  { "Keivonte", 1.0 },
  { "Keiwan", 1.0 },
  { "Keiwon", 1.0 },
  { "Keiya", 0.0 },
  { "Keiyana", 0.0 },
  { "Keiyanna", 0.0 },
  { "Keiyara", 0.0 },
  { "Keiyon", 1.0 },
  { "Keiyona", 0.0 },
  { "Keiyonna", 0.0 },
  { "Keizer", 1.0 },
  { "Keja", 0.0 },
  { "Kejaun", 1.0 },
  { "Kejohn", 1.0 },
  { "Kejon", 1.0 },
  { "Kejuan", 1.0 },
  { "Kejuana", 0.0 },
  { "Kekai", 1.0 },
  { "Keke", 0.0 },
  { "Kekeli", 0.17857143 },
  { "Kekoa", 0.9854785 },
  { "Kel", 1.0 },
  { "Kela", 0.0 },
  { "Kelab", 1.0 },
  { "Kelah", 0.0 },
  { "Kelahni", 0.0 },
  { "Kelaia", 0.0 },
  { "Kelaiah", 0.0 },
  { "Kelaine", 0.0 },
  { "Kelan", 1.0 },
  { "Kelana", 0.0 },
  { "Keland", 1.0 },
  { "Kelani", 0.059957173 },
  { "Kelanie", 0.0 },
  { "Kelany", 0.0 },
  { "Kelaya", 0.0 },
  { "Kelayah", 0.0 },
  { "Kelbe", 0.0 },
  { "Kelbi", 0.0 },
  { "Kelbie", 0.0 },
  { "Kelby", 0.72803825 },
  { "Kelce", 0.0 },
  { "Kelcea", 0.0 },
  { "Kelcee", 0.0 },
  { "Kelcei", 0.0 },
  { "Kelcey", 0.25018078 },
  { "Kelci", 0.0 },
  { "Kelcie", 0.011624254 },
  { "Kelcy", 0.07638889 },
  { "Kelda", 0.0 },
  { "Keldan", 1.0 },
  { "Kelden", 1.0 },
  { "Keldin", 1.0 },
  { "Keldon", 1.0 },
  { "Keldric", 1.0 },
  { "Keldrick", 1.0 },
  { "Keldrin", 1.0 },
  { "Keldyn", 1.0 },
  { "Kele", 0.54268295 },
  { "Kelea", 0.0 },
  { "Keleah", 0.0 },
  { "Kelechi", 0.67236465 },
  { "Kelechukwu", 1.0 },
  { "Kelecia", 0.0 },
  { "Kelee", 0.0 },
  { "Keleen", 0.0 },
  { "Kelei", 0.0 },
  { "Keleigh", 0.0 },
  { "Kelela", 0.0 },
  { "Kelen", 0.877451 },
  { "Kelena", 0.0 },
  { "Kelene", 0.0 },
  { "Keleni", 0.0 },
  { "Kelesha", 0.0 },
  { "Keley", 0.0 },
  { "Keli", 0.011215381 },
  { "Kelia", 0.0 },
  { "Keliah", 0.0 },
  { "Kelian", 1.0 },
  { "Keliana", 0.0 },
  { "Keliann", 0.0 },
  { "Kelianne", 0.0 },
  { "Kelicia", 0.0 },
  { "Kelie", 0.0 },
  { "Kelii", 1.0 },
  { "Kelijah", 0.89915967 },
  { "Kelik", 1.0 },
  { "Kelila", 0.0 },
  { "Kelilah", 0.0 },
  { "Kelin", 0.5869281 },
  { "Kelina", 0.0 },
  { "Kelinda", 0.0 },
  { "Kelis", 0.0025795356 },
  { "Kelisa", 0.0 },
  { "Kelise", 0.0 },
  { "Kelisha", 0.0 },
  { "Kelisia", 0.0 },
  { "Kelissa", 0.0 },
  { "Kelita", 0.0 },
  { "Keliyah", 0.0 },
  { "Kell", 1.0 },
  { "Kella", 0.0 },
  { "Kellam", 1.0 },
  { "Kellan", 0.94551164 },
  { "Kelland", 1.0 },
  { "Kellar", 1.0 },
  { "Kelle", 0.018336887 },
  { "Kellea", 0.0 },
  { "Kellee", 0.0043022316 },
  { "Kelleen", 0.0 },
  { "Kellei", 0.0 },
  { "Kelleigh", 0.0 },
  { "Kellen", 0.9245851 },
  { "Kellene", 0.0 },
  { "Keller", 0.89805824 },
  { "Kelley", 0.11818147 },
  { "Kelleyann", 0.0 },
  { "Kelleye", 0.0 },
  { "Kelli", 0.004002989 },
  { "Kellia", 0.0 },
  { "Kellian", 0.2857143 },
  { "Kelliann", 0.0 },
  { "Kellianne", 0.0 },
  { "Kellie", 0.0069576837 },
  { "Kellieann", 0.0 },
  { "Kellijo", 0.0 },
  { "Kellin", 0.8863346 },
  { "Kellina", 0.0 },
  { "Kelline", 0.0 },
  { "Kellis", 0.5285714 },
  { "Kellisa", 0.0 },
  { "Kellise", 0.0 },
  { "Kellisha", 0.0 },
  { "Kellissa", 0.0 },
  { "Kellogg", 1.0 },
  { "Kellon", 1.0 },
  { "Kellsea", 0.0 },
  { "Kellsey", 0.0 },
  { "Kellsi", 0.0 },
  { "Kellsie", 0.0 },
  { "Kellsy", 0.0 },
  { "Kellum", 1.0 },
  { "Kelly", 0.14754458 },
  { "Kellyann", 0.0 },
  { "Kellyanna", 0.0 },
  { "Kellyanne", 0.0 },
  { "Kellye", 0.0 },
  { "Kellyjean", 0.0 },
  { "Kellyjo", 0.0 },
  { "Kellylynn", 0.0 },
  { "Kellymarie", 0.0 },
  { "Kellyn", 0.07785642 },
  { "Kellynn", 0.0 },
  { "Kelman", 1.0 },
  { "Kelmer", 1.0 },
  { "Kelon", 1.0 },
  { "Kelsa", 0.0 },
  { "Kelsay", 0.0 },
  { "Kelse", 0.0 },
  { "Kelsea", 0.0009066183 },
  { "Kelsee", 0.0 },
  { "Kelsei", 0.0 },
  { "Kelseigh", 0.0 },
  { "Kelsen", 1.0 },
  { "Kelsey", 0.034433544 },
  { "Kelseyann", 0.0 },
  { "Kelseyanne", 0.0 },
  { "Kelsha", 0.0 },
  { "Kelsi", 0.0 },
  { "Kelsia", 0.0 },
  { "Kelsie", 0.018324746 },
  { "Kelso", 1.0 },
  { "Kelson", 1.0 },
  { "Kelston", 1.0 },
  { "Kelsy", 0.012682927 },
  { "Kelsye", 0.0 },
  { "Kelsyn", 0.0 },
  { "Keltan", 1.0 },
  { "Kelten", 1.0 },
  { "Keltie", 0.0 },
  { "Keltin", 1.0 },
  { "Kelton", 0.9956775 },
  { "Keltsey", 0.0 },
  { "Kelty", 0.0 },
  { "Keltyn", 1.0 },
  { "Kelvan", 1.0 },
  { "Kelven", 1.0 },
  { "Kelveon", 1.0 },
  { "Kelvin", 0.9938607 },
  { "Kelvina", 0.0 },
  { "Kelvion", 1.0 },
  { "Kelvis", 1.0 },
  { "Kelvisha", 0.0 },
  { "Kelvon", 1.0 },
  { "Kelvontae", 1.0 },
  { "Kelvonte", 1.0 },
  { "Kelvy", 1.0 },
  { "Kelvyn", 1.0 },
  { "Kelwin", 1.0 },
  { "Kely", 0.041322313 },
  { "Kelyn", 0.41604754 },
  { "Kelynn", 0.30882353 },
  { "Kelyse", 0.0 },
  { "Kem", 0.65827686 },
  { "Kema", 0.0 },
  { "Kemaj", 1.0 },
  { "Kemal", 1.0 },
  { "Kemani", 0.4566265 },
  { "Kemanie", 0.0 },
  { "Kemar", 1.0 },
  { "Kemara", 0.0 },
  { "Kemarah", 0.0 },
  { "Kemarcus", 1.0 },
  { "Kemare", 1.0 },
  { "Kemaree", 1.0 },
  { "Kemareon", 1.0 },
  { "Kemari", 0.723589 },
  { "Kemaria", 0.0 },
  { "Kemariah", 0.0 },
  { "Kemarian", 1.0 },
  { "Kemarie", 0.0 },
  { "Kemarii", 0.0 },
  { "Kemarion", 1.0 },
  { "Kemarius", 1.0 },
  { "Kemariya", 0.0 },
  { "Kemariyah", 0.0 },
  { "Kemarley", 1.0 },
  { "Kemarri", 1.0 },
  { "Kemarrion", 1.0 },
  { "Kemarui", 1.0 },
  { "Kemauri", 0.8357771 },
  { "Kemauria", 0.0 },
  { "Kemaurie", 1.0 },
  { "Kemaurion", 1.0 },
  { "Kemaury", 1.0 },
  { "Kemaya", 0.0 },
  { "Kemba", 0.0 },
  { "Kember", 0.0 },
  { "Kemberley", 0.0 },
  { "Kemberli", 0.0 },
  { "Kemberly", 0.0 },
  { "Kemberlyn", 0.0 },
  { "Keme", 0.0 },
  { "Kemeisha", 0.0 },
  { "Kemel", 1.0 },
  { "Kemely", 0.0 },
  { "Kemesha", 0.0 },
  { "Kemeshia", 0.0 },
  { "Kemet", 0.9220779 },
  { "Kemi", 0.0 },
  { "Kemia", 0.0 },
  { "Kemiah", 0.0 },
  { "Kemilly", 0.0 },
  { "Kemily", 0.0 },
  { "Kemira", 0.0 },
  { "Kemisha", 0.0 },
  { "Kemiya", 0.0 },
  { "Kemiyah", 0.0 },
  { "Kemma", 0.0 },
  { "Kemmie", 0.0 },
  { "Kemmy", 1.0 },
  { "Kemo", 1.0 },
  { "Kemon", 1.0 },
  { "Kemond", 1.0 },
  { "Kemondre", 1.0 },
  { "Kemone", 1.0 },
  { "Kemoni", 0.4455645 },
  { "Kemonie", 0.1440678 },
  { "Kemonta", 1.0 },
  { "Kemontae", 1.0 },
  { "Kemontay", 1.0 },
  { "Kemonte", 1.0 },
  { "Kemontez", 1.0 },
  { "Kemora", 0.0 },
  { "Kemorah", 0.0 },
  { "Kemori", 0.64285713 },
  { "Kemoria", 0.0 },
  { "Kemp", 1.0 },
  { "Kemper", 0.9070081 },
  { "Kempton", 1.0 },
  { "Kemran", 1.0 },
  { "Kemry", 0.0 },
  { "Kemuel", 1.0 },
  { "Kemya", 0.0 },
  { "Kemyah", 0.0 },
  { "Kemyatta", 0.0 },
  { "Kemyia", 0.0 },
  { "Kemyiah", 0.0 },
  { "Kemyra", 0.0 },
  { "Ken", 0.99923325 },
  { "Kena", 0.009163803 },
  { "Kenaan", 1.0 },
  { "Kenadee", 0.0 },
  { "Kenadey", 0.0 },
  { "Kenadi", 0.0 },
  { "Kenadie", 0.0 },
  { "Kenady", 0.0 },
  { "Kenae", 0.0 },
  { "Kenah", 0.0 },
  { "Kenai", 0.9461427 },
  { "Kenaja", 0.0 },
  { "Kenajah", 0.0 },
  { "Kenalee", 0.0 },
  { "Kenan", 0.9909747 },
  { "Kenaniah", 1.0 },
  { "Kenara", 0.0 },
  { "Kenard", 1.0 },
  { "Kenari", 0.0 },
  { "Kenaria", 0.0 },
  { "Kenasha", 0.0 },
  { "Kenasia", 0.0 },
  { "Kenay", 1.0 },
  { "Kenaya", 0.0 },
  { "Kenayah", 0.0 },
  { "Kenaz", 1.0 },
  { "Kenberly", 0.0 },
  { "Kency", 0.18518518 },
  { "Kenda", 0.0 },
  { "Kendahl", 0.07248521 },
  { "Kendal", 0.34137636 },
  { "Kendale", 0.94210523 },
  { "Kendall", 0.37216187 },
  { "Kendalle", 0.0 },
  { "Kendallyn", 0.0 },
  { "Kendallynn", 0.0 },
  { "Kendalyn", 0.0 },
  { "Kendalynn", 0.0 },
  { "Kendan", 1.0 },
  { "Kendar", 0.0 },
  { "Kendarion", 1.0 },
  { "Kendarious", 1.0 },
  { "Kendarius", 1.0 },
  { "Kendarrius", 1.0 },
  { "Kendasia", 0.0 },
  { "Kendayl", 0.0 },
  { "Kendee", 0.0 },
  { "Kendel", 0.49358973 },
  { "Kendell", 0.7112911 },
  { "Kendelle", 0.025510205 },
  { "Kendelyn", 0.0 },
  { "Kenden", 1.0 },
  { "Kender", 1.0 },
  { "Kendera", 0.0 },
  { "Kenderek", 1.0 },
  { "Kenderick", 1.0 },
  { "Kenderius", 1.0 },
  { "Kenderrick", 1.0 },
  { "Kendi", 0.0 },
  { "Kendia", 0.0 },
  { "Kendin", 1.0 },
  { "Kendis", 0.23076923 },
  { "Kendl", 0.0 },
  { "Kendle", 0.35271317 },
  { "Kendley", 0.0 },
  { "Kendly", 1.0 },
  { "Kendo", 1.0 },
  { "Kendol", 1.0 },
  { "Kendon", 1.0 },
  { "Kendra", 0.0037000128 },
  { "Kendrae", 1.0 },
  { "Kendrah", 0.0 },
  { "Kendral", 0.0 },
  { "Kendrall", 1.0 },
  { "Kendralyn", 0.0 },
  { "Kendraya", 0.0 },
  { "Kendre", 0.79552716 },
  { "Kendrea", 0.0 },
  { "Kendred", 1.0 },
  { "Kendree", 0.0 },
  { "Kendrel", 1.0 },
  { "Kendrell", 0.9785969 },
  { "Kendrew", 1.0 },
  { "Kendria", 0.0 },
  { "Kendriana", 0.0 },
  { "Kendrianna", 0.0 },
  { "Kendric", 1.0 },
  { "Kendrica", 0.0 },
  { "Kendrich", 1.0 },
  { "Kendrick", 0.9878661 },
  { "Kendricka", 0.0 },
  { "Kendricks", 1.0 },
  { "Kendrie", 0.0 },
  { "Kendriel", 1.0 },
  { "Kendrik", 1.0 },
  { "Kendrika", 0.0 },
  { "Kendrix", 0.9411765 },
  { "Kendron", 1.0 },
  { "Kendry", 0.8325991 },
  { "Kendrya", 0.0 },
  { "Kendryck", 1.0 },
  { "Kendryk", 1.0 },
  { "Kendrys", 1.0 },
  { "Kendu", 1.0 },
  { "Kendy", 0.034188036 },
  { "Kendyl", 0.023471145 },
  { "Kendyle", 0.0 },
  { "Kendyll", 0.014242878 },
  { "Kendylle", 0.0 },
  { "Kendyn", 1.0 },
  { "Kene", 0.8484849 },
  { "Keneasha", 0.0 },
  { "Kenechi", 1.0 },
  { "Kenechukwu", 1.0 },
  { "Kenecia", 0.0 },
  { "Kenedee", 0.0 },
  { "Kenedi", 0.0 },
  { "Kenedie", 0.0 },
  { "Kenedra", 0.0 },
  { "Kenedy", 0.034246575 },
  { "Keneen", 1.0 },
  { "Keneesha", 0.0 },
  { "Keneisha", 0.0 },
  { "Keneka", 0.0 },
  { "Kenenna", 1.0 },
  { "Kenenth", 1.0 },
  { "Kener", 1.0 },
  { "Kenesha", 0.0 },
  { "Keneshia", 0.0 },
  { "Kenesia", 0.0 },
  { "Kenessa", 0.0 },
  { "Kenet", 1.0 },
  { "Keneta", 0.0 },
  { "Keneth", 1.0 },
  { "Kenetha", 0.0 },
  { "Kenethia", 0.0 },
  { "Kenetra", 0.0 },
  { "Kenetta", 0.0 },
  { "Kenette", 0.0 },
  { "Keng", 1.0 },
  { "Kengi", 1.0 },
  { "Kengo", 1.0 },
  { "Kengston", 1.0 },
  { "Keni", 0.27561837 },
  { "Kenia", 0.0015643332 },
  { "Keniah", 0.0 },
  { "Kenichi", 1.0 },
  { "Kenichiro", 1.0 },
  { "Kenicia", 0.0 },
  { "Kenidee", 0.0 },
  { "Kenidi", 0.0 },
  { "Kenidie", 0.0 },
  { "Kenidy", 0.0 },
  { "Keniel", 1.0 },
  { "Keniesha", 0.0 },
  { "Kenija", 0.0 },
  { "Kenijah", 0.0 },
  { "Kenika", 0.0 },
  { "Kenil", 1.0 },
  { "Kenin", 1.0 },
  { "Keniqua", 0.0 },
  { "Kenise", 0.0 },
  { "Kenisha", 0.0012703252 },
  { "Kenishia", 0.0 },
  { "Kenita", 0.0 },
  { "Kenith", 1.0 },
  { "Kenitha", 0.0 },
  { "Kenithia", 0.0 },
  { "Kenitra", 0.0 },
  { "Keniya", 0.0 },
  { "Keniyah", 0.0 },
  { "Keniyha", 0.0 },
  { "Kenja", 0.0 },
  { "Kenjanae", 0.0 },
  { "Kenji", 0.9906489 },
  { "Kenjiro", 1.0 },
  { "Kenjuan", 1.0 },
  { "Kenlea", 0.0 },
  { "Kenlee", 0.032608695 },
  { "Kenlei", 0.0 },
  { "Kenleigh", 0.0 },
  { "Kenley", 0.15040839 },
  { "Kenli", 0.0 },
  { "Kenlie", 0.0 },
  { "Kenlin", 1.0 },
  { "Kenly", 0.15051547 },
  { "Kenlyn", 0.072115384 },
  { "Kenlynn", 0.0 },
  { "Kenn", 1.0 },
  { "Kenna", 0.019644806 },
  { "Kennadee", 0.0 },
  { "Kennadi", 0.0 },
  { "Kennadie", 0.0 },
  { "Kennady", 0.0 },
  { "Kennae", 0.0 },
  { "Kennah", 0.0 },
  { "Kennan", 0.95367134 },
  { "Kennard", 1.0 },
  { "Kennasia", 0.0 },
  { "Kennasyn", 0.0 },
  { "Kennay", 0.0 },
  { "Kennaya", 0.0 },
  { "Kenndra", 0.0 },
  { "Kenndrick", 1.0 },
  { "Kenndy", 0.4117647 },
  { "Kenne", 1.0 },
  { "Kennecia", 0.0 },
  { "Kennede", 0.0 },
  { "Kennedee", 0.0 },
  { "Kennedey", 0.0 },
  { "Kennedi", 0.0014329385 },
  { "Kennedie", 0.0 },
  { "Kennedii", 0.0 },
  { "Kennedy", 0.0923538 },
  { "Kenneisha", 0.0 },
  { "Kenneith", 1.0 },
  { "Kennell", 1.0 },
  { "Kennen", 1.0 },
  { "Kenner", 1.0 },
  { "Kennesha", 0.0 },
  { "Kenneshia", 0.0 },
  { "Kennet", 1.0 },
  { "Kenneth", 0.99581957 },
  { "Kennetha", 0.0 },
  { "Kennethia", 0.0 },
  { "Kennetra", 0.0 },
  { "Kennett", 1.0 },
  { "Kennetta", 0.0 },
  { "Kennette", 0.0 },
  { "Kenney", 0.9979936 },
  { "Kenni", 0.03952569 },
  { "Kennia", 0.0 },
  { "Kenniah", 0.0 },
  { "Kennice", 0.0 },
  { "Kennidee", 0.0 },
  { "Kennidi", 0.0 },
  { "Kennidy", 0.0 },
  { "Kennie", 0.82156366 },
  { "Kenniel", 1.0 },
  { "Kennieth", 1.0 },
  { "Kennika", 0.0 },
  { "Kennis", 0.8863636 },
  { "Kennise", 0.0 },
  { "Kennisha", 0.0 },
  { "Kennison", 0.3181818 },
  { "Kennita", 0.0 },
  { "Kennith", 0.99952215 },
  { "Kennitha", 0.0 },
  { "Kenniya", 0.0 },
  { "Kenniyah", 0.0 },
  { "Kennlee", 0.0 },
  { "Kennley", 0.0 },
  { "Kennon", 0.9851013 },
  { "Kennth", 1.0 },
  { "Kenny", 0.9900567 },
  { "Kennya", 0.0 },
  { "Kennyah", 0.0 },
  { "Kennyatta", 0.33333334 },
  { "Kennydi", 0.0 },
  { "Kennyel", 1.0 },
  { "Kennyetta", 0.0 },
  { "Kennysha", 0.0 },
  { "Kennyth", 1.0 },
  { "Keno", 1.0 },
  { "Kenon", 1.0 },
  { "Kenora", 0.0 },
  { "Kenosha", 0.0 },
  { "Kenra", 0.0 },
  { "Kenric", 1.0 },
  { "Kenrich", 1.0 },
  { "Kenrick", 1.0 },
  { "Kenrie", 1.0 },
  { "Kenron", 1.0 },
  { "Kenroy", 1.0 },
  { "Kenry", 1.0 },
  { "Kensei", 1.0 },
  { "Kensen", 1.0 },
  { "Kenseth", 1.0 },
  { "Kensey", 0.005820722 },
  { "Kensha", 0.0 },
  { "Kenshaun", 1.0 },
  { "Kenshawn", 1.0 },
  { "Kenshayla", 0.0 },
  { "Kenshia", 0.0 },
  { "Kenshin", 1.0 },
  { "Kenshiro", 1.0 },
  { "Kensi", 0.0 },
  { "Kensie", 0.0 },
  { "Kensington", 0.04418727 },
  { "Kensingtyn", 0.0 },
  { "Kenslea", 0.0 },
  { "Kenslee", 0.0 },
  { "Kenslei", 0.0 },
  { "Kensleigh", 0.0 },
  { "Kensley", 0.048643593 },
  { "Kensli", 0.0 },
  { "Kenslie", 0.0 },
  { "Kensly", 0.05263158 },
  { "Kenso", 1.0 },
  { "Kenson", 1.0 },
  { "Kenston", 1.0 },
  { "Kensuke", 1.0 },
  { "Kensy", 0.0 },
  { "Kensyn", 1.0 },
  { "Kent", 0.9974999 },
  { "Kenta", 0.9924357 },
  { "Kentae", 1.0 },
  { "Kentara", 0.0 },
  { "Kentarious", 1.0 },
  { "Kentarius", 1.0 },
  { "Kentaro", 1.0 },
  { "Kentasia", 0.0 },
  { "Kentavia", 0.0 },
  { "Kentavion", 1.0 },
  { "Kentavious", 1.0 },
  { "Kentavis", 1.0 },
  { "Kentavius", 1.0 },
  { "Kentay", 1.0 },
  { "Kente", 1.0 },
  { "Kenten", 1.0 },
  { "Kenteria", 0.0 },
  { "Kenterious", 1.0 },
  { "Kenterrius", 1.0 },
  { "Kentez", 1.0 },
  { "Kentin", 1.0 },
  { "Kentlee", 0.24444444 },
  { "Kentleigh", 0.0 },
  { "Kentley", 0.43055555 },
  { "Kently", 0.0 },
  { "Kento", 1.0 },
  { "Kenton", 1.0 },
  { "Kentoria", 0.0 },
  { "Kentra", 0.0 },
  { "Kentrail", 1.0 },
  { "Kentral", 1.0 },
  { "Kentrall", 1.0 },
  { "Kentravion", 1.0 },
  { "Kentravious", 1.0 },
  { "Kentravius", 1.0 },
  { "Kentre", 1.0 },
  { "Kentreal", 1.0 },
  { "Kentrel", 1.0 },
  { "Kentrell", 0.97985345 },
  { "Kentrelle", 1.0 },
  { "Kentrevious", 1.0 },
  { "Kentrez", 1.0 },
  { "Kentrice", 0.0 },
  { "Kentrina", 0.0 },
  { "Kentron", 1.0 },
  { "Kentwan", 1.0 },
  { "Kentyn", 1.0 },
  { "Kenuel", 1.0 },
  { "Kenward", 1.0 },
  { "Kenwood", 1.0 },
  { "Keny", 0.97596157 },
  { "Kenya", 0.08116244 },
  { "Kenyada", 0.20534459 },
  { "Kenyae", 0.75 },
  { "Kenyah", 0.0 },
  { "Kenyan", 0.9882491 },
  { "Kenyana", 0.0 },
  { "Kenyanna", 0.0 },
  { "Kenyara", 0.0 },
  { "Kenyarda", 0.0 },
  { "Kenyari", 0.47368422 },
  { "Kenyata", 0.26851025 },
  { "Kenyatta", 0.4241187 },
  { "Kenyatte", 0.66847825 },
  { "Kenyce", 0.0 },
  { "Kenye", 1.0 },
  { "Kenyea", 1.0 },
  { "Kenyel", 0.0 },
  { "Kenyell", 0.2413793 },
  { "Kenyen", 1.0 },
  { "Kenyeta", 0.0 },
  { "Kenyetta", 0.04470835 },
  { "Kenyette", 0.0 },
  { "Kenyi", 1.0 },
  { "Kenyia", 0.0 },
  { "Kenyiah", 0.0 },
  { "Kenyin", 1.0 },
  { "Kenyjah", 0.0 },
  { "Kenyla", 0.0 },
  { "Kenylah", 0.0 },
  { "Kenyon", 0.97579914 },
  { "Kenyona", 0.0 },
  { "Kenyonna", 0.0 },
  { "Kenyota", 0.16129032 },
  { "Kenyotta", 0.13333334 },
  { "Kenysha", 0.0 },
  { "Kenyun", 1.0 },
  { "Kenz", 0.0 },
  { "Kenza", 0.0 },
  { "Kenze", 0.0 },
  { "Kenzee", 0.0 },
  { "Kenzel", 1.0 },
  { "Kenzell", 1.0 },
  { "Kenzey", 0.0 },
  { "Kenzi", 0.008609557 },
  { "Kenzia", 0.0 },
  { "Kenzie", 0.026522843 },
  { "Kenzington", 0.0 },
  { "Kenzingtyn", 0.0 },
  { "Kenzlea", 0.0 },
  { "Kenzlee", 0.0 },
  { "Kenzlei", 0.0 },
  { "Kenzleigh", 0.0 },
  { "Kenzley", 0.0 },
  { "Kenzli", 0.0 },
  { "Kenzlie", 0.0 },
  { "Kenzly", 0.0 },
  { "Kenzlyn", 0.0 },
  { "Kenzlynn", 0.0 },
  { "Kenzo", 1.0 },
  { "Kenzy", 0.045649074 },
  { "Kenzye", 0.0 },
  { "Keo", 1.0 },
  { "Keoisha", 0.0 },
  { "Keoka", 0.0 },
  { "Keoki", 1.0 },
  { "Keola", 0.9429098 },
  { "Keolani", 0.0 },
  { "Keomi", 0.0 },
  { "Keon", 0.9808661 },
  { "Keona", 0.020766772 },
  { "Keonda", 0.0 },
  { "Keondra", 0.015192136 },
  { "Keondrae", 1.0 },
  { "Keondray", 1.0 },
  { "Keondre", 1.0 },
  { "Keondrea", 0.0 },
  { "Keondria", 0.0 },
  { "Keondric", 1.0 },
  { "Keondrick", 1.0 },
  { "Keone", 1.0 },
  { "Keoni", 0.9053747 },
  { "Keonia", 0.0 },
  { "Keonie", 0.0 },
  { "Keonna", 0.0 },
  { "Keonne", 1.0 },
  { "Keonni", 0.0 },
  { "Keonnie", 0.0 },
  { "Keonta", 0.8655462 },
  { "Keontae", 0.9735553 },
  { "Keontai", 1.0 },
  { "Keontay", 0.9848485 },
  { "Keontaye", 1.0 },
  { "Keonte", 0.962069 },
  { "Keontez", 1.0 },
  { "Keontre", 1.0 },
  { "Keonya", 0.0 },
  { "Keora", 0.0 },
  { "Keori", 0.0 },
  { "Keoria", 0.0 },
  { "Keorra", 0.0 },
  { "Keosha", 0.0 },
  { "Keoshia", 0.0 },
  { "Kepa", 1.0 },
  { "Kepler", 1.0 },
  { "Ker", 0.6666667 },
  { "Kera", 0.0 },
  { "Keragan", 0.0 },
  { "Kerah", 0.0 },
  { "Kerala", 0.0 },
  { "Keran", 0.16666667 },
  { "Kerani", 0.0 },
  { "Keraun", 1.0 },
  { "Kerbi", 0.0 },
  { "Kerbie", 0.0 },
  { "Kerby", 0.875383 },
  { "Kerea", 0.0 },
  { "Keree", 0.0 },
  { "Kereem", 1.0 },
  { "Kereen", 0.0 },
  { "Kerek", 1.0 },
  { "Kerel", 1.0 },
  { "Kerem", 1.0 },
  { "Keren", 0.0 },
  { "Kerena", 0.0 },
  { "Kerensa", 0.0 },
  { "Keresa", 0.0 },
  { "Keri", 0.010052746 },
  { "Keria", 0.0 },
  { "Keriah", 0.0 },
  { "Kerian", 0.4 },
  { "Keriana", 0.0 },
  { "Keriann", 0.0 },
  { "Kerianna", 0.0 },
  { "Kerianne", 0.0 },
  { "Keric", 1.0 },
  { "Kerica", 0.0 },
  { "Kerick", 1.0 },
  { "Kerie", 0.0 },
  { "Keriel", 1.0 },
  { "Kerigan", 0.0 },
  { "Kerilee", 0.0 },
  { "Kerilyn", 0.0 },
  { "Kerilynn", 0.0 },
  { "Kerim", 1.0 },
  { "Kerin", 0.16050956 },
  { "Kerina", 0.0 },
  { "Kerington", 0.0 },
  { "Kerion", 1.0 },
  { "Keriona", 0.0 },
  { "Kerionna", 0.0 },
  { "Keris", 0.0 },
  { "Kerisa", 0.0 },
  { "Kerisha", 0.0 },
  { "Kerissa", 0.0 },
  { "Keristen", 0.0 },
  { "Kerith", 0.0 },
  { "Keriyah", 0.0 },
  { "Kerk", 1.0 },
  { "Kerl", 0.0 },
  { "Kerlin", 0.0 },
  { "Kerline", 0.0 },
  { "Kerly", 0.0 },
  { "Kerlyht", 0.0 },
  { "Kerlyn", 0.0 },
  { "Kerma", 0.0 },
  { "Kerman", 1.0 },
  { "Kermet", 1.0 },
  { "Kermina", 0.0 },
  { "Kermit", 0.9983581 },
  { "Kermith", 1.0 },
  { "Kermitt", 1.0 },
  { "Kermon", 1.0 },
  { "Kern", 1.0 },
  { "Kernell", 1.0 },
  { "Kerney", 1.0 },
  { "Kernie", 1.0 },
  { "Kernis", 1.0 },
  { "Kernisha", 0.0 },
  { "Kerolos", 1.0 },
  { "Keron", 0.92164946 },
  { "Kerr", 1.0 },
  { "Kerra", 0.0 },
  { "Kerrah", 0.0 },
  { "Kerre", 0.0 },
  { "Kerrell", 1.0 },
  { "Kerren", 0.0 },
  { "Kerrey", 0.0 },
  { "Kerri", 0.009350315 },
  { "Kerria", 0.0 },
  { "Kerrian", 0.0 },
  { "Kerriana", 0.0 },
  { "Kerriann", 0.0 },
  { "Kerrianna", 0.0 },
  { "Kerrianne", 0.0 },
  { "Kerric", 1.0 },
  { "Kerrick", 1.0 },
  { "Kerrie", 0.011988472 },
  { "Kerrigan", 0.037320573 },
  { "Kerrilee", 0.0 },
  { "Kerrilyn", 0.0 },
  { "Kerrilynn", 0.0 },
  { "Kerrin", 0.009871668 },
  { "Kerrington", 0.19399539 },
  { "Kerrion", 0.88 },
  { "Kerris", 0.0 },
  { "Kerrisa", 0.0 },
  { "Kerrissa", 0.0 },
  { "Kerron", 1.0 },
  { "Kerry", 0.50525284 },
  { "Kerryann", 0.0 },
  { "Kerryanne", 0.0 },
  { "Kerrye", 0.0 },
  { "Kerryn", 0.0 },
  { "Kersey", 0.26760563 },
  { "Kersha", 0.0 },
  { "Kershaw", 1.0 },
  { "Kerstan", 0.0 },
  { "Kerstein", 0.0 },
  { "Kersten", 0.0 },
  { "Kersti", 0.0 },
  { "Kerstie", 0.0 },
  { "Kerstin", 0.0 },
  { "Kerstine", 0.0 },
  { "Kerston", 0.0 },
  { "Kersty", 0.0 },
  { "Kerstyn", 0.0 },
  { "Kert", 1.0 },
  { "Kertina", 0.0 },
  { "Kertis", 1.0 },
  { "Kertrina", 0.0 },
  { "Kerven", 1.0 },
  { "Kervens", 1.0 },
  { "Kervin", 1.0 },
  { "Kervins", 1.0 },
  { "Kerwin", 1.0 },
  { "Kerwyn", 1.0 },
  { "Kery", 0.44117647 },
  { "Keryl", 0.0 },
  { "Keryn", 0.0 },
  { "Kerynn", 0.0 },
  { "Kes", 0.0 },
  { "Kesa", 0.0 },
  { "Kesan", 1.0 },
  { "Kesar", 0.0 },
  { "Kesaun", 1.0 },
  { "Kescha", 0.0 },
  { "Kesean", 1.0 },
  { "Kesha", 0.00095987716 },
  { "Keshae", 0.0 },
  { "Keshan", 1.0 },
  { "Keshana", 0.0 },
  { "Keshanda", 0.0 },
  { "Keshandra", 0.0 },
  { "Keshanna", 0.0 },
  { "Keshante", 0.0 },
  { "Keshanti", 0.0 },
  { "Keshara", 0.0 },
  { "Kesharia", 0.0 },
  { "Keshaun", 0.9768917 },
  { "Keshauna", 0.0 },
  { "Keshaunna", 0.0 },
  { "Keshav", 1.0 },
  { "Keshava", 1.0 },
  { "Keshawn", 0.9414183 },
  { "Keshawna", 0.0 },
  { "Keshawnda", 0.0 },
  { "Keshawndra", 0.0 },
  { "Keshawnna", 0.0 },
  { "Keshay", 0.0 },
  { "Keshayla", 0.0 },
  { "Keshea", 0.0 },
  { "Keshera", 0.0 },
  { "Keshi", 0.0 },
  { "Keshia", 0.0011137924 },
  { "Keshira", 0.0 },
  { "Keshon", 0.9594252 },
  { "Keshona", 0.0 },
  { "Keshonda", 0.0 },
  { "Keshondra", 0.0 },
  { "Keshone", 1.0 },
  { "Keshonna", 0.0 },
  { "Keshun", 0.9880952 },
  { "Keshuna", 0.0 },
  { "Keshundra", 0.0 },
  { "Keshunna", 0.0 },
  { "Keshyra", 0.0 },
  { "Kesi", 0.0 },
  { "Kesia", 0.0 },
  { "Kesiah", 0.0 },
  { "Kesiha", 0.0 },
  { "Keslee", 0.0 },
  { "Kesleigh", 0.0 },
  { "Kesler", 1.0 },
  { "Kesley", 0.09733124 },
  { "Keslie", 0.0 },
  { "Kesly", 0.0 },
  { "Keslyn", 0.0 },
  { "Keslynn", 0.0 },
  { "Keson", 1.0 },
  { "Kess", 0.0 },
  { "Kessa", 0.0 },
  { "Kessandra", 0.0 },
  { "Kessia", 0.0 },
  { "Kessiah", 0.0 },
  { "Kessie", 0.0 },
  { "Kessler", 0.8235294 },
  { "Kessley", 0.0 },
  { "Kesslyn", 0.0 },
  { "Kesten", 1.0 },
  { "Kester", 1.0 },
  { "Kestin", 1.0 },
  { "Keston", 1.0 },
  { "Kestra", 0.0 },
  { "Kestrel", 0.02631579 },
  { "Kestutis", 1.0 },
  { "Kestyn", 1.0 },
  { "Kesuan", 1.0 },
  { "Keta", 0.0 },
  { "Ketan", 1.0 },
  { "Ketara", 0.0 },
  { "Ketch", 1.0 },
  { "Ketcher", 1.0 },
  { "Keteria", 0.0 },
  { "Ketha", 0.0 },
  { "Kethan", 1.0 },
  { "Ketherine", 0.0 },
  { "Kethia", 0.0 },
  { "Kethry", 0.0 },
  { "Ketia", 0.0 },
  { "Ketih", 1.0 },
  { "Ketina", 0.0 },
  { "Ketisha", 0.0 },
  { "Keton", 1.0 },
  { "Ketra", 0.0 },
  { "Ketrell", 1.0 },
  { "Ketric", 1.0 },
  { "Ketrick", 1.0 },
  { "Ketrina", 0.0 },
  { "Ketron", 1.0 },
  { "Ketsia", 0.0 },
  { "Ketsy", 0.0 },
  { "Ketta", 0.0 },
  { "Ketti", 0.0 },
  { "Ketty", 0.0 },
  { "Ketura", 0.0 },
  { "Keturah", 0.0 },
  { "Ketzalli", 0.0 },
  { "Ketzaly", 0.0 },
  { "Ketzia", 0.0 },
  { "Ketziah", 0.0 },
  { "Keuna", 0.0 },
  { "Keundra", 0.0 },
  { "Keundre", 1.0 },
  { "Keunna", 0.0 },
  { "Keunta", 1.0 },
  { "Keunte", 1.0 },
  { "Keva", 0.0 },
  { "Kevaeh", 0.0 },
  { "Keval", 1.0 },
  { "Kevan", 0.98605865 },
  { "Kevana", 0.0 },
  { "Kevanna", 0.0 },
  { "Kevante", 1.0 },
  { "Kevari", 1.0 },
  { "Kevaris", 1.0 },
  { "Kevaughn", 1.0 },
  { "Kevaun", 1.0 },
  { "Keveen", 1.0 },
  { "Kevelyn", 0.0 },
  { "Keven", 0.98500925 },
  { "Keveon", 1.0 },
  { "Kever", 1.0 },
  { "Kevi", 0.47619048 },
  { "Kevia", 0.0 },
  { "Kevian", 1.0 },
  { "Keviana", 0.0 },
  { "Kevianna", 0.0 },
  { "Kevin", 0.995453 },
  { "Kevina", 0.0 },
  { "Kevinanthony", 1.0 },
  { "Kevinchristophe", 1.0 },
  { "Kevine", 1.0 },
  { "Kevinisha", 0.0 },
  { "Kevinjohn", 1.0 },
  { "Kevinjoseph", 1.0 },
  { "Kevinkhang", 1.0 },
  { "Kevinmatthew", 1.0 },
  { "Kevinmichael", 1.0 },
  { "Kevinn", 1.0 },
  { "Kevins", 1.0 },
  { "Kevion", 0.9947589 },
  { "Keviona", 0.0 },
  { "Kevionna", 0.0 },
  { "Kevious", 1.0 },
  { "Kevis", 1.0 },
  { "Kevisha", 0.0 },
  { "Keviyon", 1.0 },
  { "Kevlin", 1.0 },
  { "Kevn", 1.0 },
  { "Kevon", 0.9888756 },
  { "Kevona", 0.0 },
  { "Kevonda", 0.0 },
  { "Kevondra", 0.0 },
  { "Kevondre", 1.0 },
  { "Kevondrick", 1.0 },
  { "Kevone", 1.0 },
  { "Kevonn", 1.0 },
  { "Kevonna", 0.0 },
  { "Kevonne", 0.877551 },
  { "Kevonta", 1.0 },
  { "Kevontae", 1.0 },
  { "Kevontay", 1.0 },
  { "Kevonte", 1.0 },
  { "Kevontre", 1.0 },
  { "Kevork", 1.0 },
  { "Kevrick", 1.0 },
  { "Kevron", 1.0 },
  { "Kevyn", 0.8927134 },
  { "Kevyon", 1.0 },
  { "Kewan", 1.0 },
  { "Kewana", 0.0 },
  { "Kewanda", 0.0 },
  { "Kewania", 0.0 },
  { "Kewanna", 0.0 },
  { "Kewaun", 1.0 },
  { "Kewin", 1.0 },
  { "Kewon", 1.0 },
  { "Kewuan", 1.0 },
  { "Kexin", 0.0 },
  { "Key", 0.9358974 },
  { "Keya", 0.0 },
  { "Keyaan", 1.0 },
  { "Keyah", 0.0 },
  { "Keyahna", 0.0 },
  { "Keyair", 1.0 },
  { "Keyaira", 0.0 },
  { "Keyairah", 0.0 },
  { "Keyairra", 0.0 },
  { "Keyajah", 0.0 },
  { "Keyala", 0.0 },
  { "Keyan", 0.9934272 },
  { "Keyana", 0.0 },
  { "Keyanah", 0.0 },
  { "Keyanda", 0.0 },
  { "Keyandra", 0.0 },
  { "Keyandre", 1.0 },
  { "Keyandrea", 0.0 },
  { "Keyani", 0.0 },
  { "Keyania", 0.0 },
  { "Keyanna", 0.0 },
  { "Keyannah", 0.0 },
  { "Keyanni", 0.0 },
  { "Keyanta", 0.0 },
  { "Keyante", 0.9607843 },
  { "Keyara", 0.0 },
  { "Keyarah", 0.0 },
  { "Keyari", 0.03816794 },
  { "Keyaria", 0.0 },
  { "Keyarie", 0.0 },
  { "Keyarra", 0.0 },
  { "Keyasha", 0.0 },
  { "Keyashia", 0.0 },
  { "Keyasia", 0.0 },
  { "Keyatta", 0.0 },
  { "Keyaun", 1.0 },
  { "Keyauna", 0.0 },
  { "Keyawna", 0.0 },
  { "Keydan", 1.0 },
  { "Keyden", 1.0 },
  { "Keydi", 0.0 },
  { "Keydon", 1.0 },
  { "Keydra", 0.0 },
  { "Keydren", 1.0 },
  { "Keydy", 0.0 },
  { "Keye", 1.0 },
  { "Keyen", 1.0 },
  { "Keyera", 0.0 },
  { "Keyerah", 0.0 },
  { "Keyeria", 0.0 },
  { "Keyerra", 0.0 },
  { "Keyetta", 0.0 },
  { "Keygan", 0.97002727 },
  { "Keyhla", 0.0 },
  { "Keyia", 0.0 },
  { "Keyiana", 0.0 },
  { "Keyilah", 0.0 },
  { "Keyin", 1.0 },
  { "Keyion", 1.0 },
  { "Keyiona", 0.0 },
  { "Keyira", 0.0 },
  { "Keyisha", 0.0 },
  { "Keyjuan", 1.0 },
  { "Keyla", 0.0 },
  { "Keylah", 0.0 },
  { "Keylan", 0.9477521 },
  { "Keyland", 1.0 },
  { "Keylani", 0.0 },
  { "Keylanie", 0.0 },
  { "Keyle", 0.0 },
  { "Keylea", 0.0 },
  { "Keylee", 0.0 },
  { "Keyleen", 0.0 },
  { "Keylei", 0.0 },
  { "Keyleigh", 0.0 },
  { "Keylen", 0.58313817 },
  { "Keyler", 1.0 },
  { "Keyley", 0.0 },
  { "Keyli", 0.0 },
  { "Keyliana", 0.0 },
  { "Keylianis", 0.0 },
  { "Keylie", 0.0 },
  { "Keylin", 0.26717559 },
  { "Keylli", 0.0 },
  { "Keylly", 0.0 },
  { "Keylon", 1.0 },
  { "Keylor", 1.0 },
  { "Keyly", 0.0 },
  { "Keylyn", 0.0 },
  { "Keymani", 0.40952381 },
  { "Keymar", 1.0 },
  { "Keymara", 0.0 },
  { "Keymari", 0.79375 },
  { "Keymarion", 1.0 },
  { "Keymaya", 0.0 },
  { "Keymi", 0.0 },
  { "Keymia", 0.0 },
  { "Keymiah", 0.0 },
  { "Keymiyah", 0.0 },
  { "Keymon", 1.0 },
  { "Keymond", 1.0 },
  { "Keymoni", 0.32456142 },
  { "Keymonie", 1.0 },
  { "Keymonte", 1.0 },
  { "Keymora", 0.0 },
  { "Keymya", 0.0 },
  { "Keyna", 0.0 },
  { "Keynan", 1.0 },
  { "Keynen", 1.0 },
  { "Keyner", 1.0 },
  { "Keynia", 0.0 },
  { "Keyniah", 0.0 },
  { "Keynon", 1.0 },
  { "Keynu", 0.0 },
  { "Keyo", 1.0 },
  { "Keyoka", 0.0 },
  { "Keyon", 0.9809972 },
  { "Keyona", 0.0 },
  { "Keyonah", 0.0 },
  { "Keyonce", 0.0 },
  { "Keyonda", 0.0 },
  { "Keyondra", 0.0 },
  { "Keyondre", 1.0 },
  { "Keyondria", 0.0 },
  { "Keyone", 1.0 },
  { "Keyoni", 0.044247787 },
  { "Keyonia", 0.0 },
  { "Keyonie", 0.0 },
  { "Keyonis", 1.0 },
  { "Keyonn", 1.0 },
  { "Keyonna", 0.0 },
  { "Keyonne", 1.0 },
  { "Keyonni", 0.0 },
  { "Keyonnia", 0.0 },
  { "Keyonnie", 0.0 },
  { "Keyonta", 0.682243 },
  { "Keyontae", 0.9655172 },
  { "Keyontai", 1.0 },
  { "Keyontay", 1.0 },
  { "Keyonte", 0.95866144 },
  { "Keyora", 0.0 },
  { "Keyosha", 0.0 },
  { "Keyoshia", 0.0 },
  { "Keyra", 0.0 },
  { "Keyrah", 0.0 },
  { "Keyren", 0.0 },
  { "Keyri", 0.0 },
  { "Keyria", 0.0 },
  { "Keyrin", 0.0 },
  { "Keyron", 1.0 },
  { "Keyrra", 0.0 },
  { "Keyry", 0.0 },
  { "Keysa", 0.0 },
  { "Keysan", 1.0 },
  { "Keysean", 1.0 },
  { "Keyser", 1.0 },
  { "Keysha", 0.0 },
  { "Keyshana", 0.0 },
  { "Keyshanna", 0.0 },
  { "Keyshaun", 1.0 },
  { "Keyshauna", 0.0 },
  { "Keyshawn", 0.99360514 },
  { "Keyshawna", 0.0 },
  { "Keyshia", 0.0 },
  { "Keyshla", 0.0 },
  { "Keyshon", 1.0 },
  { "Keyshona", 0.0 },
  { "Keyshonda", 0.0 },
  { "Keyshone", 1.0 },
  { "Keyshun", 1.0 },
  { "Keysi", 0.0 },
  { "Keysia", 0.0 },
  { "Keyson", 1.0 },
  { "Keystal", 0.0 },
  { "Keystle", 0.0 },
  { "Keyston", 1.0 },
  { "Keyten", 1.0 },
  { "Keyth", 1.0 },
  { "Keyton", 1.0 },
  { "Keyuana", 0.0 },
  { "Keyuanna", 0.0 },
  { "Keyun", 1.0 },
  { "Keyuna", 0.0 },
  { "Keyundra", 0.0 },
  { "Keyunna", 0.0 },
  { "Keyunta", 0.6666667 },
  { "Keyuntae", 1.0 },
  { "Keyur", 1.0 },
  { "Keyva", 0.0 },
  { "Keyvan", 1.0 },
  { "Keyveon", 1.0 },
  { "Keyvin", 1.0 },
  { "Keyvion", 1.0 },
  { "Keyvon", 1.0 },
  { "Keyvonna", 0.0 },
  { "Keyvonte", 1.0 },
  { "Keywan", 1.0 },
  { "Keywana", 0.0 },
  { "Keywanda", 0.0 },
  { "Keywanna", 0.0 },
  { "Keywaun", 1.0 },
  { "Keywon", 1.0 },
  { "Kez", 1.0 },
  { "Keza", 0.0 },
  { "Kezaria", 0.0 },
  { "Kezia", 0.0 },
  { "Keziah", 0.025338424 },
  { "Kezion", 1.0 },
  { "Keziya", 0.0 },
  { "Keziyah", 0.0 },
  { "Kezzie", 0.0 },
  { "Kezzy", 0.0 },
  { "Kfir", 1.0 },
  { "Kha", 0.94623655 },
  { "Khaalid", 1.0 },
  { "Khaaliq", 1.0 },
  { "Khaalis", 0.875 },
  { "Khabir", 1.0 },
  { "Khachik", 1.0 },
  { "Khadafi", 1.0 },
  { "Khadajah", 0.0 },
  { "Khadar", 1.0 },
  { "Khadarius", 1.0 },
  { "Khade", 1.0 },
  { "Khadedra", 0.0 },
  { "Khadeeja", 0.0 },
  { "Khadeejah", 0.0 },
  { "Khadeem", 1.0 },
  { "Khadeidra", 0.0 },
  { "Khadeijah", 0.0 },
  { "Khadeisha", 0.0 },
  { "Khadeja", 0.0 },
  { "Khadejah", 0.0 },
  { "Khadejha", 0.0 },
  { "Khadejia", 0.0 },
  { "Khaden", 1.0 },
  { "Khadence", 0.0 },
  { "Khader", 1.0 },
  { "Khadesha", 0.0 },
  { "Khadeshia", 0.0 },
  { "Khadesia", 0.0 },
  { "Khadi", 0.0 },
  { "Khadidiatou", 0.0 },
  { "Khadidja", 0.0 },
  { "Khadidra", 0.0 },
  { "Khadija", 0.0 },
  { "Khadijah", 0.0011058452 },
  { "Khadijat", 0.0 },
  { "Khadijatou", 0.0 },
  { "Khadijha", 0.0 },
  { "Khadijia", 0.0 },
  { "Khadijiah", 0.0 },
  { "Khadim", 1.0 },
  { "Khadin", 1.0 },
  { "Khadir", 1.0 },
  { "Khadisah", 0.0 },
  { "Khadisha", 0.0 },
  { "Khadiyah", 0.0 },
  { "Khadjah", 0.0 },
  { "Khadjiah", 0.0 },
  { "Khadra", 0.0 },
  { "Khady", 0.0 },
  { "Khadyn", 1.0 },
  { "Khael", 1.0 },
  { "Khaelyn", 0.0 },
  { "Khaelynn", 0.0 },
  { "Khafre", 1.0 },
  { "Khahlil", 1.0 },
  { "Khai", 0.8442997 },
  { "Khaia", 0.0 },
  { "Khaiden", 1.0 },
  { "Khaidyn", 1.0 },
  { "Khail", 1.0 },
  { "Khaila", 0.0 },
  { "Khailah", 0.0 },
  { "Khailee", 0.0 },
  { "Khailen", 1.0 },
  { "Khailey", 0.0 },
  { "Khailil", 1.0 },
  { "Khailyn", 0.0 },
  { "Khailynn", 0.0 },
  { "Khaina", 0.0 },
  { "Khair", 1.0 },
  { "Khaira", 0.0 },
  { "Khairah", 0.0 },
  { "Khairee", 1.0 },
  { "Khairi", 0.98730963 },
  { "Khairo", 1.0 },
  { "Khairy", 1.0 },
  { "Khaiya", 0.0 },
  { "Khala", 0.0 },
  { "Khalan", 1.0 },
  { "Khalani", 0.072164945 },
  { "Khalaya", 0.0 },
  { "Khalayah", 0.0 },
  { "Khale", 1.0 },
  { "Khalea", 0.0 },
  { "Khaleah", 0.0 },
  { "Khaleal", 1.0 },
  { "Khaleb", 1.0 },
  { "Khaled", 1.0 },
  { "Khalee", 0.75728154 },
  { "Khaleed", 1.0 },
  { "Khaleedah", 0.0 },
  { "Khaleef", 1.0 },
  { "Khaleel", 1.0 },
  { "Khaleelah", 0.0 },
  { "Khaleem", 1.0 },
  { "Khaleeq", 1.0 },
  { "Khalees", 0.0 },
  { "Khaleesi", 0.0 },
  { "Khaleesia", 0.0 },
  { "Khaleesy", 0.0 },
  { "Khalei", 0.4117647 },
  { "Khaleia", 0.0 },
  { "Khaleigh", 0.0 },
  { "Khaleil", 1.0 },
  { "Khalel", 1.0 },
  { "Khalema", 0.0 },
  { "Khalen", 0.96732026 },
  { "Khaleo", 1.0 },
  { "Khalessi", 0.0 },
  { "Khaley", 0.0 },
  { "Khaleya", 0.0 },
  { "Khaleyah", 0.0 },
  { "Khalfani", 1.0 },
  { "Khali", 0.4240069 },
  { "Khalia", 0.0029498525 },
  { "Khaliah", 0.0 },
  { "Khalial", 1.0 },
  { "Khalib", 1.0 },
  { "Khalid", 1.0 },
  { "Khalida", 0.0 },
  { "Khalidah", 0.0 },
  { "Khalie", 0.0 },
  { "Khalief", 1.0 },
  { "Khaliek", 1.0 },
  { "Khaliel", 1.0 },
  { "Khalif", 1.0 },
  { "Khalifa", 1.0 },
  { "Khalifah", 1.0 },
  { "Khalii", 0.0 },
  { "Khalijah", 1.0 },
  { "Khalik", 1.0 },
  { "Khalika", 0.0 },
  { "Khalil", 0.9915689 },
  { "Khalila", 0.0 },
  { "Khalilah", 0.0 },
  { "Khaliliah", 0.0 },
  { "Khalill", 1.0 },
  { "Khalim", 1.0 },
  { "Khalimah", 0.0 },
  { "Khalin", 0.95620435 },
  { "Khalina", 0.0 },
  { "Khaliq", 1.0 },
  { "Khalique", 1.0 },
  { "Khalis", 0.1875 },
  { "Khalisa", 0.0 },
  { "Khalisah", 0.0 },
  { "Khalise", 0.0 },
  { "Khalisha", 0.0 },
  { "Khalisi", 0.0 },
  { "Khaliya", 0.0 },
  { "Khaliyah", 0.0 },
  { "Khaliyl", 1.0 },
  { "Khalli", 0.0 },
  { "Khallid", 1.0 },
  { "Khallie", 0.0 },
  { "Khallil", 1.0 },
  { "Khalon", 1.0 },
  { "Khaloni", 0.0 },
  { "Khalya", 0.0 },
  { "Khalyl", 1.0 },
  { "Khalyn", 0.104166664 },
  { "Khalynn", 0.0 },
  { "Kham", 0.877551 },
  { "Khamal", 1.0 },
  { "Khamani", 0.66004187 },
  { "Khamar", 1.0 },
  { "Khamara", 0.0 },
  { "Khamare", 1.0 },
  { "Khamari", 0.77657 },
  { "Khamaria", 0.0 },
  { "Khamarion", 1.0 },
  { "Khamaron", 1.0 },
  { "Khamauri", 0.9375 },
  { "Khamaya", 0.0 },
  { "Khambrel", 1.0 },
  { "Khamden", 1.0 },
  { "Khamel", 1.0 },
  { "Khameron", 1.0 },
  { "Khamia", 0.0 },
  { "Khamiah", 0.0 },
  { "Khamil", 0.0 },
  { "Khamila", 0.0 },
  { "Khamilah", 0.0 },
  { "Khamilla", 0.0 },
  { "Khamille", 0.0 },
  { "Khamir", 1.0 },
  { "Khamira", 0.0 },
  { "Khamisi", 1.0 },
  { "Khamiya", 0.0 },
  { "Khamiyah", 0.0 },
  { "Khamon", 1.0 },
  { "Khamoni", 0.6179775 },
  { "Khamora", 0.0 },
  { "Khamren", 1.0 },
  { "Khamron", 1.0 },
  { "Khamryn", 0.43157893 },
  { "Khamya", 0.0 },
  { "Khamyah", 0.0 },
  { "Khamyla", 0.0 },
  { "Khamyra", 0.0 },
  { "Khan", 1.0 },
  { "Khance", 1.0 },
  { "Khandi", 0.0 },
  { "Khandice", 0.0 },
  { "Khandis", 0.0 },
  { "Khandra", 0.0 },
  { "Khane", 1.0 },
  { "Khang", 1.0 },
  { "Khanh", 0.5814286 },
  { "Khani", 0.8503401 },
  { "Khania", 0.0 },
  { "Khaniya", 0.0 },
  { "Khaniyah", 0.0 },
  { "Khansa", 0.0 },
  { "Khanya", 0.0 },
  { "Khanye", 1.0 },
  { "Khaos", 1.0 },
  { "Khara", 0.0 },
  { "Kharan", 1.0 },
  { "Khare", 1.0 },
  { "Kharee", 1.0 },
  { "Khareem", 1.0 },
  { "Khari", 0.80131364 },
  { "Kharis", 0.028571429 },
  { "Kharisma", 0.0 },
  { "Kharissa", 0.0 },
  { "Kharizma", 0.0 },
  { "Kharla", 0.0 },
  { "Kharlee", 0.0 },
  { "Kharley", 0.0 },
  { "Kharli", 0.0 },
  { "Kharlie", 0.0 },
  { "Kharma", 0.0 },
  { "Kharmen", 0.0 },
  { "Kharmin", 0.0 },
  { "Kharmyn", 0.0 },
  { "Kharon", 1.0 },
  { "Kharson", 1.0 },
  { "Kharsyn", 0.0 },
  { "Kharter", 0.8181818 },
  { "Khary", 1.0 },
  { "Khase", 1.0 },
  { "Khaseem", 1.0 },
  { "Khash", 1.0 },
  { "Khasim", 1.0 },
  { "Khasir", 1.0 },
  { "Khason", 1.0 },
  { "Khassidy", 0.0 },
  { "Khatia", 0.0 },
  { "Khattab", 1.0 },
  { "Khawaja", 1.0 },
  { "Khawla", 0.0 },
  { "Khawlah", 0.0 },
  { "Khaya", 0.0 },
  { "Khayden", 0.95095366 },
  { "Khaydence", 0.0 },
  { "Khaydin", 1.0 },
  { "Khayir", 1.0 },
  { "Khayla", 0.0 },
  { "Khaylah", 0.0 },
  { "Khaylani", 0.0 },
  { "Khaylee", 0.0 },
  { "Khaylie", 0.0 },
  { "Khayman", 1.0 },
  { "Khayr", 1.0 },
  { "Khayree", 1.0 },
  { "Khayri", 1.0 },
  { "Khayriyyah", 0.0 },
  { "Khaysen", 1.0 },
  { "Khayyam", 1.0 },
  { "Khaza", 0.9512195 },
  { "Khazi", 1.0 },
  { "Khea", 0.0 },
  { "Khedra", 0.0 },
  { "Kheelan", 1.0 },
  { "Kheira", 0.0 },
  { "Khelan", 1.0 },
  { "Khelani", 0.0 },
  { "Khelen", 1.0 },
  { "Kheli", 0.0 },
  { "Khelsea", 0.0 },
  { "Khelsey", 0.0 },
  { "Khemani", 1.0 },
  { "Khemari", 1.0 },
  { "Khenadi", 0.0 },
  { "Khenan", 1.0 },
  { "Khendal", 0.0 },
  { "Khendra", 0.0 },
  { "Kheng", 1.0 },
  { "Khennedi", 0.0 },
  { "Khepri", 0.0 },
  { "Kheri", 0.0 },
  { "Kherington", 0.0 },
  { "Kherrington", 0.0 },
  { "Khevin", 1.0 },
  { "Kheyla", 0.0 },
  { "Khi", 0.9066667 },
  { "Khia", 0.0 },
  { "Khiabet", 0.0 },
  { "Khiala", 0.0 },
  { "Khian", 1.0 },
  { "Khiana", 0.0 },
  { "Khiandra", 0.0 },
  { "Khianna", 0.0 },
  { "Khiara", 0.0 },
  { "Khiari", 0.5 },
  { "Khiem", 1.0 },
  { "Khila", 0.0 },
  { "Khilan", 1.0 },
  { "Khilee", 0.0 },
  { "Khileigh", 0.0 },
  { "Khilen", 1.0 },
  { "Khiley", 0.0 },
  { "Khilyn", 0.46031746 },
  { "Khilynn", 0.0 },
  { "Khimani", 1.0 },
  { "Khimora", 0.0 },
  { "Khine", 1.0 },
  { "Khing", 1.0 },
  { "Khingston", 1.0 },
  { "Khira", 0.0 },
  { "Khiree", 1.0 },
  { "Khirey", 1.0 },
  { "Khiri", 1.0 },
  { "Khiron", 1.0 },
  { "Khirsten", 0.0 },
  { "Khiry", 0.99335235 },
  { "Khiryn", 1.0 },
  { "Khisha", 0.0 },
  { "Khiya", 0.0 },
  { "Khiyah", 0.0 },
  { "Khiyan", 1.0 },
  { "Khiyon", 1.0 },
  { "Khizar", 1.0 },
  { "Khizer", 1.0 },
  { "Khloe", 0.00054949825 },
  { "Khloeann", 0.0 },
  { "Khloee", 0.0 },
  { "Khloei", 0.0 },
  { "Khloemae", 0.0 },
  { "Khloemarie", 0.0 },
  { "Khloey", 0.0 },
  { "Khloi", 0.0 },
  { "Khloie", 0.0 },
  { "Khloii", 0.0 },
  { "Khloye", 0.0 },
  { "Khmari", 1.0 },
  { "Khmya", 0.0 },
  { "Khoa", 1.0 },
  { "Khobe", 1.0 },
  { "Khodee", 1.0 },
  { "Khodi", 0.24 },
  { "Khoen", 1.0 },
  { "Khoi", 0.9788868 },
  { "Khol", 1.0 },
  { "Kholby", 1.0 },
  { "Khole", 0.13703704 },
  { "Kholee", 0.0 },
  { "Kholette", 0.0 },
  { "Kholie", 0.0 },
  { "Kholten", 1.0 },
  { "Kholton", 1.0 },
  { "Khonner", 1.0 },
  { "Khonor", 1.0 },
  { "Khora", 0.0 },
  { "Khorey", 1.0 },
  { "Khori", 0.22580644 },
  { "Khorie", 0.0 },
  { "Khory", 1.0 },
  { "Khotan", 1.0 },
  { "Khou", 0.0 },
  { "Khouri", 0.45454547 },
  { "Khoury", 1.0 },
  { "Khris", 0.8610272 },
  { "Khrishna", 0.0 },
  { "Khrissa", 0.0 },
  { "Khrista", 0.0 },
  { "Khristal", 0.0 },
  { "Khristan", 1.0 },
  { "Khristen", 0.0 },
  { "Khristi", 0.0 },
  { "Khristian", 0.816944 },
  { "Khristiana", 0.0 },
  { "Khristie", 0.0 },
  { "Khristin", 0.0 },
  { "Khristina", 0.0 },
  { "Khristine", 0.0 },
  { "Khriston", 1.0 },
  { "Khristopher", 1.0 },
  { "Khristy", 0.0 },
  { "Khriz", 1.0 },
  { "Khrysta", 0.0 },
  { "Khrystal", 0.0 },
  { "Khrysten", 0.0 },
  { "Khrystian", 0.87068963 },
  { "Khrystin", 0.0 },
  { "Khrystina", 0.0 },
  { "Khrystine", 0.0 },
  { "Khrystopher", 1.0 },
  { "Khrystyn", 0.0 },
  { "Khrystyna", 0.0 },
  { "Khrystyne", 0.0 },
  { "Khuong", 1.0 },
  { "Khup", 1.0 },
  { "Khurram", 1.0 },
  { "Khusbu", 0.0 },
  { "Khush", 1.0 },
  { "Khushal", 1.0 },
  { "Khushboo", 0.0 },
  { "Khushbu", 0.0 },
  { "Khushi", 0.0 },
  { "Khushpreet", 0.0 },
  { "Khy", 1.0 },
  { "Khya", 0.0 },
  { "Khyair", 1.0 },
  { "Khyaire", 1.0 },
  { "Khyan", 1.0 },
  { "Khyana", 0.0 },
  { "Khyara", 0.0 },
  { "Khyati", 0.0 },
  { "Khyber", 1.0 },
  { "Khyden", 1.0 },
  { "Khye", 1.0 },
  { "Khyia", 0.0 },
  { "Khyier", 1.0 },
  { "Khyion", 1.0 },
  { "Khyir", 1.0 },
  { "Khyla", 0.0 },
  { "Khylah", 0.0 },
  { "Khylan", 0.9800797 },
  { "Khylar", 0.76 },
  { "Khyle", 0.92523366 },
  { "Khylee", 0.0 },
  { "Khylei", 0.0 },
  { "Khyleigh", 0.0 },
  { "Khylen", 1.0 },
  { "Khyler", 0.8337469 },
  { "Khyli", 0.0 },
  { "Khylia", 0.0 },
  { "Khylie", 0.0 },
  { "Khylige", 1.0 },
  { "Khylil", 1.0 },
  { "Khylin", 0.75 },
  { "Khylon", 1.0 },
  { "Khylynn", 0.0 },
  { "Khymani", 0.5 },
  { "Khymari", 1.0 },
  { "Khymir", 1.0 },
  { "Khyng", 1.0 },
  { "Khyon", 1.0 },
  { "Khyra", 0.0 },
  { "Khyran", 1.0 },
  { "Khyre", 1.0 },
  { "Khyree", 0.98956156 },
  { "Khyrell", 1.0 },
  { "Khyren", 1.0 },
  { "Khyri", 0.97154474 },
  { "Khyrie", 0.8564815 },
  { "Khyrin", 1.0 },
  { "Khyron", 1.0 },
  { "Khysen", 1.0 },
  { "Khyson", 1.0 },
  { "Ki", 0.7928118 },
  { "Kia", 0.022178734 },
  { "Kiaan", 1.0 },
  { "Kiabeth", 0.0 },
  { "Kiaeem", 1.0 },
  { "Kiah", 0.07564935 },
  { "Kiahna", 0.0 },
  { "Kiahnna", 0.0 },
  { "Kiahra", 0.0 },
  { "Kiai", 1.0 },
  { "Kiaira", 0.0 },
  { "Kiairra", 0.0 },
  { "Kiaja", 0.0 },
  { "Kiajah", 0.0 },
  { "Kial", 1.0 },
  { "Kiala", 0.0 },
  { "Kialani", 0.0 },
  { "Kialee", 0.0 },
  { "Kialey", 0.0 },
  { "Kiali", 0.0 },
  { "Kialie", 0.0 },
  { "Kialyn", 0.0 },
  { "Kiam", 1.0 },
  { "Kiamani", 0.0 },
  { "Kiamara", 0.0 },
  { "Kiamber", 0.0 },
  { "Kiambu", 1.0 },
  { "Kiamesha", 0.0 },
  { "Kiamora", 0.0 },
  { "Kian", 0.9881624 },
  { "Kiana", 0.00080355257 },
  { "Kianah", 0.0 },
  { "Kianalee", 0.0 },
  { "Kiandra", 0.0 },
  { "Kiandre", 1.0 },
  { "Kiandrea", 0.0 },
  { "Kiandria", 0.0 },
  { "Kiane", 0.0 },
  { "Kianga", 0.0 },
  { "Kiani", 0.028873917 },
  { "Kiania", 0.0 },
  { "Kianie", 0.0 },
  { "Kiann", 0.0 },
  { "Kianna", 0.0 },
  { "Kiannah", 0.0 },
  { "Kianne", 0.0 },
  { "Kianni", 0.0 },
  { "Kianny", 0.0 },
  { "Kiano", 1.0 },
  { "Kiansh", 1.0 },
  { "Kianta", 0.0 },
  { "Kiante", 0.70163935 },
  { "Kianti", 0.0 },
  { "Kianu", 1.0 },
  { "Kiany", 0.0 },
  { "Kiara", 0.0046908497 },
  { "Kiarah", 0.0 },
  { "Kiaralee", 0.0 },
  { "Kiaraliz", 0.0 },
  { "Kiaralyn", 0.0 },
  { "Kiaran", 1.0 },
  { "Kiare", 0.16129032 },
  { "Kiari", 0.09779951 },
  { "Kiaria", 0.0 },
  { "Kiaron", 1.0 },
  { "Kiarra", 0.0 },
  { "Kiarrah", 0.0 },
  { "Kiarri", 0.0 },
  { "Kiary", 0.0 },
  { "Kias", 1.0 },
  { "Kiasha", 0.0 },
  { "Kiasia", 0.0 },
  { "Kiata", 0.0 },
  { "Kiauna", 0.0 },
  { "Kiaundra", 0.0 },
  { "Kiaunna", 0.0 },
  { "Kiaura", 0.0 },
  { "Kiava", 0.0 },
  { "Kiawana", 0.0 },
  { "Kiawna", 0.0 },
  { "Kiaya", 0.0 },
  { "Kiayah", 0.0 },
  { "Kiayana", 0.0 },
  { "Kiayanna", 0.0 },
  { "Kiayla", 0.0 },
  { "Kiayra", 0.0 },
  { "Kiazer", 1.0 },
  { "Kiba", 1.0 },
  { "Kibbie", 0.0 },
  { "Kibibi", 0.0 },
  { "Kibwe", 1.0 },
  { "Kicha", 0.0 },
  { "Kichelle", 0.0 },
  { "Kicia", 0.0 },
  { "Kid", 1.0 },
  { "Kida", 0.0 },
  { "Kidada", 0.0 },
  { "Kidane", 1.0 },
  { "Kidata", 0.0 },
  { "Kidd", 1.0 },
  { "Kiden", 0.0 },
  { "Kidist", 0.0 },
  { "Kidron", 1.0 },
  { "Kidus", 1.0 },
  { "Kie", 0.96031743 },
  { "Kiea", 0.0 },
  { "Kiean", 1.0 },
  { "Kieana", 0.0 },
  { "Kieandra", 0.0 },
  { "Kieanna", 0.0 },
  { "Kieara", 0.0 },
  { "Kiearah", 0.0 },
  { "Kiearra", 0.0 },
  { "Kieasha", 0.0 },
  { "Kieayla", 0.0 },
  { "Kieden", 1.0 },
  { "Kiedis", 1.0 },
  { "Kief", 1.0 },
  { "Kiefer", 1.0 },
  { "Kieffer", 1.0 },
  { "Kiegan", 0.9637681 },
  { "Kieisha", 0.0 },
  { "Kiel", 0.98769987 },
  { "Kiela", 0.0 },
  { "Kielah", 0.0 },
  { "Kielan", 1.0 },
  { "Kiele", 0.0 },
  { "Kielee", 0.0 },
  { "Kieleigh", 0.0 },
  { "Kieler", 1.0 },
  { "Kieley", 0.0 },
  { "Kieli", 0.0 },
  { "Kielin", 1.0 },
  { "Kiely", 0.0 },
  { "Kielyn", 0.065789476 },
  { "Kielynn", 0.0 },
  { "Kien", 1.0 },
  { "Kienan", 1.0 },
  { "Kienna", 0.0 },
  { "Kieon", 1.0 },
  { "Kieona", 0.0 },
  { "Kieonna", 0.0 },
  { "Kier", 0.97459584 },
  { "Kiera", 0.000979533 },
  { "Kierah", 0.0 },
  { "Kieralyn", 0.0 },
  { "Kieran", 0.89362496 },
  { "Kierce", 1.0 },
  { "Kiere", 1.0 },
  { "Kieren", 0.770428 },
  { "Kieria", 0.0 },
  { "Kierian", 1.0 },
  { "Kieriana", 0.0 },
  { "Kierica", 0.0 },
  { "Kierin", 0.6530612 },
  { "Kiernan", 0.72731906 },
  { "Kierney", 0.0 },
  { "Kieron", 1.0 },
  { "Kierra", 0.0011876485 },
  { "Kierrah", 0.0 },
  { "Kierre", 0.8016878 },
  { "Kierria", 0.0 },
  { "Kierron", 1.0 },
  { "Kiersa", 0.0 },
  { "Kiersen", 0.0 },
  { "Kierslyn", 0.0 },
  { "Kierson", 1.0 },
  { "Kiersta", 0.0 },
  { "Kierstan", 0.0 },
  { "Kiersten", 0.000349993 },
  { "Kiersti", 0.0 },
  { "Kierstie", 0.0 },
  { "Kierstin", 0.0 },
  { "Kierston", 0.015494636 },
  { "Kiersty", 0.0 },
  { "Kierstyn", 0.0 },
  { "Kierstynn", 0.0 },
  { "Kiersyn", 0.0 },
  { "Kieryn", 0.085427135 },
  { "Kiesa", 0.0 },
  { "Kiesel", 0.0 },
  { "Kiesha", 0.0 },
  { "Kieshawn", 1.0 },
  { "Kieshia", 0.0 },
  { "Kieston", 1.0 },
  { "Kiet", 1.0 },
  { "Kieth", 1.0 },
  { "Kieu", 0.0 },
  { "Kiev", 1.0 },
  { "Kieva", 0.0 },
  { "Kievan", 1.0 },
  { "Kievon", 1.0 },
  { "Kieya", 0.0 },
  { "Kieyana", 0.0 },
  { "Kiffany", 0.0 },
  { "Kiffin", 0.0 },
  { "Kihana", 0.0 },
  { "Kihanna", 0.0 },
  { "Kihara", 0.0 },
  { "Kiheem", 1.0 },
  { "Kihry", 1.0 },
  { "Kiiara", 0.0 },
  { "Kiichi", 1.0 },
  { "Kiing", 1.0 },
  { "Kiira", 0.0 },
  { "Kiirsten", 0.0 },
  { "Kiisha", 0.0 },
  { "Kija", 0.0 },
  { "Kijafa", 0.0 },
  { "Kijah", 0.0 },
  { "Kijana", 0.6583541 },
  { "Kijani", 1.0 },
  { "Kijon", 1.0 },
  { "Kijuan", 1.0 },
  { "Kijuana", 0.0 },
  { "Kika", 0.0 },
  { "Kiki", 0.024777006 },
  { "Kiko", 0.6315789 },
  { "Kiku", 0.0 },
  { "Kikue", 0.0 },
  { "Kikuko", 0.0 },
  { "Kikumi", 0.0 },
  { "Kikuye", 0.0 },
  { "Kikuyo", 0.0 },
  { "Kikyo", 0.0 },
  { "Kila", 0.005701254 },
  { "Kilah", 0.0 },
  { "Kilan", 1.0 },
  { "Kilana", 0.0 },
  { "Kilani", 0.0 },
  { "Kilbourne", 1.0 },
  { "Kilburn", 1.0 },
  { "Kilby", 1.0 },
  { "Kile", 1.0 },
  { "Kilea", 0.0 },
  { "Kileah", 0.0 },
  { "Kilean", 1.0 },
  { "Kilee", 0.0 },
  { "Kileen", 0.0 },
  { "Kilei", 0.0 },
  { "Kileigh", 0.0 },
  { "Kilen", 1.0 },
  { "Kiler", 1.0 },
  { "Kiley", 0.056628123 },
  { "Kileyann", 0.0 },
  { "Kili", 0.052173913 },
  { "Kilia", 0.0 },
  { "Kilian", 0.9823529 },
  { "Kilie", 0.0 },
  { "Kilik", 1.0 },
  { "Kiliyah", 0.0 },
  { "Killashandra", 0.0 },
  { "Killeen", 0.0 },
  { "Killian", 0.93874854 },
  { "Killion", 1.0 },
  { "Killis", 1.0 },
  { "Kilmer", 1.0 },
  { "Kilo", 1.0 },
  { "Kilolo", 0.0 },
  { "Kily", 0.0 },
  { "Kilyan", 1.0 },
  { "Kilyn", 0.12142857 },
  { "Kilynn", 0.0 },
  { "Kim", 0.16268805 },
  { "Kima", 0.0 },
  { "Kimaada", 0.0 },
  { "Kimahri", 1.0 },
  { "Kimaka", 0.0 },
  { "Kimako", 0.0 },
  { "Kimala", 0.0 },
  { "Kimalee", 0.0 },
  { "Kimana", 0.0 },
  { "Kimanh", 0.0 },
  { "Kimani", 0.5420765 },
  { "Kimann", 0.0 },
  { "Kimar", 1.0 },
  { "Kimara", 0.0 },
  { "Kimarah", 0.0 },
  { "Kimaree", 0.0 },
  { "Kimari", 0.3568216 },
  { "Kimaria", 0.0 },
  { "Kimarie", 0.0 },
  { "Kimario", 1.0 },
  { "Kimarion", 1.0 },
  { "Kimarley", 1.0 },
  { "Kimarri", 0.0 },
  { "Kimary", 0.0 },
  { "Kimatha", 0.0 },
  { "Kimathi", 1.0 },
  { "Kimauri", 0.0 },
  { "Kimaya", 0.0 },
  { "Kimayah", 0.0 },
  { "Kimba", 0.0 },
  { "Kimbal", 0.91803277 },
  { "Kimbala", 0.0 },
  { "Kimball", 0.9317073 },
  { "Kimbel", 1.0 },
  { "Kimbell", 1.0 },
  { "Kimbella", 0.0 },
  { "Kimbelry", 0.0 },
  { "Kimbely", 0.0 },
  { "Kimber", 0.043188114 },
  { "Kimbereley", 0.0 },
  { "Kimberely", 0.0 },
  { "Kimberl", 0.0 },
  { "Kimberla", 0.0 },
  { "Kimberland", 0.0 },
  { "Kimberle", 0.0 },
  { "Kimberlea", 0.0 },
  { "Kimberlee", 0.0 },
  { "Kimberlei", 0.0 },
  { "Kimberleigh", 0.0 },
  { "Kimberley", 0.0061744615 },
  { "Kimberli", 0.0 },
  { "Kimberlie", 0.0 },
  { "Kimberlin", 0.0 },
  { "Kimberlina", 0.0 },
  { "Kimberline", 0.0 },
  { "Kimberlly", 0.0 },
  { "Kimberly", 0.0050896998 },
  { "Kimberlyann", 0.0 },
  { "Kimberlyanne", 0.0 },
  { "Kimberlye", 0.0 },
  { "Kimberlyn", 0.0 },
  { "Kimberlynn", 0.0 },
  { "Kimberlyy", 0.0 },
  { "Kimbery", 0.0 },
  { "Kimberyl", 0.0 },
  { "Kimbie", 0.0 },
  { "Kimbla", 0.0 },
  { "Kimble", 0.7223587 },
  { "Kimbler", 0.0 },
  { "Kimblery", 0.0 },
  { "Kimbley", 0.0 },
  { "Kimbly", 0.0 },
  { "Kimblyn", 0.0 },
  { "Kimbra", 0.0 },
  { "Kimbre", 0.0 },
  { "Kimbree", 0.0 },
  { "Kimbrell", 0.5 },
  { "Kimbrely", 0.0 },
  { "Kimbria", 0.0 },
  { "Kimbrielle", 0.0 },
  { "Kimbrley", 0.0 },
  { "Kimbrly", 0.0 },
  { "Kimby", 0.0 },
  { "Kimchi", 0.0 },
  { "Kimeka", 0.0 },
  { "Kimeko", 0.0 },
  { "Kimela", 0.0 },
  { "Kimella", 0.0 },
  { "Kimely", 0.0 },
  { "Kimera", 0.0 },
  { "Kimerly", 0.0 },
  { "Kimery", 0.0 },
  { "Kimesha", 0.0 },
  { "Kimetha", 0.0 },
  { "Kimetra", 0.0 },
  { "Kimette", 0.0 },
  { "Kimi", 0.08592911 },
  { "Kimia", 0.0 },
  { "Kimiah", 0.0 },
  { "Kimie", 0.0 },
  { "Kimika", 0.0 },
  { "Kimiko", 0.0 },
  { "Kimila", 0.0 },
  { "Kimilee", 0.0 },
  { "Kimily", 0.0 },
  { "Kimimela", 0.0 },
  { "Kimimila", 0.0 },
  { "Kimira", 0.0 },
  { "Kimisha", 0.0 },
  { "Kimitra", 0.0 },
  { "Kimiya", 0.0 },
  { "Kimiyah", 0.0 },
  { "Kimiye", 0.0 },
  { "Kimiyo", 0.0 },
  { "Kimla", 0.0 },
  { "Kimlee", 0.0 },
  { "Kimley", 0.0 },
  { "Kimly", 0.0 },
  { "Kimlyn", 0.0 },
  { "Kimm", 0.19672132 },
  { "Kimmarie", 0.0 },
  { "Kimmberly", 0.0 },
  { "Kimmerly", 0.0 },
  { "Kimmesha", 0.0 },
  { "Kimmey", 0.0 },
  { "Kimmi", 0.0 },
  { "Kimmie", 0.096394405 },
  { "Kimmika", 0.0 },
  { "Kimmora", 0.0 },
  { "Kimmori", 0.0 },
  { "Kimmy", 0.21193573 },
  { "Kimmya", 0.0 },
  { "Kimo", 1.0 },
  { "Kimon", 1.0 },
  { "Kimona", 0.0 },
  { "Kimone", 0.0 },
  { "Kimoni", 0.59438777 },
  { "Kimonie", 0.0 },
  { "Kimonte", 1.0 },
  { "Kimora", 0.0 },
  { "Kimorah", 0.0 },
  { "Kimoralee", 0.0 },
  { "Kimore", 0.0 },
  { "Kimoree", 0.0 },
  { "Kimori", 0.0 },
  { "Kimoria", 0.0 },
  { "Kimorra", 0.0 },
  { "Kimothy", 0.78125 },
  { "Kimoya", 0.0 },
  { "Kimra", 0.0 },
  { "Kimsey", 1.0 },
  { "Kimura", 0.0 },
  { "Kimverly", 0.0 },
  { "Kimwana", 0.0 },
  { "Kimya", 0.0 },
  { "Kimyada", 0.0 },
  { "Kimyah", 0.0 },
  { "Kimyata", 0.0 },
  { "Kimyatta", 0.0 },
  { "Kimyetta", 0.0 },
  { "Kimyia", 0.0 },
  { "Kimyra", 0.0 },
  { "Kin", 0.9758065 },
  { "Kina", 0.0 },
  { "Kinadee", 0.0 },
  { "Kinady", 0.0 },
  { "Kinan", 1.0 },
  { "Kinara", 0.0 },
  { "Kinard", 1.0 },
  { "Kinaya", 0.0 },
  { "Kinberli", 0.0 },
  { "Kinberlin", 0.0 },
  { "Kinberly", 0.0 },
  { "Kincade", 1.0 },
  { "Kincaid", 1.0 },
  { "Kinda", 0.0 },
  { "Kindal", 0.0 },
  { "Kindall", 0.07188498 },
  { "Kindalyn", 0.0 },
  { "Kindel", 0.07042254 },
  { "Kindell", 0.2300885 },
  { "Kinden", 1.0 },
  { "Kinder", 0.0 },
  { "Kindle", 0.07129095 },
  { "Kindra", 0.0 },
  { "Kindrea", 0.0 },
  { "Kindred", 0.53157896 },
  { "Kindric", 1.0 },
  { "Kindrick", 1.0 },
  { "Kindsay", 0.0 },
  { "Kindsey", 0.0 },
  { "Kindu", 1.0 },
  { "Kindy", 0.0 },
  { "Kindyl", 0.0 },
  { "Kine", 0.0 },
  { "Kinesha", 0.0 },
  { "Kineshia", 0.0 },
  { "Kineta", 0.0 },
  { "King", 0.9995225 },
  { "Kinga", 0.0 },
  { "Kingamir", 1.0 },
  { "Kinganthony", 1.0 },
  { "Kingcharles", 1.0 },
  { "Kingdavid", 1.0 },
  { "Kingdom", 1.0 },
  { "Kingdon", 1.0 },
  { "Kingelijah", 1.0 },
  { "Kingg", 1.0 },
  { "Kingisaiah", 1.0 },
  { "Kingjames", 1.0 },
  { "Kingjoseph", 1.0 },
  { "Kingjosiah", 1.0 },
  { "Kingman", 1.0 },
  { "Kingmessiah", 1.0 },
  { "Kingmichael", 1.0 },
  { "Kingslee", 0.5984252 },
  { "Kingsleigh", 0.0 },
  { "Kingsley", 0.83360183 },
  { "Kingsly", 1.0 },
  { "Kingsolomon", 1.0 },
  { "Kingson", 1.0 },
  { "Kingstan", 1.0 },
  { "Kingsten", 1.0 },
  { "Kingstin", 1.0 },
  { "Kingston", 0.9946545 },
  { "Kingstun", 1.0 },
  { "Kingstyn", 1.0 },
  { "Kingzion", 1.0 },
  { "Kingzley", 1.0 },
  { "Kingzton", 1.0 },
  { "Kini", 0.0 },
  { "Kiniah", 0.0 },
  { "Kiniesha", 0.0 },
  { "Kinika", 0.0 },
  { "Kinisha", 0.0 },
  { "Kinita", 0.0 },
  { "Kiniya", 0.0 },
  { "Kiniyah", 0.0 },
  { "Kinjal", 0.0 },
  { "Kinji", 1.0 },
  { "Kinkade", 1.0 },
  { "Kinlea", 0.0 },
  { "Kinlee", 0.0 },
  { "Kinlei", 0.0 },
  { "Kinleigh", 0.0 },
  { "Kinley", 0.020986432 },
  { "Kinli", 0.0 },
  { "Kinlie", 0.0 },
  { "Kinly", 0.0 },
  { "Kinlyn", 0.0 },
  { "Kinlynn", 0.0 },
  { "Kinna", 0.0 },
  { "Kinnard", 1.0 },
  { "Kinnedi", 0.0 },
  { "Kinnedy", 0.0 },
  { "Kinneret", 0.0 },
  { "Kinneth", 1.0 },
  { "Kinney", 1.0 },
  { "Kinnick", 0.98663104 },
  { "Kinnidi", 0.0 },
  { "Kinnidy", 0.0 },
  { "Kinnie", 0.9695122 },
  { "Kinnith", 1.0 },
  { "Kinnley", 0.0 },
  { "Kinnon", 1.0 },
  { "Kinny", 1.0 },
  { "Kino", 1.0 },
  { "Kinsasha", 0.0 },
  { "Kinsee", 0.0 },
  { "Kinsely", 0.0 },
  { "Kinser", 1.0 },
  { "Kinsey", 0.04104062 },
  { "Kinshasa", 0.04918033 },
  { "Kinshasha", 0.0 },
  { "Kinsie", 0.0 },
  { "Kinsington", 0.0 },
  { "Kinslea", 0.0 },
  { "Kinslee", 0.0 },
  { "Kinslei", 0.0 },
  { "Kinsleigh", 0.0 },
  { "Kinsler", 0.9019608 },
  { "Kinsley", 0.006293061 },
  { "Kinsli", 0.0 },
  { "Kinslie", 0.0 },
  { "Kinsly", 0.0 },
  { "Kinson", 1.0 },
  { "Kinston", 1.0 },
  { "Kinta", 0.83908045 },
  { "Kinte", 0.94666666 },
  { "Kintrell", 1.0 },
  { "Kinue", 0.0 },
  { "Kinuko", 0.0 },
  { "Kinverli", 0.0 },
  { "Kinverlin", 0.0 },
  { "Kinya", 0.0 },
  { "Kinyada", 0.0 },
  { "Kinyata", 0.0 },
  { "Kinyatta", 0.0 },
  { "Kinyetta", 0.0 },
  { "Kinyon", 1.0 },
  { "Kinza", 0.0 },
  { "Kinze", 0.0 },
  { "Kinzee", 0.0 },
  { "Kinzer", 1.0 },
  { "Kinzey", 0.0 },
  { "Kinzi", 0.0 },
  { "Kinzie", 0.0 },
  { "Kinzington", 0.0 },
  { "Kinzlee", 0.0 },
  { "Kinzleigh", 0.0 },
  { "Kinzler", 1.0 },
  { "Kinzley", 0.0 },
  { "Kinzli", 0.0 },
  { "Kinzlie", 0.0 },
  { "Kinzly", 0.0 },
  { "Kinzy", 0.0 },
  { "Kio", 1.0 },
  { "Kioka", 0.0 },
  { "Kioko", 0.0 },
  { "Kiomara", 0.0 },
  { "Kiomi", 0.0 },
  { "Kiomy", 0.0 },
  { "Kion", 1.0 },
  { "Kiona", 0.0 },
  { "Kionah", 0.0 },
  { "Kiondra", 0.0 },
  { "Kiondre", 1.0 },
  { "Kione", 1.0 },
  { "Kioni", 0.0 },
  { "Kionia", 0.0 },
  { "Kionna", 0.0 },
  { "Kionne", 0.7169811 },
  { "Kiontae", 1.0 },
  { "Kionte", 0.9279279 },
  { "Kiora", 0.0 },
  { "Kiori", 0.0 },
  { "Kiosha", 0.0 },
  { "Kiotti", 1.0 },
  { "Kiowa", 0.7291667 },
  { "Kip", 0.98026145 },
  { "Kipchoge", 1.0 },
  { "Kipenie", 0.0 },
  { "Kipley", 1.0 },
  { "Kiplin", 1.0 },
  { "Kipling", 1.0 },
  { "Kipp", 0.99765587 },
  { "Kipper", 1.0 },
  { "Kippi", 0.0 },
  { "Kippie", 0.25 },
  { "Kippy", 0.619403 },
  { "Kipten", 1.0 },
  { "Kiptin", 1.0 },
  { "Kipton", 1.0 },
  { "Kiptyn", 0.9715243 },
  { "Kira", 0.00080080674 },
  { "Kirah", 0.0 },
  { "Kiralee", 0.0 },
  { "Kiralyn", 0.0 },
  { "Kiralynn", 0.0 },
  { "Kiran", 0.55206186 },
  { "Kirana", 0.0 },
  { "Kirandeep", 0.0 },
  { "Kiranjot", 0.0 },
  { "Kirara", 0.0 },
  { "Kirat", 0.0 },
  { "Kirbee", 0.0 },
  { "Kirbi", 0.0 },
  { "Kirbie", 0.0 },
  { "Kirby", 0.80149853 },
  { "Kire", 0.95098037 },
  { "Kiree", 0.86885244 },
  { "Kireina", 0.0 },
  { "Kiren", 0.8035714 },
  { "Kiri", 0.0 },
  { "Kiria", 0.0 },
  { "Kiriaki", 0.0 },
  { "Kiriakos", 1.0 },
  { "Kirian", 0.6666667 },
  { "Kiriana", 0.0 },
  { "Kirianna", 0.0 },
  { "Kirie", 0.0 },
  { "Kiril", 1.0 },
  { "Kirill", 1.0 },
  { "Kirin", 0.47605634 },
  { "Kirina", 0.0 },
  { "Kirisa", 0.0 },
  { "Kiristen", 0.0 },
  { "Kirk", 0.99708974 },
  { "Kirke", 1.0 },
  { "Kirklan", 1.0 },
  { "Kirkland", 0.9942693 },
  { "Kirklen", 1.0 },
  { "Kirklin", 1.0 },
  { "Kirklyn", 0.3529412 },
  { "Kirkpatrick", 1.0 },
  { "Kirkwood", 1.0 },
  { "Kiro", 1.0 },
  { "Kirollos", 1.0 },
  { "Kirolos", 1.0 },
  { "Kiron", 1.0 },
  { "Kiros", 1.0 },
  { "Kirpa", 0.0 },
  { "Kirra", 0.0 },
  { "Kirrah", 0.0 },
  { "Kirsey", 0.0 },
  { "Kirsha", 0.0 },
  { "Kirsi", 0.0 },
  { "Kirsta", 0.0 },
  { "Kirstan", 0.0 },
  { "Kirstee", 0.0 },
  { "Kirstein", 0.0 },
  { "Kirsten", 0.0039978023 },
  { "Kirstey", 0.0 },
  { "Kirsti", 0.0 },
  { "Kirstian", 0.0 },
  { "Kirstie", 0.0 },
  { "Kirstien", 0.0 },
  { "Kirstin", 0.0 },
  { "Kirstina", 0.0 },
  { "Kirstine", 0.0 },
  { "Kirston", 0.0 },
  { "Kirstopher", 1.0 },
  { "Kirsty", 0.0 },
  { "Kirstyn", 0.0 },
  { "Kirstynn", 0.0 },
  { "Kirsy", 0.0 },
  { "Kirt", 1.0 },
  { "Kirtan", 1.0 },
  { "Kirtana", 0.0 },
  { "Kirthana", 0.0 },
  { "Kirthik", 1.0 },
  { "Kirti", 0.0 },
  { "Kirtis", 1.0 },
  { "Kirtland", 1.0 },
  { "Kirtley", 1.0 },
  { "Kirtus", 1.0 },
  { "Kirubel", 1.0 },
  { "Kirwin", 1.0 },
  { "Kiry", 1.0 },
  { "Kirya", 0.0 },
  { "Kiryn", 0.23287672 },
  { "Kirynn", 0.0 },
  { "Kisa", 0.0 },
  { "Kisan", 1.0 },
  { "Kisara", 0.0 },
  { "Kischa", 0.0 },
  { "Kisean", 1.0 },
  { "Kiser", 1.0 },
  { "Kish", 0.25 },
  { "Kisha", 0.0018105009 },
  { "Kishan", 1.0 },
  { "Kishana", 0.0 },
  { "Kishanna", 0.0 },
  { "Kishara", 0.1923077 },
  { "Kishaun", 1.0 },
  { "Kishauna", 0.0 },
  { "Kishawn", 0.9702381 },
  { "Kishawna", 0.0 },
  { "Kishea", 0.0 },
  { "Kishen", 1.0 },
  { "Kisher", 0.0 },
  { "Kishia", 0.0 },
  { "Kishina", 0.0 },
  { "Kishla", 0.0 },
  { "Kishma", 0.0 },
  { "Kishon", 0.92907804 },
  { "Kishonna", 0.0 },
  { "Kishore", 1.0 },
  { "Kishun", 1.0 },
  { "Kisia", 0.0 },
  { "Kiska", 0.0 },
  { "Kismet", 0.0 },
  { "Kison", 1.0 },
  { "Kissa", 0.0 },
  { "Kissey", 0.0 },
  { "Kissha", 0.0 },
  { "Kissiah", 0.0 },
  { "Kissie", 0.0 },
  { "Kissy", 0.0 },
  { "Kista", 0.0 },
  { "Kisten", 0.0 },
  { "Kistin", 0.0 },
  { "Kistina", 0.0 },
  { "Kisty", 0.0 },
  { "Kiswana", 0.0 },
  { "Kiswanna", 0.0 },
  { "Kit", 0.6876252 },
  { "Kita", 0.0 },
  { "Kitai", 1.0 },
  { "Kitana", 0.0 },
  { "Kitanna", 0.0 },
  { "Kitara", 0.0 },
  { "Kitiara", 0.0 },
  { "Kitina", 0.0 },
  { "Kito", 0.93373495 },
  { "Kitra", 0.0 },
  { "Kitrick", 1.0 },
  { "Kitrina", 0.0 },
  { "Kitsia", 0.0 },
  { "Kitson", 1.0 },
  { "Kitsy", 0.0 },
  { "Kitt", 0.4053156 },
  { "Kitten", 0.0 },
  { "Kitti", 0.0 },
  { "Kittie", 0.0 },
  { "Kitty", 0.0 },
  { "Kittye", 0.0 },
  { "Kitwana", 1.0 },
  { "Kitza", 0.0 },
  { "Kitzi", 0.0 },
  { "Kitzia", 0.0 },
  { "Kitzie", 0.0 },
  { "Kitzya", 0.0 },
  { "Kiuna", 0.0 },
  { "Kiva", 0.032697547 },
  { "Kivan", 1.0 },
  { "Kiven", 1.0 },
  { "Kivin", 1.0 },
  { "Kivon", 1.0 },
  { "Kiwan", 1.0 },
  { "Kiwana", 0.0 },
  { "Kiwanda", 0.0 },
  { "Kiwane", 1.0 },
  { "Kiwanis", 0.0 },
  { "Kiwanna", 0.0 },
  { "Kiwon", 1.0 },
  { "Kix", 1.0 },
  { "Kiya", 0.006499011 },
  { "Kiyaan", 1.0 },
  { "Kiyah", 0.0 },
  { "Kiyami", 0.0 },
  { "Kiyan", 0.9739777 },
  { "Kiyana", 0.0 },
  { "Kiyanah", 0.0 },
  { "Kiyani", 0.0 },
  { "Kiyann", 0.0 },
  { "Kiyanna", 0.0 },
  { "Kiyanni", 0.0 },
  { "Kiyansh", 1.0 },
  { "Kiyara", 0.0 },
  { "Kiyarah", 0.0 },
  { "Kiyari", 0.0 },
  { "Kiyarra", 0.0 },
  { "Kiyasha", 0.0 },
  { "Kiyel", 1.0 },
  { "Kiyelle", 0.0 },
  { "Kiyen", 1.0 },
  { "Kiyla", 0.0 },
  { "Kiylah", 0.0 },
  { "Kiylan", 1.0 },
  { "Kiyle", 1.0 },
  { "Kiylee", 0.0 },
  { "Kiyler", 1.0 },
  { "Kiylie", 0.0 },
  { "Kiyo", 0.0 },
  { "Kiyoko", 0.0 },
  { "Kiyomi", 0.007643312 },
  { "Kiyon", 1.0 },
  { "Kiyona", 0.0 },
  { "Kiyonna", 0.0 },
  { "Kiyono", 0.0 },
  { "Kiyora", 0.0 },
  { "Kiyoshi", 0.9912892 },
  { "Kiyoto", 1.0 },
  { "Kiyra", 0.0 },
  { "Kiyrah", 0.0 },
  { "Kiyuana", 0.0 },
  { "Kiz", 0.0 },
  { "Kizer", 1.0 },
  { "Kizi", 0.0 },
  { "Kiziah", 0.0 },
  { "Kizmet", 0.0 },
  { "Kizuwanda", 0.0 },
  { "Kizze", 0.0 },
  { "Kizzey", 0.0 },
  { "Kizzi", 0.0 },
  { "Kizzie", 0.0 },
  { "Kizzy", 0.0051347883 },
  { "Kj", 1.0 },
  { "Kjay", 1.0 },
  { "Kjell", 1.0 },
  { "Kjersten", 0.0 },
  { "Kjersti", 0.0 },
  { "Kjerstin", 0.0 },
  { "Kjirsten", 0.0 },
  { "Kjon", 1.0 },
  { "Kjuan", 1.0 },
  { "Kla", 0.0 },
  { "Klah", 0.0 },
  { "Klair", 0.0 },
  { "Klaira", 0.0 },
  { "Klaire", 0.0 },
  { "Klairissa", 0.0 },
  { "Klani", 0.0 },
  { "Klara", 0.0 },
  { "Klare", 0.0 },
  { "Klarisa", 0.0 },
  { "Klarissa", 0.0 },
  { "Klarity", 0.0 },
  { "Klariza", 0.0 },
  { "Klark", 0.9166667 },
  { "Klarke", 0.0 },
  { "Klarrisa", 0.0 },
  { "Klarrissa", 0.0 },
  { "Klarysa", 0.0 },
  { "Klaryssa", 0.0 },
  { "Klaudia", 0.0 },
  { "Klaus", 1.0 },
  { "Klay", 1.0 },
  { "Klayre", 0.0 },
  { "Klayten", 1.0 },
  { "Klayton", 1.0 },
  { "Klea", 0.0 },
  { "Kleah", 0.0 },
  { "Kleb", 1.0 },
  { "Kleber", 1.0 },
  { "Klee", 0.0 },
  { "Kleigh", 0.0 },
  { "Klein", 1.0 },
  { "Kleo", 0.0 },
  { "Kleopatra", 0.0 },
  { "Klever", 1.0 },
  { "Klevis", 1.0 },
  { "Klhoe", 0.0 },
  { "Klifton", 1.0 },
  { "Kline", 1.0 },
  { "Klint", 1.0 },
  { "Klinton", 1.0 },
  { "Klisha", 0.0 },
  { "Kliyah", 0.0 },
  { "Kloe", 0.0 },
  { "Kloee", 0.0 },
  { "Kloei", 0.0 },
  { "Kloeigh", 0.0 },
  { "Kloey", 0.0 },
  { "Klohe", 0.0 },
  { "Klohee", 0.0 },
  { "Klohie", 0.0 },
  { "Kloi", 0.0 },
  { "Kloie", 0.0 },
  { "Kloni", 0.0 },
  { "Klorissa", 0.0 },
  { "Kloud", 1.0 },
  { "Klover", 0.0 },
  { "Klowey", 0.0 },
  { "Klowi", 0.0 },
  { "Klowie", 0.0 },
  { "Kloye", 0.0 },
  { "Klutch", 1.0 },
  { "Klyda", 0.0 },
  { "Klyde", 1.0 },
  { "Klye", 1.0 },
  { "Klyn", 0.0 },
  { "Klynn", 0.0 },
  { "Kmani", 1.0 },
  { "Kmari", 0.52592593 },
  { "Kmarie", 0.0 },
  { "Kmarion", 1.0 },
  { "Kmauri", 1.0 },
  { "Kmaya", 0.0 },
  { "Kmiya", 0.0 },
  { "Kmiyah", 0.0 },
  { "Kmora", 0.0 },
  { "Kmya", 0.0 },
  { "Kmyah", 0.0 },
  { "Kneisha", 0.0 },
  { "Knesha", 0.0 },
  { "Knia", 0.0 },
  { "Knicole", 0.0 },
  { "Knight", 1.0 },
  { "Knightley", 0.25 },
  { "Knightly", 0.72 },
  { "Knighton", 1.0 },
  { "Kniko", 1.0 },
  { "Knisha", 0.0 },
  { "Knixon", 1.0 },
  { "Kniya", 0.0 },
  { "Kniyah", 0.0 },
  { "Knoa", 1.0 },
  { "Knoah", 1.0 },
  { "Knoel", 1.0 },
  { "Knoelle", 0.0 },
  { "Knolan", 1.0 },
  { "Knoll", 1.0 },
  { "Knova", 0.0 },
  { "Knowledge", 0.97956866 },
  { "Knowlton", 1.0 },
  { "Knox", 0.9935013 },
  { "Knoxley", 1.0 },
  { "Knoxton", 1.0 },
  { "Knoxville", 1.0 },
  { "Knoxx", 1.0 },
  { "Knut", 1.0 },
  { "Knute", 1.0 },
  { "Knya", 0.0 },
  { "Knyla", 0.0 },
  { "Knylah", 0.0 },
  { "Ko", 0.8507463 },
  { "Koa", 0.9051724 },
  { "Koah", 1.0 },
  { "Koal", 1.0 },
  { "Koalii", 1.0 },
  { "Koalton", 1.0 },
  { "Koamalu", 1.0 },
  { "Koan", 1.0 },
  { "Koba", 1.0 },
  { "Kobain", 1.0 },
  { "Kobby", 1.0 },
  { "Kobe", 0.98222053 },
  { "Kobee", 0.8888889 },
  { "Kobei", 1.0 },
  { "Koben", 1.0 },
  { "Kobey", 1.0 },
  { "Kobi", 0.6759308 },
  { "Kobie", 0.8359133 },
  { "Kobimtochukwu", 1.0 },
  { "Kobin", 1.0 },
  { "Kobina", 1.0 },
  { "Koby", 0.9884047 },
  { "Kobyn", 1.0 },
  { "Koda", 0.9154275 },
  { "Kodah", 1.0 },
  { "Kodan", 1.0 },
  { "Kode", 1.0 },
  { "Kodee", 0.47766322 },
  { "Koden", 1.0 },
  { "Kodey", 0.98401165 },
  { "Kodi", 0.4827935 },
  { "Kodiak", 1.0 },
  { "Kodie", 0.53499776 },
  { "Kodjo", 1.0 },
  { "Kody", 0.9678571 },
  { "Kodyn", 1.0 },
  { "Koebe", 1.0 },
  { "Koedy", 1.0 },
  { "Koehn", 1.0 },
  { "Koemi", 0.0 },
  { "Koen", 0.9963086 },
  { "Koesha", 0.0 },
  { "Koey", 0.0 },
  { "Koffi", 1.0 },
  { "Kofi", 0.9958089 },
  { "Kogan", 1.0 },
  { "Koh", 1.0 },
  { "Kohan", 1.0 },
  { "Kohana", 0.0 },
  { "Koharu", 0.0 },
  { "Kohei", 1.0 },
  { "Kohen", 0.9888204 },
  { "Kohl", 1.0 },
  { "Kohlby", 1.0 },
  { "Kohle", 1.0 },
  { "Kohlee", 0.0 },
  { "Kohler", 1.0 },
  { "Kohlman", 1.0 },
  { "Kohlson", 1.0 },
  { "Kohlston", 1.0 },
  { "Kohlten", 1.0 },
  { "Kohlton", 1.0 },
  { "Kohner", 1.0 },
  { "Kohyn", 1.0 },
  { "Koi", 0.64285713 },
  { "Koichi", 1.0 },
  { "Kojak", 1.0 },
  { "Koji", 1.0 },
  { "Kojiro", 1.0 },
  { "Kojo", 1.0 },
  { "Kokeeta", 0.0 },
  { "Koketa", 0.0 },
  { "Koki", 1.0 },
  { "Koko", 0.0 },
  { "Kokomi", 0.0 },
  { "Kokoro", 0.0 },
  { "Kokou", 1.0 },
  { "Kol", 1.0 },
  { "Kolade", 1.0 },
  { "Kolawole", 1.0 },
  { "Kolbe", 0.9706582 },
  { "Kolbee", 0.5126051 },
  { "Kolben", 1.0 },
  { "Kolbey", 0.9126984 },
  { "Kolbi", 0.28935185 },
  { "Kolbie", 0.3006483 },
  { "Kolbin", 1.0 },
  { "Kolby", 0.90543544 },
  { "Kolbyn", 1.0 },
  { "Kolden", 1.0 },
  { "Koldyn", 1.0 },
  { "Kole", 0.9947472 },
  { "Koleby", 1.0 },
  { "Kolee", 0.0 },
  { "Koleen", 0.0 },
  { "Koleigh", 0.0 },
  { "Koleman", 1.0 },
  { "Kolene", 0.0 },
  { "Koleson", 1.0 },
  { "Koleta", 0.0 },
  { "Koleton", 1.0 },
  { "Kolette", 0.0 },
  { "Koley", 0.0 },
  { "Koli", 0.61904764 },
  { "Kolia", 0.0 },
  { "Kolie", 0.0 },
  { "Kolin", 1.0 },
  { "Kolina", 0.0 },
  { "Kolleen", 0.0 },
  { "Kollen", 1.0 },
  { "Kollette", 0.0 },
  { "Kollier", 1.0 },
  { "Kollin", 0.9979027 },
  { "Kollins", 0.094444446 },
  { "Kollyn", 0.92941177 },
  { "Kollyns", 0.0 },
  { "Kolsen", 1.0 },
  { "Kolson", 1.0 },
  { "Kolsten", 1.0 },
  { "Kolston", 1.0 },
  { "Kolsyn", 1.0 },
  { "Kolt", 1.0 },
  { "Koltan", 1.0 },
  { "Kolten", 1.0 },
  { "Kolter", 1.0 },
  { "Koltin", 1.0 },
  { "Kolton", 1.0 },
  { "Koltyn", 0.9933599 },
  { "Koltynn", 1.0 },
  { "Kolvin", 1.0 },
  { "Kolya", 0.7 },
  { "Kolyn", 1.0 },
  { "Komal", 0.0 },
  { "Komalpreet", 0.0 },
  { "Komari", 1.0 },
  { "Komeka", 0.0 },
  { "Komora", 0.0 },
  { "Kona", 0.6885246 },
  { "Konan", 1.0 },
  { "Konar", 1.0 },
  { "Konata", 1.0 },
  { "Konda", 0.0 },
  { "Kondwani", 1.0 },
  { "Koner", 1.0 },
  { "Konesha", 0.0 },
  { "Koneta", 0.0 },
  { "Kong", 1.0 },
  { "Kongcheng", 1.0 },
  { "Kongmeng", 1.0 },
  { "Kongmong", 1.0 },
  { "Kongpheng", 1.0 },
  { "Koni", 0.0 },
  { "Konica", 0.0 },
  { "Konika", 0.0 },
  { "Konisha", 0.0 },
  { "Konley", 0.5 },
  { "Konnar", 0.96478873 },
  { "Konner", 0.97205424 },
  { "Konni", 0.0 },
  { "Konnie", 0.0 },
  { "Konnor", 0.98050386 },
  { "Konny", 0.0 },
  { "Kono", 1.0 },
  { "Konor", 1.0 },
  { "Konrad", 1.0 },
  { "Konstance", 0.0 },
  { "Konstandinos", 1.0 },
  { "Konstantin", 1.0 },
  { "Konstantina", 0.0 },
  { "Konstantine", 0.98245615 },
  { "Konstantino", 1.0 },
  { "Konstantinos", 1.0 },
  { "Konstanty", 1.0 },
  { "Kontar", 1.0 },
  { "Kontina", 0.0 },
  { "Kontrina", 0.0 },
  { "Konur", 1.0 },
  { "Konya", 0.0882353 },
  { "Konyae", 1.0 },
  { "Konye", 1.0 },
  { "Kookie", 0.0 },
  { "Koon", 1.0 },
  { "Kooper", 0.9200166 },
  { "Koorosh", 1.0 },
  { "Koosha", 1.0 },
  { "Kopelyn", 0.0 },
  { "Kopelynn", 0.0 },
  { "Kor", 1.0 },
  { "Kora", 0.0 },
  { "Korah", 0.0 },
  { "Koraima", 0.0 },
  { "Koral", 0.0 },
  { "Koralee", 0.0 },
  { "Koraleigh", 0.0 },
  { "Koralie", 0.0 },
  { "Koraline", 0.0 },
  { "Koralyn", 0.0 },
  { "Koralyne", 0.0 },
  { "Koralynn", 0.0 },
  { "Koran", 0.9815157 },
  { "Koray", 1.0 },
  { "Korayma", 0.0 },
  { "Korban", 1.0 },
  { "Korben", 1.0 },
  { "Korbin", 0.9833729 },
  { "Korbon", 1.0 },
  { "Korby", 0.8867925 },
  { "Korbyn", 0.81202406 },
  { "Korbynn", 0.0 },
  { "Kord", 1.0 },
  { "Kordae", 1.0 },
  { "Kordai", 1.0 },
  { "Kordale", 1.0 },
  { "Kordan", 1.0 },
  { "Kordarius", 1.0 },
  { "Korday", 1.0 },
  { "Korde", 1.0 },
  { "Kordel", 1.0 },
  { "Kordelia", 0.0 },
  { "Kordell", 1.0 },
  { "Korden", 1.0 },
  { "Kordero", 1.0 },
  { "Kore", 1.0 },
  { "Korea", 0.0 },
  { "Korede", 1.0 },
  { "Koree", 0.38181818 },
  { "Koreem", 1.0 },
  { "Koreen", 0.0 },
  { "Koreena", 0.0 },
  { "Korei", 1.0 },
  { "Koren", 0.13001145 },
  { "Korena", 0.0 },
  { "Korene", 0.0 },
  { "Koreon", 1.0 },
  { "Koressa", 0.0 },
  { "Koretta", 0.0 },
  { "Korey", 0.8915265 },
  { "Koreyon", 1.0 },
  { "Kori", 0.10091886 },
  { "Korian", 1.0 },
  { "Koriana", 0.0 },
  { "Koriann", 0.0 },
  { "Korianna", 0.0 },
  { "Korianne", 0.0 },
  { "Korie", 0.17057396 },
  { "Korilyn", 0.0 },
  { "Korilynn", 0.0 },
  { "Korin", 0.07036424 },
  { "Korina", 0.0 },
  { "Korine", 0.0 },
  { "Korinn", 0.0 },
  { "Korinna", 0.0 },
  { "Korinne", 0.0 },
  { "Korinthian", 1.0 },
  { "Korion", 1.0 },
  { "Korionna", 0.0 },
  { "Korissa", 0.0 },
  { "Korla", 0.0 },
  { "Korlee", 0.0 },
  { "Korlyn", 0.0 },
  { "Kornel", 1.0 },
  { "Kornelia", 0.0 },
  { "Kornelius", 1.0 },
  { "Kornell", 1.0 },
  { "Koron", 1.0 },
  { "Korra", 0.0 },
  { "Korrah", 0.0 },
  { "Korren", 0.0 },
  { "Korrey", 1.0 },
  { "Korri", 0.010928961 },
  { "Korrie", 0.040053405 },
  { "Korrigan", 1.0 },
  { "Korrin", 0.0 },
  { "Korrina", 0.0 },
  { "Korrine", 0.0 },
  { "Korrion", 1.0 },
  { "Korry", 0.9677419 },
  { "Korryn", 0.0 },
  { "Korsica", 0.0 },
  { "Kort", 1.0 },
  { "Kortana", 0.0 },
  { "Kortez", 1.0 },
  { "Kortlan", 1.0 },
  { "Kortland", 1.0 },
  { "Kortlin", 1.0 },
  { "Kortlyn", 0.0 },
  { "Kortlynn", 0.0 },
  { "Kortne", 0.0 },
  { "Kortnee", 0.0 },
  { "Kortnei", 0.0 },
  { "Kortney", 0.07662566 },
  { "Kortni", 0.0 },
  { "Kortnie", 0.0 },
  { "Kortny", 0.0 },
  { "Korver", 1.0 },
  { "Korvin", 1.0 },
  { "Korvyn", 1.0 },
  { "Korwin", 1.0 },
  { "Kory", 0.9179247 },
  { "Koryn", 0.0 },
  { "Koryna", 0.0 },
  { "Koryne", 0.0 },
  { "Korynn", 0.0 },
  { "Korynne", 0.0 },
  { "Kosei", 1.0 },
  { "Kosha", 0.0 },
  { "Kosi", 1.0 },
  { "Kosisochi", 0.0 },
  { "Kosisochukwu", 0.4652778 },
  { "Kosta", 1.0 },
  { "Kostandinos", 1.0 },
  { "Kostantinos", 1.0 },
  { "Kostanty", 1.0 },
  { "Kostas", 1.0 },
  { "Kosten", 1.0 },
  { "Koston", 1.0 },
  { "Kosuke", 1.0 },
  { "Kota", 0.91463417 },
  { "Kotaro", 1.0 },
  { "Kothan", 1.0 },
  { "Kother", 0.0 },
  { "Koti", 0.0 },
  { "Kotina", 0.0 },
  { "Kotomi", 0.0 },
  { "Kotryna", 0.0 },
  { "Koty", 0.93801653 },
  { "Kou", 0.95410293 },
  { "Koua", 1.0 },
  { "Kouhei", 1.0 },
  { "Kouki", 1.0 },
  { "Koula", 0.0 },
  { "Kouper", 1.0 },
  { "Kouri", 0.0 },
  { "Kourosh", 1.0 },
  { "Kourteney", 0.0 },
  { "Kourtland", 1.0 },
  { "Kourtlyn", 0.0 },
  { "Kourtlynn", 0.0 },
  { "Kourtne", 0.0 },
  { "Kourtnee", 0.0 },
  { "Kourtnei", 0.0 },
  { "Kourtney", 0.055519965 },
  { "Kourtni", 0.0 },
  { "Kourtnie", 0.0 },
  { "Kourtny", 0.0 },
  { "Koury", 1.0 },
  { "Koushik", 1.0 },
  { "Kousuke", 1.0 },
  { "Kouta", 1.0 },
  { "Kouture", 0.0 },
  { "Kova", 0.0 },
  { "Kovan", 1.0 },
  { "Kove", 1.0 },
  { "Koven", 1.0 },
  { "Kovi", 1.0 },
  { "Kovid", 1.0 },
  { "Kovin", 1.0 },
  { "Kovu", 1.0 },
  { "Kowan", 1.0 },
  { "Kowana", 0.0 },
  { "Kowanda", 0.0 },
  { "Kowanna", 0.0 },
  { "Kowen", 1.0 },
  { "Kowsar", 0.0 },
  { "Koy", 0.95532197 },
  { "Koya", 0.0 },
  { "Koyasha", 0.0 },
  { "Koye", 1.0 },
  { "Krag", 1.0 },
  { "Kraig", 1.0 },
  { "Kramer", 0.988 },
  { "Krash", 1.0 },
  { "Kratos", 1.0 },
  { "Kraven", 1.0 },
  { "Kray", 1.0 },
  { "Krayson", 1.0 },
  { "Krayton", 1.0 },
  { "Krea", 0.0 },
  { "Kree", 0.3256351 },
  { "Kreed", 1.0 },
  { "Kreelynn", 0.0 },
  { "Kreena", 0.0 },
  { "Kreg", 1.0 },
  { "Kregg", 1.0 },
  { "Kreig", 1.0 },
  { "Kreigh", 1.0 },
  { "Kreighton", 1.0 },
  { "Kreindy", 0.0 },
  { "Krenar", 1.0 },
  { "Kresha", 0.0 },
  { "Kreshnik", 1.0 },
  { "Kressida", 0.0 },
  { "Kresta", 0.0 },
  { "Kresten", 1.0 },
  { "Kreston", 1.0 },
  { "Krew", 1.0 },
  { "Krey", 1.0 },
  { "Kreyton", 1.0 },
  { "Kricia", 0.0 },
  { "Kricket", 0.0 },
  { "Kriday", 1.0 },
  { "Krieg", 1.0 },
  { "Krik", 1.0 },
  { "Krikor", 1.0 },
  { "Krillin", 1.0 },
  { "Krimson", 0.12781955 },
  { "Krimsyn", 0.0 },
  { "Krin", 0.0 },
  { "Krina", 0.0 },
  { "Kripa", 0.0 },
  { "Kris", 0.5079423 },
  { "Krisa", 0.0 },
  { "Krisalyn", 0.0 },
  { "Krisalynn", 0.0 },
  { "Krisan", 0.0 },
  { "Krisana", 0.0 },
  { "Krisandra", 0.0 },
  { "Krisann", 0.0 },
  { "Krisanna", 0.0 },
  { "Krisanne", 0.0 },
  { "Krischan", 1.0 },
  { "Krischelle", 0.0 },
  { "Kriscinda", 0.0 },
  { "Krisean", 1.0 },
  { "Kriselda", 0.0 },
  { "Kriselle", 0.0 },
  { "Krisette", 0.0 },
  { "Krish", 0.99704754 },
  { "Krisha", 0.0 },
  { "Krishaan", 1.0 },
  { "Krishal", 1.0 },
  { "Krishan", 1.0 },
  { "Krishana", 0.0 },
  { "Krishanda", 0.0 },
  { "Krishang", 1.0 },
  { "Krishani", 0.0 },
  { "Krishanna", 0.0 },
  { "Krishara", 0.0 },
  { "Krishaun", 0.87234044 },
  { "Krishauna", 0.0 },
  { "Krishav", 1.0 },
  { "Krishawn", 0.92771083 },
  { "Krishawna", 0.0 },
  { "Krishay", 1.0 },
  { "Krisheena", 0.0 },
  { "Krishell", 0.0 },
  { "Krishelle", 0.0 },
  { "Krishen", 1.0 },
  { "Krishi", 0.7 },
  { "Krishika", 0.0 },
  { "Krishil", 1.0 },
  { "Krishina", 0.0 },
  { "Krishiv", 1.0 },
  { "Krishma", 0.0 },
  { "Krishn", 1.0 },
  { "Krishna", 0.5129762 },
  { "Krishnan", 1.0 },
  { "Krishnav", 1.0 },
  { "Krishon", 1.0 },
  { "Krishona", 0.0 },
  { "Krishonda", 0.0 },
  { "Krishtian", 1.0 },
  { "Krishuna", 0.0 },
  { "Krishunna", 0.0 },
  { "Krishvi", 0.0 },
  { "Krisi", 0.0 },
  { "Krisia", 0.0 },
  { "Krisie", 0.0 },
  { "Krisily", 0.0 },
  { "Krisina", 0.0 },
  { "Krisinda", 0.0 },
  { "Krisit", 0.0 },
  { "Krisley", 0.0 },
  { "Krislin", 0.0 },
  { "Krislyn", 0.0 },
  { "Krislynn", 0.0 },
  { "Krisna", 0.12149533 },
  { "Krisopher", 1.0 },
  { "Krispin", 1.0 },
  { "Kriss", 0.5902174 },
  { "Krissa", 0.0 },
  { "Krissandra", 0.0 },
  { "Krissi", 0.0 },
  { "Krissia", 0.0 },
  { "Krissie", 0.0 },
  { "Krissinda", 0.0 },
  { "Krisslyn", 0.0 },
  { "Krisstina", 0.0 },
  { "Krisstopher", 1.0 },
  { "Krissy", 0.0 },
  { "Krist", 0.8168317 },
  { "Krista", 0.0020178608 },
  { "Kristabel", 0.0 },
  { "Kristabella", 0.0 },
  { "Kristabelle", 0.0 },
  { "Kristafer", 1.0 },
  { "Kristah", 0.0 },
  { "Kristain", 0.21782178 },
  { "Kristal", 0.0 },
  { "Kristalee", 0.0 },
  { "Kristalle", 0.0 },
  { "Kristalyn", 0.0 },
  { "Kristalynn", 0.0 },
  { "Kristamarie", 0.0 },
  { "Kristan", 0.10068315 },
  { "Kristana", 0.0 },
  { "Kristanna", 0.0 },
  { "Kristanne", 0.0 },
  { "Kristapher", 1.0 },
  { "Kristara", 0.0 },
  { "Kristasia", 0.0 },
  { "Kriste", 0.0 },
  { "Kristee", 0.0 },
  { "Kristeen", 0.0 },
  { "Kristeena", 0.0 },
  { "Kristel", 0.0 },
  { "Kristela", 0.0 },
  { "Kristell", 0.0 },
  { "Kristella", 0.0 },
  { "Kristelle", 0.0 },
  { "Kristen", 0.009563862 },
  { "Kristena", 0.0 },
  { "Kristene", 0.0 },
  { "Kristepher", 1.0 },
  { "Krister", 0.6923077 },
  { "Kristey", 0.0 },
  { "Kristhel", 0.0 },
  { "Kristhian", 1.0 },
  { "Kristi", 0.0018043226 },
  { "Kristia", 0.0 },
  { "Kristian", 0.73582953 },
  { "Kristiana", 0.0 },
  { "Kristiane", 0.0 },
  { "Kristiann", 0.0 },
  { "Kristianna", 0.0 },
  { "Kristianne", 0.0 },
  { "Kristiano", 1.0 },
  { "Kristie", 0.0011484353 },
  { "Kristieann", 0.0 },
  { "Kristien", 0.13442624 },
  { "Kristiena", 0.0 },
  { "Kristifer", 1.0 },
  { "Kristiina", 0.0 },
  { "Kristijan", 1.0 },
  { "Kristijo", 0.0 },
  { "Kristil", 0.0 },
  { "Kristilee", 0.0 },
  { "Kristilyn", 0.0 },
  { "Kristilynn", 0.0 },
  { "Kristin", 0.009050576 },
  { "Kristina", 0.0025978435 },
  { "Kristinamarie", 0.0 },
  { "Kristine", 0.0022019092 },
  { "Kristinejoy", 0.0 },
  { "Kristinia", 0.0 },
  { "Kristinn", 0.0 },
  { "Kristinna", 0.0 },
  { "Kristion", 0.9702381 },
  { "Kristiona", 0.0 },
  { "Kristjan", 1.0 },
  { "Kristjana", 0.0 },
  { "Kristl", 0.0 },
  { "Kristle", 0.0 },
  { "Kristlyn", 0.0 },
  { "Kristlynn", 0.0 },
  { "Kristn", 0.0 },
  { "Kristna", 0.0 },
  { "Kristne", 0.0 },
  { "Kristo", 1.0 },
  { "Kristobal", 1.0 },
  { "Kristof", 1.0 },
  { "Kristofe", 1.0 },
  { "Kristofer", 1.0 },
  { "Kristoff", 1.0 },
  { "Kristoffer", 1.0 },
  { "Kristofferson", 1.0 },
  { "Kristofor", 1.0 },
  { "Kristoher", 1.0 },
  { "Kristol", 0.0 },
  { "Kriston", 0.4165316 },
  { "Kristoper", 1.0 },
  { "Kristopfer", 1.0 },
  { "Kristoph", 1.0 },
  { "Kristopher", 0.99514824 },
  { "Kristophor", 1.0 },
  { "Kristos", 1.0 },
  { "Kristupas", 1.0 },
  { "Kristy", 0.0025425775 },
  { "Kristyan", 0.75 },
  { "Kristyana", 0.0 },
  { "Kristyann", 0.0 },
  { "Kristyanna", 0.0 },
  { "Kristyanne", 0.0 },
  { "Kristye", 0.0 },
  { "Kristyl", 0.0 },
  { "Kristyle", 0.0 },
  { "Kristylee", 0.0 },
  { "Kristyn", 0.0 },
  { "Kristyna", 0.0 },
  { "Kristyne", 0.0 },
  { "Krisy", 0.0 },
  { "Krisztian", 1.0 },
  { "Krisztina", 0.0 },
  { "Krithi", 0.0 },
  { "Krithik", 1.0 },
  { "Krithika", 0.0 },
  { "Krithin", 1.0 },
  { "Krithvik", 1.0 },
  { "Kriti", 0.0 },
  { "Kritika", 0.0 },
  { "Kritin", 1.0 },
  { "Kritina", 0.0 },
  { "Kritzia", 0.0 },
  { "Kriya", 0.0 },
  { "Kriyansh", 1.0 },
  { "Kriz", 1.0 },
  { "Krizia", 0.0 },
  { "Kriztian", 1.0 },
  { "Kroix", 1.0 },
  { "Kron", 1.0 },
  { "Kronos", 1.0 },
  { "Kros", 1.0 },
  { "Krosby", 0.87096775 },
  { "Kross", 1.0 },
  { "Kroy", 1.0 },
  { "Krrish", 1.0 },
  { "Krsna", 1.0 },
  { "Kru", 1.0 },
  { "Krue", 1.0 },
  { "Kruise", 1.0 },
  { "Kruize", 1.0 },
  { "Krunal", 1.0 },
  { "Krupa", 0.0 },
  { "Kruse", 1.0 },
  { "Krush", 1.0 },
  { "Krushi", 0.0 },
  { "Krustal", 0.0 },
  { "Kruthi", 0.0 },
  { "Kruti", 0.0 },
  { "Kruz", 0.97751874 },
  { "Kruze", 0.9800797 },
  { "Krya", 0.0 },
  { "Krymson", 0.12195122 },
  { "Krynn", 0.0 },
  { "Krys", 0.18965517 },
  { "Krysha", 0.0 },
  { "Krysia", 0.0 },
  { "Kryslin", 0.0 },
  { "Kryslyn", 0.0 },
  { "Kryslynn", 0.0 },
  { "Kryssa", 0.0 },
  { "Krysta", 0.0 },
  { "Krystabella", 0.0 },
  { "Krystah", 0.0 },
  { "Krystal", 0.0025614754 },
  { "Krystalann", 0.0 },
  { "Krystale", 0.0 },
  { "Krystalee", 0.0 },
  { "Krystalin", 0.0 },
  { "Krystalina", 0.0 },
  { "Krystaline", 0.0 },
  { "Krystall", 0.0 },
  { "Krystalle", 0.0 },
  { "Krystallee", 0.0 },
  { "Krystallyn", 0.0 },
  { "Krystallynn", 0.0 },
  { "Krystalmae", 0.0 },
  { "Krystalmarie", 0.0 },
  { "Krystalrose", 0.0 },
  { "Krystalyn", 0.0 },
  { "Krystalynn", 0.0 },
  { "Krystalynne", 0.0 },
  { "Krystan", 0.0 },
  { "Kryste", 0.0 },
  { "Krysteen", 0.0 },
  { "Krysteena", 0.0 },
  { "Krystel", 0.0 },
  { "Krystell", 0.0 },
  { "Krystelle", 0.0 },
  { "Krysten", 0.0 },
  { "Krystena", 0.0 },
  { "Krysti", 0.0 },
  { "Krystian", 0.74080086 },
  { "Krystiana", 0.0 },
  { "Krystianna", 0.0 },
  { "Krystie", 0.0 },
  { "Krystil", 0.0 },
  { "Krystin", 0.0 },
  { "Krystina", 0.0 },
  { "Krystine", 0.0 },
  { "Krystl", 0.0 },
  { "Krystle", 0.0033893082 },
  { "Krystn", 0.0 },
  { "Krystof", 1.0 },
  { "Krystofer", 1.0 },
  { "Krystoffer", 1.0 },
  { "Krystol", 0.0 },
  { "Kryston", 0.0 },
  { "Krystopher", 1.0 },
  { "Krysty", 0.0 },
  { "Krystyl", 0.0 },
  { "Krystyn", 0.0 },
  { "Krystyna", 0.0 },
  { "Krystyne", 0.0 },
  { "Kryton", 1.0 },
  { "Kryztal", 0.0 },
  { "Krzysztof", 1.0 },
  { "Ksandra", 0.0 },
  { "Ksean", 1.0 },
  { "Ksenia", 0.0 },
  { "Kseniya", 0.0 },
  { "Ksha", 0.0 },
  { "Kshaun", 1.0 },
  { "Kshawn", 1.0 },
  { "Kshon", 1.0 },
  { "Kson", 1.0 },
  { "Kt", 0.0 },
  { "Ktherine", 0.0 },
  { "Ktina", 0.0 },
  { "Ku", 0.8181818 },
  { "Kuan", 1.0 },
  { "Kuba", 1.0 },
  { "Kubra", 0.0 },
  { "Kubrick", 1.0 },
  { "Kudura", 1.0 },
  { "Kue", 1.0 },
  { "Kuhao", 1.0 },
  { "Kuhu", 0.0 },
  { "Kuiana", 0.0 },
  { "Kujtim", 1.0 },
  { "Kujuan", 1.0 },
  { "Kula", 0.0 },
  { "Kule", 1.0 },
  { "Kulia", 0.0 },
  { "Kullen", 1.0 },
  { "Kullin", 1.0 },
  { "Kullyn", 1.0 },
  { "Kulsoom", 0.0 },
  { "Kulsum", 0.0 },
  { "Kum", 0.0 },
  { "Kumail", 1.0 },
  { "Kumani", 0.0 },
  { "Kumar", 1.0 },
  { "Kumari", 0.21830986 },
  { "Kumasi", 1.0 },
  { "Kumba", 0.0 },
  { "Kumi", 0.0 },
  { "Kumiko", 0.0 },
  { "Kunaal", 1.0 },
  { "Kunal", 1.0 },
  { "Kundana", 0.0 },
  { "Kunga", 0.779661 },
  { "Kuniko", 0.0 },
  { "Kunio", 1.0 },
  { "Kunj", 1.0 },
  { "Kuno", 1.0 },
  { "Kunsh", 1.0 },
  { "Kunta", 0.97427654 },
  { "Kuntakinte", 1.0 },
  { "Kunte", 1.0 },
  { "Kuol", 1.0 },
  { "Kupaa", 1.0 },
  { "Kuper", 1.0 },
  { "Kupono", 1.0 },
  { "Kura", 0.0 },
  { "Kuran", 1.0 },
  { "Kurby", 1.0 },
  { "Kurdt", 1.0 },
  { "Kurie", 0.0 },
  { "Kurissa", 0.0 },
  { "Kurk", 1.0 },
  { "Kuron", 1.0 },
  { "Kurosh", 1.0 },
  { "Kurry", 1.0 },
  { "Kursten", 0.0 },
  { "Kursti", 0.0 },
  { "Kurstie", 0.0 },
  { "Kurstin", 0.0 },
  { "Kurston", 0.0 },
  { "Kursty", 0.0 },
  { "Kurstyn", 0.0 },
  { "Kurt", 0.99770373 },
  { "Kurth", 1.0 },
  { "Kurtina", 0.0 },
  { "Kurtis", 0.9985866 },
  { "Kurtisha", 0.0 },
  { "Kurtiss", 1.0 },
  { "Kurtlin", 1.0 },
  { "Kurtlyn", 0.0 },
  { "Kurtus", 1.0 },
  { "Kurumi", 0.0 },
  { "Kush", 1.0 },
  { "Kushagra", 1.0 },
  { "Kushal", 1.0 },
  { "Kushana", 0.0 },
  { "Kushi", 0.0 },
  { "Kushtrim", 1.0 },
  { "Kutana", 0.0 },
  { "Kutina", 0.0 },
  { "Kutler", 1.0 },
  { "Kutter", 1.0 },
  { "Kuuipo", 0.0 },
  { "Kuulei", 0.0 },
  { "Kuwan", 1.0 },
  { "Kuwana", 0.0 },
  { "Kuwanda", 0.0 },
  { "Kuwanna", 0.0 },
  { "Kuyper", 1.0 },
  { "Kuzey", 1.0 },
  { "Kvaughn", 1.0 },
  { "Kveon", 1.0 },
  { "Kvin", 1.0 },
  { "Kvion", 1.0 },
  { "Kvon", 1.0 },
  { "Kwabena", 1.0 },
  { "Kwabene", 1.0 },
  { "Kwadjo", 1.0 },
  { "Kwadwo", 1.0 },
  { "Kwaku", 1.0 },
  { "Kwali", 1.0 },
  { "Kwamae", 1.0 },
  { "Kwamain", 1.0 },
  { "Kwamaine", 1.0 },
  { "Kwamane", 1.0 },
  { "Kwamayne", 1.0 },
  { "Kwame", 0.98941797 },
  { "Kwamel", 1.0 },
  { "Kwami", 1.0 },
  { "Kwamin", 1.0 },
  { "Kwan", 0.94736844 },
  { "Kwana", 0.0 },
  { "Kwanda", 0.0 },
  { "Kwane", 1.0 },
  { "Kwanesha", 0.0 },
  { "Kwang", 1.0 },
  { "Kwaniqua", 0.0 },
  { "Kwanisha", 0.0 },
  { "Kwanita", 0.0 },
  { "Kwanna", 0.0 },
  { "Kwante", 1.0 },
  { "Kwanza", 0.15613383 },
  { "Kwanzaa", 0.14705883 },
  { "Kwanze", 1.0 },
  { "Kwashawn", 1.0 },
  { "Kwasi", 1.0 },
  { "Kwaun", 1.0 },
  { "Kwayera", 0.0 },
  { "Kween", 0.0 },
  { "Kweisi", 1.0 },
  { "Kweku", 1.0 },
  { "Kweli", 0.95022625 },
  { "Kwentin", 1.0 },
  { "Kwenton", 1.0 },
  { "Kwesi", 1.0 },
  { "Kwincy", 1.0 },
  { "Kwinn", 0.0 },
  { "Kwinton", 1.0 },
  { "Kwmaine", 1.0 },
  { "Kwmane", 1.0 },
  { "Kwok", 1.0 },
  { "Kwon", 1.0 },
  { "Kwuan", 1.0 },
  { "Kwynn", 0.0 },
  { "Ky", 0.90895635 },
  { "Kya", 0.002771855 },
  { "Kyah", 0.011703511 },
  { "Kyair", 1.0 },
  { "Kyaira", 0.0 },
  { "Kyairah", 0.0 },
  { "Kyaire", 0.9842767 },
  { "Kyairra", 0.0 },
  { "Kyaisha", 0.0 },
  { "Kyajah", 0.0 },
  { "Kyal", 1.0 },
  { "Kyala", 0.0 },
  { "Kyale", 1.0 },
  { "Kyalee", 0.0 },
  { "Kyalo", 1.0 },
  { "Kyan", 0.9546947 },
  { "Kyana", 0.0 },
  { "Kyanah", 0.0 },
  { "Kyandra", 0.0 },
  { "Kyandre", 1.0 },
  { "Kyani", 0.0 },
  { "Kyann", 0.015432099 },
  { "Kyanna", 0.0 },
  { "Kyannah", 0.0 },
  { "Kyanne", 0.0 },
  { "Kyante", 0.77272725 },
  { "Kyara", 0.0 },
  { "Kyarah", 0.0 },
  { "Kyari", 0.0 },
  { "Kyaria", 0.0 },
  { "Kyarra", 0.0 },
  { "Kyasha", 0.0 },
  { "Kyashia", 0.0 },
  { "Kyasia", 0.0 },
  { "Kyauna", 0.0 },
  { "Kyaw", 1.0 },
  { "Kyce", 1.0 },
  { "Kycen", 1.0 },
  { "Kycia", 0.0 },
  { "Kyda", 0.0 },
  { "Kydan", 1.0 },
  { "Kyden", 0.97711444 },
  { "Kydin", 1.0 },
  { "Kydon", 1.0 },
  { "Kye", 0.9278819 },
  { "Kyeana", 0.0 },
  { "Kyeden", 1.0 },
  { "Kyeem", 1.0 },
  { "Kyeesha", 0.0 },
  { "Kyeir", 1.0 },
  { "Kyeisha", 0.0 },
  { "Kyel", 0.9769585 },
  { "Kyela", 0.0 },
  { "Kyelan", 1.0 },
  { "Kyelar", 1.0 },
  { "Kyele", 1.0 },
  { "Kyelee", 0.0 },
  { "Kyeleigh", 0.0 },
  { "Kyeler", 1.0 },
  { "Kyell", 1.0 },
  { "Kyelle", 0.0 },
  { "Kyelynn", 0.0 },
  { "Kyen", 1.0 },
  { "Kyeon", 1.0 },
  { "Kyer", 1.0 },
  { "Kyera", 0.0 },
  { "Kyere", 1.0 },
  { "Kyeria", 0.0 },
  { "Kyeron", 1.0 },
  { "Kyerra", 0.0 },
  { "Kyersten", 0.0 },
  { "Kyerstin", 0.0 },
  { "Kyesha", 0.0 },
  { "Kyeshia", 0.0 },
  { "Kyeson", 1.0 },
  { "Kyha", 0.0 },
  { "Kyheem", 1.0 },
  { "Kyheim", 1.0 },
  { "Kyhia", 0.0 },
  { "Kyhiem", 1.0 },
  { "Kyhir", 1.0 },
  { "Kyhlee", 0.0 },
  { "Kyi", 0.70526314 },
  { "Kyia", 0.0 },
  { "Kyiah", 0.0 },
  { "Kyian", 1.0 },
  { "Kyiana", 0.0 },
  { "Kyianna", 0.0 },
  { "Kyiara", 0.0 },
  { "Kyier", 1.0 },
  { "Kyiesha", 0.0 },
  { "Kyila", 0.0 },
  { "Kyilee", 0.0 },
  { "Kyion", 1.0 },
  { "Kyir", 1.0 },
  { "Kyira", 0.0 },
  { "Kyiren", 1.0 },
  { "Kyisha", 0.0 },
  { "Kyja", 0.0 },
  { "Kyjah", 0.0 },
  { "Kyjaun", 1.0 },
  { "Kyjuan", 1.0 },
  { "Kyl", 1.0 },
  { "Kyla", 0.00076171465 },
  { "Kylaa", 0.0 },
  { "Kylah", 0.0 },
  { "Kylamarie", 0.0 },
  { "Kylan", 0.88438636 },
  { "Kylana", 0.0 },
  { "Kyland", 1.0 },
  { "Kylani", 0.0 },
  { "Kylar", 0.6855792 },
  { "Kylara", 0.0 },
  { "Kylasia", 0.0 },
  { "Kylaya", 0.0 },
  { "Kylayah", 0.0 },
  { "Kyle", 0.98226917 },
  { "Kylea", 0.0 },
  { "Kyleah", 0.0 },
  { "Kylealexander", 1.0 },
  { "Kyleanthony", 1.0 },
  { "Kylee", 0.009561597 },
  { "Kyleeann", 0.0 },
  { "Kyleek", 1.0 },
  { "Kyleemarie", 0.0 },
  { "Kyleen", 0.0 },
  { "Kyleena", 0.0 },
  { "Kyleerae", 0.0 },
  { "Kyleerose", 0.0 },
  { "Kylei", 0.0 },
  { "Kyleia", 0.0 },
  { "Kyleigh", 0.00034544303 },
  { "Kyleigha", 0.0 },
  { "Kyleighann", 0.0 },
  { "Kylel", 1.0 },
  { "Kylem", 1.0 },
  { "Kylematthew", 1.0 },
  { "Kylen", 0.844676 },
  { "Kylena", 0.0 },
  { "Kylend", 1.0 },
  { "Kylene", 0.0 },
  { "Kylenn", 0.43333334 },
  { "Kyleon", 1.0 },
  { "Kylepatrick", 1.0 },
  { "Kyler", 0.9429398 },
  { "Kylese", 0.0 },
  { "Kylesha", 0.0 },
  { "Kyley", 0.047381546 },
  { "Kyli", 0.0 },
  { "Kylia", 0.0 },
  { "Kyliah", 0.0 },
  { "Kylian", 0.94017094 },
  { "Kyliana", 0.0 },
  { "Kyliann", 0.0 },
  { "Kylianna", 0.0 },
  { "Kylianne", 0.0 },
  { "Kylicia", 0.0 },
  { "Kylie", 0.004163848 },
  { "Kylieann", 0.0 },
  { "Kylieanna", 0.0 },
  { "Kylieanne", 0.0 },
  { "Kyliee", 0.0 },
  { "Kyliegh", 0.0 },
  { "Kyliejo", 0.0 },
  { "Kyliemarie", 0.0 },
  { "Kylier", 1.0 },
  { "Kylierae", 0.0 },
  { "Kylierose", 0.0 },
  { "Kylii", 0.0 },
  { "Kylik", 1.0 },
  { "Kylil", 1.0 },
  { "Kylin", 0.5288184 },
  { "Kylina", 0.0 },
  { "Kylinn", 0.0 },
  { "Kylis", 0.0 },
  { "Kylise", 0.0 },
  { "Kylisha", 0.0 },
  { "Kylissa", 0.0 },
  { "Kyliyah", 0.0 },
  { "Kylla", 0.0 },
  { "Kylle", 0.93333334 },
  { "Kyller", 1.0 },
  { "Kyllian", 1.0 },
  { "Kyllie", 0.0 },
  { "Kylo", 0.9770492 },
  { "Kylon", 1.0 },
  { "Kylor", 0.98740554 },
  { "Kylun", 1.0 },
  { "Kyly", 0.0 },
  { "Kylyn", 0.1891892 },
  { "Kylynn", 0.01430143 },
  { "Kylynne", 0.0 },
  { "Kym", 0.11768708 },
  { "Kymani", 0.79916644 },
  { "Kymanie", 1.0 },
  { "Kymar", 1.0 },
  { "Kymara", 0.0 },
  { "Kymari", 0.6221663 },
  { "Kymarie", 0.0 },
  { "Kymarion", 1.0 },
  { "Kymarley", 1.0 },
  { "Kymauri", 1.0 },
  { "Kymaya", 0.0 },
  { "Kymbella", 0.0 },
  { "Kymber", 0.0 },
  { "Kymberle", 0.0 },
  { "Kymberlee", 0.0 },
  { "Kymberleigh", 0.0 },
  { "Kymberley", 0.0 },
  { "Kymberli", 0.0 },
  { "Kymberlie", 0.0 },
  { "Kymberly", 0.0 },
  { "Kymberlyn", 0.0 },
  { "Kymberlynn", 0.0 },
  { "Kymbra", 0.0 },
  { "Kymbre", 0.0 },
  { "Kymbree", 0.0 },
  { "Kymbria", 0.0 },
  { "Kymeir", 1.0 },
  { "Kymel", 1.0 },
  { "Kymera", 0.0 },
  { "Kymere", 1.0 },
  { "Kymesha", 0.0 },
  { "Kymeshia", 0.0 },
  { "Kymia", 0.0 },
  { "Kymiah", 0.0 },
  { "Kymier", 1.0 },
  { "Kymiere", 1.0 },
  { "Kymir", 1.0 },
  { "Kymira", 0.0 },
  { "Kymisha", 0.0 },
  { "Kymiyah", 0.0 },
  { "Kymm", 0.0 },
  { "Kymon", 1.0 },
  { "Kymoni", 0.6834171 },
  { "Kymora", 0.0 },
  { "Kymorah", 0.0 },
  { "Kymori", 0.0 },
  { "Kymorra", 0.0 },
  { "Kymya", 0.0 },
  { "Kyna", 0.0 },
  { "Kynadee", 0.0 },
  { "Kynadi", 0.0 },
  { "Kynadie", 0.0 },
  { "Kynan", 1.0 },
  { "Kynara", 0.0 },
  { "Kynda", 0.0 },
  { "Kyndahl", 0.0 },
  { "Kyndal", 0.028370515 },
  { "Kyndall", 0.035798583 },
  { "Kyndalyn", 0.0 },
  { "Kyndel", 0.0523918 },
  { "Kyndell", 0.077220075 },
  { "Kynden", 1.0 },
  { "Kynder", 0.0 },
  { "Kyndi", 0.0 },
  { "Kyndl", 0.0 },
  { "Kyndle", 0.024844721 },
  { "Kyndra", 0.0 },
  { "Kyndrah", 0.0 },
  { "Kyndrick", 1.0 },
  { "Kyndyl", 0.0 },
  { "Kynedi", 0.0 },
  { "Kyneisha", 0.0 },
  { "Kynesha", 0.0 },
  { "Kyneshia", 0.0 },
  { "Kyng", 1.0 },
  { "Kyngsten", 1.0 },
  { "Kyngston", 1.0 },
  { "Kynia", 0.0 },
  { "Kyniah", 0.0 },
  { "Kynidee", 0.0 },
  { "Kynisha", 0.0 },
  { "Kynlea", 0.0 },
  { "Kynlee", 0.0 },
  { "Kynlei", 0.0 },
  { "Kynleigh", 0.0 },
  { "Kynley", 0.0 },
  { "Kynli", 0.0 },
  { "Kynlie", 0.0 },
  { "Kynna", 0.0 },
  { "Kynnadee", 0.0 },
  { "Kynnadi", 0.0 },
  { "Kynnady", 0.0 },
  { "Kynndal", 0.0 },
  { "Kynnedi", 0.0 },
  { "Kynnedy", 0.0 },
  { "Kynnidi", 0.0 },
  { "Kynnlee", 0.0 },
  { "Kynnleigh", 0.0 },
  { "Kynnlie", 0.0 },
  { "Kynsey", 0.0 },
  { "Kynsie", 0.0 },
  { "Kynslea", 0.0 },
  { "Kynslee", 0.0 },
  { "Kynslei", 0.0 },
  { "Kynsleigh", 0.0 },
  { "Kynsley", 0.0 },
  { "Kynsli", 0.0 },
  { "Kynslie", 0.0 },
  { "Kyntrell", 1.0 },
  { "Kynzee", 0.0 },
  { "Kynzi", 0.0 },
  { "Kynzie", 0.0 },
  { "Kynzlee", 0.0 },
  { "Kynzlei", 0.0 },
  { "Kynzleigh", 0.0 },
  { "Kynzley", 0.0 },
  { "Kynzli", 0.0 },
  { "Kynzlie", 0.0 },
  { "Kyo", 1.0 },
  { "Kyohei", 1.0 },
  { "Kyoka", 0.0 },
  { "Kyoko", 0.0 },
  { "Kyomi", 0.0 },
  { "Kyon", 1.0 },
  { "Kyona", 0.0 },
  { "Kyondra", 0.0 },
  { "Kyong", 0.0 },
  { "Kyonia", 0.0 },
  { "Kyonna", 0.0 },
  { "Kyonne", 1.0 },
  { "Kyonte", 1.0 },
  { "Kyosha", 0.0 },
  { "Kyosuke", 1.0 },
  { "Kyper", 0.625 },
  { "Kypten", 1.0 },
  { "Kyptin", 1.0 },
  { "Kypton", 1.0 },
  { "Kyra", 0.0006705344 },
  { "Kyrah", 0.0 },
  { "Kyralee", 0.0 },
  { "Kyralynn", 0.0 },
  { "Kyran", 0.94684386 },
  { "Kyre", 0.93333334 },
  { "Kyrea", 0.0 },
  { "Kyree", 0.87097394 },
  { "Kyreece", 1.0 },
  { "Kyreek", 1.0 },
  { "Kyreem", 1.0 },
  { "Kyreese", 1.0 },
  { "Kyrei", 1.0 },
  { "Kyrein", 1.0 },
  { "Kyrek", 1.0 },
  { "Kyrel", 1.0 },
  { "Kyrell", 1.0 },
  { "Kyren", 0.9612711 },
  { "Kyrene", 0.0 },
  { "Kyreon", 1.0 },
  { "Kyrese", 1.0 },
  { "Kyresha", 0.0 },
  { "Kyrha", 0.0 },
  { "Kyri", 0.3751938 },
  { "Kyria", 0.0 },
  { "Kyriah", 0.0 },
  { "Kyriaki", 0.0 },
  { "Kyriakos", 1.0 },
  { "Kyrian", 0.9797571 },
  { "Kyriana", 0.0 },
  { "Kyrianna", 0.0 },
  { "Kyric", 1.0 },
  { "Kyrie", 0.5594127 },
  { "Kyriee", 0.78125 },
  { "Kyriel", 0.0 },
  { "Kyrielle", 0.0 },
  { "Kyrien", 1.0 },
  { "Kyrii", 1.0 },
  { "Kyril", 1.0 },
  { "Kyrillos", 1.0 },
  { "Kyrin", 0.89772725 },
  { "Kyrion", 1.0 },
  { "Kyrique", 1.0 },
  { "Kyris", 1.0 },
  { "Kyro", 1.0 },
  { "Kyrollos", 1.0 },
  { "Kyron", 1.0 },
  { "Kyrone", 1.0 },
  { "Kyros", 1.0 },
  { "Kyrra", 0.0 },
  { "Kyrrah", 0.0 },
  { "Kyrsha", 0.0 },
  { "Kyrsta", 0.0 },
  { "Kyrstal", 0.0 },
  { "Kyrstan", 0.0 },
  { "Kyrsten", 0.0 },
  { "Kyrsti", 0.0 },
  { "Kyrstie", 0.0 },
  { "Kyrstin", 0.0 },
  { "Kyrstle", 0.0 },
  { "Kyrston", 0.0 },
  { "Kyrstyn", 0.0 },
  { "Kyrus", 1.0 },
  { "Kyryn", 1.0 },
  { "Kysa", 0.0 },
  { "Kysan", 1.0 },
  { "Kyse", 1.0 },
  { "Kysean", 1.0 },
  { "Kyseem", 1.0 },
  { "Kysen", 1.0 },
  { "Kyser", 1.0 },
  { "Kysha", 0.0 },
  { "Kyshana", 0.0 },
  { "Kyshaun", 1.0 },
  { "Kyshawn", 1.0 },
  { "Kyshawna", 0.0 },
  { "Kyshia", 0.0 },
  { "Kyshon", 1.0 },
  { "Kysia", 0.0 },
  { "Kysin", 1.0 },
  { "Kysir", 1.0 },
  { "Kyson", 1.0 },
  { "Kystal", 0.0 },
  { "Kystin", 1.0 },
  { "Kyston", 1.0 },
  { "Kytalin", 0.0 },
  { "Kytana", 0.0 },
  { "Kyten", 1.0 },
  { "Kyton", 1.0 },
  { "Kytzia", 0.0 },
  { "Kyu", 1.0 },
  { "Kyuana", 0.0 },
  { "Kyung", 0.23893805 },
  { "Kyus", 1.0 },
  { "Kyuss", 1.0 },
  { "Kyvan", 1.0 },
  { "Kyven", 1.0 },
  { "Kyvin", 1.0 },
  { "Kyvion", 1.0 },
  { "Kyvon", 1.0 },
  { "Kywan", 1.0 },
  { "Kywanna", 0.0 },
  { "Kywaun", 1.0 },
  { "Kywon", 1.0 },
  { "Kywuan", 1.0 },
  { "Kyya", 0.0 },
  { "Kyzar", 1.0 },
  { "Kyzen", 1.0 },
  { "Kyzer", 1.0 },
  { "Kyzier", 1.0 },
  { "Kyzin", 1.0 },
  { "Kyzir", 1.0 },
  { "La", 0.17221585 },
  { "Laaibah", 0.0 },
  { "Laakea", 0.9378882 },
  { "Laangela", 0.0 },
  { "Laanthony", 1.0 },
  { "Laasia", 0.0 },
  { "Laasya", 0.0 },
  { "Laban", 1.0 },
  { "Labarbara", 0.0 },
  { "Labaron", 1.0 },
  { "Labarron", 1.0 },
  { "Labella", 0.0 },
  { "Labelle", 0.0 },
  { "Laberta", 0.0 },
  { "Labib", 1.0 },
  { "Labiba", 0.0 },
  { "Labon", 1.0 },
  { "Laborn", 1.0 },
  { "Labradford", 1.0 },
  { "Labrandon", 1.0 },
  { "Labraya", 0.0 },
  { "Labrea", 0.0 },
  { "Labree", 0.0 },
  { "Labreeska", 0.0 },
  { "Labrenda", 0.0 },
  { "Labresha", 0.0 },
  { "Labreshia", 0.0 },
  { "Labreya", 0.0 },
  { "Labreyah", 0.0 },
  { "Labria", 0.0 },
  { "Labrian", 1.0 },
  { "Labrina", 0.0 },
  { "Labrisha", 0.0 },
  { "Labritney", 0.0 },
  { "Labrittany", 0.0 },
  { "Labrittney", 0.0 },
  { "Labron", 1.0 },
  { "Labryan", 1.0 },
  { "Labryant", 1.0 },
  { "Labyron", 1.0 },
  { "Lacalvin", 1.0 },
  { "Lacandice", 0.0 },
  { "Lacandis", 0.0 },
  { "Lacara", 0.0 },
  { "Lacarla", 0.0 },
  { "Lacarlos", 1.0 },
  { "Lacasha", 0.0 },
  { "Lacasta", 0.0 },
  { "Lacci", 0.0 },
  { "Lace", 0.039702233 },
  { "Lacedric", 1.0 },
  { "Lacee", 0.0 },
  { "Lacei", 0.0 },
  { "Lacelyn", 0.0 },
  { "Lacen", 1.0 },
  { "Lacey", 0.028059296 },
  { "Lachae", 0.0 },
  { "Lachaka", 0.0 },
  { "Lachan", 0.0 },
  { "Lachana", 0.0 },
  { "Lachanda", 0.0 },
  { "Lachandra", 0.0 },
  { "Lachanta", 0.0 },
  { "Lachante", 0.0 },
  { "Lachara", 0.0 },
  { "Lacharles", 1.0 },
  { "Lachasity", 0.0 },
  { "Lachaun", 0.0 },
  { "Lachaundra", 0.0 },
  { "Lachay", 0.0 },
  { "Lache", 0.0 },
  { "Lachel", 0.0 },
  { "Lachele", 0.0 },
  { "Lachell", 0.0 },
  { "Lachelle", 0.0 },
  { "Lacher", 0.0 },
  { "Lacheryl", 0.0 },
  { "Lachic", 0.0 },
  { "Lachina", 0.0 },
  { "Lachisa", 0.0 },
  { "Lachisha", 0.0 },
  { "Lachlan", 0.9532164 },
  { "Lachlann", 1.0 },
  { "Lachlyn", 0.0 },
  { "Lachon", 0.0 },
  { "Lachonda", 0.0 },
  { "Lachone", 0.0 },
  { "Lachonne", 0.0 },
  { "Lachrisa", 0.0 },
  { "Lachrisha", 0.0 },
  { "Lachrista", 0.0 },
  { "Lachristy", 0.0 },
  { "Lachundra", 0.0 },
  { "Lachyna", 0.0 },
  { "Laci", 0.0 },
  { "Lacia", 0.0 },
  { "Laciana", 0.0 },
  { "Laciann", 0.0 },
  { "Lacie", 0.0041367086 },
  { "Lacinda", 0.0 },
  { "Lacole", 0.0 },
  { "Laconda", 0.0 },
  { "Lacondra", 0.0 },
  { "Laconia", 0.0 },
  { "Laconya", 0.0 },
  { "Lacora", 0.0 },
  { "Lacorey", 1.0 },
  { "Lacoria", 0.0 },
  { "Lacorsha", 0.0 },
  { "Lacory", 1.0 },
  { "Lacorya", 0.0 },
  { "Lacosta", 0.0 },
  { "Lacourtney", 0.0 },
  { "Lacoya", 0.0 },
  { "Lacreasha", 0.0 },
  { "Lacrecia", 0.0 },
  { "Lacree", 0.0 },
  { "Lacreesha", 0.0 },
  { "Lacreisha", 0.0 },
  { "Lacresa", 0.0 },
  { "Lacresha", 0.0 },
  { "Lacreshia", 0.0 },
  { "Lacresia", 0.0 },
  { "Lacreta", 0.0 },
  { "Lacretia", 0.0 },
  { "Lacricia", 0.0 },
  { "Lacrisha", 0.0 },
  { "Lacrissa", 0.0 },
  { "Lacrista", 0.0 },
  { "Lacrystal", 0.0 },
  { "Lacurtis", 1.0 },
  { "Lacy", 0.24876772 },
  { "Lacye", 0.0 },
  { "Lacynda", 0.0 },
  { "Lacynthia", 0.0 },
  { "Lad", 1.0 },
  { "Lada", 0.0 },
  { "Ladacia", 0.0 },
  { "Ladaesha", 0.0 },
  { "Ladaija", 0.0 },
  { "Ladaijah", 0.0 },
  { "Ladainian", 1.0 },
  { "Ladaisha", 0.0 },
  { "Ladaisia", 0.0 },
  { "Ladaja", 0.0 },
  { "Ladajah", 0.0 },
  { "Ladajia", 0.0 },
  { "Ladale", 0.9668874 },
  { "Ladamian", 1.0 },
  { "Ladamien", 1.0 },
  { "Ladamion", 1.0 },
  { "Ladan", 0.0 },
  { "Ladana", 0.0 },
  { "Ladanian", 1.0 },
  { "Ladaniel", 1.0 },
  { "Ladanien", 1.0 },
  { "Ladanna", 0.0 },
  { "Ladante", 1.0 },
  { "Ladara", 0.0 },
  { "Ladaria", 0.0 },
  { "Ladarian", 1.0 },
  { "Ladarien", 1.0 },
  { "Ladarion", 1.0 },
  { "Ladarious", 1.0 },
  { "Ladaris", 1.0 },
  { "Ladarius", 1.0 },
  { "Ladarrell", 1.0 },
  { "Ladarren", 1.0 },
  { "Ladarrian", 1.0 },
  { "Ladarrien", 1.0 },
  { "Ladarrin", 1.0 },
  { "Ladarrion", 1.0 },
  { "Ladarrious", 1.0 },
  { "Ladarris", 1.0 },
  { "Ladarrius", 1.0 },
  { "Ladarryl", 1.0 },
  { "Ladaruis", 1.0 },
  { "Ladaryl", 1.0 },
  { "Ladasha", 0.0 },
  { "Ladashia", 0.0 },
  { "Ladasia", 0.0 },
  { "Ladaveon", 1.0 },
  { "Ladavia", 0.0 },
  { "Ladavian", 1.0 },
  { "Ladavid", 1.0 },
  { "Ladavion", 1.0 },
  { "Ladawn", 0.0 },
  { "Ladawna", 0.0 },
  { "Ladaya", 0.0 },
  { "Ladayja", 0.0 },
  { "Ladaynian", 1.0 },
  { "Ladaysha", 0.0 },
  { "Ladaysia", 0.0 },
  { "Ladazia", 0.0 },
  { "Ladd", 1.0 },
  { "Laddie", 0.9468208 },
  { "Laddy", 1.0 },
  { "Ladean", 0.008726004 },
  { "Ladeana", 0.0 },
  { "Ladeane", 0.0 },
  { "Ladeanna", 0.0 },
  { "Ladedra", 0.0 },
  { "Ladeidra", 0.0 },
  { "Ladeidre", 0.0 },
  { "Ladeja", 0.0 },
  { "Ladejah", 0.0 },
  { "Ladell", 0.6978367 },
  { "Ladelle", 0.0 },
  { "Ladema", 0.0 },
  { "Laden", 1.0 },
  { "Ladena", 0.0 },
  { "Ladene", 0.0 },
  { "Ladenna", 0.0 },
  { "Laderek", 1.0 },
  { "Ladereon", 1.0 },
  { "Laderian", 1.0 },
  { "Laderick", 1.0 },
  { "Laderion", 1.0 },
  { "Laderious", 1.0 },
  { "Laderius", 1.0 },
  { "Laderrian", 1.0 },
  { "Laderrica", 0.0 },
  { "Laderrick", 1.0 },
  { "Laderricka", 0.0 },
  { "Laderrion", 1.0 },
  { "Laderrius", 1.0 },
  { "Ladesha", 0.0 },
  { "Ladessa", 0.0 },
  { "Ladestiny", 0.0 },
  { "Ladetra", 0.0 },
  { "Ladia", 0.0 },
  { "Ladiamond", 0.0 },
  { "Ladie", 0.0 },
  { "Ladijah", 0.0 },
  { "Ladimer", 1.0 },
  { "Ladina", 0.0 },
  { "Ladine", 0.0 },
  { "Ladislado", 1.0 },
  { "Ladislao", 1.0 },
  { "Ladislaus", 1.0 },
  { "Ladislav", 1.0 },
  { "Ladislava", 0.0 },
  { "Ladon", 0.78219825 },
  { "Ladona", 0.0 },
  { "Ladonda", 0.0 },
  { "Ladonia", 0.0 },
  { "Ladonja", 0.0 },
  { "Ladonn", 0.0 },
  { "Ladonna", 0.0 },
  { "Ladonne", 0.0 },
  { "Ladonnia", 0.0 },
  { "Ladonta", 1.0 },
  { "Ladontae", 1.0 },
  { "Ladontay", 1.0 },
  { "Ladonte", 1.0 },
  { "Ladonya", 0.0 },
  { "Ladora", 0.0 },
  { "Ladoris", 0.0 },
  { "Ladre", 1.0 },
  { "Ladrea", 0.0 },
  { "Ladreama", 0.0 },
  { "Ladreka", 0.0 },
  { "Ladrena", 0.0 },
  { "Ladrick", 1.0 },
  { "Ladrina", 0.0 },
  { "Ladson", 1.0 },
  { "Lady", 0.0 },
  { "Ladye", 0.0 },
  { "Laeh", 0.0 },
  { "Laeisha", 0.0 },
  { "Laeken", 0.0 },
  { "Laekyn", 0.0 },
  { "Lael", 0.25887096 },
  { "Laela", 0.0 },
  { "Laelah", 0.0 },
  { "Laelani", 0.0 },
  { "Laelia", 0.0 },
  { "Laella", 0.0 },
  { "Laelle", 0.0 },
  { "Laeloni", 0.0 },
  { "Laelyn", 0.0 },
  { "Laelynn", 0.0 },
  { "Laena", 0.0 },
  { "Laerica", 0.0 },
  { "Laesha", 0.0 },
  { "Laetitia", 0.0 },
  { "Laeton", 1.0 },
  { "Laetyn", 0.33333334 },
  { "Laeuna", 0.0 },
  { "Laeyah", 0.0 },
  { "Lafarrah", 0.0 },
  { "Lafaun", 0.0 },
  { "Lafawn", 0.0 },
  { "Lafay", 0.0 },
  { "Lafaye", 0.0 },
  { "Lafayette", 0.99560946 },
  { "Lafe", 1.0 },
  { "Lafern", 0.0 },
  { "Laferne", 0.0 },
  { "Lafeyette", 1.0 },
  { "Lafiamma", 1.0 },
  { "Lafon", 0.47368422 },
  { "Lafonda", 0.0 },
  { "Lafondra", 0.0 },
  { "Lafonya", 0.0 },
  { "Lafonzo", 1.0 },
  { "Lafoya", 0.0 },
  { "Lafran", 0.0 },
  { "Lafrance", 0.3258427 },
  { "Lafrances", 0.0 },
  { "Lafrancine", 0.0 },
  { "Lafreda", 0.0 },
  { "Lafrederick", 1.0 },
  { "Lafredrick", 1.0 },
  { "Lagail", 0.0 },
  { "Lagarius", 1.0 },
  { "Lagatha", 0.0 },
  { "Lagena", 0.0 },
  { "Lagenia", 0.0 },
  { "Lagertha", 0.0 },
  { "Lagina", 0.0 },
  { "Lagrand", 1.0 },
  { "Lagregory", 1.0 },
  { "Lagreta", 0.0 },
  { "Lagretta", 0.0 },
  { "Laguan", 1.0 },
  { "Laguana", 0.0 },
  { "Laguanda", 0.0 },
  { "Laguisha", 0.0 },
  { "Laguita", 0.0 },
  { "Lah", 0.0 },
  { "Lahana", 0.0 },
  { "Lahari", 0.0 },
  { "Lahela", 0.0 },
  { "Lahla", 0.0 },
  { "Lahna", 0.0 },
  { "Lahni", 0.0 },
  { "Lahoma", 0.0 },
  { "Lahonda", 0.0 },
  { "Lai", 0.09727626 },
  { "Laia", 0.0 },
  { "Laiah", 0.0 },
  { "Laiana", 0.0 },
  { "Laianna", 0.0 },
  { "Laiba", 0.0 },
  { "Laibah", 0.0 },
  { "Laicee", 0.0 },
  { "Laicey", 0.0 },
  { "Laici", 0.0 },
  { "Laida", 0.0 },
  { "Laiden", 1.0 },
  { "Laidy", 0.0 },
  { "Laif", 1.0 },
  { "Laighla", 0.0 },
  { "Laighton", 0.0 },
  { "Laihla", 0.0 },
  { "Laik", 1.0 },
  { "Laika", 0.0 },
  { "Laike", 1.0 },
  { "Laiken", 0.15425889 },
  { "Laikin", 0.0 },
  { "Laiklyn", 0.0 },
  { "Laiklynn", 0.0 },
  { "Laikyn", 0.0064935065 },
  { "Laikynn", 0.0 },
  { "Laila", 0.00070893567 },
  { "Lailaa", 0.0 },
  { "Lailah", 0.0 },
  { "Lailamarie", 0.0 },
  { "Lailana", 0.0 },
  { "Lailanee", 0.0 },
  { "Lailani", 0.0 },
  { "Lailanie", 0.0 },
  { "Lailanni", 0.0 },
  { "Lailany", 0.0 },
  { "Lailarose", 0.0 },
  { "Lailee", 0.0 },
  { "Laileen", 0.0 },
  { "Laileigh", 0.0 },
  { "Lailene", 0.0 },
  { "Lailey", 0.0 },
  { "Laili", 0.0 },
  { "Lailiana", 0.0 },
  { "Lailie", 0.0 },
  { "Lailoni", 0.0 },
  { "Lailonie", 0.0 },
  { "Lailonnie", 0.0 },
  { "Laily", 0.0 },
  { "Lailyn", 0.0 },
  { "Lailynn", 0.0 },
  { "Laim", 1.0 },
  { "Laima", 0.0 },
  { "Lain", 0.9480519 },
  { "Laina", 0.0 },
  { "Lainah", 0.0 },
  { "Laine", 0.32638732 },
  { "Lainee", 0.0 },
  { "Lainey", 0.0 },
  { "Laini", 0.0 },
  { "Lainie", 0.0 },
  { "Lainy", 0.0 },
  { "Laiona", 0.0 },
  { "Laionna", 0.0 },
  { "Laira", 0.0 },
  { "Laird", 1.0 },
  { "Lairen", 0.0 },
  { "Lais", 0.0 },
  { "Laisa", 0.0 },
  { "Laisha", 0.0 },
  { "Laison", 1.0 },
  { "Laissa", 0.0 },
  { "Laiten", 1.0 },
  { "Laith", 1.0 },
  { "Laithan", 1.0 },
  { "Laithen", 1.0 },
  { "Laiton", 1.0 },
  { "Laityn", 0.03649635 },
  { "Laiya", 0.0 },
  { "Laiyah", 0.0 },
  { "Laiyana", 0.0 },
  { "Laiyanna", 0.0 },
  { "Laiyla", 0.0 },
  { "Laiylah", 0.0 },
  { "Laiza", 0.0 },
  { "Lajace", 1.0 },
  { "Lajada", 0.0 },
  { "Lajae", 0.0 },
  { "Lajarvis", 1.0 },
  { "Lajasmine", 0.0 },
  { "Lajauna", 0.0 },
  { "Lajaune", 1.0 },
  { "Lajavion", 1.0 },
  { "Lajaya", 0.0 },
  { "Lajayceon", 1.0 },
  { "Lajayla", 0.0 },
  { "Lajean", 0.0 },
  { "Lajeana", 0.0 },
  { "Lajeanna", 0.0 },
  { "Lajeanne", 0.0 },
  { "Lajerrica", 0.0 },
  { "Lajessica", 0.0 },
  { "Lajeune", 0.0 },
  { "Lajla", 0.0 },
  { "Lajohn", 1.0 },
  { "Lajoi", 0.0 },
  { "Lajoia", 0.0 },
  { "Lajon", 1.0 },
  { "Lajoy", 0.0 },
  { "Lajoya", 0.0 },
  { "Lajoyce", 0.0 },
  { "Lajuan", 0.58738136 },
  { "Lajuana", 0.0 },
  { "Lajuanda", 0.0 },
  { "Lajuane", 1.0 },
  { "Lajuanna", 0.0 },
  { "Lajune", 0.0 },
  { "Lakai", 0.86292833 },
  { "Lakaia", 0.0 },
  { "Lakaii", 0.0 },
  { "Lakaila", 0.0 },
  { "Lakaisha", 0.0 },
  { "Lakaiya", 0.0 },
  { "Lakala", 0.0 },
  { "Lakan", 0.0 },
  { "Lakara", 0.0 },
  { "Lakari", 1.0 },
  { "Lakasha", 0.0 },
  { "Lakashia", 0.0 },
  { "Lakay", 0.0 },
  { "Lakaya", 0.0 },
  { "Lakayden", 1.0 },
  { "Lakayla", 0.0 },
  { "Lakaysha", 0.0 },
  { "Lake", 0.8226164 },
  { "Lakea", 0.0 },
  { "Lakeah", 0.0 },
  { "Lakeasha", 0.0 },
  { "Lakeashia", 0.0 },
  { "Lakeba", 0.0 },
  { "Lakecha", 0.0 },
  { "Lakechia", 0.0 },
  { "Lakecia", 0.0 },
  { "Lakeda", 0.0 },
  { "Lakedia", 0.0 },
  { "Lakedra", 0.0 },
  { "Lakedria", 0.0 },
  { "Lakedrick", 1.0 },
  { "Lakee", 0.46153846 },
  { "Lakeea", 0.0 },
  { "Lakeela", 0.0 },
  { "Lakeem", 1.0 },
  { "Lakeema", 0.0 },
  { "Lakeena", 0.0 },
  { "Lakeesha", 0.0 },
  { "Lakeeshia", 0.0 },
  { "Lakeeta", 0.0 },
  { "Lakeetha", 0.0 },
  { "Lakeeva", 0.0 },
  { "Lakeeya", 0.0 },
  { "Lakeia", 0.0 },
  { "Lakeicha", 0.0 },
  { "Lakeida", 0.0 },
  { "Lakeidra", 0.0 },
  { "Lakeila", 0.0 },
  { "Lakeira", 0.0 },
  { "Lakeisa", 0.0 },
  { "Lakeish", 0.0 },
  { "Lakeisha", 0.002805049 },
  { "Lakeishia", 0.0 },
  { "Lakeita", 0.0 },
  { "Lakeith", 1.0 },
  { "Lakeitha", 0.0 },
  { "Lakeithia", 0.0 },
  { "Lakeitra", 0.0 },
  { "Lakeiya", 0.0 },
  { "Lakel", 1.0 },
  { "Lakela", 0.0 },
  { "Lakelan", 1.0 },
  { "Lakeland", 0.8684211 },
  { "Lakelee", 0.0 },
  { "Lakeley", 0.0 },
  { "Lakelia", 0.0 },
  { "Lakell", 1.0 },
  { "Lakelsha", 0.0 },
  { "Lakely", 0.0 },
  { "Lakelyn", 0.0076219514 },
  { "Lakelynn", 0.0 },
  { "Lakema", 0.0 },
  { "Lakemia", 0.0 },
  { "Laken", 0.19759862 },
  { "Lakena", 0.0 },
  { "Lakenda", 0.0 },
  { "Lakendra", 0.0 },
  { "Lakendria", 0.0 },
  { "Lakendric", 1.0 },
  { "Lakendrick", 1.0 },
  { "Lakenia", 0.0 },
  { "Lakenya", 0.0 },
  { "Lakenzi", 0.0 },
  { "Lakenzie", 0.0 },
  { "Laker", 1.0 },
  { "Lakera", 0.0 },
  { "Lakeria", 0.0 },
  { "Lakerria", 0.0 },
  { "Lakersha", 0.0 },
  { "Lakesa", 0.0 },
  { "Lakesh", 0.0 },
  { "Lakesha", 0.00073403475 },
  { "Lakeshea", 0.0 },
  { "Lakeshi", 0.0 },
  { "Lakeshia", 0.0 },
  { "Lakesia", 0.0 },
  { "Lakesiha", 0.0 },
  { "Lakeska", 0.0 },
  { "Lakessa", 0.0 },
  { "Lakessha", 0.0 },
  { "Laketa", 0.0 },
  { "Laketha", 0.0 },
  { "Lakethia", 0.0 },
  { "Laketia", 0.0 },
  { "Laketra", 0.0 },
  { "Laketria", 0.0 },
  { "Laketta", 0.0 },
  { "Lakeva", 0.0 },
  { "Lakevia", 0.0 },
  { "Lakey", 0.0 },
  { "Lakeya", 0.0 },
  { "Lakeyah", 0.0 },
  { "Lakeycha", 0.0 },
  { "Lakeyda", 0.0 },
  { "Lakeydra", 0.0 },
  { "Lakeyia", 0.0 },
  { "Lakeyla", 0.0 },
  { "Lakeyn", 0.0 },
  { "Lakeysa", 0.0 },
  { "Lakeysha", 0.0 },
  { "Lakeyshia", 0.0 },
  { "Lakeysia", 0.0 },
  { "Lakeyta", 0.0 },
  { "Lakhi", 1.0 },
  { "Lakhia", 0.0 },
  { "Laki", 1.0 },
  { "Lakia", 0.0 },
  { "Lakiah", 0.0 },
  { "Lakiara", 0.0 },
  { "Lakiaya", 0.0 },
  { "Lakicha", 0.0 },
  { "Lakicia", 0.0 },
  { "Lakida", 0.0 },
  { "Lakie", 0.0 },
  { "Lakiea", 0.0 },
  { "Lakiera", 0.0 },
  { "Lakiesha", 0.0 },
  { "Lakieshia", 0.0 },
  { "Lakieta", 0.0 },
  { "Lakieth", 1.0 },
  { "Lakietha", 0.0 },
  { "Lakika", 0.0 },
  { "Lakila", 0.0 },
  { "Lakim", 1.0 },
  { "Lakima", 0.0 },
  { "Lakimberly", 0.0 },
  { "Lakin", 0.17323601 },
  { "Lakina", 0.0 },
  { "Lakindra", 0.0 },
  { "Laking", 1.0 },
  { "Lakinia", 0.0 },
  { "Lakinya", 0.0 },
  { "Lakinzi", 0.0 },
  { "Lakira", 0.0 },
  { "Lakisa", 0.0 },
  { "Lakischa", 0.0 },
  { "Lakish", 0.0 },
  { "Lakisha", 0.00097410503 },
  { "Lakishia", 0.0 },
  { "Lakisia", 0.0 },
  { "Lakiska", 0.0 },
  { "Lakissa", 0.0 },
  { "Lakissia", 0.0 },
  { "Lakista", 0.0 },
  { "Lakita", 0.0 },
  { "Lakitha", 0.0 },
  { "Lakitia", 0.0 },
  { "Lakitra", 0.0 },
  { "Lakitsha", 0.0 },
  { "Lakitta", 0.0 },
  { "Lakiva", 0.0 },
  { "Lakiya", 0.0 },
  { "Lakiyah", 0.0 },
  { "Lakizzy", 0.0 },
  { "Lakken", 0.0 },
  { "Lakley", 0.0 },
  { "Laklyn", 0.0 },
  { "Laklynn", 0.0 },
  { "Lakoda", 0.6486486 },
  { "Lakosha", 0.0 },
  { "Lakota", 0.47608024 },
  { "Lakotah", 0.0 },
  { "Lakoya", 0.0 },
  { "Lakreisha", 0.0 },
  { "Lakresha", 0.0 },
  { "Lakreshia", 0.0 },
  { "Lakrisha", 0.0 },
  { "Lakrista", 0.0 },
  { "Lakrystal", 0.0 },
  { "Laksh", 1.0 },
  { "Laksha", 0.0 },
  { "Lakshan", 1.0 },
  { "Lakshana", 0.0 },
  { "Lakshita", 0.0 },
  { "Lakshman", 1.0 },
  { "Lakshmi", 0.0 },
  { "Lakshmy", 0.0 },
  { "Lakshya", 0.59770113 },
  { "Laksmi", 0.0 },
  { "Lakwan", 1.0 },
  { "Lakwanzaa", 0.0 },
  { "Lakya", 0.0 },
  { "Lakyah", 0.0 },
  { "Lakye", 1.0 },
  { "Lakyia", 0.0 },
  { "Lakyla", 0.0 },
  { "Lakyn", 0.025728155 },
  { "Lakyna", 0.0 },
  { "Lakyndra", 0.0 },
  { "Lakynn", 0.0 },
  { "Lakyra", 0.0 },
  { "Lakyrah", 0.0 },
  { "Lakyria", 0.0 },
  { "Lakysha", 0.0 },
  { "Lakyshia", 0.0 },
  { "Lal", 0.7826087 },
  { "Lala", 0.0 },
  { "Lalah", 0.0 },
  { "Lalaina", 0.0 },
  { "Lalaine", 0.0 },
  { "Lalainia", 0.0 },
  { "Lalana", 0.0 },
  { "Lalani", 0.0 },
  { "Lalania", 0.0 },
  { "Lalanya", 0.0 },
  { "Lalar", 0.0 },
  { "Lale", 0.0 },
  { "Laleah", 0.0 },
  { "Laleh", 0.0 },
  { "Lalelei", 0.0 },
  { "Lalena", 0.0 },
  { "Lalenia", 0.0 },
  { "Laleta", 0.0 },
  { "Lali", 0.0 },
  { "Lalia", 0.0 },
  { "Laliah", 0.0 },
  { "Laliana", 0.0 },
  { "Lalicia", 0.0 },
  { "Lalie", 0.0 },
  { "Lalinda", 0.0 },
  { "Laline", 0.0 },
  { "Lalisa", 0.0 },
  { "Lalisha", 0.0 },
  { "Lalit", 1.0 },
  { "Lalita", 0.0 },
  { "Lalith", 1.0 },
  { "Lalitha", 0.0 },
  { "Laliyah", 0.0 },
  { "Lalla", 0.0 },
  { "Lallah", 0.0 },
  { "Lallie", 0.0 },
  { "Lalo", 1.0 },
  { "Laloni", 0.0 },
  { "Lalonnie", 0.0 },
  { "Lam", 0.8950178 },
  { "Lama", 0.0 },
  { "Lamaar", 1.0 },
  { "Lamae", 0.0 },
  { "Lamaine", 1.0 },
  { "Lamaiya", 0.0 },
  { "Lamaiyah", 0.0 },
  { "Lamaj", 1.0 },
  { "Laman", 1.0 },
  { "Lamanda", 0.0 },
  { "Lamani", 0.0 },
  { "Lamanuel", 1.0 },
  { "Lamar", 0.9684685 },
  { "Lamara", 0.0 },
  { "Lamarc", 1.0 },
  { "Lamarco", 1.0 },
  { "Lamarcus", 1.0 },
  { "Lamare", 1.0 },
  { "Lamarea", 0.0 },
  { "Lamareon", 1.0 },
  { "Lamari", 0.6512195 },
  { "Lamaria", 0.0 },
  { "Lamariah", 0.0 },
  { "Lamarian", 1.0 },
  { "Lamariana", 0.0 },
  { "Lamarianna", 0.0 },
  { "Lamarie", 0.0 },
  { "Lamario", 1.0 },
  { "Lamarion", 1.0 },
  { "Lamarious", 1.0 },
  { "Lamaris", 1.0 },
  { "Lamarius", 1.0 },
  { "Lamariya", 0.0 },
  { "Lamariyah", 0.0 },
  { "Lamark", 1.0 },
  { "Lamarkus", 1.0 },
  { "Lamarquis", 1.0 },
  { "Lamarqus", 1.0 },
  { "Lamarr", 0.98932767 },
  { "Lamarra", 0.0 },
  { "Lamarria", 0.0 },
  { "Lamarrion", 1.0 },
  { "Lamart", 1.0 },
  { "Lamarus", 1.0 },
  { "Lamauri", 1.0 },
  { "Lamaya", 0.0 },
  { "Lamayah", 0.0 },
  { "Lamayia", 0.0 },
  { "Lamb", 1.0 },
  { "Lambert", 1.0 },
  { "Lamberto", 1.0 },
  { "Lambros", 1.0 },
  { "Lamea", 0.0 },
  { "Lameca", 0.0 },
  { "Lamecca", 0.0 },
  { "Lamech", 1.0 },
  { "Lamecia", 0.0 },
  { "Lameek", 1.0 },
  { "Lameeka", 0.0 },
  { "Lameen", 1.0 },
  { "Lameer", 1.0 },
  { "Lamees", 0.0 },
  { "Lameese", 0.0 },
  { "Lameika", 0.0 },
  { "Lameir", 1.0 },
  { "Lameisha", 0.0 },
  { "Lameka", 0.0 },
  { "Lamekia", 0.0 },
  { "Lamel", 1.0 },
  { "Lamell", 1.0 },
  { "Lamera", 0.0 },
  { "Lamere", 1.0 },
  { "Lameria", 0.0 },
  { "Lamerle", 0.0 },
  { "Lamesa", 0.0 },
  { "Lamese", 0.0 },
  { "Lamesha", 0.0 },
  { "Lameshia", 0.0 },
  { "Lametra", 0.0 },
  { "Lametria", 0.0 },
  { "Lametrice", 0.0 },
  { "Lameya", 0.0 },
  { "Lamia", 0.0 },
  { "Lamiah", 0.0 },
  { "Lamica", 0.0 },
  { "Lamichael", 1.0 },
  { "Lamicheal", 1.0 },
  { "Lamiek", 1.0 },
  { "Lamier", 1.0 },
  { "Lamija", 0.0 },
  { "Lamika", 0.0 },
  { "Lamin", 1.0 },
  { "Lamine", 1.0 },
  { "Lamir", 1.0 },
  { "Lamira", 0.0 },
  { "Lamiracle", 0.0 },
  { "Lamirah", 0.0 },
  { "Lamirra", 0.0 },
  { "Lamis", 0.0 },
  { "Lamisa", 0.0 },
  { "Lamisha", 0.0 },
  { "Lamiya", 0.0 },
  { "Lamiyah", 0.0 },
  { "Lamoine", 0.7955665 },
  { "Lamon", 1.0 },
  { "Lamona", 0.0 },
  { "Lamond", 1.0 },
  { "Lamonda", 0.0 },
  { "Lamondo", 1.0 },
  { "Lamondre", 1.0 },
  { "Lamone", 1.0 },
  { "Lamoni", 0.3 },
  { "Lamonica", 0.0035971224 },
  { "Lamonique", 0.0 },
  { "Lamont", 0.99298114 },
  { "Lamonta", 1.0 },
  { "Lamontae", 1.0 },
  { "Lamontay", 1.0 },
  { "Lamonte", 1.0 },
  { "Lamontez", 1.0 },
  { "Lamontray", 1.0 },
  { "Lamontre", 1.0 },
  { "Lamor", 1.0 },
  { "Lamora", 0.0 },
  { "Lamoria", 0.0 },
  { "Lamorris", 1.0 },
  { "Lamount", 1.0 },
  { "Lamour", 1.0 },
  { "Lamoyne", 0.64953274 },
  { "Lamuel", 1.0 },
  { "Lamya", 0.0 },
  { "Lamyah", 0.0 },
  { "Lamyia", 0.0 },
  { "Lamyiah", 0.0 },
  { "Lamyla", 0.0 },
  { "Lamyra", 0.0 },
  { "Lamyrah", 0.0 },
  { "Lan", 0.2631068 },
  { "Lana", 0.00029891197 },
  { "Lanadia", 0.0 },
  { "Lanae", 0.0 },
  { "Lanaea", 0.0 },
  { "Lanaeya", 0.0 },
  { "Lanah", 0.0 },
  { "Lanai", 0.0 },
  { "Lanaia", 0.0 },
  { "Lanaija", 0.0 },
  { "Lanaijah", 0.0 },
  { "Lanaisha", 0.0 },
  { "Lanaiya", 0.0 },
  { "Lanaiyah", 0.0 },
  { "Lanaja", 0.0 },
  { "Lanajah", 0.0 },
  { "Lanajia", 0.0 },
  { "Lananh", 0.0 },
  { "Lanard", 1.0 },
  { "Lanaria", 0.0 },
  { "Lanarose", 0.0 },
  { "Lanasha", 0.0 },
  { "Lanashia", 0.0 },
  { "Lanasia", 0.0 },
  { "Lanautica", 0.0 },
  { "Lanavia", 0.0 },
  { "Lanay", 0.0 },
  { "Lanaya", 0.0 },
  { "Lanayah", 0.0 },
  { "Lanaye", 0.0 },
  { "Lanayia", 0.0 },
  { "Lanayja", 0.0 },
  { "Lanaysha", 0.0 },
  { "Lanayshia", 0.0 },
  { "Lanaysia", 0.0 },
  { "Lanazia", 0.0 },
  { "Lancaster", 1.0 },
  { "Lance", 0.9966566 },
  { "Lancelot", 1.0 },
  { "Lancer", 1.0 },
  { "Lanceton", 1.0 },
  { "Lancing", 1.0 },
  { "Lancy", 1.0 },
  { "Landa", 0.0 },
  { "Landan", 0.9976145 },
  { "Landdon", 1.0 },
  { "Landee", 0.0 },
  { "Landel", 1.0 },
  { "Landell", 1.0 },
  { "Landen", 0.9920995 },
  { "Landenn", 1.0 },
  { "Lander", 1.0 },
  { "Landers", 1.0 },
  { "Landi", 0.0 },
  { "Landin", 0.9932745 },
  { "Landis", 0.88945365 },
  { "Lando", 1.0 },
  { "Landon", 0.99468434 },
  { "Landonn", 1.0 },
  { "Landra", 0.0 },
  { "Landrea", 0.0 },
  { "Landree", 0.020183487 },
  { "Landrey", 0.0846395 },
  { "Landri", 0.0 },
  { "Landria", 0.0 },
  { "Landric", 1.0 },
  { "Landrick", 1.0 },
  { "Landrie", 0.0 },
  { "Landris", 1.0 },
  { "Landrum", 1.0 },
  { "Landry", 0.54493386 },
  { "Landun", 1.0 },
  { "Landus", 1.0 },
  { "Landy", 0.7120482 },
  { "Landyn", 0.9065952 },
  { "Landynn", 0.78431374 },
  { "Landyon", 1.0 },
  { "Lane", 0.9055174 },
  { "Lanea", 0.0 },
  { "Laneah", 0.0 },
  { "Lanease", 0.0 },
  { "Lanece", 0.0 },
  { "Lanecia", 0.0 },
  { "Lanedra", 0.0 },
  { "Lanee", 0.0 },
  { "Laneeka", 0.0 },
  { "Laneesha", 0.0 },
  { "Laneice", 0.0 },
  { "Laneigh", 0.0 },
  { "Laneika", 0.0 },
  { "Laneise", 0.0 },
  { "Laneisha", 0.0 },
  { "Laneita", 0.0 },
  { "Laneka", 0.0 },
  { "Lanekia", 0.0 },
  { "Lanel", 0.15151516 },
  { "Lanelda", 0.0 },
  { "Lanell", 0.14152473 },
  { "Lanelle", 0.0 },
  { "Lanena", 0.0 },
  { "Lanequa", 0.0 },
  { "Lanequia", 0.0 },
  { "Lanesa", 0.0 },
  { "Lanese", 0.0 },
  { "Lanesha", 0.0 },
  { "Laneshia", 0.0 },
  { "Lanesia", 0.0 },
  { "Lanessa", 0.0 },
  { "Lanet", 0.0 },
  { "Laneta", 0.0 },
  { "Lanetra", 0.0 },
  { "Lanett", 0.0 },
  { "Lanetta", 0.0 },
  { "Lanette", 0.0 },
  { "Laneva", 0.0 },
  { "Laney", 0.031981725 },
  { "Laneya", 0.0 },
  { "Laneyah", 0.0 },
  { "Lanford", 1.0 },
  { "Lang", 0.9885714 },
  { "Langdon", 0.99055713 },
  { "Lange", 1.0 },
  { "Langford", 1.0 },
  { "Langley", 0.23427673 },
  { "Langston", 0.94545454 },
  { "Langstyn", 0.0 },
  { "Lanham", 1.0 },
  { "Lani", 0.025663981 },
  { "Lania", 0.0 },
  { "Laniah", 0.0 },
  { "Laniakea", 0.0 },
  { "Laniaya", 0.0 },
  { "Lanica", 0.0 },
  { "Lanice", 0.0 },
  { "Lanicia", 0.0 },
  { "Lanicka", 0.0 },
  { "Lanie", 0.0073746312 },
  { "Laniece", 0.0 },
  { "Lanier", 0.89990985 },
  { "Laniesha", 0.0 },
  { "Lanieya", 0.0 },
  { "Lanija", 0.0 },
  { "Lanijah", 0.0 },
  { "Lanika", 0.0 },
  { "Lanikai", 0.0 },
  { "Lanikki", 0.0 },
  { "Lanina", 0.0 },
  { "Laniqua", 0.0 },
  { "Lanique", 0.0 },
  { "Lanis", 0.7542373 },
  { "Lanisa", 0.0 },
  { "Lanise", 0.0 },
  { "Lanisha", 0.0 },
  { "Lanishia", 0.0 },
  { "Lanissa", 0.0 },
  { "Lanita", 0.0 },
  { "Lanitra", 0.0 },
  { "Laniya", 0.0 },
  { "Laniyah", 0.0 },
  { "Laniyha", 0.0 },
  { "Laniyia", 0.0 },
  { "Lank", 1.0 },
  { "Lankford", 1.0 },
  { "Lann", 1.0 },
  { "Lanna", 0.0 },
  { "Lannah", 0.0 },
  { "Lanndon", 1.0 },
  { "Lannes", 1.0 },
  { "Lannetta", 0.0 },
  { "Lannette", 0.0 },
  { "Lanney", 1.0 },
  { "Lanni", 0.0 },
  { "Lannie", 0.5510083 },
  { "Lanning", 1.0 },
  { "Lannis", 0.81935483 },
  { "Lannon", 1.0 },
  { "Lanny", 0.99303246 },
  { "Lanor", 0.0 },
  { "Lanora", 0.0 },
  { "Lanore", 0.0 },
  { "Lanoris", 1.0 },
  { "Lanorris", 1.0 },
  { "Lansana", 1.0 },
  { "Lansing", 1.0 },
  { "Lanson", 1.0 },
  { "Lanston", 1.0 },
  { "Lanta", 0.0 },
  { "Lantanya", 0.0 },
  { "Lanty", 1.0 },
  { "Lantz", 1.0 },
  { "Lanvy", 0.0 },
  { "Lanxton", 1.0 },
  { "Lany", 0.10869565 },
  { "Lanya", 0.0 },
  { "Lanyah", 0.0 },
  { "Lanyard", 1.0 },
  { "Lanye", 0.0 },
  { "Lanyia", 0.0 },
  { "Lanyiah", 0.0 },
  { "Lanyjah", 0.0 },
  { "Lanyla", 0.0 },
  { "Lanylah", 0.0 },
  { "Lanyra", 0.0 },
  { "Lanz", 1.0 },
  { "Lao", 1.0 },
  { "Laoma", 0.0 },
  { "Laparis", 0.034722224 },
  { "Lapatra", 0.0 },
  { "Lapatrick", 1.0 },
  { "Lapaul", 1.0 },
  { "Lapearl", 0.0 },
  { "Laphonso", 1.0 },
  { "Laporcha", 0.0 },
  { "Laporche", 0.0 },
  { "Laporchea", 0.0 },
  { "Laporchia", 0.0 },
  { "Laporscha", 0.0 },
  { "Laporsche", 0.0 },
  { "Laporschia", 0.0 },
  { "Laporsha", 0.0 },
  { "Laporshe", 0.0 },
  { "Laporshia", 0.0 },
  { "Laportia", 0.0 },
  { "Lapreal", 0.0 },
  { "Laprecious", 0.0 },
  { "Lapresha", 0.0 },
  { "Lapria", 0.0 },
  { "Lapriel", 0.0 },
  { "Lapriest", 1.0 },
  { "Laprincess", 0.0 },
  { "Laprincia", 0.0 },
  { "Laqresha", 0.0 },
  { "Laquail", 0.0 },
  { "Laquain", 1.0 },
  { "Laquaisha", 0.0 },
  { "Laquala", 0.0 },
  { "Laquan", 0.9144659 },
  { "Laquana", 0.0 },
  { "Laquanda", 0.0 },
  { "Laquandra", 0.0 },
  { "Laquandria", 0.0 },
  { "Laquane", 1.0 },
  { "Laquania", 0.0 },
  { "Laquann", 1.0 },
  { "Laquanna", 0.0 },
  { "Laquanta", 0.0 },
  { "Laquante", 1.0 },
  { "Laquanya", 0.0 },
  { "Laquanza", 0.0 },
  { "Laquarius", 0.89361703 },
  { "Laquasha", 0.0 },
  { "Laquashia", 0.0 },
  { "Laquasia", 0.0 },
  { "Laquata", 0.0 },
  { "Laquatta", 0.0 },
  { "Laquavia", 0.0 },
  { "Laquavion", 1.0 },
  { "Laquavious", 1.0 },
  { "Laquavis", 1.0 },
  { "Laquavius", 1.0 },
  { "Laquay", 0.29411766 },
  { "Laquaya", 0.0 },
  { "Laquaysha", 0.0 },
  { "Laqueda", 0.0 },
  { "Laqueen", 0.0 },
  { "Laqueena", 0.0 },
  { "Laqueeta", 0.0 },
  { "Laqueisha", 0.0 },
  { "Laqueita", 0.0 },
  { "Laquel", 0.0 },
  { "Laquela", 0.0 },
  { "Laquella", 0.0 },
  { "Laquenda", 0.0 },
  { "Laquenta", 0.0 },
  { "Laquentin", 1.0 },
  { "Laquesha", 0.0 },
  { "Laqueshia", 0.0 },
  { "Laquesta", 0.0 },
  { "Laqueta", 0.0 },
  { "Laquetha", 0.0 },
  { "Laquetta", 0.0 },
  { "Laquette", 0.0 },
  { "Laquez", 1.0 },
  { "Laquia", 0.0 },
  { "Laquicha", 0.0 },
  { "Laquida", 0.0 },
  { "Laquiesha", 0.0 },
  { "Laquieta", 0.0 },
  { "Laquietta", 0.0 },
  { "Laquil", 0.0 },
  { "Laquila", 0.0 },
  { "Laquilla", 0.0 },
  { "Laquin", 1.0 },
  { "Laquina", 0.0 },
  { "Laquincia", 0.0 },
  { "Laquincy", 1.0 },
  { "Laquinda", 0.0 },
  { "Laquinn", 0.9137931 },
  { "Laquinta", 0.02184466 },
  { "Laquinthia", 0.0 },
  { "Laquintin", 1.0 },
  { "Laquinton", 1.0 },
  { "Laquisa", 0.0 },
  { "Laquise", 0.0 },
  { "Laquisha", 0.0 },
  { "Laquishia", 0.0 },
  { "Laquista", 0.0 },
  { "Laquita", 0.0 },
  { "Laquite", 0.0 },
  { "Laquitha", 0.0 },
  { "Laquitia", 0.0 },
  { "Laquitta", 0.0 },
  { "Laqunda", 0.0 },
  { "Laqundra", 0.0 },
  { "Laqunita", 0.0 },
  { "Laquoia", 0.0 },
  { "Laquon", 1.0 },
  { "Laquonda", 0.0 },
  { "Laqusha", 0.0 },
  { "Laqwan", 1.0 },
  { "Laqwanda", 0.0 },
  { "Lara", 0.0012769953 },
  { "Larae", 0.0 },
  { "Laraea", 0.0 },
  { "Larah", 0.0 },
  { "Laraib", 0.0 },
  { "Larain", 0.0 },
  { "Laraina", 0.0 },
  { "Laraine", 0.0 },
  { "Laralee", 0.0 },
  { "Laralyn", 0.0 },
  { "Laramee", 0.0 },
  { "Laramie", 0.56295466 },
  { "Laramy", 1.0 },
  { "Larance", 1.0 },
  { "Laranda", 0.0 },
  { "Larane", 0.0 },
  { "Larasha", 0.0 },
  { "Laraven", 0.0 },
  { "Laray", 0.50339216 },
  { "Laraya", 0.0 },
  { "Larayah", 0.0 },
  { "Laraye", 0.0 },
  { "Larayna", 0.0 },
  { "Larayne", 0.0 },
  { "Larce", 1.0 },
  { "Larcenia", 0.0 },
  { "Larea", 0.0 },
  { "Larean", 0.0 },
  { "Larecia", 0.0 },
  { "Lareco", 1.0 },
  { "Laredo", 0.7916667 },
  { "Laree", 0.0 },
  { "Lareen", 0.0 },
  { "Lareesa", 0.0 },
  { "Lareese", 0.0 },
  { "Lareina", 0.0 },
  { "Lareine", 0.0 },
  { "Lareisha", 0.0 },
  { "Lareka", 0.0 },
  { "Larell", 0.98533726 },
  { "Larelle", 0.4827586 },
  { "Laremy", 1.0 },
  { "Laren", 0.41340783 },
  { "Larena", 0.0 },
  { "Larence", 1.0 },
  { "Larencia", 0.0 },
  { "Larenda", 0.0 },
  { "Larene", 0.0 },
  { "Larenz", 1.0 },
  { "Larenza", 0.0 },
  { "Larenzo", 1.0 },
  { "Laresa", 0.0 },
  { "Laresha", 0.0 },
  { "Lareshia", 0.0 },
  { "Laressa", 0.0 },
  { "Lareta", 0.0 },
  { "Laretha", 0.0 },
  { "Laretta", 0.0 },
  { "Larey", 1.0 },
  { "Lareyna", 0.0 },
  { "Larez", 1.0 },
  { "Larhea", 0.0 },
  { "Larhonda", 0.0 },
  { "Lari", 0.043230943 },
  { "Laria", 0.0 },
  { "Lariah", 0.0 },
  { "Lariana", 0.0 },
  { "Larianna", 0.0 },
  { "Larica", 0.0 },
  { "Larice", 0.0 },
  { "Laricia", 0.0 },
  { "Laricka", 0.0 },
  { "Larico", 1.0 },
  { "Larie", 0.014044944 },
  { "Lariel", 0.0 },
  { "Lariesha", 0.0 },
  { "Larika", 0.0 },
  { "Larimar", 0.0 },
  { "Larin", 0.46031746 },
  { "Larina", 0.0 },
  { "Larinda", 0.0 },
  { "Larine", 0.0 },
  { "Larion", 1.0 },
  { "Larionna", 0.0 },
  { "Laris", 1.0 },
  { "Larisa", 0.0 },
  { "Larisha", 0.0 },
  { "Larissa", 0.0 },
  { "Larita", 0.0 },
  { "Laritha", 0.0 },
  { "Laritza", 0.0 },
  { "Lariya", 0.0 },
  { "Lariyah", 0.0 },
  { "Lariza", 0.0 },
  { "Larizza", 0.0 },
  { "Lark", 0.1368421 },
  { "Larke", 0.0 },
  { "Larken", 0.12658228 },
  { "Larkie", 0.5 },
  { "Larkin", 0.60063344 },
  { "Larkyn", 0.0 },
  { "Larmar", 1.0 },
  { "Larmon", 1.0 },
  { "Larmont", 1.0 },
  { "Larna", 0.0 },
  { "Larnce", 1.0 },
  { "Larnell", 0.9943117 },
  { "Larnelle", 1.0 },
  { "Larnette", 0.0 },
  { "Larney", 1.0 },
  { "Larnice", 0.0 },
  { "Larnie", 1.0 },
  { "Larobert", 1.0 },
  { "Larod", 1.0 },
  { "Laroderick", 1.0 },
  { "Larodney", 1.0 },
  { "Laroi", 1.0 },
  { "Larome", 1.0 },
  { "Laron", 0.97642165 },
  { "Larona", 0.0 },
  { "Laronda", 0.0 },
  { "Larone", 1.0 },
  { "Laronica", 0.0 },
  { "Laronn", 1.0 },
  { "Laronna", 0.0 },
  { "Laronte", 1.0 },
  { "Laronzo", 1.0 },
  { "Larosa", 0.0 },
  { "Larose", 0.0 },
  { "Larosha", 0.0 },
  { "Laroy", 1.0 },
  { "Laroya", 0.0 },
  { "Laroyce", 0.5729167 },
  { "Larra", 0.0 },
  { "Larraine", 0.0 },
  { "Larre", 1.0 },
  { "Larrell", 1.0 },
  { "Larren", 0.7821782 },
  { "Larrene", 0.0 },
  { "Larreon", 1.0 },
  { "Larresha", 0.0 },
  { "Larrey", 1.0 },
  { "Larri", 0.23396227 },
  { "Larriah", 0.0 },
  { "Larrica", 0.0 },
  { "Larrie", 0.73264986 },
  { "Larriesha", 0.0 },
  { "Larrine", 0.0 },
  { "Larrion", 1.0 },
  { "Larris", 1.0 },
  { "Larrisa", 0.0 },
  { "Larrisha", 0.0 },
  { "Larrissa", 0.0 },
  { "Larron", 1.0 },
  { "Larrry", 1.0 },
  { "Larry", 0.99559784 },
  { "Lars", 1.0 },
  { "Larsa", 0.0 },
  { "Larsen", 0.77927065 },
  { "Larson", 0.95704055 },
  { "Larsson", 1.0 },
  { "Larsyn", 0.0 },
  { "Lartarsha", 0.0 },
  { "Lartha", 0.0 },
  { "Larua", 0.0 },
  { "Larue", 0.28002948 },
  { "Laruen", 0.0 },
  { "Laruth", 0.0 },
  { "Larvell", 1.0 },
  { "Larwance", 1.0 },
  { "Larwence", 1.0 },
  { "Larwrence", 1.0 },
  { "Lary", 1.0 },
  { "Larya", 0.0 },
  { "Laryah", 0.0 },
  { "Laryan", 1.0 },
  { "Laryiah", 0.0 },
  { "Laryn", 0.0 },
  { "Larysa", 0.0 },
  { "Laryssa", 0.0 },
  { "Larz", 1.0 },
  { "Lasalle", 0.96107054 },
  { "Lasana", 1.0 },
  { "Lasanda", 0.0 },
  { "Lasandra", 0.0 },
  { "Lasanya", 0.0 },
  { "Lasara", 0.0 },
  { "Lasaro", 1.0 },
  { "Lasasha", 0.0 },
  { "Lasaunda", 0.0 },
  { "Lasaundra", 0.0 },
  { "Lasca", 0.0 },
  { "Lascelles", 1.0 },
  { "Lasco", 1.0 },
  { "Lasean", 0.72021663 },
  { "Lasedrick", 1.0 },
  { "Lasette", 0.0 },
  { "Lasey", 0.0 },
  { "Lash", 1.0 },
  { "Lasha", 0.0 },
  { "Lashad", 1.0 },
  { "Lashada", 0.0 },
  { "Lashae", 0.0 },
  { "Lashai", 0.0 },
  { "Lashaina", 0.0 },
  { "Lashala", 0.0 },
  { "Lashalle", 0.0 },
  { "Lashan", 0.14524207 },
  { "Lashana", 0.0 },
  { "Lashanae", 0.0 },
  { "Lashanay", 0.0 },
  { "Lashanda", 0.0 },
  { "Lashandra", 0.0 },
  { "Lashandria", 0.0 },
  { "Lashane", 0.0862069 },
  { "Lashanette", 0.0 },
  { "Lashanique", 0.0 },
  { "Lashann", 0.0 },
  { "Lashanna", 0.0 },
  { "Lashannon", 0.0 },
  { "Lashanta", 0.0 },
  { "Lashante", 0.0 },
  { "Lashanti", 0.0 },
  { "Lashanya", 0.0 },
  { "Lashaon", 0.0 },
  { "Lashara", 0.0 },
  { "Lasharee", 0.0 },
  { "Lashari", 0.0 },
  { "Lasharia", 0.0 },
  { "Lasharn", 0.0 },
  { "Lasharon", 0.0 },
  { "Lasharra", 0.0 },
  { "Lashasta", 0.0 },
  { "Lashaun", 0.39053538 },
  { "Lashauna", 0.0 },
  { "Lashaunda", 0.0 },
  { "Lashaundra", 0.0 },
  { "Lashaundria", 0.0 },
  { "Lashaune", 0.0 },
  { "Lashaunna", 0.0 },
  { "Lashaunta", 0.0 },
  { "Lashaunte", 0.0 },
  { "Lashaunti", 0.0 },
  { "Lashavia", 0.0 },
  { "Lashaw", 0.5 },
  { "Lashawanda", 0.0 },
  { "Lashawda", 0.0 },
  { "Lashawn", 0.2947693 },
  { "Lashawna", 0.0 },
  { "Lashawnda", 0.0 },
  { "Lashawndra", 0.0 },
  { "Lashawne", 0.0 },
  { "Lashawnna", 0.0 },
  { "Lashawnta", 0.0 },
  { "Lashay", 0.0036245016 },
  { "Lashaya", 0.0 },
  { "Lashaye", 0.0 },
  { "Lashayla", 0.0 },
  { "Lashayna", 0.0 },
  { "Lashe", 0.0 },
  { "Lashea", 0.0 },
  { "Lasheba", 0.0 },
  { "Lasheen", 0.0 },
  { "Lasheena", 0.0 },
  { "Lasheia", 0.0 },
  { "Lasheika", 0.0 },
  { "Lasheila", 0.0 },
  { "Lasheka", 0.0 },
  { "Lashekia", 0.0 },
  { "Lashel", 0.0 },
  { "Lashelia", 0.0 },
  { "Lashell", 0.0 },
  { "Lashella", 0.0 },
  { "Lashelle", 0.0 },
  { "Lashema", 0.0 },
  { "Lashena", 0.0 },
  { "Lashenda", 0.0 },
  { "Lashenna", 0.0 },
  { "Lashera", 0.0 },
  { "Lasherrie", 0.0 },
  { "Lasheryl", 0.0 },
  { "Lasheta", 0.0 },
  { "Lashey", 0.0 },
  { "Lashia", 0.0 },
  { "Lashica", 0.0 },
  { "Lashieka", 0.0 },
  { "Lashika", 0.0 },
  { "Lashina", 0.0 },
  { "Lashinda", 0.0 },
  { "Lashira", 0.0 },
  { "Lashiya", 0.0 },
  { "Lashiyah", 0.0 },
  { "Lashley", 0.0 },
  { "Lashon", 0.3061963 },
  { "Lashona", 0.0 },
  { "Lashond", 0.0 },
  { "Lashonda", 0.0 },
  { "Lashondia", 0.0 },
  { "Lashondra", 0.0 },
  { "Lashondria", 0.0 },
  { "Lashone", 0.123893805 },
  { "Lashonia", 0.0 },
  { "Lashonn", 0.0 },
  { "Lashonna", 0.0 },
  { "Lashonne", 0.0 },
  { "Lashonta", 0.0 },
  { "Lashonte", 0.0 },
  { "Lashonya", 0.0 },
  { "Lashounda", 0.0 },
  { "Lashown", 0.0 },
  { "Lashowna", 0.0 },
  { "Lashun", 0.31199187 },
  { "Lashuna", 0.0 },
  { "Lashunda", 0.0 },
  { "Lashundia", 0.0 },
  { "Lashundra", 0.0 },
  { "Lashundria", 0.0 },
  { "Lashune", 0.0 },
  { "Lashunna", 0.0 },
  { "Lashunta", 0.0 },
  { "Lashunti", 0.0 },
  { "Lashunya", 0.0 },
  { "Lashurn", 0.0 },
  { "Lashya", 0.0 },
  { "Lashyia", 0.0 },
  { "Lashyra", 0.0 },
  { "Lasia", 0.0 },
  { "Lasiah", 0.0 },
  { "Lasiya", 0.0 },
  { "Lasiyah", 0.0 },
  { "Lason", 1.0 },
  { "Lasonda", 0.0 },
  { "Lasondra", 0.0 },
  { "Lasonia", 0.0 },
  { "Lasonja", 0.0 },
  { "Lasonya", 0.0 },
  { "Lassana", 1.0 },
  { "Lassandra", 0.0 },
  { "Lasse", 1.0 },
  { "Lassie", 0.0 },
  { "Lastacia", 0.0 },
  { "Lastar", 0.0 },
  { "Lastarr", 0.0 },
  { "Lastarza", 1.0 },
  { "Lastasha", 0.0 },
  { "Lastasia", 0.0 },
  { "Lastat", 1.0 },
  { "Laster", 1.0 },
  { "Lasunda", 0.0 },
  { "Lasundra", 0.0 },
  { "Lasya", 0.0 },
  { "Laszlo", 1.0 },
  { "Lata", 0.0 },
  { "Latacha", 0.0 },
  { "Latachia", 0.0 },
  { "Latacia", 0.0 },
  { "Latae", 0.0 },
  { "Lataesha", 0.0 },
  { "Latai", 0.0 },
  { "Lataija", 0.0 },
  { "Lataisha", 0.0 },
  { "Lataivia", 0.0 },
  { "Lataiya", 0.0 },
  { "Lataja", 0.0 },
  { "Latajah", 0.0 },
  { "Latajia", 0.0 },
  { "Latalia", 0.0 },
  { "Latalya", 0.0 },
  { "Latamara", 0.0 },
  { "Latana", 0.0 },
  { "Latandra", 0.0 },
  { "Latane", 0.0 },
  { "Latanga", 0.0 },
  { "Latangela", 0.0 },
  { "Latangie", 0.0 },
  { "Latania", 0.0 },
  { "Latanisha", 0.0 },
  { "Latanja", 0.0 },
  { "Latanya", 0.0023319477 },
  { "Latanyia", 0.0 },
  { "Latanza", 0.0 },
  { "Latara", 0.0 },
  { "Latarah", 0.0 },
  { "Latarcha", 0.0 },
  { "Latari", 0.0 },
  { "Lataria", 0.0 },
  { "Latarisha", 0.0 },
  { "Latarius", 1.0 },
  { "Latarra", 0.0 },
  { "Latarsha", 0.0 },
  { "Latarshia", 0.0 },
  { "Latarsia", 0.0 },
  { "Latarus", 1.0 },
  { "Latarya", 0.0 },
  { "Latascha", 0.0 },
  { "Latash", 0.0 },
  { "Latasha", 0.0037626247 },
  { "Latashi", 0.0 },
  { "Latashia", 0.0 },
  { "Latashua", 0.0 },
  { "Latasia", 0.0 },
  { "Lataska", 0.0 },
  { "Latassha", 0.0 },
  { "Latatia", 0.0 },
  { "Lataunya", 0.0 },
  { "Lataura", 0.0 },
  { "Lataurus", 0.9122807 },
  { "Latausha", 0.0 },
  { "Latavea", 0.0 },
  { "Lataveon", 1.0 },
  { "Latavia", 0.0 },
  { "Latavian", 1.0 },
  { "Latavion", 1.0 },
  { "Latavious", 1.0 },
  { "Latavis", 1.0 },
  { "Latavius", 1.0 },
  { "Latawn", 0.0 },
  { "Latawnya", 0.0 },
  { "Latay", 0.0 },
  { "Lataya", 0.0 },
  { "Latayia", 0.0 },
  { "Latayna", 0.0 },
  { "Lataysha", 0.0 },
  { "Latayshia", 0.0 },
  { "Lataysia", 0.0 },
  { "Latayvia", 0.0 },
  { "Latazia", 0.0 },
  { "Latea", 0.0 },
  { "Lateafa", 0.0 },
  { "Lateaka", 0.0 },
  { "Lateasa", 0.0 },
  { "Latease", 0.0 },
  { "Lateasha", 0.0 },
  { "Lateashia", 0.0 },
  { "Latece", 0.0 },
  { "Latecha", 0.0 },
  { "Latechia", 0.0 },
  { "Latecia", 0.0 },
  { "Latedra", 0.0 },
  { "Latee", 1.0 },
  { "Lateef", 1.0 },
  { "Lateefa", 0.0 },
  { "Lateefah", 0.0 },
  { "Lateek", 1.0 },
  { "Lateeka", 0.0 },
  { "Lateela", 0.0 },
  { "Lateen", 0.0 },
  { "Lateena", 0.0 },
  { "Lateesha", 0.0 },
  { "Latefa", 0.0 },
  { "Latefah", 0.0 },
  { "Latefia", 0.0 },
  { "Lateia", 0.0 },
  { "Lateika", 0.0 },
  { "Lateisha", 0.0 },
  { "Lateka", 0.0 },
  { "Latela", 0.0 },
  { "Laten", 1.0 },
  { "Latenya", 0.0 },
  { "Latera", 0.0 },
  { "Lateresa", 0.0 },
  { "Laterese", 0.0 },
  { "Lateria", 0.0 },
  { "Laterian", 1.0 },
  { "Laterica", 0.0 },
  { "Latericka", 0.0 },
  { "Laterika", 0.0 },
  { "Laterius", 1.0 },
  { "Laterra", 0.0 },
  { "Laterrance", 1.0 },
  { "Laterrell", 1.0 },
  { "Laterrence", 1.0 },
  { "Laterria", 0.0 },
  { "Laterrian", 0.9285714 },
  { "Laterrica", 0.0 },
  { "Laterrika", 0.0 },
  { "Laterrion", 1.0 },
  { "Laterrious", 1.0 },
  { "Laterrius", 1.0 },
  { "Latesa", 0.0 },
  { "Latese", 0.0 },
  { "Latesha", 0.0 },
  { "Lateshia", 0.0 },
  { "Latesia", 0.0 },
  { "Latessa", 0.0 },
  { "Latessia", 0.0 },
  { "Latetia", 0.0 },
  { "Latevia", 0.0 },
  { "Latevin", 1.0 },
  { "Lateya", 0.0 },
  { "Latha", 0.0 },
  { "Latham", 1.0 },
  { "Lathan", 1.0 },
  { "Lathaniel", 1.0 },
  { "Lathasha", 0.0 },
  { "Lathe", 1.0 },
  { "Lathem", 1.0 },
  { "Lathen", 1.0 },
  { "Lathena", 0.0 },
  { "Lathisa", 0.0 },
  { "Lathomas", 1.0 },
  { "Lathon", 1.0 },
  { "Lathyn", 1.0 },
  { "Latia", 0.0 },
  { "Latiah", 0.0 },
  { "Latiana", 0.0 },
  { "Latianna", 0.0 },
  { "Latiara", 0.0 },
  { "Latiasha", 0.0 },
  { "Latica", 0.0 },
  { "Latice", 0.0 },
  { "Laticha", 0.0 },
  { "Latichia", 0.0 },
  { "Laticia", 0.0 },
  { "Latiea", 0.0 },
  { "Latief", 1.0 },
  { "Latieka", 0.0 },
  { "Latiera", 0.0 },
  { "Latierra", 0.0 },
  { "Latiesha", 0.0 },
  { "Latif", 1.0 },
  { "Latifa", 0.0 },
  { "Latifah", 0.0 },
  { "Latiffa", 0.0 },
  { "Latiffany", 0.0 },
  { "Latifha", 0.0 },
  { "Latifia", 0.0 },
  { "Latigra", 0.0 },
  { "Latijera", 0.0 },
  { "Latika", 0.0 },
  { "Latima", 0.0 },
  { "Latimer", 1.0 },
  { "Latina", 0.0 },
  { "Latinia", 0.0 },
  { "Latinya", 0.0 },
  { "Lationa", 0.0 },
  { "Latiqua", 0.0 },
  { "Latique", 1.0 },
  { "Latira", 0.0 },
  { "Latisa", 0.0 },
  { "Latischa", 0.0 },
  { "Latise", 0.0 },
  { "Latish", 0.0 },
  { "Latisha", 0.0020672397 },
  { "Latishia", 0.0 },
  { "Latisia", 0.0 },
  { "Latissa", 0.0 },
  { "Latissha", 0.0 },
  { "Latissia", 0.0 },
  { "Latissue", 0.0 },
  { "Latita", 0.0 },
  { "Latitia", 0.0 },
  { "Lativa", 0.0 },
  { "Lativia", 0.0 },
  { "Latiya", 0.0 },
  { "Latiyah", 0.0 },
  { "Latocha", 0.0 },
  { "Latodd", 1.0 },
  { "Latoi", 0.0 },
  { "Latoia", 0.0 },
  { "Latoina", 0.0 },
  { "Latoisha", 0.0 },
  { "Latoiya", 0.0 },
  { "Laton", 1.0 },
  { "Latona", 0.0 },
  { "Latonda", 0.0 },
  { "Latondra", 0.0 },
  { "Latonga", 0.0 },
  { "Latongia", 0.0 },
  { "Latoni", 0.0 },
  { "Latonia", 0.003125 },
  { "Latonio", 1.0 },
  { "Latonja", 0.0 },
  { "Latonjia", 0.0 },
  { "Latonna", 0.0 },
  { "Latonnia", 0.0 },
  { "Latony", 1.0 },
  { "Latonya", 0.0030735354 },
  { "Latonyia", 0.0 },
  { "Latonza", 0.0 },
  { "Latora", 0.0 },
  { "Latori", 0.0 },
  { "Latoria", 0.0 },
  { "Latorie", 0.0 },
  { "Latorra", 0.0 },
  { "Latorrance", 1.0 },
  { "Latorri", 0.0 },
  { "Latorria", 0.0 },
  { "Latorrie", 0.0 },
  { "Latorsha", 0.0 },
  { "Latory", 0.0 },
  { "Latorya", 0.0 },
  { "Latosca", 0.0 },
  { "Latoscha", 0.0 },
  { "Latosha", 0.0 },
  { "Latoshia", 0.0 },
  { "Latosia", 0.0 },
  { "Latousha", 0.0 },
  { "Latova", 0.0 },
  { "Latovia", 0.0 },
  { "Latoy", 0.0 },
  { "Latoya", 0.004053728 },
  { "Latoyah", 0.0 },
  { "Latoye", 0.0 },
  { "Latoyer", 0.0 },
  { "Latoyia", 0.0 },
  { "Latoyla", 0.0 },
  { "Latoyna", 0.0 },
  { "Latoynia", 0.0 },
  { "Latoyo", 0.0 },
  { "Latoyra", 0.0 },
  { "Latoyria", 0.0 },
  { "Latoysha", 0.0 },
  { "Latoyta", 0.0 },
  { "Latoyya", 0.0 },
  { "Latracia", 0.0 },
  { "Latracy", 0.0 },
  { "Latrae", 1.0 },
  { "Latrail", 0.89166665 },
  { "Latrasha", 0.0 },
  { "Latravia", 0.0 },
  { "Latravion", 1.0 },
  { "Latravious", 1.0 },
  { "Latravis", 1.0 },
  { "Latravius", 1.0 },
  { "Latray", 1.0 },
  { "Latraya", 0.0 },
  { "Latrayvion", 1.0 },
  { "Latre", 1.0 },
  { "Latrea", 0.0 },
  { "Latreace", 0.0 },
  { "Latreal", 1.0 },
  { "Latreasa", 0.0 },
  { "Latrease", 0.0 },
  { "Latreca", 0.0 },
  { "Latrece", 0.0 },
  { "Latrecia", 0.0 },
  { "Latreece", 0.0 },
  { "Latreena", 0.0 },
  { "Latreese", 0.0 },
  { "Latreice", 0.0 },
  { "Latreisha", 0.0 },
  { "Latreka", 0.0 },
  { "Latrel", 1.0 },
  { "Latrell", 0.8645129 },
  { "Latrelle", 0.297164 },
  { "Latrena", 0.0 },
  { "Latrenda", 0.0 },
  { "Latrenia", 0.0 },
  { "Latres", 0.0 },
  { "Latresa", 0.0 },
  { "Latrese", 0.0 },
  { "Latresha", 0.0 },
  { "Latreshia", 0.0 },
  { "Latresia", 0.0 },
  { "Latress", 0.0 },
  { "Latressa", 0.0 },
  { "Latreva", 0.0 },
  { "Latrevia", 0.0 },
  { "Latrevion", 1.0 },
  { "Latrevious", 1.0 },
  { "Latrey", 1.0 },
  { "Latrez", 1.0 },
  { "Latria", 0.0 },
  { "Latric", 0.0 },
  { "Latrica", 0.0 },
  { "Latrice", 0.001867106 },
  { "Latricha", 0.0 },
  { "Latricia", 0.0 },
  { "Latriece", 0.0 },
  { "Latrika", 0.0 },
  { "Latrina", 0.0 },
  { "Latrinda", 0.0 },
  { "Latrinia", 0.0 },
  { "Latrinity", 0.0 },
  { "Latris", 0.0 },
  { "Latrisa", 0.0 },
  { "Latrise", 0.0 },
  { "Latrish", 0.0 },
  { "Latrisha", 0.0 },
  { "Latrishia", 0.0 },
  { "Latrissa", 0.0 },
  { "Latrista", 0.0 },
  { "Latrivia", 0.0 },
  { "Latron", 1.0 },
  { "Latronda", 0.0 },
  { "Latrone", 1.0 },
  { "Latronya", 0.0 },
  { "Latroy", 1.0 },
  { "Latroya", 0.0 },
  { "Latroyia", 0.0 },
  { "Latryce", 0.0 },
  { "Latsha", 0.0 },
  { "Latshia", 0.0 },
  { "Lattie", 0.25708884 },
  { "Latu", 1.0 },
  { "Latundra", 0.0 },
  { "Latunia", 0.0 },
  { "Latunja", 0.0 },
  { "Latunya", 0.0 },
  { "Latusha", 0.0 },
  { "Latwan", 1.0 },
  { "Latwana", 0.0 },
  { "Latwanda", 0.0 },
  { "Latwanna", 0.0 },
  { "Latwon", 1.0 },
  { "Latya", 0.0 },
  { "Latyia", 0.0 },
  { "Latyna", 0.0 },
  { "Latyra", 0.0 },
  { "Latysha", 0.0 },
  { "Lauana", 0.0 },
  { "Lauar", 1.0 },
  { "Lauchlin", 1.0 },
  { "Lauda", 0.0 },
  { "Laudan", 0.0 },
  { "Lauden", 1.0 },
  { "Laudie", 1.0 },
  { "Laughlin", 1.0 },
  { "Lauire", 0.0 },
  { "Launa", 0.0 },
  { "Launda", 0.0 },
  { "Laundyn", 0.0 },
  { "Launi", 0.0 },
  { "Launia", 0.0 },
  { "Launie", 0.0 },
  { "Laur", 0.0 },
  { "Laura", 0.0032388389 },
  { "Lauraann", 0.0 },
  { "Lauraanne", 0.0 },
  { "Lauraashley", 0.0 },
  { "Laurabel", 0.0 },
  { "Laurabell", 0.0 },
  { "Laurabelle", 0.0 },
  { "Laurabeth", 0.0 },
  { "Laurae", 0.0 },
  { "Lauraelizabeth", 0.0 },
  { "Lauraetta", 0.0 },
  { "Laurah", 0.0 },
  { "Lauraine", 0.0 },
  { "Laurajean", 0.0 },
  { "Laural", 0.0 },
  { "Lauralai", 0.0 },
  { "Laurale", 0.0 },
  { "Lauralea", 0.0 },
  { "Lauralee", 0.0 },
  { "Lauralei", 0.0 },
  { "Laurali", 0.0 },
  { "Lauralie", 0.0 },
  { "Lauralye", 0.0 },
  { "Lauralyn", 0.0 },
  { "Lauralynn", 0.0 },
  { "Lauramae", 0.0 },
  { "Lauramarie", 0.0 },
  { "Lauran", 0.0 },
  { "Laurana", 0.0 },
  { "Laurance", 0.99742 },
  { "Laurann", 0.0 },
  { "Lauranne", 0.0 },
  { "Laure", 0.0 },
  { "Laurea", 0.0 },
  { "Laurean", 0.0 },
  { "Laureana", 0.0 },
  { "Laureano", 1.0 },
  { "Laureat", 1.0 },
  { "Laurece", 0.0 },
  { "Lauree", 0.0 },
  { "Laureen", 0.0 },
  { "Laurel", 0.04107768 },
  { "Laurelai", 0.0 },
  { "Laurelee", 0.0 },
  { "Laurelei", 0.0 },
  { "Laureli", 0.0 },
  { "Laurelin", 0.0 },
  { "Laurell", 0.025906736 },
  { "Laurella", 0.0 },
  { "Laurelle", 0.0 },
  { "Laurelyn", 0.0 },
  { "Lauren", 0.012301365 },
  { "Laurena", 0.0 },
  { "Laurenann", 0.0 },
  { "Laurenashley", 0.0 },
  { "Laurence", 0.9885235 },
  { "Laurencia", 0.0 },
  { "Laurencio", 1.0 },
  { "Laurenda", 0.0 },
  { "Laurene", 0.0 },
  { "Laurenelizabeth", 0.0 },
  { "Laurenmarie", 0.0 },
  { "Laurenn", 0.0 },
  { "Laurenne", 0.0 },
  { "Laurennicole", 0.0 },
  { "Laurens", 1.0 },
  { "Laurent", 1.0 },
  { "Laurentina", 0.0 },
  { "Laurentine", 0.0 },
  { "Laurentino", 1.0 },
  { "Laurenz", 1.0 },
  { "Laurenzo", 1.0 },
  { "Lauressa", 0.0 },
  { "Lauretha", 0.0 },
  { "Laurett", 0.0 },
  { "Lauretta", 0.0 },
  { "Laurette", 0.0 },
  { "Laurey", 0.0 },
  { "Lauri", 0.016307253 },
  { "Lauria", 0.0 },
  { "Laurian", 0.0 },
  { "Lauriana", 0.0 },
  { "Lauriane", 0.0 },
  { "Lauriann", 0.0 },
  { "Laurianna", 0.0 },
  { "Laurianne", 0.0 },
  { "Lauriano", 1.0 },
  { "Laurica", 0.0 },
  { "Laurice", 0.13861386 },
  { "Laurie", 0.011632684 },
  { "Laurieann", 0.0 },
  { "Laurieanne", 0.0 },
  { "Lauriel", 0.0 },
  { "Laurielle", 0.0 },
  { "Laurien", 0.23809524 },
  { "Laurier", 1.0 },
  { "Laurietta", 0.0 },
  { "Laurilyn", 0.0 },
  { "Laurin", 0.23360854 },
  { "Laurina", 0.0 },
  { "Laurinda", 0.0 },
  { "Laurine", 0.0 },
  { "Lauris", 0.5585284 },
  { "Laurisa", 0.0 },
  { "Laurissa", 0.0 },
  { "Lauriston", 1.0 },
  { "Laurita", 0.0 },
  { "Lauritz", 1.0 },
  { "Laurn", 0.0 },
  { "Laurna", 0.0 },
  { "Laurne", 0.0 },
  { "Laurnea", 0.0 },
  { "Lauro", 1.0 },
  { "Lauron", 0.0 },
  { "Laurren", 0.0 },
  { "Laurrie", 0.0 },
  { "Laurrine", 0.0 },
  { "Laury", 0.056603774 },
  { "Lauryl", 0.0 },
  { "Lauryn", 0.00062666455 },
  { "Lauryne", 0.0 },
  { "Laurynn", 0.0 },
  { "Lautaro", 1.0 },
  { "Lauvenia", 0.0 },
  { "Lavada", 0.0 },
  { "Lavae", 0.0 },
  { "Lavaeh", 0.0 },
  { "Lavail", 1.0 },
  { "Laval", 1.0 },
  { "Lavale", 1.0 },
  { "Lavall", 1.0 },
  { "Lavalle", 0.96031743 },
  { "Lavan", 0.9825175 },
  { "Lavana", 0.0 },
  { "Lavance", 1.0 },
  { "Lavancha", 0.0 },
  { "Lavanda", 0.0 },
  { "Lavander", 1.0 },
  { "Lavani", 0.0 },
  { "Lavania", 0.0 },
  { "Lavanna", 0.0 },
  { "Lavant", 1.0 },
  { "Lavante", 1.0 },
  { "Lavanya", 0.0 },
  { "Lavar", 0.99455446 },
  { "Lavara", 0.0 },
  { "Lavare", 1.0 },
  { "Lavares", 1.0 },
  { "Lavaria", 0.0 },
  { "Lavarious", 1.0 },
  { "Lavaris", 1.0 },
  { "Lavarius", 1.0 },
  { "Lavarr", 1.0 },
  { "Lavarus", 1.0 },
  { "Lavasha", 0.0 },
  { "Lavasia", 0.0 },
  { "Lavatrice", 0.0 },
  { "Lavaughn", 0.47830802 },
  { "Lavaun", 0.0 },
  { "Lavaune", 0.0 },
  { "Lavay", 0.0 },
  { "Lavaya", 0.0 },
  { "Lavayah", 0.0 },
  { "Laveah", 0.0 },
  { "Laveda", 0.0 },
  { "Lavee", 0.0 },
  { "Laveeda", 0.0 },
  { "Laveen", 0.0 },
  { "Lavel", 1.0 },
  { "Lavelda", 0.0 },
  { "Lavell", 0.8945161 },
  { "Lavella", 0.0 },
  { "Lavelle", 0.5969806 },
  { "Lavelton", 1.0 },
  { "Laven", 1.0 },
  { "Lavena", 0.0 },
  { "Lavenda", 0.0 },
  { "Lavender", 0.0 },
  { "Lavene", 0.0 },
  { "Lavenia", 0.0 },
  { "Lavenna", 0.0 },
  { "Laveon", 1.0 },
  { "Laver", 1.0 },
  { "Lavera", 0.0 },
  { "Laverda", 0.0 },
  { "Lavere", 0.78341013 },
  { "Lavergne", 0.031307552 },
  { "Laveria", 0.0 },
  { "Laverl", 0.765625 },
  { "Laverle", 0.063725494 },
  { "Lavern", 0.5927182 },
  { "Laverna", 0.0 },
  { "Laverne", 0.23534659 },
  { "Lavernia", 0.0 },
  { "Lavernon", 1.0 },
  { "Lavert", 0.96 },
  { "Laverta", 0.0 },
  { "Laveryl", 0.0 },
  { "Lavesha", 0.0 },
  { "Lavesta", 0.0 },
  { "Lavester", 1.0 },
  { "Laveta", 0.0 },
  { "Lavetra", 0.0 },
  { "Lavett", 0.0 },
  { "Lavetta", 0.0 },
  { "Lavette", 0.0042052143 },
  { "Laveya", 0.0 },
  { "Laveyah", 0.0 },
  { "Lavi", 1.0 },
  { "Lavia", 0.0 },
  { "Lavickie", 0.0 },
  { "Lavida", 0.0 },
  { "Lavie", 0.0 },
  { "Lavila", 0.0 },
  { "Lavilla", 0.0 },
  { "Lavin", 1.0 },
  { "Lavina", 0.0 },
  { "Lavinda", 0.0 },
  { "Lavine", 0.0 },
  { "Lavinia", 0.0 },
  { "Lavinnia", 0.0 },
  { "Lavisha", 0.0 },
  { "Lavita", 0.0 },
  { "Lavith", 1.0 },
  { "Lavitta", 0.0 },
  { "Lavoid", 1.0 },
  { "Lavola", 0.0 },
  { "Lavon", 0.42703646 },
  { "Lavona", 0.0 },
  { "Lavonda", 0.0 },
  { "Lavonde", 0.0 },
  { "Lavondia", 0.0 },
  { "Lavondra", 0.0 },
  { "Lavone", 0.090697676 },
  { "Lavonia", 0.0 },
  { "Lavonn", 0.19753087 },
  { "Lavonna", 0.0 },
  { "Lavonne", 0.013043217 },
  { "Lavonnia", 0.0 },
  { "Lavonnie", 0.0 },
  { "Lavont", 1.0 },
  { "Lavonta", 0.9648562 },
  { "Lavontae", 1.0 },
  { "Lavontay", 1.0 },
  { "Lavonte", 0.9904762 },
  { "Lavontre", 1.0 },
  { "Lavonya", 0.0 },
  { "Lavor", 1.0 },
  { "Lavora", 0.0 },
  { "Lavoria", 0.0 },
  { "Lavoris", 0.5653333 },
  { "Lavorn", 0.6666667 },
  { "Lavoy", 1.0 },
  { "Lavoyce", 0.0 },
  { "Lavra", 0.0 },
  { "Lavren", 0.0 },
  { "Law", 1.0 },
  { "Lawaia", 1.0 },
  { "Lawan", 0.36 },
  { "Lawana", 0.0 },
  { "Lawanda", 0.0004961548 },
  { "Lawander", 0.0 },
  { "Lawandra", 0.0 },
  { "Lawanika", 0.0 },
  { "Lawanna", 0.0 },
  { "Lawanza", 0.0 },
  { "Lawarence", 1.0 },
  { "Lawarnce", 1.0 },
  { "Lawarren", 1.0 },
  { "Lawaun", 1.0 },
  { "Lawayne", 0.98062015 },
  { "Lawell", 1.0 },
  { "Lawence", 1.0 },
  { "Lawenda", 0.0 },
  { "Lawerance", 1.0 },
  { "Lawerence", 1.0 },
  { "Lawernce", 1.0 },
  { "Lawhitney", 0.0 },
  { "Lawless", 1.0 },
  { "Lawna", 0.0 },
  { "Lawney", 1.0 },
  { "Lawon", 0.5 },
  { "Lawona", 0.0 },
  { "Lawonda", 0.0 },
  { "Lawonna", 0.0 },
  { "Lawra", 0.0 },
  { "Lawrance", 1.0 },
  { "Lawren", 0.055555556 },
  { "Lawrenc", 1.0 },
  { "Lawrence", 0.9953722 },
  { "Lawrencia", 0.0 },
  { "Lawrene", 0.5833333 },
  { "Lawrenzo", 1.0 },
  { "Lawrie", 0.0 },
  { "Lawrin", 1.0 },
  { "Lawrnce", 1.0 },
  { "Lawryn", 0.0 },
  { "Lawsen", 1.0 },
  { "Lawson", 0.97353715 },
  { "Lawsyn", 0.5316456 },
  { "Lawton", 1.0 },
  { "Lawyer", 1.0 },
  { "Laxavier", 1.0 },
  { "Laxmi", 0.0 },
  { "Laxmy", 0.0 },
  { "Lay", 0.7 },
  { "Laya", 0.0 },
  { "Layaan", 0.0 },
  { "Layah", 0.0 },
  { "Layahna", 0.0 },
  { "Layal", 0.0 },
  { "Layali", 0.0 },
  { "Layan", 0.0 },
  { "Layana", 0.0 },
  { "Layanah", 0.0 },
  { "Layane", 0.0 },
  { "Layani", 0.0 },
  { "Layanie", 0.0 },
  { "Layann", 0.0 },
  { "Layanna", 0.0 },
  { "Layanne", 0.0 },
  { "Layce", 0.0 },
  { "Laycee", 0.0 },
  { "Laycen", 1.0 },
  { "Layci", 0.0 },
  { "Laycie", 0.0 },
  { "Laydell", 1.0 },
  { "Layden", 0.93491864 },
  { "Laydon", 1.0 },
  { "Layelle", 0.0 },
  { "Layeska", 0.0 },
  { "Layia", 0.0 },
  { "Layiah", 0.0 },
  { "Layke", 0.8888889 },
  { "Layken", 0.15289766 },
  { "Laykin", 0.028645834 },
  { "Laykyn", 0.0 },
  { "Layla", 0.0010726161 },
  { "Laylaa", 0.0 },
  { "Laylaann", 0.0 },
  { "Laylagrace", 0.0 },
  { "Laylah", 0.0 },
  { "Laylamarie", 0.0 },
  { "Laylan", 0.2777778 },
  { "Laylana", 0.0 },
  { "Layland", 1.0 },
  { "Laylanee", 0.0 },
  { "Laylani", 0.0 },
  { "Laylanie", 0.0 },
  { "Laylanii", 0.0 },
  { "Laylanna", 0.0 },
  { "Laylanni", 0.0 },
  { "Laylannie", 0.0 },
  { "Laylarae", 0.0 },
  { "Laylarose", 0.0 },
  { "Layle", 0.22727273 },
  { "Laylee", 0.0 },
  { "Layleen", 0.0 },
  { "Layleigh", 0.0 },
  { "Laylen", 0.6818182 },
  { "Laylene", 0.0 },
  { "Layli", 0.0 },
  { "Laylia", 0.0 },
  { "Layliah", 0.0 },
  { "Layliana", 0.0 },
  { "Laylianna", 0.0 },
  { "Laylie", 0.0 },
  { "Laylin", 0.0 },
  { "Laylina", 0.0 },
  { "Laylla", 0.0 },
  { "Laylon", 1.0 },
  { "Layloni", 0.0 },
  { "Laylonie", 0.0 },
  { "Laylonni", 0.0 },
  { "Laylonnie", 0.0 },
  { "Laylyn", 0.0 },
  { "Laylynn", 0.0 },
  { "Layman", 1.0 },
  { "Laymon", 1.0 },
  { "Laymond", 1.0 },
  { "Layn", 1.0 },
  { "Layna", 0.0 },
  { "Laynah", 0.0 },
  { "Layne", 0.7419222 },
  { "Laynee", 0.0 },
  { "Laynette", 0.0 },
  { "Layney", 0.0 },
  { "Layni", 0.0 },
  { "Laynie", 0.0 },
  { "Layona", 0.0 },
  { "Layoni", 0.0 },
  { "Layonna", 0.0 },
  { "Layota", 0.0 },
  { "Layoya", 0.0 },
  { "Layra", 0.0 },
  { "Layron", 1.0 },
  { "Laysa", 0.0 },
  { "Laysha", 0.0 },
  { "Layson", 1.0 },
  { "Layssa", 0.0 },
  { "Layten", 0.9087591 },
  { "Layth", 1.0 },
  { "Laythan", 1.0 },
  { "Laythen", 1.0 },
  { "Laython", 1.0 },
  { "Laytin", 1.0 },
  { "Layton", 0.9458427 },
  { "Laytona", 0.0 },
  { "Laytonya", 0.0 },
  { "Laytoya", 0.0 },
  { "Layvonne", 0.0 },
  { "Layza", 0.0 },
  { "Lazandra", 0.0 },
  { "Lazar", 1.0 },
  { "Lazara", 0.0 },
  { "Lazare", 1.0 },
  { "Lazareth", 1.0 },
  { "Lazaria", 0.0 },
  { "Lazariah", 0.0 },
  { "Lazarick", 1.0 },
  { "Lazario", 1.0 },
  { "Lazarion", 1.0 },
  { "Lazarius", 1.0 },
  { "Lazariya", 0.0 },
  { "Lazariyah", 0.0 },
  { "Lazaro", 1.0 },
  { "Lazaros", 1.0 },
  { "Lazarria", 0.0 },
  { "Lazarus", 1.0 },
  { "Lazavion", 1.0 },
  { "Lazaya", 0.0 },
  { "Lazell", 1.0 },
  { "Lazelle", 0.0 },
  { "Lazer", 1.0 },
  { "Lazerick", 1.0 },
  { "Lazerrick", 1.0 },
  { "Lazetta", 0.0 },
  { "Lazette", 0.0 },
  { "Lazhane", 0.0 },
  { "Laziah", 0.0 },
  { "Laziya", 0.0 },
  { "Laziyah", 0.0 },
  { "Lazlo", 1.0 },
  { "Lazuli", 0.0 },
  { "Lazzaro", 1.0 },
  { "Lb", 1.0 },
  { "Lc", 1.0 },
  { "Ld", 1.0 },
  { "Ldarius", 1.0 },
  { "Le", 0.46916565 },
  { "Lea", 0.019867338 },
  { "Leaann", 0.0 },
  { "Leaanne", 0.0 },
  { "Leabella", 0.0 },
  { "Leabeth", 0.0 },
  { "Leacy", 0.0 },
  { "Leada", 0.0 },
  { "Leady", 0.0 },
  { "Leaetta", 0.0 },
  { "Leaf", 1.0 },
  { "Leafa", 0.0 },
  { "Leafie", 0.0 },
  { "Leaford", 1.0 },
  { "Leafy", 0.0 },
  { "Leagh", 0.0 },
  { "Leah", 0.0019573446 },
  { "Leaha", 0.0 },
  { "Leahana", 0.0 },
  { "Leahann", 0.0 },
  { "Leahanna", 0.0 },
  { "Leahgrace", 0.0 },
  { "Leahla", 0.0 },
  { "Leahmarie", 0.0 },
  { "Leahna", 0.0 },
  { "Leahnna", 0.0 },
  { "Leahrae", 0.0 },
  { "Leahrose", 0.0 },
  { "Leaira", 0.0 },
  { "Leairah", 0.0 },
  { "Leaisha", 0.0 },
  { "Leaja", 0.0 },
  { "Leajah", 0.0 },
  { "Leal", 0.5106383 },
  { "Leala", 0.0 },
  { "Lealah", 0.0 },
  { "Lealan", 1.0 },
  { "Lealand", 1.0 },
  { "Lealani", 0.0 },
  { "Lealer", 0.0 },
  { "Lealia", 0.0 },
  { "Lealo", 0.0 },
  { "Lealon", 1.0 },
  { "Leam", 1.0 },
  { "Leaman", 1.0 },
  { "Leamarie", 0.0 },
  { "Leamber", 0.0 },
  { "Leamon", 1.0 },
  { "Leamond", 1.0 },
  { "Leamsi", 0.0 },
  { "Lean", 0.1707953 },
  { "Leana", 0.0 },
  { "Leanah", 0.0 },
  { "Leanard", 1.0 },
  { "Leanda", 0.0 },
  { "Leander", 0.9954061 },
  { "Leandra", 0.005680969 },
  { "Leandrae", 1.0 },
  { "Leandre", 0.98978204 },
  { "Leandrea", 0.00729927 },
  { "Leandrew", 1.0 },
  { "Leandria", 0.0 },
  { "Leandro", 1.0 },
  { "Leandros", 1.0 },
  { "Leane", 0.0 },
  { "Leaner", 0.0 },
  { "Leanetta", 0.0 },
  { "Leanette", 0.0 },
  { "Leangela", 0.0 },
  { "Leangelo", 1.0 },
  { "Leani", 0.0 },
  { "Leanie", 0.0 },
  { "Leann", 0.00029229998 },
  { "Leanna", 0.00021611342 },
  { "Leannah", 0.0 },
  { "Leanndra", 0.0 },
  { "Leanne", 0.0 },
  { "Leanner", 0.0 },
  { "Leanni", 0.0 },
  { "Leannie", 0.0 },
  { "Leanny", 0.0 },
  { "Leanor", 0.0 },
  { "Leanora", 0.0 },
  { "Leanord", 1.0 },
  { "Leanore", 0.0 },
  { "Leanthony", 1.0 },
  { "Leany", 0.0 },
  { "Leanza", 0.0 },
  { "Leaon", 1.0 },
  { "Lear", 0.06831119 },
  { "Leara", 0.0 },
  { "Learah", 0.0 },
  { "Learlean", 0.0 },
  { "Learlene", 0.0 },
  { "Learline", 0.0 },
  { "Learon", 1.0 },
  { "Learoy", 1.0 },
  { "Leart", 1.0 },
  { "Learta", 0.0 },
  { "Leartis", 1.0 },
  { "Leary", 0.9238095 },
  { "Leasa", 0.0 },
  { "Leaser", 0.0 },
  { "Leasha", 0.0 },
  { "Leashia", 0.0 },
  { "Leasia", 0.0 },
  { "Leasie", 0.0 },
  { "Leaster", 0.31034482 },
  { "Leata", 0.0 },
  { "Leath", 0.0 },
  { "Leatha", 0.0 },
  { "Leathan", 1.0 },
  { "Leathe", 0.0 },
  { "Leathea", 0.0 },
  { "Leather", 0.0 },
  { "Leathia", 0.0 },
  { "Leathie", 0.0 },
  { "Leathy", 0.0 },
  { "Leaton", 1.0 },
  { "Leatrice", 0.0 },
  { "Leatricia", 0.0 },
  { "Leatta", 0.0 },
  { "Leauna", 0.0 },
  { "Leaundra", 0.0 },
  { "Leaunna", 0.0 },
  { "Leavie", 0.0 },
  { "Leavitt", 1.0 },
  { "Leavon", 1.0 },
  { "Leavy", 0.9657143 },
  { "Leaya", 0.0 },
  { "Leayah", 0.0 },
  { "Leayla", 0.0 },
  { "Leaysia", 0.0 },
  { "Leba", 0.0 },
  { "Lebaron", 1.0 },
  { "Lebarron", 1.0 },
  { "Lebern", 1.0 },
  { "Lebert", 1.0 },
  { "Leberta", 0.0 },
  { "Lebrandon", 1.0 },
  { "Lebrea", 0.0 },
  { "Lebria", 0.0 },
  { "Lebron", 1.0 },
  { "Lebrone", 1.0 },
  { "Leburn", 1.0 },
  { "Leca", 0.0 },
  { "Lecedric", 1.0 },
  { "Lecester", 1.0 },
  { "Lech", 1.0 },
  { "Lecharles", 1.0 },
  { "Lechelle", 0.0 },
  { "Lechia", 0.0 },
  { "Lecia", 0.0 },
  { "Lecie", 0.0 },
  { "Lecil", 1.0 },
  { "Leck", 1.0 },
  { "Lecole", 0.0 },
  { "Leconte", 1.0 },
  { "Lecory", 1.0 },
  { "Lecresha", 0.0 },
  { "Lecretia", 0.0 },
  { "Lecta", 0.0 },
  { "Lector", 0.0 },
  { "Lectoria", 0.0 },
  { "Lecy", 0.0 },
  { "Leda", 0.0 },
  { "Ledaniel", 1.0 },
  { "Ledarion", 1.0 },
  { "Ledarius", 1.0 },
  { "Ledarrius", 1.0 },
  { "Ledawn", 0.0 },
  { "Leddi", 0.0 },
  { "Leddie", 0.0 },
  { "Leddy", 0.0 },
  { "Ledeja", 0.0 },
  { "Ledel", 1.0 },
  { "Ledell", 0.96263343 },
  { "Lederrick", 1.0 },
  { "Ledford", 1.0 },
  { "Ledge", 1.0 },
  { "Ledger", 1.0 },
  { "Ledia", 0.0 },
  { "Ledion", 1.0 },
  { "Ledon", 1.0 },
  { "Ledonia", 0.0 },
  { "Ledonna", 0.0 },
  { "Ledora", 0.0 },
  { "Ledra", 0.0 },
  { "Lee", 0.7877179 },
  { "Leea", 0.0 },
  { "Leeah", 0.0 },
  { "Leeaira", 0.0 },
  { "Leeam", 1.0 },
  { "Leean", 0.0 },
  { "Leeana", 0.0 },
  { "Leeanah", 0.0 },
  { "Leeander", 1.0 },
  { "Leeandra", 0.0 },
  { "Leeandre", 1.0 },
  { "Leeandrea", 0.0 },
  { "Leeandrew", 1.0 },
  { "Leeani", 0.0 },
  { "Leeann", 0.0 },
  { "Leeanna", 0.0 },
  { "Leeannah", 0.0 },
  { "Leeanne", 0.0 },
  { "Leeanthony", 1.0 },
  { "Leeara", 0.0 },
  { "Leeasia", 0.0 },
  { "Leeat", 0.0 },
  { "Leeba", 0.0 },
  { "Leecy", 0.0 },
  { "Leeda", 0.0 },
  { "Leedell", 1.0 },
  { "Leedward", 1.0 },
  { "Leeila", 0.0 },
  { "Leejay", 1.0 },
  { "Leela", 0.0 },
  { "Leelah", 0.0 },
  { "Leelan", 1.0 },
  { "Leeland", 0.9950406 },
  { "Leelani", 0.0 },
  { "Leelee", 0.0 },
  { "Leelen", 1.0 },
  { "Leelend", 1.0 },
  { "Leelin", 1.0 },
  { "Leella", 0.0 },
  { "Leellen", 0.0 },
  { "Leeloo", 0.0 },
  { "Leelou", 0.0 },
  { "Leelu", 0.0 },
  { "Leelyn", 0.90384614 },
  { "Leelynd", 1.0 },
  { "Leelynn", 0.65873015 },
  { "Leeman", 1.0 },
  { "Leemarie", 0.0 },
  { "Leemichael", 1.0 },
  { "Leemon", 1.0 },
  { "Leen", 0.0 },
  { "Leena", 0.0 },
  { "Leenah", 0.0 },
  { "Leenora", 0.0 },
  { "Leeo", 1.0 },
  { "Leeola", 0.0 },
  { "Leeoma", 0.0 },
  { "Leeon", 1.0 },
  { "Leeona", 0.0 },
  { "Leeonna", 0.0 },
  { "Leeor", 0.516129 },
  { "Leequan", 1.0 },
  { "Leeric", 1.0 },
  { "Leeron", 1.0 },
  { "Leeroy", 1.0 },
  { "Leesa", 0.0 },
  { "Leesha", 0.0 },
  { "Leeshaun", 1.0 },
  { "Leeshawn", 1.0 },
  { "Leesly", 0.0 },
  { "Leeson", 1.0 },
  { "Leester", 0.0 },
  { "Leeta", 0.0 },
  { "Leetal", 0.0 },
  { "Leetha", 0.0 },
  { "Leetta", 0.0 },
  { "Leeum", 1.0 },
  { "Leevan", 1.0 },
  { "Leevi", 1.0 },
  { "Leevon", 1.0 },
  { "Leeward", 1.0 },
  { "Leeya", 0.0 },
  { "Leeyah", 0.0 },
  { "Leeyana", 0.0 },
  { "Leeza", 0.0 },
  { "Lefa", 0.0 },
  { "Lefty", 1.0 },
  { "Legacee", 0.0 },
  { "Legaci", 0.0 },
  { "Legacie", 0.0 },
  { "Legacy", 0.21232417 },
  { "Legen", 1.0 },
  { "Legend", 0.907743 },
  { "Legendary", 1.0 },
  { "Legina", 0.0 },
  { "Legion", 1.0 },
  { "Legna", 0.0 },
  { "Legolas", 1.0 },
  { "Legrand", 1.0 },
  { "Legrande", 1.0 },
  { "Legrant", 1.0 },
  { "Leha", 0.0 },
  { "Lehana", 0.0 },
  { "Lehanna", 0.0 },
  { "Lehi", 1.0 },
  { "Lehiwa", 0.0 },
  { "Lehlani", 0.0 },
  { "Lehman", 1.0 },
  { "Lehmon", 1.0 },
  { "Lehna", 0.0 },
  { "Lehua", 0.0 },
  { "Lehuanani", 0.0 },
  { "Lei", 0.04718417 },
  { "Leia", 0.0 },
  { "Leiah", 0.0 },
  { "Leiahna", 0.0 },
  { "Leialoha", 0.0 },
  { "Leiam", 1.0 },
  { "Leiana", 0.0 },
  { "Leianah", 0.0 },
  { "Leiani", 0.0 },
  { "Leiann", 0.0 },
  { "Leianna", 0.0 },
  { "Leianne", 0.0 },
  { "Leiara", 0.0 },
  { "Leiasia", 0.0 },
  { "Leib", 1.0 },
  { "Leibish", 1.0 },
  { "Leiby", 1.0 },
  { "Leica", 0.0 },
  { "Leicha", 0.0 },
  { "Leicia", 0.0 },
  { "Leida", 0.0 },
  { "Leiden", 1.0 },
  { "Leidi", 0.0 },
  { "Leidy", 0.0 },
  { "Leif", 0.9993607 },
  { "Leigh", 0.113018885 },
  { "Leigha", 0.0 },
  { "Leigham", 1.0 },
  { "Leighan", 0.0 },
  { "Leighana", 0.0 },
  { "Leighann", 0.0 },
  { "Leighanna", 0.0 },
  { "Leighanne", 0.0 },
  { "Leighia", 0.0 },
  { "Leighla", 0.0 },
  { "Leighlah", 0.0 },
  { "Leighlan", 1.0 },
  { "Leighland", 1.0 },
  { "Leighlani", 0.0 },
  { "Leighlynn", 0.0 },
  { "Leighna", 0.0 },
  { "Leightan", 0.0 },
  { "Leighton", 0.56858426 },
  { "Leightyn", 0.0 },
  { "Leiha", 0.0 },
  { "Leihla", 0.0 },
  { "Leihlani", 0.0 },
  { "Leihum", 1.0 },
  { "Leila", 0.00013781065 },
  { "Leilaann", 0.0 },
  { "Leilah", 0.0 },
  { "Leilahni", 0.0 },
  { "Leilamarie", 0.0 },
  { "Leilan", 1.0 },
  { "Leilana", 0.0 },
  { "Leiland", 1.0 },
  { "Leilane", 0.0 },
  { "Leilanee", 0.0 },
  { "Leilaney", 0.0 },
  { "Leilani", 0.0006793721 },
  { "Leilanie", 0.0 },
  { "Leilanii", 0.0 },
  { "Leilanni", 0.0 },
  { "Leilannie", 0.0 },
  { "Leilanny", 0.0 },
  { "Leilany", 0.0 },
  { "Leilarose", 0.0 },
  { "Leilauni", 0.0 },
  { "Leileen", 0.0 },
  { "Leilena", 0.0 },
  { "Leilend", 1.0 },
  { "Leilene", 0.0 },
  { "Leili", 0.0 },
  { "Leilia", 0.0 },
  { "Leiliana", 0.0 },
  { "Leiliani", 0.0 },
  { "Leiliany", 0.0 },
  { "Leilla", 0.0 },
  { "Leiloni", 0.0 },
  { "Leilonie", 0.0 },
  { "Leilonnie", 0.0 },
  { "Leily", 0.0 },
  { "Leilyn", 0.0 },
  { "Leilynd", 1.0 },
  { "Leilynn", 0.0 },
  { "Leim", 1.0 },
  { "Leimomi", 0.0 },
  { "Leina", 0.0 },
  { "Leinaala", 0.0 },
  { "Leinani", 0.0 },
  { "Leini", 0.0 },
  { "Leiny", 0.0 },
  { "Leion", 1.0 },
  { "Leiona", 0.0 },
  { "Leionna", 0.0 },
  { "Leira", 0.0 },
  { "Leire", 0.0 },
  { "Leisa", 0.0 },
  { "Leise", 0.0 },
  { "Leisel", 0.0 },
  { "Leisha", 0.0 },
  { "Leishla", 0.0 },
  { "Leisl", 0.0 },
  { "Leisly", 0.0 },
  { "Leissa", 0.0 },
  { "Leisy", 0.0 },
  { "Leita", 0.0 },
  { "Leith", 0.8286604 },
  { "Leitha", 0.0 },
  { "Leithan", 1.0 },
  { "Leium", 1.0 },
  { "Leiya", 0.0 },
  { "Leiyah", 0.0 },
  { "Leiyana", 0.0 },
  { "Leiyla", 0.0 },
  { "Leizel", 0.0 },
  { "Lejend", 1.0 },
  { "Lejeune", 0.45454547 },
  { "Lejla", 0.0 },
  { "Lejon", 1.0 },
  { "Lejoyce", 0.0 },
  { "Lejuan", 1.0 },
  { "Lekai", 1.0 },
  { "Lekayla", 0.0 },
  { "Lekecia", 0.0 },
  { "Lekedia", 0.0 },
  { "Lekeesha", 0.0 },
  { "Lekeia", 0.0 },
  { "Lekeisha", 0.0 },
  { "Lekeith", 1.0 },
  { "Lekeitha", 0.0 },
  { "Lekendra", 0.0 },
  { "Lekendric", 1.0 },
  { "Lekendrick", 1.0 },
  { "Lekenya", 0.0 },
  { "Lekesha", 0.0 },
  { "Lekeshia", 0.0 },
  { "Lekesia", 0.0 },
  { "Leketa", 0.0 },
  { "Leketha", 0.0 },
  { "Lekethia", 0.0 },
  { "Lekevia", 0.0 },
  { "Lekeya", 0.0 },
  { "Lekeysha", 0.0 },
  { "Lekha", 0.0 },
  { "Lekhi", 1.0 },
  { "Lekia", 0.0 },
  { "Lekiesha", 0.0 },
  { "Lekisa", 0.0 },
  { "Lekisha", 0.0 },
  { "Lekishia", 0.0 },
  { "Lekita", 0.0 },
  { "Leksi", 0.0 },
  { "Lela", 0.0016155856 },
  { "Lelah", 0.0 },
  { "Lelaina", 0.0 },
  { "Lelamae", 0.0 },
  { "Lelan", 1.0 },
  { "Lelana", 0.0 },
  { "Leland", 0.9919418 },
  { "Lelandra", 0.0 },
  { "Lelani", 0.0 },
  { "Lelania", 0.0 },
  { "Lelanie", 0.0 },
  { "Lelanya", 0.0 },
  { "Lelar", 0.0 },
  { "Leldon", 1.0 },
  { "Lelea", 0.0 },
  { "Lelen", 1.0 },
  { "Lelend", 1.0 },
  { "Leler", 0.0 },
  { "Lelia", 0.0 },
  { "Leliana", 0.0 },
  { "Lelianna", 0.0 },
  { "Lelie", 0.0 },
  { "Lelind", 1.0 },
  { "Lelita", 0.0 },
  { "Lella", 0.0 },
  { "Leller", 0.0 },
  { "Lellie", 0.0 },
  { "Lelon", 1.0 },
  { "Leloni", 0.0 },
  { "Lelsie", 0.0 },
  { "Lelton", 1.0 },
  { "Lelu", 0.0 },
  { "Lelyn", 1.0 },
  { "Lelynd", 1.0 },
  { "Lelynn", 1.0 },
  { "Lem", 1.0 },
  { "Lema", 0.0 },
  { "Lemaj", 1.0 },
  { "Leman", 1.0 },
  { "Lemanuel", 1.0 },
  { "Lemar", 1.0 },
  { "Lemarco", 1.0 },
  { "Lemarcus", 1.0 },
  { "Lemari", 1.0 },
  { "Lemario", 1.0 },
  { "Lemarion", 1.0 },
  { "Lemark", 1.0 },
  { "Lemarr", 1.0 },
  { "Lemaya", 0.0 },
  { "Lemel", 1.0 },
  { "Lemia", 0.0 },
  { "Lemichael", 1.0 },
  { "Lemisha", 0.0 },
  { "Lemma", 0.0 },
  { "Lemmie", 0.8366834 },
  { "Lemmon", 1.0 },
  { "Lemmy", 0.7692308 },
  { "Lemoine", 1.0 },
  { "Lemon", 0.9469496 },
  { "Lemond", 1.0 },
  { "Lemonica", 0.0 },
  { "Lemont", 1.0 },
  { "Lemonte", 1.0 },
  { "Lemorris", 1.0 },
  { "Lemoyne", 0.85202867 },
  { "Lempi", 0.0 },
  { "Lemual", 1.0 },
  { "Lemuel", 1.0 },
  { "Lemya", 0.0 },
  { "Len", 0.97642654 },
  { "Lena", 0.0023911514 },
  { "Lenabelle", 0.0 },
  { "Lenae", 0.0 },
  { "Lenah", 0.0 },
  { "Lenai", 0.0 },
  { "Lenalee", 0.0 },
  { "Lenamae", 0.0 },
  { "Lenamarie", 0.0 },
  { "Lenar", 0.0 },
  { "Lenard", 0.9992946 },
  { "Lenardo", 1.0 },
  { "Lenasia", 0.0 },
  { "Lenay", 0.0 },
  { "Lenaya", 0.0 },
  { "Lenayah", 0.0 },
  { "Lenci", 0.0 },
  { "Lencola", 0.0 },
  { "Lency", 0.0 },
  { "Lenda", 0.0 },
  { "Lendal", 1.0 },
  { "Lendale", 1.0 },
  { "Lendall", 1.0 },
  { "Lendel", 1.0 },
  { "Lendell", 1.0 },
  { "Lenden", 1.0 },
  { "Lender", 0.0 },
  { "Lendia", 0.0 },
  { "Lendol", 1.0 },
  { "Lendon", 1.0 },
  { "Lendora", 0.0 },
  { "Lendra", 0.0 },
  { "Lendsey", 0.0 },
  { "Lendy", 0.0 },
  { "Lene", 0.0 },
  { "Lenea", 0.0 },
  { "Leneah", 0.0 },
  { "Lenear", 1.0 },
  { "Lenecia", 0.0 },
  { "Lenee", 0.0 },
  { "Leneisha", 0.0 },
  { "Leneka", 0.0 },
  { "Lenell", 0.6111111 },
  { "Lenelle", 0.0 },
  { "Lener", 0.0 },
  { "Lenes", 1.0 },
  { "Lenesha", 0.0 },
  { "Leneta", 0.0 },
  { "Lenett", 0.0 },
  { "Lenetta", 0.0 },
  { "Lenette", 0.0 },
  { "Leneve", 0.0 },
  { "Lenford", 1.0 },
  { "Leng", 0.9848024 },
  { "Leni", 0.019607844 },
  { "Lenia", 0.0 },
  { "Leniah", 0.0 },
  { "Lenice", 0.0 },
  { "Leniece", 0.0 },
  { "Leniel", 1.0 },
  { "Lenika", 0.0 },
  { "Lenin", 0.9958054 },
  { "Lenina", 0.0 },
  { "Lenis", 0.75555557 },
  { "Lenise", 0.0 },
  { "Lenisha", 0.0 },
  { "Lenita", 0.0 },
  { "Lenix", 0.6363636 },
  { "Leniya", 0.0 },
  { "Leniyah", 0.0 },
  { "Lenka", 0.0 },
  { "Lenley", 1.0 },
  { "Lenn", 0.98832685 },
  { "Lenna", 0.0 },
  { "Lennan", 0.0 },
  { "Lennard", 1.0 },
  { "Lennart", 1.0 },
  { "Lennea", 0.0 },
  { "Lennel", 1.0 },
  { "Lennell", 0.8951049 },
  { "Lenner", 0.0 },
  { "Lenneth", 1.0 },
  { "Lennett", 0.0 },
  { "Lennetta", 0.0 },
  { "Lennette", 0.0 },
  { "Lennex", 0.57575756 },
  { "Lenni", 0.0 },
  { "Lennia", 0.0 },
  { "Lennice", 0.0 },
  { "Lennie", 0.38677707 },
  { "Lennin", 0.97115386 },
  { "Lennis", 0.69384265 },
  { "Lennix", 0.53612167 },
  { "Lennon", 0.64125437 },
  { "Lennora", 0.0 },
  { "Lennox", 0.7420279 },
  { "Lenny", 0.9503203 },
  { "Lennyn", 0.3957219 },
  { "Lennyx", 0.2 },
  { "Leno", 0.99033815 },
  { "Lenoard", 1.0 },
  { "Lenoir", 0.0 },
  { "Lenola", 0.0 },
  { "Lenon", 1.0 },
  { "Lenona", 0.0 },
  { "Lenor", 0.0 },
  { "Lenora", 0.00090087473 },
  { "Lenorah", 0.0 },
  { "Lenord", 1.0 },
  { "Lenore", 0.00026857655 },
  { "Lenoria", 0.0 },
  { "Lenoris", 1.0 },
  { "Lenorris", 1.0 },
  { "Lenox", 0.8117409 },
  { "Lenoxx", 1.0 },
  { "Lens", 1.0 },
  { "Lensa", 0.0 },
  { "Lensey", 0.0 },
  { "Lensy", 1.0 },
  { "Lent", 1.0 },
  { "Lenton", 1.0 },
  { "Lenuel", 1.0 },
  { "Lenus", 1.0 },
  { "Lenvil", 1.0 },
  { "Lenville", 1.0 },
  { "Lenward", 1.0 },
  { "Lenwood", 1.0 },
  { "Leny", 0.721519 },
  { "Lenya", 0.0 },
  { "Lenyn", 0.625 },
  { "Lenyx", 0.21359223 },
  { "Lenz", 1.0 },
  { "Lenzi", 0.0 },
  { "Lenzie", 0.6757246 },
  { "Lenzy", 0.8963532 },
  { "Leo", 0.9901413 },
  { "Leoanthony", 1.0 },
  { "Leoba", 0.0 },
  { "Leobardo", 1.0 },
  { "Leoben", 1.0 },
  { "Leocadia", 0.0 },
  { "Leocadio", 1.0 },
  { "Leoda", 0.0 },
  { "Leodan", 1.0 },
  { "Leodegario", 1.0 },
  { "Leodis", 1.0 },
  { "Leodore", 1.0 },
  { "Leoh", 1.0 },
  { "Leokadia", 0.0 },
  { "Leokadja", 0.0 },
  { "Leokadya", 0.0 },
  { "Leola", 0.0018592012 },
  { "Leolani", 0.0 },
  { "Leolar", 0.0 },
  { "Leoler", 0.0 },
  { "Leolia", 0.0 },
  { "Leoma", 0.0 },
  { "Leomar", 1.0 },
  { "Leomia", 0.0 },
  { "Leon", 0.99137104 },
  { "Leona", 0.0027209122 },
  { "Leonah", 0.0 },
  { "Leonaldo", 1.0 },
  { "Leonara", 0.0 },
  { "Leonard", 0.99516386 },
  { "Leonarda", 0.0 },
  { "Leonardo", 0.997813 },
  { "Leonce", 1.0 },
  { "Leoncio", 1.0 },
  { "Leonda", 0.0 },
  { "Leondra", 0.006596306 },
  { "Leondrae", 1.0 },
  { "Leondre", 1.0 },
  { "Leondria", 0.0 },
  { "Leondro", 1.0 },
  { "Leondrus", 1.0 },
  { "Leone", 0.041110937 },
  { "Leonel", 0.9997273 },
  { "Leonela", 0.0 },
  { "Leonell", 1.0 },
  { "Leonella", 0.0 },
  { "Leoner", 0.0 },
  { "Leonetta", 0.0 },
  { "Leonette", 0.0 },
  { "Leonhard", 1.0 },
  { "Leoni", 0.0 },
  { "Leonia", 0.0 },
  { "Leonice", 0.0 },
  { "Leonid", 1.0 },
  { "Leonida", 0.0 },
  { "Leonidas", 1.0 },
  { "Leonide", 1.0 },
  { "Leonides", 0.96989965 },
  { "Leonidis", 1.0 },
  { "Leonidus", 1.0 },
  { "Leonie", 0.0 },
  { "Leonila", 0.0 },
  { "Leonilda", 0.0 },
  { "Leonilla", 0.0 },
  { "Leonis", 0.0 },
  { "Leonise", 0.0 },
  { "Leonisha", 0.0 },
  { "Leonita", 0.0 },
  { "Leonitus", 1.0 },
  { "Leonna", 0.0 },
  { "Leonnah", 0.0 },
  { "Leonne", 0.0 },
  { "Leonnie", 0.0 },
  { "Leonor", 0.0017689723 },
  { "Leonora", 0.0 },
  { "Leonord", 1.0 },
  { "Leonore", 0.0 },
  { "Leonta", 1.0 },
  { "Leontae", 1.0 },
  { "Leontay", 1.0 },
  { "Leonte", 1.0 },
  { "Leontina", 0.0 },
  { "Leontine", 0.0 },
  { "Leontre", 1.0 },
  { "Leontyne", 0.0 },
  { "Leonydus", 1.0 },
  { "Leonzo", 1.0 },
  { "Leopold", 1.0 },
  { "Leopoldo", 1.0 },
  { "Leor", 0.90804595 },
  { "Leora", 0.000544119 },
  { "Leorah", 0.0 },
  { "Leoria", 0.0 },
  { "Leorn", 1.0 },
  { "Leory", 1.0 },
  { "Leosha", 0.0 },
  { "Leosvel", 1.0 },
  { "Leota", 0.0 },
  { "Leotha", 0.24485597 },
  { "Leother", 0.0 },
  { "Leotie", 0.0 },
  { "Leotis", 0.9943946 },
  { "Leotta", 0.0 },
  { "Leoul", 1.0 },
  { "Leovani", 1.0 },
  { "Leovanni", 1.0 },
  { "Leovardo", 1.0 },
  { "Leovigildo", 1.0 },
  { "Lepha", 0.0 },
  { "Lequan", 0.98734176 },
  { "Lequentin", 1.0 },
  { "Lequesha", 0.0 },
  { "Lequetta", 0.0 },
  { "Lequient", 1.0 },
  { "Lequinta", 0.0 },
  { "Lequinton", 1.0 },
  { "Lequisha", 0.0 },
  { "Lequita", 0.0 },
  { "Lequitta", 0.0 },
  { "Lequon", 1.0 },
  { "Lera", 0.0 },
  { "Lerae", 0.0 },
  { "Lerah", 0.0 },
  { "Lerald", 1.0 },
  { "Leray", 1.0 },
  { "Lerena", 0.0 },
  { "Lerenzo", 1.0 },
  { "Leresa", 0.0 },
  { "Leretha", 0.0 },
  { "Lerhonda", 0.0 },
  { "Leri", 1.0 },
  { "Leriah", 0.0 },
  { "Lerico", 1.0 },
  { "Lerin", 0.12946428 },
  { "Lerissa", 0.0 },
  { "Lerita", 0.0 },
  { "Leriyah", 0.0 },
  { "Lerlean", 0.0 },
  { "Lerlene", 0.0 },
  { "Lerline", 0.0 },
  { "Leroi", 1.0 },
  { "Lerome", 1.0 },
  { "Leron", 1.0 },
  { "Leronda", 0.0 },
  { "Lerone", 1.0 },
  { "Leronia", 0.0 },
  { "Leroy", 0.99408334 },
  { "Leroya", 0.0 },
  { "Leroyal", 1.0 },
  { "Leroyce", 1.0 },
  { "Lerry", 1.0 },
  { "Leryn", 0.0 },
  { "Les", 0.9905993 },
  { "Lesa", 0.0010515247 },
  { "Lesandra", 0.0 },
  { "Lesane", 1.0 },
  { "Lesbia", 0.0 },
  { "Lesean", 0.97126436 },
  { "Lesedi", 0.0 },
  { "Lesette", 0.0 },
  { "Lesha", 0.0 },
  { "Leshae", 0.0 },
  { "Leshan", 0.4074074 },
  { "Leshanda", 0.0 },
  { "Leshanta", 0.0 },
  { "Leshaun", 0.8914729 },
  { "Leshauna", 0.0 },
  { "Leshaunda", 0.0 },
  { "Leshaundra", 0.0 },
  { "Leshawn", 0.7092265 },
  { "Leshawna", 0.0 },
  { "Leshawnda", 0.0 },
  { "Leshay", 0.0 },
  { "Leshe", 0.0 },
  { "Leshea", 0.0 },
  { "Lesheena", 0.0 },
  { "Lesheka", 0.0 },
  { "Leshell", 0.0 },
  { "Leshelle", 0.0 },
  { "Leshia", 0.0 },
  { "Leshly", 0.0 },
  { "Leshon", 0.7407407 },
  { "Leshonda", 0.0 },
  { "Leshun", 0.75 },
  { "Leshunda", 0.0 },
  { "Lesi", 0.0 },
  { "Lesia", 0.0 },
  { "Lesie", 0.0 },
  { "Lesieli", 0.0 },
  { "Lesile", 0.1 },
  { "Lesle", 0.0 },
  { "Leslea", 0.0 },
  { "Leslee", 0.01941355 },
  { "Leslei", 0.0 },
  { "Lesleigh", 0.0 },
  { "Lesley", 0.12396289 },
  { "Lesleyann", 0.0 },
  { "Lesleyanne", 0.0 },
  { "Lesli", 0.0 },
  { "Leslian", 0.0 },
  { "Lesliann", 0.0 },
  { "Leslianne", 0.0 },
  { "Leslie", 0.29767135 },
  { "Leslieann", 0.0 },
  { "Leslieanne", 0.0 },
  { "Lesliee", 0.0 },
  { "Leslly", 0.0 },
  { "Lesly", 0.035347134 },
  { "Leslyann", 0.0 },
  { "Leslye", 0.0 },
  { "Leslyn", 0.0 },
  { "Less", 1.0 },
  { "Lessa", 0.0 },
  { "Lessette", 0.0 },
  { "Lessia", 0.0 },
  { "Lessie", 0.062895596 },
  { "Lessley", 0.44329897 },
  { "Lessli", 0.0 },
  { "Lesslie", 0.22397476 },
  { "Lessly", 0.0 },
  { "Lesta", 0.0 },
  { "Lestat", 1.0 },
  { "Lester", 0.98963135 },
  { "Lesther", 1.0 },
  { "Lestie", 0.0 },
  { "Lestina", 0.0 },
  { "Lestine", 0.0 },
  { "Leston", 1.0 },
  { "Lesvia", 0.0 },
  { "Leta", 0.0 },
  { "Letah", 0.0 },
  { "Letanya", 0.0 },
  { "Letara", 0.0 },
  { "Letarsha", 0.0 },
  { "Letasha", 0.0 },
  { "Letavia", 0.0 },
  { "Letcher", 1.0 },
  { "Letecia", 0.0 },
  { "Leteisha", 0.0 },
  { "Letesha", 0.0 },
  { "Leteshia", 0.0 },
  { "Letetia", 0.0 },
  { "Letha", 0.0013056194 },
  { "Lethaniel", 1.0 },
  { "Lethe", 0.0 },
  { "Lethea", 0.0 },
  { "Lethel", 1.0 },
  { "Lethell", 1.0 },
  { "Lether", 0.020325202 },
  { "Lethia", 0.0 },
  { "Lethie", 0.0 },
  { "Leti", 0.0 },
  { "Letia", 0.0 },
  { "Letica", 0.0 },
  { "Leticha", 0.0 },
  { "Leticia", 0.005111016 },
  { "Letina", 0.0 },
  { "Letisa", 0.0 },
  { "Letisha", 0.0 },
  { "Letishia", 0.0 },
  { "Letisia", 0.0 },
  { "Letita", 0.0 },
  { "Letitia", 0.0 },
  { "Letizia", 0.0 },
  { "Leto", 1.0 },
  { "Letonia", 0.0 },
  { "Letonya", 0.0 },
  { "Letoria", 0.0 },
  { "Letosha", 0.0 },
  { "Letoya", 0.0 },
  { "Letrece", 0.0 },
  { "Letrecia", 0.0 },
  { "Letrell", 1.0 },
  { "Letrese", 0.0 },
  { "Letriana", 0.0 },
  { "Letrica", 0.0 },
  { "Letrice", 0.0 },
  { "Letricia", 0.0 },
  { "Letrina", 0.0 },
  { "Letrisha", 0.0 },
  { "Letroy", 1.0 },
  { "Letta", 0.0 },
  { "Letti", 0.0 },
  { "Lettia", 0.0 },
  { "Lettica", 0.0 },
  { "Lettice", 0.0 },
  { "Letticia", 0.0 },
  { "Lettie", 0.0 },
  { "Letty", 0.0 },
  { "Lettye", 0.0 },
  { "Letwan", 1.0 },
  { "Lety", 0.0 },
  { "Letycia", 0.0 },
  { "Letzy", 0.0 },
  { "Leudy", 1.0 },
  { "Leul", 1.0 },
  { "Leum", 1.0 },
  { "Leuna", 0.0 },
  { "Leura", 0.0 },
  { "Leuvenia", 0.0 },
  { "Lev", 0.98951614 },
  { "Leva", 0.0 },
  { "Levada", 0.0 },
  { "Levaeh", 0.0 },
  { "Levan", 1.0 },
  { "Levana", 0.0 },
  { "Levance", 1.0 },
  { "Levander", 1.0 },
  { "Levane", 1.0 },
  { "Levani", 0.0 },
  { "Levant", 1.0 },
  { "Levante", 1.0 },
  { "Levar", 0.9954522 },
  { "Levarn", 0.71428573 },
  { "Levarr", 1.0 },
  { "Levata", 0.0 },
  { "Levater", 0.0 },
  { "Levaughn", 1.0 },
  { "Levaun", 0.0 },
  { "Leve", 1.0 },
  { "Leveah", 0.0 },
  { "Levearn", 0.0 },
  { "Leveda", 0.0 },
  { "Levee", 0.0 },
  { "Levell", 1.0 },
  { "Levella", 0.0 },
  { "Levelle", 1.0 },
  { "Leven", 0.9632353 },
  { "Levena", 0.0 },
  { "Levenia", 0.0 },
  { "Levent", 1.0 },
  { "Levente", 1.0 },
  { "Leveon", 1.0 },
  { "Levera", 0.0 },
  { "Leverda", 0.0 },
  { "Levere", 1.0 },
  { "Leverett", 1.0 },
  { "Leverette", 1.0 },
  { "Leveria", 0.0 },
  { "Levern", 0.8326763 },
  { "Leverna", 0.0 },
  { "Leverne", 0.6691576 },
  { "Levert", 0.99156827 },
  { "Leverta", 0.0 },
  { "Levertis", 1.0 },
  { "Levester", 0.992891 },
  { "Leveta", 0.0 },
  { "Levetta", 0.0 },
  { "Levette", 0.0 },
  { "Levi", 0.9938932 },
  { "Levia", 0.0 },
  { "Leviah", 0.0 },
  { "Levian", 0.59375 },
  { "Leviathan", 1.0 },
  { "Levicy", 0.0 },
  { "Levid", 1.0 },
  { "Levie", 0.8447619 },
  { "Levii", 1.0 },
  { "Levin", 1.0 },
  { "Levina", 0.0 },
  { "Levine", 1.0 },
  { "Levinia", 0.0 },
  { "Levio", 1.0 },
  { "Levion", 1.0 },
  { "Levis", 0.94485295 },
  { "Levit", 1.0 },
  { "Levita", 0.0 },
  { "Leviticus", 1.0 },
  { "Levitt", 1.0 },
  { "Levity", 0.0 },
  { "Levolia", 0.0 },
  { "Levon", 0.9161665 },
  { "Levona", 0.0 },
  { "Levonda", 0.0 },
  { "Levone", 0.75 },
  { "Levonia", 0.0 },
  { "Levonna", 0.0 },
  { "Levonne", 0.07046071 },
  { "Levonta", 1.0 },
  { "Levontae", 1.0 },
  { "Levonte", 1.0 },
  { "Levora", 0.0 },
  { "Levorn", 0.8652482 },
  { "Levorne", 1.0 },
  { "Levoy", 1.0 },
  { "Levy", 0.9845774 },
  { "Levyn", 1.0 },
  { "Lew", 0.99313813 },
  { "Lewana", 0.0 },
  { "Lewanda", 0.0 },
  { "Lewann", 0.0 },
  { "Lewanna", 0.0 },
  { "Leward", 1.0 },
  { "Lewayne", 1.0 },
  { "Lewell", 1.0 },
  { "Lewella", 0.0 },
  { "Lewellyn", 0.9023669 },
  { "Lewey", 1.0 },
  { "Lewi", 1.0 },
  { "Lewie", 1.0 },
  { "Lewin", 1.0 },
  { "Lewis", 0.9942418 },
  { "Lewyn", 1.0 },
  { "Lex", 0.99846673 },
  { "Lexa", 0.0 },
  { "Lexah", 0.0 },
  { "Lexander", 1.0 },
  { "Lexandra", 0.0 },
  { "Lexandria", 0.0 },
  { "Lexandro", 1.0 },
  { "Lexani", 0.01510574 },
  { "Lexanie", 0.0 },
  { "Lexanne", 0.0 },
  { "Lexanni", 0.0 },
  { "Lexany", 0.0 },
  { "Lexas", 0.0 },
  { "Lexcee", 0.0 },
  { "Lexci", 0.0 },
  { "Lexcie", 0.0 },
  { "Lexe", 0.0 },
  { "Lexee", 0.0 },
  { "Lexes", 0.0 },
  { "Lexey", 0.0 },
  { "Lexi", 0.0 },
  { "Lexia", 0.0 },
  { "Lexiana", 0.0 },
  { "Lexiann", 0.0 },
  { "Lexianna", 0.0 },
  { "Lexie", 0.029657278 },
  { "Lexiee", 0.0 },
  { "Lexiel", 1.0 },
  { "Lexii", 0.0 },
  { "Lexilynn", 0.0 },
  { "Lexin", 0.7777778 },
  { "Lexine", 0.0 },
  { "Lexington", 0.37916064 },
  { "Lexis", 0.013455657 },
  { "Lexiss", 0.0 },
  { "Lexius", 0.0 },
  { "Lexly", 0.0 },
  { "Lexon", 1.0 },
  { "Lexsey", 0.0 },
  { "Lexsie", 0.0 },
  { "Lexsis", 0.0 },
  { "Lexsy", 0.0 },
  { "Lexton", 1.0 },
  { "Lexus", 0.03979422 },
  { "Lexx", 1.0 },
  { "Lexxi", 0.0 },
  { "Lexxie", 0.0 },
  { "Lexxis", 0.0 },
  { "Lexxus", 0.0 },
  { "Lexxy", 0.0 },
  { "Lexy", 0.0 },
  { "Lexys", 0.0 },
  { "Leya", 0.0 },
  { "Leyah", 0.0 },
  { "Leyan", 0.0 },
  { "Leyana", 0.0 },
  { "Leyani", 0.0 },
  { "Leyann", 0.0 },
  { "Leyanna", 0.0 },
  { "Leyanni", 0.0 },
  { "Leyat", 0.0 },
  { "Leycester", 1.0 },
  { "Leyda", 0.0 },
  { "Leyden", 1.0 },
  { "Leydi", 0.0 },
  { "Leydy", 0.0 },
  { "Leyiah", 0.0 },
  { "Leyka", 0.0 },
  { "Leyla", 0.0 },
  { "Leylah", 0.0 },
  { "Leylan", 1.0 },
  { "Leyland", 1.0 },
  { "Leylani", 0.0 },
  { "Leylanie", 0.0 },
  { "Leylanni", 0.0 },
  { "Leylany", 0.0 },
  { "Leyli", 0.0 },
  { "Leyna", 0.0 },
  { "Leynah", 0.0 },
  { "Leyni", 0.0 },
  { "Leyona", 0.0 },
  { "Leyonna", 0.0 },
  { "Leyre", 0.0 },
  { "Leysha", 0.0 },
  { "Leyton", 0.88698953 },
  { "Leyver", 1.0 },
  { "Leza", 0.0 },
  { "Lezah", 0.0 },
  { "Lezanne", 0.0 },
  { "Lezer", 1.0 },
  { "Lezette", 0.0 },
  { "Lezlee", 0.0 },
  { "Lezley", 0.0 },
  { "Lezli", 0.0 },
  { "Lezlie", 0.0 },
  { "Lezly", 0.0 },
  { "Lg", 1.0 },
  { "Lhea", 0.0 },
  { "Lhiam", 1.0 },
  { "Li", 0.31043956 },
  { "Lia", 0.00058476115 },
  { "Liad", 1.0 },
  { "Liadan", 0.0 },
  { "Liah", 0.0 },
  { "Liahm", 1.0 },
  { "Liahna", 0.0 },
  { "Liahona", 0.0 },
  { "Liala", 0.0 },
  { "Lialah", 0.0 },
  { "Liam", 0.99842614 },
  { "Liamalexander", 1.0 },
  { "Liamgabriel", 1.0 },
  { "Liamjames", 1.0 },
  { "Liamm", 1.0 },
  { "Liammatthew", 1.0 },
  { "Liammichael", 1.0 },
  { "Lian", 0.57309616 },
  { "Liana", 0.0 },
  { "Lianabel", 0.0 },
  { "Lianah", 0.0 },
  { "Liandra", 0.0 },
  { "Liandro", 1.0 },
  { "Liane", 0.0 },
  { "Lianet", 0.0 },
  { "Lianett", 0.0 },
  { "Lianette", 0.0 },
  { "Liang", 1.0 },
  { "Liani", 0.0 },
  { "Liann", 0.0 },
  { "Lianna", 0.0 },
  { "Liannah", 0.0 },
  { "Lianne", 0.0 },
  { "Liannette", 0.0 },
  { "Lianni", 0.0 },
  { "Liannie", 0.0 },
  { "Lianny", 0.0 },
  { "Liany", 0.0 },
  { "Lianys", 0.0 },
  { "Liara", 0.0 },
  { "Liasia", 0.0 },
  { "Liat", 0.0 },
  { "Liav", 1.0 },
  { "Liba", 0.0 },
  { "Liban", 1.0 },
  { "Libbey", 0.0 },
  { "Libbi", 0.0 },
  { "Libbie", 0.0 },
  { "Libby", 0.00031397174 },
  { "Libera", 0.0 },
  { "Liberato", 1.0 },
  { "Libero", 1.0 },
  { "Liberta", 0.0 },
  { "Libertad", 0.0 },
  { "Libertee", 0.0 },
  { "Liberti", 0.0 },
  { "Libertie", 0.0 },
  { "Liberty", 0.012548754 },
  { "Libeth", 0.0 },
  { "Libi", 0.0 },
  { "Libia", 0.0 },
  { "Libna", 0.0 },
  { "Libni", 0.0 },
  { "Libny", 0.0 },
  { "Liboria", 0.0 },
  { "Liborio", 1.0 },
  { "Libra", 0.0 },
  { "Librada", 0.0 },
  { "Librado", 1.0 },
  { "Libya", 0.0 },
  { "Lica", 0.0 },
  { "Licet", 0.0 },
  { "Licette", 0.0 },
  { "Lichelle", 0.0 },
  { "Licia", 0.0 },
  { "Liczy", 0.0 },
  { "Lida", 0.0 },
  { "Liddia", 0.0 },
  { "Liddie", 0.0 },
  { "Liddy", 0.0 },
  { "Lidea", 0.0 },
  { "Lidell", 1.0 },
  { "Lidia", 0.0 },
  { "Lidiana", 0.0 },
  { "Lidie", 0.0 },
  { "Lidija", 0.0 },
  { "Lidio", 1.0 },
  { "Lidiya", 0.0 },
  { "Lido", 1.0 },
  { "Liduvina", 0.0 },
  { "Lidwina", 0.0 },
  { "Lidy", 0.0 },
  { "Lidya", 0.0 },
  { "Lieba", 0.0 },
  { "Liedson", 1.0 },
  { "Lief", 1.0 },
  { "Liel", 0.13157895 },
  { "Liela", 0.0 },
  { "Lielah", 0.0 },
  { "Lielle", 0.0 },
  { "Liem", 1.0 },
  { "Lien", 0.011741683 },
  { "Lienna", 0.0 },
  { "Liera", 0.0 },
  { "Lieren", 0.0 },
  { "Lierin", 0.0 },
  { "Lierra", 0.0 },
  { "Liesa", 0.0 },
  { "Liese", 0.0 },
  { "Liesel", 0.0 },
  { "Lieselotte", 0.0 },
  { "Liesha", 0.0 },
  { "Liesl", 0.0 },
  { "Liette", 0.0 },
  { "Lieu", 0.0 },
  { "Lieutenant", 1.0 },
  { "Liev", 1.0 },
  { "Lieve", 0.0 },
  { "Lieza", 0.0 },
  { "Liezel", 0.0 },
  { "Liezl", 0.0 },
  { "Life", 0.6666667 },
  { "Ligaya", 0.0 },
  { "Lige", 1.0 },
  { "Ligeia", 0.0 },
  { "Light", 0.84615386 },
  { "Ligia", 0.0 },
  { "Ligita", 0.0 },
  { "Lihanna", 0.0 },
  { "Lihi", 0.0 },
  { "Liiam", 1.0 },
  { "Liiban", 1.0 },
  { "Liisa", 0.0 },
  { "Lija", 0.0 },
  { "Lijah", 0.91803277 },
  { "Lijana", 0.0 },
  { "Like", 1.0 },
  { "Likesha", 0.0 },
  { "Likhita", 0.0 },
  { "Likhitha", 0.0 },
  { "Likisha", 0.0 },
  { "Likita", 0.0 },
  { "Liko", 1.0 },
  { "Lil", 0.0 },
  { "Lila", 0.0008947914 },
  { "Lilac", 0.0 },
  { "Lilagrace", 0.0 },
  { "Lilah", 0.0 },
  { "Lilamae", 0.0 },
  { "Lilan", 0.0 },
  { "Lilana", 0.0 },
  { "Lilandra", 0.0 },
  { "Lilani", 0.0 },
  { "Lilany", 0.0 },
  { "Lilar", 0.0 },
  { "Lilarose", 0.0 },
  { "Lilas", 0.0 },
  { "Lilbern", 1.0 },
  { "Lilbert", 1.0 },
  { "Lilborn", 1.0 },
  { "Lilburn", 1.0 },
  { "Lile", 0.33695653 },
  { "Lilea", 0.0 },
  { "Lileah", 0.0 },
  { "Lileana", 0.0 },
  { "Lilee", 0.0 },
  { "Lileeann", 0.0 },
  { "Lileigh", 0.0 },
  { "Liler", 0.0 },
  { "Lilette", 0.0 },
  { "Liley", 0.0 },
  { "Lili", 0.0 },
  { "Lilia", 0.000416008 },
  { "Liliah", 0.0 },
  { "Liliahna", 0.0 },
  { "Liliam", 0.0 },
  { "Lilian", 0.0 },
  { "Liliana", 0.0031357354 },
  { "Lilianah", 0.0 },
  { "Liliane", 0.0 },
  { "Liliani", 0.0 },
  { "Liliann", 0.0 },
  { "Lilianna", 0.0 },
  { "Liliannah", 0.0 },
  { "Lilianne", 0.0 },
  { "Liliano", 0.0 },
  { "Liliany", 0.0 },
  { "Lilias", 0.0 },
  { "Liliauna", 0.0 },
  { "Lilibet", 0.0 },
  { "Lilibeth", 0.0 },
  { "Lilica", 0.0 },
  { "Lilie", 0.0 },
  { "Lilien", 0.0 },
  { "Lilienne", 0.0 },
  { "Lilieth", 0.0 },
  { "Lilija", 0.0 },
  { "Lilijana", 0.0 },
  { "Lilika", 0.0 },
  { "Lilikoi", 0.0 },
  { "Lilinoe", 0.0 },
  { "Liliona", 0.0 },
  { "Lilirose", 0.0 },
  { "Lilit", 0.0 },
  { "Lilita", 0.0 },
  { "Lilith", 0.0 },
  { "Lilium", 0.0 },
  { "Liliuokalani", 0.0 },
  { "Liliya", 0.0 },
  { "Liliyana", 0.0 },
  { "Liliyanna", 0.0 },
  { "Lilja", 0.0 },
  { "Lill", 0.0 },
  { "Lilla", 0.0 },
  { "Lillah", 0.0 },
  { "Lillain", 0.0 },
  { "Lillan", 0.0 },
  { "Lillar", 0.0 },
  { "Lillard", 1.0 },
  { "Lille", 0.0 },
  { "Lillea", 0.0 },
  { "Lilleah", 0.0 },
  { "Lillee", 0.0 },
  { "Lilleeana", 0.0 },
  { "Lilleigh", 0.0 },
  { "Liller", 0.0 },
  { "Lillette", 0.0 },
  { "Lilley", 0.0 },
  { "Lilli", 0.0 },
  { "Lillia", 0.0 },
  { "Lilliah", 0.0 },
  { "Lilliahna", 0.0 },
  { "Lilliam", 0.0 },
  { "Lillian", 0.0032502958 },
  { "Lilliana", 0.0 },
  { "Lillianah", 0.0 },
  { "Lilliane", 0.0 },
  { "Lilliangrace", 0.0 },
  { "Lilliani", 0.0 },
  { "Lilliann", 0.0 },
  { "Lillianna", 0.0 },
  { "Lilliannah", 0.0 },
  { "Lillianne", 0.0 },
  { "Lillianrose", 0.0 },
  { "Lillias", 0.0 },
  { "Lilliauna", 0.0 },
  { "Lilliaunna", 0.0 },
  { "Lillibeth", 0.0 },
  { "Lillie", 0.0055948626 },
  { "Lillieana", 0.0 },
  { "Lillieann", 0.0 },
  { "Lillieanna", 0.0 },
  { "Lillieanne", 0.0 },
  { "Lilliebell", 0.0 },
  { "Lilliemae", 0.0 },
  { "Lillien", 0.0 },
  { "Lillienne", 0.0 },
  { "Lillion", 0.0 },
  { "Lilliona", 0.0 },
  { "Lillionna", 0.0 },
  { "Lillis", 0.0 },
  { "Lillith", 0.0 },
  { "Lilliyan", 0.0 },
  { "Lilliyn", 0.0 },
  { "Lilly", 0.00044679426 },
  { "Lillya", 0.0 },
  { "Lillyah", 0.0 },
  { "Lillyahna", 0.0 },
  { "Lillyan", 0.0 },
  { "Lillyana", 0.0 },
  { "Lillyanah", 0.0 },
  { "Lillyann", 0.0 },
  { "Lillyanna", 0.0 },
  { "Lillyannah", 0.0 },
  { "Lillyanne", 0.0 },
  { "Lillyauna", 0.0 },
  { "Lillyaunna", 0.0 },
  { "Lillybelle", 0.0 },
  { "Lillybeth", 0.0 },
  { "Lillye", 0.0 },
  { "Lillyen", 0.0 },
  { "Lillyian", 0.0 },
  { "Lillyin", 0.0 },
  { "Lillyjo", 0.0 },
  { "Lillykate", 0.0 },
  { "Lillymae", 0.0 },
  { "Lillymay", 0.0 },
  { "Lillyn", 0.0 },
  { "Lillynn", 0.0 },
  { "Lillyona", 0.0 },
  { "Lillyonna", 0.0 },
  { "Lillyrose", 0.0 },
  { "Lillyth", 0.0 },
  { "Lilo", 0.0 },
  { "Liloa", 1.0 },
  { "Lilou", 0.0 },
  { "Lilton", 1.0 },
  { "Lilu", 0.0 },
  { "Lily", 0.0010248902 },
  { "Lilya", 0.0 },
  { "Lilyah", 0.0 },
  { "Lilyahna", 0.0 },
  { "Lilyan", 0.0 },
  { "Lilyana", 0.0 },
  { "Lilyanah", 0.0 },
  { "Lilyane", 0.0 },
  { "Lilyani", 0.0 },
  { "Lilyann", 0.0 },
  { "Lilyanna", 0.0 },
  { "Lilyannah", 0.0 },
  { "Lilyanne", 0.0 },
  { "Lilyauna", 0.0 },
  { "Lilyaunna", 0.0 },
  { "Lilybelle", 0.0 },
  { "Lilybeth", 0.0 },
  { "Lilyen", 0.0 },
  { "Lilygrace", 0.0 },
  { "Lilyian", 0.0 },
  { "Lilyiana", 0.0 },
  { "Lilyin", 0.0 },
  { "Lilykate", 0.0 },
  { "Lilylynn", 0.0 },
  { "Lilymae", 0.0 },
  { "Lilymarie", 0.0 },
  { "Lilymay", 0.0 },
  { "Lilynn", 0.0 },
  { "Lilyona", 0.0 },
  { "Lilyonna", 0.0 },
  { "Lilyrae", 0.0 },
  { "Lilyrose", 0.0 },
  { "Lilyth", 0.0 },
  { "Lim", 1.0 },
  { "Lima", 0.0 },
  { "Limairy", 0.0 },
  { "Limayri", 0.0 },
  { "Limmie", 0.8993902 },
  { "Limon", 1.0 },
  { "Limor", 0.0 },
  { "Limuel", 1.0 },
  { "Lin", 0.46375206 },
  { "Lina", 0.00032469642 },
  { "Linae", 0.0 },
  { "Linah", 0.0 },
  { "Linard", 1.0 },
  { "Linas", 1.0 },
  { "Linay", 0.0 },
  { "Linaya", 0.0 },
  { "Linayah", 0.0 },
  { "Linc", 1.0 },
  { "Lincey", 0.0 },
  { "Lincoln", 0.98449534 },
  { "Lincon", 1.0 },
  { "Lincy", 0.0 },
  { "Lind", 0.24369748 },
  { "Linda", 0.0025811265 },
  { "Lindaa", 0.0 },
  { "Lindaann", 0.0 },
  { "Lindajean", 0.0 },
  { "Lindajo", 0.0 },
  { "Lindal", 0.85714287 },
  { "Lindale", 1.0 },
  { "Lindalee", 0.0 },
  { "Lindall", 1.0 },
  { "Lindalou", 0.0 },
  { "Lindamaria", 0.0 },
  { "Lindamarie", 0.0 },
  { "Lindan", 1.0 },
  { "Lindarose", 0.0 },
  { "Lindasue", 0.0 },
  { "Lindasy", 0.0 },
  { "Linday", 0.0 },
  { "Lindberg", 1.0 },
  { "Lindbergh", 1.0 },
  { "Lindburg", 1.0 },
  { "Lindburgh", 1.0 },
  { "Linde", 0.0 },
  { "Lindee", 0.0 },
  { "Lindel", 0.9550562 },
  { "Lindell", 0.87074584 },
  { "Lindella", 0.0 },
  { "Linden", 0.70719606 },
  { "Linder", 0.03821656 },
  { "Lindey", 0.0 },
  { "Lindi", 0.0 },
  { "Lindia", 0.0 },
  { "Lindie", 0.0 },
  { "Lindita", 0.0 },
  { "Lindiwe", 0.0 },
  { "Lindle", 1.0 },
  { "Lindley", 0.43451142 },
  { "Lindly", 0.0 },
  { "Lindo", 1.0 },
  { "Lindol", 1.0 },
  { "Lindola", 0.0 },
  { "Lindon", 1.0 },
  { "Lindora", 0.0 },
  { "Lindra", 0.0 },
  { "Lindsae", 0.0 },
  { "Lindsay", 0.03815992 },
  { "Lindse", 0.0 },
  { "Lindsea", 0.0 },
  { "Lindsee", 0.0 },
  { "Lindsey", 0.047152985 },
  { "Lindsi", 0.0 },
  { "Lindsie", 0.0 },
  { "Lindsley", 0.5 },
  { "Lindsy", 0.0023584906 },
  { "Lindwood", 1.0 },
  { "Lindy", 0.1790789 },
  { "Lindyn", 0.0 },
  { "Lindzee", 0.0 },
  { "Lindzey", 0.0 },
  { "Lindzi", 0.0 },
  { "Lindzie", 0.0 },
  { "Lindzy", 0.0 },
  { "Linea", 0.0 },
  { "Linell", 0.35176992 },
  { "Linelle", 0.0 },
  { "Linen", 0.0 },
  { "Linesha", 0.0 },
  { "Linet", 0.0 },
  { "Lineth", 0.0 },
  { "Linett", 0.0 },
  { "Linetta", 0.0 },
  { "Linette", 0.0 },
  { "Linford", 1.0 },
  { "Ling", 0.08955224 },
  { "Lingxi", 0.0 },
  { "Lingyi", 0.0 },
  { "Linh", 0.13229814 },
  { "Linhchi", 0.0 },
  { "Linie", 0.0 },
  { "Linita", 0.0 },
  { "Liniyah", 0.0 },
  { "Link", 1.0 },
  { "Linken", 1.0 },
  { "Linkin", 1.0 },
  { "Linkoln", 1.0 },
  { "Linkon", 1.0 },
  { "Linkyn", 0.81333333 },
  { "Linlee", 0.0 },
  { "Linleigh", 0.0 },
  { "Linley", 0.07473309 },
  { "Linly", 0.0 },
  { "Linn", 0.63369805 },
  { "Linna", 0.0 },
  { "Linnae", 0.0 },
  { "Linnaea", 0.0 },
  { "Linnard", 1.0 },
  { "Linnda", 0.0 },
  { "Linne", 0.0 },
  { "Linnea", 0.0 },
  { "Linneah", 0.0 },
  { "Linnell", 0.40689656 },
  { "Linnet", 0.0 },
  { "Linnett", 0.0 },
  { "Linnetta", 0.0 },
  { "Linnette", 0.0 },
  { "Linnie", 0.082635984 },
  { "Linnis", 1.0 },
  { "Linny", 1.0 },
  { "Linnzi", 0.0 },
  { "Lino", 0.99839014 },
  { "Linora", 0.0 },
  { "Linoshka", 0.0 },
  { "Linoy", 0.0 },
  { "Linsay", 0.0 },
  { "Linsday", 0.0 },
  { "Linsdey", 0.0 },
  { "Linsee", 0.0 },
  { "Linsey", 0.045463156 },
  { "Linsi", 0.0 },
  { "Linsie", 0.0 },
  { "Linsley", 1.0 },
  { "Linson", 1.0 },
  { "Linsy", 0.0 },
  { "Linton", 1.0 },
  { "Linus", 1.0 },
  { "Linux", 0.84375 },
  { "Linvel", 1.0 },
  { "Linville", 1.0 },
  { "Linward", 1.0 },
  { "Linwood", 0.99960625 },
  { "Linza", 1.0 },
  { "Linze", 0.0 },
  { "Linzee", 0.0 },
  { "Linzell", 1.0 },
  { "Linzey", 0.04692082 },
  { "Linzi", 0.0 },
  { "Linzie", 0.38020247 },
  { "Linzy", 0.381499 },
  { "Lio", 1.0 },
  { "Liola", 0.0 },
  { "Lion", 1.0 },
  { "Liona", 0.0 },
  { "Lional", 1.0 },
  { "Lionardo", 1.0 },
  { "Lionel", 0.99851924 },
  { "Lionela", 0.0 },
  { "Lionell", 1.0 },
  { "Lionna", 0.0 },
  { "Lionor", 0.0 },
  { "Lionso", 1.0 },
  { "Lior", 0.67571884 },
  { "Liora", 0.0 },
  { "Lipa", 1.0 },
  { "Lipman", 1.0 },
  { "Lira", 0.0 },
  { "Liran", 1.0 },
  { "Liria", 0.0 },
  { "Liridon", 1.0 },
  { "Liridona", 0.0 },
  { "Liriel", 0.0 },
  { "Lirije", 0.0 },
  { "Lirim", 1.0 },
  { "Lirio", 0.0 },
  { "Liron", 0.6752137 },
  { "Lis", 0.0 },
  { "Lisa", 0.00288177 },
  { "Lisaann", 0.0 },
  { "Lisaanne", 0.0 },
  { "Lisabella", 0.0 },
  { "Lisabelle", 0.0 },
  { "Lisabeth", 0.0 },
  { "Lisah", 0.0 },
  { "Lisajean", 0.0 },
  { "Lisajo", 0.0 },
  { "Lisamaria", 0.0 },
  { "Lisamarie", 0.0 },
  { "Lisander", 1.0 },
  { "Lisandra", 0.0 },
  { "Lisandro", 0.9970896 },
  { "Lisania", 0.0 },
  { "Lisann", 0.0 },
  { "Lisanna", 0.0 },
  { "Lisanne", 0.0 },
  { "Lisbed", 0.0 },
  { "Lisbel", 0.0 },
  { "Lisbet", 0.0 },
  { "Lisbeth", 0.0 },
  { "Lisbett", 0.0 },
  { "Lisbon", 1.0 },
  { "Lise", 0.0 },
  { "Lised", 0.0 },
  { "Lisel", 0.0 },
  { "Liseli", 0.0 },
  { "Liselle", 0.0 },
  { "Liset", 0.0 },
  { "Lisete", 0.0 },
  { "Liseth", 0.0 },
  { "Lisett", 0.0 },
  { "Lisetta", 0.0 },
  { "Lisette", 0.0 },
  { "Lish", 0.7741935 },
  { "Lisha", 0.0 },
  { "Lishia", 0.0 },
  { "Lisi", 0.0 },
  { "Lisia", 0.0 },
  { "Lisiate", 1.0 },
  { "Lisl", 0.0 },
  { "Lisle", 0.96276593 },
  { "Lislie", 0.0 },
  { "Lismari", 0.0 },
  { "Lismarie", 0.0 },
  { "Lismary", 0.0 },
  { "Liss", 0.0 },
  { "Lissa", 0.0 },
  { "Lissandra", 0.0 },
  { "Lissandro", 1.0 },
  { "Lissbet", 0.0 },
  { "Lissbeth", 0.0 },
  { "Lisset", 0.0 },
  { "Lissete", 0.0 },
  { "Lisseth", 0.0 },
  { "Lissett", 0.0 },
  { "Lissette", 0.0006795325 },
  { "Lissianna", 0.0 },
  { "Lissie", 0.0 },
  { "Lissy", 0.0 },
  { "Lister", 1.0 },
  { "Liston", 1.0 },
  { "Lisvet", 0.0 },
  { "Lisveth", 0.0 },
  { "Lisvette", 0.0 },
  { "Lita", 0.0 },
  { "Lital", 0.0 },
  { "Litany", 0.0 },
  { "Litasha", 0.0 },
  { "Litcy", 0.0 },
  { "Litha", 0.0 },
  { "Lithzy", 0.0 },
  { "Litia", 0.0 },
  { "Liticia", 0.0 },
  { "Litisha", 0.0 },
  { "Lititia", 0.0 },
  { "Lito", 1.0 },
  { "Litonia", 0.0 },
  { "Litonya", 0.0 },
  { "Litsa", 0.0 },
  { "Litsi", 0.0 },
  { "Litsy", 0.0 },
  { "Litta", 0.0 },
  { "Littie", 0.0 },
  { "Little", 0.8779395 },
  { "Littleton", 1.0 },
  { "Littzy", 0.0 },
  { "Litza", 0.0 },
  { "Litzi", 0.0 },
  { "Litzie", 0.0 },
  { "Litzy", 0.000980873 },
  { "Litzzy", 0.0 },
  { "Lium", 1.0 },
  { "Lius", 1.0 },
  { "Liv", 0.0 },
  { "Liva", 0.0 },
  { "Livan", 1.0 },
  { "Livana", 0.0 },
  { "Livanna", 0.0 },
  { "Livea", 0.0 },
  { "Livee", 0.0 },
  { "Lively", 0.0 },
  { "Livi", 0.0 },
  { "Livia", 0.0 },
  { "Liviah", 0.0 },
  { "Livian", 0.0 },
  { "Liviana", 0.0 },
  { "Livianna", 0.0 },
  { "Livie", 0.0 },
  { "Livier", 0.0 },
  { "Livija", 0.0 },
  { "Livingston", 0.97252744 },
  { "Livinia", 0.0 },
  { "Livio", 1.0 },
  { "Liviya", 0.0 },
  { "Livvie", 0.0 },
  { "Livvy", 0.0 },
  { "Livy", 0.0 },
  { "Livya", 0.0 },
  { "Lixandro", 1.0 },
  { "Lixy", 0.0 },
  { "Liya", 0.0 },
  { "Liyah", 0.0 },
  { "Liyam", 1.0 },
  { "Liyan", 0.54545456 },
  { "Liyana", 0.0 },
  { "Liyanah", 0.0 },
  { "Liyanna", 0.0 },
  { "Liyat", 0.0 },
  { "Liyla", 0.0 },
  { "Liylah", 0.0 },
  { "Liyu", 0.0 },
  { "Liz", 0.0 },
  { "Liza", 0.0008464329 },
  { "Lizabella", 0.0 },
  { "Lizabelle", 0.0 },
  { "Lizabeth", 0.0 },
  { "Lizah", 0.0 },
  { "Lizaida", 0.0 },
  { "Lizamarie", 0.0 },
  { "Lizandra", 0.0 },
  { "Lizandro", 1.0 },
  { "Lizania", 0.0 },
  { "Lizann", 0.0 },
  { "Lizanne", 0.0 },
  { "Lizardo", 1.0 },
  { "Lizbeht", 0.0 },
  { "Lizbella", 0.0 },
  { "Lizbet", 0.0 },
  { "Lizbeth", 0.0012944173 },
  { "Lizbett", 0.0 },
  { "Lizbette", 0.0 },
  { "Lizbhet", 0.0 },
  { "Lize", 0.0 },
  { "Lizeht", 0.0 },
  { "Lizel", 0.0 },
  { "Lizelle", 0.0 },
  { "Lizet", 0.0 },
  { "Lizete", 0.0 },
  { "Lizeth", 0.0011090573 },
  { "Lizethe", 0.0 },
  { "Lizett", 0.0 },
  { "Lizetta", 0.0 },
  { "Lizette", 0.0004257493 },
  { "Lizhet", 0.0 },
  { "Lizie", 0.0 },
  { "Lizmar", 0.0 },
  { "Lizmari", 0.0 },
  { "Lizmarie", 0.0 },
  { "Lizmary", 0.0 },
  { "Lizneidy", 0.0 },
  { "Lizsandra", 0.0 },
  { "Lizvet", 0.0 },
  { "Lizveth", 0.0 },
  { "Lizvette", 0.0 },
  { "Lizy", 0.0 },
  { "Lizz", 0.0 },
  { "Lizza", 0.0 },
  { "Lizzbeth", 0.0 },
  { "Lizzet", 0.0 },
  { "Lizzete", 0.0 },
  { "Lizzeth", 0.0 },
  { "Lizzett", 0.0 },
  { "Lizzette", 0.0 },
  { "Lizzi", 0.0 },
  { "Lizzie", 0.0026281886 },
  { "Lizzieann", 0.0 },
  { "Lizzy", 0.0 },
  { "Lj", 1.0 },
  { "Ljiljana", 0.0 },
  { "Lladira", 0.0 },
  { "Llamile", 0.0 },
  { "Llana", 0.0 },
  { "Llandel", 1.0 },
  { "Llaneli", 0.0 },
  { "Llanet", 0.0 },
  { "Llareli", 0.0 },
  { "Llarely", 0.0 },
  { "Llasmin", 0.0 },
  { "Llayla", 0.0 },
  { "Llecenia", 0.0 },
  { "Llenifer", 0.0 },
  { "Llesenia", 0.0 },
  { "Llesica", 0.0 },
  { "Llewellyn", 0.91930115 },
  { "Llewelyn", 1.0 },
  { "Llewyn", 1.0 },
  { "Lleyton", 1.0 },
  { "Llia", 0.0 },
  { "Lliam", 1.0 },
  { "Llinda", 0.0 },
  { "Lloana", 0.0 },
  { "Llona", 0.0 },
  { "Llovani", 1.0 },
  { "Lloyal", 1.0 },
  { "Lloyd", 0.99379665 },
  { "Lloyde", 1.0 },
  { "Lloydene", 0.0 },
  { "Lloydine", 0.0 },
  { "Lloyed", 1.0 },
  { "Llubia", 0.0 },
  { "Lluliana", 0.0 },
  { "Llulisa", 0.0 },
  { "Lluvia", 0.0 },
  { "Lluviana", 0.0 },
  { "Lluvy", 0.0 },
  { "Llyod", 1.0 },
  { "Lo", 0.55932206 },
  { "Loa", 0.0 },
  { "Loagan", 1.0 },
  { "Loagen", 1.0 },
  { "Loah", 0.0 },
  { "Loai", 1.0 },
  { "Loal", 1.0 },
  { "Loammi", 0.0 },
  { "Loan", 0.0 },
  { "Loana", 0.0 },
  { "Loann", 0.0 },
  { "Loanna", 0.0 },
  { "Loanne", 0.0 },
  { "Loany", 0.0 },
  { "Loarine", 0.0 },
  { "Loay", 1.0 },
  { "Loberta", 0.0 },
  { "Lobsang", 1.0 },
  { "Loc", 1.0 },
  { "Locadio", 1.0 },
  { "Loch", 1.0 },
  { "Lochlain", 1.0 },
  { "Lochlainn", 1.0 },
  { "Lochlan", 0.98032784 },
  { "Lochlann", 1.0 },
  { "Lochlen", 1.0 },
  { "Lochlin", 1.0 },
  { "Lochlyn", 0.14857143 },
  { "Lochlynn", 0.0 },
  { "Locie", 0.0 },
  { "Lock", 1.0 },
  { "Locke", 1.0 },
  { "Lockie", 0.0 },
  { "Locklan", 1.0 },
  { "Locklin", 1.0 },
  { "Locklyn", 0.048543688 },
  { "Locklynn", 0.0 },
  { "Locksley", 1.0 },
  { "Lockwood", 1.0 },
  { "Locryn", 1.0 },
  { "Loda", 0.0 },
  { "Loddie", 0.53846157 },
  { "Lodean", 0.0 },
  { "Lodell", 0.0 },
  { "Lodema", 0.0 },
  { "Loden", 1.0 },
  { "Lodena", 0.0 },
  { "Lodia", 0.0 },
  { "Lodie", 0.0 },
  { "Lodine", 0.0 },
  { "Lodis", 1.0 },
  { "Loeda", 0.0 },
  { "Loegan", 1.0 },
  { "Loel", 0.98233217 },
  { "Loella", 0.0 },
  { "Loena", 0.0 },
  { "Loene", 0.0 },
  { "Loeta", 0.0 },
  { "Loetta", 0.0 },
  { "Loette", 0.0 },
  { "Loeva", 0.0 },
  { "Loey", 0.0 },
  { "Loften", 1.0 },
  { "Lofton", 1.0 },
  { "Logahn", 0.41666666 },
  { "Logan", 0.92928267 },
  { "Loganjames", 1.0 },
  { "Logann", 0.04631579 },
  { "Loganne", 0.0 },
  { "Logen", 0.8827023 },
  { "Logene", 0.0 },
  { "Loggan", 1.0 },
  { "Loghan", 0.5129717 },
  { "Logic", 1.0 },
  { "Login", 0.919708 },
  { "Logon", 1.0 },
  { "Logun", 1.0 },
  { "Logunn", 1.0 },
  { "Logyn", 0.44031832 },
  { "Logynn", 0.0 },
  { "Lohan", 1.0 },
  { "Lohgan", 0.45454547 },
  { "Lohith", 1.0 },
  { "Lohn", 1.0 },
  { "Lohoma", 0.0 },
  { "Loi", 0.8153846 },
  { "Loic", 1.0 },
  { "Loice", 0.10344828 },
  { "Loid", 1.0 },
  { "Loida", 0.0 },
  { "Loie", 0.0 },
  { "Loir", 0.0 },
  { "Lois", 0.007884375 },
  { "Loisann", 0.0 },
  { "Loisanne", 0.0 },
  { "Loise", 0.04332553 },
  { "Loisjean", 0.0 },
  { "Loismae", 0.0 },
  { "Loisteen", 0.0 },
  { "Loistene", 0.0 },
  { "Loistine", 0.0 },
  { "Lojain", 0.0 },
  { "Lokelani", 0.0 },
  { "Loken", 1.0 },
  { "Lokesh", 1.0 },
  { "Loki", 0.9663158 },
  { "Lola", 0.0031985298 },
  { "Lolabelle", 0.0 },
  { "Lolah", 0.0 },
  { "Lolamae", 0.0 },
  { "Lolan", 1.0 },
  { "Lolana", 0.0 },
  { "Lolanda", 0.0 },
  { "Lolani", 0.0 },
  { "Lolar", 0.0 },
  { "Loleatha", 0.0 },
  { "Loleatta", 0.0 },
  { "Lolene", 0.0 },
  { "Loleta", 0.0 },
  { "Loletha", 0.0 },
  { "Lolethia", 0.0 },
  { "Loletia", 0.0 },
  { "Loletta", 0.0 },
  { "Lolisa", 0.0 },
  { "Lolita", 0.0 },
  { "Lolitha", 0.0 },
  { "Lolitta", 0.0 },
  { "Lolla", 0.0 },
  { "Lollie", 0.0 },
  { "Lolly", 0.0 },
  { "Lolo", 0.3125 },
  { "Loma", 0.00218436 },
  { "Loman", 1.0 },
  { "Lomant", 1.0 },
  { "Lomar", 1.0 },
  { "Lomax", 1.0 },
  { "Lometa", 0.0 },
  { "Lomie", 0.0 },
  { "Lomont", 1.0 },
  { "Lon", 0.9979648 },
  { "Lona", 0.001575971 },
  { "Lonald", 1.0 },
  { "Lonan", 1.0 },
  { "Lonard", 1.0 },
  { "Lonas", 1.0 },
  { "Londa", 0.0 },
  { "Londale", 1.0 },
  { "Londan", 0.0 },
  { "Londell", 1.0 },
  { "Londen", 0.3677966 },
  { "Londi", 0.0 },
  { "Londie", 0.0 },
  { "Londin", 0.0 },
  { "Londo", 1.0 },
  { "London", 0.20292272 },
  { "Londonmarie", 0.0 },
  { "Londonn", 0.0 },
  { "Londonrose", 0.0 },
  { "Londrea", 0.0 },
  { "Londyn", 0.020596778 },
  { "Londynmarie", 0.0 },
  { "Londynn", 0.0 },
  { "Londynne", 0.0 },
  { "Lone", 1.0 },
  { "Lonel", 1.0 },
  { "Lonell", 0.97073793 },
  { "Lones", 1.0 },
  { "Lonesha", 0.0 },
  { "Loneta", 0.0 },
  { "Lonetta", 0.0 },
  { "Lonette", 0.0 },
  { "Loney", 0.6433121 },
  { "Long", 1.0 },
  { "Longin", 1.0 },
  { "Longina", 0.0 },
  { "Longino", 1.0 },
  { "Loni", 0.028915662 },
  { "Lonia", 0.0 },
  { "Lonie", 0.39128858 },
  { "Lonika", 0.0 },
  { "Lonis", 1.0 },
  { "Lonisha", 0.0 },
  { "Lonita", 0.0 },
  { "Lonn", 1.0 },
  { "Lonna", 0.0 },
  { "Lonne", 1.0 },
  { "Lonnel", 1.0 },
  { "Lonnell", 1.0 },
  { "Lonnesha", 0.0 },
  { "Lonnetta", 0.0 },
  { "Lonnette", 0.0 },
  { "Lonney", 1.0 },
  { "Lonni", 0.0475 },
  { "Lonnia", 0.0 },
  { "Lonnie", 0.9318657 },
  { "Lonniel", 1.0 },
  { "Lonniesha", 0.0 },
  { "Lonnisha", 0.0 },
  { "Lonnita", 0.0 },
  { "Lonny", 0.990572 },
  { "Lontrell", 1.0 },
  { "Lonya", 0.0 },
  { "Lonyea", 0.0 },
  { "Lonza", 1.0 },
  { "Lonzel", 1.0 },
  { "Lonzell", 1.0 },
  { "Lonzetta", 0.0 },
  { "Lonzie", 1.0 },
  { "Lonzo", 1.0 },
  { "Lonzy", 1.0 },
  { "Loomis", 1.0 },
  { "Looney", 1.0 },
  { "Lopaka", 1.0 },
  { "Lopez", 1.0 },
  { "Lor", 1.0 },
  { "Lora", 0.0047876327 },
  { "Lorae", 0.0 },
  { "Lorah", 0.0 },
  { "Lorain", 0.17219079 },
  { "Loraina", 0.0 },
  { "Loraine", 0.019494295 },
  { "Lorainne", 0.0 },
  { "Loral", 0.35064936 },
  { "Loralai", 0.0 },
  { "Loralea", 0.0 },
  { "Loralee", 0.0 },
  { "Loralei", 0.0 },
  { "Lorali", 0.0 },
  { "Loralie", 0.0 },
  { "Loralye", 0.0 },
  { "Loralyn", 0.0 },
  { "Loralynn", 0.0 },
  { "Loramae", 0.0 },
  { "Loran", 0.81953716 },
  { "Lorana", 0.0 },
  { "Lorance", 1.0 },
  { "Loranda", 0.0 },
  { "Lorane", 0.011682243 },
  { "Lorann", 0.0 },
  { "Loranna", 0.0 },
  { "Loranne", 0.0 },
  { "Loranza", 1.0 },
  { "Loranzo", 1.0 },
  { "Loras", 1.0 },
  { "Loray", 0.0 },
  { "Lorayn", 0.0 },
  { "Lorayna", 0.0 },
  { "Lorayne", 0.0 },
  { "Lorcan", 1.0 },
  { "Lord", 1.0 },
  { "Lorde", 0.0 },
  { "Lordina", 0.0 },
  { "Lore", 0.0 },
  { "Lorea", 0.0 },
  { "Loreal", 0.0 },
  { "Lorean", 0.0 },
  { "Loreana", 0.0 },
  { "Loreane", 0.0 },
  { "Loreatha", 0.0 },
  { "Lorece", 0.0 },
  { "Loreda", 0.0 },
  { "Loredana", 0.0 },
  { "Loree", 0.0014413375 },
  { "Loreen", 0.0 },
  { "Loreena", 0.0 },
  { "Loreene", 0.0 },
  { "Loreeta", 0.0 },
  { "Loreina", 0.0 },
  { "Loreine", 0.0 },
  { "Loreita", 0.0 },
  { "Lorel", 0.017361112 },
  { "Lorelai", 0.0 },
  { "Lorelay", 0.0 },
  { "Lorelee", 0.0 },
  { "Lorelei", 0.0 },
  { "Loreley", 0.0 },
  { "Loreli", 0.0 },
  { "Lorelie", 0.0 },
  { "Lorell", 0.12103746 },
  { "Lorella", 0.0 },
  { "Lorelle", 0.0 },
  { "Lorely", 0.0 },
  { "Lorelyn", 0.0 },
  { "Loren", 0.7918203 },
  { "Lorena", 0.005028703 },
  { "Lorence", 0.9907407 },
  { "Lorencita", 0.0 },
  { "Lorenda", 0.0 },
  { "Lorene", 0.0028753912 },
  { "Lorenia", 0.0 },
  { "Lorenn", 0.0 },
  { "Lorenna", 0.0 },
  { "Lorensa", 0.0 },
  { "Lorenso", 1.0 },
  { "Lorentz", 1.0 },
  { "Lorenz", 1.0 },
  { "Lorenza", 0.54956704 },
  { "Lorenze", 1.0 },
  { "Lorenzia", 1.0 },
  { "Lorenzo", 0.99505514 },
  { "Loresa", 0.0 },
  { "Lorese", 0.0 },
  { "Loressa", 0.0 },
  { "Loreta", 0.0 },
  { "Loretha", 0.0 },
  { "Loreto", 0.9655172 },
  { "Lorett", 0.0 },
  { "Loretta", 0.0028609445 },
  { "Lorette", 0.0 },
  { "Loretto", 0.005393743 },
  { "Lorey", 0.0 },
  { "Loreyna", 0.0 },
  { "Lorez", 0.0 },
  { "Lori", 0.002497577 },
  { "Loria", 0.0 },
  { "Loriah", 0.0 },
  { "Lorian", 0.0 },
  { "Loriana", 0.0 },
  { "Loriane", 0.0 },
  { "Loriann", 0.0 },
  { "Lorianna", 0.0 },
  { "Lorianne", 0.0 },
  { "Loribeth", 0.0 },
  { "Lorice", 0.0 },
  { "Lorida", 0.0 },
  { "Lorie", 0.0047339434 },
  { "Lorieann", 0.0 },
  { "Loriece", 0.0 },
  { "Loriel", 0.0 },
  { "Loriell", 0.0 },
  { "Lorielle", 0.0 },
  { "Lorien", 0.0 },
  { "Loriena", 0.0 },
  { "Loriene", 0.0 },
  { "Lorii", 0.0 },
  { "Lorijean", 0.0 },
  { "Lorijo", 0.0 },
  { "Lorik", 1.0 },
  { "Lorilai", 0.0 },
  { "Lorilea", 0.0 },
  { "Lorilee", 0.0 },
  { "Lorilei", 0.0 },
  { "Lorilie", 0.0 },
  { "Lorilyn", 0.0 },
  { "Lorilynn", 0.0 },
  { "Lorimar", 0.0 },
  { "Lorimer", 1.0 },
  { "Lorin", 0.6445361 },
  { "Lorina", 0.0 },
  { "Lorinda", 0.0 },
  { "Lorine", 0.0013726836 },
  { "Loring", 0.98315465 },
  { "Lorinne", 0.0 },
  { "Lorinza", 1.0 },
  { "Lorinzo", 1.0 },
  { "Loris", 0.250774 },
  { "Lorisa", 0.0 },
  { "Lorise", 0.0 },
  { "Lorisha", 0.0 },
  { "Lorissa", 0.0 },
  { "Lorita", 0.0 },
  { "Loritta", 0.0 },
  { "Loriyah", 0.0 },
  { "Lorma", 0.0 },
  { "Lorman", 1.0 },
  { "Lorn", 1.0 },
  { "Lorna", 0.00078133325 },
  { "Lornalee", 0.0 },
  { "Lorne", 0.9883314 },
  { "Lornell", 1.0 },
  { "Loron", 1.0 },
  { "Loronda", 0.0 },
  { "Lorra", 0.0 },
  { "Lorrae", 0.0 },
  { "Lorrain", 0.015479876 },
  { "Lorraina", 0.0 },
  { "Lorraine", 0.004466845 },
  { "Lorrainne", 0.0 },
  { "Lorrane", 0.0 },
  { "Lorrayne", 0.0 },
  { "Lorre", 0.0 },
  { "Lorree", 0.0 },
  { "Lorreen", 0.0 },
  { "Lorrel", 0.0 },
  { "Lorrell", 0.0 },
  { "Lorren", 0.47668394 },
  { "Lorrena", 0.0 },
  { "Lorrenda", 0.0 },
  { "Lorrene", 0.0 },
  { "Lorrenzo", 1.0 },
  { "Lorreta", 0.0 },
  { "Lorretta", 0.0 },
  { "Lorri", 0.0 },
  { "Lorria", 0.0 },
  { "Lorrian", 0.0 },
  { "Lorriane", 0.0 },
  { "Lorriann", 0.0 },
  { "Lorrianne", 0.0 },
  { "Lorrie", 0.0050697085 },
  { "Lorrieann", 0.0 },
  { "Lorrin", 0.46724892 },
  { "Lorrina", 0.0 },
  { "Lorrinda", 0.0 },
  { "Lorrine", 0.0 },
  { "Lorris", 1.0 },
  { "Lorrisa", 0.0 },
  { "Lorrita", 0.0 },
  { "Lorry", 0.17751004 },
  { "Lorryn", 0.0 },
  { "Lory", 0.10705694 },
  { "Loryanna", 0.0 },
  { "Loryn", 0.0 },
  { "Lorynn", 0.0 },
  { "Loryssa", 0.0 },
  { "Lossie", 0.019785656 },
  { "Lot", 1.0 },
  { "Lota", 0.0 },
  { "Lotanna", 0.54545456 },
  { "Lotasha", 0.0 },
  { "Lotaya", 0.0 },
  { "Lotella", 0.0 },
  { "Lotha", 0.0 },
  { "Lothar", 1.0 },
  { "Lothario", 1.0 },
  { "Lothrop", 1.0 },
  { "Lotis", 0.2173913 },
  { "Lotonya", 0.0 },
  { "Lotosha", 0.0 },
  { "Lotoya", 0.0 },
  { "Lott", 1.0 },
  { "Lotta", 0.0 },
  { "Lotte", 0.0 },
  { "Lotti", 0.0 },
  { "Lottie", 0.0028817195 },
  { "Lotty", 0.0 },
  { "Lottye", 0.0 },
  { "Lotus", 0.037119113 },
  { "Lou", 0.09561453 },
  { "Louaine", 0.0 },
  { "Louan", 0.0 },
  { "Louana", 0.0 },
  { "Louann", 0.0 },
  { "Louanna", 0.0 },
  { "Louanne", 0.0 },
  { "Louard", 1.0 },
  { "Louay", 1.0 },
  { "Loubelle", 0.0 },
  { "Louberta", 0.0 },
  { "Loubertha", 0.0 },
  { "Loucas", 1.0 },
  { "Loucile", 0.0 },
  { "Loucille", 0.0 },
  { "Loucinda", 0.0 },
  { "Loudean", 0.0 },
  { "Loudell", 0.0 },
  { "Loudella", 0.0 },
  { "Louden", 1.0 },
  { "Loudes", 0.0 },
  { "Loudon", 1.0 },
  { "Loueen", 0.0 },
  { "Louell", 0.0 },
  { "Louella", 0.0 },
  { "Louelle", 0.0 },
  { "Louellen", 0.0 },
  { "Louene", 0.0 },
  { "Louese", 0.0 },
  { "Louetta", 0.0 },
  { "Louette", 0.0 },
  { "Loueva", 0.0 },
  { "Lougene", 0.2682927 },
  { "Lougenia", 0.0 },
  { "Loui", 1.0 },
  { "Louia", 1.0 },
  { "Louida", 0.0 },
  { "Louie", 0.9270273 },
  { "Louies", 0.0 },
  { "Louiese", 0.0 },
  { "Louina", 0.0 },
  { "Louine", 0.0 },
  { "Louis", 0.99288726 },
  { "Louisa", 0.00027082657 },
  { "Louise", 0.006540852 },
  { "Louiseann", 0.0 },
  { "Louisette", 0.0 },
  { "Louisia", 0.0 },
  { "Louisiana", 0.0 },
  { "Louiza", 0.0 },
  { "Loujean", 0.0 },
  { "Louka", 1.0 },
  { "Loukas", 1.0 },
  { "Loukisha", 0.0 },
  { "Loukya", 0.0 },
  { "Loula", 0.0 },
  { "Loulou", 0.0 },
  { "Louna", 0.0 },
  { "Lounell", 0.0 },
  { "Lounette", 0.0 },
  { "Loura", 0.0 },
  { "Louraine", 0.0 },
  { "Lourd", 0.0 },
  { "Lourdes", 0.004861894 },
  { "Lourdez", 0.0 },
  { "Lourdine", 0.0 },
  { "Louree", 0.0 },
  { "Louren", 0.0 },
  { "Lourena", 0.0 },
  { "Lourence", 1.0 },
  { "Lourene", 0.0 },
  { "Louretha", 0.0 },
  { "Louretta", 0.0 },
  { "Louri", 0.0 },
  { "Louria", 0.0 },
  { "Lourie", 0.0 },
  { "Lourinda", 0.0 },
  { "Lourine", 0.0 },
  { "Lourita", 0.0 },
  { "Lousia", 0.0 },
  { "Lousie", 0.0 },
  { "Loutricia", 0.0 },
  { "Louva", 0.0 },
  { "Louvada", 0.0 },
  { "Louvella", 0.0 },
  { "Louvena", 0.0 },
  { "Louvene", 0.0 },
  { "Louvenia", 0.0 },
  { "Louvina", 0.0 },
  { "Louvine", 0.0 },
  { "Louvinia", 0.0 },
  { "Louvonia", 0.0 },
  { "Louwana", 0.0 },
  { "Louwanna", 0.0 },
  { "Lova", 0.0 },
  { "Lovada", 0.0 },
  { "Loval", 1.0 },
  { "Lovanda", 0.0 },
  { "Lovann", 0.0 },
  { "Love", 0.24255066 },
  { "Lovea", 0.0 },
  { "Loveah", 0.0 },
  { "Loveaya", 0.0 },
  { "Loveda", 0.0 },
  { "Lovel", 0.93203884 },
  { "Lovelace", 1.0 },
  { "Lovelee", 0.0 },
  { "Loveleen", 0.0 },
  { "Loveless", 1.0 },
  { "Lovelia", 0.0 },
  { "Lovell", 0.83954453 },
  { "Lovella", 0.0 },
  { "Lovelle", 0.39473686 },
  { "Lovely", 0.0073304824 },
  { "Lovelyn", 0.0 },
  { "Lovelynn", 0.0 },
  { "Lovena", 0.0 },
  { "Lovene", 0.0 },
  { "Lovenia", 0.0 },
  { "Lovera", 0.0 },
  { "Lovern", 0.52380955 },
  { "Loverne", 0.6666667 },
  { "Lovesta", 0.0 },
  { "Loveta", 0.0 },
  { "Loveth", 0.0 },
  { "Lovett", 1.0 },
  { "Lovetta", 0.0 },
  { "Lovette", 0.023310022 },
  { "Lovey", 0.0 },
  { "Loveya", 0.0 },
  { "Lovi", 0.0 },
  { "Lovia", 0.0 },
  { "Lovic", 1.0 },
  { "Lovice", 0.0 },
  { "Lovick", 1.0 },
  { "Lovie", 0.098337516 },
  { "Lovilla", 0.0 },
  { "Lovina", 0.0 },
  { "Lovine", 0.0 },
  { "Lovinia", 0.0 },
  { "Lovis", 0.7234042 },
  { "Lovisa", 0.0 },
  { "Lovita", 0.0 },
  { "Lovonda", 0.0 },
  { "Lovonia", 0.0 },
  { "Lovonne", 0.0 },
  { "Low", 1.0 },
  { "Lowana", 0.0 },
  { "Lowanda", 0.0 },
  { "Lowanna", 0.0 },
  { "Lowayne", 0.0 },
  { "Lowe", 0.8888889 },
  { "Lowel", 1.0 },
  { "Lowell", 0.99160516 },
  { "Lowella", 0.0 },
  { "Lowen", 0.9166667 },
  { "Lowene", 0.0 },
  { "Lowery", 1.0 },
  { "Loweta", 0.0 },
  { "Lowis", 1.0 },
  { "Lowman", 1.0 },
  { "Lowrey", 1.0 },
  { "Lowrie", 1.0 },
  { "Lowrine", 0.0 },
  { "Lowry", 0.96875 },
  { "Lowyn", 0.0 },
  { "Loxi", 0.0 },
  { "Loxie", 0.0 },
  { "Loxley", 0.1509434 },
  { "Loy", 0.8242644 },
  { "Loya", 0.0 },
  { "Loyal", 0.95472246 },
  { "Loyall", 1.0 },
  { "Loyaltee", 0.0 },
  { "Loyalti", 0.0 },
  { "Loyalty", 0.19395867 },
  { "Loyce", 0.18115942 },
  { "Loycie", 0.0 },
  { "Loyd", 0.9961722 },
  { "Loyda", 0.0 },
  { "Loyde", 1.0 },
  { "Loydene", 0.0 },
  { "Loye", 0.72141707 },
  { "Loyed", 1.0 },
  { "Loyette", 0.0 },
  { "Loyle", 1.0 },
  { "Loyola", 0.0 },
  { "Loys", 0.5813253 },
  { "Loyse", 0.0 },
  { "Loza", 0.0 },
  { "Lozell", 1.0 },
  { "Lozelle", 0.0 },
  { "Lozen", 0.0 },
  { "Lrey", 1.0 },
  { "Ltanya", 0.0 },
  { "Ltonya", 0.0 },
  { "Ltoya", 0.0 },
  { "Lu", 0.004492139 },
  { "Lua", 0.0 },
  { "Luada", 0.0 },
  { "Luai", 1.0 },
  { "Lual", 1.0 },
  { "Luan", 0.4679752 },
  { "Luana", 0.0 },
  { "Luanda", 0.0 },
  { "Luane", 0.0 },
  { "Luann", 0.00038915552 },
  { "Luanna", 0.0 },
  { "Luanne", 0.0 },
  { "Luar", 1.0 },
  { "Luara", 0.0 },
  { "Luay", 1.0 },
  { "Luba", 0.0 },
  { "Lubby", 1.0 },
  { "Lubell", 0.0 },
  { "Luberdie", 0.0 },
  { "Lubert", 1.0 },
  { "Luberta", 0.0 },
  { "Lubertha", 0.0 },
  { "Lubie", 0.77272725 },
  { "Lubina", 0.0 },
  { "Lubna", 0.0 },
  { "Lubov", 0.0 },
  { "Luby", 0.8483965 },
  { "Luc", 1.0 },
  { "Luca", 0.96089894 },
  { "Lucabella", 0.0 },
  { "Lucah", 1.0 },
  { "Lucais", 1.0 },
  { "Lucan", 1.0 },
  { "Lucanus", 1.0 },
  { "Lucas", 0.99812984 },
  { "Lucca", 0.865193 },
  { "Luccas", 1.0 },
  { "Lucchese", 0.54545456 },
  { "Luccia", 0.0 },
  { "Lucciana", 0.0 },
  { "Lucciano", 1.0 },
  { "Luce", 0.0 },
  { "Lucea", 0.0 },
  { "Luceal", 0.0 },
  { "Lucecita", 0.0 },
  { "Lucee", 0.0 },
  { "Luceil", 0.0 },
  { "Luceile", 0.0 },
  { "Luceille", 0.0 },
  { "Lucele", 0.0 },
  { "Lucelenia", 0.0 },
  { "Luceli", 0.0 },
  { "Lucelia", 0.0 },
  { "Lucell", 0.0 },
  { "Lucella", 0.0 },
  { "Lucelle", 0.0 },
  { "Lucely", 0.0 },
  { "Lucena", 0.0 },
  { "Lucenda", 0.0 },
  { "Lucendia", 0.0 },
  { "Lucene", 0.0 },
  { "Lucenia", 0.0 },
  { "Lucerito", 0.0 },
  { "Lucero", 0.028486203 },
  { "Lucertia", 0.0 },
  { "Lucesita", 0.0 },
  { "Lucetta", 0.0 },
  { "Lucette", 0.0 },
  { "Lucey", 0.0 },
  { "Lucha", 0.0 },
  { "Luchana", 0.0 },
  { "Luchanna", 0.0 },
  { "Luchia", 0.0 },
  { "Luchiana", 0.0 },
  { "Luchiano", 1.0 },
  { "Luchina", 0.0 },
  { "Luchious", 1.0 },
  { "Luci", 0.0 },
  { "Lucia", 0.002266896 },
  { "Lucian", 0.9967595 },
  { "Luciana", 0.0 },
  { "Luciann", 0.0 },
  { "Lucianna", 0.0 },
  { "Lucianne", 0.0 },
  { "Luciano", 1.0 },
  { "Lucias", 1.0 },
  { "Lucie", 0.0 },
  { "Luciel", 0.0 },
  { "Luciele", 0.0 },
  { "Luciell", 0.0 },
  { "Luciella", 0.0 },
  { "Lucielle", 0.0 },
  { "Lucien", 0.99938756 },
  { "Lucienna", 0.0 },
  { "Lucienne", 0.0 },
  { "Lucifer", 1.0 },
  { "Lucija", 0.0 },
  { "Lucil", 0.0 },
  { "Lucila", 0.0 },
  { "Lucile", 0.002766411 },
  { "Lucilia", 0.0 },
  { "Lucill", 0.0 },
  { "Lucilla", 0.0 },
  { "Lucille", 0.0033658096 },
  { "Lucillia", 0.0 },
  { "Lucillie", 0.0 },
  { "Lucina", 0.0 },
  { "Lucinda", 0.00029518316 },
  { "Lucindia", 0.0 },
  { "Lucindy", 0.0 },
  { "Lucine", 0.0 },
  { "Lucinia", 0.0 },
  { "Lucino", 1.0 },
  { "Lucio", 1.0 },
  { "Lucion", 1.0 },
  { "Lucious", 0.9989006 },
  { "Lucita", 0.0 },
  { "Lucius", 1.0 },
  { "Lucja", 0.0 },
  { "Luck", 1.0 },
  { "Luckas", 1.0 },
  { "Luckey", 1.0 },
  { "Luckie", 0.64285713 },
  { "Lucky", 0.875 },
  { "Lucrecia", 0.0 },
  { "Lucresha", 0.0 },
  { "Lucreshia", 0.0 },
  { "Lucresia", 0.0 },
  { "Lucreta", 0.0 },
  { "Lucretia", 0.0 },
  { "Lucrezia", 0.0 },
  { "Lucricia", 0.0 },
  { "Lucritia", 0.0 },
  { "Lucuis", 1.0 },
  { "Lucus", 1.0 },
  { "Lucy", 0.0031971252 },
  { "Lucya", 0.0 },
  { "Lucyana", 0.0 },
  { "Lucyann", 0.0 },
  { "Lucyanna", 0.0 },
  { "Lucye", 0.0 },
  { "Lucyle", 0.0 },
  { "Lucylle", 0.0 },
  { "Lucymae", 0.0 },
  { "Lucymarie", 0.0 },
  { "Lucyna", 0.0 },
  { "Lucynda", 0.0 },
  { "Lucyrose", 0.0 },
  { "Luda", 0.0 },
  { "Luddie", 0.102678575 },
  { "Luddy", 1.0 },
  { "Ludean", 0.0 },
  { "Ludell", 0.0 },
  { "Ludella", 0.0 },
  { "Ludelle", 0.0 },
  { "Ludene", 0.0 },
  { "Ludger", 1.0 },
  { "Ludia", 0.0 },
  { "Ludie", 0.09130876 },
  { "Ludine", 0.0 },
  { "Ludivina", 0.0 },
  { "Ludlow", 1.0 },
  { "Ludmila", 0.0 },
  { "Ludmilla", 0.0 },
  { "Ludora", 0.0 },
  { "Ludovic", 1.0 },
  { "Ludovica", 0.0 },
  { "Ludvig", 1.0 },
  { "Ludvik", 1.0 },
  { "Ludvina", 0.0 },
  { "Ludwell", 1.0 },
  { "Ludwig", 1.0 },
  { "Ludwik", 1.0 },
  { "Ludwika", 0.0 },
  { "Ludwin", 1.0 },
  { "Ludwina", 0.0 },
  { "Ludwing", 1.0 },
  { "Ludy", 0.09012876 },
  { "Lue", 0.09620218 },
  { "Lueann", 0.0 },
  { "Luease", 0.0 },
  { "Lueberta", 0.0 },
  { "Luebertha", 0.0 },
  { "Luecile", 0.0 },
  { "Luecille", 0.0 },
  { "Luecinda", 0.0 },
  { "Luedella", 0.0 },
  { "Luedora", 0.0 },
  { "Lueella", 0.0 },
  { "Luegene", 0.33333334 },
  { "Luegenia", 0.0 },
  { "Luel", 1.0 },
  { "Luell", 0.0 },
  { "Luella", 0.0005336891 },
  { "Luellar", 0.0 },
  { "Luelle", 0.0 },
  { "Luellen", 0.007194245 },
  { "Lueller", 0.0 },
  { "Luena", 0.0 },
  { "Luereatha", 0.0 },
  { "Luerene", 0.0 },
  { "Lueretha", 0.0 },
  { "Lueseal", 0.0 },
  { "Luethel", 0.0 },
  { "Luetricia", 0.0 },
  { "Luetta", 0.0 },
  { "Lueva", 0.0 },
  { "Luevenia", 0.0 },
  { "Luevertha", 0.0 },
  { "Luevina", 0.0 },
  { "Luevinia", 0.0 },
  { "Luffy", 1.0 },
  { "Lugarda", 0.0 },
  { "Lugardita", 0.0 },
  { "Lugene", 0.52870303 },
  { "Lugenia", 0.0 },
  { "Lugina", 0.0 },
  { "Lugman", 1.0 },
  { "Lui", 1.0 },
  { "Luian", 1.0 },
  { "Luicana", 0.0 },
  { "Luida", 0.0 },
  { "Luie", 1.0 },
  { "Luiggi", 1.0 },
  { "Luigi", 1.0 },
  { "Luigina", 0.0 },
  { "Luigino", 1.0 },
  { "Luigy", 1.0 },
  { "Luis", 0.9943695 },
  { "Luisa", 0.0027615062 },
  { "Luisafernanda", 0.0 },
  { "Luisalberto", 1.0 },
  { "Luisalejandro", 1.0 },
  { "Luisalfredo", 1.0 },
  { "Luisana", 0.0 },
  { "Luisandres", 1.0 },
  { "Luisandro", 1.0 },
  { "Luisangel", 1.0 },
  { "Luisanna", 0.0 },
  { "Luisantonio", 1.0 },
  { "Luisarmando", 1.0 },
  { "Luiscarlos", 1.0 },
  { "Luisdaniel", 1.0 },
  { "Luisdavid", 1.0 },
  { "Luisdiego", 1.0 },
  { "Luise", 0.008680556 },
  { "Luiseduardo", 1.0 },
  { "Luisenrique", 1.0 },
  { "Luisfelipe", 1.0 },
  { "Luisfernando", 1.0 },
  { "Luisgabriel", 1.0 },
  { "Luisgerardo", 1.0 },
  { "Luisgustavo", 1.0 },
  { "Luisiana", 0.0 },
  { "Luisita", 0.0 },
  { "Luisito", 1.0 },
  { "Luisjavier", 1.0 },
  { "Luisjose", 1.0 },
  { "Luismanuel", 1.0 },
  { "Luismario", 1.0 },
  { "Luismiguel", 1.0 },
  { "Luispablo", 1.0 },
  { "Luiz", 1.0 },
  { "Luiza", 0.0 },
  { "Lujack", 1.0 },
  { "Lujain", 0.0 },
  { "Lujane", 0.0 },
  { "Lujayn", 0.0 },
  { "Lujean", 0.0 },
  { "Lujuan", 0.0 },
  { "Lujuana", 0.0 },
  { "Luk", 1.0 },
  { "Luka", 0.942254 },
  { "Lukacs", 1.0 },
  { "Lukah", 1.0 },
  { "Lukan", 1.0 },
  { "Lukas", 0.9993679 },
  { "Lukasz", 1.0 },
  { "Lukaus", 1.0 },
  { "Luke", 0.99800664 },
  { "Lukeanthony", 1.0 },
  { "Luken", 1.0 },
  { "Lukes", 1.0 },
  { "Lukesha", 0.0 },
  { "Lukeshia", 0.0 },
  { "Lukeus", 1.0 },
  { "Lukian", 1.0 },
  { "Lukin", 1.0 },
  { "Lukis", 1.0 },
  { "Lukisha", 0.0 },
  { "Lukka", 1.0 },
  { "Lukkas", 1.0 },
  { "Lukman", 1.0 },
  { "Lukus", 1.0 },
  { "Lukyan", 1.0 },
  { "Lul", 0.0 },
  { "Lula", 0.00406754 },
  { "Lulabell", 0.0 },
  { "Lulabelle", 0.0 },
  { "Lulah", 0.0 },
  { "Lulamae", 0.0 },
  { "Lulani", 0.0 },
  { "Lular", 0.0 },
  { "Luler", 0.0 },
  { "Luley", 0.0 },
  { "Lulia", 0.0 },
  { "Lulie", 0.0 },
  { "Lulla", 0.0 },
  { "Lulu", 0.0 },
  { "Lulubell", 0.0 },
  { "Lulubelle", 0.0 },
  { "Lulwa", 0.0 },
  { "Luly", 0.0 },
  { "Lulya", 0.0 },
  { "Lum", 1.0 },
  { "Luma", 0.0 },
  { "Luman", 1.0 },
  { "Lumas", 1.0 },
  { "Lumen", 0.035897437 },
  { "Lumi", 0.0 },
  { "Lumin", 0.0 },
  { "Lumina", 0.0 },
  { "Lumir", 1.0 },
  { "Lummie", 0.34745762 },
  { "Lumumba", 1.0 },
  { "Lun", 0.0 },
  { "Luna", 0.0015414798 },
  { "Lunabella", 0.0 },
  { "Lunabelle", 0.0 },
  { "Lunah", 0.0 },
  { "Lunamarie", 0.0 },
  { "Lunarae", 0.0 },
  { "Lunda", 0.0 },
  { "Lunden", 0.41289198 },
  { "Lundin", 0.13157895 },
  { "Lundon", 0.30267063 },
  { "Lundy", 0.9522388 },
  { "Lundyn", 0.0077220076 },
  { "Lundynn", 0.0 },
  { "Lunell", 0.0 },
  { "Lunelle", 0.0 },
  { "Lunetta", 0.0 },
  { "Lunette", 0.0 },
  { "Lung", 1.0 },
  { "Lunie", 0.0 },
  { "Luniva", 0.0 },
  { "Lunna", 0.0 },
  { "Lunnie", 0.5925926 },
  { "Lunsford", 1.0 },
  { "Luola", 0.0 },
  { "Luong", 1.0 },
  { "Lupe", 0.24406628 },
  { "Lupie", 0.0 },
  { "Lupita", 0.0 },
  { "Luqmaan", 1.0 },
  { "Luqman", 1.0 },
  { "Lura", 0.0005278716 },
  { "Lurae", 0.0 },
  { "Lurah", 0.0 },
  { "Luraine", 0.0 },
  { "Lural", 1.0 },
  { "Lurana", 0.0 },
  { "Luray", 0.0 },
  { "Lurdes", 0.0 },
  { "Lurean", 0.0 },
  { "Lureatha", 0.0 },
  { "Luree", 0.0 },
  { "Lureen", 0.0 },
  { "Luren", 0.5 },
  { "Lurena", 0.0 },
  { "Lurenda", 0.0 },
  { "Lurene", 0.0 },
  { "Lureta", 0.0 },
  { "Luretha", 0.0 },
  { "Luretta", 0.0 },
  { "Luria", 0.0 },
  { "Lurie", 0.1300813 },
  { "Lurinda", 0.0 },
  { "Lurine", 0.0 },
  { "Lurla", 0.0 },
  { "Lurlean", 0.0 },
  { "Lurlee", 0.0 },
  { "Lurleen", 0.0 },
  { "Lurlene", 0.0 },
  { "Lurley", 0.0 },
  { "Lurlie", 0.0 },
  { "Lurline", 0.0 },
  { "Lurlyne", 0.0 },
  { "Lurton", 1.0 },
  { "Lus", 0.0 },
  { "Lusana", 0.0 },
  { "Luscious", 0.0 },
  { "Luseane", 0.0 },
  { "Luserito", 0.0 },
  { "Lusero", 0.0 },
  { "Lusetta", 0.0 },
  { "Lusha", 0.0 },
  { "Lushana", 0.0 },
  { "Lushawn", 0.0 },
  { "Lushonda", 0.0 },
  { "Lushus", 0.0 },
  { "Lusi", 0.0 },
  { "Lusia", 0.0 },
  { "Lusiana", 0.0 },
  { "Lusiano", 1.0 },
  { "Lusila", 0.0 },
  { "Lusine", 0.0 },
  { "Lusio", 1.0 },
  { "Luster", 0.9893899 },
  { "Lusvin", 1.0 },
  { "Luta", 0.0 },
  { "Lutalo", 1.0 },
  { "Lute", 1.0 },
  { "Lutece", 0.0 },
  { "Lutha", 0.0 },
  { "Luther", 0.99344444 },
  { "Luthera", 0.0 },
  { "Luthien", 0.0 },
  { "Luthor", 1.0 },
  { "Luthur", 1.0 },
  { "Luticia", 0.0 },
  { "Lutie", 0.0 },
  { "Lutina", 0.0 },
  { "Lutisha", 0.0 },
  { "Lutishia", 0.0 },
  { "Lutitia", 0.0 },
  { "Lutrecia", 0.0 },
  { "Lutrell", 0.75 },
  { "Lutrelle", 0.0 },
  { "Lutrica", 0.0 },
  { "Lutricia", 0.0 },
  { "Luttie", 0.0 },
  { "Lutz", 1.0 },
  { "Luu", 1.0 },
  { "Luukas", 1.0 },
  { "Luul", 0.0 },
  { "Luv", 0.37254903 },
  { "Luva", 0.0 },
  { "Luvada", 0.0 },
  { "Luvell", 1.0 },
  { "Luvella", 0.0 },
  { "Luvena", 0.0 },
  { "Luvender", 1.0 },
  { "Luvenia", 0.0 },
  { "Luvera", 0.0 },
  { "Luvern", 0.71313673 },
  { "Luverna", 0.0 },
  { "Luverne", 0.55723995 },
  { "Luvert", 1.0 },
  { "Luverta", 0.0 },
  { "Luvia", 0.0 },
  { "Luvina", 0.0 },
  { "Luvine", 0.0 },
  { "Luvinia", 0.0 },
  { "Luvonia", 0.0 },
  { "Luvonne", 0.0 },
  { "Luwam", 0.0 },
  { "Luwana", 0.0 },
  { "Luwanda", 0.0 },
  { "Luwanna", 0.0 },
  { "Lux", 0.21037464 },
  { "Luxanna", 0.0 },
  { "Luxe", 0.0 },
  { "Luxi", 0.0 },
  { "Luxie", 0.0 },
  { "Luxton", 1.0 },
  { "Luxx", 0.0 },
  { "Luxy", 0.0 },
  { "Luz", 0.034650248 },
  { "Luzclarita", 0.0 },
  { "Luzdari", 0.0 },
  { "Luzdary", 0.0 },
  { "Luzelena", 0.0 },
  { "Luzell", 0.5 },
  { "Luzenia", 0.0 },
  { "Luzer", 1.0 },
  { "Luzero", 0.0 },
  { "Luzetta", 0.0 },
  { "Luzette", 0.0 },
  { "Luzia", 0.0 },
  { "Luziana", 0.0 },
  { "Luziano", 1.0 },
  { "Luzma", 0.0 },
  { "Luzmar", 0.0 },
  { "Luzmaria", 0.0 },
  { "Luzmarie", 0.0 },
  { "Luzmary", 0.0 },
  { "Luzmila", 0.0 },
  { "Luzviminda", 0.0 },
  { "Lwren", 0.0 },
  { "Ly", 0.298995 },
  { "Lya", 0.0 },
  { "Lyah", 0.0 },
  { "Lyal", 1.0 },
  { "Lyall", 1.0 },
  { "Lyam", 1.0 },
  { "Lyan", 0.67676765 },
  { "Lyana", 0.0 },
  { "Lyanah", 0.0 },
  { "Lyandra", 0.0 },
  { "Lyani", 0.0 },
  { "Lyanie", 0.0 },
  { "Lyann", 0.0 },
  { "Lyanna", 0.0 },
  { "Lyannah", 0.0 },
  { "Lyanne", 0.0 },
  { "Lyanni", 0.0 },
  { "Lyasia", 0.0 },
  { "Lyba", 0.0 },
  { "Lyberti", 0.0 },
  { "Lybertie", 0.0 },
  { "Lyberty", 0.0 },
  { "Lycan", 1.0 },
  { "Lycia", 0.0 },
  { "Lycinda", 0.0 },
  { "Lycrecia", 0.0 },
  { "Lycurgus", 1.0 },
  { "Lyda", 0.001023332 },
  { "Lydale", 1.0 },
  { "Lyddia", 0.0 },
  { "Lyde", 0.0 },
  { "Lydea", 0.0 },
  { "Lydell", 0.9945534 },
  { "Lyden", 1.0 },
  { "Lydia", 0.001642077 },
  { "Lydiah", 0.0 },
  { "Lydian", 0.0 },
  { "Lydiana", 0.0 },
  { "Lydiann", 0.0 },
  { "Lydianna", 0.0 },
  { "Lydianne", 0.0 },
  { "Lydie", 0.0 },
  { "Lydon", 1.0 },
  { "Lydya", 0.0 },
  { "Lyela", 0.0 },
  { "Lyelah", 0.0 },
  { "Lyell", 1.0 },
  { "Lyfe", 0.7276786 },
  { "Lygia", 0.0 },
  { "Lyia", 0.0 },
  { "Lyiah", 0.0 },
  { "Lyiam", 1.0 },
  { "Lyjah", 1.0 },
  { "Lyka", 0.0 },
  { "Lyla", 0.0 },
  { "Lylah", 0.0 },
  { "Lylarae", 0.0 },
  { "Lylarose", 0.0 },
  { "Lylas", 0.0 },
  { "Lyle", 0.9833442 },
  { "Lylee", 0.0 },
  { "Lyleigh", 0.0 },
  { "Lyles", 1.0 },
  { "Lyli", 0.0 },
  { "Lylia", 0.0 },
  { "Lyliah", 0.0 },
  { "Lylian", 0.0 },
  { "Lyliana", 0.0 },
  { "Lylianah", 0.0 },
  { "Lyliann", 0.0 },
  { "Lylianna", 0.0 },
  { "Lylie", 0.0 },
  { "Lylis", 0.0 },
  { "Lylith", 0.0 },
  { "Lylla", 0.0 },
  { "Lyllah", 0.0 },
  { "Lylli", 0.0 },
  { "Lyllia", 0.0 },
  { "Lyllian", 0.0 },
  { "Lylliana", 0.0 },
  { "Lyllianna", 0.0 },
  { "Lyllie", 0.0 },
  { "Lyllyan", 0.0 },
  { "Lyly", 0.0 },
  { "Lylybell", 0.0 },
  { "Lyman", 1.0 },
  { "Lymari", 0.0 },
  { "Lymon", 1.0 },
  { "Lyn", 0.17851429 },
  { "Lyna", 0.0 },
  { "Lynae", 0.0 },
  { "Lynal", 1.0 },
  { "Lynann", 0.0 },
  { "Lynanne", 0.0 },
  { "Lynard", 1.0 },
  { "Lynasia", 0.0 },
  { "Lynaya", 0.0 },
  { "Lynch", 1.0 },
  { "Lyncoln", 0.68292683 },
  { "Lynda", 0.001528718 },
  { "Lyndah", 0.0 },
  { "Lyndal", 0.55977494 },
  { "Lyndale", 0.8333333 },
  { "Lyndall", 0.32707182 },
  { "Lyndan", 0.29411766 },
  { "Lynde", 0.0 },
  { "Lyndee", 0.0 },
  { "Lyndel", 0.70016474 },
  { "Lyndell", 0.6959236 },
  { "Lyndelle", 0.0 },
  { "Lynden", 0.7363531 },
  { "Lynder", 0.0 },
  { "Lyndi", 0.0 },
  { "Lyndia", 0.0 },
  { "Lyndie", 0.0 },
  { "Lyndin", 0.0 },
  { "Lyndle", 1.0 },
  { "Lyndley", 0.0 },
  { "Lyndol", 0.8918919 },
  { "Lyndon", 0.97446716 },
  { "Lyndora", 0.0 },
  { "Lyndra", 0.0 },
  { "Lyndsae", 0.0 },
  { "Lyndsay", 0.0 },
  { "Lyndse", 0.0 },
  { "Lyndsea", 0.0 },
  { "Lyndsee", 0.0 },
  { "Lyndsey", 0.0022541562 },
  { "Lyndsi", 0.0 },
  { "Lyndsie", 0.0 },
  { "Lyndsy", 0.0 },
  { "Lyndy", 0.0 },
  { "Lyndyn", 0.0 },
  { "Lyndze", 0.0 },
  { "Lyndzee", 0.0 },
  { "Lyndzey", 0.0 },
  { "Lyndzi", 0.0 },
  { "Lyndzie", 0.0 },
  { "Lyndzy", 0.0 },
  { "Lyne", 0.024154589 },
  { "Lynea", 0.0 },
  { "Lyneah", 0.0 },
  { "Lynee", 0.0 },
  { "Lyneisha", 0.0 },
  { "Lynel", 0.1875 },
  { "Lynell", 0.29103398 },
  { "Lynelle", 0.0 },
  { "Lynese", 0.0 },
  { "Lynesha", 0.0 },
  { "Lyneshia", 0.0 },
  { "Lynessa", 0.0 },
  { "Lynet", 0.0 },
  { "Lyneth", 0.0 },
  { "Lynett", 0.0 },
  { "Lynetta", 0.0 },
  { "Lynette", 0.0012373637 },
  { "Lynex", 0.0 },
  { "Lynford", 1.0 },
  { "Lynh", 0.0 },
  { "Lynia", 0.0 },
  { "Lyniah", 0.0 },
  { "Lynice", 0.0 },
  { "Lynika", 0.0 },
  { "Lynis", 0.0 },
  { "Lynise", 0.0 },
  { "Lynisha", 0.0 },
  { "Lynissa", 0.0 },
  { "Lynita", 0.0 },
  { "Lynix", 0.26785713 },
  { "Lyniya", 0.0 },
  { "Lyniyah", 0.0 },
  { "Lynk", 1.0 },
  { "Lynken", 1.0 },
  { "Lynkin", 1.0 },
  { "Lynkoln", 1.0 },
  { "Lynkon", 1.0 },
  { "Lynlea", 0.0 },
  { "Lynlee", 0.0 },
  { "Lynleigh", 0.0 },
  { "Lynley", 0.0 },
  { "Lynli", 0.0 },
  { "Lynlie", 0.0 },
  { "Lynly", 0.0 },
  { "Lynmarie", 0.0 },
  { "Lynn", 0.22321665 },
  { "Lynna", 0.0 },
  { "Lynnae", 0.0 },
  { "Lynnai", 0.0 },
  { "Lynnann", 0.0 },
  { "Lynnanne", 0.0 },
  { "Lynnasia", 0.0 },
  { "Lynnaya", 0.0 },
  { "Lynnda", 0.0 },
  { "Lynndsey", 0.0 },
  { "Lynne", 0.0073322956 },
  { "Lynnea", 0.0 },
  { "Lynneah", 0.0 },
  { "Lynneann", 0.0 },
  { "Lynnel", 0.0 },
  { "Lynnell", 0.0 },
  { "Lynnelle", 0.0 },
  { "Lynnesha", 0.0 },
  { "Lynnessa", 0.0 },
  { "Lynnet", 0.0 },
  { "Lynnete", 0.0 },
  { "Lynnett", 0.0 },
  { "Lynnetta", 0.0 },
  { "Lynnette", 0.0 },
  { "Lynnex", 0.0 },
  { "Lynnia", 0.0 },
  { "Lynniah", 0.0 },
  { "Lynnie", 0.0627451 },
  { "Lynnis", 0.0 },
  { "Lynnise", 0.0 },
  { "Lynnisha", 0.0 },
  { "Lynnita", 0.0 },
  { "Lynnix", 0.0 },
  { "Lynnlea", 0.0 },
  { "Lynnlee", 0.0 },
  { "Lynnleigh", 0.0 },
  { "Lynnley", 0.0 },
  { "Lynnly", 0.0 },
  { "Lynnmarie", 0.0 },
  { "Lynnon", 0.0 },
  { "Lynnox", 0.16949153 },
  { "Lynnsay", 0.0 },
  { "Lynnsey", 0.0 },
  { "Lynnsie", 0.0 },
  { "Lynnwood", 1.0 },
  { "Lynnzee", 0.0 },
  { "Lynnzie", 0.0 },
  { "Lynora", 0.0 },
  { "Lynore", 0.0 },
  { "Lynox", 0.41666666 },
  { "Lynsay", 0.0 },
  { "Lynse", 0.0 },
  { "Lynsea", 0.0 },
  { "Lynsee", 0.0 },
  { "Lynsey", 0.0 },
  { "Lynsi", 0.0 },
  { "Lynsie", 0.0 },
  { "Lynsy", 0.0 },
  { "Lynthia", 0.0 },
  { "Lynton", 1.0 },
  { "Lynus", 1.0 },
  { "Lynward", 1.0 },
  { "Lynwood", 0.9987975 },
  { "Lynx", 0.92424244 },
  { "Lynze", 0.0 },
  { "Lynzee", 0.0 },
  { "Lynzey", 0.0 },
  { "Lynzi", 0.0 },
  { "Lynzie", 0.0 },
  { "Lynzy", 0.0 },
  { "Lyola", 0.0 },
  { "Lyon", 1.0 },
  { "Lyonel", 1.0 },
  { "Lyonna", 0.0 },
  { "Lyra", 0.0 },
  { "Lyrae", 0.0 },
  { "Lyrah", 0.0 },
  { "Lyria", 0.0 },
  { "Lyric", 0.17242993 },
  { "Lyrica", 0.0 },
  { "Lyrical", 0.0 },
  { "Lyricc", 0.0 },
  { "Lyrick", 0.49802372 },
  { "Lyrics", 0.0 },
  { "Lyrie", 0.0 },
  { "Lyrik", 0.38203847 },
  { "Lyrika", 0.0 },
  { "Lyriq", 0.125 },
  { "Lyrique", 0.0 },
  { "Lyris", 0.0 },
  { "Lyrissa", 0.0 },
  { "Lyrix", 0.61538464 },
  { "Lyron", 1.0 },
  { "Lyrric", 0.0 },
  { "Lysa", 0.0 },
  { "Lysander", 1.0 },
  { "Lysandra", 0.0 },
  { "Lysbeth", 0.0 },
  { "Lysette", 0.0 },
  { "Lysha", 0.0 },
  { "Lyshon", 0.0 },
  { "Lysle", 1.0 },
  { "Lyssa", 0.0 },
  { "Lyssette", 0.0 },
  { "Lyta", 0.0 },
  { "Lyth", 1.0 },
  { "Lytia", 0.0 },
  { "Lytina", 0.0 },
  { "Lytisha", 0.0 },
  { "Lytle", 1.0 },
  { "Lytonia", 0.0 },
  { "Lytonya", 0.0 },
  { "Lytzi", 0.0 },
  { "Lytzy", 0.0 },
  { "Lyubov", 0.0 },
  { "Lyudmila", 0.0 },
  { "Lyvette", 0.0 },
  { "Lyvia", 0.0 },
  { "Lyvonne", 0.0 },
  { "Lywanda", 0.0 },
  { "Lyza", 0.0 },
  { "Lyzah", 0.0 },
  { "Lyzander", 1.0 },
  { "Lyzandra", 0.0 },
  { "Lyzbeth", 0.0 },
  { "Lyzeth", 0.0 },
  { "Lyzette", 0.0 },
  { "Ma", 0.0 },
  { "Maadhav", 1.0 },
  { "Maaha", 0.0 },
  { "Maahi", 0.0 },
  { "Maahika", 0.0 },
  { "Maahir", 1.0 },
  { "Maahira", 0.0 },
  { "Maaike", 0.0 },
  { "Maali", 0.0 },
  { "Maalik", 1.0 },
  { "Maaliyah", 0.0 },
  { "Maame", 0.0 },
  { "Maan", 1.0 },
  { "Maanas", 1.0 },
  { "Maanasa", 0.0 },
  { "Maanav", 1.0 },
  { "Maansi", 0.0 },
  { "Maanvi", 0.0 },
  { "Maanya", 0.0 },
  { "Maari", 0.0 },
  { "Maaria", 0.0 },
  { "Maariyah", 0.0 },
  { "Maarten", 1.0 },
  { "Maasai", 1.0 },
  { "Maaseiah", 1.0 },
  { "Maat", 0.0 },
  { "Maaya", 0.0 },
  { "Maayan", 0.023305085 },
  { "Maaz", 1.0 },
  { "Mabel", 0.0030539709 },
  { "Mabelene", 0.0 },
  { "Mabelin", 0.0 },
  { "Mabeline", 0.0 },
  { "Mabell", 0.0 },
  { "Mabelle", 0.0 },
  { "Mabelyn", 0.0 },
  { "Maber", 0.0 },
  { "Mabeth", 0.0 },
  { "Mabil", 0.0 },
  { "Mabinty", 0.0 },
  { "Mable", 0.0040569864 },
  { "Mablean", 0.0 },
  { "Mablene", 0.0 },
  { "Mabree", 0.0 },
  { "Mabrey", 0.035971224 },
  { "Mabrie", 0.0 },
  { "Mabry", 0.2223382 },
  { "Mabyn", 0.0 },
  { "Mac", 0.9959929 },
  { "Macade", 1.0 },
  { "Macaden", 1.0 },
  { "Macady", 0.0 },
  { "Macaela", 0.0 },
  { "Macai", 1.0 },
  { "Macaiah", 0.20987654 },
  { "Macaila", 0.0 },
  { "Macailah", 0.0 },
  { "Macailyn", 0.0 },
  { "Macaire", 0.5 },
  { "Macala", 0.0 },
  { "Macalah", 0.0 },
  { "Macaleb", 1.0 },
  { "Macalister", 1.0 },
  { "Macall", 0.0 },
  { "Macalla", 0.0 },
  { "Macallan", 0.9707602 },
  { "Macallen", 1.0 },
  { "Macallister", 1.0 },
  { "Macara", 0.0 },
  { "Macarena", 0.0 },
  { "Macari", 0.8607595 },
  { "Macaria", 0.0 },
  { "Macario", 1.0 },
  { "Macarius", 1.0 },
  { "Macarther", 1.0 },
  { "Macarthur", 1.0 },
  { "Macartney", 0.0 },
  { "Macaulay", 0.921875 },
  { "Macauley", 0.8795181 },
  { "Macauly", 1.0 },
  { "Macayla", 0.0 },
  { "Macaylah", 0.0 },
  { "Macayle", 0.08196721 },
  { "Macaylee", 0.0 },
  { "Maccabee", 1.0 },
  { "Maccoy", 1.0 },
  { "Macdonald", 1.0 },
  { "Mace", 1.0 },
  { "Macedonio", 1.0 },
  { "Macee", 0.0 },
  { "Macel", 0.006053269 },
  { "Macelyn", 0.0 },
  { "Macelynn", 0.0 },
  { "Macen", 0.99265784 },
  { "Macenna", 0.0 },
  { "Macenzi", 0.0 },
  { "Macenzie", 0.0 },
  { "Maceo", 1.0 },
  { "Maceon", 1.0 },
  { "Maceson", 1.0 },
  { "Macey", 0.0049770665 },
  { "Macgregor", 1.0 },
  { "Macguire", 1.0 },
  { "Macgyver", 1.0 },
  { "Mach", 1.0 },
  { "Machael", 0.975 },
  { "Machaela", 0.0 },
  { "Machai", 1.0 },
  { "Machala", 0.0 },
  { "Machayla", 0.0 },
  { "Mache", 0.0 },
  { "Macheal", 0.67346936 },
  { "Macheala", 0.0 },
  { "Machel", 0.078125 },
  { "Machela", 0.0 },
  { "Machele", 0.0 },
  { "Machell", 0.0 },
  { "Machella", 0.0 },
  { "Machelle", 0.0 },
  { "Machenzie", 0.13513513 },
  { "Machi", 1.0 },
  { "Machia", 0.0 },
  { "Machiah", 0.0 },
  { "Machias", 1.0 },
  { "Machiavelli", 1.0 },
  { "Machiya", 0.0 },
  { "Machlan", 1.0 },
  { "Maci", 0.0003820926 },
  { "Macia", 0.0 },
  { "Maciah", 0.46694216 },
  { "Macie", 0.001734848 },
  { "Maciee", 0.0 },
  { "Maciej", 1.0 },
  { "Maciel", 0.20481928 },
  { "Macii", 0.0 },
  { "Macil", 0.0 },
  { "Macilyn", 0.0 },
  { "Macilynn", 0.0 },
  { "Macin", 1.0 },
  { "Macintyre", 1.0 },
  { "Macio", 1.0 },
  { "Mack", 0.9952559 },
  { "Mackaela", 0.0 },
  { "Mackai", 1.0 },
  { "Mackala", 0.0 },
  { "Mackanzie", 0.0 },
  { "Mackay", 0.9113924 },
  { "Mackayla", 0.0 },
  { "Mackaylah", 0.0 },
  { "Mackaylee", 0.0 },
  { "Mackel", 1.0 },
  { "Mackena", 0.0 },
  { "Mackenize", 0.0 },
  { "Mackenlee", 0.0 },
  { "Mackenley", 0.0 },
  { "Mackenlie", 0.0 },
  { "Mackenna", 0.0026640026 },
  { "Mackennah", 0.0 },
  { "Mackensey", 0.0 },
  { "Mackensi", 0.0 },
  { "Mackensie", 0.005720824 },
  { "Mackenson", 1.0 },
  { "Mackensy", 0.0 },
  { "Mackenze", 0.0 },
  { "Mackenzee", 0.0 },
  { "Mackenzey", 0.0 },
  { "Mackenzi", 0.005711319 },
  { "Mackenzie", 0.056534797 },
  { "Mackenziee", 0.0 },
  { "Mackenzy", 0.10691824 },
  { "Mackenzye", 0.0 },
  { "Macker", 1.0 },
  { "Mackey", 1.0 },
  { "Mackeyla", 0.0 },
  { "Mackie", 0.6367713 },
  { "Mackinlee", 0.0 },
  { "Mackinley", 0.5258467 },
  { "Mackinnley", 1.0 },
  { "Mackinnon", 1.0 },
  { "Mackinsey", 0.0 },
  { "Mackinze", 0.0 },
  { "Mackinzee", 0.0 },
  { "Mackinzey", 0.0 },
  { "Mackinzi", 0.0 },
  { "Mackinzie", 0.018359853 },
  { "Mackinzy", 0.0 },
  { "Macklan", 1.0 },
  { "Macklen", 1.0 },
  { "Macklin", 1.0 },
  { "Macklyn", 0.2173913 },
  { "Macklynn", 0.0 },
  { "Macks", 1.0 },
  { "Macksen", 1.0 },
  { "Mackson", 1.0 },
  { "Mackston", 1.0 },
  { "Macky", 1.0 },
  { "Mackynze", 0.0 },
  { "Mackynzi", 0.0 },
  { "Mackynzie", 0.0 },
  { "Maclain", 0.9852941 },
  { "Maclaine", 0.17117117 },
  { "Maclan", 1.0 },
  { "Maclane", 1.0 },
  { "Maclaren", 0.21568628 },
  { "Maclay", 0.0 },
  { "Maclayne", 0.0 },
  { "Macle", 0.0 },
  { "Maclean", 1.0 },
  { "Maclin", 0.9672131 },
  { "Maclovia", 0.0 },
  { "Maclovio", 1.0 },
  { "Maclyn", 0.24242425 },
  { "Maclynn", 0.0 },
  { "Macola", 0.0 },
  { "Macolm", 1.0 },
  { "Macon", 0.9142857 },
  { "Macoy", 1.0 },
  { "Macrae", 1.0 },
  { "Macray", 1.0 },
  { "Macrina", 0.0 },
  { "Macsen", 1.0 },
  { "Macson", 1.0 },
  { "Macus", 1.0 },
  { "Macy", 0.019101046 },
  { "Macye", 0.0 },
  { "Macyn", 0.12945369 },
  { "Macynn", 0.0 },
  { "Mada", 0.0 },
  { "Madai", 0.0 },
  { "Madailein", 0.0 },
  { "Madajah", 0.0 },
  { "Madalaine", 0.0 },
  { "Madalee", 0.0 },
  { "Madaleine", 0.0 },
  { "Madalen", 0.0 },
  { "Madalena", 0.0 },
  { "Madalene", 0.0 },
  { "Madaleno", 1.0 },
  { "Madalie", 0.0 },
  { "Madaliene", 0.0 },
  { "Madalin", 0.0 },
  { "Madalina", 0.0 },
  { "Madaline", 0.0 },
  { "Madalinn", 0.0 },
  { "Madalon", 0.0 },
  { "Madaly", 0.0 },
  { "Madalyn", 0.0 },
  { "Madalyne", 0.0 },
  { "Madalynn", 0.0 },
  { "Madalynne", 0.0 },
  { "Madan", 1.0 },
  { "Madasen", 0.0 },
  { "Madasin", 0.0 },
  { "Madason", 0.0 },
  { "Madasyn", 0.0 },
  { "Madax", 1.0 },
  { "Maday", 0.0 },
  { "Madaya", 0.0 },
  { "Madchen", 0.0 },
  { "Maddalena", 0.0 },
  { "Maddalyn", 0.0 },
  { "Maddalynn", 0.0 },
  { "Maddan", 1.0 },
  { "Maddax", 0.97740114 },
  { "Maddee", 0.0 },
  { "Maddeline", 0.0 },
  { "Maddelyn", 0.0 },
  { "Maddelynn", 0.0 },
  { "Madden", 0.853737 },
  { "Maddex", 0.984047 },
  { "Maddi", 0.0 },
  { "Maddie", 0.0 },
  { "Maddielyn", 0.0 },
  { "Maddielynn", 0.0 },
  { "Maddigan", 0.0 },
  { "Maddilyn", 0.0 },
  { "Maddilynn", 0.0 },
  { "Maddin", 1.0 },
  { "Maddisen", 0.0 },
  { "Maddison", 0.002638078 },
  { "Maddisson", 0.0 },
  { "Maddisyn", 0.0 },
  { "Maddix", 0.86475945 },
  { "Maddlynn", 0.0 },
  { "Maddoc", 1.0 },
  { "Maddock", 1.0 },
  { "Maddon", 0.9787234 },
  { "Maddoux", 1.0 },
  { "Maddox", 0.96495384 },
  { "Maddoxx", 1.0 },
  { "Maddux", 0.93634164 },
  { "Maddy", 0.0 },
  { "Maddylynn", 0.0 },
  { "Maddyn", 0.041509435 },
  { "Maddysen", 0.0 },
  { "Maddyson", 0.0 },
  { "Maddyx", 0.5082645 },
  { "Madee", 0.0 },
  { "Madeeha", 0.0 },
  { "Madeira", 0.0 },
  { "Madel", 0.0 },
  { "Madelaine", 0.0 },
  { "Madelane", 0.0 },
  { "Madelayne", 0.0 },
  { "Madelein", 0.0 },
  { "Madeleine", 0.0003878675 },
  { "Madelen", 0.0 },
  { "Madelena", 0.0 },
  { "Madelene", 0.0 },
  { "Madeley", 0.0 },
  { "Madeleyn", 0.0 },
  { "Madeliene", 0.0 },
  { "Madelin", 0.0 },
  { "Madelina", 0.0 },
  { "Madeline", 0.0013201103 },
  { "Madelinerose", 0.0 },
  { "Madelinn", 0.0 },
  { "Madelinne", 0.0 },
  { "Madelis", 0.0 },
  { "Madeliz", 0.0 },
  { "Madell", 0.0 },
  { "Madella", 0.0 },
  { "Madelle", 0.0 },
  { "Madelline", 0.0 },
  { "Madellyn", 0.0 },
  { "Madellyne", 0.0 },
  { "Madelon", 0.0 },
  { "Madely", 0.0 },
  { "Madelyn", 0.0004540946 },
  { "Madelyne", 0.0 },
  { "Madelyngrace", 0.0 },
  { "Madelynn", 0.0 },
  { "Madelynne", 0.0 },
  { "Madelys", 0.0 },
  { "Maden", 1.0 },
  { "Madena", 0.0 },
  { "Madene", 0.0 },
  { "Madesen", 0.0 },
  { "Madeson", 0.0 },
  { "Madesyn", 0.0 },
  { "Madex", 1.0 },
  { "Madgalene", 0.0 },
  { "Madge", 0.0 },
  { "Madgel", 0.0 },
  { "Madgelene", 0.0 },
  { "Madgeline", 0.0 },
  { "Madgie", 0.0 },
  { "Madgline", 0.0 },
  { "Madhav", 1.0 },
  { "Madhavi", 0.0 },
  { "Madhulika", 0.0 },
  { "Madhumita", 0.0 },
  { "Madhumitha", 0.0 },
  { "Madhura", 0.0 },
  { "Madhuri", 0.0 },
  { "Madi", 0.0 },
  { "Madia", 0.0 },
  { "Madian", 0.0 },
  { "Madianna", 0.0 },
  { "Madiba", 1.0 },
  { "Madicella", 0.0 },
  { "Madicyn", 0.0 },
  { "Madie", 0.0 },
  { "Madielyn", 0.0 },
  { "Madielynn", 0.0 },
  { "Madigan", 0.0 },
  { "Madiha", 0.0 },
  { "Madilee", 0.0 },
  { "Madilen", 0.0 },
  { "Madilene", 0.0 },
  { "Madilin", 0.0 },
  { "Madiline", 0.0 },
  { "Madilyn", 0.0 },
  { "Madilyne", 0.0 },
  { "Madilynn", 0.0 },
  { "Madilynne", 0.0 },
  { "Madina", 0.0 },
  { "Madinah", 0.0 },
  { "Madine", 0.0 },
  { "Madis", 0.0 },
  { "Madisan", 0.0 },
  { "Madisen", 0.0 },
  { "Madisin", 0.0 },
  { "Madison", 0.020322546 },
  { "Madisongrace", 0.0 },
  { "Madisonrose", 0.0 },
  { "Madissen", 0.0 },
  { "Madisson", 0.0 },
  { "Madissyn", 0.0 },
  { "Madisun", 0.0 },
  { "Madisyn", 0.00034814092 },
  { "Madisynn", 0.0 },
  { "Madisynne", 0.0 },
  { "Madix", 1.0 },
  { "Madiyson", 0.0 },
  { "Madlen", 0.0 },
  { "Madlene", 0.0 },
  { "Madlin", 0.0 },
  { "Madline", 0.0 },
  { "Madlyn", 0.0 },
  { "Madlyne", 0.0 },
  { "Madlynn", 0.0 },
  { "Madlynne", 0.0 },
  { "Madoc", 1.0 },
  { "Madoka", 0.0 },
  { "Madolin", 0.0 },
  { "Madoline", 0.0 },
  { "Madolyn", 0.0 },
  { "Madolynn", 0.0 },
  { "Madon", 1.0 },
  { "Madona", 0.0 },
  { "Madonna", 0.0 },
  { "Madora", 0.0 },
  { "Madox", 1.0 },
  { "Madoxx", 1.0 },
  { "Madra", 0.0 },
  { "Madrene", 0.0 },
  { "Madria", 0.0 },
  { "Madrid", 0.13513513 },
  { "Madrona", 0.0 },
  { "Madrox", 1.0 },
  { "Mads", 1.0 },
  { "Madsen", 0.94623655 },
  { "Madsion", 0.0 },
  { "Madux", 1.0 },
  { "Mady", 0.0 },
  { "Madylan", 0.0 },
  { "Madylen", 0.0 },
  { "Madylin", 0.0 },
  { "Madyline", 0.0 },
  { "Madylyn", 0.0 },
  { "Madylynn", 0.0 },
  { "Madyn", 0.0 },
  { "Madysan", 0.0 },
  { "Madysen", 0.001939488 },
  { "Madysin", 0.0 },
  { "Madysn", 0.0 },
  { "Madyson", 0.0 },
  { "Madysson", 0.0 },
  { "Madysun", 0.0 },
  { "Madysyn", 0.0 },
  { "Madyx", 0.69911504 },
  { "Mae", 0.005447887 },
  { "Maeanna", 0.0 },
  { "Maebel", 0.0 },
  { "Maebell", 0.0 },
  { "Maebelle", 0.0 },
  { "Maebh", 0.0 },
  { "Maebree", 0.0 },
  { "Maebri", 0.0 },
  { "Maebrie", 0.0 },
  { "Maebry", 0.0 },
  { "Maeby", 0.0 },
  { "Maecee", 0.0 },
  { "Maeci", 0.0 },
  { "Maecie", 0.0 },
  { "Maecy", 0.0 },
  { "Maecyn", 0.0 },
  { "Maeda", 0.0 },
  { "Maedean", 0.0 },
  { "Maedell", 0.0 },
  { "Maedelle", 0.0 },
  { "Maedot", 0.0 },
  { "Maegan", 0.0 },
  { "Maegann", 0.0 },
  { "Maegen", 0.0 },
  { "Maeghan", 0.0 },
  { "Maeghen", 0.0 },
  { "Maegon", 0.0 },
  { "Maejor", 1.0 },
  { "Maekayla", 0.0 },
  { "Mael", 1.0 },
  { "Maela", 0.0 },
  { "Maelah", 0.0 },
  { "Maelana", 0.0 },
  { "Maelani", 0.0 },
  { "Maelea", 0.0 },
  { "Maeleah", 0.0 },
  { "Maelee", 0.0 },
  { "Maelei", 0.0 },
  { "Maeleigh", 0.0 },
  { "Maelena", 0.0 },
  { "Maelene", 0.0 },
  { "Maeley", 0.0 },
  { "Maeli", 0.0 },
  { "Maelia", 0.0 },
  { "Maelie", 0.0 },
  { "Maelin", 0.0 },
  { "Maelina", 0.0 },
  { "Maelis", 0.0 },
  { "Maeliyah", 0.0 },
  { "Maelle", 0.0 },
  { "Maelo", 1.0 },
  { "Maely", 0.0 },
  { "Maelyn", 0.0 },
  { "Maelynn", 0.0 },
  { "Maelynne", 0.0 },
  { "Maelys", 0.0 },
  { "Maemi", 0.0 },
  { "Maeola", 0.0 },
  { "Maeoma", 0.0 },
  { "Maera", 0.0 },
  { "Maerene", 0.0 },
  { "Maerose", 0.0 },
  { "Maeryn", 0.0 },
  { "Maesa", 0.0 },
  { "Maesen", 0.5 },
  { "Maesha", 0.0 },
  { "Maesie", 0.0 },
  { "Maeson", 0.7210777 },
  { "Maesyn", 0.12107623 },
  { "Maesynn", 0.0 },
  { "Maeta", 0.0 },
  { "Maetta", 0.0 },
  { "Maeva", 0.0 },
  { "Maeve", 0.0 },
  { "Maeven", 0.0 },
  { "Maevis", 0.0 },
  { "Maevry", 0.0 },
  { "Maevyn", 0.0 },
  { "Maewyn", 0.0 },
  { "Maeya", 0.0 },
  { "Maezell", 0.0 },
  { "Maezie", 0.0 },
  { "Maezy", 0.0 },
  { "Mafalda", 0.0 },
  { "Mafata", 0.0 },
  { "Mag", 0.0 },
  { "Magabi", 0.0 },
  { "Magaby", 0.0 },
  { "Magalena", 0.0 },
  { "Magalene", 0.0 },
  { "Magali", 0.0018242627 },
  { "Magalie", 0.0 },
  { "Magaline", 0.0 },
  { "Magalis", 0.0 },
  { "Magalli", 0.0 },
  { "Magally", 0.0 },
  { "Magaly", 0.0 },
  { "Magan", 0.0 },
  { "Magaret", 0.0 },
  { "Magarita", 0.0 },
  { "Magavi", 0.0 },
  { "Magda", 0.0 },
  { "Magdala", 0.0 },
  { "Magdalen", 0.0 },
  { "Magdalena", 0.0005820383 },
  { "Magdalene", 0.0 },
  { "Magdaleno", 1.0 },
  { "Magdali", 0.0 },
  { "Magdalia", 0.0 },
  { "Magdalina", 0.0 },
  { "Magdaline", 0.0 },
  { "Magdaly", 0.0 },
  { "Magdalyn", 0.0 },
  { "Magdalynn", 0.0 },
  { "Magdelana", 0.0 },
  { "Magdelena", 0.0 },
  { "Magdelene", 0.0 },
  { "Magdelin", 0.0 },
  { "Magdelina", 0.0 },
  { "Magdeline", 0.0 },
  { "Magdelyn", 0.0 },
  { "Magdiel", 0.9443255 },
  { "Magdlene", 0.0 },
  { "Magdline", 0.0 },
  { "Magean", 0.0 },
  { "Maged", 1.0 },
  { "Magee", 0.0 },
  { "Magel", 0.0 },
  { "Mageline", 0.0 },
  { "Magen", 0.0 },
  { "Magena", 0.0 },
  { "Magenta", 0.0 },
  { "Mager", 1.0 },
  { "Maggee", 0.0 },
  { "Maggen", 0.0 },
  { "Maggi", 0.0 },
  { "Maggie", 0.0024867945 },
  { "Maggielean", 0.0 },
  { "Maggiemae", 0.0 },
  { "Maggy", 0.0 },
  { "Maghan", 0.0 },
  { "Maghann", 0.0 },
  { "Maghen", 0.0 },
  { "Magi", 0.0 },
  { "Magic", 0.8671875 },
  { "Magie", 0.0 },
  { "Magin", 0.067567565 },
  { "Magline", 0.0 },
  { "Magno", 1.0 },
  { "Magnola", 0.0 },
  { "Magnolia", 0.0 },
  { "Magnum", 1.0 },
  { "Magnus", 1.0 },
  { "Magon", 0.0 },
  { "Maguadalupe", 0.0 },
  { "Maguerite", 0.0 },
  { "Maguette", 0.0 },
  { "Maguire", 0.8915663 },
  { "Magy", 0.0 },
  { "Mah", 0.0 },
  { "Maha", 0.0 },
  { "Mahad", 1.0 },
  { "Mahadev", 1.0 },
  { "Mahadi", 1.0 },
  { "Mahaela", 0.0 },
  { "Mahagany", 0.0 },
  { "Mahagony", 0.0 },
  { "Mahaila", 0.0 },
  { "Mahailey", 0.0 },
  { "Mahak", 0.0 },
  { "Mahal", 0.0 },
  { "Mahala", 0.0 },
  { "Mahalah", 0.0 },
  { "Mahalakshmi", 0.0 },
  { "Mahalet", 0.0 },
  { "Mahaley", 0.0 },
  { "Mahali", 0.0 },
  { "Mahalia", 0.0 },
  { "Mahaliah", 0.0 },
  { "Mahalie", 0.0 },
  { "Mahalya", 0.0 },
  { "Maham", 0.0 },
  { "Mahamadou", 1.0 },
  { "Mahamed", 1.0 },
  { "Mahammad", 1.0 },
  { "Mahamud", 1.0 },
  { "Mahan", 1.0 },
  { "Mahana", 0.0 },
  { "Mahani", 0.0 },
  { "Mahari", 0.51267606 },
  { "Mahasin", 0.0 },
  { "Mahathi", 0.0 },
  { "Mahati", 0.0 },
  { "Mahawa", 0.0 },
  { "Mahayla", 0.0 },
  { "Mahaylee", 0.0 },
  { "Mahaylia", 0.0 },
  { "Mahbeer", 1.0 },
  { "Mahd", 1.0 },
  { "Mahde", 1.0 },
  { "Mahdee", 1.0 },
  { "Mahdeen", 0.0 },
  { "Mahdi", 1.0 },
  { "Mahdia", 0.0 },
  { "Mahdiya", 0.0 },
  { "Mahdy", 1.0 },
  { "Mahealani", 0.0 },
  { "Mahee", 0.0 },
  { "Maheen", 0.011131725 },
  { "Maheer", 1.0 },
  { "Mahek", 0.0 },
  { "Mahelet", 0.0 },
  { "Mahely", 0.0 },
  { "Mahendra", 1.0 },
  { "Maher", 1.0 },
  { "Mahera", 0.0 },
  { "Mahesh", 1.0 },
  { "Mahi", 0.016465422 },
  { "Mahia", 0.0 },
  { "Mahib", 1.0 },
  { "Mahibah", 0.0 },
  { "Mahid", 1.0 },
  { "Mahie", 0.0 },
  { "Mahika", 0.0 },
  { "Mahima", 0.0 },
  { "Mahin", 0.89873415 },
  { "Mahina", 0.0 },
  { "Mahir", 1.0 },
  { "Mahira", 0.0 },
  { "Mahirah", 0.0 },
  { "Mahisha", 0.0 },
  { "Mahit", 1.0 },
  { "Mahita", 0.0 },
  { "Mahith", 1.0 },
  { "Mahitha", 0.0 },
  { "Mahiya", 0.0 },
  { "Mahiyah", 0.0 },
  { "Mahjabeen", 0.0 },
  { "Mahkai", 1.0 },
  { "Mahkayla", 0.0 },
  { "Mahkaylah", 0.0 },
  { "Mahkel", 1.0 },
  { "Mahkenna", 0.0 },
  { "Mahkenzie", 0.0 },
  { "Mahkhi", 1.0 },
  { "Mahki", 1.0 },
  { "Mahkiya", 0.0 },
  { "Mahkya", 0.0 },
  { "Mahkyla", 0.0 },
  { "Mahlah", 0.0 },
  { "Mahlani", 0.0 },
  { "Mahlaya", 0.0 },
  { "Mahlea", 0.0 },
  { "Mahleah", 0.0 },
  { "Mahleek", 1.0 },
  { "Mahlek", 1.0 },
  { "Mahlena", 0.0 },
  { "Mahlet", 0.0 },
  { "Mahli", 0.0 },
  { "Mahlia", 0.0 },
  { "Mahlik", 1.0 },
  { "Mahliya", 0.0 },
  { "Mahliyah", 0.0 },
  { "Mahlon", 0.9978752 },
  { "Mahmood", 1.0 },
  { "Mahmoud", 1.0 },
  { "Mahmud", 1.0 },
  { "Mahnaz", 0.0 },
  { "Mahniya", 0.0 },
  { "Mahnoor", 0.0 },
  { "Maho", 0.0 },
  { "Mahoganee", 0.0 },
  { "Mahoganey", 0.0 },
  { "Mahogani", 0.0 },
  { "Mahoganie", 0.0 },
  { "Mahogany", 0.0 },
  { "Mahoghany", 0.0 },
  { "Mahogony", 0.0 },
  { "Mahonri", 1.0 },
  { "Mahra", 0.0 },
  { "Mahreen", 0.0 },
  { "Mahri", 0.0 },
  { "Mahria", 0.0 },
  { "Mahriah", 0.0 },
  { "Mahrosh", 0.0 },
  { "Mahrukh", 0.0 },
  { "Mahrus", 1.0 },
  { "Mahsa", 0.0 },
  { "Mahsiah", 1.0 },
  { "Mahtab", 0.0 },
  { "Mahue", 1.0 },
  { "Mahum", 0.0 },
  { "Mahveen", 0.0 },
  { "Mahwish", 0.0 },
  { "Mahya", 0.0 },
  { "Mahyar", 1.0 },
  { "Mai", 0.0038175224 },
  { "Maia", 0.0 },
  { "Maiah", 0.0 },
  { "Maialen", 0.0 },
  { "Maiana", 0.0 },
  { "Maianh", 0.0 },
  { "Maiara", 0.0 },
  { "Maica", 0.0 },
  { "Maicee", 0.0 },
  { "Maicey", 0.0 },
  { "Maichael", 1.0 },
  { "Maicie", 0.0 },
  { "Maico", 1.0 },
  { "Maicol", 1.0 },
  { "Maicon", 1.0 },
  { "Maicy", 0.0 },
  { "Maicyn", 0.0 },
  { "Maida", 0.0 },
  { "Maidah", 0.0 },
  { "Maide", 0.0 },
  { "Maidee", 0.0 },
  { "Maidell", 0.0 },
  { "Maidelyn", 0.0 },
  { "Maiden", 0.0 },
  { "Maider", 0.0 },
  { "Maidie", 0.0 },
  { "Maidson", 0.0 },
  { "Maie", 0.0 },
  { "Maiella", 0.0 },
  { "Maielle", 0.0 },
  { "Maier", 1.0 },
  { "Maiesha", 0.0 },
  { "Maigan", 0.0 },
  { "Maigen", 0.0 },
  { "Maigon", 0.0 },
  { "Maiha", 0.0 },
  { "Maija", 0.0 },
  { "Maijah", 0.0 },
  { "Maijor", 1.0 },
  { "Maika", 0.35283992 },
  { "Maikayla", 0.0 },
  { "Maikel", 1.0 },
  { "Maiken", 0.0 },
  { "Maiki", 1.0 },
  { "Maikia", 0.0 },
  { "Maiko", 0.28282827 },
  { "Maikol", 1.0 },
  { "Maikou", 0.0 },
  { "Maila", 0.0 },
  { "Mailah", 0.0 },
  { "Mailan", 0.0 },
  { "Mailani", 0.0 },
  { "Mailany", 0.0 },
  { "Maile", 0.0 },
  { "Mailea", 0.0 },
  { "Mailee", 0.0 },
  { "Maileen", 0.0 },
  { "Mailei", 0.0 },
  { "Maileigh", 0.0 },
  { "Mailen", 0.0 },
  { "Mailene", 0.0 },
  { "Mailey", 0.0 },
  { "Maili", 0.0 },
  { "Mailia", 0.0 },
  { "Mailie", 0.0 },
  { "Mailin", 0.0 },
  { "Mailing", 0.0 },
  { "Mailinh", 0.0 },
  { "Mailk", 1.0 },
  { "Maille", 0.0 },
  { "Mailo", 1.0 },
  { "Mailon", 1.0 },
  { "Maily", 0.0 },
  { "Mailyn", 0.0 },
  { "Mailynn", 0.0 },
  { "Maimoona", 0.0 },
  { "Maimouna", 0.0 },
  { "Maimuna", 0.0 },
  { "Maina", 0.0 },
  { "Maine", 0.0 },
  { "Mainhia", 0.0 },
  { "Mainor", 1.0 },
  { "Mainou", 0.0 },
  { "Maiquel", 0.0 },
  { "Maira", 0.002936604 },
  { "Mairah", 0.0 },
  { "Mairani", 0.0 },
  { "Mairany", 0.0 },
  { "Maire", 0.0 },
  { "Mairead", 0.0 },
  { "Maireli", 0.0 },
  { "Mairely", 0.0 },
  { "Mairen", 0.0 },
  { "Mairene", 0.0 },
  { "Mairenn", 0.0 },
  { "Mairi", 0.0 },
  { "Mairim", 0.0 },
  { "Mairin", 0.0 },
  { "Mairlyn", 0.0 },
  { "Mairon", 1.0 },
  { "Mairyn", 0.0 },
  { "Mais", 0.0 },
  { "Maisa", 0.0 },
  { "Maisah", 0.0 },
  { "Maisee", 0.0 },
  { "Maisen", 0.78137654 },
  { "Maisey", 0.0 },
  { "Maisha", 0.0 },
  { "Maisie", 0.0 },
  { "Maisley", 0.0 },
  { "Maislyn", 0.0 },
  { "Maislynn", 0.0 },
  { "Maison", 0.8238824 },
  { "Maissa", 0.0 },
  { "Maisy", 0.0 },
  { "Maisyn", 0.0875 },
  { "Maita", 0.0 },
  { "Maite", 0.0 },
  { "Maitha", 0.0 },
  { "Maithe", 0.0 },
  { "Maithili", 0.0 },
  { "Maiti", 0.0 },
  { "Maitland", 0.6387283 },
  { "Maitlin", 0.0 },
  { "Maitlyn", 0.0 },
  { "Maitreya", 0.5882353 },
  { "Maitreyi", 0.0 },
  { "Maitri", 0.0 },
  { "Maitte", 0.0 },
  { "Maive", 0.0 },
  { "Maiven", 0.0 },
  { "Maivy", 0.0 },
  { "Maiya", 0.0 },
  { "Maiyah", 0.0 },
  { "Maiyana", 0.0 },
  { "Maiyer", 0.0 },
  { "Maiysha", 0.0 },
  { "Maiza", 0.0 },
  { "Maizah", 0.0 },
  { "Maize", 0.20408164 },
  { "Maizee", 0.0 },
  { "Maizen", 1.0 },
  { "Maizey", 0.0 },
  { "Maizie", 0.0 },
  { "Maizley", 0.0 },
  { "Maizlyn", 0.0 },
  { "Maizy", 0.0 },
  { "Maja", 0.0 },
  { "Majayla", 0.0 },
  { "Majd", 0.9744898 },
  { "Majda", 0.0 },
  { "Majdi", 1.0 },
  { "Majed", 1.0 },
  { "Majeed", 1.0 },
  { "Majeeda", 0.0 },
  { "Majel", 0.0 },
  { "Majella", 0.0 },
  { "Majenta", 0.0 },
  { "Majer", 1.0 },
  { "Majerle", 0.0 },
  { "Majesta", 0.0 },
  { "Majesti", 0.0 },
  { "Majestic", 0.35103244 },
  { "Majestie", 0.0 },
  { "Majesty", 0.22254616 },
  { "Majic", 1.0 },
  { "Majid", 1.0 },
  { "Majida", 0.0 },
  { "Majik", 1.0 },
  { "Majken", 0.0 },
  { "Majok", 1.0 },
  { "Major", 0.9981184 },
  { "Majorie", 0.0 },
  { "Majorlynn", 0.0 },
  { "Majour", 1.0 },
  { "Majuma", 0.0 },
  { "Majur", 1.0 },
  { "Mak", 1.0 },
  { "Maka", 0.0 },
  { "Makada", 0.0 },
  { "Makade", 1.0 },
  { "Makaden", 1.0 },
  { "Makael", 1.0 },
  { "Makaela", 0.0 },
  { "Makaelah", 0.0 },
  { "Makaelyn", 0.0 },
  { "Makaelynn", 0.0 },
  { "Makah", 1.0 },
  { "Makahi", 1.0 },
  { "Makahia", 0.0 },
  { "Makahla", 0.0 },
  { "Makai", 0.95190465 },
  { "Makaia", 0.0 },
  { "Makaiah", 0.03816794 },
  { "Makaiden", 1.0 },
  { "Makaih", 1.0 },
  { "Makaii", 1.0 },
  { "Makail", 1.0 },
  { "Makaila", 0.0 },
  { "Makailah", 0.0 },
  { "Makailee", 0.0 },
  { "Makailey", 0.0 },
  { "Makaili", 0.0 },
  { "Makaily", 0.0 },
  { "Makailyn", 0.0 },
  { "Makailynn", 0.0 },
  { "Makaio", 1.0 },
  { "Makaira", 0.0 },
  { "Makaius", 1.0 },
  { "Makaiya", 0.0 },
  { "Makaiyah", 0.0 },
  { "Makaiyla", 0.0 },
  { "Makal", 1.0 },
  { "Makala", 0.0014925373 },
  { "Makalah", 0.0 },
  { "Makalani", 1.0 },
  { "Makale", 1.0 },
  { "Makalea", 0.0 },
  { "Makaleb", 1.0 },
  { "Makalee", 0.0 },
  { "Makalei", 0.0 },
  { "Makaleigh", 0.0 },
  { "Makaley", 0.0 },
  { "Makali", 0.0 },
  { "Makalia", 0.0 },
  { "Makaliah", 0.0 },
  { "Makalie", 0.0 },
  { "Makalla", 0.0 },
  { "Makalya", 0.0 },
  { "Makalyn", 0.0 },
  { "Makalynn", 0.0 },
  { "Makamae", 0.0 },
  { "Makana", 0.61764705 },
  { "Makanalei", 0.0 },
  { "Makani", 0.864 },
  { "Makanna", 0.0 },
  { "Makao", 1.0 },
  { "Makar", 1.0 },
  { "Makara", 0.0 },
  { "Makarah", 0.0 },
  { "Makari", 0.8366606 },
  { "Makaria", 0.0 },
  { "Makarie", 1.0 },
  { "Makario", 1.0 },
  { "Makarios", 1.0 },
  { "Makarius", 1.0 },
  { "Makasha", 0.0 },
  { "Makasia", 0.0 },
  { "Makauri", 1.0 },
  { "Makaveli", 1.0 },
  { "Makay", 0.4861111 },
  { "Makaya", 0.0 },
  { "Makayah", 0.0 },
  { "Makayda", 0.0 },
  { "Makayden", 0.31578946 },
  { "Makayela", 0.0 },
  { "Makayia", 0.0 },
  { "Makayiah", 0.0 },
  { "Makayla", 0.0015279299 },
  { "Makaylah", 0.0 },
  { "Makaylan", 0.0 },
  { "Makayle", 0.0 },
  { "Makaylea", 0.0 },
  { "Makaylee", 0.0 },
  { "Makayleigh", 0.0 },
  { "Makaylen", 0.0 },
  { "Makayley", 0.0 },
  { "Makayli", 0.0 },
  { "Makaylia", 0.0 },
  { "Makayliah", 0.0 },
  { "Makaylie", 0.0 },
  { "Makaylin", 0.0 },
  { "Makaylla", 0.0 },
  { "Makayln", 0.0 },
  { "Makaylyn", 0.0 },
  { "Makaylynn", 0.0 },
  { "Makayo", 1.0 },
  { "Makbel", 1.0 },
  { "Makda", 0.0 },
  { "Makea", 0.0 },
  { "Makeba", 0.0 },
  { "Makeda", 0.0 },
  { "Makeeba", 0.0 },
  { "Makeen", 1.0 },
  { "Makeena", 0.0 },
  { "Makeesha", 0.0 },
  { "Makei", 1.0 },
  { "Makeia", 0.0 },
  { "Makeila", 0.0 },
  { "Makeira", 0.0 },
  { "Makeisha", 0.0 },
  { "Makeitha", 0.0 },
  { "Makel", 0.9296188 },
  { "Makela", 0.0 },
  { "Makelah", 0.0 },
  { "Makell", 0.64268583 },
  { "Makella", 0.0 },
  { "Makelle", 0.0 },
  { "Makena", 0.015303334 },
  { "Makenah", 0.0 },
  { "Makendra", 0.0 },
  { "Makenize", 0.0 },
  { "Makenlee", 0.0 },
  { "Makenlei", 0.0 },
  { "Makenleigh", 0.0 },
  { "Makenley", 0.0 },
  { "Makenli", 0.0 },
  { "Makenlie", 0.0 },
  { "Makenly", 0.0 },
  { "Makenna", 0.00029879733 },
  { "Makennah", 0.0 },
  { "Makennzie", 0.0 },
  { "Makensey", 0.0 },
  { "Makensi", 0.0 },
  { "Makensie", 0.0 },
  { "Makensley", 0.0 },
  { "Makensy", 0.0 },
  { "Makenya", 0.0 },
  { "Makenze", 0.0 },
  { "Makenzee", 0.0 },
  { "Makenzey", 0.0 },
  { "Makenzi", 0.0 },
  { "Makenzie", 0.0074832407 },
  { "Makenzii", 0.0 },
  { "Makenzlee", 0.0 },
  { "Makenzley", 0.0 },
  { "Makenzly", 0.0 },
  { "Makenzy", 0.0 },
  { "Makenzye", 0.0 },
  { "Makeo", 1.0 },
  { "Makesha", 0.0 },
  { "Makeshia", 0.0 },
  { "Makesia", 0.0 },
  { "Maketa", 0.0 },
  { "Maketta", 0.0 },
  { "Makeva", 0.0 },
  { "Makeya", 0.0 },
  { "Makeyla", 0.0 },
  { "Makhai", 1.0 },
  { "Makhail", 1.0 },
  { "Makhaila", 0.0 },
  { "Makhala", 0.0 },
  { "Makhari", 0.84810126 },
  { "Makhaya", 0.0 },
  { "Makhayla", 0.0 },
  { "Makhel", 1.0 },
  { "Makhi", 0.9827185 },
  { "Makhia", 0.0 },
  { "Makhii", 1.0 },
  { "Makhila", 0.0 },
  { "Makhiya", 0.0 },
  { "Makhiyah", 0.0 },
  { "Makhy", 1.0 },
  { "Makhya", 0.0 },
  { "Makhyia", 0.0 },
  { "Makhyla", 0.0 },
  { "Maki", 0.7077922 },
  { "Makia", 0.020802377 },
  { "Makiah", 0.14925373 },
  { "Makiala", 0.0 },
  { "Makiaya", 0.0 },
  { "Makiba", 0.0 },
  { "Makida", 0.0 },
  { "Makieh", 1.0 },
  { "Makiel", 1.0 },
  { "Makiesha", 0.0 },
  { "Makieya", 0.0 },
  { "Makih", 1.0 },
  { "Makii", 1.0 },
  { "Makiko", 0.0 },
  { "Makil", 1.0 },
  { "Makila", 0.0 },
  { "Makilah", 0.0 },
  { "Makiley", 0.0 },
  { "Makin", 1.0 },
  { "Makina", 0.0 },
  { "Makini", 0.0 },
  { "Makinlee", 0.0 },
  { "Makinleigh", 0.0 },
  { "Makinley", 0.0 },
  { "Makinlie", 0.0 },
  { "Makinly", 0.0 },
  { "Makinna", 0.0 },
  { "Makinnah", 0.0 },
  { "Makinnley", 0.0 },
  { "Makinsey", 0.0 },
  { "Makinsley", 0.0 },
  { "Makinze", 0.0 },
  { "Makinzee", 0.0 },
  { "Makinzey", 0.0 },
  { "Makinzi", 0.0 },
  { "Makinzie", 0.0 },
  { "Makinzy", 0.0 },
  { "Makira", 0.0 },
  { "Makirah", 0.0 },
  { "Makis", 1.0 },
  { "Makisha", 0.0 },
  { "Makita", 0.0 },
  { "Makiy", 1.0 },
  { "Makiya", 0.0051832655 },
  { "Makiyah", 0.007256894 },
  { "Makiyha", 0.0 },
  { "Makiyla", 0.0 },
  { "Makiylah", 0.0 },
  { "Makkah", 0.0 },
  { "Makkiyah", 0.0 },
  { "Maklyn", 0.0 },
  { "Maklynn", 0.0 },
  { "Mako", 0.6101695 },
  { "Makoa", 1.0 },
  { "Makoi", 1.0 },
  { "Makonnen", 1.0 },
  { "Makoto", 1.0 },
  { "Makoy", 1.0 },
  { "Maks", 1.0 },
  { "Maksen", 1.0 },
  { "Maksim", 1.0 },
  { "Maksimus", 1.0 },
  { "Makson", 1.0 },
  { "Makston", 1.0 },
  { "Maksym", 1.0 },
  { "Maksymilian", 1.0 },
  { "Makua", 1.0 },
  { "Makya", 0.08394161 },
  { "Makyah", 0.05988858 },
  { "Makye", 1.0 },
  { "Makyi", 1.0 },
  { "Makyia", 0.0 },
  { "Makyiah", 0.0 },
  { "Makyla", 0.0 },
  { "Makylah", 0.0 },
  { "Makyle", 1.0 },
  { "Makylee", 0.0 },
  { "Makyleigh", 0.0 },
  { "Makylia", 0.0 },
  { "Makylie", 0.0 },
  { "Makyna", 0.0 },
  { "Makynlee", 0.0 },
  { "Makynlei", 0.0 },
  { "Makynleigh", 0.0 },
  { "Makynley", 0.0 },
  { "Makynli", 0.0 },
  { "Makynlie", 0.0 },
  { "Makynna", 0.0 },
  { "Makynsie", 0.0 },
  { "Makynze", 0.0 },
  { "Makynzee", 0.0 },
  { "Makynzi", 0.0 },
  { "Makynzie", 0.0 },
  { "Makyra", 0.0 },
  { "Makyrah", 0.0 },
  { "Mal", 1.0 },
  { "Mala", 0.0 },
  { "Malaak", 0.0 },
  { "Malacai", 1.0 },
  { "Malachai", 1.0 },
  { "Malachi", 0.99482054 },
  { "Malachia", 0.3548387 },
  { "Malachy", 1.0 },
  { "Malacia", 0.0 },
  { "Malack", 0.0 },
  { "Malacki", 1.0 },
  { "Malae", 0.0 },
  { "Malaeka", 0.0 },
  { "Malaena", 0.0 },
  { "Malaeya", 0.0 },
  { "Malahkai", 1.0 },
  { "Malahki", 1.0 },
  { "Malahni", 0.0 },
  { "Malai", 0.0 },
  { "Malaia", 0.0 },
  { "Malaiah", 0.0 },
  { "Malaija", 0.0 },
  { "Malaijah", 0.0 },
  { "Malaika", 0.0 },
  { "Malaikah", 0.0 },
  { "Malaila", 0.0 },
  { "Malaina", 0.0 },
  { "Malaine", 0.0 },
  { "Malaisha", 0.0 },
  { "Malaisia", 0.0 },
  { "Malaiya", 0.0 },
  { "Malaiyah", 0.0 },
  { "Malaja", 0.0 },
  { "Malajah", 0.0 },
  { "Malajia", 0.0 },
  { "Malak", 0.05535966 },
  { "Malaka", 0.0 },
  { "Malakai", 0.9966162 },
  { "Malakei", 1.0 },
  { "Malakhai", 1.0 },
  { "Malakhi", 1.0 },
  { "Malaki", 0.99851763 },
  { "Malakia", 0.8596491 },
  { "Malakiah", 1.0 },
  { "Malakie", 1.0 },
  { "Malakii", 1.0 },
  { "Malaky", 1.0 },
  { "Malakye", 1.0 },
  { "Malakyi", 1.0 },
  { "Malala", 0.0 },
  { "Malalai", 0.0 },
  { "Malan", 0.037037037 },
  { "Malana", 0.0 },
  { "Malanda", 0.0 },
  { "Malaney", 0.0 },
  { "Malani", 0.0 },
  { "Malania", 0.0 },
  { "Malanie", 0.0 },
  { "Malanii", 0.0 },
  { "Malanna", 0.0 },
  { "Malanni", 0.0 },
  { "Malany", 0.0 },
  { "Malanya", 0.0 },
  { "Malaquias", 1.0 },
  { "Malaree", 0.0 },
  { "Malari", 0.0 },
  { "Malarie", 0.0 },
  { "Malary", 0.0 },
  { "Malasha", 0.0 },
  { "Malashia", 0.0 },
  { "Malasia", 0.0 },
  { "Malaun", 0.0 },
  { "Malav", 1.0 },
  { "Malavika", 0.0 },
  { "Malay", 0.0 },
  { "Malaya", 0.0 },
  { "Malayah", 0.0 },
  { "Malayasia", 0.0 },
  { "Malayha", 0.0 },
  { "Malayia", 0.0 },
  { "Malayiah", 0.0 },
  { "Malayja", 0.0 },
  { "Malayjah", 0.0 },
  { "Malayka", 0.0 },
  { "Malayla", 0.0 },
  { "Malayna", 0.0 },
  { "Malaysa", 0.0 },
  { "Malaysha", 0.0 },
  { "Malayshia", 0.0 },
  { "Malaysia", 0.0 },
  { "Malaysiah", 0.0 },
  { "Malayzia", 0.0 },
  { "Malaz", 0.0 },
  { "Malazia", 0.0 },
  { "Malchijah", 1.0 },
  { "Malcohm", 1.0 },
  { "Malcolm", 0.9980351 },
  { "Malcolmjamal", 1.0 },
  { "Malcoln", 1.0 },
  { "Malcom", 1.0 },
  { "Malcomb", 1.0 },
  { "Malcome", 1.0 },
  { "Malcon", 1.0 },
  { "Malcum", 1.0 },
  { "Malda", 0.0 },
  { "Malden", 1.0 },
  { "Male", 1.0 },
  { "Malea", 0.0 },
  { "Maleah", 0.0 },
  { "Maleaha", 0.0 },
  { "Maleak", 1.0 },
  { "Maleana", 0.0 },
  { "Maleaya", 0.0 },
  { "Malec", 1.0 },
  { "Malechi", 1.0 },
  { "Malecia", 0.0 },
  { "Maleck", 1.0 },
  { "Malee", 0.0 },
  { "Maleea", 0.0 },
  { "Maleeah", 0.0 },
  { "Maleeha", 0.0 },
  { "Maleehah", 0.0 },
  { "Maleek", 1.0 },
  { "Maleeka", 0.0 },
  { "Maleena", 0.0 },
  { "Maleesa", 0.0 },
  { "Maleeya", 0.0 },
  { "Maleeyah", 0.0 },
  { "Maleficent", 0.0 },
  { "Maleha", 0.0 },
  { "Malei", 0.0 },
  { "Maleia", 0.0 },
  { "Maleiah", 0.0 },
  { "Maleigh", 0.0 },
  { "Maleigha", 0.0 },
  { "Maleik", 1.0 },
  { "Maleika", 0.0 },
  { "Maleina", 0.0 },
  { "Maleisha", 0.0 },
  { "Maleiya", 0.0 },
  { "Maleiyah", 0.0 },
  { "Malek", 0.997006 },
  { "Maleka", 0.0 },
  { "Malekai", 1.0 },
  { "Maleke", 1.0 },
  { "Malekhi", 1.0 },
  { "Maleki", 1.0 },
  { "Malekia", 0.0 },
  { "Maleko", 1.0 },
  { "Malen", 0.75 },
  { "Malena", 0.0 },
  { "Malenda", 0.0 },
  { "Malene", 0.0 },
  { "Maleni", 0.0 },
  { "Malenie", 0.0 },
  { "Malenna", 0.0 },
  { "Malenny", 0.0 },
  { "Maleny", 0.0 },
  { "Maleri", 0.0 },
  { "Malerie", 0.0 },
  { "Malery", 0.0 },
  { "Malesa", 0.0 },
  { "Malesha", 0.0 },
  { "Malesia", 0.0 },
  { "Malessa", 0.0 },
  { "Maleta", 0.0 },
  { "Maletha", 0.0 },
  { "Malette", 0.0 },
  { "Malex", 1.0 },
  { "Maley", 0.0 },
  { "Maleya", 0.0 },
  { "Maleyah", 0.0 },
  { "Maleyna", 0.0 },
  { "Malford", 1.0 },
  { "Malgorzata", 0.0 },
  { "Malhar", 1.0 },
  { "Mali", 0.037254903 },
  { "Malia", 0.0 },
  { "Maliah", 0.0 },
  { "Maliak", 1.0 },
  { "Maliaka", 0.0 },
  { "Malialani", 0.0 },
  { "Maliana", 0.0 },
  { "Maliaya", 0.0 },
  { "Maliayah", 0.0 },
  { "Malibu", 0.0 },
  { "Malic", 1.0 },
  { "Malica", 0.0 },
  { "Malicah", 1.0 },
  { "Malicai", 1.0 },
  { "Malichai", 1.0 },
  { "Malichi", 1.0 },
  { "Malicia", 0.0 },
  { "Malick", 1.0 },
  { "Malicka", 0.0 },
  { "Malie", 0.0 },
  { "Maliea", 0.0 },
  { "Maliek", 1.0 },
  { "Malieka", 0.0 },
  { "Malieya", 0.0 },
  { "Maliha", 0.0 },
  { "Maliik", 1.0 },
  { "Malijah", 0.3472222 },
  { "Malik", 0.99389744 },
  { "Malika", 0.008395523 },
  { "Malikah", 0.079166666 },
  { "Malikai", 1.0 },
  { "Malikaih", 1.0 },
  { "Malike", 1.0 },
  { "Malikhai", 1.0 },
  { "Malikhi", 1.0 },
  { "Maliki", 1.0 },
  { "Malikia", 0.1 },
  { "Malikiah", 1.0 },
  { "Malikiya", 0.0 },
  { "Malikk", 1.0 },
  { "Malikka", 0.0 },
  { "Maliky", 1.0 },
  { "Malikye", 1.0 },
  { "Malila", 0.0 },
  { "Malillani", 0.0 },
  { "Malillany", 0.0 },
  { "Malin", 0.22853535 },
  { "Malina", 0.0 },
  { "Malinah", 0.0 },
  { "Malinalli", 0.0 },
  { "Malinda", 0.0 },
  { "Malindia", 0.0 },
  { "Maline", 0.0 },
  { "Malini", 0.0 },
  { "Maliq", 1.0 },
  { "Malique", 0.99371785 },
  { "Malira", 0.0 },
  { "Malisa", 0.0 },
  { "Malise", 0.0 },
  { "Malisha", 0.0 },
  { "Malisia", 0.0 },
  { "Malissa", 0.0 },
  { "Malissia", 0.0 },
  { "Malissie", 0.0 },
  { "Malita", 0.0 },
  { "Maliya", 0.0 },
  { "Maliyah", 0.0 },
  { "Maliyani", 0.0 },
  { "Maliyha", 0.0 },
  { "Maliza", 0.0 },
  { "Malk", 0.0 },
  { "Malka", 0.0011027791 },
  { "Malkah", 0.0 },
  { "Malke", 0.0 },
  { "Malkia", 0.0 },
  { "Malkie", 0.0 },
  { "Malkiel", 1.0 },
  { "Malkolm", 1.0 },
  { "Malky", 0.0 },
  { "Mallak", 0.0 },
  { "Mallaki", 1.0 },
  { "Mallard", 1.0 },
  { "Mallari", 0.0 },
  { "Mallarie", 0.0 },
  { "Mallary", 0.0 },
  { "Mallee", 0.0 },
  { "Mallely", 0.0 },
  { "Mallerie", 0.0 },
  { "Mallerly", 0.0 },
  { "Mallery", 0.0 },
  { "Mallex", 1.0 },
  { "Malley", 0.0 },
  { "Malli", 0.0 },
  { "Mallia", 0.0 },
  { "Mallie", 0.11780029 },
  { "Mallik", 1.0 },
  { "Mallika", 0.0 },
  { "Mallina", 0.0 },
  { "Mallisa", 0.0 },
  { "Mallissa", 0.0 },
  { "Malliyah", 0.0 },
  { "Malloree", 0.0 },
  { "Mallorey", 0.0 },
  { "Mallori", 0.0 },
  { "Mallorie", 0.0 },
  { "Mallory", 0.020083712 },
  { "Malloy", 1.0 },
  { "Mally", 0.0 },
  { "Malo", 1.0 },
  { "Malon", 0.8235294 },
  { "Malonda", 0.0 },
  { "Malone", 0.57272726 },
  { "Maloni", 0.0 },
  { "Malonie", 0.0 },
  { "Malonna", 0.0 },
  { "Malonni", 0.0 },
  { "Malonnie", 0.0 },
  { "Malora", 0.0 },
  { "Maloree", 0.0 },
  { "Malorey", 0.0 },
  { "Malori", 0.0 },
  { "Malorie", 0.0 },
  { "Malory", 0.0 },
  { "Malosi", 1.0 },
  { "Maloy", 1.0 },
  { "Malquan", 1.0 },
  { "Malta", 0.0 },
  { "Maltie", 0.0 },
  { "Malton", 1.0 },
  { "Malu", 0.0 },
  { "Maluhia", 0.0 },
  { "Malulani", 0.0 },
  { "Malva", 0.0 },
  { "Malven", 1.0 },
  { "Malvena", 0.0 },
  { "Malvene", 0.0 },
  { "Malvenia", 0.0 },
  { "Malvern", 1.0 },
  { "Malvery", 0.0 },
  { "Malvie", 0.0 },
  { "Malvika", 0.0 },
  { "Malvin", 0.995439 },
  { "Malvina", 0.0 },
  { "Malvine", 0.0 },
  { "Malwina", 0.0 },
  { "Maly", 0.0 },
  { "Malya", 0.0 },
  { "Malyah", 0.0 },
  { "Malyha", 0.0 },
  { "Malyia", 0.0 },
  { "Malyiah", 0.0 },
  { "Malyk", 1.0 },
  { "Malyka", 0.0 },
  { "Malykai", 1.0 },
  { "Malyke", 1.0 },
  { "Malyki", 1.0 },
  { "Malyn", 0.0 },
  { "Malyna", 0.0 },
  { "Malynda", 0.0 },
  { "Malynn", 0.0 },
  { "Malysa", 0.0 },
  { "Malysia", 0.0 },
  { "Malyssa", 0.0 },
  { "Malzie", 0.0 },
  { "Mama", 0.0 },
  { "Mamadi", 1.0 },
  { "Mamadou", 1.0 },
  { "Mamadu", 1.0 },
  { "Mamady", 1.0 },
  { "Mame", 0.0 },
  { "Mamediarra", 0.0 },
  { "Mami", 0.0 },
  { "Mamie", 0.0035644681 },
  { "Mammie", 0.0 },
  { "Mamon", 1.0 },
  { "Mamoon", 1.0 },
  { "Mamoru", 1.0 },
  { "Mamoudou", 1.0 },
  { "Mamoun", 1.0 },
  { "Mamta", 0.0 },
  { "Mamy", 0.0 },
  { "Mamye", 0.0 },
  { "Man", 0.72161174 },
  { "Mana", 0.11034483 },
  { "Manaal", 0.0 },
  { "Manabu", 1.0 },
  { "Manada", 0.0 },
  { "Manaf", 1.0 },
  { "Manahal", 0.0 },
  { "Manahel", 0.0 },
  { "Manahil", 0.0 },
  { "Manaia", 0.05945946 },
  { "Manal", 0.0 },
  { "Manali", 0.0 },
  { "Manami", 0.0 },
  { "Manan", 1.0 },
  { "Manar", 0.008278145 },
  { "Manard", 1.0 },
  { "Manas", 1.0 },
  { "Manasa", 0.0 },
  { "Manase", 1.0 },
  { "Manases", 1.0 },
  { "Manasi", 0.0 },
  { "Manasseh", 0.98704666 },
  { "Manasvi", 0.0 },
  { "Manasvini", 0.0 },
  { "Manaswini", 0.0 },
  { "Manaure", 1.0 },
  { "Manav", 1.0 },
  { "Manavi", 0.0 },
  { "Manaya", 0.0 },
  { "Manbir", 1.0 },
  { "Mance", 1.0 },
  { "Mancel", 1.0 },
  { "Mancie", 1.0 },
  { "Mancil", 1.0 },
  { "Mancy", 0.4 },
  { "Manda", 0.0 },
  { "Mandala", 0.0 },
  { "Mandalin", 0.0 },
  { "Mandalyn", 0.0 },
  { "Mandalynn", 0.0 },
  { "Mandana", 0.0 },
  { "Mande", 0.0 },
  { "Mandee", 0.0 },
  { "Mandeep", 0.6107056 },
  { "Mandel", 1.0 },
  { "Mandela", 1.0 },
  { "Mandell", 1.0 },
  { "Mandelyn", 0.0 },
  { "Mandey", 0.0 },
  { "Mandi", 0.0 },
  { "Mandie", 0.0 },
  { "Mandilyn", 0.0 },
  { "Mandisa", 0.0 },
  { "Mandisha", 0.0 },
  { "Mando", 1.0 },
  { "Mandolin", 0.0 },
  { "Mandolyn", 0.0 },
  { "Mandra", 0.0 },
  { "Mandre", 1.0 },
  { "Mandrel", 1.0 },
  { "Mandrell", 1.0 },
  { "Mandria", 0.0 },
  { "Mandrill", 1.0 },
  { "Mandy", 0.0032725462 },
  { "Mandye", 0.0 },
  { "Mane", 0.0 },
  { "Manee", 0.0 },
  { "Maneesh", 1.0 },
  { "Maneesha", 0.0 },
  { "Maneet", 1.0 },
  { "Maneh", 0.0 },
  { "Maneka", 0.0 },
  { "Manerva", 0.0 },
  { "Manervia", 0.0 },
  { "Manesha", 0.0 },
  { "Manessa", 0.0 },
  { "Manetta", 0.0 },
  { "Manette", 0.0 },
  { "Manferd", 1.0 },
  { "Manford", 1.0 },
  { "Manfred", 1.0 },
  { "Manfredo", 1.0 },
  { "Manfried", 1.0 },
  { "Mang", 0.71 },
  { "Manh", 1.0 },
  { "Manha", 0.0 },
  { "Manhattan", 0.071428575 },
  { "Mani", 0.9456522 },
  { "Mania", 0.0 },
  { "Maniah", 0.0 },
  { "Manica", 0.0 },
  { "Manie", 0.0 },
  { "Manifred", 1.0 },
  { "Manija", 0.0 },
  { "Manijah", 0.0 },
  { "Manijeh", 0.0 },
  { "Manik", 1.0 },
  { "Manika", 0.0 },
  { "Manila", 0.0 },
  { "Manilla", 0.0 },
  { "Manina", 0.0 },
  { "Maninder", 1.0 },
  { "Manique", 0.0 },
  { "Manisa", 0.0 },
  { "Manish", 1.0 },
  { "Manisha", 0.0 },
  { "Manit", 1.0 },
  { "Manita", 0.0 },
  { "Maniya", 0.0 },
  { "Maniyah", 0.0 },
  { "Manjari", 0.0 },
  { "Manjinder", 1.0 },
  { "Manjit", 0.0 },
  { "Manjot", 0.59745765 },
  { "Manju", 0.0 },
  { "Mankirat", 1.0 },
  { "Manley", 1.0 },
  { "Manlio", 1.0 },
  { "Manly", 1.0 },
  { "Manmeet", 0.3125 },
  { "Mann", 1.0 },
  { "Manna", 0.0 },
  { "Mannan", 1.0 },
  { "Mannat", 0.0 },
  { "Mannette", 0.0 },
  { "Mannie", 0.67250323 },
  { "Manning", 0.9771497 },
  { "Mannix", 1.0 },
  { "Mannon", 0.5833333 },
  { "Mannuel", 1.0 },
  { "Manny", 1.0 },
  { "Mano", 1.0 },
  { "Manoa", 1.0 },
  { "Manoah", 1.0 },
  { "Manogna", 0.0 },
  { "Manoj", 1.0 },
  { "Manola", 0.0 },
  { "Manolis", 1.0 },
  { "Manolito", 1.0 },
  { "Manolo", 1.0 },
  { "Manon", 0.0064935065 },
  { "Manouchka", 0.0 },
  { "Manoy", 0.0 },
  { "Manpreet", 0.37523106 },
  { "Manraj", 1.0 },
  { "Manreet", 0.0 },
  { "Manrique", 1.0 },
  { "Manroop", 0.0 },
  { "Mansa", 1.0 },
  { "Mansel", 1.0 },
  { "Mansell", 1.0 },
  { "Mansfield", 1.0 },
  { "Mansha", 0.0 },
  { "Manshi", 0.0 },
  { "Mansi", 0.0 },
  { "Mansirat", 0.0 },
  { "Manson", 1.0 },
  { "Mansoor", 1.0 },
  { "Mansour", 1.0 },
  { "Mansur", 1.0 },
  { "Mantas", 1.0 },
  { "Mantasha", 0.0 },
  { "Mantej", 1.0 },
  { "Manthan", 1.0 },
  { "Manton", 1.0 },
  { "Mantra", 1.0 },
  { "Manu", 0.99065423 },
  { "Manual", 1.0 },
  { "Manuel", 0.9937002 },
  { "Manuela", 0.0033852404 },
  { "Manuelita", 0.0 },
  { "Manuelito", 1.0 },
  { "Manuell", 1.0 },
  { "Manuella", 0.0 },
  { "Manuelle", 1.0 },
  { "Manul", 1.0 },
  { "Manus", 1.0 },
  { "Manushri", 0.0 },
  { "Manveer", 0.9431818 },
  { "Manvel", 1.0 },
  { "Manvi", 0.0 },
  { "Manvik", 1.0 },
  { "Manvil", 1.0 },
  { "Manville", 1.0 },
  { "Manvir", 0.885 },
  { "Manvith", 1.0 },
  { "Manvitha", 0.0 },
  { "Manwell", 1.0 },
  { "Many", 0.0 },
  { "Manya", 0.0 },
  { "Manyah", 0.0 },
  { "Manzell", 1.0 },
  { "Manzie", 1.0 },
  { "Mao", 0.033898305 },
  { "Maomi", 0.0 },
  { "Maor", 1.0 },
  { "Maori", 0.0 },
  { "Maple", 0.0027012427 },
  { "Mapuana", 0.0 },
  { "Maquan", 1.0 },
  { "Maquel", 0.0 },
  { "Maquela", 0.0 },
  { "Maquetta", 0.0 },
  { "Maquisha", 0.0 },
  { "Maquita", 0.0 },
  { "Mar", 0.2593583 },
  { "Mara", 0.0008127603 },
  { "Maraam", 0.0 },
  { "Marabel", 0.0 },
  { "Marabella", 0.0 },
  { "Marabelle", 0.0 },
  { "Marabeth", 0.0 },
  { "Maradee", 0.0 },
  { "Maradith", 0.0 },
  { "Marae", 0.0 },
  { "Maragaret", 0.0 },
  { "Maragret", 0.0 },
  { "Marah", 0.0 },
  { "Marai", 0.0 },
  { "Maraia", 0.0 },
  { "Maraiah", 0.0 },
  { "Maraih", 0.0 },
  { "Maraina", 0.0 },
  { "Maraiya", 0.0 },
  { "Maraja", 0.0 },
  { "Marajade", 0.0 },
  { "Maraki", 0.0 },
  { "Maral", 0.0 },
  { "Maralee", 0.0 },
  { "Maralene", 0.0 },
  { "Marali", 0.0 },
  { "Maralou", 0.0 },
  { "Maralyn", 0.0 },
  { "Maralynn", 0.0 },
  { "Maram", 0.0 },
  { "Maran", 0.0 },
  { "Maranatha", 0.0 },
  { "Maranda", 0.0 },
  { "Marandia", 0.0 },
  { "Marangely", 0.0 },
  { "Marasia", 0.0 },
  { "Marat", 1.0 },
  { "Maratha", 0.0 },
  { "Maravene", 0.0 },
  { "Maray", 0.0 },
  { "Maraya", 0.0 },
  { "Marayah", 0.0 },
  { "Marayla", 0.0 },
  { "Marayna", 0.0 },
  { "Marba", 0.0 },
  { "Marbel", 0.0 },
  { "Marbeli", 0.0 },
  { "Marbella", 0.0 },
  { "Marbely", 0.0 },
  { "Marbeth", 0.0 },
  { "Marbeya", 0.0 },
  { "Marbin", 1.0 },
  { "Marc", 0.9961784 },
  { "Marca", 0.0 },
  { "Marcail", 0.0 },
  { "Marcal", 1.0 },
  { "Marcale", 1.0 },
  { "Marcalene", 0.0 },
  { "Marcandre", 1.0 },
  { "Marcandrew", 1.0 },
  { "Marcangelo", 1.0 },
  { "Marcanthony", 1.0 },
  { "Marcas", 1.0 },
  { "Marcasia", 0.0 },
  { "Marcayla", 0.0 },
  { "Marce", 0.38321167 },
  { "Marcea", 0.0 },
  { "Marceau", 1.0 },
  { "Marceda", 0.0 },
  { "Marcedes", 0.01607717 },
  { "Marcedez", 0.0 },
  { "Marcee", 0.0 },
  { "Marceen", 0.0 },
  { "Marceia", 0.0 },
  { "Marceil", 0.0 },
  { "Marceille", 0.0 },
  { "Marcel", 0.94468606 },
  { "Marcela", 0.0 },
  { "Marcelaine", 0.0 },
  { "Marcele", 0.0 },
  { "Marceleen", 0.0 },
  { "Marcelena", 0.0 },
  { "Marcelene", 0.0 },
  { "Marcelia", 0.0 },
  { "Marcelin", 1.0 },
  { "Marcelina", 0.0 },
  { "Marceline", 0.0 },
  { "Marcelino", 1.0 },
  { "Marcelis", 1.0 },
  { "Marcelius", 1.0 },
  { "Marcell", 0.79225093 },
  { "Marcella", 0.0034871665 },
  { "Marcellas", 1.0 },
  { "Marcelle", 0.08034632 },
  { "Marcellene", 0.0 },
  { "Marcelles", 1.0 },
  { "Marcellia", 0.0 },
  { "Marcellina", 0.0 },
  { "Marcelline", 0.0 },
  { "Marcellino", 1.0 },
  { "Marcellis", 1.0 },
  { "Marcellius", 1.0 },
  { "Marcello", 0.99804115 },
  { "Marcellos", 1.0 },
  { "Marcellous", 1.0 },
  { "Marcellus", 0.999309 },
  { "Marcellyn", 0.0 },
  { "Marcelo", 1.0 },
  { "Marcelous", 1.0 },
  { "Marcelus", 1.0 },
  { "Marcelyn", 0.0 },
  { "Marcena", 0.0 },
  { "Marcene", 0.0 },
  { "Marcenia", 0.0 },
  { "Marcese", 1.0 },
  { "Marcetta", 0.0 },
  { "Marcey", 0.0 },
  { "March", 0.6048387 },
  { "Marcha", 0.0 },
  { "Marchae", 0.0 },
  { "Marchand", 0.41666666 },
  { "Marchant", 1.0 },
  { "Marche", 0.016746411 },
  { "Marchel", 0.43410853 },
  { "Marchele", 0.0 },
  { "Marchell", 0.0102739725 },
  { "Marchella", 0.085365854 },
  { "Marchelle", 0.0 },
  { "Marchello", 1.0 },
  { "Marchelo", 1.0 },
  { "Marcheta", 0.0 },
  { "Marchetta", 0.0 },
  { "Marchia", 0.0 },
  { "Marchie", 0.0 },
  { "Marchita", 0.0 },
  { "Marci", 0.00079669734 },
  { "Marcia", 0.0021313 },
  { "Marcial", 1.0 },
  { "Marciana", 0.0 },
  { "Marciann", 0.0 },
  { "Marcianna", 0.0 },
  { "Marcianne", 0.0 },
  { "Marciano", 1.0 },
  { "Marcie", 0.0008482318 },
  { "Marciel", 0.056179777 },
  { "Marciela", 0.0 },
  { "Marciella", 0.0 },
  { "Marcile", 0.0 },
  { "Marcilene", 0.0 },
  { "Marcilla", 0.0 },
  { "Marcille", 0.0 },
  { "Marcin", 1.0 },
  { "Marcina", 0.0 },
  { "Marcinda", 0.0 },
  { "Marcine", 0.0 },
  { "Marcio", 1.0 },
  { "Marcion", 1.0 },
  { "Marcis", 1.0 },
  { "Marcisha", 0.0 },
  { "Marcita", 0.0 },
  { "Marcius", 1.0 },
  { "Marcjacob", 1.0 },
  { "Marck", 1.0 },
  { "Marckel", 1.0 },
  { "Marckos", 1.0 },
  { "Marckus", 1.0 },
  { "Marclene", 0.0 },
  { "Marco", 0.9951066 },
  { "Marcoa", 1.0 },
  { "Marcoantonio", 1.0 },
  { "Marcon", 1.0 },
  { "Marcopolo", 1.0 },
  { "Marcos", 0.9966192 },
  { "Marcous", 1.0 },
  { "Marcquel", 1.0 },
  { "Marcques", 1.0 },
  { "Marcquez", 1.0 },
  { "Marcquis", 1.0 },
  { "Marcquise", 1.0 },
  { "Marctavious", 1.0 },
  { "Marcua", 1.0 },
  { "Marcum", 1.0 },
  { "Marcus", 0.9941654 },
  { "Marcusanthony", 1.0 },
  { "Marcuse", 1.0 },
  { "Marcusjames", 1.0 },
  { "Marcuss", 1.0 },
  { "Marcuz", 1.0 },
  { "Marcy", 0.011779321 },
  { "Marcyanna", 0.0 },
  { "Marcys", 1.0 },
  { "Marda", 0.0 },
  { "Mardarius", 1.0 },
  { "Mardean", 0.0 },
  { "Mardee", 0.0 },
  { "Mardeen", 0.0 },
  { "Mardel", 0.0 },
  { "Mardell", 0.041519433 },
  { "Mardella", 0.0 },
  { "Mardelle", 0.0 },
  { "Marden", 1.0 },
  { "Mardena", 0.0 },
  { "Mardene", 0.0 },
  { "Mardi", 0.0068649887 },
  { "Mardie", 0.07462686 },
  { "Mardina", 0.0 },
  { "Mardine", 0.0 },
  { "Mardis", 1.0 },
  { "Mardith", 0.0 },
  { "Mardochee", 1.0 },
  { "Mardy", 0.76842105 },
  { "Mare", 1.0 },
  { "Marea", 0.0 },
  { "Mareah", 0.0 },
  { "Mareck", 1.0 },
  { "Mareco", 1.0 },
  { "Mareda", 0.0 },
  { "Maredith", 0.0 },
  { "Maree", 0.0 },
  { "Mareen", 0.0 },
  { "Mareena", 0.0 },
  { "Mareesa", 0.0 },
  { "Mareesha", 0.0 },
  { "Mareike", 0.0 },
  { "Mareily", 0.0 },
  { "Mareisha", 0.0 },
  { "Marek", 0.9980151 },
  { "Mareka", 0.0 },
  { "Marel", 0.0 },
  { "Marelene", 0.0 },
  { "Mareli", 0.0 },
  { "Marelie", 0.0 },
  { "Marelin", 0.0 },
  { "Marelis", 0.0 },
  { "Marell", 1.0 },
  { "Marella", 0.0 },
  { "Marelle", 0.0 },
  { "Marelli", 0.0 },
  { "Marelly", 0.0 },
  { "Marely", 0.0 },
  { "Marelyn", 0.0 },
  { "Marelys", 0.0 },
  { "Mareme", 0.0 },
  { "Maren", 0.0 },
  { "Marena", 0.0 },
  { "Marenda", 0.0 },
  { "Marene", 0.0 },
  { "Marenna", 0.0 },
  { "Mareo", 1.0 },
  { "Mareon", 1.0 },
  { "Maresa", 0.0 },
  { "Maresha", 0.0 },
  { "Mareshah", 0.0 },
  { "Maressa", 0.0 },
  { "Maressia", 0.0 },
  { "Maret", 0.0 },
  { "Mareta", 0.0 },
  { "Maretha", 0.0 },
  { "Maretta", 0.0 },
  { "Marette", 0.0 },
  { "Marey", 0.0 },
  { "Mareya", 0.0 },
  { "Mareyah", 0.0 },
  { "Marfa", 0.0 },
  { "Marfil", 0.0 },
  { "Marg", 0.0 },
  { "Marga", 0.0 },
  { "Margaet", 0.0 },
  { "Margalit", 0.0 },
  { "Margan", 0.0 },
  { "Margarat", 0.0 },
  { "Margare", 0.0 },
  { "Margaree", 0.0 },
  { "Margaret", 0.0029988838 },
  { "Margareta", 0.0 },
  { "Margaretann", 0.0 },
  { "Margaretanne", 0.0 },
  { "Margarete", 0.0 },
  { "Margareth", 0.0 },
  { "Margaretha", 0.0 },
  { "Margarethe", 0.0 },
  { "Margaretjo", 0.0 },
  { "Margaretmary", 0.0 },
  { "Margarett", 0.0 },
  { "Margaretta", 0.0 },
  { "Margarette", 0.0 },
  { "Margarida", 0.0 },
  { "Margarie", 0.0 },
  { "Margarine", 0.0 },
  { "Margarit", 0.0 },
  { "Margarita", 0.004946477 },
  { "Margarite", 0.0 },
  { "Margarito", 1.0 },
  { "Margaritte", 0.0 },
  { "Margaro", 1.0 },
  { "Margart", 0.0 },
  { "Margaruite", 0.0 },
  { "Margary", 0.0 },
  { "Margaurite", 0.0 },
  { "Margaux", 0.0 },
  { "Marge", 0.0 },
  { "Margean", 0.0 },
  { "Margearet", 0.0 },
  { "Margeart", 0.0 },
  { "Margeaux", 0.0 },
  { "Margee", 0.0 },
  { "Margel", 0.0 },
  { "Margelene", 0.0 },
  { "Margena", 0.0 },
  { "Margene", 0.0029550828 },
  { "Margeree", 0.0 },
  { "Margeret", 0.0 },
  { "Margerete", 0.0 },
  { "Margerett", 0.0 },
  { "Margerette", 0.0 },
  { "Margerie", 0.0 },
  { "Margerine", 0.0 },
  { "Margerite", 0.0 },
  { "Margert", 0.0 },
  { "Margery", 0.0 },
  { "Marget", 0.0 },
  { "Margett", 0.0 },
  { "Margetta", 0.0 },
  { "Margette", 0.0 },
  { "Margeurite", 0.0 },
  { "Margey", 0.0 },
  { "Marggie", 0.0 },
  { "Margherita", 0.0 },
  { "Margherite", 0.0 },
  { "Margi", 0.0 },
  { "Margia", 0.0 },
  { "Margie", 0.0037936552 },
  { "Margil", 0.0 },
  { "Margine", 0.0 },
  { "Margit", 0.0 },
  { "Margita", 0.0 },
  { "Margo", 0.0014495843 },
  { "Margorie", 0.0 },
  { "Margory", 0.0 },
  { "Margot", 0.0 },
  { "Margoth", 0.0 },
  { "Margrate", 0.0 },
  { "Margreat", 0.0 },
  { "Margree", 0.0 },
  { "Margreet", 0.0 },
  { "Margret", 0.0 },
  { "Margreta", 0.0 },
  { "Margrete", 0.0 },
  { "Margretha", 0.0 },
  { "Margrethe", 0.0 },
  { "Margrett", 0.0 },
  { "Margretta", 0.0 },
  { "Margrette", 0.0 },
  { "Margrie", 0.0 },
  { "Margrit", 0.0 },
  { "Marguarite", 0.0 },
  { "Marguel", 1.0 },
  { "Marguerete", 0.0 },
  { "Marguerette", 0.0 },
  { "Margueriete", 0.0 },
  { "Margueriette", 0.0 },
  { "Marguerit", 0.0 },
  { "Marguerita", 0.0 },
  { "Marguerite", 0.0010811737 },
  { "Margueritt", 0.0 },
  { "Margueritta", 0.0 },
  { "Margueritte", 0.0 },
  { "Marguery", 0.0 },
  { "Margues", 1.0 },
  { "Marguetta", 0.0 },
  { "Marguette", 1.0 },
  { "Marguis", 1.0 },
  { "Marguise", 1.0 },
  { "Marguita", 0.0 },
  { "Marguitta", 0.0 },
  { "Margulia", 0.0 },
  { "Marguree", 0.0 },
  { "Marguret", 0.0 },
  { "Margurete", 0.0 },
  { "Margurette", 0.0 },
  { "Marguriete", 0.0 },
  { "Marguriette", 0.0 },
  { "Margurita", 0.0 },
  { "Margurite", 0.0 },
  { "Marguritte", 0.0 },
  { "Margus", 1.0 },
  { "Margy", 0.0 },
  { "Marhia", 0.0 },
  { "Marhonda", 0.0 },
  { "Marhsall", 1.0 },
  { "Marhta", 0.0 },
  { "Mari", 0.00064880296 },
  { "Maria", 0.0077635646 },
  { "Mariaalejandra", 0.0 },
  { "Mariaangela", 0.0 },
  { "Mariaangelica", 0.0 },
  { "Mariaann", 0.0 },
  { "Mariaantonia", 0.0 },
  { "Mariabelen", 0.0 },
  { "Mariabella", 0.0 },
  { "Mariacamila", 0.0 },
  { "Mariacecilia", 0.0 },
  { "Mariaceleste", 0.0 },
  { "Mariachristina", 0.0 },
  { "Mariaclara", 0.0 },
  { "Mariacristina", 0.0 },
  { "Mariade", 0.0 },
  { "Mariadejesus", 0.0 },
  { "Mariadel", 0.0 },
  { "Mariadelaluz", 0.0 },
  { "Mariadelcarmen", 0.0 },
  { "Mariadelos", 0.0 },
  { "Mariadelosang", 0.0 },
  { "Mariadelosangel", 0.0 },
  { "Mariadelourdes", 0.0 },
  { "Mariadelrosari", 0.0 },
  { "Mariadelrosario", 0.0 },
  { "Mariae", 0.0 },
  { "Mariaeduard", 0.0 },
  { "Mariaeduarda", 0.0 },
  { "Mariaelena", 0.0 },
  { "Mariaelisa", 0.0 },
  { "Mariaelizabeth", 0.0 },
  { "Mariaemilia", 0.0 },
  { "Mariaesther", 0.0 },
  { "Mariafernanda", 0.0 },
  { "Mariagabriela", 0.0 },
  { "Mariagrazia", 0.0 },
  { "Mariaguadalupe", 0.0 },
  { "Mariah", 0.0017319705 },
  { "Mariaha", 0.0 },
  { "Mariahann", 0.0 },
  { "Mariahlynn", 0.0 },
  { "Mariahna", 0.0 },
  { "Mariaines", 0.0 },
  { "Mariaisabel", 0.0 },
  { "Mariaisabella", 0.0 },
  { "Mariajose", 0.0 },
  { "Mariajulia", 0.0 },
  { "Marial", 0.0 },
  { "Marialaina", 0.0 },
  { "Marialana", 0.0 },
  { "Marialaura", 0.0 },
  { "Marialena", 0.0 },
  { "Mariali", 0.0 },
  { "Marialice", 0.0 },
  { "Marialicia", 0.0 },
  { "Marialis", 0.0 },
  { "Marialisa", 0.0 },
  { "Marializ", 0.0 },
  { "Marialuisa", 0.0 },
  { "Marialuiza", 0.0 },
  { "Marialy", 0.0 },
  { "Marialyce", 0.0 },
  { "Marialys", 0.0 },
  { "Mariam", 0.00036613943 },
  { "Mariama", 0.0 },
  { "Mariamawit", 0.0 },
  { "Mariame", 0.0 },
  { "Mariamne", 0.0 },
  { "Mariamu", 0.0 },
  { "Marian", 0.009428562 },
  { "Mariana", 0.0024260068 },
  { "Marianah", 0.0 },
  { "Mariane", 0.0 },
  { "Marianela", 0.0 },
  { "Marianella", 0.0 },
  { "Marianely", 0.0 },
  { "Mariangel", 0.0 },
  { "Mariangela", 0.0 },
  { "Mariangeles", 0.0 },
  { "Mariangelis", 0.0 },
  { "Mariangeliz", 0.0 },
  { "Mariangely", 0.0 },
  { "Mariani", 0.0 },
  { "Marianita", 0.0 },
  { "Mariann", 0.0 },
  { "Marianna", 0.0 },
  { "Mariannah", 0.0 },
  { "Marianne", 0.0007568386 },
  { "Marianny", 0.0 },
  { "Mariano", 1.0 },
  { "Marianthi", 0.0 },
  { "Mariany", 0.0 },
  { "Mariapaula", 0.0 },
  { "Mariapaz", 0.0 },
  { "Mariarosa", 0.0 },
  { "Mariasha", 0.0 },
  { "Mariasofia", 0.0 },
  { "Mariateresa", 0.0 },
  { "Mariatheresa", 0.0 },
  { "Mariauna", 0.0 },
  { "Mariavalentina", 0.0 },
  { "Mariavictoria", 0.0 },
  { "Mariaya", 0.0 },
  { "Maribel", 0.0077347606 },
  { "Maribela", 0.0 },
  { "Maribell", 0.0 },
  { "Maribella", 0.0 },
  { "Maribelle", 0.0 },
  { "Maribeth", 0.0 },
  { "Maribi", 0.0 },
  { "Mariby", 0.0 },
  { "Maric", 1.0 },
  { "Marica", 0.0 },
  { "Maricar", 0.0 },
  { "Maricarmen", 0.0 },
  { "Marice", 0.6049069 },
  { "Maricel", 0.0 },
  { "Maricela", 0.0044324836 },
  { "Maricelda", 0.0 },
  { "Maricelis", 0.0 },
  { "Maricella", 0.0 },
  { "Maricely", 0.0 },
  { "Maricelys", 0.0 },
  { "Maricha", 0.0 },
  { "Marichal", 1.0 },
  { "Marichelle", 0.0 },
  { "Marichuy", 0.0 },
  { "Maricia", 0.0 },
  { "Maricielo", 0.0 },
  { "Maricio", 1.0 },
  { "Marick", 1.0 },
  { "Maricka", 0.0 },
  { "Mariclare", 0.0 },
  { "Marico", 1.0 },
  { "Maricris", 0.0 },
  { "Maricrus", 0.0 },
  { "Maricruz", 0.0 },
  { "Maricsa", 0.0 },
  { "Maricus", 1.0 },
  { "Maricza", 0.0 },
  { "Marid", 1.0 },
  { "Marida", 0.0 },
  { "Maridean", 0.0 },
  { "Maridee", 0.0 },
  { "Maridel", 0.0 },
  { "Maridell", 0.0 },
  { "Marideth", 0.0 },
  { "Maridith", 0.0 },
  { "Marie", 0.004080874 },
  { "Mariea", 0.0 },
  { "Marieana", 0.0 },
  { "Marieange", 0.0 },
  { "Marieann", 0.0 },
  { "Marieanna", 0.0 },
  { "Marieanne", 0.0 },
  { "Mariechristine", 0.0 },
  { "Marieclaire", 0.0 },
  { "Marieda", 0.0 },
  { "Marieelena", 0.0 },
  { "Mariejeanne", 0.0 },
  { "Marieka", 0.0 },
  { "Marieke", 0.0 },
  { "Mariel", 0.018412698 },
  { "Mariela", 0.0007769253 },
  { "Mariele", 0.0 },
  { "Marielena", 0.0 },
  { "Marieli", 0.0 },
  { "Marielis", 0.0 },
  { "Marielisa", 0.0 },
  { "Marieliz", 0.0 },
  { "Mariell", 0.0 },
  { "Mariella", 0.0 },
  { "Marielle", 0.0 },
  { "Mariellen", 0.0 },
  { "Marielly", 0.0 },
  { "Marielos", 0.0 },
  { "Marielouise", 0.0 },
  { "Mariely", 0.0 },
  { "Marielys", 0.0 },
  { "Mariem", 0.0 },
  { "Marieme", 0.0 },
  { "Marien", 0.0 },
  { "Mariena", 0.0 },
  { "Mariene", 0.0 },
  { "Marienne", 0.0 },
  { "Marieo", 1.0 },
  { "Marierose", 0.0 },
  { "Mariesa", 0.0 },
  { "Mariesha", 0.0 },
  { "Mariessa", 0.0 },
  { "Marieta", 0.0 },
  { "Marietha", 0.0 },
  { "Marietherese", 0.0 },
  { "Marietou", 0.0 },
  { "Marietta", 0.0 },
  { "Mariette", 0.0 },
  { "Marifer", 0.0 },
  { "Marifrances", 0.0 },
  { "Marigail", 0.0 },
  { "Marigene", 0.0 },
  { "Marigny", 0.0 },
  { "Marigold", 0.0 },
  { "Marigrace", 0.0 },
  { "Mariha", 0.0 },
  { "Marihanna", 0.0 },
  { "Marihelen", 0.0 },
  { "Marija", 0.0 },
  { "Marijah", 0.0 },
  { "Marijana", 0.0 },
  { "Marijane", 0.0 },
  { "Marijayne", 0.0 },
  { "Marijean", 0.0 },
  { "Marijke", 0.0 },
  { "Marijo", 0.0 },
  { "Marijon", 0.0 },
  { "Marijose", 0.0 },
  { "Marijuana", 0.0 },
  { "Marik", 1.0 },
  { "Marika", 0.0 },
  { "Marikate", 0.0 },
  { "Marikay", 0.0 },
  { "Marike", 0.0 },
  { "Mariko", 0.00477327 },
  { "Marilda", 0.0 },
  { "Marilea", 0.0 },
  { "Marilee", 0.0 },
  { "Marileen", 0.0 },
  { "Marilena", 0.0 },
  { "Marilene", 0.0 },
  { "Marileysis", 0.0 },
  { "Marili", 0.0 },
  { "Marilia", 0.0 },
  { "Marilin", 0.0 },
  { "Marilina", 0.0 },
  { "Marilinda", 0.0 },
  { "Marilis", 0.0 },
  { "Marilisa", 0.0 },
  { "Mariliz", 0.0 },
  { "Marilla", 0.0 },
  { "Marillany", 0.0 },
  { "Marillyn", 0.0 },
  { "Marilon", 0.0 },
  { "Marilou", 0.0 },
  { "Marilouise", 0.0 },
  { "Marilu", 0.0 },
  { "Mariluz", 0.0 },
  { "Marily", 0.0 },
  { "Marilyn", 0.0028750356 },
  { "Marilyne", 0.0 },
  { "Marilynn", 0.0 },
  { "Marilynne", 0.0 },
  { "Marim", 0.0 },
  { "Marimar", 0.0 },
  { "Marin", 0.14897995 },
  { "Marina", 0.0014351644 },
  { "Marinah", 0.0 },
  { "Marinda", 0.0 },
  { "Marine", 0.0062370063 },
  { "Marinee", 0.0 },
  { "Marinel", 0.0 },
  { "Marinell", 0.0 },
  { "Marinella", 0.0 },
  { "Marinelle", 0.0 },
  { "Marines", 0.0 },
  { "Marinette", 0.0 },
  { "Marinez", 0.0 },
  { "Marini", 0.0 },
  { "Marinn", 0.0 },
  { "Marinna", 0.0 },
  { "Marino", 1.0 },
  { "Marinus", 1.0 },
  { "Mario", 0.9924151 },
  { "Marioalberto", 1.0 },
  { "Mariola", 0.0 },
  { "Mariolita", 0.0 },
  { "Marion", 0.2765762 },
  { "Mariona", 0.0 },
  { "Marionette", 0.0 },
  { "Marioni", 1.0 },
  { "Marionna", 0.0 },
  { "Marios", 1.0 },
  { "Marious", 1.0 },
  { "Maripat", 0.0 },
  { "Maripaz", 0.0 },
  { "Mariposa", 0.0 },
  { "Marique", 1.0 },
  { "Mariques", 1.0 },
  { "Mariquita", 0.0 },
  { "Marirose", 0.0 },
  { "Mariruth", 0.0 },
  { "Maris", 0.08902691 },
  { "Marisa", 0.0013667142 },
  { "Marisabel", 0.0 },
  { "Marisah", 0.0 },
  { "Marisal", 0.0 },
  { "Marise", 0.0 },
  { "Marisel", 0.0 },
  { "Marisela", 0.0028965243 },
  { "Mariselda", 0.0 },
  { "Marisella", 0.0 },
  { "Marisely", 0.0 },
  { "Marish", 0.0 },
  { "Marisha", 0.0 },
  { "Marishia", 0.0 },
  { "Marishka", 0.0 },
  { "Marisia", 0.0 },
  { "Mariska", 0.0 },
  { "Marisleysi", 0.0 },
  { "Marisleysis", 0.0 },
  { "Marisol", 0.009707242 },
  { "Marison", 0.0 },
  { "Mariss", 0.0 },
  { "Marissa", 0.0022685071 },
  { "Marissah", 0.0 },
  { "Marissia", 0.0 },
  { "Marista", 0.0 },
  { "Marisue", 0.0 },
  { "Marit", 0.0 },
  { "Marita", 0.0 },
  { "Maritere", 0.0 },
  { "Marites", 0.0 },
  { "Maritess", 0.0 },
  { "Maritha", 0.0 },
  { "Marithza", 0.0 },
  { "Maritsa", 0.0 },
  { "Maritssa", 0.0 },
  { "Maritta", 0.0 },
  { "Maritza", 0.0035414891 },
  { "Maritzabel", 0.0 },
  { "Marium", 0.0 },
  { "Marius", 1.0 },
  { "Mariusz", 1.0 },
  { "Mariuxi", 0.0 },
  { "Marivel", 0.0 },
  { "Marivell", 0.0 },
  { "Marivella", 0.0 },
  { "Marivelle", 0.0 },
  { "Marivi", 0.0 },
  { "Marivic", 0.0 },
  { "Marivy", 0.0 },
  { "Marixa", 0.0 },
  { "Marixsa", 0.0 },
  { "Marixza", 0.0 },
  { "Mariya", 0.0 },
  { "Mariyah", 0.0 },
  { "Mariyam", 0.0 },
  { "Mariyan", 0.0 },
  { "Mariyana", 0.0 },
  { "Mariyani", 0.0 },
  { "Mariyanna", 0.0 },
  { "Mariyha", 0.0 },
  { "Mariyon", 1.0 },
  { "Mariyonna", 0.0 },
  { "Mariza", 0.0 },
  { "Marizela", 0.0 },
  { "Marizol", 0.0 },
  { "Marizza", 0.0 },
  { "Marj", 0.0 },
  { "Marja", 0.0 },
  { "Marjaan", 0.0 },
  { "Marjae", 0.0 },
  { "Marjan", 0.0 },
  { "Marjane", 0.0 },
  { "Marjani", 0.0 },
  { "Marjarie", 0.0 },
  { "Marje", 0.0 },
  { "Marjean", 0.0 },
  { "Marjeanne", 0.0 },
  { "Marji", 0.0 },
  { "Marjie", 0.0 },
  { "Marjo", 0.0 },
  { "Marjoe", 1.0 },
  { "Marjolaine", 0.0 },
  { "Marjon", 0.2027027 },
  { "Marjona", 0.0 },
  { "Marjoree", 0.0 },
  { "Marjori", 0.0 },
  { "Marjoria", 0.0 },
  { "Marjorie", 0.0021794087 },
  { "Marjorine", 0.0 },
  { "Marjory", 0.0 },
  { "Mark", 0.99668 },
  { "Marka", 0.01968504 },
  { "Markael", 1.0 },
  { "Markai", 0.78723407 },
  { "Markail", 1.0 },
  { "Markaila", 0.0 },
  { "Markal", 1.0 },
  { "Markala", 0.0 },
  { "Markale", 1.0 },
  { "Markallen", 1.0 },
  { "Markan", 0.0 },
  { "Markandrew", 1.0 },
  { "Markangelo", 1.0 },
  { "Markanthony", 1.0 },
  { "Markas", 1.0 },
  { "Markasia", 0.0 },
  { "Markavion", 1.0 },
  { "Markavious", 1.0 },
  { "Markavius", 1.0 },
  { "Markay", 0.0 },
  { "Markaya", 0.0 },
  { "Markayden", 1.0 },
  { "Markayla", 0.0 },
  { "Markaylah", 0.0 },
  { "Markaysha", 0.0 },
  { "Markaysia", 0.0 },
  { "Markchristopher", 1.0 },
  { "Markco", 1.0 },
  { "Markcus", 1.0 },
  { "Markdaniel", 1.0 },
  { "Markdavid", 1.0 },
  { "Marke", 0.9695122 },
  { "Markea", 0.0 },
  { "Markeal", 1.0 },
  { "Markeas", 1.0 },
  { "Markease", 1.0 },
  { "Markece", 1.0 },
  { "Markecia", 0.0 },
  { "Markeda", 0.0 },
  { "Markee", 0.8619824 },
  { "Markeece", 1.0 },
  { "Markeeda", 0.0 },
  { "Markeem", 1.0 },
  { "Markees", 1.0 },
  { "Markeese", 1.0 },
  { "Markeesha", 0.0 },
  { "Markeeta", 0.0 },
  { "Markeia", 0.0 },
  { "Markeice", 1.0 },
  { "Markeida", 0.0 },
  { "Markeil", 1.0 },
  { "Markeis", 1.0 },
  { "Markeise", 1.0 },
  { "Markeisha", 0.0 },
  { "Markeita", 0.0 },
  { "Markeith", 1.0 },
  { "Markeitha", 0.0 },
  { "Markel", 0.9712521 },
  { "Markela", 0.0 },
  { "Markele", 1.0 },
  { "Markell", 0.94905794 },
  { "Markella", 0.0 },
  { "Markelle", 0.56747407 },
  { "Marken", 1.0 },
  { "Markena", 0.0 },
  { "Markenzie", 0.0 },
  { "Markeon", 1.0 },
  { "Markera", 0.0 },
  { "Markeria", 0.0 },
  { "Markes", 1.0 },
  { "Markesa", 0.0 },
  { "Markese", 1.0 },
  { "Markesha", 0.0 },
  { "Markeshia", 0.0 },
  { "Markesia", 0.0 },
  { "Markess", 1.0 },
  { "Markest", 1.0 },
  { "Marketa", 0.0 },
  { "Marketia", 0.0 },
  { "Marketta", 0.0 },
  { "Markette", 0.0 },
  { "Markeveon", 1.0 },
  { "Markevia", 0.0 },
  { "Markevion", 1.0 },
  { "Markevious", 1.0 },
  { "Markevis", 1.0 },
  { "Markevius", 1.0 },
  { "Markevus", 1.0 },
  { "Markey", 0.9822695 },
  { "Markeya", 0.0 },
  { "Markeyda", 0.0 },
  { "Markeyia", 0.0 },
  { "Markeyla", 0.0 },
  { "Markeys", 1.0 },
  { "Markeysha", 0.0 },
  { "Markeyta", 0.0 },
  { "Markez", 1.0 },
  { "Markham", 1.0 },
  { "Markhi", 1.0 },
  { "Markhia", 0.0 },
  { "Marki", 0.07379135 },
  { "Markia", 0.0 },
  { "Markiah", 0.0 },
  { "Markian", 1.0 },
  { "Markice", 1.0 },
  { "Markida", 0.0 },
  { "Markie", 0.17772779 },
  { "Markiea", 0.0 },
  { "Markiece", 1.0 },
  { "Markiee", 1.0 },
  { "Markiel", 1.0 },
  { "Markies", 1.0 },
  { "Markiese", 1.0 },
  { "Markiesha", 0.0 },
  { "Markiest", 1.0 },
  { "Markieta", 0.0 },
  { "Markieth", 1.0 },
  { "Markila", 0.0 },
  { "Markina", 0.0 },
  { "Marking", 1.0 },
  { "Markira", 0.0 },
  { "Markis", 1.0 },
  { "Markisa", 0.0 },
  { "Markise", 1.0 },
  { "Markisha", 0.0 },
  { "Markishia", 0.0 },
  { "Markiss", 1.0 },
  { "Markita", 0.0 },
  { "Markitta", 0.0 },
  { "Markius", 1.0 },
  { "Markiya", 0.0 },
  { "Markiyah", 0.0 },
  { "Markjoseph", 1.0 },
  { "Markk", 1.0 },
  { "Markkevin", 1.0 },
  { "Markki", 0.0 },
  { "Markkus", 1.0 },
  { "Markle", 1.0 },
  { "Markley", 1.0 },
  { "Marko", 1.0 },
  { "Markon", 1.0 },
  { "Markos", 1.0 },
  { "Markous", 1.0 },
  { "Markquan", 1.0 },
  { "Markquis", 1.0 },
  { "Markquise", 1.0 },
  { "Markryan", 1.0 },
  { "Marks", 1.0 },
  { "Markum", 1.0 },
  { "Markus", 1.0 },
  { "Markuz", 1.0 },
  { "Markwan", 1.0 },
  { "Markwilliam", 1.0 },
  { "Markwon", 1.0 },
  { "Marky", 1.0 },
  { "Markya", 0.0 },
  { "Markyia", 0.0 },
  { "Markyla", 0.0 },
  { "Markysha", 0.0 },
  { "Marl", 1.0 },
  { "Marla", 0.0009968259 },
  { "Marlaena", 0.0 },
  { "Marlaina", 0.0 },
  { "Marlaine", 0.0 },
  { "Marlan", 0.9884202 },
  { "Marlana", 0.0 },
  { "Marland", 1.0 },
  { "Marlanda", 0.0 },
  { "Marlando", 1.0 },
  { "Marlane", 0.0 },
  { "Marlania", 0.0 },
  { "Marlanna", 0.0 },
  { "Marlasia", 0.0 },
  { "Marlaya", 0.0 },
  { "Marlayah", 0.0 },
  { "Marlayna", 0.0 },
  { "Marlayne", 0.0 },
  { "Marlaysha", 0.0 },
  { "Marlayshia", 0.0 },
  { "Marlaysia", 0.0 },
  { "Marle", 0.115384616 },
  { "Marlea", 0.0 },
  { "Marleah", 0.0 },
  { "Marlean", 0.0 },
  { "Marleana", 0.0 },
  { "Marlecia", 0.0 },
  { "Marlee", 0.010354407 },
  { "Marleen", 0.0 },
  { "Marleena", 0.0 },
  { "Marleene", 0.0 },
  { "Marlei", 0.0 },
  { "Marleigh", 0.0 },
  { "Marleigha", 0.0 },
  { "Marleina", 0.0 },
  { "Marleisha", 0.0 },
  { "Marlem", 0.0 },
  { "Marlen", 0.15060443 },
  { "Marlena", 0.0 },
  { "Marlene", 0.0027735287 },
  { "Marlenea", 0.0 },
  { "Marlenee", 0.0 },
  { "Marleni", 0.0 },
  { "Marlenia", 0.0 },
  { "Marlenie", 0.0 },
  { "Marlenis", 0.0 },
  { "Marlenn", 0.0 },
  { "Marlenne", 0.0 },
  { "Marlenny", 0.0 },
  { "Marleny", 0.0 },
  { "Marles", 0.0 },
  { "Marlesa", 0.0 },
  { "Marlese", 0.0 },
  { "Marlesha", 0.0 },
  { "Marlet", 0.0 },
  { "Marleta", 0.0 },
  { "Marleth", 0.0 },
  { "Marletha", 0.0 },
  { "Marlett", 0.0 },
  { "Marletta", 0.0 },
  { "Marlette", 0.0 },
  { "Marley", 0.14336409 },
  { "Marleyann", 0.0 },
  { "Marleyjane", 0.0 },
  { "Marleyna", 0.0 },
  { "Marli", 0.0 },
  { "Marlia", 0.0 },
  { "Marliah", 0.0 },
  { "Marliana", 0.0 },
  { "Marlianna", 0.0 },
  { "Marlice", 0.0 },
  { "Marlicia", 0.0 },
  { "Marlie", 0.0 },
  { "Marliee", 0.0 },
  { "Marliegh", 0.0 },
  { "Marliene", 0.0 },
  { "Marlies", 0.0 },
  { "Marliese", 0.0 },
  { "Marlii", 0.0 },
  { "Marlik", 1.0 },
  { "Marlika", 0.0 },
  { "Marlin", 0.9295621 },
  { "Marlina", 0.0 },
  { "Marlinda", 0.0 },
  { "Marline", 0.0 },
  { "Marlis", 0.0 },
  { "Marlisa", 0.0 },
  { "Marlise", 0.0 },
  { "Marlisha", 0.0 },
  { "Marliss", 0.0 },
  { "Marlissa", 0.0 },
  { "Marlita", 0.0 },
  { "Marliyah", 0.0 },
  { "Marliz", 0.0 },
  { "Marlo", 0.31742784 },
  { "Marloe", 0.0 },
  { "Marlon", 0.9885176 },
  { "Marlone", 1.0 },
  { "Marlos", 1.0 },
  { "Marlou", 0.0 },
  { "Marlow", 0.7249899 },
  { "Marlowe", 0.44832 },
  { "Marlus", 0.0 },
  { "Marlvin", 1.0 },
  { "Marly", 0.0036101083 },
  { "Marlyce", 0.0 },
  { "Marlye", 0.0 },
  { "Marlyin", 0.0 },
  { "Marlyn", 0.2580355 },
  { "Marlyna", 0.0 },
  { "Marlynda", 0.0 },
  { "Marlyne", 0.0 },
  { "Marlynn", 0.0 },
  { "Marlynne", 0.0 },
  { "Marlys", 0.00055915903 },
  { "Marlyse", 0.0 },
  { "Marlyss", 0.0 },
  { "Marlyssa", 0.0 },
  { "Marna", 0.0 },
  { "Marnae", 0.0 },
  { "Marnay", 0.0 },
  { "Marne", 0.042792793 },
  { "Marnee", 0.0 },
  { "Marneisha", 0.0 },
  { "Marnell", 0.115523465 },
  { "Marnella", 0.0 },
  { "Marnesha", 0.0 },
  { "Marneshia", 0.0 },
  { "Marnetta", 0.0 },
  { "Marnette", 0.0 },
  { "Marney", 0.0 },
  { "Marni", 0.0015337423 },
  { "Marnia", 0.0 },
  { "Marnice", 0.0 },
  { "Marnie", 0.0 },
  { "Marnika", 0.0 },
  { "Marnina", 0.0 },
  { "Marniqua", 0.0 },
  { "Marnique", 0.0 },
  { "Marnisha", 0.0 },
  { "Marnita", 0.0 },
  { "Marniyah", 0.0 },
  { "Marny", 0.0 },
  { "Maro", 1.0 },
  { "Marolyn", 0.0 },
  { "Maron", 0.91729325 },
  { "Maronda", 0.0 },
  { "Maronica", 0.0 },
  { "Maroska", 0.0 },
  { "Marpessa", 0.0 },
  { "Marq", 1.0 },
  { "Marqavious", 1.0 },
  { "Marqel", 1.0 },
  { "Marqell", 1.0 },
  { "Marqez", 1.0 },
  { "Marqies", 1.0 },
  { "Marqis", 1.0 },
  { "Marqita", 0.0 },
  { "Marqua", 1.0 },
  { "Marquae", 1.0 },
  { "Marquael", 1.0 },
  { "Marquail", 1.0 },
  { "Marqual", 1.0 },
  { "Marquala", 0.0 },
  { "Marquale", 1.0 },
  { "Marquan", 1.0 },
  { "Marquarious", 1.0 },
  { "Marquarius", 1.0 },
  { "Marquas", 1.0 },
  { "Marquasha", 0.0 },
  { "Marquasia", 0.0 },
  { "Marquaveon", 1.0 },
  { "Marquavion", 1.0 },
  { "Marquavious", 1.0 },
  { "Marquavis", 1.0 },
  { "Marquavius", 1.0 },
  { "Marquay", 1.0 },
  { "Marque", 0.87676054 },
  { "Marquea", 0.0 },
  { "Marqueal", 1.0 },
  { "Marquece", 1.0 },
  { "Marquee", 0.864 },
  { "Marqueen", 0.0 },
  { "Marquees", 1.0 },
  { "Marqueese", 1.0 },
  { "Marqueeta", 0.0 },
  { "Marqueis", 1.0 },
  { "Marqueisha", 0.0 },
  { "Marquel", 0.90882355 },
  { "Marquell", 0.98675036 },
  { "Marquella", 0.0 },
  { "Marquelle", 0.7606838 },
  { "Marquerite", 0.0 },
  { "Marques", 0.9970479 },
  { "Marquesa", 0.0 },
  { "Marquese", 1.0 },
  { "Marquesha", 0.0 },
  { "Marqueshia", 0.0 },
  { "Marquess", 1.0 },
  { "Marquessa", 0.0 },
  { "Marquest", 1.0 },
  { "Marquet", 1.0 },
  { "Marqueta", 0.0 },
  { "Marquetia", 0.0 },
  { "Marquett", 1.0 },
  { "Marquetta", 0.0 },
  { "Marquette", 0.6553911 },
  { "Marquettia", 0.0 },
  { "Marquevious", 1.0 },
  { "Marquez", 0.9990242 },
  { "Marqueze", 1.0 },
  { "Marqui", 0.8323864 },
  { "Marquia", 0.0 },
  { "Marquian", 1.0 },
  { "Marquice", 1.0 },
  { "Marquida", 0.0 },
  { "Marquie", 0.8 },
  { "Marquies", 1.0 },
  { "Marquiese", 1.0 },
  { "Marquiesha", 0.0 },
  { "Marquieta", 0.0 },
  { "Marquietta", 0.0 },
  { "Marquiez", 1.0 },
  { "Marquil", 1.0 },
  { "Marquila", 0.0 },
  { "Marquilla", 0.0 },
  { "Marquille", 1.0 },
  { "Marquin", 1.0 },
  { "Marquina", 0.0 },
  { "Marquinn", 1.0 },
  { "Marquinta", 0.0 },
  { "Marquis", 0.97989565 },
  { "Marquisa", 0.0 },
  { "Marquise", 0.9614249 },
  { "Marquisha", 0.0 },
  { "Marquishia", 0.0 },
  { "Marquisia", 0.0 },
  { "Marquiss", 1.0 },
  { "Marquist", 1.0 },
  { "Marquita", 0.0027173914 },
  { "Marquite", 0.0 },
  { "Marquitha", 0.0 },
  { "Marquitia", 0.0 },
  { "Marquitta", 0.0 },
  { "Marquitte", 0.0 },
  { "Marquiz", 1.0 },
  { "Marquize", 1.0 },
  { "Marquon", 1.0 },
  { "Marqurita", 0.0 },
  { "Marqus", 1.0 },
  { "Marqutia", 0.0 },
  { "Marqwan", 1.0 },
  { "Marqwon", 1.0 },
  { "Marra", 0.0 },
  { "Marrah", 0.0 },
  { "Marranda", 0.0 },
  { "Marreco", 1.0 },
  { "Marrell", 1.0 },
  { "Marren", 0.0 },
  { "Marreo", 1.0 },
  { "Marreon", 1.0 },
  { "Marretta", 0.0 },
  { "Marri", 0.0 },
  { "Marria", 0.0 },
  { "Marriah", 0.0 },
  { "Marriam", 0.0 },
  { "Marrian", 0.0 },
  { "Marriana", 0.0 },
  { "Marriann", 0.0 },
  { "Marrianna", 0.0 },
  { "Marrianne", 0.0 },
  { "Marrick", 1.0 },
  { "Marrico", 1.0 },
  { "Marrie", 0.0 },
  { "Marrietta", 0.0 },
  { "Marrin", 0.3137255 },
  { "Marrina", 0.0 },
  { "Marrio", 1.0 },
  { "Marrion", 0.6078838 },
  { "Marris", 0.9117647 },
  { "Marrisa", 0.0 },
  { "Marrisha", 0.0 },
  { "Marrissa", 0.0 },
  { "Marrius", 1.0 },
  { "Marriyah", 0.0 },
  { "Marrk", 1.0 },
  { "Marry", 0.0 },
  { "Marryn", 0.0 },
  { "Mars", 0.9826087 },
  { "Marsa", 0.0 },
  { "Marsadie", 0.0 },
  { "Marsalis", 1.0 },
  { "Marschell", 0.0 },
  { "Marsden", 1.0 },
  { "Marsea", 0.0 },
  { "Marsean", 1.0 },
  { "Marseille", 0.0 },
  { "Marsel", 1.0 },
  { "Marsela", 0.0 },
  { "Marselina", 0.0 },
  { "Marselino", 1.0 },
  { "Marsell", 1.0 },
  { "Marsella", 0.0 },
  { "Marselo", 1.0 },
  { "Marsena", 0.0 },
  { "Marsenio", 1.0 },
  { "Marsetta", 0.0 },
  { "Marsh", 0.9714286 },
  { "Marsha", 0.0024067734 },
  { "Marshae", 0.0 },
  { "Marshai", 0.0 },
  { "Marshal", 0.98580474 },
  { "Marshala", 0.0 },
  { "Marshalene", 0.0 },
  { "Marshall", 0.9875712 },
  { "Marshalle", 0.0 },
  { "Marshana", 0.0 },
  { "Marshanna", 0.0 },
  { "Marshanti", 0.0 },
  { "Marshaun", 1.0 },
  { "Marshawn", 0.9642366 },
  { "Marshawna", 0.0 },
  { "Marshay", 0.0066755675 },
  { "Marshaya", 0.0 },
  { "Marshaye", 0.0 },
  { "Marshayla", 0.0 },
  { "Marshe", 0.0 },
  { "Marshea", 0.0 },
  { "Marsheela", 0.0 },
  { "Marsheena", 0.0 },
  { "Marsheila", 0.0 },
  { "Marshel", 0.97748595 },
  { "Marshelia", 0.0 },
  { "Marshell", 0.5925689 },
  { "Marshella", 0.0 },
  { "Marshelle", 0.0 },
  { "Marshia", 0.0 },
  { "Marshie", 0.0 },
  { "Marshon", 0.97252744 },
  { "Marshonda", 0.0 },
  { "Marshun", 1.0 },
  { "Marsi", 0.0 },
  { "Marsia", 0.0 },
  { "Marsie", 0.0 },
  { "Marsinah", 0.0 },
  { "Marson", 1.0 },
  { "Marston", 1.0 },
  { "Marsue", 0.0 },
  { "Mart", 1.0 },
  { "Marta", 0.002719319 },
  { "Martae", 1.0 },
  { "Martain", 1.0 },
  { "Martaja", 0.0 },
  { "Martajah", 0.0 },
  { "Martarius", 1.0 },
  { "Martasia", 0.0 },
  { "Martaveon", 1.0 },
  { "Martaveous", 1.0 },
  { "Martavia", 0.0 },
  { "Martavian", 1.0 },
  { "Martavion", 1.0 },
  { "Martavious", 1.0 },
  { "Martavis", 1.0 },
  { "Martavius", 1.0 },
  { "Martay", 1.0 },
  { "Martaz", 1.0 },
  { "Marte", 0.7916667 },
  { "Martee", 0.0 },
  { "Marteeka", 0.0 },
  { "Marteen", 0.0 },
  { "Marteisha", 0.0 },
  { "Marteka", 0.0 },
  { "Martel", 0.99369085 },
  { "Martell", 0.9855827 },
  { "Martella", 0.0 },
  { "Martelle", 0.6756757 },
  { "Martellis", 1.0 },
  { "Marten", 1.0 },
  { "Martena", 0.0 },
  { "Martene", 0.0 },
  { "Marterius", 1.0 },
  { "Marterrion", 1.0 },
  { "Marterrius", 1.0 },
  { "Martes", 1.0 },
  { "Martese", 1.0 },
  { "Martesha", 0.0 },
  { "Martez", 0.99840295 },
  { "Marteze", 1.0 },
  { "Marth", 0.0 },
  { "Martha", 0.0036201777 },
  { "Marthaann", 0.0 },
  { "Marthajane", 0.0 },
  { "Marthalee", 0.0 },
  { "Marthalene", 0.0 },
  { "Marthana", 0.0 },
  { "Marthann", 0.0 },
  { "Marthanne", 0.0 },
  { "Martharee", 0.0 },
  { "Marthe", 0.0 },
  { "Marthel", 0.0 },
  { "Marthell", 0.0 },
  { "Marthella", 0.0 },
  { "Marthena", 0.0 },
  { "Marthenia", 0.0 },
  { "Marther", 0.0 },
  { "Marthia", 0.0 },
  { "Marthie", 0.0 },
  { "Marthony", 1.0 },
  { "Marthy", 0.0 },
  { "Marti", 0.048561152 },
  { "Martia", 0.0 },
  { "Martial", 1.0 },
  { "Martiana", 0.0 },
  { "Martianna", 0.0 },
  { "Martica", 0.0 },
  { "Martice", 0.953125 },
  { "Marticia", 0.0 },
  { "Martie", 0.15988372 },
  { "Martiel", 0.0 },
  { "Martijn", 1.0 },
  { "Martika", 0.0 },
  { "Martile", 0.0 },
  { "Martin", 0.9951843 },
  { "Martina", 0.0023427433 },
  { "Martine", 0.10266802 },
  { "Martinez", 0.97399104 },
  { "Martini", 0.0 },
  { "Martiniano", 1.0 },
  { "Martinique", 0.0 },
  { "Martinis", 1.0 },
  { "Martinjr", 1.0 },
  { "Martino", 1.0 },
  { "Martinque", 0.0 },
  { "Martinus", 1.0 },
  { "Martiqua", 0.0 },
  { "Martique", 1.0 },
  { "Martir", 1.0 },
  { "Martis", 1.0 },
  { "Martisa", 0.0 },
  { "Martise", 1.0 },
  { "Martisha", 0.0 },
  { "Martita", 0.0 },
  { "Martiza", 0.0 },
  { "Martize", 1.0 },
  { "Marton", 1.0 },
  { "Martrail", 1.0 },
  { "Martravious", 1.0 },
  { "Martravius", 1.0 },
  { "Martrel", 1.0 },
  { "Martrell", 1.0 },
  { "Martrez", 1.0 },
  { "Martrice", 0.08208955 },
  { "Martricia", 0.0 },
  { "Martrina", 0.0 },
  { "Martwan", 1.0 },
  { "Martwon", 1.0 },
  { "Marty", 0.88805574 },
  { "Martyn", 1.0 },
  { "Martyna", 0.0 },
  { "Martynas", 1.0 },
  { "Maru", 1.0 },
  { "Marua", 0.0 },
  { "Marucs", 1.0 },
  { "Marueen", 0.0 },
  { "Maruf", 1.0 },
  { "Maruice", 1.0 },
  { "Maruin", 1.0 },
  { "Marus", 1.0 },
  { "Maruska", 0.0 },
  { "Marv", 1.0 },
  { "Marva", 0.00067600195 },
  { "Marval", 0.0 },
  { "Marvalee", 0.0 },
  { "Marvalene", 0.0 },
  { "Marvaline", 0.0 },
  { "Marvalyn", 0.0 },
  { "Marvan", 1.0 },
  { "Marve", 1.0 },
  { "Marveen", 0.0 },
  { "Marvel", 0.18794036 },
  { "Marvela", 0.0 },
  { "Marvelene", 0.0 },
  { "Marveline", 0.0 },
  { "Marvell", 0.76879025 },
  { "Marvella", 0.0 },
  { "Marvelle", 0.1242485 },
  { "Marvelous", 0.70526314 },
  { "Marvely", 0.0 },
  { "Marvelyn", 0.0 },
  { "Marven", 1.0 },
  { "Marvena", 0.0 },
  { "Marvene", 0.0 },
  { "Marvenia", 0.0 },
  { "Marvens", 1.0 },
  { "Marveon", 1.0 },
  { "Marvetta", 0.0 },
  { "Marvette", 0.0 },
  { "Marvi", 0.0 },
  { "Marvia", 0.0 },
  { "Marvic", 1.0 },
  { "Marvie", 0.20178041 },
  { "Marvil", 0.0 },
  { "Marvilla", 0.0 },
  { "Marvin", 0.9936023 },
  { "Marvina", 0.0 },
  { "Marvine", 0.014545455 },
  { "Marvinia", 0.0 },
  { "Marvins", 1.0 },
  { "Marvion", 1.0 },
  { "Marvis", 0.44494107 },
  { "Marvita", 0.0 },
  { "Marvlene", 0.0 },
  { "Marvon", 1.0 },
  { "Marvyl", 0.0 },
  { "Marvyn", 1.0 },
  { "Marwa", 0.0 },
  { "Marwaan", 1.0 },
  { "Marwah", 0.0 },
  { "Marwan", 1.0 },
  { "Marwin", 1.0 },
  { "Marwood", 1.0 },
  { "Marx", 0.98013246 },
  { "Mary", 0.0036652675 },
  { "Marya", 0.0 },
  { "Maryagnes", 0.0 },
  { "Maryah", 0.0 },
  { "Maryalice", 0.0 },
  { "Maryalyce", 0.0 },
  { "Maryam", 0.0 },
  { "Maryama", 0.0 },
  { "Maryamawit", 0.0 },
  { "Maryan", 0.023321554 },
  { "Maryana", 0.0 },
  { "Maryangel", 0.0 },
  { "Maryann", 0.0008379393 },
  { "Maryanna", 0.0 },
  { "Maryanne", 0.0 },
  { "Marybel", 0.0 },
  { "Marybell", 0.0 },
  { "Marybella", 0.0 },
  { "Marybelle", 0.0 },
  { "Marybeth", 0.0 },
  { "Marycarmen", 0.0 },
  { "Marycarol", 0.0 },
  { "Marycaroline", 0.0 },
  { "Marycatherine", 0.0 },
  { "Marycela", 0.0 },
  { "Marychristina", 0.0 },
  { "Maryclaire", 0.0 },
  { "Maryclare", 0.0 },
  { "Marycruz", 0.0 },
  { "Marydell", 0.0 },
  { "Marye", 0.0 },
  { "Maryeileen", 0.0 },
  { "Maryel", 0.0 },
  { "Maryela", 0.0 },
  { "Maryelaine", 0.0 },
  { "Maryeli", 0.0 },
  { "Maryelizabeth", 0.0 },
  { "Maryella", 0.0 },
  { "Maryelle", 0.0 },
  { "Maryellen", 0.0 },
  { "Maryellis", 0.0 },
  { "Maryellyn", 0.0 },
  { "Maryem", 0.0 },
  { "Maryemma", 0.0 },
  { "Maryesther", 0.0 },
  { "Maryetta", 0.0 },
  { "Maryette", 0.0 },
  { "Maryfaith", 0.0 },
  { "Maryfer", 0.0 },
  { "Maryfrances", 0.0 },
  { "Marygrace", 0.0 },
  { "Maryha", 0.0 },
  { "Maryhannah", 0.0 },
  { "Maryhelen", 0.0 },
  { "Maryia", 0.0 },
  { "Maryiah", 0.0 },
  { "Maryjane", 0.0 },
  { "Maryjayne", 0.0 },
  { "Maryjean", 0.0 },
  { "Maryjeanne", 0.0 },
  { "Maryjo", 0.0 },
  { "Maryjoan", 0.0 },
  { "Maryjoe", 0.0 },
  { "Maryjose", 0.0 },
  { "Maryjoy", 0.0 },
  { "Maryjune", 0.0 },
  { "Maryka", 0.0 },
  { "Marykate", 0.0 },
  { "Marykatherine", 0.0 },
  { "Marykathleen", 0.0 },
  { "Marykathryn", 0.0 },
  { "Marykay", 0.0 },
  { "Marykaye", 0.0 },
  { "Maryke", 0.0 },
  { "Maryl", 0.0 },
  { "Maryla", 0.0 },
  { "Marylan", 0.0 },
  { "Maryland", 0.24420401 },
  { "Marylane", 0.0 },
  { "Marylea", 0.0 },
  { "Marylean", 0.0 },
  { "Marylee", 0.0 },
  { "Marylen", 0.0 },
  { "Marylena", 0.0 },
  { "Marylene", 0.0 },
  { "Marylin", 0.0 },
  { "Marylinda", 0.0 },
  { "Maryline", 0.0 },
  { "Marylinn", 0.0 },
  { "Marylisa", 0.0 },
  { "Maryliz", 0.0 },
  { "Maryln", 0.0 },
  { "Marylois", 0.0 },
  { "Marylon", 0.0 },
  { "Marylou", 0.0 },
  { "Marylouise", 0.0 },
  { "Marylu", 0.0 },
  { "Marylue", 0.0 },
  { "Maryluz", 0.0 },
  { "Marylyn", 0.0 },
  { "Marylynn", 0.0 },
  { "Marylynne", 0.0 },
  { "Marymae", 0.0 },
  { "Marymar", 0.0 },
  { "Marymargaret", 0.0 },
  { "Marymichael", 0.0 },
  { "Maryn", 0.0 },
  { "Maryna", 0.0 },
  { "Marynel", 0.0 },
  { "Marynell", 0.0 },
  { "Maryon", 0.011764706 },
  { "Maryona", 0.0 },
  { "Maryonna", 0.0 },
  { "Maryori", 0.0 },
  { "Maryorie", 0.0 },
  { "Maryory", 0.0 },
  { "Marypat", 0.0 },
  { "Marypatricia", 0.0 },
  { "Marypaz", 0.0 },
  { "Maryrita", 0.0 },
  { "Maryrose", 0.0 },
  { "Maryruth", 0.0 },
  { "Marysa", 0.0 },
  { "Marysabel", 0.0 },
  { "Maryse", 0.0 },
  { "Marysia", 0.0 },
  { "Marysol", 0.0 },
  { "Maryssa", 0.0 },
  { "Marysue", 0.0 },
  { "Marysusan", 0.0 },
  { "Maryterese", 0.0 },
  { "Marytheresa", 0.0 },
  { "Marytza", 0.0 },
  { "Maryum", 0.0 },
  { "Maryuri", 0.0 },
  { "Maryury", 0.0 },
  { "Maryvel", 0.0 },
  { "Marz", 1.0 },
  { "Marzavion", 1.0 },
  { "Marzee", 0.0 },
  { "Marzel", 1.0 },
  { "Marzell", 0.54565704 },
  { "Marzella", 0.0 },
  { "Marzelle", 0.0 },
  { "Marzena", 0.0 },
  { "Marzetta", 0.0 },
  { "Marzette", 0.0 },
  { "Marzia", 0.0 },
  { "Marziah", 0.0 },
  { "Marzie", 0.0 },
  { "Marziya", 0.0 },
  { "Mas", 1.0 },
  { "Masa", 0.05970149 },
  { "Masaaki", 1.0 },
  { "Masada", 0.54545456 },
  { "Masae", 0.0 },
  { "Masah", 0.0 },
  { "Masaharu", 1.0 },
  { "Masahiro", 1.0 },
  { "Masai", 1.0 },
  { "Masaichi", 1.0 },
  { "Masaji", 1.0 },
  { "Masakatsu", 1.0 },
  { "Masakazu", 1.0 },
  { "Masaki", 1.0 },
  { "Masako", 0.0 },
  { "Masal", 0.0 },
  { "Masami", 0.98293513 },
  { "Masan", 1.0 },
  { "Masani", 0.0 },
  { "Masanobu", 1.0 },
  { "Masanori", 1.0 },
  { "Masao", 1.0 },
  { "Masaru", 1.0 },
  { "Masashi", 1.0 },
  { "Masataka", 1.0 },
  { "Masato", 1.0 },
  { "Masatoshi", 1.0 },
  { "Masaya", 1.0 },
  { "Masaye", 0.0 },
  { "Masayo", 0.0 },
  { "Masayoshi", 1.0 },
  { "Masayuki", 1.0 },
  { "Mascen", 1.0 },
  { "Mascud", 1.0 },
  { "Mase", 1.0 },
  { "Masel", 0.0 },
  { "Masen", 0.97225344 },
  { "Maseo", 1.0 },
  { "Masey", 0.0 },
  { "Masha", 0.0 },
  { "Mashad", 1.0 },
  { "Mashae", 0.0 },
  { "Mashal", 0.0 },
  { "Mashala", 0.0 },
  { "Mashana", 0.0 },
  { "Mashanda", 0.0 },
  { "Mashanna", 0.0 },
  { "Mashanti", 0.0 },
  { "Mashari", 0.0 },
  { "Mashaun", 1.0 },
  { "Mashauna", 0.0 },
  { "Mashaunda", 0.0 },
  { "Mashawn", 0.37373737 },
  { "Mashawna", 0.0 },
  { "Mashawnda", 0.0 },
  { "Mashay", 0.0 },
  { "Mashaya", 0.0 },
  { "Mashayla", 0.0 },
  { "Mashea", 0.0 },
  { "Masheka", 0.0 },
  { "Mashell", 0.0 },
  { "Mashelle", 0.0 },
  { "Mashia", 0.0 },
  { "Mashika", 0.0 },
  { "Mashiya", 0.0 },
  { "Mashon", 1.0 },
  { "Mashona", 0.0 },
  { "Mashonda", 0.0 },
  { "Mashunda", 0.0 },
  { "Mashyia", 0.0 },
  { "Masi", 0.7222222 },
  { "Masia", 0.0 },
  { "Masiah", 0.82682514 },
  { "Masie", 0.0 },
  { "Masiel", 0.0 },
  { "Masiela", 0.0 },
  { "Masih", 1.0 },
  { "Masika", 0.0 },
  { "Masil", 0.0 },
  { "Masin", 1.0 },
  { "Masina", 0.0 },
  { "Masion", 1.0 },
  { "Masir", 1.0 },
  { "Masiya", 0.065789476 },
  { "Masiyah", 0.29147983 },
  { "Maslah", 1.0 },
  { "Maslin", 0.0 },
  { "Maslyn", 0.0 },
  { "Maslynn", 0.0 },
  { "Maso", 1.0 },
  { "Mason", 0.9893701 },
  { "Masonjames", 1.0 },
  { "Masonlee", 1.0 },
  { "Masood", 1.0 },
  { "Masooma", 0.0 },
  { "Masoud", 1.0 },
  { "Massa", 0.0 },
  { "Massai", 1.0 },
  { "Massen", 1.0 },
  { "Massey", 0.6666667 },
  { "Massi", 1.0 },
  { "Massiah", 0.86217946 },
  { "Massie", 0.28695652 },
  { "Massiel", 0.0 },
  { "Massimiliano", 1.0 },
  { "Massimo", 1.0 },
  { "Massiyah", 1.0 },
  { "Masson", 1.0 },
  { "Master", 1.0 },
  { "Mastin", 1.0 },
  { "Maston", 1.0 },
  { "Masud", 1.0 },
  { "Masue", 0.0 },
  { "Masuko", 0.0 },
  { "Masuma", 0.0 },
  { "Masumi", 0.0 },
  { "Masun", 1.0 },
  { "Masuo", 1.0 },
  { "Masyn", 0.69793105 },
  { "Masynn", 0.0 },
  { "Mat", 1.0 },
  { "Mata", 0.0 },
  { "Mataeo", 1.0 },
  { "Matai", 1.0 },
  { "Mataia", 0.0 },
  { "Mataio", 1.0 },
  { "Matalie", 0.0 },
  { "Matalin", 0.0 },
  { "Matalyn", 0.0 },
  { "Matalynn", 0.0 },
  { "Matan", 1.0 },
  { "Matao", 1.0 },
  { "Matas", 1.0 },
  { "Matasha", 0.0 },
  { "Mataya", 0.0 },
  { "Matayah", 0.0 },
  { "Matayo", 1.0 },
  { "Mate", 1.0 },
  { "Matea", 0.0 },
  { "Mateah", 0.0 },
  { "Mateel", 0.0 },
  { "Mateen", 1.0 },
  { "Matei", 1.0 },
  { "Matej", 1.0 },
  { "Mateja", 0.26373628 },
  { "Matella", 0.0 },
  { "Matelyn", 0.0 },
  { "Matelynn", 0.0 },
  { "Mateo", 0.99946606 },
  { "Mateos", 1.0 },
  { "Matesha", 0.0 },
  { "Mateus", 1.0 },
  { "Mateusz", 1.0 },
  { "Mateya", 0.0 },
  { "Mateyo", 1.0 },
  { "Math", 1.0 },
  { "Matha", 0.0 },
  { "Mathai", 0.5652174 },
  { "Mathais", 1.0 },
  { "Mathan", 1.0 },
  { "Mathaniel", 1.0 },
  { "Mathayus", 1.0 },
  { "Mathea", 0.0 },
  { "Matheau", 1.0 },
  { "Mathel", 0.0 },
  { "Matheo", 1.0 },
  { "Matheos", 1.0 },
  { "Mather", 1.0 },
  { "Matheson", 1.0 },
  { "Matheu", 1.0 },
  { "Matheus", 1.0 },
  { "Mathew", 0.9972279 },
  { "Mathews", 1.0 },
  { "Mathhew", 1.0 },
  { "Mathias", 1.0 },
  { "Mathie", 0.0 },
  { "Mathieu", 1.0 },
  { "Mathiew", 1.0 },
  { "Mathijs", 1.0 },
  { "Mathilda", 0.0 },
  { "Mathilde", 0.0 },
  { "Mathis", 1.0 },
  { "Mathison", 0.54545456 },
  { "Mathius", 1.0 },
  { "Mathu", 1.0 },
  { "Mathue", 1.0 },
  { "Mathyas", 1.0 },
  { "Mathyis", 1.0 },
  { "Mathys", 1.0 },
  { "Mathyus", 1.0 },
  { "Mati", 0.2987013 },
  { "Matia", 0.012254902 },
  { "Matiah", 0.0 },
  { "Matiana", 0.0 },
  { "Matias", 1.0 },
  { "Maticia", 0.0 },
  { "Matie", 0.0 },
  { "Matigan", 0.0 },
  { "Matika", 0.0 },
  { "Matilda", 0.0003233107 },
  { "Matilde", 0.070164144 },
  { "Matildia", 0.0 },
  { "Matilee", 0.0 },
  { "Matilyn", 0.0 },
  { "Matilynn", 0.0 },
  { "Matin", 1.0 },
  { "Matina", 0.0 },
  { "Matis", 1.0 },
  { "Matisen", 0.0 },
  { "Matisha", 0.0 },
  { "Matison", 0.0 },
  { "Matisse", 0.4715026 },
  { "Matisyahu", 1.0 },
  { "Matisyn", 0.0 },
  { "Matix", 1.0 },
  { "Matiya", 0.0 },
  { "Matlin", 0.0 },
  { "Matline", 0.0 },
  { "Matlock", 1.0 },
  { "Matlyn", 0.0 },
  { "Matney", 0.0 },
  { "Mato", 1.0 },
  { "Matoya", 0.0 },
  { "Matraca", 0.0 },
  { "Matracia", 0.0 },
  { "Matrice", 0.0 },
  { "Matricia", 0.0 },
  { "Matrim", 1.0 },
  { "Matrix", 1.0 },
  { "Matrona", 0.0 },
  { "Mats", 1.0 },
  { "Matson", 1.0 },
  { "Matsue", 0.0 },
  { "Matsuichi", 1.0 },
  { "Matsuko", 0.0 },
  { "Matsuo", 1.0 },
  { "Matsuye", 0.0 },
  { "Matsuyo", 0.0 },
  { "Matt", 0.99978745 },
  { "Mattalyn", 0.0 },
  { "Mattalynn", 0.0 },
  { "Mattan", 1.0 },
  { "Mattaniah", 1.0 },
  { "Mattaya", 0.0 },
  { "Mattea", 0.0 },
  { "Matteen", 1.0 },
  { "Mattelyn", 0.0 },
  { "Mattelynn", 0.0 },
  { "Matten", 1.0 },
  { "Matteo", 0.9996132 },
  { "Matteson", 0.083333336 },
  { "Matteus", 1.0 },
  { "Mattew", 1.0 },
  { "Matthaeus", 1.0 },
  { "Matthais", 1.0 },
  { "Matthan", 1.0 },
  { "Matthaus", 1.0 },
  { "Matthe", 1.0 },
  { "Matthea", 0.0 },
  { "Mattheau", 1.0 },
  { "Matthen", 1.0 },
  { "Mattheo", 1.0 },
  { "Mattheq", 1.0 },
  { "Matther", 1.0 },
  { "Mattheu", 1.0 },
  { "Mattheus", 1.0 },
  { "Matthew", 0.9965308 },
  { "Matthewalexande", 1.0 },
  { "Matthewdavid", 1.0 },
  { "Matthewjames", 1.0 },
  { "Matthewjohn", 1.0 },
  { "Matthewjoseph", 1.0 },
  { "Matthewryan", 1.0 },
  { "Matthews", 1.0 },
  { "Matthey", 1.0 },
  { "Matthias", 1.0 },
  { "Matthieu", 1.0 },
  { "Matthiew", 1.0 },
  { "Matthijs", 1.0 },
  { "Matthis", 1.0 },
  { "Matthius", 1.0 },
  { "Matthrew", 1.0 },
  { "Matthue", 1.0 },
  { "Matthw", 1.0 },
  { "Matti", 0.0892562 },
  { "Mattia", 0.5612903 },
  { "Mattias", 1.0 },
  { "Mattie", 0.0059287637 },
  { "Mattielee", 0.0 },
  { "Mattigan", 0.0 },
  { "Mattilyn", 0.0 },
  { "Mattilynn", 0.0 },
  { "Mattingly", 0.17607974 },
  { "Mattis", 1.0 },
  { "Mattisen", 0.0 },
  { "Mattison", 0.09206489 },
  { "Mattisyn", 0.0 },
  { "Mattix", 1.0 },
  { "Mattlock", 1.0 },
  { "Mattlyn", 0.0 },
  { "Mattox", 0.9516129 },
  { "Mattson", 1.0 },
  { "Mattthew", 1.0 },
  { "Matty", 0.1752434 },
  { "Mattye", 0.0 },
  { "Mattyson", 0.0 },
  { "Matus", 1.0 },
  { "Matvei", 1.0 },
  { "Matvey", 1.0 },
  { "Matviy", 1.0 },
  { "Maty", 0.0 },
  { "Matyas", 1.0 },
  { "Matylda", 0.0 },
  { "Matyson", 0.0 },
  { "Mau", 1.0 },
  { "Maud", 0.0023552652 },
  { "Mauda", 0.0 },
  { "Maude", 0.0029149407 },
  { "Maudean", 0.0 },
  { "Maudeen", 0.0 },
  { "Maudeline", 0.0 },
  { "Maudell", 0.0 },
  { "Maudella", 0.0 },
  { "Maudelle", 0.0 },
  { "Maudena", 0.0 },
  { "Maudene", 0.0 },
  { "Maudestine", 0.0 },
  { "Maudie", 0.0 },
  { "Maudine", 0.0 },
  { "Maudra", 0.0 },
  { "Maudrey", 0.0 },
  { "Maudry", 0.0 },
  { "Maudy", 0.0 },
  { "Mauel", 1.0 },
  { "Mauer", 1.0 },
  { "Maui", 0.56164384 },
  { "Mauli", 0.0 },
  { "Maulik", 1.0 },
  { "Maulin", 1.0 },
  { "Mauline", 0.0 },
  { "Mauna", 0.0 },
  { "Maung", 1.0 },
  { "Maura", 0.00026543505 },
  { "Maurece", 1.0 },
  { "Maureen", 0.0022733575 },
  { "Maureena", 0.0 },
  { "Maureene", 0.0 },
  { "Mauren", 0.0 },
  { "Maurene", 0.0 },
  { "Maurese", 1.0 },
  { "Mauresha", 0.0 },
  { "Maurey", 1.0 },
  { "Mauri", 0.13773796 },
  { "Mauria", 0.0 },
  { "Mauriah", 0.0 },
  { "Mauriana", 0.0 },
  { "Maurianna", 0.0 },
  { "Maurica", 0.0 },
  { "Maurice", 0.9808484 },
  { "Mauriceo", 1.0 },
  { "Mauricia", 0.0 },
  { "Mauricio", 0.9997049 },
  { "Maurico", 1.0 },
  { "Mauricus", 1.0 },
  { "Maurie", 0.23902439 },
  { "Mauriella", 0.0 },
  { "Maurielle", 0.0 },
  { "Maurietta", 0.0 },
  { "Maurika", 0.0 },
  { "Maurilio", 1.0 },
  { "Maurin", 0.0 },
  { "Maurina", 0.0 },
  { "Maurine", 0.0 },
  { "Maurio", 1.0 },
  { "Maurion", 1.0 },
  { "Maurisa", 0.0 },
  { "Maurisha", 0.0 },
  { "Maurisio", 1.0 },
  { "Maurissa", 0.0 },
  { "Maurita", 0.0 },
  { "Mauritz", 1.0 },
  { "Mauriyah", 0.0 },
  { "Maurizio", 1.0 },
  { "Maurkice", 1.0 },
  { "Mauro", 1.0 },
  { "Maurquise", 1.0 },
  { "Maurus", 1.0 },
  { "Maury", 0.90451515 },
  { "Maurya", 0.10344828 },
  { "Mauryn", 0.0 },
  { "Mava", 0.0 },
  { "Mavel", 0.0 },
  { "Mavelyn", 0.0 },
  { "Maven", 0.45553538 },
  { "Maveric", 1.0 },
  { "Maverick", 0.992915 },
  { "Maverik", 1.0 },
  { "Maverix", 1.0 },
  { "Mavery", 0.0 },
  { "Maveryck", 1.0 },
  { "Maveryk", 1.0 },
  { "Mavi", 0.0 },
  { "Mavia", 0.0 },
  { "Mavie", 0.0 },
  { "Mavin", 0.81609195 },
  { "Mavis", 0.0069341003 },
  { "Mavourneen", 0.0 },
  { "Mavra", 0.0 },
  { "Mavric", 1.0 },
  { "Mavrick", 1.0 },
  { "Mavrik", 1.0 },
  { "Mavryck", 1.0 },
  { "Mavryk", 1.0 },
  { "Mavyn", 0.0 },
  { "Mawa", 0.0 },
  { "Mawada", 0.0 },
  { "Mawiyah", 0.0 },
  { "Max", 0.99552876 },
  { "Maxamilian", 1.0 },
  { "Maxamillian", 1.0 },
  { "Maxamillion", 1.0 },
  { "Maxamis", 1.0 },
  { "Maxamus", 1.0 },
  { "Maxcine", 0.0 },
  { "Maxden", 1.0 },
  { "Maxeen", 0.0 },
  { "Maxel", 1.0 },
  { "Maxell", 1.0 },
  { "Maxemiliano", 1.0 },
  { "Maxen", 1.0 },
  { "Maxence", 1.0 },
  { "Maxene", 0.0 },
  { "Maxey", 0.98239434 },
  { "Maxfield", 1.0 },
  { "Maxi", 0.4635193 },
  { "Maxie", 0.54052156 },
  { "Maxiel", 0.25 },
  { "Maxiene", 0.0 },
  { "Maxim", 0.9988802 },
  { "Maxima", 0.0 },
  { "Maximas", 1.0 },
  { "Maxime", 0.9438833 },
  { "Maximiano", 1.0 },
  { "Maximilan", 1.0 },
  { "Maximilano", 1.0 },
  { "Maximili", 1.0 },
  { "Maximilian", 1.0 },
  { "Maximiliano", 1.0 },
  { "Maximilien", 1.0 },
  { "Maximilion", 1.0 },
  { "Maximillan", 1.0 },
  { "Maximillian", 1.0 },
  { "Maximilliano", 1.0 },
  { "Maximillien", 1.0 },
  { "Maximillion", 1.0 },
  { "Maximina", 0.0 },
  { "Maximino", 1.0 },
  { "Maximo", 1.0 },
  { "Maximos", 1.0 },
  { "Maximous", 1.0 },
  { "Maximum", 1.0 },
  { "Maximus", 0.9994511 },
  { "Maxin", 1.0 },
  { "Maxine", 0.003367635 },
  { "Maxis", 1.0 },
  { "Maxlynn", 0.0 },
  { "Maxmilian", 1.0 },
  { "Maxmiliano", 1.0 },
  { "Maxmillian", 1.0 },
  { "Maxmillion", 1.0 },
  { "Maxola", 0.0 },
  { "Maxon", 1.0 },
  { "Maxsen", 1.0 },
  { "Maxsim", 1.0 },
  { "Maxson", 1.0 },
  { "Maxston", 1.0 },
  { "Maxten", 1.0 },
  { "Maxtin", 1.0 },
  { "Maxton", 1.0 },
  { "Maxtyn", 0.88095236 },
  { "Maxuel", 1.0 },
  { "Maxum", 1.0 },
  { "Maxus", 1.0 },
  { "Maxwel", 1.0 },
  { "Maxwell", 0.99645066 },
  { "Maxwill", 1.0 },
  { "Maxx", 0.98386115 },
  { "Maxxamus", 1.0 },
  { "Maxxen", 1.0 },
  { "Maxxim", 1.0 },
  { "Maxximus", 1.0 },
  { "Maxxine", 0.0 },
  { "Maxxis", 1.0 },
  { "Maxxon", 1.0 },
  { "Maxxton", 1.0 },
  { "Maxxwel", 1.0 },
  { "Maxxwell", 1.0 },
  { "Maxy", 1.0 },
  { "Maxym", 1.0 },
  { "Maxymilian", 1.0 },
  { "Maxymus", 1.0 },
  { "Maxyn", 1.0 },
  { "Maxyne", 0.0 },
  { "May", 0.014325532 },
  { "Maya", 0.001733043 },
  { "Mayaar", 0.0 },
  { "Mayada", 0.0 },
  { "Mayah", 0.0 },
  { "Mayala", 0.0 },
  { "Mayalen", 0.0 },
  { "Mayalyn", 0.0 },
  { "Mayan", 0.38323355 },
  { "Mayana", 0.0 },
  { "Mayani", 0.0 },
  { "Mayank", 1.0 },
  { "Mayann", 0.0 },
  { "Mayanna", 0.0 },
  { "Mayar", 0.043010753 },
  { "Mayara", 0.0 },
  { "Mayari", 0.0 },
  { "Mayarose", 0.0 },
  { "Maybel", 0.0 },
  { "Maybelin", 0.0 },
  { "Maybeline", 0.0 },
  { "Maybell", 0.0 },
  { "Maybelle", 0.0 },
  { "Maybellene", 0.0 },
  { "Maybelline", 0.0 },
  { "Maybeth", 0.0 },
  { "Mayble", 0.0 },
  { "Maybre", 0.0 },
  { "Maybree", 0.0 },
  { "Maybrie", 0.0 },
  { "Mayce", 0.0 },
  { "Maycee", 0.0 },
  { "Maycel", 0.0 },
  { "Maycen", 0.6147541 },
  { "Mayceon", 1.0 },
  { "Maycey", 0.0 },
  { "Mayci", 0.0 },
  { "Maycie", 0.0 },
  { "Maycin", 0.0 },
  { "Maycle", 0.0 },
  { "Mayco", 1.0 },
  { "Maycol", 1.0 },
  { "Maycon", 1.0 },
  { "Mayda", 0.0 },
  { "Maydean", 0.0 },
  { "Maydee", 0.0 },
  { "Maydeen", 0.0 },
  { "Maydel", 0.0 },
  { "Maydelin", 0.0 },
  { "Maydell", 0.0 },
  { "Maydelle", 0.0 },
  { "Maydelyn", 0.0 },
  { "Mayden", 0.0 },
  { "Maydene", 0.0 },
  { "Maye", 0.0 },
  { "Mayeda", 0.0 },
  { "Mayel", 1.0 },
  { "Mayela", 0.0 },
  { "Mayeli", 0.0 },
  { "Mayelin", 0.0 },
  { "Mayella", 0.0 },
  { "Mayely", 0.0 },
  { "Mayen", 1.0 },
  { "Mayer", 0.9972663 },
  { "Mayerli", 0.0 },
  { "Mayerlin", 0.0 },
  { "Mayerly", 0.0 },
  { "Mayes", 0.9350649 },
  { "Mayetta", 0.0 },
  { "Mayfield", 1.0 },
  { "Mayford", 1.0 },
  { "Mayfred", 0.0 },
  { "Maygan", 0.0 },
  { "Maygen", 0.0 },
  { "Mayghan", 0.0 },
  { "Mayha", 0.0 },
  { "Mayhem", 1.0 },
  { "Mayher", 0.0 },
  { "Mayhew", 1.0 },
  { "Mayia", 0.0 },
  { "Mayim", 0.0 },
  { "Mayisha", 0.0 },
  { "Mayjor", 1.0 },
  { "Mayka", 0.0 },
  { "Maykala", 0.0 },
  { "Maykayla", 0.0 },
  { "Maykaylee", 0.0 },
  { "Maykel", 1.0 },
  { "Mayko", 1.0 },
  { "Maykol", 1.0 },
  { "Mayla", 0.0 },
  { "Maylah", 0.0 },
  { "Maylan", 0.0 },
  { "Maylana", 0.0 },
  { "Mayland", 1.0 },
  { "Maylani", 0.0 },
  { "Maylanie", 0.0 },
  { "Maylasia", 0.0 },
  { "Mayle", 0.0 },
  { "Maylea", 0.0 },
  { "Mayleah", 0.0 },
  { "Maylee", 0.0 },
  { "Mayleen", 0.0 },
  { "Maylei", 0.0 },
  { "Mayleigh", 0.0 },
  { "Maylen", 0.0 },
  { "Maylena", 0.0 },
  { "Maylene", 0.0 },
  { "Maylet", 0.0 },
  { "Mayley", 0.0 },
  { "Mayli", 0.0 },
  { "Maylia", 0.0 },
  { "Mayliah", 0.0 },
  { "Maylie", 0.0 },
  { "Maylin", 0.0 },
  { "Maylina", 0.0 },
  { "Mayline", 0.0 },
  { "Mayling", 0.0 },
  { "Maylinn", 0.0 },
  { "Maylis", 0.0 },
  { "Maylon", 0.9766667 },
  { "Mayly", 0.0 },
  { "Maylyn", 0.0 },
  { "Maylynn", 0.0 },
  { "Mayme", 0.0 },
  { "Maymie", 0.0 },
  { "Maymouna", 0.0 },
  { "Maymuna", 0.0 },
  { "Maymunah", 0.0 },
  { "Mayna", 0.0 },
  { "Maynard", 0.9985144 },
  { "Mayne", 0.5 },
  { "Maynerd", 1.0 },
  { "Maynette", 0.0 },
  { "Maynie", 0.0 },
  { "Maynor", 1.0 },
  { "Maynord", 1.0 },
  { "Mayo", 0.82841825 },
  { "Mayola", 0.0 },
  { "Mayolo", 1.0 },
  { "Mayoma", 0.0 },
  { "Mayon", 1.0 },
  { "Mayona", 0.0 },
  { "Mayonna", 0.0 },
  { "Mayor", 1.0 },
  { "Mayowa", 1.0 },
  { "Mayra", 0.0106613105 },
  { "Mayraalejandra", 0.0 },
  { "Mayrah", 0.0 },
  { "Mayrani", 0.0 },
  { "Mayre", 0.0 },
  { "Mayree", 0.0 },
  { "Mayrel", 0.0 },
  { "Mayreli", 0.0 },
  { "Mayrelin", 0.0 },
  { "Mayrely", 0.0 },
  { "Mayren", 0.0 },
  { "Mayrene", 0.0 },
  { "Mayreni", 0.0 },
  { "Mayrim", 0.0 },
  { "Mayrin", 0.0 },
  { "Mayron", 1.0 },
  { "Mays", 0.70731705 },
  { "Maysa", 0.0 },
  { "Maysaa", 0.0 },
  { "Maysam", 0.0 },
  { "Maysan", 0.0 },
  { "Maysea", 0.0 },
  { "Maysel", 0.0 },
  { "Maysen", 0.4641676 },
  { "Maysha", 0.0 },
  { "Maysie", 0.0 },
  { "Maysin", 0.9411765 },
  { "Maysn", 1.0 },
  { "Mayson", 0.7738569 },
  { "Maysoon", 0.0 },
  { "Mayssa", 0.0 },
  { "Maysun", 0.04950495 },
  { "Maytal", 0.0 },
  { "Mayte", 0.0 },
  { "Maytee", 0.0 },
  { "Maytha", 0.0 },
  { "Maythe", 0.0 },
  { "Maythel", 0.0 },
  { "Maytte", 0.0 },
  { "Mayu", 0.0 },
  { "Mayukh", 1.0 },
  { "Mayukha", 0.0 },
  { "Mayuko", 0.0 },
  { "Mayumi", 0.0 },
  { "Mayur", 1.0 },
  { "Mayuri", 0.0 },
  { "Mayva", 0.0 },
  { "Mayve", 0.0 },
  { "Mayven", 0.0 },
  { "Mayvis", 0.0 },
  { "Maywood", 0.8125 },
  { "Mayzee", 0.0 },
  { "Mayzell", 0.0 },
  { "Mayzelle", 0.0 },
  { "Mayzi", 0.0 },
  { "Mayzie", 0.0 },
  { "Maz", 1.0 },
  { "Mazal", 0.0 },
  { "Mazani", 0.0 },
  { "Mazari", 0.0 },
  { "Mazaria", 0.0 },
  { "Mazarine", 0.0 },
  { "Maze", 1.0 },
  { "Mazee", 0.0 },
  { "Mazel", 0.0 },
  { "Mazell", 0.0 },
  { "Mazella", 0.0 },
  { "Mazelle", 0.0 },
  { "Mazelyn", 0.0 },
  { "Mazelynn", 0.0 },
  { "Mazen", 1.0 },
  { "Mazey", 0.0 },
  { "Mazhar", 1.0 },
  { "Mazi", 0.42307693 },
  { "Maziah", 0.3875 },
  { "Maziar", 1.0 },
  { "Mazie", 0.0 },
  { "Mazikeen", 0.0 },
  { "Mazin", 1.0 },
  { "Maziya", 0.0 },
  { "Maziyah", 0.0 },
  { "Mazle", 0.0 },
  { "Mazlyn", 0.0 },
  { "Mazlynn", 0.0 },
  { "Mazola", 0.0 },
  { "Mazon", 1.0 },
  { "Mazy", 0.0 },
  { "Mazyah", 0.0 },
  { "Mazzi", 0.0 },
  { "Mazzie", 0.0 },
  { "Mazzy", 0.0 },
  { "Mber", 0.0 },
  { "Mc", 0.5206422 },
  { "Mcadoo", 1.0 },
  { "Mcalister", 1.0 },
  { "Mcallister", 0.7962963 },
  { "Mcarther", 1.0 },
  { "Mcarthur", 1.0 },
  { "Mcauther", 1.0 },
  { "Mcauthor", 1.0 },
  { "Mcauthur", 1.0 },
  { "Mcayla", 0.0 },
  { "Mccabe", 1.0 },
  { "Mccade", 1.0 },
  { "Mccaela", 0.0 },
  { "Mccai", 1.0 },
  { "Mccain", 1.0 },
  { "Mccall", 0.074387945 },
  { "Mccarthy", 1.0 },
  { "Mccartney", 0.28362572 },
  { "Mccarty", 0.5 },
  { "Mccauley", 0.7586207 },
  { "Mccayla", 0.0 },
  { "Mcchael", 1.0 },
  { "Mcclain", 0.9166667 },
  { "Mcclane", 1.0 },
  { "Mcclellan", 1.0 },
  { "Mcclinton", 1.0 },
  { "Mcclure", 1.0 },
  { "Mcconnell", 1.0 },
  { "Mccord", 1.0 },
  { "Mccormick", 1.0 },
  { "Mccoy", 0.98392415 },
  { "Mccrae", 0.9489796 },
  { "Mccray", 1.0 },
  { "Mcdaniel", 1.0 },
  { "Mcdonald", 1.0 },
  { "Mcelroy", 1.0 },
  { "Mcgarrett", 1.0 },
  { "Mcgregor", 1.0 },
  { "Mcguire", 1.0 },
  { "Mcgwire", 1.0 },
  { "Mchael", 1.0 },
  { "Mchale", 1.0 },
  { "Mchenry", 1.0 },
  { "Mcihael", 1.0 },
  { "Mcihelle", 0.0 },
  { "Mckade", 1.0 },
  { "Mckaden", 1.0 },
  { "Mckae", 0.0 },
  { "Mckaela", 0.0 },
  { "Mckai", 1.0 },
  { "Mckail", 1.0 },
  { "Mckaila", 0.0 },
  { "Mckailah", 0.0 },
  { "Mckailey", 0.0 },
  { "Mckailyn", 0.0 },
  { "Mckala", 0.0 },
  { "Mckale", 0.969697 },
  { "Mckaleb", 1.0 },
  { "Mckalee", 0.0 },
  { "Mckaleigh", 0.0 },
  { "Mckaley", 0.0 },
  { "Mckall", 0.0 },
  { "Mckalyn", 0.0 },
  { "Mckalynn", 0.0 },
  { "Mckane", 1.0 },
  { "Mckay", 0.8649489 },
  { "Mckaya", 0.0 },
  { "Mckayden", 0.0 },
  { "Mckaye", 0.0 },
  { "Mckayla", 0.00040929927 },
  { "Mckaylah", 0.0 },
  { "Mckaylan", 0.0 },
  { "Mckayle", 0.0 },
  { "Mckaylee", 0.0 },
  { "Mckaylen", 0.0 },
  { "Mckayli", 0.0 },
  { "Mckaylia", 0.0 },
  { "Mckaylie", 0.0 },
  { "Mckaylin", 0.0 },
  { "Mckaylyn", 0.0 },
  { "Mckaylynn", 0.0 },
  { "Mckean", 1.0 },
  { "Mckee", 1.0 },
  { "Mckeena", 0.0 },
  { "Mckeever", 1.0 },
  { "Mckel", 0.5 },
  { "Mckell", 0.030232558 },
  { "Mckella", 0.0 },
  { "Mckelle", 0.0 },
  { "Mckelvey", 0.0 },
  { "Mckena", 0.0 },
  { "Mckenah", 0.0 },
  { "Mckendra", 0.0 },
  { "Mckenize", 0.0 },
  { "Mckenlee", 0.0 },
  { "Mckenleigh", 0.0 },
  { "Mckenley", 0.14909092 },
  { "Mckenli", 0.0 },
  { "Mckenlie", 0.0 },
  { "Mckenly", 0.09090909 },
  { "Mckenna", 0.0052260333 },
  { "Mckennah", 0.0 },
  { "Mckennan", 1.0 },
  { "Mckennon", 1.0 },
  { "Mckennzie", 0.0 },
  { "Mckensey", 0.0 },
  { "Mckensi", 0.0 },
  { "Mckensie", 0.0 },
  { "Mckensley", 0.0 },
  { "Mckensy", 0.0 },
  { "Mckenze", 0.0 },
  { "Mckenzee", 0.0 },
  { "Mckenzey", 0.0 },
  { "Mckenzi", 0.0 },
  { "Mckenzie", 0.037574366 },
  { "Mckenziee", 0.0 },
  { "Mckenzii", 0.0 },
  { "Mckenzlee", 0.0 },
  { "Mckenzlie", 0.0 },
  { "Mckenzy", 0.035750765 },
  { "Mckenzye", 0.0 },
  { "Mckeon", 1.0 },
  { "Mckeyla", 0.0 },
  { "Mckinely", 1.0 },
  { "Mckinlea", 0.0 },
  { "Mckinlee", 0.0 },
  { "Mckinleigh", 0.0 },
  { "Mckinley", 0.5311984 },
  { "Mckinli", 0.0 },
  { "Mckinlie", 0.0 },
  { "Mckinly", 0.14285715 },
  { "Mckinna", 0.0 },
  { "Mckinnah", 0.0 },
  { "Mckinney", 0.6034483 },
  { "Mckinnley", 0.15690866 },
  { "Mckinnly", 0.0 },
  { "Mckinnon", 0.9479167 },
  { "Mckinsey", 0.021912351 },
  { "Mckinsie", 0.0 },
  { "Mckinsley", 0.0 },
  { "Mckinze", 0.0 },
  { "Mckinzee", 0.0 },
  { "Mckinzey", 0.0 },
  { "Mckinzi", 0.0 },
  { "Mckinzie", 0.0 },
  { "Mckinzley", 0.0 },
  { "Mckinzy", 0.0 },
  { "Mckoy", 1.0 },
  { "Mckye", 1.0 },
  { "Mckyla", 0.0 },
  { "Mckyle", 1.0 },
  { "Mckylee", 0.0 },
  { "Mckylie", 0.0 },
  { "Mckynlee", 0.0 },
  { "Mckynleigh", 0.0 },
  { "Mckynley", 0.0 },
  { "Mckynli", 0.0 },
  { "Mckynlie", 0.0 },
  { "Mckynna", 0.0 },
  { "Mckynsie", 0.0 },
  { "Mckynze", 0.0 },
  { "Mckynzee", 0.0 },
  { "Mckynzi", 0.0 },
  { "Mckynzie", 0.0 },
  { "Mclain", 0.92893404 },
  { "Mclaine", 0.0 },
  { "Mclane", 0.9456522 },
  { "Mclaren", 0.8703704 },
  { "Mclean", 0.96615386 },
  { "Mcneal", 1.0 },
  { "Mcneil", 1.0 },
  { "Mcneva", 0.0 },
  { "Mcquade", 1.0 },
  { "Mcrae", 0.83870965 },
  { "Mcray", 1.0 },
  { "Md", 1.0 },
  { "Me", 0.0 },
  { "Mea", 0.0 },
  { "Meabh", 0.0 },
  { "Mead", 1.0 },
  { "Meada", 0.0 },
  { "Meade", 0.98848367 },
  { "Meadie", 0.0 },
  { "Meadow", 0.0 },
  { "Meagan", 0.0017376195 },
  { "Meagann", 0.0 },
  { "Meagen", 0.0 },
  { "Meaghan", 0.00050017866 },
  { "Meaghann", 0.0 },
  { "Meaghen", 0.0 },
  { "Meagin", 0.0 },
  { "Meagon", 0.0 },
  { "Meagyn", 0.0 },
  { "Meah", 0.0 },
  { "Meahan", 0.0 },
  { "Meahgan", 0.0 },
  { "Mealea", 0.0 },
  { "Mealie", 0.0 },
  { "Meanna", 0.0 },
  { "Meara", 0.0 },
  { "Mearah", 0.0 },
  { "Mearl", 0.73870575 },
  { "Mearle", 0.7592593 },
  { "Mearlene", 0.0 },
  { "Mearline", 0.0 },
  { "Measha", 0.0 },
  { "Measia", 0.0 },
  { "Meba", 0.7804878 },
  { "Meca", 0.0 },
  { "Mecaela", 0.0 },
  { "Mecayla", 0.0 },
  { "Mecca", 0.101904765 },
  { "Meccah", 0.0 },
  { "Mecedes", 0.0 },
  { "Mecha", 0.0 },
  { "Mechaela", 0.0 },
  { "Mechel", 0.6216216 },
  { "Mechele", 0.0 },
  { "Mechell", 0.0 },
  { "Mechella", 0.0 },
  { "Mechelle", 0.0 },
  { "Mechille", 0.0 },
  { "Mecia", 0.0 },
  { "Mecie", 0.0 },
  { "Mecislaus", 1.0 },
  { "Meckenzie", 0.0 },
  { "Meco", 0.5 },
  { "Mecole", 0.0 },
  { "Mecos", 1.0 },
  { "Meda", 0.0 },
  { "Medard", 1.0 },
  { "Medardo", 1.0 },
  { "Meddie", 0.22916667 },
  { "Medea", 0.0 },
  { "Medelin", 0.0 },
  { "Medeline", 0.0 },
  { "Medelyn", 0.0 },
  { "Medena", 0.0 },
  { "Mederic", 1.0 },
  { "Medford", 1.0 },
  { "Medgar", 1.0 },
  { "Medger", 1.0 },
  { "Medha", 0.0 },
  { "Medhansh", 1.0 },
  { "Media", 0.0 },
  { "Medie", 0.0 },
  { "Medin", 1.0 },
  { "Medina", 0.0 },
  { "Medinah", 0.0 },
  { "Medley", 0.2777778 },
  { "Medora", 0.0 },
  { "Medra", 0.0 },
  { "Medric", 1.0 },
  { "Medrick", 1.0 },
  { "Medrith", 0.0 },
  { "Medwin", 1.0 },
  { "Mee", 0.0 },
  { "Meea", 0.0 },
  { "Meeah", 0.0 },
  { "Meegan", 0.0 },
  { "Meeghan", 0.0 },
  { "Meeka", 0.0 },
  { "Meekah", 0.0 },
  { "Meeko", 1.0 },
  { "Meela", 0.0 },
  { "Meelad", 1.0 },
  { "Meelah", 0.0 },
  { "Meena", 0.009925558 },
  { "Meenah", 0.0 },
  { "Meenakshi", 0.0 },
  { "Meer", 1.0 },
  { "Meera", 0.0 },
  { "Meerab", 0.06 },
  { "Meerah", 0.0 },
  { "Meerub", 0.0 },
  { "Meesha", 0.0 },
  { "Meet", 1.0 },
  { "Meeya", 0.0 },
  { "Meeyah", 0.0 },
  { "Meg", 0.0 },
  { "Mega", 0.21167883 },
  { "Megaa", 1.0 },
  { "Megaan", 0.0 },
  { "Megahn", 0.0 },
  { "Megail", 1.0 },
  { "Megale", 1.0 },
  { "Megam", 0.0 },
  { "Megan", 0.0025240332 },
  { "Megana", 0.0 },
  { "Meganelizabeth", 0.0 },
  { "Meganmarie", 0.0 },
  { "Megann", 0.0 },
  { "Meganne", 0.0 },
  { "Megean", 0.0 },
  { "Megen", 0.0 },
  { "Meggan", 0.0 },
  { "Meggen", 0.0 },
  { "Megghan", 0.0 },
  { "Meggi", 0.0 },
  { "Meggie", 0.0 },
  { "Meggin", 0.0 },
  { "Megh", 1.0 },
  { "Megha", 0.0 },
  { "Meghaan", 0.0 },
  { "Megham", 0.0 },
  { "Meghan", 0.002391408 },
  { "Meghana", 0.0 },
  { "Meghann", 0.0 },
  { "Meghanne", 0.0 },
  { "Meghean", 0.0 },
  { "Meghen", 0.0 },
  { "Meghin", 0.0 },
  { "Meghna", 0.0 },
  { "Meghon", 0.0 },
  { "Meghyn", 0.0 },
  { "Megin", 0.0 },
  { "Megna", 0.0 },
  { "Megnan", 0.0 },
  { "Megon", 0.0 },
  { "Meguel", 1.0 },
  { "Megumi", 0.0 },
  { "Megyn", 0.0 },
  { "Meha", 0.0 },
  { "Mehaan", 1.0 },
  { "Mehak", 0.0 },
  { "Mehakpreet", 0.0 },
  { "Mehan", 0.18518518 },
  { "Mehana", 0.0 },
  { "Mehar", 0.09848485 },
  { "Mehdi", 1.0 },
  { "Mehek", 0.0 },
  { "Meher", 0.011547344 },
  { "Mehgan", 0.0 },
  { "Mehjabeen", 0.0 },
  { "Mehjabin", 0.0 },
  { "Mehkai", 1.0 },
  { "Mehki", 1.0 },
  { "Mehlani", 0.0 },
  { "Mehmed", 1.0 },
  { "Mehmet", 1.0 },
  { "Mehnaz", 0.0 },
  { "Mehr", 0.0 },
  { "Mehrab", 1.0 },
  { "Mehran", 1.0 },
  { "Mehrdad", 1.0 },
  { "Mehreen", 0.0 },
  { "Mehret", 0.0 },
  { "Mehrimah", 0.0 },
  { "Mehtaab", 1.0 },
  { "Mehtab", 1.0 },
  { "Mehul", 1.0 },
  { "Mehvish", 0.0 },
  { "Mehwish", 0.0 },
  { "Mei", 0.0 },
  { "Meia", 0.0 },
  { "Meiah", 0.0 },
  { "Meichele", 0.0 },
  { "Meichelle", 0.0 },
  { "Meier", 1.0 },
  { "Meigan", 0.0 },
  { "Meigha", 0.0 },
  { "Meighan", 0.0 },
  { "Meika", 0.0 },
  { "Meikah", 0.0 },
  { "Meike", 0.0 },
  { "Meikhi", 1.0 },
  { "Meiko", 0.3862069 },
  { "Meila", 0.0 },
  { "Meilah", 0.0 },
  { "Meilahni", 0.0 },
  { "Meilan", 0.0 },
  { "Meilani", 0.0 },
  { "Meilanie", 0.0 },
  { "Meilany", 0.0 },
  { "Meile", 0.0 },
  { "Meilech", 1.0 },
  { "Meilee", 0.0 },
  { "Meili", 0.0 },
  { "Meilich", 1.0 },
  { "Meilin", 0.0 },
  { "Meiling", 0.0 },
  { "Meily", 0.0 },
  { "Meilyn", 0.0 },
  { "Meilynn", 0.0 },
  { "Meina", 0.0 },
  { "Meinrad", 1.0 },
  { "Meiqi", 0.0 },
  { "Meir", 1.0 },
  { "Meira", 0.0 },
  { "Meirah", 0.0 },
  { "Meirav", 0.0 },
  { "Meisha", 0.0 },
  { "Meison", 1.0 },
  { "Meissa", 0.0 },
  { "Meital", 0.0 },
  { "Meixi", 0.0 },
  { "Meiya", 0.0 },
  { "Meiyah", 0.0 },
  { "Meiyi", 0.0 },
  { "Meka", 0.0 },
  { "Mekaal", 1.0 },
  { "Mekael", 1.0 },
  { "Mekaela", 0.0 },
  { "Mekah", 0.0 },
  { "Mekai", 1.0 },
  { "Mekail", 1.0 },
  { "Mekaila", 0.0 },
  { "Mekal", 1.0 },
  { "Mekala", 0.0 },
  { "Mekalah", 0.0 },
  { "Mekayla", 0.0 },
  { "Mekaylah", 0.0 },
  { "Mekea", 0.0 },
  { "Mekeba", 0.0 },
  { "Mekeia", 0.0 },
  { "Mekeisha", 0.0 },
  { "Mekel", 0.8888889 },
  { "Mekela", 0.0 },
  { "Mekell", 1.0 },
  { "Mekella", 0.0 },
  { "Mekelle", 0.0 },
  { "Mekena", 0.0 },
  { "Mekenna", 0.0 },
  { "Mekensie", 0.0 },
  { "Mekenzi", 0.0 },
  { "Mekenzie", 0.0 },
  { "Mekenzy", 0.0 },
  { "Mekesha", 0.0 },
  { "Mekeshia", 0.0 },
  { "Mekhai", 1.0 },
  { "Mekhi", 0.98842806 },
  { "Mekhia", 0.0 },
  { "Mekhiah", 0.0 },
  { "Mekhiya", 0.0 },
  { "Mekhya", 0.0 },
  { "Meki", 1.0 },
  { "Mekia", 0.0 },
  { "Mekiah", 0.375 },
  { "Mekiel", 1.0 },
  { "Mekih", 1.0 },
  { "Mekisha", 0.0 },
  { "Mekiya", 0.0 },
  { "Mekiyah", 0.0 },
  { "Mekka", 0.0 },
  { "Mekkah", 0.0 },
  { "Mekkhi", 1.0 },
  { "Mekko", 1.0 },
  { "Meklit", 0.0 },
  { "Meko", 0.5707965 },
  { "Mekos", 1.0 },
  { "Mekya", 0.0 },
  { "Mekyah", 0.0 },
  { "Mekyla", 0.0 },
  { "Mekyle", 1.0 },
  { "Mel", 0.9017211 },
  { "Mela", 0.0 },
  { "Melachi", 1.0 },
  { "Melad", 1.0 },
  { "Meladee", 0.0 },
  { "Meladie", 0.0 },
  { "Melady", 0.0 },
  { "Melah", 0.0 },
  { "Melahni", 0.0 },
  { "Melaina", 0.0 },
  { "Melaine", 0.0 },
  { "Melainie", 0.0 },
  { "Melakai", 1.0 },
  { "Melakhi", 1.0 },
  { "Melaki", 1.0 },
  { "Melaku", 1.0 },
  { "Melan", 0.0 },
  { "Melana", 0.0 },
  { "Melandie", 0.0 },
  { "Melandy", 0.0 },
  { "Melane", 0.0 },
  { "Melanee", 0.0 },
  { "Melaney", 0.0 },
  { "Melani", 0.0 },
  { "Melania", 0.0 },
  { "Melanie", 0.0025294726 },
  { "Melanieann", 0.0 },
  { "Melaniece", 0.0 },
  { "Melaniee", 0.0 },
  { "Melanierose", 0.0 },
  { "Melanin", 0.0 },
  { "Melanne", 0.0 },
  { "Melanney", 0.0 },
  { "Melanni", 0.0 },
  { "Melannie", 0.0 },
  { "Melanny", 0.0 },
  { "Melantha", 0.0 },
  { "Melany", 0.00040054476 },
  { "Melanya", 0.0 },
  { "Melanye", 0.0 },
  { "Melarie", 0.0 },
  { "Melasia", 0.0 },
  { "Melat", 0.0 },
  { "Melaya", 0.0 },
  { "Melayah", 0.0 },
  { "Melayna", 0.0 },
  { "Melaynie", 0.0 },
  { "Melaysia", 0.0 },
  { "Melba", 0.0014906975 },
  { "Melbalene", 0.0 },
  { "Melbern", 1.0 },
  { "Melbert", 1.0 },
  { "Melbin", 1.0 },
  { "Melborn", 1.0 },
  { "Melbourn", 1.0 },
  { "Melbourne", 1.0 },
  { "Melbra", 0.0 },
  { "Melburn", 1.0 },
  { "Melchi", 1.0 },
  { "Melchior", 1.0 },
  { "Melchizedek", 1.0 },
  { "Melchor", 1.0 },
  { "Melchora", 0.0 },
  { "Melcina", 0.0 },
  { "Melda", 0.0 },
  { "Meldia", 0.0 },
  { "Meldin", 1.0 },
  { "Meldon", 1.0 },
  { "Meldora", 0.0 },
  { "Meldoy", 0.0 },
  { "Meldrick", 1.0 },
  { "Mele", 0.0 },
  { "Melea", 0.0 },
  { "Meleah", 0.0 },
  { "Meleana", 0.0 },
  { "Meleane", 0.0 },
  { "Melech", 1.0 },
  { "Melecia", 0.0 },
  { "Melecio", 1.0 },
  { "Meledy", 0.0 },
  { "Meleena", 0.0 },
  { "Meleesa", 0.0 },
  { "Meleia", 0.0 },
  { "Meleigha", 0.0 },
  { "Melek", 0.19711539 },
  { "Meleka", 0.0 },
  { "Melena", 0.0 },
  { "Melenaite", 0.0 },
  { "Melenda", 0.0 },
  { "Melene", 0.0 },
  { "Meleni", 0.0 },
  { "Melenie", 0.0 },
  { "Meleny", 0.0 },
  { "Melesa", 0.0 },
  { "Melesha", 0.0 },
  { "Melesia", 0.0 },
  { "Melesio", 1.0 },
  { "Melessa", 0.0 },
  { "Melessia", 0.0 },
  { "Meleta", 0.0 },
  { "Melette", 0.0 },
  { "Meleya", 0.0 },
  { "Meleyah", 0.0 },
  { "Melford", 1.0 },
  { "Melfred", 1.0 },
  { "Meli", 0.0 },
  { "Melia", 0.0 },
  { "Meliah", 0.0 },
  { "Meliame", 0.0 },
  { "Melian", 0.0 },
  { "Meliana", 0.0 },
  { "Melianna", 0.0 },
  { "Melica", 0.0 },
  { "Melicent", 0.0 },
  { "Melicia", 0.0 },
  { "Melida", 0.0 },
  { "Melieck", 1.0 },
  { "Meliek", 1.0 },
  { "Melih", 1.0 },
  { "Meliha", 0.0 },
  { "Melik", 1.0 },
  { "Melika", 0.0 },
  { "Melike", 0.0 },
  { "Melina", 0.0 },
  { "Melinah", 0.0 },
  { "Melinda", 0.0024992975 },
  { "Melindasue", 0.0 },
  { "Melindy", 0.0 },
  { "Meline", 0.0 },
  { "Melinee", 0.0 },
  { "Melinna", 0.0 },
  { "Melio", 1.0 },
  { "Melique", 1.0 },
  { "Melis", 0.0 },
  { "Melisa", 0.0005377501 },
  { "Melisaa", 0.0 },
  { "Melisande", 0.0 },
  { "Melise", 0.0 },
  { "Melisha", 0.0 },
  { "Melishia", 0.0 },
  { "Melisia", 0.0 },
  { "Meliss", 0.0 },
  { "Melissa", 0.0033082124 },
  { "Melissaann", 0.0 },
  { "Melissaanne", 0.0 },
  { "Melissamarie", 0.0 },
  { "Melissasue", 0.0 },
  { "Melisse", 0.0 },
  { "Melissha", 0.0 },
  { "Melissia", 0.0 },
  { "Melissie", 0.0 },
  { "Melisssa", 0.0 },
  { "Melita", 0.0 },
  { "Meliton", 1.0 },
  { "Melitta", 0.0 },
  { "Melitza", 0.0 },
  { "Melivn", 1.0 },
  { "Meliya", 0.0 },
  { "Meliyah", 0.0 },
  { "Meliza", 0.0 },
  { "Melizza", 0.0 },
  { "Melkin", 1.0 },
  { "Melky", 1.0 },
  { "Mell", 0.6286765 },
  { "Mella", 0.0 },
  { "Mellani", 0.0 },
  { "Mellanie", 0.0 },
  { "Mellany", 0.0 },
  { "Melle", 0.0 },
  { "Meller", 0.0 },
  { "Mellette", 0.0 },
  { "Mellicent", 0.0 },
  { "Mellie", 0.0 },
  { "Mellina", 0.0 },
  { "Mellinda", 0.0 },
  { "Mellisa", 0.0 },
  { "Mellisha", 0.0 },
  { "Mellisia", 0.0 },
  { "Mellissa", 0.0 },
  { "Mellissia", 0.0 },
  { "Mellodee", 0.0 },
  { "Mellodie", 0.0 },
  { "Mellody", 0.0 },
  { "Mellon", 1.0 },
  { "Melloney", 0.0 },
  { "Mellonie", 0.0 },
  { "Mellony", 0.0 },
  { "Mellow", 0.0 },
  { "Mellssa", 0.0 },
  { "Melly", 0.0 },
  { "Mellyora", 0.0 },
  { "Mellyssa", 0.0 },
  { "Melma", 0.0 },
  { "Melna", 0.0 },
  { "Melo", 1.0 },
  { "Melode", 0.0 },
  { "Melodee", 0.0 },
  { "Melodey", 0.0 },
  { "Melodi", 0.0 },
  { "Melodia", 0.0 },
  { "Melodie", 0.0 },
  { "Melody", 0.0017769495 },
  { "Melodye", 0.0 },
  { "Melodyrose", 0.0 },
  { "Melondy", 0.0 },
  { "Melonee", 0.0 },
  { "Meloney", 0.0 },
  { "Meloni", 0.0 },
  { "Melonia", 0.0 },
  { "Melonie", 0.0 },
  { "Melony", 0.0 },
  { "Melonye", 0.0 },
  { "Melora", 0.0 },
  { "Melorie", 0.0 },
  { "Melquan", 1.0 },
  { "Melquiades", 1.0 },
  { "Melquisedec", 1.0 },
  { "Melrose", 0.05704698 },
  { "Melroy", 1.0 },
  { "Melsa", 0.0 },
  { "Melssia", 0.0 },
  { "Melster", 0.0 },
  { "Meltem", 0.0 },
  { "Melton", 1.0 },
  { "Melva", 0.0021165514 },
  { "Melvalene", 0.0 },
  { "Melvan", 1.0 },
  { "Melveen", 0.0 },
  { "Melven", 1.0 },
  { "Melvena", 0.0 },
  { "Melvene", 0.0 },
  { "Melvenia", 0.0 },
  { "Melverine", 0.0 },
  { "Melvern", 1.0 },
  { "Melvia", 0.0 },
  { "Melvie", 0.0 },
  { "Melville", 0.9938677 },
  { "Melvin", 0.99124056 },
  { "Melvina", 0.0 },
  { "Melvine", 0.084745765 },
  { "Melvinia", 0.0 },
  { "Melvis", 0.4489796 },
  { "Melvita", 0.0 },
  { "Melvon", 1.0 },
  { "Melvonia", 0.0 },
  { "Melvyn", 0.99768627 },
  { "Mely", 0.0 },
  { "Melyna", 0.0 },
  { "Melynda", 0.0 },
  { "Melynie", 0.0 },
  { "Melynn", 0.0 },
  { "Melynna", 0.0 },
  { "Melysa", 0.0 },
  { "Melyssa", 0.0 },
  { "Melzie", 0.0 },
  { "Melzina", 0.0 },
  { "Memarie", 0.0 },
  { "Meme", 0.0 },
  { "Memoree", 0.0 },
  { "Memori", 0.0 },
  { "Memorie", 0.0 },
  { "Memory", 0.004946414 },
  { "Memphis", 0.8210452 },
  { "Memphys", 0.77272725 },
  { "Mena", 0.10922659 },
  { "Menaal", 0.0 },
  { "Menachem", 1.0 },
  { "Menachemmendel", 1.0 },
  { "Menah", 0.0 },
  { "Menal", 0.0 },
  { "Menasha", 1.0 },
  { "Menashe", 1.0 },
  { "Menda", 0.0 },
  { "Mendal", 1.0 },
  { "Mende", 0.0 },
  { "Mendee", 0.0 },
  { "Mendeecee", 1.0 },
  { "Mendeecees", 1.0 },
  { "Mendel", 1.0 },
  { "Mendell", 1.0 },
  { "Mendi", 0.0 },
  { "Mendie", 0.0 },
  { "Mendy", 0.17839482 },
  { "Menelek", 1.0 },
  { "Menelik", 1.0 },
  { "Meng", 0.97711265 },
  { "Menika", 0.0 },
  { "Menley", 0.0 },
  { "Menna", 0.0 },
  { "Mennah", 0.0 },
  { "Mennie", 0.0 },
  { "Menno", 1.0 },
  { "Menphis", 1.0 },
  { "Mensah", 1.0 },
  { "Mentha", 0.0 },
  { "Mentie", 0.0 },
  { "Menucha", 0.0 },
  { "Menyon", 0.0 },
  { "Menzo", 1.0 },
  { "Meosha", 0.0 },
  { "Meoshi", 0.0 },
  { "Meoshia", 0.0 },
  { "Mera", 0.0 },
  { "Merab", 0.0 },
  { "Meradith", 0.0 },
  { "Meraiah", 0.0 },
  { "Meraj", 1.0 },
  { "Meral", 0.0 },
  { "Merald", 1.0 },
  { "Merali", 0.0 },
  { "Meraly", 0.0 },
  { "Meranda", 0.0 },
  { "Merari", 0.013322231 },
  { "Meraris", 0.0 },
  { "Merary", 0.0 },
  { "Merav", 0.0 },
  { "Meraya", 0.0 },
  { "Mercades", 0.0 },
  { "Mercadez", 0.0 },
  { "Mercadies", 0.0 },
  { "Merce", 0.5 },
  { "Merced", 0.80600214 },
  { "Merceda", 0.0 },
  { "Mercedas", 0.0 },
  { "Mercede", 0.0 },
  { "Mercedee", 0.0 },
  { "Mercedees", 0.0 },
  { "Mercedeez", 0.0 },
  { "Mercedes", 0.008231463 },
  { "Mercedese", 0.0 },
  { "Mercedez", 0.0 },
  { "Mercedi", 0.0 },
  { "Mercedies", 0.0 },
  { "Mercedita", 0.0 },
  { "Mercedys", 0.0 },
  { "Mercer", 0.88662535 },
  { "Merchant", 1.0 },
  { "Merchelle", 0.0 },
  { "Merci", 0.0 },
  { "Mercia", 0.0 },
  { "Mercides", 0.0 },
  { "Mercie", 0.0 },
  { "Mercile", 0.0 },
  { "Mercille", 0.0 },
  { "Mercury", 0.86178863 },
  { "Mercutio", 1.0 },
  { "Mercy", 0.006451613 },
  { "Mercygrace", 0.0 },
  { "Merdell", 0.0 },
  { "Merdice", 0.0 },
  { "Merdie", 0.0 },
  { "Merdis", 0.0 },
  { "Merdith", 0.504 },
  { "Merecedes", 0.0 },
  { "Meredeth", 0.0 },
  { "Meredith", 0.055399355 },
  { "Meredy", 0.0 },
  { "Meredyth", 0.0 },
  { "Meredythe", 0.0 },
  { "Merek", 1.0 },
  { "Merel", 0.8541667 },
  { "Merelene", 0.0 },
  { "Merelin", 0.0 },
  { "Merely", 0.0 },
  { "Merelyn", 0.0 },
  { "Merena", 0.0 },
  { "Merenda", 0.0 },
  { "Meret", 0.0 },
  { "Merete", 0.0 },
  { "Merette", 0.0 },
  { "Mergeron", 1.0 },
  { "Mergim", 1.0 },
  { "Meri", 0.0 },
  { "Meria", 0.0 },
  { "Meriah", 0.0 },
  { "Merial", 0.0 },
  { "Meriam", 0.0 },
  { "Merian", 0.0 },
  { "Meriann", 0.0 },
  { "Merianne", 0.0 },
  { "Meribah", 0.0 },
  { "Meribeth", 0.0 },
  { "Meric", 1.0 },
  { "Merica", 0.0 },
  { "Merick", 1.0 },
  { "Merida", 0.0061124694 },
  { "Meridee", 0.0 },
  { "Meridel", 0.0 },
  { "Merideth", 0.0034562212 },
  { "Meridian", 0.0 },
  { "Meridith", 0.011976048 },
  { "Meridth", 0.0 },
  { "Meridy", 0.0 },
  { "Meridyth", 0.0 },
  { "Merie", 0.0 },
  { "Meriel", 0.0 },
  { "Merielle", 0.0 },
  { "Meriem", 0.0 },
  { "Merik", 1.0 },
  { "Merika", 0.0 },
  { "Merikay", 0.0 },
  { "Meril", 0.8666667 },
  { "Merilda", 0.0 },
  { "Merilee", 0.0 },
  { "Merilou", 0.0 },
  { "Merilyn", 0.0 },
  { "Merilynn", 0.0 },
  { "Merima", 0.0 },
  { "Merin", 0.0 },
  { "Merina", 0.0 },
  { "Merinda", 0.0 },
  { "Merion", 0.5672727 },
  { "Meris", 0.0 },
  { "Merisa", 0.0 },
  { "Merisha", 0.0 },
  { "Merissa", 0.0 },
  { "Merit", 0.41666666 },
  { "Merita", 0.0 },
  { "Meritt", 0.4117647 },
  { "Meritta", 0.0 },
  { "Meritxell", 0.0 },
  { "Meritza", 0.0 },
  { "Merium", 0.0 },
  { "Meriwether", 0.0 },
  { "Meriya", 0.0 },
  { "Meriyah", 0.0 },
  { "Merjem", 0.0 },
  { "Merl", 0.90252453 },
  { "Merla", 0.0 },
  { "Merlan", 1.0 },
  { "Merland", 1.0 },
  { "Merle", 0.673862 },
  { "Merleah", 0.0 },
  { "Merlean", 0.0 },
  { "Merlee", 0.0 },
  { "Merleen", 0.0 },
  { "Merlen", 1.0 },
  { "Merlena", 0.0 },
  { "Merlene", 0.0 },
  { "Merley", 0.33333334 },
  { "Merli", 0.0 },
  { "Merliah", 0.0 },
  { "Merlie", 0.0 },
  { "Merlin", 0.9476988 },
  { "Merlina", 0.0 },
  { "Merlinda", 0.0 },
  { "Merline", 0.0 },
  { "Merlisa", 0.0 },
  { "Merlon", 0.9723757 },
  { "Merlot", 0.0 },
  { "Merly", 0.0 },
  { "Merlyn", 0.6853545 },
  { "Merlyne", 0.0 },
  { "Merlynn", 0.0 },
  { "Merna", 0.0 },
  { "Mernie", 0.0 },
  { "Merola", 0.0 },
  { "Merolla", 0.0 },
  { "Meron", 0.05612245 },
  { "Merranda", 0.0 },
  { "Merredith", 0.0 },
  { "Merrel", 0.9628253 },
  { "Merrell", 0.8823976 },
  { "Merri", 0.0 },
  { "Merriah", 0.0 },
  { "Merrial", 0.0 },
  { "Merriam", 0.0 },
  { "Merrian", 0.0 },
  { "Merriann", 0.0 },
  { "Merrianne", 0.0 },
  { "Merribeth", 0.0 },
  { "Merric", 1.0 },
  { "Merrick", 0.9136742 },
  { "Merridee", 0.0 },
  { "Merrideth", 0.0 },
  { "Merridith", 0.0 },
  { "Merridy", 0.0 },
  { "Merrie", 0.0 },
  { "Merriel", 0.125 },
  { "Merrik", 0.97727275 },
  { "Merrikay", 0.0 },
  { "Merril", 0.86491936 },
  { "Merrilee", 0.0 },
  { "Merrilie", 0.0 },
  { "Merrill", 0.8652506 },
  { "Merrillyn", 0.0 },
  { "Merrilu", 0.0 },
  { "Merrily", 0.0 },
  { "Merrilyn", 0.0 },
  { "Merrilynn", 0.0 },
  { "Merriman", 1.0 },
  { "Merrin", 0.0 },
  { "Merrisa", 0.0 },
  { "Merrissa", 0.0 },
  { "Merrit", 0.71428573 },
  { "Merritt", 0.7614351 },
  { "Merrium", 0.0 },
  { "Merrlyn", 0.0 },
  { "Merry", 0.0 },
  { "Merryl", 0.0 },
  { "Merrylee", 0.0 },
  { "Merryn", 0.0 },
  { "Mersadees", 0.0 },
  { "Mersades", 0.0 },
  { "Mersadez", 0.0 },
  { "Mersadie", 0.0 },
  { "Mersadies", 0.0 },
  { "Mersadiez", 0.0 },
  { "Mersaydes", 0.0 },
  { "Mersaydez", 0.0 },
  { "Mersedes", 0.0 },
  { "Mersiha", 0.0 },
  { "Mert", 1.0 },
  { "Merta", 0.0 },
  { "Merten", 1.0 },
  { "Mertha", 0.0 },
  { "Mertice", 0.0 },
  { "Mertie", 0.0 },
  { "Mertin", 1.0 },
  { "Mertis", 0.021008404 },
  { "Mertle", 0.0 },
  { "Merton", 1.0 },
  { "Meru", 1.0 },
  { "Merv", 1.0 },
  { "Merva", 0.0 },
  { "Mervat", 0.0 },
  { "Merve", 0.0 },
  { "Merveille", 0.0 },
  { "Mervel", 1.0 },
  { "Mervil", 1.0 },
  { "Merville", 1.0 },
  { "Mervin", 0.9967903 },
  { "Mervyn", 0.99684143 },
  { "Merwin", 0.9969325 },
  { "Merwyn", 0.9874372 },
  { "Mery", 0.0 },
  { "Meryah", 0.0 },
  { "Meryam", 0.0 },
  { "Meryem", 0.0 },
  { "Meryk", 1.0 },
  { "Meryl", 0.07953497 },
  { "Meryle", 0.074592076 },
  { "Meryll", 0.0 },
  { "Meryn", 0.0 },
  { "Meryssa", 0.0 },
  { "Merzedes", 0.0 },
  { "Mesa", 0.0 },
  { "Mesai", 1.0 },
  { "Mesan", 0.0 },
  { "Mescal", 0.0 },
  { "Meschelle", 0.0 },
  { "Mesha", 0.0 },
  { "Meshach", 1.0 },
  { "Meshack", 1.0 },
  { "Meshae", 0.0 },
  { "Meshal", 1.0 },
  { "Meshalia", 0.0 },
  { "Meshan", 0.0 },
  { "Meshaun", 0.0 },
  { "Meshawn", 0.0 },
  { "Meshay", 0.0 },
  { "Meshayla", 0.0 },
  { "Meshea", 0.0 },
  { "Meshel", 0.0 },
  { "Meshell", 0.0 },
  { "Meshelle", 0.0 },
  { "Meshia", 0.0 },
  { "Meshilem", 1.0 },
  { "Meshon", 0.0 },
  { "Meshonda", 0.0 },
  { "Meshulem", 1.0 },
  { "Mesiah", 0.9794239 },
  { "Mesina", 0.0 },
  { "Mesiyah", 0.5714286 },
  { "Messai", 1.0 },
  { "Messer", 1.0 },
  { "Messi", 1.0 },
  { "Messiah", 0.9201726 },
  { "Messias", 1.0 },
  { "Messina", 0.0 },
  { "Messiyah", 0.8245614 },
  { "Messyah", 1.0 },
  { "Mesut", 1.0 },
  { "Meta", 0.0 },
  { "Mete", 1.0 },
  { "Metehan", 1.0 },
  { "Metha", 0.0 },
  { "Methel", 0.0 },
  { "Method", 1.0 },
  { "Methyl", 0.0 },
  { "Meti", 0.0 },
  { "Metin", 1.0 },
  { "Metra", 0.0 },
  { "Metro", 1.0 },
  { "Metta", 0.0 },
  { "Mette", 0.0 },
  { "Metthew", 1.0 },
  { "Mettie", 0.0 },
  { "Metz", 1.0 },
  { "Metzi", 0.0 },
  { "Metzli", 0.0 },
  { "Metzly", 0.0 },
  { "Metztli", 0.0 },
  { "Meuy", 0.0 },
  { "Mevelyn", 0.0 },
  { "Mextli", 0.0 },
  { "Mey", 0.0 },
  { "Meya", 0.0 },
  { "Meyah", 0.0 },
  { "Meyana", 0.0 },
  { "Meyanna", 0.0 },
  { "Meyer", 0.9830974 },
  { "Meyers", 1.0 },
  { "Meygan", 0.0 },
  { "Meyla", 0.0 },
  { "Meylani", 0.0 },
  { "Meyli", 0.0 },
  { "Meylin", 0.0 },
  { "Meyling", 0.0 },
  { "Meyly", 0.0 },
  { "Meyosha", 0.0 },
  { "Meyra", 0.0 },
  { "Meyson", 1.0 },
  { "Meziah", 1.0 },
  { "Mezmariah", 0.0 },
  { "Meztli", 0.0 },
  { "Meztly", 0.0 },
  { "Mezziah", 1.0 },
  { "Mguadalupe", 0.0 },
  { "Mhairi", 0.0 },
  { "Mhari", 0.0 },
  { "Mher", 1.0 },
  { "Mhia", 0.0 },
  { "Mhina", 0.0 },
  { "Mhya", 0.0 },
  { "Mi", 0.0 },
  { "Mia", 0.0013188262 },
  { "Miaa", 0.0 },
  { "Miaamor", 0.0 },
  { "Miaandrea", 0.0 },
  { "Miabella", 0.0 },
  { "Miabelle", 0.0 },
  { "Miachel", 1.0 },
  { "Miagrace", 0.0 },
  { "Miah", 0.0013691129 },
  { "Miaha", 0.0 },
  { "Miaisabella", 0.0 },
  { "Miaisha", 0.0 },
  { "Miaja", 0.0 },
  { "Miajah", 0.0 },
  { "Miakayla", 0.0 },
  { "Miakka", 0.0 },
  { "Miakoda", 0.0 },
  { "Miakota", 0.0 },
  { "Mialani", 0.0 },
  { "Mialee", 0.0 },
  { "Mialuna", 0.0 },
  { "Mialyn", 0.0 },
  { "Mialynn", 0.0 },
  { "Miamarie", 0.0 },
  { "Miami", 0.0 },
  { "Miamor", 0.0 },
  { "Miamore", 0.0 },
  { "Miamour", 0.0 },
  { "Mian", 0.91803277 },
  { "Miana", 0.0 },
  { "Mianca", 0.0 },
  { "Miangel", 0.0 },
  { "Miani", 0.0 },
  { "Mianicole", 0.0 },
  { "Mianna", 0.0 },
  { "Miara", 0.0 },
  { "Miarae", 0.0 },
  { "Miarah", 0.0 },
  { "Miari", 0.0 },
  { "Miarose", 0.0 },
  { "Miasha", 0.0 },
  { "Miasia", 0.0 },
  { "Miasophia", 0.0 },
  { "Miata", 0.0 },
  { "Miatta", 0.0 },
  { "Miavalentina", 0.0 },
  { "Miavictoria", 0.0 },
  { "Miaya", 0.0 },
  { "Miaysia", 0.0 },
  { "Mic", 1.0 },
  { "Mica", 0.23322299 },
  { "Micaden", 1.0 },
  { "Micael", 1.0 },
  { "Micaela", 0.00046342783 },
  { "Micaelah", 0.0 },
  { "Micaella", 0.0 },
  { "Micah", 0.88075477 },
  { "Micahel", 1.0 },
  { "Micahi", 1.0 },
  { "Micahia", 0.0 },
  { "Micahjoseph", 1.0 },
  { "Micahla", 0.0 },
  { "Micahya", 0.0 },
  { "Micai", 1.0 },
  { "Micaiah", 0.63374734 },
  { "Micaih", 1.0 },
  { "Micaila", 0.0 },
  { "Micailah", 0.0 },
  { "Micajah", 1.0 },
  { "Mical", 0.67346936 },
  { "Micala", 0.0 },
  { "Micalah", 0.0 },
  { "Micale", 0.71428573 },
  { "Micaleb", 1.0 },
  { "Micalyn", 0.0 },
  { "Mican", 1.0 },
  { "Micaya", 0.0 },
  { "Micayah", 0.0 },
  { "Micayla", 0.0 },
  { "Micaylah", 0.0 },
  { "Micca", 0.0 },
  { "Miccah", 0.85714287 },
  { "Micchael", 1.0 },
  { "Micco", 1.0 },
  { "Miceala", 0.0 },
  { "Micel", 0.0 },
  { "Micelle", 0.0 },
  { "Mich", 1.0 },
  { "Micha", 0.4614279 },
  { "Michaael", 1.0 },
  { "Michae", 0.9052823 },
  { "Michaeal", 1.0 },
  { "Michaee", 1.0 },
  { "Michaeel", 1.0 },
  { "Michael", 0.99502826 },
  { "Michaela", 0.0013987715 },
  { "Michaelah", 0.0 },
  { "Michaelallen", 1.0 },
  { "Michaelandrew", 1.0 },
  { "Michaelangel", 1.0 },
  { "Michaelangelo", 1.0 },
  { "Michaelann", 0.0 },
  { "Michaelanne", 0.0 },
  { "Michaelanthony", 1.0 },
  { "Michaelchristop", 1.0 },
  { "Michaeldavid", 1.0 },
  { "Michaele", 0.011431682 },
  { "Michaelee", 0.0 },
  { "Michaeleen", 0.0 },
  { "Michaelena", 0.0 },
  { "Michaelene", 0.0 },
  { "Michaelgabriel", 1.0 },
  { "Michaelia", 0.0 },
  { "Michaelin", 0.0 },
  { "Michaelina", 0.0 },
  { "Michaeline", 0.0 },
  { "Michaeljames", 1.0 },
  { "Michaeljay", 1.0 },
  { "Michaeljohn", 1.0 },
  { "Michaeljoseph", 1.0 },
  { "Michaeljr", 1.0 },
  { "Michaell", 0.9877451 },
  { "Michaella", 0.0 },
  { "Michaelle", 0.0 },
  { "Michaellee", 1.0 },
  { "Michaelpatrick", 1.0 },
  { "Michaelpaul", 1.0 },
  { "Michaelray", 1.0 },
  { "Michaelryan", 1.0 },
  { "Michaels", 1.0 },
  { "Michaelthomas", 1.0 },
  { "Michaelvincent", 1.0 },
  { "Michaelyn", 0.0 },
  { "Michaelynn", 0.0 },
  { "Michah", 0.9522472 },
  { "Michai", 1.0 },
  { "Michaia", 0.0 },
  { "Michaiah", 0.19213974 },
  { "Michail", 1.0 },
  { "Michaila", 0.0 },
  { "Michal", 0.49952245 },
  { "Michala", 0.0 },
  { "Michalah", 0.0 },
  { "Michalann", 0.0 },
  { "Michale", 0.904524 },
  { "Michalea", 0.0 },
  { "Michaleen", 0.0 },
  { "Michalena", 0.0 },
  { "Michalene", 0.0 },
  { "Michalia", 0.0 },
  { "Michalina", 0.0 },
  { "Michaline", 0.0 },
  { "Michall", 1.0 },
  { "Michalla", 0.0 },
  { "Michalle", 0.0 },
  { "Michalyn", 0.0 },
  { "Micharl", 1.0 },
  { "Michaya", 0.0 },
  { "Michayla", 0.0 },
  { "Michaylah", 0.0 },
  { "Miche", 0.0 },
  { "Michea", 0.0 },
  { "Micheal", 0.9881532 },
  { "Micheala", 0.0 },
  { "Michealangelo", 1.0 },
  { "Michealanthony", 1.0 },
  { "Micheale", 0.0 },
  { "Michealene", 0.0 },
  { "Michealla", 0.0 },
  { "Michealle", 0.0 },
  { "Micheaux", 1.0 },
  { "Micheil", 1.0 },
  { "Michel", 0.7473677 },
  { "Michela", 0.0 },
  { "Michelangelo", 1.0 },
  { "Michelann", 0.0 },
  { "Michele", 0.011865834 },
  { "Michelee", 0.0 },
  { "Micheleen", 0.0 },
  { "Michelena", 0.0 },
  { "Michelene", 0.0 },
  { "Michelin", 0.0 },
  { "Michelina", 0.0 },
  { "Micheline", 0.0 },
  { "Michell", 0.07827476 },
  { "Michella", 0.0 },
  { "Michelle", 0.004009558 },
  { "Michelleann", 0.0 },
  { "Michelleanne", 0.0 },
  { "Michellee", 0.0 },
  { "Michellele", 0.0 },
  { "Michellemarie", 0.0 },
  { "Michellene", 0.0 },
  { "Michelli", 0.0 },
  { "Michellie", 0.0 },
  { "Michellle", 0.0 },
  { "Michelly", 0.0 },
  { "Michelyn", 0.0 },
  { "Micheyla", 0.0 },
  { "Michi", 0.05263158 },
  { "Michia", 0.0 },
  { "Michiah", 0.40939596 },
  { "Michial", 1.0 },
  { "Michie", 0.058252428 },
  { "Michieal", 1.0 },
  { "Michiel", 0.9951691 },
  { "Michiele", 0.0 },
  { "Michielle", 0.0 },
  { "Michigan", 1.0 },
  { "Michiko", 0.0 },
  { "Michille", 0.0 },
  { "Michio", 1.0 },
  { "Michiya", 0.0 },
  { "Michiye", 0.0 },
  { "Michiyo", 0.0 },
  { "Michla", 0.0 },
  { "Michle", 1.0 },
  { "Michoel", 1.0 },
  { "Michol", 0.0 },
  { "Micholas", 1.0 },
  { "Michole", 0.0 },
  { "Michon", 0.049657535 },
  { "Michone", 0.0 },
  { "Michonne", 0.0 },
  { "Michuel", 1.0 },
  { "Michyla", 0.0 },
  { "Micia", 0.0 },
  { "Miciah", 0.5114345 },
  { "Mick", 1.0 },
  { "Mickael", 0.9922644 },
  { "Mickaela", 0.0 },
  { "Mickail", 1.0 },
  { "Mickaila", 0.0 },
  { "Mickal", 1.0 },
  { "Mickala", 0.0 },
  { "Mickale", 1.0 },
  { "Mickalla", 0.0 },
  { "Mickayla", 0.0 },
  { "Mickaylah", 0.0 },
  { "Mickeal", 1.0 },
  { "Mickel", 0.97597134 },
  { "Mickela", 0.0 },
  { "Mickelena", 0.0 },
  { "Mickelina", 0.0 },
  { "Mickell", 0.72727275 },
  { "Mickelle", 0.0 },
  { "Mickenna", 0.0 },
  { "Mickenzi", 0.0 },
  { "Mickenzie", 0.06779661 },
  { "Mickenzy", 0.0 },
  { "Mickey", 0.8065654 },
  { "Micki", 0.012401353 },
  { "Mickia", 0.0 },
  { "Mickie", 0.14824797 },
  { "Mickiel", 1.0 },
  { "Mickinley", 0.0 },
  { "Mickinzie", 0.0 },
  { "Mickle", 1.0 },
  { "Micky", 0.81525314 },
  { "Mickyla", 0.0 },
  { "Mickyle", 1.0 },
  { "Mico", 0.82857144 },
  { "Micol", 0.0 },
  { "Micole", 0.0 },
  { "Micquel", 1.0 },
  { "Mida", 0.0 },
  { "Midajah", 0.0 },
  { "Midas", 1.0 },
  { "Middie", 0.0 },
  { "Middleton", 1.0 },
  { "Midge", 0.0 },
  { "Midgie", 0.0 },
  { "Midhuna", 0.0 },
  { "Midian", 1.0 },
  { "Midna", 0.0 },
  { "Midnight", 0.0 },
  { "Midori", 0.0 },
  { "Midred", 0.0 },
  { "Mie", 0.0 },
  { "Mieasha", 0.0 },
  { "Miechelle", 0.0 },
  { "Miecislaus", 1.0 },
  { "Mieczyslaw", 1.0 },
  { "Mieisha", 0.0 },
  { "Mieka", 0.0 },
  { "Mieke", 0.0 },
  { "Mieko", 0.01724138 },
  { "Miel", 0.0 },
  { "Miela", 0.0 },
  { "Miella", 0.0 },
  { "Mielle", 0.0 },
  { "Miera", 0.0 },
  { "Mierra", 0.0 },
  { "Miesha", 0.0 },
  { "Mieshia", 0.0 },
  { "Miette", 0.0 },
  { "Mieya", 0.0 },
  { "Migdalia", 0.0011715089 },
  { "Migel", 1.0 },
  { "Migelangel", 1.0 },
  { "Miggie", 0.0 },
  { "Migna", 0.0 },
  { "Mignon", 0.0 },
  { "Mignonette", 0.0 },
  { "Mignonne", 0.0 },
  { "Migual", 1.0 },
  { "Miguel", 0.99434334 },
  { "Miguela", 0.0 },
  { "Miguelangel", 1.0 },
  { "Miguelangelo", 1.0 },
  { "Miguelantonio", 1.0 },
  { "Miguelina", 0.0 },
  { "Miguelito", 1.0 },
  { "Miha", 0.0 },
  { "Mihael", 1.0 },
  { "Mihaela", 0.0 },
  { "Mihai", 1.0 },
  { "Mihail", 1.0 },
  { "Mihailo", 1.0 },
  { "Mihajlo", 1.0 },
  { "Mihcael", 1.0 },
  { "Mihika", 0.0 },
  { "Mihir", 1.0 },
  { "Mihira", 0.0 },
  { "Mihit", 1.0 },
  { "Miho", 0.0 },
  { "Mihran", 1.0 },
  { "Mihrimah", 0.0 },
  { "Miia", 0.0 },
  { "Miiangel", 0.0 },
  { "Miichael", 1.0 },
  { "Miika", 0.0 },
  { "Miiking", 1.0 },
  { "Miila", 0.0 },
  { "Miina", 0.0 },
  { "Mija", 0.0 },
  { "Mijael", 1.0 },
  { "Mijah", 0.0 },
  { "Mijoi", 0.0 },
  { "Mijoy", 0.0 },
  { "Mika", 0.15953061 },
  { "Mikaal", 1.0 },
  { "Mikaeel", 1.0 },
  { "Mikael", 0.9345649 },
  { "Mikaela", 0.0 },
  { "Mikaelah", 0.0 },
  { "Mikaele", 1.0 },
  { "Mikaeli", 0.0 },
  { "Mikaella", 0.0 },
  { "Mikaelyn", 0.0 },
  { "Mikaeyla", 0.0 },
  { "Mikah", 0.62122875 },
  { "Mikahla", 0.0 },
  { "Mikai", 0.98050684 },
  { "Mikaia", 0.0 },
  { "Mikaiah", 0.12658228 },
  { "Mikail", 0.9818621 },
  { "Mikaila", 0.0 },
  { "Mikailah", 0.0 },
  { "Mikaili", 0.0 },
  { "Mikailyn", 0.0 },
  { "Mikaiya", 0.0 },
  { "Mikaiyah", 0.0 },
  { "Mikal", 0.8447663 },
  { "Mikala", 0.002523341 },
  { "Mikalah", 0.0 },
  { "Mikale", 0.89373296 },
  { "Mikalee", 0.0 },
  { "Mikali", 0.0 },
  { "Mikalia", 0.0 },
  { "Mikalla", 0.0 },
  { "Mikalya", 0.0 },
  { "Mikalyn", 0.0 },
  { "Mikalynn", 0.0 },
  { "Mikan", 0.0 },
  { "Mikara", 0.0 },
  { "Mikasia", 0.0 },
  { "Mikaya", 0.0 },
  { "Mikayah", 0.0 },
  { "Mikayala", 0.0 },
  { "Mikayeel", 1.0 },
  { "Mikayel", 1.0 },
  { "Mikayela", 0.0 },
  { "Mikayia", 0.0 },
  { "Mikayla", 0.00021221183 },
  { "Mikaylah", 0.0 },
  { "Mikayle", 0.083333336 },
  { "Mikaylee", 0.0 },
  { "Mikayleigh", 0.0 },
  { "Mikaylen", 0.0 },
  { "Mikayli", 0.0 },
  { "Mikaylia", 0.0 },
  { "Mikaylie", 0.0 },
  { "Mikaylin", 0.0 },
  { "Mikaylla", 0.0 },
  { "Mikaylyn", 0.0 },
  { "Mikaylynn", 0.0 },
  { "Mike", 0.9967679 },
  { "Mikea", 0.0 },
  { "Mikeal", 0.99177843 },
  { "Mikeala", 0.0 },
  { "Mikeayla", 0.0 },
  { "Mikecia", 0.0 },
  { "Mikee", 1.0 },
  { "Mikei", 1.0 },
  { "Mikeia", 0.0 },
  { "Mikeil", 1.0 },
  { "Mikeila", 0.0 },
  { "Mikeisha", 0.0 },
  { "Mikel", 0.9141366 },
  { "Mikela", 0.0 },
  { "Mikelann", 0.0 },
  { "Mikele", 0.39252338 },
  { "Mikelia", 0.0 },
  { "Mikell", 0.7223089 },
  { "Mikella", 0.0 },
  { "Mikelle", 0.020606061 },
  { "Mikelyn", 0.0 },
  { "Miken", 1.0 },
  { "Mikena", 0.0 },
  { "Mikenlee", 0.0 },
  { "Mikenley", 0.0 },
  { "Mikenna", 0.0 },
  { "Mikenya", 0.0 },
  { "Mikenzi", 0.0 },
  { "Mikenzie", 0.0 },
  { "Mikenzy", 0.0 },
  { "Mikequan", 1.0 },
  { "Mikera", 0.0 },
  { "Mikeria", 0.0 },
  { "Mikesha", 0.0 },
  { "Mikeshia", 0.0 },
  { "Miketa", 0.0 },
  { "Mikey", 0.99593824 },
  { "Mikeya", 0.0 },
  { "Mikeyah", 0.0 },
  { "Mikeyia", 0.0 },
  { "Mikeyla", 0.0 },
  { "Mikha", 1.0 },
  { "Mikhael", 0.9888112 },
  { "Mikhaela", 0.0 },
  { "Mikhai", 1.0 },
  { "Mikhail", 0.99256504 },
  { "Mikhaila", 0.0 },
  { "Mikhal", 0.88095236 },
  { "Mikhala", 0.0 },
  { "Mikhale", 1.0 },
  { "Mikhayla", 0.0 },
  { "Mikhel", 1.0 },
  { "Mikhi", 1.0 },
  { "Mikhia", 0.0 },
  { "Mikhiya", 0.0 },
  { "Miki", 0.04082774 },
  { "Mikia", 0.0 },
  { "Mikiah", 0.24029575 },
  { "Mikial", 1.0 },
  { "Mikiala", 0.0 },
  { "Mikias", 1.0 },
  { "Mikie", 0.866426 },
  { "Mikiel", 1.0 },
  { "Mikiela", 0.0 },
  { "Mikiko", 0.0 },
  { "Mikila", 0.0 },
  { "Mikilah", 0.0 },
  { "Miking", 1.0 },
  { "Mikinlee", 0.0 },
  { "Mikinley", 0.0 },
  { "Mikinzi", 0.0 },
  { "Mikinzie", 0.0 },
  { "Mikio", 1.0 },
  { "Mikira", 0.0 },
  { "Mikisha", 0.0 },
  { "Mikita", 0.0 },
  { "Mikiya", 0.0 },
  { "Mikiyah", 0.0 },
  { "Mikiyas", 1.0 },
  { "Mikka", 0.0 },
  { "Mikkala", 0.0 },
  { "Mikkayla", 0.0 },
  { "Mikkel", 0.96821195 },
  { "Mikki", 0.0053346264 },
  { "Mikkia", 0.0 },
  { "Mikko", 0.9796748 },
  { "Mikkos", 1.0 },
  { "Mikle", 1.0 },
  { "Miklo", 1.0 },
  { "Miklos", 1.0 },
  { "Miko", 0.5095477 },
  { "Mikol", 1.0 },
  { "Mikolaj", 1.0 },
  { "Mikolas", 1.0 },
  { "Mikos", 1.0 },
  { "Mikoto", 0.0 },
  { "Miku", 0.0 },
  { "Mikya", 0.0 },
  { "Mikyah", 0.0 },
  { "Mikye", 1.0 },
  { "Mikyia", 0.0 },
  { "Mikyla", 0.0 },
  { "Mikylah", 0.0 },
  { "Mikyle", 1.0 },
  { "Mikylee", 0.0 },
  { "Mikylie", 0.0 },
  { "Mikylla", 0.0 },
  { "Mikynlee", 0.0 },
  { "Mikyra", 0.0 },
  { "Mila", 0.0010987401 },
  { "Milaan", 0.0 },
  { "Milad", 1.0 },
  { "Milada", 0.0 },
  { "Miladeen", 0.0 },
  { "Miladie", 0.0 },
  { "Milady", 0.0 },
  { "Milagro", 0.01779603 },
  { "Milagros", 0.00054247584 },
  { "Milah", 0.0 },
  { "Milahn", 0.0 },
  { "Milahni", 0.0 },
  { "Milaina", 0.0 },
  { "Milam", 1.0 },
  { "Milamarie", 0.0 },
  { "Milan", 0.6220979 },
  { "Milana", 0.0 },
  { "Milanee", 0.0 },
  { "Milani", 0.0 },
  { "Milania", 0.0 },
  { "Milanie", 0.0 },
  { "Milanii", 0.0 },
  { "Milaniya", 0.0 },
  { "Milanna", 0.0 },
  { "Milanni", 0.0 },
  { "Milano", 1.0 },
  { "Milany", 0.0 },
  { "Milanya", 0.0 },
  { "Milarae", 0.0 },
  { "Milarain", 0.0 },
  { "Milard", 1.0 },
  { "Milarose", 0.0 },
  { "Milas", 1.0 },
  { "Milasia", 0.0 },
  { "Milaun", 0.0 },
  { "Milauni", 0.0 },
  { "Milay", 0.0 },
  { "Milaya", 0.0 },
  { "Milayah", 0.0 },
  { "Milayna", 0.0 },
  { "Milaysia", 0.0 },
  { "Milbern", 1.0 },
  { "Milbert", 1.0 },
  { "Milbra", 0.0 },
  { "Milbrey", 0.0 },
  { "Milburn", 1.0 },
  { "Milburne", 1.0 },
  { "Milca", 0.0 },
  { "Milcah", 0.0 },
  { "Milda", 0.0 },
  { "Milderd", 0.0 },
  { "Mildra", 0.0 },
  { "Mildre", 0.0 },
  { "Mildred", 0.0035014146 },
  { "Mildren", 0.0 },
  { "Mildreth", 0.0 },
  { "Mildrid", 0.0 },
  { "Mile", 1.0 },
  { "Milea", 0.0 },
  { "Mileah", 0.0 },
  { "Milee", 0.0 },
  { "Mileen", 0.0 },
  { "Mileena", 0.0 },
  { "Milei", 0.0 },
  { "Mileidi", 0.0 },
  { "Mileidy", 0.0 },
  { "Mileigh", 0.0 },
  { "Milek", 1.0 },
  { "Mileke", 1.0 },
  { "Milele", 0.0 },
  { "Milen", 0.79591835 },
  { "Milena", 0.0 },
  { "Milene", 0.0 },
  { "Mileni", 0.0 },
  { "Milenia", 0.0 },
  { "Milenka", 0.0 },
  { "Milenko", 1.0 },
  { "Milenna", 0.0 },
  { "Mileny", 0.0 },
  { "Miles", 0.99727815 },
  { "Milesha", 0.0 },
  { "Milessa", 0.0 },
  { "Milett", 0.0 },
  { "Milette", 0.0 },
  { "Mileva", 0.0 },
  { "Miley", 0.006729475 },
  { "Mileya", 0.0 },
  { "Mileyah", 0.0 },
  { "Mileydi", 0.0 },
  { "Mileyna", 0.0 },
  { "Mileyshka", 0.0 },
  { "Milez", 1.0 },
  { "Milferd", 1.0 },
  { "Milford", 0.99841523 },
  { "Milfred", 1.0 },
  { "Mili", 0.0 },
  { "Milia", 0.0 },
  { "Miliah", 0.0 },
  { "Milian", 0.17391305 },
  { "Miliana", 0.0 },
  { "Miliani", 0.0 },
  { "Milianna", 0.0 },
  { "Miliano", 1.0 },
  { "Miliany", 0.0 },
  { "Milianys", 0.0 },
  { "Milica", 0.0 },
  { "Milicent", 0.0 },
  { "Milie", 0.0 },
  { "Milik", 1.0 },
  { "Milika", 0.0 },
  { "Mililani", 0.0 },
  { "Milin", 1.0 },
  { "Milina", 0.0 },
  { "Milind", 1.0 },
  { "Milinda", 0.0 },
  { "Milini", 0.0 },
  { "Milio", 1.0 },
  { "Milira", 0.0 },
  { "Milisa", 0.0 },
  { "Milisha", 0.0 },
  { "Milisia", 0.0 },
  { "Milissa", 0.0 },
  { "Milissia", 0.0 },
  { "Militza", 0.0 },
  { "Miliyah", 0.0 },
  { "Milka", 0.0 },
  { "Milla", 0.0 },
  { "Millage", 1.0 },
  { "Millah", 0.0 },
  { "Millan", 0.9230769 },
  { "Millana", 0.0 },
  { "Millani", 0.0 },
  { "Millard", 0.99802774 },
  { "Mille", 0.0 },
  { "Milledge", 1.0 },
  { "Millee", 0.0 },
  { "Milleigh", 0.0 },
  { "Millen", 1.0 },
  { "Millena", 0.0 },
  { "Millenia", 0.0 },
  { "Millenium", 0.0 },
  { "Millenna", 0.0 },
  { "Millennia", 0.0 },
  { "Millennium", 0.0 },
  { "Miller", 0.90128136 },
  { "Millerd", 1.0 },
  { "Millette", 0.0 },
  { "Milley", 0.0 },
  { "Milli", 0.0 },
  { "Millia", 0.0 },
  { "Millian", 0.0 },
  { "Milliana", 0.0 },
  { "Milliani", 0.0 },
  { "Millianna", 0.0 },
  { "Milliard", 1.0 },
  { "Millicent", 0.0 },
  { "Millie", 0.0072532324 },
  { "Millinia", 0.0 },
  { "Million", 0.95522386 },
  { "Milliona", 0.0 },
  { "Millis", 1.0 },
  { "Millisa", 0.0 },
  { "Millison", 0.0 },
  { "Millissa", 0.0 },
  { "Mills", 0.9694657 },
  { "Milly", 0.0 },
  { "Millyana", 0.0 },
  { "Milner", 1.0 },
  { "Milnor", 1.0 },
  { "Milo", 0.99134463 },
  { "Miloh", 1.0 },
  { "Milon", 1.0 },
  { "Milorad", 1.0 },
  { "Milos", 1.0 },
  { "Milosh", 1.0 },
  { "Milosz", 1.0 },
  { "Milous", 1.0 },
  { "Milove", 0.0 },
  { "Milred", 0.0 },
  { "Milroy", 1.0 },
  { "Milt", 1.0 },
  { "Milta", 0.0 },
  { "Milthon", 1.0 },
  { "Milton", 0.9947544 },
  { "Milus", 1.0 },
  { "Milven", 1.0 },
  { "Milvia", 0.0 },
  { "Milvin", 1.0 },
  { "Mily", 0.0 },
  { "Milyana", 0.0 },
  { "Milyn", 0.0 },
  { "Milynda", 0.0 },
  { "Milynn", 0.0 },
  { "Mima", 0.0 },
  { "Mimi", 0.0 },
  { "Mimia", 0.0 },
  { "Mimie", 0.0 },
  { "Mimma", 0.0 },
  { "Mimmie", 0.0 },
  { "Mimose", 0.0 },
  { "Mims", 1.0 },
  { "Min", 0.61488676 },
  { "Mina", 0.03523889 },
  { "Minaal", 0.0 },
  { "Minah", 0.0 },
  { "Minahil", 0.0 },
  { "Minako", 0.0 },
  { "Minal", 0.0 },
  { "Minami", 0.0 },
  { "Minard", 1.0 },
  { "Minas", 1.0 },
  { "Minato", 1.0 },
  { "Minaya", 0.0 },
  { "Minda", 0.0 },
  { "Minday", 0.0 },
  { "Minde", 0.0 },
  { "Mindee", 0.0 },
  { "Mindel", 0.0 },
  { "Minden", 0.0 },
  { "Mindi", 0.0 },
  { "Mindie", 0.0 },
  { "Mindy", 0.0016000383 },
  { "Minea", 0.0 },
  { "Mineko", 0.0 },
  { "Minela", 0.0 },
  { "Minelba", 0.0 },
  { "Minelly", 0.0 },
  { "Minelva", 0.0 },
  { "Minely", 0.0 },
  { "Mineola", 0.0 },
  { "Miner", 1.0 },
  { "Minerba", 0.0 },
  { "Minerva", 0.0 },
  { "Minervia", 0.0 },
  { "Minesh", 1.0 },
  { "Minesha", 0.0 },
  { "Minetta", 0.0 },
  { "Minette", 0.0 },
  { "Ming", 0.6948052 },
  { "Minga", 0.0 },
  { "Mingo", 1.0 },
  { "Mingus", 1.0 },
  { "Minh", 0.87 },
  { "Minha", 0.0 },
  { "Minhanh", 0.0 },
  { "Minhchau", 0.0 },
  { "Minho", 1.0 },
  { "Minhquan", 1.0 },
  { "Minhthu", 0.0 },
  { "Minhtri", 1.0 },
  { "Mini", 0.0 },
  { "Minica", 0.0 },
  { "Minie", 0.0 },
  { "Minika", 0.0 },
  { "Minique", 0.0 },
  { "Minisha", 0.0 },
  { "Miniya", 0.0 },
  { "Miniyah", 0.0 },
  { "Minjae", 1.0 },
  { "Minji", 0.0 },
  { "Minjoon", 1.0 },
  { "Minka", 0.0 },
  { "Minna", 0.0 },
  { "Minnah", 0.0 },
  { "Minne", 0.0 },
  { "Minneola", 0.0 },
  { "Minnetta", 0.0 },
  { "Minnette", 0.0 },
  { "Minnia", 0.0 },
  { "Minnie", 0.0049343547 },
  { "Minnielee", 0.0 },
  { "Minnis", 1.0 },
  { "Minola", 0.0 },
  { "Minor", 0.9930995 },
  { "Minori", 0.0 },
  { "Minoru", 1.0 },
  { "Minos", 1.0 },
  { "Minot", 1.0 },
  { "Minsa", 0.0 },
  { "Minseo", 0.0 },
  { "Minta", 0.0 },
  { "Minter", 1.0 },
  { "Mintha", 0.0 },
  { "Mintie", 0.0 },
  { "Minton", 1.0 },
  { "Minus", 1.0 },
  { "Minwoo", 1.0 },
  { "Minya", 0.0 },
  { "Minyard", 1.0 },
  { "Minyon", 0.0 },
  { "Mio", 0.14785992 },
  { "Miona", 0.0 },
  { "Mionna", 0.0 },
  { "Miori", 0.0 },
  { "Miosha", 0.0 },
  { "Miosoti", 0.0 },
  { "Miosotis", 0.0 },
  { "Miosoty", 0.0 },
  { "Miquan", 1.0 },
  { "Miqueas", 1.0 },
  { "Miquel", 0.9551657 },
  { "Miquela", 0.0 },
  { "Miquella", 0.0 },
  { "Miquelle", 0.0 },
  { "Miquesha", 0.0 },
  { "Miquez", 1.0 },
  { "Miquisha", 0.0 },
  { "Mir", 1.0 },
  { "Mira", 0.00058011373 },
  { "Miraan", 1.0 },
  { "Mirabai", 0.0 },
  { "Mirabel", 0.0 },
  { "Mirabell", 0.0 },
  { "Mirabella", 0.0 },
  { "Mirabelle", 0.0 },
  { "Mirac", 1.0 },
  { "Miracal", 0.0 },
  { "Miracle", 0.018913971 },
  { "Mirae", 0.0 },
  { "Mirage", 0.03787879 },
  { "Mirah", 0.0 },
  { "Miraha", 0.0 },
  { "Mirai", 0.0 },
  { "Miraj", 0.90384614 },
  { "Mirajane", 0.0 },
  { "Mirakal", 0.0 },
  { "Mirakel", 0.0 },
  { "Mirakle", 0.0 },
  { "Miral", 0.0 },
  { "Miralee", 0.0 },
  { "Miram", 0.0 },
  { "Miran", 1.0 },
  { "Miranda", 0.0016703786 },
  { "Mirandah", 0.0 },
  { "Mirandy", 0.0 },
  { "Mirannda", 0.0 },
  { "Mirari", 0.0 },
  { "Mirasol", 0.0 },
  { "Miray", 0.0 },
  { "Miraya", 0.0 },
  { "Mirayah", 0.0 },
  { "Mirca", 0.0 },
  { "Mircale", 0.0 },
  { "Mircle", 0.0 },
  { "Mirei", 0.0 },
  { "Mireia", 0.0 },
  { "Mireida", 0.0 },
  { "Mireille", 0.0 },
  { "Mireily", 0.0 },
  { "Mirel", 0.0 },
  { "Mirela", 0.0 },
  { "Mirella", 0.0 },
  { "Mirelle", 0.0 },
  { "Mirely", 0.0 },
  { "Mirelys", 0.0 },
  { "Miren", 0.0 },
  { "Mirena", 0.0 },
  { "Mirenda", 0.0 },
  { "Mireya", 0.0 },
  { "Mireyah", 0.0 },
  { "Mireyda", 0.0 },
  { "Mireyli", 0.0 },
  { "Mireyna", 0.0 },
  { "Mirha", 0.0 },
  { "Miri", 0.0 },
  { "Miria", 0.0 },
  { "Miriah", 0.0 },
  { "Miriam", 0.0025657772 },
  { "Miriama", 0.0 },
  { "Mirian", 0.0 },
  { "Miriana", 0.0 },
  { "Mirianna", 0.0 },
  { "Mirical", 0.0 },
  { "Miricale", 0.0 },
  { "Miricle", 0.0 },
  { "Miriel", 0.0 },
  { "Miriella", 0.0 },
  { "Mirielle", 0.0 },
  { "Mirin", 0.0 },
  { "Mirina", 0.0 },
  { "Mirinda", 0.0 },
  { "Mirisa", 0.0 },
  { "Mirissa", 0.0 },
  { "Mirium", 0.0 },
  { "Miriya", 0.0 },
  { "Mirjana", 0.0 },
  { "Mirka", 0.0 },
  { "Mirko", 1.0 },
  { "Mirl", 0.6126761 },
  { "Mirla", 0.0 },
  { "Mirlande", 0.0 },
  { "Mirle", 0.0 },
  { "Mirlene", 0.0 },
  { "Mirlinda", 0.0 },
  { "Mirna", 0.0012771392 },
  { "Mirnes", 1.0 },
  { "Miro", 1.0 },
  { "Mirola", 0.0 },
  { "Miron", 1.0 },
  { "Mironda", 0.0 },
  { "Miroslav", 1.0 },
  { "Miroslava", 0.0 },
  { "Miroslaw", 1.0 },
  { "Mirra", 0.0 },
  { "Mirracle", 0.0 },
  { "Mirrah", 0.0 },
  { "Mirranda", 0.0 },
  { "Mirren", 0.0 },
  { "Mirriah", 0.0 },
  { "Mirriam", 0.0 },
  { "Mirsa", 0.0 },
  { "Mirsab", 1.0 },
  { "Mirsad", 1.0 },
  { "Mirsha", 0.0 },
  { "Mirta", 0.0 },
  { "Mirth", 0.0 },
  { "Mirtha", 0.0 },
  { "Mirtie", 0.0 },
  { "Mirtis", 0.0 },
  { "Mirtle", 0.0 },
  { "Mirya", 0.0 },
  { "Miryah", 0.0 },
  { "Miryam", 0.0 },
  { "Miryan", 0.0 },
  { "Mirycal", 0.0 },
  { "Miryea", 0.0 },
  { "Mirza", 0.7769517 },
  { "Misa", 0.0 },
  { "Misae", 0.0 },
  { "Misael", 0.99791723 },
  { "Misaki", 0.0 },
  { "Misako", 0.0 },
  { "Misao", 0.0 },
  { "Misato", 0.0 },
  { "Misaye", 0.0 },
  { "Misayo", 0.0 },
  { "Misbah", 0.040650405 },
  { "Mischa", 0.11117974 },
  { "Mischel", 0.0 },
  { "Mischele", 0.0 },
  { "Mischell", 0.0 },
  { "Mischelle", 0.0 },
  { "Misel", 0.0 },
  { "Misgana", 0.0 },
  { "Misha", 0.101747975 },
  { "Mishaal", 0.0 },
  { "Mishael", 0.6542969 },
  { "Mishaela", 0.0 },
  { "Mishaila", 0.0 },
  { "Mishal", 0.029069768 },
  { "Mishalay", 0.0 },
  { "Mishale", 0.0 },
  { "Mishall", 0.0 },
  { "Mishary", 1.0 },
  { "Mishaun", 1.0 },
  { "Mishawn", 0.37433156 },
  { "Mishay", 0.0 },
  { "Mishayla", 0.0 },
  { "Mishea", 0.0 },
  { "Misheel", 0.0 },
  { "Misheeta", 0.0 },
  { "Mishel", 0.0 },
  { "Mishele", 0.0 },
  { "Mishell", 0.0 },
  { "Mishelle", 0.0 },
  { "Mishia", 0.0 },
  { "Mishie", 0.0 },
  { "Mishika", 0.0 },
  { "Mishita", 0.0 },
  { "Mishka", 0.0 },
  { "Mishon", 0.6888889 },
  { "Mishonda", 0.0 },
  { "Mishti", 0.0 },
  { "Misk", 0.0 },
  { "Miski", 0.0 },
  { "Mison", 1.0 },
  { "Misri", 0.0 },
  { "Missael", 1.0 },
  { "Missel", 0.0 },
  { "Missey", 0.0 },
  { "Missi", 0.0 },
  { "Missiah", 1.0 },
  { "Missie", 0.0 },
  { "Missouri", 0.0 },
  { "Missouria", 0.0 },
  { "Missty", 0.0 },
  { "Missy", 0.0 },
  { "Mistala", 0.0 },
  { "Mistaya", 0.0 },
  { "Miste", 0.0 },
  { "Mistee", 0.0 },
  { "Mistelle", 0.0 },
  { "Mister", 1.0 },
  { "Mistey", 0.0 },
  { "Misti", 0.0 },
  { "Mistic", 0.0 },
  { "Mistica", 0.0 },
  { "Mistie", 0.0 },
  { "Mistina", 0.0 },
  { "Mistique", 0.0 },
  { "Misty", 0.0028018157 },
  { "Mistyann", 0.0 },
  { "Mistydawn", 0.0 },
  { "Mistye", 0.0 },
  { "Misue", 0.0 },
  { "Mit", 1.0 },
  { "Mita", 0.0 },
  { "Mital", 0.23809524 },
  { "Mitali", 0.0 },
  { "Mitch", 1.0 },
  { "Mitchael", 1.0 },
  { "Mitchal", 1.0 },
  { "Mitcheal", 1.0 },
  { "Mitchel", 0.9983274 },
  { "Mitchell", 0.9931165 },
  { "Mitchelle", 0.24169184 },
  { "Mitchum", 1.0 },
  { "Mitesh", 1.0 },
  { "Mithcell", 1.0 },
  { "Mithil", 1.0 },
  { "Mithila", 0.0 },
  { "Mithra", 0.0 },
  { "Mithran", 1.0 },
  { "Mithun", 1.0 },
  { "Mitia", 0.0 },
  { "Mitra", 0.00967118 },
  { "Mits", 1.0 },
  { "Mitsi", 0.0 },
  { "Mitsie", 0.0 },
  { "Mitsu", 0.0 },
  { "Mitsue", 0.0 },
  { "Mitsugi", 1.0 },
  { "Mitsuki", 0.0 },
  { "Mitsuko", 0.0 },
  { "Mitsuo", 1.0 },
  { "Mitsuru", 0.97159094 },
  { "Mitsuye", 0.0 },
  { "Mitsuyo", 0.0 },
  { "Mitsuyoshi", 1.0 },
  { "Mitsy", 0.0 },
  { "Mitt", 1.0 },
  { "Mittie", 0.0 },
  { "Mitty", 0.0 },
  { "Mitul", 1.0 },
  { "Mitza", 0.0 },
  { "Mitze", 0.0 },
  { "Mitzel", 0.0 },
  { "Mitzi", 0.0 },
  { "Mitzie", 0.0 },
  { "Mitzy", 0.0 },
  { "Miu", 0.0 },
  { "Miura", 0.0 },
  { "Mivaan", 1.0 },
  { "Miwa", 0.0 },
  { "Mixtli", 0.0 },
  { "Mixtly", 0.0 },
  { "Miya", 0.0 },
  { "Miyabi", 0.0 },
  { "Miyah", 0.0 },
  { "Miyako", 0.0 },
  { "Miyali", 0.0 },
  { "Miyana", 0.0 },
  { "Miyanah", 0.0 },
  { "Miyani", 0.0 },
  { "Miyanna", 0.0 },
  { "Miyari", 0.0 },
  { "Miye", 0.0 },
  { "Miyeko", 0.0 },
  { "Miyesha", 0.0 },
  { "Miyisha", 0.0 },
  { "Miykael", 1.0 },
  { "Miyla", 0.0 },
  { "Miylah", 0.0 },
  { "Miyo", 0.0 },
  { "Miyoka", 0.0 },
  { "Miyoko", 0.0 },
  { "Miyon", 1.0 },
  { "Miyona", 0.0 },
  { "Miyonna", 0.0 },
  { "Miyosha", 0.0 },
  { "Miyoshi", 0.0 },
  { "Miyu", 0.0 },
  { "Miyuki", 0.0 },
  { "Mizael", 1.0 },
  { "Mizan", 0.5 },
  { "Mizani", 0.0 },
  { "Mizell", 1.0 },
  { "Mizraim", 1.0 },
  { "Mizuki", 0.039711192 },
  { "Mj", 1.0 },
  { "Mkaela", 0.0 },
  { "Mkai", 1.0 },
  { "Mkaila", 0.0 },
  { "Mkayla", 0.0 },
  { "Mkaylah", 0.0 },
  { "Mkenna", 0.0 },
  { "Mkenzi", 0.0 },
  { "Mkenzie", 0.0 },
  { "Mkiyah", 0.0 },
  { "Mkya", 0.0 },
  { "Mkyla", 0.0 },
  { "Mlak", 0.0 },
  { "Mleah", 0.0 },
  { "Mlisa", 0.0 },
  { "Mliss", 0.0 },
  { "Mlissa", 0.0 },
  { "Mlynn", 0.0 },
  { "Mmark", 1.0 },
  { "Mmary", 0.0 },
  { "Mmichael", 1.0 },
  { "Mo", 0.6666667 },
  { "Moa", 1.0 },
  { "Moaad", 1.0 },
  { "Moaaz", 1.0 },
  { "Moad", 1.0 },
  { "Moana", 0.0 },
  { "Moani", 0.0 },
  { "Moataz", 1.0 },
  { "Moath", 1.0 },
  { "Moayad", 1.0 },
  { "Moayed", 1.0 },
  { "Moaz", 1.0 },
  { "Mobeen", 1.0 },
  { "Mobin", 1.0 },
  { "Mobley", 1.0 },
  { "Mobolaji", 1.0 },
  { "Mocha", 0.0 },
  { "Moctar", 1.0 },
  { "Moctezuma", 1.0 },
  { "Mode", 1.0 },
  { "Modean", 0.0 },
  { "Modell", 0.045918368 },
  { "Modelle", 0.0 },
  { "Modena", 0.0 },
  { "Modene", 0.0 },
  { "Modenia", 0.0 },
  { "Modesireoluwa", 0.0 },
  { "Modest", 0.0 },
  { "Modesta", 0.0 },
  { "Modeste", 0.0 },
  { "Modesti", 0.0 },
  { "Modestine", 0.0 },
  { "Modesto", 1.0 },
  { "Modesty", 0.0 },
  { "Modibo", 1.0 },
  { "Modie", 0.14285715 },
  { "Modine", 0.0 },
  { "Modou", 1.0 },
  { "Modupe", 0.0 },
  { "Modupeoluwa", 0.0 },
  { "Moe", 0.92286503 },
  { "Moeez", 1.0 },
  { "Moeisha", 0.0 },
  { "Moeko", 0.0 },
  { "Moena", 0.0 },
  { "Moesha", 0.0 },
  { "Moeshia", 0.0 },
  { "Moet", 0.0 },
  { "Moez", 1.0 },
  { "Mofeoluwa", 1.0 },
  { "Mofetoluwa", 0.0 },
  { "Moffett", 1.0 },
  { "Mofiyinfoluwa", 0.0 },
  { "Mohab", 1.0 },
  { "Mohagany", 0.0 },
  { "Mohamad", 1.0 },
  { "Mohamadali", 1.0 },
  { "Mohamadou", 1.0 },
  { "Mohamed", 1.0 },
  { "Mohamedali", 1.0 },
  { "Mohamedamiin", 1.0 },
  { "Mohamedamin", 1.0 },
  { "Mohamedamine", 1.0 },
  { "Mohammad", 0.99922454 },
  { "Mohammadali", 1.0 },
  { "Mohammed", 1.0 },
  { "Mohammedali", 1.0 },
  { "Mohamud", 1.0 },
  { "Mohan", 1.0 },
  { "Mohanad", 1.0 },
  { "Mohaned", 1.0 },
  { "Mohannad", 1.0 },
  { "Mohd", 1.0 },
  { "Mohib", 1.0 },
  { "Mohid", 1.0 },
  { "Mohini", 0.0 },
  { "Mohit", 1.0 },
  { "Mohmad", 1.0 },
  { "Mohmed", 1.0 },
  { "Mohmmad", 1.0 },
  { "Mohmmed", 1.0 },
  { "Mohnish", 1.0 },
  { "Mohogany", 0.0 },
  { "Mohsen", 1.0 },
  { "Mohsin", 1.0 },
  { "Moices", 1.0 },
  { "Moina", 0.0 },
  { "Moir", 1.0 },
  { "Moira", 0.0 },
  { "Moire", 0.0 },
  { "Moise", 1.0 },
  { "Moises", 0.99800664 },
  { "Moishe", 1.0 },
  { "Moishy", 1.0 },
  { "Moisses", 1.0 },
  { "Moiya", 0.0 },
  { "Moiz", 1.0 },
  { "Mojolaoluwa", 0.24 },
  { "Mojtaba", 1.0 },
  { "Moksh", 1.0 },
  { "Moksha", 0.0 },
  { "Mokshagna", 1.0 },
  { "Mokshith", 1.0 },
  { "Mokshitha", 0.0 },
  { "Molene", 0.0 },
  { "Molette", 0.0 },
  { "Molik", 1.0 },
  { "Molika", 0.0 },
  { "Molina", 0.0 },
  { "Molinda", 0.0 },
  { "Moline", 0.0 },
  { "Molique", 1.0 },
  { "Molissa", 0.0 },
  { "Molla", 0.0 },
  { "Mollee", 0.0 },
  { "Molleigh", 0.0 },
  { "Molley", 0.0 },
  { "Molli", 0.0 },
  { "Mollie", 0.0009944655 },
  { "Molloy", 1.0 },
  { "Molly", 0.0022245566 },
  { "Mollyann", 0.0 },
  { "Mollyanne", 0.0 },
  { "Mollye", 0.0 },
  { "Mollykate", 0.0 },
  { "Mollyrose", 0.0 },
  { "Molton", 1.0 },
  { "Momar", 1.0 },
  { "Momen", 1.0 },
  { "Momie", 0.0 },
  { "Momin", 1.0 },
  { "Momina", 0.0 },
  { "Momo", 0.0 },
  { "Momodou", 1.0 },
  { "Momoe", 0.0 },
  { "Momoka", 0.0 },
  { "Momoko", 0.0 },
  { "Momoyo", 0.0 },
  { "Mon", 1.0 },
  { "Mona", 0.0006897626 },
  { "Monaca", 0.0 },
  { "Monae", 0.0 },
  { "Monah", 0.0 },
  { "Monai", 0.0 },
  { "Monajah", 0.0 },
  { "Monalee", 0.0 },
  { "Monali", 0.0 },
  { "Monalisa", 0.0 },
  { "Monasha", 0.0 },
  { "Monasia", 0.0 },
  { "Monay", 0.0 },
  { "Monaya", 0.0 },
  { "Monaye", 0.0 },
  { "Monce", 0.0 },
  { "Moncerat", 0.0 },
  { "Moncerath", 0.0 },
  { "Moncerrad", 0.0 },
  { "Moncerrat", 0.0 },
  { "Moncerrath", 0.0 },
  { "Monchel", 0.0 },
  { "Monchell", 0.0 },
  { "Monchelle", 0.0 },
  { "Monchello", 1.0 },
  { "Moncheri", 0.0 },
  { "Moncia", 0.0 },
  { "Monda", 0.0 },
  { "Mondale", 1.0 },
  { "Mondarius", 1.0 },
  { "Monday", 0.0 },
  { "Mondell", 1.0 },
  { "Mondo", 1.0 },
  { "Mondra", 1.0 },
  { "Mondre", 1.0 },
  { "Mondrell", 1.0 },
  { "Mone", 0.0 },
  { "Monea", 0.0 },
  { "Moneak", 0.0 },
  { "Moneca", 0.0 },
  { "Monecia", 0.0 },
  { "Monee", 0.0 },
  { "Moneeb", 1.0 },
  { "Moneek", 0.0 },
  { "Moneeka", 0.0 },
  { "Moneer", 1.0 },
  { "Moneik", 0.0 },
  { "Moneika", 0.0 },
  { "Moneisha", 0.0 },
  { "Monek", 0.0 },
  { "Moneka", 0.0 },
  { "Moneke", 0.0 },
  { "Monekia", 0.0 },
  { "Monell", 0.2173913 },
  { "Monelle", 0.0 },
  { "Moneque", 0.0 },
  { "Monesha", 0.0 },
  { "Moneshia", 0.0 },
  { "Monet", 0.001429252 },
  { "Moneta", 0.0 },
  { "Monetta", 0.0 },
  { "Monette", 0.0 },
  { "Money", 0.27659574 },
  { "Monez", 0.0 },
  { "Monford", 1.0 },
  { "Mong", 0.8363636 },
  { "Moni", 0.0 },
  { "Monia", 0.0 },
  { "Monic", 0.0 },
  { "Monica", 0.0039534285 },
  { "Monice", 0.0 },
  { "Monicia", 0.0 },
  { "Monick", 0.0 },
  { "Monicka", 0.0 },
  { "Monico", 0.97786456 },
  { "Monie", 0.0 },
  { "Moniece", 0.0 },
  { "Moniesha", 0.0 },
  { "Monifa", 0.0 },
  { "Monifah", 0.0 },
  { "Monigue", 0.0 },
  { "Monik", 0.0 },
  { "Monika", 0.0 },
  { "Monikah", 0.0 },
  { "Monike", 0.0 },
  { "Monikia", 0.0 },
  { "Monina", 0.0 },
  { "Monioluwa", 0.0 },
  { "Moniq", 0.0 },
  { "Moniqua", 0.0 },
  { "Monique", 0.004792109 },
  { "Moniquea", 0.0 },
  { "Moniquie", 0.0 },
  { "Monir", 1.0 },
  { "Monira", 0.0 },
  { "Monisa", 0.0 },
  { "Monise", 0.0 },
  { "Monish", 1.0 },
  { "Monisha", 0.0 },
  { "Monissa", 0.0 },
  { "Monita", 0.0 },
  { "Moniya", 0.0 },
  { "Moniyah", 0.0 },
  { "Monja", 0.0 },
  { "Monna", 0.0 },
  { "Monnette", 0.0 },
  { "Monnica", 0.0 },
  { "Monnie", 0.12210128 },
  { "Monolito", 1.0 },
  { "Monque", 0.0 },
  { "Monquel", 1.0 },
  { "Monquez", 1.0 },
  { "Monquie", 0.0 },
  { "Monquita", 0.0 },
  { "Monrad", 1.0 },
  { "Monreaux", 0.0 },
  { "Monroe", 0.8710299 },
  { "Monsanto", 1.0 },
  { "Monse", 0.0 },
  { "Monserat", 0.0 },
  { "Monserath", 0.0 },
  { "Monseratt", 0.0 },
  { "Monserrad", 0.0 },
  { "Monserrat", 0.0040360135 },
  { "Monserrate", 0.21794872 },
  { "Monserrath", 0.0 },
  { "Monserratt", 0.0 },
  { "Monserratte", 0.0 },
  { "Monsita", 0.0 },
  { "Monson", 1.0 },
  { "Monsserat", 0.0 },
  { "Monsserrat", 0.0 },
  { "Mont", 1.0 },
  { "Monta", 0.3465418 },
  { "Montae", 1.0 },
  { "Montague", 1.0 },
  { "Montana", 0.2967464 },
  { "Montanah", 0.0 },
  { "Montanez", 1.0 },
  { "Montanna", 0.03153153 },
  { "Montaque", 1.0 },
  { "Montario", 1.0 },
  { "Montarious", 1.0 },
  { "Montarius", 1.0 },
  { "Montasia", 0.0 },
  { "Montavia", 0.0 },
  { "Montavian", 1.0 },
  { "Montavion", 1.0 },
  { "Montavious", 1.0 },
  { "Montavis", 1.0 },
  { "Montavius", 1.0 },
  { "Montay", 1.0 },
  { "Montaya", 0.0 },
  { "Montaye", 1.0 },
  { "Montaysia", 0.0 },
  { "Montaz", 1.0 },
  { "Monte", 0.9644324 },
  { "Montea", 0.5 },
  { "Monteco", 1.0 },
  { "Montee", 1.0 },
  { "Monteen", 0.0 },
  { "Montego", 1.0 },
  { "Montel", 1.0 },
  { "Montell", 1.0 },
  { "Montelle", 1.0 },
  { "Montene", 0.0 },
  { "Montera", 0.0 },
  { "Monterey", 0.0 },
  { "Monteria", 0.0 },
  { "Monterio", 1.0 },
  { "Monterion", 1.0 },
  { "Monterious", 1.0 },
  { "Monterius", 1.0 },
  { "Montero", 1.0 },
  { "Monterra", 0.0 },
  { "Monterrian", 1.0 },
  { "Monterrio", 1.0 },
  { "Monterrious", 1.0 },
  { "Monterrius", 1.0 },
  { "Montese", 1.0 },
  { "Montesha", 0.0 },
  { "Montessa", 0.0 },
  { "Montevious", 1.0 },
  { "Montey", 1.0 },
  { "Montez", 0.79646504 },
  { "Monteze", 1.0 },
  { "Montford", 1.0 },
  { "Montgomery", 0.93104255 },
  { "Monti", 0.65451056 },
  { "Montia", 0.0 },
  { "Montiara", 0.0 },
  { "Montica", 0.0 },
  { "Montico", 1.0 },
  { "Montie", 0.6440353 },
  { "Montiera", 0.0 },
  { "Montina", 0.0 },
  { "Montine", 0.0 },
  { "Montinique", 0.0 },
  { "Montique", 0.45333335 },
  { "Montisha", 0.0 },
  { "Montonio", 1.0 },
  { "Montoria", 0.0 },
  { "Montoya", 0.18622848 },
  { "Montoyia", 0.0 },
  { "Montra", 0.5 },
  { "Montrae", 1.0 },
  { "Montrail", 1.0 },
  { "Montral", 1.0 },
  { "Montravion", 1.0 },
  { "Montravious", 1.0 },
  { "Montravius", 1.0 },
  { "Montray", 1.0 },
  { "Montre", 1.0 },
  { "Montreal", 1.0 },
  { "Montrece", 0.0 },
  { "Montrel", 1.0 },
  { "Montrell", 0.99229926 },
  { "Montrelle", 1.0 },
  { "Montrese", 0.18518518 },
  { "Montressa", 0.0 },
  { "Montrevious", 1.0 },
  { "Montrey", 1.0 },
  { "Montrez", 1.0 },
  { "Montrice", 0.2972973 },
  { "Montrose", 0.5 },
  { "Montry", 0.0 },
  { "Montse", 0.0 },
  { "Montserat", 0.0 },
  { "Montserrad", 0.0 },
  { "Montserrat", 0.0 },
  { "Montserrath", 0.0 },
  { "Montsho", 1.0 },
  { "Monty", 0.9817479 },
  { "Montzerrat", 0.0 },
  { "Mony", 1.0 },
  { "Monya", 0.0 },
  { "Monyae", 0.07936508 },
  { "Monyca", 0.0 },
  { "Monye", 0.0 },
  { "Monyea", 0.0 },
  { "Monyetta", 0.0 },
  { "Monyette", 0.0 },
  { "Monyka", 0.0 },
  { "Monzell", 0.0 },
  { "Monzella", 0.0 },
  { "Monzelle", 0.0 },
  { "Monzerat", 0.0 },
  { "Monzerath", 0.0 },
  { "Monzeratt", 0.0 },
  { "Monzerrad", 0.0 },
  { "Monzerrat", 0.009881423 },
  { "Monzerrath", 0.0 },
  { "Moo", 0.58666664 },
  { "Moody", 1.0 },
  { "Moon", 0.2300885 },
  { "Moona", 0.0 },
  { "Moonyean", 0.0 },
  { "Moonyeen", 0.0 },
  { "Moore", 1.0 },
  { "Moorea", 0.0 },
  { "Moosa", 1.0 },
  { "Mor", 0.2777778 },
  { "Mora", 0.0 },
  { "Morad", 1.0 },
  { "Moraes", 1.0 },
  { "Morag", 0.0 },
  { "Morah", 0.0 },
  { "Moraima", 0.0 },
  { "Moral", 1.0 },
  { "Moran", 0.9122807 },
  { "Moranda", 0.0 },
  { "Moraya", 0.0 },
  { "Morayma", 0.0 },
  { "Morayo", 0.0 },
  { "Mordcha", 1.0 },
  { "Mordchai", 1.0 },
  { "Mordche", 1.0 },
  { "Mordecai", 1.0 },
  { "Mordechai", 1.0 },
  { "Mordechi", 1.0 },
  { "Mordekai", 1.0 },
  { "Mordy", 1.0 },
  { "Morea", 0.0 },
  { "Moree", 0.0 },
  { "Moreen", 0.0 },
  { "Moreland", 1.0 },
  { "Morelia", 0.0 },
  { "Morell", 1.0 },
  { "Morella", 0.0 },
  { "Morena", 0.0 },
  { "Morene", 0.0 },
  { "Morenike", 0.0 },
  { "Moreno", 1.0 },
  { "Moretta", 0.0 },
  { "Morey", 1.0 },
  { "Morgaine", 0.0 },
  { "Morgan", 0.16503468 },
  { "Morgana", 0.0 },
  { "Morgandy", 0.0 },
  { "Morgane", 0.0 },
  { "Morgann", 0.0 },
  { "Morganna", 0.0 },
  { "Morganne", 0.0 },
  { "Morgayne", 0.0 },
  { "Morgen", 0.2373673 },
  { "Morghan", 0.004048583 },
  { "Morghann", 0.0 },
  { "Morghen", 0.0 },
  { "Morghyn", 0.0 },
  { "Morgin", 0.0 },
  { "Morgon", 0.7062937 },
  { "Morgun", 1.0 },
  { "Morgyn", 0.0 },
  { "Morgynn", 0.0 },
  { "Mori", 0.2875 },
  { "Moria", 0.0 },
  { "Moriah", 0.008108341 },
  { "Moriana", 0.0 },
  { "Morica", 0.0 },
  { "Morice", 1.0 },
  { "Moriel", 1.0 },
  { "Morina", 0.0 },
  { "Morine", 0.0 },
  { "Morio", 1.0 },
  { "Morion", 1.0 },
  { "Morireoluwa", 0.42857143 },
  { "Moris", 1.0 },
  { "Morisa", 0.0 },
  { "Morise", 1.0 },
  { "Morissa", 0.0 },
  { "Moritz", 1.0 },
  { "Moriya", 0.0 },
  { "Moriyah", 0.0 },
  { "Morjorie", 0.0 },
  { "Morland", 1.0 },
  { "Morlene", 0.0 },
  { "Morley", 0.9468864 },
  { "Morlon", 1.0 },
  { "Morna", 0.0 },
  { "Morning", 0.0 },
  { "Morningstar", 0.0 },
  { "Morocco", 1.0 },
  { "Morolaoluwa", 0.0 },
  { "Moroni", 1.0 },
  { "Morpheus", 1.0 },
  { "Morrel", 1.0 },
  { "Morrell", 1.0 },
  { "Morrey", 1.0 },
  { "Morriah", 0.0 },
  { "Morrie", 1.0 },
  { "Morrigan", 0.0 },
  { "Morrighan", 0.0 },
  { "Morrill", 1.0 },
  { "Morrine", 0.0 },
  { "Morris", 0.99314576 },
  { "Morrisa", 0.0 },
  { "Morrise", 1.0 },
  { "Morrison", 0.9633621 },
  { "Morriss", 1.0 },
  { "Morrissa", 0.0 },
  { "Morrissey", 0.77272725 },
  { "Morrow", 1.0 },
  { "Morry", 1.0 },
  { "Morsal", 0.0 },
  { "Morse", 1.0 },
  { "Mort", 1.0 },
  { "Morten", 1.0 },
  { "Mortez", 1.0 },
  { "Morticia", 0.0 },
  { "Mortimer", 1.0 },
  { "Mortisha", 0.0 },
  { "Morton", 0.999594 },
  { "Morty", 1.0 },
  { "Mory", 1.0 },
  { "Moryah", 0.0 },
  { "Mosa", 1.0 },
  { "Mosby", 1.0 },
  { "Mose", 0.9929096 },
  { "Mosella", 0.0 },
  { "Moselle", 0.0 },
  { "Moses", 0.99881446 },
  { "Mosese", 1.0 },
  { "Mosetta", 0.0 },
  { "Mosha", 1.0 },
  { "Moshay", 0.0 },
  { "Moshe", 0.99832296 },
  { "Mosheh", 1.0 },
  { "Moshood", 1.0 },
  { "Mosi", 0.85897434 },
  { "Mosiah", 1.0 },
  { "Mosie", 0.5714286 },
  { "Mosley", 0.39583334 },
  { "Moss", 1.0 },
  { "Mosses", 1.0 },
  { "Mossie", 0.0047348486 },
  { "Mossimo", 1.0 },
  { "Mostafa", 1.0 },
  { "Mostyn", 1.0 },
  { "Motaz", 1.0 },
  { "Motez", 1.0 },
  { "Motie", 0.0 },
  { "Motley", 1.0 },
  { "Motoki", 1.0 },
  { "Mott", 1.0 },
  { "Motti", 1.0 },
  { "Motty", 1.0 },
  { "Motunrayo", 0.0 },
  { "Moua", 1.0 },
  { "Mouad", 1.0 },
  { "Mouctar", 1.0 },
  { "Moudie", 0.0 },
  { "Mouhamad", 1.0 },
  { "Mouhamadou", 1.0 },
  { "Mouhamed", 1.0 },
  { "Mouhammad", 1.0 },
  { "Moukthika", 0.0 },
  { "Moulton", 1.0 },
  { "Moultrie", 1.0 },
  { "Mouna", 0.0 },
  { "Mounia", 0.0 },
  { "Mounir", 1.0 },
  { "Mount", 1.0 },
  { "Mourad", 1.0 },
  { "Mourice", 1.0 },
  { "Mourine", 0.0 },
  { "Mourya", 1.0 },
  { "Mousa", 1.0 },
  { "Moussa", 1.0 },
  { "Moustafa", 1.0 },
  { "Moustapha", 1.0 },
  { "Moutaz", 1.0 },
  { "Movita", 0.0 },
  { "Mowgli", 1.0 },
  { "Mox", 1.0 },
  { "Moxen", 1.0 },
  { "Moxie", 0.0 },
  { "Moxley", 0.72727275 },
  { "Moxon", 1.0 },
  { "Moxxi", 0.0 },
  { "Moy", 0.0 },
  { "Moya", 0.0 },
  { "Moyer", 1.0 },
  { "Moyinoluwa", 0.0 },
  { "Moyosoreoluwa", 0.5 },
  { "Moyra", 0.0 },
  { "Moyses", 1.0 },
  { "Mozart", 1.0 },
  { "Mozel", 0.021929825 },
  { "Mozell", 0.1372771 },
  { "Mozella", 0.0 },
  { "Mozelle", 0.002062629 },
  { "Mozes", 1.0 },
  { "Mozetta", 0.0 },
  { "Moziah", 1.0 },
  { "Mozter", 0.0 },
  { "Mr", 1.0 },
  { "Mria", 0.0 },
  { "Mridula", 0.0 },
  { "Mrinalini", 0.0 },
  { "Mrk", 1.0 },
  { "Mry", 0.0 },
  { "Mryah", 0.0 },
  { "Mrytle", 0.0 },
  { "Mtisha", 0.0 },
  { "Mtthew", 1.0 },
  { "Mua", 1.0 },
  { "Muaad", 1.0 },
  { "Muaaz", 1.0 },
  { "Muad", 1.0 },
  { "Muadh", 1.0 },
  { "Muammar", 1.0 },
  { "Muath", 1.0 },
  { "Muaz", 1.0 },
  { "Mubarak", 1.0 },
  { "Mubarik", 1.0 },
  { "Mubashir", 1.0 },
  { "Mubeen", 1.0 },
  { "Mubin", 1.0 },
  { "Mubina", 0.0 },
  { "Mucaad", 1.0 },
  { "Mucad", 1.0 },
  { "Mudaser", 1.0 },
  { "Mudasir", 1.0 },
  { "Mudassir", 1.0 },
  { "Mueed", 1.0 },
  { "Mueez", 1.0 },
  { "Mufasa", 1.0 },
  { "Muffy", 0.0 },
  { "Mugdha", 0.0 },
  { "Mugen", 1.0 },
  { "Muggie", 0.0 },
  { "Mugilan", 1.0 },
  { "Mugisha", 1.0 },
  { "Muguette", 0.0 },
  { "Muhab", 1.0 },
  { "Muhaimin", 1.0 },
  { "Muhamad", 1.0 },
  { "Muhamadou", 1.0 },
  { "Muhamed", 1.0 },
  { "Muhammad", 1.0 },
  { "Muhammadali", 1.0 },
  { "Muhammadamin", 1.0 },
  { "Muhammadyusuf", 1.0 },
  { "Muhammed", 1.0 },
  { "Muhammedali", 1.0 },
  { "Muhanad", 1.0 },
  { "Muhannad", 1.0 },
  { "Muhib", 1.0 },
  { "Muhmmad", 1.0 },
  { "Muhsin", 1.0 },
  { "Muhtasim", 1.0 },
  { "Mui", 0.0 },
  { "Muireann", 0.0 },
  { "Muiz", 1.0 },
  { "Muizz", 1.0 },
  { "Mujahid", 1.0 },
  { "Mujtaba", 1.0 },
  { "Mukarram", 1.0 },
  { "Mukesh", 1.0 },
  { "Mukhtar", 1.0 },
  { "Mukil", 1.0 },
  { "Mukta", 0.0 },
  { "Muktar", 1.0 },
  { "Mukul", 1.0 },
  { "Mukund", 1.0 },
  { "Mukunda", 1.0 },
  { "Mulan", 0.021097047 },
  { "Mulani", 0.0 },
  { "Muldrow", 1.0 },
  { "Mulford", 1.0 },
  { "Mulki", 0.0 },
  { "Mumin", 1.0 },
  { "Mumina", 0.0 },
  { "Mumtahina", 0.0 },
  { "Mumtas", 0.0 },
  { "Mumtaz", 0.0 },
  { "Mun", 1.0 },
  { "Muna", 0.0 },
  { "Munachi", 0.36666667 },
  { "Munachimso", 0.3970588 },
  { "Munachiso", 0.27419356 },
  { "Munajj", 1.0 },
  { "Munasar", 1.0 },
  { "Mundi", 0.0 },
  { "Mundy", 0.0 },
  { "Muneeb", 1.0 },
  { "Muneer", 1.0 },
  { "Muneera", 0.0 },
  { "Muneerah", 0.0 },
  { "Muneo", 1.0 },
  { "Mung", 1.0 },
  { "Munha", 0.0 },
  { "Munib", 1.0 },
  { "Muniba", 0.0 },
  { "Munir", 1.0 },
  { "Munira", 0.0 },
  { "Munirah", 0.0 },
  { "Munisa", 0.0 },
  { "Muniza", 0.0 },
  { "Munro", 1.0 },
  { "Munroe", 1.0 },
  { "Munson", 1.0 },
  { "Muntaha", 0.0 },
  { "Muntas", 0.0 },
  { "Muntasir", 1.0 },
  { "Muntaz", 0.0 },
  { "Mura", 0.0 },
  { "Murad", 1.0 },
  { "Mural", 0.0 },
  { "Murali", 1.0 },
  { "Muranda", 0.0 },
  { "Murari", 1.0 },
  { "Murat", 1.0 },
  { "Murdie", 0.0 },
  { "Murdis", 0.0 },
  { "Murdoc", 1.0 },
  { "Murdoch", 1.0 },
  { "Murdock", 1.0 },
  { "Murel", 0.68265164 },
  { "Murell", 1.0 },
  { "Muriah", 0.0 },
  { "Murial", 0.0 },
  { "Murice", 1.0 },
  { "Muriel", 0.013753519 },
  { "Murielle", 0.0 },
  { "Muril", 0.6511628 },
  { "Murilo", 1.0 },
  { "Murissa", 0.0 },
  { "Murjani", 0.0 },
  { "Murl", 0.7561356 },
  { "Murland", 1.0 },
  { "Murle", 0.5978723 },
  { "Murlean", 0.0 },
  { "Murlee", 0.0 },
  { "Murleen", 0.0 },
  { "Murlen", 1.0 },
  { "Murlene", 0.0 },
  { "Murlie", 0.0 },
  { "Murlin", 1.0 },
  { "Murline", 0.0 },
  { "Murlyn", 0.6451613 },
  { "Murna", 0.0 },
  { "Murnie", 0.0 },
  { "Murphee", 0.0 },
  { "Murphey", 0.625 },
  { "Murphie", 0.0 },
  { "Murphy", 0.85322946 },
  { "Murray", 0.98553854 },
  { "Murrel", 0.7956757 },
  { "Murrell", 0.85248446 },
  { "Murrey", 1.0 },
  { "Murrie", 0.0 },
  { "Murriel", 0.18957347 },
  { "Murrill", 1.0 },
  { "Murron", 0.0 },
  { "Murry", 0.9703657 },
  { "Mursal", 0.25581396 },
  { "Murtaza", 1.0 },
  { "Murtha", 0.5 },
  { "Murtie", 0.0 },
  { "Murtis", 0.0 },
  { "Murton", 1.0 },
  { "Murvel", 1.0 },
  { "Murvin", 1.0 },
  { "Musa", 0.9880493 },
  { "Musaab", 1.0 },
  { "Musab", 1.0 },
  { "Musah", 1.0 },
  { "Musashi", 1.0 },
  { "Muscab", 1.0 },
  { "Muse", 1.0 },
  { "Musetta", 0.0 },
  { "Musette", 0.0 },
  { "Musfirah", 0.0 },
  { "Musheerah", 0.0 },
  { "Mushka", 0.0 },
  { "Mushtaaq", 0.0 },
  { "Mushtaq", 0.0 },
  { "Music", 0.5 },
  { "Musie", 0.0 },
  { "Musiq", 0.5714286 },
  { "Muska", 0.0 },
  { "Muskaan", 0.0 },
  { "Muskan", 0.0 },
  { "Muslim", 1.0 },
  { "Muslima", 0.0 },
  { "Muslimah", 0.0 },
  { "Mussa", 1.0 },
  { "Musse", 1.0 },
  { "Mussie", 1.0 },
  { "Mustaf", 1.0 },
  { "Mustafa", 1.0 },
  { "Mustafaa", 1.0 },
  { "Mustafah", 1.0 },
  { "Mustafe", 1.0 },
  { "Mustapha", 1.0 },
  { "Mustaqeem", 1.0 },
  { "Mutasim", 1.0 },
  { "Mutaz", 1.0 },
  { "Mutsuko", 0.0 },
  { "Mutsuo", 1.0 },
  { "Mutt", 1.0 },
  { "Muyi", 0.0 },
  { "Muzamil", 1.0 },
  { "Muzammil", 1.0 },
  { "Muzette", 0.0 },
  { "Muzik", 1.0 },
  { "Muzna", 0.0 },
  { "Muzzammil", 1.0 },
  { "My", 0.18628216 },
  { "Mya", 0.0010466131 },
  { "Myaa", 0.0 },
  { "Myabella", 0.0 },
  { "Myah", 0.00073475385 },
  { "Myair", 0.0 },
  { "Myaira", 0.0 },
  { "Myairah", 0.0 },
  { "Myaire", 0.61538464 },
  { "Myaisha", 0.0 },
  { "Myajah", 0.0 },
  { "Myalee", 0.0 },
  { "Myalin", 0.0 },
  { "Myalise", 0.0 },
  { "Myalyn", 0.0 },
  { "Myalynn", 0.0 },
  { "Myamarie", 0.0 },
  { "Myami", 0.0 },
  { "Myan", 0.47058824 },
  { "Myana", 0.0 },
  { "Myanah", 0.0 },
  { "Myangel", 0.0 },
  { "Myanh", 0.0 },
  { "Myani", 0.0 },
  { "Myann", 0.0 },
  { "Myanna", 0.0 },
  { "Myannah", 0.0 },
  { "Myanni", 0.0 },
  { "Myar", 0.0 },
  { "Myara", 0.0 },
  { "Myarose", 0.0 },
  { "Myashia", 0.0 },
  { "Myasia", 0.0 },
  { "Myasiah", 0.0 },
  { "Myauna", 0.0 },
  { "Myazia", 0.0 },
  { "Myca", 0.10324484 },
  { "Mycaela", 0.0 },
  { "Mycah", 0.43262988 },
  { "Mycal", 1.0 },
  { "Mycala", 0.0 },
  { "Mycayla", 0.0 },
  { "Mycha", 0.0 },
  { "Mychael", 0.84147155 },
  { "Mychaela", 0.0 },
  { "Mychal", 0.8797641 },
  { "Mychala", 0.0 },
  { "Mycheal", 1.0 },
  { "Mychel", 0.5 },
  { "Mychele", 0.0 },
  { "Mychell", 0.0 },
  { "Mychelle", 0.0 },
  { "Myckala", 0.0 },
  { "Mycol", 1.0 },
  { "Myda", 0.0 },
  { "Myeasha", 0.0 },
  { "Myeesha", 0.0 },
  { "Myeisha", 0.0 },
  { "Myeka", 0.0 },
  { "Myel", 0.0 },
  { "Myella", 0.0 },
  { "Myelle", 0.0 },
  { "Myer", 0.9874572 },
  { "Myers", 0.95247936 },
  { "Myesha", 0.0 },
  { "Myesheia", 0.0 },
  { "Myeshia", 0.0 },
  { "Myesia", 0.0 },
  { "Myha", 0.0 },
  { "Myhanh", 0.0 },
  { "Myheir", 1.0 },
  { "Myia", 0.0 },
  { "Myiah", 0.0 },
  { "Myianna", 0.0 },
  { "Myiesha", 0.0 },
  { "Myion", 1.0 },
  { "Myiona", 0.0 },
  { "Myionna", 0.0 },
  { "Myisha", 0.0 },
  { "Myishia", 0.0 },
  { "Myja", 0.0 },
  { "Myjah", 0.0 },
  { "Myjoy", 0.0 },
  { "Myka", 0.119302414 },
  { "Mykael", 0.78424656 },
  { "Mykaela", 0.0 },
  { "Mykaella", 0.0 },
  { "Mykah", 0.40377805 },
  { "Mykai", 1.0 },
  { "Mykail", 1.0 },
  { "Mykaila", 0.0 },
  { "Mykailah", 0.0 },
  { "Mykal", 0.7666428 },
  { "Mykala", 0.0 },
  { "Mykalah", 0.0 },
  { "Mykale", 0.8227848 },
  { "Mykalia", 0.0 },
  { "Mykalla", 0.0 },
  { "Mykaya", 0.0 },
  { "Mykayla", 0.0 },
  { "Mykaylah", 0.0 },
  { "Myke", 1.0 },
  { "Mykea", 0.0 },
  { "Mykeal", 0.9044118 },
  { "Mykeia", 0.0 },
  { "Mykeil", 1.0 },
  { "Mykeisha", 0.0 },
  { "Mykel", 0.80553156 },
  { "Mykela", 0.0 },
  { "Mykell", 0.7155963 },
  { "Mykelle", 0.04385965 },
  { "Mykelti", 0.35238096 },
  { "Mykena", 0.0 },
  { "Mykenna", 0.0 },
  { "Mykenzee", 0.0 },
  { "Mykenzi", 0.0 },
  { "Mykenzie", 0.0 },
  { "Mykera", 0.0 },
  { "Mykeria", 0.0 },
  { "Mykerria", 0.0 },
  { "Mykesha", 0.0 },
  { "Mykeshia", 0.0 },
  { "Mykeya", 0.0 },
  { "Mykhael", 1.0 },
  { "Mykhail", 1.0 },
  { "Mykhal", 1.0 },
  { "Mykhala", 0.0 },
  { "Mykhel", 1.0 },
  { "Mykhi", 1.0 },
  { "Mykhia", 0.0 },
  { "Myki", 1.0 },
  { "Mykia", 0.0 },
  { "Mykiah", 0.0 },
  { "Mykila", 0.0 },
  { "Myking", 1.0 },
  { "Mykira", 0.0 },
  { "Mykisha", 0.0 },
  { "Mykita", 0.0 },
  { "Mykiya", 0.0 },
  { "Mykiyah", 0.0 },
  { "Mykl", 1.0 },
  { "Mykle", 1.0 },
  { "Mykol", 1.0 },
  { "Mykola", 1.0 },
  { "Mykolas", 1.0 },
  { "Mykya", 0.0 },
  { "Mykyah", 0.0 },
  { "Mykyla", 0.0 },
  { "Myla", 0.0 },
  { "Mylaa", 0.0 },
  { "Mylah", 0.0 },
  { "Mylan", 0.7241379 },
  { "Mylana", 0.0 },
  { "Mylani", 0.0 },
  { "Mylania", 0.0 },
  { "Mylasha", 0.0 },
  { "Mylashia", 0.0 },
  { "Mylasia", 0.0 },
  { "Mylaya", 0.0 },
  { "Mylayah", 0.0 },
  { "Myldred", 0.0 },
  { "Mylea", 0.0 },
  { "Myleah", 0.0 },
  { "Mylee", 0.0 },
  { "Myleen", 0.0 },
  { "Myleena", 0.0 },
  { "Myleene", 0.0 },
  { "Myleesha", 0.0 },
  { "Mylei", 0.0 },
  { "Myleigh", 0.0 },
  { "Myleigha", 0.0 },
  { "Myleisha", 0.0 },
  { "Mylek", 1.0 },
  { "Myleka", 0.0 },
  { "Mylen", 0.9456522 },
  { "Mylena", 0.0 },
  { "Mylene", 0.0 },
  { "Myler", 1.0 },
  { "Myles", 0.99592865 },
  { "Mylesha", 0.0 },
  { "Myley", 0.0 },
  { "Mylez", 1.0 },
  { "Myli", 0.0 },
  { "Mylia", 0.0 },
  { "Myliah", 0.0 },
  { "Myliana", 0.0 },
  { "Mylicia", 0.0 },
  { "Mylie", 0.0 },
  { "Myliegh", 0.0 },
  { "Mylii", 0.0 },
  { "Mylik", 1.0 },
  { "Mylika", 0.0 },
  { "Mylin", 0.22680412 },
  { "Mylina", 0.0 },
  { "Mylinda", 0.0 },
  { "Mylinh", 0.0 },
  { "Mylinn", 0.0 },
  { "Mylisa", 0.0 },
  { "Mylisha", 0.0 },
  { "Mylissa", 0.0 },
  { "Myliyah", 0.0 },
  { "Mylla", 0.0 },
  { "Mylo", 0.9385475 },
  { "Myloh", 1.0 },
  { "Mylon", 1.0 },
  { "Mylove", 0.0 },
  { "Mylynda", 0.0 },
  { "Mylynn", 0.0 },
  { "Mylz", 1.0 },
  { "Myna", 0.0 },
  { "Mynard", 1.0 },
  { "Mynasia", 0.0 },
  { "Myndee", 0.0 },
  { "Myndi", 0.0 },
  { "Myneisha", 0.0 },
  { "Mynesha", 0.0 },
  { "Mynette", 0.0 },
  { "Mynia", 0.0 },
  { "Myniah", 0.0 },
  { "Mynika", 0.0 },
  { "Mynisha", 0.0 },
  { "Mynor", 1.0 },
  { "Myon", 1.0 },
  { "Myona", 0.0 },
  { "Myong", 0.0 },
  { "Myonna", 0.0 },
  { "Myosha", 0.0 },
  { "Myoshi", 0.0 },
  { "Myoshia", 0.0 },
  { "Myphuong", 0.0 },
  { "Myquan", 1.0 },
  { "Myquisha", 0.0 },
  { "Myra", 0.0008620398 },
  { "Myraa", 0.0 },
  { "Myrabelle", 0.0 },
  { "Myracle", 0.0 },
  { "Myrah", 0.0 },
  { "Myrakle", 0.0 },
  { "Myraline", 0.0 },
  { "Myran", 0.9390244 },
  { "Myranda", 0.0 },
  { "Myrdell", 0.0 },
  { "Myrdis", 0.0 },
  { "Myree", 0.0 },
  { "Myreen", 0.0 },
  { "Myrel", 0.30851063 },
  { "Myrella", 0.0 },
  { "Myren", 1.0 },
  { "Myrene", 0.0 },
  { "Myreon", 1.0 },
  { "Myresha", 0.0 },
  { "Myreta", 0.0 },
  { "Myretta", 0.0 },
  { "Myreya", 0.0 },
  { "Myria", 0.0 },
  { "Myriah", 0.0 },
  { "Myriam", 0.0 },
  { "Myrian", 0.0 },
  { "Myriana", 0.0 },
  { "Myrianna", 0.0 },
  { "Myrical", 0.0 },
  { "Myrick", 1.0 },
  { "Myricle", 0.0 },
  { "Myrie", 0.0 },
  { "Myrikal", 0.0 },
  { "Myrikle", 0.0 },
  { "Myril", 0.7058824 },
  { "Myrin", 1.0 },
  { "Myrion", 1.0 },
  { "Myrisa", 0.0 },
  { "Myrissa", 0.0 },
  { "Myrka", 0.0 },
  { "Myrl", 0.40763506 },
  { "Myrla", 0.0 },
  { "Myrlande", 0.0 },
  { "Myrle", 0.22619048 },
  { "Myrlee", 0.0 },
  { "Myrleen", 0.0 },
  { "Myrlen", 1.0 },
  { "Myrlene", 0.0 },
  { "Myrlin", 0.5 },
  { "Myrline", 0.0 },
  { "Myrna", 0.0013278533 },
  { "Myrne", 0.0 },
  { "Myron", 0.9896875 },
  { "Myrone", 1.0 },
  { "Myrra", 0.0 },
  { "Myrt", 0.0 },
  { "Myrta", 0.0 },
  { "Myrtes", 0.0 },
  { "Myrth", 0.0 },
  { "Myrtha", 0.0 },
  { "Myrtia", 0.0 },
  { "Myrtice", 0.0 },
  { "Myrtie", 0.0 },
  { "Myrtis", 0.0008161933 },
  { "Myrtise", 0.0 },
  { "Myrtle", 0.0036364964 },
  { "Myrtlee", 0.0 },
  { "Myrtlene", 0.0 },
  { "Myrton", 1.0 },
  { "Myrtte", 0.0 },
  { "Myrtus", 0.0 },
  { "Mysean", 1.0 },
  { "Mysha", 0.0 },
  { "Myshae", 0.0 },
  { "Myshaun", 1.0 },
  { "Myshawn", 1.0 },
  { "Myshayla", 0.0 },
  { "Myshell", 0.0 },
  { "Myshia", 0.0 },
  { "Myshon", 1.0 },
  { "Mysia", 0.0 },
  { "Myson", 1.0 },
  { "Myssiah", 1.0 },
  { "Mystee", 0.0 },
  { "Myster", 1.0 },
  { "Mystery", 0.0 },
  { "Mysti", 0.0 },
  { "Mystic", 0.0 },
  { "Mystica", 0.0 },
  { "Mystical", 0.0 },
  { "Mystie", 0.0 },
  { "Mystik", 0.0 },
  { "Mystikal", 0.77272725 },
  { "Mystique", 0.0 },
  { "Mysty", 0.0 },
  { "Mytchell", 1.0 },
  { "Mythias", 1.0 },
  { "Mythili", 0.0 },
  { "Mythri", 0.0 },
  { "Mytia", 0.0 },
  { "Mytien", 0.0 },
  { "Mytisha", 0.0 },
  { "Mytrell", 1.0 },
  { "Myung", 0.0 },
  { "Myunique", 0.0 },
  { "Myya", 0.0 },
  { "Myyah", 0.0 },
  { "Myzel", 1.0 },
  { "Na", 0.13229571 },
  { "Naa", 0.0 },
  { "Naadia", 0.0 },
  { "Naadir", 1.0 },
  { "Naadira", 0.0 },
  { "Naadirah", 0.0 },
  { "Naail", 1.0 },
  { "Naailah", 0.0 },
  { "Naairah", 0.0 },
  { "Naajia", 0.0 },
  { "Naaliyah", 0.0 },
  { "Naama", 0.0 },
  { "Naamah", 0.0 },
  { "Naaman", 1.0 },
  { "Naami", 0.0 },
  { "Naana", 0.0 },
  { "Naara", 0.0 },
  { "Naarah", 0.0 },
  { "Naariah", 0.0 },
  { "Naasia", 0.0 },
  { "Naasir", 1.0 },
  { "Naason", 1.0 },
  { "Naava", 0.0 },
  { "Naavah", 0.0 },
  { "Naavya", 0.0 },
  { "Naaya", 0.0 },
  { "Naayel", 1.0 },
  { "Naaz", 0.0 },
  { "Naazir", 1.0 },
  { "Naba", 0.0 },
  { "Nabaa", 0.0 },
  { "Nabeeha", 0.0 },
  { "Nabeel", 1.0 },
  { "Nabeela", 0.0 },
  { "Nabeelah", 0.0 },
  { "Nabella", 0.0 },
  { "Nabhan", 1.0 },
  { "Nabhya", 1.0 },
  { "Nabi", 1.0 },
  { "Nabia", 0.0 },
  { "Nabiha", 0.0 },
  { "Nabihah", 0.0 },
  { "Nabil", 0.969603 },
  { "Nabila", 0.0 },
  { "Nabilah", 0.0 },
  { "Nabile", 0.0 },
  { "Nabor", 1.0 },
  { "Nabria", 0.0 },
  { "Nacala", 0.0 },
  { "Nacari", 1.0 },
  { "Nace", 1.0 },
  { "Nachaly", 0.0 },
  { "Nache", 0.0 },
  { "Nachel", 0.0 },
  { "Nachele", 0.0 },
  { "Nachelle", 0.0 },
  { "Nachman", 1.0 },
  { "Nachmen", 1.0 },
  { "Nacho", 1.0 },
  { "Nachole", 0.0 },
  { "Nachshon", 1.0 },
  { "Nachum", 1.0 },
  { "Nacier", 1.0 },
  { "Nacirema", 0.0 },
  { "Nacola", 0.0 },
  { "Nacole", 0.0 },
  { "Nacoma", 1.0 },
  { "Nacona", 0.0 },
  { "Nacy", 0.0 },
  { "Nada", 0.0 },
  { "Nadaja", 0.0 },
  { "Nadal", 1.0 },
  { "Nadalee", 0.0 },
  { "Nadaleigh", 0.0 },
  { "Nadalia", 0.0 },
  { "Nadalie", 0.0 },
  { "Nadaly", 0.0 },
  { "Nadalyn", 0.0 },
  { "Nadalynn", 0.0 },
  { "Nadar", 1.0 },
  { "Nadara", 0.0 },
  { "Nadarius", 1.0 },
  { "Nadasha", 0.0 },
  { "Nadasia", 0.0 },
  { "Nadav", 1.0 },
  { "Nadaya", 0.0 },
  { "Nadea", 0.0 },
  { "Nadean", 0.0 },
  { "Nadeane", 0.0 },
  { "Nadeem", 1.0 },
  { "Nadeen", 0.0 },
  { "Nadeene", 0.0 },
  { "Nadege", 0.0 },
  { "Nadeige", 0.0 },
  { "Nadeja", 0.0 },
  { "Nadelie", 0.0 },
  { "Nadeline", 0.0 },
  { "Nadell", 0.0 },
  { "Nadelyn", 0.0 },
  { "Nadelynn", 0.0 },
  { "Naden", 1.0 },
  { "Nadene", 0.0 },
  { "Nader", 1.0 },
  { "Nadera", 0.0 },
  { "Naderge", 0.0 },
  { "Nadeya", 0.0 },
  { "Nadezhda", 0.0 },
  { "Nadia", 0.0015333251 },
  { "Nadiah", 0.0 },
  { "Nadie", 0.0 },
  { "Nadija", 0.0 },
  { "Nadilee", 0.0 },
  { "Nadilyn", 0.0 },
  { "Nadilynn", 0.0 },
  { "Nadim", 1.0 },
  { "Nadin", 0.0 },
  { "Nadina", 0.0 },
  { "Nadine", 0.00082528836 },
  { "Nadir", 1.0 },
  { "Nadira", 0.0 },
  { "Nadirah", 0.0 },
  { "Nadiv", 1.0 },
  { "Nadiya", 0.0 },
  { "Nadiyah", 0.0 },
  { "Nadja", 0.0 },
  { "Nadjah", 0.0 },
  { "Nadolyn", 0.0 },
  { "Nadra", 0.0 },
  { "Nadrea", 0.0 },
  { "Nadya", 0.0 },
  { "Nadyah", 0.0 },
  { "Nadyalee", 0.0 },
  { "Nadyia", 0.0 },
  { "Nadyne", 0.0 },
  { "Naedelyn", 0.0 },
  { "Naeem", 1.0 },
  { "Naeema", 0.0 },
  { "Naeemah", 0.0 },
  { "Naeisha", 0.0 },
  { "Nael", 0.9526185 },
  { "Naela", 0.0 },
  { "Naelah", 0.0 },
  { "Naelani", 0.0 },
  { "Naeli", 0.0 },
  { "Naelle", 0.0 },
  { "Naelson", 1.0 },
  { "Naely", 0.0 },
  { "Naelynn", 0.0 },
  { "Naema", 0.0 },
  { "Naeomi", 0.0 },
  { "Naequan", 1.0 },
  { "Naesha", 0.0 },
  { "Naeshaun", 1.0 },
  { "Naeshawn", 1.0 },
  { "Naethan", 1.0 },
  { "Naetochukwu", 1.0 },
  { "Naeva", 0.0 },
  { "Naevia", 0.0 },
  { "Naeyli", 0.0 },
  { "Nafas", 0.0 },
  { "Nafee", 1.0 },
  { "Nafees", 1.0 },
  { "Nafeesa", 0.0 },
  { "Nafeesah", 0.0 },
  { "Nafeesha", 0.0 },
  { "Nafi", 1.0 },
  { "Nafia", 0.0 },
  { "Nafis", 1.0 },
  { "Nafisa", 0.0 },
  { "Nafisah", 0.0 },
  { "Nafiso", 0.0 },
  { "Nafissa", 0.0 },
  { "Nafiz", 1.0 },
  { "Naftali", 1.0 },
  { "Naftoli", 1.0 },
  { "Naftula", 1.0 },
  { "Naftuli", 1.0 },
  { "Naftuly", 1.0 },
  { "Nagee", 1.0 },
  { "Nagela", 0.0 },
  { "Nagelly", 0.0 },
  { "Nagely", 0.0 },
  { "Nagham", 0.0 },
  { "Nagi", 1.0 },
  { "Nagisa", 0.0 },
  { "Naguan", 1.0 },
  { "Nagwa", 0.0 },
  { "Nahal", 0.0 },
  { "Nahara", 0.0 },
  { "Nahari", 0.0 },
  { "Nahaven", 0.0 },
  { "Nahayla", 0.0 },
  { "Nahdia", 0.0 },
  { "Naheed", 0.0 },
  { "Naheem", 1.0 },
  { "Naheim", 1.0 },
  { "Nahel", 1.0 },
  { "Nahely", 0.0 },
  { "Nahema", 0.0 },
  { "Nahia", 0.0 },
  { "Nahiara", 0.0 },
  { "Nahid", 0.48 },
  { "Nahida", 0.0 },
  { "Nahide", 0.0 },
  { "Nahiem", 1.0 },
  { "Nahila", 0.0 },
  { "Nahim", 1.0 },
  { "Nahima", 0.0 },
  { "Nahin", 1.0 },
  { "Nahiomy", 0.0 },
  { "Nahir", 1.0 },
  { "Nahja", 0.0 },
  { "Nahjae", 0.0 },
  { "Nahjai", 0.0 },
  { "Nahjay", 0.0 },
  { "Nahje", 0.0 },
  { "Nahjee", 1.0 },
  { "Nahki", 1.0 },
  { "Nahla", 0.0 },
  { "Nahlah", 0.0 },
  { "Nahlani", 0.0 },
  { "Nahlia", 0.0 },
  { "Nahliyah", 0.0 },
  { "Nahmir", 1.0 },
  { "Nahoa", 1.0 },
  { "Nahom", 1.0 },
  { "Nahome", 1.0 },
  { "Nahomi", 0.0 },
  { "Nahomie", 0.0 },
  { "Nahomy", 0.0 },
  { "Nahrain", 0.0 },
  { "Nahriah", 0.0 },
  { "Nahshon", 1.0 },
  { "Nahsir", 1.0 },
  { "Nahtanha", 0.0 },
  { "Nahuel", 1.0 },
  { "Nahum", 1.0 },
  { "Nahun", 1.0 },
  { "Nahya", 0.0 },
  { "Nahyan", 1.0 },
  { "Nahyeli", 0.0 },
  { "Nahyla", 0.0 },
  { "Nahyma", 0.0 },
  { "Nahzier", 1.0 },
  { "Nahzir", 1.0 },
  { "Nai", 0.24567474 },
  { "Naia", 0.0 },
  { "Naiah", 0.0 },
  { "Naiana", 0.0 },
  { "Naiara", 0.0 },
  { "Naibe", 0.0 },
  { "Naida", 0.0 },
  { "Naidaly", 0.0 },
  { "Naidelin", 0.0 },
  { "Naideline", 0.0 },
  { "Naidely", 0.0 },
  { "Naidelyn", 0.0 },
  { "Naiden", 1.0 },
  { "Naieem", 1.0 },
  { "Naiely", 0.0 },
  { "Naiem", 1.0 },
  { "Naiema", 0.0 },
  { "Naiesha", 0.0 },
  { "Naif", 1.0 },
  { "Naihla", 0.0 },
  { "Naihomy", 0.0 },
  { "Naiim", 1.0 },
  { "Naiima", 0.0 },
  { "Naija", 0.0 },
  { "Naijah", 0.0 },
  { "Naika", 0.0 },
  { "Naikia", 0.0 },
  { "Nail", 1.0 },
  { "Naila", 0.0 },
  { "Nailah", 0.0 },
  { "Nailani", 0.0 },
  { "Nailea", 0.0 },
  { "Naileah", 0.0 },
  { "Nailee", 0.0 },
  { "Naileen", 0.0 },
  { "Naileth", 0.0 },
  { "Naily", 0.0 },
  { "Nailyn", 0.0 },
  { "Naim", 1.0 },
  { "Naima", 0.0 },
  { "Naimah", 0.0 },
  { "Naimo", 0.0 },
  { "Nain", 1.0 },
  { "Naina", 0.0 },
  { "Nainika", 0.0 },
  { "Nainoa", 1.0 },
  { "Naioma", 0.0 },
  { "Naiome", 0.0 },
  { "Naiomi", 0.0 },
  { "Naiomy", 0.0 },
  { "Naiovy", 0.0 },
  { "Naira", 0.0 },
  { "Nairah", 0.0 },
  { "Nairi", 0.0 },
  { "Nairobi", 0.083798885 },
  { "Nairoby", 0.0 },
  { "Nairy", 0.0 },
  { "Naisa", 0.0 },
  { "Naisaiah", 1.0 },
  { "Naisean", 1.0 },
  { "Naisha", 0.0 },
  { "Naishaun", 1.0 },
  { "Naishawn", 1.0 },
  { "Naissa", 0.0 },
  { "Naithan", 1.0 },
  { "Naithen", 1.0 },
  { "Naithyn", 1.0 },
  { "Naitik", 1.0 },
  { "Naiya", 0.0 },
  { "Naiyah", 0.0 },
  { "Naiyana", 0.0 },
  { "Naiyanna", 0.0 },
  { "Naiyeli", 0.0 },
  { "Naiyla", 0.0 },
  { "Naiylah", 0.0 },
  { "Naiym", 1.0 },
  { "Naiyma", 0.0 },
  { "Naizeth", 0.0 },
  { "Naja", 0.004830918 },
  { "Najaah", 0.0 },
  { "Najada", 0.0 },
  { "Najae", 0.13204509 },
  { "Najah", 0.019585254 },
  { "Najahwan", 1.0 },
  { "Najai", 0.20304568 },
  { "Najair", 1.0 },
  { "Najala", 0.0 },
  { "Najam", 1.0 },
  { "Najat", 0.0 },
  { "Najay", 0.5 },
  { "Najaya", 0.0 },
  { "Najayah", 0.0 },
  { "Naje", 0.22377622 },
  { "Najea", 0.0 },
  { "Najee", 0.8561573 },
  { "Najeeb", 1.0 },
  { "Najeebah", 0.0 },
  { "Najeh", 1.0 },
  { "Najeli", 0.0 },
  { "Najelly", 0.0 },
  { "Najely", 0.0 },
  { "Najera", 0.0 },
  { "Najha", 0.0 },
  { "Naji", 1.0 },
  { "Najia", 0.0 },
  { "Najiah", 0.0 },
  { "Najib", 1.0 },
  { "Najier", 1.0 },
  { "Najih", 1.0 },
  { "Najila", 0.0 },
  { "Najilah", 0.0 },
  { "Najir", 1.0 },
  { "Najirah", 0.0 },
  { "Najiyah", 0.0 },
  { "Najiyyah", 0.0 },
  { "Najja", 0.5176471 },
  { "Najji", 1.0 },
  { "Najla", 0.0 },
  { "Najm", 1.0 },
  { "Najma", 0.0 },
  { "Najmah", 0.0 },
  { "Najmo", 0.0 },
  { "Najon", 1.0 },
  { "Najuan", 1.0 },
  { "Najuma", 0.0 },
  { "Najwa", 0.0 },
  { "Najya", 0.0 },
  { "Nakai", 0.8 },
  { "Nakaia", 0.0 },
  { "Nakaiah", 0.0 },
  { "Nakaila", 0.0 },
  { "Nakaiya", 0.0 },
  { "Nakaiyah", 0.0 },
  { "Nakala", 0.0 },
  { "Nakara", 0.0 },
  { "Nakari", 0.41666666 },
  { "Nakaria", 0.0 },
  { "Nakasha", 0.0 },
  { "Nakasia", 0.0 },
  { "Nakaya", 0.0 },
  { "Nakayah", 0.0 },
  { "Nakayia", 0.0 },
  { "Nakayla", 0.0 },
  { "Nakaylah", 0.0 },
  { "Nakaylee", 0.0 },
  { "Nakea", 0.0 },
  { "Nakeba", 0.0 },
  { "Nakecia", 0.0 },
  { "Nakeda", 0.0 },
  { "Nakedra", 0.0 },
  { "Nakee", 0.0 },
  { "Nakeea", 0.0 },
  { "Nakeem", 1.0 },
  { "Nakeema", 0.0 },
  { "Nakeena", 0.0 },
  { "Nakeesha", 0.0 },
  { "Nakeeta", 0.0 },
  { "Nakeia", 0.0 },
  { "Nakeira", 0.0 },
  { "Nakeisha", 0.0 },
  { "Nakeita", 0.0 },
  { "Nakeitha", 0.0 },
  { "Nakel", 0.0 },
  { "Nakela", 0.0 },
  { "Nakema", 0.0 },
  { "Nakendra", 0.0 },
  { "Nakenya", 0.0 },
  { "Nakenzie", 0.0 },
  { "Nakera", 0.0 },
  { "Nakeria", 0.0 },
  { "Nakesha", 0.0 },
  { "Nakeshia", 0.0 },
  { "Nakesia", 0.0 },
  { "Naketa", 0.0 },
  { "Naketta", 0.0 },
  { "Nakevia", 0.0 },
  { "Nakeya", 0.0 },
  { "Nakeyah", 0.0 },
  { "Nakeyia", 0.0 },
  { "Nakeysha", 0.0 },
  { "Nakeyta", 0.0 },
  { "Nakhai", 1.0 },
  { "Nakhi", 1.0 },
  { "Nakhia", 0.0 },
  { "Nakhiya", 0.0 },
  { "Nakhya", 0.0 },
  { "Naki", 0.90909094 },
  { "Nakia", 0.18841676 },
  { "Nakiah", 0.0 },
  { "Nakiaya", 0.0 },
  { "Nakida", 0.0 },
  { "Nakie", 0.6363636 },
  { "Nakiea", 0.0 },
  { "Nakiera", 0.0 },
  { "Nakiesha", 0.0 },
  { "Nakieta", 0.0 },
  { "Nakieya", 0.0 },
  { "Nakijah", 0.0 },
  { "Nakila", 0.0 },
  { "Nakima", 0.0 },
  { "Nakina", 0.0 },
  { "Nakio", 1.0 },
  { "Nakira", 0.0 },
  { "Nakirah", 0.0 },
  { "Nakiria", 0.0 },
  { "Nakisa", 0.0 },
  { "Nakish", 0.0 },
  { "Nakisha", 0.0 },
  { "Nakishia", 0.0 },
  { "Nakita", 0.009465856 },
  { "Nakitia", 0.0 },
  { "Nakitta", 0.0 },
  { "Nakiya", 0.0030102348 },
  { "Nakiyah", 0.0 },
  { "Nakiyha", 0.0 },
  { "Nakiyla", 0.0 },
  { "Nakkia", 0.0 },
  { "Nakoa", 0.98630136 },
  { "Nakoda", 1.0 },
  { "Nakoma", 0.41860464 },
  { "Nakosha", 0.0 },
  { "Nakota", 0.59375 },
  { "Naksh", 1.0 },
  { "Nakshatra", 0.0 },
  { "Nakul", 1.0 },
  { "Nakuma", 1.0 },
  { "Nakya", 0.0 },
  { "Nakyah", 0.0 },
  { "Nakye", 1.0 },
  { "Nakyia", 0.0 },
  { "Nakyiah", 0.0 },
  { "Nakyla", 0.0 },
  { "Nakylah", 0.0 },
  { "Nakyra", 0.0 },
  { "Nakyrah", 0.0 },
  { "Nakysha", 0.0 },
  { "Nala", 0.0 },
  { "Nalah", 0.0 },
  { "Nalaiyah", 0.0 },
  { "Nalan", 1.0 },
  { "Nalana", 0.0 },
  { "Nalanee", 0.0 },
  { "Nalani", 0.0 },
  { "Nalanie", 0.0 },
  { "Nalany", 0.0 },
  { "Nalaya", 0.0 },
  { "Nalayah", 0.0 },
  { "Nalayiah", 0.0 },
  { "Nalayna", 0.0 },
  { "Nalaysia", 0.0 },
  { "Nalda", 0.0 },
  { "Nalea", 0.0 },
  { "Naleah", 0.0 },
  { "Naledi", 0.0 },
  { "Nalee", 0.0 },
  { "Naleea", 0.0 },
  { "Nalei", 0.0 },
  { "Naleia", 0.0 },
  { "Naleigh", 0.0 },
  { "Naleigha", 0.0 },
  { "Nalej", 1.0 },
  { "Nalen", 1.0 },
  { "Nalene", 0.0 },
  { "Naleyah", 0.0 },
  { "Nali", 0.0 },
  { "Nalia", 0.0 },
  { "Naliah", 0.0 },
  { "Naliana", 0.0 },
  { "Nalijah", 0.0 },
  { "Nalin", 0.8613861 },
  { "Nalina", 0.0 },
  { "Nalini", 0.0 },
  { "Nalisa", 0.0 },
  { "Nalisha", 0.0 },
  { "Nalissa", 0.0 },
  { "Naliya", 0.0 },
  { "Naliyah", 0.0 },
  { "Nalla", 0.0 },
  { "Nallah", 0.0 },
  { "Nalleli", 0.0 },
  { "Nallely", 0.0 },
  { "Nalley", 0.0 },
  { "Nally", 0.0 },
  { "Nalo", 0.0 },
  { "Naloni", 0.0 },
  { "Nalu", 1.0 },
  { "Naly", 0.0 },
  { "Nalya", 0.0 },
  { "Nalyah", 0.0 },
  { "Nalyjah", 0.0 },
  { "Nalyn", 0.0 },
  { "Nalynn", 0.0 },
  { "Nalyssa", 0.0 },
  { "Nam", 1.0 },
  { "Naman", 1.0 },
  { "Namari", 0.7111111 },
  { "Namarion", 1.0 },
  { "Namath", 1.0 },
  { "Namaya", 0.0 },
  { "Name", 0.6764706 },
  { "Nameer", 1.0 },
  { "Nameera", 0.0 },
  { "Nami", 0.0 },
  { "Namia", 0.0 },
  { "Namiah", 0.0 },
  { "Namie", 0.0 },
  { "Namiko", 0.0 },
  { "Namine", 0.0 },
  { "Namir", 1.0 },
  { "Namira", 0.0 },
  { "Namirah", 0.0 },
  { "Namish", 1.0 },
  { "Namit", 1.0 },
  { "Namita", 0.0 },
  { "Namitha", 0.0 },
  { "Namiya", 0.0 },
  { "Namiyah", 0.0 },
  { "Namon", 1.0 },
  { "Namrata", 0.0 },
  { "Namya", 0.0 },
  { "Namyah", 0.0 },
  { "Namyia", 0.0 },
  { "Nan", 0.0 },
  { "Nana", 0.2300163 },
  { "Nanaadwoa", 0.0 },
  { "Nanaakua", 0.0 },
  { "Nanaama", 0.0 },
  { "Nanako", 0.0 },
  { "Nanakwame", 1.0 },
  { "Nanalee", 0.0 },
  { "Nanami", 0.0 },
  { "Nanayaa", 0.0 },
  { "Nanayaw", 1.0 },
  { "Nance", 0.012970169 },
  { "Nancee", 0.0 },
  { "Nancey", 0.0 },
  { "Nanci", 0.0 },
  { "Nanciann", 0.0 },
  { "Nancianne", 0.0 },
  { "Nancie", 0.0 },
  { "Nancilee", 0.0 },
  { "Nancy", 0.0029324607 },
  { "Nancyann", 0.0 },
  { "Nancyanne", 0.0 },
  { "Nancye", 0.0 },
  { "Nancyjane", 0.0 },
  { "Nancyjean", 0.0 },
  { "Nancyjo", 0.0 },
  { "Nancylee", 0.0 },
  { "Nancylou", 0.0 },
  { "Nand", 1.0 },
  { "Nanda", 0.071428575 },
  { "Nandan", 1.0 },
  { "Nandana", 0.0 },
  { "Nandhan", 1.0 },
  { "Nandhana", 0.0 },
  { "Nandhini", 0.0 },
  { "Nandi", 0.0 },
  { "Nandika", 0.0 },
  { "Nandini", 0.0 },
  { "Nandita", 0.0 },
  { "Nanditha", 0.0 },
  { "Nandni", 0.0 },
  { "Nando", 1.0 },
  { "Nandy", 0.0 },
  { "Nanea", 0.0 },
  { "Naneka", 0.0 },
  { "Nanelle", 0.0 },
  { "Nanett", 0.0 },
  { "Nanetta", 0.0 },
  { "Nanette", 0.0 },
  { "Nang", 1.0 },
  { "Nani", 0.0 },
  { "Nanie", 0.0 },
  { "Nanika", 0.0 },
  { "Nanine", 0.0 },
  { "Nanisha", 0.0 },
  { "Nanita", 0.0 },
  { "Nann", 0.0 },
  { "Nanna", 0.0 },
  { "Nannett", 0.0 },
  { "Nannetta", 0.0 },
  { "Nannette", 0.0 },
  { "Nannie", 0.00078186084 },
  { "Nanny", 0.0 },
  { "Nanor", 0.0 },
  { "Nansi", 0.0 },
  { "Nansy", 0.0 },
  { "Nantambu", 1.0 },
  { "Nanya", 1.0 },
  { "Nanyamka", 0.0 },
  { "Nao", 0.10526316 },
  { "Naobi", 0.0 },
  { "Naod", 1.0 },
  { "Naoh", 1.0 },
  { "Naohmi", 0.0 },
  { "Naoki", 0.9637681 },
  { "Naoko", 0.0 },
  { "Naol", 1.0 },
  { "Naola", 0.0 },
  { "Naoma", 0.0 },
  { "Naome", 0.0 },
  { "Naomee", 0.0 },
  { "Naomi", 0.0017819029 },
  { "Naomia", 0.0 },
  { "Naomie", 0.0 },
  { "Naomigrace", 0.0 },
  { "Naomii", 0.0 },
  { "Naomy", 0.0 },
  { "Naor", 1.0 },
  { "Naoto", 1.0 },
  { "Naoya", 1.0 },
  { "Napat", 1.0 },
  { "Naphtali", 0.7443609 },
  { "Napolean", 1.0 },
  { "Napoleon", 0.9993479 },
  { "Naporsha", 0.0 },
  { "Napua", 0.0 },
  { "Naqeeb", 1.0 },
  { "Naquan", 1.0 },
  { "Naquana", 0.0 },
  { "Naquanda", 0.0 },
  { "Naquasha", 0.0 },
  { "Naquasia", 0.0 },
  { "Naquelle", 0.0 },
  { "Naquesha", 0.0 },
  { "Naquille", 1.0 },
  { "Naquisha", 0.0 },
  { "Naquita", 0.0 },
  { "Naquitta", 0.0 },
  { "Naquon", 1.0 },
  { "Nara", 0.02728732 },
  { "Narada", 0.96634614 },
  { "Narae", 0.0 },
  { "Narah", 0.0 },
  { "Narai", 0.0 },
  { "Narain", 1.0 },
  { "Narali", 0.0 },
  { "Naralie", 0.0 },
  { "Naraly", 0.0 },
  { "Narasha", 0.0 },
  { "Naraya", 0.0 },
  { "Narayah", 0.0 },
  { "Narayan", 1.0 },
  { "Narayana", 0.77272725 },
  { "Narayani", 0.0 },
  { "Narcedalia", 0.0 },
  { "Narcie", 0.0 },
  { "Narcille", 0.0 },
  { "Narcisa", 0.0 },
  { "Narcisco", 1.0 },
  { "Narciso", 1.0 },
  { "Narciss", 0.0 },
  { "Narcissa", 0.0 },
  { "Narcisse", 1.0 },
  { "Narcissus", 0.0 },
  { "Narcizo", 1.0 },
  { "Narda", 0.0 },
  { "Nardia", 0.0 },
  { "Nardos", 0.0 },
  { "Nare", 0.0 },
  { "Nareen", 0.0 },
  { "Nareg", 1.0 },
  { "Nareh", 0.0 },
  { "Narek", 1.0 },
  { "Nareli", 0.0 },
  { "Narell", 1.0 },
  { "Narelle", 0.0 },
  { "Narely", 0.0 },
  { "Naren", 1.0 },
  { "Naresh", 1.0 },
  { "Narges", 0.0 },
  { "Nargis", 0.0 },
  { "Nari", 0.0 },
  { "Naria", 0.0 },
  { "Nariah", 0.0 },
  { "Nariana", 0.0 },
  { "Narice", 0.0 },
  { "Narielle", 0.0 },
  { "Nariman", 0.0 },
  { "Narin", 0.74418604 },
  { "Narine", 0.0 },
  { "Narisa", 0.0 },
  { "Narissa", 0.0 },
  { "Narita", 0.0 },
  { "Narith", 1.0 },
  { "Nariya", 0.0 },
  { "Nariyah", 0.0 },
  { "Narjes", 0.0 },
  { "Narjis", 0.0 },
  { "Narkita", 0.0 },
  { "Narma", 0.0 },
  { "Narmeen", 0.0 },
  { "Narmon", 1.0 },
  { "Narnia", 0.0 },
  { "Naron", 1.0 },
  { "Narong", 1.0 },
  { "Narquise", 1.0 },
  { "Narriah", 0.0 },
  { "Narrissa", 0.0 },
  { "Narsis", 0.0 },
  { "Narsiso", 1.0 },
  { "Narumi", 0.0 },
  { "Naruto", 1.0 },
  { "Narva", 0.0 },
  { "Narvel", 1.0 },
  { "Narvell", 0.45833334 },
  { "Narvelle", 0.0 },
  { "Narvie", 0.23809524 },
  { "Narvin", 1.0 },
  { "Narvis", 0.0 },
  { "Nary", 0.0 },
  { "Narya", 0.0 },
  { "Naryah", 0.0 },
  { "Naryan", 1.0 },
  { "Naryia", 0.0 },
  { "Naryiah", 0.0 },
  { "Nas", 1.0 },
  { "Nasai", 1.0 },
  { "Nasaiah", 1.0 },
  { "Nasair", 1.0 },
  { "Nasar", 1.0 },
  { "Nasaria", 0.0 },
  { "Nasario", 1.0 },
  { "Nasasha", 0.0 },
  { "Nasaya", 0.0 },
  { "Nascha", 0.0 },
  { "Nasean", 1.0 },
  { "Nasear", 1.0 },
  { "Naseeb", 0.0 },
  { "Naseem", 0.8050571 },
  { "Naseer", 1.0 },
  { "Naseim", 1.0 },
  { "Naseir", 1.0 },
  { "Naser", 1.0 },
  { "Nasere", 1.0 },
  { "Nash", 0.99617374 },
  { "Nasha", 0.0 },
  { "Nashae", 0.0 },
  { "Nashai", 0.0 },
  { "Nashalee", 0.0 },
  { "Nashaley", 0.0 },
  { "Nashali", 0.0 },
  { "Nashalie", 0.0 },
  { "Nashally", 0.0 },
  { "Nashaly", 0.0 },
  { "Nashan", 1.0 },
  { "Nashana", 0.0 },
  { "Nashanda", 0.0 },
  { "Nashanti", 0.0 },
  { "Nashara", 0.0 },
  { "Nashari", 0.0 },
  { "Nashaun", 1.0 },
  { "Nashawn", 0.94091415 },
  { "Nashay", 0.0 },
  { "Nashaya", 0.0 },
  { "Nashayla", 0.0 },
  { "Nashea", 0.0 },
  { "Nasheed", 1.0 },
  { "Nasheem", 1.0 },
  { "Nasheema", 0.0 },
  { "Nasheena", 0.0 },
  { "Nasheka", 0.0 },
  { "Nashell", 0.0 },
  { "Nashelle", 0.0 },
  { "Nashelly", 0.0 },
  { "Nashely", 0.0 },
  { "Nashia", 0.0 },
  { "Nashid", 1.0 },
  { "Nashiem", 1.0 },
  { "Nashika", 0.0 },
  { "Nashima", 0.0 },
  { "Nashir", 1.0 },
  { "Nashira", 0.0 },
  { "Nashiya", 0.0 },
  { "Nashiyah", 0.0 },
  { "Nashla", 0.0 },
  { "Nashley", 0.0 },
  { "Nashli", 0.0 },
  { "Nashlie", 0.0 },
  { "Nashlly", 0.0 },
  { "Nashly", 0.0 },
  { "Nashmia", 0.0 },
  { "Nashoba", 1.0 },
  { "Nashon", 1.0 },
  { "Nashonda", 0.0 },
  { "Nashrah", 0.0 },
  { "Nashton", 1.0 },
  { "Nashua", 1.0 },
  { "Nashwa", 0.0 },
  { "Nashwan", 1.0 },
  { "Nashya", 0.0 },
  { "Nashyah", 0.0 },
  { "Nashyia", 0.0 },
  { "Nashyla", 0.0 },
  { "Nashyra", 0.0 },
  { "Nasi", 1.0 },
  { "Nasia", 0.0 },
  { "Nasiah", 0.38819876 },
  { "Nasiar", 1.0 },
  { "Nasiem", 1.0 },
  { "Nasier", 1.0 },
  { "Nasiere", 1.0 },
  { "Nasif", 1.0 },
  { "Nasiha", 0.0 },
  { "Nasiir", 1.0 },
  { "Nasim", 0.7629482 },
  { "Nasir", 0.99652094 },
  { "Nasira", 0.0 },
  { "Nasirah", 0.0 },
  { "Nasire", 1.0 },
  { "Nasiriyah", 0.0 },
  { "Nasiya", 0.0 },
  { "Nasiyah", 0.0 },
  { "Nasjah", 0.0 },
  { "Nasly", 0.0 },
  { "Nasma", 0.0 },
  { "Nason", 1.0 },
  { "Nasr", 1.0 },
  { "Nasra", 0.0 },
  { "Nasrallah", 1.0 },
  { "Nasreen", 0.0 },
  { "Nasri", 1.0 },
  { "Nasrin", 0.0 },
  { "Nasro", 0.0 },
  { "Nasrudin", 1.0 },
  { "Nassar", 1.0 },
  { "Nassem", 1.0 },
  { "Nasser", 1.0 },
  { "Nassiah", 0.33333334 },
  { "Nassim", 0.9027778 },
  { "Nassir", 1.0 },
  { "Nassor", 1.0 },
  { "Nastacia", 0.0 },
  { "Nastaja", 0.0 },
  { "Nastasha", 0.0 },
  { "Nastashia", 0.0 },
  { "Nastasia", 0.0 },
  { "Nastasja", 0.0 },
  { "Nastassia", 0.0 },
  { "Nastassja", 0.0 },
  { "Nastazia", 0.0 },
  { "Nasteha", 0.0 },
  { "Nasteho", 0.0 },
  { "Nastia", 0.0 },
  { "Nasya", 0.0 },
  { "Nasyah", 0.0 },
  { "Nasyr", 1.0 },
  { "Naszier", 1.0 },
  { "Naszir", 1.0 },
  { "Naszire", 1.0 },
  { "Nat", 1.0 },
  { "Nata", 0.0 },
  { "Natacha", 0.0 },
  { "Natachia", 0.0 },
  { "Natacia", 0.0 },
  { "Natae", 0.0 },
  { "Natahja", 0.0 },
  { "Natahlia", 0.0 },
  { "Natahsa", 0.0 },
  { "Nataija", 0.0 },
  { "Nataile", 0.0 },
  { "Natailia", 0.0 },
  { "Nataisha", 0.0 },
  { "Nataiya", 0.0 },
  { "Nataja", 0.0 },
  { "Natajah", 0.0 },
  { "Natajha", 0.0 },
  { "Natajia", 0.0 },
  { "Nataka", 0.0 },
  { "Nataki", 0.0 },
  { "Natala", 0.0 },
  { "Natale", 0.7560815 },
  { "Natalea", 0.0 },
  { "Nataleah", 0.0 },
  { "Natalee", 0.0 },
  { "Natalei", 0.0 },
  { "Nataleigh", 0.0 },
  { "Natalene", 0.0 },
  { "Nataley", 0.0 },
  { "Natali", 0.0 },
  { "Natalia", 0.00077602576 },
  { "Nataliah", 0.0 },
  { "Natalie", 0.0021867654 },
  { "Natalieann", 0.0 },
  { "Nataliee", 0.0 },
  { "Nataliegh", 0.0 },
  { "Nataliemarie", 0.0 },
  { "Natalierose", 0.0 },
  { "Natalii", 0.0 },
  { "Natalija", 0.0 },
  { "Natalin", 0.0 },
  { "Natalina", 0.0 },
  { "Nataline", 0.0 },
  { "Natalino", 1.0 },
  { "Natalio", 1.0 },
  { "Natalis", 0.0 },
  { "Natalise", 0.0 },
  { "Nataliya", 0.0 },
  { "Nataliyah", 0.0 },
  { "Nataliz", 0.0 },
  { "Natalle", 0.0 },
  { "Natallia", 0.0 },
  { "Natallie", 0.0 },
  { "Natally", 0.0 },
  { "Nataly", 0.0007755545 },
  { "Natalya", 0.0 },
  { "Natalyah", 0.0 },
  { "Natalye", 0.0 },
  { "Natalyia", 0.0 },
  { "Natalyn", 0.0 },
  { "Natalynn", 0.0 },
  { "Natan", 1.0 },
  { "Natanael", 1.0 },
  { "Natane", 0.0 },
  { "Natanel", 1.0 },
  { "Natani", 0.0 },
  { "Natania", 0.0 },
  { "Nataniah", 0.0 },
  { "Nataniel", 1.0 },
  { "Nataniela", 0.0 },
  { "Natanim", 1.0 },
  { "Natanya", 0.0 },
  { "Natara", 0.0 },
  { "Nataria", 0.0 },
  { "Natarsha", 0.0 },
  { "Natarshia", 0.0 },
  { "Natas", 1.0 },
  { "Natasa", 0.0 },
  { "Natascha", 0.0 },
  { "Natash", 0.0 },
  { "Natasha", 0.0029921888 },
  { "Natashi", 0.0 },
  { "Natashia", 0.0 },
  { "Natashua", 0.0 },
  { "Natashya", 0.0 },
  { "Natasia", 0.0 },
  { "Natasja", 0.0 },
  { "Nataska", 0.0 },
  { "Natassha", 0.0 },
  { "Natassia", 0.0 },
  { "Natassja", 0.0 },
  { "Natausha", 0.0 },
  { "Natavia", 0.0 },
  { "Natavion", 1.0 },
  { "Natavious", 1.0 },
  { "Natavius", 1.0 },
  { "Natay", 0.0 },
  { "Nataya", 0.0 },
  { "Natayah", 0.0 },
  { "Natayja", 0.0 },
  { "Natayla", 0.0 },
  { "Nataylia", 0.0 },
  { "Nataysha", 0.0 },
  { "Natayshia", 0.0 },
  { "Nataysia", 0.0 },
  { "Natazia", 0.0 },
  { "Nate", 0.92295957 },
  { "Natea", 0.0 },
  { "Nateal", 0.0 },
  { "Nateasha", 0.0 },
  { "Nateesha", 0.0 },
  { "Nateisha", 0.0 },
  { "Nateka", 0.0 },
  { "Natelee", 0.0 },
  { "Natelege", 0.0 },
  { "Natelie", 0.0 },
  { "Natella", 0.0 },
  { "Natelle", 0.0 },
  { "Natellie", 0.0 },
  { "Nately", 0.0 },
  { "Nateria", 0.0 },
  { "Naterra", 0.0 },
  { "Naterria", 0.0 },
  { "Natesa", 0.0 },
  { "Natesha", 0.0 },
  { "Nateshia", 0.0 },
  { "Natessa", 0.0 },
  { "Nateya", 0.0 },
  { "Nateyah", 0.0 },
  { "Nath", 1.0 },
  { "Natha", 0.09747292 },
  { "Nathael", 1.0 },
  { "Nathaiel", 1.0 },
  { "Nathain", 1.0 },
  { "Nathainel", 1.0 },
  { "Nathale", 0.0 },
  { "Nathalee", 0.0 },
  { "Nathalene", 0.0 },
  { "Nathali", 0.0 },
  { "Nathalia", 0.0 },
  { "Nathalie", 0.0003273144 },
  { "Nathally", 0.0 },
  { "Nathaly", 0.0 },
  { "Nathalya", 0.0 },
  { "Nathalye", 0.0 },
  { "Natham", 1.0 },
  { "Nathan", 0.9967773 },
  { "Nathanael", 0.9993657 },
  { "Nathanaelle", 0.0 },
  { "Nathanal", 1.0 },
  { "Nathane", 1.0 },
  { "Nathaneal", 1.0 },
  { "Nathaneil", 1.0 },
  { "Nathanel", 1.0 },
  { "Nathania", 0.0 },
  { "Nathanial", 1.0 },
  { "Nathanie", 1.0 },
  { "Nathaniel", 0.9956171 },
  { "Nathanieljames", 1.0 },
  { "Nathaniell", 1.0 },
  { "Nathanielle", 1.0 },
  { "Nathanil", 1.0 },
  { "Nathanim", 1.0 },
  { "Nathanjames", 1.0 },
  { "Nathann", 1.0 },
  { "Nathanual", 1.0 },
  { "Nathanuel", 1.0 },
  { "Nathanyal", 1.0 },
  { "Nathanyel", 1.0 },
  { "Nathanyl", 1.0 },
  { "Nathasa", 0.0 },
  { "Nathasha", 0.0 },
  { "Nathasia", 0.0 },
  { "Nathaya", 0.0 },
  { "Nathean", 1.0 },
  { "Nathel", 0.5 },
  { "Nathen", 1.0 },
  { "Nathena", 0.0 },
  { "Nathenial", 1.0 },
  { "Natheniel", 1.0 },
  { "Nathern", 1.0 },
  { "Nathia", 0.0 },
  { "Nathian", 1.0 },
  { "Nathias", 1.0 },
  { "Nathiel", 1.0 },
  { "Nathifa", 0.0 },
  { "Nathin", 1.0 },
  { "Nathina", 0.0 },
  { "Nathinal", 1.0 },
  { "Nathinel", 1.0 },
  { "Nathion", 1.0 },
  { "Nathnael", 1.0 },
  { "Nathniel", 1.0 },
  { "Nathon", 1.0 },
  { "Nathyn", 1.0 },
  { "Natia", 0.0 },
  { "Natiana", 0.0 },
  { "Natica", 0.0 },
  { "Naticha", 0.0 },
  { "Naticia", 0.0 },
  { "Natifah", 0.0 },
  { "Natika", 0.0 },
  { "Natile", 0.0 },
  { "Natilea", 0.0 },
  { "Natilee", 0.0 },
  { "Natileigh", 0.0 },
  { "Natiley", 0.0 },
  { "Natilie", 0.0 },
  { "Natily", 0.0 },
  { "Natilyn", 0.0 },
  { "Natilynn", 0.0 },
  { "Natina", 0.0 },
  { "Nation", 0.9230769 },
  { "Natira", 0.0 },
  { "Natisa", 0.0 },
  { "Natisha", 0.0 },
  { "Natishia", 0.0 },
  { "Natividad", 0.47451088 },
  { "Natiya", 0.0 },
  { "Natiyah", 0.0 },
  { "Natlaie", 0.0 },
  { "Natlee", 0.0 },
  { "Natlie", 0.0 },
  { "Natnael", 1.0 },
  { "Natoma", 0.0 },
  { "Natonia", 0.0 },
  { "Natonya", 0.0 },
  { "Natori", 0.016438356 },
  { "Natoria", 0.0 },
  { "Natorian", 1.0 },
  { "Natosha", 0.0 },
  { "Natoshia", 0.0 },
  { "Natoya", 0.0 },
  { "Natoyia", 0.0 },
  { "Natrell", 1.0 },
  { "Natrice", 0.0 },
  { "Natricia", 0.0 },
  { "Natrina", 0.0 },
  { "Natron", 1.0 },
  { "Natrone", 1.0 },
  { "Natsha", 0.0 },
  { "Natsu", 1.0 },
  { "Natsue", 0.0 },
  { "Natsuki", 0.0 },
  { "Natsuko", 0.0 },
  { "Natsumi", 0.0 },
  { "Natsuyo", 0.0 },
  { "Nattalee", 0.0 },
  { "Nattalie", 0.0 },
  { "Nattaly", 0.0 },
  { "Natthan", 1.0 },
  { "Natthew", 1.0 },
  { "Nattie", 0.0 },
  { "Nattiel", 1.0 },
  { "Natural", 0.0 },
  { "Nature", 0.20779221 },
  { "Naturell", 0.0 },
  { "Naturelle", 0.0 },
  { "Naturi", 0.0 },
  { "Natusha", 0.0 },
  { "Natya", 0.0 },
  { "Natyia", 0.0 },
  { "Natylee", 0.0 },
  { "Natylie", 0.0 },
  { "Natyra", 0.0 },
  { "Nau", 1.0 },
  { "Naudia", 0.0 },
  { "Naudica", 0.0 },
  { "Naudiya", 0.0 },
  { "Naudya", 0.0 },
  { "Naul", 1.0 },
  { "Naum", 1.0 },
  { "Nauman", 1.0 },
  { "Naun", 1.0 },
  { "Naura", 0.0 },
  { "Naureen", 0.0 },
  { "Nausheen", 0.0 },
  { "Nautia", 0.0 },
  { "Nautica", 0.023277467 },
  { "Nautika", 0.0 },
  { "Nautyca", 0.0 },
  { "Nava", 0.0 },
  { "Navada", 0.054054055 },
  { "Navae", 0.0 },
  { "Navaeh", 0.0 },
  { "Navaeha", 0.0 },
  { "Navah", 0.0 },
  { "Navaiah", 0.0 },
  { "Navajo", 1.0 },
  { "Navan", 1.0 },
  { "Navar", 1.0 },
  { "Navario", 1.0 },
  { "Navarion", 1.0 },
  { "Navarius", 1.0 },
  { "Navarre", 1.0 },
  { "Navarro", 1.0 },
  { "Navay", 0.0 },
  { "Navaya", 0.0 },
  { "Navayah", 0.0 },
  { "Navdeep", 0.5283019 },
  { "Navdia", 0.0 },
  { "Nave", 0.0 },
  { "Navea", 0.0 },
  { "Naveah", 0.0 },
  { "Navee", 0.0 },
  { "Naveed", 1.0 },
  { "Naveen", 0.7746815 },
  { "Naveena", 0.0 },
  { "Naveh", 0.0 },
  { "Navell", 1.0 },
  { "Naven", 1.0 },
  { "Navera", 0.0 },
  { "Navey", 0.0 },
  { "Naveya", 0.0 },
  { "Naveyah", 0.0 },
  { "Navi", 0.1748401 },
  { "Navia", 0.0 },
  { "Naviah", 0.0 },
  { "Navian", 1.0 },
  { "Naviana", 0.0 },
  { "Navid", 0.96363634 },
  { "Navidad", 0.0 },
  { "Navie", 0.0 },
  { "Navier", 1.0 },
  { "Navika", 0.0 },
  { "Navil", 0.040983606 },
  { "Navin", 1.0 },
  { "Navina", 0.0 },
  { "Navindra", 1.0 },
  { "Navion", 1.0 },
  { "Navira", 0.0 },
  { "Navisha", 0.0 },
  { "Naviya", 0.0 },
  { "Naviyah", 0.0 },
  { "Navjot", 0.47572815 },
  { "Navleen", 0.0 },
  { "Navneet", 0.10108303 },
  { "Navon", 1.0 },
  { "Navonte", 1.0 },
  { "Navor", 1.0 },
  { "Navpreet", 0.0 },
  { "Navraj", 1.0 },
  { "Navreet", 0.0 },
  { "Navroop", 0.0 },
  { "Navtej", 1.0 },
  { "Navy", 0.17431192 },
  { "Navya", 0.0 },
  { "Naw", 0.0 },
  { "Nawaal", 0.0 },
  { "Nawaf", 1.0 },
  { "Nawal", 0.0 },
  { "Nawana", 0.0 },
  { "Nawar", 0.0 },
  { "Nawatha", 0.0 },
  { "Nawaz", 1.0 },
  { "Naweed", 1.0 },
  { "Nawfal", 1.0 },
  { "Nay", 0.8196721 },
  { "Naya", 0.0 },
  { "Nayab", 0.0 },
  { "Nayah", 0.0 },
  { "Nayala", 0.0 },
  { "Nayalee", 0.0 },
  { "Nayali", 0.0 },
  { "Nayalis", 0.0 },
  { "Nayaliz", 0.0 },
  { "Nayaly", 0.0 },
  { "Nayami", 0.0 },
  { "Nayan", 0.98641306 },
  { "Nayana", 0.0 },
  { "Nayanah", 0.0 },
  { "Nayani", 0.0 },
  { "Nayanna", 0.0 },
  { "Nayara", 0.0 },
  { "Nayari", 0.0 },
  { "Nayarit", 0.0 },
  { "Nayda", 0.0 },
  { "Naydean", 0.0 },
  { "Naydeen", 0.0 },
  { "Naydeli", 0.0 },
  { "Naydelin", 0.0 },
  { "Naydeline", 0.0 },
  { "Naydelyn", 0.0 },
  { "Nayden", 1.0 },
  { "Naydene", 0.0 },
  { "Naydia", 0.0 },
  { "Naydine", 0.0 },
  { "Nayeem", 1.0 },
  { "Nayef", 1.0 },
  { "Nayel", 1.0 },
  { "Nayela", 0.0 },
  { "Nayele", 0.0 },
  { "Nayelee", 0.0 },
  { "Nayeli", 0.0008244023 },
  { "Nayelie", 0.0 },
  { "Nayelii", 0.0 },
  { "Nayelin", 0.0 },
  { "Nayelis", 0.0 },
  { "Nayeliz", 0.0 },
  { "Nayelle", 0.0 },
  { "Nayelli", 0.0 },
  { "Nayellie", 0.0 },
  { "Nayelly", 0.0 },
  { "Nayely", 0.0 },
  { "Nayelys", 0.0 },
  { "Nayeri", 0.0 },
  { "Nayha", 0.0 },
  { "Nayia", 0.0 },
  { "Nayib", 1.0 },
  { "Nayima", 0.0 },
  { "Nayirah", 0.0 },
  { "Nayiri", 0.0 },
  { "Nayisha", 0.0 },
  { "Nayla", 0.0 },
  { "Naylaa", 0.0 },
  { "Naylah", 0.0 },
  { "Naylan", 1.0 },
  { "Naylani", 0.0 },
  { "Naylanie", 0.0 },
  { "Naylea", 0.0 },
  { "Nayleah", 0.0 },
  { "Naylee", 0.0 },
  { "Nayleen", 0.0 },
  { "Naylen", 0.7058824 },
  { "Naylene", 0.0 },
  { "Nayli", 0.0 },
  { "Naylia", 0.0 },
  { "Naylie", 0.0 },
  { "Naylin", 0.0 },
  { "Nayloni", 0.0 },
  { "Naylor", 1.0 },
  { "Nayma", 0.0 },
  { "Naymar", 1.0 },
  { "Naymon", 1.0 },
  { "Nayna", 0.0 },
  { "Nayo", 0.0 },
  { "Nayome", 0.0 },
  { "Nayomee", 0.0 },
  { "Nayomi", 0.0 },
  { "Nayomie", 0.0 },
  { "Nayona", 0.0 },
  { "Nayonika", 0.0 },
  { "Nayonna", 0.0 },
  { "Nayquan", 1.0 },
  { "Nayra", 0.0 },
  { "Naysa", 0.0 },
  { "Naysean", 1.0 },
  { "Nayseth", 0.0 },
  { "Naysha", 0.0 },
  { "Nayshaun", 1.0 },
  { "Nayshawn", 1.0 },
  { "Nayson", 1.0 },
  { "Naytasha", 0.0 },
  { "Naythan", 1.0 },
  { "Naythen", 1.0 },
  { "Naython", 1.0 },
  { "Nayvadius", 1.0 },
  { "Nayvee", 0.0 },
  { "Nayvi", 0.0 },
  { "Nayvia", 0.0 },
  { "Nayvie", 0.0 },
  { "Nayya", 0.0 },
  { "Nayyirah", 0.0 },
  { "Nayzeth", 0.0 },
  { "Naz", 0.11111111 },
  { "Nazaiah", 1.0 },
  { "Nazair", 1.0 },
  { "Nazaire", 1.0 },
  { "Nazalia", 0.0 },
  { "Nazaneen", 0.0 },
  { "Nazanin", 0.0 },
  { "Nazar", 1.0 },
  { "Nazarah", 0.0 },
  { "Nazarena", 0.0 },
  { "Nazarene", 0.0 },
  { "Nazareno", 1.0 },
  { "Nazaret", 0.19653179 },
  { "Nazareth", 0.5898204 },
  { "Nazari", 0.28301886 },
  { "Nazaria", 0.0 },
  { "Nazariah", 0.0 },
  { "Nazario", 1.0 },
  { "Nazariy", 1.0 },
  { "Nazariyah", 0.0 },
  { "Nazarria", 0.0 },
  { "Nazavier", 1.0 },
  { "Nazaya", 0.0 },
  { "Nazayah", 0.0 },
  { "Nazeef", 1.0 },
  { "Nazeeh", 1.0 },
  { "Nazeem", 1.0 },
  { "Nazeer", 1.0 },
  { "Nazeir", 1.0 },
  { "Nazeli", 0.0 },
  { "Nazere", 1.0 },
  { "Nazhae", 0.0 },
  { "Nazhir", 1.0 },
  { "Nazia", 0.0 },
  { "Naziah", 0.32596686 },
  { "Nazier", 1.0 },
  { "Nazifa", 0.0 },
  { "Nazih", 1.0 },
  { "Nazik", 0.0 },
  { "Nazim", 1.0 },
  { "Nazir", 1.0 },
  { "Nazira", 0.0 },
  { "Nazirah", 0.0 },
  { "Nazire", 1.0 },
  { "Nazish", 0.0 },
  { "Naziya", 0.0 },
  { "Naziyah", 0.0 },
  { "Nazli", 0.0 },
  { "Nazly", 0.0 },
  { "Nazmir", 1.0 },
  { "Nazneen", 0.0 },
  { "Nazr", 1.0 },
  { "Nazuri", 0.0 },
  { "Nazya", 0.0 },
  { "Nazyah", 0.0 },
  { "Nazyia", 0.0 },
  { "Nazyiah", 0.0 },
  { "Nazzareno", 1.0 },
  { "Ncholas", 1.0 },
  { "Nciholas", 1.0 },
  { "Ncole", 0.0 },
  { "Ndea", 0.0 },
  { "Ndey", 0.0 },
  { "Ndeye", 0.0 },
  { "Ndia", 0.0 },
  { "Ndidi", 0.0 },
  { "Ndrea", 0.0 },
  { "Ndrew", 1.0 },
  { "Ndya", 0.0 },
  { "Nea", 0.0 },
  { "Neah", 0.0 },
  { "Neajah", 0.0 },
  { "Neako", 1.0 },
  { "Neal", 0.99527264 },
  { "Neala", 0.0 },
  { "Neale", 0.9945711 },
  { "Nealey", 0.0 },
  { "Nealie", 0.021573603 },
  { "Nealon", 1.0 },
  { "Nealy", 0.22615804 },
  { "Neamiah", 1.0 },
  { "Neana", 0.0 },
  { "Neariah", 0.0 },
  { "Neasa", 0.0 },
  { "Neasha", 0.0 },
  { "Neasia", 0.0 },
  { "Neata", 0.0 },
  { "Neatha", 0.0 },
  { "Neavah", 0.0 },
  { "Neave", 0.0 },
  { "Neaveh", 0.0 },
  { "Neaven", 1.0 },
  { "Nebeyu", 1.0 },
  { "Nebil", 1.0 },
  { "Nebiyu", 1.0 },
  { "Nebraska", 0.5375 },
  { "Nebula", 0.0 },
  { "Necalli", 1.0 },
  { "Necha", 0.0 },
  { "Nechama", 0.0 },
  { "Nechelle", 0.0 },
  { "Nechemia", 1.0 },
  { "Nechemiah", 1.0 },
  { "Nechemya", 1.0 },
  { "Nechole", 0.0 },
  { "Nechuma", 0.0 },
  { "Nechy", 0.0 },
  { "Neci", 0.0 },
  { "Necia", 0.0 },
  { "Necie", 0.0 },
  { "Necko", 1.0 },
  { "Neco", 0.8173077 },
  { "Necol", 0.0 },
  { "Necola", 0.0 },
  { "Necole", 0.0 },
  { "Nectarios", 1.0 },
  { "Ned", 0.9997093 },
  { "Neda", 0.0 },
  { "Nedal", 1.0 },
  { "Nedda", 0.0 },
  { "Neddie", 0.3298969 },
  { "Neddy", 1.0 },
  { "Nedenia", 0.0 },
  { "Nedia", 0.0 },
  { "Nedim", 1.0 },
  { "Nedine", 0.0 },
  { "Nedra", 0.0 },
  { "Nedrick", 1.0 },
  { "Nedved", 1.0 },
  { "Neeah", 0.0 },
  { "Neecy", 0.0 },
  { "Needham", 1.0 },
  { "Neeharika", 0.0 },
  { "Neeka", 0.0 },
  { "Neeko", 1.0 },
  { "Neel", 1.0 },
  { "Neela", 0.0 },
  { "Neelah", 0.0 },
  { "Neelam", 0.0 },
  { "Neelan", 1.0 },
  { "Neelee", 0.0 },
  { "Neelesh", 1.0 },
  { "Neeley", 0.0 },
  { "Neeli", 0.0 },
  { "Neelia", 0.0 },
  { "Neelie", 0.0 },
  { "Neelima", 0.0 },
  { "Neelisha", 0.0 },
  { "Neely", 0.16652434 },
  { "Neema", 0.35555556 },
  { "Neemah", 0.0 },
  { "Neemias", 1.0 },
  { "Neena", 0.0 },
  { "Neenah", 0.0 },
  { "Neepa", 0.0 },
  { "Neer", 1.0 },
  { "Neera", 0.0 },
  { "Neeraj", 1.0 },
  { "Neeraja", 0.0 },
  { "Neerav", 1.0 },
  { "Neeru", 0.0 },
  { "Neesa", 0.0 },
  { "Neesha", 0.0 },
  { "Neeson", 1.0 },
  { "Neeta", 0.0 },
  { "Neethu", 0.0 },
  { "Neeti", 0.0 },
  { "Neetu", 0.0 },
  { "Neev", 1.0 },
  { "Neeva", 0.0 },
  { "Neeve", 0.0 },
  { "Neeya", 0.0 },
  { "Neeyah", 0.0 },
  { "Nefateria", 0.0 },
  { "Nefeli", 0.0 },
  { "Nefertari", 0.0 },
  { "Nefertary", 0.0 },
  { "Neferteria", 0.0 },
  { "Nefertiti", 0.0 },
  { "Nefeteria", 0.0 },
  { "Neff", 1.0 },
  { "Nefi", 1.0 },
  { "Nefretiri", 0.0 },
  { "Neftali", 0.8371196 },
  { "Neftaly", 0.67863894 },
  { "Negan", 0.8 },
  { "Negar", 0.0 },
  { "Negeen", 0.0 },
  { "Neghan", 0.0 },
  { "Negin", 0.0 },
  { "Negus", 1.0 },
  { "Neha", 0.0 },
  { "Nehal", 0.33333334 },
  { "Nehamiah", 1.0 },
  { "Nehan", 1.0 },
  { "Neharika", 0.0 },
  { "Nehemia", 1.0 },
  { "Nehemiah", 0.972448 },
  { "Nehemias", 1.0 },
  { "Nehemie", 0.0 },
  { "Nehemiyah", 1.0 },
  { "Nehemyah", 1.0 },
  { "Nehimiah", 1.0 },
  { "Nehir", 0.0 },
  { "Nehki", 1.0 },
  { "Nehorai", 1.0 },
  { "Neia", 0.0 },
  { "Neiah", 0.0 },
  { "Neiba", 0.0 },
  { "Neicha", 0.0 },
  { "Neida", 0.0 },
  { "Neidelin", 0.0 },
  { "Neidelyn", 0.0 },
  { "Neidi", 0.0 },
  { "Neidra", 0.0 },
  { "Neidy", 0.0 },
  { "Neika", 0.0 },
  { "Neiko", 1.0 },
  { "Neil", 0.9962193 },
  { "Neila", 0.0 },
  { "Neilah", 0.0 },
  { "Neilan", 1.0 },
  { "Neilani", 0.0 },
  { "Neile", 0.0 },
  { "Neilesh", 1.0 },
  { "Neili", 0.0 },
  { "Neilie", 0.0 },
  { "Neilina", 0.0 },
  { "Neill", 1.0 },
  { "Neilla", 0.0 },
  { "Neilpatrick", 1.0 },
  { "Neils", 1.0 },
  { "Neilson", 1.0 },
  { "Neily", 0.0 },
  { "Neilyn", 0.0 },
  { "Neima", 0.083333336 },
  { "Neiman", 0.9837133 },
  { "Neimar", 1.0 },
  { "Neimiah", 1.0 },
  { "Neina", 0.0 },
  { "Neira", 0.0 },
  { "Neisa", 0.0 },
  { "Neisha", 0.0 },
  { "Neishaly", 0.0 },
  { "Neissa", 0.0 },
  { "Neita", 0.0 },
  { "Neithan", 1.0 },
  { "Neithen", 1.0 },
  { "Neiva", 0.0 },
  { "Neiya", 0.0 },
  { "Neizan", 1.0 },
  { "Neji", 1.0 },
  { "Nejla", 0.0 },
  { "Nejra", 0.0 },
  { "Neka", 0.0 },
  { "Nekaya", 0.0 },
  { "Nekaybaw", 0.0 },
  { "Nekayla", 0.0 },
  { "Nekea", 0.0 },
  { "Nekeda", 0.0 },
  { "Nekedra", 0.0 },
  { "Nekeia", 0.0 },
  { "Nekeidra", 0.0 },
  { "Nekeisha", 0.0 },
  { "Nekesha", 0.0 },
  { "Nekeshia", 0.0 },
  { "Nekesia", 0.0 },
  { "Neketa", 0.0 },
  { "Nekeya", 0.0 },
  { "Nekeysha", 0.0 },
  { "Nekhi", 1.0 },
  { "Nekia", 0.05807201 },
  { "Nekiesha", 0.0 },
  { "Nekira", 0.0 },
  { "Nekisha", 0.0 },
  { "Nekishia", 0.0 },
  { "Nekita", 0.0 },
  { "Nekiya", 0.0 },
  { "Nekiyah", 0.0 },
  { "Nekki", 0.0 },
  { "Nekko", 1.0 },
  { "Neko", 0.78241265 },
  { "Nekoda", 0.835443 },
  { "Nekol", 0.0 },
  { "Nekole", 0.0 },
  { "Nektarios", 1.0 },
  { "Nekya", 0.0 },
  { "Nel", 0.0 },
  { "Nela", 0.0 },
  { "Nelah", 0.0 },
  { "Nelani", 0.0 },
  { "Nelba", 0.0 },
  { "Nelcy", 0.0 },
  { "Nelda", 0.00024880574 },
  { "Neldean", 0.0 },
  { "Neldon", 1.0 },
  { "Neleah", 0.0 },
  { "Neleh", 0.0 },
  { "Neleigh", 0.0 },
  { "Nelena", 0.0 },
  { "Neli", 0.0 },
  { "Nelia", 0.0 },
  { "Neliah", 0.0 },
  { "Nelida", 0.0 },
  { "Nelie", 0.0 },
  { "Nelina", 0.0 },
  { "Nelisa", 0.0 },
  { "Nelissa", 0.0 },
  { "Nelita", 0.0 },
  { "Nelitza", 0.0 },
  { "Neliyah", 0.0 },
  { "Nell", 0.0036654698 },
  { "Nella", 0.0 },
  { "Nellann", 0.0 },
  { "Nellda", 0.0 },
  { "Nelle", 0.0 },
  { "Nellean", 0.0 },
  { "Nellee", 0.0 },
  { "Nellene", 0.0 },
  { "Nelli", 0.0 },
  { "Nellia", 0.0 },
  { "Nellie", 0.0039605414 },
  { "Nelliel", 0.0 },
  { "Nelliemae", 0.0 },
  { "Nelline", 0.0 },
  { "Nellis", 0.5 },
  { "Nello", 0.98056155 },
  { "Nellora", 0.0 },
  { "Nellwyn", 0.0 },
  { "Nelly", 0.006674757 },
  { "Nellye", 0.0 },
  { "Nelma", 0.0 },
  { "Nelo", 1.0 },
  { "Neloise", 0.0 },
  { "Nelrose", 0.0 },
  { "Nels", 1.0 },
  { "Nelsa", 0.0 },
  { "Nelse", 1.0 },
  { "Nelsi", 0.0 },
  { "Nelsie", 0.0 },
  { "Nelson", 0.9947187 },
  { "Nelsy", 0.0 },
  { "Nelta", 0.0 },
  { "Nelton", 1.0 },
  { "Nelva", 0.0 },
  { "Nelvia", 0.0 },
  { "Nelvie", 0.0 },
  { "Nelvin", 0.969697 },
  { "Nelvina", 0.0 },
  { "Nelwyn", 0.0 },
  { "Nely", 0.0 },
  { "Nelya", 0.0 },
  { "Nema", 0.058441557 },
  { "Nemanja", 1.0 },
  { "Nemecio", 1.0 },
  { "Nemesio", 1.0 },
  { "Nemesis", 0.0 },
  { "Nemiah", 0.7297297 },
  { "Nemo", 1.0 },
  { "Nemya", 0.0 },
  { "Nena", 0.0 },
  { "Nenad", 1.0 },
  { "Nenah", 0.0 },
  { "Nency", 0.0 },
  { "Nene", 0.0 },
  { "Neneh", 0.0 },
  { "Nenette", 0.0 },
  { "Neng", 0.939759 },
  { "Nenia", 0.0 },
  { "Neno", 1.0 },
  { "Neo", 1.0 },
  { "Neola", 0.0 },
  { "Neolani", 0.0 },
  { "Neoma", 0.0 },
  { "Neomi", 0.0 },
  { "Neomia", 0.0 },
  { "Neomiah", 1.0 },
  { "Neomie", 0.0 },
  { "Neomy", 0.0 },
  { "Neon", 1.0 },
  { "Neona", 0.0 },
  { "Neorah", 0.0 },
  { "Neosha", 0.0 },
  { "Neoshia", 0.0 },
  { "Nephateria", 0.0 },
  { "Nephi", 1.0 },
  { "Nephtali", 0.9541985 },
  { "Nephtalie", 0.0 },
  { "Nepoleon", 1.0 },
  { "Neppie", 0.0 },
  { "Neptali", 1.0 },
  { "Nequan", 1.0 },
  { "Nequila", 0.0 },
  { "Nequita", 0.0 },
  { "Nera", 0.0 },
  { "Nerea", 0.0 },
  { "Nereida", 0.0016551724 },
  { "Nerely", 0.0 },
  { "Nerene", 0.0 },
  { "Nereo", 1.0 },
  { "Nereyda", 0.0 },
  { "Neri", 0.7782369 },
  { "Neria", 0.0 },
  { "Neriah", 0.016129032 },
  { "Neric", 1.0 },
  { "Nerick", 1.0 },
  { "Nerida", 0.0 },
  { "Nerik", 1.0 },
  { "Nerina", 0.0 },
  { "Nerine", 0.0 },
  { "Neris", 1.0 },
  { "Nerisa", 0.0 },
  { "Nerissa", 0.0 },
  { "Nerita", 0.0 },
  { "Neriya", 0.0 },
  { "Neriyah", 0.0 },
  { "Nerlande", 0.0 },
  { "Nerma", 0.0 },
  { "Nermeen", 0.0 },
  { "Nermin", 0.72727275 },
  { "Nero", 1.0 },
  { "Neron", 1.0 },
  { "Nerriah", 0.0 },
  { "Nerrissa", 0.0 },
  { "Nerses", 1.0 },
  { "Nerva", 0.0 },
  { "Nervia", 0.0 },
  { "Nervie", 0.0 },
  { "Nery", 0.83270675 },
  { "Nerya", 0.0 },
  { "Nerys", 0.0 },
  { "Nesa", 0.0 },
  { "Nesanel", 1.0 },
  { "Nesbit", 0.775 },
  { "Nesbitt", 1.0 },
  { "Nesby", 1.0 },
  { "Nesean", 1.0 },
  { "Nesha", 0.0 },
  { "Neshama", 0.0 },
  { "Neshanta", 0.0 },
  { "Neshaun", 1.0 },
  { "Neshawn", 0.85507244 },
  { "Neshay", 0.0 },
  { "Neshell", 0.0 },
  { "Neshelle", 0.0 },
  { "Neshia", 0.0 },
  { "Nesia", 0.0 },
  { "Nesiah", 0.056818184 },
  { "Nesita", 0.0 },
  { "Nesly", 0.09615385 },
  { "Nesma", 0.0 },
  { "Nesochi", 0.0 },
  { "Nesreen", 0.0 },
  { "Nesren", 0.0 },
  { "Ness", 1.0 },
  { "Nessa", 0.0 },
  { "Nessiah", 0.41333333 },
  { "Nessie", 0.0 },
  { "Nesta", 0.53488374 },
  { "Nester", 1.0 },
  { "Nestor", 1.0 },
  { "Nestora", 0.0 },
  { "Nesya", 0.0 },
  { "Neta", 0.0 },
  { "Netanel", 1.0 },
  { "Netania", 0.0 },
  { "Netanya", 0.0 },
  { "Netasha", 0.0 },
  { "Netha", 0.0 },
  { "Nethan", 1.0 },
  { "Nethanel", 1.0 },
  { "Nethania", 0.0 },
  { "Nethaniah", 1.0 },
  { "Nethaniel", 1.0 },
  { "Nether", 0.0 },
  { "Nethra", 0.0 },
  { "Netia", 0.0 },
  { "Netisha", 0.0 },
  { "Netra", 0.0 },
  { "Netta", 0.0 },
  { "Nette", 0.0 },
  { "Netter", 0.0 },
  { "Nettie", 0.0017827479 },
  { "Netty", 0.0 },
  { "Nettye", 0.0 },
  { "Neuman", 1.0 },
  { "Nev", 0.0 },
  { "Neva", 0.0002910502 },
  { "Nevaan", 1.0 },
  { "Nevach", 0.0 },
  { "Nevada", 0.1810934 },
  { "Nevae", 0.0 },
  { "Nevaeh", 0.005412813 },
  { "Nevaeha", 0.0 },
  { "Nevaehia", 0.0 },
  { "Nevaehlee", 0.0 },
  { "Nevaehly", 0.0 },
  { "Nevaehlynn", 0.0 },
  { "Nevaehmarie", 0.0 },
  { "Nevaehtnes", 0.0 },
  { "Nevaen", 0.0 },
  { "Nevah", 0.0 },
  { "Nevaha", 0.0 },
  { "Nevaiah", 0.0 },
  { "Nevan", 0.9951028 },
  { "Nevara", 0.0 },
  { "Nevart", 0.0 },
  { "Nevaya", 0.0 },
  { "Nevayah", 0.0 },
  { "Nevayiah", 0.0 },
  { "Neve", 0.0 },
  { "Nevea", 0.0 },
  { "Neveah", 0.0 },
  { "Neveaha", 0.0 },
  { "Neveen", 0.0 },
  { "Nevel", 0.5925926 },
  { "Nevelle", 1.0 },
  { "Nevelyn", 0.0 },
  { "Neven", 0.98255813 },
  { "Nevena", 0.0 },
  { "Neveya", 0.0 },
  { "Neveyah", 0.0 },
  { "Nevia", 0.0 },
  { "Neviah", 0.0 },
  { "Nevil", 1.0 },
  { "Neville", 0.9675886 },
  { "Nevin", 0.98439574 },
  { "Nevina", 0.0 },
  { "Nevine", 0.0 },
  { "Nevins", 1.0 },
  { "Nevis", 0.0 },
  { "Neviyah", 0.0 },
  { "Nevo", 1.0 },
  { "Nevon", 1.0 },
  { "Nevyn", 0.9583333 },
  { "Newana", 0.0 },
  { "Newbern", 1.0 },
  { "Newborn", 0.62068963 },
  { "Newel", 1.0 },
  { "Newell", 0.9760896 },
  { "Newland", 1.0 },
  { "Newlin", 1.0 },
  { "Newman", 1.0 },
  { "Newsom", 1.0 },
  { "Newt", 1.0 },
  { "Newton", 0.9993702 },
  { "Nexus", 1.0 },
  { "Ney", 1.0 },
  { "Neya", 0.0 },
  { "Neyah", 0.0 },
  { "Neyamiah", 0.45454547 },
  { "Neyan", 1.0 },
  { "Neyba", 0.0 },
  { "Neyda", 0.0 },
  { "Neydelin", 0.0 },
  { "Neydi", 0.0 },
  { "Neyla", 0.0 },
  { "Neylah", 0.0 },
  { "Neylan", 0.39240506 },
  { "Neyland", 1.0 },
  { "Neylani", 0.0 },
  { "Neylin", 0.0 },
  { "Neymar", 0.9974568 },
  { "Neyo", 1.0 },
  { "Neyra", 0.0 },
  { "Neysa", 0.0 },
  { "Neysha", 0.0 },
  { "Neythan", 1.0 },
  { "Neytiri", 0.0 },
  { "Neyva", 0.0 },
  { "Neyzer", 1.0 },
  { "Nezar", 1.0 },
  { "Neziah", 0.51445085 },
  { "Neziyah", 0.0 },
  { "Nezzie", 0.0 },
  { "Ng", 1.0 },
  { "Nga", 0.0 },
  { "Ngai", 1.0 },
  { "Ngan", 0.0 },
  { "Ngawang", 0.68292683 },
  { "Ngela", 0.0 },
  { "Nghi", 0.18518518 },
  { "Nghia", 0.9768519 },
  { "Ngina", 0.0 },
  { "Ngoc", 0.077809796 },
  { "Ngocanh", 0.0 },
  { "Ngone", 0.0 },
  { "Ngozi", 0.04525862 },
  { "Ngozichukwu", 0.0 },
  { "Ngun", 0.12195122 },
  { "Nguyen", 0.78880864 },
  { "Nguyet", 0.0 },
  { "Nha", 1.0 },
  { "Nhala", 0.0 },
  { "Nhan", 0.9856322 },
  { "Nhat", 1.0 },
  { "Nhi", 0.0 },
  { "Nhia", 0.86938775 },
  { "Nhu", 0.0 },
  { "Nhung", 0.0 },
  { "Nhut", 1.0 },
  { "Nhuy", 0.0 },
  { "Nhya", 0.0 },
  { "Nhyira", 0.0 },
  { "Nhyla", 0.0 },
  { "Ni", 0.0 },
  { "Nia", 0.00048666538 },
  { "Niah", 0.0 },
  { "Niaimani", 0.0 },
  { "Niaira", 0.0 },
  { "Niaisha", 0.0 },
  { "Niaja", 0.0 },
  { "Niajah", 0.0 },
  { "Niaje", 0.0 },
  { "Niakia", 0.0 },
  { "Nial", 1.0 },
  { "Niala", 0.0 },
  { "Nialah", 0.0 },
  { "Nialani", 0.0 },
  { "Niall", 1.0 },
  { "Nialynn", 0.0 },
  { "Niam", 1.0 },
  { "Niama", 0.0 },
  { "Niamalika", 0.0 },
  { "Niamani", 0.0 },
  { "Niamarie", 0.0 },
  { "Niambi", 0.0 },
  { "Niamh", 0.0 },
  { "Niamiah", 0.0 },
  { "Niamke", 1.0 },
  { "Niamora", 0.0 },
  { "Niamya", 0.0 },
  { "Nian", 0.0 },
  { "Niana", 0.0 },
  { "Niang", 0.0 },
  { "Niani", 0.0 },
  { "Nianna", 0.0 },
  { "Niaomi", 0.0 },
  { "Niara", 0.0 },
  { "Niarah", 0.0 },
  { "Niari", 0.0 },
  { "Niarose", 0.0 },
  { "Niasha", 0.0 },
  { "Niasia", 0.0 },
  { "Niaya", 0.0 },
  { "Niayla", 0.0 },
  { "Niaylah", 0.0 },
  { "Niaz", 1.0 },
  { "Nic", 1.0 },
  { "Nica", 0.0 },
  { "Nicaela", 0.0 },
  { "Nicala", 0.0 },
  { "Nicandro", 1.0 },
  { "Nicanor", 1.0 },
  { "Nicanora", 0.0 },
  { "Nicara", 0.0 },
  { "Nicasia", 0.0 },
  { "Nicasio", 1.0 },
  { "Nicaya", 0.0 },
  { "Nicayla", 0.0 },
  { "Nicci", 0.0 },
  { "Nicco", 0.9941176 },
  { "Niccola", 0.0 },
  { "Niccolas", 1.0 },
  { "Niccole", 0.0 },
  { "Niccoli", 1.0 },
  { "Niccolo", 1.0 },
  { "Nicere", 1.0 },
  { "Nicey", 0.0 },
  { "Nicha", 0.0 },
  { "Nichael", 1.0 },
  { "Nichaela", 0.0 },
  { "Nichalas", 1.0 },
  { "Nichalaus", 1.0 },
  { "Nichalos", 1.0 },
  { "Nichalous", 1.0 },
  { "Nichalus", 1.0 },
  { "Nichaolas", 1.0 },
  { "Nichcole", 0.0 },
  { "Niche", 0.0 },
  { "Nichel", 0.0 },
  { "Nichele", 0.0 },
  { "Nichell", 0.0 },
  { "Nichella", 0.0 },
  { "Nichelle", 0.0009179365 },
  { "Nichet", 0.0 },
  { "Nichlas", 1.0 },
  { "Nichlaus", 1.0 },
  { "Nichloas", 1.0 },
  { "Nichlos", 1.0 },
  { "Nichlous", 1.0 },
  { "Nicho", 1.0 },
  { "Nichoal", 0.0 },
  { "Nichoals", 1.0 },
  { "Nichoas", 1.0 },
  { "Nichoel", 0.0 },
  { "Nichol", 0.0075290897 },
  { "Nichola", 0.12828283 },
  { "Nicholaas", 1.0 },
  { "Nicholad", 1.0 },
  { "Nicholai", 1.0 },
  { "Nicholaos", 1.0 },
  { "Nicholas", 0.99651676 },
  { "Nicholaus", 1.0 },
  { "Nichole", 0.0029026202 },
  { "Nicholes", 1.0 },
  { "Nicholetta", 0.0 },
  { "Nicholette", 0.0 },
  { "Nicholi", 1.0 },
  { "Nicholina", 0.0 },
  { "Nicholis", 1.0 },
  { "Nicholl", 0.0 },
  { "Nichollas", 1.0 },
  { "Nicholle", 0.0 },
  { "Nichollette", 0.0 },
  { "Nicholos", 1.0 },
  { "Nicholous", 1.0 },
  { "Nichols", 1.0 },
  { "Nicholsa", 1.0 },
  { "Nicholson", 1.0 },
  { "Nicholus", 1.0 },
  { "Nicia", 0.0 },
  { "Niciah", 0.0 },
  { "Nicie", 0.0 },
  { "Nick", 0.997326 },
  { "Nickai", 1.0 },
  { "Nickalas", 1.0 },
  { "Nickalaus", 1.0 },
  { "Nickales", 1.0 },
  { "Nickali", 1.0 },
  { "Nickalis", 1.0 },
  { "Nickalos", 1.0 },
  { "Nickalous", 1.0 },
  { "Nickalus", 1.0 },
  { "Nickan", 1.0 },
  { "Nickayla", 0.0 },
  { "Nickcola", 0.0 },
  { "Nickcole", 0.0 },
  { "Nickea", 0.0 },
  { "Nickelas", 1.0 },
  { "Nickell", 0.0 },
  { "Nickelle", 0.0 },
  { "Nickelous", 1.0 },
  { "Nickesha", 0.0 },
  { "Nicketa", 0.0 },
  { "Nickey", 0.83341694 },
  { "Nickeya", 0.0 },
  { "Nickhil", 1.0 },
  { "Nickholas", 1.0 },
  { "Nicki", 0.04680851 },
  { "Nickia", 0.0 },
  { "Nickie", 0.31301698 },
  { "Nickisha", 0.0 },
  { "Nickita", 0.3442623 },
  { "Nickiya", 0.0 },
  { "Nickiyah", 0.0 },
  { "Nickki", 0.0 },
  { "Nicklas", 1.0 },
  { "Nicklaus", 1.0 },
  { "Nickles", 1.0 },
  { "Nickloas", 1.0 },
  { "Nicklos", 1.0 },
  { "Nicklous", 1.0 },
  { "Nicklus", 1.0 },
  { "Nicko", 0.9670782 },
  { "Nickohlas", 1.0 },
  { "Nickol", 0.0 },
  { "Nickola", 0.036764707 },
  { "Nickolai", 1.0 },
  { "Nickolaos", 1.0 },
  { "Nickolas", 0.9987574 },
  { "Nickolaus", 1.0 },
  { "Nickole", 0.0 },
  { "Nickoles", 1.0 },
  { "Nickolette", 0.0 },
  { "Nickoli", 1.0 },
  { "Nickolie", 1.0 },
  { "Nickolis", 1.0 },
  { "Nickolos", 1.0 },
  { "Nickolous", 1.0 },
  { "Nickolus", 1.0 },
  { "Nickon", 1.0 },
  { "Nickos", 1.0 },
  { "Nickson", 1.0 },
  { "Nicky", 0.7946583 },
  { "Nickyla", 0.0 },
  { "Niclas", 1.0 },
  { "Niclaus", 1.0 },
  { "Nicle", 0.0 },
  { "Nicloe", 0.0 },
  { "Niclole", 0.0 },
  { "Nico", 0.9314473 },
  { "Nicodemo", 1.0 },
  { "Nicodemus", 1.0 },
  { "Nicoe", 0.0 },
  { "Nicoel", 0.0 },
  { "Nicohlas", 1.0 },
  { "Nicohle", 0.0 },
  { "Nicol", 0.013429256 },
  { "Nicola", 0.35479668 },
  { "Nicolaas", 1.0 },
  { "Nicolae", 1.0 },
  { "Nicolai", 1.0 },
  { "Nicolaos", 1.0 },
  { "Nicolas", 0.99792075 },
  { "Nicolasa", 0.0 },
  { "Nicolaus", 1.0 },
  { "Nicole", 0.0033377188 },
  { "Nicoleann", 0.0 },
  { "Nicoleanne", 0.0 },
  { "Nicolee", 0.0 },
  { "Nicolemarie", 0.0 },
  { "Nicolena", 0.0 },
  { "Nicolene", 0.0 },
  { "Nicoles", 1.0 },
  { "Nicolet", 0.0 },
  { "Nicoleta", 0.0 },
  { "Nicolete", 0.0 },
  { "Nicolett", 0.0 },
  { "Nicoletta", 0.0 },
  { "Nicolette", 0.0003118179 },
  { "Nicoli", 0.89285713 },
  { "Nicolia", 0.0 },
  { "Nicolina", 0.0 },
  { "Nicoline", 0.0 },
  { "Nicolino", 1.0 },
  { "Nicolis", 1.0 },
  { "Nicoll", 0.0 },
  { "Nicollas", 1.0 },
  { "Nicolle", 0.0 },
  { "Nicollette", 0.0 },
  { "Nicolly", 0.0 },
  { "Nicolo", 1.0 },
  { "Nicolos", 1.0 },
  { "Nicolus", 1.0 },
  { "Nicoly", 0.0 },
  { "Nicolyn", 0.0 },
  { "Nicomedes", 1.0 },
  { "Nicos", 1.0 },
  { "Nicosha", 0.0 },
  { "Nicosia", 0.0 },
  { "Nicoya", 0.0 },
  { "Nicquan", 1.0 },
  { "Nicson", 1.0 },
  { "Nicte", 0.0 },
  { "Nicteha", 0.0 },
  { "Nicy", 0.0 },
  { "Nida", 0.0 },
  { "Nidal", 1.0 },
  { "Nidhi", 0.0 },
  { "Nidhish", 1.0 },
  { "Nidia", 0.0 },
  { "Nidish", 1.0 },
  { "Nidra", 0.0 },
  { "Nidya", 0.0 },
  { "Nieasha", 0.0 },
  { "Nieco", 1.0 },
  { "Nieem", 1.0 },
  { "Niegel", 1.0 },
  { "Nieisha", 0.0 },
  { "Nieja", 0.0 },
  { "Nieka", 0.0 },
  { "Niekia", 0.0 },
  { "Nieko", 1.0 },
  { "Niel", 1.0 },
  { "Niela", 0.0 },
  { "Nielah", 0.0 },
  { "Nielle", 0.0 },
  { "Niels", 1.0 },
  { "Nielsen", 1.0 },
  { "Niem", 1.0 },
  { "Niema", 0.0 },
  { "Niemah", 0.0 },
  { "Nieman", 1.0 },
  { "Niera", 0.0 },
  { "Nierra", 0.0 },
  { "Niesa", 0.0 },
  { "Niesha", 0.0 },
  { "Nieshia", 0.0 },
  { "Nieta", 0.0 },
  { "Nieva", 0.0 },
  { "Nieve", 0.0 },
  { "Nieves", 0.3786531 },
  { "Nieya", 0.0 },
  { "Nigal", 1.0 },
  { "Nigee", 1.0 },
  { "Nigel", 0.9926247 },
  { "Nigell", 1.0 },
  { "Nigella", 0.0 },
  { "Niger", 0.30555555 },
  { "Nigeria", 0.0 },
  { "Nigiel", 1.0 },
  { "Nigil", 1.0 },
  { "Niguel", 1.0 },
  { "Niha", 0.0 },
  { "Nihaal", 0.939759 },
  { "Nihaan", 1.0 },
  { "Nihad", 0.0 },
  { "Nihal", 0.8033794 },
  { "Nihan", 1.0 },
  { "Nihar", 1.0 },
  { "Nihara", 0.0 },
  { "Niharika", 0.0 },
  { "Nihash", 1.0 },
  { "Nihira", 0.0 },
  { "Nihit", 1.0 },
  { "Nihitha", 0.0 },
  { "Nihla", 0.0 },
  { "Nihya", 0.0 },
  { "Nii", 1.0 },
  { "Niia", 0.0 },
  { "Niilo", 1.0 },
  { "Nija", 0.022809124 },
  { "Nijae", 0.08152174 },
  { "Nijah", 0.12098009 },
  { "Nijai", 0.0 },
  { "Nijal", 1.0 },
  { "Nijash", 1.0 },
  { "Nijay", 0.5394737 },
  { "Nijaya", 0.0 },
  { "Nijayah", 0.0 },
  { "Nijee", 1.0 },
  { "Nijel", 1.0 },
  { "Nijeria", 0.0 },
  { "Nijha", 0.0 },
  { "Nijia", 0.0 },
  { "Nijiah", 0.0 },
  { "Nijon", 1.0 },
  { "Nik", 1.0 },
  { "Nika", 0.0025773195 },
  { "Nikaela", 0.0 },
  { "Nikai", 0.9514563 },
  { "Nikaila", 0.0 },
  { "Nikaiya", 0.0 },
  { "Nikaiyah", 0.0 },
  { "Nikala", 0.0 },
  { "Nikalas", 1.0 },
  { "Nikalette", 0.0 },
  { "Nikali", 1.0 },
  { "Nikalus", 1.0 },
  { "Nikan", 1.0 },
  { "Nikara", 0.0 },
  { "Nikash", 1.0 },
  { "Nikasha", 0.0 },
  { "Nikata", 0.0 },
  { "Nikaya", 0.0 },
  { "Nikayah", 0.0 },
  { "Nikayia", 0.0 },
  { "Nikayla", 0.0 },
  { "Nikaylah", 0.0 },
  { "Nikcole", 0.0 },
  { "Nike", 0.5042373 },
  { "Nikea", 0.0 },
  { "Nikeea", 0.0 },
  { "Nikeem", 1.0 },
  { "Nikeesha", 0.0 },
  { "Nikeeta", 0.0 },
  { "Nikeia", 0.0 },
  { "Nikeisha", 0.0 },
  { "Nikeita", 0.0 },
  { "Nikel", 0.6666667 },
  { "Nikela", 0.0 },
  { "Nikelle", 0.0 },
  { "Nikema", 0.0 },
  { "Nikera", 0.0 },
  { "Nikeria", 0.0 },
  { "Nikerria", 0.0 },
  { "Nikesh", 1.0 },
  { "Nikesha", 0.0 },
  { "Nikeshia", 0.0 },
  { "Niket", 1.0 },
  { "Niketa", 0.0 },
  { "Niketh", 1.0 },
  { "Niketta", 0.0 },
  { "Nikeya", 0.0 },
  { "Nikeyia", 0.0 },
  { "Nikhaule", 0.0 },
  { "Nikhia", 0.0 },
  { "Nikhil", 1.0 },
  { "Nikhila", 0.0 },
  { "Nikhilesh", 1.0 },
  { "Nikhita", 0.0 },
  { "Nikholas", 1.0 },
  { "Niki", 0.03216292 },
  { "Nikia", 0.062121212 },
  { "Nikiah", 0.0 },
  { "Nikida", 0.0 },
  { "Nikie", 0.0 },
  { "Nikiea", 0.0 },
  { "Nikiesha", 0.0 },
  { "Nikiki", 0.0 },
  { "Nikil", 1.0 },
  { "Nikila", 0.0 },
  { "Nikima", 0.0 },
  { "Nikina", 0.0 },
  { "Nikira", 0.0 },
  { "Nikisha", 0.0 },
  { "Nikishia", 0.0 },
  { "Nikit", 1.0 },
  { "Nikita", 0.18508917 },
  { "Nikitas", 1.0 },
  { "Nikith", 1.0 },
  { "Nikitha", 0.0 },
  { "Nikitia", 0.0 },
  { "Nikitra", 0.0 },
  { "Nikitta", 0.0 },
  { "Nikiya", 0.0 },
  { "Nikiyah", 0.0 },
  { "Nikka", 0.0 },
  { "Nikkea", 0.0 },
  { "Nikkei", 0.0 },
  { "Nikkeya", 0.0 },
  { "Nikki", 0.022028612 },
  { "Nikkia", 0.008264462 },
  { "Nikkiah", 0.0 },
  { "Nikkie", 0.0 },
  { "Nikkii", 0.0 },
  { "Nikkisha", 0.0 },
  { "Nikkita", 0.0 },
  { "Nikkitta", 0.0 },
  { "Nikkiya", 0.0 },
  { "Nikko", 0.9467124 },
  { "Nikkol", 0.0 },
  { "Nikkolai", 1.0 },
  { "Nikkolas", 1.0 },
  { "Nikkolaus", 1.0 },
  { "Nikkole", 0.0 },
  { "Nikkolette", 0.0 },
  { "Nikkoli", 1.0 },
  { "Nikkos", 1.0 },
  { "Nikky", 0.0 },
  { "Niklas", 1.0 },
  { "Niklaus", 1.0 },
  { "Niko", 0.9467094 },
  { "Nikodem", 1.0 },
  { "Nikohl", 0.0 },
  { "Nikol", 0.0 },
  { "Nikola", 0.79844964 },
  { "Nikolaas", 1.0 },
  { "Nikolai", 1.0 },
  { "Nikolaj", 1.0 },
  { "Nikolaos", 1.0 },
  { "Nikolas", 0.9992517 },
  { "Nikolaus", 1.0 },
  { "Nikolay", 1.0 },
  { "Nikole", 0.0 },
  { "Nikoles", 1.0 },
  { "Nikoleta", 0.0 },
  { "Nikolett", 0.0 },
  { "Nikoletta", 0.0 },
  { "Nikolette", 0.0 },
  { "Nikoli", 1.0 },
  { "Nikolia", 1.0 },
  { "Nikolija", 0.0 },
  { "Nikolina", 0.0 },
  { "Nikolis", 1.0 },
  { "Nikoll", 1.0 },
  { "Nikolle", 0.0 },
  { "Nikolos", 1.0 },
  { "Nikoloz", 1.0 },
  { "Nikolus", 1.0 },
  { "Nikoma", 0.0 },
  { "Nikos", 1.0 },
  { "Nikshay", 1.0 },
  { "Nikson", 1.0 },
  { "Nikte", 0.0 },
  { "Nikul", 1.0 },
  { "Nikunj", 1.0 },
  { "Nikya", 0.0 },
  { "Nikyah", 0.0 },
  { "Nikyia", 0.0 },
  { "Nikyla", 0.0 },
  { "Nikylah", 0.0 },
  { "Nikyra", 0.0 },
  { "Nikyta", 0.0 },
  { "Nil", 0.74590164 },
  { "Nila", 0.0 },
  { "Nilah", 0.0 },
  { "Nilaja", 0.0 },
  { "Nilajah", 0.0 },
  { "Nilam", 0.0 },
  { "Nilan", 1.0 },
  { "Nilani", 0.0 },
  { "Nilay", 0.9777778 },
  { "Nilaya", 0.0 },
  { "Nilayah", 0.0 },
  { "Nilda", 0.0 },
  { "Nile", 0.92124486 },
  { "Nileah", 0.0 },
  { "Nilene", 0.0 },
  { "Niles", 1.0 },
  { "Nilesh", 1.0 },
  { "Niley", 0.0 },
  { "Nili", 0.0 },
  { "Niliyah", 0.0 },
  { "Nilka", 0.0 },
  { "Nilla", 0.0 },
  { "Nillie", 0.0 },
  { "Nilmar", 1.0 },
  { "Nilo", 1.0 },
  { "Nilofar", 0.0 },
  { "Niloofar", 0.0 },
  { "Niloufar", 0.0 },
  { "Nils", 1.0 },
  { "Nilsa", 0.0 },
  { "Nilson", 1.0 },
  { "Nilton", 1.0 },
  { "Nilufar", 0.0 },
  { "Nilufer", 0.0 },
  { "Nilyn", 0.0 },
  { "Nilyne", 0.0 },
  { "Nilynn", 0.0 },
  { "Nilza", 0.0 },
  { "Nim", 1.0 },
  { "Nima", 0.7728785 },
  { "Nimah", 0.0 },
  { "Nimai", 1.0 },
  { "Nimalan", 1.0 },
  { "Nimat", 0.0 },
  { "Nimco", 0.0 },
  { "Nimesh", 1.0 },
  { "Nimish", 1.0 },
  { "Nimisha", 0.0 },
  { "Nimit", 1.0 },
  { "Nimmy", 0.0 },
  { "Nimo", 0.0 },
  { "Nimra", 0.0 },
  { "Nimrah", 0.0 },
  { "Nimrat", 0.0 },
  { "Nimrit", 0.0 },
  { "Nimrod", 1.0 },
  { "Nimsi", 0.0 },
  { "Nimsy", 0.0 },
  { "Nimue", 0.0 },
  { "Nin", 0.0 },
  { "Nina", 0.0019983375 },
  { "Ninabelle", 0.0 },
  { "Ninad", 1.0 },
  { "Ninah", 0.0 },
  { "Ninamarie", 0.0 },
  { "Ninasimone", 0.0 },
  { "Nincy", 0.0 },
  { "Ninel", 0.0 },
  { "Nineth", 0.0 },
  { "Ninetta", 0.0 },
  { "Ninette", 0.0 },
  { "Nineveh", 0.0 },
  { "Ninfa", 0.0 },
  { "Nini", 0.0 },
  { "Ninia", 0.0 },
  { "Niniola", 0.0 },
  { "Ninive", 0.0 },
  { "Ninja", 0.0 },
  { "Ninna", 0.0 },
  { "Ninnette", 0.0 },
  { "Ninnie", 0.0 },
  { "Nino", 0.99107903 },
  { "Ninon", 0.0 },
  { "Ninos", 1.0 },
  { "Ninoshka", 0.0 },
  { "Ninoska", 0.0 },
  { "Ninotchka", 0.0 },
  { "Ninti", 0.0 },
  { "Nio", 1.0 },
  { "Niobe", 0.0 },
  { "Niobi", 0.0 },
  { "Nioka", 0.0 },
  { "Niola", 0.0 },
  { "Nioma", 0.0 },
  { "Niome", 0.0 },
  { "Niomi", 0.0 },
  { "Niomie", 0.0 },
  { "Niosha", 0.0 },
  { "Nipun", 1.0 },
  { "Niquan", 1.0 },
  { "Niquel", 0.0 },
  { "Niquisha", 0.0 },
  { "Niquita", 0.0 },
  { "Nir", 1.0 },
  { "Nira", 0.009287925 },
  { "Nirah", 0.0 },
  { "Niraj", 1.0 },
  { "Niral", 1.0 },
  { "Nirali", 0.0 },
  { "Niralya", 0.0 },
  { "Niran", 1.0 },
  { "Niranjan", 1.0 },
  { "Niranjana", 0.0 },
  { "Nirav", 1.0 },
  { "Niree", 0.0 },
  { "Nirek", 1.0 },
  { "Nirel", 0.07594936 },
  { "Nirgun", 0.0 },
  { "Niria", 0.0 },
  { "Niriah", 0.0 },
  { "Niriyah", 0.0 },
  { "Nirja", 0.0 },
  { "Nirmal", 1.0 },
  { "Nirmala", 0.0 },
  { "Nirmeen", 0.0 },
  { "Nirva", 0.0 },
  { "Nirvaan", 1.0 },
  { "Nirvan", 1.0 },
  { "Nirvana", 0.0 },
  { "Nirvi", 0.0 },
  { "Nirvik", 1.0 },
  { "Nisa", 0.0 },
  { "Nisaa", 0.0 },
  { "Nisaiah", 1.0 },
  { "Nisan", 1.0 },
  { "Nisarg", 1.0 },
  { "Nischal", 1.0 },
  { "Nisean", 1.0 },
  { "Nish", 1.0 },
  { "Nisha", 0.0 },
  { "Nishaan", 1.0 },
  { "Nishad", 1.0 },
  { "Nishal", 1.0 },
  { "Nishan", 1.0 },
  { "Nishant", 1.0 },
  { "Nishanth", 1.0 },
  { "Nishat", 0.0 },
  { "Nishaun", 1.0 },
  { "Nishawn", 1.0 },
  { "Nishay", 0.0 },
  { "Nishelle", 0.0 },
  { "Nishi", 0.0 },
  { "Nishia", 0.0 },
  { "Nishika", 0.0 },
  { "Nishil", 1.0 },
  { "Nishita", 0.0 },
  { "Nishith", 1.0 },
  { "Nishitha", 0.0 },
  { "Nishka", 0.0 },
  { "Nishma", 0.0 },
  { "Nishon", 1.0 },
  { "Nishta", 0.0 },
  { "Nishtha", 0.0 },
  { "Nisi", 0.0 },
  { "Nisiah", 1.0 },
  { "Nisma", 0.0 },
  { "Nisreen", 0.0 },
  { "Nissa", 0.0 },
  { "Nissan", 0.9285714 },
  { "Nissen", 1.0 },
  { "Nissi", 0.0 },
  { "Nissie", 0.0 },
  { "Nissim", 1.0 },
  { "Nisson", 1.0 },
  { "Nissy", 0.0 },
  { "Nita", 0.0 },
  { "Nitai", 1.0 },
  { "Nitalia", 0.0 },
  { "Nitara", 0.0 },
  { "Nitasha", 0.0 },
  { "Nitaya", 0.0 },
  { "Nitesh", 1.0 },
  { "Nithila", 0.0 },
  { "Nithilan", 1.0 },
  { "Nithin", 1.0 },
  { "Nithish", 1.0 },
  { "Nithya", 0.0 },
  { "Niti", 0.0 },
  { "Nitika", 0.0 },
  { "Nitin", 1.0 },
  { "Nitish", 1.0 },
  { "Nitisha", 0.0 },
  { "Nitosha", 0.0 },
  { "Nitoya", 0.0 },
  { "Nitra", 0.0 },
  { "Nitsa", 0.0 },
  { "Nittaya", 0.0 },
  { "Nitya", 0.006188119 },
  { "Nitza", 0.0 },
  { "Nitzia", 0.0 },
  { "Nitzy", 0.0 },
  { "Niurca", 0.0 },
  { "Niurka", 0.0 },
  { "Niv", 1.0 },
  { "Niva", 0.0 },
  { "Nivaan", 1.0 },
  { "Nivaeh", 0.0 },
  { "Nivan", 1.0 },
  { "Nivaya", 0.0 },
  { "Nivayah", 0.0 },
  { "Nivea", 0.0 },
  { "Niveah", 0.0 },
  { "Nived", 1.0 },
  { "Nivedh", 1.0 },
  { "Nivedha", 0.0 },
  { "Nivedita", 0.0 },
  { "Niveditha", 0.0 },
  { "Niveen", 0.0 },
  { "Nivek", 1.0 },
  { "Niven", 1.0 },
  { "Nivetha", 0.0 },
  { "Niveya", 0.0 },
  { "Niveyah", 0.0 },
  { "Nivia", 0.0 },
  { "Niviah", 0.0 },
  { "Nivin", 1.0 },
  { "Nivriti", 0.0 },
  { "Nix", 0.7058824 },
  { "Nixaliz", 0.0 },
  { "Nixen", 1.0 },
  { "Nixie", 0.0 },
  { "Nixin", 1.0 },
  { "Nixmary", 0.0 },
  { "Nixola", 0.0 },
  { "Nixon", 0.9587294 },
  { "Nixson", 1.0 },
  { "Nixxon", 1.0 },
  { "Nixzaliz", 0.0 },
  { "Niya", 0.0 },
  { "Niyae", 0.0 },
  { "Niyah", 0.0 },
  { "Niyahlee", 0.0 },
  { "Niyahna", 0.0 },
  { "Niyam", 1.0 },
  { "Niyan", 1.0 },
  { "Niyana", 0.0 },
  { "Niyanah", 0.0 },
  { "Niyanna", 0.0 },
  { "Niyara", 0.0 },
  { "Niyasia", 0.0 },
  { "Niyat", 0.0 },
  { "Niyathi", 0.0 },
  { "Niyati", 0.0 },
  { "Niyaz", 1.0 },
  { "Niyel", 0.0 },
  { "Niyeli", 0.0 },
  { "Niyelle", 0.0 },
  { "Niyema", 0.0 },
  { "Niyia", 0.0 },
  { "Niyla", 0.0 },
  { "Niylah", 0.0 },
  { "Niyoka", 0.0 },
  { "Niyomi", 0.0 },
  { "Niyon", 1.0 },
  { "Niyona", 0.0 },
  { "Niyonna", 0.0 },
  { "Niza", 0.0 },
  { "Nizaiah", 1.0 },
  { "Nizam", 1.0 },
  { "Nizar", 1.0 },
  { "Nizear", 1.0 },
  { "Nizeria", 0.0 },
  { "Nizhoni", 0.0 },
  { "Nizhonii", 0.0 },
  { "Njeri", 0.0 },
  { "Njideka", 0.0 },
  { "Nkao", 0.0 },
  { "Nkara", 0.0 },
  { "Nkauj", 0.0 },
  { "Nkayla", 0.0 },
  { "Nkechi", 0.0 },
  { "Nkechinyere", 0.0 },
  { "Nkem", 0.0 },
  { "Nkemdilim", 0.0 },
  { "Nkemjika", 1.0 },
  { "Nkenge", 0.0 },
  { "Nkiru", 0.0 },
  { "Nkiruka", 0.0 },
  { "Nkiya", 0.0 },
  { "Nkiyah", 0.0 },
  { "Nkosi", 0.978979 },
  { "Nkrumah", 1.0 },
  { "Nna", 0.0 },
  { "Nnaemeka", 1.0 },
  { "Nnamdi", 1.0 },
  { "Nnanna", 1.0 },
  { "Nneka", 0.0 },
  { "Nnenna", 0.0 },
  { "Nneoma", 0.0 },
  { "No", 0.33333334 },
  { "Noa", 0.24836601 },
  { "Noaah", 0.7368421 },
  { "Noach", 1.0 },
  { "Noah", 0.9950624 },
  { "Noahalexander", 1.0 },
  { "Noahanthony", 1.0 },
  { "Noahgabriel", 1.0 },
  { "Noahh", 1.0 },
  { "Noahjames", 1.0 },
  { "Noahmichael", 1.0 },
  { "Noal", 1.0 },
  { "Noalani", 0.0 },
  { "Noam", 0.9010417 },
  { "Noami", 0.0 },
  { "Noan", 1.0 },
  { "Noar", 1.0 },
  { "Nobel", 1.0 },
  { "Nobert", 1.0 },
  { "Noberto", 1.0 },
  { "Nobia", 0.0 },
  { "Nobie", 0.0 },
  { "Noble", 0.9597932 },
  { "Noboru", 1.0 },
  { "Nobu", 0.0 },
  { "Nobue", 0.0 },
  { "Nobuichi", 1.0 },
  { "Nobuko", 0.0 },
  { "Nobuo", 1.0 },
  { "Nobuye", 0.0 },
  { "Nobuyoshi", 1.0 },
  { "Nobuyuki", 1.0 },
  { "Nocholas", 1.0 },
  { "Nochum", 1.0 },
  { "Nocole", 0.0 },
  { "Nocona", 0.64705884 },
  { "Noda", 0.0 },
  { "Nodia", 0.0 },
  { "Nodie", 0.0 },
  { "Nodin", 1.0 },
  { "Nodra", 0.0 },
  { "Noe", 0.9857813 },
  { "Noeh", 1.0 },
  { "Noehly", 0.0 },
  { "Noehmi", 0.0 },
  { "Noel", 0.76825136 },
  { "Noela", 0.0 },
  { "Noelan", 1.0 },
  { "Noelani", 0.0 },
  { "Noelany", 0.0 },
  { "Noele", 0.0 },
  { "Noelene", 0.0 },
  { "Noeli", 0.0 },
  { "Noelia", 0.0 },
  { "Noelie", 0.0 },
  { "Noelis", 0.0 },
  { "Noeliz", 0.0 },
  { "Noell", 0.10660661 },
  { "Noella", 0.0 },
  { "Noelle", 0.0020500205 },
  { "Noelli", 0.0 },
  { "Noellia", 0.0 },
  { "Noellie", 0.0 },
  { "Noelly", 0.0 },
  { "Noely", 0.0 },
  { "Noelya", 0.0 },
  { "Noema", 0.0 },
  { "Noeme", 0.0 },
  { "Noemi", 0.0035636814 },
  { "Noemie", 0.0 },
  { "Noemy", 0.0 },
  { "Noey", 1.0 },
  { "Nofer", 1.0 },
  { "Noga", 0.0 },
  { "Noha", 0.15258856 },
  { "Nohe", 1.0 },
  { "Nohea", 0.21818182 },
  { "Nohealani", 0.0 },
  { "Nohelani", 0.0 },
  { "Noheli", 0.0 },
  { "Nohelia", 0.0 },
  { "Nohelly", 0.0 },
  { "Nohely", 0.0 },
  { "Nohemi", 0.0 },
  { "Nohemy", 0.0 },
  { "Nohl", 1.0 },
  { "Nohlan", 1.0 },
  { "Noire", 0.0 },
  { "Nojus", 1.0 },
  { "Nokia", 0.0 },
  { "Nokomis", 0.0 },
  { "Nola", 0.00040308133 },
  { "Nolah", 0.0 },
  { "Nolan", 0.9967396 },
  { "Nolana", 0.0 },
  { "Noland", 1.0 },
  { "Nolani", 0.0 },
  { "Nolawi", 1.0 },
  { "Nolawit", 0.0 },
  { "Nolberto", 1.0 },
  { "Nolda", 0.0 },
  { "Nole", 0.9709302 },
  { "Nolee", 0.0 },
  { "Noleen", 0.0 },
  { "Nolen", 0.99823755 },
  { "Nolene", 0.0 },
  { "Noli", 0.0 },
  { "Nolia", 0.0 },
  { "Nolie", 0.025179856 },
  { "Nolin", 1.0 },
  { "Nolita", 0.0 },
  { "Nollan", 1.0 },
  { "Nollie", 0.23180594 },
  { "Nolon", 1.0 },
  { "Nolton", 1.0 },
  { "Nolyn", 0.85915494 },
  { "Nolynn", 0.0 },
  { "Noma", 0.0 },
  { "Noman", 1.0 },
  { "Nomar", 1.0 },
  { "Nomi", 0.0 },
  { "Nomia", 0.0 },
  { "Nomie", 0.0 },
  { "Nona", 0.0 },
  { "Nonah", 0.0 },
  { "Noname", 0.6875 },
  { "Nonda", 0.0 },
  { "Nondas", 0.0 },
  { "Nondus", 0.0 },
  { "Nong", 1.0 },
  { "Noni", 0.0 },
  { "Nonia", 0.0 },
  { "Nonie", 0.0 },
  { "Nonna", 0.0 },
  { "Nonnie", 0.0 },
  { "Nonya", 0.0 },
  { "Nooh", 1.0 },
  { "Noomi", 0.0 },
  { "Noon", 0.0 },
  { "Noor", 0.119727895 },
  { "Noora", 0.0 },
  { "Noorah", 0.0 },
  { "Noorain", 0.0 },
  { "Nooreen", 0.0 },
  { "Noorhan", 0.0 },
  { "Noori", 0.0 },
  { "Nooria", 0.0 },
  { "Nooriya", 0.0 },
  { "Noorjahan", 0.0 },
  { "Noorulain", 0.0 },
  { "Nor", 0.3529412 },
  { "Nora", 0.0054802774 },
  { "Noraa", 0.0 },
  { "Norabelle", 0.0 },
  { "Norah", 0.006925059 },
  { "Noraida", 0.0 },
  { "Norajean", 0.0 },
  { "Norakate", 0.0 },
  { "Noral", 1.0 },
  { "Noralba", 0.0 },
  { "Noralee", 0.0 },
  { "Noraleigh", 0.0 },
  { "Noralie", 0.0 },
  { "Noralva", 0.0 },
  { "Noralyn", 0.0 },
  { "Noralynn", 0.0 },
  { "Noran", 0.0 },
  { "Norann", 0.0 },
  { "Noranne", 0.0 },
  { "Norb", 1.0 },
  { "Norbert", 0.99970436 },
  { "Norberta", 0.0 },
  { "Norberto", 1.0 },
  { "Norbu", 1.0 },
  { "Nord", 1.0 },
  { "Norda", 0.0 },
  { "Nordahl", 1.0 },
  { "Nore", 0.8611111 },
  { "Norea", 0.0 },
  { "Norean", 0.0 },
  { "Noreda", 0.0 },
  { "Noree", 0.0 },
  { "Noreen", 0.0 },
  { "Noreene", 0.0 },
  { "Noreeta", 0.0 },
  { "Noreita", 0.0 },
  { "Noreli", 0.0 },
  { "Norelia", 0.0 },
  { "Norelis", 0.0 },
  { "Norell", 0.18987341 },
  { "Norelle", 0.0 },
  { "Norely", 0.0 },
  { "Norelys", 0.0 },
  { "Noren", 1.0 },
  { "Norena", 0.0 },
  { "Norene", 0.0 },
  { "Noreta", 0.0 },
  { "Noretha", 0.0 },
  { "Noretta", 0.0 },
  { "Norfleet", 1.0 },
  { "Norhan", 0.0 },
  { "Nori", 0.043774318 },
  { "Noria", 0.0 },
  { "Noriah", 0.0 },
  { "Norian", 1.0 },
  { "Noriana", 0.0 },
  { "Norianna", 0.0 },
  { "Norie", 0.0 },
  { "Noriel", 0.91525424 },
  { "Norielle", 0.0 },
  { "Noriene", 0.0 },
  { "Norietta", 0.0 },
  { "Norika", 0.0 },
  { "Noriko", 0.0 },
  { "Norina", 0.0 },
  { "Norine", 0.0 },
  { "Norinne", 0.0 },
  { "Norio", 1.0 },
  { "Noris", 0.61904764 },
  { "Norissa", 0.0 },
  { "Norita", 0.0 },
  { "Noritta", 0.0 },
  { "Noriyah", 0.0 },
  { "Norlan", 1.0 },
  { "Norland", 1.0 },
  { "Norleen", 0.0 },
  { "Norlene", 0.0 },
  { "Norlin", 1.0 },
  { "Norlisha", 0.0 },
  { "Norlishia", 0.0 },
  { "Norlyn", 1.0 },
  { "Norm", 1.0 },
  { "Norma", 0.004591487 },
  { "Normaa", 0.0 },
  { "Normagene", 0.0 },
  { "Normajean", 0.0 },
  { "Normal", 0.7352941 },
  { "Normalea", 0.0 },
  { "Normalee", 0.0 },
  { "Normalinda", 0.0 },
  { "Norman", 0.9936697 },
  { "Normand", 1.0 },
  { "Normandie", 0.0 },
  { "Normando", 1.0 },
  { "Normandy", 0.0 },
  { "Normen", 1.0 },
  { "Normia", 0.0 },
  { "Normie", 0.0 },
  { "Normon", 1.0 },
  { "Norna", 0.0 },
  { "Norra", 0.0 },
  { "Norrah", 0.0 },
  { "Norreen", 0.0 },
  { "Norrell", 1.0 },
  { "Norrene", 0.0 },
  { "Norreta", 0.0 },
  { "Norretta", 0.0 },
  { "Norri", 0.0 },
  { "Norrie", 0.0 },
  { "Norrin", 1.0 },
  { "Norrine", 0.0 },
  { "Norris", 0.96883565 },
  { "Norrisa", 0.0 },
  { "Norrita", 0.0 },
  { "North", 0.97637796 },
  { "Northa", 0.0 },
  { "Northern", 1.0 },
  { "Norton", 1.0 },
  { "Norva", 0.0 },
  { "Norval", 1.0 },
  { "Norvan", 1.0 },
  { "Norvel", 0.99408984 },
  { "Norvell", 0.9082353 },
  { "Norvella", 0.0 },
  { "Norvelle", 0.4 },
  { "Norvie", 0.0 },
  { "Norvil", 1.0 },
  { "Norville", 1.0 },
  { "Norvin", 1.0 },
  { "Norvis", 0.0 },
  { "Norward", 1.0 },
  { "Norwin", 1.0 },
  { "Norwood", 1.0 },
  { "Nory", 0.0 },
  { "Nosakhare", 1.0 },
  { "Nosheen", 0.0 },
  { "Noshin", 0.0 },
  { "Noslen", 1.0 },
  { "Nosson", 1.0 },
  { "Not", 1.0 },
  { "Nota", 0.0 },
  { "Notasha", 0.0 },
  { "Notie", 0.0 },
  { "Notley", 1.0 },
  { "Notnamed", 0.52893776 },
  { "Notorious", 1.0 },
  { "Nou", 0.13725491 },
  { "Nouchee", 0.0 },
  { "Nouci", 0.0 },
  { "Nouf", 0.0 },
  { "Nouh", 1.0 },
  { "Nouha", 0.0 },
  { "Nouman", 1.0 },
  { "Nour", 0.17429651 },
  { "Noura", 0.0 },
  { "Nourah", 0.0 },
  { "Nouran", 0.0 },
  { "Noureddine", 1.0 },
  { "Noureen", 0.0 },
  { "Nourhan", 0.0 },
  { "Nouri", 0.75 },
  { "Nouvelle", 0.0 },
  { "Nova", 0.0769399 },
  { "Novah", 0.20035778 },
  { "Novak", 1.0 },
  { "Noval", 1.0 },
  { "Novalea", 0.0 },
  { "Novalee", 0.0 },
  { "Novalei", 0.0 },
  { "Novaleigh", 0.0 },
  { "Novalene", 0.0 },
  { "Novaley", 0.0 },
  { "Novali", 0.0 },
  { "Novalie", 0.0 },
  { "Novalise", 0.0 },
  { "Novaly", 0.0 },
  { "Novalyn", 0.0 },
  { "Novalynn", 0.0 },
  { "Novan", 1.0 },
  { "Novarae", 0.0 },
  { "Novaya", 0.0 },
  { "Novel", 0.55263156 },
  { "Novela", 0.0 },
  { "Novelia", 0.0 },
  { "Noveline", 0.0 },
  { "Novell", 0.10869565 },
  { "Novella", 0.0 },
  { "Novelle", 0.0 },
  { "November", 0.0 },
  { "Novena", 0.0 },
  { "Noveta", 0.0 },
  { "Novi", 0.11363637 },
  { "Novia", 0.0 },
  { "Novian", 1.0 },
  { "Novice", 0.0 },
  { "Novie", 0.019543974 },
  { "Novis", 0.097402595 },
  { "Nowa", 1.0 },
  { "Nowah", 1.0 },
  { "Nowell", 1.0 },
  { "Nowshin", 0.0 },
  { "Nox", 1.0 },
  { "Noxx", 1.0 },
  { "Noy", 0.0 },
  { "Noya", 0.0 },
  { "Noyan", 1.0 },
  { "Noyes", 1.0 },
  { "Nozanin", 0.0 },
  { "Nozomi", 0.0 },
  { "Nseya", 0.0 },
  { "Nsombi", 0.0 },
  { "Nthan", 1.0 },
  { "Nthony", 1.0 },
  { "Nuala", 0.0 },
  { "Nuam", 0.0 },
  { "Nubaid", 1.0 },
  { "Nube", 0.0 },
  { "Nubia", 0.0 },
  { "Nubian", 0.0 },
  { "Nuchem", 1.0 },
  { "Nuel", 1.0 },
  { "Nugent", 1.0 },
  { "Nuh", 1.0 },
  { "Nuha", 0.0 },
  { "Nuhamin", 0.0 },
  { "Numa", 0.7201835 },
  { "Numair", 1.0 },
  { "Numan", 1.0 },
  { "Nuna", 0.0 },
  { "Nuncio", 1.0 },
  { "Nuno", 1.0 },
  { "Nunziata", 0.0 },
  { "Nunzie", 1.0 },
  { "Nunzio", 1.0 },
  { "Nuoyi", 0.0 },
  { "Nupur", 0.0 },
  { "Nur", 0.25894737 },
  { "Nura", 0.0 },
  { "Nuradin", 1.0 },
  { "Nurah", 0.0 },
  { "Nuran", 0.0 },
  { "Nuray", 0.0 },
  { "Nuraz", 1.0 },
  { "Nureen", 0.0 },
  { "Nuri", 0.5205479 },
  { "Nuria", 0.0 },
  { "Nuriah", 0.0 },
  { "Nuriel", 1.0 },
  { "Nurit", 0.0 },
  { "Nuriya", 0.0 },
  { "Nuriyah", 0.0 },
  { "Nurmuhammad", 1.0 },
  { "Nurto", 0.0 },
  { "Nuru", 1.0 },
  { "Nurul", 0.0 },
  { "Nury", 0.0 },
  { "Nusaiba", 0.0 },
  { "Nusaibah", 0.0 },
  { "Nusayba", 0.0 },
  { "Nusaybah", 0.0 },
  { "Nusrat", 0.0 },
  { "Nussen", 1.0 },
  { "Nute", 1.0 },
  { "Nuvia", 0.0 },
  { "Nuzhat", 0.0 },
  { "Nwamaka", 0.0 },
  { "Ny", 0.0 },
  { "Nya", 0.0 },
  { "Nyagoa", 0.0 },
  { "Nyah", 0.001130199 },
  { "Nyahlee", 0.0 },
  { "Nyaijah", 0.0 },
  { "Nyailah", 0.0 },
  { "Nyair", 1.0 },
  { "Nyaira", 0.0 },
  { "Nyairah", 0.0 },
  { "Nyaire", 0.6603774 },
  { "Nyaisa", 0.0 },
  { "Nyaisha", 0.0 },
  { "Nyaja", 0.0 },
  { "Nyajah", 0.0 },
  { "Nyajiah", 0.0 },
  { "Nyakume", 0.0 },
  { "Nyal", 1.0 },
  { "Nyala", 0.0 },
  { "Nyalah", 0.0 },
  { "Nyalee", 0.0 },
  { "Nyalise", 0.0 },
  { "Nyamal", 0.0 },
  { "Nyan", 1.0 },
  { "Nyana", 0.0 },
  { "Nyangel", 0.0 },
  { "Nyani", 0.0 },
  { "Nyanna", 0.0 },
  { "Nyanza", 0.0 },
  { "Nyaomi", 0.0 },
  { "Nyara", 0.0 },
  { "Nyarah", 0.0 },
  { "Nyarai", 0.0 },
  { "Nyari", 0.0 },
  { "Nyarie", 0.0 },
  { "Nyasa", 0.0 },
  { "Nyasha", 0.0 },
  { "Nyashia", 0.0 },
  { "Nyasia", 0.0 },
  { "Nyasiah", 0.0 },
  { "Nyauna", 0.0 },
  { "Nyazia", 0.0 },
  { "Nyaziah", 0.0 },
  { "Nybria", 0.0 },
  { "Nycere", 1.0 },
  { "Nychelle", 0.0 },
  { "Nycholas", 1.0 },
  { "Nychole", 0.0 },
  { "Nycia", 0.0 },
  { "Nyckolas", 1.0 },
  { "Nycolas", 1.0 },
  { "Nycole", 0.0 },
  { "Nyda", 0.0 },
  { "Nydaisha", 0.0 },
  { "Nydasia", 0.0 },
  { "Nyderah", 0.0 },
  { "Nydia", 0.0 },
  { "Nydir", 1.0 },
  { "Nydirah", 0.0 },
  { "Nydra", 0.0 },
  { "Nye", 1.0 },
  { "Nyeasha", 0.0 },
  { "Nyeasia", 0.0 },
  { "Nyeem", 1.0 },
  { "Nyeema", 0.0 },
  { "Nyeemah", 0.0 },
  { "Nyeesha", 0.0 },
  { "Nyeim", 1.0 },
  { "Nyeisha", 0.0 },
  { "Nyeka", 0.0 },
  { "Nyel", 0.7647059 },
  { "Nyela", 0.0 },
  { "Nyelah", 0.0 },
  { "Nyele", 0.0 },
  { "Nyeli", 0.0 },
  { "Nyell", 0.0 },
  { "Nyella", 0.0 },
  { "Nyelle", 0.0 },
  { "Nyelli", 0.0 },
  { "Nyellie", 0.0 },
  { "Nyema", 0.0 },
  { "Nyemah", 0.0 },
  { "Nyemia", 0.0 },
  { "Nyemiah", 0.0 },
  { "Nyera", 0.0 },
  { "Nyerere", 1.0 },
  { "Nyesha", 0.0 },
  { "Nyeshia", 0.0 },
  { "Nygee", 1.0 },
  { "Nygel", 1.0 },
  { "Nygeria", 0.0 },
  { "Nygia", 0.0 },
  { "Nygil", 1.0 },
  { "Nyha", 0.0 },
  { "Nyheem", 1.0 },
  { "Nyheim", 1.0 },
  { "Nyhiem", 1.0 },
  { "Nyhla", 0.0 },
  { "Nyia", 0.007751938 },
  { "Nyiah", 0.0 },
  { "Nyiana", 0.0 },
  { "Nyiasia", 0.0 },
  { "Nyiema", 0.0 },
  { "Nyier", 1.0 },
  { "Nyiesha", 0.0 },
  { "Nyila", 0.0 },
  { "Nyilah", 0.0 },
  { "Nyima", 0.0 },
  { "Nyimah", 0.0 },
  { "Nyirah", 0.0 },
  { "Nyisha", 0.0 },
  { "Nyja", 0.0 },
  { "Nyjae", 0.0 },
  { "Nyjah", 0.39798206 },
  { "Nyjai", 0.0 },
  { "Nyjal", 1.0 },
  { "Nyjee", 1.0 },
  { "Nyjel", 1.0 },
  { "Nyjeria", 0.0 },
  { "Nyjia", 0.0 },
  { "Nyjuan", 1.0 },
  { "Nykayla", 0.0 },
  { "Nykea", 0.0 },
  { "Nykeah", 0.0 },
  { "Nykeba", 0.0 },
  { "Nykee", 1.0 },
  { "Nykeem", 1.0 },
  { "Nykeema", 0.0 },
  { "Nykeemah", 0.0 },
  { "Nykeesha", 0.0 },
  { "Nykeia", 0.0 },
  { "Nykeira", 0.0 },
  { "Nykeisha", 0.0 },
  { "Nykel", 1.0 },
  { "Nykell", 1.0 },
  { "Nykera", 0.0 },
  { "Nykeria", 0.0 },
  { "Nykerria", 0.0 },
  { "Nykesha", 0.0 },
  { "Nykeshia", 0.0 },
  { "Nykeya", 0.0 },
  { "Nykhia", 0.0 },
  { "Nykia", 0.0 },
  { "Nykiah", 0.0 },
  { "Nykiera", 0.0 },
  { "Nykira", 0.0 },
  { "Nykiria", 0.0 },
  { "Nykisha", 0.0 },
  { "Nykita", 0.0 },
  { "Nyko", 1.0 },
  { "Nykolas", 1.0 },
  { "Nykole", 0.0 },
  { "Nyla", 0.0 },
  { "Nylaa", 0.0 },
  { "Nylae", 0.0 },
  { "Nylah", 0.0 },
  { "Nylajah", 0.0 },
  { "Nylan", 0.80588233 },
  { "Nylani", 0.0 },
  { "Nylasia", 0.0 },
  { "Nylaya", 0.0 },
  { "Nylayah", 0.0 },
  { "Nyle", 0.98125756 },
  { "Nylea", 0.0 },
  { "Nyleah", 0.0 },
  { "Nylee", 0.0 },
  { "Nyleen", 0.0 },
  { "Nylei", 0.0 },
  { "Nyleigh", 0.0 },
  { "Nylen", 0.91803277 },
  { "Nylene", 0.0 },
  { "Nyles", 1.0 },
  { "Nyleve", 0.0 },
  { "Nylia", 0.0 },
  { "Nyliah", 0.0 },
  { "Nylie", 0.0 },
  { "Nylin", 0.7058824 },
  { "Nyliyah", 0.0 },
  { "Nylla", 0.0 },
  { "Nyllah", 0.0 },
  { "Nylyn", 0.0 },
  { "Nymeria", 0.0 },
  { "Nymir", 1.0 },
  { "Nyna", 0.0 },
  { "Nyobi", 0.0 },
  { "Nyoka", 0.0 },
  { "Nyome", 0.0 },
  { "Nyomee", 0.0 },
  { "Nyomi", 0.0 },
  { "Nyomie", 0.0 },
  { "Nyomii", 0.0 },
  { "Nyona", 0.0 },
  { "Nyonna", 0.0 },
  { "Nyosha", 0.0 },
  { "Nyota", 0.0 },
  { "Nyquan", 1.0 },
  { "Nyquasha", 0.0 },
  { "Nyquasia", 0.0 },
  { "Nyra", 0.0 },
  { "Nyrah", 0.0 },
  { "Nyran", 1.0 },
  { "Nyre", 0.45454547 },
  { "Nyree", 0.089699075 },
  { "Nyrek", 1.0 },
  { "Nyrell", 1.0 },
  { "Nyria", 0.0 },
  { "Nyriah", 0.0 },
  { "Nyrie", 0.0 },
  { "Nyrissa", 0.0 },
  { "Nyrobi", 0.0 },
  { "Nyron", 1.0 },
  { "Nysa", 0.0 },
  { "Nysaiah", 1.0 },
  { "Nysean", 1.0 },
  { "Nyseem", 1.0 },
  { "Nysha", 0.0 },
  { "Nyshae", 0.0 },
  { "Nyshaun", 1.0 },
  { "Nyshawn", 1.0 },
  { "Nyshay", 0.0 },
  { "Nyshea", 0.0 },
  { "Nyshia", 0.0 },
  { "Nyshon", 1.0 },
  { "Nysia", 0.0 },
  { "Nysir", 1.0 },
  { "Nysire", 1.0 },
  { "Nyson", 1.0 },
  { "Nyssa", 0.0 },
  { "Nytasha", 0.0 },
  { "Nytasia", 0.0 },
  { "Nytavious", 1.0 },
  { "Nyteria", 0.0 },
  { "Nytia", 0.0 },
  { "Nytisha", 0.0 },
  { "Nytrell", 1.0 },
  { "Nyvaeh", 0.0 },
  { "Nyvea", 0.0 },
  { "Nyveah", 0.0 },
  { "Nyvia", 0.0 },
  { "Nyx", 0.06896552 },
  { "Nyxon", 1.0 },
  { "Nyya", 0.0 },
  { "Nyzae", 1.0 },
  { "Nyzaiah", 1.0 },
  { "Nyzair", 1.0 },
  { "Nyzaire", 1.0 },
  { "Nyzere", 1.0 },
  { "Nyzeria", 0.0 },
  { "Nyzeth", 0.0 },
  { "Nyziah", 1.0 },
  { "Nyzier", 1.0 },
  { "Nyzir", 1.0 },
  { "Nzinga", 0.0 },
  { "Nzingha", 0.0 },
  { "Nzuri", 0.0 },
  { "Oak", 1.0 },
  { "Oaken", 1.0 },
  { "Oakes", 1.0 },
  { "Oakey", 1.0 },
  { "Oakie", 0.5 },
  { "Oaklan", 1.0 },
  { "Oakland", 0.9130435 },
  { "Oaklea", 0.0 },
  { "Oaklee", 0.19139966 },
  { "Oaklei", 0.0 },
  { "Oakleigh", 0.0 },
  { "Oaklen", 1.0 },
  { "Oakley", 0.63735396 },
  { "Oakli", 0.0 },
  { "Oaklie", 0.0 },
  { "Oaklin", 1.0 },
  { "Oakly", 1.0 },
  { "Oaklyn", 0.08857143 },
  { "Oaklynd", 0.0 },
  { "Oaklynn", 0.0 },
  { "Oaks", 1.0 },
  { "Oanh", 0.0 },
  { "Oasis", 0.028248588 },
  { "Oatha", 0.0 },
  { "Oather", 1.0 },
  { "Oatis", 1.0 },
  { "Oba", 1.0 },
  { "Obada", 1.0 },
  { "Obadiah", 1.0 },
  { "Obafemi", 1.0 },
  { "Obaid", 1.0 },
  { "Obaida", 1.0 },
  { "Obaloluwa", 1.0 },
  { "Obama", 1.0 },
  { "Obasi", 1.0 },
  { "Obbie", 1.0 },
  { "Obdulia", 0.0 },
  { "Obdulio", 1.0 },
  { "Obe", 1.0 },
  { "Obed", 1.0 },
  { "Obeda", 0.0 },
  { "Obediah", 1.0 },
  { "Obehi", 0.0 },
  { "Obelia", 0.0 },
  { "Ober", 1.0 },
  { "Obera", 0.0 },
  { "Oberia", 0.0 },
  { "Oberon", 1.0 },
  { "Obert", 1.0 },
  { "Oberta", 0.0 },
  { "Oberyn", 1.0 },
  { "Obet", 1.0 },
  { "Obeth", 1.0 },
  { "Obey", 1.0 },
  { "Obi", 1.0 },
  { "Obianuju", 0.0 },
  { "Obie", 0.9369914 },
  { "Obinna", 1.0 },
  { "Obiora", 1.0 },
  { "Obra", 1.0 },
  { "Obrain", 1.0 },
  { "Obrey", 1.0 },
  { "Obrian", 1.0 },
  { "Obrien", 1.0 },
  { "Obry", 1.0 },
  { "Obryan", 1.0 },
  { "Obryant", 1.0 },
  { "Obsa", 1.0 },
  { "Obsidian", 1.0 },
  { "Oby", 1.0 },
  { "Oc", 1.0 },
  { "Oca", 0.0 },
  { "Ocasio", 1.0 },
  { "Occie", 0.0 },
  { "Oce", 1.0 },
  { "Ocea", 0.0 },
  { "Oceal", 0.0 },
  { "Ocean", 0.4948143 },
  { "Oceana", 0.0 },
  { "Oceane", 0.0 },
  { "Oceania", 0.0 },
  { "Oceanna", 0.0 },
  { "Ocelia", 0.0 },
  { "Ocella", 0.0 },
  { "Ocelotl", 1.0 },
  { "Oceola", 0.0 },
  { "Ocey", 0.0 },
  { "Ocia", 0.0 },
  { "Ocie", 0.4015655 },
  { "Ociel", 1.0 },
  { "Oconnor", 1.0 },
  { "Octa", 0.0 },
  { "Octabio", 1.0 },
  { "Octava", 0.0 },
  { "Octave", 1.0 },
  { "Octavia", 0.009018362 },
  { "Octavian", 1.0 },
  { "Octaviana", 0.0 },
  { "Octaviano", 1.0 },
  { "Octavie", 0.0 },
  { "Octavien", 1.0 },
  { "Octavio", 0.9995115 },
  { "Octavion", 1.0 },
  { "Octavious", 0.99655885 },
  { "Octavis", 1.0 },
  { "Octavius", 0.99500334 },
  { "Octavus", 1.0 },
  { "Octayvia", 0.0 },
  { "Octivia", 0.0 },
  { "October", 0.007792208 },
  { "Od", 1.0 },
  { "Oda", 0.18511686 },
  { "Odai", 1.0 },
  { "Odali", 0.0 },
  { "Odalis", 0.020094562 },
  { "Odaliz", 0.0 },
  { "Odaly", 0.0 },
  { "Odalys", 0.0 },
  { "Odalyz", 0.0 },
  { "Odaniel", 1.0 },
  { "Odarius", 1.0 },
  { "Odarrius", 1.0 },
  { "Odas", 1.0 },
  { "Oday", 1.0 },
  { "Odbert", 1.0 },
  { "Oddie", 0.57166666 },
  { "Oddis", 1.0 },
  { "Ode", 1.0 },
  { "Odeal", 0.0 },
  { "Odean", 0.48214287 },
  { "Oded", 1.0 },
  { "Odeh", 1.0 },
  { "Odel", 0.9731183 },
  { "Odelia", 0.0 },
  { "Odell", 0.7790803 },
  { "Odella", 0.0 },
  { "Odelle", 0.023041476 },
  { "Oden", 1.0 },
  { "Odena", 0.0 },
  { "Odene", 0.12195122 },
  { "Oder", 1.0 },
  { "Odera", 0.4814815 },
  { "Odes", 1.0 },
  { "Odesa", 0.0 },
  { "Odessa", 0.008763943 },
  { "Odesser", 0.0 },
  { "Odessey", 0.0 },
  { "Odessia", 0.0 },
  { "Odessie", 0.0 },
  { "Odester", 0.0 },
  { "Odet", 0.0 },
  { "Odete", 0.0 },
  { "Odeth", 0.0 },
  { "Odett", 0.0 },
  { "Odetta", 0.0 },
  { "Odette", 0.0 },
  { "Odia", 0.1590909 },
  { "Odice", 1.0 },
  { "Odie", 0.61516106 },
  { "Odies", 1.0 },
  { "Odila", 0.0 },
  { "Odile", 0.0 },
  { "Odilia", 0.0 },
  { "Odilon", 1.0 },
  { "Odin", 1.0 },
  { "Odinn", 1.0 },
  { "Odis", 0.9793379 },
  { "Odom", 1.0 },
  { "Odon", 1.0 },
  { "Odos", 1.0 },
  { "Odra", 0.0 },
  { "Odus", 1.0 },
  { "Ody", 0.7058824 },
  { "Odyn", 1.0 },
  { "Odysseus", 1.0 },
  { "Odyssey", 0.010121457 },
  { "Oel", 1.0 },
  { "Oella", 0.0 },
  { "Oen", 1.0 },
  { "Oesha", 0.0 },
  { "Ofa", 0.104166664 },
  { "Ofek", 1.0 },
  { "Ofelia", 0.0006065017 },
  { "Offie", 0.67713004 },
  { "Ofilia", 0.0 },
  { "Ofir", 0.9019608 },
  { "Ofri", 0.0 },
  { "Ogden", 1.0 },
  { "Ogechi", 0.0 },
  { "Ogechukwu", 0.10526316 },
  { "Oghenemine", 0.0 },
  { "Ogheneruno", 1.0 },
  { "Ogheneruona", 0.0 },
  { "Oghenetega", 1.0 },
  { "Oghosa", 0.0 },
  { "Ogie", 0.71428573 },
  { "Ogle", 0.9186992 },
  { "Ogochukwu", 0.0 },
  { "Ogreta", 0.0 },
  { "Oguz", 1.0 },
  { "Oh", 1.0 },
  { "Ohad", 1.0 },
  { "Ohagi", 1.0 },
  { "Ohajee", 1.0 },
  { "Ohaji", 1.0 },
  { "Ohan", 1.0 },
  { "Ohana", 0.0 },
  { "Ohani", 1.0 },
  { "Ohanna", 0.0 },
  { "Ohara", 0.23076923 },
  { "Ohemaa", 0.0 },
  { "Ohene", 1.0 },
  { "Ohitika", 1.0 },
  { "Ohlin", 1.0 },
  { "Ohm", 1.0 },
  { "Ohmer", 1.0 },
  { "Ohn", 1.0 },
  { "Oisin", 1.0 },
  { "Oiva", 1.0 },
  { "Ojani", 1.0 },
  { "Ojany", 1.0 },
  { "Ojas", 1.0 },
  { "Ojasvi", 0.0 },
  { "Ojay", 1.0 },
  { "Ojetta", 0.0 },
  { "Oji", 1.0 },
  { "Ok", 0.0 },
  { "Oka", 0.0 },
  { "Okairy", 0.0 },
  { "Okal", 0.0 },
  { "Okalani", 0.0 },
  { "Okan", 1.0 },
  { "Okechukwu", 1.0 },
  { "Okeema", 0.0 },
  { "Okeima", 0.0 },
  { "Okeith", 1.0 },
  { "Okema", 0.0 },
  { "Okemia", 0.0 },
  { "Okey", 1.0 },
  { "Okie", 0.40655738 },
  { "Okima", 0.0 },
  { "Okla", 0.062992126 },
  { "Oklahoma", 0.0 },
  { "Okley", 1.0 },
  { "Okoye", 1.0 },
  { "Okpara", 1.0 },
  { "Oksana", 0.0 },
  { "Oksanna", 0.0 },
  { "Oktober", 0.0 },
  { "Ola", 0.020698577 },
  { "Olabisi", 0.0 },
  { "Olabode", 1.0 },
  { "Olachi", 0.0 },
  { "Oladapo", 1.0 },
  { "Oladele", 1.0 },
  { "Oladimeji", 1.0 },
  { "Oladipo", 1.0 },
  { "Oladipupo", 1.0 },
  { "Oladis", 0.0 },
  { "Olaf", 1.0 },
  { "Olah", 0.0 },
  { "Olaiya", 0.0 },
  { "Olajawon", 1.0 },
  { "Olajide", 1.0 },
  { "Olajumoke", 0.0 },
  { "Olajuwan", 1.0 },
  { "Olajuwon", 1.0 },
  { "Olakunle", 1.0 },
  { "Olalekan", 1.0 },
  { "Olamae", 0.0 },
  { "Olamide", 0.44519016 },
  { "Olamilekan", 1.0 },
  { "Olamiposi", 0.53846157 },
  { "Olan", 1.0 },
  { "Olana", 0.0 },
  { "Oland", 1.0 },
  { "Olanda", 0.58115184 },
  { "Olander", 1.0 },
  { "Olando", 1.0 },
  { "Olani", 0.0 },
  { "Olaniyan", 1.0 },
  { "Olanna", 0.0 },
  { "Olanrewaju", 1.0 },
  { "Olaoluwa", 1.0 },
  { "Olar", 0.0 },
  { "Olas", 1.0 },
  { "Olasunkanmi", 1.0 },
  { "Olatokunbo", 0.0 },
  { "Olatunde", 1.0 },
  { "Olatunji", 1.0 },
  { "Olav", 1.0 },
  { "Olavi", 1.0 },
  { "Olawale", 1.0 },
  { "Olaya", 0.0 },
  { "Olayinka", 0.41904762 },
  { "Olbin", 1.0 },
  { "Oldair", 1.0 },
  { "Olden", 1.0 },
  { "Oldrich", 1.0 },
  { "Ole", 1.0 },
  { "Olea", 0.0 },
  { "Olean", 0.019473081 },
  { "Oleane", 0.0 },
  { "Oleatha", 0.0 },
  { "Oleavia", 0.0 },
  { "Oleda", 0.0 },
  { "Olee", 0.38333333 },
  { "Oleen", 0.0 },
  { "Oleeta", 0.0 },
  { "Oleg", 1.0 },
  { "Olegario", 1.0 },
  { "Oleh", 1.0 },
  { "Oleita", 0.0 },
  { "Olek", 1.0 },
  { "Oleksandr", 1.0 },
  { "Oleksiy", 1.0 },
  { "Olema", 0.0 },
  { "Olen", 0.9982262 },
  { "Olena", 0.0 },
  { "Olene", 0.0 },
  { "Oler", 0.0 },
  { "Olesha", 0.0 },
  { "Olesia", 0.0 },
  { "Olester", 1.0 },
  { "Olesya", 0.0 },
  { "Oleta", 0.0 },
  { "Oletha", 0.0 },
  { "Olethea", 0.0 },
  { "Olethia", 0.0 },
  { "Oletta", 0.0 },
  { "Oleva", 0.0 },
  { "Olevia", 0.0 },
  { "Olexa", 0.0 },
  { "Olexus", 0.0 },
  { "Oley", 0.98663104 },
  { "Olga", 0.0032161654 },
  { "Olgia", 0.0 },
  { "Olgie", 0.0 },
  { "Olha", 0.0 },
  { "Oli", 1.0 },
  { "Olia", 0.0 },
  { "Oliana", 0.0 },
  { "Olianna", 0.0 },
  { "Oliber", 1.0 },
  { "Olibia", 0.0 },
  { "Olice", 0.88135594 },
  { "Olicia", 0.0 },
  { "Olida", 0.0 },
  { "Olie", 0.5766129 },
  { "Olif", 1.0 },
  { "Olijah", 1.0 },
  { "Olimpia", 0.0 },
  { "Olin", 0.9994487 },
  { "Olina", 0.0 },
  { "Olinda", 0.0 },
  { "Olindo", 1.0 },
  { "Oline", 0.0 },
  { "Olinka", 0.0 },
  { "Olis", 0.76510066 },
  { "Olisa", 0.0 },
  { "Olisaemeka", 1.0 },
  { "Olisha", 0.0 },
  { "Olita", 0.0 },
  { "Oliva", 0.017582418 },
  { "Olivama", 0.0 },
  { "Olivander", 1.0 },
  { "Olivar", 1.0 },
  { "Olive", 0.006711913 },
  { "Olivea", 0.0 },
  { "Oliveah", 0.0 },
  { "Olivene", 0.0 },
  { "Oliver", 0.9959788 },
  { "Olivera", 0.0 },
  { "Oliverio", 1.0 },
  { "Oliverjames", 1.0 },
  { "Olivet", 0.08928572 },
  { "Olivett", 0.0 },
  { "Olivette", 0.0 },
  { "Olivia", 0.0016458473 },
  { "Oliviaann", 0.0 },
  { "Oliviagrace", 0.0 },
  { "Oliviah", 0.0 },
  { "Oliviamae", 0.0 },
  { "Oliviamarie", 0.0 },
  { "Oliviana", 0.0 },
  { "Olivianna", 0.0 },
  { "Oliviarose", 0.0 },
  { "Olivier", 0.99664205 },
  { "Olivija", 0.0 },
  { "Olivine", 0.0 },
  { "Olivio", 1.0 },
  { "Oliviya", 0.0 },
  { "Oliviyah", 0.0 },
  { "Olivya", 0.0 },
  { "Olivyah", 0.0 },
  { "Oliwia", 0.0 },
  { "Oliyah", 0.0 },
  { "Olla", 0.0 },
  { "Olle", 0.5833333 },
  { "Ollen", 1.0 },
  { "Ollene", 0.0 },
  { "Olli", 1.0 },
  { "Ollia", 0.0 },
  { "Ollice", 1.0 },
  { "Ollie", 0.28147104 },
  { "Olliemae", 0.0 },
  { "Ollin", 1.0 },
  { "Olline", 0.0 },
  { "Ollis", 1.0 },
  { "Ollivander", 1.0 },
  { "Ollive", 0.0 },
  { "Olliver", 1.0 },
  { "Ollivia", 0.0 },
  { "Olliyah", 0.0 },
  { "Olly", 0.85714287 },
  { "Ollye", 0.0 },
  { "Olma", 0.0 },
  { "Olman", 1.0 },
  { "Olna", 0.0 },
  { "Olney", 1.0 },
  { "Olof", 1.0 },
  { "Ololade", 0.0 },
  { "Olon", 1.0 },
  { "Olsen", 1.0 },
  { "Olson", 1.0 },
  { "Olton", 1.0 },
  { "Olu", 1.0 },
  { "Olubukola", 0.0 },
  { "Olubunmi", 0.0 },
  { "Oluchi", 0.0 },
  { "Oluf", 1.0 },
  { "Olufemi", 1.0 },
  { "Olufunke", 0.0 },
  { "Olufunmilayo", 0.0 },
  { "Olujimi", 1.0 },
  { "Olukayode", 1.0 },
  { "Olukemi", 0.0 },
  { "Olumide", 1.0 },
  { "Olumuyiwa", 1.0 },
  { "Oluremi", 0.0 },
  { "Olurotimi", 1.0 },
  { "Olus", 1.0 },
  { "Olusegun", 1.0 },
  { "Oluseun", 1.0 },
  { "Oluseyi", 1.0 },
  { "Olushola", 1.0 },
  { "Olutosin", 0.0 },
  { "Olutoyin", 0.0 },
  { "Oluwabukola", 0.0 },
  { "Oluwabukunmi", 0.0 },
  { "Oluwabusola", 0.0 },
  { "Oluwadabira", 0.0 },
  { "Oluwadamilare", 1.0 },
  { "Oluwadamilola", 0.36827195 },
  { "Oluwadara", 0.061728396 },
  { "Oluwadarasimi", 0.34191176 },
  { "Oluwademilade", 0.6563877 },
  { "Oluwadunmininu", 0.0 },
  { "Oluwadurotimi", 1.0 },
  { "Oluwafemi", 1.0 },
  { "Oluwaferanmi", 0.7432432 },
  { "Oluwafikayo", 1.0 },
  { "Oluwafikayomi", 0.2631579 },
  { "Oluwafisayo", 0.0 },
  { "Oluwafolahanmi", 1.0 },
  { "Oluwafolajimi", 1.0 },
  { "Oluwafunmilayo", 0.0 },
  { "Oluwafunmilola", 0.0 },
  { "Oluwajoba", 1.0 },
  { "Oluwajomiloju", 0.5116279 },
  { "Oluwakayode", 1.0 },
  { "Oluwakemi", 0.0 },
  { "Oluwakorede", 1.0 },
  { "Oluwalonimi", 0.0 },
  { "Oluwamayomikun", 1.0 },
  { "Oluwamayowa", 1.0 },
  { "Oluwanifemi", 0.4375 },
  { "Oluwapelumi", 0.6944444 },
  { "Oluwasegun", 1.0 },
  { "Oluwasemilore", 0.5701754 },
  { "Oluwaseun", 0.64285713 },
  { "Oluwaseyi", 0.689781 },
  { "Oluwaseyifunmi", 0.2777778 },
  { "Oluwatamilore", 0.2 },
  { "Oluwateniola", 0.10526316 },
  { "Oluwatimilehin", 1.0 },
  { "Oluwatimileyin", 1.0 },
  { "Oluwatise", 0.625 },
  { "Oluwatishe", 0.0 },
  { "Oluwatobi", 0.7592593 },
  { "Oluwatobiloba", 0.8639053 },
  { "Oluwatofunmi", 0.18965517 },
  { "Oluwatomi", 0.5714286 },
  { "Oluwatomilola", 0.0 },
  { "Oluwatomisin", 0.36571428 },
  { "Oluwatomiwa", 1.0 },
  { "Oluwatoni", 0.270073 },
  { "Oluwatoniloba", 0.45454547 },
  { "Oluwatosin", 0.48096886 },
  { "Oluwatoyin", 0.0 },
  { "Oluwatoyosi", 0.0 },
  { "Oluwatumininu", 0.0 },
  { "Oluwayemisi", 0.0 },
  { "Oluyemi", 1.0 },
  { "Olva", 1.0 },
  { "Olvia", 0.0 },
  { "Olvie", 0.0 },
  { "Olvin", 1.0 },
  { "Olvis", 1.0 },
  { "Olwen", 0.0 },
  { "Olwyn", 0.0 },
  { "Olya", 0.0 },
  { "Olympia", 0.0 },
  { "Olyn", 1.0 },
  { "Olyne", 0.0 },
  { "Olyve", 0.0 },
  { "Olyvea", 0.0 },
  { "Olyver", 1.0 },
  { "Olyvia", 0.0 },
  { "Olyviah", 0.0 },
  { "Om", 1.0 },
  { "Oma", 0.018545082 },
  { "Omagene", 0.0 },
  { "Omah", 0.0 },
  { "Omaid", 1.0 },
  { "Omaida", 0.0 },
  { "Omair", 1.0 },
  { "Omaira", 0.0 },
  { "Omalee", 0.0 },
  { "Oman", 1.0 },
  { "Omani", 0.5070422 },
  { "Omar", 0.99514943 },
  { "Omara", 0.0 },
  { "Omarah", 0.0 },
  { "Omare", 1.0 },
  { "Omaree", 1.0 },
  { "Omareon", 1.0 },
  { "Omari", 0.9651136 },
  { "Omaria", 0.0 },
  { "Omariah", 0.0 },
  { "Omarian", 1.0 },
  { "Omariana", 0.0 },
  { "Omarianna", 0.0 },
  { "Omarie", 0.94174755 },
  { "Omarien", 1.0 },
  { "Omarii", 1.0 },
  { "Omario", 1.0 },
  { "Omarion", 0.9985747 },
  { "Omarionna", 0.0 },
  { "Omarious", 1.0 },
  { "Omaris", 0.0 },
  { "Omarius", 1.0 },
  { "Omariyon", 1.0 },
  { "Omarr", 1.0 },
  { "Omarri", 1.0 },
  { "Omarrion", 1.0 },
  { "Omaru", 1.0 },
  { "Omary", 1.0 },
  { "Omaryon", 1.0 },
  { "Omauri", 1.0 },
  { "Omaurion", 1.0 },
  { "Omaury", 1.0 },
  { "Omaya", 0.0 },
  { "Omayma", 0.0 },
  { "Omayra", 0.0 },
  { "Omead", 1.0 },
  { "Omed", 1.0 },
  { "Omeda", 0.0 },
  { "Omeed", 1.0 },
  { "Omega", 0.17792615 },
  { "Omeir", 1.0 },
  { "Omeisha", 0.0 },
  { "Omeka", 0.0 },
  { "Omekia", 0.0 },
  { "Omelia", 0.0 },
  { "Omen", 1.0 },
  { "Omer", 0.9926016 },
  { "Omera", 0.0 },
  { "Omere", 1.0 },
  { "Omeria", 0.0 },
  { "Omero", 1.0 },
  { "Omesha", 0.0 },
  { "Omeshia", 0.0 },
  { "Ometa", 0.0 },
  { "Omi", 0.7916667 },
  { "Omia", 0.0 },
  { "Omid", 1.0 },
  { "Omie", 0.008658009 },
  { "Omika", 0.0 },
  { "Omina", 0.0 },
  { "Omir", 1.0 },
  { "Omiya", 0.0 },
  { "Omiyah", 0.0 },
  { "Omkar", 1.0 },
  { "Omma", 0.0 },
  { "Ommar", 1.0 },
  { "Ommie", 0.0 },
  { "Omni", 0.07860262 },
  { "Omnia", 0.0 },
  { "Omolara", 0.0 },
  { "Omolola", 0.0 },
  { "Omon", 1.0 },
  { "Omorion", 1.0 },
  { "Omoro", 1.0 },
  { "Omotara", 0.0 },
  { "Omotayo", 0.61538464 },
  { "Omotola", 0.0 },
  { "Omran", 1.0 },
  { "Omri", 0.9746622 },
  { "Omunique", 0.0 },
  { "Omya", 0.0 },
  { "Omyra", 0.0 },
  { "Ona", 0.011840411 },
  { "Onah", 0.0 },
  { "Onaje", 1.0 },
  { "Onald", 1.0 },
  { "Onalee", 0.0 },
  { "Onaleigh", 0.0 },
  { "Onan", 1.0 },
  { "Onas", 1.0 },
  { "Onasis", 1.0 },
  { "Onassis", 1.0 },
  { "Onathan", 1.0 },
  { "Onawa", 0.0 },
  { "Onaya", 0.0 },
  { "Onda", 0.0 },
  { "Ondina", 0.0 },
  { "Ondine", 0.0 },
  { "Ondra", 0.76666665 },
  { "Ondray", 1.0 },
  { "Ondraya", 0.0 },
  { "Ondre", 1.0 },
  { "Ondrea", 0.0 },
  { "Ondrej", 1.0 },
  { "Ondreya", 0.0 },
  { "Ondria", 0.0 },
  { "Oneal", 0.958253 },
  { "Oneatha", 0.0 },
  { "Onecimo", 1.0 },
  { "Oneda", 0.0 },
  { "Onedia", 0.0 },
  { "Oneeda", 0.0 },
  { "Oneida", 0.0 },
  { "Oneika", 0.0 },
  { "Oneil", 0.9899408 },
  { "Oneill", 1.0 },
  { "Oneisha", 0.0 },
  { "Oneita", 0.0 },
  { "Oneka", 0.0 },
  { "Onekki", 0.0 },
  { "Onel", 1.0 },
  { "Onelia", 0.0 },
  { "Onell", 0.1388889 },
  { "Onella", 0.0 },
  { "Onesha", 0.0 },
  { "Oneshia", 0.0 },
  { "Onesimo", 1.0 },
  { "Onesimus", 1.0 },
  { "Onesti", 0.0 },
  { "Onesty", 0.0 },
  { "Oneta", 0.0 },
  { "Onetha", 0.0 },
  { "Onetia", 0.0 },
  { "Onetta", 0.0 },
  { "Oneva", 0.0 },
  { "Oney", 0.64117646 },
  { "Oneyda", 0.0 },
  { "Onezia", 0.0 },
  { "Ong", 0.0 },
  { "Oni", 0.0 },
  { "Onia", 0.0 },
  { "Onica", 0.0 },
  { "Onida", 0.0 },
  { "Onie", 0.13385063 },
  { "Oniel", 1.0 },
  { "Oniesha", 0.0 },
  { "Onika", 0.0 },
  { "Onis", 0.98134327 },
  { "Onisha", 0.0 },
  { "Onisty", 0.0 },
  { "Onita", 0.0 },
  { "Onix", 1.0 },
  { "Oniya", 0.0 },
  { "Oniyah", 0.0 },
  { "Onkar", 1.0 },
  { "Onna", 0.0 },
  { "Onnah", 0.0 },
  { "Onnalee", 0.0 },
  { "Onnaleigh", 0.0 },
  { "Onni", 1.0 },
  { "Onnie", 0.37954938 },
  { "Onnika", 0.0 },
  { "Onnolee", 0.0 },
  { "Onofre", 1.0 },
  { "Onofrio", 1.0 },
  { "Onolee", 0.0 },
  { "Onora", 0.0 },
  { "Onorio", 1.0 },
  { "Onslow", 1.0 },
  { "Ontaria", 0.0 },
  { "Ontario", 1.0 },
  { "Ontarius", 1.0 },
  { "Onterio", 1.0 },
  { "Onterrio", 1.0 },
  { "Ontonio", 1.0 },
  { "Onur", 1.0 },
  { "Onya", 0.0 },
  { "Onyae", 0.0 },
  { "Onyah", 0.0 },
  { "Onye", 0.0 },
  { "Onyedikachi", 1.0 },
  { "Onyekachi", 0.8513514 },
  { "Onyekachukwu", 1.0 },
  { "Onyinye", 0.0 },
  { "Onyinyechi", 0.0 },
  { "Onyinyechukwu", 0.0 },
  { "Onyka", 0.0 },
  { "Onyx", 0.6549968 },
  { "Onzell", 0.0 },
  { "Onzie", 1.0 },
  { "Oona", 0.0 },
  { "Oonagh", 0.0 },
  { "Opal", 0.010331827 },
  { "Opalee", 0.0 },
  { "Opalene", 0.0 },
  { "Opaline", 0.0 },
  { "Opel", 0.0 },
  { "Opeyemi", 0.3877551 },
  { "Opha", 0.027334852 },
  { "Ophelia", 0.0008929103 },
  { "Ophelie", 0.0 },
  { "Ophia", 0.0 },
  { "Ophie", 0.0 },
  { "Ophilia", 0.0 },
  { "Ophir", 1.0 },
  { "Opie", 0.79375 },
  { "Ople", 0.06375839 },
  { "Oplis", 1.0 },
  { "Oppie", 0.0 },
  { "Oprah", 0.0 },
  { "Optimus", 1.0 },
  { "Or", 1.0 },
  { "Ora", 0.12549989 },
  { "Orabell", 0.0 },
  { "Orabelle", 0.0 },
  { "Orace", 1.0 },
  { "Oracio", 1.0 },
  { "Oracle", 0.0 },
  { "Oradee", 0.0 },
  { "Oradell", 0.0 },
  { "Orah", 0.0 },
  { "Oral", 0.84699124 },
  { "Oralee", 0.0 },
  { "Oralene", 0.0 },
  { "Oralia", 0.0008818342 },
  { "Oram", 1.0 },
  { "Oramae", 0.0 },
  { "Oramel", 1.0 },
  { "Oran", 1.0 },
  { "Orange", 0.97077924 },
  { "Orangie", 0.0 },
  { "Orantes", 1.0 },
  { "Oras", 1.0 },
  { "Orasio", 1.0 },
  { "Orazio", 1.0 },
  { "Orba", 0.5176471 },
  { "Orban", 1.0 },
  { "Orbelin", 1.0 },
  { "Orben", 1.0 },
  { "Orbie", 0.9227273 },
  { "Orbin", 1.0 },
  { "Orbra", 1.0 },
  { "Orby", 1.0 },
  { "Orcella", 0.0 },
  { "Orchid", 0.0 },
  { "Ord", 1.0 },
  { "Orda", 0.0 },
  { "Ordan", 1.0 },
  { "Ordean", 1.0 },
  { "Ordell", 0.7773109 },
  { "Ordella", 0.0 },
  { "Orden", 1.0 },
  { "Ordie", 0.83870965 },
  { "Ordis", 1.0 },
  { "Ore", 0.5 },
  { "Orea", 0.23809524 },
  { "Oreal", 0.0 },
  { "Orean", 0.13559322 },
  { "Oreana", 0.0 },
  { "Oreane", 0.0 },
  { "Oreanna", 0.0 },
  { "Oreatha", 0.0 },
  { "Oreda", 0.0 },
  { "Oree", 0.60723513 },
  { "Oreen", 0.0 },
  { "Orel", 0.7961019 },
  { "Orelia", 0.0 },
  { "Orell", 0.7821782 },
  { "Orella", 0.0 },
  { "Oren", 0.99576116 },
  { "Orena", 0.0 },
  { "Orenda", 0.0 },
  { "Orene", 0.0 },
  { "Orentha", 1.0 },
  { "Orenthal", 1.0 },
  { "Orenthia", 1.0 },
  { "Orenthial", 1.0 },
  { "Oreofeoluwa", 0.0 },
  { "Oreoluwa", 0.37288135 },
  { "Ores", 1.0 },
  { "Oressa", 0.0 },
  { "Orest", 1.0 },
  { "Oreste", 1.0 },
  { "Orestes", 1.0 },
  { "Oresto", 1.0 },
  { "Orestus", 1.0 },
  { "Oreta", 0.0 },
  { "Oretha", 0.0 },
  { "Oretta", 0.0 },
  { "Oreva", 0.0 },
  { "Orey", 1.0 },
  { "Orfa", 0.0 },
  { "Orfalinda", 0.0 },
  { "Orfelinda", 0.0 },
  { "Orhan", 1.0 },
  { "Ori", 0.88919413 },
  { "Oria", 0.112840466 },
  { "Oriah", 0.022421524 },
  { "Orian", 0.8885299 },
  { "Oriana", 0.0 },
  { "Oriane", 0.0 },
  { "Orianna", 0.0 },
  { "Orianne", 0.0 },
  { "Orianthi", 0.0 },
  { "Orice", 1.0 },
  { "Orie", 0.7958015 },
  { "Oriel", 0.52554744 },
  { "Orielle", 0.0 },
  { "Orien", 0.9896265 },
  { "Orietta", 0.0 },
  { "Oril", 1.0 },
  { "Orilla", 0.0 },
  { "Orin", 0.998018 },
  { "Orina", 0.0 },
  { "Orinda", 0.0 },
  { "Orine", 0.0 },
  { "Oriole", 0.0 },
  { "Orion", 0.9736712 },
  { "Oriona", 0.0 },
  { "Orionna", 0.0 },
  { "Oris", 0.8770535 },
  { "Orise", 0.5 },
  { "Orisha", 0.0 },
  { "Orison", 1.0 },
  { "Orissa", 0.0 },
  { "Orit", 0.0 },
  { "Orita", 0.0 },
  { "Oriya", 0.0 },
  { "Oriyah", 0.0 },
  { "Oriyan", 1.0 },
  { "Oriyana", 0.0 },
  { "Orla", 0.27403846 },
  { "Orlagh", 0.0 },
  { "Orlaith", 0.0 },
  { "Orlan", 1.0 },
  { "Orland", 0.99757516 },
  { "Orlanda", 0.6801517 },
  { "Orlander", 1.0 },
  { "Orlandis", 1.0 },
  { "Orlando", 0.9948212 },
  { "Orlandra", 0.0 },
  { "Orlandrea", 0.0 },
  { "Orlandria", 0.0 },
  { "Orlandus", 1.0 },
  { "Orlantha", 0.0 },
  { "Orlean", 0.038938053 },
  { "Orlee", 0.10869565 },
  { "Orleen", 0.0 },
  { "Orlen", 1.0 },
  { "Orlena", 0.0 },
  { "Orlenda", 0.0 },
  { "Orlene", 0.0 },
  { "Orletta", 0.0 },
  { "Orley", 1.0 },
  { "Orli", 0.0 },
  { "Orlidia", 0.0 },
  { "Orlie", 0.96638656 },
  { "Orlin", 1.0 },
  { "Orlinda", 0.0 },
  { "Orline", 0.0 },
  { "Orlis", 1.0 },
  { "Orlo", 1.0 },
  { "Orlondo", 1.0 },
  { "Orly", 0.0 },
  { "Orlyn", 1.0 },
  { "Orma", 0.0 },
  { "Ormah", 0.0 },
  { "Ormal", 1.0 },
  { "Orman", 1.0 },
  { "Ormand", 1.0 },
  { "Ormi", 0.0 },
  { "Ormond", 1.0 },
  { "Orna", 0.0 },
  { "Ornella", 0.0 },
  { "Orneta", 0.0 },
  { "Ornie", 1.0 },
  { "Oron", 1.0 },
  { "Oronde", 1.0 },
  { "Orothy", 0.0 },
  { "Orpah", 0.0 },
  { "Orpha", 0.0 },
  { "Orpheus", 1.0 },
  { "Orphia", 0.0 },
  { "Orphie", 0.0 },
  { "Orphus", 1.0 },
  { "Orquidea", 0.0 },
  { "Orquidia", 0.0 },
  { "Orr", 0.9 },
  { "Orra", 0.11643836 },
  { "Orral", 1.0 },
  { "Orran", 1.0 },
  { "Orrel", 1.0 },
  { "Orrell", 1.0 },
  { "Orren", 1.0 },
  { "Orrey", 1.0 },
  { "Orrie", 0.6831984 },
  { "Orrin", 1.0 },
  { "Orris", 0.9501526 },
  { "Orry", 1.0 },
  { "Orryn", 1.0 },
  { "Orsen", 1.0 },
  { "Orsola", 0.0 },
  { "Orson", 1.0 },
  { "Ortega", 1.0 },
  { "Ortencia", 0.0 },
  { "Ortensia", 0.0 },
  { "Ortha", 0.0 },
  { "Ortis", 1.0 },
  { "Ortiz", 1.0 },
  { "Orton", 1.0 },
  { "Ortrude", 0.0 },
  { "Orum", 1.0 },
  { "Orus", 1.0 },
  { "Orva", 0.17128874 },
  { "Orval", 0.99848616 },
  { "Orvall", 1.0 },
  { "Orvalle", 1.0 },
  { "Orvan", 1.0 },
  { "Orvel", 1.0 },
  { "Orvell", 1.0 },
  { "Orvella", 0.0 },
  { "Orven", 1.0 },
  { "Orvetta", 0.0 },
  { "Orvid", 1.0 },
  { "Orvie", 1.0 },
  { "Orvil", 1.0 },
  { "Orvile", 1.0 },
  { "Orvill", 1.0 },
  { "Orvilla", 0.0 },
  { "Orville", 0.99558276 },
  { "Orvin", 1.0 },
  { "Orvind", 1.0 },
  { "Orvis", 1.0 },
  { "Orvo", 1.0 },
  { "Orwin", 1.0 },
  { "Ory", 1.0 },
  { "Orya", 0.0 },
  { "Oryan", 1.0 },
  { "Oryn", 1.0 },
  { "Oryon", 1.0 },
  { "Orysia", 0.0 },
  { "Osa", 0.11248285 },
  { "Osagie", 1.0 },
  { "Osahon", 1.0 },
  { "Osama", 1.0 },
  { "Osamah", 1.0 },
  { "Osamu", 1.0 },
  { "Osanna", 0.0 },
  { "Osaretin", 1.0 },
  { "Osariemen", 0.0 },
  { "Osaze", 1.0 },
  { "Osbaldo", 1.0 },
  { "Osbert", 1.0 },
  { "Osbie", 1.0 },
  { "Osbon", 1.0 },
  { "Osborn", 1.0 },
  { "Osborne", 1.0 },
  { "Osbourne", 1.0 },
  { "Osburn", 1.0 },
  { "Osby", 1.0 },
  { "Oscar", 0.994592 },
  { "Oscardaniel", 1.0 },
  { "Oscarjr", 1.0 },
  { "Osceola", 0.29186603 },
  { "Oscer", 1.0 },
  { "Osciel", 1.0 },
  { "Osco", 1.0 },
  { "Oscor", 1.0 },
  { "Ose", 1.0 },
  { "Osean", 1.0 },
  { "Oseas", 1.0 },
  { "Osee", 0.0 },
  { "Osei", 1.0 },
  { "Oseias", 1.0 },
  { "Oseph", 1.0 },
  { "Osgood", 1.0 },
  { "Osha", 0.21489972 },
  { "Oshae", 1.0 },
  { "Oshai", 0.64285713 },
  { "Oshane", 1.0 },
  { "Oshanique", 0.0 },
  { "Oshanna", 0.0 },
  { "Oshawn", 1.0 },
  { "Oshay", 0.8962766 },
  { "Oshaye", 1.0 },
  { "Oshea", 0.9445145 },
  { "Osheana", 0.0 },
  { "Osheanna", 0.0 },
  { "Oshen", 0.93333334 },
  { "Osher", 1.0 },
  { "Oshia", 0.0 },
  { "Oshiana", 0.0 },
  { "Oshie", 0.0 },
  { "Oshua", 1.0 },
  { "Oshun", 0.0 },
  { "Oshyn", 0.0 },
  { "Osia", 0.0 },
  { "Osiah", 1.0 },
  { "Osiana", 0.0 },
  { "Osias", 1.0 },
  { "Osie", 0.35585156 },
  { "Osiel", 1.0 },
  { "Osinachi", 0.7419355 },
  { "Osiria", 0.0 },
  { "Osiris", 0.6785117 },
  { "Osirus", 1.0 },
  { "Osirys", 0.0 },
  { "Osiyo", 0.0 },
  { "Oskar", 1.0 },
  { "Osker", 1.0 },
  { "Osler", 1.0 },
  { "Oslo", 1.0 },
  { "Osman", 1.0 },
  { "Osmani", 1.0 },
  { "Osmany", 1.0 },
  { "Osmar", 1.0 },
  { "Osmara", 0.0 },
  { "Osmary", 0.0 },
  { "Osmel", 1.0 },
  { "Osmer", 1.0 },
  { "Osmin", 1.0 },
  { "Osmo", 1.0 },
  { "Osmon", 1.0 },
  { "Osmond", 1.0 },
  { "Osmund", 1.0 },
  { "Osnas", 0.0 },
  { "Osric", 1.0 },
  { "Ossama", 1.0 },
  { "Ossian", 1.0 },
  { "Ossie", 0.36484426 },
  { "Ossiel", 1.0 },
  { "Ostap", 1.0 },
  { "Ostara", 0.0 },
  { "Osteen", 1.0 },
  { "Ostell", 1.0 },
  { "Osten", 1.0 },
  { "Ostin", 1.0 },
  { "Ostyn", 1.0 },
  { "Osualdo", 1.0 },
  { "Osvaldo", 0.9990992 },
  { "Osvin", 1.0 },
  { "Oswald", 1.0 },
  { "Oswaldo", 1.0 },
  { "Oswell", 1.0 },
  { "Oswin", 0.7763713 },
  { "Oswyn", 0.0 },
  { "Osyris", 1.0 },
  { "Osyrus", 1.0 },
  { "Ota", 0.0 },
  { "Otavia", 0.0 },
  { "Otavio", 1.0 },
  { "Otavious", 1.0 },
  { "Oteka", 0.0 },
  { "Otelia", 0.0 },
  { "Otella", 0.0 },
  { "Otello", 1.0 },
  { "Oteria", 0.0 },
  { "Otey", 1.0 },
  { "Otha", 0.81971216 },
  { "Othal", 0.97206706 },
  { "Othan", 1.0 },
  { "Othar", 1.0 },
  { "Othel", 0.8261851 },
  { "Othelia", 0.0 },
  { "Othell", 0.6875 },
  { "Othella", 0.0 },
  { "Othello", 0.83381087 },
  { "Other", 1.0 },
  { "Othie", 1.0 },
  { "Othman", 1.0 },
  { "Othmar", 1.0 },
  { "Othniel", 1.0 },
  { "Otho", 1.0 },
  { "Othon", 1.0 },
  { "Othoniel", 1.0 },
  { "Othor", 1.0 },
  { "Otie", 0.22222222 },
  { "Oties", 1.0 },
  { "Otila", 0.0 },
  { "Otilia", 0.0 },
  { "Otilio", 1.0 },
  { "Otilla", 0.0 },
  { "Otillia", 0.0 },
  { "Otillie", 0.0 },
  { "Otis", 0.9848275 },
  { "Otisha", 0.0 },
  { "Otley", 1.0 },
  { "Otman", 1.0 },
  { "Otmer", 1.0 },
  { "Otniel", 1.0 },
  { "Otome", 0.0 },
  { "Otoniel", 1.0 },
  { "Otsie", 0.0 },
  { "Ott", 1.0 },
  { "Otta", 0.0 },
  { "Ottavia", 0.0 },
  { "Ottavio", 1.0 },
  { "Ottaway", 1.0 },
  { "Ottelia", 0.0 },
  { "Ottice", 1.0 },
  { "Ottie", 0.32514682 },
  { "Ottilia", 0.0 },
  { "Ottilie", 0.0 },
  { "Ottillie", 0.0 },
  { "Ottis", 0.96174985 },
  { "Ottmar", 1.0 },
  { "Otto", 0.99681854 },
  { "Ottomar", 1.0 },
  { "Ottoniel", 1.0 },
  { "Otway", 1.0 },
  { "Otylia", 0.0 },
  { "Oudia", 0.0 },
  { "Ouida", 0.0 },
  { "Ouinton", 1.0 },
  { "Ouita", 0.0 },
  { "Oumar", 1.0 },
  { "Oumie", 0.0 },
  { "Oumou", 0.0 },
  { "Oumy", 0.0 },
  { "Ourania", 0.0 },
  { "Ourtney", 0.0 },
  { "Ousainou", 1.0 },
  { "Ousman", 1.0 },
  { "Ousmane", 1.0 },
  { "Oussama", 1.0 },
  { "Ova", 0.28432956 },
  { "Ovada", 0.0 },
  { "Ovadia", 1.0 },
  { "Oval", 0.83194673 },
  { "Ovalee", 0.0 },
  { "Ovalene", 0.0 },
  { "Ovaline", 0.0 },
  { "Ove", 1.0 },
  { "Oved", 1.0 },
  { "Oveda", 0.0 },
  { "Ovedia", 0.0 },
  { "Ovee", 0.0 },
  { "Oveida", 0.0 },
  { "Ovel", 0.8484849 },
  { "Ovelia", 0.0 },
  { "Oveline", 0.0 },
  { "Ovell", 0.32 },
  { "Ovella", 0.0 },
  { "Ovena", 0.0 },
  { "Overa", 0.0 },
  { "Overton", 1.0 },
  { "Oveta", 0.0 },
  { "Ovetta", 0.0 },
  { "Ovey", 1.0 },
  { "Ovi", 0.0 },
  { "Ovia", 0.0 },
  { "Ovid", 1.0 },
  { "Ovida", 0.0 },
  { "Ovide", 1.0 },
  { "Ovidio", 1.0 },
  { "Ovie", 0.5234987 },
  { "Ovila", 1.0 },
  { "Ovis", 0.5 },
  { "Oviya", 0.0 },
  { "Owain", 1.0 },
  { "Owais", 1.0 },
  { "Owan", 1.0 },
  { "Oweda", 0.0 },
  { "Owen", 0.99751484 },
  { "Owena", 0.0 },
  { "Owenn", 1.0 },
  { "Owens", 1.0 },
  { "Oweta", 0.0 },
  { "Owetta", 0.0 },
  { "Owin", 1.0 },
  { "Owyn", 0.9632867 },
  { "Owynn", 0.82222223 },
  { "Oxana", 0.0 },
  { "Oxford", 1.0 },
  { "Oxley", 1.0 },
  { "Oyd", 1.0 },
  { "Oyindamola", 0.060240965 },
  { "Oyinkansola", 0.0 },
  { "Oyinlola", 0.0 },
  { "Oyuki", 0.0 },
  { "Oyuky", 0.0 },
  { "Oz", 1.0 },
  { "Oza", 0.14444445 },
  { "Ozair", 1.0 },
  { "Ozan", 1.0 },
  { "Ozara", 0.0 },
  { "Ozaria", 0.0 },
  { "Ozeal", 0.0 },
  { "Ozel", 0.6875 },
  { "Ozelia", 0.0 },
  { "Ozell", 0.60495573 },
  { "Ozella", 0.0 },
  { "Ozellar", 0.0 },
  { "Ozelle", 0.0 },
  { "Ozelma", 0.0 },
  { "Ozena", 0.0 },
  { "Ozetta", 0.0 },
  { "Ozia", 1.0 },
  { "Oziah", 1.0 },
  { "Ozias", 1.0 },
  { "Ozie", 0.43035054 },
  { "Oziel", 1.0 },
  { "Ozil", 1.0 },
  { "Ozioma", 0.0 },
  { "Ozion", 1.0 },
  { "Ozite", 0.0 },
  { "Ozkar", 1.0 },
  { "Ozlem", 0.0 },
  { "Ozlo", 1.0 },
  { "Ozmar", 1.0 },
  { "Ozric", 1.0 },
  { "Ozro", 1.0 },
  { "Ozvaldo", 1.0 },
  { "Ozzie", 0.77040297 },
  { "Ozziel", 1.0 },
  { "Ozzy", 1.0 },
  { "Pa", 0.003723008 },
  { "Paal", 1.0 },
  { "Paarth", 1.0 },
  { "Paayal", 0.0 },
  { "Pabel", 1.0 },
  { "Pabla", 0.0 },
  { "Pablita", 0.0 },
  { "Pablito", 1.0 },
  { "Pablo", 0.9984102 },
  { "Pace", 0.9632721 },
  { "Pacen", 1.0 },
  { "Pacer", 1.0 },
  { "Pacey", 0.69920844 },
  { "Pacha", 0.0 },
  { "Pachia", 0.0 },
  { "Pacie", 0.0 },
  { "Pacience", 0.0 },
  { "Pacifica", 0.0 },
  { "Pacifico", 1.0 },
  { "Pacita", 0.0 },
  { "Packard", 1.0 },
  { "Packer", 1.0 },
  { "Paco", 1.0 },
  { "Pada", 0.0 },
  { "Paddy", 0.2462006 },
  { "Padee", 0.0 },
  { "Paden", 0.8921788 },
  { "Pader", 0.0 },
  { "Padgett", 0.0 },
  { "Padma", 0.0 },
  { "Padme", 0.0 },
  { "Padmini", 0.0 },
  { "Padon", 1.0 },
  { "Padraic", 1.0 },
  { "Padraig", 1.0 },
  { "Padriac", 1.0 },
  { "Padric", 1.0 },
  { "Padrick", 1.0 },
  { "Padro", 1.0 },
  { "Padyn", 0.0 },
  { "Paedyn", 0.0 },
  { "Paelyn", 0.0 },
  { "Paelynn", 0.0 },
  { "Paesyn", 0.0 },
  { "Paeten", 0.0 },
  { "Paetin", 0.0 },
  { "Paeton", 0.17408907 },
  { "Paetra", 0.0 },
  { "Paetyn", 0.01369863 },
  { "Paetynn", 0.0 },
  { "Paeyton", 0.0 },
  { "Pagan", 0.0 },
  { "Page", 0.28033552 },
  { "Pageant", 0.0 },
  { "Pagen", 0.0 },
  { "Paget", 0.0 },
  { "Pagie", 0.0 },
  { "Pahal", 0.0 },
  { "Pahola", 0.0 },
  { "Pahoua", 0.0 },
  { "Pai", 0.0 },
  { "Paiden", 0.46829268 },
  { "Paidyn", 0.0 },
  { "Paig", 0.0 },
  { "Paige", 0.009660643 },
  { "Paigelyn", 0.0 },
  { "Paigelynn", 0.0 },
  { "Paighton", 0.0 },
  { "Paightyn", 0.0 },
  { "Paij", 0.0 },
  { "Paije", 0.0 },
  { "Paikea", 0.0 },
  { "Pailey", 0.0 },
  { "Paili", 0.0 },
  { "Pailyn", 0.0 },
  { "Pailynn", 0.0 },
  { "Pairlee", 0.0 },
  { "Paisely", 0.0 },
  { "Paisha", 0.0 },
  { "Paishance", 0.0 },
  { "Paishence", 0.0 },
  { "Paislea", 0.0 },
  { "Paislee", 0.0 },
  { "Paislei", 0.0 },
  { "Paisleigh", 0.0 },
  { "Paisley", 0.0010301954 },
  { "Paisleyann", 0.0 },
  { "Paisleyanne", 0.0 },
  { "Paisleyjo", 0.0 },
  { "Paisleymae", 0.0 },
  { "Paisleyrose", 0.0 },
  { "Paisli", 0.0 },
  { "Paislie", 0.0 },
  { "Paisly", 0.0 },
  { "Paislyn", 0.0 },
  { "Paislynn", 0.0 },
  { "Paison", 0.0 },
  { "Paisyn", 0.0 },
  { "Paisynn", 0.0 },
  { "Paiten", 0.035885166 },
  { "Paitin", 0.0 },
  { "Paitlyn", 0.0 },
  { "Paitlynn", 0.0 },
  { "Paiton", 0.12850468 },
  { "Paityn", 0.0054347827 },
  { "Paitynn", 0.0 },
  { "Paiyton", 0.0 },
  { "Paizlea", 0.0 },
  { "Paizlee", 0.0 },
  { "Paizleigh", 0.0 },
  { "Paizley", 0.0 },
  { "Paizli", 0.0 },
  { "Paizlie", 0.0 },
  { "Paizly", 0.0 },
  { "Paizlyn", 0.0 },
  { "Paj", 0.0 },
  { "Paje", 0.0 },
  { "Pajtim", 1.0 },
  { "Pake", 1.0 },
  { "Pakhi", 0.0 },
  { "Pakou", 0.0 },
  { "Pal", 0.9492386 },
  { "Pala", 0.0 },
  { "Paladin", 1.0 },
  { "Palak", 0.017006803 },
  { "Palani", 1.0 },
  { "Palash", 1.0 },
  { "Palestine", 0.0 },
  { "Paley", 0.0 },
  { "Palin", 0.0 },
  { "Pallas", 0.0 },
  { "Pallavi", 0.0 },
  { "Pallie", 0.0 },
  { "Palma", 0.0 },
  { "Palmena", 0.0 },
  { "Palmer", 0.84378874 },
  { "Palmera", 0.0 },
  { "Palmina", 0.0 },
  { "Palmira", 0.0 },
  { "Palmyra", 0.0 },
  { "Paloma", 0.0 },
  { "Palyn", 0.0 },
  { "Palynn", 0.0 },
  { "Pam", 0.000693816 },
  { "Pama", 0.0 },
  { "Pamala", 0.0 },
  { "Pamale", 0.0 },
  { "Pamalee", 0.0 },
  { "Pamalia", 0.0 },
  { "Pamalla", 0.0 },
  { "Pamara", 0.0 },
  { "Pameal", 0.0 },
  { "Pamel", 0.0 },
  { "Pamela", 0.002329507 },
  { "Pamelia", 0.0 },
  { "Pamella", 0.0 },
  { "Pamelyn", 0.0 },
  { "Pami", 0.0 },
  { "Pamie", 0.0 },
  { "Pamila", 0.0 },
  { "Pamilla", 0.0 },
  { "Pamilyn", 0.0 },
  { "Pamla", 0.0 },
  { "Pamlea", 0.0 },
  { "Pamler", 0.0 },
  { "Pamlia", 0.0 },
  { "Pamlyn", 0.0 },
  { "Pamm", 0.0 },
  { "Pamma", 0.0 },
  { "Pammela", 0.0 },
  { "Pammi", 0.0 },
  { "Pammie", 0.0 },
  { "Pammy", 0.0 },
  { "Pamula", 0.0 },
  { "Panagiota", 0.0 },
  { "Panagiotes", 1.0 },
  { "Panagiotis", 1.0 },
  { "Panav", 1.0 },
  { "Panayiota", 0.0 },
  { "Panayiotis", 1.0 },
  { "Panayotis", 1.0 },
  { "Panchita", 0.0 },
  { "Pancho", 1.0 },
  { "Panda", 0.0 },
  { "Pandi", 0.0 },
  { "Pandora", 0.0 },
  { "Pandoria", 0.0 },
  { "Pandra", 0.0 },
  { "Pandy", 0.0 },
  { "Panela", 0.0 },
  { "Panfilo", 1.0 },
  { "Pang", 0.017571885 },
  { "Panhia", 0.0 },
  { "Pankie", 0.0 },
  { "Pannie", 0.0 },
  { "Panos", 1.0 },
  { "Pansey", 0.0 },
  { "Pansie", 0.0 },
  { "Pansy", 0.0 },
  { "Pansye", 0.0 },
  { "Pantaleon", 1.0 },
  { "Pantalion", 1.0 },
  { "Pantelis", 1.0 },
  { "Pantera", 0.0 },
  { "Panth", 1.0 },
  { "Pantha", 0.0 },
  { "Panthea", 0.0 },
  { "Pany", 0.0 },
  { "Panya", 0.0 },
  { "Panyia", 0.0 },
  { "Panzie", 0.0 },
  { "Panzy", 0.0 },
  { "Pao", 1.0 },
  { "Paola", 0.0066478015 },
  { "Paolina", 0.0 },
  { "Paolo", 1.0 },
  { "Papa", 1.0 },
  { "Pape", 1.0 },
  { "Paquita", 0.0 },
  { "Par", 0.0 },
  { "Para", 0.0 },
  { "Paradice", 0.0 },
  { "Paradise", 0.0 },
  { "Paradyce", 0.0 },
  { "Parag", 1.0 },
  { "Paralee", 0.0 },
  { "Param", 1.0 },
  { "Paramveer", 1.0 },
  { "Paras", 1.0 },
  { "Paraskevas", 1.0 },
  { "Paraskeve", 0.0 },
  { "Paraskevi", 0.0 },
  { "Pardeep", 1.0 },
  { "Pardis", 0.0 },
  { "Paree", 0.0 },
  { "Pareesa", 0.0 },
  { "Paresh", 1.0 },
  { "Parham", 1.0 },
  { "Pari", 0.0 },
  { "Paria", 0.0 },
  { "Paricia", 0.0 },
  { "Parick", 1.0 },
  { "Paridhi", 0.0 },
  { "Parie", 0.0 },
  { "Pariis", 0.0 },
  { "Parilee", 0.0 },
  { "Parin", 1.0 },
  { "Parina", 0.0 },
  { "Parinita", 0.0 },
  { "Paris", 0.24450092 },
  { "Parisa", 0.0 },
  { "Parise", 0.0 },
  { "Parish", 0.7491135 },
  { "Parisha", 0.0 },
  { "Parishay", 0.0 },
  { "Pariss", 0.02688172 },
  { "Parissa", 0.0 },
  { "Parita", 0.0 },
  { "Pariz", 0.0 },
  { "Pariza", 0.0 },
  { "Parizoda", 0.0 },
  { "Park", 1.0 },
  { "Parke", 1.0 },
  { "Parker", 0.87848896 },
  { "Parks", 1.0 },
  { "Parkyr", 0.0 },
  { "Parlee", 0.0 },
  { "Parleen", 0.0 },
  { "Parley", 1.0 },
  { "Parlie", 0.0 },
  { "Parma", 0.0 },
  { "Parmanand", 1.0 },
  { "Parmeet", 1.0 },
  { "Parmer", 1.0 },
  { "Parmida", 0.0 },
  { "Parminder", 0.516129 },
  { "Parmis", 0.0 },
  { "Parmveer", 1.0 },
  { "Parneet", 0.0 },
  { "Parnell", 0.96125084 },
  { "Parniece", 0.0 },
  { "Parnika", 0.0 },
  { "Parren", 1.0 },
  { "Parri", 0.0 },
  { "Parrie", 0.0 },
  { "Parris", 0.5453734 },
  { "Parrish", 0.8569057 },
  { "Parry", 1.0 },
  { "Pars", 1.0 },
  { "Parsa", 1.0 },
  { "Parshva", 1.0 },
  { "Parson", 1.0 },
  { "Parth", 1.0 },
  { "Partha", 1.0 },
  { "Parthena", 0.0 },
  { "Parthenia", 0.0 },
  { "Parthina", 0.0 },
  { "Parthiv", 1.0 },
  { "Particia", 0.0 },
  { "Partick", 1.0 },
  { "Parul", 0.0 },
  { "Parv", 1.0 },
  { "Parvaneh", 0.0 },
  { "Parvathi", 0.0 },
  { "Parvathy", 0.0 },
  { "Parvati", 0.0 },
  { "Parveen", 0.0 },
  { "Parvin", 1.0 },
  { "Parvina", 0.0 },
  { "Parx", 1.0 },
  { "Parys", 0.019607844 },
  { "Paryss", 0.0 },
  { "Pascal", 0.99731666 },
  { "Pascale", 0.0 },
  { "Pascha", 0.0 },
  { "Paschal", 1.0 },
  { "Pasco", 1.0 },
  { "Pascual", 1.0 },
  { "Pascuala", 0.0 },
  { "Pasha", 0.3767209 },
  { "Pashance", 0.0 },
  { "Pashen", 0.0 },
  { "Pashence", 0.0 },
  { "Pashia", 0.0 },
  { "Pashience", 0.0 },
  { "Pashion", 0.0 },
  { "Pashun", 0.0 },
  { "Pashyn", 0.0 },
  { "Pasiley", 0.0 },
  { "Paskel", 1.0 },
  { "Pasleigh", 0.0 },
  { "Pasley", 0.0 },
  { "Pason", 1.0 },
  { "Pasqua", 0.0 },
  { "Pasqual", 1.0 },
  { "Pasquale", 1.0 },
  { "Pasqualena", 0.0 },
  { "Pasqualina", 0.0 },
  { "Pasqualine", 0.0 },
  { "Pasqualino", 1.0 },
  { "Pasquinel", 1.0 },
  { "Passion", 0.0 },
  { "Passionae", 0.0 },
  { "Passionate", 0.0 },
  { "Pastor", 1.0 },
  { "Pasty", 0.0 },
  { "Pat", 0.39984444 },
  { "Patarica", 0.0 },
  { "Pataricia", 0.0 },
  { "Patch", 1.0 },
  { "Patches", 0.0 },
  { "Pate", 1.0 },
  { "Paten", 1.0 },
  { "Pater", 1.0 },
  { "Paterica", 0.0 },
  { "Patericia", 0.0 },
  { "Pati", 0.0 },
  { "Patia", 0.0 },
  { "Patiance", 0.0 },
  { "Paticia", 0.0 },
  { "Patick", 1.0 },
  { "Patience", 0.0016066534 },
  { "Patina", 0.0 },
  { "Patirica", 0.0 },
  { "Paton", 1.0 },
  { "Patra", 0.0 },
  { "Patracia", 0.0 },
  { "Patrcia", 0.0 },
  { "Patrea", 0.0 },
  { "Patrece", 0.0 },
  { "Patrecia", 0.0 },
  { "Patreece", 0.0 },
  { "Patrell", 1.0 },
  { "Patrena", 0.0 },
  { "Patrenia", 0.0 },
  { "Patresa", 0.0 },
  { "Patrese", 0.0 },
  { "Patress", 0.0 },
  { "Patria", 0.0 },
  { "Patriana", 0.0 },
  { "Patric", 0.9762833 },
  { "Patrica", 0.0010924186 },
  { "Patriccia", 0.0 },
  { "Patrice", 0.021725368 },
  { "Patrich", 1.0 },
  { "Patricha", 0.0 },
  { "Patrichia", 0.0 },
  { "Patrici", 0.0 },
  { "Patricia", 0.0031484265 },
  { "Patriciaa", 0.0 },
  { "Patriciaann", 0.0 },
  { "Patriciaanne", 0.0 },
  { "Patriciajo", 0.0 },
  { "Patrician", 0.0 },
  { "Patriciann", 0.0 },
  { "Patricie", 0.0 },
  { "Patricik", 1.0 },
  { "Patricio", 0.9987593 },
  { "Patrick", 0.9957962 },
  { "Patricka", 0.0 },
  { "Patrickjames", 1.0 },
  { "Patrickjohn", 1.0 },
  { "Patrickjoseph", 1.0 },
  { "Patriece", 0.0 },
  { "Patrik", 1.0 },
  { "Patrika", 0.0 },
  { "Patrina", 0.0 },
  { "Patrinia", 0.0 },
  { "Patriot", 1.0 },
  { "Patrisa", 0.0 },
  { "Patrise", 0.0 },
  { "Patrisha", 0.0 },
  { "Patrisia", 0.0 },
  { "Patrisio", 1.0 },
  { "Patritia", 0.0 },
  { "Patrizia", 0.0 },
  { "Patrizio", 1.0 },
  { "Patrocinia", 0.0 },
  { "Patrocinio", 1.0 },
  { "Patron", 1.0 },
  { "Patrricia", 0.0 },
  { "Patryce", 0.0 },
  { "Patrycia", 0.0 },
  { "Patrycja", 0.0 },
  { "Patryck", 1.0 },
  { "Patryk", 1.0 },
  { "Patsey", 0.009765625 },
  { "Patsi", 0.0 },
  { "Patsie", 0.0 },
  { "Patsy", 0.060734197 },
  { "Patsyann", 0.0 },
  { "Patsye", 0.0 },
  { "Patt", 0.024390243 },
  { "Patte", 0.0 },
  { "Patterson", 0.9607843 },
  { "Patti", 0.0010282777 },
  { "Pattiann", 0.0 },
  { "Pattianne", 0.0 },
  { "Pattie", 0.0005258729 },
  { "Pattii", 0.0 },
  { "Pattijo", 0.0 },
  { "Pattina", 0.0 },
  { "Patton", 0.9409863 },
  { "Pattricia", 0.0 },
  { "Pattrick", 1.0 },
  { "Pattsy", 0.0 },
  { "Patty", 0.003941154 },
  { "Pattyann", 0.0 },
  { "Pattye", 0.0 },
  { "Paty", 0.0 },
  { "Patyn", 0.0 },
  { "Patzy", 0.0 },
  { "Pau", 1.0 },
  { "Paublita", 0.0 },
  { "Paublo", 1.0 },
  { "Paul", 0.99579716 },
  { "Paula", 0.0032056 },
  { "Paulann", 0.0 },
  { "Paulanne", 0.0 },
  { "Paulanthony", 1.0 },
  { "Paule", 0.0 },
  { "Paulean", 0.0 },
  { "Pauleen", 0.0 },
  { "Paulena", 0.0 },
  { "Paulene", 0.0 },
  { "Paulet", 0.0 },
  { "Pauleta", 0.0 },
  { "Pauleth", 0.0 },
  { "Paulett", 0.0 },
  { "Pauletta", 0.0 },
  { "Paulette", 0.0014120305 },
  { "Pauli", 0.0 },
  { "Paulia", 0.0 },
  { "Paulie", 0.8103448 },
  { "Paulin", 1.0 },
  { "Paulina", 0.00029265438 },
  { "Paulinda", 0.0 },
  { "Pauline", 0.003419369 },
  { "Paulino", 1.0 },
  { "Paulisha", 0.0 },
  { "Paulita", 0.0 },
  { "Pauljohn", 1.0 },
  { "Pauljoseph", 1.0 },
  { "Paull", 1.0 },
  { "Paulla", 0.0 },
  { "Paullette", 0.0 },
  { "Paullina", 0.0 },
  { "Paulmichael", 1.0 },
  { "Paulo", 1.0 },
  { "Paulos", 1.0 },
  { "Paulson", 1.0 },
  { "Paulus", 1.0 },
  { "Paulyn", 0.0 },
  { "Paulyna", 0.0 },
  { "Paulyne", 0.0 },
  { "Pavan", 1.0 },
  { "Pavani", 0.0 },
  { "Pavel", 1.0 },
  { "Pavelle", 0.0 },
  { "Pavi", 0.0 },
  { "Pavielle", 0.0 },
  { "Pavin", 1.0 },
  { "Pavithra", 0.0 },
  { "Pavitra", 0.0 },
  { "Pavle", 1.0 },
  { "Pavlina", 0.0 },
  { "Pavlo", 1.0 },
  { "Pavlos", 1.0 },
  { "Pavly", 1.0 },
  { "Pavneet", 0.0 },
  { "Pavni", 0.0 },
  { "Paw", 0.0 },
  { "Pawan", 1.0 },
  { "Pawel", 1.0 },
  { "Pawnee", 0.0 },
  { "Pax", 0.94911504 },
  { "Paxley", 0.0 },
  { "Paxon", 1.0 },
  { "Paxson", 1.0 },
  { "Paxston", 1.0 },
  { "Paxten", 0.84729064 },
  { "Paxtin", 1.0 },
  { "Paxton", 0.91417646 },
  { "Paxtyn", 0.46516854 },
  { "Payal", 0.0 },
  { "Payam", 1.0 },
  { "Payce", 1.0 },
  { "Paycee", 0.0 },
  { "Paycen", 0.39130434 },
  { "Paycie", 0.0 },
  { "Payden", 0.6610738 },
  { "Paydin", 1.0 },
  { "Paydon", 1.0 },
  { "Payeng", 0.0 },
  { "Payeton", 0.0 },
  { "Payge", 0.0 },
  { "Paying", 0.0 },
  { "Paylan", 0.0 },
  { "Paylee", 0.0 },
  { "Paylen", 0.0 },
  { "Paylin", 0.0 },
  { "Payman", 1.0 },
  { "Paymon", 1.0 },
  { "Payne", 0.99059826 },
  { "Paysen", 0.29411766 },
  { "Payshance", 0.0 },
  { "Payshence", 0.0 },
  { "Payslee", 0.0 },
  { "Payslei", 0.0 },
  { "Paysleigh", 0.0 },
  { "Paysley", 0.0 },
  { "Paysli", 0.0 },
  { "Payslie", 0.0 },
  { "Payson", 0.58410674 },
  { "Paytan", 0.03954802 },
  { "Payten", 0.16699688 },
  { "Paytience", 0.0 },
  { "Paytin", 0.06716418 },
  { "Paytn", 0.0 },
  { "Payton", 0.31962615 },
  { "Paytten", 0.0 },
  { "Paytton", 0.0 },
  { "Paytyn", 0.0 },
  { "Paytynn", 0.0 },
  { "Payzlee", 0.0 },
  { "Payzleigh", 0.0 },
  { "Payzley", 0.0 },
  { "Payzli", 0.0 },
  { "Payzlie", 0.0 },
  { "Paz", 0.25 },
  { "Pazley", 0.0 },
  { "Pazong", 0.0 },
  { "Peace", 0.18954249 },
  { "Peaches", 0.0 },
  { "Peachie", 0.0 },
  { "Pear", 0.0 },
  { "Pearce", 0.9879154 },
  { "Pearl", 0.024567453 },
  { "Pearla", 0.0 },
  { "Pearlann", 0.0 },
  { "Pearle", 0.0 },
  { "Pearlean", 0.0 },
  { "Pearlee", 0.0 },
  { "Pearleen", 0.0 },
  { "Pearlena", 0.0 },
  { "Pearlene", 0.0 },
  { "Pearletha", 0.0 },
  { "Pearletta", 0.0 },
  { "Pearlette", 0.0 },
  { "Pearley", 0.20916569 },
  { "Pearlia", 0.0 },
  { "Pearlie", 0.03693665 },
  { "Pearlina", 0.0 },
  { "Pearline", 0.0 },
  { "Pearly", 0.11111111 },
  { "Pearse", 1.0 },
  { "Pearson", 0.9481908 },
  { "Peary", 1.0 },
  { "Peat", 1.0 },
  { "Pebble", 0.0 },
  { "Pebbles", 0.0 },
  { "Pecola", 0.0 },
  { "Pecolia", 0.0 },
  { "Pecos", 1.0 },
  { "Peder", 1.0 },
  { "Pedra", 0.0 },
  { "Pedram", 1.0 },
  { "Pedrito", 1.0 },
  { "Pedro", 0.9943094 },
  { "Pedrohenrique", 1.0 },
  { "Pedrojose", 1.0 },
  { "Pedrojr", 1.0 },
  { "Pedroluis", 1.0 },
  { "Pedrum", 1.0 },
  { "Peer", 1.0 },
  { "Peerless", 1.0 },
  { "Peg", 0.0 },
  { "Pegah", 0.0 },
  { "Pegan", 0.0 },
  { "Pegeen", 0.0 },
  { "Pegge", 0.0 },
  { "Peggi", 0.0 },
  { "Peggie", 0.0 },
  { "Peggy", 0.002723616 },
  { "Peggyann", 0.0 },
  { "Peggyanne", 0.0 },
  { "Peggye", 0.0 },
  { "Peggyjo", 0.0 },
  { "Peggylee", 0.0 },
  { "Peggysue", 0.0 },
  { "Pegi", 0.0 },
  { "Pei", 0.0 },
  { "Peighten", 0.0 },
  { "Peighton", 0.034344334 },
  { "Peightyn", 0.0 },
  { "Peirce", 1.0 },
  { "Peirre", 1.0 },
  { "Peiton", 0.0 },
  { "Peityn", 0.0 },
  { "Peja", 0.27380952 },
  { "Pejman", 1.0 },
  { "Pejmon", 1.0 },
  { "Pela", 0.0 },
  { "Pelagia", 0.0 },
  { "Pele", 0.53488374 },
  { "Peleg", 1.0 },
  { "Pelham", 1.0 },
  { "Pelia", 0.0 },
  { "Pelin", 0.0 },
  { "Pella", 0.0 },
  { "Pellegrino", 1.0 },
  { "Pellie", 0.0 },
  { "Pelma", 0.0 },
  { "Pema", 0.0 },
  { "Pemberley", 0.0 },
  { "Pembroke", 1.0 },
  { "Pemela", 0.0 },
  { "Pencie", 0.0 },
  { "Penda", 0.0 },
  { "Pender", 1.0 },
  { "Pendleton", 0.8108108 },
  { "Pene", 0.0 },
  { "Penella", 0.0 },
  { "Penellope", 0.0 },
  { "Penelope", 0.00026678861 },
  { "Penelopea", 0.0 },
  { "Peneloperose", 0.0 },
  { "Penelopi", 0.0 },
  { "Penelopie", 0.0 },
  { "Penelopy", 0.0 },
  { "Peng", 1.0 },
  { "Peni", 0.033557046 },
  { "Peniel", 0.56213015 },
  { "Penina", 0.0 },
  { "Penisimani", 1.0 },
  { "Penley", 0.0 },
  { "Penn", 0.9854862 },
  { "Penne", 0.0 },
  { "Pennee", 0.0 },
  { "Pennelope", 0.0 },
  { "Penney", 0.0 },
  { "Penni", 0.0 },
  { "Pennie", 0.0 },
  { "Pennington", 0.6666667 },
  { "Penny", 0.0039781407 },
  { "Pennyann", 0.0 },
  { "Pennye", 0.0 },
  { "Pennylane", 0.0 },
  { "Penola", 0.0 },
  { "Penrose", 0.7777778 },
  { "Pensacola", 0.0 },
  { "Peola", 0.0 },
  { "Peony", 0.0 },
  { "Pepe", 1.0 },
  { "Pepi", 1.0 },
  { "Pepper", 0.09138155 },
  { "Peppi", 0.2777778 },
  { "Peppino", 1.0 },
  { "Pepsi", 0.0 },
  { "Per", 1.0 },
  { "Percel", 1.0 },
  { "Percell", 1.0 },
  { "Percey", 1.0 },
  { "Percie", 0.6296296 },
  { "Percilla", 0.0 },
  { "Percious", 0.0 },
  { "Percival", 1.0 },
  { "Percy", 0.981792 },
  { "Perdita", 0.0 },
  { "Peregrin", 1.0 },
  { "Peregrina", 0.0 },
  { "Peregrine", 0.7659575 },
  { "Perel", 0.0 },
  { "Perette", 0.0 },
  { "Peretz", 1.0 },
  { "Perez", 1.0 },
  { "Perfect", 0.0 },
  { "Perfecta", 0.0 },
  { "Perfecto", 1.0 },
  { "Peri", 0.004580852 },
  { "Peria", 0.0 },
  { "Perian", 0.0 },
  { "Periann", 0.0 },
  { "Perianne", 0.0 },
  { "Pericles", 1.0 },
  { "Perilla", 0.0 },
  { "Perina", 0.0 },
  { "Peris", 0.21428572 },
  { "Perkins", 1.0 },
  { "Perl", 0.18478261 },
  { "Perla", 0.0015968488 },
  { "Perle", 0.26139817 },
  { "Perlean", 0.0 },
  { "Perlena", 0.0 },
  { "Perlene", 0.0 },
  { "Perley", 0.9695817 },
  { "Perlie", 0.114420064 },
  { "Perlina", 0.0 },
  { "Perline", 0.0 },
  { "Perlita", 0.0 },
  { "Perma", 0.0 },
  { "Perman", 1.0 },
  { "Permelia", 0.0 },
  { "Perna", 0.0 },
  { "Pernell", 0.91528034 },
  { "Pernella", 0.0 },
  { "Pernetta", 0.0 },
  { "Pernia", 0.0 },
  { "Pernie", 0.0 },
  { "Peron", 1.0 },
  { "Perpetua", 0.0 },
  { "Perri", 0.05107527 },
  { "Perriann", 0.0 },
  { "Perrie", 0.15547703 },
  { "Perrier", 1.0 },
  { "Perrin", 0.76117575 },
  { "Perrine", 0.0 },
  { "Perrion", 1.0 },
  { "Perris", 0.6020761 },
  { "Perrish", 1.0 },
  { "Perry", 0.9638402 },
  { "Perryn", 0.7692308 },
  { "Persaeus", 0.0 },
  { "Persais", 0.0 },
  { "Persaius", 0.0 },
  { "Persaus", 0.0 },
  { "Persaya", 0.0 },
  { "Persayis", 0.0 },
  { "Persayus", 0.0 },
  { "Perscilla", 0.0 },
  { "Perseis", 0.0 },
  { "Persell", 1.0 },
  { "Persephanie", 0.0 },
  { "Persephany", 0.0 },
  { "Persephone", 0.0 },
  { "Persephoni", 0.0 },
  { "Persephonie", 0.0 },
  { "Persephony", 0.0 },
  { "Perseus", 0.92285717 },
  { "Persey", 1.0 },
  { "Pershing", 1.0 },
  { "Persia", 0.0 },
  { "Persis", 0.0 },
  { "Personna", 0.0 },
  { "Pertina", 0.0 },
  { "Pertrina", 0.0 },
  { "Pervie", 1.0 },
  { "Pervis", 1.0 },
  { "Peryl", 0.0 },
  { "Pesach", 1.0 },
  { "Pesha", 0.0 },
  { "Peshy", 0.0 },
  { "Pessel", 0.0 },
  { "Pessi", 0.0 },
  { "Pessie", 0.0 },
  { "Pessy", 0.0 },
  { "Pet", 0.5 },
  { "Peta", 0.0 },
  { "Petal", 0.0 },
  { "Petar", 1.0 },
  { "Pete", 0.99606645 },
  { "Peter", 0.99663 },
  { "Peteris", 1.0 },
  { "Peterjames", 1.0 },
  { "Peterjohn", 1.0 },
  { "Peterson", 1.0 },
  { "Petey", 1.0 },
  { "Petie", 1.0 },
  { "Petina", 0.0 },
  { "Petite", 0.0 },
  { "Petr", 1.0 },
  { "Petra", 0.0050722994 },
  { "Petre", 1.0 },
  { "Petrea", 0.0 },
  { "Petria", 0.0 },
  { "Petrice", 0.0 },
  { "Petrick", 1.0 },
  { "Petrina", 0.0 },
  { "Petrit", 1.0 },
  { "Petrita", 0.0 },
  { "Petro", 1.0 },
  { "Petrona", 0.0 },
  { "Petronella", 0.0 },
  { "Petronia", 0.0 },
  { "Petronila", 0.0 },
  { "Petronilla", 0.0 },
  { "Petronilo", 1.0 },
  { "Petros", 1.0 },
  { "Petrus", 1.0 },
  { "Petter", 1.0 },
  { "Pettus", 1.0 },
  { "Petty", 0.0 },
  { "Petula", 0.0 },
  { "Petunia", 0.0 },
  { "Petyon", 0.0 },
  { "Peyden", 0.23809524 },
  { "Peydon", 1.0 },
  { "Peyson", 0.4247312 },
  { "Peytan", 0.0 },
  { "Peyten", 0.22241087 },
  { "Peytin", 0.098712444 },
  { "Peyton", 0.41434404 },
  { "Peytonmarie", 0.0 },
  { "Peytonn", 0.0 },
  { "Peytyn", 0.0 },
  { "Peytynn", 0.0 },
  { "Pfeiffer", 0.0 },
  { "Phabian", 1.0 },
  { "Phadra", 0.0 },
  { "Phaedra", 0.0 },
  { "Phaidra", 0.0 },
  { "Phaith", 0.0 },
  { "Phajja", 0.0 },
  { "Phala", 0.0 },
  { "Phalen", 0.53846157 },
  { "Phalicia", 0.0 },
  { "Phalla", 0.0 },
  { "Phallen", 0.0 },
  { "Phallon", 0.0 },
  { "Phallyn", 0.0 },
  { "Phalon", 0.28846154 },
  { "Phalyn", 0.0 },
  { "Phalynn", 0.0 },
  { "Phan", 0.5714286 },
  { "Phanta", 0.0 },
  { "Phantasia", 0.0 },
  { "Phanuel", 1.0 },
  { "Phara", 0.0 },
  { "Pharah", 0.0 },
  { "Pharaoh", 1.0 },
  { "Pharell", 1.0 },
  { "Phares", 1.0 },
  { "Pharez", 1.0 },
  { "Pharis", 1.0 },
  { "Pharoah", 1.0 },
  { "Pharoh", 1.0 },
  { "Pharon", 0.54545456 },
  { "Pharrah", 0.0 },
  { "Pharrell", 1.0 },
  { "Pharren", 0.0 },
  { "Pharyn", 0.0 },
  { "Phat", 1.0 },
  { "Phatima", 0.0 },
  { "Phawn", 0.0 },
  { "Phazon", 1.0 },
  { "Pheadra", 0.0 },
  { "Pheba", 0.0 },
  { "Phebe", 0.0 },
  { "Phebee", 0.0 },
  { "Phebia", 0.0 },
  { "Phebie", 0.0 },
  { "Pheby", 0.0 },
  { "Phedra", 0.0 },
  { "Phelan", 0.88829786 },
  { "Phelecia", 0.0 },
  { "Phelicia", 0.0 },
  { "Phelicity", 0.0 },
  { "Phelisha", 0.0 },
  { "Phelix", 1.0 },
  { "Phelps", 1.0 },
  { "Pheng", 1.0 },
  { "Phenicia", 0.0 },
  { "Phenie", 0.0 },
  { "Phenious", 1.0 },
  { "Phenix", 0.62393165 },
  { "Phenyx", 0.0 },
  { "Pheobe", 0.0 },
  { "Pheobie", 0.0 },
  { "Pheona", 0.0 },
  { "Pheonix", 0.5679758 },
  { "Phi", 0.9132948 },
  { "Phia", 0.0 },
  { "Phil", 0.99635184 },
  { "Phila", 0.0 },
  { "Philadelphia", 0.0 },
  { "Philamena", 0.0 },
  { "Philana", 0.0 },
  { "Philander", 1.0 },
  { "Philant", 1.0 },
  { "Philbert", 1.0 },
  { "Philece", 0.0 },
  { "Philecia", 0.0 },
  { "Philemon", 1.0 },
  { "Philena", 0.0 },
  { "Philene", 0.0 },
  { "Philesha", 0.0 },
  { "Philias", 1.0 },
  { "Philibert", 1.0 },
  { "Philicia", 0.0 },
  { "Philicity", 0.0 },
  { "Philina", 0.0 },
  { "Philip", 0.9966489 },
  { "Philipe", 1.0 },
  { "Philipp", 1.0 },
  { "Philippa", 0.0 },
  { "Philippe", 1.0 },
  { "Philippina", 0.0 },
  { "Philippine", 0.0 },
  { "Philippos", 1.0 },
  { "Philips", 1.0 },
  { "Philis", 0.0 },
  { "Philisa", 0.0 },
  { "Philisha", 0.0 },
  { "Philistine", 0.0 },
  { "Philix", 1.0 },
  { "Phill", 1.0 },
  { "Phillicia", 0.0 },
  { "Phillina", 0.0 },
  { "Phillip", 0.99567413 },
  { "Phillipa", 0.0 },
  { "Phillipe", 1.0 },
  { "Philliph", 1.0 },
  { "Phillipmichael", 1.0 },
  { "Phillipp", 1.0 },
  { "Phillippa", 0.0 },
  { "Phillippe", 1.0 },
  { "Phillips", 1.0 },
  { "Phillis", 0.0016291067 },
  { "Phillisa", 0.0 },
  { "Phillisha", 0.0 },
  { "Phillistine", 0.0 },
  { "Phillp", 1.0 },
  { "Philly", 1.0 },
  { "Phillys", 0.0 },
  { "Philmon", 1.0 },
  { "Philmore", 1.0 },
  { "Philo", 1.0 },
  { "Philomen", 0.0 },
  { "Philomena", 0.0 },
  { "Philomene", 0.0 },
  { "Philomenia", 0.0 },
  { "Philomina", 0.0 },
  { "Philomine", 0.0 },
  { "Philopateer", 1.0 },
  { "Philopater", 1.0 },
  { "Philp", 1.0 },
  { "Philson", 1.0 },
  { "Phin", 1.0 },
  { "Phineas", 1.0 },
  { "Phinehas", 1.0 },
  { "Phineus", 1.0 },
  { "Phinizy", 1.0 },
  { "Phinley", 0.19402985 },
  { "Phinn", 1.0 },
  { "Phinnaeus", 1.0 },
  { "Phinneas", 1.0 },
  { "Phinneaus", 1.0 },
  { "Phiona", 0.0 },
  { "Phoeba", 0.0 },
  { "Phoebe", 0.00015874527 },
  { "Phoebee", 0.0 },
  { "Phoebi", 0.0 },
  { "Phoebie", 0.0 },
  { "Phoenicia", 0.0 },
  { "Phoenix", 0.61258113 },
  { "Phoenixx", 0.42105263 },
  { "Phoenyx", 0.32954547 },
  { "Phonesavanh", 0.0 },
  { "Phong", 0.9947534 },
  { "Phoua", 0.0 },
  { "Phoung", 0.0 },
  { "Phronia", 0.0 },
  { "Phronie", 0.0 },
  { "Phronsie", 0.0 },
  { "Phu", 1.0 },
  { "Phuc", 0.9749104 },
  { "Phung", 0.08938547 },
  { "Phuoc", 1.0 },
  { "Phuong", 0.21444868 },
  { "Phuonganh", 0.0 },
  { "Phuongvy", 0.0 },
  { "Phyillis", 0.0 },
  { "Phylea", 0.0 },
  { "Phylecia", 0.0 },
  { "Phylica", 0.0 },
  { "Phylicia", 0.0 },
  { "Phylis", 0.0 },
  { "Phylisa", 0.0 },
  { "Phylisha", 0.0 },
  { "Phylisia", 0.0 },
  { "Phyliss", 0.0 },
  { "Phyllicia", 0.0 },
  { "Phyllip", 1.0 },
  { "Phyllis", 0.0026948173 },
  { "Phyllisann", 0.0 },
  { "Phyllisha", 0.0 },
  { "Phyllisia", 0.0 },
  { "Phylliss", 0.0 },
  { "Phyllistine", 0.0 },
  { "Phyllys", 0.0 },
  { "Phynix", 0.26956522 },
  { "Phynn", 1.0 },
  { "Pia", 0.0 },
  { "Picabo", 0.0 },
  { "Piccola", 0.0 },
  { "Pickens", 1.0 },
  { "Picola", 0.0 },
  { "Piedad", 0.0 },
  { "Pieper", 0.0 },
  { "Pier", 0.23044397 },
  { "Piera", 0.0 },
  { "Pierce", 0.9878598 },
  { "Piercen", 1.0 },
  { "Pierceson", 1.0 },
  { "Piere", 1.0 },
  { "Pierette", 0.0 },
  { "Pierina", 0.0 },
  { "Pierino", 1.0 },
  { "Piero", 1.0 },
  { "Pierra", 0.0 },
  { "Pierre", 0.9957832 },
  { "Pierrette", 0.0 },
  { "Pierrie", 1.0 },
  { "Piers", 1.0 },
  { "Piersen", 1.0 },
  { "Pierson", 0.98177874 },
  { "Pierston", 1.0 },
  { "Pieter", 1.0 },
  { "Pietra", 0.0 },
  { "Pietrina", 0.0 },
  { "Pietro", 1.0 },
  { "Piffany", 0.0 },
  { "Pihu", 0.0 },
  { "Piilani", 0.0 },
  { "Pike", 1.0 },
  { "Pilar", 0.12947042 },
  { "Pilot", 1.0 },
  { "Pina", 0.0 },
  { "Pinar", 0.0 },
  { "Pinchas", 1.0 },
  { "Pinches", 1.0 },
  { "Pinchos", 1.0 },
  { "Pinchus", 1.0 },
  { "Pinckney", 1.0 },
  { "Pincus", 1.0 },
  { "Piney", 0.0 },
  { "Ping", 0.375 },
  { "Pink", 0.9662822 },
  { "Pinkey", 0.010256411 },
  { "Pinkie", 0.0028263105 },
  { "Pinkney", 1.0 },
  { "Pinky", 0.0 },
  { "Pinny", 1.0 },
  { "Pino", 1.0 },
  { "Pio", 1.0 },
  { "Pioet", 1.0 },
  { "Pioquinto", 1.0 },
  { "Piotr", 1.0 },
  { "Piper", 0.0046151862 },
  { "Pippa", 0.0 },
  { "Pipper", 0.0 },
  { "Pippi", 0.0 },
  { "Pippin", 0.1724138 },
  { "Pirl", 0.0 },
  { "Pirley", 0.0 },
  { "Pistol", 0.74666667 },
  { "Pita", 0.2037037 },
  { "Pittman", 1.0 },
  { "Pius", 1.0 },
  { "Pixie", 0.0 },
  { "Pixley", 0.0 },
  { "Piya", 0.0 },
  { "Piyush", 1.0 },
  { "Pj", 1.0 },
  { "Placida", 0.0 },
  { "Placide", 1.0 },
  { "Placido", 1.0 },
  { "Plas", 1.0 },
  { "Plassie", 0.0 },
  { "Platinum", 0.0 },
  { "Plato", 1.0 },
  { "Platon", 1.0 },
  { "Platt", 1.0 },
  { "Playford", 1.0 },
  { "Pleas", 1.0 },
  { "Pleasant", 0.8960114 },
  { "Pleasure", 0.0 },
  { "Pledger", 1.0 },
  { "Ples", 1.0 },
  { "Pleshette", 0.0 },
  { "Plez", 1.0 },
  { "Pleze", 1.0 },
  { "Plina", 0.0 },
  { "Plinio", 1.0 },
  { "Pliny", 1.0 },
  { "Plum", 0.0 },
  { "Pluma", 0.0 },
  { "Plumer", 0.75 },
  { "Plumie", 0.0 },
  { "Plummer", 1.0 },
  { "Plummie", 0.0 },
  { "Plutarco", 1.0 },
  { "Pnina", 0.0 },
  { "Po", 1.0 },
  { "Pocahontas", 0.0 },
  { "Poe", 0.83870965 },
  { "Poem", 0.0 },
  { "Poet", 0.26206896 },
  { "Poetri", 0.0 },
  { "Poetry", 0.0 },
  { "Pol", 1.0 },
  { "Pola", 0.0 },
  { "Polaris", 0.0 },
  { "Polet", 0.0 },
  { "Poleth", 0.0 },
  { "Polett", 0.0 },
  { "Polette", 0.0 },
  { "Policarpio", 1.0 },
  { "Polina", 0.0 },
  { "Polixeni", 0.0 },
  { "Polk", 1.0 },
  { "Pollard", 1.0 },
  { "Polley", 0.0 },
  { "Polli", 0.0 },
  { "Pollie", 0.0 },
  { "Pollux", 1.0 },
  { "Polly", 0.0008242324 },
  { "Pollyann", 0.0 },
  { "Pollyanna", 0.0 },
  { "Pollyanne", 0.0 },
  { "Pollye", 0.0 },
  { "Polo", 1.0 },
  { "Pomaikai", 0.0 },
  { "Pomp", 1.0 },
  { "Pompey", 1.0 },
  { "Pomposa", 0.0 },
  { "Pomposo", 1.0 },
  { "Ponce", 1.0 },
  { "Poncho", 1.0 },
  { "Ponciano", 1.0 },
  { "Ponda", 0.0 },
  { "Pondra", 0.0 },
  { "Pong", 1.0 },
  { "Pono", 1.0 },
  { "Pooja", 0.0 },
  { "Poojan", 1.0 },
  { "Pookela", 1.0 },
  { "Poonam", 0.0 },
  { "Poorna", 0.0 },
  { "Poorvi", 0.0 },
  { "Pooya", 1.0 },
  { "Pope", 1.0 },
  { "Poppi", 0.0 },
  { "Poppie", 0.0 },
  { "Poppy", 0.0 },
  { "Porcha", 0.0 },
  { "Porchae", 0.0 },
  { "Porche", 0.0 },
  { "Porchea", 0.0 },
  { "Porcher", 0.0 },
  { "Porchia", 0.0 },
  { "Porcia", 0.0 },
  { "Porcsha", 0.0 },
  { "Porfiria", 0.0 },
  { "Porfirio", 1.0 },
  { "Porscha", 0.0 },
  { "Porsche", 0.0 },
  { "Porschea", 0.0 },
  { "Porschia", 0.0 },
  { "Porsha", 0.001506024 },
  { "Porshae", 0.0 },
  { "Porshay", 0.0 },
  { "Porshe", 0.0 },
  { "Porshea", 0.0 },
  { "Porshia", 0.0 },
  { "Porsia", 0.0 },
  { "Port", 1.0 },
  { "Porter", 0.97243255 },
  { "Portia", 0.000763869 },
  { "Portland", 0.224 },
  { "Portlyn", 0.0 },
  { "Portlynn", 0.0 },
  { "Poseidon", 1.0 },
  { "Posey", 0.6545454 },
  { "Posie", 0.5263158 },
  { "Posy", 0.0 },
  { "Pota", 0.0 },
  { "Potter", 0.5 },
  { "Poua", 0.0 },
  { "Poul", 1.0 },
  { "Pouya", 1.0 },
  { "Powell", 0.99573255 },
  { "Power", 1.0 },
  { "Powers", 1.0 },
  { "Poyraz", 1.0 },
  { "Ppatricia", 0.0 },
  { "Prabal", 1.0 },
  { "Prabhas", 1.0 },
  { "Prabhat", 1.0 },
  { "Prabhav", 1.0 },
  { "Prabhdeep", 1.0 },
  { "Prabhjot", 0.8476821 },
  { "Prabhleen", 0.0 },
  { "Prabhnoor", 0.4509804 },
  { "Prabin", 1.0 },
  { "Prabjot", 1.0 },
  { "Prachi", 0.0 },
  { "Pracilla", 0.0 },
  { "Pradeep", 1.0 },
  { "Pradhyun", 1.0 },
  { "Pradyumna", 1.0 },
  { "Pradyun", 1.0 },
  { "Pragathi", 0.0 },
  { "Pragati", 0.0 },
  { "Pragna", 0.0 },
  { "Pragnya", 0.0 },
  { "Pragya", 0.0 },
  { "Pragyan", 1.0 },
  { "Praharsh", 1.0 },
  { "Prahlad", 1.0 },
  { "Prairie", 0.0 },
  { "Praise", 0.2640118 },
  { "Prajedes", 1.0 },
  { "Prajin", 1.0 },
  { "Prajit", 1.0 },
  { "Prajna", 0.0 },
  { "Prajwal", 1.0 },
  { "Prakash", 1.0 },
  { "Prakhar", 1.0 },
  { "Prakrit", 1.0 },
  { "Prakriti", 0.0 },
  { "Prakruti", 0.0 },
  { "Pramit", 1.0 },
  { "Pramod", 1.0 },
  { "Pranathi", 0.0 },
  { "Pranati", 0.0 },
  { "Pranav", 1.0 },
  { "Pranavi", 0.0 },
  { "Pranay", 1.0 },
  { "Pranaya", 0.0 },
  { "Praneel", 1.0 },
  { "Praneet", 1.0 },
  { "Praneeth", 1.0 },
  { "Pranesh", 1.0 },
  { "Pranika", 0.0 },
  { "Pranil", 1.0 },
  { "Pranish", 1.0 },
  { "Pranisha", 0.0 },
  { "Pranit", 1.0 },
  { "Pranita", 0.0 },
  { "Pranith", 1.0 },
  { "Pranitha", 0.0 },
  { "Pranjal", 0.44230768 },
  { "Pranshi", 0.0 },
  { "Pranshu", 1.0 },
  { "Pranvi", 0.0 },
  { "Prapti", 0.0 },
  { "Prarthana", 0.0 },
  { "Prasad", 1.0 },
  { "Prashant", 1.0 },
  { "Prashanth", 1.0 },
  { "Prasoon", 1.0 },
  { "Prateek", 1.0 },
  { "Pratham", 1.0 },
  { "Pratheek", 1.0 },
  { "Prather", 1.0 },
  { "Prathik", 1.0 },
  { "Pratik", 1.0 },
  { "Pratiksha", 0.0 },
  { "Pratima", 0.0 },
  { "Pratt", 1.0 },
  { "Pratyush", 1.0 },
  { "Pratyusha", 0.0 },
  { "Pravallika", 0.0 },
  { "Praveen", 1.0 },
  { "Praveer", 1.0 },
  { "Pravin", 1.0 },
  { "Praxedis", 1.0 },
  { "Praxton", 1.0 },
  { "Pray", 1.0 },
  { "Prayag", 1.0 },
  { "Prayan", 1.0 },
  { "Prayas", 1.0 },
  { "Prayash", 1.0 },
  { "Prayer", 0.0 },
  { "Praylee", 0.0 },
  { "Prayze", 0.0 },
  { "Preana", 0.0 },
  { "Preanna", 0.0 },
  { "Prebble", 0.0 },
  { "Precieux", 1.0 },
  { "Preciliano", 1.0 },
  { "Precilla", 0.0 },
  { "Preciosa", 0.0 },
  { "Precious", 0.007189264 },
  { "Preciousa", 0.0 },
  { "Precius", 0.0 },
  { "Precyous", 0.0 },
  { "Predrag", 1.0 },
  { "Preesha", 0.0 },
  { "Preet", 0.63228697 },
  { "Preetam", 1.0 },
  { "Preetham", 1.0 },
  { "Preethi", 0.0 },
  { "Preeti", 0.0 },
  { "Preeya", 0.0 },
  { "Preksha", 0.0 },
  { "Prem", 1.0 },
  { "Prema", 0.0 },
  { "Premal", 1.0 },
  { "Premier", 1.0 },
  { "Premiere", 1.0 },
  { "Prenell", 1.0 },
  { "Prenella", 0.0 },
  { "Prenisha", 0.0 },
  { "Prentice", 0.9853244 },
  { "Prentis", 1.0 },
  { "Prentiss", 0.9780782 },
  { "Preonda", 0.0 },
  { "Prerana", 0.0 },
  { "Prerna", 0.0 },
  { "Presciliano", 1.0 },
  { "Prescilla", 0.0 },
  { "Prescious", 0.0 },
  { "Prescott", 0.9893993 },
  { "Preshus", 0.0 },
  { "President", 1.0 },
  { "Presila", 0.0 },
  { "Presiliano", 1.0 },
  { "Presilla", 0.0 },
  { "Presious", 0.0 },
  { "Preslea", 0.0 },
  { "Preslee", 0.017326733 },
  { "Preslei", 0.0 },
  { "Presleigh", 0.0 },
  { "Presley", 0.17118934 },
  { "Presli", 0.0 },
  { "Preslie", 0.0 },
  { "Presly", 0.0 },
  { "Preslyn", 0.0 },
  { "Preslynn", 0.0 },
  { "Press", 1.0 },
  { "Pressie", 0.0 },
  { "Pressley", 0.40514904 },
  { "Presslie", 0.0 },
  { "Pressly", 0.0 },
  { "Prestan", 1.0 },
  { "Presten", 1.0 },
  { "Prestin", 1.0 },
  { "Prestina", 0.0 },
  { "Prestine", 0.0 },
  { "Prestley", 0.0 },
  { "Preston", 0.99415404 },
  { "Prestyn", 0.7912371 },
  { "Pretto", 0.0 },
  { "Pretty", 0.0 },
  { "Previn", 1.0 },
  { "Preya", 0.0 },
  { "Prezlee", 0.0 },
  { "Prezleigh", 0.0 },
  { "Prezley", 0.0 },
  { "Pria", 0.0 },
  { "Priam", 1.0 },
  { "Prianka", 0.0 },
  { "Prianna", 0.0 },
  { "Priansh", 1.0 },
  { "Price", 0.98854524 },
  { "Pricella", 0.0 },
  { "Pricie", 0.0 },
  { "Pricila", 0.0 },
  { "Pricilla", 0.0 },
  { "Pricillia", 0.0 },
  { "Pricsila", 0.0 },
  { "Pricsilla", 0.0 },
  { "Pride", 1.0 },
  { "Priella", 0.0 },
  { "Priest", 1.0 },
  { "Priestly", 1.0 },
  { "Prim", 0.0 },
  { "Prima", 0.0 },
  { "Primavera", 0.0 },
  { "Prime", 1.0 },
  { "Primitivo", 1.0 },
  { "Primo", 1.0 },
  { "Primrose", 0.0 },
  { "Primus", 1.0 },
  { "Prina", 0.0 },
  { "Prince", 0.9645824 },
  { "Princeamir", 1.0 },
  { "Princeanthony", 1.0 },
  { "Princecharles", 1.0 },
  { "Princedavid", 1.0 },
  { "Princeelijah", 1.0 },
  { "Princejames", 1.0 },
  { "Princejohn", 1.0 },
  { "Princella", 0.0 },
  { "Princemichael", 1.0 },
  { "Princes", 0.0 },
  { "Princesa", 0.0 },
  { "Princess", 0.00053062465 },
  { "Princessa", 0.0 },
  { "Princeston", 1.0 },
  { "Princeten", 1.0 },
  { "Princetin", 1.0 },
  { "Princeton", 1.0 },
  { "Princetta", 0.0 },
  { "Princetyn", 1.0 },
  { "Princewill", 1.0 },
  { "Princewilliam", 1.0 },
  { "Princie", 0.0 },
  { "Princilla", 0.0 },
  { "Princton", 1.0 },
  { "Princy", 0.0 },
  { "Pringle", 1.0 },
  { "Prinsesa", 0.0 },
  { "Prinston", 1.0 },
  { "Printes", 1.0 },
  { "Printess", 1.0 },
  { "Printice", 1.0 },
  { "Printis", 1.0 },
  { "Printiss", 1.0 },
  { "Prinz", 1.0 },
  { "Prisca", 0.0 },
  { "Priscella", 0.0 },
  { "Priscila", 0.0 },
  { "Priscilia", 0.0 },
  { "Priscill", 0.0 },
  { "Priscilla", 0.0020678944 },
  { "Priscille", 0.0 },
  { "Priscillia", 0.0 },
  { "Priscyla", 0.0 },
  { "Priscylla", 0.0 },
  { "Priseis", 0.0 },
  { "Prish", 1.0 },
  { "Prisha", 0.0 },
  { "Prisicilla", 0.0 },
  { "Prisila", 0.0 },
  { "Prisilla", 0.0 },
  { "Prisma", 0.0 },
  { "Prissila", 0.0 },
  { "Prissy", 0.0 },
  { "Pristina", 0.0 },
  { "Pristine", 0.0 },
  { "Prit", 1.0 },
  { "Pritam", 1.0 },
  { "Pritesh", 1.0 },
  { "Prithika", 0.0 },
  { "Prithiv", 1.0 },
  { "Prithvi", 1.0 },
  { "Priti", 0.0 },
  { "Pritika", 0.0 },
  { "Priya", 0.0 },
  { "Priyah", 0.0 },
  { "Priyal", 0.0 },
  { "Priyam", 1.0 },
  { "Priyan", 1.0 },
  { "Priyana", 0.0 },
  { "Priyank", 1.0 },
  { "Priyanka", 0.0 },
  { "Priyansh", 1.0 },
  { "Priyanshi", 0.0 },
  { "Priyanshu", 1.0 },
  { "Priyasha", 0.0 },
  { "Procopio", 1.0 },
  { "Proctor", 1.0 },
  { "Prodigy", 1.0 },
  { "Profit", 1.0 },
  { "Prometheus", 1.0 },
  { "Promise", 0.05520287 },
  { "Promiss", 0.0 },
  { "Promisse", 0.0 },
  { "Promize", 0.0 },
  { "Promyce", 0.0 },
  { "Promyse", 0.0 },
  { "Promyss", 0.0 },
  { "Prophet", 1.0 },
  { "Prosper", 1.0 },
  { "Prosperity", 0.0 },
  { "Prospero", 1.0 },
  { "Providence", 0.0 },
  { "Providencia", 0.0 },
  { "Providenza", 0.0 },
  { "Pruda", 0.0 },
  { "Prudance", 0.0 },
  { "Prudence", 0.0 },
  { "Prudencia", 0.0 },
  { "Prudencio", 1.0 },
  { "Prudie", 0.0 },
  { "Prudy", 0.0 },
  { "Prue", 0.3529412 },
  { "Pruitt", 1.0 },
  { "Prutha", 0.0 },
  { "Pryce", 0.9656805 },
  { "Pryia", 0.0 },
  { "Prynce", 1.0 },
  { "Pryncess", 0.0 },
  { "Prynceton", 1.0 },
  { "Prynn", 0.0 },
  { "Pryor", 0.981685 },
  { "Pryscilla", 0.0 },
  { "Prysilla", 0.0 },
  { "Psalm", 0.43046358 },
  { "Psalms", 0.27027026 },
  { "Psymon", 1.0 },
  { "Ptolemy", 1.0 },
  { "Ptosha", 0.0 },
  { "Pual", 1.0 },
  { "Pualani", 0.0 },
  { "Puanani", 0.0 },
  { "Puaolena", 0.0 },
  { "Pui", 0.0 },
  { "Puja", 0.0 },
  { "Pujan", 1.0 },
  { "Punam", 0.0 },
  { "Puneet", 0.49535602 },
  { "Punit", 1.0 },
  { "Pura", 0.0 },
  { "Purab", 1.0 },
  { "Purav", 1.0 },
  { "Purcell", 1.0 },
  { "Purdy", 1.0 },
  { "Purity", 0.0 },
  { "Purl", 1.0 },
  { "Purlie", 0.0 },
  { "Purnell", 0.9921875 },
  { "Purnima", 0.0 },
  { "Purpose", 0.0 },
  { "Purva", 0.0 },
  { "Purvi", 0.0 },
  { "Purvis", 1.0 },
  { "Pushti", 0.0 },
  { "Pyper", 0.0 },
  { "Pypper", 0.0 },
  { "Qaadir", 1.0 },
  { "Qaasim", 1.0 },
  { "Qadeera", 0.0 },
  { "Qadir", 1.0 },
  { "Qadira", 0.0 },
  { "Qadirah", 0.0 },
  { "Qadree", 1.0 },
  { "Qadriyyah", 0.0 },
  { "Qadry", 1.0 },
  { "Qaiden", 1.0 },
  { "Qais", 1.0 },
  { "Qalid", 1.0 },
  { "Qamar", 0.55803573 },
  { "Qamari", 1.0 },
  { "Qasem", 1.0 },
  { "Qasim", 1.0 },
  { "Qassam", 1.0 },
  { "Qays", 1.0 },
  { "Qi", 0.8214286 },
  { "Qian", 1.0 },
  { "Qiana", 0.0 },
  { "Qianna", 0.0 },
  { "Qiara", 0.0 },
  { "Qiarah", 0.0 },
  { "Qierra", 0.0 },
  { "Qing", 0.0 },
  { "Qiona", 0.0 },
  { "Qirat", 0.0 },
  { "Qiuana", 0.0 },
  { "Qmari", 1.0 },
  { "Qorianka", 0.0 },
  { "Qua", 1.0 },
  { "Quaadir", 1.0 },
  { "Quaashie", 1.0 },
  { "Quacy", 1.0 },
  { "Quadair", 1.0 },
  { "Quadaisha", 0.0 },
  { "Quadarious", 1.0 },
  { "Quadarius", 1.0 },
  { "Quadarrious", 1.0 },
  { "Quadarrius", 1.0 },
  { "Quadasha", 0.0 },
  { "Quadasia", 0.0 },
  { "Quade", 1.0 },
  { "Quadeer", 1.0 },
  { "Quadell", 1.0 },
  { "Quadere", 1.0 },
  { "Quadeshia", 0.0 },
  { "Quadier", 1.0 },
  { "Quadijah", 0.0 },
  { "Quadir", 1.0 },
  { "Quadira", 0.0 },
  { "Quadirah", 0.0 },
  { "Quadre", 1.0 },
  { "Quadree", 1.0 },
  { "Quadreka", 0.0 },
  { "Quadrell", 1.0 },
  { "Quadrevion", 1.0 },
  { "Quadri", 1.0 },
  { "Quadry", 1.0 },
  { "Quaheem", 1.0 },
  { "Quaid", 1.0 },
  { "Quala", 0.0 },
  { "Qualan", 1.0 },
  { "Qualee", 1.0 },
  { "Qualen", 1.0 },
  { "Qualesha", 0.0 },
  { "Quali", 1.0 },
  { "Qualiek", 1.0 },
  { "Qualik", 1.0 },
  { "Qualin", 1.0 },
  { "Qualisha", 0.0 },
  { "Qualiyah", 0.0 },
  { "Qualon", 1.0 },
  { "Qualyn", 1.0 },
  { "Quamae", 1.0 },
  { "Quamain", 1.0 },
  { "Quamaine", 1.0 },
  { "Quamane", 1.0 },
  { "Quamar", 1.0 },
  { "Quamari", 1.0 },
  { "Quamarion", 1.0 },
  { "Quamayne", 1.0 },
  { "Quame", 1.0 },
  { "Quameek", 1.0 },
  { "Quameer", 1.0 },
  { "Quameir", 1.0 },
  { "Quameisha", 0.0 },
  { "Quamel", 1.0 },
  { "Quamell", 1.0 },
  { "Quamere", 1.0 },
  { "Quamesha", 0.0 },
  { "Quameshia", 0.0 },
  { "Quamine", 1.0 },
  { "Quamir", 1.0 },
  { "Quamire", 1.0 },
  { "Quamisha", 0.0 },
  { "Quamon", 1.0 },
  { "Quamya", 0.0 },
  { "Quan", 0.9703422 },
  { "Quana", 0.0 },
  { "Quanae", 0.0 },
  { "Quanah", 0.82300884 },
  { "Quanaisha", 0.0 },
  { "Quanasha", 0.0 },
  { "Quanasia", 0.0 },
  { "Quanda", 0.0 },
  { "Quandarious", 1.0 },
  { "Quandarius", 1.0 },
  { "Quandarrius", 1.0 },
  { "Quandell", 1.0 },
  { "Quandra", 0.0 },
  { "Quandre", 1.0 },
  { "Quanecia", 0.0 },
  { "Quaneeka", 0.0 },
  { "Quaneesha", 0.0 },
  { "Quaneisha", 0.0 },
  { "Quaneka", 0.0 },
  { "Quanell", 1.0 },
  { "Quanesha", 0.0 },
  { "Quaneshia", 0.0 },
  { "Quanesia", 0.0 },
  { "Quanessa", 0.0 },
  { "Quanetra", 0.0 },
  { "Quanetta", 0.0 },
  { "Quanette", 0.0 },
  { "Quang", 1.0 },
  { "Quanice", 0.0 },
  { "Quanicia", 0.0 },
  { "Quaniece", 0.0 },
  { "Quaniesha", 0.0 },
  { "Quanika", 0.0 },
  { "Quaniqua", 0.0 },
  { "Quanique", 0.0 },
  { "Quanise", 0.0 },
  { "Quanisha", 0.0 },
  { "Quanishia", 0.0 },
  { "Quanita", 0.0 },
  { "Quaniya", 0.0 },
  { "Quaniyah", 0.0 },
  { "Quanna", 0.0 },
  { "Quannah", 0.0 },
  { "Quannell", 1.0 },
  { "Quannisha", 0.0 },
  { "Quanshae", 0.0 },
  { "Quanshay", 0.0 },
  { "Quanta", 0.42944786 },
  { "Quantae", 1.0 },
  { "Quantarius", 1.0 },
  { "Quantas", 1.0 },
  { "Quantasha", 0.0 },
  { "Quantashia", 0.0 },
  { "Quantasia", 0.0 },
  { "Quantavia", 0.0 },
  { "Quantavious", 1.0 },
  { "Quantavis", 1.0 },
  { "Quantavius", 1.0 },
  { "Quantay", 1.0 },
  { "Quantaya", 0.0 },
  { "Quante", 0.97959185 },
  { "Quantel", 1.0 },
  { "Quantella", 0.0 },
  { "Quanteria", 0.0 },
  { "Quanterious", 1.0 },
  { "Quanterius", 1.0 },
  { "Quanterra", 0.0 },
  { "Quanterrious", 1.0 },
  { "Quanterrius", 1.0 },
  { "Quantesha", 0.0 },
  { "Quantez", 1.0 },
  { "Quantia", 0.0 },
  { "Quantiana", 0.0 },
  { "Quantina", 0.0 },
  { "Quantis", 1.0 },
  { "Quantisha", 0.0 },
  { "Quantravious", 1.0 },
  { "Quantre", 1.0 },
  { "Quantrel", 1.0 },
  { "Quantrell", 1.0 },
  { "Quantrez", 1.0 },
  { "Quanya", 0.0 },
  { "Quanye", 1.0 },
  { "Quanza", 0.0 },
  { "Quanzell", 1.0 },
  { "Quanzie", 1.0 },
  { "Quaran", 1.0 },
  { "Quarmaine", 1.0 },
  { "Quaron", 1.0 },
  { "Quarterrio", 1.0 },
  { "Quartey", 1.0 },
  { "Quartez", 1.0 },
  { "Quartney", 0.0 },
  { "Quartterio", 1.0 },
  { "Quasean", 1.0 },
  { "Quaseem", 1.0 },
  { "Quasha", 0.0 },
  { "Quashan", 1.0 },
  { "Quashana", 0.0 },
  { "Quashanda", 0.0 },
  { "Quashanna", 0.0 },
  { "Quashaun", 1.0 },
  { "Quashawn", 0.9924585 },
  { "Quashawna", 0.0 },
  { "Quashay", 0.0 },
  { "Quashayla", 0.0 },
  { "Quasheema", 0.0 },
  { "Quashia", 0.0 },
  { "Quashon", 1.0 },
  { "Quashonda", 0.0 },
  { "Quashun", 1.0 },
  { "Quasia", 0.0 },
  { "Quasim", 1.0 },
  { "Quasir", 1.0 },
  { "Quason", 1.0 },
  { "Quatashia", 0.0 },
  { "Quatasia", 0.0 },
  { "Quatavia", 0.0 },
  { "Quatavious", 1.0 },
  { "Quatavius", 1.0 },
  { "Quateria", 0.0 },
  { "Quaterius", 1.0 },
  { "Quaterrius", 1.0 },
  { "Quatesha", 0.0 },
  { "Quatez", 1.0 },
  { "Quatia", 0.0 },
  { "Quatina", 0.0 },
  { "Quatisha", 0.0 },
  { "Quatravious", 1.0 },
  { "Quatrell", 1.0 },
  { "Quatrina", 0.0 },
  { "Quaveon", 1.0 },
  { "Quavion", 1.0 },
  { "Quavious", 1.0 },
  { "Quavis", 1.0 },
  { "Quavius", 1.0 },
  { "Quavon", 1.0 },
  { "Quavonte", 1.0 },
  { "Quay", 0.9591281 },
  { "Quayden", 1.0 },
  { "Quayla", 0.0 },
  { "Quaylan", 1.0 },
  { "Quaylen", 1.0 },
  { "Quaylin", 1.0 },
  { "Quaylon", 1.0 },
  { "Quayon", 1.0 },
  { "Quaysean", 1.0 },
  { "Quayshaun", 1.0 },
  { "Quayshawn", 0.9717514 },
  { "Quayshon", 1.0 },
  { "Quayshun", 1.0 },
  { "Quayvon", 1.0 },
  { "Quazar", 1.0 },
  { "Quazi", 1.0 },
  { "Qubilah", 0.0 },
  { "Qudarius", 1.0 },
  { "Que", 1.0 },
  { "Queana", 0.0 },
  { "Queanna", 0.0 },
  { "Queen", 0.0021563717 },
  { "Queena", 0.0 },
  { "Queenasia", 0.0 },
  { "Queency", 0.0 },
  { "Queene", 0.0 },
  { "Queenell", 0.0 },
  { "Queenester", 0.0 },
  { "Queenie", 0.0 },
  { "Queenisha", 0.0 },
  { "Quella", 0.0 },
  { "Quency", 1.0 },
  { "Quendarious", 1.0 },
  { "Quendarius", 1.0 },
  { "Queneisha", 0.0 },
  { "Quenesha", 0.0 },
  { "Queneshia", 0.0 },
  { "Quenetta", 0.0 },
  { "Quenette", 0.0 },
  { "Quenia", 0.0 },
  { "Quenisha", 0.0 },
  { "Quenita", 0.0 },
  { "Quenna", 0.0 },
  { "Quennel", 1.0 },
  { "Quennell", 1.0 },
  { "Quennie", 0.0 },
  { "Quent", 1.0 },
  { "Quentarius", 1.0 },
  { "Quentasia", 0.0 },
  { "Quentavia", 0.0 },
  { "Quentavious", 1.0 },
  { "Quentavius", 1.0 },
  { "Quentel", 1.0 },
  { "Quentella", 0.0 },
  { "Quenten", 1.0 },
  { "Quentez", 1.0 },
  { "Quentin", 0.99521774 },
  { "Quentina", 0.0 },
  { "Quentine", 1.0 },
  { "Quention", 1.0 },
  { "Quentisha", 0.0 },
  { "Quenton", 1.0 },
  { "Quentrell", 1.0 },
  { "Quentyn", 1.0 },
  { "Queren", 0.0 },
  { "Querida", 0.0 },
  { "Quesean", 1.0 },
  { "Quesha", 0.0 },
  { "Queshaun", 1.0 },
  { "Queshawn", 1.0 },
  { "Queshon", 1.0 },
  { "Quess", 1.0 },
  { "Quessie", 0.0 },
  { "Quest", 0.98039216 },
  { "Queston", 1.0 },
  { "Queta", 0.0 },
  { "Quetcy", 0.0 },
  { "Quetin", 1.0 },
  { "Quetzal", 0.41284403 },
  { "Quetzalcoatl", 1.0 },
  { "Quetzali", 0.0 },
  { "Quetzalli", 0.0 },
  { "Quetzally", 0.0 },
  { "Quetzaly", 0.0 },
  { "Quetzy", 0.0 },
  { "Quevin", 1.0 },
  { "Quevon", 1.0 },
  { "Qui", 1.0 },
  { "Quian", 0.52380955 },
  { "Quiana", 0.0 },
  { "Quiandra", 0.0 },
  { "Quianna", 0.0 },
  { "Quiara", 0.0 },
  { "Quida", 0.0 },
  { "Quido", 1.0 },
  { "Quiency", 1.0 },
  { "Quienten", 1.0 },
  { "Quientin", 1.0 },
  { "Quienton", 1.0 },
  { "Quiera", 0.0 },
  { "Quierra", 0.0 },
  { "Quiesha", 0.0 },
  { "Quigley", 1.0 },
  { "Quila", 0.0 },
  { "Quill", 1.0 },
  { "Quilla", 0.09638554 },
  { "Quillan", 1.0 },
  { "Quillar", 1.0 },
  { "Quillen", 1.0 },
  { "Quillian", 1.0 },
  { "Quillie", 0.56626505 },
  { "Quillon", 1.0 },
  { "Quimora", 0.0 },
  { "Quin", 0.7783933 },
  { "Quina", 0.0 },
  { "Quinasia", 0.0 },
  { "Quinaya", 0.0 },
  { "Quince", 1.0 },
  { "Quincee", 0.06410257 },
  { "Quincella", 0.0 },
  { "Quincey", 0.7505548 },
  { "Quinci", 0.121710524 },
  { "Quincie", 0.020408163 },
  { "Quincy", 0.87570494 },
  { "Quindale", 1.0 },
  { "Quindara", 0.0 },
  { "Quindarious", 1.0 },
  { "Quindarius", 1.0 },
  { "Quindarrius", 1.0 },
  { "Quindell", 1.0 },
  { "Quinden", 1.0 },
  { "Quindon", 1.0 },
  { "Quineisha", 0.0 },
  { "Quineka", 0.0 },
  { "Quinell", 1.0 },
  { "Quinesha", 0.0 },
  { "Quineshia", 0.0 },
  { "Quinessa", 0.0 },
  { "Quinetta", 0.0 },
  { "Quinette", 0.0 },
  { "Quinisha", 0.0 },
  { "Quinita", 0.0 },
  { "Quiniya", 0.0 },
  { "Quiniyah", 0.0 },
  { "Quinlan", 0.7916667 },
  { "Quinlee", 0.0 },
  { "Quinleigh", 0.0 },
  { "Quinlen", 1.0 },
  { "Quinley", 0.0 },
  { "Quinlin", 0.97422683 },
  { "Quinlyn", 0.016438356 },
  { "Quinlynn", 0.0 },
  { "Quinn", 0.55117583 },
  { "Quinna", 0.0 },
  { "Quinncy", 0.6666667 },
  { "Quinne", 0.0 },
  { "Quinnel", 1.0 },
  { "Quinnell", 1.0 },
  { "Quinnesha", 0.0 },
  { "Quinneshia", 0.0 },
  { "Quinnetta", 0.0 },
  { "Quinnette", 0.0 },
  { "Quinnie", 0.060240965 },
  { "Quinnisha", 0.0 },
  { "Quinnita", 0.0 },
  { "Quinnlan", 0.79850745 },
  { "Quinnlee", 0.0 },
  { "Quinnleigh", 0.0 },
  { "Quinnley", 0.0 },
  { "Quinnly", 0.0 },
  { "Quinnlyn", 0.0 },
  { "Quinnlynn", 0.0 },
  { "Quinnshawn", 1.0 },
  { "Quinnten", 1.0 },
  { "Quinntin", 1.0 },
  { "Quinnton", 1.0 },
  { "Quinsha", 0.0 },
  { "Quinshawn", 1.0 },
  { "Quinshay", 0.0 },
  { "Quinshon", 1.0 },
  { "Quinston", 1.0 },
  { "Quint", 1.0 },
  { "Quinta", 0.1694215 },
  { "Quintae", 1.0 },
  { "Quintan", 1.0 },
  { "Quintana", 0.0 },
  { "Quintara", 0.0 },
  { "Quintarious", 1.0 },
  { "Quintarius", 1.0 },
  { "Quintarus", 1.0 },
  { "Quintasha", 0.0 },
  { "Quintashia", 0.0 },
  { "Quintasia", 0.0 },
  { "Quintavia", 0.0 },
  { "Quintavion", 1.0 },
  { "Quintavious", 1.0 },
  { "Quintavis", 1.0 },
  { "Quintavius", 1.0 },
  { "Quintay", 1.0 },
  { "Quintaya", 0.0 },
  { "Quinte", 1.0 },
  { "Quintel", 1.0 },
  { "Quintell", 0.9722222 },
  { "Quintella", 0.0 },
  { "Quinten", 1.0 },
  { "Quintera", 0.0 },
  { "Quinteria", 0.0 },
  { "Quinterious", 1.0 },
  { "Quinterius", 1.0 },
  { "Quintero", 1.0 },
  { "Quinterra", 0.0 },
  { "Quinterria", 0.0 },
  { "Quinterrious", 1.0 },
  { "Quinterrius", 1.0 },
  { "Quintesa", 0.0 },
  { "Quintesha", 0.0 },
  { "Quintessa", 0.0 },
  { "Quintez", 1.0 },
  { "Quintia", 0.0 },
  { "Quintin", 0.99951 },
  { "Quintina", 0.0 },
  { "Quintine", 1.0 },
  { "Quintion", 1.0 },
  { "Quintisha", 0.0 },
  { "Quinto", 1.0 },
  { "Quinton", 0.9985194 },
  { "Quintonia", 0.0 },
  { "Quintonio", 1.0 },
  { "Quintoria", 0.0 },
  { "Quintrel", 1.0 },
  { "Quintrell", 1.0 },
  { "Quintus", 1.0 },
  { "Quintyn", 1.0 },
  { "Quinya", 0.0 },
  { "Quinzel", 1.0 },
  { "Quinzell", 1.0 },
  { "Quion", 1.0 },
  { "Quiona", 0.0 },
  { "Quionna", 0.0 },
  { "Quiriat", 0.0 },
  { "Quirino", 1.0 },
  { "Quirt", 1.0 },
  { "Quisean", 1.0 },
  { "Quisha", 0.0 },
  { "Quishawn", 1.0 },
  { "Quisto", 1.0 },
  { "Quita", 0.0 },
  { "Quitin", 1.0 },
  { "Quitman", 1.0 },
  { "Quiton", 1.0 },
  { "Quiyana", 0.0 },
  { "Qujuan", 1.0 },
  { "Qumari", 1.0 },
  { "Quneisha", 0.0 },
  { "Qunesha", 0.0 },
  { "Qunicy", 1.0 },
  { "Qunisha", 0.0 },
  { "Qunita", 0.0 },
  { "Qunnie", 0.0 },
  { "Quoc", 1.0 },
  { "Quon", 1.0 },
  { "Quontavious", 1.0 },
  { "Quontez", 1.0 },
  { "Quora", 0.0 },
  { "Quorra", 0.0 },
  { "Quortney", 0.0 },
  { "Quovadis", 0.0 },
  { "Quraan", 1.0 },
  { "Quran", 0.98027843 },
  { "Quron", 1.0 },
  { "Qusai", 1.0 },
  { "Qusay", 1.0 },
  { "Qushawn", 1.0 },
  { "Quvondo", 1.0 },
  { "Quwan", 1.0 },
  { "Quwana", 0.0 },
  { "Quwanda", 0.0 },
  { "Quwanna", 0.0 },
  { "Quy", 1.0 },
  { "Quyen", 0.113513514 },
  { "Quynh", 0.0 },
  { "Quynhanh", 0.0 },
  { "Quynhnhu", 0.0 },
  { "Quynn", 0.0 },
  { "Quynton", 1.0 },
  { "Qwana", 0.0 },
  { "Qwanda", 0.0 },
  { "Qwanell", 1.0 },
  { "Qwanesha", 0.0 },
  { "Qwanisha", 0.0 },
  { "Qwen", 1.0 },
  { "Qwendolyn", 0.0 },
  { "Qwentin", 1.0 },
  { "Qwenton", 1.0 },
  { "Qwintin", 1.0 },
  { "Qwinton", 1.0 },
  { "Qwynn", 0.0 },
  { "Qynn", 0.0 },
  { "Ra", 0.8 },
  { "Raad", 1.0 },
  { "Raaed", 1.0 },
  { "Raafay", 1.0 },
  { "Raaga", 0.0 },
  { "Raaghav", 1.0 },
  { "Raahi", 0.08064516 },
  { "Raahil", 1.0 },
  { "Raahim", 1.0 },
  { "Raahul", 1.0 },
  { "Raaid", 1.0 },
  { "Raaina", 0.0 },
  { "Raam", 1.0 },
  { "Raamond", 1.0 },
  { "Raana", 0.0 },
  { "Raanan", 1.0 },
  { "Raashan", 1.0 },
  { "Raashi", 0.0 },
  { "Raashid", 1.0 },
  { "Raashida", 0.0 },
  { "Raavi", 0.0 },
  { "Raaya", 0.0 },
  { "Raayan", 1.0 },
  { "Raaziq", 1.0 },
  { "Rabab", 0.0 },
  { "Rabecca", 0.0 },
  { "Rabecka", 0.0 },
  { "Rabeka", 0.0 },
  { "Rabekah", 0.0 },
  { "Rabekka", 0.0 },
  { "Rabert", 1.0 },
  { "Rabi", 1.0 },
  { "Rabia", 0.0 },
  { "Rabiah", 0.0 },
  { "Rabih", 1.0 },
  { "Rabihah", 0.0 },
  { "Rabiya", 0.0 },
  { "Rabon", 1.0 },
  { "Rabun", 1.0 },
  { "Raburn", 1.0 },
  { "Raby", 1.0 },
  { "Race", 1.0 },
  { "Racel", 0.0 },
  { "Racelyn", 0.0 },
  { "Racen", 1.0 },
  { "Racer", 1.0 },
  { "Racey", 0.0 },
  { "Rachad", 1.0 },
  { "Rachae", 0.0 },
  { "Rachael", 0.0022062408 },
  { "Rachal", 0.0 },
  { "Rachale", 0.0 },
  { "Rachana", 0.0 },
  { "Rachard", 1.0 },
  { "Rache", 0.0 },
  { "Racheal", 0.0 },
  { "Rachel", 0.0031716323 },
  { "Rachelann", 0.0 },
  { "Rachelanne", 0.0 },
  { "Rachele", 0.0 },
  { "Rachell", 0.0 },
  { "Rachella", 0.0 },
  { "Rachelle", 0.0011044446 },
  { "Rachelleann", 0.0 },
  { "Rachelleanne", 0.0 },
  { "Rachelmarie", 0.0 },
  { "Rachely", 0.0 },
  { "Rachid", 1.0 },
  { "Rachiel", 0.0 },
  { "Rachit", 1.0 },
  { "Rachita", 0.0 },
  { "Rachmiel", 1.0 },
  { "Rachna", 0.0 },
  { "Rachon", 0.0 },
  { "Rachyl", 0.0 },
  { "Racie", 0.0 },
  { "Raciel", 1.0 },
  { "Racin", 1.0 },
  { "Racine", 0.008333334 },
  { "Racio", 0.0 },
  { "Rackel", 0.0 },
  { "Rackwon", 1.0 },
  { "Racqual", 0.0 },
  { "Racquel", 0.0010642827 },
  { "Racquell", 0.0 },
  { "Racquelle", 0.0 },
  { "Racy", 0.0 },
  { "Racyn", 1.0 },
  { "Rad", 1.0 },
  { "Rada", 0.0 },
  { "Radame", 1.0 },
  { "Radamel", 1.0 },
  { "Radames", 1.0 },
  { "Radarius", 1.0 },
  { "Radcliffe", 1.0 },
  { "Rade", 1.0 },
  { "Radee", 1.0 },
  { "Radeen", 1.0 },
  { "Radek", 1.0 },
  { "Radell", 0.54545456 },
  { "Raden", 1.0 },
  { "Radene", 0.0 },
  { "Rader", 1.0 },
  { "Radford", 1.0 },
  { "Radha", 0.0 },
  { "Radhames", 1.0 },
  { "Radhika", 0.0 },
  { "Radi", 1.0 },
  { "Radia", 0.0 },
  { "Radiah", 0.0 },
  { "Radiance", 0.0 },
  { "Radie", 0.035928145 },
  { "Radin", 1.0 },
  { "Radine", 0.0 },
  { "Radiya", 0.0 },
  { "Radlee", 0.84375 },
  { "Radley", 0.94984645 },
  { "Radly", 1.0 },
  { "Radoika", 0.0 },
  { "Radonna", 0.0 },
  { "Radoslav", 1.0 },
  { "Radric", 1.0 },
  { "Radu", 1.0 },
  { "Radwa", 0.0 },
  { "Radwan", 1.0 },
  { "Radyn", 1.0 },
  { "Rae", 0.039678667 },
  { "Raea", 0.0 },
  { "Raeah", 0.0 },
  { "Raeana", 0.0 },
  { "Raeanah", 0.0 },
  { "Raeann", 0.0 },
  { "Raeanna", 0.0 },
  { "Raeanne", 0.0 },
  { "Raeburn", 1.0 },
  { "Raechal", 0.0 },
  { "Raechel", 0.0 },
  { "Raechell", 0.0 },
  { "Raechelle", 0.0 },
  { "Raed", 1.0 },
  { "Raedawn", 0.0 },
  { "Raedean", 0.0 },
  { "Raeden", 0.6574074 },
  { "Raedene", 0.0 },
  { "Raedon", 1.0 },
  { "Raedyn", 0.34600762 },
  { "Raeed", 1.0 },
  { "Raees", 1.0 },
  { "Raeesah", 0.0 },
  { "Raef", 1.0 },
  { "Raeford", 1.0 },
  { "Raegan", 0.010185185 },
  { "Raegann", 0.0 },
  { "Raegen", 0.006821282 },
  { "Raegene", 0.0 },
  { "Raeghan", 0.0 },
  { "Raegin", 0.0 },
  { "Raegina", 0.0 },
  { "Raegine", 0.0 },
  { "Raegyn", 0.0 },
  { "Raehanna", 0.0 },
  { "Raeisha", 0.0 },
  { "Raejean", 0.0 },
  { "Raejon", 1.0 },
  { "Raekwan", 1.0 },
  { "Raekwon", 0.9972028 },
  { "Rael", 0.51713395 },
  { "Raela", 0.0 },
  { "Raelah", 0.0 },
  { "Raelan", 0.05952381 },
  { "Raelani", 0.0 },
  { "Raelea", 0.0 },
  { "Raeleah", 0.0 },
  { "Raelean", 0.0 },
  { "Raelee", 0.0 },
  { "Raeleen", 0.0 },
  { "Raelei", 0.0 },
  { "Raeleigh", 0.0 },
  { "Raelen", 0.0 },
  { "Raelena", 0.0 },
  { "Raelene", 0.0 },
  { "Raeley", 0.0 },
  { "Raeli", 0.0 },
  { "Raelie", 0.0 },
  { "Raeliegh", 0.0 },
  { "Raelin", 0.0 },
  { "Raelina", 0.0 },
  { "Raelinn", 0.0 },
  { "Raelle", 0.0 },
  { "Raellen", 0.0 },
  { "Raely", 0.0 },
  { "Raelyn", 0.0024257125 },
  { "Raelyne", 0.0 },
  { "Raelynn", 0.0011017882 },
  { "Raelynne", 0.0 },
  { "Raemarie", 0.0 },
  { "Raemi", 0.0 },
  { "Raemon", 1.0 },
  { "Raemona", 0.0 },
  { "Raemond", 1.0 },
  { "Raen", 0.0 },
  { "Raena", 0.0 },
  { "Raenada", 0.0 },
  { "Raenae", 0.0 },
  { "Raenah", 0.0 },
  { "Raene", 0.0 },
  { "Raenee", 0.0 },
  { "Raeneisha", 0.0 },
  { "Raenell", 0.0 },
  { "Raenelle", 0.0 },
  { "Raenesha", 0.0 },
  { "Raenette", 0.0 },
  { "Raengel", 0.0 },
  { "Raenisha", 0.0 },
  { "Raeniyah", 0.0 },
  { "Raenna", 0.0 },
  { "Raeola", 0.0 },
  { "Raeona", 0.0 },
  { "Raeonna", 0.0 },
  { "Raequan", 1.0 },
  { "Raequawn", 1.0 },
  { "Raequel", 0.0 },
  { "Raequon", 1.0 },
  { "Raeqwan", 1.0 },
  { "Raeqwon", 1.0 },
  { "Raesean", 1.0 },
  { "Raesha", 0.0 },
  { "Raeshaun", 1.0 },
  { "Raeshawn", 0.71095574 },
  { "Raeshell", 0.0 },
  { "Raeshelle", 0.0 },
  { "Raeshon", 1.0 },
  { "Raetta", 0.0 },
  { "Raevan", 0.0 },
  { "Raeven", 0.0 },
  { "Raevin", 0.0 },
  { "Raevon", 0.90178573 },
  { "Raevyn", 0.0 },
  { "Raevynn", 0.0 },
  { "Raewyn", 0.0 },
  { "Raey", 1.0 },
  { "Raeya", 0.0 },
  { "Raeyah", 0.0 },
  { "Raeyana", 0.0 },
  { "Raeyanna", 0.0 },
  { "Raeyonna", 0.0 },
  { "Rafa", 0.41379312 },
  { "Rafael", 0.99257207 },
  { "Rafaela", 0.0 },
  { "Rafaelita", 0.0 },
  { "Rafaella", 0.0 },
  { "Rafaila", 0.0 },
  { "Rafal", 1.0 },
  { "Rafan", 1.0 },
  { "Rafat", 1.0 },
  { "Rafay", 1.0 },
  { "Rafe", 1.0 },
  { "Rafeal", 1.0 },
  { "Rafee", 1.0 },
  { "Rafeef", 0.0 },
  { "Rafeek", 1.0 },
  { "Rafeeq", 1.0 },
  { "Rafel", 1.0 },
  { "Rafela", 0.0 },
  { "Rafelita", 0.0 },
  { "Rafer", 1.0 },
  { "Raffael", 1.0 },
  { "Raffaela", 0.0 },
  { "Raffaele", 1.0 },
  { "Raffaella", 0.0 },
  { "Raffeal", 1.0 },
  { "Rafferty", 1.0 },
  { "Raffi", 1.0 },
  { "Raffie", 0.17857143 },
  { "Raffiel", 1.0 },
  { "Raffinee", 0.0 },
  { "Raffy", 1.0 },
  { "Rafhael", 1.0 },
  { "Rafi", 1.0 },
  { "Rafia", 0.0 },
  { "Rafid", 1.0 },
  { "Rafiel", 1.0 },
  { "Rafif", 0.0 },
  { "Rafik", 1.0 },
  { "Rafiq", 1.0 },
  { "Raford", 1.0 },
  { "Rafuel", 1.0 },
  { "Ragad", 0.0 },
  { "Ragan", 0.17529137 },
  { "Ragav", 1.0 },
  { "Rage", 1.0 },
  { "Ragen", 0.026595745 },
  { "Ragena", 0.0 },
  { "Ragene", 0.16666667 },
  { "Ragenia", 0.0 },
  { "Raghad", 0.0 },
  { "Raghav", 1.0 },
  { "Raghavi", 0.0 },
  { "Raghib", 1.0 },
  { "Ragin", 0.0 },
  { "Ragina", 0.0 },
  { "Ragine", 0.0 },
  { "Ragini", 0.0 },
  { "Ragna", 0.0 },
  { "Ragnar", 1.0 },
  { "Ragnhild", 0.0 },
  { "Raguel", 0.0 },
  { "Raha", 0.0 },
  { "Rahab", 0.0 },
  { "Rahaf", 0.0 },
  { "Rahama", 0.0 },
  { "Rahaman", 1.0 },
  { "Rahanna", 0.0 },
  { "Rahasia", 0.0 },
  { "Rahat", 0.9019608 },
  { "Rahcel", 0.0 },
  { "Raheam", 1.0 },
  { "Raheel", 0.9395161 },
  { "Raheem", 0.99915195 },
  { "Raheema", 0.0 },
  { "Raheemah", 0.0 },
  { "Raheen", 1.0 },
  { "Raheim", 1.0 },
  { "Rahel", 0.0 },
  { "Rahela", 0.0 },
  { "Rahem", 1.0 },
  { "Rahf", 0.0 },
  { "Rahi", 0.16949153 },
  { "Rahib", 1.0 },
  { "Rahiem", 1.0 },
  { "Rahil", 0.9617978 },
  { "Rahila", 0.0 },
  { "Rahim", 1.0 },
  { "Rahima", 0.0 },
  { "Rahini", 0.0 },
  { "Rahja", 0.53846157 },
  { "Rahjae", 0.0 },
  { "Rahjon", 1.0 },
  { "Rahkeem", 1.0 },
  { "Rahkia", 0.0 },
  { "Rahkim", 1.0 },
  { "Rahm", 1.0 },
  { "Rahma", 0.0 },
  { "Rahmaan", 1.0 },
  { "Rahmad", 1.0 },
  { "Rahmah", 0.0 },
  { "Rahman", 1.0 },
  { "Rahmat", 0.0 },
  { "Rahmeek", 1.0 },
  { "Rahmeir", 1.0 },
  { "Rahmel", 1.0 },
  { "Rahmell", 1.0 },
  { "Rahmere", 1.0 },
  { "Rahmi", 1.0 },
  { "Rahmin", 1.0 },
  { "Rahmir", 1.0 },
  { "Rahmo", 0.0 },
  { "Rahmon", 1.0 },
  { "Rahmya", 0.0 },
  { "Rahn", 1.0 },
  { "Rahna", 0.0 },
  { "Rahnasia", 0.0 },
  { "Rahneisha", 0.0 },
  { "Rahni", 0.0 },
  { "Rahniya", 0.0 },
  { "Rahniyah", 0.0 },
  { "Rahquan", 1.0 },
  { "Rahsaan", 0.9961014 },
  { "Rahsan", 1.0 },
  { "Rahsaun", 1.0 },
  { "Rahsean", 1.0 },
  { "Rahshad", 1.0 },
  { "Rahshan", 1.0 },
  { "Rahshawn", 1.0 },
  { "Rahsheed", 1.0 },
  { "Rahsheen", 1.0 },
  { "Rahshida", 0.0 },
  { "Rahshon", 1.0 },
  { "Rahson", 1.0 },
  { "Rahul", 1.0 },
  { "Rahwa", 0.0 },
  { "Rahyl", 1.0 },
  { "Rahzel", 1.0 },
  { "Rai", 0.6931818 },
  { "Raia", 0.0 },
  { "Raiah", 0.0 },
  { "Raian", 1.0 },
  { "Raiana", 0.0 },
  { "Raianna", 0.0 },
  { "Raianne", 0.0 },
  { "Raice", 1.0 },
  { "Raichel", 0.0 },
  { "Raichelle", 0.0 },
  { "Raid", 1.0 },
  { "Raida", 0.0 },
  { "Raidan", 1.0 },
  { "Raidel", 1.0 },
  { "Raiden", 0.96530294 },
  { "Raider", 1.0 },
  { "Raidon", 1.0 },
  { "Raidyn", 0.859375 },
  { "Raielle", 0.0 },
  { "Raiesha", 0.0 },
  { "Raif", 1.0 },
  { "Raifa", 0.0 },
  { "Raifah", 0.0 },
  { "Raife", 1.0 },
  { "Raiford", 1.0 },
  { "Raigan", 0.0 },
  { "Raigen", 0.0 },
  { "Raighan", 0.0 },
  { "Raighlyn", 0.0 },
  { "Raigyn", 0.0 },
  { "Raihaan", 1.0 },
  { "Raihaanah", 0.0 },
  { "Raihan", 1.0 },
  { "Raihana", 0.0 },
  { "Raihanna", 0.0 },
  { "Raija", 0.0 },
  { "Raijon", 1.0 },
  { "Raika", 0.0 },
  { "Railee", 0.0 },
  { "Raileen", 0.0 },
  { "Raileigh", 0.0 },
  { "Railey", 0.07350689 },
  { "Raily", 0.5833333 },
  { "Railyn", 0.0 },
  { "Railynn", 0.0 },
  { "Railynne", 0.0 },
  { "Raima", 0.0 },
  { "Raime", 0.0 },
  { "Raimee", 0.0 },
  { "Raimey", 0.0 },
  { "Raimi", 0.0 },
  { "Raimon", 1.0 },
  { "Raimond", 1.0 },
  { "Raimondo", 1.0 },
  { "Raimund", 1.0 },
  { "Raimundo", 1.0 },
  { "Rain", 0.21303883 },
  { "Raina", 0.0 },
  { "Rainah", 0.0 },
  { "Rainan", 1.0 },
  { "Rainbow", 0.0 },
  { "Raine", 0.17206563 },
  { "Rainee", 0.0 },
  { "Rainell", 0.0 },
  { "Rainelle", 0.0 },
  { "Rainen", 1.0 },
  { "Rainer", 0.9883721 },
  { "Raines", 1.0 },
  { "Rainey", 0.22256097 },
  { "Raini", 0.10995851 },
  { "Rainie", 0.0 },
  { "Rainier", 0.98618305 },
  { "Rainn", 0.4178082 },
  { "Rainna", 0.0 },
  { "Rainne", 0.0 },
  { "Rainy", 0.0 },
  { "Raionna", 0.0 },
  { "Raiqa", 0.0 },
  { "Raiquan", 1.0 },
  { "Rais", 1.0 },
  { "Raisa", 0.0 },
  { "Raisah", 0.0 },
  { "Raisean", 1.0 },
  { "Raisha", 0.0 },
  { "Raishaun", 1.0 },
  { "Raishawn", 1.0 },
  { "Raissa", 0.0 },
  { "Raistlin", 1.0 },
  { "Raith", 1.0 },
  { "Raiven", 0.0 },
  { "Raivyn", 0.0 },
  { "Raiya", 0.0 },
  { "Raiyah", 0.0 },
  { "Raiyan", 1.0 },
  { "Raiyana", 0.0 },
  { "Raiyn", 0.0 },
  { "Raiyna", 0.0 },
  { "Raiyne", 0.0 },
  { "Raiza", 0.0 },
  { "Raizel", 0.0 },
  { "Raizo", 1.0 },
  { "Raizy", 0.0 },
  { "Raj", 1.0 },
  { "Raja", 0.6807692 },
  { "Rajaa", 0.0 },
  { "Rajab", 1.0 },
  { "Rajae", 0.65625 },
  { "Rajaee", 1.0 },
  { "Rajah", 0.67075664 },
  { "Rajahn", 1.0 },
  { "Rajai", 1.0 },
  { "Rajan", 1.0 },
  { "Rajana", 0.0 },
  { "Rajanae", 0.0 },
  { "Rajane", 0.0 },
  { "Rajanee", 0.0 },
  { "Rajani", 0.0 },
  { "Rajat", 1.0 },
  { "Rajaun", 1.0 },
  { "Rajbir", 1.0 },
  { "Rajdeep", 0.78846157 },
  { "Rajean", 0.0 },
  { "Rajeana", 0.0 },
  { "Rajeen", 0.0 },
  { "Rajeeni", 0.0 },
  { "Rajeev", 1.0 },
  { "Rajeeyah", 0.0 },
  { "Rajen", 1.0 },
  { "Rajendra", 1.0 },
  { "Rajene", 0.0 },
  { "Rajesh", 1.0 },
  { "Raji", 1.0 },
  { "Rajinder", 1.0 },
  { "Rajit", 1.0 },
  { "Rajiv", 1.0 },
  { "Rajko", 1.0 },
  { "Rajkumar", 1.0 },
  { "Rajneesh", 1.0 },
  { "Rajni", 0.0 },
  { "Rajohn", 1.0 },
  { "Rajon", 1.0 },
  { "Raju", 1.0 },
  { "Rajuan", 1.0 },
  { "Rajveer", 1.0 },
  { "Rajvi", 0.0 },
  { "Rajvir", 1.0 },
  { "Rakai", 1.0 },
  { "Rakan", 1.0 },
  { "Rakari", 1.0 },
  { "Rakaya", 0.0 },
  { "Rakayla", 0.0 },
  { "Rakeb", 0.0 },
  { "Rakee", 1.0 },
  { "Rakeem", 1.0 },
  { "Rakeen", 1.0 },
  { "Rakeia", 0.0 },
  { "Rakeim", 1.0 },
  { "Rakeisha", 0.0 },
  { "Rakel", 0.0 },
  { "Rakell", 0.0 },
  { "Rakelle", 0.0 },
  { "Rakem", 1.0 },
  { "Rakeria", 0.0 },
  { "Rakesh", 1.0 },
  { "Rakesha", 0.0 },
  { "Rakeshia", 0.0 },
  { "Rakeya", 0.0 },
  { "Rakhee", 0.0 },
  { "Rakhi", 0.084745765 },
  { "Rakhia", 0.0 },
  { "Rakia", 0.0 },
  { "Rakiah", 0.0 },
  { "Rakib", 1.0 },
  { "Rakiem", 1.0 },
  { "Rakiesha", 0.0 },
  { "Rakim", 1.0 },
  { "Rakin", 1.0 },
  { "Raking", 1.0 },
  { "Rakira", 0.0 },
  { "Rakisha", 0.0 },
  { "Rakita", 0.0 },
  { "Rakiya", 0.0 },
  { "Rakiyah", 0.0 },
  { "Raksha", 0.0 },
  { "Rakshan", 1.0 },
  { "Rakshana", 0.0 },
  { "Rakshitha", 0.0 },
  { "Raksmey", 0.0 },
  { "Rakwan", 1.0 },
  { "Rakwon", 1.0 },
  { "Rakya", 0.0 },
  { "Rakyah", 0.0 },
  { "Rakye", 1.0 },
  { "Rakyia", 0.0 },
  { "Rakyla", 0.0 },
  { "Ral", 1.0 },
  { "Ralana", 0.0 },
  { "Raland", 1.0 },
  { "Ralanda", 0.0 },
  { "Ralayah", 0.0 },
  { "Ralee", 0.0 },
  { "Raleen", 0.0 },
  { "Raleigh", 0.8226177 },
  { "Ralen", 1.0 },
  { "Ralena", 0.0 },
  { "Ralene", 0.0 },
  { "Raley", 0.0 },
  { "Ralf", 1.0 },
  { "Raliegh", 1.0 },
  { "Raliek", 1.0 },
  { "Ralik", 1.0 },
  { "Ralinda", 0.0 },
  { "Ralique", 1.0 },
  { "Ralisha", 0.0 },
  { "Raliyah", 0.0 },
  { "Rallie", 1.0 },
  { "Ralls", 1.0 },
  { "Rally", 0.9305556 },
  { "Ralna", 0.0 },
  { "Ralon", 1.0 },
  { "Ralonda", 0.0 },
  { "Ralph", 0.9955587 },
  { "Ralpha", 0.0 },
  { "Ralphael", 1.0 },
  { "Ralphe", 1.0 },
  { "Ralpheal", 1.0 },
  { "Ralphel", 1.0 },
  { "Ralphell", 1.0 },
  { "Ralphie", 1.0 },
  { "Ralphine", 0.0 },
  { "Ralphy", 1.0 },
  { "Ralston", 1.0 },
  { "Ralyn", 0.0 },
  { "Ralynn", 0.0 },
  { "Ram", 1.0 },
  { "Rama", 0.1636953 },
  { "Ramada", 0.0 },
  { "Ramadan", 1.0 },
  { "Ramah", 0.0 },
  { "Ramaj", 1.0 },
  { "Ramal", 1.0 },
  { "Raman", 1.0 },
  { "Ramana", 1.0 },
  { "Ramanda", 0.0 },
  { "Ramandeep", 0.1553398 },
  { "Ramani", 0.0 },
  { "Ramar", 1.0 },
  { "Ramari", 0.94505495 },
  { "Ramario", 1.0 },
  { "Ramarion", 1.0 },
  { "Ramata", 0.0 },
  { "Ramatou", 0.0 },
  { "Ramatoulaye", 0.0 },
  { "Ramatoulie", 0.0 },
  { "Ramatu", 0.0 },
  { "Ramaya", 0.0 },
  { "Ramayah", 0.0 },
  { "Ramazan", 1.0 },
  { "Rambo", 1.0 },
  { "Ramces", 1.0 },
  { "Ramee", 0.46153846 },
  { "Rameek", 1.0 },
  { "Rameel", 1.0 },
  { "Rameen", 0.34653464 },
  { "Rameer", 1.0 },
  { "Rameez", 1.0 },
  { "Rameir", 1.0 },
  { "Rameisha", 0.0 },
  { "Rameka", 0.0 },
  { "Ramel", 1.0 },
  { "Ramell", 1.0 },
  { "Ramelle", 0.18965517 },
  { "Ramello", 1.0 },
  { "Ramelo", 1.0 },
  { "Ramen", 1.0 },
  { "Ramere", 1.0 },
  { "Ramero", 1.0 },
  { "Rameses", 1.0 },
  { "Ramesh", 1.0 },
  { "Ramesha", 0.0 },
  { "Ramesses", 1.0 },
  { "Ramey", 0.601741 },
  { "Ramez", 1.0 },
  { "Rami", 0.9245552 },
  { "Ramia", 0.0 },
  { "Ramiah", 0.033333335 },
  { "Ramie", 0.27843988 },
  { "Ramiel", 1.0 },
  { "Ramiele", 0.0 },
  { "Ramielle", 0.0 },
  { "Ramier", 1.0 },
  { "Ramik", 1.0 },
  { "Ramika", 0.0 },
  { "Ramil", 1.0 },
  { "Ramin", 1.0 },
  { "Ramina", 0.0 },
  { "Ramir", 1.0 },
  { "Ramira", 0.0 },
  { "Ramirah", 0.0 },
  { "Ramire", 1.0 },
  { "Ramirez", 1.0 },
  { "Ramiro", 0.9988441 },
  { "Ramis", 1.0 },
  { "Ramisa", 0.0 },
  { "Ramisha", 0.0 },
  { "Ramiya", 0.0 },
  { "Ramiyah", 0.0 },
  { "Ramiz", 1.0 },
  { "Ramla", 0.0 },
  { "Ramlah", 0.0 },
  { "Rammy", 1.0 },
  { "Ramo", 1.0 },
  { "Ramoan", 1.0 },
  { "Ramola", 0.0 },
  { "Ramon", 0.99288344 },
  { "Ramona", 0.0036614416 },
  { "Ramoncita", 0.0 },
  { "Ramond", 1.0 },
  { "Ramonda", 0.0 },
  { "Ramondo", 1.0 },
  { "Ramone", 0.99469495 },
  { "Ramonia", 0.0 },
  { "Ramonica", 0.0 },
  { "Ramonita", 0.0 },
  { "Ramont", 1.0 },
  { "Ramonte", 1.0 },
  { "Ramos", 1.0 },
  { "Ramsay", 0.8825301 },
  { "Ramsen", 1.0 },
  { "Ramses", 1.0 },
  { "Ramsey", 0.7974063 },
  { "Ramsha", 0.0 },
  { "Ramsi", 0.0 },
  { "Ramsie", 0.0 },
  { "Ramson", 1.0 },
  { "Ramsses", 1.0 },
  { "Ramsy", 1.0 },
  { "Ramy", 1.0 },
  { "Ramya", 0.0 },
  { "Ramyah", 0.0 },
  { "Ramyia", 0.0 },
  { "Ramyiah", 0.0 },
  { "Ramzee", 0.7058824 },
  { "Ramzes", 1.0 },
  { "Ramzey", 1.0 },
  { "Ramzi", 0.99283665 },
  { "Ramzie", 0.0 },
  { "Ramzy", 1.0 },
  { "Ran", 0.89361703 },
  { "Rana", 0.022541966 },
  { "Ranaa", 0.0 },
  { "Ranada", 0.0 },
  { "Ranae", 0.0 },
  { "Ranaiya", 0.0 },
  { "Ranajah", 0.0 },
  { "Ranald", 1.0 },
  { "Ranaldo", 1.0 },
  { "Ranard", 1.0 },
  { "Ranarda", 0.0 },
  { "Ranardo", 1.0 },
  { "Ranasia", 0.0 },
  { "Ranata", 0.0 },
  { "Ranay", 0.0 },
  { "Ranaya", 0.0 },
  { "Ranayah", 0.0 },
  { "Ranaye", 0.0 },
  { "Ranbir", 1.0 },
  { "Rance", 1.0 },
  { "Rancel", 1.0 },
  { "Rances", 1.0 },
  { "Rand", 0.96391755 },
  { "Randa", 0.0 },
  { "Randahl", 1.0 },
  { "Randal", 0.99546516 },
  { "Randale", 1.0 },
  { "Randall", 0.99417305 },
  { "Randalle", 0.0 },
  { "Randalph", 1.0 },
  { "Randalyn", 0.0 },
  { "Randalynn", 0.0 },
  { "Randarius", 1.0 },
  { "Rande", 0.69529986 },
  { "Randee", 0.059786238 },
  { "Randeen", 0.0 },
  { "Randeep", 1.0 },
  { "Randeisha", 0.0 },
  { "Randel", 0.99314696 },
  { "Randell", 0.98524576 },
  { "Randelle", 0.0 },
  { "Randen", 1.0 },
  { "Randene", 0.0 },
  { "Randesha", 0.0 },
  { "Randey", 1.0 },
  { "Randi", 0.03735139 },
  { "Randie", 0.2531894 },
  { "Randiel", 1.0 },
  { "Randilee", 0.0 },
  { "Randilyn", 0.0 },
  { "Randilynn", 0.0 },
  { "Randin", 1.0 },
  { "Randine", 0.0 },
  { "Randisha", 0.0 },
  { "Randle", 0.99760765 },
  { "Randll", 1.0 },
  { "Randloph", 1.0 },
  { "Randol", 1.0 },
  { "Randolf", 1.0 },
  { "Randolfo", 1.0 },
  { "Randoll", 1.0 },
  { "Randolph", 0.99772805 },
  { "Randolyn", 0.0 },
  { "Random", 1.0 },
  { "Randon", 0.9941928 },
  { "Randoph", 1.0 },
  { "Randy", 0.9849882 },
  { "Randye", 0.0 },
  { "Randyl", 0.8333333 },
  { "Randyn", 1.0 },
  { "Rane", 0.5960912 },
  { "Ranea", 0.0 },
  { "Ranecia", 0.0 },
  { "Ranee", 0.0 },
  { "Raneem", 0.0 },
  { "Raneen", 0.0 },
  { "Raneesha", 0.0 },
  { "Raneisha", 0.0 },
  { "Raneka", 0.0 },
  { "Ranel", 1.0 },
  { "Ranell", 0.17613636 },
  { "Ranelle", 0.0 },
  { "Ranen", 1.0 },
  { "Ranesha", 0.0 },
  { "Raneshia", 0.0 },
  { "Ranessa", 0.0 },
  { "Ranetta", 0.0 },
  { "Ranette", 0.0 },
  { "Raney", 0.18441558 },
  { "Ranezmae", 0.0 },
  { "Ranezmay", 0.0 },
  { "Ranferi", 1.0 },
  { "Ranford", 1.0 },
  { "Range", 1.0 },
  { "Rangel", 1.0 },
  { "Ranger", 1.0 },
  { "Rangler", 1.0 },
  { "Rani", 0.09745533 },
  { "Rania", 0.0 },
  { "Raniah", 0.0 },
  { "Ranie", 0.0 },
  { "Raniel", 1.0 },
  { "Ranier", 1.0 },
  { "Raniesha", 0.0 },
  { "Ranijah", 0.0 },
  { "Ranika", 0.0 },
  { "Ranim", 0.0 },
  { "Raniqua", 0.0 },
  { "Ranique", 0.0 },
  { "Ranisha", 0.0 },
  { "Ranita", 0.0 },
  { "Raniya", 0.0 },
  { "Raniyah", 0.0 },
  { "Raniyha", 0.0 },
  { "Ranjan", 1.0 },
  { "Ranjana", 0.0 },
  { "Ranjani", 0.0 },
  { "Ranjeet", 1.0 },
  { "Ranjit", 1.0 },
  { "Rankin", 1.0 },
  { "Ranna", 0.0 },
  { "Rannah", 0.0 },
  { "Ranndy", 1.0 },
  { "Rannie", 0.82 },
  { "Rannon", 1.0 },
  { "Ranny", 0.98214287 },
  { "Ranoda", 0.0 },
  { "Ranold", 1.0 },
  { "Ranon", 1.0 },
  { "Ranota", 0.0 },
  { "Ransel", 1.0 },
  { "Ransen", 1.0 },
  { "Ransford", 1.0 },
  { "Ransom", 0.9987469 },
  { "Ransome", 1.0 },
  { "Ranson", 1.0 },
  { "Ranulfo", 1.0 },
  { "Ranveer", 1.0 },
  { "Ranvijay", 1.0 },
  { "Ranvir", 1.0 },
  { "Rany", 0.67741936 },
  { "Ranya", 0.0 },
  { "Ranyah", 0.0 },
  { "Ranyia", 0.0 },
  { "Ranyiah", 0.0 },
  { "Ranyla", 0.0 },
  { "Ranylah", 0.0 },
  { "Ranzel", 1.0 },
  { "Ranzino", 1.0 },
  { "Ranzy", 1.0 },
  { "Raoof", 1.0 },
  { "Raoul", 1.0 },
  { "Rapha", 1.0 },
  { "Raphael", 0.9831897 },
  { "Raphaela", 0.0 },
  { "Raphaella", 0.0 },
  { "Raphaelle", 0.0 },
  { "Rapheal", 1.0 },
  { "Raphel", 1.0 },
  { "Raphiel", 1.0 },
  { "Rapunzel", 0.0 },
  { "Raqeeb", 1.0 },
  { "Raqib", 1.0 },
  { "Raqiya", 0.0 },
  { "Raquael", 0.0 },
  { "Raqual", 0.0 },
  { "Raquan", 1.0 },
  { "Raquane", 1.0 },
  { "Raquawn", 1.0 },
  { "Raqueal", 0.0 },
  { "Raquel", 0.0057965633 },
  { "Raquela", 0.0 },
  { "Raquele", 0.0 },
  { "Raquelin", 0.0 },
  { "Raquell", 0.0 },
  { "Raquelle", 0.0 },
  { "Raquez", 1.0 },
  { "Raquia", 0.0 },
  { "Raquisha", 0.0 },
  { "Raquita", 0.0 },
  { "Raquon", 1.0 },
  { "Raqwan", 1.0 },
  { "Raqwon", 1.0 },
  { "Rarity", 0.0 },
  { "Ras", 1.0 },
  { "Rasa", 0.0 },
  { "Rasaan", 1.0 },
  { "Rasan", 1.0 },
  { "Rasaun", 1.0 },
  { "Raschad", 1.0 },
  { "Raschel", 0.0 },
  { "Raschell", 0.0 },
  { "Raschelle", 0.0 },
  { "Rascoe", 1.0 },
  { "Rasean", 1.0 },
  { "Raseel", 0.0 },
  { "Rasha", 0.05589124 },
  { "Rashaad", 1.0 },
  { "Rashaan", 0.9758621 },
  { "Rashad", 0.9943322 },
  { "Rashada", 0.0 },
  { "Rashadd", 1.0 },
  { "Rashade", 1.0 },
  { "Rashae", 0.0 },
  { "Rashael", 0.0 },
  { "Rashamel", 1.0 },
  { "Rashan", 0.93117833 },
  { "Rashana", 0.0 },
  { "Rashand", 1.0 },
  { "Rashanda", 0.0 },
  { "Rashandra", 0.0 },
  { "Rashanique", 0.0 },
  { "Rashann", 1.0 },
  { "Rashanna", 0.0 },
  { "Rashanti", 0.0 },
  { "Rashara", 0.0 },
  { "Rashard", 1.0 },
  { "Rashaud", 1.0 },
  { "Rashaun", 0.9414979 },
  { "Rashauna", 0.0 },
  { "Rashaunda", 0.0 },
  { "Rashaundra", 0.0 },
  { "Rashawd", 1.0 },
  { "Rashawn", 0.89360744 },
  { "Rashawna", 0.0 },
  { "Rashawnda", 0.0 },
  { "Rashay", 0.0 },
  { "Rashaya", 0.0 },
  { "Rashayla", 0.0 },
  { "Rashe", 1.0 },
  { "Rashea", 0.033333335 },
  { "Rashean", 1.0 },
  { "Rasheba", 0.0 },
  { "Rashed", 1.0 },
  { "Rasheda", 0.0 },
  { "Rashee", 1.0 },
  { "Rasheed", 0.99252087 },
  { "Rasheeda", 0.0058651026 },
  { "Rasheedah", 0.0 },
  { "Rasheedat", 0.0 },
  { "Rasheeka", 0.0 },
  { "Rasheem", 1.0 },
  { "Rasheema", 0.0 },
  { "Rasheen", 0.93389004 },
  { "Rasheena", 0.0 },
  { "Rasheeta", 0.0 },
  { "Rasheid", 1.0 },
  { "Rasheida", 0.0 },
  { "Rasheika", 0.0 },
  { "Rasheim", 1.0 },
  { "Rasheka", 0.0 },
  { "Rashel", 0.0 },
  { "Rashele", 0.0 },
  { "Rashell", 0.0 },
  { "Rashelle", 0.0 },
  { "Rashem", 1.0 },
  { "Rashema", 0.0 },
  { "Rashena", 0.0 },
  { "Rashene", 1.0 },
  { "Rasheta", 0.0 },
  { "Rashetta", 0.0 },
  { "Rashi", 0.3909091 },
  { "Rashia", 0.0 },
  { "Rashid", 1.0 },
  { "Rashida", 0.002016129 },
  { "Rashidah", 0.0 },
  { "Rashidat", 0.0 },
  { "Rashidi", 1.0 },
  { "Rashied", 1.0 },
  { "Rashieda", 0.0 },
  { "Rashieka", 0.0 },
  { "Rashiem", 1.0 },
  { "Rashik", 1.0 },
  { "Rashika", 0.0 },
  { "Rashim", 1.0 },
  { "Rashima", 0.0 },
  { "Rashina", 0.0 },
  { "Rashine", 1.0 },
  { "Rashita", 0.0 },
  { "Rashiya", 0.0 },
  { "Rashiyah", 0.0 },
  { "Rashmi", 0.0 },
  { "Rashmika", 0.0 },
  { "Rashod", 1.0 },
  { "Rashodd", 1.0 },
  { "Rashon", 0.9375 },
  { "Rashona", 0.0 },
  { "Rashonda", 0.0 },
  { "Rashondra", 0.0 },
  { "Rashone", 0.8235294 },
  { "Rashonna", 0.0 },
  { "Rashun", 0.96750903 },
  { "Rashunda", 0.0 },
  { "Rashundra", 0.0 },
  { "Rashya", 0.0 },
  { "Rashyia", 0.0 },
  { "Rasia", 0.0 },
  { "Rasika", 0.0 },
  { "Rasim", 1.0 },
  { "Rasiyah", 0.0 },
  { "Rasleen", 0.0 },
  { "Rasmine", 0.0 },
  { "Rasmus", 1.0 },
  { "Rason", 1.0 },
  { "Rasool", 1.0 },
  { "Rass", 1.0 },
  { "Rassan", 1.0 },
  { "Rassie", 0.8 },
  { "Rastus", 1.0 },
  { "Rasul", 1.0 },
  { "Rasun", 1.0 },
  { "Ratan", 1.0 },
  { "Ratana", 1.0 },
  { "Ratasha", 0.0 },
  { "Ratchel", 0.0 },
  { "Rateel", 0.0 },
  { "Ratesha", 0.0 },
  { "Ratha", 0.7352941 },
  { "Rathana", 0.796875 },
  { "Rathanak", 1.0 },
  { "Rathany", 0.0 },
  { "Ratisha", 0.0 },
  { "Ratonya", 0.0 },
  { "Rattana", 1.0 },
  { "Ratzy", 0.0 },
  { "Rauchelle", 0.0 },
  { "Raudel", 1.0 },
  { "Rauf", 1.0 },
  { "Rauha", 0.0 },
  { "Raul", 0.99424505 },
  { "Raulie", 1.0 },
  { "Raulin", 1.0 },
  { "Rauljr", 1.0 },
  { "Raun", 1.0 },
  { "Raunak", 1.0 },
  { "Raunda", 0.0 },
  { "Raunel", 1.0 },
  { "Rauni", 1.0 },
  { "Rauri", 0.2777778 },
  { "Raury", 1.0 },
  { "Raushan", 1.0 },
  { "Raushana", 0.0 },
  { "Raushanah", 0.0 },
  { "Raushaun", 1.0 },
  { "Rava", 0.0 },
  { "Ravae", 0.0 },
  { "Ravan", 0.0 },
  { "Ravaughn", 1.0 },
  { "Raveen", 0.0 },
  { "Raveena", 0.0 },
  { "Raven", 0.07191797 },
  { "Ravena", 0.0 },
  { "Ravene", 0.0 },
  { "Ravenel", 0.0 },
  { "Ravenell", 0.0 },
  { "Ravenna", 0.0 },
  { "Ravenne", 0.0 },
  { "Ravensimone", 0.0 },
  { "Ravensymone", 0.0 },
  { "Ravi", 0.9981638 },
  { "Ravien", 0.0 },
  { "Ravin", 0.20404722 },
  { "Ravina", 0.0 },
  { "Ravinder", 0.9489796 },
  { "Ravindra", 1.0 },
  { "Ravine", 0.0 },
  { "Ravion", 0.7966102 },
  { "Ravis", 1.0 },
  { "Raviv", 1.0 },
  { "Ravleen", 0.0 },
  { "Ravneet", 0.040462427 },
  { "Ravon", 0.76428574 },
  { "Ravonda", 0.0 },
  { "Ravonne", 0.0 },
  { "Ravonte", 1.0 },
  { "Ravy", 1.0 },
  { "Ravya", 0.0 },
  { "Ravyn", 0.0068593826 },
  { "Ravynn", 0.0 },
  { "Rawa", 0.0 },
  { "Rawad", 1.0 },
  { "Rawan", 0.0 },
  { "Rawda", 0.0 },
  { "Rawland", 1.0 },
  { "Rawle", 1.0 },
  { "Rawleigh", 1.0 },
  { "Rawley", 1.0 },
  { "Rawlin", 1.0 },
  { "Rawling", 1.0 },
  { "Rawlings", 1.0 },
  { "Rawlins", 1.0 },
  { "Rawly", 1.0 },
  { "Rawn", 1.0 },
  { "Rawnie", 0.0 },
  { "Rawson", 1.0 },
  { "Raxton", 1.0 },
  { "Ray", 0.97395086 },
  { "Raya", 0.0 },
  { "Rayaan", 0.83540374 },
  { "Rayah", 0.0 },
  { "Rayahna", 0.0 },
  { "Rayan", 0.8665917 },
  { "Rayana", 0.0 },
  { "Rayanah", 0.0 },
  { "Rayane", 0.94017094 },
  { "Rayann", 0.0033200532 },
  { "Rayanna", 0.0 },
  { "Rayannah", 0.0 },
  { "Rayanne", 0.0 },
  { "Rayansh", 1.0 },
  { "Rayanthony", 1.0 },
  { "Rayauna", 0.0 },
  { "Raybert", 1.0 },
  { "Raybon", 1.0 },
  { "Rayborn", 1.0 },
  { "Rayburn", 1.0 },
  { "Rayce", 0.99062234 },
  { "Raycen", 1.0 },
  { "Raychael", 0.0 },
  { "Raychel", 0.0 },
  { "Raychele", 0.0 },
  { "Raychell", 0.0 },
  { "Raychelle", 0.0 },
  { "Raycin", 1.0 },
  { "Raycine", 0.0 },
  { "Rayda", 0.0 },
  { "Raydan", 1.0 },
  { "Raydean", 0.5 },
  { "Raydel", 1.0 },
  { "Raydell", 0.9378882 },
  { "Rayden", 0.935989 },
  { "Raydene", 0.0 },
  { "Rayder", 1.0 },
  { "Raydin", 1.0 },
  { "Raydon", 1.0 },
  { "Raydrick", 1.0 },
  { "Raydyn", 1.0 },
  { "Raye", 0.12418831 },
  { "Rayea", 0.0 },
  { "Rayeann", 0.0 },
  { "Rayeanna", 0.0 },
  { "Rayegan", 0.0 },
  { "Rayel", 0.09836066 },
  { "Rayella", 0.0 },
  { "Rayelle", 0.0 },
  { "Rayelyn", 0.0 },
  { "Rayelynn", 0.0 },
  { "Rayen", 0.44886363 },
  { "Rayetta", 0.0 },
  { "Rayette", 0.0 },
  { "Rayfe", 1.0 },
  { "Rayfield", 1.0 },
  { "Rayford", 1.0 },
  { "Rayfus", 1.0 },
  { "Raygan", 0.043907795 },
  { "Raygen", 0.10114504 },
  { "Raygene", 0.0 },
  { "Raygina", 0.0 },
  { "Raygine", 0.0 },
  { "Rayha", 0.0 },
  { "Rayhaan", 1.0 },
  { "Rayhan", 0.992 },
  { "Rayhana", 0.0 },
  { "Rayhanna", 0.0 },
  { "Rayhona", 0.0 },
  { "Rayia", 0.0 },
  { "Rayion", 1.0 },
  { "Rayiona", 0.0 },
  { "Rayirth", 1.0 },
  { "Rayisha", 0.0 },
  { "Rayjanae", 0.0 },
  { "Rayjay", 1.0 },
  { "Rayjean", 0.0 },
  { "Rayjohn", 1.0 },
  { "Rayjon", 1.0 },
  { "Raykwan", 1.0 },
  { "Raykwon", 1.0 },
  { "Rayla", 0.0 },
  { "Raylah", 0.0 },
  { "Raylan", 0.9232072 },
  { "Rayland", 1.0 },
  { "Raylea", 0.0 },
  { "Rayleah", 0.0 },
  { "Raylean", 0.0 },
  { "Raylee", 0.0264879 },
  { "Rayleen", 0.0 },
  { "Raylei", 0.0 },
  { "Rayleigh", 0.0033738192 },
  { "Raylen", 0.633122 },
  { "Raylena", 0.0 },
  { "Raylend", 1.0 },
  { "Raylene", 0.0 },
  { "Raylenn", 0.0 },
  { "Rayley", 0.0 },
  { "Rayli", 0.20618556 },
  { "Raylie", 0.0 },
  { "Raylin", 0.27541268 },
  { "Raylina", 0.0 },
  { "Raylinn", 0.0 },
  { "Rayln", 0.0 },
  { "Raylnn", 0.0 },
  { "Raylon", 1.0 },
  { "Raylyn", 0.068741895 },
  { "Raylynn", 0.012537613 },
  { "Raylynne", 0.0 },
  { "Rayma", 0.0 },
  { "Raymah", 0.0 },
  { "Rayman", 1.0 },
  { "Raymand", 1.0 },
  { "Raymar", 1.0 },
  { "Raymarion", 1.0 },
  { "Raymart", 1.0 },
  { "Rayme", 0.29545453 },
  { "Raymeir", 1.0 },
  { "Raymel", 1.0 },
  { "Raymell", 0.47619048 },
  { "Raymelle", 0.0 },
  { "Raymen", 1.0 },
  { "Raymer", 1.0 },
  { "Raymere", 1.0 },
  { "Raymesha", 0.0 },
  { "Raymi", 0.0 },
  { "Raymiah", 0.0 },
  { "Raymie", 0.3902439 },
  { "Raymier", 1.0 },
  { "Raymir", 1.0 },
  { "Raymod", 1.0 },
  { "Raymon", 1.0 },
  { "Raymona", 0.0 },
  { "Raymond", 0.9954555 },
  { "Raymonda", 0.0 },
  { "Raymonde", 0.0 },
  { "Raymondo", 1.0 },
  { "Raymone", 1.0 },
  { "Raymoni", 0.0 },
  { "Raymont", 1.0 },
  { "Raymonte", 1.0 },
  { "Raymound", 1.0 },
  { "Raymund", 1.0 },
  { "Raymunda", 0.0 },
  { "Raymundo", 1.0 },
  { "Rayn", 0.5421822 },
  { "Rayna", 0.0 },
  { "Raynae", 0.0 },
  { "Raynah", 0.0 },
  { "Raynal", 1.0 },
  { "Raynald", 1.0 },
  { "Raynaldo", 1.0 },
  { "Raynard", 1.0 },
  { "Raynathan", 1.0 },
  { "Raynav", 1.0 },
  { "Rayne", 0.13796043 },
  { "Raynee", 0.0 },
  { "Rayneisha", 0.0 },
  { "Raynel", 0.8863636 },
  { "Raynell", 0.3738977 },
  { "Raynelle", 0.0 },
  { "Raynen", 1.0 },
  { "Rayner", 1.0 },
  { "Raynesha", 0.0 },
  { "Rayneshia", 0.0 },
  { "Raynetta", 0.0 },
  { "Raynette", 0.0 },
  { "Rayni", 0.0 },
  { "Raynia", 0.0 },
  { "Rayniah", 0.0 },
  { "Raynie", 0.0 },
  { "Rayniel", 1.0 },
  { "Raynier", 1.0 },
  { "Raynika", 0.0 },
  { "Raynique", 0.0 },
  { "Raynisha", 0.0 },
  { "Raynita", 0.0 },
  { "Rayniya", 0.0 },
  { "Rayniyah", 0.0 },
  { "Raynna", 0.0 },
  { "Raynne", 0.0 },
  { "Raynold", 1.0 },
  { "Raynoldo", 1.0 },
  { "Raynor", 1.0 },
  { "Rayo", 1.0 },
  { "Rayola", 0.0 },
  { "Rayon", 0.9751693 },
  { "Rayona", 0.0 },
  { "Rayonah", 0.0 },
  { "Rayonda", 0.0 },
  { "Rayonia", 0.0 },
  { "Rayonna", 0.0 },
  { "Rayquan", 1.0 },
  { "Rayquann", 1.0 },
  { "Rayquon", 1.0 },
  { "Rayqwan", 1.0 },
  { "Rayqwon", 1.0 },
  { "Raysa", 0.0 },
  { "Raysean", 1.0 },
  { "Raysen", 1.0 },
  { "Raysha", 0.0 },
  { "Rayshad", 1.0 },
  { "Rayshan", 1.0 },
  { "Rayshard", 1.0 },
  { "Rayshaud", 1.0 },
  { "Rayshaun", 1.0 },
  { "Rayshawn", 0.9527954 },
  { "Rayshawna", 0.0 },
  { "Rayshawnda", 0.0 },
  { "Rayshell", 0.0 },
  { "Rayshelle", 0.0 },
  { "Rayshod", 1.0 },
  { "Rayshon", 1.0 },
  { "Rayshonda", 0.0 },
  { "Rayshone", 1.0 },
  { "Rayshun", 1.0 },
  { "Rayson", 1.0 },
  { "Rayssa", 0.0 },
  { "Raytheon", 1.0 },
  { "Rayva", 0.0 },
  { "Rayvan", 0.0 },
  { "Rayvaughn", 1.0 },
  { "Rayven", 0.051055662 },
  { "Rayveon", 1.0 },
  { "Rayvin", 0.018237082 },
  { "Rayvion", 1.0 },
  { "Rayvn", 0.0 },
  { "Rayvon", 0.967827 },
  { "Rayvone", 1.0 },
  { "Rayvonn", 1.0 },
  { "Rayvyn", 0.0 },
  { "Raywood", 1.0 },
  { "Rayya", 0.0 },
  { "Rayyaan", 1.0 },
  { "Rayyan", 0.92959917 },
  { "Rayza", 0.0 },
  { "Raz", 1.0 },
  { "Raza", 1.0 },
  { "Razan", 0.0 },
  { "Razaria", 0.0 },
  { "Razariah", 0.0 },
  { "Razeen", 1.0 },
  { "Razel", 0.0 },
  { "Razhane", 0.0 },
  { "Razi", 1.0 },
  { "Razia", 0.0 },
  { "Raziah", 0.0 },
  { "Raziel", 0.9757576 },
  { "Raziela", 0.0 },
  { "Razin", 1.0 },
  { "Raziya", 0.0 },
  { "Raziyah", 0.0 },
  { "Razmig", 1.0 },
  { "Rb", 1.0 },
  { "Rbecca", 0.0 },
  { "Rc", 1.0 },
  { "Rchel", 0.0 },
  { "Rd", 1.0 },
  { "Rea", 0.101549976 },
  { "Reace", 0.6942149 },
  { "Reachel", 0.0 },
  { "Read", 1.0 },
  { "Reada", 0.0 },
  { "Reade", 1.0 },
  { "Readie", 0.0 },
  { "Readus", 1.0 },
  { "Reaford", 1.0 },
  { "Reagan", 0.1221564 },
  { "Reagann", 0.0 },
  { "Reaganne", 0.0 },
  { "Reagen", 0.11163522 },
  { "Reaghan", 0.0 },
  { "Reagin", 0.0 },
  { "Reagon", 0.0 },
  { "Reagyn", 0.0 },
  { "Reah", 0.0 },
  { "Reahanna", 0.0 },
  { "Reahna", 0.0 },
  { "Reaiah", 0.0 },
  { "Reaksmey", 1.0 },
  { "Reakwon", 1.0 },
  { "Real", 0.9285714 },
  { "Reality", 0.0 },
  { "Realyn", 0.0 },
  { "Realynn", 0.0 },
  { "Ream", 0.0 },
  { "Reamer", 1.0 },
  { "Rean", 0.0 },
  { "Reana", 0.0 },
  { "Reanda", 0.0 },
  { "Reanee", 0.0 },
  { "Reaner", 0.0 },
  { "Reanetta", 0.0 },
  { "Reann", 0.0 },
  { "Reanna", 0.0 },
  { "Reannah", 0.0 },
  { "Reannan", 0.0 },
  { "Reanne", 0.0 },
  { "Reannen", 0.0 },
  { "Reannon", 0.0 },
  { "Rease", 0.5633803 },
  { "Reasia", 0.0 },
  { "Reason", 0.9047619 },
  { "Reata", 0.0 },
  { "Reatha", 0.0 },
  { "Reathel", 0.04511278 },
  { "Reather", 0.005458515 },
  { "Reatta", 0.0 },
  { "Reauna", 0.0 },
  { "Reaven", 0.0 },
  { "Reaver", 0.0 },
  { "Reaves", 1.0 },
  { "Reavis", 1.0 },
  { "Reba", 0.0016996204 },
  { "Rebacca", 0.0 },
  { "Rebakah", 0.0 },
  { "Rebba", 0.0 },
  { "Rebbeca", 0.0 },
  { "Rebbecca", 0.0 },
  { "Rebbie", 0.0 },
  { "Rebcca", 0.0 },
  { "Rebe", 0.0 },
  { "Rebeca", 0.00039575747 },
  { "Rebecaa", 0.0 },
  { "Rebecah", 0.0 },
  { "Rebecc", 0.0 },
  { "Rebecca", 0.0028157786 },
  { "Rebeccaann", 0.0 },
  { "Rebeccah", 0.0 },
  { "Rebeccajo", 0.0 },
  { "Rebeccalynn", 0.0 },
  { "Rebecka", 0.0 },
  { "Rebeckah", 0.0 },
  { "Rebecker", 0.0 },
  { "Rebeka", 0.0 },
  { "Rebekah", 0.0016370665 },
  { "Rebekan", 0.0 },
  { "Rebekha", 0.0 },
  { "Rebekka", 0.0 },
  { "Rebekkah", 0.0 },
  { "Rebel", 0.42157614 },
  { "Rebella", 0.0 },
  { "Rebelle", 0.0 },
  { "Reber", 0.0 },
  { "Rebert", 1.0 },
  { "Reberta", 0.0 },
  { "Rebertha", 0.0 },
  { "Rebia", 0.0 },
  { "Rebie", 0.0 },
  { "Reble", 0.0 },
  { "Reby", 0.0 },
  { "Rebyl", 0.0 },
  { "Reca", 0.0 },
  { "Recardo", 1.0 },
  { "Recco", 1.0 },
  { "Rece", 0.9892704 },
  { "Rececca", 0.0 },
  { "Rechard", 1.0 },
  { "Reche", 1.0 },
  { "Rechel", 0.0 },
  { "Rechele", 0.0 },
  { "Rechell", 0.0 },
  { "Rechelle", 0.0 },
  { "Rechetta", 0.0 },
  { "Rechy", 0.0 },
  { "Recia", 0.0 },
  { "Recie", 0.032894738 },
  { "Recil", 1.0 },
  { "Recina", 0.0 },
  { "Reco", 1.0 },
  { "Record", 1.0 },
  { "Rector", 1.0 },
  { "Red", 1.0 },
  { "Reda", 0.120706104 },
  { "Redd", 1.0 },
  { "Redden", 1.0 },
  { "Reddick", 1.0 },
  { "Redding", 0.9609375 },
  { "Reddington", 1.0 },
  { "Redell", 0.7725857 },
  { "Redena", 0.0 },
  { "Rederick", 1.0 },
  { "Redford", 1.0 },
  { "Redge", 1.0 },
  { "Redgie", 1.0 },
  { "Redginald", 1.0 },
  { "Redia", 0.0 },
  { "Redick", 1.0 },
  { "Rediet", 0.0 },
  { "Redina", 0.0 },
  { "Redith", 0.0 },
  { "Redman", 1.0 },
  { "Redmon", 1.0 },
  { "Redmond", 1.0 },
  { "Redonda", 0.0 },
  { "Redonna", 0.0 },
  { "Redrick", 1.0 },
  { "Redus", 1.0 },
  { "Redwan", 1.0 },
  { "Ree", 0.017361112 },
  { "Reeana", 0.0 },
  { "Reeanna", 0.0 },
  { "Reeba", 0.0 },
  { "Reeca", 0.0 },
  { "Reece", 0.82574874 },
  { "Reecie", 0.07692308 },
  { "Reed", 0.97647214 },
  { "Reeda", 0.0 },
  { "Reeder", 1.0 },
  { "Reedie", 0.6666667 },
  { "Reedy", 1.0 },
  { "Reef", 0.984 },
  { "Reegan", 0.21502976 },
  { "Reeghan", 0.0 },
  { "Reeham", 0.0 },
  { "Reem", 0.0 },
  { "Reema", 0.0 },
  { "Reemas", 0.0 },
  { "Reena", 0.0 },
  { "Reene", 0.0 },
  { "Reenie", 0.0 },
  { "Rees", 0.88856304 },
  { "Reesa", 0.0 },
  { "Reese", 0.35592467 },
  { "Reeselyn", 0.0 },
  { "Reeselynn", 0.0 },
  { "Reesha", 0.0 },
  { "Reesheda", 0.0 },
  { "Reeshemah", 0.0 },
  { "Reesie", 0.0 },
  { "Reet", 0.0 },
  { "Reeta", 0.0 },
  { "Reetal", 0.0 },
  { "Reetha", 0.0 },
  { "Reeva", 0.0 },
  { "Reeve", 0.80867344 },
  { "Reeves", 0.9722628 },
  { "Reeya", 0.0 },
  { "Reeyan", 1.0 },
  { "Refa", 0.0 },
  { "Refael", 1.0 },
  { "Refoel", 1.0 },
  { "Reford", 1.0 },
  { "Refugia", 0.0038639877 },
  { "Refugio", 0.8193049 },
  { "Refujia", 0.0 },
  { "Refujio", 0.984985 },
  { "Reg", 1.0 },
  { "Regal", 1.0 },
  { "Regan", 0.18545882 },
  { "Regana", 0.0 },
  { "Regann", 0.0 },
  { "Reganne", 0.0 },
  { "Rege", 1.0 },
  { "Regeana", 0.0 },
  { "Regeina", 0.0 },
  { "Regen", 0.375 },
  { "Regena", 0.0 },
  { "Regenal", 1.0 },
  { "Regenald", 1.0 },
  { "Regene", 0.0 },
  { "Regenia", 0.0 },
  { "Regenna", 0.0 },
  { "Regetta", 0.0 },
  { "Reggan", 0.0 },
  { "Reggie", 0.96267974 },
  { "Reggina", 0.0 },
  { "Regginae", 0.0 },
  { "Regginal", 1.0 },
  { "Regginald", 1.0 },
  { "Reggy", 1.0 },
  { "Reghan", 0.0 },
  { "Regi", 1.0 },
  { "Regie", 1.0 },
  { "Regin", 0.0 },
  { "Regina", 0.003177089 },
  { "Reginae", 0.0 },
  { "Reginal", 0.99805677 },
  { "Reginald", 0.99314135 },
  { "Reginaldo", 1.0 },
  { "Reginamarie", 0.0 },
  { "Regine", 0.0 },
  { "Reginea", 0.0 },
  { "Reginia", 0.0 },
  { "Reginique", 0.0 },
  { "Reginna", 0.0 },
  { "Regino", 1.0 },
  { "Reginold", 1.0 },
  { "Regional", 1.0 },
  { "Regionald", 1.0 },
  { "Regis", 0.9370064 },
  { "Regla", 0.0 },
  { "Regna", 0.0 },
  { "Regnald", 1.0 },
  { "Regnia", 0.0 },
  { "Regniald", 1.0 },
  { "Regnold", 1.0 },
  { "Regory", 1.0 },
  { "Regulo", 1.0 },
  { "Reha", 0.0 },
  { "Rehaan", 1.0 },
  { "Rehab", 0.0 },
  { "Reham", 0.0 },
  { "Rehan", 0.9944082 },
  { "Rehana", 0.0 },
  { "Rehanna", 0.0 },
  { "Rehansh", 1.0 },
  { "Rehema", 0.0 },
  { "Rehgan", 0.0 },
  { "Rehma", 0.0 },
  { "Rehman", 1.0 },
  { "Rehmat", 0.0 },
  { "Rei", 0.48809522 },
  { "Reia", 0.0 },
  { "Reiana", 0.0 },
  { "Reianna", 0.0 },
  { "Reice", 0.9411765 },
  { "Reichen", 1.0 },
  { "Reicher", 1.0 },
  { "Reico", 1.0 },
  { "Reid", 0.9788214 },
  { "Reida", 0.0 },
  { "Reidar", 1.0 },
  { "Reide", 1.0 },
  { "Reiden", 1.0 },
  { "Reider", 1.0 },
  { "Reif", 1.0 },
  { "Reigan", 0.0 },
  { "Reigha", 0.0 },
  { "Reighan", 0.0 },
  { "Reighlyn", 0.0 },
  { "Reighlynn", 0.0 },
  { "Reighn", 0.10869565 },
  { "Reighna", 0.0 },
  { "Reign", 0.33302063 },
  { "Reigna", 0.0 },
  { "Reik", 1.0 },
  { "Reika", 0.0 },
  { "Reiken", 1.0 },
  { "Reiko", 0.018867925 },
  { "Reilani", 0.0 },
  { "Reilee", 0.0 },
  { "Reileigh", 0.0 },
  { "Reiley", 0.35656214 },
  { "Reili", 0.54545456 },
  { "Reilley", 0.3728223 },
  { "Reilly", 0.45078298 },
  { "Reily", 0.608284 },
  { "Reilyn", 0.0 },
  { "Reilynn", 0.0 },
  { "Reimi", 0.0 },
  { "Reimundo", 1.0 },
  { "Rein", 0.6666667 },
  { "Reina", 0.0 },
  { "Reinald", 1.0 },
  { "Reinalda", 0.0 },
  { "Reinaldo", 1.0 },
  { "Reinaliz", 0.0 },
  { "Reinard", 1.0 },
  { "Reinardo", 1.0 },
  { "Reine", 0.0 },
  { "Reiner", 1.0 },
  { "Reinette", 0.0 },
  { "Reinhard", 1.0 },
  { "Reinhardt", 1.0 },
  { "Reinhart", 1.0 },
  { "Reinhold", 1.0 },
  { "Reinholdt", 1.0 },
  { "Reinier", 1.0 },
  { "Reinna", 0.0 },
  { "Reino", 1.0 },
  { "Reinold", 1.0 },
  { "Reiny", 0.0 },
  { "Reion", 1.0 },
  { "Reionna", 0.0 },
  { "Reis", 0.8670968 },
  { "Reisa", 0.0 },
  { "Reise", 0.55454546 },
  { "Reisha", 0.0 },
  { "Reiss", 0.76328504 },
  { "Reita", 0.0 },
  { "Reiter", 1.0 },
  { "Reitha", 0.0 },
  { "Reiya", 0.0 },
  { "Reizel", 0.0 },
  { "Reizy", 0.0 },
  { "Reja", 0.0 },
  { "Rejanae", 0.0 },
  { "Rejane", 0.0 },
  { "Rejean", 0.6923077 },
  { "Rejeana", 0.0 },
  { "Rejeania", 0.0 },
  { "Rejeanne", 0.0 },
  { "Rejena", 0.0 },
  { "Rejetta", 0.0 },
  { "Rejina", 0.0 },
  { "Rejine", 0.0 },
  { "Rejino", 1.0 },
  { "Rejoice", 0.0 },
  { "Rejon", 1.0 },
  { "Rejoyce", 0.0 },
  { "Reka", 0.0 },
  { "Rekayla", 0.0 },
  { "Rekeisha", 0.0 },
  { "Rekesha", 0.0 },
  { "Rekha", 0.0 },
  { "Rekia", 0.0 },
  { "Rekik", 0.0 },
  { "Rekina", 0.0 },
  { "Rekisha", 0.0 },
  { "Rekita", 0.0 },
  { "Rekiya", 0.0 },
  { "Rekiyah", 0.0 },
  { "Rekker", 1.0 },
  { "Reko", 1.0 },
  { "Rela", 0.0 },
  { "Relanda", 0.0 },
  { "Relando", 1.0 },
  { "Relda", 0.0 },
  { "Reldon", 1.0 },
  { "Relena", 0.0 },
  { "Relford", 1.0 },
  { "Relia", 0.0 },
  { "Relic", 1.0 },
  { "Relina", 0.0 },
  { "Relinda", 0.0 },
  { "Relis", 1.0 },
  { "Rell", 0.7058824 },
  { "Rella", 0.0 },
  { "Relma", 0.0 },
  { "Relonda", 0.0 },
  { "Relynn", 0.0 },
  { "Rem", 1.0 },
  { "Rema", 0.008741259 },
  { "Remas", 0.0 },
  { "Rember", 1.0 },
  { "Rembert", 1.0 },
  { "Remberto", 1.0 },
  { "Rembrandt", 1.0 },
  { "Reme", 1.0 },
  { "Remedi", 0.0 },
  { "Remedios", 0.0 },
  { "Remedy", 0.02173913 },
  { "Remee", 0.0 },
  { "Remeigh", 0.0 },
  { "Remeka", 0.0 },
  { "Remel", 0.0 },
  { "Remelda", 0.0 },
  { "Remell", 0.0 },
  { "Remer", 1.0 },
  { "Remey", 0.5 },
  { "Remi", 0.2440585 },
  { "Remia", 0.0 },
  { "Remiah", 0.0 },
  { "Remick", 1.0 },
  { "Remie", 0.018939395 },
  { "Remiel", 1.0 },
  { "Remigia", 0.0 },
  { "Remigio", 1.0 },
  { "Remigius", 1.0 },
  { "Remii", 0.0 },
  { "Remijio", 1.0 },
  { "Remika", 0.0 },
  { "Remilda", 0.0 },
  { "Remilynn", 0.0 },
  { "Remingtin", 1.0 },
  { "Remington", 0.77644587 },
  { "Remingtyn", 0.0 },
  { "Remini", 0.0 },
  { "Reminisce", 0.0 },
  { "Remiyah", 0.0 },
  { "Remley", 0.0 },
  { "Remmel", 1.0 },
  { "Remmi", 0.03448276 },
  { "Remmie", 0.0 },
  { "Remmington", 0.6757576 },
  { "Remmy", 0.6424242 },
  { "Remo", 1.0 },
  { "Remon", 1.0 },
  { "Remona", 0.0 },
  { "Remond", 1.0 },
  { "Remonda", 0.0 },
  { "Remone", 1.0 },
  { "Remonia", 0.0 },
  { "Remonica", 0.0 },
  { "Remsen", 1.0 },
  { "Remus", 1.0 },
  { "Remy", 0.5883148 },
  { "Remya", 0.0 },
  { "Remyngton", 0.44444445 },
  { "Ren", 0.87149817 },
  { "Rena", 0.0031930634 },
  { "Renad", 0.0 },
  { "Renada", 0.0 },
  { "Renado", 1.0 },
  { "Renae", 0.0023972106 },
  { "Renaee", 0.0 },
  { "Renah", 0.0 },
  { "Renai", 0.0 },
  { "Renald", 1.0 },
  { "Renalda", 0.0 },
  { "Renaldo", 1.0 },
  { "Renan", 1.0 },
  { "Renard", 1.0 },
  { "Renarda", 0.034246575 },
  { "Renardo", 0.98463506 },
  { "Renarta", 0.0 },
  { "Renasha", 0.0 },
  { "Renasia", 0.0 },
  { "Renat", 1.0 },
  { "Renata", 0.00066986715 },
  { "Renate", 0.0 },
  { "Renatha", 0.0 },
  { "Renato", 0.99793744 },
  { "Renatta", 0.0 },
  { "Renaud", 1.0 },
  { "Renauda", 0.0 },
  { "Renauld", 1.0 },
  { "Renauldo", 1.0 },
  { "Renault", 1.0 },
  { "Renauta", 0.0 },
  { "Renay", 0.012244898 },
  { "Renaya", 0.0 },
  { "Renaye", 0.0 },
  { "Renda", 0.0 },
  { "Rendall", 1.0 },
  { "Rendell", 1.0 },
  { "Render", 1.0 },
  { "Rendi", 0.0 },
  { "Rendon", 1.0 },
  { "Rendy", 0.10759494 },
  { "Rene", 0.69268256 },
  { "Renea", 0.0 },
  { "Reneasha", 0.0 },
  { "Renecia", 0.0 },
  { "Reneda", 0.0 },
  { "Renee", 0.008596685 },
  { "Renegade", 1.0 },
  { "Reneisha", 0.0 },
  { "Reneka", 0.0 },
  { "Renel", 0.7948718 },
  { "Renelda", 0.0 },
  { "Renell", 0.21782178 },
  { "Renella", 0.0 },
  { "Renelle", 0.0 },
  { "Rener", 0.0 },
  { "Renesha", 0.0 },
  { "Reneshia", 0.0 },
  { "Renesmae", 0.0 },
  { "Renesmay", 0.0 },
  { "Renesme", 0.0 },
  { "Renesmee", 0.0 },
  { "Renessa", 0.0 },
  { "Reneta", 0.0 },
  { "Renetha", 0.0 },
  { "Renetia", 0.0 },
  { "Renetta", 0.0 },
  { "Renette", 0.0 },
  { "Reney", 1.0 },
  { "Renezmae", 0.0 },
  { "Renezmay", 0.0 },
  { "Renezme", 0.0 },
  { "Renezmee", 0.0 },
  { "Renford", 1.0 },
  { "Renfred", 1.0 },
  { "Reni", 0.0 },
  { "Renia", 0.0 },
  { "Reniah", 0.0 },
  { "Renica", 0.0 },
  { "Renice", 0.0 },
  { "Renick", 1.0 },
  { "Renida", 0.0 },
  { "Renie", 0.009677419 },
  { "Reniece", 0.0 },
  { "Reniel", 1.0 },
  { "Renier", 1.0 },
  { "Reniesha", 0.0 },
  { "Renika", 0.0 },
  { "Renina", 0.0 },
  { "Reniqua", 0.0 },
  { "Renise", 0.0 },
  { "Renisha", 0.0 },
  { "Renita", 0.0 },
  { "Renitha", 0.0 },
  { "Renitia", 0.0 },
  { "Renitta", 0.0 },
  { "Reniya", 0.0 },
  { "Reniyah", 0.0 },
  { "Renji", 1.0 },
  { "Renlee", 0.0 },
  { "Renleigh", 0.0 },
  { "Renley", 0.36065573 },
  { "Renly", 0.795082 },
  { "Renn", 0.8442029 },
  { "Renna", 0.0 },
  { "Rennae", 0.0 },
  { "Rennan", 1.0 },
  { "Rennard", 1.0 },
  { "Rennata", 0.0 },
  { "Renne", 0.09621451 },
  { "Rennee", 0.0 },
  { "Rennen", 1.0 },
  { "Renner", 0.9476744 },
  { "Rennetta", 0.0 },
  { "Rennette", 0.0 },
  { "Rennick", 1.0 },
  { "Rennie", 0.52944326 },
  { "Rennis", 1.0 },
  { "Renny", 0.90279824 },
  { "Reno", 0.99440914 },
  { "Renoda", 0.0 },
  { "Renold", 1.0 },
  { "Renona", 0.0 },
  { "Renorda", 0.0 },
  { "Renota", 0.0 },
  { "Renotta", 0.0 },
  { "Renso", 1.0 },
  { "Renton", 1.0 },
  { "Renu", 0.0 },
  { "Renuka", 0.0 },
  { "Renwick", 1.0 },
  { "Reny", 0.64705884 },
  { "Renya", 0.0 },
  { "Renyah", 0.0 },
  { "Renz", 1.0 },
  { "Renza", 1.0 },
  { "Renzel", 1.0 },
  { "Renzie", 1.0 },
  { "Renzo", 1.0 },
  { "Reo", 0.963807 },
  { "Reola", 0.0 },
  { "Reon", 1.0 },
  { "Reona", 0.0 },
  { "Reonna", 0.0 },
  { "Rephael", 1.0 },
  { "Reppard", 1.0 },
  { "Requan", 1.0 },
  { "Requel", 0.0 },
  { "Requita", 0.0 },
  { "Resa", 0.0 },
  { "Resean", 1.0 },
  { "Resha", 0.0 },
  { "Reshad", 1.0 },
  { "Reshae", 0.0 },
  { "Resham", 0.0 },
  { "Reshana", 0.0 },
  { "Reshanda", 0.0 },
  { "Reshard", 1.0 },
  { "Reshaud", 1.0 },
  { "Reshaun", 1.0 },
  { "Reshaunda", 0.0 },
  { "Reshawn", 0.9052864 },
  { "Reshawna", 0.0 },
  { "Reshawnda", 0.0 },
  { "Reshay", 0.0 },
  { "Reshea", 0.0 },
  { "Resheda", 0.0 },
  { "Resheena", 0.0 },
  { "Resheka", 0.0 },
  { "Reshell", 0.0 },
  { "Reshelle", 0.0 },
  { "Reshia", 0.0 },
  { "Reshika", 0.0 },
  { "Reshma", 0.0 },
  { "Reshmi", 0.0 },
  { "Reshod", 1.0 },
  { "Reshon", 0.7826087 },
  { "Reshonda", 0.0 },
  { "Reshunda", 0.0 },
  { "Resia", 0.0 },
  { "Ressa", 0.0 },
  { "Ressie", 0.009777295 },
  { "Reston", 1.0 },
  { "Reta", 0.0 },
  { "Retag", 0.0 },
  { "Retage", 0.0 },
  { "Retaj", 0.0 },
  { "Retal", 0.0 },
  { "Retha", 0.0 },
  { "Rethal", 0.0 },
  { "Rethea", 0.0 },
  { "Rethel", 0.0 },
  { "Rether", 0.0 },
  { "Rethia", 0.0 },
  { "Retia", 0.0 },
  { "Retina", 0.0 },
  { "Retisha", 0.0 },
  { "Retonia", 0.0 },
  { "Retonya", 0.0 },
  { "Rett", 1.0 },
  { "Retta", 0.0 },
  { "Rettie", 0.0 },
  { "Reuben", 0.99848455 },
  { "Reubena", 0.0 },
  { "Reubin", 1.0 },
  { "Reuel", 1.0 },
  { "Reus", 1.0 },
  { "Reut", 0.0 },
  { "Reuven", 1.0 },
  { "Rev", 1.0 },
  { "Reva", 0.000711928 },
  { "Revaan", 1.0 },
  { "Revae", 0.0 },
  { "Revan", 0.97651005 },
  { "Revansh", 1.0 },
  { "Revanth", 1.0 },
  { "Revathi", 0.0 },
  { "Revati", 0.0 },
  { "Reve", 0.24050634 },
  { "Revea", 0.0 },
  { "Reveca", 0.0 },
  { "Revecca", 0.0 },
  { "Reveille", 0.0 },
  { "Revel", 0.8669951 },
  { "Revelation", 1.0 },
  { "Revell", 1.0 },
  { "Revella", 0.0 },
  { "Revelle", 1.0 },
  { "Reven", 0.875 },
  { "Revena", 0.0 },
  { "Rever", 0.0 },
  { "Revere", 1.0 },
  { "Reverie", 0.0 },
  { "Revia", 0.0 },
  { "Revie", 0.0 },
  { "Revin", 1.0 },
  { "Revina", 0.0 },
  { "Revis", 1.0 },
  { "Revon", 0.8148148 },
  { "Revonda", 0.0 },
  { "Revyn", 0.46666667 },
  { "Rewa", 0.0 },
  { "Rex", 0.99755216 },
  { "Rexall", 1.0 },
  { "Rexalynn", 0.0 },
  { "Rexann", 0.0 },
  { "Rexanna", 0.0 },
  { "Rexanne", 0.0 },
  { "Rexene", 0.0 },
  { "Rexford", 1.0 },
  { "Rexie", 0.45714286 },
  { "Rexine", 0.0 },
  { "Rexton", 1.0 },
  { "Rexx", 1.0 },
  { "Rey", 0.99106765 },
  { "Reya", 0.0 },
  { "Reyaan", 1.0 },
  { "Reyaansh", 1.0 },
  { "Reyad", 1.0 },
  { "Reyah", 0.0 },
  { "Reyan", 0.95535713 },
  { "Reyana", 0.0 },
  { "Reyann", 0.0 },
  { "Reyanna", 0.0 },
  { "Reyanne", 0.0 },
  { "Reyansh", 1.0 },
  { "Reyanshi", 0.0 },
  { "Reyburn", 1.0 },
  { "Reyce", 0.8518519 },
  { "Reyd", 1.0 },
  { "Reydan", 1.0 },
  { "Reydavid", 1.0 },
  { "Reyden", 1.0 },
  { "Reydon", 1.0 },
  { "Reyelle", 0.0 },
  { "Reyes", 0.8860577 },
  { "Reyez", 1.0 },
  { "Reygan", 0.0 },
  { "Reyhan", 0.8064516 },
  { "Reyhana", 0.0 },
  { "Reylan", 1.0 },
  { "Reyland", 1.0 },
  { "Reylee", 0.0 },
  { "Reyleigh", 0.0 },
  { "Reylen", 1.0 },
  { "Reylene", 0.0 },
  { "Reyli", 0.9826203 },
  { "Reylin", 0.17857143 },
  { "Reyly", 1.0 },
  { "Reylyn", 0.0 },
  { "Reylynn", 0.0 },
  { "Reymi", 1.0 },
  { "Reymon", 1.0 },
  { "Reymond", 1.0 },
  { "Reymundo", 1.0 },
  { "Reyn", 0.948 },
  { "Reyna", 0.0022667705 },
  { "Reynah", 0.0 },
  { "Reynald", 1.0 },
  { "Reynalda", 0.0 },
  { "Reynaldo", 0.9988195 },
  { "Reynaliz", 0.0 },
  { "Reynard", 1.0 },
  { "Reynardo", 1.0 },
  { "Reynaud", 1.0 },
  { "Reyne", 0.0 },
  { "Reynel", 1.0 },
  { "Reynelda", 0.0 },
  { "Reynell", 0.0 },
  { "Reyner", 1.0 },
  { "Reyniel", 1.0 },
  { "Reynier", 1.0 },
  { "Reynol", 1.0 },
  { "Reynold", 1.0 },
  { "Reynoldo", 1.0 },
  { "Reynolds", 0.9750799 },
  { "Reyon", 1.0 },
  { "Reyona", 0.0 },
  { "Reyonna", 0.0 },
  { "Reyse", 0.0 },
  { "Reyshawn", 1.0 },
  { "Reyyan", 0.0 },
  { "Reza", 1.0 },
  { "Reznor", 1.0 },
  { "Rezon", 1.0 },
  { "Rhae", 0.0 },
  { "Rhaegan", 0.0 },
  { "Rhaelyn", 0.0 },
  { "Rhaelynn", 0.0 },
  { "Rhaheem", 1.0 },
  { "Rhakeem", 1.0 },
  { "Rhamel", 1.0 },
  { "Rhamir", 1.0 },
  { "Rhana", 0.0 },
  { "Rhanada", 0.0 },
  { "Rhanda", 0.0 },
  { "Rhandi", 0.0 },
  { "Rhandy", 1.0 },
  { "Rhania", 0.0 },
  { "Rhaniya", 0.0 },
  { "Rhaniyah", 0.0 },
  { "Rhapsody", 0.0 },
  { "Rhasaan", 1.0 },
  { "Rhashad", 1.0 },
  { "Rhasheen", 1.0 },
  { "Rhaven", 0.0 },
  { "Rhavyn", 0.0 },
  { "Rhawnie", 0.0 },
  { "Rhaya", 0.0 },
  { "Rhayna", 0.0 },
  { "Rhayne", 0.0 },
  { "Rhea", 0.02536998 },
  { "Rheaann", 0.0 },
  { "Rheagan", 0.0 },
  { "Rhealyn", 0.0 },
  { "Rhealynn", 0.0 },
  { "Rheana", 0.0 },
  { "Rheann", 0.0 },
  { "Rheanna", 0.0 },
  { "Rheanne", 0.0 },
  { "Rheannon", 0.0 },
  { "Rheanon", 0.0 },
  { "Rheata", 0.0 },
  { "Rheba", 0.0 },
  { "Rheda", 0.0 },
  { "Rhedyn", 0.0 },
  { "Rheece", 1.0 },
  { "Rheese", 0.0 },
  { "Rheeta", 0.0 },
  { "Rhegan", 0.0 },
  { "Rheia", 0.0 },
  { "Rhelda", 0.0 },
  { "Rhema", 0.0 },
  { "Rhemi", 0.064935066 },
  { "Rhemy", 0.15625 },
  { "Rhen", 0.76744187 },
  { "Rhena", 0.0 },
  { "Rhenda", 0.0 },
  { "Rheo", 1.0 },
  { "Rhesa", 0.0 },
  { "Rhet", 1.0 },
  { "Rheta", 0.0 },
  { "Rhetha", 0.0 },
  { "Rhett", 0.9943147 },
  { "Rhetta", 0.0 },
  { "Rheuben", 1.0 },
  { "Rheva", 0.0 },
  { "Rheya", 0.0 },
  { "Rheyanna", 0.0 },
  { "Rheyna", 0.0 },
  { "Rhia", 0.0 },
  { "Rhian", 0.12085308 },
  { "Rhiana", 0.0 },
  { "Rhianah", 0.0 },
  { "Rhiane", 0.0 },
  { "Rhiann", 0.0 },
  { "Rhianna", 0.0 },
  { "Rhiannah", 0.0 },
  { "Rhiannan", 0.0 },
  { "Rhianne", 0.0 },
  { "Rhiannen", 0.0 },
  { "Rhiannon", 0.00036258157 },
  { "Rhianon", 0.0 },
  { "Rhiauna", 0.0 },
  { "Rhilee", 0.0 },
  { "Rhileigh", 0.0 },
  { "Rhiley", 0.26996624 },
  { "Rhilyn", 0.0 },
  { "Rhilynn", 0.0 },
  { "Rhina", 0.0 },
  { "Rhinda", 0.0 },
  { "Rhion", 1.0 },
  { "Rhionna", 0.0 },
  { "Rhiver", 1.0 },
  { "Rhiya", 0.0 },
  { "Rhiyan", 0.0 },
  { "Rhiyana", 0.0 },
  { "Rhiyanna", 0.0 },
  { "Rhnea", 0.0 },
  { "Rhoads", 1.0 },
  { "Rhoan", 1.0 },
  { "Rhochelle", 0.0 },
  { "Rhoda", 0.0002735764 },
  { "Rhode", 0.0 },
  { "Rhodell", 1.0 },
  { "Rhodella", 0.0 },
  { "Rhodena", 0.0 },
  { "Rhoderick", 1.0 },
  { "Rhodes", 0.97837836 },
  { "Rhodesia", 0.0 },
  { "Rhodia", 0.0 },
  { "Rhodie", 0.035971224 },
  { "Rhodney", 1.0 },
  { "Rhodonna", 0.0 },
  { "Rhodora", 0.0 },
  { "Rhody", 1.0 },
  { "Rhoen", 1.0 },
  { "Rhoena", 0.0 },
  { "Rhoman", 1.0 },
  { "Rhome", 1.0 },
  { "Rhon", 1.0 },
  { "Rhona", 0.0 },
  { "Rhonald", 1.0 },
  { "Rhonan", 1.0 },
  { "Rhond", 0.0 },
  { "Rhonda", 0.002829641 },
  { "Rhondalyn", 0.0 },
  { "Rhondda", 0.0 },
  { "Rhondi", 0.0 },
  { "Rhondia", 0.0 },
  { "Rhone", 0.9430894 },
  { "Rhonesha", 0.0 },
  { "Rhoni", 0.0 },
  { "Rhonin", 1.0 },
  { "Rhonisha", 0.0 },
  { "Rhonna", 0.0 },
  { "Rhonnie", 1.0 },
  { "Rhory", 1.0 },
  { "Rhoslyn", 0.0 },
  { "Rhowen", 1.0 },
  { "Rhowyn", 0.0 },
  { "Rhuben", 1.0 },
  { "Rhudine", 0.0 },
  { "Rhunette", 0.0 },
  { "Rhya", 0.0 },
  { "Rhyan", 0.37827381 },
  { "Rhyana", 0.0 },
  { "Rhyane", 0.0 },
  { "Rhyann", 0.0 },
  { "Rhyanna", 0.0 },
  { "Rhyanne", 0.0 },
  { "Rhyannon", 0.0 },
  { "Rhyatt", 1.0 },
  { "Rhyden", 1.0 },
  { "Rhyder", 0.9133758 },
  { "Rhydian", 1.0 },
  { "Rhye", 1.0 },
  { "Rhyen", 0.2682927 },
  { "Rhyett", 1.0 },
  { "Rhyin", 0.0 },
  { "Rhyis", 1.0 },
  { "Rhyker", 1.0 },
  { "Rhyla", 0.0 },
  { "Rhylan", 0.6416819 },
  { "Rhyland", 0.9 },
  { "Rhylea", 0.0 },
  { "Rhylee", 0.11764706 },
  { "Rhylei", 0.0 },
  { "Rhyleigh", 0.0 },
  { "Rhylen", 0.68 },
  { "Rhyley", 0.375 },
  { "Rhyli", 0.0 },
  { "Rhylie", 0.0 },
  { "Rhylin", 0.32467532 },
  { "Rhylinn", 0.0 },
  { "Rhylon", 1.0 },
  { "Rhylynn", 0.0 },
  { "Rhyne", 1.0 },
  { "Rhyon", 0.7586207 },
  { "Rhys", 0.93877804 },
  { "Rhyse", 0.6946903 },
  { "Rhyson", 1.0 },
  { "Rhythm", 0.55949366 },
  { "Rhyus", 1.0 },
  { "Ria", 0.0 },
  { "Riaan", 1.0 },
  { "Riad", 1.0 },
  { "Riah", 0.014164306 },
  { "Riahanna", 0.0 },
  { "Riahna", 0.0 },
  { "Riahnna", 0.0 },
  { "Rial", 1.0 },
  { "Rialey", 0.0 },
  { "Rian", 0.5983029 },
  { "Riana", 0.0 },
  { "Rianah", 0.0 },
  { "Riane", 0.009451796 },
  { "Riann", 0.0 },
  { "Rianna", 0.0 },
  { "Riannah", 0.0 },
  { "Rianne", 0.0 },
  { "Riannon", 0.0 },
  { "Rianny", 0.0 },
  { "Rianon", 0.0 },
  { "Riansh", 1.0 },
  { "Rianshi", 0.0 },
  { "Rias", 0.0 },
  { "Riata", 0.0 },
  { "Riayn", 0.0 },
  { "Riaz", 1.0 },
  { "Ribaldo", 1.0 },
  { "Ribhi", 1.0 },
  { "Ric", 1.0 },
  { "Rica", 0.0 },
  { "Ricado", 1.0 },
  { "Ricahrd", 1.0 },
  { "Ricard", 1.0 },
  { "Ricarda", 0.0 },
  { "Ricardo", 0.99484736 },
  { "Ricari", 1.0 },
  { "Ricaria", 0.0 },
  { "Ricayla", 0.0 },
  { "Ricca", 0.0 },
  { "Riccardo", 1.0 },
  { "Ricci", 0.5099299 },
  { "Ricco", 1.0 },
  { "Rice", 1.0 },
  { "Rich", 1.0 },
  { "Richa", 0.0 },
  { "Richaad", 1.0 },
  { "Richad", 1.0 },
  { "Richae", 0.0 },
  { "Richael", 0.0 },
  { "Richana", 0.0 },
  { "Richand", 1.0 },
  { "Richanda", 0.0 },
  { "Richar", 1.0 },
  { "Richara", 0.0 },
  { "Richard", 0.9962911 },
  { "Richarda", 0.0 },
  { "Richardd", 1.0 },
  { "Richardean", 0.0 },
  { "Richardine", 0.0 },
  { "Richardjames", 1.0 },
  { "Richardo", 1.0 },
  { "Richards", 1.0 },
  { "Richardson", 1.0 },
  { "Richaud", 1.0 },
  { "Richel", 0.0 },
  { "Richele", 0.0 },
  { "Richell", 0.0 },
  { "Richella", 0.0 },
  { "Richelle", 0.0 },
  { "Richenda", 0.0 },
  { "Richer", 1.0 },
  { "Richerd", 1.0 },
  { "Richetta", 0.0 },
  { "Richette", 0.0 },
  { "Richey", 1.0 },
  { "Richi", 1.0 },
  { "Richia", 0.0 },
  { "Richie", 0.9419495 },
  { "Richlyn", 0.0 },
  { "Richlynn", 0.0 },
  { "Richman", 1.0 },
  { "Richmon", 1.0 },
  { "Richmond", 1.0 },
  { "Richon", 1.0 },
  { "Richrd", 1.0 },
  { "Richter", 1.0 },
  { "Richy", 1.0 },
  { "Rici", 0.0 },
  { "Ricia", 0.0 },
  { "Rick", 0.99834436 },
  { "Ricka", 0.0 },
  { "Rickard", 1.0 },
  { "Rickardo", 1.0 },
  { "Rickayla", 0.0 },
  { "Ricke", 1.0 },
  { "Rickea", 0.0 },
  { "Rickeem", 1.0 },
  { "Rickeeta", 0.0 },
  { "Rickeisha", 0.0 },
  { "Rickel", 0.0 },
  { "Rickell", 0.0 },
  { "Rickelle", 0.0 },
  { "Rickesha", 0.0 },
  { "Rickeshia", 0.0 },
  { "Ricketa", 0.0 },
  { "Ricketta", 0.0 },
  { "Rickey", 0.9891006 },
  { "Rickeya", 0.0 },
  { "Rickford", 1.0 },
  { "Ricki", 0.35349628 },
  { "Rickia", 0.0 },
  { "Rickiah", 0.0 },
  { "Rickie", 0.90151894 },
  { "Rickiesha", 0.0 },
  { "Rickira", 0.0 },
  { "Rickisha", 0.0 },
  { "Rickita", 0.0 },
  { "Rickiya", 0.0 },
  { "Rickiyah", 0.0 },
  { "Rickki", 0.0 },
  { "Rickman", 1.0 },
  { "Ricko", 1.0 },
  { "Ricks", 1.0 },
  { "Rickson", 1.0 },
  { "Ricky", 0.9916292 },
  { "Rickya", 0.0 },
  { "Rickyah", 0.0 },
  { "Rickye", 1.0 },
  { "Rickyia", 0.0 },
  { "Rickyy", 1.0 },
  { "Rico", 0.99494153 },
  { "Ricoh", 1.0 },
  { "Ricole", 0.0 },
  { "Ricquan", 1.0 },
  { "Ricquel", 0.0 },
  { "Ricquita", 0.0 },
  { "Rictavious", 1.0 },
  { "Rictoria", 0.0 },
  { "Rida", 0.06451613 },
  { "Ridaan", 1.0 },
  { "Ridah", 0.0 },
  { "Riddhi", 0.0 },
  { "Riddhima", 0.0 },
  { "Riddhish", 1.0 },
  { "Riddic", 1.0 },
  { "Riddick", 1.0 },
  { "Riddik", 1.0 },
  { "Rider", 1.0 },
  { "Ridge", 1.0 },
  { "Ridgely", 0.9324324 },
  { "Ridger", 1.0 },
  { "Ridgeway", 1.0 },
  { "Ridgley", 1.0 },
  { "Ridgway", 1.0 },
  { "Ridha", 0.31944445 },
  { "Ridhaan", 1.0 },
  { "Ridhi", 0.0 },
  { "Ridhima", 0.0 },
  { "Ridhwan", 1.0 },
  { "Ridlee", 0.0 },
  { "Ridleigh", 0.0 },
  { "Ridley", 0.65207195 },
  { "Ridwaan", 1.0 },
  { "Ridwan", 0.95185995 },
  { "Rie", 0.0 },
  { "Rieanna", 0.0 },
  { "Riece", 0.67741936 },
  { "Rieker", 1.0 },
  { "Riel", 0.7692308 },
  { "Rielee", 0.0 },
  { "Rieleigh", 0.0 },
  { "Rieley", 0.3535032 },
  { "Riella", 0.0 },
  { "Rielle", 0.0 },
  { "Rielley", 0.0 },
  { "Rielly", 0.44044945 },
  { "Riely", 0.4930362 },
  { "Rielyn", 0.026595745 },
  { "Rielynn", 0.0 },
  { "Rien", 0.768 },
  { "Riena", 0.0 },
  { "Rienhold", 1.0 },
  { "Rienna", 0.0 },
  { "Ries", 1.0 },
  { "Riesa", 0.0 },
  { "Riese", 0.627451 },
  { "Riesha", 0.0 },
  { "Rieta", 0.0 },
  { "Rietta", 0.0 },
  { "Rifa", 0.0 },
  { "Rifat", 1.0 },
  { "Rifka", 0.0 },
  { "Rifky", 0.0 },
  { "Rigby", 0.8064516 },
  { "Rigden", 1.0 },
  { "Rigdon", 1.0 },
  { "Rigel", 0.9774775 },
  { "Rigg", 1.0 },
  { "Riggan", 1.0 },
  { "Riggen", 1.0 },
  { "Riggin", 1.0 },
  { "Riggins", 0.9798387 },
  { "Riggs", 1.0 },
  { "Righley", 0.0 },
  { "Righteous", 0.9122807 },
  { "Rigina", 0.0 },
  { "Rigley", 0.7425743 },
  { "Rigo", 1.0 },
  { "Rigoberto", 0.99934715 },
  { "Rigoverto", 1.0 },
  { "Rigsby", 1.0 },
  { "Rigved", 1.0 },
  { "Riha", 0.0 },
  { "Rihaan", 1.0 },
  { "Rihab", 0.0 },
  { "Riham", 0.0 },
  { "Rihan", 0.65789473 },
  { "Rihana", 0.0 },
  { "Rihanna", 0.00093040563 },
  { "Rihannah", 0.0 },
  { "Rihansh", 1.0 },
  { "Rihcard", 1.0 },
  { "Riho", 0.0 },
  { "Rihonna", 0.0 },
  { "Riis", 1.0 },
  { "Rija", 0.0 },
  { "Rijad", 1.0 },
  { "Rijja", 0.0 },
  { "Rijul", 1.0 },
  { "Rik", 1.0 },
  { "Rika", 0.0 },
  { "Rikako", 0.0 },
  { "Rikardo", 1.0 },
  { "Rikayla", 0.0 },
  { "Rikeisha", 0.0 },
  { "Rikelme", 1.0 },
  { "Riken", 1.0 },
  { "Riker", 1.0 },
  { "Rikesh", 1.0 },
  { "Rikesha", 0.0 },
  { "Riketa", 0.0 },
  { "Rikhia", 0.0 },
  { "Rikhil", 1.0 },
  { "Riki", 0.27559564 },
  { "Rikia", 0.0 },
  { "Rikiah", 0.0 },
  { "Rikin", 1.0 },
  { "Rikio", 1.0 },
  { "Rikisha", 0.0 },
  { "Rikishi", 0.625 },
  { "Rikita", 0.0 },
  { "Rikiya", 0.0 },
  { "Rikiyah", 0.0 },
  { "Rikka", 0.0 },
  { "Rikke", 0.0 },
  { "Rikki", 0.08595359 },
  { "Rikkia", 0.0 },
  { "Rikkie", 0.0 },
  { "Rikkilee", 0.0 },
  { "Rikkilynn", 0.0 },
  { "Rikkita", 0.0 },
  { "Rikku", 0.0 },
  { "Rikky", 1.0 },
  { "Riko", 0.3539823 },
  { "Riku", 1.0 },
  { "Rikuto", 1.0 },
  { "Riky", 1.0 },
  { "Rikya", 0.0 },
  { "Rikyah", 0.0 },
  { "Rilah", 0.0 },
  { "Rilan", 0.91847825 },
  { "Riland", 1.0 },
  { "Rilda", 0.0 },
  { "Rilea", 0.0 },
  { "Rileah", 0.0 },
  { "Rilee", 0.11448141 },
  { "Rilei", 0.0 },
  { "Rileigh", 0.014667057 },
  { "Rilen", 1.0 },
  { "Riley", 0.4914509 },
  { "Rileyann", 0.0 },
  { "Rileyanne", 0.0 },
  { "Rileymae", 0.0 },
  { "Rileyn", 0.0 },
  { "Rilie", 0.0 },
  { "Riliegh", 0.0 },
  { "Rilla", 0.0 },
  { "Rilley", 0.5307692 },
  { "Rillia", 0.0 },
  { "Rillie", 0.0 },
  { "Rilma", 0.0 },
  { "Rilo", 0.72727275 },
  { "Riloh", 0.0 },
  { "Rilwan", 1.0 },
  { "Rily", 0.21621622 },
  { "Rilya", 0.0 },
  { "Rilyn", 0.1138976 },
  { "Rilynn", 0.024020888 },
  { "Rilynne", 0.0 },
  { "Rim", 0.0 },
  { "Rima", 0.0 },
  { "Rimantas", 1.0 },
  { "Rimas", 0.70652175 },
  { "Rimi", 0.0 },
  { "Rimsha", 0.0 },
  { "Rimya", 0.0 },
  { "Rin", 0.0 },
  { "Rina", 0.0 },
  { "Rinad", 0.0 },
  { "Rinah", 0.0 },
  { "Rinaldo", 1.0 },
  { "Rinata", 0.0 },
  { "Rinda", 0.0 },
  { "Rindi", 0.0 },
  { "Rindy", 0.0 },
  { "Rinehart", 1.0 },
  { "Rinesa", 0.0 },
  { "Rinesha", 0.0 },
  { "Rinette", 0.0 },
  { "Ringo", 1.0 },
  { "Rini", 0.0 },
  { "Riniya", 0.0 },
  { "Riniyah", 0.0 },
  { "Rinka", 0.0 },
  { "Rinley", 0.0 },
  { "Rinna", 0.0 },
  { "Rinnah", 0.0 },
  { "Rinnie", 1.0 },
  { "Rino", 0.9768519 },
  { "Rinoa", 0.0 },
  { "Rinor", 1.0 },
  { "Rintaro", 1.0 },
  { "Rinya", 0.0 },
  { "Rinyah", 0.0 },
  { "Rio", 0.60439247 },
  { "Rion", 0.8357143 },
  { "Riona", 0.0 },
  { "Rionna", 0.0 },
  { "Riordan", 1.0 },
  { "Riot", 0.88610476 },
  { "Riott", 1.0 },
  { "Rip", 1.0 },
  { "Ripal", 1.0 },
  { "Ripken", 1.0 },
  { "Riplee", 0.0 },
  { "Ripley", 0.24869566 },
  { "Ripsi", 0.0 },
  { "Riquel", 0.0 },
  { "Riquelme", 1.0 },
  { "Riquelmer", 1.0 },
  { "Riqui", 0.0 },
  { "Risa", 0.0 },
  { "Risako", 0.0 },
  { "Rise", 0.004621072 },
  { "Riser", 1.0 },
  { "Rish", 1.0 },
  { "Risha", 0.0 },
  { "Rishaan", 1.0 },
  { "Rishab", 1.0 },
  { "Rishabh", 1.0 },
  { "Rishad", 1.0 },
  { "Rishan", 1.0 },
  { "Rishank", 1.0 },
  { "Rishard", 1.0 },
  { "Rishaun", 1.0 },
  { "Rishav", 1.0 },
  { "Rishawn", 1.0 },
  { "Rishi", 1.0 },
  { "Rishik", 1.0 },
  { "Rishika", 0.0 },
  { "Rishikesh", 1.0 },
  { "Rishil", 1.0 },
  { "Rishima", 0.0 },
  { "Rishin", 1.0 },
  { "Rishit", 1.0 },
  { "Rishita", 0.0 },
  { "Rishith", 1.0 },
  { "Rishitha", 0.0 },
  { "Rishon", 1.0 },
  { "Rishona", 0.0 },
  { "Rishvik", 1.0 },
  { "Rissa", 0.0 },
  { "Rissie", 0.0 },
  { "Risten", 0.0 },
  { "Ristin", 0.0 },
  { "Ristina", 0.0 },
  { "Riston", 1.0 },
  { "Rita", 0.0027432616 },
  { "Ritaann", 0.0 },
  { "Ritage", 0.0 },
  { "Ritaj", 0.0 },
  { "Rital", 0.0 },
  { "Ritamae", 0.0 },
  { "Ritamarie", 0.0 },
  { "Ritanya", 0.0 },
  { "Ritch", 1.0 },
  { "Ritchard", 1.0 },
  { "Ritchey", 1.0 },
  { "Ritchie", 0.98712337 },
  { "Ritchy", 1.0 },
  { "Ritesh", 1.0 },
  { "Rith", 1.0 },
  { "Ritha", 0.0 },
  { "Rithanya", 0.0 },
  { "Rithi", 0.0 },
  { "Rithik", 1.0 },
  { "Rithika", 0.0 },
  { "Rithu", 0.0 },
  { "Rithvik", 1.0 },
  { "Rithvika", 0.0 },
  { "Rithwik", 1.0 },
  { "Rithy", 1.0 },
  { "Riti", 0.0 },
  { "Ritik", 1.0 },
  { "Ritika", 0.0 },
  { "Ritisha", 0.0 },
  { "Ritney", 0.0 },
  { "Rito", 1.0 },
  { "Ritsuko", 0.0 },
  { "Ritta", 0.0 },
  { "Rittal", 0.0 },
  { "Rittany", 0.0 },
  { "Ritter", 1.0 },
  { "Rittie", 0.0 },
  { "Rittney", 0.0 },
  { "Ritu", 0.0 },
  { "Ritvi", 0.0 },
  { "Ritvik", 1.0 },
  { "Ritvika", 0.0 },
  { "Ritwik", 1.0 },
  { "Ritzy", 0.0 },
  { "Riuka", 0.0 },
  { "Riva", 0.0 },
  { "Rivaan", 1.0 },
  { "Rivaldo", 1.0 },
  { "Rivan", 1.0 },
  { "Riven", 0.53 },
  { "River", 0.7037727 },
  { "Rivera", 0.0 },
  { "Riverlee", 0.0 },
  { "Riverly", 0.0 },
  { "Riverlyn", 0.0 },
  { "Riverlynn", 0.0 },
  { "Rivers", 0.71274036 },
  { "Rives", 1.0 },
  { "Rivian", 0.0 },
  { "Riviera", 0.0 },
  { "Rivka", 0.0 },
  { "Rivkah", 0.0 },
  { "Rivky", 0.0 },
  { "Rix", 1.0 },
  { "Rixon", 1.0 },
  { "Rixton", 1.0 },
  { "Riya", 0.0 },
  { "Riyaan", 0.8666667 },
  { "Riyaansh", 1.0 },
  { "Riyad", 1.0 },
  { "Riyadh", 1.0 },
  { "Riyah", 0.0 },
  { "Riyan", 0.5354523 },
  { "Riyana", 0.0 },
  { "Riyann", 0.0 },
  { "Riyanna", 0.0 },
  { "Riyansh", 1.0 },
  { "Riyanshi", 0.0 },
  { "Riyansika", 0.0 },
  { "Riyaq", 0.0 },
  { "Riyaz", 1.0 },
  { "Riylee", 0.0 },
  { "Riyon", 1.0 },
  { "Riyonna", 0.0 },
  { "Riza", 0.0 },
  { "Rizal", 1.0 },
  { "Rizelle", 0.0 },
  { "Rizwan", 1.0 },
  { "Rizwana", 0.0 },
  { "Rizza", 0.0 },
  { "Rizzo", 1.0 },
  { "Rj", 1.0 },
  { "Rjay", 1.0 },
  { "Rl", 1.0 },
  { "Rmani", 0.28294572 },
  { "Rmon", 1.0 },
  { "Rmoni", 0.067567565 },
  { "Rmonie", 0.0 },
  { "Rmontay", 1.0 },
  { "Ro", 0.0 },
  { "Roaa", 0.0 },
  { "Roald", 1.0 },
  { "Roam", 1.0 },
  { "Roan", 0.9206349 },
  { "Roanan", 1.0 },
  { "Roandy", 1.0 },
  { "Roane", 1.0 },
  { "Roanin", 1.0 },
  { "Roanld", 1.0 },
  { "Roann", 0.0 },
  { "Roanna", 0.0 },
  { "Roanne", 0.0 },
  { "Roarie", 0.0 },
  { "Roark", 1.0 },
  { "Roarke", 1.0 },
  { "Roary", 1.0 },
  { "Rob", 1.0 },
  { "Roba", 0.0 },
  { "Robah", 1.0 },
  { "Robann", 0.0 },
  { "Robart", 1.0 },
  { "Robb", 1.0 },
  { "Robbe", 0.79310346 },
  { "Robben", 0.65384614 },
  { "Robbert", 1.0 },
  { "Robbey", 1.0 },
  { "Robbi", 0.1691057 },
  { "Robbie", 0.48320168 },
  { "Robbin", 0.17183425 },
  { "Robbins", 1.0 },
  { "Robby", 0.9795783 },
  { "Robbye", 0.0 },
  { "Robbyn", 0.0 },
  { "Robecca", 0.0 },
  { "Robeert", 1.0 },
  { "Robel", 1.0 },
  { "Roben", 0.2927928 },
  { "Robena", 0.0 },
  { "Robenia", 0.0 },
  { "Robenson", 1.0 },
  { "Rober", 1.0 },
  { "Robere", 1.0 },
  { "Roberick", 1.0 },
  { "Roberrt", 1.0 },
  { "Roberson", 1.0 },
  { "Robert", 0.9958383 },
  { "Roberta", 0.004551778 },
  { "Robertanthony", 1.0 },
  { "Robertcharles", 1.0 },
  { "Roberte", 1.0 },
  { "Roberth", 1.0 },
  { "Robertha", 0.0 },
  { "Robertia", 0.0 },
  { "Robertina", 0.0 },
  { "Robertine", 0.0 },
  { "Robertjames", 1.0 },
  { "Robertjohn", 1.0 },
  { "Robertjr", 1.0 },
  { "Robertlee", 1.0 },
  { "Robertmichael", 1.0 },
  { "Roberto", 0.99225533 },
  { "Robertocarlos", 1.0 },
  { "Roberts", 1.0 },
  { "Robertson", 1.0 },
  { "Robertt", 1.0 },
  { "Robertta", 0.0 },
  { "Robet", 1.0 },
  { "Robeto", 1.0 },
  { "Robette", 0.0 },
  { "Robey", 1.0 },
  { "Robi", 0.2457627 },
  { "Robia", 0.0 },
  { "Robie", 0.6468647 },
  { "Robin", 0.13315785 },
  { "Robina", 0.0 },
  { "Robinann", 0.0 },
  { "Robine", 0.0 },
  { "Robinetta", 0.0 },
  { "Robinette", 0.0 },
  { "Robinho", 1.0 },
  { "Robinique", 0.0 },
  { "Robinn", 0.0 },
  { "Robins", 1.0 },
  { "Robinson", 1.0 },
  { "Robiya", 0.0 },
  { "Roble", 1.0 },
  { "Robley", 1.0 },
  { "Roblyn", 0.0 },
  { "Robney", 1.0 },
  { "Robret", 1.0 },
  { "Robrick", 1.0 },
  { "Robroy", 1.0 },
  { "Robrt", 1.0 },
  { "Robsan", 1.0 },
  { "Robson", 1.0 },
  { "Robt", 1.0 },
  { "Roby", 0.9534517 },
  { "Robyn", 0.018675044 },
  { "Robyne", 0.0 },
  { "Robynn", 0.0 },
  { "Robynne", 0.0 },
  { "Roc", 1.0 },
  { "Rocci", 1.0 },
  { "Rocco", 0.99980885 },
  { "Rocelia", 0.0 },
  { "Rocelin", 0.0 },
  { "Rocelyn", 0.0 },
  { "Roch", 1.0 },
  { "Rochan", 0.0 },
  { "Rochanda", 0.0 },
  { "Rochard", 1.0 },
  { "Roche", 0.63414633 },
  { "Rocheal", 0.0 },
  { "Rochel", 0.0 },
  { "Rochele", 0.0 },
  { "Rochell", 0.053769726 },
  { "Rochella", 0.0 },
  { "Rochelle", 0.008518684 },
  { "Rochelly", 0.0 },
  { "Rochely", 0.0 },
  { "Rochester", 1.0 },
  { "Rochon", 0.5 },
  { "Rochus", 1.0 },
  { "Rocio", 0.032434903 },
  { "Rock", 1.0 },
  { "Rocke", 1.0 },
  { "Rockeem", 1.0 },
  { "Rockell", 0.0 },
  { "Rockelle", 0.0 },
  { "Rocker", 1.0 },
  { "Rocket", 0.9086294 },
  { "Rockey", 1.0 },
  { "Rockford", 1.0 },
  { "Rocki", 0.5 },
  { "Rockie", 0.9336683 },
  { "Rocklan", 1.0 },
  { "Rockland", 1.0 },
  { "Rocklin", 1.0 },
  { "Rocklyn", 0.25263157 },
  { "Rocklynn", 0.0 },
  { "Rockne", 1.0 },
  { "Rockney", 1.0 },
  { "Rocko", 1.0 },
  { "Rockson", 1.0 },
  { "Rockston", 1.0 },
  { "Rockwell", 1.0 },
  { "Rocky", 0.9920116 },
  { "Rocquel", 0.0 },
  { "Rocsi", 0.0 },
  { "Roczen", 1.0 },
  { "Rod", 1.0 },
  { "Roda", 0.0 },
  { "Rodaina", 0.0 },
  { "Rodaisha", 0.0 },
  { "Rodarius", 1.0 },
  { "Rodas", 0.0 },
  { "Rodd", 1.0 },
  { "Roddell", 1.0 },
  { "Rodderick", 1.0 },
  { "Roddey", 1.0 },
  { "Roddick", 1.0 },
  { "Roddie", 0.98376626 },
  { "Roddney", 1.0 },
  { "Roddrick", 1.0 },
  { "Roddy", 1.0 },
  { "Rodeen", 1.0 },
  { "Rodel", 1.0 },
  { "Rodell", 0.9865526 },
  { "Rodella", 0.0 },
  { "Rodena", 0.0 },
  { "Rodeny", 1.0 },
  { "Roderic", 1.0 },
  { "Roderica", 0.0 },
  { "Roderich", 1.0 },
  { "Roderick", 0.99433875 },
  { "Rodericka", 0.0 },
  { "Rodericus", 1.0 },
  { "Roderiquez", 1.0 },
  { "Roderrick", 1.0 },
  { "Rodert", 1.0 },
  { "Rodessa", 0.0 },
  { "Rodger", 1.0 },
  { "Rodgerick", 1.0 },
  { "Rodgers", 1.0 },
  { "Rodick", 1.0 },
  { "Rodie", 0.4 },
  { "Rodin", 1.0 },
  { "Rodina", 0.0 },
  { "Rodion", 1.0 },
  { "Rodjanae", 0.0 },
  { "Rodley", 1.0 },
  { "Rodman", 1.0 },
  { "Rodna", 0.0 },
  { "Rodne", 1.0 },
  { "Rodnecia", 0.0 },
  { "Rodnee", 1.0 },
  { "Rodneisha", 0.0 },
  { "Rodneka", 0.0 },
  { "Rodnell", 1.0 },
  { "Rodner", 1.0 },
  { "Rodnesha", 0.0 },
  { "Rodneshia", 0.0 },
  { "Rodnesia", 0.0 },
  { "Rodney", 0.99522257 },
  { "Rodneysha", 0.0 },
  { "Rodnie", 1.0 },
  { "Rodnika", 0.0 },
  { "Rodnisha", 0.0 },
  { "Rodny", 1.0 },
  { "Rodolfo", 0.9974494 },
  { "Rodolph", 1.0 },
  { "Rodolphe", 1.0 },
  { "Rodolpho", 1.0 },
  { "Rodonna", 0.0 },
  { "Rodre", 1.0 },
  { "Rodrecus", 1.0 },
  { "Rodregus", 1.0 },
  { "Rodrekus", 1.0 },
  { "Rodrianna", 0.0 },
  { "Rodric", 1.0 },
  { "Rodrica", 0.0 },
  { "Rodricas", 1.0 },
  { "Rodrick", 0.9994203 },
  { "Rodricka", 0.0 },
  { "Rodrickus", 1.0 },
  { "Rodrico", 1.0 },
  { "Rodricus", 1.0 },
  { "Rodrigo", 0.9997792 },
  { "Rodrigue", 1.0 },
  { "Rodrigues", 1.0 },
  { "Rodriguez", 0.9947313 },
  { "Rodrigus", 1.0 },
  { "Rodrik", 1.0 },
  { "Rodrika", 0.0 },
  { "Rodrion", 1.0 },
  { "Rodrique", 1.0 },
  { "Rodriques", 1.0 },
  { "Rodriquez", 1.0 },
  { "Rodson", 1.0 },
  { "Rody", 1.0 },
  { "Roe", 0.8802139 },
  { "Roeberta", 0.0 },
  { "Roechelle", 0.0 },
  { "Roee", 1.0 },
  { "Roel", 1.0 },
  { "Roemello", 1.0 },
  { "Roen", 0.9131356 },
  { "Roena", 0.0 },
  { "Roene", 0.0 },
  { "Roenia", 0.0 },
  { "Roert", 1.0 },
  { "Roesha", 0.0 },
  { "Roetta", 0.0 },
  { "Roey", 1.0 },
  { "Rogan", 0.95725864 },
  { "Rogar", 1.0 },
  { "Rogel", 1.0 },
  { "Rogelia", 0.0 },
  { "Rogelio", 0.9972539 },
  { "Rogen", 1.0 },
  { "Rogena", 0.0 },
  { "Rogene", 0.0061652283 },
  { "Rogenia", 0.0 },
  { "Roger", 0.9961656 },
  { "Rogerick", 1.0 },
  { "Rogerio", 1.0 },
  { "Rogers", 0.99700683 },
  { "Roget", 1.0 },
  { "Roggie", 1.0 },
  { "Rogie", 1.0 },
  { "Rogina", 0.0 },
  { "Rogue", 0.44660193 },
  { "Roha", 0.0 },
  { "Rohaan", 1.0 },
  { "Rohail", 1.0 },
  { "Roham", 1.0 },
  { "Rohan", 0.99774 },
  { "Rohana", 0.0 },
  { "Rohanpreet", 1.0 },
  { "Rohen", 1.0 },
  { "Rohil", 1.0 },
  { "Rohin", 1.0 },
  { "Rohini", 0.0 },
  { "Rohit", 1.0 },
  { "Rohith", 1.0 },
  { "Rohma", 0.0 },
  { "Rohman", 1.0 },
  { "Rohn", 1.0 },
  { "Rohnan", 1.0 },
  { "Rohnda", 0.0 },
  { "Rohun", 1.0 },
  { "Roi", 0.7753304 },
  { "Roiann", 0.0 },
  { "Roianne", 0.0 },
  { "Roine", 0.0 },
  { "Roise", 0.0 },
  { "Roisin", 0.0 },
  { "Roiza", 0.0 },
  { "Roizy", 0.0 },
  { "Rojan", 1.0 },
  { "Rojean", 0.0 },
  { "Rojelio", 1.0 },
  { "Rokas", 1.0 },
  { "Rokaya", 0.0 },
  { "Rokeem", 1.0 },
  { "Rokeisha", 0.0 },
  { "Rokesha", 0.0 },
  { "Rokeshia", 0.0 },
  { "Rokeya", 0.0 },
  { "Rokhaya", 0.0 },
  { "Rokia", 0.0 },
  { "Rokiatou", 0.0 },
  { "Rokisha", 0.0 },
  { "Rokiya", 0.0 },
  { "Rokko", 1.0 },
  { "Roko", 1.0 },
  { "Roksana", 0.0 },
  { "Rola", 0.0 },
  { "Rolaine", 0.0 },
  { "Rolan", 1.0 },
  { "Roland", 0.9962656 },
  { "Rolanda", 0.009074941 },
  { "Rolandas", 1.0 },
  { "Rolande", 0.0 },
  { "Rolando", 0.99661195 },
  { "Rolayne", 0.0 },
  { "Roldan", 1.0 },
  { "Rolen", 1.0 },
  { "Rolena", 0.0 },
  { "Rolene", 0.0 },
  { "Roley", 1.0 },
  { "Rolf", 1.0 },
  { "Rolfe", 1.0 },
  { "Rolin", 1.0 },
  { "Rolinda", 0.0 },
  { "Roline", 0.0 },
  { "Rolisha", 0.0 },
  { "Roll", 1.0 },
  { "Rolla", 0.95258933 },
  { "Rollan", 1.0 },
  { "Rolland", 0.99948263 },
  { "Rollande", 0.0 },
  { "Rollen", 1.0 },
  { "Rollene", 0.0 },
  { "Rolley", 1.0 },
  { "Rollie", 0.9931389 },
  { "Rollin", 1.0 },
  { "Rollins", 0.95652175 },
  { "Rollo", 1.0 },
  { "Rollon", 1.0 },
  { "Rolly", 1.0 },
  { "Rollyn", 1.0 },
  { "Rolman", 1.0 },
  { "Rolonda", 0.0 },
  { "Rolondo", 1.0 },
  { "Rolph", 1.0 },
  { "Rolston", 1.0 },
  { "Rolunda", 0.0 },
  { "Roly", 1.0 },
  { "Rolyn", 1.0 },
  { "Rom", 1.0 },
  { "Roma", 0.06496578 },
  { "Romaan", 1.0 },
  { "Romain", 0.98985803 },
  { "Romaine", 0.23156089 },
  { "Romaisa", 0.0 },
  { "Romal", 1.0 },
  { "Romalda", 0.0 },
  { "Romaldo", 1.0 },
  { "Romale", 1.0 },
  { "Romalis", 1.0 },
  { "Roman", 0.9958782 },
  { "Romana", 0.0 },
  { "Romance", 1.0 },
  { "Romanda", 0.0 },
  { "Romando", 1.0 },
  { "Romane", 1.0 },
  { "Romani", 0.6436782 },
  { "Romania", 0.0 },
  { "Romanita", 0.0 },
  { "Romano", 1.0 },
  { "Romanus", 1.0 },
  { "Romar", 1.0 },
  { "Romare", 1.0 },
  { "Romari", 1.0 },
  { "Romario", 1.0 },
  { "Romarion", 1.0 },
  { "Romaro", 1.0 },
  { "Romas", 1.0 },
  { "Romayne", 0.014977974 },
  { "Rome", 0.9445676 },
  { "Romee", 0.0 },
  { "Romeesa", 0.0 },
  { "Romeisha", 0.0 },
  { "Romeka", 0.0 },
  { "Romel", 1.0 },
  { "Romelda", 0.0 },
  { "Romelia", 0.0 },
  { "Romelio", 1.0 },
  { "Romell", 0.92898554 },
  { "Romella", 0.0 },
  { "Romelle", 0.2987552 },
  { "Romello", 1.0 },
  { "Romellow", 1.0 },
  { "Romelo", 1.0 },
  { "Romen", 1.0 },
  { "Romeo", 1.0 },
  { "Romer", 1.0 },
  { "Romere", 1.0 },
  { "Romero", 1.0 },
  { "Romesha", 0.0 },
  { "Romeshia", 0.0 },
  { "Rometta", 0.0 },
  { "Romey", 0.7408313 },
  { "Romi", 0.05882353 },
  { "Romia", 0.0 },
  { "Romiah", 0.0 },
  { "Romie", 0.7923729 },
  { "Romika", 0.0 },
  { "Romil", 1.0 },
  { "Romila", 0.0 },
  { "Romilda", 0.0 },
  { "Romilly", 0.0 },
  { "Romin", 1.0 },
  { "Romina", 0.0 },
  { "Rominna", 0.0 },
  { "Romio", 1.0 },
  { "Romir", 1.0 },
  { "Romit", 1.0 },
  { "Romita", 0.0 },
  { "Romiya", 0.0 },
  { "Romiyah", 0.0 },
  { "Rommel", 1.0 },
  { "Rommell", 1.0 },
  { "Rommie", 1.0 },
  { "Rommy", 0.8 },
  { "Romney", 0.83838385 },
  { "Romola", 0.0 },
  { "Romolo", 1.0 },
  { "Romon", 1.0 },
  { "Romona", 0.0 },
  { "Romond", 1.0 },
  { "Romonda", 0.0 },
  { "Romondo", 1.0 },
  { "Romone", 1.0 },
  { "Romonia", 0.0 },
  { "Romonica", 0.0 },
  { "Romuald", 1.0 },
  { "Romualdo", 1.0 },
  { "Romulo", 1.0 },
  { "Romulus", 1.0 },
  { "Romunda", 0.0 },
  { "Romus", 1.0 },
  { "Romy", 0.097039476 },
  { "Romya", 0.0 },
  { "Romyn", 1.0 },
  { "Romyna", 0.0 },
  { "Ron", 0.9980613 },
  { "Rona", 0.0 },
  { "Ronada", 0.0 },
  { "Ronae", 0.0 },
  { "Ronaele", 0.0 },
  { "Ronak", 0.9929907 },
  { "Ronal", 1.0 },
  { "Ronald", 0.99643993 },
  { "Ronalda", 0.0 },
  { "Ronaldinho", 1.0 },
  { "Ronaldino", 1.0 },
  { "Ronaldo", 1.0 },
  { "Ronalee", 0.0 },
  { "Ronalie", 0.0 },
  { "Ronalyn", 0.0 },
  { "Ronan", 0.9902965 },
  { "Ronasia", 0.0 },
  { "Ronata", 0.0 },
  { "Ronav", 1.0 },
  { "Ronaya", 0.0 },
  { "Ronda", 0.01035806 },
  { "Rondal", 1.0 },
  { "Rondald", 1.0 },
  { "Rondale", 1.0 },
  { "Rondall", 1.0 },
  { "Rondallyn", 0.0 },
  { "Rondalyn", 0.0 },
  { "Rondarious", 1.0 },
  { "Rondarius", 1.0 },
  { "Ronde", 1.0 },
  { "Rondee", 0.0 },
  { "Rondel", 1.0 },
  { "Rondell", 0.97547585 },
  { "Rondella", 0.0 },
  { "Rondelle", 0.31578946 },
  { "Ronderick", 1.0 },
  { "Rondey", 1.0 },
  { "Rondez", 1.0 },
  { "Rondi", 0.011627907 },
  { "Rondia", 0.0 },
  { "Rondie", 0.8611111 },
  { "Rondle", 1.0 },
  { "Rondo", 1.0 },
  { "Rondol", 1.0 },
  { "Rondre", 1.0 },
  { "Rondrick", 1.0 },
  { "Rondy", 1.0 },
  { "Rone", 1.0 },
  { "Ronea", 0.0 },
  { "Roneal", 1.0 },
  { "Ronecia", 0.0 },
  { "Ronee", 0.0 },
  { "Roneesha", 0.0 },
  { "Roneika", 0.0 },
  { "Roneil", 1.0 },
  { "Roneisha", 0.0 },
  { "Roneka", 0.0 },
  { "Ronekia", 0.0 },
  { "Ronel", 0.9870968 },
  { "Ronelda", 0.0 },
  { "Ronell", 0.82604057 },
  { "Ronella", 0.0 },
  { "Ronelle", 0.10502283 },
  { "Ronen", 1.0 },
  { "Ronesha", 0.0 },
  { "Roneshia", 0.0 },
  { "Ronesia", 0.0 },
  { "Ronessa", 0.0 },
  { "Ronetta", 0.0 },
  { "Ronette", 0.0 },
  { "Roney", 0.9942595 },
  { "Roni", 0.1359741 },
  { "Ronia", 0.0 },
  { "Roniah", 0.0 },
  { "Ronica", 0.0 },
  { "Ronice", 0.0 },
  { "Ronicia", 0.0 },
  { "Ronicka", 0.0 },
  { "Ronie", 0.53488374 },
  { "Roniece", 0.0 },
  { "Roniel", 1.0 },
  { "Roniesha", 0.0 },
  { "Ronik", 1.0 },
  { "Ronika", 0.0 },
  { "Ronil", 1.0 },
  { "Ronin", 0.9834216 },
  { "Roniqua", 0.0 },
  { "Ronique", 0.029498525 },
  { "Ronise", 0.0 },
  { "Ronisha", 0.0 },
  { "Ronishia", 0.0 },
  { "Ronisue", 0.0 },
  { "Ronit", 0.58948684 },
  { "Ronita", 0.0 },
  { "Ronith", 1.0 },
  { "Roniya", 0.0 },
  { "Roniyah", 0.0 },
  { "Ronja", 0.0 },
  { "Ronke", 0.0 },
  { "Ronld", 1.0 },
  { "Ronmel", 1.0 },
  { "Ronn", 1.0 },
  { "Ronna", 0.0 },
  { "Ronnae", 0.0 },
  { "Ronnah", 0.0 },
  { "Ronnal", 1.0 },
  { "Ronnald", 1.0 },
  { "Ronnasia", 0.0 },
  { "Ronnay", 0.0 },
  { "Ronnda", 0.0 },
  { "Ronne", 0.4962963 },
  { "Ronnee", 0.0 },
  { "Ronneisha", 0.0 },
  { "Ronneka", 0.0 },
  { "Ronnel", 1.0 },
  { "Ronnell", 0.92512274 },
  { "Ronnella", 0.0 },
  { "Ronnelle", 0.3043478 },
  { "Ronnesha", 0.0 },
  { "Ronneshia", 0.0 },
  { "Ronnet", 0.0 },
  { "Ronnett", 0.0 },
  { "Ronnetta", 0.0 },
  { "Ronnette", 0.0 },
  { "Ronney", 1.0 },
  { "Ronni", 0.016496701 },
  { "Ronnia", 0.0 },
  { "Ronnica", 0.0 },
  { "Ronnice", 0.0 },
  { "Ronnie", 0.9600618 },
  { "Ronniece", 0.0 },
  { "Ronniel", 1.0 },
  { "Ronniesha", 0.0 },
  { "Ronnika", 0.0 },
  { "Ronnisha", 0.0 },
  { "Ronnita", 0.0 },
  { "Ronniya", 0.0 },
  { "Ronny", 0.96772426 },
  { "Ronnye", 0.24390244 },
  { "Ronold", 1.0 },
  { "Ronon", 1.0 },
  { "Ronreaco", 1.0 },
  { "Ronreco", 1.0 },
  { "Ronrico", 1.0 },
  { "Ronshay", 0.0 },
  { "Ronson", 1.0 },
  { "Ronta", 1.0 },
  { "Rontae", 1.0 },
  { "Rontarius", 1.0 },
  { "Rontasia", 0.0 },
  { "Rontavia", 0.0 },
  { "Rontavious", 1.0 },
  { "Rontavis", 1.0 },
  { "Rontavius", 1.0 },
  { "Ronte", 1.0 },
  { "Rontez", 1.0 },
  { "Rontrell", 1.0 },
  { "Rony", 1.0 },
  { "Ronya", 0.0 },
  { "Ronyae", 0.0 },
  { "Ronyn", 1.0 },
  { "Ronza", 0.0 },
  { "Ronzel", 1.0 },
  { "Ronzell", 1.0 },
  { "Roody", 1.0 },
  { "Roohi", 0.0 },
  { "Rook", 1.0 },
  { "Rooke", 1.0 },
  { "Rooks", 1.0 },
  { "Rooney", 0.7228525 },
  { "Roony", 1.0 },
  { "Roopa", 0.0 },
  { "Roopal", 0.0 },
  { "Rooseve", 1.0 },
  { "Roosevelt", 0.99295324 },
  { "Roosvelt", 1.0 },
  { "Roozbeh", 1.0 },
  { "Rope", 1.0 },
  { "Roper", 1.0 },
  { "Roqaya", 0.0 },
  { "Roquan", 1.0 },
  { "Roque", 1.0 },
  { "Roquel", 0.0 },
  { "Roran", 1.0 },
  { "Roree", 0.0 },
  { "Rorey", 0.58433735 },
  { "Rori", 0.008677897 },
  { "Roric", 1.0 },
  { "Rorick", 1.0 },
  { "Rorie", 0.008319467 },
  { "Rorik", 1.0 },
  { "Rorke", 1.0 },
  { "Rorrie", 0.0 },
  { "Rorry", 1.0 },
  { "Rory", 0.8072597 },
  { "Rosa", 0.006727769 },
  { "Rosaangelica", 0.0 },
  { "Rosabel", 0.0 },
  { "Rosabell", 0.0 },
  { "Rosabella", 0.0 },
  { "Rosabelle", 0.0 },
  { "Rosaelena", 0.0 },
  { "Rosaelia", 0.0 },
  { "Rosaicela", 0.0 },
  { "Rosaida", 0.0 },
  { "Rosaire", 0.9703264 },
  { "Rosaisela", 0.0 },
  { "Rosala", 0.0 },
  { "Rosaland", 0.0 },
  { "Rosalba", 0.0 },
  { "Rosale", 0.0 },
  { "Rosalea", 0.0 },
  { "Rosaleah", 0.0 },
  { "Rosalean", 0.0 },
  { "Rosalee", 0.0 },
  { "Rosaleen", 0.0 },
  { "Rosalei", 0.0 },
  { "Rosaleigh", 0.0 },
  { "Rosalena", 0.0 },
  { "Rosalene", 0.0 },
  { "Rosaleta", 0.0 },
  { "Rosaley", 0.0 },
  { "Rosali", 0.0 },
  { "Rosalia", 0.0 },
  { "Rosalie", 0.0013731745 },
  { "Rosalin", 0.0 },
  { "Rosalina", 0.0 },
  { "Rosalind", 0.0 },
  { "Rosalinda", 0.00033896006 },
  { "Rosalinde", 0.0 },
  { "Rosaline", 0.0 },
  { "Rosalino", 1.0 },
  { "Rosalio", 0.99747986 },
  { "Rosalita", 0.0 },
  { "Rosaliyah", 0.0 },
  { "Rosaliz", 0.0 },
  { "Rosalla", 0.0 },
  { "Rosalua", 0.0 },
  { "Rosalva", 0.0 },
  { "Rosaly", 0.0 },
  { "Rosalyn", 0.0 },
  { "Rosalyna", 0.0 },
  { "Rosalynd", 0.0 },
  { "Rosalynda", 0.0 },
  { "Rosalyne", 0.0 },
  { "Rosalynn", 0.0 },
  { "Rosalynne", 0.0 },
  { "Rosamae", 0.0 },
  { "Rosamaria", 0.0 },
  { "Rosamarie", 0.0 },
  { "Rosamary", 0.0 },
  { "Rosamay", 0.0 },
  { "Rosamond", 0.0018309429 },
  { "Rosamonde", 0.0 },
  { "Rosamund", 0.0 },
  { "Rosan", 0.0 },
  { "Rosana", 0.0 },
  { "Rosandra", 0.0 },
  { "Rosane", 0.0 },
  { "Rosanell", 0.0 },
  { "Rosangel", 0.0 },
  { "Rosangela", 0.0 },
  { "Rosangelica", 0.0 },
  { "Rosann", 0.0 },
  { "Rosanna", 0.00042878583 },
  { "Rosanne", 0.00034094782 },
  { "Rosanny", 0.0 },
  { "Rosaria", 0.0 },
  { "Rosarie", 0.0 },
  { "Rosario", 0.3442529 },
  { "Rosary", 0.0 },
  { "Rosaura", 0.0 },
  { "Rosaysela", 0.0 },
  { "Rosbel", 1.0 },
  { "Roschell", 0.0 },
  { "Roschelle", 0.0 },
  { "Rosco", 1.0 },
  { "Roscoe", 0.99976754 },
  { "Rose", 0.0043590185 },
  { "Rosea", 0.0 },
  { "Roseabella", 0.0 },
  { "Rosealee", 0.0 },
  { "Rosealie", 0.0 },
  { "Rosealina", 0.0 },
  { "Rosealyn", 0.0 },
  { "Rosealynn", 0.0 },
  { "Rosean", 0.8648649 },
  { "Roseana", 0.0 },
  { "Roseangela", 0.0 },
  { "Roseann", 0.0 },
  { "Roseanna", 0.0 },
  { "Roseanne", 0.0 },
  { "Rosebell", 0.0 },
  { "Rosebella", 0.0 },
  { "Rosebelle", 0.0 },
  { "Rosebud", 0.0 },
  { "Roseellen", 0.0 },
  { "Rosel", 0.0 },
  { "Rosela", 0.0 },
  { "Roseland", 0.0 },
  { "Roselani", 0.0 },
  { "Roselea", 0.0 },
  { "Roselean", 0.0 },
  { "Roselee", 0.0 },
  { "Roseleen", 0.0 },
  { "Roseleigh", 0.0 },
  { "Roselena", 0.0 },
  { "Roselene", 0.0 },
  { "Roseli", 0.0 },
  { "Roselia", 0.0 },
  { "Roselie", 0.0 },
  { "Roselin", 0.0 },
  { "Roselina", 0.0 },
  { "Roselind", 0.0 },
  { "Roselinda", 0.0 },
  { "Roseline", 0.0 },
  { "Roselio", 1.0 },
  { "Rosell", 0.15555556 },
  { "Rosella", 0.00032509753 },
  { "Rosellar", 0.0 },
  { "Roselle", 0.0 },
  { "Rosellen", 0.0 },
  { "Rosellie", 0.0 },
  { "Roselma", 0.0 },
  { "Rosely", 0.0 },
  { "Roselyn", 0.0 },
  { "Roselyne", 0.0 },
  { "Roselynn", 0.0 },
  { "Roselynne", 0.0 },
  { "Rosemae", 0.0 },
  { "Rosemaire", 0.0 },
  { "Rosemari", 0.0 },
  { "Rosemaria", 0.0 },
  { "Rosemarie", 0.0006549438 },
  { "Rosemarry", 0.0 },
  { "Rosemary", 0.0023011705 },
  { "Rosemay", 0.0 },
  { "Rosemery", 0.0 },
  { "Rosemma", 0.0 },
  { "Rosemond", 0.035363458 },
  { "Rosena", 0.0 },
  { "Rosenda", 0.0 },
  { "Rosendo", 1.0 },
  { "Rosene", 0.0 },
  { "Rosenell", 0.0 },
  { "Rosenia", 0.0 },
  { "Rosenna", 0.0 },
  { "Roser", 0.0 },
  { "Rosetta", 0.0015831324 },
  { "Rosette", 0.0 },
  { "Rosetter", 0.0 },
  { "Rosettia", 0.0 },
  { "Rosevell", 1.0 },
  { "Rosevelt", 1.0 },
  { "Rosey", 0.0 },
  { "Rosezanna", 0.0 },
  { "Rosezell", 0.0 },
  { "Rosezella", 0.0 },
  { "Rosezena", 0.0 },
  { "Rosezetta", 0.0 },
  { "Rosezina", 0.0 },
  { "Rosh", 1.0 },
  { "Rosha", 0.0 },
  { "Roshaad", 1.0 },
  { "Roshad", 1.0 },
  { "Roshae", 0.0 },
  { "Roshan", 0.9229335 },
  { "Roshana", 0.0 },
  { "Roshanda", 0.0 },
  { "Roshandra", 0.0 },
  { "Roshane", 1.0 },
  { "Roshani", 0.0 },
  { "Roshanna", 0.0 },
  { "Roshanta", 0.0 },
  { "Roshard", 1.0 },
  { "Roshaud", 1.0 },
  { "Roshaun", 0.8752834 },
  { "Roshaunda", 0.0 },
  { "Roshawn", 0.676132 },
  { "Roshawna", 0.0 },
  { "Roshawnda", 0.0 },
  { "Roshay", 0.0 },
  { "Roshea", 0.0 },
  { "Rosheda", 0.0 },
  { "Rosheen", 1.0 },
  { "Rosheena", 0.0 },
  { "Rosheka", 0.0 },
  { "Roshell", 0.01983471 },
  { "Roshelle", 0.0 },
  { "Roshena", 0.0 },
  { "Roshida", 0.0 },
  { "Roshika", 0.0 },
  { "Roshina", 0.0 },
  { "Roshini", 0.0 },
  { "Roshni", 0.0 },
  { "Roshod", 1.0 },
  { "Roshon", 0.7345133 },
  { "Roshonda", 0.0 },
  { "Roshondra", 0.0 },
  { "Roshonna", 0.0 },
  { "Roshun", 0.8125 },
  { "Roshunda", 0.0 },
  { "Roshundra", 0.0 },
  { "Rosi", 0.0 },
  { "Rosia", 0.0 },
  { "Rosibel", 0.0 },
  { "Rosicela", 0.0 },
  { "Rosie", 0.0059585916 },
  { "Rosielee", 0.0 },
  { "Rosier", 1.0 },
  { "Rosietta", 0.0 },
  { "Rosiland", 0.0 },
  { "Rosilda", 0.0 },
  { "Rosilee", 0.0 },
  { "Rosilind", 0.0 },
  { "Rosilyn", 0.0 },
  { "Rosilynn", 0.0 },
  { "Rosina", 0.0 },
  { "Rosine", 0.0 },
  { "Rosio", 0.0076643806 },
  { "Rosisela", 0.0 },
  { "Rosita", 0.0 },
  { "Rositta", 0.0 },
  { "Rosland", 0.0 },
  { "Roslie", 0.0 },
  { "Roslin", 0.0 },
  { "Roslind", 0.0 },
  { "Roslyn", 0.0005835668 },
  { "Roslynn", 0.0 },
  { "Rosmarie", 0.0 },
  { "Rosmary", 0.0 },
  { "Rosmely", 0.0 },
  { "Rosmeri", 0.0 },
  { "Rosmery", 0.0 },
  { "Rosolino", 1.0 },
  { "Rosolyn", 0.0 },
  { "Rosonda", 0.0 },
  { "Ross", 0.99501127 },
  { "Rossalind", 0.0 },
  { "Rossalyn", 0.0 },
  { "Rossana", 0.0 },
  { "Rossanna", 0.0 },
  { "Rossano", 1.0 },
  { "Rosse", 0.0 },
  { "Rosselin", 0.0 },
  { "Rossella", 0.0 },
  { "Rosselyn", 0.0 },
  { "Rosser", 1.0 },
  { "Rossetta", 0.0 },
  { "Rossi", 0.53451675 },
  { "Rossibell", 0.0 },
  { "Rossie", 0.30439952 },
  { "Rosslyn", 0.0 },
  { "Rosslynn", 0.0 },
  { "Rossmary", 0.0 },
  { "Rosston", 1.0 },
  { "Rosswell", 1.0 },
  { "Rossy", 0.0 },
  { "Rostam", 1.0 },
  { "Rostin", 1.0 },
  { "Rostislav", 1.0 },
  { "Roston", 1.0 },
  { "Rosvelt", 1.0 },
  { "Roswell", 1.0 },
  { "Rosy", 0.0 },
  { "Rosylin", 0.0 },
  { "Rosyln", 0.0 },
  { "Roszella", 0.0 },
  { "Roszetta", 0.0 },
  { "Rotasha", 0.0 },
  { "Rotem", 0.10638298 },
  { "Roth", 1.0 },
  { "Rotha", 0.03773585 },
  { "Rothana", 0.54545456 },
  { "Rothwell", 1.0 },
  { "Rotimi", 1.0 },
  { "Rotisha", 0.0 },
  { "Rotonda", 0.0 },
  { "Rotonia", 0.0 },
  { "Rotonya", 0.0 },
  { "Rotunda", 0.0 },
  { "Roua", 0.0 },
  { "Rouchelle", 0.0 },
  { "Roudolph", 1.0 },
  { "Roudy", 1.0 },
  { "Rougui", 0.0 },
  { "Rouguiatou", 0.0 },
  { "Roul", 1.0 },
  { "Roula", 0.0 },
  { "Rouldph", 1.0 },
  { "Rounak", 1.0 },
  { "Rourke", 1.0 },
  { "Rousey", 0.0 },
  { "Roux", 0.17910448 },
  { "Rovella", 0.0 },
  { "Rovena", 0.0 },
  { "Rovenia", 0.0 },
  { "Roverta", 0.0 },
  { "Roverto", 1.0 },
  { "Rovilla", 0.0 },
  { "Rovin", 1.0 },
  { "Rovina", 0.0 },
  { "Rovon", 1.0 },
  { "Rowan", 0.61098313 },
  { "Rowana", 0.0 },
  { "Rowann", 0.0 },
  { "Rowden", 1.0 },
  { "Rowdie", 0.5 },
  { "Rowdy", 0.996997 },
  { "Rowe", 0.9444444 },
  { "Rowell", 1.0 },
  { "Rowen", 0.72435415 },
  { "Rowena", 0.0 },
  { "Rowene", 0.0 },
  { "Rowenna", 0.0 },
  { "Rowin", 0.93203884 },
  { "Rowland", 1.0 },
  { "Rowley", 1.0 },
  { "Rownan", 1.0 },
  { "Rowrenia", 0.0 },
  { "Rowyn", 0.20286396 },
  { "Rowynn", 0.023923445 },
  { "Rox", 0.014925373 },
  { "Roxa", 0.0 },
  { "Roxan", 0.0 },
  { "Roxana", 0.002334351 },
  { "Roxane", 0.0 },
  { "Roxann", 0.0 },
  { "Roxanna", 0.00054440816 },
  { "Roxanne", 0.0009501762 },
  { "Roxas", 1.0 },
  { "Roxeann", 0.0 },
  { "Roxeanne", 0.0 },
  { "Roxen", 1.0 },
  { "Roxene", 0.0 },
  { "Roxette", 0.0 },
  { "Roxey", 0.0 },
  { "Roxi", 0.0 },
  { "Roxianne", 0.0 },
  { "Roxie", 0.011139768 },
  { "Roxii", 0.0 },
  { "Roxine", 0.0 },
  { "Roxsana", 0.0 },
  { "Roxsand", 0.0 },
  { "Roxsanne", 0.0 },
  { "Roxton", 1.0 },
  { "Roxxane", 0.0 },
  { "Roxxanne", 0.0 },
  { "Roxxi", 0.0 },
  { "Roxy", 0.029513888 },
  { "Roy", 0.99339193 },
  { "Roya", 0.0 },
  { "Royace", 1.0 },
  { "Royal", 0.8929137 },
  { "Royale", 0.4147651 },
  { "Royalene", 0.0 },
  { "Royall", 0.8958333 },
  { "Royalle", 0.0 },
  { "Royalte", 0.0 },
  { "Royaltee", 0.0 },
  { "Royalti", 0.0 },
  { "Royalty", 0.12644628 },
  { "Royan", 1.0 },
  { "Royann", 0.0 },
  { "Royanna", 0.0 },
  { "Royanne", 0.0 },
  { "Royce", 0.9281791 },
  { "Royd", 1.0 },
  { "Royden", 1.0 },
  { "Roydon", 1.0 },
  { "Roye", 0.7902439 },
  { "Royel", 0.75555557 },
  { "Royelle", 0.0 },
  { "Royer", 1.0 },
  { "Royesha", 0.0 },
  { "Royetta", 0.0 },
  { "Royle", 1.0 },
  { "Roylee", 1.0 },
  { "Roylene", 0.0 },
  { "Roylynn", 0.0 },
  { "Roynell", 1.0 },
  { "Roynesha", 0.0 },
  { "Royse", 1.0 },
  { "Royster", 1.0 },
  { "Royston", 1.0 },
  { "Royzell", 1.0 },
  { "Roz", 0.0 },
  { "Roza", 0.0 },
  { "Rozalee", 0.0 },
  { "Rozalia", 0.0 },
  { "Rozalie", 0.0 },
  { "Rozalin", 0.0 },
  { "Rozalind", 0.0 },
  { "Rozalyn", 0.0 },
  { "Rozalynn", 0.0 },
  { "Rozan", 0.0 },
  { "Rozana", 0.0 },
  { "Rozann", 0.0 },
  { "Rozanna", 0.0 },
  { "Rozanne", 0.0 },
  { "Rozaria", 0.0 },
  { "Rozario", 1.0 },
  { "Rozay", 0.4878049 },
  { "Roze", 0.0 },
  { "Rozel", 1.0 },
  { "Rozelia", 0.0 },
  { "Rozell", 0.6236559 },
  { "Rozella", 0.0 },
  { "Rozelle", 0.064453125 },
  { "Rozellia", 0.0 },
  { "Rozelyn", 0.0 },
  { "Rozen", 1.0 },
  { "Rozena", 0.0 },
  { "Rozenia", 0.0 },
  { "Rozeta", 0.0 },
  { "Rozetta", 0.0 },
  { "Rozia", 0.0 },
  { "Rozie", 0.0 },
  { "Rozier", 1.0 },
  { "Rozilyn", 0.0 },
  { "Rozina", 0.0 },
  { "Rozine", 0.0 },
  { "Rozita", 0.0 },
  { "Roziya", 0.0 },
  { "Rozlin", 0.0 },
  { "Rozlyn", 0.0 },
  { "Rozlynn", 0.0 },
  { "Rozyn", 0.0 },
  { "Rozzie", 0.0 },
  { "Rozzlyn", 0.0 },
  { "Rpbert", 1.0 },
  { "Rreanna", 0.0 },
  { "Rreon", 0.0 },
  { "Rua", 0.0 },
  { "Ruaa", 0.0 },
  { "Ruah", 0.0 },
  { "Ruairi", 1.0 },
  { "Rual", 1.0 },
  { "Ruan", 1.0 },
  { "Ruann", 0.0 },
  { "Ruari", 0.4074074 },
  { "Ruaridh", 1.0 },
  { "Ruark", 1.0 },
  { "Ruba", 0.0 },
  { "Rubab", 0.0 },
  { "Ruban", 1.0 },
  { "Rubani", 0.0 },
  { "Rubben", 1.0 },
  { "Rubbie", 0.0 },
  { "Rubby", 0.0 },
  { "Rube", 1.0 },
  { "Rubee", 0.0 },
  { "Rubel", 1.0 },
  { "Rubell", 0.0 },
  { "Rubelle", 0.0 },
  { "Ruben", 0.9941037 },
  { "Rubena", 0.0 },
  { "Rubenia", 0.0 },
  { "Rubens", 1.0 },
  { "Rubert", 1.0 },
  { "Ruberta", 0.0 },
  { "Ruberto", 1.0 },
  { "Rubey", 0.0 },
  { "Rubi", 0.0019214933 },
  { "Rubia", 0.0 },
  { "Rubiana", 0.0 },
  { "Rubianna", 0.0 },
  { "Rubicela", 0.0 },
  { "Rubicelia", 0.0 },
  { "Rubie", 0.00933631 },
  { "Rubiel", 1.0 },
  { "Rubin", 0.9976852 },
  { "Rubina", 0.0 },
  { "Rubio", 1.0 },
  { "Rubisela", 0.0 },
  { "Rubit", 0.0 },
  { "Rubith", 0.0 },
  { "Ruble", 0.8174157 },
  { "Ruby", 0.008642075 },
  { "Rubyann", 0.0 },
  { "Rubyanne", 0.0 },
  { "Rubye", 0.0 },
  { "Rubyjane", 0.0 },
  { "Rubylee", 0.0 },
  { "Rubylynn", 0.0 },
  { "Rubymae", 0.0 },
  { "Rubyn", 1.0 },
  { "Rubyrose", 0.0 },
  { "Rubystine", 0.0 },
  { "Ruca", 0.0 },
  { "Rucha", 0.0 },
  { "Ruchama", 0.0 },
  { "Ruchel", 0.0 },
  { "Ruchelle", 0.0 },
  { "Ruchi", 0.0 },
  { "Ruchika", 0.0 },
  { "Ruchir", 1.0 },
  { "Ruchita", 0.0 },
  { "Ruchoma", 0.0 },
  { "Ruchy", 0.0 },
  { "Rucker", 1.0 },
  { "Ruckus", 1.0 },
  { "Rudaina", 0.0 },
  { "Rudalph", 1.0 },
  { "Rudd", 1.0 },
  { "Ruddie", 1.0 },
  { "Ruddy", 0.9902534 },
  { "Rudean", 0.0 },
  { "Rudee", 0.0 },
  { "Rudeen", 0.0 },
  { "Rudell", 0.5103627 },
  { "Rudelle", 0.0 },
  { "Rudene", 0.0 },
  { "Rudhra", 1.0 },
  { "Rudhvi", 1.0 },
  { "Rudi", 0.64606744 },
  { "Rudie", 0.7205882 },
  { "Rudine", 0.0 },
  { "Rudis", 1.0 },
  { "Rudloph", 1.0 },
  { "Rudolf", 1.0 },
  { "Rudolfo", 1.0 },
  { "Rudolph", 0.9963069 },
  { "Rudolphe", 1.0 },
  { "Rudolpho", 1.0 },
  { "Rudolphus", 1.0 },
  { "Rudr", 1.0 },
  { "Rudra", 0.98497266 },
  { "Rudraksh", 1.0 },
  { "Rudransh", 1.0 },
  { "Rudy", 0.9793876 },
  { "Rudyard", 1.0 },
  { "Rue", 0.30799475 },
  { "Rueben", 1.0 },
  { "Ruebin", 1.0 },
  { "Rueger", 1.0 },
  { "Ruel", 0.99728703 },
  { "Ruella", 0.0 },
  { "Ruenell", 0.0 },
  { "Ruey", 0.11111111 },
  { "Rufas", 1.0 },
  { "Rufe", 1.0 },
  { "Rufes", 1.0 },
  { "Ruff", 1.0 },
  { "Ruffin", 1.0 },
  { "Ruffus", 1.0 },
  { "Rufina", 0.0 },
  { "Rufino", 1.0 },
  { "Rufio", 1.0 },
  { "Ruford", 1.0 },
  { "Rufus", 0.9958019 },
  { "Rugayyah", 0.0 },
  { "Rugenia", 0.0 },
  { "Ruger", 0.9951923 },
  { "Rugiatu", 0.0 },
  { "Ruhaan", 1.0 },
  { "Ruhama", 0.0 },
  { "Ruhan", 1.0 },
  { "Ruhani", 0.0 },
  { "Ruhee", 0.0 },
  { "Ruhi", 0.0 },
  { "Ruhika", 0.0 },
  { "Rui", 0.96615905 },
  { "Ruichen", 1.0 },
  { "Ruie", 0.0 },
  { "Ruiqi", 0.0 },
  { "Ruiz", 1.0 },
  { "Rujul", 1.0 },
  { "Rujuta", 0.0 },
  { "Ruka", 0.0 },
  { "Rukaiya", 0.0 },
  { "Rukaiyah", 0.0 },
  { "Rukaya", 0.0 },
  { "Rukayat", 0.0 },
  { "Rukhsar", 0.0 },
  { "Rukia", 0.0 },
  { "Rukiya", 0.0 },
  { "Rukmini", 0.0 },
  { "Rula", 0.0 },
  { "Rulon", 1.0 },
  { "Rumaisa", 0.0 },
  { "Rumalda", 0.0 },
  { "Rumaldo", 1.0 },
  { "Ruman", 0.0 },
  { "Rumana", 0.0 },
  { "Rumaysa", 0.0 },
  { "Rumaysah", 0.0 },
  { "Rumeal", 1.0 },
  { "Rumell", 0.0 },
  { "Rumer", 0.0 },
  { "Rumi", 0.5 },
  { "Rumina", 0.0 },
  { "Rumonda", 0.0 },
  { "Rumor", 0.0 },
  { "Rumsey", 1.0 },
  { "Runa", 0.0 },
  { "Runako", 0.8484849 },
  { "Runar", 1.0 },
  { "Rund", 0.0 },
  { "Runda", 0.0 },
  { "Rundy", 1.0 },
  { "Rune", 0.9202899 },
  { "Runell", 0.0 },
  { "Runelle", 0.0 },
  { "Runette", 0.0 },
  { "Runnie", 1.0 },
  { "Ruot", 1.0 },
  { "Ruoxi", 0.0 },
  { "Rupa", 0.0 },
  { "Rupal", 0.0 },
  { "Rupali", 0.0 },
  { "Rupen", 1.0 },
  { "Rupert", 0.9970332 },
  { "Ruperta", 0.0 },
  { "Ruperto", 1.0 },
  { "Rupesh", 1.0 },
  { "Rupinder", 0.0 },
  { "Ruppert", 1.0 },
  { "Ruqaiya", 0.0 },
  { "Ruqaiyah", 0.0 },
  { "Ruqaya", 0.0 },
  { "Ruqayah", 0.0 },
  { "Ruqayya", 0.0 },
  { "Ruqayyah", 0.0 },
  { "Ruqiya", 0.0 },
  { "Rural", 1.0 },
  { "Rurik", 1.0 },
  { "Rury", 1.0 },
  { "Rus", 0.3548387 },
  { "Rusbel", 1.0 },
  { "Rusell", 1.0 },
  { "Rush", 1.0 },
  { "Rusha", 0.0 },
  { "Rushabh", 1.0 },
  { "Rushad", 1.0 },
  { "Rushan", 1.0 },
  { "Rushank", 1.0 },
  { "Rushawn", 1.0 },
  { "Rushda", 0.0 },
  { "Rushell", 0.0 },
  { "Rushelle", 0.0 },
  { "Rushi", 1.0 },
  { "Rushia", 0.0 },
  { "Rushie", 0.0 },
  { "Rushika", 0.0 },
  { "Rushil", 1.0 },
  { "Rushon", 1.0 },
  { "Rushton", 1.0 },
  { "Rushun", 1.0 },
  { "Ruskin", 1.0 },
  { "Ruslan", 1.0 },
  { "Ruslana", 0.0 },
  { "Russ", 1.0 },
  { "Russchell", 0.0 },
  { "Russchelle", 0.0 },
  { "Russel", 0.9992678 },
  { "Russell", 0.995209 },
  { "Russella", 0.0 },
  { "Russelle", 1.0 },
  { "Russia", 0.0 },
  { "Russie", 0.0 },
  { "Rustam", 1.0 },
  { "Rustan", 1.0 },
  { "Rusten", 1.0 },
  { "Rusti", 0.0 },
  { "Rustie", 0.0 },
  { "Rustin", 1.0 },
  { "Rustina", 0.0 },
  { "Ruston", 1.0 },
  { "Rusty", 0.96911216 },
  { "Rustyn", 1.0 },
  { "Rut", 0.0 },
  { "Ruta", 0.0 },
  { "Rutger", 1.0 },
  { "Ruth", 0.003368036 },
  { "Rutha", 0.0 },
  { "Ruthalee", 0.0 },
  { "Ruthan", 0.0 },
  { "Ruthann", 0.0 },
  { "Ruthanna", 0.0 },
  { "Ruthanne", 0.0 },
  { "Ruthe", 0.0 },
  { "Ruthel", 0.0 },
  { "Ruthell", 0.0 },
  { "Ruthella", 0.0 },
  { "Ruthelle", 0.0 },
  { "Ruthellen", 0.0 },
  { "Ruthelma", 0.0 },
  { "Ruthena", 0.0 },
  { "Ruthene", 0.0 },
  { "Ruther", 0.009884679 },
  { "Rutherford", 1.0 },
  { "Ruthetta", 0.0 },
  { "Ruthey", 0.0 },
  { "Ruthford", 1.0 },
  { "Ruthi", 0.0 },
  { "Ruthia", 0.0 },
  { "Ruthie", 0.0015559662 },
  { "Ruthine", 0.0 },
  { "Ruthlee", 0.0 },
  { "Ruthlene", 0.0 },
  { "Ruthmae", 0.0 },
  { "Ruthmarie", 0.0 },
  { "Ruthmary", 0.0 },
  { "Ruthvik", 1.0 },
  { "Ruthvika", 0.0 },
  { "Ruthy", 0.0 },
  { "Ruthye", 0.0 },
  { "Rutila", 0.0 },
  { "Rutilio", 1.0 },
  { "Rutledge", 1.0 },
  { "Rutu", 0.0 },
  { "Rutva", 0.0 },
  { "Rutvi", 0.0 },
  { "Rutvik", 1.0 },
  { "Ruven", 1.0 },
  { "Ruvi", 0.0 },
  { "Ruvim", 1.0 },
  { "Ruwaida", 0.0 },
  { "Ruwayda", 0.0 },
  { "Ruweyda", 0.0 },
  { "Ruxin", 1.0 },
  { "Ruxton", 1.0 },
  { "Ruy", 1.0 },
  { "Ruya", 0.0 },
  { "Ruzanna", 0.0 },
  { "Ruzgar", 1.0 },
  { "Rwan", 0.0 },
  { "Rwanda", 0.0 },
  { "Ry", 1.0 },
  { "Rya", 0.019367991 },
  { "Ryaan", 1.0 },
  { "Ryad", 1.0 },
  { "Ryah", 0.0 },
  { "Ryahn", 1.0 },
  { "Ryan", 0.97643656 },
  { "Ryana", 0.0 },
  { "Ryananthony", 1.0 },
  { "Ryanchristian", 1.0 },
  { "Ryanchristopher", 1.0 },
  { "Ryane", 0.06775701 },
  { "Ryanjacob", 1.0 },
  { "Ryanjames", 1.0 },
  { "Ryanjay", 1.0 },
  { "Ryanjoseph", 1.0 },
  { "Ryanlee", 1.0 },
  { "Ryanmatthew", 1.0 },
  { "Ryanmichael", 1.0 },
  { "Ryann", 0.07121439 },
  { "Ryanna", 0.0 },
  { "Ryannah", 0.0 },
  { "Ryanne", 0.0 },
  { "Ryanpatrick", 1.0 },
  { "Ryanpaul", 1.0 },
  { "Ryant", 1.0 },
  { "Ryasia", 0.0 },
  { "Ryat", 1.0 },
  { "Ryatt", 0.95652175 },
  { "Ryce", 0.78723407 },
  { "Rychelle", 0.0 },
  { "Rydan", 1.0 },
  { "Rydar", 1.0 },
  { "Rydder", 1.0 },
  { "Rydel", 1.0 },
  { "Rydell", 1.0 },
  { "Ryden", 0.96520144 },
  { "Ryder", 0.977402 },
  { "Rydge", 1.0 },
  { "Rydia", 0.0 },
  { "Rydin", 1.0 },
  { "Rydon", 1.0 },
  { "Rye", 0.90343696 },
  { "Ryean", 1.0 },
  { "Ryeisha", 0.0 },
  { "Ryeitt", 0.0 },
  { "Ryeker", 1.0 },
  { "Ryel", 0.79310346 },
  { "Ryelan", 0.93333334 },
  { "Ryeland", 1.0 },
  { "Ryelee", 0.21509434 },
  { "Ryeleigh", 0.0 },
  { "Ryeli", 0.0 },
  { "Ryelin", 0.6764706 },
  { "Ryelle", 0.0 },
  { "Ryelyn", 0.0 },
  { "Ryelynn", 0.0 },
  { "Ryen", 0.57993114 },
  { "Ryenn", 0.0 },
  { "Ryenne", 0.0 },
  { "Ryer", 1.0 },
  { "Ryerson", 1.0 },
  { "Ryesha", 0.0 },
  { "Ryett", 1.0 },
  { "Ryhan", 1.0 },
  { "Ryhana", 0.0 },
  { "Ryhanna", 0.0 },
  { "Ryheem", 1.0 },
  { "Ryheim", 1.0 },
  { "Ryia", 0.0 },
  { "Ryiah", 0.0 },
  { "Ryian", 0.4722222 },
  { "Ryilee", 0.0 },
  { "Ryin", 0.4453125 },
  { "Ryion", 1.0 },
  { "Ryka", 0.0 },
  { "Rykan", 1.0 },
  { "Rykar", 1.0 },
  { "Rykeem", 1.0 },
  { "Ryken", 0.99011856 },
  { "Ryker", 0.98840016 },
  { "Rykia", 0.0 },
  { "Rykin", 1.0 },
  { "Rykir", 1.0 },
  { "Rykker", 1.0 },
  { "Rykki", 0.0 },
  { "Ryko", 1.0 },
  { "Ryla", 0.0 },
  { "Rylah", 0.0 },
  { "Rylan", 0.86191726 },
  { "Ryland", 0.9478022 },
  { "Rylann", 0.09157509 },
  { "Ryle", 0.8856209 },
  { "Rylea", 0.0 },
  { "Ryleah", 0.0 },
  { "Rylee", 0.09007968 },
  { "Ryleeann", 0.0 },
  { "Ryleeanne", 0.0 },
  { "Ryleeh", 0.0 },
  { "Rylei", 0.068509616 },
  { "Ryleigh", 0.014751879 },
  { "Ryleighann", 0.0 },
  { "Rylen", 0.816101 },
  { "Rylend", 1.0 },
  { "Rylene", 0.0 },
  { "Rylenn", 0.0 },
  { "Ryler", 1.0 },
  { "Ryley", 0.55659777 },
  { "Ryli", 0.0 },
  { "Rylie", 0.058748625 },
  { "Rylieann", 0.0 },
  { "Ryliee", 0.0 },
  { "Ryliegh", 0.0 },
  { "Rylii", 0.0 },
  { "Rylin", 0.3885591 },
  { "Rylind", 1.0 },
  { "Rylinn", 0.0 },
  { "Ryllie", 0.0 },
  { "Rylo", 1.0 },
  { "Rylon", 1.0 },
  { "Rylund", 1.0 },
  { "Ryly", 0.45454547 },
  { "Rylyn", 0.20998278 },
  { "Rylynn", 0.042001236 },
  { "Rylynne", 0.0 },
  { "Ryma", 0.0 },
  { "Ryman", 0.9230769 },
  { "Ryn", 0.29761904 },
  { "Ryna", 0.25 },
  { "Rynda", 0.0 },
  { "Ryne", 0.99258727 },
  { "Ryneisha", 0.0 },
  { "Rynell", 1.0 },
  { "Ryner", 1.0 },
  { "Rynesha", 0.0 },
  { "Rynette", 0.0 },
  { "Ryniah", 0.0 },
  { "Rynisha", 0.0 },
  { "Rynlee", 0.0 },
  { "Rynleigh", 0.0 },
  { "Rynn", 0.078125 },
  { "Ryo", 1.0 },
  { "Ryoko", 0.0 },
  { "Ryoma", 1.0 },
  { "Ryon", 0.9936306 },
  { "Ryonna", 0.0 },
  { "Ryosuke", 1.0 },
  { "Ryot", 1.0 },
  { "Ryota", 1.0 },
  { "Ryotaro", 1.0 },
  { "Ryott", 1.0 },
  { "Ryou", 1.0 },
  { "Ryse", 1.0 },
  { "Rysen", 1.0 },
  { "Ryshaun", 1.0 },
  { "Ryshawn", 1.0 },
  { "Ryshon", 1.0 },
  { "Ryson", 1.0 },
  { "Ryssa", 0.0 },
  { "Rystal", 0.0 },
  { "Ryston", 1.0 },
  { "Ryszard", 1.0 },
  { "Rythm", 1.0 },
  { "Ryton", 1.0 },
  { "Ryu", 1.0 },
  { "Ryuichi", 1.0 },
  { "Ryuji", 1.0 },
  { "Ryujin", 1.0 },
  { "Ryuki", 1.0 },
  { "Ryun", 1.0 },
  { "Ryunosuke", 1.0 },
  { "Ryusei", 1.0 },
  { "Ryuu", 1.0 },
  { "Ryver", 0.6388262 },
  { "Ryyan", 1.0 },
  { "Ryze", 1.0 },
  { "Ryzen", 1.0 },
  { "Ryzer", 1.0 },
  { "Sa", 0.3 },
  { "Saabir", 1.0 },
  { "Saachi", 0.0 },
  { "Saad", 1.0 },
  { "Saada", 0.0 },
  { "Saadia", 0.0 },
  { "Saadiq", 1.0 },
  { "Saadiya", 0.0 },
  { "Saadiyah", 0.0 },
  { "Saafir", 1.0 },
  { "Saagar", 1.0 },
  { "Saahas", 1.0 },
  { "Saahil", 1.0 },
  { "Saahir", 1.0 },
  { "Saahithi", 0.0 },
  { "Saahiti", 0.0 },
  { "Saaid", 1.0 },
  { "Saaim", 1.0 },
  { "Saajan", 1.0 },
  { "Saajid", 1.0 },
  { "Saajida", 0.0 },
  { "Saaketh", 1.0 },
  { "Saakshi", 0.0 },
  { "Saalih", 1.0 },
  { "Saaliha", 0.0 },
  { "Saalihah", 0.0 },
  { "Saalim", 1.0 },
  { "Saaliyah", 0.0 },
  { "Saam", 1.0 },
  { "Saamia", 0.0 },
  { "Saamir", 1.0 },
  { "Saamiya", 0.0 },
  { "Saamiyah", 0.0 },
  { "Saamya", 0.0 },
  { "Saana", 0.0 },
  { "Saanchi", 0.0 },
  { "Saani", 0.0 },
  { "Saania", 0.0 },
  { "Saanika", 0.0 },
  { "Saaniya", 0.0 },
  { "Saaniyah", 0.0 },
  { "Saanjh", 0.0 },
  { "Saanvi", 0.0 },
  { "Saanvika", 0.0 },
  { "Saanya", 0.0 },
  { "Saaphyri", 0.0 },
  { "Saara", 0.0 },
  { "Saarah", 0.0 },
  { "Saarang", 1.0 },
  { "Saarim", 1.0 },
  { "Saarth", 1.0 },
  { "Saasha", 0.0 },
  { "Saathvik", 1.0 },
  { "Saatvik", 1.0 },
  { "Saavan", 1.0 },
  { "Saavi", 0.0 },
  { "Saavon", 1.0 },
  { "Saaya", 0.0 },
  { "Saba", 0.04876325 },
  { "Sabah", 0.0 },
  { "Saban", 1.0 },
  { "Sabarin", 0.0 },
  { "Sabas", 1.0 },
  { "Sabastain", 1.0 },
  { "Sabastian", 1.0 },
  { "Sabastien", 1.0 },
  { "Sabastin", 1.0 },
  { "Sabastion", 1.0 },
  { "Sabatino", 1.0 },
  { "Sabato", 1.0 },
  { "Sabbath", 1.0 },
  { "Sabbir", 1.0 },
  { "Sabdiel", 1.0 },
  { "Sabe", 1.0 },
  { "Sabeeh", 1.0 },
  { "Sabeen", 0.0 },
  { "Sabeena", 0.0 },
  { "Sabel", 0.0 },
  { "Sabela", 0.0 },
  { "Sabella", 0.0 },
  { "Sabelle", 0.0 },
  { "Saben", 1.0 },
  { "Sabena", 0.0 },
  { "Saber", 0.84518826 },
  { "Saberina", 0.0 },
  { "Sabha", 0.0 },
  { "Sabia", 0.0 },
  { "Sabian", 0.9750297 },
  { "Sabiel", 1.0 },
  { "Sabien", 1.0 },
  { "Sabih", 1.0 },
  { "Sabiha", 0.0 },
  { "Sabin", 0.9906977 },
  { "Sabina", 0.0 },
  { "Sabine", 0.0 },
  { "Sabino", 1.0 },
  { "Sabion", 1.0 },
  { "Sabir", 1.0 },
  { "Sabira", 0.0 },
  { "Sabirah", 0.0 },
  { "Sabirin", 0.0 },
  { "Sabit", 1.0 },
  { "Sabiya", 0.0 },
  { "Sable", 0.0 },
  { "Saboor", 1.0 },
  { "Sabra", 0.0 },
  { "Sabrah", 0.0 },
  { "Sabre", 0.12065814 },
  { "Sabrea", 0.0 },
  { "Sabreana", 0.0 },
  { "Sabree", 0.06271777 },
  { "Sabreea", 0.0 },
  { "Sabreen", 0.0 },
  { "Sabreena", 0.0 },
  { "Sabrena", 0.0 },
  { "Sabrenia", 0.0 },
  { "Sabrenna", 0.0 },
  { "Sabreya", 0.0 },
  { "Sabri", 0.8156425 },
  { "Sabria", 0.0 },
  { "Sabriah", 0.0 },
  { "Sabrian", 0.0 },
  { "Sabriana", 0.0 },
  { "Sabrie", 0.0 },
  { "Sabriel", 0.2746479 },
  { "Sabrielle", 0.0 },
  { "Sabriena", 0.0 },
  { "Sabrin", 0.0 },
  { "Sabrina", 0.002565369 },
  { "Sabrinah", 0.0 },
  { "Sabrinamarie", 0.0 },
  { "Sabrine", 0.0 },
  { "Sabrinia", 0.0 },
  { "Sabrinna", 0.0 },
  { "Sabriya", 0.0 },
  { "Sabriyah", 0.0 },
  { "Sabriye", 0.0 },
  { "Sabriyya", 0.0 },
  { "Sabriyyah", 0.0 },
  { "Sabryn", 0.0 },
  { "Sabryna", 0.0 },
  { "Sabrynn", 0.0 },
  { "Sabu", 1.0 },
  { "Sabur", 1.0 },
  { "Saburo", 1.0 },
  { "Saby", 0.0 },
  { "Sacajawea", 0.0 },
  { "Sacaria", 0.0 },
  { "Sacario", 1.0 },
  { "Sacha", 0.18833694 },
  { "Sachary", 0.5 },
  { "Sache", 0.0 },
  { "Sacheen", 0.0 },
  { "Sachel", 0.0 },
  { "Sachelle", 0.0 },
  { "Sachet", 0.0 },
  { "Sachi", 0.0 },
  { "Sachie", 0.0 },
  { "Sachiel", 1.0 },
  { "Sachika", 0.0 },
  { "Sachiko", 0.0 },
  { "Sachin", 1.0 },
  { "Sachit", 1.0 },
  { "Sachiye", 0.0 },
  { "Sachiyo", 0.0 },
  { "Sacia", 0.0 },
  { "Sacora", 0.0 },
  { "Sacoria", 0.0 },
  { "Sacorya", 0.0 },
  { "Sacoya", 0.0 },
  { "Sacoyia", 0.0 },
  { "Sacramento", 1.0 },
  { "Sacred", 0.0 },
  { "Sada", 0.0 },
  { "Sadae", 0.0 },
  { "Sadaf", 0.0 },
  { "Sadako", 0.0 },
  { "Sadam", 1.0 },
  { "Sadame", 0.0 },
  { "Sadamu", 1.0 },
  { "Sadan", 0.0 },
  { "Sadane", 1.0 },
  { "Sadao", 1.0 },
  { "Sadaq", 1.0 },
  { "Sadara", 0.0 },
  { "Sadaria", 0.0 },
  { "Sadarian", 1.0 },
  { "Sadarion", 1.0 },
  { "Sadarius", 1.0 },
  { "Sadat", 1.0 },
  { "Sadavia", 0.0 },
  { "Sadaya", 0.0 },
  { "Saddam", 1.0 },
  { "Saddie", 0.0 },
  { "Saddiq", 1.0 },
  { "Saddiya", 0.0 },
  { "Sade", 0.00921659 },
  { "Sadea", 0.0 },
  { "Sadee", 0.0 },
  { "Sadeel", 0.0 },
  { "Sadeem", 0.0 },
  { "Sadeen", 0.0 },
  { "Sadeeq", 1.0 },
  { "Sadeigh", 0.0 },
  { "Sadeja", 0.0 },
  { "Sadell", 0.0 },
  { "Sadelle", 0.0 },
  { "Sadena", 0.0 },
  { "Saderia", 0.0 },
  { "Sadey", 0.0 },
  { "Sadhana", 0.0 },
  { "Sadhika", 0.0 },
  { "Sadhvi", 0.0 },
  { "Sadi", 0.008431703 },
  { "Sadia", 0.0 },
  { "Sadiah", 0.0 },
  { "Sadie", 0.0019853003 },
  { "Sadieann", 0.0 },
  { "Sadiee", 0.0 },
  { "Sadiegrace", 0.0 },
  { "Sadiel", 1.0 },
  { "Sadielee", 0.0 },
  { "Sadielynn", 0.0 },
  { "Sadiemae", 0.0 },
  { "Sadiga", 0.0 },
  { "Sadik", 1.0 },
  { "Sadika", 0.0 },
  { "Sadiki", 1.0 },
  { "Sadina", 0.0 },
  { "Sadio", 0.0 },
  { "Sadiq", 1.0 },
  { "Sadiqa", 0.0 },
  { "Sadiqua", 0.0 },
  { "Sadique", 1.0 },
  { "Sadir", 1.0 },
  { "Sadira", 0.0 },
  { "Sadiya", 0.0 },
  { "Sadiyah", 0.0 },
  { "Sadiyya", 0.0 },
  { "Sadiyyah", 0.0 },
  { "Sadler", 0.95454544 },
  { "Sadman", 1.0 },
  { "Sadonia", 0.0 },
  { "Sadonna", 0.0 },
  { "Sadonte", 1.0 },
  { "Sadora", 0.0 },
  { "Sadra", 0.0 },
  { "Sadrac", 1.0 },
  { "Sady", 0.0 },
  { "Sadye", 0.0 },
  { "Sae", 0.06849315 },
  { "Saed", 1.0 },
  { "Saedee", 0.0 },
  { "Saedi", 0.0 },
  { "Saedie", 0.0 },
  { "Saee", 0.0 },
  { "Saeed", 1.0 },
  { "Saeeda", 0.0 },
  { "Saegan", 0.0 },
  { "Saeko", 0.0 },
  { "Sael", 1.0 },
  { "Saela", 0.0 },
  { "Saelah", 0.0 },
  { "Saelor", 0.0 },
  { "Saequan", 1.0 },
  { "Saesha", 0.0 },
  { "Saevion", 1.0 },
  { "Saevon", 1.0 },
  { "Safa", 0.0 },
  { "Safaa", 0.0 },
  { "Safah", 0.0 },
  { "Safal", 1.0 },
  { "Safana", 0.0 },
  { "Safaree", 1.0 },
  { "Safari", 0.07083333 },
  { "Safeara", 0.0 },
  { "Safee", 1.0 },
  { "Safeer", 1.0 },
  { "Safeerah", 0.0 },
  { "Safet", 1.0 },
  { "Safeya", 0.0 },
  { "Saffa", 0.0 },
  { "Saffire", 0.0 },
  { "Saffiya", 0.0 },
  { "Saffiyah", 0.0 },
  { "Saffron", 0.0 },
  { "Safi", 0.8962963 },
  { "Safia", 0.0 },
  { "Safiah", 0.0 },
  { "Safiatou", 0.0 },
  { "Safin", 1.0 },
  { "Safina", 0.0 },
  { "Safir", 1.0 },
  { "Safira", 0.0 },
  { "Safire", 0.0 },
  { "Safiya", 0.0 },
  { "Safiyah", 0.0 },
  { "Safiye", 0.0 },
  { "Safiyo", 0.0 },
  { "Safiyya", 0.0 },
  { "Safiyyah", 0.0 },
  { "Safoora", 0.0 },
  { "Saford", 1.0 },
  { "Safreen", 0.0 },
  { "Safronia", 0.0 },
  { "Safura", 0.0 },
  { "Safwaan", 1.0 },
  { "Safwan", 1.0 },
  { "Safwat", 1.0 },
  { "Saga", 0.0 },
  { "Sagal", 0.0 },
  { "Sagan", 0.58847183 },
  { "Sagar", 1.0 },
  { "Sagarika", 0.0 },
  { "Sage", 0.36329865 },
  { "Sagen", 0.5208333 },
  { "Sager", 1.0 },
  { "Sagine", 0.0 },
  { "Sagrario", 0.0 },
  { "Sahaan", 1.0 },
  { "Sahaana", 0.0 },
  { "Sahai", 0.0 },
  { "Sahaira", 0.0 },
  { "Sahaj", 1.0 },
  { "Sahal", 1.0 },
  { "Sahalie", 0.0 },
  { "Saham", 0.0 },
  { "Sahan", 1.0 },
  { "Sahana", 0.0 },
  { "Sahand", 1.0 },
  { "Sahani", 0.0 },
  { "Sahanna", 0.0 },
  { "Sahar", 0.011335634 },
  { "Sahara", 0.0013970382 },
  { "Saharah", 0.0 },
  { "Sahari", 0.0 },
  { "Saharra", 0.0 },
  { "Saharrah", 0.0 },
  { "Saharsh", 1.0 },
  { "Sahas", 1.0 },
  { "Sahasra", 0.0 },
  { "Saheb", 1.0 },
  { "Saheed", 1.0 },
  { "Saheem", 1.0 },
  { "Saheim", 1.0 },
  { "Sahej", 0.40816328 },
  { "Sahel", 1.0 },
  { "Saheli", 0.0 },
  { "Sahen", 1.0 },
  { "Saher", 0.02793296 },
  { "Sahian", 0.0 },
  { "Sahib", 0.969697 },
  { "Sahiba", 0.0 },
  { "Sahibdeep", 1.0 },
  { "Sahibjot", 1.0 },
  { "Sahid", 1.0 },
  { "Sahil", 1.0 },
  { "Sahily", 0.0 },
  { "Sahim", 1.0 },
  { "Sahir", 1.0 },
  { "Sahira", 0.0 },
  { "Sahirah", 0.0 },
  { "Sahiry", 0.0 },
  { "Sahishnu", 1.0 },
  { "Sahit", 1.0 },
  { "Sahith", 1.0 },
  { "Sahithi", 0.0 },
  { "Sahiti", 0.0 },
  { "Sahitya", 0.0 },
  { "Sahiyah", 0.0 },
  { "Sahl", 1.0 },
  { "Sahmir", 1.0 },
  { "Sahmiya", 0.0 },
  { "Sahmiyah", 0.0 },
  { "Sahmya", 0.0 },
  { "Sahni", 0.0 },
  { "Sahniya", 0.0 },
  { "Sahniyah", 0.0 },
  { "Sahnnon", 0.0 },
  { "Sahnya", 0.0 },
  { "Sahori", 0.0 },
  { "Sahory", 0.0 },
  { "Sahr", 1.0 },
  { "Sahra", 0.0 },
  { "Sahrai", 0.0 },
  { "Sahri", 0.0 },
  { "Sahrish", 0.0 },
  { "Sahriya", 0.0 },
  { "Sahriyah", 0.0 },
  { "Sahron", 0.0 },
  { "Sahrye", 0.0 },
  { "Sahvana", 0.0 },
  { "Sahvanna", 0.0 },
  { "Sahvannah", 0.0 },
  { "Sahwn", 1.0 },
  { "Sahyra", 0.0 },
  { "Sai", 0.7643678 },
  { "Saia", 0.055555556 },
  { "Saiah", 0.0 },
  { "Saianish", 1.0 },
  { "Saiansh", 1.0 },
  { "Saicharan", 1.0 },
  { "Saichi", 1.0 },
  { "Said", 1.0 },
  { "Saida", 0.0 },
  { "Saidah", 0.0 },
  { "Saide", 0.0 },
  { "Saidee", 0.0 },
  { "Saiden", 1.0 },
  { "Saidey", 0.0 },
  { "Saidi", 0.0 },
  { "Saidie", 0.0 },
  { "Saidou", 1.0 },
  { "Saidy", 0.0 },
  { "Saied", 1.0 },
  { "Saif", 1.0 },
  { "Saifaldeen", 1.0 },
  { "Saifan", 1.0 },
  { "Saifuddin", 1.0 },
  { "Saifullah", 1.0 },
  { "Saige", 0.09279315 },
  { "Saiid", 1.0 },
  { "Saija", 0.0 },
  { "Saije", 0.0 },
  { "Saikou", 1.0 },
  { "Saikrishna", 1.0 },
  { "Saila", 0.0 },
  { "Saile", 0.0 },
  { "Sailee", 0.0 },
  { "Sailer", 0.0 },
  { "Sailesh", 1.0 },
  { "Sailor", 0.15321806 },
  { "Saily", 0.0 },
  { "Sailyn", 0.0 },
  { "Saim", 1.0 },
  { "Saima", 0.0 },
  { "Saimi", 0.0 },
  { "Saimon", 1.0 },
  { "Saina", 0.0 },
  { "Sainabou", 0.0 },
  { "Saindhavi", 0.0 },
  { "Saint", 0.96590906 },
  { "Saintclair", 1.0 },
  { "Saintjohn", 1.0 },
  { "Saion", 1.0 },
  { "Saiquan", 1.0 },
  { "Sair", 1.0 },
  { "Saira", 0.0 },
  { "Sairah", 0.0 },
  { "Sairam", 1.0 },
  { "Sairi", 0.0 },
  { "Sairy", 0.0 },
  { "Saish", 1.0 },
  { "Saisha", 0.0 },
  { "Sait", 1.0 },
  { "Saith", 1.0 },
  { "Saiveon", 1.0 },
  { "Saivion", 1.0 },
  { "Saivon", 1.0 },
  { "Saiya", 0.0 },
  { "Saiyan", 1.0 },
  { "Saiyuri", 0.0 },
  { "Saja", 0.0 },
  { "Sajaad", 1.0 },
  { "Sajad", 1.0 },
  { "Sajada", 0.0 },
  { "Sajah", 0.0 },
  { "Sajal", 1.0 },
  { "Sajan", 1.0 },
  { "Sajani", 0.0 },
  { "Sajata", 0.0 },
  { "Sajda", 0.0 },
  { "Sajdah", 0.0 },
  { "Saje", 0.31932774 },
  { "Sajed", 1.0 },
  { "Sajeda", 0.0 },
  { "Sajen", 1.0 },
  { "Saji", 1.0 },
  { "Sajid", 1.0 },
  { "Sajidah", 0.0 },
  { "Sajjad", 1.0 },
  { "Sajni", 0.0 },
  { "Sakae", 0.5043478 },
  { "Sakai", 0.7172996 },
  { "Sakaiya", 0.0 },
  { "Sakara", 0.0 },
  { "Sakari", 0.13822894 },
  { "Sakaria", 1.0 },
  { "Sakariya", 1.0 },
  { "Sakariye", 1.0 },
  { "Sakaye", 0.0 },
  { "Sakayla", 0.0 },
  { "Sakeef", 1.0 },
  { "Sakeena", 0.0 },
  { "Sakeenah", 0.0 },
  { "Sakeithia", 0.0 },
  { "Sakena", 0.0 },
  { "Saket", 1.0 },
  { "Saketh", 1.0 },
  { "Saketha", 0.0 },
  { "Sakeya", 0.0 },
  { "Sakhi", 0.5652174 },
  { "Sakhia", 0.0 },
  { "Saki", 0.037542664 },
  { "Sakia", 0.0 },
  { "Sakiah", 0.0 },
  { "Sakib", 1.0 },
  { "Sakiko", 0.0 },
  { "Sakile", 0.0 },
  { "Sakima", 0.6666667 },
  { "Sakina", 0.0 },
  { "Sakinah", 0.0 },
  { "Sakira", 0.0 },
  { "Sakisha", 0.0 },
  { "Sakita", 0.0 },
  { "Sakiya", 0.0 },
  { "Sakiyah", 0.0 },
  { "Sakora", 0.0 },
  { "Sakoya", 0.0 },
  { "Saksham", 1.0 },
  { "Sakshi", 0.0 },
  { "Sakthi", 0.0 },
  { "Saku", 1.0 },
  { "Sakura", 0.0 },
  { "Sakurako", 0.0 },
  { "Sakya", 0.0 },
  { "Sakyra", 0.0 },
  { "Sal", 1.0 },
  { "Sala", 0.0 },
  { "Salaam", 1.0 },
  { "Salaar", 1.0 },
  { "Saladin", 1.0 },
  { "Saladine", 1.0 },
  { "Salah", 0.9086093 },
  { "Salahaldeen", 1.0 },
  { "Salahaldin", 1.0 },
  { "Salaheddine", 1.0 },
  { "Salahuddin", 1.0 },
  { "Salahudeen", 1.0 },
  { "Salahudin", 1.0 },
  { "Salam", 0.29220778 },
  { "Salama", 0.0 },
  { "Salamasina", 0.0 },
  { "Salamata", 0.0 },
  { "Salamon", 1.0 },
  { "Salana", 0.0 },
  { "Salanda", 0.0 },
  { "Salandra", 0.0 },
  { "Salar", 1.0 },
  { "Salara", 0.0 },
  { "Salathia", 0.0 },
  { "Salathiel", 1.0 },
  { "Salatiel", 1.0 },
  { "Salay", 0.0 },
  { "Salaya", 0.0 },
  { "Salayah", 0.0 },
  { "Salayna", 0.0 },
  { "Salazar", 1.0 },
  { "Salbador", 1.0 },
  { "Salea", 0.0 },
  { "Saleah", 0.0 },
  { "Saleana", 0.0 },
  { "Saleem", 1.0 },
  { "Saleema", 0.0 },
  { "Saleemah", 0.0 },
  { "Saleen", 0.0 },
  { "Saleena", 0.0 },
  { "Saleh", 1.0 },
  { "Saleha", 0.0 },
  { "Saleigh", 0.0 },
  { "Saleisha", 0.0 },
  { "Salem", 0.51491684 },
  { "Salema", 0.0 },
  { "Salena", 0.0 },
  { "Salene", 0.0 },
  { "Salesi", 1.0 },
  { "Saleste", 0.0 },
  { "Salet", 0.0 },
  { "Saleta", 0.0 },
  { "Saleth", 0.0 },
  { "Salethia", 0.0 },
  { "Salette", 0.0 },
  { "Saleyah", 0.0 },
  { "Sali", 0.0 },
  { "Salia", 0.0 },
  { "Saliah", 0.0 },
  { "Salice", 0.0 },
  { "Salicia", 0.0 },
  { "Salida", 0.0 },
  { "Salif", 1.0 },
  { "Salih", 1.0 },
  { "Saliha", 0.0 },
  { "Salihah", 0.0 },
  { "Salik", 1.0 },
  { "Salil", 1.0 },
  { "Salim", 1.0 },
  { "Salima", 0.0 },
  { "Salimah", 0.0 },
  { "Salimata", 0.0 },
  { "Salimatou", 0.0 },
  { "Salina", 0.0 },
  { "Salinda", 0.0 },
  { "Saline", 0.0 },
  { "Salinger", 1.0 },
  { "Saliou", 1.0 },
  { "Salis", 1.0 },
  { "Salisa", 0.0 },
  { "Salise", 0.0 },
  { "Salisha", 0.0 },
  { "Salissa", 0.0 },
  { "Salita", 0.0 },
  { "Salix", 1.0 },
  { "Saliya", 0.0 },
  { "Saliyah", 0.0 },
  { "Saliym", 1.0 },
  { "Salle", 0.0 },
  { "Sallee", 0.0 },
  { "Salley", 0.0 },
  { "Salli", 0.0 },
  { "Sallie", 0.0027060073 },
  { "Sally", 0.0031457173 },
  { "Sallyann", 0.0 },
  { "Sallyanne", 0.0 },
  { "Sallye", 0.0 },
  { "Sallyjo", 0.0 },
  { "Salma", 0.00065858796 },
  { "Salmaan", 1.0 },
  { "Salmah", 0.0 },
  { "Salmai", 0.0 },
  { "Salman", 1.0 },
  { "Salmo", 0.0 },
  { "Salmon", 1.0 },
  { "Saloma", 0.0 },
  { "Salomae", 0.0 },
  { "Salome", 0.16880181 },
  { "Salomea", 0.0 },
  { "Salomon", 1.0 },
  { "Salonda", 0.0 },
  { "Salone", 0.0 },
  { "Salonge", 0.0 },
  { "Saloni", 0.0 },
  { "Salote", 0.0 },
  { "Salsabeel", 0.0 },
  { "Salsabil", 0.0 },
  { "Salud", 0.5 },
  { "Salvado", 1.0 },
  { "Salvador", 0.9960568 },
  { "Salvadora", 0.0 },
  { "Salvadore", 1.0 },
  { "Salvator", 1.0 },
  { "Salvatora", 0.0 },
  { "Salvatore", 0.9985314 },
  { "Salvatrice", 0.0 },
  { "Salvina", 0.0 },
  { "Salvotore", 1.0 },
  { "Salwa", 0.0 },
  { "Saly", 0.0 },
  { "Salyna", 0.0 },
  { "Sam", 0.98857296 },
  { "Sama", 0.0 },
  { "Samaa", 0.0 },
  { "Samaad", 1.0 },
  { "Samaah", 0.0 },
  { "Samaara", 0.0 },
  { "Samad", 1.0 },
  { "Samadhi", 0.0 },
  { "Samael", 1.0 },
  { "Samah", 0.0 },
  { "Samahj", 1.0 },
  { "Samai", 0.0 },
  { "Samaia", 0.0 },
  { "Samaiah", 0.0 },
  { "Samaira", 0.0 },
  { "Samaire", 0.0 },
  { "Samaiya", 0.0 },
  { "Samaiyah", 0.0 },
  { "Samaj", 0.93594307 },
  { "Samajae", 1.0 },
  { "Samajay", 1.0 },
  { "Samaje", 1.0 },
  { "Samala", 0.0 },
  { "Samalia", 0.0 },
  { "Samamtha", 0.0 },
  { "Saman", 0.6976744 },
  { "Samana", 0.0 },
  { "Samanatha", 0.0 },
  { "Samanda", 0.0 },
  { "Samandar", 1.0 },
  { "Samani", 0.0 },
  { "Samanntha", 0.0 },
  { "Samanta", 0.0 },
  { "Samantah", 0.0 },
  { "Samantaha", 0.0 },
  { "Samanth", 0.0 },
  { "Samantha", 0.0021109984 },
  { "Samanthaann", 0.0 },
  { "Samanthagrace", 0.0 },
  { "Samanthajean", 0.0 },
  { "Samanthajo", 0.0 },
  { "Samanthalee", 0.0 },
  { "Samanthalynn", 0.0 },
  { "Samanthamarie", 0.0 },
  { "Samanthan", 0.0 },
  { "Samanthanicole", 0.0 },
  { "Samanthe", 0.0 },
  { "Samanthea", 0.0 },
  { "Samanthia", 0.0 },
  { "Samanvi", 0.0 },
  { "Samanvita", 0.0 },
  { "Samanvitha", 0.0 },
  { "Samanyu", 1.0 },
  { "Samar", 0.42531815 },
  { "Samara", 0.0 },
  { "Samarah", 0.0 },
  { "Samare", 0.0 },
  { "Samarea", 0.0 },
  { "Samaree", 0.0 },
  { "Samari", 0.32619974 },
  { "Samaria", 0.0 },
  { "Samariah", 0.0 },
  { "Samarian", 0.0 },
  { "Samarie", 0.0 },
  { "Samario", 1.0 },
  { "Samarion", 1.0 },
  { "Samaris", 0.0 },
  { "Samariya", 0.0 },
  { "Samariyah", 0.0 },
  { "Samarjit", 1.0 },
  { "Samarpreet", 1.0 },
  { "Samarra", 0.0 },
  { "Samarrah", 0.0 },
  { "Samarri", 0.0 },
  { "Samarria", 0.0 },
  { "Samarth", 1.0 },
  { "Samartha", 0.0 },
  { "Samary", 0.0 },
  { "Samarya", 0.0 },
  { "Samata", 0.0 },
  { "Samatar", 1.0 },
  { "Samatha", 0.0 },
  { "Samaura", 0.0 },
  { "Samauri", 0.4368932 },
  { "Samauria", 0.0 },
  { "Samay", 0.9402391 },
  { "Samaya", 0.0 },
  { "Samayah", 0.0 },
  { "Samayia", 0.0 },
  { "Samayiah", 0.0 },
  { "Samayra", 0.0 },
  { "Samba", 1.0 },
  { "Sambath", 1.0 },
  { "Sambhav", 1.0 },
  { "Sambo", 1.0 },
  { "Samed", 1.0 },
  { "Samee", 1.0 },
  { "Sameed", 1.0 },
  { "Sameeha", 0.0 },
  { "Sameeksha", 0.0 },
  { "Sameem", 1.0 },
  { "Sameen", 0.08928572 },
  { "Sameena", 0.0 },
  { "Sameenah", 0.0 },
  { "Sameer", 1.0 },
  { "Sameera", 0.0 },
  { "Sameerah", 0.0 },
  { "Sameh", 1.0 },
  { "Samehesha", 0.0 },
  { "Sameia", 0.0 },
  { "Sameika", 0.0 },
  { "Sameir", 1.0 },
  { "Sameisha", 0.0 },
  { "Samej", 1.0 },
  { "Sameka", 0.0 },
  { "Samekia", 0.0 },
  { "Sameko", 0.0 },
  { "Samel", 1.0 },
  { "Samella", 0.0 },
  { "Samena", 0.0 },
  { "Samentha", 0.0 },
  { "Samer", 0.9963154 },
  { "Samera", 0.0 },
  { "Samerah", 0.0 },
  { "Samere", 1.0 },
  { "Sameria", 0.0 },
  { "Sameriah", 0.0 },
  { "Samerra", 0.0 },
  { "Samet", 1.0 },
  { "Sametra", 0.0 },
  { "Sametria", 0.0 },
  { "Sametta", 0.0 },
  { "Sameul", 1.0 },
  { "Sameya", 0.0 },
  { "Sameyah", 0.0 },
  { "Samhita", 0.0 },
  { "Samhith", 1.0 },
  { "Samhitha", 0.0 },
  { "Sami", 0.76536745 },
  { "Samia", 0.0 },
  { "Samiah", 0.0 },
  { "Samiaya", 0.0 },
  { "Samica", 0.0 },
  { "Samid", 1.0 },
  { "Samie", 0.7809406 },
  { "Samiel", 1.0 },
  { "Samien", 1.0 },
  { "Samier", 1.0 },
  { "Samiera", 0.0 },
  { "Samih", 1.0 },
  { "Samiha", 0.0 },
  { "Samihah", 0.0 },
  { "Samiir", 1.0 },
  { "Samiira", 0.0 },
  { "Samija", 0.0 },
  { "Samijah", 0.0 },
  { "Samijo", 0.0 },
  { "Samik", 1.0 },
  { "Samika", 0.0 },
  { "Samiksha", 0.0 },
  { "Samil", 1.0 },
  { "Samin", 1.0 },
  { "Samina", 0.0 },
  { "Saminah", 0.0 },
  { "Samir", 0.9920571 },
  { "Samira", 0.0 },
  { "Samirah", 0.0 },
  { "Samiria", 0.0 },
  { "Samirra", 0.0 },
  { "Samirrah", 0.0 },
  { "Samisha", 0.0 },
  { "Samisoni", 1.0 },
  { "Samit", 1.0 },
  { "Samita", 0.0 },
  { "Samith", 1.0 },
  { "Samiuela", 1.0 },
  { "Samiul", 1.0 },
  { "Samiullah", 1.0 },
  { "Samiya", 0.0 },
  { "Samiyah", 0.0 },
  { "Samiyha", 0.0 },
  { "Samiyyah", 0.0 },
  { "Samma", 0.0 },
  { "Sammael", 1.0 },
  { "Sammantha", 0.0 },
  { "Sammara", 0.0 },
  { "Sammatha", 0.0 },
  { "Sammer", 1.0 },
  { "Sammeul", 1.0 },
  { "Sammey", 1.0 },
  { "Sammi", 0.032530904 },
  { "Sammie", 0.73293334 },
  { "Sammiejo", 0.0 },
  { "Sammijo", 0.0 },
  { "Sammiyah", 0.0 },
  { "Sammual", 1.0 },
  { "Sammuel", 1.0 },
  { "Sammul", 1.0 },
  { "Sammy", 0.9643158 },
  { "Sammya", 0.0 },
  { "Sammye", 0.0 },
  { "Sammyjo", 0.0 },
  { "Samnang", 0.83233535 },
  { "Samnatha", 0.0 },
  { "Samoan", 0.0 },
  { "Samon", 0.14285715 },
  { "Samona", 0.0 },
  { "Samone", 0.0023900573 },
  { "Samoni", 0.0 },
  { "Samonie", 0.0 },
  { "Samonte", 1.0 },
  { "Samontha", 0.0 },
  { "Samora", 0.014084507 },
  { "Samorah", 0.0 },
  { "Samore", 0.0 },
  { "Samori", 1.0 },
  { "Samoria", 0.0 },
  { "Samory", 1.0 },
  { "Samoya", 0.0 },
  { "Samp", 1.0 },
  { "Sampath", 1.0 },
  { "Sampson", 1.0 },
  { "Samra", 0.0 },
  { "Samraat", 1.0 },
  { "Samrah", 0.0 },
  { "Samraj", 1.0 },
  { "Samrat", 1.0 },
  { "Samrath", 1.0 },
  { "Samrawit", 0.0 },
  { "Samreen", 0.0 },
  { "Samreet", 0.0 },
  { "Samriddhi", 0.0 },
  { "Samridh", 1.0 },
  { "Samridhi", 0.0 },
  { "Samrudh", 1.0 },
  { "Samsam", 0.0 },
  { "Samsara", 0.0 },
  { "Samson", 1.0 },
  { "Samtha", 0.0 },
  { "Samuael", 1.0 },
  { "Samual", 1.0 },
  { "Samuel", 0.9962334 },
  { "Samuela", 0.5423729 },
  { "Samueldavid", 1.0 },
  { "Samuele", 1.0 },
  { "Samuell", 1.0 },
  { "Samuella", 0.0 },
  { "Samuelle", 0.78 },
  { "Samuelu", 1.0 },
  { "Samuil", 1.0 },
  { "Samul", 1.0 },
  { "Samule", 1.0 },
  { "Samurai", 1.0 },
  { "Samus", 0.0 },
  { "Samvel", 1.0 },
  { "Samvit", 1.0 },
  { "Samwell", 1.0 },
  { "Samwise", 1.0 },
  { "Samy", 0.9923897 },
  { "Samya", 0.0 },
  { "Samyah", 0.0 },
  { "Samyak", 1.0 },
  { "Samyar", 1.0 },
  { "Samye", 0.0 },
  { "Samyha", 0.0 },
  { "Samyia", 0.0 },
  { "Samyiah", 0.0 },
  { "Samyla", 0.0 },
  { "Samyog", 1.0 },
  { "Samyr", 1.0 },
  { "Samyra", 0.0 },
  { "Samyrah", 0.0 },
  { "Samyria", 0.0 },
  { "Samyriah", 0.0 },
  { "Samyukta", 0.0 },
  { "Samyuktha", 0.0 },
  { "San", 0.24244899 },
  { "Sana", 0.0013915948 },
  { "Sanaa", 0.0 },
  { "Sanaah", 0.0 },
  { "Sanaai", 0.0 },
  { "Sanaaya", 0.0 },
  { "Sanad", 1.0 },
  { "Sanae", 0.0 },
  { "Sanah", 0.0 },
  { "Sanai", 0.006671114 },
  { "Sanaia", 0.0 },
  { "Sanaiah", 0.0 },
  { "Sanaii", 0.0 },
  { "Sanaiya", 0.0 },
  { "Sanaiyah", 0.0 },
  { "Sanam", 0.0 },
  { "Sanantha", 0.0 },
  { "Sanantonio", 1.0 },
  { "Sanara", 0.0 },
  { "Sanari", 0.0 },
  { "Sanaria", 0.0 },
  { "Sanat", 1.0 },
  { "Sanath", 1.0 },
  { "Sanav", 1.0 },
  { "Sanavi", 0.0 },
  { "Sanay", 0.80487806 },
  { "Sanaya", 0.0 },
  { "Sanayah", 0.0 },
  { "Sanaz", 0.0 },
  { "Sancha", 0.0 },
  { "Sanchez", 0.9814815 },
  { "Sanchi", 0.0 },
  { "Sanchit", 1.0 },
  { "Sancho", 1.0 },
  { "Sancia", 0.0 },
  { "Sand", 0.0 },
  { "Sanda", 0.0 },
  { "Sandal", 0.0 },
  { "Sandar", 0.0 },
  { "Sandara", 0.0 },
  { "Sande", 0.05839416 },
  { "Sandee", 0.0 },
  { "Sandeep", 0.8751357 },
  { "Sander", 0.9413512 },
  { "Sandera", 0.0 },
  { "Sanders", 0.9973475 },
  { "Sanderson", 1.0 },
  { "Sandesh", 1.0 },
  { "Sandford", 1.0 },
  { "Sandhya", 0.0 },
  { "Sandi", 0.002081475 },
  { "Sandia", 0.0 },
  { "Sandibel", 0.0 },
  { "Sandie", 0.0018389113 },
  { "Sandino", 1.0 },
  { "Sandip", 1.0 },
  { "Sandler", 1.0 },
  { "Sandon", 1.0 },
  { "Sandor", 1.0 },
  { "Sandora", 0.0 },
  { "Sandr", 0.0 },
  { "Sandra", 0.003074622 },
  { "Sandrah", 0.0 },
  { "Sandral", 0.0 },
  { "Sandralee", 0.0 },
  { "Sandre", 0.0 },
  { "Sandrea", 0.0 },
  { "Sandreka", 0.0 },
  { "Sandrell", 0.0 },
  { "Sandria", 0.0 },
  { "Sandricka", 0.0 },
  { "Sandrika", 0.0 },
  { "Sandrina", 0.0 },
  { "Sandrine", 0.0 },
  { "Sandro", 1.0 },
  { "Sandry", 0.0 },
  { "Sandy", 0.13157126 },
  { "Sandybell", 0.0 },
  { "Sandye", 0.0 },
  { "Sanea", 0.0 },
  { "Saned", 0.0 },
  { "Sanel", 1.0 },
  { "Sanela", 0.0 },
  { "Sanetra", 0.0 },
  { "Sanetta", 0.0 },
  { "Saneya", 0.0 },
  { "Sanford", 1.0 },
  { "Sanfra", 0.0 },
  { "Sang", 0.9313869 },
  { "Sangeeta", 0.0 },
  { "Sangeetha", 0.0 },
  { "Sangita", 0.0 },
  { "Sangria", 0.0 },
  { "Sani", 0.2123656 },
  { "Sania", 0.0 },
  { "Saniaa", 0.0 },
  { "Saniah", 0.0 },
  { "Saniaya", 0.0 },
  { "Sanibel", 0.0 },
  { "Sanieya", 0.0 },
  { "Sanihya", 0.0 },
  { "Sanii", 0.0 },
  { "Saniia", 0.0 },
  { "Saniiya", 0.0 },
  { "Saniiyah", 0.0 },
  { "Sanija", 0.0 },
  { "Sanijah", 0.0 },
  { "Sanika", 0.0 },
  { "Sanil", 1.0 },
  { "Sanilah", 0.0 },
  { "Sanin", 1.0 },
  { "Saniqua", 0.0 },
  { "Sanira", 0.0 },
  { "Sanisha", 0.0 },
  { "Sanita", 0.0 },
  { "Sanitra", 0.0 },
  { "Saniy", 0.0 },
  { "Saniya", 0.0 },
  { "Saniyaa", 0.0 },
  { "Saniyah", 0.0 },
  { "Saniye", 0.0 },
  { "Saniyha", 0.0 },
  { "Saniyia", 0.0 },
  { "Saniyya", 0.0 },
  { "Saniyyah", 0.0 },
  { "Sanja", 0.0 },
  { "Sanjan", 1.0 },
  { "Sanjana", 0.0 },
  { "Sanjay", 1.0 },
  { "Sanjaya", 0.7346939 },
  { "Sanjeet", 1.0 },
  { "Sanjeev", 1.0 },
  { "Sanjida", 0.0 },
  { "Sanjit", 1.0 },
  { "Sanjita", 0.0 },
  { "Sanjith", 1.0 },
  { "Sanjitha", 0.0 },
  { "Sanjiv", 1.0 },
  { "Sanjna", 0.0 },
  { "Sanjuan", 1.0 },
  { "Sanjuana", 0.0 },
  { "Sanjuanita", 0.0 },
  { "Sank", 1.0 },
  { "Sankalp", 1.0 },
  { "Sankeerth", 1.0 },
  { "Sanket", 1.0 },
  { "Sankey", 1.0 },
  { "Sanna", 0.0 },
  { "Sannah", 0.0 },
  { "Sannai", 0.0 },
  { "Sanne", 0.0 },
  { "Sanniah", 0.0 },
  { "Sannidhi", 0.0 },
  { "Sannie", 0.0 },
  { "Sanniyah", 0.0 },
  { "Sannon", 0.0 },
  { "Sanny", 1.0 },
  { "Sanoe", 0.0 },
  { "Sanora", 0.0 },
  { "Sanovia", 0.0 },
  { "Sanquan", 1.0 },
  { "Sanquenetta", 0.0 },
  { "Sanquetta", 0.0 },
  { "Sanquita", 0.0 },
  { "Sansa", 0.0 },
  { "Sanskar", 1.0 },
  { "Sanskriti", 0.0 },
  { "Sanskruti", 0.0 },
  { "Santa", 0.009287206 },
  { "Santana", 0.476576 },
  { "Santanah", 0.0 },
  { "Santangela", 0.0 },
  { "Santangelo", 1.0 },
  { "Santania", 0.0 },
  { "Santanna", 0.06944445 },
  { "Santanya", 0.0 },
  { "Santara", 0.0 },
  { "Santario", 1.0 },
  { "Santasha", 0.0 },
  { "Santasia", 0.0 },
  { "Santavia", 0.0 },
  { "Santavious", 1.0 },
  { "Sante", 1.0 },
  { "Santee", 1.0 },
  { "Santeria", 0.0 },
  { "Santerica", 0.0 },
  { "Santerria", 0.0 },
  { "Santez", 1.0 },
  { "Santford", 1.0 },
  { "Santha", 0.0 },
  { "Santhiago", 1.0 },
  { "Santhosh", 1.0 },
  { "Santi", 0.8695652 },
  { "Santia", 0.0 },
  { "Santiaga", 0.0 },
  { "Santiago", 0.99821925 },
  { "Santiana", 0.0 },
  { "Santiel", 1.0 },
  { "Santigo", 1.0 },
  { "Santina", 0.0 },
  { "Santini", 1.0 },
  { "Santino", 1.0 },
  { "Santita", 0.0 },
  { "Santo", 1.0 },
  { "Santoi", 1.0 },
  { "Santonia", 0.52380955 },
  { "Santonio", 1.0 },
  { "Santoria", 0.0 },
  { "Santos", 0.856294 },
  { "Santosh", 1.0 },
  { "Santosha", 0.0 },
  { "Santoya", 0.0 },
  { "Santrell", 0.5 },
  { "Santresa", 0.0 },
  { "Santrese", 0.0 },
  { "Santrez", 1.0 },
  { "Santrice", 0.0 },
  { "Santricia", 0.0 },
  { "Santwan", 1.0 },
  { "Santy", 1.0 },
  { "Sanuel", 1.0 },
  { "Sanura", 0.0 },
  { "Sanvi", 0.0 },
  { "Sanvika", 0.0 },
  { "Sanvitha", 0.0 },
  { "Sanya", 0.0 },
  { "Sanyah", 0.0 },
  { "Sanye", 0.0 },
  { "Sanyi", 0.0 },
  { "Sanyia", 0.0 },
  { "Sanyiah", 0.0 },
  { "Sanyla", 0.0 },
  { "Sanylah", 0.0 },
  { "Sanyra", 0.0 },
  { "Sanyu", 0.0 },
  { "Sao", 1.0 },
  { "Saoirse", 0.0 },
  { "Saori", 0.0 },
  { "Saory", 0.0 },
  { "Sapan", 1.0 },
  { "Sapana", 0.0 },
  { "Saphera", 0.0 },
  { "Sapheria", 0.0 },
  { "Saphia", 0.0 },
  { "Saphina", 0.0 },
  { "Saphira", 0.0 },
  { "Saphirah", 0.0 },
  { "Saphire", 0.0 },
  { "Saphronia", 0.0 },
  { "Saphyra", 0.0 },
  { "Saphyre", 0.0 },
  { "Sapir", 0.0 },
  { "Sapna", 0.0 },
  { "Sapphira", 0.0 },
  { "Sapphire", 0.0 },
  { "Sapphyre", 0.0 },
  { "Saprina", 0.0 },
  { "Saqib", 1.0 },
  { "Saqqara", 0.0 },
  { "Saquan", 0.9902371 },
  { "Saquana", 0.0 },
  { "Saquanna", 0.0 },
  { "Saquita", 0.0 },
  { "Saquoia", 0.0 },
  { "Saquon", 1.0 },
  { "Saquoya", 0.0 },
  { "Sara", 0.0029289615 },
  { "Saraa", 0.0 },
  { "Saraann", 0.0 },
  { "Saraanne", 0.0 },
  { "Sarabel", 0.0 },
  { "Sarabella", 0.0 },
  { "Sarabelle", 0.0 },
  { "Sarabeth", 0.0 },
  { "Sarabi", 0.0 },
  { "Sarada", 0.0 },
  { "Sarae", 0.0 },
  { "Saraelizabeth", 0.0 },
  { "Sarafina", 0.0 },
  { "Saragosa", 1.0 },
  { "Saragrace", 0.0 },
  { "Sarah", 0.0030928622 },
  { "Saraha", 0.0 },
  { "Sarahann", 0.0 },
  { "Sarahanne", 0.0 },
  { "Sarahbella", 0.0 },
  { "Sarahbeth", 0.0 },
  { "Sarahelizabeth", 0.0 },
  { "Sarahgrace", 0.0 },
  { "Sarahi", 0.0 },
  { "Sarahjane", 0.0 },
  { "Sarahjean", 0.0 },
  { "Sarahjo", 0.0 },
  { "Sarahjoy", 0.0 },
  { "Sarahkate", 0.0 },
  { "Sarahlee", 0.0 },
  { "Sarahlyn", 0.0 },
  { "Sarahlynn", 0.0 },
  { "Sarahmae", 0.0 },
  { "Sarahmarie", 0.0 },
  { "Sarahrose", 0.0 },
  { "Sarahy", 0.0 },
  { "Sarahya", 0.0 },
  { "Sarai", 0.00036471948 },
  { "Saraia", 0.0 },
  { "Saraiah", 0.0 },
  { "Saraih", 0.0 },
  { "Saraii", 0.0 },
  { "Saraina", 0.0 },
  { "Sarait", 0.0 },
  { "Saraiya", 0.0 },
  { "Saraiyah", 0.0 },
  { "Sarajane", 0.0 },
  { "Sarajean", 0.0 },
  { "Saraji", 0.0 },
  { "Sarajo", 0.0 },
  { "Sarakate", 0.0 },
  { "Saral", 1.0 },
  { "Sarala", 0.0 },
  { "Saralee", 0.0 },
  { "Saralie", 0.0 },
  { "Saralou", 0.0 },
  { "Saraly", 0.0 },
  { "Saralyn", 0.0 },
  { "Saralynn", 0.0 },
  { "Saramae", 0.0 },
  { "Saramarie", 0.0 },
  { "Saran", 0.04244482 },
  { "Saranda", 0.0 },
  { "Sarang", 0.5068493 },
  { "Sarani", 0.0 },
  { "Sarann", 0.0 },
  { "Saranna", 0.0 },
  { "Saranne", 0.0 },
  { "Saransh", 1.0 },
  { "Saranya", 0.0 },
  { "Saraphina", 0.0 },
  { "Saraphine", 0.0 },
  { "Sararose", 0.0 },
  { "Saras", 1.0 },
  { "Saraswati", 0.0 },
  { "Sarath", 0.6969697 },
  { "Saratha", 0.0 },
  { "Saray", 0.0 },
  { "Saraya", 0.0 },
  { "Sarayah", 0.0 },
  { "Saraye", 0.0 },
  { "Sarayu", 0.0 },
  { "Sarbjot", 1.0 },
  { "Sarda", 0.0 },
  { "Sardar", 1.0 },
  { "Sarde", 0.0 },
  { "Sardor", 1.0 },
  { "Sare", 0.0 },
  { "Sarea", 0.0 },
  { "Saree", 0.0 },
  { "Sareen", 0.0 },
  { "Sareena", 0.0 },
  { "Sareeta", 0.0 },
  { "Sarek", 1.0 },
  { "Sarela", 0.0 },
  { "Sareli", 0.0 },
  { "Sarell", 0.0 },
  { "Sarely", 0.0 },
  { "Sarem", 0.0 },
  { "Saren", 0.0 },
  { "Sarena", 0.0 },
  { "Sarene", 0.0 },
  { "Sareniti", 0.0 },
  { "Sarenity", 0.0 },
  { "Sarenna", 0.0 },
  { "Sarepta", 0.0 },
  { "Saresa", 0.0 },
  { "Sareth", 0.5 },
  { "Saretha", 0.0 },
  { "Saretta", 0.0 },
  { "Sareya", 0.0 },
  { "Sargent", 1.0 },
  { "Sargon", 1.0 },
  { "Sargun", 0.0 },
  { "Sarh", 0.0 },
  { "Sarha", 0.0 },
  { "Sarhiya", 0.0 },
  { "Sari", 0.010493827 },
  { "Saria", 0.0 },
  { "Sariah", 0.0 },
  { "Sariaha", 0.0 },
  { "Sariana", 0.0 },
  { "Sarianna", 0.0 },
  { "Saribel", 0.0 },
  { "Sarica", 0.0 },
  { "Sarie", 0.0 },
  { "Sariel", 1.0 },
  { "Sariha", 0.0 },
  { "Sarii", 0.0 },
  { "Sarika", 0.0 },
  { "Sarilyn", 0.0 },
  { "Sarim", 1.0 },
  { "Sarin", 0.08208955 },
  { "Sarina", 0.0 },
  { "Sarinah", 0.0 },
  { "Sarine", 0.0 },
  { "Sarinity", 0.0 },
  { "Sarinna", 0.0 },
  { "Sarisha", 0.0 },
  { "Sarissa", 0.0 },
  { "Sarit", 0.0 },
  { "Sarita", 0.0 },
  { "Sarith", 1.0 },
  { "Saritha", 0.0 },
  { "Sariya", 0.0 },
  { "Sariyah", 0.0 },
  { "Sariyha", 0.0 },
  { "Sarkis", 1.0 },
  { "Sarmad", 1.0 },
  { "Sarn", 1.0 },
  { "Sarne", 0.0 },
  { "Saro", 1.0 },
  { "Saroeun", 1.0 },
  { "Sarom", 0.6666667 },
  { "Saron", 0.16974169 },
  { "Saronda", 0.0 },
  { "Sarosh", 1.0 },
  { "Saroun", 0.0 },
  { "Saroya", 0.0 },
  { "Sarp", 1.0 },
  { "Sarra", 0.0 },
  { "Sarrah", 0.0 },
  { "Sarri", 0.0 },
  { "Sarriah", 0.0 },
  { "Sarrina", 0.0 },
  { "Sarrinah", 0.0 },
  { "Sarriyah", 0.0 },
  { "Sarsh", 0.0 },
  { "Sartaj", 1.0 },
  { "Sarthak", 1.0 },
  { "Sarva", 1.0 },
  { "Sarvesh", 1.0 },
  { "Sarvia", 0.0 },
  { "Sarvin", 1.0 },
  { "Sary", 0.06944445 },
  { "Sarya", 0.0 },
  { "Saryah", 0.0 },
  { "Saryan", 1.0 },
  { "Sarye", 0.0 },
  { "Saryia", 0.0 },
  { "Saryiah", 0.0 },
  { "Saryn", 0.0 },
  { "Saryna", 0.0 },
  { "Saryniti", 0.0 },
  { "Sarynity", 0.0 },
  { "Sasami", 0.0 },
  { "Sasan", 1.0 },
  { "Sascha", 0.24 },
  { "Sasha", 0.038444143 },
  { "Sashae", 0.0 },
  { "Sashalee", 0.0 },
  { "Sashank", 1.0 },
  { "Sasharae", 0.0 },
  { "Sashay", 0.0 },
  { "Sashe", 0.0 },
  { "Sasheen", 0.0 },
  { "Sashenka", 0.0 },
  { "Sashi", 0.0 },
  { "Sashia", 0.0 },
  { "Sashya", 0.0 },
  { "Sasia", 0.0 },
  { "Saskia", 0.0 },
  { "Sassy", 0.0 },
  { "Sasuke", 1.0 },
  { "Satanya", 0.0 },
  { "Satara", 0.0 },
  { "Sataria", 0.0 },
  { "Satasha", 0.0 },
  { "Satavia", 0.0 },
  { "Satcha", 0.0 },
  { "Satchel", 1.0 },
  { "Sateria", 0.0 },
  { "Sathvik", 1.0 },
  { "Sathvika", 0.0 },
  { "Sathwik", 1.0 },
  { "Sathya", 0.85365856 },
  { "Sati", 0.0 },
  { "Satia", 0.0 },
  { "Satin", 0.0 },
  { "Satina", 0.0 },
  { "Satine", 0.0 },
  { "Satira", 0.0 },
  { "Satish", 1.0 },
  { "Sativa", 0.0 },
  { "Satnam", 1.0 },
  { "Satoko", 0.0 },
  { "Satomi", 0.0 },
  { "Satonia", 0.0 },
  { "Satonya", 0.0 },
  { "Satori", 0.030487806 },
  { "Satoria", 0.0 },
  { "Satoru", 1.0 },
  { "Satoshi", 1.0 },
  { "Satoya", 0.0 },
  { "Satrina", 0.0 },
  { "Satsha", 0.0 },
  { "Satsuki", 0.0 },
  { "Satsuko", 0.0 },
  { "Sattam", 1.0 },
  { "Saturn", 0.0 },
  { "Saturnina", 0.0 },
  { "Saturnino", 1.0 },
  { "Satvik", 1.0 },
  { "Satvika", 0.0 },
  { "Satya", 0.3156733 },
  { "Satyam", 1.0 },
  { "Satyana", 0.0 },
  { "Satyn", 0.0 },
  { "Satyra", 0.0 },
  { "Sauannah", 0.0 },
  { "Saud", 1.0 },
  { "Sauda", 0.0 },
  { "Saudah", 0.0 },
  { "Saudi", 0.0 },
  { "Saudia", 0.0 },
  { "Sauel", 1.0 },
  { "Saugat", 1.0 },
  { "Saul", 0.9980784 },
  { "Saulius", 1.0 },
  { "Saulo", 1.0 },
  { "Saumil", 1.0 },
  { "Saumya", 0.02173913 },
  { "Saun", 1.0 },
  { "Sauna", 0.0 },
  { "Saunders", 1.0 },
  { "Saundra", 0.00061607396 },
  { "Saundria", 0.0 },
  { "Saunti", 0.0 },
  { "Saunya", 0.0 },
  { "Saura", 0.0 },
  { "Saurabh", 1.0 },
  { "Saurav", 1.0 },
  { "Saurish", 1.0 },
  { "Saurya", 1.0 },
  { "Sausha", 0.0 },
  { "Sava", 0.9593496 },
  { "Savada", 0.0 },
  { "Savaeh", 0.0 },
  { "Savage", 1.0 },
  { "Savahanna", 0.0 },
  { "Savahna", 0.0 },
  { "Savalas", 1.0 },
  { "Savan", 1.0 },
  { "Savana", 0.0 },
  { "Savanah", 0.0 },
  { "Savanaha", 0.0 },
  { "Savanh", 0.0 },
  { "Savanha", 0.0 },
  { "Savanna", 0.0005126102 },
  { "Savannah", 0.0012393512 },
  { "Savannaha", 0.0 },
  { "Savannahgrace", 0.0 },
  { "Savannahjo", 0.0 },
  { "Savannahlynn", 0.0 },
  { "Savannahmarie", 0.0 },
  { "Savannahrose", 0.0 },
  { "Savannha", 0.0 },
  { "Savant", 1.0 },
  { "Savante", 1.0 },
  { "Savar", 1.0 },
  { "Savas", 1.0 },
  { "Savasia", 0.0 },
  { "Savaughn", 0.96350366 },
  { "Savaya", 0.0 },
  { "Savayah", 0.0 },
  { "Savea", 0.0 },
  { "Saveah", 0.0 },
  { "Saveena", 0.0 },
  { "Saveer", 1.0 },
  { "Saveion", 1.0 },
  { "Saveliy", 1.0 },
  { "Savena", 0.0 },
  { "Saveon", 1.0 },
  { "Savera", 0.0 },
  { "Saveria", 0.0 },
  { "Saverio", 1.0 },
  { "Saveya", 0.0 },
  { "Saveyon", 1.0 },
  { "Savhanna", 0.0 },
  { "Savhannah", 0.0 },
  { "Savi", 0.0 },
  { "Saviah", 0.0 },
  { "Savian", 1.0 },
  { "Saviana", 0.0 },
  { "Saviel", 1.0 },
  { "Savien", 1.0 },
  { "Savieon", 1.0 },
  { "Savier", 1.0 },
  { "Savilla", 0.0 },
  { "Savin", 1.0 },
  { "Savina", 0.0 },
  { "Savine", 0.0 },
  { "Savino", 1.0 },
  { "Savio", 1.0 },
  { "Savion", 0.99588376 },
  { "Savione", 1.0 },
  { "Savior", 1.0 },
  { "Saviour", 1.0 },
  { "Savir", 1.0 },
  { "Savit", 1.0 },
  { "Savita", 0.0 },
  { "Savitri", 0.0 },
  { "Saviyon", 1.0 },
  { "Savoeun", 0.0 },
  { "Savon", 0.9577465 },
  { "Savona", 0.0 },
  { "Savone", 1.0 },
  { "Savonia", 0.0 },
  { "Savonn", 1.0 },
  { "Savonna", 0.0 },
  { "Savonnah", 0.0 },
  { "Savonne", 0.16129032 },
  { "Savonte", 1.0 },
  { "Savonya", 0.0 },
  { "Savoy", 0.93442625 },
  { "Savreen", 0.0 },
  { "Savva", 1.0 },
  { "Savvas", 1.0 },
  { "Savvy", 0.0 },
  { "Savy", 0.0 },
  { "Savya", 0.5 },
  { "Savyon", 1.0 },
  { "Saw", 1.0 },
  { "Sawan", 1.0 },
  { "Sawana", 0.0 },
  { "Sawanda", 0.0 },
  { "Sawda", 0.0 },
  { "Sawdah", 0.0 },
  { "Sawsan", 0.0 },
  { "Sawyer", 0.8368544 },
  { "Saxon", 0.8860182 },
  { "Saxton", 1.0 },
  { "Say", 0.3529412 },
  { "Saya", 0.0 },
  { "Sayah", 0.0 },
  { "Sayaka", 0.0 },
  { "Sayali", 0.0 },
  { "Sayan", 1.0 },
  { "Sayana", 0.0 },
  { "Sayani", 0.0 },
  { "Sayanna", 0.0 },
  { "Saybree", 0.0 },
  { "Sayd", 1.0 },
  { "Sayda", 0.0 },
  { "Sayde", 0.0 },
  { "Saydee", 0.0 },
  { "Sayden", 1.0 },
  { "Saydi", 0.0 },
  { "Saydie", 0.0 },
  { "Sayed", 1.0 },
  { "Sayeda", 0.0 },
  { "Sayeed", 1.0 },
  { "Sayeeda", 0.0 },
  { "Sayeh", 0.0 },
  { "Sayen", 0.0 },
  { "Sayer", 0.98407644 },
  { "Sayesha", 0.0 },
  { "Sayf", 1.0 },
  { "Sayge", 0.17156863 },
  { "Sayid", 1.0 },
  { "Sayla", 0.0 },
  { "Saylah", 0.0 },
  { "Saylar", 0.0 },
  { "Saylee", 0.0 },
  { "Saylem", 0.0 },
  { "Sayler", 0.04398148 },
  { "Sayli", 0.0 },
  { "Saylor", 0.08202765 },
  { "Saynab", 0.0 },
  { "Sayoko", 0.0 },
  { "Sayon", 1.0 },
  { "Sayona", 0.0 },
  { "Sayonara", 0.0 },
  { "Sayonna", 0.0 },
  { "Sayquan", 1.0 },
  { "Sayra", 0.0 },
  { "Sayre", 0.58389264 },
  { "Sayri", 0.0 },
  { "Saysha", 0.0 },
  { "Sayumi", 0.0 },
  { "Sayuri", 0.0 },
  { "Sayveon", 1.0 },
  { "Sayvion", 1.0 },
  { "Sayvon", 1.0 },
  { "Sayward", 0.0 },
  { "Sayyid", 1.0 },
  { "Sbeidy", 0.0 },
  { "Sbrina", 0.0 },
  { "Scan", 1.0 },
  { "Scarlet", 0.0 },
  { "Scarleth", 0.0 },
  { "Scarletrose", 0.0 },
  { "Scarlett", 0.00025350024 },
  { "Scarlette", 0.0 },
  { "Scarlettrose", 0.0 },
  { "Scarlotte", 0.0 },
  { "Scarlytt", 0.0 },
  { "Schad", 1.0 },
  { "Schae", 0.0 },
  { "Schaefer", 1.0 },
  { "Schaeffer", 1.0 },
  { "Schafer", 1.0 },
  { "Schaffer", 1.0 },
  { "Schandra", 0.0 },
  { "Schane", 1.0 },
  { "Schanel", 0.0 },
  { "Schanelle", 0.0 },
  { "Schannon", 0.0 },
  { "Schantel", 0.0 },
  { "Scharlene", 0.0 },
  { "Scharlotte", 0.0 },
  { "Scharron", 0.0 },
  { "Schaun", 1.0 },
  { "Schavon", 0.0 },
  { "Schawn", 1.0 },
  { "Scheherazade", 0.0 },
  { "Schelby", 0.0 },
  { "Schell", 0.5833333 },
  { "Schelle", 0.0 },
  { "Schelley", 0.0 },
  { "Schelly", 0.0 },
  { "Schenita", 0.0 },
  { "Scherri", 0.0 },
  { "Scherrie", 0.0 },
  { "Scherry", 0.0 },
  { "Scheryl", 0.0 },
  { "Schley", 1.0 },
  { "Schlonda", 0.0 },
  { "Schmeka", 0.0 },
  { "Schmika", 0.0 },
  { "Schneider", 1.0 },
  { "Schnell", 0.0 },
  { "Schneur", 1.0 },
  { "Scholar", 1.0 },
  { "Scholastica", 0.0 },
  { "Schon", 1.0 },
  { "Schrie", 0.0 },
  { "Schrita", 0.0 },
  { "Schronda", 0.0 },
  { "Schuylar", 0.39130434 },
  { "Schuyler", 0.771592 },
  { "Schwanda", 0.0 },
  { "Schwanna", 0.0 },
  { "Schylar", 0.32361516 },
  { "Schyler", 0.55865103 },
  { "Scion", 1.0 },
  { "Scipio", 1.0 },
  { "Scoey", 1.0 },
  { "Scooter", 1.0 },
  { "Scorpio", 1.0 },
  { "Scot", 0.9991966 },
  { "Scotia", 0.0 },
  { "Scotland", 0.45070422 },
  { "Scotlyn", 0.0 },
  { "Scotlynn", 0.0 },
  { "Scott", 0.9966228 },
  { "Scotte", 1.0 },
  { "Scotti", 0.20987654 },
  { "Scottia", 0.0 },
  { "Scottie", 0.8626677 },
  { "Scottland", 1.0 },
  { "Scottlyn", 0.0 },
  { "Scottlynn", 0.0 },
  { "Scotty", 0.9876108 },
  { "Scout", 0.30589256 },
  { "Scyler", 0.7058824 },
  { "Se", 0.33333334 },
  { "Sea", 0.52380955 },
  { "Seaanna", 0.0 },
  { "Seab", 1.0 },
  { "Seaborn", 1.0 },
  { "Seabron", 1.0 },
  { "Seabrook", 1.0 },
  { "Seager", 1.0 },
  { "Seaira", 0.0 },
  { "Seairah", 0.0 },
  { "Seairra", 0.0 },
  { "Seals", 1.0 },
  { "Sealtiel", 1.0 },
  { "Seam", 1.0 },
  { "Seaman", 1.0 },
  { "Seamas", 1.0 },
  { "Seamon", 1.0 },
  { "Seamus", 1.0 },
  { "Sean", 0.9929657 },
  { "Seana", 0.0 },
  { "Seanan", 1.0 },
  { "Seananthony", 1.0 },
  { "Seanchristopher", 1.0 },
  { "Seanda", 0.0 },
  { "Seandee", 0.0 },
  { "Seandell", 1.0 },
  { "Seandouglas", 1.0 },
  { "Seandra", 0.0 },
  { "Seandre", 1.0 },
  { "Seandrea", 0.0 },
  { "Seaneen", 0.0 },
  { "Seanette", 0.0 },
  { "Seangabriel", 1.0 },
  { "Seanice", 0.0 },
  { "Seanix", 1.0 },
  { "Seaniya", 0.0 },
  { "Seanjohn", 1.0 },
  { "Seanmatthew", 1.0 },
  { "Seanmichael", 1.0 },
  { "Seann", 0.9904215 },
  { "Seanna", 0.0 },
  { "Seanne", 0.0 },
  { "Seanpatrick", 1.0 },
  { "Seanpaul", 1.0 },
  { "Seanta", 0.0 },
  { "Seantae", 0.0 },
  { "Seante", 0.0 },
  { "Seanthomas", 1.0 },
  { "Seantyler", 1.0 },
  { "Seara", 0.0 },
  { "Searah", 0.0 },
  { "Searcy", 0.9741379 },
  { "Searia", 0.0 },
  { "Searl", 1.0 },
  { "Searle", 1.0 },
  { "Searra", 0.0 },
  { "Searria", 0.0 },
  { "Sears", 1.0 },
  { "Season", 0.0 },
  { "Seasons", 0.0 },
  { "Seath", 1.0 },
  { "Seaton", 1.0 },
  { "Seattle", 0.04950495 },
  { "Seaver", 1.0 },
  { "Seavy", 1.0 },
  { "Seba", 0.0 },
  { "Sebashtian", 1.0 },
  { "Sebastain", 1.0 },
  { "Sebastean", 1.0 },
  { "Sebasthian", 1.0 },
  { "Sebastia", 1.0 },
  { "Sebastiaan", 1.0 },
  { "Sebastian", 0.99779135 },
  { "Sebastiana", 0.0 },
  { "Sebastiano", 1.0 },
  { "Sebastien", 1.0 },
  { "Sebastin", 1.0 },
  { "Sebastino", 1.0 },
  { "Sebastion", 1.0 },
  { "Sebastyan", 1.0 },
  { "Sebastyen", 1.0 },
  { "Sebastyn", 1.0 },
  { "Sebatian", 1.0 },
  { "Sebaztian", 1.0 },
  { "Sebella", 0.0 },
  { "Sebern", 1.0 },
  { "Sebero", 1.0 },
  { "Sebert", 1.0 },
  { "Sebestian", 1.0 },
  { "Sebian", 1.0 },
  { "Sebie", 1.0 },
  { "Sebrena", 0.0 },
  { "Sebrenia", 0.0 },
  { "Sebria", 0.0 },
  { "Sebrina", 0.0 },
  { "Sebrinia", 0.0 },
  { "Sebron", 1.0 },
  { "Seburn", 1.0 },
  { "Secelia", 0.0 },
  { "Secia", 0.0 },
  { "Secilia", 0.0 },
  { "Secily", 0.0 },
  { "Secondo", 1.0 },
  { "Secora", 0.0 },
  { "Secoya", 0.0 },
  { "Secret", 0.0 },
  { "Secunda", 0.0 },
  { "Secundina", 0.0 },
  { "Secundino", 1.0 },
  { "Seda", 0.0 },
  { "Sedale", 1.0 },
  { "Sedalia", 0.0 },
  { "Sedarius", 1.0 },
  { "Seddrick", 1.0 },
  { "Sedelia", 0.0 },
  { "Sedell", 0.0 },
  { "Sederick", 1.0 },
  { "Sedgie", 1.0 },
  { "Sedgwick", 1.0 },
  { "Sedina", 0.0 },
  { "Sedney", 0.0 },
  { "Sedona", 0.0 },
  { "Sedonia", 0.0 },
  { "Sedra", 0.0 },
  { "Sedric", 1.0 },
  { "Sedrick", 1.0 },
  { "Sedricka", 0.0 },
  { "Sedwick", 1.0 },
  { "See", 0.0 },
  { "Seela", 0.0 },
  { "Seeley", 0.73278236 },
  { "Seely", 0.2631579 },
  { "Seema", 0.0 },
  { "Seena", 0.1375 },
  { "Seenaa", 0.0 },
  { "Seerat", 0.0 },
  { "Seerit", 0.0 },
  { "Seeta", 0.0 },
  { "Seeya", 0.0 },
  { "Sef", 1.0 },
  { "Seferina", 0.0 },
  { "Seferino", 1.0 },
  { "Sefora", 0.0 },
  { "Sefton", 1.0 },
  { "Segen", 0.0 },
  { "Seger", 1.0 },
  { "Segio", 1.0 },
  { "Segundo", 1.0 },
  { "Sehaj", 0.5048544 },
  { "Sehajpreet", 0.33333334 },
  { "Sehajveer", 1.0 },
  { "Seham", 0.0 },
  { "Sehana", 0.0 },
  { "Sehar", 0.0 },
  { "Sehej", 0.2 },
  { "Seher", 0.0 },
  { "Sehrish", 0.0 },
  { "Sei", 1.0 },
  { "Seianna", 0.0 },
  { "Seibert", 1.0 },
  { "Seichi", 1.0 },
  { "Seid", 1.0 },
  { "Seidi", 0.0 },
  { "Seidina", 0.0 },
  { "Seidon", 1.0 },
  { "Seidy", 0.0 },
  { "Seif", 1.0 },
  { "Seigo", 1.0 },
  { "Seiichi", 1.0 },
  { "Seija", 0.0 },
  { "Seiji", 1.0 },
  { "Seiki", 1.0 },
  { "Seikichi", 1.0 },
  { "Seiko", 0.6857143 },
  { "Seila", 0.0 },
  { "Seily", 0.0 },
  { "Sein", 1.0 },
  { "Seina", 0.0 },
  { "Seini", 0.0 },
  { "Seinna", 0.0 },
  { "Seira", 0.0 },
  { "Seirra", 0.0 },
  { "Seiry", 0.0 },
  { "Seith", 1.0 },
  { "Seitu", 1.0 },
  { "Seiya", 1.0 },
  { "Seja", 0.0 },
  { "Sejal", 0.0 },
  { "Sejin", 1.0 },
  { "Sejla", 0.0 },
  { "Seka", 0.0 },
  { "Sekai", 0.556338 },
  { "Sekani", 1.0 },
  { "Sekayi", 0.0 },
  { "Sekena", 0.0 },
  { "Sekia", 0.0 },
  { "Sekina", 0.0 },
  { "Sekou", 1.0 },
  { "Sekura", 0.0 },
  { "Sela", 0.0 },
  { "Selah", 0.0137911625 },
  { "Selam", 0.0 },
  { "Selamawit", 0.0 },
  { "Selana", 0.0 },
  { "Selasi", 1.0 },
  { "Selassie", 1.0 },
  { "Selba", 0.0 },
  { "Selbert", 1.0 },
  { "Selby", 0.7212249 },
  { "Selda", 0.0 },
  { "Selden", 1.0 },
  { "Seldon", 1.0 },
  { "Seleah", 0.0 },
  { "Seleana", 0.0 },
  { "Selebrity", 0.0 },
  { "Selecia", 0.0 },
  { "Seledonio", 1.0 },
  { "Seleem", 1.0 },
  { "Seleen", 0.0 },
  { "Seleena", 0.0 },
  { "Seleina", 0.0 },
  { "Selema", 0.0 },
  { "Selen", 0.0 },
  { "Selena", 0.0011832846 },
  { "Selenamarie", 0.0 },
  { "Selene", 0.00079088897 },
  { "Seleni", 0.0 },
  { "Selenia", 0.0 },
  { "Selenna", 0.0 },
  { "Selenne", 0.0 },
  { "Seleny", 0.0 },
  { "Selest", 0.0 },
  { "Seleste", 0.0 },
  { "Selester", 0.5 },
  { "Selestina", 0.0 },
  { "Selestine", 0.0 },
  { "Selestino", 1.0 },
  { "Seleta", 0.0 },
  { "Seletha", 0.0 },
  { "Selethia", 0.0 },
  { "Seleyna", 0.0 },
  { "Selia", 0.0 },
  { "Seliah", 0.0 },
  { "Seliana", 0.0 },
  { "Selica", 0.0 },
  { "Selicia", 0.0 },
  { "Selig", 1.0 },
  { "Selihom", 0.0 },
  { "Selika", 0.0 },
  { "Selim", 1.0 },
  { "Selima", 0.0 },
  { "Selin", 0.0 },
  { "Selina", 0.0 },
  { "Selinda", 0.0 },
  { "Seline", 0.0 },
  { "Selinna", 0.0 },
  { "Selisa", 0.0 },
  { "Selita", 0.0 },
  { "Seliyah", 0.0 },
  { "Sella", 0.0 },
  { "Sellers", 1.0 },
  { "Sellie", 1.0 },
  { "Selma", 0.0120940395 },
  { "Selman", 1.0 },
  { "Selmer", 0.9869961 },
  { "Selmon", 1.0 },
  { "Selphie", 0.0 },
  { "Selso", 1.0 },
  { "Selton", 1.0 },
  { "Selva", 0.0 },
  { "Selven", 1.0 },
  { "Selvia", 0.0 },
  { "Selvin", 1.0 },
  { "Selvyn", 1.0 },
  { "Selwyn", 0.9928058 },
  { "Selyn", 0.0 },
  { "Selyna", 0.0 },
  { "Sem", 1.0 },
  { "Sema", 0.0 },
  { "Semahj", 1.0 },
  { "Semaiah", 0.0 },
  { "Semaj", 0.7667749 },
  { "Semaja", 0.12643678 },
  { "Semajae", 0.5 },
  { "Semajah", 0.0 },
  { "Semajai", 1.0 },
  { "Semajay", 1.0 },
  { "Semaje", 0.80451125 },
  { "Semantha", 0.0 },
  { "Semara", 0.0 },
  { "Semaria", 0.0 },
  { "Semarias", 1.0 },
  { "Semaya", 0.0 },
  { "Semayah", 0.0 },
  { "Semeka", 0.0 },
  { "Semeko", 0.0 },
  { "Semere", 1.0 },
  { "Semetria", 0.0 },
  { "Semhal", 0.0 },
  { "Semhar", 0.0 },
  { "Semia", 0.0 },
  { "Semiah", 0.0 },
  { "Semico", 0.0 },
  { "Semih", 1.0 },
  { "Semika", 0.0 },
  { "Semiko", 0.0 },
  { "Semion", 1.0 },
  { "Semir", 1.0 },
  { "Semira", 0.0 },
  { "Semirah", 0.0 },
  { "Semisi", 1.0 },
  { "Semiya", 0.0 },
  { "Semiyah", 0.0 },
  { "Semmie", 1.0 },
  { "Semon", 0.7058824 },
  { "Semone", 0.0 },
  { "Semora", 0.0 },
  { "Semra", 0.0 },
  { "Semya", 0.0 },
  { "Semyah", 0.0 },
  { "Semyon", 1.0 },
  { "Semyra", 0.0 },
  { "Sen", 1.0 },
  { "Sena", 0.0513369 },
  { "Senaca", 1.0 },
  { "Senad", 1.0 },
  { "Senada", 0.0 },
  { "Senai", 0.46391752 },
  { "Senaida", 0.0 },
  { "Senaido", 1.0 },
  { "Senait", 0.0 },
  { "Senaiya", 0.0 },
  { "Senan", 1.0 },
  { "Senator", 1.0 },
  { "Senay", 1.0 },
  { "Senaya", 0.0 },
  { "Sencere", 1.0 },
  { "Senchal", 0.0 },
  { "Senda", 0.0 },
  { "Sender", 1.0 },
  { "Sendi", 0.0 },
  { "Sendy", 0.0 },
  { "Seneatha", 0.0 },
  { "Seneca", 0.51262134 },
  { "Senecca", 1.0 },
  { "Seneka", 0.0 },
  { "Senen", 1.0 },
  { "Senequa", 0.0 },
  { "Seneque", 0.0 },
  { "Senetra", 0.0 },
  { "Senetria", 0.0 },
  { "Senetta", 0.0 },
  { "Seng", 0.7399381 },
  { "Senga", 0.0 },
  { "Senia", 0.0 },
  { "Seniah", 0.0 },
  { "Senica", 0.72727275 },
  { "Senida", 0.0 },
  { "Senie", 0.0 },
  { "Senika", 0.0 },
  { "Senikka", 0.0 },
  { "Seniqua", 0.0 },
  { "Senita", 0.0 },
  { "Seniya", 0.0 },
  { "Seniyah", 0.0 },
  { "Senja", 0.0 },
  { "Senna", 0.06521739 },
  { "Senneca", 1.0 },
  { "Sennett", 1.0 },
  { "Sennie", 0.0 },
  { "Senobia", 0.0 },
  { "Senon", 1.0 },
  { "Senona", 0.0 },
  { "Senora", 0.0 },
  { "Senoria", 0.0 },
  { "Senorina", 0.0 },
  { "Senorita", 0.0 },
  { "Senovia", 0.0 },
  { "Senovio", 1.0 },
  { "Sensi", 0.0 },
  { "Senta", 0.0 },
  { "Sentell", 1.0 },
  { "Senteria", 0.0 },
  { "Sentoria", 0.0 },
  { "Sentrell", 1.0 },
  { "Senya", 0.0 },
  { "Senyah", 0.0 },
  { "Seon", 1.0 },
  { "Seona", 0.0 },
  { "Seonna", 0.0 },
  { "Seoyeon", 0.0 },
  { "Seoyoon", 0.0 },
  { "Sepehr", 1.0 },
  { "Seph", 1.0 },
  { "Sephanie", 0.0 },
  { "Sephia", 0.0 },
  { "Sephina", 0.0 },
  { "Sephira", 0.0 },
  { "Sephiroth", 1.0 },
  { "Sephora", 0.0 },
  { "Sephra", 0.0 },
  { "Sepia", 0.0 },
  { "Sepideh", 0.0 },
  { "Sepp", 1.0 },
  { "Seprina", 0.0 },
  { "September", 0.0 },
  { "Seqouia", 0.0 },
  { "Sequan", 1.0 },
  { "Sequana", 0.0 },
  { "Sequeena", 0.0 },
  { "Sequena", 0.0 },
  { "Sequetta", 0.0 },
  { "Sequia", 0.0 },
  { "Sequila", 0.0 },
  { "Sequina", 0.0 },
  { "Sequioa", 0.0 },
  { "Sequita", 0.0 },
  { "Sequoia", 0.097526304 },
  { "Sequoria", 0.0 },
  { "Sequoya", 0.010141988 },
  { "Sequoyah", 0.27941176 },
  { "Sequoyia", 0.0 },
  { "Sera", 0.0 },
  { "Serach", 0.0 },
  { "Serae", 0.0 },
  { "Serafim", 1.0 },
  { "Serafima", 0.0 },
  { "Serafin", 1.0 },
  { "Serafina", 0.0 },
  { "Serafine", 0.0 },
  { "Serafino", 1.0 },
  { "Serah", 0.0 },
  { "Serai", 0.0 },
  { "Seraiah", 0.022058824 },
  { "Seraj", 1.0 },
  { "Serana", 0.0 },
  { "Serani", 0.76 },
  { "Seraph", 0.5555556 },
  { "Seraphim", 0.26168224 },
  { "Seraphin", 0.8076923 },
  { "Seraphina", 0.0 },
  { "Seraphine", 0.022371365 },
  { "Serapio", 1.0 },
  { "Seras", 0.0 },
  { "Seray", 0.0 },
  { "Seraya", 0.0 },
  { "Serayah", 0.0 },
  { "Serbando", 1.0 },
  { "Serdar", 1.0 },
  { "Sereana", 0.0 },
  { "Sereda", 0.0 },
  { "Sereen", 0.0 },
  { "Sereena", 0.0 },
  { "Sereina", 0.0 },
  { "Serel", 0.0 },
  { "Seren", 0.027548209 },
  { "Serena", 0.00039923147 },
  { "Serenah", 0.0 },
  { "Serenaty", 0.0 },
  { "Serendipity", 0.0 },
  { "Serene", 0.0 },
  { "Serenety", 0.0 },
  { "Serenia", 0.0 },
  { "Serenidy", 0.0 },
  { "Serenitee", 0.0 },
  { "Serenitey", 0.0 },
  { "Sereniti", 0.0 },
  { "Serenitie", 0.0 },
  { "Serenity", 0.0012743927 },
  { "Serenityrose", 0.0 },
  { "Serenna", 0.0 },
  { "Serennity", 0.0 },
  { "Serentiy", 0.0 },
  { "Seresa", 0.0 },
  { "Sereta", 0.0 },
  { "Seretha", 0.0 },
  { "Seretta", 0.0 },
  { "Serge", 1.0 },
  { "Sergei", 1.0 },
  { "Sergey", 1.0 },
  { "Sergi", 1.0 },
  { "Sergie", 1.0 },
  { "Sergio", 0.9952257 },
  { "Sergiojr", 1.0 },
  { "Serguio", 1.0 },
  { "Serhat", 1.0 },
  { "Seri", 0.0 },
  { "Seria", 0.0 },
  { "Seriah", 0.0 },
  { "Seriana", 0.0 },
  { "Serianna", 0.0 },
  { "Serica", 0.0 },
  { "Seriena", 0.0 },
  { "Serigne", 1.0 },
  { "Serigo", 1.0 },
  { "Serin", 0.0 },
  { "Serina", 0.0 },
  { "Serinah", 0.0 },
  { "Serine", 0.0 },
  { "Seriniti", 0.0 },
  { "Serinity", 0.0 },
  { "Serious", 1.0 },
  { "Serissa", 0.0 },
  { "Serita", 0.0 },
  { "Seritta", 0.0 },
  { "Seriya", 0.0 },
  { "Seriyah", 0.0 },
  { "Serj", 1.0 },
  { "Serjio", 1.0 },
  { "Serkan", 1.0 },
  { "Serly", 0.0 },
  { "Seroba", 0.0 },
  { "Serra", 0.0 },
  { "Serrah", 0.0 },
  { "Serrena", 0.0 },
  { "Serrenity", 0.0 },
  { "Serria", 0.0 },
  { "Serriah", 0.0 },
  { "Serrina", 0.0 },
  { "Serrita", 0.0 },
  { "Servando", 1.0 },
  { "Seryn", 0.0 },
  { "Seryna", 0.0 },
  { "Seryniti", 0.0 },
  { "Serynity", 0.0 },
  { "Sesalie", 0.0 },
  { "Sesar", 1.0 },
  { "Sesario", 1.0 },
  { "Sesen", 0.0 },
  { "Sesha", 0.0 },
  { "Sesilia", 0.0 },
  { "Sesily", 0.0 },
  { "Sesley", 0.0 },
  { "Seslie", 0.0 },
  { "Sessily", 0.0 },
  { "Set", 1.0 },
  { "Seta", 0.0 },
  { "Setara", 0.0 },
  { "Setareh", 0.0 },
  { "Setayesh", 0.0 },
  { "Seteria", 0.0 },
  { "Seth", 0.9969105 },
  { "Sethan", 1.0 },
  { "Sethaniel", 1.0 },
  { "Sethe", 1.0 },
  { "Seton", 0.8181818 },
  { "Setsuko", 0.0 },
  { "Setsuo", 1.0 },
  { "Settimio", 1.0 },
  { "Seumas", 1.0 },
  { "Seung", 1.0 },
  { "Seva", 0.0 },
  { "Sevaeh", 0.0 },
  { "Sevag", 1.0 },
  { "Sevak", 1.0 },
  { "Sevan", 0.92015207 },
  { "Sevana", 0.0 },
  { "Sevanah", 0.0 },
  { "Sevanna", 0.0 },
  { "Sevannah", 0.0 },
  { "Sevara", 0.0 },
  { "Sevasti", 0.0 },
  { "Sevastian", 1.0 },
  { "Sevaughn", 1.0 },
  { "Sevda", 0.0 },
  { "Seve", 1.0 },
  { "Seveah", 0.0 },
  { "Seven", 0.6928934 },
  { "Sevena", 0.0 },
  { "Seveon", 1.0 },
  { "Sever", 1.0 },
  { "Severa", 0.0 },
  { "Severen", 1.0 },
  { "Severia", 0.0 },
  { "Severiano", 1.0 },
  { "Severin", 1.0 },
  { "Severina", 0.0 },
  { "Severine", 0.0 },
  { "Severino", 1.0 },
  { "Severio", 1.0 },
  { "Severn", 1.0 },
  { "Severo", 1.0 },
  { "Severt", 1.0 },
  { "Severus", 1.0 },
  { "Severyn", 1.0 },
  { "Sevi", 0.0 },
  { "Seviah", 0.0 },
  { "Sevilla", 0.0 },
  { "Seville", 0.390625 },
  { "Sevin", 0.79710144 },
  { "Sevina", 0.0 },
  { "Sevinch", 0.0 },
  { "Sevion", 1.0 },
  { "Sevon", 0.9680851 },
  { "Sevren", 1.0 },
  { "Sevrin", 1.0 },
  { "Sevryn", 1.0 },
  { "Sevy", 1.0 },
  { "Sevyn", 0.4085667 },
  { "Sevynn", 0.0 },
  { "Sewall", 1.0 },
  { "Seward", 1.0 },
  { "Sewell", 1.0 },
  { "Sewilla", 0.0 },
  { "Sewit", 0.0 },
  { "Sexton", 1.0 },
  { "Seya", 0.0 },
  { "Seychelle", 0.0 },
  { "Seydi", 0.0 },
  { "Seydina", 1.0 },
  { "Seydou", 1.0 },
  { "Seyed", 1.0 },
  { "Seyha", 1.0 },
  { "Seyla", 0.0 },
  { "Seylah", 0.0 },
  { "Seymone", 0.0 },
  { "Seymore", 1.0 },
  { "Seymour", 0.99923486 },
  { "Seymoure", 1.0 },
  { "Seynabou", 0.0 },
  { "Seyon", 1.0 },
  { "Seyram", 0.0 },
  { "Seyvon", 1.0 },
  { "Sha", 0.16245006 },
  { "Shaad", 1.0 },
  { "Shaakira", 0.0 },
  { "Shaakirah", 0.0 },
  { "Shaaliyah", 0.0 },
  { "Shaam", 0.0 },
  { "Shaan", 1.0 },
  { "Shaana", 0.0 },
  { "Shaanvi", 0.0 },
  { "Shaarav", 1.0 },
  { "Shaaron", 0.0 },
  { "Shaarwin", 1.0 },
  { "Shaasia", 0.0 },
  { "Shaaz", 1.0 },
  { "Shaba", 1.0 },
  { "Shabab", 1.0 },
  { "Shabaka", 1.0 },
  { "Shaban", 1.0 },
  { "Shabana", 0.0 },
  { "Shabaz", 1.0 },
  { "Shabazz", 1.0 },
  { "Shabd", 1.0 },
  { "Shabina", 0.0 },
  { "Shabnam", 0.0 },
  { "Shaborn", 1.0 },
  { "Shabre", 0.0 },
  { "Shabrea", 0.0 },
  { "Shabree", 0.0 },
  { "Shabreka", 0.0 },
  { "Shabri", 0.0 },
  { "Shabria", 0.0 },
  { "Shabriana", 0.0 },
  { "Shabrie", 0.0 },
  { "Shabriel", 0.0 },
  { "Shabrika", 0.0 },
  { "Shabrina", 0.0 },
  { "Shabrittany", 0.0 },
  { "Shabsi", 1.0 },
  { "Shacara", 0.0 },
  { "Shacari", 0.0 },
  { "Shacaria", 0.0 },
  { "Shacarla", 0.0 },
  { "Shacarra", 0.0 },
  { "Shachar", 1.0 },
  { "Shachi", 0.0 },
  { "Shack", 1.0 },
  { "Shacola", 0.0 },
  { "Shacole", 0.0 },
  { "Shaconda", 0.0 },
  { "Shacondra", 0.0 },
  { "Shaconna", 0.0 },
  { "Shacora", 0.0 },
  { "Shacorey", 1.0 },
  { "Shacori", 0.0 },
  { "Shacoria", 0.0 },
  { "Shacorra", 0.0 },
  { "Shacourtney", 0.0 },
  { "Shacoya", 0.0 },
  { "Shacoyia", 0.0 },
  { "Shacquille", 1.0 },
  { "Shad", 1.0 },
  { "Shada", 0.0 },
  { "Shadae", 0.0 },
  { "Shadai", 0.0 },
  { "Shadaisha", 0.0 },
  { "Shadaja", 0.0 },
  { "Shadajah", 0.0 },
  { "Shadale", 0.0 },
  { "Shadan", 0.0 },
  { "Shadana", 0.0 },
  { "Shadany", 0.0 },
  { "Shadara", 0.0 },
  { "Shadaria", 0.0 },
  { "Shadarian", 1.0 },
  { "Shadarius", 1.0 },
  { "Shadarria", 0.0 },
  { "Shadasha", 0.0 },
  { "Shadasia", 0.0 },
  { "Shadava", 0.0 },
  { "Shadavia", 0.0 },
  { "Shadawn", 0.0 },
  { "Shaday", 0.0 },
  { "Shadaya", 0.0 },
  { "Shadayah", 0.0 },
  { "Shadd", 1.0 },
  { "Shaddai", 0.21839081 },
  { "Shaddix", 1.0 },
  { "Shaddrick", 1.0 },
  { "Shaddy", 1.0 },
  { "Shade", 0.54367876 },
  { "Shadea", 0.0 },
  { "Shadee", 0.44512194 },
  { "Shadeed", 1.0 },
  { "Shadeen", 0.0 },
  { "Shadeja", 0.0 },
  { "Shadejah", 0.0 },
  { "Shadel", 0.0 },
  { "Shadell", 0.07042254 },
  { "Shaden", 0.5744235 },
  { "Shadena", 0.0 },
  { "Shadera", 0.0 },
  { "Shaderrica", 0.0 },
  { "Shadestiny", 0.0 },
  { "Shadey", 0.0 },
  { "Shadi", 0.7292818 },
  { "Shadia", 0.0 },
  { "Shadiamon", 0.0 },
  { "Shadiamond", 0.0 },
  { "Shadie", 0.14159292 },
  { "Shadijah", 0.0 },
  { "Shadimon", 0.0 },
  { "Shadin", 0.0 },
  { "Shadina", 0.0 },
  { "Shadiqua", 0.0 },
  { "Shadira", 0.0 },
  { "Shadiya", 0.0 },
  { "Shadiyah", 0.0 },
  { "Shadley", 1.0 },
  { "Shadman", 1.0 },
  { "Shadoe", 0.73333335 },
  { "Shadon", 0.42857143 },
  { "Shadona", 0.0 },
  { "Shadonna", 0.0 },
  { "Shadora", 0.0 },
  { "Shadow", 0.53243846 },
  { "Shadra", 0.0 },
  { "Shadrach", 1.0 },
  { "Shadrack", 1.0 },
  { "Shadreka", 0.0 },
  { "Shadric", 1.0 },
  { "Shadrick", 1.0 },
  { "Shadricka", 0.0 },
  { "Shadrika", 0.0 },
  { "Shadron", 1.0 },
  { "Shadwick", 1.0 },
  { "Shady", 0.607767 },
  { "Shadya", 0.0 },
  { "Shadyn", 0.29411766 },
  { "Shae", 0.22002715 },
  { "Shaeann", 0.0 },
  { "Shaeden", 1.0 },
  { "Shaedon", 1.0 },
  { "Shaefer", 1.0 },
  { "Shael", 1.0 },
  { "Shaela", 0.0 },
  { "Shaelah", 0.0 },
  { "Shaelan", 0.0 },
  { "Shaelea", 0.0 },
  { "Shaelee", 0.0 },
  { "Shaeleigh", 0.0 },
  { "Shaelen", 0.0 },
  { "Shaelene", 0.0 },
  { "Shaeley", 0.0 },
  { "Shaeli", 0.0 },
  { "Shaelie", 0.0 },
  { "Shaelin", 0.0 },
  { "Shaely", 0.0 },
  { "Shaelyn", 0.0 },
  { "Shaelynn", 0.0 },
  { "Shaelynne", 0.0 },
  { "Shaemus", 1.0 },
  { "Shaen", 1.0 },
  { "Shaena", 0.0 },
  { "Shaenna", 0.0 },
  { "Shafee", 1.0 },
  { "Shafeeq", 1.0 },
  { "Shafeqah", 0.0 },
  { "Shafer", 1.0 },
  { "Shaffer", 1.0 },
  { "Shafi", 1.0 },
  { "Shafia", 0.0 },
  { "Shafin", 1.0 },
  { "Shafina", 0.0 },
  { "Shafiq", 1.0 },
  { "Shafon", 0.0 },
  { "Shafonda", 0.0 },
  { "Shaft", 1.0 },
  { "Shafter", 1.0 },
  { "Shafton", 1.0 },
  { "Shaghayegh", 0.0 },
  { "Shaguana", 0.0 },
  { "Shaguanda", 0.0 },
  { "Shaguanna", 0.0 },
  { "Shagun", 0.0 },
  { "Shah", 0.9700599 },
  { "Shahaan", 1.0 },
  { "Shahab", 1.0 },
  { "Shahad", 0.028901733 },
  { "Shahada", 0.0 },
  { "Shahadah", 0.0 },
  { "Shahan", 1.0 },
  { "Shahana", 0.0 },
  { "Shahanna", 0.0 },
  { "Shahara", 0.0 },
  { "Shaharra", 0.0 },
  { "Shahbaz", 1.0 },
  { "Shahd", 0.0 },
  { "Shahed", 0.07751938 },
  { "Shaheed", 1.0 },
  { "Shaheeda", 0.0 },
  { "Shaheedah", 0.0 },
  { "Shaheem", 1.0 },
  { "Shaheen", 0.76567656 },
  { "Shaheer", 1.0 },
  { "Shaheerah", 0.0 },
  { "Shaheim", 1.0 },
  { "Shaher", 1.0 },
  { "Shahera", 0.0 },
  { "Shaherah", 0.0 },
  { "Shahid", 1.0 },
  { "Shahida", 0.0 },
  { "Shahidah", 0.0 },
  { "Shahied", 1.0 },
  { "Shahiem", 1.0 },
  { "Shahil", 1.0 },
  { "Shahin", 1.0 },
  { "Shahina", 0.0 },
  { "Shahir", 1.0 },
  { "Shahira", 0.0 },
  { "Shahirah", 0.0 },
  { "Shahla", 0.0 },
  { "Shahmeer", 1.0 },
  { "Shahmir", 1.0 },
  { "Shahn", 1.0 },
  { "Shahna", 0.0 },
  { "Shahnawaz", 1.0 },
  { "Shahnaz", 0.0 },
  { "Shahraan", 1.0 },
  { "Shahram", 1.0 },
  { "Shahrazad", 0.0 },
  { "Shahreen", 0.0 },
  { "Shahriar", 1.0 },
  { "Shahrukh", 1.0 },
  { "Shahryar", 1.0 },
  { "Shahrzad", 0.0 },
  { "Shahzad", 1.0 },
  { "Shahzaib", 1.0 },
  { "Shahzain", 1.0 },
  { "Shahzeb", 1.0 },
  { "Shahzoda", 0.0 },
  { "Shai", 0.47385174 },
  { "Shaia", 0.0 },
  { "Shaian", 0.0 },
  { "Shaiana", 0.0 },
  { "Shaiann", 0.0 },
  { "Shaianna", 0.0 },
  { "Shaianne", 0.0 },
  { "Shaiasia", 0.0 },
  { "Shaid", 1.0 },
  { "Shaida", 0.0 },
  { "Shaiden", 1.0 },
  { "Shaiel", 0.0 },
  { "Shaienne", 0.0 },
  { "Shaiheem", 1.0 },
  { "Shaik", 1.0 },
  { "Shaikeem", 1.0 },
  { "Shaikh", 1.0 },
  { "Shaikha", 0.0 },
  { "Shail", 1.0 },
  { "Shaila", 0.0 },
  { "Shailah", 0.0 },
  { "Shailee", 0.0 },
  { "Shaileen", 0.0 },
  { "Shaileigh", 0.0 },
  { "Shailen", 0.93333334 },
  { "Shailene", 0.0 },
  { "Shailey", 0.0 },
  { "Shaili", 0.0 },
  { "Shailie", 0.0 },
  { "Shailin", 0.0 },
  { "Shaily", 0.0 },
  { "Shailyn", 0.0 },
  { "Shailynn", 0.0 },
  { "Shailynne", 0.0 },
  { "Shaima", 0.0 },
  { "Shaimaa", 0.0 },
  { "Shain", 0.9782427 },
  { "Shaina", 0.0015799175 },
  { "Shainah", 0.0 },
  { "Shaindel", 0.0 },
  { "Shaindy", 0.0 },
  { "Shaine", 0.8168317 },
  { "Shainia", 0.0 },
  { "Shainna", 0.0 },
  { "Shaiquan", 1.0 },
  { "Shaira", 0.0 },
  { "Shairon", 0.0 },
  { "Shaisha", 0.0 },
  { "Shaista", 0.0 },
  { "Shaivi", 0.0 },
  { "Shaiya", 0.0 },
  { "Shaiyan", 0.0 },
  { "Shajuan", 0.28282827 },
  { "Shajuana", 0.0 },
  { "Shajuanna", 0.0 },
  { "Shaka", 0.81291175 },
  { "Shakai", 0.9324324 },
  { "Shakaila", 0.0 },
  { "Shakaira", 0.0 },
  { "Shakaiya", 0.0 },
  { "Shakala", 0.0 },
  { "Shakalia", 0.0 },
  { "Shakana", 0.0 },
  { "Shakar", 1.0 },
  { "Shakara", 0.0 },
  { "Shakari", 0.038338657 },
  { "Shakaria", 0.0 },
  { "Shakarra", 0.0 },
  { "Shakaya", 0.0 },
  { "Shakayia", 0.0 },
  { "Shakayla", 0.0 },
  { "Shakaylah", 0.0 },
  { "Shakea", 0.0 },
  { "Shakeah", 0.0 },
  { "Shakeal", 1.0 },
  { "Shakeara", 0.0 },
  { "Shakeba", 0.0 },
  { "Shaked", 1.0 },
  { "Shakeda", 0.0 },
  { "Shakedra", 0.0 },
  { "Shakee", 1.0 },
  { "Shakeeka", 0.0 },
  { "Shakeel", 1.0 },
  { "Shakeela", 0.0 },
  { "Shakeelah", 0.0 },
  { "Shakeem", 1.0 },
  { "Shakeema", 0.0 },
  { "Shakeemah", 0.0 },
  { "Shakeen", 1.0 },
  { "Shakeena", 0.0 },
  { "Shakeenah", 0.0 },
  { "Shakeer", 1.0 },
  { "Shakeera", 0.0 },
  { "Shakeerah", 0.0 },
  { "Shakeeta", 0.0 },
  { "Shakeia", 0.0 },
  { "Shakeidra", 0.0 },
  { "Shakeil", 1.0 },
  { "Shakeila", 0.0 },
  { "Shakeim", 1.0 },
  { "Shakeima", 0.0 },
  { "Shakeira", 0.0 },
  { "Shakeisha", 0.0 },
  { "Shakeita", 0.0 },
  { "Shakeitha", 0.0 },
  { "Shakeithia", 0.0 },
  { "Shakel", 0.6363636 },
  { "Shakela", 0.0 },
  { "Shakelah", 0.0 },
  { "Shakelia", 0.0 },
  { "Shakell", 0.74358976 },
  { "Shakella", 0.0 },
  { "Shakema", 0.0 },
  { "Shakemia", 0.0 },
  { "Shakemmia", 0.0 },
  { "Shakena", 0.0 },
  { "Shakendra", 0.0 },
  { "Shakendria", 0.0 },
  { "Shakenia", 0.0 },
  { "Shakenna", 0.0 },
  { "Shakenya", 0.0 },
  { "Shaker", 1.0 },
  { "Shakera", 0.0 },
  { "Shakerah", 0.0 },
  { "Shakeria", 0.0 },
  { "Shakerra", 0.0 },
  { "Shakerria", 0.0 },
  { "Shakesha", 0.0 },
  { "Shakeshia", 0.0 },
  { "Shaketa", 0.0 },
  { "Shaketha", 0.0 },
  { "Shakethia", 0.0 },
  { "Shaketia", 0.0 },
  { "Shaketra", 0.0 },
  { "Shaketta", 0.0 },
  { "Shakeva", 0.0 },
  { "Shakevia", 0.0 },
  { "Shakeya", 0.0 },
  { "Shakeyah", 0.0 },
  { "Shakeyda", 0.0 },
  { "Shakeyia", 0.0 },
  { "Shakeyla", 0.0 },
  { "Shakeyra", 0.0 },
  { "Shakeyta", 0.0 },
  { "Shakhia", 0.0 },
  { "Shakhzoda", 0.0 },
  { "Shaki", 1.0 },
  { "Shakia", 0.0 },
  { "Shakiah", 0.0 },
  { "Shakiara", 0.0 },
  { "Shakib", 1.0 },
  { "Shakida", 0.0 },
  { "Shakiea", 0.0 },
  { "Shakiel", 1.0 },
  { "Shakiela", 0.0 },
  { "Shakiem", 1.0 },
  { "Shakiera", 0.0 },
  { "Shakierra", 0.0 },
  { "Shakiesha", 0.0 },
  { "Shakieta", 0.0 },
  { "Shakil", 0.9717514 },
  { "Shakila", 0.0 },
  { "Shakilah", 0.0 },
  { "Shakilya", 0.0 },
  { "Shakim", 1.0 },
  { "Shakima", 0.0 },
  { "Shakina", 0.0 },
  { "Shakinah", 0.0 },
  { "Shaking", 1.0 },
  { "Shakir", 0.99059266 },
  { "Shakira", 0.0 },
  { "Shakirah", 0.0 },
  { "Shakiria", 0.0 },
  { "Shakirra", 0.0 },
  { "Shakisha", 0.0 },
  { "Shakita", 0.0 },
  { "Shakitha", 0.0 },
  { "Shakitta", 0.0 },
  { "Shakiva", 0.0 },
  { "Shakiya", 0.0 },
  { "Shakiyah", 0.0 },
  { "Shakiyla", 0.0 },
  { "Shakka", 0.0 },
  { "Shakkia", 0.0 },
  { "Shakobe", 1.0 },
  { "Shakonda", 0.0 },
  { "Shakor", 1.0 },
  { "Shakora", 0.0 },
  { "Shakoria", 0.0 },
  { "Shakota", 0.0 },
  { "Shakoya", 0.0 },
  { "Shakria", 0.0 },
  { "Shakthi", 0.0 },
  { "Shakti", 0.04950495 },
  { "Shakuan", 1.0 },
  { "Shakur", 0.99211216 },
  { "Shakura", 0.0 },
  { "Shakuria", 0.0 },
  { "Shakwan", 1.0 },
  { "Shakwon", 1.0 },
  { "Shakya", 0.0 },
  { "Shakyah", 0.0 },
  { "Shakye", 1.0 },
  { "Shakyia", 0.0 },
  { "Shakyiah", 0.0 },
  { "Shakyla", 0.0 },
  { "Shakyra", 0.0 },
  { "Shakyrah", 0.0 },
  { "Shakyria", 0.0 },
  { "Shala", 0.0 },
  { "Shalae", 0.0 },
  { "Shalah", 0.0 },
  { "Shalai", 0.0 },
  { "Shalaina", 0.0 },
  { "Shalaine", 0.0 },
  { "Shalaka", 0.0 },
  { "Shalako", 1.0 },
  { "Shalamar", 0.26182964 },
  { "Shalan", 0.042735044 },
  { "Shalana", 0.0 },
  { "Shalanda", 0.0 },
  { "Shalandra", 0.0 },
  { "Shalandria", 0.0 },
  { "Shalane", 0.0 },
  { "Shalani", 0.0 },
  { "Shalanna", 0.0 },
  { "Shalanta", 0.0 },
  { "Shalante", 0.0 },
  { "Shalaunda", 0.0 },
  { "Shalaundra", 0.0 },
  { "Shalawn", 0.0 },
  { "Shalay", 0.0 },
  { "Shalaya", 0.0 },
  { "Shalayah", 0.0 },
  { "Shalayla", 0.0 },
  { "Shalayna", 0.0 },
  { "Shalayne", 0.0 },
  { "Shale", 0.72727275 },
  { "Shalea", 0.0 },
  { "Shaleah", 0.0 },
  { "Shalean", 0.0 },
  { "Shalease", 0.0 },
  { "Shalece", 0.0 },
  { "Shalecia", 0.0 },
  { "Shalee", 0.0 },
  { "Shaleece", 0.0 },
  { "Shaleek", 1.0 },
  { "Shaleen", 0.0 },
  { "Shaleena", 0.0 },
  { "Shaleesa", 0.0 },
  { "Shaleese", 0.0 },
  { "Shalei", 0.0 },
  { "Shaleia", 0.0 },
  { "Shaleigh", 0.0 },
  { "Shaleigha", 0.0 },
  { "Shaleka", 0.0 },
  { "Shalen", 0.13253012 },
  { "Shalena", 0.0 },
  { "Shalene", 0.0 },
  { "Shalesa", 0.0 },
  { "Shalese", 0.0 },
  { "Shalesha", 0.0 },
  { "Shaleshia", 0.0 },
  { "Shaleta", 0.0 },
  { "Shaletha", 0.0 },
  { "Shalethia", 0.0 },
  { "Shaletta", 0.0 },
  { "Shalette", 0.0 },
  { "Shalev", 1.0 },
  { "Shalexis", 0.0 },
  { "Shalexus", 0.0 },
  { "Shaley", 0.0 },
  { "Shaleya", 0.0 },
  { "Shali", 0.0 },
  { "Shalia", 0.0 },
  { "Shaliah", 0.0 },
  { "Shalica", 0.0 },
  { "Shalice", 0.0 },
  { "Shalicia", 0.0 },
  { "Shalie", 0.0 },
  { "Shaliek", 1.0 },
  { "Shaliese", 0.0 },
  { "Shalik", 1.0 },
  { "Shalika", 0.0 },
  { "Shalim", 1.0 },
  { "Shalima", 0.0 },
  { "Shalimar", 0.0077639753 },
  { "Shalin", 0.64681727 },
  { "Shalina", 0.0 },
  { "Shalinda", 0.0 },
  { "Shaline", 0.0 },
  { "Shalini", 0.0 },
  { "Shaliqua", 0.0 },
  { "Shalisa", 0.0 },
  { "Shalise", 0.0 },
  { "Shalisha", 0.0 },
  { "Shalisia", 0.0 },
  { "Shalissa", 0.0 },
  { "Shalita", 0.0 },
  { "Shalitha", 0.0 },
  { "Shaliya", 0.0 },
  { "Shaliyah", 0.0 },
  { "Shalla", 0.0 },
  { "Shallah", 1.0 },
  { "Shallan", 0.0 },
  { "Shallen", 0.0 },
  { "Shallin", 0.0 },
  { "Shalliyah", 0.0 },
  { "Shallon", 0.0 },
  { "Shally", 0.0 },
  { "Shallyn", 0.0 },
  { "Shalma", 0.0 },
  { "Shalo", 0.0 },
  { "Shalom", 0.65794766 },
  { "Shalome", 0.0 },
  { "Shalon", 0.099236645 },
  { "Shalona", 0.0 },
  { "Shalonda", 0.0 },
  { "Shalondra", 0.0 },
  { "Shalone", 0.0 },
  { "Shalonna", 0.0 },
  { "Shalonte", 0.0 },
  { "Shalunda", 0.0 },
  { "Shalva", 0.0 },
  { "Shalya", 0.0 },
  { "Shalyce", 0.0 },
  { "Shalymar", 0.0 },
  { "Shalyn", 0.0 },
  { "Shalynda", 0.0 },
  { "Shalynn", 0.0 },
  { "Shalynne", 0.0 },
  { "Shalyric", 0.0 },
  { "Shalyse", 0.0 },
  { "Shalyssa", 0.0 },
  { "Sham", 0.3 },
  { "Shama", 0.0 },
  { "Shamaar", 1.0 },
  { "Shamada", 0.0 },
  { "Shamae", 0.0 },
  { "Shamael", 0.0 },
  { "Shamaia", 0.0 },
  { "Shamaiah", 0.0 },
  { "Shamaila", 0.0 },
  { "Shamain", 0.0 },
  { "Shamaine", 0.024752475 },
  { "Shamair", 0.23809524 },
  { "Shamaiya", 0.0 },
  { "Shamaka", 0.0 },
  { "Shamakia", 0.0 },
  { "Shamal", 1.0 },
  { "Shamala", 0.0 },
  { "Shaman", 1.0 },
  { "Shamanda", 0.0 },
  { "Shamane", 0.0 },
  { "Shamani", 0.0 },
  { "Shamanique", 0.0 },
  { "Shamar", 0.9571797 },
  { "Shamara", 0.0 },
  { "Shamarah", 0.0 },
  { "Shamarcus", 1.0 },
  { "Shamare", 1.0 },
  { "Shamaree", 0.14705883 },
  { "Shamari", 0.427907 },
  { "Shamaria", 0.0 },
  { "Shamariah", 0.0 },
  { "Shamarie", 0.07913669 },
  { "Shamario", 1.0 },
  { "Shamarion", 0.9822064 },
  { "Shamariona", 0.0 },
  { "Shamariya", 0.0 },
  { "Shamarr", 0.9832776 },
  { "Shamarra", 0.0 },
  { "Shamarria", 0.0 },
  { "Shamauri", 0.36666667 },
  { "Shamauria", 0.0 },
  { "Shamay", 0.0 },
  { "Shamaya", 0.0 },
  { "Shamayah", 0.0 },
  { "Shamayia", 0.0 },
  { "Shamber", 0.0 },
  { "Shambhavi", 0.0 },
  { "Shambra", 0.0 },
  { "Shambre", 0.0 },
  { "Shambrea", 0.0 },
  { "Shambreka", 0.0 },
  { "Shambria", 0.0 },
  { "Shambrica", 0.0 },
  { "Shambrika", 0.0 },
  { "Shamea", 0.0 },
  { "Shameah", 0.0 },
  { "Shameaka", 0.0 },
  { "Shameca", 0.0 },
  { "Shamecca", 0.0 },
  { "Shamecia", 0.0 },
  { "Shamecka", 0.0 },
  { "Shameek", 0.97863245 },
  { "Shameeka", 0.0 },
  { "Shameekia", 0.0 },
  { "Shameen", 0.0 },
  { "Shameer", 1.0 },
  { "Shameera", 0.0 },
  { "Shameerah", 0.0 },
  { "Shameia", 0.0 },
  { "Shameik", 1.0 },
  { "Shameika", 0.0 },
  { "Shameir", 1.0 },
  { "Shamek", 1.0 },
  { "Shameka", 0.0 },
  { "Shameki", 0.0 },
  { "Shamekia", 0.0 },
  { "Shamekka", 0.0 },
  { "Shameko", 0.0 },
  { "Shamel", 0.8648649 },
  { "Shamela", 0.0 },
  { "Shamelia", 0.0 },
  { "Shamell", 0.6632124 },
  { "Shamella", 0.0 },
  { "Shamelle", 0.0 },
  { "Shamena", 0.0 },
  { "Shamequa", 0.0 },
  { "Shamera", 0.0 },
  { "Shamere", 0.33568904 },
  { "Shameria", 0.0 },
  { "Shamese", 0.0 },
  { "Shamesha", 0.0 },
  { "Shameta", 0.0 },
  { "Shametra", 0.0 },
  { "Shametria", 0.0 },
  { "Shametrice", 0.0 },
  { "Shameya", 0.0 },
  { "Shami", 0.0 },
  { "Shamia", 0.0 },
  { "Shamiah", 0.0 },
  { "Shamica", 0.0 },
  { "Shamichael", 0.6666667 },
  { "Shamicka", 0.0 },
  { "Shamiek", 1.0 },
  { "Shamieka", 0.0 },
  { "Shamier", 1.0 },
  { "Shamik", 0.91071427 },
  { "Shamika", 0.0011857707 },
  { "Shamikia", 0.0 },
  { "Shamikka", 0.0 },
  { "Shamiko", 0.0 },
  { "Shamil", 0.7169811 },
  { "Shamila", 0.0 },
  { "Shamill", 0.0 },
  { "Shamilla", 0.0 },
  { "Shamille", 0.0 },
  { "Shamillia", 0.0 },
  { "Shamim", 0.0 },
  { "Shamina", 0.0 },
  { "Shamiqua", 0.0 },
  { "Shamir", 0.85198057 },
  { "Shamira", 0.0 },
  { "Shamiracle", 0.0 },
  { "Shamirah", 0.0 },
  { "Shamire", 0.45454547 },
  { "Shamirra", 0.0 },
  { "Shamise", 0.0 },
  { "Shamisha", 0.0 },
  { "Shamita", 0.0 },
  { "Shamiya", 0.0 },
  { "Shamiyah", 0.0 },
  { "Shamkia", 0.0 },
  { "Shamma", 0.0 },
  { "Shammah", 0.8666667 },
  { "Shammara", 0.0 },
  { "Shammie", 0.0 },
  { "Shammond", 1.0 },
  { "Shamod", 1.0 },
  { "Shamon", 0.81554675 },
  { "Shamona", 0.0 },
  { "Shamond", 1.0 },
  { "Shamonda", 0.0 },
  { "Shamone", 0.09361702 },
  { "Shamoni", 0.0 },
  { "Shamonia", 0.0 },
  { "Shamonica", 0.0 },
  { "Shamonique", 0.0 },
  { "Shamont", 1.0 },
  { "Shamonte", 0.43589744 },
  { "Shamor", 1.0 },
  { "Shamora", 0.0 },
  { "Shamore", 1.0 },
  { "Shamori", 1.0 },
  { "Shamoria", 0.0 },
  { "Shamorrow", 0.0 },
  { "Shampagne", 0.0 },
  { "Shampayne", 0.0 },
  { "Shamra", 0.0 },
  { "Shams", 0.33333334 },
  { "Shamsa", 0.0 },
  { "Shamso", 0.0 },
  { "Shamsuddin", 1.0 },
  { "Shamuel", 1.0 },
  { "Shamus", 1.0 },
  { "Shamya", 0.0 },
  { "Shamyah", 0.0 },
  { "Shamyia", 0.0 },
  { "Shamyiah", 0.0 },
  { "Shamyla", 0.0 },
  { "Shamyra", 0.0 },
  { "Shamyria", 0.0 },
  { "Shan", 0.7128614 },
  { "Shana", 0.0038307682 },
  { "Shanada", 0.0 },
  { "Shanade", 0.0 },
  { "Shanae", 0.0 },
  { "Shanah", 0.0 },
  { "Shanai", 0.0 },
  { "Shanaia", 0.0 },
  { "Shanaih", 0.0 },
  { "Shanail", 0.0 },
  { "Shanaira", 0.0 },
  { "Shanaisha", 0.0 },
  { "Shanaiya", 0.0 },
  { "Shanaja", 0.0 },
  { "Shanaka", 0.0 },
  { "Shanalee", 0.0 },
  { "Shanan", 0.14170507 },
  { "Shanana", 0.0 },
  { "Shanann", 0.0 },
  { "Shanara", 0.0 },
  { "Shanard", 1.0 },
  { "Shanari", 0.0 },
  { "Shanaria", 0.0 },
  { "Shanarra", 0.0 },
  { "Shanasha", 0.0 },
  { "Shanasia", 0.0 },
  { "Shanata", 0.0 },
  { "Shanautica", 0.0 },
  { "Shanavia", 0.0 },
  { "Shanay", 0.0 },
  { "Shanaya", 0.0 },
  { "Shanayah", 0.0 },
  { "Shanaye", 0.0 },
  { "Shanaz", 0.0 },
  { "Shanbria", 0.0 },
  { "Shance", 1.0 },
  { "Shanchez", 1.0 },
  { "Shanda", 0.0 },
  { "Shandal", 0.0 },
  { "Shandale", 0.04385965 },
  { "Shandalyn", 0.0 },
  { "Shandara", 0.0 },
  { "Shande", 0.6315789 },
  { "Shandee", 0.0 },
  { "Shandel", 0.026666667 },
  { "Shandell", 0.11447811 },
  { "Shandella", 0.0 },
  { "Shandelle", 0.0 },
  { "Shanden", 1.0 },
  { "Shander", 0.0 },
  { "Shandera", 0.0 },
  { "Shandi", 0.0 },
  { "Shandia", 0.0 },
  { "Shandice", 0.0 },
  { "Shandie", 0.0 },
  { "Shandiin", 0.0 },
  { "Shandolyn", 0.0 },
  { "Shandon", 0.7986871 },
  { "Shandora", 0.0 },
  { "Shandra", 0.0 },
  { "Shandrea", 0.0 },
  { "Shandreika", 0.0 },
  { "Shandreka", 0.0 },
  { "Shandrell", 0.0 },
  { "Shandria", 0.0 },
  { "Shandrica", 0.0 },
  { "Shandricka", 0.0 },
  { "Shandrika", 0.0 },
  { "Shandy", 0.13641755 },
  { "Shane", 0.98175377 },
  { "Shanea", 0.0 },
  { "Shaneah", 0.0 },
  { "Shaneaka", 0.0 },
  { "Shaneal", 0.0 },
  { "Shanean", 0.0 },
  { "Shaneca", 0.0 },
  { "Shanece", 0.0 },
  { "Shanecia", 0.0 },
  { "Shanecka", 0.0 },
  { "Shanecqua", 0.0 },
  { "Shaneda", 0.0 },
  { "Shanedra", 0.0 },
  { "Shanee", 0.0 },
  { "Shaneece", 0.0 },
  { "Shaneek", 0.0 },
  { "Shaneeka", 0.0 },
  { "Shaneen", 0.0 },
  { "Shaneequa", 0.0 },
  { "Shaneesa", 0.0 },
  { "Shaneese", 0.0 },
  { "Shanegua", 0.0 },
  { "Shanei", 0.0 },
  { "Shaneia", 0.0 },
  { "Shaneice", 0.0 },
  { "Shaneika", 0.0 },
  { "Shaneil", 0.0 },
  { "Shaneiqua", 0.0 },
  { "Shaneise", 0.0 },
  { "Shaneisha", 0.0 },
  { "Shaneka", 0.0 },
  { "Shanekia", 0.0 },
  { "Shanekqa", 0.0 },
  { "Shanekqua", 0.0 },
  { "Shanekwa", 0.0 },
  { "Shanel", 0.0 },
  { "Shanele", 0.0 },
  { "Shanell", 0.0026286966 },
  { "Shanella", 0.0 },
  { "Shanelle", 0.0 },
  { "Shanelly", 0.0 },
  { "Shanely", 0.0 },
  { "Shanen", 0.26815644 },
  { "Shanena", 0.0 },
  { "Shanene", 0.0 },
  { "Shaneqa", 0.0 },
  { "Shanequa", 0.0 },
  { "Shanequah", 0.0 },
  { "Shaneque", 0.0 },
  { "Shanequea", 0.0 },
  { "Shanequia", 0.0 },
  { "Shaneqwa", 0.0 },
  { "Shanera", 0.0 },
  { "Shaneria", 0.0 },
  { "Shanerica", 0.0 },
  { "Shanesa", 0.0 },
  { "Shanese", 0.0 },
  { "Shanesha", 0.0 },
  { "Shaneshia", 0.0 },
  { "Shanesia", 0.0 },
  { "Shanessa", 0.0 },
  { "Shanesse", 0.0 },
  { "Shanet", 0.0 },
  { "Shaneta", 0.0 },
  { "Shanetha", 0.0 },
  { "Shanethia", 0.0 },
  { "Shanetra", 0.0 },
  { "Shanetria", 0.0 },
  { "Shanett", 0.0 },
  { "Shanetta", 0.0 },
  { "Shanette", 0.0 },
  { "Shaneva", 0.0 },
  { "Shaney", 0.0 },
  { "Shaneya", 0.0 },
  { "Shanez", 0.0 },
  { "Shang", 1.0 },
  { "Shanga", 0.8780488 },
  { "Shangaleza", 0.0 },
  { "Shango", 1.0 },
  { "Shani", 0.0038557495 },
  { "Shania", 0.0 },
  { "Shaniah", 0.0 },
  { "Shaniaya", 0.0 },
  { "Shaniayah", 0.0 },
  { "Shanica", 0.0 },
  { "Shanice", 0.000528262 },
  { "Shanicia", 0.0 },
  { "Shanicka", 0.0 },
  { "Shanicqua", 0.0 },
  { "Shanida", 0.0 },
  { "Shanie", 0.0 },
  { "Shanieca", 0.0 },
  { "Shaniece", 0.0 },
  { "Shanieka", 0.0 },
  { "Shaniel", 0.0 },
  { "Shanielle", 0.0 },
  { "Shaniequa", 0.0 },
  { "Shaniese", 0.0 },
  { "Shaniesha", 0.0 },
  { "Shanigua", 0.0 },
  { "Shanii", 0.0 },
  { "Shanija", 0.0 },
  { "Shanijah", 0.0 },
  { "Shanik", 0.0 },
  { "Shanika", 0.001963935 },
  { "Shanike", 0.0 },
  { "Shaniki", 0.0 },
  { "Shanikia", 0.0 },
  { "Shanikka", 0.0 },
  { "Shanikqua", 0.0 },
  { "Shanikque", 0.0 },
  { "Shanikwa", 0.0 },
  { "Shanil", 0.0 },
  { "Shanila", 0.0 },
  { "Shanille", 0.0 },
  { "Shanin", 0.0 },
  { "Shanina", 0.0 },
  { "Shanine", 0.0 },
  { "Shaniqa", 0.0 },
  { "Shaniqua", 0.0 },
  { "Shaniquah", 0.0 },
  { "Shanique", 0.0 },
  { "Shaniquea", 0.0 },
  { "Shaniquia", 0.0 },
  { "Shaniquwa", 0.0 },
  { "Shaniqwa", 0.0 },
  { "Shanira", 0.0 },
  { "Shanisa", 0.0 },
  { "Shanise", 0.0 },
  { "Shanisha", 0.0 },
  { "Shanissa", 0.0 },
  { "Shanita", 0.0 },
  { "Shanitha", 0.0 },
  { "Shanitra", 0.0 },
  { "Shanitta", 0.0 },
  { "Shaniya", 0.0 },
  { "Shaniyah", 0.0 },
  { "Shaniyha", 0.0 },
  { "Shaniyia", 0.0 },
  { "Shaniyla", 0.0 },
  { "Shanka", 0.0 },
  { "Shankar", 1.0 },
  { "Shankeria", 0.0 },
  { "Shankia", 0.0 },
  { "Shanlee", 0.0 },
  { "Shanleigh", 0.0 },
  { "Shanley", 0.0 },
  { "Shanli", 0.0 },
  { "Shanmukh", 1.0 },
  { "Shann", 0.5668016 },
  { "Shanna", 0.0031131709 },
  { "Shannae", 0.0 },
  { "Shannah", 0.0 },
  { "Shannalee", 0.0 },
  { "Shannan", 0.057867836 },
  { "Shannara", 0.0 },
  { "Shannay", 0.0 },
  { "Shannda", 0.0 },
  { "Shanndolyn", 0.0 },
  { "Shanne", 0.4625 },
  { "Shannee", 0.0 },
  { "Shanneka", 0.0 },
  { "Shannel", 0.0 },
  { "Shannell", 0.0 },
  { "Shannelle", 0.0 },
  { "Shannen", 0.07260407 },
  { "Shannequa", 0.0 },
  { "Shannetta", 0.0 },
  { "Shannette", 0.0 },
  { "Shanni", 0.0 },
  { "Shannia", 0.0 },
  { "Shannice", 0.0 },
  { "Shannie", 0.0 },
  { "Shanniece", 0.0 },
  { "Shannika", 0.0 },
  { "Shannikia", 0.0 },
  { "Shannin", 0.0 },
  { "Shanniya", 0.0 },
  { "Shannon", 0.14961955 },
  { "Shannondoah", 0.0 },
  { "Shannone", 0.0 },
  { "Shanny", 0.0 },
  { "Shannya", 0.0 },
  { "Shannyn", 0.0 },
  { "Shanoa", 0.0 },
  { "Shanoah", 0.0 },
  { "Shanobia", 0.0 },
  { "Shanon", 0.26395473 },
  { "Shanona", 0.0 },
  { "Shanonn", 0.0 },
  { "Shanora", 0.0 },
  { "Shanovia", 0.0 },
  { "Shanqua", 0.0 },
  { "Shanqual", 0.0 },
  { "Shanquan", 1.0 },
  { "Shanque", 0.0 },
  { "Shanquel", 0.0 },
  { "Shanquell", 0.0 },
  { "Shanquella", 0.0 },
  { "Shanquetta", 0.0 },
  { "Shanquia", 0.0 },
  { "Shanquil", 0.0 },
  { "Shanquilla", 0.0 },
  { "Shanquille", 1.0 },
  { "Shanquita", 0.0 },
  { "Shanreka", 0.0 },
  { "Shanrica", 0.0 },
  { "Shanrika", 0.0 },
  { "Shant", 1.0 },
  { "Shanta", 0.027032172 },
  { "Shantae", 0.003787879 },
  { "Shantai", 0.0 },
  { "Shantail", 0.0 },
  { "Shantaja", 0.0 },
  { "Shantal", 0.0 },
  { "Shantala", 0.0 },
  { "Shantale", 0.0 },
  { "Shantall", 0.0 },
  { "Shantalle", 0.0 },
  { "Shantana", 0.0 },
  { "Shantanae", 0.0 },
  { "Shantania", 0.0 },
  { "Shantanique", 0.0 },
  { "Shantanu", 1.0 },
  { "Shantara", 0.0 },
  { "Shantaria", 0.0 },
  { "Shantasha", 0.0 },
  { "Shantasia", 0.0 },
  { "Shantavia", 0.0 },
  { "Shantavious", 0.0 },
  { "Shantay", 0.0 },
  { "Shantaya", 0.0 },
  { "Shantaye", 0.0 },
  { "Shantazia", 0.0 },
  { "Shante", 0.03593412 },
  { "Shantea", 0.0 },
  { "Shanteal", 0.0 },
  { "Shantee", 0.0 },
  { "Shanteka", 0.0 },
  { "Shantel", 0.005785448 },
  { "Shantela", 0.0 },
  { "Shantele", 0.0 },
  { "Shantell", 0.0051211347 },
  { "Shantella", 0.0 },
  { "Shantelle", 0.0 },
  { "Shantera", 0.0 },
  { "Shanteria", 0.0 },
  { "Shanterica", 0.0 },
  { "Shanterika", 0.0 },
  { "Shanterra", 0.0 },
  { "Shanterri", 0.0 },
  { "Shanterria", 0.0 },
  { "Shanterrica", 0.0 },
  { "Shantese", 0.0 },
  { "Shantesha", 0.0 },
  { "Shantey", 0.0 },
  { "Shanteya", 0.0 },
  { "Shantez", 0.6233766 },
  { "Shantha", 0.0 },
  { "Shanthi", 0.0 },
  { "Shanti", 0.074270554 },
  { "Shantia", 0.0 },
  { "Shantiana", 0.0 },
  { "Shantice", 0.0 },
  { "Shantie", 0.0 },
  { "Shantiece", 0.0 },
  { "Shantiel", 0.0 },
  { "Shantierra", 0.0 },
  { "Shantih", 0.0 },
  { "Shantika", 0.0 },
  { "Shantil", 0.0 },
  { "Shantina", 0.0 },
  { "Shantinique", 0.0 },
  { "Shantiqua", 0.0 },
  { "Shantique", 0.0 },
  { "Shantise", 0.0 },
  { "Shantisha", 0.0 },
  { "Shantivia", 0.0 },
  { "Shantle", 0.0 },
  { "Shanton", 0.7413793 },
  { "Shantonia", 0.0 },
  { "Shantora", 0.0 },
  { "Shantoria", 0.0 },
  { "Shantoya", 0.0 },
  { "Shantra", 0.0 },
  { "Shantrail", 0.0 },
  { "Shantrece", 0.0 },
  { "Shantrel", 0.0 },
  { "Shantrell", 0.015873017 },
  { "Shantrelle", 0.0 },
  { "Shantrese", 0.0 },
  { "Shantrice", 0.0 },
  { "Shantwana", 0.0 },
  { "Shanty", 0.0 },
  { "Shantya", 0.0 },
  { "Shanvi", 0.0 },
  { "Shanvika", 0.0 },
  { "Shanvitha", 0.0 },
  { "Shany", 0.0 },
  { "Shanya", 0.0 },
  { "Shanyah", 0.0 },
  { "Shanyce", 0.0 },
  { "Shanye", 0.0 },
  { "Shanyel", 0.0 },
  { "Shanygne", 0.0 },
  { "Shanyia", 0.0 },
  { "Shanyiah", 0.0 },
  { "Shanyk", 0.0 },
  { "Shanyka", 0.0 },
  { "Shanyla", 0.0 },
  { "Shanylah", 0.0 },
  { "Shanyn", 0.0 },
  { "Shanyra", 0.0 },
  { "Shanyse", 0.0 },
  { "Shanythia", 0.0 },
  { "Shanza", 0.0 },
  { "Shanzay", 0.0 },
  { "Shanze", 0.0 },
  { "Shao", 1.0 },
  { "Shaolin", 0.07462686 },
  { "Shaon", 1.0 },
  { "Shaonna", 0.0 },
  { "Shaparis", 0.0 },
  { "Shaphan", 1.0 },
  { "Shapree", 0.0 },
  { "Shapria", 0.0 },
  { "Shaqil", 1.0 },
  { "Shaqille", 1.0 },
  { "Shaqita", 0.0 },
  { "Shaqua", 0.0 },
  { "Shaquail", 0.0 },
  { "Shaquaila", 0.0 },
  { "Shaquala", 0.0 },
  { "Shaquale", 0.0 },
  { "Shaqualia", 0.0 },
  { "Shaqualla", 0.0 },
  { "Shaquan", 0.7895563 },
  { "Shaquana", 0.0 },
  { "Shaquanah", 0.0 },
  { "Shaquanda", 0.0 },
  { "Shaquandra", 0.0 },
  { "Shaquandria", 0.0 },
  { "Shaquane", 0.5 },
  { "Shaquania", 0.0 },
  { "Shaquann", 1.0 },
  { "Shaquanna", 0.0 },
  { "Shaquanta", 0.0 },
  { "Shaquara", 0.0 },
  { "Shaquarius", 0.0 },
  { "Shaquasha", 0.0 },
  { "Shaquashia", 0.0 },
  { "Shaquasia", 0.0 },
  { "Shaquavia", 0.0 },
  { "Shaquawn", 1.0 },
  { "Shaquay", 0.0 },
  { "Shaquaya", 0.0 },
  { "Shaquayla", 0.0 },
  { "Shaque", 0.0 },
  { "Shaquea", 0.0 },
  { "Shaqueal", 1.0 },
  { "Shaqueda", 0.0 },
  { "Shaqueel", 1.0 },
  { "Shaqueen", 0.0 },
  { "Shaqueena", 0.0 },
  { "Shaqueeta", 0.0 },
  { "Shaqueil", 1.0 },
  { "Shaquel", 0.4834437 },
  { "Shaquela", 0.0 },
  { "Shaquelia", 0.0 },
  { "Shaquell", 0.4494382 },
  { "Shaquella", 0.0 },
  { "Shaquelle", 0.61864406 },
  { "Shaquena", 0.0 },
  { "Shaquenta", 0.0 },
  { "Shaquera", 0.0 },
  { "Shaqueria", 0.0 },
  { "Shaquerra", 0.0 },
  { "Shaquesha", 0.0 },
  { "Shaqueta", 0.0 },
  { "Shaquetta", 0.0 },
  { "Shaquette", 0.0 },
  { "Shaquez", 0.8484849 },
  { "Shaquia", 0.0 },
  { "Shaquida", 0.0 },
  { "Shaquiel", 0.91071427 },
  { "Shaquielle", 1.0 },
  { "Shaquil", 0.96713614 },
  { "Shaquila", 0.0 },
  { "Shaquile", 0.9448819 },
  { "Shaquill", 0.9011628 },
  { "Shaquilla", 0.0647482 },
  { "Shaquille", 0.9330218 },
  { "Shaquillie", 1.0 },
  { "Shaquin", 0.0 },
  { "Shaquina", 0.0 },
  { "Shaquinda", 0.0 },
  { "Shaquinn", 0.0 },
  { "Shaquinna", 0.0 },
  { "Shaquinta", 0.0 },
  { "Shaquira", 0.0 },
  { "Shaquirra", 0.0 },
  { "Shaquise", 0.0 },
  { "Shaquisha", 0.0 },
  { "Shaquita", 0.0020868115 },
  { "Shaquitta", 0.0 },
  { "Shaqula", 0.0 },
  { "Shaqulia", 0.0 },
  { "Shaqulle", 1.0 },
  { "Shaquna", 0.0 },
  { "Shaqunda", 0.0 },
  { "Shaqunna", 0.0 },
  { "Shaquoia", 0.0 },
  { "Shaquon", 0.90510947 },
  { "Shaquona", 0.0 },
  { "Shaquonda", 0.0 },
  { "Shaquone", 1.0 },
  { "Shaquonna", 0.0 },
  { "Shaquor", 1.0 },
  { "Shaquoya", 0.0 },
  { "Shaqur", 1.0 },
  { "Shaqura", 0.0 },
  { "Shaquria", 0.0 },
  { "Shaqville", 1.0 },
  { "Shaqwan", 1.0 },
  { "Shaqwana", 0.0 },
  { "Shaqwanna", 0.0 },
  { "Shar", 0.0 },
  { "Shara", 0.0010220768 },
  { "Sharad", 1.0 },
  { "Sharada", 0.0 },
  { "Sharae", 0.0 },
  { "Sharaea", 0.0 },
  { "Sharah", 0.0 },
  { "Sharai", 0.0 },
  { "Sharail", 0.0 },
  { "Sharain", 0.0 },
  { "Sharaine", 0.0 },
  { "Sharal", 0.0 },
  { "Sharale", 0.0 },
  { "Sharalee", 0.0 },
  { "Sharalyn", 0.0 },
  { "Sharalynn", 0.0 },
  { "Sharan", 0.04263878 },
  { "Sharana", 0.0 },
  { "Sharanda", 0.0 },
  { "Sharane", 0.0 },
  { "Sharann", 0.0 },
  { "Sharanya", 0.0 },
  { "Sharareh", 0.0 },
  { "Sharat", 1.0 },
  { "Sharath", 1.0 },
  { "Sharav", 1.0 },
  { "Sharaven", 0.0 },
  { "Sharay", 0.00672043 },
  { "Sharaya", 0.0 },
  { "Sharayah", 0.0 },
  { "Sharaye", 0.0 },
  { "Sharayne", 0.0 },
  { "Sharayu", 0.0 },
  { "Sharbel", 1.0 },
  { "Sharda", 0.0 },
  { "Shardae", 0.0 },
  { "Shardai", 0.0 },
  { "Shardasha", 0.0 },
  { "Shardasia", 0.0 },
  { "Sharday", 0.0 },
  { "Shardaye", 0.0 },
  { "Sharde", 0.0 },
  { "Shardea", 0.0 },
  { "Shardee", 0.0 },
  { "Shardell", 0.0 },
  { "Shardey", 0.0 },
  { "Shardi", 0.0 },
  { "Shardia", 0.0 },
  { "Shardonay", 0.0 },
  { "Shardonnay", 0.0 },
  { "Shardul", 1.0 },
  { "Share", 0.0 },
  { "Sharea", 0.0 },
  { "Sharease", 0.0 },
  { "Shareca", 0.0 },
  { "Sharece", 0.0 },
  { "Shareda", 0.0 },
  { "Sharee", 0.0 },
  { "Shareece", 0.0 },
  { "Shareeda", 0.0 },
  { "Shareef", 1.0 },
  { "Shareefah", 0.0 },
  { "Shareek", 1.0 },
  { "Shareeka", 0.0 },
  { "Shareem", 1.0 },
  { "Shareema", 0.0 },
  { "Shareen", 0.0 },
  { "Shareena", 0.0 },
  { "Sharees", 0.0 },
  { "Shareese", 0.0 },
  { "Shareeta", 0.0 },
  { "Shareif", 1.0 },
  { "Shareika", 0.0 },
  { "Shareka", 0.0 },
  { "Sharekia", 0.0 },
  { "Sharel", 0.0 },
  { "Sharell", 0.011577424 },
  { "Sharella", 0.0 },
  { "Sharelle", 0.0 },
  { "Sharema", 0.0 },
  { "Sharen", 0.0 },
  { "Sharena", 0.0 },
  { "Sharenda", 0.0 },
  { "Sharene", 0.0 },
  { "Sharenna", 0.0 },
  { "Shareny", 0.0 },
  { "Sharesa", 0.0 },
  { "Sharese", 0.0 },
  { "Sharessa", 0.0 },
  { "Sharesse", 0.0 },
  { "Shareta", 0.0 },
  { "Sharetha", 0.0 },
  { "Sharetta", 0.0 },
  { "Sharette", 0.0 },
  { "Shareva", 0.0 },
  { "Sharey", 0.0 },
  { "Sharhonda", 0.0 },
  { "Shari", 0.0011245843 },
  { "Sharia", 0.0 },
  { "Shariah", 0.0 },
  { "Sharian", 0.0 },
  { "Shariana", 0.0 },
  { "Shariann", 0.0 },
  { "Sharianne", 0.0 },
  { "Sharica", 0.0 },
  { "Sharice", 0.0 },
  { "Sharicka", 0.0 },
  { "Sharida", 0.0 },
  { "Sharidan", 0.0 },
  { "Shariden", 0.0 },
  { "Sharie", 0.0 },
  { "Shariece", 0.0 },
  { "Sharief", 1.0 },
  { "Sharieff", 1.0 },
  { "Sharieka", 0.0 },
  { "Sharielle", 0.0 },
  { "Sharif", 0.9980261 },
  { "Sharifa", 0.0 },
  { "Sharifah", 0.0 },
  { "Shariff", 1.0 },
  { "Sharik", 0.0 },
  { "Sharika", 0.0 },
  { "Sharikka", 0.0 },
  { "Sharil", 0.0 },
  { "Sharilee", 0.0 },
  { "Sharilyn", 0.0 },
  { "Sharilynn", 0.0 },
  { "Sharima", 0.0 },
  { "Sharimar", 0.0 },
  { "Sharin", 0.0 },
  { "Sharina", 0.0 },
  { "Sharinda", 0.0 },
  { "Sharine", 0.0 },
  { "Sharini", 0.0 },
  { "Sharinna", 0.0 },
  { "Sharion", 0.0 },
  { "Shariq", 1.0 },
  { "Sharis", 0.0 },
  { "Sharisa", 0.0 },
  { "Sharise", 0.0 },
  { "Sharissa", 0.0 },
  { "Sharisse", 0.0 },
  { "Sharita", 0.0 },
  { "Sharitha", 0.0 },
  { "Sharitta", 0.0 },
  { "Sharity", 0.0 },
  { "Sharitza", 0.0 },
  { "Shariya", 0.0 },
  { "Shariyah", 0.0 },
  { "Shariyf", 1.0 },
  { "Shariyka", 0.0 },
  { "Sharkia", 0.0 },
  { "Sharl", 0.0 },
  { "Sharla", 0.0 },
  { "Sharlan", 0.0 },
  { "Sharlanda", 0.0 },
  { "Sharlane", 0.0 },
  { "Sharlayne", 0.0 },
  { "Sharlean", 0.0 },
  { "Sharlee", 0.0 },
  { "Sharleen", 0.0 },
  { "Sharleene", 0.0 },
  { "Sharleez", 0.0 },
  { "Sharlen", 0.0 },
  { "Sharlena", 0.0 },
  { "Sharlene", 0.0 },
  { "Sharlet", 0.0 },
  { "Sharletha", 0.0 },
  { "Sharlett", 0.0 },
  { "Sharletta", 0.0 },
  { "Sharlette", 0.0 },
  { "Sharley", 0.0 },
  { "Sharli", 0.0 },
  { "Sharlie", 0.0 },
  { "Sharlin", 0.0 },
  { "Sharlinda", 0.0 },
  { "Sharline", 0.0 },
  { "Sharlisa", 0.0 },
  { "Sharlita", 0.0 },
  { "Sharlize", 0.0 },
  { "Sharlon", 0.0 },
  { "Sharlonda", 0.0 },
  { "Sharlot", 0.0 },
  { "Sharlotte", 0.0 },
  { "Sharly", 0.0 },
  { "Sharlyn", 0.0 },
  { "Sharlyne", 0.0 },
  { "Sharlynn", 0.0 },
  { "Sharma", 0.0 },
  { "Sharmae", 0.0 },
  { "Sharmain", 0.0 },
  { "Sharmaine", 0.0029886432 },
  { "Sharmake", 1.0 },
  { "Sharman", 0.026871402 },
  { "Sharmane", 0.0 },
  { "Sharmar", 1.0 },
  { "Sharmarke", 1.0 },
  { "Sharmayne", 0.0 },
  { "Sharmeen", 0.0 },
  { "Sharmeka", 0.0 },
  { "Sharmel", 0.0 },
  { "Sharmell", 0.0 },
  { "Sharmen", 0.0 },
  { "Sharmeta", 0.0 },
  { "Sharmia", 0.0 },
  { "Sharmika", 0.0 },
  { "Sharmila", 0.0 },
  { "Sharmilla", 0.0 },
  { "Sharmin", 0.0 },
  { "Sharmine", 0.0 },
  { "Sharmon", 0.015053763 },
  { "Sharmonique", 0.0 },
  { "Sharmyn", 0.0 },
  { "Sharn", 0.53061223 },
  { "Sharna", 0.0 },
  { "Sharnae", 0.0 },
  { "Sharnay", 0.0 },
  { "Sharnaye", 0.0 },
  { "Sharne", 0.0 },
  { "Sharnea", 0.0 },
  { "Sharnece", 0.0 },
  { "Sharnee", 0.0 },
  { "Sharneice", 0.0 },
  { "Sharneka", 0.0 },
  { "Sharnel", 0.0 },
  { "Sharnele", 0.0 },
  { "Sharnell", 0.0 },
  { "Sharnelle", 0.0 },
  { "Sharnequa", 0.0 },
  { "Sharnese", 0.0 },
  { "Sharnet", 0.0 },
  { "Sharnett", 0.0 },
  { "Sharnetta", 0.0 },
  { "Sharnette", 0.0 },
  { "Sharney", 0.0 },
  { "Sharnia", 0.0 },
  { "Sharnice", 0.0 },
  { "Sharniece", 0.0 },
  { "Sharnika", 0.0 },
  { "Sharniqua", 0.0 },
  { "Sharnique", 0.0 },
  { "Sharnise", 0.0 },
  { "Sharnisha", 0.0 },
  { "Sharnita", 0.0 },
  { "Sharnyce", 0.0 },
  { "Sharo", 0.0 },
  { "Sharocka", 0.0 },
  { "Sharod", 1.0 },
  { "Sharol", 0.0 },
  { "Sharold", 0.0 },
  { "Sharoll", 0.0 },
  { "Sharolyn", 0.0 },
  { "Sharolynn", 0.0 },
  { "Sharome", 1.0 },
  { "Sharon", 0.004282951 },
  { "Sharona", 0.0 },
  { "Sharonann", 0.0 },
  { "Sharonda", 0.0 },
  { "Sharone", 0.20435968 },
  { "Sharonette", 0.0 },
  { "Sharonica", 0.0 },
  { "Sharonlee", 0.0 },
  { "Sharonn", 0.0 },
  { "Sharonna", 0.0 },
  { "Sharonne", 0.0 },
  { "Sharoya", 0.0 },
  { "Sharquita", 0.0 },
  { "Sharra", 0.0 },
  { "Sharrae", 0.0 },
  { "Sharrah", 0.0 },
  { "Sharran", 0.0 },
  { "Sharray", 0.0 },
  { "Sharre", 0.0 },
  { "Sharree", 0.0 },
  { "Sharrel", 0.0 },
  { "Sharrell", 0.0 },
  { "Sharren", 0.0 },
  { "Sharri", 0.0 },
  { "Sharrie", 0.0 },
  { "Sharrief", 1.0 },
  { "Sharrieff", 1.0 },
  { "Sharrika", 0.0 },
  { "Sharrion", 0.0 },
  { "Sharrita", 0.0 },
  { "Sharrod", 1.0 },
  { "Sharrol", 0.0 },
  { "Sharron", 0.02148914 },
  { "Sharrona", 0.0 },
  { "Sharronda", 0.0 },
  { "Sharrone", 0.0 },
  { "Sharry", 0.0 },
  { "Sharryl", 0.0 },
  { "Sharryn", 0.0 },
  { "Shartavia", 0.0 },
  { "Sharunda", 0.0 },
  { "Sharva", 1.0 },
  { "Sharvae", 0.0 },
  { "Sharvani", 0.0 },
  { "Sharvesh", 1.0 },
  { "Sharvi", 0.0 },
  { "Sharvil", 1.0 },
  { "Sharvon", 0.0 },
  { "Sharwin", 1.0 },
  { "Shary", 0.0 },
  { "Sharya", 0.0 },
  { "Sharyah", 0.0 },
  { "Sharyan", 0.0 },
  { "Sharyce", 0.0 },
  { "Sharyia", 0.0 },
  { "Sharyl", 0.0 },
  { "Sharyle", 0.0 },
  { "Sharyn", 0.0 },
  { "Sharyne", 0.0 },
  { "Sharynn", 0.0 },
  { "Sharynne", 0.0 },
  { "Sharyon", 0.0 },
  { "Shasa", 0.0 },
  { "Shasha", 0.0 },
  { "Shashana", 0.0 },
  { "Shashank", 1.0 },
  { "Shashanna", 0.0 },
  { "Shashanth", 1.0 },
  { "Shashawn", 0.0 },
  { "Shashawna", 0.0 },
  { "Shashi", 1.0 },
  { "Shashona", 0.0 },
  { "Shashvat", 1.0 },
  { "Shashwat", 1.0 },
  { "Shaska", 0.0 },
  { "Shasmeen", 0.0 },
  { "Shasta", 0.0035302425 },
  { "Shastelin", 0.0 },
  { "Shastelyn", 0.0 },
  { "Shastina", 0.0 },
  { "Shastity", 0.0 },
  { "Shata", 0.0 },
  { "Shatae", 0.0 },
  { "Shataka", 0.0 },
  { "Shatana", 0.0 },
  { "Shatanna", 0.0 },
  { "Shatanya", 0.0 },
  { "Shatara", 0.0 },
  { "Shatarah", 0.0 },
  { "Shatari", 0.0 },
  { "Shataria", 0.0 },
  { "Shatarra", 0.0 },
  { "Shatasha", 0.0 },
  { "Shatasia", 0.0 },
  { "Shataura", 0.0 },
  { "Shatavia", 0.0 },
  { "Shatay", 0.0 },
  { "Shataya", 0.0 },
  { "Shateara", 0.0 },
  { "Shateek", 1.0 },
  { "Shateema", 0.0 },
  { "Shateia", 0.0 },
  { "Shateka", 0.0 },
  { "Shatel", 0.0 },
  { "Shatell", 0.0 },
  { "Shatema", 0.0 },
  { "Shatera", 0.0 },
  { "Shateria", 0.0 },
  { "Shaterica", 0.0 },
  { "Shaterika", 0.0 },
  { "Shaterra", 0.0 },
  { "Shaterria", 0.0 },
  { "Shaterrica", 0.0 },
  { "Shaterrika", 0.0 },
  { "Shatesha", 0.0 },
  { "Shateya", 0.0 },
  { "Shatha", 0.0 },
  { "Shatia", 0.0 },
  { "Shatiah", 0.0 },
  { "Shatiana", 0.0 },
  { "Shatica", 0.0 },
  { "Shatice", 0.0 },
  { "Shatiek", 1.0 },
  { "Shatiera", 0.0 },
  { "Shatierra", 0.0 },
  { "Shatifah", 0.0 },
  { "Shatika", 0.0 },
  { "Shatila", 0.0 },
  { "Shatima", 0.0 },
  { "Shatina", 0.0 },
  { "Shatiqua", 0.0 },
  { "Shatique", 1.0 },
  { "Shatira", 0.0 },
  { "Shatisha", 0.0 },
  { "Shatiya", 0.0 },
  { "Shaton", 0.09009009 },
  { "Shatona", 0.0 },
  { "Shatondra", 0.0 },
  { "Shatonga", 0.0 },
  { "Shatoni", 0.0 },
  { "Shatonia", 0.0 },
  { "Shatonna", 0.0 },
  { "Shatonya", 0.0 },
  { "Shatora", 0.0 },
  { "Shatori", 0.0 },
  { "Shatoria", 0.0 },
  { "Shatorria", 0.0 },
  { "Shatory", 0.0 },
  { "Shatorya", 0.0 },
  { "Shatoya", 0.0 },
  { "Shatoyia", 0.0 },
  { "Shatrice", 0.0 },
  { "Shatyra", 0.0 },
  { "Shaude", 0.0 },
  { "Shaughn", 0.9872449 },
  { "Shaughnessy", 0.052083332 },
  { "Shaul", 1.0 },
  { "Shaula", 0.0 },
  { "Shaun", 0.9570249 },
  { "Shauna", 0.001965479 },
  { "Shaunacy", 0.0 },
  { "Shaunae", 0.0 },
  { "Shaunah", 0.0 },
  { "Shaunak", 1.0 },
  { "Shaunay", 0.0 },
  { "Shaunda", 0.0 },
  { "Shaundale", 1.0 },
  { "Shaundel", 0.0 },
  { "Shaundell", 0.89361703 },
  { "Shaundra", 0.0 },
  { "Shaundrea", 0.0 },
  { "Shaundreka", 0.0 },
  { "Shaundria", 0.0 },
  { "Shaundrika", 0.0 },
  { "Shaune", 0.5699615 },
  { "Shaunea", 0.0 },
  { "Shaunece", 0.0 },
  { "Shaunee", 0.0 },
  { "Shauneen", 0.0 },
  { "Shauneille", 0.0 },
  { "Shaunel", 0.0 },
  { "Shaunell", 0.0 },
  { "Shaunelle", 0.0 },
  { "Shaunese", 0.0 },
  { "Shaunessy", 0.067567565 },
  { "Shaunetta", 0.0 },
  { "Shaunette", 0.0 },
  { "Shauney", 0.0 },
  { "Shauni", 0.0 },
  { "Shaunia", 0.0 },
  { "Shaunice", 0.0 },
  { "Shaunie", 0.0 },
  { "Shauniece", 0.0 },
  { "Shaunielle", 0.0 },
  { "Shaunika", 0.0 },
  { "Shauniqua", 0.0 },
  { "Shaunique", 0.0 },
  { "Shaunise", 0.0 },
  { "Shaunita", 0.0 },
  { "Shauniya", 0.0 },
  { "Shaunmichael", 1.0 },
  { "Shaunn", 1.0 },
  { "Shaunna", 0.0 },
  { "Shaunt", 1.0 },
  { "Shaunta", 0.0300813 },
  { "Shauntae", 0.007204611 },
  { "Shauntai", 0.0 },
  { "Shauntal", 0.0 },
  { "Shauntasia", 0.0 },
  { "Shauntavia", 0.0 },
  { "Shauntavious", 1.0 },
  { "Shauntay", 0.013483146 },
  { "Shauntaya", 0.0 },
  { "Shauntaye", 0.0 },
  { "Shaunte", 0.049244545 },
  { "Shauntea", 0.0 },
  { "Shauntee", 0.0 },
  { "Shauntel", 0.010834237 },
  { "Shauntele", 0.0 },
  { "Shauntell", 0.0 },
  { "Shauntelle", 0.0 },
  { "Shaunteria", 0.0 },
  { "Shauntez", 1.0 },
  { "Shaunti", 0.0 },
  { "Shauntia", 0.0 },
  { "Shauntiana", 0.0 },
  { "Shauntice", 0.0 },
  { "Shauntina", 0.0 },
  { "Shauntoria", 0.0 },
  { "Shauntrell", 0.0 },
  { "Shauntrice", 0.0 },
  { "Shaunya", 0.0 },
  { "Shaunyce", 0.0 },
  { "Shauri", 0.0 },
  { "Shaurice", 0.0 },
  { "Shaurya", 1.0 },
  { "Shauwn", 1.0 },
  { "Shavada", 0.0 },
  { "Shavae", 0.0 },
  { "Shavan", 0.0 },
  { "Shavana", 0.0 },
  { "Shavanda", 0.0 },
  { "Shavanna", 0.0 },
  { "Shavannah", 0.0 },
  { "Shavante", 0.0 },
  { "Shavar", 1.0 },
  { "Shavaria", 0.0 },
  { "Shavaughn", 0.05945946 },
  { "Shavaun", 0.0 },
  { "Shavawn", 0.0 },
  { "Shavell", 0.05042017 },
  { "Shavella", 0.0 },
  { "Shavelle", 0.0 },
  { "Shavera", 0.0 },
  { "Shavetta", 0.0 },
  { "Shavette", 0.0 },
  { "Shavez", 1.0 },
  { "Shavia", 0.0 },
  { "Shavina", 0.0 },
  { "Shavion", 0.0 },
  { "Shavita", 0.0 },
  { "Shavna", 0.0 },
  { "Shavon", 0.08543781 },
  { "Shavona", 0.0 },
  { "Shavonda", 0.0 },
  { "Shavondra", 0.0 },
  { "Shavone", 0.0 },
  { "Shavonia", 0.0 },
  { "Shavonn", 0.0 },
  { "Shavonna", 0.0 },
  { "Shavonne", 0.0 },
  { "Shavonta", 0.06622516 },
  { "Shavontae", 0.05050505 },
  { "Shavonte", 0.17081851 },
  { "Shavonya", 0.0 },
  { "Shavy", 0.0 },
  { "Shaw", 0.9631957 },
  { "Shawaiz", 1.0 },
  { "Shawan", 0.2 },
  { "Shawana", 0.0 },
  { "Shawanda", 0.0 },
  { "Shawandra", 0.0 },
  { "Shawann", 0.0 },
  { "Shawanna", 0.0 },
  { "Shawauna", 0.0 },
  { "Shawda", 0.0 },
  { "Shawdae", 0.0 },
  { "Shawday", 0.0 },
  { "Shawen", 1.0 },
  { "Shawhin", 1.0 },
  { "Shawkat", 1.0 },
  { "Shawki", 1.0 },
  { "Shawn", 0.90135986 },
  { "Shawna", 0.0023111464 },
  { "Shawnacy", 0.0 },
  { "Shawnae", 0.0 },
  { "Shawnah", 0.0 },
  { "Shawnalee", 0.0 },
  { "Shawnay", 0.0 },
  { "Shawnda", 0.0 },
  { "Shawndale", 0.8494624 },
  { "Shawndee", 0.0 },
  { "Shawndel", 0.52577317 },
  { "Shawndell", 0.6386139 },
  { "Shawndelle", 0.0 },
  { "Shawndi", 0.0 },
  { "Shawndra", 0.0 },
  { "Shawndre", 1.0 },
  { "Shawndrea", 0.0 },
  { "Shawndreka", 0.0 },
  { "Shawndria", 0.0 },
  { "Shawndrika", 0.0 },
  { "Shawne", 0.3468068 },
  { "Shawnea", 0.0 },
  { "Shawnece", 0.0 },
  { "Shawnee", 0.030264817 },
  { "Shawneece", 0.0 },
  { "Shawneen", 0.0 },
  { "Shawneequa", 0.0 },
  { "Shawneeque", 0.0 },
  { "Shawneice", 0.0 },
  { "Shawneka", 0.0 },
  { "Shawnell", 0.078066915 },
  { "Shawnelle", 0.0 },
  { "Shawnequa", 0.0 },
  { "Shawnese", 0.0 },
  { "Shawnesha", 0.0 },
  { "Shawnessy", 0.0 },
  { "Shawnett", 0.0 },
  { "Shawnetta", 0.0 },
  { "Shawnette", 0.0 },
  { "Shawney", 0.0 },
  { "Shawni", 0.0 },
  { "Shawnia", 0.0 },
  { "Shawnic", 0.0 },
  { "Shawnice", 0.0 },
  { "Shawnie", 0.0 },
  { "Shawniece", 0.0 },
  { "Shawnika", 0.0 },
  { "Shawniqua", 0.0 },
  { "Shawnique", 0.0 },
  { "Shawnise", 0.0 },
  { "Shawnisha", 0.0 },
  { "Shawnita", 0.0 },
  { "Shawnmichael", 1.0 },
  { "Shawnn", 0.7086093 },
  { "Shawnna", 0.0 },
  { "Shawnnessy", 0.0 },
  { "Shawnon", 0.0 },
  { "Shawnta", 0.061139897 },
  { "Shawntae", 0.01726122 },
  { "Shawntai", 0.0 },
  { "Shawntal", 0.0 },
  { "Shawntale", 0.0 },
  { "Shawntane", 0.0 },
  { "Shawntasia", 0.0 },
  { "Shawntavia", 0.0 },
  { "Shawntay", 0.048010975 },
  { "Shawntaya", 0.0 },
  { "Shawntaye", 0.0 },
  { "Shawnte", 0.068686865 },
  { "Shawntea", 0.0 },
  { "Shawntee", 0.0 },
  { "Shawntel", 0.014756944 },
  { "Shawntell", 0.0 },
  { "Shawntella", 0.0 },
  { "Shawntelle", 0.0 },
  { "Shawnteria", 0.0 },
  { "Shawntey", 0.0 },
  { "Shawntez", 1.0 },
  { "Shawnti", 0.0 },
  { "Shawntia", 0.0 },
  { "Shawntice", 0.0 },
  { "Shawntika", 0.0 },
  { "Shawntina", 0.0 },
  { "Shawntrell", 0.13559322 },
  { "Shawntrice", 0.0 },
  { "Shawny", 0.0 },
  { "Shawnya", 0.0 },
  { "Shawon", 0.80487806 },
  { "Shawona", 0.0 },
  { "Shawonda", 0.0 },
  { "Shawonna", 0.0 },
  { "Shawta", 0.0 },
  { "Shawun", 1.0 },
  { "Shawyn", 0.0 },
  { "Shay", 0.40737167 },
  { "Shaya", 0.533455 },
  { "Shayaan", 1.0 },
  { "Shayah", 0.0 },
  { "Shayal", 0.0 },
  { "Shayan", 0.9651233 },
  { "Shayana", 0.0 },
  { "Shayane", 0.0 },
  { "Shayann", 0.0 },
  { "Shayanna", 0.0 },
  { "Shayanne", 0.0 },
  { "Shayd", 1.0 },
  { "Shayda", 0.0 },
  { "Shayde", 0.60144925 },
  { "Shaydee", 0.0 },
  { "Shayden", 0.7483189 },
  { "Shaydie", 0.0 },
  { "Shaydin", 1.0 },
  { "Shaydon", 1.0 },
  { "Shaye", 0.21883239 },
  { "Shayeeda", 0.0 },
  { "Shayenne", 0.0 },
  { "Shayera", 0.0 },
  { "Shayla", 0.0005779364 },
  { "Shaylah", 0.0 },
  { "Shaylan", 0.15665796 },
  { "Shayle", 0.0 },
  { "Shaylea", 0.0 },
  { "Shayleah", 0.0 },
  { "Shaylee", 0.0 },
  { "Shayleen", 0.0 },
  { "Shaylei", 0.0 },
  { "Shayleigh", 0.0 },
  { "Shaylen", 0.056140352 },
  { "Shaylene", 0.0 },
  { "Shayley", 0.0 },
  { "Shayli", 0.0 },
  { "Shaylia", 0.0 },
  { "Shaylie", 0.0 },
  { "Shaylin", 0.005694761 },
  { "Shaylinn", 0.0 },
  { "Shaylla", 0.0 },
  { "Shayln", 0.0 },
  { "Shaylon", 0.34387353 },
  { "Shaylonda", 0.0 },
  { "Shaylor", 1.0 },
  { "Shaylyn", 0.0 },
  { "Shaylynn", 0.0 },
  { "Shaylynne", 0.0 },
  { "Shayma", 0.0 },
  { "Shaymaa", 0.0 },
  { "Shaymus", 1.0 },
  { "Shayn", 1.0 },
  { "Shayna", 0.00032189532 },
  { "Shaynah", 0.0 },
  { "Shayndel", 0.0 },
  { "Shayne", 0.7887024 },
  { "Shaynee", 0.0 },
  { "Shaynell", 0.0 },
  { "Shaynelle", 0.0 },
  { "Shayni", 0.0 },
  { "Shaynia", 0.0 },
  { "Shaynie", 0.0 },
  { "Shaynna", 0.0 },
  { "Shaynne", 0.0 },
  { "Shayon", 0.9357798 },
  { "Shayona", 0.0 },
  { "Shayonna", 0.0 },
  { "Shayquan", 1.0 },
  { "Shayra", 0.0 },
  { "Shayron", 0.3125 },
  { "Shayvon", 0.15151516 },
  { "Shayvonne", 0.0 },
  { "Shaz", 1.0 },
  { "Shaza", 0.0 },
  { "Shazad", 1.0 },
  { "Shazain", 1.0 },
  { "Shazaria", 0.0 },
  { "Shazeb", 1.0 },
  { "Shazia", 0.0 },
  { "Shazil", 1.0 },
  { "Shdiamond", 0.0 },
  { "She", 0.0 },
  { "Shea", 0.411028 },
  { "Sheadon", 1.0 },
  { "Sheaira", 0.0 },
  { "Sheala", 0.0 },
  { "Shealan", 0.0 },
  { "Shealee", 0.0 },
  { "Shealeen", 0.0 },
  { "Shealeigh", 0.0 },
  { "Shealene", 0.0 },
  { "Shealey", 0.0 },
  { "Sheali", 0.0 },
  { "Shealin", 0.0 },
  { "Shealy", 0.0 },
  { "Shealyn", 0.0 },
  { "Shealynn", 0.0 },
  { "Sheamus", 1.0 },
  { "Shean", 0.98571426 },
  { "Sheana", 0.0 },
  { "Sheandra", 0.0 },
  { "Sheanna", 0.0 },
  { "Sheara", 0.0 },
  { "Shearl", 0.5 },
  { "Shearon", 0.0 },
  { "Sheary", 0.0 },
  { "Sheba", 0.0 },
  { "Shebra", 0.0 },
  { "Shebria", 0.0 },
  { "Sheccid", 0.0 },
  { "Shecid", 0.0 },
  { "Shed", 1.0 },
  { "Sheddrick", 1.0 },
  { "Shederick", 1.0 },
  { "Shedonna", 0.0 },
  { "Shedric", 1.0 },
  { "Shedrich", 1.0 },
  { "Shedrick", 1.0 },
  { "Shedricka", 0.0 },
  { "Sheeba", 0.0 },
  { "Sheehan", 0.9324324 },
  { "Sheel", 1.0 },
  { "Sheela", 0.0 },
  { "Sheelagh", 0.0 },
  { "Sheelah", 0.0 },
  { "Sheema", 0.0 },
  { "Sheen", 0.5769231 },
  { "Sheena", 0.0030040802 },
  { "Sheenah", 0.0 },
  { "Sheenamarie", 0.0 },
  { "Sheene", 0.0 },
  { "Sheeneeka", 0.0 },
  { "Sheenia", 0.0 },
  { "Sheenika", 0.0 },
  { "Sheenna", 0.0 },
  { "Sheera", 0.0 },
  { "Sheere", 0.0 },
  { "Sheereen", 0.0 },
  { "Sheery", 0.0 },
  { "Sheetal", 0.0 },
  { "Sheeva", 0.0 },
  { "Shefali", 0.0 },
  { "Sheffield", 1.0 },
  { "Shehab", 1.0 },
  { "Sheharyar", 1.0 },
  { "Shehrbano", 0.0 },
  { "Shehroz", 1.0 },
  { "Shehryar", 1.0 },
  { "Shehzad", 1.0 },
  { "Shehzeen", 0.0 },
  { "Sheida", 0.0 },
  { "Sheik", 1.0 },
  { "Sheika", 0.0 },
  { "Sheikh", 0.96633 },
  { "Sheikha", 0.0 },
  { "Sheil", 0.0 },
  { "Sheila", 0.002717437 },
  { "Sheilagh", 0.0 },
  { "Sheilah", 0.0 },
  { "Sheilamarie", 0.0 },
  { "Sheilda", 0.0 },
  { "Sheili", 0.0 },
  { "Sheilia", 0.0 },
  { "Sheilla", 0.0 },
  { "Sheilly", 0.0 },
  { "Sheily", 0.0 },
  { "Sheilyn", 0.0 },
  { "Sheina", 0.0 },
  { "Sheindel", 0.0 },
  { "Sheindy", 0.0 },
  { "Sheira", 0.0 },
  { "Sheiva", 0.0 },
  { "Sheka", 0.0 },
  { "Shekayla", 0.0 },
  { "Shekeena", 0.0 },
  { "Shekeia", 0.0 },
  { "Shekeila", 0.0 },
  { "Shekeita", 0.0 },
  { "Shekeitha", 0.0 },
  { "Shekela", 0.0 },
  { "Shekelia", 0.0 },
  { "Shekema", 0.0 },
  { "Shekena", 0.0 },
  { "Shekera", 0.0 },
  { "Shekeria", 0.0 },
  { "Shekerra", 0.0 },
  { "Sheketa", 0.0 },
  { "Sheketha", 0.0 },
  { "Shekevia", 0.0 },
  { "Shekeya", 0.0 },
  { "Shekhinah", 0.0 },
  { "Shekia", 0.0 },
  { "Shekila", 0.0 },
  { "Shekima", 0.0 },
  { "Shekina", 0.0 },
  { "Shekinah", 0.0 },
  { "Shekira", 0.0 },
  { "Shekita", 0.0 },
  { "Shekitha", 0.0 },
  { "Sheku", 1.0 },
  { "Shela", 0.0 },
  { "Shelagh", 0.0 },
  { "Shelah", 0.0 },
  { "Shelaine", 0.0 },
  { "Shelan", 0.0 },
  { "Shelana", 0.0 },
  { "Shelanda", 0.0 },
  { "Shelane", 0.0 },
  { "Shelaya", 0.0 },
  { "Shelayne", 0.0 },
  { "Shelba", 0.0023353572 },
  { "Shelbe", 0.0 },
  { "Shelbea", 0.0 },
  { "Shelbee", 0.0 },
  { "Shelbert", 1.0 },
  { "Shelbey", 0.0 },
  { "Shelbi", 0.0 },
  { "Shelbia", 0.0 },
  { "Shelbie", 0.0 },
  { "Shelbra", 0.0 },
  { "Shelby", 0.10076232 },
  { "Shelbyann", 0.0 },
  { "Shelbye", 0.0 },
  { "Shelbyjean", 0.0 },
  { "Shelbyjo", 0.0 },
  { "Shelbylyn", 0.0 },
  { "Shelbylynn", 0.0 },
  { "Shelcy", 0.0 },
  { "Shelda", 0.0 },
  { "Sheldan", 1.0 },
  { "Shelden", 0.9817518 },
  { "Sheldia", 0.0 },
  { "Sheldon", 0.9863706 },
  { "Sheldrick", 1.0 },
  { "Sheldyn", 0.47826087 },
  { "Shele", 0.0 },
  { "Shelea", 0.0 },
  { "Sheleah", 0.0 },
  { "Shelee", 0.0 },
  { "Sheleen", 0.0 },
  { "Sheleena", 0.0 },
  { "Shelena", 0.0 },
  { "Shelene", 0.0 },
  { "Shelese", 0.0 },
  { "Sheleta", 0.0 },
  { "Sheletha", 0.0 },
  { "Sheletta", 0.0 },
  { "Shelette", 0.0 },
  { "Sheley", 0.0 },
  { "Sheli", 0.0 },
  { "Shelia", 0.0023205737 },
  { "Sheliah", 0.0 },
  { "Shelica", 0.0 },
  { "Shelice", 0.0 },
  { "Shelicia", 0.0 },
  { "Shelie", 0.0 },
  { "Shelika", 0.0 },
  { "Shelina", 0.0 },
  { "Shelinda", 0.0 },
  { "Shelisa", 0.0 },
  { "Shelise", 0.0 },
  { "Shelisha", 0.0 },
  { "Shelissa", 0.0 },
  { "Shelita", 0.0 },
  { "Shelitha", 0.0 },
  { "Sheliya", 0.0 },
  { "Sheliyah", 0.0 },
  { "Sheliza", 0.0 },
  { "Shell", 0.3653846 },
  { "Shella", 0.0 },
  { "Shellaine", 0.0 },
  { "Shellane", 0.0 },
  { "Shellbe", 0.0 },
  { "Shellbie", 0.0 },
  { "Shellby", 0.0 },
  { "Shelle", 0.0 },
  { "Shellea", 0.0 },
  { "Shellee", 0.0 },
  { "Shelleen", 0.0 },
  { "Shellena", 0.0 },
  { "Shellene", 0.0 },
  { "Shelley", 0.023602 },
  { "Shelli", 0.0 },
  { "Shellia", 0.0 },
  { "Shellie", 0.08212872 },
  { "Shellina", 0.0 },
  { "Shelline", 0.0 },
  { "Shellisa", 0.0 },
  { "Shellise", 0.0 },
  { "Shellsea", 0.0 },
  { "Shelly", 0.025358962 },
  { "Shellyann", 0.0 },
  { "Shellye", 0.0 },
  { "Shelma", 0.0 },
  { "Shelon", 0.0 },
  { "Shelonda", 0.0 },
  { "Shelsea", 0.0 },
  { "Shelsey", 0.0 },
  { "Shelsie", 0.0 },
  { "Shelsy", 0.0 },
  { "Shelton", 0.9855316 },
  { "Shelva", 0.012378426 },
  { "Shelvey", 0.0 },
  { "Shelvia", 0.0 },
  { "Shelvie", 0.020066889 },
  { "Shelvin", 1.0 },
  { "Shelvy", 0.16161616 },
  { "Shely", 0.0 },
  { "Shelyn", 0.0 },
  { "Shem", 0.9920255 },
  { "Shema", 0.0 },
  { "Shemaiah", 0.044444446 },
  { "Shemaine", 0.0 },
  { "Shemaka", 0.0 },
  { "Shemar", 0.9972515 },
  { "Shemara", 0.0 },
  { "Shemari", 0.6875 },
  { "Shemariah", 0.0 },
  { "Shemarion", 1.0 },
  { "Shemarr", 1.0 },
  { "Shemeca", 0.0 },
  { "Shemecka", 0.0 },
  { "Shemeeka", 0.0 },
  { "Shemeika", 0.0 },
  { "Shemeka", 0.0 },
  { "Shemekia", 0.0 },
  { "Shemera", 0.0 },
  { "Shemia", 0.0 },
  { "Shemiah", 0.0 },
  { "Shemica", 0.0 },
  { "Shemicka", 0.0 },
  { "Shemieka", 0.0 },
  { "Shemika", 0.0 },
  { "Shemira", 0.0 },
  { "Shemita", 0.0 },
  { "Shemuel", 1.0 },
  { "Shemya", 0.0 },
  { "Shen", 1.0 },
  { "Shena", 0.0 },
  { "Shenade", 0.0 },
  { "Shenae", 0.0 },
  { "Shenai", 0.0 },
  { "Shenan", 0.48076922 },
  { "Shenandoah", 0.25986078 },
  { "Shenard", 1.0 },
  { "Shenay", 0.0 },
  { "Shenaya", 0.0 },
  { "Shenaye", 0.0 },
  { "Shenda", 0.0 },
  { "Shene", 0.0 },
  { "Shenea", 0.0 },
  { "Sheneaka", 0.0 },
  { "Sheneda", 0.0 },
  { "Shenee", 0.0 },
  { "Sheneeka", 0.0 },
  { "Sheneice", 0.0 },
  { "Sheneika", 0.0 },
  { "Sheneil", 0.0 },
  { "Sheneka", 0.0 },
  { "Shenekia", 0.0 },
  { "Shenel", 0.0 },
  { "Shenell", 0.0 },
  { "Shenelle", 0.0 },
  { "Shenequa", 0.0 },
  { "Sheneque", 0.0 },
  { "Shenequia", 0.0 },
  { "Shenese", 0.0 },
  { "Sheneshia", 0.0 },
  { "Sheneta", 0.0 },
  { "Shenetha", 0.0 },
  { "Shenetra", 0.0 },
  { "Shenetta", 0.0 },
  { "Shenette", 0.0 },
  { "Sheng", 0.041450776 },
  { "Shengyeng", 0.0 },
  { "Shenia", 0.0 },
  { "Sheniah", 0.0 },
  { "Shenica", 0.0 },
  { "Shenice", 0.0 },
  { "Shenicka", 0.0 },
  { "Sheniece", 0.0 },
  { "Shenika", 0.0 },
  { "Shenikia", 0.0 },
  { "Shenikwa", 0.0 },
  { "Shenille", 0.0 },
  { "Shenina", 0.0 },
  { "Sheniqua", 0.0 },
  { "Shenique", 0.0 },
  { "Shenise", 0.0 },
  { "Shenisha", 0.0 },
  { "Shenita", 0.0 },
  { "Shenitha", 0.0 },
  { "Shenitra", 0.0 },
  { "Shenitta", 0.0 },
  { "Sheniya", 0.0 },
  { "Sheniyah", 0.0 },
  { "Shenkia", 0.0 },
  { "Shenna", 0.0 },
  { "Shennan", 0.0 },
  { "Shennel", 0.0 },
  { "Shennon", 0.0 },
  { "Shenoa", 0.0 },
  { "Shenoah", 0.0 },
  { "Shenouda", 1.0 },
  { "Shentel", 0.0 },
  { "Shentell", 0.0 },
  { "Sheny", 0.0 },
  { "Shenya", 0.0 },
  { "Shenyah", 0.0 },
  { "Sheon", 0.5 },
  { "Sheona", 0.0 },
  { "Sheonna", 0.0 },
  { "Sheonta", 0.0 },
  { "Shep", 1.0 },
  { "Shepard", 0.9968593 },
  { "Sheperd", 1.0 },
  { "Shephanie", 0.0 },
  { "Shephard", 1.0 },
  { "Shephen", 1.0 },
  { "Shepherd", 0.9971894 },
  { "Sheppard", 1.0 },
  { "Shequan", 1.0 },
  { "Shequana", 0.0 },
  { "Shequanda", 0.0 },
  { "Shequanna", 0.0 },
  { "Shequetta", 0.0 },
  { "Shequila", 0.0 },
  { "Shequilla", 0.0 },
  { "Shequille", 1.0 },
  { "Shequira", 0.0 },
  { "Shequita", 0.0 },
  { "Shequitta", 0.0 },
  { "Sher", 0.49593496 },
  { "Shera", 0.0 },
  { "Sheradyn", 0.0 },
  { "Sherae", 0.0 },
  { "Sherah", 0.0 },
  { "Sheraine", 0.0 },
  { "Sheral", 0.0 },
  { "Sherald", 0.26666668 },
  { "Sheraldine", 0.0 },
  { "Sheralee", 0.0 },
  { "Sherall", 0.0 },
  { "Sheralyn", 0.0 },
  { "Sheran", 0.0 },
  { "Sheranda", 0.0 },
  { "Sherard", 1.0 },
  { "Sheraton", 1.0 },
  { "Sheray", 0.0 },
  { "Sheraya", 0.0 },
  { "Sherayne", 0.0 },
  { "Sheraz", 1.0 },
  { "Sherburne", 1.0 },
  { "Sherby", 0.0 },
  { "Sherdina", 0.0 },
  { "Shere", 0.0 },
  { "Sherea", 0.0 },
  { "Sherease", 0.0 },
  { "Shereca", 0.0 },
  { "Sherece", 0.0 },
  { "Shereda", 0.0 },
  { "Sheree", 0.0 },
  { "Shereece", 0.0 },
  { "Shereef", 1.0 },
  { "Shereen", 0.0 },
  { "Shereena", 0.0 },
  { "Shereese", 0.0 },
  { "Shereeta", 0.0 },
  { "Shereika", 0.0 },
  { "Shereka", 0.0 },
  { "Sherel", 0.0 },
  { "Sherelene", 0.0 },
  { "Sherell", 0.0033783785 },
  { "Sherelle", 0.0 },
  { "Sherelyn", 0.0 },
  { "Sheren", 0.0 },
  { "Sherena", 0.0 },
  { "Sherene", 0.0 },
  { "Sheresa", 0.0 },
  { "Sherese", 0.0 },
  { "Sheresse", 0.0 },
  { "Shereta", 0.0 },
  { "Sheretha", 0.0 },
  { "Sheretta", 0.0 },
  { "Sherette", 0.0 },
  { "Sherezade", 0.0 },
  { "Sherhonda", 0.0 },
  { "Sheri", 0.0018840795 },
  { "Sheria", 0.0 },
  { "Sheriah", 0.0 },
  { "Sherial", 0.0 },
  { "Sherian", 0.0 },
  { "Sheriann", 0.0 },
  { "Sherianne", 0.0 },
  { "Sherica", 0.0 },
  { "Sherice", 0.0 },
  { "Sherick", 1.0 },
  { "Shericka", 0.0 },
  { "Sherida", 0.0 },
  { "Sheridan", 0.33398917 },
  { "Sheriden", 0.0 },
  { "Sheridon", 0.0 },
  { "Sheridyn", 0.0 },
  { "Sherie", 0.0 },
  { "Sheriece", 0.0 },
  { "Sheriee", 0.0 },
  { "Sherief", 1.0 },
  { "Sheriel", 0.0 },
  { "Sherif", 1.0 },
  { "Sherifat", 0.0 },
  { "Sheriff", 1.0 },
  { "Sherika", 0.0 },
  { "Sheril", 0.007052186 },
  { "Sherile", 0.0 },
  { "Sherilee", 0.0 },
  { "Sherill", 0.05648148 },
  { "Sherille", 0.0 },
  { "Sherilyn", 0.0 },
  { "Sherilynn", 0.0 },
  { "Sherin", 0.0 },
  { "Sherina", 0.0 },
  { "Sherinda", 0.0 },
  { "Sherine", 0.0 },
  { "Sherion", 0.0 },
  { "Sherisa", 0.0 },
  { "Sherise", 0.0 },
  { "Sherissa", 0.0 },
  { "Sherisse", 0.0 },
  { "Sherita", 0.0 },
  { "Sheritha", 0.0 },
  { "Sheritta", 0.0 },
  { "Sheriyah", 0.0 },
  { "Sherkia", 0.0 },
  { "Sherkita", 0.0 },
  { "Sherl", 0.014124294 },
  { "Sherla", 0.0 },
  { "Sherlanda", 0.0 },
  { "Sherle", 0.0 },
  { "Sherlean", 0.0 },
  { "Sherlee", 0.0 },
  { "Sherleen", 0.0 },
  { "Sherlena", 0.0 },
  { "Sherlene", 0.0 },
  { "Sherley", 0.17813765 },
  { "Sherli", 0.0 },
  { "Sherlie", 0.0 },
  { "Sherlin", 0.0 },
  { "Sherline", 0.0 },
  { "Sherlita", 0.0 },
  { "Sherlly", 0.0 },
  { "Sherlock", 1.0 },
  { "Sherlon", 0.0 },
  { "Sherlonda", 0.0 },
  { "Sherly", 0.0 },
  { "Sherlyn", 0.0010373445 },
  { "Sherlyne", 0.0 },
  { "Sherlynn", 0.0 },
  { "Sherma", 0.0 },
  { "Shermain", 0.5 },
  { "Shermaine", 0.25403818 },
  { "Sherman", 0.9948103 },
  { "Shermane", 0.0 },
  { "Shermar", 1.0 },
  { "Shermeka", 0.0 },
  { "Shermia", 0.0 },
  { "Shermica", 0.0 },
  { "Shermika", 0.0 },
  { "Shermon", 1.0 },
  { "Shermona", 0.0 },
  { "Shermya", 0.0 },
  { "Sherna", 0.0 },
  { "Shernard", 1.0 },
  { "Sherneice", 0.0 },
  { "Shernell", 0.0 },
  { "Shernette", 0.0 },
  { "Shernice", 0.0 },
  { "Shernika", 0.0 },
  { "Shernita", 0.0 },
  { "Sherod", 1.0 },
  { "Sherokee", 0.0 },
  { "Sherol", 0.0 },
  { "Sherolyn", 0.0 },
  { "Sherome", 1.0 },
  { "Sheron", 0.102073364 },
  { "Sherona", 0.0 },
  { "Sheronda", 0.0 },
  { "Sherone", 0.2 },
  { "Sheronica", 0.0 },
  { "Sheronne", 0.0 },
  { "Sherquita", 0.0 },
  { "Sherr", 0.0 },
  { "Sherra", 0.0 },
  { "Sherrae", 0.0 },
  { "Sherraine", 0.0 },
  { "Sherral", 0.0 },
  { "Sherralyn", 0.0 },
  { "Sherran", 0.0 },
  { "Sherrard", 1.0 },
  { "Sherray", 0.0 },
  { "Sherre", 0.0 },
  { "Sherrea", 0.0 },
  { "Sherree", 0.0 },
  { "Sherreka", 0.0 },
  { "Sherrel", 0.12727273 },
  { "Sherrell", 0.13658991 },
  { "Sherrelle", 0.0 },
  { "Sherren", 0.0 },
  { "Sherrena", 0.0 },
  { "Sherrene", 0.0 },
  { "Sherrese", 0.0 },
  { "Sherretta", 0.0 },
  { "Sherrey", 0.0 },
  { "Sherri", 0.0021017685 },
  { "Sherria", 0.0 },
  { "Sherrian", 0.0 },
  { "Sherriann", 0.0 },
  { "Sherrianne", 0.0 },
  { "Sherrica", 0.0 },
  { "Sherrice", 0.0 },
  { "Sherrick", 0.9707602 },
  { "Sherricka", 0.0 },
  { "Sherridan", 0.0 },
  { "Sherrie", 0.00031322968 },
  { "Sherriel", 0.0 },
  { "Sherrika", 0.0 },
  { "Sherril", 0.08619718 },
  { "Sherrilee", 0.0 },
  { "Sherrill", 0.21636897 },
  { "Sherrille", 0.0 },
  { "Sherrilyn", 0.0 },
  { "Sherrilynn", 0.0 },
  { "Sherrin", 0.0 },
  { "Sherrina", 0.0 },
  { "Sherrine", 0.0 },
  { "Sherrion", 0.0 },
  { "Sherris", 0.0 },
  { "Sherrise", 0.0 },
  { "Sherrita", 0.0 },
  { "Sherritta", 0.0 },
  { "Sherrl", 0.0 },
  { "Sherrlyn", 0.0 },
  { "Sherrod", 0.99739176 },
  { "Sherrol", 0.0 },
  { "Sherron", 0.14781603 },
  { "Sherronda", 0.0 },
  { "Sherrone", 0.0 },
  { "Sherry", 0.0042327675 },
  { "Sherryann", 0.0 },
  { "Sherrye", 0.0 },
  { "Sherryl", 0.0012171373 },
  { "Sherryle", 0.0 },
  { "Sherrylee", 0.0 },
  { "Sherryll", 0.0 },
  { "Sherrylyn", 0.0 },
  { "Sherrylynn", 0.0 },
  { "Sherryn", 0.0 },
  { "Shertia", 0.0 },
  { "Sherunda", 0.0 },
  { "Shervin", 1.0 },
  { "Shervon", 0.0 },
  { "Shervonda", 0.0 },
  { "Shervonne", 0.0 },
  { "Sherwanda", 0.0 },
  { "Sherwin", 0.995925 },
  { "Sherwood", 0.99804235 },
  { "Sherwyn", 1.0 },
  { "Shery", 0.0 },
  { "Sheryal", 0.0 },
  { "Sheryce", 0.0 },
  { "Sherye", 0.0 },
  { "Sheryel", 0.0 },
  { "Sheryl", 0.002536333 },
  { "Sherylann", 0.0 },
  { "Sheryle", 0.0 },
  { "Sheryll", 0.0 },
  { "Sheryln", 0.0 },
  { "Sherylyn", 0.0 },
  { "Sheryn", 0.0 },
  { "Sheshe", 0.0 },
  { "Shesia", 0.0 },
  { "Shetal", 0.0 },
  { "Shetara", 0.0 },
  { "Shetarra", 0.0 },
  { "Sheteka", 0.0 },
  { "Shetera", 0.0 },
  { "Sheterica", 0.0 },
  { "Sheterrica", 0.0 },
  { "Shetia", 0.0 },
  { "Shetika", 0.0 },
  { "Sheva", 0.0 },
  { "Shevaun", 0.0 },
  { "Shevawn", 0.0 },
  { "Shevell", 0.0 },
  { "Shevella", 0.0 },
  { "Shevelle", 0.0 },
  { "Shevetta", 0.0 },
  { "Shevette", 0.0 },
  { "Shevi", 0.0 },
  { "Shevin", 1.0 },
  { "Shevon", 0.03456221 },
  { "Shevonda", 0.0 },
  { "Shevone", 0.0 },
  { "Shevonne", 0.0 },
  { "Shevy", 0.07239819 },
  { "Shewana", 0.0 },
  { "Shewanda", 0.0 },
  { "Shewanna", 0.0 },
  { "Shey", 0.50729924 },
  { "Sheya", 1.0 },
  { "Sheyann", 0.0 },
  { "Sheyanna", 0.0 },
  { "Sheyanne", 0.0 },
  { "Sheyda", 0.0 },
  { "Sheyenne", 0.0 },
  { "Sheyla", 0.0 },
  { "Sheyli", 0.0 },
  { "Sheylin", 0.0 },
  { "Sheylla", 0.0 },
  { "Sheyly", 0.0 },
  { "Sheylyn", 0.0 },
  { "Sheylynn", 0.0 },
  { "Sheyna", 0.0 },
  { "Sheza", 0.0 },
  { "Shi", 0.20304568 },
  { "Shia", 0.61064893 },
  { "Shiah", 0.3442623 },
  { "Shiala", 0.0 },
  { "Shian", 0.014117647 },
  { "Shiana", 0.0 },
  { "Shiane", 0.0 },
  { "Shiann", 0.0 },
  { "Shianna", 0.0 },
  { "Shianne", 0.0 },
  { "Shiara", 0.0 },
  { "Shiasia", 0.0 },
  { "Shibani", 0.0 },
  { "Shida", 0.0 },
  { "Shie", 0.54545456 },
  { "Shieda", 0.0 },
  { "Shieka", 0.0 },
  { "Shiela", 0.0 },
  { "Shielah", 0.0 },
  { "Shielda", 0.0 },
  { "Shields", 1.0 },
  { "Shiena", 0.0 },
  { "Shiera", 0.0 },
  { "Shierra", 0.0 },
  { "Shiesha", 0.0 },
  { "Shifa", 0.0 },
  { "Shiffon", 0.0 },
  { "Shiffy", 0.0 },
  { "Shifra", 0.0 },
  { "Shig", 1.0 },
  { "Shige", 1.0 },
  { "Shigeki", 1.0 },
  { "Shigeko", 0.0 },
  { "Shigemi", 1.0 },
  { "Shigeno", 0.0 },
  { "Shigenobu", 1.0 },
  { "Shigeo", 1.0 },
  { "Shigeru", 1.0 },
  { "Shigeto", 1.0 },
  { "Shigetoshi", 1.0 },
  { "Shigeyuki", 1.0 },
  { "Shihab", 1.0 },
  { "Shiheem", 1.0 },
  { "Shiheim", 1.0 },
  { "Shiho", 0.0 },
  { "Shikara", 0.0 },
  { "Shikari", 0.0 },
  { "Shikeem", 1.0 },
  { "Shikela", 0.0 },
  { "Shikema", 0.0 },
  { "Shikera", 0.0 },
  { "Shikeria", 0.0 },
  { "Shiketa", 0.0 },
  { "Shikeyla", 0.0 },
  { "Shikha", 0.0 },
  { "Shikhar", 1.0 },
  { "Shikia", 0.0 },
  { "Shikina", 0.0 },
  { "Shikira", 0.0 },
  { "Shikita", 0.0 },
  { "Shila", 0.0 },
  { "Shilah", 0.0 },
  { "Shilat", 0.0 },
  { "Shilee", 0.0 },
  { "Shiley", 0.0 },
  { "Shilla", 0.0 },
  { "Shillae", 0.0 },
  { "Shilo", 0.32384053 },
  { "Shiloah", 0.0 },
  { "Shiloe", 0.0 },
  { "Shiloh", 0.33371773 },
  { "Shilonda", 0.0 },
  { "Shilow", 0.0 },
  { "Shilpa", 0.0 },
  { "Shilyn", 0.0 },
  { "Shilynn", 0.0 },
  { "Shima", 0.0 },
  { "Shimara", 0.0 },
  { "Shimaya", 0.0 },
  { "Shimeek", 1.0 },
  { "Shimeeka", 0.0 },
  { "Shimeka", 0.0 },
  { "Shimekia", 0.0 },
  { "Shimere", 0.0 },
  { "Shimika", 0.0 },
  { "Shimira", 0.0 },
  { "Shimon", 1.0 },
  { "Shimshon", 1.0 },
  { "Shimya", 0.0 },
  { "Shin", 0.87096775 },
  { "Shina", 0.0 },
  { "Shinae", 0.0 },
  { "Shinaya", 0.0 },
  { "Shinda", 0.0 },
  { "Shindana", 0.0 },
  { "Shine", 0.36597937 },
  { "Shinea", 0.0 },
  { "Shinead", 0.0 },
  { "Shinece", 0.0 },
  { "Shinee", 0.0 },
  { "Shineka", 0.0 },
  { "Shinell", 0.0 },
  { "Shinelle", 0.0 },
  { "Shinequa", 0.0 },
  { "Shinese", 0.0 },
  { "Shinesha", 0.0 },
  { "Shinetta", 0.0 },
  { "Shing", 0.5 },
  { "Shingo", 1.0 },
  { "Shinia", 0.0 },
  { "Shiniah", 0.0 },
  { "Shinice", 0.0 },
  { "Shinichi", 1.0 },
  { "Shiniece", 0.0 },
  { "Shinika", 0.0 },
  { "Shiniqua", 0.0 },
  { "Shinique", 0.0 },
  { "Shinise", 0.0 },
  { "Shinita", 0.0 },
  { "Shiniya", 0.0 },
  { "Shinji", 1.0 },
  { "Shinna", 0.0 },
  { "Shinnosuke", 1.0 },
  { "Shinobi", 1.0 },
  { "Shinobu", 0.5 },
  { "Shintaro", 1.0 },
  { "Shiny", 0.0 },
  { "Shinya", 0.57894737 },
  { "Shioban", 0.0 },
  { "Shiobhan", 0.0 },
  { "Shiomara", 0.0 },
  { "Shion", 0.7205882 },
  { "Shiona", 0.0 },
  { "Shionna", 0.0 },
  { "Shiori", 0.0 },
  { "Shiphrah", 0.0 },
  { "Shiqi", 0.0 },
  { "Shiquan", 1.0 },
  { "Shiquana", 0.0 },
  { "Shiquita", 0.0 },
  { "Shir", 0.0 },
  { "Shira", 0.0 },
  { "Shirae", 0.0 },
  { "Shirah", 0.0 },
  { "Shiran", 0.0 },
  { "Shiraz", 0.6761364 },
  { "Shire", 0.0 },
  { "Shiree", 0.0 },
  { "Shireen", 0.0 },
  { "Shireka", 0.0 },
  { "Shirel", 0.0 },
  { "Shirell", 0.0 },
  { "Shirelle", 0.0 },
  { "Shirely", 0.0 },
  { "Shirena", 0.0 },
  { "Shirene", 0.0 },
  { "Shiretta", 0.0 },
  { "Shirey", 0.0 },
  { "Shiri", 0.0 },
  { "Shirika", 0.0 },
  { "Shirin", 0.0 },
  { "Shirita", 0.0 },
  { "Shirl", 0.25296444 },
  { "Shirla", 0.0 },
  { "Shirlane", 0.0 },
  { "Shirlann", 0.0 },
  { "Shirle", 0.0 },
  { "Shirlean", 0.0 },
  { "Shirlee", 0.00087596354 },
  { "Shirleen", 0.0 },
  { "Shirleeta", 0.0 },
  { "Shirlena", 0.0 },
  { "Shirlene", 0.0 },
  { "Shirletha", 0.0 },
  { "Shirletta", 0.0 },
  { "Shirlette", 0.0 },
  { "Shirley", 0.013750697 },
  { "Shirleyan", 0.0 },
  { "Shirleyann", 0.0 },
  { "Shirleye", 0.0 },
  { "Shirleymae", 0.0 },
  { "Shirli", 0.0 },
  { "Shirlie", 0.0 },
  { "Shirlina", 0.0 },
  { "Shirline", 0.0 },
  { "Shirlon", 0.0 },
  { "Shirly", 0.025280898 },
  { "Shirlye", 0.0 },
  { "Shirlyn", 0.0 },
  { "Shirlynn", 0.0 },
  { "Shiro", 1.0 },
  { "Shiron", 0.26666668 },
  { "Shironda", 0.0 },
  { "Shirrel", 0.6296296 },
  { "Shirrell", 0.3131313 },
  { "Shirret", 0.0 },
  { "Shirron", 0.0 },
  { "Shiryl", 0.0 },
  { "Shishir", 1.0 },
  { "Shital", 0.0 },
  { "Shiv", 1.0 },
  { "Shiva", 0.6066351 },
  { "Shivaan", 1.0 },
  { "Shivaansh", 1.0 },
  { "Shivaay", 1.0 },
  { "Shivali", 0.0 },
  { "Shivam", 1.0 },
  { "Shivan", 1.0 },
  { "Shivana", 0.0 },
  { "Shivang", 1.0 },
  { "Shivangi", 0.0 },
  { "Shivani", 0.0 },
  { "Shivank", 1.0 },
  { "Shivansh", 1.0 },
  { "Shivanshi", 0.0 },
  { "Shivany", 0.0 },
  { "Shivanya", 0.0 },
  { "Shivaun", 0.0 },
  { "Shivay", 1.0 },
  { "Shiven", 1.0 },
  { "Shivesh", 1.0 },
  { "Shivi", 0.0 },
  { "Shivika", 0.0 },
  { "Shivin", 1.0 },
  { "Shivon", 0.0 },
  { "Shivonne", 0.0 },
  { "Shivonni", 0.0 },
  { "Shivraj", 1.0 },
  { "Shivya", 0.0 },
  { "Shiwana", 0.0 },
  { "Shiwanda", 0.0 },
  { "Shiya", 0.16981132 },
  { "Shiyah", 0.0 },
  { "Shiyan", 0.0 },
  { "Shiyana", 0.0 },
  { "Shiyann", 0.0 },
  { "Shiyanna", 0.0 },
  { "Shiyanne", 0.0 },
  { "Shiye", 1.0 },
  { "Shiza", 0.0 },
  { "Shizu", 0.0 },
  { "Shizue", 0.0 },
  { "Shizuka", 0.0 },
  { "Shizuko", 0.0 },
  { "Shizuo", 1.0 },
  { "Shizuye", 0.0 },
  { "Shizuyo", 0.0 },
  { "Shjon", 1.0 },
  { "Shlanda", 0.0 },
  { "Shley", 0.0 },
  { "Shloak", 1.0 },
  { "Shloima", 1.0 },
  { "Shloime", 1.0 },
  { "Shloimy", 1.0 },
  { "Shlok", 1.0 },
  { "Shloka", 0.0 },
  { "Shloke", 1.0 },
  { "Shloma", 1.0 },
  { "Shlome", 1.0 },
  { "Shlomie", 1.0 },
  { "Shlomit", 0.0 },
  { "Shlomo", 1.0 },
  { "Shlomy", 1.0 },
  { "Shlonda", 0.0 },
  { "Shmar", 1.0 },
  { "Shmaya", 1.0 },
  { "Shmeil", 1.0 },
  { "Shmeka", 0.0 },
  { "Shmiel", 1.0 },
  { "Shmuel", 1.0 },
  { "Shmya", 0.0 },
  { "Shnea", 0.0 },
  { "Shneor", 1.0 },
  { "Shnequa", 0.0 },
  { "Shneur", 1.0 },
  { "Shnika", 0.0 },
  { "Shnita", 0.0 },
  { "Shniya", 0.0 },
  { "Shniyah", 0.0 },
  { "Shnya", 0.0 },
  { "Sho", 1.0 },
  { "Shoaib", 1.0 },
  { "Shoan", 1.0 },
  { "Shogo", 1.0 },
  { "Shohei", 1.0 },
  { "Shohjahon", 1.0 },
  { "Shohn", 1.0 },
  { "Shoichi", 1.0 },
  { "Shoji", 1.0 },
  { "Shoko", 0.0 },
  { "Shola", 0.15625 },
  { "Sholanda", 0.0 },
  { "Sholem", 1.0 },
  { "Sholom", 1.0 },
  { "Sholonda", 0.0 },
  { "Shoma", 1.0 },
  { "Shomari", 0.9374449 },
  { "Shon", 0.934115 },
  { "Shona", 0.0 },
  { "Shonae", 0.0 },
  { "Shonda", 0.0 },
  { "Shondala", 0.0 },
  { "Shondale", 0.84615386 },
  { "Shondalyn", 0.0 },
  { "Shondel", 0.5 },
  { "Shondell", 0.5250501 },
  { "Shondelle", 0.0 },
  { "Shondi", 0.0 },
  { "Shondia", 0.0 },
  { "Shondolyn", 0.0 },
  { "Shondra", 0.0 },
  { "Shondre", 1.0 },
  { "Shondrea", 0.0 },
  { "Shondreka", 0.0 },
  { "Shondria", 0.0 },
  { "Shondrick", 1.0 },
  { "Shondrika", 0.0 },
  { "Shone", 0.86787564 },
  { "Shonee", 0.0 },
  { "Shoneen", 0.0 },
  { "Shoneka", 0.0 },
  { "Shonell", 0.0 },
  { "Shonelle", 0.0 },
  { "Shonett", 0.0 },
  { "Shonetta", 0.0 },
  { "Shonette", 0.0 },
  { "Shoney", 0.3125 },
  { "Shong", 1.0 },
  { "Shoni", 0.0 },
  { "Shonia", 0.0 },
  { "Shonica", 0.0 },
  { "Shonice", 0.0 },
  { "Shonie", 0.0 },
  { "Shoniece", 0.0 },
  { "Shonika", 0.0 },
  { "Shoniqua", 0.0 },
  { "Shonique", 0.0 },
  { "Shonita", 0.0 },
  { "Shonn", 0.9797571 },
  { "Shonna", 0.0 },
  { "Shonnell", 0.0 },
  { "Shonnetta", 0.0 },
  { "Shonnette", 0.0 },
  { "Shonni", 0.0 },
  { "Shonnie", 0.0 },
  { "Shonnita", 0.0 },
  { "Shonnon", 0.42592594 },
  { "Shonski", 1.0 },
  { "Shonta", 0.04410012 },
  { "Shontae", 0.01173709 },
  { "Shontai", 0.0 },
  { "Shontal", 0.0 },
  { "Shontasia", 0.0 },
  { "Shontavia", 0.0 },
  { "Shontavius", 1.0 },
  { "Shontay", 0.017331023 },
  { "Shontaya", 0.0 },
  { "Shontaye", 0.0 },
  { "Shonte", 0.073529415 },
  { "Shontea", 0.0 },
  { "Shontee", 0.0 },
  { "Shontel", 0.024050632 },
  { "Shontell", 0.009118541 },
  { "Shontella", 0.0 },
  { "Shontelle", 0.0 },
  { "Shonteria", 0.0 },
  { "Shonterria", 0.0 },
  { "Shontez", 1.0 },
  { "Shontia", 0.0 },
  { "Shontice", 0.0 },
  { "Shontina", 0.0 },
  { "Shontisha", 0.0 },
  { "Shontoria", 0.0 },
  { "Shontoya", 0.0 },
  { "Shontrell", 0.0 },
  { "Shontrice", 0.0 },
  { "Shonya", 0.0 },
  { "Shooter", 1.0 },
  { "Shoran", 0.0 },
  { "Shorn", 1.0 },
  { "Shoronda", 0.0 },
  { "Shorty", 1.0 },
  { "Shoshana", 0.0 },
  { "Shoshanah", 0.0 },
  { "Shoshanna", 0.0 },
  { "Shoshannah", 0.0 },
  { "Shoshawna", 0.0 },
  { "Shoshona", 0.0 },
  { "Shoshone", 0.0 },
  { "Shoso", 1.0 },
  { "Shota", 1.0 },
  { "Shotaro", 1.0 },
  { "Shou", 1.0 },
  { "Shoua", 0.22739726 },
  { "Shouana", 0.0 },
  { "Shoumik", 1.0 },
  { "Shoun", 1.0 },
  { "Shounda", 0.0 },
  { "Shourya", 1.0 },
  { "Shovan", 0.0 },
  { "Shovon", 0.0 },
  { "Shovonne", 0.0 },
  { "Showanda", 0.0 },
  { "Shown", 0.9259259 },
  { "Shozo", 1.0 },
  { "Shpresa", 0.0 },
  { "Shqipe", 0.0 },
  { "Shraddha", 0.0 },
  { "Shradha", 0.0 },
  { "Shraga", 1.0 },
  { "Shravan", 1.0 },
  { "Shravani", 0.0 },
  { "Shravya", 0.0 },
  { "Shray", 1.0 },
  { "Shrea", 0.0 },
  { "Shree", 0.015421115 },
  { "Shreehan", 1.0 },
  { "Shreeja", 0.0 },
  { "Shreen", 0.0 },
  { "Shreena", 0.0 },
  { "Shreenidhi", 0.0 },
  { "Shreenika", 0.0 },
  { "Shreeta", 0.0 },
  { "Shreeya", 0.0 },
  { "Shrena", 0.0 },
  { "Shrenik", 1.0 },
  { "Shresht", 1.0 },
  { "Shreshta", 0.0 },
  { "Shresta", 0.0 },
  { "Shrey", 1.0 },
  { "Shreya", 0.0 },
  { "Shreyaan", 1.0 },
  { "Shreyan", 1.0 },
  { "Shreyank", 1.0 },
  { "Shreyans", 1.0 },
  { "Shreyansh", 1.0 },
  { "Shreyas", 1.0 },
  { "Shreyash", 1.0 },
  { "Shreyasi", 0.0 },
  { "Shreyes", 1.0 },
  { "Shrhonda", 0.0 },
  { "Shria", 0.0 },
  { "Shrihaan", 1.0 },
  { "Shrihan", 1.0 },
  { "Shrika", 0.0 },
  { "Shrikar", 1.0 },
  { "Shriley", 0.0 },
  { "Shrina", 0.0 },
  { "Shrinidhi", 0.0 },
  { "Shrinika", 0.0 },
  { "Shriram", 1.0 },
  { "Shrish", 1.0 },
  { "Shrishti", 0.0 },
  { "Shristi", 0.0 },
  { "Shristopher", 1.0 },
  { "Shrita", 0.0 },
  { "Shritan", 1.0 },
  { "Shritha", 0.0 },
  { "Shrivas", 1.0 },
  { "Shriya", 0.0 },
  { "Shriyaan", 1.0 },
  { "Shriyan", 1.0 },
  { "Shriyans", 1.0 },
  { "Shriyansh", 1.0 },
  { "Shron", 0.0 },
  { "Shronda", 0.0 },
  { "Shruthi", 0.0 },
  { "Shruthika", 0.0 },
  { "Shruti", 0.0 },
  { "Shryl", 0.0 },
  { "Shterna", 0.0 },
  { "Shu", 0.74285716 },
  { "Shua", 0.0 },
  { "Shuaib", 1.0 },
  { "Shuan", 1.0 },
  { "Shuana", 0.0 },
  { "Shuayb", 1.0 },
  { "Shuban", 1.0 },
  { "Shubh", 1.0 },
  { "Shubha", 0.0 },
  { "Shubham", 1.0 },
  { "Shubhan", 1.0 },
  { "Shubhi", 0.0 },
  { "Shuchi", 0.0 },
  { "Shue", 1.0 },
  { "Shuford", 1.0 },
  { "Shuhei", 1.0 },
  { "Shuichi", 1.0 },
  { "Shuji", 1.0 },
  { "Shukri", 0.07653061 },
  { "Shukriyyah", 0.0 },
  { "Shukura", 0.0 },
  { "Shulamis", 0.0 },
  { "Shulamit", 0.0 },
  { "Shulamite", 0.0 },
  { "Shulamith", 0.0 },
  { "Shulanda", 0.0 },
  { "Shulem", 1.0 },
  { "Shulonda", 0.0 },
  { "Shumeka", 0.0 },
  { "Shumel", 1.0 },
  { "Shun", 0.96602494 },
  { "Shuna", 0.0 },
  { "Shunashi", 0.0 },
  { "Shunda", 0.0 },
  { "Shundarius", 1.0 },
  { "Shundell", 1.0 },
  { "Shundra", 0.0 },
  { "Shundrea", 0.0 },
  { "Shundreka", 0.0 },
  { "Shundria", 0.0 },
  { "Shundrika", 0.0 },
  { "Shunell", 0.0 },
  { "Shunette", 0.0 },
  { "Shunika", 0.0 },
  { "Shunita", 0.0 },
  { "Shuniya", 0.0 },
  { "Shunn", 1.0 },
  { "Shunna", 0.0 },
  { "Shunsuke", 1.0 },
  { "Shunta", 0.038107753 },
  { "Shuntae", 0.0 },
  { "Shuntana", 0.0 },
  { "Shuntavia", 0.0 },
  { "Shuntay", 0.0 },
  { "Shunte", 0.0 },
  { "Shuntel", 0.0 },
  { "Shuntell", 0.0 },
  { "Shunteria", 0.0 },
  { "Shunterria", 0.0 },
  { "Shuntia", 0.0 },
  { "Shuntina", 0.0 },
  { "Shunya", 0.0 },
  { "Shuraim", 1.0 },
  { "Shuree", 0.0 },
  { "Shureka", 0.0 },
  { "Shurhonda", 0.0 },
  { "Shuri", 0.0 },
  { "Shurie", 0.0 },
  { "Shurla", 0.0 },
  { "Shurlene", 0.0 },
  { "Shurley", 1.0 },
  { "Shurman", 1.0 },
  { "Shuron", 0.54022986 },
  { "Shuronda", 0.0 },
  { "Shushan", 0.0 },
  { "Shuvon", 0.0 },
  { "Shuvonne", 0.0 },
  { "Shuwanda", 0.0 },
  { "Shuwanna", 0.0 },
  { "Shvonne", 0.0 },
  { "Shwana", 0.0 },
  { "Shwanda", 0.0 },
  { "Shwanna", 0.0 },
  { "Shweta", 0.0 },
  { "Shwetha", 0.0 },
  { "Shy", 0.06388206 },
  { "Shya", 0.0 },
  { "Shyah", 0.03937008 },
  { "Shyaira", 0.0 },
  { "Shyam", 1.0 },
  { "Shyan", 0.041273586 },
  { "Shyana", 0.0 },
  { "Shyane", 0.0 },
  { "Shyann", 0.0 },
  { "Shyanna", 0.0 },
  { "Shyannah", 0.0 },
  { "Shyanne", 0.0 },
  { "Shyasia", 0.0 },
  { "Shyauna", 0.0 },
  { "Shyda", 0.0 },
  { "Shydae", 0.0 },
  { "Shydia", 0.0 },
  { "Shye", 0.064935066 },
  { "Shyeeda", 0.0 },
  { "Shyenne", 0.0 },
  { "Shyera", 0.0 },
  { "Shyesha", 0.0 },
  { "Shyheed", 1.0 },
  { "Shyheem", 1.0 },
  { "Shyheim", 1.0 },
  { "Shyhem", 1.0 },
  { "Shyhiem", 1.0 },
  { "Shyia", 0.0 },
  { "Shyiem", 1.0 },
  { "Shykeem", 1.0 },
  { "Shykeim", 1.0 },
  { "Shykela", 0.0 },
  { "Shykemmia", 0.0 },
  { "Shykeria", 0.0 },
  { "Shykerria", 0.0 },
  { "Shykeyla", 0.0 },
  { "Shykia", 0.0 },
  { "Shykila", 0.0 },
  { "Shykim", 1.0 },
  { "Shykira", 0.0 },
  { "Shyla", 0.0 },
  { "Shylah", 0.0 },
  { "Shylan", 0.14705883 },
  { "Shylar", 0.0 },
  { "Shylea", 0.0 },
  { "Shyleah", 0.0 },
  { "Shylee", 0.0 },
  { "Shyleen", 0.0 },
  { "Shylei", 0.0 },
  { "Shyleigh", 0.0 },
  { "Shylene", 0.0 },
  { "Shyler", 0.38490567 },
  { "Shyli", 0.0 },
  { "Shylia", 0.0 },
  { "Shylie", 0.0 },
  { "Shylin", 0.0 },
  { "Shylla", 0.0 },
  { "Shylo", 0.15611193 },
  { "Shyloe", 0.0 },
  { "Shyloh", 0.18333334 },
  { "Shylon", 1.0 },
  { "Shylow", 0.0 },
  { "Shylyn", 0.0 },
  { "Shylynn", 0.0 },
  { "Shyma", 0.0 },
  { "Shymeek", 1.0 },
  { "Shymeka", 0.0 },
  { "Shymel", 1.0 },
  { "Shymere", 1.0 },
  { "Shymia", 0.0 },
  { "Shymir", 1.0 },
  { "Shyna", 0.0 },
  { "Shyne", 0.76527333 },
  { "Shynece", 0.0 },
  { "Shyneice", 0.0 },
  { "Shyneka", 0.0 },
  { "Shynell", 0.0 },
  { "Shynequa", 0.0 },
  { "Shynese", 0.0 },
  { "Shynesha", 0.0 },
  { "Shynia", 0.0 },
  { "Shynice", 0.0 },
  { "Shyniece", 0.0 },
  { "Shynika", 0.0 },
  { "Shyniqua", 0.0 },
  { "Shynise", 0.0 },
  { "Shynisha", 0.0 },
  { "Shynita", 0.0 },
  { "Shyon", 1.0 },
  { "Shyona", 0.0 },
  { "Shyonna", 0.0 },
  { "Shyquan", 1.0 },
  { "Shyra", 0.0 },
  { "Shyrah", 0.0 },
  { "Shyree", 0.0 },
  { "Shyreeta", 0.0 },
  { "Shyrel", 0.0 },
  { "Shyrell", 0.0 },
  { "Shyrene", 0.0 },
  { "Shyrl", 0.0 },
  { "Shyrle", 0.0 },
  { "Shyrlee", 0.0 },
  { "Shyron", 1.0 },
  { "Shyrone", 1.0 },
  { "Shytavia", 0.0 },
  { "Shyteria", 0.0 },
  { "Shyvonne", 0.0 },
  { "Si", 0.89325845 },
  { "Sia", 0.0040983604 },
  { "Siah", 0.491453 },
  { "Siahna", 0.0 },
  { "Siaka", 1.0 },
  { "Siaki", 1.0 },
  { "Siale", 1.0 },
  { "Siam", 1.0 },
  { "Sian", 0.077108435 },
  { "Siana", 0.0 },
  { "Sianah", 0.0 },
  { "Siane", 0.0 },
  { "Siani", 0.0 },
  { "Siann", 0.0 },
  { "Sianna", 0.0 },
  { "Siannah", 0.0 },
  { "Sianne", 0.0 },
  { "Sianni", 0.0 },
  { "Siany", 0.0 },
  { "Siaosi", 1.0 },
  { "Siara", 0.0 },
  { "Siarah", 0.0 },
  { "Siari", 0.0 },
  { "Siarra", 0.0 },
  { "Siarrah", 0.0 },
  { "Sias", 1.0 },
  { "Siavash", 1.0 },
  { "Siba", 0.0 },
  { "Sibbie", 0.0 },
  { "Sibel", 0.0 },
  { "Sibella", 0.0 },
  { "Sibelle", 0.0 },
  { "Sibi", 1.0 },
  { "Sibil", 0.0 },
  { "Sibilla", 0.0 },
  { "Sible", 0.0 },
  { "Sibley", 0.1978022 },
  { "Siboney", 0.0 },
  { "Sibora", 0.0 },
  { "Sibrina", 0.0 },
  { "Sibyl", 0.0 },
  { "Sibyle", 0.0 },
  { "Sibylla", 0.0 },
  { "Sicely", 0.0 },
  { "Sicilee", 0.0 },
  { "Sicilia", 0.0 },
  { "Sicily", 0.0 },
  { "Siclali", 0.0 },
  { "Siclaly", 0.0 },
  { "Sid", 0.99825233 },
  { "Sidahmed", 1.0 },
  { "Sidak", 1.0 },
  { "Sidalee", 0.0 },
  { "Sidarth", 1.0 },
  { "Sidda", 0.0 },
  { "Siddalee", 0.0 },
  { "Siddanth", 1.0 },
  { "Siddarth", 1.0 },
  { "Siddeegah", 0.0 },
  { "Siddeeq", 1.0 },
  { "Siddeeqah", 0.0 },
  { "Siddh", 1.0 },
  { "Siddhan", 1.0 },
  { "Siddhant", 1.0 },
  { "Siddhanth", 1.0 },
  { "Siddharth", 1.0 },
  { "Siddhartha", 1.0 },
  { "Siddhesh", 1.0 },
  { "Siddhi", 0.0 },
  { "Siddhiksha", 0.0 },
  { "Siddie", 0.0 },
  { "Siddiq", 1.0 },
  { "Sidea", 0.0 },
  { "Sidh", 1.0 },
  { "Sidhant", 1.0 },
  { "Sidhanth", 1.0 },
  { "Sidharth", 1.0 },
  { "Sidhartha", 1.0 },
  { "Sidi", 1.0 },
  { "Sidiki", 1.0 },
  { "Sidna", 0.0 },
  { "Sidne", 0.0 },
  { "Sidnee", 0.0 },
  { "Sidnei", 0.0 },
  { "Sidney", 0.7800513 },
  { "Sidni", 0.0 },
  { "Sidnie", 0.0 },
  { "Sidny", 0.0 },
  { "Sidon", 1.0 },
  { "Sidona", 0.0 },
  { "Sidonia", 0.0 },
  { "Sidonie", 0.0 },
  { "Sidora", 0.0 },
  { "Sidra", 0.0 },
  { "Sidrah", 0.0 },
  { "Sidratul", 0.0 },
  { "Sidy", 1.0 },
  { "Sie", 1.0 },
  { "Sieana", 0.0 },
  { "Sieanna", 0.0 },
  { "Sieara", 0.0 },
  { "Siearra", 0.0 },
  { "Siebert", 1.0 },
  { "Sieda", 0.0 },
  { "Siedah", 0.0 },
  { "Siegfried", 1.0 },
  { "Sieglinde", 0.0 },
  { "Siem", 1.0 },
  { "Siena", 0.0 },
  { "Siene", 0.0 },
  { "Sienna", 0.00043950247 },
  { "Siennah", 0.0 },
  { "Sienne", 0.0 },
  { "Sier", 1.0 },
  { "Siera", 0.0 },
  { "Sierah", 0.0 },
  { "Sieria", 0.0 },
  { "Sierra", 0.0028169309 },
  { "Sierrah", 0.0 },
  { "Sierraleone", 0.0 },
  { "Sierralynn", 0.0 },
  { "Sierramarie", 0.0 },
  { "Sierre", 0.0 },
  { "Sierrea", 0.0 },
  { "Sierria", 0.0 },
  { "Siersha", 0.0 },
  { "Siesta", 0.0 },
  { "Sievert", 1.0 },
  { "Sifa", 0.0 },
  { "Sifan", 0.0 },
  { "Sifat", 1.0 },
  { "Sig", 1.0 },
  { "Sigal", 0.0 },
  { "Sigel", 1.0 },
  { "Sigfred", 1.0 },
  { "Sigfredo", 1.0 },
  { "Sigfrid", 1.0 },
  { "Sigfrido", 1.0 },
  { "Sigifredo", 1.0 },
  { "Sigismund", 1.0 },
  { "Sigle", 1.0 },
  { "Sigmond", 1.0 },
  { "Sigmund", 1.0 },
  { "Sigmunt", 1.0 },
  { "Signa", 0.0 },
  { "Signe", 0.0 },
  { "Signora", 0.0 },
  { "Sigourney", 0.0 },
  { "Sigrid", 0.0 },
  { "Sigrun", 0.0 },
  { "Sigurd", 1.0 },
  { "Sigvard", 1.0 },
  { "Sihaam", 0.0 },
  { "Siham", 0.0 },
  { "Sihara", 0.0 },
  { "Siiri", 0.0 },
  { "Sikander", 1.0 },
  { "Sila", 0.0 },
  { "Silah", 0.0 },
  { "Silas", 0.99859357 },
  { "Silberio", 1.0 },
  { "Silbestre", 1.0 },
  { "Silbia", 0.0 },
  { "Silena", 0.0 },
  { "Silence", 0.0 },
  { "Siler", 1.0 },
  { "Silia", 0.0 },
  { "Silina", 0.0 },
  { "Silis", 1.0 },
  { "Silja", 0.0 },
  { "Silje", 0.0 },
  { "Silk", 0.0 },
  { "Silka", 0.0 },
  { "Silke", 0.0 },
  { "Silken", 0.0 },
  { "Silla", 0.0 },
  { "Siller", 0.0 },
  { "Silpa", 0.0 },
  { "Silton", 1.0 },
  { "Silus", 1.0 },
  { "Silva", 0.0 },
  { "Silvan", 1.0 },
  { "Silvana", 0.0 },
  { "Silvanna", 0.0 },
  { "Silvano", 1.0 },
  { "Silvanus", 1.0 },
  { "Silven", 1.0 },
  { "Silver", 0.21716288 },
  { "Silveria", 0.0 },
  { "Silverio", 1.0 },
  { "Silverius", 1.0 },
  { "Silvester", 1.0 },
  { "Silvestra", 0.0 },
  { "Silvestre", 1.0 },
  { "Silvestro", 1.0 },
  { "Silvi", 0.0 },
  { "Silvia", 0.002578753 },
  { "Silviana", 0.0 },
  { "Silviano", 1.0 },
  { "Silvie", 0.0 },
  { "Silvin", 1.0 },
  { "Silvina", 0.0 },
  { "Silvino", 1.0 },
  { "Silvio", 1.0 },
  { "Silvo", 1.0 },
  { "Silvya", 0.0 },
  { "Sim", 1.0 },
  { "Sima", 0.0 },
  { "Simao", 1.0 },
  { "Simar", 0.22869955 },
  { "Simara", 0.0 },
  { "Simarpreet", 0.0 },
  { "Simaya", 0.0 },
  { "Simayah", 0.0 },
  { "Simba", 1.0 },
  { "Simbarashe", 1.0 },
  { "Simbo", 0.0 },
  { "Simcha", 0.8744307 },
  { "Sime", 1.0 },
  { "Simeko", 0.0 },
  { "Simeon", 0.9967746 },
  { "Simi", 0.017857144 },
  { "Simia", 0.0 },
  { "Similoluwa", 0.0 },
  { "Simin", 0.0 },
  { "Simion", 1.0 },
  { "Simione", 1.0 },
  { "Simisola", 0.0 },
  { "Simitrio", 1.0 },
  { "Simiya", 0.0 },
  { "Simiyah", 0.0 },
  { "Simmeon", 1.0 },
  { "Simmie", 0.88023955 },
  { "Simmon", 1.0 },
  { "Simmone", 0.0 },
  { "Simmons", 1.0 },
  { "Simms", 1.0 },
  { "Simon", 0.9973018 },
  { "Simona", 0.0 },
  { "Simone", 0.019881506 },
  { "Simonetta", 0.0 },
  { "Simoni", 0.0 },
  { "Simonne", 0.0 },
  { "Simora", 0.0 },
  { "Simote", 1.0 },
  { "Simplicio", 1.0 },
  { "Simpson", 1.0 },
  { "Simra", 0.0 },
  { "Simrah", 0.0 },
  { "Simran", 0.050037622 },
  { "Simrandeep", 0.0 },
  { "Simranjit", 0.16666667 },
  { "Simranpreet", 0.0 },
  { "Simrat", 0.0 },
  { "Simreet", 0.0 },
  { "Simren", 0.0 },
  { "Simrin", 0.0 },
  { "Simrit", 0.0 },
  { "Simron", 0.0 },
  { "Simrun", 0.0 },
  { "Simryn", 0.0 },
  { "Sims", 1.0 },
  { "Simuel", 1.0 },
  { "Simy", 0.0 },
  { "Simya", 0.0 },
  { "Simyah", 0.0 },
  { "Sina", 0.2130668 },
  { "Sinae", 0.0 },
  { "Sinahi", 0.0 },
  { "Sinai", 0.09050527 },
  { "Sinaiya", 0.0 },
  { "Sinan", 0.9815498 },
  { "Sinath", 0.0 },
  { "Sinatra", 1.0 },
  { "Sinay", 0.0 },
  { "Sinaya", 0.0 },
  { "Sincear", 1.0 },
  { "Sinceer", 0.5769231 },
  { "Sincer", 1.0 },
  { "Sincere", 0.8435325 },
  { "Sincerely", 0.0 },
  { "Sincerity", 0.0 },
  { "Sinchana", 0.0 },
  { "Sinclair", 0.8123501 },
  { "Sinclaire", 0.0 },
  { "Sinda", 0.0 },
  { "Sindee", 0.0 },
  { "Sindel", 0.0 },
  { "Sindey", 1.0 },
  { "Sindhu", 0.0 },
  { "Sindhuja", 0.0 },
  { "Sindhura", 0.0 },
  { "Sindi", 0.0 },
  { "Sindia", 0.0 },
  { "Sindie", 0.0 },
  { "Sindy", 0.0 },
  { "Sinea", 0.0 },
  { "Sinead", 0.0 },
  { "Sinem", 0.0 },
  { "Sinforosa", 0.0 },
  { "Sing", 0.875 },
  { "Singleton", 1.0 },
  { "Sinh", 1.0 },
  { "Sinia", 0.0 },
  { "Siniah", 0.0 },
  { "Sinit", 0.0 },
  { "Siniya", 0.0 },
  { "Siniyah", 0.0 },
  { "Sinjin", 1.0 },
  { "Sinjon", 1.0 },
  { "Sinnamon", 0.0 },
  { "Sinnie", 0.0 },
  { "Sinthia", 0.0 },
  { "Sintia", 0.0 },
  { "Sinue", 1.0 },
  { "Sinuhe", 1.0 },
  { "Sinya", 0.0 },
  { "Siobahn", 0.0 },
  { "Sioban", 0.0 },
  { "Siobhain", 0.0 },
  { "Siobhan", 0.0 },
  { "Siobhon", 0.0 },
  { "Sioeli", 1.0 },
  { "Siohban", 0.0 },
  { "Siomara", 0.0 },
  { "Sion", 0.8799283 },
  { "Siona", 0.0 },
  { "Sione", 0.994941 },
  { "Sionna", 0.0 },
  { "Siosaia", 1.0 },
  { "Siosiua", 1.0 },
  { "Sioux", 0.0 },
  { "Siovhan", 0.0 },
  { "Sip", 1.0 },
  { "Sipriano", 1.0 },
  { "Siqi", 0.0 },
  { "Sir", 1.0 },
  { "Sira", 0.0 },
  { "Siraaj", 1.0 },
  { "Sirach", 1.0 },
  { "Sirah", 0.0 },
  { "Siraj", 1.0 },
  { "Siran", 1.0 },
  { "Siranthony", 1.0 },
  { "Sirat", 0.0 },
  { "Siraya", 0.0 },
  { "Sirayah", 0.0 },
  { "Sircharles", 1.0 },
  { "Sirchristopher", 1.0 },
  { "Sirdarius", 1.0 },
  { "Sire", 1.0 },
  { "Sireen", 0.0 },
  { "Sireena", 0.0 },
  { "Siren", 0.0 },
  { "Sirena", 0.0 },
  { "Sirenia", 0.0 },
  { "Sirenity", 0.0 },
  { "Sirenna", 0.0 },
  { "Sirgio", 1.0 },
  { "Siri", 0.0 },
  { "Siria", 0.0 },
  { "Siriah", 0.0 },
  { "Siriana", 0.0 },
  { "Sirica", 1.0 },
  { "Sirin", 0.0 },
  { "Sirina", 0.0 },
  { "Sirine", 0.0 },
  { "Sirinity", 0.0 },
  { "Siris", 1.0 },
  { "Sirita", 0.0 },
  { "Sirius", 0.98402554 },
  { "Siriya", 0.0 },
  { "Siriyah", 0.0 },
  { "Sirjames", 1.0 },
  { "Sirking", 1.0 },
  { "Sirkka", 0.0 },
  { "Sirley", 0.0 },
  { "Sirmichael", 1.0 },
  { "Sirr", 1.0 },
  { "Sirrah", 0.0 },
  { "Sirron", 1.0 },
  { "Sirus", 1.0 },
  { "Sirvon", 1.0 },
  { "Sirwilliam", 1.0 },
  { "Siryah", 0.0 },
  { "Sisco", 1.0 },
  { "Sisi", 0.0 },
  { "Sisilia", 0.0 },
  { "Sisira", 0.0 },
  { "Sisley", 0.0 },
  { "Sismai", 0.0 },
  { "Sisqo", 1.0 },
  { "Sissi", 0.0 },
  { "Sissie", 0.0 },
  { "Sissy", 0.0 },
  { "Sista", 0.0 },
  { "Sister", 0.0 },
  { "Sistine", 0.0 },
  { "Sisto", 1.0 },
  { "Sita", 0.0 },
  { "Sitara", 0.0 },
  { "Sitey", 0.0 },
  { "Sitha", 0.3846154 },
  { "Siti", 0.0 },
  { "Sitiveni", 1.0 },
  { "Sitka", 1.0 },
  { "Sitlali", 0.0 },
  { "Sitlaly", 0.0 },
  { "Sitora", 0.0 },
  { "Siu", 0.2777778 },
  { "Siva", 1.0 },
  { "Sivad", 1.0 },
  { "Sivan", 0.10229277 },
  { "Sivana", 0.0 },
  { "Sivani", 0.0 },
  { "Sivert", 1.0 },
  { "Sivi", 0.0 },
  { "Siwar", 0.0 },
  { "Siwoo", 1.0 },
  { "Sixta", 0.0 },
  { "Sixten", 1.0 },
  { "Sixto", 1.0 },
  { "Siya", 0.0 },
  { "Siyaa", 0.0 },
  { "Siyah", 0.10483871 },
  { "Siyam", 0.5833333 },
  { "Siyan", 0.0 },
  { "Siyana", 0.0 },
  { "Siyanna", 0.0 },
  { "Siyon", 1.0 },
  { "Siyona", 0.0 },
  { "Siyuan", 1.0 },
  { "Siyuri", 0.0 },
  { "Sj", 1.0 },
  { "Sjana", 0.0 },
  { "Sjon", 1.0 },
  { "Skai", 0.050925925 },
  { "Skaii", 0.0 },
  { "Skailar", 0.0 },
  { "Skanda", 1.0 },
  { "Skarlet", 0.0 },
  { "Skarleth", 0.0 },
  { "Skarlett", 0.0 },
  { "Skarlette", 0.0 },
  { "Skeet", 1.0 },
  { "Skeeter", 0.9057971 },
  { "Ski", 0.046296295 },
  { "Skie", 0.0 },
  { "Skieler", 0.0 },
  { "Skielynn", 0.0 },
  { "Skii", 0.0 },
  { "Skiilar", 0.0 },
  { "Skila", 0.0 },
  { "Skilah", 0.0 },
  { "Skilar", 0.14204545 },
  { "Skilee", 0.0 },
  { "Skiler", 0.5077882 },
  { "Skiley", 0.0 },
  { "Skilyn", 0.0 },
  { "Skilynn", 0.0 },
  { "Skip", 1.0 },
  { "Skipp", 1.0 },
  { "Skipper", 1.0 },
  { "Skippy", 1.0 },
  { "Skky", 0.0 },
  { "Sklyer", 0.84375 },
  { "Sky", 0.33343402 },
  { "Skya", 0.0 },
  { "Skyah", 0.0 },
  { "Skyana", 0.0 },
  { "Skyann", 0.0 },
  { "Skyanna", 0.0 },
  { "Skyanne", 0.0 },
  { "Skyasia", 0.0 },
  { "Skyden", 1.0 },
  { "Skye", 0.11648092 },
  { "Skyee", 0.0 },
  { "Skyela", 0.0 },
  { "Skyelar", 0.21610738 },
  { "Skyelee", 0.0 },
  { "Skyeler", 0.4978355 },
  { "Skyelin", 0.0 },
  { "Skyelyn", 0.0 },
  { "Skyelynn", 0.0 },
  { "Skyi", 0.0 },
  { "Skyia", 0.0 },
  { "Skyla", 0.0 },
  { "Skylaa", 0.0 },
  { "Skylaar", 0.0 },
  { "Skylah", 0.0 },
  { "Skylair", 0.0 },
  { "Skylan", 0.3787234 },
  { "Skyland", 0.6764706 },
  { "Skylann", 0.0 },
  { "Skylar", 0.2242059 },
  { "Skylarmarie", 0.0 },
  { "Skylarr", 0.0 },
  { "Skylarrae", 0.0 },
  { "Skylarrose", 0.0 },
  { "Skyle", 0.0 },
  { "Skylea", 0.0 },
  { "Skylee", 0.0 },
  { "Skyleen", 0.0 },
  { "Skylei", 0.0 },
  { "Skyleigh", 0.0 },
  { "Skylen", 0.38709676 },
  { "Skylene", 0.0 },
  { "Skyler", 0.62893283 },
  { "Skylette", 0.0 },
  { "Skyley", 0.0 },
  { "Skyli", 0.0 },
  { "Skylia", 0.0 },
  { "Skylie", 0.0 },
  { "Skylier", 0.56363636 },
  { "Skylin", 0.06436421 },
  { "Skylinn", 0.0 },
  { "Skylir", 0.0 },
  { "Skylit", 0.0 },
  { "Skylla", 0.0 },
  { "Skyllar", 0.0 },
  { "Skyller", 0.71875 },
  { "Skylon", 1.0 },
  { "Skylor", 0.72418773 },
  { "Skylr", 0.4074074 },
  { "Skylur", 0.5671642 },
  { "Skylyn", 0.0 },
  { "Skylynn", 0.0 },
  { "Skylynne", 0.0 },
  { "Skylyr", 0.0 },
  { "Skyra", 0.0 },
  { "Skyrah", 0.0 },
  { "Skyshun", 1.0 },
  { "Skyy", 0.07279693 },
  { "Skyye", 0.0 },
  { "Skyylar", 0.0862069 },
  { "Skyyler", 1.0 },
  { "Slade", 1.0 },
  { "Sladen", 1.0 },
  { "Slader", 1.0 },
  { "Slate", 1.0 },
  { "Slaten", 1.0 },
  { "Slater", 1.0 },
  { "Slaton", 1.0 },
  { "Slavica", 0.0 },
  { "Slavko", 1.0 },
  { "Slayde", 1.0 },
  { "Slayden", 1.0 },
  { "Slayder", 1.0 },
  { "Slayter", 1.0 },
  { "Slayton", 1.0 },
  { "Slevin", 1.0 },
  { "Slim", 1.0 },
  { "Sloan", 0.33266595 },
  { "Sloane", 0.030234657 },
  { "Sloka", 0.0 },
  { "Slone", 0.825 },
  { "Sly", 1.0 },
  { "Slyvester", 1.0 },
  { "Slyvia", 0.0 },
  { "Smanatha", 0.0 },
  { "Smantha", 0.0 },
  { "Smaran", 1.0 },
  { "Smauel", 1.0 },
  { "Smaya", 0.0 },
  { "Smayan", 1.0 },
  { "Smead", 1.0 },
  { "Smera", 0.0 },
  { "Smiley", 1.0 },
  { "Smit", 1.0 },
  { "Smita", 0.0 },
  { "Smith", 0.9771338 },
  { "Smitha", 0.0 },
  { "Smithie", 0.0 },
  { "Smithy", 1.0 },
  { "Smitty", 1.0 },
  { "Smiya", 0.0 },
  { "Smokey", 1.0 },
  { "Smrithi", 0.0 },
  { "Smriti", 0.0 },
  { "Smya", 0.0 },
  { "Smyan", 1.0 },
  { "Smyrna", 0.0 },
  { "Snaider", 1.0 },
  { "Snapper", 1.0 },
  { "Snayder", 1.0 },
  { "Sneed", 1.0 },
  { "Sneh", 0.8611111 },
  { "Sneha", 0.0 },
  { "Snehal", 0.8909091 },
  { "Sneijder", 1.0 },
  { "Sneyder", 1.0 },
  { "Snezana", 0.0 },
  { "Snezhana", 0.0 },
  { "Snigdha", 0.0 },
  { "Snithik", 1.0 },
  { "Sniya", 0.0 },
  { "Sniyah", 0.0 },
  { "Snoda", 0.0 },
  { "Snow", 0.031746034 },
  { "Snowden", 1.0 },
  { "Snowy", 0.0 },
  { "Snya", 0.0 },
  { "Snyder", 1.0 },
  { "So", 0.0 },
  { "Soana", 0.0 },
  { "Soane", 1.0 },
  { "Sobeida", 0.0 },
  { "Sobia", 0.0 },
  { "Sobra", 0.0 },
  { "Sobrina", 0.0 },
  { "Socheat", 1.0 },
  { "Socheata", 0.0 },
  { "Sochi", 0.0 },
  { "Sochikaima", 0.0 },
  { "Sochil", 0.0 },
  { "Socoro", 0.0 },
  { "Socorra", 0.0 },
  { "Socorro", 0.07433988 },
  { "Socrates", 1.0 },
  { "Soctt", 1.0 },
  { "Soda", 0.0 },
  { "Sodbileg", 1.0 },
  { "Soe", 0.53571427 },
  { "Sofee", 0.0 },
  { "Soffia", 0.0 },
  { "Soffie", 0.0 },
  { "Sofhia", 0.0 },
  { "Sofi", 0.0 },
  { "Sofia", 0.0009648425 },
  { "Sofiagrace", 0.0 },
  { "Sofiah", 0.0 },
  { "Sofiamarie", 0.0 },
  { "Sofian", 1.0 },
  { "Sofiana", 0.0 },
  { "Sofiarose", 0.0 },
  { "Sofie", 0.0 },
  { "Sofija", 0.0 },
  { "Sofina", 0.0 },
  { "Sofiya", 0.0 },
  { "Sofiyah", 0.0 },
  { "Sofya", 0.0 },
  { "Soha", 0.0 },
  { "Sohaan", 1.0 },
  { "Sohaib", 1.0 },
  { "Sohail", 1.0 },
  { "Sohaila", 0.0 },
  { "Soham", 1.0 },
  { "Sohan", 1.0 },
  { "Sohana", 0.0 },
  { "Sohani", 0.0 },
  { "Sohanny", 0.0 },
  { "Sohany", 0.0 },
  { "Soheil", 1.0 },
  { "Soheila", 0.0 },
  { "Sohela", 0.0 },
  { "Sohil", 1.0 },
  { "Sohini", 0.0 },
  { "Sohn", 1.0 },
  { "Sohna", 0.0 },
  { "Sohni", 0.0 },
  { "Sohom", 1.0 },
  { "Sohpia", 0.0 },
  { "Sohrab", 1.0 },
  { "Sohrob", 1.0 },
  { "Sohum", 1.0 },
  { "Soichi", 1.0 },
  { "Soila", 0.0 },
  { "Sojourner", 0.0 },
  { "Sok", 0.7777778 },
  { "Sokha", 0.3292683 },
  { "Sokhna", 0.0 },
  { "Sokhom", 0.52 },
  { "Sokol", 1.0 },
  { "Sokoya", 0.0 },
  { "Sol", 0.7877015 },
  { "Sola", 0.03968254 },
  { "Solace", 0.3448276 },
  { "Solach", 0.0 },
  { "Solae", 0.0 },
  { "Solage", 0.0 },
  { "Solai", 0.0 },
  { "Solaine", 0.0 },
  { "Solan", 1.0 },
  { "Solana", 0.0 },
  { "Solanch", 0.0 },
  { "Solange", 0.0 },
  { "Solangel", 0.0 },
  { "Solangie", 0.0 },
  { "Solara", 0.0 },
  { "Solarah", 0.0 },
  { "Solaris", 0.071428575 },
  { "Solash", 0.0 },
  { "Solay", 0.0 },
  { "Solaya", 0.0 },
  { "Sole", 0.0 },
  { "Solea", 0.0 },
  { "Soledad", 0.0012642225 },
  { "Solee", 0.0 },
  { "Solei", 0.0 },
  { "Soleia", 0.0 },
  { "Soleigh", 0.0 },
  { "Soleil", 0.0 },
  { "Solen", 1.0 },
  { "Solena", 0.0 },
  { "Solene", 0.0 },
  { "Solenn", 0.0 },
  { "Solenne", 0.0 },
  { "Soley", 0.0 },
  { "Soleya", 0.0 },
  { "Soli", 0.0 },
  { "Solia", 0.0 },
  { "Soliana", 0.0 },
  { "Solida", 0.0 },
  { "Solidad", 0.0 },
  { "Solie", 0.0 },
  { "Soliel", 0.0 },
  { "Soliha", 0.0 },
  { "Soliman", 1.0 },
  { "Solimar", 0.0 },
  { "Solina", 0.0 },
  { "Soline", 0.0 },
  { "Solita", 0.0 },
  { "Solitaire", 0.0 },
  { "Soliyana", 0.0 },
  { "Sollie", 1.0 },
  { "Solly", 1.0 },
  { "Solmaira", 0.0 },
  { "Solmari", 0.0 },
  { "Solmarie", 0.0 },
  { "Solmary", 0.0 },
  { "Solmayra", 0.0 },
  { "Solmon", 1.0 },
  { "Solo", 1.0 },
  { "Soloman", 1.0 },
  { "Solomia", 0.0 },
  { "Solomiya", 0.0 },
  { "Solomon", 0.99964637 },
  { "Solomona", 1.0 },
  { "Solomone", 1.0 },
  { "Solon", 1.0 },
  { "Solstice", 0.0 },
  { "Solveig", 0.0 },
  { "Solveigh", 0.0 },
  { "Solyana", 0.0 },
  { "Solymar", 0.0 },
  { "Som", 1.0 },
  { "Soma", 0.5229358 },
  { "Somaiya", 0.0 },
  { "Somalia", 0.0 },
  { "Somaly", 0.0 },
  { "Somara", 0.0 },
  { "Somarah", 0.0 },
  { "Somaria", 0.0 },
  { "Somaya", 0.0 },
  { "Somayah", 0.0 },
  { "Someeka", 0.0 },
  { "Somer", 0.0 },
  { "Somers", 1.0 },
  { "Somi", 0.0 },
  { "Somia", 0.0 },
  { "Somiah", 0.0 },
  { "Somil", 1.0 },
  { "Somiya", 0.0 },
  { "Somiyah", 0.0 },
  { "Sommar", 0.0 },
  { "Sommer", 0.0 },
  { "Sommers", 0.0 },
  { "Somnang", 1.0 },
  { "Somona", 0.0 },
  { "Somone", 0.0 },
  { "Somtochukwu", 0.76363635 },
  { "Somya", 0.0 },
  { "Son", 0.995229 },
  { "Sona", 0.0 },
  { "Sonai", 0.0 },
  { "Sonakshi", 0.0 },
  { "Sonal", 0.0 },
  { "Sonali", 0.0 },
  { "Sonam", 0.14573991 },
  { "Sonata", 0.0 },
  { "Sonaya", 0.0 },
  { "Sonceria", 0.0 },
  { "Sonda", 0.0 },
  { "Sondi", 0.0 },
  { "Sondos", 0.0 },
  { "Sondra", 0.0 },
  { "Sondrea", 0.0 },
  { "Sondria", 0.0 },
  { "Sone", 0.0 },
  { "Sonequa", 0.0 },
  { "Song", 0.26102942 },
  { "Songa", 0.0 },
  { "Soni", 0.0 },
  { "Sonia", 0.0043445555 },
  { "Soniah", 0.0 },
  { "Sonic", 1.0 },
  { "Sonica", 0.0 },
  { "Sonie", 0.0 },
  { "Sonika", 0.0 },
  { "Soniqua", 0.0 },
  { "Sonique", 0.0 },
  { "Sonita", 0.0 },
  { "Soniya", 0.0 },
  { "Soniyah", 0.0 },
  { "Sonja", 0.0011288709 },
  { "Sonjay", 1.0 },
  { "Sonje", 0.0 },
  { "Sonji", 0.0 },
  { "Sonjia", 0.0 },
  { "Sonjie", 0.0 },
  { "Sonna", 0.0 },
  { "Sonnet", 0.0 },
  { "Sonnett", 0.0 },
  { "Sonnette", 0.0 },
  { "Sonni", 0.04347826 },
  { "Sonnia", 0.0 },
  { "Sonnie", 0.56245434 },
  { "Sonny", 0.964671 },
  { "Sonnya", 0.0 },
  { "Sonoma", 0.0 },
  { "Sonora", 0.0 },
  { "Sonovia", 0.0 },
  { "Sonrisa", 0.0 },
  { "Sonseeahray", 0.0 },
  { "Sonta", 0.0 },
  { "Sonte", 0.0 },
  { "Sontee", 0.0 },
  { "Sonum", 0.0 },
  { "Sony", 0.87539935 },
  { "Sonya", 0.0028016379 },
  { "Sonye", 0.0 },
  { "Sonyia", 0.0 },
  { "Sonza", 0.0 },
  { "Soo", 0.08530806 },
  { "Soobin", 0.0 },
  { "Soojin", 0.0 },
  { "Sook", 0.0 },
  { "Sookie", 0.0 },
  { "Soomin", 0.0 },
  { "Soon", 0.104166664 },
  { "Sooner", 0.0 },
  { "Soorya", 1.0 },
  { "Sopath", 1.0 },
  { "Sopaul", 1.0 },
  { "Sopha", 0.0 },
  { "Sophal", 1.0 },
  { "Sophan", 1.0 },
  { "Sophana", 0.7368421 },
  { "Sophanna", 0.4117647 },
  { "Sophany", 0.0 },
  { "Sophat", 1.0 },
  { "Sophea", 0.01633987 },
  { "Sopheak", 1.0 },
  { "Sopheap", 0.36507937 },
  { "Sophear", 0.5555556 },
  { "Sophee", 0.0 },
  { "Sopheia", 0.0 },
  { "Sophelia", 0.0 },
  { "Sophey", 0.0 },
  { "Sopheya", 0.0 },
  { "Sophi", 0.0 },
  { "Sophia", 0.0012567191 },
  { "Sophiaann", 0.0 },
  { "Sophiaanne", 0.0 },
  { "Sophiagrace", 0.0 },
  { "Sophiah", 0.0 },
  { "Sophiaisabella", 0.0 },
  { "Sophiaisabelle", 0.0 },
  { "Sophialauren", 0.0 },
  { "Sophialynn", 0.0 },
  { "Sophiamae", 0.0 },
  { "Sophiamaria", 0.0 },
  { "Sophiamarie", 0.0 },
  { "Sophiana", 0.0 },
  { "Sophianna", 0.0 },
  { "Sophiarose", 0.0 },
  { "Sophie", 0.0010252716 },
  { "Sophiea", 0.0 },
  { "Sophieann", 0.0 },
  { "Sophieanne", 0.0 },
  { "Sophiee", 0.0 },
  { "Sophiemarie", 0.0 },
  { "Sophilia", 0.0 },
  { "Sophina", 0.0 },
  { "Sophira", 0.0 },
  { "Sophiya", 0.0 },
  { "Sophiyah", 0.0 },
  { "Sophonie", 0.0 },
  { "Sophorn", 0.0 },
  { "Sophrona", 0.0 },
  { "Sophronia", 0.0 },
  { "Sophy", 0.0 },
  { "Sophya", 0.0 },
  { "Sophyia", 0.0 },
  { "Sopia", 0.0 },
  { "Sora", 0.16749175 },
  { "Sorah", 0.0 },
  { "Soraia", 0.0 },
  { "Soraida", 0.0 },
  { "Soraide", 0.0 },
  { "Soraiya", 0.0 },
  { "Soran", 1.0 },
  { "Sorangel", 0.0 },
  { "Soraya", 0.0 },
  { "Sorayah", 0.0 },
  { "Sorayda", 0.0 },
  { "Sorcha", 0.0 },
  { "Sorella", 0.0 },
  { "Sorelle", 0.0 },
  { "Soren", 0.9434431 },
  { "Sorena", 0.0 },
  { "Soreya", 0.0 },
  { "Soria", 0.0 },
  { "Soriah", 0.0 },
  { "Sorin", 0.97282606 },
  { "Sorina", 0.0 },
  { "Soriya", 0.019379845 },
  { "Soriyah", 0.0 },
  { "Soroosh", 1.0 },
  { "Soroush", 1.0 },
  { "Soroya", 0.0 },
  { "Sorraya", 0.0 },
  { "Sorrel", 0.0 },
  { "Sorrell", 1.0 },
  { "Sorren", 1.0 },
  { "Sorsha", 0.0 },
  { "Sory", 1.0 },
  { "Soryn", 0.5 },
  { "Sosa", 1.0 },
  { "Sosaia", 1.0 },
  { "Sosefina", 0.0 },
  { "Sosha", 0.0 },
  { "Sosie", 0.0 },
  { "Sossity", 0.0 },
  { "Sostenes", 1.0 },
  { "Sosuke", 1.0 },
  { "Sota", 1.0 },
  { "Sotaro", 1.0 },
  { "Sotera", 0.0 },
  { "Soteria", 0.0 },
  { "Soterios", 1.0 },
  { "Sotero", 1.0 },
  { "Sothea", 0.6923077 },
  { "Sotheary", 0.0 },
  { "Sotiria", 0.0 },
  { "Sotirios", 1.0 },
  { "Sotiris", 1.0 },
  { "Sou", 1.0 },
  { "Soua", 0.3478261 },
  { "Souad", 0.0 },
  { "Soufiane", 1.0 },
  { "Soukaina", 0.0 },
  { "Soul", 0.72602737 },
  { "Soula", 0.0 },
  { "Souleymane", 1.0 },
  { "Soumaya", 0.0 },
  { "Soumil", 1.0 },
  { "Soumya", 0.0 },
  { "Soundra", 0.0 },
  { "Souren", 1.0 },
  { "Sourik", 1.0 },
  { "Sourish", 1.0 },
  { "Souriya", 1.0 },
  { "Sourya", 1.0 },
  { "Souta", 1.0 },
  { "South", 1.0 },
  { "Southern", 0.25581396 },
  { "Sovann", 1.0 },
  { "Sovanna", 0.26086956 },
  { "Sovanny", 0.0 },
  { "Soveida", 0.0 },
  { "Sovereign", 0.6101695 },
  { "Sovilla", 0.0 },
  { "Sowmya", 0.0 },
  { "Sowsan", 0.0 },
  { "Soyer", 1.0 },
  { "Soyini", 0.0 },
  { "Soyla", 0.0 },
  { "Soyna", 0.0 },
  { "Soyoung", 0.0 },
  { "Spandan", 1.0 },
  { "Spandana", 0.0 },
  { "Spanky", 1.0 },
  { "Sparkel", 0.0 },
  { "Sparkle", 0.0 },
  { "Sparkles", 0.0 },
  { "Sparks", 1.0 },
  { "Sparky", 1.0 },
  { "Sparrow", 0.11741683 },
  { "Sparsh", 1.0 },
  { "Spartaco", 1.0 },
  { "Spartacus", 1.0 },
  { "Spartan", 1.0 },
  { "Spaulding", 1.0 },
  { "Special", 0.0 },
  { "Speed", 1.0 },
  { "Speedy", 1.0 },
  { "Spence", 1.0 },
  { "Spencel", 1.0 },
  { "Spencer", 0.9701588 },
  { "Spenser", 0.89153683 },
  { "Spero", 1.0 },
  { "Speros", 1.0 },
  { "Sperry", 1.0 },
  { "Spessard", 1.0 },
  { "Spicie", 0.0 },
  { "Spicy", 0.0 },
  { "Spike", 1.0 },
  { "Spiridon", 1.0 },
  { "Spiridoula", 0.0 },
  { "Spirit", 0.008361204 },
  { "Spiritual", 0.0 },
  { "Spiro", 1.0 },
  { "Spiros", 1.0 },
  { "Spivey", 1.0 },
  { "Splendora", 0.0 },
  { "Spontaneous", 0.0 },
  { "Spoorthi", 0.0 },
  { "Sprague", 1.0 },
  { "Spriha", 0.0 },
  { "Spring", 0.0 },
  { "Spruha", 0.0 },
  { "Spurgeon", 1.0 },
  { "Spyridon", 1.0 },
  { "Spyros", 1.0 },
  { "Square", 1.0 },
  { "Squire", 1.0 },
  { "Sra", 0.0 },
  { "Sraavya", 0.0 },
  { "Sragvi", 0.0 },
  { "Srah", 0.0 },
  { "Sravan", 1.0 },
  { "Sravani", 0.0 },
  { "Sravya", 0.0 },
  { "Sree", 0.0 },
  { "Sreehan", 1.0 },
  { "Sreeja", 0.0 },
  { "Sreekar", 1.0 },
  { "Sreenidhi", 0.0 },
  { "Sreenika", 0.0 },
  { "Sreeram", 1.0 },
  { "Sreeya", 0.0 },
  { "Sreshta", 0.0 },
  { "Sreya", 0.0 },
  { "Sreyas", 1.0 },
  { "Sri", 0.056179777 },
  { "Sriaadya", 0.0 },
  { "Sriansh", 1.0 },
  { "Sricharan", 1.0 },
  { "Sridevi", 0.0 },
  { "Sridha", 0.0 },
  { "Sridhar", 1.0 },
  { "Srihaan", 1.0 },
  { "Srihaas", 1.0 },
  { "Srihan", 1.0 },
  { "Srihari", 1.0 },
  { "Sriharsha", 1.0 },
  { "Srihita", 0.0 },
  { "Srihith", 1.0 },
  { "Srihitha", 0.0 },
  { "Srija", 0.0 },
  { "Srijan", 1.0 },
  { "Srika", 0.0 },
  { "Srikanth", 1.0 },
  { "Srikar", 1.0 },
  { "Srikari", 0.0 },
  { "Srikrishna", 1.0 },
  { "Srikrithi", 0.0 },
  { "Srikruthi", 0.0 },
  { "Srilaasya", 0.0 },
  { "Srilakshmi", 0.0 },
  { "Sriman", 1.0 },
  { "Srimayi", 0.0 },
  { "Srimedha", 0.0 },
  { "Srinath", 1.0 },
  { "Srinav", 1.0 },
  { "Srinidhi", 0.0 },
  { "Srinija", 0.0 },
  { "Srinika", 0.0 },
  { "Sriniketh", 1.0 },
  { "Srinithi", 0.0 },
  { "Srinithya", 0.0 },
  { "Srinitya", 0.0 },
  { "Srinivas", 1.0 },
  { "Sriram", 1.0 },
  { "Srisai", 1.0 },
  { "Srisha", 0.0 },
  { "Srishti", 0.0 },
  { "Sristi", 0.0 },
  { "Sritan", 1.0 },
  { "Sritha", 0.0 },
  { "Srithan", 1.0 },
  { "Srithik", 1.0 },
  { "Srivaishnavi", 0.0 },
  { "Srivanth", 1.0 },
  { "Srivar", 1.0 },
  { "Srivatsa", 1.0 },
  { "Srivatsan", 1.0 },
  { "Srividya", 0.0 },
  { "Sriya", 0.0 },
  { "Sriyaan", 1.0 },
  { "Sriyah", 0.0 },
  { "Sriyan", 1.0 },
  { "Sriyansh", 1.0 },
  { "Srujan", 1.0 },
  { "Sruly", 1.0 },
  { "Srushti", 0.0 },
  { "Sruthi", 0.0 },
  { "Sruti", 0.0 },
  { "Ssirley", 0.0 },
  { "Sstephanie", 0.0 },
  { "St", 1.0 },
  { "Staccato", 1.0 },
  { "Stacci", 0.0 },
  { "Stace", 0.84792626 },
  { "Stacee", 0.0 },
  { "Stacey", 0.08609736 },
  { "Staceyann", 0.0 },
  { "Stacha", 0.0 },
  { "Stachia", 0.0 },
  { "Staci", 0.0023531257 },
  { "Stacia", 0.0 },
  { "Staciann", 0.0 },
  { "Stacie", 0.004953023 },
  { "Stacy", 0.1197739 },
  { "Stacyann", 0.0 },
  { "Stacye", 0.0 },
  { "Stafan", 1.0 },
  { "Stafanie", 0.0 },
  { "Staffon", 1.0 },
  { "Stafford", 1.0 },
  { "Stafon", 1.0 },
  { "Staisha", 0.0 },
  { "Stajah", 0.0 },
  { "Staley", 0.70987654 },
  { "Stalin", 1.0 },
  { "Stallone", 1.0 },
  { "Stalyn", 1.0 },
  { "Stamatia", 0.0 },
  { "Stamatina", 0.0 },
  { "Stamatios", 1.0 },
  { "Stamatis", 1.0 },
  { "Stan", 1.0 },
  { "Stana", 0.0 },
  { "Stance", 1.0 },
  { "Stancel", 1.0 },
  { "Stancil", 1.0 },
  { "Stancy", 0.0 },
  { "Standard", 1.0 },
  { "Standford", 1.0 },
  { "Standish", 1.0 },
  { "Standley", 1.0 },
  { "Standly", 1.0 },
  { "Stanely", 1.0 },
  { "Stanesha", 0.0 },
  { "Stanette", 0.0 },
  { "Staney", 1.0 },
  { "Stanford", 1.0 },
  { "Stanisha", 0.0 },
  { "Stanislaus", 1.0 },
  { "Stanislav", 1.0 },
  { "Stanislava", 0.0 },
  { "Stanislaw", 1.0 },
  { "Stanislawa", 0.0 },
  { "Stanja", 0.0 },
  { "Stanlee", 0.77011496 },
  { "Stanleigh", 1.0 },
  { "Stanley", 0.99520516 },
  { "Stanly", 1.0 },
  { "Stann", 1.0 },
  { "Stanna", 0.0 },
  { "Stanson", 1.0 },
  { "Stanton", 1.0 },
  { "Stanwood", 1.0 },
  { "Stanya", 0.0 },
  { "Staphanie", 0.0 },
  { "Staphany", 0.0 },
  { "Staphon", 1.0 },
  { "Star", 0.016240356 },
  { "Stara", 0.0 },
  { "Starasia", 0.0 },
  { "Starbuck", 1.0 },
  { "Starcia", 0.0 },
  { "Stardust", 0.0 },
  { "Staria", 0.0 },
  { "Starisha", 0.0 },
  { "Stark", 1.0 },
  { "Starkeisha", 0.0 },
  { "Starkesha", 0.0 },
  { "Starkey", 1.0 },
  { "Starkisha", 0.0 },
  { "Starla", 0.0 },
  { "Starlah", 0.0 },
  { "Starleana", 0.0 },
  { "Starlee", 0.0 },
  { "Starleen", 0.0 },
  { "Starleigh", 0.0 },
  { "Starlena", 0.0 },
  { "Starlene", 0.0 },
  { "Starlet", 0.0 },
  { "Starlett", 0.0 },
  { "Starletta", 0.0 },
  { "Starlette", 0.0 },
  { "Starley", 0.3125 },
  { "Starlight", 0.0 },
  { "Starlin", 0.81363636 },
  { "Starlina", 0.0 },
  { "Starling", 0.87476635 },
  { "Starlisa", 0.0 },
  { "Starlisha", 0.0 },
  { "Starlit", 0.0 },
  { "Starlita", 0.0 },
  { "Starlite", 0.0 },
  { "Starlyn", 0.00896861 },
  { "Starlynn", 0.0 },
  { "Starnisha", 0.0 },
  { "Starquasia", 0.0 },
  { "Starr", 0.034427796 },
  { "Starri", 0.0 },
  { "Starrla", 0.0 },
  { "Starrlynn", 0.0 },
  { "Starsha", 0.0 },
  { "Starshema", 0.0 },
  { "Starshemah", 0.0 },
  { "Starsky", 1.0 },
  { "Startasia", 0.0 },
  { "Stas", 1.0 },
  { "Stash", 1.0 },
  { "Stasha", 0.0 },
  { "Stashia", 0.0 },
  { "Stasi", 0.0 },
  { "Stasia", 0.0 },
  { "Stassa", 0.0 },
  { "Stassi", 0.0 },
  { "Staten", 1.0 },
  { "Statham", 1.0 },
  { "Stathan", 1.0 },
  { "Statia", 0.0 },
  { "Statler", 1.0 },
  { "Staton", 1.0 },
  { "Statton", 1.0 },
  { "Stav", 0.475 },
  { "Stavon", 1.0 },
  { "Stavro", 1.0 },
  { "Stavros", 1.0 },
  { "Stavroula", 0.0 },
  { "Stavya", 1.0 },
  { "Stayce", 0.0 },
  { "Staysha", 0.0 },
  { "Stayton", 1.0 },
  { "Stazia", 0.0 },
  { "Stazie", 0.0 },
  { "Stclair", 1.0 },
  { "Stcy", 0.0 },
  { "Steadman", 1.0 },
  { "Stearl", 1.0 },
  { "Stearns", 1.0 },
  { "Steave", 1.0 },
  { "Steaven", 1.0 },
  { "Steban", 1.0 },
  { "Stedman", 1.0 },
  { "Stedmen", 1.0 },
  { "Stedmon", 1.0 },
  { "Stedmond", 1.0 },
  { "Steed", 1.0 },
  { "Steel", 1.0 },
  { "Steele", 0.97963446 },
  { "Steeler", 1.0 },
  { "Steelie", 0.0 },
  { "Steely", 0.125 },
  { "Steen", 1.0 },
  { "Steeve", 1.0 },
  { "Steeven", 1.0 },
  { "Steevie", 0.0 },
  { "Stefaine", 0.0 },
  { "Stefan", 0.9977856 },
  { "Stefana", 0.0 },
  { "Stefane", 0.0 },
  { "Stefanee", 0.0 },
  { "Stefaney", 0.0 },
  { "Stefani", 0.0 },
  { "Stefania", 0.0 },
  { "Stefanie", 0.0011642074 },
  { "Stefanni", 0.0 },
  { "Stefannie", 0.0 },
  { "Stefanny", 0.0 },
  { "Stefano", 1.0 },
  { "Stefanos", 1.0 },
  { "Stefany", 0.0 },
  { "Stefaun", 1.0 },
  { "Stefen", 1.0 },
  { "Stefeni", 0.0 },
  { "Stefenie", 0.0 },
  { "Steffan", 1.0 },
  { "Steffane", 0.0 },
  { "Steffani", 0.0 },
  { "Steffanie", 0.0 },
  { "Steffano", 1.0 },
  { "Steffany", 0.0 },
  { "Steffen", 1.0 },
  { "Steffenie", 0.0 },
  { "Steffi", 0.0 },
  { "Steffie", 0.0 },
  { "Steffin", 1.0 },
  { "Steffon", 1.0 },
  { "Steffy", 0.0 },
  { "Stefhanie", 0.0 },
  { "Stefhany", 0.0 },
  { "Stefhon", 1.0 },
  { "Stefie", 0.0 },
  { "Stefin", 1.0 },
  { "Stefnie", 0.0 },
  { "Stefon", 1.0 },
  { "Stefone", 1.0 },
  { "Stefoni", 0.0 },
  { "Stefphon", 1.0 },
  { "Stehanie", 0.0 },
  { "Stehen", 1.0 },
  { "Stehpanie", 0.0 },
  { "Stehpen", 1.0 },
  { "Steicy", 0.0 },
  { "Stein", 1.0 },
  { "Steisy", 0.0 },
  { "Stela", 0.0 },
  { "Stelios", 1.0 },
  { "Stell", 0.0 },
  { "Stella", 0.0022870523 },
  { "Stellah", 0.0 },
  { "Stellaluna", 0.0 },
  { "Stellamae", 0.0 },
  { "Stellamarie", 0.0 },
  { "Stellan", 1.0 },
  { "Stellar", 0.10625 },
  { "Stellarose", 0.0 },
  { "Stelle", 0.0 },
  { "Stellina", 0.0 },
  { "Stelmo", 1.0 },
  { "Sten", 1.0 },
  { "Stena", 0.0 },
  { "Stennis", 1.0 },
  { "Steohanie", 0.0 },
  { "Stepahanie", 0.0 },
  { "Stepahnie", 0.0 },
  { "Stepan", 1.0 },
  { "Stepanie", 0.0 },
  { "Stepehen", 1.0 },
  { "Stepehn", 1.0 },
  { "Stepen", 1.0 },
  { "Stepfan", 1.0 },
  { "Stepfanie", 0.0 },
  { "Stepfon", 1.0 },
  { "Steph", 0.0 },
  { "Stephaie", 0.0 },
  { "Stephaine", 0.0 },
  { "Stephan", 0.99143416 },
  { "Stephana", 0.0 },
  { "Stephane", 0.37402886 },
  { "Stephanee", 0.0 },
  { "Stephaney", 0.0 },
  { "Stephani", 0.0 },
  { "Stephania", 0.0 },
  { "Stephanic", 0.0 },
  { "Stephanie", 0.0035610439 },
  { "Stephanieann", 0.0 },
  { "Stephanieanne", 0.0 },
  { "Stephaniemarie", 0.0 },
  { "Stephanine", 0.0 },
  { "Stephanne", 0.0 },
  { "Stephanni", 0.0 },
  { "Stephannie", 0.0 },
  { "Stephanny", 0.0 },
  { "Stephano", 1.0 },
  { "Stephanos", 1.0 },
  { "Stephany", 0.00034274746 },
  { "Stephanye", 0.0 },
  { "Stephaun", 1.0 },
  { "Stephe", 1.0 },
  { "Stephen", 0.99658406 },
  { "Stephene", 0.0 },
  { "Stephenee", 0.0 },
  { "Stepheni", 0.0 },
  { "Stephenia", 0.0 },
  { "Stephenie", 0.0 },
  { "Stephenmichael", 1.0 },
  { "Stephennie", 0.0 },
  { "Stepheno", 1.0 },
  { "Stephens", 1.0 },
  { "Stephenson", 1.0 },
  { "Stepheny", 0.0 },
  { "Stepheon", 1.0 },
  { "Stephfan", 1.0 },
  { "Stephfon", 1.0 },
  { "Stephiane", 0.0 },
  { "Stephie", 0.0 },
  { "Stephin", 1.0 },
  { "Stephina", 0.0 },
  { "Stephine", 0.0 },
  { "Stephinie", 0.0 },
  { "Stephn", 1.0 },
  { "Stephne", 0.0 },
  { "Stephney", 0.0 },
  { "Stephnie", 0.0 },
  { "Stephon", 0.9982864 },
  { "Stephone", 1.0 },
  { "Stephonie", 0.0 },
  { "Stephun", 1.0 },
  { "Stephvon", 1.0 },
  { "Stephy", 0.0 },
  { "Stepphanie", 0.0 },
  { "Sterl", 1.0 },
  { "Sterlin", 1.0 },
  { "Sterline", 0.0 },
  { "Sterling", 0.9302741 },
  { "Sterlyn", 1.0 },
  { "Stesha", 0.0 },
  { "Stetsen", 1.0 },
  { "Stetson", 1.0 },
  { "Steuart", 1.0 },
  { "Steva", 0.0 },
  { "Stevan", 1.0 },
  { "Stevana", 0.0 },
  { "Stevani", 0.0 },
  { "Stevanie", 0.0 },
  { "Stevanna", 0.0 },
  { "Steve", 0.99623966 },
  { "Stevee", 0.03021978 },
  { "Steveland", 1.0 },
  { "Steven", 0.99651927 },
  { "Stevena", 0.0 },
  { "Stevenmichael", 1.0 },
  { "Stevenn", 1.0 },
  { "Stevenray", 1.0 },
  { "Stevens", 1.0 },
  { "Stevenson", 1.0 },
  { "Stevephen", 1.0 },
  { "Steveson", 1.0 },
  { "Stevette", 0.0 },
  { "Stevey", 0.15646258 },
  { "Stevi", 0.0 },
  { "Stevie", 0.5746134 },
  { "Stevierae", 0.0 },
  { "Stevieray", 1.0 },
  { "Stevii", 0.0 },
  { "Stevin", 1.0 },
  { "Stevion", 1.0 },
  { "Stevn", 1.0 },
  { "Stevon", 1.0 },
  { "Stevona", 0.0 },
  { "Stevonna", 0.0 },
  { "Stevonte", 1.0 },
  { "Stevphen", 1.0 },
  { "Stevy", 0.13157895 },
  { "Stevyn", 1.0 },
  { "Steward", 1.0 },
  { "Stewart", 0.99726087 },
  { "Steysi", 0.0 },
  { "Sthefani", 0.0 },
  { "Sthefany", 0.0 },
  { "Sthepanie", 0.0 },
  { "Sthephanie", 0.0 },
  { "Stian", 1.0 },
  { "Stig", 1.0 },
  { "Stihl", 1.0 },
  { "Stiles", 1.0 },
  { "Stillman", 1.0 },
  { "Stina", 0.0 },
  { "Stiney", 1.0 },
  { "Stinson", 1.0 },
  { "Stirling", 0.9441417 },
  { "Stiven", 1.0 },
  { "Stjulian", 1.0 },
  { "Stockton", 1.0 },
  { "Stoddard", 1.0 },
  { "Stoic", 1.0 },
  { "Stokely", 1.0 },
  { "Stokes", 1.0 },
  { "Stokley", 1.0 },
  { "Stone", 0.99333197 },
  { "Stonewall", 1.0 },
  { "Stoney", 0.98713046 },
  { "Stonie", 0.71428573 },
  { "Stony", 1.0 },
  { "Storee", 0.0 },
  { "Storey", 0.0 },
  { "Stori", 0.0 },
  { "Storie", 0.0 },
  { "Storii", 0.0 },
  { "Storm", 0.62793297 },
  { "Storme", 0.40963855 },
  { "Stormee", 0.0 },
  { "Stormey", 0.03153153 },
  { "Stormi", 0.0 },
  { "Stormie", 0.0 },
  { "Stormy", 0.10771295 },
  { "Story", 0.056764428 },
  { "Stosh", 1.0 },
  { "Stover", 1.0 },
  { "Stowe", 1.0 },
  { "Stoy", 1.0 },
  { "Stpehanie", 0.0 },
  { "Stpehen", 1.0 },
  { "Stphanie", 0.0 },
  { "Stphen", 1.0 },
  { "Stracy", 0.0 },
  { "Stran", 1.0 },
  { "Strange", 0.0 },
  { "Stranje", 0.0 },
  { "Stratford", 1.0 },
  { "Straton", 1.0 },
  { "Stratos", 1.0 },
  { "Stratten", 1.0 },
  { "Stratton", 0.9894027 },
  { "Strauss", 1.0 },
  { "Strawberry", 0.0 },
  { "Streeter", 1.0 },
  { "Strella", 0.0 },
  { "Strickland", 1.0 },
  { "Strider", 1.0 },
  { "Striker", 1.0 },
  { "Strother", 1.0 },
  { "Strummer", 1.0 },
  { "Stryder", 1.0 },
  { "Stryker", 1.0 },
  { "Stu", 1.0 },
  { "Stuard", 1.0 },
  { "Stuart", 0.9954291 },
  { "Stuthi", 0.0 },
  { "Stuti", 0.0 },
  { "Stven", 1.0 },
  { "Styler", 1.0 },
  { "Styles", 1.0 },
  { "Stylez", 1.0 },
  { "Stylianos", 1.0 },
  { "Stylz", 1.0 },
  { "Su", 0.047965117 },
  { "Sua", 0.0 },
  { "Suad", 0.022123894 },
  { "Suada", 0.0 },
  { "Suah", 0.0 },
  { "Suan", 0.23741007 },
  { "Suann", 0.0 },
  { "Suanna", 0.0 },
  { "Suanne", 0.0 },
  { "Suany", 0.0 },
  { "Suave", 1.0 },
  { "Suban", 0.0 },
  { "Subaru", 1.0 },
  { "Subhaan", 1.0 },
  { "Subhan", 1.0 },
  { "Subhana", 0.0 },
  { "Subhi", 1.0 },
  { "Subin", 0.625 },
  { "Subira", 0.0 },
  { "Subrena", 0.0 },
  { "Subrenia", 0.0 },
  { "Subria", 0.0 },
  { "Subrina", 0.0 },
  { "Success", 0.5857143 },
  { "Sucdi", 0.0 },
  { "Sucely", 0.0 },
  { "Suchi", 0.0 },
  { "Suchir", 1.0 },
  { "Suchit", 1.0 },
  { "Suchita", 0.0 },
  { "Suda", 0.0 },
  { "Sudais", 1.0 },
  { "Sudan", 1.0 },
  { "Sudarshan", 1.0 },
  { "Sudays", 1.0 },
  { "Suddie", 0.0 },
  { "Sude", 0.0 },
  { "Sudeeksha", 0.0 },
  { "Sudeep", 1.0 },
  { "Sudeys", 1.0 },
  { "Sudha", 0.0 },
  { "Sudhanva", 1.0 },
  { "Sudhir", 1.0 },
  { "Sudie", 0.0 },
  { "Sudiksha", 0.0 },
  { "Sudip", 1.0 },
  { "Sue", 0.003579977 },
  { "Suean", 0.0 },
  { "Sueann", 0.0 },
  { "Sueanna", 0.0 },
  { "Sueanne", 0.0 },
  { "Suede", 1.0 },
  { "Suedell", 0.0 },
  { "Sueellen", 0.0 },
  { "Suehay", 0.0 },
  { "Suejin", 0.0 },
  { "Sueko", 0.0 },
  { "Sueling", 0.0 },
  { "Suella", 0.0 },
  { "Suellen", 0.0 },
  { "Suellyn", 0.0 },
  { "Suelyn", 0.0 },
  { "Suelynn", 0.0 },
  { "Suen", 1.0 },
  { "Sueno", 0.0 },
  { "Sueo", 1.0 },
  { "Suesan", 0.0 },
  { "Suetta", 0.0 },
  { "Suezanne", 0.0 },
  { "Suezette", 0.0 },
  { "Sufi", 0.0 },
  { "Sufia", 0.0 },
  { "Sufian", 1.0 },
  { "Sufiyan", 1.0 },
  { "Sufyaan", 1.0 },
  { "Sufyan", 1.0 },
  { "Sugar", 0.26136363 },
  { "Sugei", 0.0 },
  { "Sugeily", 0.0 },
  { "Sugeiri", 0.0 },
  { "Sugey", 0.0 },
  { "Suha", 0.0 },
  { "Suhaan", 1.0 },
  { "Suhaani", 0.0 },
  { "Suhaas", 1.0 },
  { "Suhad", 0.0 },
  { "Suhaib", 1.0 },
  { "Suhail", 0.91694355 },
  { "Suhaila", 0.0 },
  { "Suhailah", 0.0 },
  { "Suhailey", 0.0 },
  { "Suhaill", 0.0 },
  { "Suhaily", 0.0 },
  { "Suhaira", 0.0 },
  { "Suhan", 1.0 },
  { "Suhana", 0.0 },
  { "Suhanee", 0.0 },
  { "Suhani", 0.0 },
  { "Suhanth", 1.0 },
  { "Suhas", 1.0 },
  { "Suhavi", 0.0 },
  { "Suhayb", 1.0 },
  { "Suhayla", 0.0 },
  { "Suhaylah", 0.0 },
  { "Suheb", 1.0 },
  { "Suheib", 1.0 },
  { "Suheidy", 0.0 },
  { "Suheily", 0.0 },
  { "Suhey", 0.0 },
  { "Suheyb", 1.0 },
  { "Suheyla", 0.0 },
  { "Suhrit", 1.0 },
  { "Suhur", 0.0 },
  { "Sui", 0.0 },
  { "Suja", 0.0 },
  { "Sujal", 1.0 },
  { "Sujan", 1.0 },
  { "Sujata", 0.0 },
  { "Sujatha", 0.0 },
  { "Sujay", 1.0 },
  { "Sujei", 0.0 },
  { "Sujeily", 0.0 },
  { "Sujeiry", 0.0 },
  { "Sujey", 0.0 },
  { "Sujin", 0.0 },
  { "Sujit", 1.0 },
  { "Sujood", 0.0 },
  { "Sukaina", 0.0 },
  { "Sukanya", 0.0 },
  { "Sukari", 0.0 },
  { "Sukayna", 0.0 },
  { "Sukhdeep", 0.78723407 },
  { "Sukhjit", 0.52380955 },
  { "Sukhjot", 1.0 },
  { "Sukhleen", 0.0 },
  { "Sukhman", 0.5467626 },
  { "Sukhmani", 0.0 },
  { "Sukhpreet", 0.15384616 },
  { "Sukhraj", 1.0 },
  { "Suki", 0.0 },
  { "Sukie", 0.0 },
  { "Sukina", 0.0 },
  { "Sula", 0.0 },
  { "Sulaf", 0.0 },
  { "Sulai", 0.0 },
  { "Sulaiman", 1.0 },
  { "Sulamita", 0.0 },
  { "Sular", 0.0 },
  { "Sulay", 0.0 },
  { "Sulaymaan", 1.0 },
  { "Sulayman", 1.0 },
  { "Sule", 0.0 },
  { "Suleica", 0.0 },
  { "Suleika", 0.0 },
  { "Suleima", 0.0 },
  { "Suleiman", 1.0 },
  { "Sulekha", 0.0 },
  { "Sulema", 0.0 },
  { "Suleman", 1.0 },
  { "Suley", 0.0 },
  { "Suleyka", 0.0 },
  { "Suleyma", 0.0 },
  { "Suleyman", 1.0 },
  { "Suleymi", 0.0 },
  { "Suli", 0.0 },
  { "Sulia", 0.0 },
  { "Suliana", 0.0 },
  { "Sulie", 0.0 },
  { "Sulieman", 1.0 },
  { "Suliman", 1.0 },
  { "Sulin", 0.0 },
  { "Sulinda", 0.0 },
  { "Sullie", 0.0 },
  { "Sullivan", 0.9175851 },
  { "Sully", 0.60080105 },
  { "Sulma", 0.0 },
  { "Sulmy", 0.0 },
  { "Sulo", 1.0 },
  { "Sultan", 1.0 },
  { "Sultana", 0.0 },
  { "Sulton", 1.0 },
  { "Suly", 0.0 },
  { "Sulyn", 0.0 },
  { "Sulynn", 0.0 },
  { "Suma", 0.0 },
  { "Sumaia", 0.0 },
  { "Sumair", 1.0 },
  { "Sumaira", 0.0 },
  { "Sumaiya", 0.0 },
  { "Sumaiyah", 0.0 },
  { "Sumaiyya", 0.0 },
  { "Sumako", 0.0 },
  { "Sumalee", 0.0 },
  { "Suman", 0.1369863 },
  { "Sumana", 0.0 },
  { "Sumanth", 1.0 },
  { "Sumar", 0.0 },
  { "Sumara", 0.0 },
  { "Sumaya", 0.0 },
  { "Sumayah", 0.0 },
  { "Sumayo", 0.0 },
  { "Sumayya", 0.0 },
  { "Sumayyah", 0.0 },
  { "Sumedh", 1.0 },
  { "Sumedha", 0.0 },
  { "Sumeet", 0.9585062 },
  { "Sumehra", 0.0 },
  { "Sumeja", 0.0 },
  { "Sumeka", 0.0 },
  { "Sumeko", 0.0 },
  { "Sumer", 0.0 },
  { "Sumera", 0.0 },
  { "Sumerlyn", 0.0 },
  { "Sumeya", 0.0 },
  { "Sumeyye", 0.0 },
  { "Sumi", 0.0 },
  { "Sumia", 0.0 },
  { "Sumiah", 0.0 },
  { "Sumie", 0.0 },
  { "Sumika", 0.0 },
  { "Sumiko", 0.0 },
  { "Sumin", 0.0 },
  { "Sumio", 1.0 },
  { "Sumir", 1.0 },
  { "Sumire", 0.0 },
  { "Sumit", 1.0 },
  { "Sumita", 0.0 },
  { "Sumitra", 0.0 },
  { "Sumiya", 0.0 },
  { "Sumiyah", 0.0 },
  { "Sumiye", 0.0 },
  { "Summa", 0.0 },
  { "Summar", 0.0 },
  { "Summaya", 0.0 },
  { "Summayah", 0.0 },
  { "Summer", 0.0031129855 },
  { "Summerlee", 0.0 },
  { "Summerlin", 0.0 },
  { "Summerlyn", 0.0 },
  { "Summerlynn", 0.0 },
  { "Summerrain", 0.0 },
  { "Summerrose", 0.0 },
  { "Summers", 0.14473684 },
  { "Summie", 1.0 },
  { "Summit", 0.89337176 },
  { "Summitt", 0.0 },
  { "Sumner", 0.9244833 },
  { "Sumpter", 1.0 },
  { "Sumra", 0.0 },
  { "Sumter", 1.0 },
  { "Sumukh", 1.0 },
  { "Sumya", 0.0 },
  { "Sumyah", 0.0 },
  { "Sun", 0.45088756 },
  { "Suna", 0.0 },
  { "Sunaina", 0.0 },
  { "Sunao", 1.0 },
  { "Sunasia", 0.0 },
  { "Sunay", 1.0 },
  { "Sunbeam", 0.0 },
  { "Sunda", 0.0 },
  { "Sundae", 0.0 },
  { "Sundai", 0.0 },
  { "Sundance", 0.9285714 },
  { "Sundara", 0.0 },
  { "Sundari", 0.0 },
  { "Sundas", 0.0 },
  { "Sunday", 0.025238095 },
  { "Sunde", 0.0 },
  { "Sundee", 0.0 },
  { "Sundeep", 0.9802371 },
  { "Sundi", 0.0 },
  { "Sundiata", 1.0 },
  { "Sundie", 0.0 },
  { "Sundos", 0.0 },
  { "Sundown", 0.33333334 },
  { "Sundra", 0.0 },
  { "Sundus", 0.0 },
  { "Sundy", 0.0 },
  { "Sunee", 0.0 },
  { "Suneel", 1.0 },
  { "Sunehri", 0.0 },
  { "Sunel", 1.0 },
  { "Sunem", 0.0 },
  { "Sunflower", 0.0 },
  { "Sung", 0.86549705 },
  { "Suni", 0.0 },
  { "Sunia", 0.0 },
  { "Sunidhi", 0.0 },
  { "Sunie", 0.0 },
  { "Sunil", 1.0 },
  { "Sunita", 0.0 },
  { "Sunitha", 0.0 },
  { "Suniya", 0.0 },
  { "Suniyah", 0.0 },
  { "Sunja", 0.0 },
  { "Sunjai", 0.0 },
  { "Sunjay", 1.0 },
  { "Sunna", 0.0 },
  { "Sunnee", 0.0 },
  { "Sunni", 0.005804749 },
  { "Sunnie", 0.0128283445 },
  { "Sunniva", 0.0 },
  { "Sunny", 0.27722517 },
  { "Sunnye", 0.0 },
  { "Sunrise", 0.0 },
  { "Sunset", 0.0 },
  { "Sunshine", 0.0071802833 },
  { "Sunshyne", 0.0 },
  { "Sunya", 0.0 },
  { "Suoma", 0.0 },
  { "Suong", 0.0 },
  { "Suparna", 0.0 },
  { "Supraja", 0.0 },
  { "Supreet", 0.0 },
  { "Supreme", 1.0 },
  { "Suprena", 0.0 },
  { "Suprenia", 0.0 },
  { "Suprina", 0.0 },
  { "Supriya", 0.0 },
  { "Sura", 0.0 },
  { "Surabhi", 0.0 },
  { "Surafel", 1.0 },
  { "Surah", 0.0 },
  { "Surai", 0.0 },
  { "Suraiya", 0.0 },
  { "Suraiyah", 0.0 },
  { "Suraj", 1.0 },
  { "Suraya", 0.0 },
  { "Surayah", 0.0 },
  { "Surayya", 0.0 },
  { "Surayyah", 0.0 },
  { "Surbhi", 0.0 },
  { "Suree", 0.0 },
  { "Sureena", 0.0 },
  { "Surely", 0.0 },
  { "Suren", 1.0 },
  { "Surena", 0.0 },
  { "Surenity", 0.0 },
  { "Suresh", 1.0 },
  { "Sureya", 0.0 },
  { "Suri", 0.0 },
  { "Suria", 0.0 },
  { "Suriah", 0.0 },
  { "Suriana", 0.0 },
  { "Surianna", 0.0 },
  { "Surie", 0.0 },
  { "Suriel", 1.0 },
  { "Surina", 0.0 },
  { "Suriya", 0.053475935 },
  { "Suriyah", 0.0 },
  { "Surraya", 0.0 },
  { "Surrey", 0.0 },
  { "Surveen", 0.0 },
  { "Sury", 0.0 },
  { "Surya", 0.7653157 },
  { "Suryansh", 1.0 },
  { "Susa", 0.0 },
  { "Susaa", 0.0 },
  { "Susan", 0.002259526 },
  { "Susana", 0.0025713297 },
  { "Susanah", 0.0 },
  { "Susane", 0.0 },
  { "Susann", 0.0 },
  { "Susanna", 0.0 },
  { "Susannah", 0.0 },
  { "Susanne", 0.0 },
  { "Susano", 1.0 },
  { "Susaye", 0.0 },
  { "Susej", 0.0 },
  { "Susel", 0.0 },
  { "Susen", 0.0 },
  { "Suset", 0.0 },
  { "Susette", 0.0 },
  { "Sushant", 1.0 },
  { "Sushanth", 1.0 },
  { "Sushil", 1.0 },
  { "Sushma", 0.0 },
  { "Sushmita", 0.0 },
  { "Susi", 0.0 },
  { "Susia", 0.0 },
  { "Susian", 0.0 },
  { "Susie", 0.003265402 },
  { "Susin", 0.0 },
  { "Sussan", 0.0 },
  { "Sussie", 0.0 },
  { "Sussy", 0.0 },
  { "Susumu", 1.0 },
  { "Susy", 0.0 },
  { "Susyn", 0.0 },
  { "Suszanne", 0.0 },
  { "Sutten", 0.0 },
  { "Sutter", 0.9316547 },
  { "Sutton", 0.48078108 },
  { "Suttyn", 0.0 },
  { "Suvali", 0.0 },
  { "Suvan", 1.0 },
  { "Suvanna", 0.0 },
  { "Suveer", 1.0 },
  { "Suvi", 0.0 },
  { "Suvilla", 0.0 },
  { "Suvir", 1.0 },
  { "Suyai", 0.0 },
  { "Suyapa", 0.0 },
  { "Suyash", 1.0 },
  { "Suyeko", 0.0 },
  { "Suyog", 1.0 },
  { "Suz", 0.0 },
  { "Suzan", 0.0 },
  { "Suzana", 0.0 },
  { "Suzane", 0.0 },
  { "Suzann", 0.0 },
  { "Suzanna", 0.0 },
  { "Suzannah", 0.0 },
  { "Suzanne", 0.0022440136 },
  { "Suze", 0.0 },
  { "Suzelle", 0.0 },
  { "Suzen", 0.0 },
  { "Suzet", 0.0 },
  { "Suzeth", 0.0 },
  { "Suzett", 0.0 },
  { "Suzetta", 0.0 },
  { "Suzette", 0.0003005169 },
  { "Suzi", 0.0 },
  { "Suzie", 0.0 },
  { "Suzon", 0.0 },
  { "Suzonne", 0.0 },
  { "Suzu", 0.0 },
  { "Suzuko", 0.0 },
  { "Suzy", 0.0 },
  { "Suzzane", 0.0 },
  { "Suzzanna", 0.0 },
  { "Suzzanne", 0.0 },
  { "Suzzette", 0.0 },
  { "Suzzie", 0.0 },
  { "Svana", 0.0 },
  { "Svanik", 1.0 },
  { "Svannah", 0.0 },
  { "Svara", 0.0 },
  { "Svea", 0.0 },
  { "Sven", 1.0 },
  { "Svend", 1.0 },
  { "Sverre", 1.0 },
  { "Svetlana", 0.0 },
  { "Sveva", 0.0 },
  { "Swade", 1.0 },
  { "Swadha", 0.0 },
  { "Swain", 1.0 },
  { "Swan", 0.6294416 },
  { "Swanda", 0.0 },
  { "Swanee", 0.0 },
  { "Swanie", 0.0 },
  { "Swannie", 0.0 },
  { "Swanson", 1.0 },
  { "Swanzetta", 0.0 },
  { "Swapna", 0.0 },
  { "Swapnil", 1.0 },
  { "Swara", 0.0 },
  { "Swaraj", 1.0 },
  { "Swarit", 1.0 },
  { "Swarna", 0.0 },
  { "Swasti", 0.0 },
  { "Swastik", 1.0 },
  { "Swathi", 0.0 },
  { "Swati", 0.0 },
  { "Sway", 0.18779343 },
  { "Swayam", 1.0 },
  { "Swayde", 1.0 },
  { "Swayne", 1.0 },
  { "Swayze", 0.21446384 },
  { "Swayzee", 0.0 },
  { "Swayzi", 0.0 },
  { "Swayzie", 0.0 },
  { "Swecha", 0.0 },
  { "Sweden", 0.0 },
  { "Sweet", 0.0 },
  { "Sweetie", 0.0 },
  { "Swen", 1.0 },
  { "Swendy", 0.0 },
  { "Sweta", 0.0 },
  { "Swetha", 0.0 },
  { "Swift", 1.0 },
  { "Swindell", 1.0 },
  { "Swinton", 1.0 },
  { "Swiyyah", 0.0 },
  { "Sy", 1.0 },
  { "Sya", 0.0 },
  { "Syah", 0.0 },
  { "Syair", 1.0 },
  { "Syaira", 0.0 },
  { "Syaire", 0.95683455 },
  { "Syan", 0.20769231 },
  { "Syana", 0.0 },
  { "Syani", 0.0 },
  { "Syann", 0.0 },
  { "Syanna", 0.0 },
  { "Syanne", 0.0 },
  { "Syara", 0.0 },
  { "Syasia", 0.0 },
  { "Sybal", 0.0 },
  { "Sybastian", 1.0 },
  { "Sybel", 0.0 },
  { "Sybella", 0.0 },
  { "Sybil", 0.00026032174 },
  { "Sybile", 0.0 },
  { "Sybill", 0.0 },
  { "Sybilla", 0.0 },
  { "Sybille", 0.0 },
  { "Syble", 0.0 },
  { "Sybol", 0.0 },
  { "Sybrena", 0.0 },
  { "Sybrina", 0.0 },
  { "Syd", 0.60784316 },
  { "Sydalee", 0.0 },
  { "Sydea", 0.0 },
  { "Sydel", 0.0 },
  { "Sydell", 0.0 },
  { "Sydelle", 0.0 },
  { "Syden", 1.0 },
  { "Sydeny", 0.0 },
  { "Sydia", 0.0 },
  { "Sydna", 0.0 },
  { "Sydne", 0.0 },
  { "Sydnea", 0.0 },
  { "Sydnee", 0.0 },
  { "Sydnei", 0.0 },
  { "Sydney", 0.05189489 },
  { "Sydni", 0.0 },
  { "Sydnie", 0.0 },
  { "Sydnii", 0.0 },
  { "Sydnor", 1.0 },
  { "Sydny", 0.0 },
  { "Sydnye", 0.0 },
  { "Sydonia", 0.0 },
  { "Sydonna", 0.0 },
  { "Sydra", 0.0 },
  { "Sye", 1.0 },
  { "Syed", 0.997715 },
  { "Syeda", 0.0 },
  { "Syedah", 0.0 },
  { "Syedmuhammad", 1.0 },
  { "Syeed", 1.0 },
  { "Syeeda", 0.0 },
  { "Syeira", 0.0 },
  { "Syeisha", 0.0 },
  { "Syenna", 0.0 },
  { "Syer", 1.0 },
  { "Syera", 0.0 },
  { "Syere", 1.0 },
  { "Syeria", 0.0 },
  { "Syerra", 0.0 },
  { "Syesha", 0.0 },
  { "Syheem", 1.0 },
  { "Syhir", 1.0 },
  { "Syia", 0.0 },
  { "Syiah", 0.0 },
  { "Syid", 1.0 },
  { "Syier", 1.0 },
  { "Syiere", 1.0 },
  { "Syion", 1.0 },
  { "Syir", 1.0 },
  { "Syire", 1.0 },
  { "Sykeria", 0.0 },
  { "Sykora", 0.0 },
  { "Syl", 1.0 },
  { "Syla", 0.0 },
  { "Sylah", 0.0 },
  { "Sylar", 0.94690263 },
  { "Sylas", 0.99747604 },
  { "Sylbia", 0.0 },
  { "Sylee", 0.0 },
  { "Syleena", 0.0 },
  { "Sylena", 0.0 },
  { "Syler", 0.9710145 },
  { "Syleste", 0.0 },
  { "Sylester", 1.0 },
  { "Sylia", 0.0 },
  { "Sylias", 1.0 },
  { "Sylina", 0.0 },
  { "Sylinda", 0.0 },
  { "Sylis", 1.0 },
  { "Syliss", 1.0 },
  { "Syliva", 0.0 },
  { "Sylivia", 0.0 },
  { "Sylus", 1.0 },
  { "Sylva", 0.019977802 },
  { "Sylvain", 1.0 },
  { "Sylvan", 0.98236775 },
  { "Sylvana", 0.0 },
  { "Sylvanas", 0.0 },
  { "Sylvania", 0.00907441 },
  { "Sylvanna", 0.0 },
  { "Sylvannah", 0.0 },
  { "Sylvanus", 1.0 },
  { "Sylvene", 0.0 },
  { "Sylver", 0.04032258 },
  { "Sylvesta", 0.36619717 },
  { "Sylvester", 0.985593 },
  { "Sylvestre", 1.0 },
  { "Sylvette", 0.0 },
  { "Sylvi", 0.0 },
  { "Sylvia", 0.0037931798 },
  { "Sylvian", 1.0 },
  { "Sylviana", 0.0 },
  { "Sylvie", 0.0 },
  { "Sylvin", 1.0 },
  { "Sylvina", 0.0 },
  { "Sylvio", 1.0 },
  { "Sylvonia", 0.0 },
  { "Sylwia", 0.0 },
  { "Sylys", 1.0 },
  { "Syma", 0.0 },
  { "Symantha", 0.0 },
  { "Symanthia", 0.0 },
  { "Symara", 0.0 },
  { "Symarah", 0.0 },
  { "Symari", 0.0 },
  { "Symaria", 0.0 },
  { "Symaya", 0.0 },
  { "Symba", 0.0 },
  { "Symeir", 1.0 },
  { "Symeon", 1.0 },
  { "Symere", 1.0 },
  { "Symia", 0.0 },
  { "Symiah", 0.0 },
  { "Symir", 1.0 },
  { "Symira", 0.0 },
  { "Symirah", 0.0 },
  { "Symmone", 0.0 },
  { "Symon", 0.9744898 },
  { "Symona", 0.0 },
  { "Symone", 0.0 },
  { "Symonne", 0.0 },
  { "Symora", 0.0 },
  { "Symphani", 0.0 },
  { "Symphanie", 0.0 },
  { "Symphany", 0.0 },
  { "Symphoni", 0.0 },
  { "Symphonie", 0.0 },
  { "Symphony", 0.0 },
  { "Symya", 0.0 },
  { "Syna", 0.0 },
  { "Synae", 0.0 },
  { "Synai", 0.0 },
  { "Synaia", 0.0 },
  { "Synaya", 0.0 },
  { "Synceer", 1.0 },
  { "Synceir", 1.0 },
  { "Syncere", 0.7801746 },
  { "Synclair", 0.0 },
  { "Synclaire", 0.0 },
  { "Synda", 0.0 },
  { "Syndal", 0.0 },
  { "Syndee", 0.0 },
  { "Syndel", 0.0 },
  { "Syndey", 0.0 },
  { "Syndi", 0.0 },
  { "Syndia", 0.0 },
  { "Syndie", 0.0 },
  { "Syndney", 0.0 },
  { "Syndy", 0.0 },
  { "Synethia", 0.0 },
  { "Synetra", 0.0 },
  { "Synetta", 0.0 },
  { "Synia", 0.0 },
  { "Syniah", 0.0 },
  { "Synithia", 0.0 },
  { "Syniya", 0.0 },
  { "Syniyah", 0.0 },
  { "Synneva", 0.0 },
  { "Synnove", 0.0 },
  { "Synovia", 0.0 },
  { "Synquis", 0.0 },
  { "Synthea", 0.0 },
  { "Synthia", 0.0 },
  { "Syntia", 0.0 },
  { "Syntyche", 0.0 },
  { "Syon", 1.0 },
  { "Syona", 0.0 },
  { "Syonna", 0.0 },
  { "Syr", 1.0 },
  { "Syra", 0.0 },
  { "Syrae", 0.0 },
  { "Syrah", 0.0 },
  { "Syrai", 0.0 },
  { "Syre", 1.0 },
  { "Syree", 0.54545456 },
  { "Syreen", 0.0 },
  { "Syreena", 0.0 },
  { "Syreeta", 0.0 },
  { "Syren", 0.11764706 },
  { "Syrena", 0.0 },
  { "Syrene", 0.0 },
  { "Syreniti", 0.0 },
  { "Syrenity", 0.0 },
  { "Syrenna", 0.0 },
  { "Syreta", 0.0 },
  { "Syretha", 0.0 },
  { "Syretta", 0.0 },
  { "Syrette", 0.0 },
  { "Syri", 0.0 },
  { "Syria", 0.0 },
  { "Syriah", 0.0 },
  { "Syriana", 0.0 },
  { "Syrianna", 0.0 },
  { "Syrie", 0.0 },
  { "Syril", 0.0 },
  { "Syrina", 0.0 },
  { "Syrinity", 0.0 },
  { "Syris", 1.0 },
  { "Syrita", 0.0 },
  { "Syritta", 0.0 },
  { "Syrius", 1.0 },
  { "Syron", 1.0 },
  { "Syrus", 1.0 },
  { "Sytha", 0.0 },
  { "Syuri", 0.0 },
  { "Syvanna", 0.0 },
  { "Syvannah", 0.0 },
  { "Syvella", 0.0 },
  { "Syvester", 1.0 },
  { "Syvia", 0.0 },
  { "Syvilla", 0.0 },
  { "Szander", 1.0 },
  { "Szofia", 0.0 },
  { "Szymon", 1.0 },
  { "Ta", 0.12658228 },
  { "Taaffe", 0.0 },
  { "Taaha", 1.0 },
  { "Taahir", 1.0 },
  { "Taahira", 0.0 },
  { "Taahirah", 0.0 },
  { "Taaj", 0.93670887 },
  { "Taalia", 0.0 },
  { "Taaliah", 0.0 },
  { "Taalib", 1.0 },
  { "Taaliba", 0.0 },
  { "Taaliyah", 0.0 },
  { "Taalor", 0.0 },
  { "Taana", 0.0 },
  { "Taara", 0.0 },
  { "Taaraz", 1.0 },
  { "Taariq", 1.0 },
  { "Taarna", 0.0 },
  { "Taaron", 1.0 },
  { "Taasia", 0.0 },
  { "Taavi", 1.0 },
  { "Taavon", 1.0 },
  { "Tab", 1.0 },
  { "Taba", 0.0 },
  { "Tabaitha", 0.0 },
  { "Tabari", 1.0 },
  { "Tabassum", 0.0 },
  { "Tabata", 0.0 },
  { "Tabatha", 0.0 },
  { "Tabathe", 0.0 },
  { "Tabathia", 0.0 },
  { "Tabb", 1.0 },
  { "Tabbatha", 0.0 },
  { "Tabbetha", 0.0 },
  { "Tabbie", 0.0 },
  { "Tabbitha", 0.0 },
  { "Tabby", 0.029940119 },
  { "Tabbytha", 0.0 },
  { "Tabea", 0.0 },
  { "Taber", 0.93636364 },
  { "Tabetha", 0.0 },
  { "Tabethia", 0.0 },
  { "Tabia", 0.0 },
  { "Tabias", 1.0 },
  { "Tabiatha", 0.0 },
  { "Tabious", 1.0 },
  { "Tabish", 1.0 },
  { "Tabita", 0.0 },
  { "Tabitha", 0.002530461 },
  { "Tabithia", 0.0 },
  { "Tabius", 1.0 },
  { "Tabor", 0.87517935 },
  { "Taborah", 0.0 },
  { "Tabori", 1.0 },
  { "Taboris", 1.0 },
  { "Tabrea", 0.0 },
  { "Tabresha", 0.0 },
  { "Tabria", 0.0 },
  { "Tabrina", 0.0 },
  { "Tabrisha", 0.0 },
  { "Tabriya", 0.0 },
  { "Tabu", 1.0 },
  { "Tabytha", 0.0 },
  { "Tacara", 0.0 },
  { "Tacari", 0.8559322 },
  { "Tacarra", 0.0 },
  { "Taccara", 0.0 },
  { "Taccarra", 0.0 },
  { "Tace", 1.0 },
  { "Tacey", 0.0 },
  { "Tacha", 0.0 },
  { "Tachaka", 0.0 },
  { "Tachara", 0.0 },
  { "Tache", 0.0 },
  { "Tachelle", 0.0 },
  { "Tachiana", 0.0 },
  { "Tachic", 0.0 },
  { "Tachina", 0.0 },
  { "Taci", 0.0 },
  { "Tacia", 0.0 },
  { "Taciana", 0.0 },
  { "Tacie", 0.0 },
  { "Tacita", 0.0 },
  { "Tacoma", 0.80733943 },
  { "Tacora", 0.0 },
  { "Tacorey", 1.0 },
  { "Tacori", 0.36 },
  { "Tacoria", 0.0 },
  { "Tacorian", 1.0 },
  { "Tacorra", 0.0 },
  { "Tacory", 1.0 },
  { "Tacoya", 0.0 },
  { "Tacuma", 1.0 },
  { "Tacy", 0.0 },
  { "Tad", 1.0 },
  { "Tadajah", 0.0 },
  { "Tadan", 1.0 },
  { "Tadao", 1.0 },
  { "Tadarian", 1.0 },
  { "Tadarious", 1.0 },
  { "Tadarius", 1.0 },
  { "Tadarrius", 1.0 },
  { "Tadashi", 1.0 },
  { "Tadasia", 0.0 },
  { "Tadayoshi", 1.0 },
  { "Tadd", 1.0 },
  { "Taddeo", 1.0 },
  { "Tade", 1.0 },
  { "Tadeas", 1.0 },
  { "Tadeh", 1.0 },
  { "Tadeja", 0.0 },
  { "Taden", 1.0 },
  { "Tadeo", 1.0 },
  { "Tadesha", 0.0 },
  { "Tadeus", 1.0 },
  { "Tadeusz", 1.0 },
  { "Tadhg", 1.0 },
  { "Tadrian", 1.0 },
  { "Tadyn", 1.0 },
  { "Tadzia", 0.0 },
  { "Tae", 0.7892562 },
  { "Taea", 0.0 },
  { "Taeden", 1.0 },
  { "Taedon", 1.0 },
  { "Taedyn", 0.90909094 },
  { "Taegan", 0.2610798 },
  { "Taegen", 0.32882884 },
  { "Taeghan", 0.0 },
  { "Taegyn", 0.0 },
  { "Taeh", 0.0 },
  { "Taehyun", 1.0 },
  { "Taeisha", 0.0 },
  { "Taeja", 0.0 },
  { "Taejah", 0.0 },
  { "Taejohn", 1.0 },
  { "Taejon", 1.0 },
  { "Taejuan", 1.0 },
  { "Taejzon", 1.0 },
  { "Taeko", 0.0 },
  { "Taekwon", 1.0 },
  { "Taela", 0.0 },
  { "Taelan", 1.0 },
  { "Taelar", 0.0 },
  { "Taelee", 0.0 },
  { "Taelen", 1.0 },
  { "Taeler", 0.014450867 },
  { "Taelin", 0.6621622 },
  { "Taelon", 1.0 },
  { "Taelor", 0.07775847 },
  { "Taelour", 0.0 },
  { "Taelyn", 0.062299293 },
  { "Taelynn", 0.0 },
  { "Taelynne", 0.0 },
  { "Taelyr", 0.0 },
  { "Taemon", 1.0 },
  { "Taeo", 1.0 },
  { "Taeon", 1.0 },
  { "Taequan", 1.0 },
  { "Taeron", 1.0 },
  { "Taeryn", 0.0 },
  { "Taesean", 1.0 },
  { "Taesha", 0.0 },
  { "Taeshaun", 1.0 },
  { "Taeshawn", 1.0 },
  { "Taeshia", 0.0 },
  { "Taeshon", 1.0 },
  { "Taeson", 1.0 },
  { "Taetum", 0.13043478 },
  { "Taetym", 0.0 },
  { "Taeven", 1.0 },
  { "Taeveon", 1.0 },
  { "Taevian", 1.0 },
  { "Taevin", 1.0 },
  { "Taevion", 1.0 },
  { "Taevon", 1.0 },
  { "Taevyn", 1.0 },
  { "Taeya", 0.0 },
  { "Taeyah", 0.0 },
  { "Taeyang", 1.0 },
  { "Taeylor", 0.0 },
  { "Taeyon", 1.0 },
  { "Tafari", 0.99139166 },
  { "Taff", 1.0 },
  { "Taffani", 0.0 },
  { "Taffany", 0.0 },
  { "Taffeta", 0.0 },
  { "Taffi", 0.0 },
  { "Taffie", 0.0 },
  { "Taffney", 0.0 },
  { "Taffy", 0.0 },
  { "Taft", 1.0 },
  { "Tag", 1.0 },
  { "Tagan", 0.46153846 },
  { "Tage", 1.0 },
  { "Tagen", 0.61923075 },
  { "Tagert", 1.0 },
  { "Tagg", 1.0 },
  { "Taggart", 1.0 },
  { "Taggert", 1.0 },
  { "Taha", 1.0 },
  { "Tahaj", 1.0 },
  { "Tahan", 1.0 },
  { "Tahani", 0.0 },
  { "Tahany", 0.0 },
  { "Tahara", 0.0 },
  { "Tahari", 0.3768116 },
  { "Taheed", 1.0 },
  { "Taheem", 1.0 },
  { "Taheera", 0.0 },
  { "Taheerah", 0.0 },
  { "Taheim", 1.0 },
  { "Tahel", 0.0 },
  { "Taher", 1.0 },
  { "Tahera", 0.0 },
  { "Tahesha", 0.0 },
  { "Tahesia", 0.0 },
  { "Tahi", 1.0 },
  { "Tahia", 0.0 },
  { "Tahiem", 1.0 },
  { "Tahina", 0.0 },
  { "Tahir", 1.0 },
  { "Tahira", 0.0 },
  { "Tahirah", 0.0 },
  { "Tahiri", 0.0 },
  { "Tahirih", 0.0 },
  { "Tahiry", 0.0 },
  { "Tahis", 0.0 },
  { "Tahisa", 0.0 },
  { "Tahisha", 0.0 },
  { "Tahitha", 0.0 },
  { "Tahitia", 0.0 },
  { "Tahitoa", 1.0 },
  { "Tahiya", 0.0 },
  { "Tahiyah", 0.0 },
  { "Tahj", 0.9689668 },
  { "Tahja", 0.06048387 },
  { "Tahjae", 0.460177 },
  { "Tahjai", 0.71428573 },
  { "Tahjanae", 0.0 },
  { "Tahjanay", 0.0 },
  { "Tahjanee", 0.0 },
  { "Tahjay", 1.0 },
  { "Tahje", 0.64166665 },
  { "Tahjee", 1.0 },
  { "Tahji", 1.0 },
  { "Tahjia", 0.0 },
  { "Tahjir", 1.0 },
  { "Tahjmir", 1.0 },
  { "Tahlaya", 0.0 },
  { "Tahlea", 0.0 },
  { "Tahleah", 0.0 },
  { "Tahlee", 1.0 },
  { "Tahleek", 1.0 },
  { "Tahler", 0.0 },
  { "Tahlia", 0.0 },
  { "Tahliah", 0.0 },
  { "Tahlib", 1.0 },
  { "Tahlik", 1.0 },
  { "Tahlil", 1.0 },
  { "Tahliya", 0.0 },
  { "Tahliyah", 0.0 },
  { "Tahlor", 0.0 },
  { "Tahlya", 0.0 },
  { "Tahmeed", 1.0 },
  { "Tahmel", 1.0 },
  { "Tahmia", 0.0 },
  { "Tahmid", 1.0 },
  { "Tahmina", 0.0 },
  { "Tahmir", 1.0 },
  { "Tahmya", 0.0 },
  { "Tahna", 0.0 },
  { "Tahnee", 0.0 },
  { "Tahnesha", 0.0 },
  { "Tahni", 0.0 },
  { "Tahnia", 0.0 },
  { "Tahnisha", 0.0 },
  { "Tahniya", 0.0 },
  { "Tahnya", 0.0 },
  { "Tahra", 0.0 },
  { "Tahreem", 0.0 },
  { "Tahriq", 1.0 },
  { "Tahron", 1.0 },
  { "Tahseen", 1.0 },
  { "Tahsha", 0.0 },
  { "Tahshawn", 1.0 },
  { "Tahsin", 0.8137255 },
  { "Tahtiana", 0.0 },
  { "Tahtianna", 0.0 },
  { "Tahtyana", 0.0 },
  { "Tahya", 0.0 },
  { "Tai", 0.6089166 },
  { "Taia", 0.0 },
  { "Taiah", 0.0 },
  { "Taiana", 0.0 },
  { "Taianna", 0.0 },
  { "Taiasha", 0.0 },
  { "Taiba", 0.0 },
  { "Taichi", 1.0 },
  { "Taide", 0.0 },
  { "Taiden", 1.0 },
  { "Taidyn", 1.0 },
  { "Taiesha", 0.0 },
  { "Taifa", 0.0 },
  { "Taig", 1.0 },
  { "Taiga", 1.0 },
  { "Taigan", 0.08196721 },
  { "Taige", 0.7107438 },
  { "Taigen", 0.41935483 },
  { "Taighlor", 0.0 },
  { "Taigon", 1.0 },
  { "Taihlor", 0.0 },
  { "Taij", 1.0 },
  { "Taija", 0.0 },
  { "Taijae", 0.37931034 },
  { "Taijah", 0.0 },
  { "Taijanae", 0.0 },
  { "Taijarae", 0.0 },
  { "Taijay", 1.0 },
  { "Taijha", 0.0 },
  { "Taijon", 1.0 },
  { "Taijuan", 1.0 },
  { "Taiki", 1.0 },
  { "Taila", 0.0 },
  { "Tailah", 0.0 },
  { "Tailar", 0.0 },
  { "Tailee", 0.0 },
  { "Tailen", 1.0 },
  { "Tailer", 0.049261082 },
  { "Tailey", 0.0 },
  { "Tailon", 1.0 },
  { "Tailor", 0.14838709 },
  { "Tailore", 0.0 },
  { "Taily", 0.0 },
  { "Tailyn", 0.01904762 },
  { "Tailynn", 0.0 },
  { "Tailyr", 0.0 },
  { "Taim", 1.0 },
  { "Taima", 0.0 },
  { "Taimak", 1.0 },
  { "Taimane", 0.0 },
  { "Taimani", 0.0 },
  { "Taime", 0.0 },
  { "Taimi", 0.0 },
  { "Taimoor", 1.0 },
  { "Taimur", 1.0 },
  { "Taina", 0.0 },
  { "Taino", 1.0 },
  { "Taio", 1.0 },
  { "Taion", 1.0 },
  { "Taionna", 0.0 },
  { "Taiquan", 1.0 },
  { "Taira", 0.0 },
  { "Taire", 0.0 },
  { "Tairen", 0.0 },
  { "Tairik", 1.0 },
  { "Tairiq", 1.0 },
  { "Tairon", 1.0 },
  { "Tairra", 0.0 },
  { "Tairy", 0.0 },
  { "Tairyn", 0.0 },
  { "Tais", 0.0 },
  { "Taisa", 0.0 },
  { "Taisean", 1.0 },
  { "Taisei", 1.0 },
  { "Taisen", 1.0 },
  { "Taisha", 0.0 },
  { "Taishan", 1.0 },
  { "Taishaun", 1.0 },
  { "Taishawn", 1.0 },
  { "Taishi", 1.0 },
  { "Taishia", 0.0 },
  { "Taishima", 0.0 },
  { "Taishon", 1.0 },
  { "Taisia", 0.0 },
  { "Taisiya", 0.0 },
  { "Taislee", 0.0 },
  { "Taisley", 0.0 },
  { "Taison", 1.0 },
  { "Taissa", 0.0 },
  { "Taisto", 1.0 },
  { "Tait", 0.9859526 },
  { "Taite", 0.53974897 },
  { "Taiten", 1.0 },
  { "Taitiana", 0.0 },
  { "Taitianna", 0.0 },
  { "Taitlyn", 0.0 },
  { "Taiton", 1.0 },
  { "Taitum", 0.14975846 },
  { "Taityana", 0.0 },
  { "Taityanna", 0.0 },
  { "Taitym", 0.0 },
  { "Taiven", 1.0 },
  { "Taiveon", 1.0 },
  { "Taivion", 1.0 },
  { "Taivon", 1.0 },
  { "Taiwan", 0.9182969 },
  { "Taiwana", 0.0 },
  { "Taiwanda", 0.0 },
  { "Taiwanna", 0.0 },
  { "Taiwo", 0.7106599 },
  { "Taiwon", 1.0 },
  { "Taiya", 0.0 },
  { "Taiyah", 0.0 },
  { "Taiyana", 0.0 },
  { "Taiyanna", 0.0 },
  { "Taiyari", 0.0 },
  { "Taiyler", 0.0 },
  { "Taiylor", 0.0 },
  { "Taiyo", 1.0 },
  { "Taiyon", 1.0 },
  { "Taiz", 0.0 },
  { "Taizha", 0.0 },
  { "Taizlee", 0.0 },
  { "Taizley", 0.0 },
  { "Taizo", 1.0 },
  { "Taj", 0.9185464 },
  { "Taja", 0.015053763 },
  { "Tajae", 0.41284403 },
  { "Tajah", 0.082519 },
  { "Tajahn", 1.0 },
  { "Tajahnae", 0.0 },
  { "Tajahne", 0.0 },
  { "Tajai", 0.52884614 },
  { "Tajana", 0.0 },
  { "Tajanae", 0.0 },
  { "Tajanai", 0.0 },
  { "Tajanay", 0.0 },
  { "Tajane", 0.0 },
  { "Tajanea", 0.0 },
  { "Tajanee", 0.0 },
  { "Tajanique", 0.0 },
  { "Tajaun", 1.0 },
  { "Tajauna", 0.0 },
  { "Tajay", 0.8648649 },
  { "Taje", 0.55803573 },
  { "Tajea", 0.0 },
  { "Tajee", 0.7017544 },
  { "Tajh", 1.0 },
  { "Tajha", 0.0 },
  { "Tajhanae", 0.0 },
  { "Taji", 0.7714808 },
  { "Tajia", 0.0 },
  { "Tajiah", 0.0 },
  { "Tajiana", 0.0 },
  { "Tajianna", 0.0 },
  { "Tajiddin", 1.0 },
  { "Tajinae", 0.0 },
  { "Tajion", 1.0 },
  { "Tajir", 1.0 },
  { "Tajiri", 1.0 },
  { "Tajma", 0.0 },
  { "Tajmah", 0.0 },
  { "Tajmalik", 1.0 },
  { "Tajmere", 1.0 },
  { "Tajmir", 1.0 },
  { "Tajohn", 1.0 },
  { "Tajon", 1.0 },
  { "Tajsa", 0.0 },
  { "Tajuan", 0.9892966 },
  { "Tajuana", 0.0 },
  { "Tajuanda", 0.0 },
  { "Tajuanna", 0.0 },
  { "Tajudeen", 1.0 },
  { "Tajveer", 1.0 },
  { "Tajwar", 1.0 },
  { "Taka", 0.0 },
  { "Takahiro", 1.0 },
  { "Takai", 0.84761906 },
  { "Takaia", 0.0 },
  { "Takaila", 0.0 },
  { "Takaiya", 0.0 },
  { "Takaiyah", 0.0 },
  { "Takako", 0.0 },
  { "Takala", 0.0 },
  { "Takanori", 1.0 },
  { "Takao", 1.0 },
  { "Takara", 0.0 },
  { "Takarah", 0.0 },
  { "Takari", 0.721519 },
  { "Takaria", 0.0 },
  { "Takarra", 0.0 },
  { "Takasha", 0.0 },
  { "Takashi", 1.0 },
  { "Takashia", 0.0 },
  { "Takaya", 0.060240965 },
  { "Takayla", 0.0 },
  { "Takaylah", 0.0 },
  { "Takayuki", 1.0 },
  { "Takea", 0.0 },
  { "Takeara", 0.0 },
  { "Takeasha", 0.0 },
  { "Takecia", 0.0 },
  { "Takedra", 0.0 },
  { "Takeela", 0.0 },
  { "Takeem", 1.0 },
  { "Takeema", 0.0 },
  { "Takeena", 0.0 },
  { "Takeesha", 0.0 },
  { "Takeeta", 0.0 },
  { "Takeia", 0.0 },
  { "Takeichi", 1.0 },
  { "Takeidra", 0.0 },
  { "Takeila", 0.0 },
  { "Takeira", 0.0 },
  { "Takeisha", 0.0 },
  { "Takeita", 0.0 },
  { "Takeiya", 0.0 },
  { "Takeko", 0.0 },
  { "Takela", 0.0 },
  { "Takelia", 0.0 },
  { "Takella", 0.0 },
  { "Takema", 0.0 },
  { "Takemia", 0.0 },
  { "Takena", 0.0 },
  { "Takendra", 0.0 },
  { "Takenya", 0.0 },
  { "Takeo", 1.0 },
  { "Takera", 0.0 },
  { "Takeria", 0.0 },
  { "Takerra", 0.0 },
  { "Takerria", 0.0 },
  { "Takeru", 1.0 },
  { "Takesa", 0.0 },
  { "Takesha", 0.0 },
  { "Takeshi", 0.99267936 },
  { "Takeshia", 0.0 },
  { "Takesia", 0.0 },
  { "Taketa", 0.0 },
  { "Takevia", 0.0 },
  { "Takeya", 0.0 },
  { "Takeyah", 0.0 },
  { "Takeyia", 0.0 },
  { "Takeyla", 0.0 },
  { "Takeyra", 0.0 },
  { "Takeysha", 0.0 },
  { "Takeyshia", 0.0 },
  { "Takhi", 1.0 },
  { "Takhia", 0.0 },
  { "Takhiya", 0.0 },
  { "Taki", 0.90909094 },
  { "Takia", 0.0 },
  { "Takiah", 0.0 },
  { "Takiara", 0.0 },
  { "Takiea", 0.0 },
  { "Takiera", 0.0 },
  { "Takierra", 0.0 },
  { "Takiesha", 0.0 },
  { "Takijah", 0.0 },
  { "Takila", 0.0 },
  { "Takim", 1.0 },
  { "Takima", 0.0 },
  { "Takina", 0.0 },
  { "Takindra", 0.0 },
  { "Takira", 0.0 },
  { "Takirah", 0.0 },
  { "Takirra", 0.0 },
  { "Takisa", 0.0 },
  { "Takisha", 0.0 },
  { "Takishia", 0.0 },
  { "Takita", 0.0 },
  { "Takiya", 0.0 },
  { "Takiyah", 0.0 },
  { "Takiyla", 0.0 },
  { "Taknisha", 0.0 },
  { "Takobe", 1.0 },
  { "Takoda", 0.92373484 },
  { "Takoma", 1.0 },
  { "Takori", 1.0 },
  { "Takota", 0.91129035 },
  { "Takoya", 0.0 },
  { "Taksh", 1.0 },
  { "Taku", 1.0 },
  { "Takuma", 1.0 },
  { "Takumi", 1.0 },
  { "Takuto", 1.0 },
  { "Takuya", 1.0 },
  { "Takwon", 1.0 },
  { "Takya", 0.0 },
  { "Takyah", 0.0 },
  { "Takye", 1.0 },
  { "Takyia", 0.0 },
  { "Takyiah", 0.0 },
  { "Takyla", 0.0 },
  { "Takylah", 0.0 },
  { "Takylia", 0.0 },
  { "Takyra", 0.0 },
  { "Takyrah", 0.0 },
  { "Tal", 0.69495547 },
  { "Tala", 0.0 },
  { "Talae", 0.0 },
  { "Talah", 0.0 },
  { "Talaia", 0.0 },
  { "Talaiah", 0.0 },
  { "Talaija", 0.0 },
  { "Talaijah", 0.0 },
  { "Talaina", 0.0 },
  { "Talaisha", 0.0 },
  { "Talaiya", 0.0 },
  { "Talaiyah", 0.0 },
  { "Talaja", 0.0 },
  { "Talajah", 0.0 },
  { "Talajha", 0.0 },
  { "Talajiah", 0.0 },
  { "Talal", 1.0 },
  { "Talan", 0.98593104 },
  { "Talana", 0.0 },
  { "Talanda", 0.0 },
  { "Talandra", 0.0 },
  { "Talani", 0.0 },
  { "Talar", 0.0 },
  { "Talara", 0.0 },
  { "Talarisha", 0.0 },
  { "Talasha", 0.0 },
  { "Talasia", 0.0 },
  { "Talat", 1.0 },
  { "Talaya", 0.0 },
  { "Talayah", 0.0 },
  { "Talayasia", 0.0 },
  { "Talayeh", 0.0 },
  { "Talayia", 0.0 },
  { "Talayiah", 0.0 },
  { "Talayja", 0.0 },
  { "Talayjah", 0.0 },
  { "Talayla", 0.0 },
  { "Talayna", 0.0 },
  { "Talaysha", 0.0 },
  { "Talayshia", 0.0 },
  { "Talaysia", 0.0 },
  { "Talazia", 0.0 },
  { "Talbert", 1.0 },
  { "Talbot", 0.9758242 },
  { "Talbott", 1.0 },
  { "Talea", 0.0 },
  { "Taleah", 0.0 },
  { "Taleaha", 0.0 },
  { "Taleasha", 0.0 },
  { "Taleb", 1.0 },
  { "Talecia", 0.0 },
  { "Talee", 0.0 },
  { "Taleea", 0.0 },
  { "Taleeah", 0.0 },
  { "Taleek", 1.0 },
  { "Taleen", 0.0 },
  { "Taleena", 0.0 },
  { "Taleesa", 0.0 },
  { "Taleesha", 0.0 },
  { "Taleeya", 0.0 },
  { "Taleeyah", 0.0 },
  { "Talei", 0.0 },
  { "Taleia", 0.0 },
  { "Taleiah", 0.0 },
  { "Taleigh", 0.0 },
  { "Taleigha", 0.0 },
  { "Taleisa", 0.0 },
  { "Taleisha", 0.0 },
  { "Taleiya", 0.0 },
  { "Taleiyah", 0.0 },
  { "Taleka", 0.0 },
  { "Talen", 0.97425395 },
  { "Talena", 0.0 },
  { "Talene", 0.0 },
  { "Talent", 1.0 },
  { "Taler", 0.06849315 },
  { "Talesa", 0.0 },
  { "Talese", 0.0 },
  { "Talesha", 0.0 },
  { "Taleshia", 0.0 },
  { "Talesia", 0.0 },
  { "Taletha", 0.0 },
  { "Talethia", 0.0 },
  { "Talexis", 0.0 },
  { "Taley", 0.0 },
  { "Taleya", 0.0 },
  { "Taleyah", 0.0 },
  { "Talford", 1.0 },
  { "Talha", 1.0 },
  { "Talhah", 1.0 },
  { "Talhia", 0.0 },
  { "Tali", 0.05597015 },
  { "Talia", 0.00020348364 },
  { "Taliah", 0.0 },
  { "Taliana", 0.0 },
  { "Talianna", 0.0 },
  { "Taliayah", 0.0 },
  { "Talib", 1.0 },
  { "Taliba", 0.0 },
  { "Talibah", 0.0 },
  { "Talicia", 0.0 },
  { "Talie", 0.0 },
  { "Taliea", 0.0 },
  { "Taliek", 1.0 },
  { "Taliesha", 0.0 },
  { "Taliesin", 1.0 },
  { "Taliha", 0.0 },
  { "Talijah", 0.0 },
  { "Talik", 1.0 },
  { "Talika", 0.0 },
  { "Talim", 0.0 },
  { "Talin", 0.64638346 },
  { "Talina", 0.0 },
  { "Taline", 0.0 },
  { "Talion", 1.0 },
  { "Taliq", 1.0 },
  { "Talique", 1.0 },
  { "Talis", 0.872093 },
  { "Talisa", 0.0 },
  { "Talise", 0.0 },
  { "Talisha", 0.0 },
  { "Talishia", 0.0 },
  { "Talisia", 0.0 },
  { "Talissa", 0.0 },
  { "Talissia", 0.0 },
  { "Talita", 0.0 },
  { "Talitha", 0.0 },
  { "Talithia", 0.0 },
  { "Taliya", 0.0 },
  { "Taliyah", 0.0 },
  { "Taliyha", 0.0 },
  { "Talla", 0.0 },
  { "Tallan", 1.0 },
  { "Tallen", 1.0 },
  { "Talley", 0.2109375 },
  { "Talli", 0.0 },
  { "Tallia", 0.0 },
  { "Tallie", 0.19377163 },
  { "Tallin", 1.0 },
  { "Tallis", 0.60504204 },
  { "Talliyah", 0.0 },
  { "Tallon", 1.0 },
  { "Tallula", 0.0 },
  { "Tallulah", 0.0 },
  { "Tally", 0.22025724 },
  { "Tallyn", 0.44036698 },
  { "Talma", 0.055555556 },
  { "Talmadge", 0.9935657 },
  { "Talmage", 1.0 },
  { "Talmer", 1.0 },
  { "Talmon", 1.0 },
  { "Talon", 0.9583054 },
  { "Talona", 0.0 },
  { "Talonda", 0.0 },
  { "Talor", 0.26238286 },
  { "Talore", 0.0 },
  { "Taloria", 0.0 },
  { "Talsha", 0.0 },
  { "Talton", 1.0 },
  { "Talula", 0.0 },
  { "Talulah", 0.0 },
  { "Talulla", 0.0 },
  { "Talullah", 0.0 },
  { "Talus", 1.0 },
  { "Talvin", 1.0 },
  { "Taly", 0.0 },
  { "Talya", 0.0 },
  { "Talyah", 0.0 },
  { "Talyia", 0.0 },
  { "Talyiah", 0.0 },
  { "Talyn", 0.48370275 },
  { "Talynn", 0.018726591 },
  { "Talyor", 0.0 },
  { "Talyr", 0.0 },
  { "Talysa", 0.0 },
  { "Talysha", 0.0 },
  { "Talyssa", 0.0 },
  { "Tam", 0.52300787 },
  { "Tama", 0.0041695624 },
  { "Tamae", 0.0 },
  { "Tamah", 0.0 },
  { "Tamai", 0.0 },
  { "Tamaia", 0.0 },
  { "Tamaiah", 0.0 },
  { "Tamaica", 0.0 },
  { "Tamaika", 0.0 },
  { "Tamaine", 1.0 },
  { "Tamaira", 0.0 },
  { "Tamaiya", 0.0 },
  { "Tamaj", 1.0 },
  { "Tamajah", 0.0 },
  { "Tamaka", 0.0 },
  { "Tamaki", 0.0 },
  { "Tamakia", 0.0 },
  { "Tamal", 1.0 },
  { "Tamala", 0.0 },
  { "Tamalia", 0.0 },
  { "Tamalyn", 0.0 },
  { "Tamana", 0.0 },
  { "Tamani", 0.0 },
  { "Tamanika", 0.0 },
  { "Tamanna", 0.0 },
  { "Tamantha", 0.0 },
  { "Tamar", 0.13392997 },
  { "Tamara", 0.0031925156 },
  { "Tamarae", 0.0 },
  { "Tamarah", 0.0 },
  { "Tamaran", 0.0 },
  { "Tamarcus", 1.0 },
  { "Tamare", 0.54545456 },
  { "Tamarea", 0.0 },
  { "Tamaree", 0.0 },
  { "Tamari", 0.33773088 },
  { "Tamaria", 0.0 },
  { "Tamariah", 0.0 },
  { "Tamarian", 1.0 },
  { "Tamariana", 0.0 },
  { "Tamarick", 1.0 },
  { "Tamarie", 0.0 },
  { "Tamarik", 1.0 },
  { "Tamarin", 0.0 },
  { "Tamarind", 0.0 },
  { "Tamario", 1.0 },
  { "Tamarion", 1.0 },
  { "Tamarious", 1.0 },
  { "Tamaris", 0.44827586 },
  { "Tamarisk", 0.0 },
  { "Tamarius", 1.0 },
  { "Tamarkus", 1.0 },
  { "Tamaro", 0.0 },
  { "Tamaron", 0.0 },
  { "Tamarr", 0.7058824 },
  { "Tamarra", 0.0 },
  { "Tamarrah", 0.0 },
  { "Tamarria", 0.0 },
  { "Tamarrion", 1.0 },
  { "Tamarsha", 0.0 },
  { "Tamarya", 0.0 },
  { "Tamas", 1.0 },
  { "Tamasha", 0.0 },
  { "Tamasine", 0.0 },
  { "Tamatha", 0.0 },
  { "Tamathia", 0.0 },
  { "Tamathy", 0.0 },
  { "Tamatoa", 1.0 },
  { "Tamauri", 0.74358976 },
  { "Tamaurice", 1.0 },
  { "Tamaya", 0.0 },
  { "Tamayah", 0.0 },
  { "Tamaye", 0.0 },
  { "Tamayia", 0.0 },
  { "Tamaz", 1.0 },
  { "Tamba", 0.47826087 },
  { "Tamber", 0.0 },
  { "Tamberly", 0.0 },
  { "Tamberlyn", 0.0 },
  { "Tambi", 0.0 },
  { "Tamblyn", 0.0 },
  { "Tamboura", 0.7058824 },
  { "Tambra", 0.0 },
  { "Tambre", 0.0 },
  { "Tambresha", 0.0 },
  { "Tambrey", 0.0 },
  { "Tambria", 0.0 },
  { "Tambry", 0.0 },
  { "Tamea", 0.0 },
  { "Tameah", 0.0 },
  { "Tameaka", 0.0 },
  { "Tameca", 0.0 },
  { "Tamecca", 0.0 },
  { "Tamecha", 0.0 },
  { "Tamecia", 0.0 },
  { "Tamecka", 0.0 },
  { "Tameco", 0.0 },
  { "Tamee", 0.0 },
  { "Tameeka", 0.0 },
  { "Tameem", 1.0 },
  { "Tameer", 1.0 },
  { "Tameera", 0.0 },
  { "Tameesha", 0.0 },
  { "Tameia", 0.0 },
  { "Tameica", 0.0 },
  { "Tameika", 0.0 },
  { "Tameiko", 0.0 },
  { "Tameira", 0.0 },
  { "Tameisha", 0.0 },
  { "Tameka", 0.003211153 },
  { "Tameke", 0.0 },
  { "Tamekea", 0.0 },
  { "Tameki", 0.0 },
  { "Tamekia", 0.0 },
  { "Tamekio", 0.0 },
  { "Tamekka", 0.0 },
  { "Tameko", 0.0 },
  { "Tamel", 1.0 },
  { "Tamela", 0.0 },
  { "Tamelia", 0.0 },
  { "Tamell", 1.0 },
  { "Tamella", 0.0 },
  { "Tamem", 1.0 },
  { "Tamer", 0.9578544 },
  { "Tamera", 0.0 },
  { "Tamerah", 0.0 },
  { "Tamere", 1.0 },
  { "Tameria", 0.0 },
  { "Tamerick", 1.0 },
  { "Tameron", 0.6882353 },
  { "Tamerra", 0.0 },
  { "Tamesa", 0.0 },
  { "Tamesha", 0.0 },
  { "Tameshia", 0.0 },
  { "Tamesia", 0.0 },
  { "Tametha", 0.0 },
  { "Tametra", 0.0 },
  { "Tametria", 0.0 },
  { "Tamey", 0.0 },
  { "Tameya", 0.0 },
  { "Tameyah", 0.0 },
  { "Tameyka", 0.0 },
  { "Tamhra", 0.0 },
  { "Tami", 0.0023963766 },
  { "Tamia", 0.0 },
  { "Tamiah", 0.0 },
  { "Tamiaya", 0.0 },
  { "Tamica", 0.0 },
  { "Tamicha", 0.0 },
  { "Tamichael", 1.0 },
  { "Tamicia", 0.0 },
  { "Tamicka", 0.0 },
  { "Tamicko", 0.0 },
  { "Tamico", 0.0 },
  { "Tamie", 0.0 },
  { "Tamieka", 0.0 },
  { "Tamieko", 0.0 },
  { "Tamiera", 0.0 },
  { "Tamiesha", 0.0 },
  { "Tamiia", 0.0 },
  { "Tamiika", 0.0 },
  { "Tamija", 0.35 },
  { "Tamijah", 0.0 },
  { "Tamijo", 0.0 },
  { "Tamika", 0.0039491006 },
  { "Tamikah", 0.0 },
  { "Tamikca", 0.0 },
  { "Tamike", 0.0 },
  { "Tamiki", 0.0 },
  { "Tamikia", 0.0 },
  { "Tamikka", 0.0 },
  { "Tamiko", 0.013820335 },
  { "Tamila", 0.0 },
  { "Tamilia", 0.0 },
  { "Tamilla", 0.0 },
  { "Tamille", 0.0 },
  { "Tamillia", 0.0 },
  { "Tamilore", 0.1923077 },
  { "Tamilyn", 0.0 },
  { "Tamim", 1.0 },
  { "Tamina", 0.0 },
  { "Taminka", 0.0 },
  { "Tamio", 1.0 },
  { "Tamiqua", 0.0 },
  { "Tamir", 0.96720296 },
  { "Tamira", 0.0 },
  { "Tamiracle", 0.0 },
  { "Tamirah", 0.0 },
  { "Tamiria", 0.0 },
  { "Tamirra", 0.0 },
  { "Tamisa", 0.0 },
  { "Tamisha", 0.0 },
  { "Tamishia", 0.0 },
  { "Tamita", 0.0 },
  { "Tamitha", 0.0 },
  { "Tamitra", 0.0 },
  { "Tamiya", 0.0 },
  { "Tamiyah", 0.0 },
  { "Tamka", 0.0 },
  { "Tamkia", 0.0 },
  { "Tamkio", 0.0 },
  { "Tamla", 0.0 },
  { "Tamlyn", 0.0 },
  { "Tamma", 0.0 },
  { "Tammala", 0.0 },
  { "Tammany", 0.0 },
  { "Tammara", 0.0 },
  { "Tammarra", 0.0 },
  { "Tammatha", 0.0 },
  { "Tamme", 0.0 },
  { "Tammee", 0.0 },
  { "Tammeka", 0.0 },
  { "Tammela", 0.0 },
  { "Tammer", 1.0 },
  { "Tammera", 0.0 },
  { "Tammeria", 0.0 },
  { "Tammetha", 0.0 },
  { "Tammey", 0.0 },
  { "Tammi", 0.0 },
  { "Tammia", 0.0 },
  { "Tammica", 0.0 },
  { "Tammie", 0.0037749668 },
  { "Tammika", 0.0 },
  { "Tammila", 0.0 },
  { "Tammis", 0.0 },
  { "Tammitha", 0.0 },
  { "Tammmy", 0.0 },
  { "Tammora", 0.0 },
  { "Tammra", 0.0 },
  { "Tammy", 0.0041793645 },
  { "Tammyann", 0.0 },
  { "Tammye", 0.0 },
  { "Tammyjo", 0.0 },
  { "Tammylee", 0.0 },
  { "Tammylynn", 0.0 },
  { "Tammyra", 0.0 },
  { "Tamon", 1.0 },
  { "Tamona", 0.0 },
  { "Tamoni", 0.0 },
  { "Tamonica", 0.0 },
  { "Tamora", 0.0 },
  { "Tamorah", 0.0 },
  { "Tamorion", 1.0 },
  { "Tamotsu", 1.0 },
  { "Tamoya", 0.0 },
  { "Tampa", 0.0 },
  { "Tampatha", 0.0 },
  { "Tamra", 0.0003648836 },
  { "Tamrah", 0.0 },
  { "Tamre", 0.0 },
  { "Tamriel", 0.0 },
  { "Tamron", 0.7826087 },
  { "Tamryn", 0.029411765 },
  { "Tamsen", 0.0 },
  { "Tamsin", 0.0 },
  { "Tamson", 0.0 },
  { "Tamsyn", 0.0 },
  { "Tamu", 0.01724138 },
  { "Tamula", 0.0 },
  { "Tamura", 0.0 },
  { "Tamy", 0.0 },
  { "Tamya", 0.0 },
  { "Tamyah", 0.0 },
  { "Tamye", 0.0 },
  { "Tamyia", 0.0 },
  { "Tamyiah", 0.0 },
  { "Tamyka", 0.0 },
  { "Tamyko", 0.0 },
  { "Tamyla", 0.0 },
  { "Tamyra", 0.0 },
  { "Tamyrah", 0.0 },
  { "Tamyria", 0.0 },
  { "Tamzen", 0.0 },
  { "Tamzin", 0.0 },
  { "Tamzyn", 0.0 },
  { "Tan", 0.98109967 },
  { "Tana", 0.000653666 },
  { "Tanaa", 0.0 },
  { "Tanae", 0.0 },
  { "Tanaeja", 0.0 },
  { "Tanaejah", 0.0 },
  { "Tanah", 0.0 },
  { "Tanai", 0.0 },
  { "Tanaia", 0.0 },
  { "Tanaija", 0.0 },
  { "Tanaijah", 0.0 },
  { "Tanairi", 0.0 },
  { "Tanairy", 0.0 },
  { "Tanaisa", 0.0 },
  { "Tanaisha", 0.0 },
  { "Tanaiya", 0.0 },
  { "Tanaiyah", 0.0 },
  { "Tanaja", 0.0 },
  { "Tanajah", 0.0 },
  { "Tanajha", 0.0 },
  { "Tanajia", 0.0 },
  { "Tanaka", 0.07194245 },
  { "Tanar", 1.0 },
  { "Tanara", 0.0 },
  { "Tanard", 1.0 },
  { "Tanari", 0.0 },
  { "Tanaria", 0.0 },
  { "Tanasha", 0.0 },
  { "Tanashia", 0.0 },
  { "Tanasia", 0.0 },
  { "Tanautica", 0.0 },
  { "Tanav", 1.0 },
  { "Tanavia", 0.0 },
  { "Tanay", 0.56692916 },
  { "Tanaya", 0.0 },
  { "Tanayah", 0.0 },
  { "Tanayia", 0.0 },
  { "Tanayja", 0.0 },
  { "Tanayjah", 0.0 },
  { "Tanaysha", 0.0 },
  { "Tanayshia", 0.0 },
  { "Tanaysia", 0.0 },
  { "Tanayzia", 0.0 },
  { "Tanazia", 0.0 },
  { "Tancy", 0.0 },
  { "Tanda", 0.0 },
  { "Tandeka", 0.0 },
  { "Tanden", 1.0 },
  { "Tandi", 0.0 },
  { "Tandice", 0.0 },
  { "Tandie", 0.0 },
  { "Tandis", 0.0 },
  { "Tandon", 1.0 },
  { "Tandra", 0.0 },
  { "Tandre", 1.0 },
  { "Tandrea", 0.0 },
  { "Tandria", 0.0 },
  { "Tandy", 0.38019168 },
  { "Tane", 0.09659091 },
  { "Tanea", 0.0 },
  { "Taneah", 0.0 },
  { "Taneal", 0.0 },
  { "Taneasha", 0.0 },
  { "Taneca", 0.0 },
  { "Tanechia", 0.0 },
  { "Tanecia", 0.0 },
  { "Tanedra", 0.0 },
  { "Tanee", 0.0 },
  { "Taneeka", 0.0 },
  { "Taneesh", 1.0 },
  { "Taneesha", 0.0 },
  { "Taneha", 0.0 },
  { "Taneia", 0.0 },
  { "Taneice", 0.0 },
  { "Taneicia", 0.0 },
  { "Taneika", 0.0 },
  { "Taneil", 0.0 },
  { "Taneille", 0.0 },
  { "Taneiqua", 0.0 },
  { "Taneisha", 0.0 },
  { "Taneishia", 0.0 },
  { "Taneiya", 0.0 },
  { "Taneja", 0.0 },
  { "Tanejah", 0.0 },
  { "Tanek", 1.0 },
  { "Taneka", 0.0 },
  { "Tanekia", 0.0 },
  { "Tanekqua", 0.0 },
  { "Tanell", 0.0 },
  { "Tanelle", 0.0 },
  { "Tanequa", 0.0 },
  { "Taner", 1.0 },
  { "Tanera", 0.0 },
  { "Taneria", 0.0 },
  { "Tanesa", 0.0 },
  { "Tanesha", 0.0 },
  { "Taneshia", 0.0 },
  { "Tanesia", 0.0 },
  { "Tanessa", 0.0 },
  { "Tanetta", 0.0 },
  { "Tanette", 0.0 },
  { "Taneya", 0.0 },
  { "Taneyah", 0.0 },
  { "Taneysha", 0.0 },
  { "Tang", 0.0 },
  { "Tanga", 0.0 },
  { "Tangala", 0.0 },
  { "Tanganika", 0.0 },
  { "Tanganyika", 0.0 },
  { "Tange", 0.0 },
  { "Tangee", 0.0 },
  { "Tangela", 0.0 },
  { "Tangelia", 0.0 },
  { "Tangella", 0.0 },
  { "Tangelo", 1.0 },
  { "Tangenika", 0.0 },
  { "Tanger", 0.0 },
  { "Tangerine", 0.0 },
  { "Tangerla", 0.0 },
  { "Tangi", 0.0 },
  { "Tangia", 0.0 },
  { "Tangie", 0.0 },
  { "Tangier", 0.0 },
  { "Tangila", 0.0 },
  { "Tangina", 0.0 },
  { "Tangla", 0.0 },
  { "Tangle", 0.0 },
  { "Tanglia", 0.0 },
  { "Tango", 0.0 },
  { "Tangula", 0.0 },
  { "Tangular", 0.0 },
  { "Tangy", 0.0 },
  { "Tani", 0.010141988 },
  { "Tania", 0.002160605 },
  { "Taniah", 0.0 },
  { "Taniaya", 0.0 },
  { "Taniayah", 0.0 },
  { "Tanica", 0.0 },
  { "Tanice", 0.0 },
  { "Tanicha", 0.0 },
  { "Tanicia", 0.0 },
  { "Tanicka", 0.0 },
  { "Tanida", 0.0 },
  { "Taniece", 0.0 },
  { "Tanieka", 0.0 },
  { "Taniel", 0.0 },
  { "Taniela", 1.0 },
  { "Tanielle", 0.0 },
  { "Tanielu", 1.0 },
  { "Taniesha", 0.0 },
  { "Taniha", 0.0 },
  { "Tanihya", 0.0 },
  { "Tanija", 0.0 },
  { "Tanijah", 0.0 },
  { "Tanijha", 0.0 },
  { "Tanika", 0.0 },
  { "Tanikia", 0.0 },
  { "Tanikka", 0.0 },
  { "Taniko", 0.0 },
  { "Tanikqua", 0.0 },
  { "Tanila", 0.0 },
  { "Tanilah", 0.0 },
  { "Tanille", 0.0 },
  { "Tanin", 1.0 },
  { "Tanina", 0.0 },
  { "Taniqua", 0.0 },
  { "Tanique", 0.0 },
  { "Tanira", 0.0 },
  { "Tanis", 0.33782873 },
  { "Tanisa", 0.0 },
  { "Tanise", 0.0 },
  { "Tanish", 0.8669355 },
  { "Tanisha", 0.0010984413 },
  { "Tanishea", 0.0 },
  { "Tanishi", 0.0 },
  { "Tanishia", 0.0 },
  { "Tanishka", 0.0 },
  { "Tanishq", 1.0 },
  { "Tanisi", 0.0 },
  { "Tanisia", 0.0 },
  { "Tanita", 0.0 },
  { "Tanith", 0.0 },
  { "Tanitha", 0.0 },
  { "Tanitra", 0.0 },
  { "Taniya", 0.0 },
  { "Taniyah", 0.0 },
  { "Taniyha", 0.0 },
  { "Taniyia", 0.0 },
  { "Taniyla", 0.0 },
  { "Taniylah", 0.0 },
  { "Tanja", 0.0 },
  { "Tanjala", 0.0 },
  { "Tanjanae", 0.0 },
  { "Tanjanika", 0.0 },
  { "Tanjanique", 0.0 },
  { "Tanji", 0.0 },
  { "Tanjie", 0.0 },
  { "Tank", 1.0 },
  { "Tankia", 0.0 },
  { "Tanley", 0.0 },
  { "Tanmay", 1.0 },
  { "Tanmayi", 0.0 },
  { "Tanna", 0.0 },
  { "Tannah", 0.0 },
  { "Tannar", 0.9337748 },
  { "Tannaz", 0.0 },
  { "Tannen", 1.0 },
  { "Tanner", 0.9769571 },
  { "Tannesha", 0.0 },
  { "Tanneshia", 0.0 },
  { "Tannetta", 0.0 },
  { "Tannette", 0.0 },
  { "Tanni", 0.0 },
  { "Tannia", 0.0 },
  { "Tannie", 0.09016393 },
  { "Tannika", 0.0 },
  { "Tannille", 0.0 },
  { "Tannim", 1.0 },
  { "Tannin", 1.0 },
  { "Tannis", 0.015337423 },
  { "Tannisha", 0.0 },
  { "Tannon", 1.0 },
  { "Tannor", 1.0 },
  { "Tanny", 0.053475935 },
  { "Tannya", 0.0 },
  { "Tannyr", 1.0 },
  { "Tano", 1.0 },
  { "Tanoa", 0.0 },
  { "Tanor", 1.0 },
  { "Tansey", 0.0 },
  { "Tansha", 0.0 },
  { "Tansi", 0.0 },
  { "Tansy", 0.0 },
  { "Tantanea", 0.0 },
  { "Tantania", 0.0 },
  { "Tante", 0.0 },
  { "Tantra", 0.0 },
  { "Tanuj", 1.0 },
  { "Tanuja", 0.0 },
  { "Tanush", 1.0 },
  { "Tanusha", 0.0 },
  { "Tanushka", 0.0 },
  { "Tanushri", 0.0 },
  { "Tanvee", 0.0 },
  { "Tanveer", 0.78612715 },
  { "Tanvi", 0.0 },
  { "Tanvika", 0.0 },
  { "Tanvir", 0.972549 },
  { "Tany", 0.0 },
  { "Tanya", 0.0034700504 },
  { "Tanyah", 0.0 },
  { "Tanyanika", 0.0 },
  { "Tanyea", 0.0 },
  { "Tanyeka", 0.0 },
  { "Tanyel", 0.0 },
  { "Tanyell", 0.0 },
  { "Tanyelle", 0.0 },
  { "Tanyetta", 0.0 },
  { "Tanyette", 0.0 },
  { "Tanyha", 0.0 },
  { "Tanyia", 0.0 },
  { "Tanyiah", 0.0 },
  { "Tanyika", 0.0 },
  { "Tanyja", 0.0 },
  { "Tanyjah", 0.0 },
  { "Tanyka", 0.0 },
  { "Tanyla", 0.0 },
  { "Tanylah", 0.0 },
  { "Tanyon", 1.0 },
  { "Tanyra", 0.0 },
  { "Tanysha", 0.0 },
  { "Tanza", 0.0 },
  { "Tanzania", 0.0 },
  { "Tanzi", 0.0 },
  { "Tanzia", 0.0 },
  { "Tanzie", 0.0 },
  { "Tanzila", 0.0 },
  { "Tanzim", 1.0 },
  { "Tanzy", 0.0 },
  { "Tao", 0.9690403 },
  { "Taonna", 0.0 },
  { "Taos", 1.0 },
  { "Tapainga", 0.0 },
  { "Tapan", 1.0 },
  { "Tapanga", 0.0 },
  { "Tapasya", 0.0 },
  { "Tapatha", 0.0 },
  { "Taqee", 1.0 },
  { "Taqi", 1.0 },
  { "Taqiyya", 0.0 },
  { "Taqiyyah", 0.0 },
  { "Taquan", 1.0 },
  { "Taquana", 0.0 },
  { "Taquanda", 0.0 },
  { "Taquanna", 0.0 },
  { "Taquanta", 0.0 },
  { "Taquarius", 1.0 },
  { "Taquasha", 0.0 },
  { "Taquasia", 0.0 },
  { "Taqueena", 0.0 },
  { "Taquella", 0.0 },
  { "Taqueria", 0.0 },
  { "Taquesha", 0.0 },
  { "Taquetta", 0.0 },
  { "Taquez", 1.0 },
  { "Taquia", 0.0 },
  { "Taquil", 1.0 },
  { "Taquila", 0.0 },
  { "Taquilla", 0.0 },
  { "Taquira", 0.0 },
  { "Taquisa", 0.0 },
  { "Taquisha", 0.0 },
  { "Taquita", 0.0 },
  { "Taquoia", 0.0 },
  { "Taquon", 1.0 },
  { "Taquoya", 0.0 },
  { "Taqwa", 0.0 },
  { "Taqwan", 1.0 },
  { "Tar", 0.0 },
  { "Tara", 0.0031185031 },
  { "Taraann", 0.0 },
  { "Tarae", 0.0877193 },
  { "Taraf", 0.0 },
  { "Tarah", 0.0 },
  { "Tarahji", 0.06930693 },
  { "Taraj", 1.0 },
  { "Taraja", 0.0 },
  { "Tarajah", 0.0 },
  { "Tarajai", 0.0 },
  { "Tarajee", 0.0 },
  { "Tarajhi", 0.0 },
  { "Taraji", 0.021701388 },
  { "Tarajii", 0.0 },
  { "Tarak", 1.0 },
  { "Taralee", 0.0 },
  { "Taralyn", 0.0 },
  { "Taralynn", 0.0 },
  { "Taramarie", 0.0 },
  { "Taran", 0.7603766 },
  { "Tarana", 0.0 },
  { "Tarance", 1.0 },
  { "Taranda", 0.0 },
  { "Taraneh", 0.0 },
  { "Taraneka", 0.0 },
  { "Taranika", 0.0 },
  { "Taranique", 0.0 },
  { "Taranisha", 0.0 },
  { "Taraoluwa", 0.0 },
  { "Taras", 1.0 },
  { "Tarasa", 0.0 },
  { "Tarasha", 0.0 },
  { "Taraus", 1.0 },
  { "Taray", 1.0 },
  { "Taraya", 0.0 },
  { "Tarcha", 0.0 },
  { "Tarcia", 0.0 },
  { "Tarea", 0.0 },
  { "Taree", 0.23809524 },
  { "Tareek", 1.0 },
  { "Tareena", 0.0 },
  { "Tareia", 0.0 },
  { "Tarek", 1.0 },
  { "Tareka", 0.0 },
  { "Tarel", 1.0 },
  { "Tarell", 1.0 },
  { "Tarelle", 1.0 },
  { "Taren", 0.2809406 },
  { "Tarena", 0.0 },
  { "Tarence", 1.0 },
  { "Tareq", 1.0 },
  { "Taresa", 0.0 },
  { "Taresha", 0.0 },
  { "Taressa", 0.0 },
  { "Tareva", 0.0 },
  { "Tareyn", 0.0 },
  { "Tarez", 1.0 },
  { "Tarha", 0.0 },
  { "Tarhonda", 0.0 },
  { "Tari", 0.020087337 },
  { "Taria", 0.03687943 },
  { "Tariah", 0.0 },
  { "Tarian", 1.0 },
  { "Tariana", 0.0 },
  { "Tarianna", 0.0 },
  { "Taric", 1.0 },
  { "Tarica", 0.0 },
  { "Tarick", 1.0 },
  { "Tarie", 0.0 },
  { "Tariek", 1.0 },
  { "Tarif", 1.0 },
  { "Tarig", 1.0 },
  { "Tarijah", 0.0 },
  { "Tarik", 0.9971189 },
  { "Tarika", 0.0 },
  { "Tarin", 0.26337793 },
  { "Tarina", 0.0 },
  { "Tarini", 0.0 },
  { "Tario", 1.0 },
  { "Tarion", 1.0 },
  { "Tariq", 0.99876696 },
  { "Tarique", 1.0 },
  { "Taris", 0.74846625 },
  { "Tarisa", 0.0 },
  { "Tarisha", 0.0 },
  { "Tarissa", 0.0 },
  { "Tarita", 0.0 },
  { "Taritha", 0.0 },
  { "Tarius", 1.0 },
  { "Tariya", 0.0 },
  { "Tariyah", 0.0 },
  { "Tarja", 0.0 },
  { "Tarji", 0.0 },
  { "Tarkan", 1.0 },
  { "Tarkesha", 0.0 },
  { "Tarkeshia", 0.0 },
  { "Tarkisha", 0.0 },
  { "Tarl", 1.0 },
  { "Tarla", 0.0 },
  { "Tarlisha", 0.0 },
  { "Tarlton", 1.0 },
  { "Tarmara", 0.0 },
  { "Tarna", 0.0 },
  { "Tarneisha", 0.0 },
  { "Tarnell", 1.0 },
  { "Tarnesha", 0.0 },
  { "Tarneshia", 0.0 },
  { "Tarnisha", 0.0 },
  { "Tarnya", 0.0 },
  { "Taro", 1.0 },
  { "Tarolyn", 0.0 },
  { "Taron", 0.96303904 },
  { "Taronda", 0.0 },
  { "Tarone", 1.0 },
  { "Taronica", 0.0 },
  { "Taronza", 0.0 },
  { "Tarquin", 1.0 },
  { "Tarra", 0.0 },
  { "Tarrah", 0.0 },
  { "Tarralyn", 0.0 },
  { "Tarran", 0.4868421 },
  { "Tarrance", 1.0 },
  { "Tarrant", 1.0 },
  { "Tarrel", 1.0 },
  { "Tarrell", 0.98571426 },
  { "Tarren", 0.64777946 },
  { "Tarrence", 1.0 },
  { "Tarri", 0.0 },
  { "Tarria", 0.0 },
  { "Tarrick", 1.0 },
  { "Tarrie", 0.04587156 },
  { "Tarrik", 1.0 },
  { "Tarrin", 0.53809524 },
  { "Tarris", 1.0 },
  { "Tarrius", 1.0 },
  { "Tarrod", 1.0 },
  { "Tarron", 0.9526066 },
  { "Tarrus", 1.0 },
  { "Tarry", 0.6946688 },
  { "Tarryl", 1.0 },
  { "Tarryn", 0.013386881 },
  { "Tarsha", 0.0 },
  { "Tarsheka", 0.0 },
  { "Tarshia", 0.0 },
  { "Tarteel", 0.0 },
  { "Tarun", 1.0 },
  { "Tarus", 1.0 },
  { "Tarvares", 1.0 },
  { "Tarvaris", 1.0 },
  { "Tarvarus", 1.0 },
  { "Tarver", 1.0 },
  { "Tarvin", 1.0 },
  { "Tarvis", 1.0 },
  { "Tary", 0.6551724 },
  { "Tarya", 0.0 },
  { "Taryah", 0.0 },
  { "Taryiah", 0.0 },
  { "Taryl", 1.0 },
  { "Taryll", 1.0 },
  { "Taryn", 0.026063642 },
  { "Taryne", 0.0 },
  { "Tarynn", 0.0 },
  { "Tarzan", 1.0 },
  { "Tasa", 0.0 },
  { "Tasanee", 0.0 },
  { "Tasani", 0.0 },
  { "Tascha", 0.0 },
  { "Tasean", 0.9796748 },
  { "Taseen", 1.0 },
  { "Tasfia", 0.0 },
  { "Tash", 0.5 },
  { "Tasha", 0.0029875985 },
  { "Tashad", 1.0 },
  { "Tashae", 0.0 },
  { "Tashai", 0.0 },
  { "Tashala", 0.0 },
  { "Tashalee", 0.0 },
  { "Tashan", 0.924812 },
  { "Tashana", 0.0 },
  { "Tashanae", 0.0 },
  { "Tashanda", 0.0 },
  { "Tashani", 0.0 },
  { "Tashanika", 0.0 },
  { "Tashanique", 0.0 },
  { "Tashanna", 0.0 },
  { "Tashante", 0.0 },
  { "Tashanti", 0.0 },
  { "Tashara", 0.0 },
  { "Tashard", 1.0 },
  { "Tashari", 0.0 },
  { "Tasharia", 0.0 },
  { "Tasharra", 0.0 },
  { "Tashaun", 0.94698083 },
  { "Tashauna", 0.0 },
  { "Tashaunda", 0.0 },
  { "Tashaunti", 0.0 },
  { "Tashawn", 0.83893806 },
  { "Tashawna", 0.0 },
  { "Tashawnda", 0.0 },
  { "Tashawnna", 0.0 },
  { "Tashay", 0.0 },
  { "Tashaya", 0.0 },
  { "Tashaye", 0.0 },
  { "Tashayla", 0.0 },
  { "Tashe", 0.0 },
  { "Tashea", 0.0 },
  { "Tasheana", 0.0 },
  { "Tasheanna", 0.0 },
  { "Tasheba", 0.0 },
  { "Tashee", 0.0 },
  { "Tasheeka", 0.0 },
  { "Tasheem", 1.0 },
  { "Tasheema", 0.0 },
  { "Tasheen", 0.0 },
  { "Tasheena", 0.0 },
  { "Tasheia", 0.0 },
  { "Tasheika", 0.0 },
  { "Tasheka", 0.0 },
  { "Tashekia", 0.0 },
  { "Tashel", 0.0 },
  { "Tashell", 0.0 },
  { "Tashelle", 0.0 },
  { "Tashema", 0.0 },
  { "Tashena", 0.0 },
  { "Tashenna", 0.0 },
  { "Tasheona", 0.0 },
  { "Tashera", 0.0 },
  { "Tasheria", 0.0 },
  { "Tasherra", 0.0 },
  { "Tasheyana", 0.0 },
  { "Tashfia", 0.0 },
  { "Tashi", 0.14777328 },
  { "Tashia", 0.0 },
  { "Tashiana", 0.0 },
  { "Tashianna", 0.0 },
  { "Tashiba", 0.0 },
  { "Tashica", 0.0 },
  { "Tashie", 0.0 },
  { "Tashieka", 0.0 },
  { "Tashiem", 1.0 },
  { "Tashika", 0.0 },
  { "Tashima", 0.0 },
  { "Tashina", 0.0 },
  { "Tashiona", 0.0 },
  { "Tashionna", 0.0 },
  { "Tashira", 0.0 },
  { "Tashiya", 0.0 },
  { "Tashiyah", 0.0 },
  { "Tashiyana", 0.0 },
  { "Tashma", 0.0 },
  { "Tashon", 0.95522386 },
  { "Tashona", 0.0 },
  { "Tashonda", 0.0 },
  { "Tashonna", 0.0 },
  { "Tashua", 0.0 },
  { "Tashun", 1.0 },
  { "Tashuna", 0.0 },
  { "Tashunda", 0.0 },
  { "Tashvi", 0.0 },
  { "Tashya", 0.0 },
  { "Tashyah", 0.0 },
  { "Tashyana", 0.0 },
  { "Tashyia", 0.0 },
  { "Tashyla", 0.0 },
  { "Tashyra", 0.0 },
  { "Tasi", 0.28070176 },
  { "Tasia", 0.0 },
  { "Tasiah", 0.0 },
  { "Tasiana", 0.0 },
  { "Tasina", 0.0 },
  { "Tasir", 1.0 },
  { "Tasiya", 0.0 },
  { "Tasiyah", 0.0 },
  { "Tasja", 0.0 },
  { "Taska", 0.0 },
  { "Tasker", 1.0 },
  { "Taskin", 1.0 },
  { "Taslin", 0.0 },
  { "Tasma", 0.0 },
  { "Tasman", 1.0 },
  { "Tasmia", 0.0 },
  { "Tasmin", 0.0 },
  { "Tasmine", 0.0 },
  { "Tasneem", 0.0 },
  { "Tasnia", 0.0 },
  { "Tasnim", 0.018050542 },
  { "Tason", 1.0 },
  { "Tasos", 1.0 },
  { "Tassa", 0.0 },
  { "Tassia", 0.0 },
  { "Tassie", 0.0 },
  { "Tassy", 0.0 },
  { "Tasya", 0.0 },
  { "Tata", 0.0 },
  { "Tatania", 0.0 },
  { "Tatanisha", 0.0 },
  { "Tatanishia", 0.0 },
  { "Tatanya", 0.0 },
  { "Tatasha", 0.0 },
  { "Tatayana", 0.0 },
  { "Tatayanna", 0.0 },
  { "Tate", 0.944867 },
  { "Tateana", 0.0 },
  { "Tateanna", 0.0 },
  { "Tatelyn", 0.0 },
  { "Tatelynn", 0.0 },
  { "Tatem", 0.41749504 },
  { "Taten", 1.0 },
  { "Tateum", 0.5192308 },
  { "Tatevik", 0.0 },
  { "Tateyana", 0.0 },
  { "Tathan", 1.0 },
  { "Tathiana", 0.0 },
  { "Tatia", 0.0 },
  { "Tatiana", 0.0010671708 },
  { "Tatianah", 0.0 },
  { "Tatiania", 0.0 },
  { "Tatianna", 0.0 },
  { "Tatiannah", 0.0 },
  { "Tatianya", 0.0 },
  { "Tatiara", 0.0 },
  { "Tatiauna", 0.0 },
  { "Tatiayana", 0.0 },
  { "Tatijana", 0.0 },
  { "Tatina", 0.0 },
  { "Tationa", 0.0 },
  { "Tationna", 0.0 },
  { "Tatisha", 0.0 },
  { "Tatiyana", 0.0 },
  { "Tatiyanah", 0.0 },
  { "Tatiyanna", 0.0 },
  { "Tatiyona", 0.0 },
  { "Tatiyonna", 0.0 },
  { "Tatjana", 0.0 },
  { "Tatom", 0.0 },
  { "Taton", 1.0 },
  { "Tatrina", 0.0 },
  { "Tatsiana", 0.0 },
  { "Tatsu", 1.0 },
  { "Tatsue", 0.0 },
  { "Tatsuki", 1.0 },
  { "Tatsuko", 0.0 },
  { "Tatsumi", 1.0 },
  { "Tatsuo", 1.0 },
  { "Tatsuro", 1.0 },
  { "Tatsuya", 1.0 },
  { "Tattiana", 0.0 },
  { "Tattianna", 0.0 },
  { "Tatton", 1.0 },
  { "Tattyana", 0.0 },
  { "Tatum", 0.17906956 },
  { "Tatumn", 0.016447369 },
  { "Tatyana", 0.0 },
  { "Tatyanah", 0.0 },
  { "Tatyania", 0.0 },
  { "Tatyanna", 0.0 },
  { "Tatyiana", 0.0 },
  { "Tatym", 0.022222223 },
  { "Tatyona", 0.0 },
  { "Tatyonna", 0.0 },
  { "Tau", 1.0 },
  { "Tauheed", 1.0 },
  { "Tauheedah", 0.0 },
  { "Tauhid", 1.0 },
  { "Tauja", 0.0 },
  { "Taumi", 0.0 },
  { "Tauna", 0.0 },
  { "Taunda", 0.0 },
  { "Taundra", 0.0 },
  { "Tauni", 0.0 },
  { "Taunia", 0.0 },
  { "Taunie", 0.0 },
  { "Taunja", 0.0 },
  { "Tauno", 1.0 },
  { "Taunya", 0.0 },
  { "Taura", 0.0 },
  { "Taurance", 1.0 },
  { "Tauras", 1.0 },
  { "Taurasi", 0.0 },
  { "Taurean", 0.97937673 },
  { "Tauren", 0.8947368 },
  { "Taurence", 1.0 },
  { "Taureon", 1.0 },
  { "Tauri", 0.0 },
  { "Taurian", 1.0 },
  { "Taurice", 1.0 },
  { "Taurie", 0.0 },
  { "Tauriel", 0.0 },
  { "Taurin", 1.0 },
  { "Taurino", 1.0 },
  { "Tauris", 1.0 },
  { "Taurius", 1.0 },
  { "Taurus", 0.95723015 },
  { "Tauryn", 0.0 },
  { "Tausha", 0.0 },
  { "Taushia", 0.0 },
  { "Tausif", 1.0 },
  { "Tautiana", 0.0 },
  { "Tava", 0.0 },
  { "Tavain", 1.0 },
  { "Tavalas", 1.0 },
  { "Tavan", 1.0 },
  { "Tavania", 0.0 },
  { "Tavanna", 0.0 },
  { "Tavante", 1.0 },
  { "Tavar", 1.0 },
  { "Tavara", 0.0 },
  { "Tavaras", 1.0 },
  { "Tavare", 1.0 },
  { "Tavares", 0.9919185 },
  { "Tavarez", 1.0 },
  { "Tavari", 1.0 },
  { "Tavaria", 0.0 },
  { "Tavarian", 1.0 },
  { "Tavario", 1.0 },
  { "Tavarion", 1.0 },
  { "Tavarious", 1.0 },
  { "Tavaris", 0.99746084 },
  { "Tavarius", 1.0 },
  { "Tavarous", 1.0 },
  { "Tavarres", 1.0 },
  { "Tavarris", 1.0 },
  { "Tavarus", 1.0 },
  { "Tavaughn", 1.0 },
  { "Tavaya", 0.0 },
  { "Taveah", 0.0 },
  { "Tavean", 1.0 },
  { "Taveion", 1.0 },
  { "Taven", 0.9680706 },
  { "Taveon", 1.0 },
  { "Tavera", 0.0 },
  { "Taveres", 1.0 },
  { "Tavern", 1.0 },
  { "Tavery", 0.0 },
  { "Taves", 1.0 },
  { "Tavette", 0.0 },
  { "Tavey", 0.0 },
  { "Taveyon", 1.0 },
  { "Tavi", 0.34054053 },
  { "Tavia", 0.0 },
  { "Taviah", 0.0 },
  { "Tavian", 1.0 },
  { "Taviana", 0.0 },
  { "Tavianna", 0.0 },
  { "Tavias", 1.0 },
  { "Tavie", 0.0 },
  { "Tavien", 1.0 },
  { "Tavier", 1.0 },
  { "Tavifa", 0.0 },
  { "Tavin", 0.9863582 },
  { "Tavio", 1.0 },
  { "Tavion", 0.99514925 },
  { "Taviona", 0.0 },
  { "Tavione", 1.0 },
  { "Tavionna", 0.0 },
  { "Tavious", 1.0 },
  { "Tavis", 0.9971844 },
  { "Tavish", 0.9916388 },
  { "Tavishi", 0.0 },
  { "Tavist", 1.0 },
  { "Tavita", 0.78881985 },
  { "Tavius", 1.0 },
  { "Taviyon", 1.0 },
  { "Tavleen", 0.0 },
  { "Tavo", 1.0 },
  { "Tavon", 0.97977704 },
  { "Tavona", 0.0 },
  { "Tavonda", 0.0 },
  { "Tavone", 1.0 },
  { "Tavonn", 1.0 },
  { "Tavonna", 0.0 },
  { "Tavonne", 0.7619048 },
  { "Tavonta", 1.0 },
  { "Tavontae", 1.0 },
  { "Tavonte", 1.0 },
  { "Tavonya", 0.0 },
  { "Tavores", 1.0 },
  { "Tavorian", 1.0 },
  { "Tavoris", 1.0 },
  { "Tavorris", 1.0 },
  { "Tavy", 0.0 },
  { "Tavyn", 0.9240506 },
  { "Tavyon", 1.0 },
  { "Tawain", 1.0 },
  { "Tawaina", 0.0 },
  { "Tawan", 0.84771574 },
  { "Tawana", 0.0 },
  { "Tawanda", 0.0 },
  { "Tawania", 0.0 },
  { "Tawanica", 0.0 },
  { "Tawann", 0.0 },
  { "Tawanna", 0.0 },
  { "Tawanne", 0.0 },
  { "Tawanya", 0.0 },
  { "Tawasha", 0.0 },
  { "Tawatha", 0.0 },
  { "Tawauna", 0.0 },
  { "Tawayna", 0.0 },
  { "Tawayne", 1.0 },
  { "Tawfeeq", 1.0 },
  { "Tawfik", 1.0 },
  { "Tawfiq", 1.0 },
  { "Tawheed", 1.0 },
  { "Tawhid", 1.0 },
  { "Tawiana", 0.0 },
  { "Tawn", 0.105960265 },
  { "Tawna", 0.0 },
  { "Tawne", 0.0 },
  { "Tawnee", 0.0 },
  { "Tawney", 0.0 },
  { "Tawni", 0.0 },
  { "Tawnia", 0.0 },
  { "Tawnie", 0.0 },
  { "Tawnja", 0.0 },
  { "Tawnjai", 0.0 },
  { "Tawnni", 0.0 },
  { "Tawny", 0.0 },
  { "Tawnya", 0.0 },
  { "Tawon", 1.0 },
  { "Tawona", 0.0 },
  { "Tawonda", 0.0 },
  { "Tawonna", 0.0 },
  { "Tawsha", 0.0 },
  { "Tawsif", 1.0 },
  { "Tawyna", 0.0 },
  { "Tay", 0.75961536 },
  { "Taya", 0.0 },
  { "Tayah", 0.0 },
  { "Tayana", 0.0 },
  { "Tayani", 0.0 },
  { "Tayanna", 0.0 },
  { "Tayari", 0.84705883 },
  { "Tayber", 1.0 },
  { "Taybor", 1.0 },
  { "Taybree", 0.0 },
  { "Tayce", 1.0 },
  { "Taycee", 0.0 },
  { "Taycen", 1.0 },
  { "Taycie", 0.0 },
  { "Taydan", 1.0 },
  { "Tayde", 0.15625 },
  { "Taydem", 0.18604651 },
  { "Tayden", 0.9264345 },
  { "Taydin", 1.0 },
  { "Taydon", 1.0 },
  { "Taydra", 0.0 },
  { "Taydum", 1.0 },
  { "Taye", 0.92416227 },
  { "Tayea", 0.0 },
  { "Tayeko", 0.0 },
  { "Tayelor", 0.0 },
  { "Tayem", 1.0 },
  { "Tayen", 0.35643566 },
  { "Tayesha", 0.0 },
  { "Tayeshawn", 1.0 },
  { "Taygan", 0.11627907 },
  { "Taygen", 0.43806645 },
  { "Tayha", 0.0 },
  { "Tayhlor", 0.0 },
  { "Tayia", 0.0 },
  { "Tayiah", 0.0 },
  { "Tayisha", 0.0 },
  { "Tayja", 0.0 },
  { "Tayjah", 0.0 },
  { "Tayjon", 1.0 },
  { "Tayjuan", 1.0 },
  { "Tayla", 0.0 },
  { "Taylah", 0.0 },
  { "Taylan", 0.70043576 },
  { "Taylani", 0.0 },
  { "Taylar", 0.011079196 },
  { "Taylea", 0.0 },
  { "Tayleah", 0.0 },
  { "Taylee", 0.0 },
  { "Tayleigh", 0.0 },
  { "Taylen", 0.5870672 },
  { "Taylene", 0.0 },
  { "Taylenn", 0.0 },
  { "Tayler", 0.18220116 },
  { "Tayley", 0.0 },
  { "Tayli", 0.0 },
  { "Taylia", 0.0 },
  { "Tayliana", 0.0 },
  { "Taylianna", 0.0 },
  { "Taylie", 0.0 },
  { "Taylin", 0.17072111 },
  { "Taylinn", 0.0 },
  { "Taylir", 0.0 },
  { "Tayllor", 0.0 },
  { "Tayln", 0.0 },
  { "Taylon", 0.8517469 },
  { "Taylor", 0.25744057 },
  { "Taylorann", 0.0 },
  { "Tayloranne", 0.0 },
  { "Taylore", 0.0 },
  { "Taylorgrace", 0.0 },
  { "Taylorjames", 1.0 },
  { "Taylorjo", 0.0 },
  { "Taylorlynn", 0.0 },
  { "Taylormarie", 0.0 },
  { "Taylorrae", 0.0 },
  { "Taylorrose", 0.0 },
  { "Taylour", 0.058467742 },
  { "Taylr", 0.0 },
  { "Taylre", 0.0 },
  { "Taylur", 0.0 },
  { "Taylyn", 0.025796661 },
  { "Taylynn", 0.0 },
  { "Taym", 1.0 },
  { "Tayma", 0.0 },
  { "Taymar", 0.5675676 },
  { "Taymon", 1.0 },
  { "Taymour", 1.0 },
  { "Taymur", 1.0 },
  { "Tayna", 0.0 },
  { "Tayne", 0.6875 },
  { "Taynia", 0.0 },
  { "Tayo", 1.0 },
  { "Tayon", 0.9876847 },
  { "Tayona", 0.0 },
  { "Tayonia", 0.0 },
  { "Tayonna", 0.0 },
  { "Tayquan", 1.0 },
  { "Tayra", 0.0 },
  { "Tayron", 1.0 },
  { "Tays", 0.0 },
  { "Taysean", 1.0 },
  { "Tayseer", 1.0 },
  { "Taysen", 1.0 },
  { "Taysha", 0.0 },
  { "Tayshaun", 0.99187654 },
  { "Tayshawn", 1.0 },
  { "Tayshawna", 0.0 },
  { "Tayshia", 0.0 },
  { "Tayshon", 1.0 },
  { "Tayshun", 1.0 },
  { "Taysia", 0.0 },
  { "Taysir", 1.0 },
  { "Taysom", 1.0 },
  { "Tayson", 1.0 },
  { "Tayt", 1.0 },
  { "Tayte", 0.78937006 },
  { "Taytem", 0.08163265 },
  { "Tayten", 0.86548674 },
  { "Taytiana", 0.0 },
  { "Tayton", 0.97780126 },
  { "Taytum", 0.1280592 },
  { "Tayva", 0.0 },
  { "Tayvan", 1.0 },
  { "Tayveion", 1.0 },
  { "Tayven", 0.93284935 },
  { "Tayveon", 1.0 },
  { "Tayvia", 0.0 },
  { "Tayvian", 1.0 },
  { "Tayvianna", 0.0 },
  { "Tayvien", 1.0 },
  { "Tayvin", 0.94736844 },
  { "Tayvion", 1.0 },
  { "Tayviona", 0.0 },
  { "Tayvione", 1.0 },
  { "Tayvionna", 0.0 },
  { "Tayvon", 1.0 },
  { "Tayvone", 1.0 },
  { "Tayvonne", 1.0 },
  { "Tayyab", 1.0 },
  { "Tayyaba", 0.0 },
  { "Tayyibah", 0.0 },
  { "Tayzen", 1.0 },
  { "Tayzha", 0.0 },
  { "Tayzia", 0.0 },
  { "Tayzlee", 0.0 },
  { "Taz", 0.975 },
  { "Tazanna", 0.0 },
  { "Tazaria", 0.0 },
  { "Tazariah", 0.0 },
  { "Tazaya", 0.0 },
  { "Taze", 1.0 },
  { "Tazeem", 1.0 },
  { "Tazewell", 1.0 },
  { "Tazhane", 0.0 },
  { "Tazia", 0.0 },
  { "Taziah", 0.0 },
  { "Tazio", 1.0 },
  { "Tazion", 1.0 },
  { "Taziya", 0.0 },
  { "Taziyah", 0.10344828 },
  { "Tazkia", 0.0 },
  { "Tazlyn", 0.0 },
  { "Tazman", 1.0 },
  { "Tazmeen", 0.0 },
  { "Tazmin", 0.2173913 },
  { "Tazuko", 0.0 },
  { "Tazz", 1.0 },
  { "Tc", 1.0 },
  { "Tchalla", 1.0 },
  { "Tchanavian", 0.0 },
  { "Te", 0.0 },
  { "Tea", 0.0 },
  { "Teadora", 0.0 },
  { "Teadra", 0.0 },
  { "Teaera", 0.0 },
  { "Teaerra", 0.0 },
  { "Teag", 1.0 },
  { "Teagan", 0.19857226 },
  { "Teagann", 0.0 },
  { "Teage", 1.0 },
  { "Teagen", 0.49628407 },
  { "Teaghan", 0.0748538 },
  { "Teaghen", 0.0 },
  { "Teagin", 0.36585367 },
  { "Teagon", 0.90839696 },
  { "Teague", 0.9782609 },
  { "Teagun", 1.0 },
  { "Teagyn", 0.10778443 },
  { "Teah", 0.0 },
  { "Teahna", 0.0 },
  { "Teaira", 0.0 },
  { "Teairah", 0.0 },
  { "Teaire", 0.0 },
  { "Teairra", 0.0 },
  { "Teaisha", 0.0 },
  { "Teaja", 0.0 },
  { "Teajah", 0.0 },
  { "Teak", 1.0 },
  { "Teaka", 0.0 },
  { "Teal", 0.123774506 },
  { "Teala", 0.0 },
  { "Teale", 0.0 },
  { "Teana", 0.0 },
  { "Teancum", 1.0 },
  { "Teandra", 0.0 },
  { "Teandre", 1.0 },
  { "Teandrea", 0.0 },
  { "Teandria", 0.0 },
  { "Teangela", 0.0 },
  { "Teani", 0.0 },
  { "Teanisha", 0.0 },
  { "Teann", 0.0 },
  { "Teanna", 0.0 },
  { "Teaona", 0.0 },
  { "Teaonna", 0.0 },
  { "Tear", 0.0 },
  { "Teara", 0.0 },
  { "Tearah", 0.0 },
  { "Tearesa", 0.0 },
  { "Tearia", 0.0 },
  { "Tearle", 1.0 },
  { "Tearney", 0.0 },
  { "Tearra", 0.0 },
  { "Tearria", 0.0 },
  { "Tearsa", 0.0 },
  { "Tearza", 0.0 },
  { "Teasa", 0.0 },
  { "Teasha", 0.0 },
  { "Teashia", 0.0 },
  { "Teasia", 0.0 },
  { "Teather", 0.0 },
  { "Teauna", 0.0 },
  { "Teaundra", 0.0 },
  { "Teaunna", 0.0 },
  { "Teaya", 0.0 },
  { "Tecia", 0.0 },
  { "Teckla", 0.0 },
  { "Tecla", 0.0 },
  { "Teco", 1.0 },
  { "Tecoa", 0.0 },
  { "Tecola", 0.0 },
  { "Tecora", 0.0 },
  { "Tecumseh", 1.0 },
  { "Ted", 0.99648106 },
  { "Tedarius", 1.0 },
  { "Tedd", 1.0 },
  { "Teddi", 0.0 },
  { "Teddie", 0.60906756 },
  { "Teddra", 0.0 },
  { "Teddrick", 1.0 },
  { "Teddy", 0.95679086 },
  { "Teddye", 0.0 },
  { "Tederick", 1.0 },
  { "Tedford", 1.0 },
  { "Tedi", 0.0 },
  { "Tedman", 1.0 },
  { "Tedra", 0.0 },
  { "Tedric", 1.0 },
  { "Tedrick", 1.0 },
  { "Tedrina", 0.0 },
  { "Tedros", 1.0 },
  { "Tee", 0.8830275 },
  { "Teea", 0.0 },
  { "Teeana", 0.0 },
  { "Teeanna", 0.0 },
  { "Teeara", 0.0 },
  { "Teegan", 0.48715952 },
  { "Teegen", 1.0 },
  { "Teeghan", 0.0 },
  { "Teegun", 1.0 },
  { "Teejay", 1.0 },
  { "Teeka", 0.0 },
  { "Teel", 0.0 },
  { "Teela", 0.0 },
  { "Teena", 0.0 },
  { "Teenamarie", 0.0 },
  { "Teenia", 0.0 },
  { "Teenie", 0.0 },
  { "Teera", 0.0 },
  { "Teerica", 0.0 },
  { "Teerra", 0.0 },
  { "Teesa", 0.0 },
  { "Teesha", 0.0 },
  { "Teeya", 0.0 },
  { "Teffani", 0.0 },
  { "Teffanie", 0.0 },
  { "Teffany", 0.0 },
  { "Tefta", 0.0 },
  { "Tegan", 0.27100495 },
  { "Tegen", 0.8648649 },
  { "Tegh", 1.0 },
  { "Teghan", 0.14454277 },
  { "Teghveer", 1.0 },
  { "Tegon", 1.0 },
  { "Tegra", 0.0 },
  { "Teguila", 0.0 },
  { "Teha", 0.0 },
  { "Tehana", 0.0 },
  { "Tehani", 0.0 },
  { "Tehesha", 0.0 },
  { "Tehila", 0.0 },
  { "Tehilah", 0.0 },
  { "Tehilla", 0.0 },
  { "Tehillah", 0.0 },
  { "Tehra", 0.0 },
  { "Tehran", 0.9596774 },
  { "Tehreem", 0.0 },
  { "Tehron", 1.0 },
  { "Tehya", 0.0 },
  { "Tei", 1.0 },
  { "Teia", 0.0 },
  { "Teiah", 0.0 },
  { "Teiana", 0.0 },
  { "Teianna", 0.0 },
  { "Teiara", 0.0 },
  { "Teiarra", 0.0 },
  { "Teidra", 0.0 },
  { "Teig", 1.0 },
  { "Teigan", 0.36135265 },
  { "Teige", 1.0 },
  { "Teigen", 0.59103644 },
  { "Teighan", 0.05263158 },
  { "Teighlor", 0.0 },
  { "Teija", 0.0 },
  { "Teijah", 0.0 },
  { "Teika", 0.0 },
  { "Teila", 0.0 },
  { "Teilor", 0.0 },
  { "Tein", 1.0 },
  { "Teina", 0.0 },
  { "Teion", 1.0 },
  { "Teiona", 0.0 },
  { "Teionna", 0.0 },
  { "Teira", 0.0 },
  { "Teirney", 0.0 },
  { "Teirra", 0.0 },
  { "Teisa", 0.0 },
  { "Teisha", 0.0 },
  { "Teiya", 0.0 },
  { "Tej", 1.0 },
  { "Teja", 0.113354035 },
  { "Tejah", 0.0 },
  { "Tejal", 0.0 },
  { "Tejan", 1.0 },
  { "Tejas", 1.0 },
  { "Tejash", 1.0 },
  { "Tejasvi", 0.0 },
  { "Tejaswini", 0.0 },
  { "Tejaun", 1.0 },
  { "Tejay", 1.0 },
  { "Tejean", 1.0 },
  { "Tejon", 1.0 },
  { "Tejuan", 1.0 },
  { "Tejuana", 0.0 },
  { "Tejus", 1.0 },
  { "Tejveer", 1.0 },
  { "Teka", 0.0 },
  { "Tekara", 0.0 },
  { "Tekayla", 0.0 },
  { "Tekeema", 0.0 },
  { "Tekeira", 0.0 },
  { "Tekeisha", 0.0 },
  { "Tekela", 0.0 },
  { "Tekelia", 0.0 },
  { "Tekenya", 0.0 },
  { "Tekera", 0.0 },
  { "Tekeria", 0.0 },
  { "Tekesha", 0.0 },
  { "Tekeshia", 0.0 },
  { "Teketa", 0.0 },
  { "Tekeya", 0.0 },
  { "Tekeyah", 0.0 },
  { "Tekeyla", 0.0 },
  { "Tekhi", 1.0 },
  { "Tekia", 0.0 },
  { "Tekila", 0.0 },
  { "Tekira", 0.0 },
  { "Tekisha", 0.0 },
  { "Tekita", 0.0 },
  { "Tekiya", 0.0 },
  { "Tekiyah", 0.0 },
  { "Tekla", 0.0 },
  { "Teko", 1.0 },
  { "Tekoa", 0.32735425 },
  { "Tekoah", 0.0 },
  { "Tekulve", 1.0 },
  { "Tekyra", 0.0 },
  { "Tel", 1.0 },
  { "Tela", 0.010130246 },
  { "Telah", 0.0 },
  { "Telana", 0.0 },
  { "Telaya", 0.0 },
  { "Telaysia", 0.0 },
  { "Teleah", 0.0 },
  { "Telecia", 0.0 },
  { "Telena", 0.0 },
  { "Telesa", 0.0 },
  { "Telesfor", 1.0 },
  { "Telesfora", 0.0 },
  { "Telesforo", 1.0 },
  { "Telesha", 0.0 },
  { "Teleshia", 0.0 },
  { "Telesia", 0.0 },
  { "Telesphore", 1.0 },
  { "Teletha", 0.0 },
  { "Telethia", 0.0 },
  { "Telford", 1.0 },
  { "Telia", 0.0 },
  { "Teliah", 0.0 },
  { "Telica", 0.0 },
  { "Telicia", 0.0 },
  { "Teliea", 0.0 },
  { "Telina", 0.0 },
  { "Telio", 1.0 },
  { "Telisa", 0.0 },
  { "Telise", 0.0 },
  { "Telisha", 0.0 },
  { "Telishia", 0.0 },
  { "Telissa", 0.0 },
  { "Telitha", 0.0 },
  { "Teliyah", 0.0 },
  { "Tell", 1.0 },
  { "Tella", 0.0 },
  { "Tellas", 1.0 },
  { "Teller", 1.0 },
  { "Telley", 1.0 },
  { "Telli", 0.0 },
  { "Tellie", 0.0877193 },
  { "Tellis", 1.0 },
  { "Telly", 0.9480684 },
  { "Tellys", 1.0 },
  { "Telma", 0.0 },
  { "Telor", 0.0 },
  { "Telsa", 0.0 },
  { "Telsha", 0.0 },
  { "Telvin", 1.0 },
  { "Telvis", 1.0 },
  { "Tema", 0.0 },
  { "Temaka", 0.0 },
  { "Teman", 1.0 },
  { "Temani", 0.5 },
  { "Temar", 1.0 },
  { "Temara", 0.0 },
  { "Temarcus", 1.0 },
  { "Temari", 0.073529415 },
  { "Temaria", 0.0 },
  { "Temarion", 1.0 },
  { "Tember", 0.0 },
  { "Temeca", 0.0 },
  { "Temecia", 0.0 },
  { "Temecka", 0.0 },
  { "Temeeka", 0.0 },
  { "Temeika", 0.0 },
  { "Temeka", 0.0035236082 },
  { "Temekia", 0.0 },
  { "Temeko", 0.0 },
  { "Temera", 0.0 },
  { "Temesa", 0.0 },
  { "Temesgen", 1.0 },
  { "Temesha", 0.0 },
  { "Temeshia", 0.0 },
  { "Temetria", 0.0 },
  { "Temetrius", 0.0 },
  { "Temi", 0.0 },
  { "Temia", 0.0 },
  { "Temica", 0.0 },
  { "Temicka", 0.0 },
  { "Temidayo", 0.7457627 },
  { "Temika", 0.0 },
  { "Temikia", 0.0 },
  { "Temiko", 0.0 },
  { "Temilade", 0.0 },
  { "Temilayo", 0.0 },
  { "Temilola", 0.0 },
  { "Temiloluwa", 0.37586206 },
  { "Temima", 0.0 },
  { "Temira", 0.0 },
  { "Temisha", 0.0 },
  { "Temitayo", 0.6454545 },
  { "Temitope", 0.32960895 },
  { "Temkia", 0.0 },
  { "Temmy", 0.0 },
  { "Temon", 1.0 },
  { "Tempa", 0.0 },
  { "Tempe", 0.0 },
  { "Temperance", 0.0 },
  { "Temperence", 0.0 },
  { "Tempess", 0.0 },
  { "Tempesst", 0.0 },
  { "Tempest", 0.0 },
  { "Tempestt", 0.0 },
  { "Tempie", 0.0 },
  { "Temple", 0.35463825 },
  { "Templeton", 1.0 },
  { "Temprance", 0.0 },
  { "Temprence", 0.0 },
  { "Tempress", 0.0 },
  { "Temprince", 0.0 },
  { "Temprynce", 0.0 },
  { "Tempsett", 0.0 },
  { "Tempy", 0.0 },
  { "Temre", 0.0 },
  { "Temuge", 1.0 },
  { "Temujin", 1.0 },
  { "Temur", 1.0 },
  { "Temuujin", 1.0 },
  { "Temuulen", 1.0 },
  { "Temya", 0.0 },
  { "Tena", 0.0 },
  { "Tenae", 0.0 },
  { "Tenaj", 0.0 },
  { "Tenaja", 0.0 },
  { "Tenajah", 0.0 },
  { "Tenara", 0.0 },
  { "Tenasha", 0.0 },
  { "Tenasia", 0.0 },
  { "Tenay", 0.0 },
  { "Tenaya", 0.0 },
  { "Tenayah", 0.0 },
  { "Tench", 1.0 },
  { "Tenchi", 1.0 },
  { "Tency", 0.0 },
  { "Tendai", 1.0 },
  { "Tendra", 0.0 },
  { "Tendria", 0.0 },
  { "Tene", 0.0 },
  { "Tenea", 0.0 },
  { "Teneal", 0.0 },
  { "Teneasha", 0.0 },
  { "Teneca", 0.0 },
  { "Tenecia", 0.0 },
  { "Tenee", 0.0 },
  { "Teneeka", 0.0 },
  { "Teneesha", 0.0 },
  { "Teneeshia", 0.0 },
  { "Teneha", 0.0 },
  { "Teneia", 0.0 },
  { "Teneika", 0.0 },
  { "Teneil", 0.0 },
  { "Teneile", 0.0 },
  { "Teneille", 0.0 },
  { "Teneisha", 0.0 },
  { "Teneka", 0.0 },
  { "Teneke", 0.0 },
  { "Tenell", 0.0 },
  { "Tenelle", 0.0 },
  { "Tenequa", 0.0 },
  { "Tenesa", 0.0 },
  { "Tenesha", 0.0 },
  { "Teneshia", 0.0 },
  { "Tenesia", 0.0 },
  { "Tenessa", 0.0 },
  { "Teneya", 0.0 },
  { "Teng", 1.0 },
  { "Tengis", 1.0 },
  { "Teni", 0.0 },
  { "Tenia", 0.0 },
  { "Teniah", 0.0 },
  { "Tenica", 0.0 },
  { "Tenice", 0.0 },
  { "Tenicha", 0.0 },
  { "Tenicia", 0.0 },
  { "Tenicka", 0.0 },
  { "Tenie", 0.0 },
  { "Teniel", 0.0 },
  { "Tenielle", 0.0 },
  { "Teniesha", 0.0 },
  { "Tenijah", 0.0 },
  { "Tenika", 0.0 },
  { "Tenile", 0.0 },
  { "Tenille", 0.0 },
  { "Tenina", 0.0 },
  { "Teniola", 0.042735044 },
  { "Teniqua", 0.0 },
  { "Tenisa", 0.0 },
  { "Tenise", 0.0 },
  { "Tenisha", 0.0 },
  { "Tenishia", 0.0 },
  { "Tenita", 0.0 },
  { "Teniya", 0.0 },
  { "Teniyah", 0.0 },
  { "Tenlee", 0.0 },
  { "Tenleigh", 0.0 },
  { "Tenley", 0.0 },
  { "Tenli", 0.0 },
  { "Tenlie", 0.0 },
  { "Tenly", 0.0 },
  { "Tenna", 0.0 },
  { "Tennasyn", 0.0 },
  { "Tenneal", 0.0 },
  { "Tenneh", 0.0 },
  { "Tenneil", 0.0 },
  { "Tenneill", 0.0 },
  { "Tenneille", 0.0 },
  { "Tennell", 0.2777778 },
  { "Tennelle", 0.0 },
  { "Tenner", 0.0 },
  { "Tennesha", 0.0 },
  { "Tennesia", 0.0 },
  { "Tennessee", 0.22689076 },
  { "Tennia", 0.0 },
  { "Tennie", 0.006795422 },
  { "Tenniel", 0.0 },
  { "Tennielle", 0.0 },
  { "Tennile", 0.0 },
  { "Tennill", 0.0 },
  { "Tennille", 0.0068587107 },
  { "Tennillie", 0.0 },
  { "Tennis", 1.0 },
  { "Tennisha", 0.0 },
  { "Tennison", 0.7083333 },
  { "Tennley", 0.0 },
  { "Tenny", 0.44285715 },
  { "Tennyson", 0.7960993 },
  { "Tenoch", 1.0 },
  { "Tenor", 1.0 },
  { "Tenora", 0.0 },
  { "Tensie", 0.0 },
  { "Tenslee", 0.0 },
  { "Tensley", 0.0 },
  { "Tenuun", 1.0 },
  { "Teny", 0.0 },
  { "Tenya", 0.0 },
  { "Tenyah", 0.0 },
  { "Tenyce", 0.0 },
  { "Tenyia", 0.0 },
  { "Tenzin", 0.5694592 },
  { "Tenzing", 0.7866667 },
  { "Tenzley", 0.0 },
  { "Teo", 1.0 },
  { "Teodor", 1.0 },
  { "Teodora", 0.0 },
  { "Teodoro", 1.0 },
  { "Teodulo", 1.0 },
  { "Teofil", 1.0 },
  { "Teofila", 0.0 },
  { "Teofilo", 1.0 },
  { "Teola", 0.0 },
  { "Teoman", 1.0 },
  { "Teon", 0.98395723 },
  { "Teona", 0.0 },
  { "Teondra", 0.0 },
  { "Teondre", 1.0 },
  { "Teoni", 0.0 },
  { "Teonia", 0.0 },
  { "Teonna", 0.0 },
  { "Teonni", 0.0 },
  { "Teontae", 1.0 },
  { "Teonte", 1.0 },
  { "Teosha", 0.0 },
  { "Teoshia", 0.0 },
  { "Tepanga", 0.0 },
  { "Tephanie", 0.0 },
  { "Tephen", 1.0 },
  { "Teppei", 1.0 },
  { "Tequan", 1.0 },
  { "Tequana", 0.0 },
  { "Tequarius", 1.0 },
  { "Tequela", 0.0 },
  { "Tequella", 0.0 },
  { "Tequesta", 0.0 },
  { "Tequia", 0.0 },
  { "Tequila", 0.0 },
  { "Tequilia", 0.0 },
  { "Tequilla", 0.0 },
  { "Tequira", 0.0 },
  { "Tequisha", 0.0 },
  { "Tequita", 0.0 },
  { "Tequlia", 0.0 },
  { "Tequoia", 0.0 },
  { "Tera", 0.0 },
  { "Terae", 0.5 },
  { "Terah", 0.030583214 },
  { "Teraji", 0.0 },
  { "Teral", 1.0 },
  { "Terald", 1.0 },
  { "Teralee", 0.0 },
  { "Teralyn", 0.0 },
  { "Teralynn", 0.0 },
  { "Teran", 0.6574074 },
  { "Terance", 1.0 },
  { "Terani", 0.0 },
  { "Teranique", 0.0 },
  { "Terasa", 0.0 },
  { "Terase", 0.0 },
  { "Terasha", 0.0 },
  { "Teray", 1.0 },
  { "Tercel", 1.0 },
  { "Terdell", 1.0 },
  { "Tere", 0.015345269 },
  { "Terea", 0.0 },
  { "Tereance", 1.0 },
  { "Tereas", 0.0 },
  { "Tereasa", 0.0 },
  { "Terease", 0.0 },
  { "Tereatha", 0.0 },
  { "Tereca", 0.0 },
  { "Terece", 0.0 },
  { "Terecia", 0.0 },
  { "Terecita", 0.0 },
  { "Teree", 0.0 },
  { "Terek", 1.0 },
  { "Tereka", 0.0 },
  { "Terel", 1.0 },
  { "Terell", 0.9855128 },
  { "Terelle", 0.9859944 },
  { "Teremy", 1.0 },
  { "Teren", 0.6815643 },
  { "Terena", 0.0 },
  { "Terence", 0.9949881 },
  { "Terene", 0.0 },
  { "Terenia", 0.0 },
  { "Terenna", 0.0 },
  { "Tereon", 1.0 },
  { "Teres", 0.0 },
  { "Teresa", 0.0028109492 },
  { "Teresaann", 0.0 },
  { "Terese", 0.0 },
  { "Teresea", 0.0 },
  { "Teresha", 0.0 },
  { "Teresia", 0.0 },
  { "Teresina", 0.0 },
  { "Teresita", 0.0 },
  { "Tereska", 0.0 },
  { "Tereso", 1.0 },
  { "Teressa", 0.0 },
  { "Teressia", 0.0 },
  { "Teretha", 0.0 },
  { "Tereva", 0.0 },
  { "Terez", 0.7255639 },
  { "Tereza", 0.0 },
  { "Teri", 0.004851178 },
  { "Teria", 0.0 },
  { "Teriah", 0.0 },
  { "Teriana", 0.0 },
  { "Teriann", 0.0 },
  { "Terianna", 0.0 },
  { "Terianne", 0.0 },
  { "Teric", 1.0 },
  { "Terica", 0.0 },
  { "Terice", 0.0 },
  { "Terick", 1.0 },
  { "Tericka", 0.0 },
  { "Terico", 1.0 },
  { "Terie", 0.0 },
  { "Terik", 1.0 },
  { "Terika", 0.0 },
  { "Terilee", 0.0 },
  { "Terilyn", 0.0 },
  { "Terilynn", 0.0 },
  { "Terin", 0.34541985 },
  { "Terina", 0.0 },
  { "Terion", 1.0 },
  { "Teriona", 0.0 },
  { "Terionna", 0.0 },
  { "Teriq", 1.0 },
  { "Terique", 1.0 },
  { "Teris", 1.0 },
  { "Terisa", 0.0 },
  { "Terise", 0.0 },
  { "Terisha", 0.0 },
  { "Terissa", 0.0 },
  { "Terita", 0.0 },
  { "Terius", 1.0 },
  { "Teriya", 0.0 },
  { "Teriyah", 0.0 },
  { "Teriyana", 0.0 },
  { "Termain", 1.0 },
  { "Termaine", 0.9810219 },
  { "Termayne", 1.0 },
  { "Termell", 1.0 },
  { "Ternell", 1.0 },
  { "Ternesha", 0.0 },
  { "Ternisha", 0.0 },
  { "Teron", 1.0 },
  { "Teronda", 0.0 },
  { "Terone", 1.0 },
  { "Teronica", 0.0 },
  { "Terr", 1.0 },
  { "Terra", 0.00034034444 },
  { "Terrace", 0.8696711 },
  { "Terrae", 0.0 },
  { "Terrah", 0.0 },
  { "Terrail", 1.0 },
  { "Terrain", 1.0 },
  { "Terraine", 0.5 },
  { "Terral", 0.9669967 },
  { "Terrall", 1.0 },
  { "Terralyn", 0.0 },
  { "Terralynn", 0.0 },
  { "Terran", 0.7094914 },
  { "Terranc", 1.0 },
  { "Terrance", 0.9891413 },
  { "Terrane", 1.0 },
  { "Terraneisha", 0.0 },
  { "Terranesha", 0.0 },
  { "Terranisha", 0.0 },
  { "Terray", 1.0 },
  { "Terrayne", 0.0 },
  { "Terre", 0.10565635 },
  { "Terrea", 0.0 },
  { "Terreal", 1.0 },
  { "Terreance", 1.0 },
  { "Terree", 0.0 },
  { "Terreka", 0.0 },
  { "Terrel", 0.97276396 },
  { "Terrell", 0.956662 },
  { "Terrelle", 0.9435216 },
  { "Terren", 0.754717 },
  { "Terrena", 0.0 },
  { "Terrence", 0.992622 },
  { "Terrencia", 0.0 },
  { "Terreon", 1.0 },
  { "Terresa", 0.0 },
  { "Terresha", 0.0 },
  { "Terresia", 0.0 },
  { "Terressa", 0.0 },
  { "Terrez", 1.0 },
  { "Terri", 0.0062425463 },
  { "Terria", 0.0 },
  { "Terriah", 0.0 },
  { "Terrial", 1.0 },
  { "Terrian", 0.6868421 },
  { "Terriana", 0.0 },
  { "Terriance", 1.0 },
  { "Terriann", 0.0 },
  { "Terrianna", 0.0 },
  { "Terrianne", 0.0 },
  { "Terriauna", 0.0 },
  { "Terric", 1.0 },
  { "Terrica", 0.0 },
  { "Terrice", 0.5 },
  { "Terricia", 0.0 },
  { "Terrick", 1.0 },
  { "Terricka", 0.0 },
  { "Terrico", 1.0 },
  { "Terrie", 0.016859537 },
  { "Terriek", 1.0 },
  { "Terriel", 0.8648649 },
  { "Terriell", 1.0 },
  { "Terrielle", 0.0 },
  { "Terrijo", 0.0 },
  { "Terrik", 1.0 },
  { "Terrika", 0.0 },
  { "Terril", 0.8605042 },
  { "Terrilee", 0.0 },
  { "Terrill", 0.8569425 },
  { "Terrilyn", 0.0 },
  { "Terrilynn", 0.0 },
  { "Terrilynne", 0.0 },
  { "Terrin", 0.5812965 },
  { "Terrina", 0.0 },
  { "Terrineka", 0.0 },
  { "Terrio", 1.0 },
  { "Terrion", 0.92730844 },
  { "Terriona", 0.0 },
  { "Terrionna", 0.0 },
  { "Terriq", 1.0 },
  { "Terrique", 1.0 },
  { "Terris", 0.85288066 },
  { "Terrisa", 0.0 },
  { "Terrisha", 0.0 },
  { "Terriss", 1.0 },
  { "Terrissa", 0.0 },
  { "Territa", 0.0 },
  { "Terrius", 1.0 },
  { "Terriya", 0.0 },
  { "Terriyah", 0.0 },
  { "Terriyana", 0.0 },
  { "Terriyon", 1.0 },
  { "Terrlyn", 0.0 },
  { "Terrod", 1.0 },
  { "Terrol", 1.0 },
  { "Terrolyn", 0.0 },
  { "Terron", 0.9866833 },
  { "Terronda", 0.0 },
  { "Terrone", 1.0 },
  { "Terry", 0.8133697 },
  { "Terryana", 0.0 },
  { "Terryann", 0.0 },
  { "Terryanna", 0.0 },
  { "Terrye", 0.0 },
  { "Terryion", 1.0 },
  { "Terryl", 0.5078929 },
  { "Terrylea", 0.0 },
  { "Terrylee", 0.25 },
  { "Terrylynn", 0.0 },
  { "Terryn", 0.26839826 },
  { "Terryon", 1.0 },
  { "Terryona", 0.0 },
  { "Terryonna", 0.0 },
  { "Tersa", 0.0 },
  { "Tersea", 0.0 },
  { "Tersia", 0.0 },
  { "Tertia", 0.0 },
  { "Tertius", 1.0 },
  { "Teruko", 0.0 },
  { "Teruo", 1.0 },
  { "Teruyo", 0.0 },
  { "Tervell", 1.0 },
  { "Tervin", 1.0 },
  { "Tervon", 1.0 },
  { "Tery", 0.7939394 },
  { "Terykah", 0.0 },
  { "Teryl", 0.25239924 },
  { "Teryn", 0.16764899 },
  { "Terynn", 0.0 },
  { "Teryon", 1.0 },
  { "Tes", 0.0 },
  { "Tesa", 0.0 },
  { "Tesean", 1.0 },
  { "Tesfa", 1.0 },
  { "Tesha", 0.0 },
  { "Teshana", 0.0 },
  { "Teshara", 0.0 },
  { "Teshaun", 1.0 },
  { "Teshauna", 0.0 },
  { "Teshawn", 0.9295393 },
  { "Teshawna", 0.0 },
  { "Teshayla", 0.0 },
  { "Tesheena", 0.0 },
  { "Tesheka", 0.0 },
  { "Teshena", 0.0 },
  { "Tesher", 1.0 },
  { "Teshia", 0.0 },
  { "Teshika", 0.0 },
  { "Teshima", 0.0 },
  { "Teshina", 0.0 },
  { "Teshira", 0.0 },
  { "Teshon", 1.0 },
  { "Tesia", 0.0 },
  { "Tesla", 0.029187247 },
  { "Tesley", 0.0 },
  { "Teslyn", 0.0 },
  { "Tesneem", 0.0 },
  { "Tess", 0.0 },
  { "Tessa", 0.0002839544 },
  { "Tessah", 0.0 },
  { "Tessalee", 0.0 },
  { "Tessalyn", 0.0 },
  { "Tessalynn", 0.0 },
  { "Tessanne", 0.0 },
  { "Tesse", 0.0 },
  { "Tessi", 0.0 },
  { "Tessia", 0.0 },
  { "Tessibel", 0.0 },
  { "Tessica", 0.0 },
  { "Tessie", 0.0010091836 },
  { "Tessla", 0.0 },
  { "Tessley", 0.0 },
  { "Tesslyn", 0.0 },
  { "Tesslynn", 0.0 },
  { "Tessy", 0.0 },
  { "Testimony", 0.0 },
  { "Tetra", 0.0 },
  { "Tetsu", 1.0 },
  { "Tetsuo", 1.0 },
  { "Tetsuro", 1.0 },
  { "Tetsuya", 1.0 },
  { "Teuila", 0.0 },
  { "Teuna", 0.0 },
  { "Teuta", 0.0 },
  { "Teva", 0.11851852 },
  { "Tevan", 1.0 },
  { "Tevante", 1.0 },
  { "Tevaris", 1.0 },
  { "Tevarus", 1.0 },
  { "Tevaughn", 1.0 },
  { "Tevell", 1.0 },
  { "Teven", 1.0 },
  { "Teveon", 1.0 },
  { "Tevez", 1.0 },
  { "Tevian", 1.0 },
  { "Tevin", 0.99462515 },
  { "Tevion", 1.0 },
  { "Tevis", 0.8834244 },
  { "Tevita", 1.0 },
  { "Tevon", 1.0 },
  { "Tevonte", 1.0 },
  { "Tevy", 0.0 },
  { "Tevye", 1.0 },
  { "Tevyn", 0.9800797 },
  { "Tewana", 0.0 },
  { "Tewanda", 0.0 },
  { "Tewanna", 0.0 },
  { "Tex", 0.99429756 },
  { "Texana", 0.0 },
  { "Texanna", 0.0 },
  { "Texas", 0.6247848 },
  { "Texie", 0.0 },
  { "Teya", 0.0 },
  { "Teyah", 0.0 },
  { "Teyahna", 0.0 },
  { "Teyana", 0.0 },
  { "Teyanah", 0.0 },
  { "Teyani", 0.0 },
  { "Teyanna", 0.0 },
  { "Teyden", 1.0 },
  { "Teygan", 0.35714287 },
  { "Teyha", 0.0 },
  { "Teyla", 0.0 },
  { "Teylar", 0.0 },
  { "Teyler", 0.0 },
  { "Teylor", 0.11658031 },
  { "Teyo", 1.0 },
  { "Teyon", 1.0 },
  { "Teyona", 0.0 },
  { "Teyonce", 0.0 },
  { "Teyonda", 0.0 },
  { "Teyonna", 0.0 },
  { "Teysean", 1.0 },
  { "Teysha", 0.0 },
  { "Teyton", 1.0 },
  { "Teyvon", 1.0 },
  { "Tezra", 0.0 },
  { "Tha", 0.54545456 },
  { "Thabit", 1.0 },
  { "Thabiti", 1.0 },
  { "Thach", 1.0 },
  { "Thacher", 1.0 },
  { "Thackery", 1.0 },
  { "Thad", 0.9993842 },
  { "Thada", 0.0 },
  { "Thadd", 1.0 },
  { "Thaddaeus", 1.0 },
  { "Thaddaus", 1.0 },
  { "Thaddea", 0.0 },
  { "Thaddeaus", 1.0 },
  { "Thaddeous", 1.0 },
  { "Thaddeus", 0.9986237 },
  { "Thaddis", 1.0 },
  { "Thaddius", 1.0 },
  { "Thaddues", 1.0 },
  { "Thaddus", 1.0 },
  { "Thade", 1.0 },
  { "Thaden", 1.0 },
  { "Thadeo", 1.0 },
  { "Thadeous", 1.0 },
  { "Thadeus", 1.0 },
  { "Thadine", 0.0 },
  { "Thadis", 1.0 },
  { "Thadius", 1.0 },
  { "Thadus", 1.0 },
  { "Thaer", 1.0 },
  { "Thai", 0.856387 },
  { "Thaiden", 1.0 },
  { "Thaila", 0.0 },
  { "Thailan", 1.0 },
  { "Thailand", 0.91071427 },
  { "Thailee", 0.0 },
  { "Thailen", 1.0 },
  { "Thailer", 0.84615386 },
  { "Thailia", 0.0 },
  { "Thaily", 0.0 },
  { "Thailyn", 0.11764706 },
  { "Thailynn", 0.0 },
  { "Thain", 1.0 },
  { "Thaina", 0.0 },
  { "Thaine", 1.0 },
  { "Thair", 1.0 },
  { "Thais", 0.0 },
  { "Thaisa", 0.0 },
  { "Thaisha", 0.0 },
  { "Thaison", 1.0 },
  { "Thaissa", 0.0 },
  { "Thaiz", 0.0 },
  { "Thaj", 1.0 },
  { "Thala", 0.0 },
  { "Thalamus", 1.0 },
  { "Thalassa", 0.0 },
  { "Thaleia", 0.0 },
  { "Thalen", 1.0 },
  { "Thales", 1.0 },
  { "Thalia", 0.0 },
  { "Thaliah", 0.0 },
  { "Thaliana", 0.0 },
  { "Thalina", 0.0 },
  { "Thalita", 0.0 },
  { "Thaliya", 0.0 },
  { "Thaliyah", 0.0 },
  { "Thalma", 0.0 },
  { "Thalmus", 1.0 },
  { "Thalya", 0.0 },
  { "Thaman", 1.0 },
  { "Thamar", 0.055555556 },
  { "Thamara", 0.0 },
  { "Thamas", 1.0 },
  { "Thamer", 1.0 },
  { "Thames", 1.0 },
  { "Thamia", 0.0 },
  { "Thamos", 1.0 },
  { "Than", 1.0 },
  { "Thana", 0.0 },
  { "Thanasi", 1.0 },
  { "Thandi", 0.0 },
  { "Thandie", 0.0 },
  { "Thandiwe", 0.0 },
  { "Thane", 1.0 },
  { "Thang", 0.976 },
  { "Thanh", 0.6257916 },
  { "Thanhphong", 1.0 },
  { "Thanhtruc", 0.0 },
  { "Thania", 0.0 },
  { "Thaniel", 1.0 },
  { "Thanna", 0.0 },
  { "Thanos", 1.0 },
  { "Thanvi", 0.0 },
  { "Thanya", 0.0 },
  { "Thao", 0.15647922 },
  { "Thara", 0.0 },
  { "Tharan", 1.0 },
  { "Tharel", 1.0 },
  { "Tharen", 1.0 },
  { "Tharin", 1.0 },
  { "Tharon", 0.7402799 },
  { "Tharron", 1.0 },
  { "Tharun", 1.0 },
  { "Thary", 0.0 },
  { "Tharyn", 1.0 },
  { "Thasha", 0.0 },
  { "Thatcher", 0.99593645 },
  { "Thatiana", 0.0 },
  { "Thavy", 0.0 },
  { "Thawann", 0.0 },
  { "Thawng", 1.0 },
  { "Thaxter", 1.0 },
  { "Thaxton", 1.0 },
  { "Thaya", 0.0 },
  { "Thayden", 1.0 },
  { "Thayer", 0.9433962 },
  { "Thayla", 0.0 },
  { "Thayle", 0.0 },
  { "Thayli", 0.0 },
  { "Thaylia", 0.0 },
  { "Thayna", 0.0 },
  { "Thayne", 1.0 },
  { "Thays", 0.0 },
  { "The", 1.0 },
  { "Thea", 0.00046935136 },
  { "Theada", 0.0 },
  { "Theado", 1.0 },
  { "Theador", 1.0 },
  { "Theadora", 0.0 },
  { "Theadore", 1.0 },
  { "Theanna", 0.0 },
  { "Thearon", 1.0 },
  { "Thearsa", 0.0 },
  { "Thearthur", 1.0 },
  { "Theartis", 1.0 },
  { "Theary", 0.0 },
  { "Theatus", 0.0 },
  { "Theckla", 0.0 },
  { "Thecla", 0.0 },
  { "Theda", 0.0 },
  { "Thedford", 1.0 },
  { "Thedis", 0.0 },
  { "Thedora", 0.0 },
  { "Thedore", 1.0 },
  { "Thedosia", 0.0 },
  { "Thedra", 0.0 },
  { "Thedy", 0.0 },
  { "Theeran", 1.0 },
  { "Theia", 0.0 },
  { "Thekla", 0.0 },
  { "Thel", 1.0 },
  { "Thela", 0.0 },
  { "Thelbert", 1.0 },
  { "Thelda", 0.0 },
  { "Theldon", 1.0 },
  { "Thelia", 0.0 },
  { "Thelisa", 0.0 },
  { "Thell", 1.0 },
  { "Thella", 0.0 },
  { "Thelma", 0.005045373 },
  { "Thelman", 1.0 },
  { "Thelmar", 0.0 },
  { "Thelmer", 0.75132275 },
  { "Thelmon", 1.0 },
  { "Thelonious", 1.0 },
  { "Thelonius", 1.0 },
  { "Thelton", 1.0 },
  { "Thema", 0.0 },
  { "Themba", 1.0 },
  { "Thembi", 0.0 },
  { "Themis", 0.5 },
  { "Thena", 0.0 },
  { "Thenia", 0.0 },
  { "Theo", 0.7613941 },
  { "Theobald", 1.0 },
  { "Theoda", 0.33333334 },
  { "Theoden", 1.0 },
  { "Theodia", 1.0 },
  { "Theodie", 1.0 },
  { "Theodies", 1.0 },
  { "Theodis", 1.0 },
  { "Theodocia", 0.0 },
  { "Theodor", 1.0 },
  { "Theodora", 0.0049605183 },
  { "Theodore", 0.9961376 },
  { "Theodoric", 1.0 },
  { "Theodoro", 1.0 },
  { "Theodoros", 1.0 },
  { "Theodosia", 0.0 },
  { "Theodric", 1.0 },
  { "Theodus", 1.0 },
  { "Theofanis", 1.0 },
  { "Theofilos", 1.0 },
  { "Theola", 0.0 },
  { "Theolia", 0.0 },
  { "Theon", 0.9594595 },
  { "Theona", 0.0 },
  { "Theone", 0.015822785 },
  { "Theoni", 0.0 },
  { "Theonita", 0.0 },
  { "Theopa", 0.0 },
  { "Theopal", 0.0 },
  { "Theophane", 0.0 },
  { "Theophil", 1.0 },
  { "Theophila", 0.0 },
  { "Theophile", 1.0 },
  { "Theophilia", 0.0 },
  { "Theophilus", 1.0 },
  { "Theophus", 1.0 },
  { "Theoplis", 1.0 },
  { "Theopolis", 1.0 },
  { "Theora", 0.0 },
  { "Theordore", 1.0 },
  { "Theoren", 1.0 },
  { "Theoria", 0.0 },
  { "Theory", 0.36619717 },
  { "Theotis", 1.0 },
  { "Thera", 0.0 },
  { "Theral", 1.0 },
  { "Therald", 1.0 },
  { "Theran", 1.0 },
  { "Therasa", 0.0 },
  { "Therdore", 1.0 },
  { "Therea", 0.0 },
  { "Thereas", 0.0 },
  { "Thereasa", 0.0 },
  { "Therease", 0.0 },
  { "Theren", 1.0 },
  { "Theres", 0.0 },
  { "Theresa", 0.0030100665 },
  { "Theresaann", 0.0 },
  { "Theresamarie", 0.0 },
  { "Theresann", 0.0 },
  { "Therese", 0.0001428653 },
  { "Theresea", 0.0 },
  { "Theresia", 0.0 },
  { "Theresita", 0.0 },
  { "Theressa", 0.0 },
  { "Theressia", 0.0 },
  { "Theretha", 0.0 },
  { "Thereza", 0.0 },
  { "Theria", 0.0 },
  { "Therin", 1.0 },
  { "Therion", 1.0 },
  { "Therisa", 0.0 },
  { "Therma", 0.0 },
  { "Therman", 1.0 },
  { "Thermon", 1.0 },
  { "Thermond", 1.0 },
  { "Theron", 0.9974283 },
  { "Therone", 1.0 },
  { "Therrell", 1.0 },
  { "Therrin", 1.0 },
  { "Therron", 1.0 },
  { "Thersa", 0.0 },
  { "Therse", 0.0 },
  { "Thersea", 0.0 },
  { "Thersia", 0.0 },
  { "Theryn", 0.65217394 },
  { "Thesa", 0.0 },
  { "Theseus", 1.0 },
  { "Thesha", 0.0 },
  { "Thessalonia", 1.0 },
  { "Thessaly", 0.0 },
  { "Theta", 0.0 },
  { "Thetis", 0.0 },
  { "Theus", 1.0 },
  { "Thi", 0.13548388 },
  { "Thia", 0.0 },
  { "Thiago", 1.0 },
  { "Thian", 1.0 },
  { "Thiana", 0.0 },
  { "Thiara", 0.0 },
  { "Thibault", 1.0 },
  { "Thida", 0.0 },
  { "Thiego", 1.0 },
  { "Thiel", 1.0 },
  { "Thien", 0.9691817 },
  { "Thienan", 0.6329114 },
  { "Thienkim", 0.0 },
  { "Thienlong", 1.0 },
  { "Thierno", 1.0 },
  { "Thierry", 0.96275073 },
  { "Thijs", 1.0 },
  { "Thilda", 0.0 },
  { "Thilo", 1.0 },
  { "Thimothy", 1.0 },
  { "Thinh", 1.0 },
  { "Thira", 0.0 },
  { "Thirl", 1.0 },
  { "Thirston", 1.0 },
  { "Thirza", 0.0 },
  { "Thisbe", 0.0 },
  { "Thistle", 0.0 },
  { "Thiya", 0.0 },
  { "Thlema", 0.0 },
  { "Tho", 0.88461536 },
  { "Thoa", 0.0 },
  { "Thoams", 1.0 },
  { "Thobias", 1.0 },
  { "Thoburn", 1.0 },
  { "Thoedore", 1.0 },
  { "Thoeun", 1.0 },
  { "Thom", 1.0 },
  { "Thoma", 0.79591835 },
  { "Thomara", 0.0 },
  { "Thomas", 0.99633294 },
  { "Thomasa", 0.0 },
  { "Thomasena", 0.0 },
  { "Thomasene", 0.0 },
  { "Thomasenia", 0.0 },
  { "Thomasin", 0.0 },
  { "Thomasina", 0.0 },
  { "Thomasine", 0.0 },
  { "Thomasita", 0.0 },
  { "Thomasjames", 1.0 },
  { "Thomasjohn", 1.0 },
  { "Thomaslee", 1.0 },
  { "Thomasmichael", 1.0 },
  { "Thomason", 1.0 },
  { "Thomass", 1.0 },
  { "Thomes", 1.0 },
  { "Thomesa", 0.0 },
  { "Thomesha", 0.0 },
  { "Thomia", 0.0 },
  { "Thomias", 1.0 },
  { "Thomisha", 0.0 },
  { "Thommas", 1.0 },
  { "Thommy", 1.0 },
  { "Thomos", 1.0 },
  { "Thompson", 1.0 },
  { "Thoms", 1.0 },
  { "Thomson", 1.0 },
  { "Thon", 1.0 },
  { "Thonda", 0.0 },
  { "Thong", 1.0 },
  { "Thony", 1.0 },
  { "Thor", 1.0 },
  { "Thora", 0.0 },
  { "Thorald", 1.0 },
  { "Thoralf", 1.0 },
  { "Thoran", 1.0 },
  { "Thordis", 0.0 },
  { "Thoreau", 1.0 },
  { "Thoren", 1.0 },
  { "Thorian", 1.0 },
  { "Thorin", 1.0 },
  { "Thorine", 0.0 },
  { "Thoris", 0.0 },
  { "Thorn", 1.0 },
  { "Thorne", 1.0 },
  { "Thornell", 1.0 },
  { "Thornton", 1.0 },
  { "Thornwell", 1.0 },
  { "Thorpe", 1.0 },
  { "Thorsen", 1.0 },
  { "Thorson", 1.0 },
  { "Thorsten", 1.0 },
  { "Thorton", 1.0 },
  { "Thorval", 1.0 },
  { "Thorvald", 1.0 },
  { "Thorwald", 1.0 },
  { "Thoryn", 1.0 },
  { "Thos", 1.0 },
  { "Threasa", 0.0 },
  { "Threase", 0.0 },
  { "Thresa", 0.0 },
  { "Threse", 0.0 },
  { "Thresea", 0.0 },
  { "Thresia", 0.0 },
  { "Thressa", 0.0 },
  { "Thressia", 0.0 },
  { "Thressie", 0.0 },
  { "Thrisa", 0.0 },
  { "Thrisha", 0.0 },
  { "Thristan", 1.0 },
  { "Thristen", 1.0 },
  { "Thristian", 1.0 },
  { "Thu", 0.006134969 },
  { "Thuan", 0.9764151 },
  { "Thuhuong", 0.0 },
  { "Thula", 0.0 },
  { "Thunder", 1.0 },
  { "Thuong", 0.1590909 },
  { "Thuraya", 0.0 },
  { "Thurayya", 0.0 },
  { "Thurber", 1.0 },
  { "Thure", 1.0 },
  { "Thurgood", 1.0 },
  { "Thurl", 1.0 },
  { "Thurla", 0.0 },
  { "Thurley", 0.0 },
  { "Thurlo", 1.0 },
  { "Thurlow", 1.0 },
  { "Thurma", 0.0 },
  { "Thurman", 0.9996938 },
  { "Thurmon", 1.0 },
  { "Thurmond", 1.0 },
  { "Thurnell", 1.0 },
  { "Thursa", 0.0 },
  { "Thursie", 0.0 },
  { "Thurston", 1.0 },
  { "Thurza", 0.0 },
  { "Thusnelda", 0.0 },
  { "Thuthuy", 0.0 },
  { "Thuy", 0.015553523 },
  { "Thuytien", 0.0 },
  { "Thuytrang", 0.0 },
  { "Thuyvi", 0.0 },
  { "Thuyvy", 0.0 },
  { "Thy", 0.0 },
  { "Thyago", 1.0 },
  { "Thyais", 0.0 },
  { "Thyda", 0.0 },
  { "Thyler", 1.0 },
  { "Thyme", 0.0 },
  { "Thyra", 0.0 },
  { "Thyrome", 1.0 },
  { "Thyron", 1.0 },
  { "Thyrone", 1.0 },
  { "Thyrza", 0.0 },
  { "Thys", 1.0 },
  { "Ti", 0.82608694 },
  { "Tia", 0.0019626482 },
  { "Tiaa", 0.0 },
  { "Tiadra", 0.0 },
  { "Tiago", 1.0 },
  { "Tiah", 0.0 },
  { "Tiahja", 0.0 },
  { "Tiahna", 0.0 },
  { "Tiaira", 0.0 },
  { "Tiairra", 0.0 },
  { "Tiaisha", 0.0 },
  { "Tiaja", 0.0 },
  { "Tiajah", 0.0 },
  { "Tiajuana", 0.0 },
  { "Tiaka", 0.0 },
  { "Tiala", 0.0 },
  { "Tialisa", 0.0 },
  { "Tiam", 1.0 },
  { "Tiamara", 0.0 },
  { "Tiamarie", 0.0 },
  { "Tiamia", 0.0 },
  { "Tian", 0.59116024 },
  { "Tiana", 0.0003508225 },
  { "Tianah", 0.0 },
  { "Tianamarie", 0.0 },
  { "Tianca", 0.0 },
  { "Tianda", 0.0 },
  { "Tiandra", 0.0 },
  { "Tiandre", 1.0 },
  { "Tiandrea", 0.0 },
  { "Tiandria", 0.0 },
  { "Tiane", 0.0 },
  { "Tianee", 0.0 },
  { "Tiani", 0.0 },
  { "Tiania", 0.0 },
  { "Tianie", 0.0 },
  { "Tiann", 0.0 },
  { "Tianna", 0.0 },
  { "Tiannah", 0.0 },
  { "Tianne", 0.0 },
  { "Tianni", 0.0 },
  { "Tianny", 0.0 },
  { "Tiano", 1.0 },
  { "Tiant", 1.0 },
  { "Tiante", 0.23809524 },
  { "Tiany", 0.0 },
  { "Tianyi", 1.0 },
  { "Tiaona", 0.0 },
  { "Tiara", 0.0026402355 },
  { "Tiarah", 0.0 },
  { "Tiaraoluwa", 0.0 },
  { "Tiare", 0.0 },
  { "Tiari", 0.0 },
  { "Tiaria", 0.0 },
  { "Tiarnan", 1.0 },
  { "Tiarra", 0.0 },
  { "Tiarrah", 0.0 },
  { "Tiarre", 0.0 },
  { "Tiasha", 0.0 },
  { "Tiasia", 0.0 },
  { "Tiauna", 0.0 },
  { "Tiaundra", 0.0 },
  { "Tiaunna", 0.0 },
  { "Tiaura", 0.0 },
  { "Tiawan", 1.0 },
  { "Tiawana", 0.0 },
  { "Tiawanda", 0.0 },
  { "Tiawanna", 0.0 },
  { "Tiawna", 0.0 },
  { "Tiaya", 0.0 },
  { "Tiayana", 0.0 },
  { "Tiayanna", 0.0 },
  { "Tibby", 1.0 },
  { "Tiberias", 1.0 },
  { "Tiberius", 1.0 },
  { "Tibisay", 0.0 },
  { "Tibor", 1.0 },
  { "Tiburcia", 0.0 },
  { "Tiburcio", 1.0 },
  { "Tica", 0.0 },
  { "Ticara", 0.0 },
  { "Ticarra", 0.0 },
  { "Tice", 1.0 },
  { "Ticey", 0.0 },
  { "Ticha", 0.0 },
  { "Tichelle", 0.0 },
  { "Tichina", 0.0 },
  { "Ticia", 0.0 },
  { "Tico", 1.0 },
  { "Tida", 0.0 },
  { "Tidiane", 1.0 },
  { "Tidus", 1.0 },
  { "Tiea", 0.0 },
  { "Tieanna", 0.0 },
  { "Tieara", 0.0 },
  { "Tiearra", 0.0 },
  { "Tieasha", 0.0 },
  { "Tieesha", 0.0 },
  { "Tieg", 1.0 },
  { "Tiegan", 0.28373703 },
  { "Tiege", 1.0 },
  { "Tiegen", 1.0 },
  { "Tieisha", 0.0 },
  { "Tieka", 0.0 },
  { "Tiela", 0.0 },
  { "Tieler", 0.8464912 },
  { "Tielor", 1.0 },
  { "Tiembra", 0.0 },
  { "Tiemeyer", 1.0 },
  { "Tien", 0.6030025 },
  { "Tiena", 0.0 },
  { "Tienna", 0.0 },
  { "Tier", 0.0 },
  { "Tiera", 0.0014822134 },
  { "Tierah", 0.0 },
  { "Tieran", 0.78571427 },
  { "Tieranee", 0.0 },
  { "Tieraney", 0.0 },
  { "Tierani", 0.0 },
  { "Tierany", 0.0 },
  { "Tiere", 0.0 },
  { "Tiereney", 0.0 },
  { "Tiereny", 0.0 },
  { "Tieria", 0.0 },
  { "Tierica", 0.0 },
  { "Tiericka", 0.0 },
  { "Tierna", 0.0 },
  { "Tiernan", 0.8682336 },
  { "Tierney", 0.0062772925 },
  { "Tierni", 0.0 },
  { "Tierny", 0.0 },
  { "Tierra", 0.004749627 },
  { "Tierrah", 0.0 },
  { "Tierre", 0.7621951 },
  { "Tierria", 0.0 },
  { "Tiersa", 0.0 },
  { "Tierza", 0.0 },
  { "Tiesa", 0.0 },
  { "Tiese", 0.0 },
  { "Tiesha", 0.0 },
  { "Tieshia", 0.0 },
  { "Tieshka", 0.0 },
  { "Tieson", 1.0 },
  { "Tiessa", 0.0 },
  { "Tieysha", 0.0 },
  { "Tifa", 0.0 },
  { "Tifanee", 0.0 },
  { "Tifani", 0.0 },
  { "Tifanie", 0.0 },
  { "Tifanni", 0.0 },
  { "Tifanny", 0.0 },
  { "Tifany", 0.0 },
  { "Tifeoluwa", 1.0 },
  { "Tiferet", 0.0 },
  { "Tiffaine", 0.0 },
  { "Tiffancy", 0.0 },
  { "Tiffane", 0.0 },
  { "Tiffanee", 0.0 },
  { "Tiffaney", 0.0 },
  { "Tiffani", 0.0 },
  { "Tiffaniamber", 0.0 },
  { "Tiffanie", 0.0 },
  { "Tiffanique", 0.0 },
  { "Tiffannee", 0.0 },
  { "Tiffanni", 0.0 },
  { "Tiffannie", 0.0 },
  { "Tiffanny", 0.0 },
  { "Tiffany", 0.0035363913 },
  { "Tiffanyamber", 0.0 },
  { "Tiffanyann", 0.0 },
  { "Tiffanye", 0.0 },
  { "Tiffanymarie", 0.0 },
  { "Tiffay", 0.0 },
  { "Tiffeney", 0.0 },
  { "Tiffeny", 0.0 },
  { "Tifffany", 0.0 },
  { "Tiffiani", 0.0 },
  { "Tiffiany", 0.0 },
  { "Tiffin", 0.0 },
  { "Tiffinay", 0.0 },
  { "Tiffine", 0.0 },
  { "Tiffinee", 0.0 },
  { "Tiffiney", 0.0 },
  { "Tiffini", 0.0 },
  { "Tiffinie", 0.0 },
  { "Tiffiny", 0.0 },
  { "Tiffnay", 0.0 },
  { "Tiffney", 0.0 },
  { "Tiffni", 0.0 },
  { "Tiffnie", 0.0 },
  { "Tiffny", 0.0 },
  { "Tiffoni", 0.0 },
  { "Tiffonie", 0.0 },
  { "Tiffony", 0.0 },
  { "Tiffy", 0.0 },
  { "Tifini", 0.0 },
  { "Tifinie", 0.0 },
  { "Tifiny", 0.0 },
  { "Tifney", 0.0 },
  { "Tifni", 0.0 },
  { "Tifphanie", 0.0 },
  { "Tige", 1.0 },
  { "Tiger", 0.96428573 },
  { "Tigerlilly", 0.0 },
  { "Tigerlily", 0.0 },
  { "Tigh", 1.0 },
  { "Tighe", 1.0 },
  { "Tigra", 0.0 },
  { "Tigran", 1.0 },
  { "Tigre", 0.0 },
  { "Tihani", 0.0 },
  { "Tihanna", 0.0 },
  { "Tihara", 0.0 },
  { "Tiheim", 1.0 },
  { "Tihesha", 0.0 },
  { "Tihisha", 0.0 },
  { "Tiina", 0.0 },
  { "Tiiu", 0.0 },
  { "Tija", 0.042016808 },
  { "Tijae", 0.0 },
  { "Tijah", 0.05319149 },
  { "Tijan", 1.0 },
  { "Tijana", 0.0 },
  { "Tijanae", 0.0 },
  { "Tijay", 0.54545456 },
  { "Tijera", 0.0 },
  { "Tijon", 1.0 },
  { "Tijuan", 1.0 },
  { "Tijuana", 0.0 },
  { "Tijuanna", 0.0 },
  { "Tijwana", 0.0 },
  { "Tika", 0.0 },
  { "Tikara", 0.0 },
  { "Tikayla", 0.0 },
  { "Tikeisha", 0.0 },
  { "Tikela", 0.0 },
  { "Tikera", 0.0 },
  { "Tikeria", 0.0 },
  { "Tikesha", 0.0 },
  { "Tikeshia", 0.0 },
  { "Tikeya", 0.0 },
  { "Tikeyah", 0.0 },
  { "Tikhon", 1.0 },
  { "Tiki", 0.060686015 },
  { "Tikia", 0.0 },
  { "Tikila", 0.0 },
  { "Tikina", 0.0 },
  { "Tikira", 0.0 },
  { "Tikisha", 0.0 },
  { "Tikita", 0.0 },
  { "Tikiya", 0.0 },
  { "Tikiyah", 0.0 },
  { "Tikki", 0.0 },
  { "Tiko", 1.0 },
  { "Tikva", 0.0 },
  { "Tikvah", 0.0 },
  { "Tikyra", 0.0 },
  { "Tila", 0.0 },
  { "Tilak", 1.0 },
  { "Tilan", 1.0 },
  { "Tilar", 0.3939394 },
  { "Tilda", 0.0 },
  { "Tilden", 0.96497697 },
  { "Tildon", 1.0 },
  { "Tileah", 0.0 },
  { "Tilena", 0.0 },
  { "Tiler", 0.8289855 },
  { "Tilesha", 0.0 },
  { "Tilford", 1.0 },
  { "Tilghman", 1.0 },
  { "Tilia", 0.0 },
  { "Tilian", 1.0 },
  { "Tilio", 1.0 },
  { "Tilisa", 0.0 },
  { "Tilisha", 0.0 },
  { "Tiliyah", 0.0 },
  { "Till", 1.0 },
  { "Tilla", 0.0 },
  { "Tillee", 0.0 },
  { "Tiller", 1.0 },
  { "Tilley", 0.0 },
  { "Tillian", 1.0 },
  { "Tillie", 0.0018536317 },
  { "Tillis", 1.0 },
  { "Tillman", 1.0 },
  { "Tillmon", 1.0 },
  { "Tilly", 0.0 },
  { "Tilman", 1.0 },
  { "Tilmer", 1.0 },
  { "Tilmon", 1.0 },
  { "Tilon", 1.0 },
  { "Tilor", 1.0 },
  { "Tilton", 1.0 },
  { "Tilwanda", 0.0 },
  { "Tilynn", 0.0 },
  { "Tim", 0.9977396 },
  { "Tima", 0.0 },
  { "Timaeus", 1.0 },
  { "Timaj", 1.0 },
  { "Timaka", 0.0 },
  { "Timani", 0.0 },
  { "Timar", 1.0 },
  { "Timara", 0.0 },
  { "Timarah", 0.0 },
  { "Timarcus", 1.0 },
  { "Timaree", 0.0 },
  { "Timari", 0.35064936 },
  { "Timarie", 0.0 },
  { "Timarion", 1.0 },
  { "Timathy", 1.0 },
  { "Timaya", 0.0 },
  { "Timayah", 0.0 },
  { "Timber", 0.33310103 },
  { "Timberland", 1.0 },
  { "Timberlee", 0.0 },
  { "Timberley", 0.0 },
  { "Timberly", 0.0 },
  { "Timberlyn", 0.0 },
  { "Timberlynn", 0.0 },
  { "Timbra", 0.0 },
  { "Timbre", 0.0 },
  { "Timea", 0.0 },
  { "Timeah", 0.0 },
  { "Timeca", 0.0 },
  { "Timeisha", 0.0 },
  { "Timeka", 0.0 },
  { "Timekia", 0.0 },
  { "Timeko", 0.0 },
  { "Timera", 0.0 },
  { "Timeria", 0.0 },
  { "Timesha", 0.0 },
  { "Timeshia", 0.0 },
  { "Timetrius", 1.0 },
  { "Timi", 0.00764526 },
  { "Timia", 0.0 },
  { "Timiah", 0.0 },
  { "Timica", 0.0 },
  { "Timicka", 0.0 },
  { "Timika", 0.0 },
  { "Timiki", 0.0 },
  { "Timikia", 0.0 },
  { "Timiko", 0.0 },
  { "Timileyin", 1.0 },
  { "Timiothy", 1.0 },
  { "Timira", 0.0 },
  { "Timisha", 0.0 },
  { "Timithy", 1.0 },
  { "Timitra", 0.0 },
  { "Timiya", 0.0 },
  { "Timiyah", 0.0 },
  { "Timm", 1.0 },
  { "Timmeka", 0.0 },
  { "Timmesha", 0.0 },
  { "Timmeshia", 0.0 },
  { "Timmey", 1.0 },
  { "Timmi", 0.0 },
  { "Timmia", 0.0 },
  { "Timmiah", 0.0 },
  { "Timmie", 0.90250695 },
  { "Timmithy", 1.0 },
  { "Timmon", 1.0 },
  { "Timmons", 1.0 },
  { "Timmothy", 1.0 },
  { "Timmy", 0.9934643 },
  { "Timmya", 0.0 },
  { "Timmyah", 0.0 },
  { "Timnesha", 0.0 },
  { "Timo", 1.0 },
  { "Timofei", 1.0 },
  { "Timofey", 1.0 },
  { "Timohty", 1.0 },
  { "Timohy", 1.0 },
  { "Timolin", 0.0 },
  { "Timolyn", 0.0 },
  { "Timon", 1.0 },
  { "Timone", 1.0 },
  { "Timonthy", 1.0 },
  { "Timora", 0.0 },
  { "Timorthy", 1.0 },
  { "Timote", 1.0 },
  { "Timotea", 0.0 },
  { "Timoteo", 1.0 },
  { "Timoth", 1.0 },
  { "Timothe", 1.0 },
  { "Timothea", 0.0 },
  { "Timothee", 1.0 },
  { "Timotheus", 1.0 },
  { "Timothey", 1.0 },
  { "Timothhy", 1.0 },
  { "Timothie", 1.0 },
  { "Timothy", 0.99595875 },
  { "Timothyjames", 1.0 },
  { "Timothyjohn", 1.0 },
  { "Timothyy", 1.0 },
  { "Timoty", 1.0 },
  { "Timouthy", 1.0 },
  { "Timthoy", 1.0 },
  { "Timthy", 1.0 },
  { "Timtohy", 1.0 },
  { "Timur", 1.0 },
  { "Timya", 0.0 },
  { "Timyah", 0.0 },
  { "Timyia", 0.0 },
  { "Timyra", 0.0 },
  { "Tin", 1.0 },
  { "Tina", 0.0030972466 },
  { "Tinaka", 0.0 },
  { "Tinalouise", 0.0 },
  { "Tinamaria", 0.0 },
  { "Tinamarie", 0.0 },
  { "Tinasha", 0.0 },
  { "Tinashe", 0.2777778 },
  { "Tinasia", 0.0 },
  { "Tinaya", 0.0 },
  { "Tincie", 0.0 },
  { "Tincy", 0.0 },
  { "Tindra", 0.0 },
  { "Tine", 0.0 },
  { "Tinea", 0.0 },
  { "Tineisha", 0.0 },
  { "Tineka", 0.0 },
  { "Tineke", 0.0 },
  { "Tinelle", 0.0 },
  { "Tinesha", 0.0 },
  { "Tineshia", 0.0 },
  { "Tinette", 0.0 },
  { "Tiney", 0.015060241 },
  { "Ting", 0.17045455 },
  { "Tinh", 1.0 },
  { "Tini", 0.0 },
  { "Tinia", 0.0 },
  { "Tinica", 0.0 },
  { "Tinie", 0.0 },
  { "Tinika", 0.0 },
  { "Tiniki", 0.0 },
  { "Tinina", 0.0 },
  { "Tiniqua", 0.0 },
  { "Tinisha", 0.0 },
  { "Tinishia", 0.0 },
  { "Tinita", 0.0 },
  { "Tiniya", 0.0 },
  { "Tiniyah", 0.0 },
  { "Tinka", 0.0 },
  { "Tinker", 0.5 },
  { "Tinlee", 0.0 },
  { "Tinleigh", 0.0 },
  { "Tinley", 0.0 },
  { "Tinna", 0.0 },
  { "Tinnelle", 0.0 },
  { "Tinnie", 0.0 },
  { "Tinnley", 0.0 },
  { "Tinny", 0.0 },
  { "Tino", 1.0 },
  { "Tinothy", 1.0 },
  { "Tinsae", 1.0 },
  { "Tinslee", 0.0 },
  { "Tinsleigh", 0.0 },
  { "Tinsley", 0.041825093 },
  { "Tinslie", 0.0 },
  { "Tiny", 0.07161922 },
  { "Tinya", 0.0 },
  { "Tinyah", 0.0 },
  { "Tinzlee", 0.0 },
  { "Tinzleigh", 0.0 },
  { "Tinzley", 0.0 },
  { "Tio", 1.0 },
  { "Tiodora", 0.0 },
  { "Tiodoro", 1.0 },
  { "Tiofila", 0.0 },
  { "Tiofilo", 1.0 },
  { "Tioluwani", 1.0 },
  { "Tiombe", 0.0 },
  { "Tiomthy", 1.0 },
  { "Tion", 0.96181387 },
  { "Tiona", 0.0 },
  { "Tionah", 0.0 },
  { "Tiondra", 0.0 },
  { "Tione", 0.44444445 },
  { "Tionee", 0.0 },
  { "Tioni", 0.0 },
  { "Tionna", 0.0 },
  { "Tionne", 0.10969388 },
  { "Tionni", 0.0 },
  { "Tionnie", 0.0 },
  { "Tiosha", 0.0 },
  { "Tip", 1.0 },
  { "Tiphaine", 0.0 },
  { "Tiphanee", 0.0 },
  { "Tiphani", 0.0 },
  { "Tiphanie", 0.0 },
  { "Tiphany", 0.0 },
  { "Tippany", 0.0 },
  { "Tippi", 0.0 },
  { "Tippy", 0.45454547 },
  { "Tipton", 0.97536945 },
  { "Tiquan", 1.0 },
  { "Tiquana", 0.0 },
  { "Tiquila", 0.0 },
  { "Tiquisha", 0.0 },
  { "Tiquita", 0.0 },
  { "Tira", 0.0 },
  { "Tirah", 0.0 },
  { "Tiran", 1.0 },
  { "Tiranke", 0.0 },
  { "Tiras", 1.0 },
  { "Tircio", 1.0 },
  { "Tiree", 1.0 },
  { "Tirek", 1.0 },
  { "Tirell", 1.0 },
  { "Tirelle", 1.0 },
  { "Tirenioluwa", 0.0 },
  { "Tirese", 1.0 },
  { "Tiria", 0.0 },
  { "Tirian", 1.0 },
  { "Tirik", 1.0 },
  { "Tirina", 0.0 },
  { "Tiriq", 1.0 },
  { "Tiron", 1.0 },
  { "Tirone", 1.0 },
  { "Tirra", 0.0 },
  { "Tirrany", 0.0 },
  { "Tirrell", 0.9831606 },
  { "Tirsa", 0.0 },
  { "Tirso", 1.0 },
  { "Tirth", 1.0 },
  { "Tirus", 1.0 },
  { "Tirza", 0.0 },
  { "Tirzah", 0.0 },
  { "Tirzo", 1.0 },
  { "Tisa", 0.0 },
  { "Tischa", 0.0 },
  { "Tisean", 1.0 },
  { "Tish", 0.0 },
  { "Tisha", 0.0 },
  { "Tishae", 0.0 },
  { "Tishana", 0.0 },
  { "Tishanna", 0.0 },
  { "Tishara", 0.0 },
  { "Tishaun", 1.0 },
  { "Tishauna", 0.0 },
  { "Tishawn", 0.810219 },
  { "Tishawna", 0.0 },
  { "Tishay", 0.0 },
  { "Tishayla", 0.0 },
  { "Tishea", 0.0 },
  { "Tisheena", 0.0 },
  { "Tisheka", 0.0 },
  { "Tishena", 0.0 },
  { "Tishera", 0.0 },
  { "Tishia", 0.0 },
  { "Tishie", 0.0 },
  { "Tishika", 0.0 },
  { "Tishina", 0.0 },
  { "Tishon", 1.0 },
  { "Tishona", 0.0 },
  { "Tishonda", 0.0 },
  { "Tishonna", 0.0 },
  { "Tishya", 0.0 },
  { "Tiso", 1.0 },
  { "Tison", 1.0 },
  { "Tisya", 0.0 },
  { "Tita", 0.0 },
  { "Titan", 1.0 },
  { "Titana", 0.0 },
  { "Titania", 0.0 },
  { "Titanna", 0.0 },
  { "Titeanna", 0.0 },
  { "Tithi", 0.0 },
  { "Titia", 0.0 },
  { "Titiana", 0.0 },
  { "Titianna", 0.0 },
  { "Titilayo", 0.0 },
  { "Titilope", 0.0 },
  { "Titiyana", 0.0 },
  { "Tito", 1.0 },
  { "Titus", 0.9993895 },
  { "Tityana", 0.0 },
  { "Tityanna", 0.0 },
  { "Tiuana", 0.0 },
  { "Tiva", 0.0 },
  { "Tivis", 1.0 },
  { "Tivoli", 0.0 },
  { "Tivon", 1.0 },
  { "Tivona", 0.0 },
  { "Tiwaloluwa", 0.0 },
  { "Tiwan", 0.924812 },
  { "Tiwana", 0.0 },
  { "Tiwanda", 0.0 },
  { "Tiwanna", 0.0 },
  { "Tiwatope", 0.0 },
  { "Tiy", 0.0 },
  { "Tiya", 0.0 },
  { "Tiyah", 0.0 },
  { "Tiyana", 0.0 },
  { "Tiyanah", 0.0 },
  { "Tiyanna", 0.0 },
  { "Tiyanni", 0.0 },
  { "Tiye", 0.0 },
  { "Tiyler", 1.0 },
  { "Tiyon", 1.0 },
  { "Tiyona", 0.0 },
  { "Tiyonna", 0.0 },
  { "Tiziana", 0.0 },
  { "Tiziano", 1.0 },
  { "Tizoc", 1.0 },
  { "Tj", 0.9946495 },
  { "Tjaden", 1.0 },
  { "Tjay", 1.0 },
  { "Tjuan", 1.0 },
  { "Tjuana", 0.0 },
  { "Tkai", 0.5 },
  { "Tkaia", 0.0 },
  { "Tkaiya", 0.0 },
  { "Tkara", 0.0 },
  { "Tkayah", 0.0 },
  { "Tkeya", 0.0 },
  { "Tkeyah", 0.0 },
  { "Tkeyha", 0.0 },
  { "Tkia", 0.0 },
  { "Tkiyah", 0.0 },
  { "Tkya", 0.0 },
  { "Tkyra", 0.0 },
  { "Tlalli", 0.0 },
  { "Tlaloc", 1.0 },
  { "Tleah", 0.0 },
  { "Tliyah", 0.0 },
  { "Tlyer", 1.0 },
  { "Tmara", 0.0 },
  { "Tmarion", 1.0 },
  { "Tmaya", 0.0 },
  { "Tmia", 0.0 },
  { "Tmothy", 1.0 },
  { "Tmya", 0.0 },
  { "Tmyra", 0.0 },
  { "Tnaya", 0.0 },
  { "Tnia", 0.0 },
  { "Tnisha", 0.0 },
  { "Tniya", 0.0 },
  { "Tniyah", 0.0 },
  { "Tnya", 0.0 },
  { "Tnyah", 0.0 },
  { "Toa", 1.0 },
  { "Toan", 1.0 },
  { "Toba", 0.0 },
  { "Tobais", 1.0 },
  { "Tobbie", 0.5 },
  { "Tobby", 1.0 },
  { "Tobe", 0.9698231 },
  { "Tobechi", 1.0 },
  { "Tobechukwu", 1.0 },
  { "Toben", 1.0 },
  { "Tobenna", 1.0 },
  { "Tobey", 0.68043846 },
  { "Tobi", 0.1296526 },
  { "Tobia", 1.0 },
  { "Tobiah", 1.0 },
  { "Tobian", 1.0 },
  { "Tobiann", 0.0 },
  { "Tobias", 1.0 },
  { "Tobie", 0.39625168 },
  { "Tobijah", 1.0 },
  { "Tobiloba", 1.0 },
  { "Tobin", 0.98353827 },
  { "Tobitha", 0.0 },
  { "Tobius", 1.0 },
  { "Toby", 0.81237656 },
  { "Tobyas", 1.0 },
  { "Tobye", 0.0 },
  { "Tobyn", 0.91570884 },
  { "Tocara", 0.0 },
  { "Tocarra", 0.0 },
  { "Tocarro", 0.0 },
  { "Toccara", 0.014285714 },
  { "Toccaro", 0.0 },
  { "Toccarra", 0.0 },
  { "Toccora", 0.0 },
  { "Tocha", 0.0 },
  { "Tochi", 0.27631578 },
  { "Tochukwu", 1.0 },
  { "Tod", 1.0 },
  { "Todd", 0.9962416 },
  { "Todderick", 1.0 },
  { "Toddrick", 1.0 },
  { "Toddy", 1.0 },
  { "Toderick", 1.0 },
  { "Todo", 1.0 },
  { "Todrick", 1.0 },
  { "Toi", 0.034282017 },
  { "Toia", 0.0 },
  { "Toie", 0.0 },
  { "Toika", 0.0 },
  { "Toinetta", 0.0 },
  { "Toinette", 0.0 },
  { "Toini", 0.0 },
  { "Toisha", 0.0 },
  { "Toivo", 1.0 },
  { "Toiya", 0.0 },
  { "Tojuana", 0.0 },
  { "Toka", 0.0 },
  { "Tokala", 1.0 },
  { "Toki", 0.29411766 },
  { "Tokie", 0.0 },
  { "Tokiko", 0.0 },
  { "Tokina", 0.0 },
  { "Tokio", 1.0 },
  { "Tokuichi", 1.0 },
  { "Tokunbo", 0.5 },
  { "Tokuo", 1.0 },
  { "Tokyo", 1.0 },
  { "Tol", 1.0 },
  { "Tola", 0.06 },
  { "Tolan", 1.0 },
  { "Toland", 1.0 },
  { "Tolanda", 0.0 },
  { "Tolani", 0.0 },
  { "Tolbert", 1.0 },
  { "Toleda", 0.0 },
  { "Toledo", 0.0 },
  { "Toleen", 0.0 },
  { "Toler", 1.0 },
  { "Tolga", 1.0 },
  { "Tolisha", 0.0 },
  { "Toliver", 1.0 },
  { "Tollie", 0.9339934 },
  { "Tolliver", 1.0 },
  { "Tolly", 0.64705884 },
  { "Tolonda", 0.0 },
  { "Toltu", 0.0 },
  { "Tolulope", 0.2 },
  { "Toluwalase", 0.5416667 },
  { "Toluwalope", 0.0 },
  { "Toluwani", 0.5 },
  { "Toluwanimi", 0.36190477 },
  { "Tom", 0.9956135 },
  { "Toma", 0.6966292 },
  { "Tomacina", 0.0 },
  { "Tomacita", 0.0 },
  { "Tomaka", 0.0 },
  { "Tomako", 0.0 },
  { "Tomar", 1.0 },
  { "Tomara", 0.0 },
  { "Tomari", 1.0 },
  { "Tomario", 1.0 },
  { "Tomarion", 1.0 },
  { "Tomarra", 0.0 },
  { "Tomas", 0.9990165 },
  { "Tomasa", 0.0 },
  { "Tomasi", 1.0 },
  { "Tomasina", 0.0 },
  { "Tomasita", 0.0 },
  { "Tomasz", 1.0 },
  { "Tomatra", 1.0 },
  { "Tomaya", 0.0 },
  { "Tomaz", 1.0 },
  { "Tome", 1.0 },
  { "Tomea", 0.0 },
  { "Tomeaka", 0.0 },
  { "Tomeca", 0.0 },
  { "Tomecca", 0.0 },
  { "Tomecia", 0.0 },
  { "Tomecka", 0.0 },
  { "Tomeco", 0.0 },
  { "Tomeeka", 0.0 },
  { "Tomeica", 0.0 },
  { "Tomeika", 0.0 },
  { "Tomeisha", 0.0 },
  { "Tomeka", 0.0016515277 },
  { "Tomeki", 0.0 },
  { "Tomekia", 0.0 },
  { "Tomeko", 0.0 },
  { "Tomer", 1.0 },
  { "Tomes", 1.0 },
  { "Tomesha", 0.0 },
  { "Tomeshia", 0.0 },
  { "Tomesia", 0.0 },
  { "Tomey", 1.0 },
  { "Tomi", 0.04475942 },
  { "Tomia", 0.0 },
  { "Tomiah", 0.0 },
  { "Tomias", 1.0 },
  { "Tomica", 0.0 },
  { "Tomicia", 0.0 },
  { "Tomicka", 0.0 },
  { "Tomicko", 0.0 },
  { "Tomico", 0.0 },
  { "Tomie", 0.6576839 },
  { "Tomieka", 0.0 },
  { "Tomii", 0.0 },
  { "Tomika", 0.0017946877 },
  { "Tomikia", 0.0 },
  { "Tomiko", 0.0075675678 },
  { "Tomio", 1.0 },
  { "Tomisha", 0.0 },
  { "Tomislav", 1.0 },
  { "Tomiwa", 1.0 },
  { "Tomiya", 0.0 },
  { "Tomiyah", 0.0 },
  { "Tomiye", 0.0 },
  { "Tomkia", 0.0 },
  { "Tomm", 1.0 },
  { "Tomma", 0.0 },
  { "Tommas", 1.0 },
  { "Tommaso", 1.0 },
  { "Tomme", 1.0 },
  { "Tommesha", 0.0 },
  { "Tommey", 1.0 },
  { "Tommi", 0.010210593 },
  { "Tommia", 0.0 },
  { "Tommie", 0.6632385 },
  { "Tommisha", 0.0 },
  { "Tommy", 0.98417944 },
  { "Tommya", 0.0 },
  { "Tommye", 0.03448276 },
  { "Tommylee", 1.0 },
  { "Tomo", 1.0 },
  { "Tomoaki", 1.0 },
  { "Tomoe", 0.0 },
  { "Tomohiro", 1.0 },
  { "Tomoka", 0.0 },
  { "Tomoki", 1.0 },
  { "Tomoko", 0.0 },
  { "Tomomi", 0.0 },
  { "Tomora", 0.0 },
  { "Tomorra", 0.0 },
  { "Tomorrow", 0.0 },
  { "Tomothy", 1.0 },
  { "Tomoya", 1.0 },
  { "Tomoye", 0.0 },
  { "Tomoyuki", 1.0 },
  { "Tomy", 1.0 },
  { "Tomya", 0.0 },
  { "Tomyka", 0.0 },
  { "Tomyra", 0.0 },
  { "Ton", 1.0 },
  { "Tona", 0.0 },
  { "Tonae", 0.0 },
  { "Tonantzin", 0.0 },
  { "Tonasia", 0.0 },
  { "Tonatiuh", 1.0 },
  { "Tonay", 0.0 },
  { "Tonaya", 0.0 },
  { "Tonda", 0.0 },
  { "Tondalaya", 0.0 },
  { "Tondalayo", 0.0 },
  { "Tondelayo", 0.0 },
  { "Tondi", 0.0 },
  { "Tondia", 0.0 },
  { "Tondra", 0.0 },
  { "Tondrea", 0.0 },
  { "Tondria", 0.0 },
  { "Tone", 0.4787234 },
  { "Tonea", 0.0 },
  { "Tonecia", 0.0 },
  { "Tonee", 0.028571429 },
  { "Toneesha", 0.0 },
  { "Toneisha", 0.0 },
  { "Toneka", 0.0 },
  { "Tonesha", 0.0 },
  { "Toneshia", 0.0 },
  { "Tonesia", 0.0 },
  { "Tonetta", 0.0 },
  { "Tonette", 0.0 },
  { "Toney", 0.9703228 },
  { "Tong", 1.0 },
  { "Tonga", 0.0 },
  { "Tongela", 0.0 },
  { "Tongia", 0.0 },
  { "Toni", 0.014646409 },
  { "Tonia", 0.0023872799 },
  { "Toniah", 0.0 },
  { "Toniann", 0.0 },
  { "Tonianne", 0.0 },
  { "Tonica", 0.0 },
  { "Tonice", 0.0 },
  { "Tonicia", 0.0 },
  { "Tonicka", 0.0 },
  { "Tonie", 0.10693374 },
  { "Tonielle", 0.0 },
  { "Toniesha", 0.0 },
  { "Toniette", 0.0 },
  { "Tonii", 0.0 },
  { "Tonija", 0.0 },
  { "Tonika", 0.0 },
  { "Tonilyn", 0.0 },
  { "Tonilynn", 0.0 },
  { "Tonimarie", 0.0 },
  { "Tonio", 1.0 },
  { "Toniqua", 0.0 },
  { "Tonique", 0.0 },
  { "Tonirose", 0.0 },
  { "Tonisa", 0.0 },
  { "Tonise", 0.0 },
  { "Tonisha", 0.0 },
  { "Tonishia", 0.0 },
  { "Tonisia", 0.0 },
  { "Tonita", 0.0 },
  { "Toniya", 0.0 },
  { "Toniyah", 0.0 },
  { "Tonja", 0.00071489846 },
  { "Tonji", 0.0 },
  { "Tonjia", 0.0 },
  { "Tonjua", 0.0 },
  { "Tonka", 1.0 },
  { "Tonna", 0.0 },
  { "Tonnesha", 0.0 },
  { "Tonnetta", 0.0 },
  { "Tonnette", 0.0 },
  { "Tonni", 0.0 },
  { "Tonnia", 0.0 },
  { "Tonnica", 0.0 },
  { "Tonnie", 0.4088159 },
  { "Tonnisha", 0.0 },
  { "Tonny", 1.0 },
  { "Tonnya", 0.0 },
  { "Tonoa", 0.0 },
  { "Tonora", 0.0 },
  { "Tonta", 0.0 },
  { "Tonua", 0.0 },
  { "Tony", 0.98852277 },
  { "Tonya", 0.0036035888 },
  { "Tonye", 0.0 },
  { "Tonyeka", 0.0 },
  { "Tonyell", 0.0 },
  { "Tonyetta", 0.0 },
  { "Tonyette", 0.0 },
  { "Tonyia", 0.0 },
  { "Tonyna", 0.0 },
  { "Tonysha", 0.0 },
  { "Tonyua", 0.0 },
  { "Tonza", 0.0 },
  { "Tooba", 0.0 },
  { "Tood", 1.0 },
  { "Toosdhi", 0.0 },
  { "Tootie", 0.0 },
  { "Toots", 0.0 },
  { "Tootsie", 0.0 },
  { "Topacio", 0.0 },
  { "Topanga", 0.0 },
  { "Topaz", 0.020408163 },
  { "Topazio", 0.0 },
  { "Topeka", 0.0 },
  { "Topher", 1.0 },
  { "Topper", 1.0 },
  { "Toprak", 1.0 },
  { "Topsy", 0.0 },
  { "Tor", 1.0 },
  { "Tora", 0.028888889 },
  { "Torah", 0.0 },
  { "Toraino", 1.0 },
  { "Toral", 0.0 },
  { "Toran", 1.0 },
  { "Torance", 0.5185185 },
  { "Toranio", 1.0 },
  { "Torao", 1.0 },
  { "Toray", 1.0 },
  { "Torben", 1.0 },
  { "Torchie", 0.0 },
  { "Torchy", 0.0 },
  { "Tore", 0.872093 },
  { "Torean", 1.0 },
  { "Toree", 0.06696428 },
  { "Torei", 0.0 },
  { "Torell", 1.0 },
  { "Toren", 0.99192464 },
  { "Torence", 0.93333334 },
  { "Torenzo", 1.0 },
  { "Toretto", 1.0 },
  { "Torey", 0.6665102 },
  { "Torez", 1.0 },
  { "Tori", 0.0313484 },
  { "Toria", 0.0 },
  { "Toriah", 0.0 },
  { "Torian", 0.8543689 },
  { "Toriana", 0.00862069 },
  { "Toriann", 0.0 },
  { "Torianna", 0.0 },
  { "Torianne", 0.0 },
  { "Torianno", 1.0 },
  { "Toriano", 1.0 },
  { "Toribia", 0.0 },
  { "Toribio", 1.0 },
  { "Torica", 0.0 },
  { "Torie", 0.08909574 },
  { "Torien", 1.0 },
  { "Torii", 0.2236842 },
  { "Torika", 0.0 },
  { "Torilyn", 0.0 },
  { "Torilynn", 0.0 },
  { "Torin", 0.9556252 },
  { "Torina", 0.0 },
  { "Torino", 0.7894737 },
  { "Torion", 1.0 },
  { "Toriona", 0.0 },
  { "Torionna", 0.0 },
  { "Toriono", 1.0 },
  { "Torique", 1.0 },
  { "Toris", 1.0 },
  { "Torita", 0.0 },
  { "Torivio", 1.0 },
  { "Toriyah", 0.0 },
  { "Torlan", 1.0 },
  { "Torland", 1.0 },
  { "Tornell", 1.0 },
  { "Tornisha", 0.0 },
  { "Toron", 1.0 },
  { "Toronda", 0.0 },
  { "Toronto", 1.0 },
  { "Torr", 1.0 },
  { "Torra", 0.0 },
  { "Torraine", 1.0 },
  { "Torran", 1.0 },
  { "Torrance", 0.8767053 },
  { "Torray", 1.0 },
  { "Torre", 0.6734198 },
  { "Torrean", 1.0 },
  { "Torree", 0.0 },
  { "Torrel", 1.0 },
  { "Torrell", 1.0 },
  { "Torren", 0.95408165 },
  { "Torrence", 0.9481088 },
  { "Torreon", 1.0 },
  { "Torres", 1.0 },
  { "Torrey", 0.8037061 },
  { "Torrez", 1.0 },
  { "Torri", 0.10428134 },
  { "Torria", 0.0 },
  { "Torrian", 0.9776423 },
  { "Torriana", 0.0 },
  { "Torrianna", 0.0 },
  { "Torriano", 1.0 },
  { "Torrica", 0.0 },
  { "Torrick", 1.0 },
  { "Torrie", 0.17260678 },
  { "Torrien", 1.0 },
  { "Torrin", 0.87129986 },
  { "Torrion", 1.0 },
  { "Torris", 0.98644984 },
  { "Torriyah", 0.0 },
  { "Torron", 1.0 },
  { "Torry", 0.8584283 },
  { "Torrye", 0.0 },
  { "Torryn", 0.23387097 },
  { "Torsha", 0.0 },
  { "Torstein", 1.0 },
  { "Torsten", 1.0 },
  { "Toru", 1.0 },
  { "Torunn", 0.0 },
  { "Tory", 0.6431109 },
  { "Torya", 0.0 },
  { "Torye", 0.0 },
  { "Toryn", 0.5270506 },
  { "Torynn", 0.0 },
  { "Tosca", 0.0 },
  { "Toscha", 0.0 },
  { "Toschia", 0.0 },
  { "Tosh", 1.0 },
  { "Tosha", 0.0 },
  { "Toshana", 0.0 },
  { "Tosheba", 0.0 },
  { "Toshi", 0.14141414 },
  { "Toshia", 0.0 },
  { "Toshiaki", 1.0 },
  { "Toshiana", 0.0 },
  { "Toshiba", 0.0 },
  { "Toshie", 0.0 },
  { "Toshiharu", 1.0 },
  { "Toshika", 0.0 },
  { "Toshiki", 1.0 },
  { "Toshiko", 0.0 },
  { "Toshima", 0.0 },
  { "Toshina", 0.0 },
  { "Toshio", 1.0 },
  { "Toshiro", 1.0 },
  { "Toshiye", 0.0 },
  { "Toshiyuki", 1.0 },
  { "Toshua", 0.29891303 },
  { "Toshya", 0.0 },
  { "Tossie", 0.0 },
  { "Totiana", 0.0 },
  { "Totianna", 0.0 },
  { "Totiyana", 0.0 },
  { "Totsie", 0.0 },
  { "Tou", 1.0 },
  { "Toua", 1.0 },
  { "Toufic", 1.0 },
  { "Touger", 1.0 },
  { "Toula", 0.0 },
  { "Toure", 1.0 },
  { "Tousha", 0.0 },
  { "Toussaint", 1.0 },
  { "Tova", 0.0 },
  { "Tovah", 0.0 },
  { "Tovan", 1.0 },
  { "Tovaris", 1.0 },
  { "Tove", 0.0 },
  { "Tovi", 1.0 },
  { "Tovia", 0.8161435 },
  { "Toviah", 1.0 },
  { "Tovias", 1.0 },
  { "Tovin", 1.0 },
  { "Toviyah", 1.0 },
  { "Towa", 1.0 },
  { "Towan", 1.0 },
  { "Towana", 0.0 },
  { "Towanda", 0.0 },
  { "Towanna", 0.0 },
  { "Townes", 1.0 },
  { "Townsend", 1.0 },
  { "Towonda", 0.0 },
  { "Toxi", 0.0 },
  { "Toxie", 1.0 },
  { "Toy", 0.5434372 },
  { "Toya", 0.0057687485 },
  { "Toyah", 0.0 },
  { "Toyana", 0.0 },
  { "Toyce", 0.0 },
  { "Toye", 0.051344745 },
  { "Toyea", 0.0 },
  { "Toyia", 0.0 },
  { "Toyin", 0.0 },
  { "Toyka", 0.0 },
  { "Toyna", 0.0 },
  { "Toynelle", 0.0 },
  { "Toyoko", 0.0 },
  { "Toyota", 0.0 },
  { "Toyya", 0.0 },
  { "Tpring", 0.0 },
  { "Tquan", 1.0 },
  { "Tr", 1.0 },
  { "Tra", 0.9717742 },
  { "Trabian", 1.0 },
  { "Traca", 0.0 },
  { "Trace", 0.96895087 },
  { "Tracee", 0.0029222677 },
  { "Tracei", 0.0 },
  { "Tracen", 1.0 },
  { "Tracer", 1.0 },
  { "Tracey", 0.09071674 },
  { "Trachell", 0.0 },
  { "Trachelle", 0.0 },
  { "Traci", 0.0044528535 },
  { "Tracia", 0.0 },
  { "Traciann", 0.0 },
  { "Tracianne", 0.0 },
  { "Tracie", 0.0066573014 },
  { "Tracilyn", 0.0 },
  { "Tracilynn", 0.0 },
  { "Tracina", 0.0 },
  { "Tracine", 0.0 },
  { "Track", 1.0 },
  { "Tracker", 1.0 },
  { "Tracy", 0.19601089 },
  { "Tracyann", 0.0 },
  { "Tracye", 0.004608295 },
  { "Tracylee", 0.0 },
  { "Tracylynn", 0.0 },
  { "Tracyn", 1.0 },
  { "Trad", 1.0 },
  { "Tradarius", 1.0 },
  { "Tradarrius", 1.0 },
  { "Tradd", 1.0 },
  { "Tradell", 1.0 },
  { "Traden", 1.0 },
  { "Tradon", 1.0 },
  { "Trae", 0.9874966 },
  { "Traeden", 1.0 },
  { "Traedyn", 1.0 },
  { "Traegan", 1.0 },
  { "Traeger", 1.0 },
  { "Traeh", 0.29 },
  { "Traelon", 1.0 },
  { "Traelyn", 1.0 },
  { "Traequan", 1.0 },
  { "Traesean", 1.0 },
  { "Traeshaun", 1.0 },
  { "Traeshawn", 1.0 },
  { "Traeshon", 1.0 },
  { "Traeson", 1.0 },
  { "Traesyn", 1.0 },
  { "Traeton", 1.0 },
  { "Traetyn", 1.0 },
  { "Traeven", 1.0 },
  { "Traeveon", 1.0 },
  { "Traevin", 1.0 },
  { "Traevion", 1.0 },
  { "Traevon", 1.0 },
  { "Traevyn", 1.0 },
  { "Trafton", 1.0 },
  { "Tragen", 1.0 },
  { "Trager", 1.0 },
  { "Trahern", 1.0 },
  { "Trai", 1.0 },
  { "Traice", 0.2777778 },
  { "Traiden", 1.0 },
  { "Traie", 0.0 },
  { "Traig", 1.0 },
  { "Traigh", 1.0 },
  { "Trail", 1.0 },
  { "Trais", 1.0 },
  { "Traison", 1.0 },
  { "Traiten", 1.0 },
  { "Traiton", 1.0 },
  { "Traiveon", 1.0 },
  { "Traivon", 1.0 },
  { "Trajan", 1.0 },
  { "Trajen", 1.0 },
  { "Trajon", 1.0 },
  { "Trajuan", 1.0 },
  { "Trakeila", 0.0 },
  { "Trakelia", 0.0 },
  { "Trakia", 0.0 },
  { "Tralaina", 0.0 },
  { "Tralana", 0.0 },
  { "Tralaya", 0.0 },
  { "Tralena", 0.0 },
  { "Tralon", 1.0 },
  { "Tralyn", 1.0 },
  { "Tralynn", 0.0 },
  { "Tram", 0.0 },
  { "Tramain", 1.0 },
  { "Tramaine", 0.75829387 },
  { "Tramal", 1.0 },
  { "Tramale", 1.0 },
  { "Tramane", 1.0 },
  { "Tramanh", 0.0 },
  { "Tramar", 1.0 },
  { "Tramarcus", 1.0 },
  { "Tramari", 1.0 },
  { "Tramarion", 1.0 },
  { "Tramayne", 1.0 },
  { "Trameka", 0.0 },
  { "Tramel", 1.0 },
  { "Tramell", 1.0 },
  { "Tramelle", 1.0 },
  { "Tramesha", 0.0 },
  { "Trami", 0.0 },
  { "Tramia", 0.0 },
  { "Tramiah", 0.0 },
  { "Tramika", 0.0 },
  { "Tramine", 0.85714287 },
  { "Tramisha", 0.0 },
  { "Tramiya", 0.0 },
  { "Tramiyah", 0.0 },
  { "Trammell", 1.0 },
  { "Tramon", 1.0 },
  { "Tramond", 1.0 },
  { "Tramone", 1.0 },
  { "Trampas", 1.0 },
  { "Trampis", 1.0 },
  { "Trampus", 1.0 },
  { "Tramya", 0.0 },
  { "Tran", 0.1816839 },
  { "Trana", 0.0 },
  { "Tranae", 0.0 },
  { "Tranard", 1.0 },
  { "Tranasia", 0.0 },
  { "Tranay", 0.0 },
  { "Tranaya", 0.0 },
  { "Trance", 1.0 },
  { "Tranda", 0.0 },
  { "Trandon", 1.0 },
  { "Trane", 0.0 },
  { "Tranea", 0.0 },
  { "Tranecia", 0.0 },
  { "Tranee", 0.0 },
  { "Traneice", 0.0 },
  { "Traneisha", 0.0 },
  { "Traneka", 0.0 },
  { "Tranel", 1.0 },
  { "Tranell", 0.71014494 },
  { "Tranese", 0.0 },
  { "Tranesha", 0.0 },
  { "Traneshia", 0.0 },
  { "Tranessa", 0.0 },
  { "Tranette", 0.0 },
  { "Trang", 0.008307373 },
  { "Trania", 0.0 },
  { "Tranice", 0.0 },
  { "Traniece", 0.0 },
  { "Tranika", 0.0 },
  { "Traniqua", 0.0 },
  { "Tranis", 1.0 },
  { "Tranise", 0.0 },
  { "Tranisha", 0.0 },
  { "Tranita", 0.0 },
  { "Traniya", 0.0 },
  { "Traniyah", 0.0 },
  { "Trannie", 0.0 },
  { "Tranquilino", 1.0 },
  { "Trany", 0.0 },
  { "Tranya", 0.0 },
  { "Tranyah", 0.0 },
  { "Tranyce", 0.0 },
  { "Trapp", 1.0 },
  { "Trapper", 1.0 },
  { "Traquan", 1.0 },
  { "Traquez", 1.0 },
  { "Trasa", 0.0 },
  { "Trase", 1.0 },
  { "Trasean", 1.0 },
  { "Trasen", 1.0 },
  { "Trasha", 0.0 },
  { "Trashaun", 1.0 },
  { "Trashawn", 0.9297821 },
  { "Trashell", 0.0 },
  { "Trashon", 1.0 },
  { "Trashonda", 0.0 },
  { "Trashun", 1.0 },
  { "Trasi", 0.0 },
  { "Trasia", 0.0 },
  { "Trask", 1.0 },
  { "Trason", 1.0 },
  { "Trauis", 1.0 },
  { "Traun", 1.0 },
  { "Trava", 0.0 },
  { "Travail", 1.0 },
  { "Travan", 1.0 },
  { "Travante", 1.0 },
  { "Travanti", 1.0 },
  { "Travares", 1.0 },
  { "Travarious", 1.0 },
  { "Travaris", 1.0 },
  { "Travarius", 1.0 },
  { "Travarus", 1.0 },
  { "Travas", 1.0 },
  { "Travaughn", 1.0 },
  { "Travaun", 1.0 },
  { "Traveion", 1.0 },
  { "Travel", 1.0 },
  { "Travell", 1.0 },
  { "Travelle", 1.0 },
  { "Traven", 1.0 },
  { "Traveon", 1.0 },
  { "Traver", 1.0 },
  { "Travers", 1.0 },
  { "Traverse", 1.0 },
  { "Traves", 1.0 },
  { "Travette", 0.0 },
  { "Traveyon", 1.0 },
  { "Travez", 1.0 },
  { "Travia", 0.0 },
  { "Travian", 1.0 },
  { "Travianna", 0.0 },
  { "Travie", 1.0 },
  { "Travien", 1.0 },
  { "Travin", 1.0 },
  { "Travina", 0.0 },
  { "Travion", 1.0 },
  { "Traviona", 0.0 },
  { "Travione", 1.0 },
  { "Travionna", 0.0 },
  { "Travionne", 1.0 },
  { "Travious", 1.0 },
  { "Travis", 0.9929982 },
  { "Travisha", 0.0 },
  { "Traviss", 1.0 },
  { "Travius", 1.0 },
  { "Travolta", 1.0 },
  { "Travon", 0.99553645 },
  { "Travona", 0.0 },
  { "Travond", 1.0 },
  { "Travonda", 0.0 },
  { "Travone", 1.0 },
  { "Travonn", 1.0 },
  { "Travonna", 0.0 },
  { "Travonne", 1.0 },
  { "Travonta", 1.0 },
  { "Travontae", 1.0 },
  { "Travontay", 1.0 },
  { "Travonte", 1.0 },
  { "Travor", 1.0 },
  { "Travoris", 1.0 },
  { "Travus", 1.0 },
  { "Travyn", 1.0 },
  { "Travyon", 1.0 },
  { "Travys", 1.0 },
  { "Trawn", 1.0 },
  { "Trax", 1.0 },
  { "Traxton", 1.0 },
  { "Tray", 1.0 },
  { "Trayana", 0.0 },
  { "Trayanna", 0.0 },
  { "Trayc", 1.0 },
  { "Trayce", 0.63826084 },
  { "Traycee", 0.0 },
  { "Traycen", 1.0 },
  { "Trayci", 0.0 },
  { "Trayden", 1.0 },
  { "Traydon", 1.0 },
  { "Traye", 1.0 },
  { "Traylan", 1.0 },
  { "Traylen", 1.0 },
  { "Traylin", 1.0 },
  { "Traylon", 1.0 },
  { "Traylor", 1.0 },
  { "Traymon", 1.0 },
  { "Trayon", 1.0 },
  { "Trayona", 0.0 },
  { "Trayonna", 0.0 },
  { "Trayquan", 1.0 },
  { "Trayse", 1.0 },
  { "Traysean", 1.0 },
  { "Traysen", 1.0 },
  { "Trayshaun", 1.0 },
  { "Trayshawn", 1.0 },
  { "Trayshon", 1.0 },
  { "Trayshun", 1.0 },
  { "Trayson", 1.0 },
  { "Trayten", 1.0 },
  { "Trayton", 1.0 },
  { "Trayven", 1.0 },
  { "Trayveon", 1.0 },
  { "Trayvin", 1.0 },
  { "Trayvion", 1.0 },
  { "Trayvis", 1.0 },
  { "Trayvon", 1.0 },
  { "Trayvond", 1.0 },
  { "Trayvone", 1.0 },
  { "Trayvonne", 1.0 },
  { "Trcy", 0.0 },
  { "Tre", 0.9893147 },
  { "Trea", 0.7281553 },
  { "Treacy", 0.0 },
  { "Treagan", 1.0 },
  { "Treana", 0.0 },
  { "Treanna", 0.0 },
  { "Treanthony", 1.0 },
  { "Treasa", 0.0 },
  { "Trease", 0.0 },
  { "Treasea", 0.0 },
  { "Treasia", 0.0 },
  { "Treasie", 0.0 },
  { "Treasure", 0.020653132 },
  { "Treat", 1.0 },
  { "Treauna", 0.0 },
  { "Treava", 0.0 },
  { "Treaven", 1.0 },
  { "Treaver", 1.0 },
  { "Treavion", 1.0 },
  { "Treavon", 1.0 },
  { "Treavor", 1.0 },
  { "Treazure", 0.0 },
  { "Treba", 0.0 },
  { "Treble", 1.0 },
  { "Trebor", 1.0 },
  { "Treca", 0.0 },
  { "Trechelle", 0.0 },
  { "Trecia", 0.0 },
  { "Trecie", 0.0 },
  { "Tred", 1.0 },
  { "Treda", 0.0 },
  { "Tredan", 1.0 },
  { "Tredarius", 1.0 },
  { "Tredell", 1.0 },
  { "Tredena", 0.0 },
  { "Tredon", 1.0 },
  { "Tredyn", 1.0 },
  { "Treena", 0.0 },
  { "Treesa", 0.0 },
  { "Treg", 1.0 },
  { "Tregan", 1.0 },
  { "Tregg", 1.0 },
  { "Trei", 1.0 },
  { "Treigh", 1.0 },
  { "Treighton", 1.0 },
  { "Treina", 0.0 },
  { "Treisa", 0.0 },
  { "Treisha", 0.0 },
  { "Treivon", 1.0 },
  { "Trejan", 1.0 },
  { "Trejaun", 1.0 },
  { "Trejean", 1.0 },
  { "Trejohn", 1.0 },
  { "Trejon", 1.0 },
  { "Trejuan", 1.0 },
  { "Trejure", 0.0 },
  { "Trek", 1.0 },
  { "Treka", 0.0 },
  { "Trekwan", 1.0 },
  { "Trekwon", 1.0 },
  { "Trela", 0.0 },
  { "Trelan", 1.0 },
  { "Trelana", 0.0 },
  { "Trelen", 1.0 },
  { "Trelin", 1.0 },
  { "Trelisa", 0.0 },
  { "Trell", 1.0 },
  { "Trella", 0.0 },
  { "Trellany", 0.0 },
  { "Trellis", 0.23042506 },
  { "Trelon", 1.0 },
  { "Trelyn", 1.0 },
  { "Trelynn", 0.54545456 },
  { "Trema", 0.0 },
  { "Tremain", 1.0 },
  { "Tremaine", 0.9347383 },
  { "Tremal", 1.0 },
  { "Tremane", 1.0 },
  { "Tremar", 1.0 },
  { "Tremarcus", 1.0 },
  { "Tremari", 1.0 },
  { "Tremarion", 1.0 },
  { "Tremaya", 0.0 },
  { "Tremayne", 0.98976916 },
  { "Tremeka", 0.0 },
  { "Tremekia", 0.0 },
  { "Tremel", 1.0 },
  { "Tremell", 1.0 },
  { "Tremelle", 1.0 },
  { "Tremere", 1.0 },
  { "Tremesha", 0.0 },
  { "Tremia", 0.0 },
  { "Tremica", 0.0 },
  { "Tremika", 0.0 },
  { "Tremir", 1.0 },
  { "Tremiyah", 0.0 },
  { "Tremon", 1.0 },
  { "Tremond", 1.0 },
  { "Tremone", 1.0 },
  { "Tremont", 1.0 },
  { "Tremonte", 1.0 },
  { "Tremya", 0.0 },
  { "Tremyah", 0.0 },
  { "Tren", 1.0 },
  { "Trena", 0.0 },
  { "Trenace", 0.0 },
  { "Trenadi", 0.0 },
  { "Trenady", 0.0 },
  { "Trenae", 0.0 },
  { "Trenard", 1.0 },
  { "Trenasia", 0.0 },
  { "Trenaty", 0.0 },
  { "Trenay", 0.0 },
  { "Trence", 1.0 },
  { "Trenda", 0.0 },
  { "Trendan", 1.0 },
  { "Trendarius", 1.0 },
  { "Trendell", 1.0 },
  { "Trenden", 1.0 },
  { "Trendon", 1.0 },
  { "Trendyn", 1.0 },
  { "Trene", 0.0 },
  { "Trenea", 0.0 },
  { "Trenece", 0.0 },
  { "Trenecia", 0.0 },
  { "Trenedy", 0.0 },
  { "Trenee", 0.0 },
  { "Treneice", 0.0 },
  { "Treneisha", 0.0 },
  { "Treneka", 0.0 },
  { "Trenell", 0.66511625 },
  { "Trenese", 0.0 },
  { "Trenesha", 0.0 },
  { "Trenetta", 0.0 },
  { "Trenette", 0.0 },
  { "Trenia", 0.0 },
  { "Trenica", 0.0 },
  { "Trenice", 0.0 },
  { "Trenicia", 0.0 },
  { "Trenidad", 0.5272727 },
  { "Trenidy", 0.0 },
  { "Treniece", 0.0 },
  { "Trenika", 0.0 },
  { "Trenise", 0.0 },
  { "Trenisha", 0.0 },
  { "Trenita", 0.0 },
  { "Trenitee", 0.0 },
  { "Treniti", 0.0 },
  { "Trenity", 0.030955585 },
  { "Treniya", 0.0 },
  { "Treniyah", 0.0 },
  { "Trenna", 0.0 },
  { "Trennan", 1.0 },
  { "Trennedy", 0.0 },
  { "Trennen", 1.0 },
  { "Trennis", 1.0 },
  { "Trennon", 1.0 },
  { "Trenny", 0.0 },
  { "Trenon", 1.0 },
  { "Trent", 0.9979464 },
  { "Trentan", 1.0 },
  { "Trentavious", 1.0 },
  { "Trenten", 1.0 },
  { "Trentin", 1.0 },
  { "Trentity", 0.0 },
  { "Trento", 1.0 },
  { "Trenton", 0.9985788 },
  { "Trentyn", 1.0 },
  { "Trenya", 0.0 },
  { "Trenyce", 0.0 },
  { "Treon", 1.0 },
  { "Treona", 0.0 },
  { "Treonna", 0.0 },
  { "Trequan", 1.0 },
  { "Trequon", 1.0 },
  { "Treqwan", 1.0 },
  { "Treron", 1.0 },
  { "Tres", 0.9878049 },
  { "Tresa", 0.0 },
  { "Tresaun", 1.0 },
  { "Tresca", 0.0 },
  { "Trese", 0.0 },
  { "Tresea", 0.0 },
  { "Tresean", 1.0 },
  { "Tresha", 0.0 },
  { "Treshad", 1.0 },
  { "Treshaun", 0.99346405 },
  { "Treshawn", 0.9964235 },
  { "Treshell", 0.0 },
  { "Treshia", 0.0 },
  { "Treshon", 1.0 },
  { "Treshun", 1.0 },
  { "Tresia", 0.0 },
  { "Treson", 1.0 },
  { "Tresor", 0.7368421 },
  { "Tress", 0.0 },
  { "Tressa", 0.0 },
  { "Tressel", 1.0 },
  { "Tressia", 0.0 },
  { "Tressie", 0.0011252813 },
  { "Tressy", 0.0 },
  { "Tresta", 0.0 },
  { "Trestan", 1.0 },
  { "Tresten", 1.0 },
  { "Trestin", 1.0 },
  { "Treston", 1.0 },
  { "Trestyn", 1.0 },
  { "Tresure", 0.0 },
  { "Tresvon", 1.0 },
  { "Tretha", 0.0 },
  { "Treton", 1.0 },
  { "Trev", 1.0 },
  { "Treva", 0.00062337663 },
  { "Trevahn", 1.0 },
  { "Trevail", 1.0 },
  { "Trevan", 1.0 },
  { "Trevante", 1.0 },
  { "Trevar", 1.0 },
  { "Trevares", 1.0 },
  { "Trevarious", 1.0 },
  { "Trevaris", 1.0 },
  { "Trevarius", 1.0 },
  { "Trevathan", 1.0 },
  { "Trevaughn", 1.0 },
  { "Trevaun", 1.0 },
  { "Treve", 1.0 },
  { "Treveion", 1.0 },
  { "Trevel", 1.0 },
  { "Trevell", 1.0 },
  { "Trevelle", 1.0 },
  { "Trevelyan", 1.0 },
  { "Treven", 1.0 },
  { "Treveon", 1.0 },
  { "Trever", 1.0 },
  { "Treves", 1.0 },
  { "Treveyon", 1.0 },
  { "Trevez", 1.0 },
  { "Trevhon", 1.0 },
  { "Trevi", 0.0 },
  { "Trevia", 0.0 },
  { "Trevian", 1.0 },
  { "Trevien", 1.0 },
  { "Trevier", 1.0 },
  { "Trevin", 1.0 },
  { "Trevina", 0.0 },
  { "Trevino", 1.0 },
  { "Trevion", 1.0 },
  { "Trevione", 1.0 },
  { "Trevionne", 1.0 },
  { "Trevious", 1.0 },
  { "Trevira", 0.0 },
  { "Trevis", 0.99429876 },
  { "Trevius", 1.0 },
  { "Trevohn", 1.0 },
  { "Trevon", 0.9981396 },
  { "Trevona", 0.0 },
  { "Trevond", 1.0 },
  { "Trevone", 1.0 },
  { "Trevonn", 1.0 },
  { "Trevonna", 0.0 },
  { "Trevonne", 1.0 },
  { "Trevonta", 1.0 },
  { "Trevontae", 1.0 },
  { "Trevontay", 1.0 },
  { "Trevonte", 1.0 },
  { "Trevor", 0.9960793 },
  { "Trevore", 1.0 },
  { "Trevoris", 1.0 },
  { "Trevorjames", 1.0 },
  { "Trevour", 1.0 },
  { "Trevun", 1.0 },
  { "Trevvor", 1.0 },
  { "Trevyn", 1.0 },
  { "Trevyon", 1.0 },
  { "Trew", 0.89719623 },
  { "Trexton", 1.0 },
  { "Trey", 0.9973755 },
  { "Treya", 0.0 },
  { "Treyana", 0.0 },
  { "Treyanna", 0.0 },
  { "Treyce", 1.0 },
  { "Treycen", 1.0 },
  { "Treydan", 1.0 },
  { "Treyden", 1.0 },
  { "Treydon", 1.0 },
  { "Treye", 1.0 },
  { "Treygan", 1.0 },
  { "Treylan", 1.0 },
  { "Treylen", 1.0 },
  { "Treylin", 1.0 },
  { "Treylon", 1.0 },
  { "Treylyn", 1.0 },
  { "Treymaine", 1.0 },
  { "Treymane", 1.0 },
  { "Treymon", 1.0 },
  { "Treyon", 1.0 },
  { "Treyonna", 0.0 },
  { "Treyor", 1.0 },
  { "Treyquan", 1.0 },
  { "Treysean", 1.0 },
  { "Treysen", 1.0 },
  { "Treyshaun", 1.0 },
  { "Treyshawn", 1.0 },
  { "Treyshon", 1.0 },
  { "Treyshun", 1.0 },
  { "Treysin", 1.0 },
  { "Treyson", 1.0 },
  { "Treyston", 1.0 },
  { "Treyten", 1.0 },
  { "Treytin", 1.0 },
  { "Treyton", 1.0 },
  { "Treyvan", 1.0 },
  { "Treyvaughn", 1.0 },
  { "Treyven", 1.0 },
  { "Treyveon", 1.0 },
  { "Treyvian", 1.0 },
  { "Treyvin", 1.0 },
  { "Treyvion", 1.0 },
  { "Treyvon", 1.0 },
  { "Treyvond", 1.0 },
  { "Treyvone", 1.0 },
  { "Treyvonn", 1.0 },
  { "Treyvonne", 1.0 },
  { "Treyvonte", 1.0 },
  { "Treyvor", 1.0 },
  { "Trez", 1.0 },
  { "Trezden", 1.0 },
  { "Trezdon", 1.0 },
  { "Trezon", 1.0 },
  { "Trezure", 0.0 },
  { "Tri", 1.0 },
  { "Tria", 0.0 },
  { "Triana", 0.0 },
  { "Triandos", 1.0 },
  { "Trianna", 0.0 },
  { "Tribeca", 0.0 },
  { "Tribecca", 0.0 },
  { "Trica", 0.0 },
  { "Trice", 1.0 },
  { "Tricha", 0.0 },
  { "Trichelle", 0.0 },
  { "Trichia", 0.0 },
  { "Tricia", 0.0021927638 },
  { "Tricity", 0.0 },
  { "Triena", 0.0 },
  { "Trier", 0.0 },
  { "Trieste", 0.0 },
  { "Trieu", 1.0 },
  { "Trifon", 1.0 },
  { "Trig", 1.0 },
  { "Trigg", 1.0 },
  { "Trigger", 1.0 },
  { "Trigo", 1.0 },
  { "Trijal", 1.0 },
  { "Trilba", 0.0 },
  { "Trilby", 0.0 },
  { "Trill", 1.0 },
  { "Trilla", 0.0 },
  { "Trillian", 0.11904762 },
  { "Trillion", 1.0 },
  { "Trillis", 0.0 },
  { "Trillium", 0.0 },
  { "Trimaine", 1.0 },
  { "Trimayne", 1.0 },
  { "Trimeka", 0.0 },
  { "Trimika", 0.0 },
  { "Trina", 0.0009247557 },
  { "Trinadee", 0.0 },
  { "Trinadi", 0.0 },
  { "Trinady", 0.0 },
  { "Trinae", 0.0 },
  { "Trinatee", 0.0 },
  { "Trinati", 0.0 },
  { "Trinaty", 0.0 },
  { "Trinay", 1.0 },
  { "Trinda", 0.0 },
  { "Trinden", 1.0 },
  { "Trindon", 1.0 },
  { "Trine", 0.11527377 },
  { "Trinea", 0.0 },
  { "Trineka", 0.0 },
  { "Trinell", 0.11111111 },
  { "Trinere", 0.0 },
  { "Trinese", 0.0 },
  { "Trinesha", 0.0 },
  { "Trinetta", 0.0 },
  { "Trinette", 0.0 },
  { "Trinety", 0.0 },
  { "Trinh", 0.0 },
  { "Trini", 0.2513312 },
  { "Trinia", 0.0 },
  { "Trinice", 0.0 },
  { "Trinicia", 0.0 },
  { "Trinida", 1.0 },
  { "Trinidad", 0.548028 },
  { "Trinidee", 0.0 },
  { "Trinidi", 0.0 },
  { "Trinidie", 0.0 },
  { "Trinidy", 0.039951574 },
  { "Trinie", 0.0 },
  { "Triniece", 0.0 },
  { "Trinika", 0.0 },
  { "Trinisa", 0.0 },
  { "Trinise", 0.0 },
  { "Trinisha", 0.0 },
  { "Trinita", 0.0 },
  { "Trinite", 0.0 },
  { "Trinitee", 0.0 },
  { "Trinitey", 0.0 },
  { "Triniti", 0.0 },
  { "Trinitie", 0.0 },
  { "Trinitty", 0.0 },
  { "Trinity", 0.03540261 },
  { "Triniya", 0.0 },
  { "Triniyah", 0.0 },
  { "Trinka", 0.0 },
  { "Trinket", 0.0 },
  { "Trinley", 0.0 },
  { "Trinna", 0.0 },
  { "Trinnie", 0.0 },
  { "Trinnity", 0.0 },
  { "Trino", 1.0 },
  { "Trint", 1.0 },
  { "Trinten", 1.0 },
  { "Trintin", 1.0 },
  { "Trintiy", 0.0 },
  { "Trinton", 1.0 },
  { "Trinty", 0.0 },
  { "Trintyn", 1.0 },
  { "Triny", 0.0 },
  { "Trinyti", 0.0 },
  { "Trion", 1.0 },
  { "Triona", 0.0 },
  { "Trionna", 0.0 },
  { "Trip", 1.0 },
  { "Tripp", 1.0 },
  { "Tripper", 1.0 },
  { "Trippton", 1.0 },
  { "Tripton", 1.0 },
  { "Tris", 0.33333334 },
  { "Trisa", 0.0 },
  { "Trisca", 0.0 },
  { "Trischa", 0.0 },
  { "Triscia", 0.0 },
  { "Triscilla", 0.0 },
  { "Trisco", 1.0 },
  { "Trisden", 1.0 },
  { "Trish", 0.0 },
  { "Trisha", 0.001330789 },
  { "Trishalana", 0.0 },
  { "Trishamae", 0.0 },
  { "Trishan", 1.0 },
  { "Trishana", 0.0 },
  { "Trishanna", 0.0 },
  { "Trishawn", 0.45454547 },
  { "Trishawna", 0.0 },
  { "Trishell", 0.0 },
  { "Trishelle", 0.0 },
  { "Trishia", 0.0 },
  { "Trishika", 0.0 },
  { "Trishna", 0.0 },
  { "Trishona", 0.0 },
  { "Trishonda", 0.0 },
  { "Trisia", 0.0 },
  { "Trisity", 0.0 },
  { "Triska", 0.0 },
  { "Trison", 1.0 },
  { "Triss", 0.0 },
  { "Trissa", 0.0 },
  { "Trissie", 0.0 },
  { "Trista", 0.00040426909 },
  { "Tristaca", 0.0 },
  { "Tristah", 0.0 },
  { "Tristain", 0.96892655 },
  { "Tristam", 1.0 },
  { "Tristan", 0.94394004 },
  { "Tristana", 0.0 },
  { "Tristanjames", 1.0 },
  { "Tristann", 0.0 },
  { "Tristano", 1.0 },
  { "Tristany", 0.0 },
  { "Triste", 0.0 },
  { "Tristen", 0.8254063 },
  { "Tristi", 0.0 },
  { "Tristia", 0.0 },
  { "Tristian", 0.91143703 },
  { "Tristiana", 0.0 },
  { "Tristianna", 0.0 },
  { "Tristica", 0.0 },
  { "Tristien", 1.0 },
  { "Tristin", 0.7664731 },
  { "Tristina", 0.0 },
  { "Tristine", 0.024271844 },
  { "Tristion", 1.0 },
  { "Triston", 0.9754216 },
  { "Tristram", 1.0 },
  { "Tristun", 1.0 },
  { "Tristy", 0.0 },
  { "Tristyn", 0.48568428 },
  { "Tristynn", 0.0 },
  { "Tritan", 1.0 },
  { "Tritia", 0.0 },
  { "Triton", 1.0 },
  { "Triumph", 1.0 },
  { "Triva", 0.0 },
  { "Trivia", 0.0 },
  { "Trivian", 0.0 },
  { "Trivon", 1.0 },
  { "Trixi", 0.0 },
  { "Trixie", 0.0 },
  { "Trixy", 0.0 },
  { "Triya", 0.0 },
  { "Troas", 0.0 },
  { "Troi", 0.1402439 },
  { "Troian", 0.0 },
  { "Tromaine", 1.0 },
  { "Tron", 0.9869403 },
  { "Trona", 0.0 },
  { "Trond", 1.0 },
  { "Tronda", 0.0 },
  { "Trone", 1.0 },
  { "Trong", 1.0 },
  { "Trooper", 1.0 },
  { "Trotter", 1.0 },
  { "Trovon", 1.0 },
  { "Trowa", 1.0 },
  { "Troy", 0.98765624 },
  { "Troya", 0.0 },
  { "Troyal", 1.0 },
  { "Troyanne", 0.0 },
  { "Troyce", 0.8637363 },
  { "Troye", 0.77319586 },
  { "Troylene", 0.0 },
  { "Troylynn", 0.0 },
  { "Troyvon", 1.0 },
  { "Tru", 0.4965902 },
  { "Truby", 0.49253732 },
  { "Truc", 0.06995885 },
  { "Truce", 1.0 },
  { "Trucilla", 0.0 },
  { "Trucker", 1.0 },
  { "Truda", 0.0 },
  { "Truddie", 0.0 },
  { "Trude", 0.0 },
  { "Trudee", 0.0 },
  { "Trudell", 0.0 },
  { "Trudence", 0.0 },
  { "Trudi", 0.0 },
  { "Trudie", 0.0 },
  { "Trudis", 0.0 },
  { "Trudith", 0.0 },
  { "Trudy", 0.00019678842 },
  { "True", 0.5579499 },
  { "Truel", 1.0 },
  { "Truely", 0.0 },
  { "Trueman", 1.0 },
  { "Truen", 1.0 },
  { "Truett", 0.9973531 },
  { "Truette", 1.0 },
  { "Truitt", 1.0 },
  { "Trula", 0.0 },
  { "Trulee", 0.0 },
  { "Truley", 0.0 },
  { "Trulie", 0.0 },
  { "Truly", 0.008038585 },
  { "Truma", 0.0 },
  { "Trumaine", 1.0 },
  { "Truman", 0.998238 },
  { "Trumell", 1.0 },
  { "Trumon", 1.0 },
  { "Trunell", 1.0 },
  { "Trunetta", 0.0 },
  { "Trung", 1.0 },
  { "Truong", 1.0 },
  { "Trupti", 0.0 },
  { "Trusha", 0.0 },
  { "Trust", 0.6878049 },
  { "Trusten", 1.0 },
  { "Trustin", 1.0 },
  { "Truston", 1.0 },
  { "Trustyn", 1.0 },
  { "Truth", 0.5933076 },
  { "Truxton", 1.0 },
  { "Trvis", 1.0 },
  { "Tryan", 1.0 },
  { "Tryce", 1.0 },
  { "Tryell", 1.0 },
  { "Tryg", 1.0 },
  { "Trygg", 1.0 },
  { "Trygve", 1.0 },
  { "Tryna", 0.0 },
  { "Trynati", 0.0 },
  { "Trynitee", 0.0 },
  { "Tryniti", 0.0 },
  { "Trynity", 0.0 },
  { "Trynt", 1.0 },
  { "Trynton", 1.0 },
  { "Tryon", 1.0 },
  { "Tryone", 1.0 },
  { "Tryp", 1.0 },
  { "Tryphena", 0.0 },
  { "Trypp", 1.0 },
  { "Trysha", 0.0 },
  { "Tryson", 1.0 },
  { "Trysta", 0.0 },
  { "Trystal", 0.0 },
  { "Trystan", 0.8610631 },
  { "Trysten", 0.8171905 },
  { "Trystian", 1.0 },
  { "Trystin", 0.7270974 },
  { "Trystn", 1.0 },
  { "Tryston", 0.9921445 },
  { "Trystyn", 0.71684587 },
  { "Tryton", 1.0 },
  { "Tsai", 1.0 },
  { "Tsean", 1.0 },
  { "Tsega", 0.0 },
  { "Tselane", 0.0 },
  { "Tsering", 0.23 },
  { "Tshai", 0.0 },
  { "Tshaka", 1.0 },
  { "Tshanti", 0.0 },
  { "Tshara", 0.0 },
  { "Tshawn", 1.0 },
  { "Tshaye", 0.0 },
  { "Tshombe", 1.0 },
  { "Tshwanda", 0.0 },
  { "Tsianina", 0.0 },
  { "Tsianna", 0.0 },
  { "Tsion", 0.1456693 },
  { "Tsitsiki", 0.0 },
  { "Tsosie", 1.0 },
  { "Tsubaki", 0.0 },
  { "Tsubasa", 1.0 },
  { "Tsugie", 0.0 },
  { "Tsugio", 1.0 },
  { "Tsukasa", 1.0 },
  { "Tsunade", 0.0 },
  { "Tsunami", 0.0 },
  { "Tsuneko", 0.0 },
  { "Tsuneo", 1.0 },
  { "Tsurue", 0.0 },
  { "Tsuruko", 0.0 },
  { "Tsuruyo", 0.0 },
  { "Tsutako", 0.0 },
  { "Tsutomu", 1.0 },
  { "Tsuyako", 0.0 },
  { "Tsuyoshi", 1.0 },
  { "Tsuyuko", 0.0 },
  { "Tsvia", 0.0 },
  { "Tu", 0.74473065 },
  { "Tuan", 1.0 },
  { "Tuana", 0.0 },
  { "Tuba", 0.0 },
  { "Tuck", 1.0 },
  { "Tucker", 0.9922499 },
  { "Tucson", 1.0 },
  { "Tudor", 1.0 },
  { "Tudy", 0.0 },
  { "Tue", 1.0 },
  { "Tuere", 0.0 },
  { "Tuesdae", 0.0 },
  { "Tuesdai", 0.0 },
  { "Tuesday", 0.0 },
  { "Tuesdee", 0.0 },
  { "Tuf", 1.0 },
  { "Tuff", 1.0 },
  { "Tug", 1.0 },
  { "Tugg", 1.0 },
  { "Tujuan", 1.0 },
  { "Tujuana", 0.0 },
  { "Tukesha", 0.0 },
  { "Tukker", 1.0 },
  { "Tula", 0.0 },
  { "Tulah", 0.0 },
  { "Tulani", 0.0 },
  { "Tulasi", 0.0 },
  { "Tuleen", 0.0 },
  { "Tulia", 0.0 },
  { "Tulio", 1.0 },
  { "Tulip", 0.0 },
  { "Tulisa", 0.0 },
  { "Tullia", 0.0 },
  { "Tullie", 1.0 },
  { "Tullio", 1.0 },
  { "Tullis", 1.0 },
  { "Tully", 0.9517454 },
  { "Tulon", 1.0 },
  { "Tulsa", 0.0 },
  { "Tulsi", 0.0 },
  { "Tumeka", 0.0 },
  { "Tumekia", 0.0 },
  { "Tumika", 0.0 },
  { "Tunde", 1.0 },
  { "Tundra", 0.0 },
  { "Tunesia", 0.0 },
  { "Tung", 1.0 },
  { "Tunis", 1.0 },
  { "Tunisa", 0.0 },
  { "Tunisha", 0.0 },
  { "Tunisia", 0.0 },
  { "Tunja", 0.0 },
  { "Tunney", 1.0 },
  { "Tunny", 1.0 },
  { "Tunya", 0.0 },
  { "Tunyia", 0.0 },
  { "Tuong", 0.54545456 },
  { "Tuongvi", 0.0 },
  { "Tuongvy", 0.0 },
  { "Tupac", 1.0 },
  { "Tupou", 0.29411766 },
  { "Tura", 0.0 },
  { "Turan", 1.0 },
  { "Turea", 0.0 },
  { "Tureka", 0.0 },
  { "Turell", 1.0 },
  { "Turhan", 1.0 },
  { "Turi", 0.20833333 },
  { "Turia", 0.0 },
  { "Turin", 1.0 },
  { "Turina", 0.0 },
  { "Turiya", 0.0 },
  { "Turk", 1.0 },
  { "Turkesa", 0.0 },
  { "Turkesha", 0.0 },
  { "Turkessa", 0.0 },
  { "Turki", 1.0 },
  { "Turley", 1.0 },
  { "Turmaine", 1.0 },
  { "Turner", 0.9719705 },
  { "Turney", 1.0 },
  { "Turon", 1.0 },
  { "Turquoise", 0.0 },
  { "Turrell", 1.0 },
  { "Turron", 1.0 },
  { "Tuscan", 1.0 },
  { "Tuscany", 0.0 },
  { "Tusha", 0.0 },
  { "Tushar", 1.0 },
  { "Tushara", 0.0 },
  { "Tushima", 0.0 },
  { "Tuson", 1.0 },
  { "Tustin", 1.0 },
  { "Tuula", 0.0 },
  { "Tuvia", 1.0 },
  { "Tuvya", 1.0 },
  { "Tuwana", 0.0 },
  { "Tuwanda", 0.0 },
  { "Tuwanna", 0.0 },
  { "Tuyen", 0.13978495 },
  { "Tuyet", 0.0 },
  { "Tuyetnhi", 0.0 },
  { "Tuyetnhung", 0.0 },
  { "Tvisha", 0.0 },
  { "Twain", 1.0 },
  { "Twaina", 0.0 },
  { "Twala", 0.0 },
  { "Twalla", 0.0 },
  { "Twan", 0.8865546 },
  { "Twana", 0.0 },
  { "Twanda", 0.0 },
  { "Twanetta", 0.0 },
  { "Twanette", 0.0 },
  { "Twania", 0.0 },
  { "Twanika", 0.0 },
  { "Twaniqua", 0.0 },
  { "Twanisha", 0.0 },
  { "Twanna", 0.0 },
  { "Twanya", 0.0 },
  { "Twayne", 1.0 },
  { "Twig", 1.0 },
  { "Twiggy", 0.0 },
  { "Twila", 0.0 },
  { "Twilah", 0.0 },
  { "Twilia", 0.0 },
  { "Twilight", 0.0 },
  { "Twilla", 0.0 },
  { "Twina", 0.0 },
  { "Twinkle", 0.0 },
  { "Twisha", 0.0 },
  { "Twon", 1.0 },
  { "Twonda", 0.0 },
  { "Twyla", 0.0 },
  { "Twylah", 0.0 },
  { "Twylia", 0.0 },
  { "Twylla", 0.0 },
  { "Twyman", 1.0 },
  { "Twynette", 0.0 },
  { "Ty", 0.9841316 },
  { "Tya", 0.0 },
  { "Tyah", 0.0 },
  { "Tyahna", 0.0 },
  { "Tyair", 1.0 },
  { "Tyaira", 0.0 },
  { "Tyairah", 0.0 },
  { "Tyaire", 0.969697 },
  { "Tyaisa", 0.0 },
  { "Tyaisha", 0.0 },
  { "Tyaja", 0.0 },
  { "Tyajah", 0.0 },
  { "Tyan", 0.7553957 },
  { "Tyana", 0.0 },
  { "Tyanah", 0.0 },
  { "Tyandra", 0.0 },
  { "Tyandre", 1.0 },
  { "Tyane", 0.0 },
  { "Tyani", 0.0 },
  { "Tyania", 0.0 },
  { "Tyann", 0.0 },
  { "Tyanna", 0.0 },
  { "Tyannah", 0.0 },
  { "Tyanne", 0.0 },
  { "Tyanni", 0.0 },
  { "Tyannia", 0.0 },
  { "Tyanthony", 1.0 },
  { "Tyara", 0.0 },
  { "Tyaria", 0.0 },
  { "Tyarra", 0.0 },
  { "Tyas", 1.0 },
  { "Tyasha", 0.0 },
  { "Tyashia", 0.0 },
  { "Tyasia", 0.0 },
  { "Tyasiah", 0.0 },
  { "Tyaun", 1.0 },
  { "Tyauna", 0.0 },
  { "Tyaunna", 0.0 },
  { "Tyazia", 0.0 },
  { "Tybee", 0.0 },
  { "Tyber", 1.0 },
  { "Tyberious", 1.0 },
  { "Tyberius", 1.0 },
  { "Tybrea", 0.0 },
  { "Tybresha", 0.0 },
  { "Tybria", 0.0 },
  { "Tybriana", 0.0 },
  { "Tyce", 1.0 },
  { "Tycee", 0.0 },
  { "Tycen", 1.0 },
  { "Tycere", 1.0 },
  { "Tyche", 0.0 },
  { "Tychell", 0.0 },
  { "Tychelle", 0.0 },
  { "Tychicus", 1.0 },
  { "Tychina", 0.0 },
  { "Tycho", 1.0 },
  { "Tycie", 0.0 },
  { "Tydaisha", 0.0 },
  { "Tydan", 1.0 },
  { "Tydarian", 1.0 },
  { "Tydarious", 1.0 },
  { "Tydarius", 1.0 },
  { "Tydarrius", 1.0 },
  { "Tydashia", 0.0 },
  { "Tyde", 1.0 },
  { "Tydell", 1.0 },
  { "Tyden", 1.0 },
  { "Tydon", 1.0 },
  { "Tydre", 1.0 },
  { "Tydrick", 1.0 },
  { "Tydus", 1.0 },
  { "Tye", 0.9354839 },
  { "Tyease", 0.0 },
  { "Tyeasha", 0.0 },
  { "Tyeast", 0.0 },
  { "Tyece", 0.0 },
  { "Tyecha", 0.0 },
  { "Tyechia", 0.0 },
  { "Tyecia", 0.0 },
  { "Tyee", 1.0 },
  { "Tyeesha", 0.0 },
  { "Tyeisha", 0.0 },
  { "Tyeishia", 0.0 },
  { "Tyeka", 0.0 },
  { "Tyelar", 1.0 },
  { "Tyeler", 0.90816325 },
  { "Tyelise", 0.0 },
  { "Tyell", 1.0 },
  { "Tyelor", 1.0 },
  { "Tyena", 0.0 },
  { "Tyenna", 0.0 },
  { "Tyequan", 1.0 },
  { "Tyer", 1.0 },
  { "Tyera", 0.0 },
  { "Tyere", 1.0 },
  { "Tyerell", 1.0 },
  { "Tyerra", 0.0 },
  { "Tyesa", 0.0 },
  { "Tyese", 0.0 },
  { "Tyesha", 0.0 },
  { "Tyeshawn", 1.0 },
  { "Tyeshia", 0.0 },
  { "Tyesia", 0.0 },
  { "Tyeson", 1.0 },
  { "Tyfani", 0.0 },
  { "Tyffani", 0.0 },
  { "Tyffanie", 0.0 },
  { "Tyffany", 0.0 },
  { "Tyga", 1.0 },
  { "Tygan", 1.0 },
  { "Tyge", 1.0 },
  { "Tyger", 0.8666667 },
  { "Tygh", 1.0 },
  { "Tyghe", 1.0 },
  { "Tyhecia", 0.0 },
  { "Tyheed", 1.0 },
  { "Tyheem", 1.0 },
  { "Tyheim", 1.0 },
  { "Tyheir", 1.0 },
  { "Tyheisha", 0.0 },
  { "Tyhesha", 0.0 },
  { "Tyheshia", 0.0 },
  { "Tyhesia", 0.0 },
  { "Tyhessia", 0.0 },
  { "Tyhiem", 1.0 },
  { "Tyhir", 1.0 },
  { "Tyhira", 0.0 },
  { "Tyhisa", 0.0 },
  { "Tyhisha", 0.0 },
  { "Tyhler", 1.0 },
  { "Tyi", 1.0 },
  { "Tyia", 0.0 },
  { "Tyiana", 0.0 },
  { "Tyianna", 0.0 },
  { "Tyice", 0.0 },
  { "Tyiesha", 0.0 },
  { "Tyieshia", 0.0 },
  { "Tyina", 0.0 },
  { "Tyion", 1.0 },
  { "Tyiona", 0.0 },
  { "Tyionna", 0.0 },
  { "Tyira", 0.0 },
  { "Tyisha", 0.0 },
  { "Tyishia", 0.0 },
  { "Tyissha", 0.0 },
  { "Tyius", 1.0 },
  { "Tyja", 0.0 },
  { "Tyjae", 0.48055556 },
  { "Tyjah", 0.25 },
  { "Tyjai", 0.52873564 },
  { "Tyjana", 0.0 },
  { "Tyjanae", 0.0 },
  { "Tyjanay", 0.0 },
  { "Tyjanea", 0.0 },
  { "Tyjanique", 0.0 },
  { "Tyjaun", 1.0 },
  { "Tyjay", 1.0 },
  { "Tyjhawn", 1.0 },
  { "Tyjhon", 1.0 },
  { "Tyji", 1.0 },
  { "Tyjier", 1.0 },
  { "Tyjir", 1.0 },
  { "Tyjohn", 1.0 },
  { "Tyjon", 1.0 },
  { "Tyjuan", 1.0 },
  { "Tyjuana", 0.0 },
  { "Tyka", 0.0 },
  { "Tykara", 0.0 },
  { "Tykaria", 0.0 },
  { "Tykayla", 0.0 },
  { "Tyke", 1.0 },
  { "Tykea", 0.0 },
  { "Tykee", 1.0 },
  { "Tykeem", 1.0 },
  { "Tykeia", 0.0 },
  { "Tykeira", 0.0 },
  { "Tykeisha", 0.0 },
  { "Tykeith", 1.0 },
  { "Tykel", 1.0 },
  { "Tykell", 1.0 },
  { "Tykenya", 0.0 },
  { "Tyker", 1.0 },
  { "Tykera", 0.0 },
  { "Tykeria", 0.0 },
  { "Tykeriah", 0.0 },
  { "Tykerra", 0.0 },
  { "Tykerria", 0.0 },
  { "Tykese", 1.0 },
  { "Tykesha", 0.0 },
  { "Tykeshia", 0.0 },
  { "Tykevion", 1.0 },
  { "Tykevious", 1.0 },
  { "Tykeya", 0.0 },
  { "Tykeyah", 0.0 },
  { "Tyki", 1.0 },
  { "Tykia", 0.0 },
  { "Tykiana", 0.0 },
  { "Tykiara", 0.0 },
  { "Tykie", 1.0 },
  { "Tykiera", 0.0 },
  { "Tykierra", 0.0 },
  { "Tykiesha", 0.0 },
  { "Tykim", 1.0 },
  { "Tyking", 1.0 },
  { "Tykira", 0.0 },
  { "Tykirah", 0.0 },
  { "Tykiria", 0.0 },
  { "Tykirra", 0.0 },
  { "Tykisha", 0.0 },
  { "Tykwan", 1.0 },
  { "Tykwon", 1.0 },
  { "Tyla", 0.002536232 },
  { "Tylah", 0.0 },
  { "Tylaisha", 0.0 },
  { "Tylajah", 0.0 },
  { "Tylan", 0.9429429 },
  { "Tyland", 1.0 },
  { "Tylane", 1.0 },
  { "Tylar", 0.56752765 },
  { "Tylasha", 0.0 },
  { "Tylashia", 0.0 },
  { "Tylasia", 0.0 },
  { "Tylaya", 0.0 },
  { "Tylayah", 0.0 },
  { "Tylayshia", 0.0 },
  { "Tylaysia", 0.0 },
  { "Tyle", 1.0 },
  { "Tylea", 0.0 },
  { "Tyleah", 0.0 },
  { "Tylecia", 0.0 },
  { "Tylee", 0.23268326 },
  { "Tyleek", 1.0 },
  { "Tyleen", 0.0 },
  { "Tyleesha", 0.0 },
  { "Tylei", 0.5 },
  { "Tyleia", 0.0 },
  { "Tyleigh", 0.0 },
  { "Tyleik", 1.0 },
  { "Tyleisha", 0.0 },
  { "Tylek", 1.0 },
  { "Tyleke", 1.0 },
  { "Tylen", 0.99115825 },
  { "Tylena", 0.0 },
  { "Tylene", 0.0 },
  { "Tyler", 0.97277206 },
  { "Tylere", 1.0 },
  { "Tylerjames", 1.0 },
  { "Tylerjohn", 1.0 },
  { "Tylerlee", 1.0 },
  { "Tylese", 0.0 },
  { "Tylesha", 0.0 },
  { "Tyleshia", 0.0 },
  { "Tyletha", 0.0 },
  { "Tylette", 0.0 },
  { "Tyley", 0.0 },
  { "Tyleyah", 0.0 },
  { "Tyli", 0.0 },
  { "Tylia", 0.0 },
  { "Tyliah", 0.0 },
  { "Tylicia", 0.0 },
  { "Tylie", 0.0 },
  { "Tyliek", 1.0 },
  { "Tylier", 1.0 },
  { "Tylik", 1.0 },
  { "Tylil", 1.0 },
  { "Tylin", 0.7582822 },
  { "Tylina", 0.0 },
  { "Tyliq", 1.0 },
  { "Tylique", 1.0 },
  { "Tylis", 1.0 },
  { "Tylisa", 0.0 },
  { "Tylisha", 0.0 },
  { "Tylissa", 0.0 },
  { "Tyliyah", 0.0 },
  { "Tyller", 0.7553192 },
  { "Tylo", 0.675 },
  { "Tylon", 1.0 },
  { "Tylone", 1.0 },
  { "Tylor", 0.9457542 },
  { "Tylr", 1.0 },
  { "Tylre", 1.0 },
  { "Tylun", 1.0 },
  { "Tylur", 1.0 },
  { "Tylyn", 0.2082019 },
  { "Tylynn", 0.09255533 },
  { "Tymaine", 1.0 },
  { "Tymar", 1.0 },
  { "Tymara", 0.0 },
  { "Tymari", 1.0 },
  { "Tymarian", 1.0 },
  { "Tymarion", 1.0 },
  { "Tymarius", 1.0 },
  { "Tymber", 0.042821158 },
  { "Tymberlyn", 0.0 },
  { "Tyme", 0.28985506 },
  { "Tymeek", 1.0 },
  { "Tymeer", 1.0 },
  { "Tymeir", 1.0 },
  { "Tymeisha", 0.0 },
  { "Tymeka", 0.0 },
  { "Tymekia", 0.0 },
  { "Tymel", 1.0 },
  { "Tymell", 1.0 },
  { "Tymera", 0.0 },
  { "Tymere", 1.0 },
  { "Tymeria", 0.0 },
  { "Tymerius", 1.0 },
  { "Tymesha", 0.0 },
  { "Tymeshia", 0.0 },
  { "Tymetrius", 1.0 },
  { "Tymia", 0.0 },
  { "Tymier", 1.0 },
  { "Tymiere", 1.0 },
  { "Tymiesha", 0.0 },
  { "Tymika", 0.0 },
  { "Tymir", 1.0 },
  { "Tymira", 0.0 },
  { "Tymirah", 0.0 },
  { "Tymire", 1.0 },
  { "Tymisha", 0.0 },
  { "Tymon", 1.0 },
  { "Tymond", 1.0 },
  { "Tymonte", 1.0 },
  { "Tymothy", 1.0 },
  { "Tymya", 0.0 },
  { "Tyna", 0.0 },
  { "Tynae", 0.0 },
  { "Tynaisha", 0.0 },
  { "Tynaja", 0.0 },
  { "Tynajah", 0.0 },
  { "Tynan", 1.0 },
  { "Tynasha", 0.0 },
  { "Tynashia", 0.0 },
  { "Tynasia", 0.0 },
  { "Tynaya", 0.0 },
  { "Tynayah", 0.0 },
  { "Tyndale", 1.0 },
  { "Tyndall", 0.0 },
  { "Tyne", 0.040268455 },
  { "Tynea", 0.0 },
  { "Tyneal", 0.0 },
  { "Tyneasha", 0.0 },
  { "Tynecia", 0.0 },
  { "Tyneeka", 0.0 },
  { "Tyneesha", 0.0 },
  { "Tyneice", 0.0 },
  { "Tyneika", 0.0 },
  { "Tyneil", 1.0 },
  { "Tyneisha", 0.0 },
  { "Tyneka", 0.0 },
  { "Tynell", 1.0 },
  { "Tynequa", 0.0 },
  { "Tyner", 1.0 },
  { "Tynese", 0.0 },
  { "Tynesha", 0.0 },
  { "Tyneshia", 0.0 },
  { "Tynesia", 0.0 },
  { "Tynetta", 0.0 },
  { "Tynette", 0.0 },
  { "Tynia", 0.0 },
  { "Tyniah", 0.0 },
  { "Tynica", 0.0 },
  { "Tynie", 0.0 },
  { "Tyniece", 0.0 },
  { "Tynieka", 0.0 },
  { "Tyniesha", 0.0 },
  { "Tynija", 0.0 },
  { "Tynijah", 0.0 },
  { "Tynika", 0.0 },
  { "Tyniqua", 0.0 },
  { "Tynique", 0.0 },
  { "Tynisa", 0.0 },
  { "Tynise", 0.0 },
  { "Tynisha", 0.0 },
  { "Tynishia", 0.0 },
  { "Tynisia", 0.0 },
  { "Tynita", 0.0 },
  { "Tyniya", 0.0 },
  { "Tyniyah", 0.0 },
  { "Tynlee", 0.0 },
  { "Tynleigh", 0.0 },
  { "Tynley", 0.0 },
  { "Tynli", 0.0 },
  { "Tynlie", 0.0 },
  { "Tynnetta", 0.0 },
  { "Tynslee", 0.0 },
  { "Tynsley", 0.0 },
  { "Tynzlee", 0.0 },
  { "Tyon", 1.0 },
  { "Tyona", 0.0 },
  { "Tyone", 1.0 },
  { "Tyonia", 0.0 },
  { "Tyonna", 0.0 },
  { "Tyonnah", 0.0 },
  { "Tyonne", 0.67346936 },
  { "Typhani", 0.0 },
  { "Typhanie", 0.0 },
  { "Tyquail", 1.0 },
  { "Tyquan", 0.99707603 },
  { "Tyquana", 0.0 },
  { "Tyquane", 1.0 },
  { "Tyquann", 1.0 },
  { "Tyquanna", 0.0 },
  { "Tyquarious", 1.0 },
  { "Tyquarius", 1.0 },
  { "Tyquasha", 0.0 },
  { "Tyquashia", 0.0 },
  { "Tyquasia", 0.0 },
  { "Tyquavion", 1.0 },
  { "Tyquavious", 1.0 },
  { "Tyquavius", 1.0 },
  { "Tyquawn", 1.0 },
  { "Tyquell", 1.0 },
  { "Tyquese", 1.0 },
  { "Tyquesha", 0.0 },
  { "Tyquez", 1.0 },
  { "Tyquil", 1.0 },
  { "Tyquin", 1.0 },
  { "Tyquis", 1.0 },
  { "Tyquise", 1.0 },
  { "Tyquisha", 0.0 },
  { "Tyquita", 0.0 },
  { "Tyquon", 1.0 },
  { "Tyqwan", 1.0 },
  { "Tyr", 1.0 },
  { "Tyra", 0.009875513 },
  { "Tyrae", 0.8982558 },
  { "Tyrael", 1.0 },
  { "Tyrah", 0.0 },
  { "Tyrail", 1.0 },
  { "Tyral", 1.0 },
  { "Tyrale", 1.0 },
  { "Tyran", 0.97390497 },
  { "Tyranae", 0.0 },
  { "Tyrance", 1.0 },
  { "Tyrand", 1.0 },
  { "Tyrane", 1.0 },
  { "Tyranesha", 0.0 },
  { "Tyranika", 0.0 },
  { "Tyranique", 0.0 },
  { "Tyranisha", 0.0 },
  { "Tyrann", 1.0 },
  { "Tyranny", 0.0 },
  { "Tyras", 1.0 },
  { "Tyray", 1.0 },
  { "Tyraya", 0.0 },
  { "Tyre", 0.975574 },
  { "Tyrea", 0.12871288 },
  { "Tyreace", 1.0 },
  { "Tyreak", 1.0 },
  { "Tyreana", 0.0 },
  { "Tyreanna", 0.0 },
  { "Tyrease", 1.0 },
  { "Tyrec", 1.0 },
  { "Tyrece", 1.0 },
  { "Tyrecia", 0.0 },
  { "Tyreck", 1.0 },
  { "Tyreco", 1.0 },
  { "Tyree", 0.94894767 },
  { "Tyreec", 1.0 },
  { "Tyreece", 1.0 },
  { "Tyreeck", 1.0 },
  { "Tyreef", 1.0 },
  { "Tyreek", 1.0 },
  { "Tyreeka", 0.0 },
  { "Tyreem", 1.0 },
  { "Tyreeq", 1.0 },
  { "Tyrees", 1.0 },
  { "Tyreese", 0.97396386 },
  { "Tyreesha", 0.0 },
  { "Tyrei", 1.0 },
  { "Tyreice", 1.0 },
  { "Tyreick", 1.0 },
  { "Tyreik", 1.0 },
  { "Tyreike", 1.0 },
  { "Tyreion", 1.0 },
  { "Tyreise", 1.0 },
  { "Tyreisha", 0.0 },
  { "Tyrek", 1.0 },
  { "Tyreka", 0.0 },
  { "Tyreke", 1.0 },
  { "Tyrekus", 1.0 },
  { "Tyrel", 1.0 },
  { "Tyrell", 0.9904215 },
  { "Tyrelle", 0.97979796 },
  { "Tyren", 0.9973615 },
  { "Tyrena", 0.0 },
  { "Tyrence", 1.0 },
  { "Tyrene", 0.0 },
  { "Tyreon", 1.0 },
  { "Tyreona", 0.0 },
  { "Tyreonna", 0.0 },
  { "Tyreq", 1.0 },
  { "Tyreque", 1.0 },
  { "Tyres", 1.0 },
  { "Tyresa", 0.0 },
  { "Tyrese", 0.9826291 },
  { "Tyresha", 0.0 },
  { "Tyreshia", 0.0 },
  { "Tyress", 1.0 },
  { "Tyresse", 1.0 },
  { "Tyrez", 1.0 },
  { "Tyrhiana", 0.0 },
  { "Tyrhianna", 0.0 },
  { "Tyrhonda", 0.0 },
  { "Tyri", 1.0 },
  { "Tyria", 0.017699115 },
  { "Tyriah", 0.0 },
  { "Tyrian", 0.9577465 },
  { "Tyriana", 0.0 },
  { "Tyrianna", 0.0 },
  { "Tyrianne", 0.0 },
  { "Tyric", 1.0 },
  { "Tyrica", 0.0 },
  { "Tyrice", 0.9693446 },
  { "Tyrick", 1.0 },
  { "Tyricka", 0.0 },
  { "Tyrico", 1.0 },
  { "Tyrie", 0.9413448 },
  { "Tyriece", 1.0 },
  { "Tyriek", 1.0 },
  { "Tyriel", 0.0 },
  { "Tyrielle", 0.0 },
  { "Tyriese", 1.0 },
  { "Tyrihanna", 0.0 },
  { "Tyrik", 1.0 },
  { "Tyrika", 0.0 },
  { "Tyrike", 1.0 },
  { "Tyrin", 0.99355596 },
  { "Tyrina", 0.0 },
  { "Tyrine", 0.0 },
  { "Tyrion", 0.98383373 },
  { "Tyriona", 0.0 },
  { "Tyrionna", 0.0 },
  { "Tyriq", 1.0 },
  { "Tyrique", 1.0 },
  { "Tyris", 1.0 },
  { "Tyrise", 1.0 },
  { "Tyrisha", 0.0 },
  { "Tyrissa", 0.0 },
  { "Tyrita", 0.0 },
  { "Tyrius", 1.0 },
  { "Tyrod", 1.0 },
  { "Tyrome", 1.0 },
  { "Tyron", 0.9977949 },
  { "Tyrona", 0.0 },
  { "Tyronda", 0.0 },
  { "Tyrone", 0.99099725 },
  { "Tyronesha", 0.0 },
  { "Tyronica", 0.0 },
  { "Tyronn", 1.0 },
  { "Tyronne", 1.0 },
  { "Tyronza", 0.05263158 },
  { "Tyronzia", 0.0 },
  { "Tyrra", 0.0 },
  { "Tyrrel", 1.0 },
  { "Tyrrell", 1.0 },
  { "Tyrun", 1.0 },
  { "Tyrus", 1.0 },
  { "Tyruss", 1.0 },
  { "Tyryn", 1.0 },
  { "Tysa", 0.0 },
  { "Tysaiah", 1.0 },
  { "Tysan", 1.0 },
  { "Tysaun", 1.0 },
  { "Tyse", 1.0 },
  { "Tysean", 1.0 },
  { "Tyseana", 0.0 },
  { "Tysen", 0.99285203 },
  { "Tysha", 0.0 },
  { "Tyshae", 0.0 },
  { "Tyshai", 0.0 },
  { "Tyshan", 1.0 },
  { "Tyshana", 0.0 },
  { "Tyshanna", 0.0 },
  { "Tyshanta", 0.0 },
  { "Tyshanti", 0.0 },
  { "Tyshara", 0.0 },
  { "Tyshaun", 0.99716556 },
  { "Tyshauna", 0.0 },
  { "Tyshawn", 0.97254664 },
  { "Tyshawna", 0.0 },
  { "Tyshawnna", 0.0 },
  { "Tyshay", 0.0 },
  { "Tyshayla", 0.0 },
  { "Tyshe", 0.0 },
  { "Tyshea", 0.0 },
  { "Tysheana", 0.0 },
  { "Tysheanna", 0.0 },
  { "Tysheed", 1.0 },
  { "Tysheem", 1.0 },
  { "Tysheema", 0.0 },
  { "Tysheen", 1.0 },
  { "Tysheena", 0.0 },
  { "Tysheera", 0.0 },
  { "Tysheia", 0.0 },
  { "Tysheika", 0.0 },
  { "Tysheim", 1.0 },
  { "Tysheka", 0.0 },
  { "Tyshekia", 0.0 },
  { "Tyshell", 0.0 },
  { "Tyshelle", 0.0 },
  { "Tyshema", 0.0 },
  { "Tysheonna", 0.0 },
  { "Tyshera", 0.0 },
  { "Tysheria", 0.0 },
  { "Tyshia", 0.0 },
  { "Tyshiana", 0.0 },
  { "Tyshianna", 0.0 },
  { "Tyshiem", 1.0 },
  { "Tyshika", 0.0 },
  { "Tyshira", 0.0 },
  { "Tyshon", 0.99153847 },
  { "Tyshonda", 0.0 },
  { "Tyshone", 1.0 },
  { "Tyshonna", 0.0 },
  { "Tyshun", 1.0 },
  { "Tysie", 0.0 },
  { "Tysin", 1.0 },
  { "Tysir", 1.0 },
  { "Tyski", 1.0 },
  { "Tyson", 0.9925039 },
  { "Tysun", 1.0 },
  { "Tysyn", 1.0 },
  { "Tytan", 1.0 },
  { "Tytana", 0.0 },
  { "Tytanna", 0.0 },
  { "Tyteana", 0.0 },
  { "Tyteanna", 0.0 },
  { "Tyten", 1.0 },
  { "Tyteona", 0.0 },
  { "Tyteonna", 0.0 },
  { "Tytiana", 0.0 },
  { "Tytianna", 0.0 },
  { "Tytiauna", 0.0 },
  { "Tytin", 1.0 },
  { "Tytiona", 0.0 },
  { "Tytionna", 0.0 },
  { "Tytiyana", 0.0 },
  { "Tyton", 1.0 },
  { "Tytrell", 1.0 },
  { "Tytus", 1.0 },
  { "Tyuana", 0.0 },
  { "Tyun", 1.0 },
  { "Tyunna", 0.0 },
  { "Tyus", 1.0 },
  { "Tyvaeh", 0.0 },
  { "Tyvan", 1.0 },
  { "Tyvaughn", 1.0 },
  { "Tyvell", 1.0 },
  { "Tyven", 1.0 },
  { "Tyveon", 1.0 },
  { "Tyvez", 1.0 },
  { "Tyvin", 1.0 },
  { "Tyvion", 1.0 },
  { "Tyvon", 1.0 },
  { "Tyvone", 1.0 },
  { "Tyvonna", 0.0 },
  { "Tyvonne", 1.0 },
  { "Tywain", 1.0 },
  { "Tywan", 0.9877351 },
  { "Tywana", 0.0 },
  { "Tywand", 1.0 },
  { "Tywanda", 0.0 },
  { "Tywane", 1.0 },
  { "Tywania", 0.0 },
  { "Tywann", 1.0 },
  { "Tywanna", 0.0 },
  { "Tywaun", 1.0 },
  { "Tywayne", 1.0 },
  { "Tywin", 1.0 },
  { "Tywon", 1.0 },
  { "Tywona", 0.0 },
  { "Tywonda", 0.0 },
  { "Tywone", 1.0 },
  { "Tywuan", 1.0 },
  { "Tyyana", 0.0 },
  { "Tyyanna", 0.0 },
  { "Tyyne", 0.0 },
  { "Tyyon", 1.0 },
  { "Tyyonna", 0.0 },
  { "Tyzae", 1.0 },
  { "Tyzaiah", 1.0 },
  { "Tyzell", 1.0 },
  { "Tyzen", 1.0 },
  { "Tyzer", 1.0 },
  { "Tyzhane", 0.0 },
  { "Tyzion", 1.0 },
  { "Tyzir", 1.0 },
  { "Tyzjuan", 1.0 },
  { "Tyzon", 1.0 },
  { "Tziah", 0.76 },
  { "Tzion", 1.0 },
  { "Tzipa", 0.0 },
  { "Tzipora", 0.0 },
  { "Tziporah", 0.0 },
  { "Tzippora", 0.0 },
  { "Tzipporah", 0.0 },
  { "Tzippy", 0.0 },
  { "Tzirel", 0.0 },
  { "Tziry", 0.0 },
  { "Tzivi", 0.0 },
  { "Tzivia", 0.0 },
  { "Tzivy", 0.0 },
  { "Tzivya", 0.0 },
  { "Tzofia", 0.0 },
  { "Tzuriel", 1.0 },
  { "Tzurty", 0.0 },
  { "Tzvi", 1.0 },
  { "Ualani", 0.0 },
  { "Uarda", 0.0 },
  { "Uba", 0.0 },
  { "Ubah", 0.0 },
  { "Ubaid", 1.0 },
  { "Ubaidullah", 1.0 },
  { "Ubald", 1.0 },
  { "Ubaldo", 1.0 },
  { "Uber", 1.0 },
  { "Ubong", 1.0 },
  { "Uche", 1.0 },
  { "Uchechi", 0.0 },
  { "Uchechukwu", 0.82978725 },
  { "Uchenna", 0.6666667 },
  { "Uciel", 1.0 },
  { "Uda", 0.0 },
  { "Udana", 0.0 },
  { "Uday", 1.0 },
  { "Udbhav", 1.0 },
  { "Udelia", 0.0 },
  { "Udell", 0.7849117 },
  { "Udella", 0.0 },
  { "Udit", 1.0 },
  { "Udochukwu", 1.0 },
  { "Udona", 0.0 },
  { "Udonis", 1.0 },
  { "Udonna", 0.0 },
  { "Udora", 0.0 },
  { "Udy", 0.0 },
  { "Uel", 1.0 },
  { "Ufemia", 0.0 },
  { "Uganda", 0.20689656 },
  { "Ugene", 1.0 },
  { "Ugne", 0.0 },
  { "Ugo", 1.0 },
  { "Ugochi", 0.0 },
  { "Ugochukwu", 1.0 },
  { "Ugonna", 0.73888886 },
  { "Uhl", 1.0 },
  { "Uhura", 0.0 },
  { "Uhuru", 0.8484849 },
  { "Uilani", 0.0 },
  { "Uilliam", 1.0 },
  { "Ujala", 0.0 },
  { "Ujjwal", 1.0 },
  { "Ujwal", 1.0 },
  { "Ukari", 0.0 },
  { "Ukiah", 0.9302326 },
  { "Ula", 0.019195046 },
  { "Uladimir", 1.0 },
  { "Ulah", 0.0 },
  { "Ulan", 1.0 },
  { "Ulana", 0.0 },
  { "Uland", 1.0 },
  { "Ulanda", 0.0 },
  { "Ulani", 0.0 },
  { "Ulas", 1.0 },
  { "Uldean", 0.0 },
  { "Uldene", 0.0 },
  { "Uldine", 0.0 },
  { "Uldis", 1.0 },
  { "Ules", 1.0 },
  { "Uless", 1.0 },
  { "Ulesses", 1.0 },
  { "Uleta", 0.0 },
  { "Uli", 0.0 },
  { "Uliana", 0.0 },
  { "Ulice", 1.0 },
  { "Ulices", 1.0 },
  { "Ulis", 1.0 },
  { "Ulisa", 0.0 },
  { "Ulise", 1.0 },
  { "Ulises", 0.9984708 },
  { "Ulisess", 1.0 },
  { "Ulisis", 1.0 },
  { "Ulissa", 0.0 },
  { "Ulisses", 1.0 },
  { "Ulizes", 1.0 },
  { "Ulla", 0.0 },
  { "Ullanda", 0.0 },
  { "Ulma", 0.0 },
  { "Ulmer", 1.0 },
  { "Ulonda", 0.0 },
  { "Ulric", 1.0 },
  { "Ulrica", 0.0 },
  { "Ulrich", 1.0 },
  { "Ulrick", 1.0 },
  { "Ulrik", 1.0 },
  { "Ulrika", 0.0 },
  { "Ulrike", 0.0 },
  { "Ulus", 1.0 },
  { "Ulva", 0.0 },
  { "Ulyana", 0.0 },
  { "Ulyess", 1.0 },
  { "Ulyesses", 1.0 },
  { "Ulys", 1.0 },
  { "Ulysee", 1.0 },
  { "Ulysees", 1.0 },
  { "Ulyses", 1.0 },
  { "Ulysess", 1.0 },
  { "Ulysis", 1.0 },
  { "Ulyss", 1.0 },
  { "Ulyssa", 0.0 },
  { "Ulysse", 1.0 },
  { "Ulyssee", 1.0 },
  { "Ulysses", 0.9985374 },
  { "Ulyssess", 1.0 },
  { "Ulyssia", 0.0 },
  { "Ulyssis", 1.0 },
  { "Uma", 0.0 },
  { "Umaima", 0.0 },
  { "Umair", 1.0 },
  { "Umaira", 0.0 },
  { "Umaiza", 0.0 },
  { "Umama", 0.0 },
  { "Umamah", 0.0 },
  { "Umang", 1.0 },
  { "Umar", 1.0 },
  { "Umarbek", 1.0 },
  { "Umari", 1.0 },
  { "Umaya", 0.0 },
  { "Umayah", 0.0 },
  { "Umayma", 0.0 },
  { "Umber", 0.0 },
  { "Umbert", 1.0 },
  { "Umberto", 1.0 },
  { "Ume", 0.0 },
  { "Umeka", 0.0 },
  { "Umeki", 0.0 },
  { "Umekia", 0.0 },
  { "Umeko", 0.0 },
  { "Umeno", 0.0 },
  { "Umer", 1.0 },
  { "Umesh", 1.0 },
  { "Umeyma", 0.0 },
  { "Umeyo", 0.0 },
  { "Umi", 0.0 },
  { "Umika", 0.0 },
  { "Umit", 1.0 },
  { "Umme", 0.0 },
  { "Ummehani", 0.0 },
  { "Umoja", 1.0 },
  { "Umrah", 0.0 },
  { "Umu", 0.0 },
  { "Umut", 1.0 },
  { "Un", 0.5 },
  { "Una", 0.0 },
  { "Unborn", 0.2 },
  { "Underwood", 1.0 },
  { "Undine", 0.0 },
  { "Undra", 0.8815166 },
  { "Undrae", 1.0 },
  { "Undray", 1.0 },
  { "Undre", 1.0 },
  { "Undrea", 0.2734375 },
  { "Undria", 0.0 },
  { "Uneeda", 0.0 },
  { "Uneek", 0.0 },
  { "Uneka", 0.0 },
  { "Unia", 0.0 },
  { "Uniah", 0.0 },
  { "Unica", 0.0 },
  { "Unice", 0.09090909 },
  { "Unika", 0.0 },
  { "Union", 1.0 },
  { "Uniqua", 0.0 },
  { "Unique", 0.10032582 },
  { "Uniquea", 0.0 },
  { "Uniqueka", 0.0 },
  { "Uniquewa", 0.0 },
  { "Uniquia", 0.0 },
  { "Unita", 0.0 },
  { "Unity", 0.0 },
  { "Uniya", 0.0 },
  { "Uniyah", 0.0 },
  { "Unk", 0.6851852 },
  { "Unknown", 0.49653432 },
  { "Unkown", 0.53846157 },
  { "Unnamed", 0.5706897 },
  { "Uno", 1.0 },
  { "Unseld", 1.0 },
  { "Unto", 1.0 },
  { "Unyque", 0.0 },
  { "Upshur", 1.0 },
  { "Upton", 1.0 },
  { "Ura", 0.11343805 },
  { "Uraina", 0.0 },
  { "Ural", 1.0 },
  { "Urana", 0.0 },
  { "Urania", 0.0 },
  { "Uranus", 0.0 },
  { "Urbain", 1.0 },
  { "Urban", 1.0 },
  { "Urbana", 0.0 },
  { "Urbane", 1.0 },
  { "Urbano", 1.0 },
  { "Urbin", 1.0 },
  { "Ureka", 0.0 },
  { "Urena", 0.0 },
  { "Urenna", 0.0 },
  { "Uretta", 0.0 },
  { "Urey", 1.0 },
  { "Urho", 1.0 },
  { "Urhonda", 0.0 },
  { "Uri", 0.9512712 },
  { "Uria", 1.0 },
  { "Uriah", 0.92266124 },
  { "Urian", 1.0 },
  { "Uriana", 0.0 },
  { "Urias", 1.0 },
  { "Urica", 0.0 },
  { "Urie", 1.0 },
  { "Uriel", 0.98741615 },
  { "Urielle", 0.0 },
  { "Urijah", 0.9711081 },
  { "Urijha", 1.0 },
  { "Urika", 0.0 },
  { "Urina", 0.0 },
  { "Urith", 0.0 },
  { "Uriyah", 0.7218935 },
  { "Urja", 0.0 },
  { "Urlin", 1.0 },
  { "Urline", 0.0 },
  { "Urmi", 0.0 },
  { "Urmila", 0.0 },
  { "Uronda", 0.0 },
  { "Urooj", 0.0 },
  { "Uroosa", 0.0 },
  { "Uros", 1.0 },
  { "Ursa", 0.0 },
  { "Ursala", 0.0 },
  { "Ursaline", 0.0 },
  { "Ursel", 0.23809524 },
  { "Ursela", 0.0 },
  { "Ursla", 0.0 },
  { "Ursula", 0.0 },
  { "Ursule", 0.0 },
  { "Ursuline", 0.0 },
  { "Ursulla", 0.0 },
  { "Urszula", 0.0 },
  { "Urvi", 0.0 },
  { "Urwa", 0.0 },
  { "Uryah", 1.0 },
  { "Usama", 1.0 },
  { "Usamah", 1.0 },
  { "Usayd", 1.0 },
  { "Usbaldo", 1.0 },
  { "Usef", 1.0 },
  { "Usha", 0.0 },
  { "Usher", 1.0 },
  { "Ushna", 0.0 },
  { "Usiel", 1.0 },
  { "Usmaan", 1.0 },
  { "Usman", 1.0 },
  { "Usra", 0.0 },
  { "Ustin", 1.0 },
  { "Usvaldo", 1.0 },
  { "Uswa", 0.0 },
  { "Uta", 0.0 },
  { "Utah", 1.0 },
  { "Utahna", 0.0 },
  { "Utako", 0.0 },
  { "Ute", 0.0 },
  { "Utha", 0.0 },
  { "Uthman", 1.0 },
  { "Utica", 0.0 },
  { "Utkarsh", 1.0 },
  { "Utopia", 0.0 },
  { "Utsav", 1.0 },
  { "Uuno", 1.0 },
  { "Uva", 0.0 },
  { "Uvaldo", 1.0 },
  { "Uver", 1.0 },
  { "Uvonka", 0.0 },
  { "Uwais", 1.0 },
  { "Uwe", 1.0 },
  { "Uyen", 0.0 },
  { "Uyiosa", 1.0 },
  { "Uyless", 1.0 },
  { "Uzair", 1.0 },
  { "Uzay", 1.0 },
  { "Uzayr", 1.0 },
  { "Uzella", 0.0 },
  { "Uziah", 1.0 },
  { "Uzias", 1.0 },
  { "Uziel", 1.0 },
  { "Uzma", 0.0 },
  { "Uzoamaka", 0.0 },
  { "Uzoma", 0.93333334 },
  { "Uzuri", 0.0 },
  { "Uzziah", 0.98747766 },
  { "Uzziel", 1.0 },
  { "Va", 1.0 },
  { "Vaahin", 1.0 },
  { "Vaani", 0.0 },
  { "Vaanya", 0.0 },
  { "Vachan", 1.0 },
  { "Vache", 1.0 },
  { "Vachel", 1.0 },
  { "Vachon", 1.0 },
  { "Vacie", 0.0 },
  { "Vaclav", 1.0 },
  { "Vada", 0.015643105 },
  { "Vadah", 0.0 },
  { "Vadal", 1.0 },
  { "Vadell", 1.0 },
  { "Vaden", 0.98767966 },
  { "Vader", 0.10204082 },
  { "Vadhir", 1.0 },
  { "Vadia", 0.0 },
  { "Vadie", 0.0 },
  { "Vadim", 1.0 },
  { "Vadin", 1.0 },
  { "Vadir", 1.0 },
  { "Vadis", 0.057471264 },
  { "Vadna", 0.0 },
  { "Vadra", 0.0 },
  { "Vadys", 0.0 },
  { "Vae", 0.0 },
  { "Vaeda", 0.0 },
  { "Vaelyn", 0.0 },
  { "Vagas", 1.0 },
  { "Vahagn", 1.0 },
  { "Vahan", 1.0 },
  { "Vahe", 1.0 },
  { "Vahid", 1.0 },
  { "Vahin", 1.0 },
  { "Vahn", 1.0 },
  { "Vahram", 1.0 },
  { "Vai", 1.0 },
  { "Vaibhav", 1.0 },
  { "Vaibhavi", 0.0 },
  { "Vaida", 0.0 },
  { "Vaidehi", 0.0 },
  { "Vaiden", 1.0 },
  { "Vaiga", 0.0 },
  { "Vail", 0.3 },
  { "Vaila", 0.0 },
  { "Vaile", 0.0 },
  { "Vaimoana", 0.0 },
  { "Vainavi", 0.0 },
  { "Vaino", 1.0 },
  { "Vaira", 0.0 },
  { "Vairon", 1.0 },
  { "Vaishali", 0.0 },
  { "Vaishnav", 1.0 },
  { "Vaishnavi", 0.0 },
  { "Vaishvi", 0.0 },
  { "Vaitea", 0.0 },
  { "Vaitiare", 0.0 },
  { "Vakesha", 0.0 },
  { "Val", 0.7618982 },
  { "Vala", 0.0 },
  { "Valada", 0.0 },
  { "Valaida", 0.0 },
  { "Valaire", 0.0 },
  { "Valancia", 0.0 },
  { "Valanda", 0.0 },
  { "Valaree", 0.0 },
  { "Valari", 0.0 },
  { "Valaria", 0.0 },
  { "Valarie", 0.0 },
  { "Valary", 0.0 },
  { "Valasia", 0.0 },
  { "Valaska", 0.0 },
  { "Valasta", 0.0 },
  { "Valaysia", 0.0 },
  { "Valbona", 0.0 },
  { "Valborg", 0.0 },
  { "Valda", 0.0 },
  { "Valdean", 0.54545456 },
  { "Valdemar", 1.0 },
  { "Valdene", 0.0 },
  { "Valder", 1.0 },
  { "Valdez", 1.0 },
  { "Valdimir", 1.0 },
  { "Valdine", 0.0 },
  { "Valdir", 1.0 },
  { "Valdis", 1.0 },
  { "Valdo", 1.0 },
  { "Valdon", 1.0 },
  { "Vale", 0.4573991 },
  { "Valecia", 0.0 },
  { "Valeda", 0.0 },
  { "Valeen", 0.0 },
  { "Valeire", 0.0 },
  { "Valek", 1.0 },
  { "Valeka", 0.0 },
  { "Valen", 0.6540484 },
  { "Valena", 0.0 },
  { "Valencia", 0.0010204081 },
  { "Valencio", 1.0 },
  { "Valency", 0.0 },
  { "Valenda", 0.0 },
  { "Valene", 0.0 },
  { "Valenica", 0.0 },
  { "Valente", 1.0 },
  { "Valentia", 0.0 },
  { "Valentin", 1.0 },
  { "Valentina", 0.00023171902 },
  { "Valentine", 0.7748236 },
  { "Valentinna", 0.0 },
  { "Valentino", 1.0 },
  { "Valentyna", 0.0 },
  { "Valera", 0.0 },
  { "Valere", 0.0 },
  { "Valeree", 0.0 },
  { "Valeri", 0.0 },
  { "Valeria", 0.0016651336 },
  { "Valerian", 0.98356164 },
  { "Valeriana", 0.0 },
  { "Valeriano", 1.0 },
  { "Valerie", 0.002705885 },
  { "Valerieann", 0.0 },
  { "Valerieanne", 0.0 },
  { "Valerine", 0.0 },
  { "Valerio", 1.0 },
  { "Valeriy", 1.0 },
  { "Valeriya", 0.0 },
  { "Valery", 0.015873017 },
  { "Valerya", 0.0 },
  { "Valerye", 0.0 },
  { "Valesha", 0.0 },
  { "Valeshia", 0.0 },
  { "Valesia", 0.0 },
  { "Valeska", 0.0 },
  { "Valesta", 0.0 },
  { "Valeta", 0.0 },
  { "Valetina", 0.0 },
  { "Valetta", 0.0 },
  { "Valette", 0.0 },
  { "Valgene", 0.94148934 },
  { "Vali", 0.08928572 },
  { "Valia", 0.0 },
  { "Valiant", 1.0 },
  { "Valice", 1.0 },
  { "Valicia", 0.0 },
  { "Valicity", 0.0 },
  { "Valier", 0.0 },
  { "Valiere", 0.0 },
  { "Valik", 1.0 },
  { "Valin", 0.9590164 },
  { "Valina", 0.0 },
  { "Valincia", 0.0 },
  { "Valinda", 0.0 },
  { "Valine", 0.0 },
  { "Valire", 0.0 },
  { "Valiree", 0.0 },
  { "Valirie", 0.0 },
  { "Valisa", 0.0 },
  { "Valisha", 0.0 },
  { "Valissa", 0.0 },
  { "Valita", 0.0 },
  { "Valiyah", 0.0 },
  { "Valjean", 0.08405172 },
  { "Valkyrie", 0.0 },
  { "Valla", 0.0 },
  { "Vallarie", 0.0 },
  { "Valle", 0.0 },
  { "Vallee", 0.375 },
  { "Vallen", 0.7638889 },
  { "Vallene", 0.0 },
  { "Valleri", 0.0 },
  { "Valleria", 0.0 },
  { "Vallerie", 0.0 },
  { "Vallery", 0.0 },
  { "Valley", 0.12391931 },
  { "Valli", 0.0 },
  { "Vallie", 0.043624163 },
  { "Vallon", 0.3125 },
  { "Vallorie", 0.0 },
  { "Vallory", 0.0 },
  { "Vally", 0.0 },
  { "Vallyn", 0.0 },
  { "Valma", 0.0 },
  { "Valmond", 1.0 },
  { "Valmore", 1.0 },
  { "Valo", 1.0 },
  { "Valois", 0.0 },
  { "Valon", 0.97409326 },
  { "Valonda", 0.0 },
  { "Valor", 0.9100205 },
  { "Valora", 0.0 },
  { "Valoree", 0.0 },
  { "Valori", 0.0 },
  { "Valoria", 0.0 },
  { "Valorie", 0.0 },
  { "Valoris", 0.0 },
  { "Valory", 0.0 },
  { "Valoy", 0.0 },
  { "Valoyce", 0.0 },
  { "Valrea", 0.0 },
  { "Valree", 0.0 },
  { "Valri", 0.0 },
  { "Valrie", 0.0054704593 },
  { "Valta", 0.0 },
  { "Valton", 1.0 },
  { "Valyn", 0.0 },
  { "Valyncia", 0.0 },
  { "Valynda", 0.0 },
  { "Valynn", 0.0 },
  { "Valyre", 0.0 },
  { "Valyssa", 0.0 },
  { "Vamika", 0.0 },
  { "Vamsi", 1.0 },
  { "Van", 0.942277 },
  { "Vana", 0.0 },
  { "Vanae", 0.0 },
  { "Vananh", 0.0 },
  { "Vanasia", 0.0 },
  { "Vanassa", 0.0 },
  { "Vanaya", 0.0 },
  { "Vanburen", 1.0 },
  { "Vance", 0.99449056 },
  { "Vancil", 1.0 },
  { "Vancy", 0.0 },
  { "Vanda", 0.0 },
  { "Vandalia", 0.0 },
  { "Vandan", 1.0 },
  { "Vandana", 0.0 },
  { "Vandela", 0.0 },
  { "Vandelia", 0.0 },
  { "Vandell", 1.0 },
  { "Vandella", 0.0 },
  { "Vanden", 1.0 },
  { "Vander", 1.0 },
  { "Vanderbilt", 1.0 },
  { "Vanderlei", 1.0 },
  { "Vandetta", 0.0 },
  { "Vandi", 0.0 },
  { "Vandiver", 1.0 },
  { "Vandon", 1.0 },
  { "Vandora", 0.0 },
  { "Vandra", 0.0 },
  { "Vandrea", 0.0 },
  { "Vandy", 0.6077236 },
  { "Vane", 1.0 },
  { "Vanecia", 0.0 },
  { "Vaneda", 0.0 },
  { "Vaneesa", 0.0 },
  { "Vaneeza", 0.0 },
  { "Vaneisha", 0.0 },
  { "Vaneka", 0.0 },
  { "Vanella", 0.0 },
  { "Vanellope", 0.0 },
  { "Vanelly", 0.0 },
  { "Vanelope", 0.0 },
  { "Vanely", 0.0 },
  { "Vanesa", 0.00079503894 },
  { "Vanesha", 0.0 },
  { "Vaneshia", 0.0 },
  { "Vanesia", 0.0 },
  { "Vaness", 0.0 },
  { "Vanessa", 0.0039009484 },
  { "Vanessaanne", 0.0 },
  { "Vanessah", 0.0 },
  { "Vanessamarie", 0.0 },
  { "Vanesse", 0.0 },
  { "Vanessia", 0.0 },
  { "Vanesta", 0.0 },
  { "Vanester", 0.0 },
  { "Vaneta", 0.0 },
  { "Vanetta", 0.0 },
  { "Vanette", 0.0 },
  { "Vaneza", 0.0 },
  { "Vanezza", 0.0 },
  { "Vang", 0.9672619 },
  { "Vangela", 0.0 },
  { "Vangie", 0.0 },
  { "Vani", 0.0 },
  { "Vania", 0.0 },
  { "Vaniah", 0.0 },
  { "Vanice", 0.0 },
  { "Vanicia", 0.0 },
  { "Vanida", 0.0 },
  { "Vanie", 0.0 },
  { "Vaniecia", 0.0 },
  { "Vanika", 0.0 },
  { "Vanilla", 0.0 },
  { "Vanille", 0.0 },
  { "Vanis", 1.0 },
  { "Vanisa", 0.0 },
  { "Vanise", 0.0 },
  { "Vanisha", 0.0 },
  { "Vanissa", 0.0 },
  { "Vanita", 0.0 },
  { "Vanitie", 0.0 },
  { "Vanity", 0.0 },
  { "Vaniya", 0.0 },
  { "Vaniyah", 0.0 },
  { "Vanja", 0.0 },
  { "Vann", 0.9868781 },
  { "Vanna", 0.036464773 },
  { "Vannah", 0.0 },
  { "Vannak", 1.0 },
  { "Vannara", 1.0 },
  { "Vannary", 0.0 },
  { "Vanner", 1.0 },
  { "Vannesa", 0.0 },
  { "Vanness", 0.7222222 },
  { "Vannessa", 0.0 },
  { "Vannetta", 0.0 },
  { "Vannette", 0.0 },
  { "Vannia", 0.0 },
  { "Vannida", 0.0 },
  { "Vannie", 0.18627451 },
  { "Vanny", 0.24615385 },
  { "Vannya", 0.0 },
  { "Vanora", 0.0 },
  { "Vansh", 1.0 },
  { "Vanshika", 0.0 },
  { "Vanshon", 1.0 },
  { "Vanson", 1.0 },
  { "Vantasia", 0.0 },
  { "Vanteen", 0.0 },
  { "Vantrell", 1.0 },
  { "Vantrice", 0.0 },
  { "Vanus", 1.0 },
  { "Vanya", 0.0 },
  { "Vanyah", 0.0 },
  { "Vara", 0.0 },
  { "Varad", 1.0 },
  { "Vard", 1.0 },
  { "Varda", 0.0 },
  { "Vardaan", 1.0 },
  { "Vardaman", 1.0 },
  { "Vardan", 1.0 },
  { "Vardell", 1.0 },
  { "Varden", 1.0 },
  { "Vardhan", 1.0 },
  { "Vardie", 1.0 },
  { "Vareck", 1.0 },
  { "Varee", 0.0 },
  { "Varek", 1.0 },
  { "Varen", 1.0 },
  { "Varena", 0.0 },
  { "Varenya", 0.0 },
  { "Varetta", 0.0 },
  { "Variah", 0.0 },
  { "Varian", 0.98381877 },
  { "Varick", 1.0 },
  { "Varie", 0.0 },
  { "Varik", 1.0 },
  { "Varin", 1.0 },
  { "Varina", 0.0 },
  { "Varinia", 0.0 },
  { "Varion", 1.0 },
  { "Varish", 1.0 },
  { "Varna", 0.0 },
  { "Varnell", 0.64748204 },
  { "Varner", 1.0 },
  { "Varney", 1.0 },
  { "Varnie", 0.0 },
  { "Varnika", 0.0 },
  { "Varnum", 1.0 },
  { "Varon", 1.0 },
  { "Varonda", 0.0 },
  { "Varonica", 0.0 },
  { "Varsha", 0.0 },
  { "Varshika", 0.0 },
  { "Varshini", 0.0 },
  { "Varshita", 0.0 },
  { "Varshitha", 0.0 },
  { "Vartan", 1.0 },
  { "Varun", 1.0 },
  { "Varvara", 0.0 },
  { "Vasanth", 1.0 },
  { "Vasanti", 0.0 },
  { "Vasco", 1.0 },
  { "Vasean", 1.0 },
  { "Vash", 1.0 },
  { "Vasha", 0.0 },
  { "Vashanti", 0.0 },
  { "Vashaun", 0.9714286 },
  { "Vashawn", 0.93491125 },
  { "Vashion", 0.6875 },
  { "Vashni", 0.0 },
  { "Vashon", 0.91544116 },
  { "Vashonda", 0.0 },
  { "Vashone", 1.0 },
  { "Vashti", 0.0 },
  { "Vashtie", 0.0 },
  { "Vasia", 0.0 },
  { "Vasil", 1.0 },
  { "Vasile", 1.0 },
  { "Vasili", 1.0 },
  { "Vasilia", 0.0 },
  { "Vasilije", 1.0 },
  { "Vasilike", 0.0 },
  { "Vasiliki", 0.0 },
  { "Vasilios", 1.0 },
  { "Vasilis", 1.0 },
  { "Vasilisa", 0.0 },
  { "Vasiliy", 1.0 },
  { "Vasily", 1.0 },
  { "Vasisht", 1.0 },
  { "Vasken", 1.0 },
  { "Vaso", 1.0 },
  { "Vassar", 1.0 },
  { "Vassie", 0.0 },
  { "Vassilios", 1.0 },
  { "Vasta", 0.0 },
  { "Vasthi", 0.0 },
  { "Vasti", 0.0 },
  { "Vastie", 0.0 },
  { "Vastine", 1.0 },
  { "Vasu", 1.0 },
  { "Vasudev", 1.0 },
  { "Vasudha", 0.0 },
  { "Vatasha", 0.0 },
  { "Vatche", 1.0 },
  { "Vatsal", 1.0 },
  { "Vauda", 0.0 },
  { "Vaudie", 0.0 },
  { "Vaudine", 0.0 },
  { "Vaudis", 0.0 },
  { "Vaugh", 1.0 },
  { "Vaughan", 0.985705 },
  { "Vaughn", 0.9546512 },
  { "Vaughna", 0.0 },
  { "Vaun", 1.0 },
  { "Vauna", 0.0 },
  { "Vaunda", 0.0 },
  { "Vaune", 0.0 },
  { "Vaunita", 0.0 },
  { "Vava", 0.0 },
  { "Vawn", 0.0 },
  { "Vay", 0.0 },
  { "Vaya", 0.0 },
  { "Vayah", 0.0 },
  { "Vayda", 0.0 },
  { "Vayden", 1.0 },
  { "Vayla", 0.0 },
  { "Vaylen", 1.0 },
  { "Vayne", 1.0 },
  { "Vayron", 1.0 },
  { "Veachel", 1.0 },
  { "Veanna", 0.0 },
  { "Veanne", 0.0 },
  { "Vear", 0.0 },
  { "Vearl", 0.8661568 },
  { "Veasna", 1.0 },
  { "Veatrice", 0.0 },
  { "Ved", 1.0 },
  { "Veda", 0.0 },
  { "Vedaant", 1.0 },
  { "Vedad", 1.0 },
  { "Vedansh", 1.0 },
  { "Vedanshi", 0.0 },
  { "Vedant", 1.0 },
  { "Vedanth", 1.0 },
  { "Vedat", 1.0 },
  { "Vedder", 1.0 },
  { "Vedette", 0.0 },
  { "Vedh", 1.0 },
  { "Vedha", 0.0 },
  { "Vedhika", 0.0 },
  { "Vedia", 0.0 },
  { "Vedika", 0.0 },
  { "Vedra", 0.0 },
  { "Vee", 0.037453182 },
  { "Veeda", 0.0 },
  { "Veeha", 0.0 },
  { "Veeksha", 0.0 },
  { "Veena", 0.0 },
  { "Veer", 1.0 },
  { "Veera", 0.0 },
  { "Veeraj", 1.0 },
  { "Veeta", 0.0 },
  { "Veeya", 0.0 },
  { "Vega", 0.17316018 },
  { "Vegas", 0.63258785 },
  { "Vegeta", 1.0 },
  { "Veida", 0.0 },
  { "Veikko", 1.0 },
  { "Veira", 0.0 },
  { "Vel", 0.45833334 },
  { "Vela", 0.0 },
  { "Velah", 0.0 },
  { "Velan", 1.0 },
  { "Velanie", 0.0 },
  { "Velarie", 0.0 },
  { "Velcie", 0.0 },
  { "Velda", 0.0 },
  { "Veldia", 0.0 },
  { "Veldon", 1.0 },
  { "Velecia", 0.0 },
  { "Veleda", 0.0 },
  { "Veleka", 0.0 },
  { "Velen", 0.0 },
  { "Velena", 0.0 },
  { "Velencia", 0.0 },
  { "Velera", 0.0 },
  { "Veleria", 0.0 },
  { "Velerie", 0.0 },
  { "Veleta", 0.0 },
  { "Veletta", 0.0 },
  { "Velia", 0.0 },
  { "Velicia", 0.0 },
  { "Velicity", 0.0 },
  { "Velika", 0.0 },
  { "Velina", 0.0 },
  { "Velinda", 0.0 },
  { "Velisa", 0.0 },
  { "Velisha", 0.0 },
  { "Velissa", 0.0 },
  { "Velita", 0.0 },
  { "Velkan", 1.0 },
  { "Vella", 0.0 },
  { "Velma", 0.005169799 },
  { "Velmar", 0.28947368 },
  { "Velmarie", 0.0 },
  { "Velmer", 0.46341464 },
  { "Velmo", 0.0 },
  { "Velmon", 0.0 },
  { "Velna", 0.0 },
  { "Velois", 0.0 },
  { "Velora", 0.0 },
  { "Veloria", 0.0 },
  { "Velouria", 0.0 },
  { "Velsie", 0.0 },
  { "Velta", 0.0 },
  { "Velton", 1.0 },
  { "Velva", 0.0 },
  { "Velveeta", 0.0 },
  { "Velvet", 0.0016688919 },
  { "Velvett", 0.0 },
  { "Velvetta", 0.0 },
  { "Velvette", 0.0 },
  { "Velvia", 0.0 },
  { "Velvie", 0.0 },
  { "Vema", 0.0 },
  { "Vena", 0.0 },
  { "Venancio", 1.0 },
  { "Venard", 1.0 },
  { "Venassa", 0.0 },
  { "Venba", 0.0 },
  { "Vence", 1.0 },
  { "Vencent", 1.0 },
  { "Vencie", 0.0 },
  { "Vencil", 1.0 },
  { "Venda", 0.0 },
  { "Vendela", 0.0 },
  { "Vendetta", 0.0 },
  { "Vendla", 0.0 },
  { "Venecia", 0.0 },
  { "Veneda", 0.0 },
  { "Veneita", 0.0 },
  { "Venellope", 0.0 },
  { "Venelope", 0.0 },
  { "Venera", 0.0 },
  { "Venesa", 0.0 },
  { "Venesha", 0.0 },
  { "Veneshia", 0.0 },
  { "Venesia", 0.0 },
  { "Venessa", 0.0 },
  { "Venessia", 0.0 },
  { "Venesta", 0.0 },
  { "Veneta", 0.0 },
  { "Venetia", 0.0 },
  { "Venetta", 0.0 },
  { "Venette", 0.0 },
  { "Venezia", 0.0 },
  { "Venia", 0.0 },
  { "Veniamin", 1.0 },
  { "Venica", 0.0 },
  { "Venice", 0.0748852 },
  { "Venicia", 0.0 },
  { "Venicio", 1.0 },
  { "Venida", 0.0 },
  { "Venie", 0.0 },
  { "Veniece", 0.0 },
  { "Venika", 0.0 },
  { "Venis", 0.18965517 },
  { "Venisa", 0.0 },
  { "Venise", 0.0 },
  { "Venisha", 0.0 },
  { "Venishia", 0.0 },
  { "Venissa", 0.0 },
  { "Venita", 0.0 },
  { "Venitra", 0.0 },
  { "Venitta", 0.0 },
  { "Venkat", 1.0 },
  { "Venna", 0.0 },
  { "Vennela", 0.0 },
  { "Vennesa", 0.0 },
  { "Vennessa", 0.0 },
  { "Vennetta", 0.0 },
  { "Vennie", 0.05035461 },
  { "Vennis", 1.0 },
  { "Venola", 0.0 },
  { "Venona", 0.0 },
  { "Venora", 0.0 },
  { "Venoy", 1.0 },
  { "Venson", 1.0 },
  { "Vent", 1.0 },
  { "Venton", 1.0 },
  { "Ventrell", 1.0 },
  { "Ventura", 0.84965986 },
  { "Venture", 1.0 },
  { "Ventus", 1.0 },
  { "Venus", 0.010244287 },
  { "Venya", 0.0 },
  { "Veo", 0.71428573 },
  { "Veola", 0.0 },
  { "Veolia", 0.0 },
  { "Veon", 1.0 },
  { "Veona", 0.0 },
  { "Veonica", 0.0 },
  { "Veonka", 0.0 },
  { "Veora", 0.0 },
  { "Vera", 0.0028752699 },
  { "Verabelle", 0.0 },
  { "Verah", 0.0 },
  { "Veral", 0.7903226 },
  { "Verald", 1.0 },
  { "Veralee", 0.0 },
  { "Veralyn", 0.0 },
  { "Veralynn", 0.0 },
  { "Veramae", 0.0 },
  { "Veranda", 0.0 },
  { "Verania", 0.0 },
  { "Verba", 0.0 },
  { "Verbal", 0.0 },
  { "Verbena", 0.0 },
  { "Verbia", 0.0 },
  { "Verbie", 0.026595745 },
  { "Verble", 0.4160584 },
  { "Verbon", 1.0 },
  { "Vercie", 0.0 },
  { "Verda", 0.00086751767 },
  { "Verdale", 1.0 },
  { "Verdean", 0.0 },
  { "Verdeen", 0.0 },
  { "Verdel", 0.5 },
  { "Verdell", 0.39124086 },
  { "Verdella", 0.0 },
  { "Verdelle", 0.0 },
  { "Verden", 1.0 },
  { "Verdena", 0.0 },
  { "Verdene", 0.0 },
  { "Verdi", 0.6875 },
  { "Verdia", 0.0 },
  { "Verdie", 0.07469027 },
  { "Verdina", 0.0 },
  { "Verdine", 0.0 },
  { "Verdis", 0.706383 },
  { "Verdo", 1.0 },
  { "Verdon", 1.0 },
  { "Verdonna", 0.0 },
  { "Verdun", 0.9101124 },
  { "Vere", 0.9190476 },
  { "Vereda", 0.0 },
  { "Veree", 0.0 },
  { "Vereen", 0.0 },
  { "Verena", 0.0 },
  { "Verenda", 0.0 },
  { "Verene", 0.0 },
  { "Verenice", 0.0 },
  { "Verenis", 0.0 },
  { "Verenise", 0.0 },
  { "Verenisse", 0.0 },
  { "Vereniz", 0.0 },
  { "Verenize", 0.0 },
  { "Veretta", 0.0 },
  { "Verga", 0.0 },
  { "Vergal", 1.0 },
  { "Verge", 1.0 },
  { "Vergean", 0.0 },
  { "Vergel", 1.0 },
  { "Vergene", 0.0 },
  { "Vergia", 0.0 },
  { "Vergie", 0.010153381 },
  { "Vergil", 0.970386 },
  { "Vergina", 0.0 },
  { "Verginia", 0.0 },
  { "Veria", 0.0 },
  { "Verian", 0.0 },
  { "Veridiana", 0.0 },
  { "Veril", 0.8148148 },
  { "Verilea", 0.0 },
  { "Verina", 0.0 },
  { "Verinda", 0.0 },
  { "Verinica", 0.0 },
  { "Verita", 0.0 },
  { "Verity", 0.0 },
  { "Verjean", 0.0 },
  { "Verl", 0.9226256 },
  { "Verla", 0.0 },
  { "Verlaine", 0.0 },
  { "Verlan", 0.9845679 },
  { "Verlana", 0.0 },
  { "Verland", 1.0 },
  { "Verlanda", 0.0 },
  { "Verlane", 0.0 },
  { "Verle", 0.771391 },
  { "Verlean", 0.0 },
  { "Verlee", 0.0 },
  { "Verleen", 0.0 },
  { "Verlen", 0.73913044 },
  { "Verlena", 0.0 },
  { "Verlene", 0.0 },
  { "Verletta", 0.0 },
  { "Verley", 0.2 },
  { "Verlia", 0.0 },
  { "Verlie", 0.07238715 },
  { "Verlin", 0.92888564 },
  { "Verlina", 0.0 },
  { "Verlinda", 0.0 },
  { "Verline", 0.0 },
  { "Verlis", 1.0 },
  { "Verlisa", 0.0 },
  { "Verlla", 0.0 },
  { "Verlon", 0.8388532 },
  { "Verly", 0.0 },
  { "Verlyn", 0.74583334 },
  { "Verlynn", 0.5 },
  { "Verma", 0.0 },
  { "Vermel", 0.0 },
  { "Vermell", 0.0 },
  { "Vermelle", 0.0 },
  { "Vermon", 1.0 },
  { "Vermonica", 0.0 },
  { "Vermont", 0.75 },
  { "Vern", 0.95940745 },
  { "Verna", 0.0075636194 },
  { "Vernabelle", 0.0 },
  { "Vernada", 0.0 },
  { "Vernadean", 0.0 },
  { "Vernadeen", 0.0 },
  { "Vernadette", 0.0 },
  { "Vernadine", 0.0 },
  { "Vernae", 0.0 },
  { "Vernal", 0.82846546 },
  { "Vernalee", 0.0 },
  { "Vernall", 1.0 },
  { "Vernamae", 0.0 },
  { "Vernan", 1.0 },
  { "Vernando", 1.0 },
  { "Vernard", 1.0 },
  { "Vernasia", 0.0 },
  { "Vernay", 0.0 },
  { "Verne", 0.8693258 },
  { "Verneal", 0.028409092 },
  { "Vernease", 0.0 },
  { "Vernece", 0.0 },
  { "Vernecia", 0.0 },
  { "Verneda", 0.0 },
  { "Vernee", 0.0 },
  { "Verneeda", 0.0 },
  { "Verneice", 0.0 },
  { "Verneil", 0.0 },
  { "Verneisha", 0.0 },
  { "Verneita", 0.0 },
  { "Vernel", 0.74550897 },
  { "Vernelda", 0.0 },
  { "Vernell", 0.31795117 },
  { "Vernella", 0.0 },
  { "Vernelle", 0.004 },
  { "Vernen", 1.0 },
  { "Verner", 0.9198312 },
  { "Vernes", 1.0 },
  { "Vernesa", 0.0 },
  { "Vernese", 0.0 },
  { "Vernesha", 0.0 },
  { "Vernessa", 0.0 },
  { "Vernessia", 0.0 },
  { "Vernest", 1.0 },
  { "Vernesta", 0.0 },
  { "Vernesteen", 0.0 },
  { "Vernestine", 0.0 },
  { "Vernet", 0.52380955 },
  { "Verneta", 0.0 },
  { "Vernetha", 0.0 },
  { "Vernetia", 0.0 },
  { "Vernett", 0.0 },
  { "Vernetta", 0.0 },
  { "Vernette", 0.0 },
  { "Verney", 1.0 },
  { "Vernez", 0.0 },
  { "Vernia", 0.0 },
  { "Vernica", 0.0 },
  { "Vernice", 0.0976606 },
  { "Vernicia", 0.0 },
  { "Vernida", 0.0 },
  { "Vernie", 0.41074884 },
  { "Verniece", 0.0 },
  { "Vernika", 0.0 },
  { "Vernique", 0.0 },
  { "Vernis", 0.60960335 },
  { "Vernisa", 0.0 },
  { "Vernise", 0.0 },
  { "Vernisha", 0.0 },
  { "Vernita", 0.0 },
  { "Vernitta", 0.0 },
  { "Verniya", 0.0 },
  { "Vernoica", 0.0 },
  { "Vernola", 0.0 },
  { "Vernon", 0.98658603 },
  { "Vernona", 0.0 },
  { "Vernonica", 0.0 },
  { "Vernor", 1.0 },
  { "Vernus", 1.0 },
  { "Vero", 0.5 },
  { "Veron", 0.98940676 },
  { "Verona", 0.0 },
  { "Veroncia", 0.0 },
  { "Veronda", 0.0 },
  { "Verone", 0.0 },
  { "Veronia", 0.0 },
  { "Veronic", 0.0 },
  { "Veronica", 0.003841624 },
  { "Veronice", 0.0 },
  { "Veronicia", 0.0 },
  { "Veronicka", 0.0 },
  { "Veronika", 0.0 },
  { "Veronique", 0.0 },
  { "Veronnica", 0.0 },
  { "Veronyca", 0.0 },
  { "Verree", 0.0 },
  { "Verron", 1.0 },
  { "Verrona", 0.0 },
  { "Verronica", 0.0 },
  { "Versa", 0.0 },
  { "Versace", 1.0 },
  { "Versailles", 0.0 },
  { "Versal", 0.0 },
  { "Versavia", 0.0 },
  { "Versel", 1.0 },
  { "Versey", 0.0 },
  { "Vershaun", 1.0 },
  { "Vershawn", 0.9411765 },
  { "Vershon", 1.0 },
  { "Vershonda", 0.0 },
  { "Versia", 0.0 },
  { "Versie", 0.03823311 },
  { "Verta", 0.0 },
  { "Vertell", 0.0 },
  { "Vertia", 0.0 },
  { "Vertice", 0.0 },
  { "Vertie", 0.003229974 },
  { "Vertina", 0.0 },
  { "Vertis", 0.63804346 },
  { "Vertle", 1.0 },
  { "Vertner", 1.0 },
  { "Verton", 1.0 },
  { "Veruca", 0.0 },
  { "Verva", 0.0 },
  { "Veryl", 0.6471698 },
  { "Veryle", 0.1764706 },
  { "Vesa", 0.0 },
  { "Vesenia", 0.0 },
  { "Veshawn", 1.0 },
  { "Vesna", 0.0 },
  { "Vesper", 0.09192825 },
  { "Vessica", 0.0 },
  { "Vessie", 0.009652509 },
  { "Vesta", 0.0033120706 },
  { "Vestal", 0.74593496 },
  { "Vestel", 1.0 },
  { "Vester", 0.79491943 },
  { "Vestie", 0.0 },
  { "Vestina", 0.0 },
  { "Veston", 1.0 },
  { "Veta", 0.0 },
  { "Vetha", 0.0 },
  { "Veto", 1.0 },
  { "Veton", 1.0 },
  { "Vetra", 0.0 },
  { "Vetrice", 0.0 },
  { "Vetta", 0.0 },
  { "Veva", 0.0 },
  { "Vevelyn", 0.0 },
  { "Veverly", 0.0 },
  { "Veya", 0.0 },
  { "Veyah", 0.0 },
  { "Veyda", 0.0 },
  { "Veyonce", 0.0 },
  { "Veyron", 1.0 },
  { "Vhonda", 0.0 },
  { "Vi", 0.12562814 },
  { "Via", 0.0 },
  { "Viaan", 1.0 },
  { "Vian", 0.82945734 },
  { "Viana", 0.0 },
  { "Vianay", 0.0 },
  { "Vianca", 0.0 },
  { "Viance", 0.0 },
  { "Vianet", 0.0 },
  { "Vianeth", 0.0 },
  { "Vianette", 0.0 },
  { "Vianey", 0.0 },
  { "Viani", 0.0 },
  { "Vianka", 0.0 },
  { "Viann", 0.0 },
  { "Vianna", 0.0 },
  { "Vianne", 0.0 },
  { "Vianney", 0.0 },
  { "Vianni", 0.0 },
  { "Vianny", 0.0 },
  { "Viany", 0.0 },
  { "Vibha", 0.0 },
  { "Vibhav", 1.0 },
  { "Vibhu", 1.0 },
  { "Vibiana", 0.0 },
  { "Vibol", 1.0 },
  { "Vic", 1.0 },
  { "Vica", 0.0 },
  { "Vicci", 0.0 },
  { "Vice", 1.0 },
  { "Vicent", 1.0 },
  { "Vicenta", 0.0 },
  { "Vicente", 1.0 },
  { "Vicenzo", 1.0 },
  { "Vicey", 0.0 },
  { "Vichelle", 0.0 },
  { "Vici", 0.0 },
  { "Vicie", 0.0 },
  { "Vick", 0.9915825 },
  { "Vicke", 0.0 },
  { "Vickee", 0.0 },
  { "Vicken", 1.0 },
  { "Vickey", 0.023545241 },
  { "Vicki", 0.001922062 },
  { "Vickiana", 0.0 },
  { "Vickiann", 0.0 },
  { "Vickie", 0.0035158303 },
  { "Vickii", 0.0 },
  { "Vickilee", 0.0 },
  { "Vickilyn", 0.0 },
  { "Vickilynn", 0.0 },
  { "Vickki", 0.0 },
  { "Vicktor", 1.0 },
  { "Vicktoria", 0.0 },
  { "Vicky", 0.008190451 },
  { "Vickye", 0.0 },
  { "Vicotria", 0.0 },
  { "Victoire", 0.0 },
  { "Victor", 0.9942573 },
  { "Victora", 0.0 },
  { "Victoralfonso", 1.0 },
  { "Victorea", 0.0 },
  { "Victorene", 0.0 },
  { "Victorhugo", 1.0 },
  { "Victori", 0.0 },
  { "Victoria", 0.0025641345 },
  { "Victoriaann", 0.0 },
  { "Victoriah", 0.0 },
  { "Victorialynn", 0.0 },
  { "Victoriamarie", 0.0 },
  { "Victorian", 0.0 },
  { "Victoriana", 0.0 },
  { "Victorianna", 0.0 },
  { "Victoriano", 1.0 },
  { "Victoriarose", 0.0 },
  { "Victorie", 0.0 },
  { "Victorina", 0.0 },
  { "Victorine", 0.0 },
  { "Victorino", 1.0 },
  { "Victorio", 1.0 },
  { "Victorious", 0.51666665 },
  { "Victoriya", 0.0 },
  { "Victorjr", 1.0 },
  { "Victormanuel", 1.0 },
  { "Victorria", 0.0 },
  { "Victory", 0.29605263 },
  { "Victorya", 0.0 },
  { "Vicy", 0.0 },
  { "Vida", 0.003596101 },
  { "Vidal", 0.99743986 },
  { "Vidala", 0.0 },
  { "Vidale", 1.0 },
  { "Vidalia", 0.0 },
  { "Videl", 0.24358974 },
  { "Videll", 0.7222222 },
  { "Vidella", 0.0 },
  { "Vidette", 0.0 },
  { "Vidhaan", 1.0 },
  { "Vidhi", 0.0 },
  { "Vidhu", 1.0 },
  { "Vidhur", 1.0 },
  { "Vidhya", 0.0 },
  { "Vidia", 0.0 },
  { "Vidisha", 0.0 },
  { "Vidit", 1.0 },
  { "Vidula", 0.0 },
  { "Vidur", 1.0 },
  { "Vidushi", 0.0 },
  { "Vidya", 0.0 },
  { "Vidyut", 1.0 },
  { "Vidyuth", 1.0 },
  { "Vie", 0.0 },
  { "Vieda", 0.0 },
  { "Vielka", 0.0 },
  { "Viella", 0.0 },
  { "Vien", 1.0 },
  { "Viena", 0.0 },
  { "Vience", 1.0 },
  { "Vienna", 0.0 },
  { "Vienne", 0.0 },
  { "Vieno", 0.0 },
  { "Viera", 0.0 },
  { "Vieri", 1.0 },
  { "Vierra", 0.0 },
  { "Viesha", 0.0 },
  { "Viet", 0.9897959 },
  { "Vietta", 0.0 },
  { "Vieva", 0.0 },
  { "Viggo", 1.0 },
  { "Viginia", 0.0 },
  { "Vignesh", 1.0 },
  { "Vigo", 1.0 },
  { "Viha", 0.0 },
  { "Vihaa", 0.0 },
  { "Vihaan", 1.0 },
  { "Vihaanreddy", 1.0 },
  { "Vihaas", 1.0 },
  { "Vihan", 1.0 },
  { "Vihana", 0.0 },
  { "Vihas", 1.0 },
  { "Vija", 0.0 },
  { "Vijay", 1.0 },
  { "Vijaya", 0.0 },
  { "Vika", 0.0 },
  { "Vikas", 1.0 },
  { "Vikash", 1.0 },
  { "Vikesh", 1.0 },
  { "Vikhyath", 1.0 },
  { "Viki", 0.0 },
  { "Vikie", 0.0 },
  { "Vikita", 0.0 },
  { "Vikki", 0.0 },
  { "Vikkie", 0.0 },
  { "Vikky", 0.0 },
  { "Vikram", 1.0 },
  { "Vikramjit", 1.0 },
  { "Vikrant", 1.0 },
  { "Vikranth", 1.0 },
  { "Viktor", 1.0 },
  { "Viktoria", 0.0 },
  { "Viktorija", 0.0 },
  { "Viktoriya", 0.0 },
  { "Viktorya", 0.0 },
  { "Viky", 0.0 },
  { "Vila", 0.0 },
  { "Vilas", 0.9723866 },
  { "Vilate", 0.0 },
  { "Vilda", 0.0 },
  { "Vildan", 0.0 },
  { "Vilena", 0.0 },
  { "Vilene", 0.0 },
  { "Viletta", 0.0 },
  { "Vilho", 1.0 },
  { "Vili", 1.0 },
  { "Vilia", 0.0 },
  { "Viliami", 1.0 },
  { "Viliamu", 1.0 },
  { "Vilija", 0.0 },
  { "Vilinda", 0.0 },
  { "Villa", 0.03624161 },
  { "Villard", 1.0 },
  { "Ville", 1.0 },
  { "Villie", 0.0 },
  { "Vilma", 0.0 },
  { "Vilmarie", 0.0 },
  { "Vimal", 1.0 },
  { "Vimala", 0.0 },
  { "Vin", 1.0 },
  { "Vina", 0.0 },
  { "Vinal", 1.0 },
  { "Vinathi", 0.0 },
  { "Vinay", 1.0 },
  { "Vinaya", 0.0 },
  { "Vinayak", 1.0 },
  { "Vincci", 0.0 },
  { "Vince", 0.9995671 },
  { "Vincel", 1.0 },
  { "Vincen", 1.0 },
  { "Vincene", 0.0 },
  { "Vincent", 0.9958757 },
  { "Vincenta", 0.0 },
  { "Vincente", 1.0 },
  { "Vincentia", 0.0 },
  { "Vincentina", 0.0 },
  { "Vincentine", 0.0 },
  { "Vincentmichael", 1.0 },
  { "Vincenza", 0.0 },
  { "Vincenzia", 0.0 },
  { "Vincenzina", 0.0 },
  { "Vincenzio", 1.0 },
  { "Vincenzo", 1.0 },
  { "Vincetta", 0.0 },
  { "Vinchenzo", 1.0 },
  { "Vinci", 0.61538464 },
  { "Vincie", 0.0 },
  { "Vincient", 1.0 },
  { "Vincil", 1.0 },
  { "Vincint", 1.0 },
  { "Vincy", 0.0 },
  { "Vinda", 0.0 },
  { "Vindetta", 0.0 },
  { "Vindhya", 0.0 },
  { "Vine", 0.0 },
  { "Vinecia", 0.0 },
  { "Vineel", 1.0 },
  { "Vineet", 1.0 },
  { "Vineeta", 0.0 },
  { "Vineeth", 1.0 },
  { "Vineisha", 0.0 },
  { "Vinell", 0.0 },
  { "Vinesh", 1.0 },
  { "Vinesha", 0.0 },
  { "Vinessa", 0.0 },
  { "Vineta", 0.0 },
  { "Vinetta", 0.0 },
  { "Vinette", 0.0 },
  { "Viney", 0.0 },
  { "Vinh", 0.99293286 },
  { "Vinia", 0.0 },
  { "Vinicio", 1.0 },
  { "Vinicius", 1.0 },
  { "Vinie", 0.0 },
  { "Vinisha", 0.0 },
  { "Vinit", 1.0 },
  { "Vinita", 0.0 },
  { "Vinn", 1.0 },
  { "Vinna", 0.0 },
  { "Vinni", 0.0 },
  { "Vinnia", 0.0 },
  { "Vinnie", 0.18835616 },
  { "Vinny", 1.0 },
  { "Vinod", 1.0 },
  { "Vinola", 0.0 },
  { "Vinona", 0.0 },
  { "Vinson", 1.0 },
  { "Vinston", 1.0 },
  { "Vint", 1.0 },
  { "Vinton", 1.0 },
  { "Vintrell", 1.0 },
  { "Vinyette", 0.0 },
  { "Vinzent", 1.0 },
  { "Viola", 0.0033033444 },
  { "Violanda", 0.0 },
  { "Violar", 0.0 },
  { "Violet", 0.0019371664 },
  { "Violeta", 0.00089461444 },
  { "Violeth", 0.0 },
  { "Violetrose", 0.0 },
  { "Violett", 0.0 },
  { "Violetta", 0.0 },
  { "Violette", 0.0 },
  { "Violia", 0.0 },
  { "Viona", 0.0 },
  { "Vionda", 0.0 },
  { "Vione", 0.0 },
  { "Vionette", 0.0 },
  { "Vionna", 0.0 },
  { "Viora", 0.0 },
  { "Vipul", 1.0 },
  { "Vir", 1.0 },
  { "Vira", 0.0 },
  { "Viraaj", 1.0 },
  { "Viraat", 1.0 },
  { "Viraj", 1.0 },
  { "Virak", 1.0 },
  { "Viral", 1.0 },
  { "Viransh", 1.0 },
  { "Virat", 1.0 },
  { "Virble", 0.0 },
  { "Virda", 0.0 },
  { "Virdell", 0.1491228 },
  { "Virden", 1.0 },
  { "Virdia", 0.0 },
  { "Virdiana", 0.0 },
  { "Virdie", 0.0 },
  { "Viren", 1.0 },
  { "Virene", 0.0 },
  { "Virga", 0.0 },
  { "Virgal", 0.9777778 },
  { "Virge", 1.0 },
  { "Virgean", 0.0 },
  { "Virgel", 0.9949707 },
  { "Virgen", 0.0 },
  { "Virgena", 0.0 },
  { "Virgene", 0.0 },
  { "Virgia", 0.0 },
  { "Virgie", 0.029745737 },
  { "Virgil", 0.9800959 },
  { "Virgilene", 0.0 },
  { "Virgilia", 0.0 },
  { "Virgiline", 0.0 },
  { "Virgilio", 1.0 },
  { "Virgin", 0.119521916 },
  { "Virgina", 0.0 },
  { "Virgine", 0.0 },
  { "Virgini", 0.0 },
  { "Virginia", 0.003091806 },
  { "Virginialee", 0.0 },
  { "Virginiamae", 0.0 },
  { "Virginie", 0.0 },
  { "Virginio", 1.0 },
  { "Virginius", 1.0 },
  { "Virginnia", 0.0 },
  { "Virgle", 0.98519516 },
  { "Virgnia", 0.0 },
  { "Virgus", 1.0 },
  { "Viridiana", 0.0012336541 },
  { "Virigina", 0.0 },
  { "Viriginia", 0.0 },
  { "Virignia", 0.0 },
  { "Viririana", 0.0 },
  { "Virjean", 0.0 },
  { "Virjinia", 0.0 },
  { "Virl", 0.8789238 },
  { "Virla", 0.0 },
  { "Virlan", 1.0 },
  { "Virlee", 0.0 },
  { "Virley", 0.0 },
  { "Virlie", 0.0 },
  { "Virlinda", 0.0 },
  { "Virlyn", 1.0 },
  { "Virma", 0.0 },
  { "Virna", 0.0 },
  { "Viron", 1.0 },
  { "Vironica", 0.0 },
  { "Virsaviya", 0.0 },
  { "Virsie", 0.0 },
  { "Virtie", 0.0 },
  { "Virtue", 0.0 },
  { "Virtus", 1.0 },
  { "Visente", 1.0 },
  { "Vishaal", 1.0 },
  { "Vishaan", 1.0 },
  { "Vishak", 1.0 },
  { "Vishakan", 1.0 },
  { "Vishal", 1.0 },
  { "Vishan", 1.0 },
  { "Vishanth", 1.0 },
  { "Vishesh", 1.0 },
  { "Vishika", 0.0 },
  { "Vishnu", 1.0 },
  { "Vishrut", 1.0 },
  { "Vishruth", 1.0 },
  { "Vishv", 1.0 },
  { "Vishva", 0.5 },
  { "Vishwa", 0.91534394 },
  { "Vishwak", 1.0 },
  { "Vishwesh", 1.0 },
  { "Vision", 0.0 },
  { "Vismaya", 0.0 },
  { "Vista", 0.0 },
  { "Vita", 0.0 },
  { "Vitali", 1.0 },
  { "Vitalia", 0.0 },
  { "Vitalina", 0.0 },
  { "Vitaliy", 1.0 },
  { "Vitaly", 1.0 },
  { "Vitani", 0.0 },
  { "Vitas", 1.0 },
  { "Vitina", 0.0 },
  { "Vito", 1.0 },
  { "Vitold", 1.0 },
  { "Vitor", 1.0 },
  { "Vitoria", 0.0 },
  { "Vitorio", 1.0 },
  { "Vittoria", 0.0 },
  { "Vittorio", 1.0 },
  { "Vitula", 0.0 },
  { "Vitus", 1.0 },
  { "Viv", 0.0 },
  { "Viva", 0.0 },
  { "Vivaan", 1.0 },
  { "Vivan", 0.40695652 },
  { "Vivana", 0.0 },
  { "Viveca", 0.0 },
  { "Vivek", 1.0 },
  { "Viveka", 0.0 },
  { "Vivenne", 0.0 },
  { "Vivette", 0.0 },
  { "Vivi", 0.0 },
  { "Vivia", 0.0 },
  { "Vivian", 0.012520917 },
  { "Viviana", 0.0010972569 },
  { "Viviane", 0.0 },
  { "Viviani", 0.0 },
  { "Viviann", 0.0 },
  { "Vivianna", 0.0 },
  { "Vivianne", 0.0 },
  { "Viviano", 1.0 },
  { "Vivica", 0.0 },
  { "Vividiana", 0.0 },
  { "Vivie", 0.0 },
  { "Vivien", 0.004310345 },
  { "Viviene", 0.0 },
  { "Vivienna", 0.0 },
  { "Vivienne", 0.0 },
  { "Viviette", 0.0 },
  { "Vivika", 0.0 },
  { "Vivin", 0.7619048 },
  { "Vivion", 0.10869565 },
  { "Vivvian", 0.0 },
  { "Vivy", 0.0 },
  { "Vivyan", 0.0 },
  { "Vivyana", 0.0 },
  { "Vivyanna", 0.0 },
  { "Viya", 0.0 },
  { "Viyaan", 1.0 },
  { "Viyan", 1.0 },
  { "Viyana", 0.0 },
  { "Viyona", 0.0 },
  { "Vlad", 1.0 },
  { "Vlada", 0.0 },
  { "Vladik", 1.0 },
  { "Vladimir", 1.0 },
  { "Vladislav", 1.0 },
  { "Vladislava", 0.0 },
  { "Vladyslav", 1.0 },
  { "Vlasta", 0.0 },
  { "Vlora", 0.0 },
  { "Voctoria", 0.0 },
  { "Voda", 0.0 },
  { "Vohn", 1.0 },
  { "Void", 0.72727275 },
  { "Voight", 1.0 },
  { "Voila", 0.0 },
  { "Voilet", 0.0 },
  { "Vol", 1.0 },
  { "Vola", 0.024495678 },
  { "Volanda", 0.0 },
  { "Volare", 0.0 },
  { "Volena", 0.0 },
  { "Voleta", 0.0 },
  { "Volf", 1.0 },
  { "Volia", 0.0 },
  { "Volina", 0.0 },
  { "Volker", 1.0 },
  { "Volley", 1.0 },
  { "Vollie", 0.8366446 },
  { "Volney", 1.0 },
  { "Voltaire", 1.0 },
  { "Volvi", 1.0 },
  { "Volvy", 1.0 },
  { "Von", 0.9159087 },
  { "Vona", 0.0 },
  { "Vonceal", 0.0 },
  { "Vonceil", 0.0 },
  { "Voncia", 0.0 },
  { "Vonciel", 0.0 },
  { "Voncil", 0.0 },
  { "Voncile", 0.0 },
  { "Voncille", 0.0 },
  { "Vonda", 0.0010657882 },
  { "Vondal", 1.0 },
  { "Vondale", 1.0 },
  { "Vondalee", 0.0 },
  { "Vondell", 0.42236024 },
  { "Vondella", 0.0 },
  { "Vondia", 0.0 },
  { "Vondra", 0.0 },
  { "Vondre", 1.0 },
  { "Vondrell", 1.0 },
  { "Voneda", 0.0 },
  { "Vonell", 0.0 },
  { "Vonessa", 0.0 },
  { "Voneta", 0.0 },
  { "Vonetta", 0.0 },
  { "Vonette", 0.0 },
  { "Vong", 1.0 },
  { "Voni", 0.0 },
  { "Vonica", 0.0 },
  { "Vonice", 0.0 },
  { "Vonie", 0.0 },
  { "Vonisha", 0.0 },
  { "Vonita", 0.0 },
  { "Vonley", 1.0 },
  { "Vonmarie", 0.0 },
  { "Vonn", 0.9747475 },
  { "Vonna", 0.0 },
  { "Vonne", 0.0 },
  { "Vonnell", 0.0 },
  { "Vonnesha", 0.0 },
  { "Vonnetta", 0.0 },
  { "Vonnette", 0.0 },
  { "Vonnie", 0.103896104 },
  { "Vonnita", 0.0 },
  { "Vonshae", 0.0 },
  { "Vontae", 1.0 },
  { "Vontarius", 1.0 },
  { "Vontavious", 1.0 },
  { "Vontavius", 1.0 },
  { "Vonte", 1.0 },
  { "Vontella", 0.0 },
  { "Vontese", 0.0 },
  { "Vontez", 1.0 },
  { "Vontina", 0.0 },
  { "Vontray", 1.0 },
  { "Vontre", 1.0 },
  { "Vontrel", 1.0 },
  { "Vontrell", 1.0 },
  { "Vontresa", 0.0 },
  { "Vontressa", 0.0 },
  { "Vontrice", 0.0 },
  { "Vonya", 0.0 },
  { "Vonzel", 1.0 },
  { "Vonzell", 1.0 },
  { "Vonzella", 0.0 },
  { "Vonzetta", 0.0 },
  { "Vora", 0.0 },
  { "Voris", 1.0 },
  { "Voronica", 0.0 },
  { "Voshon", 1.0 },
  { "Voss", 1.0 },
  { "Voula", 0.0 },
  { "Voy", 1.0 },
  { "Voyd", 1.0 },
  { "Voyle", 1.0 },
  { "Vraj", 1.0 },
  { "Vrinda", 0.0 },
  { "Vrisa", 0.0 },
  { "Vrishank", 1.0 },
  { "Vriti", 0.0 },
  { "Vritika", 0.0 },
  { "Vrunda", 0.0 },
  { "Vsevolod", 1.0 },
  { "Vu", 1.0 },
  { "Vue", 1.0 },
  { "Vuk", 1.0 },
  { "Vukasin", 1.0 },
  { "Vula", 0.0 },
  { "Vulnavia", 0.0 },
  { "Vung", 0.0 },
  { "Vuong", 1.0 },
  { "Vurla", 0.0 },
  { "Vuthy", 1.0 },
  { "Vy", 0.032918148 },
  { "Vyaan", 1.0 },
  { "Vyacheslav", 1.0 },
  { "Vyan", 1.0 },
  { "Vyanna", 0.0 },
  { "Vyas", 1.0 },
  { "Vyctoria", 0.0 },
  { "Vyctorya", 0.0 },
  { "Vydia", 0.0 },
  { "Vyktoria", 0.0 },
  { "Vyla", 0.0 },
  { "Vylet", 0.0 },
  { "Vylette", 0.0 },
  { "Vyncent", 1.0 },
  { "Vynessa", 0.0 },
  { "Vyola", 0.0 },
  { "Vyolet", 0.0 },
  { "Vyolette", 0.0 },
  { "Vyom", 1.0 },
  { "Vyonne", 0.0 },
  { "Vyrl", 1.0 },
  { "Vyron", 1.0 },
  { "Vyshnavi", 0.0 },
  { "Vyshonn", 1.0 },
  { "Vyshonne", 1.0 },
  { "Vytas", 1.0 },
  { "Vytautas", 1.0 },
  { "Vyto", 1.0 },
  { "Vyvian", 0.0 },
  { "Vyvy", 0.0 },
  { "Vyvyan", 0.0 },
  { "Wa", 1.0 },
  { "Waad", 0.0 },
  { "Waard", 1.0 },
  { "Waaris", 1.0 },
  { "Wacey", 1.0 },
  { "Wachovia", 0.0 },
  { "Waclaw", 1.0 },
  { "Waco", 1.0 },
  { "Waconda", 0.0 },
  { "Waddell", 1.0 },
  { "Waddie", 0.8076923 },
  { "Waddy", 1.0 },
  { "Wade", 0.99760276 },
  { "Wadee", 1.0 },
  { "Wadeeah", 0.0 },
  { "Wadell", 1.0 },
  { "Waden", 1.0 },
  { "Wadena", 0.0 },
  { "Wadia", 0.0 },
  { "Wadie", 0.48387095 },
  { "Wadine", 0.0 },
  { "Wadiya", 0.0 },
  { "Wadley", 1.0 },
  { "Wadsworth", 1.0 },
  { "Wael", 1.0 },
  { "Waelynn", 0.0 },
  { "Wafa", 0.0 },
  { "Wafaa", 0.0 },
  { "Wafi", 1.0 },
  { "Waford", 1.0 },
  { "Wagner", 1.0 },
  { "Wah", 1.0 },
  { "Wahab", 1.0 },
  { "Wahaj", 1.0 },
  { "Waheed", 1.0 },
  { "Waheedah", 0.0 },
  { "Wahid", 1.0 },
  { "Wahneeta", 0.0 },
  { "Wahneta", 0.0 },
  { "Wahnetta", 0.0 },
  { "Wahnita", 0.0 },
  { "Wai", 0.66412216 },
  { "Waid", 1.0 },
  { "Waide", 1.0 },
  { "Wailyn", 1.0 },
  { "Wain", 1.0 },
  { "Waine", 1.0 },
  { "Waino", 1.0 },
  { "Wainwright", 1.0 },
  { "Wais", 1.0 },
  { "Waite", 1.0 },
  { "Waitman", 1.0 },
  { "Waitus", 1.0 },
  { "Waive", 0.0 },
  { "Waiz", 1.0 },
  { "Wajd", 0.0 },
  { "Wajeeha", 0.0 },
  { "Wajiha", 0.0 },
  { "Wakana", 0.0 },
  { "Wake", 1.0 },
  { "Wakeel", 1.0 },
  { "Wakeelah", 0.0 },
  { "Wakeem", 1.0 },
  { "Wakeen", 1.0 },
  { "Wakefield", 1.0 },
  { "Wakeisha", 0.0 },
  { "Wakely", 0.13157895 },
  { "Wakenda", 0.0 },
  { "Waker", 1.0 },
  { "Wakesha", 0.0 },
  { "Wakia", 0.0 },
  { "Wakil", 1.0 },
  { "Wakina", 0.0 },
  { "Wakinyan", 1.0 },
  { "Wakisha", 0.0 },
  { "Wakita", 0.0 },
  { "Wakiyan", 1.0 },
  { "Wala", 0.0 },
  { "Walaa", 0.0 },
  { "Walace", 1.0 },
  { "Walbert", 1.0 },
  { "Walberto", 1.0 },
  { "Walburga", 0.0 },
  { "Walcie", 0.0 },
  { "Walda", 0.0 },
  { "Waldean", 0.23255815 },
  { "Waldemar", 1.0 },
  { "Walden", 1.0 },
  { "Waldena", 0.0 },
  { "Waldene", 0.0 },
  { "Waldine", 0.0 },
  { "Waldo", 0.9989431 },
  { "Waldon", 1.0 },
  { "Waldron", 1.0 },
  { "Waldy", 1.0 },
  { "Wale", 1.0 },
  { "Waled", 1.0 },
  { "Walee", 1.0 },
  { "Waleed", 1.0 },
  { "Walela", 0.0 },
  { "Walene", 0.0 },
  { "Wales", 1.0 },
  { "Walesca", 0.0 },
  { "Waleska", 0.0 },
  { "Walford", 1.0 },
  { "Walfred", 1.0 },
  { "Walfrid", 1.0 },
  { "Wali", 1.0 },
  { "Walia", 0.0 },
  { "Walid", 1.0 },
  { "Walida", 0.0 },
  { "Walidah", 0.0 },
  { "Walik", 1.0 },
  { "Walisha", 0.0 },
  { "Waliyah", 0.0 },
  { "Walker", 0.98740155 },
  { "Walkidia", 0.0 },
  { "Walkiria", 0.0 },
  { "Wallace", 0.9933641 },
  { "Wallen", 1.0 },
  { "Wallene", 0.0 },
  { "Waller", 1.0 },
  { "Walley", 1.0 },
  { "Wallice", 1.0 },
  { "Wallie", 0.86165047 },
  { "Walline", 0.0 },
  { "Wallis", 0.568574 },
  { "Wally", 0.97042847 },
  { "Walmer", 1.0 },
  { "Walsh", 1.0 },
  { "Walsie", 0.0 },
  { "Walt", 1.0 },
  { "Walta", 0.0 },
  { "Walter", 0.9941689 },
  { "Walterene", 0.0 },
  { "Walterine", 0.0 },
  { "Walters", 1.0 },
  { "Walther", 1.0 },
  { "Waltina", 0.0 },
  { "Walton", 0.99820435 },
  { "Wambui", 0.0 },
  { "Wamon", 1.0 },
  { "Wan", 0.33333334 },
  { "Wana", 0.0 },
  { "Wanakee", 0.0 },
  { "Wanda", 0.0034472118 },
  { "Wandalea", 0.0 },
  { "Wandalee", 0.0 },
  { "Wandalene", 0.0 },
  { "Wandalid", 0.0 },
  { "Wandalyn", 0.0 },
  { "Wandell", 0.9425287 },
  { "Wander", 0.0 },
  { "Wanderlei", 1.0 },
  { "Wandra", 0.0 },
  { "Wandy", 0.0 },
  { "Wane", 1.0 },
  { "Waneda", 0.0 },
  { "Waneeta", 0.0 },
  { "Waneita", 0.0 },
  { "Wanell", 0.0 },
  { "Wanema", 0.0 },
  { "Wanesha", 0.0 },
  { "Waneta", 0.0 },
  { "Wanetah", 0.0 },
  { "Wanetta", 0.0 },
  { "Wanette", 0.0 },
  { "Wang", 1.0 },
  { "Wani", 1.0 },
  { "Wania", 0.0 },
  { "Wanika", 0.0 },
  { "Wanisha", 0.0 },
  { "Wanita", 0.0 },
  { "Waniya", 0.0 },
  { "Wanjiku", 0.0 },
  { "Wanjiru", 0.0 },
  { "Wanna", 0.0 },
  { "Wannell", 0.0 },
  { "Wannetta", 0.0 },
  { "Wannette", 0.0 },
  { "Wannie", 0.22 },
  { "Wanona", 0.0 },
  { "Wanya", 0.9642105 },
  { "Wanyae", 1.0 },
  { "Wanye", 1.0 },
  { "Wanza", 0.0 },
  { "Wanzie", 0.0 },
  { "Waqar", 1.0 },
  { "Waqas", 1.0 },
  { "Ward", 0.997751 },
  { "Warda", 0.0 },
  { "Wardah", 0.0 },
  { "Wardean", 0.0 },
  { "Wardel", 1.0 },
  { "Wardell", 0.99164927 },
  { "Warden", 1.0 },
  { "Warder", 1.0 },
  { "Wardie", 0.7903226 },
  { "Wardine", 0.0 },
  { "Wardner", 1.0 },
  { "Ware", 1.0 },
  { "Wareef", 0.0 },
  { "Warees", 1.0 },
  { "Wareesha", 0.0 },
  { "Waren", 1.0 },
  { "Warfield", 1.0 },
  { "Waring", 1.0 },
  { "Waris", 1.0 },
  { "Warith", 1.0 },
  { "Warn", 1.0 },
  { "Warna", 0.0 },
  { "Warne", 1.0 },
  { "Warnell", 0.90909094 },
  { "Warner", 0.99345446 },
  { "Warnetta", 0.0 },
  { "Warnie", 1.0 },
  { "Warran", 1.0 },
  { "Warren", 0.9948962 },
  { "Warrene", 0.0 },
  { "Warrenetta", 0.0 },
  { "Warrick", 1.0 },
  { "Warrine", 0.0 },
  { "Warrior", 1.0 },
  { "Warsame", 1.0 },
  { "Warwick", 1.0 },
  { "Wasay", 1.0 },
  { "Wasco", 1.0 },
  { "Waseem", 1.0 },
  { "Wash", 1.0 },
  { "Washington", 1.0 },
  { "Wasi", 1.0 },
  { "Wasif", 1.0 },
  { "Wasil", 1.0 },
  { "Wasim", 1.0 },
  { "Wasseem", 1.0 },
  { "Wassil", 1.0 },
  { "Wassillie", 1.0 },
  { "Wassim", 1.0 },
  { "Wasyl", 1.0 },
  { "Wataru", 1.0 },
  { "Watasha", 0.0 },
  { "Wateen", 0.0 },
  { "Watha", 0.0 },
  { "Wathena", 0.0 },
  { "Watie", 1.0 },
  { "Watisha", 0.0 },
  { "Watkins", 1.0 },
  { "Watler", 1.0 },
  { "Watson", 1.0 },
  { "Watt", 1.0 },
  { "Wattie", 1.0 },
  { "Watts", 1.0 },
  { "Waukesha", 0.0 },
  { "Wauline", 0.0 },
  { "Waunda", 0.0 },
  { "Waunell", 0.0 },
  { "Wauneta", 0.0 },
  { "Waunetta", 0.0 },
  { "Waunita", 0.0 },
  { "Wava", 0.0 },
  { "Wavalene", 0.0 },
  { "Wave", 0.0 },
  { "Wavel", 0.0 },
  { "Wavelene", 0.0 },
  { "Waveline", 0.0 },
  { "Wavell", 1.0 },
  { "Wavely", 1.0 },
  { "Wavelyn", 0.0 },
  { "Waver", 0.1375 },
  { "Waverlee", 0.0 },
  { "Waverley", 0.14666666 },
  { "Waverly", 0.5314628 },
  { "Wavie", 0.15181518 },
  { "Wavy", 1.0 },
  { "Way", 1.0 },
  { "Wayburn", 1.0 },
  { "Wayd", 1.0 },
  { "Wayde", 1.0 },
  { "Wayden", 1.0 },
  { "Wayford", 1.0 },
  { "Waylan", 1.0 },
  { "Wayland", 1.0 },
  { "Waylen", 1.0 },
  { "Waylin", 0.96842104 },
  { "Waylon", 0.99722683 },
  { "Waylyn", 0.8484849 },
  { "Waylynn", 0.28 },
  { "Wayman", 1.0 },
  { "Waymon", 1.0 },
  { "Waymond", 1.0 },
  { "Wayna", 0.0 },
  { "Waynard", 1.0 },
  { "Wayne", 0.99546033 },
  { "Wayneisha", 0.0 },
  { "Waynesha", 0.0 },
  { "Wayneshia", 0.0 },
  { "Waynetta", 0.0 },
  { "Waynette", 0.0 },
  { "Waynisha", 0.0 },
  { "Waynne", 1.0 },
  { "Wc", 1.0 },
  { "Wealtha", 0.0 },
  { "Wealthy", 0.0 },
  { "Weatherly", 0.0 },
  { "Weaver", 1.0 },
  { "Webb", 1.0 },
  { "Webber", 1.0 },
  { "Weber", 1.0 },
  { "Webster", 1.0 },
  { "Weda", 0.0 },
  { "Wedad", 0.0 },
  { "Wednesday", 0.0 },
  { "Weeda", 0.0 },
  { "Weena", 0.0 },
  { "Wei", 0.7790368 },
  { "Weida", 0.0 },
  { "Weikko", 1.0 },
  { "Weiland", 1.0 },
  { "Weir", 1.0 },
  { "Wejdan", 0.0 },
  { "Welbert", 1.0 },
  { "Welborn", 1.0 },
  { "Welburn", 1.0 },
  { "Welby", 1.0 },
  { "Welch", 1.0 },
  { "Welcome", 0.86470586 },
  { "Welda", 0.0 },
  { "Welden", 1.0 },
  { "Weldon", 0.9981999 },
  { "Welford", 1.0 },
  { "Welker", 1.0 },
  { "Welles", 1.0 },
  { "Wellesley", 0.5 },
  { "Wellington", 1.0 },
  { "Wellman", 1.0 },
  { "Wells", 0.9723024 },
  { "Wellsley", 0.0 },
  { "Welma", 0.0 },
  { "Welmer", 1.0 },
  { "Welsey", 1.0 },
  { "Weltha", 0.0 },
  { "Welton", 1.0 },
  { "Welty", 1.0 },
  { "Wen", 0.57236844 },
  { "Wencel", 1.0 },
  { "Wences", 1.0 },
  { "Wenceslao", 1.0 },
  { "Wenceslaus", 1.0 },
  { "Wencil", 1.0 },
  { "Wenda", 0.0 },
  { "Wendal", 1.0 },
  { "Wendall", 0.997463 },
  { "Wendalyn", 0.0 },
  { "Wende", 0.0 },
  { "Wendee", 0.0 },
  { "Wendel", 1.0 },
  { "Wendelin", 0.33989266 },
  { "Wendeline", 0.0 },
  { "Wendell", 0.98951435 },
  { "Wendella", 0.0 },
  { "Wendellyn", 0.0 },
  { "Wendelyn", 0.0 },
  { "Wendey", 0.0 },
  { "Wendi", 0.0 },
  { "Wendie", 0.0 },
  { "Wendla", 0.0 },
  { "Wendle", 1.0 },
  { "Wendlyn", 0.0 },
  { "Wendolee", 0.0 },
  { "Wendoly", 0.0 },
  { "Wendolyn", 0.008665511 },
  { "Wendolyne", 0.0 },
  { "Wendra", 0.0 },
  { "Wendy", 0.0036419977 },
  { "Wendyann", 0.0 },
  { "Wendye", 0.0 },
  { "Wendyl", 1.0 },
  { "Wendylee", 0.0 },
  { "Wenford", 1.0 },
  { "Wengel", 0.0 },
  { "Wenndy", 0.0 },
  { "Wenola", 0.0 },
  { "Wenona", 0.0 },
  { "Wenonah", 0.0 },
  { "Wensel", 1.0 },
  { "Wensley", 1.0 },
  { "Wenson", 1.0 },
  { "Wentworth", 1.0 },
  { "Wenzel", 1.0 },
  { "Weona", 0.0 },
  { "Wera", 0.0 },
  { "Werner", 1.0 },
  { "Weronika", 0.0 },
  { "Wert", 1.0 },
  { "Wes", 1.0 },
  { "Wesam", 1.0 },
  { "Wesely", 1.0 },
  { "Weslee", 0.873807 },
  { "Wesleigh", 0.02793296 },
  { "Wesley", 0.9887011 },
  { "Wesli", 0.0 },
  { "Weslie", 0.62456745 },
  { "Weslin", 1.0 },
  { "Wesly", 1.0 },
  { "Weslyn", 0.15897436 },
  { "Weslynn", 0.0 },
  { "Wesner", 1.0 },
  { "Wess", 1.0 },
  { "Wessam", 1.0 },
  { "Wessen", 1.0 },
  { "Wessie", 0.0 },
  { "Wessley", 1.0 },
  { "Wesson", 1.0 },
  { "West", 1.0 },
  { "Westan", 1.0 },
  { "Westen", 1.0 },
  { "Wester", 1.0 },
  { "Western", 1.0 },
  { "Westin", 0.99832654 },
  { "Westlee", 1.0 },
  { "Westleigh", 1.0 },
  { "Westley", 0.9985655 },
  { "Westly", 1.0 },
  { "Westlyn", 0.0 },
  { "Westlynn", 0.0 },
  { "Westmoreland", 1.0 },
  { "Weston", 0.9990519 },
  { "Westonia", 0.0 },
  { "Westyn", 0.9399261 },
  { "Weta", 0.0 },
  { "Wetona", 0.0 },
  { "Wetzel", 1.0 },
  { "Weylan", 1.0 },
  { "Weyland", 1.0 },
  { "Weylin", 1.0 },
  { "Weylon", 1.0 },
  { "Weylyn", 0.8596491 },
  { "Weyman", 1.0 },
  { "Weymon", 1.0 },
  { "Weymouth", 1.0 },
  { "Wezley", 1.0 },
  { "Whalen", 1.0 },
  { "Whaley", 1.0 },
  { "Wharton", 1.0 },
  { "Wheaton", 1.0 },
  { "Wheeler", 0.9969212 },
  { "Whelan", 1.0 },
  { "Whendy", 0.0 },
  { "Whilma", 0.0 },
  { "Whisper", 0.0 },
  { "Whit", 1.0 },
  { "Whitaker", 0.9859155 },
  { "White", 1.0 },
  { "Whitelaw", 1.0 },
  { "Whiteny", 0.0 },
  { "Whitfield", 1.0 },
  { "Whitford", 1.0 },
  { "Whitlee", 0.0 },
  { "Whitleigh", 0.0 },
  { "Whitley", 0.057091236 },
  { "Whitli", 0.0 },
  { "Whitlie", 0.0 },
  { "Whitman", 1.0 },
  { "Whitmore", 1.0 },
  { "Whitnay", 0.0 },
  { "Whitne", 0.0 },
  { "Whitnee", 0.0 },
  { "Whitnei", 0.0 },
  { "Whitney", 0.058117107 },
  { "Whitni", 0.0 },
  { "Whitnie", 0.0 },
  { "Whitny", 0.0 },
  { "Whitnye", 0.0 },
  { "Whitson", 1.0 },
  { "Whitt", 1.0 },
  { "Whittaker", 1.0 },
  { "Whittany", 0.0 },
  { "Whitten", 0.9769585 },
  { "Whittley", 0.0 },
  { "Whittney", 0.003052503 },
  { "Whittni", 0.0 },
  { "Whittnie", 0.0 },
  { "Whitton", 1.0 },
  { "Whtiney", 0.0 },
  { "Whyatt", 1.0 },
  { "Whynter", 0.0 },
  { "Whysper", 0.0 },
  { "Wialliam", 1.0 },
  { "Wiam", 0.0 },
  { "Wiatt", 1.0 },
  { "Wicahpi", 0.0 },
  { "Wick", 1.0 },
  { "Widad", 0.0 },
  { "Widline", 0.0 },
  { "Wigberto", 1.0 },
  { "Wiiliam", 1.0 },
  { "Wiktor", 1.0 },
  { "Wiktoria", 0.0 },
  { "Wil", 1.0 },
  { "Wila", 0.0 },
  { "Wiladean", 0.0 },
  { "Wilard", 1.0 },
  { "Wilba", 0.0 },
  { "Wilber", 0.9983371 },
  { "Wilbern", 1.0 },
  { "Wilbert", 0.99554765 },
  { "Wilberta", 0.0 },
  { "Wilberth", 1.0 },
  { "Wilberto", 1.0 },
  { "Wilbon", 1.0 },
  { "Wilbor", 1.0 },
  { "Wilborn", 1.0 },
  { "Wilbourn", 1.0 },
  { "Wilbur", 0.9949037 },
  { "Wilburn", 0.9985482 },
  { "Wilburne", 1.0 },
  { "Wilburt", 1.0 },
  { "Wilburta", 0.0 },
  { "Wilby", 1.0 },
  { "Wilce", 1.0 },
  { "Wilcie", 0.0 },
  { "Wilco", 1.0 },
  { "Wilda", 0.001174904 },
  { "Wildan", 1.0 },
  { "Wilde", 1.0 },
  { "Wilder", 0.94062006 },
  { "Wildon", 1.0 },
  { "Wildred", 0.1388889 },
  { "Wileen", 0.0 },
  { "Wilella", 0.0 },
  { "Wilem", 1.0 },
  { "Wilena", 0.0 },
  { "Wilene", 0.0 },
  { "Wilesha", 0.0 },
  { "Wiletta", 0.0 },
  { "Wiley", 0.9887214 },
  { "Wilferd", 1.0 },
  { "Wilford", 0.9995188 },
  { "Wilfred", 0.9964819 },
  { "Wilfreda", 0.0 },
  { "Wilfredo", 1.0 },
  { "Wilfrid", 1.0 },
  { "Wilfrido", 1.0 },
  { "Wilfried", 1.0 },
  { "Wilgus", 1.0 },
  { "Wilhelm", 1.0 },
  { "Wilhelmena", 0.0 },
  { "Wilhelmenia", 0.0 },
  { "Wilhelmina", 0.00041830502 },
  { "Wilhelmine", 0.0 },
  { "Wilhelminia", 0.0 },
  { "Wilhem", 1.0 },
  { "Wilhemena", 0.0 },
  { "Wilhemenia", 0.0 },
  { "Wilhemina", 0.0 },
  { "Wilho", 1.0 },
  { "Wiliam", 1.0 },
  { "Wilian", 1.0 },
  { "Wilibaldo", 1.0 },
  { "Wilie", 1.0 },
  { "Wilisha", 0.0 },
  { "Wiljo", 1.0 },
  { "Wilkens", 1.0 },
  { "Wilkes", 1.0 },
  { "Wilkie", 0.95027626 },
  { "Wilkin", 1.0 },
  { "Wilkins", 1.0 },
  { "Wilkinson", 1.0 },
  { "Will", 0.9853066 },
  { "Willa", 0.0009349071 },
  { "Willabell", 0.0 },
  { "Willabelle", 0.0 },
  { "Willadean", 0.0 },
  { "Willadeen", 0.0 },
  { "Willadene", 0.0 },
  { "Willadine", 0.0 },
  { "Willah", 0.0 },
  { "Willaim", 1.0 },
  { "Willam", 1.0 },
  { "Willamae", 0.0 },
  { "Willamena", 0.0 },
  { "Willamina", 0.0 },
  { "Willanna", 0.0 },
  { "Willar", 0.0 },
  { "Willard", 0.98869747 },
  { "Willas", 1.0 },
  { "Willasia", 0.0 },
  { "Willavene", 0.0 },
  { "Willbert", 1.0 },
  { "Willburn", 1.0 },
  { "Wille", 0.8990148 },
  { "Willean", 0.0 },
  { "Willease", 0.0 },
  { "Willeen", 0.0 },
  { "Willella", 0.0 },
  { "Willem", 1.0 },
  { "Willemina", 0.0 },
  { "Willena", 0.0 },
  { "Willene", 0.0 },
  { "Willer", 0.054347824 },
  { "Willes", 1.0 },
  { "Willesha", 0.0 },
  { "Willet", 0.94 },
  { "Willeta", 0.0 },
  { "Willett", 0.78205127 },
  { "Willetta", 0.0 },
  { "Willette", 0.0 },
  { "Willey", 1.0 },
  { "Willford", 1.0 },
  { "Willfred", 1.0 },
  { "Willfredo", 1.0 },
  { "Willi", 1.0 },
  { "Willia", 0.14293098 },
  { "Williaa", 1.0 },
  { "Williadean", 0.0 },
  { "William", 0.99611765 },
  { "Williametta", 0.0 },
  { "Williamhenry", 1.0 },
  { "Williamina", 0.0 },
  { "Williamjames", 1.0 },
  { "Williamjohn", 1.0 },
  { "Williamjoseph", 1.0 },
  { "Williamm", 1.0 },
  { "Williammichael", 1.0 },
  { "Williamrobert", 1.0 },
  { "Williams", 0.9958159 },
  { "Williamson", 1.0 },
  { "Willian", 0.995421 },
  { "Williard", 0.9953917 },
  { "Williaw", 1.0 },
  { "Willibaldo", 1.0 },
  { "Willidean", 0.0 },
  { "Willie", 0.7542024 },
  { "Willielee", 0.0 },
  { "Williem", 1.0 },
  { "Williemae", 0.0 },
  { "Willien", 0.0 },
  { "Williene", 0.0 },
  { "Willies", 1.0 },
  { "Williesha", 0.0 },
  { "Willietta", 0.0 },
  { "Williette", 0.0 },
  { "Williford", 1.0 },
  { "Williiam", 1.0 },
  { "Willilam", 1.0 },
  { "Willim", 1.0 },
  { "Williman", 1.0 },
  { "Willimena", 0.0 },
  { "Willine", 0.0 },
  { "Willington", 1.0 },
  { "Williom", 1.0 },
  { "Willis", 0.9843851 },
  { "Willisa", 0.0 },
  { "Willisha", 0.0 },
  { "Willison", 1.0 },
  { "Willistine", 0.0 },
  { "Willkie", 1.0 },
  { "Willliam", 1.0 },
  { "Willma", 0.0 },
  { "Willman", 1.0 },
  { "Willmar", 1.0 },
  { "Willmer", 0.9809524 },
  { "Willmetta", 0.0 },
  { "Willmon", 1.0 },
  { "Willo", 0.0 },
  { "Willodean", 0.0 },
  { "Willodeen", 0.0 },
  { "Willodene", 0.0 },
  { "Willodine", 0.0 },
  { "Willoe", 0.0 },
  { "Willoh", 0.0 },
  { "Willola", 0.0 },
  { "Willona", 0.0 },
  { "Willora", 0.0 },
  { "Willoughby", 0.8913044 },
  { "Willow", 0.008267538 },
  { "Willowdean", 0.0 },
  { "Wills", 1.0 },
  { "Willson", 1.0 },
  { "Willum", 1.0 },
  { "Willus", 1.0 },
  { "Willy", 0.96940124 },
  { "Willye", 0.0 },
  { "Willys", 1.0 },
  { "Wilma", 0.0053679235 },
  { "Wilmah", 0.0 },
  { "Wilmajean", 0.0 },
  { "Wilman", 1.0 },
  { "Wilmar", 0.9714795 },
  { "Wilmarie", 0.0 },
  { "Wilmary", 0.0 },
  { "Wilmer", 0.95310825 },
  { "Wilmeth", 0.5 },
  { "Wilmetta", 0.0 },
  { "Wilmina", 0.0 },
  { "Wilmon", 1.0 },
  { "Wilmont", 1.0 },
  { "Wilmor", 1.0 },
  { "Wilmore", 1.0 },
  { "Wilmot", 0.9891068 },
  { "Wilmoth", 0.14423077 },
  { "Wilmur", 1.0 },
  { "Wilmuth", 0.0 },
  { "Wilna", 0.0 },
  { "Wilnelia", 0.0 },
  { "Wilner", 1.0 },
  { "Wilnesha", 0.0 },
  { "Wilnette", 0.0 },
  { "Wilniya", 0.0 },
  { "Wilodean", 0.0 },
  { "Wilodene", 0.0 },
  { "Wilodyne", 0.0 },
  { "Wilona", 0.0 },
  { "Wilroy", 1.0 },
  { "Wilsey", 1.0 },
  { "Wilsie", 0.091836736 },
  { "Wilson", 0.99585265 },
  { "Wilta", 0.0 },
  { "Wilton", 0.9952019 },
  { "Wiltz", 1.0 },
  { "Wilva", 0.0 },
  { "Wilver", 1.0 },
  { "Wily", 1.0 },
  { "Wimberley", 0.0 },
  { "Wimberly", 0.0 },
  { "Win", 0.90434784 },
  { "Wina", 0.0 },
  { "Winafred", 0.0 },
  { "Winburn", 1.0 },
  { "Wince", 1.0 },
  { "Winchell", 1.0 },
  { "Winchester", 1.0 },
  { "Wincie", 0.0 },
  { "Winda", 0.0 },
  { "Windale", 1.0 },
  { "Windee", 0.0 },
  { "Windel", 1.0 },
  { "Windell", 0.9929907 },
  { "Windfield", 1.0 },
  { "Windford", 1.0 },
  { "Windham", 1.0 },
  { "Windi", 0.0 },
  { "Windie", 0.0 },
  { "Windle", 1.0 },
  { "Windol", 1.0 },
  { "Windom", 1.0 },
  { "Windsor", 0.6894977 },
  { "Windy", 0.002434077 },
  { "Winefred", 0.0 },
  { "Winell", 0.0 },
  { "Winetta", 0.0 },
  { "Winette", 0.0 },
  { "Winferd", 1.0 },
  { "Winfield", 1.0 },
  { "Winford", 1.0 },
  { "Winfred", 0.9663277 },
  { "Winfrey", 1.0 },
  { "Winfried", 1.0 },
  { "Wing", 0.88297874 },
  { "Wingate", 1.0 },
  { "Wini", 0.0 },
  { "Winiferd", 0.0 },
  { "Winiford", 1.0 },
  { "Winifred", 0.025024889 },
  { "Winifrede", 0.0 },
  { "Winn", 1.0 },
  { "Winna", 0.0 },
  { "Winnefred", 0.0 },
  { "Winnell", 0.0 },
  { "Winner", 0.46875 },
  { "Winnette", 0.0 },
  { "Winni", 0.0 },
  { "Winnie", 0.009698413 },
  { "Winniefred", 0.0 },
  { "Winnifred", 0.0 },
  { "Winnona", 0.0 },
  { "Winny", 0.0 },
  { "Winogene", 0.0 },
  { "Winola", 0.0 },
  { "Winona", 0.0 },
  { "Winonah", 0.0 },
  { "Winrey", 0.0 },
  { "Winry", 0.0 },
  { "Winsel", 1.0 },
  { "Winslet", 0.0 },
  { "Winsley", 1.0 },
  { "Winslow", 0.91917294 },
  { "Winsome", 0.0 },
  { "Winson", 1.0 },
  { "Winsor", 1.0 },
  { "Winsten", 1.0 },
  { "Winston", 0.9964308 },
  { "Wint", 1.0 },
  { "Winta", 0.0 },
  { "Wintana", 0.0 },
  { "Winter", 0.064366125 },
  { "Winterrose", 0.0 },
  { "Wintford", 1.0 },
  { "Winthrop", 1.0 },
  { "Winton", 0.9973517 },
  { "Wintress", 0.0 },
  { "Wiona", 0.0 },
  { "Wirt", 1.0 },
  { "Wisam", 1.0 },
  { "Wisdom", 0.4698276 },
  { "Wise", 1.0 },
  { "Wishard", 1.0 },
  { "Wisin", 1.0 },
  { "Wisler", 1.0 },
  { "Wisper", 0.0 },
  { "Wissam", 1.0 },
  { "Wister", 1.0 },
  { "Wisteria", 0.0 },
  { "Witney", 0.0 },
  { "Witold", 1.0 },
  { "Witt", 1.0 },
  { "Witten", 0.9673913 },
  { "Wittney", 0.0 },
  { "Wiyaka", 0.0 },
  { "Wizdom", 1.0 },
  { "Wladimir", 1.0 },
  { "Wladyslaw", 1.0 },
  { "Wladyslawa", 0.0 },
  { "Wlliam", 1.0 },
  { "Wm", 0.96856725 },
  { "Wnedy", 0.0 },
  { "Wofford", 1.0 },
  { "Wojciech", 1.0 },
  { "Wolcott", 1.0 },
  { "Wolf", 1.0 },
  { "Wolfe", 1.0 },
  { "Wolfgang", 1.0 },
  { "Wolford", 1.0 },
  { "Wolfram", 1.0 },
  { "Won", 1.0 },
  { "Wonda", 0.0 },
  { "Wonder", 0.0 },
  { "Wondra", 0.0 },
  { "Wong", 1.0 },
  { "Wonnie", 0.2037037 },
  { "Wonya", 1.0 },
  { "Woo", 1.0 },
  { "Wood", 1.0 },
  { "Woodard", 1.0 },
  { "Woodfin", 1.0 },
  { "Woodford", 1.0 },
  { "Woodie", 0.85057473 },
  { "Woodland", 1.0 },
  { "Woodley", 1.0 },
  { "Woodro", 1.0 },
  { "Woodroe", 1.0 },
  { "Woodrow", 0.9963849 },
  { "Woodruff", 1.0 },
  { "Woods", 1.0 },
  { "Woodson", 1.0 },
  { "Woodward", 1.0 },
  { "Woody", 0.9975834 },
  { "Woojin", 1.0 },
  { "Worden", 1.0 },
  { "Wordie", 1.0 },
  { "Worley", 1.0 },
  { "Worn", 1.0 },
  { "Worner", 1.0 },
  { "Worren", 1.0 },
  { "Worth", 1.0 },
  { "Wortha", 0.0 },
  { "Wortham", 1.0 },
  { "Worthie", 1.0 },
  { "Worthington", 1.0 },
  { "Worthy", 0.98641306 },
  { "Wraith", 1.0 },
  { "Wrangler", 1.0 },
  { "Wray", 0.9738933 },
  { "Wrayanne", 0.0 },
  { "Wreatha", 0.0 },
  { "Wren", 0.17142184 },
  { "Wrenda", 0.0 },
  { "Wrenlee", 0.0 },
  { "Wrenleigh", 0.0 },
  { "Wrenley", 0.0 },
  { "Wrenly", 0.0 },
  { "Wrenn", 0.03649635 },
  { "Wrenna", 0.0 },
  { "Wrennyn", 1.0 },
  { "Wretha", 0.0 },
  { "Wrett", 1.0 },
  { "Wright", 1.0 },
  { "Wrigley", 0.6060606 },
  { "Wriley", 0.42857143 },
  { "Wryan", 1.0 },
  { "Wryder", 1.0 },
  { "Wrylee", 0.0 },
  { "Wryn", 0.0 },
  { "Wrynn", 0.0 },
  { "Wuanita", 0.0 },
  { "Wuendi", 0.0 },
  { "Wuendy", 0.0 },
  { "Wuilber", 1.0 },
  { "Wuilian", 1.0 },
  { "Wuilmer", 1.0 },
  { "Wulfric", 1.0 },
  { "Wwilliam", 1.0 },
  { "Wwlliam", 1.0 },
  { "Wyat", 1.0 },
  { "Wyatt", 0.99719673 },
  { "Wyatte", 1.0 },
  { "Wyconda", 0.0 },
  { "Wydell", 0.90909094 },
  { "Wydia", 0.0 },
  { "Wyeth", 1.0 },
  { "Wyett", 1.0 },
  { "Wyetta", 0.0 },
  { "Wyette", 1.0 },
  { "Wyhatt", 1.0 },
  { "Wyitt", 1.0 },
  { "Wykeem", 1.0 },
  { "Wykeisha", 0.0 },
  { "Wykena", 0.0 },
  { "Wykeria", 0.0 },
  { "Wykesha", 0.0 },
  { "Wykeshia", 0.0 },
  { "Wykisha", 0.0 },
  { "Wyla", 0.0 },
  { "Wylan", 1.0 },
  { "Wyland", 1.0 },
  { "Wylda", 0.0 },
  { "Wylde", 1.0 },
  { "Wylder", 0.9299363 },
  { "Wyle", 1.0 },
  { "Wylean", 0.0 },
  { "Wylee", 0.8035714 },
  { "Wyleen", 0.0 },
  { "Wyleigh", 0.0 },
  { "Wylen", 1.0 },
  { "Wylene", 0.0 },
  { "Wyler", 1.0 },
  { "Wyley", 1.0 },
  { "Wylie", 0.9350453 },
  { "Wyline", 0.0 },
  { "Wylla", 0.0 },
  { "Wylliam", 1.0 },
  { "Wyllis", 1.0 },
  { "Wyllow", 0.0 },
  { "Wyllys", 1.0 },
  { "Wylma", 0.0 },
  { "Wylodean", 0.0 },
  { "Wylodene", 0.0 },
  { "Wylodine", 0.0 },
  { "Wyma", 0.0 },
  { "Wyman", 1.0 },
  { "Wymer", 1.0 },
  { "Wymon", 1.0 },
  { "Wymond", 1.0 },
  { "Wyn", 0.82474226 },
  { "Wynda", 0.0 },
  { "Wynde", 0.0 },
  { "Wyndee", 0.0 },
  { "Wyndell", 1.0 },
  { "Wyndham", 1.0 },
  { "Wyndi", 0.0 },
  { "Wyndie", 0.0 },
  { "Wyndsor", 0.0 },
  { "Wyndy", 0.0 },
  { "Wynee", 0.0 },
  { "Wynell", 0.0058139535 },
  { "Wynelle", 0.0 },
  { "Wynema", 0.0 },
  { "Wynesha", 0.0 },
  { "Wynetta", 0.0 },
  { "Wynette", 0.0 },
  { "Wynisha", 0.0 },
  { "Wynn", 0.7162064 },
  { "Wynna", 0.0 },
  { "Wynne", 0.10351437 },
  { "Wynnell", 0.0 },
  { "Wynnette", 0.0 },
  { "Wynnie", 0.0 },
  { "Wynnston", 1.0 },
  { "Wynogene", 0.0 },
  { "Wynola", 0.0 },
  { "Wynoma", 0.0 },
  { "Wynona", 0.0 },
  { "Wynonah", 0.0 },
  { "Wynonia", 0.0 },
  { "Wynonna", 0.0 },
  { "Wynston", 1.0 },
  { "Wynter", 0.023763802 },
  { "Wynton", 1.0 },
  { "Wyolene", 0.0 },
  { "Wyoma", 0.0 },
  { "Wyomia", 0.0 },
  { "Wyoming", 0.0 },
  { "Wyona", 0.0 },
  { "Wyonda", 0.0 },
  { "Wyonia", 0.0 },
  { "Wyonna", 0.0 },
  { "Wyonne", 0.0 },
  { "Wyonnia", 0.0 },
  { "Wyota", 0.0 },
  { "Wystan", 1.0 },
  { "Wysteria", 0.0 },
  { "Wyteria", 0.0 },
  { "Wytina", 0.0 },
  { "Wyvetta", 0.0 },
  { "Wyvette", 0.0 },
  { "Wyvonia", 0.0 },
  { "Wyvonna", 0.0 },
  { "Wyvonne", 0.0 },
  { "Wyvonnia", 0.0 },
  { "Wyzetta", 0.0 },
  { "Xabi", 1.0 },
  { "Xabian", 1.0 },
  { "Xabier", 1.0 },
  { "Xabriel", 1.0 },
  { "Xachary", 1.0 },
  { "Xachery", 1.0 },
  { "Xaden", 1.0 },
  { "Xadiel", 1.0 },
  { "Xadrian", 1.0 },
  { "Xadyn", 1.0 },
  { "Xaeden", 1.0 },
  { "Xaedyn", 1.0 },
  { "Xael", 1.0 },
  { "Xaela", 0.0 },
  { "Xaelah", 0.0 },
  { "Xai", 0.8507463 },
  { "Xaia", 0.0 },
  { "Xaida", 0.0 },
  { "Xaiden", 1.0 },
  { "Xaidyn", 1.0 },
  { "Xailey", 0.0 },
  { "Xain", 1.0 },
  { "Xaine", 1.0 },
  { "Xaira", 0.0 },
  { "Xaire", 1.0 },
  { "Xairexis", 0.0 },
  { "Xaiver", 1.0 },
  { "Xaivier", 1.0 },
  { "Xaivion", 1.0 },
  { "Xalen", 1.0 },
  { "Xalia", 0.0 },
  { "Xally", 0.0 },
  { "Xamara", 0.0 },
  { "Xamir", 1.0 },
  { "Xamira", 0.0 },
  { "Xamiyah", 0.0 },
  { "Xan", 0.48523206 },
  { "Xana", 0.0 },
  { "Xanadu", 0.0 },
  { "Xandar", 1.0 },
  { "Xande", 1.0 },
  { "Xanden", 1.0 },
  { "Xander", 0.9990224 },
  { "Xandir", 1.0 },
  { "Xandra", 0.0 },
  { "Xandrea", 0.0 },
  { "Xandria", 0.0 },
  { "Xandyr", 1.0 },
  { "Xane", 1.0 },
  { "Xang", 1.0 },
  { "Xania", 0.0 },
  { "Xaniah", 0.0 },
  { "Xaniel", 1.0 },
  { "Xaniya", 0.0 },
  { "Xaniyah", 0.0 },
  { "Xannder", 1.0 },
  { "Xannon", 1.0 },
  { "Xanthe", 0.0 },
  { "Xanthia", 0.0 },
  { "Xanthus", 1.0 },
  { "Xanti", 1.0 },
  { "Xao", 1.0 },
  { "Xara", 0.0 },
  { "Xareni", 0.0 },
  { "Xareny", 0.0 },
  { "Xaria", 0.0 },
  { "Xariah", 0.0 },
  { "Xarianna", 0.0 },
  { "Xarielys", 0.0 },
  { "Xariya", 0.0 },
  { "Xariyah", 0.0 },
  { "Xaven", 1.0 },
  { "Xaveon", 1.0 },
  { "Xaver", 1.0 },
  { "Xaveria", 0.0 },
  { "Xavi", 0.9962963 },
  { "Xavia", 0.020833334 },
  { "Xavian", 1.0 },
  { "Xaviana", 0.0 },
  { "Xavianna", 0.0 },
  { "Xaviar", 1.0 },
  { "Xaviea", 0.0 },
  { "Xaviel", 1.0 },
  { "Xavien", 1.0 },
  { "Xavier", 0.993472 },
  { "Xaviera", 0.0 },
  { "Xavion", 1.0 },
  { "Xaviona", 0.0 },
  { "Xavior", 1.0 },
  { "Xaviour", 1.0 },
  { "Xavious", 1.0 },
  { "Xavius", 1.0 },
  { "Xavy", 1.0 },
  { "Xavyer", 1.0 },
  { "Xayden", 1.0 },
  { "Xaydin", 1.0 },
  { "Xayla", 0.0 },
  { "Xaylee", 0.0 },
  { "Xaylen", 1.0 },
  { "Xaylie", 0.0 },
  { "Xayne", 1.0 },
  { "Xayvier", 1.0 },
  { "Xayvion", 1.0 },
  { "Xazavier", 1.0 },
  { "Xazier", 1.0 },
  { "Xcaret", 0.0 },
  { "Xea", 0.0 },
  { "Xee", 0.0 },
  { "Xela", 0.0 },
  { "Xelha", 0.0 },
  { "Xen", 0.8 },
  { "Xena", 0.0 },
  { "Xendaya", 0.0 },
  { "Xeng", 1.0 },
  { "Xenia", 0.0 },
  { "Xeniah", 0.0 },
  { "Xeniya", 0.0 },
  { "Xeniyah", 0.0 },
  { "Xenna", 0.0 },
  { "Xenophon", 1.0 },
  { "Xenya", 0.0 },
  { "Xerxes", 1.0 },
  { "Xeryus", 1.0 },
  { "Xhaiden", 1.0 },
  { "Xhavier", 1.0 },
  { "Xi", 0.0 },
  { "Xia", 0.0 },
  { "Xiadani", 0.0 },
  { "Xiah", 0.0 },
  { "Xiamara", 0.0 },
  { "Xian", 0.7147541 },
  { "Xiana", 0.0 },
  { "Xiang", 1.0 },
  { "Xiani", 0.0 },
  { "Xianna", 0.0 },
  { "Xiao", 0.08450704 },
  { "Xiaoxi", 0.0 },
  { "Xiara", 0.0 },
  { "Xiaver", 1.0 },
  { "Xica", 0.0 },
  { "Xiclali", 0.0 },
  { "Xiclaly", 0.0 },
  { "Xienna", 0.0 },
  { "Ximena", 0.0005703693 },
  { "Ximenna", 0.0 },
  { "Xin", 0.1448598 },
  { "Xina", 0.0 },
  { "Xing", 0.41666666 },
  { "Xinia", 0.0 },
  { "Xinran", 0.0 },
  { "Xinrui", 0.0 },
  { "Xinyan", 0.0 },
  { "Xinyi", 0.0 },
  { "Xinyu", 0.0 },
  { "Xinyue", 0.0 },
  { "Xiola", 0.0 },
  { "Xiomar", 1.0 },
  { "Xiomara", 0.0 },
  { "Xiomari", 0.0 },
  { "Xiomary", 0.0 },
  { "Xiomayra", 0.0 },
  { "Xiomy", 0.0 },
  { "Xion", 0.9038961 },
  { "Xiong", 1.0 },
  { "Xirena", 0.0 },
  { "Xithlaly", 0.0 },
  { "Xitlali", 0.0 },
  { "Xitlalic", 0.0 },
  { "Xitlalie", 0.0 },
  { "Xitlalit", 0.0 },
  { "Xitlalith", 0.0 },
  { "Xitlalli", 0.0 },
  { "Xitlally", 0.0 },
  { "Xitlaly", 0.0 },
  { "Xitllali", 0.0 },
  { "Xiya", 0.0 },
  { "Xiyah", 0.0 },
  { "Xizavier", 1.0 },
  { "Xjavier", 1.0 },
  { "Xoaquin", 1.0 },
  { "Xochi", 0.0 },
  { "Xochil", 0.0 },
  { "Xochilt", 0.0 },
  { "Xochilth", 0.0 },
  { "Xochiquetzal", 0.0 },
  { "Xochithl", 0.0 },
  { "Xochitl", 0.0 },
  { "Xoe", 0.0 },
  { "Xoey", 0.0 },
  { "Xoi", 0.0 },
  { "Xoie", 0.0 },
  { "Xola", 0.0 },
  { "Xolani", 0.0 },
  { "Xong", 0.23809524 },
  { "Xophia", 0.0 },
  { "Xotchil", 0.0 },
  { "Xou", 1.0 },
  { "Xsavier", 1.0 },
  { "Xsavion", 1.0 },
  { "Xsavior", 1.0 },
  { "Xu", 1.0 },
  { "Xuan", 0.09615385 },
  { "Xue", 0.79136693 },
  { "Xuri", 0.0 },
  { "Xuxa", 0.0 },
  { "Xxavier", 1.0 },
  { "Xya", 0.0 },
  { "Xyaire", 1.0 },
  { "Xyan", 1.0 },
  { "Xyana", 0.0 },
  { "Xyanna", 0.0 },
  { "Xyion", 0.7916667 },
  { "Xyla", 0.0 },
  { "Xylah", 0.0 },
  { "Xylan", 1.0 },
  { "Xylar", 1.0 },
  { "Xylee", 0.0 },
  { "Xylen", 1.0 },
  { "Xyler", 1.0 },
  { "Xylia", 0.0 },
  { "Xyliah", 0.0 },
  { "Xyliana", 0.0 },
  { "Xylie", 0.0 },
  { "Xylina", 0.0 },
  { "Xylon", 1.0 },
  { "Xylus", 1.0 },
  { "Xymena", 0.0 },
  { "Xyomara", 0.0 },
  { "Xyon", 0.9054054 },
  { "Xyra", 0.0 },
  { "Xyriah", 0.0 },
  { "Xzadrian", 1.0 },
  { "Xzaevion", 1.0 },
  { "Xzaiden", 1.0 },
  { "Xzaiver", 1.0 },
  { "Xzaivier", 1.0 },
  { "Xzander", 1.0 },
  { "Xzandria", 0.0 },
  { "Xzaria", 0.0 },
  { "Xzariah", 0.0 },
  { "Xzarion", 1.0 },
  { "Xzaveon", 1.0 },
  { "Xzavia", 0.30232558 },
  { "Xzavian", 1.0 },
  { "Xzaviar", 1.0 },
  { "Xzavien", 1.0 },
  { "Xzavier", 1.0 },
  { "Xzavion", 1.0 },
  { "Xzavior", 1.0 },
  { "Xzavious", 1.0 },
  { "Xzavius", 1.0 },
  { "Xzavyer", 1.0 },
  { "Xzayden", 1.0 },
  { "Xzayvian", 1.0 },
  { "Xzayvien", 1.0 },
  { "Xzayvier", 1.0 },
  { "Xzayvion", 1.0 },
  { "Xzorion", 1.0 },
  { "Ya", 0.12820514 },
  { "Yaa", 0.0 },
  { "Yaacov", 1.0 },
  { "Yaakov", 1.0 },
  { "Yaalini", 0.0 },
  { "Yaara", 0.0 },
  { "Yaaseen", 1.0 },
  { "Yaasir", 1.0 },
  { "Yaasmeen", 0.0 },
  { "Yaasmiyn", 0.0 },
  { "Yabdiel", 1.0 },
  { "Yabriel", 1.0 },
  { "Yabsera", 0.25 },
  { "Yachet", 0.0 },
  { "Yachi", 0.0 },
  { "Yachica", 0.0 },
  { "Yachira", 0.0 },
  { "Yachiyo", 0.0 },
  { "Yaciel", 1.0 },
  { "Yacine", 0.5714286 },
  { "Yacir", 1.0 },
  { "Yackelin", 0.0 },
  { "Yackov", 1.0 },
  { "Yacob", 1.0 },
  { "Yacoub", 1.0 },
  { "Yacouba", 1.0 },
  { "Yacov", 1.0 },
  { "Yacqub", 1.0 },
  { "Yacquelin", 0.0 },
  { "Yacqueline", 0.0 },
  { "Yad", 1.0 },
  { "Yadel", 1.0 },
  { "Yadelin", 0.0 },
  { "Yadelis", 0.0 },
  { "Yadeliz", 0.0 },
  { "Yaden", 1.0 },
  { "Yader", 1.0 },
  { "Yadhiel", 1.0 },
  { "Yadhir", 1.0 },
  { "Yadhira", 0.0 },
  { "Yadian", 1.0 },
  { "Yadid", 1.0 },
  { "Yadiel", 1.0 },
  { "Yadieliz", 0.0 },
  { "Yadier", 1.0 },
  { "Yadin", 1.0 },
  { "Yadir", 1.0 },
  { "Yadira", 0.0018599385 },
  { "Yadirah", 0.0 },
  { "Yaditzel", 0.0 },
  { "Yadon", 1.0 },
  { "Yadria", 0.0 },
  { "Yadrian", 1.0 },
  { "Yadriel", 1.0 },
  { "Yae", 0.0 },
  { "Yaeko", 0.0 },
  { "Yael", 0.47402078 },
  { "Yaeli", 0.0 },
  { "Yaelin", 0.0 },
  { "Yaelis", 0.0 },
  { "Yaelle", 0.0 },
  { "Yaely", 0.0 },
  { "Yaeno", 0.0 },
  { "Yafa", 0.0 },
  { "Yafet", 1.0 },
  { "Yaffa", 0.0 },
  { "Yafi", 1.0 },
  { "Yagaira", 0.0 },
  { "Yagmur", 0.0 },
  { "Yago", 1.0 },
  { "Yahaida", 0.0 },
  { "Yahaira", 0.0 },
  { "Yahani", 0.0 },
  { "Yahara", 0.0 },
  { "Yahari", 0.0 },
  { "Yaharia", 0.0 },
  { "Yahav", 0.5 },
  { "Yahayra", 0.0 },
  { "Yahdira", 0.0 },
  { "Yahel", 0.971223 },
  { "Yaheli", 0.0 },
  { "Yahia", 1.0 },
  { "Yahilin", 0.0 },
  { "Yahir", 0.99495816 },
  { "Yahira", 0.0 },
  { "Yahire", 0.0 },
  { "Yahli", 0.6333333 },
  { "Yahmir", 1.0 },
  { "Yahna", 0.0 },
  { "Yahne", 0.0 },
  { "Yahoshua", 1.0 },
  { "Yahriel", 1.0 },
  { "Yahshua", 1.0 },
  { "Yahsir", 1.0 },
  { "Yahushua", 1.0 },
  { "Yahve", 1.0 },
  { "Yahveh", 1.0 },
  { "Yahvi", 0.0 },
  { "Yahweh", 1.0 },
  { "Yahya", 0.9972426 },
  { "Yahye", 1.0 },
  { "Yahzarah", 0.0 },
  { "Yaicha", 0.0 },
  { "Yaidden", 1.0 },
  { "Yaidel", 1.0 },
  { "Yaiden", 1.0 },
  { "Yaila", 0.0 },
  { "Yaileen", 0.0 },
  { "Yailem", 0.0 },
  { "Yailen", 0.0 },
  { "Yailene", 0.0 },
  { "Yaileny", 0.0 },
  { "Yailenys", 0.0 },
  { "Yailet", 0.0 },
  { "Yaili", 0.0 },
  { "Yailin", 0.0 },
  { "Yailine", 0.0 },
  { "Yailyn", 0.0 },
  { "Yailynn", 0.0 },
  { "Yaima", 0.0 },
  { "Yaimara", 0.0 },
  { "Yain", 1.0 },
  { "Yaindhi", 0.0 },
  { "Yaindhy", 0.0 },
  { "Yair", 1.0 },
  { "Yaira", 0.0 },
  { "Yaire", 0.0 },
  { "Yairely", 0.0 },
  { "Yairet", 0.0 },
  { "Yaireth", 0.0 },
  { "Yairi", 0.0 },
  { "Yairis", 0.0 },
  { "Yairon", 1.0 },
  { "Yairy", 0.0 },
  { "Yaisa", 0.0 },
  { "Yaisha", 0.0 },
  { "Yaitza", 0.0 },
  { "Yaiza", 0.0 },
  { "Yajahira", 0.0 },
  { "Yajaida", 0.0 },
  { "Yajaira", 0.00092421443 },
  { "Yajat", 1.0 },
  { "Yajayra", 0.0 },
  { "Yakeem", 1.0 },
  { "Yakeisha", 0.0 },
  { "Yakelin", 0.0 },
  { "Yakeline", 0.0 },
  { "Yakelyn", 0.0 },
  { "Yakema", 0.0 },
  { "Yakia", 0.0 },
  { "Yakima", 0.0 },
  { "Yakina", 0.0 },
  { "Yakini", 0.0 },
  { "Yakir", 1.0 },
  { "Yakira", 0.0 },
  { "Yakirah", 0.0 },
  { "Yakisha", 0.0 },
  { "Yakita", 0.0 },
  { "Yakob", 1.0 },
  { "Yakov", 1.0 },
  { "Yaksh", 1.0 },
  { "Yakub", 1.0 },
  { "Yakyra", 0.0 },
  { "Yalaina", 0.0 },
  { "Yalana", 0.0 },
  { "Yalanda", 0.0 },
  { "Yalani", 0.0 },
  { "Yalda", 0.0 },
  { "Yale", 1.0 },
  { "Yaleena", 0.0 },
  { "Yaleiza", 0.0 },
  { "Yalena", 0.0 },
  { "Yaleni", 0.0 },
  { "Yaleska", 0.0 },
  { "Yalexa", 0.0 },
  { "Yalexi", 0.0 },
  { "Yalexia", 0.0 },
  { "Yalexis", 0.0 },
  { "Yaleyza", 0.0 },
  { "Yali", 0.3968254 },
  { "Yalia", 0.0 },
  { "Yalimar", 0.0 },
  { "Yalina", 0.0 },
  { "Yalini", 0.0 },
  { "Yalisa", 0.0 },
  { "Yalissa", 0.0 },
  { "Yalitza", 0.0 },
  { "Yalixa", 0.0 },
  { "Yaliyah", 0.0 },
  { "Yalonda", 0.0 },
  { "Yama", 1.0 },
  { "Yamaira", 0.0 },
  { "Yamajesty", 1.0 },
  { "Yamal", 1.0 },
  { "Yaman", 1.0 },
  { "Yamani", 0.104166664 },
  { "Yamara", 0.0 },
  { "Yamari", 0.07534247 },
  { "Yamarie", 0.0 },
  { "Yamaris", 0.0 },
  { "Yamato", 1.0 },
  { "Yamaya", 0.0 },
  { "Yameen", 1.0 },
  { "Yameka", 0.0 },
  { "Yamel", 0.1452514 },
  { "Yameli", 0.0 },
  { "Yamelin", 0.0 },
  { "Yamely", 0.0 },
  { "Yamen", 1.0 },
  { "Yami", 0.0 },
  { "Yamia", 0.0 },
  { "Yamiah", 0.0 },
  { "Yamika", 0.0 },
  { "Yamil", 0.95301026 },
  { "Yamila", 0.0 },
  { "Yamile", 0.0 },
  { "Yamiled", 0.0 },
  { "Yamilee", 0.0 },
  { "Yamiles", 0.0 },
  { "Yamilet", 0.0 },
  { "Yamileth", 0.0 },
  { "Yamilett", 0.0 },
  { "Yamilette", 0.0 },
  { "Yamiletz", 0.0 },
  { "Yamilex", 0.0 },
  { "Yamiley", 0.0 },
  { "Yamilez", 0.0 },
  { "Yamili", 0.0 },
  { "Yamilka", 0.0 },
  { "Yamille", 0.0 },
  { "Yamillet", 0.0 },
  { "Yamily", 0.0 },
  { "Yamin", 1.0 },
  { "Yamina", 0.0 },
  { "Yaminah", 0.0 },
  { "Yamini", 0.0 },
  { "Yamir", 1.0 },
  { "Yamira", 0.0 },
  { "Yamisha", 0.0 },
  { "Yamiyah", 0.0 },
  { "Yamna", 0.0 },
  { "Yan", 0.8764783 },
  { "Yana", 0.0 },
  { "Yanabah", 0.0 },
  { "Yanae", 0.0 },
  { "Yanahi", 0.0 },
  { "Yanai", 0.0 },
  { "Yanaira", 0.0 },
  { "Yanairis", 0.0 },
  { "Yanais", 0.0 },
  { "Yanaiza", 0.0 },
  { "Yanal", 1.0 },
  { "Yanali", 0.0 },
  { "Yanalis", 0.0 },
  { "Yanaliz", 0.0 },
  { "Yanara", 0.0 },
  { "Yanari", 0.0 },
  { "Yanay", 0.0 },
  { "Yancarlo", 1.0 },
  { "Yancarlos", 1.0 },
  { "Yance", 1.0 },
  { "Yancey", 0.97766095 },
  { "Yanci", 0.0 },
  { "Yancy", 0.84061515 },
  { "Yandel", 0.99725443 },
  { "Yandell", 1.0 },
  { "Yander", 1.0 },
  { "Yanderi", 0.0 },
  { "Yandi", 0.2777778 },
  { "Yandiel", 1.0 },
  { "Yandier", 1.0 },
  { "Yandriel", 1.0 },
  { "Yandry", 1.0 },
  { "Yandy", 0.375 },
  { "Yaned", 0.0 },
  { "Yaneira", 0.0 },
  { "Yaneiry", 0.0 },
  { "Yaneisha", 0.0 },
  { "Yaneisi", 0.0 },
  { "Yaneisy", 0.0 },
  { "Yanel", 0.0 },
  { "Yanela", 0.0 },
  { "Yaneli", 0.0 },
  { "Yanelie", 0.0 },
  { "Yanelis", 0.0 },
  { "Yaneliz", 0.0 },
  { "Yanell", 0.0 },
  { "Yanelle", 0.0 },
  { "Yanelli", 0.0 },
  { "Yanellie", 0.0 },
  { "Yanelly", 0.0 },
  { "Yanely", 0.0 },
  { "Yanelys", 0.0 },
  { "Yaneri", 0.0 },
  { "Yaneris", 0.0 },
  { "Yanesa", 0.0 },
  { "Yanesha", 0.0 },
  { "Yanessa", 0.0 },
  { "Yanet", 0.0 },
  { "Yaneth", 0.0 },
  { "Yanett", 0.0 },
  { "Yanette", 0.0 },
  { "Yanetzi", 0.0 },
  { "Yanetzy", 0.0 },
  { "Yanexi", 0.0 },
  { "Yang", 0.85079366 },
  { "Yani", 0.2559055 },
  { "Yania", 0.0 },
  { "Yaniah", 0.0 },
  { "Yanibel", 0.0 },
  { "Yanice", 0.0 },
  { "Yanichel", 0.0 },
  { "Yanick", 0.5588235 },
  { "Yaniece", 0.0 },
  { "Yaniel", 1.0 },
  { "Yanielis", 0.0 },
  { "Yanieliz", 0.0 },
  { "Yanik", 1.0 },
  { "Yanika", 0.0 },
  { "Yanil", 0.0 },
  { "Yanilen", 0.0 },
  { "Yanilet", 0.0 },
  { "Yanileth", 0.0 },
  { "Yanill", 0.0 },
  { "Yanin", 0.0 },
  { "Yanina", 0.0 },
  { "Yanine", 0.0 },
  { "Yanique", 0.0 },
  { "Yanira", 0.0 },
  { "Yanire", 0.0 },
  { "Yaniris", 0.0 },
  { "Yanis", 0.6944444 },
  { "Yanisa", 0.0 },
  { "Yanise", 0.0 },
  { "Yanisha", 0.0 },
  { "Yanisley", 0.0 },
  { "Yanissa", 0.0 },
  { "Yanisse", 0.0 },
  { "Yanita", 0.0 },
  { "Yanitza", 0.0 },
  { "Yanitzi", 0.0 },
  { "Yanitzia", 0.0 },
  { "Yaniv", 1.0 },
  { "Yanixa", 0.0 },
  { "Yanixan", 1.0 },
  { "Yaniya", 0.0 },
  { "Yaniyah", 0.0 },
  { "Yaniz", 0.0 },
  { "Yankarlo", 1.0 },
  { "Yankee", 1.0 },
  { "Yankel", 1.0 },
  { "Yanko", 1.0 },
  { "Yanky", 1.0 },
  { "Yann", 1.0 },
  { "Yanna", 0.0 },
  { "Yannai", 1.0 },
  { "Yannel", 0.0 },
  { "Yanneli", 0.0 },
  { "Yannely", 0.0 },
  { "Yannery", 0.0 },
  { "Yannet", 0.0 },
  { "Yanneth", 0.0 },
  { "Yannette", 0.0 },
  { "Yanni", 0.7620222 },
  { "Yannick", 1.0 },
  { "Yannie", 0.0 },
  { "Yannik", 1.0 },
  { "Yannira", 0.0 },
  { "Yannis", 0.7535885 },
  { "Yannuel", 1.0 },
  { "Yansel", 1.0 },
  { "Yansi", 0.0 },
  { "Yansiel", 1.0 },
  { "Yanuel", 1.0 },
  { "Yanxiel", 1.0 },
  { "Yanzel", 1.0 },
  { "Yanziel", 1.0 },
  { "Yao", 1.0 },
  { "Yaphet", 0.9791667 },
  { "Yaphett", 1.0 },
  { "Yaqeen", 0.5 },
  { "Yaqoob", 1.0 },
  { "Yaqoub", 1.0 },
  { "Yaqub", 1.0 },
  { "Yaquelin", 0.0 },
  { "Yaqueline", 0.0 },
  { "Yaquelyn", 0.0 },
  { "Yar", 0.0 },
  { "Yara", 0.0 },
  { "Yarah", 0.0 },
  { "Yarali", 0.0 },
  { "Yaralis", 0.0 },
  { "Yaraliz", 0.0 },
  { "Yarazet", 0.0 },
  { "Yarazeth", 0.0 },
  { "Yarden", 0.40449437 },
  { "Yardena", 0.0 },
  { "Yardley", 0.53513515 },
  { "Yarecsi", 0.0 },
  { "Yared", 0.83428574 },
  { "Yareily", 0.0 },
  { "Yarel", 0.25464192 },
  { "Yareli", 0.0 },
  { "Yarelie", 0.0 },
  { "Yarelii", 0.0 },
  { "Yarelin", 0.0 },
  { "Yarelis", 0.0 },
  { "Yareliz", 0.0 },
  { "Yarell", 1.0 },
  { "Yarelli", 0.0 },
  { "Yarelly", 0.0 },
  { "Yarely", 0.0 },
  { "Yarelyn", 0.0 },
  { "Yarelys", 0.0 },
  { "Yaremi", 0.0 },
  { "Yaremy", 0.0 },
  { "Yaren", 0.0 },
  { "Yareni", 0.0 },
  { "Yarenis", 0.0 },
  { "Yareny", 0.0 },
  { "Yarenys", 0.0 },
  { "Yaresi", 0.0 },
  { "Yaresli", 0.0 },
  { "Yaresly", 0.0 },
  { "Yaressi", 0.0 },
  { "Yaret", 0.36363637 },
  { "Yaretci", 0.0 },
  { "Yaretcy", 0.0 },
  { "Yareth", 0.42821783 },
  { "Yarethzi", 0.0 },
  { "Yarethzy", 0.0 },
  { "Yaretsi", 0.0 },
  { "Yaretsy", 0.0 },
  { "Yarett", 0.0 },
  { "Yaretza", 0.0 },
  { "Yaretzi", 0.0 },
  { "Yaretzie", 0.0 },
  { "Yaretzy", 0.0 },
  { "Yarexi", 0.0 },
  { "Yarexy", 0.0 },
  { "Yari", 0.0754717 },
  { "Yariah", 0.0 },
  { "Yariana", 0.0 },
  { "Yarianna", 0.0 },
  { "Yarianny", 0.0 },
  { "Yaribel", 0.0 },
  { "Yaribeth", 0.0 },
  { "Yaricelis", 0.0 },
  { "Yaricsa", 0.0 },
  { "Yaricza", 0.0 },
  { "Yarida", 0.0 },
  { "Yariel", 0.99224204 },
  { "Yariela", 0.0 },
  { "Yarieli", 0.0 },
  { "Yarielis", 0.0 },
  { "Yarieliz", 0.0 },
  { "Yariely", 0.0 },
  { "Yarielys", 0.0 },
  { "Yarik", 1.0 },
  { "Yarilis", 0.0 },
  { "Yariliz", 0.0 },
  { "Yarilyn", 0.0 },
  { "Yarima", 0.0 },
  { "Yarimar", 0.0 },
  { "Yarin", 0.8611111 },
  { "Yarira", 0.0 },
  { "Yaris", 0.0 },
  { "Yarisa", 0.0 },
  { "Yarisamar", 0.0 },
  { "Yarisbel", 0.0 },
  { "Yarisbeth", 0.0 },
  { "Yarisel", 0.0 },
  { "Yarisleidi", 0.0 },
  { "Yarisleidy", 0.0 },
  { "Yarisley", 0.0 },
  { "Yarisma", 0.0 },
  { "Yarismar", 0.0 },
  { "Yarissa", 0.0 },
  { "Yarithza", 0.0 },
  { "Yaritsa", 0.0 },
  { "Yaritssa", 0.0 },
  { "Yaritza", 0.0007308873 },
  { "Yaritzel", 0.0 },
  { "Yaritzi", 0.0 },
  { "Yaritzia", 0.0 },
  { "Yaritzy", 0.0 },
  { "Yaritzza", 0.0 },
  { "Yarixa", 0.0 },
  { "Yarixsa", 0.0 },
  { "Yariza", 0.0 },
  { "Yarizel", 0.0 },
  { "Yarizmar", 0.0 },
  { "Yarleth", 0.0 },
  { "Yarley", 1.0 },
  { "Yarlin", 0.0 },
  { "Yarlyn", 0.0 },
  { "Yarnell", 0.42105263 },
  { "Yarnelle", 0.0 },
  { "Yaro", 1.0 },
  { "Yaron", 1.0 },
  { "Yaroslav", 1.0 },
  { "Yaroslava", 0.0 },
  { "Yarrow", 0.22857143 },
  { "Yary", 0.0 },
  { "Yasaira", 0.0 },
  { "Yasaman", 0.0 },
  { "Yasameen", 0.0 },
  { "Yasamin", 0.0 },
  { "Yasamine", 0.0 },
  { "Yasari", 0.0 },
  { "Yaschia", 0.0 },
  { "Yaschica", 0.0 },
  { "Yaseen", 1.0 },
  { "Yaseer", 1.0 },
  { "Yasel", 1.0 },
  { "Yaselin", 0.0 },
  { "Yasemin", 0.0 },
  { "Yasen", 1.0 },
  { "Yasenia", 0.0 },
  { "Yaser", 1.0 },
  { "Yash", 1.0 },
  { "Yasha", 0.175 },
  { "Yashar", 1.0 },
  { "Yashas", 1.0 },
  { "Yashasvi", 0.0 },
  { "Yashaswini", 0.0 },
  { "Yashdeep", 1.0 },
  { "Yasheca", 0.0 },
  { "Yasheika", 0.0 },
  { "Yasheka", 0.0 },
  { "Yashekia", 0.0 },
  { "Yashi", 0.0 },
  { "Yashia", 0.0 },
  { "Yashica", 0.0 },
  { "Yashika", 0.0 },
  { "Yashima", 0.0 },
  { "Yashir", 1.0 },
  { "Yashira", 0.0 },
  { "Yashita", 0.0 },
  { "Yashley", 0.0 },
  { "Yashmit", 1.0 },
  { "Yashoda", 0.0 },
  { "Yashraj", 1.0 },
  { "Yashua", 1.0 },
  { "Yashveer", 1.0 },
  { "Yashvi", 0.0 },
  { "Yashwant", 1.0 },
  { "Yashwanth", 1.0 },
  { "Yashwin", 1.0 },
  { "Yasiah", 0.0 },
  { "Yasiel", 1.0 },
  { "Yasiin", 1.0 },
  { "Yasim", 1.0 },
  { "Yasin", 0.9970743 },
  { "Yasina", 0.0 },
  { "Yasine", 1.0 },
  { "Yasir", 1.0 },
  { "Yasira", 0.0 },
  { "Yasirah", 0.0 },
  { "Yasiris", 0.0 },
  { "Yasleen", 0.0 },
  { "Yaslen", 0.0 },
  { "Yaslene", 0.0 },
  { "Yasley", 0.0 },
  { "Yaslin", 0.0 },
  { "Yasline", 0.0 },
  { "Yasly", 0.0 },
  { "Yaslyn", 0.0 },
  { "Yaslynn", 0.0 },
  { "Yasmaine", 0.0 },
  { "Yasmani", 1.0 },
  { "Yasmany", 1.0 },
  { "Yasmean", 0.0 },
  { "Yasmeen", 0.0 },
  { "Yasmeena", 0.0 },
  { "Yasmein", 0.0 },
  { "Yasmeli", 0.0 },
  { "Yasmely", 0.0 },
  { "Yasmen", 0.0 },
  { "Yasmene", 0.0 },
  { "Yasmiin", 0.0 },
  { "Yasmim", 0.0 },
  { "Yasmin", 0.0041373605 },
  { "Yasmina", 0.0 },
  { "Yasminda", 0.0 },
  { "Yasmine", 0.0004072988 },
  { "Yasmir", 0.0 },
  { "Yasmira", 0.0 },
  { "Yasmyn", 0.0 },
  { "Yasmyne", 0.0 },
  { "Yasna", 0.0 },
  { "Yasniel", 1.0 },
  { "Yasseen", 1.0 },
  { "Yassen", 1.0 },
  { "Yasser", 1.0 },
  { "Yassiel", 1.0 },
  { "Yassin", 0.98311687 },
  { "Yassine", 1.0 },
  { "Yassir", 1.0 },
  { "Yassmin", 0.0 },
  { "Yassmine", 0.0 },
  { "Yastelin", 0.0 },
  { "Yasuhiro", 1.0 },
  { "Yasuko", 0.0 },
  { "Yasunobu", 1.0 },
  { "Yasuo", 1.0 },
  { "Yasuri", 0.0 },
  { "Yatana", 0.0 },
  { "Yatasha", 0.0 },
  { "Yates", 1.0 },
  { "Yatharth", 1.0 },
  { "Yathziri", 0.0 },
  { "Yathziry", 0.0 },
  { "Yatin", 1.0 },
  { "Yatisha", 0.0 },
  { "Yatniel", 1.0 },
  { "Yatta", 0.0 },
  { "Yatzari", 0.0 },
  { "Yatzary", 0.0 },
  { "Yatzel", 0.0 },
  { "Yatziel", 1.0 },
  { "Yatzil", 0.0 },
  { "Yatziri", 0.0 },
  { "Yatziry", 0.0 },
  { "Yavanna", 0.0 },
  { "Yavette", 0.0 },
  { "Yavian", 1.0 },
  { "Yaviel", 1.0 },
  { "Yavier", 1.0 },
  { "Yavin", 1.0 },
  { "Yavonda", 0.0 },
  { "Yavonna", 0.0 },
  { "Yavonne", 0.0 },
  { "Yavuz", 1.0 },
  { "Yaw", 1.0 },
  { "Yawanda", 0.0 },
  { "Yaxaira", 0.0 },
  { "Yaxel", 1.0 },
  { "Yaxeni", 0.0 },
  { "Yaxiel", 1.0 },
  { "Yaxiry", 0.0 },
  { "Yaxye", 1.0 },
  { "Yaya", 1.0 },
  { "Yayeko", 0.0 },
  { "Yaylin", 0.0 },
  { "Yayoi", 0.0 },
  { "Yayra", 0.0 },
  { "Yaz", 0.0 },
  { "Yazaira", 0.0 },
  { "Yazan", 1.0 },
  { "Yazareth", 0.0 },
  { "Yazayra", 0.0 },
  { "Yazbeth", 0.0 },
  { "Yazeed", 1.0 },
  { "Yazel", 0.0 },
  { "Yazen", 1.0 },
  { "Yazhini", 0.0 },
  { "Yazid", 1.0 },
  { "Yaziel", 1.0 },
  { "Yazil", 0.0 },
  { "Yazin", 1.0 },
  { "Yazir", 1.0 },
  { "Yazira", 0.0 },
  { "Yazlee", 0.0 },
  { "Yazleemar", 0.0 },
  { "Yazleen", 0.0 },
  { "Yazlen", 0.0 },
  { "Yazlene", 0.0 },
  { "Yazlin", 0.0 },
  { "Yazline", 0.0 },
  { "Yazlyn", 0.0 },
  { "Yazlynn", 0.0 },
  { "Yazmarie", 0.0 },
  { "Yazmeen", 0.0 },
  { "Yazmen", 0.0 },
  { "Yazmene", 0.0 },
  { "Yazmin", 0.00085490965 },
  { "Yazmina", 0.0 },
  { "Yazmine", 0.0 },
  { "Yazmyn", 0.0 },
  { "Yazmyne", 0.0 },
  { "Yazn", 1.0 },
  { "Yazuri", 0.0 },
  { "Yazzie", 1.0 },
  { "Yazziel", 1.0 },
  { "Yazzmin", 0.0 },
  { "Yazzmine", 0.0 },
  { "Ydania", 0.0 },
  { "Ye", 0.57894737 },
  { "Yeab", 1.0 },
  { "Yeabsera", 0.0 },
  { "Yeabsira", 0.72 },
  { "Yecenia", 0.004048583 },
  { "Yecheskel", 1.0 },
  { "Yechezkel", 1.0 },
  { "Yechiel", 1.0 },
  { "Yecica", 0.0 },
  { "Yedaiah", 1.0 },
  { "Yedda", 0.0 },
  { "Yedid", 0.0 },
  { "Yedida", 0.0 },
  { "Yedidya", 1.0 },
  { "Yediel", 1.0 },
  { "Yee", 0.58298755 },
  { "Yeeleng", 1.0 },
  { "Yeferson", 1.0 },
  { "Yefim", 1.0 },
  { "Yefri", 1.0 },
  { "Yegor", 1.0 },
  { "Yehezkel", 1.0 },
  { "Yehia", 1.0 },
  { "Yehonatan", 1.0 },
  { "Yehoshua", 1.0 },
  { "Yehuda", 1.0 },
  { "Yehudah", 1.0 },
  { "Yehudis", 0.0 },
  { "Yehudit", 0.0 },
  { "Yehya", 1.0 },
  { "Yeicelyn", 0.0 },
  { "Yeico", 1.0 },
  { "Yeicob", 1.0 },
  { "Yeidan", 1.0 },
  { "Yeiden", 1.0 },
  { "Yeidy", 0.0 },
  { "Yeila", 0.0 },
  { "Yeilani", 0.0 },
  { "Yeilen", 0.0 },
  { "Yeili", 0.0 },
  { "Yeilin", 0.0 },
  { "Yeily", 0.0 },
  { "Yeilyn", 0.0 },
  { "Yeimi", 0.0 },
  { "Yeimy", 0.0 },
  { "Yeira", 0.0 },
  { "Yeiren", 1.0 },
  { "Yeiri", 0.0 },
  { "Yeiry", 0.0 },
  { "Yeison", 1.0 },
  { "Yeisy", 0.0 },
  { "Yeji", 0.0 },
  { "Yejide", 0.0 },
  { "Yejin", 0.0 },
  { "Yejun", 1.0 },
  { "Yekaterina", 0.0 },
  { "Yekusiel", 1.0 },
  { "Yelaina", 0.0 },
  { "Yeleina", 0.0 },
  { "Yeleini", 0.0 },
  { "Yelena", 0.0 },
  { "Yeleni", 0.0 },
  { "Yelenis", 0.0 },
  { "Yelina", 0.0 },
  { "Yelisa", 0.0 },
  { "Yelisey", 1.0 },
  { "Yelissa", 0.0 },
  { "Yelitza", 0.0 },
  { "Yelixa", 0.0 },
  { "Yeliz", 0.0 },
  { "Yelizaveta", 0.0 },
  { "Yelonda", 0.0 },
  { "Yeltsin", 1.0 },
  { "Yema", 0.0 },
  { "Yemariam", 0.0 },
  { "Yemaya", 0.0 },
  { "Yemisi", 0.0 },
  { "Yen", 0.0704698 },
  { "Yena", 0.0 },
  { "Yenci", 0.0 },
  { "Yency", 0.0 },
  { "Yendi", 0.0 },
  { "Yendriel", 1.0 },
  { "Yendy", 0.0 },
  { "Yeneisy", 0.0 },
  { "Yener", 1.0 },
  { "Yenesis", 0.0 },
  { "Yeng", 0.8129771 },
  { "Yengkong", 1.0 },
  { "Yeni", 0.0 },
  { "Yenia", 0.0 },
  { "Yeniel", 1.0 },
  { "Yenifer", 0.0 },
  { "Yenis", 0.0 },
  { "Yenisel", 0.0 },
  { "Yenitza", 0.0 },
  { "Yenna", 0.0 },
  { "Yennhi", 0.0 },
  { "Yenni", 0.0 },
  { "Yennie", 0.0 },
  { "Yennifer", 0.0 },
  { "Yenny", 0.0 },
  { "Yensi", 0.0 },
  { "Yensid", 0.0 },
  { "Yensy", 0.0 },
  { "Yenta", 0.0 },
  { "Yentel", 0.0 },
  { "Yentl", 0.0 },
  { "Yenty", 0.0 },
  { "Yeny", 0.0 },
  { "Yer", 0.19887955 },
  { "Yerachmiel", 1.0 },
  { "Yerai", 1.0 },
  { "Yeraldi", 0.0 },
  { "Yeraldin", 0.0 },
  { "Yeraldine", 0.0 },
  { "Yeraldo", 1.0 },
  { "Yeraldy", 0.0 },
  { "Yerani", 0.0 },
  { "Yerania", 0.0 },
  { "Yeray", 0.9794239 },
  { "Yered", 1.0 },
  { "Yerelin", 0.0 },
  { "Yeremi", 1.0 },
  { "Yeremiah", 1.0 },
  { "Yeremy", 1.0 },
  { "Yerenia", 0.0 },
  { "Yerica", 0.0 },
  { "Yerick", 1.0 },
  { "Yeriel", 1.0 },
  { "Yerik", 1.0 },
  { "Yerika", 0.0 },
  { "Yerimar", 0.0 },
  { "Yerin", 0.0 },
  { "Yeritza", 0.0 },
  { "Yerli", 0.0 },
  { "Yerlin", 0.0 },
  { "Yerly", 0.0 },
  { "Yermi", 1.0 },
  { "Yero", 1.0 },
  { "Yerucham", 1.0 },
  { "Yeruchem", 1.0 },
  { "Yesania", 0.0 },
  { "Yescenia", 0.0 },
  { "Yesel", 0.0 },
  { "Yesemia", 0.0 },
  { "Yesena", 0.0 },
  { "Yesenia", 0.00813598 },
  { "Yesenio", 0.0 },
  { "Yesennia", 0.0 },
  { "Yesenya", 0.0 },
  { "Yesh", 1.0 },
  { "Yesha", 0.0 },
  { "Yeshaya", 1.0 },
  { "Yeshayahu", 1.0 },
  { "Yeshi", 0.0 },
  { "Yeshia", 0.13513513 },
  { "Yeshua", 1.0 },
  { "Yeshwin", 1.0 },
  { "Yesica", 0.0021758052 },
  { "Yesika", 0.0 },
  { "Yesina", 0.0 },
  { "Yesinia", 0.0 },
  { "Yesley", 0.0 },
  { "Yesli", 0.0 },
  { "Yeslie", 0.0 },
  { "Yeslin", 0.0 },
  { "Yesly", 0.0 },
  { "Yesmeen", 0.0 },
  { "Yesmi", 0.0 },
  { "Yesmin", 0.0 },
  { "Yesmine", 0.0 },
  { "Yesnia", 0.0 },
  { "Yessel", 0.0 },
  { "Yessenia", 0.0016783216 },
  { "Yessenya", 0.0 },
  { "Yessi", 0.0 },
  { "Yessia", 0.0 },
  { "Yessica", 0.0 },
  { "Yessika", 0.0 },
  { "Yestin", 1.0 },
  { "Yetive", 0.0 },
  { "Yetta", 0.0 },
  { "Yetunde", 0.0 },
  { "Yetzael", 1.0 },
  { "Yetzali", 0.0 },
  { "Yetzaly", 0.0 },
  { "Yetziel", 1.0 },
  { "Yeudiel", 1.0 },
  { "Yeva", 0.0 },
  { "Yevetta", 0.0 },
  { "Yevette", 0.0 },
  { "Yevgeniy", 1.0 },
  { "Yevonne", 0.0 },
  { "Yewande", 0.0 },
  { "Yewell", 1.0 },
  { "Yewon", 0.0 },
  { "Yexalen", 0.0 },
  { "Yexenia", 0.0 },
  { "Yexian", 1.0 },
  { "Yexiel", 1.0 },
  { "Yeyetzi", 0.0 },
  { "Yeymi", 0.0 },
  { "Yeyson", 1.0 },
  { "Yezan", 1.0 },
  { "Yezen", 1.0 },
  { "Yezenia", 0.0 },
  { "Ygnacia", 0.0 },
  { "Ygnacio", 1.0 },
  { "Yhadira", 0.0 },
  { "Yhael", 1.0 },
  { "Yhair", 1.0 },
  { "Yhoalibeth", 0.0 },
  { "Yi", 0.5870307 },
  { "Yia", 1.0 },
  { "Yian", 1.0 },
  { "Yiana", 0.0 },
  { "Yianna", 0.0 },
  { "Yianni", 1.0 },
  { "Yiannis", 1.0 },
  { "Yicel", 0.0 },
  { "Yichen", 0.64285713 },
  { "Yicheng", 1.0 },
  { "Yida", 1.0 },
  { "Yiddy", 1.0 },
  { "Yidel", 1.0 },
  { "Yides", 0.0 },
  { "Yien", 1.0 },
  { "Yiesha", 0.0 },
  { "Yifan", 0.8360656 },
  { "Yifei", 0.0 },
  { "Yigit", 1.0 },
  { "Yihan", 0.25531915 },
  { "Yilda", 0.0 },
  { "Yilia", 0.0 },
  { "Yilin", 0.083333336 },
  { "Yimi", 1.0 },
  { "Yiming", 1.0 },
  { "Yin", 0.0 },
  { "Yina", 0.0 },
  { "Yinessa", 0.45454547 },
  { "Ying", 0.35 },
  { "Yinon", 1.0 },
  { "Yinuo", 0.54545456 },
  { "Yiorgos", 1.0 },
  { "Yiovanni", 1.0 },
  { "Yiqing", 0.0 },
  { "Yira", 0.0 },
  { "Yiran", 0.29411766 },
  { "Yireh", 0.0 },
  { "Yirmeyah", 1.0 },
  { "Yiru", 0.0 },
  { "Yisel", 0.0 },
  { "Yisela", 0.0 },
  { "Yisell", 0.0 },
  { "Yiselle", 0.0 },
  { "Yisenia", 0.0 },
  { "Yishai", 1.0 },
  { "Yishay", 1.0 },
  { "Yisleine", 0.0 },
  { "Yisrael", 1.0 },
  { "Yisroel", 1.0 },
  { "Yissachar", 1.0 },
  { "Yissel", 0.0 },
  { "Yissell", 0.0 },
  { "Yisselle", 0.0 },
  { "Yissochor", 1.0 },
  { "Yita", 0.0 },
  { "Yitel", 0.0 },
  { "Yitong", 0.0 },
  { "Yitsel", 0.0 },
  { "Yitta", 0.0 },
  { "Yittel", 0.0 },
  { "Yitty", 0.0 },
  { "Yitzchak", 1.0 },
  { "Yitzchock", 1.0 },
  { "Yitzchok", 1.0 },
  { "Yitzel", 0.0 },
  { "Yitzhak", 1.0 },
  { "Yixin", 0.0 },
  { "Yixuan", 0.23809524 },
  { "Yiyang", 1.0 },
  { "Yiyi", 0.0 },
  { "Yizel", 0.0 },
  { "Yizelle", 0.0 },
  { "Yizza", 0.0 },
  { "Ykeisha", 0.0 },
  { "Yker", 1.0 },
  { "Ylan", 0.0 },
  { "Ylana", 0.0 },
  { "Ylanda", 0.0 },
  { "Ylani", 0.0 },
  { "Ylario", 1.0 },
  { "Yle", 1.0 },
  { "Yleana", 0.0 },
  { "Ylenia", 0.0 },
  { "Yler", 1.0 },
  { "Yliana", 0.0 },
  { "Ylianna", 0.0 },
  { "Yll", 1.0 },
  { "Ylonda", 0.0 },
  { "Yma", 0.0 },
  { "Ymani", 0.0 },
  { "Ymari", 0.0 },
  { "Ymelda", 0.0 },
  { "Yna", 0.0 },
  { "Ynes", 0.5897436 },
  { "Ynez", 0.03731343 },
  { "Yngwie", 1.0 },
  { "Ynhi", 0.0 },
  { "Yo", 0.0 },
  { "Yoali", 0.0 },
  { "Yoaly", 0.0 },
  { "Yoan", 1.0 },
  { "Yoana", 0.0 },
  { "Yoandi", 1.0 },
  { "Yoandra", 0.0 },
  { "Yoandri", 1.0 },
  { "Yoandry", 1.0 },
  { "Yoandy", 1.0 },
  { "Yoani", 1.0 },
  { "Yoann", 1.0 },
  { "Yoanna", 0.0 },
  { "Yoany", 0.0 },
  { "Yoav", 1.0 },
  { "Yobana", 0.0 },
  { "Yobani", 1.0 },
  { "Yobany", 1.0 },
  { "Yocasta", 0.0 },
  { "Yocelin", 0.0 },
  { "Yoceline", 0.0 },
  { "Yocelyn", 0.0 },
  { "Yocelyne", 0.0 },
  { "Yocelynn", 0.0 },
  { "Yochanan", 1.0 },
  { "Yochanon", 1.0 },
  { "Yocheved", 0.0 },
  { "Yodahe", 0.0 },
  { "Yodit", 0.0 },
  { "Yoel", 1.0 },
  { "Yoeli", 0.0 },
  { "Yoenis", 1.0 },
  { "Yoexis", 1.0 },
  { "Yogesh", 1.0 },
  { "Yogi", 1.0 },
  { "Yohaan", 1.0 },
  { "Yohali", 0.0 },
  { "Yohan", 1.0 },
  { "Yohana", 0.0 },
  { "Yohanan", 0.8979592 },
  { "Yohance", 1.0 },
  { "Yohandri", 1.0 },
  { "Yohandry", 1.0 },
  { "Yohanes", 1.0 },
  { "Yohann", 1.0 },
  { "Yohanna", 0.0 },
  { "Yohannes", 1.0 },
  { "Yohei", 1.0 },
  { "Yohel", 1.0 },
  { "Yohsuke", 1.0 },
  { "Yoichi", 1.0 },
  { "Yojaira", 0.0 },
  { "Yojan", 1.0 },
  { "Yojana", 0.0 },
  { "Yojhan", 1.0 },
  { "Yokasta", 0.0 },
  { "Yoko", 0.0 },
  { "Yola", 0.0 },
  { "Yolaine", 0.0 },
  { "Yolan", 0.0 },
  { "Yolana", 0.0 },
  { "Yoland", 0.0 },
  { "Yolanda", 0.004761145 },
  { "Yolande", 0.0 },
  { "Yolander", 0.0 },
  { "Yolandi", 0.0 },
  { "Yolandia", 0.0 },
  { "Yolandita", 0.0 },
  { "Yolando", 0.010570824 },
  { "Yolandra", 0.0 },
  { "Yolani", 0.0 },
  { "Yolanta", 0.0 },
  { "Yolany", 0.0 },
  { "Yolaunda", 0.0 },
  { "Yolenda", 0.0 },
  { "Yolet", 0.0 },
  { "Yoleth", 0.0 },
  { "Yolette", 0.0 },
  { "Yoli", 0.0 },
  { "Yolimar", 0.0 },
  { "Yolinda", 0.0 },
  { "Yolisma", 0.0 },
  { "Yollanda", 0.0 },
  { "Yolonda", 0.0 },
  { "Yolonde", 0.0 },
  { "Yolotzin", 0.0 },
  { "Yoltzin", 0.47540984 },
  { "Yolunda", 0.0 },
  { "Yom", 1.0 },
  { "Yomaira", 0.0 },
  { "Yomaly", 0.0 },
  { "Yomar", 1.0 },
  { "Yomara", 0.0 },
  { "Yomari", 0.0 },
  { "Yomaris", 0.0 },
  { "Yomayra", 0.0 },
  { "Yomira", 0.0 },
  { "Yomna", 0.0 },
  { "Yomo", 1.0 },
  { "Yomtov", 1.0 },
  { "Yon", 1.0 },
  { "Yona", 0.56487024 },
  { "Yonael", 1.0 },
  { "Yonah", 1.0 },
  { "Yonas", 1.0 },
  { "Yonason", 1.0 },
  { "Yonatan", 1.0 },
  { "Yonathan", 1.0 },
  { "Yonaton", 1.0 },
  { "Yonda", 0.0 },
  { "Yone", 0.0 },
  { "Yoneko", 0.0 },
  { "Yonel", 1.0 },
  { "Yoneo", 1.0 },
  { "Yong", 0.71095574 },
  { "Yoni", 0.9706745 },
  { "Yonic", 1.0 },
  { "Yoniel", 1.0 },
  { "Yonika", 0.0 },
  { "Yonina", 0.0 },
  { "Yonis", 1.0 },
  { "Yonna", 0.0 },
  { "Yonni", 1.0 },
  { "Yonnie", 0.0 },
  { "Yonny", 1.0 },
  { "Yony", 1.0 },
  { "Yoo", 0.0 },
  { "Yoon", 0.73394495 },
  { "Yoona", 0.0 },
  { "Yoonseo", 0.0 },
  { "Yoram", 1.0 },
  { "Yordan", 1.0 },
  { "Yordanny", 1.0 },
  { "Yordano", 1.0 },
  { "Yordanos", 0.0 },
  { "Yordany", 1.0 },
  { "Yordi", 1.0 },
  { "Yordin", 1.0 },
  { "Yordy", 1.0 },
  { "Yordyn", 1.0 },
  { "Yorel", 1.0 },
  { "Yorgelis", 0.0 },
  { "Yori", 0.13333334 },
  { "Yorick", 1.0 },
  { "Yoriel", 1.0 },
  { "York", 1.0 },
  { "Yorlei", 0.0 },
  { "Yorleni", 0.0 },
  { "Yorleny", 0.0 },
  { "Yorlet", 0.0 },
  { "Yorley", 0.0 },
  { "Yorman", 1.0 },
  { "Yoruba", 0.0 },
  { "Yorvin", 1.0 },
  { "Yosaira", 0.0 },
  { "Yosalin", 0.0 },
  { "Yosan", 0.0 },
  { "Yosbel", 1.0 },
  { "Yoscar", 1.0 },
  { "Yosef", 1.0 },
  { "Yosel", 1.0 },
  { "Yoseli", 0.0 },
  { "Yoselin", 0.0 },
  { "Yoselina", 0.0 },
  { "Yoseline", 0.0 },
  { "Yoselyn", 0.0 },
  { "Yoselyne", 0.0 },
  { "Yoselynn", 0.0 },
  { "Yosemite", 0.0 },
  { "Yoseph", 1.0 },
  { "Yosgar", 1.0 },
  { "Yosgard", 1.0 },
  { "Yosgart", 1.0 },
  { "Yosgarth", 1.0 },
  { "Yosha", 0.0 },
  { "Yoshani", 0.0 },
  { "Yosheka", 0.0 },
  { "Yoshi", 0.6268657 },
  { "Yoshiaki", 1.0 },
  { "Yoshica", 0.0 },
  { "Yoshida", 0.0 },
  { "Yoshie", 0.0 },
  { "Yoshigei", 0.0 },
  { "Yoshigey", 0.0 },
  { "Yoshiharu", 1.0 },
  { "Yoshihiko", 1.0 },
  { "Yoshihiro", 1.0 },
  { "Yoshika", 0.0 },
  { "Yoshikatsu", 1.0 },
  { "Yoshikazu", 1.0 },
  { "Yoshiki", 1.0 },
  { "Yoshiko", 0.0 },
  { "Yoshimi", 0.68421054 },
  { "Yoshino", 0.0 },
  { "Yoshinobu", 1.0 },
  { "Yoshinori", 1.0 },
  { "Yoshio", 1.0 },
  { "Yoshira", 0.0 },
  { "Yoshiro", 1.0 },
  { "Yoshitaka", 1.0 },
  { "Yoshito", 1.0 },
  { "Yoshiyah", 1.0 },
  { "Yoshiye", 0.0 },
  { "Yoshiyuki", 1.0 },
  { "Yoshua", 1.0 },
  { "Yosiah", 1.0 },
  { "Yosief", 1.0 },
  { "Yosiel", 1.0 },
  { "Yosif", 1.0 },
  { "Yosilin", 0.0 },
  { "Yosimar", 1.0 },
  { "Yoskar", 1.0 },
  { "Yoslan", 1.0 },
  { "Yosmairy", 0.0 },
  { "Yosman", 1.0 },
  { "Yosmar", 1.0 },
  { "Yosmely", 0.0 },
  { "Yosniel", 1.0 },
  { "Yossef", 1.0 },
  { "Yosselin", 0.0 },
  { "Yosseline", 0.0 },
  { "Yosselyn", 0.0 },
  { "Yossi", 1.0 },
  { "Yost", 1.0 },
  { "Yostin", 1.0 },
  { "Yosue", 1.0 },
  { "Yosuel", 1.0 },
  { "Yosuf", 1.0 },
  { "Yosuke", 1.0 },
  { "Yosvani", 1.0 },
  { "Yosvany", 1.0 },
  { "Yotam", 1.0 },
  { "You", 1.0 },
  { "Youa", 0.0 },
  { "Youcef", 1.0 },
  { "Youel", 1.0 },
  { "Youki", 1.0 },
  { "Youlanda", 0.0 },
  { "Youlander", 0.0 },
  { "Youlonda", 0.0 },
  { "Younes", 1.0 },
  { "Youness", 1.0 },
  { "Young", 0.80527085 },
  { "Younger", 1.0 },
  { "Younique", 0.0 },
  { "Younis", 1.0 },
  { "Younus", 1.0 },
  { "Youri", 1.0 },
  { "Yousaf", 1.0 },
  { "Yousef", 1.0 },
  { "Youseph", 1.0 },
  { "Yousif", 1.0 },
  { "Yousof", 1.0 },
  { "Yousra", 0.0 },
  { "Youssef", 1.0 },
  { "Youssif", 1.0 },
  { "Youssouf", 1.0 },
  { "Youstina", 0.0 },
  { "Yousuf", 1.0 },
  { "Yousuke", 1.0 },
  { "Yovan", 1.0 },
  { "Yovana", 0.0 },
  { "Yovani", 0.99108636 },
  { "Yovanka", 0.0 },
  { "Yovanna", 0.0 },
  { "Yovanni", 1.0 },
  { "Yovanny", 1.0 },
  { "Yovany", 1.0 },
  { "Yovela", 0.0 },
  { "Yovonda", 0.0 },
  { "Yovonne", 0.0 },
  { "Yowanda", 0.0 },
  { "Yoyo", 0.0 },
  { "Yozelin", 0.0 },
  { "Yrania", 0.0 },
  { "Yridiana", 0.0 },
  { "Ysa", 0.0 },
  { "Ysabel", 0.14423077 },
  { "Ysabela", 0.0 },
  { "Ysabell", 0.0 },
  { "Ysabella", 0.0 },
  { "Ysabelle", 0.0 },
  { "Ysamar", 0.0 },
  { "Ysatis", 0.0 },
  { "Ysaura", 0.0 },
  { "Ysela", 0.0 },
  { "Ysenia", 0.0 },
  { "Ysidora", 0.0 },
  { "Ysidoro", 1.0 },
  { "Ysidra", 0.0 },
  { "Ysidro", 1.0 },
  { "Ysmael", 1.0 },
  { "Ysobel", 0.0 },
  { "Ysrael", 1.0 },
  { "Ytzel", 0.0 },
  { "Yu", 0.50272477 },
  { "Yuan", 0.82051283 },
  { "Yuba", 0.0 },
  { "Yubal", 1.0 },
  { "Yubia", 0.0 },
  { "Yubin", 0.0 },
  { "Yuchen", 0.7818182 },
  { "Yucheng", 1.0 },
  { "Yuda", 1.0 },
  { "Yudani", 0.0 },
  { "Yudany", 0.0 },
  { "Yudelca", 0.0 },
  { "Yudelka", 0.0 },
  { "Yudi", 0.0 },
  { "Yudiel", 1.0 },
  { "Yudit", 0.0 },
  { "Yudith", 0.0 },
  { "Yue", 0.6328125 },
  { "Yuen", 0.0 },
  { "Yuepeng", 1.0 },
  { "Yuepheng", 1.0 },
  { "Yuette", 0.0 },
  { "Yufei", 0.0 },
  { "Yug", 1.0 },
  { "Yuga", 1.0 },
  { "Yugan", 1.0 },
  { "Yugo", 1.0 },
  { "Yuhan", 0.0 },
  { "Yuhanna", 1.0 },
  { "Yui", 0.0 },
  { "Yuika", 0.0 },
  { "Yuina", 0.0 },
  { "Yuisa", 0.0 },
  { "Yuito", 1.0 },
  { "Yuji", 1.0 },
  { "Yujin", 0.0 },
  { "Yuka", 0.0 },
  { "Yukari", 0.0 },
  { "Yukary", 0.0 },
  { "Yuki", 0.35895464 },
  { "Yukia", 0.0 },
  { "Yukie", 0.0 },
  { "Yukiko", 0.0 },
  { "Yukina", 0.0 },
  { "Yukino", 0.0 },
  { "Yukio", 1.0 },
  { "Yukiye", 0.0 },
  { "Yuko", 0.0 },
  { "Yukon", 1.0 },
  { "Yukta", 0.0 },
  { "Yuktha", 0.0 },
  { "Yukti", 0.0 },
  { "Yul", 1.0 },
  { "Yula", 0.0 },
  { "Yulanda", 0.0 },
  { "Yulander", 0.0 },
  { "Yulani", 0.0 },
  { "Yule", 1.0 },
  { "Yuleidi", 0.0 },
  { "Yuleidy", 0.0 },
  { "Yuleiky", 0.0 },
  { "Yuleimi", 0.0 },
  { "Yuleimy", 0.0 },
  { "Yuleisi", 0.0 },
  { "Yuleisy", 0.0 },
  { "Yulema", 0.0 },
  { "Yulemi", 0.0 },
  { "Yulemni", 0.0 },
  { "Yulene", 0.0 },
  { "Yuleni", 0.0 },
  { "Yulexi", 0.0 },
  { "Yuleydi", 0.0 },
  { "Yuleysi", 0.0 },
  { "Yuli", 0.0 },
  { "Yulia", 0.0 },
  { "Yulian", 1.0 },
  { "Yuliana", 0.0 },
  { "Yuliani", 0.0 },
  { "Yulianna", 0.0 },
  { "Yulie", 0.0 },
  { "Yuliet", 0.0 },
  { "Yulieth", 0.0 },
  { "Yuliett", 0.0 },
  { "Yuliette", 0.0 },
  { "Yulinda", 0.0 },
  { "Yulisa", 0.0 },
  { "Yulissa", 0.0 },
  { "Yulitza", 0.0 },
  { "Yulitzy", 0.0 },
  { "Yuliya", 0.0 },
  { "Yuliza", 0.0 },
  { "Yulizza", 0.0 },
  { "Yulliana", 0.0 },
  { "Yulma", 0.0 },
  { "Yulonda", 0.0 },
  { "Yulunda", 0.0 },
  { "Yuly", 0.0 },
  { "Yuma", 0.79027355 },
  { "Yumaira", 0.0 },
  { "Yumalai", 0.0 },
  { "Yumalay", 0.0 },
  { "Yumara", 0.0 },
  { "Yume", 0.0 },
  { "Yumeka", 0.0 },
  { "Yumi", 0.0 },
  { "Yumika", 0.0 },
  { "Yumiko", 0.0 },
  { "Yumin", 1.0 },
  { "Yumira", 0.0 },
  { "Yumna", 0.0 },
  { "Yun", 0.25 },
  { "Yuna", 0.0 },
  { "Yunalesca", 0.0 },
  { "Yunay", 1.0 },
  { "Yuneicy", 0.0 },
  { "Yuneisy", 0.0 },
  { "Yunes", 1.0 },
  { "Yung", 1.0 },
  { "Yuni", 0.0 },
  { "Yuniel", 1.0 },
  { "Yunier", 1.0 },
  { "Yunior", 1.0 },
  { "Yunique", 0.0 },
  { "Yunis", 1.0 },
  { "Yunnuen", 0.0 },
  { "Yunqi", 1.0 },
  { "Yunuen", 0.017857144 },
  { "Yunus", 1.0 },
  { "Yunxi", 0.0 },
  { "Yupheng", 1.0 },
  { "Yura", 0.0 },
  { "Yuraima", 0.0 },
  { "Yurani", 0.0 },
  { "Yuranni", 0.0 },
  { "Yurany", 0.0 },
  { "Yureimy", 0.0 },
  { "Yureli", 0.0 },
  { "Yurely", 0.0 },
  { "Yurem", 1.0 },
  { "Yuren", 1.0 },
  { "Yuri", 0.3936294 },
  { "Yuria", 0.0 },
  { "Yuriah", 0.46376812 },
  { "Yuriana", 0.0 },
  { "Yurianna", 0.0 },
  { "Yurico", 0.0 },
  { "Yuridia", 0.0 },
  { "Yuridiana", 0.0 },
  { "Yurie", 0.0 },
  { "Yuriel", 1.0 },
  { "Yurik", 1.0 },
  { "Yurika", 0.0 },
  { "Yuriko", 0.0 },
  { "Yuriria", 0.0 },
  { "Yurisa", 0.0 },
  { "Yurith", 0.0 },
  { "Yuritza", 0.0 },
  { "Yuritzi", 0.0 },
  { "Yuritzia", 0.0 },
  { "Yuritzy", 0.0 },
  { "Yurivia", 0.0 },
  { "Yuriy", 1.0 },
  { "Yury", 0.082125604 },
  { "Yusayrah", 0.0 },
  { "Yusef", 1.0 },
  { "Yuseff", 1.0 },
  { "Yusei", 1.0 },
  { "Yusha", 1.0 },
  { "Yushica", 0.0 },
  { "Yushin", 1.0 },
  { "Yusif", 1.0 },
  { "Yusmeri", 0.0 },
  { "Yusof", 1.0 },
  { "Yusra", 0.0 },
  { "Yusrah", 0.0 },
  { "Yussef", 1.0 },
  { "Yussuf", 1.0 },
  { "Yustin", 1.0 },
  { "Yusuf", 1.0 },
  { "Yusufbek", 1.0 },
  { "Yusuke", 1.0 },
  { "Yusupha", 1.0 },
  { "Yuta", 1.0 },
  { "Yutaka", 1.0 },
  { "Yutaro", 1.0 },
  { "Yuting", 0.0 },
  { "Yuto", 1.0 },
  { "Yutong", 0.0 },
  { "Yutzil", 0.0 },
  { "Yuuka", 0.0 },
  { "Yuuki", 0.9183673 },
  { "Yuuna", 0.0 },
  { "Yuuta", 1.0 },
  { "Yuv", 1.0 },
  { "Yuva", 1.0 },
  { "Yuvaan", 1.0 },
  { "Yuval", 0.666 },
  { "Yuvan", 1.0 },
  { "Yuvansh", 1.0 },
  { "Yuvawn", 0.0 },
  { "Yuven", 1.0 },
  { "Yuvette", 0.0 },
  { "Yuvi", 1.0 },
  { "Yuvia", 0.0 },
  { "Yuvika", 0.0 },
  { "Yuvin", 1.0 },
  { "Yuvonda", 0.0 },
  { "Yuvonka", 0.0 },
  { "Yuvonne", 0.0 },
  { "Yuvraaj", 1.0 },
  { "Yuvraj", 1.0 },
  { "Yuxi", 0.0 },
  { "Yuxin", 0.0 },
  { "Yuxuan", 0.6551724 },
  { "Yuya", 1.0 },
  { "Yuyang", 1.0 },
  { "Yuze", 1.0 },
  { "Yuzu", 0.0 },
  { "Yuzuki", 0.0 },
  { "Yva", 0.0 },
  { "Yvain", 1.0 },
  { "Yvaine", 0.0 },
  { "Yvalondra", 0.0 },
  { "Yvan", 0.98740554 },
  { "Yvana", 0.0 },
  { "Yvanna", 0.0 },
  { "Yvannah", 0.0 },
  { "Yvannia", 0.0 },
  { "Yveline", 0.0 },
  { "Yvelisse", 0.0 },
  { "Yvelle", 0.0 },
  { "Yvens", 1.0 },
  { "Yves", 0.9686636 },
  { "Yveth", 0.0 },
  { "Yvett", 0.0 },
  { "Yvetta", 0.0 },
  { "Yvette", 0.0029458813 },
  { "Yvianna", 0.0 },
  { "Yvie", 0.0 },
  { "Yvon", 0.8297101 },
  { "Yvonda", 0.0 },
  { "Yvone", 0.0 },
  { "Yvonia", 0.0 },
  { "Yvonna", 0.0 },
  { "Yvonnda", 0.0 },
  { "Yvonne", 0.0039720736 },
  { "Yvonnia", 0.0 },
  { "Yvonnie", 0.0 },
  { "Ywa", 1.0 },
  { "Yzabel", 0.0 },
  { "Yzabella", 0.0 },
  { "Yzabelle", 0.0 },
  { "Yzamar", 0.0 },
  { "Za", 1.0 },
  { "Zaafir", 1.0 },
  { "Zaahir", 1.0 },
  { "Zaaire", 1.0 },
  { "Zaakir", 1.0 },
  { "Zaakira", 0.0 },
  { "Zaakirah", 0.0 },
  { "Zaakiyah", 0.0 },
  { "Zaaliyah", 0.0 },
  { "Zaara", 0.0 },
  { "Zaaron", 1.0 },
  { "Zaavan", 1.0 },
  { "Zabdi", 0.2037037 },
  { "Zabdiel", 1.0 },
  { "Zabdy", 0.0 },
  { "Zabel", 0.0 },
  { "Zabella", 0.0 },
  { "Zabelle", 0.0 },
  { "Zabian", 1.0 },
  { "Zabien", 1.0 },
  { "Zabria", 0.0 },
  { "Zabrian", 1.0 },
  { "Zabriel", 1.0 },
  { "Zabrien", 1.0 },
  { "Zabrina", 0.0 },
  { "Zabryna", 0.0 },
  { "Zac", 1.0 },
  { "Zacai", 1.0 },
  { "Zacara", 0.0 },
  { "Zacardi", 1.0 },
  { "Zacari", 0.6873065 },
  { "Zacaria", 0.19014084 },
  { "Zacariah", 0.986911 },
  { "Zacarias", 1.0 },
  { "Zacarion", 1.0 },
  { "Zacarius", 1.0 },
  { "Zacarri", 1.0 },
  { "Zacary", 1.0 },
  { "Zaccai", 1.0 },
  { "Zaccari", 1.0 },
  { "Zaccaria", 1.0 },
  { "Zaccariah", 1.0 },
  { "Zaccary", 1.0 },
  { "Zacchaeus", 1.0 },
  { "Zacchary", 1.0 },
  { "Zaccheaus", 1.0 },
  { "Zaccheus", 1.0 },
  { "Zace", 1.0 },
  { "Zacery", 1.0 },
  { "Zach", 1.0 },
  { "Zachaery", 1.0 },
  { "Zachaeus", 1.0 },
  { "Zachai", 1.0 },
  { "Zacharay", 1.0 },
  { "Zacharee", 0.95098037 },
  { "Zacharey", 1.0 },
  { "Zachari", 1.0 },
  { "Zacharia", 0.986403 },
  { "Zachariah", 0.9985801 },
  { "Zacharian", 1.0 },
  { "Zacharias", 1.0 },
  { "Zacharie", 1.0 },
  { "Zacharius", 1.0 },
  { "Zachariyah", 1.0 },
  { "Zacharry", 1.0 },
  { "Zachary", 0.9971419 },
  { "Zacharya", 1.0 },
  { "Zacharyah", 1.0 },
  { "Zacherey", 1.0 },
  { "Zacheri", 1.0 },
  { "Zacheria", 1.0 },
  { "Zacheriah", 1.0 },
  { "Zacherie", 1.0 },
  { "Zachery", 0.99870455 },
  { "Zacheus", 1.0 },
  { "Zachi", 1.0 },
  { "Zachiary", 1.0 },
  { "Zachory", 1.0 },
  { "Zachrey", 1.0 },
  { "Zachriah", 1.0 },
  { "Zachry", 1.0 },
  { "Zack", 1.0 },
  { "Zackaree", 1.0 },
  { "Zackarey", 1.0 },
  { "Zackari", 1.0 },
  { "Zackaria", 1.0 },
  { "Zackariah", 1.0 },
  { "Zackarias", 1.0 },
  { "Zackarie", 1.0 },
  { "Zackariya", 1.0 },
  { "Zackariyah", 1.0 },
  { "Zackary", 0.99976665 },
  { "Zackeri", 1.0 },
  { "Zackeria", 1.0 },
  { "Zackeriah", 1.0 },
  { "Zackerie", 1.0 },
  { "Zackery", 0.9997102 },
  { "Zackhary", 1.0 },
  { "Zackie", 1.0 },
  { "Zackorie", 1.0 },
  { "Zackory", 1.0 },
  { "Zackrey", 1.0 },
  { "Zackry", 1.0 },
  { "Zacora", 0.0 },
  { "Zacorey", 1.0 },
  { "Zacori", 1.0 },
  { "Zacoria", 0.0 },
  { "Zacorian", 1.0 },
  { "Zacory", 1.0 },
  { "Zada", 0.0 },
  { "Zadah", 0.0 },
  { "Zadan", 1.0 },
  { "Zadarius", 1.0 },
  { "Zadaya", 0.0 },
  { "Zaddie", 0.0 },
  { "Zade", 0.9836695 },
  { "Zadee", 0.0 },
  { "Zaden", 1.0 },
  { "Zadey", 0.0 },
  { "Zadi", 0.0 },
  { "Zadia", 0.0 },
  { "Zadian", 1.0 },
  { "Zadie", 0.0 },
  { "Zadiel", 1.0 },
  { "Zadien", 1.0 },
  { "Zadin", 1.0 },
  { "Zadkiel", 1.0 },
  { "Zadok", 1.0 },
  { "Zadon", 1.0 },
  { "Zadquiel", 1.0 },
  { "Zadrian", 1.0 },
  { "Zadrien", 1.0 },
  { "Zady", 0.0 },
  { "Zadyn", 1.0 },
  { "Zae", 1.0 },
  { "Zaeda", 0.0 },
  { "Zaedan", 1.0 },
  { "Zaeden", 1.0 },
  { "Zaedin", 1.0 },
  { "Zaedon", 1.0 },
  { "Zaedyn", 0.9868421 },
  { "Zaeem", 1.0 },
  { "Zael", 1.0 },
  { "Zaela", 0.0 },
  { "Zaelah", 0.0 },
  { "Zaelee", 0.0 },
  { "Zaeleigh", 0.0 },
  { "Zaelen", 1.0 },
  { "Zaelia", 0.0 },
  { "Zaelie", 0.0 },
  { "Zaelin", 1.0 },
  { "Zaelon", 1.0 },
  { "Zaely", 0.0 },
  { "Zaelyn", 0.5416667 },
  { "Zaelynn", 0.0 },
  { "Zaen", 1.0 },
  { "Zaena", 0.0 },
  { "Zaequan", 1.0 },
  { "Zaeveon", 1.0 },
  { "Zaevian", 1.0 },
  { "Zaevion", 1.0 },
  { "Zaevon", 1.0 },
  { "Zaeya", 0.0 },
  { "Zafar", 1.0 },
  { "Zafer", 1.0 },
  { "Zafina", 0.0 },
  { "Zafir", 1.0 },
  { "Zafira", 0.0 },
  { "Zafirah", 0.0 },
  { "Zafiro", 0.0 },
  { "Zafreen", 0.0 },
  { "Zaha", 0.0 },
  { "Zahair", 1.0 },
  { "Zahaira", 0.0 },
  { "Zahan", 1.0 },
  { "Zahar", 1.0 },
  { "Zahara", 0.0 },
  { "Zaharah", 0.0 },
  { "Zahari", 0.41040462 },
  { "Zaharia", 0.0 },
  { "Zaharra", 0.0 },
  { "Zahava", 0.0 },
  { "Zahcary", 1.0 },
  { "Zaheem", 1.0 },
  { "Zaheen", 1.0 },
  { "Zaheer", 1.0 },
  { "Zaheim", 1.0 },
  { "Zaher", 1.0 },
  { "Zahi", 1.0 },
  { "Zahia", 0.0 },
  { "Zahid", 1.0 },
  { "Zahida", 0.0 },
  { "Zahidah", 0.0 },
  { "Zahiem", 1.0 },
  { "Zahier", 1.0 },
  { "Zahin", 1.0 },
  { "Zahir", 0.99689853 },
  { "Zahira", 0.0 },
  { "Zahirah", 0.0 },
  { "Zahire", 1.0 },
  { "Zahiya", 0.0 },
  { "Zahiyah", 0.0 },
  { "Zahkai", 1.0 },
  { "Zahkari", 1.0 },
  { "Zahkee", 1.0 },
  { "Zahkeem", 1.0 },
  { "Zahki", 1.0 },
  { "Zahkir", 1.0 },
  { "Zahlia", 0.0 },
  { "Zahmari", 0.85714287 },
  { "Zahmaria", 0.0 },
  { "Zahmere", 1.0 },
  { "Zahmier", 1.0 },
  { "Zahmir", 1.0 },
  { "Zahmira", 0.0 },
  { "Zahmiya", 0.0 },
  { "Zahmya", 0.0 },
  { "Zahn", 1.0 },
  { "Zahna", 0.0 },
  { "Zahnae", 0.0 },
  { "Zahnia", 0.0 },
  { "Zahniah", 0.0 },
  { "Zahniya", 0.0 },
  { "Zahniyah", 0.0 },
  { "Zahnya", 0.0 },
  { "Zahra", 0.0 },
  { "Zahraa", 0.0 },
  { "Zahrah", 0.0 },
  { "Zahran", 1.0 },
  { "Zahrea", 0.0 },
  { "Zahri", 0.0 },
  { "Zahria", 0.0 },
  { "Zahriah", 0.0 },
  { "Zahriya", 0.0 },
  { "Zahriyah", 0.0 },
  { "Zahro", 0.0 },
  { "Zahrya", 0.0 },
  { "Zahyan", 1.0 },
  { "Zahyir", 1.0 },
  { "Zahyr", 1.0 },
  { "Zahyra", 0.0 },
  { "Zahyrah", 0.0 },
  { "Zai", 0.74747473 },
  { "Zaia", 0.0 },
  { "Zaiah", 0.42222223 },
  { "Zaiana", 0.0 },
  { "Zaid", 1.0 },
  { "Zaida", 0.0 },
  { "Zaidah", 0.0 },
  { "Zaidan", 1.0 },
  { "Zaide", 0.5670996 },
  { "Zaidee", 0.0 },
  { "Zaiden", 0.994476 },
  { "Zaidin", 1.0 },
  { "Zaidon", 1.0 },
  { "Zaidy", 0.0 },
  { "Zaidyn", 0.9604592 },
  { "Zaier", 1.0 },
  { "Zaiiden", 1.0 },
  { "Zaila", 0.0 },
  { "Zailah", 0.0 },
  { "Zailan", 1.0 },
  { "Zailee", 0.0 },
  { "Zaileigh", 0.0 },
  { "Zailen", 1.0 },
  { "Zailey", 0.0 },
  { "Zailie", 0.0 },
  { "Zailin", 1.0 },
  { "Zaily", 0.0 },
  { "Zailyn", 0.32824427 },
  { "Zailynn", 0.0 },
  { "Zaim", 1.0 },
  { "Zaima", 0.0 },
  { "Zaimah", 0.0 },
  { "Zaimar", 1.0 },
  { "Zain", 0.9628384 },
  { "Zaina", 0.0 },
  { "Zainab", 0.0 },
  { "Zainah", 0.0 },
  { "Zainaldeen", 1.0 },
  { "Zainb", 0.0 },
  { "Zaine", 0.98826116 },
  { "Zaineb", 0.0 },
  { "Zainub", 0.0 },
  { "Zaion", 1.0 },
  { "Zaionna", 0.0 },
  { "Zair", 1.0 },
  { "Zaira", 0.0 },
  { "Zairah", 0.0 },
  { "Zaire", 0.78327125 },
  { "Zairia", 0.0 },
  { "Zairy", 0.0 },
  { "Zaisha", 0.0 },
  { "Zaith", 1.0 },
  { "Zaiven", 1.0 },
  { "Zaiveon", 1.0 },
  { "Zaivian", 1.0 },
  { "Zaivier", 1.0 },
  { "Zaivion", 1.0 },
  { "Zaiya", 0.0 },
  { "Zaiyah", 0.0 },
  { "Zaiyan", 1.0 },
  { "Zaiyanna", 0.0 },
  { "Zaiyden", 1.0 },
  { "Zaiyon", 1.0 },
  { "Zak", 1.0 },
  { "Zakai", 0.9951362 },
  { "Zakaiden", 1.0 },
  { "Zakaira", 0.0 },
  { "Zakaiya", 0.0 },
  { "Zakaiyah", 0.0 },
  { "Zakar", 1.0 },
  { "Zakara", 0.0 },
  { "Zakarah", 0.0 },
  { "Zakaree", 1.0 },
  { "Zakarey", 1.0 },
  { "Zakari", 0.86730766 },
  { "Zakaria", 0.83340776 },
  { "Zakariah", 0.8934081 },
  { "Zakarian", 1.0 },
  { "Zakarias", 1.0 },
  { "Zakarie", 1.0 },
  { "Zakariya", 0.9645283 },
  { "Zakariyah", 0.79347825 },
  { "Zakariye", 1.0 },
  { "Zakariyya", 1.0 },
  { "Zakarri", 1.0 },
  { "Zakary", 1.0 },
  { "Zakarya", 1.0 },
  { "Zakaryah", 1.0 },
  { "Zakaya", 0.0 },
  { "Zakayah", 0.0 },
  { "Zakayden", 1.0 },
  { "Zakayla", 0.0 },
  { "Zakaylah", 0.0 },
  { "Zake", 1.0 },
  { "Zakee", 1.0 },
  { "Zakeea", 0.0 },
  { "Zakeem", 1.0 },
  { "Zakeia", 0.0 },
  { "Zakera", 0.0 },
  { "Zakeri", 1.0 },
  { "Zakeria", 0.02293578 },
  { "Zakerria", 0.0 },
  { "Zakery", 1.0 },
  { "Zakeya", 0.0 },
  { "Zakeyah", 0.0 },
  { "Zakeyia", 0.0 },
  { "Zakhai", 1.0 },
  { "Zakhar", 1.0 },
  { "Zakhari", 0.9431818 },
  { "Zakharia", 0.0 },
  { "Zakhary", 1.0 },
  { "Zakhi", 1.0 },
  { "Zakhia", 0.0 },
  { "Zakhiya", 0.0 },
  { "Zakhya", 0.0 },
  { "Zaki", 1.0 },
  { "Zakia", 0.009894867 },
  { "Zakiah", 0.26359832 },
  { "Zakiel", 1.0 },
  { "Zakir", 1.0 },
  { "Zakira", 0.0 },
  { "Zakirah", 0.0 },
  { "Zakiria", 0.0 },
  { "Zakiya", 0.0 },
  { "Zakiyah", 0.0 },
  { "Zakiyya", 0.0 },
  { "Zakiyyah", 0.0 },
  { "Zakk", 1.0 },
  { "Zakkai", 1.0 },
  { "Zakkariah", 1.0 },
  { "Zakkary", 1.0 },
  { "Zakkery", 1.0 },
  { "Zakkiyah", 0.0 },
  { "Zakkiyya", 0.0 },
  { "Zakori", 1.0 },
  { "Zakoria", 0.0 },
  { "Zakory", 1.0 },
  { "Zakrey", 1.0 },
  { "Zakri", 1.0 },
  { "Zakria", 1.0 },
  { "Zakry", 1.0 },
  { "Zakwon", 1.0 },
  { "Zaky", 1.0 },
  { "Zakya", 0.0 },
  { "Zakyah", 0.0 },
  { "Zakye", 1.0 },
  { "Zakyia", 0.0 },
  { "Zakyiah", 0.0 },
  { "Zakyla", 0.0 },
  { "Zakylah", 0.0 },
  { "Zakyra", 0.0 },
  { "Zakyrah", 0.0 },
  { "Zakyria", 0.0 },
  { "Zakyriah", 0.0 },
  { "Zakyrie", 1.0 },
  { "Zala", 0.0 },
  { "Zalaiya", 0.0 },
  { "Zalaiyah", 0.0 },
  { "Zalan", 1.0 },
  { "Zalaya", 0.0 },
  { "Zalayah", 0.0 },
  { "Zalaysia", 0.0 },
  { "Zale", 1.0 },
  { "Zalea", 0.0 },
  { "Zaleah", 0.0 },
  { "Zalee", 0.0 },
  { "Zaleena", 0.0 },
  { "Zaleia", 0.0 },
  { "Zaleigh", 0.0 },
  { "Zaleigha", 0.0 },
  { "Zalen", 1.0 },
  { "Zalena", 0.0 },
  { "Zalet", 0.0 },
  { "Zaleth", 0.0 },
  { "Zaley", 0.0 },
  { "Zaleyah", 0.0 },
  { "Zali", 0.0 },
  { "Zalia", 0.0 },
  { "Zaliah", 0.0 },
  { "Zalie", 0.0 },
  { "Zalika", 0.0 },
  { "Zalina", 0.0 },
  { "Zaliya", 0.0 },
  { "Zaliyah", 0.0 },
  { "Zalma", 0.0 },
  { "Zalman", 1.0 },
  { "Zalmen", 1.0 },
  { "Zalon", 1.0 },
  { "Zalyn", 0.91780823 },
  { "Zalynn", 0.23255815 },
  { "Zam", 1.0 },
  { "Zamair", 1.0 },
  { "Zamaira", 0.0 },
  { "Zamaiya", 0.0 },
  { "Zamaiyah", 0.0 },
  { "Zaman", 1.0 },
  { "Zamani", 0.116504855 },
  { "Zamanta", 0.0 },
  { "Zamantha", 0.0 },
  { "Zamar", 1.0 },
  { "Zamara", 0.0 },
  { "Zamarah", 0.0 },
  { "Zamarcus", 1.0 },
  { "Zamare", 1.0 },
  { "Zamaree", 1.0 },
  { "Zamareon", 1.0 },
  { "Zamari", 0.68114513 },
  { "Zamaria", 0.0 },
  { "Zamariah", 0.0 },
  { "Zamarian", 1.0 },
  { "Zamarie", 0.2777778 },
  { "Zamarien", 1.0 },
  { "Zamarii", 1.0 },
  { "Zamarion", 1.0 },
  { "Zamarious", 1.0 },
  { "Zamarius", 1.0 },
  { "Zamariya", 0.0 },
  { "Zamariyah", 0.0 },
  { "Zamarria", 0.0 },
  { "Zamarrion", 1.0 },
  { "Zamauri", 0.9142857 },
  { "Zamauria", 0.0 },
  { "Zamaurion", 1.0 },
  { "Zamaya", 0.0 },
  { "Zamayah", 0.0 },
  { "Zamayra", 0.0 },
  { "Zameer", 1.0 },
  { "Zameir", 1.0 },
  { "Zamera", 0.0 },
  { "Zamere", 1.0 },
  { "Zameria", 0.0 },
  { "Zamia", 0.0 },
  { "Zamiah", 0.0 },
  { "Zamian", 1.0 },
  { "Zamiaya", 0.0 },
  { "Zamiel", 1.0 },
  { "Zamien", 1.0 },
  { "Zamier", 1.0 },
  { "Zamiere", 1.0 },
  { "Zamil", 1.0 },
  { "Zamina", 0.0 },
  { "Zamion", 1.0 },
  { "Zamir", 1.0 },
  { "Zamira", 0.0 },
  { "Zamirah", 0.0 },
  { "Zamire", 1.0 },
  { "Zamirra", 0.0 },
  { "Zamiya", 0.0 },
  { "Zamiyah", 0.0 },
  { "Zamon", 1.0 },
  { "Zamoni", 0.0 },
  { "Zamonte", 1.0 },
  { "Zamora", 0.0 },
  { "Zamorah", 0.0 },
  { "Zamoria", 0.0 },
  { "Zamorion", 1.0 },
  { "Zamuel", 1.0 },
  { "Zamya", 0.0 },
  { "Zamyah", 0.0 },
  { "Zamyia", 0.0 },
  { "Zamyiah", 0.0 },
  { "Zamyla", 0.0 },
  { "Zamyra", 0.0 },
  { "Zamyrah", 0.0 },
  { "Zamyria", 0.0 },
  { "Zamyriah", 0.0 },
  { "Zamzam", 0.0 },
  { "Zan", 0.8407821 },
  { "Zana", 0.0 },
  { "Zanaa", 0.0 },
  { "Zanab", 0.0 },
  { "Zanae", 0.0 },
  { "Zanah", 0.0 },
  { "Zanai", 0.0 },
  { "Zanaia", 0.0 },
  { "Zanaiah", 0.0 },
  { "Zanaii", 0.0 },
  { "Zanaiya", 0.0 },
  { "Zanaiyah", 0.0 },
  { "Zanajah", 0.0 },
  { "Zanari", 0.0 },
  { "Zanaria", 0.0 },
  { "Zanariah", 0.0 },
  { "Zanasia", 0.0 },
  { "Zanay", 0.0 },
  { "Zanaya", 0.0 },
  { "Zanayah", 0.0 },
  { "Zanayla", 0.0 },
  { "Zanda", 0.0 },
  { "Zandalee", 0.0 },
  { "Zandar", 1.0 },
  { "Zandaya", 0.0 },
  { "Zandel", 1.0 },
  { "Zanden", 1.0 },
  { "Zander", 0.99858797 },
  { "Zandon", 1.0 },
  { "Zandra", 0.0 },
  { "Zandre", 1.0 },
  { "Zandrea", 0.0 },
  { "Zandria", 0.0 },
  { "Zandur", 1.0 },
  { "Zandyn", 1.0 },
  { "Zandyr", 1.0 },
  { "Zane", 0.98883647 },
  { "Zanea", 0.0 },
  { "Zaneb", 0.0 },
  { "Zanelle", 0.0 },
  { "Zaneria", 0.0 },
  { "Zaneta", 0.0 },
  { "Zanetta", 0.0 },
  { "Zanette", 0.0 },
  { "Zaney", 0.0 },
  { "Zaneya", 0.0 },
  { "Zani", 0.34351146 },
  { "Zania", 0.0 },
  { "Zaniah", 0.0 },
  { "Zaniaya", 0.0 },
  { "Zanib", 0.0 },
  { "Zanie", 0.0 },
  { "Zaniel", 1.0 },
  { "Zanieya", 0.0 },
  { "Zaniiya", 0.0 },
  { "Zanijah", 0.0 },
  { "Zanilah", 0.0 },
  { "Zanique", 0.0 },
  { "Zanisha", 0.0 },
  { "Zanita", 0.0 },
  { "Zaniya", 0.0 },
  { "Zaniyah", 0.0 },
  { "Zaniyha", 0.0 },
  { "Zaniyla", 0.0 },
  { "Zaniylah", 0.0 },
  { "Zaniyyah", 0.0 },
  { "Zann", 1.0 },
  { "Zanna", 0.0 },
  { "Zannah", 0.0 },
  { "Zannie", 0.5070422 },
  { "Zanniyah", 0.0 },
  { "Zanobia", 0.0 },
  { "Zanovia", 0.0 },
  { "Zanoviah", 0.0 },
  { "Zanquisha", 0.0 },
  { "Zantasia", 0.0 },
  { "Zantavious", 1.0 },
  { "Zanteria", 0.0 },
  { "Zanthia", 0.0 },
  { "Zanthony", 1.0 },
  { "Zantiago", 1.0 },
  { "Zanya", 0.0 },
  { "Zanyah", 0.0 },
  { "Zanye", 0.0 },
  { "Zanyia", 0.0 },
  { "Zanyiah", 0.0 },
  { "Zanyla", 0.0 },
  { "Zanylah", 0.0 },
  { "Zanyra", 0.0 },
  { "Zanyria", 0.0 },
  { "Zanyyah", 0.0 },
  { "Zao", 1.0 },
  { "Zaon", 1.0 },
  { "Zaory", 0.0 },
  { "Zaphira", 0.0 },
  { "Zaquan", 1.0 },
  { "Zaquavious", 1.0 },
  { "Zara", 0.0 },
  { "Zaraan", 1.0 },
  { "Zarae", 0.0 },
  { "Zaragosa", 1.0 },
  { "Zaragoza", 1.0 },
  { "Zarah", 0.0 },
  { "Zarahi", 0.0 },
  { "Zarahy", 0.0 },
  { "Zarai", 0.0 },
  { "Zaraiah", 0.0 },
  { "Zaraiyah", 0.0 },
  { "Zarak", 1.0 },
  { "Zarar", 1.0 },
  { "Zaravion", 1.0 },
  { "Zaray", 0.0 },
  { "Zaraya", 0.0 },
  { "Zarayah", 0.0 },
  { "Zarchary", 1.0 },
  { "Zarea", 0.0 },
  { "Zareah", 0.0 },
  { "Zareb", 1.0 },
  { "Zared", 1.0 },
  { "Zareen", 0.0 },
  { "Zareena", 0.0 },
  { "Zareh", 1.0 },
  { "Zareia", 0.0 },
  { "Zarek", 1.0 },
  { "Zarela", 0.0 },
  { "Zarelia", 0.0 },
  { "Zarella", 0.0 },
  { "Zarely", 0.0 },
  { "Zaren", 1.0 },
  { "Zarena", 0.0 },
  { "Zareon", 1.0 },
  { "Zaret", 0.0 },
  { "Zareth", 0.16666667 },
  { "Zareya", 0.0 },
  { "Zareyah", 0.0 },
  { "Zarhia", 0.0 },
  { "Zarhianna", 0.0 },
  { "Zarhiya", 0.0 },
  { "Zari", 0.019556714 },
  { "Zaria", 0.0 },
  { "Zariah", 0.0075946217 },
  { "Zariaha", 0.0 },
  { "Zarian", 0.9125 },
  { "Zariana", 0.0 },
  { "Zarianna", 0.0 },
  { "Zarie", 0.0 },
  { "Zariea", 0.0 },
  { "Zariel", 0.12371134 },
  { "Zarielle", 0.0 },
  { "Zarien", 1.0 },
  { "Zarif", 1.0 },
  { "Zarifa", 0.0 },
  { "Zariha", 0.0 },
  { "Zarihanna", 0.0 },
  { "Zarihya", 0.0 },
  { "Zarik", 1.0 },
  { "Zarin", 0.5285714 },
  { "Zarina", 0.0 },
  { "Zarinah", 0.0 },
  { "Zarion", 1.0 },
  { "Zariona", 0.0 },
  { "Zarionna", 0.0 },
  { "Zarious", 1.0 },
  { "Zarita", 0.0 },
  { "Zarius", 1.0 },
  { "Zariya", 0.0 },
  { "Zariyah", 0.0 },
  { "Zariyan", 1.0 },
  { "Zariyana", 0.0 },
  { "Zariyha", 0.0 },
  { "Zarlish", 0.0 },
  { "Zarmeen", 0.0 },
  { "Zarmina", 0.0 },
  { "Zaron", 1.0 },
  { "Zaroon", 1.0 },
  { "Zarra", 0.0 },
  { "Zarrah", 0.0 },
  { "Zarrar", 1.0 },
  { "Zarrea", 0.0 },
  { "Zarreah", 0.0 },
  { "Zarren", 1.0 },
  { "Zarria", 0.0 },
  { "Zarriah", 0.0 },
  { "Zarrian", 1.0 },
  { "Zarrien", 1.0 },
  { "Zarrin", 0.0 },
  { "Zarrion", 1.0 },
  { "Zarrius", 1.0 },
  { "Zarriyah", 0.0 },
  { "Zarya", 0.0 },
  { "Zaryab", 1.0 },
  { "Zaryah", 0.0 },
  { "Zaryan", 1.0 },
  { "Zaryha", 0.0 },
  { "Zaryia", 0.0 },
  { "Zaryiah", 0.0 },
  { "Zaryk", 1.0 },
  { "Zaryn", 0.4909091 },
  { "Zasha", 0.0 },
  { "Zashawn", 1.0 },
  { "Zasia", 0.0 },
  { "Zatanna", 0.0 },
  { "Zatara", 0.0 },
  { "Zatavia", 0.0 },
  { "Zathan", 1.0 },
  { "Zatoria", 0.0 },
  { "Zaul", 1.0 },
  { "Zaundra", 0.0 },
  { "Zauria", 0.0 },
  { "Zavaeh", 0.0 },
  { "Zavanna", 0.0 },
  { "Zavannah", 0.0 },
  { "Zaveah", 0.0 },
  { "Zaveion", 1.0 },
  { "Zaven", 1.0 },
  { "Zaveon", 1.0 },
  { "Zavery", 0.9019608 },
  { "Zavhary", 1.0 },
  { "Zavi", 1.0 },
  { "Zavia", 0.047244094 },
  { "Zavian", 1.0 },
  { "Zaviar", 1.0 },
  { "Zavien", 1.0 },
  { "Zavier", 0.99897563 },
  { "Zaviera", 0.0 },
  { "Zaviere", 1.0 },
  { "Zavin", 1.0 },
  { "Zavion", 1.0 },
  { "Zavior", 1.0 },
  { "Zavious", 1.0 },
  { "Zavius", 1.0 },
  { "Zaviyar", 1.0 },
  { "Zaviyon", 1.0 },
  { "Zavon", 1.0 },
  { "Zavyer", 1.0 },
  { "Zavyn", 1.0 },
  { "Zavyon", 1.0 },
  { "Zawadi", 0.0 },
  { "Zawdie", 1.0 },
  { "Zaxon", 1.0 },
  { "Zaxton", 1.0 },
  { "Zaxtyn", 1.0 },
  { "Zay", 0.8965517 },
  { "Zaya", 0.024120603 },
  { "Zayaan", 1.0 },
  { "Zayah", 0.043010753 },
  { "Zayan", 1.0 },
  { "Zayana", 0.0 },
  { "Zayanah", 0.0 },
  { "Zayani", 0.0 },
  { "Zayanna", 0.0 },
  { "Zayannah", 0.0 },
  { "Zaybree", 0.0 },
  { "Zaybrien", 1.0 },
  { "Zayceon", 1.0 },
  { "Zayd", 1.0 },
  { "Zayda", 0.0 },
  { "Zaydah", 0.0 },
  { "Zaydan", 1.0 },
  { "Zayde", 0.87623763 },
  { "Zaydee", 0.0 },
  { "Zayden", 0.99122745 },
  { "Zaydenn", 1.0 },
  { "Zaydi", 0.0 },
  { "Zaydian", 1.0 },
  { "Zaydie", 0.0 },
  { "Zaydien", 1.0 },
  { "Zaydin", 1.0 },
  { "Zaydn", 1.0 },
  { "Zaydon", 1.0 },
  { "Zaydren", 1.0 },
  { "Zaydrian", 1.0 },
  { "Zaydyn", 1.0 },
  { "Zaye", 1.0 },
  { "Zayed", 1.0 },
  { "Zayen", 1.0 },
  { "Zayia", 0.0 },
  { "Zayiah", 0.0 },
  { "Zayid", 1.0 },
  { "Zayin", 1.0 },
  { "Zayion", 1.0 },
  { "Zayir", 1.0 },
  { "Zaykeese", 1.0 },
  { "Zayla", 0.0 },
  { "Zaylaa", 0.0 },
  { "Zaylah", 0.0 },
  { "Zaylan", 1.0 },
  { "Zayland", 1.0 },
  { "Zaylani", 0.0 },
  { "Zaylea", 0.0 },
  { "Zayleah", 0.0 },
  { "Zaylee", 0.0 },
  { "Zayleen", 0.0 },
  { "Zaylei", 0.0 },
  { "Zayleigh", 0.0 },
  { "Zaylen", 0.9720339 },
  { "Zaylene", 0.0 },
  { "Zayley", 0.0 },
  { "Zayli", 0.0 },
  { "Zaylia", 0.0 },
  { "Zayliah", 0.0 },
  { "Zayliana", 0.0 },
  { "Zaylie", 0.0 },
  { "Zaylin", 0.62115383 },
  { "Zaylon", 1.0 },
  { "Zaylor", 1.0 },
  { "Zaylyn", 0.6454545 },
  { "Zaylynn", 0.09815951 },
  { "Zaymar", 1.0 },
  { "Zaymere", 1.0 },
  { "Zaymir", 1.0 },
  { "Zayn", 0.99031985 },
  { "Zayna", 0.0 },
  { "Zaynab", 0.0 },
  { "Zaynah", 0.0 },
  { "Zayne", 0.9796189 },
  { "Zayneb", 0.0 },
  { "Zaynn", 1.0 },
  { "Zayon", 1.0 },
  { "Zayona", 0.0 },
  { "Zayonna", 0.0 },
  { "Zayquan", 1.0 },
  { "Zayra", 0.0 },
  { "Zayriah", 0.0 },
  { "Zaysean", 1.0 },
  { "Zaysha", 0.0 },
  { "Zayshaun", 1.0 },
  { "Zayshawn", 1.0 },
  { "Zaysia", 0.0 },
  { "Zayuri", 0.0 },
  { "Zayven", 1.0 },
  { "Zayveon", 1.0 },
  { "Zayvian", 1.0 },
  { "Zayvien", 1.0 },
  { "Zayvier", 1.0 },
  { "Zayvin", 1.0 },
  { "Zayvion", 1.0 },
  { "Zayvon", 1.0 },
  { "Zayyan", 1.0 },
  { "Zazil", 0.0 },
  { "Zbigniew", 1.0 },
  { "Zdenka", 0.0 },
  { "Ze", 1.0 },
  { "Zea", 0.0 },
  { "Zeah", 0.0 },
  { "Zeak", 1.0 },
  { "Zeal", 0.7962963 },
  { "Zealan", 1.0 },
  { "Zealand", 0.92248064 },
  { "Zealyn", 0.0 },
  { "Zean", 1.0 },
  { "Zeana", 0.0 },
  { "Zeandre", 1.0 },
  { "Zeanna", 0.0 },
  { "Zearl", 1.0 },
  { "Zeasia", 0.0 },
  { "Zeb", 1.0 },
  { "Zeba", 0.0 },
  { "Zebadiah", 1.0 },
  { "Zebariah", 1.0 },
  { "Zebastian", 1.0 },
  { "Zebbie", 0.94285715 },
  { "Zebdee", 1.0 },
  { "Zebedee", 1.0 },
  { "Zebediah", 1.0 },
  { "Zebidiah", 1.0 },
  { "Zeborah", 0.0 },
  { "Zebra", 0.0 },
  { "Zebrina", 0.0 },
  { "Zebula", 1.0 },
  { "Zebulah", 1.0 },
  { "Zebulan", 1.0 },
  { "Zebulen", 1.0 },
  { "Zebulin", 1.0 },
  { "Zebulon", 1.0 },
  { "Zebulun", 1.0 },
  { "Zecharia", 1.0 },
  { "Zechariah", 0.9985398 },
  { "Zechary", 1.0 },
  { "Zecharya", 1.0 },
  { "Zecharyah", 1.0 },
  { "Zecheriah", 1.0 },
  { "Zed", 1.0 },
  { "Zeda", 0.0 },
  { "Zedan", 1.0 },
  { "Zeddicus", 1.0 },
  { "Zeddie", 0.8076923 },
  { "Zedek", 1.0 },
  { "Zedekiah", 1.0 },
  { "Zederick", 1.0 },
  { "Zedric", 1.0 },
  { "Zedrick", 1.0 },
  { "Zee", 0.5733333 },
  { "Zeek", 1.0 },
  { "Zeel", 0.0 },
  { "Zeena", 0.0 },
  { "Zeenah", 0.0 },
  { "Zeenat", 0.0 },
  { "Zeenia", 0.0 },
  { "Zeeshan", 1.0 },
  { "Zeev", 1.0 },
  { "Zeeva", 0.0 },
  { "Zef", 1.0 },
  { "Zeferino", 1.0 },
  { "Zeffie", 0.0 },
  { "Zefram", 1.0 },
  { "Zehava", 0.0 },
  { "Zehra", 0.0 },
  { "Zeid", 1.0 },
  { "Zeida", 0.0 },
  { "Zeidan", 1.0 },
  { "Zeiden", 1.0 },
  { "Zeidi", 0.0 },
  { "Zeidy", 0.0 },
  { "Zeik", 1.0 },
  { "Zeila", 0.0 },
  { "Zeilah", 0.0 },
  { "Zeilin", 0.0 },
  { "Zeily", 0.0 },
  { "Zeilyn", 0.0 },
  { "Zein", 1.0 },
  { "Zeina", 0.0 },
  { "Zeinab", 0.0 },
  { "Zeinabou", 0.0 },
  { "Zeinah", 0.0 },
  { "Zeineb", 0.0 },
  { "Zek", 1.0 },
  { "Zekai", 1.0 },
  { "Zekariah", 1.0 },
  { "Zeke", 1.0 },
  { "Zekeriah", 1.0 },
  { "Zekharyah", 1.0 },
  { "Zekhi", 1.0 },
  { "Zeki", 1.0 },
  { "Zekia", 0.0 },
  { "Zekiah", 0.66255146 },
  { "Zekial", 1.0 },
  { "Zekiel", 1.0 },
  { "Zekira", 0.0 },
  { "Zekiyah", 0.0 },
  { "Zel", 1.0 },
  { "Zela", 0.0 },
  { "Zelah", 0.0 },
  { "Zelan", 1.0 },
  { "Zelaya", 0.0 },
  { "Zelbert", 1.0 },
  { "Zelda", 0.00038952945 },
  { "Zelder", 0.0 },
  { "Zeldia", 0.0 },
  { "Zeldon", 1.0 },
  { "Zeldy", 0.0 },
  { "Zelena", 0.0 },
  { "Zelene", 0.0 },
  { "Zelenia", 0.0 },
  { "Zelia", 0.0 },
  { "Zelie", 0.0 },
  { "Zelig", 1.0 },
  { "Zelina", 0.0 },
  { "Zelinda", 0.0 },
  { "Zeline", 0.0 },
  { "Zeljko", 1.0 },
  { "Zell", 0.3428064 },
  { "Zella", 0.0 },
  { "Zellah", 0.0 },
  { "Zellamae", 0.0 },
  { "Zellar", 0.0 },
  { "Zelle", 0.0 },
  { "Zellia", 0.0 },
  { "Zellie", 0.027777778 },
  { "Zelline", 0.0 },
  { "Zelma", 0.0066599273 },
  { "Zelmadene", 0.0 },
  { "Zelmarie", 0.0 },
  { "Zelmer", 0.7647059 },
  { "Zelmodene", 0.0 },
  { "Zelna", 0.0 },
  { "Zelpha", 0.0 },
  { "Zelphia", 0.0 },
  { "Zelta", 0.0 },
  { "Zeltzin", 0.0 },
  { "Zelva", 0.0 },
  { "Zema", 0.0 },
  { "Zemar", 1.0 },
  { "Zemarion", 1.0 },
  { "Zemira", 0.0 },
  { "Zemirah", 0.0 },
  { "Zemiyah", 0.0 },
  { "Zemora", 0.0 },
  { "Zen", 0.85898405 },
  { "Zena", 0.0 },
  { "Zenab", 0.0 },
  { "Zenae", 0.0 },
  { "Zenah", 0.0 },
  { "Zenai", 0.0 },
  { "Zenaida", 0.0 },
  { "Zenaido", 1.0 },
  { "Zenaiya", 0.0 },
  { "Zenani", 0.0 },
  { "Zenas", 0.98868775 },
  { "Zenaya", 0.0 },
  { "Zenayah", 0.0 },
  { "Zenayda", 0.0 },
  { "Zenda", 0.0 },
  { "Zendaiya", 0.0 },
  { "Zendaya", 0.0 },
  { "Zendayah", 0.0 },
  { "Zende", 1.0 },
  { "Zendell", 1.0 },
  { "Zenden", 1.0 },
  { "Zender", 1.0 },
  { "Zendeya", 0.0 },
  { "Zendiya", 0.0 },
  { "Zendra", 0.0 },
  { "Zendre", 0.0 },
  { "Zendy", 0.0 },
  { "Zene", 0.0 },
  { "Zeneida", 0.0 },
  { "Zenen", 1.0 },
  { "Zenetta", 0.0 },
  { "Zeni", 0.0 },
  { "Zenia", 0.0 },
  { "Zeniah", 0.0 },
  { "Zenida", 0.0 },
  { "Zenis", 1.0 },
  { "Zenita", 0.0 },
  { "Zenith", 0.23267327 },
  { "Zeniya", 0.0 },
  { "Zeniyah", 0.0 },
  { "Zenja", 0.0 },
  { "Zenna", 0.0 },
  { "Zennia", 0.0 },
  { "Zennie", 0.08130081 },
  { "Zennon", 1.0 },
  { "Zeno", 1.0 },
  { "Zenoba", 0.0 },
  { "Zenobia", 0.0 },
  { "Zenola", 0.0 },
  { "Zenolia", 0.0 },
  { "Zenon", 1.0 },
  { "Zenona", 0.0 },
  { "Zenora", 0.0 },
  { "Zenorah", 0.0 },
  { "Zenos", 1.0 },
  { "Zenovia", 0.0 },
  { "Zentavious", 1.0 },
  { "Zenus", 1.0 },
  { "Zeny", 0.0 },
  { "Zenya", 0.0 },
  { "Zenzele", 0.0 },
  { "Zeola", 0.0 },
  { "Zeon", 1.0 },
  { "Zeonna", 0.0 },
  { "Zeph", 1.0 },
  { "Zepha", 0.0 },
  { "Zephan", 1.0 },
  { "Zephania", 0.0 },
  { "Zephaniah", 0.87643677 },
  { "Zephen", 1.0 },
  { "Zepher", 0.0 },
  { "Zephery", 1.0 },
  { "Zephora", 0.0 },
  { "Zephram", 1.0 },
  { "Zephyn", 1.0 },
  { "Zephyr", 0.74519575 },
  { "Zephyra", 0.0 },
  { "Zephyrus", 1.0 },
  { "Zeplin", 1.0 },
  { "Zeplyn", 0.0 },
  { "Zeplynn", 0.0 },
  { "Zeporia", 0.0 },
  { "Zeppelin", 0.8944954 },
  { "Zeppelyn", 0.0 },
  { "Zepplin", 1.0 },
  { "Zequan", 1.0 },
  { "Zer", 0.0 },
  { "Zera", 0.017152658 },
  { "Zerah", 0.0 },
  { "Zerayah", 0.0 },
  { "Zerek", 1.0 },
  { "Zerelda", 0.0 },
  { "Zerena", 0.0 },
  { "Zerenity", 0.0 },
  { "Zereon", 1.0 },
  { "Zeriah", 0.023584906 },
  { "Zeric", 1.0 },
  { "Zerick", 1.0 },
  { "Zerik", 1.0 },
  { "Zerin", 1.0 },
  { "Zerina", 0.0 },
  { "Zerion", 1.0 },
  { "Zerita", 0.0 },
  { "Zeriyah", 0.0 },
  { "Zerlene", 0.0 },
  { "Zerlina", 0.0 },
  { "Zerline", 0.0 },
  { "Zeron", 1.0 },
  { "Zerriah", 0.0 },
  { "Zerrick", 1.0 },
  { "Zerrion", 1.0 },
  { "Zeruiah", 0.0 },
  { "Zeryk", 1.0 },
  { "Zesar", 1.0 },
  { "Zeshan", 1.0 },
  { "Zeshaun", 1.0 },
  { "Zeshawn", 1.0 },
  { "Zeta", 0.0 },
  { "Zeth", 1.0 },
  { "Zetha", 0.0 },
  { "Zethan", 1.0 },
  { "Zetta", 0.0 },
  { "Zettie", 0.006811989 },
  { "Zeus", 1.0 },
  { "Zev", 1.0 },
  { "Zeva", 0.0 },
  { "Zevaeh", 0.0 },
  { "Zevan", 1.0 },
  { "Zeven", 1.0 },
  { "Zevi", 1.0 },
  { "Zevin", 1.0 },
  { "Zevion", 1.0 },
  { "Zevon", 1.0 },
  { "Zevulun", 1.0 },
  { "Zevyn", 1.0 },
  { "Zeya", 0.0 },
  { "Zeyad", 1.0 },
  { "Zeyah", 0.0 },
  { "Zeyana", 0.0 },
  { "Zeyd", 1.0 },
  { "Zeyda", 0.0 },
  { "Zeyden", 1.0 },
  { "Zeyla", 0.0 },
  { "Zeylah", 0.0 },
  { "Zeyna", 0.0 },
  { "Zeynab", 0.0 },
  { "Zeyneb", 0.0 },
  { "Zeynep", 0.0 },
  { "Zeza", 0.0 },
  { "Zhaiden", 1.0 },
  { "Zhair", 1.0 },
  { "Zhaira", 0.0 },
  { "Zhaire", 1.0 },
  { "Zhaki", 1.0 },
  { "Zhakira", 0.0 },
  { "Zhalen", 1.0 },
  { "Zhamir", 1.0 },
  { "Zhamira", 0.0 },
  { "Zhana", 0.0 },
  { "Zhanae", 0.0 },
  { "Zhanasia", 0.0 },
  { "Zhanaya", 0.0 },
  { "Zhander", 1.0 },
  { "Zhane", 0.13107026 },
  { "Zhanea", 0.0 },
  { "Zhanee", 0.0 },
  { "Zhanel", 0.0 },
  { "Zhanelle", 0.0 },
  { "Zhania", 0.0 },
  { "Zhaniya", 0.0 },
  { "Zhaniyah", 0.0 },
  { "Zhanna", 0.0 },
  { "Zhanya", 0.0 },
  { "Zhara", 0.0 },
  { "Zhari", 0.0 },
  { "Zharia", 0.0 },
  { "Zhariah", 0.0 },
  { "Zharick", 0.0 },
  { "Zhariya", 0.0 },
  { "Zhariyah", 0.0 },
  { "Zharya", 0.0 },
  { "Zharyah", 0.0 },
  { "Zhayden", 1.0 },
  { "Zhayne", 1.0 },
  { "Zhen", 0.8484849 },
  { "Zheneavia", 0.0 },
  { "Zheng", 1.0 },
  { "Zhi", 0.89440995 },
  { "Zhia", 0.0 },
  { "Zhian", 1.0 },
  { "Zhianna", 0.0 },
  { "Zhion", 1.0 },
  { "Zhivago", 1.0 },
  { "Zhiya", 0.0 },
  { "Zhoe", 0.0 },
  { "Zhoemi", 0.0 },
  { "Zhoey", 0.0 },
  { "Zhoie", 0.0 },
  { "Zhuri", 0.0 },
  { "Zhyair", 1.0 },
  { "Zhyaire", 1.0 },
  { "Zhyier", 1.0 },
  { "Zhyir", 1.0 },
  { "Zhymir", 1.0 },
  { "Zhyon", 0.8208955 },
  { "Zi", 0.51282054 },
  { "Zia", 0.102813296 },
  { "Ziad", 1.0 },
  { "Ziah", 0.22187005 },
  { "Ziair", 1.0 },
  { "Ziaira", 0.0 },
  { "Ziaire", 0.8598575 },
  { "Zian", 0.73406196 },
  { "Ziana", 0.0 },
  { "Zianah", 0.0 },
  { "Ziane", 0.0 },
  { "Ziani", 0.0 },
  { "Ziann", 0.0 },
  { "Zianna", 0.0 },
  { "Ziannah", 0.0 },
  { "Zianne", 0.0 },
  { "Zianny", 0.0 },
  { "Zianya", 0.0 },
  { "Ziar", 1.0 },
  { "Ziara", 0.0 },
  { "Ziarah", 0.0 },
  { "Ziare", 0.81333333 },
  { "Ziarra", 0.0 },
  { "Ziarre", 0.0 },
  { "Ziasia", 0.0 },
  { "Ziba", 1.0 },
  { "Zichen", 1.0 },
  { "Ziclali", 0.0 },
  { "Ziclaly", 0.0 },
  { "Zico", 1.0 },
  { "Zida", 0.0 },
  { "Zidaan", 1.0 },
  { "Zidan", 1.0 },
  { "Zidane", 1.0 },
  { "Ziden", 1.0 },
  { "Zidon", 1.0 },
  { "Ziel", 0.6 },
  { "Zien", 1.0 },
  { "Ziena", 0.0 },
  { "Zienna", 0.0 },
  { "Zier", 1.0 },
  { "Ziera", 0.0 },
  { "Ziere", 1.0 },
  { "Zierra", 0.0 },
  { "Zierre", 1.0 },
  { "Ziggie", 1.0 },
  { "Ziggy", 0.9527027 },
  { "Zigmond", 1.0 },
  { "Zigmont", 1.0 },
  { "Zigmund", 1.0 },
  { "Zigmunt", 1.0 },
  { "Ziham", 0.0 },
  { "Zihan", 0.51724136 },
  { "Zihanna", 0.0 },
  { "Zihao", 1.0 },
  { "Ziheir", 1.0 },
  { "Ziheng", 1.0 },
  { "Ziion", 1.0 },
  { "Zijun", 1.0 },
  { "Zikayla", 0.0 },
  { "Zikee", 1.0 },
  { "Zikeria", 0.0 },
  { "Zikeya", 0.0 },
  { "Zikeyah", 0.0 },
  { "Zikia", 0.0 },
  { "Zikira", 0.0 },
  { "Zikiria", 0.0 },
  { "Zikiya", 0.0 },
  { "Zikomo", 1.0 },
  { "Zikora", 0.8064516 },
  { "Zikra", 0.0 },
  { "Zila", 0.0 },
  { "Zilah", 0.0 },
  { "Zilas", 1.0 },
  { "Zilda", 0.0 },
  { "Zildjian", 1.0 },
  { "Ziley", 0.0 },
  { "Zilin", 0.0 },
  { "Zilla", 0.0 },
  { "Zillah", 0.0 },
  { "Zillie", 0.0 },
  { "Zilpah", 0.0 },
  { "Zilpha", 0.0 },
  { "Zilphia", 0.0 },
  { "Zilynn", 0.0 },
  { "Zim", 1.0 },
  { "Zimal", 0.0 },
  { "Zimaria", 0.0 },
  { "Zimarion", 1.0 },
  { "Zimaya", 0.0 },
  { "Zimbabwe", 1.0 },
  { "Zimbalist", 1.0 },
  { "Zimere", 1.0 },
  { "Zimir", 1.0 },
  { "Zimmie", 0.0 },
  { "Zimo", 1.0 },
  { "Zimora", 0.0 },
  { "Zimri", 0.9059829 },
  { "Zimya", 0.0 },
  { "Zina", 0.0020955575 },
  { "Zinab", 0.0 },
  { "Zinachidi", 0.0 },
  { "Zinachimdi", 0.0 },
  { "Zinaya", 0.0 },
  { "Zinda", 0.0 },
  { "Zineb", 0.0 },
  { "Zinedin", 1.0 },
  { "Zinedine", 1.0 },
  { "Zing", 0.0 },
  { "Zinia", 0.0 },
  { "Ziniah", 0.0 },
  { "Ziniya", 0.0 },
  { "Ziniyah", 0.0 },
  { "Zinn", 1.0 },
  { "Zinna", 0.0 },
  { "Zinnia", 0.0 },
  { "Ziomara", 0.0 },
  { "Zion", 0.8167448 },
  { "Ziona", 0.0 },
  { "Zionah", 0.0 },
  { "Zione", 1.0 },
  { "Zionn", 1.0 },
  { "Zionna", 0.0 },
  { "Zionnah", 0.0 },
  { "Zionne", 0.0 },
  { "Zipora", 0.0 },
  { "Ziporah", 0.0 },
  { "Zipporah", 0.0 },
  { "Ziqi", 0.0 },
  { "Ziquan", 1.0 },
  { "Zira", 0.0 },
  { "Zirah", 0.0 },
  { "Ziraili", 0.0 },
  { "Ziraily", 0.0 },
  { "Zirel", 0.0 },
  { "Zirui", 1.0 },
  { "Zirwa", 0.0 },
  { "Zisel", 0.0 },
  { "Zisha", 1.0 },
  { "Zishan", 1.0 },
  { "Zishe", 1.0 },
  { "Zissel", 0.0 },
  { "Zissy", 0.0 },
  { "Zita", 0.0 },
  { "Zithlaly", 0.0 },
  { "Zitlali", 0.0 },
  { "Zitlalic", 0.0 },
  { "Zitlalli", 0.0 },
  { "Zitlally", 0.0 },
  { "Zitlaly", 0.0 },
  { "Zitong", 0.0 },
  { "Ziv", 1.0 },
  { "Ziva", 0.0 },
  { "Zivah", 0.0 },
  { "Ziven", 1.0 },
  { "Zivon", 1.0 },
  { "Zixi", 0.0 },
  { "Zixin", 0.0 },
  { "Zixuan", 0.56363636 },
  { "Ziya", 0.03085554 },
  { "Ziyaad", 1.0 },
  { "Ziyad", 1.0 },
  { "Ziyadah", 0.0 },
  { "Ziyah", 0.026785715 },
  { "Ziyan", 0.5608856 },
  { "Ziyana", 0.0 },
  { "Ziyanah", 0.0 },
  { "Ziyang", 1.0 },
  { "Ziyanna", 0.0 },
  { "Ziyear", 1.0 },
  { "Ziyi", 0.5 },
  { "Ziyla", 0.0 },
  { "Ziyon", 0.8267091 },
  { "Ziyona", 0.0 },
  { "Ziyonna", 0.0 },
  { "Ziyu", 1.0 },
  { "Ziyun", 1.0 },
  { "Ziza", 0.0 },
  { "Zizi", 0.0 },
  { "Zkai", 1.0 },
  { "Zkye", 1.0 },
  { "Zlata", 0.0 },
  { "Zlatan", 1.0 },
  { "Zlaty", 0.0 },
  { "Zmari", 1.0 },
  { "Zmarion", 1.0 },
  { "Zmaya", 0.0 },
  { "Zmiyah", 0.0 },
  { "Zmya", 0.0 },
  { "Zniah", 0.0 },
  { "Zniya", 0.0 },
  { "Zniyah", 0.0 },
  { "Znya", 0.0 },
  { "Znyah", 0.0 },
  { "Zo", 0.14814815 },
  { "Zoa", 0.0 },
  { "Zoann", 0.0 },
  { "Zoanna", 0.0 },
  { "Zoanne", 0.0 },
  { "Zobeida", 0.0 },
  { "Zobia", 0.0 },
  { "Zody", 0.0 },
  { "Zoe", 0.003827784 },
  { "Zoeann", 0.0 },
  { "Zoeanna", 0.0 },
  { "Zoeanne", 0.0 },
  { "Zoee", 0.0 },
  { "Zoeey", 0.0 },
  { "Zoegrace", 0.0 },
  { "Zoeh", 0.0 },
  { "Zoei", 0.0 },
  { "Zoeie", 0.0 },
  { "Zoeigh", 0.0 },
  { "Zoeii", 0.0 },
  { "Zoeiy", 0.0 },
  { "Zoejane", 0.0 },
  { "Zoel", 0.6515151 },
  { "Zoelie", 0.0 },
  { "Zoella", 0.0 },
  { "Zoelle", 0.0 },
  { "Zoely", 0.0 },
  { "Zoelyn", 0.0 },
  { "Zoelynn", 0.0 },
  { "Zoelys", 0.0 },
  { "Zoemarie", 0.0 },
  { "Zoen", 1.0 },
  { "Zoey", 0.0027200584 },
  { "Zoeya", 0.0 },
  { "Zoeyann", 0.0 },
  { "Zoeygrace", 0.0 },
  { "Zoeyjane", 0.0 },
  { "Zoeylynn", 0.0 },
  { "Zoeymarie", 0.0 },
  { "Zofia", 0.0 },
  { "Zofie", 0.0 },
  { "Zoha", 0.0 },
  { "Zohaib", 1.0 },
  { "Zohair", 1.0 },
  { "Zohal", 0.0 },
  { "Zohan", 1.0 },
  { "Zohar", 0.35323384 },
  { "Zohara", 0.0 },
  { "Zohe", 0.0 },
  { "Zoheb", 1.0 },
  { "Zohie", 0.0 },
  { "Zohra", 0.0 },
  { "Zohrab", 1.0 },
  { "Zoi", 0.0 },
  { "Zoia", 0.0 },
  { "Zoie", 0.0 },
  { "Zoiee", 0.0 },
  { "Zoiey", 0.0 },
  { "Zoii", 0.0 },
  { "Zoiie", 0.0 },
  { "Zoila", 0.0 },
  { "Zoilo", 1.0 },
  { "Zoja", 0.0 },
  { "Zola", 0.0029506814 },
  { "Zolah", 0.0 },
  { "Zolan", 1.0 },
  { "Zolar", 0.0 },
  { "Zolee", 0.0 },
  { "Zoli", 0.0 },
  { "Zolie", 0.0 },
  { "Zolin", 1.0 },
  { "Zollie", 0.8599791 },
  { "Zoltan", 1.0 },
  { "Zolton", 1.0 },
  { "Zoma", 0.0 },
  { "Zon", 1.0 },
  { "Zona", 0.0 },
  { "Zonda", 0.0 },
  { "Zondra", 0.0 },
  { "Zong", 0.8119658 },
  { "Zonia", 0.0 },
  { "Zonie", 0.0 },
  { "Zonna", 0.0 },
  { "Zonnie", 0.0625 },
  { "Zonnique", 0.0 },
  { "Zonya", 0.0 },
  { "Zooey", 0.0 },
  { "Zophia", 0.0 },
  { "Zophie", 0.0 },
  { "Zora", 0.0027829313 },
  { "Zorah", 0.0 },
  { "Zoraida", 0.0 },
  { "Zoraiz", 1.0 },
  { "Zoran", 1.0 },
  { "Zorana", 0.0 },
  { "Zoravar", 1.0 },
  { "Zoraver", 1.0 },
  { "Zorawar", 1.0 },
  { "Zoraya", 0.0 },
  { "Zorayda", 0.0 },
  { "Zoree", 0.0 },
  { "Zoren", 1.0 },
  { "Zori", 0.0 },
  { "Zoria", 0.0 },
  { "Zoriah", 0.0 },
  { "Zorian", 1.0 },
  { "Zoriana", 0.0 },
  { "Zorianna", 0.0 },
  { "Zoribel", 0.0 },
  { "Zorica", 0.0 },
  { "Zorie", 0.0 },
  { "Zorien", 1.0 },
  { "Zorina", 0.0 },
  { "Zorion", 1.0 },
  { "Zorita", 0.0 },
  { "Zoriyah", 0.0 },
  { "Zorka", 0.0 },
  { "Zorria", 0.0 },
  { "Zorriah", 0.0 },
  { "Zorrie", 0.0 },
  { "Zory", 0.0 },
  { "Zorya", 0.0 },
  { "Zoryana", 0.0 },
  { "Zosha", 0.0 },
  { "Zosia", 0.0 },
  { "Zoua", 0.0 },
  { "Zowee", 0.0 },
  { "Zowey", 0.0 },
  { "Zowie", 0.0 },
  { "Zowii", 0.0 },
  { "Zoya", 0.0 },
  { "Zoyah", 0.0 },
  { "Zoye", 0.0 },
  { "Zoyla", 0.0 },
  { "Zriah", 0.0 },
  { "Zriyah", 0.0 },
  { "Zsa", 0.0 },
  { "Zsazsa", 0.0 },
  { "Zshakira", 0.0 },
  { "Zsofia", 0.0 },
  { "Zsolt", 1.0 },
  { "Zsophia", 0.0 },
  { "Zubaida", 0.0 },
  { "Zubaidah", 0.0 },
  { "Zubair", 1.0 },
  { "Zubayr", 1.0 },
  { "Zuberi", 1.0 },
  { "Zubeyr", 1.0 },
  { "Zubie", 0.0 },
  { "Zubin", 1.0 },
  { "Zudie", 0.0 },
  { "Zudora", 0.0 },
  { "Zuella", 0.0 },
  { "Zuha", 0.0 },
  { "Zuhaib", 1.0 },
  { "Zuhair", 1.0 },
  { "Zuhal", 0.0 },
  { "Zuhayr", 1.0 },
  { "Zuheily", 0.0 },
  { "Zuhey", 0.0 },
  { "Zuheyla", 0.0 },
  { "Zuhra", 0.0 },
  { "Zuhur", 0.0 },
  { "Zui", 1.0 },
  { "Zujeily", 0.0 },
  { "Zujey", 0.0 },
  { "Zuko", 1.0 },
  { "Zula", 0.0 },
  { "Zulai", 0.0 },
  { "Zulaika", 0.0 },
  { "Zular", 0.0 },
  { "Zulay", 0.0 },
  { "Zuleica", 0.0 },
  { "Zuleidy", 0.0 },
  { "Zuleika", 0.0 },
  { "Zuleikha", 0.0 },
  { "Zuleima", 0.0 },
  { "Zuleimy", 0.0 },
  { "Zulekha", 0.0 },
  { "Zulema", 0.0 },
  { "Zulene", 0.0 },
  { "Zuley", 0.0 },
  { "Zuleyca", 0.0 },
  { "Zuleyka", 0.0 },
  { "Zuleyma", 0.0 },
  { "Zuleymi", 0.0 },
  { "Zuli", 0.0 },
  { "Zulia", 0.0 },
  { "Zuliana", 0.0 },
  { "Zulie", 0.0 },
  { "Zulimar", 0.0 },
  { "Zulla", 0.0 },
  { "Zully", 0.0 },
  { "Zulma", 0.0 },
  { "Zulmarie", 0.0 },
  { "Zulmira", 0.0 },
  { "Zulmy", 0.0 },
  { "Zuly", 0.0 },
  { "Zuma", 0.0 },
  { "Zumra", 0.0 },
  { "Zunaira", 0.0 },
  { "Zunairah", 0.0 },
  { "Zunilda", 0.0 },
  { "Zura", 0.0 },
  { "Zuree", 0.0 },
  { "Zuri", 0.085219294 },
  { "Zuria", 0.0 },
  { "Zuriah", 0.032967035 },
  { "Zuriana", 0.0 },
  { "Zurianna", 0.0 },
  { "Zurich", 1.0 },
  { "Zurie", 0.0 },
  { "Zuriel", 0.83395755 },
  { "Zuriela", 0.0 },
  { "Zurielle", 0.0 },
  { "Zurii", 0.0 },
  { "Zurina", 0.0 },
  { "Zurisadai", 0.0 },
  { "Zurisaday", 0.0 },
  { "Zuriya", 0.0 },
  { "Zuriyah", 0.0 },
  { "Zurri", 0.0 },
  { "Zury", 0.0 },
  { "Zuszeth", 0.0 },
  { "Zuwena", 0.0 },
  { "Zuzana", 0.0 },
  { "Zuzanna", 0.0 },
  { "Zuzu", 0.0 },
  { "Zvi", 1.0 },
  { "Zxavian", 1.0 },
  { "Zy", 0.85 },
  { "Zya", 0.0 },
  { "Zyad", 1.0 },
  { "Zyah", 0.039285716 },
  { "Zyahir", 1.0 },
  { "Zyahna", 0.0 },
  { "Zyair", 0.88555557 },
  { "Zyaira", 0.0 },
  { "Zyairah", 0.0 },
  { "Zyaire", 0.80763054 },
  { "Zyairr", 1.0 },
  { "Zyairra", 0.0 },
  { "Zyan", 0.74515235 },
  { "Zyana", 0.0 },
  { "Zyanah", 0.0 },
  { "Zyani", 0.0 },
  { "Zyann", 0.0 },
  { "Zyanna", 0.0 },
  { "Zyannah", 0.0 },
  { "Zyanne", 0.0 },
  { "Zyanya", 0.0 },
  { "Zyara", 0.0 },
  { "Zyarah", 0.0 },
  { "Zyare", 1.0 },
  { "Zyaria", 0.0 },
  { "Zyariah", 0.0 },
  { "Zyashia", 0.0 },
  { "Zyasia", 0.0 },
  { "Zyaun", 1.0 },
  { "Zyauna", 0.0 },
  { "Zyda", 0.0 },
  { "Zydarius", 1.0 },
  { "Zyden", 1.0 },
  { "Zydia", 0.0 },
  { "Zye", 1.0 },
  { "Zyeer", 1.0 },
  { "Zyeir", 1.0 },
  { "Zyeire", 1.0 },
  { "Zyel", 1.0 },
  { "Zyell", 1.0 },
  { "Zyelle", 0.0 },
  { "Zyen", 1.0 },
  { "Zyera", 0.0 },
  { "Zyere", 1.0 },
  { "Zyeria", 0.0 },
  { "Zyerre", 1.0 },
  { "Zyesha", 0.0 },
  { "Zygmond", 1.0 },
  { "Zygmont", 1.0 },
  { "Zygmund", 1.0 },
  { "Zygmunt", 1.0 },
  { "Zyhaire", 1.0 },
  { "Zyheem", 1.0 },
  { "Zyheim", 1.0 },
  { "Zyheir", 1.0 },
  { "Zyhier", 1.0 },
  { "Zyhir", 1.0 },
  { "Zyia", 0.0 },
  { "Zyiah", 0.0 },
  { "Zyian", 0.41666666 },
  { "Zyiana", 0.0 },
  { "Zyianna", 0.0 },
  { "Zyien", 0.8181818 },
  { "Zyier", 1.0 },
  { "Zyiere", 1.0 },
  { "Zyin", 1.0 },
  { "Zyion", 0.836403 },
  { "Zyiona", 0.0 },
  { "Zyionna", 0.0 },
  { "Zyir", 1.0 },
  { "Zyira", 0.0 },
  { "Zyire", 1.0 },
  { "Zykaria", 0.0 },
  { "Zykayla", 0.0 },
  { "Zykee", 1.0 },
  { "Zykeem", 1.0 },
  { "Zykei", 1.0 },
  { "Zykeia", 0.0 },
  { "Zykeira", 0.0 },
  { "Zykeith", 1.0 },
  { "Zykel", 1.0 },
  { "Zykell", 1.0 },
  { "Zykera", 0.0 },
  { "Zykeria", 0.0 },
  { "Zykeriah", 0.0 },
  { "Zykerion", 1.0 },
  { "Zykeriya", 0.0 },
  { "Zykerria", 0.0 },
  { "Zykerriah", 0.0 },
  { "Zykevious", 1.0 },
  { "Zykia", 0.0 },
  { "Zykiah", 0.0 },
  { "Zykiera", 0.0 },
  { "Zykierra", 0.0 },
  { "Zyking", 1.0 },
  { "Zykir", 1.0 },
  { "Zykira", 0.0 },
  { "Zykirah", 0.0 },
  { "Zykiria", 0.0 },
  { "Zyla", 0.0 },
  { "Zylah", 0.0 },
  { "Zylan", 1.0 },
  { "Zylar", 1.0 },
  { "Zylas", 1.0 },
  { "Zyleah", 0.0 },
  { "Zylee", 0.0 },
  { "Zyleigh", 0.0 },
  { "Zylen", 1.0 },
  { "Zyler", 1.0 },
  { "Zylia", 0.0 },
  { "Zyliah", 0.0 },
  { "Zylie", 0.0 },
  { "Zylin", 1.0 },
  { "Zylis", 1.0 },
  { "Zyliyah", 0.0 },
  { "Zyllah", 0.0 },
  { "Zylon", 1.0 },
  { "Zylpha", 0.0 },
  { "Zylphia", 0.0 },
  { "Zylus", 1.0 },
  { "Zylyn", 1.0 },
  { "Zylynn", 0.0 },
  { "Zymair", 1.0 },
  { "Zymaire", 1.0 },
  { "Zymal", 0.0 },
  { "Zymari", 1.0 },
  { "Zymaria", 0.0 },
  { "Zymarion", 1.0 },
  { "Zymeer", 1.0 },
  { "Zymeir", 1.0 },
  { "Zymeire", 1.0 },
  { "Zymere", 1.0 },
  { "Zymeria", 0.0 },
  { "Zymia", 0.0 },
  { "Zymiah", 0.0 },
  { "Zymier", 1.0 },
  { "Zymiere", 1.0 },
  { "Zymir", 1.0 },
  { "Zymira", 0.0 },
  { "Zymirah", 0.0 },
  { "Zymire", 1.0 },
  { "Zymya", 0.0 },
  { "Zynab", 0.0 },
  { "Zynae", 0.0 },
  { "Zynah", 0.0 },
  { "Zynaria", 0.0 },
  { "Zynasia", 0.0 },
  { "Zyndall", 1.0 },
  { "Zyndaya", 0.0 },
  { "Zynia", 0.0 },
  { "Zyniah", 0.0 },
  { "Zynique", 0.0 },
  { "Zyniya", 0.0 },
  { "Zyniyah", 0.0 },
  { "Zynovia", 0.0 },
  { "Zyon", 0.8000703 },
  { "Zyona", 0.0 },
  { "Zyonah", 0.0 },
  { "Zyonn", 1.0 },
  { "Zyonna", 0.0 },
  { "Zyonnah", 0.0 },
  { "Zyonne", 1.0 },
  { "Zyquan", 1.0 },
  { "Zyquarius", 1.0 },
  { "Zyquavion", 1.0 },
  { "Zyquavious", 1.0 },
  { "Zyquez", 1.0 },
  { "Zyra", 0.0 },
  { "Zyrah", 0.0 },
  { "Zyran", 1.0 },
  { "Zyree", 0.8072289 },
  { "Zyreion", 1.0 },
  { "Zyrek", 1.0 },
  { "Zyrell", 1.0 },
  { "Zyren", 1.0 },
  { "Zyreon", 1.0 },
  { "Zyrese", 1.0 },
  { "Zyrhiana", 0.0 },
  { "Zyri", 0.0 },
  { "Zyria", 0.0 },
  { "Zyriah", 0.0 },
  { "Zyrian", 1.0 },
  { "Zyriana", 0.0 },
  { "Zyrianna", 0.0 },
  { "Zyrie", 0.24 },
  { "Zyriel", 0.0 },
  { "Zyrielle", 0.0 },
  { "Zyrihanna", 0.0 },
  { "Zyrin", 1.0 },
  { "Zyrion", 1.0 },
  { "Zyriyah", 0.0 },
  { "Zyron", 1.0 },
  { "Zyrus", 1.0 },
  { "Zysean", 1.0 },
  { "Zyshaun", 1.0 },
  { "Zyshawn", 1.0 },
  { "Zyshon", 1.0 },
  { "Zyshonne", 1.0 },
  { "Zytaevius", 1.0 },
  { "Zytaveon", 1.0 },
  { "Zytavion", 1.0 },
  { "Zytavious", 1.0 },
  { "Zyus", 1.0 },
  { "Zyva", 0.0 },
  { "Zyvion", 1.0 },
  { "Zyvon", 1.0 },
  { "Zyyanna", 0.0 },
  { "Zyyon", 1.0 },
  { "Zzyzx", 1.0 }
};
